/*!
*  filename: ej.web.all.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

window.ej = window.Syncfusion = window.Syncfusion || {};

(function ($, ej, undefined) {
    'use strict';

    ej.consts = {
        NamespaceJoin: '-'
    };
    ej.textAlign = {
        Center: 'center',
        Justify: 'justify',
        Left: 'left',
        Right: 'right'
    };
    ej.orientation = { Horizontal: "horizontal", Vertical: "vertical" };

    if (!Object.prototype.hasOwnProperty) {
        Object.prototype.hasOwnProperty = function (obj, prop) {
            return obj[prop] !== undefined;
        };
    }

    String.format = function () {
        var source = arguments[0];
        for (var i = 0; i < arguments.length - 1; i++)
            source = source.replace(new RegExp("\\{" + i + "\\}", "gm"), arguments[i + 1]);

        source = source.replace(/\{[0-9]\}/g, "");
        return source;
    };

    // Function to create new class
    ej.defineClass = function (className, constructor, proto, replace) {
        /// <summary>Creates the javascript class with given namespace & class name & constructor etc</summary>
        /// <param name="className" type="String">class name prefixed with namespace</param>
        /// <param name="constructor" type="Function">constructor function</param>
        /// <param name="proto" type="Object">prototype for the class</param>
        /// <param name="replace" type="Boolean">[Optional]Replace existing class if exists</param>
        /// <returns type="Function">returns the class function</returns>
        if (!className || !proto) return undefined;

        var parts = className.split(".");

        // Object creation
        var obj = window, i = 0;
        for (; i < parts.length - 1; i++) {

            if (ej.isNullOrUndefined(obj[parts[i]]))
                obj[parts[i]] = {};

            obj = obj[parts[i]];
        }

        if (replace || ej.isNullOrUndefined(obj[parts[i]])) {

            //constructor
            constructor = typeof constructor === "function" ? constructor : function () {
            };

            obj[parts[i]] = constructor;

            // prototype
            obj[parts[i]].prototype = proto;
        }

        return obj[parts[i]];
    };

    ej.util = {
        getNameSpace: function (className) {
            /// <summary>Internal function, this will create namespace for plugins using class name</summary>
            /// <param name="className" type="String"></param>
            /// <returns type="String"></returns>
            var splits = className.toLowerCase().split(".");
            splits[0] === "ej" && (splits[0] = "e");

            return splits.join(ej.consts.NamespaceJoin);
        },

        getObject: function (nameSpace, from) {
            if (!from) return undefined;

            var value = from, splits = nameSpace.split('.');

            for (var i = 0; i < splits.length; i++) {

                if (ej.util.isNullOrUndefined(value)) break;

                value = value[splits[i]];
            }

            return value;
        },

        createObject: function (nameSpace, value, initIn) {
            var splits = nameSpace.split('.'), start = initIn || window, from = start, i;

            for (i = 0; i < splits.length; i++) {

                if (i + 1 == splits.length)
                    from[splits[i]] = ej.isNullOrUndefined(value) ? {} : value;
                else if (ej.isNullOrUndefined(from[splits[i]]))
                    from[splits[i]] = {};

                from = from[splits[i]];
            }

            return start;
        },

        isNullOrUndefined: function (value) {
            /// <summary>Util to check null or undefined</summary>
            /// <param name="value" type="Object"></param>
            /// <returns type="Boolean"></returns>
            return value === undefined || value === null;
        },

        buildTag: function (tag, innerHtml, styles, attrs) {
            /// <summary>Helper to build jQuery element</summary>
            /// <param name="tag" type="String">tagName#id.cssClass</param>
            /// <param name="innerHtml" type="String"></param>
            /// <param name="styles" type="Object">A set of key/value pairs that configure styles</param>
            /// <param name="attrs" type="Object">A set of key/value pairs that configure attributes</param>
            /// <returns type="jQuery"></returns>
            var tagName = /^[a-z]*[0-9a-z]+/ig.exec(tag)[0];

            var id = /#([a-z]+[-_0-9a-z]+)/ig.exec(tag);
            id = id ? id[id.length - 1] : undefined;

            var className = /\.([a-z]+[-_0-9a-z ]+)/ig.exec(tag);
            className = className ? className[className.length - 1] : undefined;

            return $(document.createElement(tagName))
                .attr(id ? { "id": id } : {})
                .addClass(className || "")
                .css(styles || {})
                .attr(attrs || {})
                .html(innerHtml || "");
        }
    };

    $.extend(ej, ej.util);

    // base class for all ej widgets. It will automatically inhertied
    ej.widgetBase = {
        droppables: { 'default': [] },
        resizables: { 'default': [] },

        destroy: function () {

            if (this._trigger("destroy"))
                return;

            if (this.model.persist)
                this.persistState();

            this._destroy();

            var arr = this.element.data("ejWidgets") || [];
            for (var i = 0; i < arr.length; i++) {
                if (arr[i] == this.pluginName) {
                    arr.splice(i, 1);
                }
            }
            if (!arr.length)
                this.element.removeData("ejWidgets");

            while (this._events) {
                var item = this._events.pop(), args = [];

                if (!item)
                    break;

                for (var i = 0; i < item[1].length; i++)
                    if (!$.isPlainObject(item[1][i]))
                        args.push(item[1][i]);

                $.fn.off.apply(item[0], args);
            }

            this._events = null;

            this.element
                .removeClass(ej.util.getNameSpace(this.sfType))
                .removeClass("e-js")
                .removeData(this.pluginName);

            this.element = null;
            this.model = null;
        },

        _on: function (element) {
            if (!this._events)
                this._events = [];
            var args = [].splice.call(arguments, 1, arguments.length - 1);

            var handler = {}, i = args.length;
            while (handler && typeof handler !== "function") {
                handler = args[--i];
            }

            args[i] = ej.proxy(args[i], this);

            this._events.push([element, args, handler, args[i]]);

            $.fn.on.apply(element, args);

            return this;
        },

        _off: function (element, eventName, selector, handlerObject) {
            var e = this._events, temp;
            if (!e || !e.length)
                return this;
            if (typeof selector == "function") {
                temp = handlerObject;
                handlerObject = selector;
                selector = temp;
            }
            for (var i = 0; i < e.length; i++) {
                var arg = e[i],
                r = arg[0].length && (!handlerObject || arg[2] === handlerObject) && arg[1][0] === eventName && (!selector || arg[1][1] === selector) && $.inArray(element[0], arg[0]) > -1;
                if (r) {
                    $.fn.off.apply(element, handlerObject ? [eventName, selector, arg[3]] : [eventName, selector]);
                    e.splice(i, 1);
                    break;
                }
            }

            return this;
        },

        // Client side events wire-up / trigger helper.
        _trigger: function (eventName, eventProp) {
            var fn = this.model[eventName], returnValue, args;

            if (fn) {
                if (typeof fn === "string") {
                    fn = ej.util.getObject(fn, window);
                }

                if ($.isFunction(fn)) {

                    args = ej.event(eventName, this.model, eventProp);

                    returnValue = fn.call(this, args);

                    // sending changes back - deep copy option should not be enabled for this $.extend 
                    if (eventProp) $.extend(eventProp, args);

                    if (args.cancel || !ej.isNullOrUndefined(returnValue))
                        return returnValue === false || args.cancel;
                }
            }

            var isPropDefined = Boolean(eventProp);
            eventProp = eventProp || {};
            eventProp.originalEventType = eventName;
            eventProp.type = this.pluginName + eventName;

            args = $.Event(eventProp.type, ej.event(eventProp.type, this.model, eventProp));

            this.element.trigger(args);

            // sending changes back - deep copy option should not be enabled for this $.extend 
            if (isPropDefined) $.extend(eventProp, args);

            return args.cancel;
        },

        setModel: function (options, forceSet) {
            // check for whether to apply values are not. if _setModel function is defined in child,
            //  this will call that function and validate it using return value

            if (this._trigger("modelChange", { "changes": options }))
                return;


            for (var prop in options) {
                if (!forceSet && this.model[prop] === options[prop]) {
                    delete options[prop];
                    continue;
                }

                if (this.dataTypes) {
                    var returnValue = this._isValidModelValue(prop, this.dataTypes, options);
                    if (returnValue !== true)
                        throw "setModel - Invalid input for property :" + prop + " - " + returnValue;
                }
                if (this.model.notifyOnEachPropertyChanges && this.model[prop] !== options[prop]) {
                    var arg = {
                        oldValue: this.model[prop],
                        newValue: options[prop]
                    };

                    options[prop] = this._trigger(prop + "Change", arg) ? this.model[prop] : arg.newValue;
                }
            }
            if ($.isEmptyObject(options))
                return;

            if (this._setFirst) {
                var ds = options.dataSource;
                if (ds) delete options.dataSource

                $.extend(true, this.model, options);
                if (ds) {
                    var tempds = [];
                    this.model.dataSource = $.extend(true, tempds, ds);
                    options["dataSource"] = tempds;
                }
                !this._setModel || this._setModel(options);

            } else if (!this._setModel || this._setModel(options) !== false) {
                $.extend(true, this.model, options);
            }
            if ("persist" in options) {
                if (options.persist === true) {
                    this._persistHandler = ej.proxy(this.persistState, this);
                    $(window).on("unload", this._persistHandler);
                } else {
                    $(window).off("unload", this._persistHandler);
                }
            }
        },

        option: function (prop, value, forceSet) {
            if (!prop)
                return this.model;

            if ($.isPlainObject(prop))
                return this.setModel(prop);

            if (typeof prop === "string") {
                prop = prop.replace(/^model\./, "");
                var oldValue = ej.getObject(prop, this.model);

                if (value === undefined)
                    return oldValue;

                if (forceSet && value === ej.extensions.modelGUID) {
                    return this._setModel(ej.createObject(prop, ej.getObject(prop, this.model), {}));
                }

                if (forceSet || ej.getObject(prop, this.model) !== value)
                    return this.setModel(ej.createObject(prop, value, {}), forceSet);
            }
            return undefined;
        },

        _isValidModelValue: function (prop, types, options) {
            var value = types[prop], option = options[prop], returnValue;

            if (!value)
                return true;

            if (typeof value === "string") {
                if (value == "enum") {
                    options[prop] = option ? option.toString().toLowerCase() : option;
                    value = "string";
                }

                if (value === "array") {
                    if (option instanceof Array)
                        return true;
                }
                else if (value === "data") {
                    return true;
                }
                else if (typeof option === value)
                    return true;

                return "Expected type - " + value;
            }

            if (option instanceof Array) {
                for (var i = 0; i < option.length; i++) {
                    returnValue = this._isValidModelValue(prop, types, option[i]);
                    if (returnValue !== true) {
                        return " [" + i + "] - " + returnValue;
                    }
                }
                return true;
            }

            for (var innerProp in option) {
                returnValue = this._isValidModelValue(innerProp, value, option);
                if (returnValue !== true)
                    return innerProp + " : " + returnValue;
            }

            return true;
        },

        persistState: function () {
            var model = copyObject({}, this.model);

            if (this.ignoreOnPersist) {
                for (var i = 0; i < this.ignoreOnPersist; i++) {
                    delete model[this.ignoreOnPersist[i]];
                }
            }

            if (this._persistState) {
                model.customPersists = {};
                this._persistState(model.customPersists);
            }

            if (window.localStorage)
                window.localStorage.setItem(this.pluginName + this._id, JSON.stringify(model));
            else if (document.cookie)
                ej.cookie.set(this.pluginName + this._id, model);
        },

        restoreState: function (silent) {
            var value = null;
            if (window.localStorage)
                value = window.localStorage.getItem(this.pluginName + this._id);
            else if (document.cookie)
                value = ej.cookie.get(this.pluginName + this._id);

            if (value) {
                var model = JSON.parse(value);

                if (this._restoreState) {
                    this._restoreState(model.customPersists);
                    delete model.customPersists;
                }

                this.model = $.extend(true, this.model, model);
            }

            if (!silent && value && this._setModel)
                this._setModel(this.model);
        }
    };

    ej.widget = function (pluginName, className, proto) {
        /// <summary>Widget helper for developers, this set have predefined function to jQuery plug-ins</summary>
        /// <param name="pluginName" type="String">the plugin name that will be added in jquery.fn</param>
        /// <param name="className" type="String">the class name for your plugin, this will help create default cssClas</param>
        /// <param name="proto" type="Object">prototype for of the plug-in</param>

        if (typeof pluginName === "object") {
            proto = className;
            for (var prop in pluginName) {
                var name = pluginName[prop];

                if (name instanceof Array) {
                    proto._rootCSS = name[1];
                    name = name[0];
                }

                ej.widget(prop, name, proto);

                if (pluginName[prop] instanceof Array)
                    proto._rootCSS = "";
            }

            return;
        }

        var nameSpace = proto._rootCSS || ej.getNameSpace(className);

        proto = ej.defineClass(className, function (element, options) {

            this.sfType = className;
            this.pluginName = pluginName;
            this.instance = pInstance;

            if (ej.isNullOrUndefined(this._setFirst))
                this._setFirst = true;

            this["ob.values"] = {};

            $.extend(this, ej.widgetBase);

            if (this.dataTypes) {
                for (var property in options) {
                    var returnValue = this._isValidModelValue(property, this.dataTypes, options);
                    if (returnValue !== true)
                        throw "setModel - Invalid input for property :" + property + " - " + returnValue;
                }
            }

            var arr = (element.data("ejWidgets") || []);
            arr.push(pluginName);
            element.data("ejWidgets", arr);

            for (var i = 0; ej.widget.observables && this.observables && i < this.observables.length; i++) {
                var t = ej.getObject(this.observables[i], options);
                if (t) ej.createObject(this.observables[i], ej.widget.observables.register(t, this.observables[i], this, element), options);
            }

            this.element = element.jquery ? element : $(element);
            this.model = copyObject(true, {}, proto.prototype.defaults, options);
            this.model.keyConfigs = copyObject(this.keyConfigs);

            this._id = element[0].id;

            if (!this.element.attr("tabIndex"))
                this.element.attr("tabIndex", "");

            if (this.model.persist) {
                this._persistHandler = ej.proxy(this.persistState, this);
                $(window).on("unload", this._persistHandler);
                this.restoreState(true);
            }

            this._init();

            if (typeof this.model.keyConfigs === "object" && !(this.model.keyConfigs instanceof Array)) {
                var requiresEvt = false;
                if (this.model.keyConfigs.focus)
                    this.element.attr("accesskey", this.model.keyConfigs.focus);

                for (var keyProps in this.model.keyConfigs) {
                    if (keyProps !== "focus") {
                        requiresEvt = true;
                        break;
                    }
                }

                if (requiresEvt && this._keyPressed) {
                    var el = element, evt = "keydown";

                    if (this.keySettings) {
                        el = this.keySettings.getElement ? this.keySettings.getElement() || el : el;
                        evt = this.keySettings.event || evt;
                    }

                    this._on(el, evt, function (e) {
                        if (!this.model.keyConfigs) return;

                        var action = keyFn.getActionFromCode(this.model.keyConfigs, e.which, e.ctrlKey, e.shiftKey, e.altKey);
                        var arg = {
                            code: e.which,
                            ctrl: e.ctrlKey,
                            alt: e.altKey,
                            shift: e.shiftKey
                        };
                        if (!action) return;

                        if (this._keyPressed(action, e.target, arg, e) === false)
                            e.preventDefault();
                    });
                }
            }

            this._trigger("create");
        }, proto);

        $.fn[pluginName] = function (options) {

            for (var i = 0; i < this.length; i++) {

                var $this = $(this[i]),
                    pluginObj = $this.data(pluginName),
                    isAlreadyExists = pluginObj && $this.hasClass(nameSpace),
                    obj = null;

                // ----- plug-in creation/init
                if (!isAlreadyExists) {
                    if (!options || typeof options === "object") {
                        $this.addClass(nameSpace + " e-js").data(pluginName, new proto($this, options));
                    }
                    else {
                        throwError(pluginName + ": methods/properties can be accessed only after plugin creation");
                    }
                    continue;
                }

                if (!options) continue;

                // --- Function/property set/access
                if ($.isPlainObject(options)) {
                    // setModel using JSON object
                    pluginObj.setModel(options);
                }

                    // function/property name starts with "_" is private so ignore it.
                else if (options.indexOf('_') !== 0
                    && !ej.isNullOrUndefined(obj = ej.getObject(options, pluginObj))
                    || options.indexOf("model.") === 0) {

                    if (!obj || !$.isFunction(obj)) {

                        // if property is accessed, then break the jquery chain
                        if (arguments.length == 1)
                            return obj;

                        //setModel using string input
                        pluginObj.option(options, arguments[1]);

                        continue;
                    }

                    var value = obj.apply(pluginObj, [].splice.call(arguments, 1, arguments.length - 1));

                    // If function call returns any value, then break the jquery chain
                    if (!ej.util.isNullOrUndefined(value))
                        return value;

                } else {
                    throwError(className + ": function/property - " + options + " does not exist");
                }
            }

            // maintaining jquery chain
            return this;
        };

        ej.widget.register(pluginName, className, proto.prototype);
    };

    $.extend(ej.widget, (function () {
        var _widgets = {},

        register = function (pluginName, className, prototype) {
            if (!ej.isNullOrUndefined(_widgets[pluginName]))
                throwError("ej.widget : The widget named " + pluginName + " is trying to register twice.");

            _widgets[pluginName] = { name: pluginName, className: className, proto: prototype };
        }

        return {
            register: register,
            registeredWidgets: _widgets
        };

    })());

    ej.widget.destroyAll = function (elements) {
        if (!elements || !elements.length) return;

        for (var i = 0; i < elements.length; i++) {
            var data = elements.eq(i).data();
            if (data["ejWidgets"] && data["ejWidgets"].length) {
                for (var j = data["ejWidgets"].length - 1; j > -1; j--) {
                    if (data[data["ejWidgets"][j]] && data[data["ejWidgets"][j]].destroy)
                        data[data["ejWidgets"][j]].destroy();
                }
            }
        }
    };

    ej.cookie = {
        get: function (name) {
            var value = RegExp(name + "=([^;]+)").exec(document.cookie);

            if (value && value.length > 1)
                return value[1];

            return undefined;
        },
        set: function (name, value, expiryDate) {
            if (typeof value === "object")
                value = JSON.stringify(value);

            value = escape(value) + ((expiryDate == null) ? "" : "; expires=" + expiryDate.toUTCString());
            document.cookie = name + "=" + value;
        }
    };

    var keyFn = {
        getActionFromCode: function (keyConfigs, keyCode, isCtrl, isShift, isAlt) {
            isCtrl = isCtrl || false;
            isShift = isShift || false;
            isAlt = isAlt || false;

            for (var keys in keyConfigs) {
                if (keys === "focus") continue;

                var key = keyFn.getKeyObject(keyConfigs[keys]);
                if (keyCode === key.code && isCtrl == key.isCtrl && isShift == key.isShift && isAlt == key.isAlt)
                    return keys;
            }
            return null;
        },
        getKeyObject: function (key) {
            var res = {
                isCtrl: false,
                isShift: false,
                isAlt: false
            };
            key = key.split("+");
            for (var i = 0; i < key.length; i++) {
                if (key[i] === "ctrl")
                    res.isCtrl = true;
                else if (key[i] === "shift")
                    res.isShift = true;
                else if (key[i] === "alt")
                    res.isAlt = true;
                else res.code = parseInt(key[i], 10);
            }
            return res;
        }
    };

    ej.browserInfo = function () {
        var browser = {}, clientInfo = [],
        browserClients = {
            webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
            opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        return browser;
    };

    ej.eventType = {
        mouseDown: "mousedown touchstart",
        mouseMove: "mousemove touchmove",
        mouseUp: "mouseup touchend",
        mouseLeave: "mouseleave touchcancel",
        click: "click touchend"
    };

    ej.event = function (type, data, eventProp) {

        var e = $.extend(eventProp || {},
            {
                "type": type,
                "model": data,
                "cancel": false
            });

        return e;
    };

    ej.proxy = function (fn, context, arg) {
        if (!fn || typeof fn !== "function")
            return null;

        if ('bind' in fn)
            return arg ? fn.bind(context, arg) : fn.bind(context);

        return function () {
            var args = arg ? [arg] : []; args.push.apply(args, arguments);
            fn.apply(context, args);
        };
    };

    ej.hasStyle = function (prop) {
        var style = document.documentElement.style;

        if (prop in style) return true;

        var prefixs = ['ms', 'Moz', 'Webkit', 'O', 'Khtml'];

        prop = prop[0].toUpperCase() + prop.slice(1);

        for (var i = 0; i < prefixs.length; i++) {
            if (prefixs[i] + prop in style)
                return true;
        }

        return false;
    };

    Array.prototype.indexOf = Array.prototype.indexOf || function (searchElement) {
        var len = this.length;

        if (len === 0) return -1;

        for (var i = 0; i < len; i++) {
            if (i in this && this[i] === searchElement)
                return i;
        }
        return -1;
    };

    String.prototype.startsWith = String.prototype.startsWith || function (key) {
        return this.slice(0, key.length) === key;
    };
    var copyObject = ej.copyObject = function (isFirstDefault, target) {
        var start = 2, current, source;
        if (typeof isFirstDefault !== "boolean") {
            start = 1;
        }
        var objects = [].slice.call(arguments, start);
        if (start === 1) {
            target = isFirstDefault;
            isFirstDefault = undefined;
        }

        for (var i = 0; i < objects.length; i++) {
            for (var prop in objects[i]) {
                current = target[prop], source = objects[i][prop];

                if (source === undefined || current === source || objects[i] === source || target === source)
                    continue;
                if (source instanceof Array) {
                    if (i === 0 && isFirstDefault)
                        target[prop] = source.slice();
                    else
                        target[prop] = source;
                } else if (ej.isPlainObject(source)) {
                    target[prop] = current || {};
                    if (isFirstDefault)
                        copyObject(isFirstDefault, target[prop], source);
                    else
                        copyObject(target[prop], source);
                } else
                    target[prop] = source;
            }
        }
        return target;
    };
    var pInstance = function () {
        return this;
    }
    ej.isPlainObject = function (obj) {
        if (!obj) return false;

        if (typeof obj !== "object" || obj.nodeType || jQuery.isWindow(obj)) {
            return false;
        }

        try {
            if (obj.constructor &&
                !obj.constructor.prototype.hasOwnProperty("isPrototypeOf")) {
                return false;
            }
        } catch (e) {
            return false;
        }

        var key, ownLast = ej.support.isOwnLast;
        for (key in obj) {
            if (ownLast) break;
        }

        return key === undefined || obj.hasOwnProperty(key);
    };
    ej.util.valueFunction = function (prop) {
        return function (value) {
            var val = ej.getObject(prop, this.model);

            if (value === undefined)
                return typeof val === "function" ? val() : val;

            if (typeof val === "function") {
                this["ob.values"][prop] = value;
                val.call(this, value);
            }
            else
                ej.createObject(prop, value, this.model);
        }
    };
    ej.util.getVal = function (val) {
        if (typeof val === "function")
            return val();
        return val;
    };
    ej.support = {
        isOwnLast: function () {
            var fn = function () { this.a = 1; };
            fn.prototype.b = 1;

            for (var p in new fn()) {
                return p === "b";
            }
        }(),
        outerHTML: function () {
            return document.createElement("div").outerHTML !== undefined;
        }()
    };

    var throwError = ej.throwError = function (er) {
        try {
            throw new Error(er);
        } catch (e) {
            throw e.message + "\n" + e.stack;
        }
    };

    ej.extensions = {};
    ej.extensions.modelGUID = "{0B1051BA-1CCB-42C2-A3B5-635389B92A50}";
})(window.jQuery, window.Syncfusion);;
window.ej = window.Syncfusion = window.Syncfusion || {};

(function ($, ej, doc, undefined) {
    'use strict';

    ej.DataManager = function (dataSource, query, adaptor) {
        if (!(this instanceof ej.DataManager))
            return new ej.DataManager(dataSource, query, adaptor);

        if (!dataSource)
            throwError("DataManger : a dataSource is required to create Manager");

        var data = [], self = this;

        if (dataSource instanceof Array) {
            // JSON array
            data = {
                json: dataSource,
                offline: true
            };

        } else if (typeof dataSource === "object") {
            if ($.isPlainObject(dataSource)) {

                if (dataSource.table)
                    dataSource.json = this._getJsonFromElement(dataSource.table, dataSource.headerOption);

                data = {
                    url: dataSource.url,
                    insertUrl: dataSource.insertUrl,
                    removeUrl: dataSource.removeUrl,
                    updateUrl: dataSource.updateUrl,
                    crudUrl: dataSource.crudUrl,
                    batchUrl: dataSource.batchUrl,
                    json: dataSource.json,
                    headers: dataSource.headers,
                    accept: dataSource.accept,
                    data: dataSource.data,
                    requestType: dataSource.requestType,
                    key: dataSource.key,
                    crossDomain: dataSource.crossDomain,
                    jsonp: dataSource.jsonp,
                    dataType: dataSource.dataType,
                    offline: dataSource.offline !== undefined ? dataSource.offline : dataSource.url ? false : true,
                    requiresFormat: dataSource.requiresFormat
                };
            } else if (dataSource.jquery || isHtmlElement(dataSource)) {
                data = {
                    json: this._getJsonFromElement(dataSource),
                    offline: true,
                    table: dataSource
                };
            }
        } else if (typeof dataSource === "string") {
            data = {
                url: dataSource,
                offline: false,
                dataType: "json"
            };
        }

        if (data.requiresFormat === undefined && !ej.support.cors)
            data.requiresFormat = isNull(data.crossDomain) ? true : data.crossDomain;
        if (data.dataType === undefined)
            data.dataType = "json";

        this.dataSource = data;
        this.defaultQuery = query;

        if (data.url && data.offline && !data.json) {
            this.isDataAvailable = false;
            this.adaptor = adaptor || new ej.ODataAdaptor();
            this.dataSource.offline = false;
            this.ready = this.executeQuery(query || ej.Query()).done(function (e) {
                self.dataSource.offline = true;
                self.isDataAvailable = true;
                data.json = e.result;
                self.adaptor = new ej.JsonAdaptor();
            });
        } else
            this.adaptor = data.offline ? new ej.JsonAdaptor() : new ej.ODataAdaptor();
        if (!data.jsonp && this.adaptor instanceof ej.ODataAdaptor)
            data.jsonp = "callback";
        this.adaptor = adaptor || this.adaptor;
        return this;
    };

    ej.DataManager.prototype = {
        setDefaultQuery: function (query) {
            this.defaultQuery = query;
        },

        executeQuery: function (query, done, fail, always) {
            if (typeof query === "function") {
                always = fail;
                fail = done;
                done = query;
                query = null;
            }

            if (!query)
                query = this.defaultQuery;

            if (!(query instanceof ej.Query))
                throwError("DataManager - executeQuery() : A query is required to execute");

            var deffered = $.Deferred();

            deffered.then(done, fail, always);
            var args = { query: query };

            if (!this.dataSource.offline) {
                this._makeRequest(this.adaptor.processQuery(this, query), deffered, args, query);
            } else {
                nextTick(function () {
                    var res = this.executeLocal(query);
                    if (query._requiresCount) {
                        args.result = res.result;
                        args.count = res.count;
                    } else
                        args.result = res;
                    args.getTableModel = getTableModel(query._fromTable, args.result, this);
                    args.getKnockoutModel = getKnockoutModel(args.result);

                    deffered.resolveWith(this, [args]);
                }, this);
            }

            return deffered.promise();
        },

        executeLocal: function (query) {
            if (!this.defaultQuery && !(query instanceof ej.Query))
                throwError("DataManager - executeLocal() : A query is required to execute");

            if (!this.dataSource.json)
                throwError("DataManager - executeLocal() : Json data is required to execute");

            var ret, key;
            query = query || this.defaultQuery;

            var result = this.adaptor.processQuery(this, query);

            if (query._subQuery) {
                var from = query._subQuery._fromTable, lookup = query._subQuery._lookup,
                    res = query._requiresCount ? result.result : result;

                if (lookup && lookup instanceof Array) {
                    buildHierarchy(query._subQuery._fKey, from, res, lookup);
                }

                for (var j = 0; j < res.length; j++) {
                    if (res[j][from] instanceof Array) {
                        res[j] = $.extend({}, res[j]);
                        res[j][from] = this.adaptor.processResponse(query._subQuery.using(ej.DataManager(res[j][from].slice(0))).executeLocal(), this, query);
                    }
                }
            }

            return this.adaptor.processResponse(result, this, query);
        },

        _makeRequest: function (url, deffered, args, query) {
            var isSelector = !!query._subQuerySelector;

            var fnFail = $proxy(function (e) {
                args.error = e;
                deffered.rejectWith(this, [args]);
            }, this);

            var process = $proxy(function (data, count, xhr, request, actual) {
                if (isSelector) return;

                args.xhr = xhr;
                args.count = parseInt(count, 10);
                args.result = data;
                args.request = request;
                args.getTableModel = getTableModel(query._fromTable, data, this);
                args.getKnockoutModel = getKnockoutModel(data);
                args.actual = actual;
                deffered.resolveWith(this, [args]);

            }, this);

            var fnQueryChild = $proxy(function (data, selector) {
                var subDeffer = $.Deferred(),
                    childArgs = { parent: args };

                query._subQuery._isChild = true;

                var subUrl = this.adaptor.processQuery(this, query._subQuery, data ? this.adaptor.processResponse(data) : selector);

                var childReq = this._makeRequest(subUrl, subDeffer, childArgs, query._subQuery);

                subDeffer.then(function (subData) {
                    if (data) {
                        buildHierarchy(query._subQuery._fKey, query._subQuery._fromTable, data, subData);
                        process(data);
                    }
                }, fnFail);

                return childReq;
            }, this);

            var fnSuccess = proxy(function (data, status, xhr, request) {
                data = p.parseJson(data);
                var result = this.adaptor.processResponse(data, this, query, xhr, request), count = 0;
                if (query._requiresCount) {
                    count = result.count;
                    result = result.result;
                }

                if (!query._subQuery) {
                    process(result, count, xhr, request, data);
                    return;
                }

                if (!isSelector)
                    fnQueryChild(result);

            }, this);

            var req = $.extend({
                type: "GET",
                dataType: this.dataSource.dataType,
                crossDomain: this.dataSource.crossDomain,
                jsonp: this.dataSource.jsonp,
                cache: true,
                beforeSend: $proxy(this._beforeSend, this),
                processData: false,
                success: fnSuccess,
                error: fnFail
            }, url);

            req = $.ajax(req);

            if (isSelector) {
                var res = query._subQuerySelector.call(this, { query: query._subQuery, parent: query });

                if (res && res.length) {
                    req = $.when(req, fnQueryChild(null, res));

                    req.then(proxy(function (pData, cData, requests) {
                        var pResult = this.adaptor.processResponse(pData[0], this, query, pData[2], requests[0]), count = 0;
                        if (query._requiresCount) {
                            count = pResult.count;
                            pResult = pResult.result;
                        }
                        var cResult = this.adaptor.processResponse(cData[0], this, query._subQuery, cData[2], requests[1]), count = 0;
                        if (query._subQuery._requiresCount) {
                            count = cResult.count;
                            cResult = cResult.result;
                        }

                        buildHierarchy(query._subQuery._fKey, query._subQuery._fromTable, pResult, cResult);
                        isSelector = false;
                        process(pResult, count, pData[2]);

                    }, this), fnFail);
                } else {
                    isSelector = false;
                }
            }

            return req;
        },

        _beforeSend: function (request, settings) {
            this.adaptor.beforeSend(this, request, settings);

            var headers = this.dataSource.headers, props;
            for (var i = 0; headers && i < headers.length; i++) {
                props = [];
                for (var prop in headers[i]) props.push(prop);
                request.setRequestHeader(headers[props[0]], headers[props[1]]);
            }
        },

        saveChanges: function (changes, key, tableName) {
            var args = {
                url: tableName,
                key: key || this.dataSource.key
            };

            var req = this.adaptor.batchRequest(this, changes, args);

            if (this.dataSource.offline) {
                return req;
            }

            var deff = $.Deferred();
            $.ajax($.extend({
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (data, status, xhr, request) {
                    deff.resolveWith(this, this.adaptor.processResponse(data, this, null, xhr, request, changes));
                }, this),
                fail: function (e) {
                    deff.rejectWith(this, e);
                }
            }, req));

            return deff;
        },

        insert: function (data, tableName) {
            var res = this.adaptor.insert(this, data, tableName);
            if (this.dataSource.offline) {
                return res;
            }

            var deffer = $.Deferred();

            $.ajax($.extend({
                type: "POST",
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                processData: false,
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (record, status, xhr, request) {
                    record = this.adaptor.processResponse(p.parseJson(record), this, null, xhr, request);
                    deffer.resolveWith(this, [{ record: record, dataManager: this }]);
                }, this),
                error: function (e) {
                    deffer.rejectWith(this, [{ error: e, dataManager: this }]);
                }
            }, res));

            return deffer.promise();
        },

        remove: function (keyField, value, tableName) {
            if (typeof value === "object")
                value = value[keyField];

            var res = this.adaptor.remove(this, keyField, value, tableName);

            if (this.dataSource.offline)
                return res;

            return $.ajax($.extend({
                type: "POST",
                contentType: "application/json; charset=utf-8",
                beforeSend: $proxy(this._beforeSend, this),
                dataType: "json"
            }, res));
        },

        update: function (keyField, value, tableName) {
            var res = this.adaptor.update(this, keyField, value, tableName);

            if (this.dataSource.offline) {
                return res;
            }

            return $.ajax($.extend({
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                beforeSend: $proxy(this._beforeSend, this),
            }, res));
        },

        _getJsonFromElement: function (ds) {
            if (typeof (ds) == "string")
                ds = $($(ds).html());

            ds = ds.jquery ? ds[0] : ds;

            var tagName = ds.tagName.toLowerCase();

            if (tagName !== "table")
                throwError("ej.DataManager : Unsupported htmlElement : " + tagName);

            return ej.parseTable(ds);
        }
    };

    var buildHierarchy = function (fKey, from, source, lookup) {
        var i, grp = {}, t;
        if (lookup.result) lookup = lookup.result;

        if (lookup.GROUPGUID)
            throwError("ej.DataManager: Do not have support Grouping in hierarchy");

        for (i = 0; i < lookup.length; i++) {
            t = grp[lookup[i][fKey]] || (grp[lookup[i][fKey]] = []);

            t.push(lookup[i]);
        }

        for (i = 0; i < source.length; i++) {
            source[i][from] = grp[source[i][fKey]];
        }
    };

    var oData = {
        accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
        multipartAccept: "multipart/mixed",
        batch: "$batch",
        changeSet: "--changeset_",
        batchPre: "batch_",
        contentId: "Content-Id: ",
        batchContent: "Content-Type: multipart/mixed; boundary=",
        changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
        batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
    };
    var p = {
        parseJson: function (jsonText) {
            var type = typeof jsonText;
            if (type === "string") {
                jsonText = JSON.parse(jsonText, p.jsonReviver);
            } else if (jsonText instanceof Array) {
                p.iterateAndReviveArray(jsonText);
            } else if (type === "object")
                p.iterateAndReviveJson(jsonText);
            return jsonText;
        },
        iterateAndReviveArray: function (array) {
            for (var i = 0; i < array.length; i++) {
                if (typeof array[i] === "object")
                    p.iterateAndReviveJson(array[i]);
                else
                    array[i] = p.parseJson(array[i]);
            }
        },
        iterateAndReviveJson: function (json) {
            var value;

            for (var prop in json) {
                if (prop.startsWith("__"))
                    continue;

                value = json[prop];
                if (typeof value === "object") {
                    if (value instanceof Array)
                        p.iterateAndReviveArray(value);
                    else
                        p.iterateAndReviveJson(value);
                } else
                    json[prop] = p.jsonReviver(prop, value);
            }
        },
        jsonReviver: function (field, value) {
            var s = value;
            if (typeof value === "string") {
                var ms = /^\/Date\(([+-]?[0-9]+)([+-][0-9]{4})?\)\/$/.exec(value);
                if (ms)
                    return new Date(parseInt(ms[1]));
                else if (/^(\d{4}\-\d\d\-\d\d([tT][\d:\.]*)?)([zZ]|([+\-])(\d\d):?(\d\d))?$/.test(value)) {
                    value = new Date(value);
                    if (value == "Invalid Date") {
                        var a = s.split(/[^0-9]/);
                        value = new Date(a[0], a[1] - 1, a[2], a[3], a[4], a[5]);
                    }
                }
            }

            return value;
        },
        isJson: function (jsonData) {
            if (typeof jsonData[0] == "string")
                return jsonData;
            return ej.parseJSON(jsonData);
        }
    };

    ej.isJSON = p.isJson;
    ej.parseJSON = p.parseJson;

    ej.Query = function (from) {
        if (!(this instanceof ej.Query))
            return new ej.Query(from);

        this.queries = [];
        this._key = "";
        this._fKey = "";

        if (typeof from === "string")
            this._fromTable = from || "";
        else if (from && from instanceof Array)
            this._lookup = from;

        this._expands = [];
        this._sortedColumns = [];
        this._groupedColumns = [];
        this._subQuery = null;
        this._isChild = false;
        this._params = [];
        return this;
    };

    ej.Query.prototype = {
        key: function (field) {
            if (typeof field === "string")
                this._key = field;

            return this;
        },

        using: function (dataManager) {
            if (dataManager instanceof ej.DataManager) {
                this.dataManagar = dataManager;
                return this;
            }

            return throwError("Query - using() : 'using' function should be called with parameter of instance ej.DataManager");
        },

        execute: function (dataManager, done, fail, always) {
            dataManager = dataManager || this.dataManagar;

            if (dataManager && dataManager instanceof ej.DataManager)
                return dataManager.executeQuery(this, done, fail, always);

            return throwError("Query - execute() : dataManager needs to be is set using 'using' function or should be passed as argument");
        },

        executeLocal: function (dataManager) {
            // this does not support for URL binding

            dataManager = dataManager || this.dataManagar;

            if (dataManager && dataManager instanceof ej.DataManager)
                return dataManager.executeLocal(this);

            return throwError("Query - executeLocal() : dataManager needs to be is set using 'using' function or should be passed as argument");
        },

        clone: function () {
            var cl = new ej.Query();
            cl.queries = this.queries.slice(0);
            cl._key = this._key;
            cl._isChild = this._isChild;
            cl.dataManagar = this.dataManager;
            cl._fromTable = this._fromTable;
            cl._params = this._params.slice(0);
            cl._expands = this._expands.slice(0);
            cl._sortedColumns = this._sortedColumns.slice(0);
            cl._groupedColumns = this._groupedColumns.slice(0);
            cl._subQuerySelector = this._subQuerySelector;
            cl._subQuery = this._subQuery;
            cl._fKey = this._fKey;
            cl._requiresCount = this._requiresCount;
            return cl;
        },

        from: function (tableName) {
            if (typeof tableName === "string")
                this._fromTable = tableName;

            return this;
        },

        addParams: function (key, value) {
            if (typeof value !== "function" && !ej.isPlainObject(value))
                this._params.push({ key: key, value: value });
            else if (typeof value === "function")
                this._params.push({ key: key, fn: value });

            return this;
        },

        expand: function (tables) {
            if (typeof tables === "string")
                this._expands = [].slice.call(arguments, 0);
            else
                this._expands = tables.slice(0);

            return this;
        },

        where: function (fieldName, operator, value, ignoreCase) {
            operator = (operator || ej.FilterOperators.equal).toLowerCase();
            var predicate = null;

            if (typeof fieldName === "string")
                predicate = new ej.Predicate(fieldName, operator, value, ignoreCase);
            else if (fieldName instanceof ej.Predicate)
                predicate = fieldName;
            else
                throwError("Query - where : Invalid arguments");

            this.queries.push({
                fn: "onWhere",
                e: predicate
            });
            return this;
        },

        search: function (searchKey, fieldNames, operator, ignoreCase) {
            if (!fieldNames || typeof fieldNames === "boolean") {
                fieldNames = [];
                ignoreCase = fieldNames;
            } else if (typeof fieldNames === "string")
                fieldNames = [fieldNames];

            if (typeof operator === "boolean") {
                ignoreCase = operator;
                operator = null;
            }
            operator = operator || ej.FilterOperators.contains;
            if (operator.length < 3)
                operator = ej.data.operatorSymbols[operator];

            var comparer = ej.data.fnOperators[operator] || ej.data.fnOperators.processSymbols(operator);

            this.queries.push({
                fn: "onSearch",
                e: {
                    fieldNames: fieldNames,
                    operator: operator,
                    searchKey: searchKey,
                    ignoreCase: ignoreCase,
                    comparer: comparer
                }
            });
            return this;
        },

        sortBy: function (fieldName, comparer, isFromGroup) {
            var order = ej.sortOrder.Ascending, sorts, t;

            if (typeof fieldName === "string" && fieldName.toLowerCase().endsWith(" desc")) {
                fieldName = fieldName.replace(/ desc$/i, '');
                comparer = ej.sortOrder.Descending;
            }

            if (typeof comparer === "boolean")
                comparer = !comparer ? ej.sortOrder.Ascending : ej.sortOrder.Descending;
            else if (typeof comparer === "function")
                order = "custom";

            if (!comparer || typeof comparer === "string") {
                order = comparer ? comparer.toLowerCase() : ej.sortOrder.Ascending;
                comparer = ej.pvt.fnSort(comparer);
            }
            if (isFromGroup) {
                sorts = filterQueries(this.queries, "onSortBy");

                for (var i = 0; i < sorts.length; i++) {
                    t = sorts[i].e.fieldName;
                    if (typeof t === "string") {
                        if (t === fieldName) return;
                    } else if (t instanceof Array) {
                        for (var j = 0; j < t.length; j++)
                            if (t[j] === fieldName || fieldName.toLowerCase() === t[j] + " desc")
                                return;
                    }
                }
            }

            this.queries.push({
                fn: "onSortBy",
                e: {
                    fieldName: fieldName,
                    comparer: comparer,
                    direction: order
                }
            });

            return this;
        },

        sortByDesc: function (fieldName) {
            return this.sortBy(fieldName, ej.sortOrder.Descending);
        },

        group: function (fieldName) {
            this.sortBy(fieldName, null, true);

            this.queries.push({
                fn: "onGroup",
                e: {
                    fieldName: fieldName
                }
            });
            return this;
        },

        page: function (pageIndex, pageSize) {
            this.queries.push({
                fn: "onPage",
                e: {
                    pageIndex: pageIndex,
                    pageSize: pageSize
                }
            });
            return this;
        },

        range: function (start, end) {
            if (typeof start !== "number" || typeof end !== "number")
                throwError("Query() - range : Arguments type should be a number");

            this.queries.push({
                fn: "onRange",
                e: {
                    start: start,
                    end: end
                }
            });
            return this;
        },

        take: function (nos) {
            if (typeof nos !== "number")
                throwError("Query() - Take : Argument type should be a number");

            this.queries.push({
                fn: "onTake",
                e: {
                    nos: nos
                }
            });
            return this;
        },

        skip: function (nos) {
            if (typeof nos !== "number")
                throwError("Query() - Skip : Argument type should be a number");

            this.queries.push({
                fn: "onSkip",
                e: { nos: nos }
            });
            return this;
        },

        select: function (fieldNames) {
            if (typeof fieldNames === "string")
                fieldNames = [].slice.call(arguments, 0);

            if (!(fieldNames instanceof Array)) {
                throwError("Query() - Select : Argument type should be String or Array");
            }

            this.queries.push({
                fn: "onSelect",
                e: { fieldNames: fieldNames }
            });
            return this;
        },

        hierarchy: function (query, selectorFn) {
            if (!query || !(query instanceof ej.Query))
                throwError("Query() - hierarchy : query must be instance of ej.Query");

            if (typeof selectorFn === "function")
                this._subQuerySelector = selectorFn;

            this._subQuery = query;
            return this;
        },

        foreignKey: function (key) {
            if (typeof key === "string")
                this._fKey = key;

            return this;
        },

        requiresCount: function () {
            this._requiresCount = true;

            return this;
        }
    };

    ej.Adaptor = function (ds) {
        this.dataSource = ds;
        this.pvt = {};
    };

    ej.Adaptor.prototype = {
        options: {
            from: "table",
            requestType: "json",
            sortBy: "sorted",
            select: "select",
            skip: "skip",
            group: "group",
            take: "take",
            search: "search",
            count: "requiresCounts",
            where: "where",
        },
        extend: function (overrides) {
            var fn = function (ds) {
                this.dataSource = ds;

                if (this.options)
                    this.options = $.extend({}, this.options);

                this.pvt = {};
            };
            fn.prototype = new this.type();
            fn.prototype.type = fn;

            var base = fn.prototype.base = {};
            for (var p in overrides) {
                if (!fn.prototype[p])
                    base[p] = fn.prototype[p];
            }
            $.extend(true, fn.prototype, overrides);
            return fn;
        },
        processQuery: function (dm, query) {
            // this needs to be overridden
        },
        processResponse: function (data, ds, query, xhr) {
            return data;
        },
        convertToQueryString: function (req, query, dm) {
            return $.param(req);
        },
        type: ej.Adaptor
    };

    ej.UrlAdaptor = new ej.Adaptor().extend({
        processQuery: function (dm, query, hierarchyFilters) {
            var sorted = filterQueries(query.queries, "onSortBy"),
                grouped = filterQueries(query.queries, "onGroup"),
                filters = filterQueries(query.queries, "onWhere"),
                searchs = filterQueries(query.queries, "onSearch"),
                singles = filterQueryLists(query.queries, ["onSelect", "onPage", "onSkip", "onTake", "onRange"]),
                params = query._params,
                url = dm.dataSource.url, tmp, skip, take = null,
                op = this.options;

            var r = {
                sorted: [],
                grouped: [],
                filters: [],
                searches: []
            };

            // calc Paging & Range
            if (singles["onPage"]) {
                tmp = singles["onPage"];
                skip = getValue(tmp.pageIndex, query);
                take = getValue(tmp.pageSize, query);

                skip = (skip - 1) * take;
            } else if (singles["onRange"]) {
                tmp = singles["onRange"];
                skip = tmp.start;
                take = tmp.end - tmp.start;
            }

            // Sorting
            for (var i = 0; i < sorted.length; i++) {
                tmp = getValue(sorted[i].e.fieldName, query);

                r.sorted.push(callAdaptorFunc(this, "onEachSort", { name: tmp, direction: sorted[i].e.direction }, query));
            }

            // hierarchy
            if (hierarchyFilters) {
                tmp = this.getFiltersFrom(hierarchyFilters, query);
                if (tmp)
                    r.filters.push(callAdaptorFunc(this, "onEachWhere", tmp.toJSON(), query));
            }

            // Filters
            for (var i = 0; i < filters.length; i++) {
                r.filters.push(callAdaptorFunc(this, "onEachWhere", filters[i].e.toJSON(), query));

                for (var prop in r.filters[i]) {
                    if (isNull(r[prop]))
                        delete r[prop];
                }
            }

            // Searches
            for (var i = 0; i < searchs.length; i++) {
                tmp = searchs[i].e;
                r.searches.push(callAdaptorFunc(this, "onEachSearch", {
                    fields: tmp.fieldNames,
                    operator: tmp.operator,
                    key: tmp.searchKey,
                    ignoreCase: tmp.ignoreCase
                }, query));
            }

            // Grouping
            for (var i = 0; i < grouped.length; i++) {
                r.grouped.push(getValue(grouped[i].e.fieldName, query));
            }

            var req = {};
            req[op.from] = query._fromTable;
            if (op.expand) req[op.expand] = query._expands;
            req[op.select] = singles["onSelect"] ? callAdaptorFunc(this, "onSelect", getValue(singles["onSelect"].fieldNames, query), query) : "";
            req[op.count] = query._requiresCount ? callAdaptorFunc(this, "onCount", query._requiresCount, query) : "";
            req[op.search] = r.searches.length ? callAdaptorFunc(this, "onSearch", r.searches, query) : "";
            req[op.skip] = singles["onSkip"] ? callAdaptorFunc(this, "onSkip", getValue(singles["onSkip"].nos, query), query) : "";
            req[op.take] = singles["onTake"] ? callAdaptorFunc(this, "onTake", getValue(singles["onTake"].nos, query), query) : "";
            req[op.where] = r.filters.length || r.searches.length ? callAdaptorFunc(this, "onWhere", r.filters, query) : "";
            req[op.sortBy] = r.sorted.length ? callAdaptorFunc(this, "onSortBy", r.sorted, query) : "";
            req[op.group] = r.grouped.length ? callAdaptorFunc(this, "onGroup", r.grouped, query) : "";

            // Params
            for (var i = 0; tmp = params[i]; i++) {
                if (req[tmp.key]) throwError("ej.Query: Custom Param is conflicting other request arguments");

                req[tmp.key] = tmp.value;
                if (tmp.fn)
                    req[tmp.key] = tmp.fn.call(query, tmp.key, query, dm);
            }

            // cleanup
            for (var prop in req) {
                if (isNull(req[prop]) || req[prop] === "" || req[prop].length === 0)
                    delete req[prop];
            }

            if (!(op.skip in req && op.take in req) && take !== null) {
                req[op.skip] = callAdaptorFunc(this, "onSkip", skip, query);
                req[op.take] = callAdaptorFunc(this, "onTake", take, query);
            }
            var p = this.pvt;
            this.pvt = {};

            if (this.options.requestType === "json") {
                return {
                    data: JSON.stringify(req),
                    url: url,
                    ejPvtData: p,
                    type: "POST",
                    contentType: "application/json; charset=utf-8"
                }
            }
            tmp = this.convertToQueryString(req, query, dm);
            return {
                type: "GET",
                url: tmp.length ? url.replace(/\/*$/, "/" + tmp) : url,
                ejPvtData: p
            };
        },
        convertToQueryString: function (req, query, dm) {
            if (dm.dataSource.url && dm.dataSource.url.indexOf("?") !== -1)
                return $.param(req);
            return "?" + $.param(req);
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var pvt = request.ejPvtData || {};
            var d = JSON.parse(request.data)
            if (d && d.action === "batch" && data.added) {
                changes.added = data.added;
                return changes;
            }

            if (pvt.groups && pvt.groups.length) {
                var groups = pvt.groups, args = {};
                if (data["count"]) args.count = data.count;
                if (data["result"]) data = data.result;
                for (var i = 0; i < groups.length; i++)
                    data = ej.group(data, groups[i]);
                if (args.count)
                    args.result = data;
                else
                    args = data;
                return args;
            }
            return data;
        },
        onGroup: function (e) {
            this.pvt.groups = e;
        },
        batchRequest: function (dm, changes, e) {
            return {
                type: "POST",
                url: dm.dataSource.batchUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                data: JSON.stringify({
                    changed: changes.changed,
                    added: changes.added,
                    deleted: changes.deleted,
                    action: "batch",
                    table: e.url,
                    key: e.key
                })
            };
        },
        beforeSend: function (dm, request) {
        },
        insert: function (dm, data, tableName) {
            return {
                url: dm.dataSource.insertUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    value: data,
                    table: tableName,
                    action: "insert"
                })
            };
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.removeUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    key: value,
                    keyColumn: keyField,
                    table: tableName,
                    action: "remove"
                })
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.updateUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    value: value,
                    action: "update",
                    keyColumn: keyField,
                    key: value[keyField],
                    table: tableName
                })
            };
        },
        getFiltersFrom: function (data, query) {
            if (!(data instanceof Array) || !data.length)
                throwError("ej.SubQuery: Array of key values required");
            var key = query._fKey, value, prop = key, predicats = [],
                isValues = typeof data[0] !== "object";

            if (typeof data[0] !== "object") prop = null;

            for (var i = 0; i < data.length; i++) {
                value = !isValues ? ej.pvt.getObject(prop, data[i]) : data[i];
                predicats.push(new ej.Predicate(key, "==", value));
            }

            return ej.Predicate.or(predicats);
        }
    });

    var filterQueries = function (queries, name) {
        return queries.filter(function (q) {
            return q.fn === name;
        }) || [];
    };
    var filterQueryLists = function (queries, singles) {
        var filtered = queries.filter(function (q) {
            return singles.indexOf(q.fn) !== -1;
        }), res = {};
        for (var i = 0; i < filtered.length; i++) {
            if (!res[filtered[i].fn])
                res[filtered[i].fn] = filtered[i].e;
        }
        return res;
    };
    var callAdaptorFunc = function (obj, fnName, param, param1) {
        if (obj[fnName]) {
            var res = obj[fnName](param, param1);
            if (!isNull(res)) param = res;
        }
        return param;
    };

    ej.ODataAdaptor = new ej.UrlAdaptor().extend({
        options: {
            requestType: "get",
            accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
            multipartAccept: "multipart/mixed",
            sortBy: "$orderby",
            select: "$select",
            skip: "$skip",
            take: "$top",
            count: "$inlinecount",
            where: "$filter",
            expand: "$expand",
            batch: "$batch",
            changeSet: "--changeset_",
            batchPre: "batch_",
            contentId: "Content-Id: ",
            batchContent: "Content-Type: multipart/mixed; boundary=",
            changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
            batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
        },
        onEachWhere: function (filter, requiresCast) {
            return filter.isComplex ? this.onComplexPredicate(filter, requiresCast) : this.onPredicate(filter, requiresCast);
        },
        onPredicate: function (pred, query, requiresCast) {
            var returnValue = "",
                operator = pred.operator,
                val = pred.value,
                type = typeof val,
                field = this._p(pred.field);

            if (val instanceof Date)
                val = "datetime'" + dateSerialize(val) + "'";

            if (type === "string") {
                val = "'" + val + "'";

                if (requiresCast) {
                    field = "cast(" + field + ", 'Edm.String')";
                }

                if (pred.ignoreCase) {
                    field = "tolower(" + field + ")";
                    val = val.toLowerCase();
                }
            }

            operator = ej.data.odBiOperator[pred.operator];
            if (operator) {
                returnValue += field;
                returnValue += operator;

                return returnValue + val;
            }

            operator = ej.data.odUniOperator[pred.operator];
            if (!operator || type !== "string") return "";

            if (operator === "substringof") {
                var t = val;
                val = field;
                field = t;
            }

            returnValue += operator + "(";
            returnValue += field + ",";
            returnValue += val + ")";

            return returnValue;
        },
        onComplexPredicate: function (pred, requiresCast) {
            var res = [];
            for (var i = 0; i < pred.predicates.length; i++) {
                res.push("(" + this.onEachWhere(pred.predicates[i], requiresCast) + ")");
            }
            return res.join(" " + pred.condition + " ");
        },
        onWhere: function (filters) {
            if (this.pvt.searches)
                filters.push(this.onEachWhere(this.pvt.searches, null, true));

            return filters.join(" and ");
        },
        onEachSearch: function (e) {
            if (e.fields.length === 0)
                throwError("Query() - Search : oData search requires list of field names to search");

            var filter = this.pvt.searches || [];
            for (var i = 0; i < e.fields.length; i++) {
                filter.push(new ej.Predicate(e.fields[i], e.operator, e.key, e.ignoreCase));
            }
            this.pvt.searches = filter;
        },
        onSearch: function (e) {
            this.pvt.searches = ej.Predicate.or(this.pvt.searches);
            return "";
        },
        onEachSort: function (e) {
            var res = [];
            if (e.name instanceof Array) {
                for (var i = 0; i < e.name.length; i++)
                    res.push(this._p(e.name[i]));
            } else
                res.push(this._p(e.name) + (e.direction === "descending" ? " desc" : ""));

            return res.join(",");
        },
        onSortBy: function (e) {
            return e.join(",");
        },
        onGroup: function (e) {
            this.pvt.groups = e;
            return "";
        },
        onSelect: function (e) {
            for (var i = 0; i < e.length; i++)
                e[i] = this._p(e[i]);

            return e.join(',');
        },
        onCount: function (e) {
            return e === true ? "allpages" : "";
        },
        beforeSend: function (dm, request, settings) {
            if (settings.url.endsWith(this.options.batch) && settings.type.toLowerCase() === "post") {
                request.setRequestHeader("Accept", oData.multipartAccept);
                request.setRequestHeader("DataServiceVersion", "2.0");
                request.overrideMimeType("text/plain; charset=x-user-defined");
            }

            if (!dm.dataSource.crossDomain) {
                request.setRequestHeader("DataServiceVersion", "2.0");
                request.setRequestHeader("MaxDataServiceVersion", "2.0");
            }
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var pvt = request && request.ejPvtData;
            if (request && request.url.endsWith(this.options.batch) && request.type.toLowerCase() === "post") {
                var guid = xhr.getResponseHeader("Content-Type"), cIdx, jsonObj;
                guid = guid.substring(guid.indexOf("=batchresponse") + 1);
                data = data.split(guid);
                if (data.length < 2) return;

                data = data[1];
                var exVal = /(?:\bContent-Type.+boundary=)(changesetresponse.+)/i.exec(data);
                data.replace(exVal[0], "");

                var changeGuid = exVal[1];
                data = data.split(changeGuid);

                for (var i = data.length; i > -1; i--) {
                    if (!/\bContent-ID:/i.test(data[i]) || !/\bHTTP.+201/.test(data[i]))
                        continue;

                    cIdx = parseInt(/\bContent-ID: (\d+)/i.exec(data[i])[1]);

                    if (changes.added[cIdx]) {
                        jsonObj = p.parseJson(/^\{.+\}/m.exec(data[i])[0]);
                        $.extend(changes.added[cIdx], this.processResponse(jsonObj));
                    }
                }
                return changes;
            }
            var version = xhr && xhr.getResponseHeader("DataServiceVersion"), count = null;
            version = (version && parseInt(version, 10)) || 2;

            if (query && query._requiresCount) {
                if (data.__count || data['odata.count']) count = data.__count || data['odata.count'];
                if (data.d) data = data.d;
                if (data.__count || data['odata.count']) count = data.__count || data['odata.count'];
            }

            if (version === 3 && data.value) data = data.value;
            if (data.d) data = data.d;
            if (version < 3 && data.results) data = data.results;

            if (pvt && pvt.groups && pvt.groups.length) {
                var groups = pvt.groups;
                for (var i = 0; i < groups.length; i++)
                    data = ej.group(data, groups[i]);
            }
            return isNull(count) ? data : { result: data, count: count };
        },
        convertToQueryString: function (req, query, dm) {
            var res = [], tableName = req.table || "";
            delete req.table;

            if (dm.dataSource.requiresFormat)
                req["$format"] = "json";

            for (var prop in req)
                res.push(prop + "=" + req[prop]);

            res = res.join("&");

            if (dm.dataSource.url && dm.dataSource.url.indexOf("?") !== -1 && !tableName)
                return res;

            return res.length ? tableName + "?" + res : tableName || "";
        },
        insert: function (dm, data, tableName) {
            return {
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : ''),
                data: JSON.stringify(data)
            }
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "DELETE",
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : '') + '(' + value + ')'
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "PUT",
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : '') + '(' + value[keyField] + ')',
                data: JSON.stringify(value),
                accept: this.options.accept
            };
        },
        batchRequest: function (dm, changes, e) {
            var initialGuid = e.guid = ej.getGuid(oData.batchPre);
            var url = dm.dataSource.url.replace(/\/*$/, '/' + this.options.batch);
            var args = {
                url: e.url,
                key: e.key,
                cid: 1,
                cSet: ej.getGuid(oData.changeSet)
            };
            var req = "--" + initialGuid + "\n";

            req += "Content-Type: multipart/mixed; boundary=" + args.cSet.replace("--", "") + "\n";

            req += this.generateInsertRequest(changes.added, args);
            req += this.generateUpdateRequest(changes.changed, args);
            req += this.generateDeleteRequest(changes.deleted, args);

            req += args.cSet + "--\n";
            req += "--" + initialGuid + "--";

            return {
                type: "POST",
                url: url,
                contentType: "multipart/mixed; charset=UTF-8;boundary=" + initialGuid,
                data: req
            };
        },
        generateDeleteRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "DELETE ";
                req += e.url + "(" + arr[i][e.key] + ") HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += oData.batchChangeSetContentType + "\n";
            }

            return req + "\n";
        },
        generateInsertRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "POST ";
                req += e.url + " HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += "Content-Id: " + i + "\n";
                req += oData.batchChangeSetContentType + "\n\n";

                req += JSON.stringify(arr[i]) + "\n";
            }

            return req;
        },
        generateUpdateRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "PUT ";
                req += e.url + "(" + arr[i][e.key] + ")" + " HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += oData.batchChangeSetContentType + "\n\n";

                req += JSON.stringify(arr[i]) + "\n\n";
            }

            return req;
        },
        _p: function (prop) {
            return prop.replace(/\./g, "/");
        }
    });

    ej.JsonAdaptor = new ej.Adaptor().extend({
        processQuery: function (ds, query) {
            var result = ds.dataSource.json.slice(0), count = result.length, cntFlg = true, ret, key;

            for (var i = 0; i < query.queries.length; i++) {
                key = query.queries[i];
                ret = this[key.fn].call(this, result, key.e, query);
                result = ret !== undefined ? ret : result;

                if (key.fn === "onPage" || key.fn === "onSkip" || key.fn === "onTake" || key.fn === "onRange") cntFlg = false;

                if (cntFlg) count = result.length;
            }

            if (query._requiresCount) {
                result = {
                    result: result,
                    count: count
                };
            }

            return result;
        },
        batchRequest: function (dm, changes, e) {
            for (var i = 0; i < changes.added.length; i++)
                this.insert(dm, changes.added[i]);
            for (var i = 0; i < changes.changed.length; i++)
                this.update(dm, e.key, changes.changed[i]);
            for (var i = 0; i < changes.deleted.length; i++)
                this.remove(dm, e.key, changes.deleted[i]);
            return changes;
        },
        onWhere: function (ds, e) {
            if (!ds) return ds;

            return ds.filter(function (obj) {
                return e.validate(obj);
            });
        },
        onSearch: function (ds, e) {
            if (!ds || !ds.length) return ds;

            if (e.fieldNames.length === 0) {
                ej.pvt.getFieldList(ds[0], e.fieldNames);
            }

            return ds.filter(function (obj) {
                for (var j = 0; j < e.fieldNames.length; j++) {
                    if (e.comparer.call(obj, ej.pvt.getObject(e.fieldNames[j], obj), e.searchKey, e.ignoreCase))
                        return true;
                }
                return false;
            });
        },
        onSortBy: function (ds, e, query) {
            if (!ds) return ds;
            var fnCompare, field = getValue(e.fieldName, query);
            if (!field)
                return ds.sort(e.comparer);

            if (field instanceof Array) {
                field = field.slice(0);

                for (var i = field.length - 1; i >= 0; i--) {
                    if (!field[i]) continue;

                    fnCompare = e.comparer;

                    if (field[i].endsWith(" desc")) {
                        fnCompare = ej.pvt.fnSort(ej.sortOrder.Descending);
                        field[i] = field[i].replace(" desc", "");
                    }

                    ds = stableSort(ds, field[i], fnCompare);
                }
                return ds;
            }
            return stableSort(ds, field, e.comparer);
        },
        onGroup: function (ds, e, query) {
            if (!ds) return ds;

            return ej.group(ds, getValue(e.fieldName, query));
        },
        onPage: function (ds, e, query) {
            var size = getValue(e.pageSize, query),
                start = (getValue(e.pageIndex, query) - 1) * size, end = start + size;

            if (!ds) return ds;

            return ds.slice(start, end);
        },
        onRange: function (ds, e) {
            if (!ds) return ds;
            return ds.slice(getValue(e.start), getValue(e.end));
        },
        onTake: function (ds, e) {
            if (!ds) return ds;

            return ds.slice(0, getValue(e.nos));
        },
        onSkip: function (ds, e) {
            if (!ds) return ds;
            return ds.slice(getValue(e.nos));
        },
        onSelect: function (ds, e) {
            if (!ds) return ds;
            return ej.select(ds, getValue(e.fieldNames));
        },
        insert: function (dm, data) {
            return dm.dataSource.json.push(data);
        },
        remove: function (dm, keyField, value, tableName) {
            var ds = dm.dataSource.json, i;
            if (typeof value === "object")
                value = value[keyField];
            for (i = 0; i < ds.length; i++) {
                if (ds[i][keyField] === value) break;
            }

            return i !== ds.length ? ds.splice(i, 1) : null;
        },
        update: function (dm, keyField, value, tableName) {
            var ds = dm.dataSource.json, i, key = value[keyField];

            for (i = 0; i < ds.length; i++) {
                if (ds[i][keyField] === key) break;
            }

            return i < ds.length ? $.extend(ds[i], value) : null;
        },
    });

    var getValue = function (value, inst) {
        if (typeof value === "function")
            return value.call(inst || {});
        return value;
    }

    ej.TableModel = function (name, jsonArray, dataManager, modelComputed) {
        if (!instance(this, ej.TableModel))
            return new ej.TableModel(jsonArray);

        if (!instance(jsonArray, Array))
            throwError("ej.TableModel - Json Array is required");

        var rows = [], model, dirtyFn = $proxy(setDirty, this);

        for (var i = 0; i < jsonArray.length; i++) {
            model = new ej.Model(jsonArray[i], this);
            model.state = "unchanged";
            model.on("stateChange", dirtyFn);
            if (modelComputed)
                model.computes(modelComputed);
            rows.push(model);
        }

        this.name = name || "table1";

        this.rows = ej.NotifierArray(rows);
        this._deleted = [];

        this._events = $({});

        this.dataManager = dataManager;

        this._isDirty = false;

        return this;
    };

    ej.TableModel.prototype = {
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },

        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },

        setDataManager: function (dataManager) {
            this.dataManagar = dataManager;
        },

        saveChanges: function () {
            if (!this.dataManager || !instance(this.dataManager, ej.DataManager))
                throwError("ej.TableModel - saveChanges : Set the dataManager using setDataManager function");

            if (!this.isDirty())
                return;

            var promise = this.dataManager.saveChanges(this.getChanges(), this.key, this.name);

            promise.done($proxy(function (changes) {
                var rows = this.toArray();
                for (var i = 0; i < rows.length; i++) {
                    if (rows.state === "added") {
                        rows.set(this.key, changes.added.filter(function (e) {
                            return e[this.key] === rows.get(this.key);
                        })[0][this.key]);
                    }
                    rows[i].markCommit();
                }

                this._events.triggerHandler({ type: "save", table: this });

            }, this));

            promise.fail($proxy(function (e) {
                this.rejectChanges();
                this._events.triggerHandler({ type: "reject", table: this, error: e });
            }, this));

            this._isDirty = false;
        },

        rejectChanges: function () {
            var rows = this.toArray();
            for (var i = 0; i < rows.length; i++)
                rows[i].revert(true);

            this._isDirty = false;
            this._events.triggerHandler({ type: "reject", table: this });
        },

        insert: function (json) {
            var model = new ej.Model(json);
            model._isDirty = this._isDirty = true;

            this.rows.push(model);

            this._events.triggerHandler({ type: "insert", model: model, table: this });
        },

        update: function (value) {
            if (!this.key)
                throwError("TableModel - update : Primary key should be assigned to TableModel.key");

            var row = value, model, key = this.key, keyValue = row[key];

            model = this.rows.array.filter(function (obj) {
                return obj.get(key) === keyValue;
            });

            model = model[0];

            for (var col in row) {
                model.set(col, row[col]);
            }

            this._isDirty = true;

            this._events.triggerHandler({ type: "update", model: model, table: this });
        },

        remove: function (key) {
            if (!this.key)
                throwError("TableModel - update : Primary key should be assigned to TableModel.key");

            var field = this.key;

            var index = -1, model;

            if (key && typeof key === "object") {
                key = key[field] !== undefined ? key[field] : key.get(field);
            }

            for (var i = 0; i < this.rows.length() ; i++) {
                if (this.rows.array[i].get(field) === key) {
                    index = i;
                    break;
                }
            }

            if (index > -1) {
                model = this.rows.removeAt(index);
                model.markDelete();

                this._deleted.push({ model: model, position: index });

                this._isDirty = true;
                this._events.triggerHandler({ type: "remove", model: model, table: this });
            }
        },

        isDirty: function () {
            return this._isDirty;
        },

        getChanges: function () {

            var changes = {
                added: [],
                changed: []
            };
            var rows = this.toArray();
            for (var i = 0; i < rows.length; i++) {
                if (changes[rows[i].state])
                    changes[rows[i].state].push(rows[i].json);
            }

            changes.deleted = ej.select(this._deleted, ["model"]);

            return changes;
        },

        toArray: function () {
            return this.rows.toArray();
        },

        setDirty: function (dirty, model) {
            if (this._isDirty === !!dirty) return;

            this._isDirty = !!dirty;

            this._events.triggerHandler({ type: "dirty", table: this, model: model });
        },
        get: function (index) {
            return this.rows.array[index];
        },
        length: function () {
            return this.rows.array.length;
        },

        bindTo: function (element) {
            var marker = tDiv, template = $(element.html()), rows = this.toArray(), cur;
            if ($.inArray(element.prop("tagName").toLowerCase(), ["table", "tbody"]))
                marker = tTR;

            marker.insertBefore(element);
            element.detach().empty();

            for (var i = 0; i < rows.length; i++) {
                cur = template.clone();
                rows[i].bindTo(cur);
                element.append(cur);
            }

            element.insertAfter(marker);
            marker.remove();
        }
    };

    var tDiv = doc ? $(document.createElement("div")) : {},
        tTR = doc ? $(document.createElement("tr")) : {};

    ej.Model = function (json, table, name) {
        if (typeof table === "string") {
            name = table;
            table = null;
        }
        this.$id = getUid("m");

        this.json = json;
        this.table = table instanceof ej.TableModel ? table : null;
        this.name = name || (this.table && this.table.name);
        this.dataManager = (table instanceof ej.DataManager) ? table : table.dataManagar;
        this.actual = {};
        this._events = $({});
        this.isDirty = false;
        this.state = "added";
        this._props = [];
        this._computeEls = {};
        this._fields = {};
        this._attrEls = {};
        this._updates = {};
        this.computed = {};
    };

    ej.Model.prototype = {
        computes: function (value) {
            $.extend(this.computed, value);
        },
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },
        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },
        set: function (field, value) {
            var obj = this.json, actual = field, prev;
            field = field.split('.');

            for (var i = 0; i < field.length - 1; i++) {
                field = field[0];
                obj = obj[field[0]];
            }

            this.isDirty = true;
            this.changeState("changed", { from: "set" });

            prev = obj[field];

            if (this.actual[field] === undefined && !(field in this.actual))
                this.actual[field] = value; // Complex property ?

            obj[field] = value;

            this._updateValues(field, value);
            this._events.triggerHandler({ type: actual, current: value, previous: prev, model: this });
        },
        get: function (field) {
            return ej.pvt.getObject(field, this.json);
        },
        revert: function (suspendEvent) {
            for (var prop in this.actual) {
                this.json[prop] = this.actual[prop];
            }

            this.isDirty = false;

            if (suspendEvent)
                this.state = "unchanged";
            else
                this.changeState("unchanged", { from: "revert" });
        },
        save: function (dm, key) {
            dm = dm || this.dataManagar;
            key = key || dm.dataSource.key;
            if (!dm) throwError("ej.Model - DataManager is required to commit the changes");
            if (this.state === "added") {
                return dm.insert(this.json, this.name).done(ej.proxy(function (e) {
                    $.extend(this.json, e.record);
                }, this));
            }
            else if (this.state === "changed") {
                return dm.update(key, this.json, this.name);
            }
            else if (this.state === "deleted") {
                return dm.remove(key, this.json, this.name);
            }
        },
        markCommit: function () {
            this.isDirty = false;
            this.changeState("unchanged", { from: "commit" });
        },
        markDelete: function () {
            this.changeState("deleted", { from: "delete" });
        },
        changeState: function (state, args) {
            if (this.state === state) return;

            if (this.state === "added") {
                if (state === "deleted")
                    state = "unchanged";
                else return;
            }

            var prev = state;
            args = args || {};

            this.state = state;
            this._events.triggerHandler($.extend({ type: "stateChange", current: state, previous: prev, model: this }, args));
        },
        properties: function () {
            if (this._props.length)
                return this._props;

            for (var pr in this.json) {
                this._props.push(pr);
                this._updates[pr] = { read: [], input: [] };
            }

            return this._props;
        },
        bindTo: function (element) {
            var props = this.properties(), el = $(element), ctl, field,
                elements = el.find("[ej-observe], [ej-computed], [ej-prop]"), len = elements.length;

            el.data("ejModel", this);
            var unbindData = { fields: [], props: [], computes: [] };
            for (var i = 0; i < len; i++) {
                ctl = elements.eq(i);

                field = ctl.attr("ej-prop")
                if (field) {
                    this._processAttrib(field, ctl, unbindData);
                }
                field = ctl.attr("ej-observe");
                if (field && this._props.indexOf(field) !== -1) {
                    this._processField(ctl, field, unbindData);
                    continue;
                }

                field = ctl.attr("ej-computed");
                if (field) {
                    this._processComputed(field, ctl, unbindData);
                    continue;
                }
            }
            el.data("ejModelBinding" + this.$id, unbindData);
        },
        unbind: function (element) {
            var props = this.properties(), tmp, data = {
                props: this._attrEls,
                computes: this._computeEls
            }, isCustom = false;

            if (element) {
                data = $(element).removeData("ejModel").data("ejModelBinding" + this.$id) || data;
                isCustom = true;
            }

            for (var p in this.computed) {
                tmp = data.computes[p], p = this.computed[p];
                if (tmp && p.deps) {
                    this.off(p.deps.join(' '), tmp.handle);
                    if (isCustom)
                        delete this._computeEls[p];
                }
            }
            if (!isCustom)
                this._computeEls = {};

            for (var p in data.props) {
                tmp = data.props[p];
                if (tmp) {
                    this.off(tmp.deps.join(' '), tmp.handle);
                    delete data.props[p];
                    if (isCustom)
                        delete this._attrEls[p];
                }
            }
            if (!isCustom)
                this._attrEls = {};

            if (data.fields && data.fields.length) {
                var len = data.fields.length, ctl, idx, ty;
                for (var i = 0; i < len; i++) {
                    ctl = data.fields[i];
                    $(ctl).off("change", null, this._changeHandler);

                    ty = this.formElements.indexOf(ctl.tagName.toLowerCase()) !== -1 ? "input" : "read";
                    idx = this._updates[ty].indexOf(ctl);
                    if (idx !== -1)
                        this._updates[ty].splice(idx, 1);
                }
            }
        },
        _processComputed: function (value, element, data) {
            if (!value) return;

            var val, deps = [], safeVal = safeStr(value),
            type = this.formElements.indexOf(element[0].tagName.toLowerCase()) !== -1 ? "val" : "html";

            if (!this.computed[value] || !this.computed[safeVal]) {
                this.computed[safeVal] = {
                    value: new Function("var e = this; return " + value),
                    deps: this._generateDeps(value)
                }
                value = safeVal;
            }

            val = this.computed[value];
            if (!val.get) {
                val.get = function () {
                    val.value.call(this.json);
                }
            }

            deps = val.deps;
            val = val.value;

            this._updateDeps(deps);
            this._updateElement(element, type, val);

            val = { el: element, handle: $proxy(this._computeHandle, this, { value: value, type: type }) };
            this._computeEls[value] = val;
            data.computes[value] = val;

            this.on(deps.join(' '), val.handle);
        },
        _computeHandle: function (e) {
            var el = this._computeEls[e.value];
            if (el && this.computed[e.value])
                this._updateElement(el.el, e.type, this.computed[e.value].value);
        },
        _updateElement: function (el, type, val) {
            el[type](val.call($.extend({}, this.json, this.computed)));
        },
        _updateDeps: function (deps) {
            for (var i = 0; i < deps.length; i++) {
                if (!(deps[i] in this.json) && deps[i] in this.computed)
                    ej.merge(deps, this.computed[deps[i]].deps);
            }
        },
        _generateDeps: function (value) {
            var splits = value.replace(/(^e\.)|( e\.)/g, '#%^*##ej.#').split("#%^*#"),
                field, deps = [];

            for (var i = 0; i < splits.length; i++) {
                if (splits[i].startsWith("#ej.#")) {
                    field = splits[i].replace("#ej.#", "").split(' ')[0];
                    if (field && this._props.indexOf(field) !== -1)
                        deps.push(field);
                }
            }

            return deps;
        },
        _processAttrib: function (value, el, data) {
            var prop, val, res = {};
            value = value.replace(/^ +| +$/g, "").split(";");
            for (var i = 0; i < value.length; i++) {
                value[i] = value[i].split(":");
                if (value[i].length < 2) continue;

                prop = value[i][0].replace(/^ +| +$/g, "").replace(/^'|^"|'$|"$/g, "");
                res[prop] = value[i][1].replace(/^ +| +$/g, "").replace(/^'|^"|'$|"$/g, "");
            }
            value = res;
            var deps = [];
            for (var prop in value)
                deps.push(value[prop]);

            this._updateDeps(deps);
            this._updateProps(el, value);

            res = getUid("emak");
            val = { el: el, handle: $proxy(this._attrHandle, this, res), value: value, deps: deps };
            el.prop("ejmodelattrkey", res);

            data.props[res] = val;
            this._attrEls[res] = val;

            this.on(deps.join(' '), val.handle);
        },
        _attrHandle: function (res) {
            var el = this._attrEls[res];
            if (el)
                this._updateProps(el.el, el.value);
        },
        _updateProps: function (element, value) {
            var json = this.json, t, c = this.computed;
            for (var prop in value) {
                t = value[prop];
                if (t in json)
                    t = json[t];
                else if (t in this.computed) {
                    t = this.computed[t];
                    if (t) {
                        t = t.value.call($.extend({}, this.json, this.computed));
                    }
                }

                if (!isNull(t)) {
                    element.prop(prop, t);
                }
            }
        },
        _updateValues: function (prop, value) {
            var arr = this._updates[prop];

            if (!arr || (!arr.read && !arr.input)) return;

            this._ensureItems(arr.read, "html", value);
            this._ensureItems(arr.input, "val", value);
        },
        _ensureItems: function (a, type, value) {
            if (!a) return;

            for (var i = a.length - 1; i > -1; i--) {
                if (!a[i].offsetParent) {
                    a.splice(i, 1);
                    continue;
                }
                $(a[i])[type](value);
            }
        },
        _changeHandler: function (e) {
            e.data.self.set(e.data.prop, $(this).val());
        },
        _processField: function (ctl, field, data) {
            var e = { self: this, prop: field }, val = this.get(field);

            data.fields.push(ctl[0]);

            if (this.formElements.indexOf(ctl[0].tagName.toLowerCase()) === -1) {
                ctl.html(val);
                return this._updates[field].read.push(ctl[0]);
            }

            ctl.val(val)
                    .off("change", null, this._changeHandler)
                    .on("change", null, e, this._changeHandler);

            return this._updates[field].input.push(ctl[0]);
        },
        formElements: ["input", "select", "textarea"]
    };

    var safeReg = /[^\w]+/g;
    var safeStr = function (value) {
        return value.replace(safeReg, "_");
    };
    var setDirty = function (e) {
        this.setDirty(true, e.model);
    };

    ej.Predicate = function (field, operator, value, ignoreCase) {
        if (!(this instanceof ej.Predicate))
            return new ej.Predicate(field, operator, value, ignoreCase);

        if (typeof field === "string") {
            this.field = field;
            this.operator = operator;
            this.value = value;
            this.ignoreCase = ignoreCase;
            this.isComplex = false;

            this._comparer = ej.data.fnOperators.processOperator(this.operator);

        } else if (field instanceof ej.Predicate && value instanceof ej.Predicate || value instanceof Array) {
            this.isComplex = true;
            this.condition = operator.toLowerCase();
            this.predicates = [field];
            if (value instanceof Array)
                [].push.apply(this.predicates, value);
            else
                this.predicates.push(value);
        }
        return this;
    };

    ej.Predicate.and = function () {
        return pvtPredicate._combinePredicates([].slice.call(arguments, 0), "and");
    };

    ej.Predicate.or = function () {
        return pvtPredicate._combinePredicates([].slice.call(arguments, 0), "or");
    };

    // Private fn
    var pvtPredicate = {
        _combinePredicates: function (predicates, operator) {
            if (!predicates.length) return undefined;
            if (predicates.length === 1) {
                if (!instance(predicates[0], Array))
                    return predicates[0];
                predicates = predicates[0];
            }
            return new ej.Predicate(predicates[0], operator, predicates.slice(1));;
        },

        _combine: function (pred, field, operator, value, condition, ignoreCase) {
            if (field instanceof ej.Predicate)
                return ej.Predicate[condition](pred, field);

            if (typeof field === "string")
                return ej.Predicate[condition](pred, new ej.Predicate(field, operator, value, ignoreCase));

            return throwError("Predicate - " + condition + " : invalid arguments");
        },
    };

    ej.Predicate.prototype = {
        and: function (field, operator, value, ignoreCase) {
            return pvtPredicate._combine(this, field, operator, value, "and", ignoreCase);
        },
        or: function (field, operator, value, ignoreCase) {
            return pvtPredicate._combine(this, field, operator, value, "or", ignoreCase);
        },
        validate: function (record) {
            var value, p = this.predicates, isAnd, ret;

            if (!this.isComplex) {
                return this._comparer.call(this, ej.pvt.getObject(this.field, record), this.value, this.ignoreCase);
            }

            isAnd = this.condition === "and";

            for (var i = 0; i < p.length; i++) {
                ret = p[i].validate(record);
                if (isAnd) {
                    if (!ret) return false;
                } else {
                    if (ret) return true;
                }
            }

            return isAnd;
        },
        toJSON: function () {
            var predicates, p;
            if (this.isComplex) {
                predicates = [], p = this.predicates;
                for (var i = 0; i < p.length; i++)
                    predicates.push(p[i].toJSON());
            }
            return {
                isComplex: this.isComplex,
                field: this.field,
                operator: this.operator,
                value: this.value,
                ignoreCase: this.ignoreCase,
                condition: this.condition,
                predicates: predicates
            }
        }
    };

    ej.dataUtil = {
        swap: function (array, x, y) {
            if (x == y) return;

            var tmp = array[x];
            array[x] = array[y];
            array[y] = tmp;
        },

        mergeSort: function (jsonArray, fieldName, comparer) {
            if (!comparer || typeof comparer === "string")
                comparer = ej.pvt.fnSort(comparer, true);

            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.mergeSort(jsonArray, fieldName, comparer);
        },

        max: function (jsonArray, fieldName, comparer) {
            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.getItemFromComparer(jsonArray, fieldName, comparer || ej.pvt.fnDescending);
        },

        min: function (jsonArray, fieldName, comparer) {
            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.getItemFromComparer(jsonArray, fieldName, comparer || ej.pvt.fnAscending);
        },

        distinct: function (json, fieldName, requiresCompleteRecord) {
            var result = [], val, tmp = {};
            for (var i = 0; i < json.length; i++) {
                val = getVal(json, fieldName, i);
                if (!(val in tmp)) {
                    result.push(!requiresCompleteRecord ? val : json[i]);
                    tmp[val] = 1;
                }
            }
            return result;
        },

        sum: function (json, fieldName) {
            var result = 0, val;
            for (var i = 0; i < json.length; i++) {
                val = getVal(json, fieldName, i);
                if (val !== undefined)
                    result += val;
            }
            return result;
        },

        avg: function (json, fieldName) {
            return ej.sum(json, fieldName) / json.length;
        },

        select: function (jsonArray, fields) {
            var newData = [];

            for (var i = 0; i < jsonArray.length; i++) {
                newData.push(ej.pvt.extractFields(jsonArray[i], fields));
            }

            return newData;
        },

        group: function (jsonArray, field, /* internal */ level) {
            level = level || 1;

            if (jsonArray.GROUPGUID == ej.pvt.consts.GROUPGUID) {
                for (var j = 0; j < jsonArray.length; j++) {
                    jsonArray[j].items = ej.group(jsonArray[j].items, field, jsonArray.level + 1);
                    jsonArray[j].count = jsonArray[j].items.length;
                }

                jsonArray.childLevels += 1;
                return jsonArray;
            }

            var grouped = {}, groupedArray = [];

            groupedArray.GROUPGUID = ej.pvt.consts.GROUPGUID;
            groupedArray.level = level;
            groupedArray.childLevels = 0;
            groupedArray.records = jsonArray;

            for (var i = 0; i < jsonArray.length; i++) {
                var val = getVal(jsonArray, field, i);

                if (!grouped[val]) {
                    grouped[val] = {
                        key: val,
                        count: 0,
                        items: [],
                        field: field
                    };
                    groupedArray.push(grouped[val]);
                }

                grouped[val].count += 1;
                grouped[val].items.push(jsonArray[i]);
            }

            return groupedArray;
        },

        parseTable: function (table, headerOption, headerRowIndex) {
            var tr = table.rows, headerRow, headerTds = [], data = [], i;

            if (!tr.length) return [];

            headerRowIndex = headerRowIndex || 0;

            switch ((headerOption || "").toLowerCase()) {
                case ej.headerOption.tHead:
                    headerRow = table.tHead.rows[headerRowIndex];
                    break;
                case ej.headerOption.row:
                default:
                    headerRow = table.rows[headerRowIndex];
                    break;
            }

            var hTd = headerRow.cells;

            for (i = 0; i < hTd.length; i++)
                headerTds.push($.trim(hTd[i].innerHTML));

            for (i = headerRowIndex + 1; i < tr.length; i++) {
                var json = {}, td = tr[i].cells;
                for (var j = 0; j < td.length; j++) {
                    json[headerTds[j]] = td[j].innerHTML;
                }
                data.push(json);
            }
            return data;
        }
    };

    ej.headerOption = {
        tHead: "thead",
        row: "row"
    };

    ej.pvt = {
        filterQueries: filterQueries,
        mergeSort: function (jsonArray, fieldName, comparer) {
            if (jsonArray.length <= 1)
                return jsonArray;

            // else list size is > 1, so split the list into two sublists
            var middle = parseInt(jsonArray.length / 2, 10);

            var left = jsonArray.slice(0, middle),
                right = jsonArray.slice(middle);

            left = ej.pvt.mergeSort(left, fieldName, comparer);
            right = ej.pvt.mergeSort(right, fieldName, comparer);

            return ej.pvt.merge(left, right, fieldName, comparer);
        },

        getItemFromComparer: function (array, field, comparer) {
            var keyVal = getVal(array, field, 0), current, key = array[0];
            for (var i = 1; i < array.length; i++) {
                current = getVal(array, field, i);
                if (comparer(keyVal, current) > 0) {
                    keyVal = current;
                    key = array[i];
                }
            }
            return key;
        },

        quickSelect: function (array, fieldName, left, right, k, comparer) {
            if (left == right)
                return array[left];

            var pivotNewIndex = ej.pvt.partition(array, fieldName, left, right, comparer);

            var pivotDist = pivotNewIndex - left + 1;

            if (pivotDist == k)
                return array[pivotNewIndex];

            else if (k < pivotDist)
                return ej.pvt.quickSelect(array, fieldName, left, pivotNewIndex - 1, k, comparer);
            else
                return ej.pvt.quickSelect(array, fieldName, pivotNewIndex + 1, right, k - pivotDist, comparer);
        },

        extractFields: function (obj, fields) {
            var newObj = {};

            if (fields.length == 1)
                return ej.pvt.getObject(fields[0], obj);

            for (var i = 0; i < fields.length; i++) {
                newObj[fields[i].replace('.', ej.pvt.consts.complexPropertyMerge)] = ej.pvt.getObject(fields[i], obj);
            }

            return newObj;
        },

        partition: function (array, field, left, right, comparer) {

            var pivotIndex = parseInt((left + right) / 2, 10),
                pivot = getVal(array, field, pivotIndex);

            ej.swap(array, pivotIndex, right);

            pivotIndex = left;

            for (var i = left; i < right; i++) {
                if (comparer(getVal(array, field, i), pivot)) {
                    ej.swap(array, i, pivotIndex);
                    pivotIndex++;
                }
            }

            ej.swap(array, pivotIndex, right);

            return pivotIndex;
        },

        fnSort: function (order) {
            order = order ? order.toLowerCase() : ej.sortOrder.Ascending;

            if (order == ej.sortOrder.Ascending)
                return ej.pvt.fnAscending;

            return ej.pvt.fnDescending;
        },

        fnGetComparer: function (field, fn) {
            return function (x, y) {
                return fn(ej.pvt.getObject(field, x), ej.pvt.getObject(field, y));
            }
        },

        fnAscending: function (x, y) {
            if (x && x.localeCompare)
                return x.localeCompare(y);
            if (x === undefined) {
                if (y === undefined)
                    return 0;
                return 1;
            }
            if (y === undefined)
                return -1;
            return x - y;
        },

        fnDescending: function (x, y) {
            if (x && x.localeCompare)
                return x.localeCompare(y) * -1;
            if (x === undefined) {
                if (y === undefined)
                    return 0;
                return -1;
            }
            if (y === undefined)
                return 1;
            return y - x;
        },

        merge: function (left, right, fieldName, comparer) {
            var result = [], current;

            while (left.length > 0 || right.length > 0) {
                if (left.length > 0 && right.length > 0) {
                    if (comparer)
                        current = comparer(getVal(left, fieldName, 0), getVal(right, fieldName, 0)) < 1 ? left : right;
                    else
                        current = left[0][fieldName] < left[0][fieldName] ? left : right;
                } else {
                    current = left.length > 0 ? left : right;
                }

                result.push(current.shift());
            }

            return result;
        },

        getObject: function (nameSpace, from) {
            if (!from) return undefined;
            if (!nameSpace) return from;

            if (nameSpace.indexOf('.') === -1) return from[nameSpace];

            var value = from, splits = nameSpace.split('.');

            for (var i = 0; i < splits.length; i++) {

                if (value == null) break;

                value = value[splits[i]];
            }

            return value;
        },

        createObject: function (nameSpace, value, initIn) {
            var splits = nameSpace.split('.'), start = initIn || window, from = start, i;

            for (i = 0; i < splits.length; i++) {

                if (i + 1 == splits.length)
                    from[splits[i]] = value === undefined ? {} : value;
                else if (from[splits[i]] == null)
                    from[splits[i]] = {};

                from = from[splits[i]];
            }

            return start;
        },

        getFieldList: function (obj, fields, prefix) {
            if (prefix === undefined)
                prefix = "";

            if (fields === undefined || fields === null)
                return ej.pvt.getFieldList(obj, [], prefix);

            for (var prop in obj) {
                if (typeof obj[prop] === "object" && !(obj[prop] instanceof Array))
                    ej.pvt.getFieldList(obj[prop], fields, prefix + prop + ".");
                else
                    fields.push(prefix + prop);
            }

            return fields;
        }
    };

    ej.FilterOperators = {
        lessThan: "lessthan",
        greaterThan: "greaterthan",
        lessThanOrEqual: "lessthanorequal",
        greaterThanOrEqual: "greaterthanorequal",
        equal: "equal",
        contains: "contains",
        startsWith: "startswith",
        endsWith: "endswith",
        notEqual: "notequal"
    };

    ej.data = {};

    ej.data.operatorSymbols = {
        "<": "lessthan",
        ">": "greaterthan",
        "<=": "lessthanorequal",
        ">=": "greaterthanorequal",
        "==": "equal",
        "!=": "notequal",
        "*=": "contains",
        "$=": "endswith",
        "^=": "startswith"
    };

    ej.data.odBiOperator = {
        "<": " lt ",
        ">": " gt ",
        "<=": " le ",
        ">=": " ge ",
        "==": " eq ",
        "!=": " ne ",
        "lessthan": " lt ",
        "lessthanorequal": " le ",
        "greaterthan": " gt ",
        "greaterthanorequal": " ge ",
        "equal": " eq ",
        "notequal": " ne "
    };

    ej.data.odUniOperator = {
        "$=": "endswith",
        "^=": "startswith",
        "*=": "substringof",
        "endswith": "endswith",
        "startswith": "startswith",
        "contains": "substringof"
    };

    ej.data.fnOperators = {
        equal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) == toLowerCase(expected);

            return actual == expected;
        },
        notequal: function (actual, expected, ignoreCase) {
            return !ej.data.fnOperators.equal(actual, expected, ignoreCase);
        },
        lessthan: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) < toLowerCase(expected);

            return actual < expected;
        },
        greaterthan: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) > toLowerCase(expected);

            return actual > expected;
        },
        lessthanorequal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) <= toLowerCase(expected);

            return actual <= expected;
        },
        greaterthanorequal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) >= toLowerCase(expected);

            return actual >= expected;
        },
        contains: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return !isNull(actual) && !isNull(expected) && toLowerCase(actual).indexOf(toLowerCase(expected)) != -1;

            return !isNull(actual) && !isNull(expected) && actual.toString().indexOf(expected) != -1;
        },
        notnull: function (actual) {
            return actual !== null;
        },
        isnull: function (actual) {
            return actual === null;
        },
        startswith: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return actual && expected && toLowerCase(actual).startsWith(toLowerCase(expected));

            return actual && expected && actual.startsWith(expected);
        },
        endswith: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return actual && expected && toLowerCase(actual).endsWith(toLowerCase(expected));

            return actual && expected && actual.endsWith(expected);
        },

        processSymbols: function (operator) {
            var fnName = ej.data.operatorSymbols[operator];
            if (fnName) {
                var fn = ej.data.fnOperators[fnName];
                if (fn) return fn;
            }

            return throwError("Query - Process Operator : Invalid operator");
        },

        processOperator: function (operator) {
            var fn = ej.data.fnOperators[operator];
            if (fn) return fn;
            return ej.data.fnOperators.processSymbols(operator);
        }
    };

    ej.NotifierArray = function (array) {
        if (!instance(this, ej.NotifierArray))
            return new ej.NotifierArray(array);

        this.array = array;

        this._events = $({});
        this._isDirty = false;

        return this;
    };

    ej.NotifierArray.prototype = {
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },
        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },
        push: function (item) {
            var ret;

            if (instance(item, Array))
                ret = [].push.apply(this.array, item);
            else
                ret = this.array.push(item);

            this._raise("add", { item: item, index: this.length() - 1 });

            return ret;
        },
        pop: function () {
            var ret = this.array.pop();

            this._raise("remove", { item: ret, index: this.length() - 1 });

            return ret;
        },
        addAt: function (index, item) {
            this.array.splice(index, 0, item);

            this._raise("add", { item: item, index: index });

            return item;
        },
        removeAt: function (index) {
            var ret = this.array.splice(index, 1)[0];

            this._raise("remove", { item: ret, index: index });

            return ret;
        },
        remove: function (item) {
            var index = this.array.indexOf(item);

            if (index > -1) {
                this.array.splice(index, 1);
                this._raise("remove", { item: item, index: index });
            }

            return index;
        },
        length: function () {
            return this.array.length;
        },
        _raise: function (e, args) {
            this._events.triggerHandler($.extend({ type: e }, args));
            this._events.triggerHandler({ type: "all", name: e, args: args });
        },
        toArray: function () {
            return this.array;
        }
    };

    $.extend(ej, ej.dataUtil);

    // For IE8
    Array.prototype.forEach = Array.prototype.forEach || function (fn, scope) {
        for (var i = 0, len = this.length; i < len; ++i) {
            fn.call(scope, this[i], i, this);
        }
    };

    Array.prototype.indexOf = Array.prototype.indexOf || function (searchElement) {
        var len = this.length;

        if (len === 0) return -1;

        for (var i = 0; i < len; i++) {
            if (i in this && this[i] === searchElement)
                return i;
        }
        return -1;
    };

    Array.prototype.filter = Array.prototype.filter || function (fn) {
        if (typeof fn != "function")
            throw new TypeError();

        var res = [];
        var thisp = arguments[1] || this;
        for (var i = 0; i < this.length; i++) {
            var val = this[i]; // in case fun mutates this
            if (fn.call(thisp, val, i, this))
                res.push(val);
        }

        return res;
    };

    String.prototype.endsWith = String.prototype.endsWith || function (key) {
        return this.slice(-key.length) === key;
    };

    String.prototype.startsWith = String.prototype.startsWith || function (key) {
        return this.slice(0, key.length) === key;
    };

    if (!ej.support) ej.support = {};
    ej.support.stableSort = function () {
        var res = [0, 1, 2, 3, 4].sort(function () { return 0; });
        return res[0] === 0 && res[1] === 1 && res[2] === 2 && res[3] === 3 && res[4] === 4;
    }();
    ej.support.cors = $.support.cors;

    if (!$.support.cors && window.XDomainRequest) {
        var httpRegEx = /^https?:\/\//i;
        var getOrPostRegEx = /^get|post$/i;
        var sameSchemeRegEx = new RegExp('^' + location.protocol, 'i');
        var xmlRegEx = /\/xml/i;

        // ajaxTransport exists in jQuery 1.5+
        $.ajaxTransport('text html xml json', function (options, userOptions, jqXHR) {
            // XDomainRequests must be: asynchronous, GET or POST methods, HTTP or HTTPS protocol, and same scheme as calling page
            if (options.crossDomain && options.async && getOrPostRegEx.test(options.type) && httpRegEx.test(userOptions.url) && sameSchemeRegEx.test(userOptions.url)) {
                var xdr = null;
                var userType = (userOptions.dataType || '').toLowerCase();
                return {
                    send: function (headers, complete) {
                        xdr = new XDomainRequest();
                        if (/^\d+$/.test(userOptions.timeout)) {
                            xdr.timeout = userOptions.timeout;
                        }
                        xdr.ontimeout = function () {
                            complete(500, 'timeout');
                        };
                        xdr.onload = function () {
                            var allResponseHeaders = 'Content-Length: ' + xdr.responseText.length + '\r\nContent-Type: ' + xdr.contentType;
                            var status = {
                                code: 200,
                                message: 'success'
                            };
                            var responses = {
                                text: xdr.responseText
                            };

                            try {
                                if (userType === 'json') {
                                    try {
                                        responses.json = JSON.parse(xdr.responseText);
                                    } catch (e) {
                                        status.code = 500;
                                        status.message = 'parseerror';
                                        //throw 'Invalid JSON: ' + xdr.responseText;
                                    }
                                } else if ((userType === 'xml') || ((userType !== 'text') && xmlRegEx.test(xdr.contentType))) {
                                    var doc = new ActiveXObject('Microsoft.XMLDOM');
                                    doc.async = false;
                                    try {
                                        doc.loadXML(xdr.responseText);
                                    } catch (e) {
                                        doc = undefined;
                                    }
                                    if (!doc || !doc.documentElement || doc.getElementsByTagName('parsererror').length) {
                                        status.code = 500;
                                        status.message = 'parseerror';
                                        throw 'Invalid XML: ' + xdr.responseText;
                                    }
                                    responses.xml = doc;
                                }
                            } catch (parseMessage) {
                                throw parseMessage;
                            } finally {
                                complete(status.code, status.message, responses, allResponseHeaders);
                            }
                        };
                        xdr.onerror = function () {
                            complete(500, 'error', {
                                text: xdr.responseText
                            });
                        };
                        xdr.open(options.type, options.url);
                        //xdr.send(userOptions.data);
                        xdr.send();
                    },
                    abort: function () {
                        if (xdr) {
                            xdr.abort();
                        }
                    }
                };
            }
        });
    }

    $.support.cors = true;

    ej.sortOrder = {
        Ascending: "ascending",
        Descending: "descending"
    };

    // privates
    ej.pvt.consts = {
        GROUPGUID: "{271bbba0-1ee7}",
        complexPropertyMerge: "_"
    };

    // private utils
    var nextTick = function (fn, context) {
        if (context) fn = $proxy(fn, context);
        (window.setImmediate || window.setTimeout)(fn, 0);
    };

    ej.support.localizedSort = false;

    var stableSort = function (ds, field, comparer) {
        if (ej.support.stableSort) {
            if (!ej.support.localizedSort && typeof ej.getObject(field, ds[0] || {}) == "string"
                && (comparer === ej.pvt.fnAscending || comparer === ej.pvt.fnDescending))
                return fastSort(ds, field, comparer === ej.pvt.fnDescending);
            return ds.sort(ej.pvt.fnGetComparer(field, comparer));
        }
        return ej.mergeSort(ds, field, comparer);
    };

    var fastSort = function (ds, field, isDesc) {
        var old = Object.prototype.toString;
        Object.prototype.toString = (field.indexOf('.') === -1) ? function () {
            return this[field];
        } : function () {
            return ej.getObject(field, this);
        };
        ds = ds.sort();
        Object.prototype.toString = old;
        if (isDesc)
            ds.reverse();
    }

    var toLowerCase = function (val) {
        return val ? val.toLowerCase ? val.toLowerCase() : val.toString() : (val === 0 || val === false) ? val.toString() : "";
    };

    var getVal = function (array, field, index) {
        return field ? ej.pvt.getObject(field, array[index]) : array[index];
    };

    var isHtmlElement = function (e) {
        return typeof HTMLElement === "object" ? e instanceof HTMLElement :
            e && e.nodeType === 1 && typeof e === "object" && typeof e.nodeName === "string";
    };

    var instance = function (obj, element) {
        return obj instanceof element;
    };

    var getTableModel = function (name, result, dm, computed) {
        return function (tName) {
            if (typeof tName === "object") {
                computed = tName;
                tName = null;
            }
            return new ej.TableModel(tName || name, result, dm, computed);
        };
    };

    var getKnockoutModel = function (result) {
        return function (computedObservables, ko) {
            ko = ko || window.ko;

            if (!ko) throwError("Knockout is undefined");

            var model, koModels = [], prop, ob;
            for (var i = 0; i < result.length; i++) {
                model = {};
                for (prop in result[i]) {
                    if (!prop.startsWith("_"))
                        model[prop] = ko.observable(result[i][prop]);
                }
                for (prop in computedObservables) {
                    ob = computedObservables[prop];

                    if ($.isPlainObject(ob)) {
                        if (!ob.owner) ob.owner = model;
                        ob = ko.computed(ob);
                    } else
                        ob = ko.computed(ob, model);

                    model[prop] = ob;
                }
                koModels.push(model);
            }

            return ko.observableArray(koModels);
        };
    };

    var uidIndex = 0;
    var getUid = function (prefix) {
        uidIndex += 1;
        return prefix + uidIndex;
    };

    ej.getGuid = function (prefix) {
        var hexs = '0123456789abcdef', rand;
        return (prefix || "") + '00000000-0000-4000-0000-000000000000'.replace(/0/g, function (val, i) {
            rand = Math.random() * 16 | 0;
            return hexs[i === 19 ? rand & 0x3 | 0x8 : rand];
        });
    };

    var proxy = function (fn, context) {
        return function () {
            var args = [].slice.call(arguments, 0);
            args.push(this);

            return fn.apply(context || this, args);
        };
    };

    var $proxy = function (fn, context, arg) {
        if ('bind' in fn)
            return arg ? fn.bind(context, arg) : fn.bind(context);

        return function () {
            var args = arg ? [arg] : []; args.push.apply(args, arguments);
            fn.apply(context || this, args);
        };
    };

    var dateSerialize = function (date) {
        date = JSON.parse(JSON.stringify({ val: date })).val;
        return date.slice(0);
    };

    ej.merge = function (first, second) {
        if (!first || !second) return;

        Array.prototype.push.apply(first, second);
    };

    var isNull = function (val) {
        return val === undefined || val === null;
    };

    var throwError = function (er) {
        try {
            throw new Error(er);
        } catch (e) {
            throw e.message + "\n" + e.stack;
        }
    };

})(window.jQuery, window.Syncfusion, window.document);;

(function ($, ej, undefined) {

    var $document = $(document);
    // add new event shortcuts
    $.each(("touchstart touchmove touchend " +
		"tap doubletap taphold " +
		"swipe swipeleft swiperight " + "pinch pinchin pinchout pinchstop " +
		"scrollstart scrollstop").split(" "), function (i, name) {

		    $.fn[name] = function (fn) {
		        return fn ? this.bind(name, fn) : this.trigger(name);
		    };

		    // jQuery < 1.8
		    if ($.attrFn) {
		        $.attrFn[name] = true;
		    }
		});

    var isPointer = browserInfo().isMSPointerEnabled,
    isIE11Pointer = browserInfo().pointerEnabled,
    supportTouch = 'ontouchstart' in window,
	scrollEvent = "scroll",
	touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : (supportTouch ? "touchstart" : "mousedown"),
	touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : (supportTouch ? "touchend" : "mouseup"),
	touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : (supportTouch ? "touchmove" : "mousemove"),
    touchCancelEvent = isPointer ? (isIE11Pointer ? "pointercancel" : "MSPointerCancel") : (supportTouch ? "touchcancel" : "mouseleave"),
    mouseStartEvent = isPointer ? touchStartEvent : "touchstart mousedown",
    mouseStopEvent = isPointer ? touchStopEvent : "touchend mouseup",
    mouseMoveEvent = isPointer ? touchMoveEvent : "touchmove mousemove";

    function browserInfo() {
        var browser = {}, clientInfo = [],
        browserClients = {
            webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
            opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        browser.pointerEnabled = window.navigator.pointerEnabled;
        return browser;
    }
    function triggerCustomEvent(obj, eventType, event) {
        var originalType = event.type;
        event.type = eventType;
        $.event.dispatch.call(obj, event);
        event.type = originalType;
    }
    function copyObject(event, origEvent) {
        if (origEvent) {
            for (prop in origEvent) {
                if (!(prop in event)) {
                    event[prop] = origEvent[prop];
                }
            }
        }
    }
    $.event.special.ejtouchmove = {

        setup: function () {
            var thisObject = this,
				$this = $(thisObject);
            $this.bind(touchStartEvent, startMoveHandler);
            $document.bind(touchStopEvent, clearTouchMoveHandlers);
            function clearTouchMoveHandlers() {
                // $this.unbind(touchMoveEvent, moveHandler)
            }
            var coords = {};
            function startMoveHandler(event) {
                if (!(event.which && event.which !== 1)) {
                    var origTarget = event.target,
					origEvent = event.originalEvent;
                    if (isPointer)
                        coords = { x: origEvent.x, y: origEvent.y };
                    $this.bind(touchMoveEvent, moveHandler);
                }
            }
            function moveHandler(event) {
                if (!(event.which && event.which !== 1)) {
                    var origTarget = event.target,
					origEvent = event.originalEvent;
                    copyObject(event, event.originalEvent);
                    if (!isPointer || !coords || (Math.abs(coords.x - origEvent.x) > 10 || Math.abs(coords.y - origEvent.y) > 10 && isPointer))
                        triggerCustomEvent(thisObject, "ejtouchmove", event);

                }
            }



        }
    };
    function touchObj(event) {
        return event.originalEvent.touches ?
					event.originalEvent.touches[0] : isPointer ? event.originalEvent : event;
    }
    // handles swipeup and swipedown
    $.event.special.swipeupdown = {
        setup: function () {
            var thisObject = this, $this = $(thisObject);
            checkMsieTouch($this);

            $this.bind(touchStartEvent, function (event) {
                var _startevent = event;
                var data = touchObj(event),
                            start = {
                                time: (new Date).getTime(),
                                coords: [data.pageX, data.pageY],
                                origin: $(event.target)
                            },
                            stop;

                function moveHandler(event) {
                    event.preventDefault();
                    event.stopPropagation();
                    if (!start) {
                        return;
                    }

                    var data = touchObj(event);
                    stop = {
                        time: (new Date).getTime(),
                        coords: [data.pageX, data.pageY]
                    };

                    // prevent scrolling
                    if (Math.abs(start.coords[1] - stop.coords[1]) > 10) {
                        event.preventDefault();
                    }
                }

                $this
                            .bind(touchMoveEvent, moveHandler)
                            .one(touchStopEvent, function (event) {
                                event.preventDefault();
                                $this.unbind(touchMoveEvent, moveHandler);
                                if (start && stop) {
                                    if (stop.time - start.time < 1000 &&
                                    Math.abs(start.coords[1] - stop.coords[1]) > 30 &&
                                    Math.abs(start.coords[0] - stop.coords[0]) < 75) {
                                        var _addDetails = { time: stop.time - start.time, _isSwipe: true, _isDelta: true, stop: stop };
                                        var _options = _getOptions(event, _addDetails, _startevent);

                                        start.origin
                                        .trigger($.extend(true, { type: "swipeupdown" }, _options))
                                        .trigger($.extend(true, { type: start.coords[1] > stop.coords[1] ? "swipeup" : "swipedown" }, _options));
                                    }
                                }
                                start = stop = undefined;
                            });
            });
        }
    };

    // also handles scrollstop
    $.event.special.scrollstart = {

        enabled: true,

        setup: function () {
            var thisObject = this, $this = $(thisObject), scrolling, timer;

            function trigger(event, state) {
                scrolling = state;
                triggerCustomEvent(thisObject, scrolling ? "scrollstart" : "scrollstop", event);
            }

            // iPhone triggers scroll after a small delay; use touchmove instead
            $this.bind(scrollEvent, function (event) {

                if (!$.event.special.scrollstart.enabled) {
                    return;
                }

                if (!scrolling) {
                    trigger(event, true);
                }

                clearTimeout(timer);
                timer = setTimeout(function () {
                    trigger(event, false);
                }, 250);
            });
        }
    };

    // also handles doubletap, taphold
    $.event.special.tap = {
        doubleTapThreshold: 500,
        tapholdThreshold: 750,

        canDoubleTap: function (d) {
            return ((getTimeSpan() - d.doubleTapStartTime) <= $.event.special.tap.doubleTapThreshold);
        },

        setup: function () {
            var thisObject = this, $this = $(thisObject), d = $this.data();
            checkMsieTouch($this);
            d.isDoubleTapWait = false;
            d.stopProcess = false;
            d.preTouchend = null;
            d.preTouchstart = null;

            $this.bind(mouseStartEvent, function (event) {
                d = $this.data();
                d.startTime = getTimeSpan();
                if (!d.isDoubleTapWait) d.doubleTapStartTime = d.startTime;
                if (event.type == "touchstart") d.preTouchstart = d.startTime;

                // checked mousedown event arrives within 300'ms after the touchend completes
                if (event.type == "mousedown" && (d.startTime - d.preTouchend < 300 || d.startTime - d.preTouchstart < 30))
                    d.stopProcess = true;
                else d.stopProcess = false;

                var origTarget = event.currentTarget,
					origEvent = event.originalEvent,
					timer;

                function clearTapHandlers() {
                    clearTimeout(timer);

                    $this.unbind(mouseStopEvent, clickHandler);
                    $this.unbind(touchCancelEvent, clearTapHandlers);
                    $document.unbind(mouseMoveEvent, touchMoveAction);
                }
                function touchMoveAction(e) {
                    var coor1 = e, coor2 = event;
                    if (isPointer) coor1 = e.originalEvent, coor2 = event.originalEvent;
                    else if (e.type == "touchmove" && event.type == "touchmove")
                        coor1 = e.originalEvent.changedTouches[0],
                        coor2 = event.originalEvent.changedTouches[0];
                    if (!(coor1.pageX == coor2.pageX && coor1.pageY == coor2.pageY))
                        clearTapHandlers();
                }
                function clickHandler(event) {
                    if (event.type == "touchend") d.preTouchend = getTimeSpan();
                    clearTapHandlers();

                    // ONLY trigger a 'tap' event if the start target is
                    // the same as the stop target.
                    if (origTarget === event.currentTarget) {
                        triggerCustomEvent(thisObject, "tap", $.extend(_getBaseOptions(event), {
                            time: getTimeSpan() - d.startTime
                        }));

                        if (d.isDoubleTapWait && $.event.special.tap.canDoubleTap(d)) {
                            d.isDoubleTapWait = false;

                            triggerCustomEvent(thisObject, "doubletap", $.extend(_getBaseOptions(event), {
                                time: getTimeSpan() - d.doubleTapStartTime
                            }));
                        }
                        else {
                            if (d.isDoubleTapWait) {
                                d.isDoubleTapWait = false;
                                d.doubleTapStartTime = d.startTime;
                            }
                            if ($.event.special.tap.canDoubleTap(d)) {
                                d.isDoubleTapWait = true;
                            }
                        }
                    }
                }

                if (!(event.which && event.which !== 1) && !d.stopProcess) {

                    $this.bind(mouseStopEvent, clickHandler);
                    $this.bind(touchCancelEvent, clearTapHandlers);
                    $document.bind(mouseMoveEvent, touchMoveAction);

                    timer = setTimeout(function () {
                        if (d.isDoubleTapWait) d.isDoubleTapWait = false;
                        triggerCustomEvent(thisObject, "taphold", $.extend(_getBaseOptions(event), {
                            options: origEvent,
                            time: getTimeSpan() - d.startTime
                        }));

                    }, $.event.special.tap.tapholdThreshold);
                }
                else if (d.stopProcess) d.stopProcess = false;
            });
        }
    };

    // also handles swipeleft, swiperight
    $.event.special.swipe = {
        scrollSupressionThreshold: 10, // More than this horizontal displacement, and we will suppress scrolling.

        durationThreshold: 1000, // More time than this, and it isn't a swipe.

        horizontalDistanceThreshold: 30,  // Swipe horizontal displacement must be more than this.

        verticalDistanceThreshold: 75,  // Swipe vertical displacement must be less than this.
        pointers: window.navigator.msPointerEnabled,
        start: function (event) {
            var data = touchObj(event);
            return {
                time: (new Date()).getTime(),
                coords: [data.pageX, data.pageY],
                origin: $(event.target)
            };
        },

        stop: function (event) {
            var data = touchObj(event);
            return {
                time: (new Date()).getTime(),
                coords: [data.pageX, data.pageY]
            };
        },

        handleSwipe: function (start, stop, event, _startevent) {

            if (stop.time - start.time < $.event.special.swipe.durationThreshold &&
				Math.abs(start.coords[0] - stop.coords[0]) > $.event.special.swipe.horizontalDistanceThreshold &&
				Math.abs(start.coords[1] - stop.coords[1]) < $.event.special.swipe.verticalDistanceThreshold) {

                var _addDetails = { time: stop.time - start.time, _isSwipe: true, _isDelta: true, stop: stop };
                var _options = _getOptions(event, _addDetails, _startevent);

                start.origin.trigger($.extend(true, { type: "swipe" }, _options))
					.trigger($.extend(true,
                    { type: start.coords[0] > stop.coords[0] ? "swipeleft" : "swiperight" }, _options));
            }
        },

        setup: function () {
            var thisObject = this, $this = $(thisObject);
            checkMsieTouch($this);

            $this.bind(touchStartEvent, function (event) {
                var start = $.event.special.swipe.start(event),
					stop;
                var _startevent = event;
                $(event.target).data('_dataTouchStart', { event: event, _now: new Date().getTime() });

                function moveHandler(event) {
                    if (!start) {
                        return;
                    }

                    stop = $.event.special.swipe.stop(event);

                    // prevent scrolling
                    if (Math.abs(start.coords[0] - stop.coords[0]) > $.event.special.swipe.scrollSupressionThreshold) {
                        event.preventDefault();
                    }
                }

                $this.bind(touchMoveEvent, moveHandler)
					.one(touchStopEvent, function (event) {
					    $this.unbind(touchMoveEvent, moveHandler);

					    if (start && stop) {
					        $.event.special.swipe.handleSwipe(start, stop, event, _startevent);
					    }
					    start = stop = undefined;
					});
            });
        }
    };
    // also handles pinchin, pinchout
    $.event.special.pinch = {

        distance: function (event) {
            if (event.originalEvent.touches.length < 2) return null;
            return $.event.special.pinch._getdistance(event.originalEvent.touches[0], event.originalEvent.touches[1]);
        },
        _getdistance: function (coor1, coor2) {
            return Math.sqrt((coor1.pageX - coor2.pageX) * (coor1.pageX - coor2.pageX) +
                (coor1.pageY - coor2.pageY) * (coor1.pageY - coor2.pageY));
        },

        setup: function () {
            var thisObject = this, $this = $(thisObject);
            checkMsieTouch($this);

            $this.bind(touchStartEvent, function (event) {
                var _startevent = event;
                if (event.originalEvent.touches && event.originalEvent.touches.length >= 2) {
                    var start = $.event.special.pinch.distance(event), stop, minDistance = 5;

                    var _pinchDistance = start, moveEvent;
                    var _options = _getOptions(event, { _isPinch: true, _pinchDistance: _pinchDistance }, _startevent);

                    $(event.target).trigger($.extend(true, { type: "pinch" }, _options));

                    function moveHandler(event) {
                        moveEvent = event;
                        stop = $.event.special.pinch.distance(event) || null;
                        if (start && stop && Math.abs(start - stop) > minDistance) {

                            $(event.target).trigger($.extend(true,
                                { type: start > stop ? "pinchin" : "pinchout" }, _getOptions(event,
                                { _isPinch: true, _pinchDistance: _pinchDistance }, _startevent)));

                            start = stop;
                        }
                    }

                    $this.bind(touchMoveEvent, moveHandler)
                        .one(touchStopEvent, function () {
                            $this.unbind(touchMoveEvent, moveHandler);
                            $(event.target).trigger($.extend(true, { type: "pinchstop" }, _getOptions(moveEvent,
                                { _isPinch: true, _pinchDistance: stop }, _startevent)));
                            start = stop = undefined;
                        });
                }
            });
        }
    };
    //  handles touchdrag, touchdrag
    $.event.special.touchdrag = {

        setup: function () {
            var thisObject = this, $this = $(thisObject);
            checkMsieTouch($this);

            $this.bind(touchStartEvent, function (event) {

                var start = touchObj(event),
					stop;
                var _startevent = event;
                $(event.target).data('_dataTouchStart', { event: event, _now: new Date().getTime() });

                function moveHandler(event) {
                    if (!start) {
                        return;
                    }
                    stop = touchObj(event);

                    if ($.event.special.pinch._getdistance(start, stop) > 5)
                        $(event.target).trigger($.extend(true, { type: "touchdrag" },
                            _getOptions(event, { _isdrag: true, stop: stop, _isDelta: true }, _startevent)));
                }

                $this.bind(touchMoveEvent, moveHandler)
					.one(touchStopEvent, function (event) {
					    $this.unbind(touchMoveEvent, moveHandler);
					    start = stop = undefined;
					});
            });
        }
    };
    function _getBaseOptions(e) {
        var _pointer = isPointer ? e.originalEvent.pointerType : (!e.originalEvent.touches ? "mouse" : "touch")
        var pointerType = (isPointer && isIE11Pointer == undefined) ? (_pointer == 4 ? "mouse" : "touch") : _pointer;
        e["pointerType"] = pointerType;
        if (e.type != "mousedown" && e.type != "mouseup") {
            copyObject(e, e.originalEvent);
        }
        return e;
    }
    function _getOptions(event, _details, _startevent) {
        var _distance, _time, _scale, _iDelta = {}, _startXY, _endXY;
        if (_startevent) {
            var coor;
            if (!_startevent.originalEvent.touches) coor = [_startevent.originalEvent, event.originalEvent];
            else coor = [_startevent.originalEvent.touches[0], event.originalEvent.changedTouches[0]];

            if (_details._isSwipe || _details._isdrag) {
                _distance = $.event.special.pinch._getdistance(coor[0], coor[1]);
                _time = _details.time;
                _endXY = { pageX: _details.stop.pageX, pageY: _details.stop.pageY };
            }
            else if (_details._isPinch) {
                _distance = $.event.special.pinch.distance(event);
                _time = event.timeStamp - _startevent.timeStamp;
                _scale = _details._pinchDistance;
            }
            if (_details._isDelta) {
                _iDelta._dTime = event.timeStamp - _startevent.timeStamp;
                _iDelta._x = coor[1].pageX - coor[0].pageX;
                _iDelta._y = coor[1].pageY - coor[0].pageY;
            }
        }

        return {
            options: event,
            delta: {
                time: _iDelta._dTime || null,
                X: _iDelta._x || null,
                Y: _iDelta._y || null
            },
            distance: _distance,
            scale: _details._isPinch ? _scale : null,
            time: _time,
            velocity: {
                XY: _distance / _iDelta._dTime || null,
                X: _iDelta._x / _iDelta._dTime || null,
                Y: _iDelta._y / _iDelta._dTime || null
            },
            currentPosition: { pageX: _endXY ? _endXY.pageX : null, pageY: _endXY ? _endXY.pageY : null }
        };
    }

    function getTimeSpan() {
        var now = new Date();
        return now.getTime();
    }
    function checkMsieTouch($this) {
        if (isPointer)
            $this.css("-ms-touch-action", "pinch-zoom").css("touch-action", "pinch-zoom");
    }

    $.each({
        scrollstop: "scrollstart",
        doubletap: "tap",
        taphold: "tap",
        swipeleft: "swipe",
        swiperight: "swipe",
        swipedown: "swipeupdown",
        swipeup: "swipeupdown",
        pinchin: "pinch",
        pinchout: "pinch",
        pinchstop: "pinch"
    }, function (event, sourceEvent) {

        $.event.special[event] = {
            setup: function () {
                $(this).bind(sourceEvent, $.noop);
            }
        };
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to drag the html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejDraggable
	* @requires jQuery
	* @requires ej.core
	* @classdesc Custom Design for dragging HTML elements.
	* @example 
	* &lt;div  id="dragable" &gt;&lt;/ div &gt; <br> 
	* &lt;script&gt;
	* // Create Dragable
    * $('#dragable').ejDraggable(); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejDraggable", "ej.Draggable", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span", "a"],

        // default model
        defaults: {
            /**		
			* Used to group sets of draggable and droppable items, in addition to droppable's accept option. A draggable with the same scope value as a droppable will be accepted by the droppable.
			* @default 'default'
			* @type {string}
			* @example 
			* //To set scope API value during initialization  
			* 	$("#dragable").ejDraggable({ scope: 'default' });				* 
			* @example 
			* //Get or set the scope API, after initialization:<br>
			*	//Gets the scope value  
			*	$("#dragable").ejDraggable('option', 'scope');<br>			
			* @memberof ejDraggable
			* @instance
			*/
            scope: 'default', /*Used to group sets of draggable and droppable items, in addition to droppable's accept option. A draggable with the same scope value as a droppable will be accepted by the droppable.*/
            /**		
			* If specified, restricts drag start click to the specified element(s).
			* @default null
			* @type {string}
			* @example 
			* //To set handle API value during initialization  
			* 	$("#dragable").ejDraggable({ handle: null });				* 
			* @example 
			* //Get or set the handle API, after initialization:<br>
			*	//Gets the handle value  
			*	$("#dragable").ejDraggable('option', 'handle');<br>			
			* @memberof ejDraggable
			* @instance
			*/
            handle: null,  /*If specified, restricts drag start click to the specified element(s).*/
            /**		
			* If Drag area is specified.
			* @default false
			* @type {boolean}
			* @example 
			* //To set dragArea API value during initialization  
			* 	$("#dragable").ejDraggable({ dragArea: true });				* 
			* @example 
			* //Get or set the dragArea API, after initialization:<br>
			*	//Gets the dragArea value  
			*	$("#dragable").ejDraggable('option', 'dragArea');<br>			
			* @memberof ejDraggable
			* @instance
			*/
            dragArea: false,
            /**		
            * If clone is specified.
            * @default false
            * @type {boolean}
            * @example 
            * //To set clone API value during initialization  
            * 	$("#dragable").ejDraggable({ clone: true });				* 
            * @example 
            * //Get or set the clone API, after initialization:<br>
            *	//Gets the clone value  
            *	$("#dragable").ejDraggable('option', 'clone');<br>			
            * @memberof ejDraggable
            * @instance
            */
            clone: false,
            /**		
            * Distance in pixels after mousedown the mouse must move before dragging should start. This option can be used to prevent unwanted drags when clicking on an element.
            * @default 1
            * @type {number}
            * @example 
            * //To set distance API value during initialization  
            * 	$("#dragable").ejDraggable({ distance: 1 });				* 
            * @example 
            * //Get or set the distance API, after initialization:<br>
            *	//Gets the distance value  
            *	$("#dragable").ejDraggable('option', 'distance');<br>			
            * @memberof ejDraggable
            * @instance
            */
            distance: 1, /* Distance in pixels after mousedown the mouse must move before dragging should start. This option can be used to prevent unwanted drags when clicking on an element. */
            /**		
            *Sets the offset of the dragging helper relative to the mouse cursor.
            * @default { top: -1, left: -2 }
            * @type {object}
            * @example 
            * //To set cursorAt API value during initialization  
            * 	$("#dragable").ejDraggable({ cursorAt:  { top: 1, left: -2 } });				* 
            * @example 
            * //Get or set the cursorAt API, after initialization:<br>
            *	//Gets the cursorAt value  
            *	$("#dragable").ejDraggable('option', 'cursorAt');<br>			
            * @memberof ejDraggable
            * @instance
            */
            cursorAt: { top: -1, left: -2 }, /* Sets the offset of the dragging helper relative to the mouse cursor.  */
            /**     
			* Supply a callback function to handle the drag start event as an init option.
			* @event
			* @name ejDraggable#dragStart 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the autocomplete model
			* @param {string}  argument.type returns the name of the event
			* @example 
			* //dragStart event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	dragStart: function(args) {}
            * });      
			* @memberof ejDraggable
			* @instance
			*/
            dragStart: null, /* Supply a callback function to handle the drag start event as an init option. */
            /**     
			* This event is triggered when the mouse is moved during the dragging.
			* @event
			* @name ejDraggable#drag 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the autocomplete model
			* @param {string}  argument.type returns the name of the event
			* @example 
			* //drag event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	drag: function(args) {}
            * });      
			* @memberof ejDraggable
			* @instance
			*/
            drag: null, /* This event is triggered when the mouse is moved during the dragging. */
            /**     
            * This event is triggered when the mouse is moved during the dragging.
            * @event
            * @name ejDraggable#dragStop 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //dragStop event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	dragStop: function(args) {}
            * });      
            * @memberof ejDraggable
            * @instance
            */
            dragStop: null, /* This event is triggered when dragging stops. */
            /**     
            * This event is triggered when dragging events are destroyed.
            * @event
            * @name ejDraggable#destroy 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //destroy event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	destroy: function(args) {}
            * });      
            * @memberof ejDraggable
            * @instance
            */
            destroy: null, /* This event is triggered when dragging events are destroyed. */
            /**     
            * This event is triggered when dragged.
            * @event
            * @name ejDraggable#helper 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //helper event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	helper: function () {
                return $('<div class="e-drag-helper" />').html("draggable").appendTo(document.body);}
            * });      
            * @memberof ejDraggable
            * @instance
            */
            helper: function () {
                return $('<div class="e-drag-helper" />').html("draggable").appendTo(document.body);
            }
        },

        // constructor function
        _init: function () {
            this.handler = function () { },
			this.resizables = {},
			helpers = {};
            this._wireEvents();
        },

        _setModel: function (options) {

        },
        /**
        * destroy in the dragable.
        * @return jQuery
        * @example 
        * &lt; div  id="dragable" &gt; &lt;/div &gt; <br> 
        * &lt;script&gt;
        * // Create dragableObj
        * var dragableObj  = $("#dragable").data("ejDraggable");
        * dragableObj.destroy(); 
        * &lt;/script&gt;
        *@memberof ejDraggable
        * @instance
        */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            $(document)
                .unbind(ej.eventType.mouseUp, this._destroyHandler)
                .unbind(ej.eventType.mouseUp, this._dragStopHandler)
                .unbind(ej.eventType.mouseMove, this._dragStartHandler)
                .unbind(ej.eventType.mouseMove, this._dragHandler)
                .unbind('selectstart', false);

            // this._raise(e, this.destroy);

            ej.widgetBase.droppables[this.scope] = null;
            //helpers[this.handle] = null;
        },

        _initialize: function (e) {
            var ori = e;
            e.preventDefault();
            e = this._getCoordinate(e);
            this.target = $(ori.currentTarget);
            this._initPosition = { x: e.pageX, y: e.pageY };

            $(document).bind(ej.eventType.mouseMove, this._dragStartHandler).bind(ej.eventType.mouseUp, this._destroyHandler);
            if (!this.model.clone) {
                var _offset = this.element.offset();
                this._relXposition = e.pageX - _offset.left;
                this._relYposition = e.pageY - _offset.top;
            }
            $(document.documentElement).trigger(ej.eventType.mouseDown, ori); // The next statement will prevent 'mousedown', so manually trigger it.
            //return false;
        },
        _setDragArea: function () {
            var _dragElement = $(this.model.dragArea)[0]; if (!_dragElement) return;
            var over = ($(_dragElement).css("overflow") != 'hidden');

            this.dragArea = [
                (parseInt($(_dragElement).css("borderLeftWidth"), 10) || 0) + (parseInt($(_dragElement).css("paddingLeft"), 10) || 0),
                (parseInt($(_dragElement).css("borderTopWidth"), 10) || 0) + (parseInt($(_dragElement).css("paddingTop"), 10) || 0),
                (over ? Math.max(_dragElement.scrollWidth, _dragElement.offsetWidth) : _dragElement.offsetWidth) - (parseInt($(_dragElement).css("borderLeftWidth"), 10) || 0) - (parseInt($(_dragElement).css("paddingRight"), 10) || 0) - this.helper.outerWidth() - this.margins.left - this.margins.right,
                (over ? Math.max(_dragElement.scrollHeight, _dragElement.offsetHeight) : _dragElement.offsetHeight) - (parseInt($(_dragElement).css("borderTopWidth"), 10) || 0) - (parseInt($(_dragElement).css("paddingBottom"), 10) || 0) - this.helper.outerHeight() - this.margins.top - this.margins.bottom
            ];
            this.relativeArea = $(this.model.dragArea);
        },
        _dragStart: function (e) {
            var ori = e;
            e = this._getCoordinate(e);

            this.margins = {
                left: (parseInt(this.element.css("marginLeft"), 10) || 0),
                top: (parseInt(this.element.css("marginTop"), 10) || 0),
                right: (parseInt(this.element.css("marginRight"), 10) || 0),
                bottom: (parseInt(this.element.css("marginBottom"), 10) || 0)
            };
            this.offset = this.element.offset();
            this.offset = {
                top: this.offset.top - this.margins.top,
                left: this.offset.left - this.margins.left
            };
            this.position = this._getMousePosition(ori);
            var x = this._initPosition.x - e.pageX, y = this._initPosition.y - e.pageY;
            var distance = Math.sqrt((x * x) + (y * y));

            if (distance >= this.model.distance) {
                var dragTargetElmnt = this.model.handle = this.helper = this.model.helper({ sender: ori, element: this.target });
                if (this.model.dragArea)
                    this._setDragArea();

                //if (!this.model.clone)
                //    this.position.left = this.position.left - (ori.target.offsetWidth / 2);
                //this.position.top = this.position.top - (ori.target.offsetHeight - ori.offsetY);

                if (this.model.dragStart) {
                    var currTarget = null;
                    if (ori.type == 'touchmove') {
                        var coor = ori.originalEvent.changedTouches[0];
                        currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                    }
                    else currTarget = ori.originalEvent.target || ori.target;
                    if (this._trigger("dragStart", { event: ori, element: this.element, target: currTarget })) {
                        this._destroy();
                        return false;
                    }
                }

                $(document).unbind(ej.eventType.mouseMove, this._dragStartHandler).unbind(ej.eventType.mouseUp, this._destroyHandler)
                    .bind(ej.eventType.mouseMove, this._dragHandler).bind(ej.eventType.mouseUp, this._dragStopHandler).bind("selectstart", false);
                ej.widgetBase.droppables[this.model.scope] = {
                    draggable: this.element,
                    helper: dragTargetElmnt.css({ position: 'absolute', left: this.position.left, top: this.position.top }),
                    destroy: this._destroyHandler
                }
            }
        },

        _drag: function (e) {
            this.position = this._getMousePosition(e);
            if (this.position.top < 0)
                this.position.top = 0;
            if ($(document).height() < this.position.top)
                this.position.top = $(document).height();
            if ($(document).width() < this.position.left)
                this.position.left = $(document).width();
            var helperElement = ej.widgetBase.droppables[this.model.scope].helper;
            if (this.model.drag) {
                var currTarget = null;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                else currTarget = e.originalEvent.target || e.target;
                this._trigger("drag", { event: e, element: this.target, target: currTarget });// Raise the dragging event
            }
            ej.widgetBase.droppables[this.model.scope].helper.css({ left: this.position.left, top: this.position.top });
        },

        _dragStop: function (e) {
            if (e.type == 'mouseup' || e.type == 'touchend')
                this._destroy(e);
            if (this.model.dragStop) {
                var currTarget = null;
                if (e.type == 'touchend') {
                    var coor = e.originalEvent.changedTouches[0];
                    currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                else currTarget = e.originalEvent.target || e.target;
                this._trigger("dragStop", { event: e, element: this.target, target: currTarget });// Raise the dragstop event
            }
        },

        _wireEvents: function () {
            this._on(this.element, ej.eventType.mouseDown, this._initialize);
            this._dragStartHandler = $.proxy(this._dragStart, this);
            this._destroyHandler = $.proxy(this._destroy, this);
            this._dragStopHandler = $.proxy(this._dragStop, this);
            this._dragHandler = $.proxy(this._drag, this);
        },
        _getMousePosition: function (event) {
            event = this._getCoordinate(event);
            var pageX = this.model.clone ? event.pageX : event.pageX - this._relXposition;
            var pageY = this.model.clone ? event.pageY : event.pageY - this._relYposition;
            if (this.dragArea) {
                if (this.relativeArea) {
                    var co = this.relativeArea.offset();
                    _area = [this.dragArea[0] + co.left, this.dragArea[1] + co.top, this.dragArea[2] + co.left, this.dragArea[3] + co.top];
                }
                else {
                    _area = this.dragArea;
                }
                if (event.pageX - this.margins.left < _area[0]) pageX = _area[0] + this.margins.left;
                if (event.pageY - this.margins.top < _area[1]) pageY = _area[1] + this.margins.top;
                if (event.pageX - this.margins.left > _area[2]) pageX = _area[2] + this.margins.left;
                if (event.pageY - this.margins.top > _area[3]) pageY = _area[3] + this.margins.top;
            }
            return { left: pageX - this.margins.left - this.model.cursorAt.left, top: pageY - this.margins.top - this.model.cursorAt.top };
        },
        _getCoordinate: function (evt) {
            var coor = evt;
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend")
                coor = evt.originalEvent.changedTouches[0];
            return coor;
        }
    });

})(jQuery, Syncfusion);

/**
* @fileOverview Plugin to drop the html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejDroppable
   * @requires jQuery
   * @requires ej.core
   * @classdesc Custom Design for droping HTML elements.
   * @example 
   * &lt;div  id="dropable" &gt;&lt;/ div &gt; <br> 
   * &lt;script&gt;
   * // Create Dragable
   * $('#dropable').ejDroppable(); 	
   * &lt;/script&gt;
   */
    // Example plugin creation code
    // sfSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejDroppable", "ej.Droppable", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span", "a"],

        // default model
        defaults: {
            /**		
			* Used to accept the droppable items.
			* @default null
			* @type {object}
			* @example 
			* //To set scope API value during initialization  
			* 	$("#droppable").ejDroppable({ accept: null });				* 
			* @example 
			* //Get or set the accept API, after initialization:<br>
			*	//Gets the accept value  
			*	$("#droppable").ejDroppable('option', 'accept');<br>			
			* @memberof ejDroppable
			* @instance
			*/
            accept: null,
            /**		
			* Used to group sets of droppable items, in addition to droppable's accept option. A draggable with the same scope value as a droppable will be accepted by the droppable.
			* @default 'default'
			* @type {string}
			* @example 
			* //To set scope API value during initialization  
			* 	$("#droppable").ejDroppable({ scope: 'default' });				* 
			* @example 
			* //Get or set the scope API, after initialization:<br>
			*	//Gets the scope value  
			*	$("#droppable").ejDroppable('option', 'scope');<br>			
			* @memberof ejDroppable
			* @instance
			*/
            scope: 'default',
            /**     
			* This event is triggered when the mouse up is moved during the dragging.
			* @event
			* @name ejDroppable#drop 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the autocomplete model
			* @param {string}  argument.type returns the name of the event
			* @example 
			* //drop event for Droppable
            * $("#droppable").ejDroppable({ 
            *   	drop: function(args) {}
            * });      
			* @memberof ejDroppable
			* @instance
			*/
            drop: null,
            /**     
            * This event is triggered when the mouse  is moved over.
            * @event
            * @name ejDroppable#over 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //drop event for Droppable
            * $("#droppable").ejDroppable({ 
            *   	over: function(args) {}
            * });      
            * @memberof ejDroppable
            * @instance
            */
            over: null,
            /**     
            * This event is triggered when the mouse  is moved out.
            * @event
            * @name ejDroppable#out 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //drop event for Droppable
            * $("#droppable").ejDroppable({ 
            *   	out: function(args) {}
            * });      
            * @memberof ejDroppable
            * @instance
            */
            out: null
        },

        // constructor function
        _init: function () {
            if (this.model.accept) {
                $(this.element).delegate(this.accept, 'mouseenter', $.proxy(this._over, this))
							.delegate(this.accept, 'mouseup', $.proxy(this._drop, this))
							.delegate(this.accept, 'mouseleave', $.proxy(this._out, this));
            }
            else {
                $(this.element).bind('mouseup', $.proxy(this._drop, this));
                $(this.element).bind("mouseenter", $.proxy(this._over, this));
                $(this.element).bind("mouseleave", $.proxy(this._out, this));
            }
            this._on($(document), 'touchend', this._drop);
        },

        _setModel: function (options) {

        },
        /**
        * destroy in the Droppable.
        * @return jQuery
        * @example 
        * &lt; div  id="droppable" &gt; &lt;/div &gt; <br> 
        * &lt;script&gt;
        * // Create droppabaleObj
        * var droppabaleObj  = $("#droppable").data("ejDroppable");
        * droppabaleObj.destroy(); 
        * &lt;/script&gt;
        *@memberof ejDroppable
        * @instance
        */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {

        },

        _over: function (e) {
            this._trigger(e, this.model.over);
        },
        _out: function (e) {
            this._trigger(e, this.model.out);
        },
        _drop: function (e) {
            var drag = ej.widgetBase.droppables[this.model.scope];
            var isDragged = !ej.isNullOrUndefined(drag.helper) && drag.helper.is(":visible");
            var area = this._isDropArea(e);
            if (drag && !ej.isNullOrUndefined(this.model.drop) && isDragged && area.canDrop) {
                this.model.drop($.extend(e, { dropTarget: area.target }, true), drag);
            }
        },
        _isDropArea: function (e) {
            // check for touch devices only
            var area = { canDrop: true, target: $(e.target) };
            if (e.type == "touchend") {
                var coor = e.originalEvent.changedTouches[0], _target;
                _target = document.elementFromPoint(coor.pageX, coor.pageY);
                area.canDrop = false;
                var _parents = $(_target).parents();

                for (var i = 0; i < this.element.length; i++) {
                    if ($(_target).is($(this.element[i]))) area = { canDrop: true, target: $(_target) };
                    else for (var j = 0; j < _parents.length; j++) {
                        if ($(this.element[i]).is($(_parents[j]))) {
                            area = { canDrop: true, target: $(_target) };
                            break;
                        }
                    }
                    if (area.canDrop) break;
                }
            }
            return area;
        }
    });

})(jQuery, Syncfusion);

/**
* @fileOverview Plugin to resize the Html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejResizable
    * @requires jQuery
    * @requires ej.core
    * @classdesc Custom Design for resizing HTML elements.
    * @example 
    * &lt;div  id="resizing" &gt;&lt;/ div &gt; <br> 
    * &lt;script&gt;
    * // Create resizing
    * $('#resizing').ejResizable(); 	
    * &lt;/script&gt;
    */
    ej.widget("ejResizable", "ej.resizable", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span", "a"],

        // default model
        defaults: {
            /**		
           * Used to group sets of resizeable items.
           * @default 'default'
           * @type {string}
           * @example 
           * //To set scope API value during initialization  
           * 	$("#resizing").ejResizable({ scope: 'default' });				* 
           * @example 
           * //Get or set the scope API, after initialization:<br>
           *	//Gets the scope value  
           *	$("#resizing").ejResizable('option', 'scope');<br>			
           * @memberof ejResizable
           * @instance
           */
            scope: 'default',
            /**		
			* If specified, restricts resize start click to the specified element(s).
			* @default null
			* @type {string}
			* @example 
			* //To set handle API value during initialization  
			* 	$("#resizing").ejResizable({ handle: null });				* 
			* @example 
			* //Get or set the handle API, after initialization:<br>
			*	//Gets the handle value  
			*	$("#resizing").ejResizable('option', 'handle');<br>			
			* @memberof ejResizable
			* @instance
			*/
            handle: null,
            /**		
            * Distance in pixels after mousedown the mouse must move before resizing should start. This option can be used to prevent unwanted drags when clicking on an element.
            * @default 1
            * @type {number}
            * @example 
            * //To set distance API value during initialization  
            * 	$("#resizing").ejResizable({ distance: 1 });				* 
            * @example 
            * //Get or set the distance API, after initialization:<br>
            *	//Gets the distance value  
            *	$("#resizing").ejResizable('option', 'distance');<br>			
            * @memberof ejResizable
            * @instance
            */
            distance: 1,
            /**		
            *Sets the max height for resizing
            * @default null
            * @type {number}
            * @example 
            * //To set maxHeight API value during initialization  
            * 	$("#resizing").ejResizable({ maxHeight: null });				* 
            * @example 
            * //Get or set the maxHeight API, after initialization:<br>
            *	//Gets the maxHeight value  
            *	$("#resizing").ejResizable('option', 'maxHeight');<br>			
            * @memberof ejResizable
            * @instance
            */
            maxHeight: null,
            /**		
            *Sets the max width for resizing
            * @default null
            * @type {number}
            * @example 
            * //To set maxWidth API value during initialization  
            * 	$("#resizing").ejResizable({ maxWidth: null });				* 
            * @example 
            * //Get or set the maxWidth API, after initialization:<br>
            *	//Gets the maxWidth value  
            *	$("#resizing").ejResizable('option', 'maxWidth');<br>			
            * @memberof ejResizable
            * @instance
            */
            maxWidth: null,
            /**		
            *Sets the min Height for resizing
            * @default 10
            * @type {number}
            * @example 
            * //To set minHeight API value during initialization  
            * 	$("#resizing").ejResizable({ minHeight: null });				* 
            * @example 
            * //Get or set the minHeight API, after initialization:<br>
            *	//Gets the minHeight value  
            *	$("#resizing").ejResizable('option', 'minHeight');<br>			
            * @memberof ejResizable
            * @instance
            */
            minHeight: 10,
            /**		
            *Sets the min Width for resizing
            * @default 10
            * @type {number}
            * @example 
            * //To set minWidth API value during initialization  
            * 	$("#resizing").ejResizable({ minWidth: null });				* 
            * @example 
            * //Get or set the minWidth API, after initialization:<br>
            *	//Gets the minWidth value  
            *	$("#resizing").ejResizable('option', 'minWidth');<br>			
            * @memberof ejResizable
            * @instance
            */
            minWidth: 10,
            /**		
            *Sets the offset of the resizing helper relative to the mouse cursor.
            * @default { top: -1, left: -2 }
            * @type {object}
            * @example 
            * //To set cursorAt API value during initialization  
            * 	$("#resizing").ejResizable({ cursorAt:  { top: 1, left: -2 } });				* 
            * @example 
            * //Get or set the cursorAt API, after initialization:<br>
            *	//Gets the cursorAt value  
            *	$("#resizing").ejResizable('option', 'cursorAt');<br>			
            * @memberof ejResizable
            * @instance
            */
            cursorAt: { top: 1, left: 1 },
            /**     
            * This event is triggered when the resizing is start.
            * @event
            * @name ejResizable#resizeStart 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //resizeStart event for Resizable
            * $("#resizing").ejResizable({ 
            *   	resizeStart: function(args) {}
            * });      
            * @memberof ejResizable
            * @instance
            */
            resizeStart: null,
            /**     
            * This event is triggered when the resizing in progress.
            * @event
            * @name ejResizable#resize 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //resizeStart event for Resizable
            * $("#resizing").ejResizable({ 
            *   	resize: function(args) {}
            * });      
            * @memberof ejResizable
            * @instance
            */
            resize: null,
            /**     
            * This event is triggered when the resizing is stop.
            * @event
            * @name ejResizable#resizeStop 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //resizeStop event for Resizable
            * $("#resizing").ejResizable({ 
            *   	resizeStop: function(args) {}
            * });      
            * @memberof ejResizable
            * @instance
            */
            resizeStop: null,
            /**     
           * This event is triggered when the widget destroys.
           * @event
           * @name ejResizable#resizeStop 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the autocomplete model
           * @param {string}  argument.type returns the name of the event
           * @example 
           * //destroy event for Resizable
           * $("#resizing").ejResizable({ 
           *   	destroy: function(args) {}
           * });      
           * @memberof ejResizable
           * @instance
           */
            destroy: null,
            /**     
           * This event is triggered when resized.
           * @event
           * @name ejResizable#helper 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the autocomplete model
           * @param {string}  argument.type returns the name of the event
           * @example 
           * //helper event for Resizable
           * $("#resizing").ejResizable({ 
           *   	helper: function () {
               return $('<div class="e-resize-helper" />').html("resizable").appendTo(document.body);}
           * });      
           * @memberof ejResizable
           * @instance
           */
            helper: function () {
                return $('<div class="e-resize-helper" />').html("resizable").appendTo(document.body);
            }
        },
        // constructor function
        _init: function () {
            this.target = this.element;
            if (this.handle != null) {
                $(this.target).delegate(this.handle, ej.eventType.mouseDown, $.proxy(this._mousedown, this))
                .delegate(this.handle, 'resizestart', this._blockDefaultActions);
            }
            else {
                //$(this.target).bind("mousedown", $.proxy(this._mousedown, this));
                $(this.target).bind("mouseover", $.proxy(this._mouseover, this));
                // .bind("resizestart", $.proxy(this, $.Handler(this._init, this)));
            }

            this._resizeStartHandler = $.proxy(this._resizeStart, this);
            this._destroyHandler = $.proxy(this._destroy, this);
            this._resizeStopHandler = $.proxy(this._resizeStop, this);
            this._resizeHandler = $.proxy(this._resize, this);
        },
        _mouseover: function (e) {
            if ($(e.target).hasClass("e-resizable")) {
                $(e.target).css({ cursor: "se-resize" });
                $(this.target).bind(ej.eventType.mouseDown, $.proxy(this._mousedown, this));
            }
            else {
                $(this.target).unbind(ej.eventType.mouseDown);
                $(this.target).css({ cursor: "" });
            }
        },
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
        _setModel: function (options) {

        },
        _mousedown: function (e) {
            var ori = e;
            e = this._getCoordinate(e);
            this.target = $(ori.currentTarget);
            this._initPosition = { x: e.pageX, y: e.pageY };

            $(document).bind(ej.eventType.mouseMove, this._resizeStartHandler).bind(ej.eventType.mouseUp, this._destroyHandler);

            $(document.documentElement).trigger(ej.eventType.mouseDown, ori); // The next statement will prevent 'mousedown', so manually trigger it.
            return false;
        },

        _resizeStart: function (e) {
            if ($(e.target).hasClass("e-resizable")) {
                e = this._getCoordinate(e);
                var x = this._initPosition.x - e.pageX, y = this._initPosition.y - e.pageY;
                var distance = Math.sqrt((x * x) + (y * y));
                if (distance >= this.model.distance) {
                    if (this.model.resizeStart != null)
                        this.model.resizeStart({ element: this.target });  // Raise the resize start event
                    var resizeTargetElmnt = this.model.helper({ element: this.target });
                    var pos = this.getElementPosition(resizeTargetElmnt);
                    $(document).unbind(ej.eventType.mouseMove, this._resizeStartHandler).unbind(ej.eventType.mouseUp, this._destroyHandler)
                        .bind(ej.eventType.mouseMove, this._resizeHandler).bind(ej.eventType.mouseUp, this._resizeStopHandler).bind("selectstart", false);
                    ej.widgetBase.resizables[this.scope] = {
                        resizable: this.target,
                        helper: resizeTargetElmnt.css({ width: (e.pageX - pos.left) + this.model.cursorAt.left, height: (e.pageY - pos.top) + this.model.cursorAt.top }),
                        destroy: this._destroyHandler
                    }
                }
            }
        },

        _resize: function (e) {
            e = this._getCoordinate(e);
            if (this.model.resize != null)
                this.model.resize({ element: this.target });  // Raise the drag start event
            var pos = this.getElementPosition(ej.widgetBase.resizables[this.scope].helper);
            var _width = (e.pageX - pos.left) + this.model.cursorAt.left;
            var _height = (e.pageY - pos.top) + this.model.cursorAt.top;
            if (_width < this.model.minWidth)
                _width = this.model.minWidth;
            if (_height < this.model.minHeight)
                _height = this.model.minHeight;
            if (this.model.maxHeight != null && _height > this.model.maxHeight)
                _height = this.model.maxHeight;
            if (this.model.maxWidth != null && _width > this.model.maxWidth)
                _width = this.model.maxWidth;
            ej.widgetBase.resizables[this.scope].helper.css({ width: _width, height: _height });
        },

        _resizeStop: function (e) {
            if (this.model.resizeStop != null)
                this.model.resizeStop({ element: this.target });  // Raise the resize stop event
            if (e.type == 'mouseup' || e.type == 'touchend')
                this._destroy(e);
        },
        /**
        * destroy in the Resizable.
        * @return jQuery
        * @example 
        * &lt; div  id="resizing" &gt; &lt;/div &gt; <br> 
        * &lt;script&gt;
        * // Create resizingObj
        * var resizingObj  = $("#resizing").data("ejResizable");
        * resizingObj.destroy(); 
        * &lt;/script&gt;
        *@memberof ejResizable
        * @instance
        */
        _destroy: function (e) {
            $(document)
                .unbind(ej.eventType.mouseUp, this._destroyHandler)
                .unbind(ej.eventType.mouseUp, this._resizeStopHandler)
                .unbind(ej.eventType.mouseMove, this._resizeStartHandler)
                .unbind(ej.eventType.mouseMove, this._resizeHandler)
                .unbind('selectstart', false);

            // this._raise(e, this.destroy);

            ej.widgetBase.resizables[this.scope] = null;
            //helpers[this.handle] = null;
        },

        getElementPosition: function (elemnt) {
            if (elemnt != null && elemnt.length > 0)
                return {
                    left: elemnt[0].offsetLeft,
                    top: elemnt[0].offsetTop
                };
            else
                return null;
        },
        _getCoordinate: function (evt) {
            var coor = evt;
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend")
                coor = evt.originalEvent.changedTouches[0];
            return coor;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, window, undefined) {
    'use strict';
    /**
* @namespace ej
* @class ejScroller
* @requires jQuery
* @requires ej.common.all
* @classdesc Custom Design for Html Scroller control.
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Create Scroller
* $('#scrollcontent').ejScroller(); 	
* &lt;/script&gt;
*/
    ej.widget("ejScroller", "ej.Scroller", {
        defaults: {
            /**		
* Specifies the height of Scroll panel and scrollbars.
* @default 250
* @type {number}
* @example 
* //To set height API value during initialization  
* 	$("#scrollcontent").ejScroller({height: 200 });	
* @example 
* //Get or set the height API, after initialization:<br>
*	//Gets the height value
*	$("#scrollcontent").ejScroller("option", "height"); 			
*	//Sets the height value
*	$("#scrollcontent").ejScroller("option", "height", 200);
 * @memberof ejScroller
* @instance
*/
            height: 250,
            /**		
* Specifies the width of Scroll panel and scrollbars.
* @default 0
* @type {number}
* @example 
* //To set width API value during initialization
* 	$("#scrollcontent").ejScroller({width: 500 });	
* @example 
* //Get or set the width API, after initialization:<br>
*	//Gets the width value
*	$("#scrollcontent").ejScroller("option", "width"); 			
*	//Sets the width value
*	$("#scrollcontent").ejScroller("option", "width", 500);
 * @memberof ejScroller
* @instance
*/
            width: 0,
            /**		
* While press on the arrow key the scrollbar position added to the given pixel value.
* @default 57
* @type {number}
* @example 
* //To set oneStep API value during initialization
* 	$("#scrollcontent").ejScroller({oneStep: 40 });	
* @example 
* //Get or set the oneStep API, after initialization:<br>
*	//Gets the oneStep value
*	$("#scrollcontent").ejScroller("option", "oneStep"); 			
*	//Sets the oneStep value
*	$("#scrollcontent").ejScroller("option", "oneStep", 40);
* @memberof ejScroller
* @instance
*/
            oneStep: 57,
            /**		
* Specifies the button height for vertical scrollbar; for horizontal scrollbar specifies the width of the button in the scrollbar.
* @default 18
* @type {number}
* @example 
* //To set Button Size of Scroller during initialization
* 	$("#scrollcontent").ejScroller({buttonSize: 20 });	
* @example 
* //Get or set the buttonSize API, after initialization:<br>
*	//Gets the buttonSize value
*	$("#scrollcontent").ejScroller("option", "buttonSize"); 			
*	//Sets the buttonSize value
*	$("#scrollcontent").ejScroller("option", "buttonSize", 20);
* @memberof ejScroller
* @instance
*/
            buttonSize: 18,
            /**		
* The Scroller content and scrollbars move left with given value. 
* @default 0
* @type {number}
* @example 
* //To set Scroll left API during initialization
* 	$("#scrollcontent").ejScroller({scrollLeft: 40 });	
* @example 
* //Get or set the scrollLeft API, after initialization:<br>
*	//Gets the scrollLeft value
*	$("#scrollcontent").ejScroller("option", "scrollLeft"); 			
*	//Sets the scrollLeft value
*	$("#scrollcontent").ejScroller("option", "scrollLeft", 40);
* @memberof ejScroller
* @instance
*/
            scrollLeft: 0,
            /**		
* The Scroller content and scrollbars move to top position with specified value. 
* @default 0
* @type {number}
* @example 
* //To set Scroll top API during initialization
* 	$("#scrollcontent").ejScroller({scrollTop: 40 });	
* @example 
* //Get or set the scrollTop API, after initialization:<br>
*	//Gets the scrollTop value
*	$("#scrollcontent").ejScroller("option", "scrollTop"); 			
*	//Sets the scrollTop value
*	$("#scrollcontent").ejScroller("option", "scrollTop", 40);
* @memberof ejScroller
* @instance
*/
            scrollTop: 0,
            /**		
* Indicates the target area to which scroller have to appear. 
* @default null
* @type {string}
* @example 
* //To set Scroller for the specified target panel during initialization
* 	$("#scrollcontent").ejScroller({targetPane: "contentarea" });	
* @example 
* //Get or set the targetPane API, after initialization:<br>
*	//Gets the targetPane value
*	$("#scrollcontent").ejScroller("option", "targetPane"); 			
*	//Sets the targetPane value
*	$("#scrollcontent").ejScroller("option", "targetPane", "contentarea");
* @memberof ejScroller
* @instance
*/
            targetPane: null,
            /**		
* Set the Scrollbar height and width for this API; if vertical scrollbar,apply the width else apply the height.
* @default 18
* @type {number}
* @example 
* //Enable scrollerSize on initialization 
* 	//To set scroller Size API value 
* 	$("#scrollcontent").ejScroller({scrollerSize: 20 });	
* @example 
* //Get or set the scroller Size API, after initialization:<br>
*	$("#scrollcontent").ejScroller("option", "scrollerSize"); 			
*	//Sets the scrollerSize value
*	$("#scrollcontent").ejScroller("option", "scrollerSize", 20);
* @memberof ejScroller
* @instance
*/
            scrollerSize: 18,
            /**		
* Save current model value to browser cookies for state maintanence. While refresh the page Rating control values are retained.  
* @default false
* @type {boolean}
* @example 
* //Enable persist API on initialization
* 	$("#scrollcontent").ejScroller({persist: true });	
* @example 
* //Get or set the persist API, after initialization:<br>
*	//Gets the persist value
*	$("#scrollcontent").ejScroller("option", "persist"); 			
*	//Sets the persist value
*	$("#scrollcontent").ejScroller("option", "persist", true);
* @memberof ejScroller
* @instance
*/
            persist: false,
            /**		
* Indicates the Right to Left direction to scroller  
* @default undefined
* @type {boolean}
* @example 
* //Enable RTL API on initialization
* 	$("#scrollcontent").ejScroller({rtl: true });	
* @example 
* //Get or set the RTL API, after initialization:<br>
*	//Gets the rtl value
*	$("#scrollcontent").ejScroller("option", "rtl"); 			
*	//Sets the rtl value
*	$("#scrollcontent").ejScroller("option", "rtl", true);
* @memberof ejScroller
* @instance
*/
            rtl: undefined,
            /**		
* Enables or Disbale the touch Scroll  
* @default true
* @type {boolean}
* @example 
* //Disable Touch Scroll API on initialization
* 	$("#scrollcontent").ejScroller({touchScroll: false });	
* @example 
* //Get or set the Touch Scroll API, after initialization:<br>
*	//Gets the touchScroll value
*	$("#scrollcontent").ejScroller("option", "touchScroll"); 			
*	//Sets the touchScroll value
*	$("#scrollcontent").ejScroller("option", "touchScroll", false);
* @memberof ejScroller
* @instance
*/
            touchScroll: true,
            /**     
            * Fires when Scroller control is created.
            * @event
            * @name ejScroller#create	
            * @param {Object} argument Event parameters from scroller     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the scroller model
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //create event for scroller
            * $("#scrollcontent").ejScroller({
            *    create: function (args) {}
            * });      
            * @memberof ejScroller
            * @instance
            */
            create: null,
            /**     
* Fires when Scroller control is destroyed.
* @event
* @name ejScroller#destroy	
* @param {Object} argument Event parameters from scroller     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the scroller model
* @param {string}  argument.type returns the name of the event.
* @example 
* //destroy event for scroller
* $("#scrollcontent").ejScroller({
*    destroy: function (args) {}
* });      
* @memberof ejScroller
* @instance
*/
            destroy: null
        },
        validTags: ["div"],
        type: "transclude",
        /**
* Specify the data types for default properties 
* @private
*/
        dataTypes: {
            buttonSize: "number",
            oneStep: "number"
        },
        observables: ["scrollTop", "scrollLeft"],
        scrollTop: ej.util.valueFunction("scrollTop"),
        scrollLeft: ej.util.valueFunction("scrollLeft"),
        /**
* Specifies the keys used for keyboard navigation. 
* @private
*/
        keyConfigs: {
            up: "38",
            down: "40",
            left: "37",
            right: "39",
            pageUp: "33",
            pageDown: "34",
            pageLeft: "ctrl+37",
            pageRight: "ctrl+39"
        },
        /**
* Specify the content area for which the scrollerbar is specified.
* @private
*/
        content: function () {
            if (!this._content || !this._content.length || !this._content[0].offsetParent)
                this._content = this.element.children("div").first().addClass("e-content");

            return this._content;
        },
        _setFirst: true,
        /**
* Create the scroller widget
* @private
*/
        _init: function () {
            this.element.addClass("e-widget");
            this.content();
            this._ensureScrollers();
            if (this.model.rtl === undefined) {
                this.model.rtl = this.element.css("direction") === "rtl";
            }
            if (this.model.rtl) {
                this.element.addClass("e-rtl");
            }
            if (this.scrollLeft())
                this.content().scrollLeft(this.scrollLeft());
            if (this.scrollTop())
                this.content().scrollTop(this.scrollTop());

            this._on(this.content(), "scroll", this._scroll);
            this.model.targetPane != null && this._on(this.content().find(this.model.targetPane), "scroll", this._scroll);
        },
        /**
* Ensures the scrollbar height and width.
* @private
*/
        _ensureScrollers: function () {
            var jqVersion = $.fn.jquery, height, width;
            if (this.model.height) {
                this.element.height(this.model.height);
            }
            if (this.model.width) {
                this.element.width(this.model.width);
            }

            this._off(this.content(), "mousedown touchstart");

            if (this.isVScroll()) {
                if (!this._v) {
                    var d1 = this.createScroller("Height", "Y", "Top", "e-v", this.isHScroll());

                    if (this.model.touchScroll)
                        this._on(this.content(), "mousedown touchstart", { d: d1 }, this._mouseDownOnContent);
                }
            } else {
                this._v = null;
                this.element.find(".e-vscroll").remove();
            }
            if (this.isHScroll()) {
                if (!this._h) {
                    var d2 = this.createScroller("Width", "X", "Left", "e-h", this.isVScroll());

                    if (this.model.touchScroll)
                        this._on(this.content(), "mousedown touchstart", { d: d2 }, this._mouseDownOnContent);
                }
            } else {
                this.element.find(".e-hscroll").remove();
                this._h = null;
            }

            if (!this._v && !this._h)
                this.content().css({ width: "auto", height: "auto" });

            if (this.element.find(".e-hscroll").length > 0)
                !this.element.hasClass("e-hscrollbar") && this.element.addClass("e-hscrollbar");
            else {
                if (this.element.hasClass("e-hscrollbar")) {
                    this.element.removeClass("e-hscrollbar");
                    if (this._v) {
                        this.content().outerHeight(this.content().outerHeight() - 1);
                    }
                }
            }

            if (this.isHScroll() || this.isVScroll()) {
                this.content().addClass("e-content");
                jqVersion === "1.7.1" || jqVersion === "1.7.2" ? (height = "height", width = "width") : (height = "outerHeight", width = "outerWidth");
                this.content()[height](this.element.height() - (this.isHScroll() ? this.model.scrollerSize : 0));
                this.content()[width](this.element.width() - (this.isVScroll() ? this.model.scrollerSize : 0));
            } else
                this.content().removeClass("e-content");
        },
        /**
* Returns horizontal scrollbar is shown or not.		
* @return Boolean value
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // To check horizontal scrollbar is rendered or not
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.isHScroll(); // Returns horizontal scrollbar is shown or not.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // To check horizontal scrollbar is rendered or not
* $("#scrollcontent").ejScroller("isHScroll");	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        isHScroll: function () {
            if (this.model.width > 0) {
                var $paneObject = this.content().find(this.model.targetPane);
                if (this.model.targetPane != null && $paneObject.length)
                    return ($paneObject[0].scrollWidth + $paneObject.siblings().width()) > this.model.width;
                else
                    return this.content()[0].scrollWidth > this.model.width;
            }
            return false;
        },
        /**
* Returns vertical scrollbar is shown or not.		
* @return Boolean value
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // To check vertical scrollbar is rendered or not
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.isVScroll(); // Returns vertical scrollbar is shown or not.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // To check vertical scrollbar is rendered or not
* $("#scrollcontent").ejScrollbar("isVScroll");	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        isVScroll: function () {
            return this.model.height > 0 && this.content()[0].scrollHeight > this.model.height;
        },
        /**
* To configure the properties at runtime using SetModel		
* @private
*/
        _setModel: function (option) {
            for (var prop in option) {
                if (prop === "rtl") {
                    if (option[prop])
                        this.element.addClass("e-rtl");
                    else
                        this.element.removeClass("e-rtl");
                } else if (prop === "scrollLeft") {
                    this.content().scrollLeft(this.scrollLeft());
                } else if (prop === "scrollTop") {
                    this.content().scrollTop(this.scrollTop());
                } else if (prop === "touchScroll") {
                    if (!this.model.touchScroll)
                        this._off(this.content(), "mousedown touchstart");
                    else {
                        if (this._v)
                            this._on(this.content(), "mousedown touchstart", { d: this._v }, this._mouseDownOnContent);
                        if (this._h)
                            this._on(this.content(), "mousedown touchstart", { d: this._h }, this._mouseDownOnContent);
                    }
                } else {
                    this.scrollTop(0);
                    this.scrollLeft(0);
                    this.refresh();
                    break;
                }
            }
        },
        /**
* Create Scroller on demand time. 		
* @return jQuery
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // To create Scroller on demand time.
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.createScroller("createScroller", "Height", "Y", "Top", "v", true); // Creates Scroller on demand time.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // call createScroller method
* $("#scrollcontent").ejScroller("createScroller", "Height", "Y", "Top", "v", true);	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        createScroller: function (dimension, xy, position, css, isOtherScroll) {
            var temp = css == "e-v" ? "v" : "h", height;
            var d = this["_" + temp] = {};
            var jqVersion = $.fn.jquery;
            d.dimension = dimension;
            d.xy = xy;
            d.position = position;
            d.css = css;
            d.isOtherScroll = isOtherScroll;
            d.uDimension = dimension;

            var dim = dimension === "Width" ? "Height" : "Width";
            jqVersion === "1.7.1" || jqVersion === "1.7.2" ? height = dim.toLowerCase() : height = "outer" + dim;
            this.content()[height](this.element[dim.toLowerCase()]() - (this.isVScroll() ? this.model.scrollerSize : 0));
            this._calculateLayout(d);
            this._createLayout(d);
            var buttons = this[d.main].find(".e-button");

            this._off(buttons, "mousedown")
                ._on(buttons, "mousedown", { d: d, step: 1 }, this._spaceMouseDown);
            this._off(css === "e-v" ? this.element : this[d.scroll], "mousewheel DOMMouseScroll")
                ._on(css === "e-v" ? this.element : this[d.scroll], "mousewheel DOMMouseScroll", { d: d }, this._mouseWheel);
            this._off(this[d.scroll], "mousedown")
                ._on(this[d.scroll], "mousedown", { d: d }, this._spaceMouseDown);
            this._off(this[d.handler], "mousedown touchstart")
                ._on(this[d.handler], "mousedown touchstart", { d: d }, this._mouseDown);

            return d;
        },
        /**
* Creates the layout for the scrollbars to be rendered.	
* @private
*/
        _createLayout: function (d) {
            var divString = "<div class='" + d.css + "{0}' style='" + d.dimension + ":{1}px'>{2}</div>";
            var jqVersion = $.fn.jquery;
            var lit = {}, height;
            lit[d.dimension] = d.modelDim;

            var el = ej.buildTag(
                "div." + d.css + "scroll",
                    String.format(divString, "up e-icon e-button", d.buttonSize) +
                    String.format(divString, "handlespace", d.handleSpace,
                        String.format(divString, "handle", d.handle)) +
                    String.format(divString, "down e-icon e-button", d.buttonSize),
                lit
            );

            if (d.css == "e-v" && $(this.element.find(".e-content")).length > 0) $(this.element.find(".e-content")).after(el);
            else this.element.append(el);
            jqVersion === "1.7.1" || jqVersion === "1.7.2" ? height = d.uDimension.toLowerCase() : height = "outer" + d.uDimension;
            this[d.handler] = this.element.find("." + d.handler)[height](d.handle);
            this[d.scroll] = this[d.handler].parent();
            this[d.main] = this[d.scroll].parent();
            this[d.main].find(".e-button")["outer" + d.uDimension](d.buttonSize);
        },
        /**
* Calculates the scrollbar layout based on the values specified for scrollbr to appear
* @private
*/
        _calculateLayout: function (d) {
            d.rtl = this.model.rtl === undefined ? this.element.css("direction") === "rtl" : this.model.rtl;
            d.rtl = d.css === "e-h" ? d.rtl : false;

            d.scrollDim = "scroll" + d.dimension;
            d.lPosition = d.position.toLowerCase();
            d.clientXy = "page" + d.xy;
            d.scrollVal = "scroll" + d.position;
            d.oneStep = this.model.oneStep;
            d.modelDim = this.model[(d.dimension = d.dimension.toLowerCase())] - (d.isOtherScroll ? this.model.scrollerSize : 0);
            d.handler = d.css + "handle";
            d.buttonSize = this.model.buttonSize;
            d.main = d.css + "scroll";
            d.scroll = d.css + "ScrollSpace";
            d.handleSpace = d.modelDim - 2 * d.buttonSize;

            var $pane = this.content().find(this.model.targetPane);
            if (this.model.targetPane != null && $pane.length && d.dimension != "height")
                d.sScrollDim = $pane[0][d.scrollDim] + $pane.parent().width() - $pane.width();
            else
                d.sScrollDim = this.content()[0][d.scrollDim];
            d.scrollable = d.sScrollDim - d.modelDim;
            d.handle = Math.floor(d.modelDim / d.sScrollDim * d.handleSpace);
            if (d.handle < 20) d.handle = 20;
            d.onePx = d.scrollable / (d.handleSpace - d.handle);
            d.fromScroller = false;
            d.up = true;
            d.vInterval = undefined;
            d.scrollable *= d.rtl ? -1 : 1;
        },
        /**
* Updates scrollbar layout based on ondemand values
* @private
*/
        _updateLayout: function (d) {
            this.element.find("." + d.css + "scroll").css(d.dimension, d.modelDim + "px")
                .find(".e-button").css(d.dimension, this.model.buttonSize).end()
                .find("." + d.css + "handlespace").css(d.dimension, d.handleSpace + "px")
                .find("." + d.css + "handle").css(d.dimension, d.handle + "px");
        },
        /**
* User refreshes the Scroller control at any time.		
* @return jQuery
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // To refresh the Scroller control at any time.
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.refresh(); // refreshes the Scroller control at any time
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Refresh the scroller control
* $("#scrollcontent").ejScroller("refresh");	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        refresh: function () {
            this._ensureScrollers();

            if (this.scrollLeft())
                this.content().scrollLeft(this.scrollLeft());
            if ((this.scrollTop() && this._v == null) || (this._v != null && !this._v.skipChange))
                this.content().scrollTop(this.scrollTop());

            if (this._v) {
                this._v.dimension = "Height";
                this._v.isOtherScroll = this.isVScroll() && this.isHScroll() ? true : false;
                this.isVScroll() && !this._calculateLayout(this._v) && this._updateLayout(this._v);
            }
            if (this._h) {
                this._h.dimension = "Width";
                this._h.isOtherScroll = this.isVScroll() && this.isHScroll() ? true : false;
                this.isHScroll() && !this._calculateLayout(this._h) && this._updateLayout(this._h);
            }
            this.model.targetPane != null && this._on(this.content().find(this.model.targetPane), "scroll", this._scroll);
        },
        /**
* Section for handling scrollbar based on keypressed
* @private
*/
        _keyPressed: function (action, target) {
            if (["input", "select", "textarea"].indexOf(target.tagName.toLowerCase()) !== -1)
                return true;

            var d, iChar;

            if (["up", "down", "pageUp", "pageDown"].indexOf(action) !== -1) {
                d = this._v;
                iChar = "o";
            } else if (["left", "right", "pageLeft", "pageRight"].indexOf(action) !== -1) {
                d = this._h;
                iChar = "i";
            } else return true;
            if (!d) return true;

            return !this._changeTop(d, (action.indexOf(iChar) < 0 ? -1 : 1) * (action[0] !== "p" ? 1 : 3) * d.oneStep, "key");
        },
        /**
* Scroller moves to given pixel in Y (top) position.		
* @return jQuery
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Moves scroller to given pixel in Y (top) position.
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.scrollY("scrollY", 25, true); // call scrollY method.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //Moves scroller to given pixel in Y (top) position.
* $("#scrollcontent").ejScroller("scrollY", 25, true);	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        scrollY: function (pixel, noAnimation, source) {
            if (noAnimation) {
                if (this._trigger("scroll", { source: source || "custom", scrollData: this._v, scrollTop: pixel }))
                    return;
                this.scrollTop(pixel);
                this.content().scrollTop(pixel);
                return;
            }
            this.content().stop().animate({
                scrollTop: pixel
            }, 100, 'linear');
        },
        /**
* Scroller moves to given pixel in X (left) position.		
* @return jQuery
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Moves scroller to given pixel in X (left) position.
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.scrollX("scrollX", 25, true); // call scrollX method.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //Moves scroller to given pixel in X (left) position.
* $("#scrollcontent").ejScroller("scrollX", 25, true);	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        scrollX: function (pixel, noAnimation, source) {
            if (noAnimation) {
                if (this._trigger("scroll", { source: source || "custom", scrollData: this._h, scrollLeft: pixel }))
                    return;
                this.scrollLeft(pixel);
                if (this.model.targetPane != null)
                    this.content().find(this.model.targetPane).scrollLeft(pixel);
                else
                    this.content().scrollLeft(pixel);
                return;
            }
            this.content().stop().animate({
                scrollLeft: pixel
            }, 100, 'linear');
        },
        /**
* Updates top value based on scroller movement
* @private
*/
        _changeTop: function (d, step, source) {
            var start = this.model.targetPane != null && d.dimension != "height" ? this.content().find(this.model.targetPane)[d.scrollVal]() : this.content()[d.scrollVal](), t;

            if (d.dimension == "height" && start == 0)
                start = this.scrollTop() != 0 ? this.scrollTop() : 0;
            t = start + step;
            if (!d.rtl ? t > d.scrollable : t < d.scrollable) t = d.scrollable;
            if (!d.rtl ? t < 0 : t > 0) t = 0;

            if (t !== start) {
                this["scroll" + d.xy](t, true, source);
            }

            return t !== start;
        },
        /**
* To handle scroll based on movement via mouse wheel
* @private
*/
        _mouseWheel: function (e) {
            if (!e.data) return;

            var delta = 0, data = e.data.d, ori = e;
            e = e.originalEvent;
            if (e.wheelDelta) {
                delta = -e.wheelDelta / 120;
                if (window.opera) {
                    if (parseFloat(window.opera.version, 10) < 10)
                        delta = -delta;
                }
            } else if (e.detail) delta = e.detail / 3;

            if (!delta) return;

            if (this._changeTop(data, delta * data.oneStep, "wheel")) {
                e.preventDefault ? e.preventDefault() : ori.preventDefault();
                if (data.css === "e-h") {
                    ori.stopImmediatePropagation();
                    ori.stopPropagation();
                }
            }
        },
        /**
* To handle mouse up movement
* @private
*/
        _mouseUp: function (e) {
            if (!e.data) return;

            var d = e.data.d;
            e.type === "mouseup" && this[d.handler].removeClass("e-active");
            if (e.type === "mouseup" || e.type === "touchend" || (!e.toElement && !e.relatedTarget)) {
                this._off($(document), "mousemove touchmove");
                $(document).off("mouseout mouseup touchend", ej.proxy(this._mouseUp, this));
                d.fromScroller = false;
                this[d.scroll].off("mousemove");
                this[d.handler].off("mousemove").css("transition", "");

                if (e.data.source === "thumb") {
                    $.when(this.content()).done(ej.proxy(function () {
                        this._trigger("thumbEnd", { originalEvent: e, scrollData: d });
                    }, this));
                }
            }
            d.up = true;
        },
        /**
* To handle mouse down movement on content
* @private
*/
        _mouseDownOnContent: function (down) {
            var d = down.data.d;
            if (this._trigger("thumbStart", { originalEvent: down, scrollData: d }))
                return;

            d.fromScroller = true;
            this[d.handler].css("transition", "none");

            var prevY = null, skip = 1, min = 5, direction;

            this._on($(document), "mousemove touchmove", (function (move) {
                move.preventDefault();
                var pageXY = move.type == "mousemove" ? move[d.clientXy] : move.originalEvent.changedTouches[0][d.clientXy];

                if (prevY && pageXY !== prevY) {
                    var diff = pageXY - prevY, sTop = this.model[d.scrollVal] - (diff * d.onePx / min);

                    if (skip == 1 && Math.abs(diff) > min) {
                        direction = d.position;
                        skip = 0;
                    }
                    if (skip == 0) prevY = pageXY;

                    if (sTop >= 0 && sTop <= d.scrollable && direction === d.position) {
                        this["scroll" + d.xy](sTop, true, "thumb");
                        this[d.handler].css(d.position.toLowerCase(), sTop / d.onePx + "px");
                        this.content().css("cursor", "pointer");
                        this._trigger("thumbMove", { originalEvent: move, scrollData: d });
                    }
                }
                if (prevY == null) prevY = pageXY;
            }));

            $(document).one("mouseup touchend", { d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
        },
        /**
* To handle mouse down movement
* @private
*/
        _mouseDown: function (down) {
            //if (down.which !== 1) return;
            var d = down.data.d;

            d.fromScroller = true;
            this[d.handler].css("transition", "none");

            var prevY, top = parseInt(this[d.handler].css(d.lPosition)) || 0;

            this._on($(document), "mousemove touchmove", (function (move) {
                move.preventDefault();
                var skip = 1;
                var pageXY = move.type == "mousemove" ? move[d.clientXy] : move.originalEvent.changedTouches[0][d.clientXy];
                if (prevY && pageXY !== prevY) {
                    top += (pageXY - prevY);

                    if (d.rtl ? top > 0 : top < 0) skip = top = 0;

                    if ((top * (d.rtl ? -1 : 1)) + d.handle >= d.handleSpace)
                        skip = top = (d.handleSpace - d.handle) * (d.rtl ? -1 : 1);

                    this["scroll" + d.xy](Math.ceil(top * d.onePx), true, "thumb");
                    this[d.handler].css(d.position.toLowerCase(), top + "px");
                    this.content().css("cursor", "pointer");
                    this._trigger("thumbMove", { originalEvent: move, scrollData: d });
                }

                if (skip === 1)
                    prevY = pageXY;
            }));

            this._trigger("thumbStart", { originalEvent: down, scrollData: d });

            $(document).one("mouseup touchend", { d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
        },
        /**
* To handle mouse down based on the interval 
* @private
*/
        _spaceMouseDown: function (e) {
            if (!e.data) return;

            var d = e.data.d;

            if (e.which !== 1 || e.target === this[d.handler][0]) return;

            var step = (e.data.step || 3) * d.oneStep, hTop = e.data.top || this[d.handler].offset()[d.lPosition];

            e[d.clientXy] = e[d.clientXy] || 0;

            if (e[d.clientXy] < hTop) step *= -1;

            this._changeTop(d, step, step === 3 ? "track" : "button");

            if (e.data.step !== 1) {
                this[d.scroll].mousemove(function () {
                    d.up = true;
                });
            }

            d.up = false;
            d.vInterval = setInterval(ej.proxy(function () {

                if (step < 0 ? hTop + (step / d.onePx) < e[d.clientXy] : hTop + d.handle + (step / d.onePx) > e[d.clientXy])
                    d.up = true;

                if (d.up) {
                    clearInterval(d.vInterval);
                    return;
                }

                this._changeTop(d, step, step === 3 ? "track" : "button");

                hTop = e.data.top || this[d.handler].offset()[d.lPosition];
            }, this), 150);

            $(document).one("mouseup", { d: d }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d }, ej.proxy(this._mouseUp, this));
        },
        /**
* To handle scroll movement
* @private
*/
        _scroll: function (e) {
            var dS = [this._v, this._h];

            for (var i = 0; i < 2; i++) {
                var d = dS[i];
                if (!d || d.skipChange) continue;

                if (this.model.targetPane != null && i == 1)
                    d.sTop = this.content().find(this.model.targetPane)[0][d.scrollVal];
                else
                    d.sTop = e.target[d.scrollVal];
                this[d.scrollVal](d.sTop);
                if (d.fromScroller) return;
                this[d.handler].css(d.lPosition, (d.sTop / d.onePx) + "px");
            }
        },
        /**
* Changes vertical handler position.		
* @return jQuery
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Changes vertical handler position.
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.changevHandlerPosition(20); // call changevHandlerPosition method.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //Changes vertical handler position.
* $("#scrollcontent").ejScroller("changevHandlerPosition",20);	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        changevHandlerPosition: function (top) {
            top = this._v != null && top >= this._v.scrollable ? this._v.scrollable : top;
            if (this._v != null && top >= 0 && top <= this._v.scrollable)
                this[this._v.handler].css(this._v.lPosition, (top / this._v.onePx) + "px");
        },
        /**
* Changes horizontal handler position.		
* @return jQuery
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Changes horizontal handler position.
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.changehHandlerPosition(20); // call changehHandlerPosition method.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //Changes horizontal handler position.
* $("#scrollcontent").ejScroller("changehHandlerPosition",20);	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        changehHandlerPosition: function (left) {
            left = this._h != null && left >= this._h.scrollable ? this._h.scrollable : left;
            if (this._h != null && left >= 0 && left <= this._h.scrollable)
                this[this._h.handler].css(this._h.lPosition, (left / this._h.onePx) + "px");
        },
        /**
* destroy the Scroller control
* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
* @alias destroy
* @return jQuery
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Destroy Scroller
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.destroy(); // destroy the scroller
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // destroy the scroller
* $("#scrollcontent").ejScroller("destroy");	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        _destroy: function () {
            this.element.css({ "width": "", "height": "" }).find(".e-vscroll,.e-hscroll").remove();
            this.content().removeClass("e-content").css({ "width": "", "height": "" });
        }
    });
})(jQuery, Syncfusion, window);;
/**
* @fileOverview Plugin to style the Waiting pop elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/(function ($, ej, undefined) {
    /**
       * @namespace ej
       * @class ejWaitingPopup
       * @requires jQuery
       * @requires ej.common.all
       * @classdesc Custom Design for Html Button control.
       * @example 
       *  &lt;div id="target"&gt;&lt;/div&gt;
       * &lt;script&gt;
       * // Simple waiting popup creation.
       * $("#target").ejWaitingPopup({ autoDisplay: true });
       * &lt;/script&gt;
       */
    // ejWaitingPopup is the plugin name 
    // "ej.WaitingPopup" is "namespace.className" will hold functions and properties

    ej.widget("ejWaitingPopup", "ej.WaitingPopup", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        _rootCSS: "e-waitingpopup",

        // default model
        defaults: {
            /**		
			* Enables the visibility of the WaitingPopup control
			* @default false
			* @type {Boolean}
			* @example 
			* //To set autoDisplay API value during initialization  
			* 	$("#target").ejWaitingPopup({ autoDisplay: true });
			* @example 
			* //Get or set the autoDisplay API, after initialization:<br>
			*	//Gets the autoDisplay API value  
			*	$("#target").ejWaitingPopup('option','autoDisplay'); 		
			*	//Sets the autoDisplay API 
			*	$("#target").ejWaitingPopup('option','autoDisplay', true);
			 * @memberof ejWaitingPopup
			* @instance
			*/
            autoDisplay: false,
            /**		
			* Enables or disables the default loading icon.
			* @default true
			* @type {Boolean}
			* @example 
			* //To set showImage API value during initialization  
			* 	$("#target").ejWaitingPopup({ showImage: false });
			* @example 
			* //Get or set the autoDisplay API, after initialization:<br>
			*	//Gets the showImage API value  
			*	$("#target").ejWaitingPopup('option', 'showImage');	
			*	//Sets the showImage API 
			*	$("#target").ejWaitingPopup('option', 'showImage', false);
			 * @memberof ejWaitingPopup
			* @instance
			*/
            showImage: true,
            /**		
			* Sets the root class for the WaitingPopup control theme
			* @default null
			* @type {String}
			* @example 
			* //To Initialize the WaitingPopup with the cssClass value specified. 
			* 	$("#target").ejWaitingPopup({ cssClass: 'gradient-lime ' });
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the cssClass API value  
			*	$("#target").ejWaitingPopup('option', 'cssClass');
			*	//Sets the cssClass API 
			*	$("#target").ejWaitingPopup('option', 'cssClass', 'gradient-lime ');
			 * @memberof ejWaitingPopup
			* @instance
			*/
            cssClass: "",
            /**		
			* Sets the custom text in the pop-up panel to notify the waiting process
			* @default null
			* @type {String}
			* @example 
			* //To Initialize the WaitingPopup with the text value specified
			* 	$("#target").ejWaitingPopup({ text: 'waiting…' });
			* @example 
			* //Get or set the text API, after initialization:<br>
			*	//Gets the text API value  
			*	$("#target").ejWaitingPopup('option', 'text');
			*	//Sets the text API 
			*	$("#target").ejWaitingPopup('option', 'text', 'waiting…');
			 * @memberof ejWaitingPopup
			* @instance
			*/
            text: null,
            /**		
			* Loads HTML content inside the popup panel instead of the default icon
			* @default null
			* @type {object}
			* @example 
			*&lt;div id="content"&gt;
			*   &lt;div class="loader-icon"&gt;&lt;/div&gt;
			*    &lt;div class="text"&gt;
			*     &lt;p&gt; Content is loading ... &lt;/p&gt;
			*      &lt;/div&gt;
			*&lt;/div&gt;
			* //To Initialize the WaitingPopup control with the template value specified.
			* 	$("#target").ejWaitingPopup({ template: $('#content') });
			* @example 
			* //Get or set the text API, after initialization:<br>
			*	//Gets the template API value  
			*	$("#target").ejWaitingPopup('option', 'template');
			*	//Sets the template API 
			*	$("#target").ejWaitingPopup('option', 'template', $('#content'));
			 * @memberof ejWaitingPopup
			* @instance
			*/
            template: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            autoDisplay: "boolean",
            showImage: "boolean",
            cssClass: "string"
        },
        /**
       * To show the waiting popup
       * @return jQuery
       * @example 
       *  &lt;div id="target"&gt;&lt;/div&gt;
       * &lt;script&gt;
       * // Initialize the WaitingPopup object.
       *   var popupObj = $("#target").data("ejWaitingPopup");
       * // Calls the show method of WaitingPopup to display.
       * popupObj.show();
       * &lt;/script&gt;
       * @example 
       *  &lt;div id="target"&gt;&lt;/div&gt;
       * &lt;script&gt;
       * // Display WaitingPopup using the show method.
       * $("#target").ejWaitingPopup("show");
       * &lt;/script&gt;
       *@memberof ejWaitingPopup
       * @instance
       */
        show: function () {
            this._refreshPanel();
            this.maindiv.css("display", "block");
            this.model.autoDisplay = true;
        },
        /**
       * To hide the waiting popup
       * @return jQuery
       * @example 
       *  &lt;div id="target"&gt;&lt;/div&gt;
       * &lt;script&gt;
       * // hide WaitingPopup using the hide method.
       * $("#target").ejWaitingPopup('hide');
       * &lt;/script&gt;
       *@memberof ejWaitingPopup
       * @instance
       */
        hide: function () {
            this.maindiv.css("display", "none");
            this.model.autoDisplay = false;
        },
        /**
        * Refreshes the WaitingPopup control by resetting the pop-up panel position and content position
		* @return jQuery
		* @example 
		*  &lt;div id="target"&gt;&lt;/div&gt;
		* &lt;script&gt;
		* $("#target").ejWaitingPopup('refresh');
		* &lt;/script&gt;
		*@memberof ejWaitingPopup
		* @instance
        */
        refresh: function () {
            this._refreshPanel();
        },
        /**
        * To configure text for the waiting popup.		
        * @private
        */
        _setText: function (text) {
            if (text) {
                if (this.popupText) this.popupText.html(text);
                else {
                    this._generateTextTag(text);
                    this._setContentPosition();
                }
            }
            else if (this.popupText) {
                this.popupText.remove();
                this.popupText = null;
            }
        },
        /**
        * To configure showImage for the waiting popup.		
        * @private
        */
        _showImage: function (boolean) {
            if (boolean) {
                this.popupImage = ej.buildTag("span.e-image");
                if (this.popupText) this.popupImage.insertBefore(this.popupText);
                else this.maindiv.append(this.popupImage);
            }
            else if (this.popupImage) {
                this.popupImage.remove();
                this.popupImage = null;
            }
        },
        /**
         * To configure setTemplate for the waiting popup.		
		 * @private
         */
        _setTemplate: function () {
            var template = this.model.template;
            if (typeof template === "string") template = $(template);
            if (typeof template === "object" && typeof template.css === "function")
                this.templateObj = template;
            else
                this.templateObj = ej.buildTag("div", "", { "text-align": "center" }).append(template);

            this.templateObj.css({ "visibility": "visible", "display": "block" });
            this.maindiv.append(this.templateObj);
        },
        /**
         * To configure setTheme for the waiting popup.		
		 * @private
         */
        _setTheme: function (skin) {
            this.maindiv.removeClass(this.model.cssClass).addClass(skin);
        },

        // constructor function
        /**
         * Create the WaitingPopup widget
		 * @private
         */
        _init: function () {
            this._initialize();
            this._render();
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "text": this._setText(options[option]); break;
                    case "cssClass": this._setTheme(options[option]); break;
                    case "autoDisplay": this._setVisibility(options[option]); break;
                    case "showImage": this._showImage(options[option]); this._setContentPosition(); break;
                    case "template":
                        this.maindiv.empty();
                        if (options[option]) {
                            this.model.template = options[option];
                            this._setTemplate();
                        }
                        else {
                            this.model.template = options[option] = null;
                            this._showImage(this.model.showImage);
                            if (this.model.text) this._generateTextTag(this.model.text);
                        }
                        this._setContentPosition();
                        break;
                }
            }
        },

        _destroy: function () {
            this.maindiv.remove();
        },
        /**
        * To initialize the waiting popup		
        * @private
        */
        _initialize: function () {
            this.maindiv = null;
            this.popupText = null;
            this.popupImage = null;
            this.templateObj = null;
        },

        /**
        * Render Section For waiting popup		
        * @private
        */
        _render: function () {
            this.maindiv = ej.buildTag("div.e-waitpopup-pane e-widget " + this.model.cssClass + "#" + this.element[0].id + "_WaitingPopup");
            if (this.model.template) {
                this._setTemplate();
            }
            else {
                this._showImage(this.model.showImage);
                if (this.model.text) {
                    this._generateTextTag(this.model.text);
                }
            }
            $('body').append(this.maindiv);
            this._setVisibility(this.model.autoDisplay);
        },
        /**
         * To configure refreshPanel for the waiting popup	
		 * @private
         */
        _refreshPanel: function () {
            this.maindiv.width(this.element.outerWidth());
            this.maindiv.height(this.element.outerHeight());
            this._setPanelPosition();
            this._setContentPosition();
        },
        /**
         * To configure setPanelPosition for the waiting popup	
		 * @private
         */
        _setPanelPosition: function () {
            var location = this.element.offset();
            this.maindiv.css({
                "left": Math.ceil(location.left) + "px",
                "top": Math.ceil(location.top) + "px",
                "z-index": this._maxZindex() + 1
            });
        },
        /**
         * To configure setContentPosition for the waiting popup	
		 * @private
         */
        _setContentPosition: function () {
            if (this.model.template == null) {
                var textHeight = 0, imgHeight = 0, targetHeight, top = null;
                targetHeight = this.element.outerHeight();
                if (this.popupText) textHeight = this.popupText.outerHeight();
                if (this.popupImage) imgHeight = this.popupImage.outerHeight();

                if (this.popupImage) {
                    top = Math.ceil((targetHeight - (imgHeight + textHeight)) / 2);
                    this.popupImage.css("top", top + "px");
                }
                if (this.popupText) {
                    if (!top) top = Math.ceil((targetHeight - textHeight) / 2);
                    this.popupText.css("top", top + "px");
                }
            }
            else {
                this.templateObj.css({
                    "position": "relative",
                    "left": Math.ceil((this.element.outerWidth() - this.templateObj.outerWidth()) / 2),
                    "top": Math.ceil((this.element.outerHeight() - this.templateObj.outerHeight()) / 2)
                });
            }
        },
        /**
         * To configure generateTextTag for the waiting popup	
		 * @private
         */
        _generateTextTag: function (text) {
            this.popupText = ej.buildTag("div.e-text", text);
            this.maindiv.append(this.popupText);
        },
        /**
         * To configure setVisibility for the waiting popup	
		 * @private
         */
        _setVisibility: function (autoDisplay) {
            if (autoDisplay) this.show();
            else this.hide();
        },
        /**
         * To configure maxZindex for the waiting popup	
		 * @private
         */
        _maxZindex: function () {
            var elements = document.getElementsByTagName("div"), maxIndex = 0, i, zindex;
            for (i = 0; i < elements.length - 1; i++) {
                zindex = parseInt(elements[i].style.zIndex, 10);  // 10 - radix value
                if (zindex > maxIndex) {
                    maxIndex = zindex;
                }
            }
            return maxIndex;
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejButton
	* @requires jQuery
	* @requires ej.common.all
	* @classdesc Custom Design for Html Button control.
	* @example 
	* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
	* &lt;script&gt;
	* // Create Button
    * $('#button1').ejButton(); 	
	* &lt;/script&gt;
	*/

    ej.widget("ejButton", "ej.Button", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["button", "input"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-button",

        defaults: {
            /**		
			* Specifies the size of the Button.	See {@link buttonSize}
			* @default ej.buttonSize.normal
			* @type {enum}
			* @example 
			* //To set size API value during initialization  
			* 	$("#button1").ejButton({  size: ej.buttonSize.mini});					* 
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#button1").ejButton("option", "size");<br>			
			*	//Sets the size value 
			*	$("#button1").ejButton("option", "size",  ej.buttonSize.large ); 
			 * @memberof ejButton
			* @instance
			*/
            size: "normal",
            /**		
			* Specifies the height of the Button.
			* @default 28
			* @type {number}
			* @example 
			* //To set height API value during initialization  
			* 	$("#button1").ejButton({ height: 30 });					* 
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value
			*	$("#button1").ejButton("option", "height");			
			*	//Sets the height value
			*	$("#button1").ejButton("option", "height", 30 );
			 * @memberof ejButton
			* @instance
			*/
            height: "",
            /**		
			* Specifies the width of the Button.
			* @default 100
			* @type {number}
			* @example 
			* //To set width API value during initialization  
			* 	//To set width API value 
			* 	$("#button1").ejButton({ width: 30 });	
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value
			*	$("#button1").ejButton("option", "width");			
			*	//Sets the width value
			*	$("#button1").ejButton("option", "width", 30 );
			 * @memberof ejButton
			* @instance
			*/
            width: "",
            /**		
			* Specifies the button control state.
			* @default true
			* @type {boolean}
			* @example 
			* // Enable Button on initialization. 
			* 	//To set width API value 
			* 	$("#button1").ejButton({ enabled:  true });			* 
			* @example 
			* //Get or set the button state, after initialization:<br>
			*	// Gets the Enabled API value.		
			*	$("#button1").ejButton("option", "enabled");			
			*	// Sets the Enabled API
			*	$("#button1").ejButton("option", "enabled",  false );
			 * @memberof ejButton
			* @instance
			*/
            enabled: true,
            /**		
			* Specifies the text content for Button.
			* @default null
			* @type {string}
			* @example 
			* // Set the button text on initialization. 			
			* 	$("#button1").ejButton({  text: "Hello Word" });			* 
			* @example 
			* //Get or set the button text, after initialization:<br>
			*	// Gets the button text value.		
			*	$("#button1").ejButton("option", "text");			
			*	// Sets the text content to button
			*	$("#button1").ejButton("option", "text",  "Hello Word" );
			 * @memberof ejButton
			* @instance
			*/
            text: null,
            /**		
			* Specifies the contentType of the Button. See {@link contents}
			* @default ej.contents.textOnly
			* @type {enum}
			* @example 
			* // Set the button contentType on initialization. 			
			* 	$("#button1").ejButton({  contentType : ej.contents.imageOnly });			* 
			* @example 
			* //Get or set the button contentType, after initialization:<br>
			*	// Gets the button contentType.		
			*	$("#button1").ejButton("option", "contentType");			
			*	// Sets the contentType to button
			*	$("#button1").ejButton("option", "contentType",  ej.contents.imageOnly );
			 * @memberof ejButton
			* @instance
			*/
            contentType: "textonly",
            /**		
			*  Specifies the image position of the Button. This image position is applicable only with the textandimage contentType property. The images can be positioned in both imageLeft and imageRight options. See {@link imagePositions}
			* @default ej.imagePositions.imageLeft
			* @type {enum}
			* @example 
			* // Set the image position for button during initialization. 			
			* 	$("#button1").ejButton(
            * {
            *    contentType: ej.contents.textAndImage,
            *    imagePosition: ej.imagePositions.imageRight
            * });			* 
			* @example 
			* //Get or set image position for button after initialization:<br>
			*	// Gets the image position.		
			*	$("#button1").ejButton("option", "imagePosition");			
			*	// Sets the image position
			*	$("#button1").ejButton("option", "imagePosition",  ej.contents.imageRight );
			 * @memberof ejButton
			* @instance
			*/
            imagePosition: "imageleft",
            /**		
			* Specify the rounded corner to button
			* @default false
			* @type {boolean}
			* @example 
			* // Set the rounded corner during initialization. 			
			* 	$("#button1").ejButton({  roundedCorner : true });			* 
			* @example 
			* //Get or set the rounded corner after initialization:<br>
			*	// Gets the rounded corner value.		
			*	$("#button1").ejButton("option", "roundedCorner");			
			*	// Sets the rounded corner to button
			*	$("#button1").ejButton("option", "roundedCorner",  true );
			 * @memberof ejButton
			* @instance
			*/
            roundedCorner: false,
            /**		
			* Specify the CSS class to button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the CSS class during initialization. 			
			* 	$("#button1").ejButton({  cssClass : "gradient-lime" });			* 
			* @example 
			* //Get or set the rounded corner after initialization:<br>
			*	// Gets the rounded corner value.		
			*	$("#button1").ejButton("option", "cssClass");			
			*	// Sets the rounded corner to button
			*	$("#button1").ejButton("option", "cssClass",  "gradient-lime" );
			 * @memberof ejButton
			* @instance
			*/
            cssClass: "",
            /**		
			* Specifies the primary icon for Button. This is applicable for the content type’s imageOnly, textandimage, imagetextimage and imageboth.
			* @default null
			* @type {string}
			* @example 
			* // Set the primary icon during initialization. 			
			* 	$("#button1").ejButton(
            *  {
            *     contentType: "imageonly",
            *     prefixIcon: "e-uiLight e-icon-handup"
            *  });		
			* @example 
			* //Get or set the primary icon after initialization:<br>
			*	// Gets the primary icon.		
			*	$("#button1").ejButton("option", "prefixIcon");			
			*	// Sets the primary icon to button
			*	$("#button1").ejButton("option", "prefixIcon",  "e-uiLight e-icon-handup" );
			 * @memberof ejButton
			* @instance
			*/
            prefixIcon: null,
            /**		
			* Specifies the secondary icon for Button. This is applicable for the content type’s imagetextimage and imageboth.
			* @default null
			* @type {string}
			* @example 
			* // Set the secondary icon during initialization. 			
			* 	$("#button1").ejButton(
            *  {
            *     contentType: "imageonly",
			*		prefixIcon: "e-uiLight e-icon-handup",
            *	    suffixIcon: "e-uiLight e-icon-padlockclosed"
            *  });		
			* @example 
			* //Get or set the secondary icon after initialization:<br>
			*	// Gets the secondary icon.		
			*	$("#button1").ejButton("option", "suffixIcon");			
			*	// Sets the secondary icon to button
			*	$("#button1").ejButton("option", "suffixIcon",  "e-uiLight e-icon-padlockclosed" );
			 * @memberof ejButton
			* @instance
			*/
            suffixIcon: null,
            /**		
			* Specify the Right to Left direction to button
			* @default false
			* @type {boolean}
			* @example 
			* // Set the RTL during initialization. 			
			* 	$("#button1").ejButton({  rtl : true });			* 
			* @example 
			* //Get or set the RTL after initialization:<br>
			*	// Gets the rounded corner value.		
			*	$("#button1").ejButton("option", "rtl");			
			*	// Sets the RTL to button
			*	$("#button1").ejButton("option", "rtl",  true );
			 * @memberof ejButton
			* @instance
			*/
            rtl: false,
            /**		
			* Convert the button as repeat button. It raises the 'Click' event repeattedly from the it is pressed until it is released.
			* @default false
			* @type {boolean}
			* @example 
			* // Create repeat button during initialization. 			
			* 	$("#button1").ejButton({  repeatButton : true });			* 
			* @example 
			* //Enable or disable the repeat behaviour after initialization:<br>
			*	// Gets the repeat button status.		
			*	$("#button1").ejButton("option", "repeatButton");			
			*	// Sets the repeat button status to button
			*	$("#button1").ejButton("option", "repeatButton",  true );
			 * @memberof ejButton
			* @instance
			*/
            repeatButton: false,
            /**		
			* Specified the timeInterval between two 'click' events while button in repeat button mode.
			* @default "150"
			* @type {string}
			* @example 
			* // Set interval during initialization. 			
			* 	$("#button1").ejButton({  
			*			repeatButton : true,
			* 			timeInterval : "100" });
			* @example 
			* //Get or set the interval for repeat button after initialization:<br>
			*	// Gets the interval value.		
			*	$("#button1").ejButton("option", "timeInterval");			
			*	// Sets the interval for repeat button
			*	$("#button1").ejButton("option", "timeInterval",  "100" );
			 * @memberof ejButton
			* @instance
			*/
            timeInterval: "150",
            /**     
            * Fires after Button control is created.
            * @event
            * @name ejButton#create		
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the button model
            * @param {string}  argument.type returns the name of the event
            * @param {boolean}  argument.status return the button state
            * @example 
            * //create event for button
            * $("#button1").ejButton({
            *    create: function (args) {}
            * });
            * @memberof ejButton
            * @instance
            */
            create: null,
            /**     
            * Fires when Button control is clicked successfully.
            * @event
            * @name ejButton#click 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the button model
            * @param {string}  argument.type returns the name of the event
            * @param {boolean}  argument.status return the button state
            * @example 
            * //click event for button
            * $("#button1").ejButton({
            *    click: function (args) {}
            * });      
            * @memberof ejButton
            * @instance
            */
            click: null,
            /**    
            * Fires when the button is destroyed successfully.
            * @event
            * @name ejButton#destroy 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the button model
            * @param {string}  argument.type returns the name of the event
            * @param {boolean}  argument.status return the button state
            * @example 
            * //destroy event for button
            * $("#button1").ejButton({
            *    destroy: function (args) {}
            * });
            * @memberof ejButton
            * @instance
            */
            destroy: null
        },

        /**
       * Specify the data types for default properties 
       * @private
       */
        dataTypes: {
            size: "enum",
            height: "string",
            width: "string",
            enabled: "boolean",
            roundedCorner: "boolean",
            text: "string",
            contentType: "enum",
            imagePosition: "enum",
            prefixIcon: "string",
            suffixIcon: "string",
            cssClass: "string",
            repeatButton: "boolean",
            rtl: "boolean",
            timeInterval: "string"
        },
        /**
        * To disable the button  
		* @return jQuery
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // Create Button
		* var btnObj = $("#button1").data("ejButton");
		* btnObj.disable(); // disable the button
		* &lt;/script&gt;
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // disable the button
		* $("#button1").ejButton("disable");	
		* &lt;/script&gt;
		*@memberof ejButton
		* @instance
        */
        disable: function () {
            /// <summary>This function is  used to Disable the Button Object</summary>
            this.element.addClass("e-disable").attr("aria-disabled", true);
            this.model.enabled = false;
        },
        /**
        * To enable the button 		
		* @return jQuery
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // Create Button
		* var btnObj = $("#button1").data("ejButton");
		* btnObj.enable(); // enable the button
		* &lt;/script&gt;
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // enable the button
		* $("#button1").ejButton("enable");	
		* &lt;/script&gt;
		* @memberof ejButton
		* @instance
         */
        enable: function () {
            /// <summary>This function is  used to Enable the Button Object</summary>
            this.element.removeClass("e-disable").attr("aria-disabled", false);
            this.model.enabled = true;
        },
        /**
         * Create the butto widget
		 * @private
         */
        _init: function () {
            this._initialize();
            this._render();
            this._controlStatus(this.model.enabled);
            this._wireEvents(this.model.repeatButton);
        },

        /**
       * destroy the button widget
       * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
       * @alias destroy
       * @return jQuery
       * @example 
       * &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
       * &lt;script&gt;
       * // Create Button
       * var btnObj = $("#button1").data("ejButton");
       * btnObj.destroy(); // destroy the button
       * &lt;/script&gt;
       * @example 
       * &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
       * &lt;script&gt;
       * // enable the button
       * $("#button1").ejButton("destroy");	
       * &lt;/script&gt;
       * @memberof ejButton
       * @instance
        */
        _destroy: function () {
            /// <summary>This function is  used to destroy the Button Object</summary>
            this.element.removeClass(this.model.cssClass + "e-btn e-select e-disable e-corner-all e-widget");
        },

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "text":
                        this._setText(options[option]);
                        break;
                    case "prefixIcon":
                        this._setMajorIcon(options[option]);
                        break;
                    case "suffixIcon":
                        this._setMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "roundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "rtl":
                        this._setRTL(options[option]);
                        break;
                    case "timeInterval":
                        this.model.timeInterval = options[option];
                        break;
                }
            }
        },

        /**
        * To configure size of the button		
        * @private
        */
        _setSize: function (val) {
            this.element.removeClass('e-btn-mini e-btn-medium e-btn-small e-btn-large e-btn-normal');
            this.element.addClass("e-btn-" + val);
        },
        /**
        * To configure the height		
        * @private
        */
        _setHeight: function (val) {
            /// <summary>This will set Text property of button </summary>
            this.element.css('height', val);
        },
        /**
        * To configure width.		
        * @private
        */
        _setWidth: function (val) {
            /// <summary>This will set Text property of button </summary>
            this.element.css('width', val);
        },
        /**
        * To configure the button text		
        * @private
        */
        _setText: function (val) {
            /// <summary>This will set Text property of button </summary>
            if (this.buttonType == "inputButton") {
                this.element.val(val);
            } else {
                if (this.model.contentType == ej.contents.textOnly) {
                    this.element.text(val);
                } else {
                    this.textspan.text(val);
                }
            }
            this.model.text = val;
        },
        /**
        * To configure button's prefix image		
        * @private
        */
        _setMajorIcon: function (val) {
            /// <summary>This will set Major Icon property of button </summary>
            this.majorimgtag.removeClass(this.model.prefixIcon);
            this.majorimgtag.addClass(val);
            this.model.prefixIcon = val;
        },
        /**
        * To configure button's suffix image		
        * @private
        */
        _setMinorIcon: function (val) {
            /// <summary>This will set Minor Icon property of button </summary>
            this.minorimgtag.removeClass(this.model.suffixIcon);
            this.minorimgtag.addClass(val);
            this.model.suffixIcon = val;
        },
        /**
        * To configure button contents type		
        * @private
        */
        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.element.empty();
                this.model.contentType = val;
                this._renderButtonNormal();
            }
        },
        /**
        * To configure images position inside the button		
        * @private
        */
        _setImagePosition: function (val) {
            if ((this.model.contentType == ej.contents.textAndImage) && (val != this.model.imagePosition)) {
                this.element.empty();
                this.model.imagePosition = val;
                this._renderButtonNormal();
            }
        },
        /**
        * To enable or disable the Right to Left behaviour 		
        * @private
        */
        _setRTL: function (val) {
            if (val) {
                this.element.addClass("e-rtl");
            } else {
                this.element.removeClass("e-rtl");
            }
        },
        /**
        * To enable or disable the button state		
        * @private
        */
        _controlStatus: function (value) {
            //disable status
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },
        /**
        * To configure the custom theme for button using cssClass property		
        * @private
        */
        _setSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass);
                this.element.addClass(skin);
            }
        },
        /**
        * To initialize the button		
        * @private
        */
        _initialize: function () {
            /// <summary>This function is  used to Initialize the Button Object</summary>
            if ((this.element.is("input[type=button]")) || (this.element.is("input[type=submit]")) || (this.element.is("input[type=reset]"))) {
                this.buttonType = "inputButton";
            } else if ((this.element.is("a")) || (this.element.is("button"))) {
                this.buttonType = "tagButton";
            } else {
                this.element.removeClass("e-button");
            }
            this._timeout = null;
        },

        /**
        * Render Section For DifferentTypes		
        * @private
        */
        _render: function () {
            /// <summary>This function is  used to Render the Button Object</summary>
            this._setSize(this.model.size);
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
            this._setRTL(this.model.rtl);
            this.element.addClass(this.model.cssClass + " e-btn e-select e-widget").attr("role", "button");
            if (this.buttonType == "inputButton") {
                this.element.addClass("e-txt");
                if ((this.model.text != null) && (this.model.text != "")) {
                    this.element.val(this.model.text);
                } else {
                    this.model.text = this.element.val();
                }
            } else { this._renderButtonNormal(); }
            this._roundedCorner(this.model.roundedCorner);
            if (this.model.text)
                this.element.attr("aria-describedby", this.model.text);
        },
        /**
        * Render Section For Normal button		
        * @private
        */
        _renderButtonNormal: function () {
            if ((this.model.text == null) || (this.model.text == "")) {
                this.model.text = this.element.text();
            }
            this.element.empty();
            /*Image and Text*/
            this.textspan = ej.buildTag('span.e-btntxt', this.model.text);
            if (this.model.contentType.indexOf("image") > -1) {
                this.majorimgtag = ej.buildTag('span.e-icon ' + this.model.prefixIcon);
                this.minorimgtag = ej.buildTag('span.e-icon ' + this.model.suffixIcon);
                this.imgtxtwrap = ej.buildTag('div');
            }
            /*Rendering Option*/
            if (this.model.contentType == ej.contents.textAndImage) {
                if (this.model.imagePosition == ej.imagePositions.imageRight) {
                    this.imgtxtwrap.append(this.textspan, this.majorimgtag);
                    this.element.append(this.imgtxtwrap);
                } else {
                    this.imgtxtwrap.append(this.majorimgtag, this.textspan);
                    this.element.append(this.imgtxtwrap);
                }
            } else if (this.model.contentType == ej.contents.imageTextImage) {
                this.imgtxtwrap.append(this.majorimgtag, this.textspan, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.contents.imageBoth) {
                this.imgtxtwrap.append(this.majorimgtag, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.contents.imageOnly) {
                this.imgtxtwrap.append(this.majorimgtag);
                this.element.append(this.imgtxtwrap);
            } else {
                this.element.addClass("e-txt");
                this.element.text(this.model.text);
            }
        },
        /**
        * To enable or disable the rounded corner behaviour 		
        * @private
        */
        _roundedCorner: function (value) {
            value == true ? this.element.addClass('e-corner-all') : this.element.removeClass('e-corner-all');
        },
        /**
        * Wiring the events to button control		
        * @private
        */
        _wireEvents: function (val) {
            /// <summary>This function is  used to Apply Events in the Button Object</summary>
            if (val) {
                this._on(this.element, "mousedown", this._btnRepatMouseClickEvent);
                this._on($(document), 'mouseup', this._mouseUpClick);
                this._on(this.element, "keyup", this._btnRepatKeyUpEvent);
                this._on($(document), "keypress", this._btnRepatKeyDownEvent);

            }
            this._on(this.element, "click", this._btnMouseClickEvent);
        },
        /**
        * Section For handle the click event
        * @private
        */
        _btnMouseClickEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                var args = { status: self.model.enabled };
                self._trigger("click", args);
            }
        },
        /**
         * Section For handle repeat button click action.
		 * @private
         */
        _btnRepatMouseClickEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                var args = { status: self.model.enabled };
                if ((e.button == 0) || (e.which == 1)) {

                    self._timeout = setInterval(function () { self._trigger("click", { target: e.currentTarget, status: self.model.enabled }); }, this.model.timeInterval);
                }
            }
        },
        /**
         * Section to track the mouse up event		
		 * @private
         */
        _mouseUpClick: function (event) {
            clearTimeout(this._timeout);
        },
        /**
         * Section to watch the repeat key down action.		
		 * @private
         */
        _btnRepatKeyDownEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                var args = { status: self.model.enabled };
                if ((e.keyCode == 32) || (e.keyCode == 13)) {
                    self._trigger("click", args);
                }
            }
        },
        /**
         * Section to watch the repeat key up action.		
		 * @private
         */
        _btnRepatKeyUpEvent: function (e) {
            if ((e.keyCode == 32) || (e.keyCode == 13)) {
                clearTimeout(this._timeout);
            }
        },
    });

    /**
	 * Enum for button content mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.contents = {
        /**  Supports only for text content only */
        textOnly: "textonly",
        /** Supports only for image content only */
        imageOnly: "imageonly",
        /** Supports image for both ends of the button */
        imageBoth: "imageboth",
        /** Supports image with the text content */
        textAndImage: "textandimage",
        /** Supports image with both ends of the text */
        imageTextImage: "imagetextimage"
    };

    /**
	 * Enum for Button positioning	 
	 * @enum {string}
	 * @global 
	 */
    ej.imagePositions = {
        /**  support for aligning text in right and image in left. */
        imageRight: "imageright",
        /**  support for aligning text in left and image in right. */
        imageLeft: "imageleft"
    };
    /**
	 * Enum for various button sizes	 
	 * @enum {string}
	 * @global 
	 */
    ej.buttonSize = {
        /**  Creates button with inbuilt default size height, width specified */
        normal: "normal",
        /**  Creates button with inbuilt mini size height, width specified */
        mini: "mini",
        /**  Creates button with inbuilt small size height, width specified */
        small: "small",
        /**  Creates button with inbuilt medium size height, width specified */
        medium: "medium",
        /**  Creates button with inbuilt large size height, width specified */
        large: "large"
    };

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html CheckBox elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejCheckBox
   * @requires jQuery
   * @requires ej.common.all
   * @classdesc Custom Design for Html CheckBox control.
   * @example 
   * &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
   * &lt;script&gt; 
   * // Create Checkbox  
   * $("#chkbox").ejCheckBox(); 
   * &lt;/script&gt;
   */

    ej.widget("ejCheckBox", "ej.CheckBox", {
        _rootCSS: "e-checkbox",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Specifies the id atribute of the Checkbox.
			* @default null
			* @type {String}
			* @example 
			* //To set id API value during initialization  
			* 	$("#chkbox").ejCheckBox({  id: "sync" });				 
			* @example 
			* //Get or set the id API, after initialization:<br>
			*	//Gets the id value  
			*	$("#chkbox").ejCheckBox("option", "id");<br>			
			*	//Sets the id value 
			*	$("#chkbox").ejCheckBox("option", "id",  "sync" ); 
			 * @memberof ejCheckBox
			* @instance
			*/
            id: null,
            /**		
			* Specifies the name atribute of the Checkbox.
			* @default null
			* @type {String}
			* @example 
			* //To set name API value during initialization  
			* 	$("#chkbox").ejCheckBox({  name: "sync" });					
			* @example 
			* //Get or set the name API, after initialization:<br>
			*	//Gets the name value  
			*	$("#chkbox").ejCheckBox("option", "name");<br>			
			*	//Sets the name value 
			*	$("#chkbox").ejCheckBox("option", "name",  "sync" ); 
			 * @memberof ejCheckBox
			* @instance
			*/
            name: null,
            /**		
			* Specifies the value atribute of the Checkbox.
			* @default null
			* @type {String}
			* @example 
			* //To set value API value during initialization  
			* 	$("#chkbox").ejCheckBox({ value: "Hello World"});					* 
			* @example 
			* //Get or set the value API, after initialization:<br>
			*	//Gets the value value  
			*	$("#chkbox").ejCheckBox("option", "value");<br>			
			*	//Sets the value value 
			*	$("#chkbox").ejCheckBox("option", "value", "Hello World" ); 
			 * @memberof ejCheckBox
			* @instance
			*/
            value: null,
            /**		
			* Specifies the check atribute of the Checkbox.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set check API value during initialization  
			* 	$("#chkbox").ejCheckBox({ check:  true });					* 
			* @example 
			* //Get or set the check API, after initialization:<br>
			*	//Gets the check value  
			*	$("#chkbox").ejCheckBox("option", "check");<br>			
			*	//Sets the check value 
			*	$("#chkbox").ejCheckBox("option", "check",  false );  
			 * @memberof ejCheckBox
			* @instance
			*/
            check: null,
            /**		
			* Specifies the checkbox control state.
			* @default true
			* @type {boolean}
			* @example 
			* // Enable checkbox on initialization. 
			* 	//To set width API value 
			* 	 $("#chkbox").ejCheckBox ({ enabled: true });			
			* @example 
			* //Get or set the checkbox state, after initialization:<br>
			*	// Gets the Enabled API value.		
			*	$("#chkbox").ejCheckBox ("option", "enabled");			
			*	// Sets the Enabled API
			*	$("#chkbox").ejCheckBox ("option", "enabled", true);
			 * @memberof ejCheckBox
			* @instance
			*/
            enabled: true,
            /**		
			* Specifies the enable or disable Tri-State for checkbox control.
			* @default false
			* @type {boolean}
			* @example 
			* // Specifies to enable or disable Tri-State option checkbox while initialization. 
			* 	//To set indeterminate API value 
			* $("#chkbox").ejCheckBox({  indeterminate: true });		
			* @example 
			* //Get or set the indeterminate API, after initialization
			*	//Gets the  indeterminate value 
			* $("#chkbox").ejCheckBox("option", "indeterminate");	
			*	//Sets the  indeterminate value 
			* $("#chkbox").ejCheckBox("option", "indeterminate", false  ); 
			 * @memberof ejCheckBox
			* @instance
			*/
            indeterminate: false,
            /**		
			* Specifies the Tri-State property for checkbox control.
			* @default false
			* @type {boolean}
			* @example 
			* // Specifies the value for the Tri-State property of the Checkbox.
			* 	//To set indeterminateState  API value 
			* $("#chkbox").ejCheckBox({  indeterminateState : true });		
			* @example 
			* //Get or set the indeterminateState  API, after initialization
			*	//Gets the  indeterminateState  value 
			* $("#chkbox").ejCheckBox("option", "indeterminateState ");	
			*	//Sets the  indeterminateState  value 
			* $("#chkbox").ejCheckBox("option", "indeterminateState ", false  ); 
			 * @memberof ejCheckBox
			* @instance
			*/
            indeterminateState: false,
            /**		
			* Specify the rounded corner to checkbox
			* @default false
			* @type {boolean}
			* @example 
			* // Set the rounded corner during initialization. 			
			* 	$("#chkbox").ejCheckBox({ roundedCorner: false });			* 
			* @example 
			* //Get or set the rounded corner after initialization:<br>
			*	// Gets the rounded corner value.		
			*	$("#chkbox").ejCheckBox("option", "roundedCorner");			
			*	// Sets the rounded corner to checkbox
			*	$("#chkbox").ejCheckBox("option", "roundedCorner",  true ); 
			 * @memberof ejCheckBox
			* @instance
			*/
            roundedCorner: false,
            /**		
			* Specifies the persist property for Checkbox while initialization. The persist API save current model value to browser cookies for state maintains. While refreshing the Checkbox control page the model value apply from browser cookies. 
			* @default false
			* @type {boolean}
			* @example 
			* // To set persist API value 
			* $("#chkbox").ejCheckBox({ persist: false });			
			* @example 
			* //Get or set the persist API, after initialization:
			*	//Gets the persist value 
			* $("#chkbox").ejCheckBox("option", "persist");		
			*	// Sets the persist value
			* $("#chkbox").ejCheckBox("option", "persist",  true ); 
			 * @memberof ejCheckBox
			* @instance
			*/
            persist: false,
            /**		
			* Specify the CSS class to CheckBox to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the root class for Checkbox control theme. This cssClass API helps to use custom skinning option for Checkbox control. By defining the root class using this API, we need to include this root class in CSS. 			
			* 	$("#chkbox").ejCheckBox({cssClass: "gradient-lime"}); 
			* //Get or set the cssClass API, after initialization:
			* //Gets the cssClass applied for the control 
			* $("#chkbox").ejCheckBox("option", "cssClass");
			*	// Sets the cssClass for the control  
			* $("#chkbox").ejCheckBox("option", "cssClass", "gradient-lime");
			* @memberof ejCheckBox
			* @instance
			*/
            cssClass: "",
            /**		
			* Specifies the text content for CheckBox.
			* @default ""
			* @type {string}
			* @example 
			* // To set text API value 
			* $("#chkbox").ejCheckBox({ text: "Hello World"});		 
			* @example 
			* //Get or set the text API, after initialization:
			* //Gets the text value  
			* $("#chkbox").ejCheckBox("option", "text");			
			*	// Sets the text value 
			*$("#chkbox").ejCheckBox("option", "text", "Hello World" ); 
			* @memberof ejCheckBox
			* @instance
			*/
            text: "",
            /**		
			* Specify the Right to Left direction to CheckBox
			* @default false
			* @type {boolean}
			* @example 
			* // Set the RTL during initialization. 			
			* 	$("#chkbox").ejCheckBox({  rtl : true });			
			* @example 
			* //Get or set the RTL after initialization:<br>
			*	// Gets the rounded corner value.		
			*	$("#chkbox").ejCheckBox("option", "rtl");			
			*	// Sets the RTL to button
			*	$("#chkbox").ejCheckBox("option", "rtl",  true );
			* @memberof ejCheckBox
			* @instance
			*/
            rtl: false,
            /**		
			* Specify the idprefix value to be added before the current id of the checkbox.
			* @default "ej"
			* @type {String}
			* @example 
			* // To set  idPrefix  API value
			* $("#chkbox").ejCheckBox ({  idPrefix : true }); 
			* @example 
			* //Get or set the idPrefix API, after initialization:
			* //Get the  idPrefix  value
			* $("#chkbox").ejCheckBox ("option", " idPrefix ");			
			*	// Set the  idPrefix  value
			* $("#chkbox").ejCheckBox ("option", " idPrefix ", “ej”);
			* @memberof ejCheckBox
			* @instance
			*/
            idPrefix: "ej",
            /**		
			* Specifies the size of the CheckBox.	See {@link buttonSize}
			* @default "mini"
			* @type {enum}
			* @example 
			* //To set size API value during initialization  
			* 	$("#chkbox").ejCheckBox({  size: "medium"});					 
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#chkbox").ejCheckBox("option", "size");<br>			
			*	//Sets the size value 
			*	$("#chkbox").ejCheckBox("option", "size",  "small" ); 
			* @memberof ejCheckBox
			* @instance
			*/
            size: "",
            /**    
			 * Fires before the Checkbox is going to changed its state successfully
			 * @event
			 * @name ejCheckBox#beforeChange 	
			 * @param {Object} argument Event parameters from CheckBox     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {boolean} argument.model returns the Checkbox model
             * @param {boolean} argument.type returns the name of the event
             * @param {boolean} argument.data.element returns the current element
             * @param {boolean} argument.data.isChecked  returns the status of the element
			 * @example 
			 * // //beforeChange event for checkbox
             * $("#chkbox").ejCheckBox({
             *   beforeChange: function (args) { //handle the event }
             *});           
			 * @memberof ejCheckBox
			 * @instance
			 */
            beforeChange: null,
            /**    
			 * Fires when the Checkbox state  is changed successfully
			 * @event
			 * @name ejCheckBox#change 	
			 * @param {Object} argument Event parameters from CheckBox     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {boolean} argument.model returns the Checkbox model
             * @param {boolean} argument.type returns the name of the event
             * @param {boolean} argument.data.element returns the current element
             * @param {boolean} argument.data.isChecked  returns the status of the element
			 * @example 
			 * // //change event for checkbox
             * $("#chkbox").ejCheckBox({
             *   change: function (args) { //handle the event }
             *});           
			 * @memberof ejCheckBox
			 * @instance
			 */
            change: null,
        },
        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            id: "string",
            name: "string",
            check: "boolean",
            persist: "boolean",
            indeterminate: "boolean",
            size: "enum",
            enabled: "boolean",
            idPrefix: "string"
        },
        /**
        * Create the CheckBox widget
        * @private
        */

        // constructor function
        _init: function () {
            this._setValue();
            this._renderControl();
            if (this.model.enabled)
                this._wireEvents();
            this._setEnabled(this.model.enabled);
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */

        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var prop in options) {
                switch (prop) {
                    case "check":
                        this.isChecked = options[prop];
                        if (this.isChecked)
                            this._checked();
                        else
                            this._unChecked();
                        break;
                    case "indeterminate":
                        if (options[prop])
                            this._setIndeterminate(this.model.indeterminateState);
                        break;
                    case "indeterminateState":
                        if (this.model.indeterminate)
                            this._setIndeterminate(options[prop]);
                        break;
                    case "cssClass": this._changeSkin(options[prop]); break;
                    case "rtl":
                        if (options[prop])
                            $("#" + this.model.idPrefix + this.model.id + "_wrapper").addClass("e-rtl");
                        else
                            $("#" + this.model.idPrefix + this.model.id + "_wrapper").removeClass("e-rtl");
                        break;
                    case "text": this._setText(options[prop]); break;
                    case "id": this._setIdAttr(options[prop]); break;
                    case "name": this.chkbx.attr('name', options[prop]); break;
                    case "value": this.chkbx.attr('value', options[prop]); break;
                    case "size": this._setSize(options[prop]); break;
                    case "roundedCorner": this._setRoundedCorner(options[prop]); break;
                    case "enabled": this._setEnabled(options[prop]); break;
                }
            }
        },
        /**
        * destroy the CheckBox widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
		* &lt;script&gt; 
		* // Create Checkbox  
		* var chkObj = $("#chkbox").data("ejCheckBox");
		* chkObj.destroy();
		* &lt;/script&gt;
		* @example 
		* &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
		* &lt;script&gt;
		* // enable the CheckBox
		* $("#chkbox").ejCheckBox("destroy");	
		* &lt;/script&gt;
		* @memberof ejCheckBox
		* @instance
         */

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.chkbx.removeClass("e-checkbox e-input");
            this.chkbx.insertBefore(this.element);
            this.element.remove();
            this.element = this.chkbx;
        },
        /**
         * To configure the custom theme for checkbox using cssClass property		
		 * @private
         */
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
                $("#" + this.model.idPrefix + this.model.id + "_wrapper").removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
         * To configure the Value		
		 * @private
         */
        _setValue: function () {
            if (this.model.id == null)
                this.model.id = this.element.attr('id');
            if (!this.model.id)
                this.model.id = this.element.attr('name');
            !ej.isNullOrUndefined(this.element.attr('name')) && (this.model.name = this.element.attr('name'));
            !ej.isNullOrUndefined(this.element.attr('value')) && (this.model.value = this.element.attr('value'));
            this.isChecked = this.element.attr('checked') == 'checked' ? true : false;
            this.isChecked = this.model.check == null ? this.isChecked : this.model.check;
        },
        /**
         * To configure the Size of the CheckBox		
		 * @private
         */
        _setSize: function (val) {
            if (val == ej.checkboxSize.medium)
                this.innerdiv.removeClass('e-chkbox-small').addClass('e-chkbox-medium');
            else
                this.innerdiv.removeClass('e-chkbox-medium').addClass('e-chkbox-small');
        },
        /**
         * To configure the RoundedCorner of the CheckBox		
		 * @private
         */
        _setRoundedCorner: function (val) {
            if (val)
                this.span.addClass("e-corner-all");
            else
                this.span.removeClass("e-corner-all");
        },
        /**
         * To configure the Enable status of the CheckBox		
		 * @private
         */
        _setEnabled: function (val) {
            if (val) {
                this.enable();
            } else {
                this.disable();
            }
        },
        /**
         * Render Section For CheckBox control		
		 * @private
         */
        _renderControl: function () {
            this.maindiv = ej.buildTag("span.e-chkbox-wrap e-widget " + this.model.cssClass + "#" + this.model.idPrefix + this.model.id, "", {}, { "role": "checkbox", tabindex: 0 });
            this.innerdiv = $('<div></div>');
            this._setSize(this.model.size);
            this.span = $('<span></span>');
            this.spanImg = $('<span class="e-chk-image"></span>').attr("role", "presentation");
            this.element.addClass("e-input");
            this.element.attr("id", this.model.id);
            //this.span.addClass("e-innerspan");
            this._setRoundedCorner(this.model.roundedCorner);
            if (this.isChecked) {
                this.spanImg.addClass("e-chk-activeicon");
                this.span.addClass("e-chk-act");
                this.maindiv.attr({ "aria-checked": true });
            }
            else {
                this.span.addClass("e-chk-inact");
                this.maindiv.attr({ "aria-checked": false });
            }
            this.span.append(this.spanImg);
            this.innerdiv.append(this.span);
            this.maindiv.insertBefore(this.element);
            this.maindiv.append(this.element);
            this.maindiv.append(this.innerdiv);
            this._setTextWrapper(this.model.text);
            this.chkbx = this.element;
            this.element = this.maindiv;
            if (this.model.indeterminate == true && this.model.indeterminateState == true)
                this._setIndeterminate(this.model.indeterminateState);
        },
        /**
        * To configure checkbox tri-state type		
        * @private
        */
        _setIndeterminate: function (indeter) {
            if (indeter) {
                this.spanImg.removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-inact").addClass("e-chk-indeter");
                this.maindiv.attr({ "aria-checked": "mixed" });
                this.element.find('input').prop('indeterminate', true);
            }
            else {
                this.element.find(".e-chk-indeter").removeClass("e-chk-indeter");
                this.element.find(".e-chk-image").removeClass("e-chk-indetericon");
                this.element.find('input').removeAttr('indeterminate');
                this.element.find('input').prop('indeterminate', false);
                if (this.model.check)
                    this._checked();
                else
                    this._unChecked();
            }
        },
        /**
         * To configure checkbox textwrapper		
		 * @private
         */
        _setTextWrapper: function (val) {
            if (val != "") {
                //this.textWrapDiv = ej.buildTag("div.e-chkbox-wrap " + this.model.cssClass + "#" + this.model.idPrefix + this.model.id + "_wrapper");
                //this.maindiv.wrap(this.textWrapDiv);
                this.txtSpan = ej.buildTag("div.e-text", val);
                this.maindiv.append(this.txtSpan);
                if (this.model.rtl)
                    $("#" + this.model.idPrefix + this.model.id + "_wrapper").addClass("e-rtl");
            }
        },
        /**
         * To configure checkbox text value		
		 * @private
         */
        _setText: function (val) {
            if ((this.model.text == "") && (val != "")) {
                this._setTextWrapper(val);
            } else {
                this.txtSpan.html(val);
            }
        },
        /**
         * To configure checkbox ID attribute value		
		 * @private
         */
        _setIdAttr: function (val) {
            $("#" + this.model.idPrefix + this.model.id + "_wrapper").attr('id', this.model.idPrefix + val + "_wrapper");
            this.chkbx.attr('id', val);
        },
        /**
        * Wiring the events to checkbox control		
        * @private
        */
        _wireEvents: function () {
            this._on(this.element, "click", this._checkedHandler);
            this._on(this.element, "focus", this._focusIn);
            this._on(this.element, "focusout", this._focusOut);
        },
        /**
         * Unwiring the events to checkbox control		
		 * @private
         */

        _unWireEvents: function () {
            this._off(this.element, "click");
            this._off(this.element, "focus");
            this._off(this.element, "focusout");
        },
        _focusIn: function (evt) {
            $(this.element).addClass("e-focus");
            $(this.element).bind("keypress", $.proxy(this._checkUnCheck, this));
        },
        _focusOut: function (evt) {
            $(this.element).removeClass("e-focus");
            $(this.element).unbind("keypress", $.proxy(this._checkUnCheck, this));
        },
        _checkUnCheck: function (evt) {
            //Space bar to check and uncheck
            if (evt.keyCode == 32) {
                evt.preventDefault();
                this._checkedHandler();
            }
        },
        _checkedHandler: function (evt) {
            var data = { isChecked: this.isChecked };
            if (true == this._trigger("beforeChange", data)) {
                return false;
            }
            if (this.element.find("span:first").hasClass("e-chk-inact")) {
                this._checked();
                this.isChecked = this.model.check = true;
                if (this.model.indeterminate) {
                    this.model.indeterminateState = true;
                }
            }
            else if (this.element.find("span:first").hasClass("e-chk-act")) {
                if ((this.model.indeterminate == true) && (this.model.indeterminateState == true)) {
                    this._setIndeterminate(true);
                    this.isChecked = this.model.check = true;
                } else {
                    this._unChecked();
                    this.isChecked = this.model.check = false;
                }


            }
            else if (this.element.find("span:first").hasClass("e-chk-indeter")) {
                this.isChecked = this.model.check = false;
                this._setIndeterminate(false);
                this.model.indeterminateState = false;
            }
            var data = { isChecked: this.isChecked };
            this._trigger("change", data);
            return true;
        },
        /**
         * Section For handling the Check event handler
		 * @private
         */

        _checked: function () {
            this.element.find("span:first").removeClass("e-chk-inact").addClass("e-chk-act");
            this.element.find(".e-chk-image").addClass("e-chk-activeicon");
            this.maindiv.attr({ "aria-checked": true });
            this.element.find('input[type=checkbox]').prop('checked', true);
        },
        /**
         * Section For handling the Uncheck event handler
		 * @private
         */

        _unChecked: function () {
            this.element.find("span:first").removeClass("e-chk-act").addClass("e-chk-inact");
            this.maindiv.attr({ "aria-checked": false });
            this.element.find(".e-chk-image").removeClass("e-chk-activeicon").removeClass("e-chk-indetericon");
            this.element.find('input[type=checkbox]').removeAttr('checked');
        },
        /**
        * To disable the checkbox
		* @return jQuery
		* @example 
		* &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
		* &lt;script&gt; 
		* // Create Checkbox  
		* var chkObj = $("#chkbox").data("ejCheckBox");
		* chkObj.disable();
		* &lt;/script&gt;
		* @example 
		* &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
		* &lt;script&gt;
		* // enable the CheckBox
		* $("#chkbox").ejCheckBox("disable");	
		* &lt;/script&gt;
		* @memberof ejCheckBox
		* @instance
        */

        disable: function () {
            if (!this.element.hasClass("e-disable")) {
                this.element.addClass("e-disable").attr("aria-disabled", true);
                this._unWireEvents();
                this.model.enabled = false;
            }
        },
        /**
        * To enable the checkbox
		* @return jQuery
		* @example 
		* &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
		* &lt;script&gt; 
		* // Create Checkbox  
		* var chkObj = $("#chkbox").data("ejCheckBox");
		* chkObj.enable();
		* &lt;/script&gt;
		* @example 
		* &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
		* &lt;script&gt;
		* // enable the CheckBox
		* $("#chkbox").ejCheckBox("enable");
		* &lt;/script&gt;
		* @memberof ejCheckBox
		* @instance
        */
        enable: function () {
            if (this.element.hasClass("e-disable")) {
                this.element.removeClass("e-disable").attr("aria-disabled", false);
                this._wireEvents();
                this.model.enabled = true;
            }
        },
        /**
        * To Check the status of checkbox
		* @return jQuery
		* @example 
		* &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
		* &lt;script&gt; 
		* // Create Checkbox  
		* var chkObj = $("#chkbox").data("ejCheckBox");
		* chkObj.isChecked();
		* &lt;/script&gt;
		* @example 
		* &lt;input type="checkbox" id="chkbox"&gt;Checkbox&lt;/checkbox&gt<br>
		* &lt;script&gt;
		* // enable the CheckBox
		* $("#chkbox").ejCheckBox("isChecked");	
		* &lt;/script&gt;
		* @memberof ejCheckBox
		* @instance
        */
        isChecked: function () {
            if ((this.isChecked != null) && (this.isChecked != undefined))
                return this.isChecked;
        }
    });
    ej.checkboxSize = { small: "small", medium: "medium" };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Radiobutton elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejRadioButton
	* @requires jQuery
	* @requires ej.common.all
	* @classdesc Custom Design for RadioButton control. 
	* @example
	*&lt;input type="radio" id="radiobtn"/&gt;
	*&lt;script&gt;
	*$("#radiobtn").ejRadioButton();
	*&lt;/script&gt;
	*/
    ej.widget("ejRadioButton", "ej.RadioButton", {
        _rootCSS: "e-radiobtn",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Specifies the id attribute for the Radio Button while initialization.
			* @default null
			* @type {String}
			* @example 
			* //To set id API value during initialization  
			* 	$("#radiobtn").ejRadioButton({  id: "sync" });
			* @example 
			* //Get or set the id API, after initialization:<br>
			*	//Gets the id value  
			*	$("#radiobtn").ejRadioButton("option", "id");
			*	//Sets the id value 
			*	$("#radiobtn").ejRadioButton("option", "id",  "sync" ); 
			 * @memberof ejRadioButton
			* @instance
			*/
            id: null,
            /**		
			* Specifies the name attribute for the Radio Button while initialization.
			* @default null
			* @type {String}
			* @example 
			* //To set name API value during initialization  
			* 	$("#radiobtn").ejRadioButton({  name: "sync" });					
			* @example 
			* //Get or set the name API, after initialization:<br>
			*	//Gets the name value  
			*	$("#radiobtn").ejRadioButton("option", "name");<br>			
			*	//Sets the name value 
			*	$("#radiobtn").ejRadioButton("option", "name",  "sync" ); 
			 * @memberof ejRadioButton
			* @instance
			*/
            name: null,
            /**		
			* Specifies the value atribute of the Radio Button.
			* @default null
			* @type {String}
			* @example 
			* //To set value API value during initialization  
			* 	$("#radiobtn").ejRadioButton({ value: "Hello World"});					* 
			* @example 
			* //Get or set the value API, after initialization:<br>
			*	//Gets the value value  
			*	$("#radiobtn").ejRadioButton("option", "value");<br>			
			*	//Sets the value value 
			*	$("#radiobtn").ejRadioButton("option", "value", "Hello World" ); 
			 * @memberof ejRadioButton
			* @instance
			*/
            value: null,
            /**		
			* Specifies the check atribute of the Radio Button.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set check API value during initialization  
			* 	$("#radiobtn").ejRadioButton({ check:  true });					* 
			* @example 
			* //Get or set the check API, after initialization:<br>
			*	//Gets the check value  
			*	$("#radiobtn").ejRadioButton("option", "check");<br>			
			*	//Sets the check value 
			*	$("#radiobtn").ejRadioButton("option", "check",  false );  
			 * @memberof ejRadioButton
			* @instance
			*/
            check: null,
            /**		
			* Specify the CSS class to RadioButton to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the root class for RadioButton control theme. This cssClass API helps to use custom skinning option for RadioButton control. By defining the root class using this API, we need to include this root class in CSS. 			
			* 	$("#radiobtn").ejRadioButton({cssClass: "gradient-lime"}); 
			* //Get or set the cssClass API, after initialization:
			* //Gets the cssClass applied for the control 
			* $("#radiobtn").ejRadioButton("option", "cssClass");
			*	// Sets the cssClass for the control  
			* $("#radiobtn").ejRadioButton("option", "cssClass", "gradient-lime");
			* @memberof ejRadioButton
			* @instance
			*/
            cssClass: "",
            /**		
			* Specifies the text content for RadioButton.
			* @default ""
			* @type {string}
			* @example 
			* // To set text API value 
			* $("#radiobtn").ejRadioButton({ text: "Hello World"});		 
			* @example 
			* //Get or set the text API, after initialization:
			* //Gets the text value  
			* $("#radiobtn").ejRadioButton("option", "text");			
			*	// Sets the text value 
			*$("#radiobtn").ejRadioButton("option", "text", "Hello World" ); 
			* @memberof ejRadioButton
			* @instance
			*/
            text: "",
            /**		
			* Specify the Right to Left direction to RadioButton
			* @default false
			* @type {boolean}
			* @example 
			* // Set the RTL during initialization. 			
			* 	$("#radiobtn").ejRadioButton({  rtl : true });			
			* @example 
			* //Get or set the RTL after initialization:<br>
			*	// Gets the rounded corner value.		
			*	$("#radiobtn").ejRadioButton("option", "rtl");			
			*	// Sets the RTL to button
			*	$("#radiobtn").ejRadioButton("option", "rtl",  true );
			* @memberof ejRadioButton
			* @instance
			*/
            rtl: false,
            /**		
			* Specifies the persist property for RadioButton while initialization. The persist API save current model value to browser cookies for state maintains. While refreshing the radiobutton control page the model value apply from browser cookies. 
			* @default false
			* @type {boolean}
			* @example 
			* // To set persist API value 
			* $("#radiobtn").ejRadioButton({ persist: false });			
			* @example 
			* //Get or set the persist API, after initialization:
			*	//Gets the persist value 
			* $("#radiobtn").ejRadioButton("option", "persist");		
			*	// Sets the persist value
			* $("#radiobtn").ejRadioButton("option", "persist",  true ); 
			 * @memberof ejRadioButton
			* @instance
			*/
            persist: false,
            /**		
			* Specify the idprefix value to be added before the current id of the RadioButton.
			* @default "ej"
			* @type {String}
			* @example 
			* // To set  idPrefix  API value
			* $("#radiobtn").ejRadioButton ({  idPrefix : true }); 
			* @example 
			* //Get or set the idPrefix API, after initialization:
			* //Get the  idPrefix  value
			* $("#radiobtn").ejRadioButton ("option", " idPrefix ");			
			*	// Set the  idPrefix  value
			* $("#radiobtn").ejRadioButton ("option", " idPrefix ", “ej”);
			* @memberof ejRadioButton
			* @instance
			*/
            idPrefix: "ej",
            /**		
			* Specifies the size of the RadioButton.	See {@link RadioButtonSize}
			* @default "small"
			* @type {enum}
			* @example 
			* //To set size API value during initialization  
			* 	$("#radiobtn").ejRadioButton({  size: "medium"});					 
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#radiobtn").ejRadioButton("option", "size");<br>			
			*	//Sets the size value 
			*	$("#radiobtn").ejRadioButton("option", "size",  "small" ); 
			* @memberof ejRadioButton
			* @instance
			*/
            size: "smaller",
            /**		
			* Specifies the RadioButton control state.
			* @default true
			* @type {boolean}
			* @example 
			* // Enable RadioButton on initialization. 
			* 	//To set width API value 
			* 	 $("#radiobtn").ejRadioButton ({ enabled: true });			
			* @example 
			* //Get or set the RadioButton state, after initialization:<br>
			*	// Gets the Enabled API value.		
			*	$("#radiobtn").ejRadioButton ("option", "enabled");			
			*	// Sets the Enabled API
			*	$("#radiobtn").ejRadioButton ("option", "enabled", true);
			 * @memberof ejRadioButton
			* @instance
			*/
            enabled: true,
            /**    
			 * Fires before the RadioButton is going to changed its state successfully
			 * @event
			 * @name ejRadioButton#beforeChange 	
			 * @param {Object} argument Event parameters from RadioButton     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {boolean} argument.model returns the RadioButton model
             * @param {boolean} argument.type returns the name of the event
             * @param {boolean} argument.data.element returns the current element
             * @param {boolean} argument.data.isChecked  returns the status of the element
			 * @example 
			 * // //beforeChange event for RadioButton
             * $("#radiobtn").ejRadioButton({
             *   beforeChange: function (args) { //handle the event }
             *});           
			 * @memberof ejRadioButton
			 * @instance
			 */
            beforeChange: null,
            /**    
			 * Fires when the RadioButton state  is changed successfully
			 * @event
			 * @name ejRadioButton#change 	
			 * @param {Object} argument Event parameters from RadioButton     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {boolean} argument.model returns the RadioButton model
             * @param {boolean} argument.type returns the name of the event
             * @param {boolean} argument.data.element returns the current element
             * @param {boolean} argument.data.isChecked  returns the status of the element
			 * @example 
			 * // //change event for RadioButton
             * $("#radiobtn").ejRadioButton({
             *   change: function (args) { //handle the event }
             *});           
			 * @memberof ejRadioButton
			 * @instance
			 */
            change: null,
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */

        dataTypes: {
            id: "string",
            name: "string",
            check: "boolean",
            persist: "boolean",
            size: "enum",
            enabled: "boolean",
            idPrefix: "string"
        },
        /**
		 * Create the RadioButton widget
		 * @private
		 */
        // constructor function
        _init: function () {
            this._setValue();
            this._renderControl();
            this._setEnabled(this.model.enabled);
            this._wireEvents();
            if (this.isChecked)
                this._checkedHandler();
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */

        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var prop in options) {
                switch (prop) {
                    case "cssClass": this._changeSkin(options[prop]); break;
                    case "rtl":
                        if (options[prop])
                            this.textWrapDiv.addClass("e-rtl");
                        else
                            this.textWrapDiv.removeClass("e-rtl");
                        break;
                    case "text": this._setText(options[prop]); break;
                    case "size": this._setSize(options[prop]); break;
                    case "check": this.model.checked = options[prop]; this._checkedChange(this.model.checked); break;
                    case "enabled": this._setEnabled(options[prop]); break;
                }
            }
        },
        /**
        * destroy the RadioButton widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;input type="radio" id="radiobtn"&gt;RadioButton&lt;/radio&gt<br>
		* &lt;script&gt; 
		* // Create RadioButton
		* var chkObj = $("#radiobtn").data("ejRadioButton");
		* chkObj.destroy();
		* &lt;/script&gt;
		* @example 
		* &lt;input type="radio" id="radiobtn"&gt;RadioButton&lt;/radio&gt<br>
		* &lt;script&gt;
		* // enable the RadioButton
		* $("#radiobtn").ejRadioButton("destroy");	
		* &lt;/script&gt;
		* @memberof ejRadioButton
		* @instance
         */

        // all events bound using this._on will be unbind automatically _checkedChange
        _destroy: function () {
            this.radbtn.removeClass("e-radiobtn e-input");
            this.radbtn.insertBefore(this.element);
            this.element.remove();
            this.element = this.radbtn;
        },
        /**
         * To configure the custom theme for RadioButton using cssClass property		
		 * @private
         */
        //Skin Change at run time
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
                this.textWrapDiv.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
         * To configure the Value		
		 * @private
         */
        _setValue: function () {
            this.model.id = this.element.attr('id');
            if (!this.model.id)
                this.model.id = this.element.attr('name');
            !ej.isNullOrUndefined(this.element.attr('name')) && (this.model.name = this.element.attr('name'));
            !ej.isNullOrUndefined(this.element.attr('value')) && (this.model.value = this.element.attr('value'));
            this.isChecked = this.element.attr('checked') == 'checked' ? true : false;
            this.isChecked = this.model.check == null ? this.isChecked : this.model.check;
        },
        /**
         * To configure the Size of the RadioButton		
		 * @private
         */
        _setSize: function (val) {
            if (val == ej.radiobuttonSize.medium)
                this.span.removeClass('e-radsmaller').addClass('e-radmedium');
            else
                this.span.removeClass('e-radmedium').addClass('e-radsmaller');
        },
        /**
         * To configure the Enable status of the RadioButton		
		 * @private
         */
        _setEnabled: function (val) {
            if (val) {
                this.enable();
            } else {
                this.disable();
            }
        },
        /**
         * Render Section For RadioButton control		
		 * @private
         */
        _renderControl: function () {
            this.div = $('<div class="e-radiobtn-wrap e-widget" ></div>');
            this.div.attr({ 'id': this.model.idPrefix + this.model.id, "role": "radio", "tabindex": -1, "aria-checked": false });
            this.span = $('<span></span>');
            this.span.addClass("e-spanicon");
            this._setSize(this.model.size);
            this.spanImg = $('<span class="e-rad-icon e-rad-select"></span>', "", {}, { "role": "presentation" });
            this.element.addClass("e-input");
            this.element.attr("id", this.model.id);
            this.div.addClass(this.model.cssClass);
            this.span.append(this.spanImg);
            this.div.insertBefore(this.element);
            this.div.append(this.element);
            this.div.append(this.span);
            this._setTextWrapper(this.model.text);
            this.radbtn = this.element;
            this.element = this.div;
            if (this.isChecked) {
                this.element.find(".e-input").attr('checked', true);
            }
        },
        /**
         * To configure RadioButton textwrapper		
		 * @private
         */
        _setTextWrapper: function (val) {
            if (val != "") {
                this.textWrapDiv = ej.buildTag("div.e-radiobtn-wrap " + this.model.cssClass + "#" + this.model.idPrefix + this.model.id + "_wrapper")
                this.div.wrap(this.textWrapDiv);
                this.txtSpan = ej.buildTag("div.e-text", this.model.text);
                this.textWrapDiv.append(this.txtSpan);
                if (this.model.rtl)
                    this.textWrapDiv.addClass("e-rtl");
            }
        },
        /**
         * To configure RadioButton text value		
		 * @private
         */
        _setText: function (val) {
            if ((this.model.text == "") && (val != "")) {
                this._setTextWrapper(val);
            } else {
                this.txtSpan.html(val);
            }
        },
        /**
        * Wiring the events to RadioButton control		
        * @private
        */
        _wireEvents: function () {
            this._on(this.element, "click", this._checkedHandler);
            this._on(this.element, "focus", this._focusIn);
            this._on(this.element, "focusout", this._focusOut);
        },
        _focusIn: function (evt) {
            $(this.element).addClass("e-focus");
        },
        _focusOut: function (evt) {
            $(this.element).removeClass("e-focus");
        },
        _checkedHandler: function (evt) {
            if (!this.element.hasClass('e-disable')) {
                this.isChecked = this.element.find('input.e-radiobtn:radio').attr('checked') == 'checked' ? true : false;
                this._changeEvent();
            }
        },
        /**
         * Section For handle the change in check for radiobutton
		 * @private
         */
        _checkedChange: function (val) {
            this.isChecked = val;
            if ((this.isChecked) && (this.model.enabled))
                this._changeEvent();
        },
        /**
         * Section For handle the change event for radiobutton
		 * @private
         */
        _changeEvent: function () {
            var data = { isChecked: this.isChecked };
            if (true == this._trigger("beforeChange", data)) {
                return false;
            }
            if (!$(this.element).find(".e-rad-icon").hasClass("e-rad-active")) {
                var curname = this.element.find(".e-input").attr('name'),
                input = $('input.e-radiobtn[name=' + curname + ']:radio'),
                proxy = this,
                currElement = this.element.find('.e-input'),
                currObj = $(currElement).data("ejRadioButton");
                if (data.isChecked) {
                    this.spanImg.addClass("e-rad-active").removeClass('e-rad-select');
                    this.div.attr({ "tabindex": 0, "aria-checked": true });
                }
                $.each(input, function (i, obj) {
                    $(obj).parent().parent().find(".e-rad-icon").removeClass("e-rad-active").addClass("e-rad-select");
                    $(obj).parent().parent().find('.e-radiobtn-wrap').attr({ "tabindex": -1, "aria-checked": false });
                    var prevObj = $(obj).data("ejRadioButton");
                    if (prevObj != null) {
                        prevObj.model.check = false;
                    }
                });
                if (currObj != null)
                    currObj.model.check = true;
                this.element.find(".e-rad-icon").addClass("e-rad-active").removeClass("e-rad-select");
                this.div.attr({ "tabindex": 0, "aria-checked": true });
                this.element.find(".e-input").click();
                this.isChecked = true;
            }
            var data = { isChecked: this.isChecked };
            this._trigger("change", data)
        },
        /**
        * To disable the RadioButton
		* @return jQuery
		* @example 
		* &lt;input type="radio" id="chkbox"&gt;RadioButton&lt;/radio&gt<br>
		* &lt;script&gt; 
		* // Create RadioButton
		* var chkObj = $("#radiobtn").data("ejRadioButton");
		* chkObj.disable();
		* &lt;/script&gt;
		* @example 
		* &lt;input type="radio" id="chkbox"&gt;RadioButton&lt;/radio&gt<br>
		* &lt;script&gt;
		* // disable the RadioButton
		* $("#radiobtn").ejRadioButton("disable");	
		* &lt;/script&gt;
		* @memberof ejRadioButton
		* @instance
        */
        disable: function () {
            if (!this.element.hasClass("e-disable"))
                this.element.addClass("e-disable");
            this.div.attr("aria-disabled", true);
            this.model.enabled = false;
        },
        /**
        * To enable the RadioButton
		* @return jQuery
		* @example 
		* &lt;input type="radio" id="chkbox"&gt;RadioButton&lt;/radio&gt<br>
		* &lt;script&gt; 
		* // Create RadioButton
		* var chkObj = $("#radiobtn").data("ejRadioButton");
		* chkObj.enable();
		* &lt;/script&gt;
		* @example 
		* &lt;input type="radio" id="chkbox"&gt;RadioButton&lt;/radio&gt<br>
		* &lt;script&gt;
		* // enable the RadioButton
		* $("#radiobtn").ejRadioButton("enable");	
		* &lt;/script&gt;
		* @memberof ejRadioButton
		* @instance
        */
        enable: function () {
            if (this.element.hasClass("e-disable"))
                this.element.removeClass("e-disable");
            this.div.attr("aria-disabled", false);
            this.model.enabled = true;
        }
    });
    ej.radiobuttonSize = { small: "small", medium: "medium" };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejAutocomplete
	* @requires jQuery
	* @requires ej.common.all
	* @classdesc Custom Design for Html Textbox control.
	* @example 
	* &lt;input type="text" id="autocomplete" /&gt; <br> 
	* &lt;script&gt;
	* // Create AutoComplete
    * $('#autocomplete').ejAutocomplete(); 	
	* &lt;/script&gt;
	*/

    // ejAutocomplete is the plugin name 
    // "ej.Autocomplete" is "namespace.className" will hold functions and properties

    ej.widget("ejAutocomplete", "ej.Autocomplete", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-autocomplete",
        type: "editor",

        // default model
        defaults: {
            /**		
			* Specifies the Data Source of the AutoComplete.	
			* @default null
			* @type {data}
			* @example 
			* //To set dataSource API value during initialization  
			* 	$("#autocomplete").ejAutocomplete({ dataSource: countries });				* 
			* @example 
			* //Get or set the dataSource API, after initialization:<br>
			*	//Gets the dataSource value  
			*	$("#autocomplete").ejAutocomplete('option', 'dataSource');<br>			
			*	//Sets the dataSource value 
			*	$("#autocomplete").ejAutocomplete('option', 'dataSource', countries );
			* @memberof ejAutocomplete
			* @instance
			*/
            dataSource: null,
            /**		
             * Specifies the query to retrieve the data from online server.	
             * @default null
             * @type {object}
             * @example 
             * //To set query API value during initialization  
             * var dataManger = ej.DataManager({       url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"});
             * var queryString = ej.Query().from("Suppliers").select("ContactName");
             * 	$("#autocomplete").ejAutocomplete({ dataSource: dataManger, query: queryString, fields: { text: "ContactName" }});
             * @example 
             * //Get or set the query API, after initialization:<br>
             *	//Gets the dataSource value  
             *	$("#autocomplete").ejAutocomplete('option',  'query');<br>			
             *	//Sets the query value 
             *	$("#autocomplete").ejAutocomplete('option',  'query',  queryString );
             * @memberof ejAutocomplete
             * @instance
             */
            query: null,
            /**		
             * Specifies mapping fields for the data items of the Autocomplete textbox.	
             * @default null
             * @type {object}
             * @example 
             * //To set fields API value during initialization  
             * var countries = [{name: "Austria", key: "flag-dz"}, {name: "Argentina, key: "flag-ar"},{name: "Antarctica", key: "flag-am", htmlAttr: {"style": "border: 1px solid;"}} ];
             * 	$("#autocomplete").ejAutocomplete({ dataSource: countries,       fields: { text: "name", uniqueKey: "key" }});
             * @example 
             * //Get or set the fields  API, after initialization:<br>
             *	//Gets the fields  value  
             *	$("#autocomplete").ejAutocomplete('option',  'fields' );<br>			
             *	//Sets the fields value 
             *	$("#autocomplete").ejAutocomplete('option', 'fields', { text: "name", uniqueKey: "key"} );
             * @memberof ejAutocomplete
             * @instance
             */
            fields: null,
            /**		
			* Specifies the template for Autocomplete.
			* @default null
			* @type {string}
			* @example 
			* // To set template API value during initialization  . 	
            * var countries = [	{ text: "Algeria", icon: "dz.png" },	{ text: "Argentina", icon: "ar.png" },	{ text: "Armenia", icon: "am.png" },	{ text: "Brazil", icon: "br.png" }];
			* 	$("#autocomplete").ejAutocomplete({ dataSource: countries, template: '<img src="../images/${icon}" alt="/${text}">'});
			* @example 
			* //Get or set the template , after initialization:<br>
			*	// Gets the template  value.		
			*	$("#autocomplete").ejAutocomplete('option',  'template');
			*	// Sets the template value.
			*	$("#autocomplete").ejAutocomplete('option', 'template', '<img src="../${icon}">');
			* @memberof ejAutocomplete
			* @instance
			*/
            template: null,
            /**		
			* Groups the search result based on the category value.
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the Autocomplete with the grouping value specified.
            * var vehicle = [{ text: "BMW 7", category: "Car" },{ text: "Ferrari", category: "Car" },{ text: "Lamborghini", category: "Car" },{ text: "Hodaka", category: "Bike" },{ text: "Cycle-Scoot", category: "Bike" }];
			* $("#autocomplete").ejAutocomplete({ dataSource: vehicle,	grouping: true});
			* @example 
			* //Get or set the grouping API, after initialization:<br>
			*	// Get the grouping API value.		
			*	$("#autocomplete").ejAutocomplete('option', 'grouping');			
			*	// Set the grouping API
			*	$("#autocomplete").ejAutocomplete('option', 'grouping',  true);
			* @memberof ejAutocomplete
			* @instance
			*/
            grouping: false,
            /**		
            * Prevents the duplicate names presents in the search result.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the distinct value specified.
            * $("#autocomplete").ejAutocomplete({ distinct: true });
            * @example 
            * //Get or set the distinct API, after initialization:<br>
            *	// Get the distinct API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'distinct');			
            *	// Set the distinct API
            *	$("#autocomplete").ejAutocomplete('option', 'distinct',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            distinct: false,
            /**		
            * Sorts the lists value in ascending order if set to true.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the allowSorting value specified.
            * $("#autocomplete").ejAutocomplete({allowSorting: true });
            * @example 
            * //Get or set the allowSorting API, after initialization:<br>
            *	// Get the allowSorting API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'allowSorting');			
            *	// Set the allowSorting API
            *	$("#autocomplete").ejAutocomplete('option', 'allowSorting',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            allowSorting: true,
            /**		
            * Sort order specifies whether the suggestion list values has to display in ascending or descending order. 	See {@link SortOrder mode}
            * @default ej.sortOrder.Ascending
            * @type {enum}
            * @example 
            * //Initialize the Autocomplete with the sortOrder value specified
            * 	$("#autocomplete").ejAutocomplete({sortOrder: ascending });					* 
            * @example 
            * //Get or set the sortOrder API, after initialization:<br>
            *	//Gets the sortOrder value  
            *	$("#autocomplete").ejAutocomplete("option", "sortOrder");<br>			
            *	//Sets the sortOrder value 
            *	$("#autocomplete").ejAutocomplete("option", "sortOrder",  ej.sortOrder.Ascending ); 
            * @memberof ejAutocomplete
            * @instance
            */
            sortOrder: "ascending",
            /**		
            * Allows to select multiple values from the suggestion list. Multiple values can be selected through either of the following options,Delimiter-Multiple values separated using comma.Visual mode- Each values are displayed in separate box with close button. 	See {@link selection mode}
            * @default ej.multiSelectMode.None
            * @type {enum}
            * @example 
            * //Initialize the Autocomplete with the multiSelectMode value specified
            * 	$("#autocomplete").ejAutocomplete({multiSelectMode: ej.multiSelectMode.Delimiter  });					* 
            * @example 
            * //Get or set the multiSelectMode API, after initialization:<br>
            *	//Gets the multiSelectMode value  
            *	$("#autocomplete").ejAutocomplete("option", "multiSelectMode");<br>			
            *	//Sets the multiSelectMode value 
            *	$("#autocomplete").ejAutocomplete("option", "multiSelectMode",  ej.multiSelectMode.Delimiter ); 
            * @memberof ejAutocomplete
            * @instance
            */
            multiSelectMode: "none",
            /**		
            * Sets the separator to allow multiple word searches. While typing the texts in the text box, if we enter the delimiter value, the texts after the delimiter are considered as a separate word or query. The delimiter string should have a single character and must be a symbol. Mostly the delimiter symbol is used as (comma ,) or (semi-colon ;) or any other special character.
            * @default ';'
            * @type {string}
            * @example 
            * //Initialize the Autocomplete with the delimiterChar value specified
            * 	$("#autocomplete").ejAutocomplete({delimiterChar: ';' });
            * @example 
            * Get or set the delimiterChar API, after initialization:<br>
            *	//Gets the delimiterChar  value
            *	$("#autocomplete").ejAutocomplete("option", "delimiterChar ");			
            *	//Sets the delimiterChar  value
            *	$("#autocomplete").ejAutocomplete("option", "delimiterChar ", 30 );
            * @memberof ejAutocomplete
            * @instance
            */
            delimiterChar: ',',
            /**		
            * Specifies new values can be added to the autocomplete input other than the values in the suggestion list.
            * @default false
            * @type {boolean}
            * @example 
            * //Initialize the Autocomplete with the allowNew value specified.
            * $("#autocomplete").ejAutocomplete({allowNew: false });
            * @example 
            * //Get or set the alloeNew API, after initialization:<br>
            *	// Get the allowNew API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'allowNew');			
            *	// Set the allowNew API
            *	$("#autocomplete").ejAutocomplete('option', 'allowNew',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            allowNew: false,
            /**		
            * Allows new text to be added to in the dropdown list when there are no suggestions. This property can only be used in the Visual mode only.
            * @default "Add New"
            * @type {string}
            * @example 
            * //Initialize the Autocomplete with the addNewText value specified
            * 	$("#autocomplete").ejAutocomplete({addNewText: "Add New" });
            * @example 
            * Get or set the addNewText API, after initialization:<br>
            *	//Gets the addNewText  value
            *	$("#autocomplete").ejAutocomplete("option", "addNewText ");			
            *	//Sets the addNewText  value
            *	$("#autocomplete").ejAutocomplete("option", "addNewText ", "Add New" );
            * @memberof ejAutocomplete
            * @instance
            */
            addNewText: "Add New",
            /**		
            * Autocomplete textbox to be displayed with rounded corner style.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the roundedCorner value specified.
            * $("#autocomplete").ejAutocomplete({roundedCorner: true });
            * @example 
            * //Get or set the roundedCorner API, after initialization:<br>
            *	// Get the roundedCorner API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'roundedCorner');			
            *	// Set the roundedCorner API
            *	$("#autocomplete").ejAutocomplete('option', 'roundedCorner',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            roundedCorner: false,
            /**		
            * Indicates that the autocomplete textbox values can only be read.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the readOnly value specified.
            * $("#autocomplete").ejAutocomplete({readOnly: true });
            * @example 
            * //Get or set the readOnly API, after initialization:<br>
            *	// Get the readOnly API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'readOnly');			
            *	// Set the readOnly API
            *	$("#autocomplete").ejAutocomplete('option', 'readOnly',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            readOnly: false,
            /**		
            * Sets the root class for Autocomplete theme. This cssClass API helps to use custom skinning option for Autocomplete control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
            * //Initialize the Autocomplete with the cssClass value specified
            * 	$("#autocomplete").ejAutocomplete({ cssClass: 'gradient-lime'});
            * @example 
            * Get or set the cssClass API, after initialization:<br>
            *	//Gets the cssClass  value
            *	$("#autocomplete").ejAutocomplete("option", "cssClass ");			
            *	//Sets the cssClass  value
            *	$("#autocomplete").ejAutocomplete("option", "cssClass ", "gradient-lime" );
            * @memberof ejAutocomplete
            * @instance
            */
            cssClass: "",
            /**		
            * Sets the watermark text. When the textbox is empty the watermark text is visible like a shaded text. 
            * @default "Enter text"
            * @type {string}
            * @example 
            * //Initialize the Autocomplete with the watermark value specified
            * 	$("#autocomplete").ejAutocomplete({ watermark: 'Enter text' });
            * @example 
            * Get or set the watermark API, after initialization:<br>
            *	//Gets the watermark  value
            *	$("#autocomplete").ejAutocomplete("option", "watermark");			
            *	//Sets the watermark  value
            *	$("#autocomplete").ejAutocomplete("option", "watermark ", "Enter text" );
            * @memberof ejAutocomplete
            * @instance
            */
            watermark: null,
            /**		
            * Defines the default value to be display in the autocomplete textbox.
            * @default Null
            * @type {string}
            * @example 
            * //Initialize the Autocomplete value property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ value: “audi” });
            * @example 
            * Get or set the value API, after initialization:<br>
            *	//Gets the value  value
            *	$("#autocomplete").ejAutocomplete("option", "value");			
            *	//Sets the value  value
            *	$("#autocomplete").ejAutocomplete("option", "value ", "audi" );
            * @memberof ejAutocomplete
            * @instance
            */
            value: "",
            /**		
            * Sets the search filter type. There are several filter types are available such as ‘startswith’, ‘contains’, ‘endswith’, ‘lessthan’, ‘lessthanorequal’, ‘greaterthan’, ‘greaterthanorequal’, ‘equal’, ‘notequal’. 	See {@link Filter mode}
            * @default ej.filter.StartsWith
            * @type {enum}
            * @example 
            * //Initialize the Autocomplete with the filter value specified
            * 	$("#autocomplete").ejAutocomplete({filter: 'contains'  });					* 
            * @example 
            * //Get or set the filter API, after initialization:<br>
            *	//Gets the filter value  
            *	$("#autocomplete").ejAutocomplete("option", "filter");<br>			
            *	//Sets the filter value 
            *	$("#autocomplete").ejAutocomplete("option", "filter",  'contains' ); 
            * @memberof ejAutocomplete
            * @instance
            */
            filter: "startswith",
            /**		
            * Sets the case sensitivity of the search operation..
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the caseSensitive value specified.
            * $("#autocomplete").ejAutocomplete({ caseSensitive: true });
            * @example 
            * //Get or set the caseSensitive API, after initialization:<br>
            *	// Get the caseSensitive API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'caseSensitive');			
            *	// Set the caseSensitive API
            *	$("#autocomplete").ejAutocomplete('option', 'caseSensitive',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            caseSensitive: false,
            /**		
            * Enables the loading icon to intimate the searching operation. The loading icon is visible when there is a time delay to perform the search.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the loadingImage value specified.
            * $("#autocomplete").ejAutocomplete({ loadingImage: false });
            *  @example 
            * //Get or set the loadingImage API, after initialization:<br>
            *	// Get the loadingImage API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'loadingImage');			
            *	// Set the loadingImage API
            *	$("#autocomplete").ejAutocomplete('option', 'loadingImage',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            loadingImage: true,
            listSize: 0,
            minCharacter: 1,
            /**		
           * Enables the dropdown button. When the dropdown button clicks, it displays the full list from the dataSource.
           * @default false
           * @type {boolean}
           * @example 
           * // Initialize the Autocomplete with the dropdown  value specified.
           * $("#autocomplete").ejAutocomplete({ dropdown : false });
           * @example 
           * //Get or set the dropdown  API, after initialization:<br>
           *	// Get the loadingImage API value.		
           *	$("#autocomplete").ejAutocomplete('option', 'dropdown');			
           *	// Set the dropdown  API
           *	$("#autocomplete").ejAutocomplete('option', 'dropdown',  true);
           * @memberof ejAutocomplete
           * @instance
           */
            dropdown: false,
            /**		
            * Enables the highlight search option. When the highlightSearch option set to true, the corresponding string entered in the textbox is highlighted in the suggestion list.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the highlightSearch  value specified.
            * $("#autocomplete").ejAutocomplete({ highlightSearch : false });
            * @example 
            * //Get or set the highlightSearch  API, after initialization:<br>
            *	// Get the highlightSearch API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'highlightSearch');			
            *	// Set the highlightSearch  API
            *	$("#autocomplete").ejAutocomplete('option', 'highlightSearch',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            highlightSearch: false,
            /**		
            * Automatically fills the first item from the suggestion list in an AutoComplete text box. The autoFill property is only applicable for “startswith” filter type.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the autoFill  value specified.
            * $("#autocomplete").ejAutocomplete({ autoFill : false });
            * @example 
            * //Get or set the autoFill  API, after initialization:<br>
            *	// Get the autoFill API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'autoFill');			
            *	// Set the autoFill  API
            *	$("#autocomplete").ejAutocomplete('option', 'autoFill',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            autoFill: false,
            /**		
            * Sets the Autocomplete textbox direction as right to left alignment.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the rtl  value specified.
            * $("#autocomplete").ejAutocomplete({ rtl : false });
            * @example 
            * //Get or set the rtl  API, after initialization:<br>
            *	// Get the rtl API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'rtl');			
            *	// Set the rtl  API
            *	$("#autocomplete").ejAutocomplete('option', 'rtl',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            rtl: false,
            /**		
            * When this property sets to false, it disables the Autocomplete control.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the enabled  value specified.
            * $("#autocomplete").ejAutocomplete({ enabled : false });
            * @example 
            * //Get or set the enabled  API, after initialization:<br>
            *	// Get the enabled API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'enabled');			
            *	// Set the enabled  API
            *	$("#autocomplete").ejAutocomplete('option', 'enabled',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            enabled: true,
            /**		
            * Defines the height of the Autocomplete textbox.
            * @default Null
            * @type {string | number}
            * @example 
            * //Initialize the Autocomplete height property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ height: 30 });
            * @example 
            * Get or set the height API, after initialization:<br>
            *	//Gets the height  value
            *	$("#autocomplete").ejAutocomplete("option", "height");			
            *	//Sets the height  value
            *	$("#autocomplete").ejAutocomplete("option", "height ", 30 );
            * @memberof ejAutocomplete
            * @instance
            */
            height: "",
            /**		
            * Defines the width of the Autocomplete textbox.
            * @default Null
            * @type {string | number}
            * @example 
            * //Initialize the Autocomplete width property with the width value specified
            * 	$("#autocomplete").ejAutocomplete({ width: 30 });
            * @example 
            * Get or set the width API, after initialization:<br>
            *	//Gets the width  value
            *	$("#autocomplete").ejAutocomplete("option", "width");			
            *	//Sets the width  value
            *	$("#autocomplete").ejAutocomplete("option", "width ", 30 );
            * @memberof ejAutocomplete
            * @instance
            */
            width: "",
            /**		
            * Sets the noResults message text. When there is no suggestions are available at the time this 	message will be shown.
            * @default "No suggestions"
            * @type {string | number}
            * @example 
            * //Initialize the Autocomplete noResults property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ noResults: 'No Results Found' });
            * @example 
            * Get or set the noResults API, after initialization:<br>
            *	//Gets the noResults  value
            *	$("#autocomplete").ejAutocomplete("option", "noResults");			
            *	//Sets the noResults  value
            *	$("#autocomplete").ejAutocomplete("option", "noResults ", 'No Results Found' );
            * @memberof ejAutocomplete
            * @instance
            */
            noResults: "No suggestions",
            /**		
            * Sets whether the noResults message will be shown or not.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the showNoResults  value specified.
            * $("#autocomplete").ejAutocomplete({ showNoResults : false });
            * @example 
            * //Get or set the showNoResults  API, after initialization:<br>
            *	// Get the showNoResults API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'showNoResults');			
            *	// Set the showNoResults  API
            *	$("#autocomplete").ejAutocomplete('option', 'showNoResults',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            showNoResults: true,
            /**		
            * Save current model value to browser cookies for state maintains. While refresh the Autocomplete control page retains the model value apply from browser cookies.  
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the Autocomplete with the persist  value specified.
            * $("#autocomplete").ejAutocomplete({ persist : false });
            * @example 
            * //Get or set the persist  API, after initialization:<br>
            *	// Get the persist API value.		
            *	$("#autocomplete").ejAutocomplete('option', 'persist');			
            *	// Set the persist  API
            *	$("#autocomplete").ejAutocomplete('option', 'persist',  true);
            * @memberof ejAutocomplete
            * @instance
            */
            persist: false,
            /**		
            * Defines the suggestionBoxHeight of the suggestion box.
            * @default "152px"
            * @type {string | number}
            * @example 
            * //Initialize the Autocomplete suggestionBoxHeight property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ suggestionBoxHeight: '152px' });
            * @example 
            * Get or set the suggestionBoxHeight API, after initialization:<br>
            *	//Gets the suggestionBoxHeight  value
            *	$("#autocomplete").ejAutocomplete("option", "suggestionBoxHeight");			
            *	//Sets the suggestionBoxHeight  value
            *	$("#autocomplete").ejAutocomplete("option", "suggestionBoxHeight ", '152px' );
            * @memberof ejAutocomplete
            * @instance
            */
            suggestionBoxHeight: "152px",
            /**		
            * Defines the suggestionBoxWidth of the suggestion box.
            * @default "auto"
            * @type {string | number}
            * @example 
            * //Initialize the Autocomplete suggestionBoxWidth property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ suggestionBoxWidth: '152px' });
            * @example 
            * Get or set the suggestionBoxWidth API, after initialization:<br>
            *	//Gets the suggestionBoxWidth  value
            *	$("#autocomplete").ejAutocomplete("option", "suggestionBoxWidth");			
            *	//Sets the suggestionBoxWidth  value
            *	$("#autocomplete").ejAutocomplete("option", "suggestionBoxWidth ", '152px' );
            * @memberof ejAutocomplete
            * @instance
            */
            suggestionBoxWidth: "auto",
            /**     
			* Fires when focusIn successfully.
			* @event
			* @name ejAutocomplete#focusIn 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the autocomplete model
			* @param {string}  argument.type returns the name of the event
			* @example 
			* //focusIn event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	focusIn: function(args) {
            * });      
			* @memberof ejAutocomplete
			* @instance
			*/
            focusIn: null,
            /**     
            * Fires when focusOut successfully.
            * @event
            * @name ejAutocomplete#focusOut 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //focusOut event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	focusOut: function(args) {
            * });      
            * @memberof ejAutocomplete
            * @instance
            */
            focusOut: null,
            /**     
            * Fires when change successfully.
            * @event
            * @name ejAutocomplete#change 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //change event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	change: function(args) {
            * });      
            * @memberof ejAutocomplete
            * @instance
            */
            change: null,
            /**     
            * Fires when select successfully.
            * @event
            * @name ejAutocomplete#focusOut 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //select event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	select: function(args) {
            * });      
            * @memberof ejAutocomplete
            * @instance
            */
            select: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            filter: "enum",
            caseSensitive: "boolean",
            loadingImage: "boolean",
            template: "string",
            grouping: "boolean",
            distinct: "boolean",
            allowSorting: "boolean",
            sortOrder: "enum",
            allowNew: "boolean",
            addNewText: "string",
            roundedCorner: "boolean",
            readOnly: "boolean",
            listSize: "number",
            minCharacter: "number",
            dropdown: "boolean",
            highlightSearch: "boolean",
            autoFill: "boolean",
            rtl: "boolean",
            multiSelectMode: "enum",
            delimiterChar: "string",
            noResults: "string",
            showNoResults: "boolean",
            enabled: "boolean",
            persist: "boolean",
            dataSource: "data",
            fields: "data"
        },
        observables: ["value"],
        /**
       * To enable the autocomplete  
       * @return jQuery
       * @example 
       * &lt;input type="text" id="autocomplete" /&gt; <br> 
       * &lt;script&gt;
       * // Create autocomplete
       * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
       * autocompleteObj.enable(); // enable the autocomplete
       * &lt;/script&gt;
       * @example 
       * &lt;input type="text" id="autocomplete" /&gt; <br> 
	   * &lt;script&gt;
	   * // Create autocomplete
       * $('#autocomplete').ejAutocomplete(); 	
	   * &lt;/script&gt;
       *@memberof ejAutocomplete
       * @instance
       */
        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this.target.disabled = false;
                this.element.removeClass("e-disable").attr({ "aria-disabled": false });
                if (this.model.dropdown) this.dropdownbutton.removeClass("e-disable").attr({ "aria-disabled": false });
                if (this.model.multiSelectMode == "visualmode") this._ulBox.removeClass("e-disable").attr({ "aria-disabled": false });
            }
        },
        /**
      * To disable the autocomplete  
      * @return jQuery
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
      * // Create autocomplete
      * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
      * autocompleteObj.disable(); // disable the autocomplete
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
      * // Create autocomplete
      * $('#autocomplete').ejAutocomplete(); 	
      * &lt;/script&gt;
      *@memberof ejAutocomplete
      * @instance
      */
        disable: function () {
            if (this.model.enabled) {
                this._hideResult();
                this.element.attr("aria-expanded", false);
                this.model.enabled = false;
                this.target.disabled = true;
                this.element.addClass("e-disable").attr({ "aria-disabled": true });
                if (this.model.dropdown) this.dropdownbutton.addClass("e-disable").attr({ "aria-disabled": true });
                if (this.model.multiSelectMode == "visualmode") this._ulBox.addClass("e-disable").attr({ "aria-disabled": true });
            }
        },
        /**
      * Clears the text in the Autocomplete textbox.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
      * // Create autocomplete
      * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
      * autocompleteObj.clearText(); // clear the autocomplete text
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
      * // Create autocomplete
      * $('#autocomplete').ejAutocomplete(); 	
      * &lt;/script&gt;
      *@memberof ejAutocomplete
      * @instance
      */
        clearText: function () {
            if (this.model.enabled) {
                if (this.model.multiSelectMode == "visualmode")
                    this._deleteBox(this._ulBox.children("li"));
                this.element.val("");
                this._valueChange();
                if (this._isFocused) this.element.blur();
                else this._targetBlur();
            }
        },
        /**
      * Returns the current value selected in the Autocomplete textbox.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
      * // Create autocomplete
      * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
      * autocompleteObj.getValue(); // getValue of the autocomplete text
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
      * // Create autocomplete
      * $('#autocomplete').ejAutocomplete(); 	
      * &lt;/script&gt;
      *@memberof ejAutocomplete
      * @instance
      */
        getValue: function () {
            if (this.model.multiSelectMode == "visualmode")
                return this._hiddenInput.val();
            if (this.element.hasClass("e-watermark"))
                return "";
            return this.target.value;
        },
        /**
     * Returns the values selected in the Autocomplete textbox.
     * @return jQuery
     * @example 
     * &lt;input type="text" id="autocomplete" /&gt; <br> 
     * &lt;script&gt;
     * // Create autocomplete
     * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
     * autocompleteObj.getSelectedItems(); // getSelectedItems the autocomplete text
     * &lt;/script&gt;
     * @example 
     * &lt;input type="text" id="autocomplete" /&gt; <br> 
     * &lt;script&gt;
     * // Create autocomplete
     * $('#autocomplete').ejAutocomplete(); 	
     * &lt;/script&gt;
     *@memberof ejAutocomplete
     * @instance
     */
        getSelectedItems: function () {
            return this._selectedItems;
        },
        _setValue: function (value) {
            if (this.element.hasClass("e-watermark"))
                this.element.removeClass("e-watermark")
            if (typeof value === "object" || (typeof value === "number" && isNaN(value))) value = "";
            value = $.trim(value);
            this.element.val("");
            if (value) {
                if (this.model.multiSelectMode == "visualmode") {
                    this._selectedItems = [];
                    this._deleteBox(this._ulBox.children("li"));
                    this._hiddenInput.val(value);
                    var values = value.split(this.model.delimiterChar);
                    for (var i = 0; i < values.length; i++) {
                        if (values[i]) {
                            this._ulBox.append(this._createBox(values[i]));
                            this._selectedItems.push(values[i]);
                        }
                    }
                }
                else {
                    this.element.val(value);
                    this._updateSelectedItemArray();
                }
            }
            this._preVal = this.element.val();
            !this._isWatermark && this._setWatermarkTxt();
            return value;
        },
        /**
    * search values in the Autocomplete textbox.
    * @return jQuery
    * @example 
    * &lt;input type="text" id="autocomplete" /&gt; <br> 
    * &lt;script&gt;
    * // Create autocomplete
    * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
    * autocompleteObj.search(); // search the autocomplete text
    * &lt;/script&gt;
    * @example 
    * &lt;input type="text" id="autocomplete" /&gt; <br> 
    * &lt;script&gt;
    * // Create autocomplete
    * $('#autocomplete').ejAutocomplete(); 	
    * &lt;/script&gt;
    *@memberof ejAutocomplete
    * @instance
    */
        search: function () {
            if (this.model.enabled && this._checkDelimiter()) {
                this._hideResult();
                this.element.attr("aria-expanded", false);
                this._autoFill = false;
                this._queryString = $.trim(this._queryString);
                if (this._queryString.length > 0) this._OnTextEnter();
            }
        },
        /**
   * hides in the Autocomplete textbox.
   * @return jQuery
   * @example 
   * &lt;input type="text" id="autocomplete" /&gt; <br> 
   * &lt;script&gt;
   * // Create autocomplete
   * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
   * autocompleteObj.hide(); // hide the autocomplete text
   * &lt;/script&gt;
   * @example 
   * &lt;input type="text" id="autocomplete" /&gt; <br> 
   * &lt;script&gt;
   * // Create autocomplete
   * $('#autocomplete').ejAutocomplete(); 	
   * &lt;/script&gt;
   *@memberof ejAutocomplete
   * @instance
   */
        hide: function () {
            this._hideResult();
            this.element.attr("aria-expanded", false);
        },

        _changeWatermark: function (text) {
            if (this._isWatermark) this.element.attr("placeholder", text);
            else if (this.element.hasClass("e-watermark")) this.element.val(text);
        },
        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            this.suggestionList.removeClass(this.model.cssClass).addClass(skin);
        },
        _setDropdown: function (boolean) {
            this.model.dropdown = boolean;
            if (boolean) this._renderDropdown();
            else this._destroyDropdown();
        },
        _changeHeight: function (height) {
            this.wrapper.height(height);
        },
        _changeWidth: function (width) {
            this.wrapper.width(width);
            this._setListWidth();
        },

        // constructor function
        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            this._initialize();
            this._render();
            this._wireEvents();
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "dataSource": this.model.dataSource = null; break;
                    case "watermark": this._changeWatermark(options[option]); break;
                    case "value":
                        options[option] = this._setValue(options[option]);
                        this._valueChange();
                        break;
                    case "dropdown": this._setDropdown(options[option]); break;
                    case "rtl": this._RightToLeft(options[option]); break;
                    case "roundedCorner": this._setRoundedCorner(options[option]); break;
                    case "readOnly": this.model.readOnly = options[option]; this._checkReadOnly(); break;
                    case "delimiterChar": options[option] = this._validateDelimiter(options[option]); break;
                    case "multiSelectMode":
                        this.model.multiSelectMode = options[option];
                        if (options[option] == "visualmode") this._renderBoxModel();
                        else if (this.element.hasClass("e-visual-mode")) this._destroyBoxModel();
                        this._setValue(this.model.value);
                        break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "height": this._changeHeight(options[option]); break;
                    case "width": this._changeWidth(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "listSize": if (options[option] <= 0 || isNaN(options[option])) options[option] = 0; break;
                    case "suggestionBoxHeight": this.suggestionList.css({ "max-height": options[option] }); break;
                    case "suggestionBoxWidth": this.model.suggestionBoxWidth = options[option]; this._setListWidth(); break;
                }
            }
        },
        /**
 * destroy in the Autocomplete textbox.
 * @return jQuery
 * @example 
 * &lt;input type="text" id="autocomplete" /&gt; <br> 
 * &lt;script&gt;
 * // Create autocomplete
 * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
 * autocompleteObj.destroy(); // hide the autocomplete text
 * &lt;/script&gt;
 * @example 
 * &lt;input type="text" id="autocomplete" /&gt; <br> 
 * &lt;script&gt;
 * // Create autocomplete
 * $('#autocomplete').ejAutocomplete(); 	
 * &lt;/script&gt;
 *@memberof ejAutocomplete
 * @instance
 */
        _destroy: function () {
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            this.element.removeClass("e-input e-watermark").val("");
            if (this._isWatermark) this.element.removeAttr("placeholder");
            this.suggestionList.remove();
        },

        _initialize: function () {
            this.element.attr("role", "combobox").attr("aria-label", "Autocomplete textbox").attr("aria-expanded", false).attr("tabindex", 0).attr("aria-autocomplete", "list");
            this.target = this.element[0];
            this.dropdownbutton = null;
            this.showSuggestionBox = false;
            this.noresult = true;
            this._queryString = null;
            this.suggLen = 0;
            this._selectedItems = [];

            this._activeItem = 0;
            this.ctrlKeyPressed = false;
            this._isFocused = false;
            this._isOpened = false;
            this._isWatermark = this._checkWatermarkSupport();
        },

        _render: function () {
            this._renderWrapper();
            this._setDimentions();
            this._renderDropdown();
            this._checkProperties();
            this._setWatermark();
            this._renderSuggestionList();
            this._RightToLeft(this.model.rtl);
            this._setRoundedCorner(this.model.roundedCorner);
        },

        _renderWrapper: function () {
            this.element.addClass("e-input");
            this.wrapper = ej.buildTag("span.e-atc e-widget " + this.model.cssClass + "#" + this.target.id + "_wrapper").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
        },

        _renderDropdown: function () {
            if (this.model.dropdown) {
                var span = ej.buildTag("span.e-icon e-search");
                this.dropdownbutton = ej.buildTag("span.e-select#" + this.target.id + "_dropdown").append(span);
                this.container.append(this.dropdownbutton).addClass("e-padding");
                this.dropdownbutton.bind("mousedown", $.proxy(this._OnDropdownClick, this));
            }
        },

        _setDimentions: function () {
            if (this.model.height)
                this.wrapper.height(this.model.height);
            if (this.model.width)
                this.wrapper.width(this.model.width);
        },

        _renderBoxModel: function () {
            this._ulBox = ej.buildTag("ul.e-ul e-boxes");
            this._hiddenInput = ej.buildTag("input").attr({ type: "hidden", name: this.element.attr("name") }).css("display", "none");
            this.element.val("").removeAttr("name").width(1).addClass("e-visual-mode");
            this.container.prepend(this._hiddenInput, this._ulBox);
            this.wrapper.height("auto");

            this._on(this.container, "mousedown", function (e) {
                if (!this.model.enabled) return false;
                var $target = $(e.target);
                if (!$target.is(this.element)) {
                    e.preventDefault();
                    if (!this._isFocused) this.element.focus();
                    if ($target.hasClass("e-options")) {
                        if (!e.ctrlKey && $target.siblings().hasClass("e-active")) this._removeActive();
                        if ($target.hasClass("e-active")) $target.removeClass("e-active");
                        else $target.addClass("e-active");
                    }
                    else this._moveCaretToEnd(this.element[0]);
                }
                if (!e.ctrlKey && ($target.hasClass("e-boxes") || $target.hasClass("e-input"))) this._removeActive();
            });
        },
        _destroyBoxModel: function () {
            this.container.prepend(this.element);
            this.element.attr({ name: this._hiddenInput.attr("name") }).removeAttr("style").removeClass("e-visual-mode");
            this.wrapper.height(this.model.height);
            this._hiddenInput.remove();
            this._ulBox.remove();
            this._off(this.container, "mousedown");
        },
        _deleteLastBox: function () {
            var items = this._ulBox.children();
            var item = items.last(), flag = item.hasClass("e-active");
            this._removeActive();
            flag ? this._deleteBox(item) : item.addClass("e-active");
        },
        _deleteBox: function (items) {
            for (var i = 0; i < items.length; i++) {
                var boxes = this._ulBox.children();
                var index = boxes.index(items[i]);
                this._selectedItems.splice(index, 1);

                var deli = this.model.delimiterChar;
                var values = this._hiddenInput.val().split(deli);
                values.splice(index, 1);
                this._hiddenInput.val(values.join(deli));
                $(items[i]).remove();
            }
            if (this.showSuggestionBox) this._refreshPopup();
            this._valueChange();
        },
        _removeActive: function () {
            this._ulBox.children("li.e-active").removeClass("e-active");
        },
        _adjustWidth: function () {
            var tempSpan = ej.buildTag("span", this.element.val()), wid, minWidth;
            this.container.append(tempSpan);
            minWidth = 30;  //  some additional width for textbox in visualmode
            wid = tempSpan.width() + minWidth;
            if (this.element.width() != wid)
                this.element.width(wid);
            tempSpan.remove();
        },

        _checkProperties: function () {
            this._checkReadOnly();
            this.model.delimiterChar = this._validateDelimiter(this.model.delimiterChar);
            if (!this.model.enabled) {
                this.model.enabled = true;
                this._disabled(true);
            }
            this._checkNameAttr();
            if (this.model.multiSelectMode == "visualmode") this._renderBoxModel();
            this.model.value = this._setValue(this.model.value);
        },

        _checkNameAttr: function () {
            if (!this.element.attr("name")) this.element.attr({ "name": this.element[0].id });
        },

        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        _destroyDropdown: function () {
            this.dropdownbutton.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
            this.dropdownbutton.remove();
            this.dropdownbutton = null;
            this.container.removeClass("e-padding");
        },

        _validateDelimiter: function (deli) {
            if (deli.length == 1) {
                var RegEx = /^[a-zA-Z0-9]+$/;
                if (!RegEx.test(deli)) return deli;
            }
            return ",";
        },

        _checkWatermarkSupport: function () {
            return 'placeholder' in document.createElement('input');
        },
        _setWatermark: function () {
            if (this._isWatermark) this.element.attr("placeholder", this.model.watermark);
        },
        _setWatermarkTxt: function () {
            if (this.model.watermark != null && $.trim(this.element.val()) == "") {
                this.element.addClass("e-watermark");
                this.element.val(this.model.watermark);
            }
        },

        _renderSuggestionList: function () {
            this.suggestionList = ej.buildTag("div.e-atc-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.target.id + "_suggestion", "", { "display": "none" }).attr("role", "listbox");
            this.element.attr("aria-owns", this.target.id + "_suggestion");
            this.popup = this.suggestionList;
            var scrollerDiv = ej.buildTag("div");
            this.ul = ej.buildTag("ul.e-ul").attr("role", "listbox");
            scrollerDiv.append(this.ul);
            this.suggestionList.append(scrollerDiv);
            $('body').append(this.suggestionList);
            this._setListWidth();
            this._setListHeight();
            this.suggestionList.ejScroller({ height: this.suggestionList.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.suggestionList.ejScroller("instance");
        },

        _checkEmptyList: function () {
            if (this.model.multiSelectMode == "visualmode") this._removeRepeated();
            if (this.suggestionListItems.length == 0) {
                this.suggestionListItems.push(this.model.noResults);
                this.noresult = true;
            }
            else this.noresult = false;
        },
        _showSuggestionList: function () {
            this._checkEmptyList();
            this._addNewTemplate = null;
            if (this.noresult && this.model.multiSelectMode == "visualmode" && this.model.allowNew && this.element.val() != "") {
                this.noresult = false;
                this.suggestionListItems.pop();
                this._addNewTemplate = "   (" + this.model.addNewText + ")";
                this.suggestionListItems.push(this.element.val() + this._addNewTemplate);
                this._checkEmptyList();
            }

            if (!this.noresult || this.model.showNoResults)
                this._generateSuggestionList();
        },
        /**
* generate suggestion list using the given object
* @private
*/
        _generateSuggestionList: function () {
            var list = this.suggestionListItems, i, suggList = [];
            this.ul.empty();

            if (typeof list[0] != "object") {
                if (this.model.distinct) list = ej.dataUtil.distinct(list, "", true);
                for (i = 0; i < list.length; i++) {
                    var _txt = (this.model.highlightSearch && !this.noresult) ? this._highlightSuggestion(list[i]) : list[i];
                    this.ul.append(ej.buildTag("li", _txt).attr("role", "option"));
                }
                this._currList = list;
                this._mapper = { txt: null, key: null };
            }
            else {
                var mapper = this.model.fields, mapFld = { _key: null, _text: null, _attr: null };
                mapFld._key = (mapper && mapper.uniqueKey) ? mapper["uniqueKey"] : "uniqueKey";
                mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
                mapFld._attr = (mapper && mapper.htmlAttr) ? mapper["htmlAttr"] : "htmlAttr";
                this._mapper = { txt: mapFld._text, key: mapFld._key }, this._currList = [];
                if (this.model.distinct) list = ej.dataUtil.distinct(list, mapFld._text, true);

                if (this.model.grouping) {
                    var mapCateg = (mapper && mapper.category) ? mapper["category"] : "category", groupedList, _query;
                    _query = ej.Query().group(mapCateg);
                    this._addSortingQuery(_query, "key");
                    groupedList = ej.DataManager(list).executeLocal(_query);
                    this._swapUnCategorized(groupedList);
                    for (i = 0; i < groupedList.length; i++) {
                        if (groupedList[i].key)
                            this.ul.append(ej.buildTag("li.e-category", groupedList[i].key).attr("role", "option"));
                        this._generateLi(groupedList[i].items, mapFld);
                    }
                }
                else this._generateLi(list, mapFld);
            }
            if (this._getLiTags().length > 0) this._showResult();
        },
        /**
* swaping uncategorized values in the sorting order
* @private
*/
        _swapUnCategorized: function (list) {
            $(list).each(function (i, obj) {
                if (!obj.key) {
                    for (var j = i; j > 0; j--) {
                        list[j] = list[j - 1];
                    }
                    list[j] = obj;
                    return false;
                }
            });
        },
        /**
* generate li tagd with the given data source
* @private
*/
        _generateLi: function (list, mapFld) {
            for (var j = 0; j < list.length; j++) {
                var _text = this._getField(list[j], mapFld._text);
                if (!ej.isNullOrUndefined(_text)) {
                    if (this.model.highlightSearch) _text = this._highlightSuggestion(_text);
                    if (this.model.template) _text = this._getTemplatedString(list[j], mapFld._text, _text);

                    var li = $(document.createElement("li")).append(_text);
                    this._setAttributes(this._getField(list[j], mapFld._attr), li);
                    this.ul.append(li);
                    this._currList = $.merge(this._currList, [list[j]]);
                }
            }
        },
        /**
* generate li tagd for the box model
* @private
*/
        _getLiTags: function () {
            return this.ul.children("li:not('.e-category')");
        },
        /**
* refresh popup using templated string
* @private
*/
        _getTemplatedString: function (list, searchLabl, searchTxt) {
            var str = this.model.template, start = str.indexOf("${"), end = str.indexOf("}");
            while (start != -1 && end != -1) {
                var content = str.substring(start, end + 1);
                var field = content.replace("${", "").replace("}", "");
                var replace = this._getField(list, field);
                // if highlightSearch is enabled, it replaces the highlighted search text
                if (searchLabl == field) replace = searchTxt;
                if (!replace) replace = "";
                str = str.replace(content, replace);
                start = str.indexOf("${"), end = str.indexOf("}");
            }
            return str;
        },
        /**
* get the field name for the given object
* @private
*/
        _getField: function (obj, fieldName) {
            return ej.pvt.getObject(fieldName, obj);
        },
        /**
* Set the key attribute for the input element
* @private
*/
        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    element.attr(key, data[key]);
            }
        },
        /**
* Set width for the popuplist
* @private
*/
        _setListWidth: function () {
            var width = this.model.suggestionBoxWidth;
            if (width && width != "auto") this.suggestionList.css({ "width": width });
            else this.suggestionList.css({ "width": this.wrapper.width() });
        },
        /**
* Set height for the popuplist
* @private
*/
        _setListHeight: function () {
            this.suggestionList.css({ "max-height": this.model.suggestionBoxHeight });
        },
        /**
* Refresh the popup with the given width
* @private
*/
        _refreshPopup: function () {
            if (this.model.suggestionBoxWidth == "auto" && this.wrapper.outerWidth() != this.suggestionList.outerWidth()) {
                this.suggestionList.css({ "width": this.wrapper.width() });
                this._refreshScroller();
            }
            this._setListPosition();
        },
        /**
* showing the popup with the result
* @private
*/
        _showResult: function () {
            this._refreshPopup();
            this._refreshScroller();
            if (this._isOpened)
                $(document).bind("mousedown", $.proxy(this._OnDocumentClick, this));
            else {
                this.suggestionList.css("display", "none");
                var tis = this;
                this.suggestionList.slideDown(200, "easeOutQuad", function () {
                    $(document).bind("mousedown", $.proxy(tis._OnDocumentClick, tis));
                });
            }

            this._isOpened = true;
            this.showSuggestionBox = true;
            var _suggestionListItems = this._getLiTags();
            this._listSize = _suggestionListItems.size();

            _suggestionListItems.bind("mouseenter", $.proxy(this._OnMouseEnter, this));
            _suggestionListItems.bind("mouseleave", $.proxy(this._OnMouseLeave, this));
            _suggestionListItems.bind("click", $.proxy(this._OnMouseClick, this));
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
        },
        /**
* Refresh the scroller popup functions
* @private
*/
        _hideResult: function () {
            if (this.showSuggestionBox) {
                this.showSuggestionBox = false;
                this._activeItem = 0;
                if (this._isOpened) this.suggestionList.css("display", "none");
                else this.suggestionList.slideUp(100, "easeOutQuad");
                $(document).unbind("mousedown", $.proxy(this._OnDocumentClick, this));
                $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
            }
        },
        /**
* Refresh the scroller popup functions
* @private
*/
        _refreshScroller: function () {
            this.suggestionList.css("height", "auto");
            this.suggestionList.find(".e-content, .e-vscroll").removeAttr("style");
            this.suggestionList.find(".e-vscroll div").removeAttr("style");
            this.suggestionList.css("display", "block");

            this.scrollerObj.model.height = this.suggestionList.height();
            this.scrollerObj.refresh();
            this.scrollerObj.option("scrollTop", 0);
            this.suggestionList.css("height", "auto");
        },
        /**
* set list position while opening the popup
* @private
*/
        _setListPosition: function () {
            var elementObj = this.wrapper;
            var pos = elementObj.offset(),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial();
            if (this.model.rtl) left -= this.suggestionList.outerWidth() - elementObj.outerWidth();

            this.suggestionList.css({
                "left": left + "px",
                "top": pos.top + totalHeight - border + 3 + "px",
                "z-index": maxZ
            });
        },
        /**
* Z-index calculation for body element
* @private
*/
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        /**
* define target focus for the value.
* @private
*/
        _targetFocus: function () {
            var minWidth = 30;  //  minimum width for textbox in visualmode
            if (this.model.multiSelectMode == "visualmode") this.element.width(minWidth);
            if (this.element.hasClass("e-watermark")) {
                this.element.removeClass("e-watermark");
                this.element.val("");
            }
            this.wrapper.addClass("e-focus");
            this._isFocused = true;
            /* Raise focusIn Event*/
            if (this.model.focusIn)
                this._trigger("focusIn");
        },
        /**
* define visual mode and other modes with value
* @private
*/
        _targetBlur: function () {
            this._isFocused = false;
            this.wrapper.removeClass("e-focus");
            !this._isWatermark && this._setWatermarkTxt();
            this._removeSelection();

            if (this.model.multiSelectMode == "visualmode") {
                this._removeActive();
                this.element.val("").width(1);
                this._preVal = "";
            }
            else {
                if (this.model.multiSelectMode == "delimiter") {
                    var val, deli, last;
                    val = $.trim(this.element.val());
                    deli = this.model.delimiterChar;
                    last = val.substr(val.length - deli.length, val.length);
                    if (last == deli) {
                        this.element.val(val.substr(0, val.length - deli.length));
                        this._valueChange();
                    }
                }
                this._updateSelectedItemArray();
            }

            /* Raise focusOut Event*/
            if (this.model.focusOut)
                this._trigger("focusOut");
        },
        /**
* remove selection from the autocomplete textbox.
* @private
*/
        _removeSelection: function () {
            if (this.model.autoFill) {
                this.element.attr("aria-autocomplete", "both");
                var selection = this._getCaretSelection();
                if (selection.end - selection.start != 0 && selection.end - selection.start != this.element.val().length)
                    this.target.value = this.target.value.substr(0, selection.start);
            }
        },
        /**
* remove list hover after hover item is changed.
* @private
*/
        _removeListHover: function () {
            this._getLiTags().removeClass("e-hover");
        },
        /**
* add hover class for the hovered  list item.
* @private
*/
        _addListHover: function () {
            var activeItem = $(this._getLiTags()[this._activeItem - 1]);
            activeItem.addClass("e-hover");
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
            activeItem.focus();
        },
        /**
* Caluculate scroll top for  the scroller.
* @private
*/
        _calcScrollTop: function () {
            var ulH = this.ul.outerHeight(), li = this.ul.find("li"), liH = 0, index, top, i;
            index = this.ul.find("li.e-hover").index();
            for (i = 0; i < index; i++) { liH += li.eq(i).outerHeight(); }
            top = liH - ((this.suggestionList.outerHeight() - li.eq(index).outerHeight()) / 2);
            return top;
        },
        /**
* Retrives the text from the active item.
* @private
*/
        _getActiveText: function () {
            if (this._mapper.txt) return this._getField(this._currList[this._activeItem - 1], this._mapper.txt);
            else return this._currList[this._activeItem - 1];
        },
        /**
* Retrives the unique key of the curent selected item.
* @private
*/
        _getUniqueKey: function () {
            var key = null;
            if (this._mapper.key) key = this._getField(this._currList[this._activeItem - 1], this._mapper.key);
            ej.isNullOrUndefined(key) && (key = null);
            return key;
        },
        /**
* set the textbox value for selecting the function
* @private
*/
        _setTextBoxValue: function () {
            if (this._activeItem && !this.noresult) {
                var currentValue, selection, val, text;
                currentValue = this._getActiveText();
                selection = this._getCaretSelection();
                this.suggLen = selection.start;
                val = this.target.value.substr(0, this.suggLen);

                if (val && val.toLowerCase() == currentValue.substr(0, this.suggLen).toLowerCase()) {
                    if (this.model.multiSelectMode == "delimiter") {
                        var _deliIndex = (this.target.value).lastIndexOf(this.model.delimiterChar);
                        var query = $.trim(val.substr(_deliIndex + 1, this.suggLen));
                        text = val + currentValue.substr(query.length, currentValue.length);
                    }
                    else {
                        text = val + currentValue.substr(val.length, currentValue.length);
                    }
                    this.element.val(text);
                    this._autofilSelection();
                    if (this.model.multiSelectMode == "visualmode")
                        this._adjustWidth();
                }
                else this._removeSelection();
            }
        },
        /**
* entering textbox value with the value selected.
* @private
*/
        _enterTextBoxValue: function () {
            if (this._activeItem && !this.noresult && !this.model.readOnly) {
                var currentValue = this._getActiveText(), currItem = this._currList[this._activeItem - 1];

                if (this.model.multiSelectMode == "visualmode") {
                    if (this._addNewTemplate) currentValue = currentValue.substr(0, currentValue.length - this._addNewTemplate.length);
                    var deli = this._hiddenInput.val() ? this.model.delimiterChar : "";
                    this._hiddenInput.val(this._hiddenInput.val() + deli + currentValue);
                    this.element.val("");
                    this._ulBox.append(this._createBox(currentValue));
                }
                else if (this.model.multiSelectMode == "delimiter") {
                    var _deliIndex = (this.target.value).lastIndexOf(this.model.delimiterChar)
                    this.element.val(this._queryString.substr(0, _deliIndex + 1) + currentValue + this.model.delimiterChar + " ");
                }
                else this.element.val(currentValue);
                this._selectedItems.push(currItem);
                this._moveCaretToEnd(this.element[0]);

                var _value = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
                if (this.model.select)
                    this._trigger("select", { value: _value, selectedText: currentValue, uniqueKey: this._getUniqueKey() });
                this._valueChange();
            }
        },
        /**
* creating box functionality for the box model.
* @private
*/
        _createBox: function (value) {
            var span = ej.buildTag("span.e-icon e-close");
            var li = ej.buildTag("li.e-options", value).append(span);

            this._on(span, "click", function (e) {
                if (!this.model.enabled) return false;
                this._deleteBox($(e.target).parent());
            });
            return li;
        },
        /**
* add loading class while data in search.
* @private
*/
        _addLoadingClass: function () {
            if (this.model.loadingImage)
                this.element.addClass("e-load");
        },
        /**
* Remove loading class after data is loaded.
* @private
*/
        _removeLoadingClass: function () {
            this.element.removeClass("e-load");
        },
        /**
* define functionalities for the highlighted search.
* @private
*/

        _highlightSuggestion: function (suggestion) {
            if ($.trim(this._queryString) != "") {
                var caseSensitive, tempQueryString, RegEx, mch, split, query;
                caseSensitive = this.model.caseSensitive ? "g" : "gi";
                query = $.trim(this._queryString);
                tempQueryString = this.model.filter == "startswith" ? "^" + query : query;
                RegEx = new RegExp(tempQueryString, caseSensitive);

                if (RegEx.test(suggestion)) {
                    mch = suggestion.match(RegEx);
                    split = suggestion.split(RegEx);
                    suggestion = "";
                    $(split).each(function (i, val) {
                        if (mch[i])
                            suggestion += val + "<span class='e-hilight-txt'>" + mch[i] + "</span>";
                        else
                            suggestion += val;
                    });
                }
            }
            return suggestion;
        },
        /**
* Applying rtl option for the autocomplete widget.
* @private
*/
        _RightToLeft: function (value) {
            if (value) {
                this.wrapper.addClass("e-rtl");
                this.suggestionList.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.suggestionList.removeClass("e-rtl");
            }
        },
        /**
* Set rounded corner for the autocomplete widget.
* @private
*/
        _setRoundedCorner: function (value) {
            if (value) {
                this.container.addClass("e-corner-all");
                this.suggestionList.addClass("e-corner-all");
            }
            else {
                this.container.removeClass("e-corner-all");
                this.suggestionList.removeClass("e-corner-all");
            }
        },
        /**
*Checking readonly functionality of autocomplete widget.
* @private
*/
        _checkReadOnly: function () {
            if (this.model.readOnly) {
                this.element.attr({ "readonly": "readonly", "aria-readonly": true });
                this._off(this.element, "keydown", this._OnKeyDown);
                this._off(this.element, "keyup", this._OnKeyUp);
            }
            else {
                this.element.removeAttr("readonly").removeAttr("aria-readonly");
                this._on(this.element, "keydown", this._OnKeyDown);
                this._on(this.element, "keyup", this._OnKeyUp);
            }
        },
        /**
* Defining key down functionality for autocomplete widget.
* @private
*/
        _OnKeyDown: function (e) {
            if (this.model.filter != "startswith")
                this.model.autoFill = false;

            switch (e.keyCode) {
                case 37:    // Left arrow
                case 35:    // End Key
                case 36:    // Home Key
                    this._removeSelection();
                case 39:    //Right arrow
                    break;
                case 38:    //Up arrow
                    e.preventDefault();
                    if (this.showSuggestionBox && this.suggestionList) {
                        this._removeListHover();
                        if (this._activeItem > 1)
                            this._activeItem -= 1;
                        else
                            this._activeItem = this._listSize;
                        this._addListHover();
                        if (this.model.autoFill) {
                            this.element.attr("aria-autocomplete", "both");
                            this._queryString = this.target.value;
                            this._setTextBoxValue();
                        }
                    }
                    break;
                case 40:    //Down arrow
                    e.preventDefault();
                    if (this.showSuggestionBox && this.suggestionList) {
                        this._removeListHover();
                        if (this._activeItem < this._listSize)
                            this._activeItem += 1;
                        else
                            this._activeItem = 1;
                        this._addListHover();
                        if (this.model.autoFill) {
                            this.element.attr("aria-autocomplete", "both");
                            this._queryString = this.target.value;
                            this._setTextBoxValue();
                        }
                    }
                    break;
                case 8:    // Backspace key
                    if ($.trim(this.element.val()) == "") this._isOpened = false;
                    if (this.model.multiSelectMode == "visualmode" && this.element.val() == "") {
                        this._deleteLastBox();
                    }
                    break;
                case 17:    // Ctrl key
                    this.ctrlKeyPressed = true;
                    break;
                case 9:    // Tab key
                    if (this.showSuggestionBox) {
                        e.preventDefault();
                        this._queryString = this.element.val();
                        this._enterTextBoxValue();
                        this._hideResult();
                        this.element.attr("aria-expanded", false);
                    }
                    break;
                case 27:    // Esc key
                    this._isOpened = false;
                    this._hideResult();
                    this.element.attr("aria-expanded", false);
                    break;
            }
            if (this.model.multiSelectMode == "visualmode") {
                if (e.keyCode != 17 && e.keyCode != 8 && e.keyCode != 46) this._removeActive();
                this._adjustWidth();
            }
        },
        /**
 * Defining key up  for autocomplete widget.
 * @private
 */
        _OnKeyUp: function (e) {
            this._keyDownComplete();
            if (this.ctrlKeyPressed) {
                if (e.keyCode == 17)
                    this.ctrlKeyPressed = false;
                return false;
            }

            if ($.trim(this.element.val()) == "" && e.keyCode == 38 && e.keyCode == 40) {
                this._hideResult();
                this.element.attr("aria-expanded", false);
                return false;
            }
            if (!this._checkDelimiter()) return false;
            this._queryString = $.trim(this._queryString);

            switch (e.keyCode) {
                //Restricts the keyup event for other functional keys          

                case 38: // Up Key
                case 40: // Down Key
                case 37: // Left Key
                case 39: // Right Key
                case 20: // CapsLk
                case 16: // Shift Key
                case 17: // Ctrl Key
                case 18: // Alt Key
                case 35: // End Key
                case 36: // Home Key
                case 144: // Num Lock
                case 9:    // Tab key
                case 27: break;  // Esc Key

                case 13:    // Enter Key
                    e.preventDefault();
                    this._queryString = this.element.val();
                    this._enterTextBoxValue();
                    if (this.model.autoFill) {
                        this.element.attr("aria-autocomplete", "both");
                        this.suggLen = this.element.val().length;
                        this._autofilSelection();
                    }
                    this._isOpened = false;
                    this._hideResult();
                    this.element.attr("aria-expanded", false);
                    break;

                case 46:    // Delete Key
                    if (this.model.multiSelectMode == "visualmode" && this.element.val() == "") {
                        this._deleteBox(this._ulBox.children("li.e-active"));
                        break;
                    }
                case 8:     // Backspace Key
                    if (this._queryString.length >= this.model.minCharacter) {
                        this._autoFill = false;
                        this._OnTextEnter();
                    }
                    else {
                        this.noresult = true;
                        this._hideResult();
                        this.element.attr("aria-expanded", false);
                        if ($.trim(this.element.val()) == "") this._isOpened = false;
                    }
                    break;

                default:
                    if (this._queryString.length >= this.model.minCharacter) {
                        this._autoFill = true;
                        this._OnTextEnter();
                    }
                    else this.noresult = true;
                    break;
            }
        },
        /**
* Getting filtered list from the data source.
* @private
*/
        _getFilteredList: function (list) {
            if (!list || !list.length || list.length < 1) this.suggestionListItems = [];
            else {
                var tempQuery = ej.Query();
                this._addQuery(tempQuery, typeof list[0] == "object");
                this.suggestionListItems = ej.DataManager(list).executeLocal(tempQuery);
            }
            this._doneRemaining();
        },
        /**
* Performing searching operation using query.
* @private
*/
        _performSearch: function () {
            var source = this.model.dataSource;
            if (ej.DataManager && source instanceof ej.DataManager) {
                window.clearTimeout(this.timer);
                var proxy = this;
                // a time delay to avoid the continuous request
                this.timer = window.setTimeout(function () {
                    proxy._fetchRemoteDat(source);
                }, 700);
            }
            else this._getFilteredList(source);
        },
        /**
* Fetching data from the remote location using query.
* @private
*/
        _fetchRemoteDat: function (source) {
            var proxy = this, queryPromise, tempQuery = this._getQuery();
            this._addQuery(tempQuery, true);
            queryPromise = source.executeQuery(tempQuery);
            queryPromise.fail(function (e) {
                proxy.suggestionListItems = null;
                proxy._removeLoadingClass();
            }).done(function (e) {
                proxy.suggestionListItems = e.result;
                proxy._doneRemaining();
            });
        },
        /**
 * Add sorting operation for the query.
 * @private
 */
        _addSortingQuery: function (query, key) {
            if (this.model.allowSorting) {
                var order = (this.model.sortOrder == "descending") ? true : false;
                query.sortBy(key, order);
            }
        },
        /**
 * Add value to the query for by retriving value from maaper value field
 * @private
 */
        _addQuery: function (_query, checkMapper) {
            var bindTo = "";
            if (checkMapper) {
                var mapper = this.model.fields;
                bindTo = (mapper && mapper.text) ? mapper["text"] : "text";
            }
            if (this._queryString) _query.where(bindTo, this.model.filter, this._queryString, !this.model.caseSensitive);
            this._addSortingQuery(_query, bindTo);
            if (this.model.listSize > 0) _query.take(this.model.listSize);
        },
        /**
 * creates a query for the given value to process the data spurce
 * @private
 */
        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query.clone();
            return queryManager;
        },
        /**
 * While typing the text in text box the functionalities has to be done are mentioned here
 * @private
 */
        _OnTextEnter: function () {
            this._addLoadingClass();
            this._hideResult();
            this.element.attr("aria-expanded", false);
            this._performSearch();
        },
        /**
     * Fill the text box with the complete word in the auto fill option
     * @private
     */
        _doneRemaining: function () {
            this._showSuggestionList();
            this.element.attr("aria-expanded", true).attr("aria-haspopup", true);
            if (this.model.autoFill && this._autoFill && !this.noresult) {
                this.element.attr("aria-autocomplete", "both");
                this._activeItem = 1;
                this._queryString = this.target.value;
                this._setTextBoxValue();
            }
            this._removeLoadingClass();
        },
        /**
      * Remove repeated values in the suggestion list
      * @private
      */
        _removeRepeated: function () {
            var results = this.suggestionListItems;
            if (!results || results.length == 0 || this._selectedItems.length == 0)
                return false;
            for (var i = 0; i < this._selectedItems.length; i++) {
                var index = results.indexOf(this._selectedItems[i]);
                if (index != -1) this.suggestionListItems.splice(index, 1);
            }
        },
        /**
      * Check deleimiter while typing in textbox widget
      * @private
      */
        _checkDelimiter: function () {
            this._queryString = !this.element.hasClass("e-watermark") ? this.element.val() : "";
            var _deliIndex = (this.model.multiSelectMode != "delimiter") ? -1 : this._queryString.lastIndexOf(this.model.delimiterChar);

            /*Check Delimiter Conditions and current text*/
            if (_deliIndex == -1) {
                return true;
            }
            else if (_deliIndex + 1 == this._queryString.length) {
                this._hideResult();
                this.element.attr("aria-expanded", false);
                return false;
            }
            else {
                this._queryString = this._queryString.substr(_deliIndex + 1, this._queryString.length);
                return true;
            }
        },
        /**
      * autofill selection in autocomplete is processed with below functionalities
      * @private
      */
        _autofilSelection: function () {
            var element = this.element[0], totLen = this.element.val().length;

            if (element.setSelectionRange)
                element.setSelectionRange(this.suggLen, totLen);
            else if (element.createTextRange) {
                // For lower version browsers (IE8, IE7 ...)
                element = element.createTextRange();
                element.collapse(true);
                element.moveEnd('character', totLen);
                element.moveStart('character', this.suggLen);
                element.select();
            }
        },
        /**
      * dropdown button click is processed with the following action
      * @private
      */
        _OnDropdownClick: function (e) {
            if (this.model.enabled && !this.model.readOnly) {
                e.preventDefault();
                this._addLoadingClass();
                if (this.showSuggestionBox) {
                    this._isOpened = false;
                    this._hideResult();
                    this.element.attr("aria-expanded", false);
                    this._removeLoadingClass();
                }
                else this._showFullList();
            }
        },
        /**
      * Shows the autocomplete popup with full list
      * @private
      */
        _showFullList: function () {
            if (!this._isFocused)
                this.element.focus();
            this._queryString = null;
            this._autoFill = false;
            this._performSearch();
        },
        /**
      * Mouse enter in popup is detected and processed here
      * @private
      */
        _OnMouseEnter: function (e) {
            var targetEle = e.target;
            if (e.target.tagName != "LI") targetEle = $(e.target).parents("li");
            this._getLiTags().removeClass("e-hover");
            $(targetEle).addClass("e-hover");

            this._queryString = this.element.val();
            this._activeItem = this._getLiTags().index($(targetEle)) + 1;
        },
        /**
      * Mouse leave in popup is detected and processed here
      * @private
      */
        _OnMouseLeave: function (e) {
            this._getLiTags().removeClass("e-hover");
            if (this.model.highlightSearch)
                this._getLiTags().find(".e-hilight-txt").removeClass("e-hover");
        },
        /**
      * Mouse click of popup is detected and processed here	
      * @private
      */
        _OnMouseClick: function (e) {
            if (!this.noresult) {
                this._enterTextBoxValue();
                this._isOpened = false;
                this._hideResult();
                this.element.attr("aria-expanded", false);
            }
        },
        /**
      * document click detection and binding and prodcessing
      * @private
      */
        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.suggestionList) && !$(e.target).parents(".e-atc-popup").is(this.suggestionList) &&
                !$(e.target).is(this.element) && !$(e.target).parents(".e-atc").is(this.wrapper)) {
                this._isOpened = false;
                this._hideResult();
                this.element.attr("aria-expanded", false);
            }
            else if ($(e.target).is(this.suggestionList) || $(e.target).parents(".e-atc-popup").is(this.suggestionList))
                e.preventDefault();
        },
        /**
      * window resize events for popup control		
      * @private
      */
        _OnWindowResize: function (e) {
            this._refreshPopup();
        },
        /**
      * value change fuction in checkbox events	
      * @private
      */
        _valueChange: function () {
            var currValue = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
            if (this.model.value != currValue) {
                this.model.value = currValue;
                if (this.model.change)
                    this._trigger("change", { value: currValue });
            }
        },
        /**
      * Update selected array of selection from datasource 
      * @private
      */
        _updateSelectedItemArray: function (e) {
            var value = this.element.val();
            this._selectedItems = [];
            if (this.model.multiSelectMode == "delimiter") {
                var values = value.split(this.model.delimiterChar);
                for (var i = 0; i < values.length; i++)
                    if (values[i]) this._selectedItems.push(values[i]);
            }
            else if (this.model.multiSelectMode == "none")
                this._selectedItems.push(value);
        },
        /**
       * KeyDown complete of selection event		
       * @private
       */
        _keyDownComplete: function () {
            var currValue = this.element.val();
            if (this._preVal != currValue) {
                this._preVal = currValue;
                if (this.model.multiSelectMode == "visualmode") this._adjustWidth();
                this._valueChange();
            }
        },
        /**
       * character Move to End of Autocomple control		
       * @private
       */
        _moveCaretToEnd: function (el) {
            if (typeof el.selectionStart == "number") {
                el.selectionStart = el.selectionEnd = el.value.length;
            } else if (typeof el.createTextRange != "undefined") {
                var range = el.createTextRange();
                range.collapse(false);
                range.select();
            }
        },
        /**
       * character selection of Autocomple control		
       * @private
       */
        _getCaretSelection: function () {
            var input = this.element[0], start = 0, end = 0;

            if (!isNaN(input.selectionStart)) {
                start = input.selectionStart;
                end = input.selectionEnd;
                return { start: Math.abs(start), end: Math.abs(end) };
            }
            // For lower version browsers (IE8, IE7 ...)
            var bookmark = document.selection.createRange().getBookmark();
            var selection = input.createTextRange();
            selection.moveToBookmark(bookmark);

            var before = input.createTextRange();
            before.collapse(true);
            before.setEndPoint("EndToStart", selection);
            var beforeLength = before.text.length, selLength = selection.text.length;
            return { start: beforeLength, end: beforeLength + selLength };
        },

        //-------------------- Event Wire-up -------------------------//
        /**
         * Wiring the events to Autocomple control		
		 * @private
         */
        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
        }
    });
    /**
	 * Enum for Autocomplete Filter mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.filter = {
        /**  Supports to search text with startswith  */
        StartsWith: "startswith",
        /**  Supports to search text with contains */
        Contains: "contains",
        /**  Supports to search text with endswith */
        EndsWith: "endswith",
        /**  Supports only for number lessthan only */
        LessThan: "lessthan",
        /**  Supports only for number greaterthan only */
        GreaterThan: "greaterthan",
        /**  Supports only for number lessthanorequal only */
        LessThanOrEqual: "lessthanorequal",
        /**  Supports only for number greaterthanorequal only */
        GreaterThanOrEqual: "greaterthanorequal",
        /**  Supports only for number equal only */
        Equal: "equal",
        /** Supports only for number notequal only */
        NotEqual: "notequal"
    };
    /**
	 * Enum for Autocomplete SortOrder mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.sortOrder = {
        /** Supports to sorts with ascending only */
        Ascending: "ascending",
        /** Supports to sorts with descending only */
        Descending: "descending"
    };
    /**
	 * Enum for Autocomplete selection mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.multiSelectMode = {
        /** Supports to selection mode with none only */
        None: "none",
        /** Supports to selection mode with delimitter only */
        Delimiter: "delimiter",
        /** Supports to selection mode with visualmode only */
        VisualMode: "visualmode"
    }
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin provides support to display calendar within your web page and allows to pick the date.
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejDatePicker
	* @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires jquery.globalize.js
    * @requires globalize.cultures.min.js
	* @requires ej.web.all.js
	* @classdesc Date selection with the input field.
	* @example 
	* &lt;input type="text" id="datepicker" /&gt;
	* &lt;script&gt;
	* // Create DatePicker
    * $("#datepicker").ejDatePicker();
	* &lt;/script&gt;
	*/

    ej.widget("ejDatePicker", "ej.DatePicker", {
        // widget element will be automatically set in this
        element: null,
        _rootCss: "e-datepicker",
        // user defined model will be automatically set in this
        model: null,
        validTags: ["input", "div", "span"],
        _setFirst: false,
        type: "editor",

        // default model
        defaults: {
            /**		
			* Specifies the header format of days in short, longer or min types. See {@link Header}
			* @default ej.DatePicker.Header.ShowHeaderMin
			* @type {String | Enum}
			* @example 
			* //To set dayHeaderFormat API value during initialization  
			* 	$("#datepicker").ejDatePicker({  dayHeaderFormat: ej.dayHeaderFormat.ShowHeaderShort });
			* @example 
			* //Get or set the datepicker API, after initialization:<br>
			*	//Gets the dayHeaderFormat value  
			*	$("#datepicker").ejDatePicker("option", "dayHeaderFormat");<br>			
			*	//Sets the dayHeaderFormat value 
			*	$("#datepicker").ejDatePicker("option", "dayHeaderFormat",  ej.dayHeaderFormat.ShowHeaderShort ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            dayHeaderFormat: "ShowHeaderMin",

            /**		
			* Shows the date icon button at right side of textbox and shows DatePicker popup on clicking it.
			* @default true
			* @type {Boolean}
			* @example 
			* //To set showDateIcon API during initialization  
			* 	$("#datepicker").ejDatePicker({  showDateIcon: false });
			* @example 
			* //Get or set the showDateIcon API, after initialization:<br>
			*	//Gets the showDateIcon value  
			*	$("#datepicker").ejDatePicker("option", "showDateIcon");<br>			
			*	//Sets the showDateIcon value 
			*	$("#datepicker").ejDatePicker("option", "showDateIcon",  false ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            showDateIcon: true,

            /**		
			* It allows to show footer in DatePicker calendar to select today date.
			* @default true
			* @type {Boolean}
			* @example 
			* //To set showFooter API during initialization  
			* 	$("#datepicker").ejDatePicker({  showFooter: false });
			* @example 
			* //Get or set the showFooter API, after initialization:<br>
			*	//Gets the showFooter value  
			*	$("#datepicker").ejDatePicker("option", "showFooter");<br>			
			*	//Sets the showFooter value 
			*	$("#datepicker").ejDatePicker("option", "showFooter",  false ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            showFooter: true,

            /**		
			* Allows to embed the DatePicker in the page. Also associate DatePicker with div element instead of input.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set displayInline API during initialization  
			* 	$("#datepicker").ejDatePicker({  displayInline: true });
			* @example 
			* //Get or set the displayInline API, after initialization:<br>
			*	//Gets the displayInline value  
			*	$("#datepicker").ejDatePicker("option", "displayInline");<br>			
			*	//Sets the displayInline value 
			*	$("#datepicker").ejDatePicker("option", "displayInline",  true ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            displayInline: false,

            /**		
			* Specifies the date format to be displayed in the input textbox of Datepicker. The selected Datepicker value will be displayed in specified date format.
			* @default "MM/dd/yyyy"
			* @type {String}
			* @example 
			* //To set dateFormat API during initialization  
			* 	$("#datepicker").ejDatePicker({  dateFormat: true });
			* @example 
			* //Get or set the dateFormat API, after initialization:<br>
			*	//Gets the dateFormat value  
			*	$("#datepicker").ejDatePicker("option", "dateFormat");<br>			
			*	//Sets the dateFormat value 
			*	$("#datepicker").ejDatePicker("option", "dateFormat",  true ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            dateFormat: '',

            /**		
			* Specifies the water mark text to be display in input text.
			* @default "Select date"
			* @type {String}
			* @example 
			* //To set waterMarkText during initialization  
			* 	$("#datepicker").ejDatePicker({  waterMarkText: "Enter date" });
			* @example 
			* //Get or set the waterMarkText API, after initialization:<br>
			*	//Gets the waterMarkText value  
			*	$("#datepicker").ejDatePicker("option", "waterMarkText");<br>			
			*	//Sets the waterMarkText value 
			*	$("#datepicker").ejDatePicker("option", "waterMarkText",  "Enter date" ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            waterMarkText: "Select date",

            /**		
			* Allow to display default date value in input textbox.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set displayDefaultDate API during initialization  
			* 	$("#datepicker").ejDatePicker({  displayDefaultDate: false });
			* @example 
			* //Get or set the displayDefaultDate API, after initialization:<br>
			*	//Gets the displayDefaultDate value  
			*	$("#datepicker").ejDatePicker("option", "displayDefaultDate");<br>			
			*	//Sets the displayDefaultDate value 
			*	$("#datepicker").ejDatePicker("option", "displayDefaultDate",  false ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            displayDefaultDate: true,

            /**		
			* Specifies the selected date value.
			* @default null
			* @type {String | DateObject}
			* @example 
			* //To set the datepicker value during initialization  
			* 	$("#datepicker").ejDatePicker({  value: new Date("5/5/2014") });
			* @example 
			* //Get or set the datepicker value, after initialization:<br>
			*	//Gets the datepicker value  
			*	$("#datepicker").ejDatePicker("option", "value");<br>			
			*	//Sets the datepicker value 
			*	$("#datepicker").ejDatePicker("option", "value",  new Date("5/5/2014") ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            value: null,

            /**		
            * Specifies the minimum date range to be displayed in DatePicker calendar.
            * @default new Date(1900, 00, 01)
            * @type {String | DateObject}
            * @example 
            * //To set minDate value during initialization  
            * 	$("#datepicker").ejDatePicker({  minDate: new Date("5/1/2013") });
            * @example 
            * //Get or set the minDate value, after initialization:<br>
            *	//Gets the minDate value
            *	$("#datepicker").ejDatePicker("option", "minDate");<br>			
            *	//Sets the minDate value
            *	$("#datepicker").ejDatePicker("option", "minDate",  new Date("5/1/2013") ); 
             * @memberof ejDatePicker
            * @instance
            */
            minDate: new Date(1900, 00, 01),

            /**		
           * Specifies the maximum date range to be displayed in DatePicker calendar.
           * @default new Date(2099, 11, 31)
           * @type {String | DateObject}
           * @example 
           * //To set maxDate value during initialization  
           * 	$("#datepicker").ejDatePicker({  maxDate : new Date("5/30/2015") });
           * @example 
           * //Get or set the maxDate value, after initialization:<br>
           *	//Gets the maxDate value
           *	$("#datepicker").ejDatePicker("option", "maxDate");<br>			
           *	//Sets the maxDate value
           *	$("#datepicker").ejDatePicker("option", "maxDate",  new Date("5/30/2015") ); 
            * @memberof ejDatePicker
           * @instance
           */
            maxDate: new Date(2099, 11, 31),

            /**		
			* Specifies the start level view in DatePicker calendar. See {@link Level}
			* @default ej.DatePicker.Level.Month
			* @type {String | Enum}
			* @example 
			* //To set startLevel API during initialization  
			* 	$("#datepicker").ejDatePicker({  startLevel: ej.startLevel.Year });
			* @example 
			* //Get or set the startLevel API, after initialization:<br>
			*	//Gets the startLevel value  
			*	$("#datepicker").ejDatePicker("option", "startLevel");<br>			
			*	//Sets the startLevel value 
			*	$("#datepicker").ejDatePicker("option", "startLevel",  ej.startLevel.Year ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            startLevel: "Month",

            /**		
			* Specifies the start level view in DatePicker calendar. See {@link Level}
			* @default ""
			* @type {String | Enum}
			* @example 
			* //To set depthLevel API during initialization  
			* 	$("#datepicker").ejDatePicker({  depthLevel: ej.depthLevel.Year });
			* @example 
			* //Get or set the depthLevel API, after initialization:<br>
			*	//Gets the depthLevel value  
			*	$("#datepicker").ejDatePicker("option", "depthLevel");<br>			
			*	//Sets the depthLevel value 
			*	$("#datepicker").ejDatePicker("option", "depthLevel",  ej.depthLevel.Year ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            depthLevel: "",

            /**		
			* Set the root class for DatePicker theme. This cssClass API helps to use custom skinning option for DatePicker control. 
			* @default ""
			* @type {String}
			* @example 
			* //To set cssClass API during initialization  
			* 	$("#datepicker").ejDatePicker({  cssClass: "gradient-lime" });
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the cssClass value  
			*	$("#datepicker").ejDatePicker("option", "cssClass");<br>			
			*	//Sets the cssClass value 
			*	$("#datepicker").ejDatePicker("option", "cssClass", "gradient-lime" ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            cssClass: "",

            /**		
			* Specifies the start day of the week in DatePicker calendar.
			* @default 1
			* @type {Number}
			* @example 
			* //To set startDay API during initialization  
			* 	$("#datepicker").ejDatePicker({  startDay: 2 });
			* @example 
			* //Get or set the startDay API, after initialization:<br>
			*	//Gets the startDay value  
			*	$("#datepicker").ejDatePicker("option", "startDay");<br>			
			*	//Sets the startDay value 
			*	$("#datepicker").ejDatePicker("option", "startDay", 2 ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            startDay: 0,

            /**		
			* Specifies the number of months to navigate at one click in next and previous button.
			* @default 1
			* @type {Number}
			* @example 
			* //To set stepMonths API during initialization  
			* 	$("#datepicker").ejDatePicker({  stepMonths: 2 });
			* @example 
			* //Get or set the stepMonths API, after initialization:<br>
			*	//Gets the stepMonths value  
			*	$("#datepicker").ejDatePicker("option", "stepMonths");<br>			
			*	//Sets the stepMonths value 
			*	$("#datepicker").ejDatePicker("option", "stepMonths", 2 ); 
			 * @memberof ejDatePicker
			* @instance
			*/
            stepMonths: 1,

            /**		
			* Localize the language of DatePicker calendar.
			* @default "en-US"
			* @type {String}
			* @example 
			* //To set localize API during initialization  
			* 	$("#datepicker").ejDatePicker({  localize: "de-DE" });
			* @example 
			* //Get or set the localize API, after initialization:<br>
			*	//Gets the localize value  
			*	$("#datepicker").ejDatePicker("option", "localize");<br>			
			*	//Sets the localize value 
			*	$("#datepicker").ejDatePicker("option", "localize", "de-DE"); 
			 * @memberof ejDatePicker
			* @instance
			*/
            localize: "en-US",

            /**		
			* It allows to show days in other months of DatePicker calendar.
			* @default true
			* @type {Boolean}
			* @example 
			* //To set showOtherMonths API during initialization  
			* 	$("#datepicker").ejDatePicker({  showOtherMonths: false });
			* @example 
			* //Get or set the showOtherMonths API, after initialization:<br>
			*	//Gets the showOtherMonths value  
			*	$("#datepicker").ejDatePicker("option", "showOtherMonths");<br>			
			*	//Sets the showOtherMonths value 
			*	$("#datepicker").ejDatePicker("option", "showOtherMonths", false); 
			 * @memberof ejDatePicker
			* @instance
			*/
            showOtherMonths: true,

            /**		
			* When strictMode true it allows the value outside of the range also, otherwise it internally changed to the correct value.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set strictMode API during initialization  
			* 	$("#datepicker").ejDatePicker({  strictMode: true });
			* @example 
			* //Get or set the strictMode API, after initialization:<br>
			*	//Gets the strictMode value  
			*	$("#datepicker").ejDatePicker("option", "strictMode");<br>			
			*	//Sets the strictMode value 
			*	$("#datepicker").ejDatePicker("option", "strictMode", true); 
			 * @memberof ejDatePicker
			* @instance
			*/
            strictMode: false,

            /**		
			* Enables or disables the state maintenance of DatePicker.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set persist API during initialization  
			* 	$("#datepicker").ejDatePicker({  persist: true });
			* @example 
			* //Get or set the persist API, after initialization:<br>
			*	//Gets the persist value  
			*	$("#datepicker").ejDatePicker("option", "persist");<br>			
			*	//Sets the persist value 
			*	$("#datepicker").ejDatePicker("option", "persist", true); 
			 * @memberof ejDatePicker
			* @instance
			*/
            persist: false,

            /**		
			* Enables or disables the datepicker control.
			* @default true
			* @type {Boolean}
			* @example 
			* //To set enabled API during initialization  
			* 	$("#datepicker").ejDatePicker({  enabled: false });
			* @example 
			* //Get or set the enabled API, after initialization:<br>
			*	//Gets the enabled value  
			*	$("#datepicker").ejDatePicker("option", "enabled");<br>			
			*	//Sets the enabled value 
			*	$("#datepicker").ejDatePicker("option", "enabled", false); 
			 * @memberof ejDatePicker
			* @instance
			*/
            enabled: true,

            /**		
			* Specifies the width of the datepicker input text.
			* @default "160px"
			* @type {String}
			* @example 
			* //To set width API during initialization  
			* 	$("#datepicker").ejDatePicker({  width: 200 });
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value  
			*	$("#datepicker").ejDatePicker("option", "width");<br>			
			*	//Sets the width value 
			*	$("#datepicker").ejDatePicker("option", "width", 200); 
			 * @memberof ejDatePicker
			* @instance
			*/
            width: "",

            /**		
			* Specifies the height of the datepicker input text.
			* @default "28px"
			* @type {String}
			* @example 
			* //To set height API during initialization  
			* 	$("#datepicker").ejDatePicker({  height: 35 });
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value  
			*	$("#datepicker").ejDatePicker("option", "height");<br>			
			*	//Sets the height value 
			*	$("#datepicker").ejDatePicker("option", "height", 35); 
			 * @memberof ejDatePicker
			* @instance
			*/
            height: "",

            /**		
			* Display Right to Left direction of DatePicker calendar and input box.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set rtl API during initialization  
			* 	$("#datepicker").ejDatePicker({  rtl : true });
			* @example 
			* //Get or set the rtl API, after initialization:<br>
			*	//Gets the rtl value  
			*	$("#datepicker").ejDatePicker("option", "rtl");<br>			
			*	//Sets the rtl value 
			*	$("#datepicker").ejDatePicker("option", "rtl", true); 
			 * @memberof ejDatePicker
			* @instance
			*/
            rtl: false,

            /**		
			* DatePicker input will be displayed in rounded corner style, when this property is set to true.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set roundedCorner API during initialization  
			* 	$("#datepicker").ejDatePicker({  roundedCorner : true });
			* @example 
			* //Get or set the roundedCorner API, after initialization:<br>
			*	//Gets the roundedCorner value  
			*	$("#datepicker").ejDatePicker("option", "roundedCorner");<br>			
			*	//Sets the roundedCorner value 
			*	$("#datepicker").ejDatePicker("option", "roundedCorner", true); 
			 * @memberof ejDatePicker
			* @instance
			*/
            roundedCorner: false,

            /**		
			* Specifies the header format to be displayed in the pop up of Datepicker.
			* @default "MMMM yyyy"
			* @type {String}
			* @example 
			* //To set headerFormat API during initialization  
			* 	$("#datepicker").ejDatePicker({  headerFormat : "MMMM yy" });
			* @example 
			* //Get or set the headerFormat API, after initialization:<br>
			*	//Gets the headerFormat value  
			*	$("#datepicker").ejDatePicker("option", "headerFormat");<br>			
			*	//Sets the headerFormat value 
			*	$("#datepicker").ejDatePicker("option", "headerFormat", "MMMM yy"); 
			 * @memberof ejDatePicker
			* @instance
			*/
            headerFormat: 'MMMM yyyy',

            /**		
			* Set the text name for the today button in the datepicker popup. 
			* @default "Today"
			* @type {String}
			* @example 
			* //To set buttonText API during initialization  
			* 	$("#datepicker").ejDatePicker({  buttonText : "Now" });
			* @example 
			* //Get or set the buttonText API, after initialization:<br>
			*	//Gets the buttonText value  
			*	$("#datepicker").ejDatePicker("option", "buttonText");<br>			
			*	//Sets the buttonText value 
			*	$("#datepicker").ejDatePicker("option", "buttonText", "Now"); 
			 * @memberof ejDatePicker
			* @instance
			*/
            buttonText: 'Today',

            /**		
			* Indicates that the datepicker value can only be read. 
			* @default false
			* @type {Boolean}
			* @example 
			* //To set readOnly API during initialization  
			* 	$("#datepicker").ejDatePicker({  readOnly : true });
			* @example 
			* //Get or set the readOnly API, after initialization:<br>
			*	//Gets the readOnly value  
			*	$("#datepicker").ejDatePicker("option", "readOnly");<br>			
			*	//Sets the readOnly value 
			*	$("#datepicker").ejDatePicker("option", "readOnly", true); 
			 * @memberof ejDatePicker
			* @instance
			*/
            readOnly: false,

            /**     
            * Fires when DatePicker popup opened.
            * @event
            * @name ejDatePicker#open 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the eventclose"
            * @param {string}  argument.value returns the current date value
            * @param {string}  argument.previousValue returns the previously selected value
            * @example 
            * //open event for datepicker
            * $("#datepicker").ejDatePicker({
            *    open: function (args) {}
            * });      
            * @memberof ejDatePicker
            * @instance
            */
            open: null,

            /**     
            * Fires when DatePicker popup closed.
            * @event
            * @name ejDatePicker#close 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the current date value
            * @param {string}  argument.previousValue returns the previously selected value
            * @example 
            * //close event for datepicker
            * $("#datepicker").ejDatePicker({
            *    close: function (args) {}
            * });      
            * @memberof ejDatePicker
            * @instance
            */
            close: null,

            /**     
            * Fires when a date is selected from the datepicker popup.
            * @event
            * @name ejDatePicker#select 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the current date value
            * @param {string}  argument.perviousDate returns the previously selected value
            * @example 
            * //select event for datepicker
            * $("#datepicker").ejDatePicker({
            *    select: function (args) {}
            * });      
            * @memberof ejDatePicker
            * @instance
            */
            select: null,

            /**     
            * Fires when the datepicker input value is changed.
            * @event
            * @name ejDatePicker#change 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the datepicker input value
            * @param {string}  argument.perviousDate returns the previously selected value
            * @example 
            * //change event for datepicker
            * $("#datepicker").ejDatePicker({
            *    change: function (args) {}
            * });      
            * @memberof ejDatePicker
            * @instance
            */
            change: null,

            /**     
            * Fires when datePicker input gets focus.
            * @event
            * @name ejDatePicker#focusIn 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //focusIn event for datepicker
            * $("#datepicker").ejDatePicker({
            *    focusIn: function (args) {}
            * });      
            * @memberof ejDatePicker
            * @instance
            */
            focusIn: null,

            /**     
            * Fires when datePicker input losses the focus.
            * @event
            * @name ejDatePicker#focusOut 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //focusOut event for datepicker
            * $("#datepicker").ejDatePicker({
            *    focusOut: function (args) {}
            * });      
            * @memberof ejDatePicker
            * @instance
            */
            focusOut: null
        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            startDay: "number",
            stepMonths: "number",
            showOtherMonths: "boolean",
            strictMode: "boolean",
            roundedCorner: "boolean",
            rtl: "boolean",
            displayDefaultDate: "boolean",
            displayInline: "boolean",
            showDateIcon: "boolean",
            localize: "string",
            readOnly: "boolean",
            cssClass: "string",
            dateFormat: "string",
            waterMarkText: "string",
            headerFormat: "string",
            buttonText: "string"
        },

        _setModel: function (jsondata) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(jsondata["enabled"])) return false;
            var callRefresh = false, start = false, validate = false;
            for (var key in jsondata) {
                switch (key) {
                    case "dayHeaderFormat":
                        this.model.dayHeaderFormat = jsondata[key];
                        callRefresh = true;
                        break;
                    case "showDateIcon":
                        this._renderDateIcon(jsondata[key], true);
                        break;
                    case "displayInline":
                        this._setDisplayInline(jsondata[key]);
                        break;
                    case "value":
                        this._setDateValue(jsondata[key]);
                        jsondata[key] = this.model.value;
                        validate = true;
                        callRefresh = true;
                        break;
                    case "dateFormat":
                        this.model.dateFormat = jsondata[key];
                        this._setDateValue(this.model.value);
                        break;
                    case "minDate":
                        this._setMinDate(jsondata[key]);
                        jsondata[key] = this.model.minDate;
                        validate = true;
                        callRefresh = true;
                        break;
                    case "maxDate":
                        this._setMaxDate(jsondata[key]);
                        jsondata[key] = this.model.maxDate;
                        validate = true;
                        callRefresh = true;
                        break;
                    case "localize":
                        this.model.localize = jsondata[key];
                        this.model.dateFormat = '';
                        this._setCulture(jsondata[key]);
                        this._setDateValue(this.model.value);
                        callRefresh = true;
                        break;
                    case "showOtherMonths":
                        this.model.showOtherMonths = jsondata[key];
                        callRefresh = true;
                        break;
                    case "strictMode":
                        this.model.strictMode = jsondata[key];
                        validate = true;
                        callRefresh = true;
                        break;
                    case "readOnly":
                        this.model.readOnly = jsondata[key];
                        this._disbleMaualInput();
                        break;
                    case "width":
                        this._setWidth(jsondata[key]);
                        break;
                    case "height":
                        this._setHeight(jsondata[key]);
                        break;
                    case "cssClass":
                        this._setSkin(jsondata[key]);
                        break;
                    case "rtl":
                        this._setRTL(jsondata[key]);
                        break;
                    case "roundedCorner":
                        this._setRoundedCorner(jsondata[key]);
                        break;
                    case "enabled":
                        if (!jsondata[key]) this.disable();
                        else this.enable();
                        break;
                    case "buttonText":
                        this._setFooterText(jsondata[key]);
                        break;
                    case "showFooter":
                        this._enableFooter(jsondata[key]);
                        break;
                    case "waterMarkText":
                        this.model.waterMarkText = jsondata[key]; this._setWaterMark();
                        break;
                    case "startDay":
                        if (parseInt(jsondata[key]) < 0 || parseInt(jsondata[key]) > 6) jsondata[key] = 0;
                        this.model.startDay = jsondata[key];
                        callRefresh = true;
                        break;
                    case "startLevel":
                        this.model.startLevel = jsondata[key]; start = true;
                        callRefresh = true;
                        break;
                    case "headerFormat":
                        this.model.headerFormat = jsondata[key];
                        callRefresh = true;
                        break;
                    case "depthLevel":
                        this.model.depthLevel = jsondata[key]; start = true;
                        callRefresh = true;
                        break;
                }
            }
            if (validate) {
                this._validateMinMaxDate();
                jsondata["value"] = this.model.value;
                jsondata["maxDate"] = this.model.maxDate;
                jsondata["minDate"] = this.model.minDate;
            }
            this._setWaterMark();

            if (callRefresh && this.isValidState && !this.element.hasClass("e-watermark"))
                this._refreshDatepicker();
            if (start) this._startLevel(this.model.startLevel);
            this._triggerChangeEvent();
            this._checkErrorClass();
        },
        observables: ["value"],
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.sfCalendar.remove();
            if (this.wrapper) {
                this.element.insertAfter(this.wrapper);
                this.wrapper.remove();
            }
            this.element.removeClass('e-datepicker e-input');
        },
        // constructor function
        _init: function () {
            this._setValues();
            this._createDatePicker();
        },
        //------------------------------Private Methods---------------------------------------------
        _setValues: function () {
            this.Date = new Date();
            this._id = this.element[0].id;
            this.isValidState = true;
            this._setCulture(this.model.localize);
            this._setMinDate(this.model.minDate);
            this._setMaxDate(this.model.maxDate);
            this._calendarDate = this._zeroTime(new Date());
            if (this.model.startDay < 0 || this.model.startDay > 6) this.model.startDay = 0;
            this.Date.firstDayOfWeek = this.model.startDay;
            this.Date.fullYearStart = '20';
            this._showHeader = true,
            this._validateMinMaxDate();
            this._dateValue = new Date(this._calendarDate.toString());
            this._isIE7 = this._checkIE7();
            // this variable is set to true in DateTimePicker control
            this._getInternalEvents = false;
            this._flag = true;
            this._ejHLWeekEnd = false;
            this._isOpen = false;
            this._prevDate = null;
            this._isFocused = false;
        },

        _createDatePicker: function () {
            this._createWrapper();
            this._wireEvents();
            if (this.model.displayInline)
                this.show();
            if (this.model.rtl) this._setRTL(true);
            if (this.model.roundedCorner) this._setRoundedCorner(true);
        },

        _createWrapper: function () {
            if (this._isInputBox()) {
                this.element.addClass("e-input").attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', 'tabindex': '0' }); //
                this.wrapper = ej.buildTag("span.e-datewidget e-widget " + this.model.cssClass);
                this.innerWrapper = ej.buildTag("span.e-in-wrap e-box");
                this.wrapper.append(this.innerWrapper).insertBefore(this.element);
                this.innerWrapper.append(this.element);
                this.innerWrapper.addClass('e-padding');
                this.dateIcon = ej.buildTag("span.e-select#" + this._id + "-img", "", null)
                    .append(ej.buildTag("span.e-icon e-date", "", {}, { 'aria-label': 'Select' })).insertAfter(this.element);
            }
            this._setHeight(this.model.height);
            if (this.model.width != "") this._setWidth(this.model.width);
            this.sfCalendar = ej.buildTag('div.e-datepicker e-popup e-widget ' + this.model.cssClass + ' e-calendar', "", {}, { id: 'e-' + this._id }).attr({ 'aria-hidden': 'true' })
                .insertBefore(this.element);
            this.popup = this.sfCalendar;
            this._resizeCalender();
            if (this.model.displayDefaultDate) this._setDateValue(this.model.value);
            this._preValue = this._textboxVal();
            this._setWaterMark();
            this._createCalender();
            this._setDisplayInline(this.model.displayInline);
            if (this.model.readOnly) this._disbleMaualInput();
            if (!this.model.enabled) this.disable();
            this._layoutChanged();
            this._checkErrorClass();
        },
        _isInputBox: function () {
            return (this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type')));
        },

        _renderDateIcon: function (bool, reRender) {
            if (reRender && this.model.showDateIcon == bool) return;
            if (!bool && this.dateIcon) {
                this.dateIcon.css('display', 'none');
                this.innerWrapper.removeClass('e-padding');
                this._bindInputEvent();
            }
            else {
                if (this.innerWrapper) {
                    this.innerWrapper.addClass('e-padding');
                    this.dateIcon.css('display', 'block');
                }
                if (!this.model.displayInline)
                    this._bindDateButton();
            }
            this.model.showDateIcon = bool;
        },

        _resizeCalender: function () {
            var wrapperWidth;
            if (this.model.dayHeaderFormat == "ShowHeaderShort")
                wrapperWidth = "18em";
            else if (this.model.dayHeaderFormat == "ShowHeaderLong") {
                wrapperWidth = "35em";
            } else wrapperWidth = "18em";
            this.sfCalendar.width(wrapperWidth);
        },

        _setWidth: function (value) {
            if (this.wrapper) this.wrapper.width(value);
            else this.element.width(value);
        },
        _setHeight: function (value) {
            if (value) {
                if (this.wrapper) this.wrapper.height(value);
                else this.element.height(value);
            }
            if (this._isIE7) this.element.height(this.innerWrapper.height());
        },
        _setRTL: function (isRTL) {
            if (isRTL) {
                if (this.model.showDateIcon && this.wrapper)
                    this.wrapper.addClass("e-rtl");
                else this.element.addClass("e-rtl");
                this.sfCalendar.addClass("e-rtl");
            }
            else {
                if (this.model.showDateIcon && this.wrapper)
                    this.wrapper.removeClass("e-rtl");
                else this.element.removeClass("e-rtl");
                this.sfCalendar.removeClass("e-rtl");
            }
        },
        _setRoundedCorner: function (bool) {
            if (bool) {
                if (this.innerWrapper) this.innerWrapper.addClass("e-corner-all");
                this.sfCalendar.addClass("e-corner-all");
            }
            else {
                if (this.innerWrapper) this.innerWrapper.removeClass("e-corner-all");
                this.sfCalendar.removeClass("e-corner-all");
            }
        },

        _refreshDatepicker: function () {
            this._setDateValue(this._textboxVal());
            $(".e-text", this.sfCalendar).text(this._asString(this._calendarDate, this.model.headerFormat));
            this._resizeCalender();
            this._dateValue = new Date(this._calendarDate.toString());
            this._hoverDate = this._calendarDate.getDate() - 1;
            this._renderCalendar(this, this._dateValue);
            this._setFooterText(this.model.buttonText);
            this._enableFooter(this.model.showFooter);
            this._layoutChanged();
        },
        _setFooterText: function (footerText) {
            $('.e-footer-text', this.sfCalendar).html(footerText);
        },
        _setSkin: function (skin) {
            if (this.wrapper) {
                this.wrapper.removeClass(this.model.cssClass);
                this.wrapper.addClass(skin);
            }
            else {
                this.element.removeClass(this.model.cssClass);
                this.element.addClass(skin);
            }
            this.sfCalendar.removeClass(this.model.cssClass);
            this.sfCalendar.addClass(skin);
        },
        _setDisplayInline: function (isDisplayInline) {
            this.model.displayInline = isDisplayInline;
            if (isDisplayInline && this._isInputBox()) {
                this.sfCalendar.insertAfter(this.wrapper);
                this._setDatePickerPosition();
            }
            else if (isDisplayInline) {
                this.element.append(this.sfCalendar);
            }
            else {
                this.sfCalendar.css('display', 'none');
                $('body').append(this.sfCalendar);
                this._isOpen = false;
            }
            if (isDisplayInline) {
                this.show();
                this._off($(this.dateIcon, this.wrapper), "click", $.proxy(this._showDatePopUp, this));
                this.element.unbind("click", $.proxy(this._showDatePopUp, this));
            }
            else this._renderDateIcon(this.model.showDateIcon, false);
        },
        /*Disabling Manual input */
        _disbleMaualInput: function () {
            if (this.model.readOnly) {
                $(this.element).attr("readonly", "readonly");
                if (!this.model.displayInline) this.hide();
            }
            else {
                $(this.element).removeAttr("readonly");
            }
        },
        _checkDateObject: function (date) {
            if (!date || (typeof JSON === "object" && JSON.stringify(date) === "{}")) return date = "";
            else if (!(date instanceof Date)) {
                date = this._parseDate(date);
            }
            if (!isNaN(Date.parse(date))) return this._dateValue = this._calendarDate = this._zeroTime(date);
            return null;
        },
        _checkInstanceType: function (date) {
            date = this._stringToObject(date);
            if (!date) return null;
            else if (!(date instanceof Date)) {
                date = this._parseDate(date);
            }
            if (!isNaN(Date.parse(date))) return this._zeroTime(date);
            return null;
        },
        _stringToObject: function (value) {
            if (typeof value === "string")
                return Globalize.parseDate(value, this.model.dateFormat);
            return value;
        },
        _validateMinMaxDate: function () {
            var dateChange = false, valueExceed = false;
            if (this.model.maxDate < this.model.minDate) this.model.minDate = this.model.maxDate;
            if (!this.model.strictMode) {
                if (this.model.value) {
                    if (this.model.value < this.model.minDate) {
                        this._calendarDate = this.model.value = this.model.minDate;
                        dateChange = true;
                    }
                    else if (this.model.value > this.model.maxDate) {
                        this._calendarDate = this.model.value = this.model.maxDate;
                        dateChange = true;
                    }
                }
                else {
                    this.element.val("");
                    if (this._calendarDate < this.model.minDate) this._calendarDate = this.model.minDate;
                    else if (this._calendarDate > this.model.maxDate) this._calendarDate = this.model.maxDate;
                }
                this.isValidState = true;
            }
            else {
                if (this.model.value) {
                    if (this.model.value < this.model.minDate) {
                        this._calendarDate = this.model.minDate;
                        this.isValidState = false;
                        valueExceed = true;
                    }
                    else if (this.model.value > this.model.maxDate) {
                        this._calendarDate = this.model.maxDate;
                        this.isValidState = false;
                        valueExceed = true;
                    }
                    else this.isValidState = true;
                }
                else {
                    if (this._calendarDate < this.model.minDate) this._calendarDate = this.model.minDate;
                    else if (this._calendarDate > this.model.maxDate) this._calendarDate = this.model.maxDate;
                }
            }
            if (dateChange) this.element.val(this._formatter(this.model.value, this.model.dateFormat));
            if (valueExceed && this._getInternalEvents) this._trigger("outOfRange");
        },
        _setCulture: function (culture) {
            this.culture = Globalize.culture(culture);
            if (this.culture) {
                this.Date.dayNames = this.culture.calendar.days.names;
                this.Date.dayNamesMin = this.culture.calendar.days.namesShort;
                this.Date.abbrDayNames = this.culture.calendar.days.namesAbbr;
                this.Date.monthNames = this.culture.calendar.months.names;
                this.Date.abbrMonthNames = this.culture.calendar.months.namesAbbr;
                this.Date.format = this.culture.calendar.patterns.d;
                if (this.model.dateFormat == '') this.model.dateFormat = this.culture.calendar.patterns.d;
            } else {
                this.Date.dayNames = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                this.Date.dayNamesMin = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"];
                this.Date.abbrDayNames = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
                this.Date.monthNames = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
                this.Date.abbrMonthNames = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
                this.Date.format = 'MM/dd/yyyy';
                if (this.model.dateFormat == '') this.model.dateFormat = 'MM/dd/yyyy';
            }
            this._separator = this._getSeparator();
        },
        /* Set Water Mark Text */
        _setWaterMark: function () {
            if (this.element != null && this.element.hasClass("e-input") && this._textboxVal() == "" && this.model.waterMarkText) {
                this.isValidState = true;
                this._checkErrorClass();
                this.element.addClass("e-watermark").val(this.model.waterMarkText);
                this.model.value = null;
                return true;
            }
        },
        _setDatePickerPosition: function () {
            var pos = this.wrapper.offset();
            var left = pos.left;
            var totalHeight = this.wrapper.outerHeight();
            var maxZ = this._getZindexPartial();
            if (this.model.rtl) left += this.wrapper.width() - this.sfCalendar.width();

            this.sfCalendar.css({
                "left": left + "px",
                'position': 'absolute',
                "top": (pos.top + totalHeight + 3) + "px",
                "z-index": maxZ
            });
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },

        _setMinDate: function (d) {
            this.model.minDate = this._checkInstanceType(d);
            if (!this.model.minDate) {
                this.model.minDate = (new Date('11/31/1899'));
            }
        },

        _setMaxDate: function (d) {
            this.model.maxDate = this._checkInstanceType(d);
            if (!this.model.maxDate) {
                this.model.maxDate = (new Date('12/31/2099')); // using the JS Date.parse function which expects mm/dd/yyyy
            }
        },
        _setDateValue: function (date) {
            this._removeWatermark();
            var newDate = this._checkDateObject(date);
            if (newDate != null) {
                this.isValidState = true;
                if (newDate == "") {
                    this.element.val("");
                    this.model.value = "";
                } else {
                    this.model.value = newDate;
                    this._validateMinMaxDate();
                    this._preTxtValue = this.element.val(this._formatter(this.model.value, this.model.dateFormat));
                }
            }
            else {
                (typeof date === "string" && this.model.strictMode) ? this.element.val(date) : this.element.val("");
                this.model.value = "";
                this.isValidState = (this._textboxVal() == "") ? true : false;
            }
        },
        _updateInputVal: function () {
            var val = this._validateValue();
            if (val != null) {
                if (this.sfCalendar.find('.e-datepicker-days').is(':visible')) {
                    this._refreshDatepicker();
                }
                else if (this.model.displayDefaultDate) {
                    this._setDateValue(val);
                }
            }
        },
        _validateInputVal: function () {
            var val = this._validateValue();
            if (val != null) {
                if (val <= this.model.maxDate && val >= this.model.minDate)
                    this.isValidState = true;
                else {
                    this.model.value = "";
                    this.isValidState = false;
                }
            }
        },

        _validateValue: function () {
            return this._parseDate(this._textboxVal());
        },
        _getSeparator: function () {
            if (this.culture) {
                formats = this.culture.calendar.patterns.d;
            }
            else formats = this.model.dateFormat;
            var regex = new RegExp("^[a-zA-Z0-9]+$");
            for (var i = 0; i < formats.length; i++) {
                if (!regex.test(formats[i])) return formats[i];
            }
        },
        _checkIE7: function () {
            if (navigator.appName == 'Microsoft Internet Explorer') {
                var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})"), version = -1;
                if (re.exec(navigator.userAgent) != null)
                    version = parseFloat(RegExp.$1);
                if (version >= 7 && version < 8) return true;
            }
            return false;
        },

        /*----------------------------------------------region date methods------------------------------------------------------*/

        //Date formatter - Convert date object to specific date format
        _formatter: function (date, format) {
            return Globalize.format(date, format, this.model.localize);
        },
        _parseDate: function (date) {
            return Globalize.parseDate(date, this.model.dateFormat);
        },
        // Checks if the year is a leap year.
        isLeapYear: function (year) {
            return (year % 4 == 0 && year % 100 != 0) || year % 400 == 0;
        },
        //Sets the time component of this Date to zero for cleaner, easier comparison of dates where time is not relevant.
        _zeroTime: function (date) {
            var newDate = new Date(date);
            newDate.setMilliseconds(0);
            newDate.setSeconds(0);
            newDate.setMinutes(0);
            newDate.setHours(0);
            return newDate;
        },
        _zeroPad: function (number) {
            // Pad a number with a zero, to make it 2 digits
            return ((number < 10) ? "0" : "") + String(number);
        },
        //Convert date object to specific format
        _formatDate: function (date) {
            return this._formatter(date, this.Date.format);
        },
        //Gets the number of days in the month.
        _getDaysInMonth: function (date) {
            return [31, (this.isLeapYear(date) ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][date.getMonth()];
        },
        //Add a number of days to the date object.
        _addDays: function (d, number) {
            d.setDate(d.getDate() + number);
            return d;
        },
        //Add a number of years to the date object.
        _addYears: function (d, number) {
            d.setFullYear(d.getFullYear() + number);
            return d;
        },
        //Add a number of months to the date object.
        _addMonths: function (d, number) {
            var tempDatedateMonth = d.getDate();
            d.setMonth(d.getMonth() + number);
            if (tempDatedateMonth > d.getDate())
                this._addDays(d, -d.getDate());
            return d;
        },
        //Checks if the day is a weekend day (Sat or Sun).
        _isWeekend: function (date) {
            return date.getDay() == 0 || date.getDay() == 6;
        },
        // Returns a string representation of the date object according to Date.Format.
        _asString: function (date, format) {
            var r = format || this.Date.format;
            return r
                .split('yyyy').join(date.getFullYear())
                .split('yy').join((date.getFullYear() + '').substring(2))
                .split('MMMM').join(this._getMonthName(false, date))
                .split('MMM').join(this._getMonthName(true, date))
                .split('MM').join(this._zeroPad(date.getMonth() + 1))
                .split('dd').join(this._zeroPad(date.getDate()))
                .split('hh').join(this._zeroPad(date.getHours()))
                .split('min').join(this._zeroPad(date.getMinutes()))
                .split('ss').join(this._zeroPad(date.getSeconds()));
        },
        //Gets the name of the month.
        _getMonthName: function (abbreviated, date) {
            return abbreviated ? this.Date.abbrMonthNames[date.getMonth()] : this.Date.monthNames[date.getMonth()];
        },
        _currentDay: function () {
            var today = new Date();
            var dd = today.getDate();
            var mm = today.getMonth() + 1; //January is 0!
            var yyyy = today.getFullYear();
            return new Date(mm + '/' + dd + '/' + yyyy);
        },
        /*---------------------------------------------region Basic Functionality---------------------------------------*/

        _displayNewMonth: function (m, y) {
            this._setDisplayedMonth(this.displayedMonth + m, this.displayedYear + y, true);
            return false;
        },

        _setDisplayedMonth: function (m, y, rerender) {
            if (this.model.minDate == undefined || this.model.maxDate == undefined) {
                return;
            }
            var s = new Date(this.model.minDate.getTime());
            s.setDate(1);
            var e = new Date(this.model.maxDate.getTime());
            e.setDate(1);

            var t;
            if ((!m && !y) || (isNaN(m) && isNaN(y))) {
                // no month or year passed - default to current month
                t = this._zeroTime(new Date());
                t.setDate(1);
            } else if (isNaN(m)) {
                // just year passed in - presume we want the displayedMonth
                t = new Date(y, this.displayedMonth, 1);
            } else if (isNaN(y)) {
                // just month passed in - presume we want the displayedYear
                t = new Date(this.displayedYear, m, 1);
            } else {
                // year and month passed in - that's the date we want!
                t = new Date(y, m, 1);
            }
            // check if the desired date is within the range of our defined minDate and maxDate
            if (t.getTime() < s.getTime()) {
                t = s;
            } else if (t.getTime() > e.getTime()) {
                t = e;
            }
            var oldMonth = this.displayedMonth;
            var oldYear = this.displayedYear;
            this.displayedMonth = t.getMonth();
            this.displayedYear = t.getFullYear();
            var tempDate = t;
            if (rerender && (this.displayedMonth != oldMonth || this.displayedYear != oldYear)) {
                this._renderCalendar(this, tempDate);
                this._dateValue = tempDate;
                if (this.model.monthChanged)
                    this._trigger("monthChanged", [this.displayedMonth, this.displayedYear]);
            }
        },
        _clearSelected: function () {
            this.numSelected = 0;
            $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false).parent().removeClass('selectedWeek');
        },
        _hideOtherMonths: function (sfCalendar) {
            $('td.other-month', sfCalendar).css("visibility", "hidden");
        },
        _showOtherMonths: function (sfCalendar) {
            $('td.other-month', sfCalendar).css({ 'visibility': 'visible' });
        },


        /*---------------------Region Rendering simple calender----------------------------------------------------------*/

        _createCalender: function () {
            ej.buildTag("div.e-header")
                    .append(ej.buildTag("span.e-prev").append(ej.buildTag('a.e-icon e-left-arrow e-icon-' + this.model.cssClass + '').attr({ 'role': 'button' })))
                    .append(ej.buildTag("span.e-text").text(this._asString(this._calendarDate, this.model.headerFormat)).attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', 'role': 'heading' }))
                    .append(ej.buildTag("span.e-next").append(ej.buildTag('a.e-icon e-right-arrow e-icon-' + this.model.cssClass + '').attr({ 'role': 'button' })))
                    .appendTo(this.sfCalendar);
            this._enableHeader(this._showHeader);
            var table = ej.buildTag("table.e-dp-viewdays", "", {}, { 'cellspacing': 2 }).data("e-table", "data").attr({ 'role': 'grid', 'aria-activedescendant': '' });
            this.sfCalendar.append(table);

            this._renderCalendar(this);
            this._startLevel(this.model.startLevel);
            ej.buildTag("div.e-footer")
                .append(ej.buildTag("span.e-footer-icon"))
                .append(ej.buildTag("span.e-footer-text"))
                .appendTo(this.sfCalendar);
            $('.e-footer-text', this.sfCalendar).html(this.model.buttonText);
            this._enableFooter(this.model.showFooter);
        },
        _enableHeader: function (show) {
            if (show) $(".e-header", this.sfCalendar).show();
            else $(".e-header", this.sfCalendar).hide();
        },
        _enableFooter: function (show) {
            if (show) $('.e-footer', this.sfCalendar).show();
            else $('.e-footer', this.sfCalendar).hide();
        },

        //Date picker navigation arrows enable/disable operations
        _checkArrows: function (min, max) {
            this._preArrowCondition(min, this.model.minDate.getFullYear());
            this._nextArrowCondition(max, this.model.maxDate.getFullYear());
        },
        _checkDateArrows: function () {
            this._preArrowCondition(this._tempMinDate, this.model.minDate);
            this._nextArrowCondition(this._tempMaxDate, this.model.maxDate);
        },
        _preArrowCondition: function (val1, val2) {
            if (val1 <= val2) this.sfCalendar.find(".e-left-arrow").addClass("e-disable").attr({ "aria-disabled": true });
            else this.sfCalendar.find(".e-left-arrow").removeClass("e-disable").attr({ "aria-disabled": false });
        },
        _nextArrowCondition: function (val1, val2) {
            if (val1 >= val2) this.sfCalendar.find(".e-right-arrow").addClass("e-disable").attr({ "aria-disabled": true });
            else this.sfCalendar.find(".e-right-arrow").removeClass("e-disable").attr({ "aria-disabled": false });
        },

        _previousNextHandler: function (event) {
            if (this.model.readOnly || !this.model.enabled) return false;
            event.preventDefault();
            var element = ($(event.target).is('a')) ? $(event.target.parentNode) : $(event.target);
            var progress = element.hasClass('e-prev') ? true : false;
            this._processNextPrevDate(progress);
        },
        _processNextPrevDate: function (progress) {
            if (progress && this.sfCalendar.find(".e-left-arrow").hasClass("e-disable")) return false;
            else if (!progress && this.sfCalendar.find(".e-right-arrow").hasClass("e-disable")) return false;

            var currentTable = $("table", this.sfCalendar), temp;
            var tClassName = currentTable.get(0).className;
            switch (tClassName) {
                case 'e-dp-viewdays':
                    var step = this.model.stepMonths;
                    if (progress) {
                        if (this._dateValue <= this.model.minDate) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (this._dateValue >= this.model.maxDate) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    this._addMonths(this._dateValue, (progress ? -step : step));
                    this._renderCalendar(this, this._dateValue);
                    $('.e-text', this.sfCalendar).text(this._asString(this._dateValue, this.model.headerFormat));
                    this._addFocus('day', this._hoverDate);
                    break;
                case 'e-dp-viewmonths':
                    if (progress) {
                        if (this._dateValue.getFullYear() <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (this._dateValue.getFullYear() >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    this._addYears(this._dateValue, (progress ? -1 : 1));
                    this._renderCalendar(this, this._dateValue);
                    temp = this._dateValue.getFullYear();
                    $('.e-text', this.sfCalendar).text(temp);
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-months').hide();
                    $($(currentTable).find('.e-datepicker-months')).show();
                    this._addFocus('month', this._hoverMonth);
                    this._checkArrows(temp, temp);
                    break;
                case 'e-dp-viewyears':
                    if (progress) {
                        if (parseInt($('td.e-year-first:first').text()) <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (parseInt($('td.e-year-last:first').prev().text()) >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    this._dateValue.setFullYear(this._dateValue.getFullYear() + (progress ? -10 : 10));
                    this._renderCalendar(this, this._dateValue);
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 10) + 1);
                    $('.e-text').text((setYear + 1) + ' - ' + (setYear + 10));
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-years').hide();
                    $($(currentTable).find('.e-datepicker-years')).show();
                    this._addFocus('year', this._hoverYear);
                    this._checkArrows(setYear + 1, setYear + 10);
                    break;
                case 'e-dp-viewallyears':
                    var headYears;
                    if (progress) {
                        headYears = parseFloat($('td.e-allyear-first', currentTable.get(0)).text().split('-')[1]);
                        if (headYears <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        } else {
                            this._flag = true;
                        }

                    } else {
                        headYears = parseFloat($('td.e-allyear-last', currentTable.get(0)).prev().text().split('-')[1]);
                        if (headYears >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        } else
                            this._flag = true;
                    }
                    this._dateValue.setFullYear(headYears + (progress ? -10 : 10));
                    this._renderCalendar(this, this._dateValue);
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 100) + 1);
                    temp = parseFloat($('td.e-allyear-last', currentTable.get(0)).prev().text().split('-')[1]);
                    $('.e-text', this.sfCalendar).text((setYear + 1) + ' - ' + temp);
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-allyears').hide();
                    $($(currentTable).find('.e-datepicker-allyears')).show();
                    this._addFocus('allyear', this._hoverAllYear);
                    this._checkArrows(setYear + 1, temp);
                    break;
            }
            this._layoutChanged();
        },
        _addFocus: function (selection, index) {
            var cls = 'e-current-' + selection;
            if (selection == 'day') cls = 'current-month';
            var items = this.sfCalendar.find('tbody tr td.' + cls);
            var cell = items[index];
            if (!cell) cell = items.last();
            this.sfCalendar.find('table td').removeClass("e-state-hover");
            $(cell).addClass("e-state-hover");
            return $(cell);
        },
        _setFocusByName: function (name, value) {
            var allValues = this.sfCalendar.find('tbody tr td.e-current-' + name), index, cell;
            $(allValues).each(function (i, ele) {
                if (parseInt(ele.innerHTML) == parseInt(value)) {
                    index = i;
                    return;
                }
            });
            cell = allValues[index];
            if (!cell) cell = allValues.last();
            this.sfCalendar.find('table td').removeClass("e-state-hover");
            $(cell).addClass("e-state-hover");
            return index;
        },
        _getHeaderTxt: function () {
            return this.sfCalendar.find(".e-text").text();
        },
        _forwardNavHandler: function (event) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (event) event.preventDefault();

            var currentTable = $("table", this.sfCalendar);
            var tclassName = $("table", this.sfCalendar).get(0).className, proxy = this, headerTxt;
            switch (tclassName) {
                case 'e-dp-viewdays':
                    this._hoverMonth = this._getDateObj(currentTable.find(".e-state-hover")).getMonth() ||
                                this._getDateObj(currentTable.find(".e-active")).getMonth() || 0;
                    this._startLevel("Year");
                    this._addFocus('month', this._hoverMonth);
                    break;
                case 'e-dp-viewmonths':
                    headerTxt = this._getHeaderTxt();
                    this._startLevel("Decade");
                    this._hoverYear = this._setFocusByName('year', headerTxt);
                    break;
                case 'e-dp-viewyears':
                    headerTxt = this._getHeaderTxt();
                    this._startLevel("Century");
                    this._hoverAllYear = this._setFocusByName('allyear', headerTxt);
                    break;
            }
            this._layoutChanged();
        },
        _backwardNavHandler: function (event) {
            if (this.model.readOnly || !this.model.enabled) return false;
            var element;
            if (event.type) {
                event.preventDefault();
                element = $(event.target);
            }
            else element = event;
            var cTable = $("table", this.sfCalendar), temp;
            var tclassName = $("table", this.sfCalendar).get(0).className, proxy = this;
            switch (tclassName) {
                case 'e-dp-viewmonths':
                    cTable.removeClass("e-dp-viewmonths").addClass("e-dp-viewdays");
                    this._dateValue = new Date(this._dateValue.getFullYear(), parseInt($(element).attr('index')), 1);
                    this._renderCalendar(this, this._dateValue);
                    $('tbody', cTable).not('.e-datepicker-days,.e-week-header').hide();
                    $($(cTable).find('.e-datepicker-days,.e-week-header')).fadeIn("fast", function () {
                        proxy._addFocus('day', proxy._hoverDate || 0);
                    });
                    $('.e-text', this.sfCalendar).text(this._asString(this._dateValue, this.model.headerFormat));
                    break;
                case 'e-dp-viewyears':
                    cTable.removeClass("e-dp-viewyears").addClass("e-dp-viewmonths");
                    this._dateValue.setFullYear(parseInt(element.text()));
                    this._renderCalendar(this, this._dateValue);
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-months').hide();
                    $($(cTable).find('.e-datepicker-months')).fadeIn("fast", function () {
                        proxy._addFocus('month', proxy._hoverMonth || 0);
                    });
                    temp = element.text();
                    $('.e-text', this.sfCalendar).text(temp);
                    this._checkArrows(temp, temp);
                    break;
                case 'e-dp-viewallyears':
                    var headYears = element.text().split('-');
                    cTable.removeClass("e-dp-viewallyears").addClass("e-dp-viewyears");
                    if (headYears[0] < this.model.minDate.getFullYear()) headYears[0] = this.model.minDate.getFullYear().toString();
                    else if (headYears[0] > this.model.maxDate.getFullYear()) headYears[0] = this.model.maxDate.getFullYear().toString();
                    this._renderCalendar(this, (new Date(headYears[0], 0, 1)));
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-years').hide();
                    $($(cTable).find('.e-datepicker-years')).fadeIn("fast", function () {
                        proxy._addFocus('year', proxy._hoverYear || 0);
                    });
                    $('.e-text', this.sfCalendar).text(headYears[0] + '-' + headYears[1]);
                    this._checkArrows(headYears[0], headYears[1]);
                    break;
                default:
                    this._clearSelected();
                    this.sfCalendar.find('table td').removeClass("e-state-hover");
                    element.not('td.other-month').addClass('e-active e-state-hover').attr('aria-selected', true);
                    //this.sfCalendar.find('table').attr({ 'aria-activedescendant': element[0].id, 'aria-describedby': element[0].id });                  
                    this._hoverDate = this._getDateObj(element).getDate() - 1;
                    this._dateValue = new Date(element.attr('date'));
                    break;
            }
            this._layoutChanged();
        },

        _startLevel: function (start) {
            var cTable = $("table", this.sfCalendar);
            var headerText = $(".e-text", this.sfCalendar), s, e;
            switch (start) {
                case "Decade":
                    cTable.removeClass("e-dp-viewmonths e-dp-viewdays").addClass("e-dp-viewyears");
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-years').hide();
                    $($(cTable).find('.e-datepicker-years')).show();
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 10) + 1);
                    s = setYear + 1;
                    e = setYear + 10;
                    headerText.text(s + ' - ' + e);
                    this._checkArrows(s, e);
                    break;
                case "Century":
                    this._renderCalendar(this, this._dateValue);
                    cTable.removeClass("e-dp-viewyears e-dp-viewdays e-dp-viewmonths").addClass("e-dp-viewallyears");
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-allyears').hide();
                    $($(cTable).find('.e-datepicker-allyears')).show();
                    s = parseFloat($('td.e-allyear-first', cTable.get(0)).text().split('-')[1]) + 1;
                    e = parseFloat($('td.e-allyear-last', cTable.get(0)).prev().text().split('-')[1]);
                    var headYears = s + '-' + e;
                    headerText.text(headYears);
                    this._checkArrows(s, e);
                    break;
                case "Year":
                    cTable.removeClass("e-dp-viewdays").addClass("e-dp-viewmonths");
                    $('tbody,tr.e-week-header', cTable).hide();
                    $($(cTable).find('.e-datepicker-months')).show();
                    s = this._dateValue.getFullYear();
                    headerText.text(s);
                    this._checkArrows(s, s);
                    break;
            }
        },
        _depthLevel: function (depth) {
            var calendarTable = this.sfCalendar;
            switch (depth) {
                case "Year":
                    $(calendarTable.find('.e-current-year,.e-current-allyear')).bind("click", $.proxy(this._backwardNavHandler, this));
                    this._on($('.e-current-month', this.sfCalendar), "click", $.proxy(this._onDepthSelectHandler, this));
                    break;
                case "Decade":
                    $(calendarTable.find('.e-current-allyear')).bind("click", $.proxy(this._backwardNavHandler, this));
                    $('.e-current-year', this.sfCalendar).bind("click", $.proxy(this._onDepthSelectHandler, this));
                    break;
                case "Century":
                    $(calendarTable.find('.e-current-allyear')).bind("click", $.proxy(this._onDepthSelectHandler, this));
                    break;
            }
        },
        _onDepthSelectHandler: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if ($(e.target).hasClass("e-current-month"))
                this._dateValue = new Date(this._dateValue.setMonth(parseInt(e.target.attributes["index"].value)));
            else if ($(e.target).hasClass("e-current-year"))
                this._dateValue = new Date(this._dateValue.setFullYear(parseInt(e.target.innerHTML)));
            else if ($(e.target).hasClass("e-current-allyear"))
                this._dateValue = new Date(this._dateValue.setFullYear(parseInt(e.target.innerHTML)));
            this._onSetCancelDateHandler(e);
        },
        //Generates datepicker months view
        _datepickerMonths: function (tbody, calendarTable, currentDate) {
            var dc = function (a) {
                return document.createElement(a);
            };
            var month = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(dc('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(dc('td'))
                        .addClass('e-current-month e-state-default')
                        .attr('index', month)
                        .html(this.Date.abbrMonthNames[month++]);
                    if (currentDate.getFullYear() < this.model.minDate.getFullYear() || currentDate.getFullYear() > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate e-disable');
                        td.removeClass('e-current-month');
                    }
                    else if ((currentDate.getFullYear() <= this.model.minDate.getFullYear() && month < this.model.minDate.getMonth() + 1) ||
                        (currentDate.getFullYear() >= this.model.maxDate.getFullYear() && month > this.model.maxDate.getMonth() + 1)) {
                        td.addClass('e-hidedate e-disable');
                        td.removeClass('e-current-month');
                    }
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
            var s = currentDate.getFullYear();
            this._checkArrows(s, s);
        },
        //Generates datepicker years view
        _datepickerYears: function (tbody, calendarTable, currentYear) {
            var dc = function (a) {
                return document.createElement(a);
            };
            var Year = parseInt(currentYear) - ((parseInt(currentYear) % 10) + 1);
            var years = [];
            for (var j = 0; j < 12; j++) {
                years.push(Year + j);
            }
            var year = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(dc('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(dc('td'));
                    if (year == 0) {
                        if (parseInt(years[year]) <= this.model.minDate.getFullYear())
                            td.css("visibility", "hidden");
                        else
                            td.css("visibility", "visible");
                        td.addClass('e-year-first e-current-year ');
                    } else if (year == 11) {
                        if (parseInt(years[year]) >= this.model.maxDate.getFullYear()) {
                            td.css("visibility", "hidden");
                        }
                        else {
                            td.css("visibility", "visible");
                        }
                        td.addClass('e-year-last e-current-year ');
                    } else
                        td.addClass('e-current-year e-state-default');
                    if (years[year] < this.model.minDate.getFullYear() || years[year] > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate e-disable');
                        td.removeClass('e-current-year');
                    }
                    td.html(years[year++]);
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
            this._checkArrows(years[0], years[years.length]);
        },
        //Generates datepicker all-years view
        _datepickerAllYears: function (tbody, calendarTable, currentYear) {
            var Year = parseInt(currentYear) - ((parseInt(currentYear) % 100) + 10);
            var headYear = Year;
            var years = [];

            for (var j = 0; j < 12; j++) {
                years.push(parseInt(Year) + "-\n" + parseInt(Year + 9));
                Year = Year + 10;
            }
            var year = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(document.createElement('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(document.createElement('td'));
                    if (year == 0) {
                        if (parseInt(years[year].split('-\n')[1]) <= this.model.minDate.getFullYear())
                            td.css("visibility", "hidden");
                        else
                            td.css("visibility", "visible");
                        td.addClass('e-allyear-first e-current-allyear ');
                    } else if (year == 11) {
                        if (parseInt(years[year].split('-\n')[0] - 1) >= this.model.maxDate.getFullYear()) {
                            td.css("visibility", "hidden");
                        } else {
                            td.css("visibility", "visible");
                        }
                        td.addClass('e-allyear-last e-current-allyear ');
                    } else
                        td.addClass('e-current-allyear e-state-default');
                    if (parseInt(years[year].split('-\n')[1]) < this.model.minDate.getFullYear() || parseInt(years[year].split('-\n')[0]) > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate e-disable');
                        td.removeClass('e-current-allyear');
                    }
                    td.html(years[year++]);
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
        },
        _renderHeader: function (dpObject) {
            var thead = $(document.createElement('thead'));
            if (dpObject.model.dayHeaderFormat != "ShowHeaderNone") {
                var headRow = ej.buildTag("tr.e-week-header").attr('role', 'columnheader');
                for (var i = this.Date.firstDayOfWeek; i < this.Date.firstDayOfWeek + 7; i++) {
                    var weekday = i % 7;
                    var day = this.Date.dayNames[weekday];
                    var headerday;
                    if (dpObject.model.dayHeaderFormat == "ShowHeaderShort")
                        headerday = day.substr(0, 3);
                    else if (dpObject.model.dayHeaderFormat == "ShowHeaderLong") headerday = day;
                    else headerday = day.substr(0, 2);
                    var th = ej.buildTag("th", "", {}, { 'scope': 'col', 'abbr': day, 'date': day, 'title': this._formatter(day, "dddd"), 'class': (weekday == 0 || weekday == 6 ? 'weekend' : 'weekday') })
                            .html(headerday);
                    headRow.append(th);
                }
            };
            return thead.append(headRow);
        },
        // Render a calendar table into any matched elements.
        _renderCalendar: function (dpObject, date) {
            var proxy = this, today;
            dpObject = $.extend({}, ej.DatePicker.prototype.defaults, dpObject);
            this.Date.firstDayOfWeek = this.model.startDay;
            if (date) today = date;
            else if (this._calendarDate) today = this._calendarDate;
            else today = proxy._zeroTime(new Date());
            var calendarTable = $('table', this.sfCalendar);
            calendarTable.empty();
            // header render section
            calendarTable.append(this._renderHeader(dpObject));
            //all years render section
            var tbody = ej.buildTag('tbody.e-datepicker-allyears', "", { 'display': 'none' }, {});
            this._datepickerAllYears(tbody, calendarTable, today.getFullYear());
            //all years render section
            tbody = ej.buildTag("tbody.e-datepicker-years", "", { 'display': 'none' }, {});
            this._datepickerYears(tbody, calendarTable, today.getFullYear());
            var month = dpObject.model.month == undefined ? today.getMonth() : dpObject.model.month;
            var year = dpObject.model.year || today.getFullYear();
            var currentDate = (new Date(year, month, 1, 0, 0, 0));
            var firstDayOffset = this.Date.firstDayOfWeek - currentDate.getDay() + 1;
            if (firstDayOffset > 1) firstDayOffset -= 7;
            var weeksToDraw = Math.ceil(((-1 * firstDayOffset + 1) + this._getDaysInMonth(currentDate)) / 7);
            this._addDays(currentDate, (firstDayOffset - 1));
            var newdate = proxy._zeroTime(new Date());
            var selected = this._calendarDate;
            tbody = ej.buildTag('tbody.e-datepicker-months', "", { 'display': 'none' }, {});
            // month rennder section
            this._datepickerMonths(tbody, calendarTable, today);
            //days render section
            tbody = ej.buildTag('tbody.e-datepicker-days', "", { 'display': 'none' }, {});
            var w = 0, _first = true, _last = true;
            while (w++ < weeksToDraw) {
                var r = jQuery(document.createElement('tr'));
                for (var i = 0; i < 7; i++) {
                    var thisMonth = currentDate.getMonth() == month;
                    var d = $(document.createElement('td'))
                        .text(currentDate.getDate() + '')
                        .attr({
                            //'id':this._formatter(currentDate, "ddd MMM dd yyyy").replace(/ /g,''),
                            'date': currentDate.toDateString(),
                            'title': this._formatter(currentDate, "ddd MMM dd yyyy"),
                            'aria-selected': false,
                            'role': 'gridcell'

                        })
                        .addClass((thisMonth ? 'current-month e-state-default ' : 'other-month e-state-default ') +
                            (this._isWeekend(currentDate) ? (this._ejHLWeekEnd ? 'e-dp-weekend weekend ' : 'weekend ') : 'weekday ') +
                            (thisMonth && currentDate.getTime() == newdate.getTime() ? 'today ' : ''));
                    if (selected.getTime() == currentDate.getTime()) {
                        d.addClass('e-active').attr({ 'aria-selected': true });
                        //this.sfCalendar.find('table').attr({ 'aria-activedescendant': d[0].id, 'aria-describedby': d[0].id });
                        if (!this._hoverDate) {
                            d.addClass('e-state-hover');
                            this._hoverDate = currentDate.getDate() - 1;
                        }
                    }
                    var cond = true;
                    if (currentDate < this.model.minDate || currentDate > this.model.maxDate) {
                        d.addClass('e-hidedate e-disable');
                        d.removeClass('current-month other-month');
                        cond = _last = false;
                    }
                    if (thisMonth) {
                        if (cond && _first) {
                            this._tempMinDate = currentDate;
                            _first = false; _last = true;
                        }
                        if (_last) this._tempMaxDate = currentDate;
                    }
                    r.append(d);
                    currentDate = new Date(currentDate.getFullYear(), currentDate.getMonth(), currentDate.getDate() + 1, 0, 0, 0);
                }
                tbody.append(r);
            }
            calendarTable.append(tbody);
            $(tbody).fadeIn("fast");
            if (this.model.startLevel === this.model.depthLevel)
                this._depthLevel(this.model.depthLevel);
            else if (this.model.depthLevel != "Month" && this.model.depthLevel != "") {
                if (this.model.startLevel == "Century")
                    this._depthLevel(this.model.depthLevel);
                else if (this.model.startLevel == "Decade" && this.model.depthLevel != "Century")
                    this._depthLevel(this.model.depthLevel);
                else if (this.model.startLevel == "Year" && this.model.depthLevel != "Decade" && this.model.depthLevel != "Century")
                    this._depthLevel(this.model.depthLevel);
                else {
                    this._on(calendarTable.find('.current-month,.other-month,.e-current-month,.e-current-year,.e-current-allyear'), "click", $.proxy(this._backwardNavHandler, this));
                    this._on(calendarTable.find('.current-month , .other-month'), "click", $.proxy(this._onSetCancelDateHandler, this));
                }
            }
            else {
                this._on(calendarTable.find('.current-month,.other-month,.e-current-month,.e-current-year,.e-current-allyear'), "click", $.proxy(this._backwardNavHandler, this));
                this._on(calendarTable.find('.current-month , .other-month'), "click", $.proxy(this._onSetCancelDateHandler, this));
            }

            if (this.model.showOtherMonths)
                this._showOtherMonths(this.sfCalendar);
            else
                this._hideOtherMonths(this.sfCalendar);
            this._checkDateArrows();
        },

        /*---------------------Keyboard Support----------------------------------------------------------*/

        _keyboardNavigation: function (e) {
            if ((this._isOpen) && (e.keyCode == 37 || e.keyCode == 38 || e.keyCode == 39 || e.keyCode == 40 || e.keyCode == 13 || e.keyCode == 36 || e.keyCode == 35)) {
                e.preventDefault && e.preventDefault();
                var t = { row: null, col: null };

                t.col = this.sfCalendar.find('tbody tr td.e-state-hover').index();
                t.row = this.sfCalendar.find('tbody tr td.e-state-hover').parent().index();

                t.col = (t.col != -1) ? t.col + 1 : this.sfCalendar.find('tbody tr td.e-active').index() + 1;
                t.row = (t.row != -1) ? t.row + 1 : this.sfCalendar.find('tbody tr td.e-active').parent().index() + 1;

                var tableClass = this.sfCalendar.find('table')[0].className, next, rowLength = 3, colLength = 4;
                switch (tableClass) {
                    case "e-dp-viewallyears":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "yearall", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverAllYear = this.sfCalendar.find('tbody.e-datepicker-allyears tr td').index(next);
                        break;
                    case "e-dp-viewyears":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "year", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverYear = this.sfCalendar.find('tbody.e-datepicker-years tr td').index(next);
                        break;
                    case "e-dp-viewmonths":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "month", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverMonth = this.sfCalendar.find('tbody.e-datepicker-months tr td').index(next);
                        break;
                    case "e-dp-viewdays":
                        rowLength = this.sfCalendar.find('tbody.e-datepicker-days tr').length, colLength = 7;
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "day", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverDate = this._getDateObj(next).getDate() - 1;
                        break;
                }
                if (!e.ctrlKey) {
                    this.sfCalendar.find('table td').removeClass("e-state-hover");
                    next.addClass("e-state-hover");
                }
            }
            else if (!this.model.displayInline && (e.keyCode == 27 || e.keyCode == 9)) { this.hide(); e.stopPropagation(); }
            else if (e.altKey && e.keyCode == 40) { this.show(); return false; }
        },
        _changeRowCol: function (t, key, rows, cols, target, ctrlKey) {
            var eleClass, cls = { parent: null, child: null };
            switch (target) {
                case "day": eleClass = "tbody.e-datepicker-days tr td.current-month";
                    cls.parent = ".e-datepicker-days", cls.child = ".current-month";
                    break;
                case "month": eleClass = "tbody.e-datepicker-months tr td.e-current-month";
                    cls.parent = ".e-datepicker-months", cls.child = ".e-current-month";
                    break;
                case "year": eleClass = "tbody.e-datepicker-years tr td.e-current-year";
                    cls.parent = ".e-datepicker-years", cls.child = ".e-current-year";
                    break;
                case "yearall": eleClass = "tbody.e-datepicker-allyears tr td.e-current-allyear";
                    cls.parent = ".e-datepicker-allyears", cls.child = ".e-current-allyear";
                    break;
            }
            if (t.row <= 0 && t.col <= 0)
                return this.sfCalendar.find(eleClass + ':first');
            var cell, proxy = this;
            switch (key) {
                case 36:         // Home Key
                    return this.sfCalendar.find(eleClass + ':first');
                    break;
                case 35:         // End Key
                    return this.sfCalendar.find(eleClass + ':last');
                    break;
                case 38:         // Up Key
                    if (ctrlKey) {
                        this._forwardNavHandler();
                    }
                    else if (t.row > 1) {
                        t.row -= 1;
                    }
                    else {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):last');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):last');
                    }
                    return cell;
                    break;
                case 37:        // Left Key
                    if (ctrlKey) {
                        this._processNextPrevDate(true);
                        return this.sfCalendar.find('tbody tr td.e-state-hover');
                    }
                    else if (t.col > 1)
                        t.col -= 1;
                    else if (t.row > 1) {
                        t = { row: t.row - 1, col: cols }
                    }
                    else {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':last');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':last');
                    }
                    return cell;
                    break;
                case 39:         // Right Key
                    if (ctrlKey) {
                        this._processNextPrevDate(false);
                        return this.sfCalendar.find('tbody tr td.e-state-hover');
                    }
                    else if (t.col < cols)
                        t.col += 1;
                    else if (t.row < rows) {
                        t = { row: t.row + 1, col: 1 }
                    }
                    else {
                        this._processNextPrevDate(false);
                        cell = this.sfCalendar.find(eleClass + ':first');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        this._processNextPrevDate(false);
                        cell = this.sfCalendar.find(eleClass + ':first');
                    }
                    return cell;
                    break;
                case 40:        // Down Key
                    if (!ctrlKey) {
                        if (t.row < rows) {
                            t.row += 1;
                        }
                        else {
                            this._processNextPrevDate(false);
                            cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):first');
                            return cell;
                        }
                        cell = this._getCell(t, cls);
                        if (cell.length <= 0) {
                            this._processNextPrevDate(false);
                            cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):first');
                        }
                        return cell;
                        break;
                    }
                case 13:    // Enter Key
                    var tclassName = $("table", this.sfCalendar).get(0).className;
                    this._backwardNavHandler(this._getCell(t, cls));
                    if (tclassName == "e-dp-viewdays")
                        this._onSetCancelDateHandler({ type: null });
                    break;
            }
            return this._getCell(t, cls);
        },
        _getCell: function (t, cls) {
            return this.sfCalendar.find('tbody' + cls.parent + ' tr:nth-child(' + t.row + ') td' + cls.child + ':nth-child(' + t.col + ')');
        },
        _getDateObj: function (element) {
            return new Date(element.attr("date"));
        },
        _touchCalendar: function (e) {
            var tableClass = this.sfCalendar.find('table')[0].className;
            switch (e.type) {
                case "pinchin":
                    if (tableClass != "e-dp-viewdays")
                        this._keyboardNavigation({ keyCode: 13 });
                    break;
                case "pinchout":
                    if (tableClass != "e-dp-viewallyears")
                        this._forwardNavHandler();
                    break;
                case "swipeleft":
                    this._processNextPrevDate(false);
                    break;
                case "swiperight":
                    this._processNextPrevDate(true);
                    break;
            }
        },

        /*---------------------client side methods----------------------------------------------------------*/

        /**
        * Opens the datepicker popup  
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.show(); // shows the datepicker popup
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
	    * // shows the datepicker popup
        * $("#datepicker").ejDatePicker("show");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        show: function () {
            if (this._isOpen) return false;
            var proxy = this;
            if (!this.model.enabled) return;
            if (!this.model.displayInline) this._setDatePickerPosition();
            this.sfCalendar.attr({ 'aria-hidden': 'false' })
            this.sfCalendar.slideDown(200, "easeOutQuad", function () {
                proxy._isOpen = true;
                if (!proxy.model.displayInline)
                    $(document).bind("mousedown", $.proxy(proxy._onDocumentClick, proxy));
            });
            if (this._textboxVal() !== "") this._updateInputVal();
            if (this.model.open)
                this._trigger("open", { previousValue: this._prevDate, value: this._formatter(this.model.value, this.model.dateFormat) });

        },

        /**
        * Hides the datepicker popup, if in opended state.
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.hide(); // hides the datepicker popup
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
	    * // hides the datepicker popup
        * $("#datepicker").ejDatePicker("hide");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        hide: function () {
            if (!this._isOpen) return false;
            var proxy = this;
            this.sfCalendar.attr({ 'aria-hidden': 'true' })
            this.sfCalendar.slideUp(100, "easeOutQuad", function () {
                proxy._isOpen = false;
                $(document).unbind("mousedown", $.proxy(proxy._onDocumentClick, proxy));
                proxy._setWaterMark();
            });
            if (this._textboxVal() != "" && !this.element.hasClass("e-watermark")) this._validateInputVal();
            if (this.model.close)
                this._trigger("close", { previousValue: this._prevDate, value: this._formatter(this.model.value, this.model.dateFormat) });
        },

        /**
        * Enables the datepicker control
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.enable(); // enables the datepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
	    * // enables the datepicker
        * $("#datepicker").ejDatePicker("enable");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        enable: function () {
            this.model.enabled = true;
            this.element.removeClass('e-disable').attr({ "aria-disabled": false });
            if (this.dateIcon) this.dateIcon.removeClass('e-disable').attr({ "aria-disabled": false });
            this.element.removeAttr("disabled");
            this.sfCalendar.removeClass('e-disable').attr({ "aria-disabled": false });
        },

        /**
        * Disables the datepicker control
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.disable(); // disables the datepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
	    * // disables the datepicker
        * $("#datepicker").ejDatePicker("disable");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        disable: function () {
            this.model.enabled = false;
            this.element.addClass('e-disable').attr({ "aria-disabled": true });
            if (this.dateIcon) this.dateIcon.addClass('e-disable').attr({ "aria-disabled": true });
            this.element.attr("disabled", "disabled");
            this.sfCalendar.addClass('e-disable').attr({ "aria-disabled": true });
            if (this._isOpen) {
                if (this.element.is(':input')) this.element.blur();
                if (!this.model.displayInline) this.hide();
            }
        },

        /**
        * Returns the current date value in the datepicker control
		* @return Date value
		* @example 
		* * &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.getValue(); // returns the date value
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
	    * // returns the date value
        * $("#datepicker").ejDatePicker("getValue");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        getValue: function () { return this._formatter(this.model.value, this.model.dateFormat); },

        /*----------------------------------------region - Event Binding and Handlers---------------------------------------- */

        _wireEvents: function () {
            this._on($('.e-text', this.sfCalendar), "click", $.proxy(this._forwardNavHandler, this));
            this._on($('.e-next', this.sfCalendar), "click", $.proxy(this._previousNextHandler, this));
            this._on($('.e-prev', this.sfCalendar), "click", $.proxy(this._previousNextHandler, this));
            if (this.element.is(":input")) {
                this._on(this.element, "blur", this._onFocusOut);
                this._on(this.element, "focus", this._onFocusIn);
                this._on(this.element, "keyup", this._keyUp);
            }
            if (this.model.showFooter) {
                this._on($('.e-footer', this.sfCalendar), "click", this._setCurrDate);
            }

            this._on(this.sfCalendar, "pinchin pinchout swipeleft swiperight", $.proxy(this._touchCalendar, this));
        },
        _bindDateButton: function () {
            this._on($(this.dateIcon, this.wrapper), "mousedown", $.proxy(this._showDatePopUp, this));
            this.element.unbind("mousedown", $.proxy(this._showDatePopUp, this));
        },
        _bindInputEvent: function () {
            this._off($(this.dateIcon, this.wrapper), "mousedown", $.proxy(this._showDatePopUp, this));
            if (this.element.is(":input") && !this.model.displayInline) {
                this.element.bind("mousedown", $.proxy(this._showDatePopUp, this));
            }
        },
        _onFocusOut: function (e) {
            var calenderDate = this._formatter(this._calendarDate, this.model.dateFormat);
            var currDate = this._formatter(this._currentDay(), this.model.dateFormat);
            this._isFocused = false;
            var val = this._validateValue();
            if (val != null && !this.model.strictMode) {
                this._validateMinMaxDate();
                var _currentVal = this._textboxVal();
                if (this._preValue != _currentVal) {
                    this._triggerChangeEvent(e);
                }
            }
            else if (val == null && !this.model.strictMode) {
                if (this._preTxtValue == null)
                    this.element.addClass("e-watermark").val(this.model.waterMarkText);
                else
                    this.element.val(calenderDate);
                this._triggerChangeEvent(e);
            }
            else
                this.isValidState = false;
            this.wrapper.removeClass("e-focus");
            if ((!this._isOpen || this.model.displayInline) && !this._setWaterMark() && this._preValue != this._textboxVal()) this._updateInputVal();
            if (this._textboxVal() != "" && (!this._isOpen || this.model.displayInline)) this._validateInputVal();
            this.element.unbind("keydown", $.proxy(this._keyboardNavigation, this));
            if (this.model.focusOut)
                this._trigger("focusOut");
            this._checkErrorClass();
        },
        _onFocusIn: function (e) {
            e.preventDefault();
            this._isFocused = true;
            this.wrapper.removeClass('e-error').attr('aria-invalid', "false");
            this.isValidState = true;
            this.wrapper.addClass("e-focus");
            if (this.model.readOnly)
                return;
            this._removeWatermark();
            this.element.bind("keydown", $.proxy(this._keyboardNavigation, this));
            if (this.model.focusIn)
                this._trigger("focusIn");
        },
        _removeWatermark: function () {
            if (this.element.val() == this.model.waterMarkText) {
                this.element.val("");
                this.element.removeClass('e-watermark');
            }
        },
        _keyUp: function (e) {
            var _currentVal = this._textboxVal();
            this._prevDate = this._formatter(this.model.value, this.model.dateFormat);
            if (this._preValue != _currentVal)
                this._triggerChangeEvent();
        },
        _textboxVal: function (e) {
            return this.element.hasClass("e-watermark") ? "" : this.element.val();
        },
        _showhidePopup: function () {
            if (!this.model.enabled) return false;
            if (this._isOpen) {
                if (!this._isFocused && this.element.is(':input')) this.element.focus();
                this.hide();
            }
            else {                              // Open
                if (!this._isFocused && this.element.is(':input')) this.element.focus();
                this.show();
            }
        },

        _triggerChangeEvent: function (e) {
            var _currentVal = this._textboxVal();
            this.model.value = this._parseDate(_currentVal);
            var data = { previousDate: this._prevDate, value: _currentVal };
            if (this._preValue != _currentVal) {
                this._preValue = _currentVal;
                if (this.model.change) this._trigger("change", data);
            }
        },

        _triggerSelectEvent: function (e) {
            var val = this._textboxVal();
            if (this._parseDate(val)) {
                var data = { previousDate: this._prevDate, value: val };
                if (this._prevDate != val) {
                    if (this._parseDate(data.value) && (this.model.value >= this.model.minDate && this.model.value <= this.model.maxDate))
                        if (this.model.select) this._trigger("select", data);
                }
            }
        },

        _onDocumentClick: function (e) {
            if (!$(e.target).is(this.popup) && !$(e.target).parents(".e-popup").is(this.popup) &&
                !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-datewidget").is(this.wrapper)) {
                this._showhidePopup();
            }
            else if ($(e.target).is(this.popup) || $(e.target).parents(".e-popup").is(this.popup)) {
                e.preventDefault();
            }
        },
        _showDatePopUp: function (e) {
            if (this.model.readOnly) return;
            e.preventDefault();
            if (!this.model.enabled && this.model.displayInline) return false;
            this._showhidePopup();
        },
        _layoutChanged: function (e) {
            // this event internally used to observe the layout change in "DateTimePicker" control
            if (this._getInternalEvents) this._trigger("layoutChange");
        },
        _setCurrDate: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (e) e.preventDefault();
            var proxy = this;
            this._prevDate = this._formatter(this.model.value, this.model.dateFormat);
            this._dateValue = this._zeroTime(new Date());
            this.model.value = this._calendarDate = this._dateValue;
            this._setDateValue(this.model.value);
            this._triggerSelectEvent(e);
            this._triggerChangeEvent(e);
            this._refreshDatepicker();
            this._changeDayClass();
            this._startLevel(this.model.startLevel);
            this._onSetCancelDateHandler(e);
            this._layoutChanged();
        },
        _changeDayClass: function () {
            var className = this.popup.children("table")[0].className;
            if (className != "e-dp-viewdays") {
                this.popup.children("table").removeClass(className).addClass("e-dp-viewdays");
            }
        },
        //handler for setting dates to input field
        _onSetCancelDateHandler: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (e && e.type) e.preventDefault();
            this.model.value = this._parseDate(this._textboxVal());
            //sets prev date before value gets change to current selected date
            this._prevDate = this._formatter(this.model.value, this.model.dateFormat);
            this._setDateValue(this._dateValue);
            this._triggerSelectEvent(e);
            this._triggerChangeEvent(e);
            if (this.element.is(':input') && !this.model.displayInline) {
                this._showhidePopup();
            }
            if (e && $(e.target).hasClass("other-month"))
                this._refreshDatepicker();
        },
        _closeCalendar: function (ele) {
            if (!ele || ele == this.element) {
                this.sfCalendar.empty().remove();
            }
        },
        //Error class for input value validation
        _checkErrorClass: function () {
            if (this.wrapper) {
                if (this.isValidState) this.wrapper.removeClass("e-error").attr('aria-invalid', "false");
                else this.wrapper.addClass("e-error").attr('aria-invalid', "true");
            }
        }
    });

    /**
	 * Enum for DatePicker dayHeaderFormat	 
	 * @enum {String}
	 * @global 
	 */
    ej.DatePicker.Header = {
        /**  Shows the day header format in short like Sun, Mon, Tue … */
        ShowHeaderNone: "ShowHeaderNone",
        /**  Shows the day header format in min like Su, Mo, Tu … */
        ShowHeaderShort: "ShowHeaderShort",
        /**  Shows the day header format in long like Sunday, Monday, Tuesday … */
        ShowHeaderMin: "ShowHeaderMin",
        /**  Removes the day header */
        ShowHeaderLong: "ShowHeaderLong"
    };

    /**
	 * Enum for DatePicker startLevel and depthLevel	 
	 * @enum {String}
	 * @global 
	 */
    ej.DatePicker.Level = {
        /**  Starts from month level view. */
        Month: "Month",
        /**  Starts from year level view. */
        Year: "Year",
        /**  Starts from year decade level view. */
        Decade: "Decade",
        /**  Starts from century level view.  */
        Century: "Century"
    };

})(jQuery, Syncfusion);

;
/**
* @fileOverview Plugin to craete a Timepicker with the Html input element
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejTimePicker
	* @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires jquery.globalize.js
    * @requires globalize.cultures.min.js
	* @requires ej.web.all.js
	* @classdesc Time selection with the input field.
	* @example 
	* &lt;input type="text" id="timepicker" /&gt;
	* &lt;script&gt;
	* // Create TimePicker
    * $("#timepicker").ejTimePicker();
	* &lt;/script&gt;
	*/

    ej.widget("ejTimePicker", "ej.TimePicker", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _rootCSS: "e-timepicker",
        _setFirst: false,
        type: "editor",

        // default model
        defaults: {
            /**		
			* Specify the CSS class to timepicker to achieve custom theme.
			* @default ""
			* @type {String}
			* @example 
			* // Set the CSS class during initialization. 			
			* 	$("#timepicker").ejTimePicker({  cssClass : "gradient-lime" });
			* @example 
			* //Get or set the rounded corner after initialization:<br>
			*	// Gets the cssClass value.		
			*	$("#timepicker").ejTimePicker("option", "cssClass");			
			*	// Sets the cssClass to timepicker
			*	$("#timepicker").ejTimePicker("option", "cssClass",  "gradient-lime" );
			 * @memberof ejTimePicker
			* @instance
			*/
            cssClass: "",

            /**		
			* Defines the time format displayed in the TimePicker.
			* @default "h:mm tt"
			* @type {String}
			* @example 
			* // Set the timeFormat during initialization. 			
			* 	$("#timepicker").ejTimePicker({  timeFormat : "h:mm:ss tt" });
			* @example 
			* //Get or set the timeFormat after initialization:<br>
			*	// Gets the timeFormat value.		
			*	$("#timepicker").ejTimePicker("option", "timeFormat");			
			*	// Sets the timeFormat to timepicker
			*	$("#timepicker").ejTimePicker("option", "timeFormat",  "h:mm:ss tt" );
			 * @memberof ejTimePicker
			* @instance
			*/
            timeFormat: "",

            /**		
			* Sets a specified time value on the TimePicker.
			* @default null
			* @type {String | DateObject}
			* @example 
			* // Set the time value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  value : "5:10 PM" });
			* @example 
			* //Get or set the value after initialization:<br>
			*	// Gets the time value.		
			*	$("#timepicker").ejTimePicker("option", "value");			
			*	// Sets the time value to timepicker
			*	$("#timepicker").ejTimePicker("option", "value",  "5:10 PM" );
			 * @memberof ejTimePicker
			* @instance
			*/
            value: null,

            /**		
			* Defines the localization culture for TimePicker.
			* @default "en-US"
			* @type {String}
			* @example 
			* // Set the localize value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  localize : "de-DE" });
			* @example 
			* //Get or set the localize value after initialization:<br>
			*	// Gets the localize value.		
			*	$("#timepicker").ejTimePicker("option", "localize");			
			*	// Sets the localize value to timepicker
			*	$("#timepicker").ejTimePicker("option", "localize",  "de-DE" );
			 * @memberof ejTimePicker
			* @instance
			*/
            localize: "en-US",

            /**		
			* Indicates that the timepicker value can only be read.
			* @default true
			* @type {Boolean}
			* @example 
			* // Set the readOnly value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  readOnly : false });
			* @example 
			* //Get or set the readOnly value after initialization:<br>
			*	// Gets the readOnly value.		
			*	$("#timepicker").ejTimePicker("option", "readOnly");			
			*	// Sets the readOnly value to timepicker
			*	$("#timepicker").ejTimePicker("option", "readOnly",  false );
			 * @memberof ejTimePicker
			* @instance
			*/
            readOnly: false,

            /**		
			* Shows or hides the arrow button from the TimePicker textbox.
			* @default true
			* @type {Boolean}
			* @example 
			* // Set the showButton property during initialization. 			
			* 	$("#timepicker").ejTimePicker({  showButton : false });
			* @example 
			* //Get or set the showButton property after initialization:<br>
			*	// Gets the showButton value.		
			*	$("#timepicker").ejTimePicker("option", "showButton");			
			*	// Sets the showButton property to timepicker
			*	$("#timepicker").ejTimePicker("option", "showButton",  false );
			 * @memberof ejTimePicker
			* @instance
			*/
            showButton: true,

            /**		
			* Sets the time interval between the two adjacent time values in the popup.
			* @default 30
			* @type {Number}
			* @example 
			* // Set the interval value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  interval : 60 });
			* @example 
			* //Get or set the interval value after initialization:<br>
			*	// Gets the interval value.		
			*	$("#timepicker").ejTimePicker("option", "interval");			
			*	// Sets the interval value to timepicker
			*	$("#timepicker").ejTimePicker("option", "interval",  60 );
			 * @memberof ejTimePicker
			* @instance
			*/
            interval: 30,

            /**		
			* Sets the step value for increment an hour value through arrow keys or mouse scroll.
			* @default 1
			* @type {Number}
			* @example 
			* // Set the hourInterval value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  hourInterval : 2 });
			* @example 
			* //Get or set the hourInterval value after initialization:<br>
			*	// Gets the hourInterval value.		
			*	$("#timepicker").ejTimePicker("option", "hourInterval");			
			*	// Sets the hourInterval value to timepicker
			*	$("#timepicker").ejTimePicker("option", "hourInterval",  2 );
			 * @memberof ejTimePicker
			* @instance
			*/
            hourInterval: 1,

            /**		
			* Sets the step value for increment the minute value through arrow keys or mouse scroll.
			* @default 1
			* @type {Number}
			* @example 
			* // Set the minute interval value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  minInterval : 5 });
			* @example 
			* //Get or set the minute interval value after initialization:<br>
			*	// Gets the minute interval value.		
			*	$("#timepicker").ejTimePicker("option", "minInterval");			
			*	// Sets the minute interval value to timepicker
			*	$("#timepicker").ejTimePicker("option", "minInterval",  5 );
			 * @memberof ejTimePicker
			* @instance
			*/
            minInterval: 1,

            /**		
			* Sets the step value for increment the seconds value through arrow keys or mouse scroll.
			* @default 1
			* @type {Number}
			* @example 
			* // Set the seconds interval value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  secInterval : 5 });
			* @example 
			* //Get or set the seconds interval value after initialization:<br>
			*	// Gets the seconds interval value.		
			*	$("#timepicker").ejTimePicker("option", "secInterval");			
			*	// Sets the seconds interval value to timepicker
			*	$("#timepicker").ejTimePicker("option", "secInterval",  5 );
			 * @memberof ejTimePicker
			* @instance
			*/
            secInterval: 1,

            /**		
			* Defines the height of the TimePicker textbox.
			* @default ""
			* @type {String | Number}
			* @example 
			* // Set the height value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  height : "35" });
			* @example 
			* //Get or set the height after initialization:<br>
			*	// Gets the height value.		
			*	$("#timepicker").ejTimePicker("option", "height");			
			*	// Sets the height to timepicker
			*	$("#timepicker").ejTimePicker("option", "height",  "35" );
			 * @memberof ejTimePicker
			* @instance
			*/
            height: "",

            /**		
			* Defines the width of the TimePicker textbox.
			* @default ""
			* @type {String | Number}
			* @example 
			* // Set the width value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  width : "120" });
			* @example 
			* //Get or set the width after initialization:<br>
			*	// Gets the width value.		
			*	$("#timepicker").ejTimePicker("option", "width");			
			*	// Sets the width to timepicker
			*	$("#timepicker").ejTimePicker("option", "width",  "120" );
			 * @memberof ejTimePicker
			* @instance
			*/
            width: "",

            /**		
			* Sets the minimum time value to the TimePicker.
			* @default "12:00:00 AM"
			* @type {String}
			* @example 
			* // Set the minTime value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  minTime : "8:00 AM" });
			* @example 
			* //Get or set the minTime after initialization:<br>
			*	// Gets the minTime value.		
			*	$("#timepicker").ejTimePicker("option", "minTime");			
			*	// Sets the minTime to timepicker
			*	$("#timepicker").ejTimePicker("option", "minTime",  "8:00 AM" );
			 * @memberof ejTimePicker
			* @instance
			*/
            minTime: "12:00:00 AM",

            /**		
			* Sets the maximum time value to the TimePicker.
			* @default "11:59:59 PM"
			* @type {String}
			* @example 
			* // Set the maxTime value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  maxTime : "5:00 PM" });
			* @example 
			* //Get or set the maxTime after initialization:<br>
			*	// Gets the maxTime value.		
			*	$("#timepicker").ejTimePicker("option", "maxTime");			
			*	// Sets the maxTime to timepicker
			*	$("#timepicker").ejTimePicker("option", "maxTime",  "5:00 PM" );
			 * @memberof ejTimePicker
			* @instance
			*/
            maxTime: "11:59:59 PM",

            /**		
			* Changes the sharped edges into rounded corner for the TimePicker textbox and popup.
			* @default false
			* @type {Boolean}
			* @example 
			* // Set the roundedCorner value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  roundedCorner : true });
			* @example 
			* //Get or set the maxTime after initialization:<br>
			*	// Gets the roundedCorner value.		
			*	$("#timepicker").ejTimePicker("option", "roundedCorner");			
			*	// Sets the roundedCorner to timepicker
			*	$("#timepicker").ejTimePicker("option", "roundedCorner",  true );
			 * @memberof ejTimePicker
			* @instance
			*/
            roundedCorner: false,

            /**		
			* Sets the TimePicker direction as right to left alignment.
			* @default false
			* @type {Boolean}
			* @example 
			* // Set the rtl value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  rtl : true });
			* @example 
			* //Get or set the rtl after initialization:<br>
			*	// Gets the rtl value.		
			*	$("#timepicker").ejTimePicker("option", "rtl");			
			*	// Sets the rtl to timepicker
			*	$("#timepicker").ejTimePicker("option", "rtl",  true );
			 * @memberof ejTimePicker
			* @instance
			*/
            rtl: false,

            /**		
			* Defines the height of the TimePicker popup.
			* @default "191px"
			* @type {String | Number}
			* @example 
			* // Set the popupHeight value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  popupHeight : "250px" });
			* @example 
			* //Get or set the popupHeight after initialization:<br>
			*	// Gets the popupHeight value.		
			*	$("#timepicker").ejTimePicker("option", "popupHeight");			
			*	// Sets the popupHeight to timepicker
			*	$("#timepicker").ejTimePicker("option", "popupHeight",  "250px" );
			 * @memberof ejTimePicker
			* @instance
			*/
            popupHeight: "191px",

            /**		
			* Defines the width of the TimePicker popup.
			* @default "auto"
			* @type {String | Number}
			* @example 
			* // Set the popupWidth value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  popupWidth : "150px" });
			* @example 
			* //Get or set the popupWidth after initialization:<br>
			*	// Gets the popupWidth value.		
			*	$("#timepicker").ejTimePicker("option", "popupWidth");			
			*	// Sets the popupWidth to timepicker
			*	$("#timepicker").ejTimePicker("option", "popupWidth",  "150px" );
			 * @memberof ejTimePicker
			* @instance
			*/
            popupWidth: "auto",

            /**		
			* When this property is set to false, it disables the timepicker control.
			* @default true
			* @type {Boolean}
			* @example 
			* // Set the enabled value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  enabled : false });
			* @example 
			* //Get or set the enabled value after initialization:<br>
			*	// Gets the enabled value.		
			*	$("#timepicker").ejTimePicker("option", "enabled");			
			*	// Sets the enabled value to timepicker
			*	$("#timepicker").ejTimePicker("option", "enabled",  false );
			 * @memberof ejTimePicker
			* @instance
			*/
            enabled: true,

            /**		
			* Enables or disables the state maintenance of TimePicker.
			* @default false
			* @type {Boolean}
			* @example 
			* // Set the persist value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  persist : true });
			* @example 
			* //Get or set the persist value after initialization:<br>
			*	// Gets the persist value.		
			*	$("#timepicker").ejTimePicker("option", "persist");			
			*	// Sets the persist value to timepicker
			*	$("#timepicker").ejTimePicker("option", "persist",  true );
			 * @memberof ejTimePicker
			* @instance
			*/
            persist: false,

            /**     
			 * Fires when the timepicker control gets focus.
			 * @event
			 * @name ejTimePicker#focusIn 	
			 * @param {Object} argument Event parameters from timepicker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the timepicker model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.value returns the current time value
			 * @example 
			 * //focusIn event for timepicker
             * $("#timepicker").ejTimePicker({
             *    focusIn: function (args) {}
             * });      
			 * @memberof ejTimePicker
			 * @instance
			 */
            focusIn: null,

            /**     
			 * Fires when the timepicker control get lost focus.
			 * @event
			 * @name ejTimePicker#focusOut 	
			 * @param {Object} argument Event parameters from timepicker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the timepicker model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.value returns the current time value
			 * @example 
			 * //focusOut event for timepicker
             * $("#timepicker").ejTimePicker({
             *    focusOut: function (args) {}
             * });      
			 * @memberof ejTimePicker
			 * @instance
			 */
            focusOut: null,

            /**     
            * Fires when the time value changed in the TimePicker.
            * @event
            * @name ejTimePicker#change 	
            * @param {Object} argument Event parameters from timepicker     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the modified time value
            * @example 
            * //change event for timepicker
            * $("#timepicker").ejTimePicker({
            *    change: function (args) {}
            * });      
            * @memberof ejTimePicker
            * @instance
            */
            change: null,

            /**     
            * Fires when the value is selected from the timepicker dropdownlist.
            * @event
            * @name ejTimePicker#select 	
            * @param {Object} argument Event parameters from timepicker     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the selected time value
            * @example 
            * //select event for timepicker
            * $("#timepicker").ejTimePicker({
            *    select: function (args) {}
            * });      
            * @memberof ejTimePicker
            * @instance
            */
            select: null
        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            timeFormat: "string",
            minTime: "string",
            maxTime: "string",
            readOnly: "boolean",
            interval: "number",
            showButton: "boolean",
            localize: "string",
            hourInterval: "number",
            minInterval: "number",
            secInterval: "number",
            enabled: "boolean",
            persist: "boolean"
        },

        observables: ["value"],
        // public functions

        /**
        * To enable the timepicker  
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="timepicker" /&gt;
		* &lt;script&gt;
		* // Create TimePicker instance
		* var timeObj = $("#timepicker").data("ejTimePicker");
		* timeObj.enable(); // enables the timepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="timepicker" /&gt;
	    * &lt;script&gt;
	    * // enables the timepicker
        * $("#timepicker").ejTimePicker("enable");
	    * &lt;/script&gt;
		*@memberof ejTimePicker
		* @instance
        */
        enable: function () {
            if (!this.model.enabled) {
                this.element[0].disabled = false;
                this.model.enabled = true;
                this.element.removeClass("e-disable").attr("aria-disabled", false);
                if (this.model.showButton) {
                    this.timeIcon.removeClass("e-disable").attr("aria-disabled", false);
                    this.popupList.removeClass("e-disable").attr("aria-disabled", false);
                }
            }
        },

        /**
        * To disable the timepicker  
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="timepicker" /&gt;
		* &lt;script&gt;
		* // Create TimePicker instance
		* var timeObj = $("#timepicker").data("ejTimePicker");
		* timeObj.disable(); // disable the timepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="timepicker" /&gt;
	    * &lt;script&gt;
	    * // disable the timepicker
        * $("#timepicker").ejTimePicker("disable");
	    * &lt;/script&gt;
		*@memberof ejTimePicker
		* @instance
        */
        disable: function () {
            if (this.model.enabled) {
                this.element[0].disabled = true;
                this.model.enabled = false;
                this.element.addClass("e-disable").attr("aria-disabled", true);
                if (this.model.showButton) {
                    this.timeIcon.addClass("e-disable").attr("aria-disabled", true);
                    this.popupList.addClass("e-disable").attr("aria-disabled", true);
                }
                this._hideResult();
            }
        },

        /**
        * returns the current time value
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="timepicker" /&gt;
		* &lt;script&gt;
		* // Create TimePicker instance
		* var timeObj = $("#timepicker").data("ejTimePicker");
		* timeObj.getTime(); // returns the timepicker value
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="timepicker" /&gt;
	    * &lt;script&gt;
	    * // to get the time value
        * $("#timepicker").ejTimePicker("getTime");
	    * &lt;/script&gt;
		*@memberof ejTimePicker
		* @instance
        */
        getTime: function () {
            return this.element.val();
        },

        /**
        * updates the current system time to timepicker
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="timepicker" /&gt;
		* &lt;script&gt;
		* // Create TimePicker instance
		* var timeObj = $("#timepicker").data("ejTimePicker");
		* timeObj.updateCurrentTime(); // updates the current system
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="timepicker" /&gt;
	    * &lt;script&gt;
	    * // updates the current system
        * $("#timepicker").ejTimePicker("updateCurrentTime");
	    * &lt;/script&gt;
		*@memberof ejTimePicker
		* @instance
        */
        updateCurrentTime: function () {
            if (this.model.enabled && !this.model.readOnly) this._setMask();
        },

        _timeFormat: function (format) {
            var validatedformat = this._validateTimeFormat(format);
            if (validatedformat) {
                this.model.timeFormat = validatedformat;
                this.model.value = this._localizeTime(this.getTime());
                this.element.val(this.model.value);
            }
            return validatedformat;
        },

        _getTimeFormat: function () {
            this.model.timeFormat = Globalize.culture(this.model.localize).calendar.patterns.t || "h:mm tt";
            this.seperator = this._getSeperator();
        },

        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            if (this.popupList) this.popupList.removeClass(this.model.cssClass).addClass(skin);
        },

        _localize: function (culture) {
            var currentTime = this._createObject(this.element.val());
            this.model.localize = culture;
            this._getTimeFormat()

            this.model.value = this._localizeTime(currentTime);
            this.element.val(this.model.value);
            this._getAmPm();
        },

        // constructor function
        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            this._initialize();
            this._render();
            this._wireEvents();
        },

        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;

            var change = false, prev = this.model.value;
            if (!ej.isNullOrUndefined(options["minTime"]) || !ej.isNullOrUndefined(options["maxTime"])) {
                if (!ej.isNullOrUndefined(options["minTime"]) && $.trim(options["minTime"]) && this._isValid(options["minTime"]))
                    this.model.minTime = options["minTime"];
                if (!ej.isNullOrUndefined(options["maxTime"]) && $.trim(options["maxTime"]) && this._isValid(options["maxTime"]))
                    this.model.maxTime = options["maxTime"];

                this._validateMinMax();
                if (!ej.isNullOrUndefined(options["minTime"])) options["minTime"] = this.model.minTime;
                if (!ej.isNullOrUndefined(options["maxTime"])) options["maxTime"] = this.model.maxTime;

                if (this.model.showButton) this._reRenderDropdown();
                if (options["value"] == undefined) this._enableMask();
                change = true;
            }

            var option;
            for (option in options) {
                switch (option) {
                    case "timeFormat":
                        var newFormat = this._timeFormat(options[option]);
                        options[option] = this.model.timeFormat;
                        if (newFormat) {
                            this.seperator = this._getSeperator();
                            if (this.model.showButton) this._reRenderDropdown();
                        }
                        break;
                    case "localize":
                        this._localize(options[option]);
                        if (this.model.showButton) this._reRenderDropdown();
                        break;
                    case "interval":
                        this.model.interval = options[option];
                        if (this.model.showButton) this._reRenderDropdown();
                        break;
                    case "showButton": this._showButton(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "roundedCorner": this._setRoundedCorner(options[option]); break;
                    case "rtl": this._setRtl(options[option]); break;
                    case "height":
                        this._setHeight(options[option]); break;
                    case "width":
                        this.wrapper.width(options[option]);
                        this._setListWidth();
                        break;
                    case "value":
                        if (this._isValid(options[option])) {
                            this.model.value = options[option];
                            this._enableMask();
                        }
                        options[option] = this.model.value;
                        change = true;
                        break;
                    case "popupHeight": this.model.popupHeight = options[option]; this._setListHeight(); break;
                    case "popupWidth": this.model.popupWidth = options[option]; this._setListWidth(); break;
                    case "enabled": if (options[option]) this.enable(); else this.disable(); break;
                }
            }
            if (change) this._raiseChangeEvent(prev);
            this._checkErrorClass();
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            this.element.removeClass("e-input").removeAttr("ondragstart draggable").val("");
            if (this.popupList) this.popupList.remove();
        },

        _initialize: function () {
            this.target = this.element[0];
            this.timeIcon = null;
            this.popupList = null;
            this.focused = false;
            this.start = 0;
            this.end = 0;
            this.min = null;
            this.max = null;
            this.incomplete = false;
            this.downPosition = 0;
            this._getAmPm();
            this.showDropdown = false;
            this._activeItem = 0;
            this.isValidState = true;
            this._manualFocus = false;
            this._isIE7 = this._checkIE7();
            // _getInternalEvents is used when TimePicker used as a subcontrol of DateTimePicker 
            this._getInternalEvents = false;
            if (!this.model.timeFormat) this._getTimeFormat();
            else this.seperator = this._getSeperator();
        },

        _render: function () {
            this._renderWrapper();
            this._setDimentions();
            this._renderTimeIcon();
            this._validateTimes();
            this._renderDropdown();
            this._checkProperties();
            this._enableMask();
            this._checkErrorClass();
            this.element.attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', "aria-readonly": this.model.readOnly });
            this.ul.find("li").attr({ 'tabindex': -1, 'aria-selected': false });
        },

        _renderWrapper: function () {
            this.element.addClass("e-input").attr("tabindex", "0");
            this.wrapper = ej.buildTag("span.e-timewidget e-widget " + this.model.cssClass + "#" + this.target.id + "_timewidget").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
        },

        _renderTimeIcon: function () {
            if (this.model.showButton) {
                this.timeIcon = ej.buildTag("span.e-select").attr({ 'role': 'button', 'aria-label': 'select' });
                var icon = ej.buildTag("span.e-icon e-time").attr('role', 'presentation');
                this.timeIcon.append(icon);
                this.container.append(this.timeIcon).addClass("e-padding");
                this._on(this.timeIcon, "mousedown", this._timeIconClick);
            }
        },

        _renderDropdown: function () {
            if (!this.model.showButton || this.popupList) return false;
            this.popupList = ej.buildTag("div.e-time-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.target.id + "_popup", "", {}, { 'tabindex': 0, 'aria-activedescendant': '' });
            this.popup = this.popupList;
            this.ul = ej.buildTag("ul.e-ul");
            var scrollDiv = ej.buildTag("div").append(this.ul);
            $('body').append(this.popupList.append(scrollDiv));
            this._renderLiTags();
            this._setListHeight();
            this._setListWidth();
            this.popupList.ejScroller({ height: this.popupList.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.popupList.ejScroller("instance");
            this.popupList.css("display", "none");
            this._listSize = this.ul.find("li").size();
        },
        _renderLiTags: function () {
            if (this.model.interval < 1) return false;
            var start, end, timeVal, interval = this.model.interval * 60000
            start = (this.model.minTime) ? this._createObject(this.model.minTime) : this._createObject("12:00:00 AM");
            end = (this.model.maxTime) ? this._createObject(this.model.maxTime) : this._createObject("11:59:59 PM");

            while (this._compareTime(end, start, true)) {
                timeVal = this._localizeTime(start);
                this.ul.append(ej.buildTag("li", timeVal));
                start = this._createObject(start).getTime() + interval;
            }

            var liTags = this.ul.find("li");
            this._on(liTags, "mouseenter", $.proxy(this._OnMouseEnter, this));
            this._on(liTags, "mouseleave", $.proxy(this._OnMouseLeave, this));
            this._on(liTags, "click", $.proxy(this._OnMouseClick, this));
        },
        _reRenderDropdown: function () {
            this.ul.empty();
            this._renderLiTags();
            this._refreshScroller();
            this._changeActiveEle();
        },
        _refreshScroller: function () {
            var flag = this.popupList.css("display") == "none" ? true : false;
            this.popupList.css("height", "auto");
            this.popupList.find(".e-content, .e-vscroll").removeAttr("style");
            this.popupList.find(".e-vscroll div").removeAttr("style");

            if (flag) this.popupList.css("display", "block");  // For get the height of the popup
            this.scrollerObj.model.height = this.popupList.height();
            this.scrollerObj.model.scrollTop = 0;
            this.scrollerObj.refresh();
            if (flag) this.popupList.css("display", "none");
        },

        _setListWidth: function () {
            if (this.popupList) {
                var width = this.model.popupWidth;
                if (width && width != "auto") this.popupList.css({ "width": width });
                else this.popupList.css({ "width": this.wrapper.width() });
            }
        },
        _setListHeight: function () {
            if (this.popupList) this.popupList.css({ "max-height": this.model.popupHeight || "191px" });
            if (this.scrollerObj) {
                this._refreshScroller();
                this._updateScrollTop();
            }
        },
        _updateScrollTop: function () {
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
        },
        _refreshPopup: function () {
            if (this.model.popupWidth == "auto") this.popupList.css({ "width": this.wrapper.width() });
            this._setListPosition();
            this._refreshScroller();
        },
        _setListPosition: function () {
            var pos = this.wrapper.offset(),
            left = pos.left,
            totalHeight = this.wrapper.outerHeight(),
            border = (totalHeight - this.wrapper.height()) / 2,
            maxZ = this._getZindexPartial();
            if (this.model.rtl) left -= this.popupList.outerWidth() - this.wrapper.outerWidth();

            this.popupList.css({
                "left": left + "px",
                "top": pos.top + totalHeight - border + 3 + "px",
                "z-index": maxZ
            });
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (maxZ == undefined || maxZ == null || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },

        _enableMask: function () {
            if (this.model.value) {
                var flag = false;
                if ((this.model.minTime && this._compareTime(this.model.minTime, this.model.value)) ||
                    this.model.maxTime && this._compareTime(this.model.value, this.model.maxTime))
                    this.isValidState = false;
                else this.isValidState = true;

                this._setTime(this.model.value);
                if (this._getInternalEvents && !this.isValidState) this._trigger("outOfRange");
            }
            else this._setMask();
            this._changeActiveEle();
            this._preVal = this.element.val();
        },
        _setTime: function (time) {
            var modifiedTime = this._localizeTime(time);
            this.element.val(modifiedTime);
            this.model.value = modifiedTime;
        },
        _setMask: function () {
            this.model.value = new Date();
            this._enableMask();
        },

        _validateTimes: function () {
            var validatedformat = this._validateTimeFormat(this.model.timeFormat);
            if (validatedformat) this.model.timeFormat = validatedformat;
            else this.model.timeFormat = "h:mm tt";

            if (!this._isValid(this.model.minTime)) this.model.minTime = "12:00:00 AM";
            if (!this._isValid(this.model.maxTime)) this.model.maxTime = "11:59:59 PM";
            if (!this._isValid(this.model.value)) this.model.value = null;

            this._validateMinMax();
        },
        _validateMinMax: function () {
            if (this.model.minTime && this.model.maxTime && this._compareTime(this.model.minTime, this.model.maxTime)) {
                this.model.minTime = this.model.maxTime;
            }
        },
        _checkProperties: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this.disable();
            }
            this._addProperty();
            this._checkAttributes();
        },
        _addProperty: function () {
            this._setRtl(this.model.rtl);
            this._setRoundedCorner(this.model.roundedCorner);
        },
        _setRtl: function (boolean) {
            if (boolean) {
                this.wrapper.addClass("e-rtl");
                if (this.popupList) this.popupList.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                if (this.popupList) this.popupList.removeClass("e-rtl");
            }
        },
        _setRoundedCorner: function (boolean) {
            if (boolean) {
                this.container.addClass("e-corner-all");
                if (this.popupList) this.popupList.addClass("e-corner-all");
            }
            else {
                this.container.removeClass("e-corner-all");
                if (this.popupList) this.popupList.removeClass("e-corner-all");
            }
        },
        _showButton: function (show) {
            this.model.showButton = show;
            if (show) {
                this.container.addClass("e-padding");
                this._renderTimeIcon();
                this._renderDropdown();
                this._addProperty();
            }
            else {
                this.container.removeClass("e-padding");
                this.timeIcon.remove();
                this.popupList.remove();
                this.timeIcon = this.popupList = null;
            }
        },
        _checkAttributes: function () {
            if (!this.element.attr("name"))
                this.element.attr({ "name": this.element[0].id });
            if ('ondragstart' in document.createElement('input'))
                this.element.attr({ "ondragstart": "return false" });
            if ('draggable' in document.createElement('input'))
                this.element.attr({ "draggable": "false" });
        },

        _getAmPm: function () {
            this.ttAM = $.trim(this._localizeMeridian("00:00"));
            this.ttPM = $.trim(this._localizeMeridian("23:00"));
        },

        _setDimentions: function () {
            this._setHeight(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
        },
        _setHeight: function (height) {
            if (height) this.wrapper.height(height);
            if (this._isIE7) this.element.height(this.container.height());
        },

        _validateTimeFormat: function (timeFormat) {
            var parts = timeFormat.split(" "), format = "";
            if (parts.length == 1 || parts.length == 2) {
                $(parts).each(function (i, part) {
                    format += $.trim(part) + " ";
                });
                return $.trim(format);
            }
            else return null;
        },

        _getSeperator: function () {
            var p = this._getElePlace(), formats = this.model.timeFormat.split(" ")[p.time];
            var regex = new RegExp("^[a-zA-Z0-9]+$");

            for (var i = 0; i < formats.length; i++) {
                if (!regex.test(formats.charAt(i))) return formats.charAt(i);
            }
        },

        _checkInComplete: function () {
            var pos = this._getCaretSelection(), cursor = this._getStartEnd(pos);
            var replace = "00", selected = this._getSelectedValue(cursor), category = this._getCategory(cursor);
            if (pos.end - pos.start == this.element.val().length) this._checkAll();

            if (category && category != "tt") {
                this._findCategoryPosition(category);
                if (selected == "__") {
                    if (category == "h" || category == "hh") replace = "12";
                    this._changeToDefault(replace);
                }
                else if (category.length != 1 && selected.length == 1) {
                    selected = this._changeWhole(selected);
                    this.element.val(this._replaceAt(this.target.value, this.start, this.end, selected));
                }
            }
        },
        _checkAll: function () {
            var i, p = this._getElePlace(), categories = this.model.timeFormat.split(" ")[p.time].split(this.seperator);
            for (i = 0; i < categories.length; i++) {
                this._findCategoryPosition(categories[i]);
                var selected = this._getSelectedValue({ start: this.start, end: this.end });

                if (categories[i].length != 1 && selected.length == 1) {
                    selected = this._changeWhole(selected);
                    this.element.val(this._replaceAt(this.element.val(), this.start, this.end, selected));
                }
            }
        },

        _changeToDefault: function (replace) {
            this.incomplete = true;
            var preVal = this.element[0].value
            this.element[0].value = this._replaceAt(this.target.value, this.start, this.end, replace);
            var timeValue = this._checkExceedRange(this.target.value);
            if (!!timeValue) {
                this._setTime(this.model[timeValue]);
            }
            this._setSelection(this.start, this.end);
            this._raiseChangeEvent(preVal);
        },

        _setSelection: function (start, end) {
            var element = this.element[0];

            if (element.setSelectionRange)
                element.setSelectionRange(start, end);
            else if (element.createTextRange) {
                // For lower version browsers (IE8, IE7 ...)
                element = element.createTextRange();
                element.collapse(true);
                element.moveEnd('character', end);
                element.moveStart('character', start);
                element.select();
            }
        },

        _getSelectedValue: function (cursor) {
            return this.target.value.substring(cursor.start, cursor.end);
        },

        _getMinMax: function (currPart, keydown) {
            if (currPart == "hh" || currPart == "h") {
                this.min = 1; this.max = 11;
                if (keydown) this.max = 12;
            }
            else if (currPart == "HH" || currPart == "H") {
                this.min = 0; this.max = 23;
            }
            else if (currPart == "mm" || currPart == "m" || currPart == "ss" || currPart == "s") {
                this.min = 0; this.max = 59;
            }
        },

        _focusElement: function () {
            this._manualFocus = true;
            this.element.focus();
        },
        _targetFocus: function (e) {
            e.preventDefault();
            this.focused = true;
            this.element.bind('mousewheel DOMMouseScroll', $.proxy(this._mouseWheel, this));
            this.wrapper.addClass("e-focus").removeClass("e-error").attr('aria-invalid', "false");
            if (!this._manualFocus) {
                this._findCategoryPosition(this._getLeast(false));
                this._setSelection(this.start, this.end);
            }
            this._manualFocus = false;
            this._raiseEvent("focusIn");
        },
        _targetBlur: function () {
            this.focused = false;
            this.element.unbind('mousewheel DOMMouseScroll', $.proxy(this._mouseWheel, this));
            this.wrapper.removeClass("e-focus");
            this._checkInComplete();
            if (!this._checkMinMax(this.target.value))
                this.isValidState = false;
            else this.isValidState = true;
            this._checkErrorClass();
            this._raiseEvent("focusOut");
        },
        _checkErrorClass: function () {
            if (this.isValidState) this.wrapper.removeClass("e-error").attr('aria-invalid', "false");
            else this.wrapper.addClass("e-error").attr('aria-invalid', "true");
        },

        _getCaretSelection: function () {
            var input = this.element[0], start = 0, end = 0;

            if (!isNaN(input.selectionStart)) {
                start = input.selectionStart;
                end = input.selectionEnd;
                return { start: Math.abs(start), end: Math.abs(end) };
            }
            // For lower version browsers (IE8, IE7 ...)
            var bookmark = document.selection.createRange().getBookmark();
            var selection = input.createTextRange();
            selection.moveToBookmark(bookmark);

            var before = input.createTextRange();
            before.collapse(true);
            before.setEndPoint("EndToStart", selection);
            var beforeLength = before.text.length, selLength = selection.text.length;
            return { start: beforeLength, end: beforeLength + selLength };
        },

        _mouseDownOnInput: function (e) {
            if (!this.focused) this._focusElement();
            this.downPosition = this._getCaretSelection();
            this._checkInComplete();
            $(document).bind("mouseup", $.proxy(this._mouseUpOnInput, this));
        },

        _mouseUpOnInput: function (e) {
            e.preventDefault();
            $(document).unbind("mouseup", $.proxy(this._mouseUpOnInput, this));
            var pos = this._getCaretSelection();

            if (this.incomplete) {
                this.incomplete = false;
                pos = this.downPosition;
            }
            var cursor = this._getStartEnd(pos);
            this._setSelection(cursor.start, cursor.end);
        },

        _getCategoryPosition: function (category) {
            var s = 0, e = 0, parts = this.target.value.split(" "), p = this._getElePlace(), sep = this.seperator, valid = false;
            var fParts = this.model.timeFormat.split(" ")[p.time].split(sep);
            var tParts = parts[p.time].split(sep);
            if (fParts.length > tParts.length) return { start: s, end: e, isValid: valid };

            if (category == "tt") {
                if (parts[p.tt] == this.ttAM || parts[p.tt] == this.ttPM) {
                    if (p.tt == 0) s = 0;
                    else s = parts[p.time].length + 1;
                    e = s + parts[p.tt].length;
                    valid = true;
                }
            }
            else {
                if (p.time == 0) s = 0;
                else s = parts[p.tt].length + 1;

                var index = fParts.indexOf(category);
                if (index != -1) {
                    for (var i = 0; i < fParts.length; i++) {
                        e = tParts[i].length + 1;
                        if (i == index) break;
                        else s += e;
                    }
                    e += s - 1;
                    valid = true;
                }
            }
            return { start: s, end: e, isValid: valid };
        },
        _getCategory: function (cursor) {
            var parts = this.model.timeFormat.split(" "), sep = this.seperator;

            var p = this._getElePlace();

            if (cursor.isTT) return parts[p.tt];
            else return parts[p.time].split(sep)[cursor.index];
        },

        _getStartEnd: function (pos) {
            var sep = this.seperator;
            var value = this.element.val(), parts = value.split(" "), s = 0, e = 0, place = tt = null, i, j;

            for (j = 0; j < parts.length; j++) {
                if (parts[j] != this.ttAM && parts[j] != this.ttPM) {
                    var time = parts[j].split(sep), tempS = s, tempE = s + time[0].length;
                    for (i = 0; i < time.length; i++) {
                        e = time[i].length + s;
                        if (pos.start <= e) {
                            place = i;
                            tt = false;
                            j = parts.length;
                            break;
                        }
                        else s += time[i].length + 1;
                    }
                }
                else {
                    if (pos.start <= s + parts[j].length) {
                        e = parts[j].length + s;
                        place = 0;
                        tt = true;
                        j = parts.length;
                        break;
                    }
                    else s += parts[j].length + 1;
                }
            }
            if (place == null) s = tempS, e = tempE, place = 0, tt = false;

            return { start: s, end: e, index: place, isTT: tt };
        },

        _modifyValue: function (isIncrement) {
            this._checkInComplete();
            var pos = this._getCaretSelection(), cursor;
            if (pos.start == pos.end) {
                var cate = this._getLeast(true);
                var position = this._getCategoryPosition(cate);
                cursor = this._getStartEnd(position);
            }
            else cursor = this._getStartEnd(pos);
            this.start = cursor.start; this.end = cursor.end;

            this._changeValue(cursor, isIncrement);
        },

        _keyUpOnInput: function (e) {
            e.preventDefault();
            if (this._preVal != this.element.val()) {
                this._preVal = this.element.val();
                this._raiseChangeEvent();
            }
        },

        _getNextCategory: function (cate, direction) {
            var categories = [], sep = this.seperator;
            var fParts = this.model.timeFormat.split(" ");
            $(fParts).each(function (i, part) {
                if (part == "tt") categories.push(part);
                else {
                    var inner = part.split(sep);
                    categories = inner.concat(categories);
                }
            });
            var index = categories.indexOf(cate), ix;
            if (index != -1) {
                if (direction) {
                    if (index == 0) ix = categories.length - 1;
                    else ix = index - 1;
                }
                else {
                    if (index == categories.length - 1) ix = 0;
                    else ix = index + 1;
                }
                return categories[ix];
            }
            return cate;
        },
        _getElePlace: function () {
            var fParts = this.model.timeFormat.split(" "), time, tt;
            if (fParts[0] == "tt") time = 1, tt = 0;
            else time = 0, tt = 1;
            return { time: time, tt: tt };
        },
        _movePosition: function (pos, direction) {
            var cursor = this._getStartEnd(pos);
            var currCate = this._getCategory(cursor);
            if (!currCate) currCate = this._getLeast(direction);
            var next = this._getNextCategory(currCate, direction);
            var cursor = this._getCategoryPosition(next);

            if (cursor.isValid) {
                this._setSelection(cursor.start, cursor.end);
            }
        },

        _keyDownOnInput: function (e) {
            if (this.model.readOnly && !this._readOnlyKeys(e)) return false;

            var pos, cursor, category, key = e.keyCode;
            // _getInternalEvents is set to true when TimePicker used inside DateTimePicker control
            // in DateTimePicker control it allows Up, Down, Home, End, Tab keys only
            if (this._getInternalEvents && key != 38 && key != 40 && key != 36 && key != 35 && key != 9) return false;
            // Up, Down, Esc
            if (this.showDropdown && key != 38 && key != 40 && key != 27 && !this._readOnlyKeys(e)) return false;
            else if (this.showDropdown && (key == 37 || key == 39)) e.keyCode = (key == 37) ? 38 : 40;

            pos = this._getCaretSelection();
            cursor = this._getStartEnd(pos);
            category = this._getCategory(cursor);

            switch (e.keyCode) {
                case 38:        // Up Key
                    e.preventDefault();
                    if (!this.showDropdown)
                        this._modifyValue(true);
                    else {
                        if (this._activeItem > 1) this._activeItem -= 1;
                        //else this._activeItem = this._listSize;         // For cycle rotation
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 40:        // Down Key
                    e.preventDefault();
                    if (e.altKey && this.model.showButton)
                        this._showhidePopup();
                    else if (!this.showDropdown)
                        this._modifyValue(false);
                    else {
                        if (this._activeItem < this._listSize) this._activeItem += 1;
                        //else this._activeItem = 1;        // For cycle rotation
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 37:        // Left Key
                    e.preventDefault();
                    this._checkInComplete();
                    if (pos.start == pos.end) this._setSelection(pos.start - 1, pos.start - 1);
                    else this._movePosition(pos, true);
                    break;
                case 39:         // Right Key
                    e.preventDefault();
                    this._checkInComplete();
                    if (pos.start == pos.end) this._setSelection(pos.start + 1, pos.start + 1);
                    else this._movePosition(pos, false);
                    break;

                case 36:         // Home Key
                    e.preventDefault();
                    if (!this.showDropdown) {
                        var homecate = this._firstlastVal(true);
                        var hPos = this._getCategoryPosition(homecate);
                        if (hPos.isValid) this._setSelection(hPos.start, hPos.end);
                    }
                    else {
                        this._activeItem = 1;
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 35:         // End Key
                    e.preventDefault();
                    if (!this.showDropdown) {
                        var endcate = this._firstlastVal(false);
                        var ePos = this._getCategoryPosition(endcate);
                        if (ePos.isValid) this._setSelection(ePos.start, ePos.end);
                    }
                    else {
                        this._activeItem = this._listSize;
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 9:     // Tab Key
                    if (this._getInternalEvents) break;
                    this._hideResult();
                    var flag = null;
                    if (e.shiftKey && pos.start > 0) flag = true;
                    else if (!e.shiftKey && pos.end < this.element.val().length) flag = false;
                    if (flag != null) {
                        e.preventDefault();
                        this._checkInComplete();
                        this._movePosition(pos, flag);
                    }
                    break;
                case 27:    // Esc Key
                case 13:    // Enter Key
                    this._hideResult();
                    break;
                case 8:          // Backspace Key
                case 46:         // Delete Key
                    e.preventDefault();
                    if (category && category != "tt") {
                        this._findCategoryPosition(category);
                        var _doBackspace = (key == 8 && pos.start != this.start), _doDelete = (key == 46 && pos.end != this.end), len;
                        len = this.end - this.start;

                        if ((pos.start != pos.end || len == 1) && (_doBackspace || _doDelete || pos.start != pos.end)) {
                            var s1 = this.start, s2 = this.end, te;
                            this.element[0].value = this._replaceAt(this.target.value, s1, s2, "__");
                            te = (s2 - s1 != 2) ? s2 + 1 : s2;
                            this._setSelection(s1, te);
                        }
                        else {
                            if (_doBackspace) {
                                this.element[0].value = this._replaceAt(this.target.value, pos.start - 1, pos.start, "");
                                this._setSelection(pos.start - 1, pos.start - 1);
                            }
                            else if (_doDelete) {
                                this.element[0].value = this._replaceAt(this.target.value, pos.end, pos.end + 1, "");
                                this._setSelection(pos.end, pos.end);
                            }
                        }
                    }
                    break;
            }

            var currSelection = this._getSelectedValue(cursor);
            var unicode = e.keyCode ? e.keyCode : e.charCode, actualkey;

            if (e.keyCode > 47 && e.keyCode < 58)
                actualkey = String.fromCharCode(unicode);
            else if (e.keyCode > 95 && e.keyCode < 106)
                actualkey = String.fromCharCode(unicode - 48);

            if (category == "tt" && ((!e.shiftKey && !e.ctrlKey && !e.altKey) && (e.keyCode > 64 && e.keyCode < 91) || (e.keyCode > 47 && e.keyCode < 58) || (e.keyCode > 95 && e.keyCode < 106))) {
                e.preventDefault();
                var ttPos = this._getCategoryPosition(category);
                this.start = ttPos.start;
                this.end = ttPos.end;
                this._changeAmPm(currSelection);
                this._raiseChangeEvent();
            }

            if ((!e.shiftKey && !e.ctrlKey && !e.altKey) && (e.keyCode > 47 && e.keyCode < 58) || (e.keyCode > 95 && e.keyCode < 106)) {
                if (category != "tt") {
                    this._getMinMax(category, true);
                    if (pos.start == pos.end) {
                        this._findCategoryPosition(category);
                        var newVal;
                        if (pos.start == this.start) newVal = actualkey + currSelection;
                        else newVal = currSelection + actualkey;
                        if (newVal.length > 2 || !(Number(newVal) >= this.min && this.max >= Number(newVal)))
                            e.preventDefault();
                    }
                    else if (!(Number(actualkey) >= this.min && this.max >= Number(actualkey)))
                        e.preventDefault();
                }
            }
            else if (!this._allowKeyCodes(e))
                e.stopPropagation();
        },

        _allowKeyCodes: function (e) {
            // ctrl + A, C, Z, Y,  Tab, F5, Enter
            if ((e.ctrlKey && (e.keyCode == 65 || e.keyCode == 67 || e.keyCode == 90 || e.keyCode == 89))
                || e.keyCode == 9 || e.keyCode == 116 || e.keyCode == 13)
                return true;
            return false;
        },
        _readOnlyKeys: function (e) {
            // Left, Right, Home, End
            if (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || this._allowKeyCodes(e))
                return true;
            return false;
        },

        _firstlastVal: function (initial) {
            var parts = this.model.timeFormat.split(" "), sep = this.seperator;
            if (initial) {
                if (parts[0] != "tt") return parts[0].split(sep)[0];
                return "tt";
            }
            else {
                if (parts[0] != "tt") return "tt";
                else if (parts[1]) {
                    var lastItem = parts[1].split(sep);
                    return lastItem.length ? lastItem[lastItem.length - 1] : "tt";
                }
                return "tt";
            }
        },

        _mouseWheel: function (event) {
            event.preventDefault();
            if (this.model.readOnly) return false;

            var delta, rawEvent = event.originalEvent;
            if (rawEvent.wheelDelta) {
                // IE and Opera use wheelDelta, which is a multiple of 120 (possible values -120, 0, 120).
                delta = rawEvent.wheelDelta / 120;
                // In Opera, value is negated.
                //if (Sys.Browser.agent === Sys.Browser.Opera) delta = -delta;
            }
            else if (rawEvent.detail) {
                // Firefox uses detail property, which is a multiple of 3.
                delta = -rawEvent.detail / 3;
            }
            if (delta > 0)
                this._modifyValue(true);
            else if (delta < 0)
                this._modifyValue(false);
        },

        _addListHover: function () {
            this._addSelected();
            this._updateScrollTop();
        },
        _addSelected: function () {
            this.ul.find("li").removeClass("e-active");
            var activeItem = this._getActiveItem();
            activeItem.addClass("e-active");
        },
        _getActiveItem: function () {
            return $(this.ul.find("li")[this._activeItem - 1]);
        },

        _timeIconClick: function (event) {
            event.preventDefault();
            if (!this.model.enabled || this.model.readOnly || this.ul.find("li").length < 1) return false;
            this._showhidePopup();
            var len = this.element.val().length;
            this._setSelection(len, len);
        },
        _showhidePopup: function () {
            if (this._getInternalEvents) return false;
            if (!this.showDropdown)
                this._showResult();
            else
                this._hideResult();
        },
        _showResult: function () {
            this._refreshPopup();
            if (!this.focused) this._focusElement();
            this._changeActiveEle();
            var proxy = this, sTop = this._vissibleAndCalculateTop();
            this.popupList.slideDown(200, "easeOutQuad", function () {
                $(document).bind("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
            });
            this.scrollerObj.setModel({ "scrollTop": sTop });
            this.showDropdown = true;
            this._listSize = this.ul.find("li").size();
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
        },
        _hideResult: function () {
            if (this.showDropdown && !this._getInternalEvents) {
                this.showDropdown = false;
                this.popupList.slideUp(100, "easeOutQuad");
                $(document).unbind("mousedown", $.proxy(this._OnDocumentClick, this));
                $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
            }
        },

        _vissibleAndCalculateTop: function () {
            this.popupList.css({ "display": "block" });  // For get the height of the popup
            var scrollTop = this._calcScrollTop();
            this.popupList.css({ "display": "none" });
            return scrollTop;
        },
        _calcScrollTop: function () {
            var ulH = this.ul.outerHeight(), liH = this.ul.find("li").outerHeight(), index, top;
            index = this.ul.find("li.e-active").index();
            top = (liH * index) - ((this.popupList.outerHeight() - liH) / 2);
            return top;
        },
        _changeActiveEle: function () {
            var elements = this.ul.find("li");
            var currTime = this.element.val(), firstTime = elements.first().html(), index;
            index = (this._parse(currTime) - this._parse(firstTime)) / (this.model.interval * 60000);
            index = Math.round(index);
            this._activeItem = (index == elements.length) ? index : index + 1;
            if (this._activeItem < 0 || this._activeItem > elements.length || isNaN(this._activeItem)) this._activeItem = 1;
            this._addListHover();
        },

        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.popupList) && !$(e.target).parents(".e-time-popup").is(this.popupList) &&
                !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-timewidget").is(this.wrapper)) {
                this._hideResult();
            }
            else if ($(e.target).is(this.popupList) || $(e.target).parents(".e-time-popup").is(this.popupList))
                e.preventDefault();
        },
        _OnWindowResize: function (e) {
            this._refreshPopup();
        },

        _OnMouseEnter: function (e) {
            var targetEle = e.target;
            this.ul.find("li").removeClass("e-hover");
            $(targetEle).addClass("e-hover");
        },
        _OnMouseLeave: function (e) {
            this.ul.find("li").removeClass("e-hover");
        },
        _OnMouseClick: function (e) {
            e.preventDefault();
            if (this.model.enabled && !this.model.readOnly) {
                this._activeItem = $(e.target).index() + 1;
                this.ul.find("li").attr({ 'tabindex': -1, 'aria-selected': false });
                $(e.target).attr({ 'aria-selected': true, 'tabindex': 0 });
                this._addSelected();
                this._selectTimeItem($(e.target));
            }
            this._showhidePopup();
        },
        _selectTimeItem: function (ele) {
            this.element.val(ele.text());

            var flag = this._raiseChangeEvent();
            if (this.model.select && flag)
                this._trigger("select", { value: this.model.value });
        },

        _findCategoryPosition: function (category) {
            if (category == "least") category = this._getLeast(true);
            var pos = this._getCategoryPosition(category);
            this.start = pos.start;
            this.end = pos.end;
        },

        _getLeast: function (lower) {
            var formats = this.model.timeFormat.split(" "), sep = this.seperator, res = null;
            $(formats).each(function (i, e) {
                if (e != "tt") {
                    var times = e.split(sep);
                    if (lower) res = times[times.length - 1];
                    else res = times[0];
                }
            });
            return res;
        },

        _changeValue: function (cursor, isIncrement) {
            var preVal = this.target.value, currValue, category = this._getCategory(cursor);
            if (!category) return false;
            this._setSelection(this.start, this.end);
            currValue = this.target.value.substring(this.start, this.end);
            if (this._checkMinMax(this.target.value)) {
                if (currValue != this.ttAM && currValue != this.ttPM) {
                    currValue = this._changeCurrentValue(currValue, category, isIncrement);
                    if (category.length != 1) currValue = this._changeWhole(currValue);
                    this._findCategoryPosition(category);
                    this.model.value = this._replaceAt(this.target.value, this.start, this.end, currValue);
                    this.element.val(this.model.value);
                    this.end = this.start + currValue.toString().length;
                    this._setSelection(this.start, this.end);
                }
                else this._changeAmPm(currValue);
            }
            else {
                var timeValue = this._checkExceedRange(this.target.value);
                this._setTime(this.model[timeValue]);
                this._findCategoryPosition(category);
                this._setSelection(this.start, this.end);
            }
            // Min Max validation
            if (!this._checkMinMax(this.target.value)) {
                this.element.val(preVal);
                this._findCategoryPosition(category);
                this._setSelection(this.start, this.end);
            }
            else this._raiseChangeEvent(preVal);
        },

        _checkMinMax: function (value) {
            var res = this._checkExceedRange(value);
            return !res;
        },
        _checkExceedRange: function (value) {
            if (this.model.minTime && !this._compareTime(value, this.model.minTime, true)) return "minTime";
            if (this.model.maxTime && !this._compareTime(this.model.maxTime, value, true)) return "maxTime";
            return null;
        },

        _changeWhole: function (currValue) {
            return currValue > 9 ? "" + currValue : "0" + currValue;
        },
        _changeAmPm: function (ampm) {
            ampm = ampm == this.ttAM ? this.ttPM : this.ttAM;
            this.element.val(this._replaceAt(this.target.value, this.start, this.end, ampm));
            this._setSelection(this.start, this.end);
        },
        _changeMinute: function (isIncrement) {
            var formats = ["mm", "m"];
            var currFormat = this._getExactFormat(formats);
            if (currFormat) {
                this._findCategoryPosition(currFormat);
                var minute = Number(this.target.value.substring(this.start, this.end));
                this._getMinMax(currFormat);
                if (isIncrement) {
                    if (minute == this.max) {
                        minute = this.min;
                        this._changeHour();
                    }
                    else minute += 1;
                }
                else {
                    if (minute == this.min) {
                        minute = this.max;
                        this._changeHour();
                    }
                    else minute -= 1;
                }
                this._findCategoryPosition(currFormat);
                if (currFormat.length != 1) minute = this._changeWhole(minute);
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, minute));
            }
        },
        _changeHour: function (isIncrement) {
            var formats = ["hh", "h", "HH", "H"];
            var currFormat = this._getExactFormat(formats);
            if (currFormat) {
                this._findCategoryPosition(currFormat);
                var hour = Number(this.target.value.substring(this.start, this.end));
                this._getMinMax(currFormat);
                if (isIncrement) {
                    if (hour == this.max) {
                        hour += 1;
                        this._changeMeridian();
                    }
                    else if (hour > this.max) hour = this.min;
                    else hour += 1;
                }
                else {
                    if (hour == this.min) hour = this.max + 1;
                    else if (hour > this.max) {
                        hour = this.max;
                        this._changeMeridian();
                    }
                    else hour -= 1;
                }
                this._findCategoryPosition(currFormat);
                if (currFormat.length != 1) hour = this._changeWhole(hour);
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, hour));
            }
        },
        _getExactFormat: function (cate) {
            var tFormat = this.model.timeFormat;
            for (var i = 0; i < cate.length; i++) {
                if (tFormat.indexOf(cate[i]) != -1) return cate[i];
            }
            return null;
        },
        _changeMeridian: function () {
            var start = this.model.timeFormat.indexOf("tt");
            if (start != -1) {
                this._findCategoryPosition("tt");
                var meridian = this.target.value.substring(this.start, this.end);
                meridian = (meridian == this.ttAM) ? this.ttPM : this.ttAM;
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, meridian));
            }
        },
        _changeCurrentValue: function (current, category, isIncrement) {
            current = Number(current);
            var c = category, step = 1, change = true;
            this._getMinMax(c);

            if (c == "hh" || c == "h" || c == "HH" || c == "H") step = this.model.hourInterval;
            else if (c == "mm" || c == "m") step = this.model.minInterval;
            else if (c == "ss" || c == "s") step = this.model.secInterval;
            if (step <= 0) return current;

            if (isIncrement) {
                if ((c == "hh" || c == "h") && current > this.max) current = this.min - 1 + step;
                else if (current < this.max) current += step;
                else {
                    change = false;
                    if (c != "hh" && c != "h") current = this.min - 1 + step;
                    else current += step;
                    this._changeAdjacent(c, isIncrement);
                }
                if ((c == "hh" || c == "h") && current == this.max + 1)
                    change && this._changeAdjacent(c, isIncrement);
                else if (current > this.max + 1) {
                    current = current - (this.max + 1);
                    change && this._changeAdjacent(c, isIncrement);
                }
                if ((c != "hh" && c != "h") && current == this.max + 1) {
                    current = this.min;
                    change && this._changeAdjacent(c, isIncrement);
                }
            }
            else {
                if ((c != "hh" && c != "h") && current > this.min) current -= step;
                else if ((c == "hh" || c == "h") && current > this.min && current <= this.max) current -= step;
                else if ((c == "hh" || c == "h") && current == this.min) current = this.max + 2 - step;
                else {
                    change = false;
                    current = this.max + 1 - step;
                    this._changeAdjacent(c, isIncrement);
                }
                if (current < this.min) {
                    current = current + (this.max + 1);
                    change && this._changeAdjacent(c, isIncrement);
                }
            }
            return current;
        },
        _changeAdjacent: function (c, isIncrement) {
            if (c == "ss" || c == "s") this._changeMinute(isIncrement);
            else if (c == "mm" || c == "m") this._changeHour(isIncrement);
            else if (c == "hh" || c == "h" || c == "HH" || c == "H") this._changeMeridian();
        },

        _valueChange: function (e) {
            this._raiseChangeEvent();
        },

        _raiseChangeEvent: function (prev) {
            prev = (prev == undefined) ? this.model.value : prev;
            if (prev == this.target.value) return false;
            if (this._checkMinMax(this.target.value)) this.isValidState = true;
            else this.isValidState = false;
            this.model.value = this.target.value;
            this._raiseEvent("change");
            return true;
        },
        _raiseEvent: function (name) {
            if (this.model[name])
                this._trigger(name, { value: this.model.value });
        },
        _checkIE7: function () {
            if (navigator.appName == 'Microsoft Internet Explorer') {
                var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})"), version = -1;
                if (re.exec(navigator.userAgent) != null)
                    version = parseFloat(RegExp.$1);
                if (version >= 7 && version < 8) return true;
            }
            return false;
        },
        _replaceAt: function (mainString, from, to, replace) {
            return mainString.substring(0, from) + replace + mainString.substring(to);
        },
        _localizeTime: function (value) {
            return $.trim(Globalize.format(this._createObject(value), this.model.timeFormat, this.model.localize));
        },
        _localizeMeridian: function (value) {
            return $.trim(Globalize.format(this._createObject(value), "tt", this.model.localize));
        },
        _compareTime: function (time1, time2, orEqual) {
            orEqual = (!orEqual) ? false : true;
            if (orEqual) return this._parse(time1) >= this._parse(time2);
            else return this._parse(time1) > this._parse(time2);
        },
        _isValid: function (time) {
            time = this._createObject(time);
            return time && typeof time.getTime === "function" && isFinite(time.getTime());
        },
        _parse: function (time) {
            return Date.parse(this._createObject(time));
        },
        _setEmptyDate: function (date) {
            var newDate = new Date(date);
            newDate.setDate(1);
            newDate.setMonth(0);
            newDate.setFullYear(2000);
            return newDate;
        },
        _createObject: function (value) {
            if (typeof value === "string" || value == null) {
                var obj = Globalize.parseDate("1/1/2000 " + value, "d/M/yyyy " + this.model.timeFormat);
                if (!obj) obj = new Date("1/1/2000 " + value);
                return obj;
            }
            else if (typeof value === "number")
                return new Date(value);
            else if (value instanceof Date)
                return this._setEmptyDate(value);
        },

        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.element, "mousedown", this._mouseDownOnInput);
            this._on(this.element, "keydown", this._keyDownOnInput);
            this._on(this.element, "keyup", this._keyUpOnInput);
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to select the date and time values.
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejDateTimePicker
	* @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires jquery.globalize.js
    * @requires globalize.cultures.min.js
	* @requires ej.web.all.js
	* @classdesc Date and Time selection with the input field.
	* @example 
	* &lt;input type="text" id="datetime" /&gt;
	* &lt;script&gt;
	* // Create DateTimePicker
    * $("#datetime").ejDateTimePicker();
	* &lt;/script&gt;
	*/

    ej.widget("ejDateTimePicker", "ej.DateTimePicker", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        _rootCSS: "e-datetimepicker",
        type: "editor",

        // default model
        defaults: {
            // Common properties
            /**		
			* Set the root class for DateTimePicker theme. This cssClass API helps to use custom skinning option for DateTimePicker control. 
			* @default ""
			* @type {string}
			* @example 
			* //To set cssClass API during initialization  
			* 	$("#datetime").ejDateTimePicker({  cssClass: "gradient-lime" });
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the cssClass value  
			*	$("#datetime").ejDateTimePicker("option", "cssClass");<br>			
			*	//Sets the cssClass value 
			*	$("#datetime").ejDateTimePicker("option", "cssClass", "gradient-lime" ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            cssClass: "",

            /**		
			* Defines the localization culture for DateTimePicker.
			* @default "en-US"
			* @type {string}
			* @example 
			* //To set localize API during initialization  
			* 	$("#datetime").ejDateTimePicker({  localize: "zh-CN" });
			* @example 
			* //Get or set the localize API, after initialization:<br>
			*	//Gets the localize value  
			*	$("#datetime").ejDateTimePicker("option", "localize");<br>			
			*	//Sets the localize value 
			*	$("#datetime").ejDateTimePicker("option", "localize", "zh-CN" ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            localize: "en-US",

            /**		
			* Indicates that the DateTimePicker value can only be read and can’t change.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set readOnly API during initialization  
			* 	$("#datetime").ejDateTimePicker({  readOnly: true });
			* @example 
			* //Get or set the readOnly API, after initialization:<br>
			*	//Gets the readOnly value  
			*	$("#datetime").ejDateTimePicker("option", "readOnly");<br>			
			*	//Sets the readOnly value 
			*	$("#datetime").ejDateTimePicker("option", "readOnly", true ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            readOnly: false,

            /**		
			* Changes the sharped edges into rounded corner for the DateTimePicker textbox and popup.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set roundedCorner API during initialization  
			* 	$("#datetime").ejDateTimePicker({  roundedCorner: true });
			* @example 
			* //Get or set the roundedCorner API, after initialization:<br>
			*	//Gets the roundedCorner value  
			*	$("#datetime").ejDateTimePicker("option", "roundedCorner");<br>			
			*	//Sets the roundedCorner value 
			*	$("#datetime").ejDateTimePicker("option", "roundedCorner", true ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            roundedCorner: false,

            /**		
			* Sets the DateTimePicker direction as right to left alignment.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set rtl API during initialization  
			* 	$("#datetime").ejDateTimePicker({  rtl: true });
			* @example 
			* //Get or set the rtl API, after initialization:<br>
			*	//Gets the rtl value  
			*	$("#datetime").ejDateTimePicker("option", "rtl");<br>			
			*	//Sets the rtl value 
			*	$("#datetime").ejDateTimePicker("option", "rtl", true ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            rtl: false,

            /**		
			* When this property is set to false, it disables the DateTimePicker control.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set enabled API during initialization  
			* 	$("#datetime").ejDateTimePicker({  enabled: true });
			* @example 
			* //Get or set the enabled API, after initialization:<br>
			*	//Gets the enabled value  
			*	$("#datetime").ejDateTimePicker("option", "enabled");<br>			
			*	//Sets the enabled value 
			*	$("#datetime").ejDateTimePicker("option", "enabled", true ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            enabled: true,

            /**		
			* Sets the DateTime value to the control.
			* @default ""
			* @type {String | DateObject}
			* @example 
			* //To set value API during initialization  
			* 	$("#datetime").ejDateTimePicker({  value: true });
			* @example 
			* //Get or set the value API, after initialization:<br>
			*	//Gets the datetime value 
			*	$("#datetime").ejDateTimePicker("option", "value");<br>			
			*	//Sets the datetime value 
			*	$("#datetime").ejDateTimePicker("option", "value", true ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            value: "",

            /**		
			* Sets the minimum value to the DateTimePicker. Behind the minimum value an error class is added to the wrapper element.
			* @default new Date("1/1/1900 12:00:00 AM")
			* @type {String | DateObject}
			* @example 
			* //To set min API during initialization  
			* 	$("#datetime").ejDateTimePicker({  min: new Date("5/5/2010 12:00:00 AM") });
			* @example 
			* //Get or set the min API, after initialization:<br>
			*	//Gets the min value 
			*	$("#datetime").ejDateTimePicker("option", "min");<br>			
			*	//Sets the min value 
			*	$("#datetime").ejDateTimePicker("option", "min", new Date("5/5/2010 12:00:00 AM") ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            min: new Date("1/1/1900 12:00:00 AM"),

            /**		
			* Sets the maximum value to the DateTimePicker. Beyond the maximum value an error class is added to the wrapper element.
			* @default new Date("12/31/2099 11:59:59 PM")
			* @type {String | DateObject}
			* @example 
			* //To set max API during initialization  
			* 	$("#datetime").ejDateTimePicker({  min: new Date("12/10/2050 8:00:00 PM") });
			* @example 
			* //Get or set the max API, after initialization:<br>
			*	//Gets the max value 
			*	$("#datetime").ejDateTimePicker("option", "max");<br>			
			*	//Sets the max value 
			*	$("#datetime").ejDateTimePicker("option", "max", new Date("12/10/2050 8:00:00 PM") ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            max: new Date("12/31/2099 11:59:59 PM"),

            /**		
			* Defines the height of the DateTimePicker textbox.
			* @default 30
			* @type {String | Number}
			* @example 
			* //To set height API during initialization  
			* 	$("#datetime").ejDateTimePicker({  height: 40 });
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value 
			*	$("#datetime").ejDateTimePicker("option", "height");<br>			
			*	//Sets the height value 
			*	$("#datetime").ejDateTimePicker("option", "height", 40 ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            height: "",

            /**		
			* Defines the width of the DateTimePicker textbox.
			* @default 143
			* @type {String | Number}
			* @example 
			* //To set width API during initialization  
			* 	$("#datetime").ejDateTimePicker({  width: 210 });
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value 
			*	$("#datetime").ejDateTimePicker("option", "width");<br>			
			*	//Sets the width value 
			*	$("#datetime").ejDateTimePicker("option", "width", 210 ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            width: "",

            /**		
			* Defines the datetime format displayed in the DateTimePicker. The value should be a combination of date format and time format. 
			* @default "M/d/yyyy h:mm tt"
			* @type { String }
			* @example 
			* //To set dateTimeFormat API during initialization  
			* 	$("#datetime").ejDateTimePicker({  dateTimeFormat: 210 });
			* @example 
			* //Get or set the dateTimeFormat API, after initialization:<br>
			*	//Gets the dateTimeFormat value 
			*	$("#datetime").ejDateTimePicker("option", "dateTimeFormat");<br>			
			*	//Sets the dateTimeFormat value 
			*	$("#datetime").ejDateTimePicker("option", "dateTimeFormat", 210 ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            dateTimeFormat: "",

            /**		
			* Shows or hides the arrow button from the DateTimePicker textbox. 
              When the button disabled, the DateTimePicker popup opens while focus in the textbox and hides while focusout from the textbox.
			* @default true
			* @type { Boolean }
			* @example 
			* //To set showButton API during initialization  
			* 	$("#datetime").ejDateTimePicker({  showButton: false });
			* @example 
			* //Get or set the showButton API, after initialization:<br>
			*	//Gets the showButton value 
			*	$("#datetime").ejDateTimePicker("option", "showButton");<br>			
			*	//Sets the showButton value 
			*	$("#datetime").ejDateTimePicker("option", "showButton", false ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            showButton: true,

            /**		
			* Displays the custom text for the buttons inside the DateTimePicker popup.
              when the localize value changed, we can change the buttons text based on the culture. 
			* @default { today: "Today", now: "Now", done: "Done", timeTitle: "Time" }
			* @type { JSONObject }
			* @example 
			* //To set buttonText API during initialization  
			* 	$("#datetime").ejDateTimePicker({  buttonText: { done: "做过" } });
			* @example 
			* //Get or set the buttonText API, after initialization:<br>
			*	//Gets the buttonText value 
			*	$("#datetime").ejDateTimePicker("option", "buttonText");<br>			
			*	//Sets the buttonText value 
			*	$("#datetime").ejDateTimePicker("option", "buttonText", { done: "做过" } ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            buttonText: /** @lends ejDateTimePicker# */ {
                /**		
                 * Sets the text for the Today button inside the datetime popup.
				 * @alias ejDateTimePicker#buttonText->today
				 * @type String
                 */
                today: "Today",

                /**		
                 * Sets the text for the Now button inside the datetime popup.
				 * @alias ejDateTimePicker#buttonText->now
				 * @type String
                 */
                now: "Now",

                /**		
                 * Sets the text for the Done button inside the datetime popup.
				 * @alias ejDateTimePicker#buttonText->done
				 * @type String
                 */
                done: "Done",

                /**		
                 * Sets the header text for the Time dropdown.
				 * @alias ejDateTimePicker#buttonText->timeTitle
				 * @type String
                 */
                timeTitle: "Time"
            },

            /**		
			* Enables or disables the state maintenance of DateTimePicker. 
			* @default false
			* @type { Boolean }
			* @example 
			* //To set persist API during initialization  
			* 	$("#datetime").ejDateTimePicker({  persist: true });
			* @example 
			* //Get or set the persist API, after initialization:<br>
			*	//Gets the persist value 
			*	$("#datetime").ejDateTimePicker("option", "persist");<br>			
			*	//Sets the persist value 
			*	$("#datetime").ejDateTimePicker("option", "persist", true ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            persist: false,

            /**		
			* Sets the time interval between the two adjacent time values in the time popup. 
			* @default 30
			* @type { Number }
			* @example 
			* //To set interval API during initialization  
			* 	$("#datetime").ejDateTimePicker({  interval: 60 });
			* @example 
			* //Get or set the interval API, after initialization:<br>
			*	//Gets the interval value 
			*	$("#datetime").ejDateTimePicker("option", "interval");<br>			
			*	//Sets the interval value 
			*	$("#datetime").ejDateTimePicker("option", "interval", 60 ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            interval: 30,

            /**		
			* Defines the time format displayed in the time dropdown inside the DateTimePicker popup. 
			* @default "h:mm tt"
			* @type { String }
			* @example 
			* //To set timeDisplayFormat API during initialization  
			* 	$("#datetime").ejDateTimePicker({  timeDisplayFormat: "HH:mm" });
			* @example 
			* //Get or set the timeDisplayFormat API, after initialization:<br>
			*	//Gets the timeDisplayFormat value 
			*	$("#datetime").ejDateTimePicker("option", "timeDisplayFormat");<br>			
			*	//Sets the timeDisplayFormat value 
			*	$("#datetime").ejDateTimePicker("option", "timeDisplayFormat", "HH:mm" ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            timeDisplayFormat: "",

            /**		
			* Defines the width of the time dropdown inside the DateTimePicker popup.
			* @default 100
			* @type { String | Number }
			* @example 
			* //To set timePopupWidth API during initialization  
			* 	$("#datetime").ejDateTimePicker({  timePopupWidth: 150 });
			* @example 
			* //Get or set the timePopupWidth API, after initialization:<br>
			*	//Gets the timePopupWidth value 
			*	$("#datetime").ejDateTimePicker("option", "timePopupWidth");<br>			
			*	//Sets the timePopupWidth value 
			*	$("#datetime").ejDateTimePicker("option", "timePopupWidth", 150 ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            timePopupWidth: 100,

            /**		
			* Specifies the header format of the datepicker inside the DateTimePicker popup
			* @default ej.DatePicker.Header.ShowHeaderMin
			* @type { String | Enum }
			* @example 
			* //To set dayHeaderFormat API during initialization  
			* 	$("#datetime").ejDateTimePicker({  dayHeaderFormat: "showHeaderShort" });
			* @example 
			* //Get or set the dayHeaderFormat API, after initialization:<br>
			*	//Gets the dayHeaderFormat value 
			*	$("#datetime").ejDateTimePicker("option", "dayHeaderFormat");<br>			
			*	//Sets the dayHeaderFormat value 
			*	$("#datetime").ejDateTimePicker("option", "dayHeaderFormat", "showHeaderShort" ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            dayHeaderFormat: "ShowHeaderShort",

            /**		
			* Specifies the start level view in datepicker inside the DateTimePicker popup. 
			* @default ej.DatePicker.Level.Month or "Month"
			* @type { String | Enum }
			* @example 
			* //To set startLevel API during initialization  
			* 	$("#datetime").ejDateTimePicker({  startLevel: "year" });
			* @example 
			* //Get or set the startLevel API, after initialization:<br>
			*	//Gets the startLevel value 
			*	$("#datetime").ejDateTimePicker("option", "startLevel");<br>			
			*	//Sets the startLevel value 
			*	$("#datetime").ejDateTimePicker("option", "startLevel", "year" ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            startLevel: "Month",

            /**		
			* Specifies the drill down level in datepicker inside the DateTimePicker popup. 
			* @default ""
			* @type { String | Enum }
			* @example 
			* //To set depthLevel API during initialization  
			* 	$("#datetime").ejDateTimePicker({  depthLevel: "decade" });
			* @example 
			* //Get or set the depthLevel API, after initialization:<br>
			*	//Gets the depthLevel value 
			*	$("#datetime").ejDateTimePicker("option", "depthLevel");<br>			
			*	//Sets the depthLevel value 
			*	$("#datetime").ejDateTimePicker("option", "depthLevel", "decade" ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            depthLevel: "",

            /**		
			* Specifies the start day of the week in datepicker inside the DateTimePicker popup. 
			* @default 1
			* @type { Number }
			* @example 
			* //To set startDay API during initialization  
			* 	$("#datetime").ejDateTimePicker({  startDay: 3 });
			* @example 
			* //Get or set the startDay API, after initialization:<br>
			*	//Gets the startDay value 
			*	$("#datetime").ejDateTimePicker("option", "startDay");<br>			
			*	//Sets the startDay value 
			*	$("#datetime").ejDateTimePicker("option", "startDay", 3 ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            startDay: 0,

            /**		
			* Specifies the number of months to navigate at one click of next and previous button in datepicker inside the DateTimePicker popup. 
			* @default 1
			* @type { Number }
			* @example 
			* //To set stepMonths API during initialization  
			* 	$("#datetime").ejDateTimePicker({  stepMonths: 2 });
			* @example 
			* //Get or set the stepMonths API, after initialization:<br>
			*	//Gets the stepMonths value 
			*	$("#datetime").ejDateTimePicker("option", "stepMonths");<br>			
			*	//Sets the stepMonths value 
			*	$("#datetime").ejDateTimePicker("option", "stepMonths", 2 ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            stepMonths: 1,

            /**		
			* It allows showing days in other months of DatePicker calendar inside the DateTimePicker popup.
			* @default true
			* @type { Boolean }
			* @example 
			* //To set showOtherMonths API during initialization  
			* 	$("#datetime").ejDateTimePicker({  showOtherMonths: false });
			* @example 
			* //Get or set the showOtherMonths API, after initialization:<br>
			*	//Gets the showOtherMonths value 
			*	$("#datetime").ejDateTimePicker("option", "showOtherMonths");<br>			
			*	//Sets the showOtherMonths value 
			*	$("#datetime").ejDateTimePicker("option", "showOtherMonths", false ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            showOtherMonths: true,

            /**		
			* Specifies the header format to be displayed in the DatePicker calendar inside the DateTimePicker popup. 
			* @default "MMMM yyyy"
			* @type { String }
			* @example 
			* //To set headerFormat API during initialization  
			* 	$("#datetime").ejDateTimePicker({  headerFormat: "MM - yyyy" });
			* @example 
			* //Get or set the headerFormat API, after initialization:<br>
			*	//Gets the headerFormat value 
			*	$("#datetime").ejDateTimePicker("option", "headerFormat");<br>			
			*	//Sets the headerFormat value 
			*	$("#datetime").ejDateTimePicker("option", "headerFormat", "MM - yyyy" ); 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            headerFormat: 'MMMM yyyy',

            /**     
            * Fires when DateTimePicker popup opens.
            * @event
            * @name ejDateTimePicker#open 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //open event for datetimepicker
            * $("#datetime").ejDateTimePicker({
            *    open: function (args) {}
            * });      
            * @memberof ejDateTimePicker
            * @instance
            */
            open: null,

            /**     
            * Fires when DateTimePicker popup closes.
            * @event
            * @name ejDateTimePicker#close 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //close event for datetimepicker
            * $("#datetime").ejDateTimePicker({
            *    close: function (args) {}
            * });      
            * @memberof ejDateTimePicker
            * @instance
            */
            close: null,

            /**     
            * Fires when the datetime value changed in the DateTimePicker textbox.
            * @event
            * @name ejDateTimePicker#change 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {boolean}  argument.isValidState returns the current value is valid or not
            * @param {string}  argument.value returns the modified datetime value
            * @example 
            * //change event for datetimepicker
            * $("#datetime").ejDateTimePicker({
            *    change: function (args) {}
            * });      
            * @memberof ejDateTimePicker
            * @instance
            */
            change: null
        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            localize: "string",
            readOnly: "boolean",
            roundedCorner: "boolean",
            rtl: "boolean",
            enabled: "boolean",

            dateTimeFormat: "string",
            showButton: "boolean",
            buttonText: "data",
            persist: "boolean",

            interval: "number",
            timeDisplayFormat: "string",

            dayHeaderFormat: "string",
            startLevel: "string",
            depthLevel: "string",
            startDay: "number",
            stepMonths: "number",
            showOtherMonths: "boolean",
            headerFormat: "string"
        },

        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option, validate = false;
            for (option in options) {
                switch (option) {
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "localize": this._localize(options[option]); break;
                    case "readOnly": this._readOnly(options[option]); break;
                    case "roundedCorner": this._setRoundedCorner(options[option]); break;
                    case "rtl": this._setRtl(options[option]); break;
                    case "enabled": this._enabled(options[option]); break;

                    case "value":
                        options[option] = this._setValue(options[option]);
                        validate = true; break;
                    case "min":
                    case "max":
                        var temp = this._stringToObject(options[option]);
                        if (this._isValidDate(temp)) {
                            this.datePicker.option(option + "Date", temp);
                            options[option] = temp;
                        }
                        else options[option] = this.model[option];
                        validate = true; break;
                    case "height": this.wrapper.height(options[option]); break;
                    case "width": this.wrapper.width(options[option]); break;
                    case "dateTimeFormat":
                        this.model.dateTimeFormat = options[option];
                        if (this.isValidState) this._setValue(this.model.value);
                        break;
                    case "showButton": this._showButton(options[option]); break;
                    case "buttonText": this._buttonText(options[option]); break;

                    case "interval": this.timePicker.option("interval", options[option]); break;
                    case "timeDisplayFormat": this.timePicker.option("timeFormat", options[option]); break;
                    case "timePopupWidth": this.timePicker.option("popupWidth", options[option]); break;

                    case "dayHeaderFormat": this.datePicker.option("dayHeaderFormat", options[option]); break;
                    case "startLevel": this.datePicker.option("startLevel", options[option]); break;
                    case "depthLevel": this.datePicker.option("depthLevel", options[option]); break;
                    case "startDay": this.datePicker.option("startDay", options[option]); break;
                    case "stepMonths": this.datePicker.option("stepMonths", options[option]); break;
                    case "showOtherMonths": this.datePicker.option("showOtherMonths", options[option]); break;
                    case "headerFormat": this.datePicker.option("headerFormat", options[option]); break;
                }
            }
            if (validate) this._validateMinMax();
            this._valueChange();
            this._checkErrorClass();
        },
        observables: ["value"],
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.insertAfter(this.wrapper);
            this.element.removeClass("e-input").val("");
            this.wrapper.remove();
            this.popup.remove();
        },

        // constructor function
        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            this._initialize();
            this._render();
            this._wireEvents();
        },

        _initialize: function () {
            this.popup = null;
            this.isPopupOpen = false;
            this.isValidState = true;
            if (!this.model.dateTimeFormat || !this.model.timeDisplayFormat) this._getDateTimeFormat();
            if (typeof this.model.value == "string" && typeof this.model.value != "") {
                var val = this._stringToObject(this.model.value);
                if (val) this.model.value = val;
            }
            var min = this.model.min = this._stringToObject(this.model.min);
            if (!min || !this._isValidDate(min)) this.model.min = this.defaults.min;
            var max = this.model.max = this._stringToObject(this.model.max);
            if (!max || !this._isValidDate(max)) this.model.max = this.defaults.max;
        },

        _render: function () {
            this._renderWrapper();
            this._renderIcon();
            this._setDimentions();
            this._renderDropdown();
            this._checkProperties();
        },

        _renderWrapper: function () {
            this.element.addClass("e-input").attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', "tabindex": "0" });
            this.wrapper = ej.buildTag("span.e-datetime-wrap e-widget " + this.model.cssClass + "#" + this.element[0].id + "_wrapper").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
        },
        _renderIcon: function () {
            if (!this.model.showButton) return false;
            this.datetimeIcon = ej.buildTag("span.e-select");
            var icon = ej.buildTag("span.e-icon e-datetime", "", {}, { "aria-label": "select" });
            this.datetimeIcon.append(icon);
            this.container.append(this.datetimeIcon).addClass("e-padding");
            this._on(this.datetimeIcon, "click", this._iconClick);
            this._on(this.datetimeIcon, "mousedown", function (e) { e.preventDefault(); });
        },
        _setDimentions: function () {
            if (this.model.height) this.wrapper.height(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
        },

        _renderDropdown: function () {
            this.popup = ej.buildTag("div.e-datetime-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.element[0].id + "_popup").css("visibility", "hidden");
            $('body').append(this.popup);
            this._renderControls();
            var _timeTitle, _dateContainer, _timeContainer, popupContainer, _today, _now, _done, buttonContainer;

            _timeTitle = ej.buildTag("div.e-header", this.model.buttonText.timeTitle);
            _dateContainer = ej.buildTag("div.e-datecontainer").append(this.datePicker.popup);
            _timeContainer = ej.buildTag("div.e-timecontainer").append(_timeTitle, this.timePicker.popup);
            popupContainer = ej.buildTag("div.e-popup-container").append(_dateContainer, _timeContainer);

            _today = ej.buildTag("div.e-dt-button e-dt-today", this.model.buttonText.today);
            _now = ej.buildTag("div.e-dt-button e-dt-now", this.model.buttonText.now);
            _done = ej.buildTag("div.e-dt-button e-dt-done", this.model.buttonText.done);
            buttonContainer = ej.buildTag("div.e-button-container").append(_today, _now, _done);

            this.popup.append(popupContainer, buttonContainer);
            this._updateTimeHeight();

            this._bindOperations();
            this._updateValues();
            this.popup.css({ "visibility": "visible", "display": "none" });

            this._on(_today, "click", this._todayClick);
            this._on(_now, "click", this._nowClick);
            this._on(_done, "click", this._doneClick);
        },
        _renderControls: function () {
            this._renderDateControl();
            this._renderTimeControl();

            var tempContainer = ej.buildTag("span").append(this.datePicker.wrapper, this.timePicker.wrapper);
            tempContainer.find("span").css("display", "none");
            this.popup.append(tempContainer);
        },
        _renderDateControl: function () {
            var dateInput = ej.buildTag("input#" + this.element[0].id + "_date", "", {}, { "type": "text" });
            this.popup.append(dateInput);

            dateInput.ejDatePicker({
                // static values //
                height: "0px", width: "0px",
                displayInline: true,
                showDateIcon: false,
                showFooter: false,
                strictMode: true,

                // default values //
                minDate: this._stringToObject(this.model.min),
                maxDate: this._stringToObject(this.model.max),
                //value: this.model.value,
                dayHeaderFormat: this.model.dayHeaderFormat,
                startLevel: this.model.startLevel,
                depthLevel: this.model.depthLevel,
                startDay: this.model.startDay,
                stepMonths: this.model.stepMonths,
                showOtherMonths: this.model.showOtherMonths,
                headerFormat: this.model.headerFormat,
                buttonText: this.model.buttonText.today,

                // common values //
                enabled: this.model.enabled,
                rtl: this.model.rtl,
                roundedCorner: this.model.roundedCorner,
                readOnly: this.model.readOnly,
                cssClass: this.model.cssClass,
                localize: this.model.localize
            });
            this.datePicker = dateInput.data("ejDatePicker");
            this.datePicker._getInternalEvents = true;
            this.datePicker.popup.css({ "position": "static", "display": "block" });
        },
        _renderTimeControl: function () {
            var timeInput = ej.buildTag("input#" + this.element[0].id + "_time", "", {}, { "type": "text" });
            this.popup.append(timeInput);

            timeInput.ejTimePicker({
                // static values //
                height: "0px", width: "0px",

                // default values //
                interval: this.model.interval,
                //value: this.model.value,
                timeFormat: this.model.timeDisplayFormat,
                popupWidth: this.model.timePopupWidth,

                // common values //
                enabled: this.model.enabled,
                rtl: this.model.rtl,
                roundedCorner: this.model.roundedCorner,
                readOnly: this.model.readOnly,
                cssClass: this.model.cssClass,
                localize: this.model.localize
            });
            this.timePicker = timeInput.data("ejTimePicker");
            this.timePicker._getInternalEvents = true;
            this.timePicker.showDropdown = true;
            this.timePicker.popup.css({ "position": "static", "display": "block" });
        },
        _updateTimeHeight: function () {
            var height = this.datePicker.popup.outerHeight() - this.popup.find(".e-header").outerHeight() - 2;
            this.timePicker.option("popupHeight", height);
        },

        _bindOperations: function () {
            var proxy = this;
            this.datePicker.option("layoutChange", function () { proxy._updateTimeHeight(); });
            this.datePicker.option("outOfRange", function () { proxy.isValidState = false; });
            this.timePicker.option("outOfRange", function () { proxy.isValidState = false; });
            this.datePicker.option("change", function (a) { proxy._refreshTimes(a); });
            this.datePicker.option("select", function () { proxy._updateInput(); });
            this.timePicker.option("select", function () { proxy._updateInput(); });
        },
        _updateInput: function () {
            var date = this._getDate() || new Date(), time = this._getTime() || new Date();
            this.model.value = new Date(date.getFullYear(), date.getMonth(), date.getDate(),
                time.getHours(), time.getMinutes(), time.getSeconds());

            this._updateDateTime();
            this._raiseChangeEvent();
        },
        _updateDateTime: function () {
            this.isValidState = true;
            var datetime = this._objectToString(this.model.value);
            this.element.val(datetime);
        },
        _refreshTimes: function (args) {
            var value = this._getDateObj(args.value, this.datePicker.model.dateFormat);
            if (!value) return false;
            this.isValidState = true;

            if (this._compare(value, this._setEmptyTime(this.model.min))) {
                var mintime = this._getFormat(this.model.min, "HH:mm:ss");
                var preTime = this._getTime();
                this.timePicker.option("minTime", mintime);
                if (!this._compare(preTime, this._getTime())) {
                    this.timePicker.model.value = this._getFormat(preTime, this.timePicker.model.timeFormat);
                    this.isValidState = false;
                }
            }
            else this.timePicker.option("minTime", "12:00:00 AM");

            if (this._compare(value, this._setEmptyTime(this.model.max))) {
                var maxtime = this._getFormat(this.model.max, "HH:mm:ss");
                var preTime = this._getTime();
                this.timePicker.option("maxTime", maxtime);
                if (!this._compare(preTime, this._getTime())) {
                    this.timePicker.model.value = this._getFormat(preTime, this.timePicker.model.timeFormat);
                    this.isValidState = false;
                }
            }
            else this.timePicker.option("maxTime", "11:59:59 PM");
        },

        _updateValues: function () {
            this._setValue(this.model.value);

            this.datePicker.option("value", this.model.value);
            this.timePicker.option("value", this.model.value);
            this._validateMinMax();
            this._preVal = this.element.val();
            this._checkErrorClass();
        },
        _setValue: function (value) {
            if (value == null || (typeof JSON === "object" && JSON.stringify(value) === "{}")) {
                this.element.val("");
                this.model.value = null;
                this.isValidState = true;
            }
            else if (typeof value === "string") {
                this.element.val(value);
                this._updateModel();
            }
            else if (value instanceof Date && this._isValidDate(value)) {
                this.model.value = value;
                this._updateDateTime();
            }

            this._checkErrorClass();
            return this.model.value;
        },
        _validateValue: function (value) {
            var dateObj = this._stringToObject(value);
            if (!dateObj) {
                this.model.value = value;
                this.isValidState = false;
            }
            else {
                this.isValidState = true;
                this.model.value = dateObj;
            }
        },
        _validateMinMax: function () {
            var value, min, max;
            value = (this.model.value) ? this._stringToObject(this.model.value) : null;
            min = (this.model.min) ? this._stringToObject(this.model.min) : this.defaults.min;
            max = (this.model.max) ? this._stringToObject(this.model.max) : this.defaults.max;
            if (!value || !min || !max) return false;

            if (min > max) this.model.min = this.model.max;
            if (value < min || value > max) this.isValidState = false;
            else this.isValidState = true;
        },

        _checkProperties: function () {
            this.model.readOnly && this._readOnly(true);
            this.model.roundedCorner && this._setRoundedCorner(true);
            this.model.rtl && this._setRtl(true);
            this.model.enabled && this._enabled(true);
            if (!this.element.attr("name")) this.element.attr({ "name": this.element[0].id });
            this._checkErrorClass();
        },

        _targetFocus: function (e) {
            e.preventDefault();
            this.isFocused = true;
            this.wrapper.addClass("e-focus");
            this.wrapper.removeClass("e-error");
            if (!this.model.showButton && !this.model.readOnly) this._showResult();
        },
        _targetBlur: function () {
            this.isFocused = false;
            this.wrapper.removeClass("e-focus");
            if (!this.model.showButton) this._hideResult();
            this._checkErrorClass();
        },

        _keyDownOnInput: function (e) {
            switch (e.keyCode) {
                case 40:        // Down Key
                    if (e.altKey) this._showhidePopup();
                    break;
                case 37:        // Left Key
                case 39:         // Right Key
                    if (e.altKey && this.isPopupOpen) {
                        e.preventDefault();
                        this._addPrevNextFocus(e.keyCode == 37);
                    }
                    break;
                case 27:        // Esc Key
                    e.preventDefault();
                case 9:         // Tab Key
                    this._hideResult();
                    break;
            }
        },
        _addFocus: function (target) {
            if (!target.hasClass("e-focus")) {
                this._removeFocus();
                target.addClass("e-focus");
                if (target.hasClass("e-datecontainer"))
                    $(document).bind("keydown", $.proxy(this.datePicker._keyboardNavigation, this.datePicker));
                else if (target.hasClass("e-timecontainer"))
                    $(document).bind("keydown", $.proxy(this.timePicker._keyDownOnInput, this.timePicker));
                else if (target.hasClass("e-dt-button"))
                    $(document).bind("keydown", $.proxy(this._buttonClick, this));
            }
        },
        _removeFocus: function () {
            var target = this._getFocusedElement();
            if (target.length > 0) {
                target.removeClass("e-focus");
                if (target.hasClass("e-datecontainer"))
                    $(document).unbind("keydown", $.proxy(this.datePicker._keyboardNavigation, this.datePicker));
                else if (target.hasClass("e-timecontainer"))
                    $(document).unbind("keydown", $.proxy(this.timePicker._keyDownOnInput, this.timePicker));
                else if (target.hasClass("e-dt-button"))
                    $(document).unbind("keydown", $.proxy(this._buttonClick, this));
            }
        },
        _addPrevNextFocus: function (flag) {
            // flag true means previous focus, false means next focus
            var target = this._getFocusedElement(), next;
            if (target.length > 0) {
                if (target.hasClass("e-datecontainer"))
                    next = flag ? this.popup.find(".e-dt-done") : this.popup.find(".e-timecontainer");
                else if (target.hasClass("e-timecontainer"))
                    next = flag ? this.popup.find(".e-datecontainer") : this.popup.find(".e-dt-today");
                else if (target.hasClass("e-dt-today"))
                    next = flag ? this.popup.find(".e-timecontainer") : this.popup.find(".e-dt-now");
                else if (target.hasClass("e-dt-now"))
                    next = flag ? this.popup.find(".e-dt-today") : this.popup.find(".e-dt-done");
                else if (target.hasClass("e-dt-done"))
                    next = flag ? this.popup.find(".e-dt-now") : this.popup.find(".e-datecontainer");
            }
            else next = flag ? this.popup.find(".e-dt-done") : this.popup.find(".e-datecontainer");
            this._addFocus(next);
        },
        _getFocusedElement: function () {
            return this.popup.children("div").children("div.e-focus");
        },

        _valueChange: function () {
            if (this._preVal != this.element.val()) {
                this._preVal = this.element.val();
                this._updateModel();
                this._validateMinMax();
                this._raiseChangeEvent();
            }
        },
        _updateModel: function () {
            var value = this.element.val();
            if (value == "") {
                this.model.value = null;
                this.isValidState = true;
            }
            else {
                var dateObj = this._stringToObject(value);
                if (dateObj) {
                    this.model.value = dateObj;
                    this.isValidState = true;
                    this._refreshPopup();
                }
                else {
                    this.model.value = value;
                    this.isValidState = false;
                }
            }
        },
        _refreshPopup: function () {
            if (this.isValidState && this.isPopupOpen) {
                var date = this._setEmptyTime(this.model.value), time = this._setEmptyDate(this.model.value);
                var getDate = this._getDate(), getTime = this._getTime();
                if (!getDate || !this._compare(getDate, date)) this.datePicker.option("value", date);
                if (!getTime || !this._compare(getTime, time)) this.timePicker.option("value", time);
            }
        },

        _buttonClick: function (e) {
            if (e.keyCode == 13) {
                var target = this._getFocusedElement();
                if (target.hasClass("e-dt-today"))
                    this._todayClick();
                else if (target.hasClass("e-dt-now"))
                    this._nowClick();
                else if (target.hasClass("e-dt-done"))
                    this._doneClick();
            }
        },
        _todayClick: function () {
            if (!this.model.enabled || this.model.readOnly) return false;
            if (!this.datePicker.popup.find(".today").hasClass("e-active") ||
                !this.datePicker.popup.children("table").hasClass("e-dp-viewdays") ||
                this.element.val() == "" || !this.isValidState) {
                this.datePicker._setCurrDate();
                this._updateInput();
            }
        },
        _nowClick: function () {
            if (!this.model.enabled || this.model.readOnly) return false;
            this.timePicker.updateCurrentTime();
            this._updateInput();
        },
        _doneClick: function () {
            this._hideResult();
        },

        _iconClick: function (e) {
            e.preventDefault();
            if (!this.isFocused) this.element.focus();
            this._showhidePopup();
        },
        _showhidePopup: function () {
            if (this.model.readOnly) return false;
            if (!this.isPopupOpen)
                this._showResult();
            else
                this._hideResult();
        },
        _showResult: function () {
            if (this.isPopupOpen || !this.model.enabled) return false;
            if (this._raiseEvent("open")) return false;
            this.isPopupOpen = true;
            this._setListPosition();
            var proxy = this;
            this.popup.slideDown(200, "easeOutQuad", function () {
                proxy._on($(document), "mousedown", proxy._OnDocumentClick);
            });
            this._updateModel();
            this._updateTimeHeight();
            this._validateMinMax();
            this._on($(window), "resize", this._OnWindowResize);
        },
        _hideResult: function () {
            if (!this.isPopupOpen) return false;
            if (this._raiseEvent("close")) return false;
            this.isPopupOpen = false;
            this._removeFocus();
            this.popup.slideUp(100, "easeOutQuad");
            this._off($(document), "mousedown", this._OnDocumentClick);
            this._off($(window), "resize", this._OnWindowResize);
        },
        _setListPosition: function () {
            var pos = this.wrapper.offset(),
            left = pos.left,
            top = pos.top + this.wrapper.outerHeight() + 3 + "px",
            maxZ = this._getZindexPartial();
            if (this.model.rtl) left -= this.popup.outerWidth() - this.wrapper.outerWidth();

            this.popup.css({ "left": left + "px", "top": top, "z-index": maxZ });
        },

        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.popup) && !$(e.target).parents(".e-datetime-popup").is(this.popup) &&
                !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-datetime-wrap").is(this.wrapper)) {
                this._hideResult();
            }
            else if ($(e.target).is(this.popup) || $(e.target).parents(".e-datetime-popup").is(this.popup)) {
                e.preventDefault();
                if ($(e.target).parents(".e-datecontainer").length > 0) this._addFocus($(e.target).parents(".e-datecontainer"));
                else if ($(e.target).parents(".e-timecontainer").length > 0) this._addFocus($(e.target).parents(".e-timecontainer"));
                else if ($(e.target).hasClass("e-dt-button")) this._addFocus($(e.target));
                else this._removeFocus();
            }
        },
        _OnWindowResize: function (e) {
            this._setListPosition();
        },

        _raiseChangeEvent: function () {
            if (this.element != null && this.model.change)
                this._trigger("change", {
                    value: this.element.val(),
                    isValidState: this.isValidState
                });
        },
        _raiseEvent: function (name) {
            if (this.element != null && this.model[name]) return this._trigger(name);
            return false;
        },
        _getDateTimeFormat: function () {
            var pattern = Globalize.culture(this.model.localize).calendar.patterns;

            if (!this.model.dateTimeFormat) this.model.dateTimeFormat = pattern.d + " " + pattern.t;
            if (!this.model.timeDisplayFormat) this.model.timeDisplayFormat = pattern.t;
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (maxZ == undefined || maxZ == null || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        _checkErrorClass: function () {
            if (this.isValidState) this.wrapper.removeClass("e-error");
            else this.wrapper.addClass("e-error");
        },
        _getDate: function () {
            return this.datePicker.model.value;
        },
        _getTime: function () {
            return this._getDateObj(this.timePicker.model.value, this.timePicker.model.timeFormat);
        },
        _setEmptyTime: function (date) {
            var newDate = new Date(date);
            newDate.setMilliseconds(0);
            newDate.setSeconds(0);
            newDate.setMinutes(0);
            newDate.setHours(0);
            return newDate;
        },
        _setEmptyDate: function (date) {
            var newDate = new Date(date);
            newDate.setDate(1);
            newDate.setMonth(0);
            newDate.setFullYear(2000);
            return newDate;
        },
        _objectToString: function (obj) {
            return this._getFormat(obj, this.model.dateTimeFormat);
        },
        _stringToObject: function (value) {
            return this._getDateObj(value, this.model.dateTimeFormat);
        },
        _getFormat: function (value, format) {
            if (value instanceof Date)
                return Globalize.format(value, format, this.model.localize);
            else return value;
        },
        _getDateObj: function (value, format) {
            if (typeof value === "string")
                return Globalize.parseDate(value, format);
            else return value;
        },
        _compare: function (obj1, obj2) {
            return obj1 && obj2 && obj1.getTime() == obj2.getTime();
        },
        _isValidDate: function (dateObj) {
            return dateObj && typeof dateObj.getTime === "function" && isFinite(dateObj.getTime());
        },

        //-------------------- Private Methods for setModel -------------------------//
        _change: function (property, value) {
            this.datePicker.option(property, value);
            this.timePicker.option(property, value);
        },
        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            this.popup.removeClass(this.model.cssClass).addClass(skin);

            this._change("cssClass", skin);
        },
        _localize: function (culture) {
            this.model.localize = culture;
            this.model.dateTimeFormat = this.model.timeDisplayFormat = "";
            this._getDateTimeFormat();
            if (this.isValidState || (this.model.value instanceof Date && this._isValidDate(this.model.value)))
                this.element.val(this._objectToString(this.model.value));
            this._preVal = this.element.val();
            this._change("localize", culture);
        },
        _readOnly: function (boolean) {
            if (boolean) this.element.attr("readonly", "readonly");
            else this.element.removeAttr("readonly");

            this._change("readOnly", boolean);
        },
        _setRoundedCorner: function (boolean) {
            if (boolean) {
                this.container.addClass("e-corner-all");
                this.popup.addClass("e-corner-all");
                this.popup.find(".e-dt-button").addClass("e-corner-all");
            }
            else {
                this.container.removeClass("e-corner-all");
                this.popup.removeClass("e-corner-all");
                this.popup.find(".e-dt-button").removeClass("e-corner-all");
            }
            this.datePicker.option("roundedCorner", boolean);
        },
        _setRtl: function (boolean) {
            if (boolean) {
                this.wrapper.addClass("e-rtl");
                this.popup.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.popup.removeClass("e-rtl");
            }

            this._change("rtl", boolean);
        },
        _enabled: function (boolean) {
            if (boolean) this.enable();
            else {
                this.model.enabled = true; // for initial rendering
                this.disable();
            }
        },
        _showButton: function (show) {
            this.model.showButton = show;
            if (show) {
                this.container.addClass("e-padding");
                this._renderIcon();
            }
            else {
                this.container.removeClass("e-padding");
                this.datetimeIcon.remove();
                this.datetimeIcon = null;
            }
        },
        _buttonText: function (data) {
            $.extend(this.model.buttonText, data);
            this.popup.find(".e-dt-today").html(this.model.buttonText.today);
            this.popup.find(".e-dt-now").html(this.model.buttonText.now);
            this.popup.find(".e-dt-done").html(this.model.buttonText.done);
            this.popup.find(".e-timecontainer").find(".e-header").html(this.model.buttonText.timeTitle);
        },

        //-------------------- Public Methods -------------------------//

        /**
        * Enables the DateTimePicker control.
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.enable(); // enables the datetimepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
	    * // enables the datetimepicker
        * $("#datetime").ejDateTimePicker("enable");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        enable: function () {
            if (!this.model.enabled) {
                this.element[0].disabled = false;
                this.model.enabled = true;
                this.element.removeClass("e-disable").attr("aria-disabled", false);
                if (this.datetimeIcon) this.datetimeIcon.removeClass("e-disable").attr("aria-disabled", false);
                this.popup.children("div").removeClass("e-disable").attr("aria-disabled", false);
                this._change("enabled", true);
            }
        },

        /**
        * Disables the DateTimePicker control.
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.disable(); // disables the datetimepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
	    * // disables the datetimepicker
        * $("#datetime").ejDateTimePicker("disable");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        disable: function () {
            if (this.model.enabled) {
                this.element[0].disabled = true;
                this.model.enabled = false;
                this.element.addClass("e-disable").attr("aria-disabled", true);
                if (this.datetimeIcon) this.datetimeIcon.addClass("e-disable").attr("aria-disabled", true);
                this.popup.children("div").addClass("e-disable").attr("aria-disabled", true);
                this._hideResult();

                this._change("enabled", false);
                this.datePicker.popup.removeClass("e-disable").attr("aria-disabled", false);
                this.timePicker.popup.removeClass("e-disable").attr("aria-disabled", false);
            }
        },

        /**
        * Returns the current datetime value in the DateTimePicker.
		* @return datetime value
		* @example 
		* * &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.getValue(); // returns the datetime value
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
	    * // returns the datetime value
        * $("#datetime").ejDateTimePicker("getValue");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        getValue: function () {
            return this._objectToString(this.model.value);
        },

        /**
        * Updates the current system date value and time value to the DateTimePicker.
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.dateTimeNow(); // updates the current datetime value
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
	    * // updates the current datetime value
        * $("#datetime").ejDateTimePicker("dateTimeNow");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        dateTimeNow: function () {
            if (this.model.enabled && !this.model.readOnly)
                this._setValue(new Date());
        },

        /**
        * Shows or opens the DateTimePicker popup.
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.show(); // opens the datetime popup
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
	    * // opens the datetime popup
        * $("#datetime").ejDateTimePicker("show");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        show: function () {
            this._showResult();
        },

        /**
        * Hides or closes the DateTimePicker popup.
		* @return jQuery
		* @example 
		* * &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.hide(); // hides the datetime popup
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
	    * // hides the datetime popup
        * $("#datetime").ejDateTimePicker("hide");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        hide: function () {
            this._hideResult();
        },

        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.element, "keydown", this._keyDownOnInput);
            this._on(this.element, "keyup", this._valueChange);
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Dialog control
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
       * @namespace ej
       * @class ejDialog
       * @requires jQuery
       * @requires ej.common.all
       * @classdesc Custom Design for Html Button control.
       *@example
       * &lt;div id="dialog" title="WPF"&gt;
       *Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
       * &lt;/div&gt;
       * &lt;script&gt;
       * // Create Dialog control
       *  $("#dialog").ejDialog(); 	
       * &lt;/script&gt;
       */
    ej.widget("ejDialog", "ej.Dialog", {
        _rootCSS: "e-dialog",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,

        // default model
        defaults: {
            /// <summary>This Contains default property of Sycnfusion Dialog </summary>
            autoOpen: true,
            /**
			* Enables the Dialog window to open automatically.
			* @default true
			* @type {Boolean}
			* @example 
			* // Enables the Dialog window to open automatically during initialization  
			* 	 $("#dialog").ejDialog({ closeOnEscape: false});				* 
			* @example 
			* //Get or set the autoOpen API after initialization.
			*	//Gets the autoOpen value  
			*	$("#dialog").ejDialog("option", "autoOpen"); 		
			*	//Sets the autoOpen value 
			*	$("#dialog").ejDialog("option", "autoOpen", false); 
			 * @memberof ejDialog
			* @instance
			*/

            autoOpen: true,
            /**		
			* Allows the Dialog window to be closed by pressing the Esc key.
			* @default true
			* @type {Boolean}
			* @example 
			* // Enable closeOnEscape during initialization  
			* 	 $("#dialog").ejDialog({ closeOnEscape: false});				* 
			* @example 
			* //Get or set the closeOnEscape API after initialization.
			*	//Gets the closeOnEscape
			*	$("#dialog").ejDialog("option", "closeOnEscape"); 	
			*	//Sets the closeOnEscape 
			*	$("#dialog").ejDialog("option", "closeOnEscape", false);
			 * @memberof ejDialog
			* @instance
			*/
            closeOnEscape: true,
            /**		
			* Customizes the close button tooltip text.
			* @default close
			* @type {String}
			* @example 
			* //To set closeText API value during initialization  
			* 	$("#dialog").ejDialog({ closeText: "hide" });					* 
			* @example 
			* //Get or set the closeText API, after initialization:<br>
			*	//Gets the closeText 
			*	$("#dialog").ejDialog("option", "closeText"); 		
			*	//Sets the closeText
			*	$("#dialog").ejDialog("option", "closeText", "hide"); 
			 * @memberof ejDialog
			* @instance
			*/
            closeText: "close",
            /**		
        * To enable the dialog window to dragged within the page.
        * @default true
        * @type {Boolean}
        * @example 
        * //Enable draggable during initialization  
        * 	$("#dialog").ejDialog({ draggable: false});					* 
        * @example 
        * //Get or set the closeText API, after initialization:<br>
        *	//Gets the closeText 
        *	$("#dialog").ejDialog("option", "closeText"); 		
        *	//Sets the closeText
        *	$("#dialog").ejDialog("option", "closeText", "hide"); 
         * @memberof ejDialog
        * @instance
        */
            draggable: true,
            /**		
    * Set the height for the dialog control.
    * @default auto
    * @type {String | integer}
    * @example 
    * //Set Height during initialization  
    * 	$("#dialog").ejDialog({ height: 400 });				* 
    * @example 
    * //Get or set the height API, after initialization:<br>
    *	//Gets the height 
    *	$("#dialog").ejDialog("option", "height");  		
    *	//Sets the height
    *	$("#dialog").ejDialog("option", "height", 400);
     * @memberof ejDialog
    * @instance
    */
            height: "auto",
            /**		
			* Set the minimum height for the dialog control.
			* @default 150
			* @type {Integer}
			* @example 
			* //Set minimum Height during initialization  
			* 	$("#dialog").ejDialog({ minHeight: 400 });				* 
			* @example 
			* //Get or set the minHeight API, after initialization:<br>
			*	//Gets the minHeight 
			*	$("#dialog").ejDialog("option", "minHeight");  		
			*	//Sets the minHeight
			*	$("#dialog").ejDialog("option", "minHeight", 400);
			 * @memberof ejDialog
			* @instance
			*/
            minHeight: 0,
            /**		
			* Set the minimum width for the dialog control.
			* @default 150
			* @type {Integer}
			* @example 
			* //Set minimum Width during initialization  
			* 	 $("#dialog").ejDialog({ minWidth: 400 });			* 
			* @example 
			* //Get or set the minWidth API, after initialization:<br>
			*	//Gets the minWidth
			*	$("#dialog").ejDialog("option", "minWidth"); 		
			*	//Sets the minWidth
			*	$("#dialog").ejDialog("option", "minWidth", 400);
			 * @memberof ejDialog
			* @instance
			*/
            minWidth: 0,
            /**		
			* Set the maximum height for the dialog control.
			* @default null
			* @type {Integer}
			* @example 
			* //Set maximum height during initialization  
			* 	  $("#dialog").ejDialog({ maxHeight: 600 });		
			* @example 
			* //Get or set the maxHeight API, after initialization:<br>
			*	//Gets the maxHeight
			*	$("#dialog").ejDialog("option", "maxHeight"); 
			*	//Sets the maxHeight
			*	$("#dialog").ejDialog("option", "maxHeight", 600);
			 * @memberof ejDialog
			* @instance
			*/
            maxHeight: null,
            /**		
			* Set the maximum width for the dialog control.
			* @default 150
			* @type {Integer}
			* @example 
			* //Set maximum width during initialization  
			* 	   $("#dialog").ejDialog({ maxWidth: 600 });
			* @example 
			* //Get or set the maxHeight API, after initialization:<br>
			*	//Gets the maxWidth
			*	$("#dialog").ejDialog("option", "maxWidth"); 
			*	//Sets the maxWidth
			*	$("#dialog").ejDialog("option", "maxWidth", 600);
			 * @memberof ejDialog
			* @instance
			*/
            maxWidth: null,
            /**		
			* Allows the dialog window to be used as a modal form.
			* @default false
			* @type {Boolean}
			* @example 
			* //Enable modal during initialization  
			* 	    $("#dialog").ejDialog({ modal: true});
			* @example 
			* //Get or set the modal API, after initialization:<br>
			*	//Gets the modal value
			*	$("#dialog").ejDialog("option", "modal"); 
			*	//Sets the modal value
			*	$("#dialog").ejDialog("option", "modal", true);
			 * @memberof ejDialog
			* @instance
			*/
            modal: false,
            /**		
			* Displays the dialog window at the given X and Y position.
			* X: Dialog set the left position value.
			* Y: Dialog set the top position value.
			* @default Null
			* @type {JSONobject}
			* @example 
			* //Enable position during initialization  
			* 	    $("#dialog").ejDialog({position: { X: 20, Y: 26 }});
			* @example 
			* //Get or set the position API, after initialization:<br>
			*	//Gets the position value
			*	$("#dialog").ejDialog("option", "position"); 
			*	//Sets the position value
			*	$("#dialog").ejDialog("option", "position", { X: 20, Y: 26 });
			 * @memberof ejDialog
			* @instance
			*/
            position: null,
            /**		
			* Allows the dialog to be resized above the minimum height and minimum width values.
			* @default true
			* @type {Boolean}
			* @example 
			* //Allows the dialog to be resized during initialization  
			* 	    $("#dialog").ejDialog({ resizable: false});
			* @example 
			* //Get or set the resizable API, after initialization:<br>
			*	//Gets the resizable value
			* $("#dialog").ejDialog("option", "resizable"); 
			*	//Sets the resizable property
			*	$("#dialog").ejDialog("option", "resizable", false);
			 * @memberof ejDialog
			* @instance
			*/
            resizable: true,
            /**		
			* Shows or hides the dialog header element.
			* @default true
			* @type {Boolean}
			* @example 
			* // Enable showHeader on initialization.  
			* 	   $("#dialog").ejDialog({ showHeader: false});
			* @example 
			* //Get or set the showHeader API, after initialization:<br>
			*	//Gets the showHeader value
			*$("#dialog").ejDialog("option", "showHeader"); 
			*	//Sets the showHeader property
			*	$("#dialog").ejDialog("option", "showHeader", false);
			* @memberof ejDialog
			* @instance
			*/
            showHeader: true,
            /**		
			* To load the dialog content at run time, such as ajax, images, and iframe.
			* @default null
			* @type {String}
			* @example 
			* //Enable content on initialization.  
			* 	   $("#dialog").ejDialog({ content: "ajax" });
			* @example 
			* //Get or set the  content  API, after initialization:<br>
			*	//Gets the content value
			* $("#dialog").ejDialog("option", "content"); 
			*	//Sets the content
			*	 $("#dialog").ejDialog("option", "content", "ajax");
			* @memberof ejDialog
			* @instance
			*/
            content: null, //'ajax', 'iframe','image'
            /**		
			* Loads AJAX, image, and iframe content from a given location (URL).
			* @default null
			* @type {String}
			* @example 
			* //loadUrl on initialization.  
			* 	   $("#dialog").ejDialog({ loadUrl: "../samplecontent/twitter.html" });
			* @example 
			* //Get or set the  loadUrl  API, after initialization:<br>
			*	//Gets the loadUrl value
			* $("#dialog").ejDialog("option", "loadUrl"); 
			*	//Sets the loadUrl
			*	 $("#dialog").ejDialog("option", "loadUrl", "../themes/samplecontent/twitter.html" );
			* @memberof ejDialog
			* @instance
			*/
            loadUrl: null,
            /**		
			* Helps load AJAX content 
			* @default 
			* @type {JSONobject}
			* @example 
			* // Ajax options on initialization.  
			* 	   $("#dialog").ejDialog({
			*	ajaxOptions: {
			*		type: 'GET',
            *       dataType: "html",
            *       contentType: "html",
            *     async: true
            *    }
            *  });
			* @example 
			* //Get or set the ajaxOptions API, after initialization:<br>
			*	//Gets the ajaxOptions value
			*$("#dialog").ejDialog("option", "ajaxOptions"); 
			*	//Sets the ajaxOptions
			*	 $("#dialog").ejDialog("option", "ajaxOptions", {
			*  type: 'GET',
            *   dataType: "html",
            *   contentType: "html",
            *   async: true
            * });
			* @memberof ejDialog
			* @instance
			*/
            ajaxOptions: { type: 'GET', cache: false, url: "", data: {}, dataType: "html", contentType: "html", async: true },
            /**		
                       *Provides title text for the Dialog control.
                       * @default ""
                       * @type {String}
                       * @example 
                       * //Title on initialization.  
                       * 	    $("#dialog").ejDialog({ title: "Custom title" });
                       * @example 
                       * //Get or set the  title  API, after initialization:<br>
                       *	//Gets the title value
                       * $("#dialog").ejDialog("option", "title");
                       *	//Sets the title
                       *	 $("#dialog").ejDialog("option", "title", "Custom title");
                       * @memberof ejDialog
                       * @instance
                       */
            title: "",
            /**		
			*Sets the width of the Dialog control.
			* @default ""
			* @type {String}
			* @example 
			* //Width on initialization.  
			* 	 $("#dialog").ejDialog({ width: 500 });
			* @example 
			* //Get or set the  width  API, after initialization:<br>
			*	//Gets the width value
			* $("#dialog").ejDialog("option", "width"); 
			*	//Sets the width
			*	 $("#dialog").ejDialog("option", "width", 500);
			* @memberof ejDialog
			* @instance
			*/
            width: 400,
            /**		
			*Sets the z-index value of the control for the Dialog control.
			* @default 1000
			* @type {Integer}
			* @example 
			* //zIndex on initialization.  
			* 	 $("#dialog").ejDialog({ width: 500 });
			* @example 
			* //Get or set the  zIndex  API, after initialization:<br>
			*	//Gets the zIndex value
			* $("#dialog").ejDialog("option", "zIndex"); 
			*	//Sets the zIndex API value
			*	 $("#dialog").ejDialog("option", "zIndex", 1000);
			* @memberof ejDialog
			* @instance
			*/
            zIndex: 1000,
            /**		
			*Set the root class for the Dialog control theme.
			* @default ""
			* @type {String}
			* @example 
			* //cssClass on initialization.  
			* 	$("#dialog").ejDialog({ cssClass: "gradient-lime" });
			* @example 
			* //Get or set the  zIndex  API, after initialization:<br>
			*	//Gets the cssClass value
			* $("#dialog").ejDialog("option", "cssClass"); 
			*	//Sets the cssClass API value
			* $("#dialog").ejDialog("option", "cssClass", "gradient-lime");
			* @memberof ejDialog
			* @instance
			*/
            cssClass: "",
            /**		
			*Displays dialog content from right to left when enabled
			* @default false
			* @type {Boolean}
			* @example 
			* //Enable rtl on initialization.  
			* 	 $("#dialog").ejDialog({ rtl: true});
			* @example 
			* //Get or set the  rtl  API, after initialization:<br>
			*	//Gets the rtl value
			* $("#dialog").ejDialog("option", "rtl"); 
			*	//Sets the rtl API value
			* $("#dialog").ejDialog("option", "rtl", true);
			* @memberof ejDialog
			* @instance
			*/
            rtl: false,
            /**		
			*Enables users to navigate right, left, up, and down in the control with keyboard action
			* @default true
			* @type {Boolean}
			* @example 
			* //Enable  allowKeyboardNavigation on initialization.  
			* 	 $("#dialog").ejDialog({ rtl: true});
			* @example 
			* //Get or set the  allowKeyboardNavigation  API, after initialization:<br>
			*	//Gets the allowKeyboardNavigation value
			* $("#dialog").ejDialog("option", "allowKeyboardNavigation"); 
			*	//Sets the allowKeyboardNavigation API value
			* $("#dialog").ejDialog("option", "allowKeyboardNavigation", true);
			* @memberof ejDialog
			* @instance
			*/
            allowKeyboardNavigation: true,
            /**		
			*Gives dialog window rounded corners.
			* @default false
			* @type {Boolean}
			* @example 
			* //Enable roundedCorner on initialization.  
			* 	 $("#dialog").ejDialog({ roundedCorner: true});
			* @example 
			* //Get or set the  roundedCorner  API, after initialization:<br>
			*	//Gets the roundedCorner value
			* $("#dialog").ejDialog("option", "roundedCorner"); 
			*	//Sets the roundedCorner API value
			*$("#dialog").ejDialog("option", "roundedCorner", true);
			* @memberof ejDialog
			* @instance
			*/
            roundedCorner: false,
            /**		
			*Gives dialog window rounded corners.
			* @default [“close”]
			* @type {StringArray}
			* @example 
			* //iconAction on initialization.  
			* 	 $("#dialog").ejDialog({ iconAction: ["close","collapsible","pin"]});
			* @example 
			* //Get or set the  iconAction  API, after initialization:<br>
			*	//Gets the iconAction value
			* $("#dialog").ejDialog("option", "iconAction"); 
			*	//Sets the iconAction API value
			*$("#dialog").ejDialog("option", "iconAction",["close","collapsible","pin"]);
			* @memberof ejDialog
			* @instance
			*/
            iconAction: ["close"], // ["close","collapsible","maximize","minimize","custom","pin"]
            /**		
                       *Custom icons for custom actions can be provided by using a CSS class.
                       * @default null
                       * @type {String}
                       * @example 
                       * //customIconCss on initialization.  
                       * 	 ("#dialog").ejDialog({ customIconCss: "custom-icon" });
                       * @example 
                       * //Get or set the  customIconCss  API, after initialization:<br>
                       *	//Gets the customIconCss value
                       * $("#dialog").ejDialog("option", "customIconCss"); 
                       *	//Sets the customIconCss API value
                       *$("#dialog").ejDialog("option", "customIconCss", "custom-icon");
                       * @memberof ejDialog
                       * @instance
                       */
            customIconCss: null,
            /**		
         *Places the dialog in the given container.
         * @default null
         * @type {String}
         * @example 
         * // Enable contentContainer on initialization.  
         * 	$("#dialog").ejDialog({ contentContainer: "#samplearea" });
         * @example 
         * //Get or set the  contentContainer  API, after initialization:<br>
         *	//Gets the contentContainer value
         * $("#dialog").ejDialog("option", "contentContainer"); 
         *	//Sets the contentContainer 
         *$("#dialog").ejDialog("option", "contentContainer", "#samplearea" );
         * @memberof ejDialog
         * @instance
         */
            contentContainer: null,
            /**		
			*Saves the current model value to browser cookies for maintaining state.
			* @default False
			* @type {Boolean}
			* @example 
			* //Enable persist on initialization.  
			* $("#dialog").ejDialog({ persist: true});
			* @example 
			* //Get or set the  persist  API, after initialization:<br>
			*	//Gets the persist value
			* $("#dialog").ejDialog("option", "persist"); 
			*	//Sets the persist 
			*$("#dialog").ejDialog("option", "persist", true);
			* @memberof ejDialog
			* @instance
			*/
            persist: false,
            /**		
			*Enables or disables the Dialog control.
			* @default true
			* @type {Boolean}
			* @example 
			* //Set enabled on initialization.  
			*  $("#dialog").ejDialog({ enabled: true });
			* @example 
			* //Get or set the  persist  API, after initialization:<br>
			*	//Gets the enabled API value
			*$("#dialog").ejDialog("option", "enabled"); 
			*	//Sets the enabled 
			*$("#dialog").ejDialog("option", "enabled", true);
			* @memberof ejDialog
			* @instance
			*/
            enabled: true,
            /**		
			*Enables  the window resizing.
			* @default true
			* @type {Boolean}
			* @example 
			* //Enable windowResizing on initialization.  
			*  $("#dialog").ejDialog({ enabled: true });
			* @example 
			* //Get or set the  windowResizing  API, after initialization:<br>
			*	//Gets the windowResizing API value
			*$("#dialog").ejDialog("option", "windowResizing"); 
			*	//Sets the windowResizing 
			*$("#dialog").ejDialog("option", "windowResizing", true);
			* @memberof ejDialog
			* @instance
			*/
            windowResizing: false,
            //Events
            /**     
            * Fires before dialog control is closed.
            * @event
            * @name ejDialog#beforeClose		
            * @param {Object} argument.event returns the close icon click event args    
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the dialog model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //beforeClose event for dialog
            *  $("#dialog").ejDialog({
            *  beforeClose: function (args) {}
            * });
            * @memberof ejDialog
            * @instance
            */
            beforeClose: null,
            /**     
			 * Fires  after dialog control is closed.
			 * @event
			 * @name ejDialog#Close		
			 * @param {Object} argument.event returns the close icon click event args    
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * //beforeClose event for dialog
             *  $("#dialog").ejDialog({
			 *  Close: function (args) {}
             * });
			 * @memberof ejDialog
			 * @instance
			 */
            close: null,
            /**     
			 * Fires  before dialog control is opened.
			 * @event
			 * @name ejDialog# beforeOpen		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * // beforeOpen event for dialog
             *  $("#dialog").ejDialog({
			 *   beforeOpen: function (args) {}
             * });
			 * @memberof ejDialog
			 * @instance
			 */
            beforeOpen: null,
            /**     
         * Fires  after dialog control is opened.
         * @event
         * @name ejDialog#open		
         * @param {boolean}  argument.cancel returns the cancel option value
         * @param {object}  argument.model returns the dialog model
         * @param {string}  argument.type returns the name of the event
         * @example 
         * //  open event for dialog
         *  $("#dialog").ejDialog({
         *    open: function (args) {}
         * });
         * @memberof ejDialog
         * @instance
         */
            open: null,
            /**     
            * Fires  when dialog control is dragged.
            * @event
            * @name ejDialog#drag		
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the dialog model
            * @param {string}  argument.type returns the name of the event
            * @param {Object} argument.event  returns the mouse move event args
            * @example 
            * //   drag event for dialog
            *  $("#dialog").ejDialog({
            *     drag: function (args) {}
            * });
            * @memberof ejDialog
            * @instance
            */
            drag: null,
            /**     
			 * Fires when the dialog is starting to be dragged
			 * @event
			 * @name ejDialog#dragStart		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {Object} argument.event  returns the mouse down event args
			 * @example 
			 * //    dragStart event for dialog
             *  $("#dialog").ejDialog({
			 *      dragStart: function (args) {}
             * });
			 * @memberof ejDialog
			 * @instance
			 */
            dragStart: null,
            /**     
         * Fires after the dialog dragging stops
         * @event
         * @name ejDialog#dragStop		
         * @param {boolean}  argument.cancel returns the cancel option value
         * @param {object}  argument.model returns the dialog model
         * @param {string}  argument.type returns the name of the event
         * @param {Object} argument.event  returns the mouse down event args
         * @example 
         * //     dragStop event for dialog
         *  $("#dialog").ejDialog({
         *       dragStop: function (args) {}
         * });
         * @memberof ejDialog
         * @instance
         */
            dragStop: null,
            /**    
            * Fires on dialog resize action
            * @event
            * @name ejDialog#resize		
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the dialog model
            * @param {string}  argument.type returns the name of the event
            * @param {Object} argument.event  returns the mouse move event args
            * @example 
            * //      resize event for dialog
            *  $("#dialog").ejDialog({
            *        resize: function (args) {}
            * });
            * @memberof ejDialog
            * @instance
            */
            resize: null,
            /**    
            * Fires on dialog resize start action
            * @event
            * @name ejDialog#resizeStart		
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the dialog model
            * @param {string}  argument.type returns the name of the event
            * @param {Object} argument.event  returns the mouse down event args
            * @example 
            * //       resizeStart event for dialog
            *  $("#dialog").ejDialog({
            *         resizeStart: function (args) {}
            * });
            * @memberof ejDialog
            * @instance
            */
            resizeStart: null,
            /**    
            * Fires on dialog resize stop action
            * @event
            * @name ejDialog#resizeStop		
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the dialog model
            * @param {string}  argument.type returns the name of the event
            * @param {Object} argument.event  returns the mouse leave event args
            * @example 
            * //        resizeStop event for dialog
            *  $("#dialog").ejDialog({
            *          resizeStop: function (args) {}
            * });
            * @memberof ejDialog
            * @instance
            */
            resizeStop: null,
            /**    
        * Fires on AJAX content, iframe, or image load actions
        * @event
        * @name ejDialog#load		
        * @param {boolean}  argument.cancel returns the cancel option value
        * @param {object}  argument.model returns the dialog model
        * @param {string}  argument.type returns the name of the event
        * @param {string}  argument.url  returns the content location
        * @param {Object} argument.contentType  returns the content type
        * @example 
        * //         load event for dialog
        *  $("#dialog").ejDialog({
        *           load: function (args) {}
        * });
        * @memberof ejDialog
        * @instance
        */
            load: null,
            /**    
			 * Fires when AJAX content is loaded successfully
			 * @event
			 * @name ejDialog#ajaxSuccess		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url  returns the content location
			 * @param {string} argument.data  returns the content text
			 * @example 
			 * //          ajaxSuccess event for dialog
             *  $("#dialog").ejDialog({
			 *            ajaxSuccess: function (args) {}
             * });
			 * @memberof ejDialog
			 * @instance
			 */
            ajaxSuccess: null,
            /**    
			 * Fires after an AJAX content loading error
			 * @event
			 * @name ejDialog#ajaxError		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url  returns the content location
			 * @param {string} argument.data  returns the content text
			 * @example 
			 * //           ajaxError event for dialog
             *  $("#dialog").ejDialog({
			 *             ajaxError: function (args) {}
             * });
			 * @memberof ejDialog
			 * @instance
			 */
            ajaxError: null,
        },

        dataTypes: {
            autoOpen: "boolean",
            closeOnEscape: "boolean",
            closeText: "string",
            draggable: "boolean",
            modal: "boolean",
            resizable: "boolean",
            windowResizing: "boolean",
            showHeader: "boolean",
            title: "string",
            zIndex: "number",
            cssClass: "string",
            loadUrl: "string",
            content: "string",
            ajaxOptions: "data",
            iconAction: "data"
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "closeText": this._ejDialog.find("div.e-dialog-icon").attr("title", options[key]); break;
                    case "title": this._ejDialog.find("span.e-title").html(options[key]); break;
                    case "width": this.model.width = options[key]; this._setSize(); this._setContainerSize(); this._resetScroller(); break;
                    case "height": this.model.height = options[key]; this._setSize(); this._setContainerSize(); this._resetScroller(); break;
                    case "position": this.model.position = options[key]; this._dialogPosition(); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "roundedCorner": this._roundedCorner(options[key]); break;
                    case "content": { this._appendContent(options[key]); break; }
                    case "enabled": { this.model.enabled = options[key]; this._enabledAction(options[key]); break; }
                    case "loadUrl": { this.model.loadUrl = options[key]; this._appendContent(this.model.content); break; }
                    case "contentContainer": { this._ejDialog.appendTo($(options[key])); break; }
                    case "minHeight": { this._ejDialog.css("minHeight", options[key]); break; }
                    case "minWidth": { this._ejDialog.css("minWidth", options[key]); break; }
                    case "zIndex": { this._ejDialog.css('z-index', options[key]); break; }
                    case "customIconCss": { this._dialogCustom.find("span").removeClass().addClass(options[key]); break; }
                    case "windowResizing": {
                        this.model.windowResizing = options[key];
                        this._ejDialog.addClass("e-dialog-resize");
                        this._wireResizing();
                        break;
                    }
                    case "draggable": {
                        this.model.draggable = options[key];
                        if (options[key])
                            this._enableDrag();
                        else {
                            this._dialogTitlebar.removeClass("e-draggable");
                        }
                        break;
                    }
                    case "resizable": {
                        this.model.resizable = options[key];
                        if (options[key])
                            this._enableResize();
                        else {
                            this._ejDialog.removeClass("e-resizable");
                            this._ejDialog.find(".e-resize-handle").remove();
                        }
                        break;
                    }
                    case "showHeader": {
                        this.model.showHeader = options[key];
                        if (options[key]) {
                            this._renderTitleBar();
                            this._iconsRender(this.model.iconAction);
                            this._enableDrag();
                        }
                        else
                            this._ejDialog.find(".e-titlebar").remove();
                        break;
                    }
                    case "rtl":
                        {
                            if (options[key])
                                this._ejDialog.addClass("e-rtl");
                            else
                                this._ejDialog.removeClass("e-rtl");
                            break;
                        }
                    case "iconAction":
                        {
                            if (!ej.isNullOrUndefined(this._dialogTitlebar)) {
                                this._removeAllIcons();
                                this._iconsRender(options[key]);
                            }
                        }
                }
            }
        },

        // all events bound using this._on will be unbind automatically
        /**
        * destroy the dialog widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="dialog" title="WPF"&gt;
		*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
		* &lt;/div&gt;
		* &lt;script&gt;
		* // Create Dialog control
		*  var dialogObj = $("#dialog").data("ejDialog");	//Initialize the Dialog object.
		*  dialogObj.destroy(); //Calls the destroy method of Dialog to destroy
		* &lt;/script&gt;
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		*  //call destroy method
		*  $("#dialog").ejDialog("destroy"); 
		* &lt;/script&gt;
		* @memberof ejDialog
		* @instance
         */
        _destroy: function () {
            if (this._overLay) this._overLay.remove();
            this._cloneElement.appendTo(this._ejDialog.parent());
            this._ejDialog.remove();
            this._cloneElement.removeClass("e-dialog");
            this.element = this._cloneElement;
            this._isOpen = false;
        },

        keyConfigs: [37, 38, 39, 40], //keyConfigs for up,down,left and right keys

        // constructor function
        /**
         * Create the dialog widget
		 * @private
         */
        _init: function () {
            this._widthPercent = null;
            this._heightPercent = null;
            this._renderControl();
            this._wireEvents();
        },
        /**
         * Render Section For Dialog	
		 * @private
         */
        // render dialog control
        _renderControl: function () {
            //Clone element     
            this._cloneElement = this.element.clone();
            this.element.attr("tabindex", 0).attr("role", "dialog").attr("aria-labelledby", this.element.prop("id") + "_title");
            this._ejDialog = ej.buildTag("div.e-dialog e-widget e-box" + this.model.cssClass + " e-dialog-wrap e-shadow#" + this.element.prop("id") + "_wrapper", "", { display: "none", zIndex: this.model.zIndex }, { tabidex: 0 });
            this.model.windowResizing ? this._ejDialog.addClass("e-dialog-resize") : "";
            this.wrapper = this._ejDialog;
            if (!ej.isNullOrUndefined(this.model.contentContainer))
                this._ejDialog.appendTo($(this.model.contentContainer));
            else
                this._ejDialog.appendTo(document.body);
            if (this.model.rtl)
                this._ejDialog.addClass("e-rtl");
            //Render the Title Bar
            if (this.model.showHeader)
                this._renderTitleBar();
            //Icons Rendering
            if (this.model.showHeader)
                this._iconsRender(this.model.iconAction);
            //Append Content
            this._appendContent(this.model.content);
            this._enabledAction(this.model.enabled);
            this._enableDrag();
            this._setSize();
            this._sizeInPercent();
            if (this.model.autoOpen && this.model.content != "ajax")
                this.open();
            this._setContainerSize();
            this._renderScroller();
            this._roundedCorner(this.model.roundedCorner);
            this.scroller = this.contentDiv.data("ejScroller");
        },
        /**
        * To set Container Size for the Dialog control		
        * @private
        */
        _setContainerSize: function () {
            if (this.model.height != "auto") {
                this.contentDiv.height(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
                this.element.outerHeight(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
            }
        },
        /**
        * To enable drag for the Dialog control		
        * @private
        */
        //Enable Drag
        _enableDrag: function () {
            if (this.model.draggable && this.model.showHeader) {
                this._dialogTitlebar.addClass("e-draggable");
                this._dragDialog();
            }
        },
        /**
         * To add resize icon for the Dialog control		
		 * @private
         */
        //Add resize icon
        _enableResize: function () {
            if (this.model.resizable && this.model.enabled) {
                this._ejDialog.addClass("e-resizable");
                var resizeDiv = ej.buildTag("div.e-icon e-resize-handle");
                resizeDiv.appendTo(this._ejDialog);
                this._resizeDialog();
            }
        },
        /**
         * To change skin for the Dialog control		
		 * @private
         */
        //Skin Change at run time
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this._ejDialog.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
         * To enabledAction  for the Dialog control		
		 * @private
         */
        _enabledAction: function (flag) {
            if (flag) {
                this._ejDialog.removeClass("e-disable");
                this._enableResize();
            }
            else {
                this._ejDialog.addClass("e-disable");
                this._ejDialog.removeClass("e-resizable");
                this._ejDialog.find(".e-resize-handle").remove();
            }
        },
        /**
         * To render title bar  for the Dialog control		
		 * @private
         */
        _renderTitleBar: function () {
            this._elementTitle = this.element.attr("title");
            if (typeof this._elementTitle !== "string") {
                this._elementTitle = "";
            }
            this.model.title = this.model.title || this._elementTitle;
            this._dialogTitlebar = ej.buildTag("div#" + this.element.prop("id") + "_title.e-titlebar e-header").prependTo(this._ejDialog);
            if (this.model.title) {
                //Add Title 
                ej.buildTag("span.e-title", this.model.title).prependTo(this._dialogTitlebar);
            }
        },
        /**
         * To render icons  for the Dialog control		
		 * @private
         */
        _iconsRender: function (iconArray) {
            for (var icon = 0; icon < iconArray.length; icon++) {
                switch (iconArray[icon]) {
                    case "close": {
                        this._closeIcon();
                        break;
                    }
                    case "collapsible": {
                        this._collapsibleIcon();
                        break;
                    }
                    case "maximize": {
                        this._maximizeIcon();
                        break;
                    }
                    case "minimize": {
                        this._minimizeIcon();
                        break;
                    }
                    case "pin": {
                        this._pinIcon();
                        break;
                    }
                    case "custom": {
                        this._customIcon();
                        break;
                    }
                }
            }
        },
        /**
         * To remove all the icons for the Dialog control		
		 * @private
         */
        _removeAllIcons: function () {
            this._dialogTitlebar.find("div.e-dialog-icon").remove();
        },
        /**
         * To append content  for the Dialog control		
		 * @private
         */
        _appendContent: function (contentType) {
            this.contentDiv = ej.isNullOrUndefined(this.contentDiv) ? ej.buildTag("div.e-dialog-scroller") : this.contentDiv;
            this.element.removeAttr("title").addClass("e-widget-content");
            if (!ej.isNullOrUndefined(this.model.loadUrl) && !ej.isNullOrUndefined(contentType)) {
                if (contentType == "ajax") {
                    this.model.ajaxOptions.url = this.model.loadUrl;
                    this._sendAjaxOptions(this.element, this.model.ajaxOptions.url);
                }
                else if (contentType == "iframe") {
                    var iframe = ej.buildTag("iframe.e-iframe", "", { width: "100%" }, { scrolling: "no", frameborder: 0, src: this.model.loadUrl });
                    this.element.append(iframe).show().appendTo(this.contentDiv);
                    this._trigger("load", { contentType: contentType, url: this.model.loadUrl });
                }
                else if (contentType == "image") {
                    var img = ej.buildTag("img.e-images", "", "", { src: this.model.loadUrl });
                    this.element.append(img).show().appendTo(this.contentDiv);
                    this._trigger("load", { contentType: contentType, url: this.model.loadUrl });
                }
            }
            else {
                this.element.show()
                .appendTo(this.contentDiv);
            }
            this.contentDiv.appendTo(this._ejDialog);
        },
        /**
         * To set roundedCorner  for the Dialog control		
		 * @private
         */
        _roundedCorner: function (value) {
            if (value) {
                this._ejDialog.addClass('e-corner-all');
                this._dialogTitlebar.addClass('e-corner-top');
                this._dialogClose.addClass('e-corner-all');
            }
            else if (this._ejDialog.hasClass('e-corner-all')) {
                this._ejDialog.removeClass('e-corner-all');
                this._dialogTitlebar.removeClass('e-corner-top');
                this._dialogClose.removeClass('e-corner-all');
            }
        },
        /**
         * To reRender scroller for the Dialog control		
		 * @private
         */
        _reRenderScroller: function () {
            if (this.scroller != undefined)
                this.scroller.refresh();
        },
        /**
         * To Render scroller  for the Dialog control		
		 * @private
         */
        _renderScroller: function () {
            var scrollerWidth = this.model.width;
            if (!(this.model.height === "auto" || this.model.height === "100%")) {
                if (!(this.model.width === "auto" || this.model.width === "100%"))
                    scrollerWidth = this.element.outerWidth();
                var scrHeight = this.element.outerHeight();
                this.contentDiv.ejScroller({ width: scrollerWidth, height: scrHeight, rtl: this.model.rtl });
                this.scroller = this.contentDiv.data("ejScroller");
                this.contentDiv.width("100%");
            }
        },
        //Gets the maximum z-index in the document
        _dialogMaxZindex: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        //Set the Zindex for dialog
        /**
         * To set the maximum z-index  for the Dialog control		
		 * @private
         */
        _setZindex: function () {
            var zindex = this._dialogMaxZindex();
            if (this.model.zIndex <= zindex) {
                this._ejDialog.css({ zIndex: zindex + 1 });
            }
        },
        //Create the modal overlay div
        /**
         * To Create the modal overlay div for the Dialog control		
		 * @private
         */
        _createOverlay: function () {
            var zindex = this._ejDialog.css('zIndex');//this._dialogMaxZindex() < this.model.zIndex ? this.model.zIndex : this._dialogMaxZindex();
            this._overLay = ej.buildTag("div#" + this.element.attr("id") + "_overLay.e-overlay", "", { zIndex: zindex - 1 });
            this._overLay.appendTo(document.body);
        },

        //handling the ajax request
        /**
         * To handle ajax request for the Dialog control		
		 * @private
         */
        _sendAjaxOptions: function (content, link) {
            //load waiting popup
            content.addClass("e-load");
            var proxy = this;
            var curTitle = this.model.title;
            var hrefLink = link;
            var ajaxOptions = {

                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, content, link, curTitle);
                    } catch (e) {

                    }
                }, "error": function (e) {
                    try {
                        proxy._ajaxErrorHandler(e.statusText, content, link, curTitle);
                    } catch (e) {

                    }
                }
            };
            $.extend(true, ajaxOptions, this.model.ajaxOptions);
            this._sendAjaxRequest(ajaxOptions);
        },
        /**
         * To send ajax request for the Dialog control		
		 * @private
         */
        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },
        /**
                 * Section For handle the ajaxSuccess event
                 * @private
                 */
        _ajaxSuccessHandler: function (data, content, link, curTitle) {
            content.removeClass("e-load");
            content.html(data).addClass("e-dialog-loaded").appendTo(this._ejDialog);
            content.appendTo(this.contentDiv);
            this._dialogPosition();
            this.open();
            this._trigger("ajaxSuccess", { data: data, url: link });
        },
        /**
         * Section For handle the ajaxSuccess event
		 * @private
         */
        _ajaxErrorHandler: function (data, content, link, curTitle) {
            content.addClass("e-dialog-loaded");
            content.appendTo(this.contentDiv);
            this._dialogPosition();
            this.open();
            this._trigger("ajaxError", { data: data, url: link });
        },
        /**
         * To set close icon for the dialog control
		 * @private
         */
        _closeIcon: function () {
            this._dialogClose = ej.util.buildTag("div.e-dialog-icon");
            var span = ej.util.buildTag("span.e-icon e-close", "", {}, { role: "presentation" });
            span.appendTo(this._dialogClose);
            this._dialogClose.appendTo(this._dialogTitlebar).attr("title", this.model.closeText);
            this._on(this._dialogClose, "touchstart mousedown", this._closeClick);
        },
        /**
         * To set collapsible icon for the dialog control.
		 * @private
         */
        _collapsibleIcon: function () {
            this._dialogCollapsible = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-icon e-collapse-arrow", "", {}, { role: "presentation" });
            span.appendTo(this._dialogCollapsible);
            this._dialogCollapsible.appendTo(this._dialogTitlebar);
            this._on(this._dialogCollapsible, "mousedown", this._collapsibleClick);
        },
        /**
         * To set maximize icon for the dialog control.
		 * @private
         */
        _maximizeIcon: function () {
            this._dialogMaximize = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-icon e-maximize", "", {}, { role: "presentation" });
            span.appendTo(this._dialogMaximize);
            this._dialogMaximize.appendTo(this._dialogTitlebar);
            this._on(this._dialogMaximize, "mousedown", this._maximizeClick);
        },
        /**
         * To set minimize icon for the dialog control.
		 * @private
         */
        _minimizeIcon: function () {
            this._dialogMinimize = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-icon e-minimize", "", {}, { role: "presentation" });
            span.appendTo(this._dialogMinimize);
            this._dialogMinimize.appendTo(this._dialogTitlebar);
            this._on(this._dialogMinimize, "mousedown", this._minimizeClick);
        },
        /**
         * To set pin icon for the dialog control.
		 * @private
         */
        _pinIcon: function () {
            this._dialogPin = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-icon e-unpin", "", {}, { role: "presentation" });
            span.appendTo(this._dialogPin);
            this._dialogPin.appendTo(this._dialogTitlebar);
            this._on(this._dialogPin, "mousedown", this._pinClick);
        },
        /**
         * To set custom icon for the dialog control.
		 * @private
         */
        _customIcon: function () {
            this._dialogCustom = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-dialog-custom", "", {}, { role: "presentation" });
            span.appendTo(this._dialogCustom);
            this._dialogCustom.appendTo(this._dialogTitlebar);
            if (!ej.isNullOrUndefined(this.model.customIconCss))
                span.addClass(this.model.customIconCss);
        },
        /**
         * To set size for the dialog control.
		 * @private
         */
        _setSize: function () {
            var mdl = this.model;
            this._ejDialog.css({ width: mdl.width, minWidth: mdl.minWidth });
            this._ejDialog.css({ height: mdl.height, minHeight: mdl.minHeight });
        },
        /**
         * To reset Scroller for the dialog control.
		 * @private
         */
        _resetScroller: function () {
            var scrHeight = parseInt(this.model.height) - $(this._ejDialog.find("div.e-titlebar")).outerHeight();
            this.contentDiv.ejScroller({ "width": this.model.width, "height": scrHeight });
            this.element.width("auto");
            this.scroller = this.contentDiv.data("ejScroller");
            this.scroller.refresh();
        },
        /**
         * To drag dialog control.
		 * @private
         */
        _dragDialog: function () {
            var proxy = this;
            this._dialogTitlebar.ejDraggable({
                handle: ".e-titlebar",
                cursorAt: { top: 0, left: 0 },
                dragStart: function (event) {
                    proxy._clickHandler();
                    if (proxy.dialogPin || !proxy.model.draggable || !proxy.model.enabled)
                        event.cancel = true;
                    if (proxy._trigger("dragStart", { event: event })) {
                        event.cancel = true;
                        return false;
                    }
                },
                drag: function (event) {
                    proxy._trigger("drag", { event: event });
                },
                dragStop: function (event) {
                    proxy._ejDialog.focus();
                    proxy._trigger("dragStop", { event: event });
                },
                helper: function (event) {
                    return $(proxy._ejDialog).addClass("dragClone");
                }
            });
        },
        /**
         * To resize dialog control.
		 * @private
         */
        _resizeDialog: function () {
            var proxy = this, started = false;
            this._ejDialog.find("div.e-resize-handle").ejResizable(
                {
                    minHeight: parseInt(proxy.model.minHeight),
                    minWidth: parseInt(proxy.model.minWidth),
                    maxHeight: parseInt(proxy.model.maxHeight),
                    maxWidth: parseInt(proxy.model.maxWidth),
                    handle: "e-widget-content",
                    resizeStart: function (event) {
                        !started && proxy._trigger("resizeStart", { event: event });
                        started = true;
                    },
                    resize: function (event) {
                        var reElement = $(event.element).parents("div.e-dialog-wrap");
                        proxy.model.height = $(reElement).height();
                        proxy.model.width = $(reElement).width();
                        proxy._setSize();
                        proxy._resetScroller();
                        proxy._trigger("resize", { event: event });
                    },
                    resizeStop: function (event) {
                        proxy._ejDialog.focus();
                        proxy._sizeInPercent();
                        started && proxy._trigger("resizeStop", { event: event });
                        started = false;
                    },
                    helper: function (event) {
                        var reElement = $(event.element).parents("div.e-dialog-wrap");
                        proxy.model.height = $(reElement).height();
                        proxy.model.width = $(reElement).width();
                        proxy._setSize();
                        proxy._resetScroller();
                        return $(proxy._ejDialog);
                    }
                });
        },
        /**
         * To set position for the dialog control.
		 * @private
         */
        _dialogPosition: function () {
            if (this._ejDialog.parents("form").length > 0 && ej.isNullOrUndefined(this.model.contentContainer)) {
                this._ejDialog.appendTo(this._ejDialog.parents("form"));
            }
            if (this.model.position != null) {
                this._ejDialog.css("position", "absolute");
                this._ejDialog.css("left", this.model.position.X + "px");
                this._ejDialog.css("top", this.model.position.Y + "px");
            }
            else {
                var x, y;
                if (!ej.isNullOrUndefined(this.model.contentContainer)) {
                    var location = $(this.model.contentContainer).position();
                    x = (($(this.model.contentContainer).outerWidth() - this._ejDialog.width()) / 2) + Math.ceil(location.left);
                    y = (($(this.model.contentContainer).outerHeight() - this._ejDialog.height()) / 2) + Math.ceil(location.top);
                }
                else {
                    x = ($(window).outerWidth() - this._ejDialog.outerWidth()) / 2;
                    y = ($(window).outerHeight() - this._ejDialog.outerHeight()) / 2;
                }
                this._ejDialog.css({ top: y, left: x });
                this._ejDialog.css("position", "absolute");
            }
        },
        /**
         * Section handles when close icon is clicked
		 * @private
         */
        _closeClick: function (event) {
            event.preventDefault();
            this.close(event);
        },
        /**
     * Section handles when collapsible icon is clicked
     * @private
     */
        _collapsibleClick: function (e) {
            if ($(e.target).hasClass("e-collapse-arrow")) {
                this._dialogCollapsible.find("span.e-collapse-arrow").removeClass("e-collapse-arrow").addClass("e-expand-arrow");
                this._ejDialog.find("div.e-resize-handle").hide();
                this._ejDialog.find(".e-widget-content").parent().slideUp("fast");
                this._ejDialog.removeClass("e-shadow");
                this._ejDialog.css("minHeight", "0");
                this._ejDialog.height("auto");
            }
            else if ($(e.target).hasClass("e-expand-arrow")) {
                this._dialogCollapsible.find("span.e-expand-arrow").removeClass("e-expand-arrow").addClass("e-collapse-arrow");
                this._ejDialog.addClass("e-shadow");
                this._ejDialog.find(".e-widget-content").parent().slideDown("fast");
                this._ejDialog.find("div.e-resize-handle").show();
                this._ejDialog.height(this.model.height);
            }
        },
        /**
         * Section handles when maximize icon is clicked
		 * @private
         */
        _maximizeClick: function (e) {
            var _target = $(e.target);
            if (_target.hasClass("e-maximize")) {
                this._ejDialog.css("top", "0px").css("left", "0px").css("position", "fixed");
                this._ejDialog.css({ width: "100%", height: "100%" });
                this.element.css({ width: "100%", height: "100%" });
                this.contentDiv.css({ width: "100%", height: "100%" });
                _target.removeClass("e-maximize").addClass("e-restore");
                this._maximize = true;
                this.dialogPin = true;
            }
            else if (_target.hasClass("e-restore")) {
                this.element.height("");
                this.element.width("");
                this._restoreDialog();
                _target.removeClass("e-restore").addClass("e-maximize");
            }
        },
        /**
         * Section handles when minimize icon is clicked
		 * @private
         */
        _minimizeClick: function (e) {
            var _target = $(e.target);
            if (_target.hasClass("e-minimize")) {
                if (this._maximize)
                    this._setSize();
                var top = ($(window).height() - this._ejDialog.height()) + this.element.height() + 14;
                this._ejDialog.css("top", "").css("bottom", "0").css("left", "0").css("position", "fixed");
                this._ejDialog.css("minHeight", "0");
                this._ejDialog.css("height", (this._ejDialog.height() - this.contentDiv.height()));
                this.contentDiv.hide();
                _target.parent().hide();
                this._dialogTitlebar.find(".e-maximize").removeClass("e-maximize").addClass("e-restore");
                this.dialogPin = true;
            }
        },
        /**
         * Section handles when pinicon is clicked
		 * @private
         */
        _pinClick: function (e) {
            var _target = $(e.target);
            if (_target.hasClass("e-unpin")) {
                this.dialogPin = true;
                _target.removeClass("e-unpin").addClass("e-pin");
            }
            else if (_target.hasClass("e-pin")) {
                this.dialogPin = false;
                _target.removeClass("e-pin").addClass("e-unpin");
            }
        },
        /**
         * to restore dialog control
		 * @private
         */
        _restoreDialog: function () {
            this.dialogPin = false;
            this.contentDiv.show();
            this._ejDialog.css("position", "absolute").css("bottom", "");
            this._setSize();
            this._dialogPosition();
            this._reRenderScroller();
            this._dialogTitlebar.find(".e-minimize").parent().show();
        },
        /**
         * Section handles  click event
		 * @private
         */
        _clickHandler: function (e) {
            var zindex = this._dialogMaxZindex();
            if (parseInt(this._ejDialog.css("zIndex")) < zindex) {
                this._ejDialog.css({ zIndex: zindex + 1 });
            }
        },
        /**
         * Section handles keydown event
		 * @private
         */
        _docKeyDown: function (e) {
            if (this.model.enabled) {
                if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
                else if (e.which) code = e.which; // ns4 and opera
                else code = e.charCode;
                if (this.model.closeOnEscape && code === 27) {
                    this.close(e);
                    e.preventDefault();
                }
            }
        },
        /**
         * Section handles keydown event
		 * @private
         */
        _keyDown: function (e) {
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            if (this.model.allowKeyboardNavigation && this.model.enabled && $(e.target).hasClass("e-dialog")) {
                if ($.inArray(code, this.keyConfigs) > -1 && this.model.draggable && !this.dialogPin) {
                    this._keyPressed(code, e.ctrlKey);
                    e.preventDefault();
                }
            }
        },
        /**
         * Section handles keypress event for the dialog control
		 * @private
         */
        _keyPressed: function (code, ctrlKey) {
            switch (code) {
                case 40:    //Down arrow
                    ctrlKey ? this._resetWidthHeight(this.model.width, this._ejDialog.height() + 3) : this._ejDialog.css("top", this._ejDialog.position().top + 3 + "px");
                    break;
                case 39:    //Right arrow
                    ctrlKey ? this._resetWidthHeight(this._ejDialog.width() + 3, this._ejDialog.height()) : this._ejDialog.css("left", this._ejDialog.position().left + 3 + "px");
                    break;
                case 38:    //Up arrow
                    ctrlKey ? this._resetWidthHeight(this._ejDialog.width() - 3, this._ejDialog.height()) : this._ejDialog.css("top", this._ejDialog.position().top - 3 + "px");
                    break;
                case 37:    //Left arrow
                    ctrlKey ? this._ejDialog.width(this._ejDialog.width() - 3) : this._ejDialog.css("left", this._ejDialog.position().left - 3 + "px");
                    break;
            }
        },
        /**
         * to set for the dialog control in percent
		 * @private
         */
        _sizeInPercent: function () {
            if (!this._enableWindowResize()) return false;
            var parentObj = this._getParentObj();
            this._widthPercent = this._convertPixelToPercentage(parentObj.outerWidth(), this._ejDialog.outerWidth());
            this._heightPercent = this._convertPixelToPercentage(parentObj.outerHeight(), this._ejDialog.outerHeight());
            if (this._widthPercent > 100) {
                this._widthPercent = 100;
                this._ejDialog.outerWidth(parentObj.outerWidth());
                this.model.width = this._ejDialog.width();
            }
            if (this._heightPercent > 100) {
                this._heightPercent = 100;
                this._ejDialog.outerHeight(parentObj.outerHeight());
                this.model.height = this._ejDialog.height();
            }
        },
        /**
         * to get the parent of the dialog control
		 * @private
         */
        _getParentObj: function () {
            if (!ej.isNullOrUndefined(this.model.contentContainer))
                return $(this.model.contentContainer);
            else
                return (this._ejDialog.parent()[0].nodeName == "BODY" ? $(window) : "");
        },
        /**
         * to convert the percentage value to pixels
		 * @private
         */
        _convertPercentageToPixel: function (parent, child) {
            return Math.round((child * parent) / 100);  //round       
        },
        /**
         * to convert the pixel value to percentage
		 * @private
         */
        _convertPixelToPercentage: function (parent, child) {
            return Math.round((child / parent) * 100); //round
        },
        //_setPosition: function () {
        //    if ((this.model.windowResizing) || ((isNaN(this.model.width)&&(this.model.width.indexOf("%") > 0))||(isNaN(this.model.height)&&(this.model.height.indexOf("%") > 0)))) this._dialogPosition();
        //},
        /**
         * section handles when window is resized
		 * @private
         */
        _reSizeHandler: function () {
            var parentObj;
            this._dialogPosition();
            parentObj = this._getParentObj();
            this._ejDialog.outerWidth(this._convertPercentageToPixel(parentObj.outerWidth(), this._widthPercent));
            this._ejDialog.outerHeight(this._convertPercentageToPixel(parentObj.outerHeight(), this._heightPercent));
            this.contentDiv.width(this._ejDialog.width());
            this.element.outerWidth(this.contentDiv.width());
            this.contentDiv.height(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
            this.element.outerHeight(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
            this.scroller = this.contentDiv.ejScroller({ width: this._ejDialog.width(), height: this.element.outerHeight(), rtl: this.model.rtl });
            this.scroller = this.contentDiv.data("ejScroller");
            this.scroller.refresh();
            this._dialogPosition();
        },
        /**
         * to enable window resize
		 * @private
         */
        _enableWindowResize: function () {
            if (this.model.windowResizing || ((isNaN(this.model.width) && (this.model.width.indexOf("%") > 0)) && (isNaN(this.model.height) && (this.model.height.indexOf("%") > 0) && (this.model.height != "auto"))))
                return true;
            else
                return false;
        },
        _wireResizing: function () {
            this._enableWindowResize() ? $(window).bind('resize', $.proxy(this._reSizeHandler, this)) : $(window).unbind('resize', $.proxy(this._reSizeHandler, this))
        },

        //-------------------- Event Wire-up -------------------------//
        /**
         * Wiring the events to dialog control		
		 * @private
         */
        _wireEvents: function () {
            //tabs Click evnts
            this._on($(document), "keydown", this._docKeyDown);
            this._on(this._ejDialog, "keydown", this._keyDown);
            this._wireResizing();
        },

        // ------------------Client side methods-------------------------//
        /**
         * section handles when dialog window is opened
		 * @private
         */
        open: function () {
            if (!this.model.enabled) return false;
            if (this._isOpen) {
                return true;
            }
            if (true == this._trigger("beforeOpen")) {
                return false;
            }
            this.element.css("display", "block");
            this._setZindex();
            this._dialogPosition();
            var proxy = this;
            this._ejDialog.fadeIn(400, "easeOutQuad", function () {
                // Animation complete
                proxy._trigger("open");
            });
            if (this.model.modal == true)
                this._createOverlay();
            this._ejDialog.eq(0).focus();
            this._isOpen = true;
            return this;
        },
        /**
         * section handles when dialog window is closed
		 * @private
         */
        close: function (event) {
            if (!this._isOpen || !this.model.enabled) {
                return true;
            }
            if (true == this._trigger("beforeClose", { event: event })) {
                return false;
            }
            this._isOpen = false;
            this.element.css("display", "none");
            var proxy = this;
            this._ejDialog.fadeOut(300, "easeOutQuad", function () {
                // Animation complete
                proxy._trigger("close", { event: event });
            });
            if (this.model.modal) {
                this._overLay.remove();
            }
            return this;
        },
        /**
         * to get dialog control isopened 
		 * @private
         */
        isOpened: function () {
            return this._isOpen;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejDropDownList
   * @requires jQuery
   * @requires ej.common.all
   * @classdesc Custom Design for Html Textbox control.
   * @example 
   * &lt;input type="text" id="drpdwn" /&gt; <br> 
   * &lt;script&gt;
   * // Create DropDownList
   * $('#drpdwn').ejDropDownList(); 	
   * &lt;/script&gt;
   */
    // ejDropdownlist is the plugin name 
    // "ej.Dropdownlist" is "namespace.className" will hold functions and properties

    ej.widget("ejDropDownList", "ej.DropDownList", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["select", "input"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-dropdownlist",

        // default model
        defaults: {
            /**		
            * Sets the root class for DropDownList theme. This cssClass API helps to use custom skinning option for DropDownList control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
            * //Initialize the DropDownList with the cssClass value specified
            * 	$("#drpdwn").ejDropDownList({ cssClass: 'gradient-lime'});
            * @example 
            * Get or set the cssClass API, after initialization:<br>
            *	//Gets the cssClass  value
            *	$("#drpdwn").ejDropDownList("option", "cssClass ");			
            *	//Sets the cssClass  value
            *	$("#drpdwn").ejDropDownList("option", "cssClass ", "gradient-lime" );
            * @memberof ejDropDownList
            * @instance
            */
            cssClass: "",
            /**		
            * Defines the default value to be display in the DropDownList textbox.
            * @default Null
            * @type {string}
            * @example 
            * //Initialize the DropDownList value property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ value: “audi” });
            * @example 
            * Get or set the value API, after initialization:<br>
            *	//Gets the value  value
            *	$("#drpdwn").ejDropDownList("option", "value");			
            *	//Sets the value  value
            *	$("#drpdwn").ejDropDownList("option", "value ", "audi" );
            * @memberof ejDropDownList
            * @instance
            */
            value: "",
            /**		
            * Defines the default text in the DropDownList textbox.
            * @default null
            * @type {string}
            * @example 
            * //Initialize the DropDownList text property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ text: “audi” });
            * @example 
            * Get or set the text API, after initialization:<br>
            *	//Gets the text  value
            *	$("#drpdwn").ejDropDownList("option", "text");			
            *	//Sets the text  value
            *	$("#drpdwn").ejDropDownList("option", "text ", "audi" );
            * @memberof ejDropDownList
            * @instance
            */
            text: "",
            /**		
            * Defines the default itemValue  in the DropDownList textbox.
            * @default Null
            * @type {string}
            * @example 
            * //Initialize the DropDownList itemValue property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ itemValue: “audi” });
            * @example 
            * Get or set the itemValue API, after initialization:<br>
            *	//Gets the text  value
            *	$("#drpdwn").ejDropDownList("option", "itemValue");			
            *	//Sets the text  value
            *	$("#drpdwn").ejDropDownList("option", "itemValue ", "audi" );
            * @memberof ejDropDownList
            * @instance
            */
            itemValue: "",
            /**		
            * Specifies the listSize  for DropDownList.
            * @default 0
            * @type {number}
            * @example 
            * // To set listSize  API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({  listSize : 2 });
            * @example 
            * //Get or set the listSize , after initialization:<br>
            *	// Gets the listSize  value.		
            *	$("#drpdwn").ejDropDownList("option", "listSize");
            *	// Sets the listSize value.
            *	$("#drpdwn").ejDropDownList("option", "listSize",2);
            * @memberof ejDropDownList
            * @instance
            */
            listSize: 0,
            /**		
            * Specifies the Data Source of the DropDownList.	
            * @default null
            * @type {data}
            * @example 
            * //To set dataSource API value during initialization  
            * 	$("#drpdwn").ejDropDownList({ dataSource: countries });				* 
            * @example 
            * //Get or set the dataSource API, after initialization:<br>
            *	//Gets the dataSource value  
            *	$("#drpdwn").ejDropDownList('option', 'dataSource');<br>			
            *	//Sets the dataSource value 
            *	$("#drpdwn").ejDropDownList('option', 'dataSource', countries );
            * @memberof ejDropDownList
            * @instance
            */
            dataSource: null,
            /**		
            * Specifies the query to retrieve the data from online server.	
            * @default null
            * @type {object}
            * @example 
            * //To set query API value during initialization  
            * var dataManger = ej.DataManager({       url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"});
            * var queryString = ej.Query().from("Suppliers").select("ContactName");
            * 	$("#drpdwn").ejDropDownList({ dataSource: dataManger, query: queryString, fields: { text: "ContactName" }});
            * @example 
            * //Get or set the query API, after initialization:<br>
            *	//Gets the dataSource value  
            *	$("#drpdwn").ejDropDownList('option',  'query');<br>			
            *	//Sets the query value 
            *	$("#drpdwn").ejDropDownList('option',  'query',  queryString );
            * @memberof ejDropDownList
            * @instance
            */
            query: null,
            /**		
            * Specifies mapping fields for the data items of the DropDownList textbox.	
            * @default null
            * @type {object}
            * @example 
            * //To set fields API value during initialization  
            * var countries = [{name: "Austria", key: "flag-dz"}, {name: "Argentina, key: "flag-ar"},{name: "Antarctica", key: "flag-am", htmlAttr: {"style": "border: 1px solid;"}} ];
            * 	$("#drpdwn").ejDropDownList({ dataSource: countries,       fields: { text: "name", value: "key" }});
            * @example 
            * //Get or set the fields  API, after initialization:<br>
            *	//Gets the fields  value  
            *	$("#drpdwn").ejDropDownList('option',  'fields' );<br>			
            *	//Sets the fields value 
            *	$("#drpdwn").ejDropDownList('option', 'fields', { text: "name", value: "key" } );
            * @memberof ejDropDownList
            * @instance
            */
            fields: null,
            /**		
            * Sets the watermark text. When the textbox is empty the watermark text is visible like a shaded text. 
            * @default "Enter text"
            * @type {string}
            * @example 
            * //Initialize the DropDownList with the watermark value specified
            * 	$("#drpdwn").ejDropDownList({ watermark: 'Enter text' });
            * @example 
            * Get or set the watermark API, after initialization:<br>
            *	//Gets the watermark  value
            *	$("#drpdwn").ejDropDownList("option", "watermark");			
            *	//Sets the watermark  value
            *	$("#drpdwn").ejDropDownList("option", "watermark ", "Enter text" );
            * @memberof ejDropDownList
            * @instance
            */
            watermark: null,
            /**		
            * Defines the height of the DropDownList textbox.
            * @default Null
            * @type {string | number}
            * @example 
            * //Initialize the DropDownList height property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ height: 30 });
            * @example 
            * Get or set the height API, after initialization:<br>
            *	//Gets the height  value
            *	$("#drpdwn").ejDropDownList("option", "height");			
            *	//Sets the height  value
            *	$("#drpdwn").ejDropDownList("option", "height ", 30 );
            * @memberof ejDropDownList
            * @instance
            */
            height: "",
            /**		
            * Defines the width of the DropDownList textbox.
            * @default Null
            * @type {string | number}
            * @example 
            * //Initialize the DropDownList width property with the width value specified
            * 	$("#drpdwn").ejDropDownList({ width: 30 });
            * @example 
            * Get or set the width API, after initialization:<br>
            *	//Gets the width  value
            *	$("#drpdwn").ejDropDownList("option", "width");			
            *	//Sets the width  value
            *	$("#drpdwn").ejDropDownList("option", "width ", 30 );
            * @memberof ejDropDownList
            * @instance
            */
            width: "",
            /**		
            * Defines the popupPanelHeight of the suggestion box.
            * @default "152px"
            * @type {string | number}
            * @example 
            * //Initialize the DropDownList popupPanelHeight property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ popupPanelHeight: '152px' });
            * @example 
            * Get or set the popupPanelHeight API, after initialization:<br>
            *	//Gets the popupPanelHeight  value
            *	$("#drpdwn").ejDropDownList("option", "popupPanelHeight");			
            *	//Sets the suggestionBoxHeight  value
            *	$("#drpdwn").ejDropDownList("option", "popupPanelHeight ", '152px' );
            * @memberof ejDropDownList
            * @instance
            */
            popupPanelHeight: "152px",
            /**		
            * Defines the popupPanelWidth of the suggestion box.
            * @default "auto"
            * @type {string | number}
            * @example 
            * //Initialize the DropDownList popupPanelWidth property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ popupPanelWidth: '152px' });
            * @example 
            * Get or set the popupPanelWidth API, after initialization:<br>
            *	//Gets the popupPanelWidth  value
            *	$("#drpdwn").ejDropDownList("option", "popupPanelWidth");			
            *	//Sets the suggestionBoxHeight  value
            *	$("#drpdwn").ejDropDownList("option", "popupPanelWidth ", 'auto' );
            * @memberof ejDropDownList
            * @instance
            */
            popupPanelWidth: "auto",
            /**		
            * Specifies the targetId for DropDownList.
            * @default null
            * @type {string}
            * @example 
            * // To set targetId API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({  ({  targetId :"target-Id" });
            * @example 
            * //Get or set the targetId , after initialization:<br>
            *	// Gets the targetId  value.		
            *	$("#drpdwn").ejDropDownList('option',  'targetId');
            *	// Sets the targetId value.
            *	$("#drpdwn").ejDropDownList('option', 'targetId', 'target-Id');
            * @memberof ejDropDownList
            * @instance
            */
            targetId: null,
            /**		
            * Specifies the template for DropDownList.
            * @default null
            * @type {string}
            * @example 
            * // To set template API value during initialization  . 	
            * var countries = [	{ text: "Algeria", icon: "dz.png" },	{ text: "Argentina", icon: "ar.png" },	{ text: "Armenia", icon: "am.png" },	{ text: "Brazil", icon: "br.png" }];
            * 	$("#drpdwn").ejDropDownList({ dataSource: countries, template: '<img src="../images/${icon}" alt="/${text}">'});
            * @example 
            * //Get or set the template , after initialization:<br>
            *	// Gets the template  value.		
            *	$("#drpdwn").ejDropDownList('option',  'template');
            *	// Sets the template value.
            *	$("#drpdwn").ejDropDownList('option', 'template', '<img src="../${icon}">');
            * @memberof ejDropDownList
            * @instance
            */
            template: null,
            /**		
            * Specifies the selectedItem  for DropDownList.
            * @default null
            * @type {number}
            * @example 
            * // To set selectedItem  API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({  selectedItem : 2 });
            * @example 
            * //Get or set the selectedItem , after initialization:<br>
            *	// Gets the selectedItem  value.		
            *	$("#drpdwn").ejDropDownList("option", "selectedItem");
            *	// Sets the selectedItem value.
            *	$("#drpdwn").ejDropDownList("option", "selectedItem",2);
            * @memberof ejDropDownList
            * @instance
            */
            selectedItem: null,
            /**		
            * Specifies the multiSelectedItemsIndex  for DropDownList.
            * @default []
            * @type {integerarray}
            * @example 
            * // To set multiSelectedItemsIndex   API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({  multiSelectedItemsIndex  : [1,2] });
            * @example 
            * //Get or set the multiSelectedItemsIndex  , after initialization:<br>
            *	// Gets the multiSelectedItemsIndex   value.		
            *	$("#drpdwn").ejDropDownList("option", "multiSelectedItemsIndex ");
            *	// Sets the multiSelectedItemsIndex  value.
            *	$("#drpdwn").ejDropDownList("option", "multiSelectedItemsIndex ",[1,2]);
            * @memberof ejDropDownList
            * @instance
            */
            multiSelectedItemsIndex: [],
            /**		
            * Specifies the selectedTo for DropDownList.
            * @default null
            * @type {string}
            * @example 
            * // To set selectedTo API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({  ({  selectedTo :"target-Id" });
            * @example 
            * //Get or set the selectedTo , after initialization:<br>
            *	// Gets the selectedTo  value.		
            *	$("#drpdwn").ejDropDownList('option',  'selectedTo');
            *	// Sets the selectedTo value.
            *	$("#drpdwn").ejDropDownList('option', 'selectedTo', 'target-Id');
            * @memberof ejDropDownList
            * @instance
            */
            selectedTo: null,
            /**		
            * DropDownList textbox to be displayed with rounded corner style.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the roundedCorner value specified.
            * $("#drpdwn").ejDropDownList({roundedCorner: true });
            * @example 
            * //Get or set the roundedCorner API, after initialization:<br>
            *	// Get the roundedCorner API value.		
            *	$("#drpdwn").ejDropDownList('option', 'roundedCorner');			
            *	// Set the roundedCorner API
            *	$("#drpdwn").ejDropDownList('option', 'roundedCorner',  true);
            * @memberof ejDropDownList
            * @instance
            */
            roundedCorner: false,
            /**		
            * DropDownList textbox to be displayed with Popup shown.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the showPopupOnLoad value specified.
            * $("#drpdwn").ejDropDownList({showPopupOnLoad: true });
            * @example 
            * //Get or set the showPopupOnLoad API, after initialization:<br>
            *	// Get the showPopupOnLoad API value.		
            *	$("#drpdwn").ejDropDownList('option', 'showPopupOnLoad');			
            *	// Set the showPopupOnLoad API
            *	$("#drpdwn").ejDropDownList('option', 'showPopupOnLoad',  true);
            * @memberof ejDropDownList
            * @instance
            */
            showPopupOnLoad: false,
            /**		
            * Sets the DropDownList textbox direction as right to left alignment.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the rtl  value specified.
            * $("#drpdwn").ejDropDownList({ rtl : false });
            * @example 
            * //Get or set the rtl  API, after initialization:<br>
            *	// Get the rtl API value.		
            *	$("#drpdwn").ejDropDownList('option', 'rtl');			
            *	// Set the rtl  API
            *	$("#drpdwn").ejDropDownList('option', 'rtl',  true);
            * @memberof ejDropDownList
            * @instance
            */
            rtl: false,
            /**		
            * When this property sets to false, it disables the DropDownList control.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the enabled  value specified.
            * $("#drpdwn").ejDropDownList({ enabled : false });
            * @example 
            * //Get or set the enabled  API, after initialization:<br>
            *	// Get the enabled API value.		
            *	$("#drpdwn").ejDropDownList('option', 'enabled');			
            *	// Set the enabled  API
            *	$("#drpdwn").ejDropDownList('option', 'enabled',  true);
            * @memberof ejDropDownList
            * @instance
            */
            enabled: true,
            /**		
            * Sets the case sensitivity of the search operation..
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the caseSensitive value specified.
            * $("#drpdwn").ejDropDownList({ caseSensitive: true });
            * @example 
            * //Get or set the caseSensitive API, after initialization:<br>
            *	// Get the caseSensitive API value.		
            *	$("#drpdwn").ejDropDownList('option', 'caseSensitive');			
            *	// Set the caseSensitive API
            *	$("#drpdwn").ejDropDownList('option', 'caseSensitive',  true);
            * @memberof ejDropDownList
            * @instance
            */
            caseSensitive: false,
            /**		
            * Specifies the multi selection option in DropDownList with the help of checkbox control. For this we have to enable it by showCheckbox option true.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the showCheckbox value specified.
            * $("#drpdwn").ejDropDownList({ showCheckbox: true });
            * @example 
            * //Get or set the showCheckbox API, after initialization:<br>
            *	// Get the showCheckbox API value.		
            *	$("#drpdwn").ejDropDownList('option', 'showCheckbox');			
            *	// Set the showCheckbox API
            *	$("#drpdwn").ejDropDownList('option', 'showCheckbox',  true);
            * @memberof ejDropDownList
            * @instance
            */
            showCheckbox: false,
            /**		
            * Specifies to select all the items of DropDownList can be done with the help of this checkAll property, it supports only when the showCheckbox property true.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the checkAll value specified.
            * $("#drpdwn").ejDropDownList({ showCheckbox: true, checkAll: true });
            * @example 
            * //Get or set the checkAll API, after initialization:<br>
            *	// Get the checkAll API value.		
            *	$("#drpdwn").ejDropDownList('option', 'checkAll');			
            *	// Set the checkAll API
            *	$("#drpdwn").ejDropDownList('option', 'checkAll',  true);
            * @memberof ejDropDownList
            * @instance
            */
            checkAll: false,
            /**		
            * Specifies to unselect all the items of DropDownList can be done with the help of this checkAll property, it supports only when the showCheckbox property true.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the uncheckAll value specified.
            * $("#drpdwn").ejDropDownList({ showCheckbox: true, checkAll: true });
            * @example 
            * //Get or set the uncheckAll API, after initialization:<br>
            *	// Get the uncheckAll API value.		
            *	$("#drpdwn").ejDropDownList('option', 'uncheckAll');			
            *	// Set the uncheckAll API
            *	$("#drpdwn").ejDropDownList('option', 'uncheckAll',  true);
            * @memberof ejDropDownList
            * @instance
            */
            uncheckAll: false,
            /**		
            * Save current model value to browser cookies for state maintains. While refresh the DropDownList control page retains the model value apply from browser cookies.  
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the persist  value specified.
            * $("#drpdwn").ejDropDownList({ persist : false });
            * @example 
            * //Get or set the persist  API, after initialization:<br>
            *	// Get the persist API value.		
            *	$("#drpdwn").ejDropDownList('option', 'persist');			
            *	// Set the persist  API
            *	$("#drpdwn").ejDropDownList('option', 'persist',  true);
            * @memberof ejDropDownList
            * @instance
            */
            persist: false,
            /**		
            * Indicates that the incrementalSearch  can only be displayed.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the incrementalSearch with the value specified.
            * $("#drpdwn").ejDropDownList({incrementalSearch: true });
            * @example 
            * //Get or set the incrementalSearch API, after initialization:<br>
            *	// Get the incrementalSearch API value.		
            *	$("#drpdwn").ejDropDownList('option', 'incrementalSearch');			
            *	// Set the incrementalSearch API
            *	$("#drpdwn").ejDropDownList('option', 'incrementalSearch',  true);
            * @memberof ejDropDownList
            * @instance
            */
            incrementalSearch: false,
            /**		
            * Indicates that the DropDownList textbox values can only be read.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the DropDownList with the readOnly value specified.
            * $("#drpdwn").ejDropDownList({readOnly: true });
            * @example 
            * //Get or set the readOnly API, after initialization:<br>
            *	// Get the readOnly API value.		
            *	$("#drpdwn").ejDropDownList('option', 'readOnly');			
            *	// Set the readOnly API
            *	$("#drpdwn").ejDropDownList('option', 'readOnly',  true);
            * @memberof ejDropDownList
            * @instance
            */
            readOnly: false,
            /**		
            * Indicates that the multiSelectMode values can only be read.
            * @default true
            * @type {boolean}
            * @example 
            * // Initialize the multiSelectMode with the value specified.
            * $("#drpdwn").ejDropDownList({multiSelectMode: true });
            * @example 
            * //Get or set the multiSelectMode API, after initialization:<br>
            *	// Get the multiSelectMode API value.		
            *	$("#drpdwn").ejDropDownList('option', 'multiSelectMode');			
            *	// Set the multiSelectMode API
            *	$("#drpdwn").ejDropDownList('option', 'multiSelectMode',  true);
            * @memberof ejDropDownList
            * @instance
            */
            multiSelectMode: false,

            /**     
           * Fires when create successfully.
           * @event
           * @name ejDropDownList#create 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the DropDownList model
           * @param {string}  argument.type returns the name of the event
           * @example 
           * //change event for DropDownList
           * $("#drpdwn").ejDropDownList({ 
           *   	create: function(args) {
           * });      
           * @memberof ejDropDownList
           * @instance
           */
            create: null,
            /**     
          * Fires when popupHide successfully.
          * @event
          * @name ejDropDownList#popupHide 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DropDownList model
          * @param {string}  argument.type returns the name of the event
          * @example 
          * //popupHide event for DropDownList
          * $("#drpdwn").ejDropDownList({ 
          *   	popupHide: function(args) {
          * });      
          * @memberof ejDropDownList
          * @instance
          */
            popupHide: null,
            /**     
         * Fires when popupShown successfully.
         * @event
         * @name ejDropDownList#popupShown 	
         * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
         * @param {object}  argument.model returns the DropDownList model
         * @param {string}  argument.type returns the name of the event
         * @example 
         * //popupShown event for DropDownList
         * $("#drpdwn").ejDropDownList({ 
         *   	popupShown: function(args) {
         * });      
         * @memberof ejDropDownList
         * @instance
         */
            popupShown: null,
            /**     
         * Fires when beforePopupShown successfully.
         * @event
         * @name ejDropDownList#beforePopupShown 	
         * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
         * @param {object}  argument.model returns the DropDownList model
         * @param {string}  argument.type returns the name of the event
         * @example 
         * //beforePopupShown event for DropDownList
         * $("#drpdwn").ejDropDownList({ 
         *   	beforePopupShown: function(args) {
         * });      
         * @memberof ejDropDownList
         * @instance
         */
            beforePopupShown: null,
            /**     
           * Fires when change successfully.
           * @event
           * @name ejDropDownList#change 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the DropDownList model
           * @param {string}  argument.type returns the name of the event
           * @example 
           * //change event for DropDownList
           * $("#drpdwn").ejDropDownList({ 
           *   	change: function(args) {
           * });      
           * @memberof ejDropDownList
           * @instance
           */
            change: null,
            /**     
          * Fires when select successfully.
          * @event
          * @name ejDropDownList#select 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DropDownList model
          * @param {string}  argument.type returns the name of the event
          * @example 
          * //select event for DropDownList
          * $("#drpdwn").ejDropDownList({ 
          *   	select: function(args) {
          * });      
          * @memberof ejDropDownList
          * @instance
          */
            select: null,
            /**     
          * Fires when checkChange successfully.
          * @event
          * @name ejDropDownList#checkChange 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DropDownList model
          * @param {string}  argument.type returns the name of the event
          * @example 
          * //checkChange event for DropDownList
          * $("#drpdwn").ejDropDownList({ 
          *   	checkChange: function(args) {
          * });      
          * @memberof ejDropDownList
          * @instance
          */
            checkChange: null,
            /**     
          * Fires when destroy successfully.
          * @event
          * @name ejDropDownList#destroy 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DropDownList model
          * @param {string}  argument.type returns the name of the event
          * @example 
          * //destroy event for DropDownList
          * $("#drpdwn").ejDropDownList({ 
          *   	destroy: function(args) {
          * });      
          * @memberof ejDropDownList
          * @instance
          */
            destroy: null

        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            listSize: "number",
            watermark: "string",
            height: "string",
            width: "string",
            popupPanelHeight: "string",
            popupPanelWidth: "string",
            template: "string",
            selectedItem: "number",
            selectedTo: "string",
            roundedCorner: "boolean",
            showPopupOnLoad: "boolean",
            rtl: "boolean",
            persist: "boolean",
            enabled: "boolean",
            readOnly: "boolean",
            multiSelectMode: "boolean",
            dataSource: "data",
            multiSelectedItemsIndex: "data"
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),
        /**
      * To enable the DropDownList  
      * @return jQuery
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
      * &lt;script&gt;
      * // Create DropDownList
      * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
      * DropDownListObj.enable(); // enable the DropDownList
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
      * &lt;script&gt;
      * // Create DropDownList
      * $('#drpdwn').ejDropDownList(); 	
      * &lt;/script&gt;
      *@memberof ejDropDownList
      * @instance
      */
        //public methods
        enable: function () {
            if (this.element.hasClass("e-disable")) {
                this.target.disabled = false;
                this.model.enabled = true;
                this.element.removeClass('e-disable');
                this.dropdownbutton.removeClass('e-disable');
                this.element.bind("mousedown", $.proxy(this._OnDropdownClick, this));
                this.dropdownbutton.bind("mousedown", $.proxy(this._OnDropdownClick, this));
                if (this.model.multiSelectMode) this._ulBox.removeClass("e-disable");
            }
        },
        /**
     * To disable the DropDownList  
     * @return jQuery
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
     * &lt;script&gt;
     * // Create DropDownList
     * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
     * DropDownListObj.disable(); // disable the DropDownList
     * &lt;/script&gt;
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
     * &lt;script&gt;
     * // Create DropDownList
     * $('#drpdwn').ejDropDownList(); 	
     * &lt;/script&gt;
     *@memberof ejDropDownList
     * @instance
     */
        disable: function () {
            if (!this.element.hasClass("e-disable")) {
                this.target.disabled = true;
                this.model.enabled = false;
                this.element.addClass('e-disable');
                this.dropdownbutton.addClass('e-disable');
                if (this.model.multiSelectMode) this._ulBox.addClass("e-disable");
                this.element.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
                this.dropdownbutton.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
            }
        },
        /**
      * Returns the current value selected in the DropDownList textbox.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
      * &lt;script&gt;
      * // Create DropDownList
      * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
      * DropDownListObj.getValue(); // getValue of the DropDownList text
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
      * &lt;script&gt;
      * // Create DropDownList
      * $('#drpdwn').ejDropDownList(); 	
      * &lt;/script&gt;
      *@memberof ejDropDownList
      * @instance
      */
        getValue: function () {
            if (this.element.hasClass("e-watermark"))
                return "";
            return this.target.value;
        },
        _setValue: function (value) {
            if (!this.model.enabled) return false;
            if (this.element.hasClass("e-watermark"))
                this.element.removeClass("e-watermark")
            this.element.val(value);
        },
        _setText: function (text) {
            this.model.text = text;
        },
        _setItemValue: function (itemValue) {
            this.model.itemValue = itemValue;
        },
        _changeWatermark: function (text) {
            if (!this.model.enabled) return false;
            if (this.element.hasClass("e-watermark"))
                this.element.val(text);
        },
        /**
     * popup list hide in the DropDownList textbox.
     * @return jQuery
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
     * &lt;script&gt;
     * // Create DropDownList
     * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
     * DropDownListObj.popupListHide(); // popupListHide of the DropDownList 
     * &lt;/script&gt;
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
     * &lt;script&gt;
     * // Create DropDownList
     * $('#drpdwn').ejDropDownList(); 	
     * &lt;/script&gt;
     *@memberof ejDropDownList
     * @instance
     */
        popupListHide: function () {
            if (!this.model.enabled) return false;
            if (this.ultag.find('li').length > 0)
                this._hideResult();
        },
        /**
   * popup list show in the DropDownList textbox.
   * @return jQuery
   * @example 
   * &lt;input type="text" id="drpdwn" /&gt; <br> 
   * &lt;script&gt;
   * // Create DropDownList
   * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
   * DropDownListObj.popupListShow(); // popupListHide of the DropDownList 
   * &lt;/script&gt;
   * @example 
   * &lt;input type="text" id="drpdwn" /&gt; <br> 
   * &lt;script&gt;
   * // Create DropDownList
   * $('#drpdwn').ejDropDownList(); 	
   * &lt;/script&gt;
   *@memberof ejDropDownList
   * @instance
   */
        popupListShow: function () {
            if (!this.model.enabled) return false;
            if (this.ultag.find('li').length > 0)
                this._showResult();
        },
        /**
     * Clears the text in the DropDownList textbox.
     * @return jQuery
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
     * &lt;script&gt;
     * // Create DropDownList
     * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
     * DropDownListObj.clearText(); // clear the DropDownList text
     * &lt;/script&gt;
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
     * &lt;script&gt;
     * // Create DropDownList
     * $('#drpdwn').ejDropDownList(); 	
     * &lt;/script&gt;
     *@memberof ejDropDownList
     * @instance
     */
        clearText: function () {
            if (this.model.enabled) {
                this.element.val("");
                this.value("");
                this.model.text = "";
                if (this.model.watermark != null) {
                    if ($.trim(this.element.val()) === '') {
                        this.element.val(this.model.watermark);
                        this.element.addClass("e-watermark");
                    }
                }
            }
        },
        /**
   * This method is used to select a list item in DropDownList using given index field.
   * @example 
   * &lt;input type="text" id="drpdwn" /&gt; <br> 
   * &lt;script&gt;
   * // Create DropDownList
   * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
   * DropDownListObj.selectItemByIndex(integer); // selectItemByIndex for the DropDownList text
   * &lt;/script&gt;
   * @example 
   * &lt;input type="text" id="drpdwn" /&gt; <br> 
   * &lt;script&gt;
   * // Create DropDownList
   * $('#drpdwn').ejDropDownList(); 	
   * &lt;/script&gt;
   *@memberof ejDropDownList
   * @instance
   */
        selectItemByIndex: function (index) {
            if (!this.model.enabled) return false;
            if (index != null) {
                this._activeItem = index;
                if (this._activeItem < 0) {
                    this._activeItem = 0;
                }
                else if (this._activeItem == this._listSize - 1) {
                    this._activeItem = this._listSize - 1;
                }
                if (this.model.template) {
                    this._templateValue();
                } else {
                    this._enterTextBoxValue();
                }
            }
        },
        unselectItemByIndex: function (index) {
            if (!this.model.enabled) return false;
            if (this.selectedIndexValue == index) {
                this._activeItem = index;
                this._removeTextBoxValue();
            }
        },
        /**
* This method is used to select a list item in DropDownList using given index field.
* @example 
* &lt;input type="text" id="drpdwn" /&gt; <br> 
* &lt;script&gt;
* // Create DropDownList
* var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
* DropDownListObj.selectItemByValue(string); // selectItemByValue for the DropDownList text
* &lt;/script&gt;
* @example 
* &lt;input type="text" id="drpdwn" /&gt; <br> 
* &lt;script&gt;
* // Create DropDownList
* $('#drpdwn').ejDropDownList(); 	
* &lt;/script&gt;
*@memberof ejDropDownList
* @instance
*/
        selectItemByValue: function (idvalue) {
            if (!this.model.enabled) return false;
            var i, hidelement;
            this.listitems = this.popupList.find("ol,ul").children("li");
            for (i = 0; i < this.listitems.length; i++) {
                if ($(this.listitems[i]).attr("value") != null) {
                    this._selectedValue = $(this.listitems[i]).attr("value");
                    if (this._selectedValue == idvalue) {
                        this._activeItem = i;
                        this._enterTextBoxValue();
                        return false;
                    }
                }
            }
        },
        unselectItemByValue: function (idvalue) {
            if (!this.model.enabled) return false;
            var i, hidelement;
            this.listitems = this.popupList.find("ol,ul").children("li");
            for (i = 0; i < this.listitems.length; i++) {
                if ($(this.listitems[i]).attr("value") != null) {
                    this._selectedValue = $(this.listitems[i]).attr("value");
                    if (this._selectedValue == idvalue) {
                        this._activeItem = i;
                        this._removeTextBoxValue();
                        return false;
                    }
                }
            }
        },
        /**
* This method is used to select a list item in DropDownList using given index field.
* @example 
* &lt;input type="text" id="drpdwn" /&gt; <br> 
* &lt;script&gt;
* // Create DropDownList
* var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
* DropDownListObj.selectItemByText(string); // selectItemByText for the DropDownList text
* &lt;/script&gt;
* @example 
* &lt;input type="text" id="drpdwn" /&gt; <br> 
* &lt;script&gt;
* // Create DropDownList
* $('#drpdwn').ejDropDownList(); 	
* &lt;/script&gt;
*@memberof ejDropDownList
* @instance
*/
        selectItemByText: function (idvalue) {
            if (!this.model.enabled) return false;
            var i, hidelement;
            this.listitems = this.popupList.find("ol,ul").children("li");
            for (i = 0; i < this.listitems.length; i++) {
                this.selectedTextValue = $(this.listitems[i]).text();
                if (this.selectedTextValue == idvalue) {
                    this._activeItem = i;
                    this._enterTextBoxValue();
                    return false;
                }
            }
        },

        unselectItemByText: function (idvalue) {
            if (!this.model.enabled) return false;
            var i, hidelement;
            this.listitems = this.popupList.find("ol,ul").children("li");
            for (i = 0; i < this.listitems.length; i++) {
                this.unselectedTextValue = $(this.listitems[i]).text();
                if (this.unselectedTextValue == idvalue) {
                    this._activeItem = i;
                    this._removeTextBoxValue();
                    return false;
                }
            }
        },

        //get the selected items
        getCheckedItems: function () {
            var checkeditems = "";
            for (i = 0; i < this.listitems.length; i++) {
                if ($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked') == true) {
                    checkeditems += $(this.listitems[i]).text() + ",";
                }
            }
            return checkeditems;
        },
        getSelectedItem: function () {
            return this.selectedTextValue;
        },
        getSelectedIndex: function () {
            if (this.element[0].value != "")
                return this.selectedIndexValue;
            else return "";
        },
        getSelectedValue: function () {
            if (this.element[0].value != "")
                return this._selectedValue;
            else return "";
        },
        _removeText: function (currentValue) {
            var res, index
            res = (this.element[0].value).split(",");
            index = res.indexOf(currentValue);
            if (index != -1) {
                res.splice(index, 1);
                this.element[0].value = res;
            }
        },
        _addText: function (currentValue) {
            if ((this.model.watermark) && (this.element.hasClass("e-watermark"))) this.element.val(this.element[0].value.replace(this.model.watermark, ""));
            this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
            if (((this._delimiterIndex == -1) || (this._delimiterIndex != this.element[0].value.length - 1)) && (this.element[0].value != "")) {
                this.element.val(this.element[0].value + ",");
                this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
            }
            if (!this._checkContains(currentValue))
                this.element.val(this.element[0].value.substr(0, this._delimiterIndex + 1) + currentValue);
        },
        _checkContains: function (Value) {
            var chkValue = Value;
            var values = this.element[0].value.split(",");
            this.contains = false;
            for (var i = 0; i < values.length; i++) {
                if (values[i] == chkValue)
                    this.contains = true;
            }
            return this.contains;
        },
        _init: function () {
            if ((this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type'))) || this.element.is("select")) {
                this._initialize();
                this._render();
                this._enabled(this.model.enabled);
                this._wireEvents();
                this._initValue = false;
            }
        },
        _setInitialPopup: function (value) {
            value == false ? this._hideResult() : this._showResult();
        },
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.wrapper.removeClass(this.model.cssClass).addClass(skin);
                this.popupList.removeClass(this.model.cssClass).addClass(skin);
            }
        },

        _setRTL: function (boolean) {
            if (this.model.rtl != boolean) {
                this.model.rtl = boolean;
                this._RightToLeft();
                this._dropbtnRTL();
            }
        },
        //height width
        _changeHeight: function (height) {
            this.wrapper.height(height);
            this._reposDDB();
        },
        _changeWidth: function (width) {
            this.wrapper.width(width);
            this._setListWidth();
        },
        /**
       * To configure the properties at runtime using SetModel		
       * @private
       */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            if (options.dataSource != null) {
                this.element.val('');
            }
            for (option in options) {
                switch (option) {
                    case "dataSource": this._checkModelDataBinding(options[option]); break;
                    case "query": this._queryCheck(options[option]); break;
                    case "fields": this.model.fields = options[option]; this._checkModelDataBinding(this.model.dataSource); break;
                    case "template": this.model.template = options[option]; this._checkModelDataBinding(this.model.dataSource); break;
                    case "value": this._setValue(ej.util.getVal(options[option])); break;
                    case "text": this._setText(options[option]); break;
                    case "itemValue": this._setItemValue(options[option]); break;
                    case "rtl": this._setRTL(options[option]); break;
                    case "enabled": this._enabled(options[option]); break;
                    case "height": this._changeHeight(options[option]); break;
                    case "width": this._changeWidth(options[option]); break;
                    case "popupPanelHeight": this.model.popupPanelHeight = options[option]; this._setListHeight(options[option]); break;
                    case "popupPanelWidth": this.model.popupPanelWidth = options[option]; this._setListWidth(); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showCheckbox": this._checkboxHideShow(options[option]); break;
                    case "checkAll": this._setCheckAll(options[option]); break;
                    case "uncheckAll": this._setUncheckAll(options[option]); break;
                    case "watermark": this._changeWatermark(options[option]); break;
                    case "roundedCorner": this._roundedCorner(options[option]); this.model.roundedCorner = options[option]; break;
                    case "showPopupOnLoad": this._setInitialPopup(options[option]); break;
                    case "targetId": this.model.targetId = options[option]; this._showFullList(); break;
                    case "selectedItem": this.selectItemByIndex(options[option]); break;
                    case "multiSelectedItemsIndex": this._selectCheckedItem(options[option]); this.model.multiSelectedItemsIndex = options[option]; break;
                    case "multiSelectMode": this.model.multiSelectMode = options[option];
                        options[option] ? this._renderBoxModel() : this._destroyBoxModel();
                        break;
                }
            }
        },
        /**
 * destroy in the DropDownList textbox.
 * @return jQuery
 * @example 
 * &lt;input type="text" id="DropDownList" /&gt; <br> 
 * &lt;script&gt;
 * // Create DropDownList
 * var DropDownListObj  = $("#DropDownList").data("ejDropDownList");
 * DropDownListObj.destroy(); // hide the DropDownList text
 * &lt;/script&gt;
 * @example 
 * &lt;input type="text" id="DropDownList" /&gt; <br> 
 * &lt;script&gt;
 * // Create DropDownList
 * $('#DropDownList').ejDropDownList(); 	
 * &lt;/script&gt;
 *@memberof ejDropDownList
 * @instance
 */

        _destroy: function () {
            this._destroyDropdown();
            this.element.removeClass("e-input " + this.model.cssClass + " e-watermark");
            this.element.attr('accesskey', this.wrapper.attr('accesskey'));
            this.element[0].value = "";
            if (this.model.dataSource != null)
                this.popupPanelWrapper.remove();
            else {
                this.docbdy.append(this.ultag.html());
                this.popupPanelWrapper.remove();
            }
        },
        /* cookie */

        _finalize: function () {
            if (this.model.selectedItem != null) {
                this.selectItemByIndex(this.model.selectedItem);
            } else if ((this.model.showCheckbox == true) && (this.model.multiSelectedItemsIndex.length > 0)) {
                this._selectCheckedItem(this.model.multiSelectedItemsIndex);
            }
        },

        /**/
        _initialize: function () {
            if (this.element.is("select")) {
                this.selectelement = true;
                this._renderSelectToDropdown();
            }
            this.target = this.element[0];
            this.showPopupBox = false;
            this._queryString = null;
            this.suggLen = 0;
            this._itemId = null;
            this.checkedStatus = false;
            this._incqueryString = "";
            this._activeItem = null;
            this.ddWidth = 0;
            this._initValue = true;
        },
        _renderSelectToDropdown: function () {
            var i, optionLength;
            this.inputElement = ej.buildTag("input.e-dropdownlist#" + this.element[0].id + "_input", "", {}, { "type": "text" });
            this.inputElement.insertAfter(this.element);
            this.optionDiv = ej.buildTag("div#" + this.element[0].id + "_list");
            this.optionDiv.insertAfter(this.inputElement);
            this.optionUl = ej.buildTag("ul");
            this.optionDiv.append(this.optionUl);
            this.selectOptions = this.element;
            this.selectOptions.attr('id', this.element[0].id);
            this.selectOptionItems = this.element.children("option");
            optionLength = this.selectOptionItems.length;
            this.optionDummyUl = $();

            for (i = 0; i < optionLength; i++) {
                if ((this.selectOptionItems[i].text != null) && ((this.selectOptionItems[i].text != ""))) {
                    this.optionLi = ej.buildTag("li", this.selectOptionItems[i].text, {}, { value: this.selectOptionItems[i].value });
                    this.optionDummyUl.push(this.optionLi[0]);
                }
            }
            this.optionUl.append(this.optionDummyUl);

            this.element.css('display', 'none');
            this.element = this.inputElement;
        },


        _render: function () {
            this._renderDropdown();
            this._setWatermark();
            this._renderPopupPanelWrapper();
            this._initDataSource(this.model.dataSource);
            this._showFullList();
            this._finalize();
        },

        _queryCheck: function (value) {
            this.model.query = value;
            this._checkModelDataBinding(this.model.dataSource);
        },

        _checkModelDataBinding: function (source) {
            this.model.dataSource = source;
            if (source != null) {
                if (ej.DataManager && source instanceof ej.DataManager) {
                    this._initDataSource(source);
                } else {
                    this._showFullList();
                }
            }
            else {
                this.ultag.empty();
            }
        },
        _initDataSource: function (source) {
            var proxy = this;
            if (ej.DataManager && source instanceof ej.DataManager) {
                proxy.popupListItems = proxy.model.dataSource;
                proxy._addLoadingClass();
                var queryPromise = source.executeQuery(this._getQuery());
                queryPromise.done(function (e) {
                    proxy.popupListItems = e.result;
                    proxy._removeLoadingClass();
                    proxy._showFullList();

                }).fail(function (e) {
                    proxy.model.dataSource = null;
                    proxy._addLoadingClass();
                });
            }
        },
        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },

        _addLoadingClass: function () {
            this.dropdownbutton.addClass("e-load");
            this.drpbtnspan.removeClass("e-icon e-down-arrow");
            this.model.readOnly = true;
        },
        _removeLoadingClass: function () {
            this.dropdownbutton.removeClass("e-load");
            this.drpbtnspan.addClass("e-icon e-down-arrow");
            this.model.readOnly = false;
        },
        //render drop button text box
        _renderDropdown: function () {
            this.element.addClass("e-input " + this.model.cssClass);
            if (this.value())
                this.element.val(this.value());
            this._checkNameAttr();
            this.wrapper = ej.buildTag("span.e-ddl e-widget " + this.model.cssClass + "#" + this.target.id + "_wrapper", "", {}, { "tabindex": "0", "accesskey": this.element.attr("accesskey") });//
            this.container = ej.buildTag("span.e-in-wrap e-box " + this.model.cssClass + "#" + this.target.id + "_container");
            this.element.removeAttr('accesskey');
            this.element.attr({ "readonly": true, "tabindex": -1, "role": "combobox", "aria-expanded": false, "aria-autocomplete": "list", "aria-haspopup": true, "aria-owns": this.element[0].id + "_popup" });
            this.drpbtnspan = ej.buildTag("span.e-icon e-down-arrow", "", {}, { "aria-label": "select" });
            this.dropdownbutton = ej.buildTag("span.e-select#" + this.target.id + "_dropdown", "", {}, { "role": "button" }).append(this.drpbtnspan);
            this.container.insertAfter(this.element);
            this.container.append(this.element);
            this.container.append(this.dropdownbutton);
            this.wrapper.insertBefore(this.container);
            this.wrapper.append(this.container);
            if (this.selectelement) {
                this.selectOptions.insertBefore(this.element);
            }
            this._setDimentions();
            this._RightToLeft();
            this.ddWidth = (this.dropdownbutton.outerWidth() > 0) ? this.dropdownbutton.outerWidth() : 24;
            this.element.bind("mousedown", $.proxy(this._OnDropdownClick, this));
            this.dropdownbutton.bind("mousedown", $.proxy(this._OnDropdownClick, this));
            this._roundedCorner(this.model.roundedCorner);
            this._renderBoxModel();
        },
        /*name attribute for input box*/
        _checkNameAttr: function () {
            if (!this.element.attr("name")) this.element.attr({ "name": this.element[0].id });
        },
        _renderBoxModel: function () {
            if (!this.model.multiSelectMode) return false;
            this.element.css('display', 'none');
            this._ulBox = ej.buildTag("ul.e-ul e-boxes");
            this.container.prepend(this._ulBox);
            this._ulBox.css('min-height', '30px');
            this.wrapper.css({ 'height': 'auto' });

            this._on(this.container, "mousedown", function (e) {
                if (!this.model.enabled) return false;
                var $target = $(e.target);
                if ($target.hasClass("e-options")) {
                    if (!e.ctrlKey && $target.siblings().hasClass("e-active")) this._removeActive();
                    if ($target.hasClass("e-active")) $target.removeClass("e-active");
                    else $target.addClass("e-active");
                }
                if (!e.ctrlKey && ($target.hasClass("e-boxes"))) this._removeActive();
            });
        },
        //render popup pannel wrapper
        _renderPopupPanelWrapper: function () {
            this.popupPanelWrapper = ej.buildTag("div#" + this.element[0].id + "_popup_wrapper");
            $('body').append(this.popupPanelWrapper);
            //render popup list 
            this.popupList = ej.buildTag("div.e-ddl-popup e-box e-popup e-widget " + this.model.cssClass + "#" + this.element[0].id + "_popup", "", { "visibility": "hidden" }, { "tabIndex": 0 });
            this.popup = this.popupList;
            this.popupScroller = ej.buildTag("div"); this.ultag = ej.buildTag("ul.e-ul", "", {}, { "role": "listbox" });
            this.popupScroller.append(this.ultag);
            this.popupList.append(this.popupScroller);
            this.popupPanelWrapper.append(this.popupList);
        },
        //render popup list items
        _renderPopupList: function () {
            var list = this.popupListItems, i, ulempty, ulno, litag, _id, _txt, mapper = this.model.fields, predecessor;
            this.ultag.empty();
            this.dummyUl = $();
            if (this.model.rtl)
                this.popupList.addClass("e-rtl");
            if (this.model.dataSource == null || this.model.dataSource.length < 1) {
                predecessor = this.element.parents().last();
                if (this.model.targetId)
                    this.docbdy = predecessor.find("#" + this.model.targetId);
                else if (this.optionDiv)
                    this.docbdy = this.optionDiv;
                else return false;
                this.itemsContainer = this.docbdy.children("ol,ul");
                this.itemsContainer.children("ol,ul").remove();
                this.items = this.itemsContainer.children('li');
                this.items.children("img").addClass("e-align");
                this.items.children("div").addClass("e-align");
                this.itemsContainer.children("span").addClass("e-ghead");
                //This will append the list with the popup wrapper
                this.ultag.append(this.itemsContainer.children());
                this.itemsContainer.empty();
                this.docbdy.css({ 'display': 'none' }); this.itemsContainer.css({ 'display': 'none' });
            }
            else if (this.model.dataSource != null && typeof list[0] != "object") {
                if (list.length > 0) {
                    for (i = 0; i < list.length; i++) {
                        litag = ej.buildTag("li", list[i]);
                        this.dummyUl.push(litag[0]);
                    }
                    this.ultag.append(this.dummyUl);
                }
            }
            else {
                this.mapFld = { _id: null, _imageUrl: null, _imageAttributes: null, _spriteCSS: null, _text: null, _value: null, _htmlAttributes: null, _selected: null };
                this.mapFld._id = (mapper && mapper.id) ? mapper["id"] : "id";
                this.mapFld._imageUrl = (mapper && mapper.imageUrl) ? mapper["imageUrl"] : "imageUrl";
                this.mapFld._imageAttributes = (mapper && mapper.imageAttributes) ? mapper["imageAttributes"] : "imageAttributes";
                this.mapFld._spriteCSS = (mapper && mapper.spriteCSS) ? mapper["spriteCSS"] : "spriteCSS";
                this.mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
                this.mapFld._value = (mapper && mapper.value) ? mapper["value"] : "value";
                this.mapFld._htmlAttributes = (mapper && mapper.htmlAttributes) ? mapper["htmlAttributes"] : "htmlAttributes";
                this.mapFld._selected = (mapper && mapper.selected) ? mapper["selected"] : "selected";

                if (list.length > 0) {
                    if (this.model.template != null) {
                        for (i = 0; i < list.length; i++) {
                            litag = ej.buildTag("li", list[i]);
                            if (this.model.template) litag.append(this._getTemplatedString(list[i]));
                            this.dummyUl.push(litag[0]);
                        }
                        this.ultag.append(this.dummyUl);
                    } else {
                        for (i = 0; i < list.length; i++) {
                            var _did = this._getField(list[i], this.mapFld._id);
                            var _dimageUrl = this._getField(list[i], this.mapFld._imageUrl);
                            var _dimageAttributes = this._getField(list[i], this.mapFld._imageAttributes);
                            var _dspriteCss = this._getField(list[i], this.mapFld._spriteCSS);
                            var _dtext = this._getField(list[i], this.mapFld._text);
                            var _dvalue = this._getField(list[i], this.mapFld._value);
                            var _dhtmlAttributes = this._getField(list[i], this.mapFld._htmlAttributes);
                            var _dselected = this._getField(list[i], this.mapFld._selected);
                            if ((_dvalue) && (_dvalue != ""))
                                litag = ej.buildTag('li', "", {}, { value: _dvalue });
                            else
                                litag = ej.buildTag('li');
                            if (_did)
                                litag.attr('id', _did);
                            if ((_dimageUrl) && (_dimageUrl != "")) {
                                //Creating the image tag
                                imgtag = ej.buildTag('img.e-align', '', {}, { 'src': _dimageUrl, 'alt': _dtext });
                                if ((_dimageAttributes) && (_dimageAttributes != ""))
                                    imgtag.attr(_dimageAttributes);
                                litag.append(imgtag);
                            }
                            if ((_dspriteCss) && (_dspriteCss != "")) {
                                //Creating the Sprite image tag
                                divtag = ej.buildTag('div.e-align ' + _dspriteCss + ' sprite-image');
                                litag.append(divtag);
                            }
                            if ((_dtext) && (_dtext != "")) {
                                //Creating text Content inside the  tag
                                litag.append(_dtext);
                            }
                            if ((_dhtmlAttributes) && (_dhtmlAttributes != ""))
                                litag.attr(_dhtmlAttributes);
                            if (_dselected) {
                                litag.addClass("chkselect");
                            }
                            //
                            this.dummyUl.push(litag[0]);
                        }
                    }
                    //ko binding
                    this.ultag.append(this.dummyUl);
                }

            }
            this.ultag.children('li').attr({ "role": "option" });
            this._setListWidth();
            this._setListHeight();
            this._setListPosition();
            this.popupList.css("height", ""); this.popupScroller.css({ "height": "", "width": "" });
            this.popupList.ejScroller({ height: this.popupList.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.popupList.ejScroller("instance");
            this.popupList.css({ 'display': 'none', 'visibility': 'visible' });
            this._checkboxHideShow(this.model.showCheckbox);
            this._setCheckAll(this.model.checkAll);
            this._setUncheckAll(this.model.uncheckAll);
            this.model.showPopupOnLoad && this._showResult();
        },
        //get object
        _getField: function (obj, fieldName) {
            return ej.pvt.getObject(fieldName, obj);
        },
        //template
        _getTemplatedString: function (list) {

            var str = this.model.template, start = str.indexOf("${"), end = str.indexOf("}");
            while (start != -1 && end != -1) {
                var content = str.substring(start, end + 1);
                var field = content.replace("${", "").replace("}", "");
                str = str.replace(content, this._getField(list, field));
                start = str.indexOf("${"), end = str.indexOf("}");
            }
            return str;
        },
        //water mark
        _setWatermark: function () {
            if ((this.model.watermark != null) && (this.element.val() == "")) {
                this.element.addClass("e-watermark");
                this.element.val(this.model.watermark);
            }
        },
        //checkbox properties
        _checkboxHideShow: function (value) {
            if (value) {
                this._createCheckbox();
            } else {
                this._removeCheckbox();
            }
            this.model.showCheckbox = value;
        },
        _createCheckbox: function () {
            var i, _extchk, chklist, me = this;
            this.listitems = this.popupList.find("ol,ul").children("li");
            chklist = this.listitems.find('input[type=checkbox]');
            if (chklist.length == 0) {
                for (i = 0; i < this.listitems.length; i++) {
                    $checkbox = ej.buildTag("input.listcheckbox e-align#popuplist" + i + "_" + this.element[0].id, "", {}, { type: "checkbox", name: "list" + i });
                    $(this.listitems[i]).prepend($checkbox);
                }
                $(".listcheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: $.proxy(this._OnClickCheckList, this) });
                this.element.val("");
                this._setWatermark();
                for (i = 0; i < this.listitems.length; i++) {
                    if ($(this.listitems[i]).hasClass('chkselect')) {
                        $(this.listitems[i]).find(".listcheckbox").ejCheckBox({ "checked": true });
                        this._activeItem = i;
                        this.checkedStatus = true;
                        if ((this.model.checkChange) && (!this._initValue))
                            this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                        this._enterTextBoxValue();
                        $(this.listitems[i]).removeClass('chkselect');
                    }
                }
            }
        },

        _removeCheckbox: function () {
            var i, checkbox;
            this.listitems = this.popupList.find("ol,ul").children("li");
            checkbox = this.listitems.find('.listcheckbox');
            if (checkbox.length > 0) {
                this.listitems.find('.listcheckbox').ejCheckBox('destroy');
                this.listitems.find('input[type=checkbox]').remove();
            }
        },
        _setCheckAll: function (value) {
            //Check all the nodes on CheckAll = true
            if ((this.model.showCheckbox) && (value))
                this.checkAll();
        },
        _setUncheckAll: function (value) {
            //UnCheck all the nodes on UnCheckAll = true
            if ((this.model.showCheckbox) && (value))
                this.unCheckAll();
        },
        /**
      * This method is used to set all the  items to check.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
      * &lt;script&gt;
      * // Create DropDownList
      * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
      * DropDownListObj.checkAll(); // checkAll values the DropDownList
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
      * &lt;script&gt;
      * // Create DropDownList
      * $('#drpdwn').ejDropDownList(); 	
      * &lt;/script&gt;
      *@memberof ejDropDownList
      * @instance
      */
        checkAll: function () {
            if (!this.model.enabled) return false;
            var _nodes = this.model.multiSelectedItemsIndex;
            for (i = 0; i < this.listitems.length; i++) {
                if ($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked') == false) {
                    $(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'check', true);
                    if ((this.model.watermark) && (this.element.hasClass("e-watermark"))) this.element.val(this.element[0].value.replace(this.model.watermark, ""));
                    var currentValue = $(this.listitems[i]).text();
                    this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
                    if ((this._delimiterIndex == -1) && (this.element[0].value != "")) {
                        this.element.val(this.element[0].value + ",");
                        this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
                    }
                    this.element.val(this.element[0].value.substr(0, this._delimiterIndex + 1) + currentValue + ",");
                    //
                    var _nodes = this.model.multiSelectedItemsIndex;
                    if ($.inArray(i, _nodes) == -1) {
                        this.model.multiSelectedItemsIndex.push(i);
                    }
                    //
                    this.checkedStatus = true;
                    if ($(this.listitems[i]).attr("value") != null)
                        this._selectedValue = $(this.listitems[i]).attr("value");
                    else
                        this._selectedValue = "";
                    if ((this.model.checkChange) && (!this._initValue))
                        this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                    args = { text: this.element[0].value, selectedText: currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
                    if ((this.model.change) && (!this._initValue))
                        this._trigger("change", args);
                    this._cascadeAction();
                }
            }
            this.value(this.element.val());
            this.model.text = currentValue;
            this.model.itemValue = this._selectedValue;
            /*Watermark*/
            if (this.model.watermark != null) {
                if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermark)) {
                    this.element.val(this.model.watermark);
                    this.element.addClass("e-watermark");
                } else {
                    this.element.removeClass("e-watermark");
                }
            }
        },
        _selectCheckedItem: function (chkitems) {
            if (chkitems.length > 0) {
                for (i = 0; i < chkitems.length; i++) {
                    if ($(this.listitems[chkitems[i]].firstChild).find('.listcheckbox').ejCheckBox('isChecked') == false) {
                        $(this.listitems[chkitems[i]].firstChild).find('.listcheckbox').ejCheckBox('option', 'check', true);
                        this.checkedStatus = true;
                        var currentValue = $(this.listitems[chkitems[i]]).text();
                        if (this.model.multiSelectMode)
                            this._ulBox.append(this._createBox(currentValue));
                        this._addText(currentValue);
                        if ($(this.listitems[chkitems[i]]).attr("value") != null)
                            this._selectedValue = $(this.listitems[chkitems[i]]).attr("value");
                        else
                            this._selectedValue = "";
                        if ((this.model.checkChange) && (!this._initValue))
                            this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                        args = { text: this.element[0].value, selectedText: currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
                        if ((this.model.change) && (!this._initValue))
                            this._trigger("change", args);
                        this._cascadeAction();
                    }
                }
                /*Watermark*/
                if (this.model.watermark != null) {
                    if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermark)) {
                        this.element.val(this.model.watermark);
                        this.element.addClass("e-watermark");
                    } else {
                        this.element.removeClass("e-watermark");
                    }
                }
            }
        },
        /**
      * This method is used to set all the  items to uncheck.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
      * &lt;script&gt;
      * // Create DropDownList
      * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
      * DropDownListObj.unCheckAll(); // checkAll values the DropDownList
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
      * &lt;script&gt;
      * // Create DropDownList
      * $('#drpdwn').ejDropDownList(); 	
      * &lt;/script&gt;
      *@memberof ejDropDownList
      * @instance
      */
        unCheckAll: function () {
            if (!this.model.enabled) return false;
            for (i = 0; i < this.listitems.length; i++) {
                if ($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                    $(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'check', false);
                    var currentValue = $(this.listitems[i]).text();
                    this._txtpos = (this.element[0].value).lastIndexOf(currentValue);
                    this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
                    if ((this._txtpos == 0) && (this._delimiterIndex <= -1)) {
                        this.element.val(this.element[0].value.replace(currentValue, ""));
                    } else if ((this._txtpos == 0) && (this._delimiterIndex > -1)) {
                        this.element.val(this.element[0].value.replace(currentValue + ",", ""));
                    } else if ((this._txtpos > -1) && (this._delimiterIndex > this._txtpos)) {
                        this.element.val(this.element[0].value.replace(currentValue + ",", ""));
                    } else if ((this._txtpos > -1) && (this._delimiterIndex < this._txtpos)) {
                        this.element.val(this.element[0].value.replace("," + currentValue, ""));
                    } else if ((this._txtpos > -1) && (this._delimiterIndex < this._txtpos)) {
                        this.element.val(this.element[0].value.replace(currentValue, ""));
                    }
                }
                this.checkedStatus = false;
                var _nodes = this.model.multiSelectedItemsIndex;
                if ($.inArray(this.selectedIndexValue, _nodes) > -1) {
                    this.model.multiSelectedItemsIndex.splice($.inArray(i, _nodes), 1);
                }
                if ($(this.listitems[i]).attr("value") != null)
                    this._selectedValue = $(this.listitems[i]).attr("value");
                else
                    this._selectedValue = "";
                if ((this.model.checkChange) && (!this._initValue))
                    this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                args = { text: this.element[0].value, selectedText: currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
                if ((this.model.change) && (!this._initValue))
                    this._trigger("change", args);
                this._cascadeAction();
            }
            this.value(this.element.val());
            this.model.text = currentValue;
            this.model.itemValue = this._selectedValue;
            /*Watermark*/
            if (this.model.watermark != null) {
                if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermark)) {
                    this.element.val(this.model.watermark);
                    this.element.addClass("e-watermark");
                } else {
                    this.element.removeClass("e-watermark");
                }
            }
        },

        //List Properties
        _refreshScroller: function () {
            this.popupList.css("height", "auto");
            this.popupList.find(".e-content, .e-vhandle").removeAttr("style");
            this.popupList.find(".e-vhandle div").removeAttr("style");

            this.popupList.css({ "display": "block" });  // For get the height of the popup
            this.scrollerObj.model.height = this.popupList.height();
            this.scrollerObj.refresh();
            this.scrollerObj.option("scrollTop", 0);
            this.popupList.css({ "height": "auto" });
        },
        _setListWidth: function () {
            var width = this.model.popupPanelWidth;
            if (width && width != "auto") this.popupList.css({ "width": width });
            else this.popupList.css({ "width": this.wrapper.width() });
        },

        _setListHeight: function () {
            if (this.model.listSize > 0) this.popupList.css({ "max-height": this.model.listSize * 30 });
            else this.popupList.css({ "max-height": this.model.popupPanelHeight });
        },
        _refreshPopup: function () {
            if (this.model.popupPanelWidth == "auto") this.popupList.css({ "width": this.wrapper.width() });
            this._setListPosition();
            this._refreshScroller();
        },
        _setListPosition: function () {
            var elementObj = this.wrapper;
            var pos = elementObj.offset(),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial();
            if (this.model.rtl) left -= this.popupList.outerWidth() - elementObj.outerWidth();
            this.popupList.css({
                "left": left + "px",
                "top": pos.top + totalHeight - border + 3 + "px",
                "z-index": maxZ
            });
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static')
                    return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        //Popup status
        _showResult: function () {
            var proxy = this;
            if (this.model.beforePopupShown)
                this._trigger("beforePopupShown", { popupStatus: this.showPopupBox });
            this._refreshPopup();
            this.popupList.css("display", "none");
            this.popupList.slideDown(200, "easeOutQuad", function () {
                proxy.popupList.focus();
                $(document).bind("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
            });
            this.showPopupBox = true;
            this.element.attr({ "aria-expanded": true }); var _popupListItems = this.ultag.children("li");
            this._listSize = _popupListItems.size();

            _popupListItems.bind("mouseenter", $.proxy(this._OnMouseEnter, this));
            _popupListItems.bind("mouseleave", $.proxy(this._OnMouseLeave, this));
            _popupListItems.bind("click", $.proxy(this._OnMouseClick, this));
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
            if (this.model.popupShown)
                this._trigger("popupShown", { popupStatus: this.showPopupBox });
        },

        _OnWindowResize: function (e) {
            this._refreshPopup();
            this.popupList.css("display", "block");
        },
        _hideResult: function () {
            if (this.showPopupBox) {
                this.showPopupBox = false;
                var proxy = this;
                this.popupList.slideUp(100, "easeOutQuad", function () {
                    proxy.wrapper.focus();
                    $(document).unbind("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
                });
                this.element.attr({ "aria-expanded": false }); var _popupListItems = this.ultag.children("li");
                $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
                _popupListItems.unbind("mouseenter", $.proxy(this._OnMouseEnter, this));
                _popupListItems.unbind("mouseleave", $.proxy(this._OnMouseLeave, this));
                _popupListItems.unbind("click", $.proxy(this._OnMouseClick, this));
                if (this.element != null && this.model.popupHide)
                    this._trigger("popupHide", { popupStatus: this.showPopupBox });
            }
        },
        // selecting text box value
        _enterTextBoxValue: function () {
            var args;
            this._chooseSelectionType();
            args = { text: this.currentValue, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if ((this.model.select) && (!this._initValue)) this._trigger("select", args);
            if (this.element[0].value != this.currentValue) {
                if (this.model.showCheckbox) {
                    if (this.model.multiSelectMode) {
                        if (this.activeItem.find('.listcheckbox').ejCheckBox('isChecked') == false) {
                            this.activeItem.find('.listcheckbox').ejCheckBox('option', 'check', true)
                            this.checkedStatus = true;
                        }
                        this._ulBox.append(this._createBox(this.currentValue));
                        if (this.showPopupBox)
                            this._setListPosition();
                    }
                    this._addText(this.currentValue);
                    var _nodes = this.model.multiSelectedItemsIndex;
                    if ($.inArray(this.selectedIndexValue, _nodes) == -1) {
                        this.model.multiSelectedItemsIndex.push(this.selectedIndexValue);
                    }
                } else {
                    this.ultag.children("li").removeClass('e-hover').removeClass('e-active');
                    this.activeItem.addClass('e-active');
                    this.element.val(this.currentValue);
                    this.model.selectedItem = this.selectedIndexValue;
                }

                args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };

                if ((this.model.change) && (!this._initValue))
                    this._trigger("change", args);
                this._cascadeAction();
                this.value(this.element.val());
                this.model.itemValue = this._selectedValue;
                this.model.text = this.currentValue;
                /*Watermark*/
                if (this.model.watermark != null) {
                    if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermark)) {
                        this.element.val(this.model.watermark);
                        this.element.addClass("e-watermark");
                    } else {
                        this.element.removeClass("e-watermark");
                    }
                }
            }
        },
        _chooseSelectionType: function () {
            this.activeItem = this._getActiveItem();
            this.selectedIndexValue = this._activeItem;
            if (this.model.dataSource != null && typeof this.model.dataSource[0] == "object") {
                this.currentValue = this._getField(this.popupListItems[this._activeItem], this.mapFld._text);
                this.hidelement = this._getField(this.popupListItems[this._activeItem], this.mapFld._value);
                this._selectedValue = this.hidelement;
            } else {
                this.currentValue = $.trim(this.activeItem.text());
                if ($(this.activeItem).attr("value") != null)
                    this._selectedValue = $(this.activeItem).attr("value");
                else
                    this._selectedValue = "";
            }
            this.selectedTextValue = this.currentValue;
        },
        _removeTextBoxValue: function () {
            this._chooseSelectionType();
            if (this.model.showCheckbox) {
                this._removeText(this.currentValue);
                var _nodes = this.model.multiSelectedItemsIndex;
                if ($.inArray(this.selectedIndexValue, _nodes) > -1) {
                    this.model.multiSelectedItemsIndex.splice($.inArray(this.selectedIndexValue, _nodes), 1);
                }
                if (this.model.multiSelectMode) {
                    this._deleteBoxCheck(this.currentValue);
                    if (this.showPopupBox)
                        this._setListPosition();
                }
            } else {
                this.ultag.children("li").removeClass('e-hover').removeClass('e-active');
                this._txtpos = (this.element[0].value).lastIndexOf(this.currentValue);
                if (this._txtpos > -1)
                    this.element.val(this.element[0].value.replace(this.currentValue, ""));
            }
            args = { text: this.currentValue, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if ((this.model.select) && (!this._initValue))
                this._trigger("select", args);
            args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if ((this.model.change) && (!this._initValue))
                this._trigger("change", args);
            this.value(this.element.val());
            this.model.itemValue = this._selectedValue;
            this.model.text = this.currentValue;
            /*Watermark*/
            if (this.model.watermark != null) {
                if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermark)) {
                    this.element.val(this.model.watermark);
                    this.element.addClass("e-watermark");
                } else {
                    this.element.removeClass("e-watermark");
                }
            }
        },
        /* box */

        _createBox: function (value) {
            if (!this._checkContains(value) && this.checkedStatus) {
                var span = ej.buildTag("span.e-icon e-close");
                var li = ej.buildTag("li.e-options", value).append(span);
                this._on(span, "click", function (e) {
                    if (!this.model.enabled) return false;
                    this._deleteBox($(e.target).parent());
                });
                return li;
            }
        },
        _deleteBoxCheck: function (val) {
            var items = this._ulBox.children('li');
            for (var i = 0; i < items.length; i++) {
                if ($(items[i]).text() == val) {
                    $(items[i]).remove();
                }
            }
        },
        _deleteLastBox: function () {
            var items = this._ulBox.children("li:not(.e-search-box)");
            var item = items.last();
            if (item.hasClass("e-active")) this._deleteBox(item);
            else {
                this._removeActive();
                item.addClass("e-active");
            }
        },
        _deleteBox: function (items) {
            for (var i = 0; i < items.length; i++) {
                var cobj = $(items[i]);
                deltext = cobj.text();
                var listItems = this.ultag.children('li');
                for (var j = 0; j < listItems.length; j++) {
                    if ($(listItems[j]).text() == deltext) {
                        this._activeItem = j;
                        this._uncheckboxModel(this._activeItem);
                    }
                }
                cobj.remove();
            }
        },
        _removeActive: function () {
            this._ulBox.children("li").removeClass("e-active");
        },
        _adjustWidth: function () {
            var tempSpan = ej.buildTag("span", this.element.val());
            this.container.append(tempSpan);
            this.element.width(tempSpan.width() + 30);
            tempSpan.remove();
        },
        _uncheckboxModel: function (val) {
            var listItems = this.ultag.children('li');
            if ($(listItems[val].firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                $(listItems[val].firstChild).find('.listcheckbox').ejCheckBox('option', 'check', false);
                this._removeTextBoxValue();
                if ((this.model.checkChange) && (!this._initValue))
                    this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
            }
        },
        _destroyBoxModel: function () {
            this.container.prepend(this.element);
            this.element.css('display', 'block');
            this.wrapper.height(this.model.height);
            this._ulBox.remove();
            this._off(this.container, "mousedown");
        },
        //
        _removeListHover: function () {
            this.ultag.children("li").removeClass("e-hover");
        },

        _addListHover: function () {
            var activeItem = this._getActiveItem();
            activeItem.addClass("e-hover");
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
            activeItem.focus();
        },
        _calcScrollTop: function () {
            var ulH = this.ultag.outerHeight(), li = this.ultag.find("li"), liH = 0, index, top, i;
            index = this.ultag.find("li.e-hover").index();
            for (i = 0; i < index; i++) { liH += li.eq(i).outerHeight(); }
            top = liH - ((this.popupList.outerHeight() - li.eq(index).outerHeight()) / 2);
            return top;
        },
        _getActiveItem: function () {
            return $(this.ultag.children("li")[this._activeItem]);
        },
        _setDimentions: function () {
            if (this.model.height)
                this.wrapper.height(this.model.height);
            if (this.model.width)
                this.wrapper.width(this.model.width);
        },


        _roundedCorner: function (val) {
            if (val) {
                this.container.addClass("e-corner-all");
            }
            else {
                this.container.removeClass("e-corner-all");
            }

        },

        _enabled: function (boolean) {
            if (boolean) this.enable();
            else this.disable();
        },

        _destroyDropdown: function () {
            this.element.insertAfter(this.wrapper);
            this.element.width(this.element.width() + this.dropdownbutton.outerWidth());
            this.wrapper.remove();
            this.container.remove();
            this.element.unbind("mousedown", $.proxy(this._OnDropdownClick, this));//need to write for removing the list 
            this.dropdownbutton.unbind("mousedown", $.proxy(this._OnDropdownClick, this));//need to write for removing the list 
        },


        _RightToLeft: function () {
            if (this.model.rtl) {
                this.wrapper.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
            }

        },
        _dropbtnRTL: function () {
            if (this.model.rtl) {
                this.popupList.addClass("e-rtl");
            }
            else {
                this.popupList.removeClass("e-rtl");
            }
        },
        _OnDropdownClick: function (e) {
            e.preventDefault();
            if (this.model.readOnly) return false;
            if (this.ultag.find('li').length > 0) {
                if (this.showPopupBox) {
                    this._hideResult();
                }
                else {
                    this._showResult();
                    this.wrapper.focus();
                }
            }
        },

        _showFullList: function () {
            if (this.model.dataSource != null) {
                if (!(ej.DataManager && this.model.dataSource instanceof ej.DataManager)) {
                    this.popupListItems = this.model.dataSource;
                }
            }
            this._renderPopupList();
        },

        // cascade options
        _cascadeAction: function () {
            if (this.model.selectedTo) {
                this._currentValue = this._getField(this.popupListItems[this._activeItem], this.mapFld._value);
                this.selectDropObj = $('#' + this.model.selectedTo).ejDropDownList('instance');
                if (ej.isNullOrUndefined(this._dSource))
                    this._dSource = this.selectDropObj.model.dataSource;
                if (ej.DataManager && this._dSource instanceof ej.DataManager) {
                    this._performOdataInit();
                } else {
                    this._performJsonDataInit();
                }
            }
        },
        _performOdataInit: function () {
            var proxy = this, queryPromise, tempQuery;
            if (ej.isNullOrUndefined(this._dQuery)) {
                this._dQuery = this.selectDropObj.model.query.clone();
            }
            tempQuery = this._dQuery.clone();
            proxy.selectDropObj._addLoadingClass();
            queryPromise = this._dSource.executeQuery(tempQuery.where(this.mapFld._value, "equal", this._currentValue));
            queryPromise.fail(function (e) {
                proxy._changedSource = null;
                proxy.selectDropObj.setModel({ dataSource: proxy._changedSource, enabled: false });
            }).done(function (e) {
                proxy._changedSource = e.result;
                proxy.selectDropObj.setModel({ dataSource: proxy._changedSource, enabled: true });
                proxy.selectDropObj._removeLoadingClass();
            });
        },
        _performJsonDataInit: function () {
            this._changedSource = ej.DataManager(this._dSource).executeLocal(ej.Query().where(this.mapFld._value, "==", this._currentValue));
            this.selectDropObj.setModel({ dataSource: this._changedSource, enabled: true });
        },
        //
        _OnMouseEnter: function (e) {
            var targetEle;
            this.ultag.children("li").removeClass("e-hover");
            if ($(e.target).is("li")) { $(e.target).addClass("e-hover"); }
            else if (e.target.tagName != "li") {
                targetEle = $(e.target).parents("li");
                $(targetEle).addClass("e-hover");
            }
            var activeItem = 0;
            this.ultag.children("li").each(function (index) {
                if ($(this).hasClass("e-hover")) {
                    activeItem = index;
                    return false;
                }
            });
            this._activeItem = activeItem;
        },
        _OnMouseLeave: function (e) {
            this.ultag.children("li").removeClass("e-hover");
        },
        _OnMouseClick: function (e) {
            if (!this.model.showCheckbox) {
                if (this.model.template) {
                    this._templateValue();
                } else {
                    this._enterTextBoxValue();
                }
                this._hideResult();
            } else {
                if (($(e.currentTarget).is("li")) && ($(e.target).is("li"))) {
                    if ($(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                        $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'check', false);
                        this.checkedStatus = false;
                        this._removeTextBoxValue();
                    } else {
                        $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'check', true);
                        this.checkedStatus = true;
                        this._enterTextBoxValue();
                    }
                }
                if ((this.model.checkChange) && (!this._initValue))
                    this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
            }
        },
        _OnClickCheckList: function (e) {
            if (e.isChecked) {
                this.checkedStatus = true;
                this._enterTextBoxValue();
            } else {
                this.checkedStatus = false;
                this._removeTextBoxValue();
            }
            if ((this.model.checkChange) && (!this._initValue))
                this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });

        },
        _OnListSelect: function (e) {
            e.preventDefault();
            activeItem = this._getActiveItem();
            if ($(activeItem).find('.listcheckbox').ejCheckBox('isChecked')) {
                $(activeItem).find('.listcheckbox').ejCheckBox('option', 'check', false);
                this.checkedStatus = false;
                this._removeTextBoxValue();
            } else {
                $(activeItem).find('.listcheckbox').ejCheckBox('option', 'check', true);
                this.checkedStatus = true;
                this._enterTextBoxValue();
            }

            if ((this.model.checkChange) && (!this._initValue))
                this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
        },
        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.popupList) && !$(e.target).parents(".e-ddl-popup").is(this.popupList) &&
                !$(e.target).is(this.element) && !$(e.target).parents(".e-ddl").is(this.wrapper)) {
                this._hideResult();
            }
            else if ($(e.target).is(this.popupList) || $(e.target).parents(".e-ddl-popup").is(this.popupList))
                e.preventDefault();
        },

        //keyboard events
        _OnKeyDown: function (e) {
            if ((!this.model.enabled) || (this.model.readOnly)) return false;
            this._itemId = null;
            var _popupListItems = this.ultag.children("li");
            this._listSize = this.ultag.children("li").size();

            switch (e.keyCode) {
                case 38:    //Up arrow
                    if (e.altKey) {
                        if (this.ultag.find('li').length > 0)
                            this._hideResult();
                    } else {
                        if (this.showPopupBox) {
                            this._removeListHover();
                            this.ultag.children("li").removeClass('e-active');
                            if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                                this._activeItem = this._listSize - 1;
                            else
                                this._activeItem -= 1;
                            this._addListHover();
                            if ((!this.model.showCheckbox) && (this.model.template == null))
                                this._enterTextBoxValue();
                        } else {
                            this.ultag.children("li").removeClass('e-active');
                            if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                                this._activeItem = this._listSize - 1;
                            else
                                this._activeItem -= 1;
                            if ((!this.model.showCheckbox) && (this.model.template == null))
                                this._enterTextBoxValue();
                        }
                        e.preventDefault();
                        return false;
                    }
                    break;

                case 40:    //Down arrow
                    if (e.altKey) {
                        if (this.ultag.find('li').length > 0)
                            this._showResult();
                    } else {
                        if (this.showPopupBox) {
                            this._removeListHover();
                            this.ultag.children("li").removeClass('e-active');
                            if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                                this._activeItem = 0;
                            else
                                this._activeItem += 1;
                            this._addListHover();
                            if ((!this.model.showCheckbox) && (this.model.template == null))
                                this._enterTextBoxValue();
                        } else {
                            this.ultag.children("li").removeClass('e-active');
                            if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                                this._activeItem = 0;
                            else
                                this._activeItem += 1;
                            if ((!this.model.showCheckbox) && (this.model.template == null))
                                this._enterTextBoxValue();
                        }
                        e.preventDefault();
                        return false;
                    }
                    break;
                case 37:// Left Key equals up
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= 1;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= 1;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;  // Left Key
                case 39:// Right Key equals down
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += 1;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += 1;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;  // Right Key
                case 8:    // Backspace key
                    if ((this.model.multiSelectMode == true) && (this.model.showCheckbox)) {
                        this._deleteLastBox();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 9:    // Tab key
                    this._hideResult();
                    break;
                case 33:
                    break;//up key
                case 34:
                    break;//down key
                case 35:
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = this._listSize - 1;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    else {
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = this._listSize - 1;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;  // End Key
                case 36:
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = 0;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    else {
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = 0;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;  // Home Key
                case 17:    // Ctrl key
                    break;
                case 18:    // Alt key
                    break;
                case 27:    // Esc key
                    this._hideResult();
                    e.stopPropagation();
                    break;
                case 32:    // space Key
                    e.preventDefault();
                    return false;
                    break;
            }
        },

        _OnKeyUp: function (e) {
            if ((!this.model.enabled) || (this.model.readOnly)) return false;
            e.preventDefault();
            var target = e.target;
            if ($.trim(this.element.val()) == "" && e.keyCode == 38 && e.keyCode == 40) {
                this._hideResult();
                return false;
            }

            switch (e.keyCode) {
                //Restricts the keyup event for other functional keys
                case 38: break;  // Up Key
                case 40: break;  // Down Key
                case 37: break;  // Left Key
                case 39: break;  // Right Key

                case 20: break;  // CapsLk
                case 16: break;  // Shift Key
                case 17: break;  // Ctrl Key
                case 18: break;  // Alt Key
                case 35: break;  // End Key
                case 36: break;  // Home Key
                case 144: break;  // Num Lock
                case 27: break;  // Esc Key
                case 9: break;  // Tab Key

                case 13:    // Enter Key
                    e.preventDefault();
                    if (this.model.showCheckbox) {
                        this._OnListSelect(e);
                    } else if (this.model.template) {
                        this._templateValue();
                        this._hideResult();
                    }
                    else {
                        this._enterTextBoxValue();
                        this._hideResult();
                    }

                    return false;
                    break;
                case 32:    // space Key
                    e.preventDefault();
                    if (this.model.showCheckbox) {
                        this._OnListSelect(e);
                    } else if (this.model.template) {
                        this._templateValue();
                        this._hideResult();
                    }
                    else {
                        this._enterTextBoxValue();
                        this._hideResult();
                    }
                    return false;
                    break;
                case 8:     // Backspace Key
                    // this._OnTextEnter("backspace");
                    this._hideResult();
                    break;
                case 46:    // Delete Key
                    if ((this.model.multiSelectMode == true) && (this.model.showCheckbox)) {
                        this._deleteBox(this._ulBox.children("li.e-active"));
                        break;
                    }

                default:
                    if ((!this.model.showCheckbox) && ((this.model.dataSource != null) && (this.model.dataSource.length > 0) && (this.model.incrementalSearch))) {
                        this._OnTextEnter(e.keyCode);
                    }
                    break;
            }
        },
        //
        //template type
        _templateValue: function () {
            var activeitem = $(this.ultag.children("li")[this._activeItem]);
            currentText = this._getField(this.popupListItems[this._activeItem], this.mapFld._text);
            currentValue = this._getField(this.popupListItems[this._activeItem], this.mapFld._value);
            args = { text: currentText, selectedText: currentText, itemId: this._activeItem, value: currentValue };
            if ((this.model.select) && (!this._initValue))
                this._trigger("select", args);
            if (this._activeItem != this._aselectedItem) {
                this.ultag.children("li").removeClass('e-hover').removeClass('e-active');
                activeitem.addClass('e-active');
                this._aselectedItem = this._activeItem;
                this.element.val(currentText);
                args = { text: this.element[0].value, selectedText: currentText, itemId: this._activeItem, value: currentValue };
                if ((this.model.change) && (!this._initValue))
                    this._trigger("change", args);
                this.value(this.element.val());
                this.model.itemValue = currentValue;
                this.model.text = currentText;
            }
            /*Watermark*/
            if (this.model.watermark != null) {
                if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermark)) {
                    this.element.val(this.model.watermark);
                    this.element.addClass("e-watermark");
                } else {
                    this.element.removeClass("e-watermark");
                }
            }
        },
        //
        //On Text type
        _OnTextEnter: function (from) {
            // this._hideResult();
            _proxy = this;
            this._incqueryString += String.fromCharCode(from);
            if (this._incqueryString.length > 0) {
                setTimeout(function () { _proxy._incqueryString = ""; }, 100);
            }
            var list = this.model.dataSource, i, strlen,
            caseSence = this.model.caseSensitive, mapper = this.model.fields,
            tempSuggestion = [],
            str, queryStr = this._incqueryString,
            querylength = this._incqueryString.length, searchflag = false;

            if (!caseSence) queryStr = queryStr.toLowerCase();

            for (i = 0; i < list.length; i++) {
                str = (typeof list[i] == "string") ? list[i] : (!(mapper && mapper.text) ? list[i].text : list[i][mapper.text]);
                str = caseSence ? str : str.toLowerCase();
                if (str.substr(0, querylength) == queryStr) {
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this._activeItem = i;
                        this._enterTextBoxValue();
                        this._addListHover();
                    } else {
                        this._activeItem = i;
                        this._enterTextBoxValue();
                    }
                    searchflag = true;
                } else if ((i == list.length - 1) && (searchflag == false)) {
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this._activeItem = i;
                        this._enterTextBoxValue();
                        this._addListHover();
                    } else {
                        this._activeItem = i;
                        this._enterTextBoxValue();
                    }
                    searchflag = true;
                }
                if (searchflag) break;
            }

        },

        //foucus in and out
        _targetFocus: function () {
            if (this.model.enabled) {
                if (this.element.hasClass("e-watermark")) {
                    this.element.removeClass("e-watermark");
                    this.element.val(null);
                }
                this.wrapper.addClass("e-focus");
                this._isFocused = true;
                /* Raise focusIn Event*/
                if (this.model.focusIn)
                    this._trigger("focusIn");
            }
        },

        _targetBlur: function () {
            if (this.model.enabled) {
                this._isFocused = false;
                this.wrapper.removeClass("e-focus");
                if (this.model.watermark != null) {
                    if ($.trim(this.element.val()) === '') {
                        this.element.val(this.model.watermark);
                        this.element.addClass("e-watermark");
                    }
                }
                /* Raise focusOut Event*/
                if (this.model.focusOut)
                    this._trigger("focusOut");
            }
        },

        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.wrapper, "focus", this._targetFocus);
            this._on(this.wrapper, "blur", this._targetBlur);
            // this._on(this.element, "input", this._valueChange);
            //this._on(this.element, "keydown", this._OnKeyDown);
            //this._on(this.element, "keyup", this._OnKeyUp);


            this._on(this.wrapper, "keydown", this._OnKeyDown);
            this._on(this.popupList, "keydown", this._OnKeyDown);
            this._on(this.popupList, "keyup", this._OnKeyUp);
            this._on(this.wrapper, "keyup", this._OnKeyUp);

        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
	* @requires jQuery
	* @requires ej.web.all
	* @classdesc Custom Design for Html Textbox Editors control.
	* @example 
	* &lt;input id="numeric" type="text" /&gt; <br> 
	* &lt;input id="currency" type="text" /&gt; <br> 
	* &lt;input id="percentage" type="text" /&gt; <br> 
	* &lt;script&gt;<br>
	* // Create Textbox Editors <br>
    * $('#numeric').ejNumericTextbox(); <br>	
	* $('#currency').ejCurrencyTextbox(); <br>
	* $('#percentage').ejPercentageTextbox(); <br>
	* &lt;/script&gt;
	*/

    ej.widget(
    {
        "ejNumericTextbox": ["ej.NumericTextbox", "e-numerictextbox"],
        "ejPercentageTextbox": ["ej.PercentageTextbox", "e-percentagetextbox"],
        "ejCurrencyTextbox": ["ej.CurrencyTextbox", "e-currencytextbox"]
    },
    {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        type: "editor",

        // default model
        defaults: {
            /**		
                * Specifies the width of the editor.
                * @default 143pixel
                * @type {String}
                * @example 
                * //To set width API value during initialization  
                * 	$("#numeric").ejNumericTextbox({ width: "143px" });	
                * 	$("#currency").ejCurrencyTextbox({ width: "143px" });
                * 	$("#percentage").ejPercentageTextbox({ width: "143px" });			
                * @example 
                * //Get or set the width API, after initialization:<br>
                *	//Gets the width value
                *	$("#numeric").ejNumericTextbox("option", "width");	
                *	$("#currency").ejCurrencyTextbox("option", "width");
                *	$("#percentage").ejPercentageTextbox("option", "width");			
                *	//Sets the width value
                *	$("#numeric").ejNumericTextbox("option", "width", "143px" );
                *	$("#currency").ejCurrencyTextbox("option", "width", "143px" );
                *	$("#percentage").ejPercentageTextbox("option", "width", "143px" );
                * @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
                * @instance
                */
            width: "",

            /**		
			* Specifies the height of the editor.
			* @default 30pixel
			* @type {String}
			* @example 
			* //To set height API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ height: "30px" });	
			* 	$("#currency").ejCurrencyTextbox({ height: "30px" });
			* 	$("#percentage").ejPercentageTextbox({ height: "30px" });			 
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value
			*	$("#numeric").ejNumericTextbox("option", "height");	
			*	$("#currency").ejCurrencyTextbox("option", "height");
			*	$("#percentage").ejPercentageTextbox("option", "height");			
			*	//Sets the height value
			*	$("#numeric").ejNumericTextbox("option", "height", "30px" );
			*	$("#currency").ejCurrencyTextbox("option", "height", "30px" );
			*	$("#percentage").ejPercentageTextbox("option", "height", "30px" );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            height: "",

            /**		
			* Specifies the value of the editor.
			* @default 0
			* @type {number}
			* @example 
			* //To set value API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ value: 10 });	
			* 	$("#currency").ejCurrencyTextbox({ value: 10 });
			* 	$("#percentage").ejPercentageTextbox({ value: 10 });			 
			* @example 
			* //Get or set the value API, after initialization:<br>
			*	//Gets the value value
			*	$("#numeric").ejNumericTextbox("option", "value");	
			*	$("#currency").ejCurrencyTextbox("option", "value");
			*	$("#percentage").ejPercentageTextbox("option", "value");			
			*	//Sets the value value
			*	$("#numeric").ejNumericTextbox("option", "value", 10 );
			*	$("#currency").ejCurrencyTextbox("option", "value", 10 );
			*	$("#percentage").ejPercentageTextbox("option", "value", 10 );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            value: null,

            /**		
			* Specifies the name of the editor.
			* @default Sets id as name if it is null.
			* @type {string}
			* @example 
			* //To set name API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ name: "numeric" });	
			* 	$("#currency").ejCurrencyTextbox({ name: "currency" });
			* 	$("#percentage").ejPercentageTextbox({ name: "percentage" });			 
			* @example 
			* //Get or set the name API, after initialization:<br>
			*	//Gets the name value
			*	$("#numeric").ejNumericTextbox("option", "name");	
			*	$("#currency").ejCurrencyTextbox("option", "name");
			*	$("#percentage").ejPercentageTextbox("option", "name");			
			*	//Sets the name value
			*	$("#numeric").ejNumericTextbox("option", "name", "numeric" );
			*	$("#currency").ejCurrencyTextbox("option", "name", "currency" );
			*	$("#percentage").ejPercentageTextbox("option", "name", "percentage" );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            name: null,

            /**		
			* Specifies the minValue of the editor.
			* @default 0 for CurrencyTextbox
			* @default null for NumericTextbox
			* @default null for PercentageTextbox
			* @type {number}
			* @example 
			* //To set minValue API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ minValue: 0 });	
			* 	$("#currency").ejCurrencyTextbox({ minValue: 0 });
			* 	$("#percentage").ejPercentageTextbox({ minValue: 0 });			 
			* @example 
			* //Get or set the minValue API, after initialization:<br>
			*	//Gets the minValue value
			*	$("#numeric").ejNumericTextbox("option", "minValue");	
			*	$("#currency").ejCurrencyTextbox("option", "minValue");
			*	$("#percentage").ejPercentageTextbox("option", "minValue");			
			*	//Sets the minValue value
			*	$("#numeric").ejNumericTextbox("option", "minValue", 0 );
			*	$("#currency").ejCurrencyTextbox("option", "minValue", 0 );
			*	$("#percentage").ejPercentageTextbox("option", "minValue", 0 );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            minValue: -(Number.MAX_VALUE),

            /**		
			* Specifies the maxValue of the editor.
			* @default null for CurrencyTextbox
			* @default null for NumericTextbox
			* @default 100 for PercentageTextbox
			* @type {number}
			* @example 
			* //To set maxValue API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ maxValue: 100 });	
			* 	$("#currency").ejCurrencyTextbox({ maxValue: 100 });
			* 	$("#percentage").ejPercentageTextbox({ maxValue: 100 });			 
			* @example 
			* //Get or set the maxValue API, after initialization:<br>
			*	//Gets the maxValue value
			*	$("#numeric").ejNumericTextbox("option", "maxValue");	
			*	$("#currency").ejCurrencyTextbox("option", "maxValue");
			*	$("#percentage").ejPercentageTextbox("option", "maxValue");			
			*	//Sets the maxValue value
			*	$("#numeric").ejNumericTextbox("option", "maxValue", 100 );
			*	$("#currency").ejCurrencyTextbox("option", "maxValue", 100 );
			*	$("#percentage").ejPercentageTextbox("option", "maxValue", 100 );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            maxValue: Number.MAX_VALUE,

            /**		
			* Specifies the incrementStep of the editor.
			* @default 1		
			* @type {number}
			* @example 
			* //To set incrementStep API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ incrementStep: 2 });	
			* 	$("#currency").ejCurrencyTextbox({ incrementStep: 2 });
			* 	$("#percentage").ejPercentageTextbox({ incrementStep: 2 });			 
			* @example 
			* //Get or set the incrementStep API, after initialization:<br>
			*	//Gets the incrementStep value
			*	$("#numeric").ejNumericTextbox("option", "incrementStep");	
			*	$("#currency").ejCurrencyTextbox("option", "incrementStep");
			*	$("#percentage").ejPercentageTextbox("option", "incrementStep");			
			*	//Sets the incrementStep value
			*	$("#numeric").ejNumericTextbox("option", "incrementStep", 100 );
			*	$("#currency").ejCurrencyTextbox("option", "incrementStep", 100 );
			*	$("#percentage").ejPercentageTextbox("option", "incrementStep", 100 );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            incrementStep: 1,

            /**		
			* Specifies the decimals of the editor.
			* @default 0		
			* @type {number}
			* @example 
			* //To set decimals API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ decimals: 2 });	
			* 	$("#currency").ejCurrencyTextbox({ decimals: 2 });
			* 	$("#percentage").ejPercentageTextbox({ decimals: 2 });			 
			* @example 
			* //Get or set the decimals API, after initialization:<br>
			*	//Gets the decimals value
			*	$("#numeric").ejNumericTextbox("option", "decimals");	
			*	$("#currency").ejCurrencyTextbox("option", "decimals");
			*	$("#percentage").ejPercentageTextbox("option", "decimals");			
			*	//Sets the decimals value
			*	$("#numeric").ejNumericTextbox("option", "decimals", 100 );
			*	$("#currency").ejCurrencyTextbox("option", "decimals", 100 );
			*	$("#percentage").ejPercentageTextbox("option", "decimals", 100 );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            decimals: 0,

            /**		
			* Specify the CSS class to editor to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* //To set cssClass API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ cssClass: "gradient-lime" });	
			* 	$("#currency").ejCurrencyTextbox({ cssClass: "gradient-lime" });
			* 	$("#percentage").ejPercentageTextbox({ cssClass: "gradient-lime" });			 
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the cssClass value
			*	$("#numeric").ejNumericTextbox("option", "cssClass");	
			*	$("#currency").ejCurrencyTextbox("option", "cssClass");
			*	$("#percentage").ejPercentageTextbox("option", "cssClass");			
			*	//Sets the cssClass value
			*	$("#numeric").ejNumericTextbox("option", "cssClass", "gradient-lime" );
			*	$("#currency").ejCurrencyTextbox("option", "cssClass", "gradient-lime" );
			*	$("#percentage").ejPercentageTextbox("option", "cssClass", "gradient-lime" );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            cssClass: "",

            /**		
			* Specify the persist to editor to save current model value to browser cookies for state maintains
			* @default false
			* @type {boolean}
			* @example 
			* //To set persist API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ persist: true });	
			* 	$("#currency").ejCurrencyTextbox({ persist: true });
			* 	$("#percentage").ejPercentageTextbox({ persist: true });			 
			* @example 
			* //Get or set the persist API, after initialization:<br>
			*	//Gets the persist value
			*	$("#numeric").ejNumericTextbox("option", "persist");	
			*	$("#currency").ejCurrencyTextbox("option", "persist");
			*	$("#percentage").ejPercentageTextbox("option", "persist");			
			*	//Sets the persist value
			*	$("#numeric").ejNumericTextbox("option", "persist", true );
			*	$("#currency").ejCurrencyTextbox("option", "persist", true );
			*	$("#percentage").ejPercentageTextbox("option", "persist", true );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            persist: false,

            /**		
			* Specify the showSpinButton to editor 
			* @default true
			* @type {boolean}
			* @example 
			* //To set showSpinButton API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ showSpinButton: true });	
			* 	$("#currency").ejCurrencyTextbox({ showSpinButton: true });
			* 	$("#percentage").ejPercentageTextbox({ showSpinButton: true });			 
			* @example 
			* //Get or set the showSpinButton API, after initialization:<br>
			*	//Gets the showSpinButton value
			*	$("#numeric").ejNumericTextbox("option", "showSpinButton");	
			*	$("#currency").ejCurrencyTextbox("option", "showSpinButton");
			*	$("#percentage").ejPercentageTextbox("option", "showSpinButton");			
			*	//Sets the showSpinButton value
			*	$("#numeric").ejNumericTextbox("option", "showSpinButton", true );
			*	$("#currency").ejCurrencyTextbox("option", "showSpinButton", true );
			*	$("#percentage").ejPercentageTextbox("option", "showSpinButton", true );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            showSpinButton: true,

            /**		
			* Specify the Localization to editor 
			* @default en-US
			* @type {string}
			* @example 
			* //To set localize API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ localize: "en-US" });	
			* 	$("#currency").ejCurrencyTextbox({ localize: "en-US" });
			* 	$("#percentage").ejPercentageTextbox({ localize: "en-US" });			 
			* @example 
			* //Get or set the localize API, after initialization:<br>
			*	//Gets the localize value
			*	$("#numeric").ejNumericTextbox("option", "localize");	
			*	$("#currency").ejCurrencyTextbox("option", "localize");
			*	$("#percentage").ejPercentageTextbox("option", "localize");			
			*	//Sets the localize value
			*	$("#numeric").ejNumericTextbox("option", "localize", "en-US" );
			*	$("#currency").ejCurrencyTextbox("option", "localize", "en-US" );
			*	$("#percentage").ejPercentageTextbox("option", "localize", "en-US" );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            localize: "en-US",

            /**		
			* Specify the strictMode to editor 
			* @default false
			* @type {booelan}
			* @example 
			* //To set strictMode API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ strictMode: true });	
			* 	$("#currency").ejCurrencyTextbox({ strictMode: true });
			* 	$("#percentage").ejPercentageTextbox({ strictMode: true });			
			* @example 
			* //Get or set the strictMode API, after initialization:<br>
			*	//Gets the strictMode value
			*	$("#numeric").ejNumericTextbox("option", "strictMode");	
			*	$("#currency").ejCurrencyTextbox("option", "strictMode");
			*	$("#percentage").ejPercentageTextbox("option", "strictMode");			
			*	//Sets the strictMode value
			*	$("#numeric").ejNumericTextbox("option", "strictMode", true );
			*	$("#currency").ejCurrencyTextbox("option", "strictMode", true );
			*	$("#percentage").ejPercentageTextbox("option", "strictMode", true );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            strictMode: false,

            /**		
			* Specify the rounded Corner to editor 
			* @default false
			* @type {boolean}
			* @example 
			* //To set roundedCorner API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ roundedCorner: true });	
			* 	$("#currency").ejCurrencyTextbox({ roundedCorner: true });
			* 	$("#percentage").ejPercentageTextbox({ roundedCorner: true });			 
			* @example 
			* //Get or set the roundedCorner API, after initialization:<br>
			*	//Gets the roundedCorner value
			*	$("#numeric").ejNumericTextbox("option", "roundedCorner");	
			*	$("#currency").ejCurrencyTextbox("option", "roundedCorner");
			*	$("#percentage").ejPercentageTextbox("option", "roundedCorner");			
			*	//Sets the roundedCorner value
			*	$("#numeric").ejNumericTextbox("option", "roundedCorner", true );
			*	$("#currency").ejCurrencyTextbox("option", "roundedCorner", true );
			*	$("#percentage").ejPercentageTextbox("option", "roundedCorner", true );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            roundedCorner: false,

            /**		
			* Specify the readOnly to editor 
			* @default false
			* @type {boolean}
			* @example 
			* //To set readOnly API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ readOnly: true });	
			* 	$("#currency").ejCurrencyTextbox({ readOnly: true });
			* 	$("#percentage").ejPercentageTextbox({ readOnly: true });			 
			* @example 
			* //Get or set the readOnly API, after initialization:<br>
			*	//Gets the readOnly value
			*	$("#numeric").ejNumericTextbox("option", "readOnly");	
			*	$("#currency").ejCurrencyTextbox("option", "readOnly");
			*	$("#percentage").ejPercentageTextbox("option", "readOnly");			
			*	//Sets the readOnly value
			*	$("#numeric").ejNumericTextbox("option", "readOnly", true );
			*	$("#currency").ejCurrencyTextbox("option", "readOnly", true );
			*	$("#percentage").ejPercentageTextbox("option", "readOnly", true );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            readOnly: false,

            /**		
			* Specify the editor control state.
			* @default true
			* @type {boolean}
			* @example 
			* //To set enabled API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ enabled: true });	
			* 	$("#currency").ejCurrencyTextbox({ enabled: true });
			* 	$("#percentage").ejPercentageTextbox({ enabled: true });			 
			* @example 
			* //Get or set the enabled API, after initialization:<br>
			*	//Gets the enabled value
			*	$("#numeric").ejNumericTextbox("option", "enabled");	
			*	$("#currency").ejCurrencyTextbox("option", "enabled");
			*	$("#percentage").ejPercentageTextbox("option", "enabled");			
			*	//Sets the enabled value
			*	$("#numeric").ejNumericTextbox("option", "enabled", true );
			*	$("#currency").ejCurrencyTextbox("option", "enabled", true );
			*	$("#percentage").ejPercentageTextbox("option", "enabled", true );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            enabled: true,

            /**		
			* Specify the Right to Left Direction to editor.
			* @default false
			* @type {boolean}
			* @example 
			* //To set rtl API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ rtl: true });	
			* 	$("#currency").ejCurrencyTextbox({ rtl: true });
			* 	$("#percentage").ejPercentageTextbox({ rtl: true });			 
			* @example 
			* //Get or set the rtl API, after initialization:<br>
			*	//Gets the rtl value
			*	$("#numeric").ejNumericTextbox("option", "rtl");	
			*	$("#currency").ejCurrencyTextbox("option", "rtl");
			*	$("#percentage").ejPercentageTextbox("option", "rtl");			
			*	//Sets the rtl value
			*	$("#numeric").ejNumericTextbox("option", "rtl", true );
			*	$("#currency").ejCurrencyTextbox("option", "rtl", true );
			*	$("#percentage").ejPercentageTextbox("option", "rtl", true );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            rtl: false,

            /**		
			* Specify the watermark text to editor.
			* @default ""
			* @type {string}
			* @example 
			* //To set rtl API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ waterMarkText: "gradient-lime" });	
			* 	$("#currency").ejCurrencyTextbox({ waterMarkText: "gradient-lime" });
			* 	$("#percentage").ejPercentageTextbox({ waterMarkText: "gradient-lime" });			 
			* @example 
			* //Get or set the rtl API, after initialization:<br>
			*	//Gets the rtl value
			*	$("#numeric").ejNumericTextbox("option", "waterMarkText");	
			*	$("#currency").ejCurrencyTextbox("option", "waterMarkText");
			*	$("#percentage").ejPercentageTextbox("option", "waterMarkText");			
			*	//Sets the rtl value
			*	$("#numeric").ejNumericTextbox("option", "waterMarkText", "gradient-lime" );
			*	$("#currency").ejCurrencyTextbox("option", "waterMarkText", "gradient-lime" );
			*	$("#percentage").ejPercentageTextbox("option", "waterMarkText", "gradient-lime" );
			* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			* @instance
			*/

            waterMarkText: "",

            /**     
            * Fires after editor control is value is changed.
            * @event
            * @name ejNumericTextbox#change	
            * @name ejCurrencyTextbox#change
            * @name ejPercentageTextbox#change 
            * @param {Object} argument Event parameters from editors.    			 
            * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
            * @param {object}  argument.model returns the corresponding editor model.
            * @param {string}  argument.type returns the name of the event.	
            * @param {number}  argument.value returns the corresponding editor control value.			 
            * @example 
            * //change event for editors
            * $("#numeric").ejNumericTextbox({
            *    change: function (args) {}
            * });
            * $("#currency").ejCurrencyTextbox({
            *    change: function (args) {}
            * });
            * $("#percentage").ejPercentageTextbox({
            *    change: function (args) {}
            * });
            * @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
            * @instance
            */

            change: null,

            /**     
            * Fires after editor control is focused.
            * @event
            * @name ejNumericTextbox#focusIn	
            * @name ejCurrencyTextbox#focusIn
            * @name ejPercentageTextbox#focusIn 
            * @param {Object} argument Event parameters from editors.    			 
            * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
            * @param {object}  argument.model returns the corresponding editor model.
            * @param {string}  argument.type returns the name of the event.				 		
            * @example 
            * //focusIn event for editors
            * $("#numeric").ejNumericTextbox({
            *    focusIn: function (args) {}
            * });
            * $("#currency").ejCurrencyTextbox({
            *    focusIn: function (args) {}
            * });
            * $("#percentage").ejPercentageTextbox({
            *    focusIn: function (args) {}
            * });
            * @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
            * @instance
            */

            focusIn: null,

            /**     
			 * Fires after editor control is loss the focus.
			 * @event
			 * @name ejNumericTextbox#focusOut	
			 * @name ejCurrencyTextbox#focusOut
			 * @name ejPercentageTextbox#focusOut 
			 * @param {Object} argument Event parameters from editors.    			 
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.model returns the corresponding editor model.
			 * @param {string}  argument.type returns the name of the event.				 		
			 * @example 
			 * //focusOut event for editors
             * $("#numeric").ejNumericTextbox({
             *    focusOut: function (args) {}
             * });
			 * $("#currency").ejCurrencyTextbox({
             *    focusOut: function (args) {}
             * });
			 * $("#percentage").ejPercentageTextbox({
             *    focusOut: function (args) {}
             * });
			 * @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
			 * @instance
			 */
            focusOut: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            minValue: "number",
            maxValue: "number",
            incrementStep: "number",
            decimals: "number",
            showSpinButton: "boolean",
            strictMode: "boolean",
            roundedCorner: "boolean",
            rtl: "boolean",
            localize: "string",
            waterMarkText: "string",
            cssClass: "string",
            readOnly: "boolean",
            enabled: "boolean"
        },
        observables: ["value"],
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (jsondata) {
            var validate = false;
            if (!this.model.enabled && !jsondata["enabled"]) return;
            for (var key in jsondata) {
                switch (key) {
                    case "value":
                        if (ej.isNullOrUndefined(jsondata["minValue"]) && ej.isNullOrUndefined(jsondata["maxValue"])) {
                            this._setValue(jsondata[key]);
                            jsondata[key] = this.model.value;
                        }
                        else {
                            this.model.value = this._checkNumValue(jsondata[key]);
                            this._localizedFormat();
                            this._raiseChangeEvent();
                            validate = true;
                        }
                        break;
                    case "rtl": this._enableRTL(jsondata[key]); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "minValue":
                        if (isNaN(jsondata[key])) return;
                        this.model.minValue = parseFloat(jsondata[key].toFixed(this.model.decimals));
                        validate = true;
                        break;
                    case "maxValue":
                        if (isNaN(jsondata[key])) return;
                        this.model.maxValue = parseFloat(jsondata[key].toFixed(this.model.decimals));
                        validate = true;
                        break;
                    case "incrementStep":
                        if (isNaN(jsondata[key])) return;
                        this.model.incrementStep = parseFloat(jsondata[key].toFixed(this.model.decimals));
                        break;
                    case "showSpinButton": this._showSpin(jsondata[key]); break;
                    case "roundedCorner": this._roundedCorner(jsondata[key]); break;
                    case "localize": this._setLocalize(jsondata[key]); break;
                    case "decimals": this._setDecimal(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "readOnly": this._setReadOnly(jsondata[key]); break;
                    case "enabled": if (jsondata[key]) this.enable(); else this.disable(); break;
                    case "waterMarkText":
                        this.model.waterMarkText = jsondata[key]; this._setWaterMark(); break;
                }
            }
            if (validate) {
                this._validateMinMaxValue(true);
                jsondata["value"] = this.model.value;
                jsondata["maxValue"] = this.model.maxValue;
                jsondata["minValue"] = this.model.minValue;
                if (this.model.minValue != -(Number.MAX_VALUE)) this._startValue = this.model.minValue;
                this.element.attr({ 'aria-valuemin': this.model.minValue, 'aria-valuemax': this.model.maxValue, 'aria-valuenow': this.model.value });
            }
            this._checkErrorClass();
        },
        /**
       * destroy the editor widgets
       * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
       * @alias destroy
       * @return jQuery
       * @example 
       * &lt;input id="numeric" type="text" /&gt; <br> 
       * &lt;input id="currency" type="text" /&gt; <br> 
       * &lt;input id="percentage" type="text" /&gt; <br> 
       * &lt;script&gt;
       * // Create Editors
       * var numObj = $("#numeric").data("ejNumericTextbox");
       * var curObj = $("#currency").data("ejCurrencyTextbox");
       * var perObj = $("#percentage").data("ejPercentageTextbox");
       * numObj.destroy(); // destroy the numericTextbox
       * curObj.destroy(); // destroy the currencyTextbox
       * perObj.destroy(); // destroy the percentagTextbox
       * &lt;/script&gt;
       * @example 
       * &lt;input id="numeric" type="text" /&gt; <br> 
       * &lt;input id="currency" type="text" /&gt; <br> 
       * &lt;input id="percentage" type="text" /&gt; <br> 
       * &lt;script&gt;
       * // enable the editors
       * $("#numeric").ejNumericTextbox("destroy");
       * $("#currency").ejCurrencyTextbox("destroy");
       * $("#percentage").ejPercentageTextbox("destroy");		
       * &lt;/script&gt;
       * @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
       * @instance
        */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            if (this.wrapper) {
                this.element.insertBefore(this.wrapper);
                this.wrapper.remove();
            }
            if (this.model.name) this.element.removeAttr("name");
            this.element.val("").removeClass('e-input e-watermark').empty();
        },

        /**
        * Initialize the editors widget
        * @private
        */
        // constructor function
        _init: function () {
            if (this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type'))) {
                this._setValues();
                this._renderControl();
                this._wireEvents();
                this._initObjects();
            }
            else {
                this._destroy();
                return false;
            }
        },

        // -----------------------------------private function----------------------------------//

        /**
         * To configure the editors value		
		 * @private
         */
        _setValues: function () {
            this._id = this.element[0].id;
            this._textBox = this.element[0];
            this._error = false;
            this._timeout = null;
            this.isValidState = true;
            this._allowkeyboard = true;
            this._validateOnType = false;
            this._focused = false;
            this._startValue = 0;
            if (this.sfType === "ej.CurrencyTextbox" && this.model.minValue == -(Number.MAX_VALUE))
                this.model.minValue = 0;
            this.model.minValue = parseFloat(this.model.minValue.toFixed(this.model.decimals));
            this.model.maxValue = parseFloat(this.model.maxValue.toFixed(this.model.decimals));
            if (this.model.minValue != -(Number.MAX_VALUE)) this._startValue = this.model.minValue;
            this.model.value = this._checkNumValue(this.model.value);
            this._localizedFormat();
            this._validateMinMaxValue(true);
            this.model.waterMarkText = this._checkWaterMark(this.model.waterMarkText);
            this._percentSymbol = Globalize.culture(this.model.localize).numberFormat.percent.symbol;
            this._currencySymbol = Globalize.culture(this.model.localize).numberFormat.currency.symbol;
        },

        /**
        * Render Section For DifferentTypes		
        * @private
        */
        _renderControl: function () {
            var editorwidget = $(document.createElement('span')).addClass('e-widget');
            var editorwidgetInner = $(document.createElement('span')).addClass('e-in-wrap e-box');
            if (this.sfType === "ej.NumericTextbox")
                editorwidget.addClass('e-numeric');
            else if (this.sfType === "ej.PercentageTextbox")
                editorwidget.addClass('e-percent');
            else if (this.sfType === "ej.CurrencyTextbox")
                editorwidget.addClass('e-currency');
            editorwidget.append(editorwidgetInner).insertAfter(this.element);
            editorwidgetInner.append(this.element);
            this.model.name = this.model.name != null ? this.model.name : this.element[0].id;
            this.element.attr('name', this.model.name).addClass('e-input');
            this.element.attr({ 'role': 'spinbutton', 'aria-valuemin': this.model.minValue, 'aria-valuemax': this.model.maxValue, 'aria-valuenow': this.model.value, 'tabindex': '0', 'aria-live': 'assertive' });
            var spinbutton = $('<span class="e-select"><span class="e-spin e-spin-up " role="button" "aria-label"="Increase Value" /><span class="e-spin e-spin-down" role="button" "aria-label"="Decrease Value" /></span>');
            var spinimg = ej.buildTag('span.e-icon e-arrow').attr('role', 'presentation');
            editorwidgetInner.append(spinbutton);
            spinbutton.find('.e-spin').append(spinimg);
            this.innerWrap = this.element.parent();
            this.wrapper = this.innerWrap.parent();
            this.spin = this.wrapper.find('.e-select');
            this.spinUp = this.wrapper.find('.e-spin-up');
            this.spinDown = this.wrapper.find('.e-spin-down');
            this._setWidth(this.model.width);
            this._setHeight(this.model.height);
            if (this.model.cssClass != "") this._setSkin(this.model.cssClass);
            this._showSpin(this.model.showSpinButton);
            if (this.model.roundedCorner) this._roundedCorner(this.model.roundedCorner);
            if (this.model.rtl) this._enableRTL(this.model.rtl);
            if (this.model.readOnly) this._setReadOnly(this.model.readOnly);
            if (!this.model.enabled) this.disable();
            this.wrapper = editorwidget;
        },

        /**
         * To initialize the editors		
		 * @private
         */
        _initObjects: function () {
            this._preVal = this._numberValue();
            if (this._textBox.value == "" && this.model.waterMarkText != "")
                this._setWaterMark();
            if (this.sfType === "ej.PercentageTextbox" && this._textBox.value != "")
                this._appendPercentSymbol(this._textBox.value);
            else if (this.sfType === "ej.CurrencyTextbox" && this._textBox.value != "")
                this._appendCurrencySymbol(this._textBox.value);
            if (this.model.value == "") this.isValidState = true;
            else if ((this.model.value < this.model.minValue) || (this.model.value > this.model.maxValue))
                this.isValidState = false;
            this._checkErrorClass();
        },

        /**
        * To enable or disable the show spin button in editors.
        * @private
        */
        _showSpin: function (value) {
            if (!value) {
                if (this.spin) {
                    this.spin.hide();
                    this.innerWrap.removeClass('e-padding');
                }
                this._unbindSpinEvents();
            }
            else {
                if (this.spin) {
                    this.spin.show();
                    this.innerWrap.addClass('e-padding');
                }
                this._bindSpinEvents();
            }
        },

        /**
        * To enable or disable the rounded corner behaviour 		
        * @private
        */
        _roundedCorner: function (value) {
            if (value && !this.innerWrap.hasClass('e-corner-all'))
                this.innerWrap.addClass('e-corner-all');
            else if (this.innerWrap.hasClass('e-corner-all'))
                this.innerWrap.removeClass('e-corner-all');
        },

        /**
        * To enable or disable the Right to Left Direction behaviour 		
        * @private
        */
        _enableRTL: function (rtl) {
            if (rtl) {
                if (this.spin) this.wrapper.addClass("e-rtl");
                else this.element.addClass("e-rtl");
            }
            else {
                if (this.spin) this.wrapper.removeClass("e-rtl");
                else this.element.removeClass("e-rtl");
            }
        },

        /**
        * To configure editors width		
        * @private
        */
        _setWidth: function (value) {
            this.wrapper.width(value);
        },

        /**
        * To configure editors height		
        * @private
        */
        _setHeight: function (value) {
            this.wrapper.height(value);
        },

        /**
        * To configure the custom theme for editors using cssClass property		
        * @private
        */
        _setSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass);
            this.wrapper.addClass(skin);
        },

        /**
        * To configure value to editors		
        * @private
        */
        _setValue: function (value) {
            this.model.value = this._checkNumValue(value);
            this._validateMinMaxValue(false);
            if ((this.model.minValue <= this.model.value) && (this.model.value <= this.model.maxValue)) {
                this.isValidState = true;
            } else
                this.isValidState = false;
            this._checkErrorClass();
            this._localizedFormat();
            this._raiseChangeEvent();
        },

        /**
        * To configure localization to editors		
        * @private
        */
        _setLocalize: function (val) {
            this.model.localize = val;
            this._percentSymbol = Globalize.culture(this.model.localize).numberFormat.percent.symbol;
            this._currencySymbol = Globalize.culture(this.model.localize).numberFormat.currency.symbol;
            this._localizedFormat();
        },

        /**
        * To configure decimal property to allow decimal values to editors		
        * @private
        */
        _setDecimal: function (val) {
            this.model.decimals = val;
            this._localizedFormat();
        },

        /**
         * To check whether value within min max range.		
		 * @private
         */
        _validateMinMaxValue: function (fromMinMax) {
            var valChange = false;
            if (this.model.minValue > this.model.maxValue) this.model.minValue = this.model.maxValue;
            if (this.model.value != "" && this.model.minValue > this.model.value) {
                if (this.model.strictMode != true) {
                    this.isValidState = true;
                    this._startValue = this.model.value = this.model.minValue;
                }
                else this.isValidState = false;
                valChange = true;
            }
            else if (this.model.value != "" && this.model.maxValue < this.model.value) {
                if (this.model.strictMode != true) {
                    this.isValidState = true;
                    this.model.value = this.model.maxValue;
                }
                else this.isValidState = false;
                valChange = true;
            }
            else this.isValidState = true;
            if (this.model.minValue == this.model.maxValue) this._startValue = this.model.minValue;
            if ((valChange && fromMinMax)) {
                this._localizedFormat();
                this._raiseChangeEvent();
            }
        },

        /**
         * To configure localization format to editors		
		 * @private
         */
        _localizedFormat: function () {
            this._wipeWaterMark();
            this._textBox.value = Globalize.format((this.model.value), "n" + this.model.decimals, this.model.localize);
            if (!this._focused && this._textBox.value != "") {
                if (this.sfType === "ej.PercentageTextbox") this._appendPercentSymbol(this._textBox.value);
                else if (this.sfType === "ej.CurrencyTextbox") this._appendCurrencySymbol(this._textBox.value);
            }
        },

        /**
         * To check whether value is proper format in editors		
		 * @private
         */
        _checkNumValue: function (value) {
            if (typeof value == "string" && !isNaN(Globalize.parseFloat(value, this.model.localize))) {
                value = Globalize.parseFloat(value, this.model.localize);
                return parseFloat(value);
            }
            else if ((typeof value == "number") && !isNaN(value))
                return value;
            else return "";
        },

        /**
         * To check whether waterMark in editors		
		 * @private
         */
        _checkWaterMark: function (val) {
            if (!val)
                return "";
            else return val;
        },

        /**
        * To enable or disable the readOnly behaviour 		
        * @private
        */
        _setReadOnly: function (bool) {
            if (bool) this.element.attr("readonly", true);
            else this.element.removeAttr("readonly");
        },

        /**
         * To configure the editors waterMark text		
		 * @private
         */
        /* Set Water Mark Text */
        _setWaterMark: function () {
            this.element.addClass("e-watermark");
            this._textBox.value = this.model.waterMarkText;
        },

        /**
         * To remove the editors waterMark text		
		 * @private
         */
        /* Clear Water Mark Text */
        _wipeWaterMark: function () {
            this.element.removeClass("e-watermark");
        },

        /**
         * To configure the editors value selection range. 		
		 * @private
         */
        _setSelectionRange: function (selectionStart, selectionEnd) {
            var input = this._textBox;
            try {
                if (input.setSelectionRange) {
                    input.setSelectionRange(selectionStart, selectionEnd);
                    input.focus();
                }
                else if (input.createTextRange) {
                    var range = input.createTextRange();
                    _setselction(range);
                }
            }
            catch (e) {
                var control = this;
                window.setTimeout(function () {
                    document.body.focus();
                    control._textBox.select();
                    var range = document.selection.createRange();
                    _setselction(range);
                }, 1);
            }
            function _setselction(range) {
                range.collapse(true);
                range.moveEnd('character', selectionEnd);
                range.moveStart('character', selectionStart);
                range.select();
            }
        },

        /**
         * To obtain the editors value in selection range. 		
		 * @private
         */
        _getSelection: function (value) {
            var oSel = null;
            if (document.selection) {
                oSel = document.selection.createRange();
                return (oSel.text == "" ? oSel.text : this._removeFormats(oSel.text));
            }
            else {
                if (value == null)
                    return this.model.value;
                else {
                    oSel = this._removeFormats(value.substring(this._textBox.selectionStart, this._textBox.selectionEnd));
                    return oSel;
                }
            }
        },

        /**
         * To calculate selection range position in editors value. 		
		 * @private
         */
        _caretPosition: function () {
            var oField = this._textBox;
            // Initialize
            var iCaretPos = 0;
            // IE Support
            if (document.selection) {
                // Set focus on the element
                oField.focus();
                // To get cursor position, get empty selection range
                var oSel = document.selection.createRange();
                // Move selection start to 0 position
                oSel.moveStart('character', -oField.value.length);
                // The caret position is selection length
                iCaretPos = oSel.text.length;
            }
                // Firefox support
            else if (oField.selectionStart || oField.selectionStart == '0')
                iCaretPos = oField.selectionEnd;
            // Return results
            return (iCaretPos);
        },

        /**
         * To configure the percentage symbol in percentageTextbox value. 		
		 * @private
         */
        _appendPercentSymbol: function (value) {
            if (this._percentSymbol) value = value.replace(this._percentSymbol, "");
            if (value.indexOf(Globalize.culture(this.model.localize).numberFormat.percent.symbol) < 0)
                this._textBox.value = value + Globalize.culture(this.model.localize).numberFormat.percent.symbol;
            this._percentSymbol = Globalize.culture(this.model.localize).numberFormat.percent.symbol;

        },

        /**
         * To configure the currency symbol in currencyTextbox value. 		
		 * @private
         */
        _appendCurrencySymbol: function (value) {
            if (this._currencySymbol) value = value.replace(this._currencySymbol, "");
            if (value.indexOf(Globalize.culture(this.model.localize).numberFormat.currency.symbol) < 0)
                if (Number(value) < 0)
                    this._textBox.value = "(" + Globalize.culture(this.model.localize).numberFormat.currency.symbol + Math.abs(value) + ")";
                else
                    this._textBox.value = Globalize.culture(this.model.localize).numberFormat.currency.symbol + value;
            this._currencySymbol = Globalize.culture(this.model.localize).numberFormat.currency.symbol;

        },

        /**
         * To remove the localization format in editors. 		
		 * @private
         */
        _removeFormats: function (val) {
            var grpSep = Globalize.culture(this.model.localize).numberFormat[","];
            if (val != null) {
                if (grpSep == ".")
                    return (val.toString().replace(/["."]/g, ""));
                else if (val.toString().match(new RegExp(grpSep, "g")))
                    return (val.toString().replace(new RegExp(grpSep, "g"), ""));
                else
                    return val;
            }
            else
                return null;
        },

        /**
         * To check whether error class added or not editors. 		
		 * @private
         */
        _checkErrorClass: function (obj) {
            if (this.isValidState) this.wrapper.removeClass("e-error");
            else this.wrapper.addClass("e-error");
        },
        // -----------------------------------public function----------------------------------//
        /**
        * To enable the corresponding editors 		
		* @return jQuery
		* @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* // Create Editors
		* var numObj = $("#numeric").data("ejNumericTextbox");
		* var curObj = $("#currency").data("ejCurrencyTextbox");
		* var perObj = $("#percentage").data("ejPercentageTextbox");
		* numObj.enable(); // enable the numericTextbox
		* curObj.enable(); // enable the currencyTextbox
		* perObj.enable(); // enable the percentagTextbox
		* &lt;/script&gt;
		 @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* // enable the editors
		* $("#numeric").ejNumericTextbox("enable");
		* $("#currency").ejCurrencyTextbox("enable");
		* $("#percentage").ejPercentageTextbox("enable");		
		* &lt;/script&gt;
		* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
		* @instance
         */
        enable: function () {
            this.model.enabled = true;
            this._textBox.disabled = false;
            this.element.removeAttr("disabled");
            this.element.removeClass('e-disable').attr({ "aria-disabled": false });
            this.wrapper.find(".e-select").removeClass('e-disable').attr({ "aria-disabled": false });
        },

        /**
        * To disable the corresponding editors 		
		* @return jQuery
		* @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* // Create Editors
		* var numObj = $("#numeric").data("ejNumericTextbox");
		* var curObj = $("#currency").data("ejCurrencyTextbox");
		* var perObj = $("#percentage").data("ejPercentageTextbox");
		* numObj.disable(); // enable the numericTextbox
		* curObj.disable(); // enable the currencyTextbox
		* perObj.disable(); // enable the percentagTextbox
		* &lt;/script&gt;
		 @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* // enable the editors
		* $("#numeric").ejNumericTextbox("disable");
		* $("#currency").ejCurrencyTextbox("disable");
		* $("#percentage").ejPercentageTextbox("disable");		
		* &lt;/script&gt;
		* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
		* @instance
         */
        disable: function () {
            this.model.enabled = false;
            this._textBox.disabled = true;
            this.element.attr("disabled", "disabled");
            this.element.addClass('e-disable').attr({ "aria-disabled": true });
            this.wrapper.find(".e-select").addClass('e-disable').attr({ "aria-disabled": true });
        },

        /**
        * To get value from corresponding editors 		
		* @return jQuery
		* @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* // Create Editors
		* var numObj = $("#numeric").data("ejNumericTextbox");
		* var curObj = $("#currency").data("ejCurrencyTextbox");
		* var perObj = $("#percentage").data("ejPercentageTextbox");
		* numObj.getValue(); // get value from numericTextbox
		* curObj.getValue(); // get value from currencyTextbox
		* perObj.getValue(); // get value from percentagTextbox
		* &lt;/script&gt;
		 @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* // get value from editors
		* $("#numeric").ejNumericTextbox("getValue");
		* $("#currency").ejCurrencyTextbox("getValue");
		* $("#percentage").ejPercentageTextbox("getValue");		
		* &lt;/script&gt;
		* @memberof ejNumericTextbox,ejCurrencyTextbox,ejPercentageTextbox
		* @instance
         */
        getValue: function () {
            return this.model.value == null ? "" : this.model.value;
        },
        // -----------------------------------wire and unwire events and handlers function----------------------------------//

        /**
         * Wiring the events to editors control		
		 * @private
         */
        _wireEvents: function () {
            this._on(this.element, 'focus', this._focusIn);
            this._on(this.element, 'blur', this._focusOut);
            this._on(this.element, 'keydown', this._keyDown);
            this._on(this.element, "keyup", this._keyUp);
            this._on(this.element, 'mousewheel', this._mouseWheel);
            this._on(this.element, 'DOMMouseScroll', this._mouseWheel);
        },
        /**
         * Wiring the events to editors spinbutton		
		 * @private
         */
        _bindSpinEvents: function () {
            this._on(this.spinUp, "mousedown", this._spinUpClick);
            this._on(this.spinDown, "mousedown", this._spinDownClick);
            this._on(this.spinUp, "mouseup", this._spinUpClick);
            this._on(this.spinDown, "mouseup", this._spinDownClick);
        },
        /**
         * unWiring the events from editors spinbutton		
		 * @private
         */
        _unbindSpinEvents: function () {
            this._off(this.spinUp, "mousedown", this._spinUpClick);
            this._off(this.spinDown, "mousedown", this._spinDownClick);
            this._off(this.spinUp, "mouseup", this._spinUpClick);
            this._off(this.spinDown, "mouseup", this._spinDownClick);
        },
        /**
        * Section For handle the spin button over mouse up click event
        * @private
        */
        _spinUpClick: function (event) {
            var self = this;
            event.preventDefault();
            clearTimeout(this._timeout);
            if (!this.model.enabled || this.model.readOnly) return;
            this._on(this.spinUp, 'mouseleave', this._mouseUpClick);
            this.spinUp.addClass("e-active");
            var self = this;
            if (event.type == "mouseup") {
                this._incrementValue(event);
                this.spinUp.removeClass("e-active");
                this._off($(document), 'mouseup', this._mouseUpClick);
            }
            else if (event.type == "mousedown") {
                if (!this._focused) this.element[0].focus();
                this._timeout = setInterval(function () { self._incrementValue(event); }, 150);
                this._on($(document), 'mouseup', this._mouseUpClick);
            }
        },
        /**
        * Section For handle the spin button over mouse down click event
        * @private
        */
        _spinDownClick: function (event) {
            var self = this;
            event.preventDefault();
            clearTimeout(this._timeout);
            if (!this.model.enabled || this.model.readOnly) return;
            this._on(this.spinDown, 'mouseleave', this._mouseUpClick);
            this.spinDown.addClass("e-active");
            if (event.type == "mouseup") {
                this._decrementValue(event);
                this.spinDown.removeClass("e-active");
                this._off($(document), 'mouseup', this._mouseUpClick);
            }
            else if (event.type == "mousedown") {
                if (!this._focused) this.element[0].focus();
                this._timeout = setInterval(function () { self._decrementValue(event); }, 150);
                this._on($(document), 'mouseup', this._mouseUpClick);
            }
        },
        /**
         * Section For handle the mouse up click event
		 * @private
         */
        _mouseUpClick: function (event) {
            event.stopPropagation();
            clearTimeout(this._timeout);
            this._off(this.spinUp, 'mouseleave', this._mouseUpClick);
            this._off(this.spinDown, 'mouseleave', this._mouseUpClick);
            this.spinDown.removeClass("e-active");
            this.spinUp.removeClass("e-active");
        },
        /**
         * Section For handle the mouse wheel event
		 * @private
         */
        _mouseWheel: function (event) {
            event.preventDefault();
            if (!this._focused) this.element[0].focus();
            if (this.model.readOnly) return;
            var delta;
            var rawEvent = event.originalEvent;
            if (rawEvent.wheelDelta) {
                // IE and Opera use wheelDelta, which is a multiple of 120 (possible values -120, 0, 120).
                delta = rawEvent.wheelDelta / 120;
                // In Opera, value is negated.
                //if (Sys.Browser.agent === Sys.Browser.Opera) delta = -delta;
            }
            else if (rawEvent.detail) {
                // Firefox uses detail property, which is a multiple of 3.
                delta = -rawEvent.detail / 3;
            }
            if (delta > 0)
                this._incrementValue(event);
            else if (delta < 0)
                this._decrementValue(event);
            this._cancelEvent(event);
        },

        /**
         * Section For handle number value from editors
		 * @private
         */
        _numberValue: function () {
            var value = this._textBox.value;
            if (this.sfType === "ej.PercentageTextbox") {
                value = value.split(this._percentSymbol)[0];
            }
            else if (this.sfType === "ej.CurrencyTextbox") {
                var temp = value.split(this._currencySymbol)[1];
                value = temp ? temp.replace(")", "") : value;
            }
            return value;
        },
        /**
         * Section For handle the focus in event
		 * @private
         */
        _focusIn: function (event) {
            this._focused = true;
            this.wrapper.addClass("e-focus");
            // this.element.addClass("e-focus");
            if (this.model.readOnly)
                return;
            this.wrapper.removeClass('e-error');
            if (this._textBox.value == this.model.waterMarkText) {
                this._textBox.value = "";
                if (this.element.hasClass("e-watermark")) this._wipeWaterMark();
            }
            if (!this._error) {
                if (this._textBox.value != "")
                    this._textBox.value = this._numberValue();
                this._setSelectionRange(0, this._textBox.value.length);
            }
            if (this.model.focusIn) this._trigger("focusIn");
        },
        /**
         * Section For handle the focus out event
		 * @private
         */
        _focusOut: function (event) {
            this._focused = false;
            this.wrapper.removeClass("e-focus");
            if (!this._error) {
                if (this._textBox.value != "" && this._textBox.value != this.model.waterMarkText) {
                    if (parseFloat(this._textBox.value) < this.model.minValue) {
                        if (this.model.strictMode == false)
                            this._textBox.value = this.model.minValue.toFixed(this.model.decimals);
                        else
                            this.isValidState = false
                    }
                    else if (parseFloat(this._textBox.value) > this.model.maxValue) {
                        if (this.model.strictMode == false)
                            this._textBox.value = this.model.maxValue.toFixed(this.model.decimals);
                        else
                            this.isValidState = false;
                    }
                    if (this.sfType === "ej.NumericTextbox")
                        this.model.value = Globalize.parseFloat(this._textBox.value, this.model.localize);
                    else if (this.sfType === "ej.PercentageTextbox" || this.sfType === "ej.CurrencyTextbox") {
                        var editorvalue = this._textBox.value, value = editorvalue;
                        if (editorvalue.indexOf(Globalize.culture(this.model.localize).numberFormat.percent.symbol) > -1)
                            value = editorvalue.substring(0, editorvalue.length - 1);
                        else if (editorvalue.indexOf(Globalize.culture(this.model.localize).numberFormat.currency.symbol) > -1)
                            value = editorvalue.substring(1);
                        this.model.value = Globalize.parseFloat(value, this.model.localize);
                    }
                    if (isNaN(this.model.value) && !this.model.value) {
                        this.model.value = "";
                        this.isValidState = false;
                    }
                    this._localizedFormat();
                    this._checkErrorClass();
                }
                else {
                    if (this._textBox.value == "" && this.model.waterMarkText == "")
                        this._textBox.value = "";
                    else if (this._textBox.value == "" && this.model.waterMarkText != "")
                        this._setWaterMark();
                    if (this._textBox.value != "") {
                        if (this._textBox.value == this.model.waterMarkText)
                            this.model.value = "";
                        else
                            this._setValue(this._textBox.value);
                    }
                }
                if (this.model.value == "" || this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) this.isValidState = true;
                else if (this.model.strictMode) this.isValidState = false;
                this._keyUp();
                if (this.model.focusOut) this._trigger("focusOut");
            }
            this._checkErrorClass();
        },
        /**
         * Section For handle the cancel event
		 * @private
         */
        _cancelEvent: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            e.stopPropagation();
            e.preventDefault();
            return false;
        },
        /**
         * Section For handle the increment value in editors.
		 * @private
         */
        /* Increment the Value */
        _incrementValue: function (e) {
            this.isValidState = true;
            if (this._textBox.value != this.model.waterMarkText || this._textBox.value == "") {
                if (this._textBox.value == "")
                    this._textBox.value = this._startValue;
                if (this.sfType === "ej.PercentageTextbox" || this.sfType === "ej.CurrencyTextbox") {
                    var editorvalue = this._textBox.value, value = editorvalue;
                    if (editorvalue.indexOf(Globalize.culture(this.model.localize).numberFormat.percent.symbol) > -1)
                        value = editorvalue.substring(0, editorvalue.length - 1);
                    else if (editorvalue.indexOf(Globalize.culture(this.model.localize).numberFormat.currency.symbol) > -1)
                        value = editorvalue.substring(1);
                    this.model.value = Globalize.parseFloat(value, this.model.localize);
                }
                else if (this.sfType == "ej.NumericTextbox")
                    this.model.value = Globalize.parseFloat(this._textBox.value, this.model.localize);

                if (isNaN(this.model.value) && !this.model.value) {
                    this.model.value = this._startValue;
                }
                if (this.model.strictMode == true) {
                    if (this.model.value > this.model.minValue && this.model.value > this.model.maxValue) {
                        this.model.value = this.model.maxValue;
                        this._setValue(this.model.value);
                        this._wipeWaterMark();
                    }
                    else if (this.model.value < this.model.minValue && this.model.value < this.model.maxValue) {
                        this.model.value = this.model.minValue;
                        this._setValue(this.model.value);
                        this._wipeWaterMark();
                    }
                    else
                        if (this.model.value >= this.model.minValue && this.model.value < this.model.maxValue) {
                            this.model.value = this.model.value + this.model.incrementStep;
                            if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                                this._setValue(this.model.value);
                                this._wipeWaterMark();
                            }
                        };
                }
                else
                    if (this.model.value >= this.model.minValue && this.model.value < this.model.maxValue) {
                        this.model.value = this.model.value + this.model.incrementStep;
                        if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                            this._setValue(this.model.value);
                            this._wipeWaterMark();
                        }
                    }
            }
            this._checkErrorClass();
        },
        /**
         * Section For handle the decrement value in editors.
		 * @private
         */
        /* Decrement the Value */
        _decrementValue: function (e) {
            this.isValidState = true;
            var itemValue = this.model.value;
            if (this._textBox.value != this.model.waterMarkText || this._textBox.value == "") {
                if (this._textBox.value == "")
                    this._textBox.value = this._startValue;
                else
                    if (this._textBox.value.indexOf(" ") >= 0)
                        this._textBox.value = this._textBox.value.replace(" ", "");
                if (this.sfType === "ej.PercentageTextbox" || this.sfType === "ej.CurrencyTextbox") {
                    var editorvalue = this._textBox.value, value = editorvalue;
                    if (editorvalue.indexOf(Globalize.culture(this.model.localize).numberFormat.percent.symbol) > -1)
                        value = editorvalue.substring(0, editorvalue.length - 1);
                    else if (editorvalue.indexOf(Globalize.culture(this.model.localize).numberFormat.currency.symbol) > -1)
                        value = editorvalue.substring(1);
                    this.model.value = Globalize.parseFloat(value, this.model.localize);
                }
                else if (this.sfType == "ej.NumericTextbox")
                    this.model.value = Globalize.parseFloat(this._textBox.value, this.model.localize);

                if (isNaN(this.model.value) && !this.model.value) {
                    this.model.value = this._startValue;
                }
                if (this.model.strictMode == true) {
                    if (this.model.value >= this.model.minValue && this.model.value > this.model.maxValue) {
                        this.model.value = this.model.maxValue;
                        this._setValue(this.model.value);
                        this._wipeWaterMark();
                    }
                    else if (this.model.value < this.model.minValue && this.model.value <= this.model.maxValue) {
                        this.model.value = this.model.minValue;
                        this._setValue(this.model.value);
                        this._wipeWaterMark();
                    }
                    else if (this.model.value > this.model.minValue && this.model.value <= this.model.maxValue) {
                        this.model.value = this.model.value - this.model.incrementStep;
                        if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                            this._setValue(this.model.value);
                            this._wipeWaterMark();
                        }
                    }
                }
                else
                    if (this.model.value > this.model.minValue && this.model.value <= this.model.maxValue) {
                        this.model.value = this.model.value - this.model.incrementStep;
                        if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                            this._setValue(this.model.value);
                            this._wipeWaterMark();
                        }
                    }
            }
            this._checkErrorClass();
        },
        /**
         * Section For handle the validate decimal value
		 * @private
         */
        _validateDecimal: function (e) {
            if (window.navigator.appName == "Microsoft Internet Explorer") {
                if (e.originalEvent.char == Globalize.culture(this.model.localize).numberFormat["."])
                    return true;
            }
            else {
                var char = String.fromCharCode(e.keyCode);
                if (e.keyCode == 188) char = ",";
                else if (e.keyCode == 190 || e.keyCode == 110) char = ".";
                if (Globalize.culture(this.model.localize).numberFormat["."].charCodeAt(0) == char.charCodeAt(0))
                    return true;
            }
            return false;
        },
        /**
         * Section For handle the keycodes
		 * @private
         */
        _allowKeyCodes: function (e) {
            var keys = new Array(38, 40, 35, 36, 109, 189, 46, 8, 127, 37, 39, 190, 9, 13, 16, 17, 18, 20, 110, 173, 86, 88, 67);
            for (var i = 0; i < keys.length; i++) {
                if (e.keyCode == keys[i] || (this._validateDecimal(e) && this.model.decimals != 0))
                    return true;
            }
            return false;
        },
        /**
         * Section For raise the event.
		 * @private
         */
        _raiseChangeEvent: function () {
            var currVal = this._numberValue();
            if (!ej.isNullOrUndefined(this._preVal) && (this._preVal.toString() != currVal)) {
                this._preVal = currVal;
                this.model.value = this._checkNumValue(this._preVal);
                this.element.attr('aria-valuenow', this.model.value);
                if (this.model.change)
                    this._trigger("change", { value: this.model.value });
            }
        },
        /**
         * Section For handle the key up event
		 * @private
         */
        _keyUp: function () {
            this._raiseChangeEvent();
        },
        /**
         * Section For handle the key down event
		 * @private
         */
        _keyDown: function (e) {
            if (this.model.readOnly) return;
            var cursor, cancelEvent, beforeCursor, afterCursor, sel;
            if (e.keyCode >= 48 && e.keyCode <= 57 || e.keyCode >= 96 && e.keyCode <= 105 || this._allowKeyCodes(e)) {
                if (e.shiftKey && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || e.keyCode == 46 || e.keyCode == 127) || (e.ctrlKey && (e.keyCode == 86 || e.keyCode == 118 || e.keyCode == 67 || e.keyCode == 88)))
                    return true;
                if (((e.shiftKey == true || e.ctrlKey == true) && (e.keyCode != 9 && e.keyCode != 17 && e.keyCode != 86 && e.keyCode != 67)) || (e.keyCode == 67 || e.keyCode == 86 || e.keyCode == 88)) {//Prevent Shift + Tab event
                    this._keypressFlag = false;
                    this._cancelEvent(e);
                    return false;
                }

                if (e.keyCode >= 48 && e.keyCode <= 57 || e.keyCode >= 96 && e.keyCode <= 105 || e.keyCode == 110) {//Numbers from 0 to 9 in keyboard and numpad (110 for decimal)
                    this._wipeWaterMark();
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.toString().substring(0, cursor);
                    afterCursor = this._textBox.value.toString().substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                    // For Numpad issue
                    if (e.keyCode >= 96 && e.keyCode <= 105)
                        e.keyCode -= 48;
                    if (sel.length == 0)
                        // if (beforeCursor.length != 0 && String.fromCharCode(e.keyCode) == 0)                           
                        this.model.value = Globalize.parseFloat((beforeCursor + String.fromCharCode(e.keyCode) + afterCursor), this.model.localize);
                    else
                        this.model.value = Globalize.parseFloat((beforeCursor.substring(0, beforeCursor.length - sel.length) + String.fromCharCode(e.keyCode) + afterCursor), this.model.localize);
                    if (this._validateOnType) {
                        var decSep = Globalize.culture(this.model.localize).numberFormat["."];
                        var temp = this.model.value.toString();
                        var tempIndex = temp.indexOf(decSep);
                        var value = this.model.value;
                        var replaceminVal = this.model.minValue.toFixed(this.model.decimals);
                        var replacemaxVal = this.model.maxValue.toFixed(this.model.decimals);
                        var exp = false;
                        if (replaceminVal.toString().indexOf("e") > 0 || replacemaxVal.toString().indexOf("e") > 0)
                            exp = true;
                        if (!exp) {
                            if (decSep != ".") {
                                temp = this._textBox.value.toString();
                                if (this.model.minValue.toString().match(new RegExp(".", "g")))
                                    replaceminVal = this.model.minValue.toString().replace(/["."]/g, decSep);
                                if (this.model.maxValue.toString().match(new RegExp(".", "g")))
                                    replacemaxVal = this.model.maxValue.toString().replace(/["."]/g, decSep);
                            }
                            if (replaceminVal.toString().indexOf(decSep) > 0)
                                minVal = Number(replaceminVal.toString().substring(0, replaceminVal.toString().indexOf(decSep)));
                            else
                                minVal = this.model.minValue;
                            if (replacemaxVal.toString().indexOf(decSep) > 0)
                                maxVal = Number(replacemaxVal.toString().toString().substring(0, replacemaxVal.toString().indexOf(decSep)));
                            else
                                maxVal = this.model.maxValue;
                            if (tempIndex > 0) {
                                value = Number(temp.substring(temp.indexOf(decSep) + 1, temp.toString().length));
                                if (replaceminVal.toString().indexOf(decSep) > 0)
                                    minVal = Number(replaceminVal.toString().substring(replaceminVal.toString().indexOf(decSep) + 1, replaceminVal.toString().length));
                                else
                                    minVal = 0;
                                if (replacemaxVal.toString().indexOf(decSep) > 0)
                                    maxVal = Number(replacemaxVal.toString().substring(replacemaxVal.toString().indexOf(decSep) + 1, replacemaxVal.toString().length));
                                else
                                    maxVal = 0;
                                var valb4Dec = Number(temp.substring(0, temp.indexOf(decSep)));
                                var minValb4Dec = Number(replaceminVal.toString().substring(0, replaceminVal.toString().indexOf(decSep)));
                                var maxValb4Dec = Number(replacemaxVal.toString().substring(0, replacemaxVal.toString().indexOf(decSep)));
                                if (!this._validateValue(value, minVal, maxVal, true, valb4Dec, minValb4Dec, maxValb4Dec))
                                    cancelEvent = true;
                                else
                                    cancelEvent = false;
                            }
                            else {
                                if (!this._validateValue(value, minVal, maxVal, false))
                                    cancelEvent = true;
                                else
                                    cancelEvent = false;
                            }
                        }
                        else if (this.model.decimals != 0) {
                            var afterdeci = (this._textBox.value).split(".")[1];
                            if (temp.indexOf(decSep) > 0 || (Number(temp) < Number(replaceminVal) || Number(temp) > Number(replacemaxVal))) {
                                if ((temp.substring(temp.indexOf(decSep) + 1, temp.length)).length > this.model.decimals)
                                    cancelEvent = true;
                                else if (sel.length == 0 && cursor > (this._textBox.value.indexOf(decSep)) && afterdeci && afterdeci.length >= this.model.decimals) cancelEvent = true;
                                else cancelEvent = false;
                            }
                            else if (sel.length == 0 && cursor > (this._textBox.value.indexOf(decSep)) && afterdeci && afterdeci.length >= this.model.decimals) cancelEvent = true;
                            else
                                cancelEvent = false;
                        }
                        else
                            cancelEvent = false;
                        if (cancelEvent) {
                            if (sel.length == 0)
                                this.model.value = parseFloat(beforeCursor + afterCursor);
                            else
                                this.model.value = parseFloat(beforeCursor.substring(0, beforeCursor.length - sel.length) + afterCursor);
                            this._keypressFlag = false;
                            this._cancelEvent(e);
                            return false;
                        }
                    }
                    this._raiseChangeEvent();
                }
                if (e.keyCode == 38 && this._allowkeyboard)//Up Arrow  
                {
                    this._incrementValue(e);
                    this._cancelEvent(e);
                }

                if (e.keyCode == 40 && this._allowkeyboard)//Down Arrow 
                {
                    this._decrementValue(e);
                    this._cancelEvent(e);

                }
                if (e.keyCode == 8) {//BackSpace Key
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.substring(0, cursor);
                    afterCursor = this._textBox.value.substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                    if (sel.length == 0)
                        this.model.value = beforeCursor.substring(0, beforeCursor.length - 1) + afterCursor;
                    else
                        this.model.value = beforeCursor.substring(0, beforeCursor.length - sel.length) + afterCursor;

                    this._raiseChangeEvent();
                }

                if (e.keyCode == 46 || e.keyCode == 127) {//Delete key
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.substring(0, cursor);
                    afterCursor = this._textBox.value.substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                    if (sel.length == 0)
                        this.model.value = beforeCursor + afterCursor.substring(1, afterCursor.length);
                    else
                        this.model.value = beforeCursor.substring(0, beforeCursor.length - sel.length) + afterCursor.substring(0, afterCursor.length);
                    this._raiseChangeEvent();
                }
                if (this._validateDecimal(e) && this.model.decimals != 0 && e.keyCode != 46)//'.' char key press
                {
                    var decChar = Globalize.culture(this.model.localize).numberFormat["."];
                    var minVal, maxVal;
                    var dotSplit = this._textBox.value.split(decChar);
                    if (dotSplit[1] == undefined) {
                        var curPosition = this._caretPosition();
                        var strBeforeCursor = dotSplit[0].substring(0, curPosition);
                        var strAfterCursor = dotSplit[0].substring(curPosition);
                        if (this.model.minValue.toString().indexOf("e") > 0 || this.model.maxValue.toString().indexOf("e") > 0)
                            exp = true;
                        if (this._validateOnType && !exp) {
                            if (this.model.minValue.toString().match(new RegExp(".", "g")))
                                minVal = this.model.minValue.toString().replace(/["."]/g, decChar);
                            if (this.model.maxValue.toString().match(new RegExp(".", "g")))
                                maxVal = this.model.maxValue.toString().replace(/["."]/g, decChar);
                            if (minVal.indexOf(decChar) > 0)
                                minVal = Number(minVal.substring(0, minVal.indexOf(decChar)));
                            else
                                minVal = Number(minVal);
                            if (maxVal.indexOf(decChar) > 0)
                                maxVal = Number(maxVal.substring(0, maxVal.indexOf(decChar)));
                            else
                                maxVal = Number(maxVal);
                            if (this._validateValue(this.model.value, minVal, maxVal, "DecimalKeyPressed"))
                                this.model.value = strBeforeCursor + decChar + strAfterCursor;
                            else {
                                this._keypressFlag = false;
                                this._cancelEvent(e);
                                return false;
                            }
                        }
                        else
                            this.model.value = strBeforeCursor + decChar + strAfterCursor;
                        this._setSelectionRange(this.model.value.length, this.model.value.length);
                        this._textBox.value = this.model.value;
                    } else {
                        this.model.value = this._textBox.value;
                    }
                    this._cancelEvent(e);
                } else if (e.keyCode == 190 || e.keyCode == 110) {
                    this._cancelEvent(e);
                }
                if (e.keyCode == 109 || e.keyCode == 189 || e.keyCode == 173)//'-' char key press.
                {
                    if ((this._caretPosition() != 0 && this._getSelection(this._removeFormats(this._textBox.value)) != this.model.value) || this.model.minValue >= 0 || (this._textBox.value.toString().match(new RegExp("-", "g")) && this._getSelection(this._textBox.value) == ""))
                        this._cancelEvent(e);

                    else if (this._getSelection() == this.model.value)
                        this.model.value = null;
                }
            }
                //For watermark issue while press ctrl+z
            else if (e.keyCode != 27 && !e.ctrlKey || (e.ctrlKey && e.keyCode == 90 && $.trim(this._textBox.value) == "")) {
                this._keypressFlag = false;
                this._cancelEvent(e);
            }
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejMaskEdit
	* @requires jQuery
	* @requires ej.web.all
	* @classdesc Custom Design for Html ejMaskEdit Textbox control.
	* @example 	
	* &lt;input id="mask" type="text" /&gt; <br> 
	* &lt;script&gt; <br>
	* // Create Slider <br>
    * $('#mask').ejMaskEdit(); <br>	
	* &lt;/script&gt; <br>
	*/
    ej.widget("ejMaskEdit", "ej.MaskEdit", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        type: "editor",

        // default model
        defaults: {
            /**		
			* Specifies the mask string for the maskedit textbox control.	
			* @default null
			* @type {string}
			* @example 
			* //To set mask API value during initialization  
			* 	$("#mask").ejMaskEdit({ mask: "99-9999"});					 
			* @example 
			* //Get or set the mask API, after initialization:<br>
			*	//Gets the mask value  
			*	$("#mask").ejMaskEdit("option", "mask");<br>			
			*	//Sets the mask value 
			*	$("#mask").ejMaskEdit("option", "mask",  "99-9999" ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            mask: "",
            /**		
			* Specifies the value string for the maskedit textbox control.	
			* @default null
			* @type {string}
			* @example 
			* //To set value API  during initialization  
			* 	$("#mask").ejMaskEdit({ value: "999999"});					 
			* @example 
			* //Get or set the value API, after initialization:<br>
			*	//Gets the  value  
			*	$("#mask").ejMaskEdit("option", "value");<br>			
			*	//Sets the  value 
			*	$("#mask").ejMaskEdit("option", "value",  "999999" ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            value: "",
            /**		
			* Specifies the watermark text string for the maskedit textbox control.	
			* @default null
			* @type {string}
			* @example 
			* //To set waterMarkText API value during initialization  
			* 	$("#mask").ejMaskEdit({ waterMarkText: "Enter value"});					
			* @example 
			* //Get or set the waterMarkText API, after initialization:<br>
			*	//Gets the  waterMarkText  
			*	$("#mask").ejMaskEdit("option", "waterMarkText");<br>			
			*	//Sets the  waterMarkText 
			*	$("#mask").ejMaskEdit("option", "waterMarkText",  "Enter value" ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            waterMarkText: "",
            /**		
			* Specifies the height for the maskedit textbox control.	
			* @default 28pixel
			* @type {string}
			* @example 
			* //To set height API value during initialization  
			* 	$("#mask").ejMaskEdit({ height: "28px"});					 
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the  height  
			*	$("#mask").ejMaskEdit("option", "height");<br>			
			*	//Sets the  height 
			*	$("#mask").ejMaskEdit("option", "height",  "28px" ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            height: "",
            /**		
			* Specifies the width for the maskedit textbox control.	
			* @default 143pixel
			* @type {string}
			* @example 
			* //To set width API value during initialization  
			* 	$("#mask").ejMaskEdit({ width: 143});					 
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the  width  
			*	$("#mask").ejMaskEdit("option", "width");<br>			
			*	//Sets the  width 
			*	$("#mask").ejMaskEdit("option", "width",  143 ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            width: "",
            /**		
			* Specifies the error until correct value entered in the maskedit textbox control.	
			* @default false
			* @type {boolean}
			* @example 
			* //To set error API value during initialization  
			* 	$("#mask").ejMaskEdit({ error: true});					 
			* @example 
			* //Get or set the error API, after initialization:<br>
			*	//Gets the  error  
			*	$("#mask").ejMaskEdit("option", "error");<br>			
			*	//Sets the  error 
			*	$("#mask").ejMaskEdit("option", "error",  true ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            error: false,
            /**		
			* Specify the CSS class to maskedit textbox control to achieve custom theme.
			* @default null
			* @type {string}
			* @example 
			* //To set cssClass API value during initialization  
			* 	$("#mask").ejMaskEdit({ cssClass: "gradient-lime"});					 
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the  cssClass  
			*	$("#mask").ejMaskEdit("option", "cssClass");<br>			
			*	//Sets the  cssClass 
			*	$("#mask").ejMaskEdit("option", "cssClass",  "gradient-lime" ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            cssClass: "",
            /**		
			* Specify the custom character allowed to entered in maskedit textbox control.
			* @default null
			* @type {string}
			* @example 
			* //To set customChar API value during initialization  
			* 	$("#mask").ejMaskEdit({ customChar: "gradient-lime"});					
			* @example 
			* //Get or set the customChar API, after initialization:<br>
			*	//Gets the  customChar  
			*	$("#mask").ejMaskEdit("option", "customChar");<br>			
			*	//Sets the  customChar 
			*	$("#mask").ejMaskEdit("option", "customChar",  "gradient-lime" ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            customChar: null,
            /**		
			* Specify the inputMode for maskedit textbox control. See {@link inputMode}
			* @default ej.inputMode.Text 
			* @type {enum}
			* @example 
			* //To set inputMode API value during initialization  
			* 	$("#mask").ejMaskEdit({ inputMode: ej.inputMode.Password });					 
			* @example 
			* //Get or set the inputMode API, after initialization:<br>
			*	//Gets the  inputMode  
			*	$("#mask").ejMaskEdit("option", "inputMode");<br>			
			*	//Sets the  inputMode 
			*	$("#mask").ejMaskEdit("option", "inputMode",  ej.inputMode.Password  ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            inputMode: "text",
            /**		
			* Specify the readOnly for maskedit textbox control.
			* @default false 
			* @type {boolean}
			* @example 
			* //To set readOnly API value during initialization  
			* 	$("#mask").ejMaskEdit({ readOnly: false });					 
			* @example 
			* //Get or set the readOnly API, after initialization:<br>
			*	//Gets the  readOnly  
			*	$("#mask").ejMaskEdit("option", "readOnly");<br>			
			*	//Sets the  readOnly 
			*	$("#mask").ejMaskEdit("option", "readOnly",  true  ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            readOnly: false,
            /**		
			* Specify the text alignment for maskedit textbox control.
			* @default "left" 
			* @type {enum}
			* @example 
			* //To set textAlign API value during initialization  
			* 	$("#mask").ejMaskEdit({ textAlign: "left" });					 
			* @example 
			* //Get or set the textAlign API, after initialization:<br>
			*	//Gets the  textAlign  
			*	$("#mask").ejMaskEdit("option", "textAlign");<br>			
			*	//Sets the  textAlign 
			*	$("#mask").ejMaskEdit("option", "textAlign",  "left"  ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            textAlign: ej.textAlign.Left,
            /**		
			* Specify the hidePromptOnLeave for maskedit textbox control to hide the mask on focus out.
			* @default false 
			* @type {string}
			* @example 
			* //To set hidePromptOnLeave API value during initialization  
			* 	$("#mask").ejMaskEdit({ hidePromptOnLeave: "left" });					 
			* @example 
			* //Get or set the hidePromptOnLeave API, after initialization:<br>
			*	//Gets the  hidePromptOnLeave  
			*	$("#mask").ejMaskEdit("option", "hidePromptOnLeave");<br>			
			*	//Sets the  hidePromptOnLeave 
			*	$("#mask").ejMaskEdit("option", "hidePromptOnLeave",  "left"  ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            hidePromptOnLeave: false,
            /**		
			* Specify the rounded corner for maskedit textbox control.
			* @default false 
			* @type {boolean}
			* @example 
			* //To set roundedCorner API value during initialization  
			* 	$("#mask").ejMaskEdit({ roundedCorner: "left" });					 
			* @example 
			* //Get or set the roundedCorner API, after initialization:<br>
			*	//Gets the  roundedCorner  
			*	$("#mask").ejMaskEdit("option", "roundedCorner");<br>			
			*	//Sets the  roundedCorner 
			*	$("#mask").ejMaskEdit("option", "roundedCorner",  false  ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            roundedCorner: false,
            /**		
			* Specify the persist to mask edit textbox control to save current model value to browser cookies for state maintains.
			* @default false 
			* @type {boolean}
			* @example 
			* //To set persist API value during initialization  
			* 	$("#mask").ejMaskEdit({ persist: "left" });					 
			* @example 
			* //Get or set the persist API, after initialization:<br>
			*	//Gets the  persist  
			*	$("#mask").ejMaskEdit("option", "persist");<br>			
			*	//Sets the  persist 
			*	$("#mask").ejMaskEdit("option", "persist",  false  ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            persist: false,
            /**		
			* Specify the state of the maskedit textbox control.
			* @default true 
			* @type {boolean}
			* @example 
			* //To set enabled API value during initialization  
			* 	$("#mask").ejMaskEdit({ enabled: "left" });					 
			* @example 
			* //Get or set the enabled API, after initialization:<br>
			*	//Gets the  enabled  
			*	$("#mask").ejMaskEdit("option", "enabled");<br>			
			*	//Sets the  enabled 
			*	$("#mask").ejMaskEdit("option", "enabled",  false  ); 
			* @memberof ejMaskEdit
			* @instance
			*/
            enabled: true,
            /**     
			 * Fires when keydown in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#onKeyDown 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unStrippedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * //onKeyDown event for mask edit textbox control
             * $("#mask").ejMaskEdit({
             *    onKeyDown: function (args) {}
             * });      
			 * @memberof ejMaskEdit
			 * @instance
			 */
            onKeyDown: null,
            /**     
			 * Fires when keyup in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#keyUp 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unStrippedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * //keyUp event for mask edit textbox control
             * $("#mask").ejMaskEdit({
             *    keyUp: function (args) {}
             * });      
			 * @memberof ejMaskEdit
			 * @instance
			 */
            keyUp: null,
            /**     
			 * Fires when keypress in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#keyPress 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unStrippedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * //keyPress event for mask edit textbox control
             * $("#mask").ejMaskEdit({
             *    keyPress: function (args) {}
             * });      
			 * @memberof ejMaskEdit
			 * @instance
			 */
            keyPress: null,
            /**     
			 * Fires when value changed in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#change 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unStrippedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * //change event for mask edit textbox control
             * $("#mask").ejMaskEdit({
             *    change: function (args) {}
             * });      
			 * @memberof ejMaskEdit
			 * @instance
			 */
            change: null,
            /**     
			 * Fires when mouse over in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#mouseOver 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unStrippedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * //mouseOver event for toggle button
             * $("#mask").ejMaskEdit({
             *    mouseOver: function (args) {}
             * });      
			 * @memberof ejMaskEdit
			 * @instance
			 */
            mouseOver: null,
            /**     
			 * Fires when mouse out in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#mouseOut 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unStrippedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * //mouseOut event for toggle button
             * $("#mask").ejMaskEdit({
             *    mouseOut: function (args) {}
             * });      
			 * @memberof ejMaskEdit
			 * @instance
			 */
            mouseOut: null,
            /**     
			 * Fires when focused in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#focusIn 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unStrippedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * //focusIn event for mask edit textbox control
             * $("#mask").ejMaskEdit({
             *    focusIn: function (args) {}
             * });      
			 * @memberof ejMaskEdit
			 * @instance
			 */
            focusIn: null,
            /**     
			 * Fires when focused out in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#focusOut 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unStrippedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * //focusOut event for mask edit textbox control
             * $("#mask").ejMaskEdit({
             *    focusOut: function (args) {}
             * });      
			 * @memberof ejMaskEdit
			 * @instance
			 */
            focusOut: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            mask: "string",
            error: "boolean",
            enabled: "boolean",
            customChar: "string",
            cssClass: "string",
            waterMarkText: "string",
            roundedCorner: "boolean",
            inputMode: "enum",
            textAlign: "enum",
            hidePromptOnLeave: "boolean",
            readOnly: "boolean"
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (jsondata) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(jsondata["enabled"])) return false;
            for (var key in jsondata) {
                switch (key) {
                    case "value": this._setValue(jsondata[key]); jsondata[key] = this.get_StrippedValue(); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "waterMarkText": this.model.waterMarkText = jsondata[key]; this._setWaterMark(); break;
                    case "roundedCorner": this._roundedCorner(jsondata[key]); break;
                    case "textAlign": this._setTextAlign(jsondata[key]); break;
                    case "inputMode": this._setInputMode(jsondata[key]); break;
                    case "mask": this._setMask(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "readOnly": this._setReadOnly(jsondata[key]); break;
                    case "enabled": this.model.enabled = jsondata[key]; this._controlStatus(jsondata[key]); break;
                }
            }
        },
        observables: ["value"],
        // all events bound using this._on will be unbind automatically
        /**
         * To configure destroy of the maskedit textbox control.		
		 * @private
         */
        _destroy: function () {
            if (this.model.name) this.element.removeAttr("name");
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            if (this.model.textAlign) this.element.css("text-align", "");
            this.element.val("").removeClass(' e-mask e-input e-corner-all e-disable e-watermark').empty();
        },
        /**
         * To initialize the maskedit textbox control.		
		 * @private
         */
        // constructor function
        _init: function () {
            if (this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type'))) {
                this._setValues();
                this._renderControl();
                this._initObjects();
                this._wireEvents();
            } else {
                this._destroy();
                return false;
            }
        },
        /**
         * To enable or disable the state of the maskedit control.		
		 * @private
         */
        _controlStatus: function (value) {
            //disable status
            value != true ? this.disable() : this.enable();
        },
        /**
         * To configure value for maskedit textbox control.		
		 * @private
         */
        _setValues: function () {
            this._tempMask = null;
            this._unStrippedMask = null;
            //Predefined character definitions
            this._charMap = {
                '9': "[0-9]",
                'a': "[A-Za-z]",
                '*': "[A-Za-z0-9]",
                'A': "[A-Za-z]",
                'N': "[0-9]",
                '<': "",
                '>': "",
                'C': "",
                '?': "(.)",
                '$': "[A-Za-z ]"
            };
        },
        /**
         * Render Section For maskedit textbox control		
		 * @private
         */
        _renderControl: function () {
            this.model.name = this.model.name != null ? this.model.name : this.element[0].id;
            this.wrapper = ej.buildTag("span.e-mask e-widget " + this.model.cssClass);
            this.innerWrapper = ej.buildTag("span.e-in-wrap e-box");
            this.element.attr({ 'name': this.model.name, 'aria-invalid': false }).addClass('e-input ');
            this.wrapper.append(this.innerWrapper).insertBefore(this.element);
            this.innerWrapper.append(this.element);
            this._setInputMode(this.model.inputMode);
            this._setWidth(this.model.width);
            this._setHeight(this.model.height);
            this._roundedCorner(this.model.roundedCorner);
            this._setTextAlign(this.model.textAlign);
            this._setReadOnly(this.model.readOnly);
            this._controlStatus(this.model.enabled);
            this.previousValue = this.model.value;
        },
        /**
         * To initialize the maskedit textbox control properties.		
		 * @private
         */
        _initObjects: function () {
            this._textbox = this.element[0];
            this._keypressFlag = 0;
            this._selectedTextKeyDown = 0;
            // KeyDown trace [ 0 - None, 1 - BackSpace, 2 - Delete]
            this._keydownFlag = 0;
            // Length of the Mask
            this._maskLength = this.model.mask.length;
            //Replacing all [9?$CANa*] to '_'
            this._maskModel = this.model.mask.replace(/[9?$CANa*]/g, '_');
            this._validatedValue = this._maskModel;
            if (this.model.inputMode != "password") {
                this._setValue(this.model.value);
                // Setting WaterMark Text        
                this._setWaterMark();
            }

            //Check If Error Flag is true add the css class
            if (this.model.error) {
                this.element.addClass("e-error").attr('aria-invalid', "true");
            }
        },

        //----------------private function--------------------------------------//
        /**
		 * To configure width of the maskedit textbox control.
		 * @private
         */
        _setWidth: function (value) {
            this.wrapper.width(value);
        },
        /**
		 * To configure height of the maskedit textbox control.
		 * @private
         */
        _setHeight: function (value) {
            this.wrapper.height(value);
        },
        /**
         * To enable or disable the rounded corner behaviour. 		
		 * @private
         */
        _roundedCorner: function (value) {
            if (value) {
                this.wrapper.removeClass('e-corner-all').addClass('e-corner-all');
            }
            else if (this.wrapper.hasClass('e-corner-all')) {
                this.wrapper.removeClass('e-corner-all');
            }
        },
        /**
		 * To configure text alignment of the maskedit textbox control.
		 * @private
         */
        _setTextAlign: function (align) {
            if (align == "right") { this.element.css('padding-right', '2px'); }
            else { this.element.css('padding-right', ""); }
            this.element.css('text-align', align);
        },
        /**
		 * To configure input mode for maskedit textbox control.
		 * @private
         */
        _setInputMode: function (type) {
            this.element.attr('type', type);
        },
        /**
         * To enable or disable the readonly behaviour. 		
		 * @private
         */
        _setReadOnly: function (bool) {
            if (bool) this.element.attr("readonly", true);
            else this.element.removeAttr("readonly");
        },
        /**
         * To configure the custom theme for maskedit textbox control using cssClass property		
		 * @private
         */
        _setSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass);
            this.wrapper.addClass(skin);
        },
        /**
         * To configure the watermark text to the maskedit textbox control.
		 * @private
         */
        /* Setting the WaterMark text*/
        _setWaterMark: function () {
            this._maskModel = this.model.mask.replace(/[9?$CANa*]/g, '_');
            if (this.model.waterMarkText && (this.model.value == '' || this.model.value == this._maskModel)) {
                this._textbox.value = this.model.waterMarkText;
                this.element.addClass("e-watermark");
            } else if (this.model.value == '' && this._maskModel != '') {
                this._textbox.value = this._maskModel;
            }
        },
        /**
         * To remove the watermark text from the maskedit textbox control.
		 * @private
         */
        /* Clearing the WaterMark Text*/
        _removeWaterMark: function () {
            if (this.element.hasClass("e-watermark")) {
                this._textbox.value = this._maskModel;
                this.element.removeClass("e-watermark");
            }
        },
        /**
         * To configure alert when user press wrong key in maskedit textbox control.
		 * @private
         */
        /* Show alert when user press Wrong key*/
        _showAlert: function () {
            var ptr = this;
            this.element.addClass("e-error").attr('aria-invalid', "true").animate({ Opacity: 1 }, 700, null, function () {
                $(ptr._textbox).removeClass("e-error").attr('aria-invalid', "false");
            });
        },
        /**
         * To check whether user entered value in maskedit textbox control.
		 * @private
         */
        /* Validating User's TextBox Value*/
        _validateValue: function () {
            var tempValue = this.model.value;
            var tempModel = this._maskModel;
            var valueIndex = -1;
            // Check if Mask is Empty display the text
            if (this._maskLength == 0) {
                if (this.element.hasClass("e-watermark")) {
                    this.element.removeClass("e-watermark");
                }
                this._textbox.value = tempValue;
                return true;
            }
            else {
                tempValue = this.model.value = this.model.value.replace(/[-]/g, "");
            }

            for (var maskIndex = 0; maskIndex < this._maskLength; maskIndex++) {
                if (this._charMap[this.model.mask.charAt(maskIndex)]) {
                    valueIndex++;
                    var charCode = tempValue.charCodeAt(valueIndex);
                    if (this._validateChars(charCode, maskIndex)) {
                        var strBefore = tempModel.substring(0, maskIndex);
                        var strAfter = tempModel.substring(maskIndex);
                        tempModel = strBefore + tempValue.charAt(valueIndex) + strAfter.substr(1, strAfter.length);
                    }
                }
            }
            if (this.element.hasClass("e-watermark")) {
                if (this.model.value) {
                    this.element.removeClass("e-watermark");
                    this._textbox.value = tempModel;
                }
                else
                    this._setWaterMark();
            }
            else {
                if (this.model.value) {
                    this._textbox.value = tempModel;
                }
                else
                    this._textbox.value = this.model.mask.replace(/[9?$CANa*]/g, '_');
            }

        },
        /**
         * To clear selected text in maskedit textbox control.
		 * @private
         */
        /* If it is Selected Text opreation, This Function clears the Selected Text to MaskModel*/
        _selectionText: function (begin, end) {

            var replaceValue = this._maskModel.substring(begin, end);
            this._textbox.value = this._textbox.value.substring(0, begin) + replaceValue + this._textbox.value.substring(end);
            if (this._keydownFlag == 1) {
                this._setCaretPosition(begin);
                return begin;
            }
            else if (this._keydownFlag == 2) {
                this._setCaretPosition(end);
                return end;
            }
            return begin;
        },
        /**
         * To obtained CaretpPosition of the cursor in maskedit textbox control.
		 * @private
         */
        /* Get the CaretPsoition of the cursor, if it selected 1/more text call _selectionText*/
        _caretPosition: function (textbox) {
            //caretPos starts with 0
            var caretPos = 0;

            // Microsoft supports only IE for using document.selection
            if (document.selection) {
                //textbox.focus();
                // Create a Range of selected position
                var selectedRange = document.selection.createRange();
                var selectionLength = selectedRange.text.length;

                // Move selection start to 0 position
                selectedRange.moveStart('character', -textbox.value.length);

                // Get caret position by selection length, becoz now range is from Zero to current focus
                caretPos = selectedRange.text.length - selectionLength;

                if (selectionLength != 0) {
                    caretPos = this._selectionText(caretPos, selectedRange.text.length);
                    this._selectedTextKeyDown = 1;
                }
            }

                // For Firefox
            else if (textbox.selectionStart || textbox.selectionStart == '0') {
                caretPos = textbox.selectionStart;
                if (textbox.selectionStart != textbox.selectionEnd) {
                    if (this._keydownFlag)
                        caretPos = this._selectionText(textbox.selectionStart, textbox.selectionEnd);
                    this._selectedTextKeyDown = 1;
                }
            }

            // Return caret Position
            return (caretPos);
        },
        /**
         * To set the cursor in required position in maskedit textbox control.
		 * @private
         */
        /* Set the curosr at required position in textbox*/
        _setCaretPosition: function (caretPos) {

            //element will have MaskEdit Textbox
            var element = this._textbox;

            //For IE
            if (window.navigator.appName == "Microsoft Internet Explorer") {
                if (element.createTextRange) {
                    var range = element.createTextRange();
                    range.move('character', caretPos);
                    range.select();
                }
            }
            else
                /* For FireFox and Chrome */
                if (element.selectionStart || element.selectionStart == '0') {
                    element.focus();
                    element.setSelectionRange(caretPos, caretPos);
                }
                else
                    element.focus();

        },
        /**
         * To check whether user pressed character with mask format in maskedit textbox control.
		 * @private
         */
        /* Validate the User Pressed Char with the Mask Format*/
        _validateChars: function (keyChar, caretPos) {
            var charmap = this._charMap, match = false;
            var maskChar = this.model.mask.substr(caretPos, 1);
            var customChar = this.model.customChar;
            var actualkey = String.fromCharCode(keyChar);
            $.each(charmap, function (key, value) {
                if (maskChar == key) {
                    if (customChar != null) {
                        if (key == "C")
                            value = "[" + customChar + "]";
                        else if (key == "A" || key == "N")
                            value = value.replace(("]"), "") + customChar + "]";
                    }
                    if (actualkey.match(new RegExp(value))) match = true;
                    else match = false;
                }
            });
            return match;
        },
        /**
         * To configure the next valid position on right in maskedit textbox control.
		 * @private
         */
        /* Seek to the next valid position on right*/
        _seekNext: function () {
            var currentPos = this._caretPosition(this._textbox);
            var tempPos = currentPos;
            var seekFlag = true;
            //seeking the focus to next valid position on right of MaskEdit TextBox
            while (seekFlag) {
                if (currentPos >= 0 && currentPos < this._maskLength) {
                    if (this.model.mask.charAt(currentPos) != "C") {
                        if (!this._charMap[this.model.mask.charAt(currentPos)]) {
                            this._setCaretPosition(currentPos);
                            currentPos++;
                        }
                    }
                    if (currentPos != tempPos) {
                        tempPos = currentPos;
                        continue;
                    }
                }
                seekFlag = false;
            }
            return currentPos;
        },
        /**
         * To configure the next valid position on left in maskedit textbox control.
		 * @private
         */
        /* Seek to the next valid position on left*/
        _seekBefore: function (caretPos) {
            var currentPos = this._caretPosition(this._textbox);
            var tempPos = --currentPos;
            var seekFlag = true;

            // Stop Seek when backspace of select more than a char
            if (this._selectedTextKeyDown == 1)
                seekFlag = false;

            //seeking the focus to next valid position on left of MaskEdit TextBox
            while (seekFlag) {
                if (currentPos >= 0 && currentPos < this._maskLength) {
                    if (!this._charMap[this.model.mask.charAt(currentPos)]) {
                        this._setCaretPosition(--currentPos);
                    }
                    if (currentPos != tempPos) {
                        tempPos = currentPos;
                        continue;
                    }
                }
                seekFlag = false;
            }
            return currentPos;
        },
        /**
         * To write the user pressed character in maskedit textbox control.
		 * @private
         */
        /* Write the User Pressed Character*/
        _writeBuffer: function (key, cursorPos) {
            if (cursorPos <= this._maskLength) {
                var input = this._textbox.value;
                if (this._tempMask != null)
                    var tempkey = this._updateCasing(key, cursorPos);
                key = (tempkey == undefined) ? key : tempkey;
                var strBeforeCursor = input.substring(0, cursorPos);
                var strAfterCursor = input.substring(cursorPos);
                this._textbox.value = strBeforeCursor + key + strAfterCursor.substr(1, strAfterCursor.length);
                this._setCaretPosition(cursorPos + 1);
                //Raising ClientSide Event
                this._raiseEvents("change");
            }
        },
        /**
         * To configure the user entered character in lower or upper case character in maskedit textbox control.
		 * @private
         */
        _updateCasing: function (key, pos) {
            for (var i = 0; i < pos + 1; i++) {
                if (this._tempMask.substr(i, 1) == '<' || this._tempMask.substr(i, 1) == '>')
                    pos++;
            }
            // find case-modificator
            for (var j = pos; j > -1; j--) {
                if (this._tempMask.substr(j, 1) == '<') {
                    return key.toLowerCase();
                }
                else if (this._tempMask.substr(j, 1) == '>') {
                    return key.toUpperCase();
                }
            }
        },
        /**
         * To obtained the pure value of the textvalue, removes all the symbols in maskedit textbox control.
		 * @private
         */
        /* Return the pure value of the textvalue, removes all the symbols*/
        get_StrippedValue: function () {
            var i, value, mask = this.model.mask, stripVal = "";
            value = (this.element.hasClass("e-watermark")) ? this._maskModel : this._textbox.value;
            if (mask.length == 0) return value;

            for (i in mask) {
                var char = mask[i], exp = null;

                if ("9?$a*".indexOf(char) != -1)
                    exp = this._charMap[char];
                else if (char == "A" || char == "N")
                    exp = this._charMap[char].replace(("]"), "") + this.model.customChar + "]";
                else if (char == "C")
                    exp = "[" + this.model.customChar + "]";

                if (exp && value[i] && value[i].match(new RegExp(exp)))
                    stripVal += value[i];
            }
            return stripVal;
        },
        /**
         * To obtained the textbox value as such that, Just replace all '_' to ' '(space) in maskedit textbox control.
		 * @private
         */
        /* Return the textbox value as such that, Just replace all '_' to ' '(space)*/
        get_UnstrippedValue: function () {
            var value = (this.element.hasClass("e-watermark")) ? this._maskModel : this._textbox.value, unstripVal = "";
            if (this.model.mask.length == 0) return value;

            if (value != undefined)
                unstripVal = value.replace(/[_]/g, " ");
            return unstripVal;
        },
        /**
         * To configure the model value in maskedit textbox control.
		 * @private
         */
        _setValue: function (value) {
            if (ej.isNullOrUndefined(value)) value = "";
            this.model.value = value;
            if (!ej.isNullOrUndefined(this.model.mask) && this.model.mask != "")
                this._validateValue();
            else
                this._textbox.value = value;
            this.model.value = this.get_StrippedValue();
        },
        /**
         * To set the mask value with it;s validation in maskedit textbox control.
		 * @private
         */
        _setMask: function (maskValue) {
            this._maskLength = maskValue.length;
            this.model.mask = maskValue;
            this._maskModel = maskValue.replace(/[9?$CANa*]/g, '_');
            this._validateValue();
        },

        //----------------public function--------------------------------------//			 
        /**
        * To enable the maskedit textbox control.		
		* @return jQuery
		* @example 
		* &lt;input id="mask" type="text" /&gt; <br> 		
		* &lt;script&gt;
		* // Create MaskEdit control
		* var maskObj = $("#mask").data("ejMaskEdit");		
		* maskObj.enable(); // enable the maskedit control		
		* &lt;/script&gt;
		 @example 
		* &lt;input id="mask" type="text" /&gt; <br> 			
		* &lt;script&gt;
		* // enable the maskedit control
		* $("#mask").ejMaskEdit("enable");		
		* &lt;/script&gt;
		* @memberof ejMaskEdit
		* @instance
         */
        enable: function () {
            this.element.disabled = false;
            this.element.removeAttr("disabled").removeClass('e-disable').attr({ "aria-disabled": false });
            this.model.enabled = true;
        },
        /**
        * To disable the maskedit textbox control.		
		* @return jQuery
		* @example 
		* &lt;input id="mask" type="text" /&gt; <br> 		
		* &lt;script&gt;
		* // Create MaskEdit control
		* var maskObj = $("#mask").data("ejMaskEdit");		
		* maskObj.disable(); // disable the maskedit control		
		* &lt;/script&gt;
		 @example 
		* &lt;input id="mask" type="text" /&gt; <br> 			
		* &lt;script&gt;
		* // disable the maskedit control
		* $("#mask").ejMaskEdit("disable");		
		* &lt;/script&gt;
		* @memberof ejMaskEdit
		* @instance
         */
        disable: function () {
            this.element.disabled = true;
            this.element.attr("disabled", "disabled").addClass('e-disable').attr({ "aria-disabled": true });
            this.model.enabled = false;
        },
        /**
        * To clear the text in maskedit textbox control.		
		* @return jQuery
		* @example 
		* &lt;input id="mask" type="text" /&gt; <br> 		
		* &lt;script&gt;
		* // Create MaskEdit control
		* var maskObj = $("#mask").data("ejMaskEdit");		
		* maskObj.clear(); // clear the maskedit control		
		* &lt;/script&gt;
		 @example 
		* &lt;input id="mask" type="text" /&gt; <br> 			
		* &lt;script&gt;
		* // clear the maskedit control
		* $("#mask").ejMaskEdit("clear");		
		* &lt;/script&gt;
		* @memberof ejMaskEdit
		* @instance
         */
        clear: function () {
            if (this.model.enabled)
                this._textbox.value = this.model.mask.replace(/[9?$aCAN*]/g, '_');
        },

        /*-------------------------- Register all the events at initialize and handlers-----------------------*/
        /**
         * Wiring the events to MaskEdit textbox control		
		 * @private
         */
        _wireEvents: function () {
            this._on(this.element, 'focus', this._OnFocusHandler);
            this._on(this.element, 'blur', this._OnBlurHandler);
            this._on(this.element, 'keydown', this._OnKeyDownHandler);
            this._on(this.element, 'keypress', this._OnKeyPressHandler);
            this._on(this.element, 'keyup', this._OnKeyUpHandler);
            this._on(this.element, 'mouseover', this._OnMouseOverHandler);
            this._on(this.element, 'mouseout', this._OnMouseOutHandler);
            this._on(this.element, 'paste', this._OnPasteHandler);
            this._on(this.element, 'cut', this._OnCutHandler);
        },
        /**
        * Section For handle the selected text cut from maskedit textbox control.
        * @private
        */
        /* Cut Handler */
        _OnCutHandler: function (e) {
            var selectedValue = this._maskModel.substring(this._textbox.selectionStart, this._textbox.selectionEnd);
            var beforeSelection = this._textbox.value.substring(0, this._textbox.selectionStart);
            var afterSelection = this._textbox.value.substring(this._textbox.selectionEnd);
            var cursorPosition = this._textbox.selectionStart;
            var context = this;
            setTimeout(function () {
                context._textbox.value = beforeSelection + selectedValue + afterSelection;
                context._setCaretPosition(cursorPosition);
                context._raiseEvents("change");
            }, 0);
        },
        /**
         * Section For handle the paste text to maskedit textbox control.
		 * @private
         */
        /* Paste Handler */
        _OnPasteHandler: function (e) {
            var context = this;
            setTimeout(function () {
                var text = $(context._textbox).val();
                context._setValue(text);
                context._raiseEvents("change");
            }, 0);
            return true;
        },
        /**
         * Section For handle maskedit textbox control in focused in.
		 * @private
         */
        /* On Focus Handler */
        _OnFocusHandler: function (e) {
            this.element.addClass("e-focus");
            this._removeWaterMark();
            if (this.model.readOnly)
                return;
            if (this._textbox.value.indexOf("<") >= 0 || this._textbox.value.indexOf(">") >= 0) {
                this._textbox.value = this._textbox.value.replace(/[<>]/g, '');
                this._maskModel = this._textbox.value;
                this._tempMask = this.model.mask;
                this.model.mask = this.model.mask.replace(/[<>]/g, '');
            }

            if (this._textbox.value != this._maskModel && this._unStrippedMask != null && this.model.hidePromptOnLeave)
                this._textbox.value = this._unStrippedMask;
            $.fn.selectRange = function (start, end) {
                return this.each(function () {
                    if (this.setSelectionRange) {
                        this.focus();
                        this.setSelectionRange(start, end);
                    } else if (this.createTextRange) {
                        var range = this.createTextRange();
                        range.collapse(true);
                        range.moveEnd('character', end);
                        range.moveStart('character', start);
                        range.select();
                    }
                });
            };
            if (this.model.mask) {
                this.element.selectRange(0, 0);
            }
            //Raising ClientSide Event
            this._raiseEvents("focusIn");
        },
        /**
         * Section For handle the maskedit textbox control is focused out.
		 * @private
         */
        _OnBlurHandler: function (e) {
            this.element.removeClass("e-focus");
            this.model.value = this.get_StrippedValue();
            if (this._textbox.value != undefined && this.model.hidePromptOnLeave) {
                this._unStrippedMask = this._textbox.value;
                this._textbox.value = this.get_UnstrippedValue();
            }
            if (this.model.inputMode != "password")
                this._setWaterMark();

            //Raising ClientSide Event
            this._raiseEvents("focusOut");
        },
        /**
         * Section For handle the keypress down event in maskedit textbox control.
		 * @private
         */
        /* On Key Down Handler */
        _OnKeyDownHandler: function (e) {
            if (this.model.readOnly) return;
            this._keypressFlag = 0;
            this._raiseEvents("onKeyDown");
            // Remove the Error Flag
            this.element.removeClass("error");

            if (this._maskLength == 0)
                return true;

            var unicode = e.keyCode ? e.keyCode : e.charCode;
            var actualkey = String.fromCharCode(unicode);

            if (unicode >= 35 && unicode <= 41) {
                if (window.navigator.appCodeName == "Mozilla" || window.navigator.appCodeName == "opera") {
                    this._keypressFlag = 1;
                }
            }
            if (e.shiftKey && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || e.keyCode == 46 || e.keyCode == 127) || (e.ctrlKey && (e.keyCode == 86 || e.keyCode == 65 || e.keyCode == 67 || e.keyCode == 88))) {
                this._keypressFlag = 1;
                return;
            }
            else if (unicode == 8) { // For Backspace Key Event

                if (this.model.inputMode != "password") {
                    this._keydownFlag = 1;
                    // Seek to next valid position
                    var currentPos = this._seekBefore();
                    this._raiseEvents("change");
                    if (!this._selectedTextKeyDown) {
                        if (currentPos >= 0 && currentPos < this._maskLength)
                            this._writeBuffer("_", currentPos);
                        this._setCaretPosition(currentPos);
                    }
                    this._keydownFlag = 0;
                    this._selectedTextKeyDown = 0;
                    e.preventDefault();
                    return false;
                }
                this._keypressFlag = 1;
                return true;
            }
            else if (e.keyCode == 46 || e.keyCode == 127) { //  For Delete Key Events
                if (this.model.inputMode != "password") {
                    this._keydownFlag = 2;
                    // Seek to next valid position
                    currentPos = this._seekNext();
                    this._raiseEvents("change");
                    if (!this._selectedTextKeyDown) {
                        if (currentPos >= 0 && currentPos < this._maskLength)
                            this._writeBuffer("_", currentPos);
                    }
                    this._keydownFlag = 0;
                    this._selectedTextKeyDown = 0;
                    e.preventDefault();
                    return false;
                }
                this._keypressFlag = 1;
                return true;
            }
        },
        /**
         * Section For handle the keypress up event in maskedit textbox control.
		 * @private
         */
        /* On Key Up Handler */
        _OnKeyPressHandler: function (e) {
            if (this.model.readOnly) return;
            this._raiseEvents("keyPress");
            if (this._maskLength == 0)
                return true;

            var unicode = e.keyCode ? e.keyCode : e.charCode;
            var actualkey = String.fromCharCode(unicode);
            var currentPos = this._seekNext();
            var val = ej.browserInfo().name;
            if (this._validateChars(unicode, currentPos)) {
                if (((val == "mozilla") || (val == "opera")) && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39)) {
                }
                else if (((val == "mozilla") || (val == "opera")) && (e.ctrlKey && (unicode == 97 || unicode == 99 || unicode == 118 || unicode == 120))) {
                }
                else
                    this._writeBuffer(actualkey, currentPos);
            }
            else
                if (((val == "mozilla") || (val == "opera")) && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39)) {
                }
                else if (((val == "mozilla") || (val == "opera")) && (e.ctrlKey && (unicode == 97 || unicode == 99 || unicode == 118 || unicode == 120))) {
                }
                else
                    this._showAlert();

            // To stop default press actions
            if (!this._keypressFlag && unicode != 9) {//Shift + Tab Navigation
                this._keypressFlag = 0;
                e.preventDefault();
                return false;
            }
            this._keypressFlag = 0;

        },
        /**
         * Section For raise the key up event in maskedit textbox control.
		 * @private
         */
        _OnKeyUpHandler: function (e) {
            if (this._maskLength == 0) this._raiseEvents("change");
            this._raiseEvents("keyUp");
        },
        /**
         * Section For raise the mouse over event in maskedit textbox control.
		 * @private
         */
        /* On Mouse Over Handler */
        _OnMouseOverHandler: function (e) {
            // Raising ClideSide Event
            this._raiseEvents("mouseOver");
        },
        /**
         * Section For raise the mouse out event in maskedit textbox control.
		 * @private
         */
        /* On Mouse Out Handler */
        _OnMouseOutHandler: function (e) {
            // Raising ClideSide Event
            this._raiseEvents("mouseOut");
        },
        /**
         * Section For raise the corresponding event in maskedit textbox control.
		 * @private
         */
        //To raise Corresponding Client Side Events
        _raiseEvents: function (eventName) {
            var eventArgs, strippedVal = this.get_StrippedValue();
            this.model.value = strippedVal;
            if (eventName == "change") {
                if (this.previousValue != this.model.value) {
                    this.previousValue = this.model.value;
                }
                else
                    return false;
            }

            if (this.model[eventName]) {
                eventArgs = { value: strippedVal, unStrippedValue: this.get_UnstrippedValue() };
                this._trigger(eventName, eventArgs);
            }
        },
        /**
         * Section For set the value can be set after changes happened in maskedit textbox control.
		 * @private
         */
        _OnValueChange: function () {
            if (this._textbox.value == "" && this._maskModel != "")
                this._textbox.value = this._maskModel;
            this._setValue(this._textbox.value);
        }

    });
    /**
	 * Enum for slider type.	 
	 * @enum {string}
	 * @global 
	 */
    ej.inputMode = {
        /**  support for user enter character in password format. */
        Password: "password",
        /**  support for user enter character in normal format. */
        Text: "text"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Menu control.
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
       * @namespace ej
       * @class ejMenu
       * @requires jQuery
       * @requires ej.web.all
       * @classdesc Custom Design for Menu control.
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
       * // Create Menu
       * $("#menu").ejMenu(); 	
       * &lt;/script&gt;
       */

    ej.widget("ejMenu", "ej.Menu", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["ul"],
        _setFirst: false,
        _rootCss: "e-menu",

        // default model
        defaults: {
            /**		
                * Specifies the height of the root menu.
                * @default "auto"
                * @type {string | number}
                * @example 
                * //To set height API value during initialization  
                * 	$("#menu").ejMenu({ height: 22 });					* 
                * @example 
                * //Get or set the height API, after initialization:<br>
                *	//Gets the height value
                *	$("#menu").ejMenu("option", "height");			
                *	//Sets the height value
                *	$("#menu").ejMenu("option", "height", 22 ); 
                 * @memberof ejMenu
                * @instance
                */
            height: "",
            /**		
			* Specifies the width of the main menu.
			* @default auto
			* @type {string | number}
			* @example 
			* //To set width API value during initialization  
			* 	//To set width API value 
			* 	$("#menu").ejMenu({ width: 200 });	
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value
			*	$("#menu").ejMenu("option", "width");		
			*	//Sets the width value
			*	$("#menu").ejMenu("option", "width", 200 ); 
			 * @memberof ejMenu
			* @instance
			*/
            width: "",
            /**		
			* To enable or disable the Animation while hover or click an menu items.
			* @default ej.animation.Default
			* @type {enum}
			* @example 
			* //To set animation API value during initialization  
			* 	//To set aniamtion API value 
			* 	$("#menu").ejMenu({ animation: ej.animation.Default });	
			* @example 
			* //Get or set the animation API, after initialization:<br>
			*	//Gets the animation value
			*	$("#menu").ejMenu("option", "animation");		
			*	//Sets the animation value
			*	$("#menu").ejMenu("option", "animation", ej.animation.None ); 
			 * @memberof ejMenu
			* @instance
			*/
            animation: "default",
            /**		
			* Specifies the orientation of normal menu. Normal menu can rendered in horizontal or vertical direction by using this API.
			* @default ej.orientation.Horizontal
			* @type {string | enum}
			* @example 
			* //To set orientation API value during initialization  
			* 	//To set orientation API value 
			* 	$("#menu").ejMenu({ orientation: ej.orientation.Horizontal });
			* @example 
			*  //Get or set the orientation API, after initialization:<br>
			*	//Gets the orientation value
			*	$("#menu").ejMenu(option","orientation"); 		
			*	//Sets the orientation value
			*	$("#menu").ejMenu("option","orientation",ej.orientation.Vertical ); 
			 * @memberof ejMenu
			* @instance
			*/
            orientation: ej.orientation.Horizontal,
            /**		
			* Specifies the type of the menu. Essential JavaScript Menu consists of two type of menu, they are Normal Menu and Context Menu both Normal and Context Menu mode.
			* @default ej.menuType.NormalMenu 
			* @type {string | enum}
			* @example 
			* //To set menuType API value during initialization  
			* 	//To set menuType API value 
			* 	$("#menu").ejMenu({ menuType: "NormalMenu" });
			* @example 
			*  //Get or set the menuType API, after initialization:<br>
			*	//Gets the menuType value
			*	$("#menu").ejMenu(option","menuType"); 		
			*	//Sets the menuType value
			*	$("#menu").ejMenu("option","menuType",ej.menuType.ContextMenu);  
			 * @memberof ejMenu
			* @instance
			*/
            menuType: "NormalMenu",
            /**		
			*Specifies the target id of context menu. On right clicking the specified contextTargetId element, context menu gets shown.
			* @default null 
			* @type {string}
			* @example
			*	&lt;div id="target" class="textarea"&gt;
			*                            HTML is written in the form of HTML elements consisting of tags enclosed in angle
			*                            brackets (like
			*                            &lt;html&gt;
			*                            ),within the web page content. HTML tags most commonly come in pairs like and ,although
			*                            some tags, known as empty elements, are unpaired, for example
			*                            &lt;img&gt;. The purpose of a web browser is to read HTML documents and compose them into
			*                            visible or audible web pages. The browser does not display the HTML tags, but uses
			*                            the tags to interpret the content of the page.
			*                        &lt;/div&gt;
			*                        &lt;ul id="contextMenu"&gt;
			*                            &lt;li&gt;&lt;a&gt;Cut&lt;/a&gt;&lt;/li&gt; 
			*                            &lt;li&gt;&lt;a&gt;Copy&lt;/a&gt;&lt;/li&gt; 
			*                            &lt;li&gt;&lt;a&gt;/span&gt;Paste&lt;/a&gt;&lt;/li&gt; 
			*                            &lt;li class="separator"&gt;&lt;/li&gt; 
			*                            &lt;li&gt;&lt;a&gt;Comments&lt;/a&gt;&lt;/li&gt;
			*                            &lt;li&gt;&lt;a&gt;Links&lt;/a&gt;&lt;/li&gt;
			*                            &lt;li&gt;&lt;a&gt;Clear Formatting&lt;/a&gt;&lt;/li&gt;  
			*                        &lt;/ul&gt;           
			*                &lt;/div&gt;
			* @example 
			* //To set contextTargetId API value during initialization  
			* 	//To set contextTargetId API value 
			* 	$("#contextMenu").ejMenu({ contextTargetId:"target"});  
			 * @memberof ejMenu
			* @instance
			*/
            contextTargetId: null,
            /**		
			* Specify the CSS class to Menu to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the CSS class during initialization. 			
			* 	$("#menu").ejMenu({  cssClass : "gradient-lime" });			* 
			* @example 
			* //Get or set the CSS class after initialization:<br>
			*	// Gets the CSS class value.		
			*	$("#menu").ejMenu("option", "cssClass");			
			*	// Sets the CSS class to Menu
			*	$("#menu").ejMenu("option", "cssClass",  "gradient-lime" );
			 * @memberof ejMenu
			* @instance
			*/
            cssClass: "",
            /**		
			* Specifies the sub menu items to be show or open only on click.
			* @default false
			* @type {boolean}
			* @example 
			* //To set openOnClick API value during initialization  
			* 	//To set openOnClick API value 
			* 	$("#menu").ejMenu({ openOnClick: true });
			* @example 
			*  //Get or set the openOnClick API, after initialization:<br>
			*	//Gets the openOnClick value
			*	$("#menu").ejMenu(option","openOnClick"); 		
			*	//Sets the openOnClick value
			*	$("#menu").ejMenu("option","openOnClick",false );  
			 * @memberof ejMenu
			* @instance
			*/
            openOnClick: false,
            /**		
			* Specifies the sub menu popup direction.
			* @default ej.Direction.Right
			* @type {string | enum}
			* @example 
			* //To set subMenuDirection API value during initialization  
			* 	//To set subMenuDirection API value 
			* 	$("#menu").ejMenu({ subMenuDirection: ej.Direction.Right });
			* @example 
			*  //Get or set the subMenuDirection API, after initialization:<br>
			*	//Gets the subMenuDirection value
			*	$("#menu").ejMenu(option","subMenuDirection"); 		
			*	//Sets the subMenuDirection value
			*	$("#menu").ejMenu("option","subMenuDirection",ej.Direction.Left );  
			 * @memberof ejMenu
			* @instance
			*/
            subMenuDirection: "Right",
            /**		
			* Specifies the root menu items to be aligned center in horizontal menu.
			* @default false
			* @type {boolean}
			* @example 
			* //To set centerAlign API value during initialization  
			* 	//To set centerAlign API value 
			* 	$("#menu").ejMenu({ centerAlign: true });
			* @example 
			*  //Get or set the centerAlign API, after initialization:<br>
			*	//Gets the centerAlign value
			*	$("#menu").ejMenu(option","centerAlign"); 		
			*	//Sets the centerAlign value
			*	$("#menu").ejMenu("option","centerAlign",false );  
			 * @memberof ejMenu
			* @instance
			*/
            centerAlign: false,
            /**		
			* Specifies the main menu items arrows only to be shown if it contains child items.
			* @default true
			* @type {boolean}
			* @example 
			* //To set showTopLevelArrows API value during initialization  
			* 	//To set showTopLevelArrows API value 
			* 	$("#menu").ejMenu({ showTopLevelArrows: true });
			* @example 
			*  //Get or set the showTopLevelArrows API, after initialization:<br>
			*	//Gets the showTopLevelArrows value
			*	$("#menu").ejMenu(option","showTopLevelArrows"); 		
			*	//Sets the showTopLevelArrows value
			*	$("#menu").ejMenu("option","showTopLevelArrows",false );  
			 * @memberof ejMenu
			* @instance
			*/
            showTopLevelArrows: true,
            /**		
			* Specifies the sub menu items arrows only to be shown if it contains child items.
			* @default true
			* @type {boolean}
			* @example 
			* //To set showBottomLevelArrows API value during initialization  
			* 	//To set showBottomLevelArrows API value 
			* 	$("#menu").ejMenu({ showBottomLevelArrows: true });
			* @example 
			*  //Get or set the showBottomLevelArrows API, after initialization:<br>
			*	//Gets the showBottomLevelArrows value
			*	$("#menu").ejMenu(option","showBottomLevelArrows"); 		
			*	//Sets the showBottomLevelArrows value
			*	$("#menu").ejMenu("option","showBottomLevelArrows",false );  
			 * @memberof ejMenu
			* @instance
			*/
            showBottomLevelArrows: true,
            /**		
			* When this property sets to false, the menu list is displayed without any separators.
			* @default true
			* @type {boolean}
			* @example 
			* //To set enableSeparator API value during initialization  
			* 	//To set enableSeparator API value 
			* 	$("#menu").ejMenu({ enableSeparator: true });
			* @example 
			*  //Get or set the enableSeparator API, after initialization:<br>
			*	//Gets the enableSeparator value
			*	$("#menu").ejMenu(option","enableSeparator"); 		
			*	//Sets the enableSeparator value
			*	$("#menu").ejMenu("option","enableSeparator",false );  
			 * @memberof ejMenu
			* @instance
			*/
            enableSeparator: true,
            /**		
			* Enable / Disable the Menu control.
			* @default true
			* @type {boolean}
			* @example 
			* //To set enabled API value during initialization  
			* 	//To set enabled API value 
			* 	$("#menu").ejMenu({ enabled: true });
			* @example 
			*  //Get or set the enabled API, after initialization:<br>
			*	//Gets the enabled value
			*	$("#menu").ejMenu(option","enabled"); 		
			*	//Sets the enabled value
			*	$("#menu").ejMenu("option","enabled",false );  
			 * @memberof ejMenu
			* @instance
			*/
            enabled: true,
            /**		
			* Fields used to bind the data source and it includes following field members to make databind easier.
			* @default null
			* @type {Object}
			* @example 
			* //To set fields API value during initialization  
			* 	//To set fields API value 
			* 	      $("#menu").ejMenu({
            *                fields: {
            *                   dataSource: dataManger, query: query, id: "CategoryID", text: "CategoryName"
            *               }
            *           });
			* @example 
			* //Get the data from two different tables through ODATA service.  
			*             var dataManger = ej.DataManager({
            *   url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"  });
            *   var query = ej.Query().from("Categories").select("CategoryID,CategoryName").take(3);
            *      $("#menu").ejMenu({
            *             fields: {
            *                   dataSource: dataManger, query: query, id: "CategoryID", text: "CategoryName",
            *                    child: { dataSource: dataManger, tableName: "Products", id: "ProductID", parentId: "CategoryID", text: "ProductName" }
            *                }
            *            });
            * //JSON Databind.
            * @example 			
			* var data = [
            * { id: 1, text: "Group A", parentId: null },
            * { id: 2, text: "Group B", parentId: null },
            * { id: 3, text: "Group C", parentId: null },
            * { id: 4, text: "Group D", parentId: null },
            * { id: 5, text: "Group E", parentId: null },
            * //first level child
            * { id: 11, parentId: 1, text: "Algeria", sprite: "flag-dz" },
            * { id: 12, parentId: 1, text: "Armenia", sprite: "flag-am" },
            * { id: 13, parentId: 1, text: "Bangladesh", sprite: "flag-bd" },
            * { id: 14, parentId: 1, text: "Cuba", sprite: "flag-cu" }
            * ];
            * $(function () {
            * $("#menujson").ejMenu({                
            *     fields: { dataSource: data, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }
            * });        
			 * @memberof ejMenu
			* @instance
			*/
            fields: {
                /**		
                * datasource receives  Essential DataManager object and JSON object. 
                * @default null
                * @type {Object} 
                 * @memberof ejMenu
                * @instance
                */
                dataSource: null,
                /**		
			* It receives query to retrieve data from the table (query is same as SQL).  
			* @default null
			* @type {Object} 
			* @example 		
			*	ej.Query().from("Categories").select("CategoryID,CategoryName").take(3);
			 * @memberof ejMenu
			* @instance
			*/
                query: null,
                /**	
                * It receives table name to execute query on the corresponding table  
                * @default null
                * @type {string} 
                 * @memberof ejMenu
                * @instance
                */
                tableName: null,
                /**	
                * Specifies the id to menu items list
                * @default null
                * @type {string} 
                 * @memberof ejMenu
                * @instance
                */
                id: null,
                /**	
                * Specifies the parent id of the table.
                * @default null
                * @type {string} 
                 * @memberof ejMenu
                * @instance
                */
                parentId: null,
                /**	
               * Specifies the text of menu items list.
               * @default null
               * @type {string} 
                * @memberof ejMenu
               * @instance
               */
                text: null,
                /**	
                * Specifies the sprite CSS class to “li” item list
                * @default null
                * @type {string} 
                 * @memberof ejMenu
                * @instance
                */
                spriteCssClass: null,
                /**	
                * Specifies the link attribute to “a” tag in item list.
                * @default null
                * @type {string} 
                 * @memberof ejMenu
                * @instance
                */
                linkAttribute: null,
                /**	
                * Specifies the image attribute to “img” tag inside items list. 
                * @default null
                * @type {string} 
                 * @memberof ejMenu
                * @instance
                */
                imageAttribute: null,
                /**	
                * Specifies the html attributes to “li” item list.
                * @default null
                * @type {string} 
                 * @memberof ejMenu
                * @instance
                */
                htmlAttribute: null,
                /**	
                * Specifies the image URL to “img” tag inside item list.
                * @default null
                * @type {string} 
                 * @memberof ejMenu
                * @instance
                */
                imageUrl: null,
            },
            /**		
			* Specifies the menu items to be displayed in right to left direction.
			* @default false
			* @type {boolean}
			* @example 
			* //To set rtl API value during initialization  
			* 	//To set rtl API value 
			* 	$("#menu").ejMenu({ rtl: true });
			* @example 
			*  //Get or set the rtl API, after initialization:<br>
			*	//Gets the rtl value
			*	$("#menu").ejMenu(option","rtl"); 		
			*	//Sets the rtl value
			*	$("#menu").ejMenu("option","rtl",false );  
			 * @memberof ejMenu
			* @instance
			*/
            rtl: false,
            /**		
			* Specifies the title to responsive menu.
			* @default "Menu"
			* @type {string}
			* @example 
			* //To set resTitle API value during initialization  
			* 	//To set resTitle API value 
			* 	$("#menu").ejMenu({ resTitle: true });
			* @example 
			*  //Get or set the resTitle API, after initialization:<br>
			*	//Gets the resTitle value
			*	$("#menu").ejMenu(option","resTitle"); 		
			*	//Sets the resTitle value
			*	$("#menu").ejMenu("option","resTitle",false );  
			 * @memberof ejMenu
			* @instance
			*/
            resTitle: "Menu",
            /**     
            * Fires before context menu gets open.
            * @event
            * @name ejMenu#beforeContextOpen 	
            * @param {Object} argument Event parameters from context menu     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {object}  argument.target returns the target element
            * @example 
            *  //before context menu open event for menu
            *  $("#menu").ejMenu({
            *  beforeContextOpen: function (args) {}
            *  });   
            * @memberof ejMenu
            * @instance
            */
            beforeContextOpen: null,
            /**     
            * Fires when context menu on open.
            * @event
            * @name ejMenu#contextOpen 	
            * @param {Object} argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {object}  argument.target returns the target element
            * @example 
            *  //context menu open event for menu
            *  $("#menu").ejMenu({
            *  contextOpen: function (args) {}
            *  });   
            * @memberof ejMenu
            * @instance
            */
            contextOpen: null,
            /**     
			 * Fire when context menu on close.
			 * @event
			 * @name ejMenu#contextClose 	
			 * @param {Object} argument Event parameters from menu     
			 * @param {object}  argument.model returns the menu model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object}  argument.target returns the target element
			 * @example 
			 *  //context menu close event for menu
			 *  $("#menu").ejMenu({
             *  contextClose: function (args) {}
			 *  });   
			 * @memberof ejMenu
			 * @instance
			 */
            contextClose: null,
            /**     
			 * Fires when mouse over the Menu items.
			 * @event
			 * @name ejMenu#mouseOver 	
			 * @param {Object}  argument Event parameters from menu     
			 * @param {object}  argument.model returns the menu model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.menuId returns menu id
			 * @param {string}  argument.menuText returns clicked menu item text
             * @param {object}  argument.element returns clicked menu item element
			 * @param {object}  argument.event returns the event
			 * @example 
			 *   //mouse over event for menu
			 *  $("#menu").ejMenu({
             *  mouseOver: function (args) {}
			 *  });   
			 * @memberof ejMenu
			 * @instance
			 */
            mouseOver: null,
            /**     
            * Fires when mouse out from menu items.
            * @event
            * @name ejMenu#mouseOut 	
            * @param {Object}  argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.menuId returns menu id
            * @param {string}  argument.menuText returns clicked menu item text
            * @param {object}  argument.element returns clicked menu item element
			* @param {object}  argument.event returns the event
            * @example 
            *  //mouse out event for menu
            *  $("#menu").ejMenu({
            *  mouseOut: function (args) {}
            *  });   
            * @memberof ejMenu
            * @instance
            */
            mouseOut: null,
            /**     
            * Fires when mouse click on menu items.
            * @event
            * @name ejMenu#click 	
            * @param {Object}  argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.menuId returns menu id
            * @param {string}  argument.menuText returns clicked menu item text
            * @param {object}  argument.element returns clicked menu item element
			* @param {object}  argument.event returns the event
            * @param {number}  argument.selectedItem returns the selected item
            * @example 
            *  //click event for menu
            *  $("#menu").ejMenu({
            *  click: function (args){}
            *  });   
            * @memberof ejMenu
            * @instance
            */
            click: null,
            /**     
            * Fires when key down on menu items.
            * @event
            * @name ejMenu#keyDown 	
            * @param {Object}  argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.menuId returns menu id
            * @param {string}  argument.menuText returns clicked menu item text
            * @param {object}  argument.element returns clicked menu item element
            * @param {object}  argument.event returns the event
            * @example 
            *  //keyDown event for menu
            *  $("#menu").ejMenu({
            *  keyDown: function (args){}
            *  });   
            * @memberof ejMenu
            * @instance
            */
            keyDown: null
        },
        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            animation: "enum",
            cssClass: "string",
            resTitle: "string",
            openOnClick: "boolean",
            enabled: "boolean",
            centerAlign: "boolean",
            showArrow: "boolean",
            showTopLevelArrows: "boolean",
            showBottomLevelArrows: "boolean",
            enableSeparator: "boolean",
            rtl: "boolean"
        },

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (jsondata) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(jsondata["enabled"])) return false;
            for (var key in jsondata) {
                switch (key) {
                    case "menuType":
                        this.model.menuType = jsondata[key];
                        this.element.removeClass("e-menu" + this.model.cssClass);
                        this._renderControl();
                        if (jsondata[key] == "ContextMenu") this._contextMenuEvents();
                        break;
                    case "fields":
                        this.element.empty().insertBefore(this.wrapper);
                        this.wrapper.remove();
                        $.extend(this.model.fields, jsondata[key]);
                        this._intializeData();
                        break;
                    case "orientation": this._setOrientation(jsondata[key]); break;
                    case "showTopLevelArrow": this._addArrow(jsondata[key], this.model.showBottomLevelArrow); break;
                    case "showBottomLevelArrow": this._addArrow(this.model.showTopLevelArrows, jsondata[key]); break;
                    case "enableSeparator": this._setSeparator(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "rtl": this._setRTL(jsondata[key]); break;
                    case "centerAlign": this._centerAlign(jsondata[key]); break;
                    case "enabled": this.model.enabled = jsondata[key]; this._controlStatus(jsondata[key]); break;
                    case "animation":
                        this._setAnimation(jsondata[key]);
                        break;
                }
            }
        },
        /**
       * destroy the menu widget
       * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
       * @alias destroy
       * @return jQuery
       * @example 
       * var menuObj = $("#menu").data("ejMenu");
       * menuObj.destroy(); // destroy the menu
       * @example 
       * // Destroy the menu
       * $("#menu").ejMenu("destroy");	
       * @memberof ejMenu
       * @instance
        */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            /// <summary>This function is  used to destroy the Menu Object</summary>
            this._cloneElement.insertBefore(this.element);
            this._cloneElement.removeClass('e-menu');
            this.element.remove();
        },
        /**
       * Create the Menu widget
       * @private
       */
        // constructor function
        _init: function () {
            /// <summary>This function is  used to Initialize the Menu Object</summary>
            this.element.css("visibility", "hidden");
            this._cloneElement = this.element.clone();
            this._setValues();
            this._intializeData();
            this.element.css("visibility", "visible");
        },

        //-------------------------Private Methods---------------------------------------------    
        /**
       * To configure the property values
       * @private
       */
        _setValues: function () {
            this._mouseOver = true;
            this._hoverOpen = true;
            this._hoverClose = true;
            this._isMenuOpen = false;
            this._hideSpeed = 100;
            this._showSpeed = 100;
            this._isSubMenuOpen = false;
            this._isContextMenuOpen = false;
            this._disabledMenuItems = new Array();
            this._delayMenuHover = 0;
            this._delaySubMenuHover = 0;
            this._delaySubMenuShow = 0;
            this._preventContextOpen = true;
            this._setAnimation(this.model.animation);
        },
        _intializeData: function () {
            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) {
                this._generateTemplate(this.model.fields["dataSource"]);
                this._renderMenu();
            }
            else {
                this._renderMenu();
                this._wireEvents();
            }
        },
        _renderMenu: function () {
            this._renderControl();
            this._addArrow(this.model.showTopLevelArrows, this.model.showBottomLevelArrows);
            this._intializeMenu();
        },
        /**
        * Render Section For Different Types Menu		
        * @private
        */
        _renderControl: function () {
            /// <summary>This function is  used to Render the Menu Object</summary>
            var label, checkBox, checkObj, list, spanlist, i;
            this.wrapper = ej.buildTag("div.e-menu-wrap");
            if ((this.model.menuType != ej.menuType.ContextMenu) && (this.model.orientation != "vertical")) {
                this.resWrap = ej.buildTag('span.e-menu-res-wrap');
                this.inResWrap = ej.buildTag('span.e-in-wrap e-box e-menu-res-in-wrap');
                this.label = ej.buildTag('span.e-res-title', this.model.resTitle);
                this.check = ej.buildTag('span.e-check-wrap e-icon');
                this.checkBox = ej.buildTag('input#responsive.e-res-icon', '', {}, { type: 'checkbox' });
                this.wrapper.append(this.resWrap)
                this.resWrap.append(this.inResWrap);
                this.inResWrap.append(this.label).append(this.check);
                this.check.append(this.checkBox);
            }
            if (this.model.menuType != ej.menuType.ContextMenu) {
                this.wrapper.insertBefore(this.element);
                this.wrapper.append(this.element);
                if (this.checkBox != null) {
                    this.checkBox.ejCheckBox({ size: "medium", change: this._oncheck });

                }
            }
            this.element.addClass("e-menu e-widget").attr({ "role": "menu", "tabindex": 0 });
            if (this.model.rtl) this._setRTL(this.model.rtl);
            if (this.model.menuType == "NormalMenu") {
                this.model.orientation == "horizontal" ? this.element.addClass(this.model.cssClass + "e-horizontal") : this.element.addClass(this.model.cssClass + "e-vertical");
            }
                //For ContextMenu Mode
            else this._contextMenu_Template();
            //Adding arrows to items with sub items
            this.element.find('li:has("> ul")').find('> a,> span').addClass('aschild');
            this.element.find('>li').addClass('e-list').attr({ "role": "menuitem" });
            list = this.element.find('.e-list a.aschild');
            spanlist = this.element.find('.e-list span.aschild');
            for (i = 0; i < list.length; i++) {
                $(list[i]).siblings().attr({ "aria-hidden": true });
                $(list[i]).parent().attr({ "aria-haspopup": true, "role": "menu" });
                $(list[i]).siblings('ul').children('li').addClass('e-list').attr("role", "menuitem");
            }
            for (i = 0; i < spanlist.length; i++) {
                $(spanlist[i]).siblings().attr({ "aria-hidden": true });
                $(spanlist[i]).parent().attr({ "aria-haspopup": true, "role": "menu" });
                $(spanlist[i]).siblings('ul').children('li').addClass('e-list').attr("role", "menuitem");
            }
            if (this.model.centerAlign) this._centerAlign(this.model.centerAlign);
            if (this.model.enableSeparator) this._setSeparator(true);
            this._controlStatus(this.model.enabled);
        },
        /**
        * Wiring the Events to Responsive Icon 	
        * @private
        */
        _oncheck: function (e) {
            var obj = this.element.parents('.e-menu-wrap').children('.e-menu');
            e.isChecked ? obj.removeClass('e-res-hide').addClass('e-res-show') : obj.removeClass('e-res-show').addClass('e-res-hide');
        },
        /**
        * Render Section For Menu through Remote Data and DataSource	
        * @private
        */
        _generateTemplate: function (data) {
            var proxy = this, queryPromise;
            if (data instanceof ej.DataManager) {
                queryPromise = data.executeQuery(this._columnToSelect(this.model.fields));
                queryPromise.done(function (e) {
                    proxy._generateItemTemplate(e.result);
                    proxy._wireEvents();
                });
            } else {
                this._generateItemTemplate(proxy.model.fields['dataSource']);
                this._wireEvents();
            }
        },
        /**
        * Render Section For Menu through DataSource		
        * @private
        */
        _generateItemTemplate: function (items) {
            for (var i = 0; i < items.length; i++) {
                if (items[i][this.model.fields.parentId] == null) {
                    var subItem = this._menuTemplate(items[i], items, this.model.fields);
                    this.element.append(subItem);
                }
            }
        },
        /**
      * Render Section For Menu template		
      * @private
      */
        _menuTemplate: function (item, tableData, mapper) {
            var liTag, aTag, imgTag, spanTag;
            liTag = $(document.createElement('li'));
            liTag.attr("class", 'e-list');
            if (item[mapper.htmlAttribute]) this._setAttributes(item[mapper.htmlAttribute], liTag);

            if (item[mapper.text] && item[mapper.text] != "") {
                aTag = $(document.createElement('a'));
                if (item[mapper.imageUrl] && item[mapper.imageUrl] != "") {
                    imgTag = $(document.createElement('img'));
                    imgTag.attr('src', item[mapper.imageUrl]);
                    if (item[mapper.imageAttribute]) this._setAttributes(item[mapper.imageAttribute], imgTag);
                    aTag.append(imgTag);
                }
                else if (item[mapper.spriteCssClass] && item[mapper.spriteCssClass] != "") {
                    spanTag = $(document.createElement('span'));
                    spanTag.addClass(item[mapper.spriteCssClass]);
                    aTag.append(spanTag);
                }
                aTag.append(item[mapper.text]);
                if (item[mapper.linkAttribute])
                    aTag.attr('href', item[mapper.linkAttribute]);
                else aTag.attr('href', "#");
                liTag.append(aTag);
            }
            if (item[mapper.id]) {
                liTag.prop("id", item[mapper.id]);
            }
            if (!ej.isNullOrUndefined(mapper["child"])) {
                this._odataFlag = true;
                if (mapper["child"]["dataSource"] instanceof ej.DataManager) {
                    var proxy = this, queryManager = ej.Query();
                    queryManager = this._columnToSelect(mapper["child"]);
                    queryManager.where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]);
                    var queryPromise = mapper["child"]["dataSource"].executeQuery(queryManager);
                    queryPromise.done(function (e) {
                        var childItems = e.result;
                        if (childItems && childItems.length > 0) {
                            var ul = $(document.createElement('ul'));
                            for (var i = 0; i < childItems.length; i++) {
                                var liItem = proxy._menuTemplate(childItems[i], mapper["child"]["dataSource"], mapper["child"]);
                                ul.append(liItem);
                            }
                            liTag.append(ul);
                            $(liTag).children('a').addClass('aschild');
                            if ($(liTag).parent().hasClass('e-menu') && (proxy.model.showTopLevelArrows))
                                $(liTag).children('a.aschild').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
                            else if (proxy.model.showBottomLevelArrows)
                                $(liTag).children('a.aschild').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
                        }
                    });
                }
                else {
                    var childItems = ej.DataManager(mapper["child"]["dataSource"]).executeLocal(ej.Query().where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]));
                    if (childItems && childItems.length > 0) {
                        var ul = $(document.createElement('ul'));
                        for (var i = 0; i < childItems.length; i++) {
                            var liItem = this._menuTemplate(childItems[i], mapper["child"]["dataSource"], mapper["child"]);
                            ul.append(liItem);
                        }
                        liTag.append(ul);
                    }
                }
            }
            else if (!this._odataFlag) {
                var childItems = ej.DataManager(mapper["dataSource"]).executeLocal(ej.Query().where(mapper["parentId"], ej.FilterOperators.equal, item[mapper.id]));
                if (childItems && childItems.length > 0) {
                    var ul = ej.buildTag('ul');
                    for (var i = 0; i < childItems.length; i++) {
                        var liItem = this._menuTemplate(childItems[i], mapper["dataSource"], mapper);
                        ul.append(liItem);
                    }
                    liTag.append(ul);
                }
            }
            return liTag;
        },
        /**
        * To configure the Attributes of Menu items
        * @private
        */
        _setAttributes: function (data, element) {
            for (var key in data) {
                element.attr(key, data[key]);
            }
        },
        /**
        * To configure the Top and Bottom Level Arrows	
        * @private
        */
        _addArrow: function (topArrows, bottomArrows) {
            if (topArrows)
                this.element.find('>li.e-list:has("> ul")').children('a,span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else {
                this.element.find('>li.e-list:has("> ul")').children('a,span').removeClass("e-arrow-space").children('span').remove();
            }

            if (bottomArrows)
                this.element.find('>li.e-list > ul li.e-list:has(>ul)').children('a,span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else {
                this.element.find('>li.e-list > ul li.e-list:has(>ul)').children('a,span').removeClass("e-arrow-space").children('span').remove();
            }

        },
        /**
         * To initialize the Menu		
		 * @private
         */
        _intializeMenu: function () {
            if (this.model.height != 0) this._setHeight(this.model.height);
            if (this.model.width != 0) this._setWidth(this.model.width);
            if (this.model.menuType == "ContextMenu")
                this.model.openOnClick = false;
            if (this.model.openOnClick) {
                this._hoverOpen = false;
                this._hoverClose = false;
            }
        },
        /**
        * To configure the Orientation of Menu	
        * @private
        */
        _setOrientation: function (val) {
            if (val == "horizontal") {
                this.element.removeClass("e-vertical e-horizontal").addClass("e-horizontal");
            } else {
                this.element.removeClass("e-horizontal e-vertical").addClass("e-vertical");
            }
        },
        /**
        * To configure the height		
        * @private
        */
        _setHeight: function (value) {
            this.element.height(value);
        },
        /**
        * To configure the width		
        * @private
        */
        _setWidth: function (value) {
            this.element.width(value);
        },
        /**
        * To enable or disable the Right to Left behaviour 			
        * @private
        */
        _setRTL: function (isRTL) {
            if (isRTL) {
                this.element.removeClass("rtl").addClass("rtl");
                this.model.subMenuDirection = "Left";
            } else {
                this.element.removeClass("rtl");
                this.model.subMenuDirection = "Right";
            }
        },
        /**
        * To enable or disable the Animation while Hover or Click an Menu items		
        * @private
        */
        _setAnimation: function (value) {
            value === "none" ? (this._showAnim = "none", this._hideAnim = "none") : (this._showAnim = "slideDown", this._hideAnim = "slideUp");
        },
        /**
        * To configure the control status	
        * @private
        */
        _controlStatus: function (value) {
            //disable status
            value != true ? this.disable() : this.enable();
        },
        /**
        * To configure centerAlign of Menu items.		
        * @private
        */
        _centerAlign: function (centerAlign) {
            if (this.model.orientation == "horizontal" && centerAlign)
                this.element.css('text-align', 'center');
            else if (this.model.orientation == "horizontal") this.element.css('text-align', 'inherit');
        },
        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "child" && col !== "dataSource")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },
        /**
        * To Find maximum z-index in the document	
        * @private
        */
        //Gets the maximum z-index in the document
        _max_zindex: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },

        _recursiveFunction: function (items, menuText) {
            var context = this;
            var isFound = false;
            $.each(items, function (key, value) {
                if (value.Text == menuText) {
                    context.selectedItem = value;
                    isFound = true;
                    return false;
                }
                else if (value.ChildItems != null) {
                    context._recursiveFunction(value.ChildItems, menuText);
                }
                if (isFound)
                    return false;
            });
        },
        /**
        * Render section for Context Menu Template	
        * @private
        */
        _contextMenu_Template: function () {
            this.model.orientation = "vertical";
            this._hideAnimation(this.element, this._hideAnim);
            this.element.addClass(this.model.cssClass + "e-context");
            $("body").append(this.element);
            this.wrapper.insertBefore(this.element);
            this.wrapper.append(this.element);
        },
        /**
        * To close currently visible Menu items	
        * @private
        */
        _closeMenu: function () {
            this._hideAnimation(this.element.find('li.e-list:has("> ul")').find('> ul:visible'), this._hideAnim);
        },

        _onMenuIntent: function (element, obj, canOpen) {
            obj._delayMenuHover = window.setTimeout(function () {
                if (obj._mouseOver == true && canOpen) {
                    var showanim = obj._showAnim;
                    var hideanim = obj._hideAnim;
                    var showSpeed = obj._showSpeed;
                    var hideSpeed = obj._hideSpeed;
                    obj._show(element, showanim, hideanim);
                }
            }, this._showSpeed);
        },
        /**
         * To hide Menu items 
		 * @private
         */
        _onHide: function (element, obj, canHide) {
            obj._delaySubMenuHover = window.setTimeout(function () {
                if (obj._mouseOver == false && canHide) {
                    var id = obj._id;
                    var hideanim = obj._hideAnim;
                    var hideSpeed = obj._hideSpeed;
                    obj._closeAll();
                }

            }, obj._hideSpeed);
        },
        /**
        * To configure Sub Menu Position 
        * @private
        */
        _subMenuPos: function (element, direction) {
            var parentPos = $(this.wrapper).parent().offset();
            var pos = $(element).offset();
            var posLeft = pos.left - parentPos.left;
            var subMenu = $('ul:first', element);
            if (pos == null || pos == undefined)
                return false;
            var submenuWidth = subMenu.outerWidth();
            var left = document.documentElement.clientWidth + $(document).scrollLeft();
            if (this.model.menuType == "NormalMenu") {
                if ($(element.parentNode).is(this.element)) {
                    if (this.model.orientation == "horizontal")
                        subMenu.css("top", $(element).outerHeight() + "px");
                    else if (direction == "Left" && posLeft > submenuWidth)
                        subMenu.css("left", -(submenuWidth + 4) + "px");
                } else if (direction == "Left" && posLeft > submenuWidth) {
                    subMenu.css("left", -(submenuWidth + 4) + "px");
                }
            }
            else {
                left -= (pos.left + (2 * submenuWidth) + 4);
                if (left < 0) {
                    var menuLeftPos = (submenuWidth == null) ? "-206.5px" : "-" + (submenuWidth + 5) + "px";
                    subMenu.css("left", menuLeftPos);
                }
                else {
                    if (subMenu.parent('li.e-list').parent('ul').width() && direction == "Right") {
                        subMenu.css("left", (subMenu.parent('li.e-list').parent('ul').width() + 4) + "px");
                    }
                    else if (pos.left > submenuWidth)
                        subMenu.css("left", -(submenuWidth + 4) + "px");
                }
            }
        },

        /**
        * To configure the custom theme for Menu using cssClass property		
        * @private
        */
        _setSkin: function (skin) {
            if (this.model.menuType == "NormalMenu") {
                if (this.model.orientation == "horizontal") {
                    this.element.removeClass(this.model.cssClass + "e-horizontal");
                    this.element.addClass(skin + " e-horizontal");
                } else {
                    this.element.removeClass(this.model.cssClass + "e-vertical");
                    this.element.addClass(skin + " e-vertical");
                }

            } else {
                this.element.removeClass(this.model.cssClass + "e-context");
                this.element.addClass(skin + " e-context");
            }
        },
        /**
        * To configure seperator for Menu items		
        * @private
        */
        _setSeparator: function (separator) {
            if (separator)
                this.element.addClass("e-separator");
            else this.element.removeClass("e-separator");
        },
        /**
         * Wiring the events to Context Menu 	
		 * @private
         */
        _contextMenuEvents: function () {
            this._on($(document.getElementById(this.model.contextTargetId)), "mouseup taphold", this._ContextMenuHandler);
            this._on(this.element, "contextmenu", this._onDefaultPreventer);
            this._on($(document.getElementById(this.model.contextTargetId)), "contextmenu", this._onDefaultPreventer);
            this._on($(document), "click", this._onContextClose);
        },
        /**
        * To show the Menu items		
        * @private
        */
        _show: function (element, showanim, hideanim) {
            var siblingElement;
            var sibling = $('> ul', element);
            sibling.attr({ "aria-hidden": false });
            this._hideAnimation($(element).siblings().find(' > ul:visible'), hideanim);
            if (!($.inArray(this._disabledMenuItems, element) > -1)) {
                if (sibling.css('display') != "none") {
                    siblingElement = this.model.openOnClick ? $(sibling) : sibling.children().find('> ul');
                    this._hideAnimation(siblingElement, hideanim);
                }
                else $('> ul', element).children().find('> ul').hide();
                this._subMenuPos(element, this.model.subMenuDirection);
                sibling.css({ "z-index": this._max_zindex() + 1 });
                if ($('> ul', element).css('display') != 'block')
                    this._showAnimation(sibling, showanim);
            }
        },
        /**
        * To close all visible Menu items		
        * @private
        */
        _closeAll: function () {
            this._hideAnimation(this.element.find('li.e-list:has("> ul")').find('> ul:visible'), this._hideAnim);
            this._hideAnimation(this.element.find('> ul:visible'), this._hideAnim);
        },
        /**
        * To show the Menu items with animation		
        * @private
        */
        _showAnimation: function (element, anim) {
            switch (anim) {
                case "slideDown":
                    element.slideDown(200, "easeOutQuad"); break;
                case "none":
                    element.css("display", "block"); break;
            }
        },
        /**
        * To hide the Menu items with animation		
        * @private
        */
        _hideAnimation: function (element, anim) {
            switch (anim) {
                case "slideUp":
                    $(element).attr({ "aria-hidden": true });
                    element.slideUp(100, "easeOutQuad"); break;
                case "none":
                    element.css("display", "none"); break;
            }
        },

        _removeValue: function (text, disableList) {
            var $browInfo = ej.browserInfo(), elementText;
            $browInfo.version === "8.0" && $browInfo.name === "msie" ? elementText = text[0].outerText : elementText = text[0].textContent;
            var count = $(disableList).length, i = 0;
            while (i <= count) {
                if (disableList[i] === $.trim(elementText))
                    return i;
                i++;
            }
        },

        //-------------------------Public Methods---------------------------------------------//
        /**
       * Shows the Context Menu .
       * @alias showContextMenu
       * @return jQuery
       * @param {number}  locationX  x co-ordinate position of context menu. 
       * @param {number}  locationY   y co-ordinate position of context menu. 
       * @param {object}  targetElement  target element 
       * @param {object}  event name of the event
       * @example 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To show context menu
       *	menuObj.ShowContextMenu();
       * @example 
       * //To show context menu
       * $("#menu").ejMenu("ShowContextMenu");
       * @memberof ejMenu
       * @instance
        */
        //Displays the context menu
        showContextMenu: function (locationX, locationY, targetElement, e) {
            this._closeMenu();
            this._eventArgs = e;
            if ((e != undefined) && e.stopPropagation) e.stopPropagation();
            if (this._trigger("beforeContextOpen", { target: targetElement })) return false;
            if (this._preventContextOpen) {
                this.element.css({ "left": locationX, "top": locationY });
                this._showAnimation(this.element, this._showAnim);
                this._isContextMenuOpen = true;
                if (this.model.contextOpen)
                    this._trigger("contextOpen", { target: targetElement });
            }
            return false;
        },
        /**
       * Hides the Context Menu control.
       * @alias hideContextMenu
       * @return jQuery
       * @example 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To hide context menu
       *	menuObj.hideContextMenu();
       * @example 
       * //To hide context menu
       * $("#menu").ejMenu("hideContextMenu");
       * @memberof ejMenu
       * @instance
        */
        //Hides the context Menu
        hideContextMenu: function (e) {
            this._closeMenu();
            this._hideAnimation(this.element, this._hideAnim);
            this._isContextMenuOpen = false;
            if (this.model.contextClose)
                this._trigger("contextClose", { Events: e });
        },
        /**
       * Specifies the Menu Item to be disabled by using the Menu Item Text.
       * @alias disableMenuItem 
       * @return jQuery
	   * @param {string} itemtext  Specifies the Menu Item Text to be disabled. 
       * @example 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To disable Menu item using item text
       *	menuObj.disableMenuItem("itemtext");
       * @example 
       * //To disable Menu item using item text
       * $("#menu").ejMenu("disableMenuItem","itemtext");
       * @memberof ejMenu
       * @instance
        */
        //To Disable menu item 
        disableMenuItem: function (itemToDisable) {
            var isMenuItem = $(this.element.find('li.e-list >a ,li.e-list >span')).filter(function () { return $.trim($(this).text()) === itemToDisable; });
            if (isMenuItem.length > 0 && !($.inArray(itemToDisable, this._disabledMenuItems) > -1)) {
                isMenuItem.parent().addClass("e-disable-item").attr({ "aria-disabled": true });
                this._disabledMenuItems.push($.trim(isMenuItem.text()));
            }
        },
        /**
        * Specifies the Menu Item to be disabled by using the Menu Item Id.
		* @alias disableMenuItembyID
		* @return jQuery
		* @param {string|number} itemid  Specifies the Menu Item id to be disabled 
		* @example 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To disable Menu item using item id
		*	menuObj.disableMenuItembyID("itemid");
		* @example 
		* //To disable Menu item using item id
		* $("#menu").ejMenu("disableMenuItembyID","itemid");
		* @memberof ejMenu
		* @instance
        */
        //To disable menu item by id
        disableMenuItembyID: function (itemId) {
            if (itemId && itemId != "") {
                var itemToDisable = this.element.find("#" + itemId) ? this.element.find("#" + itemId)[0] : undefined;
                if (itemToDisable && !($.inArray(itemToDisable, this._disabledMenuItems) > -1)) {
                    $(itemToDisable).addClass("e-disable-item").attr({ "aria-disabled": true });
                    this._disabledMenuItems.push(itemToDisable);
                }
            }
        },

        /**
        *Specifies the Menu Item to be enabled by using the Menu Item Text.
		* @alias enableMenuItem
		* @return jQuery
		* @param {string} itemtext  Specifies the Menu Item Text to be enabled.
		* @example 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To enable Menu item using item text
		*	menuObj.enableMenuItem("item text");
		* @example 
		* //To enable Menu item using item text
		* $("#menu").ejMenu("enableMenuItem","item text");
		* @memberof ejMenu
		* @instance
        */
        //To enable menu item 
        enableMenuItem: function (itemToEnable) {
            var isMenuItem = $(this.element.find('li.e-list >a ,li.e-list >span')).filter(function () { return $.trim($(this).text()) === itemToEnable; });
            if (isMenuItem.length > 0 && ($.inArray(itemToEnable, this._disabledMenuItems) > -1)) {
                isMenuItem.parent().removeClass("e-disable-item").attr({ "aria-disabled": false });
                var index = this._removeValue(isMenuItem, this._disabledMenuItems);
                this._disabledMenuItems.splice(index, 1);
            }
        },
        /**
        *Specifies the Menu Item to be enabled by using the Menu Item Id.
		* @alias enableMenuItembyID
		* @return jQuery
		* @param {string|number} itemid  Specifies the Menu Item id to be enabled. 
		* @example 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To enable Menu item using item id
		*	menuObj.enableMenuItembyID("itemid");
		* @example 
		* //To enable Menu item using item id
		* $("#menu").ejMenu("enableMenuItembyID","itemid");
		* @memberof ejMenu
		* @instance
        */
        //To enable menu item by id
        enableMenuItembyID: function (itemId) {
            if (itemId && itemId != "") {
                var itemToEnable = this.element.find("#" + itemId)[0];
                if (itemToEnable && ($.inArray(itemToEnable, this._disabledMenuItems) > -1)) {
                    $(itemToEnable).removeClass("e-disable-item").attr({ "aria-disabled": false });
                    this._disabledMenuItems.pop(itemToEnable);
                }
            }
        },
        /**
        * Disables the Menu control.
		* @alias disable
		* @return jQuery
		* @example 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To disable Menu control
		*	menuObj.disable();
		* @example 
		* //To disable Menu control
		* $("#menu").ejMenu("disable");
		* @memberof ejMenu
		* @instance
        */
        //To disable menu
        disable: function () {
            this.model.enabled = false;
            var menuItemCollection = this.element.find('>li[class!=e-separator]');
            var proxy = this;
            $.each(menuItemCollection, function (key, value) {
                $(value).addClass("e-disable-item").attr({ "aria-disabled": true });
                proxy._disabledMenuItems.push(value);
            });
        },
        /**
        * Enables the Menu control.
		* @alias enable
		* @return jQuery
		* @example 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To enable Menu control
		*	menuObj.enable();
		* @example 
		* //To enable Menu control
		* $("#menu").ejMenu("enable");
		* @memberof ejMenu
		* @instance
        */
        //To enable menu
        enable: function () {
            this.model.enabled = true;
            var menuItemCollection = this.element.find('li.e-disable-item');
            $.each(menuItemCollection, function (key, value) {
                $(value).removeClass("e-disable-item").attr({ "aria-disabled": false });
            });
            this._disabledMenuItems.pop(this._disabledMenuItems);
        },

        //---------dreprecate method----------------//
        ShowContextMenu: function (locationX, locationY, targetElement, e) {
            this._closeMenu();
            this._eventArgs = e;
            if ((e != undefined) && e.stopPropagation) e.stopPropagation();
            if (this._trigger("beforeContextOpen", { target: targetElement })) return false;
            if (this._preventContextOpen) {
                this.element.css({ "left": locationX, "top": locationY });
                this._showAnimation(this.element, this._showAnim);
                this._isContextMenuOpen = true;
                if (this.model.contextOpen)
                    this._trigger("contextOpen", { target: targetElement });
            }
            return false;
        },

        HideContextMenu: function (e) {
            this._closeMenu();
            this._hideAnimation(this.element, this._hideAnim);
            this._isContextMenuOpen = false;
            if (this.model.contextClose)
                this._trigger("contextClose", { Events: e });
        },

        //To Disable menu item 
        DisableMenuItem: function (itemToDisable) {
            var $browInfo = ej.browserInfo(), elementText;
            var isMenuItem = this.element.find('li.e-list >a ,li.e-list >span').filter(function () { return $.trim($(this).text()) === itemToDisable; });
            if (isMenuItem.length > 0 && !($.inArray(itemToDisable, this._disabledMenuItems) > -1)) {
                isMenuItem.parent().addClass("e-disable-item").attr({ "aria-disabled": true });
                $browInfo.version === "8.0" && $browInfo.name === "msie" ? elementText = isMenuItem[0].outerText : elementText = isMenuItem[0].textContent;
                this._disabledMenuItems.push($.trim(elementText));
            }
        },

        //To disable menu item by id
        DisableMenuItembyID: function (itemId) {
            if (itemId && itemId != "") {
                var itemToDisable = this.element.find("#" + itemId) ? this.element.find("#" + itemId)[0] : undefined;
                if (itemToDisable && !($.inArray(itemToDisable, this._disabledMenuItems) > -1)) {
                    $(itemToDisable).addClass("e-disable-item").attr({ "aria-disabled": true });
                    this._disabledMenuItems.push(itemToDisable);
                }
            }
        },

        //To enable menu item 
        EnableMenuItem: function (itemToEnable) {
            var isMenuItem = this.element.find('li.e-list >a ,li.e-list >span').filter(function () { return $.trim($(this).text()) === itemToEnable; });
            if (isMenuItem && ($.inArray(itemToEnable, this._disabledMenuItems) > -1)) {
                isMenuItem.parent().removeClass("e-disable-item").attr({ "aria-disabled": false });
                var index = this._removeValue(isMenuItem, this._disabledMenuItems);
                this._disabledMenuItems.splice(index, 1);
            }
        },

        //To enable menu item by id
        EnableMenuItembyID: function (itemId) {
            if (itemId && itemId != "") {
                var itemToEnable = this.element.find("#" + itemId)[0];
                if (itemToEnable && ($.inArray(itemToEnable, this._disabledMenuItems) > -1)) {
                    $(itemToEnable).removeClass("e-disable-item").attr({ "aria-disabled": false });
                    this._disabledMenuItems.pop(itemToEnable);
                }
            }
        },
        //--------- ---------------------------------------------------//
        /**
        * Wiring the events to menu control		
        * @private
        */
        /* Register all the events at initialize*/
        _wireEvents: function () {
            this._on(this.element.find("li.e-list"), "mouseout", this._mouseOutHandler);
            this._on(this.element.find("li.e-list"), "mouseover", this._mouseOverHandler);
            this._on(this.element.find("li.e-list"), "click", this._onClickHandler);
            this._on(this.element, "keydown", this._onKeyDownHandler);
            this._on($(document), "keydown", this._onDocumentKeyDown);
            this._on(this.element, "focus", this._OnFocusHandler);
            this._on(this.element, "blur", this._OnFocusOutHandler);
            if (this.model.menuType == "ContextMenu" && $("#" + this.model.contextTargetId)[0] != null) {
                this._contextMenuEvents();
            }
            else if (this.model.openOnClick) {
                this._on($(document), "click", this._onDocumentClick);
            }

        },

        //--------------------------------Handlers--------------------------

        /**
         * Section to track the keydown event on Document		
		 * @private
         */
        _onDocumentKeyDown: function () {
            this._isFocused = true;
        },
        /**
         * Section to track the mouse over event		
		 * @private
         */
        // handles the Mouseover event
        _mouseOverHandler: function (event) {
            var element;
            this.element.find(".e-mhover").removeClass("e-mhover e-mfocused");
            if ($(event.currentTarget).hasClass('e-disable-item'))
                event.preventDefault();
            else {
                if (event.stopPropagation)
                    event.stopPropagation();
                if (typeof (this._delaySubMenuHover) !== 'undefined') {
                    clearTimeout(this._delaySubMenuHover);
                }
                if (typeof (this._delaySubMenuHover) !== 'undefined') {
                    clearTimeout(this._delayMenuHover);
                }
                this._mouseOver = true;
                this._isMenuOpen = true;
                if ($(event.currentTarget.parentNode.parentNode).is(this.element)) {
                    this._isSubMenuOpen = false;
                }
                else {
                    this._isSubMenuOpen = true;
                }
                if (event.currentTarget.nodeName == "LI" && !$(event.currentTarget).hasClass('e-separator'))
                    element = event.currentTarget;
                else if (event.currentTarget.parentNode) {
                    if (event.currentTarget.parentNode.nodeName == "LI" && !$(event.currentTarget.parentNode).hasClass('e-separator'))
                        element = event.currentTarget.parentNode;
                    else
                        return false;
                }
                else {
                    event.preventDefault();
                    return false;
                }
                this._onMenuIntent(element, this, this._hoverOpen);
                if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                    var menuText = $(element).children('a,span').text();
                    var eventArgs = { "menuId": this.element[0].id, "menuText": menuText, "element": element, "event": event };
                    if (this.model.mouseOver)
                        this._trigger("mouseOver", { Events: eventArgs });
                }
            }
        },
        /**
        * Section to track the mouse out event		
        * @private
        */
        // handles the Mouseout event
        _mouseOutHandler: function (event) {
            var element;
            if ($(event.currentTarget).hasClass('e-disable-item'))
                event.preventDefault();
            else {
                if (event.stopPropagation)
                    event.stopPropagation();
                if (typeof (this._delaySubMenuHover) !== 'undefined') {
                    clearTimeout(this._delaySubMenuHover);
                }
                if (typeof (this._delaySubMenuHover) !== 'undefined') {
                    clearTimeout(this._delayMenuHover);
                }
                this._mouseOver = false;
                this._isMenuOpen = false;

                if (event.currentTarget.nodeName == "LI" && !$(event.currentTarget).hasClass('e-separator'))
                    element = event.currentTarget;
                else if (event.currentTarget.parentNode) {
                    if (event.currentTarget.parentNode.nodeName == "LI" && !$(event.currentTarget.parentNode).hasClass('e-separator'))
                        element = event.currentTarget.parentNode;
                    else
                        return false;
                }
                else {
                    event.preventDefault();
                    return false;
                }
                this._onHide(element, this, this._hoverClose);
                if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                    var menuText = $(element).children('a,span').text();
                    var eventArgs = { "menuId": this.element[0].id, "menuText": menuText, "element": element, "event": event };
                    if (this.model.mouseOut)
                        this._trigger("mouseOut", { Events: eventArgs });
                }
            }
        },
        /**
         * Section For handle click action.
		 * @private
         */
        _onClickHandler: function (event) {
            event.stopPropagation();
            var element;
            this._isFocused = false;
            var openOnClickStart = false;
            if (event.target.nodeName == "LI" && !$(event.target).hasClass('e-separator') && !$(event.target).hasClass('e-disable-item'))
                element = event.target;
            else if (event.target.parentNode && event.target.parentNode.nodeName == "LI" &&
                !$(event.target.parentNode).hasClass('e-separator') && !$(event.target.parentNode).hasClass('e-disable-item')) { //&& event.target.parentNode.parentNode.parentNode.nodeName == "LI") {
                element = event.target.parentNode;
            }
            else if ($(event.target).hasClass('e-arrows') && !$(event.target.parentNode.parentNode).hasClass('e-disable-item')) {
                element = event.target.parentNode.parentNode;
            } else {
                event.preventDefault();
                return false;
            }

            if (!this._hoverOpen && $(element).find(">a,>span").hasClass('aschild')) {
                this._show(element, this._showAnim, this._hideAnim);
                this._hoverOpen = false;
                openOnClickStart = true;
            }
            if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                //Check if Context Menu, then hide the context menu firing the events
                if (this.model.menuType == "ContextMenu") {
                    if (this._isContextMenuOpen) {
                        this._hideAnimation(this.element, this._hideAnim);
                        this._isContextMenuOpen = false;
                        if (this.model.contextClose)
                            this._trigger("contextClose", { Events: event });
                    }
                }
                if (!openOnClickStart) {
                    //                    if (window.navigator.appName == "Microsoft Internet Explorer")
                    //                        this._mouseOutHandler(event, true);
                    //                    else
                    if (this.element.find('li.e-list:has("> ul")').find('> a,> span').hasClass('aschild')) {
                        this._closeMenu();
                        if (this.model.openOnClick)
                            this._hoverOpen = false;
                    }
                }
                var menuText = $(element).children('a,span').text();
                // if (this.model.dataSource) {
                // var Items = this.model.dataSource;
                // this._recursiveFunction(Items, menuText);
                // }
                var eventArgs = { "menuId": this.element[0].id, "menuText": menuText, "element": element, "event": event, "selectedItem": this.selectedItem };
                if (this.model.click)
                    this._trigger("click", { Events: eventArgs });
                this.selectedItem = null;
            }
        },

        /**
       * Section to watch the key down action.		
       * @private
       */
        _onKeyDownHandler: function (e) {
            e.preventDefault();
            var element, hoverElement = this.element.find(".e-mhover"), focusedElement = this.element.find(".e-mfocused");
            //key down
            if (e.keyCode == 40) {
                if (this.model.orientation != "horizontal" && this.element.find(">li.e-mhover").children("ul").length > 0) {
                    focusedElement.removeClass("e-mfocused");
                    if (hoverElement.next().hasClass("e-separator"))
                        hoverElement.next().next().addClass("e-mfocused");
                    else if (hoverElement.is(':last-child')) {
                        hoverElement.siblings().first().addClass("e-mfocused");
                    }
                }
                if (this.model.orientation == "horizontal" && this.element.find(">li.e-mhover").children("ul").length > 0) {
                    this._show(hoverElement[0], this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").children("ul:first").find("li:first").addClass("e-mhover");
                }
                else {
                    if (hoverElement.is(':last-child')) {
                        hoverElement.removeClass("e-mhover").siblings().first().addClass("e-mhover");
                    }
                    else if (hoverElement.next().hasClass("e-separator"))
                        hoverElement.removeClass("e-mhover").next().next().addClass("e-mhover");
                    else {
                        hoverElement.removeClass("e-mhover").next().addClass("e-mhover");
                    }
                }
            }
            //key right
            if (e.keyCode == 39) {
                //for root level li
                if (this.model.orientation == "horizontal" && this.element.find(">li.e-list").hasClass("e-mhover")) {
                    if (hoverElement.is(":last-child")) {
                        hoverElement.removeClass("e-mhover e-mfocused");
                        if (this.element.find("li.e-list").first().hasClass("e-separator")) this.element.find("li.e-list").first().next().addClass("e-mhover e-mfocused");
                        else this.element.find("li.e-list").first().addClass("e-mhover e-mfocused");
                    }
                    else if (hoverElement.next().hasClass("e-separator")) {
                        hoverElement.removeClass("e-mhover e-mfocused").next().next().addClass("e-mhover e-mfocused");
                    }
                    else {
                        hoverElement.removeClass("e-mhover e-mfocused").next().addClass("e-mhover e-mfocused");
                    }
                }
                else if (this.element.find("li.e-mhover").children("ul").length > 0) {
                    this._show(hoverElement[0], this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").children("ul:first").find("li:first").addClass("e-mhover");
                }
                else if (hoverElement.children("ul").length <= 0) {
                    this.element.find(".e-mfocused >ul , .e-mfocused >ul li.e-list >ul").hide();
                    hoverElement.removeClass("e-mhover");
                    if (focusedElement.next().hasClass("e-separator")) {
                        focusedElement.removeClass("e-mfocused").next().next().addClass("e-mhover e-mfocused");

                    } else if (focusedElement.next().length <= 0) {
                        focusedElement.removeClass("e-mfocused").siblings().first().addClass("e-mhover e-mfocused");
                    } else focusedElement.removeClass("e-mfocused").next().addClass("e-mhover e-mfocused");
                }
            }
            //key up
            if (e.keyCode == 38) {
                if (this.model.orientation != "horizontal" && this.element.find(">li.e-mhover").children("ul").length > 0) {
                    hoverElement.children("ul").hide();
                    focusedElement.removeClass("e-mfocused");
                    if (hoverElement.prev().hasClass("e-separator"))
                        hoverElement.prev().prev().addClass("e-mfocused");
                    else if (hoverElement.is(':first-child')) {
                        hoverElement.siblings().last().addClass("e-mfocused");
                    }
                }
                if (hoverElement.is(':first-child')) {
                    hoverElement.removeClass("e-mhover").siblings().last().addClass("e-mhover");
                }
                else if (hoverElement.prev().hasClass("e-separator"))
                    hoverElement.removeClass("e-mhover").prev().prev().addClass("e-mhover");
                else
                    hoverElement.removeClass("e-mhover").prev().addClass("e-mhover");

            }
            //Key left
            if (e.keyCode == 37) {
                //for root level li
                if (this.model.orientation == "horizontal" && this.element.find(">li.e-list").hasClass("e-mhover")) {
                    if (hoverElement.is(":first-child")) {
                        hoverElement.removeClass("e-mhover e-mfocused");
                        if (this.element.find(">li.e-list").last().hasClass("e-separator")) this.element.find(">li.e-list").last().prev().addClass("e-mhover e-mfocused");
                        else this.element.find(">li.e-list").last().addClass("e-mhover e-mfocused");
                    }
                    else if (hoverElement.prev().hasClass("e-separator")) {
                        hoverElement.removeClass("e-mhover e-mfocused").prev().prev().addClass("e-mhover e-mfocused");
                    }
                    else {
                        hoverElement.removeClass("e-mhover e-mfocused").prev().addClass("e-mhover e-mfocused");
                    }
                }
                else if (this.element.find("li.e-mhover")) {
                    this._hideAnimation(hoverElement.parent(), this._hideAnim);
                    hoverElement.parent().closest("li.e-list").addClass("e-mhover");
                    hoverElement.removeClass("e-mhover");
                }
                else {
                    this._hideAnimation(hoverElement, this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").parent().addClass("e-mhover");
                }
            }
            if (e.keyCode == 13) {
                this.element.find(".e-mhover >a,.e-mhover >span ").focus();
                this.element.find("li.e-list").removeClass("e-mhover e-mfocused");
                this._closeAll();
            }
            if (e.keyCode == 27) {
                this._closeAll();
            }
            if (e.target.nodeName == "DIV" && $(e.target).children().is(this.element))
                element = $(e.target).children()[0];
            else if (e.target.nodeName == "LI" && !$(e.target).hasClass('e-separator'))
                element = e.target;
            else if (e.target.parentNode) {
                if (e.target.parentNode.nodeName == "LI" && !$(e.target.parentNode).hasClass('e-separator'))
                    element = e.target.parentNode;
            }
            else
                return false;
            if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                var menuText = $(element).children('a,span').text();
                var eventArgs = { "menuId": this.element[0].id, "menuText": menuText, "element": element, "event": e };
                if (this.model.keyDown)
                    this._trigger("keyDown", { Events: eventArgs });
            }

        },
        /**
         * Section For handle Focus action.
		 * @private
         */
        _OnFocusHandler: function (event) {
            if (this._isFocused) {
                this.element.find(">li:first").addClass("e-mhover e-mfocused");
            }
            else this._isFocused = false;
        },
        /**
         * Section For handle FocusOut action.
		 * @private
         */
        _OnFocusOutHandler: function () {
            if (!this._isFocused) {
                this.element.find("li.e-list").removeClass("e-mhover e-mfocused");
                this._closeAll();
            }
        },
        /**
        * Section For handle Click action on Document 
        * @private
        */
        _onDocumentClick: function (event) {
            if (!$(event.target).parents("ul.e-menu").is(this.element)) {
                this._hoverOpen = false;
                this._closeMenu();
            }
        },
        /**
        * Section For handle the context menu event 
        * @private
        */
        //handles the context menu event
        _ContextMenuHandler: function (e) {
            var isRightClick = false;
            if (e.type == "taphold")
                isRightClick = true;
            else if (e.button)
                isRightClick = (e.button == 2);
            else if (e.which)
                isRightClick = (e.which == 3); //for Opera
            var targetElement = e.target;
            if (isRightClick) {
                var evt = e;
                if (e.type == "taphold") {
                    if (e.options.type == "touchstart") evt = e.options.touches[0];
                    else evt = e.options;
                }
                var locationX = (evt.clientX + this.element.width() < $(window).width()) ? evt.pageX : evt.pageX - this.element.width();
                var locationY = (evt.clientY + this.element.height() < $(window).height()) ? evt.pageY : evt.pageY - this.element.height();
                var showSpeed = this._showSpeed;
                this.showContextMenu(locationX, locationY, targetElement, e);
            }
            else {
                if (this._isContextMenuOpen) {
                    var hideanim = this._hideAnim;
                    var hideSpeed = this._hideSpeed;
                    this.hideContextMenu(hideanim, hideSpeed, e);
                }
            }
        },
        /**
         * Section For handles preventing the default (browser's) context menu
		 * @private
         */
        //handles preventing the default (browser's) context menu
        _onDefaultPreventer: function (e) {
            e.preventDefault();
            e.stopPropagation();
            return false;
        },
        /**
         * Section For handles context close on document click
		 * @private
         */
        //handles context close on document click
        _onContextClose: function (e) {
            var proxy = this;
            if (this._isContextMenuOpen) {
                var isRightClick = false;
                if (e.button)
                    isRightClick = (e.button == 2);
                else if (e.which)
                    isRightClick = (e.which == 3); //for Opera

                if (!isRightClick) {
                    if (e.target.id == this.model.contextTargetId)
                        return;

                    else {
                        var hideanim = this._hideAnim;
                        var hideSpeed = this._hideSpeed;
                        this.hideContextMenu(hideanim, hideSpeed, e);
                    }
                    var parentElements = $(e.target).parents();
                    $.each(parentElements, function (index, value) {
                        if (value.id == proxy._ContextTargetId) {
                            return;
                        }
                    });

                }
            }
        }

    });

    /**
	 * Enum for Menu type	 
	 * @enum {string}
	 * @global 
	 */
    ej.menuType = {
        /**  support for list of items appears as normal menu in horizontal or vertical direction. */
        NormalMenu: "NormalMenu",
        /**  support for list of items appears as menu when right clicked on target area, thereby preventing browser’s default right click.. */
        ContextMenu: "ContextMenu"
    };
    /**
	 * Enum for Sub Menu Popup direction	 
	 * @enum {string}
	 * @global 
	 */
    ej.direction = {
        /**  support for Render sub menu popup in left direction. */
        Left: "Left",
        /**  support for Render sub menu popup in Right direction. */
        Right: "Right"
    };
    /**
	 * Enum for animation	 
	 * @enum {string}
	 * @global 
	 */
    ej.animation = {
        /**  support for disable the Animation while hover or click an menu items. */
        None: "none",
        /**  support for enable the Animation while hover or click an menu items. */
        Default: "default"
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {

    ej.widget("ejPager", "ej.Pager", {
        _rootCSS: "e-pager",
        validTags: ["div"],
        // default model
        defaults: {
            pageSize: 12,
            pageCount: 10,
            currentPage: 1,
            enableExternalMessage: false,
            externalMessage: "",
            enableQueryString: false,
            localization: "en-US",
            masterObject: null,
            click: null,
            rtl: false,
            totalRecordsCount: null,
            totalPages: null
        },

        // constructor function
        _init: function () {
            this._initPrivateProperties();
            this.model.enableQueryString && this._queryStringValue();
            this.renderPager();
            this._wireEvents();
        },
        _initPrivateProperties: function () {
            this._links = [];
            this._$prev = null;
            this._$first = null;
            this._$PP = null;
            this._$NP = null;
            this._lastNP = false;
            this._lastpageCount = null;
            this._$last = null;
            this._$next = null;
            this._prevPageNo = null;
        },
        _wireEvents: function () {
            if (this.model.click != undefined)
                this._on(this.element, "click", this._pagerClickHandler);

        },
        renderPager: function () {
            var $pagerContainer = ej.buildTag('div.e-pagercontainer', "", {}, { unselectable: "on" });
            var $parentMsgBar = ej.buildTag('div.e-parentmsgbar');
            this._renderPagerContainer($pagerContainer);
            this._renderMsgBar($parentMsgBar);
            this.element.append($pagerContainer).append($parentMsgBar).addClass(this.model.rtl ? "e-pager e-rtl" : "e-pager");
            this.model.enableExternalMessage && this._renderPagerMessage();
        },
        _queryStringValue: function () {
            var results = new RegExp('[\\?&]page=([^&#]*)').exec(window.location.href);
            !results ? this.model.currentPage = 1 : this.model.currentPage = parseInt(results[1] || 1);
        },
        _renderPagerMessage: function () {
            var $messageDiv = ej.buildTag('div.e-pagermessage');
            if (this.model.externalMessage.toString().length)
                $messageDiv.html(this.model.externalMessage);
            else
                $messageDiv.hide();
            this.element.append($messageDiv);
        },
        _renderPagerContainer: function ($pagerContainer) {
            //Update pager styles here for next versions
            this._renderBackwardButton($pagerContainer);
            this._renderpreviousPager($pagerContainer);
            this._renderNumericItem($pagerContainer);
            this._renderForwardPager($pagerContainer);
            this._renderForwardButton($pagerContainer);
        },
        _renderMsgBar: function ($ParentMsgBar) {
            var $msgBar = ej.buildTag('span', String.format(this._getLocalizedLabels("pagerInfo"), this.model.currentPage, this.model.totalPages || 0, this.model.totalRecordsCount || 0));
            $ParentMsgBar.append($msgBar);
            $ParentMsgBar.css("text-align", ej.textAlign.Right);
        },
        _renderpreviousPager: function ($pagerContainer) {
            this._$PP = ej.buildTag('a.e-link e-nextprevitemdisabled e-disable e-spacing e-PP', "...", {}, { title: this._getLocalizedLabels("previousPagerTooltip"), role: "link" });
            $pagerContainer.append(this._$PP);
        },
        _renderForwardPager: function ($pagerContainer) {
            this._$NP = ej.buildTag('a.e-link e-NP e-numericitem e-spacing e-default', "...", {}, { title: this._getLocalizedLabels("nextPagerTooltip"), role: "link" });
            $pagerContainer.append(this._$NP);
        },
        _renderBackwardButton: function ($pagerContainer) {
            this._$first = ej.buildTag('div.e-first e-icon e-mediaback  e-firstpagedisabled e-disable', "", {}, { unselectable: "on", title: this._getLocalizedLabels("firstPageTooltip") });
            this._$prev = ej.buildTag('div.e-prev e-icon e-arrowheadleft-2x  e-prevpagedisabled e-disable', "", {}, { unselectable: "on", title: this._getLocalizedLabels("previousPageTooltip") });
            $pagerContainer.append(this._$first);
            $pagerContainer.append(this._$prev);
        },
        _renderNumericItem: function ($pagerContainer) {
            var $numericContainer = ej.buildTag('div.e-numericcontainer e-default', "", {}, { unselectable: "on", id: "NumericContainer" });
            this._renderNumericLinks($numericContainer, this.model.pageCount);
            $pagerContainer.append($numericContainer);
        },
        _renderNumericLinks: function ($numericContainer) {
            $numericContainer.empty();
            for (var page = 1; page <= this.model.pageCount; page++) {
                var $link = ej.buildTag('a.e-link', page, {}, { role: "link" }).addClass("e-numericitem e-spacing e-default");
                if (page == this.model.currentPage)
                    $link.removeClass("e-default").addClass("e-currentitem e-active");
                $numericContainer.append($link);
            }
            this._links = $numericContainer.children();
        },
        _renderForwardButton: function ($pagerContainer) {
            this._$next = ej.buildTag('div.e-nextpage e-icon e-arrowheadright-2x  e-default', "", {}, { unselectable: "on", title: this._getLocalizedLabels("nextPageTooltip") });
            this._$last = ej.buildTag('div.e-lastpage e-icon e-mediaforward  e-default', "", {}, { unselectable: "on", title: this._getLocalizedLabels("lastPageTooltip") });
            $pagerContainer.append(this._$next);
            $pagerContainer.append(this._$last);

        },
        _applyCss: function () {
            if (this.model.currentPage > 1) {
                this._$prev.removeClass("e-prevpagedisabled").removeClass("e-disable").addClass("e-prevpage e-default");
                this._$first.removeClass("e-firstpagedisabled").removeClass("e-disable").addClass("e-firstpage e-default");
            } else {
                this._$prev.addClass("e-prevpagedisabled e-disable").removeClass("e-prevpage").removeClass("e-default");
                this._$first.addClass("e-firstpagedisabled e-disable").removeClass("e-firstpage").removeClass("e-default");
            }
            (this.model.currentPage > this.model.pageCount) ?
                 (this._$PP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default")) : (this._$PP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default"));

            if (parseInt(this._links[0].innerHTML, 10) + this.model.pageCount > this.model.totalPages)
                this._lastNP = true;
            else
                this._lastNP = false;

            if (this._lastNP == false)
                this._$NP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default");
            else
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");

            this._lastpageCount = this.model.totalPages % this.model.pageCount;
            if (this._lastpageCount == 0)
                this._lastpageCount = this.model.pageCount;
            if (this.model.currentPage > (this.model.totalPages - this._LastpageCount)) {
                this._$PP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            }
            if (this.model.currentPage == this.model.totalPages || this.model.totalRecordsCount == 0) {
                this._$last.addClass("e-lastpagedisabled e-disable").removeClass("e-lastpage").removeClass("e-default");
                this._$next.addClass("e-nextpagedisabled e-disable").removeClass("e-nextpage").removeClass("e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            } else {
                this._$last.addClass("e-lastpage e-default").removeClass("e-lastpagedisabled").removeClass("e-disable");
                this._$next.addClass("e-nextpage e-default").removeClass("e-nextpagedisabled").removeClass("e-disable");
            }
            if (this._links.length) {
                this._links.removeClass("e-currentitem").removeClass("e-active").addClass("e-default");
                $(this._links[(this.model.currentPage - 1) % this.model.pageCount]).removeClass("e-default").addClass("e-currentitem e-active");
                $(this._links[(this._prevPageNo - 1) % this.model.pageCount]).removeClass("e-default").addClass("e-numericitem e-active");
            }
            if (this.model.pageSize >= (this.model.totalRecordsCount / this.model.pageCount) && this._$PP != null && this._$PP.length != 0) {
                this._$PP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            }
        },
        _SetTotalPages: function () {
            this.model.totalPages = (this.model.totalRecordsCount % this.model.pageSize == 0) ? (this.model.totalRecordsCount / this.model.pageSize) : (parseInt(this.model.totalRecordsCount / this.model.pageSize, 10) + 1);
        },
        _refreshNumericItem: function () {
            if (this._links.length != 0 && this._links != null) {
                this.model.currentPage = this.model.totalPages == 1 ? 1 : this.model.currentPage;
                if (this.model.currentPage > this.model.totalPages && this.model.totalPages != 0)
                    this.model.currentPage = this.model.totalPages;
                var _pagerTarget = parseInt(this.model.currentPage / this.model.pageCount, 10);
                if (this.model.currentPage % this.model.pageCount == 0)
                    if (_pagerTarget > 0)
                        _pagerTarget = _pagerTarget - 1;
                this._links.css("display", "none");
                for (var i = 0; i < this.model.pageCount; i++) {
                    var start = (_pagerTarget * this.model.pageCount) + 1 + i;
                    if (start <= this.model.totalPages) {
                        this._links[i].style.display = '';
                        this._links[i].innerHTML = start;
                    }
                }
            }
        },
        _refreshPagerInfo: function () {
            if (this.model.totalRecordsCount == 0)
                this.model.currentPage = 0;
            this.element.find("span").text(String.format(this._getLocalizedLabels("pagerInfo"), this.model.currentPage, this.model.totalPages || 0, this.model.totalRecordsCount || 0));
        },
        _refreshExternalMessage: function () {
            if (this.model.externalMessage.toString().length)
                this.element.find(".e-pagermessage").empty().html(this.model.externalMessage).show();
            else
                this.element.find(".e-pagermessage").hide();
        },
        refreshPager: function () {
            this._SetTotalPages();
            this._refreshNumericItem();
            this._refreshPagerInfo();
            this._applyCss();
            this.model.enableExternalMessage && this._refreshExternalMessage();
            if (this.model.rtl)
                this.element.addClass("e-rtl");
            else
                this.element.removeClass("e-rtl");
        },

        _kDownHandler: function (e) {
            var code;
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            e.target = null;
            if (this.model.masterObject.checkKey("firstPage", code, e))
                e.target = this._$first;
            else if (this.model.masterObject.checkKey("previousPager", code, e))
                e.target = this._$PP;
            else if (this.model.masterObject.checkKey("previousPage", code, e))
                e.target = this._$prev;
            else if (this.model.masterObject.checkKey("lastPage", code, e))
                e.target = this._$last;
            else if (this.model.masterObject.checkKey("nextPager", code, e))
                e.target = this._$NP;
            else if (this.model.masterObject.checkKey("nextPage", code, e))
                e.target = this._$next;
            else
                return false;
            this._pagerClickHandler(e);
        },

        _pagerClickHandler: function (e) {
            this._prevPageNo = this.model.currentPage;
            var $target = $(e.target);
            if ($.inArray(e.target, this._links) != -1) {
                this.model.currentPage = parseInt(e.target.innerHTML, 10);

            }
            else if ($target.hasClass("e-nextpage") && $target.hasClass("e-nextpagedisabled") != true) {
                if (this.model.currentPage % this.model.pageCount == 0) {
                    this.model.currentPage++;
                    if (this._links != undefined && this._links.length != 0)
                        this.model.currentPage = parseInt(this._links[0].innerHTML, 10) + this.model.pageCount;
                    if (parseInt(this._links[this.model.pageCount - 1].innerHTML, 10) + this.model.pageCount >= this.model.totalPages)
                        this._lastNP = true;
                }
                else
                    this.model.currentPage++;
            }
            else if ($target.hasClass("e-prevpage") && $target.hasClass("e-prevpagedisabled") != true) {
                if (this.model.currentPage % this.model.pageCount == 1)
                    this._lastNP = false;
                this.model.currentPage--;
                if (this.model.currentPage < 0) { this.model.currentPage = 0; }
            }
            else if ($target.hasClass("e-lastpage") && $target.hasClass("e-lastpagedisabled") != true) {
                this._LastpageCount = this.model.totalPages % this.model.pageCount;
                (this._LastpageCount == 0) ? (this._LastpageCount = this.model.pageCount) : null;
                this.model.currentPage = this.model.totalPages;
                this._lastNP = true;
            }
            else if ($target.hasClass("e-firstpage") && $target.hasClass("e-firstpagedisabled") != true) {
                this.model.currentPage = 1;
                this._lastNP = false;
            }
            else if ($target.hasClass("e-NP") && $target.hasClass("e-nextprevitemdisabled") != true) {
                if (this._links != undefined)
                    this.model.currentPage = parseInt(this._links[0].innerHTML, 10) + this.model.pageCount;
                if (parseInt(this._links[this.model.pageCount - 1].innerHTML, 10) + this.model.pageCount >= this.model.totalPages) {
                    this._lastNP = true;
                    if ((this.model.totalRecordsCount - this.model.pageSize) < this.model.pageSize)
                        this._LastpageCount = this.model.totalRecordsCount - this.model.pageSize;
                    else
                        this._LastpageCount = ((this.model.totalRecordsCount / this.model.pageSize) % this.model.pageCount);
                    (this._LastpageCount == 0) ? (this._LastpageCount = this.model.pageCount) : null;
                    if (this._links != undefined)
                        this.model.currentPage = parseInt(this._links[this.model.pageCount - 1].innerHTML, 10) + 1;
                }
            }
            else if ($target.hasClass("e-PP") && $target.hasClass("e-nextprevitemdisabled") != true) {
                if (this._links != undefined)
                    this.model.currentPage = parseInt(this._links[0].innerHTML, 10) - this.model.pageCount;
                this._lastNP = false;
            }
            this.goToPage(this.model.currentPage);
            return false;
        },
        goToPage: function (pageIndex) {
            if (pageIndex != this.model.currentPage)
                this._prevPageNo = this.model.currentPage;
            if (this._prevPageNo !== pageIndex && (pageIndex >= 1 && pageIndex <= this.model.totalPages)) {
                this.model.currentPage = pageIndex;
                this.model.enableQueryString && this._updateQueryString(this.model.currentPage);
                this._trigger("click", { "currentPage": pageIndex });
            }
        },
        _updateQueryString: function (value) {
            var _newUrl = this._getUpdatedURL(window.location.href, "page", value);
            if (history.pushState) {
                window.history.pushState({ path: _newUrl }, '', _newUrl);
            }
            else
                window.location.href = _newUrl;
        },

        _getUpdatedURL: function (uri, key, value) {
            var re = new RegExp("([?|&])" + key + "=.*?(&|#|$)", "i");
            if (uri.match(re)) {
                return uri.replace(re, '$1' + key + "=" + value + '$2');
            } else {
                var hash = '';
                var separator = uri.indexOf('?') !== -1 ? "&" : "?";
                if (uri.indexOf('#') !== -1) {
                    hash = uri.replace(/.*#/, '#');
                    uri = uri.replace(/#.*/, '');
                }
                return uri + separator + key + "=" + value + hash;
            }
        },
        _getLocalizedLabels: function (property) {
            return ej.Pager.localization[this.model.localization][property] === undefined ? ej.Pager.localization["en-US"][property] : ej.Pager.localization[this.model.localization][property];
        },
        _setFirst: true,
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "pageCount":
                        this._renderNumericLinks(this.element.find("#NumericContainer"));
                        break;
                }
            }
            this.refreshPager();
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.empty().removeClass("e-pager");
        }
    });
    ej.Pager.localization = {};
    ej.Pager.localization["en-US"] = {
        pagerInfo: "{0} of {1} pages ({2} items)",
        firstPageTooltip: "Go to first page",
        lastPageTooltip: "Go to last page",
        nextPageTooltip: "Go to next page",
        previousPageTooltip: "Go to previous page",
        nextPagerTooltip: "Go to next Pager",
        previousPagerTooltip: "Go to previous Pager"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    // ejSlider is the plugin name 
    // "ej.Slider" is "namespace.className" will hold functions and properties

    /**
    * @namespace ej
	* @class ejSlider
	* @requires jQuery
	* @requires ej.web.all
	* @classdesc Custom Design for Html Slider control.
	* @example 
	* &lt;div id="slider"&gt; &lt;/div&gt; <br> 	
	* &lt;script&gt; <br>
	* // Create Slider <br>
    * $('#slider').ejSlider(); <br>		
	* &lt;/script&gt; <br>
	*/

    ej.widget("ejSlider", "ej.Slider", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-slider",
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Specifies the orientation of the slider.	
			* @default ej.orientation.Horizontal
			* @type {enum}
			* @example 
			* //To set orientation API value during initialization  
			* 	$("#slider").ejSlider({ orientation: ej.orientation.Vertical});					 
			* @example 
			* //Get or set the orientation API, after initialization:<br>
			*	//Gets the orientation value  
			*	$("#slider").ejSlider("option", "orientation");<br>			
			*	//Sets the orientation value 
			*	$("#slider").ejSlider("option", "orientation",  ej.orientation.Vertical ); 
			* @memberof ejSlider
			* @instance
			*/
            orientation: "horizontal",
            /**		
			* Specifies the animation behaviour of the slider.	
			* @default true
			* @type {boolean}
			* @example 
			* //To set animate API value during initialization  
			* 	$("#slider").ejSlider({ animate: true});					 
			* @example 
			* //Get or set the animate API, after initialization:<br>
			*	//Gets the animate value  
			*	$("#slider").ejSlider("option", "animate");<br>			
			*	//Sets the animate value 
			*	$("#slider").ejSlider("option", "animate",  true ); 
			* @memberof ejSlider
			* @instance
			*/
            animate: true,
            /**		
			* Specifies the animationSpeed of the slider.	
			* @default 500
			* @type {number}
			* @example 
			* //To set animationSpeed API value during initialization  
			* 	$("#slider").ejSlider({ animationSpeed: true});					 
			* @example 
			* //Get or set the animationSpeed API, after initialization:<br>
			*	//Gets the animationSpeed value  
			*	$("#slider").ejSlider("option", "animationSpeed");<br>			
			*	//Sets the animationSpeed value 
			*	$("#slider").ejSlider("option", "animationSpeed",  500 ); 
			* @memberof ejSlider
			* @instance
			*/
            animationSpeed: 500,
            /**		
			* Specifies the showTooltip to shows the current Slider value, while moving the Slider handle of the slider.	
			* @default true
			* @type {boolean}
			* @example 
			* //To set showTooltip API value during initialization  
			* 	$("#slider").ejSlider({ showTooltip: true});					 
			* @example 
			* //Get or set the showTooltip API, after initialization:<br>
			*	//Gets the showTooltip value  
			*	$("#slider").ejSlider("option", "showTooltip");<br>			
			*	//Sets the showTooltip value 
			*	$("#slider").ejSlider("option", "showTooltip",  true ); 
			* @memberof ejSlider
			* @instance
			*/
            showTooltip: true,
            /**		
			* Specify the CSS class to slider to achieve custom theme.	
			* @default ""
			* @type {string}
			* @example 
			* //To set cssClass API value during initialization  
			* 	$("#slider").ejSlider({ cssClass: "gradient-lime"});					 
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the cssClass value  
			*	$("#slider").ejSlider("option", "cssClass");<br>			
			*	//Sets the cssClass value 
			*	$("#slider").ejSlider("option", "cssClass",  "gradient-lime" ); 
			* @memberof ejSlider
			* @instance
			*/
            cssClass: "",
            /**		
			* Specifies the rounded corner behaviour for slider.	
			* @default false
			* @type {boolean}
			* @example 
			* //To set roundedCorner API value during initialization  
			* 	$("#slider").ejSlider({ roundedCorner: true});					 
			* @example 
			* //Get or set the roundedCorner API, after initialization:<br>
			*	//Gets the roundedCorner value  
			*	$("#slider").ejSlider("option", "roundedCorner");<br>			
			*	//Sets the roundedCorner value 
			*	$("#slider").ejSlider("option", "roundedCorner",  true ); 
			* @memberof ejSlider
			* @instance
			*/
            roundedCorner: false,
            /**		
			* Specifies the readOnly of the slider.	
			* @default false
			* @type {boolean}
			* @example 
			* //To set readOnly API value during initialization  
			* 	$("#slider").ejSlider({ readOnly: true});					 
			* @example 
			* //Get or set the readOnly API, after initialization:<br>
			*	//Gets the readOnly value  
			*	$("#slider").ejSlider("option", "readOnly");<br>			
			*	//Sets the readOnly value 
			*	$("#slider").ejSlider("option", "readOnly",  true ); 
			* @memberof ejSlider
			* @instance
			*/
            readOnly: false,
            /**		
			* Specifies the Right to Left Direction of the slider.	
			* @default false
			* @type {boolean}
			* @example 
			* //To set rtl API value during initialization  
			* 	$("#slider").ejSlider({ rtl: false});					 
			* @example 
			* //Get or set the rtl API, after initialization:<br>
			*	//Gets the rtl value  
			*	$("#slider").ejSlider("option", "rtl");<br>			
			*	//Sets the rtl value 
			*	$("#slider").ejSlider("option", "rtl",  false ); 
			* @memberof ejSlider
			* @instance
			*/
            rtl: false,
            /**		
			* Specifies the starting value of the slider.	
			* @default 0
			* @type {number}
			* @example 
			* //To set startValue API value during initialization  
			* 	$("#slider").ejSlider({ startValue: 0});					 
			* @example 
			* //Get or set the startValue API, after initialization:<br>
			*	//Gets the startValue value  
			*	$("#slider").ejSlider("option", "startValue");<br>			
			*	//Sets the startValue value 
			*	$("#slider").ejSlider("option", "startValue",  0 ); 
			* @memberof ejSlider
			* @instance
			*/
            startValue: 0,
            /**		
			* Specifies the ending value of the slider.	
			* @default 100
			* @type {number}
			* @example 
			* //To set endValue API value during initialization  
			* 	$("#slider").ejSlider({ endValue: 60});					 
			* @example 
			* //Get or set the endValue API, after initialization:<br>
			*	//Gets the endValue value  
			*	$("#slider").ejSlider("option", "endValue");<br>			
			*	//Sets the endValue value 
			*	$("#slider").ejSlider("option", "endValue",  60 ); 
			* @memberof ejSlider
			* @instance
			*/
            endValue: 100,
            /**		
			* Specifies the sliderType of the slider.	See {@link sliderType}
			* @default ej.sliderType.default
			* @type {enum}
			* @example 
			* //To set sliderType API value during initialization  
			* 	$("#slider").ejSlider({ sliderType: ej.sliderType.default});					 
			* @example 
			* //Get or set the sliderType API, after initialization:<br>
			*	//Gets the sliderType value  
			*	$("#slider").ejSlider("option", "sliderType");<br>			
			*	//Sets the sliderType value 
			*	$("#slider").ejSlider("option", "sliderType",  ej.sliderType.default ); 
			* @memberof ejSlider
			* @instance
			*/
            sliderType: "default",
            /**		
			* Specifies the value of the slider.	
			* @default 0
			* @type {number}
			* @example 
			* //To set value API  during initialization  
			* 	$("#slider").ejSlider({ value: 60});					 
			* @example 
			* //Get or set the value API, after initialization:<br>
			*	//Gets the slider value  
			*	$("#slider").ejSlider("option", "value");<br>			
			*	//Sets the slider value 
			*	$("#slider").ejSlider("option", "value",  60 ); 
			* @memberof ejSlider
			* @instance
			*/
            value: null,
            /**		
			* Specifies the values of the range slider.	
			* @default [startValue,endValue]
			* @type {array}
			* @example 
			* //To set values API during initialization  
			* 	$("#slider").ejSlider({ values: [30,60]});					 
			* @example 
			* //Get or set the values API, after initialization:<br>
			*	//Gets the range slider values  
			*	$("#slider").ejSlider("option", "values");<br>			
			*	//Sets the range slider values 
			*	$("#slider").ejSlider("option", "values",  [30,60] ); 
			* @memberof ejSlider
			* @instance
			*/
            values: null,
            /**		
			* Specifies the incremental step value of the slider.	
			* @default 1
			* @type {number}
			* @example 
			* //To set step API value during initialization  
			* 	$("#slider").ejSlider({ step: 2});					 
			* @example 
			* //Get or set the step API, after initialization:<br>
			*	//Gets the slider step value  
			*	$("#slider").ejSlider("option", "step");<br>			
			*	//Sets the slider step value 
			*	$("#slider").ejSlider("option", "step",  2 ); 
			* @memberof ejSlider
			* @instance
			*/
            step: 1,
            /**		
			* Specifies the height of the slider.	
			* @default 14
			* @type {String}
			* @example 
			* //To set height API value during initialization  
			* 	$("#slider").ejSlider({ height: 14});					 
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height of the slider   
			*	$("#slider").ejSlider("option", "height");<br>			
			*	//Sets the height of the slider 
			*	$("#slider").ejSlider("option", "height",  14 ); 
			* @memberof ejSlider
			* @instance
			*/
            height: null,
            /**		
			* Specifies the width of the slider.	
			* @default 100%
			* @type {String}
			* @example 
			* //To set width API value during initialization  
			* 	$("#slider").ejSlider({ width: "300px"});					 
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width of the slider   
			*	$("#slider").ejSlider("option", "width");<br>			
			*	//Sets the width of the slider 
			*	$("#slider").ejSlider("option", "width",  "300px" ); 
			* @memberof ejSlider
			* @instance
			*/
            width: null,
            /**		
			* Specifies the state of the slider.	
			* @default true
			* @type {boolean}
			* @example 
			* //To set enabled API value during initialization  
			* 	$("#slider").ejSlider({ enabled: false});					 
			* @example 
			* //Get or set the enabled API, after initialization:<br>
			*	//Gets the enabled value  
			*	$("#slider").ejSlider("option", "enabled");<br>			
			*	//Sets the enabled value 
			*	$("#slider").ejSlider("option", "enabled",  false ); 
			* @memberof ejSlider
			* @instance
			*/
            enabled: true,
            /**		
			* Specifies the major (large) and minor (small) ticks of the slider.	
			* @default false
			* @type {boolean}
			* @example 
			* //To set enabled API value during initialization  
			* 	$("#slider").ejSlider({ showScale: false});					 
			* @example 
			* //Get or set the showScale API, after initialization:<br>
			*	//Gets the showScale value  
			*	$("#slider").ejSlider("option", "showScale");<br>			
			*	//Sets the showScale value 
			*	$("#slider").ejSlider("option", "showScale",  false ); 
			* @memberof ejSlider
			* @instance
			*/
            showScale: false,
            /**		
			* Specifies the distance between two major (large) ticks from the scale of the slider.	
			* @default 10
			* @type {number}
			* @example 
			* //To set largeStep API value during initialization  
			* 	$("#slider").ejSlider({ largeStep: 2});					
			* @example 
			* //Get or set the largeStep API, after initialization:<br>
			*	//Gets the slider largeStep value  
			*	$("#slider").ejSlider("option", "largeStep");<br>			
			*	//Sets the slider largeStep value 
			*	$("#slider").ejSlider("option", "largeStep",  2 ); 
			* @memberof ejSlider
			* @instance
			*/
            largeStep: 10,
            /**		
			* Specifies the distance between two minor (small) ticks from the scale of the slider.	
			* @default 10
			* @type {number}
			* @example 
			* //To set smallStep API value during initialization  
			* 	$("#slider").ejSlider({ smallStep: 2});					 
			* @example 
			* //Get or set the smallStep API, after initialization:<br>
			*	//Gets the slider smallStep value  
			*	$("#slider").ejSlider("option", "smallStep");<br>			
			*	//Sets the slider smallStep value 
			*	$("#slider").ejSlider("option", "smallStep",  2 ); 
			* @memberof ejSlider
			* @instance
			*/
            smallStep: 1,
            /**		
			* Specifies the showSmallTicks of the slider.	
			* @default true
			* @type {boolean}
			* @example 
			* //To set showSmallTicks API value during initialization  
			* 	$("#slider").ejSlider({ showSmallTicks: false});					 
			* @example 
			* //Get or set the showSmallTicks API, after initialization:<br>
			*	//Gets the showSmallTicks value  
			*	$("#slider").ejSlider("option", "showSmallTicks");<br>			
			*	//Sets the showSmallTicks value 
			*	$("#slider").ejSlider("option", "showSmallTicks",  false ); 
			* @memberof ejSlider
			* @instance
			*/
            showSmallTicks: true,
            /**		
			* Specify the persist to slider to save current model value to browser cookies for state maintains	
			* @default true
			* @type {boolean}
			* @example 
			* //To set showSmallTicks API value during initialization  
			* 	$("#slider").ejSlider({ showSmallTicks: false});					 
			* @example 
			* //Get or set the showSmallTicks API, after initialization:<br>
			*	//Gets the showSmallTicks value  
			*	$("#slider").ejSlider("option", "showSmallTicks");<br>			
			*	//Sets the showSmallTicks value 
			*	$("#slider").ejSlider("option", "showSmallTicks",  false ); 
			* @memberof ejSlider
			* @instance
			*/
            persist: false,

            // Events
            /**     
			 * Fires when Slider control is started successfully.
			 * @event
			 * @name ejSlider#start 	
			 * @param {Object} argument Event parameters from slider control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {number}  argument.handleNo returns current handle number or index	
			 * @param {string}  argument.id returns slider id	
			 * @param {object}  argument.model returns the slider model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the slider value			 			
			 * @example 
			 * //start event for slider control
             * $("#slider").ejSlider({
             *    start: function (args) {}
             * });      
			 * @memberof ejSlider
			 * @instance
			 */
            start: null,
            /**     
			 * Fires when Slider control is stopped successfully.
			 * @event
			 * @name ejSlider#stop 	
			 * @param {Object} argument Event parameters from slider control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {number}  argument.handleNo returns current handle number or index	
			 * @param {string}  argument.id returns slider id	
			 * @param {object}  argument.model returns the slider model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the slider value			 			
			 * @example 
			 * //stop event for slider control
             * $("#slider").ejSlider({
             *    stop: function (args) {}
             * });      
			 * @memberof ejSlider
			 * @instance
			 */
            stop: null,
            /**     
			 * Fires when Slider control is sliding successfully.
			 * @event
			 * @name ejSlider#slide 	
			 * @param {Object} argument Event parameters from slider control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {number}  argument.handleNo returns current handle number or index	
			 * @param {string}  argument.id returns slider id	
			 * @param {object}  argument.model returns the slider model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the slider value			 			
			 * @example 
			 * //slide event for slider control
             * $("#slider").ejSlider({
             *    slide: function (args) {}
             * });      
			 * @memberof ejSlider
			 * @instance
			 */
            slide: null,
            /**     
			 * Fires when Slider control value is changed successfully.
			 * @event
			 * @name ejSlider#change 	
			 * @param {Object} argument Event parameters from slider control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {number}  argument.handleNo returns current handle number or index	
			 * @param {string}  argument.id returns slider id	
			 * @param {object}  argument.model returns the slider model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the slider value			 			
			 * @example 
			 * //change event for slider control
             * $("#slider").ejSlider({
             *    change: function (args) {}
             * });      
			 * @memberof ejSlider
			 * @instance
			 */
            change: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            orientation: "enum",
            animate: "boolean",
            animationSpeed: "number",
            cssClass: "string",
            roundedCorner: "boolean",
            readOnly: "boolean",
            rtl: "boolean",
            startValue: "number",
            endValue: "number",
            sliderType: "enum",
            step: "number",
            enabled: "boolean",
            showScale: "boolean",
            largeStep: "number",
            smallStep: "number",
            showSmallTicks: "boolean",
            persist: "boolean"
        },

        observables: ["value", "values"],
        value: ej.util.valueFunction("value"),
        values: ej.util.valueFunction("values"),
        /**
        * To enable the slider  
		* @return jQuery
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Create slider control
		* var sliderObj = $("#slider").data("ejSlider");
		* sliderObj.enable(); // enable the slider control
		* &lt;/script&gt;
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* // enable the slider control
		* $("#slider").ejSlider("enable");	
		* &lt;/script&gt;
		*@memberof ejSlider
		* @instance
        */
        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                if (this.wrapper) this.wrapper.removeClass("e-disable");
                else this.element.removeClass("e-disable");
                this._wireEvents();
            }
        },
        /**
        * To disable the slider  
		* @return jQuery
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Create slider control
		* var sliderObj = $("#slider").data("ejSlider");
		* sliderObj.disable(); // disable the slider control
		* &lt;/script&gt;
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* // disable the slider control
		* $("#slider").ejSlider("disable");	
		* &lt;/script&gt;
		*@memberof ejSlider
		* @instance
        */
        disable: function () {
            if (this.model.enabled) {
                this.model.enabled = false;
                if (this.wrapper) this.wrapper.addClass("e-disable");
                else this.element.addClass("e-disable");
                this._unWireEvents();
            }
        },
        /**
         * To check whether the value and its type.		
		 * @private
         */
        _validateValue: function (value) {
            if (value == null) value = this.model.startValue;
            else if (typeof value === "string") value = parseFloat(value);

            if (this._isNumber(value))
                this.value(value);
            else if (!this._isNumber(this.value()))
                this.value(this.model.startValue);

            if (this.model.sliderType != "range") this._setValue();
        },
        /**
         * To check whether and configure the range slider value.		
		 * @private
         */
        _validateRangeValue: function (value) {
            if (value == null) value = new Array(this.model.startValue, this.model.endValue);
            else if (typeof value === "string") {
                var vals = value.split(",");
                if (vals.length > 1) value = new Array(parseFloat(vals[0]), parseFloat(vals[1]));
            }

            if (typeof value === "object" && this._isNumber(value[0]) && this._isNumber(value[1]))
                this.values(new Array(value[0], value[1]));
            else if (!(typeof this.values() === "object" && this._isNumber(this.values()[0]) && this._isNumber(this.values()[1])))
                this.values(new Array(this.model.startValue, this.model.endValue));

            if (this.model.sliderType == "range") this._setRangeValue();
        },
        /**
         * To check whether slider starting and ending value .		
		 * @private
         */
        _validateStartEnd: function () {
            if (isNaN(this.model.startValue)) this.model.startValue = 0;
            if (isNaN(this.model.endValue)) this.model.endValue = 100;
        },
        /**
         * To check whether slider value type.		
		 * @private
         */
        _isNumber: function (number) {
            return typeof number === "number" && !isNaN(number);
        },
        /**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */
        _outerCorner: function (boolean) {
            if (boolean) this._roundedCorner();
            else this._sharpedCorner();
        },
        /**
         * To configure the custom theme for slider using cssClass property		
		 * @private
         */
        _changeSkin: function (skin) {
            this.element.removeClass(this.model.cssClass).addClass(skin);
            if (this.model.showScale)
                this.ul.removeClass(this.model.cssClass).addClass(skin);
        },
        /**
        * To get value from slider handle		
		* @return jQuery
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 		
		* &lt;script&gt;
		* // Create Editors
		* var sliderObj = $("#slider").data("ejSlider");		
		* sliderObj.getValue(); // getValue the slider handle		
		* &lt;/script&gt;
		 @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 			
		* &lt;script&gt;
		* // get value from slider handle
		* $("#slider").ejSlider("getValue");		
		* &lt;/script&gt;
		* @memberof ejSlider
		* @instance
         */
        getValue: function () {
            /// <summary>Returns the Slider value.</summary>
            return this._getHandleValue();
        },
        /**
         * To initialize the slider		
		 * @private
         */
        // constructor function
        _init: function () {
            this._initialize();
            this._render();
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;

            if (!ej.isNullOrUndefined(options["startValue"]) || !ej.isNullOrUndefined(options["endValue"])) {
                if (this._isNumber(options["startValue"])) this.model.startValue = options["startValue"];
                else options["startValue"] = this.model.startValue;

                if (this._isNumber(options["endValue"])) this.model.endValue = options["endValue"];
                else options["endValue"] = this.model.endValue;

                if (this.model.sliderType == "range" && options["values"] == undefined) this._setRangeValue();
                else if (this.model.sliderType != "range" && options["value"] == undefined) this._setValue();
            }

            var option;
            for (option in options) {
                switch (option) {
                    case "value":
                        this._validateValue(ej.util.getVal(options[option]));
                        break;
                    case "values":
                        this._validateRangeValue(ej.util.getVal(options[option]));
                        break;
                    case "height": this.model.height = options[option]; this._setDimension();
                        if (this.model.showScale) this._scaleAlignment();
                        break;
                    case "width": this.model.width = options[option]; this._setDimension();
                        if (this.model.showScale) this._scaleAlignment();
                        break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "roundedCorner": this._outerCorner(options[option]); break;
                    case "rtl": this.model.rtl = options[option]; this._checkRTL();
                        options[option] = this.model.rtl;
                        break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showScale": this._renderScale(options[option]); break;
                    case "orientation":
                        var t = this.model.height;
                        this.model.height = this.model.width;
                        this.model.width = t;
                    case "sliderType":
                        this._sliderOptions(option, options[option]); break;
                    case "smallStep":
                    case "largeStep":
                    case "showSmallTicks":
                    case "startValue":
                    case "endValue":
                        this._scaleOptions(option, options[option]); break;
                }
            }
        },
        /**
         * To configure destroy of the slider control.		
		 * @private
         */
        _destroy: function () {
            if (this.model.showScale) this._destroyScale();
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            this.element.removeClass("e-widget e-box " + this.model.cssClass).empty();
        },
        /**
         * To initialize the slider control position		
		 * @private
         */
        _initialize: function () {
            this.target = this.element[0];
            this.horDir = "left";
            this.verDir = "bottom";
            this._isFocused = false;
        },
        /**
         * Render Section For DifferentTypes		
		 * @private
         */
        _render: function () {
            this.wrapper = ej.buildTag("div.e-slider-wrap " + this.model.cssClass + "#" + this.target.id + "_wrapper", { tabindex: "0", role: "slider" })
                .insertAfter(this.element);
            this.wrapper.append(this.element);

            this.element.addClass("e-widget e-box " + this.model.cssClass);
            if (this.model.sliderType != "default") {
                this.header = ej.buildTag("div.e-range");
                this.element.append(this.header);
                if (this.model.sliderType == "range") {
                    this.secondHandle = this._createHandle();
                }
            }
            this.firstHandle = this._createHandle();
            this._setOrientation();
            this._setDimension();
            this._insertHiddenField();
            this._checkProperties();
            this._setSliderValue();
        },
        /**
         * Render Section For scale		
		 * @private
         */
        _renderScale: function (showScale) {
            if (showScale) {
                var width = "width", orien = "h";
                if (this.model.orientation == "vertical") {
                    width = "height";
                    orien = "v";
                }

                var _smallStep = this.model.smallStep;
                if (!this.model.showSmallTicks) {
                    if (this.model.largeStep > 0)
                        _smallStep = this.model.largeStep;
                    else
                        _smallStep = this.model.endValue - this.model.startValue;
                }
                else if (_smallStep <= 0)
                    _smallStep = this.model.step;
                var count = Math.abs(this.model.endValue - this.model.startValue) / _smallStep;

                this.ul = ej.buildTag("ul.e-scale e-" + orien + "-scale " + this.model.cssClass);
                this.wrapper.append(this.ul);

                var li, start = this.model.startValue, left = 0, tickWidth = 100 / count;
                if (orien == "v") start = this.model.endValue;
                for (var i = 0; i <= count; i++) {
                    li = ej.buildTag("li.e-tick", "", {}, { "title": start });
                    li.css(width, tickWidth + "%");

                    if (start % this.model.largeStep == 0) {
                        var span = ej.buildTag("span.e-tick-value", "" + start);
                        li.addClass("e-large").append(span);
                    }
                    this.ul.append(li);

                    if (orien == "h") start += _smallStep;
                    else start -= _smallStep;
                    left += _smallStep;
                }

                this.ul.children().first().addClass("e-first-tick").css(width, (tickWidth / 2) + "%");
                this.ul.children().last().addClass("e-last-tick").css(width, (tickWidth / 2) + "%");

                this._scaleAlignment();
            }
            else this._destroyScale();
        },
        _destroyScale: function () {
            this.ul.remove();
            this.ul = null;
        },
        /**
         * To configure tick value position		
		 * @private
         */
        _tickValuePosition: function () {
            var width = (this.model.orientation == "vertical") ? "height" : "width";
            var left = (this.model.orientation == "vertical") ? "top" : "left";

            var firstTick = this.ul.find(".e-tick.e-first-tick");
            var first = firstTick.find(".e-tick-value");
            var other = this.ul.find(".e-tick.e-large:not(.e-first-tick)").find(".e-tick-value");

            tickWidth = firstTick[width]() * 2;
            first.css(left, -first[width]() / 2);
            other.css(left, (tickWidth - other[width]()) / 2);
        },
        /**
         * To configure scale alignment in slider		
		 * @private
         */
        _scaleAlignment: function () {
            this._tickValuePosition();

            var smallTick = 12, largeTick = 20, half = largeTick / 2;
            var height = "height", top = "top", orien = "h";
            if (this.model.orientation == "vertical") {
                height = "width";
                top = "right";
                orien = "v";
            }
            // scale
            this.ul.css(top, -(this.wrapper[height]() + half));
            if (orien == "v") this.ul.css("top", -this.wrapper.height()).css(top, half);
            this.ul[height](this.wrapper[height]() + largeTick);
            // small-ticks
            var topSize = -(largeTick - smallTick) / 2;
            if (this.model.largeStep == null && orien != "v") topSize = -topSize;
            this.ul.find(".e-tick:not(.e-large)").css(height, this.wrapper[height]() + smallTick).css(top, topSize);
            // tick-values   // 4 - distance between tick value and tick
            if (orien == "v") this.ul.children(".e-large").find(".e-tick-value").css("right", this.wrapper.width() + largeTick + 4);
        },
        /**
         * To create handle for slider		
		 * @private
         */
        _createHandle: function () {
            handle = ej.buildTag("a.e-handle e-select", "", {}, { "aria-label": "drag", "tabindex": 0, role: "slider", "aria-valuemin": this.model.startValue, "aria-valuemax": this.model.endValue });
            this.element.append(handle);
            return handle;
        },
        /**
         * To configure dimension for slider		
		 * @private
         */
        _setDimension: function () {
            if (this.model.height) this.wrapper.height(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
            this._setHandleSize();
            this._handleAlignment(this.model.rtl);
        },
        /**
         * To configure scale alignment in slider		
		 * @private
         */
        _insertHiddenField: function () {
            this._hidden = ej.buildTag("input", "", {},
                { "type": "hidden", "name": this.element[0].id }).val(this._getHandleValue());
            this.element.append(this._hidden);
        },
        /**
         * To configure properties of the slider		
		 * @private
         */
        _checkProperties: function () {
            if (!this.model.enabled) {
                if (this.wrapper) this.wrapper.addClass("e-disable");
                else this.element.addClass("e-disable");
            }
            else this._wireEvents();

            if (this.model.showScale) this._renderScale(true);
            if (this.model.rtl) this._checkRTL();
            if (this.model.roundedCorner) this._roundedCorner();
        },
        /**
         * To configure the rounded corner behaviour 		
		 * @private
         */
        _roundedCorner: function () {
            this.element.addClass("e-corner-all");
            this.element.children(".e-handle").addClass("e-corner-all");
        },
        /**
         * To configure the shaped corner behaviour 		
		 * @private
         */
        _sharpedCorner: function () {
            this.element.removeClass("e-corner-all");
            this.element.children(".e-handle").removeClass("e-corner-all");
        },
        /**
         * To configure the handle alignment in slider control 		
		 * @private
         */
        _handleAlignment: function (rtl) {
            var mar = -(this.firstHandle.outerWidth() / 2) + "px", margin;
            if (this.model.orientation != "vertical") {
                if (!rtl) margin = "0 0 0 " + mar;
                else margin = "0 " + mar + " 0 0";
            }
            else {
                if (!rtl) margin = "0 0 " + mar + " 0";
                else margin = mar + " 0 0 0";
            }
            this.element.children('.e-handle').css("margin", margin);
        },
        /**
         * To check whether Right to Left Direction in slider control 		
		 * @private
         */
        _checkRTL: function () {
            if (this.model.showScale && this.model.orientation == "vertical" && this.model.rtl) this.model.rtl = false;
            var rtl = this.model.rtl, preDir = (this.model.orientation != "vertical") ? this.horDir : this.verDir;
            if (rtl) {
                this.wrapper.addClass("e-rtl");
                this.horDir = "right";
                this.verDir = "top";
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.horDir = "left";
                this.verDir = "bottom";
            }
            var currDir = (this.model.orientation != "vertical") ? this.horDir : this.verDir;

            if (preDir != currDir) {
                this.firstHandle.css(currDir, this.firstHandle.css(preDir)).css(preDir, "auto");;
                if (this.model.sliderType != "default") {
                    this.header.css(currDir, this.header.css(preDir)).css(preDir, "auto");
                    if (this.model.sliderType == "range")
                        this.secondHandle.css(currDir, this.secondHandle.css(preDir)).css(preDir, "auto");
                }
            }
            this._handleAlignment(rtl);
        },
        /**
         * To configure the direction orientation in slider control 		
		 * @private
         */
        _setOrientation: function () {
            if (this.model.orientation != "vertical") {
                this.wrapper.addClass("e-horizontal");
            }
            else {
                this.wrapper.addClass("e-vertical");
                this.firstHandle.css(this.verDir, "0");
            }
        },
        /**
         * To configure the handle size in slider control 		
		 * @private
         */
        _setHandleSize: function () {
            var size;
            if (this.model.orientation != "vertical")
                size = this.element.height() + 4;
            else
                size = this.element.width() + 4;
            this.element.find(".e-handle").height(size).width(size);
        },
        /**
         * To enable or disable the slider control state.
		 * @private
         */
        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },
        /**
         * To configure slider control with its options 		
		 * @private
         */
        _sliderOptions: function (prop, value) {
            this._unWireEvents();
            this._destroy();
            this.model[prop] = value;
            this._init();
        },
        /**
         * To configure the scale used in slider control 		
		 * @private
         */
        _scaleOptions: function (prop, value) {
            if (this.model.showScale) {
                this._destroyScale();
                this.model[prop] = value;
                this._renderScale(true);
            }
        },
        /**
         * To configure the showTooltip in slider control 		
		 * @private
         */
        _showTooltip: function () {
            if (this.model.showTooltip) {
                $('body .e-tooltip').remove();
                this.tooltip = ej.buildTag("div.e-tooltip " + this.model.cssClass + " e-corner-all", { role: "tooltip" });
                $("body").append(this.tooltip);
                this._setTooltipPosition();
            }
        },
        /**
         * To configure the hideTooltip slider control 		
		 * @private
         */
        _hideTooltip: function () {
            if (this.model.showTooltip)
                this.tooltip.fadeOut(800);
        },
        /**
         * To enable or disable the slider control tooltip.
		 * @private
         */
        _showhideTooltip: function (showTooltip) {
            if (this.model.showTooltip && showTooltip) {
                this._showTooltip();
                this._hideTooltip();
            }
        },
        /**
         * To configure the Tooltip position for handle slider control 		
		 * @private
         */
        _setTooltipPosition: function () {
            if (this.model.showTooltip) {
                this._updateTooltipValue();
                var top, left, remainLeft, remainTop, handle, gap = 5; // gap -> distance between tooltip and slider
                handle = this._getHandle();

                if (this.model.orientation == "vertical") {
                    remainTop = (this.tooltip.outerHeight() - handle.outerHeight()) / 2;
                    remainLeft = handle.outerWidth() + gap;
                    top = handle.offset().top - remainTop;
                    left = handle.offset().left + remainLeft;
                }
                else {
                    remainLeft = (this.tooltip.outerWidth() - handle.outerWidth()) / 2;
                    remainTop = this.tooltip.outerHeight() + gap;
                    top = handle.offset().top - remainTop;
                    left = handle.offset().left - remainLeft;
                }
                this.tooltip.css({ "top": top, "left": left });
            }
        },
        /**
         * To obtain the Tooltip value from handle in slider control 		
		 * @private
         */
        _updateTooltipValue: function () {
            var one = 0, two = 1;
            if (this.model.rtl) { one = 1, two = 0; }
            var val = this._getHandleValue();
            if (this.model.sliderType != "range")
                this.tooltip[0].innerHTML = val;
            else
                this.tooltip[0].innerHTML = val[one] + " - " + val[two];
        },
        /**
         * To configure the slider control header width based on it's type.		
		 * @private
         */
        _increaseHeaderWidth: function (animation) {
            if (this.model.sliderType != "default") {
                var size = "width", direction = this.horDir, properties = {};
                if (this.model.orientation == "vertical") { size = "height", direction = this.verDir; }

                if (this.model.sliderType == "range") {
                    properties[size] = this.handlePos - this.handlePos2 + "%";
                    properties[direction] = this.handlePos2 + "%";
                }
                else {
                    properties[size] = this.handlePos + "%";
                    properties[direction] = 0;
                }
                if (!animation) this.header.css(properties);
                else this.header.animate(properties, this.model.animationSpeed);
            }
        },
        /**
         * To configure the value based on it's slider type
		 * @private
         */
        _setSliderValue: function () {
            this._validateStartEnd();

            if (this.model.sliderType == "range")
                this._validateRangeValue(this.values());
            else
                this._validateValue(this.value());
        },

        /**
        * Section For handle the mouse hover event on slider handle.
        * @private
        */
        _hoverOnHandle: function (evt) {
            $(evt.target).addClass("e-hover");
        },
        /**
        * Section For handle the mouse leave event on slider handle.
        * @private
        */
        _leaveFromHandle: function (evt) {
            $(evt.target).removeClass("e-hover");
        },
        /**
        * Section For handle the click event on slider handle.
        * @private
        */
        _firstHandleClick: function (evt) {
            evt.preventDefault();
            this.firstHandle.focus();
            if (this._raiseEvent("start")) return false;

            this.mouseDownPos = this.handlePos;
            if (!this.model.readOnly)
                $(document).bind(ej.eventType.mouseMove, $.proxy(this._firstHandleMove, this));
            $(document).bind(ej.eventType.mouseUp, $.proxy(this._firstHandleUp, this));
            $(document).bind("mouseleave", $.proxy(this._firstHandleUp, this));
            this._showTooltip();
        },
        /**
        * Section For handle the drag event over slider handle.
        * @private
        */
        _firstHandleMove: function (evt) {
            evt.preventDefault();
            evt = evt.type == "touchmove" ? evt.originalEvent.changedTouches[0] : evt;
            var position = { x: evt.pageX, y: evt.pageY };
            this.handlePos = this._xyToPosition(position);

            if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                this.handlePos = this.handlePos2;
            }
            if (this.handlePos != this.preHandlePos) {
                this.preHandlePos = this.handlePos;
                this.handleVal = this._positionToValue(this.handlePos);
                this._increaseHeaderWidth(false);
                this._setHandlePosition(false, false, false);
                this._setTooltipPosition();

                this._updateModelValue();
                this._raiseEvent("slide");
            }
        },
        /**
        * Section For handle the mouse up event over slider handle.
        * @private
        */
        _firstHandleUp: function (evt) {
            evt.preventDefault();
            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._firstHandleMove, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._firstHandleUp, this));
            $(document).unbind("mouseleave", $.proxy(this._firstHandleUp, this));
            this._hideTooltip();

            if (this.mouseDownPos != this.handlePos) this._raiseChangeEvent();
            this._raiseEvent("stop");
        },
        /**
        * Section For handle the click event on range slider second handle.
        * @private
        */
        _secondHandleClick: function (evt) {
            evt.preventDefault();
            this.secondHandle.focus();
            if (this._raiseEvent("start")) return false;

            this.mouseDownPos2 = this.handlePos2;
            if (!this.model.readOnly)
                $(document).bind(ej.eventType.mouseMove, $.proxy(this._secondHandleMove, this));
            $(document).bind(ej.eventType.mouseUp, $.proxy(this._secondHandleUp, this));
            $(document).bind("mouseleave", $.proxy(this._secondHandleUp, this));
            this._showTooltip();
        },
        /**
        * Section For handle the drag event over range slider second handle.
        * @private
        */
        _secondHandleMove: function (evt) {
            evt.preventDefault();
            evt = evt.type == "touchmove" ? evt.originalEvent.changedTouches[0] : evt;
            var position2 = { x: evt.pageX, y: evt.pageY };
            this.handlePos2 = this._xyToPosition(position2);

            if (this.handlePos2 > this.handlePos) {
                this.handlePos2 = this.handlePos;
            }
            if (this.handlePos2 != this.preHandlePos2) {
                this.preHandlePos2 = this.handlePos2;
                this.handleVal2 = this._positionToValue(this.handlePos2);
                this._increaseHeaderWidth(false);
                this._setHandlePosition(false, false, false);
                this._setTooltipPosition();

                this._updateModelValue();
                this._raiseEvent("slide");
            }
        },
        /**
        * Section For handle the mouse up event on range slider second handle.
        * @private
        */
        _secondHandleUp: function (evt) {
            evt.preventDefault();
            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._secondHandleMove, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._secondHandleUp, this));
            $(document).unbind("mouseleave", $.proxy(this._secondHandleUp, this));
            this._hideTooltip();

            if (this.mouseDownPos2 != this.handlePos2) this._raiseChangeEvent();
            this._raiseEvent("stop");
        },
        /**
        * Section For handle the focus in event on slider handle.
        * @private
        */
        _focusInHandle: function (evt) {
            if (!this._isFocused) {
                this._isFocused = true;
                $(evt.target).addClass("e-focus");
                if (!this.model.readOnly)
                    $(document).bind("keydown", $.proxy(this._moveHandle, this));
                this.activeHandle = $(evt.target).is(this.firstHandle) ? 1 : 2;
                this._setZindex();
            }
        },
        /**
        * Section For handle the focus out event on slider handle.
        * @private
        */
        _focusOutHandle: function (evt) {
            this._isFocused = false;
            $(evt.target).removeClass("e-focus");
            $(document).unbind("keydown", $.proxy(this._moveHandle, this));
        },
        /**
        * Section For handle the slider handle move.
        * @private
        */
        _moveHandle: function (e) {
            var oper, val, handleNo;
            handleNo = this._getHandleIndex(this.activeHandle) - 1;

            switch (e.keyCode) {
                case 37:        // Left Key
                case 40:        // Down Key
                    e.preventDefault();
                    oper = "sub";
                    break;
                case 38:        // Up Key
                case 39:         // Right Key
                    e.preventDefault();
                    oper = "add";
                    break;
                case 36:         // Home Key
                    e.preventDefault();
                    if (this._raiseEvent("start")) return false;
                    if (this.model.sliderType != "range" && this.value() != this.model.startValue) {
                        this._changeHandleValue(this.model.startValue, this.model.animate);
                    }
                    else if (this.model.sliderType == "range") {
                        val = (this.activeHandle == 2) ? this.model.startValue : this.handleVal2;
                        if (this.values()[handleNo] != val)
                            this._changeHandleValue(val, this.model.animate);
                    }
                    break;
                case 35:         // End Key
                    e.preventDefault();
                    if (this._raiseEvent("start")) return false;
                    if (this.model.sliderType != "range" && this.value() != this.model.endValue) {
                        this._changeHandleValue(this.model.endValue, this.model.animate);
                    }
                    else if (this.model.sliderType == "range") {
                        val = (this.activeHandle == 1) ? this.model.endValue : this.handleVal;
                        if (this.values()[handleNo] != val)
                            this._changeHandleValue(val, this.model.animate);
                    }
                    break;
                case 27:         // Esc Key
                    e.preventDefault();
                    this._getHandle().focusout();
                    break;
            }

            if (oper == "add" || oper == "sub") {
                if (this._raiseEvent("start")) return false;
                var hVal = (this.activeHandle == 1) ? this.handleVal : this.handleVal2;
                var value = (oper == "add") ? this._add(hVal, this.model.step, true) : this._add(hVal, this.model.step, false);
                this._changeHandleValue(value, false);
            }
        },
        /**
        * Section For handle the slider handle value change.
        * @private
        */
        _changeHandleValue: function (value, animate) {
            var position = null;
            if (this.activeHandle == 1) {
                this.handleVal = this._checkHandleValue(value);
                this.handlePos = this._checkHandlePosition(this.handleVal);

                if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                    this.handlePos = this.handlePos2;
                    this.handleVal = this.handleVal2;
                }
                if (this.handlePos != this.preHandlePos)
                    position = this.preHandlePos = this.handlePos;
            }
            else {
                this.handleVal2 = this._checkHandleValue(value);
                this.handlePos2 = this._checkHandlePosition(this.handleVal2);

                if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                    this.handlePos2 = this.handlePos;
                    this.handleVal2 = this.handleVal;
                }
                if (this.handlePos2 != this.preHandlePos2)
                    position = this.preHandlePos2 = this.handlePos2;
            }

            if (position != null) {
                this._increaseHeaderWidth(animate);
                this._setHandlePosition(animate, true, true);
            }
        },
        /**
        * Section For handle the slider bar click event.
        * @private
        */
        _sliderBarClick: function (evt) {
            if (this.model.readOnly) return false;
            if (evt.target == this.target || (this.model.sliderType != "default" && evt.target == this.header[0])) {
                evt.preventDefault();
                if (this._raiseEvent("start")) return false;
                var pos = { x: evt.pageX, y: evt.pageY },
                handlepos = this._xyToPosition(pos),
                handleVal = this._positionToValue(handlepos);

                if (this.model.sliderType == "range" && (this.handlePos - handlepos) > (handlepos - this.handlePos2)) {
                    this.handlePos2 = this.preHandlePos2 = handlepos;
                    this.handleVal2 = handleVal;
                    this.activeHandle = 2;
                }
                else {
                    this.handlePos = this.preHandlePos = handlepos;
                    this.handleVal = handleVal;
                    this.activeHandle = 1;
                }

                this._getHandle().focus();
                if (this.model.sliderType != "default") this._increaseHeaderWidth(this.model.animate);
                this._setHandlePosition(this.model.animate, true, true);
            }
        },
        /**
        * To configure the slider handle position.
        * @private
        */
        _setHandlePosition: function (animation, showTooltip, changeEvt) {
            var Handle = this._getHandle(), proxy = this, properties = {}, pos, val, direction;
            pos = (this.activeHandle == 1) ? this.handlePos : this.handlePos2;
            val = (this.activeHandle == 1) ? this.handleVal : this.handleVal2;
            Handle.attr("aria-label", val);
            direction = (this.model.orientation == "vertical") ? this.verDir : this.horDir;
            properties[direction] = pos + "%";

            if (!animation) {
                Handle.css(properties);
                this._showhideTooltip(showTooltip);
                if (changeEvt) this._raiseChangeEvent();
            }
            else Handle.animate(properties, this.model.animationSpeed, function () {
                proxy._showhideTooltip(showTooltip);
                if (changeEvt) proxy._raiseChangeEvent();
            });
        },
        /**
        * Section For handle the X,Y co-ordinate position for slider.
        * @private
        */
        _xyToPosition: function (position) {
            if (this.model.startValue == this.model.endValue)
                return 100;
            if (this.model.orientation != "vertical") {
                var left = position.x - this.element.offset().left,
                num = this.element.width() / 100,
                val = (left / num);
            }
            else {
                var top = position.y - this.element.offset().top,
                num = this.element.height() / 100,
                val = 100 - (top / num);
            }
            val = this._stepValueCalculation(val);
            if (val < 0) val = 0;
            else if (val > 100) val = 100;
            if (this.model.rtl) return 100 - val;
            return val;
        },
        /**
        * Section For handle the slider handle value updation.
        * @private
        */
        _updateValue: function () {
            this.handleVal = this._checkHandleValue(this.value());
            this.handlePos = this._checkHandlePosition(this.handleVal);
            this.preHandlePos = this.handlePos;
            this.activeHandle = 1;
        },
        /**
        * Section For handle the slider handle operations.
        * @private
        */
        _setValue: function () {
            this._updateValue();

            this._increaseHeaderWidth(this.model.animate);
            this._setHandlePosition(this.model.animate, false, true);
        },
        /**
        * Section For handle slider range values.
        * @private
        */
        _updateRangeValue: function () {
            var values = this.values();
            this.handleVal = this._checkHandleValue(values[1]);
            this.handleVal2 = this._checkHandleValue(values[0]);
            this.handlePos = this._checkHandlePosition(this.handleVal);
            this.handlePos2 = this._checkHandlePosition(this.handleVal2);

            if (this.handlePos < this.handlePos2) {
                this.handlePos = this.handlePos2;
                this.handleVal = this.handleVal2;
            }
            this.preHandlePos = this.handlePos;
            this.preHandlePos2 = this.handlePos2;
        },
        /**
        * To configure slider range values.
        * @private
        */
        _setRangeValue: function () {
            this._updateRangeValue();
            this._increaseHeaderWidth(this.model.animate);
            this.activeHandle = 1;
            this._setHandlePosition(this.model.animate, false, false);
            this.activeHandle = 2;
            this._setHandlePosition(this.model.animate, false, true);
        },
        /**
         * To Check whether slider handle position.
		 * @private
         */
        _checkHandlePosition: function (value) {
            if (this.model.startValue == this.model.endValue)
                return 100;
            var handle = this._tempStartEnd();
            if (value >= handle.start && value <= handle.end)
                value = (100 * (value - this.model.startValue)) / (this.model.endValue - this.model.startValue);
            else if (value < handle.start) value = 0;
            else value = 100;
            return value;
        },
        /**
         * To Check whether slider handle values.
		 * @private
         */
        _checkHandleValue: function (value) {
            if (this.model.startValue == this.model.endValue)
                return this.model.startValue;
            var handle = this._tempStartEnd();
            if (value < handle.start) value = handle.start;
            else if (value > handle.end) value = handle.end;
            return value;
        },
        /**
         * To configure slider starting and ending values.
		 * @private
         */
        _tempStartEnd: function () {
            if (this.model.startValue > this.model.endValue)
                return {
                    start: this.model.endValue,
                    end: this.model.startValue
                };
            else
                return {
                    start: this.model.startValue,
                    end: this.model.endValue
                };
        },
        /**
         * To configure slider values based on slider handle present position.
		 * @private
         */
        _positionToValue: function (pos) {
            var diff = this.model.endValue - this.model.startValue,
            val = this._round(diff * pos / 100),
            total = this._add(val, this.model.startValue, true);
            return total;
        },
        /**
         * To obtained active slider handle value.
		 * @private
         */
        _getHandle: function () {
            return (this.activeHandle == 1) ? this.firstHandle : this.secondHandle;
        },
        /**
         * To obtained slider handle index value.
		 * @private
         */
        _getHandleIndex: function (no) {
            if (this.model.sliderType == "range" && no == 1)
                return 2;
            return 1;
        },
        /**
         * To obtained slider handle values based on it's type.
		 * @private
         */
        _getHandleValue: function () {
            if (this.model.sliderType == "range") return [this.handleVal2, this.handleVal];
            else return this.handleVal;
        },
        /**
         * To configure the slider value based on it's type.
		 * @private
         */
        _updateModelValue: function () {
            var value = this._getHandleValue();
            if (this.model.sliderType == "range") this.values(value);
            else this.value(value);
            this._hidden.val(value);
        },
        /**
         * To configure slider value.
		 * @private
         */
        _add: function (a, b, addition, precision) {
            var x = Math.pow(10, precision || 3), val;
            if (addition) val = (Math.round(a * x) + Math.round(b * x)) / x;
            else val = (Math.round(a * x) - Math.round(b * x)) / x;
            return val;
        },
        /**
         * To configure slider value as rounded.
		 * @private
         */
        _round: function (a) {
            var _f = this.model.step.toString().split(".");
            return _f[1] ? parseFloat(a.toFixed(_f[1].length)) : Math.round(a);
        },
        /**
         * To configure slider change event.
		 * @private
         */
        _raiseChangeEvent: function () {
            this._updateModelValue();
            this._raiseEvent("change");
        },
        /**
         * Render section for raising event in slider .
		 * @private
         */
        _raiseEvent: function (name) {
            if (this.model[name]) {
                return this._trigger(name, {
                    id: this.target.id,
                    value: this._getHandleValue(),
                    handleNo: this._getHandleIndex(this.activeHandle)
                });
            }
            return false;
        },
        /**
         * To configure slider handle z-index value.
		 * @private
         */
        _setZindex: function () {
            if (this.model.sliderType == "range") {
                if (this.activeHandle == 1) {
                    this.firstHandle.css("z-index", 2);
                    this.secondHandle.css("z-index", 1);
                }
                else {
                    this.firstHandle.css("z-index", 1);
                    this.secondHandle.css("z-index", 2);
                }
            }
        },
        /**
         * To obtained slider handle value based on step value.
		 * @private
         */
        _stepValueCalculation: function (value) {
            if (this.model.step == 0) this.model.step = 1;
            var percentStep = this.model.step / ((this.model.endValue - this.model.startValue) / 100);
            var remain = value % Math.abs(percentStep);
            if (remain != 0) {
                if ((percentStep / 2) > remain) value -= remain;
                else value += Math.abs(percentStep) - remain;
            }
            return value;
        },

        //-------------------- Event Wire-up -------------------------//
        /**
         * Wiring the events to slider control		
		 * @private
         */
        _wireEvents: function () {
            this._on(this.element, "mousedown", this._sliderBarClick);
            this._on(this.firstHandle, ej.eventType.mouseDown, this._firstHandleClick);
            this._on(this.firstHandle, "mouseenter", this._hoverOnHandle);
            this._on(this.firstHandle, "mouseleave", this._leaveFromHandle);
            this._on(this.firstHandle, "focus", this._focusInHandle);
            this._on(this.firstHandle, "focusout", this._focusOutHandle);

            if (this.model.sliderType == "range") {
                this._on(this.secondHandle, ej.eventType.mouseDown, this._secondHandleClick);
                this._on(this.secondHandle, "mouseenter", this._hoverOnHandle);
                this._on(this.secondHandle, "mouseleave", this._leaveFromHandle);
                this._on(this.secondHandle, "focus", this._focusInHandle);
                this._on(this.secondHandle, "focusout", this._focusOutHandle);
            }
        },
        //-------------------- Event Wire-up -------------------------//
        /**
         * unWiring the events from slider control.		
		 * @private
         */
        _unWireEvents: function () {
            this._off(this.element, "mousedown");
            this._off(this.firstHandle, ej.eventType.mouseDown);
            this._off(this.firstHandle, "mouseenter");
            this._off(this.firstHandle, "mouseleave");
            this._off(this.firstHandle, "focus");
            this._off(this.firstHandle, "focusout");

            if (this.model.sliderType == "range") {
                this._off(this.secondHandle, ej.eventType.mouseDown);
                this._off(this.secondHandle, "mouseenter");
                this._off(this.secondHandle, "mouseleave");
                this._off(this.secondHandle, "focus");
                this._off(this.secondHandle, "focusout");
            }
        }
    });
    /**
	 * Enum for slider type.	 
	 * @enum {string}
	 * @global 
	 */
    ej.sliderType = {
        /**  support for slider control to select a single value. */
        Default: "default",
        /**  support for slider control to select a single value considered from start value to current handle. */
        MinRange: "minrange",
        /**  support for slider control to select a range of value between the two handles. */
        Range: "range"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
       * @namespace ej
       * @class ejSplitButton
       * @requires jQuery
       * @requires ej.common.all
       * @classdesc Custom Design for Html Button control.
       * @example 
       *&lt;button id="sbutton"&gt;Save&lt;/button&gt;
       *&lt;ul id="target"&gt;
       *    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
       *    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
       *    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
       *&lt;/ul&gt;
       * &lt;script&gt;
       * // simple control creation
       *  $("#sbutton").ejSplitButton({targetId: "target",});
       * &lt;/script&gt;
       */

    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejSplitButton", "ej.SplitButton", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["button"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-splitbutton",

        // default model
        defaults: {
            /// <summary>This Contains default property of Split button </summary>
            /**		
			* Specifies the size of the Button.	See {@link buttonSize}
			* @default ej.buttonSize.normal
			* @type {String | Enum}
			* @example 
			* //To set size API value during initialization  
			* 	$("#sbutton").ejSplitButton({  size: ej.buttonSize.mini});				* 
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#sbutton").ejSplitButton("option", "size");			
			*	//Sets the size value 
			*	$("#sbutton").ejSplitButton("option", "size",  ej.buttonSize.large ); 
			 * @memberof ejSplitButton
			* @instance
			*/
            size: "normal",
            /**		
			* Specifies the width of the Split Button.
			* @default “28” pixel
			* @type {String | Number}
			* @example 
			* //To set width API value during initialization  
			* $("#sbutton").ejSplitButton({ width: 28 });			* 
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value  
			*	$("#sbutton").ejSplitButton("option", "width");		
			*	//Sets the width value 
			*	$("#sbutton").ejSplitButton("option", "width", 28 ); 
			 * @memberof ejSplitButton
			* @instance
			*/
            width: "",
            /**		
			* Specifies the height of the Split Button.
			* @default “28” pixel
			* @type {String | Number}
			* @example 
			* //To set height API value during initialization  
			* $("#sbutton").ejSplitButton({ height: 28 });			* 
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value  
			*	$("#sbutton").ejSplitButton("option", "height");		
			*	//Sets the height value 
			*	$("#sbutton").ejSplitButton("option", "height", 28 ); 
			 * @memberof ejSplitButton
			* @instance
			*/
            height: "",
            /**		
			*Specifies the disabling of Split Button if enabled is set to false.
			* @default true
			* @type {Boolean}
			* @example 
			* //To set enabled API value during initialization  
			* $("#sbutton").ejSplitButton({ enabled:  true });		* 
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the enabled value  
			*	$("#sbutton").ejSplitButton("option", "enabled");		
			*	//Sets the enabled value 
			*	$("#sbutton").ejSplitButton("option", "enabled",  true ); 
			 * @memberof ejSplitButton
			* @instance
			*/
            enabled: true,
            /**		
			*Specifies the text content for Split Button while initialization.
			* @default ""
			* @type {String}
			* @example 
			* //To set text API value during initialization  
			* $("#sbutton").ejSplitButton({  text: "Hello Word" });		 
			* @example 
			* //Get or set the text API, after initialization:<br>
			*	//Gets the text value  
			*	$("#sbutton").ejSplitButton("option", "text");		
			*	//Sets the text value 
			*	$("#sbutton").ejSplitButton("option", "text",  "click" ); 
			 * @memberof ejSplitButton
			* @instance
			*/
            text: null,
            /**		
			*Specifies the contentType of the Split Button.
			* @default ej.contents.textOnly
			* @type {String | Enum}
			* @example 
			* //To set contentType API value during initialization  
			* $("#sbutton").ejSplitButton({ contentType:  ej.contents.textOnly}); 
			* @example 
			* //Get or set the contentType API, after initialization:<br>
			*	//Gets the contentType value  
			*	$("#sbutton").ejSplitButton("option", "contentType");	
			*	//Sets the contentType value 
			*	$("#sbutton").ejSplitButton("option", "contentType", ej.contents.imageOnly ); 
			 * @memberof ejSplitButton
			* @instance
			*/
            contentType: "textonly",
            /**		
			*Specifies the imagePosition of the Split Button.
			* @default ej.imagePositions.imageRight
			* @type {String | Enum}
			* @example 
			* //To set imagePositions API value during initialization  
			* $("#sbutton").ejSplitButton({ contentType: ej.contents.textAndImage,
            *   imagePosition: ej.imagePositions.imageRight});
			* @example 
			* //Get or set the imagePositions API, after initialization:<br>
			*	//Gets the imagePositions value  
			*	$("#sbutton").ejSplitButton("option", "imagePosition");
			*	//Sets the imagePositions value 
			*	$("#sbutton").ejSplitButton("option",
            *               "imagePosition", ej.imagePositions.imageRight); 
			* @memberof ejSplitButton
			* @instance
			*/
            imagePosition: "imageleft",
            /**		
			*Specifies the list content for Split Button while initialization
			* @default ""
			* @type {String}
			* @example 
			* //To set targetId API value during initialization  
			* $("#sbutton").ejSplitButton({ targetId: "list1" });
			* @example 
			* //Get or set the targetId API, after initialization:<br>
			*	//Gets the targetId value  
			*	$("#sbutton").ejSplitButton("option", "targetId");
			*	//Sets the targetId value 
			*	$("#sbutton").ejSplitButton("option", "targetId",  "list2" ); 
			* @memberof ejSplitButton
			* @instance
			*/
            targetId: null,
            /**		
			*Specifies the roundedCorner property for Split Button while initialization.
			* @default ""
			* @type {String}
			* @example 
			* //To set roundedCorner API value during initialization  
			* $("#sbutton").ejSplitButton({ roundedCorner: false });
			* @example 
			* //Get or set the roundedCorner API, after initialization:<br>
			*	//Gets the roundedCorner value  
			*	$("#sbutton").ejSplitButton("option", "roundedCorner");
			*	//Sets the roundedCorner value 
			*	$("#sbutton").ejSplitButton("option", "roundedCorner",  true );  
			* @memberof ejSplitButton
			* @instance
			*/
            roundedCorner: false,
            /**		
			*Specifies the image content for Split Button while initialization.
			* @default ""
			* @type {String}
			* @example 
			* //To set prefixIcon API value during initialization  
			* $("#sbutton").ejSplitButton({contentType: "imageonly",prefixIcon: "e-uiLight e-icon-handup"              });
			* @example 
			* //Get or set the prefixIcon API, after initialization:<br>
			*	//Gets the prefixIcon value  
			*	$("#sbutton").ejSplitButton("option", "prefixIcon", "e-uiLight e-icon-handup"); 
			*	//Sets the prefixIcon value 
			*	$("#sbutton").ejSplitButton("option", "prefixIcon");
 			* @memberof ejSplitButton
			* @instance
			*/
            prefixIcon: null,
            /**		
			*Specifies the image content for Split Button while initialization.
			* @default ""
			* @type {String}
			* @example 
			* //To set suffixIcon API value during initialization  
			* $("#sbutton").ejSplitButton({contentType: "imageboth",prefixIcon: "e-uiLight e-icon-handup",suffixIcon: "e-uiLight e-icon-padlockclosed"});
			* @example 
			* //Get or set the suffixIcon API, after initialization:<br>
			*	//Gets the suffixIcon value  
			*	$("#sbutton").ejSplitButton("option", "suffixIcon");
			*	//Sets the suffixIcon value 
			*	$("#sbutton").ejSplitButton("option","suffixIcon",  "e-uiLight e-icon-padlockclose
 			* @memberof ejSplitButton
			* @instance
			*/
            suffixIcon: null,
            /**		
			*Set the root class for Split Button control theme
			* @default ""
			* @type {String}
			* @example 
			* //To set cssClass API value during initialization  
			* $("#sbutton").ejSplitButton({cssClass: "gradient-lime"});
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the cssClass value  
			*	$("#sbutton").ejSplitButton("option", "cssClass");
			*	//Sets the cssClass value 
			*	$("#sbutton").ejSplitButton("option", "cssClass", "gradient-lime");
 			* @memberof ejSplitButton
			* @instance
			*/
            cssClass: "",
            /**		
			*Specifies the rtl property for Split Button while initialization.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set rtl API value during initialization  
			* $("#sbutton").ejSplitButton({cssClass: "gradient-lime"});
			* @example 
			* //Get or set the rtl API, after initialization:<br>
			*	//Gets the rtl value  
			*	$("#sbutton").ejSplitButton("option", "rtl");
			*	//Sets the rtl value 
			*	$("#sbutton").ejSplitButton("option", "rtl",  true ); 
 			* @memberof ejSplitButton
			* @instance
			*/
            rtl: false,
            /**     
			 * Fires after Split Button control is created.
			 * @event
			 * @name ejSplitButton#create		
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the splite button model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * //create event for split button
             * $("#sbutton"). ejSplitButton ({
             *      create: function (args) {}
             * });
			 * @memberof ejSplitButton
			 * @instance
			 */
            create: null,
            /**     
			 * Fires when Button control is clicked successfully
			 * @event
			 * @name ejSplitButton#click		
			 * @param {Object} argument Event parameters from split button     
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the split button model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.target  returns the target of the current object.
			 * @param {boolean}  argument.status return the button state
			 * @example 
			 * //click event for split button
             * $("#sbutton"). ejSplitButton ({
             *      click: function (args) {}
             * });
			 * @memberof ejSplitButton
			 * @instance
			 */
            click: null,
            /**     
			 * Fires when a menu item is Hovered in successfully
			 * @event
			 * @name ejSplitButton#itemMouseOver		
			 * @param {Object} argument Event parameters from split button     
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the split button model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.event.element  returns the clicked menu item element
			 * @param {object} argument.event returns the event
			 * @param {String}  argument.event.menuId  return the menu id
			 * @param {String}  argument.event.menuText  return the clicked menu item text
			 * @example 
			 * //itemMouseOver event for split button
             * $("#sbutton"). ejSplitButton ({
             *      itemMouseOver: function (args) {}
             * });
			 * @memberof ejSplitButton
			 * @instance
			 */
            itemMouseOver: null,
            /**     
			 * Fires when a menu item is Hovered out successfully
			 * @event
			 * @name ejSplitButton#itemMouseOut		
			 * @param {Object} argument Event parameters from split button     
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the split button model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.event.element  returns the clicked menu item element
			 * @param {object} argument.event returns the event
			 * @param {String}  argument.event.menuId  return the menu id
			 * @param {String}  argument.event.menuText  return the clicked menu item text
			 * @example 
			 * //itemMouseOut event for split button
             * $("#sbutton"). ejSplitButton ({
             *      itemMouseOut: function (args) {}
             * });
			 * @memberof ejSplitButton
			 * @instance
			 */
            itemMouseOut: null,
            /**     
            * Fires when a menu item is clicked successfully
            * @event
            * @name ejSplitButton#itemSelected		
            * @param {Object} argument Event parameters from split button     
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the split button model
            * @param {string}  argument.type returns the name of the event
            * @param {object} argument.event.element  returns the clicked menu item element
            * @param {object} argument.selectedItem - returns the selected item
            * @param {String}  argument.event.menuId  return the menu id
            * @param {String}  argument.event.menuText  return the clicked menu item text
            * @example 
            * //itemSelected event for split button
            * $("#sbutton"). ejSplitButton ({
            *      itemSelected: function (args) {}
            * });
            * @memberof ejSplitButton
            * @instance
            */
            itemSelected: null,
            /**     
            * Fires when the Split Button is destroyed successfully
            * @event
            * @name ejSplitButton#destroy		
            * @param {Object} argument Event parameters from  split button     
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the split button model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //destroy event for split button
            * $("#sbutton"). ejSplitButton ({
            *      destroy: function (args) {}
            * });
            * @memberof ejSplitButton
            * @instance
            */
            destroy: null
        },
        //Data Types
        dataTypes: {
            size: "string",
            height: "string",
            width: "string",
            enabled: "boolean",
            roundedCorner: "boolean",
            text: "string",
            contentType: "enum",
            imagePosition: "enum",
            targetId: "string",
            prefixIcon: "string",
            suffixIcon: "string",
            cssClass: "string",
            rtl: "boolean"
        },
        // sample public function
        /**
        * To disable the split button  
		* @return jQuery
		* @example 
		* *&lt;button id="sbutton"&gt;Save&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* //To Disable the Split Button control.
		* $("#sbutton").ejSplitButton("disable");
		* &lt;/script&gt;
		*@memberof ejSplitButton
		* @instance
        */
        disable: function () {
            /// <summary>This function is  used to Disable the Split button Object</summary>
            this.element.addClass("e-disable");
            this.dropbutton.addClass("e-disable").attr("aria-disabled", true);
            this.model.enabled = false;
        },
        /**
        * To Enable the split button  
		* @return jQuery
		* @example 
		* *&lt;button id="sbutton"&gt;Save&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* //To Enable the Split Button control.
		* $("#sbutton").ejSplitButton("enable");
		* &lt;/script&gt;
		*@memberof ejSplitButton
		* @instance
        */
        enable: function () {
            /// <summary>This function is  used to Enable the Split button Object</summary>
            this.element.removeClass("e-disable");
            this.dropbutton.removeClass("e-disable").attr("aria-disabled", false);
            this.model.enabled = true;
        },
        // constructor function
        /**
         * Create the split button widget
		 * @private
         */
        _init: function () {
            this._initialize();
            this._controlStatus(this.model.enabled);
            this._wireEvents();
        },
        // all events bound using this._on will be unbind automatically
        /**
        * destroy the split button widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* *&lt;button id="sbutton"&gt;Save&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* // to destroy the button
		* $("#sbutton").ejSplitButton("destroy");	
		* &lt;/script&gt;
		* @memberof ejSplitButton
		* @instance
         */
        _destroy: function () {
            /// <summary>This function is  used to destroy the Split button Object</summary>
            this.element.removeClass(this.model.cssClass + " e-select e-corner-left e-btn e-disable e-split-btn");
            /*Remove dropbtn*/
            this.dropbutton.remove();
            $("#" + this.model.targetId).ejMenu('destroy');
            //this has to be worked out in Menu
            $("#" + this.model.targetId).insertAfter(this.element);
            $("#" + this.model.targetId).show();
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "width":
                        this._splitbtnWidth(options[option]);
                        break;
                    case "height":
                        this._splitbtnHeight(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "text":
                        this._setText(options[option]);
                        break;
                    case "prefixIcon":
                        this._setMajorIcon(options[option]);
                        break;
                    case "suffixIcon":
                        this._setMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "targetId":
                        this._setTargetId(options[option]);
                        break;
                    case "roundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "rtl":
                        this._setRTL(options[option]);
                        $("#" + this.model.targetId).ejMenu({ rtl: options[option] });
                        break;
                    case "itemMouseOver":
                        $("#" + this.model.targetId).ejMenu({ mouseOver: options[option] });
                        break;
                    case "itemMouseOut":
                        $("#" + this.model.targetId).ejMenu({ mouseOut: options[option] });
                        break;
                    case "itemSelected":
                        $("#" + this.model.targetId).ejMenu({ click: options[option] });
                        break;
                }
            }
        },
        /**
         * To configure the button text		
		 * @private
         */
        _setText: function (val) {
            /// <summary>This will set Text property of split button </summary>
            if (this.model.contentType == ej.contents.textOnly) {
                this.element.html(val);
            } else {
                this.textspan.html(val);
            }
        },
        /**
         * To configure button's prefix image		
		 * @private
         */
        _setMajorIcon: function (val) {
            /// <summary>This will set Major Icon property of split button </summary>
            this.majorimgtag.removeClass(this.model.prefixIcon);
            this.majorimgtag.addClass(val);
        },
        /**
         * To configure button's suffix image		
		 * @private
         */
        _setMinorIcon: function (val) {
            /// <summary>This will set Minor Icon property of split button </summary>
            this.minorimgtag.removeClass(this.model.suffixIcon);
            this.minorimgtag.addClass(val);
        },
        _setTargetId: function (val) {
            this.model.targetId = val;
            this._renderContxtMenu();
        },
        /**
         * To configure button contents type		
		 * @private
         */
        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.element.empty();
                this.model.contentType = val;
                this._renderButtonContent();
            }
        },
        /**
         * To configure images position inside the button		
		 * @private
         */
        _setImagePosition: function (val) {
            if ((this.model.contentType == ej.contents.textAndImage) && (val != this.model.imagePosition)) {
                this.element.empty();
                this.model.imagePosition = val;
                this._renderButtonContent();
            }
        },
        /**
         * To enable or disable the Right to Left behaviour 		
		 * @private
         */
        _setRTL: function (val) {
            if (val) {
                this.splitwrap.addClass("e-rtl e-btnrtl");
                this.dropbutton.css("border-right", "medium none");
                this.dropbutton.css("border-left-style", "");
            } else {
                this.splitwrap.removeClass("e-rtl e-btnrtl");
                this.dropbutton.css("border-left", "medium none");
                this.dropbutton.css("border-right-style", "");
            }
            this.model.rtl = val;
            this._roundedCorner(this.model.roundedCorner);
        },
        /**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */
        _roundedCorner: function (value) {
            if (value) {
                if (this.model.rtl) {
                    this.element.addClass("e-corner-right");
                    this.dropbutton.addClass("e-corner-left");

                    this.element.removeClass('e-corner-left');
                    this.dropbutton.removeClass('e-corner-right');
                }
                else {
                    this.element.addClass("e-corner-left");
                    this.dropbutton.addClass("e-corner-right");
                    this.element.removeClass("e-corner-right");
                    this.dropbutton.removeClass("e-corner-left");
                }
            }
            else {
                if (this.model.rtl) {
                    this.element.removeClass("e-corner-right");
                    this.dropbutton.removeClass("e-corner-left");
                }
                else {
                    this.element.removeClass('e-corner-left');
                    this.dropbutton.removeClass('e-corner-right');
                }
            }
        },
        /**
         * To configure enable/disable the splitbutton	
		 * @private
         */
        _controlStatus: function (value) {
            //disable status
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },
        /**
        * To configure the custom theme for button using cssClass property		
        * @private
        */
        _setSkin: function (skin) {
            this.element.removeClass(this.model.cssClass);
            this.dropbutton.removeClass(this.model.cssClass);
            this.element.addClass(skin);
            this.dropbutton.addClass(skin);
            $('#' + this.model.targetId).ejMenu('option', 'skin', skin);
        },
        /**
         * To initialize the button		
		 * @private
         */
        _initialize: function () {
            /// <summary>This function is  used to Initialize the Split button Object</summary>
            if (this.element.is("button")) {
                this._render();
            } else {
                this.element.removeClass("e-splitbutton");//need to change in src level
            }
            this._timeout = null;
        },
        /*Render Section For DifferentTypes*/
        /**
         * Render Section For DifferentTypes		
		 * @private
         */
        _render: function () {
            /// <summary>This function is  used to Render the Split button Object</summary>
            this.element.addClass(this.model.cssClass + "e-btn e-select e-split-btn").attr("role", "button");
            if ((this.model.text == null) || (this.model.text == "")) {
                this.model.text = this.element.text();
            }
            else
                this.element.attr("aria-describedby", this.model.text);
            this.element.empty();
            /*Creating Wrapper*/
            this.splitwrap = ej.buildTag('span.e-split e-widget');
            this.splitwrap.insertBefore(this.element);
            this.innerWrap = ej.buildTag('span.e-in-wrap e-box e-padding');
            this.splitwrap.append(this.innerWrap);
            this.wrapper = this.splitwrap;
            /*DropDown Button*/
            this.dropbutton = ej.buildTag('button.e-split-btn e-btn e-select ' + this.model.cssClass + ' e-drp-btn#' + this.element[0].id + 'drpbtn', "", {}, { type: "button" });
            this.dropdownimg = ej.buildTag('span.e-icon e-down-arrow');
            this.btnimgwrap = ej.buildTag('div.e-split-btn-div');
            this.btnimgwrap.append(this.dropdownimg);
            this.dropbutton.append(this.btnimgwrap);
            this.dropbutton.insertAfter(this.element);
            if (this.model.contentType == ej.contents.textOnly) {
                this.dropbutton.addClass("e-btn-txt");
            } else {
                this.dropbutton.addClass("e-rht-btn");
            }

            /*width and height setting*/
            this._setSize(this.model.size);
            this.element.addClass("e-left-btn");
            this._renderButtonContent();
            this.innerWrap.append(this.element, this.dropbutton);
            /*Rendering Context menu*/
            this._renderContxtMenu();
            this._roundedCorner(this.model.roundedCorner);
            this._setRTL(this.model.rtl);
        },
        /**
         * Render section for button content
		 * @private
         */
        _renderButtonContent: function () {
            /*Image and Text*/
            this.textspan = ej.buildTag('span.e-btntxt', this.model.text);
            this.majorimgtag = ej.buildTag('span.e-icon ' + this.model.prefixIcon);
            this.minorimgtag = ej.buildTag('span.e-icon ' + this.model.suffixIcon);
            this.imgtxtwrap = ej.buildTag('div');
            /*Rendering Option*/
            if (this.model.contentType == ej.contents.textAndImage) {
                if (this.model.imagePosition == ej.imagePositions.imageRight) {
                    this.imgtxtwrap.append(this.textspan, this.majorimgtag);
                    this.element.append(this.imgtxtwrap);
                } else {
                    this.imgtxtwrap.append(this.majorimgtag, this.textspan);
                    this.element.append(this.imgtxtwrap);
                }
            } else if (this.model.contentType == ej.contents.imageTextImage) {
                this.imgtxtwrap.append(this.majorimgtag, this.textspan, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.contents.imageBoth) {
                this.imgtxtwrap.append(this.majorimgtag, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.contents.imageOnly) {
                this.imgtxtwrap.append(this.majorimgtag);
                this.element.append(this.imgtxtwrap);
            } else {
                this.element.addClass("e-txt");
                this.element.text(this.model.text);
            }
        },
        /*Height of the splitButton*/
        /**
         * To configure size of the button		
		 * @private
         */
        _setSize: function (val) {
            var ht = this.model.height, wd = this.model.width;
            switch (val) {
                case "mini":
                    if (ht == "") ht = "28px";
                    if (wd == "") wd = "63px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '12px');
                    break;
                case "small":
                    if (ht == "") ht = "32px";
                    if (wd == "") wd = "89px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '13px');
                    break;
                case "medium":
                    if (ht == "") ht = "36px";
                    if (wd == "") wd = "113px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '14px');
                    break;
                case "large":
                    if (ht == "") ht = "40px";
                    if (wd == "") wd = "130px";
                    this.element.css('font-size', '15px');
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    break;
                default:
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    break;
            }
        },
        /**
         * To configure height for the split button		
		 * @private
         */
        _splitbtnHeight: function (val) {
            if ((val == "") || (val == null)) val = '30px';
            this.splitwrap.css("height", val);
        },
        /*Width of the splitButton*/
        /**
         * To configure width for the split button		
		 * @private
         */
        _splitbtnWidth: function (val) {
            this.splitwrap.css("width", val);
        },
        /*Context menu Rendering*/
        /**
         * Render section for Context menu Rendering	
		 * @private
         */
        _renderContxtMenu: function () {
            $("#" + this.model.targetId).ejMenu({
                menuType: ej.menuType.ContextMenu,
                openOnClick: false,
                contextTargetId: "",
                fields: this.model.fields,
                showArrow: true,
                cssClass: this.model.cssClass,
                rtl: this.model.rtl,
                mouseOver: $.proxy(this._itemMouseOver, this),
                mouseOut: $.proxy(this._itemMouseOut, this),
                click: $.proxy(this._itemClick, this)
            });
        },
        /**
         * Section For handle the item click event for the split button
		 * @private
         */
        _itemClick: function (args) {
            this._trigger("itemSelected", args);
        },
        /**
         * Section For handle the item MouseOver event for the split button
		 * @private
         */
        _itemMouseOver: function (args) {
            this._trigger("itemMouseOver", args);
        },
        /**
         * Section For handle the item MouseOut event for the split button
		 * @private
         */
        _itemMouseOut: function (args) {
            this._trigger("itemMouseOut", args);
        },
        /*-----------------------Event Handlers -----------------------------------------*/
        /**
         * Wiring the events to button control		
		 * @private
         */
        _wireEvents: function () {
            /// <summary>This function is  used to Apply Events in the Split button Object</summary>
            /*element*/
            this._on(this.element, "click", this._btnMouseClick);
            this._on(this.element, "mousedown", this._btnMouseDown);

            /*DrpBTN*/
            this._on(this.dropbutton, "click", this._btnMouseClick);
            /*DocClk*/
            this._on($(document), "mousedown", this._documentClick);
            this._on($(document), "mousedown", this._docrhtclk);
        },
        /*EVENT functionalities*/
        /**
         * Section For handle the click event
		 * @private
         */
        _btnMouseClick: function (e) {
            var args, btnpos, btnposx, btnposy, targetElement, poscur = 1;
            if (!$(e.currentTarget).hasClass("e-disable")) {
                if (e.currentTarget.id != this.element[0].id + "drpbtn") {
                    args = { status: this.model.enabled };
                    this._trigger("click", args);
                } else {
                    /*Context Menu Functionality*///need to write rtl
                    targetElement = this.dropbutton;
                    btnpos = this.dropbutton.offset();
                    btnposx = btnpos.left - poscur;
                    btnposy = (btnpos.top + this.dropbutton.outerHeight()) - poscur;
                    if (this.contstatus) {
                        this._hidecontext(e);
                    } else {
                        if (this.model.rtl) {
                            $("#" + this.model.targetId).ejMenu('ShowContextMenu', btnposx - ($("#" + this.model.targetId).width() - this.dropbutton.width()), btnposy, targetElement, e);
                            this.contstatus = true;
                        } else {
                            $("#" + this.model.targetId).ejMenu('ShowContextMenu', btnposx, btnposy, targetElement, e);
                            this.contstatus = true;
                        }
                        this.element.bind("click", $.proxy(this._hidecontext, this));
                    }

                }
            }
        },
        /**
         * Section For handle mouse down action.
		 * @private
         */
        _btnMouseDown: function (e) {
            if (!$(e.currentTarget).hasClass("e-disable")) {
                this._docrhtclk(e);
            }
        },
        /**
         * Section For handle hidecontext action.
		 * @private
         */
        _hidecontext: function (e) {
            if (!$(e.target).is(this.element) && !$(e.target).is(this.textspan) &&
					  !$(e.target).is(this.imgtxtwrap) && !$(e.target).is(this.splitwrap) && !$(e.target).parents().is($("#" + this.model.targetId))) {
                $("#" + this.model.targetId).ejMenu('HideContextMenu', e);
                this.contstatus = false;
                this.element.unbind("click", $.proxy(this._hidecontext, this));
            }
        },
        /**
         * Section For handle on document click action.
		 * @private
         */
        _documentClick: function (e) {
            if (!$(e.target).is(this.element) && !$(e.target).is(this.textspan) &&
            !$(e.target).is(this.imgtxtwrap) && !$(e.target).is(this.splitwrap) && !$(e.target).parents().is($("#" + this.model.targetId))) {
                $("#" + this.model.targetId).ejMenu('HideContextMenu', e);
                this.contstatus = false;
            }
        },
        /**
         * Section For handle document right click action.
		 * @private
         */
        _docrhtclk: function (e) {
            var isRightClick, targetElement;
            isRightClick = false;
            if (e.button) {
                isRightClick = (e.button == 2);
            } else if (e.which) {
                isRightClick = (e.which == 3); //for Opera
            }
            targetElement = e.target;
            if (isRightClick) {
                e.preventDefault();
            }
        },
        /**/
    });
    /**
	 * Enum for button content mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.contents = { textOnly: "textonly", imageOnly: "imageonly", imageBoth: "imageboth", textAndImage: "textandimage", imageTextImage: "imagetextimage" };
    /**
	 * Enum for Button positioning	 
	 * @enum {string}
	 * @global 
	 */
    ej.imagePositions = { imageRight: "imageright", imageLeft: "imageleft" };
    /**
	 * Enum for various button sizes	 
	 * @enum {string}
	 * @global 
	 */
    ej.buttonSize = { mini: "mini", small: "small", medium: "medium", large: "large" };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    /**
    * @namespace ej
	* @class ejToggleButton
	* @requires jQuery
	* @requires ej.web.all
	* @classdesc Custom Design for Html ToggleButton control.
	* @example 
	* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
	* // Create ToggleButton <br>
    * $('#toggle').ejToggleButton(); 	<br>	
	* &lt;/script&gt; <br>
	*/

    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejToggleButton", "ej.ToggleButton", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-togglebutton",

        // default model
        defaults: {
            /// <summary>This Contains default property of toggle button </summary>
            /**		
			* Specifies the size of the ToggleButton.	See {@link buttonSize}
			* @default ej.buttonSize.normal
			* @type {enum}
			* @example 
			* //To set size API value during initialization  
			* 	$("#toggle").ejToggleButton({  size: ej.buttonSize.mini});					 
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#toggle").ejToggleButton("option", "size");<br>			
			*	//Sets the size value 
			*	$("#toggle").ejToggleButton("option", "size",  ej.buttonSize.large ); 
			* @memberof ejToggleButton
			* @instance
			*/
            size: "normal",
            /**		
			* Specifies the width of the ToggleButton.
			* @default 100pixel
			* @type {String}
			* @example 
			* //To set width API value during initialization  
			* 	$("#toggle").ejToggleButton({ width: "100px" });						
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value
			*	$("#toggle").ejToggleButton("option", "width");						
			*	//Sets the width value
			*	$("#toggle").ejToggleButton("option", "width", "100px" );			
			* @memberof ejToggleButton
			* @instance
			*/
            width: "",

            /**		
			* Specifies the height of the ToggleButton.
			* @default 28pixel
			* @type {String}
			* @example 
			* //To set height API value during initialization  
			* 	$("#toggle").ejToggleButton({ height: "28px" });						
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value
			*	$("#toggle").ejToggleButton("option", "height");						
			*	//Sets the height value
			*	$("#toggle").ejToggleButton("option", "height", "28px" );			
			* @memberof ejToggleButton
			* @instance
			*/
            height: "",

            /**		
			* Specifies the state of the ToggleButton.
			* @default true
			* @type {boolean}
			* @example 
			* //To set enabled API value during initialization  
			* 	$("#toggle").ejToggleButton({ enabled: true });						
			* @example 
			* //Get or set the enabled API, after initialization:<br>
			*	//Gets the enabled value
			*	$("#toggle").ejToggleButton("option", "enabled");						
			*	//Sets the enabled value
			*	$("#toggle").ejToggleButton("option", "enabled", true );			
			* @memberof ejToggleButton
			* @instance
			*/
            enabled: true,
            /**		
			* Specifies the checkedStatus of the ToggleButton.
			* @default false
			* @type {boolean}
			* @example 
			* //To set checkedStatus API value during initialization  
			* 	$("#toggle").ejToggleButton({ checkedStatus: false });						
			* @example 
			* //Get or set the checkedStatus API, after initialization:<br>
			*	//Gets the checkedStatus value
			*	$("#toggle").ejToggleButton("option", "checkedStatus");						
			*	//Sets the checkedStatus value
			*	$("#toggle").ejToggleButton("option", "checkedStatus", false );			
			* @memberof ejToggleButton
			* @instance
			*/
            checkedStatus: false,
            /**		
			* Specifies the defaultText of the ToggleButton.
			* @default null
			* @type {string}
			* @example 
			* //To set defaultText API value during initialization  
			* 	$("#toggle").ejToggleButton({ defaultText: "click" });						
			* @example 
			* //Get or set the defaultText API, after initialization:<br>
			*	//Gets the defaultText value
			*	$("#toggle").ejToggleButton("option", "defaultText");						
			*	//Sets the defaultText value
			*	$("#toggle").ejToggleButton("option", "defaultText", "click" );			
			* @memberof ejToggleButton
			* @instance
			*/
            defaultText: null,
            /**		
			* Specifies the activeText of the ToggleButton.
			* @default null
			* @type {string}
			* @example 
			* //To set activeText API value during initialization  
			* 	$("#toggle").ejToggleButton({ activeText: "Play" });						
			* @example 
			* //Get or set the activeText API, after initialization:<br>
			*	//Gets the activeText value
			*	$("#toggle").ejToggleButton("option", "activeText");						
			*	//Sets the activeText value
			*	$("#toggle").ejToggleButton("option", "activeText", "Play" );			
			* @memberof ejToggleButton
			* @instance
			*/
            activeText: null,
            /**		
			* Specifies the contentType of the ToggleButton. See {@link contents}
			* @default ej.contents.textOnly
			* @type {enum}
			* @example 
			* // Set the button contentType on initialization. 			
			* 	$("#toggle").ejToggleButton({  contentType : ej.contents.imageOnly });			 
			* @example 
			* //Get or set the button contentType, after initialization:<br>
			*	// Gets the button contentType.		
			*	$("#toggle").ejToggleButton("option", "contentType");			
			*	// Sets the contentType to button
			*	$("#toggle").ejToggleButton("option", "contentType",  ej.contents.imageOnly );
			* @memberof ejToggleButton
			* @instance
			*/
            contentType: "textonly",
            /**		
			*  Specifies the image position of the ToggleButton. This image position is applicable only with the textandimage contentType property. The images can be positioned in both imageLeft and imageRight options. See {@link imagePositions}
			* @default ej.imagePositions.imageLeft
			* @type {enum}
			* @example 
			* // Set the image position for toggle button during initialization. 			
			* 	$("#toggle").ejToggleButton(
            * {
            *    contentType: ej.contents.textAndImage,
            *    imagePosition: ej.imagePositions.imageRight
            * });			* 
			* @example 
			* //Get or set image position for button after initialization:<br>
			*	// Gets the image position.		
			*	$("#toggle").ejToggleButton("option", "imagePosition");			
			*	// Sets the image position
			*	$("#toggle").ejToggleButton("option", "imagePosition",  ej.contents.imageRight );
			* @memberof ejToggleButton
			* @instance
			*/
            imagePosition: "imageleft",
            /**		
			* Specify the rounded corner to Togglebutton
			* @default false
			* @type {boolean}
			* @example 
			* // Set the rounded corner during initialization. 			
			* 	$("#toggle").ejToggleButton({  roundedCorner : true });			 
			* @example 
			* //Get or set the rounded corner after initialization:<br>
			*	// Gets the rounded corner value.		
			*	$("#toggle").ejToggleButton("option", "roundedCorner");			
			*	// Sets the rounded corner to button
			*	$("#toggle").ejToggleButton("option", "roundedCorner",  true );
			* @memberof ejToggleButton
			* @instance
			*/
            roundedCorner: false,
            /**					
			* Specify the persist to Togglebutton to save current model value to browser cookies for state maintains
			* @default false
			* @type {boolean}
			* @example 
			* // Set the rounded corner during initialization. 			
			* 	$("#toggle").ejToggleButton({  persist : true });			 
			* @example 
			* //Get or set the rounded corner after initialization:<br>
			*	// Gets the rounded corner value.		
			*	$("#toggle").ejToggleButton("option", "persist");			
			*	// Sets the rounded corner to button
			*	$("#toggle").ejToggleButton("option", "persist",  true );
			* @memberof ejToggleButton
			* @instance
			*/
            persist: false,
            /**		
			* Specify the CSS class to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the CSS class during initialization. 			
			* 	$("#button1").ejToggleButton({  cssClass : "gradient-lime" });			 
			* @example 
			* //Get or set the cssClass after initialization:<br>
			*	// Gets the cssClass value.		
			*	$("#toggle").ejToggleButton("option", "cssClass");			
			*	// Sets the cssClass to button
			*	$("#toggle").ejToggleButton("option", "cssClass",  "gradient-lime" );
			* @memberof ejToggleButton
			* @instance
			*/
            cssClass: "",
            /**		
			* Specify the defaultPrefixIcon to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the defaultPrefixIcon during initialization. 			
			* 	$("#toggle").ejToggleButton({ 
            *    contentType: "imageboth",
            *    defaultPrefixIcon: "e-uiLight e-icon-handup"
            *  });			
			* @example 
			* //Get or set the defaultPrefixIcon after initialization:<br>
			*	// Gets the defaultPrefixIcon value.		
			*	$("#button1").ejToggleButton("option", "defaultPrefixIcon");			
			*	// Sets the defaultPrefixIcon to button
			*	$("#button1").ejToggleButton("option", "defaultPrefixIcon",  "e-uiLight e-icon-handup" );
			* @memberof ejToggleButton
			* @instance
			*/
            defaultPrefixIcon: null,
            /**		
			* Specify the defaultSuffixIcon to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the defaultSuffixIcon during initialization. 			
			*  $("#toggle").ejToggleButton({ 
            *   contentType: "imageboth",
			*	defaultPrefixIcon: "e-uiLight e-icon-handup",
            *   defaultSuffixIcon: "e-uiLight e-icon-padlockclosed"
            *  });			
			* @example 
			* //Get or set the defaultSuffixIcon after initialization:<br>
			*	// Gets the defaultSuffixIcon value.		
			*	$("#toggle").ejToggleButton("option", "defaultSuffixIcon");			
			*	// Sets the defaultSuffixIcon to button
			*	$("#toggle").ejToggleButton("option", "defaultSuffixIcon",  "e-uiLight e-icon-padlockclosed" );
			* @memberof ejToggleButton
			* @instance
			*/
            defaultSuffixIcon: null,
            /**		
			* Specify the activePrefixIcon to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the activePrefixIcon during initialization. 			
			* 	$("#toggle").ejToggleButton({ 
            *    contentType: "imageboth",
            *    activePrefixIcon: "e-uiLight e-icon-handup"
            *  });			
			* @example 
			* //Get or set the activePrefixIcon after initialization:<br>
			*	// Gets the activePrefixIcon value.		
			*	$("#button1").ejToggleButton("option", "activePrefixIcon");			
			*	// Sets the activePrefixIcon to button
			*	$("#button1").ejToggleButton("option", "activePrefixIcon",  "e-uiLight e-icon-handup" );
			* @memberof ejToggleButton
			* @instance
			*/
            activePrefixIcon: null,
            /**		
			* Specify the activeSuffixIcon to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the activeSuffixIcon during initialization. 			
			* 	$("#toggle").ejToggleButton({ 
            *    contentType: "imageboth",
            *    activeSuffixIcon: "e-uiLight e-icon-handup"
            *  });			
			* @example 
			* //Get or set the activeSuffixIcon after initialization:<br>
			*	// Gets the activeSuffixIcon value.		
			*	$("#button1").ejToggleButton("option", "activeSuffixIcon");			
			*	// Sets the activeSuffixIcon to button
			*	$("#button1").ejToggleButton("option", "activeSuffixIcon",  "e-uiLight e-icon-handup" );
			* @memberof ejToggleButton
			* @instance
			*/
            activeSuffixIcon: null,
            /**		
			* Specify the Right to Left Direction to Togglebutton
			* @default false
			* @type {boolean}
			* @example 
			* // Set the rtl during initialization. 			
			* 	$("#toggle").ejToggleButton({  rtl : true });			 
			* @example 
			* //Get or set the rtl after initialization:<br>
			*	// Gets the rtl value.		
			*	$("#toggle").ejToggleButton("option", "rtl");			
			*	// Sets the rtl to button
			*	$("#toggle").ejToggleButton("option", "rtl",  true );
			* @memberof ejToggleButton
			* @instance
			*/
            rtl: false,
            /**     
			 * Fires when ToggleButton control is created successfully.
			 * @event
			 * @name ejToggleButton#create 	
			 * @param {Object} argument Event parameters from toggle button     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.model returns the toggle button model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example 
			 * //create event for toggle button
             * $("#toggle").ejToggleButton({
             *    create: function (args) {}
             * });      
			 * @memberof ejToggleButton
			 * @instance
			 */
            create: null,
            /**     
			 * Fires when ToggleButton control is clicked successfully.
			 * @event
			 * @name ejToggleButton#click 	
			 * @param {Object} argument Event parameters from toggle button     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {boolean}  argument.isChecked return the toggle button checked state
			 * @param {object}  argument.model returns the toggle button model
			 * @param {boolean}  argument.status return the toggle button state			 
			 * @param {string}  argument.type returns the name of the event			 			
			 * @example 
			 * //click event for toggle button
             * $("#toggle").ejToggleButton({
             *    click: function (args) {}
             * });      
			 * @memberof ejToggleButton
			 * @instance
			 */
            click: null,
            /**     
			 * Fires when ToggleButton control state is changed successfully.
			 * @event
			 * @name ejToggleButton#change 	
			 * @param {Object} argument Event parameters from toggle button     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {boolean}  argument.isChecked return the toggle button checked state
			 * @param {object}  argument.model returns the toggle button model		
			 * @param {string}  argument.type returns the name of the event			 			
			 * @example 
			 * //change event for toggle button
             * $("#toggle").ejToggleButton({
             *    change: function (args) {}
             * });      
			 * @memberof ejToggleButton
			 * @instance
			 */
            change: null,
            /**     
			 * Fires when ToggleButton control is destroyed successfully.
			 * @event
			 * @name ejToggleButton#destroy 	
			 * @param {Object} argument Event parameters from toggle button     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.model returns the toggle button model		
			 * @param {string}  argument.type returns the name of the event			 			
			 * @example 
			 * //destroy event for toggle button
             * $("#toggle").ejToggleButton({
             *    destroy: function (args) {}
             * });      
			 * @memberof ejToggleButton
			 * @instance
			 */
            destroy: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        //Data Types
        dataTypes: {
            size: "string",
            height: "string",
            weight: "string",
            enabled: "boolean",
            roundedCorner: "boolean",
            checkedStatus: "boolean",
            defaultText: "string",
            activeText: "string",
            contentType: "enum",
            imagePosition: "enum",
            defaultPrefixIcon: "string",
            defaultSuffixIcon: "string",
            activePrefixIcon: "string",
            activeSuffixIcon: "string",
            cssClass: "string",
            rtl: "boolean",
            persist: "boolean"
        },
        /**
        * To disable the toggle button  
		* @return jQuery
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // Create toggle button
		* var toggleObj = $("#toggle").data("ejToggleButton");
		* toggleObj.disable(); // disable the toggle button
		* &lt;/script&gt;
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // disable the toggle button
		* $("#toggle").ejToggleButton("disable");	
		* &lt;/script&gt;
		*@memberof ejToggleButton
		* @instance
        */
        // sample public function
        disable: function () {
            /// <summary>This Contains disable functionality to wire with the toggle button </summary>
            this.buttontag.addClass("e-disable").attr({ "aria-disabled": true });
            this.model.enabled = false;
        },
        /**
        * To enable the toggle button  
		* @return jQuery
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // Create toggle button
		* var toggleObj = $("#toggle").data("ejToggleButton");
		* toggleObj.enable(); // enable the toggle button
		* &lt;/script&gt;
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // enable the toggle button
		* $("#toggle").ejToggleButton("enable");	
		* &lt;/script&gt;
		*@memberof ejToggleButton
		* @instance
        */
        enable: function () {
            /// <summary>This Contains enable functionality to wire with the toggle button </summary>
            if (this.buttontag.hasClass("e-disable")) {
                this.buttontag.removeClass("e-disable").attr({ "aria-disabled": false });
                this.model.enabled = true;
            }
        },
        /**
         * Create the toggle button widget
		 * @private
         */
        // constructor function
        _init: function () {
            /// <summary>This function contains initialization </summary>
            this._initialize();
            this._controlStatus(this.model.enabled);
            this._wireEvents();
        },
        /**
        * To destroy the toggle button  
		* @return jQuery
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // Create toggle button
		* var toggleObj = $("#toggle").data("ejToggleButton");
		* toggleObj.destroy(); // destroy the toggle button
		* &lt;/script&gt;
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // destroy the toggle button
		* $("#toggle").ejToggleButton("destroy");	
		* &lt;/script&gt;
		*@memberof ejToggleButton
		* @instance
        */
        _destroy: function () {
            /// <summary>This Contains destroy of toggle button </summary>
            this.element.unwrap();
            this.element.removeClass('e-chkbx-hidden');
            this.labelFinder.empty();
            this.labelFinder.text(this.model.defaultText);
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "defaultText":
                        this._setDefaultText(options[option]);
                        break;
                    case "activeText":
                        this._setActiveText(options[option]);
                        break;
                    case "defaultPrefixIcon":
                        this._setDefaultMajorIcon(options[option]);
                        break;
                    case "defaultSuffixIcon":
                        this._setDefaultMinorIcon(options[option]);
                        break;
                    case "activePrefixIcon":
                        this._setActiveMajorIcon(options[option]);
                        break;
                    case "activeSuffixIcon":
                        this._setActiveMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "checkedStatus":
                        this._tglevaluestatus(options[option]);
                        break;
                    case "roundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "rtl":
                        this._setRTL(options[option]);
                        break;
                }
            }
        },
        /**
         * To configure size of the toggle button		
		 * @private
         */
        _setSize: function (val) {
            this.buttontag.removeClass('e-btn-mini e-btn-medium e-btn-small e-btn-large e-btn-normal');
            this.buttontag.addClass("e-btn-" + val);
        },
        /**
         * To configure height of the toggle button		
		 * @private
         */
        _setHeight: function (val) {
            /// <summary>This will set Text property of toggle button </summary>
            this.buttontag.css('height', val);
        },
        /**
         * To configure width of the toggle button		
		 * @private
         */
        _setWidth: function (val) {
            /// <summary>This will set Text property of toggle button </summary>
            this.buttontag.css('width', val);
        },
        /**
         * To configure default text to the toggle button		
		 * @private
         */
        _setDefaultText: function (val) {
            /// <summary>This will set Text property of toggle button </summary>
            if (this.model.contentType == ej.contents.textOnly) {
                this.buttontag.html(val);
            } else {
                this.defaulttxtspan.html(val);
            }
        },
        /**
         * To configure active text to the toggle button		
		 * @private
         */
        _setActiveText: function (val) {
            /// <summary>This will set Text property of toggle button </summary>
            if (this.model.checkedStatus) {
                if (this.model.contentType == ej.contents.textOnly) {
                    this.buttontag.html(val);
                } else {
                    this.defaulttxtspan.html(val);
                }
            }
        },
        /**
         * To configure default major icon of the toggle button		
		 * @private
         */
        _setDefaultMajorIcon: function (val) {
            /// <summary>This will set Major Icon property of toggle button </summary>
            this.defMainIcon.removeClass(this.model.defaultPrefixIcon);
            this.defMainIcon.addClass(val);
        },
        /**
         * To configure default minor icon of the toggle button		
		 * @private
         */
        _setDefaultMinorIcon: function (val) {
            /// <summary>This will set Minor Icon property of toggle button </summary>
            this.defMiniIcon.removeClass(this.model.defaultSuffixIcon);
            this.defMiniIcon.addClass(val);
        },
        /**
         * To configure active major icon of the toggle button		
		 * @private
         */
        _setActiveMajorIcon: function (val) {
            /// <summary>This will set Major Icon property of toggle button </summary>
            if (this.model.checkedStatus) {
                this.defMainIcon.removeClass(this.model.activePrefixIcon);
                this.defMainIcon.addClass(val);
            }
        },
        /**
         * To configure active minor icon of the toggle button		
		 * @private
         */
        _setActiveMinorIcon: function (val) {
            /// <summary>This will set Minor Icon property of toggle button </summary>
            if (this.model.checkedStatus) {
                this.defMiniIcon.removeClass(this.model.activeSuffixIcon);
                this.defMiniIcon.addClass(val);
            }
        },
        /**
         * To configure content type of the toggle button		
		 * @private
         */
        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.buttontag.empty();
                this.model.contentType = val;
                this._renderButtonContent();
            }
        },
        /**
         * To configure image position of the toggle button		
		 * @private
         */
        _setImagePosition: function (val) {
            if ((this.model.contentType == ej.contents.textAndImage) && (val != this.model.imagePosition)) {
                this.buttontag.empty();
                this.model.imagePosition = val;
                this._renderButtonContent();
            }
        },
        /**
        * To enable or disable the Right to Left behaviour 		
        * @private
        */
        _setRTL: function (val) {
            if (val) {
                this.buttontag.addClass("e-rtl");
            } else {
                this.buttontag.removeClass("e-rtl");
            }
        },
        /**
        * To enable or disable the toggle button state 		
        * @private
        */
        _controlStatus: function (value) {
            //disable status
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },
        /**
         * To configure the custom theme for toggle button using cssClass property		
		 * @private
         */
        _setSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.buttontag.removeClass(this.model.cssClass);
                this.buttontag.addClass(skin);
            }
        },
        /**
         * To initialize the toggle button		
		 * @private
         */
        _initialize: function () {
            /// <summary>This Contains initialize of toggle button </summary>
            if (this.element.is('[type = "checkbox"]')) {
                this._render();
            } else {
                this.element.removeClass("e-togglebutton");//need to change in source level
            }
        },
        /**
        * Render Section For DifferentTypes		
        * @private
        */
        /*Render Section For DifferentTypes*/
        _render: function () {
            /// <summary>This Contains rendering of toggle button </summary>
            var predecessor, labelSelector;
            /*Determine Button Type*/
            this.element.addClass('e-chkbx-hidden e-tbtn');

            /*Finding the label element in the dom*/
            predecessor = this.element.parents().last();
            labelSelector = "label[for='" + this.element[0].id + "']";
            this.labelFinder = predecessor.find(labelSelector);
            if (!this.labelFinder.length) {
                this.labelFinder = ej.buildTag('label', "Button", {}, { "for": this.element[0].id });
                this.labelFinder.insertAfter(this.element);
            }
            if ((this.model.defaultText == null) || (this.model.defaultText == "")) {
                this.model.defaultText = this.labelFinder.text();
            }
            this.labelFinder.empty();
            this.wrapper = $('<span id="' + this.element[0].id + '-wrapper" class="e-tbtn-wrap e-widget"></span>');
            $(this.element).wrap(this.wrapper);
            this.labelFinder.insertAfter(this.element);
            this.buttontag = ej.buildTag('button.e-togglebutton e-btn e-tbtn ' + this.model.cssClass + ' e-select', "", {}, { "role": "button", "tabindex": 0 });//need to change in source level
            this.labelFinder.append(this.buttontag);
            /*Initializing the content*/
            if (ej.util.isNullOrUndefined(this.model.activeText)) {
                this.model.activeText = this.model.defaultText;
            }
            if (ej.util.isNullOrUndefined(this.model.activePrefixIcon)) {
                this.model.activePrefixIcon = this.model.defaultPrefixIcon;
            }
            if (ej.util.isNullOrUndefined(this.model.activeSuffixIcon)) {
                this.model.activeSuffixIcon = this.model.defaultSuffixIcon;
            }
            /*Height Width*/
            this._setSize(this.model.size);
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
            this._setRTL(this.model.rtl);
            this._renderButtonContent();
            this._tglevaluestatus(this.model.checkedStatus);
            this._roundedCorner(this.model.roundedCorner);
        },
        /**
         * Render Section For toggle button content
		 * @private
         */
        _renderButtonContent: function () {
            /*Image and Text*/
            this.imgtxtwrap = ej.buildTag('div');
            /*Text Content*/
            this.defaulttxtspan = ej.buildTag('span.e-btntxt' + '#' + this.element[0].id + 'textstatic', this.model.defaultText);
            /*Image Content*/
            if (this.model.contentType.indexOf("image") > -1) {
                this.defMainIcon = ej.buildTag('span.e-icon ' + this.model.defaultPrefixIcon + '#' + this.element[0].id + 'mainiconstatic');
                this.defMiniIcon = ej.buildTag('span.e-icon ' + this.model.defaultSuffixIcon + '#' + this.element[0].id + 'miniconstatic');
            }
            /*Rendering Option*/
            if (this.model.contentType == ej.contents.textAndImage) {
                if (this.model.imagePosition == ej.imagePositions.imageRight) {
                    this.imgtxtwrap.append(this.defaulttxtspan, this.defMainIcon);
                    this.buttontag.append(this.imgtxtwrap);
                } else {
                    this.imgtxtwrap.append(this.defMainIcon, this.defaulttxtspan);
                    this.buttontag.append(this.imgtxtwrap);
                }
            } else if (this.model.contentType == ej.contents.imageTextImage) {
                this.imgtxtwrap.append(this.defMainIcon, this.defaulttxtspan, this.defMiniIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.contents.imageOnly) {
                this.imgtxtwrap.append(this.defMainIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.contents.imageBoth) {
                this.imgtxtwrap.append(this.defMainIcon, this.defMiniIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else {
                this.buttontag.addClass("e-txt");
                this.buttontag.text(this.model.defaultText);
            }
            /**/
        },
        /**
         * To check whether Status of Toggle Button
		 * @private
         */
        _tglevaluestatus: function (value) {
            /*Checking Status of Toggle Button*/
            if (value) {
                this._toggleButtonStatus(value);
                this.element.attr("checked", "checked");
            } else {
                this._toggleButtonStatus(value);
                this.element.removeAttr("checked");
            }
        },
        /**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */
        _roundedCorner: function (value) {
            value == true ? this.buttontag.addClass('e-corner-all') : this.buttontag.removeClass('e-corner-all');
        },
        /*-----------------------Event Handlers -----------------------------------------*/
        /**
        * Wiring the events to toggle button control		
        * @private
        */
        _wireEvents: function () {
            /// <summary>This Contains Events to wire with the toggle button </summary>

            this._on(this.buttontag, "click", this._tglebtnclicked);
        },
        /**
        * Section For handle the click event
        * @private
        */
        _tglebtnclicked: function (e) {
            var position, end, version = 0;
            if (navigator.userAgent.search("MSIE") > 0) {
                position = navigator.userAgent.search("MSIE") + 5;
                end = navigator.userAgent.search("; Windows");
                version = navigator.userAgent.substring(position, end);
            }
            if (!this.buttontag.hasClass("e-disable")) {
                if (!this.model.checkedStatus) {
                    this.model.checkedStatus = true;
                    if ((window.navigator.appName == "Microsoft Internet Explorer") && (version == "8.0")) {
                        this._tglevaluestatus(this.model.checkedStatus);
                    } else {
                        this._toggleButtonStatus(this.model.checkedStatus);
                    }
                } else {
                    this.model.checkedStatus = false;
                    if ((window.navigator.appName == "Microsoft Internet Explorer") && (version == "8.0")) {
                        this._tglevaluestatus(this.model.checkedStatus);
                    } else {
                        this._toggleButtonStatus(this.model.checkedStatus);
                    }
                }
            }
            this._trigger("click", { "isChecked": this.model.checkedStatus, status: this.model.enabled });
        },
        /**
        * Section For handle the toggle button status
        * @private
        */
        _toggleButtonStatus: function (buttonstatus) {
            if (buttonstatus) {
                if (this.model.contentType == ej.contents.textOnly) {
                    this.buttontag.html(this.model.activeText);
                } else {
                    this.defaulttxtspan.html(this.model.activeText);
                    this.defMainIcon.removeClass(this.model.defaultPrefixIcon).addClass(this.model.activePrefixIcon);
                    this.defMiniIcon.removeClass(this.model.defaultSuffixIcon).addClass(this.model.activeSuffixIcon);
                }
                this.buttontag.addClass("e-active").attr("aria-pressed", true);
            } else {
                if (this.model.contentType == ej.contents.textOnly) {
                    this.buttontag.html(this.model.defaultText);
                } else {
                    this.defaulttxtspan.html(this.model.defaultText);
                    this.defMainIcon.removeClass(this.model.activePrefixIcon).addClass(this.model.defaultPrefixIcon);
                    this.defMiniIcon.removeClass(this.model.activeSuffixIcon).addClass(this.model.defaultSuffixIcon);
                }
                this.buttontag.removeClass("e-active").attr("aria-pressed", false);
            }
            this._trigger('change', { "isChecked": this.model.checkedStatus });
        },

    });
    /**
	 * Enum for toggle button content mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.contents = {
        /**  Supports only for text content only */
        textOnly: "textonly",
        /** Supports only for image content only */
        imageOnly: "imageonly",
        /** Supports image for both ends of the button */
        imageBoth: "imageboth",
        /** Supports image with the text content */
        textAndImage: "textandimage",
        /** Supports image with both ends of the text */
        imageTextImage: "imagetextimage"
    };

    /**
	 * Enum for toggle button positioning	 
	 * @enum {string}
	 * @global 
	 */
    ej.imagePositions = {
        /**  support for aligning text in right and image in left. */
        imageRight: "imageright",
        /**  support for aligning text in left and image in right. */
        imageLeft: "imageleft"
    };
    /**
	 * Enum for various toggle button sizes	 
	 * @enum {string}
	 * @global 
	 */
    ej.buttonSize = {
        /**  Creates button with inbuilt default size height, width specified */
        normal: "normal",
        /**  Creates button with inbuilt mini size height, width specified */
        mini: "mini",
        /**  Creates button with inbuilt small size height, width specified */
        small: "small",
        /**  Creates button with inbuilt medium size height, width specified */
        medium: "medium",
        /**  Creates button with inbuilt large size height, width specified */
        large: "large"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Toolbar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
        * @namespace ej
        * @class ejToolbar
        * @requires jQuery
        * @requires ej.common.all
        * @classdesc Custom Design for Toolbar control.
        * @example 
        * &lt;div id="toolbar1"&gt;
        *&lt;ul&gt;
        *    &lt;li id="Left" title="Left"&gt;
        *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
        *   &lt;/li&gt;
        *    &lt;li id="Center" title="Center"&gt;
        *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
        *    &lt;/li&gt;
        *    &lt;li id="Right" title="Right"&gt;
        *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
        *    &lt;/li&gt;
        *    &lt;li id="Justify" title="Justify"&gt;
        *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
        *    &lt;/li&gt;
        *&lt;/ul&gt;
        *&lt;ul&gt;
        *    &lt;li id="Bold" title="Bold"&gt;
        *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
        *    &lt;/li&gt;
        *    &lt;li id="Italic" title="Italic"&gt;
        *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
        *    &lt;/li&gt;
        *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
        *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
        *    &lt;/li&gt;
        *    &lt;li id="UndeLine" title="UnderLine"&gt;
        *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
        *    &lt;/li&gt;
        *&lt;/ul&gt;
        *&lt;/div&gt;
        * &lt;script&gt;
        * // Create Toolbar
        *  $("#toolbar1").ejToolbar();	
        * &lt;/script&gt;
        */
    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejToolbar", "ej.Toolbar", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-toolbar",

        // default model
        defaults: {
            /// <summary>This Contains default property of toolbar </summary>
            /**		
			* Specifies the height of the Toolbar.
			* @default 28
			* @type {number}
			* @example 
			* //To set height API value during initialization  
			* 	$("#toolbar1").ejToolbar({ height: 30 });					* 
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value
			*	$("#toolbar1").ejToolbar("option", "height");			
			*	//Sets the height value
			*	$("#toolbar1").ejToolbar("option", "height", 30 );
			 * @memberof ejToolbar
			* @instance
			*/
            height: "",
            /**		
			* Specifies the width of the Toolbar.
			* @default 100
			* @type {number}
			* @example 
			* //To set width API value during initialization  
			* 	//To set width API value 
			* 	$("#toolbar1").ejToolbar({ width: 30 });	
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value
			*	$("#toolbar1").ejToolbar("option", "width");			
			*	//Sets the width value
			*	$("#toolbar1").ejToolbar("option", "width", 30 );
			 * @memberof ejToolbar
			* @instance
			*/
            width: "",
            /**		
			* Specifies the Toolbar control state.
			* @default true
			* @type {boolean}
			* @example 
			* // Enable Toolbar on initialization. 
			* 	//To set enabled API value 
			* 	$("#toolbar1").ejToolbar({ enabled:  true });			* 
			* @example 
			* //Get or set the Toolbar state, after initialization:<br>
			*	// Gets the Enabled API value.		
			*	$("#toolbar1").ejToolbar("option", "enabled");			
			*	// Sets the Enabled API
			*	$("#toolbar1").ejToolbar("option", "enabled",  false );
			 * @memberof ejToolbar
			* @instance
			*/
            enabled: true,
            /**		
			* Specifies the to show or hide.
			* @default false
			* @type {boolean}
			* @example 
			* // Hide Toolbar on initialization. 
			* 	//To set hide API value 
			* 	$("#toolbar1").ejToolbar({  hide: false });			* 
			* @example 
			* //Get or set the hide, after initialization:<br>
			*	// Gets the hide API value.		
			*	$("#toolbar1").ejToolbar("option", "hide");			
			*	// Sets the hide API
			*	$("#toolbar1").ejToolbar("option", "hide",  true ); 
			 * @memberof ejToolbar
			* @instance
			*/
            hide: false,
            /**		
			* Specifies the to itemSeparator.
			* @default false
			* @type {boolean}
			* @example 
			* //specifies itemSeparator Toolbar on initialization. 
			* 	//To set itemSeparator API value 
			* 	$("#toolbar1").ejToolbar("option", "itemSeparator");			 
			* @example 
			* //Get or set the Toolbar itemSeparator, after initialization:<br>
			*	// Gets the itemSeparator value.		
			*	$("#toolbar1").ejToolbar("option", "itemSeparator");		
			*	// Sets the itemSeparator API
			*	$("#toolbar1").ejToolbar("option", "itemSeparator",  true ); 
			 * @memberof ejToolbar
			* @instance
			*/
            itemSeparator: true,
            /**		
			* Specifies the to itemSeparator.
			* @default Horizontal
			* @type {Enum | String}
			* @example 
			* // Specifies orientation of  Toolbar on initialization. 
			* 	//To set orientation API value 
			* 	$("#toolbar1").ejToolbar({ orientation: ej.orientation.Horizontal }); 
			* @example 
			* //Get or set the Toolbar state, after initialization:<br>
			*	// Gets the orientation API value.		
			*	$("#toolbar1").ejToolbar("option", "orientation");		
			*	// Sets the orientation API
			*	$("#toolbar1").ejToolbar("option", "orientation",   ej.orientation.Vertical ); 
			 * @memberof ejToolbar
			* @instance
			*/
            orientation: "horizontal",
            /**		
			* Specifies rtl property for the Toolbar during initialization.
			* @default false
			* @type {boolean}
			* @example 
			* // to set rtl Toolbar on initialization. 
			* 	//To set rtl API value 
			* 	$("#toolbar1").ejToolbar({ rtl: false });			 
			* @example 
			* //Get or set the rtl, after initialization:<br>
			*	// Gets the rtl API value.		
			*	$("#toolbar1").ejToolbar("option", "rtl");		
			*	// Sets the rtl API
			*	$("#toolbar1").ejToolbar("option", "rtl",  true ); 
			 * @memberof ejToolbar
			* @instance
			*/
            rtl: false,
            /**		
			* Specifies rounderCorner property for the Toolbar during initialization.
			* @default false
			* @type {boolean}
			* @example 
			* // to set rounderCorner for Toolbar on initialization. 
			* 	//To set rtl API value 
			* 	$("#toolbar1").ejToolbar({ roundedCorner: false });			 
			* @example 
			* //Get or set the rounderCorner, after initialization:<br>
			*	// Gets the rounderCorner API value.		
			*	$("#toolbar1").ejToolbar("option", "roundedCorner");	
			*	// Sets the rounderCorner API
			*	$("#toolbar1").ejToolbar("option", "roundedCorner",  true ); 
			 * @memberof ejToolbar
			* @instance
			*/
            roundedCorner: false,
            /**		
        * Specifies dataSource value for the Toolbar during initialization.
        * @default null
        * @type {ObjectArray}
        * @example 
        * // to set dataSource value during initialization. 
        * 	//To set dataSource  API value 
        * 	items = [{ edid: "1", spriteCss: "editTools cursor", text: "Cursor" },
        *{ edid: "2", spriteCss: "editTools select", text: "Select" },
        *{ edid: "3", spriteCss: "editTools move", text: "Move" },
        *{ edid: "4", spriteCss: "editTools rectselect", text: "Rectangle Select" }];

        $("#toolbar1").ejToolbar({ dataSource:  items}); 
        * @example 
        * //Get or set the Toolbar state, after initialization:<br>
        *	// Gets the dataSource value.		
        *	$("#toolbar1").ejToolbar("option", "dataSource");	
        *	// Sets the dataSource value
        *	$("#toolbar1").ejToolbar("option", "dataSource", itemList ); 
        * @memberof ejToolbar
        * @instance
        */
            dataSource: null,
            /**		
        * Specifies Specifies the query to retrieve the data from the online server. The query is used only when the online dataSource is used.
        * @default null
        * @type {object}
        * @example 
        * // To set query API value on initialization. 
        * 	//To set query API value.
        * 	// DataManager creation.
        * var dataManger = ej.DataManager({
        * url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"
        *});
        *  // Query creation.
        *var query = ej.Query()
        *		.from("Orders").take(6);
        *			$("#toolbar1").ejToolbar(
        *                {
        *              dataSource: dataManger,
        *               query: query
        *         });
        * @example 
        * //Get or set the query, after initialization:<br>
        *	// Gets the query value.		
        *	$("#toolbar1").ejToolbar("option", "query");
        *	// Sets the query value
        *	$("#toolbar1").ejToolbar("option", "query",   query2 ); 
        * @memberof ejToolbar
        * @instance
        */
            query: null,
            /**		
        * Specifies the mapping fields for the data items of the Toolbar
        * @default null
        * @type {string}
        * @example 
        * // To set fields API value during initialization. 
        * 	//To set the fields API value.
        * 	items = [{ edid: "1", spriteCss: "editTools cursor", text: "Cursor" },
        *  { edid: "2", spriteCss: "editTools select", text: "Select" },
        *  { edid: "3", spriteCss: "editTools move", text: "Move" },
        *  { edid: "4", spriteCss: "editTools rectselect", text: "Rectangle Select"
        *  }];
        *$("#toolbar1").ejToolbar(
        *		{
        *			dataSource:  items,
        *           fields:  { id: "empid",spriteCss: "spriteCss"}
        *         });
        * @example 
        * //Get or set the Fields, after initialization:<br>
        *	// Gets the fields value.		
        *	$("#toolbar1").ejToolbar("option", "fields");
        *	// Sets the fields value
        *	$("#toolbar1").ejToolbar("option","fields",  { id: "empid",spriteCss: "spriteCss"}); 
        * @memberof ejToolbar
        * @instance
        */
            fields: {
                id: "id",
                tooltipText: "tooltipText",
                imageUrl: "imageUrl",
                text: null,
                imageAttributes: "imageAttributes",
                spriteCSS: "spriteCSS",
                htmlAttributes: "htmlAttributes"
            },
            /**		
			* Sets the root class for Toolbar control theme
			* @default ""
			* @type {string}
			* @example 
			* // Sets the root class for Toolbar control theme on initialization. 
			* 	//To set the cssClass API value.
			* 	$("#toolbar1").ejToolbar({cssClass: "gradient-lime"});
			* @example 
			* //Get or set the cssClass, after initialization:<br>
			*	//Gets the cssClass applied for the control.	
			*	$("#toolbar1").ejToolbar("option", "cssClass");
			*	// Sets the fields value
			*	$("#toolbar1").ejToolbar("option", "cssClass", "gradient-lime"); 
			* @memberof ejToolbar
			* @instance
			*/
            cssClass: "",
            /**     
			 * Fires after Toolbar control is created.
			 * @event
			 * @name ejToolbar#create		
			 * @param {Object} argument Event parameters from Toolbar    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the Toolbar model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * //create event for Toolbar
             *  $("#toolbar1").ejToolbar({
			 *  create: function (args) {}
             * });
			 * @memberof ejToolbar
			 * @instance
			 */
            create: null,
            /**     
			 * Fires after Toolbar control is clicked.
			 * @event
			 * @name ejToolbar#click		
			 * @param {Object} argument Event parameters from Toolbar    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the Toolbar model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.target  returns the target of the current object.
			 * @param {object} argument.currentTarget returns the target of the current object. 
			 * @param {boolean}  argument.status return the Toolbar state
			 * @example 
			 * //create event for Toolbar
             *  $("#toolbar1").ejToolbar({
			 *  click: function (args) {}
             * });
			 * @memberof ejToolbar
			 * @instance
			 */
            click: null,
            /**     
     * Fires after Toolbar control is itemHovered.
     * @event
     * @name ejToolbar#itemHover		
     * @param {Object} argument Event parameters from Toolbar    
     * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
     * @param {object}  argument.model returns the Toolbar model
     * @param {string}  argument.type returns the name of the event
     * @param {object} argument.target  returns the target of the current object.
     * @param {object} argument.currentTarget returns the target of the current object. 
     * @param {boolean}  argument.status return the Toolbar state
     * @example 
     * //itemHover event for Toolbar
     *  $("#toolbar1").ejToolbar({
     *  itemHover: function (args) {}
     * });
     * @memberof ejToolbar
     * @instance
     */
            itemHover: null,
            /**     
			 * Fires after Toolbar control is itemLeave.
			 * @event
			 * @name ejToolbar#itemLeave		
			 * @param {Object} argument Event parameters from Toolbar    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the Toolbar model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.target  returns the target of the current object.
			 * @param {object} argument.currentTarget returns the target of the current object. 
			 * @param {boolean}  argument.status return the Toolbar state
			 * @example 
			 * //itemLeave event for Toolbar
             *  $("#toolbar1").ejToolbar({
			 *  itemLeave: function (args) {}
             * });
			 * @memberof ejToolbar
			 * @instance
			 */
            itemLeave: null,
            /**    
        * Fires when the Toolbar is destroyed successfully.
        * @event
        * @name ejToolbar#destroy 	
        * @param {Object} argument Event parameters from Toolbar    
        * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
        * @param {object}  argument.model returns the Toolbar model
        * @param {string}  argument.type returns the name of the event
        * @example 
        * //destroy event for Toolbar
        * $("#toolbar1").ejToolbar({
        *    destroy: function (args) {}
        * });
        * @memberof ejToolbar
        * @instance
        */
            destroy: null

        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        //Data Types
        dataTypes: {
            height: "string",
            width: "string",
            enabled: "boolean",
            hide: "boolean",
            itemSeparator: "boolean",
            orientation: "enum",
            rtl: "boolean",
            roundedCorner: "boolean",
            cssClass: "string"
        },

        // sample public function
        /**
        * To hide the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		* $("#toolbar1").ejToolbar("hide");// to hide the toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        hide: function () {
            /// <summary>This will set hide function of toolbar </summary>
            if (!this.model.enabled) return false;
            this.element.css("display", "none");
        },
        /**
        * To show the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		* $("#toolbar1").ejToolbar("show");// to show the toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        show: function () {
            /// <summary>This will set show function of toolbar </summary>
            if (!this.model.enabled) return false;
            this.element.css("display", "inline-block");
        },

        /**
         * Create the Toolbar widget
		 * @private
         */

        // constructor function
        _init: function () {
            /// <summary>This will set initialization of toolbar </summary>
            if (this.model.dataSource != null) {
                this._generateTemplate();
            }
            this._initialize();

        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "hide":
                        this._controlVisibleOptions(options[option]);
                        break;
                    case "orientation":
                        this._setOrientation(options[option]);
                        break;
                    case "rtl":
                        this._enableRTL(options[option]);
                        break;
                    case "roundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "fields":
                    case "query":
                    case "dataSource":
                        this._refreshTagItems(option, options[option]);
                        break;
                }
            }
        },
        _refreshTagItems: function (key, value) {
            this.model[key] = value;
            this.element.empty();
            this._generateTemplate();
            this._initialize();
        },
        /**
         * To configure the height		
		 * @private
         */
        _setHeight: function (val) {
            /// <summary>This will set Text property of button </summary>
            this.element.css('height', val);
        },
        /**
         * To configure the width		
		 * @private
         */
        _setWidth: function (val) {
            /// <summary>This will set Text property of button </summary>
            this.element.css('width', val);
        },
        /**
         * To configure the orientation		
		 * @private
         */
        _setOrientation: function (value) {
            /// <summary>This will set orientation of toolbar </summary>
            if (value != ej.orientation.Vertical) {
                this.element.removeClass(this.model.cssClass);
                this.element.addClass("e-toolbarspan " + this.model.cssClass);
                this.itemsContainer.removeClass("e-ul e-vertical");
                this.itemsContainer.addClass("e-ul e-horizontal");
                //this.items.removeClass("e-vertical");
                //this.items.addClass("e-horizontal");

            } else {
                this.element.removeClass("e-toolbarspan " + this.model.cssClass);
                this.element.addClass(this.model.cssClass);
                this.itemsContainer.removeClass("e-ul e-horizontal");
                this.itemsContainer.addClass("e-ul e-vertical");
                //this.items.removeClass("e-horizontal");
                //this.items.addClass("e-vertical");
            }
            this.items.children("img").addClass("e-align e-spriteimg");
            this.items.children("span").addClass("e-align e-spriteimg");
            this.items.children("div").addClass("e-align e-spriteimg");
            this.items.addClass("e-tooltxt");
        },
        /**
         * To configure the custom theme for Toolbar using cssClass property			
		 * @private
         */
        _setSkin: function (skin) {
            this.element.removeClass(this.model.cssClass);
            this.itemsContainer.removeClass(this.model.cssClass);
            this.items.removeClass(this.model.cssClass);
            this.element.addClass(skin);
            this.itemsContainer.addClass(skin);
            this.items.addClass(skin);
        },
        /**
        * destroy the Toolbar widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
	    * &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		* //Initialize the Toolbar object.
		* var toolbarObj = $("#toolbar1").data("ejToolbar");
        *  toolbarObj.destroy();       //Calls the destroy method of the Toolbar.
		* &lt;/script&gt;
				//Destroy the Toolbar using the destroy method.
		* @example 
		* &lt;script&gt;
        *     $("#toolbar1").ejToolbar("destroy");
		*		 &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        _destroy: function () {
            /// <summary>This will destroy toolbar </summary>
            this.element.removeClass("e-widget e-horizontal " + this.model.cssClass);
            this.element.removeClass("e-vertical " + this.model.cssClass);
            this.itemsContainer.removeClass("e-horizontal " + this.model.cssClass);
            this.itemsContainer.removeClass("e-vertical " + this.model.cssClass);
            this.items.removeClass("e-horizontal " + this.model.cssClass);
            this.items.removeClass("e-vertical " + this.model.cssClass);
        },
        /**
         * To initialize the ToolBar		
		 * @private
         */
        _initialize: function () {
            /// <summary>This will initialize the toolbar </summary>
            /*Rendering the controls*/
            $(this.element).attr({ "role": "toolbar", "tabindex": "0" });
            var liList = $($(this.element).children()).find("li");
            this._focusEnable = true;
            this._renderControl();
            this._roundedCorner(this.model.roundedCorner);
            this._wireEvents();
            this._controlVisibleOptions(this.model.hide);
            this._enableRTL(this.model.rtl);
            this._controlStatus(this.model.enabled);
        },
        /**
         * To configure controlVisibleOption to show/hide the toolbar		
		 * @private
         */
        _controlVisibleOptions: function (value) {
            /*Enable Disable Option*/
            value != false ? this.hide() : this.show();
        },
        /**
         * To configure controlStatus ie., to enable/disable the toolbar	
		 * @private
         */
        _controlStatus: function (value) {
            //disable status
            value != true ? this.disableAll() : this.enableAll();
        },
        /**
        * To enable or disable the rounded corner behaviour for the toolbar		
        * @private
        */
        _roundedCorner: function (value) {
            value != false ? this.element.addClass('e-corner-all') : this.element.removeClass('e-corner-all');
        },
        /**
         * To generate Template for the ToolBar control		
		 * @private
         */
        _generateTemplate: function () {
            var proxy = this, queryPromise;
            this.element.css("visibility", "hidden");
            if (this.model.dataSource instanceof ej.DataManager) {
                queryPromise = this.model.dataSource.executeQuery(this.model.query);
                queryPromise.done(function (e) {
                    proxy.itemsSource = e.result;
                    proxy.element.append(proxy._generateTagitems());
                    proxy._initialize();
                    proxy.element.css("visibility", "visible");
                });
            } else {
                proxy.itemsSource = proxy.model.dataSource;
                proxy.element.append(proxy._generateTagitems());
                proxy.element.css("visibility", "visible");
            }
        },
        //
        /**
         * Render Section For Toolbar	
		 * @private
         */
        _renderControl: function () {
            /// <summary>This will render the toolbar </summary>
            this.element.addClass("e-widget");
            this._renderToolbarItems();
            this._setOrientation(this.model.orientation);
            this._renderToolbarSeparator();
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
        },
        /**
         * Render Section For ToolbarItems	
		 * @private
         */
        _renderToolbarItems: function () {
            this.target = this.element[0];
            this.itemsContainer = this.element.children("ol,ul");
            this.itemsContainer.children("ol,ul").remove();
            this.items = this.itemsContainer.children('li');
            for (var i = 0; i < this.items.length; i++) {
                if (ej.isNullOrUndefined($(this.items[i]).attr("title")))
                    $(this.items[i]).attr("aria-label", this.items[i].id);
            }
            this._liCount = this.items.length;
        },
        /**
         * To generate Tag items for the ToolBar control		
		 * @private
         */
        _generateTagitems: function () {
            var list, i;
            list = this.itemsSource;
            this.ultag = ej.buildTag('ul');
            for (i = 0; i < list.length; i++) {
                this.ultag.append(this._generateLi(list[i]));
            }
            return this.ultag;
        },
        /**
         * To generate Li  for the ToolBar control		
		 * @private
         */
        _generateLi: function (toolbarItem) {
            var litag, imgtag, divtag, i;
            litag = ej.buildTag('li');
            if (toolbarItem[this.model.fields.id])
                litag.attr('id', toolbarItem[this.model.fields.id]);
            if (toolbarItem[this.model.fields.tooltipText])
                litag.attr('title', toolbarItem[this.model.fields.tooltipText]);
            if ((toolbarItem[this.model.fields.imageUrl]) && (toolbarItem[this.model.fields.imageUrl] != "")) {
                //Creating the image tag
                imgtag = ej.buildTag('img.e-align', '', {}, { 'src': toolbarItem[this.model.fields.imageUrl], 'alt': toolbarItem[this.model.fields.text] });
                if ((toolbarItem[this.model.fields.imageAttributes]) && (toolbarItem[this.model.fields.imageAttributes] != "")) {
                    for (i = 0; i < toolbarItem[this.model.fields.imageAttributes].length; i++) {
                        $.each(toolbarItem[this.model.fields.imageAttributes][i], function (index, element) {
                            imgtag.attr(index, element);
                        });
                    }
                }
                litag.append(imgtag);
            }
            if ((toolbarItem[this.model.fields.spriteCSS]) && (toolbarItem[this.model.fields.spriteCSS] != "")) {
                //Creating the Sprite image tag
                divtag = ej.buildTag('div.e-align ' + toolbarItem[this.model.fields.spriteCSS] + ' e-spriteimg');
                litag.append(divtag);
            }
            if ((toolbarItem[this.model.fields.text]) && (toolbarItem[this.model.fields.text] != "")) {
                //Creating text Content inside the  tag
                litag.append(toolbarItem[this.model.fields.text]);
            }
            if ((toolbarItem[this.model.fields.htmlAttributes]) && (toolbarItem[this.model.fields.htmlAttributes] != "")) {
                for (i = 0; i < toolbarItem[this.model.fields.htmlAttributes].length; i++) {
                    $.each(toolbarItem[this.model.fields.htmlAttributes][i], function (index, element) {
                        litag.attr(index, element);
                    });
                }
            }
            return litag;
        },
        /**
         * To renderToolbarSeparator for the ToolBar control		
		 * @private
         */
        _renderToolbarSeparator: function () {
            /// <summary>This will render toolbar items </summary>
            /*Adding separator to each group of elements*/
            var i;
            if (this.model.itemSeparator) {
                for (i = 0; i < this.itemsContainer.length - 1; i++) {
                    $(this.itemsContainer[i]).addClass("e-separator");
                }
            }
        },
        /**
         * To enableRTL for the ToolBar control		
		 * @private
         */
        _enableRTL: function (value) {
            if (value) {
                this.element.addClass('e-rtl');
                this.items.addClass('e-comnrtl');
                if (this.model.orientation == ej.orientation.Horizontal) {
                    this.itemsContainer.addClass('e-comnrtl');
                }
            } else {
                this.element.removeClass('e-rtl');
                this.items.removeClass('e-comnrtl');
                if (this.model.orientation == ej.orientation.Horizontal) {
                    this.itemsContainer.removeClass('e-comnrtl');
                }
            }
            this.model.rtl = value;
            this._renderToolbarSeparator();
        },
        /**
    * To disable an item the Toolbar 		
    * @return jQuery
    * @example 
    * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    * 	&lt;/ul&gt;
    *	&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;/div&gt;
    * &lt;script&gt;
    * $("#toolbar1").ejToolbar("disableItem",$("li")[3]);// to disable the third item in the toolbar
    * &lt;/script&gt;
    * @memberof ejToolbar
    * @instance
     */
        //Disables the toolbarItem of given index
        disableItem: function (lielement) {
            /// <summary>This will disable toolbar item </summary>
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            if (!current.hasClass("e-disable")) {
                current.addClass("e-disable").attr("aria-disabled", true).removeAttr("aria-label");
            }
        },
        //Enables the toolbarItem of given index
        /**
* To enable an item the Toolbar 		
* @return jQuery
* @example 
* &lt;div id="toolbar1"&gt;
*&lt;ul&gt;
*    &lt;li id="Left" title="Left"&gt;
*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
*   &lt;/li&gt;
*    &lt;li id="Center" title="Center"&gt;
*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="Right" title="Right"&gt;
*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="Justify" title="Justify"&gt;
*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
* 	&lt;/ul&gt;
*	&lt;ul&gt;
*    &lt;li id="Bold" title="Bold"&gt;
*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="Italic" title="Italic"&gt;
*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="UndeLine" title="UnderLine"&gt;
*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*&lt;/ul&gt;
*&lt;/div&gt;
* &lt;script&gt;
* $("#toolbar1").ejToolbar("enableItem",$("li")[3]);// to enable the third item in the toolbar
* &lt;/script&gt;
* @memberof ejToolbar
* @instance
 */
        enableItem: function (lielement) {
            /// <summary>This will enable toolbar item </summary>
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            current.removeClass("e-disable").attr("aria-disabled", false);
        },
        //Disables the toolbarItem of given index
        /**
    * To Disable the Toolbar item by item id in the Toolbar 		
    * @return jQuery
    * @example 
    * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    * 	&lt;/ul&gt;
    *	&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;/div&gt;
    * &lt;script&gt;
    *$("#toolbar1").ejToolbar("disableItembyID","ID-name"); //Disable the Toolbar item by item id
    * &lt;/script&gt;
    * @memberof ejToolbar
    * @instance
     */
        disableItembyID: function (liid) {
            /// <summary>This will disable toolbar item by id </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.disableItem(lielement);
        },
        /**
    * To Disable the Toolbar item by item id in the Toolbar 		
    * @return jQuery
    * @example 
    * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    * 	&lt;/ul&gt;
    *	&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;/div&gt;
    * &lt;script&gt;
    *$("#toolbar1").ejToolbar("disableItembyID","ID-name"); //Disable the Toolbar item by item id
    * &lt;/script&gt;
    * @memberof ejToolbar
    * @instance
     */
        enableItembyID: function (liid) {
            /// <summary>This will disable toolbar item by id </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.enableItem(lielement);
        },
        //Disables all the toolbar item

        /**
    * To Disableall item in the Toolbar 		
    * @return jQuery
    * @example 
    * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    * 	&lt;/ul&gt;
    *	&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;/div&gt;
    * &lt;script&gt;
    *$("#toolbar1").ejToolbar("disableAll"); //Disable all item in the Toolbar
    * &lt;/script&gt;
    * @memberof ejToolbar
    * @instance
     */
        disableAll: function () {
            /// <summary>This will disable all toolbar items </summary>
            this.items.addClass("e-disable");
            $(this.element).attr("aria-disabled", true).removeAttr("aria-label");
            this.model.enabled = false;
        },
        //Enables all the toolbar item
        /**
        * To enable all item in the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar("enable"); //enable all item in the Toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        enableAll: function () {
            /// <summary>This will enable all toolbar items </summary>
            this.items.removeClass("e-disable");
            $(this.element).attr("aria-disabled", false);
            this.model.enabled = true;
        },
        //Selects the toolbarItem of given object
        /**
        * To Select the Toolbar item		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar("selectItem",$("li")[3]);//Select the Toolbar item.
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        selectItem: function (lielement) {
            /// <summary>This will select a toolbar item </summary>
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            if (!current.hasClass("e-disable")) {
                current.removeClass("e-active");
                current.addClass("e-active");
            }
        },
        //Deselects the toolbarItem of given object
        /**
    * To Deselect the Toolbar item		
    * @return jQuery
    * @example 
    * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    * 	&lt;/ul&gt;
    *	&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;/div&gt;
    * &lt;script&gt;
    *$("#toolbar1").ejToolbar("deselectItem",$("li")[3]); //Deselect the Toolbar item.
    * &lt;/script&gt;
    * @memberof ejToolbar
    * @instance
     */
        deselectItem: function (lielement) {
            /// <summary>This will deselect toolbar item </summary>
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            if (!current.hasClass("e-disable")) {
                current.removeClass("e-active");
            }
        },

        //Selects the toolbarItem of given ID
        /**
        * To Select the Toolbar item by id	
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar("selectItembyID","ID-name");//Select the Toolbar item by id.
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        selectItembyID: function (liid) {
            /// <summary>This will select an toolbar item by id </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.selectItem(lielement);
        },
        //Deselects the toolbarItem of given ID
        /**
        * To Deselect the Toolbar item by id		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar("deselectItembyID","ID-name"); // To Deselect the Toolbar item by id.
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        deselectItembyID: function (liid) {
            /// <summary>This will deselect an toolbar item by id. </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.deselectItem(lielement);
        },
        //Remove the toolbarItem of given object
        /**
        * To Remove the Toolbar item	
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar("removeItem",$("li")[3]); // Remove the Toolbar item
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        removeItem: function (lielement) {
            /// <summary>This will remove the toolbar item </summary>
            if (!this.model.enabled) return false;
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            current.remove();
        },
        /**
* To Remove the Toolbar item by id	
* @return jQuery
* @example 
* &lt;div id="toolbar1"&gt;
*&lt;ul&gt;
*    &lt;li id="Left" title="Left"&gt;
*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
*   &lt;/li&gt;
*    &lt;li id="Center" title="Center"&gt;
*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="Right" title="Right"&gt;
*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="Justify" title="Justify"&gt;
*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
* 	&lt;/ul&gt;
*	&lt;ul&gt;
*    &lt;li id="Bold" title="Bold"&gt;
*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="Italic" title="Italic"&gt;
*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*    &lt;li id="UndeLine" title="UnderLine"&gt;
*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
*    &lt;/li&gt;
*&lt;/ul&gt;
*&lt;/div&gt;
* &lt;script&gt;
*$("#toolbar1").ejToolbar("removetItembyID","ID-name"); // Remove the Toolbar item by id
* &lt;/script&gt;
* @memberof ejToolbar
* @instance
 */
        //Remove the toolbarItem of given ID
        removeItembyID: function (liid) {
            /// <summary>This will remove the toolbar item by ID </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.removeItem(lielement);
        },
        /*-----------------------Event Handlers -----------------------------------------*/
        /*
		 * Wiring the events to ToolBar control		
		 * @private
         */
        _wireEvents: function () {
            /// <summary>This will wire events for toolbar item </summary>
            var args;
            this._on(this.element, "focus", this._focusElement);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.items, "mouseenter", this._onItemHover);
            this._on(this.items, "mousedown", this._onItemClick);
            this._on(this.items, "mouseup", this._onItemClick);
            this._on(this.items, "mouseleave", this._onItemLeave);
            //this._on(this.items, "keyup", this._OnKeyUp);
            //this._on(this.items, "keydown", this._OnKeyDown);
        },
        /**
        * Section For handle the Item hover event
        * @private
        */
        _onItemHover: function (e) {
            var currentItem = e.currentTarget, targetItem = e.target;
            if (!$(currentItem).hasClass("e-disable")) {
                this.items.removeClass("e-hover");
                $(currentItem).addClass("e-hover");
                args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled };
                this._trigger("itemHover", args);
            }
        },
        /**
        * Section For handle the Item click event
        * @private
        */
        _onItemClick: function (e) {
            var currentItem = e.currentTarget, targetItem = e.target;
            if (e.type == "mousedown") { this._focusEnable = false; }
            else if (e.type == "mouseup") {
                if (!$(currentItem).hasClass("e-disable")) {
                    args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled };
                    this._activeItem = $(e.currentTarget).index();
                    this._trigger("click", args);
                }
            }
        },
        /**
        * Section For handle the Mouse Leave on item event
        * @private
        */
        _onItemLeave: function (e) {
            var currentItem = e.currentTarget, targetItem = e.target;
            if (!$(currentItem).hasClass("e-disable")) {
                $(currentItem).removeClass("e-hover");
                args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled };
                this._trigger("itemLeave", args);
            }
        },
        /**
        * Section to watch the  key down event
        * @private
        */
        _OnKeyDown: function (e) {
            this._focusEnable = true;
            this.e = e;
            var itemsLength = this.items.length - 1, activeItem;
            switch (e.keyCode) {
                case 38: //Up
                    e.preventDefault();
                    this._removeListHover();
                    if ((this._activeItem < 0) || (this._activeItem == null) || (this._activeItem > itemsLength))
                        this._activeItem = 0;
                    else if (this._activeItem == 0)
                        this._activeItem = itemsLength;
                    else
                        this._activeItem -= 1;
                    activeItem = this._getActiveItem();
                    if (!activeItem.hasClass("e-disable")) {
                        this._addListHover();
                    }
                    break;
                case 40: //Down
                    e.preventDefault();
                    this._removeListHover();
                    if ((this._activeItem > itemsLength) || (this._activeItem == null) || (this._activeItem < 0))
                        this._activeItem = itemsLength;
                    else if (this._activeItem == itemsLength)
                        this._activeItem = 0;
                    else
                        this._activeItem += 1;
                    activeItem = this._getActiveItem();
                    if (!activeItem.hasClass("e-disable")) {
                        this._addListHover();
                    }
                    break;
                case 37: //Left
                    e.preventDefault();
                    this._removeListHover();
                    if ((this._activeItem < 0) || (this._activeItem == null) || (this._activeItem > itemsLength))
                        this._activeItem = 0;
                    else if (this._activeItem == 0)
                        this._activeItem = itemsLength;
                    else
                        this._activeItem -= 1;
                    activeItem = this._getActiveItem();
                    if (!activeItem.hasClass("e-disable")) {
                        this._addListHover();
                    }
                    break;
                case 39: //Right
                    e.preventDefault();
                    this._removeListHover();
                    if ((this._activeItem > itemsLength) || (this._activeItem == null) || (this._activeItem < 0))
                        this._activeItem = itemsLength;
                    else if (this._activeItem == itemsLength)
                        this._activeItem = 0;
                    else
                        this._activeItem += 1;
                    activeItem = this._getActiveItem();
                    if (!activeItem.hasClass("e-disable")) {
                        this._addListHover();
                    }
                    break;
                case 13: //enter key
                    break;
                case 32: //space key
                    break;
                case 33: //pup
                    e.preventDefault();
                    this._removeListHover();
                    this._activeItem = 0;
                    activeItem = this._getActiveItem();
                    if (!activeItem.hasClass("e-disable")) {
                        this._addListHover();
                    }
                    break;
                case 34: //pdwn
                    e.preventDefault();
                    this._removeListHover();
                    this._activeItem = itemsLength;
                    activeItem = this._getActiveItem();
                    if (!activeItem.hasClass("e-disable")) {
                        this._addListHover();
                    }
                    break;
                case 35: //endkey
                    e.preventDefault();
                    this._removeListHover();
                    this._activeItem = itemsLength;
                    activeItem = this._getActiveItem();
                    if (!activeItem.hasClass("e-disable")) {
                        this._addListHover();
                    }
                    break;
                case 36: //Home key
                    e.preventDefault();
                    this._removeListHover();
                    this._activeItem = 0;
                    activeItem = this._getActiveItem();
                    if (!activeItem.hasClass("e-disable")) {
                        this._addListHover();
                    }
                    break;
            }
        },
        /**
        * Section to watch the  KeyUp event
        * @private
        */
        _OnKeyUp: function (e) {
            this.e = e;
            var itemsLength = this.items.length, activeItem;
            switch (e.keyCode) {
                case 38: //Up
                    break;
                case 40: //Down
                    break;
                case 37: //Left
                    break;
                case 39: //Right
                    break;
                case 13: //enter key
                    activeItem = this._getActiveItem();
                    if ($(e.target).is("div")) {
                        var currentItem = this._getTargetItem(), targetItem = this._getTargetItem();
                    } else {
                        var currentItem = $(e.target).closest("li")[0], targetItem = e.target;
                    }
                    if (!$(currentItem).hasClass("e-disable")) {
                        args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled, event: this.e };
                        this._trigger("click", args);
                        this._removeListHover();
                    }
                    break;
                case 32: //space key
                    activeItem = this._getActiveItem();
                    if ($(e.target).is("div")) {
                        var currentItem = this._getTargetItem(), targetItem = this._getTargetItem();
                    } else {
                        var currentItem = $(e.target).closest("li")[0], targetItem = e.target;
                    }
                    if (!activeItem.hasClass("e-disable")) {
                        args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled, event: this.e };
                        this._trigger("click", args);
                        this._removeListHover();
                    }
                    break;
                case 33: //pup
                    break;
                case 34: //pdwn
                    break;
                case 35: //endkey
                    break;
                case 36: //Home key
                    break;
                case 27:         // Esc Key
                    this.element.focusout();
                    this._removeListHover();
                    break;
            }
        },
        /**
        * Section For handle to remove class after hover event
        * @private
        */
        _removeListHover: function () {
            $(this.items).removeClass("e-hover");
        },
        /**
         * Section For handle to add class for hover event
		 * @private
         */
        _addListHover: function () {
            var activeItem = this._getActiveItem();
            activeItem.addClass("e-hover");
            activeItem[0].scrollIntoView(false);
            activeItem.focus();
        },
        /**
         *  To get the ActiveItem		
		 * @private
         */
        _getActiveItem: function () {
            return $($(this.items)[this._activeItem]);
        },
        /**
         * To get the TargetItem		
		 * @private
         */
        _getTargetItem: function () {
            return $(this.items)[this._activeItem];
        },
        /**
         * Section For handle Blur event
		 * @private
         */
        _targetBlur: function (e) {
            e.preventDefault();
            this.element.focusout();
            this._removeListHover();
            this.element.removeClass("e-focus");
            this._off(this.element, "keyup", this._OnKeyUp);
            this._off(this.element, "keydown", this._OnKeyDown);
            // this._isFocused = false;
            /* Raise focusOut Event*/
            if (this.model.focusOut)
                this._trigger("focusOut");
        },
        /**
         * Section For handle Focus event
		 * @private
         */
        _focusElement: function (e) {
            this.element.addClass("e-focus");
            this._on(this.element, "keyup", this._OnKeyUp);
            this._on(this.element, "keydown", this._OnKeyDown);

            if (!this._focusEnable) {
                this._focusEnable = true;
                return;
            }
            this._removeListHover();
            this._activeItem = 0;
            activeItem = this._getActiveItem();
            if (!activeItem.hasClass("e-disable")) {
                this._addListHover();
            }
        },
    });

})(jQuery, Syncfusion);


;
/**
* @fileOverview Plugin to style the Html Accordion elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejAccordion
   * @requires jQuery
   * @requires ej.common.all
   * @classdesc Custom Design for Accordion control.
   * @example 
   * &lt;div id="accordion"&gt;
   *        &lt;h3&gt;
   *            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
   *        &lt;div&gt;
   *Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
   *        &lt;/div&gt;
   *        &lt;h3&gt;
   *            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
   *        &lt;div&gt;
   *Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
   *        &lt;/div&gt;
   *        &lt;h3&gt;
   *            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
   *        &lt;div&gt;
   *Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
   *        &lt;/div&gt;
   *    &lt;/div&gt;
   *  &lt;script type="text/javascript"&gt;
   *        $(function () {
   *            // Initialize Accordion control creation.
   *            $("# accordion").ejAccordion();
   *        });
   *    &lt;/script&gt; 
   */

    ej.widget("ejAccordion", "ej.Accordion", {
        _rootCSS: "e-acrdn",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,

        // default model
        defaults: {
            /// <summary>This Contains default property of Sycnfusion Accordion </summary>
            /**		
			* Specifies the collapsible state of accordion control.
			* @default false
			* @type {boolean}
			* @example 
			* // Allow to collapsible on initialization. 
			* 	//To set collapsible API value 
			* 	 $("#accordion").ejAccordion({ collapsible: true});
			* @example 
			* //Get or set the Accordion state, after initialization:<br>
			*	// Gets the collapsible API value.		
			*	$("#accordion").ejAccordion("option", "collapsible");			
			*	// Sets the collapsible API
			*	$("#accordion").ejAccordion("option", "collapsible",  false );
			 * @memberof ejAccordion
			* @instance
			*/
            collapsible: false,
            /**		
			*  AJAX setting options  help to customize AJAX loading.
			* @default ""
			* @type {JSONObject}
			* @example
			* //Ajax options on initialization. 
			* $("#accordion").ejAccordion({
			*  ajaxOptions: {
			*      type: 'GET',
			*      dataType: "html",
			*      contentType: "html",
			*      async: true
			*     }
			*   });
			*@example
			* Get or set the ajaxOptions API after initialization. 
			* //Get the ajaxOptions value. 
			* $("#accordion").ejAccordion("option", "ajaxOptions"); 
			* //Set the ajaxOptions. 
			* $("#accordion").ejAccordion("option", "ajaxOptions", { type: 'GET', dataType: "html", contentType: "html", async: true });
			* @memberof ejAccordion
			* @instance
			*/
            ajaxOptions: { type: 'GET', cache: false, data: {}, dataType: "html", contentType: "html", async: true },
            /**		
			*  The events API binds the action for activating the accordion header. Users can activate the header by using mouse actions such as mouse-over, mouse-up, mouse-down, and so on.
			* @default "click"
			* @type {String}
			* @example			 
			* // Bind events API on initialization. 
			* $("#accordion").ejAccordion({ events: "mouseover" });
			*@example
			* Get or set the events API after initialization.
			*	//Get the events API value. 
			* $("#accordion").ejAccordion("option", "events"); 
			* //Set the events API value.
			* $("#accordion").ejAccordion("option", "events", "mouseover");
			* @memberof ejAccordion
			* @instance
			*/
            events: "click",
            /**		
			* @example
			* The header icon can be customized using this API. It has two properties: header as the collapsing header CSS class name, and selectedHeader as the active header CSS class name. It accepts CSS class names with string as the type.		*
			*•header: This class name set to collapsing header. This accepts the string type value.
			*•selectedHeader: This class name set to expanded header. This accepts the string type value.
			* @default "{  header: "e-acrdn-collapsicon",  selectedHeader: "e-acrdn-expandicon"   }"
			* @type {JSONObject}
			* @example			 
			* //iconCSS on initialization. 
			*   $("#accordion").ejAccordion({
			*		iconCSS: {
			*			header: "collapsicon",
			*			selectedHeader: "activeicon"
			*		}
			*	});
			*@example
			* Get or set the iconCSS API after initialization. 
			* //Get the iconCSS API value. 
			* $("#accordion").ejAccordion("option", "iconCSS");  
			* //Set the iconCSS API.
			* $("#accordion").ejAccordion("option", "iconCSS", { header: "collapsicon", selectedHeader: "activeicon" });
			* @memberof ejAccordion
			* @instance
			*/
            iconCSS: {
                header: "e-collaps",
                selectedHeader: "e-expand"
            },
            /**		
			*  Adjusts the content panel height based on the given option (content, auto, or fill). By default, the panel heights are adjusted based on the content.
			* @default "content"
			* @type {Enum/String}
			*@example
			* //heightStyle for content div on initialization. 
			* $("#basicAccordion").ejAccordion({ heightStyle: ej.Accordion.heightStyle.Auto }); //enum 
			* //Pass value on string type. 
			*$("#basicAccordion").ejAccordion({ heightStyle: "auto" }); 
			*@example
			* Get or set the heightStyle API after initialization:
			*	//Get the heightStyle value. 
			* $("#accordion").ejAccordion("option", "heightStyle"); 
			* //Set the heightStyle.
			* $("#accordion").ejAccordion("option", "heightStyle", "auto");
			* @memberof ejAccordion
			* @instance
			*/
            heightStyle: "content",
            /**		
			* The given index header will activate (open). If collapsible is set to true, and a negative value is given, then all headers are collapsed. Otherwise, the first panel is activated.
			* @default 0
			* @type {Integer}
			*@example
			*  //Activate given header on initialization.
			*	$("#accordion").ejAccordion({ selectedItemIndex: 1 });
			*@example
			* Get or set the selectedItemIndex API after initialization.
			*	//Get the selectedItemIndex value.
			* $("#accordion").ejAccordion("option", "selectedItemIndex"); 
			* //Set the selectedItemIndex. 
			* $("#accordion").ejAccordion("option", "selectedItemIndex", 1);
			* @memberof ejAccordion
			* @instance
			*/
            selectedItemIndex: 0,
            /**		
			* Sets the root class for the Accordion theme. This cssClass API enables the use of custom skinning options for the Accordion control. By defining the root class using this API, we need to include this root class in the CSS.
			* @default ""
			* @type {String}
			*@example
			*  //cssClass on initialization.
			*	$("#accordion").ejAccordion({cssClass: "gradient-lime" });
			*@example
			* Get or set the cssClass API after initialization:
			*	//Get the cssClass value.
			* $("#accordion").ejAccordion("option", "cssClass"); 
			*  //Set the cssClass.
			* $("#accordion").ejAccordion("option", "cssClass", "gradient-lime");
			* @memberof ejAccordion
			* @instance
			*/
            cssClass: "",
            /**		
			* Display headers and panel text from right-to-left.
			* @default false
			* @type {boolean}
			*@example
			*   // Enable rtl on initialization.
			*	$("#accordion").ejAccordion({ rtl: true});
			*@example
			* Get or set the rtl API after initialization.
			*	//Get the rtl value.
			* $("#accordion").ejAccordion("option", "rtl"); 
			* //Set the rtl.
			* $("#accordion").ejAccordion("option", "rtl", true);
			* @memberof ejAccordion
			* @instance
			*/
            rtl: false,
            /**		
			* Displays rounded corner borders on the Accordion control's panels and headers.
			* @default false
			* @type {boolean}
			*@example
			*   // Enable roundedCorner on initialization.
			*	$("#accordion").ejAccordion({ roundedCorner: true});
			*@example
			* Get or set the roundedCorner API after initialization.
			*	//Get the roundedCorner value.
			* $("#accordion").ejAccordion("option", "roundedCorner"); 
			* //Set the roundedCorner.
			* $("#accordion").ejAccordion("option", "roundedCorner", true);
			* @memberof ejAccordion
			* @instance
			*/
            roundedCorner: false,
            /**		
			* Accordion headers can be expanded and collapsed on keyboard action. 
			* @default true
			* @type {boolean}
			*@example
			*   // Enable allowKeyboardNavigation on initialization.
			*	$("#accordion").ejAccordion({ allowKeyboardNavigation: true});
			*@example
			* Get or set the allowKeyboardNavigation API after initialization.
			*	//Get the allowKeyboardNavigation value.
			* $("#accordion").ejAccordion("option", "allowKeyboardNavigation"); 
			* //Set the allowKeyboardNavigation.
			* $("#accordion").ejAccordion("option", "allowKeyboardNavigation", true);
			* @memberof ejAccordion
			* @instance
			*/
            allowKeyboardNavigation: true,
            /**		
			* Multiple content panels to activate at a time. 
			* @default false
			* @type {boolean}
			*@example
			*   // Enable multipleOpen on initialization.
			*	$("#accordion").ejAccordion({ multipleOpen: true});
			*@example
			* Get or set the multipleOpen API after initialization.
			*	//Get the multipleOpen value.
			* $("#accordion").ejAccordion("option", "multipleOpen"); 
			* //Set the multipleOpen.
			* $("#accordion").ejAccordion("option", "multipleOpen", true);
			* @memberof ejAccordion
			* @instance
			*/
            multipleOpen: false,
            /**		
			* To activate multiple headers for the accordion, give the index value for array type for this API. The multipleOpen property should be set as true. 
			* @default [0]
			* @type {Integerarray}
			*@example
			*   // selectedItems on initialization.
			*	$("#accordion").ejAccordion({ selectedItems: [0,1] });
			*@example
			* //Get or set the selectedItems API after initialization.
			*	//Get the selectedItems value.
			* $("#accordion").ejAccordion("option", "selectedItems"); 
			* //Set the selectedItems.
			* $("#accordion").ejAccordion("option", "selectedItems", [0,1] );
			* @memberof ejAccordion
			* @instance
			*/
            selectedItems: [0],
            /**		
			* To deactivate multiple headers for the accordion, give the index value for array type for this API. The multipleOpen property should be set as true. 
			* @default []
			* @type {Integerarray}
			*@example
			*   // disabledItems on initialization.
			*	$("#accordion").ejAccordion({ disabledItems: [0,1] });
			*@example
			* * //Get or set the disabledItems API after initialization.
			*	//Get the disabledItems value.
			* $("#accordion").ejAccordion("option", "disabledItems"); 
			* //Set the disabledItems.
			* $("#accordion").ejAccordion("option", "disabledItems", [0,1] );
			* @memberof ejAccordion
			* @instance
			*/
            disabledItems: [],
            /**		
			* Save current model value to browser cookies for maintaining states. When refreshing the accordion control page, the model value is applied from browser cookies or HTML 5 local storage.
			* @default false
			* @type {Boolean}
			*@example
			*   // Enable persist on initialization.
			*	$("#accordion").ejAccordion({ persist: true});
			*@example
			* //Get or set the persist API after initialization.
			*	//Get the persist value.
			* $("#accordion").ejAccordion("option", "persist"); 
			* //Set the persist.
			* $("#accordion").ejAccordion("option", "persist", true);
			* @memberof ejAccordion
			* @instance
			*/
            persist: false,
            /**		
			* With the Enabled property, you can enable or disable the Accordion.
			* @default true
			* @type {Boolean}
			*@example
			*   // Enable accordion on initialization.
			*	$("#accordion").ejAccordion({ enabled: true});
			*@example
			* //Get or set the Enabled API after initialization.
			*	//Get the enabled API value.
			* $("#accordion").ejAccordion("option", "enabled"); 
			* //Set the Enabled API value.
			* $("#accordion").ejAccordion("option", "enabled", true);
			* @memberof ejAccordion
			* @instance
			*/
            enabled: true,
            //Events
            /**     
            * Triggered after the AJAX content loads. Arguments have current model values.
            * @event
            * @name ejAccordion#ajaxLoad		
            * @param {Object} argument Event parameters from accordion     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the accordion model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //initial ajaxLoad event
            *	$("#accordion").ejAccordion({ ajaxLoad: function (args) {}
            * });
            * @memberof ejAccordion
            * @instance
            */
            ajaxLoad: null,
            /**     
			 * Triggered before the AJAX content is loaded in a content panel. Arguments have location of the content (URL) and current model value.
			 * @event
			 * @name ejAccordion#ajaxBeforeLoad		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url returns current ajax content location
			 * @example 
			 * //initial ajaxBeforeLoad event
			 *	$("#accordion").ejAccordion({ ajaxBeforeLoad: function (args) {}
			 * });
			 * @memberof ejAccordion
			 * @instance
			 */
            ajaxBeforeLoad: null,
            /**     
			 * Triggered after a Accordion item is activated. Argument values are activeindex, activeHeader, activePanel, and current model value.
			 * @event
			 * @name ejAccordion#active		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {number} argument.activeIndex returns active index
             * @param {string} argument.activeHeader returns current active header
             * @param {string} argument.activePanel returns current active panel
			 * @example 
			 * //initial active event
			 *	$("#accordion").ejAccordion({ active: function (args) {}
			 * });
			 * @memberof ejAccordion
			 * @instance
			 */
            active: null,
            /**     
			 * Triggered before a tab item is active. Arguments have active index and model values.
			 * @event
			 * @name ejAccordion#beforeActive		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {number} argument.activeIndex returns active index
			 * @example 
			 * //initial beforeActive event
			 *	$("#accordion").ejAccordion({ beforeActive: function (args) {}
			 * });
			 * @memberof ejAccordion
			 * @instance
			 */
            beforeActive: null,
            /**     
			 * Triggered after AJAX success action. Arguments have URL, content, and current model values.
			 * @event
			 * @name ejAccordion#ajaxSuccess		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url returns current ajax content location
			 * @param {string}  argument.data returns the succesfull data sent.
			 * @param {string}  argument.content returns the ajax content.
			 * @example 
			 * //initial ajaxSuccess event
			 *	$("#accordion").ejAccordion({ ajaxSuccess: function (args) {}
			 * });
			 * @memberof ejAccordion
			 * @instance
			 */
            ajaxSuccess: null,
            /**     
			 * Triggered after AJAX load failed action. Arguments have URL, error message, and current model value.
			 * @event
			 * @name ejAccordion#ajaxError		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url returns current ajax content location
			 * @param {string}  argument.data returns the failed data sent.
			 * @example 
			 * //initial ajaxError event
			 *	$("#accordion").ejAccordion({ ajaxError: function (args) {}
			 * });
			 * @memberof ejAccordion
			 * @instance
			 */
            ajaxError: null
        },
        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            cssClass: "string",
            collapsible: "boolean",
            enabled: "boolean",
            events: "string",
            heightStyle: "enum",
            ajaxOptions: "data",
            iconCSS: "data",
            selectedItems: "data",
            disabledItems: "data"
        },

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "events": {
                        this._off(this._headers, this.model.events);
                        this._on(this._headers, options[key], this._headerEventHandler);
                        break;
                    }
                    case "disabledItems": this.disableItems(options[key]); break;
                    case "enabled": this._enabledAction(options[key]); break;
                    case "selectedItemIndex": this._activeTab(options[key]); break;
                    case "heightStyle": this._setHeightStyle(options[key]); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "roundedCorner": this._roundedCorner(options[key]); break;
                    case "iconCSS":
                        {
                            var icons = this.model.iconCSS;
                            var newIcons = options[key];
                            this._headers.find("span." + icons.header).removeClass(icons.header).addClass(newIcons.header);
                            this._headers.find("span." + icons.selectedHeader).removeClass(icons.selectedHeader).addClass(newIcons.selectedHeader);
                            break;
                        }
                    case "allowKeyboardNavigation": {
                        if (options[key]) {
                            this._off(this.element, "keydown");
                            this._on(this.element, 'keydown', this._keyPress);
                        }
                        else
                            this._off(this.element, "keydown");
                        break;
                    }
                    case "rtl":
                        {
                            if (options[key])
                                this.element.addClass("e-rtl");
                            else
                                this.element.removeClass("e-rtl");
                            break;
                        }
                    case "selectedItems": this._selectedItemsAction(options[key]); break;
                }
            }
        },

        // all events bound using this._on will be unbind automatically
        /**
       * destroy the Accordion widget
       * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
       * @alias destroy
       * @return jQuery
       * @example
       * //HTML template
       *    &lt;div id="accordion"&gt;
       *        &lt;h3&gt;
       *            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
       *        &lt;div&gt;
       *Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
       *        &lt;/div&gt;
       *        &lt;h3&gt;
       *            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
       *        &lt;div&gt;
       *Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
       *        &lt;/div&gt;
       *        &lt;h3&gt;
       *            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
       *        &lt;div&gt;
       *Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
       *        &lt;/div&gt;
       *    &lt;/div&gt;
       *&lt;script&gt;
       *var accObj = $("#accordion").data("ejAccordion");
       *accObj.destroy(); //Calls the destroy method of Accordion.
       *&lt;/script&gt;
       * @example 
       * //HTML template
       *    &lt;div id="accordion"&gt;
       *        &lt;h3&gt;
       *            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
       *        &lt;div&gt;
       *Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
       *        &lt;/div&gt;
       *        &lt;h3&gt;
       *            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
       *        &lt;div&gt;
       *Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
       *        &lt;/div&gt;
       *        &lt;h3&gt;
       *            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
       *        &lt;div&gt;
       *Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
       *        &lt;/div&gt;
       *    &lt;/div&gt;
       *&lt;script&gt;
       * // destroy the Accordion
       * $("#Accordion").ejAccordion("destroy");	
       * &lt;/script&gt;
       * @memberof ejAccordion
       * @instance
       */
        _destroy: function () {
            /// <summary>This function is  used to destroy the Accordion Object</summary>
            this._removeBaseClass();
            this.element.children('.e-disable').removeClass("e-disable");
            this.element.removeClass("e-acrdn");
            this._headers.removeClass("e-active  e-state-disabled");
            this._contentPanels.removeClass("e-content-active e-disable");
            $(this._headers.find(".e-icon")).remove();
        },

        // constructor function
        /**
         * Create the Accordion widget
		 * @private
         */
        _init: function () {
            this._renderControl();
        },

        // render accordion control
        /**
         * Render Section For accordion control		
		 * @private
         */
        _renderControl: function () {
            /// <summary>This function is  used to Render the Accordion Object</summary>
            this._headers = this.element.find("> :even");
            this.element.attr("tabindex", 0).attr("role", "tablist");
            this._contentPanels = this._headers.next();
            //Index check negative values
            this._nagativeIndexCheck(this.model.selectedItemIndex);
            // Add the default class
            this._addBaseClass();
            this._setHeightStyle(this.model.heightStyle);
            // Add header icons
            this._renderHeaderIcon();
            this._wireEvents();
            this._roundedCorner(this.model.roundedCorner);
            //Activate the header & content
            this._selectedItemsAction(this.model.selectedItems);
            this._enabledAction(this.model.enabled);
        },
        //Skin Change at run time
        /**
         * To configure the custom theme for accordion using cssClass property		
		 * @private
         */
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
         * To configure the Selected ites for accordion using SelectedIndex property		
		 * @private
         */
        _selectedItemsAction: function (activeList) {
            if (this.model.multipleOpen) {
                for (var seletedindex = 0; seletedindex < activeList.length; seletedindex++) {
                    this._activeTab(activeList[seletedindex]);
                }
            }
            else
                this._activeTab(this.model.selectedItemIndex);
        },
        /**
         * To enable or disable the accordion state		
		 * @private
         */
        _enabledAction: function (flag) {
            if (flag) {
                this.element.removeClass("e-disable");
            }
            else {
                this.element.addClass("e-disable");
            }
        },
        //Add base class for headers and content panels
        /**
         * To configure the default classes for rendering Accordion control		
		 * @private
         */
        _addBaseClass: function () {
            this.element.addClass(this.model.cssClass + "e-widget");
            if (this.model.rtl)
                this.element.addClass("e-rtl");
            this._headers.addClass(" e-select").attr("role", "tab").attr("tabindex", 0);
            this._contentPanels.addClass("e-content").attr("role", "tabpanel").attr("aria-hidden", true).hide(); // hide all the content panels while loding
            $(this._contentPanels[this.model.selectedItemIndex]).show();
        },
        //Remove base class for headers and content panels
        /**
         * To remove the default classes for rendering Accordion control		
		 * @private
         */
        _removeBaseClass: function () {
            this._headers.removeClass(" e-select");
            this._contentPanels.removeClass("e-content").show(); // hide all the content panels while loding
        },
        /**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */
        _roundedCorner: function (value) {
            if (value) {
                this._headers.addClass('e-corner-all');
                this.element.find('.e-active').removeClass('e-corner-all').addClass('e-corner-top');
                this.element.find('.e-acrdn-content-active').addClass('e-corner-bottom');
            }
            else if (this._headers.hasClass('e-corner-all')) {
                this._headers.removeClass('e-corner-all');
                this.element.find('.e-active').removeClass('e-corner-top');
                this.element.find('.e-acrdn-content-active').addClass('e-corner-bottom');
            }
        },
        /**
         * To check the negative index value 		
		 * @private
         */
        _nagativeIndexCheck: function (index) {
            /// <summary>This function is  used to Check the nagative value of the index</summary>
            if (index < 0 || index >= this._headers.length)
                this.model.selectedItemIndex = 0;
        },
        /**
         * To activate the selected Item of the accordion control		
		 * @private
         */
        _activeTab: function (index) {
            /// <summary>This function is  used Activate the selected Item of the accordion control</summary>
            if ($.inArray(index, this.model.disabledItems) < 0) {
                var data = {
                    activeIndex: index
                };
                //triggering the onBeforeActive event
                if (true === (this._trigger("beforeActive", data))) {
                    return false;
                }
                if (!this.model.multipleOpen)
                    this._hideTab(this.model.selectedItemIndex);
                this.model.selectedItemIndex = index;
                var icons = this.model.iconCSS;
                this._headers.attr("tabindex", 0).removeAttr("aria-expanded");
                this._activeHeader = this._headers.eq(index).addClass("e-active ").attr("aria-selected", true).attr("tabindex", 0).attr("aria-expanded", true);
                this._activeHeader.find(".e-icon")
                        .removeClass(icons.header)
                        .addClass(icons.selectedHeader);
                this._addSelectedItems(index);
                this._ajaxLoad();
                this._roundedCorner(this.model.roundedCorner);
                data = {
                    activeHeader: this._activeHeader,
                    activePanel: this._activeContent,
                    activeIndex: this.model.selectedItemIndex
                };
                var proxy = this;
                this._activeContent = this._activeHeader.next().addClass("e-acrdn-content-active").removeAttr("aria-hidden", false).slideDown(300, "easeOutQuad", function () {
                    // Animation complete  //triggering the onActive event
                    proxy._trigger("active", data);
                }); //.animate({ height: "show" }, 'fast');
            }
        },
        /**
         * Configure to hide the previous active item of the accordion control		
		 * @private
         */
        //This function is  used hide the prev active item of the accordion control
        _hideTab: function (index) {
            var icons = this.model.iconCSS;
            if (index >= 0) {
                this._activeHeader = this._headers.eq(index).removeClass("e-active ").removeAttr("aria-selected").attr("tabindex", 0).attr("aria-expanded", false);
                this._activeHeader.find(".e-icon")
                        .removeClass(icons.selectedHeader)
                        .addClass(icons.header);
                this._activeHeader.next().removeClass("e-acrdn-content-active").slideUp(200, "easeOutQuad").attr("aria-hidden", true);//.animate({ height: "hide" }, 'fast');
                this._deleteSelectedItems(index);
            }
        },
        /**
         * Configure to create the Header icon for the accordion control		
		 * @private
         */
        //Cretae Header Icon 
        _renderHeaderIcon: function () {
            if (this._headers.find(".e-icon").length <= 0) {
                var icons = this.model.iconCSS;
                if (icons) {
                    var iconSpan = ej.buildTag("span.e-icon " + icons.header).prependTo(this._headers);
                    this._headers.eq(this.model.selectedItemIndex).find(".e-icon")
                        .removeClass(icons.header)
                        .addClass(icons.selectedHeader);
                }
            }
        },
        /**
         * To Configure ajaxload for the accordion control		
		 * @private
         */
        //Ajax Load function 
        _ajaxLoad: function () {
            var anchor = this._activeHeader.find("a[href]");
            if (anchor.length > 0) {
                var content = $(this._contentPanels[this.model.selectedItemIndex]);
                var href = $(anchor).attr("href");
                if (href && href !== "#")
                    this._sendAjaxOptions(content, anchor[0]);
            }
        },
        /**
         * To handle the ajax request for the accordion control		
		 * @private
         */
        //handling the ajax request
        _sendAjaxOptions: function (content, link) {
            // TODO load waiting popup
            //triggering the onBeforeLoad event
            if (true === (this._trigger("ajaxBeforeLoad", { url: link })))
                return false;
            content.addClass("e-load");
            var proxy = this;
            var curTabTitle = $(link).html();
            var _hrefLink = link.href.replace("#", "");
            var _ajaxOptions = {
                type: this.model.ajaxOptions.type, cache: this.model.ajaxOptions.cache, url: _hrefLink, data: this.model.ajaxOptions.data,
                dataType: this.model.ajaxOptions.dataType, contentType: this.model.ajaxOptions.contentType, async: this.model.ajaxOptions.async,
                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, content, link, curTabTitle);
                    } catch (e) {

                    }
                }, "error": function () {
                    try {
                        proxy._ajaxErrorHandler(data, link, proxy.model.selectedItemIndex, curTabTitle);
                    } catch (e) {

                    }
                }
            };
            this._sendAjaxRequest(_ajaxOptions);
        },
        /**
         * Configure to map the Ajax request	
		 * @private
         */
        //Ajax Request mapping
        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },
        /**
         * Configure to handle the Ajax load success event	
		 * @private
         */
        //Ajax load success event handler
        _ajaxSuccessHandler: function (data, content, link, curTabTitle) {
            if (curTabTitle != null)
                $(link).html(curTabTitle);
            content.removeClass("e-load");
            content.html(data).addClass("e-acrdn-loaded"); //to indicate the content is already loaded
            var eventData = { data: data, url: link, content: content };
            this._trigger("ajaxSuccess", eventData);
            //triggering the onLoad event
            this._trigger("ajaxLoad");
        },
        /**
         * Configure to handle the Ajax load failed event	
		 * @private
         */
        //Ajax load erroe event handler
        _ajaxErrorHandler: function (data, link, index, title) {
            this._trigger("ajaxError", { data: data, url: link });
            //triggering the onLoad event
            this._trigger("ajaxLoad");
        },
        /**
         * Configure to set the content panels height based on the height style
		 * @private
         */
        //Set the content panels height based on the height style
        _setHeightStyle: function (heightStyle) {
            var proxy = this, maxHeight;
            if (ej.Accordion.heightStyle.Fill == heightStyle) {
                maxHeight = this._getDimension($(this.element).parent(), "height");
                $(this.element).parent().css({ "overflow": "auto" });
                proxy._headers.each(function () {
                    maxHeight -= proxy._getDimension($(this), "outerHeight");
                });
                var maxPadding = 0;
                proxy._headers.next().each(function () {
                    maxPadding = Math.max(maxPadding, proxy._getDimension($(this), "innerHeight") - proxy._getDimension($(this), "height"));
                }).height(maxHeight - maxPadding).css({ "overflow": "auto" });

            } else if (ej.Accordion.heightStyle.Auto == heightStyle) {
                maxHeight = 0;
                proxy._headers.next().each(function () {
                    maxHeight = Math.max(maxHeight, proxy._getDimension($(this), "outerHeight"));
                }).height(maxHeight);
                this.maxAutoHeight = maxHeight;
            }
        },
        /**
         * Configure to set the dimensions for accordion control header and content panel
		 * @private
         */
        _getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            var prop = { visibility: 'hidden', display: 'block' };
            var tmp = [];
            $hidden.each(function () {
                var temp = {}, name;
                for (name in prop) {
                    temp[name] = this.style[name];
                    this.style[name] = prop[name];
                }
                tmp.push(temp);
            });
            value = /(outer)/g.test(method) ?
            $(element)[method](true) :
           $(element)[method]();

            $hidden.each(function (i) {
                var temp = tmp[i], name;
                for (name in prop) {
                    this.style[name] = temp[name];
                }
            });

            return value;
        },
        /**
         * Configure to handle header click event
		 * @private
         */
        // Header Click event handler
        _headerEventHandler: function (event) {
            if (this.model.enabled) {
                event.preventDefault();
                var clicked = $(event.currentTarget);
                var index = this._headers.index(clicked);
                if (this.model.multipleOpen && this._headers.eq(index).hasClass("e-active")) {
                    this._hideTab(index);
                }
                else if (this.model.selectedItemIndex == index && this.model.collapsible) {
                    this._hideTab(this.model.selectedItemIndex);
                    this.model.selectedItemIndex = -1
                }
                else if (!this._headers.eq(index).hasClass("e-active")) {
                    this._activeTab(index);
                }
            }
        },
        /**
         * Configure to add selected items based on index
		 * @private
         */
        _addSelectedItems: function (index) {
            if ($.inArray(index, this.model.selectedItems) < 0)
                this.model.selectedItems.push(index);
        },
        /**
         * Configure to delete selected items based on index
		 * @private
         */
        _deleteSelectedItems: function (index) {
            if ($.inArray(index, this.model.selectedItems) > -1) {
                position = $.inArray(index, this.model.selectedItems);
                this.model.selectedItems.splice(position, 1);
            }
        },
        /**
         * Configure to delete disabled items based on index
		 * @private
         */
        _deleteDisabledItems: function (index) {
            if ($.inArray(index, this.model.disabledItems) > -1) {
                position = $.inArray(index, this.model.disabledItems);
                this.model.disabledItems.splice(position, 1);
            }
        },
        /**
         * Configure for keyboard navigation 
		 * @private
         */
        _keyPress: function (e) {
            if (this.model.enabled) {
                var index, currentEle, targetEle = $(e.target);
                if ((targetEle.hasClass("e-select")) || (targetEle.hasClass("e-acrdn"))) {
                    index = this.model.selectedItemIndex;//this._headers.index(targetEle.siblings(".e-active")[0])
                }
                //else if(targetEle.hasClass("e-select"))
                if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
                else if (e.which) code = e.which; // ns4 and opera
                else code = e.charCode;
                if ((targetEle.hasClass("e-select")) || (targetEle.hasClass("e-acrdn"))) {
                    switch (code) {
                        //down arrow
                        case 39:
                        case 40:
                            e.preventDefault();
                            index = index == (this.panelCount() - 1) ? 0 : index + 1;
                            this._activeTab(index);
                            currentEle = $(this._headers[index]);
                            break;
                            //up arrow
                        case 38:
                        case 37:
                            e.preventDefault();
                            index = index == 0 ? (this.panelCount() - 1) : index - 1;
                            this._activeTab(index);
                            currentEle = $(this._headers[index]);
                            break;
                            //End key
                        case 35:
                            e.preventDefault();
                            currentEle = $(this._headers[this.panelCount() - 1]);
                            this._activeTab((this.panelCount() - 1));
                            break;
                            //Home key
                        case 36:
                            e.preventDefault();
                            currentEle = $(this._headers[0]);
                            this._activeTab(0);
                            break;
                        case 32:
                        case 13:
                            e.preventDefault();
                            $(this._headers[index]).hasClass("e-active") && (this.model.multipleOpen || this.model.collapsible) ? this._hideTab(index) : this._activeTab(index);
                            break;
                    }
                }
                else if (e.ctrlKey && !targetEle.hasClass("e-acrdn")) {
                    switch (code) {
                        case 38:
                            e.preventDefault();
                            index = this._contentPanels.index(targetEle.parent(".e-content"));
                            currentEle = $(this._headers[index]);
                            this._activeTab(index);
                            break;
                        case 33:
                            e.preventDefault();
                            currentEle = $(this._headers[0]);
                            this._activeTab(0);
                            break;
                        case 34:
                            e.preventDefault();
                            currentEle = $(this._headers[this.panelCount() - 1]);
                            this._activeTab((this.panelCount() - 1));
                            break;
                    }
                }
                if (!ej.isNullOrUndefined(currentEle)) {
                    currentEle.addClass("e-focus");
                    currentEle.focus();
                }
            }
        },
        /**
         * Wiring the events to accordion control		
		 * @private
         */
        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            //tabs Click evnts
            this._on(this._headers, this.model.events, this._headerEventHandler);
            if (this.model.allowKeyboardNavigation) {
                this._on(this.element, 'keydown', this._keyPress);
            }
        },
        /**
         * Unwiring the events to accordion control		
		 * @private
         */
        //-------------------- Event UnWire-up -------------------------//
        _unWireEvents: function () {
            this._off(this._headers, this.model.events);
            if (this.model.allowKeyboardNavigation) {
                this._off(this.element, 'keydown');
            }
        },
        /*---------------------Public methods---------------------*/
        //client side methods
        /**
        * enable the accordion widget
		* Enables given index header and content panel.
		* @alias enableItems
		* @return jQuery
		**@example
		* //HTML template
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // Call enableItems method.
		* $("#accordion").ejAccordion("enableItems", 1);
		*&lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */
        enableItems: function (itemIndexes) {
            /// <summary>This function enable the given item of the Accordion control</summary>
            /// <param name="itemIndexes" type="int"></param>
            if (!this.model.enabled) return false;
            if ($.isArray(itemIndexes)) {
                for (var i = 0; i < itemIndexes.length; i++) {
                    $(this._headers[itemIndexes[i]]).removeClass("e-disable");
                    $(this._contentPanels[itemIndexes[i]]).removeClass("e-disable");
                    this._on($(this._headers[itemIndexes[i]]), this.model.events, this._headerEventHandler);
                    this._deleteDisabledItems(itemIndexes[i]);
                }
            }
            else {
                $(this._headers[itemIndexes]).removeClass("e-disable");
                $(this._contentPanels[itemIndexes]).removeClass("e-disable");
                this._on($(this._headers[itemIndexes]), this.model.events, this._headerEventHandler);
                this._deleteDisabledItems(itemIndexes);
            }
        },
        /**
        * disables the accordion widget
		* disables given index header and content panel.
		* @alias disableItems
		* @return jQuery
		**@example
		* //HTML template
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // Call disableItems method.
		* $("#accordion").ejAccordion("disableItems", 1);
		*&lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */
        disableItems: function (itemIndexes) {
            /// <summary>This function disable the given item of the Accordion control</summary>
            /// <param name="itemIndexes" type="int"></param>
            if (!this.model.enabled) return false;
            if ($.isArray(itemIndexes)) {
                for (var i = 0; i < itemIndexes.length; i++) {
                    $(this._headers[itemIndexes[i]]).addClass("e-disable");
                    $(this._contentPanels[itemIndexes[i]]).addClass("e-disable");
                    $(this._headers[itemIndexes[i]]).unbind(this.model.events);
                    this.model.disabledItems.push(itemIndexes[i]);
                }
            }
            else {
                $(this._headers[itemIndexes]).addClass("e-disable");
                $(this._contentPanels[itemIndexes]).addClass("e-disable");
                $(this._headers[itemIndexes]).unbind(this.model.events);
                this.model.disabledItems.push(itemIndexes);
            }
        },
        /**
        * enables the accordion widget
		* enables given index header and content panel.Deprecate this methods with enableItems()
		* @alias enableItems
		* @return jQuery
		* @memberof ejAccordion
		* @instance
         */
        //Deprecate this methods with enableItems()
        enable: function (itemIndexes) {
            this.enableItems(itemIndexes);
        },
        /**
        * disables the accordion widget
		* disables given index header and content panel.Deprecate this methods with disableItems()
		* @alias disableItems
		* @return jQuery
		* @memberof ejAccordion
		* @instance
         */
        //Deprecate this methods with disableItems()
        disable: function (itemIndexes) {
            this.disableItems(itemIndexes);
        },
        /**
        * disables the accordion widget
		* disables All the header and content panel.
		* @alias disableAll
		* @return jQuery
		**@example
		* //HTML template
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // Call disableAll method.
		* $("#accordion").ejAccordion("disableAll");
		*&lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */

        disableAll: function () {
            /// <summary>This function disable all item of the Accordion control</summary>
            for (var i = 0; i < this._headers.length; i++) {
                $(this._headers[i]).addClass("e-disable");
                $(this._contentPanels[i]).addClass("e-disable");
                this.model.disabledItems.push(i);
            }
            this._unWireEvents();
        },
        /**
        * enable the accordion widget
		* Enables all the headers and content panels.
		* @alias enableAll
		* @return jQuery
		**@example
		* //HTML template
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // Call enableAll method.
		* $("#accordion").ejAccordion("enableAll");
		*&lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */
        enableAll: function () {
            /// <summary>This function enable all item of the Accordion control</summary>
            for (var i = 0; i < this._headers.length; i++) {
                $(this._headers[i]).removeClass("e-disable");
                $(this._contentPanels[i]).removeClass("e-disable");
            }
            this.model.disabledItems = [];
            this._unWireEvents();
            this._wireEvents();
        },
        /**
        * selects the accordion widget
		* Activates the given index header and panel.
		* @alias selected
		* @return jQuery
		**@example
		* //HTML template
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // Call selected method.
		* $("#accordion").ejAccordion("selected",1);
		*&lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */
        selected: function (index) {
            /// <summary>To Activated give index of the Accordion control</summary>
            /// <param name="index" type="int"></param>
            if (!this.model.enabled) return false;
            if (index != null && this.model.selectedItemIndex != index) {
                this._activeTab(index);
            }
        },
        /**
		* Shows the hidden Accordion control.
		* @alias show
		* @return jQuery
		**@example
		* //HTML template
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // Call show method.
		* $("#accordion").ejAccordion("show");
		*&lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */
        show: function () {
            /// <summary>To Show the Accordion control</summary>
            if (!this.model.enabled) return false;
            this.element.css("visibility", "visible");
        },
        /**
		* Hides the visible Accordion control.
		* @alias hide
		* @return jQuery
		**@example
		* //HTML template
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // Call hide method.
		* $("#accordion").ejAccordion("hide");
		*&lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */
        hide: function () {
            /// <summary>To Hide the Accordion control</summary>
            if (!this.model.enabled) return false;
            this.element.css("visibility", "hidden");
        },
        /**
        * Shows the hidden accordion control.Deprecate this methods with show()
		* @alias showAccordion
		* @return jQuery
		* @memberof ejAccordion
		* @instance
         */
        //Deprecate this methods with show()
        showAccordion: function () {
            this.show();
        },
        /**
        * Hides the accordion control from the page.Deprecate this methods with hide()
		* @alias hideAccordion
		* @return jQuery
		* @memberof ejAccordion
		* @instance
         */
        //Deprecate this methods with hide()
        hideAccordion: function () {
            this.hide();
        },
        /**
		* Returns the total number of panels in the control.
		* @alias panelCount
		* @return jQuery
		**@example
		* //HTML template
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // Call panelCount method.
		* $("#accordion").ejAccordion("panelCount");
		*&lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */
        panelCount: function () {
            /// <summary>This function get the number of panel rendered</summary>
            if (this._headers) {
                return this._headers.length;
            }
        }
    });
    /**
	 * Enum for accordion Content height style	 
	 * @enum {string}
	 * @global 
	 */
    //Content height style enum
    ej.Accordion.heightStyle = {
        /**  Height fit to the content in the panel */
        Content: "content",
        /**  Height set to the largest content in the panel */
        Auto: "auto",
        /**  Height filled to the content of the panel */
        Fill: "fill"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Progressbar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejProgressBar
   * @requires jQuery
   * @requires ej.common.all
   * @classdesc Custom Design for Progressbar control.
   * @example
   *&lt;div id="progress"&gt;&lt;/div&gt;
   *&lt;script&gt;
   *$("#progress").ejProgressBar();
   *&lt;/script&gt;

   */
    // ejProgressbar is the plugin name 
    // "ej.Progressbar" is "namespace.className" will hold functions and properties

    ej.widget("ejProgressBar", "ej.ProgressBar", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        _rootCSS: "e-progressbar",

        // default model
        defaults: {
            /// <summary>This Contains default property of Sycnfusion Tab control </summary>
            /**		
			* Sets the custom text for the ProgressBar. The text placed in the middle of the ProgressBar and it can be customizable using the class 'e-progress-text'. 
			* @default null
			* @type {String}
			* @example 
			* // Initialize the ProgressBar with the text value specified
			* $("#progress").ejProgressBar({ text: 'loading...' });
			* @example 
			* //Get or set the text API, after initialization:
			* //Get the text API value
			* $("#progress").ejProgressBar('option', 'text');
			* //Set the text API
			* $("#progress").ejProgressBar('option', 'text', 'loading...');
			* @memberof ejProgressBar
			* @instance
			*/
            text: "",
            /**		
			* Sets the root class for ProgressBar theme. This cssClass API helps to use custom skinning option for ProgressBar control. By defining the root class using this API, we need to include this root class in CSS.
			* @default null
			* @type {String}
			* @example 
			* //Initialize the ProgressBar with the cssClass value specified
			* $("#progress").ejProgressBar({ cssClass: 'gradient-lime ' });
			* @example 
			* //Get or set the cssClass API, after initialization:
			* //Get the cssClass API value
			* $("#progress").ejProgressBar('option', 'cssClass');
			* //Set the cssClass API
			* $("#progress").ejProgressBar('option', 'cssClass', 'gradient-lime ');
			* @memberof ejProgressBar
			* @instance
			*/
            cssClass: "",
            /**		
			* Sets the minimum value of the ProgressBar.
			* @default 0
			* @type {Number}
			* @example 
			* //Initialize the ProgressBar with the min value specified
			* $("#progress").ejProgressBar({ min: 50 });
			* @example 
			* //Get or set the min API, after initialization:
			*	//Get the min API value
			* $("#progress").ejProgressBar('option', 'min');
			* //Set the min API
			* $("#progress").ejProgressBar('option', 'min', 50 );
			* @memberof ejProgressBar
			* @instance
			*/
            min: 0,
            /**		
			* Sets the maximum value of the ProgressBar.
			* @default 100
			* @type {Number}
			* @example 
			* //Initialize the ProgressBar with the max value specified
			* $("#progress").ejProgressBar({ max: 200 });
			* @example 
			* //Get or set the max API, after initialization:
			*	//Get the max API value
			* $("#progress").ejProgressBar('option', 'max');
			* //Set the max API
			* $("#progress").ejProgressBar('option', 'max', 200 );
			* @memberof ejProgressBar
			* @instance
			*/
            max: 100,
            /**		
			* Sets the ProgressBar value. The value should be in between min and max values.
			* @default 0
			* @type {Number}
			* @example 
			* //Initialize the ProgressBar with the progress value specified
			* $("#progress").ejProgressBar({ value: 70 });
			* @example 
			* //Get or set the value API, after initialization:
			*	//Get the value API value
			* $("#progress").ejProgressBar('option', 'value');
			* //Set the value API
			* $("#progress").ejProgressBar('option', 'value', 70 );
			* @memberof ejProgressBar
			* @instance
			*/
            value: 0,
            /**		
			* Sets the ProgressBar value in percentage. The value should be in between 0 to 100.
			* @default 0
			* @type {Number}
			* @example 
			* //Initialize the ProgressBar with the progress value specified in percent
			* $("#progress").ejProgressBar({ percentage : 35 });
			* @example 
			* //Get or set the percentage  API, after initialization:
			*	//Get the percentage  API value
			* $("#progress").ejProgressBar('option', 'percentage');
			* //Set the percentage  API
			* $("#progress").ejProgressBar('option', 'percentage', 35 );
			* @memberof ejProgressBar
			* @instance
			*/
            percentage: 0,
            /**		
			* Defines the height of the ProgressBar.
			* @default null
			* @type {Number/String}
			* @example 
			* //Initialize the ProgressBar with the height value specified
			* $("#progress").ejProgressBar({ height: 20 });
			* @example 
			* //Get or set the height API, after initialization:
			*	//Get the height API value
			* $("#progress").ejProgressBar('option', 'height');
			*	//Set the height API
			* $("#progress").ejProgressBar('option', 'height', 30 );
			* @memberof ejProgressBar
			* @instance
			*/
            height: null,
            /**		
			* Defines the width of the ProgressBar.
			* @default null
			* @type {Number/String}
			* @example 
			* //Initialize the ProgressBar with the width value specified
			* $("#progress").ejProgressBar({ width: 200 });
			* @example 
			* //Get or set the width API, after initialization:
			*	//Get the width API value
			* $("#progress").ejProgressBar('option', 'width');
			*	//Set the width API
			* $("#progress").ejProgressBar('option', 'width', 300 );
			* @memberof ejProgressBar
			* @instance
			*/
            width: null,
            /**		
			* When this property sets to false, it disables the ProgressBar control
			* @default true
			* @type {boolean}
			* @example 
			* //Initialize the ProgressBar with the enabled value specified
			* $("#progress").ejProgressBar({ enabled: true });
			* @example 
			* //Get or set the enabled API, after initialization:
			*	//Get the enabled API value
			* $("#progress").ejProgressBar('option', 'enabled');
			*	//Set the enabled API
			* $("#progress").ejProgressBar('option', 'enabled', true);
			* @memberof ejProgressBar
			* @instance
			*/
            enabled: true,
            /**		
			* Sets the ProgressBar direction as right to left alignment.
			* @default false
			* @type {boolean}
			* @example 
			* //Initialize the Progress bar with the rtl value specified
			* $("#progress").ejProgressBar({ rtl: true });
			* @example 
			* //Get or set the rtl API, after initialization:
			* //Get the rtl API value
			* $("#progress").ejProgressBar('option', 'rtl');
			* //Set the rtl API
			* $("#progress").ejProgressBar('option', 'rtl', true);
			* @memberof ejProgressBar
			* @instance
			*/
            rtl: false,
            /**		
			* Save current model value to browser cookies for state maintains. While refresh the progressBar control page retains the model value apply from browser cookies
			* @default false
			* @type {boolean}
			* @example 
			* //Initialize the Progress bar with the persist value specified
			* $("#progress").ejProgressBar({ persist: true });
			* @example 
			* //Get or set the persist API, after initialization:
			* //Get the persist API value
			* $("#progress").ejProgressBar('option', 'persist');
			* //Set the persist API
			* $("#progress").ejProgressBar('option', 'persist', true);
			* @memberof ejProgressBar
			* @instance
			*/
            persist: false,
            /**     
			 * Event triggers when the process starts (from 0%)
			 * @event
			 * @name ejProgressBar#start		
			 * @param {Object} argument Event parameters from progressbar   
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the ProgressBar model
			 * @param {object}  argument.percentage returns the current progress percentage
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument returns the current progress value
			 * @example 
			 * //initial start event
			 *$("#progress").ejProgressBar({ 
			 *	focusIn: function(args) { //handle the event } 
			 *});
			 * @memberof ejProgressBar
			 * @instance
			 */
            start: null,
            /**     
			 * Event triggers when the process completes (at 100%)
			 * @event
			 * @name ejProgressBar#complete		
			 * @param {Object} argument Event parameters from progressbar   
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the ProgressBar model
			 * @param {object}  argument.percentage returns the current progress percentage
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument returns the current progress value
			 * @example 
			 * //initial complete event
			 *$("#progress").ejProgressBar({ 
			 *	change: function(args) { //handle the event } 
			 *});
			 * @memberof ejProgressBar
			 * @instance
			 */
            complete: null,
            /**     
			 * Event triggers when the progress value changed
			 * @event
			 * @name ejProgressBar#change		
			 * @param {Object} argument Event parameters from progressbar   
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the ProgressBar model
			 * @param {object}  argument.percentage returns the current progress percentage
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument returns the current progress value
			 * @example 
			 * //initial complete event
			 *$("#progress").ejProgressBar({ 
			 *	focusOut: function(args) { //handle the event } 
			 *});
			 * @memberof ejProgressBar
			 * @instance
			 */
            change: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */

        dataTypes: {
            cssClass: "string",
            min: "number",
            max: "number",
            enabled: "boolean",
            rtl: "boolean"
        },
        /**
        * Sets the Progress bar value. The value should be range between min and max values.
        * @private
        */

        _setValue: function (value) {
            /// <summary>Sets the Progress bar value. The value should be range between min and max values.</summary>
            if (value == null) value = this.model.min;
            else if (typeof value === "string") value = parseFloat(value);

            if (this._isNumber(value)) this.model.value = value;
            else if (!this._isNumber(this.model.value)) this.model.value = this.model.min;

            this.model.value = this._validateRange(this.model.value, this.model.min, this.model.max);
            this._setProgressValue();
        },
        /**
         * Sets the Progress bar value in percentage. The value should be range between 0 to 100.
		 * @private
         */
        _setPercent: function (percent) {
            /// <summary>Sets the Progress bar value in percentage. The value should be range between 0 to 100.</summary>
            this.initial = this.model.percentage;
            if (percent == null) percent = 0;
            else if (typeof percent === "string") percent = parseFloat(percent);

            if (this._isNumber(percent)) this.model.percentage = percent;
            else if (!this._isNumber(this.model.percentage)) this.model.percentage = 0;

            this.model.percentage = this._validateRange(this.model.percentage, 0, 100);
            this.model.value = this._percentToValue(this.model.percentage);
            this._increaseProgressWidth();
        },
        /**
         * Configure to check whether the min and max value are valid
		 * @private
         */
        _validateMinMax: function () {
            if (isNaN(this.model.min)) this.model.min = 0;
            if (isNaN(this.model.max)) this.model.max = 100;
        },
        /**
         * Sets the Progress bar text
		 * @private
         */
        _setText: function (text) {
            /// <summary>Sets the Progress bar value in percentage. The value should be range between 0 to 100.</summary>
            if (text) {
                if (this.text) this.text.html(text);
                else {
                    this.text = ej.buildTag("div.e-progress-txt", text);
                    this.element.append(this.text);
                    this._setTop();
                }
            }
            else if (this.text) {
                this.text.remove();
                this.text = null;
            }
        },
        /**
         * To configure the custom theme for progressbar using cssClass property		
		 * @private
         */
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
		* Enables the progressbar control.
		* @alias enable
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
		*$("#progress").ejProgressBar(“enable”);
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */

        enable: function () {
            /// <summary>Enables the Progress bar control.</summary>
            if (!this.model.enabled) {
                this.element.removeClass("e-disable");
                this.model.enabled = true;
            }
        },
        /**
		* Disables the progressbar control.
		* @alias disable
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
		*$("#progress").ejProgressBar("disable");
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */
        disable: function () {
            /// <summary>Disables the Progress bar control.</summary>
            if (this.model.enabled) {
                this.element.addClass("e-disable");
                this.model.enabled = false;
            }
        },
        /**
		* Returns the current progress value.
		* @alias getValue
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
		*$("#progress").ejProgressBar("getValue");
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */
        getValue: function () {
            /// <summary>Returns the Progress bar value.</summary>
            return this.model.value;
        },
        /**
		* Returns the current progress value in percent.
		* @alias getPercent
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
		*$("#progress").ejProgressBar("getPercent");
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */
        getPercent: function () {
            /// <summary>Returns the Progress bar value in percentage.</summary>
            return this.model.percentage;
        },
        /**
         * Create the Progressbar widget
		 * @private
         */
        // constructor function
        _init: function () {
            this._initialize();
            this._render();
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            /// <summary>Sets the model values to the Progress bar object</summary>
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "value":
                        this._setValue(options[option]);
                        options[option] = this.model.value; break;
                    case "percentage":
                        this._setPercent(options[option]);
                        options[option] = this.model.percentage; break;
                    case "min":
                        if (!isNaN(options[option])) this._minValidation(options[option]);
                        options[option] = this.model.min;
                    case "max":
                        if (!isNaN(options[option])) this._maxValidation(options[option]);
                        options[option] = this.model.max;
                    case "text": this._setText(options[option]); break;
                    case "height": this._setHeight(options[option]); if (this.text) this._setTop(); break;
                    case "width": this._setWidth(options[option]); break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "rtl": this._rtl(options[option]); break;
                }
            }
        },
        /**
       * destroy the Progressbar widget
       * @alias destroy
       * @return jQuery
       *@example
       *&lt;div id="progress"&gt;&lt;/div&gt;
       *&lt;script&gt;
       *$("#progress").ejProgressBar("destroy");
       *&lt;/script&gt;
       * @memberof ejProgressBar
       * @instance
       */
        _destroy: function () {
            /// <summary>This function is  used to destroy the Progress bar Object</summary>
            this.element.empty();
            this.element.removeClass("e-widget " + this.model.cssClass);
        },
        /**
         * Initialization Section For Progress bar control		
		 * @private
         */

        _initialize: function () {
            this.text = null;
            this.header = null;
            this._preVal = null;
        },
        /**
         * Render Section For Progress bar control		
		 * @private
         */
        // renders the Progress bar control
        _render: function () {
            this.element.addClass("e-widget " + this.model.cssClass).attr("role", "ProgressBar");
            this._setDimention();

            this.header = ej.buildTag("div.e-progress");
            this.element.append(this.header);
            this._setText(this.model.text);

            this._setInitialValue();
            this._checkProperties();
        },
        /**
         * Configure dimensions For Progress bar control		
		 * @private
         */
        _setDimention: function () {
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
        },
        /**
         * Configure to set height For Progress bar control		
		 * @private
         */
        _setHeight: function (height) {
            if (height) this.element.height(height);
        },
        /**
         * Configure to set width For Progress bar control		
		 * @private
         */
        _setWidth: function (width) {
            if (width) this.element.width(width);
        },
        /**
         * Configure to set the value or percentage For Progress bar control initially		
		 * @private
         */
        // initially sets the value or percentage to Progress bar
        _setInitialValue: function () {
            this._validateMinMax();
            if (this.model.percentage) this._setPercent(this.model.percentage);
            else this._setValue(this.model.value);
        },
        /**
         * Configure to disable the progressbar control		
		 * @private
         */
        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },
        /**
         * Configure to check the properties the progressbar control		
		 * @private
         */
        _checkProperties: function () {
            if (this.model.rtl) this._rtl(this.model.rtl);
            this._minValidation(this.model.min);
            this._maxValidation(this.model.max);
            if (!this.model.enabled) this._disabled(true);
        },
        /**
         * Configure rtl for the progressbar control		
		 * @private
         */
        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },
        /**
         * Configure to valiate min value of the progressbar control		
		 * @private
         */
        _minValidation: function (min) {
            if (this.model.max && this.model.max < min) this.model.max = min;
            if (this.model.value < min)
                this.model.value = min;
            this.model.min = min;
            this._setProgressValue();
        },
        /**
         * Configure to valiate max value of the progressbar control		
		 * @private
         */
        _maxValidation: function (max) {
            if (this.model.min && this.model.min > max) this.model.min = max;
            if (this.model.value > max)
                this.model.value = max;
            this.model.max = max;
            this._setProgressValue();
        },
        /**
         * Configure position of the progressbar control inner text		
		 * @private
         */

        // for the center alignment of the inner text
        _setTop: function () {
            var top = (this.element.height() - this.text.height()) / 2;
            this.text.css("top", Math.floor(top));
        },
        /**
         * Configure to change value of the progressbar control		
		 * @private
         */
        // for change the progress bar value
        _increaseProgressWidth: function () {
            this.header.css("width", this.model.percentage + "%");

            if (this.initial == 0 && this.model.percentage != this.initial)
                this._raiseEvent("start");

            if (this._preVal != this.model.value) {
                this._preVal = this.model.value;
                $(this.header).attr("aria-label", this.model.percentage);
                this._raiseEvent("change");

                if (this.model.percentage == 100)
                    this._raiseEvent("complete");
            }
        },
        /**
         * Configure to raise events of the progressbar control		
		 * @private
         */
        _raiseEvent: function (event) {
            if (this.model[event])
                this._trigger(event, { value: this.model.value, percentage: this.model.percentage });
        },
        /**
         * Configure to set value of the progressbar control		
		 * @private
         */
        _setProgressValue: function () {
            this.initial = this.model.percentage;
            this.model.percentage = this._valueToPercent(this.model.value);
            this._increaseProgressWidth();
        },
        /**
         * Configure to validate type of value in the progressbar control		
		 * @private
         */

        _isNumber: function (number) {
            return typeof number === "number" && !isNaN(number);
        },
        /**
         * Configure to validate value of the progressbar control between min and max		
		 * @private
         */
        // check whether the value in the min, max range
        _validateRange: function (value, min, max) {
            if (value < min) return min;
            else if (value > max) return max;
            return value;
        },
        /**
         * Configure to convert the value to percentage in the progressbar control		
		 * @private
         */
        // converts the value into percentage
        _valueToPercent: function (value) {
            if (this.model.max <= this.model.min) return 100;
            value = this._validateRange(value, this.model.min, this.model.max);
            value = (100 * (value - this.model.min)) / (this.model.max - this.model.min);
            return value;
        },
        /**
         * Configure to convert the percentage to value in the progressbar control		
		 * @private
         */
        // converts the percentage into value
        _percentToValue: function (percent) {
            if (this.model.max <= this.model.min) { return this.model.min; }

            if (percent >= 0 && percent <= 100) {
                var diff = this.model.max - this.model.min;
                var val = diff * percent / 100;
                percent = val + this.model.min;
            }
            else if (percent < 0) percent = this.model.min;
            else if (percent > 100) percent = this.model.max;
            return percent;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
* @namespace ej
* @class ejRating
* @requires jQuery
* @requires ej.common.all
* @classdesc Custom Design for Html Rating control.
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Create Rating
* $('#rating').ejRating(); 	
* &lt;/script&gt;
*/
    ej.widget("ejRating", "ej.Rating", {
        _rootCSS: "e-rating",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,

        // default model
        defaults: {
            /**		
* Allow to render the maximum number of Rating shape(star).
* @default 5
* @type {number}
* @example 
* //To set maxValue API value during initialization  
* 	$("#rating").ejRating({maxValue: 10 });					* 
* @example 
* //Get or set the maxValue API, after initialization:<br>
*	//Gets the maxValue value  
*	$("#rating").ejRating("option", "maxValue");<br>			
*	//Sets the maxValue value 
*	$("#rating").ejRating("option", "maxValue", 10); 
 * @memberof ejRating
* @instance
*/
            maxValue: 5,
            /**		
* Allow to render the minimum number of Rating shape(star).
* @default 0
* @type {number}
* @example 
* //To set minValue API value during initialization  
* 	$("#rating").ejRating({minValue: 10 });					* 
* @example 
* //Get or set the minValue API, after initialization:<br>
*	//Gets the minValue value  
*	$("#rating").ejRating("option", "minValue");<br>			
*	//Sets the minValue value 
*	$("#rating").ejRating("option", "minValue", 10); 
* @memberof ejRating
* @instance
*/
            minValue: 0,
            /**		
* To specify the number of stars to be selected while rendering.
* @default 1
* @type {number}
* @example 
* //To set currentValue API value during initialization  
* 	$("#rating").ejRating({currentValue: 3 });					* 
* @example 
* //Get or set the currentValue API, after initialization:<br>
*	//Gets the currentValue value  
*	$("#rating").ejRating("option", "currentValue");<br>			
*	//Sets the currentValue value 
*	$("#rating").ejRating("option", "currentValue", 3); 
* @memberof ejRating
* @instance
*/
            currentValue: 1,
            /**		
* Enables the rating control with reset button.It can be used to reset the rating control value.
* @default true
* @type {boolean}
* @example 
* //To set allowReset API value during initialization  
* 	$("#rating").ejRating({allowReset: true});					* 
* @example 
* //Get or set the allowReset API, after initialization:<br>
*	//Gets the allowReset value  
*	$("#rating").ejRating("option", "allowReset");<br>			
*	//Sets the allowReset value 
*	$("#rating").ejRating("option", "allowReset", true); 
* @memberof ejRating
* @instance
*/
            allowReset: true,
            /**		
* To specify the width of each shape in Rating control.
* @default 23
* @type {number}
* @example 
* //To set shapeWidth API value during initialization  
* 	$("#rating").ejRating({shapeWidth: 25 });					* 
* @example 
* //Get or set the shapeWidth API, after initialization:<br>
*	//Gets the shapeWidth value  
*	$("#rating").ejRating("option", "shapeWidth");<br>			
*	//Sets the shapeWidth value 
*	$("#rating").ejRating("option", "shapeWidth", 25); 
* @memberof ejRating
* @instance
*/
            shapeWidth: 23,
            /**		
* To specify the height of each shape in Rating control.
* @default 23
* @type {number}
* @example 
* //To set shapeHeight API value during initialization  
* 	$("#rating").ejRating({shapeHeight: 25 });					* 
* @example 
* //Get or set the shapeHeight API, after initialization:<br>
*	//Gets the shapeHeight value  
*	$("#rating").ejRating("option", "shapeHeight");<br>			
*	//Sets the shapeHeight value 
*	$("#rating").ejRating("option", "shapeHeight", 25); 
* @memberof ejRating
* @instance
*/
            shapeHeight: 23,
            /**		
* Specifies the orientation of Rating control.    See {@link Orientation}
* @default ej.orientation.Horizontal
* @type {enum}
* @example 
* //To set orientation API value during initialization  
* 	$("#rating").ejRating({orientation: ej.orientation.Horizontal });					* 
* @example 
* //Get or set the orientation API, after initialization:<br>
*	//Gets the orientation value  
*	$("#rating").ejRating("option", "orientation");<br>			
*	//Sets the orientation value 
*	$("#rating").ejRating("option", "orientation", ej.orientation.Horizontal); 
* @memberof ejRating
* @instance
*/
            orientation: "horizontal",
            /**		
* Specifies the value to be increased while navigating between shapes(stars) in Rating control.
* @default 1
* @type {number}
* @example 
* //To set incrementStep API value during initialization  
* 	$("#rating").ejRating({incrementStep: 2 });					* 
* @example 
* //Get or set the incrementStep API, after initialization:<br>
*	//Gets the incrementStep value  
*	$("#rating").ejRating("option", "incrementStep");<br>			
*	//Sets the incrementStep value 
*	$("#rating").ejRating("option", "incrementStep", 2); 
* @memberof ejRating
* @instance
*/
            incrementStep: 1,
            /**		
* Interaction with Rating control can be prevented by enabling this API.
* @default false
* @type {boolean}
* @example 
* //To set readOnly API value during initialization  
* 	$("#rating").ejRating({readOnly: true});					* 
* @example 
* //Get or set the readOnly API, after initialization:<br>
*	//Gets the readOnly value  
*	$("#rating").ejRating("option", "readOnly");<br>			
*	//Sets the readOnly value 
*	$("#rating").ejRating("option", "readOnly", true); 
* @memberof ejRating
* @instance
*/
            readOnly: false,
            /**		
* When this property is set to false, it disables the rating control.
* @default true
* @type {boolean}
* @example 
* //To set enabled API value during initialization  
* 	$("#rating").ejRating({enabled: true });					* 
* @example 
* //Get or set the enabled API, after initialization:<br>
*	//Gets the enabled value  
*	$("#rating").ejRating({enabled: true });<br>			
*	//Sets the enabled value 
*	$("#rating").ejRating("option", "enabled", true); 
* @memberof ejRating
* @instance
*/
            enabled: true,
            /**		
* Enables the tooltip option.Currently selected value value will be displayed in tooltip.
* @default true
* @type {boolean}
* @example 
* //To set showTooltip API value during initialization  
* 	$("#rating").ejRating({showTooltip: true});					* 
* @example 
* //Get or set the showTooltip API, after initialization:<br>
*	//Gets the showTooltip value  
*	$("#rating").ejRating("option", "showTooltip");<br>			
*	//Sets the showTooltip value 
*	$("#rating").ejRating("option", "showTooltip", true);
* @memberof ejRating
* @instance
*/
            showTooltip: true,
            /**		
* Helps to provide more precise ratings.Rating control supports three precision modes - full, half, and exact. See {@link Precision}
* @default full
* @type {enum}
* @example 
* //To set precision API value during initialization  
* 	$("#rating").ejRating({precision: ej.Rating.Precision.Half});					* 
* @example 
* //Get or set the precision API, after initialization:<br>
*	//Gets the precision value  
*	$("#rating").ejRating("option", "precision");<br>			
*	//Sets the precision value 
*	$("#rating").ejRating("option", "precision", ej.Rating.Precision.Half);
* @memberof ejRating
* @instance
*/
            precision: "full",
            /**		
* Specify the CSS class to rating to achieve custom theme.
* @default ""
* @type {string}
* @example 
* // Set the CSSclass during initialization. 			
* 	$("#rating").ejRating({  cssClass : "gradient-lime" });			* 
* @example 
* //Get or set the CSSclass after initialization:<br>
*	// Gets the CSSclass value.		
*	$("#rating").ejRating("option", "cssClass");			
*	// Sets the CSSclass to rating
*	$("#rating").ejRating("option", "cssClass",  "gradient-lime" );
 * @memberof ejRating
* @instance
*/
            cssClass: "",
            /**		
* Specifies the width of the Rating control wrapper.
* @default null
* @type {string}
* @example 
* //To set width API value during initialization  
* 	$("#rating").ejRating({width: "200" });					* 
* @example 
* //Get or set the width API, after initialization:<br>
*	//Gets the width value  
*	$("#rating").ejRating("option", "width");<br>			
*	//Sets the width value 
*	$("#rating").ejRating("option", "width", "200"); 
* @memberof ejRating
* @instance
*/
            width: null,
            /**		
* Specifies the height of the Rating control wrapper.
* @default null
* @type {string}
* @example 
* //To set height API value during initialization  
* 	$("#rating").ejRating({height: "50" });					* 
* @example 
* //Get or set the height API, after initialization:<br>
*	//Gets the height value  
*	$("#rating").ejRating("option", "height");<br>			
*	//Sets the height value 
*	$("#rating").ejRating("option", "height", "50"); 
* @memberof ejRating
* @instance
*/
            height: null,
            /**		
* Save current model value to browser cookies for state maintanence. While refresh the page Rating control values are retained.  
* @default false
* @type {boolean}
* @example 
* //To set persist API value during initialization  
* 	$("#rating").ejRating({persist: true });					* 
* @example 
* //Get or set the persist API, after initialization:<br>
*	//Gets the persist value  
*	$("#rating").ejRating("option", "persist");<br>			
*	//Sets the persist value 
*	$("#rating").ejRating("option", "persist", true); 
* @memberof ejRating
* @instance
*/
            persist: false,
            /**     
 * Fires when Rating control is created.
 * @event
 * @name ejRating#create	
 * @param {Object} argument Event parameters from rating     
 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
 * @param {object}  argument.model returns the rating model
 * @param {string}  argument.type returns the name of the event.
 * @example 
 * //create event for rating
 * $("#rating").ejRating({
 *    create: function (args) {}
 * });      
 * @memberof ejRating
 * @instance
 */
            create: null,
            /**     
			 * Fires when Rating control is clicked successfully.
			 * @event
			 * @name ejRating#click 
             * @param {Object} argument Event parameters from rating  
			 * @param {string} argument.Value returns the current value.     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the rating model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object}  argument.event returns the mouse click event args values.
			 * @example 
			 * //click event for button
             * $("#rating").ejRating({
             *    click: function (args) {}
             * });      
			 * @memberof ejRating
			 * @instance
			 */
            click: null,
            /**     
 * Fires when mouse hovered over the Rating control
 * @event
 * @name ejRating#mouseOver 
 * @param {Object} argument Event parameters from rating  
 * @param {string} argument.Value returns the current value.     
 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
 * @param {object}  argument.model returns the rating model
 * @param {string}  argument.type returns the name of the event
 * @param {object}  argument.event returns the mouse click event args values.
 * @param {object}  argument.index returns the current index value.
 * @example 
 * //mouseOver event for button
 * $("#rating").ejRating({
 *    mouseOver: function (args) {}
 * });      
 * @memberof ejRating
 * @instance
 */
            mouseOver: null,
            /**     
* Fires when mouse hover is removed from Rating control
* @event
* @name ejRating#mouseOut 
* @param {Object} argument Event parameters from rating  
* @param {string} argument.Value returns the current value.     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the rating model
* @param {string}  argument.type returns the name of the event
* @param {object}  argument.event returns the mouse click event args values.
* @example 
* //mouseOver event for button
* $("#rating").ejRating({
*    mouseOut: function (args) {}
* });      
* @memberof ejRating
* @instance
*/
            mouseOut: null,
            /**     
* Fires when Rating value changes.
* @event
* @name ejRating#valueChanged 
* @param {Object} argument Event parameters from rating  
* @param {string} argument.Value returns the current value.     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the rating model
* @param {string}  argument.type returns the name of the event
* @param {object}  argument.event returns the mouse click event args values.
* @example 
* //mouseOver event for button
* $("#rating").ejRating({
*    valueChanged: function (args) {}
* });      
* @memberof ejRating
* @instance
*/
            valueChanged: null,
            /**     
 * Fires when Rating control is destroyed successfully.
 * @event
 * @name ejRating#destroy 
 * @param {Object} argument Event parameters from rating      
 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
 * @param {object}  argument.model returns the rating model
 * @param {string}  argument.type returns the name of the event
 * @example 
 * //destroy event for rating
 * $("#rating").ejRating({
 *    destroy: function (args) {}
 * });      
 * @memberof ejRating
 * @instance
 */
            destroy: null
        },
        /**
* Specify the data types for default properties 
* @private
*/
        dataTypes: {
            maxValue: "number",
            minValue: "number",
            allowReset: "boolean",
            shapeWidth: "number",
            shapeHeight: "number",
            orientation: "enum",
            incrementStep: "number",
            readOnly: "boolean",
            precision: "enum",
            enabled: "boolean"
        },

        observables: ["currentValue"],
        currentValue: ej.util.valueFunction("currentValue"),
        /**
* To configure the properties at runtime using SetModel		
* @private
*/
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "allowReset": {
                        if (options[key]) {
                            this.showResetButton();
                        }
                        else {
                            this.hideResetButton();
                        }
                        break;
                    }
                    case "currentValue": {
                        this.setCurrentValue(ej.util.getVal(options[key]));
                        break;
                    }
                    case "enabled": this._enabledAction(options[key]); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "height": this._mainDiv.height(options[key]); break;
                    case "width": this._mainDiv.width(options[key]); break;
                    case "readOnly": {
                        this._unWireEvents();
                        break;
                    }
                    case "orientation":
                        {
                            this.model.orientation = options[key];
                            this.refresh();
                            break;
                        }
                    case "maxValue":
                        {
                            this.model.maxValue = options[key];
                            this.refresh();
                            break;
                        }
                    case "minValue":
                        {
                            this.model.minValue = options[key];
                            this.refresh();
                            break;
                        }
                    case "incrementStep":
                        {
                            this.model.incrementStep = options[key];
                            this.refresh();
                            break;
                        }
                    case "shapeWidth":
                        {
                            this.model.shapeWidth = options[key];
                            this.refresh();
                            break;
                        }
                    case "shapeHeight":
                        {
                            this.model.shapeHeight = options[key];
                            this.refresh();
                            break;
                        }
                }
            }
        },
        /**
* destroy the Rating widget
* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
* @alias destroy
* @return jQuery
* @example 
* &lt;rating id="rating"&gt;Rating&lt;/rating&gt; <br> 
* &lt;script&gt;
* // Create Rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.destroy(); // destroy the button
* &lt;/script&gt;
* @example 
* &lt;rating id="rating"&gt;Rating&lt;/rating&gt; <br> 
* &lt;script&gt;
* // destroy the rating
* $("#rating").ejRating("destroy");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/

        _destroy: function () {
            this.element.show();
            this._unWireEvents();
            this._mainDiv.remove();
        },

        // constructor function
        /**
 * Create the butto widget
 * @private
 */
        _init: function () {
            this._initialize();
        },
        /**
* To initialize the button		
* @private
*/
        _initialize: function () {
            this.element.hide();
            this._mainDiv = ej.buildTag("div.e-rating e-widget " + this.model.cssClass, "", {}, { tabindex: "0", role: "group", "aria-label": "Rating" });
            if (!ej.isNullOrUndefined(this.model.width))
                this._mainDiv.width(this.model.width);
            if (!ej.isNullOrUndefined(this.model.height))
                this._mainDiv.height(this.model.height);
            this._mainDiv.insertBefore(this.element);
            if (this.model.orientation == ej.orientation.Horizontal) {
                this._mainDiv.addClass("e-horizontal");
            }
            else
                this._mainDiv.addClass("e-vertical");

            if (this.model.allowReset && !this.model.readOnly) {
                this._createReset();
            }
            this._validation();
            this._renderShape();
            this._shapes = this._mainDiv.find("div.e-shape");
            this._wireEvents();
            this._CurrentIndex = 0;
            this._initCurrentValue();
            this._enabledAction(this.model.enabled);
        },
        /**
        * To change the skin at runtime		
        * @private
        */
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this._mainDiv.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
* To enable or disable the Rating widget	
* @private
*/
        _enabledAction: function (flag) {
            if (flag) {
                this._mainDiv.removeClass("e-disable");
            }
            else {
                this._mainDiv.addClass("e-disable");
            }
        },
        /**
* To validate the user provided incrementStep value	
* @private
*/
        _validation: function () {
            if (this.model.incrementStep < 1) {
                this.model.incrementStep = 1;
            }
            else if (this.model.incrementStep > this.model.maxValue) {
                this.model.incrementStep = this.model.maxValue;
            }
        },
        /**
* Creates reset buttton option in the Rating widget
* @private
*/
        _createReset: function () {
            if (this._mainDiv.find("div.e-reset").length <= 0)
                ej.buildTag("div.e-reset", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight, title: "Reset" }, { role: "button", "aria-label": "reset", "aria-hidden": false }).appendTo(this._mainDiv);
            else
                this._mainDiv.find("div.e-reset").show();
        },
        /**
* To create desired shape to be displayed in the rating control
* @private
*/
        _renderShape: function () {
            var _shapeCount = Math.round(this.model.maxValue / this.model.incrementStep);
            var _startCount = Math.round(this.model.minValue / this.model.incrementStep);
            var containerWith, containerHeight;
            if (this.model.orientation == ej.orientation.Horizontal) {
                containerWith = (_shapeCount + 1) * this.model.shapeWidth;
                containerHeight = this.model.shapeHeight;
            } else if (this.model.orientation == ej.orientation.Vertical) {
                containerWith = this.model.shapeWidth;
                containerHeight = this.model.shapeHeight * _shapeCount;
            }
            if (_shapeCount > 0) {
                var _ulTag = ej.buildTag("ul.e-ul", "", { width: containerWith + "px", height: containerHeight }, {});
                for (var i = _startCount ; i < _shapeCount; i++) {
                    var _liTag = ej.buildTag("li.e-shape-list", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight }, { tabindex: -1, "aria-describedby": this.element.prop("id") + "_tooltip" });
                    ej.buildTag("div.e-shape inactive", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight }).appendTo(_liTag);
                    _liTag.appendTo(_ulTag);
                }
                _ulTag.appendTo(this._mainDiv);
            }
        },
        /**
* To load the current value when the Rating control is rendered
* @private
*/
        _initCurrentValue: function () {
            if (ej.isNullOrUndefined(this.currentValue()) || this.currentValue() == "")
                this._CurrentIndex = 0;
            else if (this.currentValue() > this.model.minValue) {
                var count = (this.currentValue() / this.model.incrementStep) - (this.model.minValue / this.model.incrementStep);
                this._CurrentIndex = count;
                this._valueRefresh(this._CurrentIndex);
                for (var i = 0; i < count; i++) {
                    if (this._shapes[i]) {
                        $(this._shapes[i]).removeClass('inactive').removeClass('active').addClass('selected');
                    }
                    else {
                        this._CurrentIndex = i;
                        break;
                    }
                }
            }
            this.currentValue(this._CurrentIndex);
        },
        /**
* To refresh the rating on every mouse out with previously selected values
* @private
*/
        _valueRefresh: function (index) {
            index = Math.round(index);
            var value = (this.currentValue() / this.model.incrementStep).toFixed(1);
            if (this.model.orientation == ej.orientation.Horizontal) {
                value = ((value % 1).toFixed(1) != 0 ? (value % 1).toFixed(1) : (value / this._CurrentIndex)) * this.model.shapeWidth;
                for (var i = 0; i < index; i++) {
                    $(this._shapes[i]).css({ width: this.model.shapeWidth + "px" });
                }
                if (this._shapes != null)
                    $(this._shapes[index - 1]).css({ width: value + "px" });
            }
            else {
                value = ((value % 1).toFixed(1) != 0 ? (value % 1).toFixed(1) : (value / this._CurrentIndex)) * this.model.shapeHeight;
                for (var i = 0; i < index; i++) {
                    $(this._shapes[i]).css({ height: this.model.shapeHeight + "px" });
                }
                if (this._shapes != null) $(this._shapes[index - 1]).css({ height: value + "px" });
            }
        },

        /**
* To refresh the rating control values.
* @private
*/
        _reset: function () {
            if (!this.model.enabled) return false;
            $(this._shapes).removeClass('active').removeClass('selected').addClass('inactive');
            this.currentValue(0);
            this._CurrentIndex = 0;
            //Update Hidden field
            this.element.value = this._CurrValue;

            ////Fire Value Change event
            this._valueChanged();
        },
        /**
* To fill exact precision on mouse move
* @private
*/
        _fillExactPrecision: function (element, position) {
            var index = this._shapes.index(element) + (this.model.minValue / this.model.incrementStep) + 1;
            var option = this.model;
            if (option.orientation == ej.orientation.Horizontal) {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ width: option.shapeWidth + "px" });
                    else
                        $(this._shapes[i]).css({ width: "0px" });
                }
                //Set width and title
                $(element).css({ width: position + "px" });
                this.toolTipValue = ((index - 1) * option.incrementStep + (position / option.shapeWidth) * option.incrementStep).toFixed(1);
            }
            else {
                for (var i = 0; i < index; i++) {
                    $(this._shapes[i]).css({ height: option.shapeHeight + "px" });
                }
                for (var i = index; i < this._shapes.length; i++) {
                    $(this._shapes[i]).css({ height: "0px" });
                }
                //Set width and title
                $(element).css({ height: position + "px" });
                this.toolTipValue = ((index - 1) * option.incrementStep + (position / option.shapeHeight) * option.incrementStep).toFixed(1);
            }
        },

        /**
* To fill half precision on mouse move
* @private
*/
        _fillHalfPrecision: function (element, position) {
            var index = this._shapes.index(element) + (this.model.minValue / this.model.incrementStep) + 1;
            var options = this.model;
            if (options.orientation == ej.orientation.Horizontal) {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ width: options.shapeWidth + "px" });
                    else
                        $(this._shapes[i]).css({ width: "0px" });
                }
                if ((position + 1) % 2 == 0) {
                    position = options.shapeWidth / 2;
                    $(element).css({ width: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeWidth) * options.incrementStep).toFixed(1);
                }
                else {
                    position = options.shapeWidth;
                    $(element).css({ width: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeWidth) * options.incrementStep).toFixed(1);
                }
            }
            else {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ height: this._starHeight + "px" });
                    else
                        $(this._shapes[i]).css({ height: "0px" });
                }
                //Set width and title
                if (position <= options.shapeHeight / 2) {
                    position = options.shapeHeight / 2;
                    $(element).css({ height: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeHeight) * options.incrementStep).toFixed(1);
                }
                else {
                    position = options.shapeHeight;
                    $(element).css({ height: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeHeight) * options.incrementStep).toFixed(1);
                }
            }
        },
        /**
         * Section For handle clcik action in rating control.
		 * @private
         */
        _ClickHandler: function (e) {
            //Check if stars and rate it
            if (this.model.enabled) {
                var element;
                if (e.target.tagName == "LI")
                    element = e.target.firstChild;
                else if (e.target.parentNode.tagName == "LI")

                    element = e.target;
                if (element) {
                    var index = this._shapes.index(element) + 1;
                    for (var i = 0; i < index; i++) {
                        if (this._shapes[i]) {
                            $(this._shapes[i]).removeClass('inactive').removeClass('active').addClass('selected');
                        }
                    }
                    this._CurrentIndex = index;
                    index = index + (this.model.minValue / this.model.incrementStep);
                    var _IncrementStep = this.model.incrementStep;
                    var _CurrValue;
                    if (this.model.orientation == ej.orientation.Horizontal)
                        _CurrValue = ((index - 1) * _IncrementStep + (element.clientWidth / this.model.shapeWidth) * _IncrementStep).toFixed(1);
                    else
                        _CurrValue = ((index - 1) * _IncrementStep + (element.clientHeight / this.model.shapeHeight) * _IncrementStep).toFixed(1);

                    //Update Hidden field        
                    this.element.value = _CurrValue;
                    this.currentValue(_CurrValue);

                    //Fire Click event
                    //Client side Click event event handler
                    if (this.model.click)
                        this._trigger("click", { event: e, Value: this.currentValue() });
                    ////Fire Value Change event
                    this._valueChanged();
                }

                //Check if reset button and reset
                if ($(e.target).hasClass('e-reset')) {
                    this._reset();
                }
            }
        },
        /**
 * Section For handle mouse over action in rating control.
 * @private
 */
        _MouseOverHandler: function (e) {
            if (this.model.enabled) {
                var element, target = e.target;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    target = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                if (target.tagName == "LI")
                    element = target.firstChild;
                else if (target.parentNode.tagName == "LI")
                    element = target;
                else
                    return false;

                var index = this._shapes.index(element) + 1;
                for (var i = 0; i <= this._shapes.length; i++) {
                    if (this._shapes[i]) {
                        if (i < index)
                            $(this._shapes[i]).removeClass('inactive').removeClass('selected').addClass('active');
                        else
                            $(this._shapes[i]).removeClass('active').removeClass('selected').addClass('inactive');
                    }
                }
                //Client side onMouseOut event handler
                if (this.model.mouseOver)
                    this._trigger("mouseOver", { event: e, index: index, Value: this.currentValue() });
            }
        },
        /**
 * Section For handle mouse out action in rating control.
 * @private
 */
        _MouseOutHandler: function (e) {
            if (!this.model.readOnly && this.model.enabled) {
                var index = this._CurrentIndex;
                if (index != 0) {
                    for (var i = 0; i < this._shapes.length; i++) {
                        if (this._shapes[i]) {
                            if (i < index)
                                $(this._shapes[i]).removeClass('active').removeClass('inactive').addClass('selected');
                            else
                                $(this._shapes[i]).removeClass('active').addClass('inactive');
                        }
                    }
                } else
                    $(this._shapes).removeClass('active').addClass('inactive');
                //On mouse out, keep selected the stars w.r.t current Value
                if (this.model.precision !== ej.Rating.Precision.Full)
                    this._valueRefresh(index);
                //Client side onMouseOut event handler
                if (this.model.mouseOut)
                    this._trigger("mouseOut", { event: e, Value: this.currentValue() });
            }
            this._hideTooltip(1000);
        },
        /**
 * Section For handle mouse move action in rating control.
 * @private
 */
        _MouseMoveHandler: function (e) {
            if (!this.model.readOnly && this.model.enabled) {
                var element, position, target = e.target;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    target = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                if (target.tagName == "LI")
                    element = target.firstChild;
                else if (target.parentNode.tagName == "LI")
                    element = target;
                else
                    return false;
                if (this.model.orientation == ej.orientation.Horizontal)
                    if (ej.isNullOrUndefined(e.offsetX))
                        position = e.pageX - $(target).offset().left;
                    else
                        position = e.offsetX + 1;
                else {
                    if (ej.isNullOrUndefined(e.offsetY))
                        position = e.pageY - $(target).offset().top;
                    else
                        position = e.offsetY + 1;
                }
                this.toolTipValue = (this.model.minValue + (this._shapes.index(element) + 1)) * this.model.incrementStep;
                if (this.model.precision == ej.Rating.Precision.Exact)
                    this._fillExactPrecision(element, position);
                else if (this.model.precision == ej.Rating.Precision.Half)
                    this._fillHalfPrecision(element, Math.round(position));
            } else {
                this.toolTipValue = (this.currentValue() == null || this.currentValue() == "" ? 0 : this.currentValue()) + ' / ' + this.model.maxValue;
            }
            $(e.target).attr("aria-label", this.toolTipValue);
            this._showhideTooltip(e.target);
        },
        /**
* To enable tooltip option
* @private
*/
        _showTooltip: function (element) {
            if (this.model.showTooltip) {
                $('.e-tooltip').remove();
                this.tooltip = ej.buildTag("div#" + this.element.prop("id") + "_tooltip.e-tooltip " + this.model.cssClass + " e-corner-all", "", {}, { role: "tooltip" });
                $("body").append(this.tooltip);
                this._isTooltip = true;
                this._setTooltipPosition(element);
            }
        },
        /**
* To hide the tooltip.
* @private
*/
        _hideTooltip: function (animationSpeed) {
            if (this.model.showTooltip) {
                $(this.tooltip).fadeOut(animationSpeed, function () {
                    $('.e-tooltip').remove();
                });
                this._isTooltip = false;
            }
        },
        /**
* Validates tooltip option is enabled or not
* @private
*/
        _showhideTooltip: function (element) {
            if (this.model.showTooltip) {
                this._showTooltip(element);
            }
        },
        /**
* To set the tooltip position
* @private
*/
        _setTooltipPosition: function (element) {
            if (this.model.showTooltip) {
                this._updateTooltipValue();
                var top, left, remainLeft, remainTop, handle, gap = 5; // gap -> distance between tooltip and slider
                handle = $(element);

                if (this.model.orientation == "vertical") {
                    remainTop = (this.tooltip.outerHeight() - handle.outerHeight()) / 2;
                    remainLeft = handle.outerWidth() + gap;
                    top = handle.offset().top - remainTop;
                    left = handle.offset().left + remainLeft;
                }
                else {
                    remainLeft = (this.tooltip.outerWidth() - handle.outerWidth()) / 2;
                    remainTop = this.tooltip.outerHeight() + gap;
                    top = handle.offset().top - remainTop;
                    left = handle.offset().left - remainLeft;
                }
                this.tooltip.css({ "top": top, "left": left });
            }
        },
        /**
* To update the tooltip value based on current value.
* @private
*/
        _updateTooltipValue: function () {
            //var val = this._getCurrentHandleValue();
            //if (this.model.range != "true")
            this.tooltip[0].innerHTML = this.toolTipValue;
            //else
            //    this.tooltip[0].innerHTML = val[0] + " - " + val[1];
        },

        //-------------------- Event Wire-up -------------------------//
        /**
* Wiring the events to Rating control		
* @private
*/
        _wireEvents: function () {
            //tabs Click evnts
            if (!this.model.readOnly) {
                this._on(this._mainDiv, "mousedown", this._ClickHandler);
                this._on(this._mainDiv.find("li"), "mouseenter touchmove", this._MouseOverHandler);
            }
            this._on(this._mainDiv.find("li"), "mouseleave touchend", this._MouseOutHandler);
            //if (this.model.precision !== ej.Rating.Precision.Full) {
            this._on(this._mainDiv.find("li"), ej.eventType.mouseMove, this._MouseMoveHandler);
            //}
        },
        //-------------------- Event UnWire-up -------------------------//
        /**
* To UnWire the binded events from Rating control		
* @private
*/
        _unWireEvents: function () {
            this._mainDiv.find("li").unbind("mouseenter touchmove");
            this._mainDiv.find("li").unbind("mouseleave touchend");
            this._mainDiv.unbind("mousedown");
            if (this.model.precision !== ej.Rating.Precision.Full) {
                this._mainDiv.find("li").unbind(ej.eventType.mouseMove);
            }
        },
        /**
* Section to specify value changed		
* @private
*/
        _valueChanged: function () {
            //Client side onMouseOut event handler
            if (this.model.valueChanged)
                this._trigger("valueChanged", { Value: this.currentValue() });
        },

        // ------------------Client side methods-------------------------//
        /**
* To reset the rating value		
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Reset Rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.reset(); // reset the rating values
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // reset the rating values
* $("#rating").ejRating("reset");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
 */
        reset: function () {
            /// <summary>To Reset the Rating control</summary>
            this._reset();
        },
        /**
* To show the rating control		
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Show Rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.show(); // shows the rating control
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // show the rating control
* $("#rating").ejRating("show");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        show: function () {
            /// <summary>To Show the Rating control</summary>
            this._mainDiv.css("visibility", "visible");
        },
        /**
* To hide the rating control		
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Hide Rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.hide(); // hides the rating control
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // hide the rating control
* $("#rating").ejRating("hide");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        hide: function () {
            /// <summary>To Hide the Rating control</summary>
            this._mainDiv.css("visibility", "hidden");
        },
        //Deprecate this methods with show()
        showRating: function () {
            /// <summary>To Show the Rating control</summary>
            if (!this.model.enabled) return false;
            this.show();
        },
        hideRating: function () {
            /// <summary>To Hide the Rating control</summary>
            if (!this.model.enabled) return false;
            this.hide();
        },
        /**
* To show the reset button in rating control		
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Show reset button
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.showResetButton(); // shows the reset button control
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // show the reset button
* $("#rating").ejRating("showResetButton");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        showResetButton: function () {
            /// <summary>To Show the Reset Button</summary>
            if (!this.model.enabled) return false;
            this._createReset();
        },
        /**
* To hide the reset button in rating control		
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Hide reset button
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.hideResetButton(); // hides the reset button control
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // hide the reset button
* $("#rating").ejRating("hideResetButton");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        hideResetButton: function () {
            /// <summary>To Hide the Reset Button</summary>
            if (!this.model.enabled) return false;
            this._mainDiv.find("div.e-reset").hide()
        },
        /**
* To get the current value of rating control		
* @return Current Rating value
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Get current value of rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.getCurrentValue(); // get the current value of rting
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // To get the current value of rating
* $("#rating").ejRating("getCurrentValue");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        getCurrentValue: function () {
            /// <summary>To get current value of Rating control</summary>
            return this.currentValue() == null || this.currentValue() == "" ? "" : this.currentValue();
        },
        /**
* To set the current value of rating control		
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Set current value of rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.setCurrentValue(); // set the current value of rating
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // To set the current value of rating
* $("#rating").ejRating("setCurrentValue");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        setCurrentValue: function (value) {
            /// <summary>To set current value of Rating control</summary>
            if (!this.model.enabled) return false;
            if (value != null) {
                this.currentValue(value);
                $(this._shapes).removeClass('selected').addClass('inactive');
                this._initCurrentValue();
            }
        },
        /**
* user can refresh the rating control to identify changes		
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Refresh the rating control
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.refresh(); // refreshes the rating control
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // To refresh the rating values
* $("#rating").ejRating("refresh");	
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        refresh: function () {
            this._destroy();
            this._unWireEvents();
            this._initialize();
        }
    });
    /**
 * Enum for various rating precisions	 
 * @enum {string}
 * @global 
 */
    ej.Rating.Precision = {
        /**  Allows user select full shape of Rating. */
        Full: "full",
        /**  Allows user select half shape of Rating. */
        Half: "half",
        /**  Allows user select exact shape of Rating. */
        Exact: "exact"
    }
    /**
* Enum for various rating orientations	 
* @enum {string}
* @global 
*/
    ej.Rating.Orientation = {
        /**  Renders Rating control in horizontal direction. */
        Horizontal: "Horizontal",
        /**  Renders Rating control in vertical direction. */
        Vertical: "vertical"
    }
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Rotator control.
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejRotator
   * @requires jQuery
   * @requires ej.web.all
   * @classdesc Custom Design for Rotator control.
   *@example
   * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
   * //Initialize the Rotator control in the script as follows.
   *$(function (){
   * // document ready
   * // simple Rotator creation
   *$("#sliderContent").ejRotator();
   *});
   */
    ej.widget("ejRotator", "ej.Rotator", {
        element: null,
        model: null,
        _setFirst: false,

        defaults: {
            /**		
			* Specify the CSS class to Rotator to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the CSS class during initialization. 			
			* 	$("#sliderContent").ejRotator({  cssClass : "gradient-lime" });			
			* @example 
			* //Get or set the cssClass after initialization:<br>
			*	//Get the cssClass API value.		
			*	$("#sliderContent").ejRotator("option", "cssClass");			
			*	//Set the cssClass API value
			*	$("#sliderContent").ejRotator("option", "cssClass",  "gradient-lime" );
			 * @memberof ejRotator
			* @instance
			*/
            cssClass: "",
            /**		
			* Specify the list of data which contains a set of data fields. Each data value is used to render an item for the Rotator. 
			* @default null
			* @type {object}
			* @example 
			* var items = [
            *{ text:"Nature", url:".../images/rotator/nature.jpg"},
			*{ text:"Bird", url:".../images/rotator/bird.jpg"},
			*{ text:"Sculpture ", url:".../images/rotator/ sculpture.jpg"}
			*];
			* // Set the dataSource during initialization. 			
			* 	$("#sliderContent").ejRotator({  dataSource :items });			
			* @example 
			* //Get or set the dataSource after initialization:<br>
			* var items = [
            *{ text:"Nature", url:".../images/rotator/nature.jpg"},
			*{ text:"Bird", url:".../images/rotator/bird.jpg"},
			*{ text:"Sculpture ", url:".../images/rotator/ sculpture.jpg"}
			*];
			*	//Get the dataSource API value.		
			*	$("#sliderContent").ejRotator("option", "dataSource");			
			*	//Set the dataSource API value
			*	$("#sliderContent").ejRotator("option", "dataSource",items);
			 * @memberof ejRotator
			* @instance
			*/
            dataSource: null,
            /**		
			* Retrieves data from remote data. This property is applicable only when a remote data source is used.
			* @default null
			* @type {string}
			* @example 
			* Initialize the Rotator with the query value specified		
            * *var dataManger = ej.DataManager({
			*       url: &lt;http://mvc.syncfusion.com/Services/Northwnd.svc/&gt;
			*});
			*var queryString = ej.Query().from("Suppliers").select("SupplierName", "SupplierUrl");
			*$("#sliderContent").ejRotator(
			*{  dataSource: dataManger, query: queryString,
			*   fields: { text: " SupplierName ", url: " SupplierUrl " }
			*});
			* @example 
			* //Get or set the query after initialization:<br>
			*	//Get the query API value.		
			*	$("#sliderContent").ejRotator('option', 'query');			
			*	//Set the query API value
			*	$("#sliderContent").ejRotator('option', 'query', queryString);
			 * @memberof ejRotator
			* @instance
			*/
            query: null,
            /**		
			* Defines mapping fields for the data items of the Rotator.
			* @default null
			* @type {object}
			* @example 
			* var items = [
            *{ text:"Nature", url:".../images/rotator/nature.jpg"},
			*{ text:"Bird", url:".../images/rotator/bird.jpg"},
			*{ text:"Sculpture ", url:".../images/rotator/ sculpture.jpg"}
			*];
			* // Set the fields during initialization. 			
			* 	$("#sliderContent").ejRotator({  dataSource :items, fields: {text:"text",url:"url"}});
			* @example 
			* //Get or set the fields after initialization:<br>
			* var items = [
            *{ text:"Nature", url:".../images/rotator/nature.jpg"},
			*{ text:"Bird", url:".../images/rotator/bird.jpg"},
			*{ text:"Sculpture ", url:".../images/rotator/ sculpture.jpg"}
			*];
			*	//Get the fields API value.		
			*	$("#sliderContent").ejRotator("option", "fields");			
			*	//Set the fields API value
			*	$("#sliderContent").ejRotator("option", "fields",{text:"text",url:"url"});
			 * @memberof ejRotator
			* @instance
			*/
            fields: { text: "text", url: "url" },
            /**		
           * Enables or disables the Rotator control.
           * @default true
           * @type {boolean} 
           * @example 
           * // Set the enabled during initialization. 			
           * 	$("#sliderContent").ejRotator({  enabled : true});		 
           * @example 
           * //Get or set the enabled after initialization:<br>
           *	//Get the enabled API value.		
           *	$("#sliderContent").ejRotator("option", "enabled");			
           *	//Set the enabled API value
           *	$("#sliderContent").ejRotator("option", "enabled",false );
           * @memberof ejRotator
           * @instance
           */
            enabled: true,
            /**		
           * Specifies the number of Rotator Items to be displayed.
           * @default "1"
           * @type {string | number}
           * @example 
           * // Set the itemDisplay during initialization. 			
           * 	$("#sliderContent").ejRotator({ itemDisplay : "1"});			
           * @example 
           * //Get or set the itemDisplay after initialization:<br>
           *	//Get the itemDisplay API value.		
           *	$("#sliderContent").ejRotator("option", "itemDisplay");			
           *	//Set the itemDisplay API value
           *	$("#sliderContent").ejRotator("option", "itemDisplay","2" );
            * @memberof ejRotator
           * @instance
           */
            itemDisplay: "1",
            /**		
			* Specifies the number of Rotator Items to navigate on a single click (next/previous/play buttons). The itemMove property value must be less than or equal to the itemDisplay property value.
			* @default "1"
			* @type {string | number}
			* @example 
			* // Set the itemMove during initialization. 			
			* 	$("#sliderContent").ejRotator({ itemMove : "1"});		 
			* @example 
			* //Get or set the itemMove after initialization:<br>
			*	//Get the itemMove API value.		
			*	$("#sliderContent").ejRotator("option", "itemMove");			
			*	//Set the itemMove API value
			*	$("#sliderContent").ejRotator("option", "itemMove","2" );
			 * @memberof ejRotator
			* @instance
			*/
            itemMove: "1",
            /**		
			* Sets the speed of slide transition.
			* @default 600
			* @type {string | number}
			* @example 
			* // Set the speed during initialization. 			
			* 	$("#sliderContent").ejRotator({ speed : 600});			
			* @example 
			* //Get or set the speed after initialization:<br>
			*	//Get the speed API value.		
			*	$("#sliderContent").ejRotator("option", "speed");			
			*	//Set the speed API value
			*	$("#sliderContent").ejRotator("option", "speed",400 );
			 * @memberof ejRotator
			* @instance
			*/
            speed: 600,
            /**		
			* Sets the index of the slide that must be displayed first.
			* @default "1"
			* @type {string | number}
			* @example 
			* // Set the startIndex during initialization. 			
			* 	$("#sliderContent").ejRotator({ startIndex : "1"});			
			* @example 
			* //Get or set the startIndex after initialization:<br>
			*	//Get the startIndex API value.		
			*	$("#sliderContent").ejRotator("option", "startIndex");			
			*	//Set the startIndex API value
			*	$("#sliderContent").ejRotator("option", "startIndex","2" );
			 * @memberof ejRotator
			* @instance
			*/
            startIndex: "0",
            /**		
			* Enable play / pause button on rotator.
			* @default false
			* @type {boolean}
			* @example 
			* // Set the enablePlaybtn during initialization. 			
			* 	$("#sliderContent").ejRotator({ enablePlaybtn : false});			
			* @example 
			* //Get or set the enablePlaybtn after initialization:<br>
			*	//Get the enablePlaybtn API value.		
			*	$("#sliderContent").ejRotator("option", "enablePlaybtn");			
			*	//Set the enablePlaybtn API value
			*	$("#sliderContent").ejRotator("option", "enablePlaybtn",true );
			 * @memberof ejRotator
			* @instance
			*/
            enablePlaybtn: false,
            /**		
			* Rotates the Rotator Items continuously without user interference.
			* @default false
			* @type {boolean}
			* @example 
			* // Set the autoPlay during initialization. 			
			* 	$("#sliderContent").ejRotator({ autoPlay : false});		 
			* @example 
			* //Get or set the autoPlay after initialization:<br>
			*	//Get the autoPlay API value.		
			*	$("#sliderContent").ejRotator("option", "autoPlay");			
			*	//Set the autoPlay API value
			*	$("#sliderContent").ejRotator("option", "autoPlay",true );
			 * @memberof ejRotator
			* @instance
			*/
            autoPlay: false,
            /**		
			* Turns on or off the slide buttons (next and previous) in the Rotator Items. Slide buttons are used to navigate the Rotator Items.
			* @default false
			* @type {boolean}
			* @example 
			* // Set the slideButton during initialization. 			
			* 	$("#sliderContent").ejRotator({ slideButton : false});			
			* @example 
			* //Get or set the slideButton after initialization:<br>
			*	//Get the slideButton API value.		
			*	$("#sliderContent").ejRotator("option", "slideButton");			
			*	//Set the slideButton API value
			*	$("#sliderContent").ejRotator("option", "slideButton",true );
			 * @memberof ejRotator
			* @instance
			*/
            slideButton: true,
            /**		
			* Sets the width of a Rotator Item.
			* @default ""
			* @type {string | number}
			* @example 
			* // Set the slideWidth during initialization. 			
			* 	$("#sliderContent").ejRotator({ slideWidth : "600px"});			
			* @example 
			* //Get or set the slideWidth after initialization:<br>
			*	//Get the slideWidth API value.		
			*	$("#sliderContent").ejRotator("option", "slideWidth");			
			*	//Set the slideWidth API value
			*	$("#sliderContent").ejRotator("option", "slideWidth","400px" );
			 * @memberof ejRotator
			* @instance
			*/
            slideWidth: "",
            /**		
			* Sets the height of a Rotator Item.
			* @default ""
			* @type {string | number}
			* @example 
			* // Set the slideHeight during initialization. 			
			* 	$("#sliderContent").ejRotator({ slideHeight : "600px"});			
			* @example 
			* //Get or set the slideHeight after initialization:<br>
			*	//Get the slideHeight API value.		
			*	$("#sliderContent").ejRotator("option", "slideHeight");			
			*	//Set the slideHeight API value
			*	$("#sliderContent").ejRotator("option", "slideHeight","400px" );
			 * @memberof ejRotator
			* @instance
			*/
            slideHeight: "",
            /**		
			* Sets the space between the Rotator Items.
			* @default ""
			* @type {string | number}
			* @example 
			* // Set the frameSpace during initialization. 			
			* 	$("#sliderContent").ejRotator({ frameSpace : "10px"});			
			* @example 
			* //Get or set the frameSpace after initialization:<br>
			*	//Get the frameSpace API value.		
			*	$("#sliderContent").ejRotator("option", "frameSpace");			
			*	//Set the frameSpace API value
			*	$("#sliderContent").ejRotator("option", "frameSpace","20px" );
			 * @memberof ejRotator
			* @instance
			*/
            frameSpace: "",
            /**		
			* Resizes the Rotator when the browser is resized. 
			* @default false
			* @type {boolean}
			* @example 
			* // Set the allowResize during initialization. 			
			* 	$("#sliderContent").ejRotator({ allowResize : false});			
			* @example 
			* //Get or set the allowResize after initialization:<br>
			*	//Get the allowResize API value.		
			*	$("#sliderContent").ejRotator("option", "allowResize");			
			*	//Set the allowResize API value
			*	$("#sliderContent").ejRotator("option", "allowResize",true );
			 * @memberof ejRotator
			* @instance
			*/
            allowResize: false,
            /**		
          * Specifies the orientation for the Rotator control, that is, whether it must be rendered horizontally or vertically.
          * @default ej.orientation.Horizontal
          * @type {enum}
          * @example 
          * // Set the orientation during initialization. 			
          * 	$("#sliderContent").ejRotator({ orientation :  ej.orientation.Horizontal });			
          * @example 
          * //Get or set the orientation after initialization:<br>
          *	//Get the orientation API value.		
          *	$("#sliderContent").ejRotator("option", "orientation");			
          *	//Set the orientation API value
          *	$("#sliderContent").ejRotator("option", "orientation", ej.orientation.Vertical );
           * @memberof ejRotator
          * @instance
          */
            orientation: "horizontal",
            /**		
          * Specifies the position of the pager in the Rotator Item.
          * @default "outside"
          * @type {string | enum}
          * @example 
          * // Set the pagerPosition during initialization. 			
          * 	$("#sliderContent").ejRotator({ pagerPosition :  "outside" });			
          * @example 
          * //Get or set the pagerPosition after initialization:<br>
          *	//Get the pagerPosition API value.		
          *	$("#sliderContent").ejRotator("option", "pagerPosition");			
          *	//Set the pagerPosition API value
          *	$("#sliderContent").ejRotator("option", "pagerPosition", "topleft" );
           * @memberof ejRotator
          * @instance
          */
            pagerPosition: "outside",
            /**		
			* Turns on or off thumbnail support in the Rotator control. Thumbnail is used to navigate between slides. Thumbnail supports only single slide transition
			* You must specify the source for thumbnail elements through the thumbSource property.
			* @default false
			* @type {boolean}
			* @example 
			* // Set the thumbItem during initialization. 			
			* 	$("#sliderContent").ejRotator({ thumbItem : false});			
			* @example 
			* //Get or set the thumbItem after initialization:<br>
			*	//Get the thumbItem API value.		
			*	$("#sliderContent").ejRotator("option", "thumbItem");			
			*	//Set the thumbItem API value
			*	$("#sliderContent").ejRotator("option", "thumbItem",true );
			 * @memberof ejRotator
			* @instance
			*/
            thumbItem: false,
            /**		
          * Turns on or off the pager support in the Rotator control. The Pager is used to navigate the Rotator Items.
          * @default true
          * @type {boolean}
          * @example 
          * // Set the pager during initialization. 			
          * 	$("#sliderContent").ejRotator({ pager : false});			
          * @example 
          * //Get or set the pager after initialization:<br>
          *	//Get the pager API value.		
          *	$("#sliderContent").ejRotator("option", "pager");			
          *	//Set the pager API value
          *	$("#sliderContent").ejRotator("option", "pager",true );
           * @memberof ejRotator
          * @instance
          */
            pager: true,
            /**		
			* Specifies the source for thumbnail elements.
			* @default null
			* @type {object}
			* @example 
			*&lt;ul id=" thumbElement "&gt;
			*
			*&lt;li&gt;&lt;img src="../images/rotator/nature.jpg"/&gt;&lt;/li&gt;
			*
			*&lt;li&gt;&lt;img src="../images/rotator/bird.jpg" /&gt;&lt;/li&gt;
			*
			*&lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
			*
			*&lt;li&gt;&lt;img src="../images/rotator/seaview.jpg"/&gt;&lt;/li&gt;
			*
			*&lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
			*
			*&lt;/ul&gt;

			* // Set the thumbSource during initialization. 			
			* 	$("#sliderContent").ejRotator({ thumbSource : thumbElement});			 
			* @example 
			* //Get or set the thumbSource after initialization:<br>
			*	//Get the thumbSource API value.		
			*	$("#sliderContent").ejRotator("option", "thumbSource");			
			*	//Set the thumbSource API value
			*	$("#sliderContent").ejRotator("option", "thumbSource",thumbElement );
			 * @memberof ejRotator
			* @instance
			*/
            thumbSource: null,
            /**		
          * If the Rotator Item is an image, you can specify a caption for the Rotator Item. The caption text for each Rotator Item must be set by using the title attribute of the respective <image> tag.
          * The caption cannot be displayed if multiple Rotator Items are present.
          * @default false
          * @type {boolean}
          * @example 
          * // Set the caption during initialization. 			
          * 	$("#sliderContent").ejRotator({ caption : false});			
          * @example 
          * //Get or set the caption after initialization:<br>
          *	//Get the caption API value.		
          *	$("#sliderContent").ejRotator("option", "caption");			
          *	//Set the caption API value
          *	$("#sliderContent").ejRotator("option", "caption",true );
           * @memberof ejRotator
          * @instance
          */
            caption: false,
            /**		
          * Turns on keyboard interaction with the Rotator items. You must set this property to true to access the following keyboard shortcuts:
          * @default true
          * @type {boolean}
          * @example 
          * // Set the allowKeyboardNavigation during initialization. 			
          * 	$("#sliderContent").ejRotator({ allowKeyboardNavigation : false});		
          * @example 
          * //Get or set the allowKeyboardNavigation after initialization:<br>
          *	//Get the allowKeyboardNavigation API value.		
          *	$("#sliderContent").ejRotator("option", "allowKeyboardNavigation");			
          *	//Set the allowKeyboardNavigation API value
          *	$("#sliderContent").ejRotator("option", "allowKeyboardNavigation",true );
           * @memberof ejRotator
          * @instance
          */
            allowKeyboardNavigation: true,
            /**		
          * Specifies right to left transition of slides. 
          * @default false
          * @type {boolean}
          * @example 
          * // Set the rtl during initialization. 			
          * 	$("#sliderContent").ejRotator({ rtl : false});			
          * @example 
          * //Get or set the rtl after initialization:<br>
          *	//Get the rtl API value.		
          *	$("#sliderContent").ejRotator("option", "rtl");			
          *	//Set the rtl API value
          *	$("#sliderContent").ejRotator("option", "rtl",true );
           * @memberof ejRotator
          * @instance
          */
            rtl: true,
            /**		
      * Specifies the animation type for the Rotator Item. Animation options include slide, fastSlide, slowSlide, and other custom easing animations.
      * @default "slide"
      * @type {string | enum}
      * @example 
      * // Set the animation during initialization. 			
      * 	$("#sliderContent").ejRotator({ animation :  "slide" });			
      * @example 
      * //Get or set the animation after initialization:<br>
      *	//Get the animation API value.		
      *	$("#sliderContent").ejRotator("option", "animation");			
      *	//Set the animation API value
      *	$("#sliderContent").ejRotator("option", "animation", "slide" );
       * @memberof ejRotator
      * @instance
      */
            animation: "slide",
            //Events............ //
            /**     
            * This event is fired when the Rotator control is initialized.
            * @event
            * @name ejRotator#create 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the rotator model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * $("#sliderContent").ejRotator({
            *    create: function (args) {}
            * });      
            * @memberof ejRotator
            * @instance
            */
            create: null,
            /**     
            * This event is fired when the Rotator slides are changed.
            * @event
            * @name ejRotator#change 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the rotator model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.id the current rotator id.
            * @param {number}  argument.index returns the current slide index.
            * @example 
            * $("#sliderContent").ejRotator({
            *    change: function (args) {}
            * });      
            * @memberof ejRotator
            * @instance
            */
            change: null,
            /**     
            * This event is fired when autoplay is started.
            * @event
            * @name ejRotator#start 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the rotator model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.id the current rotator id.
            * @param {number}  argument.index returns the current slide index.
            * @example 
            * $("#sliderContent").ejRotator({
            *    start: function (args) {}
            * });      
            * @memberof ejRotator
            * @instance
            */
            start: null,
            /**     
            * This event is fired when autoplay is stopped or paused.
            * @event
            * @name ejRotator#stop 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the rotator model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.id the current rotator id.
            * @param {number}  argument.index returns the current slide index.
            * @example 
            * $("#sliderContent").ejRotator({
            *    stop: function (args) {}
            * });      
            * @memberof ejRotator
            * @instance
            */
            stop: null,
            /**     
			 * This event is fired when a thumbnail pager is clicked.
			 * @event
			 * @name ejRotator#thumbClick 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the rotator model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.id the current rotator id.
			 * @param {number}  argument.index returns the current slide index.
			 * @example 
             * $("#sliderContent").ejRotator({
             *    thumbClick: function (args) {}
             * });      
			 * @memberof ejRotator
			 * @instance
			 */
            thumbClick: null,
            /**     
			 * This event is fired when a pager is clicked.
			 * @event
			 * @name ejRotator#pagerClick 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the rotator model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.id the current rotator id.
			 * @param {number}  argument.index returns the current slide index.
			 * @example 
             * $("#sliderContent").ejRotator({
             *    pagerClick: function (args) {}
             * });      
			 * @memberof ejRotator
			 * @instance
			 */
            pagerClick: null

        },
        //Data Types
        dataTypes: {
            cssClass: "string",
            fields: "object",
            enabled: "boolean",
            itemDisplay: "",
            itemMove: "",
            speed: "",
            transitionDelay: "",
            startIndex: "",
            enablePlaybtn: "boolean",
            autoPlay: "boolean",
            slideWidth: "",
            slideHeight: "",
            frameSpace: "",
            allowResize: "boolean",
            orientation: "enum",
            pagerPosition: "enum",
            thumbItem: "boolean",
            thumbSource: "",
            pager: "boolean",
            caption: "boolean",
            rtl: "boolean",
            allowKeyboardNavigation: "boolean",
            animation: "string",
        },

        /**
       * Create the Rotator widget.
       * @private
       */
        _init: function () {
            this._initialize();
            if (this.model.dataSource == null)
                this._setValues();
            this._render();
            this._wireEvent();
            if (this.model.autoPlay) this.play();
        },
        /**
        * To initialize the Rotator.		
        * @private
        */
        _initialize: function () {
            this.element.attr("tabindex", 0);
            this._liCount = null;
            this.transitionDelay = null;
            this._animating = false;
            this._interval = null;
            this._thumbCount = null;
            this._liSize = null;
            this._containerSize = null;
            this.containerCss = this.model.orientation == ej.orientation.Horizontal ? "outerWidth" : "outerHeight";
            this.displayCss = this.model.orientation == ej.orientation.Horizontal ? "left" : "top";
            $.extend(jQuery.easing, {
                slowSlide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    return b + c * (ts * ts);
                },
                fastSlide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    var tc = ts * t;
                    return b + c * (tc * ts + -5 * ts * ts + 10 * tc + -10 * ts + 5 * t);
                },
                slide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    var tc = ts * t;
                    return b + c * (6 * tc * ts + -15 * ts * ts + 10 * tc);
                }

            })
        },
        /**
       * To configure the property values.
       * @private
       */
        _setValues: function (val) {
            this._setLicount();
            this._setSpeed(this.model.speed);
            this._setVisibleItemCount(this.model.itemDisplay);
            this._setItemMove(this.model.itemMove);
            this._setFrameSpace(this.model.frameSpace);
            this._setSlideWidth(this.model.slideWidth);
            this._setSlideHeight(this.model.slideHeight);
            this._changeProperty();
            this._setInitial();
        },
        _refresh: function () {
            this._setValues();
            this._setDimension();
            this._changeSkin();
        },
        _reRenderControl: function () {
            this._setLicount();
            this._setSpeed(this.model.speed);
            this._setVisibleItemCount(this.model.itemDisplay);
            this._setItemMove(this.model.itemMove);
            this._setFrameSpace(this.model.frameSpace);
            this._setSlideWidth(this.model.slideWidth);
            this._setSlideHeight(this.model.slideHeight);
            this._changeProperty();
            this._cloneItem();
            this._createCaption();
            this._changeSkin();
        },
        _refreshControl: function () {
            if (this.model.pager) {
                this._bulletWrapper.remove();
                this._createBulletControl();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-pager-wrap"))
                this._bulletWrapper.remove();

            if (this.model.caption) {
                if (this.element.siblings().hasClass("e-caption"))
                    this._caption.remove();
                this._createCaption();
            } else if (this.element.siblings().hasClass("e-caption"))
                this._caption.remove();

            if (this.model.thumbItem) {
                if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb")) {
                    this._thumb.remove();
                    this._thumbControl.remove();
                }
                this._createThumb();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb")) {
                this._thumb.remove();
                this._thumbControl.remove();
            }
            this._changeSkin();
        },
        /**
        * Section for check whether collection of items is local or remote data.	
        * @private
        */
        _checkDataBinding: function () {
            var source = this.model.dataSource;
            if (source != null) {
                this.element.addClass("onloading");
                if (ej.DataManager && source instanceof ej.DataManager)
                    this._initDataSource(source);
                else
                    this._renderItems(source);
            }
            this._setValues();
        },
        /**
        * Section for render items with Remote Data.	
        * @private
        */
        _initDataSource: function (source) {
            var proxy = this, queryPromise;
            queryPromise = source.executeQuery(this._getQuery());
            queryPromise.done(function (e) {
                proxy._renderItems(e.result);
            }).fail(function (e) {
                proxy.element.removeClass("onloading");
            });
        },
        _getQuery: function () {
            var column;
            if (ej.isNullOrUndefined(this.model.query)) {
                column = [], queryManager = ej.Query(), mapper = this.model.fields, col;
                for (col in mapper) {
                    if (col !== "tableName")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },
        /**
        * Section for rendering Rotator items.
        * @private
        */
        _renderItems: function (list) {
            this._generateTagItems(list);
            this.element.removeClass("onloading");
        },
        /**
        * Section for render Rotator items with DataSource.
        * @private
        */
        _generateTagItems: function (list) {
            var i, mapField = this._getMapper();
            for (i = 0; i < list.length; i++) {
                this.element.append(this._generateLi(list[i], mapField));
            }
        },
        /**
        * Section for mapping fields.
        * @private
        */
        _getMapper: function () {
            var mapper = this.model.fields, mapFld = { _text: null, _url: null };
            mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            mapFld._url = (mapper && mapper.url) ? mapper["url"] : "url";
            mapFld._attr = (mapper && mapper.htmlAttr) ? mapper["htmlAttr"] : "htmlAttr";
            return mapFld;
        },
        /**
         * Section for Render rotator items.
		 * @private
         */
        _generateLi: function (list, map) {
            var li = ej.buildTag("li"), imgTag;
            imgTag = $('<img  src="' + list[map._url] + '"title="' + list[map._text] + '"/>');
            this._setAttributes((list[map._attr]), imgTag);
            li.append(imgTag);
            return li;
        },
        /**
        * Section for set attributes to Rotator items.
        * @private
        */
        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    element.attr(key, data[key]);
            }
        },
        _createWrapper: function () {
            this._wrapper = ej.buildTag("div.e-inner e-" + this.model.orientation).insertBefore(this.element);
            if (this.model.dataSource != null)
                this._checkDataBinding();
            this._wrapper.append(this.element);
            this._innerWrapper = ej.buildTag("div.e-in-wrap").insertBefore(this._wrapper);
            this._innerWrapper.append(this._wrapper);
            this._outerWrapper = ej.buildTag("div.e-rotator-wrap e-widget " + this.model.cssClass).insertBefore(this._innerWrapper);
            this._outerWrapper.append(this._innerWrapper);
            if (!this.model.enabled) this._outerWrapper.addClass("e-disable").attr({ "aria-disabled": true });
        },
        _refreshTagItems: function (key, value) {
            this.model[key] = value;
            this.element.children('li').remove();
            this._checkDataBinding();
            this._cloneItem();
            this._refreshControl();
        },
        /**
         * Render Section for slide Button control.		
		 * @private
         */
        _createButtonControl: function () {
            this._buttonWrapper = ej.buildTag("div.e-nav").insertAfter(this._wrapper);
            if (this.model.slideButton) {
                this._prevButton = ej.buildTag("span.e-nav-btn e-icon e-previous").attr({ 'role': 'button' }).appendTo(this._buttonWrapper);
                this._nextButton = ej.buildTag("span.e-nav-btn e-icon e-next").attr({ 'role': 'button' }).appendTo(this._buttonWrapper);
            }
            this._buttonWrapper.appendTo(this._wrapper);
            this._createAutoPlay();
            if (this.model.enabled && this.model.slideButton) this._wireBtnEvents();
        },
        /**
         * Render Section for Play / Pause Button control.		
		 * @private
         */
        _createAutoPlay: function () {
            var icon;
            if (!this.model.enablePlaybtn) return false;
            icon = this._interval != null ? 'pause' : 'play';
            this._autoButton = ej.buildTag("span.e-nav-btn e-icon " + icon, "", {}, { role: "button" }).appendTo(this._buttonWrapper);
            if (this.model.enabled) this._wireAutoPlayEvents();
        },
        /**
         * Render Section for pager control.		
		 * @private
         */
        _createBulletControl: function () {
            var end, liElement;
            this._bulletWrapper = ej.buildTag("div.e-pager-wrap", "", {}, { role: "group", "aria-label": "pager", tabindex: "0" }).insertAfter(this._innerWrapper);
            this._bullet = ej.buildTag("ul.e-bullet e-default");
            end = (this._liCount - (this.model.itemDisplay * 2)) / this.model.itemMove;
            if ((this._liCount - (this.model.itemDisplay * 2)) % this.model.itemMove != 0)
                end++;
            for (i = 1; i <= end ; i++)
                ej.buildTag("li.e-icon", "", {}, { tabindex: "-1", "aria-selected": "false" }).appendTo(this._bullet);
            this._bullet.appendTo(this._bulletWrapper);
            this._setActiveBullet();
            this._setPagerposition(this.model.pagerPosition);
            if (this.model.enabled) this._wireBulletEvents();
        },
        /**
         * Section for Rendering caption.		
		 * @private
         */
        _createCaption: function () {
            var index;
            if (!this.model.caption) return false;
            this._caption = ej.buildTag("div.e-caption").append($('<span>')).insertAfter(this._buttonWrapper);
            index = Math.round((-Math.round(this.element.position()[this.displayCss])) / this._liSize); index--;
            this._setCaptionText(index);
        },
        /**
         * Section for Rendering thumbnail item.		
		 * @private
         */
        _createThumb: function () {
            var liElements, i;
            if (this.model.thumbSource != null) {
                this._thumb = this.model.pager ? ej.buildTag("div.e-thumb", "", {}, { tabindex: "0", role: "group" }).insertAfter(this._bulletWrapper) : ej.buildTag("div.e-thumb", "", {}, { tabindex: "0", role: "group" }).insertAfter(this._innerWrapper);
                this._thumbItems = ej.buildTag("ul.e-thumb-items e-ul");
                for (i = 0; i < $('#' + this.model.thumbSource).children('li').length ; i++) {
                    liElements = $('#' + this.model.thumbSource).children('li').eq(i).clone();
                    liElements.addClass("e-thumb-ele").attr('tabindex', -1).removeAttr("style").appendTo(this._thumbItems);
                }
                this._thumbItems.appendTo(this._thumb);
                $('#' + this.model.thumbSource).css("display", "none");
                this._setThumbProperty();
                this._setActiveThumb();

            }
        },
        /**
         * Section for Rendering thumbnail control.		
		 * @private
         */
        _createThumbControl: function () {
            this._thumbControl = ej.buildTag("div.e-thumb-nav").insertAfter(this._thumb);
            this._previous = ej.buildTag("span.e-thumb-btn e-icon e-previous e-disable", "", {}, { "aria-disabled": true }).appendTo(this._thumbControl);
            this._next = ej.buildTag("span.e-thumb-btn e-icon e-next e-enable", "", {}, { "aria-disabled": false }).appendTo(this._thumbControl);
            this._thumbControl.appendTo(this._outerWrapper);
            if (this.model.enabled) this._wireThumbEvents();
        },
        /**
        * To configure the custom theme for Rotator using cssClass property.		
        * @private
        */
        _setSkin: function (skin) {
            this._outerWrapper.removeClass(this.model.cssClass).addClass(skin);
        },
        /**
         * Section for clone Rotator items.		
		 * @private
         */
        _cloneItem: function () {
            var i = 0, CloneElements = ej.buildTag("ul");
            for (i = 0; i < this.model.itemDisplay; i++)
                this.element.children('li').eq(i).clone().addClass("clone").appendTo(this.element);
            for (i = (this._liCount - this.model.itemDisplay) ; i < this._liCount; i++)
                this.element.children('li').eq(i).clone().addClass("clone").prependTo(CloneElements);
            for (i = 0; i < CloneElements.children('li').length; i++)
                CloneElements.children('li').eq(i).clone().prependTo(this.element);
            this._setDimension();
        },
        /**
         * Section for undo the clone Rotator items.		
		 * @private
         */
        _undoClone: function () {
            for (i = 0; i < this._liCount; i++) {
                if (this.element.children('li').eq(i).hasClass("clone")) {
                    this.element.children('li').eq(i).remove();
                    this._setLicount();
                    i--;
                }
            }
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option, valueChange = false, controlChange = false;
            for (option in options) {
                switch (option) {
                    case "fields":
                    case "query":
                    case "dataSource":
                        this._refreshTagItems(option, options[option]);
                        break;
                    case "enabled":
                        this._changeState(options[option]);
                        break;
                    case "slideWidth":
                        this._setSlideWidth(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "slideHeight":
                        this._setSlideHeight(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "itemDisplay":
                        this._undoClone();
                        this._setVisibleItemCount(options[option]);
                        this._cloneItem();
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "itemMove":
                        options[option] = this._validateItemMove(options[option]);//this._setItemMove(options[option]);
                        this.model.itemMove = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "pagerPosition":
                        this._setPagerposition(options[option]);
                        break;
                    case "pager":
                        this.model.pager = options[option];
                        controlChange = true;
                        break;
                    case "thumbItem":
                        this.model.thumbItem = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "thumbSource":
                        this.model.thumbSource = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "caption":
                        this.model.caption = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "allowResize":
                        this.model.allowResize = options[option];
                        this._wireResizeEvents();
                        break;
                    case "orientation":
                        this.model.orientation = options[option];
                        this._setOrientation(options[option]);
                        break;
                    case "speed":
                        this._setSpeed(options[option]);
                        break;
                    case "frameSpace":
                        this._setFrameSpace(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "enablePlaybtn":
                        this.model.enablePlaybtn = options[option];
                        this._setAutoplay(options[option]);
                        this._changeSkin();
                        break;
                    case "startIndex":
                        this.model.startIndex = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "allowKeyboardNavigation":
                        this.model.allowKeyboardNavigation = options[option];
                        this._wireKeyboardEvents();
                        break;
                    case "slideButton":
                        this.model.slideButton = options[option];
                        this._setSlideButton(options[option]);
                        this._changeSkin();
                        break;
                    case "animation":
                        this.model.animation = options[option];
                        this._setSpeed(this.model.speed);
                        break;
                    case "cssClass": this._setSkin(options[option]); break;
                    case "rtl":
                        this.model.rtl = options[option];
                        break;
                }
            }

            if (valueChange) this._refresh();
            if (controlChange) this._refreshControl();
        },
        /**
         * Section for set values to Rotator items.		
		 * @private
         */
        _setDimension: function () {
            this._setLicount();
            this._setUlInitial();
            this._setWidth();
            this._setHeight();
            this._setContainerPercent();
            this._changeProperty();
        },
        /**
        * Render Section For Rotator		
        * @private
        */
        _render: function () {
            this._createWrapper();
            this._cloneItem();
            this._createButtonControl();
            this._createCaption();
            if (this.model.pager) this._createBulletControl();
            if (this.model.thumbItem) this._createThumb();
            this._outerWrapper.find(this.element).addClass("e-ul");
            this._changeSkin();
        },
        /**
        * To configure the Rotator container width		
        * @private
        */
        _setWidth: function () {
            this._containerWidth = this.model.orientation == "horizontal" ?
            ((parseInt(this.model.slideWidth) * this.model.itemDisplay) + (parseInt(this.model.frameSpace) * (parseInt(this.model.itemDisplay) - 1))) : this.model.slideWidth;
            this.element.parent().css('width', this._containerWidth);
            this.element.parents('.e-in-wrap').css('width', this._containerWidth);
            this.element.parents('.e-rotator-wrap').css('width', this._containerWidth);
            this._changeProperty();
        },
        _setHeight: function () {
            this._containerHeight = this.model.orientation == "vertical" ?
            ((parseInt(this.model.slideHeight) * this.model.itemDisplay) + (parseInt(this.model.frameSpace) * (parseInt(this.model.itemDisplay) - 1))) : this.model.slideHeight;
            this.element.parent().css('height', this._containerHeight);
            this._changeProperty();
        },
        /**
        * To configure the rotator base element width and height 		
        * @private
        */
        _setUlInitial: function () {
            if (this.model.orientation == ej.orientation.Horizontal) {
                this.element.css({
                    "width": this._liCount * this._liWidth,
                    "height": this._liHeight
                });
            }
            else {
                this.element.css({
                    "height": this._liCount * this._liHeight,
                    "width": this._liWidth

                });
            }
        },

        _setInitial: function () {
            this.element.css(this.displayCss, -(this._liSize * this.model.itemDisplay) - (this.model.startIndex * this.model.itemMove * this._liSize));
        },
        _setBegin: function () {
            this.element.css(this.displayCss, -(this._liSize * this.model.itemDisplay));
        },
        /**
         * To configure the rotator item width		
		 * @private
         */
        _setSlideWidth: function (val) {
            if (isNaN(parseInt(val)))
                this.model.slideWidth = this._getMaxSize("width");
            else {
                val = val.toString();
                this.model.slideWidth = this._checkValue(val, "width");
            }
            this.element.children('li').css("width", this.model.slideWidth);
            this._liWidth = this.element.children('li').outerWidth(true);
        },
        /**
         * To configure the rotator item height		
		 * @private
         */
        _setSlideHeight: function (val) {
            if (isNaN(parseInt(val)))
                this.model.slideHeight = this._getMaxSize("height");
            else {
                val = val.toString();
                this.model.slideHeight = this._checkValue(val, "height");
            }
            this.element.children('li').css("height", this.model.slideHeight);
            this._liHeight = this.element.children('li').outerHeight(true);
        },
        /**
         * To configure the pager position.		
		 * @private
         */
        _setPagerposition: function (val) {
            if (this.model.pager) {
                this._bullet.removeClass('e-default e-thumb-pos e-topleft e-topright e-bottomleft e-bottomright e-topCenter e-outside');
                this._bullet.addClass("e-" + val);
                if ((val == "bottomleft" || val == "bottomright") && this.model.thumbItem && this.model.thumbSource != null)
                    this._bullet.addClass('e-thumb-pos');
            }
        },
        /**
         * To configure the caption text.		
		 * @private
         */
        _setCaptionText: function (index) {
            if (this.model.caption)
                this._caption.children().html(this.element.children('li').eq(++index).children('img').attr("title"));
        },
        /**
         * To configure the autoplay mode.		
		 * @private
         */
        _setAutoplay: function (val) {
            if (val)
                this._createAutoPlay();
            else if (this._autoButton != null) {
                this._setPause();
                this._autoButton.remove();
            }
        },
        /**
         * To configure the slide button.		
		 * @private
         */
        _setSlideButton: function (val) {
            if (val)
                this._createButtonControl()
            else {
                this._prevButton.remove();
                this._nextButton.remove();
            }
        },
        /**
         * To configure the orientation.		
		 * @private
         */
        _setOrientation: function (val) {
            this._wrapper.removeClass("e-" + this.model.orientation).addClass("e-" + val);
            this._setFrameSpace(this.model.frameSpace);
            this._refresh();
            if (this.model.orientation == ej.orientation.Horizontal)
                this.element.css({ "left": -(Math.round(-(this.element.position().top) / this._liHeight) * this._liSize), "top": 0 });
            else
                this.element.css({ "top": -(Math.round(-(this.element.position().left) / this._liWidth) * this._liSize), "left": 0 });
        },
        /**
         * Section for set Active pager.		
		 * @private
         */
        _setActiveBullet: function () {
            var index = Math.round(-Math.round(this.element.position()[this.displayCss]) / (this._liSize * this.model.itemDisplay));
            if (this.model.itemMove < this.model.itemDisplay)
                index = -(Math.round(this.element.position()[this.displayCss]) + (this._liSize * this.model.itemDisplay)) / (this._liSize * this.model.itemMove);
            index = this.model.itemMove == this.model.itemDisplay ? index - 1 : index;
            this._bullet.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
        },
        /**
         * Section for set Active thumbnail item.		
		 * @private
         */
        _setActiveThumb: function () {
            var index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize);
            index--;
            this._thumbItems.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
            this._transferThumb(index);
        },
        _checkValue: function (val, orient) {
            if ((val.indexOf("%")) > 0) {
                if (Math.round(parseInt(val)) == "100") {
                    this.model.itemDisplay = 1;
                    this._setVisibleItemCount(this.model.itemDisplay);
                }
                val = this._convertPercentageToPixel($(this.element[0]).parent()[orient](), parseInt(val));
            }
            return (val);
        },
        /**
         * Section for converting Percentage To Pixel.		
		 * @private
         */
        _convertPercentageToPixel: function (parent, child) {
            return Math.round((child * parent) / 100);  //round       
        },
        /**
         * Section for converting Pixel To Percentage.	
		 * @private
         */
        _convertPixelToPercentage: function (parent, child) {
            return Math.round((child / parent) * 100); //round
        },
        _changeProperty: function () {
            this.containerCss = this.model.orientation == ej.orientation.Horizontal ? "outerWidth" : "outerHeight";
            this.displayCss = this.model.orientation == ej.orientation.Horizontal ? "left" : "top";
            if (this.model.orientation == ej.orientation.Horizontal) {
                this._liSize = this._liWidth;
                this._containerSize = this._containerWidth;
            }
            else {
                this._liSize = this._liHeight;
                this._containerSize = this._containerHeight;
            }
        },
        /**
         * To configure FrameSpace.	
		 * @private
         */
        _setFrameSpace: function (val) {
            if (isNaN(parseInt(val))) {
                this.model.frameSpace = "0px";
            } else {
                this.model.frameSpace = val;
                this.model.orientation == ej.orientation.Horizontal ? this.element.children('li').css({ "margin-right": val, "margin-bottom": 0 }) : this.element.children('li').css({ "margin-bottom": val, "margin-right": 0 });
            }
        },
        /**
         * Section for finding rotator items count.	
		 * @private
         */
        _setLicount: function () {
            this._liCount = this.element.children('li').length;
        },
        /**
         * Section for finding Visible Rotator Item Count.	
		 * @private
         */
        _setVisibleItemCount: function (val) {
            this.model.itemDisplay = val;
            this.model.itemMove = (this.model.itemMove <= this.model.itemDisplay) ? this.model.itemMove : this.model.itemDisplay;
            this._checkAndSet();
        },
        _checkAndSet: function () {
            if ((this.model.itemDisplay > 1) || (this.model.itemMove > 1)) {
                if (this.model.thumbItem) this.model.thumbItem = false;
                if (this.model.caption) this.model.caption = false;
            }
        },
        /**
         * Section for changing the border based on visible item count.	
		 * @private
         */
        _changeSkin: function () {
            var value;
            if ((this.model.itemDisplay > 1) || (this.model.itemMove > 1)) {
                this._innerWrapper.addClass("e-multiple");
                this._wrapper.removeClass("e-single");
            } else {
                this._innerWrapper.removeClass('e-multiple');
                this._wrapper.addClass("e-single");
            }
            if (this.model.enablePlaybtn)
                this._autoButton.css({
                    "left": (((this._innerWrapper.outerWidth(true) / 2) - (this._autoButton.width() / 2)) / this._innerWrapper.outerWidth(true)) * (100) + "%",
                    "top": (((this._innerWrapper.outerHeight(true) / 2) - (this._autoButton.height() / 2)) / this._innerWrapper.outerHeight(true)) * (100) + "%"
                });

            if (this.model.slideButton) {
                value = (((this._innerWrapper.outerHeight(true) / 2) - (this._prevButton.height() / 2)) / this._innerWrapper.outerHeight(true)) * (100) + "%";
                this._prevButton.css("top", value);
                this._nextButton.css("top", value);
            }
            if ((this.model.thumbItem) && (this.model.thumbSource != null) && (this._previous != null) && (this._next != null)) {
                value = this._outerWrapper.outerHeight(true) + (this._thumbItems.children('li').outerHeight(true) / 2) - (this._thumbItems.children('li').outerHeight(true)) - (this._previous.height() / 2);
                value = (value / this._outerWrapper.outerHeight(true)) * 100 + "%";
                this._previous.css("top", value);
                this._next.css("top", value);
            }
        },
        /**
         * To configure Number of items Move.	
		 * @private
         */
        _setItemMove: function (val) {
            this.model.itemMove = this._validateItemMove(val);
            this._checkAndSet();
        },
        _validateItemMove: function (val) {
            return ((val <= this.model.itemDisplay) ? val : this.model.itemDisplay);
        },
        /**
         * To configure Play Action.	
		 * @private
         */
        _setPlay: function () {
            if ((this.model.enablePlaybtn) && (this._autoButton.hasClass("play")))
                this._autoButton.removeClass("play").addClass("pause");
            this._raiseEvent("start");
        },
        /**
         * To configure Pause Action.	
		 * @private
         */
        _setPause: function () {
            if (this._interval != null) {
                if ((this.model.enablePlaybtn) && (this._autoButton.hasClass("pause")))
                    this._autoButton.removeClass("pause").addClass("play");
                clearInterval(this._interval);
                this._interval = null;
                this._raiseEvent("stop");
            }
        },
        _setContainerPercent: function () {
            this._containerPercent = this._convertPixelToPercentage(this.element.parents('.e-rotator-wrap').parent().width(), this.element.parent().width());
        },
        /**
         * To configure animation speed.	
		 * @private
         */
        _setSpeed: function (val) {
            var delay = 500;
            this.transitionDelay = this.model.animation != "slideshow" ? (val + delay) : ((val * 2) + delay);
        },
        /**
         * Section for set thumbnail element properties	
		 * @private
         */
        _setThumbProperty: function () {
            var thumbwidth, itemWidth, margin;
            itemWidth = this._thumbItems.children('li').outerWidth(true);
            thumbwidth = (this.element.parent().outerWidth(true)) - (((this.element.parent().outerWidth(true)) * 12) / 100);
            this._thumb.css('width', thumbwidth);
            this._thumbCount = Math.floor(this._thumb.innerWidth() / itemWidth);
            margin = Math.round((this.element.parent().outerWidth(true) * 6) / 100) + Math.floor((thumbwidth - (this._thumbCount * itemWidth)) / 2);
            this._thumb.css({ 'margin-left': margin, 'margin-right': margin, 'width': this._thumbCount * itemWidth });
            this._thumbItems.css({ 'width': (this._thumbItems.children('li').length) * itemWidth, 'height': this._thumbItems.children('li').outerHeight(true) });
            this._checkState();
        },
        _checkState: function () {
            if (this._thumbCount < (this._liCount - (this.model.itemDisplay * 2))) {
                if (!this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb-nav"))
                    this._createThumbControl();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb-nav"))
                this._thumbControl.remove();
        },
        /**
         * Section for removing active Rotator item.	
		 * @private
         */
        _removeActiveItem: function () {
            if (this.model.pager)
                this._bullet.children().removeClass('e-active').attr({ 'tabindex': -1, 'aria-selected': "false" });
            if (this.model.thumbItem)
                this._thumbItems.children().removeClass('e-active').attr({ 'tabindex': -1, 'aria-selected': "false" });
        },
        /**
         * Section for set active Rotator item.	
		 * @private
         */
        _setActiveItem: function (index) {
            if (this.model.pager)
                this._bullet.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
            if (this.model.thumbItem) {
                this._thumbItems.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
                this._transferThumb(index);
                for (i = 0; i < this._thumbItems.children('li').length; i++)
                    if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover'))
                        this._thumbItems.children('li').eq(i).removeClass('e-thumbhover');
            }
        },
        /**
         * Section for transfering thumbnail item to particular index.	
		 * @private
         */
        _transferThumb: function (index) {
            var move, right;
            if (this.model.thumbItem) {
                move = (index * this._thumbItems.children('li').outerWidth(true)) - (-this._thumbItems.position().left);
                right = -(this._thumbItems.position().left) + this._thumb.width();
                if ((this._thumbItems.width() - right) > move)
                    this._thumbMove(move);
                else
                    this._thumbMove((this._thumbItems.width() - right));
            }
        },

        _changeThumbControl: function (index) {
            var licount;
            if ((this.model.thumbItem) && (this.model.thumbSource != null)) {
                licount = $('#' + this.model.thumbSource).children('li').length;
                if ((index * this._thumbItems.children('li').outerWidth(true)) > 0) {
                    if (this._previous.hasClass("e-disable"))
                        this._previous.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
                }
                else if (this._previous.hasClass("e-enable"))
                    this._previous.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });

                if ((index * this._thumbItems.children('li').outerWidth(true)) < ((licount - this._thumbCount) * (this._thumbItems.children('li').outerWidth(true)))) {
                    if (this._next.hasClass("e-disable"))
                        this._next.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
                }
                else if (this._next.hasClass("e-enable"))
                    this._next.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
            }
        },
        /**
        * Section for find maximum item size.	
        * @private
        */
        _getMaxSize: function (display) {
            var max = 0, current = 0, i = 0, proxy = this;
            for (i = 0; i < this._liCount; i++) {
                current = this.element.children('li').eq(i)[display]();
                max = current > max ? current : max;
            }
            return max;
        },
        _calculateValue: function () {
            return (this._convertPercentageToPixel((this._containerWidth - ((this.model.itemDisplay - 1) * parseInt(this.model.frameSpace))), (100 / this.model.itemDisplay)));
        },
        /**
         * Section for enable / disable the rotator control.	
		 * @private
         */
        _changeState: function (val) {
            val ? this.enable() : this.disable();
        },
        /**
         * Section for find active item.	
		 * @private
         */
        _findActive: function () {
            var index, i;
            if (this.model.pager) {
                for (i = 0; i < this._bullet.children('li').length; i++) {
                    if (this._bullet.children('li').eq(i).hasClass('e-active'))
                        return i;
                }
            }
            else if (this.model.thumbItem) {
                for (i = 0; i < this._thumbItems.children('li').length; i++) {
                    if (this._thumbItems.children('li').eq(i).hasClass('e-active'))
                        return i;
                }
            }
            else if (!(this.model.pager) && !(this.model.thumbItem))
                index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize);
            index--;
            return index;
        },
        /**
        * Wiring the events to rotator control		
        * @private
        */
        _wireEvent: function () {
            if (!this.model.enabled) return false;
            this._wrapper.bind("mouseenter", $.proxy(this._showControl, this));
            this._wrapper.bind("mouseleave", $.proxy(this._hideControl, this));
            this._wireKeyboardEvents();
            this._wireResizeEvents();
            this._on(this._wrapper, "swiperight swipeleft swipedown swipeup", { control: true }, this._touchOperations);
            if (this.model.thumbSource) this._on(this._thumb, "swiperight swipeleft", { control: false }, this._touchOperations);
        },
        /**
        * Wiring the Keyboard events to rotator control		
        * @private
        */
        _wireKeyboardEvents: function () {
            if (this.model.allowKeyboardNavigation) {
                this._outerWrapper.bind("focusin", $.proxy(this._onFocusIn, this));
                this._outerWrapper.bind("focusout", $.proxy(this._onFocusOut, this));
            }
            else {
                this._outerWrapper.unbind("focusin", $.proxy(this._onFocusIn, this));
                this._outerWrapper.unbind("focusout", $.proxy(this._onFocusOut, this));
            }
        },
        /**
         * Wiring the events to slide button control		
		 * @private
         */
        _wireBtnEvents: function () {
            if (this.model.slideButton) {
                this._prevButton.bind("click", $.proxy(this._prevAction, this));
                this._nextButton.bind("click", $.proxy(this._nextAction, this));
            }
            else {
                this._prevButton.unbind("click", $.proxy(this._prevAction, this));
                this._nextButton.unbind("click", $.proxy(this._nextAction, this));
            }
        },
        /**
         * Wiring the events to Play / pause button control		
		 * @private
         */
        _wireAutoPlayEvents: function () {
            if (this.model.enablePlaybtn)
                this._on(this._autoButton, "click", this._autoPlay);
            else
                this._off(this._autoButton, "click", this._autoPlay);
        },
        /**
         * Wiring the  Resize events to Rotator control		
		 * @private
         */
        _wireResizeEvents: function () {
            if (this.model.allowResize)
                $(window).bind("resize", $.proxy(this._resizeEvent, this));
            else
                $(window).unbind("resize", $.proxy(this._resizeEvent, this));
        },
        /**
         * Wiring the events to thumbnail items.		
		 * @private
         */
        _wireThumbEvents: function () {
            if (this.model.thumbItem) {
                this._on(this._thumbItems.children(), "click", this._thumbClick);
                this._on(this._next, "click", this._thumbNext);
                this._on(this._previous, "click", this._thumbPrev);
            }
            else {
                this._off(this._thumbItems.children(), "click", this._thumbClick);
                this._off(this._next, "click", this._thumbNext);
                this._off(this._previous, "click", this._thumbPrev);
            }
        },
        /**
         * Wiring the events to pager.		
		 * @private
         */
        _wireBulletEvents: function () {
            if (this.model.pager)
                this._on(this._bullet.children(), "click", this._bulletClick);
            else
                this._off(this._bullet.children(), "click", this._bulletClick);
        },
        /**
        * Section for move to next thumbnail item.		
        * @private
        */
        _thumbNext: function () {
            var proxy = this, rightValue;
            if (!this.model.enabled) return false;
            if (this._previous.hasClass("e-disable"))
                this._previous.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
            rightValue = (-Math.round(this._thumbItems.position().left)) + this._thumb.width();
            if (this._thumbItems.width() - rightValue >= this._thumbItems.children('li').outerWidth(true))
                this._thumbMove(this._thumbItems.children('li').outerWidth(true));
            if ((this._thumbItems.width() - rightValue) <= this._thumbItems.children('li').outerWidth(true))
                this._next.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
        },
        /**
         * Section for move thumbnail item.		
		 * @private
         */
        _thumbMove: function (move) {
            var proxy = this;
            proxy._thumbItems.animate({
                "left": "-=" + move + "px",
            }, 600, function () { proxy._thumbItems.stop(true, true); });
        },
        /**
         * Section for move to previous thumbnail item.		
		 * @private
         */
        _thumbPrev: function () {
            var proxy = this, leftValue, itemWidth;
            if (!this.model.enabled) return false;
            if (this._next.hasClass("e-disable"))
                this._next.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
            leftValue = Math.round(this._thumbItems.position().left);
            itemWidth = this._thumbItems.children('li').outerWidth(true);
            if ((-leftValue) >= itemWidth) this._thumbMove(-itemWidth);
            if ((-leftValue) <= itemWidth) this._previous.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
        },
        _findLength: function () {
            var proxy = this, len;
            if (this.model.thumbItem) len = proxy._thumbItems.children('li').length - 1;
            if (this.model.pager) len = proxy._bullet.children('li').length - 1;
            if (!(this.model.pager) && !(this.model.thumbItem))
                len = ((this.element[this.containerCss]()) / this._liSize) - ((this.model.itemDisplay) * 2);
            return len;
        },
        /**
         * Section for change Active State.		
		 * @private
         */
        _changeActiveState: function (action) {
            var proxy = this, index, len;
            index = proxy._findActive();
            len = proxy._findLength();
            proxy._removeActiveItem();
            if (action == "prev") {
                if ((index > 0) && (this.model.animation != "slideshow")) {
                    proxy._setActiveItem((index - 1));
                    if (this.model.animation != "slideshow") this._setCaptionText((index - 1));
                }
                else {
                    proxy._setActiveItem(len);
                    if (this.model.animation != "slideshow") this._setCaptionText((len));
                }
            }
            else if (index < len) {
                proxy._setActiveItem((index + 1));
                if (this.model.animation != "slideshow") this._setCaptionText((index + 1));
            }
            else {
                proxy._setActiveItem(0);
                if (this.model.animation != "slideshow") this._setCaptionText(0);
            }
        },
        /**
        * Section for move slide.		
        * @private
        */
        _moveSlide: function (move, action) {
            var proxy = this, animation, speed, index, len, properties = {};
            animation = this.model.animation != "slideshow" ? this.model.animation : "slideshow";
            speed = this.model.animation != "slideshow" ? proxy.model.speed : 0;
            index = proxy._findActive();
            len = proxy._findLength();
            properties[this.displayCss] = proxy.element.position()[proxy.displayCss];
            properties[this.displayCss] -= move;
            index = action == "next" ? (index < len ? index + 1 : 0) : action == "prev" ? (index > 0 ? index - 1 : len) : 0;
            if (this.model.animation == "slideshow") { proxy.element.fadeOut(proxy.model.speed / 2); if (proxy.model.caption) proxy._caption.fadeOut(proxy.model.speed / 2); }
            proxy.element.animate(properties, speed, animation, function () { if (action == "end") proxy._setBegin(); proxy._animating = false; proxy._setCaptionText(index); });
            if (this.model.animation == "slideshow") { proxy.element.fadeIn(proxy.model.speed / 2); if (proxy.model.caption) proxy._caption.fadeIn(proxy.model.speed / 2); }
        },
        /**
        * Section for move to next slide.		
        * @private
        */
        _moveNext: function () {
            var proxy = this, ltValue, endValue, rbValue, move;
            proxy._animating = true;
            ltValue = Math.round(proxy.element.position()[proxy.displayCss]);
            endValue = (proxy.model.itemDisplay * proxy._liSize) - (proxy._liCount * proxy._liSize);
            rbValue = Math.round(proxy.element.position()[proxy.displayCss]) + proxy.element[proxy.containerCss]();//here width() only
            proxy._calculateItemMove();
            if ((ltValue > endValue) && (rbValue - proxy._containerSize > (proxy._move + parseInt(proxy.model.frameSpace)))) {
                return { move: proxy._move, action: "next" }
            }
            else {
                if (rbValue - proxy._containerSize <= (proxy._move + parseInt(proxy.model.frameSpace))) {
                    move = rbValue - proxy._containerSize - parseInt(proxy.model.frameSpace);
                    return { move: move, action: "end" }
                }
            }
        },
        /**
         * Section for move to previous slide.		
		 * @private
         */
        _movePrev: function () {
            var proxy = this, ltValue, rbValue, endValue, setValue;
            ltValue = Math.round(proxy.element.position()[proxy.displayCss]);
            rbValue = proxy.element.position()[proxy.displayCss]; +proxy.element[proxy.containerCss]();//here using width() only
            endValue = proxy.element[proxy.containerCss]();//here using width() only
            proxy._calculateItemMove();
            if ((rbValue < endValue) && (-ltValue > (proxy.model.itemDisplay * proxy._liSize))) {
                return { move: -proxy._move, action: "prev" }
            }
            else {
                setValue = ((proxy.model.itemDisplay * proxy._liSize) * 2) + ltValue - (proxy._liCount * proxy._liSize);
                proxy._move = ((proxy._liCount - (proxy.model.itemDisplay * 2)) % proxy.model.itemMove) * proxy._liSize;
                if (proxy._move == 0) proxy._calculateItemMove();
                if (proxy.model.orientation == ej.orientation.Horizontal)
                    proxy.element.css("left", setValue);
                else
                    proxy.element.css("top", setValue);
                return { move: -proxy._move, action: "prev" }
            }
        },
        /**
         * Section for enable auto play mode.		
		 * @private
         */
        _autoPlay: function (e) {
            if (!this.model.enabled) return false;
            var proxy = this, value, move;
            if (this._interval == null) {
                this._setPlay();
                move = function () {
                    proxy._animating = true;
                    value = proxy.model.rtl ? proxy._moveNext() : proxy._movePrev();
                    proxy._moveSlide(value.move, value.action);
                    proxy._changeActiveState(value.action);
                    proxy._changeThumbControl(proxy._findActive());
                    proxy._raiseEvent("change");
                }
                function SetInterval(move, speed) {
                    move();
                    return (setInterval(move, speed))
                }
                if (!this._animating)
                    this._interval = SetInterval(move, this.transitionDelay);
            }
            else
                this._setPause();
        },
        /**
        * Section For handle the click event on thumbnail Next button 
        * @private
        */
        _nextAction: function () {
            if (!this.model.enabled) return false;
            var proxy = this, interval, next, value;
            next = function () {
                value = proxy._moveNext();
                proxy._moveSlide(value.move, value.action);
                clearInterval(interval);
                proxy._changeActiveState("next");
                proxy._changeThumbControl(proxy._findActive());
                proxy._raiseEvent("change");
            }
            if (this._interval != null)
                this._setPause();

            if (!this._animating) {
                this._animating = true;
                interval = setInterval(next, "");
            }
        },
        /**
       * Section For handle the click event on thumbnail Previous button 
       * @private
       */
        _prevAction: function () {
            if (!this.model.enabled) return false;
            var proxy = this, interval, previous, value;
            previous = function () {
                value = proxy._movePrev();
                proxy._moveSlide(value.move, value.action);
                clearInterval(interval);
                proxy._changeActiveState("prev");
                proxy._changeThumbControl(proxy._findActive());
                proxy._raiseEvent("change");
            }
            if (this._interval != null)
                this._setPause();

            if (!this._animating) {
                this._animating = true;
                interval = setInterval(previous, "")
            }
        },
        /**
       * Section For handle the touch event 
       * @private
       */
        _touchOperations: function (e) {
            var isThumb = !e.data.control, direction, orient = this.model.orientation;
            if (isThumb)
                direction = e.type == "swipeleft" ? "left" : "right";
            else {
                if ((orient == "horizontal" && e.type == "swipeleft") ||
                    (orient == "vertical" && e.type == "swipeup"))
                    direction = "left";
                else if ((orient == "horizontal" && e.type == "swiperight") ||
                    (orient == "vertical" && e.type == "swipedown"))
                    direction = "right";
            }
            if (direction == "left")
                isThumb ? this._thumbNext() : this.nextSlide();
            else if (direction == "right")
                isThumb ? this._thumbPrev() : this.previousSlide();
        },
        /**
        * Section For handle the resize event 
        * @private
        */
        _resizeEvent: function () {
            var index, parent;
            if (!this.model.enabled) return false;
            index = Math.round(-Math.round(this.element.position().left) / this._liSize);
            parent = Math.round(this.element.parents('.e-rotator-wrap').parent().width());
            this._containerWidth = this._convertPercentageToPixel(parent, this._containerPercent);
            this._setSlideWidth(this._calculateValue().toString());
            this._setWidth();
            this.element.css("left", -(index * this._liSize));
            this._setUlInitial();
            if ((this.model.thumbItem) && (this.model.thumbSource != null))
                this._setThumbProperty();
        },
        /**
        * Section For Calculate item move value 
        * @private
        */
        _calculateItemMove: function () {
            if (!isNaN(this.model.itemMove))
                this._move = this._liSize * this.model.itemMove;
            else {
                this.model.itemMove = 1;
                this._move = this._liSize * this.model.itemMove;
            }
        },
        /**
         * Section For handle click event on thumbnail item
		 * @private
         */
        _thumbClick: function (event) {
            var index;
            if (!this.model.enabled) return false;

            if (this._interval != null)
                this._setPause();

            this._removeActiveItem();
            index = ((event.target.className == 'e-thumb-ele e-thumbhover') || (event.target.className == 'e-thumb-ele')) ? this._thumbItems.children('li').index($(event.target)) :
            this._thumbItems.children('li').index($(event.target).parents('.e-thumb-ele'));
            this._changeThumbControl(index);
            this._raiseEvent("thumbClick");
            this._setActiveItem(index);
            this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.itemDisplay * this._liSize) + (index * this.model.itemMove * this._liSize)), index);
        },
        /**
         * Section For handle click event on pager item
		 * @private
         */
        _bulletClick: function (event) {
            var index;
            if (!this.model.enabled) return false;
            if (this._interval != null)
                this._setPause();

            this._removeActiveItem();
            index = this._bullet.children('').index(event.target);
            this._changeThumbControl(index);
            this._setActiveItem(index);
            this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.itemDisplay * this._liSize) + (index * this.model.itemMove * this._liSize)), index);
            this._raiseEvent("pagerClick");
        },
        /**
         * Section For handle slide transition
		 * @private
         */
        _moveSlideContent: function (Current, Target, index) {
            var proxy = this, speed, animation, properties = {};
            animation = this.model.animation != "slideshow" ? this.model.animation : "slideshow";
            speed = this.model.animation != "slideshow" ? proxy.model.speed : 0;
            properties[this.displayCss] = proxy.element.position()[proxy.displayCss];
            properties[this.displayCss] += (Target - Current);
            if (this.model.animation == "slideshow") { proxy.element.fadeOut(proxy.model.speed / 2); if (proxy.model.caption) proxy._caption.fadeOut(proxy.model.speed / 2); }
            proxy.element.animate(properties, speed, animation, function () { proxy._setCaptionText(index); });
            if (this.model.animation == "slideshow") { proxy.element.fadeIn(proxy.model.speed / 2); if (proxy.model.caption) proxy._caption.fadeIn(proxy.model.speed / 2); }
            proxy._raiseEvent("change");

        },
        /**
       * destroy the Rotator widget.
       * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
       * @private
       */
        _destroy: function () {
            this._interval != null ? this.pause() : "";
            this._undoClone();
            this.element.removeAttr("style").insertAfter(this._outerWrapper);
            this.element.children().removeAttr("style");
            this._outerWrapper.remove();
        },
        /**
          * Section For handle events
          * @private
          */
        _raiseEvent: function (event) {
            if (this.model[event]) {
                return this._trigger(event, {
                    index: this._findActive(),
                    id: this.element.attr('id'),
                });
            }
            return false;
        },
        /**
        * Section For show the slide button controls with animation.
        * @private
        */
        _showControl: function (event) {
            if (!this.model.enabled) return false;
            if (this.model.slideButton) {
                this._prevButton.stop(true, true).fadeIn('slow');
                this._nextButton.stop(true, true).fadeIn('slow');
            }
            if (this.model.enablePlaybtn)
                this._autoButton.stop(true, true).fadeIn('slow');
            if ((this.model.caption) && (this._caption != null))
                this._caption.stop(true, true).fadeIn('slow');
            event.stopPropagation();
        },
        /**
        * Section For hide the slide button controls with animation.
        * @private
        */
        _hideControl: function (event) {
            if (!this.model.enabled) return false;
            if (this.model.slideButton) {
                this._prevButton.stop(true, true).fadeOut('slow');
                this._nextButton.stop(true, true).fadeOut('slow');
            }
            if (this.model.enablePlaybtn)
                this._autoButton.stop(true, true).fadeOut('slow');
            if ((this.model.caption) && (this._caption != null))
                this._caption.stop(true, true).fadeOut('slow');
            event.stopPropagation();
        },
        /**
        * Section For Handle focusin Event.
        * @private
        */
        _onFocusIn: function () {
            if (!this.model.enabled) return false;
            this._outerWrapper.addClass("e-Focused");
            $(this._outerWrapper).bind("keydown", $.proxy(this._focuseHandle, this));
        },
        /**
         * Section For Handle focusout in Event.
		 * @private
         */
        _onFocusOut: function () {
            this._outerWrapper.removeClass("e-Focused");
            $(this._outerWrapper).unbind("keydown", $.proxy(this._focuseHandle, this));
        },
        /**
         * Section For Handle keyboard navigation.
		 * @private
         */
        _focuseHandle: function (e) {
            if (!this.model.enabled) return false;
            if (this._outerWrapper.hasClass("e-Focused")) {
                e.keyCode = ((e.altKey) && ((e.keyCode == 37) || (e.keyCode == 40))) ? 1000 : e.keyCode;
                e.keyCode = ((e.altKey) && ((e.keyCode == 38) || (e.keyCode == 39))) ? 1001 : e.keyCode;
                switch (e.keyCode) {
                    case 37:        // Left Key
                    case 40:        // Down Key
                        e.preventDefault();
                        this._prevAction();
                        break;
                    case 38:        // Up Key
                    case 39:         // Right Key     
                        e.preventDefault();
                        this._nextAction();
                        break;
                    case 32:         // Space Key
                        e.preventDefault();
                        this._autoPlay();
                        break;
                    case 1000:
                        e.preventDefault();
                        this._thumbPrev();
                        this._thumbHover("prev");
                        break;
                    case 1001:
                        e.preventDefault();
                        this._thumbNext();
                        this._thumbHover("next");
                        break;
                    case 13:
                        e.preventDefault();
                        for (i = 0; i < this._thumbItems.children('li').length; i++)
                            if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover'))
                                this.gotoIndex(i);

                }
            }
        },
        /**
         * Section For Handle mouseover event on thumbnail item.
		 * @private
         */
        _thumbHover: function (action) {
            var proxy = this, index = null, len;
            for (i = 0; i < this._thumbItems.children('li').length; i++) {
                if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover')) {
                    index = i;
                    this._thumbItems.children('li').eq(i).removeClass('e-thumbhover');
                    break;
                }
            }
            if (index == null)
                index = this._findActive();
            len = proxy._findLength();
            if (action == "prev") {
                if (index > 0)
                    this._thumbItems.children().eq(--index).addClass('e-thumbhover');
                else
                    this._thumbItems.children().eq(0).addClass('e-thumbhover');
            }
            else if (index < len)
                this._thumbItems.children().eq(++index).addClass('e-thumbhover');
            else
                this._thumbItems.children().eq(len).addClass('e-thumbhover');
        },
        //Public Methods...
        /**
       * Enables the Rotator control.
       * @alias enable
       * @return jQuery
       * @example 
       * //initialize the Rotator object
       *	var slideObj = $("#sliderContent").data("ejRotator");
       *	//To Enables the Rotator control.
       *	slideObj.enable();
       * @example 
       * $("#sliderContent").ejRotator("enable");
       * @memberof ejRotator
       * @instance
       */
        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this._outerWrapper.removeClass("e-disable").attr({ "aria-disabled": false });
            }
        },
        /**
       * Disables the Rotator control.
       * @alias disable
       * @return jQuery
       * @example 
       * //initialize the Rotator object
       *	var slideObj = $("#sliderContent").data("ejRotator");
       *	//To Disables the Rotator control.
       *	slideObj.disable();
       * @example 
       * $("#sliderContent").ejRotator("disable");
       * @memberof ejRotator
       * @instance
       */
        disable: function () {
            if (this.model.enabled) {
                this.model.enabled = false;
                this._outerWrapper.addClass("e-disable").attr({ "aria-disabled": true });
                if (this._interval != null)
                    this._setPause();
            }
        },
        /**
        * This method is used to move to the next slide from the current slide. If the current slide is the last slide, then the first slide will be treated as the next slide.
		* @alias nextSlide
		* @return jQuery
		* @example 
		* //initialize the Rotator object
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	//Moves to the next slide.
		*	slideObj.nextSlide();
		* @example 
        * $("#sliderContent").ejRotator("nextSlide");
		* @memberof ejRotator
		* @instance
        */
        nextSlide: function () {
            if (this.model.enabled)
                this._nextAction();
        },
        /**
        * This method is used to move to the previous slide from the current slide. If the current slide is the first slide, then the last slide will be treated as the previous slide.
		* @alias previousSlide
		* @return jQuery
		* @example 
		* //initialize the Rotator object
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	//Moves to the next slide.
		*	slideObj.previousSlide();
		* @example 
        * $("#sliderContent").ejRotator("previousSlide");
		* @memberof ejRotator
		* @instance
        */
        previousSlide: function () {
            if (this.model.enabled)
                this._prevAction();
        },
        /**
        * This method is used to move slides continuously (or start autoplay) in the specified autoplay direction.
		* @alias play
		* @return jQuery
		* @example 
		* //initialize the Rotator object
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	//To Starts autoplay.
		*	slideObj.play();
		* @example 
        * $("#sliderContent").ejRotator("play");
		* @memberof ejRotator
		* @instance
        */
        play: function () {
            if (this._interval == null)
                this._autoPlay();
        },
        /**
        * This method is used to pause autoplay.
		* @alias pause
		* @return jQuery
		* @example 
		* //initialize the Rotator object
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	//To Pauses autoplay.
		*	slideObj.pause();
		* @example 
        * $("#sliderContent").ejRotator("pause");
		* @memberof ejRotator
		* @instance
        */
        pause: function () {
            if (this._interval != null)
                this._setPause();
        },
        /**
        * This method is used to get the current slide index.
		* @alias getIndex
		* @return number
		* @example 
		* //initialize the Rotator object
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	// Gets the index value of the current slide.
		*	slideObj.getIndex();
		* @example 
        * $("#sliderContent").ejRotator("getIndex");
		* @memberof ejRotator
		* @instance
        */
        getIndex: function () {
            var index, count, display = "", i;
            if (this.model.itemDisplay > 1) {
                index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize); index--;
                count = this._liCount - (this.model.itemDisplay * 2);
                index = index - (this.model.itemDisplay - 1);
                for (i = 0; i < this.model.itemDisplay; i++) {
                    if (index < count) {
                        display += "  " + index.toString(); +" ";
                        index++;
                    }
                    else {
                        index = 0; i--;
                    }
                }
                return display;
            }
            else
                return this._findActive();
        },
        /**
        * This method is used to move a slide to the specified index.
		* @alias getIndex
		* @return jQuery
		* @param {number} index  index of an slide
		* @example 
		* //initialize the Rotator object
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	// Moves the slide to the specified index.
		*	slideObj.gotoIndex(3);
		* @example 
        * $("#sliderContent").ejRotator("gotoIndex",3);
		* @memberof ejRotator
		* @instance
        */
        gotoIndex: function (index) {
            if (this._interval != null)
                this._setPause();
            this._removeActiveItem();
            if (this.model.thumbItem) this._changeThumbControl(index);
            this._setActiveItem(index);
            this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.itemDisplay * this._liSize) + (index * this.model.itemMove * this._liSize)), index);
        },
    })
    /**
    * Enum for pager position	 
    * @enum {string}
    * @global 
    */
    ej.Rotator.pagerPosition = {
        /**  Pager is positioned at the top-left corner of an Item. */
        TopLeft: "topleft",
        /**  Pager is positioned at the top-left corner of an Item. */
        TopRight: "topright",
        /**  Pager is positioned at the top-right corner of an Item. */
        BottomLeft: "bottomleft",
        /**  Pager is positioned at the bottom-left corner of an Item. */
        BottomRight: "bottomright",
        /**  Pager is positioned at the bottom-right corner of an Item. */
        TopCenter: "topCenter",
        /**  Pager is positioned outside an Item. */
        Outside: "outside"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    // ejSplitter is the plugin name 
    // "ej.Splitter" is "namespace.className" will hold functions and properties
    /**
    * @namespace ej
	* @class ejSplitter
	* @requires jQuery
	* @requires ej.web.all
	* @classdesc Custom Design for Html splitter control.<br>
	* &lt;div id="innerSpliter"&gt;<br>
	* 	&lt;div&gt;<br>
	* 		&lt;div class="cont"&gt;Pane 1 &lt;/div&gt;<br>
	* 	&lt;/div&gt;<br>
	* 	&lt;div&gt;<br>
	* 		&lt;div class="cont"&gt;Pane 2 &lt;/div&gt;<br>
	* 	&lt;/div&gt; <br>
	* &lt;/div&gt; <br>
	* &lt;script&gt;
	* // Create Splitter <br>
    * $('#innerSpliter').ejSplitter(); 	<br>	
	* &lt;/script&gt;
	*/
    ej.widget("ejSplitter", "ej.Splitter", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-splitter",
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Specify the CSS class to splitter control to achieve custom theme.
			* @default null
			* @type {string}
			* @example 
			* //To set cssClass API value during initialization  
			* 	$("#innerSplitter").ejSplitter({ cssClass: "gradient-lime"});					 
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the  cssClass  
			*	$("#innerSplitter").ejSplitter("option", "cssClass");<br>			
			*	//Sets the  cssClass 
			*	$("#innerSplitter").ejSplitter("option", "cssClass",  "gradient-lime" ); 
			* @memberof ejSplitter
			* @instance
			*/
            cssClass: "",
            /**		
			* Specify the orientation for spliter control.
			* @default ej.orientation.Horizontal or “horizontal”  
			* @type {Enum}
			* @example 
			* //To set orientation API value during initialization  
			* 	$("#innerSplitter").ejSplitter({ orientation: ej.orientation.Horizontal });					 
			* @example 
			* //Get or set the orientation API, after initialization:<br>
			*	//Gets the  orientation  
			*	$("#innerSplitter").ejSplitter("option", "orientation");<br>			
			*	//Sets the  orientation 
			*	$("#innerSplitter").ejSplitter("option", "orientation",  ej.orientation.Horizontal  ); 
			* @memberof ejSplitter
			* @instance
			*/
            orientation: "horizontal",
            /**		
			* Specify properties for each pane.
			* @default null
			* @type {array}
			* @example 
			* //To set properties API value during initialization  
			* 	$("#innerSplitter").ejSplitter(properties: [{ paneSize: "100px" }, { paneSize: "50px"}]);					 
			* @example 
			* //Get or set the properties API, after initialization:<br>
			*	//Gets the  properties  
			*	$("#innerSplitter").ejSplitter("option", "properties");<br>			
			*	//Sets the  properties 
			*	$("#innerSplitter").ejSplitter("option", "properties",  [{ paneSize: "100px"}] ); 
			* @memberof ejSplitter
			* @instance
			*/
            properties: [],
            /**		
			* Specify height for splitter control.
			* @default “100%”
			* @type {string}
			* @example 
			* //To set height API value during initialization  
			* 	$("#innerSplitter").ejSplitter(height: “100%” );					 
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the  height  
			*	$("#innerSplitter").ejSplitter("option", "height");<br>			
			*	//Sets the  height 
			*	$("#innerSplitter").ejSplitter("option", "height", “100%” ); 
			* @memberof ejSplitter
			* @instance
			*/
            height: null,
            /**		
			* Specify width for splitter control.
			* @default “100%”
			* @type {string}
			* @example 
			* //To set width API value during initialization  
			* 	$("#innerSplitter").ejSplitter(width: 600 );					 
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the  width  
			*	$("#innerSplitter").ejSplitter("option", "width");<br>			
			*	//Sets the  width 
			*	$("#innerSplitter").ejSplitter("option", "width", 600 ); 
			* @memberof ejSplitter
			* @instance
			*/
            width: null,
            /**		
			* Specify window resizing behaviour for splitter control.
			* @default true
			* @type {boolean}
			* @example 
			* //To set windowResizing API value during initialization  
			* 	$("#innerSplitter").ejSplitter(windowResizing: true );					 
			* @example 
			* //Get or set the windowResizing API, after initialization:<br>
			*	//Gets the  windowResizing  
			*	$("#innerSplitter").ejSplitter("option", "windowResizing");<br>			
			*	//Sets the  windowResizing 
			*	$("#innerSplitter").ejSplitter("option", "windowResizing", false ); 
			* @memberof ejSplitter
			* @instance
			*/
            windowResizing: false,
            /**		
			* Specify Right to Left Direction for splitter control.
			* @default false
			* @type {boolean}
			* @example 
			* //To set rtl API value during initialization  
			* 	$("#innerSplitter").ejSplitter(rtl: true );					 
			* @example 
			* //Get or set the rtl API, after initialization:<br>
			*	//Gets the  rtl  
			*	$("#innerSplitter").ejSplitter("option", "rtl");<br>			
			*	//Sets the  rtl 
			*	$("#innerSplitter").ejSplitter("option", "rtl", false ); 
			* @memberof ejSplitter
			* @instance
			*/
            rtl: false,
            /**		
			* Specify animation speed for the Splitter pane movement while collapsing and expanding.
			* @default 300
			* @type {number}
			* @example 
			* //To set animationSpeed API value during initialization  
			* 	$("#innerSplitter").ejSplitter(animationSpeed: 150 );					 
			* @example 
			* //Get or set the animationSpeed API, after initialization:<br>
			*	//Gets the  animationSpeed  
			*	$("#innerSplitter").ejSplitter("option", "animationSpeed");<br>			
			*	//Sets the  animationSpeed 
			*	$("#innerSplitter").ejSplitter("option", "animationSpeed", 150 ); 
			* @memberof ejSplitter
			* @instance
			*/
            animationSpeed: 300,

            //Events
            /**     
			 * Fires when before expand collapse in splitter control.
			 * @event
			 * @name ejSplitter#beforeExpandCollapse 	
			 * @param {Object} argument Event parameters from splitter control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.collapsed returns collapsed pane details.
			 * @param {object}  argument.expanded returns expanded pane details.	
			 * @param {object}  argument.model returns the splitter model.
			 * @param {number}  argument.splitbarIndex returns the current split bar index.			 			 
			 * @param {string}  argument.splitterId  returns the Splitter id.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //beforeExpandCollapse event for splitter control
             * $("#innerSplitter").ejSplitter({
             *    beforeExpandCollapse: function (args) {}
             * });      
			 * @memberof ejSplitter
			 * @instance
			 */
            beforeExpandCollapse: null,
            /**     
			 * Fires when expandCollapse in splitter control pane.
			 * @event
			 * @name ejSplitter#expandCollapse 	
			 * @param {Object} argument Event parameters from splitter control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.collapsed returns collapsed pane details.
			 * @param {object}  argument.expanded returns expanded pane details.	
			 * @param {object}  argument.model returns the splitter model.
			 * @param {number}  argument.splitbarIndex returns the current split bar index.			 			 
			 * @param {string}  argument.splitterId  returns the Splitter id.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //expandCollapse event for splitter control
             * $("#innerSplitter").ejSplitter({
             *    expandCollapse: function (args) {}
             * });      
			 * @memberof ejSplitter
			 * @instance
			 */
            expandCollapse: null,
            /**     
			 * Fires when resize in splitter control pane.
			 * @event
			 * @name ejSplitter#resize 	
			 * @param {Object} argument Event parameters from splitter control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.collapsed returns collapsed pane details.
			 * @param {object}  argument.expanded returns expanded pane details.	
			 * @param {object}  argument.model returns the splitter model.
			 * @param {number}  argument.splitbarIndex returns the current split bar index.			 			 
			 * @param {string}  argument.splitterId  returns the Splitter id.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //resize event for splitter control
             * $("#innerSplitter").ejSplitter({
             *    resize: function (args) {}
             * });      
			 * @memberof ejSplitter
			 * @instance
			 */
            resize: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            orientation: "enum",
            properties: "data",
            windowResizing: "boolean",
            rtl: "boolean",
            animationSpeed: "number"
        },
        /**
         * To initialize the maskedit textbox control.		
		 * @private
         */
        // constructor function
        _init: function () {
            this._initialize();
            this._render();
            this._wireEvents(this.model.windowResizing);
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "windowResizing": this._windowResizing(options[option]); break;
                    case "rtl": this._rtl(options[option]); break;
                    case "orientation": this._setOrientation(options[option]); break;
                }
            }
        },
        /**
        * To refresh the splitter control pane resizing.		
		* @return jQuery
		* @example 
		* &lt;div id="innerSpliter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;		
		* &lt;script&gt;
		* // Create splitter control
		* var splitterObj = $("#innerSplitter").data("ejSplitter");		
		* splitterObj.refresh(); // refresh the splitter control pane resizing.		
		* &lt;/script&gt;
		* &lt;div id="innerSpliter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;	 			
		* &lt;script&gt;
		* // refresh the splitter control
		* $("#innerSplitter").ejSplitter("refresh");		
		* &lt;/script&gt;
		* @memberof ejSplitter
		* @instance
         */
        refresh: function () {
            this._setPanesSize();
            this._getPanesPercent();
        },
        /**
        * To collapse the splitter control pane.		
		* @return jQuery
		* @example 
		* &lt;div id="innerSpliter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;		
		* &lt;script&gt;
		* // Create splitter control
		* var splitterObj = $("#innerSplitter").data("ejSplitter");		
		* maskObj.collapse(); // collpase the splitter control pane.		
		* &lt;/script&gt;
		* &lt;div id="innerSpliter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;	 			
		* &lt;script&gt;
		* // collpase the splitter control
		* $("#innerSplitter").ejSplitter("collpase");		
		* &lt;/script&gt;
		* @memberof ejSplitter
		* @instance
         */
        collapse: function (paneIndex) {
            this._clickArrow(paneIndex, true, true);
        },
        /**
        * To expand the splitter control pane.		
		* @return jQuery
		* @example 
		* &lt;div id="innerSpliter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;		
		* &lt;script&gt;
		* // Create splitter control
		* var splitterObj = $("#innerSplitter").data("ejSplitter");		
		* maskObj.expand(); // expand the splitter control pane.		
		* &lt;/script&gt;
		* &lt;div id="innerSpliter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;	 			
		* &lt;script&gt;
		* // expand the splitter control
		* $("#innerSplitter").ejSplitter("expand");		
		* &lt;/script&gt;
		* @memberof ejSplitter
		* @instance
         */
        expand: function (paneIndex) {
            this._clickArrow(paneIndex, false, true);
        },
        /**
         * To configure the arrow symbol in splitter control pane..
		 * @private
         */
        _clickArrow: function (index, bool, canClick) {
            if (this._inMovement || index < 0 || index > this.panes.length) return false;
            var arrow, cls = bool ? "e-collapse" : "e-expand", splitbars = this.element.children(".e-splitbar:not(.e-shadowbar)");

            if (index == splitbars.length) arrow = this._clickArrow(index - 1, !bool, false);
            else arrow = $(splitbars[index]).children("." + cls);

            if (canClick) arrow.css("display") != "none" && arrow.mousedown();
            else return arrow;
        },
        /**
         * To add the new pane to the splitter control.
		 * @private
         */
        addPane: function (content, property, index) {
            var paneCount = this.panes.length;
            index = this._getNumber(index);
            if (ej.isNullOrUndefined(index)) index = paneCount;
            if (index < 0 || index > paneCount) return "";

            var property = this._getPaneProperty(property), paneDiv, paneDivSize, requiredSize;
            paneDiv = ej.buildTag("div.e-pane e-" + this.model.orientation.substr(0, 1) + "-pane");
            this.element.append(paneDiv[this.containerCss](property.paneSize));
            paneDivSize = property.paneSize = paneDiv[this.containerCss]();
            paneDiv.remove();

            var start = index, end = paneCount, i, j, insert, before, direction = 2, getters = {}, taken = 0, canInsert = false;
            requiredSize = paneDivSize + this._bar;

            for (j = 0; j < direction; j++) {
                for (i = start; i < end; i++) {
                    var _paneSize = $(this.panes[i])[this.containerCss]();
                    var availableSpace = _paneSize - this.model.properties[i].minSize;

                    if (availableSpace >= requiredSize - taken) {
                        getters[i] = _paneSize - (requiredSize - taken);
                        canInsert = true;
                        break;
                    }
                    else if (availableSpace > 0) {
                        getters[i] = this.model.properties[i].minSize;
                        taken += availableSpace;
                    }
                }
                if (canInsert) break;
                else end = start, start = 0;
            }
            if (!canInsert) return "";

            for (var pos in getters)
                $(this.panes[pos])[this.containerCss](getters[pos]);

            if (index == paneCount) {
                insert = "insertBefore", before = 1;
                paneDiv.insertAfter($(this.panes[index - 1]));
            }
            else {
                insert = "insertAfter", before = 0;
                paneDiv.insertBefore($(this.panes[index]));
            }
            this.model.properties.splice(index, 0, property);
            this.panes.splice(index, 0, paneDiv);
            var splitBar = this._createSplitBar(index - before);
            splitBar[insert](paneDiv);
            paneDiv.append(content);
            this._updateModel();

            return paneDiv;
        },
        /**
         * To remove the new pane from the splitter control.
		 * @private
         */
        removePane: function (index) {
            var paneCount = this.panes.length - 1;
            index = this._getNumber(index);
            if (ej.isNullOrUndefined(index)) index = paneCount;
            if (index < 0 || index > paneCount) return null;

            var targetPane, nextPane, splitbars, removedSize;
            targetPane = $(this.panes[index]);
            removedSize = targetPane[this.containerCss]() + this._bar;
            targetPane.remove();

            splitbars = this.element.children(".e-splitbar:not(.e-shadowbar)");
            if (index == paneCount) {
                nextPane = $(this.panes[index - 1]);
                $(splitbars[index - 1]).remove();
            }
            else {
                nextPane = $(this.panes[index + 1]);
                $(splitbars[index]).remove();
            }
            nextPane[this.containerCss](nextPane[this.containerCss]() + removedSize);

            this._removeArrays(index);
            this._updateModel();
        },
        /**
		 * To remove the specified pane from the splitter control pane list.
		 * @private
         */
        _removeArrays: function (index) {
            this.model.properties.splice(index, 1);
            this.panes.splice(index, 1);
            this.oldPaneSize.splice(index, 1);
            this.oldPanePercent.splice(index, 1);
            this._sizePercent.splice(index, 1);
        },
        /**
         * To return the value as a valid one.
		 * @private
         */
        _getNumber: function (value) {
            value = parseFloat(value);
            return isNaN(value) ? null : value;
        },
        /**
         * To update the pane size in percentage.
		 * @private
         */
        _updateModel: function () {
            for (var i = 0; i < this.panes.length; i++)
                this.model.properties[i].paneSize = $(this.panes[i])[this.containerCss]();
            this._getPanesPercent();
        },
        /**
         * To get the pane properties.
		 * @private
         */
        _getPaneProperty: function (property) {
            var _default = { paneSize: 10, minSize: 10, maxSize: null, collapsible: true, resizable: true };
            return $.extend(_default, property);
        },
        /**
         * To configure the custom theme for splitter control using cssClass property		
		 * @private
         */
        _changeSkin: function (skin) {
            this.element.removeClass(this.model.cssClass).addClass(skin);
        },
        /**
         * To configure the custom theme for splitter control using cssClass property		
		 * @private
         */
        _windowResizing: function (boolean) {
            if (boolean) this._wireEvents(boolean);
            else this._unWireEvents();
        },
        /**
         * To configure the splitter control orientation.		
		 * @private
         */
        _setOrientation: function (boolean) {
            this._unWireEvents();
            this._destroy();
            this.model.orientation = boolean;
            this._init();
        },
        /**
         * To destroy the splitter control.
		 * @private
         */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.removeClass("e-widget " + this.model.cssClass + " e-" + this.model.orientation);
            this.element.children(".e-splitbar").remove();
            this.element.children(".e-pane").removeClass("e-pane e-" + this.model.orientation.substr(0, 1) + "-pane").height("").width("");
        },
        /**
         * To initialize the splitter control with it's property.
		 * @private
         */
        _initialize: function () {
            this.panes = [];
            this.oldPaneSize = [];
            this.oldPanePercent = [];
            this.shadowBar = null;
            this._inMovement = false;
            this.containerCss = this.model.orientation == "horizontal" ? "width" : "height";
            this.displayCss = this.model.orientation == "horizontal" ? "left" : "top";
            this._bar = 7;      // For the center splitbar size
        },
        /**
         * Render section for the splitter control.
		 * @private
         */
        _render: function () {
            this.element.addClass("e-widget " + this.model.cssClass + " e-" + this.model.orientation).attr("data-role", "splitter");
            var i, j, target = this.element[0];
            for (i = 0, j = 0; i < target.children.length; i++) {
                $(target.children[i]).addClass("e-pane e-" + this.model.orientation.substr(0, 1) + "-pane");
                this.panes.push(target.children[i]);
            }
            this._setPanesProperty();
            this._insertSplitBar();
            this._setDimentions();
            this._setPanesSize();
            this._getPanesPercent();
            this._checkProperties();
        },
        /**
         * To set the splitter control pane with it's properties.
		 * @private
         */
        _setPanesProperty: function () {
            for (var p = 0; p < this.panes.length; p++) {
                if (this.model.properties[p] != undefined) {
                    this.model.properties[p].paneSize = this.model.properties[p].paneSize == undefined ? "0px" : this.model.properties[p].paneSize;
                    this.model.properties[p].minSize = isNaN(parseFloat(this.model.properties[p].minSize)) ? 10 : parseFloat(this.model.properties[p].minSize);
                    this.model.properties[p].maxSize = isNaN(parseFloat(this.model.properties[p].maxSize)) ? null : parseFloat(this.model.properties[p].maxSize);
                    this.model.properties[p].collapsible = this.model.properties[p].collapsible != false ? true : false;
                    this.model.properties[p].resizable = this.model.properties[p].resizable != false ? true : false;
                }
                else this.model.properties.push({ paneSize: "0px", minSize: 10, maxSize: null, collapsible: true, resizable: true });
            }
        },
        /**
         * To configure the split bar in splitter control.
		 * @private
         */
        _insertSplitBar: function () {
            if (this.panes.length > 1) {
                var i, splitBar;
                for (i = 0; i < this.panes.length - 1; i++) {
                    splitBar = this._createSplitBar(i);
                    splitBar.insertAfter(this.panes[i]);
                }
            }
        },
        /**
         * To create the split bar in splitter control.
		 * @private
         */
        _createSplitBar: function (i) {
            var orient = this.model.orientation.substr(0, 1), arrow1, arrow2, splitBar, accessible = false;

            splitBar = ej.buildTag("span.e-box e-splitbar e-" + orient + "-bar").attr("aria-expanded", true);

            if (this.model.properties[i].collapsible && this.model.properties[i + 1].collapsible) {
                arrow1 = ej.buildTag("span.e-icon e-collapse e-" + orient + "-arrow " + orient + "-backward");
                splitBar.append(arrow1);
                arrow2 = ej.buildTag("span.e-icon e-expand e-" + orient + "-arrow " + orient + "-forward");
                splitBar.append(arrow2);
                accessible = true;

                this._on(arrow1, "mousedown", this._collapseArrowClick);
                this._on(arrow2, "mousedown", this._expandArrowClick);
            }
            if (this.model.properties[i].resizable && this.model.properties[i + 1].resizable) {
                splitBar.addClass("e-resize");
                accessible = true;
                this._on(splitBar, ej.eventType.mouseDown, this._mouseDownOnDivider);
            }
            if (accessible) {
                splitBar.attr({ "role": "separator", "tabindex": "0" });
                this._on(splitBar, "focus focusout", this._focusOnDivider);
            }
            else splitBar.attr({ "role": "presentation" });
            return splitBar;
        },
        /**
         * To obtained the pane size in percentage.
		 * @private
         */
        _getPanesPercent: function () {
            this._sizePercent = [];
            var size = this.element[this.containerCss](), outerSize = size - ((this.panes.length - 1) * this._bar), i;

            for (i = 0; i < this.panes.length; i++) {
                if (!$(this.panes[i]).hasClass("collapsed"))
                    this.oldPaneSize[i] = $(this.panes[i])[this.containerCss]();
                this.oldPanePercent[i] = this._convertToPercent(outerSize, this.oldPaneSize[i]);
                this._sizePercent.push(this._convertToPercent(outerSize, $(this.panes[i])[this.containerCss]()));
            }
        },
        /**
         * To set the dimension for splitter control.
		 * @private
         */
        _setDimentions: function () {
            if (this.model.height)
                this.element.css("height", this.model.height);

            if (this.model.width)
                this.element.css("width", this.model.width);
        },
        /**
         * To check the splitter control properties.
		 * @private
         */
        _checkProperties: function () {
            if (this.model.rtl) this._rtl(this.model.rtl);

            this._prevSize = this.element[this.containerCss]();
        },
        /**
         * To enable or disable the Right to Left behaviour 		
		 * @private
         */
        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },
        /**
         * To set the pane size for splitter control. 		
		 * @private
         */
        _setPanesSize: function () {
            var attr = this.containerCss,
            zeroPanes = 0,
            totalPaneSize = 0,
            totalSize = this.element[attr](),
            remainZero = false,
            bar = this._bar,
            zerothPanes = [],
            panLength, j, paneCount = this.panes.length;

            if (paneCount > 1) {
                for (j = 0; j < paneCount ; j++) {
                    $(this.panes[j]).css(attr, this.model.properties[j].paneSize);
                    bar = (j == paneCount - 1) ? 0 : bar;
                    panLength = Math.round($(this.panes[j])[attr]());

                    if (panLength <= 0) {
                        zeroPanes++;
                        zerothPanes.push(j);
                        totalPaneSize += bar;
                    }
                    else {
                        if (remainZero) {
                            $(this.panes[j]).css(attr, 0);
                            totalPaneSize += panLength + bar;
                            this.model.properties[j].paneSize = 0;
                        }
                        else {
                            totalPaneSize += panLength + bar;
                            if (totalPaneSize > totalSize) {
                                totalPaneSize -= panLength - bar;
                                var currPane = totalSize - totalPaneSize,
                                remainDivider = paneCount - j - 1;
                                currPane -= remainDivider * bar;
                                $(this.panes[j]).css(attr, currPane);
                                remainZero = true;
                                totalPaneSize += currPane + bar;
                            }
                            this.model.properties[j].paneSize = panLength;
                        }
                    }
                }
            }
            else if (paneCount == 1) {
                $(this.panes[0]).css(attr, "100%");
                totalPaneSize = totalSize;
            }

            if (paneCount > 1 && totalPaneSize != totalSize) {
                var remainingSize, lastPane = $(this.panes[paneCount - 1]);
                if (totalPaneSize > totalSize) {
                    remainingSize = totalPaneSize - totalSize;
                    lastPane.css(attr, remainingSize);
                }
                else if (totalPaneSize < totalSize) {
                    remainingSize = totalSize - totalPaneSize;
                    if (zeroPanes > 0) {
                        var z, avgWid = Math.round(remainingSize / zeroPanes);
                        for (z = 0; z < zeroPanes ; z++) {
                            $(this.panes[zerothPanes[z]]).css(attr, avgWid);
                            this.model.properties[zerothPanes[z]].paneSize = avgWid;
                        }
                    }
                    else {
                        lastPane.css(attr, Math.round(lastPane[attr]() + remainingSize));
                        this.model.properties[paneCount - 1].paneSize = lastPane[attr]();
                    }
                }
            }
            if (paneCount > 1) this._checkPaneSize();
        },
        /**
         * To return the value based on it's type.		
		 * @private
         */
        _getUnit: function (str) {
            if (str == "px") return "px";
            else if (str == "pt") return "pt";
            else if (str.substr(1) == "%") return "%";
            else return "px";
        },
        /**
         * To configure the splitter position based on it's type.
		 * @private
         */
        _getNormalValue: function (position) {
            var currentLOB, currentLOBPercent, totalValue, currentValue; //currentLOT = current left or bottom
            if (this.model.orientation == "vertical") {
                currentLOB = position.y - this.element.offset().top;
                currentLOBPercent = currentLOB / this.element.outerHeight();
                totalValue = this.element.height();
            }
            else {
                currentLOB = position.x - this.element.offset().left;
                currentLOBPercent = currentLOB / this.element.outerWidth();
                totalValue = this.element.width();
            }
            if (currentLOBPercent > 1) {
                currentLOBPercent = 1;
            }
            if (currentLOBPercent < 0) {
                currentLOBPercent = 0;
            }

            currentValue = currentLOBPercent * totalValue;

            return this._trimValue(currentValue);
        },
        /**
         * To obtained the value based on step value.
		 * @private
         */
        _trimValue: function (value) {
            var step, stepModValue, correctedValue;
            step = 1;
            stepModValue = (value) % step;
            correctedValue = value - stepModValue;
            if (Math.abs(stepModValue) * 2 >= step)
                correctedValue += (stepModValue > 0) ? step : (-step);
            return parseFloat(correctedValue.toFixed(5));
        },
        /**
         * To get the split bar index. 		
		 * @private
         */
        _getSplitbarIndex: function () {
            return this.element.children(".e-splitbar:not(.e-shadowbar)").index(this.currentSplitBar);
        },
        /**
         * To configure the pane resize in splitter control. 		
		 * @private
         */
        _paneResize: function () {
            if (this.shadowBar == null) return false;
            this.currentSplitBar = this.shadowBar.next();
            var newPosition, prevPane, nextPane, prevPaneIndex, nextPaneIndex, index = this._getSplitbarIndex();
            prevPane = this.shadowBar.prev(), nextPane = this.currentSplitBar.next();
            prevPaneIndex = index, nextPaneIndex = index + 1;

            newPosition = this.shadowBar.offset()[this.displayCss];
            newPosition = newPosition - this.currentSplitBar.offset()[this.displayCss];
            $(prevPane).css(this.containerCss, newPosition + $(prevPane)[this.containerCss]() + "px");
            $(nextPane).css(this.containerCss, $(nextPane)[this.containerCss]() - newPosition + "px");
            this.oldPaneSize[prevPaneIndex] = $(prevPane)[this.containerCss]();
            this.oldPaneSize[nextPaneIndex] = $(nextPane)[this.containerCss]();

            this.shadowBar.remove();
            this._checkPaneSize();

            var prevObj = { paneObj: prevPane, paneIndex: prevPaneIndex, paneSize: this.oldPaneSize[prevPaneIndex] };
            var nextObj = { paneObj: nextPane, paneIndex: nextPaneIndex, paneSize: this.oldPaneSize[nextPaneIndex] };

            this._updateModelValue(prevObj, nextObj);
            if (this.model.resize)
                this._trigger("resize", {
                    prevPane: prevObj,
                    nextPane: nextObj,
                    splitbarIndex: index,
                    splitterId: this.element[0].id
                });
        },
        /**
         * To check whether pane size in splitter control. 		
		 * @private
         */
        _checkPaneSize: function () {
            var total = 0, len, i, splitterLen, paneCount = this.panes.length;
            for (i = 0; i < paneCount; i++) {
                len = $(this.panes[i])[this.containerCss]();
                total += len + this._bar;
            }
            total -= this._bar;
            splitterLen = this.element[this.containerCss]();
            if (total != splitterLen) {
                var remain = splitterLen - total;
                var last = $(this.panes[paneCount - 1])[this.containerCss]();
                $(this.panes[paneCount - 1]).css(this.containerCss, Math.floor(last + remain));
            }
        },
        /**
         * To configure the maximum draggable range for pane in splitter control.
		 * @private
         */
        _maxminDraggableRange: function (shadowbarPos) {
            var prevPane, nextPane, prevPaneSize, nextPaneSize, splitbarPosition, prevPaneRange, nextPaneRange,
                prevPaneIndex, nextPaneIndex, PaneMax1, PaneMax2, PaneMin1, PaneMin2, index;
            prevPane = this.shadowBar.prev();
            this.currentSplitBar = this.shadowBar.next();
            nextPane = this.currentSplitBar.next();
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();
            splitbarPosition = this.displayCss == "left" ? this.currentSplitBar[0].offsetLeft : this.currentSplitBar[0].offsetTop;
            prevPaneRange = splitbarPosition - prevPaneSize;
            nextPaneRange = nextPaneSize + splitbarPosition;

            index = this._getSplitbarIndex();
            prevPaneIndex = index;
            nextPaneIndex = index + 1;

            PaneMax1 = this.model.properties[prevPaneIndex].maxSize;
            PaneMax2 = this.model.properties[nextPaneIndex].maxSize;
            PaneMax1 = PaneMax1 != null ? parseInt(PaneMax1, 10) : null;
            PaneMax2 = PaneMax2 != null ? parseInt(PaneMax2, 10) : null;

            this.model.properties[prevPaneIndex].minSize = parseInt(this.model.properties[prevPaneIndex].minSize, 10);
            this.model.properties[nextPaneIndex].minSize = parseInt(this.model.properties[nextPaneIndex].minSize, 10);
            PaneMin1 = this.model.properties[prevPaneIndex].minSize;
            PaneMin2 = this.model.properties[nextPaneIndex].minSize;

            this.shadowBar.removeClass("e-end-indicaton");

            // checking expand limit

            if (shadowbarPos > nextPaneRange - PaneMin2) {
                this.resizedPosition = nextPaneRange - PaneMin2;
                this.shadowBar.addClass("e-end-indicaton");
            }
            else if (shadowbarPos < prevPaneRange + PaneMin1) {
                this.resizedPosition = prevPaneRange + PaneMin1;
                this.shadowBar.addClass("e-end-indicaton");
            }

            if (PaneMax1 != null) {
                if (shadowbarPos > prevPaneRange + PaneMax1) {
                    this.resizedPosition = prevPaneRange + PaneMax1;
                    this.shadowBar.addClass("e-end-indicaton");
                }
            }

            if (PaneMax2 != null) {
                if (shadowbarPos < nextPaneRange - PaneMax2) {
                    this.resizedPosition = nextPaneRange - PaneMax2;
                    this.shadowBar.addClass("e-end-indicaton");
                }
            }
        },
        /**
         * To configure the arrow click in collapse time 	.
		 * @private
         */
        _collapseArrowClick: function (event) {
            if (this.shadowBar != null) return;
            var $target = $(event.target);
            this._inMovement = true;
            this.currentSplitBar = $target.parent();
            var currBarNo, prevPane, nextPane, prevPaneIndex, nextPaneIndex, prevPaneSize, nextPaneSize, properties = {};
            var paneCount = this.panes.length;
            currBarNo = this._getSplitbarIndex();
            prevPane = this.currentSplitBar.prev();
            nextPane = this.currentSplitBar.next();
            prevPaneIndex = currBarNo;
            nextPaneIndex = currBarNo + 1;
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();

            var proxy = this, collapsed, expanded;
            collapsed = { paneObj: prevPane, paneIndex: prevPaneIndex, prevSize: prevPaneSize };
            expanded = { paneObj: nextPane, paneIndex: nextPaneIndex, prevSize: nextPaneSize };

            if (this._raiseEvent("beforeExpandCollapse", collapsed, expanded, currBarNo))
                return false;

            if (!nextPane.hasClass("collapsed")) {
                this.oldPaneSize[prevPaneIndex] = prevPaneSize;

                prevPane.addClass("collapsed");
                nextPane.addClass("expanded");
                this.currentSplitBar.attr("aria-expanded", false);
                $target.parent().removeClass("e-resize");
                $target.css("display", "none");

                if (prevPaneIndex != 0) {
                    var preBar = prevPane.prev();
                    preBar.find(".e-expand").css("display", "none");
                    preBar.removeClass("e-resize");
                }

                properties[this.containerCss] = 0;
                prevPane.animate(properties, this.model.animationSpeed);
                properties[this.containerCss] = prevPaneSize + nextPaneSize;
                nextPane.animate(properties, this.model.animationSpeed, function () {
                    proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                });
            }
            else {
                if (prevPaneSize < this.oldPaneSize[nextPaneIndex]) {
                    $target.addClass("e-end-indicaton");
                    this._inMovement = false;
                    $(document).bind("mouseup", $.proxy(this._mouseUpOnArrow, this));
                    return false;
                }
                else {
                    prevPane.removeClass("expanded");
                    nextPane.removeClass("collapsed");
                    $target.parent().addClass("e-resize");
                    $target.siblings().css("display", "block");

                    if (nextPaneIndex != paneCount - 1) {
                        var nextBar = nextPane.next();
                        nextBar.find(".e-collapse").css("display", "block");
                        if (!nextBar.next().hasClass("collapsed")) {
                            nextBar.addClass("e-resize");
                            nextBar.attr("aria-expanded", true);
                        }
                    }

                    properties[this.containerCss] = this.oldPaneSize[nextPaneIndex];
                    nextPane.animate(properties, this.model.animationSpeed);
                    properties[this.containerCss] = prevPaneSize - this.oldPaneSize[nextPaneIndex];
                    prevPane.animate(properties, this.model.animationSpeed, function () {
                        proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                    });
                }
            }
        },
        /**
         * To configure the arrow click in expand time 	.
		 * @private
         */
        _expandArrowClick: function (event) {
            if (this.shadowBar != null) return;
            var $target = $(event.target);
            this._inMovement = true;
            this.currentSplitBar = $target.parent();
            var currBarNo, prevPane, nextPane, prevPaneIndex, nextPaneIndex, prevPaneSize, nextPaneSize, properties = {};
            var paneCount = this.panes.length;
            currBarNo = this._getSplitbarIndex();
            prevPane = this.currentSplitBar.prev();
            nextPane = this.currentSplitBar.next();
            prevPaneIndex = currBarNo;
            nextPaneIndex = currBarNo + 1;
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();

            var proxy = this, collapsed, expanded;
            collapsed = { paneObj: nextPane, paneIndex: nextPaneIndex, prevSize: nextPaneSize };
            expanded = { paneObj: prevPane, paneIndex: prevPaneIndex, prevSize: prevPaneSize };

            if (this._raiseEvent("beforeExpandCollapse", collapsed, expanded, currBarNo))
                return false;

            if (!prevPane.hasClass("collapsed")) {
                this.oldPaneSize[nextPaneIndex] = nextPaneSize;

                prevPane.addClass("expanded");
                nextPane.addClass("collapsed");
                $target.parent().removeClass("e-resize");
                $target.css("display", "none");

                if (nextPaneIndex != paneCount - 1) {
                    var nextBar = nextPane.next();
                    nextBar.find(".e-collapse").css("display", "none");
                    nextBar.removeClass("e-resize");
                    nextBar.attr("aria-expanded", false);
                }

                properties[this.containerCss] = prevPaneSize + nextPaneSize;
                prevPane.animate(properties, this.model.animationSpeed);
                properties[this.containerCss] = 0;
                nextPane.animate(properties, this.model.animationSpeed, function () {
                    proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                });
            }
            else {
                if (nextPaneSize < this.oldPaneSize[prevPaneIndex]) {
                    $target.addClass("e-end-indicaton");
                    this._inMovement = false;
                    $(document).bind("mouseup", $.proxy(this._mouseUpOnArrow, this));
                    return false;
                }
                else {
                    prevPane.removeClass("collapsed");
                    nextPane.removeClass("expanded");
                    this.currentSplitBar.attr("aria-expanded", true);
                    $target.parent().addClass("e-resize");
                    $target.siblings().css("display", "block");

                    if (prevPaneIndex != 0) {
                        var preBar = prevPane.prev();
                        preBar.find(".e-expand").css("display", "block");
                        if (!preBar.prev().hasClass("collapsed")) preBar.addClass("e-resize");
                    }

                    properties[this.containerCss] = this.oldPaneSize[prevPaneIndex];
                    prevPane.animate(properties, this.model.animationSpeed);
                    properties[this.containerCss] = nextPaneSize - this.oldPaneSize[prevPaneIndex];
                    nextPane.animate(properties, this.model.animationSpeed, function () {
                        proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                    });
                }
            }
        },
        /**
         * To raise the event when it's triggered. 	.
		 * @private
         */
        _raiseEvent: function (evtName, collapsed, expanded, index) {
            if (evtName == "expandCollapse") {
                this._inMovement = false;
                this._updateModelValue(collapsed, expanded);
            }

            if (this.model[evtName]) {
                return this._trigger(evtName, {
                    collapsed: collapsed,
                    expanded: expanded,
                    splitbarIndex: index,
                    splitterId: this.element[0].id
                });
            }
            return false;
        },
        /**
         * To update the model value for splitter control.
		 * @private
         */
        _updateModelValue: function (collapsed, expanded) {
            this.model.properties[collapsed.paneIndex].paneSize = collapsed.paneObj[this.containerCss]();
            this.model.properties[expanded.paneIndex].paneSize = expanded.paneObj[this.containerCss]();

            this._getPanesPercent();
        },
        /**
         * Section For handle the mouse up event over the arrow.
		 * @private
         */
        _mouseUpOnArrow: function (event) {
            this.element.find(".e-end-indicaton").removeClass("e-end-indicaton");
            $(document).unbind("mouseup", $.proxy(this._mouseUpOnArrow, this));
        },
        /**
         * Section For handle the key down event on splitter control pane divider.
		 * @private
         */
        _keydownOnDivider: function (e) {
            var key = e.keyCode;
            if (key == 37 || key == 38 || key == 39 || key == 40) {
                e.preventDefault();
                var oriTarget = $(e.data.target);
                if (e.ctrlKey) {
                    if (this.shadowBar == null) {
                        this.currentSplitBar = oriTarget;
                        var index = this._getSplitbarIndex();
                        if (this.model.orientation == "vertical") {
                            if (e.keyCode == 38) this.collapse(index);       // Up Key
                            else if (e.keyCode == 40) this.expand(index);    // Down Key
                        }
                        else {
                            if (e.keyCode == 37) this.collapse(index);       // Left Key
                            else if (e.keyCode == 39) this.expand(index);    // Right Key
                        }
                    }
                }
                else if (oriTarget.hasClass("e-resize")) {
                    var target = (this.shadowBar != null) ? this.shadowBar : oriTarget;
                    var offset = target.offset(), location = { pageX: offset.left, pageY: offset.top };
                    $.extend(true, e, location);
                    if ((this.model.orientation == "vertical" && (e.keyCode == 38 || e.keyCode == 40)) ||
                        (this.model.orientation == "horizontal" && (e.keyCode == 37 || e.keyCode == 39))) {
                        if (e.keyCode == 38) e.pageY -= 5;          // Up Key
                        else if (e.keyCode == 40) e.pageY += 5;     // Down Key
                        else if (e.keyCode == 37) e.pageX -= 5;     // Left Key
                        else if (e.keyCode == 39) e.pageX += 5;     // Right Key
                        this._mouseMoveOnDivider(e);
                    }
                }
            }
            else if (key == 13) {    // Enter Key
                e.preventDefault();
                this._mouseUpOnDivider();
            }
            else if (key == 27) {    // Esc Key
                e.preventDefault();
                if (this.shadowBar != null) this.shadowBar.remove();
                this.shadowBar = null;
                this._mouseUpOnDivider();
                this.element.children(".e-splitbar.e-hover").focusout();
            }
        },
        /**
         * Section For handle the focused in splitter control pane divider.
		 * @private
         */
        _focusOnDivider: function (e) {
            if (e.type == "focus") {
                if (!$(e.target).hasClass("e-hover")) {
                    $(e.target).addClass("e-hover");
                    $(document).bind("keydown", { target: e.target }, $.proxy(this._keydownOnDivider, this));
                }
            }
            else {
                this.element.children(".e-splitbar.e-hover").removeClass("e-hover");
                this._mouseUpOnDivider();
                $(document).unbind("keydown", $.proxy(this._keydownOnDivider, this));
            }
        },
        /**
         * Section For handle the mouse down over splitter control pane divider.
		 * @private
         */
        _mouseDownOnDivider: function (event) {
            event.preventDefault();
            var $target = $(event.target);
            if ($target.hasClass("e-splitbar") && $target.hasClass("e-resize")) {
                if (!$target.hasClass("e-hover")) $target.focus();
                $(document).bind(ej.eventType.mouseMove, { target: event.target }, $.proxy(this._mouseMoveOnDivider, this));
                $(document).bind(ej.eventType.mouseUp, $.proxy(this._mouseUpOnDivider, this));
                $(document).bind("mouseleave", $.proxy(this._mouseUpOnDivider, this));
            }
            else if ($target.hasClass("e-expand") || $target.hasClass("e-collapse")) {
                $target.parent().focus();
            }
        },
        /**
         * Section For handle the mouse move over splitter control pane divider.
		 * @private
         */
        _mouseMoveOnDivider: function (event) {
            event = event.type == "touchmove" ? event.originalEvent.changedTouches[0] : event;
            var position = { x: event.pageX, y: event.pageY };
            this.resizedPosition = this._getNormalValue(position);

            if (this.shadowBar == null) {
                var target = $(event.data.target);
                this.shadowBar = target.clone().addClass("e-shadowbar").removeClass("e-hover").insertBefore(target);
                this.shadowBar.children().remove();
            }
            this._maxminDraggableRange(this.resizedPosition);
            this.shadowBar.css(this.displayCss, this.resizedPosition);
        },
        /**
         * Section For handle the mouse up over splitter control pane divider.
		 * @private
         */
        _mouseUpOnDivider: function (event) {
            this._paneResize();

            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._mouseMoveOnDivider, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._mouseUpOnDivider, this));
            $(document).unbind("mouseleave", $.proxy(this._mouseUpOnDivider, this));
            // sets shadowBar null after removing shadowBar element
            this.shadowBar = null;
        },
        /**
         * Section For handle the window resized.
		 * @private
         */
        _windowResized: function (event) {
            var size = this.element[this.containerCss]();
            if (this._prevSize == size) return false;

            var paneCount = this.panes.length, outerSize = size - ((paneCount - 1) * this._bar), i, val;
            this._prevSize = size;

            for (i = 0; i < paneCount; i++) {
                val = this._convertToPixel(outerSize, this._sizePercent[i]);
                $(this.panes[i]).css(this.containerCss, val + "px");
            }
            for (i = 0; i < this.oldPaneSize.length; i++)
                this.oldPaneSize[i] = this._convertToPixel(outerSize, this.oldPanePercent[i]);

            this._checkPaneSize();
        },

        _convertToPercent: function (outer, pane) {
            return (pane * 100) / outer;
        },
        _convertToPixel: function (tot, percent) {
            return Math.round((tot * percent) / 100);
        },

        //-------------------- Event Wire-up -------------------------//
        /**
         * Wiring the events to splitter control		
		 * @private
         */
        _wireEvents: function (boolean) {
            if (boolean) $(window).bind('resize', $.proxy(this._windowResized, this));
        },
        /**
         * unWiring the events from splitter control.		
		 * @private
         */
        _unWireEvents: function () {
            $(window).unbind('resize', $.proxy(this._windowResized, this));
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the tab control.
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejTab
	* @requires jQuery
	* @requires ej.web.all
	* @classdesc Custom Design for tab control.
	*@example
	* //HTML template  
	*&lt;div id="tab"&gt;                  
	*&lt;ul&gt;                      
	*&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
	*&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
	*&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
	*&lt;/ul&gt;                  
	*&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
	*It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
	*&lt;/div&gt;                  
	*&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
	*&lt;/div&gt;                  
	*&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
	*&lt;/div&gt;              
	*&lt;/div&gt; 	
	*
	* //Initialization of ejTab in script tag.
	*
	* &lt;script type="text/javascript"&gt;         
	*$(function () {             
	* // document ready            
	* // Initialize Tab control creation             
	* $("#tab").ejTab();         
	*}); &lt;/script&gt; 	
	*/
    ej.widget("ejTab", "ej.Tab", {
        _rootCSS: "e-tab",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,

        // default model
        defaults: {
            /// <summary>This Contains default property of Sycnfusion Tab control </summary>
            /**		
           * Allow to collapsing the active item, while click on the active header.
           * @default false
           * @type {boolean}
           * @example 
           * // Set the collapsible during initialization. 			
           * 	$("#tab").ejTab({collapsible: false }); 
           * @example 
           * //Get or set the collapsible after initialization:<br>
           *	//Get the collapsible API value.		
           *	$("#tab").ejTab("option", "collapsible");			
           *	//Set the collapsible API value
           *	$("#tab").ejTab("option", "collapsible", true );
            * @memberof ejTab
           * @instance
           */
            collapsible: false,
            /**		
           * Disables the given tab headers and content panels.
           * @default null
           * @type {IntegerArray}
           * @example 
           * // Set the ajaxOptions during initialization. 			
           * 	$("#tab").ejTab({ ajaxOptions: {
            *       dataType: "html",
            *       contentType: "html",
            *      async: true
            *  }
           *	}); 
           * @example 
           * //Get or set the ajaxOptions after initialization:<br>
           *	//Get the ajaxOptions API value.		
           *	$("#tab").ejTab("option", "ajaxOptions");			
           *	//Set the ajaxOptions API value
           *	$("#tab").ejTab("option", "ajaxOptions",{
            *   dataType: "html",
            *   contentType: "html",
            *  async: true
           *}
           *);
            * @memberof ejTab
           * @instance
           */
            ajaxOptions: { type: 'GET', cache: false, data: {}, dataType: "html", contentType: "html", async: true },
            /**		
           * Disables the given tab headers and content panels.
           * @default []
           * @type {IntegerArray}
           * @example 
           * // Set the disabledItems during initialization. 			
           * 	$("#tab").ejTab({disabledItems: [0,1] }); 
           * @example 
           * //Get or set the disabledItems after initialization:<br>
           *	//Get the disabledItems API value.		
           *	$("#tab").ejTab("option", "disabledItems");			
           *	//Set the disabledItems API value
           *	$("#tab").ejTab("option", "disabledItems",[1,2] );
            * @memberof ejTab
           * @instance
           */
            disabledItems: [],
            /**		
           * The event API to bind the action for active the tab items.
           * @default "click"
           * @type {string}
           * @example 
           * // Set the events during initialization. 			
           * 	$("#tab").ejTab({events: "click" }); 
           * @example 
           * //Get or set the events after initialization:<br>
           *	//Get the events API value.		
           *	$("#tab").ejTab("option", "events");			
           *	//Set the events API value
           *	$("#tab").ejTab("option", "events", "mouseover" );
            * @memberof ejTab
           * @instance
           */
            events: "click",
            /**		
           * The idPrefix property appends the give string on runtime added tab item id’s.
           * @default "ej-tab-"
           * @type {string}
           * @example 
           * // Set the idPrefix during initialization. 			
           * 	$("#tab").ejTab({idPrefix: "ej-tab-" }); 
           * @example 
           * //Get or set the idPrefix after initialization:<br>
           *	//Get the idPrefix API value.		
           *	$("#tab").ejTab("option", "idPrefix");			
           *	//Set the idPrefix API value
           *	$("#tab").ejTab("option", "idPrefix", "ej-tab-item" );
            * @memberof ejTab
           * @instance
           */
            idPrefix: "ej-tab-",
            /**		
           * Adjust the content panel height for given option (content, auto and fill), by default panels height adjust based on the content
           * @default "content"
           * @type {string | enum}
           * @example 
           * // Set the heightStyle during initialization. 			
           * 	$("#tab").ejTab({heightStyle: ej.tab.height.content }); 
           * @example 
           * //Get or set the heightStyle after initialization:<br>
           *	//Get the heightStyle API value.		
           *	$("#tab").ejTab("option", "heightStyle");			
           *	//Set the heightStyle API value
           *	$("#tab").ejTab("option", "heightStyle", ej.tab.height.fill );
            * @memberof ejTab
           * @instance
           */
            heightStyle: "content", //ej.tab.height.content,
            /**		
			* Tab header to active for given index value. 
			* @default 0
			* @type {number}
			* @example 
			* // Set the selectedItemIndex during initialization. 			
			* 	$("#tab").ejTab({selectedItemIndex: 0 }); 
			* @example 
			* //Get or set the selectedItemIndex after initialization:<br>
			*	//Get the selectedItemIndex API value.		
			*	$("#tab").ejTab("option", "selectedItemIndex");			
			*	//Set the selectedItemIndex API value
			*	$("#tab").ejTab("option", "selectedItemIndex", 1 );
			 * @memberof ejTab
			* @instance
			*/
            selectedItemIndex: 0,
            /**		
           * Set the root class for Tab theme. This cssClass API helps to use custom skinning option for Tab control. 
           * @default ""
           * @type {string}
           * @example 
           * // Set the CSS class during initialization. 			
           * 	$("#tab").ejTab({cssClass: "gradient-lime" }); 
           * @example 
           * //Get or set the cssClass after initialization:<br>
           *	//Get the cssClass API value.		
           *	$("#tab").ejTab("option", "cssClass");			
           *	//Set the cssClass API value
           *	$("#tab").ejTab("option", "cssClass",  "gradient-lime" );
            * @memberof ejTab
           * @instance
           */
            cssClass: "",
            /**		
           * Display the close button for each tab items. While click on the close icon particular tab item removed. 
           * @default false
           * @type {boolean}
           * @example 
           * // Set the showCloseButton during initialization. 			
           * 	$("#tab").ejTab({showCloseButton: false }); 
           * @example 
           * //Get or set the showCloseButton after initialization:<br>
           *	//Get the showCloseButton API value.		
           *	$("#tab").ejTab("option", "showCloseButton");			
           *	//Set the showCloseButton API value
           *	$("#tab").ejTab("option", "showCloseButton", true );
            * @memberof ejTab
           * @instance
           */
            showCloseButton: false,
            /**		
           * Display the Reload button for each tab items. 
           * @default false
           * @type {boolean}
           * @example 
           * // Set the showReloadButton during initialization. 			
           * 	$("#tab").ejTab({showReloadButton: false }); 
           * @example 
           * //Get or set the showReloadButton after initialization:<br>
           *	//Get the showReloadButton API value.		
           *	$("#tab").ejTab("option", "showReloadButton");			
           *	//Set the showReloadButton API value
           *	$("#tab").ejTab("option", "showReloadButton", true );
            * @memberof ejTab
           * @instance
           */
            showReloadButton: false,
            /**		
        * Tab header display top,bottom,left or right  
        * @default "top"
        * @type {string | enum}
        * @example 
        * // Set the headerPosition during initialization. 			
        * 	$("#tab").ejTab({headerPosition: "top" }); 
        * @example 
        * //Get or set the headerPosition after initialization:<br>
        *	//Get the headerPosition API value.		
        *	$("#tab").ejTab("option", "headerPosition");			
        *	//Set the headerPosition API value
        *	$("#tab").ejTab("option", "headerPosition", "bottom" );
         * @memberof ejTab
        * @instance
        */
            headerPosition: "top",
            /**		
        * Set the width for outer panel element, if not it’s take parent width. 
        * @default null
        * @type { string | number}
        * @example 
        * // Set the width during initialization. 			
        * 	$("#tab").ejTab({width: "600px" }); 
        * @example 
        * //Get or set the width after initialization:<br>
        *	//Get the width API value.		
        *	$("#tab").ejTab("option", "width");			
        *	//Set the width API value
        *	$("#tab").ejTab("option", "width","400px" );
         * @memberof ejTab
        * @instance
        */
            width: null,
            /**		
        * Height set the outer panel element. Default this property value is null, so height take content height.
        * @default null
        * @type {string | number}
        * @example 
        * // Set the height during initialization. 			
        * 	$("#tab").ejTab({height: "600px" }); 
        * @example 
        * //Get or set the height after initialization:<br>
        *	//Get the height API value.		
        *	$("#tab").ejTab("option", "height");			
        *	//Set the height API value
        *	$("#tab").ejTab("option", "height", "400px" );
         * @memberof ejTab
        * @instance
        */
            height: null,
            /**		
           * Display Right to Left direction for headers and panels text.
           * @default false
           * @type {boolean}
           * @example 
           * // Set the rtl during initialization. 			
           * 	$("#tab").ejTab({rtl: false }); 
           * @example 
           * //Get or set the rtl after initialization:<br>
           *	//Get the rtl API value.		
           *	$("#tab").ejTab("option", "rtl");			
           *	//Set the rtl API value
           *	$("#tab").ejTab("option", "rtl", true );
            * @memberof ejTab
           * @instance
           */
            rtl: false,
            /**		
           * Tab items interaction with keyboard keys, like headers active navigation.
           * @default true
           * @type {boolean}
           * @example 
           * // Set the allowKeyboardNavigation during initialization. 			
           * 	$("#tab").ejTab({allowKeyboardNavigation: false }); 
           * @example 
           * //Get or set the allowKeyboardNavigation after initialization:<br>
           *	//Get the allowKeyboardNavigation API value.		
           *	$("#tab").ejTab("option", "allowKeyboardNavigation");			
           *	//Set the allowKeyboardNavigation API value
           *	$("#tab").ejTab("option", "allowKeyboardNavigation", true );
            * @memberof ejTab
           * @instance
           */
            allowKeyboardNavigation: true,
            /**		
           * Tab panels and headers to display the rounded corner style.
           * @default false
           * @type {boolean}
           * @example 
           * // Set the roundedCorner during initialization. 			
           * 	$("#tab").ejTab({roundedCorner: false }); 
           * @example 
           * //Get or set the roundedCorner after initialization:<br>
           *	//Get the roundedCorner API value.		
           *	$("#tab").ejTab("option", "roundedCorner");			
           *	//Set the roundedCorner API value
           *	$("#tab").ejTab("option", "roundedCorner", true );
            * @memberof ejTab
           * @instance
           */
            roundedCorner: false,
            /**		
           * Save current model value to browser cookies for state maintains. While refresh the Tab control page the model value apply from browser cookies.  
           * @default false
           * @type {boolean}
           * @example 
           * // Set the persist during initialization. 			
           * 	$("#tab").ejTab({persist: false }); 
           * @example 
           * //Get or set the persist after initialization:<br>
           *	//Get the persist API value.		
           *	$("#tab").ejTab("option", "persist");			
           *	//Set the persist API value
           *	$("#tab").ejTab("option", "persist", true );
            * @memberof ejTab
           * @instance
           */
            persist: false,
            /**		
           * When this property is set to false, it disables the tab control.
           * @default true
           * @type {boolean}
           * @example 
           * // Set the enabled during initialization. 			
           * 	$("#tab").ejTab({enabled: false }); 
           * @example 
           * //Get or set the enabled after initialization:<br>
           *	//Get the enabled API value.		
           *	$("#tab").ejTab("option", "enabled");			
           *	//Set the enabled API value
           *	$("#tab").ejTab("option", "enabled", true );
            * @memberof ejTab
           * @instance
           */
            enabled: true,
            //Events
            /**     
			 * Triggered after ajax content load action.
			 * @event
			 * @name ejTab#ajaxLoad 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.oldTab returns previous active tab header.
			 * @param {object}  argument.oldContentPanel returns previous active tab panel.
			 * @param {number}  argument.oldSelectedIndex returns previous active index.
			 * @param {object}  argument.newTab returns current active tab header .
			 * @param {object}  argument.newContentPanel returns current active tab panel.
			 * @param {number}  argument.newSelectedIndex returns current active index.
			 * @example 
             * $("#tab").ejTab({
             *    ajaxLoad: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            ajaxLoad: null,
            /**     
			 * Triggered before ajax content load action.
			 * @event
			 * @name ejTab#ajaxBeforeLoad 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
		     * @param {object}  argument.oldTab returns previous active tab header.
			 * @param {object}  argument.oldContentPanel returns previous active tab panel.
			 * @param {number}  argument.oldSelectedIndex returns previous active index.
			 * @param {object}  argument.newTab returns current active tab header .
			 * @param {object}  argument.newContentPanel returns current active tab panel.
			 * @param {number}  argument.newSelectedIndex returns current active index.
			 * @example 
             * $("#tab").ejTab({
             *    ajaxBeforeLoad: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            ajaxBeforeLoad: null,
            /**     
			 * Triggered after a tab item activated.
			 * @event
			 * @name ejTab#active 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
		     * @param {object}  argument.oldTab returns previous active tab header.
			 * @param {object}  argument.oldContentPanel returns previous active tab panel.
			 * @param {number}  argument.oldSelectedIndex returns previous active index.
			 * @param {object}  argument.newTab returns current active tab header .
			 * @param {object}  argument.newContentPanel returns current active tab panel.
			 * @param {number}  argument.newSelectedIndex returns current active index.
			 * @example 
             * $("#tab").ejTab({
             *    active: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            active: null,
            /**     
			 * Triggered before a tab item activated.
			 * @event
			 * @name ejTab#beforeActive 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.oldTab returns previous active tab header.
			 * @param {object}  argument.oldContentPanel returns previous active tab panel.
			 * @param {number}  argument.oldSelectedIndex returns previous active index.
			 * @param {object}  argument.newTab returns current active tab header .
			 * @param {object}  argument.newContentPanel returns current active tab panel.
			 * @param {number}  argument.newSelectedIndex returns current active index.
			 * @example 
             * $("#tab").ejTab({
             *    beforeActive: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            beforeActive: null,
            /**     
			 * Triggered after new tab item add
			 * @event
			 * @name ejTab#itemAdd 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.addTab returns new added tab header.
			 * @param {object}  argument.addContentPanel returns new added tab content panel.
			 * @example 
             * $("#tab").ejTab({
             *    itemAdd: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            itemAdd: null,
            /**     
			 * Triggered after tab item removed.
			 * @event
			 * @name ejTab#itemRemove 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.removedTab returns removed tab header.
			 * @param {object}  argument.removedPanel returns removed tab content panel.
			 * @example 
             * $("#tab").ejTab({
             *    itemRemove: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            itemRemove: null,
            /**     
			 * Triggered before a tab item remove.
			 * @event
			 * @name ejTab#beforeItemRemove 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {number}  argument.deleteIndex returns current tab item index			
			 * @example 
             * $("#tab").ejTab({
             *    beforeItemRemove: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            beforeItemRemove: null,
            /**     
			 * Triggered after a tab item enabled.
			 * @event
			 * @name ejTab#itemEnable 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
             * $("#tab").ejTab({
             *    itemEnable: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            itemEnable: null,
            /**     
			 * Triggered after a tab item disabled.
			 * @event
			 * @name ejTab#itemDisable 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
             * $("#tab").ejTab({
             *    itemDisable: function (args) {}
             * });      
			 * @memberof ejTab
			 * @instance
			 */
            itemDisable: null,
        },
        dataTypes: {
            cssClass: "string",
            collapsible: "boolean",
            events: "string",
            heightStyle: "enum",
            selectedItemIndex: "number",
            enabled: "boolean",
            ajaxOptions: "data",
            disabledItems: "data"
        },
        /**
        * destroy the tab widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @private
        */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            /// <summary>This function is  used to destroy the Tab Object</summary>
            this._unWireEvents();
            this._removeBaseClass();
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            /// <summary>This function is used to set the Tab model</summary>
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "events": {
                        this._off(this.anchors, this.model.events);
                        this._on(this.anchors, options[key], this._tabItemClick);
                        break;
                    }
                    case "disabledItems": this.model.disabledItems = options[key]; this._disableTabs(); break;
                    case "enabled": this._enabledAction(options[key]); break;
                    case "selectedItemIndex": this.showItem(options[key]); break;
                    case "heightStyle": this._setTabsHeightStyle(options[key]); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "roundedCorner": this._roundedCorner(options[key]); break;
                    case "height": this.element.height(options[key]); break;
                    case "width": this.element.width(options[key]); break;
                    case "allowKeyboardNavigation": {
                        if (options[key])
                            this._on(this.element, 'keydown', this._keyPress);
                        else
                            this._off(this.element, "keydown");
                        break;
                    }
                    case "headerPosition":
                        {
                            this.model.headerPosition = options[key];
                            if (this.model.headerPosition == ej.Tab.Position.Top) {
                                this._removeVerticalClass();
                                this.itemsContainer.remove();
                                this.itemsContainer.insertBefore(this.element.find(">div").first());
                                this.element.find("div.e-active-content").removeClass("e-activebottom");
                            }
                            else if (this.model.headerPosition == ej.Tab.Position.Bottom) {
                                this._removeVerticalClass();
                                this.element.find("div.e-active-content").removeClass("e-activetop");
                            }
                            else if (this.model.headerPosition == ej.Tab.Position.Left || this.model.headerPosition == ej.Tab.Position.Right) {
                                this._removeHeaderClass();
                            }
                            this._refresh();
                            break;
                        }
                    case "showCloseButton":
                        {
                            if (options[key]) {
                                this._addDeleteIcon();
                                this._on(this.element.find("div.e-delete"), "click", this._tabDeleteClick);
                            } else
                                this.element.find("div.e-delete").remove();
                            break;
                        }
                    case "showReloadButton":
                        {
                            if (options[key]) {
                                this._addReloadIcon();
                            } else
                                this.element.find("div.e-reload").remove();
                            break;
                        }
                    case "rtl":
                        {
                            if (options[key])
                                this.element.addClass("e-rtl");
                            else
                                this.element.removeClass("e-rtl");
                            break;
                        }
                }
            }
        },
        /**
        * Create the tab widget
        * @private
        */
        // constructor function
        _init: function () {
            this.tabId = 0;
            this.initialize();
        },
        /**
        * To configure the custom theme for tab using cssClass property		
        * @private
        */
        //Skin Change at run time
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },

        /**
         * To initialize the tab		
		 * @private
         */
        // Render the tab control
        initialize: function () {
            this.element.attr("tabindex", 0).attr("role", "tablist");
            this.itemsContainer = this.element.find("ol,ul").eq(0);
            this.items = this.itemsContainer.find(" > li:has(a[href])");
            this.anchors = this.items.find('a[href]');
            $(this.anchors).addClass("e-link");
            this._preTabIndex = -1;
            if (!ej.isNullOrUndefined(this.model.width))
                this.element.width(this.model.width);
            if (!ej.isNullOrUndefined(this.model.height))
                this.element.height(this.model.height);
            this._setTabPosition(this.model.headerPosition);
            if (this.model.showCloseButton)
                this._addDeleteIcon();
            if (this.model.showReloadButton)
                this._addReloadIcon();
            this._wireEvents(this.model.events);
            this.contentPanels = [];
            for (var i = 0; i < this.anchors.length; i++) {
                var hid = this.anchors[i];
                var href = $(hid).attr("href");
                var hrefBase = href.split("#")[0];
                if (hrefBase && (hrefBase === location.toString().split("#")[0])) {
                    href = a.hash;
                    hid.href = href;
                }
                if (href && href !== "#") {
                    this._addContentTag(href, i);
                } else {
                    this.model.disabledItems.push(i);
                }
            }
            this._addBaseClass();
            if (!(this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                this._setTabsHeightStyle(this.model.heightStyle);
            this._disableTabs();
            this._roundedCorner(this.model.roundedCorner);
            // Added for rendering the Windows tab...
            this.showItem(this.model.selectedItemIndex);
            this._enabledAction(this.model.enabled);
        },
        /**
        * section for enable / disable the action
        * @private
        */
        _enabledAction: function (flag) {
            if (flag) {
                this.element.removeClass("e-disable");
            }
            else {
                this.element.addClass("e-disable");
            }
        },
        /**
        * To configure Tab position
        * @private
        */
        //Set the Tab position
        _setTabPosition: function (position) {
            if (position == ej.Tab.Position.Bottom) {
                this.itemsContainer.remove();
                this.element.append(this.itemsContainer);
                this.items.removeClass("e-bottom-line");
                this.items.addClass("e-top-line");
            }
            else if (position == ej.Tab.Position.Top) {
                this.items.removeClass("e-top-line");
                this.items.addClass("e-bottom-line");
            }
            else if (position == ej.Tab.Position.Left || position == ej.Tab.Position.Right) {
                if (this.items.length > 0) {
                    if (this.model.height)
                        this.itemsContainer.css("height", this.model.height);
                    else
                        this.itemsContainer.css("height", "100vh");
                    this.element.addClass("e-vertical");
                }
            }
        },
        /**
         * Section for Add delete icon for each tab
		 * @private
         */
        //Add delete icon for each tab
        _addDeleteIcon: function () {
            if (this.element.find("div.e-delete").length <= 0 && this.items.length > 0) {
                var deleteIcon = ej.buildTag('div.e-icon e-delete', "", {}, { role: "presentation" }).css("visibility", "hidden");
                if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                    var delIconPosition = this.items.find("a");
                    deleteIcon.insertBefore(delIconPosition);
                }
                else
                    this.items.append(deleteIcon);
            }
        },
        /**
         * Section for Add reload icon for each tab
		 * @private
         */
        //Add reload icon for each tab
        _addReloadIcon: function () {
            if (this.element.find("div.e-reload").length <= 0 && this.items.length > 0) {
                var reloadIcon = ej.buildTag('div.e-icon e-reload', "", {}, { role: "presentation" }).css("visibility", "hidden");
                if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                    var iconPosition = this.items.find("a");
                    reloadIcon.insertBefore(iconPosition);
                }
                else
                    this.items.append(reloadIcon);
            }
        },
        /**
         * Section for Add base class for headers and content panels
		 * @private
         */
        //Add base class for headers and content panels
        _addBaseClass: function () {
            this.element.addClass("e-widget " + this.model.cssClass);
            if (this.model.rtl)
                this.element.addClass("e-rtl");
            if (this.model.headerPosition == "left") {
                this.items.length > 0 && this.itemsContainer.addClass("e-left");
            }
            else if (this.model.headerPosition == "right") {
                this.items.length > 0 && this.itemsContainer.addClass("e-right");
            }
            else
                this.items.length > 0 && this.itemsContainer.addClass("e-header");
            this.items.addClass("e-select e-item").attr("role", "tab").attr("tabindex", -1).attr("aria-expanded", true).attr("aria-selected", false);
            $(this.contentPanels).addClass("e-content  e-content-item").attr("role", "tabpanel").attr("aria-hidden", true);
        },
        /**
         * Section for remove class for headers 
		 * @private
         */
        _removeHeaderClass: function () {
            this.itemsContainer.remove();
            this.itemsContainer.insertBefore(this.element.find(">div").first());
            this.items.removeClass("e-bottom-line e-top-line");
            $(this.contentPanels).removeClass("e-content-bottom e-corner-bottom e-activetop e-activebottom");
            this.itemsContainer.removeClass("e-header e-left e-right");
        },
        /**
         * Section for remove vertical class from Tab
		 * @private
         */
        _removeVerticalClass: function () {
            this.element.removeClass("e-vertical");
            this.itemsContainer.removeClass("e-left e-right").removeAttr("style");
        },
        /**
         * Section for remove base class for headers and content panels
		 * @private
         */
        //Remove base class for headers and content panels
        _removeBaseClass: function () {
            this.element.removeClass("e-tab e-widget");
            if ((this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                this._removeVerticalClass();
            this.itemsContainer.removeClass("e-header e-clearall e-select");
            this.anchors.removeClass("e-link");
            this.items.removeClass("e-select e-item e-active e-bottom-line e-top-line");
            $(this.contentPanels).removeClass("e-content  e-content-item");
            $(this.contentPanels).removeClass("e-content  e-content-item e-content-bottom e-corner-bottom e-activetop e-activebottom e-active-content");
            $(this.contentPanels).css("display", "");
        },
        /**
         * Section for Create content tag for ajax load and new tab add function
		 * @private
         */
        // Create content tag for ajax load and new tab add function
        _addContentTag: function (href, index) {
            var id = this._getTabId(href);
            var panel = this.element.find("#" + id);
            if (!panel.length) {
                panel = ej.buildTag("div.e-content  e-content-item e-content-bottom #" + id)
                    .insertAfter(this.contentPanels[index - 1] || this.itemsContainer);
            }
            this.contentPanels = $(this.contentPanels).add(panel);
        },
        /**
         * Section for enable/disable roundedCorner behaviour
		 * @private
         */
        _roundedCorner: function (value) {
            if (value) {
                this.element.addClass('e-corner-all');
                this.items.addClass('e-corner-top');
                $(this.contentPanels).addClass('e-corner-bottom');
            }
            else if (this.element.hasClass('e-corner-all')) {
                this.element.removeClass('e-corner-all');
                this.items.removeClass('e-corner-top');
                $(this.contentPanels).removeClass('e-corner-bottom');
            }
        },
        /**
         * To configure tab content panels height
		 * @private
         */
        // Set the tab content panels height
        _setTabsHeightStyle: function (heightStyle) {
            $(this.contentPanels).height("");
            if (ej.Tab.heightStyle.Fill == heightStyle) {
                var maxHeight = this._getDimension($(this.element).parent(), "height");
                $(this.element).parent().css({ "overflow": "auto" });
                for (var i = 0; i < this.items.length; i++) {
                    maxHeight -= this._getDimension($(this.items[i]), "outerHeight");
                }
                var maxPadding = 0;
                for (var i = 0; i < this.contentPanels.length; i++) {
                    maxPadding = Math.max(maxPadding, this._getDimension($(this.contentPanels[i]), "innerHeight") - this._getDimension($(this.contentPanels[i]), "height"));
                }
                $(this.contentPanels).height(maxHeight - maxPadding).css({ "overflow": "auto" });
            }
            else if (ej.Tab.heightStyle.Auto == heightStyle) {
                maxHeight = 0;
                for (var i = 0; i < this.contentPanels.length; i++) {
                    maxHeight = Math.max(maxHeight, this._getDimension($(this.contentPanels[i]), "outerHeight"));
                }
                $(this.contentPanels).height(maxHeight);
                this.maxAutoHeight = maxHeight;
            }
        },
        _getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            var prop = { visibility: 'hidden', display: 'block' };
            var hiddenCollection = [];
            $hidden.each(function () {
                var hidden = {}, name;
                for (name in prop) {
                    hidden[name] = this.style[name];
                    this.style[name] = prop[name];
                }
                hiddenCollection.push(hidden);
            });
            value = /(outer)/g.test(method) ?
            $(element)[method](true) :
           $(element)[method]();

            $hidden.each(function (i) {
                var hidden = hiddenCollection[i], name;
                for (name in prop) {
                    this.style[name] = hidden[name];
                }
            });
            return value;
        },
        // Tab to active given index value
        showItem: function (index) {
            if ($.inArray(index, this.model.disabledItems) < 0) {
                var proxy = this;
                this._preTabIndex = this.model.selectedItemIndex;
                this.model.selectedItemIndex = index;
                if (this.model.selectedItemIndex >= this.contentPanels.length) {
                    this.model.selectedItemIndex = 0;
                }
                $(this.items[this.model.selectedItemIndex]).attr("aria-expanded", true).attr("aria-selected", true).attr("tabindex", 0);
                if (this.model.selectedItemIndex != null && this.model.selectedItemIndex < this.contentPanels.length) {
                    this._ajaxLoad();
                    this.hideItem(this._preTabIndex);
                    if (index >= 0)
                        if (this._onBeforeActive(index))
                            return false;
                    $(this.contentPanels[this.model.selectedItemIndex]).fadeIn(200, "easeOutQuad", function () {
                        if (proxy._onActive())
                            return true;
                    });
                    if (!(this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                        var activeClass = this.model.headerPosition == ej.Tab.Position.Top ? "e-activetop" : "e-activebottom";
                    else
                        $(this.contentPanels[this.model.selectedItemIndex]).addClass("e-active-content ");
                    $(this.items[this.model.selectedItemIndex]).addClass("e-active").removeClass("e-select");
                    $(this.contentPanels[this.model.selectedItemIndex]).addClass("e-active-content " + activeClass).removeAttr("aria-hidden", false);
                    //$(this.items[this._preTabIndex]).attr("aria-expanded", false).attr("tabindex", -1);
                }
            }
        },
        // Tab to hide given index value
        hideItem: function (index) {
            $(this.contentPanels[index]).fadeOut(0);
            if (!(this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                var activeClass = this.model.headerPosition == ej.Tab.Position.Top ? "e-activetop" : "e-activebottom";
            $(this.items[index]).removeClass("e-active").addClass("e-select");
            $(this.contentPanels[index]).removeClass("e-active-content " + activeClass).attr("aria-hidden", true);
        },
        /**
         * To configure Ajax load
		 * @private
         */
        //Ajax load function
        _ajaxLoad: function () {
            var content = $(this.contentPanels[this.model.selectedItemIndex]);
            var link = this.anchors[this.model.selectedItemIndex];
            var href = $(link).attr("href");
            if (content.is(':empty') && href.indexOf("#") !== 0)
                this._sendAjaxOptions(content, link);
        },
        /**
         * Section for get tab id 
		 * @private
         */
        //get tab id for new added tabs
        _getTabId: function (href) {
            return !href.indexOf("#") ? href.replace("#", "") : this.model.idPrefix + this._getNextTabId();
        },
        /**
         * Section for get Next tab id 
		 * @private
         */
        _getNextTabId: function () {
            return ++this.tabId;
        },
        /**
         * Section for Disable tab 
		 * @private
         */
        // Disable tabs in given index
        _disableTabs: function () {
            for (var i = 0, li; (li = this.items[i]) ; i++) {
                if ($.inArray(i, this.model.disabledItems) > -1) {
                    $(li).find("a").unbind(this.model.events);
                    this._off($(li).find("div.e-delete"), "click");
                    $(this.contentPanels[i]).addClass("e-disable");
                }
                $(li)[$.inArray(i, this.model.disabledItems) != -1 &&
                    !$(li).hasClass("e-tab-selected") ? "addClass" : "removeClass"]("e-disable");
            }
        },
        /**
         * Section for handle click event
		 * @private
         */
        // Tabs event handler
        _tabItemClick: function (args) {
            if (this.model.enabled) {
                args.preventDefault(); // Prevent the ancher tag url action
                var index;
                if (this.model.selectedItemIndex == $(this.items).index($(args.currentTarget)) && this.model.collapsible) {
                    index = -1;
                }
                else {
                    index = $(this.items).index($(args.currentTarget));
                }
                if (index != this.model.selectedItemIndex)
                    this.showItem(index);
            }
        },
        /**
         * Section for handle click event on tab delete icon
		 * @private
         */
        _tabDeleteClick: function (args) {
            if (this.model.enabled) {
                var currentTab = $(args.target);
                if (currentTab.hasClass("e-delete"))
                    var index = $(this.items).index($(args.target).parent());
                if (index == this.model.selectedItemIndex && this.items.length > index)
                    this.model.selectedItemIndex++;
                this.remove(index);
            }
        },
        /**
        * Section for handle click event
        * @private
        */
        _tabReloadClick: function (args) {
            if (this.model.enabled) {
                var currentTab = $(args.target);
                if (currentTab.hasClass("e-reload")) {
                    var link = this.anchors[this.model.selectedItemIndex];
                    var href = $(link).attr("href");
                    var content = $(this.contentPanels[this.model.selectedItemIndex]);
                    if (href.indexOf("#") !== 0)
                        this._sendAjaxOptions(content, link);
                    else
                        this.showItem(this.model.selectedItemIndex);
                }
            }
        },
        /**
        * Section for handle ajax request
        * @private
        */
        //handling the ajax request
        _sendAjaxOptions: function (content, link) {
            //load waiting popup
            if (this._onBeforeLoad())
                return true;
            content.addClass("e-load");
            var proxy = this;
            var curTabTitle = $(link).html();
            var hrefLink = link.href.replace("#", "");
            var ajaxOptions = {
                type: this.model.ajaxOptions.type, cache: this.model.ajaxOptions.cache, url: hrefLink, data: this.model.ajaxOptions.data,
                dataType: this.model.ajaxOptions.dataType, contentType: this.model.ajaxOptions.contentType, async: this.model.ajaxOptions.async,
                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, content, link, curTabTitle);
                    } catch (e) {

                    }
                }, "error": function () {
                    try {
                        proxy._ajaxErrorHandler(link, proxy.model.selectedItemIndex, curTabTitle);
                    } catch (e) {

                    }
                }
            };
            //$.extend(true, {}, proto.prototype.defaults, options);
            //this.model.ajaxOptions = $.extend(true, _ajaxOptions, this.model.ajaxOptions);
            this._sendAjaxRequest(ajaxOptions);
        },
        // Ajax Request function
        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },
        /**
        * Section for handle ajax success
        * @private
        */
        //Ajax sucess function
        _ajaxSuccessHandler: function (data, content, link, curTabTitle) {
            if (curTabTitle != null)
                $(link).html(curTabTitle);
            content.removeClass("e-load");
            content.html(data).addClass("e-tab-loaded"); //to indicate the content is already loaded
            if (this._onLoad())
                return true;
        },
        /**
        * Section for handle ajax error
        * @private
        */
        // ajax error function
        _ajaxErrorHandler: function (data, link, index, title) {
            this._onLoad();
        },
        /**
         * Section for create content panel
		 * @private
         */
        _createContentPanel: function (id) {
            return $('<div></div>')
				.attr("id", id)
				.addClass("e-content  e-content-item e-content-bottom");
        },
        /**
         * Section for Refresh conrol.
		 * @private
         */
        // Refresh conrol
        _refresh: function () {
            this._unWireEvents();
            this.initialize();
        },
        /**
         * Section to watch keypress action.		
		 * @private
         */
        _keyPress: function (e) {
            if (this.model.enabled) {
                var index, currentEle, targetEle = $(e.target);
                if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
                else if (e.which) code = e.which; // ns4 and opera
                else code = e.charCode;
                if (targetEle.hasClass("e-link") || targetEle.hasClass("e-item")) {
                    switch (code) {
                        //Right arrow
                        case 39:
                        case 38:
                            {
                                e.preventDefault();
                                this.showItem(this.model.selectedItemIndex + 1);
                                break;
                            }
                            //Left arrow
                        case 37:
                        case 40:
                            {
                                e.preventDefault();
                                if (this.model.selectedItemIndex != 0)
                                    this.showItem(this.model.selectedItemIndex - 1);
                                else if (this.model.selectedItemIndex == 0)
                                    this.showItem(this.tabCount() - 1);
                                break;
                            }
                            //End key
                        case 35:
                            {
                                e.preventDefault();
                                this.showItem(this.tabCount() - 1);
                                break;
                            }
                            //Home key
                        case 36:
                            {
                                e.preventDefault();
                                this.showItem(0);
                                break;
                            }
                    }
                }
                else if (e.ctrlKey && !targetEle.hasClass("e-tab")) {
                    switch (code) {
                        case 38:
                            e.preventDefault();
                            index = $(this.contentPanels).index(targetEle.parent(".e-content"));
                            currentEle = $(this.items[index]);
                            break;
                        case 33:
                            e.preventDefault();
                            currentEle = $(this.items[0]);
                            this.showItem(0);
                            break;
                        case 34:
                            e.preventDefault();
                            currentEle = $(this._headers[this.tabCount() - 1]);
                            this.showItem(this.tabCount() - 1);
                            break;
                    }
                }
                if (!ej.isNullOrUndefined(currentEle)) {
                    currentEle.addClass("e-focus");
                    currentEle.focus();
                }
            }
        },
        /**
         * Section to watch hover action on tab.		
		 * @private
         */
        _hoverHandler: function (args) {
            args.preventDefault();
            if (this.model.enabled) {
                var index = $(this.items).index($(args.target).parent());
                if (index == -1)
                    index = $(this.items).index($(args.target));
                args.type === "mouseout" ? $(this.element.find("div.e-delete")[index]).css("visibility", "hidden") : $(this.element.find("div.e-delete")[index]).css("visibility", "visible");
                args.type === "mouseout" ? $(this.element.find("div.e-reload")[index]).css("visibility", "hidden") : $(this.element.find("div.e-reload")[index]).css("visibility", "visible");
            }
        },
        //-------------------- Event Wire-up -------------------------//
        /**
        * Wiring the events to tab control		
        * @private
        */
        _wireEvents: function (event) {
            //tabs Click evnts
            this._on(this.items, event, this._tabItemClick);
            this._on(this.itemsContainer, "mouseover", this._hoverHandler);
            this._on(this.itemsContainer, "mouseout", this._hoverHandler);
            this._on(this.element.find("div.e-delete"), "click", this._tabDeleteClick);
            this._on(this.itemsContainer, "focusin", this._focusIn);
            this._on(this.itemsContainer, "focusout", this._focusOut);
            this._on(this.element.find("div.e-reload"), "click", this._tabReloadClick);
        },
        //-------------------- Event UnWire-up -------------------------//
        /**
        * UnWiring the events from tab control		
        * @private
        */
        _unWireEvents: function () {
            this._off(this.anchors, this.model.events);
            this._off(this.element.find("div.e-delete"), "click");
            this._off(this.itemsContainer, "mouseover", this._hoverHandler);
            this._off(this.itemsContainer, "mouseout", this._hoverHandler);
            this._off(this.itemsContainer, "focusin", this._focusIn);
            this._off(this.itemsContainer, "focusout", this._focusOut);
            this._off(this.element.find("div.e-reload"), "click");
        },
        //--------------------- Client side Methods ----------------------//
        /**
       * This function disable the given item of the tab control
       * @alias disableItems
       * @return jQuery
	   * @param {number} index  index of tab item.
       * @example 
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// disable the item from the Tab based on the index
       *	tagObj.disableItems(1);
       * @example 
       * $("#tab").ejTab("disableItems",1);
       * @memberof ejTab
       * @instance
       */
        disableItems: function (index) {
            /// <summary>This function disable the given item of the tab control</summary>
            /// <param name="index" type="int"></param>
            if (!this.model.enabled) return false;
            if (index != null) {
                this.model.disabledItems.push(index);
                this.model.disabledItems.sort();
                this._disableTabs();
            }
        },
        /**
       * This function enable the given item of the tab control
       * @alias enableItems
       * @return jQuery
	   * @param {number} index  index of tab item.
       * @example 
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// enable the item from the Tab based on the index
       *	tagObj.enableItems(1);
       * @example 
       * $("#tab").ejTab("enableItems",1);
       * @memberof ejTab
       * @instance
       */
        enableItems: function (index) {
            /// <summary>This function enable the given item of the tab control</summary>
            /// <param name="index" type="int"></param>
            if (!this.model.enabled) return false;
            !ej.isNullOrUndefined(this.contentPanels[index]) && $(this.contentPanels[index]).removeClass("e-disable");
            if ($.inArray(index, this.model.disabledItems) == -1) {
                return false;
            }
            this.model.disabledItems = $.grep(this.model.disabledItems, function (n, i) {
                return n != index;
            });
            this._refresh();
            this._disableTabs();
        },
        //Deprecate this methods with enableItems()
        /**
       * Given index header and content panel disabled.
       * @alias disable
       * @return jQuery
	   * @param {number} index  index of tab item.
       * @example 
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// disable the item from the Tab based on the index
       *	tagObj.disable(1);
       * @example 
       * $("#tab").ejTab("disable",1);
       * @memberof ejTab
       * @instance
       */
        disable: function (index) {
            this.disableItems(index);
        },
        /**
       * Given index header and content panel enabled.
       * @alias enable
       * @return jQuery
	   * @param {number} index  index of tab item.
       * @example 
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// enable the item from the Tab based on the index
       *	tagObj.enable(1);
       * @example 
       * $("#tab").ejTab("enable",1);
       * @memberof ejTab
       * @instance
       */
        enable: function (index) {
            this.enableItems(index);
        },
        /**
       * This function get the number of tab rendered
       * @alias tabCount
       * @return number
       * @example 
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// get the number of tab rendered
       *	tagObj.tabCount();
       * @example 
       * $("#tab").ejTab("tabCount");
       * @memberof ejTab
       * @instance
       */
        tabCount: function () {
            /// <summary>This function get the number of tab rendered</summary>
            if (this.items) {
                return this.items.length;
            }
        },
        /**
        * Add new tab items with given name, url and given index position, if index null it’s add last item.
		* @alias add
		* @return jQuery
		* @param {string} url  URL name / tab id.
		* @param {string} displayLabel  Tab Display name.
		* @param {number} index  Index position to placed , this is optional.
		* @example 
		* //initialize the tab object
		*	var tabObj = $("#tab").data("ejTab");
		*	// Add new tab items with given list
		*	tagObj.add("#tabItem1","New Item",3);
		* @example 
        * $("#tab").ejTab("add","#tabItem1","New Item",3); 
		* @memberof ejTab
		* @instance
        */
        add: function (url, displayLabel, index, cssClass) {
            /// <summary>This function new tab item in dynamicaly</summary>
            /// <param name="url" type="String">URL name / tab id</param>
            /// <param name="displayLabel" type="String"> Tab Display name</param>
            /// <param name="index" type="int">Index position to placed , this is optional</param>
            if (this.model.headerPosition == "left") {
                this.items.length >= 0 && this.itemsContainer.addClass("e-left");
            }
            else if (this.model.headerPosition == "right") {
                this.items.length >= 0 && this.itemsContainer.addClass("e-right");
            }
            else
                this.items.length == 0 && this.itemsContainer.addClass("e-header");
            var liTag = ej.buildTag("li.e-select e-item");
            if (!ej.isNullOrUndefined(cssClass)) {
                var span = ej.buildTag('span').addClass(cssClass);
                liTag.append(span);
            }
            var aTag = ej.buildTag("a", displayLabel, {}, { href: url });
            aTag.appendTo(liTag);
            if (this.model.showCloseButton) {
                var deleteIcon = ej.buildTag('div.e-icon e-delete ');
                liTag.append(deleteIcon);
                this._on(deleteIcon, "click", this._tabDeleteClick);
            }
            var id = url.indexOf("#") == 0 ? url.replace("#", "") : this.model.idPrefix + this._getNextTabId();
            if (index === undefined) {
                index = this.anchors.length;
            }
            var insertIndex = index >= this.items.length;
            if (insertIndex) {
                liTag.appendTo(this.itemsContainer);
            } else {
                liTag.insertBefore(this.items[index]);
            }
            var panel = this.element.find("#" + id);
            if (!panel.length) {
                panel = this._createContentPanel(id);
                if (insertIndex) {
                    if (index > 0) {
                        panel.insertAfter(this.contentPanels.eq(-1));
                    } else {
                        panel.appendTo(this.element);
                    }
                } else {
                    panel.insertBefore(this.contentPanels[index]);
                }
            }
            if (this.model.selectedItemIndex == index) {
                this.hideItem[index];
                this.model.selectedItemIndex += 1;
            } else {
                this.hideItem[index];
                if (index < this.model.selectedItemIndex)
                    this.model.selectedItemIndex += 1;
            }
            this._refresh();
            var data = {
                addTab: this.anchors[index],
                addContentPanel: this.contentPanels[index]
            };
            this._onAdd(data);
        },
        /**
       * Remove the given index tab item.
       * @alias remove
       * @return jQuery
	   * @param {number} index  index of tab item.
       * @example 
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// Remove the given index tab item.
       *	tagObj.remove(3);
       * @example 
       * $("#tab").ejTab("remove",3); 
       * @memberof ejTab
       * @instance
       */
        remove: function (index) {
            /// <summary>This function Remove the given index</summary>
            /// <param name="index" type="int"></param>
            if (!this.model.enabled) return false;
            if (index != null && index > -1 && index < this.items.length) {
                if (this._onBrforeRemove({ deleteIndex: index }) === true)
                    return false;
                var removedTab = $(this.items[index]).remove();
                var removedPanel = $(this.contentPanels[index]).remove();
                if (removedTab.hasClass("e-active")) {
                    if (index == 0) {
                        this.model.selectedItemIndex = index;
                    }
                    else
                        this.model.selectedItemIndex = index - 1;
                }
                if (index < 0 || index >= this.anchors.length) {
                    this.model.selectedItemIndex = 0;
                }
                if ((this.model.headerPosition == "left" || this.model.headerPosition == "right") && this.items.length == 1)
                    this._removeVerticalClass();
                else
                    this.items.length == 1 && this.itemsContainer.removeClass("e-header");
                this._refresh();
                var data = {
                    removedTab: removedTab,
                    removedPanel: removedPanel
                };
                this._onRemove(data);
            }
        },
        /**
       * This function is to show the tab control.
       * @alias show
       * @return jQuery
       * @example 
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// To show the tab control.
       *	tagObj.show();
       * @example 
       * $("#tab").ejTab("show"); 
       * @memberof ejTab
       * @instance
       */
        show: function () {
            /// <summary>To Show the Tab control</summary>
            if (!this.model.enabled) return false;
            this.element.css("visibility", "visible");
        },
        /**
       * This function hides the tab control.
       * @alias hide
       * @return jQuery
       * @example 
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// To hide the tab control..
       *	tagObj.hide();
       * @example 
       * $("#tab").ejTab("hide"); 
       * @memberof ejTab
       * @instance
       */
        hide: function () {
            /// <summary>To Hide the Tab control</summary>
            if (!this.model.enabled) return false;
            this.element.css("visibility", "hidden");
        },
        //--------------------- End - Client side Methods ----------------------//

        //--------------------- Client side events ----------------------//
        /**
         * Section For handling before load event
		 * @private
         */
        _onBeforeLoad: function () {
            var data;
            if (this.model.selectedItemIndex == -1 && this.model.collapsible) {
                data = {
                    oldTab: this.items[this._preTabIndex],
                    oldContentPanel: this.contentPanels[this._preTabIndex],
                    OldSelectedIndex: this._preTabIndex,
                    newTab: null,
                    newContentPanel: null,
                    newSelectedIndex: null
                };
            }
            else
                data = {
                    oldTab: this.items[this._preTabIndex],
                    oldContentPanel: this.contentPanels[this._preTabIndex],
                    OldSelectedIndex: this._preTabIndex,
                    newTab: this.items[this.model.selectedItemIndex],
                    newContentPanel: this.contentPanels[this.model.selectedItemIndex],
                    newSelectedIndex: this.model.selectedItemIndex
                };
            if (this.model.ajaxBeforeLoad != null) {
                return this._trigger("ajaxBeforeLoad", data);
            }
        },
        /**
        * Section For handle the focusin event
        * @private
        */
        _focusIn: function () {
            if (!this.model.readOnly && this.model.allowKeyboardNavigation)
                $(this.element).bind("keydown", $.proxy(this._keyPress, this));
        },
        /**
         * Section For handle the focus out event
		 * @private
         */
        _focusOut: function (e) {
            $(this.element).unbind("keydown", $.proxy(this._keyPress, this));
        },
        /**
         * Section For handle the load event
		 * @private
         */
        _onLoad: function () {
            var data;
            if (this.model.selectedItemIndex == -1 && this.model.collapsible) {
                data = {
                    oldTab: this.items[this._preTabIndex],
                    oldContentPanel: this.contentPanels[this._preTabIndex],
                    oldSelectedIndex: this._preTabIndex,
                    newTab: null,
                    newContentPanel: null,
                    newSelectedIndex: null
                };
            }
            else
                data = {
                    oldTab: this.items[this._preTabIndex],
                    oldContentPanel: this.contentPanels[this._preTabIndex],
                    oldSelectedIndex: this._preTabIndex,
                    newTab: this.items[this.model.selectedItemIndex],
                    newContentPanel: this.contentPanels[this.model.selectedItemIndex],
                    newSelectedIndex: this.model.selectedItemIndex
                };
            if (this.model.ajaxLoad != null) {
                return this._trigger("ajaxLoad", data);
            }
        },
        /**
        * Section For handle the Active event
        * @private
        */
        _onActive: function () {
            if (this.model.active) {
                var data;
                if (this.model.selectedItemIndex == -1 && this.model.collapsible) {
                    data = {
                        oldTab: this.items[this._preTabIndex],
                        oldContentPanel: this.contentPanels[this._preTabIndex],
                        oldSelectedIndex: this._preTabIndex,
                        newTab: null,
                        newContentPanel: null,
                        newSelectedIndex: null
                    };
                }
                else
                    data = {
                        oldTab: this.items[this._preTabIndex],
                        oldContentPanel: this.contentPanels[this._preTabIndex],
                        oldSelectedIndex: this._preTabIndex,
                        newTab: this.items[this.model.selectedItemIndex],
                        newContentPanel: this.contentPanels[this.model.selectedItemIndex],
                        newSelectedIndex: this.model.selectedItemIndex
                    };
                return this._trigger("active", data);
            }
        },
        /**
        * Section For handle the Before Active event.
        * @private
        */
        _onBeforeActive: function (index) {
            if (this.model.beforeActive != null) {
                var data;
                if (this.model.selectedItemIndex == -1 && this.model.collapsible) {
                    data = {
                        oldTab: this.items[this._preTabIndex],
                        oldContentPanel: this.contentPanels[this._preTabIndex],
                        oldSelectedIndex: this._preTabIndex,
                        newTab: null,
                        newContentPanel: null,
                        newSelectedIndex: null
                    };
                }
                else
                    data = {
                        oldTab: this.items[this._preTabIndex],
                        oldContentPanel: this.contentPanels[this._preTabIndex],
                        oldSelectedIndex: this._preTabIndex,
                        newTab: this.items[index],
                        newContentPanel: this.contentPanels[index],
                        newSelectedIndex: index
                    };
                return this._trigger("beforeActive", data);
            }
        },
        /**
        * Section For trigger add event.
        * @private
        */
        _onAdd: function (data) {
            if (this.model.itemAdd != null) {
                return this._trigger("itemAdd", data);
            }
        },
        /**
         * Section For trigger remove event.
		 * @private
         */
        _onRemove: function (data) {
            if (this.model.itemRemove != null) {
                return this._trigger("itemRemove", data);
            }
        },
        /**
         * Section For trigger before remove event.
		 * @private
         */
        _onBrforeRemove: function (data) {
            if (this.model.beforeItemRemove != null) {
                return this._trigger("beforeItemRemove", data);
            }
        }
    });
    /**
 * Enum for tab height Style.	 
 * @enum {string}
 * @global 
 */
    ej.Tab.heightStyle = {
        /**  Panel height adjusts based on the content */
        Content: "content",
        /**  All panel height will be set the tallest panel height. */
        Auto: "auto",
        /**  Content panel take based on the parent height. */
        Fill: "fill"
    };
    /**
 * Enum for Tab Position	 
 * @enum {string}
 * @global 
 */
    ej.Tab.Position = {
        /**  Tab headers display to top position. */
        Top: "top",
        /**  Tab headers display to bottom position. */
        Bottom: "bottom",
        /**  Tab headers display to left position. */
        Left: "left",
        /** Tab headers display to right position. */
        Right: "right"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the tagCloud control.
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    // ejTagcloud is the plugin name 
    // "ej.Tagcloud" is "namespace.className" will hold functions and properties
    /**
    * @namespace ej
	* @class ejTagCloud
	* @requires jQuery
	* @requires ej.web.all
	* @classdesc Custom Design for tagCloud control.
	* @example 
	* &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
    *
	*$(function () {
	*	// document ready
	*	// initialize the array of data for tagcloud input
	*	var websiteCollection = [
	*             { text: "Crave", url: "http://news.cnet.com/crave/", frequency: 4 },
	*             { text: "Daily Tech", url: "http://www.dailytech.com/", frequency: 1 },
	*             { text: "Tech Republic", url: "http://techrepublic.com/", frequency: 4 },
	*             { text: "Arts Technica", url: "http://arstechnica.com/", frequency: 7 },
	*             { text: "WebProNews", url: "http://www.webpronews.com/", frequency: 1 }
	*       ];
	*
	* // simple tagcloud creation
	*	$("#tagcloud").ejTagCloud({ 
	*             dataSource: websiteCollection,
	*             title: "Tech Sites"
	*       });
	*});
    */
    ej.widget("ejTagCloud", "ej.TagCloud", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-tagcloud",
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Specify the CSS class to button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the CSS class during initialization. 			
			* 	$("#tagcloud").ejTagCloud({  cssClass : "gradient-lime" });			* 
			* @example 
			* //Get or set the cssClass after initialization:<br>
			*	//Get the cssClass API value.		
			*	$("#tagcloud").ejTagCloud("option", "cssClass");			
			*	//Set the cssClass API value
			*	$("#tagcloud").ejTagCloud("option", "cssClass",  "gradient-lime" );
			 * @memberof ejTagCloud
			* @instance
			*/
            cssClass: "",
            /**		
			* The dataSource contains the list of data to display in a cloud format. Each data contains a link url, frequency to categorize the font size and a display text.
			* @default null
			* @type {object}
			* @example 
			* Initialize the TagCloud with the dataSource value specified		
            * var items = [
			*	{ text: "Google", url: "http://www.google.com", frequency: 12 },
			*	{ text: "All Things Digital", url: "http://allthingsd.com/", frequency: 3 },
			*	{ text: "Arts Technica", url: "http://arstechnica.com/", frequency: 8 }
            * ];
            * $("#tagcloud").ejTagCloud({ dataSource: items });
			* @example 
			* //Get or set the dataSource after initialization:<br>
			*	//Get the dataSource API value.		
			*	$("#tagcloud").ejTagCloud('option', 'dataSource');			
			*	//Set the dataSource API value
			*	$("#tagcloud").ejTagCloud('option', 'dataSource', items);
			 * @memberof ejTagCloud
			* @instance
			*/
            dataSource: null,
            /**		
			* Define the query to retrieve the data from online server. The query is used only when the online dataSource is used.
			* @default null
			* @type {object}
			* @example 
			* Initialize the TagCloud with the query value specified		
            * var dataManger = ej.DataManager({
			*       url: &lt;http://mvc.syncfusion.com/Services/Northwnd.svc/&gt;
			*});
			*
			*var queryString = ej.Query().from("Suppliers").select("SupplierID", "ContactName");
			*
			*$("#tagcloud").ejTagCloud({ 					
			*	dataSource: dataManger,
			*       query: queryString,
			*	fields: { text: "ContactName", frequency: "SupplierID" }
			*});
			* @example 
			* //Get or set the query after initialization:<br>
			*	//Get the query API value.		
			*	$("#tagcloud").ejTagCloud('option', 'query');			
			*	//Set the query API value
			*	$("#tagcloud").ejTagCloud('option', 'query', queryString);
			 * @memberof ejTagCloud
			* @instance
			*/
            query: null,
            /**		
			* Defines the mapping fields for the data items of the TagCloud.
			* @default null 
			* @type {Object}
			* @param {Object} options Defines the mapping fields for the data items of the TagCloud.
			* @param {String} options.text - This component's name, sets {@link Component#text}
            * @param {String} options.url - Whether this component is vislble, sets {@link Component#url}
			* @example 
			* Initialize the TagCloud with the fields value specified		
            * var items = [
			*	{ text: "Google", url: "http://www.google.com", frequency: 12 },
			*	{ text: "All Things Digital", url: "http://allthingsd.com/", frequency: 3 },
			*	{ text: "Arts Technica", url: "http://arstechnica.com/", frequency: 8 }
            * ];
			* $("#tagcloud").ejTagCloud({ 
			* dataSource: items,
			* fields: { text: "name", url: "link" }
			* });
			* @example 
			* //Get or set the fields after initialization:<br>
			*	//Get the fields API value.		
			*	$("#tagcloud").ejTagCloud('option', 'fields');			
			*	//Set the fields API value
			*	$("#tagcloud").ejTagCloud('option', 'fields', { text: "name", url: "link" });
			* @memberof ejTagCloud
			* @instance
			*/
            fields: { text: "text", url: "url", frequency: "frequency" },
            /**		
           * Shows or hides the TagCloud title. When this set to false, it hides the TagCloud header.
           * @default true
           * @type {boolean}
           * @example 
           * // Set the showTitle during initialization. 			
           * 	$("#tagcloud").ejTagCloud({ showTitle : false  });			* 
           * @example 
           * //Get or set the showTitle after initialization:<br>
           *	//Get the showTitle API value.		
           *	$("#tagcloud").ejTagCloud("option", "showTitle");			
           *	//Set the showTitle API value
           *	$("#tagcloud").ejTagCloud("option", "showTitle", true );
            * @memberof ejTagCloud
           * @instance
           */
            showTitle: true,
            /**		
			* Sets the title text for the TagCloud. To show the title text, the showTitle property should be enabled.
			* @default "Title"
			* @type {string}
			* @example 
			* // Set the title during initialization. 			
			* 	$("#tagcloud").ejTagCloud({ title : "Cloud Data"  });			* 
			* @example 
			* //Get or set the title after initialization:<br>
			*	//Get the title API value.		
			*	$("#tagcloud").ejTagCloud("option", "title");			
			*	//Set the title API value
			*	$("#tagcloud").ejTagCloud("option", "title", "Cloud Data" );
			 * @memberof ejTagCloud
			* @instance
			*/
            title: "Title",
            /**		
			* Sets the title image for the TagCloud. To show the title image, the showTitle property should be enabled.
			* @default null
			* @type {string}
			* @example 
			* // Set the titleImage during initialization. 			
			* 	$("#tagcloud").ejTagCloud({ titleImage: '../images/tag.png'   });			* 
			* @example 
			* //Get or set the titleImage after initialization:<br>
			*	//Get the titleImage API value.		
			*	$("#tagcloud").ejTagCloud("option", "titleImage");			
			*	//Set the titleImage API value
			*	$("#tagcloud").ejTagCloud("option", "titleImage", "../images/tagCloud.png" );
			 * @memberof ejTagCloud
			* @instance
			*/
            titleImage: null,
            /**		
			* Defines the format for the TagCloud to display the tag items. 
			* @default ej.format.Cloud
			* @type {String | Enum}
			* @example 
			* // Set the format during initialization. 			
			* 	$("#tagcloud").ejTagCloud({ format: ej.format.Cloud });			* 
			* @example 
			* //Get or set the format after initialization:<br>
			*	//Get the format API value.		
			*	$("#tagcloud").ejTagCloud("option", "format");			
			*	//Set the format API value
			*	$("#tagcloud").ejTagCloud("option", "format", "list" );
			 * @memberof ejTagCloud
			* @instance
			*/
            format: "cloud",
            /**		
           * Sets the TagCloud and tag items direction as right to left alignment.
           * @default false
           * @type {boolean}
           * @example 
           * // Set the rtl during initialization. 			
           * 	$("#tagcloud").ejTagCloud({ rtl : false  });			* 
           * @example 
           * //Get or set the rtl after initialization:<br>
           *	//Get the rtl API value.		
           *	$("#tagcloud").ejTagCloud("option", "rtl");			
           *	//Set the rtl API value
           *	$("#tagcloud").ejTagCloud("option", "rtl", true );
            * @memberof ejTagCloud
           * @instance
           */
            rtl: false,
            /**		
           * Sets the minimum font size value for the tag items. The font size for the tag items will be generated in between the minimum and maximum font size values.
           * @default "10px"
           * @type {String | Number}
           * @example 
           * // Set the minFontSize during initialization. 			
           * 	$("#tagcloud").ejTagCloud({ minFontSize : "10px"  });			* 
           * @example 
           * //Get or set the minFontSize after initialization:<br>
           *	//Get the minFontSize API value.		
           *	$("#tagcloud").ejTagCloud("option", "minFontSize");			
           *	//Set the minFontSize API value
           *	$("#tagcloud").ejTagCloud("option", "minFontSize", "20px" );
            * @memberof ejTagCloud
           * @instance
           */
            minFontSize: "10px",
            /**		
           * Sets the maximum font size value for the tag items. The font size for the tag items will be generated in between the minimum and maximum font size values.
           * @default "40px"
           * @type {String | Number}
           * @example 
           * // Set the maxFontSize during initialization. 			
           * 	$("#tagcloud").ejTagCloud({ maxFontSize : "10px"  });			* 
           * @example 
           * //Get or set the maxFontSize after initialization:<br>
           *	//Get the maxFontSize API value.		
           *	$("#tagcloud").ejTagCloud("option", "maxFontSize");			
           *	//Set the maxFontSize API value
           *	$("#tagcloud").ejTagCloud("option", "maxFontSize", "20px" );
            * @memberof ejTagCloud
           * @instance
           */
            maxFontSize: "40px",

            //Events
            /**     
            * Event triggers when the cursor hovers on a tag item
            * @event
            * @name ejTagCloud#mouseover 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the tagcloud model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.text return current tag name
            * @param {string}  argument.url return current url link
            * @example 
            * //mouse over event for tagCloud
            * $("#tagcloud").ejTagCloud({
            *    mouseover: function (args) {}
            * });      
            * @memberof ejTagCloud
            * @instance
            */
            mouseover: null,
            /**     
            * Event triggers when the cursor leaves out from a tag item
            * @event
            * @name ejTagCloud#mouseout 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the tagcloud model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.text return current tag name
            * @param {string}  argument.url return current url link
            * @example 
            * //mouseout event for tagCloud
            * $("#tagcloud").ejTagCloud({
            *    mouseout: function (args) {}
            * });      
            * @memberof ejTagCloud
            * @instance
            */
            mouseout: null,
            /**     
            * Event triggers when the TagCloud items are clicked
            * @event
            * @name ejTagCloud#click 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the tagcloud model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.text return current tag name
            * @param {string}  argument.url return current url link
            * @example 
            * //click event for tagCloud
            * $("#tagcloud").ejTagCloud({
            *    click: function (args) {}
            * });      
            * @memberof ejTagCloud
            * @instance
            */
            click: null
        },
        /**
      * Specify the data types for default properties 
      * @private
      */
        dataTypes: {
            fields: "object",
            cssClass: "string",
            showTitle: "boolean",
            title: "string",
            titleImage: "string",
            format: "enum",
            rtl: "boolean",
            dataSource: "data",
            fields: "data"
        },

        // constructor function
        /**
       * Create the TagCloud widget
       * @private
       */
        _init: function () {
            this._initialize();
            this._render();
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "fields":
                    case "query":
                    case "dataSource":
                    case "minFontSize":
                    case "maxFontSize":
                        this._refreshTagItems(option, options[option]); break;
                    case "showTitle": this._showTitle(options[option]); break;
                    case "title": this._title(options[option]); break;
                    case "titleImage": this._titleImage(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "format": this._format(options[option]); break;
                    case "rtl": this._rtl(options[option]); break;
                }
            }
        },

        _refreshTagItems: function (key, value) {
            this.model[key] = value;
            this.ul.empty();
            this._checkDataBinding();
        },
        /**
         * Section For enable or disable the title
		 * @private
         */
        _showTitle: function (boolean) {
            if (boolean) this._generateTitle();
            else {
                this.title.remove();
                this.title = null;
            }
        },
        /**
        * To configure TagCloud Title		
        * @private
        */
        _title: function (text) {
            if (this.title) {
                if (text) {
                    if (this.text) this.text.html(text);
                    else this._generateTextTag(text);
                }
                else if (this.text) {
                    this.text.remove();
                    this.text = null;
                }
            }
        },
        /**
         * Section For Sets the title image for the TagCloud
		 * @private
         */
        _titleImage: function (imagePath) {
            if (this.title) {
                if (imagePath) {
                    if (this.image) this.image.attr("src", imagePath);
                    else this._generateImageTag(imagePath);
                }
                else if (this.image) {
                    this.image.remove();
                    this.image = null;
                }
            }
        },
        /**
        * To configure the custom theme for TagCloud using cssClass property		
        * @private
        */
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
        * Inserts a new item into the TagCloud
		* @alias insert
		* @return jQuery
		* @example 
		* var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
		* //initialize the tagCloud object
		*	var tagObj = $("#tagcloud").data("ejTagCloud");
		*	//To Inserts a new item into the TagCloud
		*	tagObj.insert(tag);
		* @example 
		* var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
        * $("#tagcloud").ejTagCloud("insert", tag);
		* @memberof ejTagCloud
		* @instance
        */
        insert: function (tag) {
            if ($.trim(tag.text)) {
                this.ul.append(this._generateLi(tag, this._getMapper()));
            }
        },
        /**
        * Inserts a new item into the TagCloud at a particular position. 
		* @alias insertAt
		* @return jQuery
		* @example 
		* var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
		* //initialize the tagCloud object
		*	var tagObj = $("#tagcloud").data("ejTagCloud");
		*	//To Inserts a new item into the TagCloud at a particular position.
		*	tagObj.insertAt(tag,5);
		* @example 
		* var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
        * $("#tagcloud").ejTagCloud("insertAt", tag, 5);
		* @memberof ejTagCloud
		* @instance
        */
        insertAt: function (tag, position) {
            if ($.trim(tag.text)) {
                $(this.ul.children()[position - 1]).before(this._generateLi(tag, this._getMapper()));
            }
        },
        /**
        * Removes the item from the TagCloud based on the name. It removes all the tags which have the corresponding name 
		* @alias remove
		* @return jQuery
		 * @param {string} name  name of the tag.
		* @example 
		* var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
		* //initialize the tagCloud object
		*	var tagObj = $("#tagcloud").data("ejTagCloud");
		*	// Removes the item from the TagCloud based on the name
		*	tagObj.remove("google");
		* @example 
        * $("#tagcloud").ejTagCloud("remove", "google");
		* @memberof ejTagCloud
		* @instance
        */
        remove: function (txt) {
            var li = this.ul.children(), liTag, i;
            for (i = 0; i < li.length; i++) {
                liTag = $(li[i]);
                if (liTag.children()[0].innerHTML == txt)
                    liTag.remove();
            }
        },
        /**
        * Removes the item from the TagCloud based on the position. It removes the tags from the the corresponding position only. 
		* @alias removeAt
		* @return jQuery
		* @param {number} position  position of tag item.
		* @example 
		* var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
		* //initialize the tagCloud object
		*	var tagObj = $("#tagcloud").data("ejTagCloud");
		*	// Removes the item from the TagCloud based on the position.
		*	tagObj.removeAt(4);
		* @example 
        * $("#tagcloud").ejTagCloud("removeAt", 4);
		* @memberof ejTagCloud
		* @instance
        */
        removeAt: function (position) {
            var li = this.ul.children();
            $(li[position - 1]).remove();
        },
        /**
        * To configure the TagCloud format		
        * @private
        */
        _format: function (format) {
            if (format == "cloud") {
                this.ul.removeClass("e-list");
                this.ul.addClass("e-cloud");
            }
            else if (format == "list") {
                this.ul.removeClass("e-cloud");
                this.ul.addClass("e-list");
            }
        },
        /**
        * destroy the tagCloud widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @private
        */
        _destroy: function () {
            this.element.removeClass("e-widget " + this.model.cssClass);
            this.element.empty();
        },
        /**
        * To initialize the TagCloud		
        * @private
        */
        _initialize: function () {
            this.minFreq = 0;
            this.maxFreq = 30;
            this.ul = null;
            this.title = null;
            this.image = null;
            this.text = null;
        },
        /**
        * Render Section For Tagcloud		
        * @private
        */
        _render: function () {
            this.element.addClass("e-widget " + this.model.cssClass);
            if (this.model.showTitle) this._generateTitle();
            this._renderWrapper();
            this._checkDataBinding();
            this._checkProperties();
        },
        /**
        * To configure the tagCloud title Text
        * @private
        */
        _generateTitle: function () {
            this.title = ej.buildTag("div.e-header e-title");
            if (this.model.titleImage)
                this._generateImageTag(this.model.titleImage);
            if (this.model.title)
                this._generateTextTag(this.model.title);
            if (this.ul) this.title.insertBefore(this.ul);
            else this.element.append(this.title);
        },
        /**
         * To configure the tagCloud title image
		 * @private
         */
        _generateImageTag: function (titleImage) {
            if (!this.image)
                this.image = $('<img class="e-title-img" src="' + titleImage + '" />');
            if (this.text && !this.model.titleImage) this.image.insertBefore(this.text);
            else this.title.append(this.image);
        },
        /**
        * Section for generate Title Text 
        * @private
        */
        _generateTextTag: function (title) {
            if (!this.text) this.text = ej.buildTag("span", title);
            this.title.append(this.text);
        },
        /**
         * Render Section For TagCloud		
		 * @private
         */
        _renderWrapper: function () {
            var format;
            format = (this.model.format == "list") ? "list" : "cloud";
            this.ul = ej.buildTag("ul.e-ul e-box e-" + format);
            this.element.append(this.ul);
        },
        /**
        * Section for rendering tagCloud items
        * @private
        */
        _renderItems: function (list) {
            this._generateTagItems(list);
            this.ul.removeClass("e-load");
        },

        _checkProperties: function () {
            if (this.model.rtl) this._rtl(this.model.rtl);
        },
        /**
        * To enable or disable the Right to Left behaviour 		
        * @private
        */
        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },
        /**
        * Section for check whether collection of items is local or remote data	
        * @private
        */
        _checkDataBinding: function () {
            var source = this.model.dataSource;
            if (source != null) {
                this.ul.addClass("e-load");
                if (ej.DataManager && source instanceof ej.DataManager)
                    this._initDataSource(source);
                else
                    this._renderItems(source);
            }
        },
        /**
       * Section for render items with Remote Data	
       * @private
       */
        _initDataSource: function (source) {
            var proxy = this;
            var queryPromise = source.executeQuery(this._getQuery());
            queryPromise.done(function (e) {
                proxy._renderItems(e.result);
            }).fail(function (e) {
                proxy.ul.removeClass("e-load");
            });
        },
        /**
        * Section for set attributes to tagCloud items
        * @private
        */
        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    element.attr(key, data[key]);
            }
        },

        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },

        _generateTagItems: function (list) {
            var i, mapField = this._getMapper();
            this.minFreq = Math.min.apply(Math, list.map(function (o) { return o[mapField._freq]; }))
            this.maxFreq = Math.max.apply(Math, list.map(function (o) { return o[mapField._freq]; }))

            for (i = 0; i < list.length; i++) {
                this.ul.append(this._generateLi(list[i], mapField));
            }
        },
        /**
        * Section for mapping fields
        * @private
        */
        _getMapper: function () {
            var mapper = this.model.fields, mapFld = { _text: null, _freq: null, _url: null };
            mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            mapFld._freq = (mapper && mapper.frequency) ? mapper["frequency"] : "frequency";
            mapFld._url = (mapper && mapper.url) ? mapper["url"] : "url";
            mapFld._attr = (mapper && mapper.htmlAttr) ? mapper["htmlAttr"] : "htmlAttr";
            return mapFld;
        },
        /**
        * Section for render tagCloud items
        * @private
        */
        _generateLi: function (list, map) {
            var li = ej.buildTag("li.e-tagitems"), aTag;
            aTag = ej.buildTag("a.e-txt", list[map._text] || list[map._url],
                    { "font-size": this._calculateFontSize(list[map._freq]) },
                    { "href": list[map._url] || "#", "target": "blank", role: "link" });
            li.append(aTag);
            this._setAttributes((list[map._attr]), li);
            this._on(aTag, "mouseenter", this._mouseEnter);
            this._on(aTag, "mouseleave", this._mouseLeave);
            this._on(aTag, "click", this._mouseClick);
            return li;
        },

        /**
        * Section For track mouseEnter event
        * @private
        */
        _mouseEnter: function (e) {
            $(e.target).addClass("hover");
            this._raiseEvent(e, "mouseover");
        },
        /**
        * Section For track mouseLeave event
        * @private
        */
        _mouseLeave: function (e) {
            $(e.target).removeClass("hover");
            this._raiseEvent(e, "mouseout");
        },
        /**
        * Section For handle the click event
        * @private
        */
        _mouseClick: function (e) {
            $(e.target).removeClass("hover");
            this._raiseEvent(e, "click");
        },

        _raiseEvent: function (e, _type) {
            if (this.model[_type])
                this._trigger(_type, { value: $(e.target).html(), url: $(e.target).attr('href'), eventType: _type });
        },
        /**
        * Section for calculate Font Size
        * @private
        */
        _calculateFontSize: function (frequency) {
            if (frequency) {
                var C = 2, k, fontRange, fontSize,
                minSize = parseInt(this.model.minFontSize, 10), maxSize = parseInt(this.model.maxFontSize, 10);
                k = (frequency - this.minFreq) / (this.maxFreq - this.minFreq);
                fontRange = maxSize - minSize;
                fontSize = minSize + (C * Math.floor(k * (fontRange / C)));
                return fontSize;
            }
            else return this.model.minFontSize;
        }
    });
    /**
	 * Enum for various format	 
	 * @enum {string}
	 * @global 
	 */
    ej.format = {
        /**  Render the tagCloud items in cloud format */
        Cloud: "cloud",
        /**  Render the tagCloud items in list format */
        List: "list"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html div elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejTreeview
    * @requires jQuery
    * @requires ej.tools.all
    * @classdesc Custom Design for Html div control.
    * @example 
    * &lt;ul id="treeView"&gtau
            &lt;li&gt;Artwork
                &lt;ul&gt;
                    &lt;li&gt;Abstract
                        &lt;ul&gt;
                            &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                            &lt;li&gt;Creative Acrylic&lt;/li&gt;
                            &lt;li&gt;Modern Painting&lt;/li&gt;
                            &lt;li&gt;Canvas Art&lt;/li&gt;
                            &lt;li&gt;Black white&lt;/li&gt;
                        &lt;/ul&gt;
                    &lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Books
                &lt;ul&gt;
                    &lt;li&gt;Entertaining&lt;/li&gt;
                    &lt;li&gt;Design&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Music
                &lt;ul&gt;
                    &lt;li&gt;Mass&lt;/li&gt;
                    &lt;li&gt;Folk&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
        &lt;/ul&gt;
     <br> 
    * &lt;script&gt;
    * // Create TreeView
    * $('#treeView').ejTreeView(); 	
    * &lt;/script&gt;
    */
    ej.widget("ejTreeView", "ej.TreeView", {
        _rootCSS: "e-treeview",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["ul", "div"],
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* This API to enable the checkbox for all nodes. 
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the showCheckbox value specified.
			* $("#treeView").ejTreeView({ showCheckbox: true });
			* @example 
			* //Get or set the showCheckbox API, after initialization:<br>
			*	// Get the showCheckbox API value.		
			*	$("#treeView").ejTreeView('option', 'showCheckbox');			
			*	// Set the showCheckbox API
			*	$("#treeView").ejTreeView('option', 'showCheckbox',  true);
			* @memberof ejTreeView
			* @instance
			*/
            showCheckbox: false,
            /**		
			* To enable the drag and drop nodes. 
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the dragAndDrop value specified.
			* $("#treeView").ejTreeView({ dragAndDrop: true });
			* @example 
			* //Get or set the dragAndDrop API, after initialization:<br>
			*	// Get the dragAndDrop API value.		
			*	$("#treeView").ejTreeView('option', 'dragAndDrop');			
			*	// Set the dragAndDrop API
			*	$("#treeView").ejTreeView('option', 'dragAndDrop',  true);
			* @memberof ejTreeView
			* @instance
			*/
            dragAndDrop: false,
            /**		
			* Allow to drop the child node for dragging node. 
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the dropChild value specified.
			* $("#treeView").ejTreeView({ dropChild: true });
			* @example 
			* //Get or set the dropChild API, after initialization:<br>
			*	// Get the dropChild API value.		
			*	$("#treeView").ejTreeView('option', 'dropChild');			
			*	// Set the dropChild API
			*	$("#treeView").ejTreeView('option', 'dropChild',  true);
			* @memberof ejTreeView
			* @instance
			*/
            dropChild: true,
            /**		
			* Allow to drop the root node for dragging node. 
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the dropSibling value specified.
			* $("#treeView").ejTreeView({ dropSibling: true });
			* @example 
			* //Get or set the dropSibling API, after initialization:<br>
			*	// Get the dropSibling API value.		
			*	$("#treeView").ejTreeView('option', 'dropSibling');			
			*	// Set the dropSibling API
			*	$("#treeView").ejTreeView('option', 'dropSibling',  true);
			* @memberof ejTreeView
			* @instance
			*/
            dropSibling: true,
            /**		
			* Allow to drag the node out of the controls, like one TreeView to another.
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the dragAndDropAcrossControl value specified.
			* $("#treeView").ejTreeView({ dragAndDropAcrossControl: true });
			* @example 
			* //Get or set the dragAndDropAcrossControl API, after initialization:<br>
			*	// Get the dragAndDropAcrossControl API value.		
			*	$("#treeView").ejTreeView('option', 'dragAndDropAcrossControl');			
			*	// Set the dragAndDropAcrossControl API
			*	$("#treeView").ejTreeView('option', 'dragAndDropAcrossControl',  true);
			* @memberof ejTreeView
			* @instance
			*/
            dragAndDropAcrossControl: true,
            /**		
			* Allow to user can edit the node text.
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the allowEdit value specified.
			* $("#treeView").ejTreeView({ allowEdit: true });
			* @example 
			* //Get or set the allowEdit API, after initialization:<br>
			*	// Get the allowEdit API value.		
			*	$("#treeView").ejTreeView('option', 'allowEdit');			
			*	// Set the allowEdit API
			*	$("#treeView").ejTreeView('option', 'allowEdit',  true);
			* @memberof ejTreeView
			* @instance
			*/
            allowEdit: false,
            /**		
			* Enable the keyboard interaction with TreeView nodes.
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the allowKeyboardNavigation value specified.
			* $("#treeView").ejTreeView({ allowKeyboardNavigation: true });
			* @example 
			* //Get or set the allowKeyboardNavigation API, after initialization:<br>
			*	// Get the allowKeyboardNavigation API value.		
			*	$("#treeView").ejTreeView('option', 'allowKeyboardNavigation');			
			*	// Set the allowKeyboardNavigation API
			*	$("#treeView").ejTreeView('option', 'allowKeyboardNavigation',  true);
			* @memberof ejTreeView
			* @instance
			*/
            allowKeyboardNavigation: true,
            /**		
             * Items API have id, text, spriteCssClass,expanded, childItems ,selected ,linkAttribute ,value, imageAttribute, htmlAttribute,imageUrl. 
             * @default null
             * @type {object}
             * @example 
             * //To set items API value during initialization  
             * 	$("#treeView").ejTreeView(items: { id: "id",text: "name"});
             * @example 
             * //Get or set the items  API, after initialization:<br>
             *	//Gets the items  value  
             *	$("#treeView").ejTreeView('option',  'items' );<br>			
             *	//Sets the items value 
             *	$("#treeView").ejTreeView('option', 'items', { id: "id",text: "name"} );
             * @memberof ejTreeView
             * @instance
             */
            items: null, //id, text, spriteCssClass,expanded, childItems ,selected ,linkAttribute ,value, imageAttribute, htmlAttribute,imageUrl
            /**		
             * Fields API have dataSource, query, tableName and mapper fields. In dataSource, we can pass the JSON data object or Essential Datamanager objects.  query; it have ej.Query() object. tableName; It have data get form given table name. mapper fields;  columns of a table can be mapped to the TreeView properties, namely id, parentId, text, spriteCssClass,expanded, childItems, hasChild, selected, linkAttribute , imageAttribute, htmlAttribute, imageUrl, isChecked.
             * @default null
             * @type {object}
             * @example 
             * //To set fields API value during initialization  
             * 	$("#treeView").ejTreeView({fields: {dataSource: localData,id: "id",parentId: "pid",text: "name",hasChild: "hasChild"}});
             * @example 
             * //Get or set the fields  API, after initialization:<br>
             *	//Gets the fields  value  
             *	$("#treeView").ejTreeView('option',  'fields' );<br>			
             *	//Sets the fields value 
             *	$("#treeView").ejTreeView('option', 'fields',  {dataSource: localData,id: "id",parentId: "pid",text: "name",hasChild: "hasChild"});
             * @memberof ejTreeView
             * @instance
             */
            fields: {
                dataSource: null,
                query: null,
                tableName: null,
                child: null,
                id: "id",
                parentId: "parentId",
                text: "text",
                spriteCssClass: "spriteCssClass",
                expanded: "expanded",
                hasChild: "hasChild",
                selected: "selected",
                linkAttribute: "linkAttribute",
                value: "value",
                imageAttribute: "imageAttribute",
                htmlAttribute: "htmlAttribute",
                imageUrl: "imageUrl",
                isChecked: "isChecked"
            }, //null, //id, parentId, text, spriteCssClass,expanded, childItems, hasChild, selected, linkAttribute ,value, imageAttribute, htmlAttribute, imageUrl, isChecked, tableName
            /**		
			* Allow to check the parent node automatically if this property is enabled.
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the autoCheckParentNode value specified.
			* $("#treeView").ejTreeView({ autoCheckParentNode: true });
			* @example 
			* //Get or set the autoCheckParentNode API, after initialization:<br>
			*	// Get the autoCheckParentNode API value.		
			*	$("#treeView").ejTreeView('option', 'autoCheckParentNode');			
			*	// Set the autoCheckParentNode API
			*	$("#treeView").ejTreeView('option', 'autoCheckParentNode',  true);
			* @memberof ejTreeView
			* @instance
			*/
            autoCheckParentNode: false,
            /**		
			* Child node loaded on demand, while expand the parent node at the time the entire child loaded.
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the loadOnDemand value specified.
			* $("#treeView").ejTreeView({ loadOnDemand: true });
			* @example 
			* //Get or set the loadOnDemand API, after initialization:<br>
			*	// Get the loadOnDemand API value.		
			*	$("#treeView").ejTreeView('option', 'loadOnDemand');			
			*	// Set the loadOnDemand API
			*	$("#treeView").ejTreeView('option', 'loadOnDemand',  true);
			* @memberof ejTreeView
			* @instance
			*/
            loadOnDemand: false,
            /**		
            * Sets the root class for TreeView theme. This cssClass API helps to use custom skinning option for TreeView control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
            * //Initialize the TreeView with the cssClass value specified
            * 	$("#treeView").ejTreeView({ cssClass: 'gradient-lime'});
            * @example 
            * Get or set the cssClass API, after initialization:<br>
            *	//Gets the cssClass  value
            *	$("#treeView").ejTreeView("option", "cssClass ");			
            *	//Sets the cssClass  value
            *	$("#treeView").ejTreeView("option", "cssClass ", "gradient-lime" );
            * @memberof ejTreeView
            * @instance
            */
            cssClass: "",
            /**		
            * We can customize the node our own style.
            * @default null
            * @type {string}
            * @example 
            * //JSrender template 
            *&lt;script id="treeview-template"&gt;
            *&lt;li&gt;{{&gt;text}} &lt;a class='delete-link' href='\#'&gt;&lt;/a&gt;&lt;/li&gt;
            *&lt;/script&gt;
            * //Initialize the TreeView with the template value specified
            * 	$("#treeView").ejTreeView({ template: "treeview-template"});
            * @example 
            * Get or set the template API, after initialization:<br>
            *	//Gets the template  value
            *	$("#treeView").ejTreeView("option", "template ");			
            *	//Sets the template  value
            *	$("#treeView").ejTreeView("option", "template ", "treeview-template" );
            * @memberof ejTreeView
            * @instance
            */
            template: null,
            /**		
			* Display Right to Left direction for node text.
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the rtl value specified.
			* $("#treeView").ejTreeView({ rtl: true });
			* @example 
			* //Get or set the rtl API, after initialization:<br>
			*	// Get the rtl API value.		
			*	$("#treeView").ejTreeView('option', 'rtl');			
			*	// Set the rtl API
			*	$("#treeView").ejTreeView('option', 'rtl',  true);
			* @memberof ejTreeView
			* @instance
			*/
            rtl: false,
            /**		
            * Node expanded on specified events
            * @default "gradient-lime"
            * @type {string}
            * @example 
            * //Initialize the TreeView with the expandEvent value specified
            * 	$("#treeView").ejTreeView({ expandEvent: 'dblclick'});
            * @example 
            * Get or set the expandEvent API, after initialization:<br>
            *	//Gets the expandEvent  value
            *	$("#treeView").ejTreeView("option", "expandEvent ");			
            *	//Sets the expandEvent  value
            *	$("#treeView").ejTreeView("option", "expandEvent ", "dblclick" );
            * @memberof ejTreeView
            * @instance
            */
            expandEvent: "dblclick",
            /**		
			* Save current model value to browser cookies for state maintains. While refresh the TreeView control page the model value apply from browser cookies and HTML5 local storage.  
			* @default false
			* @type {boolean}
			* @example 
			* // Initialize the TreeView with the persist value specified.
			* $("#treeView").ejTreeView({ persist: true });
			* @example 
			* //Get or set the persist API, after initialization:<br>
			*	// Get the persist API value.		
			*	$("#treeView").ejTreeView('option', 'persist');			
			*	// Set the persist API
			*	$("#treeView").ejTreeView('option', 'persist',  true);
			* @memberof ejTreeView
			* @instance
			*/
            persist: false,
            /**		
            * Allow to check the parent node automatically if this property is enabled.
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the TreeView with the enabled value specified.
            * $("#treeView").ejTreeView({ enabled: true });
            * @example 
            * //Get or set the enabled API, after initialization:<br>
            *	// Get the enabled API value.		
            *	$("#treeView").ejTreeView('option', 'enabled');			
            *	// Set the enabled API
            *	$("#treeView").ejTreeView('option', 'enabled',  true);
            * @memberof ejTreeView
            * @instance
            */
            enabled: true,
            /**		
            * Expanded nodes index collection given to integer array
            * @default false
            * @type {boolean}
            * @example 
            * // Initialize the TreeView with the expandedNodes value specified.
            * $("#treeView").ejTreeView({ expandedNodes: [0, 2, 5]  });
            * @example 
            * //Get or set the expandedNodes API, after initialization:<br>
            *	// Get the expandedNodes API value.		
            *	$("#treeView").ejTreeView('option', 'expandedNodes');			
            *	// Set the expandedNodes API
            *	$("#treeView").ejTreeView('option', 'expandedNodes', [0, 3, 5]  );
            * @memberof ejTreeView
            * @instance
            */
            expandedNodes: [],
            /**		
            * Defines the width of the TreeView.
            * @default Null
            * @type {string | number}
            * @example 
            * //Initialize the TreeView height property with the  value specified
            * 	$("#treeView").ejTreeView({ width: 30 });
            * @example 
            * Get or set the height API, after initialization:<br>
            *	//Gets the height  value
            *	$("#treeView").ejTreeView("option", "width");			
            *	//Sets the height  value
            *	$("#treeView").ejTreeView("option", "width ", 30 );
            * @memberof ejTreeView
            * @instance
            */
            width: null,
            /**		
            * Defines the height of the TreeView.
            * @default Null
            * @type {string | number}
            * @example 
            * //Initialize the TreeView height property with the  value specified
            * 	$("#treeView").ejTreeView({ height: 30 });
            * @example 
            * Get or set the height API, after initialization:<br>
            *	//Gets the height  value
            *	$("#treeView").ejTreeView("option", "height");			
            *	//Sets the height  value
            *	$("#treeView").ejTreeView("option", "height ", 30 );
            * @memberof ejTreeView
            * @instance
            */
            height: null,
            /**     
            * Fires when click successfully.
            * @event
            * @name ejTreeView#click 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //click event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	click: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            click: null,
            /**     
            * Fires when beforeExpand successfully.
            * @event
            * @name ejTreeView#beforeExpand 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //beforeExpand event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	beforeExpand: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            beforeExpand: null,
            /**     
            * Fires when expand successfully.
            * @event
            * @name ejTreeView#expand 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //expand event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	beforeExpand: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            expand: null,
            /**     
            * Fires when beforeCollapse successfully.
            * @event
            * @name ejTreeView#beforeCollapse 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //beforeCollapse event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	beforeCollapse: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            beforeCollapse: null,
            /**     
            * Fires when collapse successfully.
            * @event
            * @name ejTreeView#collapse 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //collapse event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	collapse: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            collapse: null,
            /**     
            * Fires when select successfully.
            * @event
            * @name ejTreeView#select 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //select event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	select: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            select: null,
            /**     
            * Fires when check successfully.
            * @event
            * @name ejTreeView#check 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //select event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	check: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            check: null,
            /**     
            * Fires when uncheck successfully.
            * @event
            * @name ejTreeView#uncheck 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //uncheck event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	uncheck: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            uncheck: null,
            /**     
            * Fires when inlineEditValidation successfully.
            * @event
            * @name ejTreeView#inlineEditValidation 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //inlineEditValidation event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	inlineEditValidation: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            inlineEditValidation: null,
            /**     
            * Fires when beforeEdit successfully.
            * @event
            * @name ejTreeView#beforeEdit 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //beforeEdit event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	beforeEdit: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            beforeEdit: null,
            /**     
            * Fires when keyPress successfully.
            * @event
            * @name ejTreeView#keyPress 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //keyPress event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	keyPress: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            keyPress: null,
            /**     
            * Fires when dragStart successfully.
            * @event
            * @name ejTreeView#dragStart 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //dragStart event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	dragStart: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            dragStart: null,
            /**     
            * Fires when drag successfully.
            * @event
            * @name ejTreeView#drag 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //drag event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	drag: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            drag: null,
            /**     
            * Fires when dragStop successfully.
            * @event
            * @name ejTreeView#dragStop 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //dragStop event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	dragStop: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            dragStop: null,
            /**     
            * Fires when dropped successfully.
            * @event
            * @name ejTreeView#dropped 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //dropped event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	dropped: function(args) {}
            * });      
            * @memberof ejTreeView
            * @instance
            */
            dropped: null
        },
        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            showCheckbox: "boolean",
            dragAndDrop: "boolean",
            dropChild: "boolean",
            dragAndDropAcrossControl: "boolean",
            allowEdit: "boolean",
            allowKeyboardNavigation: "boolean",
            autoCheckParentNode: "boolean",
            loadOnDemand: "boolean",
            rtl: "boolean",
            expandEvent: "string",
            persist: "boolean",
            items: "data",
            fields: "data",
            expandedNodes: "data"
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            /// <summary>This function is used to set the Treeview model</summary>
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "fields":
                        this._unWireEvents();
                        var f = this.element.hasClass("e-js") ? false : true, element = f ? this.element.children("ul") : this.element;
                        element.empty();
                        $.extend(this.model.fields, options[key]);
                        this._checkDataBinding();
                        if (f) {
                            var tempUl = this.element.children(".e-treeview-ul");
                            element.append(tempUl.children());
                            tempUl.remove();
                        }
                        break;
                    case "dragAndDropAcrossControl":
                        {
                            this.model.dragAndDropAcrossControl = options[key];
                            this._enableDargDrop();
                            break;
                        }
                    case "enabled": this._enabledAction(options[key]); break;
                    case "showCheckbox": {
                        if (options[key]) {
                            this.model.showCheckbox = options[key];
                            this._showCheckBox();
                        } else {
                            this.element.find('span.e-chkbox-wrap').next().remove();
                            this.element.find('span.e-chkbox-wrap').remove();
                        }
                        break;
                    }
                    case "expandedNodes": {
                        for (var i = 0; i < options[key].length; i++) {
                            this._expandNode(this._liList[options[key][i]]);
                        }
                        break;
                    }
                    case "expandEvent": {
                        this._on(this.element, options[key], this._expandEventHandler);
                        break;
                    }
                    case "allowEdit": {
                        if (options[key]) {
                            this._enableAllowEdit();
                            this._allowEditable();
                        }
                        else {
                            this._disableAllowEdit();
                            this._preventEditable();
                        }
                        break;
                    }
                    case "allowKeyboardNavigation": {
                        if (options[key])
                            this._on($(document), 'keydown', this._KeyPress);
                        else
                            $(document).unbind('keydown', this._KeyPress);
                        break;
                    }
                    case "dragAndDrop":
                        {
                            if (options[key])
                                this._enableDargDrop();
                            break;
                        }
                    case "rtl":
                        {
                            if (options[key])
                                this.element.addClass("e-rtl");
                            else
                                this.element.removeClass("e-rtl");
                            break;
                        }
                    case "height":
                        {
                            this.element.height(options[key]);
                            break;
                        }
                    case "width":
                        {
                            this.element.width(options[key]);
                            break;
                        }
                }
            }
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.html("");
            this.element.parent().append(this._cloneElement);
            this.element.remove();
            this.element = this._cloneElement;
            this.element.removeClass("e-treeview e-widget");
        },

        // constructor function
        _init: function () {

            this._id = this.element.prop("id");
            if (this.model.items != null) {
                var main = this._renderTemplate(this.model.items);
                this.element.append(main);
            }
            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) {
                this._checkDataBinding();
            } else
                this.initialize();

        },

        initialize: function () {
            this._cloneElement = $(this.element).clone();
            this._cutCopyNodeText = null;
            this.beforeEditText = null;
            if (this.element.is("ul")) {
                this._createWrapper();
            }
            else {
                this.element.addClass("e-treeview-wrap e-widget");
                if (this.model.width != null) {
                    this.element.width(this.model.width);
                }
                if (this.model.height != null) {
                    this.element.height(this.model.height);
                }
                if (this.model.rtl)
                    this.element.addClass("e-rtl");
            }
            this.element.attr("role", "tree").attr("aria-activedescendant", this._id + "_active");
            this._ulList = $("ul", this.element);
            this._liList = $("li", this.element).attr("tabindex", -1);
            this._addBaseClass();
            this._controlRender();
            this._addDragableClass();
            this._finalize();
            this._enabledAction(this.model.enabled);
        },

        _createWrapper: function () {
            var mainWidget = ej.buildTag("div.e-treeview-wrap e-widget " + this.model.cssClass, "", "", { tabindex: 0 });
            !ej.isNullOrUndefined(this.element.attr("accesskey")) && mainWidget.attr("accesskey", this.element.attr("accesskey"));
            if (this.model.width != null) {
                mainWidget.width(this.model.width);
            }
            if (this.model.height != null) {
                mainWidget.height(this.model.height);
            }
            if (this.model.rtl)
                mainWidget.addClass("e-rtl");
            mainWidget.insertAfter(this.element);
            mainWidget.append(this.element.addClass("e-ul"));
            this.element = mainWidget;
        },
        //Skin Change at run time
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        //Enable avtion chnaged.
        _enabledAction: function (flag) {
            if (flag) {
                this.element.removeClass("e-disable");
                this._wireEvents();
            }
            else {
                this.element.addClass("e-disable");
                this._unWireEvents();
            }
        },

        // data source checking
        _checkDataBinding: function () {
            if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                this._initDataSource();
            } else {
                this._ensureDataSource(this.model.fields["dataSource"]);
                this.initialize();
            }
        },

        _initDataSource: function () {
            this.element.ejWaitingPopup();
            var waitingPopup = this.element.ejWaitingPopup("instance");
            this.element.ejWaitingPopup("refresh");
            this.element.ejWaitingPopup("show");
            var query = this._columnToSelect(this.model.fields);
            var proxy = this;
            var queryPromise = this.model.fields["dataSource"].executeQuery(query);
            queryPromise.done(function (e) {
                proxy.element.ejWaitingPopup("hide");
                proxy.retriveData = e.result;
                proxy._ensureDataSource(e.result);
                proxy.initialize();
            });
        },

        //query check and create select based on the mapperField.

        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "child" && col !== "dataSource")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },

        // data source checking
        _ensureDataSource: function (dataSource) {
            this.currentSelectedData = dataSource;
            this.element.append(this._renderTemplate(dataSource));
        },

        //Render Treeview from template.

        _renderTemplate: function (item) {
            var ulTag = ej.buildTag("ul"), subItem = "";
            this._odataFlag = false;
            this._hasSelected = [];
            for (i = 0; i < item.length; i++) {
                if (item[i][this.model.fields.selected])
                    this._hasSelected = item[i][this.model.fields.id];
            }
            for (var i = 0; i < item.length; i++) {
                if (this.model.fields["dataSource"] != null) {
                    if (ej.isNullOrUndefined(item[i][this.model.fields.parentId]) || item[i][this.model.fields.parentId] == 0)
                        subItem = this._tempalteSub(item[i], item, this.model.fields);
                } else
                    subItem = this._tempalteSub(item[i], item, this.model.fields);
                ulTag.append(subItem);
            }
            return ulTag;
        },
        _tempalteSub: function (item, dataSource, mapper) {
            var liTag, spanTag, imgTag, aTag, childItems, ulTag, loadFlag = false;//, mapper = this.model.mapperField;
            if (this.model.fields["dataSource"] != null)
                if (this.model.loadOnDemand && item[mapper.hasChild]) {
                    childItems = ej.DataManager(this.currentSelectedData).executeLocal(ej.Query().where(mapper.parentId, ej.FilterOperators.equal, item[mapper.id]));
                    ulTag = $("<ul></ul>");
                    loadFlag = true;
                } else if (!ej.isNullOrUndefined(mapper.parentId))
                    childItems = ej.DataManager(this.currentSelectedData).executeLocal(ej.Query().where(mapper.parentId, ej.FilterOperators.equal, item[mapper.id]));
                else
                    childItems = item[mapper.childItems];
            if (ej.isNullOrUndefined(this.model.template)) {
                liTag = $('<li></li>');
                if (item[mapper.id])
                    liTag.prop("id", item[mapper.id]);
                if (item[mapper.htmlAttribute]) this._setAttributes(item[mapper.htmlAttribute], liTag);
                if (item[mapper.imageUrl]) {
                    imgTag = ej.buildTag('img.e-align');
                    imgTag.attr('src', item[mapper.imageUrl]);
                    if (item[mapper.imageAttribute]) this._setAttributes(item[mapper.imageAttribute], imgTag);
                    liTag.append(imgTag);
                }
                else if (item[mapper.spriteCssClass]) {
                    spanTag = ej.buildTag('span.' + item[mapper.spriteCssClass]);
                    liTag.append(spanTag);
                }
                if (item[mapper.text] != "") {
                    aTag = ej.buildTag('a', item[mapper.text], "", { href: "#" });
                    if (item[mapper.linkAttribute])
                        aTag.attr('href', item[mapper.linkAttribute]);
                    liTag.append(aTag);
                }
            }
            else {
                var divEle = $("<div></div>");
                divEle.append($("#" + this.model.template).render(item));
                liTag = divEle.find("li");
            }
            this._hasExpanded = false;
            if (item[mapper.expanded]) {
                liTag.addClass("expanded");
                this._hasExpanded = true;
            }
            item[mapper.isChecked] && liTag.addClass("ischecked");
            if (item[mapper.id] == this._hasSelected)
                aTag.addClass("e-text CanSelect e-active");
            if (loadFlag)
                liTag.append(ulTag);

            if (!ej.isNullOrUndefined(mapper["child"]) && !this.model.loadOnDemand) {
                this._odataFlag = true;
                if (mapper["child"]["dataSource"] instanceof ej.DataManager) {
                    var proxy = this, queryManager = ej.Query();
                    queryManager = this._columnToSelect(mapper["child"]);
                    queryManager.where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]);
                    var queryPromise = mapper["child"]["dataSource"].executeQuery(queryManager);
                    queryPromise.done(function (e) {
                        childItems = e.result;
                        if (childItems && childItems.length > 0) {
                            proxy._createSubNodesWhenLoadOnDemand(childItems, liTag, mapper["child"]);
                            $(liTag).find(" >div >div").first().addClass("e-icon e-plus");
                        }
                    });
                }
                else {
                    var childItems = ej.DataManager(mapper["child"]["dataSource"]).executeLocal(ej.Query().where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]));
                    if (childItems && childItems.length > 0) {
                        var ul = $(document.createElement('ul'));
                        for (var i = 0; i < childItems.length; i++) {
                            var liItem = this._tempalteSub(childItems[i], mapper["child"]["dataSource"], mapper["child"]);
                            ul.append(liItem);
                        }
                        liTag.append(ul);
                    }
                }
            }
            else if (this.model.loadOnDemand) {
                if (childItems && childItems.length > 0 && this._hasExpanded) {
                    for (var i = 0; i < childItems.length; i++) {
                        var liItem = this._tempalteSub(childItems[i], dataSource, mapper);
                        ulTag.append(liItem);
                    }
                    this._hasExpanded = false;
                    liTag.append(ulTag);
                }
            }
            else if (!this._odataFlag) {
                if (childItems && childItems.length > 0) {
                    var ul = ej.buildTag('ul');
                    for (var i = 0; i < childItems.length; i++) {
                        var liItem = this._tempalteSub(childItems[i], dataSource, mapper);
                        ul.append(liItem);
                    }
                    liTag.append(ul);
                }
            }
            return liTag;
        },
        _setAttributes: function (data, element) {
            for (var key in data) {
                element.attr(key, data[key]);
            }
        },
        _addDragableClass: function () {
            if (this.model.dragAndDrop) {
                this._anchors = this._liList.map(function () {
                    return $("a", this)[0];
                });
                this._anchors.addClass("e-draggable e-droppable");
            }
        },
        _addBaseClass: function () {
            this._ulList.addClass("e-treeview-ul").attr("role", "group");
            this._liList.addClass("e-item").attr("role", "treeitem");
            this.element.find("ul").first().removeClass("e-treeview-ul");
        },
        //Applies the autoformat
        _controlRender: function () {
            if (this.element != null) {
                var element = this.element.find("> ul");
                var licoll = element.find('li');
                var nodecount = -1;
                for (var i = 0; i < licoll.length; i++) {
                    var listEl = $(licoll[i]);
                    var textElement;
                    var customElement;
                    var nodeText;
                    //Check for sub-tree view items.Take back up and remove them
                    var subItems = listEl.children('ul')[0];
                    if (this.model.showCheckbox == true && $(subItems).length != 0) {
                        var parentli = element.find('>li');
                        parentli.each(function (liindex) {
                            if (($(subItems).parents('li.e-item').first())[0] == $(parentli[liindex])[0])
                                nodecount = nodecount + 1;
                        });
                        this._checkboxrecursive(element, subItems, nodecount);
                    }
                    if (subItems)
                        $(listEl.children('ul')[0]).remove();

                    //Check for text elements(a) and add class if any
                    var linkElement = listEl.children('a')[0];
                    if (linkElement) {
                        nodeText = $(linkElement).text();
                        textElement = $(linkElement).clone();
                        $(linkElement).remove();
                        $(textElement).addClass('e-text');
                        $(textElement).addClass('CanSelect');
                        customElement = listEl.clone();
                        listEl.html('');
                    }
                        //Otherwise build text elements(a)
                    else {
                        nodeText = this._getText(listEl);
                        customElement = listEl.clone();
                        listEl.html('');
                        textElement = ej.buildTag("a.e-text CanSelect", nodeText, "", { href: "#", alt: "" });
                    }

                    //Span to show the images for Themes
                    var span = ej.buildTag('span');
                    if (subItems) {
                        $(span).addClass('collapseimg');
                    }
                    else {
                        $(span).addClass('child');
                    }

                    // Element to show expand, collapse image
                    var exCollpasediv = ej.buildTag('div', "", {}, { role: "presentation" });

                    //Outter div element 
                    var outerdiv = ej.buildTag('div', "", {}, { role: "presentation" });
                    $(outerdiv).append(exCollpasediv).append(span).append(customElement.children()).append(textElement);
                    listEl.prepend(outerdiv);
                    //Check for sub-tree view items and append them
                    if (subItems)
                        listEl.append(subItems);

                    //Check if ID is null for the node, then set the NodeText as ID
                    if (listEl.prop('name') == undefined) {
                        listEl.prop('name', nodeText);
                    }
                }
                if (this.model.showCheckbox) {
                    this._addCheckBox(element);
                    this.element.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                }
            }

            var liCollection = element.find('li');
            liCollection.find('>ul').hide();
            var acollection = liCollection.find('a');
            acollection.focus(function () {
                $(this).blur();
            });
            //Adding the Background position Class for all first and last Parent Nodes   
            liCollection.filter('li:last-child').addClass('last');
            $(liCollection[0]).addClass('first');

            if ($(liCollection[0]).hasClass('first') && $(liCollection[0]).hasClass('last'))
                $(liCollection[0]).removeClass('first');

            $(liCollection.filter(':has(ul)')).each(function () {
                $(this).attr("aria-expanded", false).attr("aria-selected", false)
                var liHasul = $($(this).children()).filter('ul');
                if ($(liHasul).is(':hidden')) {
                    $($(this).children().find('div')[0]).removeClass('e-icon e-minus').addClass('e-icon e-plus');
                }
                else {
                    $(this.childNodes[1]).removeClass('e-icon e-plus').addClass('e-icon e-minus');
                }
            });
        },

        _getText: function (element) {
            return $(element)
                      .clone()    //clone the element
                      .children() //select all the children
                      .remove()   //remove all the children
                      .end()  //again go back to selected element
                      .text();    //get the text of element
        },

        _finalize: function () {
            //Onfocus flag.
            this._isFocus = false;
            //Expand the specific nodes.
            var expandList = this.element.find(".expanded");
            for (var i = 0; i < expandList.length; i++) {
                this._expandNode(expandList[i]);
            }
            //is Checked checkbox element.
            this.model.showCheckbox && this._isCheckedAction();

            //Expanded Nodes
            for (var i = 0; i < this.model.expandedNodes.length; i++) {
                this._expandNode(this._liList[this.model.expandedNodes[i]]);
            }

            //Allow Editing on AllowEdit = true
            this.model.allowEdit && this._enableAllowEdit();

            $(this._liList[0]).find("a.e-text").attr("id", this._id + "_active");

            this.element.attr("tabindex", 0);
        },

        _isCheckedAction: function () {
            var checkedList = this.element.find(".ischecked");
            for (var chk = 0; chk < checkedList.length; chk++) {
                this._nodeCheckedAction($(checkedList[chk]).find("span.e-chkbox-wrap").first()); //e-checkbox
            }
            this.element.find(".ischecked").removeClass("ischecked");
        },
        //Add Check box
        _addCheckBox: function (element) {
            var parentli = element.find('>li');
            for (var i = 0; i < parentli.length; i++) {
                var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + i, value: "False" });

                //Create a hidden Checkbox
                var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + 1, value: i });

                $(parentli[i]).children('div').children('span').first().after($(checkboxhdn));
                $(parentli[i]).children('div').children('span').first().after($(checkbox));
            }
        },

        _checkboxrecursive: function (element, subItems, nodecount) {
            var proxy = this, nodeindex;
            var ulcollection = $(subItems).find('>li');

            if ($($($(subItems).parents('li.e-item')[0]).find('>input[type= hidden]')[0]).length == 0)
                nodeindex = nodecount;
            else
                nodeindex = $($(subItems).parents('li.e-item')[0]).find('>input[type=hidden]')[0].value;

            ulcollection.each(function (ulindex) {
                var nodecountvalue = nodecount;
                var nodeindexvalue = nodeindex + "." + ulindex;
                proxy._checkboxrecursive(nodeindex, this, nodecountvalue);
                var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + nodeindexvalue, value: "False" });
                //Create a hidden Checkbox
                var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + nodeindexvalue, value: nodeindexvalue });
                var liEle = $(ulcollection[ulindex]);
                if (liEle.find("a.e-text ").length > 0) {
                    checkbox.insertBefore(liEle.find("a.e-text ")[0]);
                    checkboxhdn.insertBefore(liEle.find("a.e-text ")[0]);
                } else {
                    liEle.prepend(checkbox).prepend(checkboxhdn);
                }
            });
        },

        _checkedChange: function (args) {
            var treeview = $(this.element).closest(".e-treeview").data("ejTreeView");
            if (args.isChecked)
                treeview._nodeCheckedAction(this.element)
            else
                treeview._nodeUnCheckedAction(this.element)
        },

        //Click event handler
        _ClickEventHandler: function (event) {
            var proxy = this;
            var element = event.target;
            if (!this._isclicked) {
                this._isclicked = true;
                this._onClick({ Event: event });
                if (element.tagName.toUpperCase() == 'DIV') {
                    if ($(element).hasClass('e-plus') || $(element).hasClass('e-minus')) {
                        var expandUl = null;
                        if (element.tagName.toUpperCase() == 'SPAN') {
                            expandUl = $(element.parentNode).parent().children().filter('ul');
                            element = $(element.parentNode).children().filter('div')[0];
                        }
                        else {
                            expandUl = $(element.parentNode).parent().children().filter('ul');
                        }
                        var isExpanded = $(expandUl).children().filter('li').length > 0 ? true : false;
                        if (!isExpanded && this.model.loadOnDemand) {
                            $(element).addClass("e-load");
                            var nodeid = $($(element).parents('li.e-item')[0]).attr('id');
                            var nodeText = $($($($(element).parents('li.e-item')[0]).children('div')[0]).children('a[class*=e-text]')[0]).text();
                            var args = { id: nodeid, value: nodeText };
                            this._loadOnDemand(args, element);
                        }
                        else
                            this._expandCollapseAction(element);
                        this._nodeSelectionAction($(element).parents('li.e-item').first().find('a[class*=e-text]').first());
                    }
                }
                if (element.tagName.toUpperCase() == 'A') {
                    $(element).hasClass('CanSelect') && !$(element).hasClass('e-active') && this._nodeSelectionAction(element);
                }
            }


            setTimeout(function () { proxy._isclicked = false; }, 300);
            if (element.tagName.toUpperCase() == 'A') {
                ($(element).attr("href") == "#") && event.preventDefault();
            }

        },

        _loadOnDemand: function (args, element) {
            var childItems, proxy = this;;
            if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                var query = ej.Query().where(this.model.fields.parentId, ej.FilterOperators.equal, args.id);
                var queryPromise = this.model.fields["dataSource"].executeQuery(query);
                queryPromise.done(function (e) {
                    childItems = e.result;
                    if (childItems.length > 0) {
                        proxy._createSubNodesWhenLoadOnDemand(childItems, element, proxy.model.fields);
                        proxy._expandCollapseAction(element);
                    }
                });
            } else {
                childItems = ej.DataManager(this.currentSelectedData).executeLocal(ej.Query().where(this.model.fields.parentId, ej.FilterOperators.equal, args.id));
                (childItems.length > 0) && setTimeout(function () { proxy._createSubNodesWhenLoadOnDemand(childItems, element, proxy.model.fields); proxy._expandCollapseAction(element); }, 400);
            }
        },

        _getPath: function (element) {
            var path = $(element).prop('name');
            var ulelement = $(element).parents('li')[0];
            while (ulelement != null && ulelement.parentNode.id != this._id) {
                path = $(ulelement).prop('name') + '/' + path;
                ulelement = $(ulelement).parents('li')[0];
            }
            path = this._id + "/" + path;
            return path;
        },

        //Action on Node Selection
        _nodeSelectionAction: function (element) {
            if ($(element) == null) return;
            //var isExpanded = this._isExpanded(element);
            if ($($(element).parents('li')).hasClass('e-node-disable')) return;
            // remove previous selection
            $($(this.element).find('.e-active')).removeClass('e-active');
            $(this.element).find("#" + this._id + "_active").removeAttr("id");
            $(this.element).find('[aria-selected=true]').attr("aria-selected", false);
            $(element).addClass('e-active');
            var currentElement = $($(element).parents('li.e-item')[0]);
            currentElement.attr("aria-selected", true).find("a.e-text").attr("id", this._id + "_active");
            var nodeText = $(element).text();
            var data = { currentElement: currentElement, value: nodeText };
            this._onNodeSelect(data);

        },
        //Action on Node Selection
        _nodeUnSelectionAction: function (element) {
            if (element[0] == null) return;
            $(element).removeClass('e-active');
        },

        //Action on Node Selection
        _nodeEnableAction: function (element) {
            if (element[0] == null) return;
            element = $(element);
            this.model.showCheckbox && $($(element.parents('li.e-item')[0]).find('input')[0]).prop('disabled', false);
            $(element.parents('li.e-item')[0]).removeClass('e-node-disable');
            $(element.parents('li.e-item')[0]).find('a').removeClass('e-node-disable');
            $(element.parents('li.e-item')[0])[0].disabled = false;
        },

        //Action on Node Selection
        _nodeDisableAction: function (element) {
            if (element[0] == null) return;
            // collapse the parent node
            element = $(element);
            this._collpaseNode(element.parents('li.e-item')[0]);
            this.model.showCheckbox && $($(element.parents('li.e-item')[0]).find('input')[0]).prop('disabled', true);
            $(element.parents('li.e-item')[0]).addClass('e-node-disable');
            $(element.parents('li.e-item')[0]).find('a').addClass('e-node-disable');
            if ($(element.parents('li.e-item')[0]).find('a').hasClass('e-active'))
                $(element.parents('li.e-item')[0]).find('a').removeClass('e-active');
            $(element.parents('li.e-item')[0])[0].disabled = true;
        },
        //Expands or collapse the selected node
        _expandCollapseAction: function (element) {
            var expandUl;
            var proxy = this;
            if ($(element.parentNode).parent().hasClass('e-node-disable')) return;
            if (!ej.isNullOrUndefined(element.tagName) && element.tagName.toUpperCase() == 'SPAN') {
                expandUl = $(element.parentNode).parent().children().filter('ul');
                element = $(element.parentNode).children().filter('div')[0];
            }
            else {
                expandUl = $(element.parentNode).parent().children().filter('ul');
            }
            var elmt;
            if (element.tagName == "DIV")
                elmt = $(element.parentNode.parentNode)[0];
            else
                elmt = element;
            var liElement = $($(element).parents('li.e-item')[0]);
            if (!this._isExpanded(elmt)) {
                liElement.attr("aria-expanded", true).attr("tabindex", 0);
                var nodeText = $($($($(element).parents('li.e-item')[0]).children('div')[0]).children('a[class*=e-text]')[0]).text();
                var data = { currentElement: liElement, value: nodeText };
                if (this._onBeforeExpand(data) === true)
                    return false;
                $(expandUl).animate({ height: 'toggle' }, 350, 'linear', function () {
                    proxy._onNodeExpand(data);
                    proxy._addExpandedNodes(proxy._liList.index(liElement));
                });
                $(element).removeClass('e-icon e-plus').addClass('e-icon e-minus');
                $(element.parentNode.parentNode).addClass('e-collapse');
                $($(element.parentNode).children()[1]).removeClass('collapseimg').addClass('expandimg');
            }
            else {
                liElement.attr("aria-expanded", false).attr("tabindex", -1);
                var nodeText = $($($($(element).parents('li.e-item')[0]).children('div')[0]).children('a[class*=e-text]')[0]).text();
                var data = { currentElement: $($(element).parents('li.e-item')[0]), value: nodeText };
                if (this._onBeforeCollapse(data) === true)
                    return false;
                $(expandUl).animate({ height: 'toggle' }, 200, 'linear', function () {
                    proxy._onNodeCollapse(data);
                    proxy._removeExpandedNodes(proxy._liList.index(data.currentElement));
                });
                $(element).removeClass('e-icon e-minus').addClass('e-icon e-plus');
                if (element.parentNode != null) {
                    $(element.parentNode.parentNode).removeClass('e-collapse');
                    $($(element.parentNode).children()[1]).addClass('collapseimg').removeClass('expandimg');
                }
            }
        },

        _isExpanded: function (element) {
            if (element.tagName == 'A')
                element = $(element.parentNode.parentNode)[0];
            var isExpanded = this.isNodeExpanded(element);
            if ($($(element).children().filter("ul")) != null)
                if ($($(element).children().filter("ul").find('li')) != null)
                    isExpanded = $($(element).children().filter("ul").find('li')).is(':not(:hidden)');
                else
                    isExpanded = false;
            return isExpanded;
        },
        _isChecked: function (element) {
            var isChecked;
            if ($(element).val() === "True")
                isChecked = true;
            else
                isChecked = false;
            return isChecked;
        },

        _nodeCheckedAction: function (element) {
            //var checkboxClass = element.className;

            var proxy = this;
            var chkEle = $($(element).find('input.nodecheckbox[type=checkbox]')[0]);
            chkEle.addClass("checked").prop('value', 'True');
            chkEle.ejCheckBox({ "indeterminateState": false, "indeterminate": false, "check": true });
            this._onChecked(chkEle);
            $($(element).parents('li.e-item')[0]).children('ul').find('input.nodecheckbox[type=checkbox]').prop('checked', true).addClass("checked").prop('value', 'True');
            var chkCollection = $($(element).parents('li.e-item')[0]).children('ul').find('input.nodecheckbox[type=checkbox]');
            chkCollection.each(function (index) {
                $(this).ejCheckBox({ "check": true });
                // Client side checked events
                proxy._onChecked(this);
            });
            var autoCheckParentNode = this.model.autoCheckParentNode;
            var parentlicollection = $($(element).parents('li.e-item')[0]).parents('li.e-item');
            parentlicollection.each(function (index) {
                var childulcollection = $(parentlicollection[index]).children('ul').find('li.e-item');
                var count = $(parentlicollection[index]).children('ul').find('li.e-item').find('input.nodecheckbox[value="True"]').length;
                var chkEle = $($(parentlicollection[index]).find('input.nodecheckbox[type=checkbox]')[0]);
                if (count == childulcollection.length || autoCheckParentNode) {
                    chkEle.addClass("checked").prop('value', 'True');
                    $(chkEle).ejCheckBox({ "indeterminateState": false });
                    $(chkEle).ejCheckBox({ "indeterminate": false, "check": true, });
                    proxy._onChecked(chkEle);
                } else {
                    chkEle.ejCheckBox({ "indeterminate": true });
                    chkEle.ejCheckBox({ "indeterminateState": true });
                    chkEle.val("False");
                }
            });
            var checkedcoll = this.element.find('input.nodecheckbox[value="True"]');
            for (var i = 0; i < checkedcoll.length; i++) {
                var chkEle = $(checkedcoll[i]).parents('.e-chkbox-wrap');
                var value = $(chkEle.next()[0])[0].value;
                var nodeid = $($(checkedcoll[i]).parents('li.e-item')[0]).attr('id');
                var text = $(checkedcoll[i]).parents("li.e-item").first().find(".e-text").first().text();

                var checkboxhdnvalue = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + value + ".Value", value: nodeid });
                //Create a hidden Checkbox
                var checkboxhdntext = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + value + ".Text", value: text });
                try {
                    $(checkedcoll[i]).append($(checkboxhdnvalue));
                    $(checkedcoll[i]).append($(checkboxhdntext));
                }
                catch (err) {
                }
            }
        },
        _nodeUnCheckedAction: function (element) {
            var proxy = this;
            element = $(element);
            var chkEle = $($(element).find('input.nodecheckbox[type=checkbox]')[0]);
            chkEle.removeClass('checked').prop('value', 'False').children().remove();
            chkEle.ejCheckBox({ "indeterminate": false, "check": false });
            this._onUnChecked(chkEle);
            var unCheckedList = $(element.parents('li.e-item')[0]).children('ul').find('input.nodecheckbox[type=checkbox]');
            unCheckedList.removeClass("checked").prop('value', 'False').children().remove();
            unCheckedList.each(function (index) {
                $(this).ejCheckBox({ "check": false });
                proxy._onUnChecked(this);
            });
            var autoCheckParentNode = this.model.autoCheckParentNode;
            var parentlicollection = $(element.parents('li.e-item')[0]).parents('li.e-item');
            for (var index = 0; index < parentlicollection.length; index++) {
                var count = $(parentlicollection[index]).find('input.nodecheckbox[value="True"]').length;
                var chkEle = $($(parentlicollection[index]).find('input.nodecheckbox[type=checkbox]')[0]);
                if (count <= 0) {
                    chkEle.ejCheckBox({ "indeterminateState": false });
                    chkEle.ejCheckBox({ "indeterminate": false, "check": false });
                    //Client side unchecked event
                    this._onUnChecked(chkEle);
                }
                if (count > 1 && autoCheckParentNode) {
                    chkEle.addClass("checked").prop('value', 'True');
                    chkEle.ejCheckBox({ "check": true });
                }
                else if (count == 1 && autoCheckParentNode) {
                    chkEle.addClass("checked").prop('value', 'False');
                    chkEle.ejCheckBox({ "check": false });
                }
                else if (count > 0) {
                    chkEle.removeClass('checked').prop('value', 'False').children().remove();
                    chkEle.ejCheckBox({ "indeterminate": true });
                    chkEle.ejCheckBox({ "indeterminateState": true });
                }
            }
        },

        //Enables Edit
        _enableAllowEdit: function () {
            var element = this.element;
            var liCollection = $(element).find('li');
            liCollection.each(function () {
                $(this).addClass('AllowEdit');
            });
        },

        //Disables Edit
        _disableAllowEdit: function () {
            var liCollection = this.element.find('li');
            liCollection.each(function () {
                $(this).removeClass('AllowEdit');
            });
        },

        _expandNode: function (value) {
            if (value != null) {
                //if ($(element).find('ul').length > 0) {
                if (this.model.loadOnDemand)
                    this._expandCollapseAction($($(value).children('div')).children()[0]);
                else {
                    if ($($($(value).children('div')).children()[0]).hasClass('e-plus')) {
                        var isExpanded = $(value).children('ul').find('li').length > 0 ? true : false;
                        if (isExpanded) {
                            this._expandCollapseAction($($(value).children('div')).children()[0]);
                        }
                    }
                }
            }
            return true;
        },

        //Collapses the selected node
        _collpaseNode: function (value) {
            var element = value;
            if (element != null) {
                if ($(element).find('ul').length > 0) {
                    if ($($($(element).children('div')).children()[0]).hasClass('e-minus')) {
                        this._expandCollapseAction($($(element).children('div')).children()[0]);
                    }
                }
            }
        },

        //Expands all the nodes
        _expandAll: function () {
            var proxy = this;
            var element = this.element;
            var liCollection = $(element).find("li:not([class*='e-node-disable'])");
            var collapsedNodes = liCollection.find('>ul:hidden').parents('li.e-item');
            if (liCollection.find('>ul:hidden').find("li").length > 0)
                liCollection.find('>ul:hidden').animate({ height: 'toggle' }, 350);
            $(collapsedNodes).each(function () {
                if (!$(this).hasClass("e-node-disable")) {
                    var liElement = $($(this).parents('li.e-item')[0]);
                    var nodeText = $($(this).find(">div >a")[0]).text();
                    var data = { currentElement: liElement, value: nodeText };
                    if (proxy._onBeforeExpand(data) === true)
                        return false;
                    if ($(this).find('li').length > 0) {
                        $(this).addClass('expand');
                        $($($(this).children('div')[0]).children()[1]).removeClass('collapseimg').addClass('expandimg');
                        $($($(this).children('div')[0]).children()[0]).removeClass('e-icon e-plus').addClass('e-icon e-minus');
                    }
                    proxy._onNodeExpand(data);
                }
            });
        },

        //Collapse all the nodes
        _collapseAll: function () {
            var liCollection = this.element.find('li');
            this._collapseNodes(liCollection);
        },
        //Check all the nodes of the checkbox
        _checkAll: function () {
            if (this.model.showCheckbox) {
                var element = this.element;
                var liCollection = $(element).find("li");
                for (var i = 0; i < liCollection.length; i++) {
                    var checkboxelement = $(liCollection[i]).children().find('input.nodecheckbox')[0];
                    if ($(checkboxelement)[0].disabled)
                        $(checkboxelement).ejCheckBox("setModel", { check: false });
                    else {
                        $(checkboxelement).ejCheckBox("setModel", { check: true });
                        $(checkboxelement).attr("value", "True");
                        this._onChecked(checkboxelement);
                    }
                }
            }
            else
                alert("Please Enable the ShowCheckbox property");
        },
        _unCheckAll: function () {
            if (this.model.showCheckbox) {
                var element = this.element;
                var liCollection = $(element).find("li");
                for (var i = 0; i < liCollection.length; i++) {
                    var chkEle = $(liCollection[i]).find('input.nodecheckbox')[0];
                    $(chkEle).ejCheckBox("setModel", { check: false });
                    $(chkEle).attr("value", "False");
                    this._onUnChecked(chkEle);
                }
            }
            else
                alert("Please Enable the ShowCheckbox property");
        },

        _collapseNodes: function (liCollection) {
            var proxy = this;
            var _expandedNodes = liCollection.find('>ul:not(:hidden)').parents('li.e-item');
            liCollection.find('>ul:not(:hidden)').animate({ height: 'toggle' }, 350);
            $(_expandedNodes).each(function () {
                var nodeText = $($(this).find(">div >a")[0]).text();
                var data = { currentElement: this, value: nodeText };
                if (proxy._onBeforeCollapse(data) == true)
                    return false;
                //var isExpanded = false;
                if ($(this).find('li').length > 0) {
                    $(this).removeClass('e-collapse');
                    $($($(this).children('div')[0]).children()[1]).removeClass('expandimg').addClass('collapseimg');
                    $($($(this).children('div')[0]).children()[0]).removeClass('e-icon e-minus').addClass('e-icon e-plus');
                }
                proxy._onNodeCollapse(data);
            });
        },
        _showCheckBox: function () {
            var element = this.element.find("> ul");
            var licoll = element.find('li');
            for (var i = 0; i < licoll.length; i++) {
                var nodecount = -1;
                var listEl = licoll[i];
                //Check for sub-tree view items.Take back up and remove them
                var subItems = $(listEl).children('ul')[0];
                if (this.model.showCheckbox == true && $(subItems).length != 0) {
                    var parentli = element.find('>li');
                    parentli.each(function (liindex) {
                        if ($($(subItems).parents('li.e-item')[0])[0].id == $(parentli[liindex])[0].id)
                            nodecount = nodecount + 1;
                    });
                    this._checkboxrecursive(element, subItems, nodecount);
                }
            }
            this._addCheckBox(element);
            element.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
        },

        //Drag and Drop Events handlers

        //dragEventHandler
        _drag: function () {
            var proxy = this;
            var pre = false;
            var _clonedElement = null;
            var dragContainment = null;
            this._treeView = this.element.parent();
            if (!this.model.dragAndDropAcrossControl)
                dragContainment = this.element.parent();
            //else
            //    dragContainment = this.element.parent();
            $(this._treeView).find("ul li div a").ejDraggable({
                dragArea: dragContainment,
                clone: true,
                dragStart: function (args) {
                    var data = { target: args.target };
                    if (proxy._onDragStarts(data)) {
                        args.cancel = true;
                        _clonedElement && _clonedElement.remove();
                        return false;
                    }
                },
                drag: function (args) {
                    $(".e-sibling").remove();
                    var target = args.target;
                    var data = { target: target };
                    if (proxy._onDrag(data))
                        return false;
                    if ($(target).hasClass('e-droppable') || $(target).parent().hasClass('e-droppable')) {
                        document.body.style.cursor = '';
                        $(target).addClass("allowDrop");
                    }
                    else {
                        document.body.style.cursor = 'not-allowed';
                        $(target).removeClass('showline-hover');
                        $(target).removeClass('noline-hover');
                    }
                    if (target.nodeName != 'A') {
                        if (target.nodeName == 'UL')
                            target = $(target).children()[0];
                        if (target.nodeName != 'LI')
                            target = $(target).parent('.e-droppable')[0] || $(target).parent();
                        if (target.nodeName == 'LI' && $(target).hasClass("e-droppable") && proxy.model.dropSibling) {
                            var targetY = $(".e-treeview").has(target).offset().top + $(target).children().find('a')[0].offsetTop || -1;
                            var div = ej.buildTag('div.e-sibling');
                            if ($(target).hasClass("last")) {
                                $(target).append(div);
                            }
                            else if ($(target).hasClass("first"))
                                $(target).prepend(div);
                            else {
                                pre = args.event.pageY < targetY ? false : true;
                                pre ? $(target).prepend(div) : $(target).append(div);
                            }
                        }
                    }
                    else
                        $(".e-sibling").remove();
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        _clonedElement && _clonedElement.remove();
                        document.body.style.cursor = '';
                    }
                    var target = args.target;
                    if (target.className == "e-sibling") {
                        target = $(target).parent("li")[0];
                    }
                    var targetObj = proxy;
                    var position = pre ? "Before" : "After";
                    position = target.nodeName == 'A' ? "Over" : position;
                    var data = { target: target, position: position };
                    if (proxy._onDragStop(data))
                        return false;
                    $(args.element).removeClass("e-active");
                    if (target.nodeName == 'A' && proxy.model.dropChild && $(target).hasClass('e-droppable') || (target.nodeName == 'UL' && $(target).children().length == 0)) {
                        position = "Over";
                        if (($(args.element).parent().parent().has($(target)).length == 0) && ($(target).parent().parent().has($(args.element)).length == 0 || proxy._isDescendant($(target).parents("li:last").find('>ul>li'), $(args.element).parents("li:first")[0]))) {
                            proxy._dropAsChildNode(target, args.element, args.event);
                            targetObj._applyFirstLastChildClass();
                        }
                    }
                    else {
                        if (target.nodeName == 'UL')
                            target = $(target).children()[0];
                        if (target.nodeName != 'LI')
                            target = $(target).parent('.e-droppable')[0] || $(target).parent();
                        if (target.nodeName == 'LI' && proxy.model.dropSibling && $(target).hasClass('e-droppable')) {
                            $(".e-sibling").remove();
                            if ($(args.element).parent().parent().has($(target)).length < 1) {
                                proxy._dropAsSublingNode(target, args.element, pre, args.event);
                                targetObj._applyFirstLastChildClass();
                            }
                        }
                    }
                    $(".e-sibling").remove();
                    $(".allowDrop").removeClass("allowDrop");
                    if (!proxy.model.dropChild) {
                        _clonedElement && _clonedElement.remove();
                    }
                    var data = { target: target, event: args.event, dropedElement: args.element };
                    if (proxy._onDropped(data))
                        return false;
                    document.body.style.cursor = '';
                },
                helper: function (event, ui) {
                    _clonedElement = $(event.sender.target).clone().addClass("dragClone");
                    return _clonedElement.appendTo($("body"));
                }
            });
        },
        _isDescendant: function (src, target) {
            var match = true;
            $(src).each(function (i, item) {
                if (item == target) {
                    match = false;
                    return false;
                }
                else
                    match = true;
            });
            return match;
        },
        //DropChildEventHandler
        _childDrop: function () {
            $(this._treeView).find("ul li div a").ejDroppable({
                accept: $(this._treeView).find("ul li div a"),
                drop: function (event, ui) {
                    $(ui.helper).hide();
                }
            });
        },
        //DropSiblingEventHandler
        _siblingDrop: function () {
            $(this._treeView).find("ul li").ejDroppable({
                drop: function (event, ui) {
                    $(ui.helper).hide();
                }
            });
        },

        //DropAsSubling
        _dropAsSublingNode: function (target, element, pre, event) {
            var li = event.ctrlKey ? $(element).parent().parent().clone() : $(element).parent().parent();
            var parentNode = $(element).parents('li.e-item')[1];
            pre ? $(li).insertBefore(target) : $(li).insertAfter(target);
            this._checkBox(target, li);
            this._modifiCss(parentNode);
            this._applyFirstLastChildClass();
        },
        //DropAsChild
        _dropAsChildNode: function (target, element, event) {
            var li = event.ctrlKey ? $(element).parent().parent().clone() : $(element).parent().parent();
            var parentNode = $(element).parents('li.e-item')[1];
            if (target.nodeName == 'UL')
                $(target).append(li);
            else
                this._appendNode(target, li);
            this._checkBox($(target).parent().parent(), li);
            this._modifiCss(parentNode);
            this._applyFirstLastChildClass();
            this._expandNode($(target).parent().parent()[0]);
        },
        _appendNode: function (selectedNode, outerLi) {
            if (this._showline && selectedNode.length != 0) {
                if ($($($(selectedNode).parents('li.e-item')[0]).find('li')).hasClass('last'))
                    $($($(selectedNode).parents('li.e-item')[0]).find('li')).removeClass('last');
                $(outerLi).addClass('last');
            }
            if (selectedNode.length != 0) {
                if ($($(selectedNode).parents('li.e-item')[0]).find('ul')[0] == null) {
                    var outerUl = ej.buildTag('ul.e-treeview-ul');
                    $(outerUl).append($(outerLi));
                    if (!$($($(selectedNode).parents('li.e-item')[0]).find('div')[1]).hasClass('e-minus')) {
                        $($($(selectedNode).parents('li.e-item')[0]).find('div')[1]).addClass('e-icon e-minus');
                    }
                    $($(selectedNode).parents('li.e-item')[0]).append($(outerUl));
                }
                else
                    $($($(selectedNode).parents('li.e-item')[0]).find('ul')[0]).append($(outerLi));
            }
        },

        _checkBox: function (target, li) {
            if (!$(li).hasClass("nodecheckbox") && $(li).find(".nodecheckbox").length > 0) {
                $(li).find(".e-chkbox-wrap").next().remove();
                $(li).find(".e-chkbox-wrap").remove();
            }
            if ($(target).find(".e-checkbox").length > 0) {
                var element = $(".e-treeview").has(target).find('ul')[0];
                $(element).find(".e-checkbox").next().remove();
                $(element).find(".e-checkbox").remove();
                var context = this;
                var nodecount = -1;
                var licoll = $(element).find('li');
                licoll.each(function (index) {
                    var subItems = $(this).children('ul')[0];
                    if ($(subItems).length != 0) {
                        var parentli = $(element).find('>li');
                        parentli.each(function (liindex) {
                            if ($($(subItems).parents('li.e-item')[0])[0].id == $(parentli[liindex])[0].id)
                                nodecount = nodecount + 1;
                        });
                        context._childcheckbox(element, subItems, nodecount);
                    }
                });
                this._addCheckBox($(element));
                this.element.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                $(target).closest(".e-treeview").find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
            }
        },
        _childcheckbox: function (element, subItems, nodecount) {
            var proxy = this, nodeindex;
            var ulcollection = $(subItems).find('>li');

            if ($($($($(subItems).parents('li.e-item')[0]).children()[0]).find('>input[type= hidden]')[0]).length == 0)
                nodeindex = nodecount;
            else
                nodeindex = $($($(subItems).parents('li.e-item')[0]).children()[0]).find('>input[type= hidden]')[0].value;

            ulcollection.each(function (ulindex) {
                var nodecountvalue = nodecount;
                var nodeindexvalue = nodeindex + "." + ulindex;
                proxy._childcheckbox(nodeindex, this, nodecountvalue);

                var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + nodeindexvalue, value: "False" });

                //Create a hidden Checkbox
                var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox.Index", value: nodeindexvalue });

                $(ulcollection[ulindex]).children('div').children('span').after($(checkboxhdn));
                $(ulcollection[ulindex]).children('div').children('span').after($(checkbox));
            });
        },
        _modifiCss: function (parentNode) {
            if ($(parentNode).find('li').length == 0) {
                $(parentNode).find('ul').remove();
                $(parentNode).removeClass('e-collapse');
                $(parentNode).find('div.e-icon').removeClass('e-icon e-minus');
            }
            else {
                if (!$($(parentNode).find('li')[$(parentNode).find('li').length - 1]).hasClass('last'))
                    $($(parentNode).find('li')[$(parentNode).find('li').length - 1]).addClass('last');
            }
        },

        _applyFirstLastChildClass: function () {
            var element = this.element;
            var liCollection = element.find('li');
            liCollection.removeClass('first');
            liCollection.removeClass('last');
            liCollection.filter('li:last-child').addClass('last');
            $(liCollection[0]).addClass('first');
            if ($(liCollection[0]).hasClass('first') && $(liCollection[0]).hasClass('last'))
                $(liCollection[0]).removeClass('first');
        },

        _expandEventHandler: function (event) {
            var target = $(event.target);
            if (target.hasClass("e-icon")) return;
            if (!(event.type === "dblclick" && this.model.allowEdit)) {
                event.preventDefault();
                var selectedNode = $(target.siblings('div'));
                var liElement = target.parent().parent();

                if (selectedNode.hasClass('e-minus'))
                    this._collpaseNode(liElement);
                else {
                    if (this.model.loadOnDemand && selectedNode.find(".e-item").length == 0) {
                        this._isclicked = false;
                        this._ClickEventHandler({ target: selectedNode[0] });
                    }
                    else this._expandNode(liElement);
                }
            }
        },

        //Double Click Handler
        _inlineEdit: function (event) {
            event.preventDefault();
            if (event.target.tagName == 'A' && !$(event.target).hasClass("e-node-disable")) {
                if ($($(event.target).parents('li.e-item')[0]).hasClass('AllowEdit')) {
                    if ($(event.target).hasClass('e-text')) {
                        $(event.target).removeClass('e-active');
                        this._inlineEditAction(event.target);
                    }
                }
            }
            return false;
        },

        //Node Editing 
        _inlineEditAction: function (element) {

            var editTextBox = $('#Edit_Input')[0];
            if (editTextBox == null) {
                this._createEditTextBox(element);
            }
            else {
                editTextBox = $('#Edit_Input')[0];
                $(editTextBox.parentNode).html(editTextBox.value);
                $(editTextBox).remove();
                this._createEditTextBox(element);
            }
            $(element).css("display", "none");
            //this.set_Selected(true);
        },

        //Creates Edit texbox
        _createEditTextBox: function (values) {
            var argsData = { currentNode: values };
            if (this._trigger("beforeEdit", argsData))
                return false;
            var editTextBox = $('#Edit_Input')[0];
            if (editTextBox == null) {
                var textBox = ej.buildTag('Input.Input_Text#Edit_Input', "", "", { type: 'text', value: $.trim(values.innerHTML).replace(/\n\s+/g, " "), name: 'inplace_value' });
                $(textBox).width($(values).outerWidth() + 5);
                $(textBox).height($(values).outerHeight() - 2);
                this.beforeEditText = $(values).text();
                $(values).html('');
                $(values).before(textBox);
                var size = 0;
                editTextBox = $('#Edit_Input')[0];
                if (editTextBox.value.length == '')
                    size = 3;
                else
                    size = $(values).outerWidth() + 20;

                this._mousePositionAtEnd(editTextBox);
                this._currentEditableNode = values;
                var txtbox = $(editTextBox);
                //Keypress event handler for the edit textbox
                this._on(txtbox, 'keypress', this._editTextBoxKeyPress);
                //Keydown event handler for edit textbox
                this._on(txtbox, 'keydown', this, this._pressEscKey);

                // Focus out event handler for the edit textbox
                this._on(txtbox, 'blur', this._focusout);
            }
            return editTextBox;
        },

        _editTextBoxKeyPress: function (event) {
            event.target.size = event.target.value.length + 1;
        },

        _mousePositionAtEnd: function (ctl) {
            if (ctl.focus)
                ctl.focus();
            if (ctl.select)
                ctl.select();
            return true;
        },

        //Focus out event handler for Edit textbox
        _focusout: function (e) {

            var editTextBox = $('#Edit_Input')[0];
            var currentText = editTextBox.value;
            var id = $($(editTextBox).parents('li.e-item')[0]).attr('id');
            var requestArgs = {
                NodeID: id,
                OriginalText: this.beforeEditText,
                ModifiedText: currentText,
                UpdateChanges: true
            };

            if (this._trigger("inlineEditValidation", requestArgs) == true) {
                this._cancelAction(editTextBox);
            }
            else
                this._saveAction(editTextBox);
        },


        //Keydown event handler for edit textbox
        _pressEscKey: function (event) {
            event.cancelBubble = true;
            event.returnValue = false;
            var editTextBox = $('#Edit_Input')[0];

            if (editTextBox != null) {
                if (event.keyCode == 13) {
                    this._focusout();
                }

                if (event.keyCode == 27)
                    this._cancelAction(editTextBox);
            }
        },
        _onFocusHandler: function (event) {
            if (this.model.allowKeyboardNavigation && this._isFocus) {
                this.element.find(".e-active").removeClass("e-active");
                nextElement = this.element.find('li').first().find('a').first();
                $(nextElement).addClass('e-active');
                event.preventDefault();
                this._isFocus = false;
            }
        },

        //Key down event handler
        _KeyPress: function (e) {
            var code, node;
            var proxy = this;
            this._isFocus = true;
            var element = this.element.find("> ul");
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            if (code == 113) {
                e.preventDefault();
                this._isFocus = false;
                var currentlySelectedItem = element.find(".e-active")[0];
                if (currentlySelectedItem != null) {
                    if ($($(currentlySelectedItem).parents('li.e-item')[0]).hasClass('AllowEdit')) {
                        $(currentlySelectedItem).removeClass('e-active');
                        this._createEditTextBox(currentlySelectedItem);
                    }
                }
            }
            if (proxy.model.allowKeyboardNavigation && (proxy.element.find('#Edit_Input').length < 1)) {
                var nextElement, selectedItem;
                selectedItem = $($(element).find('.e-active').parents('li.e-item')[0]);
                var liVisible = $(element).find('li').not(':hidden');
                // Down arrow key(next node Move)
                if (code == 40 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    if (selectedItem.length > 0) {
                        e.preventDefault();
                        for (var i = 0; i < liVisible.length; i++) {
                            if ($($(liVisible[i]).find('a')[0]).hasClass('e-active')) {
                                if (i == liVisible.length - 1)
                                    this._nodeSelectionAction($(selectedItem.find('a')[0]).addClass('e-active'));
                                else {
                                    $($(liVisible[i]).find('a')[0]).removeClass('e-active');
                                    this._nodeSelectionAction($($(liVisible[i + 1]).find('a')[0]));
                                    break;
                                }
                            }
                        }
                        proxy._KeyPressEventHandler(nextElement, proxy, code);
                    }
                }
                    // Up arrow key(next node Move)
                else if (code == 38 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    if (selectedItem.length > 0) {
                        e.preventDefault();
                        for (var i = 0; i < liVisible.length; i++) {
                            if ($($(liVisible[i]).find('a')[0]).hasClass('e-active')) {
                                if (i == 0)
                                    this._nodeSelectionAction($(selectedItem.find('a')[0]).addClass('e-active'));
                                else {
                                    $($(liVisible[i]).find('a')[0]).removeClass('e-active');
                                    this._nodeSelectionAction($($(liVisible[i - 1]).find('a')[0]).addClass('e-active'));
                                    break;
                                }
                            }
                        }
                        proxy._KeyPressEventHandler(nextElement, proxy, code);
                    }
                }

                    //Right arrow key(Expand)
                else if (code == 39 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    var arrowIcon = $(selectedItem).children('div').find('div');
                    if (arrowIcon.hasClass('e-plus')) {
                        e.preventDefault();
                        if (this.model.loadOnDemand && selectedItem.find(".e-item").length == 0) {
                            this._ClickEventHandler({ target: arrowIcon[0] });
                        }
                        else this._expandNode(selectedItem);
                    }
                    else {
                        if ($(selectedItem).find("div.e-icon e-plus,div.e-icon e-minus").first().length > 0)
                            this.selectNode($(selectedItem).children("ul").find("li").first()[0]);
                        else {
                            var child = $(selectedItem).parent().find("li");
                            var index = child.index(selectedItem);
                            !ej.isNullOrUndefined(child[index + 1]) && $(selectedItem).children("ul").length > 0 && this.selectNode(child[index + 1]);
                        }
                    }
                    nextElement = $(selectedItem.find('a')[0]);
                    proxy._KeyPressEventHandler(nextElement, proxy, code);
                }

                    //Left arrow key(Collapse)
                else if (code == 37 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    var arrowIcon = $(selectedItem).children('div').find('div');
                    if (arrowIcon.hasClass('e-minus')) {
                        e.preventDefault();
                        this._collpaseNode(selectedItem);
                    }
                    else if ($(selectedItem).parents().first().parents("li").first().length > 0) {
                        $(selectedItem).find("a.e-active").first().removeClass('e-active');
                        $(selectedItem).parents().first().parents("li").first().find('a').first().addClass("e-active");

                    }
                    nextElement = $(selectedItem.find('a')[0]);
                    proxy._KeyPressEventHandler(nextElement, proxy, code);
                }

                    //Enter Key(Expand and Collapse)
                else if (code == 13) {
                    if (selectedItem.length > 0) {
                        var arrowIcon = $(selectedItem).children('div').find('div');
                        if (arrowIcon.hasClass('e-minus'))
                            this._collpaseNode(selectedItem);
                        else {
                            if (this.model.loadOnDemand && selectedItem.find(".e-item").length == 0) {
                                this._ClickEventHandler({ target: arrowIcon[0] });
                            }
                            else this._expandNode(selectedItem);
                        }
                        nextElement = $(selectedItem.find('a')[0]);
                        proxy._KeyPressEventHandler(nextElement, proxy, code);
                    }
                }
                    //Space Key(toCheck and Uncheck node)
                else if (code == 32) {
                    if (selectedItem.length > 0 && this.model.showCheckbox) {
                        e.preventDefault();
                        var chkBoxEle = selectedItem.find('input.nodecheckbox[type=checkbox]').first();
                        if (chkBoxEle[0].checked)
                            this._nodeUnCheckedAction(chkBoxEle.parent().parent());
                        else
                            this._nodeCheckedAction(chkBoxEle.parent().parent());
                    }
                }
                    //Del key to delete the e-active node
                else if (code == 46) {
                    if (selectedItem.length > 0) {
                        e.preventDefault();
                        this._removeNode(selectedItem.find("a.e-text").first());
                    }
                }
                else if (e && e.ctrlKey == true) {
                    //Cut the node
                    if (code == 88) {
                        var cutNodeLi;
                        if ($(selectedItem).hasClass('first')) {
                            cutNodeLi = $(selectedItem).removeClass('first').addClass('last');
                            $(selectedItem).next().addClass('first');
                        }
                        else if (!$(selectedItem).hasClass('last'))
                            cutNodeLi = $(selectedItem).addClass('last');
                        else {
                            cutNodeLi = $(selectedItem);
                            $(selectedItem).prev().addClass('last');
                        }
                        this._cutCopyNodeText = cutNodeLi;
                        proxy._removeNode();
                        this.element.focus();
                    }

                        //Copy the node
                    else if (code == 67) {
                        var copyNodeLi = $(selectedItem).clone();
                        this._cutCopyNodeText = copyNodeLi;
                    }
                        // Paste the node
                    else if (code == 86 && this._cutCopyNodeText != null) {
                        $(selectedItem).length === 0 && (selectedItem = this.element);
                        var addChildNode;
                        var childNode = $(this._cutCopyNodeText).clone();
                        if ($(childNode.find('a')[0]).hasClass('e-active'))
                            $(childNode.find('a')[0]).removeClass('e-active');
                        if ($(childNode).hasClass('first'))
                            addChildNode = $(childNode).removeClass('first').addClass('last');
                        else if (!$(childNode).hasClass('last'))
                            addChildNode = $(childNode).addClass('last');
                        else
                            addChildNode = $(childNode);

                        if (this.model.showCheckbox) {
                            var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" });
                            checkbox.insertAfter(childNode.find(".e-chkbox-wrap"));
                            childNode.find(".e-chkbox-wrap").remove();
                            childNode.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                        }
                        // wire mouse over event 
                        this._on(childNode.find(".e-text"), "mouseenter", this._mouseEnterEvent);
                        this._on(childNode.find(".e-text"), "mouseleave", this._mouseLeaveEvent);

                        if ($(selectedItem).children('div').children('span').hasClass('child'))
                            $(selectedItem).children('div').children('span').removeClass('child').addClass('expandimg');
                        if ($(selectedItem).children('ul').length != 0) {
                            $(selectedItem).children('ul').find('>li').last().removeClass('last');
                            $(selectedItem).find('>ul').append($(addChildNode));
                        }
                        else {
                            var wrapUl = ej.buildTag('ul.e-treeview-ul');
                            node = $(wrapUl).append($(addChildNode));
                            $(selectedItem).append($(node));
                            $(selectedItem).children('div').find('>div').first().addClass('e-icon e-minus');
                        }
                        var liCollection = $(selectedItem).parents('li.e-item');
                        liCollection.each(function (index) {
                            var licoll = $(liCollection[index]).children('ul').find('>li');
                            if (licoll.length == 1) {
                                $(licoll).addClass('last');
                            }
                        });
                        this._expandNode(selectedItem);
                    }
                }
                    //Home Key(Select the first node)
                else if (code == 36 && !e.ctrlKey && !e.altKey && !e.shiftKey) {

                    $($(selectedItem).find('a')[0]).removeClass('e-active');
                    nextElement = $($(element).find('>li')[0]).find('a')[0];
                    $(nextElement).addClass('e-active');
                    proxy._KeyPressEventHandler(nextElement, proxy, code);
                    e.preventDefault();
                }

                    //End Key(Select the last node)
                else if (code == 35 && !e.ctrlKey && !e.altKey && !e.shiftKey) {

                    $($(selectedItem).find('a')[0]).removeClass('e-active');
                    node = $(element).find('>li').last();
                    if ($(node).find('div').hasClass('e-minus')) {
                        nextElement = $($(node).find('li').last().find('a')[0]);
                        $(nextElement).addClass('e-active');
                    }
                    else {
                        nextElement = $(node.find('a')[0]);
                        $(nextElement).addClass('e-active');
                    }
                    proxy._KeyPressEventHandler(nextElement, proxy, code);
                    e.preventDefault();
                }

            }
        },
        //Remove the node
        _removeNode: function (node) {
            var selectedNode = ej.isNullOrUndefined(node) ? this.element.find('.e-active') : node;
            if (selectedNode != null) {
                var parentNode = $($(selectedNode).parents('li.e-item')[1]);
                $($(selectedNode).parents('li.e-item')[0]).remove();
                this._modifiCss(parentNode);
            }
        },
        _KeyPressEventHandler: function (nextElement, proxy, code) {
            var isExpanded;
            var element = $($(nextElement).parents('li.e-item')[0]);
            var path = proxy._getPath($(nextElement).parents('li.e-item')[0]);
            var nodeText = $(nextElement).text();
            if ($($(nextElement).parents('li.e-item')[0]).children('div').find('div').hasClass('e-minus'))
                isExpanded = true;
            else
                isExpanded = false;
            var argsData = { keyCode: code, currentElement: element, value: nodeText, isExpanded: isExpanded, path: path };
            this._trigger("keyPress", argsData);
        },

        //Document click event handler
        _documentClick: function (event) {
            if (event.target.id != 'Edit_Input')
                var editTextBox = $('#Edit_Input')[0];
            if (editTextBox != null) {
                $(editTextBox).next().html(editTextBox.value).removeAttr('style');
                $(editTextBox).remove();
                //this.element.focus();
            }
        },
        //Save Changes on the edited node
        _saveAction: function (values) {
            $(values).next().html(values.value).removeAttr('style');
            $(values).remove();
        },

        //Cancels the changes 
        _cancelAction: function (values) {
            $(values).next().html(this.beforeEditText).removeClass('e-node-hover').addClass('e-active').removeAttr('style');
            this._hideEditTextBox();
        },

        //Hides the edit textbox
        _hideEditTextBox: function () {
            var editTextBox = $('#Edit_Input')[0];
            if (editTextBox != null) {
                $(editTextBox).remove();
            }
        },

        _mouseEnterEvent: function (event) {
            if ($(event.target).hasClass("e-text") && !$(event.target).hasClass("e-node-disable"))
                $(event.target).addClass("e-node-hover");
        },
        _mouseLeaveEvent: function (event) {
            if ($(event.target).hasClass("e-node-hover"))
                $(event.target).removeClass("e-node-hover");
        },

        _createSubNodesWhenLoadOnDemand: function (jsonData, element, mapper) {
            ej.isNullOrUndefined(mapper) && (mapper = this.model.fields);
            var customElement, imgSpan;
            var context = this;
            var idField = mapper.id;
            var textField = mapper.text;
            var parentIdField = mapper.parentId;
            var childField = mapper.childItems;
            var hasChildField = mapper.hasChild;
            var imgField = mapper.imageUrl;
            var checkedField = mapper.isChecked;
            var spriteCssClass = mapper.spriteCssClass;
            var outerMainUl = ej.buildTag("ul.e-treeview-ul", "", { display: "none" });
            $.each(jsonData, function (i, val) {
                var textElement = ej.buildTag("a.e-text CanSelect", val[textField], "", { href: "#" });
                context._on($(textElement), "mouseenter", context._mouseEnterEvent);
                context._on($(textElement), "mouseleave", context._mouseLeaveEvent);

                // Element to show expand, collapse image
                var exCollpasediv = ej.buildTag('div');
                //Span to show the images for Themes
                var span = ej.buildTag('span');
                if (val[childField] || val[hasChildField]) {
                    $(span).addClass('collapseimg');
                    $(exCollpasediv).addClass("e-icon e-plus");
                }
                else {
                    $(span).addClass('child');
                }

                //To add image
                if (val[imgField] != null) {
                    var imgDiv = ej.buildTag('img.e-align');
                    imgDiv.src = (val[imgField].substr(0, 1) == "~") ? val[imgField].substr(1, val[imgField].length) : val[imgField];
                }

                if (context.model.showCheckbox) {
                    //Create a checkbox
                    var index = $($(element).parents('li.e-item')[0]).find('div').find('>input[type=hidden]')[0].value;

                    var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + index + '_' + i + ".Checked" });

                    //Create a hidden Checkbox
                    var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox.Index", value: index + '_' + i });

                    if (val[checkedField] != null && val[checkedField])
                        $(checkbox).prop("checked", true);
                    $(checkbox).prop("value", val[checkedField] != null ? "False" : val[checkedField]);
                }
                //Outter div element 
                var outerdiv = ej.buildTag('div');
                $(outerdiv).append(exCollpasediv);
                if (context.model.showCheckbox) {
                    $(outerdiv).append(checkbox);
                    $(outerdiv).append(checkboxhdn);
                }
                $(outerdiv).append(span);
                if (val[spriteCssClass] != null) {
                    var imgSpan = ej.buildTag('spane.' + val[spriteCssClass]);
                    $(outerdiv).append(imgSpan);
                }
                $(outerdiv).append(imgDiv);
                $(outerdiv).append(customElement);
                $(outerdiv).append(textElement);


                //Check for sub-tree view items and append them
                var outerli = ej.buildTag('li.e-item', "", "", { id: val[idField] });
                if (context.model.allowEdit)
                    outerli.addClass('AllowEdit');
                if (context.model.showCheckbox && val[checkedField] != null && val[checkedField])
                    outerli.addClass("ischecked");
                if (jsonData.length == (i + 1))
                    outerli.addClass('last');
                outerli.append($(outerdiv));

                var liHasul = $(outerli);
                if (val["ChildCount"] > 0) {
                    $($(liHasul).children().find('div')[0]).removeClass('e-icon e-minus').addClass('e-icon e-plus');
                }
                $(outerMainUl).append($(outerli));
            });
            this.element.find(" li#" + jsonData[0][parentIdField] + " ul ").remove();
            this.element.find(" li#" + jsonData[0][parentIdField] + "").append($(outerMainUl));
            var parentLi = this.element.find(" li#" + jsonData[0][parentIdField]);
            if ($(parentLi.find(".nodecheckbox[type=checkbox]")).prop('checked'))
                this._nodeCheckedAction($(parentLi).find("span.e-chkbox-wrap").first());
            //var licoll = $(outerMainUl).find('li');
            this.model.showCheckbox && $(outerMainUl).find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
            $(element).removeClass("e-load");
            this._isCheckedAction();
            //this._expandCollapseAction(element);
            //this.waitingPopup.HidePopUp();
        },

        _createTreeElement: function (nodeText, selectedNode, nodeUrl) {
            var value;
            if (this.model.showCheckbox) {
                if (!ej.isNullOrUndefined(selectedNode)) {
                    if ($(selectedNode.parents('li.e-item')[0]).children('ul').length == 0)
                        value = $($(selectedNode.parents('li.e-item')[0]).find('div')[0]).find('>input[type=hidden]')[0].value + '.' + 0;
                    else {
                        var prevlivalue = $($(selectedNode.parents('li.e-item')[0]).children('ul').find('li').last().find('div').find('>input[type=hidden]')[0]).val();
                        var splitvalue = prevlivalue.split('.');
                        var indexvalue = parseInt(splitvalue[splitvalue.length - 1]) + 1;
                        var previndexvalue = prevlivalue.substring(0, (prevlivalue.length) - 1);
                        value = previndexvalue + indexvalue;
                    }
                }
                else {
                    var list = this.element.find(">li");
                    value = list.length + 1;
                }

                var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + value + ".Checked", value: "False" });

                //Create a hidden Checkbox
                var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox.Index", value: value });

            }
            var textElement = ej.buildTag('a.e-text CanSelect', nodeText, "", { href: "#" });
            if (!ej.isNullOrUndefined(nodeUrl))
                textElement.attr("href", nodeUrl);
            var textNode = $(textElement);
            this._on(textNode, "mouseenter", this._mouseEnterEvent);
            this._on(textNode, "mouseleave", this._mouseLeaveEvent);
            //Span to show the images for Themes
            var span = ej.buildTag('span.child');

            // Element to show expand, collapse image
            var exCollpasediv = ej.buildTag('div');
            //Outter div element 
            var outerdiv = ej.buildTag('div');
            $(outerdiv).append(exCollpasediv);
            $(outerdiv).append(span);
            if (this.model.showCheckbox) {
                $(outerdiv).append(checkbox);
                $(outerdiv).append(checkboxhdn);
            }
            $(outerdiv).append(textElement);

            //Check for sub-tree view items and append them
            var outerli = ej.buildTag('li.e-item#' + nodeText);
            if (this.model.allowEdit)
                outerli.addClass('AllowEdit');
            outerli.append($(outerdiv));

            return $(outerli);
        },
        _addExpandedNodes: function (index) {
            var _nodes = this.model.expandedNodes;
            if ($.inArray(index, _nodes) == -1) {
                this.model.expandedNodes.push(index);
            }
        },

        _removeExpandedNodes: function (index) {
            var _nodes = this.model.expandedNodes;
            if ($.inArray(index, _nodes) > -1) {
                this.model.expandedNodes.splice($.inArray(index, _nodes), 1);
            }
        },

        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.element, "click", this._ClickEventHandler);
            this._on(this.element.find(".e-text"), "mouseenter", this._mouseEnterEvent);
            this._on(this.element.find(".e-text"), "mouseleave", this._mouseLeaveEvent);
            if (this.model.allowEdit) this._allowEditable();
            this._on(this.element, this.model.expandEvent, this._expandEventHandler);
            this._on($(document), 'keydown', this._KeyPress);
            this._on(this.element, "focus", this._onFocusHandler);
            this._enableDargDrop();
        },

        //-------------------- Event UnWire-up -------------------------//
        _unWireEvents: function () {
            this._off(this.element, "click");
            this._off(this.element.find(".e-text"), "mouseenter");
            this._off(this.element.find(".e-text"), "mouseleave");
            this._preventEditable();
            this._off(this.element, this.model.expandEvent);
            this._off($(document), 'keydown');
            this._on(this.element, "focus");
        },

        //Darg and Drop Events

        _enableDargDrop: function () {
            if (this.model.dragAndDrop) {
                this._drag();
                if (this.model.dropChild)
                    this._childDrop();
                if (this.model.dropSibling)
                    this._siblingDrop();
            }
        },

        //Allow Editable option  on nodes

        _allowEditable: function () {
            this._on($(document), 'click', this._documentClick);
            this._on(this.element.parent(), 'dblclick', this._inlineEdit);
        },

        // Prevent Editable option on nodes

        _preventEditable: function () {
            this._off($(document), 'click');
            this._off(this.element.parent(), 'dblclick');
        },
        //--------------------- Client side Methods ----------------------//

        /**
        * To refresh the TreeView   
        * @example 
        *  * &lt;ul id="treeView"&gt;
            &lt;li&gt;Artwork
                &lt;ul&gt;
                    &lt;li&gt;Abstract
                        &lt;ul&gt;
                            &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                            &lt;li&gt;Creative Acrylic&lt;/li&gt;
                            &lt;li&gt;Modern Painting&lt;/li&gt;
                            &lt;li&gt;Canvas Art&lt;/li&gt;
                            &lt;li&gt;Black white&lt;/li&gt;
                        &lt;/ul&gt;
                    &lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Books
                &lt;ul&gt;
                    &lt;li&gt;Entertaining&lt;/li&gt;
                    &lt;li&gt;Design&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Music
                &lt;ul&gt;
                    &lt;li&gt;Mass&lt;/li&gt;
                    &lt;li&gt;Folk&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.refresh(); // refresh the TreeView
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        refresh: function () {
            this._unWireEvents();
            this._init();
        },
        /**
        * To expand all the TreeView nodes.
        * @example 
        *  * &lt;ul id="treeView"&gt;
            &lt;li&gt;Artwork
                &lt;ul&gt;
                    &lt;li&gt;Abstract
                        &lt;ul&gt;
                            &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                            &lt;li&gt;Creative Acrylic&lt;/li&gt;
                            &lt;li&gt;Modern Painting&lt;/li&gt;
                            &lt;li&gt;Canvas Art&lt;/li&gt;
                            &lt;li&gt;Black white&lt;/li&gt;
                        &lt;/ul&gt;
                    &lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Books
                &lt;ul&gt;
                    &lt;li&gt;Entertaining&lt;/li&gt;
                    &lt;li&gt;Design&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Music
                &lt;ul&gt;
                    &lt;li&gt;Mass&lt;/li&gt;
                    &lt;li&gt;Folk&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.expandAll(); // expandAll the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        expandAll: function () {
            this.model.enabled && this._expandAll();
        },
        /**
        * To expand all the TreeView nodes.
        * @example 
        *  * &lt;ul id="treeView"&gt;
            &lt;li&gt;Artwork
                &lt;ul&gt;
                    &lt;li&gt;Abstract
                        &lt;ul&gt;
                            &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                            &lt;li&gt;Creative Acrylic&lt;/li&gt;
                            &lt;li&gt;Modern Painting&lt;/li&gt;
                            &lt;li&gt;Canvas Art&lt;/li&gt;
                            &lt;li&gt;Black white&lt;/li&gt;
                        &lt;/ul&gt;
                    &lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Books
                &lt;ul&gt;
                    &lt;li&gt;Entertaining&lt;/li&gt;
                    &lt;li&gt;Design&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Music
                &lt;ul&gt;
                    &lt;li&gt;Mass&lt;/li&gt;
                    &lt;li&gt;Folk&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.collapseAll(); // collapseAll the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        collapseAll: function () {
            this.model.enabled && this._collapseAll();
        },
        /**
        * To check all the TreeView nodes.
        * @example 
        *  * &lt;ul id="treeView"&gt;
            &lt;li&gt;Artwork
                &lt;ul&gt;
                    &lt;li&gt;Abstract
                        &lt;ul&gt;
                            &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                            &lt;li&gt;Creative Acrylic&lt;/li&gt;
                            &lt;li&gt;Modern Painting&lt;/li&gt;
                            &lt;li&gt;Canvas Art&lt;/li&gt;
                            &lt;li&gt;Black white&lt;/li&gt;
                        &lt;/ul&gt;
                    &lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Books
                &lt;ul&gt;
                    &lt;li&gt;Entertaining&lt;/li&gt;
                    &lt;li&gt;Design&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Music
                &lt;ul&gt;
                    &lt;li&gt;Mass&lt;/li&gt;
                    &lt;li&gt;Folk&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.checkAll(); // checkAll the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        checkAll: function () {
            if (this.model.showCheckbox && this.model.enabled) {
                this._checkAll();
            }
        },
        /**
        * To uncheck all the TreeView nodes.
        * @example 
        *  * &lt;ul id="treeView"&gt;
            &lt;li&gt;Artwork
                &lt;ul&gt;
                    &lt;li&gt;Abstract
                        &lt;ul&gt;
                            &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                            &lt;li&gt;Creative Acrylic&lt;/li&gt;
                            &lt;li&gt;Modern Painting&lt;/li&gt;
                            &lt;li&gt;Canvas Art&lt;/li&gt;
                            &lt;li&gt;Black white&lt;/li&gt;
                        &lt;/ul&gt;
                    &lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Books
                &lt;ul&gt;
                    &lt;li&gt;Entertaining&lt;/li&gt;
                    &lt;li&gt;Design&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Music
                &lt;ul&gt;
                    &lt;li&gt;Mass&lt;/li&gt;
                    &lt;li&gt;Folk&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.unCheckAll(); // uncheckAll the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        unCheckAll: function () {
            if (this.model.showCheckbox && this.model.enabled) {
                this._unCheckAll();
            }
        },
        /**
        * To get the selected nodes in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
            &lt;li&gt;Artwork
                &lt;ul&gt;
                    &lt;li&gt;Abstract
                        &lt;ul&gt;
                            &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                            &lt;li&gt;Creative Acrylic&lt;/li&gt;
                            &lt;li&gt;Modern Painting&lt;/li&gt;
                            &lt;li&gt;Canvas Art&lt;/li&gt;
                            &lt;li&gt;Black white&lt;/li&gt;
                        &lt;/ul&gt;
                    &lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Books
                &lt;ul&gt;
                    &lt;li&gt;Entertaining&lt;/li&gt;
                    &lt;li&gt;Design&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Music
                &lt;ul&gt;
                    &lt;li&gt;Mass&lt;/li&gt;
                    &lt;li&gt;Folk&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.getSelectedNode(); // getSelectedNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Get Selected node
        getSelectedNode: function () {
            var selectedNodeElement = this.element.find('.e-active');
            var selectedNode = $($(selectedNodeElement).parents('li.e-item')[0]);
            return selectedNode;
        },
        /**
        * To get the text of the nodes in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
            &lt;li&gt;Artwork
                &lt;ul&gt;
                    &lt;li&gt;Abstract
                        &lt;ul&gt;
                            &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                            &lt;li&gt;Creative Acrylic&lt;/li&gt;
                            &lt;li&gt;Modern Painting&lt;/li&gt;
                            &lt;li&gt;Canvas Art&lt;/li&gt;
                            &lt;li&gt;Black white&lt;/li&gt;
                        &lt;/ul&gt;
                    &lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Books
                &lt;ul&gt;
                    &lt;li&gt;Entertaining&lt;/li&gt;
                    &lt;li&gt;Design&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
            &lt;li&gt;Music
                &lt;ul&gt;
                    &lt;li&gt;Mass&lt;/li&gt;
                    &lt;li&gt;Folk&lt;/li&gt;
                &lt;/ul&gt;
            &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.getText(string nodename); // getText in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        getText: function (node) {
            if (ej.isNullOrUndefined(node)) return false;
            var nodeText = $($(node).find('a')[0]).text();
            return nodeText;
        },
        /**
        * To select  the node in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.selectNode(string nodename); // selectNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Select the node
        selectNode: function (node) {
            var element;
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                if ($(node).hasClass("e-text")) {
                    element = $(node);
                }
                else
                    element = $($(node).find(">div >a")[0]);
                this._nodeSelectionAction(element);
            }
        },
        /**
        * To unselect  the node in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.unSelectNode(string nodename); // unSelectNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //unselected node
        unSelectNode: function (node) {
            var element;
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                if ($(node).hasClass("e-text")) {
                    element = $(node);
                }
                else
                    element = $($(node).find(">div >a")[0]);
                this._nodeUnSelectionAction(element);
            }
        },
        /**
        * To enable the node in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.enableNode(string nodename); // enableNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Enable the node
        enableNode: function (node) {
            var element;
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                if ($(node).hasClass("e-text")) {
                    element = $(node);
                }
                else
                    element = $($(node).find("a.e-text")[0]);
                this._nodeEnableAction(element);
                $(node).parent().find(".nodecheckbox").ejCheckBox("enable");
            }
        },
        /**
        * To disable the node in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.disableNode(string nodename); // disableNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Disable node
        disableNode: function (node) {
            var element;
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                if ($(node).hasClass("e-text")) {
                    element = $(node);
                }
                else
                    element = $($(node).find(">div >a")[0]);
                this._nodeDisableAction(element);
                $(node).find(".nodecheckbox").ejCheckBox("disable");
            }
        },
        /**
        * To add Node Collection in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.addNodeCollection(object collection,string targetnode); // addNodeCollection in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        addNodeCollection: function (collection, targetNode) {
            if (collection && collection.length > 0) {
                var mapper = this.model.fields;

                for (var i = 0; i < collection.length; i++) {
                    var node = collection[i], text = node[mapper.text];

                    if (text) {
                        var id = node[mapper.id], pid = node[mapper.parentId],
                        url = node[mapper.linkAttribute] ? node[mapper.linkAttribute] : "#",
                        parent = (targetNode && targetNode.length != 0) ? targetNode : null;

                        if (ej.isNullOrUndefined(parent)) {
                            var tar = pid ? this.element.find("#" + pid) : null;
                            parent = (tar && tar.length != 0) ? tar.children("div").find("a") : null;
                        }
                        if (ej.isNullOrUndefined(parent)) continue;

                        var nodeElement = this._createTreeElement(text, null, url);
                        id ? nodeElement.attr("id", id) : nodeElement.removeAttr("id");
                        this._appendNode(parent, nodeElement);
                        nodeElement.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                        if (this.model.dragAndDrop) nodeElement.find("a").addClass("e-draggable e-droppable");
                    }
                }
            }
        },
        /**
        * To add Node in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.addNode(newNodeText, nodeUrl, targetNode); // addNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Add the node
        addNode: function (newNodeText, nodeUrl, targetNode) {
            if (this.model.enabled) {
                var outerLi = null;
                var innerUl = null;
                var element = this.element;
                var childCount = element.children().length;
                var selectedNode = ej.isNullOrUndefined(targetNode) ? element.find('.e-active') : targetNode;
                if (selectedNode.length == 0) {
                    if (childCount == 0 || element.children().children().length == 0)
                        innerUl = ej.buildTag('ul');
                    else
                        innerUl = element.children()[childCount - 1];
                    outerLi = this._createTreeElement(newNodeText, null, nodeUrl);
                    $(innerUl).append(outerLi);
                    this._appendNode(selectedNode, innerUl);
                    $(outerLi).find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                }
                else {
                    this._expandNode($($(selectedNode).parents('li.e-item')[0]));
                    outerLi = this._createTreeElement(newNodeText, selectedNode, nodeUrl);
                    this._appendNode(selectedNode, outerLi);
                    this.model.showCheckbox && $(outerLi).find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                }
                this._modifiCss();
            }
        },
        /**
        * To remove Node in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.removeNode(node); // removeNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Remove the node
        removeNode: function (node) {
            if (this.model.enabled) {
                if (!ej.isNullOrUndefined(node)) {
                    if ($(node).hasClass("e-text")) {
                        element = $(node);
                    }
                    else
                        element = $($(node).find(">div >a")[0]);
                }
                this._removeNode(element);
            }
        },
        /**
        * To getAllCheckedNodes in the TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.getAllCheckedNodes(node); // getAllCheckedNodes in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Get all the checked  nodes
        getAllCheckedNodes: function () {
            var nodeCollection = this.element.find('input.nodecheckbox[value="True"]').closest('li');
            return nodeCollection;
        },
        /**
        * To Check particular node checkbox in TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.checkNode(node); // checkNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Check particular node checkbox
        checkNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                this._nodeCheckedAction($(node).find("span.e-chkbox-wrap").first());
            }
        },
        /**
        * To Uncheck particular node checkbox in TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.checkNode(node); // uncheckNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //UnCheck particular node textbox
        unCheckNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                this._nodeUnCheckedAction($(node).find("span.e-chkbox-wrap").first());
            }
        },
        /**
        * To expandNode particular node  in TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.expandNode(node); // expandNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        expandNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                this._expandNode(node);
            }
        },
        /**
        * To collapseNode particular node  in TreeView.
        * @return jQuery
        * @example 
        *  * &lt;ul id="treeView"&gt;
           &lt;li&gt;Artwork
               &lt;ul&gt;
                   &lt;li&gt;Abstract
                       &lt;ul&gt;
                           &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                           &lt;li&gt;Creative Acrylic&lt;/li&gt;
                           &lt;li&gt;Modern Painting&lt;/li&gt;
                           &lt;li&gt;Canvas Art&lt;/li&gt;
                           &lt;li&gt;Black white&lt;/li&gt;
                       &lt;/ul&gt;
                   &lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Books
               &lt;ul&gt;
                   &lt;li&gt;Entertaining&lt;/li&gt;
                   &lt;li&gt;Design&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
           &lt;li&gt;Music
               &lt;ul&gt;
                   &lt;li&gt;Mass&lt;/li&gt;
                   &lt;li&gt;Folk&lt;/li&gt;
               &lt;/ul&gt;
           &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.collapseNode(node); // collapseNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Collapses the selected node
        collapseNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                this._collpaseNode(node);
            }
        },
        /**
        * To showNode particular node  in TreeView.
        * @example 
        *  * &lt;ul id="treeView"&gt;
          &lt;li&gt;Artwork
              &lt;ul&gt;
                  &lt;li&gt;Abstract
                      &lt;ul&gt;
                          &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                          &lt;li&gt;Creative Acrylic&lt;/li&gt;
                          &lt;li&gt;Modern Painting&lt;/li&gt;
                          &lt;li&gt;Canvas Art&lt;/li&gt;
                          &lt;li&gt;Black white&lt;/li&gt;
                      &lt;/ul&gt;
                  &lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Books
              &lt;ul&gt;
                  &lt;li&gt;Entertaining&lt;/li&gt;
                  &lt;li&gt;Design&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Music
              &lt;ul&gt;
                  &lt;li&gt;Mass&lt;/li&gt;
                  &lt;li&gt;Folk&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.showNode(node); // showNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        showNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                node.css("visibility", "");
            }
        },
        /**
        * To hideNode particular node  in TreeView.
        * @example 
        *  * &lt;ul id="treeView"&gt;
          &lt;li&gt;Artwork
              &lt;ul&gt;
                  &lt;li&gt;Abstract
                      &lt;ul&gt;
                          &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                          &lt;li&gt;Creative Acrylic&lt;/li&gt;
                          &lt;li&gt;Modern Painting&lt;/li&gt;
                          &lt;li&gt;Canvas Art&lt;/li&gt;
                          &lt;li&gt;Black white&lt;/li&gt;
                      &lt;/ul&gt;
                  &lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Books
              &lt;ul&gt;
                  &lt;li&gt;Entertaining&lt;/li&gt;
                  &lt;li&gt;Design&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Music
              &lt;ul&gt;
                  &lt;li&gt;Mass&lt;/li&gt;
                  &lt;li&gt;Folk&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.hideNode(node); // hideNode in  TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        hideNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                node.css("visibility", "hidden");
            }
        },
        /**
        * To show nodes  in TreeView.
        * @example 
        *  * &lt;ul id="treeView"&gt;
          &lt;li&gt;Artwork
              &lt;ul&gt;
                  &lt;li&gt;Abstract
                      &lt;ul&gt;
                          &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                          &lt;li&gt;Creative Acrylic&lt;/li&gt;
                          &lt;li&gt;Modern Painting&lt;/li&gt;
                          &lt;li&gt;Canvas Art&lt;/li&gt;
                          &lt;li&gt;Black white&lt;/li&gt;
                      &lt;/ul&gt;
                  &lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Books
              &lt;ul&gt;
                  &lt;li&gt;Entertaining&lt;/li&gt;
                  &lt;li&gt;Design&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Music
              &lt;ul&gt;
                  &lt;li&gt;Mass&lt;/li&gt;
                  &lt;li&gt;Folk&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.show(); // show the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        show: function () {
            /// <summary>To Show the TreeView control</summary>
            this.element.css("visibility", "");
        },
        /**
        * To show nodes  in TreeView.
        * @example 
        *  * &lt;ul id="treeView"&gt;
          &lt;li&gt;Artwork
              &lt;ul&gt;
                  &lt;li&gt;Abstract
                      &lt;ul&gt;
                          &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                          &lt;li&gt;Creative Acrylic&lt;/li&gt;
                          &lt;li&gt;Modern Painting&lt;/li&gt;
                          &lt;li&gt;Canvas Art&lt;/li&gt;
                          &lt;li&gt;Black white&lt;/li&gt;
                      &lt;/ul&gt;
                  &lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Books
              &lt;ul&gt;
                  &lt;li&gt;Entertaining&lt;/li&gt;
                  &lt;li&gt;Design&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Music
              &lt;ul&gt;
                  &lt;li&gt;Mass&lt;/li&gt;
                  &lt;li&gt;Folk&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.hide(); // hide the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        hide: function () {
            /// <summary>To Hide the TreeView control</summary>
            this.element.css("visibility", "hidden");
        },
        /**
        * To retrive the checked status of the given TreeView node.
        * @example 
        *  * &lt;ul id="treeView"&gt;
          &lt;li&gt;Artwork
              &lt;ul&gt;
                  &lt;li&gt;Abstract
                      &lt;ul&gt;
                          &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                          &lt;li&gt;Creative Acrylic&lt;/li&gt;
                          &lt;li&gt;Modern Painting&lt;/li&gt;
                          &lt;li&gt;Canvas Art&lt;/li&gt;
                          &lt;li&gt;Black white&lt;/li&gt;
                      &lt;/ul&gt;
                  &lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Books
              &lt;ul&gt;
                  &lt;li&gt;Entertaining&lt;/li&gt;
                  &lt;li&gt;Design&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Music
              &lt;ul&gt;
                  &lt;li&gt;Mass&lt;/li&gt;
                  &lt;li&gt;Folk&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.isNodeChecked(element); // isNodeChecked the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        isNodeChecked: function (element) {
            return this._isChecked($(element).find('input.nodecheckbox')[0]);
        },
        /**
        * To retrive the expand status of the given TreeView node.
        * @example 
        *  * &lt;ul id="treeView"&gt;
          &lt;li&gt;Artwork
              &lt;ul&gt;
                  &lt;li&gt;Abstract
                      &lt;ul&gt;
                          &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                          &lt;li&gt;Creative Acrylic&lt;/li&gt;
                          &lt;li&gt;Modern Painting&lt;/li&gt;
                          &lt;li&gt;Canvas Art&lt;/li&gt;
                          &lt;li&gt;Black white&lt;/li&gt;
                      &lt;/ul&gt;
                  &lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Books
              &lt;ul&gt;
                  &lt;li&gt;Entertaining&lt;/li&gt;
                  &lt;li&gt;Design&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Music
              &lt;ul&gt;
                  &lt;li&gt;Mass&lt;/li&gt;
                  &lt;li&gt;Folk&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.isExpanded(element); // isExpanded the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        isExpanded: function (element) {
            return this._isExpanded(element);
        },
        /**
        * To retrive the expand status of the given TreeView node.
        * @example 
        *  * &lt;ul id="treeView"&gt;
          &lt;li&gt;Artwork
              &lt;ul&gt;
                  &lt;li&gt;Abstract
                      &lt;ul&gt;
                          &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                          &lt;li&gt;Creative Acrylic&lt;/li&gt;
                          &lt;li&gt;Modern Painting&lt;/li&gt;
                          &lt;li&gt;Canvas Art&lt;/li&gt;
                          &lt;li&gt;Black white&lt;/li&gt;
                      &lt;/ul&gt;
                  &lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Books
              &lt;ul&gt;
                  &lt;li&gt;Entertaining&lt;/li&gt;
                  &lt;li&gt;Design&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Music
              &lt;ul&gt;
                  &lt;li&gt;Mass&lt;/li&gt;
                  &lt;li&gt;Folk&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.isNodeExpanded(element); // isNodeExpanded the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        isNodeExpanded: function (element) {
            if (!ej.isNullOrUndefined(element)) {
                var expandUl = $(element).children().filter('ul');
                var isExpanded = $(expandUl).is(':not(:hidden)');
                return isExpanded;
            }
        },
        /**
        * To retrive the  status of the collection of childs for the given TreeView node.
        * @example 
        *  * &lt;ul id="treeView"&gt;
          &lt;li&gt;Artwork
              &lt;ul&gt;
                  &lt;li&gt;Abstract
                      &lt;ul&gt;
                          &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
                          &lt;li&gt;Creative Acrylic&lt;/li&gt;
                          &lt;li&gt;Modern Painting&lt;/li&gt;
                          &lt;li&gt;Canvas Art&lt;/li&gt;
                          &lt;li&gt;Black white&lt;/li&gt;
                      &lt;/ul&gt;
                  &lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Books
              &lt;ul&gt;
                  &lt;li&gt;Entertaining&lt;/li&gt;
                  &lt;li&gt;Design&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
          &lt;li&gt;Music
              &lt;ul&gt;
                  &lt;li&gt;Mass&lt;/li&gt;
                  &lt;li&gt;Folk&lt;/li&gt;
              &lt;/ul&gt;
          &lt;/li&gt;
        &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.hasChildNode(element); // hasChildNode the TreeView nodes
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        hasChildNode: function (element) {
            var child = false;
            if (!ej.isNullOrUndefined(element)) {
                child = $(element).children().length < 2 ? false : true;
            }
            return child;
        },
        //--------------------- Client side events ----------------------//

        _onNodeExpand: function (data) {
            if (this.model.expand) {
                this._trigger("expand", data);
            }
        },
        _onNodeCollapse: function (data) {
            if (this.model.collapse) {
                this._trigger("collapse", data);
            }
        },
        _onBeforeExpand: function (data) {
            if (this.model.beforeExpand) {
                return this._trigger("beforeExpand", data);
            }
        },
        _onBeforeCollapse: function (data) {
            if (this.model.beforeCollapse) {
                return this._trigger("beforeCollapse", data);
            }
        },
        _onNodeSelect: function (data) {
            if (this.model.select) {
                this._trigger("select", data);
            }
        },
        _onChecked: function (element) {
            var liElement = $(element).parents("li").first();
            var nodeText = liElement.find(".e-text").first().text();
            var isChecked = this._isChecked(element);
            var data = { currentElement: liElement, value: nodeText, checked: isChecked };
            if (this.model.check) {
                this._trigger("check", data);
            }
        },
        _onUnChecked: function (element) {
            var liElement = $(element).parents("li").first();
            var nodeText = liElement.find(".e-text").first().text();
            var isUnChecked = this._isChecked(element);
            var data = { currentElement: liElement, value: nodeText, checked: isUnChecked };
            if (this.model.uncheck) {
                this._trigger("uncheck", data);
            }
        },
        _onDragStarts: function (data) {
            if (this.model.dragStart) {
                return this._trigger("dragStart", data);
            }
        },
        _onDrag: function (data) {
            if (this.model.drag) {
                return this._trigger("drag", data);
            }
        },
        _onDragStop: function (data) {
            if (this.model.dragStop) {
                return this._trigger("dragStop", data);
            }
        },
        _onDropped: function (data) {
            if (this.model.dropped) {
                return this._trigger("dropped", data);
            }
        },
        _onClick: function (data) {
            this._trigger("click", data);
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
/*global _comma_separated_list_of_variables_*/

(function ($, ej, undefined) {
    /**
* @namespace ej
* @class ejUploadbox
* @requires jQuery
* @requires ej.common.all
* @classdesc Custom Design for Html Uploadbox control.
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Create Uploadbox
* $('#uploadbox1').ejUploadbox(); 	
* &lt;/script&gt;
*/
    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejUploadbox", "ej.Uploadbox", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-uploadbox",

        // default model
        defaults: {
            /// <summary>This Contains default property of upload button </summary>
            /**		
* Specifies the text content for Uploadbox browse button while initialization.
* @default "Browse"
* @type {string}
* @example 
* //To set  browseButtonText API value 
* 	$("#uploadbox1").ejUploadbox({ browseButtonText: "HelloWorld" });	
* @example 
* //Get or set the browseButtonText API, after initialization:<br>
*	//Gets the browseButtonText value
*	$("#uploadbox1").ejUploadbox("option", "browseButtonText"); 			
*	//Sets the browseButtonText value
*	$("#uploadbox1").ejUploadbox("option", "browseButtonText", "HelloWorld");
* @memberof ejUploadbox
* @instance
*/
            browseButtonText: "Browse",
            /**		
* Specifies the text content for Uploadbox upload button while initialization.
* @default "Upload"
* @type {string}
* @example 
* //To set  uploadButtonText API value during initialization
* 	$("#uploadbox1").ejUploadbox({ uploadButtonText: "HelloWorld" });	
* @example 
* //Get or set the uploadButtonText API, after initialization:<br>
*	//Gets the uploadButtonText value
*	$("#uploadbox1").ejUploadbox("option", "uploadButtonText"); 			
*	//Sets the uploadButtonText value
*	$("#uploadbox1").ejUploadbox("option", "uploadButtonText", "HelloWorld");
* @memberof ejUploadbox
* @instance
*/
            uploadButtonText: "Upload",
            /**		
* Uploadbox supports both synchronous and asynchronous upload. This can be achieved by this property asyncUpload.
* @default true
* @type {boolean}
* @example 
* //To set  asyncUpload API value during initialization
* 	$("#uploadbox1").ejUploadbox({ asyncUpload: false });	
* @example 
* //Get or set the asyncUpload API, after initialization:<br>
*	//Gets the asyncUpload value
*	$("#uploadbox1").ejUploadbox("option", "asyncUpload"); 			
*	//Sets the asyncUpload value
*	$("#uploadbox1").ejUploadbox("option", "asyncUpload", false);
* @memberof ejUploadbox
* @instance
*/
            asyncUpload: true,
            /**		
* Enables or disables Uploadbox based on boolean value.
* @default true
* @type {boolean}
* @example 
* //To set enabled API value during initialization
* 	$("#uploadbox1").ejUploadbox({ enabled: false });	
* @example 
* //Get or set the enabled API, after initialization:<br>
*	//Gets the enabled value
*	$("#uploadbox1").ejUploadbox("option", "enabled"); 			
*	//Sets the enabled value
*	$("#uploadbox1").ejUploadbox("option", "enabled", false);
* @memberof ejUploadbox
* @instance
*/
            enabled: true,
            /**		
* Enables to select multiple files
* @default false
* @type {boolean}
* @example 
* //To set multipleFilesSelect API value during initialization
* 	$("#uploadbox1").ejUploadbox({ multipleFilesSelect: true });	
* @example 
* //Get or set the multipleFilesSelect API, after initialization:<br>
*	//Gets the multipleFilesSelect value
*	$("#uploadbox1").ejUploadbox("option", "multipleFilesSelect"); 			
*	//Sets the multipleFilesSelect value
*	$("#uploadbox1").ejUploadbox("option", "multipleFilesSelect", true);
* @memberof ejUploadbox
* @instance
*/
            multipleFilesSelect: false,
            /**		
* Uploadbox supports auto uploading of files after the file selection is done.The auto upload is performed if autoUpload is set to true.   
* @default false
* @type {boolean}
* @example 
* //To set autoUpload API value during initialization
* 	$("#uploadbox1").ejUploadbox({ autoUpload: true });	
* @example 
* //Get or set the autoUpload API, after initialization:<br>
*	//Gets the autoUpload value
*	$("#uploadbox1").ejUploadbox("option", "autoUpload"); 			
*	//Sets the autoUpload value
*	$("#uploadbox1").ejUploadbox("option", "autoUpload", true);
* @memberof ejUploadbox
* @instance
*/
            autoUpload: false,
            /**		
* Specifies to display the file details of files while selected for uploading can be done when showFileDetails set to true.  
* @default true
* @type {boolean}
* @example 
* //To set showFileDetails API value during initialization
* 	$("#uploadbox1").ejUploadbox({ showFileDetails: false });	
* @example 
* //Get or set the showFileDetails API, after initialization:<br>
*	//Gets the showFileDetails value
*	$("#uploadbox1").ejUploadbox("option", "showFileDetails"); 			
*	//Sets the showFileDetails value
*	$("#uploadbox1").ejUploadbox("option", "showFileDetails", false);
* @memberof ejUploadbox
* @instance
*/
            showFileDetails: true,
            /**		
* Specifies the file extension to allow for uploading the file while initialization. This could be mentioned in string format.  
* @default ""
* @type {string}
* @example 
* //To set fileFormatAllow API value during initialization
* 	$("#uploadbox1").ejUploadbox({ fileFormatAllow: ".zip"  });	
* @example 
* //Get or set the fileFormatAllow API, after initialization:<br>
*	//Gets the fileFormatAllow value
*	$("#uploadbox1").ejUploadbox("option", "fileFormatAllow"); 			
*	//Sets the fileFormatAllow value
*	$("#uploadbox1").ejUploadbox("option", "fileFormatAllow", ".zip" );
* @memberof ejUploadbox
* @instance
*/
            fileFormatAllow: "",
            /**		
* Specifies the file extension to deny for uploading the file while initialization. This could be mentioned in string format.  
* @default ""
* @type {string}
* @example 
* //To set fileFormatDeny API value during initialization
* 	$("#uploadbox1").ejUploadbox({ fileFormatDeny: ".zip"  });	
* @example 
* //Get or set the fileFormatDeny API, after initialization:<br>
*	//Gets the fileFormatDeny value
*	$("#uploadbox1").ejUploadbox("option", "fileFormatDeny"); 			
*	//Sets the fileFormatDeny value
*	$("#uploadbox1").ejUploadbox("option", "fileFormatDeny", ".zip" );
* @memberof ejUploadbox
* @instance
*/
            fileFormatDeny: "",
            /**		
* Specifies the save action which has to be performed after the file is pushed for uploading. Here we have to mention the server address which has to perform this action.  
* @default ""
* @type {string}
* @example 
* //To set saveUrl API value during initialization
* 	$("#uploadbox1").ejUploadbox({ saveUrl: "http://js.syncfusion.com/demos/beta/saveFiles.ashx"});	
* @example 
* //Get or set the saveUrl API, after initialization:<br>
*	//Gets the saveUrl value
*	$("#uploadbox1").ejUploadbox("option", "saveUrl"); 			
*	//Sets the saveUrl value
*	$("#uploadbox1").ejUploadbox("option", "saveUrl", "http://js.syncfusion.com/demos/beta/saveFiles.ashx");
* @memberof ejUploadbox
* @instance
*/
            saveUrl: "",
            /**		
* Specifies the remove action which has to be performed after the file uploading is completed. Here we have to mention the server address which has to perform this action.  
* @default ""
* @type {string}
* @example 
* //To set removeUrl API value during initialization
* 	$("#uploadbox1").ejUploadbox({ removeUrl: "http://js.syncfusion.com/demos/beta/removeFiles.ashx"});	
* @example 
* //Get or set the removeUrl API, after initialization:<br>
*	//Gets the removeUrl value
*	$("#uploadbox1").ejUploadbox("option", "removeUrl"); 			
*	//Sets the removeUrl value
*	$("#uploadbox1").ejUploadbox("option", "removeUrl", "http://js.syncfusion.com/demos/beta/removeFiles.ashx");
* @memberof ejUploadbox
* @instance
*/
            removeUrl: "",
            /**		
* Set the root class for Uploadbox control theme. This cssClass API helps to use custom skinning option for Uploadbox control.   
* @default ""
* @type {string}
* @example 
* //To set cssClass API value during initialization
* 	$("#uploadbox1").ejUploadbox({ cssClass: "gradient-lime"});	
* @example 
* //Get or set the cssClass API, after initialization:<br>
*	//Gets the cssClass value
*	$("#uploadbox1").ejUploadbox("option", "cssClass"); 			
*	//Sets the cssClass value
*	$("#uploadbox1").ejUploadbox("option", "cssClass", "gradient-lime");
* @memberof ejUploadbox
* @instance
*/
            cssClass: "",
            /**		
* Specifies the Right to Left direction property for Uploadbox control.   
* @default false
* @type {boolean}
* @example 
* //To set rtl API value during initialization
* 	$("#uploadbox1").ejUploadbox({ rtl: true});	
* @example 
* //Get or set the rtl API, after initialization:<br>
*	//Gets the rtl value
*	$("#uploadbox1").ejUploadbox("option", "rtl"); 			
*	//Sets the rtl value
*	$("#uploadbox1").ejUploadbox("option", "rtl", true);
* @memberof ejUploadbox
* @instance
*/
            rtl: false,
            /**     
* Fires when Uploadbox control is created.
* @event
* @name ejUploadbox#create	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* //create event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    create: function (args) {}
* });      
* @memberof ejUploadbox
* @instance
*/
            create: null,
            /**     
* Fires when the file is selected for upload successfully.
* @event
* @name ejUploadbox#fileSelect	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* //fileSelect event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    fileSelect: function (args) {}
* });      
* @memberof ejUploadbox
* @instance
*/
            fileSelect: null,
            /**     
* Fires when the upload progress begins.
* @event
* @name ejUploadbox#begin	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* //begin event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    begin: function (args) {}
* });      
* @memberof ejUploadbox
* @instance
*/
            begin: null,
            /**     
* Fires when the upload progress is cancelled.
* @event
* @name ejUploadbox#cancel	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* //cancel event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    cancel: function (args) {}
* });      
* @memberof ejUploadbox
* @instance
*/
            cancel: null,
            /**     
* Fires when the file upload progress is completed.
* @event
* @name ejUploadbox#complete	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @param {object}  argument.files returns file entries and its details
* @example 
* //complete event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    complete: function (args) {}
* });      
* @memberof ejUploadbox
* @instance
*/
            complete: null,
            /**     
* Fires when the uploaded file is removed successfully.
* @event
* @name ejUploadbox#remove	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @param {object}  argument.fileStatus returns the file details of the file object
* @example 
* //remove event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    remove: function (args) {}
* });      
* @memberof ejUploadbox
* @instance
*/
            remove: null,
            /**     
* Fires when the Upload process ends in Error.
* @event
* @name ejUploadbox#error	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @param {string}  argument.action returns the action name
* @param {object}  argument.files returns the file details of the file uploaded
* @example 
* //error event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    error: function (args) {}
* });      
* @memberof ejUploadbox
* @instance
*/
            error: null,
            /**     
* Fires when Uploadbox control is destroyed.
* @event
* @name ejUploadbox#destroy	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* //destroy event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    destroy: function (args) {}
* });      
* @memberof ejUploadbox
* @instance
*/
            destroy: null
        },
        /**
* Specify the data types for default properties 
* @private
*/
        //Data Types
        dataTypes: {
            browseButtonText: "string",
            uploadButtonText: "string",
            disbled: "boolean",
            multipleFilesUpload: "boolean",
            autoUpload: "boolean",
            showFileDetails: "boolean",
            fileFormatAllow: "string",
            fileFormatDeny: "string",
            saveUrl: "string",
            removeUrl: "string",
            cssClass: "string",
            rtl: "boolean"
        },
        // sample public function
        //Disables the upload box
        /**
* To disable the Uploadbox control  
* @return jQuery
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Disable Uploadbox
* var uploadObj = $("#uploadbox1").data("ejUploadbox");
* uploadObj.disable(); // disable the Uploadbox
* &lt;/script&gt;
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // disable the Uploadbox
* $("#uploadbox1").ejUploadbox("disable");	
* &lt;/script&gt;
*@memberof ejUploadbox
* @instance
*/
        disable: function () {
            /// <summary>This will disable the upload button </summary>
            this.element.addClass("e-disable");
            this.model.enabled = false;
        },
        /**
* To enable the Uploadbox control  
* @return jQuery
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // enable Uploadbox
* var uploadObj = $("#uploadbox1").data("ejUploadbox");
* uploadObj.enable(); // enable the Uploadbox
* &lt;/script&gt;
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // enable the Uploadbox
* $("#uploadbox1").ejUploadbox("enable");	
* &lt;/script&gt;
*@memberof ejUploadbox
* @instance
*/
        enable: function () {
            /// <summary>This will enable upload button </summary>
            if (this.element.hasClass("e-disable")) {
                this.element.removeClass("e-disable");
                this.model.enabled = true;
            }
        },
        // constructor function
        /**
* Create the Uploadbox widget
* @private
*/
        _init: function () {
            /// <summary>This will initialize upload button </summary>
            this.s = ej.browserInfo();
            this._initialize();
            this._wireEvents();
            /*Sync Uploads*/
            if (!this.model.asyncUpload) {
                this._initObjectsSyncUpload();
            }
            //disable status
            this._controlStatus(this.model.enabled);
        },
        /**
* To configure the properties at runtime using SetModel		
* @private
*/
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "browseButtonText":
                        this._setBrowseButtonText(options[option]);
                        break;
                    case "uploadButtonText":
                        this._setUploadButtonText(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "rtl":
                        this._setRTL(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                }
            }
        },
        /**
* To ensure the status of the control;either enabled or disabled	
* @private
*/
        _controlStatus: function (value) {
            //disable status
            value != true ? this.disable() : this.enable();
        },
        /**
* To set rtl property if enabled	
* @private
*/
        _setRTL: function (val) {
            if (val) {
                this.element.addClass("e-rtl");
                if (this.s.name == "msie") {
                    this.inputupload.css('right', '-153px');
                }
            } else {
                this.element.removeClass("e-rtl");
                if (this.s.name == "msie") {
                    this.inputupload.css('right', '0px');
                }
            }
            if (this.updialog) this.updialog.ejDialog({ rtl: val });
        },
        /**
* To set Browse Button Text property.
* @private
*/
        _setBrowseButtonText: function (val) {
            this.buttondiv.val(val);
        },
        /**
* To set Upload Button Text property.
* @private
*/
        _setUploadButtonText: function (val) {
            this.element.find(".e-uploadbtn").html(val);
        },
        // all events bound using this._on will be unbind automatically
        /**
* destroy the Upload control
* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
* @alias destroy
* @return jQuery
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Destroy Upload
* var uploadObj = $("#uploadbox1").data("ejUploadbox");
* uploadObj.destroy(); // destroy the Upload control
* &lt;/script&gt;
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // destroy the Upload control
* $("#uploadbox1").ejUploadbox("destroy");	
* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
        _destroy: function () {
            /// <summary>This will destroy the  upload button </summary>
            if (this.element.hasClass("e-uploadbox")) {
                this.element.removeClass("e-uploadbox e-widget");
                this.element.empty();
            }
        },
        /**
* To set custom skin theme if cssClass property is enabled.
* @private
*/
        _setSkin: function (skin) {
            this.element.removeClass(this.model.cssClass);
            this.element.addClass(skin);
        },
        /**
* To initialize the Uploadbox control	
* @private
*/
        //initialize the control
        _initialize: function () {
            /// <summary>This will initialize upload button </summary>
            /*Upload Control*/
            this.control = this.element[0];
            this.element.addClass("e-widget " + this.model.cssClass);
            this.innerdiv = ej.buildTag('div.e-selectpart e-select');
            this.element.append(this.innerdiv);
            this.buttondiv = ej.buildTag('input.e-inputbtn e-btn#' + this.control.id + '_SelectButton', '', {}, { type: 'button', value: this.model.browseButtonText });
            this.inputupload = ej.buildTag('input.e-uploadinput', "", {}, { type: 'file', name: this.control.id });
            if (this.model.multipleFilesSelect) {
                this.inputupload.attr(multiple, 'multiple');
            }
            this.innerdiv.append(this.buttondiv);
            this.innerdiv.append(this.inputupload);
            this._Selector = this.buttondiv[0];
            this._setRTL(this.model.rtl);
            this.Uploadframes = []; //For IFrame
            this.UploadForms = [];
            this.UploadType = this._isXhrSupported() ? "Xhr" : "IFrame";


        },
        //Wiring Events
        /**
* Wiring the events to Uploadbox control		
* @private
*/
        _wireEvents: function () {
            /// <summary>This will wire events of  upload button </summary>
            this._on(this.element, "click", this._disableclickselect);
            this._bindInputChangeEvent();
        },
        // disable click of select button.
        /**
* To prevent click action on disable state		
* @private
*/
        _disableclickselect: function (e) {
            if (this.element.hasClass("e-disable")) {
                e.preventDefault();
            }
        },
        //Bind input change function
        /**
* This will bind input change event of upload button 	
* @private
*/
        _bindInputChangeEvent: function () {
            //file selection changed event
            this._on(this.inputupload, "change", this._inputValueChange);
        },
        //Internal Events 
        /**
* This will bind input value change event of upload button 	
* @private
*/
        _inputValueChange: function (e) {
            var input, files;
            input = $(e.target);
            this.element.find(".e-uploadbtn").removeAttr('disabled').removeClass('e-disable');
            files = this._getInputFileInfo(input);
            this._trigger("fileSelect");
            if (this._isAllowed(files)) {
                if (!this.model.asyncUpload) {
                    this._onSelectSyncUpload(e); //Sync Upload
                } else {
                    if (this.UploadType == "Xhr") {
                        this._onXhrSelect(e); //removed uploadcore // For XHR Upload
                    } else {
                        this._onSelectIFrame(e); //For Iframe
                    }
                }
            }
        },
        // When Auto upload is false
        /**
* This will fire on upload button	
* @private
*/
        __uploadButtonClick: function (e) {
            //removed uploadcore
            if (this.UploadType == "Xhr") {
                this._xhrOnUploadButtonClick(e); // Xhr Upload
            } else {
                this._onUploadButtonClickIFrame(e); //Iframe Upload
            }
            $(e.target).attr('disabled', 'disabled').addClass('e-disable');
        },
        /**
* This will fire the upload button
* @private
*/
        _actionClick: function (e) {
            var targetAction, fileItem, file;
            targetAction = $(e.target);
            fileItem = targetAction.closest(".e-upload-file");
            file = $(fileItem).data("file");
            if (targetAction.hasClass("e-file-delete")) {
                //Trigger on Remove hanlder
                this._trigger("remove", { fileStatus: file });
                //removed uploadcore
                if (this.UploadType == "Xhr") {
                    this._xhrOnRemove(e, fileItem); //XHr Remove
                } else {
                    this._onRemoveIFrame(e, fileItem); //IFrame File Upload
                }
            } else if (targetAction.hasClass("e-file-cancel")) {
                //Trigger on Cancel hanlder
                this._trigger("cancel", { fileStatus: file }); //removed uploadcore
                if (!this.model.asyncUpload) {
                    this._onCancelSyncUpload(e, fileItem); //sync
                } else {
                    if (this.UploadType == "Xhr") {
                        this._xhrOnCancel(e, fileItem); //Xhr Cancel Item
                    } else {
                        this._onCancelIFrame(e, fileItem); //IFrame Cancel Item
                    }
                }
            } else if (targetAction.hasClass("e-file-retry")) {//removed uploadcore
                if (this.UploadType == "Xhr") {
                    this._xhrOnRetry(e, fileItem); //xhr remove
                } else {
                    this._onRetryIFrame(e, fileItem); //IFrame Remove
                }
            }

        },
        /**
* This Remove file entry upload button 
* @private
*/
        _removeFileEntry: function (file) {
            file.remove();
        },
        // Internal Methods
        /**
* This will fire is on file upload button 
* @private
*/
        _isFileUpload: function (fileEntry) {
            var actiondiv = $(fileEntry).find("div.e-icon");
            return actiondiv.is(".e-file-cancel");
        },
        /*Supporting Browsers */
        /**
* Validates upload type is xhr or not
* @private
*/
        _isXhrSupported: function () {
            return (((this.s.name == "msie" && parseInt(this.s.version) < 9) || ((this.s.name == "safari" && this.s.name == "chrome") && this.s.version == "536")) ? false : (typeof (FormData) != "undefined") && (typeof (File) != "undefined"));
        },
        /*Getting File Details*/
        /**
* To get the file details of the uploaded file
* @private
*/
        _getFileName: function (input) {
            /// <summary>This will get file name in upload button </summary>
            return $.map(this._getAllFileInfo(input), function (file) {
                return file.name;
            }).join(", ");
        },
        /**
* To get the file size of the uploaded file
* @private
*/
        _getFileSize: function (input) {
            /// <summary>This will get file size in upload button </summary>
            var tempProxy = this;
            return $.map(this._getAllFileInfo(input), function (file) {
                return tempProxy._formatSize(file.size);
            }).join(", ");
        },
        /**
* Pushes the uploaded file in uplaod dialog box
* @private
*/
        _pushFile: function (files, datapart) {
            /// <summary>This will push file in upload button </summary>
            var fileListDetails, addedFile, actionlist, i, action, diaObj, addedheading;
            addedheading = $("<div class='e-head-content'><div class='e-file-head e-head-name'>Name</div><div class='e-file-head e-head-size'>Size</div><div class='e-file-head e-head-status'>Status</div></div>");
            filedialog = this.updialog;
            if (filedialog && filedialog.length != 0) {
                if (this.model.showFileDetails) {
                    this.diaObj.open();
                }
            }
            else {
                this.updialog = ej.buildTag('div.uploaddialog#' + this.element[0].id + '_dialog', "", {}, { 'title': 'Upload Box' });
                this.element.append(this.updialog);
            }
            fileListDetails = this.element.find(".e-ul"); /// <reference path="IframeUpload.js" />
            if (fileListDetails.length == 0) {
                addedheading.appendTo(this.updialog);
                fileListDetails = ej.buildTag('ul.e-ul').appendTo(this.updialog);
                fileListActions = ej.buildTag('div.e-file-upload').appendTo(this.updialog);
                dialogActions = ej.buildTag('button.e-uploadclosebtn e-btn e-select', "Cancel").appendTo(fileListActions);
                this._on(dialogActions, "click", this._dialogclose);
            }
            var xpos = (this.element.offset().left + this.element.width()) / 3, ypos = this.element.offset().top;
            this.updialog.ejDialog({ autoOpen: false, width: 510, cssClass: this.model.cssClass, close: $.proxy(this._uploadFileListDelete, this), position: { X: xpos, Y: ypos }, rtl: this.model.rtl, contentContainer: "#" + this.element[0].id, resizable: false });
            this.diaObj = this.updialog.data('ejDialog');
            for (i = 0; i < files.length; i++) {//localization can be given for not started
                /*things to be redefined*/
                addedFile = $("<li class='e-upload-file'><div class='e-file-list'>" +
                    "<div class='e-file-progress e-file-view'><div class='e-file-name e-file-view'><span class='e-file-name-txt'>"
                    + files[i].name + "</span></div></div><div class='e-file-size e-file-view'><span class='e-file-name-txt'>"
                    + files[i].size + "</span></div>" +
                    "<div class='e-file-percentage e-file-view'><div class='e-file-progress-bar'><div class='e-file-progress-status'></div></div></div><div class='e-action-perform'><div class='e-icon e-file-view'></div></div></div></li>").appendTo(fileListDetails).data(datapart);

                if (this._getFileSize(files[i]).toString().toLowerCase() == "nankb" || files[i].size == null) {
                    addedFile.find(".e-file-size").remove();
                }
                action = "cancel";
                addedFile.find(".e-icon").remove().addClass(action.toString());
                if (action == "cancel") {
                    actionlist = ej.buildTag('div.e-icon e-file-cancel', '', {}, { title: 'cancel' });
                } else if (action == "remove") {
                    actionlist = ej.buildTag('div.e-icon e-file-delete', '', {}, { title: 'remove' });
                } else if (action == "retry") {
                    actionlist = ej.buildTag('div.e-icon e-file-retry', '', {}, { title: 'retry' });
                }
                //appending list
                addedFile.find(".e-action-perform").append(actionlist);
                //FileList Click
                this._on(actionlist, "click", this._actionClick);
            }
            if (this.model.showFileDetails) {
                this.diaObj.open();
            }

            return addedFile;
        },
        /**
* This will push file details in upload button
* @private
*/
        _pushFileDetails: function (files) {
            var fileListDetails, addedFile, actionlist, i, action, me, diaObj, addedheading;
            addedheading = $("<div class='e-head-content'><div class='e-file-head e-head-name'>Name</div><div class='e-file-head e-head-size'>Size</div><div class='e-file-head e-head-status'>Status</div></div>");
            me = this;
            filedialog = this.updialog;
            if (filedialog && filedialog.length != 0) {
                if (this.model.showFileDetails) {
                    me.diaObj.open();
                }
            } else {
                this.updialog = ej.buildTag('div.uploaddialog#' + this.element[0].id + '_dialog', "", {}, { 'title': 'Upload Box' });
                this.element.append(this.updialog);
            }
            var fileListDetails, addedFile;
            fileListDetails = this.element.find(".e-ul");
            if (fileListDetails.length == 0) {
                addedheading.appendTo(this.updialog);
                fileListDetails = ej.buildTag('ul.e-ul').appendTo(this.updialog);
                fileListActions = ej.buildTag('div.e-file-upload').appendTo(this.updialog);
                dialogActions = ej.buildTag('button.e-uploadclosebtn e-btn e-select', "Cancel").appendTo(fileListActions);
                this._on(dialogActions, "click", this._dialogclose);
            }
            var xpos = (this.element.offset().left + this.element.width()) / 3, ypos = this.element.offset().top + this.element.height();
            addedFile = $("<li class='e-upload-file'><div class='e-file-list'><div class='e-file-progress e-file-view'><div class='e-file-name e-file-view'><span class='e-file-name-txt'>" + files.name + "</span></div></div><div class='e-file-size e-file-view'><span class='e-file-name-txt'>" + this._formatSize(0) + "\\" + this._formatSize(files.size) + "</span></div><div class='e-file-percentage e-file-view'><div class='e-file-progress-bar'><div class='e-file-progress-status'></div></div></div><div class='e-action-perform'><div class='e-icon e-file-view'></div></div></div></li>").appendTo(fileListDetails).data("file", files);
            this.updialog.ejDialog({ autoOpen: false, width: 520, height: "auto", cssClass: this.model.cssClass, close: $.proxy(this._uploadFileListDelete, this), position: { X: xpos, Y: ypos }, rtl: this.model.rtl, contentContainer: "#" + this.element[0].id, resizable: false })
            //create object for ejDialog 
            me.diaObj = this.updialog.data("ejDialog");
            if (this.model.showFileDetails) {
                me.diaObj.open();
            }
            return addedFile;
        },
        /**
* Specifies uploading/ progress state of the file uploaded
* @private
*/
        _setProgress: function (filelist, percentage, e) {
            var progressbar, progress, filesize, loaded, total;
            progressbar = $(filelist).find(".e-file-progress-status");
            progressbar.width(percentage + "%");

            filesize = $(filelist).find(".e-file-size .e-file-name-txt");
            loaded = this._formatSize(e.loaded);
            total = this._formatSize(e.total);
            filesize.html(loaded + "\\" + total);
        },
        /**
* Specifies the current action of the file uploaded
* @private
*/
        _setAction: function (element, action) {
            //localization can be provided for the strings cancel,remove and retry.
            var actionlist;
            element.find(".e-icon").remove().addClass(action.toString());
            if (action == "cancel") {
                actionlist = ej.buildTag('div.e-icon e-file-cancel', '', {}, { title: 'cancel' });
            } else if (action == "remove") {
                actionlist = ej.buildTag('div.e-icon e-file-delete', '', {}, { title: 'remove' });
            } else if (action == "retry") {
                actionlist = ej.buildTag('div.e-icon e-file-retry', '', {}, { title: 'retry' });
            }
            //appending list
            element.find(".e-action-perform").append(actionlist);
            //FileList Click
            this._on(actionlist, "click", this._actionClick);
        },
        /**
* Sets status of the file uploaded
* @private
*/
        _setStatus: function (element, status) {
            var progress, upstatus = ej.buildTag('div');
            if (status == "success") {
                element.find(".file-status").addClass("e-file-status-success").html("Completed");
                element.find(".e-file-percentage").html("").attr("title", "Completed");
                upstatus.addClass("e-icon e-file-percentage-success");
                element.find(".e-file-percentage").append(upstatus);
            }
            if (status == "failed") {
                element.find(".file-status").addClass("e-file-status-failed").html("Failed");
                element.find(".e-file-percentage").html("").attr("title", "Failed");
                upstatus.addClass("e-icon e-file-percentage-failed");
                element.find(".e-file-percentage").append(upstatus);
            }
            if (status == "progress") {
                element.find(".file-status").addClass("file-status-inprogress").html("in progress");
            }
            if (status == "uploading") {
                element.find(".file-status").addClass("file-status-inprogress").html("uploading");
                progress = element.find(".e-file-percentage");
                progress.html("");
            }
        },
        /**
* Sets input field for multiple file upload
* @private
*/
        _createInputandBind: function () {
            // creating input field. 
            var tempInput = ej.buildTag('input', '', {}, { type: 'file' });
            tempInput.attr("name", this.control.id).attr("autocomplete", "off").attr("class", "e-uploadinput");
            if (this.model.multipleFilesUpload) {
                tempInput.attr("multiple", "multiple");
            }
            tempInput.appendTo(this.element.find(".e-selectpart"));
            this._bindInputChangeEvent();
        },
        /**
* This will show upload button
* @private
*/
        _showUploadButton: function () {
            var uploadbutton = this.element.find(".e-uploadbtn");
            if (uploadbutton.length == 0) {
                uploadbutton = ej.buildTag('button.e-uploadbtn e-btn e-select', this.model.uploadButtonText);
                this.element.find(".e-file-upload").append(uploadbutton);
                //File_upload_button Click
                this._on(uploadbutton, "click", this.__uploadButtonClick);
            }

        },
        /**
* This will reset the uplaod button
* @private
*/
        _resetFileInput: function ($element) {
            var clone = $element.clone(false, false);
            this._on(clone, "change", this._inputValueChange);
            $element.replaceWith(clone);
        },
        /**
* Ensures the file format
* @private
*/
        _isAllowed: function (files) {
            var inputfield, denied, uploadControl, args, allowExtension, denyExtension;
            inputfield = this.element.find(".e-uploadinput");
            denied = false;
            uploadControl = this;
            if (this.model.fileFormatAllow != "") {
                allowExtension = this.model.fileFormatAllow.split(",");
                $(files).each(function () {
                    if ($.inArray(this.extension, allowExtension) == -1) {
                        // Raise Event
                        args = { action: "ExtensionsAllow", files: files };
                        uploadControl._trigger('error', args);
                        denied = true;
                        return false;
                    }
                }
                    );
            }
            if (this.model.fileFormatDeny != "") {
                denyExtension = this.model.fileFormatDeny.split(",");
                $(files).each(function () {
                    if ($.inArray(this.extension, denyExtension) != -1) {
                        // Raise Event
                        args = { action: "ExtensionsDeny", files: files };
                        uploadControl._trigger('error', args);
                        denied = true;
                        return false;
                    }
                });
            }
            if (denied) {
                this._resetFileInput(inputfield);
            }
            return !denied;
        },
        /**
* This will sets the file remove status.
* @private
*/
        _fileListRemove: function () {
            fileList = this.element.find(".e-upload-file .e-file-delete");
            if (fileList.length == 0) {
                this.element.find(".e-uploadbtn").attr('disabled', 'disabled').addClass('e-disable');
                this.updialog.ejDialog('close');
            }
        },
        /**
* This will sets the uplaod disabled
* @private
*/
        _uploadHide: function () {
            fileList = this.element.find(".e-upload-file .e-file-cancel");
            if (fileList.length == 0) {
                this.element.find(".e-ul").empty();
                this.element.find(".e-uploadbtn").attr('disabled', 'disabled').addClass('e-disable');
                this.updialog.ejDialog('close');
            }
        },
        /**
* Section of handling dialog close action
* @private
*/
        _dialogclose: function () {
            var inputfield;
            this.updialog.find(".e-ul").empty();
            inputfield = this.element.find(".e-uploadinput");
            this._resetFileInput(inputfield);
            this.updialog.ejDialog('close');
        },
        /**
* Deletes the file list from the dialog box.
* @private
*/
        _uploadFileListDelete: function () {
            var inputfield;
            this.updialog.find(".e-ul").empty();
            inputfield = this.element.find(".e-uploadinput");
            this._resetFileInput(inputfield);
        },
        /* XHR Upload  */
        /**/
        /**
* Section for handling XHR upload
* @private
*/
        _onXhrSelect: function (e) {
            var inputField, files, xhrUpload, addedFile;
            inputField = $(e.target);
            files = this._getInputFileInfo(inputField);
            this._xhrBeforeUpload(files, inputField);
            xhrUpload = this;
            $.each(files, function (i, fileItem) {
                addedFile = $(fileItem).data("filelist");
                xhrUpload._setAction(addedFile, "cancel"); // XhrUpload._Uploader._setAction(addedFile, "cancel");
                if (xhrUpload.model.autoUpload) {
                    xhrUpload._xhrPerformUpload(fileItem);
                } else {
                    xhrUpload._showUploadButton();
                }
            });
        },
        /**
* Action performed before XHR upload
* @private
*/
        _xhrBeforeUpload: function (files, inputField) {
            var fileEntry, xhrUpload, formdata, addedFile;
            fileEntry = files;
            xhrUpload = this;
            $.each(fileEntry, function (i, fileItem) {
                formdata = xhrUpload._createFormObjectXhr(fileItem);
                $(fileItem).data("formobject", formdata);
                addedFile = xhrUpload._pushFileDetails(fileItem);
                $(fileItem).data("filelist", addedFile);
            });

            return fileEntry;
        },
        /**
* Action performed during XHR upload
* @private
*/
        _xhrPerformUpload: function (fileItem) {
            var isPrevented, url, xhrUpload, formdata, xhr;
            this._trigger('begin');
            url = this.model.saveUrl; //
            xhrUpload = this;
            formdata = $(fileItem).data("formobject");
            xhr = new XMLHttpRequest();
            $(fileItem).data("xhr", xhr);
            xhr.addEventListener("load", function (e) {
                xhrUpload._onRequestSuccess(xhrUpload, e, fileItem);
            }, false);
            xhr.addEventListener("error", function (e) {
                xhrUpload._onRequestError(xhrUpload, e, fileItem);
            }, false);
            xhr.upload.addEventListener("progress", function (e) {
                xhrUpload._onRequestProgress(xhrUpload, e, fileItem);
            }, false);
            xhr.open("POST", url);
            xhr.send(formdata);
        },
        /**
* secifies Action performed during XHR upload button click
* @private
*/
        _xhrOnUploadButtonClick: function (e) {
            var xhrUpload, fileEntry, started;
            xhrUpload = this;
            $(".e-ul li.e-upload-file", xhrUpload.control).each(function () {
                fileEntry = $(this);
                started = xhrUpload._isFileUpload(fileEntry);
                if (started) {
                    xhrUpload._xhrPerformUpload($(fileEntry).data("file"));
                }
            });
        },
        /**
* secifies Action performed during XHR uploaded file remove
* @private
*/
        _xhrOnRemove: function (e, fileItem) {
            var filename = $(fileItem).find(".e-file-name").text().toString().split(","), proxy = this;
            $.ajax({
                url: this.model.removeUrl,
                type: "POST",
                data: "fileNames=" + filename,
                success: function () {
                    $(fileItem).remove();
                    proxy._fileListRemove();
                }
            });
        },
        /**
* secifies cancel action performed during XHR uploaded
* @private
*/
        _xhrOnCancel: function (e, fileItem) {
            var file, xhr;
            file = $(fileItem).data("file");
            xhr = $(file).data("xhr");
            if (xhr) {
                $(file).data("xhr").abort();
            }
            $(file).data("xhr", null);
            $(fileItem).data("file", null);
            $(fileItem).remove();
            this._uploadHide();
        },
        /**
* secifies retry action performed during XHR uploaded
* @private
*/
        _xhrOnRetry: function (e, fileItem) {
            var file = $(fileItem).data("file");
            this._xhrPerformUpload(file);
        },
        /**
* section for handling request success for uploading
* @private
*/
        _onRequestSuccess: function (xhrUpload, e, fileEntry) {
            var xhr = $(fileEntry).data("xhr");
            if (xhr.status >= 200 && xhr.status <= 299) {
                xhrUpload._onXhrUploadSuccess(xhrUpload, e, fileEntry);
            }
            else {
                xhrUpload._onRequestError(xhrUpload, e, fileEntry);
            }
        },
        /**
* section for handling when upload is successful
* @private
*/
        _onXhrUploadSuccess: function (xhrUpload, e, fileEntry) {
            var addedFile, xhr, progressbar, size, fSize, filesize, args;
            addedFile = $(fileEntry).data("filelist");
            xhr = $(fileEntry).data("xhr");
            if (xhrUpload.model.removeUrl) {
                xhrUpload._setAction(addedFile, "remove");
                xhrUpload._setStatus(addedFile, "success");
            } else {
                addedFile.find(".e-icon").remove();
                xhrUpload._setStatus(addedFile, "success");
            }
            if ($(fileEntry).length > 0) {
                progressbar = $(addedFile).find(".e-file-progress-status");
                progressbar.width("100%");
                size = $(fileEntry)[0].size;
                fSize = this._formatSize(size);
                filesize = $(addedFile).find(".e-file-size .e-file-name-txt");
                filesize.html(fSize + "\\" + fSize);
            }
            args = { files: fileEntry, xhr: xhr, e: e };
            xhrUpload._trigger('complete', args);
        },
        /**
* section for handling when upload is failed
* @private
*/
        _onRequestError: function (xhrUpload, e, fileEntry) {
            var addedFile, xhr, args;
            addedFile = $(fileEntry).data("filelist");
            xhr = $(fileEntry).data("xhr");
            xhrUpload._setAction(addedFile, "retry");
            xhrUpload._setStatus(addedFile, "failed");
            args = { action: "Error", files: fileEntry, xhr: xhr, e: e };
            xhrUpload._trigger('error', args);
        },
        /**
* section for handling when upload is in progress
* @private
*/
        _onRequestProgress: function (xhrUpload, e, fileEntry) {
            var percentage, addedFile;
            percentage = Math.round(e.loaded * 100 / e.total);
            addedFile = $(fileEntry).data("filelist");
            xhrUpload._setProgress(addedFile, percentage, e);
            xhrUpload._setStatus(addedFile, "progress");
        },
        /**
* section for handling xhr form object
* @private
*/
        _createFormObjectXhr: function (file) {
            var formData = new FormData();
            formData.append(this.control.id, file.rawFile);
            return formData;
        },
        /**/
        //Utility functions
        /**/
        /**
* This will input File Info upload button 
* @private
*/
        _getInputFileInfo: function ($input) {
            var input = $input[0];
            if (input.files) {
                return this._getAllFileInfo(input.files);
            } else {
                return [{
                    name: this._GetName(input.value),
                    extension: this._getFileExtensionType(input.value),
                    size: this._getFileSizeinIE(input.value)
                }];
            }
        },
        /**
* This will get File Size IE upload button
* @private
*/
        _getFileSizeinIE: function (fileName) {
            var actievXObj, fileSize;
            actievXObj = null;
            fileSize = null;
            try {
                actievXObj = new ActiveXObject("Scripting.FileSystemObject");
            } catch (e) {
                fileSize = null;
            }
            if (actievXObj) {
                fileSize = actievXObj.getFile(fileName).size;
            }
            return fileSize;
        },
        /**
* This will get FileExtensionType of upload file
* @private
*/
        _getFileExtensionType: function (fileName) {
            return fileName.match ? (fileName.match(/\.([^\.]+)$/)[0] || "") : "";
        },
        /**
* This will return information about all files uploaded
* @private
*/
        _getAllFileInfo: function (rawFiles) {
            var tempProxy = this;
            return $.map(rawFiles, function (file) {
                return tempProxy._getFileInfo(file || rawFiles);
            });
        },
        /**
* This will return full names of files
* @private
*/
        _GetName: function (fullname) {
            var nameIndex = fullname.lastIndexOf("\\");
            return (nameIndex != -1) ? fullname.substr(nameIndex + 1) : fullname;
        },
        /**
* This will return file details in firefox older versions
* @private
*/
        _getFileInfo: function (rawFile) {
            // Older Firefox versions (before 3.6) use fileName and fileSize
            var fileName = rawFile.name || rawFile.fileName || rawFile;
            return {
                name: fileName,
                extension: this._getFileExtensionType(fileName),
                size: rawFile.size || rawFile.fileSize,
                rawFile: rawFile
            };
        },
        /**
* This will return file size type
* @private
*/
        _formatSize: function (bytes) {
            var i = -1;
            do {
                bytes = bytes / 1024;
                i++;
            } while (bytes > 99);
            return Math.max(bytes, 0.1).toFixed(1) + ['KB', 'MB', 'GB', 'TB', 'PB', 'EB'][i];
        },
        /**/
        //Start
        // IFrame Uploads
        /**
* Section for handling IFrame uploads
* @private
*/
        _onSelectIFrame: function (e) {
            var input, files, addedFile, uploadframe;
            input = $(e.target);
            files = this._getInputFileInfo(input);
            addedFile = this._beforeUploadIFrame(files);
            uploadframe = addedFile.data("iframe");
            if (this.model.autoUpload) {
                this._performUploadIFrame(addedFile);
            } else {
                this._showUploadButton();
            }
            this._off(this.inputupload, "change");
            this._bindInputChangeEvent();
        },
        /**
* Section for handling files removed from IFrame
* @private
*/
        _onRemoveIFrame: function (e, fileItem) {
            var iframe, fileNames, proxy, fileDetails;
            iframe = fileItem.data("iframe");
            fileDetails = $(fileItem).data("file");
            fileNames = fileDetails[0].name;
            proxy = this;
            if (iframe) {
                this._removeFileEntry(fileItem);
                if (this.model.removeUrl) {
                    $.ajax({
                        url: this.model.removeUrl,
                        type: "POST",
                        data: "fileNames=" + fileNames,
                        success: function () {
                            proxy._fileListRemove();
                        }
                    });
                }
            } else {
                this._removeFileEntry(fileItem);
            }
        },
        /**
* Section for handling cancel action performed during file upload
* @private
*/
        _onCancelIFrame: function (e, fileItem) {
            var iframe;
            //trigger  cancel
            this._trigger('cancel', { Status: fileItem });
            iframe = fileItem.data("iframe");
            if (iframe) {
                this._removeFileEntry(fileItem);
                if (typeof (iframe.stop) != "undefined") {
                    iframe.stop();
                } else if (iframe.document) {
                    iframe.document.execCommand("Stop");
                    iframe.contentWindow.location.href = iframe.contentWindow.location.href;
                }
                this._processServerResponse(iframe, "");
            }
            this._uploadHide();
        },
        /**
* Section for handling retry action performed during file upload
* @private
*/
        _onRetryIFrame: function (e, fileItem) {
            this._performUploadIFrame(fileItem);
        },
        /**
* Section for handling actions performed during before IFrame upload
* @private
*/
        _beforeUploadIFrame: function (files) {
            var uploadframe, uploadform, addedfile;
            //creating iframe and adding it to the upload div block.
            uploadframe = this._createFrame(this.control.id + "_Iframe" + this.Uploadframes.length);
            this.Uploadframes.push(uploadframe);
            uploadform = this._createForm(this.model.saveUrl, uploadframe[0].id);
            this.element.find("input.e-uploadinput").removeClass("e-uploadinput").css("display", "none").appendTo(uploadform);
            this._createInputandBind();
            addedfile = this._pushFile(files, { "iframe": uploadframe, "form": uploadform, "file": files });
            uploadframe.data({ "filelist": addedfile });
            this._setAction(addedfile, "cancel");
            return addedfile;
        },
        /**
* Section for handling actions performed during IFrame upload
* @private
*/
        _performUploadIFrame: function (addedFile) {
            var isPrevented, files, uploadframe, uploadform;
            this._trigger('begin');
            files = addedFile.data("file");
            this._setStatus(addedFile, "uploading");
            uploadframe = addedFile.data("iframe");
            uploadform = addedFile.data("form");
            uploadframe.appendTo(document.body);
            uploadform.appendTo(document.body);
            //error here calls the upload even if the url is wrong
            this._on(uploadframe, "load", this._removeFramesIFrame);
            uploadform.submit();
        },
        /**
* Section for handling actions performed during IFrame upload button click
* @private
*/
        _onUploadButtonClickIFrame: function (e) {
            var iframeUpload, fileEntry, started;
            iframeUpload = this;
            $(".e-ul li.e-upload-file", iframeUpload.control).each(function () {
                fileEntry = $(this);
                started = iframeUpload._isFileUpload(fileEntry);
                if (started) {
                    iframeUpload._performUploadIFrame(fileEntry);
                }
            });
        },
        /**
* Section for handling actions when IFrame files removed before upload
* @private
*/
        _removeFramesIFrame: function (e) {
            var uploadframe, response, filelist, fileEntry, args;
            uploadframe = $(e.target);
            filelist = uploadframe.data("filelist");
            try {
                response = uploadframe.contents().text();
            } catch (e) {
                response = "Error trying to get server response: " + e;
            }
            if (response == "") {
                this._processServerResponse(uploadframe, response);
                this._setIframeProgress(filelist, 100, e);
                this._setStatus(filelist, "progress");
                this._successIframeUpload(filelist);
            }
            else { this._failureIframeUpload(filelist); }
        },
        //prog bar
        /**
* Specifies the progress state during IFrame upload
* @private
*/
        _setIframeProgress: function (filelist, percentage, e) {
            var progressbar, progress, filesize, loaded, total;
            progressbar = $(filelist).find(".e-file-progress-status");
            progressbar.width(percentage + "%");
        },
        /**
* Section for handling actions when IFrame upload is successfull
* @private
*/
        _successIframeUpload: function (filelist) {
            fileEntry = filelist.data("file");
            //trigger complete functionality
            if (this.model.removeUrl) {
                this._setAction(filelist, "remove");
                this._setStatus(filelist, "success");
            } else {
                filelist.find(".file-action").remove();
                this._setStatus(filelist, "success");
            }
            args = { files: fileEntry };
            this._trigger('complete', args);
        },
        /**
* Section for handling actions when IFrame upload fails
* @private
*/
        _failureIframeUpload: function (filelist) {
            fileEntry = filelist.data("file");
            //trigger Error functionality
            if (this.model.saveUrl) {
                this._setAction(filelist, "retry");
                this._setStatus(filelist, "failed");
            } else {
                filelist.find(".file-action").remove();
                this._setStatus(filelist, "failed");
            }
            args = { files: fileEntry };
            this._trigger('error', args);
        },
        /**
* Section for handling response from server
* @private
*/
        _processServerResponse: function (uploadframe) {
            var uploadform;
            uploadform = $(document.body).find("form[target='" + $(uploadframe).attr("id") + "']");
            setTimeout(function () {
                uploadform.remove();
                uploadframe.remove();
            }, 0);
        },
        /**
* Returns div block created
* @private
*/
        _createDivBlock: function (className) {
            return ej.buildTag('div.' + className);
        },
        /**
* Returns form tag as object
* @private
*/
        _createForm: function (action, target) {
            return ej.buildTag('form', '', {}, { enctype: 'multipart/form-data', method: 'POST', action: action, target: target });
        },
        /**
* Section for creating IFrame for handling IFrame uploads
* @private
*/
        _createFrame: function (id) {
            return ej.buildTag('iframe#' + id, '', { display: 'none' }, { name: id });
        },
        /**
* Returns file type input tag whwn id is specified 
* @private
*/
        _createInput: function (id) {
            return ej.buildTag('input', '', {}, { type: 'file', name: id });
        },
        //end
        /*Sync Start*/
        /**
* Section for synchronous upload
* @private
*/
        _initObjectsSyncUpload: function () {

            this.element.closest("form")
                    .attr("enctype", "multipart/form-data")
                    .attr("encoding", "multipart/form-data");
            this._wireEventsSyncUpload();
        },
        /**
* Binds events for handling sync upload 
* @private
*/
        _wireEventsSyncUpload: function () {
            var closestform = this.element.closest("form")[0];
            //form submit event. 
            this._on($(closestform), "submit", this._formSubmitSyncUpload);
            // form reset event.
            this._on($(closestform), "reset", this._formResetSyncUpload);
        },
        /**
* handles actions performed while sync upload 
* @private
*/
        _onSelectSyncUpload: function (e) {
            var input, files, selection, addedfile;
            input = $(e.target);
            files = this._getInputFileInfo(input);
            selection = $(".e-selectpart", this.control);
            this.element.find("input.e-uploadinput").removeClass("e-uploadinput").css("display", "none").appendTo(selection);
            this._createInputandBind();
            addedfile = this._pushFile(files, { "file": files, "Input": input });
        },
        /**
* Binds events for handling sync upload canceled
* @private
*/
        _onCancelSyncUpload: function (e, fileItem) {
            var inputfield = fileItem.data("Input");
            fileItem.data("file", null);
            fileItem.data("Input", null);
            fileItem.remove();
            inputfield.remove();
        },
        /**
* Performs upload during form submit
* @private
*/
        _formSubmitSyncUpload: function (e) {
            var input, uploader;
            input = $(".e-uploadinput", this.control);
            input.attr("name", "");
            uploader = this.control;
            setTimeout(function () {
                input.attr("name", uploader.id);
            }, 0);
        },
        /**
* Section for handling actions when form is reset.
* @private
*/
        _formResetSyncUpload: function (e) {
            $(".e-selectpart", this.control).children('input[type="file"]').each(function () {
                if (this.className == " ") {
                    $(this).remove();
                }
            });
            $(".e-ul", this.control).remove();
        }
        /*Sync End*/
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html TextArea
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejRTE
	* @requires jQuery
	* @requires ej.common.all
    * @requires ej.tools.all
	* @classdesc Custom Design for Html TextArea control.
	* @example 
	* &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
     <br> 
	* &lt;script&gt;
	* // Create RTE
    * $("#rteSample").ejRTE();
	* &lt;/script&gt;
	*/
    // "ej.RTE" is "namespace.className" will hold functions and properties
    var xhr = null;
    ej.widget("ejRTE", "ej.RTE", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        _rootCSS: "e-rte",
        validTags: ["textarea"],
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Enables/Disables the editing of the content.
			* @default 	True
			* @type {boolean}
			* @example 
			* // Initialize the RTE with the allowEdit value specified.
            * $("#rteSample").ejRTE({ allowEdit: false });
			* @example 
			* //Get or set the allowEdit API, after initialization:<br>
			*	// Get the allowEdit API value.		
			*	$("#rteSample").ejRTE('option', 'allowEdit');			
			*	// Set the allowEdit API
			*	$("#rteSample").ejRTE('option', 'allowEdit',  true);
			* @memberof ejRTE
			* @instance
			*/
            allowEdit: true,
            /**		
			* RTE control comments can access through keyboard shortcut keys.
			* @default 	True
			* @type {boolean}
			* @example 
			* // Initialize the RTE with the allowKeyboardNavigation value specified.
            * $("#rteSample").ejRTE({allowKeyboardNavigation: false });
			* @example 
			* //Get or set the allowKeyboardNavigation API, after initialization:<br>
			*	// Get the allowKeyboardNavigation API value.		
			*	$("#rteSample").ejRTE('option', 'allowKeyboardNavigation');			
			*	// Set the allowKeyboardNavigation API
			*	$("#rteSample").ejRTE('option', 'allowKeyboardNavigation',  true);
			* @memberof ejRTE
			* @instance
			*/
            allowKeyboardNavigation: true,
            /**		
            * Sets the root class for RTE theme. This cssClass API helps to use custom skinning option for RTE control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
            * //Initialize the RTE with the cssClass value specified
            * 	$("#rteSample").ejRTE({ cssClass: 'gradient-lime'});
            * @example 
            * Get or set the cssClass API, after initialization:<br>
            *	//Gets the cssClass  value
            *	$("#rteSample").ejRTE("option", "cssClass ");			
            *	//Sets the cssClass  value
            *	$("#rteSample").ejRTE("option", "cssClass ", "gradient-lime" );
            * @memberof ejRTE
            * @instance
            */
            cssClass: "",
            /**		
            * Defines the height of the RTE textbox.
            * @default Null
            * @type {string | number}
            * @example 
            * //Initialize the RTE height property with the  value specified
            * 	$("#rteSample").ejRTE({ height: 30 });
            * @example 
            * Get or set the height API, after initialization:<br>
            *	//Gets the height  value
            *	$("#rteSample").ejRTE("option", "height");			
            *	//Sets the height  value
            *	$("#rteSample").ejRTE("option", "height ", 30 );
            * @memberof ejRTE
            * @instance
            */
            height: "370",
            /**		
            * Defines the width of the RTE textbox.
            * @default Null
            * @type {string |number}
            * @example 
            * //Initialize the RTE width property with the  value specified
            * 	$("#rteSample").ejRTE({ width: 30 });
            * @example 
            * Get or set the width API, after initialization:<br>
            *	//Gets the height  value
            *	$("#rteSample").ejRTE("option", "width");			
            *	//Sets the width  value
            *	$("#rteSample").ejRTE("option", "width ", 30 );
            * @memberof ejRTE
            * @instance
            */
            width: "786",
            /**		
			* 	Enable | Disable the RTE control accessible or interaction..
			* @default 	True
			* @type {boolean}
			* @example 
			* // Initialize the RTE with the enabled value specified.
            * $("#rteSample").ejRTE({enabled: false });
			* @example 
			* //Get or set the enabled API, after initialization:<br>
			*	// Get the enabled API value.		
			*	$("#rteSample").ejRTE('option', 'enabled');			
			*	// Set the enabled API
			*	$("#rteSample").ejRTE('option', 'enabled',  true);
			* @memberof ejRTE
			* @instance
			*/
            enabled: true,
            /**		
			* Set the max Length  for RTE text. 
			* @default 	400
			* @type {number}
			* @example 
			* // Initialize the RTE with the maxLength value specified.
            * $("#rteSample").ejRTE({ maxLength: 900});
			* @example 
			* //Get or set the maxLength API, after initialization:<br>
			*	// Get the maxLength API value.		
			*	$("#rteSample").ejRTE('option', 'maxLength');			
			*	// Set the maxLength API
			*	$("#rteSample").ejRTE('option', 'maxLength',  900);
			* @memberof ejRTE
			* @instance
			*/
            maxLength: 7000,
            /**		
			* Set the minimum width for RTE outer wrapper element. 
			* @default 	400
			* @type {number}
			* @example 
			* // Initialize the RTE with the minWidth value specified.
            * $("#rteSample").ejRTE({ minWidth: 900});
			* @example 
			* //Get or set the minWidth API, after initialization:<br>
			*	// Get the minWidth API value.		
			*	$("#rteSample").ejRTE('option', 'minWidth');			
			*	// Set the minWidth API
			*	$("#rteSample").ejRTE('option', 'minWidth',  true);
			* @memberof ejRTE
			* @instance
			*/
            minWidth: 400,
            /**		
           * Set the maximum width for RTE outer wrapper element. 
           * @default 	400
           * @type {number}
           * @example 
           * // Initialize the RTE with the maxWidth value specified.
           * $("#rteSample").ejRTE({ maxWidth: 900});
           * @example 
           * //Get or set the maxWidth API, after initialization:<br>
           *	// Get the maxWidth API value.		
           *	$("#rteSample").ejRTE('option', 'maxWidth');			
           *	// Set the maxWidth API
           *	$("#rteSample").ejRTE('option', 'maxWidth',  true);
           * @memberof ejRTE
           * @instance
           */
            maxWidth: null,
            /**		
            * Set the minimum height for RTE outer wrapper element. 
            * @default 	280
            * @type {number}
            * @example 
            * // Initialize the RTE with the maxWidth value specified.
            * $("#rteSample").ejRTE({ minHeight: 900});
            * @example 
            * //Get or set the minHeight API, after initialization:<br>
            *	// Get the minHeight API value.		
            *	$("#rteSample").ejRTE('option', 'minHeight');			
            *	// Set the minHeight API
            *	$("#rteSample").ejRTE('option', 'minHeight',  true);
            * @memberof ejRTE
            * @instance
            */
            minHeight: 280,
            /**		
            * Set the maximum height for RTE outer wrapper element. 
            * @default 	280
            * @type  {string | number}
            * @example 
            * // Initialize the RTE with the maxHeight value specified.
            * $("#rteSample").ejRTE({ maxHeight: 900});
            * @example 
            * //Get or set the maxHeight API, after initialization:<br>
            *	// Get the maxHeight API value.		
            *	$("#rteSample").ejRTE('option', 'maxHeight');			
            *	// Set the maxHeight API
            *	$("#rteSample").ejRTE('option', 'maxHeight',  true);
            * @memberof ejRTE
            * @instance
            */
            maxHeight: null,
            /**		
             * Shows toolbar in RTE. 
             * @default 	True
             * @type {boolean}
             * @example 
             * // Initialize the RTE with the showToolBar value specified.
             * $("#rteSample").ejRTE({showToolBar: false });
             * @example 
             * //Get or set the showToolBar API, after initialization:<br>
             *	// Get the showToolBar API value.		
             * $("#rteSample").ejRTE('option', 'showToolBar');			
             *	// Set the showToolBar API
             *	$("#rteSample").ejRTE('option', 'showToolBar',  true);
             * @memberof ejRTE
             * @instance
             */
            showToolBar: true,
            /**		
            * Shows footer in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the showFooter value specified.
            * $("#rteSample").ejRTE({showFooter: false });
            * @example 
            * //Get or set the showToolBar API, after initialization:<br>
            *	// Get the showFooter API value.		
            *	$("#rteSample").ejRTE('option', 'showFooter');			
            *	// Set the showFooter API
            *	$("#rteSample").ejRTE('option', 'showFooter',  true);
            * @memberof ejRTE
            * @instance
            */
            showFooter: false,
            /**		
            * Shows HtmlSource in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the showHtmlSource value specified.
            * $("#rteSample").ejRTE({showHtmlSource: false });
            * @example 
            * //Get or set the showHtmlSource API, after initialization:<br>
            *	// Get the showHtmlSource API value.		
            *	$("#rteSample").ejRTE('option', 'showHtmlSource');			
            *	// Set the showHtmlSource API
            *	$("#rteSample").ejRTE('option', 'showHtmlSource',  true);
            * @memberof ejRTE
            * @instance
            */
            showHtmlSource: true,
            /**		
            * Shows WordCount in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the showWordCount value specified.
            * $("#rteSample").ejRTE({showWordCount: false });
            * @example 
            * //Get or set the showWordCount API, after initialization:<br>
            *	// Get the showWordCount API value.		
            *	$("#rteSample").ejRTE('option', 'showWordCount');			
            *	// Set the showWordCount API
            *	$("#rteSample").ejRTE('option', 'showWordCount',  true);
            * @memberof ejRTE
            * @instance
            */
            showWordCount: true,
            /**		
            * Shows HtmlTagInfo in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the showHtmlTagInfo value specified.
            * $("#rteSample").ejRTE({showHtmlTagInfo: false });
            * @example 
            * //Get or set the showHtmlTagInfo API, after initialization:<br>
            *	// Get the showHtmlTagInfo API value.		
            *	$("#rteSample").ejRTE('option', 'showHtmlTagInfo');			
            *	// Set the showHtmlTagInfo API
            *	$("#rteSample").ejRTE('option', 'showHtmlTagInfo',  true);
            * @memberof ejRTE
            * @instance
            */
            showHtmlTagInfo: true,
            /**		
            * Shows ClearAll in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the showClearAll value specified.
            * $("#rteSample").ejRTE({showClearAll: false });
            * @example 
            * //Get or set the showClearAll API, after initialization:<br>
            *	// Get the showClearAll API value.		
            *	$("#rteSample").ejRTE('option', 'showClearAll');			
            *	// Set the showClearAll API
            *	$("#rteSample").ejRTE('option', 'showClearAll',  true);
            * @memberof ejRTE
            * @instance
            */
            showClearAll: true,
            /**		
            * Sets the iframe attribute in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * // Initialize the RTE with the iframeAttribute value specified.
            * $("#rteSample").ejRTE({iframeAttribute: null });
            * @example 
            * //Get or set the iframeAttribute API, after initialization:<br>
            *	// Get the iframeAttribute API value.		
            *	$("#rteSample").ejRTE('option', 'iframeAttribute');			
            *	// Set the iframeAttribute API
            *	$("#rteSample").ejRTE('option', 'iframeAttribute',  null);
            * @memberof ejRTE
            * @instance
            */
            iframeAttribute: "color:#5C5C5C",
            /**		
            * Sets the iframe attribute in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * // Initialize the RTE with the iframeAttribute value specified.
            * $("#rteSample").ejRTE({iframeAttribute: null });
            * @example 
            * //Get or set the iframeAttribute API, after initialization:<br>
            *	// Get the iframeAttribute API value.		
            *	$("#rteSample").ejRTE('option', 'iframeAttribute');			
            *	// Set the iframeAttribute API
            *	$("#rteSample").ejRTE('option', 'iframeAttribute',  null);
            * @memberof ejRTE
            * @instance
            */
            showClearFormat: true,
            /**		
            * Shows FontOption in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the showFontOption value specified.
            * $("#rteSample").ejRTE({showFontOption: false });
            * @example 
            * //Get or set the showFontOption API, after initialization:<br>
            *	// Get the showFontOption API value.		
            *	$("#rteSample").ejRTE('option', 'showFontOption');			
            *	// Set the showFontOption API
            *	$("#rteSample").ejRTE('option', 'showFontOption',  true);
            * @memberof ejRTE
            * @instance
            */
            showFontOption: true,
            /**		
            * Shows Dimensions in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the showDimensions value specified.
            * $("#rteSample").ejRTE({showDimensions: false });
            * @example 
            * //Get or set the showDimensions API, after initialization:<br>
            *	// Get the showDimensions API value.		
            *	$("#rteSample").ejRTE('option', 'showDimensions');			
            *	// Set the showDimensions API
            *	$("#rteSample").ejRTE('option', 'showDimensions',  true);
            * @memberof ejRTE
            * @instance
            */
            showDimensions: false,
            /**		
            * Sets the localization in RTE. 
            * @default 	"en-US"
            * @type {string}
            * @example 
            * // Initialize the RTE with the localization value specified.
            * $("#rteSample").ejRTE({localization: "en-US" });
            * @example 
            * //Get or set the localization API, after initialization:<br>
            *	// Get the localization API value.		
            *	$("#rteSample").ejRTE('option', 'localization');			
            *	// Set the localization API
            *	$("#rteSample").ejRTE('option', 'localization',  'en-US');
            * @memberof ejRTE
            * @instance
            */
            localization: "en-US",
            /**		
            * Sets the name in RTE. 
            * @default 	""
            * @type {string}
            * @example 
            * // Initialize the RTE with the name value specified.
            * $("#rteSample").ejRTE({name: "ecommentblog" });
            * @example 
            * //Get or set the name API, after initialization:<br>
            *	// Get the name API value.		
            *	$("#rteSample").ejRTE('option', 'name');			
            *	// Set the name API
            *	$("#rteSample").ejRTE('option', 'name',  'commentblog');
            * @memberof ejRTE
            * @instance
            */
            name: "",
            /**		
            * Sets the tools in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
                //font: ["fontName", "fontSize", "fontColor", "backgrounColor"],
                formatStyle: ["format"],
                style: ["bold", "italic", "underline", "strikethrough"],
                alignment: ["justifyLeft", "justifyCenter", "justifyRight", "justifyFull"],
                lists: ["unorderedList", "orderedList"],
                indenting: ["outdent", "indent"],
                //copyPaste: ["cut", "copy", "paste"],
                doAction: ["undo", "redo"],
                //clear: ["clearFormat", "clearAll"],
                links: ["createLink"],
                images: ["image", "video"],
                tables: ["createTable", "addRowAbove", "addRowBelow", "addColumnLeft", "addColumnRight", "deleteRow", "deleteColumn", "deleteTable"],
                //editTable:[""],
                //scripts: ["superscript", "subscript"],
                //casing: ["upperCase", "lowerCase"],
                //paragraph: ["paragraph"],
                //custom: null
            }});
            * @example 
            * //Get or set the tools API, after initialization:<br>
            *	// Get the tools API value.		
            *	$("#rteSample").ejRTE('option', 'tools');			
            * @memberof ejRTE
            * @instance
            */
            tools: /** @lends ejRTE# */{
                //font: ["fontName", "fontSize", "fontColor", "backgrounColor"],
                /**		
                * Include Format styles in RTE toolbar.
                * @alias ejRTE#tools->formatStyle
                * @type Object
                */
                formatStyle: ["format"],
                /**		
                * Include font styles in RTE toolbar.
                * @alias ejRTE#tools->style
                * @type Object
                */
                style: ["bold", "italic", "underline", "strikethrough"],
                /**		
                * Include font alignment options in RTE toolbar.
                * @alias ejRTE#tools->alignment
                * @type Object
                */
                alignment: ["justifyLeft", "justifyCenter", "justifyRight", "justifyFull"],
                /**		
                * Include list type options in RTE toolbar.
                * @alias ejRTE#tools->lists
                * @type Object
                */
                lists: ["unorderedList", "orderedList"],
                /**		
                * Include text indenting options in RTE toolbar.
                * @alias ejRTE#tools->indenting
                * @type Object
                */
                indenting: ["outdent", "indent"],
                //copyPaste: ["cut", "copy", "paste"],
                /**		
                * Include action options in RTE toolbar.
                * @alias ejRTE#tools->doAction
                * @type Object
                */
                doAction: ["undo", "redo"],
                //clear: ["clearFormat", "clearAll"],
                /**		
                * Include links options in RTE toolbar.
                * @alias ejRTE#tools->links
                * @type Object
                */
                links: ["createLink"],
                /**		
                * Include image  and video options in RTE toolbar.
                * @alias ejRTE#tools->images
                * @type Object
                */
                images: ["image", "video"],
                /**		
                * Include tables options in RTE toolbar.
                * @alias ejRTE#tools->tables
                * @type Object
                */
                tables: ["createTable", "addRowAbove", "addRowBelow", "addColumnLeft", "addColumnRight", "deleteRow", "deleteColumn", "deleteTable"],
                //editTable:[""],
                //scripts: ["superscript", "subscript"],
                //casing: ["upperCase", "lowerCase"],
                //paragraph: ["paragraph"],
                //custom: null
            },
            /**		
            * Sets the toolsList in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * // Initialize the RTE with the toolsList value specified.
            * $("#rteSample").ejRTE({ toolsList: ["formatStyle", "font", "style", "scripts", "alignment", "lists", "indenting", "copyPaste", "doAction", "clear", "links", "images", "tables", "casing", "custom"]});
            * @example 
            * //Get or set the toolsList API, after initialization:<br>
            *	// Get the toolsList API value.		
            *	$("#rteSample").ejRTE('option', 'toolsList');			
            * @memberof ejRTE
            * @instance
            */
            toolsList: ["formatStyle", "font", "style", "scripts", "alignment", "lists", "indenting", "copyPaste", "doAction", "clear", "links", "images", "tables", "casing", "custom"],
            /**		
            * Sets the colorCode in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * // Initialize the RTE with the colorCode value specified.
            * $("#rteSample").ejRTE({ colorCode: [
			            "000000", "FFFFFF", "C4C4C4", "ADADAD", "595959", "262626", "4f81bd", "dbe5f1", "b8cce4", "95b3d7", "366092", "244061", "c0504d", "f2dcdb", "e5b9b7", "d99694", "953734",
			            "632423", "9bbb59", "ebf1dd", "d7e3bc", "c3d69b", "76923c", "4f6128", "8064a2", "e5e0ec", "ccc1d9", "b2a2c7", "5f497a", "3f3151", "f79646", "fdeada", "fbd5b5", "fac08f",
			            "e36c09", "974806"
            ]});
            * @example 
            * //Get or set the colorCode API, after initialization:<br>
            *	// Get the colorCode API value.		
            *	$("#rteSample").ejRTE('option', 'colorCode');			
            * @memberof ejRTE
            * @instance
            */
            colorCode: [
			            "000000", "FFFFFF", "C4C4C4", "ADADAD", "595959", "262626", "4f81bd", "dbe5f1", "b8cce4", "95b3d7", "366092", "244061", "c0504d", "f2dcdb", "e5b9b7", "d99694", "953734",
			            "632423", "9bbb59", "ebf1dd", "d7e3bc", "c3d69b", "76923c", "4f6128", "8064a2", "e5e0ec", "ccc1d9", "b2a2c7", "5f497a", "3f3151", "f79646", "fdeada", "fbd5b5", "fac08f",
			            "e36c09", "974806"
            ],
            /**		
            * Sets the format in RTE. 
            * @default 	""
            * @type {string}
            * @example 
            * // Initialize the RTE with the format value specified.
            * $("#rteSample").ejRTE({format: [
                { text: "Paragraph", value: "<p>", spriteCSS: "e-paragraph" },
                { text: "Quotation", value: "<blockquote>", spriteCSS: "e-quotation" },
                { text: "Heading 1", value: "<h1>", spriteCSS: "e-h1" },
                { text: "Heading 2", value: "<h2>", spriteCSS: "e-h2" },
                { text: "Heading 3", value: "<h3>", spriteCSS: "e-h3" },
                { text: "Heading 4", value: "<h4>", spriteCSS: "e-h4" },
                { text: "Heading 5", value: "<h5>", spriteCSS: "e-h5" },
                { text: "Heading 6", value: "<h6>", spriteCSS: "e-h6" }
            ]});
            * @example 
            * //Get or set the format API, after initialization:<br>
            *	// Get the format API value.		
            *	$("#rteSample").ejRTE('option', 'format');			
            * @memberof ejRTE
            * @instance
            */
            format: [
                { text: "Paragraph", value: "<p>", spriteCSS: "e-paragraph" },
                { text: "Quotation", value: "<blockquote>", spriteCSS: "e-quotation" },
                { text: "Heading 1", value: "<h1>", spriteCSS: "e-h1" },
                { text: "Heading 2", value: "<h2>", spriteCSS: "e-h2" },
                { text: "Heading 3", value: "<h3>", spriteCSS: "e-h3" },
                { text: "Heading 4", value: "<h4>", spriteCSS: "e-h4" },
                { text: "Heading 5", value: "<h5>", spriteCSS: "e-h5" },
                { text: "Heading 6", value: "<h6>", spriteCSS: "e-h6" }
            ],
            /**		
            * Sets the fontName in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * // Initialize the RTE with the fontName value specified.
            * $("#rteSample").ejRTE({format: [
                { text: "Paragraph", value: "<p>", spriteCSS: "e-paragraph" },
                { text: "Quotation", value: "<blockquote>", spriteCSS: "e-quotation" },
                { text: "Heading 1", value: "<h1>", spriteCSS: "e-h1" },
                { text: "Heading 2", value: "<h2>", spriteCSS: "e-h2" },
                { text: "Heading 3", value: "<h3>", spriteCSS: "e-h3" },
                { text: "Heading 4", value: "<h4>", spriteCSS: "e-h4" },
                { text: "Heading 5", value: "<h5>", spriteCSS: "e-h5" },
                { text: "Heading 6", value: "<h6>", spriteCSS: "e-h6" }
            ]});
            * @example 
            * //Get or set the fontName API, after initialization:<br>
            *	// Get the fontName API value.		
            *	$("#rteSample").ejRTE('option', 'fontName');			
            * @memberof ejRTE
            * @instance
            */
            fontName: [
                       {
                           text: "Segoe UI",
                           value: "Segoe UI"
                       },
                       {
                           text: "Arial",
                           value: "Arial,Helvetica,sans-serif"
                       },
                       {
                           text: "Courier New",
                           value: "'Courier New',Courier,monospace"
                       },
                       {
                           text: "Georgia",
                           value: "Georgia,serif"
                       },
                       {
                           text: "Impact",
                           value: "Impact,Charcoal,sans-serif"
                       },
                       {
                           text: "Lucida Console",
                           value: "'Lucida Console',Monaco,monospace"
                       },
                       {
                           text: "Tahoma",
                           value: "Tahoma,Geneva,sans-serif"
                       },
                       {
                           text: "Times New Roman",
                           value: "'Times New Roman',Times,serif"
                       },
                       {
                           text: "Trebuchet MS",
                           value: "'Trebuchet MS',Helvetica,sans-serif"
                       },
                       {
                           text: "Verdana",
                           value: "Verdana,Geneva,sans-serif"
                       }
            ],
            /**		
            * Sets the fontSize in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * // Initialize the RTE with the fontSize value specified.
            * $("#rteSample").ejRTE({ fontSize: [
                {
                    text: "1",
                    value: "1"
                },
                {
                    text: "2 (10pt)",
                    value: "2"
                },
                {
                    text: "3 (12pt)",
                    value: "3"
                },
                {
                    text: "4 (14pt)",
                    value: "4"
                },
                {
                    text: "5 (18pt)",
                    value: "5"
                },
                {
                    text: "6 (24pt)",
                    value: "6"
                },
                {
                    text: "7 (36pt)",
                    value: "7"
                }
            ]});
            * @example 
            * //Get or set the fontSize API, after initialization:<br>
            *	// Get the fontSize API value.		
            *	$("#rteSample").ejRTE('option', 'fontSize');			
            * @memberof ejRTE
            * @instance
            */
            fontSize: [
                {
                    text: "1",
                    value: "1"
                },
                {
                    text: "2 (10pt)",
                    value: "2"
                },
                {
                    text: "3 (12pt)",
                    value: "3"
                },
                {
                    text: "4 (14pt)",
                    value: "4"
                },
                {
                    text: "5 (18pt)",
                    value: "5"
                },
                {
                    text: "6 (24pt)",
                    value: "6"
                },
                {
                    text: "7 (36pt)",
                    value: "7"
                }
            ],
            /**		
            * Given number for rows render the insert table pop. 
            * @default 	""
            * @type {number}
            * @example 
            * // Initialize the RTE with the 'tableRows' value specified.
            * $("#rteSample").ejRTE({tableRows: 70 });
            * @example 
            * //Get or set the tableRows API, after initialization:<br>
            *	// Get the tableRows API value.		
            *	$("#rteSample").ejRTE('option', 'tableRows');			
            *	// Set the tableRows API
            *	$("#rteSample").ejRTE('option', 'tableRows', 5 );
            * @memberof ejRTE
            * @instance
            */
            tableRows: 7,
            /**		
            * Given number for columns render the insert table pop. 
            * @default 	""
            * @type {number}
            * @example 
            * // Initialize the RTE with the 'tableColumns' value specified.
            * $("#rteSample").ejRTE({tableColumns: 70 });
            * @example 
            * //Get or set the tableColumns API, after initialization:<br>
            *	// Get the tableColumns API value.		
            *	$("#rteSample").ejRTE('option', 'tableColumns');			
            *	// Set the tableColumns API
            *	$("#rteSample").ejRTE('option', 'tableColumns', 5 );
            * @memberof ejRTE
            * @instance
            */
            tableColumns: 9,
            /**		
            * Given number for rows render the color palete pop up.
            * @default 	""
            * @type {number}
            * @example 
            * // Initialize the RTE with the 'colorPaletteRows' value specified.
            * $("#rteSample").ejRTE({colorPaletteRows: 70 });
            * @example 
            * //Get or set the colorPaletteRows API, after initialization:<br>
            *	// Get the colorPaletteRows API value.		
            *	$("#rteSample").ejRTE('option', 'colorPaletteRows');			
            *	// Set the colorPaletteRows API
            *	$("#rteSample").ejRTE('option', 'colorPaletteRows', 5 );
            * @memberof ejRTE
            * @instance
            */
            colorPaletteRows: 6,
            /**		
            * Given number for columns render the color palete pop up. 
            * @default 	""
            * @type {number}
            * @example 
            * // Initialize the RTE with the 'colorPaletteColumns' value specified.
            * $("#rteSample").ejRTE({colorPaletteColumns: 70 });
            * @example 
            * //Get or set the colorPaletteColumns API, after initialization:<br>
            *	// Get the colorPaletteColumns API value.		
            *	$("#rteSample").ejRTE('option', 'colorPaletteColumns');			
            *	// Set the colorPaletteColumns API
            *	$("#rteSample").ejRTE('option', 'colorPaletteColumns', 5 );
            * @memberof ejRTE
            * @instance
            */
            colorPaletteColumns: 6,
            /**		
            * Shows CustomTable in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the showCustomTable value specified.
            * $("#rteSample").ejRTE({showCustomTable: false });
            * @example 
            * //Get or set the showCustomTable API, after initialization:<br>
            *	// Get the showCustomTable API value.		
            *	$("#rteSample").ejRTE('option', 'showCustomTable');			
            *	// Set the showCustomTable API
            *	$("#rteSample").ejRTE('option', 'showCustomTable',  true);
            * @memberof ejRTE
            * @instance
            */
            showCustomTable: true,
            /**		
            * Given string value to display in the editable area. 
            * @default 	""
            * @type {string}
            * @example 
            * // Initialize the RTE with the 'value' value specified.
            * $("#rteSample").ejRTE({value: "The Rich Text Editor (RTE) control is an easy to render in client side. Customer easy to edit the contents, insert table, images and get the HTML content for the displayed content." });
            * @example 
            * //Get or set the value API, after initialization:<br>
            *	// Get the value API value.		
            *	$("#rteSample").ejRTE('option', 'value');			
            *	// Set the value API
            *	$("#rteSample").ejRTE('option', 'value',  'commentblog');
            * @memberof ejRTE
            * @instance
            */
            value: null,
            /**		
            * Undo operation provide the number of step limit. 
            * @default 	""
            * @type {number}
            * @example 
            * // Initialize the RTE with the 'vaundoStackLimitlue' value specified.
            * $("#rteSample").ejRTE({undoStackLimit: 70 });
            * @example 
            * //Get or set the undoStackLimit API, after initialization:<br>
            *	// Get the undoStackLimit API value.		
            *	$("#rteSample").ejRTE('option', 'undoStackLimit');			
            *	// Set the undoStackLimit API
            *	$("#rteSample").ejRTE('option', 'undoStackLimit', 70 );
            * @memberof ejRTE
            * @instance
            */
            undoStackLimit: 50,
            /**		
            * Shows resizable in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the resizable value specified.
            * $("#rteSample").ejRTE({resizable: false });
            * @example 
            * //Get or set the resizable API, after initialization:<br>
            *	// Get the resizable API value.		
            *	$("#rteSample").ejRTE('option', 'resizable');			
            *	// Set the resizable API
            *	$("#rteSample").ejRTE('option', 'resizable',  true);
            * @memberof ejRTE
            * @instance
            */
            resizable: true,
            /**		
            *  persist the values in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the persist value specified.
            * $("#rteSample").ejRTE({persist: false });
            * @example 
            * //Get or set the persist API, after initialization:<br>
            *	// Get the persist API value.		
            *	$("#rteSample").ejRTE('option', 'persist');			
            *	// Set the persist API
            *	$("#rteSample").ejRTE('option', 'persist',  true);
            * @memberof ejRTE
            * @instance
            */
            persist: false,
            /**		
            * shows rtl in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * // Initialize the RTE with the rtl value specified.
            * $("#rteSample").ejRTE({rtl: false });
            * @example 
            * //Get or set the rtl API, after initialization:<br>
            *	// Get the rtl API value.		
            *	$("#rteSample").ejRTE('option', 'rtl');			
            *	// Set the rtl API
            *	$("#rteSample").ejRTE('option', 'rtl',  true);
            * @memberof ejRTE
            * @instance
            */
            rtl: false,
            //events
            /**     
            * Fires when change successfully.
            * @event
            * @name ejRTE#change 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the RTE model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //change event for RTE
            * $("#rteSample").ejRTE({ 
            *   	change: function(args) {
            * });      
            * @memberof ejRTE
            * @instance
            */
            change: null,
            /**     
            * Fires when execute successfully.
            * @event
            * @name ejRTE#execute 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the RTE model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //execute event for RTE
            * $("#rteSample").ejRTE({ 
            *   	execute: function(args) {
            * });      
            * @memberof ejRTE
            * @instance
            */
            execute: null,
            /**     
            * Fires when keydown successfully.
            * @event
            * @name ejRTE#keydown 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the RTE model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //keydown event for RTE
            * $("#rteSample").ejRTE({ 
            *   	keydown: function(args) {
            * });      
            * @memberof ejRTE
            * @instance
            */
            keydown: null,
            /**     
            * Fires when keyup successfully.
            * @event
            * @name ejRTE#keyup 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the RTE model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //keyup event for RTE
            * $("#rteSample").ejRTE({ 
            *   	keyup: function(args) {
            * });      
            * @memberof ejRTE
            * @instance
            */
            keyup: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            iframeAttribute: "string",
            cssClass: "string",
            name: "string",
            enabled: "boolean",
            persist: "boolean",
            tools: "data",
            toolsList: "data",
            colorCode: "data",
            format: "data",
            fontName: "data",
            fontSize: "data"
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "allowEdit": this._enableEdit(options[option]); break;
                    case "showToolBar": {
                        options[option] ? ej.isNullOrUndefined(this._rteToolbar) ? this._renderToolBar() : this._rteToolbar.show() : this._rteToolbar.hide();
                        this._setIframeHeight();
                        break;
                    }
                    case "showFooter": {
                        this.model.showFooter = options[option];
                        options[option] ? ej.isNullOrUndefined(this._rteFooter) ? this._renderFooter() : this._rteFooter.show() : this._rteFooter.hide();
                        this._setIframeHeight();
                        break;
                    }
                    case "enabled": this._disabled(!options[option]); break;
                    case "height": this._rteWapper.height(options[option]); this._setIframeHeight(); break;
                    case "width": this._rteWapper.width(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showHtmlSource": this._footerElement("div.e-rte-source", options[option]); break;
                    case "showHtmlTagInfo": this._footerElement("div.e-rte-htmltaginfo", options[option]); break;
                    case "showWordCount": this._footerElement("div.e-rte-wordcount", options[option]); break;
                    case "showClearAll": this._footerElement("div.clearAll", options[option]); break;
                    case "showClearFormat": this._footerElement("div.clearFormat", options[option]); break;
                    case "resizable": this._footerElement("div.e-rte-resize", options[option]); break;
                    case "fontName": this._fontStyleDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "fontSize": this._fontSizeDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "format": this._formatDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "value": this._getDocument().body.innerHTML = ej.util.getVal(options[option]); break;
                    case "showDimensions":
                        if (options[option]) {
                            this._rteWapper.find("#" + this._rteId + "_imgdimensions").show();
                            this._rteWapper.find("#" + this._rteId + "_videodimensions").show();
                        }
                        else {
                            this._rteWapper.find("#" + this._rteId + "_imgdimensions").hide();
                            this._rteWapper.find("#" + this._rteId + "_videodimensions").hide();
                        }
                        break;
                    case "localization":
                        this.model.localization = options[option];
                        var model = this.model;
                        this.element.ejRTE("destroy").ejRTE(model);
                        break;
                    case "tableRows": {
                        this.model.tableRows = options[option];
                        var tblDiv = this._createTable.find("div.e-rte-table").html("");
                        this._drawTable(tblDiv);
                        break;
                    }
                    case "iframeAttribute": {
                        this._updateIframeSkin(options[option]);
                        break;
                    }
                    case "tableColumns": {
                        this.model.tableColumns = options[option];
                        var tblDiv = this._createTable.find("div.e-rte-table").html("");
                        this._drawTable(tblDiv);
                        break;
                    }
                    case "showCustomTable": {
                        var custLink = this._createTable.find("div.customtable-group");
                        options[option] ? ej.isNullOrUndefined(this._customTableDialog) ? this._createCustomTable() : custLink.show() : custLink.hide();
                        break;
                    }
                    case "colorPaletteRows": {
                        this.model.colorPaletteRows = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "colorPaletteColumns": {
                        this.model.colorPaletteColumns = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "colorCode": {
                        this.model.colorCode = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "rtl": this._enableRTL(options[option]); break;
                }
            }
        },

        // constructor function
        _init: function () {
            if (!ej.isNullOrUndefined(this.element) && this.element[0].type == "textarea") {
                this._initialize();
                this._render();
                this.model.allowEdit && this._wireEvents();
                this._updateIframeSkin(this.model.iframeAttribute);
            }
        },

        _destroy: function () {
            this.element.insertBefore(this._rteWapper);
            !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontColorSplit) && this._fontColorSplit.ejSplitButton("destroy");
            !ej.isNullOrUndefined(this._bgColorSplit) && this._bgColorSplit.ejSplitButton("destroy");
            !ej.isNullOrUndefined(this._customTableDialog.find("#" + this._rteId + "_ddlAlignment")) && this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").ejDropDownList("destroy");
            this._rteWapper.remove();
            this.element.show();
            this._createTable = null;
        },

        _initialize: function () {
            this._toolsList = this.model.toolsList;
            this._rteId = this.element[0].id;
            this._backupArray = new Array();
            this._undoRedoPosition = 0;
            this._fontColor = "";
            this._bgColor = "";
            this._imgWidth = 0;
            this._imgHeight = 0;
            this._videoWidth = 0;
            this._videoHeight = 0;
            this._keypressFlag = true;
            this._styleItems = ej.isNullOrUndefined(this.model.tools["style"]) ? [] : this.model.tools["style"];
            this._alignItems = ej.isNullOrUndefined(this.model.tools["alignment"]) ? [] : this.model.tools["alignment"];
            this._listItems = ej.isNullOrUndefined(this.model.tools["lists"]) ? [] : this.model.tools["lists"];
            this._scriptsItems = ej.isNullOrUndefined(this.model.tools["scripts"]) ? [] : this.model.tools["scripts"];
        },

        _render: function () {
            this._renderWrapper();
            this._checkNameAttr();
            if (this.model.showToolBar) {
                this._renderToolBar();
            }
            this._renderEditArea();
            this.model.showFooter && this._renderFooter();
            this._setIframeHeight();
            this._renderAlertDialog();
            this._focus();
        },
        _checkNameAttr: function () {
            if (!this.model.name)
                this.element.attr({ "name": this.element[0].id });
            else
                this.element.attr({ "name": this.model.name });
        },
        _enableEdit: function (args) {
            if (args) {
                this._wireEvents();
                this.model.showToolBar && this._toolBarItems.ejToolbar("option", "click", this._toolBarClick);
                this._rteIframe.contents().find("body").attr("contenteditable", true);
                //this._getDocument().designMode = "On"
            }
            else {
                this._unwireEvents();
                this.model.showToolBar && this._toolBarItems.ejToolbar("option", "click", null);
                this._rteIframe.contents().find("body").attr("contenteditable", false);
                //this._getDocument().designMode = "off"
            }
        },
        _changeSkin: function (value) {
            this._rteWapper.removeClass(this.model.cssClass).addClass(value);
            this._subControlsSetModel("cssClass", value);
        },

        _updateIframeSkin: function (value) {
            var target = this._rteIframe.contents().find("body");
            this.model.rtl === true ? (target.attr("style", value), this._enableRTL(this.model.rtl)) : target.attr("style", value);
        },

        // RTE wrapper creating.
        _renderWrapper: function () {
            this._rteWapper = ej.buildTag("div.e-rte e-rte-wrapper " + this.model.cssClass + "#" + this._rteId + "_wrapper", "", {}, { role: "presentation" }).insertBefore(this.element);
            this._rteWapper.height(this.model.height).width(this.model.width);
            this._rteWapper.append(this.element.hide());
        },
        //Toolbar creating
        _renderToolBar: function () {
            this._rteToolbar = ej.buildTag("div.e-rte-toolbar #" + this._rteId + "_tools").insertAfter(this.element);
            this._crateToolbarTemplate();
            var model = {};
            model.click = this.model.allowEdit ? this._toolBarClick : null;
            model.cssClass = this.model.cssClass;
            model.rtl = this.model.rtl;
            this._toolBarItems.ejToolbar(model);

            //Init the toolbar items
            this._initToolBarItems();

        },

        _crateToolbarTemplate: function () {
            this._toolBarItems = ej.buildTag("div#" + this._rteId + "_toolbar").appendTo(this._rteToolbar).height(30);
            for (var items in this._toolsList) {
                items = this._toolsList[items];
                if (!ej.isNullOrUndefined(this.model.tools[items])) {
                    if (items == "custom")
                        !ej.isNullOrUndefined(this.model.tools[items]) && this._customTools(this.model.tools[items]);
                    else
                        this.model.tools[items].length > 0 && this._createToolsItems(this.model.tools[items], items);
                }
            }
        },

        _createToolsItems: function (items, itemName) {
            if (itemName == "font" && !this.model.showFontOption)
                return false;
            var ulTag = ej.buildTag("ul#" + (this._rteId + itemName)), liTag;
            (itemName == "font") && ulTag.addClass("e-rte-fontgroup");
            (itemName == "formatStyle") && ulTag.addClass("e-rte-format");
            for (var i = 0; i < items.length; i++) {
                liTag = $("<li id='" + (this._rteId + items[i]) + "' title='" + this._getLocalizedLabels(items[i]) + "' ><div class='e-rte-toolbar-icon " + items[i] + "'></div></li>");
                liTag.appendTo(ulTag);
            }
            ulTag.appendTo(this._toolBarItems);
        },

        _customTools: function (toolbarItems) {
            for (var item in toolbarItems) {
                var ulTag = ej.buildTag("ul"), liTag;
                liTag = $("<li id='" + toolbarItems[item].name + "' title='" + toolbarItems[item].tooltip + "' ><div class='" + (ej.isNullOrUndefined(toolbarItems[item].css) ? "" : toolbarItems[item].css) + "'></div></li>");
                !ej.isNullOrUndefined(toolbarItems[item].action) && this._on(liTag, "click", toolbarItems[item].action);
                $(toolbarItems[item].template).appendTo(liTag.find("div"));
                liTag.appendTo(ulTag);
                ulTag.appendTo(this._toolBarItems);
            }
        },

        _initToolBarItems: function () {
            this._toolBarObj = this._toolBarItems.ejToolbar("instance");
            this._rteToolbar.find("#" + this._rteId + "format").length > 0 && this._renderFormat();
            this._rteToolbar.find("#" + this._rteId + "createLink").length > 0 && this._renderLinkDialog();
            this._rteToolbar.find("#" + this._rteId + "video").length > 0 && this._renderVideoDialog();
            this._rteToolbar.find("#" + this._rteId + "image").length > 0 && this._renderImageDialog();
            this._rteToolbar.find("#" + this._rteId + "createTable").length > 0 && this._renderTableDialog();
            this._rteToolbar.find("#" + this._rteId + "fontName").length > 0 && this._renderFontStyle();
            this._rteToolbar.find("#" + this._rteId + "fontSize").length > 0 && this._renderFontSize();
            this._rteToolbar.find("#" + this._rteId + "fontColor").length > 0 && this._renderFontColor();
            this._rteToolbar.find("#" + this._rteId + "backgrounColor").length > 0 && this._renderBGColor();
            !this.model.showClearAll && this._rteToolbar.find("#" + this._rteId + "clearAll").hide();
            !this.model.showClearAll && this._rteToolbar.find("#" + this._rteId + "clearAll").hide();
            this._toolBarObj.disableItembyID(this._rteId + "undo");
            this._toolBarObj.disableItembyID(this._rteId + "redo");
            this._toolBarObj.selectItembyID(this._rteId + "justifyLeft");
            //Hide the editable table icons
            this._toolBarItems.find("#" + this._rteId + "addColumnLeft, #" + this._rteId + "addColumnRight, #" + this._rteId + "addRowAbove, #"
             + this._rteId + "addRowBelow, #" + this._rteId + "deleteRow, #" + this._rteId + "deleteColumn, #" + this._rteId + "deleteTable").hide();
        },
        //Editable Area
        _renderEditArea: function () {
            var _rteEditer = ej.buildTag("div.editarea #" + this._rteId + "_editer").appendTo(this._rteWapper);
            this._rteIframe = ej.buildTag("iframe.content-iframe #" + this._rteId + "_Iframe").appendTo(_rteEditer);
            this._setIFrames();
        },

        //Sets the editor and source editors
        _setIFrames: function () {
            //var RTE = $find(this._rteId);
            this.value(this.value() == null ? $.trim(this.element[0].value) : this.value());
            this._iframeId = this._rteId + "_Iframe";
            var _htmlText = ("<!DOCTYPE html><html><head><meta charset='utf-8' /><style>html,body{padding:0;margin:-1px 5px 0;height:100%;min-height:100%; padding-top: 1px;font-size: 13px;font-family:Segoe UI;word-wrap: break-word;-webkit-nbsp-mode: space;-webkit-line-break: after-white-space;}"
                + "h1{font-size:2em;margin:.67em 0}h2{font-size:1.5em}h3{font-size:1.16em}h4{font-size:1em}h5{font-size:.83em}h6{font-size:.7em}"
                + "p{margin:0 0 1em;padding:0 .2em} ul,ol{padding-left:2.5em} a{color:#00a} blockquote{margin-right: 0}"
                + ".e-rte-table{border-spacing:0;margin: 0.2em 0 0.5em;border: 1px solid;}.e-rte-table caption{border: 1px solid;border-bottom:none}"
                + ".e-rte-table,.e-rte-table td{outline:0;border-collapse: collapse; min-height:1px;vertical-align: top;text-indent: 5px;} .e-rte-table p{margin:0;padding:0;}"
                + "</style></head><body autocorrect='off' contenteditable=" + (this.model.allowEdit ? 'true' : 'false') + ">" + $.trim(this.value()) + "</body></html>");
            this._rteIframe.css({ "width": "100%" });
            //this.model.allowEdit && (this._getDocument().designMode = "On");
            this._setContent(_htmlText);
            if (!ej.isNullOrUndefined(this.model.undoStackLimit) || this.model.undoStackLimit !== "" || this.model.undoStackLimit > 0)
                this._backupArray[0] = this.getHtmlString();
        },

        //Set the Iframe height 
        _setIframeHeight: function () {
            var _height = (!ej.isNullOrUndefined(this._rteToolbar) && this._rteToolbar.is(':visible') ? this._rteToolbar.outerHeight() : 6) + (!ej.isNullOrUndefined(this._rteFooter) && this._rteFooter.is(':visible') ? this._rteFooter.outerHeight() : 0);
            this._rteIframe.height(_height == 0 ? this.model.height : (this._rteWapper.outerHeight() - _height) - 1);
        },

        // Footer Div render 
        _renderFooter: function () {
            var options = this.model;
            this._rteFooter = ej.buildTag("div.e-rte-footer #" + this._rteId + "_footer").appendTo(this._rteWapper);
            var _leftPan = ej.buildTag("div.e-rte-footer-left");
            var _htmlSource = ej.buildTag("div.e-rte-icons e-rte-footeritems e-rte-footericon e-rte-source", "", {}, { title: this._getLocalizedLabels("viewHtml") }).appendTo(_leftPan);
            this._htmlInfo = ej.buildTag("div.e-rte-htmltaginfo e-rte-footeritems").appendTo(_leftPan);
            _leftPan.appendTo(this._rteFooter);
            var _rightPan = ej.buildTag("div.e-rte-footer-right");
            this._wordCount = ej.buildTag("div.e-rte-wordcount e-rte-footeritems").appendTo(_rightPan);
            var _clearFormat = ej.buildTag("div.e-rte-icons clearFormat e-rte-footericon e-rte-footeritems", "", {}, { title: this._getLocalizedLabels("clearFormat") }).appendTo(_rightPan);
            var _clearAll = ej.buildTag("div.e-rte-icons clearAll e-rte-footericon e-rte-footeritems", "", {}, { title: this._getLocalizedLabels("clearAll") }).appendTo(_rightPan);
            var _resize = ej.buildTag("div.e-icons e-rte-resize e-rte-footeritems").appendTo(_rightPan);
            _rightPan.appendTo(this._rteFooter);
            this._on(_htmlSource, "click", this._sourceCodeManager);
            this._on(_clearFormat, "click", this._onClearFormat);
            this._on(_clearAll, "click", this._clearAllManager);
            !this.model.showHtmlSource && _htmlSource.hide().removeClass("e-rte-footeritems");
            !this.model.showHtmlTagInfo && this._htmlInfo.hide().removeClass("e-rte-footeritems");
            !this.model.showWordCount && this._wordCount.hide().removeClass("e-rte-footeritems");
            !this.model.showClearAll && _clearAll.hide().removeClass("e-rte-footeritems");
            !this.model.showClearFormat && _clearFormat.hide().removeClass("e-rte-footeritems");
            if (options.resizable)
                this._enableResize();
            else
                _resize.hide().removeClass("e-rte-footeritems");
            this._updateCharCount();
        },
        //Enable Footer element
        _footerElement: function (selector, visible) {
            var ele = this._rteFooter.find(selector);
            visible ? ele.show().addClass("e-rte-footeritems") : ele.hide().addClass("e-rte-footeritems");
        },
        //Update Color Palettes
        _updateColorPalette: function () {
            var _fontColor = $("ul#" + this._rteId + "_colorTable");
            var _bgColor = $("ul#" + this._rteId + "_colorBGTable");
            _fontColor.find("div.e-rte-table").remove();
            _fontColor.find("li").append(this._colorTable());
            //Bg Color table Update
            _bgColor.find("div.e-rte-table").remove();
            _bgColor.find("li").append(this._colorTable());
        },

        //Enable RTE
        _enableRTL: function (value) {
            if (value) {
                //this._rteWapper.addClass("e-rtl");
                this._rteIframe.contents().find("body").css("direction", "rtl");
            }
            else {
                this._rteIframe.contents().find("body").css("direction", "");
            }
            this._subControlsSetModel("rtl", value);
        },

        _subControlsSetModel: function (prop, value) {
            !ej.isNullOrUndefined(this._toolBarItems) && this._toolBarItems.ejToolbar({ prop: value });
            !ej.isNullOrUndefined(this._linkDialog) && this._linkDialog.ejDialog({ prop: value });
            !ej.isNullOrUndefined(this._imgDialog) && this._imgDialog.ejDialog({ prop: value });
            !ej.isNullOrUndefined(this._videoDialog) && this._videoDialog.ejDialog({ prop: value });
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog({ prop: value });
            !ej.isNullOrUndefined(this._sourceDialog) && this._sourceDialog.ejDialog({ prop: value });
            if (!ej.isNullOrUndefined(this._customTableDialog)) {
                !ej.isNullOrUndefined(this._customTableDialog) && this._customTableDialog.ejDialog({ prop: value });
                this._customTableDialog.find(".numerictextbox").ejNumericTextbox({ prop: value });
                this._customTableDialog.find("#ddlAlignment").ejDropDownList({ prop: value });
                !ej.isNullOrUndefined(this._customTableDialog) && this._customTableDialog.find(".e-rte-btn").ejButton({ prop: value });
            }
            !ej.isNullOrUndefined(this._videoDialog) && this._videoDialog.find(".e-rte-btn").ejButton({ prop: value });
            !ej.isNullOrUndefined(this._imgDialog) && this._imgDialog.find(".e-rte-btn").ejButton({ prop: value });
            !ej.isNullOrUndefined(this._linkDialog) && this._linkDialog.find(".e-rte-btn").ejButton({ prop: value });
            !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.ejDropDownList({ prop: value });
            !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.ejDropDownList({ prop: value });
            !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.ejDropDownList({ prop: value });
            !ej.isNullOrUndefined(this._fontColorSplit) && this._fontColorSplit.ejSplitButton({ prop: value });
            !ej.isNullOrUndefined(this._bgColorSplit) && this._bgColorSplit.ejSplitButton({ prop: value });
        },
        _createHtmlSource: function () {
            if (ej.isNullOrUndefined(this._htmlSource)) {
                this._htmlSource = ej.buildTag("div.e-rte-icons e-rte-footeritems e-rte-source").appendTo(this._rteFooter);
                this._on(this._htmlSource, "click", this._sourceCodeManager);
                this._renderSourceDialog()
            }
            else
                this._htmlSource.show();
        },
        _createHtmlTagsInfo: function () {
            //var _htmlInfo = this._rteFooter.find("div.e-rte-htmltaginfo");
            if (ej.isNullOrUndefined(this._htmlInfo)) {
                this._htmlInfo = ej.buildTag("div.e-rte-htmltaginfo e-rte-footeritems").appendTo(this._rteFooter);
            }
            else
                this._htmlInfo.show();
        },
        _createWordCount: function () {
            if (ej.isNullOrUndefined(this._wordCount)) {
                this._wordCount = ej.buildTag("div.e-rte-wordcount e-rte-footeritems").appendTo(this._rteFooter);
            }
            else
                this._wordCount.show();
        },
        //Add resize icon
        _enableResize: function () {
            if (this.model.resizable && this.model.enabled) {
                this._rteWapper.addClass("e-resizable");
                this._resizeRTE();
            }
        },
        // Resizeable 
        _resizeRTE: function () {
            var proxy = this;
            this._rteWapper.find("div.e-rte-resize").ejResizable(
                {
                    minHeight: parseInt(proxy.model.minHeight),
                    minWidth: parseInt(proxy.model.minWidth),
                    maxHeight: parseInt(proxy.model.maxHeight),
                    maxWidth: parseInt(proxy.model.maxWidth),
                    resizeStart: function (event) {
                        proxy._trigger("resizeStart", { event: event });
                    },
                    resize: function (event) {
                        var reElement = $(event.element).parents("div.e-rte-wrapper");
                        proxy._rteWapper.height($(reElement).height());
                        proxy._rteWapper.width($(reElement).width());
                        proxy._setIframeHeight();
                        proxy._trigger("resize", { event: event });
                    },
                    resizeStop: function (event) {
                        proxy._trigger("resizeStop", { event: event });
                    },
                    helper: function (event) {
                        var reElement = $(event.element).parents("div.e-rte-wrapper");
                        proxy._rteWapper.height($(reElement).height());
                        proxy._rteWapper.width($(reElement).width());
                        proxy._setIframeHeight();
                        return $(proxy._rteWapper);
                    }
                });
        },

        _getDialogModel: function () {
            return dialogModel = {
                resizable: false,
                autoOpen: false,
                modal: true,
                cssClass: this.model.cssClass,
                contentContainer: "#" + this._rteId + "_wrapper",
                rtl: this.model.rtl
            };
        },

        //Render Link Dialog
        _renderLinkDialog: function () {
            var createLink = this._linkDialog = ej.buildTag("div#" + this._rteId + "_link");//.appendTo(this._rteWapper);
            var content = $("<div class='e-rte-label'><label for=" + this._rteId + "_link_url>" + this._getLocalizedLabels("linkWebUrl") + "</label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_link_url'/></div><div class='e-rte-label'>" +
                           "<label for=" + this._rteId + "_link_text>" + this._getLocalizedLabels("linkText") + "</label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_link_text'></div><div class='e-rte-label'>" +
                           "<label for=" + this._rteId + "_link_title>" + this._getLocalizedLabels("linkToolTip") + " </label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_link_title'></div><div class='e-rte-label'></div>" +
                           "<div class='e-rte-field'><input type='checkbox' id='" + this._rteId + "_link_target'><label for='" + this._rteId + "_link_target' style='padding: 5px;'>" + this._getLocalizedLabels("linkOpenInNewWindow") + "</label></div>" +
                           "<div class='e-rte-button e-fieldseparate'><button class='e-rte-btn' id='link_insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn' id='link_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(createLink);
            createLink.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.title = this._getLocalizedLabels("createLink");
            createLink.ejDialog(model);
            this._chkTarget = createLink.find("#" + this._rteId + "_link_target");
            this._chkTarget.ejCheckBox({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            createLink.find(".e-rte-btn").ejButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._on(createLink.find(".e-rte-btn"), "click", this._linkBtnClick)
            this._on(this._linkDialog.find("#" + this._rteId + "_link_url"), "keypress", this._urlValidation);
        },

        //Render Link Dialog
        _renderImageDialog: function () {
            this._imgDialog = ej.buildTag("div#" + this._rteId + "_Image");
            var content = $("<div class='e-rte-label'><label for=" + this._rteId + "_img_url>" + this._getLocalizedLabels("imageWebUrl") + " </label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_img_url'></div><div class='e-rte-label'>" +
                 "<label for=" + this._rteId + "_img_text>" + this._getLocalizedLabels("imageAltText") + " </label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_img_text'></div>" +
                 "<div id='" + this._rteId + "_imgdimensions'><div class='e-rte-label'><label>" + this._getLocalizedLabels("dimensions") + " </label></div><div class='e-rte-field'><input type='text' title='" + this._getLocalizedLabels("dimensions") + "' class='e-inputtext e-dimensions' maxlength='3' id='" + this._rteId + "_imgX'><label style='padding: 5px;'>X</label><input type='text' title='" + this._getLocalizedLabels("dimensions") + "' style='margin-right: 5px;' class='e-inputtext e-dimensions' maxlength='3' id='" + this._rteId + "_imgY'></div>" +
                 "<input type='checkbox' id='" + this._rteId + "_img_consrn'><label for='" + this._rteId + "_img_consrn'  style='padding: 5px;'>" + this._getLocalizedLabels("constrainProportions") + "</label></div>" +
                 "<div class='e-rte-button e-fieldseparate'><button class='e-rte-btn' id='img_insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn' id='img_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._imgDialog);
            this._imgDialog.appendTo(this._rteWapper);
            !this.model.showDimensions && this._rteWapper.find("#" + this._rteId + "_imgdimensions").hide();
            var model = this._getDialogModel();
            model.width = "470px";
            model.title = this._getLocalizedLabels("image");
            this._imgDialog.ejDialog(model);
            this._chkImgDimensions = this._imgDialog.find("#" + this._rteId + "_img_consrn");
            this._chkImgDimensions.ejCheckBox({ check: true, rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._imgDialog.find(".e-rte-btn").ejButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._on(this._imgDialog.find(".e-rte-btn"), "click", this._imageBtnClick)
            this._on(this._imgDialog.find("#" + this._rteId + "_img_url"), "keypress", this._urlValidation);
            this._on(this._imgDialog.find("input.e-dimensions"), "change", this._recalcImgSize);
        },

        //Render the Table selection rows and columans popup
        _renderVideoDialog: function () {
            this._videoDialog = ej.buildTag("div#" + this._rteId + "_video");
            var content = $("<div><label>" + this._getLocalizedLabels("embedVideo") + "</label></div><textarea class='e-rte-video e-inputtext'  aria-label=" + this._getLocalizedLabels("embedVideo") + "></textarea>" +
               "<div id='" + this._rteId + "_videodimensions' style='margin-top:7px'><div class='e-rte-label'><label>" + this._getLocalizedLabels("dimensions") + " </label></div><div><input type='text' title='" + this._getLocalizedLabels("dimensions") + "' class='e-inputtext e-dimensions' maxlength='3' id='" + this._rteId + "_videoX'><label style='padding: 5px;'>X</label><input type='text' title='" + this._getLocalizedLabels("dimensions") + "' style='margin-right: 5px;' class='e-inputtext e-dimensions' maxlength='3' id='" + this._rteId + "_videoY'>" +
               "<input type='checkbox' id='" + this._rteId + "_video_consrn'><label for='" + this._rteId + "_video_consrn'  style='padding: 5px;'>" + this._getLocalizedLabels("constrainProportions") + "</label></div></div>" +
                "<div class='e-rte-button e-fieldseparate'><button id='video_insert' class='e-rte-btn'>" + this._getLocalizedLabels("dialogInsert") + "</button><button id='video_cancel' class='e-rte-btn'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._videoDialog);
            this._videoDialog.appendTo(this._rteWapper);
            !this.model.showDimensions && this._rteWapper.find("#" + this._rteId + "_videodimensions").hide();
            var model = this._getDialogModel();
            model.width = "460px";
            model.title = this._getLocalizedLabels("video");
            this._videoDialog.ejDialog(model);
            this._chkvideoDimensions = this._videoDialog.find("#" + this._rteId + "_video_consrn");
            this._chkvideoDimensions.ejCheckBox({ check: true, rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._videoDialog.find(".e-rte-btn").ejButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._on(this._videoDialog.find(".e-rte-btn"), "click", this._insertVideo);
            this._on(this._videoDialog.find("input.e-dimensions"), "change", this._recalcVideoSize);
        },

        //Render the Table selection rows and columans popup
        _renderTableDialog: function () {
            this._createTable = ej.buildTag("div.e-rte-table-picker#" + this._rteId + "_table");
            var tableCell, rowDiv, colDiv, tableDiv;
            this._tblheaderDiv = ej.buildTag("div.e-rte-tableheader");
            this._tblheaderDiv.html(this._getLocalizedLabels("createTable"));
            this._tblheaderDiv.appendTo(this._createTable);
            var tableDiv = ej.buildTag("div.e-rte-table");
            this._drawTable(tableDiv);
            tableDiv.appendTo(this._createTable);
            if (this.model.showCustomTable) {
                this._createCustomTable();
            }

            this._createTable.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.modal = false;
            model.showHeader = false;
            model.width = "auto";
            this._createTable.ejDialog(model);
        },

        _createCustomTable: function () {
            var customTable = $("<div class='customtable-group'><span class='e-rte-toolbar-icon customtable-image'></span><a class='customtable-link' href='#' role='button' title='" + this._getLocalizedLabels("customTable") + "'>" + this._getLocalizedLabels("customTable") + "</a></div>");
            this._on(customTable, "click", this._openCustomTable);
            customTable.appendTo(this._createTable);
            //Create custom table Dialog.
            this._customTableDialog = ej.buildTag("div#" + this._rteId + "_customTable");
            var content = $("<div class='e-fieldgroup'><div class='e-rte-tablefields'><label for=" + this._rteId + "_txtColumns>" + this._getLocalizedLabels("tableColumns") + "</label></div><div class='e-rte-tablefields'><input type='text' class='numerictextbox' id='" + this._rteId + "_txtColumns'></div><div class='e-rte-tablefields'>" +
                 "<label for=" + this._rteId + "_txtRows>" + this._getLocalizedLabels("tableRows") + "</label></div><div class='e-rte-tablefields'><input type='text' class='numerictextbox' id='" + this._rteId + "_txtRows'></div><div class='e-rte-tablefields'>" +
                 "<label for=" + this._rteId + "_txtWidth>" + this._getLocalizedLabels("tableWidth") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtWidth'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_txtHeight>" + this._getLocalizedLabels("tableHeight") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtHeight'></div></div>" +
                 "<div class='e-fieldseparate'><div class='e-rte-tablefields'><label for=" + this._rteId + "_txtSpacing>" + this._getLocalizedLabels("tableCellSpacing") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtSpacing'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_txtPadding>" + this._getLocalizedLabels("tableCellPadding") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtPadding'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_txtBorder>" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtBorder'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_chkCaption>" + this._getLocalizedLabels("tableCaption") + "</label></div><div class='e-rte-tablefields'><input type='checkbox' id='" + this._rteId + "_chkCaption'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_ddlAlignment>" + this._getLocalizedLabels("tableAlignment") + "</label></div><div class='e-rte-tablefields'><select id='" + this._rteId + "_ddlAlignment'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div></div>" +
                 "<div class='e-rte-button e-fieldseparate'><button class='e-rte-btn' id='insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn' id='cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._customTableDialog);
            var model = this._getDialogModel();
            model.width = "480px";
            model.title = this._getLocalizedLabels("customTable");
            this._customTableDialog.ejDialog(model);
            this._customTableDialog.find(".e-rte-btn").ejButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._customTableDialog.find(".numerictextbox").ejNumericTextbox({ rtl: this.model.rtl, cssClass: this.model.cssClass, width: "100%", minValue: 1, value: 3 });
            this._customTableDialog.find("#" + this._rteId + "_txtColumns").ejNumericTextbox({ maxValue: 63 });
            this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").ejDropDownList({ rtl: this.model.rtl, cssClass: this.model.cssClass, width: "100%" });
            this._chkTblCaption = this._customTableDialog.find("#" + this._rteId + "_chkCaption");
            this._chkTblCaption.ejCheckBox({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._on(this._customTableDialog.find(".e-rte-btn"), "click", this._insertCustomTable);
        },

        _drawTable: function (tableDiv) {
            for (var row = 0; row < this.model.tableRows ; row++) {
                rowDiv = ej.buildTag("div.e-rtetablerow");
                for (var col = 0; col < this.model.tableColumns; col++) {
                    tableCell = ej.buildTag("div.e-rte-tablecell e-default");
                    tableCell.appendTo(rowDiv);
                }
                rowDiv.appendTo(tableDiv);
            }
        },

        //Render Source code Dialog.
        _renderSourceDialog: function () {
            this._sourceDialog = ej.buildTag("div#" + this._rteId + "_Source");
            var content = $("<textarea class='e-rte-srctextarea e-inputtext'></textarea><div class='e-rte-button e-fieldseparate'><button id='src_update' class='e-rte-btn'>" + this._getLocalizedLabels("dialogUpdate") + "</button><button id='src_cancel' class='e-rte-btn'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._sourceDialog);
            var model = this._getDialogModel();
            model.title = this._getLocalizedLabels("viewHtml");
            model.width = "auto";
            model.contentContainer = null,
            this._sourceDialog.ejDialog(model);
            this._sourceDialog.find(".e-rte-btn").ejButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._on(this._sourceDialog.find(".e-rte-btn"), "click", this._srcBtnClick)
        },

        //Render Alert Dialog
        _renderAlertDialog: function () {
            this._alertWindow = ej.buildTag("div");
            var content = $("<p class='e-alerttext'></p><div class='e-rte-button e-fieldseparate'><button class='e-rte-btn e-alert-ok'>" + this._getLocalizedLabels("dialogOk") + "</button><button class='e-rte-btn e-alert-cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._alertWindow);
            var model = this._getDialogModel();
            model.showHeader = false;
            model.width = "auto";
            this._alertWindow.ejDialog(model);
            this._alertWindow.find(".e-rte-btn").ejButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            this._on(this._alertWindow.find(".e-rte-btn"), "click", this._alertBtnClick)
            this._alertWindow.find(".e-alert-cancel").hide();
        },

        _alertBtnClick: function (e) {
            if ($.trim(this._alertWindow.find(".e-alerttext").html()) == this._getLocalizedLabels("deleteAlert") && $(e.target).hasClass("e-alert-ok")) {
                this._getDocument().body.innerHTML = "";
                this._toolBarObj.selectItembyID(this._rteId + "justifyLeft");
                this._setBackupData();
                this._updateCharCount();
                this._onChange();
            }
            this._alertWindow.ejDialog("close");
            this._alertWindow.find(".e-alert-cancel").hide();
        },

        _openAlert: function (alertText) {
            this._alertWindow.find(".e-alerttext").html(alertText);
            this._alertWindow.ejDialog("open");
        },

        //Render Format 
        _renderFormat: function () {
            this._formatDDL = ej.buildTag("input#" + this._rteId + "_format", "", "", { type: "text", title: this._getLocalizedLabels("format") });
            var model = {};
            model.dataSource = this.model.format;
            model.height = "25px";
            model.width = "105px";
            model.rtl = this.model.rtl;
            model.popupPanelWidth = "175px";
            model.popupPanelHeight = "auto";
            model.selectedItem = 0;
            model.popupHide = this._formatChange;
            model.fields = { text: "text", value: "value", spriteCSS: "spriteCSS" };
            this._formatDDL.appendTo(this._rteToolbar.find("#" + this._rteId + "format").html(""));
            this._formatDDL.ejDropDownList(model);
            this._formatPopupStyle();
        },
        //Apply the css class for LI elemet
        _formatPopupStyle: function () {
            var _liList = $("#" + this._rteId + "_format_popup").find("li");
            for (var i = 0; i < _liList.length; i++) {
                $(_liList[i]).addClass(this.model.format[i].spriteCSS);
            }
        },

        // Render Font style Dropdown list
        _renderFontStyle: function () {
            this._fontStyleDDL = ej.buildTag("input#" + this._rteId + "_fontName", "", "", { type: "text", title: this._getLocalizedLabels("fontName") });
            var model = {};
            model.dataSource = this.model.fontName;
            model.height = "25px";
            model.rtl = this.model.rtl;
            model.selectedItem = 0;
            model.popupHide = this._fontStyleChange;
            model.fields = { text: "text", value: "value" };
            this._fontStyleDDL.appendTo(this._rteToolbar.find("#" + this._rteId + "fontName").html(""));
            this._fontStyleDDL.ejDropDownList(model);
        },

        // Render Font style Dropdown list
        _renderFontSize: function () {
            this._fontSizeDDL = ej.buildTag("input#" + this._rteId + "_fontSize", "", "", { type: "text", title: this._getLocalizedLabels("fontSize") });
            var model = {};
            model.dataSource = this.model.fontSize;
            model.height = "25px";
            model.width = "100px";
            model.rtl = this.model.rtl
            model.selectedItem = 1;
            model.popupHide = this._fontSizeChange;
            model.fields = { text: "text", value: "value" };
            this._fontSizeDDL.appendTo(this._rteToolbar.find("#" + this._rteId + "fontSize").html(""));
            this._fontSizeDDL.ejDropDownList(model);
        },

        //Render Split button for font color
        _renderFontColor: function () {
            this._fontColorSplit = ej.buildTag("button#" + this._rteId + "_fontColor", "", {}, { type: "button" });
            var _liTemplte = $("<ul id='" + this._rteId + "_colorTable' class='e-rte-colorpalette'><li></li></ul>");
            _liTemplte.find("li").append(this._colorTable());
            var model = {};
            model.height = "25px";
            model.width = "50px";
            model.contentType = "textandimage";
            model.targetId = this._rteId + "_colorTable";
            model.rtl = this.model.rtl;
            model.prefixIcon = "e-fontcolor-icon";
            this._fontColorSplit.appendTo(this._rteToolbar.find("#" + this._rteId + "fontColor").html(""));
            _liTemplte.appendTo(this._rteToolbar.find("#" + this._rteId + "fontColor"));
            this._fontColorSplit.ejSplitButton(model);
            this._splitMenu = _liTemplte.ejMenu("instance");
            this._on(_liTemplte.find("div.e-rte-palettetable"), "click", this._colorPaletteClick);
            this._on(this._fontColorSplit, "click", this._fontColorClick);
            this._fontColorSplit.find("span.e-fontcolor-icon").removeClass("e-icon");
            this._fontColorSplit.find("span.e-btntxt").css("background-color", "black");
        },

        //Render Split button for font color
        _renderBGColor: function () {
            this._bgColorSplit = ej.buildTag("button#" + this._rteId + "_backgrounColor", "", {}, { type: "button" });
            var _liTemplte = $("<ul id='" + this._rteId + "_colorBGTable' class='e-rte-colorpalette'><li></li></ul>");
            _liTemplte.find("li").append(this._colorTable());
            var model = {};
            model.height = "25px";
            model.width = "50px";
            model.contentType = "textandimage";
            model.targetId = this._rteId + "_colorBGTable";
            model.rtl = this.model.rtl;
            model.prefixIcon = "e-bgcolor-icon";
            this._bgColorSplit.appendTo(this._rteToolbar.find("#" + this._rteId + "backgrounColor").html(""));
            _liTemplte.appendTo(this._rteToolbar.find("#" + this._rteId + "backgrounColor"));
            this._bgColorSplit.ejSplitButton(model);
            this._bgSplitMenu = _liTemplte.ejMenu("instance");
            this._on(_liTemplte.find("div.e-rte-palettetable"), "click", this._bgColorPaletteClick);
            this._on(this._bgColorSplit, "click", this._bgColorClick);
            this._bgColorSplit.find("span.e-bgcolor-icon").removeClass("e-icon");
            this._bgColorSplit.find("span.e-btntxt").css("background-color", "white");
        },

        _colorTable: function () {
            var tableDiv = ej.buildTag("div.e-rte-table");
            var color = 0, hexCode, rowDiv;
            for (var row = 0; row < this.model.colorPaletteRows ; row++) {
                rowDiv = ej.buildTag("div.e-rtetablerow");
                for (var col = 0; col < this.model.colorPaletteColumns ; col++) {
                    if (color < this.model.colorCode.length) {
                        hexCode = "#" + this.model.colorCode[color];
                        tableCell = ej.buildTag("div.e-rte-palettetable", "", { "background-color": hexCode }, { title: hexCode, "color-code": hexCode });
                        tableCell.appendTo(rowDiv);
                        color++;
                    }
                }
                rowDiv.appendTo(tableDiv);
            }
            return tableDiv;
        },

        _recalcVideoSize: function (e) {
            var txtWidth, txtheight, newWidth, newHeight;
            txtWidth = this._videoDialog.find("#" + this._rteId + "_videoX");
            txtheight = this._videoDialog.find("#" + this._rteId + "_videoY");

            newWidth = txtWidth.val();
            newHeight = txtheight.val();

            if (this._chkvideoDimensions.ejCheckBox("isChecked") && this._videoWidth && this._videoHeight && newWidth && newHeight) {
                if (e.target.id == txtWidth[0].id) {
                    newHeight = Math.round((newWidth / this._videoWidth) * newHeight);
                    txtheight.val(newHeight);
                } else {
                    newWidth = Math.round((newHeight / this._videoHeight) * newWidth);
                    txtWidth.val(newWidth);
                }
            }
            this._videoWidth = newWidth;
            this._videoHeight = newHeight;
        },


        _recalcImgSize: function (e) {
            var txtWidth, txtheight, newWidth, newHeight;
            txtWidth = this._imgDialog.find("#" + this._rteId + "_imgX");
            txtheight = this._imgDialog.find("#" + this._rteId + "_imgY");

            newWidth = txtWidth.val();
            newHeight = txtheight.val();

            if (this._chkImgDimensions.ejCheckBox("isChecked") && this._imgWidth && this._imgHeight && newWidth && newHeight) {
                if (e.target.id == txtWidth[0].id) {
                    newHeight = Math.round((newWidth / this._imgWidth) * newHeight);
                    txtheight.val(newHeight);
                } else {
                    newWidth = Math.round((newHeight / this._imgHeight) * newWidth);
                    txtWidth.val(newWidth);
                }
            }
            this._imgWidth = newWidth;
            this._imgHeight = newHeight;
        },

        _urlValidation: function (e) {
            var _target = $(e.target);
            _target.hasClass("e-error") && _target.removeClass("e-error");
        },

        _linkBtnClick: function (sender) {
            if (sender.target.id === "link_cancel") {
                this._clearLinkFields();
                this._linkDialog.ejDialog("close");
            }
            else if (sender.target.id === "link_insert") {
                this._restoreSelection(this._selectionRange);
                if (this._txtURL.val() != "")
                    this._onInsertLink();
                else {
                    this._onUnlink();
                    this._linkDialog.ejDialog("close");
                }
                this._onChange();
                this._setBackupData();
            }
        },

        _imageBtnClick: function (sender) {
            if (sender.target.id === "img_cancel") {
                this._imgDialog.ejDialog("close");
            }
            else if (sender.target.id === "img_insert") {
                this._onInsertImage();
                this._onChange();
                this._setBackupData();
            }
        },

        //Source Dialog button click event
        _srcBtnClick: function (sender) {
            if (sender.target.id === "src_update") {
                var _editHTML = $.trim(this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value);
                this._getDocument().body.innerHTML = _editHTML;
                this._onChange();
                this._updateCharCount();
            }
            this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value = "";
            this._sourceDialog.ejDialog("close");
        },

        _insertCustomTable: function (sender) {
            if (sender.target.id === "insert") {
                var cols = this._customTableDialog.find("#" + this._rteId + "_txtColumns").ejNumericTextbox("getValue");
                var row = this._customTableDialog.find("#" + this._rteId + "_txtRows").ejNumericTextbox("getValue");
                var height = this._customTableDialog.find("#" + this._rteId + "_txtHeight").val();
                var width = this._customTableDialog.find("#" + this._rteId + "_txtWidth").val();
                var align = this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").ejDropDownList("getSelectedValue");
                var spacing = this._customTableDialog.find("#" + this._rteId + "_txtSpacing").val();
                var padding = this._customTableDialog.find("#" + this._rteId + "_txtPadding").val();
                var border = this._customTableDialog.find("#" + this._rteId + "_txtBorder").val();
                var caption = this._chkTblCaption.ejCheckBox("isChecked");
                if (!ej.isNullOrUndefined(this._tableInsertAt) && !ej.isNullOrUndefined(this._tableInsertAt.startContainer))
                    this._restoreSelection(this._tableInsertAt);
                var tableEle = this._tableGenerator(row, cols, false, width, height, spacing, padding, align, border, caption);
                this.executeCommand('inserthtml', tableEle);
                var _tableEle = $(this._getDocument()).find("table.e-rte-table");
                _tableEle.attr("contentEditable", false);
                _tableEle.find("td").attr("contentEditable", true);
                //this._getWindow().document.execCommand('inserthtml', false, tableEle);
                this._onChange();
            }
            this._customTableDialog.ejDialog("close");
            this._clearTableFields();
        },

        _clearTableFields: function () {
            this._customTableDialog.find("#" + this._rteId + "_txtColumns").ejNumericTextbox("option", "value", 3);
            this._customTableDialog.find("#" + this._rteId + "_txtRows").ejNumericTextbox("option", "value", 3);
            this._customTableDialog.find("#" + this._rteId + "_txtHeight").val("");
            this._customTableDialog.find("#" + this._rteId + "_txtWidth").val("");
            this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").ejDropDownList("clearText");
            this._customTableDialog.find("#" + this._rteId + "_txtSpacing").val("");
            this._customTableDialog.find("#" + this._rteId + "_txtPadding").val("");
            var border = this._customTableDialog.find("#" + this._rteId + "_txtBorder").val("");
            var caption = this._chkTblCaption.ejCheckBox({ check: false });
        },

        _clearLinkFields: function () {
            this._txtURL = this._linkDialog.find("#" + this._rteId + "_link_url");
            this._txtURL.removeClass("e-error");
            this._txtURL.val("http://");
            this._txtLinkText = this._linkDialog.find("#" + this._rteId + "_link_text");
            this._txtLinkText.val("");
            this._txtLinkTitle = this._linkDialog.find("#" + this._rteId + "_link_title");
            this._txtLinkTitle.val("");
            this._chkTarget.ejCheckBox({ check: false });
            this._txtURL.focus();
        },
        _clearImgFields: function () {
            this._imgDialog.find("#" + this._rteId + "_img_url").val("http://").removeClass("e-error");
            this._imgDialog.find("#" + this._rteId + "_img_text").val("");
            this._chkImgDimensions.ejCheckBox({ check: true });
        },

        _videoManager: function () {
            var selectNode = this._getSelectedNode();
            this._editVideo = null;
            if (selectNode && selectNode.tagName.toLowerCase() === "iframe") {
                this._editVideo = selectNode;
                this._videoWidth = parseInt(selectNode.width);
                this._videoHeight = parseInt(selectNode.height);
                this._videoDialog.find("textarea.e-rte-video").val(selectNode.outerHTML);
                this._videoDialog.find("#" + this._rteId + "_videoX").val(this._videoWidth);
                this._videoDialog.find("#" + this._rteId + "_videoY").val(this._videoHeight);
            }
            this._videoDialog.ejDialog("open");
            this._videoDialog.find("textarea.e-rte-video").focus();
        },

        _sourceCodeManager: function () {
            ej.isNullOrUndefined(this._sourceDialog) && this._renderSourceDialog();
            this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value = this.getHtmlString();
            this._sourceDialog.ejDialog("open");
            this._sourceDialog.find("textarea.e-rte-srctextarea").focus();
        },

        _hyperLinkManager: function () {
            this._linkDialog.ejDialog("open");
            this._clearLinkFields();
            this._txtLinkText.val(this._getSelText());
            this._selectedHTML = this._getSelText();
            this._hyperLinkSelection = this._getDocument().selection;
            var _linkSibling = this._getSelectedNode();
            if (_linkSibling && (/^(A)$/).test(_linkSibling.nodeName)) {
                this._txtURL.val(_linkSibling.href);
                this._txtLinkTitle.val(_linkSibling.title);
            }
            if (this._hyperLinkSelection != null)
                this._hyperLinkSelRange = this._hyperLinkSelection.createRange();
        },

        // table inser 
        _tableManager: function () {
            var tableLi = this._rteToolbar.find("#" + this._rteId + "createTable");
            this._createTable.ejDialog("option", "position", {
                X: tableLi.position().left,
                Y: tableLi.position().top + tableLi.height() + 7
            });
            this._createTable.ejDialog("open");
        },

        _imageManager: function () {
            this._clearImgFields();
            var selectNode = this._getSelectedNode();
            if (selectNode && selectNode.tagName.toLowerCase() === "img") {
                this._imgWidth = parseInt(selectNode.width);
                this._imgHeight = parseInt(selectNode.height);
                this._imgDialog.find("#" + this._rteId + "_img_url").val(selectNode.src);
                this._imgDialog.find("#" + this._rteId + "_img_text").val(selectNode.alt);
                this._imgDialog.find("#" + this._rteId + "_imgX").val(this._imgWidth);
                this._imgDialog.find("#" + this._rteId + "_imgY").val(this._imgHeight);
            }
            this._imgDialog.ejDialog("open");
            this._imgDialog.find("#" + this._rteId + "_img_url").focus();
        },

        _toolBarClick: function (sender) {
            var $rteEle = $(this.itemsContainer).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            var _itemId = sender.currentTarget.id.replace(rteId, "");
            if ((sender.currentTarget.id == rteId + "copy" || sender.currentTarget.id == rteId + "cut" || sender.currentTarget.id == rteId + "paste") && !rteInstcne._isIE())
                rteInstcne._openAlert(rteInstcne._getLocalizedLabels("copyPastAlert"));
            else
                rteInstcne._selectCommand(sender.currentTarget.id);
            //For Style Tools
            if (rteInstcne._styleItems.indexOf(_itemId) >= 0 || rteInstcne._scriptsItems.indexOf(_itemId) >= 0) {
                if (!$(sender.currentTarget).hasClass("e-active"))
                    this.selectItembyID(sender.currentTarget.id);
                else
                    this.deselectItembyID(sender.currentTarget.id);
            }

                //For Align Tools
            else if (rteInstcne._alignItems.indexOf(_itemId) >= 0) {
                this.selectItembyID(sender.currentTarget.id);
                var index = rteInstcne._alignItems.indexOf(_itemId)
                for (var i = 0; i < rteInstcne._alignItems.length; i++) {
                    if (i != index)
                        this.deselectItembyID(rteId + rteInstcne._alignItems[i]);
                }
            }
                //For Bullets & Numbering tools
            else if (rteInstcne._listItems.indexOf(_itemId) >= 0) {
                if (!$(sender.currentTarget).hasClass("e-active"))
                    this.selectItembyID(sender.currentTarget.id);
                else
                    this.deselectItembyID(sender.currentTarget.id);

                var index = rteInstcne._listItems.indexOf(_itemId);
                if (index == 0)
                    this.deselectItembyID(rteId + rteInstcne._listItems[1]);
                else
                    this.deselectItembyID(rteId + rteInstcne._listItems[0]);
            }
        },

        _toggleEditTable: function () {
            var selTag = this._getSelectedNode();
            var _editTableItems = this._toolBarItems.find("#" + this._rteId + "addColumnLeft, #" + this._rteId + "addColumnRight, #" + this._rteId + "addRowAbove, #"
             + this._rteId + "addRowBelow, #" + this._rteId + "deleteRow, #" + this._rteId + "deleteColumn, #" + this._rteId + "deleteTable");
            selTag && selTag.tagName && (selTag.tagName.toLowerCase() == "td" || selTag.tagName.toLowerCase() == "table" || selTag.tagName.toLowerCase() == "tr") ?
            _editTableItems.show() : _editTableItems.hide();
            this._setIframeHeight();
        },

        //Format changes
        _formatChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFormatBlock(this.getSelectedValue());
            rteInstcne._focus();
            rteInstcne._onChange();
        },

        // Font style chnage event
        _fontStyleChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFontName(this.getSelectedValue());
            rteInstcne._onChange();
        },

        // Font style chnage event
        _fontSizeChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFontSize(this.getSelectedValue());
            rteInstcne._onChange();
        },

        //Color select 
        _colorPaletteClick: function (e) {
            var value;
            if ((value = e.target.getAttribute('color-code'))) {
                this._focus();
                this._onFontColor(value);
                this._splitMenu.hideContextMenu(e);
                this._bgSplitMenu.hideContextMenu(e);
                this._fontColor = value;
                this._fontColorSplit.find("span.e-btntxt").css("background-color", value);
            }
        },

        //Color select 
        _bgColorPaletteClick: function (e) {
            var value;
            if ((value = e.target.getAttribute('color-code'))) {
                this._focus();
                this._onBGColor(value);
                this._splitMenu.hideContextMenu(e);
                this._bgSplitMenu.hideContextMenu(e);
                this._bgColor = value;
                this._bgColorSplit.find("span.e-btntxt").css("background-color", value);
            }
        },

        _fontColorClick: function (e) {
            this._onFontColor(this._fontColor);
            this._splitMenu.hideContextMenu(e);
        },

        _bgColorClick: function (e) {
            this._onBGColor(this._bgColor);
            this._bgSplitMenu.hideContextMenu(e);
        },

        _selectCommand: function (selectedId) {
            var rteId = this._rteId;
            if (!(selectedId == rteId + "format" || selectedId == rteId + "fontName" || selectedId == rteId + "fontSize" || selectedId == rteId + "fontColor" || selectedId == rteId + "backgrounColor")) {
                this._focus();
            }
            switch (selectedId) {
                case rteId + "bold":
                    this._onBold();
                    break;
                case rteId + "italic":
                    this._onItalics();
                    break;
                case rteId + "underline":
                    this._onUnderLine();
                    break;
                case rteId + "strikethrough":
                    this._onStrikeThrough();
                    break;
                case rteId + "justifyLeft":
                    this._onJustifyLeft();
                    break;
                case rteId + "justifyRight":
                    this._onJustifyRight();
                    break;
                case rteId + "justifyCenter":
                    this._onJustifyCenter();
                    break;
                case rteId + "justifyFull":
                    this._onJustifyFull();
                    break;
                case rteId + "cut":
                    this._onCut();
                    break;
                case rteId + "copy":
                    this._onCopy();
                    break;
                case rteId + "paste":
                    this._onPaste();
                    break;
                case rteId + "clearFormat":
                    this._onClearFormat();
                    break;
                case rteId + "clearAll":
                    this._clearAllManager();
                    break;
                case rteId + "orderedList":
                    this._onOrderList();
                    break;
                case rteId + "unorderedList":
                    this._onUnorderList();
                    break;
                case rteId + "undo":
                    this._undo();
                    break;
                case rteId + "redo":
                    this._redo();
                    break;
                case rteId + "indent":
                    this._onIndent();
                    this._indentdepth++;
                    break;
                case rteId + "outdent":
                    this._onOutdent();
                    if (this._indentdepth > 0)
                        this._indentdepth--;
                    break;
                case rteId + "createLink":
                    this._hyperLinkManager();
                    break;
                case rteId + "image":
                    this._imageManager();
                    break;
                case rteId + "createTable":
                    this._tableManager();
                    break;
                case rteId + "addRowAbove":
                    this.insertRow(true);
                    break;
                case rteId + "addRowBelow":
                    this.insertRow(false);
                    break;
                case rteId + "addColumnLeft":
                    this.insertColumn(true);
                    break;
                case rteId + "addColumnRight":
                    this.insertColumn(false);
                    break;
                case rteId + "deleteRow":
                    this.removeRow();
                    break;
                case rteId + "deleteColumn":
                    this.removeColumn();
                    break;
                case rteId + "deleteTable":
                    this.removeTable();
                    break;
                case rteId + "superscript":
                    this._onSuperScript();
                    break;
                case rteId + "subscript":
                    this._onSubScript();
                    break;
                case rteId + "upperCase":
                    this._onUpperCase();
                    break;
                case rteId + "lowerCase":
                    this._onLowerCase();
                    break;
                case rteId + "video":
                    this._videoManager();
                    break;
                default:
                    this._onChange();
                    break;
            }
            if (selectedId != rteId + "createLink" && selectedId != rteId + "image" && selectedId != rteId + "video" && selectedId != rteId + "createTable") {
                this._onChange();
            }
            if ((selectedId != rteId + "undo") && (selectedId != rteId + "redo"))
                this._setBackupData();
            var selectEle = this._getSelectedNode();
            this._updateTagInfo(selectEle);
        },
        _focus: function () {
            this._getWindow().focus();
            //this._selectRange();
            this._restoreSelection(this._selectionRange);
        },
        _getWindow: function () {
            var iframe = document.getElementById(this._iframeId);
            return iframe.contentWindow;
        },
        _getDocument: function () {
            var iframe = document.getElementById(this._iframeId);
            var iframeObject = iframe.contentWindow.document;
            return iframeObject;
        },
        // Iframe command function
        _onBold: function () {
            var args;
            this.executeCommand('bold', args);
        },

        _onItalics: function () {
            var args;
            this.executeCommand('italic', args);
        },
        _onStrikeThrough: function () {
            var args;
            this.executeCommand('strikethrough', args);
        },

        _onUnderLine: function () {
            var args;
            this.executeCommand('underline', args);
        },

        _onJustifyLeft: function () {
            var args;
            this.executeCommand('JustifyLeft', args);
        },

        _onJustifyRight: function () {
            var args;
            this.executeCommand('JustifyRight', args);
        },

        _onJustifyCenter: function () {
            var args;
            this.executeCommand('JustifyCenter', args);
        },

        _onJustifyFull: function () {
            this._alignFlag = true;
            this.executeCommand('justifyfull');
        },

        _onCut: function () {
            var args;
            this.executeCommand('cut', args);
        },
        _onCopy: function () {
            var args;
            this.executeCommand('copy', args);
        },
        _onPaste: function () {
            var args;
            this.executeCommand('paste', args);

        },
        _onOrderList: function () {
            var args;
            this.executeCommand('insertorderedlist', args);
        },
        _onUnorderList: function () {
            var args;
            this.executeCommand('insertunorderedlist', args);
        },
        _onUndo: function () {
            var args;
            this.executeCommand('undo', null);
        },
        _onDelete: function () {
            var args;
            this.executeCommand('delete', null);
        },

        _onRedo: function () {
            var args;
            this.executeCommand('redo', null);
        },

        _onClearFormat: function () {
            var args;
            this.executeCommand('removeformat', args);
            //this.executeCommand('foreColor', 'black');
            this._setBackupData();
            this._onChange();
            this._updateIframeSkin(this.model.iframeAttribute);
        },

        _onFormatBlock: function (formatBlock) {
            this._focus();
            this.executeCommand('formatBlock', formatBlock);
        },

        _onFontName: function (fontsname) {
            this.executeCommand("fontName", fontsname);
        },

        _onFontSize: function (Size) {
            this.executeCommand('fontSize', Size);
        },

        _onFontColor: function (color) {
            this.executeCommand('foreColor', color);
            this._onChange();
            this._updateIframeSkin(this.model.iframeAttribute);
        },

        _onBGColor: function (color) {
            this.executeCommand('backColor', color);
            this._onChange();
        },

        _onIndent: function () {
            this._getWindow().document.body.style.wordWrap = "break-word";
            var node = this._getWindow().document.getSelection().focusNode;
            if (node && (/^(OL|UL|LI)$/).test(node.nodeName))
                this._listIndent(node);
            else if (node && (/^(#text)$/).test(node.nodeName) && (/^(OL|UL|LI)$/).test($(node).parent()[0].nodeName)) {
                this._listIndent($(node).parents("li")[0], node);
            }
            else
                this.executeCommand('indent', false);
            node && $(node).focus();
            this._updateIframeSkin(this.model.iframeAttribute);
        },

        _onOutdent: function () {
            var node = this._getWindow().document.getSelection().focusNode;
            if (node && (/^(OL|UL|LI)$/).test(node.nodeName))
                this._listOutdent(node);
            else if (node && (/^(#text)$/).test(node.nodeName) && (/^(OL|UL|LI)$/).test($(node).parent()[0].nodeName)) {
                this._listOutdent($(node).parents("li")[0], node);
            }
            else
                this.executeCommand('outdent', false);
        },

        _onSelectAll: function () {
            this.executeCommand('selectall');
        },
        _onHyperLink: function (val) {
            this.executeCommand('createlink', val);
        },

        _onInsertLink: function () {
            var oEl;
            var sHtml;
            if (!this._isUrl($.trim(this._txtURL.val()))) {
                this._txtURL.addClass("e-error");
                return false;
            }
            oEl = ej.buildTag("a", "", {}, { href: this._txtURL.val(), title: this._txtLinkTitle.val() });
            if (this._chkTarget.ejCheckBox("isChecked"))
                oEl.target = "_blank";
            else
                oEl.target = "_self";

            //}
            var oSelection;
            var oSelRange;
            this._focus();
            if (this._isIE()) {
                if (this._selectedHTML.length == 0) {
                    this._selectedHTML = this._txtLinkText.val();
                }
                oEl[0].innerHTML = this._selectedHTML;
                sHtml = oEl[0].outerHTML;
            }
            else {
                oEl.html(this._txtLinkText.val());
                sHtml = $('<div>').append($(oEl).clone()).html();
            }
            if (sHtml) {
                if (this._isIE()) {
                    this._pasteHtml(sHtml);
                }
                else {
                    this._getDocument().execCommand('inserthtml', false, sHtml);
                }
                this._focus();
            }
            this._linkDialog.ejDialog("close");
        },

        _onInsertImage: function () {
            var url = this._imgDialog.find("#" + this._rteId + "_img_url");
            var altText = this._imgDialog.find("#" + this._rteId + "_img_text");
            var imgX = this._imgDialog.find("#" + this._rteId + "_imgX").val();
            var imgY = this._imgDialog.find("#" + this._rteId + "_imgY").val();
            var proxy = this;
            this._restoreSelection(this._selectionRange);
            //Url Validation
            var _img = $("<img>", {
                src: url.val(),
                error: function () { url.addClass("e-error"); },
                load: function () {
                    var imgEle = ej.buildTag("img", "", {}, { alt: altText.val(), src: url.val() });
                    if (proxy.model.showDimensions) {
                        (imgX != "") && imgEle.width(imgX);
                        (imgY != "") && imgEle.height(imgY);
                    }
                    proxy.executeCommand('inserthtml', imgEle[0].outerHTML);
                    proxy._imgDialog.ejDialog("close");
                }
            });

        },
        //Insert vedio
        _insertVideo: function (sender) {
            var videoLink = $.trim(this._videoDialog.find("textarea.e-rte-video")[0].value);
            var videoX = this._videoDialog.find("#" + this._rteId + "_videoX").val();
            var videoY = this._videoDialog.find("#" + this._rteId + "_videoY").val();
            this._restoreSelection(this._selectionRange);
            var ele = $(videoLink);
            if (sender.target.id === "video_insert" && videoLink !== "") {
                var _index, _link = ele[0].src;
                if (!ej.isNullOrUndefined(_link) && _link.indexOf("http:") == -1) {
                    _index = videoLink.indexOf("src=") + 5;
                    videoLink = videoLink.substr(0, _index) + "http:" + videoLink.substr(_index);
                }
                var _index, ele = $(videoLink)[0];
                if (this.model.showDimensions) {
                    (videoX != "") && (ele.width = videoX);
                    (videoX != "") && (ele.height = videoY);
                }
                this._editVideo && $(this._editVideo).remove();
                this.executeCommand('inserthtml', ele.outerHTML);
                //this._getWindow().document.execCommand('inserthtml', false, ele.outerHTML);
                this._onChange();
                this._setBackupData();
                this._videoDialog.ejDialog("close");
            }
            else if (sender.target.id === "video_cancel")
                this._videoDialog.ejDialog("close");
            else
                this._openAlert(this._getLocalizedLabels("videoError"));
            this._videoDialog.find("textarea.e-rte-video")[0].value = "";
            this._videoDialog.find("#" + this._rteId + "_videoX").val("");
            this._videoDialog.find("#" + this._rteId + "_videoY").val("");
            this._chkvideoDimensions.ejCheckBox({ check: true });
        },

        //Clears all the contents
        _clearAllManager: function () {
            this._alertWindow.find(".e-alert-cancel").show();
            this._openAlert(this._getLocalizedLabels("deleteAlert"));
        },

        _onUnlink: function () {
            var args;
            this.executeCommand('unlink', args);
        },
        _onSuperScript: function () {
            var args;
            this.executeCommand('SuperScript', args);
        },
        _onSubScript: function () {
            var args;
            this.executeCommand('SubScript', args);
        },
        _onInsertTable: function (rows, cols, designTime) {
            var tableEle = this._tableGenerator(rows, cols, designTime);
            this.executeCommand('inserthtml', tableEle);
        },

        _onUpperCase: function () {
            if (this._isIE()) {
                if (this._getWindow().document.selection.type.toLowerCase() == "control") {
                    return;
                }

                var p = this._getWindow().document.selection.createRange();
                this._getDocument().selection.createRange().pasteHTML(this._seleText.toUpperCase());
            }
            else {
                var selection = this._getWindow().getSelection();
                var range = selection.getRangeAt(0);
                var node = range.commonAncestorContainer;
                var children = range.commonAncestorContainer.children;
                if (children != null) {
                    var contents = range.extractContents();
                    var result = this._changeCase(contents, "Upper");
                    var resultCopy = $(result).clone();
                    var tempDiv = document.createElement('div');
                    $(tempDiv).append($(resultCopy));
                    if (tempDiv.innerHTML != null && tempDiv.innerHTML != "")
                        this._getWindow().document.execCommand('inserthtml', false, tempDiv.innerHTML);
                }
                else {
                    var replaceString = node.nodeValue.substring(range.startOffset, range.endOffset).toUpperCase();
                    var startString = node.nodeValue.substr(0, range.startOffset);
                    var endString = node.nodeValue.substring(range.endOffset);
                    node.nodeValue = startString + replaceString + endString;
                }
            }

        },

        _changeCase: function (parentNode, toChangeCase) {
            var s = parentNode.childNodes;
            if (s.length > 0) {
                for (var t = 0; t < s.length; t++) {
                    if (s[t].nodeType == 3) {
                        s[t].nodeValue = toChangeCase == "Upper" ? s[t].nodeValue.toUpperCase() : s[t].nodeValue.toLowerCase();
                    }
                    else {
                        if (s[t].nodeType == 1 && s[t].tagName.toUpperCase() != "FONT") {
                            this.changeCase(s[t], toChangeCase);
                        }
                    }
                }
            }
            return s;
        },

        _onLowerCase: function () {
            var selection = this._getWindow().getSelection();
            var range = selection.getRangeAt(0);
            var node = range.commonAncestorContainer;
            var children = range.commonAncestorContainer.children;
            if (children != null) {
                var contents = range.extractContents();
                var result = this.changeCase(contents, "Lower");
                var resultCopy = $(result).clone();
                var tempDiv = document.createElement('div');
                $(tempDiv).append($(resultCopy));
                if (tempDiv.innerHTML != null && tempDiv.innerHTML != "")
                    this._getWindow().document.execCommand('inserthtml', false, tempDiv.innerHTML);
            }
            else {
                var replaceString = node.nodeValue.substring(range.startOffset, range.endOffset).toLowerCase();
                var startString = node.nodeValue.substr(0, range.startOffset);
                var endString = node.nodeValue.substring(range.endOffset);
                node.nodeValue = startString + replaceString + endString;
            }
            //}
        },

        _listIndent: function (node, selection) {
            var sibling, newList;
            sibling = node.previousSibling;

            if (sibling && this._isListNode(sibling)) {
                sibling.appendChild(node);
                return true;
            }
            if (sibling && sibling.nodeName == 'LI' && this._isListNode(sibling.lastChild)) {
                sibling.lastChild.appendChild(node);
                this._combineLists(node.lastChild, sibling.lastChild, node);
                return true;
            }

            sibling = node.nextSibling;

            if (sibling && this._isListNode(sibling)) {
                sibling.insertBefore(node, sibling.firstChild);
                return true;
            }

            if (sibling && sibling.nodeName == 'LI' && this._isListNode(node.lastChild)) {
                return false;
            }

            sibling = node.previousSibling;
            if (sibling && sibling.nodeName == 'LI') {
                newList = ej.buildTag(node.parentNode.nodeName)[0];
                sibling.appendChild(newList);
                newList.appendChild(node);
                this._combineLists(node.lastChild, newList, node);
                return true;
            }
            return true;
        },

        _combineLists: function (from, to, li) {
            var node;

            if (this._isListNode(from)) {
                while ((node = li.lastChild.firstChild)) {
                    to.appendChild(node);
                }

                $(from).remove();
            }
        },
        _isListNode: function (node) {
            return node && (/^(OL|UL)$/).test(node.nodeName);
        },
        _isFirstChild: function (node) {
            return node.parentNode.firstChild == node;
        },

        _isLastChild: function (node) {
            return node.parentNode.lastChild == node;
        },
        _removeEmptyList: function (node) {
            if (node && $(node).children().length === 0)
                $(node).remove();
        },
        _listOutdent: function (node) {
            var ul = $(node.parentNode), ulParent = ul[0].parentNode, newBlock;
            node = $(node);
            if (this._isFirstChild(node[0]) && this._isLastChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                    this._removeEmptyList(ulParent);
                    ul.remove();
                } else if (this._isListNode(ulParent)) {
                    ul.remove();
                } else {
                    this.executeCommand('outdent', false);
                    ////ulParent.insertBefore(createNewTextBlock(node[0]), ul[0]);
                    //ul.remove();
                }

                return true;
            } else if (this._isFirstChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                    node[0].appendChild(ul[0]);
                    this._removeEmptyList(ulParent);
                } else if (this._isListNode(ulParent)) {
                    ulParent.insertBefore(node[0], ul[0]);
                } else {
                    this.executeCommand('outdent', false);
                    //ulParent.insertBefore(createNewTextBlock(node[0]), ul[0]);
                    //node.remove();
                }

                return true;
            } else if (this._isLastChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                } else if (this._isListNode(ulParent)) {
                    node.insertAfter(ul[0]);
                } else {
                    this.executeCommand('outdent', false);
                    //node.insertAfter(createNewTextBlock(node[0]), ul[0]);
                    //node.remove();
                }

                return true;
            } else {
                this.executeCommand('outdent', false);
            }

            return false;
        },

        _updateIndent: function (node) {
            this._toolBarObj.enableItembyID(this._rteId + "indent");
            if (node && (/^(LI)$/).test(node.nodeName)) {
                if ($(node).parents(node.parentNode.nodeName).last().find("li").index(node) === 0)
                    this._toolBarObj.disableItembyID(this._rteId + "indent");
            }
        },

        //Get char Count
        _updateCharCount: function () {
            if (this.model.showFooter && this.model.showWordCount) {
                var iframeHtml = this._getText();
                this._wordCount.html(this._getLocalizedLabels("characters") + " : " + $.trim(iframeHtml).length);
            }
        },

        //Update tag info 
        _updateTagInfo: function (element) {
            var text = "", pTag, curNode = element.tagName.toLowerCase();
            if (this.model.showFooter && this.model.showHtmlTagInfo && curNode != "body") {
                var parentNodes = $(element).parents();
                for (var i = parentNodes.length - 1; i >= 0; i--) {
                    pTag = parentNodes[i].tagName.toLowerCase();
                    if (pTag != "body" && pTag != "html")
                        text += " » " + pTag;
                }
                text += " » " + curNode;
                this._htmlInfo.html(text);
            }
        },

        //Update Model value 
        _updateValue: function () {
            this.value(this._getDocument().body.innerHTML);
        },

        //Clear Formats
        _setClearFormat: function () {
            var selItemslength = this._rteToolbar.find(".e-active").length;
        },


        //Updates the status of the toolbar items
        _updateToolbarStatus: function () {
            if (this.model.showToolBar) {
                //Style Tools
                for (var i = 0; i < this._styleItems.length; i++) {
                    if (this._getCommandStatus(this._styleItems[i].toLowerCase()))
                        this._toolBarObj.selectItembyID(this._rteId + this._styleItems[i]);
                    else
                        this._toolBarObj.deselectItembyID(this._rteId + this._styleItems[i]);
                }

                //Align Tools
                for (var i = 0; i < this._alignItems.length; i++) {
                    if (this._getCommandStatus(this._alignItems[i].toLowerCase()))
                        this._toolBarObj.selectItembyID(this._rteId + this._alignItems[i]);
                    else
                        this._toolBarObj.deselectItembyID(this._rteId + this._alignItems[i]);
                }

                //Formatting Tools
                for (var i = 0; i < this._scriptsItems.length; i++) {
                    if (this._getCommandStatus(this._scriptsItems[i].toLowerCase()))
                        this._toolBarObj.selectItembyID(this._rteId + this._scriptsItems[i]);
                    else
                        this._toolBarObj.deselectItembyID(this._rteId + this._scriptsItems[i]);
                }


                //List Tools
                if (this._getCommandStatus('InsertOrderedList'))
                    this._toolBarObj.selectItembyID(this._rteId + "orderedList");
                else
                    this._toolBarObj.deselectItembyID(this._rteId + "orderedList");

                if (this._getCommandStatus('InsertUnorderedList'))
                    this._toolBarObj.selectItembyID(this._rteId + "unorderedList");
                else
                    this._toolBarObj.deselectItembyID(this._rteId + "unorderedList");
            }

        },

        _updateFormat: function () {
            if (this._formatDDL && this.model.showToolBar && !ej.isNullOrUndefined(this.model.tools["formatStyle"])) {
                var formatName = $.trim(this._getCommandValue('formatblock').replace("'", "").replace("'", ""));
                if (ej.isNullOrUndefined(formatName) || formatName == "")
                    this._formatDDL.ejDropDownList({ "value": this._getLocalizedLabels("format") });
                else
                    this._formatDDL.ejDropDownList("selectItemByValue", formatName);
            }
        },
        // Update font options
        _updateFontOptionStatus: function () {
            if (this.model.showFontOption && this.model.showToolBar && !ej.isNullOrUndefined(this.model.tools["font"])) {
                var fontName = $.trim(this._getCommandValue('fontname').replace("'", "").replace("'", ""));
                ej.isNullOrUndefined(fontName) || fontName == "" && (fontName = "Segoe UI");
                this._fontStyleDDL.ejDropDownList("selectItemByValue", fontName);
                // Font Size DDL
                var fontSize = this._getCommandValue('fontsize');
                ej.isNullOrUndefined(fontSize) || fontSize == "" && (fontSize = 2);
                this._fontSizeDDL.ejDropDownList("selectItemByValue", fontSize);
            }
        },

        _getSelText: function () {
            var selectedtext = '';
            var selection;
            var textnode;
            if (!this._isIE()) {
                if (window.getSelection) {
                    selection = this._getWindow().getSelection();
                    var range = document.createRange();
                    range = selection.getRangeAt(0);
                    textnode = document.createTextNode(range.toString());
                    selectedtext = textnode.nodeValue;
                }
                else if (document.getSelection) {
                    selectedtext = this._getWindow().document.getSelection();
                }
            }
            else
                selectedtext = this._seleText;
            return selectedtext;
        },

        _isIE: function () {
            var _ie = false, browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie') {
                _ie = true;
            }
            return _ie;
        },
        //Vaidate the input URL string
        _isUrl: function (url) {
            var regexp = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;
            return regexp.test(url);
        },
        _pasteHtml: function (html) {
            var sel, range;
            if (window.getSelection) {
                // IE9 and non-IE
                sel = this._getWindow().getSelection();
                if (sel.getRangeAt && sel.rangeCount) {
                    range = sel.getRangeAt(0);
                    range.deleteContents();
                    // some browsers (IE9, for one)
                    var el = document.createElement("div");
                    el.innerHTML = html;
                    var frag = this._getDocument().createDocumentFragment(), node, lastNode;
                    while ((node = el.firstChild)) {
                        lastNode = frag.appendChild(node);
                    }
                    range.insertNode(frag);

                    // Preserve the selection
                    if (lastNode) {
                        range = range.cloneRange();
                        range.setStartAfter(lastNode);
                        range.collapse(true);
                        sel.removeAllRanges();
                        sel.addRange(range);
                    }
                }
            } else if (this._getDocument().selection && this._getDocument().selection.type != "Control") {
                // IE < 9
                this._getDocument().selection.createRange().pasteHTML(html);
            }
        },
        //get Localization text
        _getLocalizedLabels: function (property) {
            return ej.RTE.localization[this.model.localization][property] === undefined ? ej.RTE.localization["en-US"][property] : ej.RTE.localization[this.model.localization][property];
        },

        //get Selection Node
        _getSelectedNode: function () {
            var element, rng = this._getRange();
            var startContainer = rng.startContainer, endContainer = rng.endContainer;
            var startOffset = rng.startOffset, endOffset = rng.endOffset;
            if (!rng) {
                return this._getDocument().selection;
            }

            if (rng.setStart) {
                element = rng.commonAncestorContainer;
                //Image tage and link selction
                if (!rng.collapsed) {
                    if (startContainer == endContainer) {
                        if (endOffset - startOffset < 2) {
                            if (startContainer.hasChildNodes()) {
                                element = startContainer.childNodes[startOffset];
                            }
                        }
                    }
                    if (startContainer.nodeType === 3 && endContainer.nodeType === 3) {
                        if (startContainer.length === startOffset) {
                            startContainer = this._skipEmptyNode(startContainer.nextSibling, true);
                        } else {
                            startContainer = startContainer.parentNode;
                        }

                        if (endOffset === 0) {
                            endContainer = this._skipEmptyNode(endContainer.previousSibling, false);
                        } else {
                            endContainer = endContainer.parentNode;
                        }

                        if (startContainer && startContainer === endContainer) {
                            return startContainer;
                        }
                    }
                }

                if (element && element.nodeType == 3) {
                    return element.parentNode;
                }
                return element;
            }
            element = rng.item ? rng.item(0) : rng.parentElement();
            return element;
        },

        _skipEmptyNode: function (node, forwards) {
            var orig = node;

            while (node && node.nodeType === 3 && node.length === 0) {
                node = forwards ? node.nextSibling : node.previousSibling;
            }

            return node || orig;
        },

        _getSelection: function () {

            var selCurrent;
            if (this._isIE()) {
                selCurrent = this._getDocument().selection.createRange();
                selCurrent.type = this._getDocument().selection.type;
            }
            else {
                selCurrent = window.getSelection();
                if (selCurrent.rangeCount) {
                    sText = selCurrent.getRangeAt(0);
                }
                selCurrent.type = this._getWindow().getSelection().type;
            }
            return selCurrent;
        },

        _saveSelection: function () {
            var win = this._getWindow();
            var doc = win.document;
            var sel = win.getSelection ? win.getSelection() : doc.selection;
            var range;

            if (sel) {
                if (sel.createRange) {
                    range = sel.createRange();
                } else if (sel.getRangeAt(0)) {
                    range = sel.getRangeAt(0);
                } else if (sel.anchorNode && sel.focusNode && doc.createRange) {
                    // Older WebKit browsers
                    range = doc.createRange();
                    range.setStart(sel.anchorNode, sel.anchorOffset);
                    range.setEnd(sel.focusNode, sel.focusOffset);

                    // Handle the case when the selection was selected backwards (from the end to the start in the
                    // document)
                    if (range.collapsed !== sel.isCollapsed) {
                        range.setStart(sel.focusNode, sel.focusOffset);
                        range.setEnd(sel.anchorNode, sel.anchorOffset);
                    }
                }
            }
            return range;
        },

        _restoreSelection: function (range) {
            var win = this._getWindow();
            var doc = win.document;
            var sel = win.getSelection ? win.getSelection() : doc.selection;

            if (sel && range) {
                if (range.select) {
                    range.select();
                } else if (sel.removeAllRanges && sel.addRange) {
                    sel.removeAllRanges();
                    sel.addRange(range);
                }
            }
        },

        _getRange: function () {
            if (this._isIE())
                return ej.isNullOrUndefined(this._selectionRange) ? this._saveSelection() : this._selectionRange;
            else
                return this._getWindow().getSelection().getRangeAt(0);
        },
        //Sets Back up data
        _setBackupData: function () {
            var htmlTxt = this.getHtmlString();
            var bkuplen = this._backupArray.length;
            if (bkuplen == this.model.undoStackLimit + 1)
                this._backupArray.splice(0, 1);
            if (htmlTxt != this._backupArray[bkuplen - 1] && this.model.undoStackLimit > 0)
                this._backupArray.push(htmlTxt);
            this._undoRedoPosition = this._backupArray.length - 1;
            if (this.model.showToolBar && this._rteToolbar.find("#" + this._rteId + "undo") && this._backupArray.length > 1)
                this._toolBarObj.enableItembyID(this._rteId + "undo");
        },
        _getSelectedHtmlString: function () {
            var selection = "";
            var range;
            var node = "";
            if (document.selection) {
                node = this._getWindow().document.selection.createRange().htmlText;
                return node;
            }
            else
                return this._seleText;

            return node.innerHTML;
        },

        _getText: function () {
            var _body = this._getDocument().body;
            if (_body != null) {
                if (!this._isIE())
                    return _body.textContent;
                else
                    return _body.innerText;
            }
        },
        _validateMaxLength: function (e) {
            if (this.model.maxLength != null) {    // MaxLength function - Restricts the user for entering text in RTE
                var len = this.model.maxLength;
                var usertext = this._getText();
                if (len <= usertext.length) {
                    if (!(e.keyCode < 47 || e.ctrlKey && e.keyCode == 65 || e.ctrlKey && e.keyCode == 67 || e.ctrlKey && e.keyCode == 86 || e.ctrlKey && e.keyCode == 88)) {
                        this._keypressFlag = false;
                        this._cancelEvent(e);
                    }
                }
            }
        },
        _cancelEvent: function (e) {
            e.returnValue = false;
            e.stopPropagation();
            e.preventDefault();
            return false;
        },

        _alignToolUpdate: function (name) {
            for (var i = 0; i < this._alignItems.length; i++) {
                if (this._alignItems[i] === name)
                    this._toolBarObj.selectItembyID(this._rteId + "underline");
                else
                    this._toolBarObj.deselectItembyID(this._rteId + "underline");
            }
        },

        //Gets the status of the formats of the current text
        _getCommandStatus: function (commandName) {
            var state = this._getDocument().queryCommandState(commandName);
            return state;
        },

        //Gets the value of the font-size and font-name of the current text
        _getCommandValue: function (commandName) {
            var value = this._getDocument().queryCommandValue(commandName);
            return value;
        },

        //Undo 
        _undo: function () {
            if (this._undoRedoPosition > 0) {
                this.setHtmlString(this._backupArray[this._undoRedoPosition - 1]);
                this._undoRedoPosition--;
                this._updateToolbarStatus();
                this._updateFontOptionStatus();
                this._updateFormat();
                this._updateCharCount();
                this._toolBarObj.enableItembyID(this._rteId + "redo");
            }
            if (this._undoRedoPosition == 0) {
                this._toolBarObj.disableItembyID(this._rteId + "undo");
                this._toolBarObj.enableItembyID(this._rteId + "redo");
            }
            this._focus();
        },

        //Redo 
        _redo: function () {
            if (this._undoRedoPosition != this._backupArray.length) {
                if (this._backupArray[this._undoRedoPosition + 1] != null) {
                    var redoString = this._backupArray[this._undoRedoPosition + 1];
                    this.setHtmlString(redoString);
                    this._undoRedoPosition++;
                }
                else {
                    var redoString = this._backupArray[this._backupArray.length - 2];
                    this.setHtmlString(redoString);
                    this._undoRedoPosition = this._backupArray.length - 2;
                }
                this._updateToolbarStatus();
                this._updateFontOptionStatus();
                this._updateFormat();
                this._updateCharCount();
                if (!ej.isNullOrUndefined(this._toolBarObj)) {
                    if (this._backupArray.length - 1 > 0)
                        this._toolBarObj.enableItembyID(this._rteId + "undo");
                    if (this._undoRedoPosition == this._backupArray.length - 1)
                        this._toolBarObj.disableItembyID(this._rteId + "redo");
                }
            }
            this._focus();
        },

        //set content
        _setContent: function (htmltxt) {
            var editdoc = this._getDocument();
            editdoc.open();
            editdoc.write(htmltxt);
            editdoc.close();
            editdoc.EditMode = true;
        },

        //table start to create 
        _tableCellStart: function (e) {
            this._tableInsertAt = this._saveSelection();
        },

        //Select the table Cell on mouse over
        _tableCellSelect: function (e) {
            var cellEle, target = $(e.target);
            var row = target.parent().index();
            var col = target.index();
            this._createTable.find("div.e-rte-tablecell").each(function (index) {
                cellEle = $(this);
                if (cellEle.index() <= col && cellEle.parent().index() <= row) {
                    cellEle.addClass("e-active");
                }
                else {
                    cellEle.removeClass("e-active");
                }
            });
            this._tblheaderDiv.html((col + 1) + "x" + (row + 1) + " Table");
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            if (!ej.isNullOrUndefined(this._tableInsertAt) && !ej.isNullOrUndefined(this._tableInsertAt.startContainer)) {
                this._restoreSelection(this._tableInsertAt);
                this._onInsertTable((row + 1), (col + 1), true);
            }
        },
        _tableCellLeave: function (e) {
            this._createTable.find("div.e-rte-tablecell").removeClass("e-active");
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            this._tblheaderDiv.html(this._getLocalizedLabels("createTable"));
        },
        _tableCellDown: function (e) {
            var target = $(e.target);
            var row = target.parent().index() + 1;
            var col = target.index() + 1;
            this._focus();
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            if (!ej.isNullOrUndefined(this._tableInsertAt) && !ej.isNullOrUndefined(this._tableInsertAt.startContainer)) {
                this._restoreSelection(this._tableInsertAt);
                this._onInsertTable(row, col, false);
            }
            var _tableEle = $(this._getDocument()).find("table.e-rte-table");
            _tableEle.attr("contentEditable", false);
            _tableEle.find("td").attr("contentEditable", true);
            this._createTable.ejDialog("close");
            this._setBackupData();
            this._onChange();
        },

        _tableGenerator: function (rows, cols, designTime, width, height, spacing, padding, align, border, caption) {
            var tblClass = "e-rte-table", colTemplate = "<td><br _moz_dirty=''/></td>";
            if (designTime)
                tblClass += " e-rte-tableremove";
            width = ej.isNullOrUndefined(width) || width == "" ? "width='100%'" : "width=" + width + "'";
            height = ej.isNullOrUndefined(height) || height == "" ? "" : "height='" + height + "'";
            spacing = ej.isNullOrUndefined(spacing) || spacing == "" ? "" : "cellspacing='" + spacing + "'";
            padding = ej.isNullOrUndefined(padding) || padding == "" ? "" : "cellpadding='" + padding + "'";
            border = ej.isNullOrUndefined(border) || border == "" ? "border='1'" : "border='" + border + "'";
            align = ej.isNullOrUndefined(align) ? "" : " align='" + align + "'";
            return "<table class='" + tblClass + "' " + width + " " + height + " " + border + " " + spacing + " " + padding + " " + align + ">" + (caption ? "<caption></caption>" : "") + Array(rows + 1).join("<tr>" + Array(cols + 1).join(colTemplate) + "</tr>") + "</table>";
        },
        //custom table link click event 
        _openCustomTable: function (e) {
            this._customTableDialog.ejDialog("open");
        },
        _documentClick: function (e) {
            if ((!ej.isNullOrUndefined(this._createTable)) && ($(e.target).parents("#" + this._rteId + "tables").length <= 0))
                this._createTable.ejDialog("isOpened") && this._createTable.ejDialog("close");
        },
        // iframe events 
        _iframeFocus: function (e) {
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog("isOpened") && this._createTable.ejDialog("close");
            !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.ejDropDownList("popupListHide");
            !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.ejDropDownList("popupListHide");
            !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.ejDropDownList("popupListHide");
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog("close");
            !ej.isNullOrUndefined(this._splitMenu) && this._splitMenu.hideContextMenu(e);
            !ej.isNullOrUndefined(this._bgSplitMenu) && this._bgSplitMenu.hideContextMenu(e);
        },
        _iframeKeyDown: function (e) {
            var _toolEle;
            if (this.model.showToolBar && this.model.allowKeyboardNavigation) {
                if (e && e.ctrlKey == true) {
                    /// Ctrl-b is pressed
                    if (e.keyCode == 66) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "bold");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onBold();
                            if (!_toolEle.hasClass("e-active"))
                                this._toolBarObj.selectItembyID(this._rteId + "bold");
                            else
                                this._toolBarObj.deselectItembyID(this._rteId + "bold");
                        }
                        return false;
                    }
                        /// Ctrl-i is pressed
                    else if (e.keyCode == 73) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "italic");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onItalics();
                            if (!_toolEle.hasClass("e-active"))
                                this._toolBarObj.selectItembyID(this._rteId + "italic");
                            else
                                this._toolBarObj.deselectItembyID(this._rteId + "italic");
                        }
                        return false;
                    }
                        /// Ctrl-u is pressed.
                    else if (e.keyCode == 85) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "underline");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onUnderLine();
                            if (!_toolEle.hasClass("e-active"))
                                this._toolBarObj.selectItembyID(this._rteId + "underline");
                            else
                                this._toolBarObj.deselectItembyID(this._rteId + "underline");
                        }
                        return false;
                    }
                        //justify Left Crtl-l
                    else if (e.keyCode == 76 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "justifyLeft");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyLeft();
                            this._alignToolUpdate("justifyLeft");
                        }
                        return false;
                    }
                        //justify right Crtl-r
                    else if (e.keyCode == 82) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "justifyRight");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyRight();
                            this._alignToolUpdate("justifyRight");
                        }
                        return false;
                    }
                        //justify center Crtl-e
                    else if (e.keyCode == 69 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "justifyCenter");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyCenter();
                            this._alignToolUpdate("justifyCenter");
                        }
                        return false;
                    }
                        //justify center Crtl-J
                    else if (e.keyCode == 74 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "justifyFull");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyFull();
                            this._alignToolUpdate("justifyFull");
                        }
                        return false;
                    }
                        //Undo
                    else if (e.keyCode == 90 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "undo");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onUndo();
                        }
                        return false;
                    }
                        //REDO
                    else if (e.keyCode == 89) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "rddo");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onRedo();
                        }
                        return false;
                    }
                        //COPY
                    else if (e.keyCode == 67) {
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "copy");
                        if (_toolEle.hasClass("e-disable")) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }
                    }
                        //CUT
                    else if (e.keyCode == 88) {
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "cut");
                        if (_toolEle.hasClass("e-disable")) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }

                    }
                        //PASTE
                    else if (e.keyCode == 86) {
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "paste");
                        if (_toolEle.hasClass("e-disable")) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }
                    }
                        //INDENT
                    else if (e.keyCode == 77) {

                        if (e.shiftKey == true) {
                            e.preventDefault();
                            _toolEle = this._toolBarItems.find("#" + this._rteId + "outdent");
                            if (!_toolEle.hasClass("e-disable")) {
                                this._onOutdent();
                                if (this._indentdepth > 0)
                                    this._indentdepth--;
                            }
                            return false;
                        }
                        else {
                            e.preventDefault();
                            _toolEle = this._toolBarItems.find("#" + this._rteId + "indent");
                            if (!_toolEle.hasClass("e-disable")) {
                                this._onIndent();
                                this._indentdepth++;
                            }
                            return false;
                        }
                    }
                        //Hyperlink
                    else if (e.keyCode == 75) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "createLink");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._hyperLinkManager();
                        }
                        return false;

                    }
                }
                else if (e.keyCode == 9) {
                    var tdIndex, tdList, ele = this._getSelectedNode();
                    if (ele && ele.tagName.toLowerCase() == "td") {
                        tdList = $(ele).closest("table").find("td");
                        tdIndex = tdList.index(ele);
                        if (tdIndex == tdList.length - 1)
                            this.insertRow(false, ele);
                    }
                    else if (!this._isIE()) {
                        e.preventDefault();
                        this._getDocument().execCommand('inserthtml', false, "&nbsp;&nbsp;&nbsp;&nbsp;");
                    }
                }
                    // Delete and Delete All
                else if (e.keyCode == 127) {
                    if ((this.getHtmlString() == this.getSelHtmlString()) || (this._currentIFrame.get_Text() == this._currentIFrame.getSelText())) {
                        var isClearAll = this.ClearAllManager();
                        if (!isClearAll) {
                            e.preventDefault();
                            return isClearAll;
                        }
                    }
                }
            }
            if (this.model.maxLength != null) {
                this._validateMaxLength(e);
            }
            this._trigger("keydown", { keyCode: e.keyCode });
            // if (e.keyCode != 16 || e.keyCode != 17 || e.keyCode != 18)
            // this._onChange();
        },
        //Key press event
        _iframeKeypress: function (e) {
            if (!this._keypressFlag) {
                e.preventDefault();
                this._keypressFlag = true;
                return false;
            }
        },
        _iframeKeyUp: function (e) {
            this._toggleEditTable();
            this._setBackupData();
            this._updateToolbarStatus();
            this._updateFontOptionStatus();
            this._updateFormat();
            this._updateCharCount();
            var selectEle = this._getSelectedNode();
            this._updateTagInfo(selectEle);
            this._onChange();
            this._trigger("keyup", { keyCode: e.keyCode });
        },
        _iframeMouseUp: function (e) {
            this._toggleEditTable();
            this._updateToolbarStatus();
            this._updateFontOptionStatus();
            this._updateFormat();
        },
        _iframeMouseDown: function (e) {
            this._updateTagInfo(e.target);
            this._updateIndent(e.target);
        },
        _iframeFocusOut: function (e) {
            this._onChange();
            this._selectionRange = this._saveSelection();
            this._seleText = ej.isNullOrUndefined(this._getWindow().document.selection) ? this._selectionRange.toString() : this._getWindow().document.selection.createRange().text;
        },
        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            var _iframe = $(this._getDocument());
            if (!ej.isNullOrUndefined(this._createTable)) {
                this._on(this._createTable, "mouseenter", "div.e-rte-tablecell", this._tableCellStart);
                this._on(this._createTable, "mousemove", "div.e-rte-tablecell", this._tableCellSelect);
                this._on(this._createTable, "mouseleave", "div.e-rte-table", this._tableCellLeave);
                this._on(this._createTable, "mousedown", "div.e-rte-tablecell", this._tableCellDown);
                this._on($(document), "click", this._documentClick);
            }
            this._on(_iframe, "focus", this._iframeFocus);
            this._on(_iframe, "keydown", this._iframeKeyDown);
            this._on(_iframe, "keypress", this._iframeKeypress);
            this._on(_iframe, "keyup", this._iframeKeyUp);
            this._on(_iframe, "mouseup", this._iframeMouseUp);
            this._on(_iframe, "mousedown", this._iframeMouseDown);
            this._on(_iframe, "focusout", this._iframeFocusOut);
        },

        _unwireEvents: function () {
            var _iframe = $(this._getDocument());
            if (!ej.isNullOrUndefined(this._createTable)) {
                this._off(this._createTable.find("div.e-rte-tablecell"), "mousemove");
                this._off(this._createTable.find("div.e-rte-table"), "mouseleave");
                this._off(this._createTable.find("div.e-rte-tablecell"), "mousedown");
                this._off($(document), "click");
            }
            this._off(_iframe, "focus");
            this._off(_iframe, "keydown");
            this._off(_iframe, "keyup");
            this._off(_iframe, "mouseup");
            this._off(_iframe, "mousedown");
            this._off(_iframe, "blur");
        },

        /*----------------------- Event handler------------------------------------------*/
        _onChange: function () {
            _prevhtml = $.trim(this.value()).replace(/<\/(p|li|ul|ol|h[1-6]|table|tr|td|th)>/gi, "</$1>\n").replace(/<(ul|ol)([^>]*)><li/gi, "<$1$2>\n<li").replace(/<br \/>/gi, "<br />\n").replace(/\n$/, "").replace(/&nbsp;/g, ' ');
            if (_prevhtml !== this.getHtmlString()) {
                this._updateValue();
                var args = { text: this._getText(), htmlText: this.getHtmlString() };
                this._trigger("change", args);
            }
        },

        /*----------------------- Public Methods-----------------------------------------*/

        //Disables toolbar item
        /**
        * disable the RTE toolbar item.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.disableToolbarItem(1); // disable toolbar item
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        disableToolbarItem: function (id) {
            !ej.isNullOrUndefined(id) && this._toolBarObj.disableItembyID(id);
        },
        /**
        * enable  the RTE toolbar item.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.enableToolbarItem(1); // enable toolbar item
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //To enable toolbar item
        enableToolbarItem: function (id) {
            !ej.isNullOrUndefined(id) && this._toolBarObj.enableItembyID(id);
        },
        /**
        * removes  the RTE toolbar item.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.removeToolbarItem(1); // remove toolbar item
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Removes toolbar item
        removeToolbarItem: function (id) {
            !ej.isNullOrUndefined(id) && this._toolBarObj.removeItembyID(id);
        },
        /**
        * Displays the RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.show(); // shows rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Displays the control
        show: function () {
            this._rteWapper.show();
        },
        /**
        * Hides the RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.hide(); // hides rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Hides the control
        hide: function () {
            this._rteWapper.hide();
        },
        /**
        * gets the HTML string from the RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.getHtmlString(); // gets the html string from rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        getHtmlString: function () {
            var _html = this._getDocument().body.innerHTML;
            _html = $.trim(_html).replace(/<\/(p|li|ul|ol|h[1-6]|table|tr|td|th)>/gi, "</$1>\n").replace(/<(ul|ol)([^>]*)><li/gi, "<$1$2>\n<li").replace(/<br \/>/gi, "<br />\n").replace(/\n$/, "").replace(/&nbsp;/g, ' ');
            return _html;
        },
        /**
        * sets the HTML string from the RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.setHtmlString(value); // sets the html string to rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Set the 
        setHtmlString: function (value) {
            this._getDocument().body.innerHTML = value;
        },
        /**
        * gets the content as string from the RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.setHtmlString(); // gets the content as string from rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Get plan text
        getText: function () {
            return this._getText();
        },
        /**
        * gets the content as string from the RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.executeCommand(command name ,arguments); // gets the content as string from rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Execute Command
        executeCommand: function (cmdName, args) {
            if (this._trigger("execute", { commandName: cmdName }))
                return false;
            var anchor = "";
            if (cmdName == "underline") {
                if (window.getSelection) {
                    anchor = $(this._getWindow().getSelection().anchorNode).parents("a");
                    if (anchor.length > 0) {
                        $(anchor).css("text-decoration", ($(anchor).css("text-decoration") == "none") ? "underline" : "none");
                        return;
                    }
                }
            }
            else if (cmdName.toLowerCase() == "inserthtml") {
                if (this._isIE()) {
                    this._pasteHtml(args);
                }
                else
                    this._getDocument().execCommand(cmdName, false, args);
                return;
            }
            //this._focus();
            this._getDocument().execCommand(cmdName, false, args);
            this._setBackupData();
        },
        /**
        * focus the RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.executeCommand(command name ,arguments); // gets the content as string from rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //focus the editable area.
        focus: function () {
            this._focus();
        },
        /**
        * gets the command status of the given RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.commandStatus(arguments); // gets the content as string from rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //get the command status
        commandStatus: function (value) {
            if (ej.isNullOrUndefined(value) || value != "")
                return this._getCommandStatus(value);
        },
        /**
        * gets the command status of the given RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.commandStatus(arguments); // gets the content as string from rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //currently selected text html
        selectedHtml: function () {
            return this._getSelectedHtmlString();
        },
        /**
        * inert rows in the given RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.insertRow(before,cell); // insert row in RTE
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Tables methosd
        //insert table Rows
        insertRow: function (before, cell) {
            //$("#rte1_Iframe").contents().find("#test").parents("tr").find("td").length
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var colTemplate = "<td contentEditable='true'><br _moz_dirty=''/></td>", curRow = $(seletedCell).closest("tr");
            var cols = curRow.find("td").length;
            var newRow = $("<tr>" + Array(cols + 1).join(colTemplate) + "</tr>");
            before ? newRow.insertBefore(curRow) : newRow.insertAfter(curRow);
            curRow.find("td:first").focus();
        },
        /**
        * inert column in the given RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.insertColumn(before,cell); // Insert column in RTE
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Insert table Columns
        insertColumn: function (before, cell) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var colTemplate = "<td contentEditable='true'><br _moz_dirty=''/></td>", curRow = $(seletedCell).closest("tr"), curCell;
            var allRows = curRow.closest("table").find("tr");
            var colIndex = curRow.find("td").index(seletedCell);
            for (var i = 0; i < allRows.length; i++) {
                curCell = $(allRows[i]).find("td")[colIndex];
                before ? $(colTemplate).insertBefore(curCell) : $(colTemplate).insertAfter(curCell);
            }
        },
        /**
        * remove row in the given RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.removeRow(cell); // Remove row in RTE
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Remove table Row
        removeRow: function (cell) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            curRow = $(seletedCell).closest("tr").remove();
        },
        /**
        * remove column in the given RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.removeColumn(cell); // Remove column in RTE
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Remove table Column
        removeColumn: function (cell) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var curRow = $(seletedCell).closest("tr");
            var allRows = curRow.closest("table").find("tr");
            var colIndex = curRow.find("td").index(seletedCell);
            for (var i = 0; i < allRows.length; i++) {
                $($(allRows[i]).find("td")[colIndex]).remove();
            }
        },
        /**
        * remove table in the given RTE control.
        * @example 
        * &lt;textarea   id="rte"&gt; 	                  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        <br> 
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.removeTable(table); // Remove table in RTE
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Remove table 
        removeTable: function (table) {
            var seletedTable = ej.isNullOrUndefined(table) ? $(this._getSelectedNode()).closest("table") : table;
            $(seletedTable).remove();
        }
    });

    ej.RTE.localization = {};

    ej.RTE.localization["en-US"] = {
        bold: "Bold",
        italic: "Italic",
        underline: "Underline",
        strikethrough: "Strikethrough",
        superscript: "Superscript",
        subscript: "Subscript",
        justifyCenter: "Align text center",
        justifyLeft: "Align text left",
        justifyRight: "Align text right",
        justifyFull: "Justify",
        unorderedList: "Insert unordered list",
        orderedList: "Insert ordered list",
        indent: "Indent",
        outdent: "Outdent",
        cut: "Cut",
        copy: "Copy",
        paste: "Paste",
        paragraph: "Paragraph",
        undo: "Undo",
        redo: "Redo",
        upperCase: "Upper Case",
        lowerCase: "Lower Case",
        clearAll: "Clear All",
        clearFormat: "Clear Format",
        createLink: "Insert/Edit hyperlink",
        image: "Insert image",
        video: "Insert video",
        embedVideo: "Paste your embed code below",
        viewHtml: "View HTML",
        fontName: "Select font family",
        fontSize: "Select font size",
        fontColor: "Select color",
        format: "Format",
        backgrounColor: "Background color",
        style: "Styles",
        deleteAlert: "Are you sure you want to clear all the contents?",
        copyPastAlert: "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.",
        videoError: "The text area can not be empty",
        imageWebUrl: "Web Address",
        imageAltText: "Image description",
        dimensions: "Dimensions",
        constrainProportions: "Constrain Proportions",
        linkWebUrl: "Web Address",
        linkText: "Text",
        linkToolTip: "ToolTip",
        linkOpenInNewWindow: "Open link in new window",
        tableColumns: "No.of Columns",
        tableRows: "No.of Rows",
        tableWidth: "Table width",
        tableHeight: "Table height",
        tableCellSpacing: "Cell spacing",
        tableCellPadding: "Cell padding",
        tableBorder: "Border",
        tableCaption: "Caption",
        tableAlignment: "Alignment",
        dialogUpdate: "Update",
        dialogInsert: "Insert",
        dialogCancel: "Cancel",
        dialogOk: "Ok",
        createTable: "Create table",
        addColumnLeft: "Add column on the left",
        addColumnRight: "Add column on the right",
        addRowAbove: "Add row above",
        addRowBelow: "Add row below",
        deleteRow: "Delete the row",
        deleteColumn: "Delete the column",
        deleteTable: "Delete the table",
        customTable: "Create custom table...",
        characters: "Characters"
    };

})(jQuery, Syncfusion);;
/**
 * @fileOverview Plugin to style the Html Grid elements
 * @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
 *  Use of this code is subject to the terms of our license.
 *  A copy of the current license can be obtained at any time by e-mailing
 *  licensing@syncfusion.com. Any infringement will be prosecuted under
 *  applicable laws. 
 * @version 12.1 
 * @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
 */

(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejGrid
    * @requires jQuery
    * @requires jsrender
    * @requires ej.common.all
    * @classdesc Custom Design for Html Grid control.
    * @example 
    * &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create Grid
    * $('#Grid').ejGrid();         
    * &lt;/script&gt;
    */
    /** 
     * @class edit
     * @classdesc This property is used to customize the editing behavior of the grid.     
     */
    /** 
    * @class pageSettings
    * @classdesc This property is used to modify pager default configuration.   
    */
    /** 
    * @class groupSettings
    * @classdesc This property is used to customize the grouping behavior of the grid.    
    */
    /** 
    * @class filterSettings
    * @classdesc This property is used to customize the filtering behavior of the grid. 
    */
    /** 
    * @class sortSettings
    * @classdesc This property is used to define initial sorting for particular columns in the grid.
    */
    /** 
    * @class toolBar
    * @classdesc Enable toolbar for editing action by setting the property as true.
    */
    /** 
    * @class scrolling
    * @classdesc Specify the width for the control and the grid automatically displays horizontal scroll bars if needed.  
    */
    ej.widget("ejGrid", "ej.Grid", {
        //Root Css Class
        _rootCSS: "e-grid",
        // widget element will be automatically set in this
        element: null,
        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,
        keyConfigs: {
            focus: "e",
            insertRecord: "45", //Insert
            deleteRecord: "46", // delete
            editRecord: "113", //F2
            saveRequest: "13", // enter
            cancelRequest: "27", //Esc
            nextPage: "34", // PgDn
            previousPage: "33", // PgUp
            lastPage: "ctrl+alt+34", //"CtrlAltPgDn",
            firstPage: "ctrl+alt+33", //"CtrlPlusAltPlusPgUp",
            nextPager: "alt+34",//"AltPlusPgDown",
            previousPager: "alt+33", //"AltPlusPgUp",
            firstCellSelection: "36", //"Home",
            lastCellSelection: "35", //"End",
            firstRowSelection: "ctrl+36", //"CtrlPlusHome",
            lastRowSelection: "ctrl+35",  //"CtrlPlusEnd",
            upArrow: "38", //Up arrow
            downArrow: "40", //Down arrow
            moveCellRight: "9",//tab
            moveCellLeft: "shift+9",//shifttab
            selectedGroupExpand: "alt+40", //"AltPlusDownArrow",
            totalGroupExpand: "ctrl+40", //"CtrlPlusDownArrow",
            selectedGroupCollapse: "alt+38", //"AltPlusUpArrow",
            totalGroupCollapse: "ctrl+38" //"CtrlPlusUpArrow",
        },
        _ignoreOnPersist: ["query", "isEdit", "toolBarClick", "queryCellInfo", "currentViewData", "altRow", "rtl",
            "rowDataBound", "rowTemplate", "detailData", "detailTemplate", "summaryRows", "toolBar",
            "edit", "allowMultiSorting", "autoSaveOnRowSelection", "localization", "allowScrolling",
            "cssClass", "dataSource", "enableEffects", "rowHover", "allowSummary",
            "headerEffects", "allowKeyboardNavigation", "scrolling.frozenRows", "scrolling.frozenColumns"],

        observables: ["dataSource", "selectedRow", "pageSettings.currentPage"],
        _dataSource: ej.util.valueFunction("dataSource"),
        _selectedRow: ej.util.valueFunction("selectedRow"),
        _currentPage: ej.util.valueFunction("pageSettings.currentPage"),
        // default model
        defaults: {
            /**		
             * This will enable or disable the grid pager.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowPaging during initialization. 			
             * 	$("#Grid").ejGrid({allowPaging: true });			 
             * @example 
             * //Get or set the allowPaging after initialization:<br>
             *	// Get allowPaging of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowPaging");			
             *	// Set allowPaging of grid.
             *	$("#Grid").ejGrid("option", { allowPaging: true });
              * @memberof ejGrid
             * @instance
             */
            allowPaging: false,
            /**		
             * This will enable or disable the sorting behavior for the grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowSorting during initialization. 			
             * 	$("#Grid").ejGrid({ allowSorting: true });			 
             * @example 
             * //Get or set the allowSorting after initialization:<br>
             *	// Get allowSorting of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowSorting");		
             *	// Set allowSorting of grid.
             *	$("#Grid").ejGrid("option", { allowSorting: true });
              * @memberof ejGrid
             * @instance
             */
            allowSorting: false,
            /**		
             * This will enable or disable the filtering behavior for the grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowFiltering during initialization. 			
             * 	$("#Grid").ejGrid({ allowFiltering: true });			 
             * @example 
             * //Get or set the allowFiltering after initialization:<br>
             *	// Get allowFiltering of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowFiltering");		
             *	// Set allowFiltering of grid.
             *	$("#Grid").ejGrid("option", { allowFiltering: true });
              * @memberof ejGrid
             * @instance
             */
            allowFiltering: false,
            /**		
             * This will enable or disable selection behavior for the grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowSelection during initialization. 			
             * 	$("#Grid").ejGrid({ allowSelection: true });			 
             * @example 
             * //Get or set the allowSelection after initialization:<br>
             *	// Get allowSelection of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowSelection");		
             *	// Set allowSelection of grid.
             *	$("#Grid").ejGrid("option", { allowSelection: true });
              * @memberof ejGrid
             * @instance
             */
            allowSelection: true,
            /**		
             * This will enable or disable grouping behavior for the grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowGrouping during initialization. 			
             * 	$("#Grid").ejGrid({ allowGrouping: true });			 
             * @example 
             * //Get or set the allowGrouping after initialization:<br>
             *	// Get allowGrouping of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowGrouping");		
             *	// Set allowGrouping of grid.
             *	$("#Grid").ejGrid("option", { allowGrouping: true });
              * @memberof ejGrid
             * @instance
             */
            allowGrouping: false,
            /**		
             * Apply animation effects to the group drop area collapse and expand.
             * @default true
             * @type {boolean}
             * @example 
             * // Set the allowGrouping during initialization. 			
             * 	$("#Grid").ejGrid({ enableEffects: true });			 
             * @example 
             * //Get or set the enableEffects after initialization:<br>
             *	// Get enableEffects of grid.	
             *	var value = $("#Grid").ejGrid("option", "enableEffects");		
             *	// Set enableEffects of grid.
             *	$("#Grid").ejGrid("option", { enableEffects: false });
              * @memberof ejGrid
             * @instance
             */
            enableEffects: true,
            /**		
             * Enable or disable the summary behavior of the grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowSummary during initialization. 			
             * 	$("#Grid").ejGrid({ allowSummary : true });			 
             * @example 
             * //Get or set the allowSummary after initialization:<br>
             *	// Get allowSummary of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowSummary");		
             *	// Set allowSummary of grid.
             *	$("#Grid").ejGrid("option", { allowSummary : true });
              * @memberof ejGrid
             * @instance
             */
            allowSummary: false,
            /**		
             * Enable or disable the resizing column behavior of the grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowResizing during initialization. 			
             * 	$("#Grid").ejGrid({ allowResizing : true });			 
             * @example 
             * //Get or set the allowResizing after initialization:<br>
             *	// Get  allowResizing  of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowResizing");		
             *	// Set allowResizing of grid.
             *	$("#Grid").ejGrid("option", { allowResizing : true });
              * @memberof ejGrid
             * @instance
             */
            allowResizing: false,
            /**		
             * This will enable or disable the row hovering effect of the grid.
             * @default true
             * @type {boolean}
             * @example 
             * // Set the rowHover during initialization. 			
             * 	$("#Grid").ejGrid({ rowHover : true });			 
             * @example 
             * //Get or set the rowHover after initialization:<br>
             *	// Get rowHover of grid.	
             *	var value = $("#Grid").ejGrid("option", "rowHover");		
             *	// Set rowHover of grid.
             *	$("#Grid").ejGrid("option", { rowHover : false });
              * @memberof ejGrid
             * @instance
             */
            rowHover: true,
            /**		
             * Apply state maintenance to the grid
             * @default false
             * @type {boolean}
             * @example 
             * // Set the persist during initialization. 			
             * 	$("#Grid").ejGrid({ persist : true });			 
             * @example 
             * //Get or set the persist after initialization:<br>
             *	// Get persist of grid.	
             *	var value = $("#Grid").ejGrid("option", "persist");		
             *	// Set persist of grid.
             *	$("#Grid").ejGrid("option", { persist : false });
              * @memberof ejGrid
             * @instance
             */
            persist: false,
            /**		
             * This will apply row selection based on row index value.
             * @default -1
             * @type {number}
             * @example 
             * // Set the selectedRow during initialization. 			
             * 	$("#Grid").ejGrid({selectedRow: 2});		 
             * @example 
             * //Get or set the selectedRow after initialization:<br>
             *	// Get selectedRow of grid.	
             *	var value = $("#Grid").ejGrid("option", "selectedRow");	
             *	// Set selectedRow of grid.
             *	$("#Grid").ejGrid("option", { selectedRow: 2});
              * @memberof ejGrid
             * @instance
             */
            selectedRow: -1,
            /**		
             * Enable or disable searching behavior for the grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowSearching during initialization. 			
             * 	$("#Grid").ejGrid({ allowSearching : true });			 
             * @example 
             * //Get or set the allowSearching after initialization:<br>
             *	// Get allowSearching of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowSearching");		
             *	// Set allowSearching of grid.
             *	$("#Grid").ejGrid("option", { allowSearching : false });
              * @memberof ejGrid
             * @instance
             */
            allowSearching: false,
            /**		
             * Apply hover selection effect to the header cell of a grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the headerEffects during initialization. 			
             * 	$("#Grid").ejGrid({ headerEffects : true });			 
             * @example 
             * //Get or set the headerEffects after initialization:<br>
             *	// Get headerEffects of grid.	
             *	var value = $("#Grid").ejGrid("option", "headerEffects");		
             *	// Set headerEffects of grid.
             *	$("#Grid").ejGrid("option", { headerEffects : true });
              * @memberof ejGrid
             * @instance
             */
            headerEffects: false,
            /**		
             * Enable the column reorder feature by setting the property as true.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowReordering during initialization. 			
             * 	$("#Grid").ejGrid({ allowReordering : true });			 
             * @example 
             * //Get or set the allowReordering after initialization:<br>
             *	// Get allowReordering of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowReordering");		
             *	// Set allowReordering of grid.
             *	$("#Grid").ejGrid("option", { allowReordering : true });
              * @memberof ejGrid
             * @instance
             */
            allowReordering: false,
            /**		
             * Disable the keyboard navigation feature by setting the property as false.
             * @default true
             * @type {boolean}
             * @example 
             * // Set the allowKeyboardNavigation during initialization. 			
             * 	$("#Grid").ejGrid({ allowKeyboardNavigation : false });			 
             * @example 
             * //Get or set the allowKeyboardNavigation after initialization:<br>
             *	// Get allowKeyboardNavigation of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowKeyboardNavigation");		
             *	// Set allowKeyboardNavigation of grid.
             *	$("#Grid").ejGrid("option", { allowKeyboardNavigation : false });
              * @memberof ejGrid
             * @instance
             */
            allowKeyboardNavigation: true,
            /**		
			 * Defines the row selection behavior of grid; this will enable single selection or multiple selections of the grid.	See {@link selectionType}
			 * @default ej.Grid.selectionType.Single
			 * @type {enum}
			 * @example 
			 * //Set the selectionType during initialization. 
			 * 	$("#Grid").ejGrid({ selectionType: ej.Grid.selectionType.Multiple });					 
			 * @example 
			 * //Get or set the selectionType after initialization:<br>
			 *	//Get selectionType of grid.	 
			 *	var value = $("#Grid").ejGrid("option", "selectionType");			
			 *	//Set selectionType of grid.
			 *	$("#Grid").ejGrid("option", { selectionType: ej.Grid.selectionType.Multiple }); 
			  * @memberof ejGrid
			 * @instance
			 */
            selectionType: "single",
            /**		
			 * Render grid records by the specified dataSource.
			 * @default null
			 * @type {object}
			 * @example 
			 * //Set the dataSource during initialization. 
			 * 	$("#Grid").ejGrid({dataSource: data });					 
			 * @example 
			 * //Get or set the dataSource after initialization:<br>
			 *	//Get dataSource of grid.	 
			 *	var value = $("#Grid").ejGrid("option", "dataSource");		
			 *	//Set dataSource of grid.
			 *	$("#Grid").ejGrid("option", { dataSource: data }); 
			  * @memberof ejGrid
			 * @instance
			 */
            dataSource: null,
            /**		
			 * Specify the CSS class to grid to achieve custom theme.
			 * @default ""
			 * @type {string}
			 * @example 
			 * // Set the CSS class during initialization. 			
			 * 	$("#Grid").ejGrid({  cssClass : "gradient-lime" });			 
			 * @example 
			 * //Get or set the cssClass after initialization:<br>
			 *	// Gets the cssClass corner value.		
			 *	$("#Grid").ejGrid("option", "cssClass");			
			 *	// Sets the cssClass corner to grid
			 *	$("#Grid").ejGrid("option", "cssClass",  "gradient-lime" );
			  * @memberof ejGrid
			 * @instance
			 */
            cssClass: "",
            /**		
             * This will enable or disable scrolling for the grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowScrolling during initialization. 			
             * 	$("#Grid").ejGrid({ allowScrolling : true });			 
             * @example 
             * //Get or set the allowScrolling after initialization:<br>
             *	// Get allowScrolling of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowScrolling");		
             *	// Set allowScrolling of grid.
             *	$("#Grid").ejGrid("option", { allowScrolling : true });
              * @memberof ejGrid
             * @instance
             */
            allowScrolling: false,
            /**		
            * Defines the localization for grid.
            * @default "en-US"
            * @type {string}
            * @example 
            * // Set the localization during initialization. 			
            * 	$("#Grid").ejGrid({ localization : "es-ES" });			 
            * @example 
            * //Get or set the localization after initialization:<br>
            *	// Get localization of grid.	
            *	var value = $("#Grid").ejGrid("option", "localization");		
            *	// Set localization of grid.
            *	$("#Grid").ejGrid("option", { localization : "es-ES" });
             * @memberof ejGrid
            * @instance
            */
            localization: "en-US",
            /**		
             * Enables or disables the save action in the grid by row selection.
             * @default true
             * @type {boolean}
             * @example 
             * // Set the autoSaveOnRowSelection during initialization. 			
             * 	$("#Grid").ejGrid({ autoSaveOnRowSelection : false });			 
             * @example 
             * //Get or set the autoSaveOnRowSelection after initialization:<br>
             *	// Get autoSaveOnRowSelection of grid.	
             *	var value = $("#Grid").ejGrid("option", "autoSaveOnRowSelection");		
             *	// Set autoSaveOnRowSelection of grid.
             *	$("#Grid").ejGrid("option", { autoSaveOnRowSelection : true });
              * @memberof ejGrid
             * @instance
             */
            autoSaveOnRowSelection: true,
            /**		
             * Enables or disables the multi sorting behavior of grid.
             * @default false
             * @type {boolean}
             * @example 
             * // Set the allowMultiSorting during initialization. 			
             * 	$("#Grid").ejGrid({ allowMultiSorting : true });			 
             * @example 
             * //Get or set the allowMultiSorting after initialization:<br>
             *	// Get allowMultiSorting of grid.	
             *	var value = $("#Grid").ejGrid("option", "allowMultiSorting");		
             *	// Set allowMultiSorting of grid.
             *	$("#Grid").ejGrid("option", { allowMultiSorting : false });
              * @memberof ejGrid
             * @instance
             */
            allowMultiSorting: false,
            /**		
             * This property is used to customize the editing behavior of the grid. 
             * @default 
             * @type {Object}
             * @example            
             * $("#Grid").ejGrid({
             *     edit: { allowEditing: true, allowAdding: true, allowDeleting: true }                                      
             *});             
              * @memberof ejGrid
             * @instance
             */
            edit: {
                /**		
                 * Enables or disables editing behavior of the grid.
                 * @name edit.allowEditing
                 * @default false
                 * @type {boolean}
                 * @example
                 * Defining allowEditing with the edit option”
                 * $("#Grid").ejGrid({
                 *     edit: { allowEditing: true }
                 * });
                 * // Get or set the allowEditing after grid initialize.
                 *  // Get allowEditing of grid.
                 *  var value = $("#Grid").ejGrid("option", "edit.allowEditing");
                 *  // Set allowEditing of grid.
                 * $("#Grid").ejGrid("option", "edit.allowEditing", false);
                 * @memberof edit
                 * @instance
                 */
                allowEditing: false,
                /**		
                 * Enables or disables adding a new record to the grid.
                 * @name edit.allowAdding
                 * @default false
                 * @type {boolean}
                 * @example
                 * Defining allowAdding with the edit option”
                 * $("#Grid").ejGrid({
                 *     edit: { allowAdding: true }
                 * });
                 * // Get or set the allowAdding after grid initialize.
                 *  // Get allowAdding of grid.
                 *  var value = $("#Grid").ejGrid("option", "edit.allowAdding");
                 *  // Set allowAdding of grid.
                 * $("#Grid").ejGrid("option", "edit.allowAdding", false);
                 * @memberof edit
                 * @instance
                 */
                allowAdding: false,
                /**		
                 * Enables or disables the deleting record behavior of the grid.
                 * @name edit.allowDeleting
                 * @default false
                 * @type {boolean}
                 * @example
                 * Defining allowDeleting with the edit option”
                 * $("#Grid").ejGrid({
                 *     edit: { allowDeleting: true }
                 * });
                 * // Get or set the allowDeleting after grid initialize.
                 *  // Get allowDeleting of grid.
                 *  var value = $("#Grid").ejGrid("option", "edit.allowDeleting");
                 *  // Set allowDeleting of grid.
                 * $("#Grid").ejGrid("option", "edit.allowDeleting", false);
                 * @memberof edit
                 * @instance
                 */
                allowDeleting: false,
                /**		
                 * This is used to define mode of editing or adding of grid.    See {@link editMode}
                 * @name edit.editMode
                 * @default normal
                 * @type {enum}
                 * @example
                 * Defining editMode with edit option:
                 * $("#Grid").ejGrid({
                 *       edit: { editMode: ej.Grid.editMode.Normal}
                 * });
                 * //Get or set the editMode after grid initialize:
                 *   // Get editMode of grid.
                 *   var value = $("#Grid").ejGrid("option", "edit.editMode");
                 *   // Set editMode of grid.
                 *   $("#Grid").ejGrid("option", "edit.editMode", ej.Grid.editMode.Dialog);
                 * @memberof edit
                 * @instance
                 */
                editMode: "normal",
                /**		
                 * An element’s id which can be used for dialog edit template rendering.
                 * @default null
                 * @type {string}
                 * @example               
                 * Defining dialogEditorTemplateId with edit option
                 * $("#Grid").ejGrid({
                 *     edit:{
                 *          editMode:ej.Grid.editMode.DialogTemplate,
                 *          dialogEditorTemplateId: "#template"
                 *     }
                 * });
                 * //Get or set the dialogEditorTemplateId after grid initialize.
                 *  // get dialogEditorTemplateId of grid
                 *  var value = $("#Grid").ejGrid("option", "edit.dialogEditorTemplateId");
                 *  // set dialogEditorTemplateId of grid
                 *  $("#Grid").ejGrid("option", "edit.dialogEditorTemplateId", "#modifyTemplate");
                  * @memberof edit
                 * @instance
                 */
                dialogEditorTemplateId: null,
                /**		
                 * This is used to define edit event option for editing .    See {@link editEvent}
                 * @name edit.editEvent
                 * @default ej.Grid.editEvent.DoubleClick
                 * @type {enum}
                 * @example
                 * Defining editEvent with edit option:
                 * $("#Grid").ejGrid({
                 *       edit: { editMode: ej.Grid.editEvent.Single}
                 * });
                 * //Get or set the editEvent after grid initialize:
                 *   // Get editEvent of grid.
                 *   var value = $("#Grid").ejGrid("option", "edit.editEvent");
                 *   // Set editEvent of grid.
                 *   $("#Grid").ejGrid("option", "edit.editEvent", ej.Grid.editEvent.DoubleClick);
                 * @memberof edit
                 * @instance
                 */
                editEvent: "doubleclick",
                /**		
                 * An element’s id which can be used for external edit template rendering.
                 * @default null
                 * @type {string}
                 * @example               
                 * Defining externalFormTemplateId with edit option
                 * $("#Grid").ejGrid({
                 *     edit:{
                 *          editMode:ej.Grid.editMode.ExternalFormTemplate,
                 *          externalFormTemplateId: "#template"
                 *     }
                 * });
                 * //Get or set the externalFormTemplateId after grid initialize.
                 *  // get externalFormTemplateId of grid
                 *  var value = $("#Grid").ejGrid("option", "edit.externalFormTemplateId");
                 *  // set externalFormTemplateId of grid
                 *  $("#Grid").ejGrid("option", "edit.externalFormTemplateId", "#modifyTemplate");
                  * @memberof edit
                 * @instance
                 */
                externalFormTemplateId: null,
                /**		
                 * An element’s id which can be used for inline edit template rendering.
                 * @default null
                 * @type {string}
                 * @example               
                 * Defining inlineFormTemplateId with edit option
                 * $("#Grid").ejGrid({
                 *     edit:{
                 *          editMode:ej.Grid.editMode.InlineFormTemplate,
                 *          inlineFormTemplateId: "#template"
                 *     }
                 * });
                 * //Get or set the inlineFormTemplateId after grid initialize.
                 *  // get inlineFormTemplateId of grid
                 *  var value = $("#Grid").ejGrid("option", "edit.inlineFormTemplateId");
                 *  // set inlineFormTemplateId of grid
                 *  $("#Grid").ejGrid("option", "edit.inlineFormTemplateId", "#modifyTemplate");
                  * @memberof edit
                 * @instance
                 */
                inlineFormTemplateId: null,
                /**		
                 * This is used to position the edit form either in the top-right corner or bottom-left corner of the Grid.    See {@link formPosition}
                 * @name edit.formPosition
                 * @default ej.Grid.formPosition.BottomLeft
                 * @type {enum}
                 * @example
                 * $("#Grid").ejGrid({
                 *       edit:{
                 *           editMode:ej.Grid.editMode.ExternalFormTemplate, 
                 *           externalFormTemplateId:"#template",
                 *           formPosition: ej.Grid.formPosition.BottomLeft
                 *       }
                 * });
                 * //Get or set the formPosition after grid initialize.
                 *  // get formPosition of grid
                 *  var value = $("#Grid").ejGrid("option", "edit.formPosition");
                 *  // set formPosition of grid
                 *  $("#Grid").ejGrid("option","edit.formPosition", ej.Grid.formPosition.TopRight);
                  * @memberof edit
                 * @instance
                 */
                formPosition: "bottomleft"
            },
            /**		
             * This property is used to modify pager default configuration.
             * @default 
             * @type {Object}
             * @example
             * pageSettings at grid initialize.
             * $("#Grid").ejGrid(
             *     pageSettings: { pageCount: 5 }
             * );
              * @memberof ejGrid
             * @instance
             */
            pageSettings: {
                /**		
                 * This is used to define the number of records displayed per page.
                 * @name pageSettings.pageSize
                 * @default 12
                 * @type {number}
                 * @example
                 * Defining pageSize with paging option:
                 * $("#Grid").ejGrid({
                 *     pageSettings: { pageSize: 10}
                 * });
                 * //Get or set the pageSize after grid initialize.
                 *  // Get pageSize of grid.
                 *  var value = $("#Grid").ejGrid("option", "pageSettings.pageSize");
                 *  // Set pageSize of grid.
                 * $("#Grid").ejGrid("option", "pageSettings.pageSize", 22);
                  * @memberof pageSettings
                 * @instance
                 */
                pageSize: 12,
                /**		
                 * This is used to define the number of pages displayed in the view.
                 * @name pageSettings.pageCount
                 * @default 8
                 * @type {number}
                 * @example
                 * Defining pageCount with paging option:
                 * $("#Grid").ejGrid({
                 *     pageSettings: { pageCount: 10}
                 * });
                 * //Get or set the pageCount after grid initialize.
                 *  // Get pageCount of grid.
                 *  var value = $("#Grid").ejGrid("option", "pageSettings.pageCount");
                 *  // Set pageCount of grid.
                 * $("#Grid").ejGrid("option", "pageSettings.pageCount", 22);
                  * @memberof pageSettings
                 * @instance
                 */
                pageCount: 8,
                /**		
                 * This is used to define which page to display currently.
                 * @name pageSettings.currentPage
                 * @default 1
                 * @type {number}
                 * @example
                 * Defining currentPage with paging option:
                 * $("#Grid").ejGrid({
                 *     pageSettings: { currentPage: 10}
                 * });
                 * //Get or set the currentPage after grid initialize.
                 *  // Get currentPage of grid.
                 *  var value = $("#Grid").ejGrid("option", "pageSettings.currentPage");
                 *  // Set currentPage of grid.
                 * $("#Grid").ejGrid("option", "pageSettings.currentPage", 22);
                  * @memberof pageSettings
                 * @instance
                 */
                currentPage: 1,
                /**		
                 * totalPages counts internally based on count of number of pagesize and totalrecords of grid. You do not need to set totalPages.
                 * @name pageSettings.totalPages
                 * @default null
                 * @type {number}
                 * @example
                 * Get  totalPages after grid initialize:
                 *  // Get totalPages of grid.
                 *  var value = $("#Grid").ejGrid("option", "pageSettings.totalPages");
                  * @memberof pageSettings
                 * @instance
                 */
                totalPages: null,
                /**		
                 * totalRecordsCount counts internally based on dataSource. Don’t need to set totalRecordsCount for pager.
                 * @name pageSettings.totalRecordsCount
                 * @default null
                 * @type {number}
                 * @example
                 * Get  totalRecordsCount after grid initialize:
                 *  // Get totalRecordsCount of grid.
                 *  var value = $("#Grid").ejGrid("option", "pageSettings.totalRecordsCount");
                  * @memberof pageSettings
                 * @instance
                 */
                totalRecordsCount: null,
                /**		
                 * Enables or disables the query string value passed along with the url while navigating to other page.
                 * @name pageSettings.enableQueryString
                 * @default false
                 * @type {boolean}
                 * @example
                 * Defining enableQueryString with paging option:
                 * $("#Grid").ejGrid({
                 *     pageSettings: { enableQueryString: true}
                 * });
                 * //Get or set the enableQueryString after grid initialize.
                 *  // Get enableQueryString of grid.
                 *  var value = $("#Grid").ejGrid("option", "pageSettings.enableQueryString");
                 *  // Set enableQueryString of grid.
                 * $("#Grid").ejGrid("option", "pageSettings.enableQueryString", false);
                  * @memberof pageSettings
                 * @instance
                 */
                enableQueryString: false
            },
            /**		
             * This property is used to customize the grouping behavior of the grid. 
             * @default 
             * @type {Object}
             * @example
             * pageSettings at grid initialize.
             * $("#Grid").ejGrid({
             *     groupSettings: {
             *         groupedColumns: ["ShipCountry"]
             *   } 
             * });
              * @memberof ejGrid
             * @instance
             */
            groupSettings: {
                /**		
                 * Enable the toggle grouping option in the grid by setting the property as true. 
                 * @name groupSettings.toggleGrouping
                 * @default false
                 * @type {boolean}
                 * @example
                 * Initialize the grouping with the toggleGrouping option specified:
                 * $("#Grid").ejGrid({groupSettings: { toggleGrouping: true }});
                 * Get or set the toggleGrouping option for grouping, after initialization:
                 *  // Get:
                 *  var toggleGroup = $("#Grid").ejGrid("option", "groupSettings.toggleGrouping");
                 *  // Set:
                 *  $("#Grid").ejGrid("option", {groupSettings: { toggleGrouping: true }});
                  * @memberof groupSettings
                 * @instance
                 */
                toggleGrouping: false,
                /**		
                 * Disable the grouped column show option in the grid by setting the property as false.
                 * @name groupSettings.groupedColumnShow
                 * @default false
                 * @type {boolean}
                 * @example
                 * Initialize the grouping with the groupedColumnShow option specified:
                 *  $("#Grid").ejGrid({groupSettings: { groupedColumnShow: false }});
                 * Get or set the groupedColumnShow option for grouping, after initialization:
                 *  // Get:
                 *  var showGroupColumn = $("#Grid").ejGrid("option", "groupSettings.groupedColumnShow");
                 *  // Set:
                 *  $("#Grid").ejGrid("option", {groupSettings: { groupedColumnShow: false }});
                  * @memberof groupSettings
                 * @instance
                 */
                groupedColumnShow: true,
                /**		
                 * Disable the show ungroup button option in the grid by setting the property as false.
                 * @name groupSettings.showUngroupButton
                 * @default false
                 * @type {boolean}
                 *@example
                 * Initialize the grouping with the showUngroupButton option specified:
                 * $("#Grid").ejGrid({groupSettings: { showUngroupButton: false }});
                 * Get or set the showUngroupButton option for grouping, after initialization:
                 *  // Get:
                 *  var unGroupButton = $("#Grid").ejGrid("option", "groupSettings.showUngroupButton");
                 *  // Set:
                 *  $("#Grid").ejGrid("option", {groupSettings: { showUngroupButton: false }});
                  * @memberof groupSettings
                 * @instance
                 */
                showUngroupButton: true,
                /**		
                 * Disable the show animation button option in the grid by setting the property as false.
                 * @name groupSettings.showAnimateButton
                 * @default false
                 * @type {boolean}
                 * @example
                 * Initialize the grouping with the showAnimateButton option specified:
                 * $("#Grid").ejGrid({groupSettings: { showAnimateButton: false }});
                 *  Get or set the showAnimateButton option for grouping, after initialization:
                 *  // Get:
                 *  var unGroupButton = $("#Grid").ejGrid("option", "groupSettings.showAnimateButton");
                 *  // Set:
                 *  $("#Grid").ejGrid("option", {groupSettings: { showAnimateButton: false }});
                  * @memberof groupSettings
                 * @instance
                 */
                showAnimateButton: true,
                /**		
                 * Add grouped columns programmatically at initial load.
                 * @name groupSettings.groupedColumns
                 * @default []
                 * @type {object}
                 * @example
                 * Initialize the grouping with the groupedColumns option specified:
                 * $("#Grid").ejGrid({groupSettings: { groupedColumns: ["ShipCountry"] }});
                 * Get or set the groupedColumns option for grouping, after initialization:
                 * // Get:
                 * var groupcolumn = $("#Grid").ejGrid("option", "groupSettings.groupedColumns");
                 * // Set:
                 *$("#Grid").ejGrid("option", { groupSettings: { groupedColumns: ["ShipCountry"] }});
                  * @memberof groupSettings
                 * @instance
                 */
                groupedColumns: []
            },
            /**		
             * This property is used to customize the filtering behavior of the grid.
             * @default 
             * @type {Object}
             * @example
             * $("#Grid").ejGrid({
             *     filterSettings: {showFilterBarMessage: true}                             
             * });
              * @memberof ejGrid
             * @instance
             */
            filterSettings: {
                /**		
                 * Enable the filter menu option in the grid by setting the property as menu. Accepted filterTypes are "filterbar" and "menu".      See{@link filterType}
                 * @name filterSettings.filterType
                 * @default ej.Grid.filterType.FilterBar
                 * @type {enum}
                 * @example
                 * Initialize the filtering with the filterType option specified:
                 * $("#Grid").ejGrid({filterSettings: { filterType: ej.Grid.filterType.Menu }});
                 * Get or set the filterType option for filtering, after initialization:
                 * // Get:
                 * var filterMenu = $("#Grid").ejGrid("option", "filterSettings.filterType");
                 * // Set:
                 * $("#Grid").ejGrid("option", { filterSettings: { filterType: ej.Grid.filterType.FilterBar }});
                  * @memberof filterSettings
                 * @instance
                 */
                filterType: "filterbar",
                /**		
                 * Set the filter bar mode option in the grid. Accepted filterBarModes are "immediate" and "onenter".      See{@link filterBarMode}
                 * @name filterSettings.filterBarMode
                 * @default ej.Grid.filterBarMode.Immediate
                 * @type {enum}
                 * @example
                 * Initialize the filtering with the filterBarMode option specified:
                 * $("#Grid").ejGrid({filterSettings: {filterBarMode: ej.Grid.filterBarMode.OnEnter}});
                 * Get or set the filterType option for filtering, after initialization:
                 * // Get:
                 * var filterMode = $("#Grid").ejGrid("option", "filterSettings.filterBarMode");
                 * // Set:
                 * $("#Grid").ejGrid("option", { filterSettings: {filterBarMode: ej.Grid.filterBarMode.Immediate}});
                  * @memberof filterSettings
                 * @instance
                 */
                filterBarMode: "immediate",
                /**		
                 * Disable the filter bar message in the grid by setting the property as false.     
                 * @name filterSettings.showFilterBarMessage
                 * @default true
                 * @type {boolean}
                 * @example
                 * Initialize the filtering with the showFilterBarMessage option specified:
                 * $("#Grid").ejGrid({filterSettings: { showFilterBarMessage: false }});
                 * Get or set the showFilterBarMessage option for filtering, after initialization:
                 * // Get:
                 * var msgBar = $("#Grid").ejGrid("option", "filterSettings.showFilterBarMessage");
                 * // Set:
                 * $("#Grid").ejGrid("option", { filterSettings: { showFilterBarMessage: true }});
                  * @memberof filterSettings
                 * @instance
                 */
                showFilterBarMessage: true,
                /**		
                * Specify the filter message bar width in the grid.     
                * @name filterSettings.statusBarWidth
                * @default 450
                * @type {number}
                * @example
                * Initialize the filtering with the showFilterBarMessage option specified:
                * $("#Grid").ejGrid({filterSettings: { statusBarWidth: 550 }});
                * Get or set the  statusBarWidth option for filtering, after initialization:
                * // Get:
                * var msgBarWidth = $("#Grid").ejGrid("option", "filterSettings.statusBarWidth");
                * // Set:
                * $("#Grid").ejGrid("option", { filterSettings: { statusBarWidth: 750 }});
                 * @memberof filterSettings
                * @instance
                */
                statusBarWidth: 450,
                /**		
                 * Enable the show predicate in the filtering menu by setting the property as true.   
                 * @name filterSettings.showPredicate
                 * @default false
                 * @type {boolean}
                 * @example
                 * Initialize the filtering with the showPredicate option specified:
                 * $("#Grid").ejGrid({filterSettings: { showPredicate: true }});
                 * Get or set the  showPredicate option for filtering, after initialization:
                 * // Get:
                 * var predicate = $("#Grid").ejGrid("option", "filterSettings.showPredicate");
                 * // Set:
                 * $("#Grid").ejGrid("option", { filterSettings: { showPredicate: false }});
                  * @memberof filterSettings
                 * @instance
                 */
                showPredicate: false,
                /**		
                 * Get filtered columns details programmatically after filtering.   
                 * @name filterSettings.filteredColumns
                 * @default []
                 * @type {object}
                 * @example                 
                 * Get the filteredColumns option for filtering, after initialization:
                 * // Get:
                 * var filtercolumn = $("#Grid").ejGrid("option", "filterSettings.filteredColumns");                
                  * @memberof filterSettings
                 * @instance
                 */
                filteredColumns: []
            },
            /**		
            * This property is used to define initial sorting for particular columns in the grid.
            * @default 
            * @type {Object}
            * @example
            * $("#Grid").ejGrid({
            *     sortSettings: {
            *           sortedColumns: [{ field: "OrderID", direction: ej.sortOrder.Descending }]
            *     }                    
            * });
             * @memberof ejGrid
            * @instance
            */
            sortSettings: {
                /**		
                * Get sorted Columns details programmatically after sorting.   
                * @name sortSettings.sortedColumns
                * @default []
                * @type {object}
                * @example                 
                * Get the sortedColumns option for sorting, after initialization:
                * // Get:
                * var sortcolumn = $("#Grid").ejGrid("option", "sortSettings.sortedColumns");                
                 * @memberof sortSettings
                * @instance
                */
                sortedColumns: []
            },
            /**		
             * This property is used to enable the toolbar and add toolbar items.
             * @default 
             * @type {Object}
             * @example
             * toolbar at grid initialize:
             * $("#Grid").ejGrid({
             *      toolBar: {
             *         allowToolBar: true,
             *         toolBarItems: [ej.Grid.toolBarItems.Add, ej.Grid.toolBarItems.Edit]
             *      }                                       
             * });
              * @memberof ejGrid
             * @instance
             */
            toolBar: {
                /**		
                 * Enable toolbar for editing action by setting the property as true.
                 * @name toolBar.allowToolBar
                 * @default false
                 * @type {boolean}
                 * @example
                 * Initialize the grid with the allowToolBar option specified:
                 * $("#Grid").ejGrid({toolBar: { allowToolBar: true}});
                 * Get or set the allowToolBar option for the grid, after initialization:
                 * // Get:
                 * var toolBarValue = $("#Grid").ejGrid("option", "toolBar.allowToolBar");
                 * // Set:
                 * $("#Grid").ejGrid("option", { toolBar: { allowToolBar: false }});
                  * @memberof toolBar
                 * @instance
                 */
                allowToolBar: false,
                /**		
                 * Enable the toolbar with default editing actions. Acceptable toolbarItems are "add”, "edit", "delete", "update", "cancel" and "search".   See{@link toolBarItems}
                 * @name toolBar.toolBarItems
                 * @default []
                 * @type {enum}
                 * @example
                 * Initialize the grid toolbar with the toolBarItems option specified:
                 * $("#Grid").ejGrid({toolBar: { toolBarItems: ej.Grid.toolBarItems.Add }});
                 * Get or set the toolBarItems option for the grid toolbar, after initialization:
                 * // Get:
                 * var items = $("#Grid").ejGrid("option", "toolBar.toolBarItems");
                 * // Set:
                 * $("#Grid").ejGrid("option", { toolBar: { toolBarItems: ej.Grid.toolBarItems.Edit } });
                  * @memberof toolBar
                 * @instance
                 */
                toolBarItems: [],
                /**		
                 * Enable custom toolbar for the grid.
                 * @name toolBar.customToolbarItems
                 * @default []
                 * @type {object}
                 * @example
                 * Initialize the custom grid toolbar with the customToolbarItems  option specified:
                 * $("#Grid").ejGrid({ toolBar: { customToolbarItems: ["Expand"] });
                 * Get or set the customToolbarItems  option for the grid  custom toolbar, after initialization:
                 * // Get:
                 * var customValue = $("#Grid").ejGrid("option", "toolBar.customToolbarItems");
                 * // Set:
                 * $("#Grid").ejGrid("option", { toolBar: { customToolbarItems  : ["Expand"]} });
                  * @memberof toolBar
                 * @instance
                 */
                customToolbarItems: []
            },
            /**		
             * Scrolling is used to adjust the width and height of the scrollbar.
             * @default 
             * @type {Object}
             * @example
             * scrolling at grid initialize.
             * $("#Grid").ejGrid({
             *     scrolling: {
             *         height: 300,
             *         width: 800
             *     }
             * });
              * @memberof ejGrid
             * @instance
             */
            scrolling:
			{
			    /**		
                 * Specify the width for the control and the grid automatically displays horizontal scroll bars if needed.   
                 * @name scrolling.width
                 * @default 250
                 * @type {number}
                 * @example
                 * Initialize the grid scrollbar with the width option specified:
                 * $("#Grid").ejGrid({scrolling: { width: 900 }});
                 * Get or set the width option for the scrollbar, after initialization:
                 * // Get:
                 * var width = $("#Grid").ejGrid("option", "scrolling.width");
                 * // Set:
                 * $("#Grid").ejGrid("option", { scrolling: { width: 500 } });
                  * @memberof scrolling
                 * @instance
                 */
			    width: 250,
			    /**		
                 * Specify the height for the control, and the grid automatically displays vertical scroll bars if needed. 
                 * @name scrolling.height
                 * @default 0
                 * @type {number}
                 * @example
                 * Initialize the grid scrollbar with the height option specified:
                 * $("#Grid").ejGrid({scrolling: { height: 500 }});
                 * Get or set the height option for the scrollbar, after initialization:
                 * // Get:
                 * var height = $("#Grid").ejGrid("option", "scrolling.height");
                 * // Set:
                 * $("#Grid").ejGrid("option", { scrolling: { height: 300 } });
                  * @memberof scrolling
                 * @instance
                 */
			    height: 0,
			    /**		
                 * Enable or disable the virtual scrolling behavior of grid.
                 * @name scrolling.allowVirtualScrolling
                 * @default false
                 * @type {boolean}
                 * @example
                 * Initialize the grid with the allowVirtualScrolling option specified:
                 * $("#Grid").ejGrid({
                 *     allowScrolling: true,               
                 *     scrolling: {
                 *        width: 950, height: 300,
                 *        allowVirtualScrolling: true
                 *     }
                 * });
                 * Get or set the allowVirtualScrolling option for scrollbar, after initialization:
                 * // getter
                 * var scrollbar = $("#Grid").ejGrid("option", "scrolling.allowVirtualScrolling");
                 * // setter
                 * $("#Grid").ejGrid("option", { scrolling: { allowVirtualScrolling: false } });
                  * @memberof scrolling
                 * @instance
                 */
			    allowVirtualScrolling: false,
			    /**		
                 * You can use this property to freeze particular rows in grid at the time of scrolling
                 * @name scrolling.frozenRows
                 * @default 0
                 * @type {number}
                 * @example
                 * Initialize the grid with the frozenRows option specified:
                 * $("#Grid").ejGrid({
                 *     allowScrolling: true,               
                 *     scrolling:{frozenRows:2 }
                 * });
                 * Get or set the frozenRows option for scrollbar, after initialization:
                 * // getter
                 * var scrollbar = $("#Grid").ejGrid("option", "scrolling.frozenRows");
                 * // setter
                 * $("#Grid").ejGrid("option", { scrolling: { frozenRows: 5 } });
                  * @memberof scrolling
                 * @instance
                 */
			    frozenRows: 0,
			    /**		
                * You can use this property to freeze particular columns in grid at the time of scrolling.
                * @name scrolling.frozenColumns
                * @default 0
                * @type {number}
                * @example
                * Initialize the grid with the frozenColumns option specified:
                * $("#Grid").ejGrid({
                *     allowScrolling: true,               
                *     scrolling:{frozenColumns:2 }
                * });
                * Get or set the frozenColumns option for scrollbar, after initialization:
                * // getter
                * var scrollbar = $("#Grid").ejGrid("option", "scrolling.frozenColumns");
                * // setter
                * $("#Grid").ejGrid("option", { scrolling: { frozenColumns: 5 } });
                 * @memberof scrolling
                * @instance
                */
			    frozenColumns: 0
			},
            /**		
             * The summaryRows property is used to define summary processing of a particular column.
             * @default 
             * @type {array}
             * @example
             * $("#Grid").ejGrid({
             *       columns: [
             *         { field: "Freight" }
             *       ] 
             *       summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.summaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight",
             *            format: "{0:C2}"
             *         }]
             *     }
             *    ],                                     
             * });
              * @memberof ejGrid
             * @instance
             */
            summaryRows: [],
            /**		
             * Align content in the grid control from right to left by setting the property as true.
             * @default false
             * @type {boolean}
             * @example
             * Initialize the grid with the rtl option specified:
             * $("#Grid").ejGrid({rtl: true });
             * Get or set the rtl option, after initialization:
             * // Get:
             * var rtlValue = $("#Grid").ejGrid("option", "rtl");
             * // Set:
             * $("#Grid").ejGrid("option", { rtl: true });
                 * @memberof ejGrid
             * @instance
             */
            rtl: false,
            /**		
             * Enables or disables altRow behavior in the grid.
             * @default true
             * @type {boolean}
             * @example
             * altRow at grid initialize:
             * $("#Grid").ejGrid({altRow: true });
             * Get or set the altRow after grid initialize:
             * // Get altRow of grid.
             * var value = $("#Grid").ejGrid("option", "altRow");
             * // Set altRow of grid.
             * $("#Grid").ejGrid("option", { altRow: true });
                 * @memberof ejGrid
             * @instance
             */
            altRow: true,
            /**		
            * Get the current page data of the grid
            * @default null
            * @type {array}
            * @example           
            * Get the currentViewData after grid initialize:
            * // Get altRow of grid.
            * var value = $("#Grid").ejGrid("option", "currentViewData");            
                * @memberof ejGrid
            * @instance
            */
            currentViewData: null,
            /**		
             * Render the detail row for the corresponding expanded master row. 
             * @default null
             * @type {string}
             * @example
             * Initialize the grid with the detailTemplate option specified:
             * $("#Grid").ejGrid({detailTemplate: "#tabGridContents" });           
                 * @memberof ejGrid
             * @instance
             */
            detailTemplate: null,
            /**		
             * Render the template rows in the grid. The template row must be a table row. That table row must have the JavaScript render binding format ({{:columnName}}) then the grid data source binds the data to the corresponding table row of the template.
             * @default null
             * @type {string}
             * @example
             * Initialize the grid with the rowTemplate option specified:
             * $("#Grid").ejGrid({rowTemplate: "#templateData" });
                 * @memberof ejGrid
             * @instance
             */
            rowTemplate: null,
            /**		
             * Triggered detail template row is initialized.
             * @event
			 * @name ejGrid#detailData 	
			 * @param {Object} argument Event parameters from grid              
			 * @param {boolean}  argument.cancel Returns the cancel option value.
			 * @param {object}  argument.templateDetail Returns detail row element.
			 * @param {object}  argument.model Returns the grid model.
			 * @param {string}  argument.type Returns the name of the event.
			 * @example 
			 * //rowDataBound event for grid
             * $("#Grid").ejGrid({
             *    detailData: function (args){}
             * });
               * @memberof ejGrid
             * @instance
             */
            detailData: null,
            rowTemplate: null,
            /**    
			 * Triggered every time a request is made to access row information, element and data.
			 * @event
			 * @name ejGrid#rowDataBound 	
			 * @param {Object} argument Event parameters from grid              
             * @param {object}  argument.Element Returns grid row.
			 * @param {boolean}  argument.cancel Returns the cancel option value.
			 * @param {object}  argument.data Returns current row record object (JSON).
			 * @param {object}  argument.model Returns the grid model.
			 * @param {string}  argument.type Returns the name of the event.
			 * @example 
			 * //rowDataBound event for grid
             * $("#Grid").ejGrid({
             *    rowDataBound: function (args){}
             * });
			  * @memberof ejGrid
			 * @instance
			 */
            rowDataBound: null,
            /**    
			 * Triggered every time a request is made to access particular cell information, element and data.
			 * @event
			 * @name ejGrid#queryCellInfo 	
			 * @param {Object} argument Event parameters from grid              
             * @param {object}  argument.Element Returns grid row.
			 * @param {boolean}  argument.cancel Returns the cancel option value.
			 * @param {object}  argument.data Returns current row record object (JSON).
			 * @param {object}  argument.model Returns the grid model.
			 * @param {string}  argument.type Returns the name of the event.
			 * @example 
			 * //queryCellInfo event for grid
             * $("#Grid").ejGrid({
             *    queryCellInfo: function (args){}
             * });
			  * @memberof ejGrid
			 * @instance
			 */
            queryCellInfo: null,
            /**    
			 * Triggered when the grid is rendered completely.
			 * @event
			 * @name ejGrid#create 	
			 * @param {Object} argument Event parameters from grid             
			 * @param {boolean}  argument.cancel Returns the cancel option value.
			 * @param {object}  argument.model Returns the grid model.
			 * @param {string}  argument.type Returns the name of the event.
			 * @example 
			 * //create event for grid
             * $("#Grid").ejGrid({
             *    create: function (args){}
             * });
			  * @memberof ejGrid
			 * @instance
			 */
            create: null,
            /**    
              * Triggered for every grid action before its starts.
              * @event
              * @name ejGrid#actionBegin              
              * @param {Object} argument Event parameters when grid is initialized:          
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.type Returns the name of the event.
              
              * @param {Object} argument Event parameters when grid paging action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {number} argument.currentPage  Returns the current selected page number.
              * @param {number} argument.endIndex Returns the end row index of that current page.
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.requestType Returns request type.
              * @param {number} argument.startIndex  Returns the start row index of that current page.
              * @param {string}  argument.type Returns the name of the event.
 
              * @param {Object} argument Event parameters when grid sorting action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {string}  argument.columnName Returns the current grouped column field name.
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.requestType Returns request type.
              * @param {string}  argument.type Returns the name of the event. 
              * @param {string}  argument.columnSortDirection  Returns the column sort direction.
 
              * @param {Object} argument Event parameters when grid grouping action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {string}  argument.columnName Returns the current grouped column field name.
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.requestType Returns request type.
              * @param {string}  argument.type Returns the name of the event.        
 
              * @param {Object} argument Event parameters when grid record editing action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {object}  argument.currentTr  Returns current edited row.
              * @param {object}  argument.model Returns the grid model.
              * @param {string} argument.originalEventType Returns the current action event type.
              * @param {string} argument.primaryKey Returns primary key.
              * @param {string} argument.primaryKeyValue Returns primary key value.
              * @param {string}  argument.requestType Returns request type.
              * @param {number}  argument.rowIndex Returns the edited row index.
              * @param {string}  argument.type Returns the name of the event.
 
              * @param {Object} argument Event parameters when grid record save action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {object}  argument.data Returns the record object (JSON).
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.requestType Returns request type.
              * @param {string}  argument.type Returns the name of the event. 
 
              * @param {Object} argument Event parameters when grid record cancel action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.requestType Returns request type.
              * @param {string}  argument.type Returns the name of the event. 
 
              * @param {Object} argument Event parameters when grid record delete action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {object}  argument.data Returns the record object (JSON).
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.requestType Returns request type.
              * @param {object}  argument.tr Returns selected row for delete.
              * @param {string}  argument.type Returns the name of the event.
 
              * @param {Object} argument Event parameters when add new record action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {object}  argument.data Returns the record object (JSON).
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.requestType Returns request type.
              * @param {string}  argument.type Returns the name of the event.
 
              * @param {Object} argument Event parameters when grid filtering action starts:
              * @param {boolean}  argument.cancel Returns the cancel option value.
              * @param {object}  argument.currentFilteringColumn Returns current filtering column field name.
              * @param {object}  argument.filterCollection Returns filter details.
              * @param {object}  argument.model Returns the grid model.
              * @param {string}  argument.requestType Returns request type.
              * @param {string}  argument.type Returns the name of the event.
 
              * @example 
              * //create event for grid
              * $("#Grid").ejGrid({
              *    actionBegin: function (args){}
              * });
               * @memberof ejGrid
              * @instance
              */
            actionBegin: null,
            /**    
            * Triggered for every grid action success event.
            * @event
            * @name ejGrid#actionComplete 	
            * @param {Object} argument Arguments in actionComplete when grid is initialized.     
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.requestType Returns request type.
            * @param {string} argument.type Returns the name of the event.
            * @param {Object} argument Arguments in actionComplete after grid paging action is completed.
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {number} argument.currentPage Returns the current selected page number.
            * @param {number} argument.endIndex Returns the end row index of that current page.
            * @param {object} argument.model Returns the grid model.				
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.requestType Returns request type.
            * @param {number} argument.startIndex Returns the start row index of the current page.
            * @param {object} argument.target Returns grid element.
            * @param {string} argument.type Returns the name of the event.
            * @param {object} argument Arguments in actionComplete after grid sorting action is completed.				
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {string} argument.columnName Returns the current sorted column field name.
            * @param {string} argument.columnSortDirection Returns the column sort direction.
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.requestType Returns request type.
            * @param {object} argument.target Returns grid element.
            * @param {string} argument.type Returns the name of the event.
            * @param {object} argument Arguments in actionComplete after grid grouping action is completed.
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {string} argument.columnName Returns the current grouped column field name.
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.requestType Returns request type.
            * @param {object} argument.target Returns grid element.
            * @param {string} argument.type Returns the name of the event.
            * @param {object} argument Arguments in actionComplete after grid record editing action is completed.
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.currentTr Returns current edited row.
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.primaryKey Returns primary key.
            * @param {string} argument.primaryKeyValue Returns primary key value.
            * @param {string} argument.requestType Returns request type.
            * @param {number} argument.rowIndex Returns the edited row index.
            * @param {object} argument.target Returns grid element.
            * @param {string} argument.type Returns the name of the event.
            * @param {object} argument Arguments in actionComplete after grid record save action is completed.
            * @param {object} argument.cancel Returns the cancel option value.
            * @param {object} argument.data Returns the record object (JSON).
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.requestType Returns request type.
            * @param {object} argument.target Returns grid element.
            * @param {string} argument.type Returns the name of the event.
            * @param {object} argument Arguments in actionComplete after grid record cancel action is completed.
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.requestType Returns request type.
            * @param {object} argument.target Returns grid element.
            * @param {string} argument.type Returns the name of the event.
            * @param {object} argument Arguments in actionComplete after grid record delete action is completed.
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.data Returns the record object (JSON).
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.requestType Returns request type.
            * @param {object} argument.target Returns grid element.
            * @param {object} argument.tr Returns selected row for delete.
            * @param {string} argument.type Returns the name of the event.
            * @param {object} argument Arguments in actionComplete after add new record action is completed.
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.data Returns empty record object (JSON).
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.requestType Returns request type.
            * @param {object} argument.target Returns grid element.
            * @param {string} argument.type Returns the name of the event.
            * @param {object} argument Arguments in actionComplete after grid filtering action is completed.
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {string} argument.currentFilteringColumn Returns current filtering column field name.
            * @param {object} argument.filterCollection Returns filter details.
            * @param {object} argument.model Returns the grid model.
            * @param {string} argument.originalEventType Returns current action event type.
            * @param {string} argument.requestType Returns request type.
            * @param {object} argument.target Returns grid element.
            * @param {string} argument.type Returns the name of the event.
            * @example 
            * //actionComplete event for Grid
            * $("#Grid").ejGrid({
            *    actionComplete: function (args) {}
            * });
            * @example 
            * //Bind actionComplete event using jquery "on"
            * $("#Grid").on("ejGridactionComplete", function(e){}); 
            * @memberof ejGrid
            * @instance
        */
            actionComplete: null,
            /**    
				* Triggered before the record is going to be edited.
				* @event
				* @name ejGrid#beginEdit 	
				* @param {Object} argument Arguments when beginEdit event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.currentTr Returns the current edited row.
				* @param {object} argument.model Returns the grid model.
				* @param {string} argument.primaryKey Returns primary key.
				* @param {string} argument.primaryKeyValue Returns primary key value.
				* @param {number} argument.rowIndex Returns the edited row index.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //beginEdit event for Grid
				* $("#Grid").ejGrid({
				*    beginEdit: function (args) {}
				* });
				* @example 
				* //Bind beginEdit event using jquery "on"
				* $("#Grid").on("ejGridbeginEdit", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            beginEdit: null,
            /**    
				* Triggered before the row is going to be selected.
				* @event
				* @name ejGrid#rowSelecting 	
				* @param {Object} argument Arguments when rowSelecting event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.model Returns the grid model.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //rowSelecting event for Grid
				* $("#Grid").ejGrid({
				*    rowSelecting: function (args) {}
				* });
				* @example 
				* //Bind rowSelecting event using jquery "on"
				* $("#Grid").on("ejGridrowSelecting", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            rowSelecting: null,
            /**    
				* Triggered after the row is selected.
				* @event
				* @name ejGrid#rowSelected 	
				* @param {Object} argument Arguments when rowSelected event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.currentData Returns current record object (JSON).
				* @param {number} argument.currentRowIndex Returns the row index of the selected row.
				* @param {object} argument.currentRow Returns the current selected row.
				* @param {object} argument.model Returns the grid model.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //rowSelected event for Grid
				* $("#Grid").ejGrid({
				*    rowSelected: function (args) {}
				* });
				* @example 
				* //Bind rowSelected event using jquery "on"
				* $("#Grid").on("ejGridrowSelected", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            rowSelected: null,
            /**    
				* Triggered when column dragging begins.
				* @event
				* @name ejGrid#dragStart 	
				* @param {Object} argument Arguments when dragStart event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.draggableType Returns draggable element type.
				* @param {object} argument.model Returns the grid model.
				* @param {object} argument.target Returns drag start element.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //dragStart event for Grid
				* $("#Grid").ejGrid({
				*    dragStart: function (args) {}
				* });
				* @example 
				* //Bind dragStart event using jquery "on"
				* $("#Grid").on("ejGriddragStart", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            dragStart: null,
            /**    
				* Triggered when the column is being dragged.
				* @event
				* @name ejGrid#drag 	
				* @param {Object} argument Arguments when drag event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.draggableType Returns draggable element type.
				* @param {object} argument.model Returns the grid model.
				* @param {object} argument.target Returns target elements based on mouse move position.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //drag event for Grid
				* $("#Grid").ejGrid({
				*    drag: function (args) {}
				* });
				* @example 
				* //Bind drag event using jquery "on"
				* $("#Grid").on("ejGriddrag", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            drag: null,
            /**    
				* Triggered when the column is dropped.
				* @event
				* @name ejGrid#dragStop 	
				* @param {Object} argument Arguments when dragStop event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.draggableType Returns draggable element type.
				* @param {object} argument.model Returns the grid model.
				* @param {object} argument.target Returns dropped dragged element.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //dragStop event for Grid
				* $("#Grid").ejGrid({
				*    dragStop: function (args) {}
				* });
				* @example 
				* //Bind dragStop event using jquery "on"
				* $("#Grid").on("ejGriddragStop", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            dragStop: null,
            /**    
				* Triggered when record is clicked.
				* @event
				* @name ejGrid#recordClick 	
				* @param {Object} argument Arguments when recordClick event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.currentData Returns current record object (JSON).
				* @param {number} argument.currentRowIndex Returns the row index of the selected row.
				* @param {object} argument.currrentRow Returns the current selected row.
				* @param {object} argument.model Returns the grid model.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //recordClick event for Grid
				* $("#Grid").ejGrid({
				*    recordClick: function (args) {}
				* });
				* @example 
				* //Bind recordClick event using jquery "on"
				* $("#Grid").on("ejGridrecordClick", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            recordClick: null,
            /**    
				* Triggered when record is double clicked.
				* @event
				* @name ejGrid#recordDoubleClick 	
				* @param {Object} argument Arguments when recordDoubleClick event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.currentData Returns current record object (JSON).
				* @param {number} argument.currentRowIndex Returns the row index of the selected row.
				* @param {object} argument.currrentRow Returns the current selected row.
				* @param {object} argument.model Returns the grid model.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //recordDoubleClick event for Grid
				* $("#Grid").ejGrid({
				*    recordDoubleClick: function (args) {}
				* });
				* @example 
				* //Bind recordDoubleClick event using jquery "on"
				* $("#Grid").on("ejGridrecordDoubleClick", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            recordDoubleClick: null,
            /**    
				* Triggered when right clicked on grid element.
				* @event
				* @name ejGrid#gridRightClick 	
				* @param {Object} argument Arguments when gridRightClick event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.currentData Returns current record object (JSON).
				* @param {number} argument.currentRowIndex Returns the row index of the selected row.
				* @param {object} argument.currrentRow Returns the current selected row.
				* @param {object} argument.model Returns the grid model.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //gridRightClick event for Grid
				* $("#Grid").ejGrid({
				*    gridRightClick: function (args) {}
				* });
				* @example 
				* //Bind gridRightClick event using jquery "on"
				* $("#Grid").on("ejGridgridRightClick", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            gridRightClick: null,
            /**    
				* Triggered when detail template row is clicked to collapse.
				* @event
				* @name ejGrid#detailCollapse 	
				* @param {Object} argument Arguments when detailCollapse event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.detailRow Returns detail row element.
				* @param {object} argument.masterData Returns master row of detail row record object (JSON).
				* @param {object} argument.masterRow Returns master row element.
				* @param {object} argument.model Returns the grid model.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //detailCollapse event for Grid
				* $("#Grid").ejGrid({
				*    detailCollapse: function (args) {}
				* });
				* @example 
				* //Bind detailCollapse event using jquery "on"
				* $("#Grid").on("ejGriddetailCollapse", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            detailCollapse: null,
            /**    
				* Triggered when detail template row is clicked to expand.
				* @event
				* @name ejGrid#detailExpand 	
				* @param {Object} argument Arguments when detailExpand event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.detailRow Returns detail row element.
				* @param {object} argument.masterData Returns master row of detail row record object (JSON).
				* @param {object} argument.masterRow Returns master row element.
				* @param {object} argument.model Returns the grid model.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //detailExpand event for Grid
				* $("#Grid").ejGrid({
				*    detailExpand: function (args) {}
				* });
				* @example 
				* //Bind detailExpand event using jquery "on"
				* $("#Grid").on("ejGriddetailExpand", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            detailExpand: null,
            /**    
				* Triggered when toolbar item is clicked in grid.
				* @event
				* @name ejGrid#toolBarClick 	
				* @param {Object} argument Arguments when toolBarClick event is triggered.  
				* @param {boolean} argument.cancel Returns the cancel option value.
				* @param {object} argument.currentTarget Returns the current item.
				* @param {object} argument.model Returns the grid model.
				* @param {boolean} argument.status Returns the status of toolbar item which denotes its enabled state
				* @param {object} argument.target Returns the target item.
				* @param {string} argument.type Returns the name of the event.
				* @example 
				* //toolBarClick event for Grid
				* $("#Grid").ejGrid({
				*    toolBarClick: function (args) {}
				* });
				* @example 
				* //Bind toolBarClick event using jquery "on"
				* $("#Grid").on("ejGridtoolBarClick", function(e){}); 
				* @memberof ejGrid
				* @instance
			*/
            toolBarClick: null,
            /**		
             * It is used to define the collection of columns to bind grid
             * @default []
             * @type {array}
             * @example
             * columns at grid initialize:
			 * // field: This is used to map the Column name with the field name of the dataSource.
             * $("#Grid").ejGrid({ columns: [{ field: "OrderID" }] });
			 * <br>
			 * // headerText: The headerText property is used to declare the title of that particular column.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", headerText: "Order ID" }] });
			 * <br>
			 * // allowGrouping: Enables or disables grouping for a particular column.
             * $("#Grid").ejGrid({ columns: [{field: "fieldName", allowGrouping: false }] });
			 * <br>
			 * // allowFiltering: Enables or disables filtering for a particular column.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", allowFiltering: false  }] });
			 * <br>
			 * // allowSorting: Enables or disables sorting for a particular column.
             * $("#Grid").ejGrid({ columns: [{ field:  "fieldName", allowSorting: false }] });
			 * <br>
			 * // allowEditing: Enables or disables editing for a particular column.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", allowEditing: false }] });
			 * <br>
			 * // isUnBound: This enables or disables whether a particular column is unbound to grid columns.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", isUnBound: true }] });
			 * <br>
			 * // columnTemplate: Enables or disables the column template for a particular column.
             * $("#Grid").ejGrid({ columns: [{  field: "fieldName", columnTemplate: true }] });
			 * <br>
			 * // columnEditType: This defines the edit type of a particular column.
             * $("#Grid").ejGrid({ columns: [{ field:  "fieldName",columnEditType: ej.Grid.editingType.NumericEdit }] });
			 * <br>
			 * // textAlign: This defines the text alignment of a particular column cell value.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", textAlign: ej.textAlign.Right }] });
			 * <br>
			 * // key: This declares a particular column as the primary key of the grid.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", key: true }] });
			 * <br>
			 * // isIdentity defines that a particular column has an identity in the database.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", isIdentity: true }] });
			 * <br>
			 * // visible: Visible is used to hide or display a particular column during grid initialize.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", visible: false }] });
			 * <br>
			 * // dropdownData defines binding external data to the drop-down cell or column.
			 * // Where data is array of JSON objects of text and value for the drop-down.
			 * // dropdownData: It is necessary to define the columnEditType as "dropdownedit" along with dropdownData.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", dropdownData: data }] }); 
			 * <br>
			 * // headerTemplateId: An element’s ID which will be used for header template rendering.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", headerTemplateId: "#template" }] });
			 * <br>
			 * // width: This is used to define the width for a particular column in the grid.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", width: 100 }] });
			 * <br>
			 * // validationRules is used to define constraints for saving data to the database.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", validationRules: { required: true } }] });
			 * <br>
			 * // format: This is used to format the cell values of the grid.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", format: "{0:MM/dd/yyyy}" }] });
			 * <br>
			 * // customAttributes: Based on this customAttribute, users can apply attribute values to the td element of a 
			 * // particular column.
             * $("#Grid").ejGrid({ columns: [{ field: " fieldName", customAttributes: {"style":"color:red"} }] });
			 * <br>
			 * // templateId: An element’s ID which will be used for column template rendering. If you want to use
			 * // templateId for the column template, then columnTemplate must be set to true.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", templateId: "#columnTemplate"  }] });
			 * <br>
			 * // commands: This property is used to define a command column in grid. isUnBound must be true while declaring
			 * // commands button.
             * $("#Grid").ejGrid({ columns: [{ headerText: "Manage Records", commands: [
			 * { type: ej.Grid.unboundType.Edit, buttonOptions: { text: "Edit" } },
			 * { type: ej.Grid.unboundType.Delete, buttonOptions: { text: "Delete" } },
			 * { type: ej.Grid.unboundType.Save, buttonOptions: { text: "Save" } },
			 * { type: ej.Grid.unboundType.Cancel, buttonOptions: { text: "Cancel" } }], 
			 *   isUnBound: true, width: 130 }] });
			 * <br>
			 * // numericEditParams: This is used to customize ejNumericTextbox of an editable column.
             * $("#Grid").ejGrid({ columns: [{ field: "fieldName", numericEditParams:{ decimal: 2 } }] });
			 * <br>
             * Get the columns after grid initialize:
             * var value = $("#Grid").ejGrid("option", "columns");
             * @memberof ejGrid
             * @instance
             */
            columns: [],
            query: null,
            isEdit: false
        },

        dataTypes: {
            dataSource: "data",
            columns: "array",
            summaryRows: "array",
            toolBar: {
                toolBarItems: "array"
            },
            sortSettings: {
                sortedColumns: "array"
            },
            filterSettings: {
                filteredColumns: "array",
                filterType: "enum",
                filterBarMode: "enum",
            },
            groupSettings: {
                groupedColumns: "array"
            },
            edit: {
                editMode: "enum",
                formPosition: "enum"
            }
        },

        /**
			* It is used to get the header table of grid control.
			* @return gridHeaderTable
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridHeaderTable(); // Gets header table of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets header table of grid control
			* $("#Grid").ejGrid("getGridHeaderTable");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridHeaderTable: function () {
            return this._gridHeaderTable;
        },

        setGridHeaderTable: function (value) {
            this._gridHeaderTable = value;
        },

        /**
			* It is used to get the content table of grid control.
			* @return gridContentTable
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridContentTable(); // Gets content table of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets content table of grid control
			* $("#Grid").ejGrid("getGridContentTable");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridContentTable: function () {
            return this._gridContentTable;
        },

        setGridContentTable: function (value) {
            this._gridContentTable = value;
        },

        /**
			* It is used to get the content of grid control.
			* @return gridContent
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridContent(); // Gets content of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets content of grid control
			* $("#Grid").ejGrid("getGridContent");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridContent: function () {
            return this._gridContent;
        },

        setGridContent: function (value) {
            this._gridContent = value;
        },
        /**
			* It is used to get the header content of grid control.
			* @return gridHeaderContent
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridHeaderContent(); // Gets grid header content of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets grid header content of grid control
			* $("#Grid").ejGrid("getGridHeaderContent");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridHeaderContent: function () {
            return this._gridHeaderContent;
        },

        setGridHeaderContent: function (value) {
            this._gridHeaderContent = value;
        },
        /**
			* It is used to get the footer content of grid control.
			* @return gridFooterContent
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridFooterContent(); // Gets grid footer content of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets grid footer content of grid control
			* $("#Grid").ejGrid("getGridFooterContent");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridFooterContent: function () {
            return this._gridFooterContent;
        },

        setGridFooterContent: function (value) {
            this._gridFooterContent = value;
        },
        /**
			* It is used to get the footer table of grid control.
			* @return gridFooterTable
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridFooterTable(); // Gets grid footer table of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets grid footer table of grid control
			* $("#Grid").ejGrid("getGridFooterTable");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridFooterTable: function () {
            return this._gridFooterTable;
        },

        setGridFooterTable: function (value) {
            this._gridFooterTable = value;
        },
        /**
			* It is used to get the grid pager of grid control.
			* @return gridPager
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridPager(); // Gets grid pager of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets grid pager of grid control
			* $("#Grid").ejGrid("getGridPager");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridPager: function () {
            return this._gridPager;
        },
        /**
			* It is used to get the grid rows of grid control.
			* @return gridRows
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridRows(); // Gets grid rows of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets grid rows of grid control
			* $("#Grid").ejGrid("getGridRows");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridRows: function () {
            return this._gridRows;
        },
        /**
			* It is used to get the scroll object of grid control.
			* @return scrollObject
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getScrollObject(); // Gets scroll object of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets scroll object of grid control
			* $("#Grid").ejGrid("getScrollObject");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getScrollObject: function () {
            if (this._scrollObject == null)
                this._scrollObject = this.getGridContent().ejScroller("instance");
            return this._scrollObject;
        },
        setGridPager: function (value) {
            this._gridPager = value;
        },
        /**
			* It is used to get the filter bar of grid control.
			* @return gridFilterBar
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridFilterBar(); // Gets filter bar of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets filter bar of grid control
			* $("#Grid").ejGrid("getGridFilterBar");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridFilterBar: function () {
            return this._gridFilterBar;
        },

        setGridFilterBar: function (value) {
            this._gridFilterBar = value;
        },
        /**
			* It is used to get the current view data of grid control.
			* @return currentJsonData
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getCurrentViewData(); // Gets current view data of grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets current view data of grid control
			* $("#Grid").ejGrid("getCurrentViewData");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getCurrentViewData: function () {
            return this._currentJsonData;
        },

        // constructor function
        _init: function () {
            this._trigger("load");
            if (ej.isNullOrUndefined(this.model.query))
                this.model.query = ej.Query();
            this._initPrivateProperties();
            if ((this.model.scrolling.frozenColumns > 0 || this.model.scrolling.frozenRows > 0) && (this.model.allowGrouping || this.model.rowTemplate != null || this.model.detailTemplate != null || this.model.scrolling.allowVirtualScrolling || this.model.edit.editMode == "batch")) {
                this._renderAlertDialog();
                this._alertDialog.find(".e-content").text(this._getLocalizedLabels("frozenNotSupportedException"));
                this._alertDialog.ejDialog("open");
                return;
            }
            if (this.model.scrolling.allowVirtualScrolling && this.model.allowScrolling) {
                this.model.pageSettings.pageSize = this.model.pageSettings.pageSize == 12 ? Math.round(this.model.scrolling.height / 38) + 1 : this.model.pageSettings.pageSize;
                this.model.pageSettings.totalPages = Math.ceil(this._gridRecordsCount / this.model.pageSettings.pageSize);
            }
            if (this.model.width || this.model.height) {
                this.model.allowScrolling = true;
                if (this.model.width) this.model.scrolling.width = this.model.width;
                if (this.model.height) this.model.scrolling.height = this.model.height;
            }
            if (this.model.edit.allowEditing || this.model.edit.allowAdding)
                this._processEditing();
            this._checkDataBinding();
        },

        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "pageSettings":
                        var pageModel = this.getGridPager().ejPager("model");
                        if (ej.isNullOrUndefined(options[prop]["currentPage"]) || pageModel.currentPage != this._currentPage()) {
                            for (var pageProp in options[prop]) {
                                if (pageProp != "currentPage" && options[prop][pageProp] === pageModel[pageProp])
                                    delete options[prop][pageProp];
                            }
                            if ($.isEmptyObject(options[prop]))
                                break;
                            $.extend(this.model.pageSettings, options[prop]);
                            options[prop]["currentPage"] = this._currentPage();
                            this.getGridPager().ejPager("option", options[prop]);
                            this.refreshGridContent();
                        }
                        break;
                    case "allowPaging":
                        this.model.allowPaging = options[prop];
                        if (options[prop] && this.element.children(".e-pager").length == 0) {
                            this.element.append(this._renderGridPager());
                            this.refreshGridContent();
                            this.getGridPager().ejPager("refreshPager");
                        } else {
                            this.getGridPager().remove();
                            this.setGridPager(null);
                            this.refreshGridContent();
                            if (this.model.filterSettings.filterType == "filterbar" && this.model.allowFiltering)
                                this._createPagerStatusBar();
                        }
                        break;
                    case "allowGrouping":
                        if (options[prop] && this.element.children(".e-groupdroparea").length == 0) {
                            this.model.allowGrouping = options[prop];
                            this.addGroupingTemplate();
                            this.element.prepend(this._renderGroupDropArea());
                            this._enableGroupingEvents();
                            this._headerCellgDragDrop();
                        } else
                            this.element.children(".e-groupdroparea").remove();
                        break;
                    case "groupSettings":
                        $.extend(this.model.groupSettings, options[prop]);
                        this.templateRefresh();
                        if (this.model.groupSettings.toggleGrouping) {
                            var $table = this._renderGridHeader().find("table");
                            this.element.children(".e-gridheader").find("table").replaceWith($table);
                            this.setGridHeaderTable($table);
                            this.setWidthToColumns();
                        }
                        break;
                    case "cssClass":
                        this.element.removeClass(this.model.cssClass).addClass(options[prop]);
                        break;
                    case "allowFiltering":
                    case "filterSettings":
                        if (prop == "filterSettings")
                            $.extend(this.model.filterSettings, options[prop]);
                        else
                            this.model.allowFiltering = options[prop];
                        if (!this.model.allowFiltering) {
                            if (this.model.filterSettings.filterType == ej.Grid.filterType.FilterBar)
                                this.getGridHeaderTable().find(".e-filterbar").remove();
                            else if (this.model.filterSettings.filterType == ej.Grid.filterType.Menu)
                                this.getGridHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                            this.model.filterSettings.filteredColumns = [];
                            this.refreshGridContent();
                        } else {
                            if (this.model.filterSettings.filterType == ej.Grid.filterType.FilterBar) {
                                this.getGridHeaderTable().find(".e-filterbar").remove();
                                this.getGridHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                                this._renderFiltering();
                                if (this.model.filterSettings.showFilterBarMessage && !this.model.allowPaging)
                                    this._createPagerStatusBar();
                                else if (this.model.filterSettings.showFilterBarMessage && this.model.allowPaging)
                                    this.getGridPager().ejPager({ enableExternalMessage: true });
                                var $filterbar = this.getGridHeaderTable().find(".e-filterbar");
                                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                                    $filterbar.prepend(this._getEmptyFilterBarCell());
                            } else if (this.model.filterSettings.filterType == ej.Grid.filterType.Menu) {
                                this.getGridHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                                this.getGridHeaderTable().find(".e-filterbar").remove();
                                this.getGridHeaderTable().find(".e-columnheader").find(".e-headercell").addClass("e-headercellfilter").append(ej.buildTag('div.e-filtericon'));
                                this._renderFilterDialogs();
                            }
                            this._enableFilterEvents();
                        }
                        break;
                    case "rowHover":
                        this.model.rowHover = options[prop];
                        this._enableRowHover();
                        break;
                    case "allowScrolling":
                    case "scrolling":
                        var $content = this.getGridContent();
                        if (!ej.util.isNullOrUndefined(options["scrolling"]))
                            $.extend(this.model.scrolling, options["scrolling"]);
                        if (options["scrolling"]["frozenColumns"] !== undefined || options["scrolling"]["frozenRows"] !== undefined) {
                            var model = this.model;
                            model.query = this._commonQuery.clone();
                            this.element.ejGrid("destroy").ejGrid(model);
                        }
                        else {
                            if (!ej.util.isNullOrUndefined(options["allowScrolling"]))
                                this.model.allowScrolling = options["allowScrolling"];
                            !ej.util.isNullOrUndefined($content.data("ejScroller")) && $content.ejScroller("destroy");
                            if (this.model.allowScrolling) {
                                this.getGridHeaderTable().parent().addClass("e-headercontent");
                                this._renderScroller();
                            } else {
                                this.element.children(".e-gridheader").removeClass("e-scrollcss");
                                this.element.get(0).style.width.length == 0 && this.element.css("width", "auto");
                            }
                        }
                        break;
                    case "localization":
                        this.model.localization = options[prop];
                        var model = this.model;
                        model.query = this._commonQuery.clone();
                        this.element.ejGrid("destroy").ejGrid(model);
                        break;
                    case "dataSource":
                        this.resetModelCollections();
                        this._refreshDataSource(this._dataSource());
                        break;
                    case "selectedRow":
                        if (this._selectedRow() != -1 && $.inArray(this._selectedRow(), this._selectedRowsIndexes) == -1)
                            this.selectRows(this._selectedRow());
                        break;
                    case "edit":
                        $.extend(this.model.edit, options[prop]);
                        this._processEditing();
                        this.refreshBatchEditMode();
                        this._tdsOffsetWidth = [];
                        $("#" + this._id + "_dialogEdit_wrapper,#" + this._id + "_dialogEdit").remove();
                        $("#" + this._id + "_externalEdit").remove();
                        if (this.model.edit.allowEditing || this.model.edit.allowAdding) {
                            if (this.model.edit.editMode == "dialog" || this.model.edit.editMode == "dialogtemplate")
                                this.element.append(this._renderDialog());
                            else if (this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate")
                                this.element.append(this._renderExternalForm());
                        }
                        this._enableEditingEvents();
                        break;
                    case "enableEffects":
                        this.model.enableEffects = true;
                        if (!options[prop])
                            this.expandGroupDropArea();
                        else
                            this.collapseGroupDropArea();
                        break;
                    case "allowSummary":
                    case "summaryRows":
                        if (prop == "allowSummary" && !options[prop])
                            this.element.children(".e-gridfooter").remove();
                        else {
                            var $content = this.element.find(".e-gridcontent").first();
                            this._renderGridFooter().insertAfter($content);
                        }
                        break;
                    case "altRow":
                        this.model.altRow = options[prop];
                        this.addInitTemplate();
                        this.refreshGridContent();
                        break;
                    case "toolBar":
                        $.extend(this.model.toolBar, options[prop]);
                        this.element.children(".e-gridtoolbar").remove();
                        if (this.model.toolBar.allowToolBar)
                            this._renderToolBar().insertBefore(this.element.find(".e-gridheader").first());
                        break;
                    case "sortSettings":
                        $.extend(this.model.sortSettings, options[prop]);
                        this.refreshGridContent();
                        break;
                    case "rtl":
                        this.model.rtl = options[prop];
                        this.refreshGridContent();
                        break;
                    case "allowSelection":
                        if (options[prop])
                            this._off(this.element, "click")._on(this.element, "click", this._clickHandler);
                        break;
                    case "query":
                        this._commonQuery = $.extend(true, {}, options[prop]);
                        break;
                }
            }

        },
        resetModelCollections: function () {
            this.model.groupSettings.groupedColumns = [];
            this.model.filterSettings.filteredColumns = [];
            this.model.sortSettings.sortedColumns = [];
            $.extend(this.model.pageSettings, this.defaults.pageSettings);
            this.model.summaryRows = [];
        },
        _renderAfterColumnInitialize: function () {
            this.element.append(this._renderGridHeader());
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar")
                this._renderFiltering();
            if (this.model.allowPaging)
                this.element.append(this.element.find(".e-pager").first());
        },
        _checkDataBinding: function () {
            if (this.model.persist)
                this.model.query = new ej.Query();
            this._initialRenderings();
            this._initialRender = true;
            this.model.rtl && this.element.addClass("e-rtl");
            if (this.model.cssClass != null)
                this.element.addClass(this.model.cssClass);
            if (this.model.allowGrouping)
                this.element.append(this._renderGroupDropArea());
            if (this.model.toolBar.allowToolBar)
                this.element.append(this._renderToolBar());
            var columns = this.model.columns;
            if (columns && columns.length) {
                var expands = [];
                if (typeof columns[0] === "string")
                    for (var i = 0; i < columns.length; i++)
                        columns[i] = { field: columns[i] };
                for (var i = 0; i < columns.length; i++) {
                    if (!columns[i].field || columns[i].field.indexOf('.') === -1) continue;
                    this._getExpands(columns[i].field, expands);
                }
                this.model.query.expand(expands);
                this._renderAfterColumnInitialize();
            }
            if (this.model.allowPaging)
                this.element.append(this._renderGridPager());
            if ($.isFunction($.fn.ejWaitingPopup)) {
                this.element.ejWaitingPopup({ autoDisplay: false });
                $("#" + this._id + "_WaitingPopup").addClass("e-gridwaitingpopup");
            }
            if (this._dataSource() instanceof ej.DataManager) {
                this.element.ejWaitingPopup("show");
                if (this._dataSource().ready != undefined) {
                    var proxy = this;
                    this._dataSource().ready.done(function (args) {
                        proxy._initDataSource();
                        proxy.model.dataSource = ej.DataManager(args.result);
                    });
                }
                else {
                    this.element.ejWaitingPopup("show");
                    this._initDataSource();
                }
            } else {
                this._ensureDataSource();
                this._trigger("actionBegin");
                this._initGridRender();
            }
        },
        _getExpands: function (field, arr) {
            var splits = field.split('.'), tmp = "";
            splits.splice(splits.length - 1, 1);
            for (var i = 0; i < splits.length; i++, tmp = "") {
                for (var j = 0; j < i; j++)
                    tmp += splits[j] + "/";
                tmp = tmp + splits[i];
                if (arr.indexOf(tmp) === -1)
                    arr.push(tmp);
            }
        },
        _initDataSource: function () {
            this._ensureDataSource();
            this._trigger("actionBegin");
            var queryPromise = this._dataSource().executeQuery(this.model.query);
            if (this._dataManager.dataSource.table != null)
                this._dataManager.dataSource.table.css("display", "none");
            queryPromise.done(ej.proxy(function (e) {
                this.element.ejWaitingPopup("hide");
                this.model.currentViewData = e.result;
                if (this.model.pageSettings.totalRecordsCount != null && this.model.filterSettings.filteredColumns.length == 0)
                    this._gridRecordsCount = this.model.pageSettings.totalRecordsCount;
                else if (e.count == 0 && e.result.length)
                    this._gridRecordsCount = e.result.length;
                else
                    this._gridRecordsCount = e.count;
                this._initGridRender();
            }, this));
        },
        _renderFilterDialogs: function () {
            var $strDlg, $numDlg, $boolDlg, $dateDlg;

            $.each(this.model.columns, ej.proxy(function (indx, col) {
                if (col.type == "string" && !$strDlg) {
                    $strDlg = true;
                    this._renderStringDialog();
                } else if (col.type == "number" && !$numDlg) {
                    $numDlg = true;
                    this._renderNumDialog();
                } else if (col.type == "date" && !$dateDlg) {
                    $dateDlg = true;
                    this._renderDateDialog();
                } else if (col.type == "boolean" && !$boolDlg) {
                    $boolDlg = true;
                    this._renderBoolDialog();
                }
            }, this));
        },
        _renderStringDialog: function () {
            var $id = this._id + "_stringDlg";
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            this._renderDlgContent($content, "string");

        },
        _renderBoolDialog: function () {
            var $id = this._id + "_booleanDlg";
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            if (!this.model.filterSettings.showPredicate)
                $("#" + $id).ejDialog({ minHeight: 90 });
            else
                $("#" + $id).ejDialog({ minHeight: 136 });
            this._renderDlgContent($content, "boolean");
        },
        _renderNumDialog: function () {
            var $id = this._id + "_numberDlg";
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            this._renderDlgContent($content, "number");

        },
        _renderDateDialog: function () {
            var $id = this._id + "_dateDlg";
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            this._renderDlgContent($content, "date");

        },
        _renderFDialog: function (id) {
            $("#" + id).ejDialog({ autoOpen: false, "rtl": this.model.rtl, "cssClass": this.model.cssClass, "showHeader": false, width: 260, resizable: false, allowKeyboardNavigation: false, contentContainer: "#" + this._id });
        },
        _renderDlgContent: function (content, type) {
            content.addClass("e-grid");
            var $predicate = ej.buildTag("div.e-predicate"), $operator = ej.buildTag("div.e-operator"), $value = ej.buildTag("div.e-value");
            var $strOp = ["StartsWith", "EndsWith", "Contains", "Equal", "NotEqual"];
            var $numOp = ["LessThan", "GreaterThan", "LessThanOrEqual", "GreaterThanOrEqual", "Equal", "NotEqual"];
            var $drdown = ej.buildTag("input#" + this._id + type + "_ddinput", {}, {}, { "type": "text" });
            var $drdownDiv = ej.buildTag("div#" + this._id + type + "_dropdown");
            var $drdownUl = ej.buildTag("ul");
            var $radio = ej.buildTag("input", {}, {}, { "type": "radio", "name": this._id + "_predicate" + type, "value": "or" });
            var $andRadio = ej.buildTag("input", {}, {}, { "type": "radio", "name": this._id + "_predicate" + type, "value": "and", "checked": "checked" });
            var $cbox;
            $predicate.append($andRadio)
                .append(ej.buildTag("span.e-caption").html("AND"))
                .append($radio)
                .append(ej.buildTag("span.e-caption").html("OR"));
            !this.model.filterSettings.showPredicate && $predicate.hide();
            if (type == "string") {
                $cbox = ej.buildTag("input", {}, {}, { "type": "checkbox" });
                $predicate.append($cbox)
                    .append(ej.buildTag("span.e-caption").html("Match Case"));
                $.each($strOp, function (indx, operator) {
                    $drdownUl.append(ej.buildTag("li").html(operator));
                });
            }
            if (type == "number" || type == "date") {
                $.each($numOp, function (indx, operator) {
                    $drdownUl.append(ej.buildTag("li").html(operator));
                });
            }
            if (type != "boolean") {
                $drdownDiv.append($drdownUl);
                $operator.append($drdown);
                $operator.append($drdownDiv);
            }
            var $tBox = ej.buildTag("input", {}, {}, { "type": "text" });
            var $tchkBox = ej.buildTag("input", {}, {}, { "type": "checkbox" });
            var $filter = ej.buildTag("input.e-filter", {}, {}, { "type": "button", "value": "Filter" });
            var $clear = ej.buildTag("input.e-clear", {}, {}, { "type": "button", "value": "Clear" });
            $value.append(ej.buildTag("span.e-caption").html("Filter : "));
            content.append($predicate);
            if (type == "boolean")
                $value.append($tchkBox);
            else {
                $value.append(ej.buildTag("br")).append($tBox);
                content.append($operator);
            }
            content.append($value);
            content.append(ej.buildTag("div.e-dlgBtns").append($filter)
                .append($clear));
            $drdown.ejDropDownList({ "cssClass": this.model.cssClass, "rtl": this.model.rtl, "targetId": this._id + type + "_dropdown", width: "230px", height: "26px", selectedItem: 0 });
            $radio.ejRadioButton({ "cssClass": this.model.cssClass, "rtl": this.model.rtl });
            $andRadio.ejRadioButton({ "cssClass": this.model.cssClass, "rtl": this.model.rtl, check: true });
            if ($cbox)
                $cbox.ejCheckBox({ "cssClass": this.model.cssClass, "rtl": this.model.rtl });
            content.css("display", "none");
            this._createButton("filter", $filter);
            this._createButton("clear", $clear);
            if (type == "number")
                $tBox.ejNumericTextbox({ "cssClass": this.model.cssClass, "rtl": this.model.rtl, "value": 0, showSpinButton: false, height: "26px", decimals: 2 });
            else if (type == "date")
                $tBox.ejDatePicker({ "cssClass": this.model.cssClass, "rtl": this.model.rtl, width: 230 });
            else if (type == "boolean")
                $tchkBox.ejCheckBox({ "cssClass": this.model.cssClass, "rtl": this.model.rtl });
            else if (type == "string") {
                $tBox.attr("id", this._id + "_acString");
                $tBox.ejAutocomplete({ "cssClass": this.model.cssClass, "rtl": this.model.rtl, "dataSource": this._dataSource(), width: 230, height: 26, distinct: true });
            }
        },
        _createButton: function (name, element) {
            var $func = name == "filter" ? ej.proxy(this._fltrBtnHandler, this) : ej.proxy(this._fltrClrHandler, this);
            element.ejButton({ "cssClass": this.model.cssClass, "rtl": this.model.rtl, "click": $func });
        },
        _fltrBtnHandler: function (e) {
            var $par = $("#" + this._id + "_" + this._$colType + "Dlg");
            var $input = $par.find(".e-value input"), $operator, result;
            var value = $input.val(), matchcase = undefined, filterValue;
            if (this._$colType == "number") {
                value = parseFloat($input.ejNumericTextbox("getValue"));
                matchcase = true;
            }
            if (this._$colType == "string")
                matchcase = $par.find(".e-predicate input[type='checkbox']").is(":checked");
            if (this._$colType == "date")
                value = Globalize.parseDate(value, this._$colFormat);
            if (this._$colType == "boolean") {
                value = $input.ejCheckBox("model.check") != null ? $input.ejCheckBox("model.check") : false;
                $operator = "equal";
            } else
                $operator = $("#" + this._id + this._$colType + "_ddinput").ejDropDownList("getSelectedItem").toLowerCase();
            if (this._$colForeignKeyValue && this._$colDropdownData)
                result = this._fltrForeignKeyValue($operator, value, matchcase, this._$colDropdownData, this._$colForeignKeyField, this._$colForeignKeyValue);
            if (ej.isNullOrUndefined(result))
                this.sendFilteringRequest(this._$curFieldName, $operator, value, $par.find(".e-predicate input[type='radio']:checked").attr("value"), matchcase);
            else {
                if ($par.find(".e-predicate input[type='radio']:checked").css("display") == "none" && result.value.length > 1)
                    this.sendFilteringRequest(this._$curFieldName, result.operator, result.value, "or", matchcase, result.filterValue);
                else
                    this.sendFilteringRequest(this._$curFieldName, result.operator, result.value, $par.find(".e-predicate input[type='radio']:checked").attr("value"), matchcase, result.filterValue);
            }
        },
        _fltrClrHandler: function (e) {
            var id = "#" + this._id + "_" + this._$colType + "Dlg";
            if (this._$colType == "boolean")
                $(id).find(".e-value input").ejCheckBox("model.check", false)
            else
                if (this._$colType == "number")
                    $(id).find(".e-value input").ejNumericTextbox("model.value", "");
                else
                    $(id).find(".e-value input").val("");
            this.sendFilteringRequest(this._$curFieldName, "", "", "or");
        },

        _fltrForeignKeyValue: function (operator, value, matchcase, dataSource, fieldName, mapFieldName, colType) {
            if (ej.isNullOrUndefined(matchcase))
                matchcase = true;
            var operatorCol = [], query;
            var data = ej.DataManager(dataSource);
            if (colType == "date") {
                var $prevDate = new Date(value.setDate(value.getDate() - 1));
                var $nextDate = new Date(value.setDate(value.getDate() + 2));
                if (operator == "equal" || operator == "notequal") {
                    if (operator == "equal")
                        query = new ej.Query().where(ej.Predicate(mapFieldName, ">", $prevDate, !matchcase)
                            .and(mapFieldName, "<", $nextDate, !matchcase)).select(fieldName);
                    else
                        query = new ej.Query().where(ej.Predicate(mapFieldName, "<=", $prevDate, !matchcase)
                           .or(mapFieldName, ">=", $nextDate, !matchcase)).select(fieldName);
                }
                else
                    query = new ej.Query().where(mapFieldName, operator, value, !matchcase).select(fieldName);
            }
            else
                query = new ej.Query().where(mapFieldName, operator, value, !matchcase).select(fieldName);
            filterValue = value;
            value = data.executeLocal(query);
            operatorCol.push("equal");
            for (i = 1; i < value.length ; i++) {
                operatorCol.push("equal");
            }
            return { operator: operatorCol, value: value, filterValue: filterValue };
        },

        _processEditing: function () {
            var query = this._columnToSelect();
            this.model.query._fromTable != "" && query.from(this.model.query._fromTable);
            if (this._dataSource() instanceof ej.DataManager && query.queries.length) {
                var queryPromise = this._dataSource().executeQuery(query);
                queryPromise.done(ej.proxy(function (e) {
                    this._dropDownManager = ej.DataManager(e.result);
                    if (this.model.edit.editMode == "normal") this.addEditingTemplate();
                    else if (this.model.edit.editMode == "batch") this.addBulkEditTemplate();
                    else if (this.model.edit.editMode == "dialog" ||
                             this.model.edit.editMode == "externalform" ||
                             this.model.edit.editMode == "inlineform")
                        this.addDialogEditingTemplate();
                    else this.addExternalDialogEditingTemplate();
                }, this));
            } else {
                if (this.model.edit.editMode == "normal") this.addEditingTemplate();
                else if (this.model.edit.editMode == "batch") this.addBulkEditTemplate();
                else if (this.model.edit.editMode == "dialog" ||
                         this.model.edit.editMode == "externalform" ||
                         this.model.edit.editMode == "inlineform")
                    this.addDialogEditingTemplate();
                else this.addExternalDialogEditingTemplate();
            }
        },
        _initGridRender: function () {
            this.addInitTemplate();
            if (this.model.scrolling.frozenColumns > 0)
                this.addFrozenTemplate();
            this.model.allowGrouping && this.addGroupingTemplate();
            this.renderGrid();
            this.model.allowGrouping && this._headerCellgDragDrop();
            if (this.model.allowReordering) {
                this._headerCellreorderDragDrop();
                this._initIndicators();
            }
            this._wireEvents();
            if (this.model.allowResizing || this.model.allowResizeToFit)
                this._resizer = new ej.Grid.gridResize(this);
            this._initialRender = false;
            if (this.model.width && !this.model.allowScrolling)
                this.element.width(this.model.width);
            if (this.model.edit.allowEditing || this.model.edit.allowAdding)
                this._processEditing();
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "menu")
                this._renderFilterDialogs();
        },
        _checkSkipAction: function (args) {
            switch (args.requestType) {
                case ej.Grid.actions.Save:
                case ej.Grid.actions.Delete:
                    return true;
            }
            return false;
        },
        _bulkChangesAcquired: function () {
            if (this._batchChanges.added.length > 0 || this._batchChanges.changed.length || this._batchChanges.deleted.length)
                return true;
            return false;
        },
        _processBindings: function (args) {
            this.model.query = this.model.persist ? new ej.Query() : this._commonQuery.clone();
            if (!this._checkSkipAction(args) && this._trigger("actionBegin", args))
                return true;
            if (this.model.edit.editMode == "batch" && args.requestType != "bulksave" && args.requestType != "cancel" && !this._confirmedValue && this._bulkChangesAcquired()) {
                this._confirmDialog.find(".e-content").html(this._getLocalizedLabels("BulkSaveLostChanges"));
                this._confirmDialog.ejDialog("open");
                this._requestArgs = args;
                return false;
            }
            this._ensureDataSource(args);
            if (this.model.rtl) {
                !this.element.hasClass("e-rtl") && this.element.addClass("e-rtl");
            }
            else {
                this.element.hasClass("e-rtl") && this.element.removeClass("e-rtl")
            }
            if (args.requestType == ej.Grid.actions.Delete && this.model.groupSettings.groupedColumns.length == 0)
                args.tr.remove();
            if (this._dataSource() instanceof ej.DataManager && args.requestType != ej.Grid.actions.BeginEdit && args.requestType != ej.Grid.actions.Cancel && args.requestType != ej.Grid.actions.Add) {
                var queryPromise = this._dataSource().executeQuery(this.model.query);
                var waitingPopup = this.element.ejWaitingPopup("instance");
                this.element.ejWaitingPopup("show");
                queryPromise.done(ej.proxy(function (e) {
                    this.element.ejWaitingPopup("hide");
                    this.model.currentViewData = e.result;
                    if (e.count == 0 && e.result.length)
                        this._gridRecordsCount = e.result.length;
                    else
                        this._gridRecordsCount = e.count;
                    if (args.requestType == ej.Grid.actions.Filtering || (this.model.filterSettings.filteredColumns.length > 0 && args.requestType == ej.Grid.actions.Refresh))
                        this._filteredRecordsCount = e.count;
                    this.sendDataRenderingRequest(args);
                }, this));
                queryPromise.fail(ej.proxy(function (e) { this.element.ejWaitingPopup("hide"); }, this));
            } else
                this.sendDataRenderingRequest(args);
        },
        _initPrivateProperties: function () {
            this._gridHeaderTable = null;
            this._gridWidth = this.element.width();
            this._id = this.element.attr("id");
            this._gridRows = null;
            this._gridContentTable = null;
            this._gridContent = null;
            this._gridHeaderContent = null;
            this._gridFooterContent = null;
            this._gridFooterTable = null;
            this._gridRecordsCount = this._dataSource() !== null ? (this.model.pageSettings.totalRecordsCount == null ? this._dataSource().length : this.model.pageSettings.totalRecordsCount) : 0;
            this._links = null;
            this._gridPager = null;
            this._cSortedColumn = null;
            this._cSortedDirection = null;
            this._$curSElementTarget = null;
            this._gridFilterBar = null;
            this._$curFieldName = null;
            this._$prevFieldName = null;
            this._$fDlgIsOpen = false;
            this._$colType = null;
            this._$colFormat = null;
            this._$prevColType = null;
            this._$prevSElementTarget = null;
            this._currentFilterColumn = null;
            this._filteredRecordsCount = null;
            this._filteredRecords = [];
            this.filterColumnCollection = [];
            this._previousFilterCount = null;
            this._primaryKeys = [];
            this._primaryKeyValues = [];
            this._modifiedRecords = [];
            this._addedRecords = [];
            this._tdsOffsetWidth = [];
            this._deletedRecords = [];
            this._disabledToolItems = $();
            this._validationRules = {};
            this._groupedColumns = [];
            this._currentJsonData = [];
            this._groupingColumnIndex = 0;
            this._dataManager = this._dataSource() instanceof ej.DataManager ? this._dataSource() : this._dataSource() != null ? ej.DataManager(this._dataSource()) : null;
            if (this._dataManager != null && this.model.allowScrolling && this.model.scrolling.allowVirtualScrolling && this.model.pageSettings.totalRecordsCount != null && this._dataManager.dataSource.json != null)
                this._dataManager.dataSource.json.splice(this.model.pageSettings.totalRecordsCount);
            this._disabledSortableColumns = [];
            this._disabledGroupableColumns = [];
            this._disabledFilterableColumns = [];
            this._hiddenColumns = [];
            this._visibleColumns = [];
            this._sortedColumns = [];
            this._multiSortRequest = false;
            this._multiSelectCtrlRequest = false;
            this._multiSelectShiftRequest = false;
            this._enableSelectMultiTouch = false;
            this._enableSortMultiTouch = false;
            this._initialRender = false;
            this._fieldColumnNames = {};
            this._headerColumnNames = {};
            this._selectedRowsIndexes = [];
            this._isReorder = false;
            this._searchString = "";
            this._searchCount = null;
            this._columnsWidthCollection = [];
            this._Indicator = null;
            this._resizer = null;
            this._dropDownTemplate = this._compiledDropDownTemplate();
            this._bulkEditCellDetails = {
                cellValue: null,
                rowIndex: -1,
                columnIndex: -1,
                fieldName: null,
                _data: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                rowData: null
            };
            this._batchChanges = {
                added: [],
                deleted: [],
                changed: []
            };
            this._bulkEditTemplate = $();
            this._confirmDialog = null;
            this._confirmedValue = false;
            this._lastRow = false;
            this._isVirtualRecordsLoaded = false;
            this._scrollValue = 0;
            this._currentTopFrozenRow = this.model.scrolling.frozenRows;
            this._rowHeightCollection = [];
            this._scrollObject = null;
            this._customPop = null;
            this._commonQuery = $.extend(true, {}, this.model.query);
            this._rowCol = this._captionSummary();
            this._isCaptionSummary = (this._rowCol != null && this._rowCol.length) > 0 ? true : false;
        },

        _captionSummary: function () {
            var summary = null;
            for (k = 0; k < this.model.summaryRows.length; k++) {
                if (this.model.summaryRows[k].showCaptionSummary == true) {
                    var summary = $(this.model.summaryRows[k]);
                    break;
                }
            }
            return summary;
        },

        addInitTemplate: function () {
            var headerCellDiv = this.element.find(".e-headercelldiv"), templates = {};
            var tbody = document.createElement('tbody'), $tbody = $(tbody);
            if (this.model.rowTemplate == null) {
                var tr = document.createElement('tr'), $tr = $(tr),
                    columns = this.model.columns, i;
                if (this._gridRecordsCount)
                    this._initColumns(this.model.currentViewData[0] != undefined ? this.model.currentViewData[0] : this.model.currentViewData.value);
                var helpers = { _gridFormatting: this.formatting };
                $.views.helpers(helpers);

                var viewHelper = {};
                viewHelper["_foreignKey"] = this._foreignKeyBinding;
                $.views.helpers(viewHelper);

                if (this.model.detailTemplate) {
                    var $tdDetailCell = ej.buildTag("td.e-detailrowcollapse", "<div class='e-icon e-gnextforward'></div>");
                    $tr.append($tdDetailCell);
                }
                for (i = 0; i < this.model.columns.length; i++) {
                    var $tdCell = ej.buildTag("td.e-rowcell");
                    columns[i]["visible"] == false && $tdCell.addClass("e-hide");
                    !this.model.groupSettings.groupedColumnShow && $tdCell.addClass("{{for ~groupedColumns}}" +
                        " {{if #data == '" + this.model.columns[i]["field"] + "'}}e-hide{{/if}}" +
                        "{{/for}}");
                    if (columns[i]["columnTemplate"]) {
                        var viewHelper = {};
                        viewHelper["_" + this._id + "ColumnTemplating"] = this._gridTemplate;
                        $.views.helpers(viewHelper);
                        $("#" + this._id + columns[i].headerText + i + "_Template").remove();
                        var scriptElement = this._createTemplateElement(columns[i]);
                        this.model.columns[i]["allowGrouping"] = this.model.columns[i]["allowFiltering"] = this.model.columns[i]["allowSorting"] = false;
                        $tdCell.addClass("e-templatecell").html("{{:~_" + this._id + "ColumnTemplating('" + scriptElement.id + "')}}");
                    } else {
                        var splits = (columns[i].field || "").split("."), sLen = splits.length - 1, braces = "";
                        while (sLen) {
                            braces += "(";
                            sLen--;
                        }
                        switch (columns[i].type) {
                            case "boolean":
                                $tdCell.addClass("e-boolrowcell").html('{{if ' + columns[i].field + '}} <input type ="checkbox" disabled="disabled" checked="checked"></input>{{else}} <input type ="checkbox" disabled="disabled"  ></input> {{/if}}');
                                break;
                            default:
                                $tdCell.html("{{:" + braces + "#data['" + splits.join("'] || {})['") + "']}}");
                        }
                        if (columns[i]["format"] != undefined && (!columns[i]["foreignKeyValue"] && !columns[i]["dataSource"]))
                            $tdCell.html("{{:~_gridFormatting('" + columns[i]["format"] + "'," + columns[i]["field"] + ")}}");
                        if (columns[i]["foreignKeyValue"] && columns[i]["dataSource"]) {
                            var foreignKeyColumn = columns[i]["field"] != columns[i]["foreignKeyField"] ? columns[i]["field"] : columns[i]["foreignKeyField"];
                            $tdCell.html("{{:~_foreignKey('" + JSON.stringify(columns[i]) + "'," + foreignKeyColumn + ",'" + this._id + "Object" + "')}}");
                        }
                        if (columns[i]["isUnBound"]) {
                            var viewHelper = {};
                            viewHelper["_" + this._id + "UnboundTemplate"] = this._unboundTemplateRendering;
                            $.views.helpers(viewHelper);
                            this.model.columns[i]["allowGrouping"] = this.model.columns[i]["allowFiltering"] = this.model.columns[i]["allowSorting"] = false;
                            $("#" + this._id + columns[i].headerText + "_UnboundTemplate").remove();
                            divElement = this._createUnboundElement(columns[i]);
                            $tdCell.addClass("e-unboundcell").addClass("e-" + columns[i]["headerText"].replace(/\s+/g, "")).html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                            this.model.scrolling.frozenColumns > 0 && $tdCell.addClass("e-frozenunbound");
                            this._isUnboundColumn = true;
                        }

                    }
                    if (columns[i]["textAlign"] != undefined) {
                        $tdCell.css("text-align", columns[i]["textAlign"]);
                        $(headerCellDiv[i]).css("text-align", columns[i]["textAlign"]);
                    }
                    if (!ej.isNullOrUndefined(columns[i]["cssClass"])) {
                        $tdCell.addClass(columns[i]["cssClass"]);
                    }
                    if (!ej.isNullOrUndefined(columns[i]["customAttributes"]))
                        $tdCell.attr(columns[i]["customAttributes"]);
                    $tdCell.attr("role", "gridcell");
                    $tr.append($tdCell);
                    if (this.model.altRow) {
                        helpers["_" + this._id + "AlternateRow"] = this._alternateRow;
                        $.views.helpers(helpers);
                        $tr.addClass("{{:~_" + this._id + "AlternateRow()}}");
                    }
                    $tr.attr("role", "row");
                    if (this.model.scrolling.frozenColumns > 0 && this.model.scrolling.frozenColumns == i + 1) {
                        tbody.appendChild(tr);
                        templates[this._id + "_JSONFrozenTemplate"] = $tbody.html();
                        $tr.empty();
                        $tbody.empty();
                    }
                }
                tbody.appendChild(tr);
            }
            templates[this._id + "_JSONTemplate"] = this.model.rowTemplate != null ? $(this.model.rowTemplate).html() : $tbody.html();
            $.templates(templates);
        },
        addFrozenTemplate: function () {
            var template = "<div class='e-frozencontentdiv'>"
            + "<table cellspacing='0.25px' class='e-table'>{{:~colgroup1}}<tbody>"
            + "{{for datas tmpl='" + this._id + "_JSONFrozenTemplate'/}}"
            + "</tbody></table></div>"
            + "<div class='e-movablecontent'><div class='e-movablecontentdiv'><table cellspacing='0.25px' class='e-table'>{{:~colgroup2}}<tbody>"
            + "{{for datas tmpl='" + this._id + "_JSONTemplate'/}}"
            + "</tbody></table></div></div>", templates = {};
            templates[this._id + "_FrozenTemplate"] = template;
            $.templates(templates);
        },
        _foreignKeyBinding: function (curColumn, cellValue, gridId) {
            if (typeof (curColumn) == "string")
                curColumn = JSON.parse(curColumn);
            var cellData;
            curColumn.dataSource.filter(function (col) {
                if (col[curColumn.foreignKeyField] == cellValue)
                    return cellData = curColumn.type == "date" ? new Date(col[curColumn.foreignKeyValue]) : col[curColumn.foreignKeyValue];
            });
            if (curColumn.format) {
                var gridObj = ej.isNullOrUndefined(gridId) ? this : this.getRsc("helpers", gridId);
                cellData = gridObj.formatting(curColumn.format, cellData);
            }
            return cellData;
        },
        _createTemplateElement: function (column) {
            var scriptElement = document.createElement("script");
            scriptElement.id = (this._id + column.headerText + $.inArray(column, this.model.columns) + "_Template").split(" ").join("");
            scriptElement.type = "text/x-jsrender";
            scriptElement.text = $(column["templateId"]).html();
            $("body").append(scriptElement);
            $(column["templateId"]).hide();
            return scriptElement;
        },
        _createUnboundElement: function (column) {
            var divElement = document.createElement("div");
            divElement.id = this._id + column.headerText.replace(/\s+/g, "") + "_UnboundTemplate";
            var $div = ej.buildTag("div.e-unboundcelldiv"), commands = column["commands"];
            for (var unbounType = 0; unbounType < commands.length; unbounType++) {
                var $button = ej.buildTag("input.e-" + commands[unbounType].type + "button", "", {}, { type: "button" });
                $button.val(commands[unbounType].type);
                $div.append($button);
            }
            $("body").append($(divElement).html($div).hide());
            return divElement;
        },
        addGroupingTemplate: function () {
            var tbody = document.createElement('tbody');
            var expandTd = "<td class='e-recordplusexpand'><div class='e-icon e-gdiagonalnext'></div></td>";
            var proxy = this;
            var helpers = {};
            helpers["_" + proxy._id + "ColSpanAdjust"] = this._colSpanAdjust;
            helpers["_" + proxy._id + "Colgroup"] = this._getColGroup;
            helpers["_" + proxy._id + "CaptionFormat"] = this._captionFormat;
            helpers["_" + proxy._id + "GroupSummaryRow"] = this._groupSummaryRow;
            helpers[proxy._id + "Object"] = this;
            $.views.helpers(helpers);
            var cpationTd = expandTd + "<td class='e-groupcaption' colspan='{{:~_" + proxy._id + "ColSpanAdjust('" + proxy._id + "Object" + "','groupcaption') }}'>{{:~_" + proxy._id + "CaptionFormat('" + proxy._id + "Object')}}</td";
            if (this._isCaptionSummary && this.model.allowSummary)
                cpationTd = cpationTd + "{{:~_" + proxy._id + "GroupSummaryRow(items,'" + proxy._id + "Object')}}";
            var captionTr = "<tr>" + cpationTd + "</tr>";
            var $tbody = ej.buildTag("tbody");
            $tbody.html("{{if items.GROUPGUID}}" +
                "{{for items tmpl='" + proxy._id + "_GroupingTemplate'/}}" +
                "{{else}}" +
                "{{for items tmpl='" + proxy._id + "_JSONTemplate'/}}" +
                "{{/if}}");
            var indentTd = "<td class='e-indentcell'></td>";
            var table = "<table class='e-table {{if items.GROUPGUID}}{{else}}e-recordtable{{/if}}' cellspacing='0.25px' >" +
                "{{:~_" + proxy._id + "Colgroup('" + proxy._id + "Object')}}" +
                $tbody.html() + (this._isCaptionSummary ? "</table>" : "{{:~_" + proxy._id + "GroupSummaryRow(items,'" + proxy._id + "Object')}}" +
            "</table>");
            var tableTd = "<td class='e-tabletd' colspan='{{:~_" + proxy._id + "ColSpanAdjust('" + proxy._id + "Object" + "','table')}}'>" + table + "</td>";
            var tr = "<tr>" + indentTd + tableTd + "</tr>";
            $.templates(proxy._id + "_GroupingTemplate", captionTr + tr);
        },
        _groupSummaryRow: function (item, gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            if (gridObject.model.allowSummary) {
                if (!ej.isNullOrUndefined(item.level) && !gridObject._isCaptionSummary) return;
                if (gridObject.getGridFooterTable() == null)
                    gridObject._renderGridFooter();
                gridObject._createSummaryRows(gridObject.getGridFooterTable(), item.records == null ? item : item.records);
                if (gridObject._isCaptionSummary) {
                    var colspan = gridObject._colSpanAdjust(null, "groupcaption", this);
                    colspan = item.childLevels != null ? colspan - (item.childLevels + 1) : colspan;
                    gridObject.getGridFooterTable().find("tbody td:lt(" + colspan + ")").remove();
                    gridObject.getGridFooterTable().find("tbody td").removeClass("e-summaryrow").addClass("e-groupcaptionsummary");
                }
                return gridObject.getGridFooterTable().find("tbody").html();
            }
        },
        _captionFormat: function (gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            var keyValue;
            var column = gridObject.getColumnByField(this.data.field);
            if (column.foreignKeyValue && column.dataSource)
                keyValue = gridObject._foreignKeyBinding(column, this.data.key);
            else
                keyValue = this.data.key;
            var formatted = column.format ? gridObject.formatting(column.format, keyValue) : keyValue;
            return column.headerText + ": " + formatted + " - " + this.data.count + " " + gridObject._getLocalizedLabels("GroupCaptionFormat");
        },
        _alternateRow: function () {
            return this.getIndex() % 2 == 0 ? "" : "e-alt_row";
        },
        _getColGroup: function (gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            if (gridObject.model.groupSettings.groupedColumns.length == 1)
                var level = this.data.items.level === undefined ? 0 : this.data.items.level - 1;
            else
                var level = this.data.items.level === undefined ? gridObject.model.groupSettings.groupedColumns.length : this.data.items.level - 1;
            var $div = $(document.createElement("div"));
            var $colGroup;
            if (gridObject._isCaptionSummary)
                $colGroup = gridObject._getCaptionColGroup(level);
            else {
                $colGroup = gridObject._getMetaColGroup();
                if (level != gridObject.model.groupSettings.groupedColumns.length && gridObject.model.groupSettings.groupedColumns.length > 1)
                    $colGroup.prepend(gridObject._getIndentCol());
            }
            $div.html($colGroup);
            return $div.html();
        },
        _colSpanAdjust: function (gridObjectId, type, captionDetails) {
            var gridObject, groupData;
            if (ej.isNullOrUndefined(gridObjectId)) {
                gridObject = this;
                groupData = captionDetails;
            }
            else {
                gridObject = this.getRsc("helpers", gridObjectId);
                groupData = this;
            }
            if (gridObject.model.groupSettings.groupedColumns.length == 1) {
                var level = groupData.data.items.level === undefined ? 1 : groupData.data.items.level - 1;
                if (type == "groupcaption")
                    gridObject._currentJsonData = gridObject._currentJsonData.concat(groupData.data.items);
            } else {
                var level = groupData.data.items.level === undefined ? gridObject.model.groupSettings.groupedColumns.length : groupData.data.items.level - 1;
                if (type == "groupcaption" && groupData.data.items.level === undefined)
                    gridObject._currentJsonData = gridObject._currentJsonData.concat(groupData.data.items);
            }
            var hideGroupColumnCount = !gridObject.model.groupSettings.groupedColumnShow ? gridObject.model.groupSettings.groupedColumns.length : 0;
            var colspan = gridObject.model.columns.length + gridObject.model.groupSettings.groupedColumns.length - level - gridObject._hiddenColumns.length - hideGroupColumnCount;
            colspan = gridObject.model.detailTemplate != null ? colspan + 1 : colspan;

            if (gridObject._isCaptionSummary && type == "groupcaption") {
                var index = [];
                var row = gridObject._captionSummary();
                $.each(row[0].summaryColumns, function (cindx, col) {
                    index.push(gridObject.getColumnIndexByField(col.displayColumn));
                });
                if (index.length > 0)
                    colspan = ej.min(index);
                colspan = groupData.data.items.childLevels != null ? colspan + (groupData.data.items.childLevels + 1) : colspan;
            }
            return colspan;
        },
        _getCaptionColGroup: function (level) {
            var cloneColGroup = this.getGridHeaderTable().find("colgroup").clone();
            var colColl = cloneColGroup.find("col");
            var indentCol = colColl.length - this.model.columns.length;
            cloneColGroup.find("col:lt(" + indentCol + ")").remove();
            if (level > 0 && level != this.model.groupSettings.groupedColumns.length) {
                if (this.model.groupSettings.groupedColumns.length > 2 && level != this.model.groupSettings.groupedColumns.length - 1) {
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length - level; i++) {
                        cloneColGroup.prepend(this._getIndentCol());
                    }
                }
                else
                    cloneColGroup.prepend(this._getIndentCol());
            }
            return cloneColGroup;
        },
        _gridTemplate: function (templateId) {
            return $("#" + templateId).render(this.data);
        },
        _unboundTemplateRendering: function (unboundTemplateId) {
            return $("#" + unboundTemplateId).html();
        },
        _compiledDropDownTemplate: function (valueField, textField, colType, format) {
            var helpers = { _gridFormatting: this.formatting };
            $.views.helpers(helpers);
            var $select = ej.buildTag('select');
            var $option = ej.buildTag("option", ej.isNullOrUndefined(textField) ? "{{:text}}" : colType == "date" && format != null ? "{{:~_gridFormatting('" + format + "'," + textField + ")}}" : "{{:" + textField + "}}", {}, { value: ej.isNullOrUndefined(valueField) ? "{{:value}}" : "{{:" + valueField + "}}" });
            $select.append($option);
            return $.templates($select.html());
        },
        _getForeignKeyData: function (data) {
            var proxy = this;
            var column;
            for (i = 0; i < this.model.columns.length; i++) {
                if (this.model.columns[i].foreignKeyValue && this.model.columns[i].dataSource) {
                    var fieldName = ej.isNullOrUndefined(proxy.model.columns[i]["foreignKeyField"]) ? proxy.model.columns[i]["field"] : proxy.model.columns[i]["foreignKeyField"];
                    this.model.columns[i].dataSource.filter(function (col) {
                        var value = data[proxy.model.columns[i]["field"]];
                        var fValue = !isNaN(parseFloat(value)) && isFinite(value) ? parseFloat(value) : value;
                        if (col[fieldName] == fValue)
                            return column = col;
                    });
                }
            }
            return column;
        },

        _initCellEditType: function (columnCount, element) {
            if (this.model.columns[columnCount]["foreignKeyValue"])
                this.model.columns[columnCount]["columnEditType"] = "dropdownedit";
            if (ej.isNullOrUndefined(this.model.columns[columnCount]["columnEditType"]))
                this.model.columns[columnCount]["columnEditType"] = "stringedit";
            switch (this.model.columns[columnCount]["columnEditType"]) {
                case "stringedit":
                    element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{:" + this.model.columns[columnCount].field + "}}", id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field }));
                    break;
                case "booleanedit":
                    element.html('{{if ' + this.model.columns[columnCount].field + '}} <input class="e-field e-checkbox" type ="checkbox" id=' + this._id + this.model.columns[columnCount].field + ' name=' + this.model.columns[columnCount].field + ' checked="checked"></input>{{else}} <input class="e-field e-checkbox" type ="checkbox" id=' + this._id + this.model.columns[columnCount].field + ' name=' + this.model.columns[columnCount].field + ' > {{/if}}');
                    if (this.model.edit.editMode == "normal")
                        element.addClass("e-boolcell");
                    break;
                case "numericedit":
                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, { type: "text", value: "{{:" + this.model.columns[columnCount].field + "}}", id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field });
                    element.append($numericText);
                    break;
                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + this.model.columns[columnCount]["columnEditType"] + ' e-field', "", {}, { type: "text", value: "{{:" + this.model.columns[columnCount].field + "}}", id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field });
                    element.append($datePicker);
                    break;
                case "dropdownedit":
                    var currColumn = this.model.columns[columnCount];
                    if (ej.isNullOrUndefined(currColumn.dataSource)) {
                        var arrayOfDatas;
                        var query = new ej.Query().select(currColumn.field);
                        if (ej.isNullOrUndefined(this._dropDownManager))
                            arrayOfDatas = this._dataManager.executeLocal(query);
                        else
                            arrayOfDatas = this._dropDownManager.executeLocal(query);
                        var selectedItems = [];
                        var uniqueData = uniqueData = ej.dataUtil.mergeSort(ej.distinct(arrayOfDatas));
                        for (var index = 0; index < uniqueData.length; index++)
                            selectedItems.push({ text: uniqueData[index], value: uniqueData[index] });
                    }
                    else
                        selectedItems = currColumn.dataSource;
                    var dropDownTemplate;
                    var fieldName = ej.isNullOrUndefined(currColumn.foreignKeyField) ? currColumn.field : currColumn.foreignKeyField;
                    if (currColumn.foreignKeyValue)
                        dropDownTemplate = this._compiledDropDownTemplate(fieldName, currColumn.foreignKeyValue, currColumn.type, currColumn.format);
                    else
                        dropDownTemplate = this._compiledDropDownTemplate();
                    element.get(0).innerHTML = ["<select>", dropDownTemplate.render(selectedItems), "</select>"].join("");
                    element.find("select").prop({ id: this._id + currColumn.field, name: currColumn.field }).addClass("e-field e-dropdownlist");
                    break;
            }
        },
        addBulkEditTemplate: function () {
            //$("#" + this._id + "_BulkEditTemplate").remove();
            if (this.model.columns.length == 0)
                return;
            var $outerDiv = ej.buildTag('div', "", { display: "none" }, { id: this._id + "_BulkEditTemplate" }), i, columnCount, $innerDiv;
            for (i = 0, columnCount = this.model.columns.length; i < columnCount; i++) {
                if (!this.model.columns[i]["columnTemplate"] || !this.model.columns[i]["isUnBound"]) {
                    $innerDiv = ej.buildTag('div', "", {}, { id: this.model.columns[i].field + "_BulkEdit" });
                    this._initCellEditType(i, $innerDiv);
                    $outerDiv.append($innerDiv);
                }
                if (this.model.columns[i]["key"] === true) {
                    this._primaryKeys.push($.trim(this.model.columns[i].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
                if (this.model.columns[i]["isIdentity"] === true)
                    $innerDiv.find(".e-field").addClass("e-identity");
            }
            if ($outerDiv.children().length)
                this._bulkEditTemplate = $outerDiv;

        },
        addEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $tbody = ej.buildTag('tbody');
            var $tr = ej.buildTag('tr');
            var $td = ej.buildTag('td', "", {}, { colSpan: this.model.scrolling.frozenColumns > 0 ? this.model.scrolling.frozenColumns : this.model.columns.length });
            var $form = ej.buildTag('form', "", {}, { id: this._id + "EditForm", "class": "gridform" });
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: 0.25 });
            var $innerTbody = ej.buildTag('tbody');
            var $innerTr = ej.buildTag('tr');
            $tbody.append($tr);
            $tr.append($td);
            $td.append($form);
            var $colGroup = $(document.createElement('colgroup'));
            $form.append($table);
            $table.append($colGroup);
            $innerTbody.append($innerTr).appendTo($table);
            if (this.model.scrolling.frozenColumns > 0) {
                var $tbodyClone = $tbody.clone();
                $tbodyClone.find("td").first().prop("colSpan", this.model.columns.length - this.model.scrolling.frozenColumns);
            }
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                var $innerTd = ej.buildTag('td.e-rowcell');
                $innerTr.append($innerTd.get(0));
                if (ej.isNullOrUndefined(this.model.columns[columnCount]["isUnBound"]) && ej.isNullOrUndefined(this.model.columns[columnCount]["columnTemplate"])) {
                    this._initCellEditType(columnCount, $innerTd);
                } else if (this.model.columns[columnCount]["columnTemplate"]) {
                    var helpers = {};
                    helpers["_" + this._id + "ColumnTemplating"] = this._gridTemplate;
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + columnCount + "_Template").remove();
                    var scriptElement = this._createTemplateElement(this.model.columns[columnCount]);
                    $innerTd.addClass("e-templatecell").html("{{:~_" + this._id + "ColumnTemplating('" + scriptElement.id + "')}}");
                } else if (this.model.columns[columnCount]["isUnBound"]) {
                    var helpers = {};
                    helpers["_" + this._id + "UnboundTemplate"] = this._unboundTemplateRendering;
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + "_UnboundTemplate").remove();
                    divElement = this._createUnboundElement(this.model.columns[columnCount]);
                    $innerTd.addClass("e-unboundcell").addClass("e-" + this.model.columns[columnCount]["headerText"].replace(/\s+/g, "")).html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                    this.model.scrolling.frozenColumns > 0 && $innerTd.addClass("e-frozenunbound");
                    this._isUnboundColumn = true;
                }
                if (this.model.columns[columnCount]["textAlign"] != undefined)
                    $innerTd.css("text-align", this.model.columns[columnCount]["textAlign"]);
                this.model.columns[columnCount]["allowEditing"] == false && $innerTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                if (this.model.columns[columnCount]["key"] === true) {
                    $innerTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
                if (this.model.columns[columnCount]["isIdentity"] === true)
                    $innerTd.find(".e-field").addClass("e-identity");
                var $col = $(document.createElement('col'));
                if (this.model.columns[columnCount]["visible"] === false) {
                    $col.css("display", "none");
                    $innerTd.addClass("e-hide");
                }
                !this.model.groupSettings.groupedColumnShow && $innerTd.addClass("{{for ~groupedColumns}}" +
                    " {{if #data == '" + this.model.columns[columnCount]["field"] + "'}}e-hide{{/if}}" +
                    "{{/for}}") && $col.css("display", "none");
                $colGroup.append($col);
                if (columnCount == this.model.scrolling.frozenColumns - 1) {
                    $innerTr = $tbodyClone.find("tr").last();
                    $colGroup = $tbodyClone.find("colgroup");
                    $.templates(this._id + "_JSONFrozenEditingTemplate", $tbody.html());
                    $tbody = $tbodyClone;
                }
            }
            $.templates(this._id + "_JSONEditingTemplate", $tbody.html());
        },

        addDialogEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $tbody = ej.buildTag('div');
            var $form = ej.buildTag('form.gridform', "", {}, { id: this._id + "EditForm" });
            var $table = ej.buildTag('table', "", {}, { cellspacing: "14px" });
            var $innerTr, $labelTd, $valueTd, trElement, tdElement;
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                if (this.model.edit.editMode == "dialog") {
                    trElement = 'tr';
                    tdElement = 'td';
                }
                else trElement = tdElement = 'div';
                $innerTr = ej.buildTag(trElement);
                $labelTd = ej.buildTag(tdElement, "", { "text-align": "right" });
                $valueTd = ej.buildTag(tdElement, "", { "text-align": "left" }).addClass("e-rowcell");

                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
                if (this.model.columns[columnCount].headerText == undefined)
                    this.model.columns[columnCount].headerText = this.model.columns[columnCount].field;
                $labelTd.append("<label for='" + this.model.columns[columnCount].field + "'>" + this.model.columns[columnCount].headerText + "</label>");
                this._initCellEditType(columnCount, $valueTd);
                if (this.model.columns[columnCount]["key"] === true)
                    $valueTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                if (this.model.columns[columnCount]["visible"] === false)
                    $innerTr.addClass("e-hide");
                if (this.model.edit.editMode == "dialog") {
                    $form.append($table);
                    $table.append($innerTr)
                }
                else
                    $form.append($innerTr)
                $form.appendTo($tbody);
                if (this.model.columns[columnCount]["key"] === true) {
                    $valueTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
            }
            if (this.model.edit.editMode == "dialog") $form.append($table);
            $tbody = this.renderDiaglogButton($form, $tbody);
            $.templates(this._id + "_JSONDialogEditingTemplate", $tbody.html());
        },

        renderDiaglogButton: function (form, tbody) {
            var btnId;
            if (this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate")
                btnId = "EditExternalForm_";
            else if (this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate") {
                btnId = "InlineEditForm_";
                var inlineTitleBar = ej.buildTag("div", "", "", { id: this._id + "_inlineFormTitle", 'class': "e-inlineform-titlebar" });
                inlineTitleBar.appendTo(tbody);
            }
            else
                btnId = "EditDialog_";
            var savebtn = ej.buildTag('input', "", { 'margin-left': '30px' }, { type: "button", id: btnId + this._id + "_Save" });
            savebtn.ejButton({ cssClass: this.model.cssClass, rtl: this.model.rtl, text: this._getLocalizedLabels("SaveButton"), width: "100" });
            var cancelbtn = ej.buildTag('input', "", { 'margin-left': '19px', 'margin-right': '13px' }, { type: "button", id: btnId + this._id + "_Cancel" });
            cancelbtn.ejButton({ cssClass: this.model.cssClass, rtl: this.model.rtl, text: this._getLocalizedLabels("CancelButton"), width: "100" });
            var btnDiv = (this.model.edit.editMode != "dialog" && this.model.edit.editMode != "dialogtemplate") ? ej.buildTag('div', "", "", { 'class': "e-editform-btn" }) : ej.buildTag('div');
            btnDiv.append(savebtn);
            btnDiv.append(cancelbtn);
            form.appendTo(tbody);
            if (this.model.edit.editMode != "dialog" && this.model.edit.editMode != "dialogtemplate")
                btnDiv.appendTo(tbody);
            else
                form.append(btnDiv);
            return tbody;
        },

        addExternalDialogEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                if (this.model.columns[columnCount]["key"] === true) {
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
            }
            var $tbody = ej.buildTag('div', "", { 'display': 'none' });
            var $form = ej.buildTag('form.gridform', "", {}, { id: this._id + "EditForm" });
            var cloneElement;
            if (this.model.edit.editMode == "dialogtemplate" && this.model.edit.dialogEditorTemplateId != null)
                cloneElement = this.model.edit.dialogEditorTemplateId;
            else if (this.model.edit.editMode == "externalformtemplate" && this.model.edit.externalFormTemplateId != null)
                cloneElement = this.model.edit.externalFormTemplateId;
            else
                cloneElement = this.model.edit.inlineFormTemplateId;

            $form.html($(cloneElement).html());
            $tbody = this.renderDiaglogButton($form, $tbody);
            $.templates(this._id + "_JSONdialogTemplateMode", $tbody.html());
        },

        _buttonClick: function (e) {
            if (e.type == "close") {
                if (!this.model.isEdit)
                    return;
                this.model.isEdit = false;
                this.refreshToolBar();
                return;
            }
            if (e.keyCode !== undefined && e.keyCode != 13 || this.model == null)
                return true;
            if (this.model.edit.editMode == "dialog" || this.model.edit.editMode == "dialogtemplate") {
                if (e.target.id == "EditDialog_" + this._id + "_Save") {
                    if (this.element.ejGrid("sendSaveRequest").length !== undefined)
                        $("#" + this._id + "_dialogEdit").ejDialog("close");
                } else if (e.target.id == "EditDialog_" + this._id + "_Cancel") {
                    this.element.ejGrid("sendCancelRequest");
                    $("#" + this._id + "_dialogEdit").ejDialog("close");
                }
            }
            else if (this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate") {
                if ($(e.target).hasClass("e-form-toggle-icon")) {
                    this.element.ejGrid("sendCancelRequest");
                    $("#" + this._id + "_externalEdit").css("display", "none");
                }
                else {
                    if (e.target.id == "EditExternalForm_" + this._id + "_Save") {
                        if (this.element.ejGrid("sendSaveRequest").length !== undefined)
                            $("#" + this._id + "_externalEdit").css("display", "none");
                    } else if (e.target.id == "EditExternalForm_" + this._id + "_Cancel") {
                        this.element.ejGrid("sendCancelRequest");
                        $("#" + this._id + "_externalEdit").css("display", "none");
                    }
                }
            }
            else if (this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate") {
                if (e.target.id == "InlineEditForm_" + this._id + "_Save")
                    this.element.ejGrid("sendSaveRequest");
                else if (e.target.id == "InlineEditForm_" + this._id + "_Cancel")
                    this.element.ejGrid("sendCancelRequest");
            }
            else
                this.element.ejGrid("sendCancelRequest");
        },

        _headerCellgDragDrop: function () {
            var proxy = this;
            this.dragHeaderElement();
            var $droppableElements = this.element.children("div.e-groupdroparea");
            $droppableElements.ejDroppable({
                accept: $droppableElements,
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible"))
                        return;
                    var column = proxy.getColumnByHeaderText($.trim(ui.helper.text()));
                    ui.helper.remove();
                    if ($.inArray(column["headerText"], proxy._disabledGroupableColumns) != -1)
                        return;
                    if (!ej.isNullOrUndefined(column.field))
                        proxy.sendGroupingRequest(column.field);
                }
            });
        },

        _headerCellreorderDragDrop: function () {
            var proxy = this;
            this.dragHeaderElement();
            var $droppableElements = this.element.find(".e-headercelldiv");
            $droppableElements.ejDroppable({
                accept: $droppableElements,
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible"))
                        return;
                    var column = proxy.getColumnByHeaderText($.trim(ui.helper.text()));
                    ui.helper.remove();
                    if ($.inArray(column["headerText"], proxy._disabledGroupableColumns) != -1)
                        return;
                    if (!ej.isNullOrUndefined(column.field)) {
                        var toColumn = proxy.getColumnByHeaderText($.trim($(event.dropTarget).text()));
                        proxy.sendReorderingRequest(column.field, toColumn.field);

                    }
                }
            });
        },

        dragHeaderElement: function () {
            var proxy = this;
            var $dragableElements = this.element.children("div.e-gridheader").find("th.e-headercell").not(".e-headertemplate");
            var $visualElement = ej.buildTag('div.e-cloneproperties e-grid', "", { 'height': '20px', 'z-index': 2 });
            //header element drag
            $dragableElements.ejDraggable({
                cursorAt: { top: -35, left: -2 },
                helper: function (event, ui) {
                    if (proxy.element.find(".e-dragclone").length > 0) proxy.element.find(".e-dragclone").remove();
                    var $th = $(event.sender.target).hasClass("e-headercell") ? $(event.sender.target) : $(event.sender.target).closest("th");
                    return $visualElement.html($th.text()).clone().width($th.outerWidth() + 2).height($th.height() + 2).css({ "font-size": parseInt(($th.height() + 3) / 2) }).addClass("e-dragclone").appendTo(proxy.element);
                },
                dragStart: function (args) {
                    var target = args.target;
                    var data = { target: target, draggableType: "headercell" };
                    if (proxy._resizer != null && proxy._resizer._expand) {
                        $(".e-dragclone").remove();
                        return false;
                    }
                    proxy._dragActive = true;
                    var $target = proxy.element.find(".e-groupdroparea");
                    //proxy.model.groupSettings.groupedColumns.length == 0 && proxy.model.enableEffects && $target.animate({ height: "30px" }, 150, function () {
                    //    proxy.model.groupSettings.groupedColumns.length == 0 && $target.html(proxy._getLocalizedLabels("GroupDropArea"));
                    //    $target.dequeue().css("height", "30px");
                    //});
                    proxy.expandGroupDropArea();
                    if (proxy._trigger("dragStart", data))
                        return false;
                },
                drag: function (args) {
                    var $target = $(args.target);
                    var data = { target: $target, draggableType: "headercell" };
                    if (proxy._trigger("drag", data))
                        return false;

                    if ($target.hasClass('e-headercelldiv') && proxy.model.allowReordering) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                        proxy.getGridHeaderTable().find(".e-reorderindicate").removeClass("e-reorderindicate");
                        //proxy._showImg(args.event.originalEvent);
                        $target.parent().addClass("e-reorderindicate");
                    }
                    if ($target.hasClass('e-groupdroparea') || $target.closest('.e-groupdroparea').length) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                    } else
                        document.body.style.cursor = 'not-allowed';
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        var $target = $(args.target);
                        var data = { target: $target, draggableType: "headercell" };
                        proxy._trigger("dragStop", data);
                        proxy.element.find(".e-groupdroparea").removeClass("e-hover");
                        proxy.getGridHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive").removeClass("e-active");
                        $(".e-dragclone").remove();
                        proxy._dragActive = false;
                        var $target = proxy.element.find(".e-groupdroparea");
                        proxy.getGridHeaderTable().find(".e-reorderindicate").removeClass("e-reorderindicate");
                        //proxy.model.groupSettings.groupedColumns.length == 0 && proxy.model.enableEffects && $target.animate({ height: "0px" }, 150, function () {
                        //    if (proxy.model.groupSettings.groupedColumns.length == 0) {
                        //        $target.html("");
                        //        $target.dequeue().css("height", "auto");
                        //    }
                        //});
                        proxy.collapseGroupDropArea();
                        document.body.style.cursor = '';
                        $(proxy._Indicator).css('display', 'none');
                    }
                }
            });
        },
        _initIndicators: function () {
            var indicatorId = this._id + "_ColumnDropIndicator";
            if ($("#" + indicatorId).length)
                $("#" + indicatorId).remove();
            this._Indicator = document.createElement("DIV");
            $(this._Indicator).attr('id', indicatorId).addClass("e-columndropindicator").addClass("e-dropAcceptor").appendTo(document.body);
            $(this._Indicator).css({ "display": "none" });

        },
        _showImg: function (e) {
            this._locationHelper(e);
            cell = (e.target.tagName == 'DIV' && e.target.parentNode.tagName == 'TH') ? (e.target.parentNode) : ((e.target.tagName == 'TH') ? (e.target) : (null));
            if (cell.cellIndex >= 0) {
                var jquerybounds = $(cell).offset();
                var ltemp = (navigator.userAgent.search("Mozilla") >= 0 && this._groupedColumns.length > 0) ? 3 : 4;
                var ttemp = (navigator.userAgent.search("Mozilla") >= 0 && this._groupedColumns.length > 0) ? 2 : 1;
                $(this._Indicator).css('left', parseInt(jquerybounds.left - ltemp)).css('position', 'absolute').css('top', parseInt(jquerybounds.top + ttemp)).css('display', '').css('z-index', this._maxZindex() + 1);
            }
        },

        _maxZindex: function () {
            var maxZ = 1;
            maxZ = Math.max.apply(null, $.map($('body *'), function (e, n) {
                if ($(e).css('position') == 'absolute')
                    return parseInt($(e).css('z-index')) || 1;
            })
            );
            if (maxZ == undefined || maxZ == null)
                maxZ = 1;
            return maxZ;
        },

        _locationHelper: function (evt) {
            this._x = document.all ? evt.clientX : document.layers ? evt.x : evt.clientX;
            this._y = document.all ? evt.clientY : document.layers ? evt.y : evt.clientY;
        },

        /**
			* It is used to re-order the column based on the given index in grid control.
			* @return jQuery
			* @param {number} fromIndex Pass the column index position from which the column needs to be changed
			* @param {number} toIndex Pass the column index position to which the column needs to be changed
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.set_dropColumn(4, 2); // Reorders the column based on the given index
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Reorders the column based on the given index
			* $("#Grid").ejGrid("set_dropColumn", 4, 2);
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        set_dropColumn: function (from, to) {
            if (this.model.allowReordering && from != to) {
                this.model.columns.splice(to, 0, this.model.columns.splice(from, 1)[0]);
                var columns = this.model.columns;
                var $headerCell = this.getGridHeaderTable().find(".e-headercelldiv");
                var $attributeCollection = { "attributes": [] };
                var $tmp = {};
                var fromIndex = from < to ? from : to;
                var toIndex = from < to ? to : from;
                var item = 0;
                for (var i = fromIndex; i <= toIndex; i++) {
                    var cellAttributes = $headerCell[i].attributes;
                    for (var j = 0; j < cellAttributes.length; j++) {
                        $tmp[cellAttributes[j].name] = cellAttributes[j].value;
                    }
                    $attributeCollection.attributes.push($tmp);
                    $tmp = $attributeCollection.attributes[item];
                    for (var attr in $tmp) {
                        $headerCell[i].removeAttribute(attr);
                    }
                    $tmp = {};
                    item++;
                }
                var spliceFrom = from < to ? $attributeCollection.attributes.length - 1 : 0;
                var spliceTo = from < to ? 0 : $attributeCollection.attributes.length - 1;
                $attributeCollection.attributes.splice(spliceFrom, 0, $attributeCollection.attributes.splice(spliceTo, 1)[0]);
                //Add attributes
                var j = 0;
                for (var i = fromIndex; i <= toIndex; i++) {

                    var $attrColl = $attributeCollection.attributes[j];
                    for (var attr in $attrColl) {
                        $headerCell[i].setAttribute(attr, $attrColl[attr]);
                    }
                    j++;
                }
                this._fieldColumnNames = this._headerColumnNames = [];
                for (var count = 0; count < columns.length; count++) {
                    this._fieldColumnNames[columns[count].headerText] = columns[count].field;
                    this._headerColumnNames[columns[count].field] = columns[count].headerText;
                    $($headerCell[count]).text(columns[count].headerText);
                }
                var args = {};
                args.requestType = ej.Grid.actions.Reorder;
                this.addInitTemplate();
                this.sendDataRenderingRequest(args);
            }
        },

        _groupHeaderCelldrag: function () {
            //grouped header cell drag.
            var $visualElement = ej.buildTag('div.e-cloneproperties e-grid', "", { 'height': '20px', 'z-index': 2 });
            var proxy = this;
            var $groupedHeaderCells = this.element.children(".e-groupdroparea").find(".e-groupheadercell");
            $groupedHeaderCells.ejDraggable({
                cursorAt: { top: -35, left: -2 },
                helper: function (event, ui) {
                    var $div = $(event.sender.target).closest(".e-grid-icon");
                    return $visualElement.html($(event.sender.target).closest(".e-groupheadercell").text()).clone().width($div.width() + 2).height($div.height() + 2).addClass("e-dragclone").appendTo(proxy.element);
                },
                dragStart: function (args) {
                    var target = args.target;
                    var data = { target: target, draggableType: "groupheadercell" };
                    if (proxy._trigger("dragStart", data))
                        return false;
                },
                drag: function (args) {
                    $(".Sibling").remove();
                    var $target = $(args.target);
                    var data = { target: $target, draggableType: "groupheadercell" };
                    if (proxy._trigger("drag", data))
                        return false;
                    if ($target.closest('div.e-gridcontent').length) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                    } else
                        document.body.style.cursor = 'not-allowed';
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        var $target = $(args.target);
                        var data = { target: $target, draggableType: "groupheadercell" };
                        proxy._trigger("dragStop", data);
                        $(".e-dragclone").remove();
                        document.body.style.cursor = '';
                    }
                }
            });

            //grid content drop
            var $contentDroppableElements = this.element.children(".e-gridcontent");
            $contentDroppableElements.ejDroppable({
                accept: proxy.element.children("div.e-groupdroparea").find(".e-groupheadercell"),
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible") || !ui.draggable.hasClass("e-groupheadercell"))
                        return;
                    var field = proxy.getFieldNameByHeaderText($.trim(ui.helper.text()));
                    ui.helper.remove();
                    if (!ej.isNullOrUndefined(field))
                        proxy.sendUngroupingRequest(field);


                }
            });
        },

        /**
			* To render the grid  
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.renderGrid (); // It renders the grid.
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // It renders the grid.
			* $("#Grid").ejGrid("renderGrid");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        renderGrid: function () {
            this._renderGridContent().insertAfter(this.element.children(".e-gridheader"));
            this.model.allowResizeToFit && this.setWidthToColumns();
            this.model.allowGrouping && this._groupHeaderCelldrag();
            this.model.allowSummary = this.model.summaryRows.length > 0 || this.model.allowSummary;
            if (this.model.allowSummary)
                this._renderGridFooter().insertAfter(this.getGridContent());
            if (this.model.edit.allowEditing || this.model.edit.allowAdding) {
                if (this.model.edit.editMode == "dialog" || this.model.edit.editMode == "dialogtemplate")
                    this.element.append(this._renderDialog());
                else if (this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate")
                    this.element.append(this._renderExternalForm());
            }
            this._initialEndRendering();
            if (this.model.scrolling.frozenColumns > 0 && !this.model.allowScrolling) {
                this.getGridContent().remove();
                this.getGridHeaderTable().eq(1).remove();
                this._alertDialog.find(".e-content").text(this._getLocalizedLabels("frozenColumnsScrollAlert"));
                this._alertDialog.ejDialog("open");
                return;
            }
            this.model.scrolling.allowVirtualScrolling && this._createPagerStatusBar();
            this._getRowHeights();
            this.model.allowScrolling && this._renderScroller();
            if (this.model.scrolling.allowVirtualScrolling) {
                this._refreshVirtualContent();
                this.getGridContent().first().ejScroller("refresh");
                if (this.getGridContent().ejScroller("isVScroll"))
                    this.element.find(".e-gridheader").addClass("e-scrollcss");
                else
                    this.element.find(".e-gridheader").removeClass("e-scrollcss");
            }
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar" && !this.model.allowPaging && !this.model.scrolling.allowVirtualScrolling)
                this._createPagerStatusBar();
        },

        _renderGroupDropArea: function () {
            if (!this.model.groupSettings.showAnimateButton)
                this.model.enableEffects = this.model.groupSettings.showAnimateButton;
            var dragLabel = this.model.enableEffects ? "" : this._getLocalizedLabels("GroupDropArea");
            var $div = ej.buildTag("div.e-groupdroparea", dragLabel);
            this.model.groupSettings.showAnimateButton && $div.append(ej.buildTag("div.e-animatebutton e-icon").addClass(this.model.enableEffects ? "e-animatebuttondown e-gdownarrow" : "e-animatebuttonup e-guparrow"));
            return $div;
        },

        _renderToolBar: function () {
            var $div = ej.buildTag('div.e-gridtoolbar', "", {}, { id: this._id + "_toolbarItems" });
            var $ul = ej.buildTag("ul");
            (!ej.isNullOrUndefined(this.model.toolBar.toolBarItems) && this.model.toolBar.toolBarItems.length) && this._renderLi($ul);
            $div.append($ul);
            var $customUl = ej.buildTag("ul");
            $div.append($customUl);
            (!ej.isNullOrUndefined(this.model.toolBar.customToolbarItems) && this.model.toolBar.customToolbarItems.length) && this._renderCustomLi($customUl);
            var model = {};
            model.click = this._toolBarClick;
            model.cssClass = this.model.cssClass;
            model.rtl = this.model.rtl;
            model.itemSeparator = false;
            $div.ejToolbar(model);
            $div.ejToolbar("disableItem", this._disabledToolItems);
            this._disabledToolItems = $();
            return $div;
        },
        _renderCustomLi: function ($ul) {
            for (var i = 0; i < this.model.toolBar.customToolbarItems.length; i++) {
                var $li = ej.buildTag("li", "", {}, { title: this.model.toolBar.customToolbarItems[i] });
                switch (typeof this.model.toolBar.customToolbarItems[i]) {
                    case "string":
                        var $item = ej.buildTag("a.e-toolbaricons e-icon", "", {}).addClass(this.model.toolBar.customToolbarItems[i]);
                        break;
                    case "object":
                        $li.attr("title", this.model.toolBar.customToolbarItems[i]["templateId"].replace("#", ""));
                        var $item = $(this.model.toolBar.customToolbarItems[i]["templateId"]).hide().html();
                        break;
                }
                $li.html($item);
                $ul.append($li);
            }
        },
        _renderLi: function ($ul) {
            for (var i = 0; i < this.model.toolBar.toolBarItems.length; i++) {
                var $li = ej.buildTag("li", "", {}, { id: this._id + "_" + this.model.toolBar.toolBarItems[i], title: this._getLocalizedLabels(this.model.toolBar.toolBarItems[i].slice(0, 1).toUpperCase() + this.model.toolBar.toolBarItems[i].slice(1)), "tabindex": "0" });
                this._renderLiContent($li, this.model.toolBar.toolBarItems[i]);
                $ul.append($li);
            }
        },
        _renderLiContent: function ($li, item) {
            var $a, $input;
            switch (item) {
                case "add":
                    $a = ej.buildTag("a.e-addnewitem e-toolbaricons e-icon e-addnew", "", {});
                    break;
                case "edit":
                    $a = ej.buildTag("a.e-edititem e-toolbaricons e-icon e-edit", "", {});
                    break;
                case "delete":
                    $a = ej.buildTag("a.e-deleteitem e-toolbaricons e-icon e-delete", "", {});
                    break;
                case "update":
                    $a = ej.buildTag("a.e-saveitem e-toolbaricons e-disabletool e-icon e-save", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;
                case "cancel":
                    $a = ej.buildTag("a.e-cancel e-toolbaricons e-disabletool e-icon e-gcancel", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;
                case "search":
                    $input = ej.buildTag("input.e-ejinputtext", "", {}, { type: "text" });
                    $a = ej.buildTag("a.e-searchitem e-toolbaricons e-disabletool e-icon e-searchfind", "", {});
                    $li.append($input);
                    this.model.allowSearching = true;
                    break;
            }
            $li.append($a);

        },
        _renderGridHeaderInternalDesign: function (columns) {
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: "0.25px", role: "grid" });
            var $thead = ej.buildTag('thead');
            var $tbody = ej.buildTag('tbody.e-hide');
            var $columnHeader = ej.buildTag('tr.e-columnheader');
            var $colGroup = $(document.createElement('colgroup'));
            var $rowBody = $(document.createElement('tr'));
            if (this.model.detailTemplate) {
                $columnHeader.append(ej.buildTag('th.e-headercell e-detailheadercell', '<div></div>'));
                $colGroup.append(this._getIndentCol());
            }
            for (var columnCount = 0; columnCount < columns.length; columnCount++) {
                var $headerCell = ej.buildTag('th.e-headercell e-default', "", {}, { role: "columnheader" });
                var bodyCell = document.createElement('td');
                var $headerCellDiv = ej.buildTag('div.e-headercelldiv', columns[columnCount]["headerText"] === undefined ? columns[columnCount]["headerText"] = columns[columnCount]["field"] : columns[columnCount]["headerText"], {}, { "ej-mappingname": columns[columnCount]["field"] });
                $headerCell.append($headerCellDiv);
                if (this.model.allowFiltering && this.model.filterSettings.filterType == "menu" &&
                    (columns[columnCount]["allowFiltering"] == undefined ||
                    columns[columnCount]["allowFiltering"] === true))
                    $headerCell.append(ej.buildTag('div.e-filtericon e-icon e-filterset'));
                var col = document.createElement('col');
                $rowBody.append(bodyCell);
                $columnHeader.append($headerCell);
                $colGroup.append(col);
                if (columns[columnCount]["visible"] === false)
                    $headerCell.addClass("e-hide") && $(col).css("display", "none") && this._hiddenColumns.push(columns[columnCount].headerText);
                else {
                    this._visibleColumns.push(columns[columnCount].headerText);
                    columns[columnCount]["visible"] = true;
                }
                columns[columnCount]["allowSorting"] === false && this._disabledSortableColumns.push(columns[columnCount].headerText);
                columns[columnCount]["allowGrouping"] === false && this._disabledGroupableColumns.push(columns[columnCount].headerText);
                if (!ej.isNullOrUndefined(columns[columnCount]["headerTemplateId"])) {
                    $headerCellDiv.addClass(this.model.columns[columnCount]["cssClass"]);
                    if (!ej.isNullOrUndefined(this.model.columns[columnCount]["headerTemplateId"]))
                        $headerCellDiv.html($(this.model.columns[columnCount]["headerTemplateId"]).hide().html()).parent().addClass("e-headertemplate");
                    var index = $.inArray(this.model.columns[columnCount].headerText, this._disabledGroupableColumns);
                    index == -1 && this._disabledGroupableColumns.push(this.model.columns[columnCount].headerText);
                }
                if (this.model.groupSettings.toggleGrouping && $.inArray(columns[columnCount].headerText, this._disabledGroupableColumns) == -1) {
                    if ($.inArray(columns[columnCount].field, this.model.groupSettings.groupedColumns) != -1)
                        $headerCellDiv.append(this._getToggleButton().addClass("e-toggleungroup"));
                    else
                        $headerCellDiv.append(this._getToggleButton().addClass("e-togglegroup"));
                }
                this._columnsWidthCollection.push(columns[columnCount]["width"]);
                if (columns[columnCount]["width"] == undefined && this.model.commonWidth !== undefined)
                    this._columnsWidthCollection[columnCount] = this.model.commonWidth;
                this._fieldColumnNames[columns[columnCount].headerText] = columns[columnCount].field;
                this._headerColumnNames[columns[columnCount].field] = columns[columnCount].headerText;
                this.model.allowFiltering && ej.Grid.filterType.Menu == this.model.filterSettings.filterType && $headerCell.addClass("e-headercellfilter");
            }
            $thead.append($columnHeader);
            $tbody.append($rowBody);
            $table.append($colGroup).append($thead).append($tbody);
            return $table;
        },
        _renderGridHeader: function () {
            var $div = ej.buildTag('div.e-gridheader'), temp, $frozenDiv, $movableDiv;
            var $innerDiv = ej.buildTag('div');
            if (this.model.allowScrolling)
                $innerDiv.addClass("e-headercontent");
            this.setGridHeaderContent($div);
            this._hiddenColumns = [];
            this._columnsWidthCollection = [];
            this._visibleColumns = [];
            this._disabledGroupableColumns = [];
            this._fieldColumnNames = {};
            this._headerColumnNames = {};
            if (this.model.scrolling.frozenColumns > 0) {
                $frozenDiv = ej.buildTag("div.e-frozenheaderdiv", this._renderGridHeaderInternalDesign(this.model.columns.slice(0, this.model.scrolling.frozenColumns)));
                $movableDiv = ej.buildTag("div.e-movableheader", ej.buildTag("div.e-movableheaderdiv", this._renderGridHeaderInternalDesign(this.model.columns.slice(this.model.scrolling.frozenColumns))));
                $innerDiv.append($frozenDiv).append($movableDiv);
            }
            else
                $innerDiv.append(this._renderGridHeaderInternalDesign(this.model.columns));
            $div.html($innerDiv);
            this.setGridHeaderTable(this.getGridHeaderContent().find(".e-table"));
            return $div;
        },
        _renderGridFooter: function () {
            if (this.model.summaryRows.length > 0) {
                var _$gridFooter = ej.buildTag("div.e-gridfooter");
                var $table = ej.buildTag("table.e-gridsummary", "", {}, { cellspacing: "0.25px" });
                this.setGridFooterContent(_$gridFooter);
                if (this.model.scrolling.frozenColumns > 0) {
                    var $frozenFooterDiv = ej.buildTag("div.e-frozenfooterdiv"), $movableFooter = ej.buildTag("div.e-movablefooter")
                        , $tableClone = $table.clone(), $movableFooterDiv = ej.buildTag("div.e-movablefooterdiv");
                    $movableFooter.append($movableFooterDiv);
                    $table.append(this.getGridHeaderTable().first().find('colgroup').clone());
                    $tableClone.append(this.getGridHeaderTable().last().find('colgroup').clone());
                    $frozenFooterDiv.append($table);
                    $movableFooterDiv.append($tableClone);
                    this.setGridFooterTable($table.add($tableClone));
                    this._createSummaryRows(this.getGridFooterTable());
                    _$gridFooter.append($frozenFooterDiv.add($movableFooter));
                    _$gridFooter.find(".e-frozenfooterdiv").outerWidth(this.getGridHeaderContent().find(".e-frozenheaderdiv").width())
                          .end().find(".e-movablefooterdiv").outerWidth(this.getGridContent().find(".e-movablecontentdiv").width());
                }
                else {
                    $table.append(this.getGridHeaderTable().find('colgroup').clone());
                    this.setGridFooterTable($table);
                    this._createSummaryRows(this.getGridFooterTable());
                    _$gridFooter.append($table);
                }
                return _$gridFooter;
            } else
                throw "summary row collection is missing";
        },
        _createSummaryRows: function (table, summaryData) {
            var col = table.find("col");
            if (table.find("tbody").length > 0)
                table.find("tbody").remove();
            var $tBody = ej.buildTag('tbody'), proxy = this, $tBodyClone = $tBody.clone();
            var summaryCol = this.model.summaryRows;
            if (!ej.isNullOrUndefined(summaryData) && this._isCaptionSummary)
                summaryCol = this._captionSummary();

            $.each(summaryCol, function (indx, row) {
                if (row.showTotalSummary === false && ej.isNullOrUndefined(summaryData)) return true;
                var $tr = ej.buildTag('tr');
                var $cells = proxy.getGridHeaderTable().find('td').clone().addClass("e-summaryrow");
                $cells.first().html(row.title);
                proxy._hideSummaryColumn($cells, col);
                if (proxy.model.scrolling.frozenColumns > 0) {
                    var $trClone = $tr.clone();
                    $tBody.append($tr.append($cells.slice(0, proxy.model.scrolling.frozenColumns)));
                    $tBodyClone.append($trClone.append($cells.slice(proxy.model.scrolling.frozenColumns)));
                }
                else
                    $tBody.append($tr.append($cells));
                $.each(row.summaryColumns, function (cindx, col) {
                    var value = proxy._getSummaryValues(col, summaryData), prefix = col.prefix ? col.prefix : "";
                    var index = proxy.getColumnIndexByField(col.displayColumn), suffix = col.suffix ? col.suffix : "";
                    $($cells[index]).html(prefix + (col.format ? proxy.formatting(col.format, value) : value) + suffix).css("text-align", proxy.model.columns[index].textAlign);
                });
            });
            if (this.model.scrolling.frozenColumns > 0) {
                table.first().append($tBody);
                table.last().append($tBodyClone);
            }
            else
                table.append($tBody);
        },
        _getSummaryValues: function (summaryCol, summaryData) {
            var $value, jsonData;
            if (!ej.isNullOrUndefined(summaryData))
                jsonData = summaryData;
            else if (this.model.filterSettings.filteredColumns.length > 0)
                jsonData = this._filteredRecords;
            else
                jsonData = this._dataSource();
            var dbMgr = jsonData instanceof ej.DataManager ? jsonData : ej.DataManager(jsonData);
            switch (summaryCol.summaryType) {
                case ej.Grid.summaryType.Maximum:
                    var obj = ej.max(jsonData, summaryCol.dataMember);
                    $value = obj[summaryCol.dataMember];
                    break;
                case ej.Grid.summaryType.Minimum:
                    var obj = ej.min(jsonData, summaryCol.dataMember);
                    $value = obj[summaryCol.dataMember];
                    break;
                case ej.Grid.summaryType.Average:
                    $value = ej.avg(jsonData, summaryCol.dataMember);
                    break;
                case ej.Grid.summaryType.Sum:
                    $value = ej.sum(jsonData, summaryCol.dataMember);
                    break;
                case ej.Grid.summaryType.Count:
                    $value = jsonData.length;
                    break;
                case ej.Grid.summaryType.TrueCount:
                    var predicate = ej.Predicate(summaryCol.dataMember, "equal", true);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.Grid.summaryType.FalseCount:
                    var predicate = ej.Predicate(summaryCol.dataMember, "equal", false);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.Grid.summaryType.Custom:
                    $value = summaryCol.customSummaryValue;
                    break;
            }
            return $value;
        },
        _renderDialog: function () {
            var $dialog = ej.buildTag("div.e-dialog e-dialog-content e-shadow e-widget-content", "", { display: "none" }, { id: this._id + "_dialogEdit" });
            return $dialog;
        },

        _renderExternalForm: function () {
            var $externalform = ej.buildTag("div", "", { display: "none" }, { id: this._id + "_externalEdit", 'class': "e-form-container" });
            var $eformHeader = ej.buildTag("div", "", "", { id: this._id + "_eFormHeader", 'class': "e-form-titlebar" });
            var $eformTitle = ej.buildTag("span", "", "", { 'class': "e-form-title" });
            var $eformToggleBtn = ej.buildTag("div", "", "", { id: this._id + "_eFormToggleBtn", 'class': "e-form-togglebtn" });
            var $eformToggleIcon = ej.buildTag("span", "", "", { 'class': "e-form-toggle-icon e-icon" });
            $eformToggleBtn.append($eformToggleIcon);
            $eformHeader.append($eformTitle).append($eformToggleBtn);

            var $eformContent = ej.buildTag("div", "", "", { id: this._id + "_eFormContent", 'class': "e-form-content" });
            var $eform = ej.buildTag("div", "", "", { id: this._id + "_externalForm", 'class': "e-externalform" });
            var $contentOuterDiv = ej.buildTag("div", "", "", { 'class': "e-externalformedit" });
            $eform.append($contentOuterDiv);
            $eformContent.append($eform);
            return $externalform.append($eformHeader).append($eformContent);;
        },

        _renderGridContent: function () {
            var $div = ej.buildTag('div.e-gridcontent');
            var $innderDiv = ej.buildTag('div');
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: "0.25px" });
            var $tbody = $(document.createElement('tbody'));
            $table.append(this.getGridHeaderTable().find('colgroup').clone()).append($tbody);
            $innderDiv.html($table);
            $div.html($innderDiv);
            this.setGridContentTable($table);
            this.setGridContent($div);
            $table.attr("role", "grid");
            if (this._dataSource() == null) {
                var $emptyTd = ej.buildTag('td', this._getLocalizedLabels("EmptyRecord"), {}, { colSpan: this.model.columns.length });
                $tbody.append($(document.createElement("tr")).append($emptyTd));
                this.setWidthToColumns();
            } else {
                var args = {};
                if (this.model.groupSettings.groupedColumns.length) {
                    if (this._initialRender)
                        args.columnName = this.model.groupSettings.groupedColumns[this.model.groupSettings.groupedColumns.length - 1];
                    args.requestType = ej.Grid.actions.Grouping;
                } else
                    args.requestType = ej.Grid.actions.Refresh;
                this.sendDataRenderingRequest(args);
            }
            if (this._isCaptionSummary && args.requestType == "grouping" && this.model.groupSettings.groupedColumns.length > 1) {
                var colgroup = this.getGridContentTable().find(".e-table").not(".e-recordtable").children("colgroup");
                var $cols1 = $(this.getGridContentTable().find(".e-recordtable")[0]).children("colgroup").find("col");
                for (i = 0; i < colgroup.length; i++) {
                    var colCount = $(colgroup[i]).find("col").length;
                    $(colgroup[i]).find("col:gt(" + (colCount - $cols1.length - 1) + ")").remove();
                    $(colgroup[i]).append($cols1.clone());
                }
            }
            return $div;
        },

        _refreshVirtualContent: function (currentPage) {
            var rowHeight = this.getGridRowHeight();
            var recordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            if (currentPage != null) {
                this._currentPage(currentPage);
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
            }
            var currentIndex = this.getCurrentIndex();
            var tbody = this.getGridContentTable()[0].tBodies[0];
            if (this.getCurrentIndex() > 0) {
                var virtualTRTop = document.createElement("tr");
                $(virtualTRTop).addClass("e-virtualrow").css("height", rowHeight * currentIndex).prependTo(tbody);
            } if (currentIndex + this.model.pageSettings.pageSize <= recordsCount) {
                var virtualTRBottom = document.createElement("tr");
                $(virtualTRBottom).addClass("e-virtualrow").css("height", rowHeight * (recordsCount - (currentIndex + this.model.pageSettings.pageSize))).appendTo($(tbody));
            }
            this.virtualLoadedPages = new Array();
            this.virtualLoadedPages.push(currentIndex >= 0 ? currentIndex : 0);
            var focusTR = $(tbody).find('tr:not(.e-virtualrow)').attr('name', currentIndex >= 0 ? currentIndex : 0)[0];
            if (focusTR && focusTR.previousSibling && ($(focusTR.previousSibling).hasClass("e-virtualrow") || focusTR.previousSibling.offsetTop > (currentIndex * this.getGridContent().height()))) {
                this.getGridContent().children("div").first().scrollTop(this.getGridContent().find(".content").scrollTop() - (this.getGridContent().find(".content").scrollTop() - focusTR.offsetTop));
                this._isVirtualRecordsLoaded = true;
            }
        },
        refreshScrollerEvent: function () {
            var proxy = this;
            this.getGridContent().find(".e-content,.e-movablecontent").scroll(ej.proxy(function (e) {
                if (this.model.scrolling.targetPane)
                    this.getGridHeaderContent().find(".e-movableheader").scrollLeft($(e.currentTarget).scrollLeft());
                else
                    this.getGridHeaderContent().find("div").first().scrollLeft($(e.currentTarget).scrollLeft());
                if (this.model.scrolling.frozenRows > 0 && this.model.edit.editMode.indexOf("inlineform") != -1 && this.model.isEdit) {
                    var scrollTop = e.target.scrollTop;
                    this.getGridContent().find(".e-content").scrollTop(0);
                    this.getScrollObject().scrollY(this.getScrollObject().model.scrollTop + scrollTop, true);
                }
            }, this));
            this.element.find(".e-gridheader").find(".e-headercontent,.e-movableheader").scroll(ej.proxy(function (e) {
                var $currentTarget = $(e.currentTarget);
                if (this.model.scrolling.targetPane) {
                    this.getGridContent().find(".e-movablecontent").scrollLeft($currentTarget.scrollLeft());
                    this.model.allowSummary && this.getGridFooterContent().find(".e-movablefooter").scrollLeft($currentTarget.scrollLeft());;
                }
                else {
                    this.model.allowSummary && this.getGridFooterContent().scrollLeft($currentTarget.scrollLeft());
                    this.getGridContent().find(".e-content").first().scrollLeft($currentTarget.scrollLeft());
                }
            }, this));
        },
        _getTopRow: function (offsetTop) {
            var currentTopRow = this.model.scrolling.frozenRows, i = 0;
            if (offsetTop > 10) {
                for (var i = 0; i < this._rowHeightCollection.length; i++) {
                    if (this._rowHeightCollection[i] > offsetTop) {
                        currentTopRow = this.model.scrolling.frozenRows + i - 1;
                        break;
                    }
                }
            }
            return { imaginaryIndex: currentTopRow, actualIndex: i };
        },
        _showHideRow: function (from, to, action, scrollPosition) {
            var rows = this.getGridRows();
            if (this.model.scrolling.frozenColumns > 0)
                $(rows[0]).slice(from, to).add($(rows[1]).slice(from, to).toArray())[action]();
            else
                $(rows).slice(from, to)[action]();
            this._currentTopFrozenRow = action == "show" ? from : to;
            this.getScrollObject().changevHandlerPosition(scrollPosition);
        },
        _scroll: function (args) {
            if (args.scrollData != null && args.scrollData.dimension != "width") {
                args.cancel = true;
                var rows = this.getGridRows(), indexes = this._getTopRow(args.scrollTop), currentTopRow = indexes.imaginaryIndex, frozenRows;
                if (currentTopRow > this._currentTopFrozenRow)
                    this._showHideRow(this.model.scrolling.frozenRows, currentTopRow, "hide", this._rowHeightCollection[indexes.actualIndex]);
                else if (currentTopRow < this._currentTopFrozenRow)
                    this._showHideRow(currentTopRow, this._currentTopFrozenRow + 1, "show", this._rowHeightCollection[indexes.actualIndex]);
                args.model.scrollTop = args.scrollTop;
            }
            else {
                if (this.getScrollObject()._v != null)
                    this.getScrollObject()._v.skipChange = true;
            }
        },
        _getRowHeights: function () {
            var trs = this.getGridRows();
            if (trs !== null) {
                this._rowHeightCollection = [];
                if ((this.model.scrolling.frozenColumns > 0 && trs[0] !== undefined) || (typeof trs[0].item !== "undefined" && typeof trs[0].length == "number"))
                    trs = trs[0];
                for (var i = 0 ; i < trs.length ; i++) {
                    this._rowHeightCollection[i] = trs[i].offsetTop;
                }
            }
            return this._rowHeightCollection;
        },
        _initFrozenRows: function () {
            var gridRows = this.getGridRows();
            if (!this.model.currentViewData || this.model.currentViewData.length == 0)
                return;
            if (this.model.scrolling.frozenRows > 0 && gridRows != null) {
                this.model.scrolling["scroll"] = $.proxy(this._scroll, this);
                this.getGridContent().find(".e-frozeny").removeClass("e-frozeny")
                    .end().find(".e-frozenrow").removeClass("e-frozenrow");
                if (!ej.isNullOrUndefined(gridRows[0][this.model.scrolling.frozenRows - 1]) && !ej.isNullOrUndefined(gridRows[1][this.model.scrolling.frozenRows - 1]) && this.model.scrolling.frozenColumns > 0)
                    $(gridRows[0][this.model.scrolling.frozenRows - 1].cells).add(gridRows[1][this.model.scrolling.frozenRows - 1].cells).addClass("e-frozeny").parent().addClass("e-frozenrow");
                else
                    $(gridRows[this.model.scrolling.frozenRows - 1].cells).addClass("e-frozeny").parent().addClass("e-frozenrow");
                this.model.scrolling.height = this._rowHeightCollection[Math.floor(this.model.scrolling.height / this._rowHeightCollection[1])] + 18;
            }
            else
                delete this.model.scrolling["scroll"];
        },
        _renderScroller: function () {
            if (!this.model.scrolling)
                this.model.scrolling = {};

            if (this.model.scrolling.width || this.model.width)
                this.element.width(this.model.scrolling.width || this.model.width);

            var $content = this.getGridContent().attr("tabindex", "0"), staticWidth, direction, gridRows = this.getGridRows();
            if (this.getBrowserDetails().browser != "msie" && this.model.scrolling.frozenColumns == 0) {
                $content.find("div table").first().width(this.getGridHeaderTable().width());
                this.getGridHeaderTable().width(this.getGridHeaderTable().width());
            }
            if (this.model.scrolling.frozenColumns > 0) {
                staticWidth = this.getGridContent().find(".e-frozencontentdiv").width();
                if (staticWidth > this.model.scrolling.width) {
                    this.getGridContent().remove();
                    this.getGridHeaderTable().eq(1).remove();
                    this._alertDialog.find(".e-content").text(this._getLocalizedLabels("frozenColumnsViewAlert"));
                    this._alertDialog.ejDialog("open");
                    return;
                }
                direction = this.model.rtl ? "margin-right" : "margin-left";
                this.getGridContent().find(".e-movablecontent").css(direction, staticWidth + "px");
                this.getGridHeaderContent().find(".e-movableheader").css(direction, staticWidth + "px");
                this.model.scrolling["targetPane"] = ".e-movablecontent";
            }
            this._initFrozenRows();
            $content.ejScroller(this.model.scrolling);
            if ($content.ejScroller("isVScroll")) {
                this.element.find(".e-gridheader").addClass("e-scrollcss");
            } else
                this.element.find(".e-gridheader").removeClass("e-scrollcss");
            this.refreshScrollerEvent();
            var proxy = this;
            if (proxy.model.scrolling.allowVirtualScrolling) {
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
                $content.ejScroller({
                    scroll: function (e) {
                        (e.source == "button" || e.source == "key" || e.source == "wheel") && proxy._virtualScroll(e);
                    },
                    thumbEnd: function (e) {
                        if (!$(e.originalEvent.target).hasClass("e-rowcell"))
                            proxy._virtualScroll(e);
                    }
                });
            }
        },

        _virtualScroll: function (e) {
            if (e != null) {
                var flag = 0;
                var recordCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
                var pageInfo = this.model.pageSettings;
                var tbody = this.getGridContentTable()[0].tBodies[0];
                var virtualRows = $(tbody).find('tr.e-virtualrow');
                pageInfo.totalPages = Math.ceil(recordCount / pageInfo.pageSize);
                if (e.scrollTop !== undefined)
                    e.model.scrollTop = e.scrollTop;
                var currentPageNo = this._calculateCurrenPage(virtualRows, this.getGridContentTable(), e.model);
                if (currentPageNo > pageInfo.totalPages)
                    currentPageNo = pageInfo.totalPages;
                if (pageInfo.currentPage != currentPageNo && $.inArray((currentPageNo - 1) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                    this._isVirtualRecordsLoaded = false;
                }
                if (!this._isVirtualRecordsLoaded) {
                    if ($.inArray((currentPageNo - 1) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                        if (currentPageNo == pageInfo.totalPages && $.inArray((currentPageNo - 2) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                            flag++;
                            this.set_currentPageIndex(currentPageNo - 1);
                        }
                        if (flag == 1) this._lastRow = true;
                        this.set_currentPageIndex(currentPageNo);
                    }
                    pageInfo.currentPage = currentPageNo;
                }
                else
                    pageInfo.currentPage = currentPageNo;
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
            }
        },

        _createPagerStatusBar: function () {
            var $statusBar = this.element.find(".e-pagerstatusbar");
            if ($statusBar.length)
                $statusBar.remove();
            var $pagermsgDiv = ej.buildTag('div.e-pagermsgdiv');
            this.$pagerStatusBarDiv = ej.buildTag('div.e-pagerstatusbar').append($pagermsgDiv);
            this.$pagerStatusBarDiv.appendTo(this.element);
            this.$pagerStatusBarDiv.css("display", "none");
        },

        _calculateCurrenPage: function (virtualRows, target, args) {
            var pageSize = this.model.pageSettings.pageSize;
            var currentPage;
            if (args.scrollTop >= this._scrollValue)
                currentPage = Math.ceil((args.scrollTop + this.model.scrolling.height) / this.getGridRowHeight() / pageSize);
            else
                currentPage = Math.floor((args.scrollTop + this.model.scrolling.height) / this.getGridRowHeight() / pageSize);

            this._scrollValue = args.scrollTop;
            for (var index = 0; index < virtualRows.length; index++) {
                var val = virtualRows[index];
                if (val.offsetTop + val.offsetHeight >= args.scrollTop) {
                    var prevVirtualPage = this._calculatePrevPage(virtualRows, target, args);
                    this._prevPageNo = prevVirtualPage;
                    if ((currentPage != this.model.pageSettings.totalPages) && currentPage != prevVirtualPage + 1 && $(val).next().is("tr[name=" + (currentPage - 1) * pageSize + "]"))
                        currentPage -= 1;
                    if (currentPage == 0)
                        currentPage = 1;
                    return currentPage > this.model.pageSettings.totalPages ? this.model.pageSettings.totalPages : currentPage;
                }
            }
            return currentPage;
        },
        _calculatePrevPage: function (virtualRows, target, args) {
            for (var i = 0; i < virtualRows.length; i++) {
                var val = virtualRows[i];
                if (val.offsetTop + val.offsetHeight >= args.scrollTop) {
                    var trElement = $(val).prevAll('tr[name]')[0];
                    if (trElement != null) {
                        return Math.ceil(parseInt($(trElement).attr('name'), 10) / this.model.pageSettings.pageSize) + 1;
                    }
                }
            }
            return -1;
        },
        _eventBindings: function () {
            var rowLength = this.model.frozenColumns > 0 ? this._gridRows[0].length : this._gridRows.length;
            this._refreshUnboundTemplate(this.getGridContentTable());
            if (this._gridRecordsCount != 0) {
                if (this.model.queryCellInfo != null || this.model.rowDataBound != null) {
                    for (var row = 0; row < rowLength; row++)
                        this._rowEventTrigger(this.getRowByIndex(row), this._dataSource()[row]);
                }
            }
        },
        _rowEventTrigger: function (row, data) {
            var args = { Element: row, data: data };
            this._trigger("rowDataBound", args);
            var tdCells = row.cells;
            var $tdRowcells = $(row).find(".e-rowcell");
            for (var cellIndex = 0; cellIndex < $tdRowcells.length; cellIndex++) {
                var args = { Element: $tdRowcells[cellIndex], Data: data, Text: $tdRowcells[cellIndex].innerHTML };
                var foreignKeyData = this._getForeignKeyData(args.Data);
                if ($($tdRowcells[cellIndex]).hasClass("e-rowcell"))
                    args.column = this.model.columns[cellIndex];
                if (!ej.isNullOrUndefined(foreignKeyData))
                    args.foreignKeyData = foreignKeyData;
                this._trigger("queryCellInfo", args);
            }

        },
        _renderGridPager: function () {
            var $div = $(document.createElement('div'));
            var pagerModel = {};
            this.model.pageSettings.click = this._gPagerClickHandler;
            this.model.pageSettings.totalRecordsCount = this._gridRecordsCount;
            this.model.pageSettings.rtl = this.model.rtl;
            this.model.pageSettings.localization = this.model.localization;
            this.model.pageSettings.enableQueryString = this.model.pageSettings.enableQueryString;
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar")
                pagerModel.enableExternalMessage = true;
            $.extend(pagerModel, this.model.pageSettings);
            pagerModel.currentPage = this._currentPage();
            pagerModel.masterObject = this;
            this.setGridPager($div);
            $div.ejPager(pagerModel);
            $div.ejPager("refreshPager");
            pagerModel = $div.ejPager("model");
            this.model.pageSettings.totalPages = pagerModel.totalPages;
            if (this._currentPage() !== pagerModel.currentPage)
                this._currentPage(pagerModel.currentPage);
            return $div;
        },
        _renderFiltering: function () {
            var $headerTable = this.getGridHeaderTable();
            var $tr = ej.buildTag('tr.e-filterbar'), $trClone;
            if (this.model.detailTemplate) $tr.append(ej.buildTag('th.e-filterbarcell e-mastercell'));
            for (var column = 0; column < this.model.columns.length; column++) {
                var $input = ej.buildTag('input.e-ejinputtext e-filtertext', "", {}, { title: this.model.columns[column]["field"] + "'s filterbar cell", type: "text", id: this.model.columns[column]["field"] + "_filterBarcell" });
                var $th = ej.buildTag('th.e-filterbarcell'), $div = ej.buildTag('div.e-filterdiv'), $span = ej.buildTag('span.e-cancel e-icon e-hide');
                if (this.model.columns[column]["allowFiltering"] === false) {
                    $input.attr("disabled", true).addClass("e-disable");
                    this._disabledFilterableColumns.push(this.model.columns[column]["headerText"]);
                }
                this.model.columns[column]["visible"] === false && $th.addClass("e-hide");
                $div.append($input).append($span);
                $tr.append($th.append($div));
                if (column == this.model.scrolling.frozenColumns - 1) {
                    $trClone = $tr.clone();
                    $headerTable.find("thead").first().append($trClone);
                    $tr.empty();
                }
            }
            $headerTable.find("thead").last().append($tr);
            this.setGridFilterBar($tr);
        },

        _initialRenderings: function () {
            if (this.model.groupSettings.groupedColumns.length) {
                var sortedColumns = new Array();
                for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                    sortedColumns.push(this.model.sortSettings.sortedColumns[i].field);
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                    if ($.inArray(this.model.groupSettings.groupedColumns[i], sortedColumns) == -1)
                        this.model.sortSettings.sortedColumns.push({ field: this.model.groupSettings.groupedColumns[i], direction: ej.sortOrder.Ascending });
                }
            }
        },
        _triggerConfirm: function (args) {
            if (args !== undefined && document.activeElement.value == this._getLocalizedLabels("OkButton")) {
                if (this._confirmDialog.find(".e-content").text() == this._getLocalizedLabels("BulkSaveConifrm"))
                    this.bulkSave();
                else {
                    this._confirmedValue = true;
                    this._processBindings(this._requestArgs);
                }
            }
            else {
                if (this._confirmDialog.find(".e-content").text() != this._getLocalizedLabels("BulkSaveConifrm")) {
                    switch (this._requestArgs.requestType) {
                        case "grouping":
                            this.model.groupSettings.groupedColumns.pop();
                            break;
                        case "ungrouping":
                            this.model.groupSettings.groupedColumns.push(this._requestArgs.columnName);
                            break;
                        case "sorting":
                            this._cSortedDirection = this._cSortedColumn = null;
                            break
                        case "filtering":
                            this.model.filterSettings.filteredColumns.reverse().splice(0, this._requestArgs.currentFilterObject);
                            this.model.filterSettings.filteredColumns.reverse();
                            break;
                        case "paging":
                            this._currentPage(this._requestArgs.previousPage);
                            this.getGridPager().ejPager("model.currentPage", this._requestArgs.previousPage);
                            break

                    }
                }
                this._confirmedValue = false;
            }
            this._requestArgs = null;
            this._confirmDialog.ejDialog("close");
        },
        _renderAlertDialog: function () {
            var $contentDiv = ej.buildTag('div.e-content', this._getLocalizedLabels("frozenColumnsMessage"))
                , $buttons = ej.buildTag('span.e-buttons', "<input type='button' id=" + this._id + 'ConfirmDialogOK' + " value='" + this._getLocalizedLabels("OkButton") + "'/>");
            this._alertDialog = ej.buildTag('div#' + this._id + 'AlertDialog');
            this._alertDialog.append($contentDiv).append($buttons);
            this.element.append(this._alertDialog);
            $buttons.find("input").ejButton({
                cssClass: this.model.cssClass,
                roundedCorner: true,
                size: "mini",
                click: $.proxy(function (args) {
                    this._alertDialog.ejDialog("close");
                }, this)
            });
            this._renderFDialog(this._id + 'AlertDialog');
            this._alertDialog.ejDialog({ width: "auto", modal: true });
        },
        _renderConfirmDialog: function () {
            var $contentDiv = ej.buildTag('div.e-content', this._getLocalizedLabels("BulkSaveConifrm"))
            , $buttons = ej.buildTag('span.e-buttons', "<input type='button' id=" + this._id + 'ConfirmDialogOK' + " value='" + this._getLocalizedLabels("OkButton") + "' /> "
                + "<input type='button' id=" + this._id + 'ConfirmDialogCancel' + " value='" + this._getLocalizedLabels("CancelButton") + "' />");

            this._confirmDialog = ej.buildTag('div#' + this._id + 'ConfirmDialog');
            this._confirmDialog.append($contentDiv).append($buttons);
            this.element.append(this._confirmDialog);
            $buttons.find("input").ejButton({
                cssClass: this.model.cssClass,
                roundedCorner: true,
                size: "mini",
                click: $.proxy(this._triggerConfirm, this)
            });
            this._renderFDialog(this._id + 'ConfirmDialog');
            this._confirmDialog.ejDialog({ width: "auto", modal: true });
        },
        _renderMultiTouchDialog: function () {
            this._customPop = ej.buildTag("div.e-gridpopup", "", { display: "none" });
            var $content = ej.buildTag("div.e-content"), $downTail = ej.buildTag("div.e-downtail e-tail");
            if (this.model.allowMultiSorting) {
                var $selElement = ej.buildTag("span.e-sortdirect e-icon");
                $content.append($selElement);
            }
            if (this.model.selectionType == ej.Grid.selectionType.Multiple) {
                var $selElement = ej.buildTag("span.e-rowselect e-icon");
                $content.append($selElement);
            }
            this._customPop.append($content);
            this._customPop.append($downTail);
            this.element.append(this._customPop);
        },
        _initialEndRendering: function () {
            // use this method to add behaviour after grid render.
            if (this._selectedRow() != -1)
                this.selectRows(this._selectedRow());
            this.model.edit.editMode == "batch" && this._renderConfirmDialog();
            this.model.scrolling.frozenColumns > 0 && this._renderAlertDialog();
            if (this.model.allowMultiSorting || this.model.selectionType == ej.Grid.selectionType.Multiple)
                this._renderMultiTouchDialog();
        },
        _externalFormPosition: function () {
            var pos = $(this.element).offset();
            var width = $(this.element).width();
            var height = $(this.element).height();
            var DivElement = $("#" + this._id + "_externalEdit");
            switch (this.model.edit.formPosition) {
                case "topright":
                    $(DivElement).find('.e-form-toggle-icon').removeClass('e-bottomleft').addClass('e-topright');
                    $(DivElement).css({ "left": (pos.left + width + 1) + "px", "top": pos.top + "px" });
                    $("#" + this._id + "_eFormContent").height((height - $("#" + this._id + "_eFormHeader")[0].offsetHeight - 1));
                    break;
                case "bottomleft":
                    $(DivElement).find('.e-form-toggle-icon').removeClass('e-topright').addClass('e-bottomleft');
                    $(DivElement).css({ "left": (pos.left) + "px", "top": (pos.top + height + 1) + "px" });
                    $("#" + this._id + "_eFormContent").width(width);
                    break;
            }
        },
        _createSortElement: function () {
            return ej.buildTag('span.e-icon', "&nbsp;");
        },

        _initColumns: function (object) {
            while (object.items != undefined)
                object = object.items[0];
            if (this.model.columns.length == 0 && object) {
                for (var field in object) {
                    if (object.hasOwnProperty(field)) {
                        var value = object[field];
                        this.model.columns.push({
                            field: field,
                            type: value != null ? (value.getDay ? "date" : typeof (value)) : null
                        });
                    }
                }
                this.model.columns.length && this._renderAfterColumnInitialize();
            }
            else {
                for (var index = 0; index < this.model.columns.length; index++) {
                    if (ej.isNullOrUndefined(this.model.columns[index].type)) {
                        var $field;
                        if (this.model.columns[index].dataSource && this.model.columns[index].foreignKeyValue)
                            $field = this.model.columns[index].dataSource[index][this.model.columns[index].foreignKeyValue];
                        else
                            $field = ej.getObject(this.model.columns[index].field, object);
                        this.model.columns[index].type = $field != null ? ($field.getDay ? "date" : typeof ($field)) : null;
                    }
                }
            }
        },
        _findColumnsWidth: function () {
            var j = this.getGridHeaderTable().find("colgroup").find("col");
            if (this.model.groupSettings.groupedColumns.length)
                j = j.slice(this.model.groupSettings.groupedColumns.length);
            for (var i = 0; i < this.model.columns.length; i++)
                this._columnsWidthCollection[i] = j.eq(i).width();
        },
        _calculateWidth: function () {
            var j = this.getGridHeaderTable().find("colgroup").find("col"), width = 0;
            for (var i = 0; i < j.length; i++)
                width += j.eq(i).width();
            return width;

        },
        _getEmptyTbody: function () {
            var $emptyTd = ej.buildTag('td.emptyrecord', this._getLocalizedLabels("EmptyRecord"), {}, { colSpan: this.model.columns.length });
            return $(document.createElement("tr")).append($emptyTd);
        },
        _getLocalizedLabels: function (property) {
            return ej.Grid.localization[this.model.localization][property] === undefined ? ej.Grid.localization["en-US"][property] : ej.Grid.localization[this.model.localization][property];
        },
        _getToggleButton: function () {
            return ej.buildTag("span.e-togglegroupbutton e-icon e-groupbutton", "&nbsp;");
        },
        _getRightHeaderCell: function () {
            return ej.buildTag("th.e-rightheadercell");
        },
        _getGroupTopLeftCell: function () {
            var $th = ej.buildTag("th.e-grouptopleftcell");
            $th.append(ej.buildTag("div.e-headercelldiv", "&nbsp;"));
            return $th;
        },
        _getIndentCol: function () {
            return ej.buildTag("col", "", { width: "30px" });
        },
        _getEmptyFilterBarCell: function () {
            return ej.buildTag("th.e-filterbarcell");
        },
        _getMetaColGroup: function () {
            var $colgroup = ej.buildTag("colgroup");
            for (var i = 0; i < this.model.columns.length; i++) {
                var $col = $(document.createElement("col"));
                this.model.columns[i]["visible"] === false && $col.css("display", "none");
                if (!this.model.groupSettings.groupedColumnShow && $.inArray(this.model.columns[i]["field"], this.model.groupSettings.groupedColumns) != -1)
                    $col.css("display", "none");
                $colgroup.append($col);
            }
            return $colgroup;
        },
        _setoffsetWidth: function () {
            var tds, $form = this.model.scrolling.frozenColumns > 0 ? this.element.find(".gridform") : $("#" + this._id + "EditForm");
            if (this._gridRecordsCount == 0 && this.model.edit.editMode != "batch")
                return;
            if (this.model.edit.editMode == "batch")
                tds = $form.closest("td");
            else if (this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate" || this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate")
                tds = $form.find(".e-rowcell").not(".e-unboundcell,.e-templatecell");
            else
                tds = $form.find("tr").find(".e-rowcell").not(".e-unboundcell,.e-templatecell");
            for (var i = 0; i < tds.length; i++)
                this._tdsOffsetWidth[i] = tds.get(i).offsetWidth;
        },

        _setCurrentRow: function (requestType) {
            if (requestType == ej.Grid.actions.Refresh || requestType == ej.Grid.actions.Filtering || requestType == ej.Grid.actions.Sorting || requestType == ej.Grid.actions.Delete || requestType == ej.Grid.actions.Save || requestType == ej.Grid.actions.Cancel || requestType == ej.Grid.actions.Paging) {
                this._selectedRow(-1);
                this._selectedRowsIndexes = [];
            }
        },

        /**
			* To refresh the grid data source.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.dataSource(); // Refreshes the grid data source
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Refreshes the grid data source
			* $("#Grid").ejGrid("dataSource");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        dataSource: function (dataSource) {
            this._dataSource(dataSource);
            this._refreshDataSource(dataSource);
        },
        _refreshDataSource: function (dataSource) {
            if (dataSource instanceof ej.DataManager)
                this._dataManager = dataSource;
            else
                this._dataManager = ej.DataManager(dataSource);
            this.refreshGridContent(true);
            if (!ej.isNullOrUndefined(this.getGridPager())) {
                this.getGridPager().ejPager("model.currentPage", 1);
                this._refreshGridPager();
            }
        },
        /**
			* It is used to refresh the grid contents. The template refreshment is based on the argument passed along with this method.
			* @return jQuery
			* @param {number} templateRefresh When templateRefresh is set true, template and grid contents both are refreshed in grid else only grid content is refreshed
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.refreshGridContent(); // Refreshes the grid contents only
			* gridObj.refreshGridContent(true); // Refreshes the template and grid contents
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Refreshes the grid contents only
			* $("#Grid").ejGrid("refreshGridContent");        
			* // Refreshes the template and grid contents
			* $("#Grid").ejGrid("refreshGridContent", true);        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        refreshGridContent: function (templateRefresh) {
            templateRefresh && this.templateRefresh();
            var args = {};
            args.requestType = ej.Grid.actions.Refresh;
            this._processBindings(args);
        },
        /**
			* To refresh the template of the grid control
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.templateRefresh(); // Refreshes the template of the grid control
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Refreshes the template of the grid control.
			* $("#Grid").ejGrid("templateRefresh");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        templateRefresh: function () {
            this.addInitTemplate();
            if (this.model.edit.allowEditing || this.model.edit.allowAdding) {
                if (this.model.edit.editMode == "normal")
                    this.addEditingTemplate();
                else if (this.model.edit.editMode == "dialog" || this.model.edit.editMode == "externalform" ||
                         this.model.edit.editMode == "inlineform")
                    this.addDialogEditingTemplate();
                else if (this.model.edit.editMode == "dialogtemplate" || this.model.edit.editMode == "externalformtemplate" ||
                         this.model.edit.editMode == "inlineformtemplate")
                    this.addExternalDialogEditingTemplate();
            }
            if (this.model.allowGrouping) this.addGroupingTemplate();
        },
        /**
			* It is used to hide columns based on the header text in grid control.
			* @return jQuery
			* @param {array/string} headerText you can pass either array of header text of various columns or a header text of a column to hide 
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.hideColumns("Order ID"); // Hides column based on the given header text of the column
			* gridObj.hideColumns(["Order ID", "Customer ID"]); // Hide columns based on the array of header text of the columns given
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Hide column based on the given header text of the column
			* $("#Grid").ejGrid("hideColumns", "Order ID"); 
			* // Hide columns based on the array of header text of the columns given
			* $("#Grid").ejGrid("hideColumns", ["Order ID", "Customer ID"]); 			
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        hideColumns: function (c) {
            var i, count = 0, args = {}, index;
            if ($.isArray(c)) {
                for (i = 0; i < c.length; i++) {
                    index = $.inArray(c[i], this._visibleColumns);
                    if (index != -1) {
                        this._hiddenColumns.push(c[i]);
                        this._visibleColumns.splice(index, 1);
                    }
                }
            } else {
                index = $.inArray(c, this._visibleColumns);
                if (index != -1) {
                    this._hiddenColumns.push(c);
                    this._visibleColumns.splice(index, 1);
                }
            }
            for (i = 0; i < this.model.columns.length; i++) {
                if ($.inArray(this.model.columns[i].headerText, this._hiddenColumns) != -1) {
                    this.model.columns[i].visible = false;
                    count++;
                }
                if (this._hiddenColumns.length == count)
                    break;
            }
            this.templateRefresh();
            args.requestType = "refresh";
            this.sendDataRenderingRequest(args);
            this._hideHeaderColumn(this._hiddenColumns);
            this.model.allowScrolling && this.getScrollObject().refresh();
        },
        /**
			* It is used to show columns based on the header text in grid control.
			* @return jQuery
			* @param {array/string} headerText you can pass either array of header text of various columns or a header text of a column to show 
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.showColumns("Order ID"); // Shows column based on the given header text of the column
			* gridObj.showColumns(["Order ID", "Customer ID"]); // Shows columns based on the array of header text of the columns given
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Shows column based on the given header text of the column
			* $("#Grid").ejGrid("showColumns", "Order ID"); 
			* // Shows columns based on the array of header text of the columns given
			* $("#Grid").ejGrid("showColumns", ["Order ID", "Customer ID"]); 			
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        showColumns: function (c) {
            var i, count = 0, args = {}, index, column;
            if ($.isArray(c)) {
                for (i = 0; i < c.length; i++) {
                    index = $.inArray(c[i], this._hiddenColumns);
                    if (index != -1) {
                        this._hiddenColumns.splice(index, 1);
                        this._visibleColumns.push(c[i]);
                    }
                }
            } else {
                index = $.inArray(c, this._hiddenColumns);
                if (index != -1) {
                    this._hiddenColumns.splice(index, 1);
                    this._visibleColumns.push(c);
                }
            }
            for (i = 0; i < this.model.columns.length; i++) {
                if ($.inArray(this.model.columns[i].headerText, this._visibleColumns) != -1) {
                    this.model.columns[i].visible = true;
                    count++;
                }
                if (this._visibleColumns.length == count)
                    break;
            }
            this.templateRefresh();
            args.requestType = "refresh";
            this.sendDataRenderingRequest(args);
            this._showHeaderColumn(this._visibleColumns);
        },
        /**
			* It is used to clear the sorting from columns in the grid
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.clearSorting(); // Clears the sorting from columns in the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Clears the sorting from columns in the grid
			* $("#Grid").ejGrid("clearSorting");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        clearSorting: function () {
            var proxy = this;
            this.model.sortSettings.sortedColumns = $.grep(this.model.sortSettings.sortedColumns, function (value, index) {
                if ($.inArray(value.field, proxy.model.groupSettings.groupedColumns) != -1)
                    return true;
                return false;
            });
            this._$prevSElementTarget = null;
            this._$curSElementTarget = null;
            this.refreshGridContent();
        },
        /**
			* It is used to get the browser details.
			* @return browserDetails
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getBrowserDetails(); // Gets the browser details of the application being run
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the browser details of the application being run
			* $("#Grid").ejGrid("getBrowserDetails");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getBrowserDetails: function () {
            var b = navigator.userAgent.match(/(firefox|chrome|opera|msie|safari)\s?\/?(\d+(.\d+)*)/i);
            if (!!navigator.userAgent.match(/Trident\/7\./))
                return { browser: "msie", version: jQuery.uaMatch(navigator.userAgent).version };
            return { browser: b[1].toLowerCase(), version: b[2] };
        },
        /**
			* It is used to get the column field name from the given header text in grid control.
			* @return fieldName
			* @param {string} headerText Pass header text of the column to get its corresponding field name
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getFieldNameByHeaderText("Order ID"); // Gets the column field name from the given headerText
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the column field name from the given headerText
			* $("#Grid").ejGrid("getFieldNameByHeaderText", "Order ID");
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getFieldNameByHeaderText: function (headerText) {
            if (ej.isNullOrUndefined(this._fieldColumnNames[headerText]))
                return null;
            return this._fieldColumnNames[headerText];
        },
        /**
			* It is used to get the names of all the visible column collections in grid control.
			* @return visibleColumns
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getVisibleColumnNames(); // Gets the names of all the visible column collections
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the names of all the visible column collections
			* $("#Grid").ejGrid("getVisibleColumnNames");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getVisibleColumnNames: function (headerText) {
            return this._visibleColumns;
        },
        /**
			* It is used to get the names of all the hidden column collections in grid control.
			* @return hiddenColumns
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getHiddenColumnNames(); // Gets names of all the hidden column collections
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets names of all the hidden column collections
			* $("#Grid").ejGrid("getHiddenColumnNames");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getHiddenColumnNames: function (headerText) {
            return this._hiddenColumns;
        },
        /**
			* It is used to get the column header text from the given field name in grid control.
			* @return headerText
			* @param {string} field Pass field name of the column to get its corresponding header text
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getHeaderTextByFieldName("OrderID"); // Gets the column header text from the given field name
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the column header text from the given field name
			* $("#Grid").ejGrid("getHeaderTextByFieldName", "OrderID");
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getHeaderTextByFieldName: function (field) {
            if (ej.isNullOrUndefined(this._headerColumnNames[field]))
                return null;
            return this._headerColumnNames[field];
        },
        /**
			* It is used to get the grid row height.
			* @return rowHeight
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getGridRowHeight();  // Gets the row height of the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the row height of the grid
			* $("#Grid").ejGrid("getGridRowHeight");   
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getGridRowHeight: function () {
            var rowHeight = -1;
            var trColl = this.getGridContentTable().find('tr:not(.e-virtualrow)');
            if (trColl.length > 1) {
                if (trColl[0].getBoundingClientRect() && this.getGridContentTable().find("tr")[0].getBoundingClientRect().height) {
                    rowHeight = trColl[0].getBoundingClientRect().height;
                } else
                    rowHeight = trColl[0].offsetHeight;
            }
            return rowHeight == -1 ? 38 : rowHeight;
        },
        /**
			* It is used to get the current page index in grid.
			* @return Current PageIndex
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getCurrentIndex();  // Gets the current page index in grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the current page index in grid
			* $("#Grid").ejGrid("getCurrentIndex”);   
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getCurrentIndex: function () {
            return ((this._currentPage() - 1) * (this.model.pageSettings.pageSize));
        },
        /**
			* It is used to set the current page index in grid.
			* @return jQuery
			* @param {number} val Pass the value of current page index to be set in grid
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.set_currentPageIndex(3);  // Sets the current page index in grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sets the current page index in grid
			* $("#Grid").ejGrid("set_currentPageIndex”, 3);   
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        set_currentPageIndex: function (val) {
            var pageSetting = this.model.pageSettings;
            var recordCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            if (pageSetting.totalPages == null)
                pageSetting.totalPages = Math.ceil(recordCount / pageSetting.pageSize);
            if (val > pageSetting.totalPages || val < 1 || val == this._currentPage())
                return false;
            if (ej.isNullOrUndefined(this._prevPageNo))
                this._prevPageNo = this._currentPage();
            this._currentPage(val);
            if (this._currentPage() != this._prevPageNo) {
                var args = {};
                args.requestType = "paging";
                this._isVirtualRecordsLoaded = false;
                this.sendPagingRequest(this._currentPage(), args);
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
                return true;
            }
            else
                return false;
        },
        _showPagerInformation: function (model) {
            var from = (model.currentPage - 1) * model.pageSize;
            $(this.$pagerStatusBarDiv).find("div:first").html(String.format(this._getLocalizedLabels("PagerInfo"), model.currentPage, model.totalPages, model.totalRecordsCount), from, from + model.pageSize);
            $(this.$pagerStatusBarDiv).css('display', 'block');
        },
        /**
			* It is used to expand all the group caption rows in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.expandAll(); // Expand all the group caption rows
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Expand all the group caption rows
			* $("#Grid").ejGrid("expandAll");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        expandAll: function () {
            var recordPlus = this.element.find(".e-recordpluscollapse");
            for (var i = 0; i < recordPlus.length; i++)
                this.expandCollapse($(recordPlus[i]));
        },
        /**
			* It is used to collapse all the group caption rows in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.collapseAll(); // Collapse all the group caption rows
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Collapse all the group caption rows
			* $("#Grid").ejGrid("collapseAll");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        collapseAll: function () {
            var recordPlus = this.element.find(".e-recordplusexpand");
            for (var i = 0; i < recordPlus.length; i++)
                this.expandCollapse($(recordPlus[i]));
        },
        /**
			* It is used to clear all the row selection or  at specific row selection based on the index provided.
			* @return jQuery
			* @param {number} index The index parameter is optional. If index of the row is specified then it will remove the selection from the particular row else it will clears all of the row selection
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.cleanUpSelection(2);  // Removes the selection based on the row index
			* gridObj.cleanUpSelection();  // clears all of the row selection
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Removes the selection based on the row index
			* $("#Grid").ejGrid("cleanUpSelection", 2);   
			* // clears all of the row selection
			* $("#Grid").ejGrid("cleanUpSelection");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        cleanUpSelection: function (index) {
            if (this._selectedRow() != -1) {
                var $gridRows = $(this.getGridRows()), index;
                if (!ej.isNullOrUndefined(index)) {
                    this.getRowByIndex(index).removeAttr("aria-selected").find(".e-selectionbackground").removeClass("e-selectionbackground").removeClass("e-active");
                    index = $.inArray(index, this._selectedRowsIndexes);
                    if (index != -1)
                        this._selectedRowsIndexes.splice(index, 1);
                } else {
                    if (this.model.scrolling.frozenColumns > 0)
                        $gridRows = $($gridRows[0]).add($gridRows[1]);
                    $gridRows.removeAttr("aria-selected").find(".e-rowcell, .e-detailrowcollapse, .e-detailrowexpand").removeClass("e-selectionbackground").removeClass("e-active");
                }
                if (!this._selectedRowsIndexes.length)
                    this._selectedRow(-1);
            }
            return true;
        },
        _recalculateIndentWidth: function () {
            var proxy = this;
            var browserDetails = this.getBrowserDetails();
            var indentWidth = this.getGridHeaderTable().find(".e-lastgrouptopleftcell").width(), newWidth, perPixel = indentWidth / 30, $col;
            if (perPixel > 1)
                newWidth = (30 / perPixel);
            this.getGridHeaderTable().find("colgroup").find("col").slice(0, this.model.groupSettings.groupedColumns.length).css("width", newWidth + "px");
            indentWidth = this.getGridHeaderTable().find(".e-lastgrouptopleftcell").width();
            if (indentWidth > 30 || (this._isCaptionSummary && indentWidth >= 30)) {
                if (this._isCaptionSummary) {
                    var colgroup = this.getGridContentTable().find("table").filter(":not(.e-recordtable)").children("colgroup");
                    $.each(colgroup, function (index, item) {
                        var indentCol = $(item).find("col").length - proxy.model.columns.length;
                        $(item).find("col").slice(0, indentCol).css("width", newWidth + "px");
                    });
                }
                else
                    this.getGridContentTable().find("table").filter(":not(.e-recordtable)").children("colgroup").find("col:first-child").css("width", indentWidth + "px");
                $col = this.getGridContentTable().find("colgroup").first().find("col").slice(0, this.model.groupSettings.groupedColumns.length);
                if (browserDetails.browser != "msie")
                    $col.css("width", newWidth + "px");
                else
                    $col.first().css("width", ((indentWidth / this.element.width()) * 100) + "%");
            } else
                browserDetails.browser != "msie" && this.getGridContentTable().find("colgroup").first().find("col").slice(0, this.model.groupSettings.groupedColumns.length).css("width", newWidth + "px");

        },
        /**
			* It is used to set width of the columns after performing resizing, grouping, editing and other operations in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.setWidthToColumns(); // Sets the width of the columns after resizing, grouping, editing, etc
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sets the width of the columns after resizing, grouping, editing, etc
			* $("#Grid").ejGrid("setWidthToColumns");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        setWidthToColumns: function () {
            var $cols1 = this.getGridContentTable().children("colgroup").find("col");
            var $cols2 = this.getGridHeaderTable().children("colgroup").find("col");
            var width = this.element.width(), frozenWidth = 0;
            if (this.model.groupSettings.groupedColumns.length && !this.model.allowScrolling) {
                var browserDetails = this.getBrowserDetails();
                if (browserDetails.browser == "msie" && parseInt(browserDetails.version, 10) > 8)
                    $cols1.first().css("width", ((30 / width) * 100) + "%");
            }
            var colCount = this.model.columns.length;
            if ($cols1.length > colCount) $cols1.splice(0, ($cols1.length - colCount));
            if ($cols2.length > colCount) $cols2.splice(0, ($cols2.length - colCount));

            for (var i = 0; i < $cols2.length; i++) {
                if (this.model.allowResizeToFit && this.model.columns[i]["width"] === undefined) {
                    hCellIndex = this.model.groupSettings.groupedColumns.length ? (i + this.model.groupSettings.groupedColumns.length) : i;
                    var contentWidth = this._getContentWidth(i);
                    var headerWidth = this._getHeaderContentWidth(this.getGridHeaderTable().find('.e-headercelldiv').eq(hCellIndex));
                    var finalWidth = contentWidth > headerWidth ? contentWidth : headerWidth;
                    finalWidth = finalWidth + (finalWidth * 0.2);
                    this._columnsWidthCollection[i] = finalWidth;
                }
                if (!ej.isNullOrUndefined(this._columnsWidthCollection[i])) {
                    $cols1.eq(i).width(this._columnsWidthCollection[i]);
                    $cols2.eq(i).width(this._columnsWidthCollection[i]);
                } else if (this.model.allowScrolling) {
                    var colWidth = (width / this.model.columns.length).toFixed(2) + "px";
                    $cols1.eq(i).css("width", colWidth);
                    $cols2.eq(i).css("width", colWidth);
                    this._columnsWidthCollection[i] = parseFloat((width / this.model.columns.length).toFixed(2));
                }
            }
            if (this.model.isEdit) {
                var clonedCol = $cols1.clone(), $colGroup = this.model.scrolling.frozenColumns > 0 ? this.getGridContent().find(".gridform").find("colgroup") : $("#" + this._id + "EditForm").find("colgroup");
                this.model.scrolling.frozenColumns > 0 && $colGroup.first().empty().append(clonedCol.splice(0, this.model.scrolling.frozenColumns));
                $colGroup.last().empty().append(clonedCol);
                this.model.detailTemplate != null && $colGroup.prepend(this._getIndentCol());
            }
            if (this.model.groupSettings.groupedColumns.length) {
                var $grouedColGroup = this.getGridContentTable().find(".e-recordtable").children("colgroup");
                for (var i = 0; i < $grouedColGroup.length; i++) {
                    var clonedCol = $cols1.clone();
                    if (this.model.detailTemplate != null) clonedCol.splice(0, 0, this._getIndentCol()[0]);
                    $grouedColGroup.eq(i).empty().append(clonedCol);
                }
            }
            if (this.model.scrolling.frozenColumns > 0) {
                var totalWidth = 0, frozenWidth;
                for (var i = 0; i < this._columnsWidthCollection.length; i++) {
                    totalWidth += this._columnsWidthCollection[i];
                    if (this.model.scrolling.frozenColumns - 1 == i)
                        frozenWidth = Math.ceil(totalWidth);
                }
                this.getGridContent().find(".e-frozencontentdiv").outerWidth(frozenWidth)
                    .end().find(".e-movablecontentdiv").outerWidth(totalWidth - frozenWidth);
                this.getGridHeaderContent().find(".e-frozenheaderdiv").outerWidth(frozenWidth)
                    .end().find(".e-movableheaderdiv").outerWidth(totalWidth - frozenWidth);
            }
        },
        /**
			* It is used expand or collapse the row based on the row state in grid control.
			* @return jQuery
			* @param {object} $target Pass the target object to expand/collapse the row based on its row state
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.expandCollapse($("tr td.recordplusexpand > div").first());  // Expands or collapses the row based on the row state
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Expands or collapses the row based on the row state
			* $("#Grid").ejGrid("expandCollapse", $("tr td.recordplusexpand > div").first());        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        expandCollapse: function ($target) {
            if ($target.prop("tagName") == "DIV" && ($target.parent().hasClass("e-recordplusexpand") || $target.parent().hasClass("e-recordpluscollapse") || $target.parent().hasClass("e-detailrowcollapse") || $target.parent().hasClass("e-detailrowexpand")))
                $target = $target.parent();
            if ($target.hasClass("e-recordplusexpand") && this.model.groupSettings.groupedColumns.length) {
                var cellIndex = $target.index();
                var $rows = $target.closest('tr').next();
                $rows.hide();
                $target.removeClass("e-recordplusexpand").addClass("e-recordpluscollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
            } else if ($target.hasClass("e-recordpluscollapse") && this.model.groupSettings.groupedColumns.length) {
                var cellIndex = $target.index();
                var $rows = $target.closest('tr').next();
                var toExpandRows = [];
                var $row = $rows;
                if ($($row[0].cells[cellIndex]).hasClass("e-indentcell")) {
                    if ($row.children(".e-indentcell").length == ($target.parent().children('.e-indentcell').length) + 1) {
                        $row.show();
                        var $expand = $row.children(".e-recordplusexpand");
                        if ($expand != null && $expand.length > 0) {
                            toExpandRows.push($expand);
                        }
                    }
                }
                $target.removeClass("e-recordpluscollapse").addClass("e-recordplusexpand").find("div").removeClass("e-gnextforward").addClass("e-gdiagonalnext");
                for (var i = 0; i < toExpandRows.length; i++) {
                    toExpandRows[i].removeClass("e-recordplusexpand").addClass("e-recordpluscollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
                    this.expandCollapse(toExpandRows[i]);
                }
            } else if ($target.hasClass("e-detailrowexpand")) {
                var cellIndex = $target.index(), proxy = this;
                var rowIndexValue = $target.closest('tr').index() - $target.closest('tr').prevAll("tr.e-detailrow").length;
                var $rows = $target.closest('tr').next();
                $rows.hide(0, function () {
                    var args = { masterRow: $target.closest('tr'), detailRow: $rows, masterData: proxy._currentJsonData[rowIndexValue] };
                    var foreignKeyData = proxy._getForeignKeyData(args.masterData);
                    if (!ej.isNullOrUndefined(foreignKeyData))
                        args.foreignKeyData = foreignKeyData;
                    proxy._trigger("detailCollapse", args);

                });
                $target.removeClass("e-detailrowexpand").addClass("e-detailrowcollapse").find("div").addClass("e-gnextforward").removeClass("e-gdiagonalnext");
            } else if ($target.hasClass("e-detailrowcollapse")) {
                var cellIndex = $target.index(), proxy = this;
                var rowIndexValue = $target.closest('tr').index() - $target.closest('tr').prevAll("tr.e-detailrow").length;
                var detailrow = $target.closest('tr').next();
                if (detailrow.hasClass("e-detailrow"))
                    $rows = detailrow;
                else {
                    var detailtr = ej.buildTag("tr.e-detailrow", "", { 'display': 'none' }, {});
                    var indenttd = ej.buildTag("td.e-detailindentcell");
                    var detailstd = ej.buildTag("td.e-detailcell", "", {}, { colspan: this.model.columns.length });
                    $(detailtr).append(indenttd);
                    $(detailtr).append(detailstd.append($(this.model.detailTemplate).render(this._currentJsonData[rowIndexValue])));

                    $($target.closest('tr')).after(detailtr);
                    this._trigger("detailData", { templateDetail: detailtr }); //    $(tbody).append(trchild);
                    $rows = detailtr;
                }
                var toExpandRows = [];
                var $row = $rows;
                if ($($row[0].cells[cellIndex]).hasClass("e-detailindentcell")) {
                    $row.show(0, function () {
                        var args = { masterRow: $target.closest('tr'), detailRow: $rows, masterData: proxy._currentJsonData[rowIndexValue] };
                        var foreignKeyData = proxy._getForeignKeyData(args.masterData);
                        if (!ej.isNullOrUndefined(foreignKeyData))
                            args.foreignKeyData = foreignKeyData;
                        proxy._trigger("detailExpand", args);
                    });
                    var $expand = $row.children(".e-detailrowexpand");
                    if ($expand != null && $expand.length > 0) {
                        toExpandRows.push($expand);
                    }
                }
                $target.removeClass("e-detailrowcollapse").addClass("e-detailrowexpand").find("div").addClass("e-gdiagonalnext").removeClass("e-gnextforward");
                for (var i = 0; i < toExpandRows.length; i++) {
                    toExpandRows[i].removeClass("e-detailrowexpand").addClass("e-detailrowcollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
                    this.expandCollapse(toExpandRows[i]);
                }
            }
        },
        /**
			* It is used to get the selected records in grid control.
			* @return selectedRecords
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getSelectedRecords(); // Gets the selected row list
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the selected row list
			* $("#Grid").ejGrid("getSelectedRecords");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getSelectedRecords: function () {
            var records = [];
            for (var i = 0; i < this._selectedRowsIndexes.length; i++) {
                if (this._selectedRowsIndexes[i] != -1)
                    records.push(this._currentJsonData[this._selectedRowsIndexes[i]]);
            }
            return records;
        },
        /**
			* It is used to get the rows from the given from and to index in grid.
			* @return gridRows
			* @param {number} from Pass the from index from which the rows to be returned
			* @param {number} to Pass the to index to which the rows to be returned
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getRowByIndex(3, 6);  // Gets the rows from the specified row index 
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the rows from the specified row index
			* $("#Grid").ejGrid("getRowByIndex”, 3, 6);   
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getRowByIndex: function (from, to) {
            try {
                var gridRows = this.getGridRows(), $gridRows = $(gridRows), $row = $();
                if (ej.isNullOrUndefined(to)) {
                    if (this.model.scrolling.frozenColumns > 0) {
                        $row.push(gridRows[0][from]);
                        $row.push(gridRows[1][from]);
                        return $row;
                    }
                    return $(gridRows[from]);
                }
                else {
                    if (this.model.scrolling.frozenColumns > 0) {
                        $row.push($gridRows.eq(0).slice(from, to));
                        $row.push($gridRows.eq(0).slice(from, to));
                        return $row;
                    }
                    return $($gridRows.slice(from, to));
                }
            }
            catch (e) {
                return $();
            }
        },
        /**
			* It is used to get the row index based on the given tr element in grid control.
			* @return row index
			* @param {object} $tr Pass the tr element in grid content to get its row index
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getIndexByRow($(".gridcontent tr").first()); // Gets the row index based on the given row
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the row index based on the given row
			* $("#Grid").ejGrid("getIndexByRow", $(".gridcontent tr").first());        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getIndexByRow: function ($tr) {
            var gridRows = this.getGridRows(), $gridRows = $(gridRows), rowIndex;
            if (this.model.scrolling.frozenColumns > 0) {
                rowIndex = $(gridRows[0]).index($tr);
                if (rowIndex == -1)
                    rowIndex = $(gridRows[1]).index($tr);
                return rowIndex;
            }
            else
                return $gridRows.index($tr);
        },
        /**
			* It is used to select rows in grid control.
			* @return jQuery
			* @param { number } fromIndex It is used to set the starting index of row for selecting rows.
			* @param { number } toIndex It is used to set the ending index of row for selecting rows.
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.selectRows(1, 4); // Selects rows based on the given index
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Selects rows based on the given index
			* $("#Grid").ejGrid("selectRows", 1, 4);
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        selectRows: function (rowIndex, toIndex) {
            if (!this.model.allowSelection)
                return false;
            var $gridRows = $(this.getGridRows());
            var args = {};
            if (this._selectedRow() != -1)
                args = { currentRowIndex: this._selectedRow(), currrentRow: $gridRows.eq(this._selectedRow()), currentData: this.model.currentViewData[this._selectedRow()] };

            if (this._trigger("rowSelecting", args))
                return;
            if ((this.model.edit.allowEditing || this.model.edit.allowAdding) && this.model.isEdit && this.model.autoSaveOnRowSelection) {
                if (this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate")
                    $("#" + this._id + "_externalEdit").css("display", "none");
                if (this.sendSaveRequest())
                    return;
                var $gridRows = $(this.getGridRows());
            }
            if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(rowIndex)) {
                rowIndex = ej.isNullOrUndefined(rowIndex) ? toIndex : rowIndex;
                switch (this.model.selectionType) {
                    case ej.Grid.selectionType.Multiple:
                        if (this._multiSelectCtrlRequest) {
                            var selectedRowIndex = $.inArray(rowIndex, this._selectedRowsIndexes);
                            selectedRowIndex != -1 && this.cleanUpSelection(rowIndex) && this._selectedRowsIndexes.splice(selectedRowIndex, 0);
                            if (selectedRowIndex == -1) {
                                this._selectedRowsIndexes.push(rowIndex);
                                this.getRowByIndex(rowIndex).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                            }
                            break;
                        }
                    case ej.Grid.selectionType.Single:
                        this.cleanUpSelection();
                        this._selectedRowsIndexes = [];
                        this._selectedRowsIndexes.push(rowIndex);
                        this.getRowByIndex(rowIndex).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                        break;
                }
            } else {
                if (this.model.selectionType == ej.Grid.selectionType.Multiple) {
                    this.cleanUpSelection();
                    this._selectedRowsIndexes = [];
                    rows = rowIndex - toIndex < 0 ? this.getRowByIndex(rowIndex, toIndex + 1) : this.getRowByIndex(toIndex, rowIndex + 1);
                    for (var i = 0; i < rows.length; i++)
                        this._selectedRowsIndexes.push(rows[i].rowIndex);
                    $(rows).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                }
            }
            this._previousIndex = this._selectedRowsIndexes.length ? rowIndex : undefined;
            if (this._selectedRow() !== rowIndex)
                this._selectedRow(rowIndex);
            var args = { currentRowIndex: this._selectedRow(), currrentRow: this.getRowByIndex(this._selectedRow()), currentData: this.model.currentViewData[this._selectedRow()] };
            this._trigger("rowSelected", args);
            this._multiSelectCtrlRequest = false;
        },
        /**
			* It is used to get the column details  based on the given header text in grid control.
			* @return Column Object
			* @param {string} headerText Pass the header text of the column to get the corresponding column object
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getColumnByHeaderText("Order ID"); // Gets the column details based on the given headerText
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the column details based on the given headerText
			* $("#Grid").ejGrid("getColumnByHeaderText", "Order ID");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getColumnByHeaderText: function (headerText) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["headerText"] == headerText)
                    break;
            }
            return column == this.model.columns.length ? null : this.model.columns[column];
        },
        /**
			* It is used to get the column details  based on the given field name in grid control.
			* @return Column Object
			* @param {string} field Pass the field name of the column to get the corresponding column object
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getColumnByField("OrderID"); // Gets the column details based on the given field name
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the column details based on the given field name
			* $("#Grid").ejGrid("getColumnByField", "OrderID");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getColumnByField: function (field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"] == field)
                    break;
            }
            return column == this.model.columns.length ? null : this.model.columns[column];
        },
        /**
			* It is used to get the column details based on the given column index in grid control.
			* @return Column Object
			* @param {number} index Pass the index of the column to get the corresponding column object
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getColumnByIndex(1); // Gets the column details based on the given column index
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the column details based on the given column index
			* $("#Grid").ejGrid("getColumnByIndex", 1);        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getColumnByIndex: function (index) {
            if (index < this.model.columns.length)
                return this.model.columns[index];
            return null;
        },
        /**
			* It is used to get the column index of the given field in grid control.
			* @return Column Index
			* @param {string} field Pass the field name of the column to get the corresponding column index
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getColumnIndexByField("OrderID"); // Gets the column index based on the given field name
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the column index based on the given field name
			* $("#Grid").ejGrid("getColumnIndexByField", "OrderID");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getColumnIndexByField: function (field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"] == field)
                    break;
            }
            return column;
        },
        /**
			* It is used to get the list of field names from column collection in grid control.
			* @return columnFieldNames
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getColumnFieldNames(); // Gets the column field names based on the given index
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the column field names based on the given index
			* $("#Grid").ejGrid("getColumnFieldNames");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getColumnFieldNames: function () {
            var columnNames = [];
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"])
                    columnNames.push(this.model.columns[column]["field"]);
            }
            return columnNames;
        },
        _initValidator: function () {
            var gridObject = this, elements = this.model.scrolling.frozenColumns > 0 ? this.element.find(".gridform") : $("#" + this._id + "EditForm");
            for (var i = 0; i < elements.length ; i++) {
                elements.eq(i).validate({
                    errorClass: 'e-field-validation-error',
                    errorElement: 'div',
                    wrapper: "div",
                    errorPlacement: function (error, element) {
                        var $td = element.closest("td"), $container = $(error).addClass("e-error")
                        , $tail = ej.buildTag("div.e-errortail e-toparrow"), $container;
                        $td.find(".e-error").remove();
                        if (element.hasClass("e-dropdownlist"))
                            $container.insertAfter(element.closest(".e-ddl"));
                        else if (element.hasClass("e-numerictextbox"))
                            $container.insertAfter(element.closest(".e-numeric"));
                        else
                            $container.insertAfter(element);
                        $container.prepend($tail)
                        gridObject.model.edit.editMode != "dialog" && $container.offset({ left: element.offset().left });
                        $container.fadeIn("slow");
                    },

                });
            }
        },
        setValidation: function () {
            this._initValidator();
            for (var i = 0; i < this.model.columns.length; i++) {
                if (!ej.isNullOrUndefined(this.model.columns[i]["validationRules"])) {
                    this.setValidationToField(this.model.columns[i].field, this.model.columns[i].validationRules);
                }
            }
        },
        /**
			* It is used to set validation to a field during editing.
			* @return jQuery
			* @param {string} fieldName Specify the field name of the column to set validation rules
			* @param {object} rules Specify the validation rules for the field
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* // It is used to set validation to a field during editing
			* gridObj.setValidationToField("OrderID", { required: true }); 
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // It is used to set validation to a field during editing
			* $("#Grid").ejGrid("setValidationToField", "OrderID", { required: true });
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        setValidationToField: function (name, rules) {
            var ele = $("#" + this._id + name).length > 0 ? $("#" + this._id + name) : $("#" + name);
            ele.rules("add", rules);
            var validator = $("#" + this._id + "EditForm").validate();
            if (!ej.isNullOrUndefined(rules["required"])) {
                validator.settings.messages[name] = {};
                validator.settings.messages[name]["required"] = this.getColumnByField(name).headerText + " is required";
            }
        },
        // data Prcoessing

        _columnToSelect: function () {
            var column = [];
            for (var i = 0; i < this.model.columns.length; i++) {
                if (this.model.columns[i]["columnEditType"] === ej.Grid.editingType.Dropdown && ej.isNullOrUndefined(this.model.columns[i]["dataSource"]))
                    column.push(this.model.columns[i].field);
            }
            if (column.length)
                return ej.Query().select(column);
            return ej.Query();
        },

        _ensureDataSource: function (args) {
            if (this._dataSource() == null && !(this._dataSource() instanceof ej.DataManager))
                return;
            this.model.query.requiresCount();
            var queryManagar = this.model.query;
            if (!(this._dataSource() instanceof ej.DataManager))
                this.model.currentViewData = this._dataSource();
            if (!(this._dataSource() instanceof ej.DataManager) && (this.model.edit.allowEditing || this.model.edit.allowAdding) && (!ej.isNullOrUndefined(this._cModifiedData) || !ej.isNullOrUndefined(this._cAddedRecord))) {
                if (ej.isNullOrUndefined(this._cAddedRecord)) {
                    for (var index = 0; index < this._primaryKeys.length; index++)
                        queryManagar = queryManagar.where(this._primaryKeys[index], ej.FilterOperators.equal, this._primaryKeyValues[index]);
                    var currentData = this._dataManager.executeLocal(queryManagar);
                    $.extend(this._dataSource()[$.inArray(currentData.result[0], this._dataSource())], this._cModifiedData);
                    this._cModifiedData = null;
                } else {
                    this._dataSource().unshift(this._cAddedRecord);
                    this._cAddedRecord = null;
                }
                queryManagar.queries = [];
                this.model.isEdit = false;
            }

            if (this.model.edit.allowDeleting && !ej.isNullOrUndefined(this._cDeleteData) && !(this._dataSource() instanceof ej.DataManager)) {
                var index = $.inArray(this._cDeleteData[0], this._dataSource());
                this._dataSource().splice(index, 1);
            }

            if (this.model.sortSettings.sortedColumns.length) {
                for (var i = this.model.sortSettings.sortedColumns.length - 1; i >= 0; i--)
                    queryManagar.sortBy(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);

            }

            if (this.model.allowSearching && this._searchString.length) {
                queryManagar.search(this._searchString, this.getColumnFieldNames(), ej.FilterOperators.contains, true);
                if (args.requestType == "searching")
                    this._currentPage(1);
            }

            if (this.model.allowFiltering && this.model.filterSettings.filteredColumns.length) {
                var predicate;
                var firstFilterCondition = this.model.filterSettings.filteredColumns[0];
                predicate = ej.Predicate(firstFilterCondition.field, firstFilterCondition.operator, firstFilterCondition.value, !firstFilterCondition.matchcase);
                for (var i = 1; i < this.model.filterSettings.filteredColumns.length; i++) {
                    predicate = predicate[this.model.filterSettings.filteredColumns[i].predicate](this.model.filterSettings.filteredColumns[i].field, this.model.filterSettings.filteredColumns[i].operator, this.model.filterSettings.filteredColumns[i].value, !this.model.filterSettings.filteredColumns[i].matchcase);
                }
                queryManagar.where(predicate);
                if (!(this._dataSource() instanceof ej.DataManager)) {
                    this._filteredRecordsCount = this._dataManager.executeLocal(queryManagar).count;
                    var lastPage = (this._filteredRecordsCount % this.model.pageSettings.pageSize == 0) ? (this._filteredRecordsCount / this.model.pageSettings.pageSize) : (parseInt(this._filteredRecordsCount / this.model.pageSettings.pageSize, 10) + 1);
                    if (this._currentPage() > lastPage)
                        this._currentPage(lastPage);
                } else if (!ej.isNullOrUndefined(args) && args.requestType == ej.Grid.actions.Filtering)
                    this._currentPage(1);
                if (this.model.allowSummary && this.model.summaryRows.length > 0) {
                    var fQueryMgr = new ej.Query();
                    this._filteredRecords = this._dataManager.executeLocal(fQueryMgr.where(predicate));
                }
            }


            if (this.model.allowPaging || (this.model.scrolling.allowVirtualScrolling && this.model.allowScrolling)) {
                if (this._currentPage() == 0) {
                    if (this._prevPageNo == 0)
                        this._currentPage(1);
                    else
                        this._currentPage(this._prevPageNo);
                }
                queryManagar.page(this._currentPage(), this.model.pageSettings.pageSize);
            }

            if (this.model.allowGrouping) {
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                    queryManagar.group(this.model.groupSettings.groupedColumns[i]);
                }
            }
            if (args != undefined && args.requestType == "add" && !(this._dataSource() instanceof ej.DataManager) && this.model.groupSettings.groupedColumns.length == 0 && this.model.scrolling.frozenColumns == 0 && this.model.scrolling.frozenRows == 0)
                this._dataSource().unshift(args.data);
            if (!(this._dataSource() instanceof ej.DataManager)) {
                var result = this._dataManager.executeLocal(queryManagar);
                this.model.currentViewData = result.result;
                this._gridRecordsCount = result.count;
                this._searchCount = this._searchString.length ? result.count : null;
            }

        },
        _renderByFrozenDesign: function () {
            var $div = $(document.createElement('div')), col = this._getMetaColGroup().find("col"), colgroups = {};
            colgroups["colgroup1"] = $div.append(ej.buildTag("colgroup").append(col.splice(0, this.model.scrolling.frozenColumns))).html();
            colgroups["colgroup2"] = $div.html(ej.buildTag("colgroup").append(col)).html();
            this.getGridContent().find("div").first().get(0).innerHTML = $.render[this._id + "_FrozenTemplate"]({ datas: this.model.currentViewData }, colgroups);
            this.setGridContentTable(this.getGridContent().find(".e-table").attr("role", "grid"));
        },
        sendDataRenderingRequest: function (args) {
            if (args.requestType == "add" || this.model.currentViewData.length) {
                switch (args.requestType) {
                    case ej.Grid.actions.Refresh:
                    case ej.Grid.actions.Paging:
                    case ej.Grid.actions.Sorting:
                    case ej.Grid.actions.Filtering:
                    case ej.Grid.actions.Save:
                    case ej.Grid.actions.Cancel:
                    case ej.Grid.actions.Delete:
                    case ej.Grid.actions.Search:
                    case ej.Grid.actions.Reorder:
                    case ej.Grid.actions.BulkSave:
                        if (this.model.groupSettings.groupedColumns.length == 0) {
                            var temp = document.createElement('div');
                            this.getGridContentTable().find("colgroup").first().replaceWith(this._getMetaColGroup());
                            this.model.detailTemplate != null && this.getGridContentTable().find("colgroup").first().prepend(this._getIndentCol());
                            var currentPage = this._currentPage();
                            if (this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate")
                                $("#" + this._id + "_externalEdit").css("display", "none");
                            if (this.model.scrolling.frozenColumns > 0)
                                temp.innerHTML = this._renderByFrozenDesign();
                            else {
                                temp.innerHTML = ['<table>', $.render[this._id + "_JSONTemplate"](this.model.currentViewData), '</table>'].join("");
                                tableEle = this.getGridContentTable().get(0);
                                tbodyEle = tableEle.lastChild;
                                if ((args.requestType == "save" || args.requestType == "cancel") && this.model.edit.editMode != "batch") {
                                    rowIndex = this._selectedRow();
                                    if (this.model.detailTemplate != null) {
                                        rowTr = $(tbodyEle.childNodes[rowIndex]);
                                        rowEle = $(tbodyEle.childNodes).not('.e-detailrow');
                                        for (var i = 0; i < rowEle.length; i++) {
                                            if (rowTr.is(rowEle[i]))
                                                rowIndex = i;
                                        }
                                    }

                                    if (this.model.edit.editMode == "normal" || this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate") {
                                        $newChild = temp.firstChild.firstChild.firstChild;
                                        $oldChild = this.getGridContentTable().find('.e-addedrow').get(0);
                                        $editedTr = this.getGridContentTable().find('.e-editedrow');
                                        if ($editedTr.length) {
                                            $newChild = temp.firstChild.firstChild.childNodes[rowIndex];
                                            if (this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate") {
                                                $oldChild = $editedTr.prev('tr').get(0);
                                                $editedTr.remove();
                                            }
                                            else
                                                $oldChild = $editedTr.get(0);
                                            if (args.requestType == "cancel") {
                                                $($oldChild).replaceWith($($newChild));
                                            }
                                            else if (!ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount < this._previousFilterCount) {
                                                if (this.model.detailTemplate != null && $($oldChild).next('tr.e-detailrow').length)
                                                    tbodyEle.removeChild($($oldChild).next('tr.e-detailrow').get(0));
                                                $($oldChild).remove();
                                                if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                                                    tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                            }
                                            else {
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                            }
                                        }
                                        else {
                                            if (args.requestType == "cancel" || (args.requestType == "save" && !ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount == this._previousFilterCount)) {
                                                $($oldChild).remove();
                                                if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                                                    tableEle.lastChild.appendChild(temp.firstChild.firstChild.lastChild);
                                            }
                                            else if (this.model.currentViewData.length == 1) {
                                                $(tbodyEle).empty();
                                                tbodyEle.appendChild($newChild);
                                            }
                                            else if (!(this._dataSource() instanceof ej.DataManager))
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                            else {
                                                $($oldChild).remove();
                                                if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                                                    tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                            }
                                        }
                                    }
                                    else if (this.model.edit.editMode == "dialog" || this.model.edit.editMode == "dialogtemplate" || this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate") {
                                        $editedTr = this.element.find('.e-editedrow');
                                        if (args.requestType == "cancel" || (!$editedTr.length && !ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount == this._previousFilterCount)) {
                                            this.model.allowPaging && this._refreshGridPager();
                                            break;
                                        } else if ($editedTr.length) {
                                            $newChild = temp.firstChild.firstChild.childNodes[rowIndex];
                                            $oldChild = tbodyEle.childNodes[rowIndex];
                                            if (this.model.detailTemplate != null)
                                                $oldChild = $(tbodyEle.childNodes).not('.e-detailrow').eq(rowIndex).get(0);
                                            if (!ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount < this._previousFilterCount) {
                                                if (this.model.detailTemplate != null && $($oldChild).next('tr.e-detailrow').length)
                                                    tbodyEle.removeChild($($oldChild).next('tr.e-detailrow').get(0));
                                                $($oldChild).remove();
                                                if (this.model.pageSettings.pageSize <= this.model.currentViewData.length)
                                                    tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                            }
                                            else
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                        }
                                        else if (this.model.currentViewData.length == 1 && this.getGridContentTable().find('td.e-rowcell').length == 0) {
                                            $newChild = temp.firstChild.firstChild.firstChild;
                                            $(tbodyEle).empty();
                                            tbodyEle.appendChild($newChild);
                                        }
                                        else if (!(this._dataSource() instanceof ej.DataManager)) {
                                            newChild = temp.firstChild.firstChild.firstChild;
                                            this.getGridContentTable().find('tbody').first().prepend($(newChild));
                                            if (this.model.pageSettings.pageSize <= this.model.currentViewData.length)
                                                tbodyEle.removeChild(tbodyEle.lastChild);
                                            if (this.model.detailTemplate != null && $(tableEle.lastChild.lastChild).children('.e-detailrowexpand').length)
                                                tbodyEle.removeChild(tbodyEle.lastChild);
                                        }
                                    }
                                    if (this.model.altRow)
                                        this._refreshAltRow();
                                }
                                else if (args.requestType == "delete") {
                                    if (this.model.pageSettings.pageSize <= this.model.currentViewData.length)
                                        tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                    if (this.model.detailTemplate != null) {
                                        var visibleRow = this.getGridContentTable().find('.e-detailrow:visible');
                                        $.each(visibleRow, function (indx, item) {
                                            if (visibleRow.eq(indx).closest('tr').prev().children('.e-detailrowexpand').length == 0)
                                                visibleRow.eq(indx).remove();
                                        });
                                    }
                                    if (this.model.altRow)
                                        this._refreshAltRow();
                                }
                                else
                                    this.getGridContentTable().get(0).replaceChild(this.model.rowTemplate != null ? temp.firstChild.lastChild : temp.firstChild.firstChild, this.getGridContentTable().get(0).lastChild);

                            }
                            this._currentJsonData = this.model.currentViewData;
                            this._gridRows = this.getGridContentTable().get(0).rows;
                            if (this.model.scrolling.frozenColumns > 0)
                                this._gridRows = [this._gridRows, this.getGridContentTable().get(1).rows];
                            this._eventBindings();
                            var model = {};
                            if ((args.requestType == "sorting" || args.requestType == "filtering") && this.model.scrolling.allowVirtualScrolling) {
                                if (args.requestType == "filtering") {
                                    this.getGridContent().first().ejScroller("refresh").ejScroller("isVScroll") ? this.element.find(".gridheader").addClass("e-scrollcss") : this.element.find(".gridheader").removeClass("e-scrollcss");
                                    var model = this._refreshVirtualPagerInfo();
                                }
                                this._refreshVirtualContent(currentPage, model);
                                args.requestType == "filtering" && this.getGridContent().first().ejScroller("refresh");
                            }
                            if (this.model.allowPaging) {
                                if (this.model.filterSettings.filteredColumns.length)
                                    this.getGridPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
                                else
                                    this.getGridPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
                                this._refreshGridPager();
                            }
                            break;
                        }
                    case ej.Grid.actions.Grouping:
                        if (this.model.groupSettings.groupedColumns.length && this.model.currentViewData) {
                            this._currentJsonData = [];
                            var $tbody = this.getGridContentTable().children('tbody');
                            $tbody.empty();
                            var temp = document.createElement('div');
                            if (!this.model.groupSettings.groupedColumnShow) {
                                this._hideHeaderColumn(this.model.groupSettings.groupedColumns, true);
                                this.getGridContentTable().children("colgroup").replaceWith(this.getGridHeaderTable().find('colgroup').clone());
                            }
                            if (args.requestType == "reorder")
                                this._isReorder = true;
                            else
                                this._isReorder = false;
                            var $col = this.getGridContentTable().children("colgroup").find('col');
                            var length = $col.length - this.model.columns.length;
                            if (this.model.detailTemplate != null)
                                length--;
                            if ($col.length > this.model.columns.length)
                                this.getGridContentTable().children("colgroup").find('col:lt(' + length + ')').remove();
                            this.getGridContentTable().find("colgroup").first().replaceWith(this._getMetaColGroup());
                            for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                                this.getGridContentTable().children("colgroup").prepend(this._getIndentCol());
                            temp.innerHTML = ['<table>', $.render[this._id + "_GroupingTemplate"](this.model.currentViewData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                            this.getGridContentTable().get(0).replaceChild(temp.firstChild.firstChild, this.getGridContentTable().get(0).lastChild);
                            this._groupingAction();
                            this._recalculateIndentWidth();
                            this._gridRows = this.getGridContentTable().find(".e-recordtable").find("tbody").find("tr").toArray();
                            this._eventBindings();
                        }
                        break;
                    case ej.Grid.actions.BeginEdit:
                        var editingManager = ej.DataManager(this._currentJsonData);
                        var queryManager = new ej.Query();
                        if (this.model.allowFiltering)
                            this._previousFilterCount = this._filteredRecordsCount;
                        for (var index = 0; index < this._primaryKeys.length; index++)
                            queryManager = queryManager.where(this._primaryKeys[index], ej.FilterOperators.equal, this._primaryKeyValues[index]);
                        this._currentData = editingManager.executeLocal(queryManager);
                        var temp = document.createElement('div');
                        var $temp = $(temp), $tempSecondTR;
                        if (this.model.edit.editMode == "normal") {
                            temp.innerHTML = ['<table>', $.render[this._id + "_JSONEditingTemplate"](this._currentData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                            var $tr = $temp.find("tr").first(), detailCount = 0, $tempFirstTR, firstHidden = this.model.columns.length
                            , $currentTrFr = args.currentTr.first();
                            if (this.model.scrolling.frozenColumns > 0) {
                                $temp.prepend(['<table>', $.render[this._id + "_JSONFrozenEditingTemplate"](this._currentData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join(""));
                                $tr.splice(0, 0, $temp.find("table").first().find("tr").first().get(0));
                                $currentTrLa = args.currentTr.last();
                                $tempLastTR = $tr.last();
                            }
                            $tempFirstTR = $temp.find("tr").first();
                            $temp.find('td').not(".e-rowcell").addClass("e-editcell e-normaledit");
                            var $select = $temp.find("select.e-field");
                            for (var i = 0; i < $select.length; i++)
                                $select.eq(i).val(this._currentData[0][$select[i].name]);
                            if (this.model.groupSettings.groupedColumns.length >= 2) {
                                var $indentCell = args.currentTr.find("td.e-indentcell");
                                $temp.find("tr").first().prepend($indentCell);
                            }
                            var hideCount = !this.model.groupSettings.groupedColumnShow ? this.model.groupSettings.groupedColumns.length : 0;
                            if (this.model.detailTemplate != null) {
                                detailCount++;
                                $temp.find(".e-editcell").find("tr").prepend(args.currentTr.find("[class*='detailrow']"));
                            }
                            if (this.model.scrolling.frozenColumns > 0) {
                                $temp.find(".e-editcell").get(1).colSpan = this.model.columns.length - this.model.scrolling.frozenColumns - args.currentTr.last().find(".e-hide").length - hideCount + detailCount;
                                firstHidden = this.model.scrolling.frozenColumns;
                                $currentTrLa.hasClass("e-alt_row") && $tempLastTR.addClass("e-alt_row")
                            }
                            $temp.find(".e-editcell").get(0).colSpan = firstHidden - $currentTrFr.find(".e-hide").length - hideCount + detailCount;
                            $currentTrFr.hasClass("e-alt_row") && $tempFirstTR.addClass("e-alt_row");
                            $currentTrFr.empty().replaceWith($tempFirstTR.addClass("e-editedrow"));
                            if (this.model.scrolling.frozenColumns > 0)
                                $currentTrLa.empty().replaceWith($tempLastTR.addClass("e-editedrow"));
                            this._refreshUnboundTemplate($tr.find(".gridform"));
                            this._gridRows = this.getGridContentTable().first().find(".e-rowcell").closest("tr").toArray();
                            if (this.model.scrolling.frozenColumns > 0)
                                this.getScrollObject().scrollY(this.getScrollObject().model.scrollTop, true);
                            if (this.model.scrolling.frozenColumns > 0)
                                this._gridRows = [this._gridRows, this.getGridContentTable().last().find(".e-rowcell").closest("tr").toArray()];
                        }
                        else if (this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate") {
                            var hideCount = !this.model.groupSettings.groupedColumnShow ? this.model.groupSettings.groupedColumns.length : 0;
                            var detailCount = 0;
                            temp.innerHTML = this.model.edit.editMode == "inlineform" ? $.render[this._id + "_JSONDialogEditingTemplate"](this._currentData) : $.render[this._id + "_JSONdialogTemplateMode"](this._currentData);
                            var $select = $temp.find("select.e-field");
                            for (var i = 0; i < $select.length; i++)
                                $select.eq(i).val(this._currentData[0][$select[i].name]);
                            var tr = ej.buildTag('tr');
                            var td = ej.buildTag('td');
                            tr.addClass("e-editedrow");
                            td.addClass("e-inlineformedit");
                            if (this.model.scrolling.frozenColumns > 0)
                                td.prop("colspan", this.model.columns.length - this.model.scrolling.frozenColumns - this._hiddenColumns.length - hideCount + detailCount);
                            else
                                td.prop("colspan", this.model.columns.length - this._hiddenColumns.length - hideCount + detailCount);
                            td.html($(temp).children());
                            tr.append(td);
                            if (this.model.scrolling.frozenColumns > 0) {
                                var $trClone = tr.clone();
                                $trClone.find("td").empty().prop("colspan", this.model.scrolling.frozenColumns);
                                args.currentTr.eq(1).after(tr).end().eq(0).after($trClone);
                                this._gridRows = [this._gridRows, this.getGridContentTable().last().find("tr").toArray()];
                            }
                            else
                                args.currentTr.after(tr);
                            this._gridRows = this.getGridContentTable().first().find(".e-rowcell,.e-inlineformedit").closest("tr").toArray();
                            if (this.model.scrolling.frozenColumns > 0)
                                this._gridRows = [this._gridRows, this.getGridContentTable().last().find("tr").toArray()];
                            $("#" + this._id + "_inlineFormTitle").text(this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]]);
                        }
                        else {
                            $temp.addClass("e-editedrow");
                            temp.innerHTML = this.model.edit.editMode == "dialog" || this.model.edit.editMode == "externalform" ? $.render[this._id + "_JSONDialogEditingTemplate"](this._currentData) : $.render[this._id + "_JSONdialogTemplateMode"](this._currentData);
                            var $select = $temp.find("select.e-field");
                            for (var i = 0; i < $select.length; i++)
                                $select.eq(i).val(this._currentData[0][$select[i].name]);
                            if (this.model.edit.editMode == "dialog" || this.model.edit.editMode == "dialogtemplate") {
                                $("#" + this._id + "_dialogEdit").html($(temp));
                                var model = {};
                                model.cssClass = this.model.cssClass;
                                model.rtl = this.model.rtl;
                                model.width = "auto";
                                model.resizable = false;
                                model.close = $.proxy(this._buttonClick, this);
                                model.contentContainer = "#" + this._id;
                                model.modal = true;
                                model.allowKeyboardNavigation = false;
                                model.title = this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]];
                                $("#" + this._id + "_dialogEdit").ejDialog(model);
                                $("#" + this._id + "_dialogEdit").ejDialog("open");
                            }
                            else {
                                $("#" + this._id + "_externalEdit").css("display", "block").css('z-index', this._maxZindex() + 1);
                                $(".e-externalformedit").html($(temp));
                                $("#" + this._id + "_eFormHeader").find(".e-form-title").text(this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]]);
                                this._externalFormPosition();
                            }
                        }
                        break;
                    case ej.Grid.actions.Add:
                        var temp = document.createElement('div');
                        if (!(this._dataSource() instanceof ej.DataManager) && this.model.groupSettings.groupedColumns.length == 0 && this.model.scrolling.frozenColumns == 0 && this.model.scrolling.frozenRows == 0)
                            this._dataSource().splice(0, 1);
                        this._previousFilterCount = this._filteredRecordsCount;
                        if (this.model.edit.editMode == "normal") {
                            var $tempFirstTR, $temp = $(temp), frozenColSpan = this.model.columns.length;
                            temp.innerHTML = ['<table>', $.render[this._id + "_JSONEditingTemplate"](args.data, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                            var $select = $(temp).find('select.e-field');
                            for (var i = 0; i < $select.length; i++)
                                $select.eq(i).val(args.data[$select[i].name]);
                            if (this.model.scrolling.frozenColumns > 0) {
                                $tempLastTR = $temp.find("table").first().find("tr").first();
                                $temp.prepend(['<table>', $.render[this._id + "_JSONFrozenEditingTemplate"](args.data, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join(""));
                            }
                            $tempFirstTR = $temp.find("tr").first();
                            var td = $(temp).find(".e-editcell").get(0);
                            if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                                this.getGridContentTable().get(0).lastChild.removeChild(this.getGridContentTable().get(0).lastChild.lastChild);
                            if (this.model.detailTemplate != null && $(this.getGridContentTable().get(0).lastChild.lastChild).children('.e-detailrowexpand').length)
                                this.getGridContentTable().get(0).lastChild.removeChild(this.getGridContentTable().get(0).lastChild.lastChild);
                            if (this.model.currentViewData.length == 0 || this.getGridContentTable().find('td.e-rowcell').length == 0)
                                this.getGridContentTable().find('tr').first().replaceWith($(temp).find("tr").first().addClass("e-addedrow e-normaledit"));
                            else {
                                this.getGridContentTable().first().find('tbody').first().prepend($tempFirstTR.addClass("e-addedrow e-normaledit"));
                                if (this.model.scrolling.frozenColumns > 0)
                                    this.getGridContentTable().last().find('tbody').first().prepend($tempLastTR.addClass("e-addedrow e-normaledit"));
                            }
                            var $editTr = this.getGridContentTable().find("tr.e-addedrow");
                            if (this.model.detailTemplate != null)
                                $editTr.find('tr').first().prepend(ej.buildTag('td.e-detailrowcollapse'));
                            var hideCount = !this.model.groupSettings.groupedColumnShow ? this.model.groupSettings.groupedColumns.length : 0;
                            if (this.model.groupSettings.groupedColumns.length) {
                                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                                    if (i == 0)
                                        $editTr.prepend(ej.buildTag("td.e-indentcell"));
                                    else {
                                        $editTr.find("tr").prepend(ej.buildTag("td.e-indentcell"));
                                    }
                                }
                                if (this.model.groupSettings.groupedColumns.length >= 2)
                                    $editTr.find("td.e-editcell").prop("colspan", (this.model.columns.length + (this.model.groupSettings.groupedColumns.length - 1) - this._hiddenColumns.length - hideCount));
                                else
                                    $editTr.find("td.e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount));
                            } else if (this.model.detailTemplate != null)
                                $editTr.find(".e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount + 1));
                            else {
                                if (this.model.scrolling.frozenColumns > 0) {
                                    $editTr.find(".e-editcell").last().prop("colspan", (this.model.columns.length - this.model.scrolling.frozenColumns - $tempLastTR.find(".e-hide").length - hideCount));
                                    frozenColSpan = this.model.scrolling.frozenColumns;
                                }
                                $editTr.find(".e-editcell").first().prop("colspan", (frozenColSpan - $tempFirstTR.find(".e-hide").length - hideCount));

                            }
                            this._refreshUnboundTemplate($editTr.find(".gridform"));
                            this._gridRows = this.getGridContentTable().first().find(".e-rowcell").closest("tr").toArray();
                            if (this.model.scrolling.frozenColumns > 0)
                                this._gridRows = [this._gridRows, this.getGridContentTable().last().find(".e-rowcell").closest("tr").toArray()];
                        }
                        else if (this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate") {
                            temp.innerHTML = this.model.edit.editMode == "inlineform" ? $.render[this._id + "_JSONDialogEditingTemplate"](args.data) : $.render[this._id + "_JSONdialogTemplateMode"](args.data);
                            var $select = $(temp).find('select.e-field');
                            for (var i = 0; i < $select.length; i++)
                                $select.eq(i).val(args.data[$select[i].name]);
                            var hideCount = !this.model.groupSettings.groupedColumnShow ? this.model.groupSettings.groupedColumns.length : 0;
                            var detailCount = 0;
                            var tr = ej.buildTag('tr');
                            var td = ej.buildTag('td');
                            tr.addClass("e-addedrow");
                            td.addClass("e-inlineformedit e-editcell");
                            td.html($(temp).children());
                            tr.append(td);
                            var hideCount = !this.model.groupSettings.groupedColumnShow ? this.model.groupSettings.groupedColumns.length : 0;
                            if (this.model.groupSettings.groupedColumns.length) {
                                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                                    tr.prepend(ej.buildTag("td.e-indentcell"));
                                }
                            } else
                                tr.find("td.e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount));
                            if (this.model.scrolling.frozenColumns > 0) {
                                var $trClone = tr.clone(), $divs = td.find(".gridform").children();
                                $trClone.find("td").empty().prop("colSpan", this.model.scrolling.frozenColumns - this._getHiddenCount($divs.slice(0, this.model.scrolling.frozenColumns)));
                                td.prop("colSpan", this.model.columns.length - this.model.scrolling.frozenColumns - this._getHiddenCount($divs.slice(this.model.scrolling.frozenColumns)));
                                this.getGridContentTable().first().find('tbody').first().prepend($trClone);
                                this.getGridContentTable().last().find('tbody').first().prepend(tr);
                            }
                            else {
                                td.prop("colspan", this.model.columns.length - this._hiddenColumns.length - hideCount + detailCount);
                                if (this.model.currentViewData.length == 0 || this.getGridContentTable().find('td.e-rowcell').length == 0)
                                    this.getGridContentTable().find('tr').first().replaceWith($(tr));
                                else
                                    this.getGridContentTable().find('tbody').first().prepend(tr);
                            }
                            if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                                this.getGridContentTable().get(0).lastChild.removeChild(this.getGridContentTable().get(0).lastChild.lastChild);
                            if (this.model.detailTemplate != null && $(this.getGridContentTable().get(0).lastChild.lastChild).children('.e-detailrowexpand').length)
                                this.getGridContentTable().get(0).lastChild.removeChild(this.getGridContentTable().get(0).lastChild.lastChild);
                            $("#" + this._id + "_inlineFormTitle").text(this._getLocalizedLabels("AddFormTitle"));
                            this._refreshUnboundTemplate($("#" + this._id + "EditForm"));
                            this._gridRows = this.getGridContentTable().first().find(".e-rowcell,.e-inlineformedit").closest("tr").toArray();
                            if (this.model.scrolling.frozenColumns > 0)
                                this._gridRows = [this._gridRows, this.getGridContentTable().last().find(".e-rowcell").closest("tr").toArray()];
                        }
                        else {
                            $(temp).addClass("e-addedrow");
                            temp.innerHTML = this.model.edit.editMode == "dialog" || this.model.edit.editMode == "externalform" ? $.render[this._id + "_JSONDialogEditingTemplate"](args.data) : $.render[this._id + "_JSONdialogTemplateMode"](args.data);
                            var $select = $(temp).find('select.e-field');
                            for (var i = 0; i < $select.length; i++)
                                $select.eq(i).val(args.data[$select[i].name]);
                            if (this.model.edit.editMode == "dialog" || this.model.edit.editMode == "dialogtemplate") {
                                $("#" + this._id + "_dialogEdit").html($(temp));
                                var model = {};
                                model.cssClass = this.model.cssClass;
                                model.modal = true;
                                model.width = "auto";
                                model.resizable = false;
                                model.contentContainer = "#" + this._id;
                                model.modal = true;
                                model.close = $.proxy(this._buttonClick, this);
                                model.rtl = this.model.rtl;
                                model.allowKeyboardNavigation = false;
                                model.title = this._getLocalizedLabels("AddFormTitle");
                                $("#" + this._id + "_dialogEdit").ejDialog(model);
                                $("#" + this._id + "_dialogEdit").ejDialog("open");
                            }
                            else {
                                $("#" + this._id + "_externalEdit").css("display", "block").css('z-index', this._maxZindex() + 1);
                                $(".e-externalformedit").html($(temp));
                                $("#" + this._id + "_eFormHeader").find(".e-form-title").text(this._getLocalizedLabels("AddFormTitle"));
                                this._externalFormPosition();
                            }
                        }
                        if (this.model.allowPaging) {
                            if (this.model.filterSettings.filteredColumns.length)
                                this.getGridPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
                            else
                                this.getGridPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
                            this._refreshGridPager();
                        }
                        break;
                    case ej.Grid.actions.Ungrouping:
                        var $header = this.element.children(".e-gridheader");
                        var $filterInput = $header.find(".e-filterbar").find("th").find("input");
                        $header.find("div").first().empty().append(this._renderGridHeader().find("table"));
                        this.setGridHeaderContent($header);
                        if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar") {
                            this._renderFiltering();
                            var filterThNew = this.element.children(".e-gridheader").find(".e-filterbar").find("th").find("input");
                            for (var i = 0; i < $filterInput.length; i++)
                                filterThNew.eq(i).val($filterInput.eq(i).val());
                        }
                        if (!this.model.groupSettings.groupedColumnShow)
                            this._hideHeaderColumn(this.model.groupSettings.groupedColumns, true);
                        this.element.find(".e-gridcontent").children("div").first().empty().append(this._renderGridContent().find("table").first());
                        this.setGridContent(this.element.find(".e-gridcontent"));
                        //this._gridRows = this.getGridContentTable().get(0).rows;
                        //this._eventBindings();
                        break;
                    case ej.Grid.actions.VirtualScroll:
                        !this._isVirtualRecordsLoaded && this._replacingContent();
                        break;
                }
            } else
                this.getGridContentTable().find('tbody').empty().first().append(this._getEmptyTbody());
            this._completeAction(args);
        },
        _getHiddenCount: function (elements) {
            var count = 0;
            for (var i = 0; i < elements.length; i++) {
                if (elements.eq(i).hasClass("e-hide"))
                    count++;
            }
            return count;
        },
        _replacingContent: function () {
            var temp = document.createElement('div');
            var currentIndex = this.getCurrentIndex();
            var contentTable = this.getGridContentTable()[0];
            var colGroup = $(contentTable).find("colgroup").first();
            var rowHeight = this.getGridRowHeight();
            colGroup.replaceWith(this._getMetaColGroup());
            this.model.detailTemplate != null && colGroup.prepend(this._getIndentCol());
            var tbody = contentTable.tBodies[0];
            var cloneEle = $(tbody).clone();
            var elementTbody = $("<tbody></tbody>").append($.render[this._id + "_JSONTemplate"](this.model.currentViewData));
            var proxy = this;
            this.virtualLoadedPages.push(currentIndex >= 0 ? currentIndex : 0);
            var orderedVirtualPages = ej.dataUtil.mergeSort(ej.distinct(this.virtualLoadedPages));
            var tempTbody = $("<tbody></tbody>");
            $(elementTbody).children('tr').attr('name', currentIndex);
            var minValue = ej.dataUtil.min(orderedVirtualPages);
            var maxValue = ej.dataUtil.max(orderedVirtualPages);
            $(cloneEle).find('tr.e-virtualrow').remove();
            $(cloneEle).append($(elementTbody).children('tr'));
            $.each(orderedVirtualPages, function (index, val) {
                $(tempTbody).append($(cloneEle).find('tr[name=' + val + ']'));
                if (val != 0) {
                    var prevValue = val == minValue ? minValue : orderedVirtualPages[index - 1];
                    var virtualTRMiddle = document.createElement("tr");
                    var middleRows = val - prevValue - proxy.model.pageSettings.pageSize;
                    if (middleRows > 0)
                        $(virtualTRMiddle).addClass("e-virtualrow").css("height", rowHeight * middleRows).insertBefore($(tempTbody).children('tr[name=' + val + ']:first'));
                }
                if (val == maxValue) {
                    var virtualTRBottom = document.createElement("tr");
                    var bottomRows = proxy._gridRecordsCount - maxValue - proxy.model.pageSettings.pageSize;
                    if (bottomRows > 0)
                        $(virtualTRBottom).addClass("e-virtualrow").css("height", rowHeight * bottomRows).appendTo(tempTbody);
                }
            });
            if (minValue > 0) {
                var virtualTRTop = document.createElement("tr");
                $(virtualTRTop).addClass("e-virtualrow").css("height", rowHeight * minValue).prependTo(tempTbody);
            }
            var focusTR = $(tbody).html($(tempTbody).html()).find("tr[name=" + currentIndex + "]")[0];
            if ((focusTR && focusTR.previousSibling && ($(focusTR.previousSibling).hasClass("e-virtualrow") || focusTR.previousSibling.offsetTop > (currentIndex * this.getGridContent().height()))
            && (this._gridRecordsCount - currentIndex > this.model.pageSettings.pageSize || focusTR.offsetParent.offsetHeight - focusTR.offsetTop < this.getGridContent().height())) || this._lastRow) {
                if (this._lastRow) this._lastRow = false;
                this._isVirtualRecordsLoaded = true;
                this.getGridContent().children("div").first().scrollTop(this.getGridContent().find(".content").scrollTop() - (this.getGridContent().find(".content").scrollTop() - focusTR.offsetTop));
            }
            $(temp).append($("<table></table>").append(tempTbody));
            $(contentTable).get(0).replaceChild(this.model.rowTemplate != null ? temp.firstChild.lastChild : temp.firstChild.firstChild, $(contentTable).get(0).lastChild);
            this._currentJsonData = this.model.currentViewData;
            this._gridRows = $(contentTable).get(0).rows;
            var lastVirtualRow = $(contentTable).find(".e-virtualrow").last();
            lastVirtualRow.css("height", (lastVirtualRow.height() - ($(contentTable).height() - (this._gridRecordsCount * rowHeight))));
            this._eventBindings();
        },
        _hideHeaderColumn: function (hiddenColumns, field) {
            var $head = this.getGridHeaderTable().find("thead");
            var $headerCell = $head.find(".e-headercell");
            var $filterBarCell = $head.find(".e-filterbar").find(".e-filterbarcell");
            var $col = this.getGridHeaderTable().find("colgroup").find("col");
            for (var i = 0; i < hiddenColumns.length; i++) {
                if (field)
                    var column = this.getColumnByField(hiddenColumns[i]);
                else
                    var column = this.getColumnByHeaderText(hiddenColumns[i]);
                var index = $.inArray(column, this.model.columns);
                var thIndex = $headerCell.eq(index).addClass("e-hide").index();
                $filterBarCell.eq(thIndex).addClass("e-hide");
                if ($col.length > this.model.columns.length)
                    $col = $col.slice($col.length - this.model.columns.length);
                $col.eq(index).css("display", "none");
            }

        },
        _hideSummaryColumn: function (td, col) {
            if (col.length > this.model.columns.length)
                col = col.slice(col.length - this.model.columns.length);
            for (i = 0; i < this.model.columns.length; i++) {
                if (!this.model.columns[i]["visible"]) {
                    col.eq(i).css("display", "none");
                    $(td[i]).addClass("e-hide");
                }
                else {
                    if (col.eq(i).css("display") == "none")
                        col.eq(i).css("display", "");
                }
            }
        },
        _showHeaderColumn: function (showColumns, field) {
            var $head = this.getGridHeaderTable().find("thead");
            var $headerCell = $head.find(".e-headercell");
            var $filterBarCell = $head.find(".e-filterbar").find(".e-filterbarcell");
            var $col = this.getGridHeaderTable().find("colgroup").find("col");
            for (var i = 0; i < showColumns.length; i++) {
                if (field)
                    var column = this.getColumnByField(showColumns[i]);
                else
                    var column = this.getColumnByHeaderText(showColumns[i]);
                var index = $.inArray(column, this.model.columns);
                var thIndex = $headerCell.eq(index).removeClass("e-hide").index();
                $filterBarCell.eq(thIndex).removeClass("e-hide");
                $col.eq(index).css("display", "");
            }
        },
        _groupingAction: function () {
            var $groupTopCell = this.getGridHeaderTable().find("thead").find(".e-columnheader").find(".e-grouptopleftcell"), $col = this.getGridHeaderTable().find("colgroup").find("col");
            var groupColumn = $groupTopCell.length;
            if (groupColumn) {
                this.getGridHeaderTable().find("colgroup").replaceWith(this._getMetaColGroup());
                this.model.detailTemplate != null && this.getGridHeaderTable().find("colgroup").prepend(this._getIndentCol());
                $groupTopCell.remove();
                this.getGridHeaderTable().find("thead").find(".e-filterbar").find(".e-filterbarcell:lt(" + groupColumn + ")").remove();
            }
            !this.model.allowResizeToFit && this.setWidthToColumns();
            for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                this.getGridHeaderTable().find("colgroup").prepend(this._getIndentCol());
                this.getGridHeaderTable().find("thead").find(".e-columnheader").prepend(this._getGroupTopLeftCell());
                this.getGridHeaderTable().find("thead").find(".e-filterbar").prepend(this._getEmptyFilterBarCell());

            }
            this.getGridHeaderTable().find("thead").find("th.e-grouptopleftcell").last().addClass("e-lastgrouptopleftcell");
        },
        _completeAction: function (args) {
            this.model.isEdit = false;
            var imageDirection;
            this._confirmedValue = false;
            if (ej.Grid.actions.Grouping == args.requestType && ej.isNullOrUndefined(args.columnName))
                return;
            //if (this.model.groupSettings.groupedColumns.length == 0)
            this.setWidthToColumns();
            if (ej.Grid.actions.Paging == args.requestType)
                this._refreshGridPager();
            else if ((ej.Grid.actions.Sorting == args.requestType && this.model.allowSorting) || ej.Grid.actions.Refresh == args.requestType) {
                this.getGridHeaderTable().find(".e-columnheader").find(".e-headercelldiv")
                    .find(".e-ascending,.e-descending").remove();
                this.getGridHeaderTable().find("[aria-sort]").removeAttr("aria-sort");
                for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                    this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
                if (this.model.groupSettings.groupedColumns.length && this._$curSElementTarget != null) {
                    var $element = this._checkEinGroupDrop($.trim(this._$curSElementTarget.text()));
                    if (!ej.isNullOrUndefined($element)) {
                        imageDirection = args.columnSortDirection == "ascending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
                        $element.find(".e-ascending,.e-descending").removeClass().addClass("e-icon e-" + args.columnSortDirection + " " + imageDirection);
                    }
                }
                this._refreshGridPager();
                if (!this._initialRender && (this.model.scrolling.frozenRows > 0 || this.model.scrolling.frozenColumns > 0))
                    this._refreshScroller(args);
                this._multiSortRequest = false;
            } else if ((ej.Grid.actions.Delete == args.requestType || ej.Grid.actions.Save == args.requestType || ej.Grid.actions.Search == args.requestType) && this.model.allowPaging) {
                if (this.model.filterSettings.filteredColumns.length)
                    this.getGridPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
                else
                    this.getGridPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
            } else if (ej.Grid.actions.Filtering == args.requestType) {
                this._refreshGridPager();
                if (this.model.filterSettings.filterType == "menu") {
                    this._closeFilterDlg();
                    var column = this.getColumnByField(this._$curFieldName);
                    var index = $.inArray(column, this.model.columns), proxy = this, _addicon = false;
                    $.each(this.model.filterSettings.filteredColumns, function (indx, col) {
                        if (col.field == proxy._$curFieldName) {
                            _addicon = true;
                            return false;
                        }
                    });
                    var $fIcon = this.getGridHeaderTable().find("thead").find(".e-headercell").eq(index).find(".e-filtericon");
                    if (_addicon)
                        $fIcon.addClass("e-filteredicon e-filternone");
                    else
                        $fIcon.removeClass("e-filteredicon e-filternone");
                }
            } else if (ej.Grid.actions.BeginEdit == args.requestType || ej.Grid.actions.Add == args.requestType) {
                var $form = this.element.find(".gridform");
                this.model.isEdit = true;
                this.setWidthToColumns();
                if (ej.Grid.actions.Add == args.requestType) {
                    $form.find(".e-field:disabled").not(".e-identity").removeAttr("disabled").removeClass("e-disable");
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length - 1; i++)
                        $form.find("colgroup").prepend(this._getIndentCol());
                }
                if (this._tdsOffsetWidth.length == 0 || this.model.groupSettings.groupedColumns.length)
                    this._setoffsetWidth();
                this.model.edit.editMode != "dialogtemplate" && this._refreshEditForm();
                if (this.model.scrolling.frozenColumns > 0 && (this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate")) {
                    if (args.requestType == "beginedit")
                        args.currentTr.eq(0).next().find("td").height(args.currentTr.eq(1).next().find("td").height());
                    else
                        this.getGridContentTable().first().find("tr").first().find("td").height(this.getGridContentTable().last().find("tr").first().find("td").height());
                }
                if (this.model.scrolling.frozenRows > 0 && args.requestType == "beginedit")
                    this._initFrozenRows();
                if ($.isFunction($.validator))
                    this.setValidation();

            } else if (ej.Grid.actions.Grouping == args.requestType) {
                var $groupDrop = this.element.children(".e-groupdroparea");
                if (this.model.groupSettings.groupedColumns.length && $groupDrop.find(".e-grid-icon").length == 0)
                    $groupDrop.empty();
                if (this._initialRender) {
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                        this._addColumnToGroupDrop(this.model.groupSettings.groupedColumns[i]);
                } else
                    this._addColumnToGroupDrop(args.columnName);
                this.getGridHeaderTable().find(".e-columnheader").find(".e-headercelldiv").find(".e-ascending,.e-descending").remove();
                for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                    this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
                !this._initialRender && this._groupHeaderCelldrag();
                this.model.allowScrolling && this.getGridContentTable().parent().scrollLeft(this.getGridHeaderTable().parent().scrollLeft() - 1);
                this.element.children(".e-cloneproperties").remove();

            } else if (ej.Grid.actions.Ungrouping == args.requestType) {
                var $groupDrop = this.element.children(".e-groupdroparea");
                this._removeColumnFromDrop(args.columnName);
                this.getGridHeaderTable().find(".e-columnheader").find(".e-headercelldiv").find(".e-ascending,.e-descending").remove();
                for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                    this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
                if (this.model.groupSettings.groupedColumns.length == 0) {
                    $groupDrop.html(this.model.enableEffects ? "" : this._getLocalizedLabels("GroupDropArea"))
                    this.model.groupSettings.showAnimateButton && $groupDrop.append(ej.buildTag("div.e-animatebutton e-icon").addClass(this.model.enableEffects ? "e-animatebuttondown e-gdownarrow" : "e-animatebuttonup e-guparrow"));
                    $groupDrop.css("height", "auto");
                }
                this._headerCellgDragDrop();
                this.model.allowScrolling && this.getGridContentTable().parent().scrollLeft(this.getGridHeaderTable().parent().scrollLeft() + 10);
                this.element.children(".e-cloneproperties").remove();
            }
            else if (ej.Grid.actions.BulkSave == args.requestType)
                this._refreshGridPager();
            if (this.model.toolBar.allowToolBar)
                this.refreshToolBar();
            if (!this._initialRender && this.model.allowSummary && this.model.summaryRows.length > 0)
                this._createSummaryRows(this.getGridFooterTable());
            !this._initialRender && this._setCurrentRow(args.requestType);
            this.refreshBatchEditMode();
            if (!this._initialRender && this.model.allowScrolling && (this._checkScrollActions(args.requestType) || (this.model.edit.editMode.indexOf("inline") != -1 && args.requestType == "beginedit")))
                this._refreshScroller(args);
            if (this._customPop != null && args.requestType != "sorting") {
                this._customPop.hide();
            }
            if (this.model.allowScrolling && (ej.Grid.actions.Filtering == args.requestType || ej.Grid.actions.Paging == args.requestType || ej.Grid.actions.Sorting == args.requestType ||
                ej.Grid.actions.Grouping == args.requestType || ej.Grid.actions.Ungrouping == args.requestType)) {
                this.getGridContentTable().find("tr:last").find("td").addClass("e-lastrowcell");
            }
            this._trigger("actionComplete", args);
        },
        _refreshScroller: function (args) {
            var gridContent = this.getGridContent().first(), direction;
            if (ej.isNullOrUndefined(gridContent.data("ejScroller")))
                return;
            if (this.model.scrolling.frozenColumns > 0) {
                direction = this.model.rtl ? "margin-right" : "margin-left";
                gridContent.find(".e-movablecontent").css(direction, gridContent.find(".e-frozencontentdiv").width() + "px");
                this.getGridHeaderContent().find(".e-movableheader").removeAttr("style").css(direction, this.getGridHeaderContent().find(".e-frozenheaderdiv").width() + "px");
                this.refreshScrollerEvent();
                gridContent.find(".e-movablecontent").scrollLeft(this.getGridHeaderContent().find(".e-movableheader").scrollLeft());
            }
            if (this.model.scrolling.frozenRows > 0) {
                this._initFrozenRows();
                var temp = this.getScrollObject().model.scrollTop;
                if (args.requestType == ej.Grid.actions.Add)
                    this.getScrollObject().scrollY(0, true);
                if (!ej.isNullOrUndefined(this.getScrollObject()._v))
                    this.getScrollObject()._v.skipChange = true;
            }
            if (args.requestType == "beginedit") {
                var temp = this.getScrollObject().model.scrollTop;
                this.getScrollObject().scrollY(0, true);
            }
            gridContent.ejScroller("refresh");
            gridContent.ejScroller("model.rtl", this.model.rtl);
            if (gridContent.ejScroller("isVScroll"))
                this.element.find(".e-gridheader").addClass("e-scrollcss");
            else
                this.element.find(".e-gridheader").removeClass("e-scrollcss");
            this._getRowHeights();
            if (temp && args.requestType != ej.Grid.actions.Add) {
                this._currentTopFrozenRow = 0;
                this.getScrollObject().scrollY(temp, true);
            }
        },
        _checkScrollActions: function (requestType) {
            if (requestType == ej.Grid.actions.Grouping || requestType == ej.Grid.actions.Ungrouping || requestType == ej.Grid.actions.Add || requestType == ej.Grid.actions.Cancel
                || requestType == ej.Grid.actions.Save || requestType == ej.Grid.actions.Delete || requestType == ej.Grid.actions.Filtering || requestType == ej.Grid.actions.Paging || requestType == ej.Grid.actions.Refresh)
                return true;
            return false;
        },
        refreshBatchEditMode: function () {
            if (this.model.edit.editMode == "batch") {
                this.refreshBatchEditChanges();
                this._bulkEditCellDetails._data = $.extend(true, [], this.getCurrentViewData());
            }
        },
        _refreshGridPager: function () {
            if (this.getGridPager() != null) {
                var pagerModel = this.getGridPager().ejPager("model"), model = {};
                model.currentPage = this._currentPage();
                if (this._filteredRecordsCount == 0 && this.model.currentViewData.length == 0 && (ej.isNullOrUndefined(this._prevPageNo) || this._prevPageNo)) {
                    model.currentPage = 0;
                    this._prevPageNo = pagerModel.currentPage;
                    this.model.pageSettings.currentPage = 0;
                } else if (pagerModel.currentPage == 0 && (ej.isNullOrUndefined(this._prevPageNo) || this._prevPageNo))
                    model.currentPage = this._prevPageNo;
                model.totalRecordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
                if (ej.util.isNullOrUndefined(model.currentPage))
                    model.currentPage = this._currentPage();
                this.getGridPager().ejPager("option", model).ejPager("refreshPager");
            }
        },

        _refreshVirtualPagerInfo: function () {
            var model = {};
            model.pageSize = this.model.pageSettings.pageSize;
            model.currentPage = this._currentPage();
            model.totalRecordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            model.totalPages = Math.ceil(model.totalRecordsCount / model.pageSize);

            return model;
        },
        _refreshUnboundTemplate: function ($target) {
            if (this._isUnboundColumn) {
                var index = 0;
                for (var column = 0; column < this.model.columns.length; column++) {
                    if (this.model.columns[column]["isUnBound"]) {
                        var $unboundDivs = $target.find(".e-unboundcell.e-" + this.model.columns[column]["headerText"].replace(/\s+/g, "")).find(".e-unboundcelldiv");
                        var commands = this.model.columns[column].commands;
                        for (var j = 0; j < commands.length; j++) {
                            var width = ej.isNullOrUndefined(commands[j].buttonOptions.width) ? "52" : commands[j].buttonOptions.width;
                            var height = ej.isNullOrUndefined(commands[j].buttonOptions.height) ? "30" : commands[j].buttonOptions.height;
                            commands[j].buttonOptions.width = ej.isNullOrUndefined(commands[j].buttonOptions.width) ? "52" : commands[j].buttonOptions.width;
                            commands[j].buttonOptions.height = ej.isNullOrUndefined(commands[j].buttonOptions.height) ? "28" : commands[j].buttonOptions.height;
                            commands[j].buttonOptions.cssClass = this.model.cssClass;
                            commands[j].buttonOptions.rtl = this.model.rtl;
                            var $buttons = $unboundDivs.find(".e-" + commands[j].type + "button");
                            if ($target.closest(".e-editcell").length) {
                                if (commands[j].type == "save" || commands[j].type == "cancel")
                                    $buttons.show();
                                else {
                                    $buttons.hasClass("e-deletebutton") && $buttons.hide();
                                    $buttons.hasClass("e-editbutton") && $buttons.hide();
                                }
                            } else {
                                if (commands[j].type == "save" || commands[j].type == "cancel")
                                    $buttons.hide();
                                else {
                                    $buttons.hasClass("e-deletebutton") && $buttons.show();
                                    $buttons.hasClass("e-editbutton") && $buttons.show();
                                }
                            }
                            if ($($buttons[0]).data("ejButton"))
                                $buttons.filter(".e-button").ejButton("destroy");
                            $buttons.ejButton(commands[j].buttonOptions);
                        }
                    } else
                        continue;
                }
            }
        },
        _refreshAltRow: function () {
            $gridRows = this._gridRows;
            for (var r = 0; r < $gridRows.length; r++) {
                $row = $($gridRows[r]);
                $row.hasClass("e-alt_row") && $row.removeClass("e-alt_row");
                if ($row.index() % 2 != 0)
                    $row.addClass("e-alt_row");
            }
        },
        /**
			* It is used to refresh the toolbar items in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.refreshToolBar(); // Refreshes the toolbar items state
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Refreshes the toolbar items state
			* $("#Grid").ejGrid("refreshToolBar");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        refreshToolBar: function () {
            var $toolbar = $("#" + this._id + "_toolbarItems");
            var lis = $toolbar.find("li");
            $toolbar.ejToolbar("enableItem", lis);
            for (var i = 0; i < lis.length; i++) {
                switch (lis[i].id) {
                    case this._id + "_add":
                    case this._id + "_edit":
                    case this._id + "_delete":
                        if (this.model.isEdit) {
                            $(lis[i]).hasClass("e-hover") && $(lis[i]).removeClass("e-hover");
                            this._disabledToolItems.push(lis[i]);
                        }
                        break;
                    case this._id + "_update":
                    case this._id + "_cancel":
                        if (!this.model.isEdit) {
                            $(lis[i]).hasClass("e-hover") && $(lis[i]).removeClass("e-hover");
                            this._disabledToolItems.push(lis[i]);
                        }
                        break;
                }
            }

            $toolbar.ejToolbar("disableItem", this._disabledToolItems);
            $toolbar.ejToolbar("model.rtl", this.model.rtl);
            this._disabledToolItems = $();

        },
        /**
			* It is used to send an delete record request in grid control.
			* @return jQuery
			* @param {object} $tr Pass the tr- selected row element to be deleted in grid
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendDeleteRequest($(".gridcontent tr").first()); // Sends a delete record request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends a delete record request to the grid
			* $("#Grid").ejGrid("sendDeleteRequest", $(".gridcontent tr").first());        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendDeleteRequest: function ($tr) {
            if (!this.model.edit.allowDeleting)
                return;
            if (this._primaryKeys.length == 0 && !this.model.edit.allowEditing && !this.model.edit.allowAdding) {
                for (i = 0; i < this.model.columns.length; i++) {
                    if (this.model.columns[i]["key"] === true) {
                        this._primaryKeys.push($.trim(this.model.columns[i].field));
                        this._primaryKeys = $.unique(this._primaryKeys);
                    }
                }
            }
            if (this._selectedRow() == -1 && ej.isNullOrUndefined($tr)) {
                alert(this._getLocalizedLabels("DeleteOperationAlert"));
                return;
            }
            if (ej.isNullOrUndefined($tr))
                $tr = this.getRowByIndex(this._selectedRow());
            this._primaryKeyValues = [];
            for (var index = 0; index < this._primaryKeys.length; index++) {
                var column = this.getColumnByField(this._primaryKeys[index]);
                var index = $.inArray(column, this.model.columns);
                this._primaryKeyValues.push($tr.find(".e-rowcell").eq(index).html());
            }
            var deleteManager = ej.DataManager(this._currentJsonData);
            var query = new ej.Query();
            for (var i = 0; i < this._primaryKeys.length; i++)
                query = query.where(this._primaryKeys[i], ej.FilterOperators.equal, this._primaryKeyValues[i]);
            currentData = deleteManager.executeLocal(query);
            var args = {};
            args.tr = $tr;
            args.data = currentData[0];
            var foreignKeyData = this._getForeignKeyData(args.data);
            if (!ej.isNullOrUndefined(foreignKeyData))
                args.foreignKeyData = foreignKeyData;
            args.requestType = ej.Grid.actions.Delete;
            if (this._trigger("actionBegin", args))
                return true;
            this._cDeleteData = currentData;
            var promise;
            if (this._dataSource() instanceof ej.DataManager) {
                promise = this._dataManager.remove(this._primaryKeys[0], currentData[0][this._primaryKeys[0]]);
                var proxy = this;
                if ($.isFunction(promise.promise)) {
                    promise.done(function (e) {
                        proxy._processBindings(args);
                    });
                    promise.fail(function (e) {

                    });
                }
                else
                    this._processBindings(args);
            } else
                this._processBindings(args);
            this._primaryKeyValues = [];
            this._cDeleteData = null;
        },
        /**
			* It is used to send an edit record request in grid control.
			* @return jQuery
			* @param {object} $tr Pass the tr- selected row element to be edited in grid
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendEditingRequest($(".gridcontent tr").first()); // Sends an edit record request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends an edit record request to the grid
			* $("#Grid").ejGrid("sendEditingRequest", $(".gridcontent tr").first());        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendEditingRequest: function ($tr) {
            if (!this.model.edit.allowEditing)
                return;
            if (this._selectedRow() == -1 && ej.isNullOrUndefined($tr)) {
                alert(this._getLocalizedLabels("EditOperationAlert"));
                return;
            }
            if (ej.isNullOrUndefined($tr)) {
                this._currentTrIndex = this._selectedRow();
                this._$currentTr = this.getRowByIndex(this._currentTrIndex);
            } else {
                this._currentTrIndex = $tr.index();
                this._$currentTr = $tr;
            }
            for (var index = 0; index < this._primaryKeys.length; index++) {
                var column = this.getColumnByField(this._primaryKeys[index]);
                var index = $.inArray(column, this.model.columns);
                this._primaryKeyValues.push(this._$currentTr.find(".e-rowcell").eq(index).html());
            }
            var args = { currentTr: this._$currentTr, rowIndex: this._currentTrIndex, primaryKey: this._primaryKeys, primaryKeyValue: this._primaryKeyValues };
            var cancel = this._trigger("beginEdit", args);
            if (cancel) {
                this._primaryKeyValues = [];
                return;
            }
            args.requestType = ej.Grid.actions.BeginEdit;
            this._processBindings(args);

        },
        /**
			* It is used to send an add new request in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendAddRequest(); // Sends an add new record request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends an add new record request to the grid
			* $("#Grid").ejGrid("sendAddRequest");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendAddRequest: function () {
            if (!this.model.edit.allowAdding)
                return;
            cloneData = {};
            for (var i = 0; i < this.model.columns.length; i++)
                cloneData[this.model.columns[i].field] = !ej.isNullOrUndefined(this.model.columns[i].defaultValue) ? this.model.columns[i].defaultValue : "";
            var args = {};
            args.data = cloneData;
            var foreignKeyData = this._getForeignKeyData(args.data);
            if (!ej.isNullOrUndefined(foreignKeyData))
                args.foreignKeyData = foreignKeyData;
            args.requestType = "add";
            this.cleanUpSelection();
            var returnValue = this._processBindings(args);
            if (!returnValue)
                this._selectedRow(0);
            var groupedColumns = this.model.groupSettings.groupedColumns.length;
            if (groupedColumns > 1) {
                var $editCol = this.getGridContentTable().find(".e-addedrow").find("table").find("colgroup").children();
                $($editCol.slice(0, groupedColumns - 1)).css('width', this.getGridHeaderTable().find('colgroup').children()[0].style.width);
            }
        },
        /**
			* It is used to send a save request in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendSaveRequest(); // Sends a save request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends a save request to the grid
			* $("#Grid").ejGrid("sendSaveRequest");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendSaveRequest: function () {
            if (this.model.isEdit) {
                var formElement, $formElement;
                if (!this.editFormValidate())
                    return true;
                obj = {};
                var editedRowWrap;
                if (this.model.edit.editMode == "batch")
                    this.saveCell();
                else {
                    formElement = this.model.scrolling.frozenColumns > 0 ? this.element.find(".gridform") : document.getElementById(this._id + "EditForm");
                    $formElement = $(formElement);
                    if (this.model.edit.editMode == "normal" || this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate")
                        editedRowWrap = $formElement.closest('tr');
                    else
                        editedRowWrap = $formElement.closest('div');
                    formElement = this.model.scrolling.frozenColumns > 0 ? formElement[0] : formElement;
                    for (var index = 0; index < formElement.length; index++) {
                        if (editedRowWrap.hasClass("e-addedrow") && $(formElement[index]).hasClass("e-identity"))
                            continue;
                        var columnName = formElement[index].name, $element = $(formElement[index]);
                        if (columnName != undefined) {
                            if (columnName == "")
                                columnName = formElement[index].id.replace(this._id, "");
                            if (columnName != "" && obj[columnName] == null) {
                                var column = this.getColumnByField(columnName), value = formElement[index].value;
                                if ($(formElement[index]).hasClass("e-datepicker")) {
                                    value = $element.ejDatePicker("model.value");
                                    $element.ejDatePicker("destroy");
                                }
                                else if ($(formElement[index]).hasClass("e-datetimepicker")) {
                                    value = $element.ejDateTimePicker("model.value");
                                    $element.ejDateTimePicker("destroy");
                                }
                                else if ($element.is(".e-numerictextbox"))
                                    value = $element.ejNumericTextbox("getValue");
                                else if ($element.data("ejDropDownList")) {
                                    value = $element.ejDropDownList("getSelectedValue");
                                    $element.ejDropDownList("destroy");
                                }
                                if (column == null)
                                    value = !isNaN(parseFloat(value)) && isFinite(value) ? parseFloat(value) : value;
                                else if (column.type == "number" && value.length)
                                    value = parseFloat(value);
                                if (formElement[index].type != "checkbox")
                                    obj[columnName] = value;
                                else
                                    obj[columnName] = $(formElement[index]).is(':checked');
                            }
                        }
                        if (index == formElement.length - 1 && $formElement.length > 1 && $formElement.index(formElement) == 0) {
                            formElement = $formElement[1];
                            index = -1;
                        }

                    }
                    var args = { data: obj };
                    var foreignKeyData = this._getForeignKeyData(args.data);
                    if (!ej.isNullOrUndefined(foreignKeyData))
                        args.foreignKeyData = foreignKeyData;
                    args.requestType = ej.Grid.actions.Save;
                    var currentData;
                    if (this._trigger("actionBegin", args))
                        return true;
                    if (editedRowWrap.hasClass("e-editedrow"))
                        this._cModifiedData = obj;
                    else if (editedRowWrap.hasClass("e-addedrow"))
                        this._cAddedRecord = obj;
                    var promise;
                    if (this._dataSource() instanceof ej.DataManager) {
                        if (!ej.isNullOrUndefined(this._cModifiedData))
                            promise = this._dataManager.update(this._primaryKeys[0], args.data);
                        else
                            promise = this._dataManager.insert(args.data);
                        var proxy = this;
                        if ($.isFunction(promise.promise)) {
                            promise.done(function (e) {
                                proxy._processBindings(args);
                            });
                            promise.fail(function (e) {
                            });
                        } else
                            proxy._processBindings(args);
                    } else
                        this._processBindings(args);
                    this._cModifiedData = null;
                    this._cAddedRecord = null;
                    this._primaryKeyValues = [];
                }
            }
        },
        addRecord: function (data) {
            this._dataManager.insert(data);
            this.getGridContentTable().prepend($.render[this._id + "_JSONTemplate"](data));
            if (this.model.allowPaging && this.getGridPager() != null) {
                var model = this.getGridPager().ejPager("model");
                model.totalRecordsCount++;
                this.getRowByIndex(model.pageSize - 1).remove();
                this.getGridPager().ejPager("refreshPager");
            }
        },
        updateRecord: function (keyField, data, action) {
            this._updateDeleteRecord(keyField, data, "update");
        },
        _updateDeleteRecord: function (keyField, data, action) {
            var dataMgr = ej.DataManager(this._currentJsonData), dataFilter, index, $row, $newrow;
            dataFilter = dataMgr.executeLocal(ej.Query().where(keyField, ej.FilterOperators.equal, data[keyField]));
            if (dataFilter.length) {
                index = $.inArray(dataFilter[0], this._currentJsonData);
                if (index != -1) {
                    $row = this.getRowByIndex(index);
                    if (action == "update") {
                        $newrow = $($.render[this._id + "_JSONTemplate"](data));
                        $row.replaceWith($newrow);
                        !$row.hasClass("e-alt_row") && $newrow.removeClass("e-alt_row");
                    }
                    else {
                        this._currentJsonData.splice(index, 1);
                        $row.remove();
                    }
                }
            }
            this._dataManager[action](keyField, data);
        },
        deleteRecord: function (keyField, data) {
            this._updateDeleteRecord(keyField, data, "remove");
        },
        /**
			* It is used to send a group column request in grid control.
			* @return jQuery
			* @param {string} columnName Pass the field Name of the column to be grouped in grid control
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendGroupingRequest("OrderID"); // Sends a group column request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends a group column request to the grid
			* $("#Grid").ejGrid("sendGroupingRequest", "OrderID");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendGroupingRequest: function (columnName) {
            if (!this.model.allowGrouping || $.inArray(columnName, this._disabledGroupableColumns) != -1)
                return;
            if (ej.isNullOrUndefined(this.getColumnByField(columnName)) || $.inArray(columnName, this.model.groupSettings.groupedColumns) != -1)
                return;
            this.model.groupSettings.groupedColumns.push(columnName);
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == columnName) {
                    break;
                }
            }
            this.model.sortSettings.sortedColumns.length == i && this.model.sortSettings.sortedColumns.push({ field: columnName, direction: ej.sortOrder.Ascending });
            var args = {};
            args.columnName = columnName;
            args.requestType = ej.Grid.actions.Grouping;
            var returnValue = this._processBindings(args);
            if (returnValue)
                this.model.groupSettings.groupedColumns.pop();
            this._primaryKeyValues = [];
        },
        /**
			* It is used to un-group a column from grouped columns collection in grid control.
			* @return jQuery
			* @param {string} columnName Pass the field Name of the column to be ungrouped from grouped column collection
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendUngroupingRequest("OrderID"); // Sends an ungroup column request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends an ungroup column request to the grid
			* $("#Grid").ejGrid("sendUngroupingRequest", "OrderID");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendUngroupingRequest: function (columnName) {
            if (!this.model.allowGrouping)
                return;
            if ($.inArray(columnName, this.model.groupSettings.groupedColumns) != -1)
                this.model.groupSettings.groupedColumns.splice($.inArray(columnName, this.model.groupSettings.groupedColumns), 1);
            else
                return null;
            var args = {};
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == columnName) {
                    this.model.sortSettings.sortedColumns.splice(i, 1);
                    break;
                }
            }
            args.columnName = columnName;
            args.requestType = ej.Grid.actions.Ungrouping;
            var returnValue = this._processBindings(args);
            if (returnValue)
                this.model.groupSettings.groupedColumns.push(columnName);
            this._primaryKeyValues = [];
        },
        /**
			* It is used to send a cancel request in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendCancelRequest(); // Sends a cancel request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends a cancel request to the grid
			* $("#Grid").ejGrid("sendCancelRequest");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendCancelRequest: function () {
            var args = {};
            $("#" + this._id + "EditForm").find(".e-datepicker").ejDatePicker("destroy");
            args.requestType = ej.Grid.actions.Cancel;
            this._processBindings(args);
            this._primaryKeyValues = [];
            this._currentData = null;
        },
        /**
			* It is used to send a search request to grid with specified string passed in it.
			* @return jQuery
			* @param {string} searchString Pass the string to search in Grid records
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendSearchRequest("France"); // Sends a search request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends a search request to the grid
			* $("#Grid").ejGrid("sendSearchRequest", "France");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendSearchRequest: function (searchString) {
            var args = {};
            args.requestType = ej.Grid.actions.Search;
            args.keyValue = searchString;
            this._searchString = searchString;
            this._processBindings(args);
            this._primaryKeyValues = [];
        },
        /**
			* It is used to remove a column or collection of columns from a sorted column collections in grid control.
			* @return jQuery
			* @param {array/string} fieldName Pass array of field names of the columns to remove a collection of sorted columns or pass a string of field name to remove a column from sorted column    collections
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.removeSortedColumns("OrderID"); // Removes a column from sorted column collections
			* gridObj.removeSortedColumns(["CustomerID”, “ShipCity"]); // Removes specified collection of columns from sorted column collections
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Removes a column from sorted column collections
			* $("#Grid").ejGrid("removeSortedColumns", "OrderID");        
			* // Removes specified collection of columns from sorted column collections
			* $("#Grid").ejGrid("removeSortedColumns", ["CustomerID", "ShipCity"]);        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        removeSortedColumns: function (fieldName) {
            if ($.isArray(fieldName)) {
                for (var i = 0; i < fieldName.length; i++) {
                    this._removeSortedColumnFromCollection(fieldName[i]);
                }
            }
            else
                this._removeSortedColumnFromCollection(fieldName);
            this._multiSortRequest = true;
            this.sendSortingRequest(null, null);
        },
        _removeSortedColumnFromCollection: function (fieldName) {
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == fieldName) {
                    this.model.sortSettings.sortedColumns.splice(i, 1);
                    break;
                }
            }
        },
        /**
			* It is used to send a sorting request in grid control.
			* @return jQuery
			* @param {string} columnName Pass the field name of the column as columnName for which sorting have to be performed
			* @param {string} columnSortDirection Pass the sort direction ascending/descending by which the column have to be sort
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendSortingRequest("OrderID", "ascending"); // Sends a sorting request to the grid with specified columnName and sortDirection
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends a sorting request to the grid with specified columnName and sortDirection
			* $("#Grid").ejGrid("sendSortingRequest", "OrderID", "ascending");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendSortingRequest: function (columnName, columnSortDirection) {
            if (!this.model.allowSorting || $.inArray(columnName, this._disabledSortableColumns) != -1 || (columnName != null && columnName.length == 0))
                return;
            var args = {};
            if (!this._multiSortRequest) {
                var proxy = this;
                this.model.sortSettings.sortedColumns = $.grep(this.model.sortSettings.sortedColumns, function (value, index) {
                    if ($.inArray(value.field, proxy.model.groupSettings.groupedColumns) != -1)
                        return true;
                    return false;
                });
            }
            args.requestType = ej.Grid.actions.Sorting;
            this._cSortedColumn = args.columnName = columnName;
            this._cSortedDirection = args.columnSortDirection = columnSortDirection === undefined ? ej.sortOrder.Ascending : columnSortDirection;
            if (this._cSortedColumn !== null) {
                this._removeSortedColumnFromCollection(columnName);
                this.model.sortSettings.sortedColumns.push({ field: this._cSortedColumn, direction: this._cSortedDirection });
            }
            var returnValue = this._processBindings(args);
            if (returnValue)
                this._cSortedDirection = this._cSortedColumn = null;
            this._primaryKeyValues = [];
        },
        /**
			* It is used to send a paging request at specified page in grid control.
			* @return jQuery
			* @param {number} pageIndex Pass the page index to perform paging at specified page index
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendPagingRequest(3); // Sends a paging request to the grid with specified page index
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends a paging request to the grid with specified page index
			* $("#Grid").ejGrid("sendPagingRequest", 3);        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendPagingRequest: function (pageIndex) {
            if (!this.model.allowPaging && (!this.model.allowScrolling && !this.model.scrolling.allowVirtualScrolling))
                return;
            var args = {}, returnValue;
            args.previousPage = this._currentPage();
            this._currentPage(pageIndex);
            args.endIndex = ((this._currentPage() * this.model.pageSettings.pageSize) > this._gridRecordsCount) ? (this._gridRecordsCount) : (this._currentPage() * this.model.pageSettings.pageSize);
            args.startIndex = (this._currentPage() * this.model.pageSettings.pageSize) - this.model.pageSettings.pageSize;
            args.currentPage = pageIndex;
            if (this.model.allowPaging) {
                //this.model.pageSettings.currentPage = pageIndex;
                //this.getGridPager().ejPager("refreshPager");
                args.requestType = ej.Grid.actions.Paging;
            }
            if (this.model.scrolling.allowVirtualScrolling && this.model.allowScrolling) {
                args.requestType = ej.Grid.actions.VirtualScroll;
            }
            returnValue = this._processBindings(args);
            if (returnValue)
                this._currentPage(args.previousPage);
            this._primaryKeyValues = [];
        },
        _setDateFilters: function (filterObject) {
            var $prevDate = new Date(filterObject.value.setDate(filterObject.value.getDate() - 1));
            var $nextDate = new Date(filterObject.value.setDate(filterObject.value.getDate() + 2));
            var $prevObj = $.extend({}, filterObject);
            var $nextObj = $.extend({}, filterObject);
            $prevObj.value = $prevDate;
            $nextObj.value = $nextDate;
            if (filterObject.operator == "equal") {
                $prevObj.operator = ">";
                $prevObj.predicate = "and";
                $nextObj.operator = "<";
                $nextObj.predicate = "and";
            } else {
                $prevObj.operator = "<=";
                $prevObj.predicate = "or";
                $nextObj.operator = ">=";
                $nextObj.predicate = "or";
            }
            this.model.filterSettings.filteredColumns.push($prevObj);
            this.model.filterSettings.filteredColumns.push($nextObj);
        },
        /**
			* It is used to send a filtering request to grid control.
			* @return jQuery
			* @param {string} fieldName Pass the field name of the column 
			* @param {string} filterOperator string/integer/dateTime operator
			* @param {string/number} filterValue Pass the value  to be filtered in a column
			* @param {string} predicate Pass the predicate as and/or
			* @param {boolean} matchcase Pass the match case valueas true/false
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendFilteringRequest("OrderID","equal","10248","and", true); // Sends a filtering request to the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Sends a filtering request to the grid
			* $("#Grid").ejGrid("sendFilteringRequest","OrderID","equal","10248","and", true);
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendFilteringRequest: function (fieldName, filterOperator, filterValue, predicate, matchcase, actualFilterValue) {
            if (!this.model.allowFiltering)
                return;
            var args = {};
            args.requestType = ej.Grid.actions.Filtering;
            args.currentFilterObject = [];
            if (!$.isArray(filterOperator))
                filterOperator = $.makeArray(filterOperator);
            if (!$.isArray(filterValue))
                filterValue = $.makeArray(filterValue);
            var firstLoop = false;
            var filterCol = this._filterCollection;
            if (ej.util.isNullOrUndefined(this._currentFilterColumn))
                this._currentFilterColumn = this.getColumnByField(fieldName);
            for (var index = 0; index < filterOperator.length; index++) {
                var filterObject = {
                    field: fieldName,
                    operator: filterOperator[index],
                    value: filterValue[index],
                    matchcase: matchcase,
                    predicate: predicate,
                    actualFilterValue: actualFilterValue
                };
                if (this.model.filterSettings.filterType == "filterbar")
                    this._$colType = this._currentFilterColumn.type;
                if (this.model.filterSettings.filteredColumns.length == 0 && filterObject.value !== "") {
                    if (this._$colType == "date" && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                        this._setDateFilters(filterObject);
                    else
                        this.model.filterSettings.filteredColumns.push(filterObject);
                } else {
                    var proxy = this;
                    if (!firstLoop) {
                        var dataManger = ej.DataManager(this.model.filterSettings.filteredColumns);
                        var query = new ej.Query().where("field", ej.FilterOperators.equal, filterObject.field);
                        var object = dataManger.executeLocal(query);
                        for (var i = 0; i < object.length; i++) {
                            var objectIndex = $.inArray(object[i], this.model.filterSettings.filteredColumns)
                            if (objectIndex != -1)
                                this.model.filterSettings.filteredColumns.splice(objectIndex, 1);
                        }
                    }
                    if (filterObject.value !== "") {
                        if (this._$colType == "date" && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                            this._setDateFilters(filterObject);
                        else
                            this.model.filterSettings.filteredColumns.push(filterObject);
                    }
                }
                firstLoop = true;
                args.currentFilterObject.push(filterObject);
            }
            args.filterCollection = this.model.filterSettings.filteredColumns;
            args.currentFilteringColumn = fieldName;
            var returnValue = this._processBindings(args);
            if (returnValue) {
                this.model.filterSettings.filteredColumns.reverse().splice(0, filterOperator.length);
                this.model.filterSettings.filteredColumns.reverse();
            }

        },
        /**
			* It is used to re-order the column in grid control.
			* @return jQuery
			* @param {string} fromFieldName Pass the from field name of the column needs to be changed
			* @param {string} toFieldName Pass the to field name of the column needs to be changed
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.sendReorderingRequest("OrderID", "CustomerID"); // Reorders the column based on the given index
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Reorders the column based on the given index
			* $("#Grid").ejGrid("sendReorderingRequest", "OrderID", "CustomerID");
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        sendReorderingRequest: function (fromfname, tofname) {
            var fromindex = this.model.columns.indexOf(this.getColumnByField(fromfname));
            var toindex = this.model.columns.indexOf(this.getColumnByField(tofname));
            this.set_dropColumn(fromindex, toindex);
        },
        /**
			* It is used to collapse the group drop area in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.collapseGroupDropArea(); // Collapse the group drop area of the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Collapse the group drop area of the grid
			* $("#Grid").ejGrid("collapseGroupDropArea");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        collapseGroupDropArea: function () {
            $groupDropArea = this.element.find(".e-groupdroparea").first(), proxy = this;
            this.model.groupSettings.groupedColumns.length == 0 && this.model.enableEffects && $groupDropArea.animate({ height: "10px" }, 200, function () {
                if (proxy.model.groupSettings.groupedColumns.length == 0) {
                    $(this).html("").append(ej.buildTag("div.e-animatebutton e-animatebuttondown e-icon e-gdownarrow"));
                    $(this).dequeue().css("height", "auto");
                }
            });
        },
        /**
			* It is used to expand the group drop area in grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.expandGroupDropArea(); // Expands the group drop area of the grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Expands the group drop area of the grid
			* $("#Grid").ejGrid("expandGroupDropArea");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        expandGroupDropArea: function () {
            $groupDropArea = this.element.find(".e-groupdroparea").first(), proxy = this;
            this.model.groupSettings.groupedColumns.length == 0 && proxy.model.enableEffects && $groupDropArea.animate({ height: "30px" }, 150, function () {
                proxy.model.groupSettings.groupedColumns.length == 0 && $groupDropArea.html(proxy._getLocalizedLabels("GroupDropArea")).append(ej.buildTag("div.e-animatebutton e-animatebuttonup e-icon e-guparrow"));
                $groupDropArea.dequeue().css("height", "30px");
            });
        },
        /**
			* It is used to add or remove columns in grid column collections
			* @return jQuery
			* @param {array/string} columnDetails Pass array of field names or string of field name to add/remove the column in grid 
			* @param {string} action Pass add/remove action to be performed
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.columns("OrderID", "remove"); // remove grid columns
			* gridObj.columns("CustomerID", "add"); // Add grid columns
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // remove grid columns
			* $("#Grid").ejGrid("columns","OrderID", "remove");   
			* // add grid columns			
			* $("#Grid").ejGrid("columns","CustomerID", "add");    			
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        columns: function (details, action) {
            if (ej.isNullOrUndefined(details)) return;
            var isString = false;
            if (typeof details === "string") {
                details = [details];
                isString = true;
            }
            else if (details instanceof Array && details.length && typeof details[0] === "string")
                isString = true;
            for (i = 0; i < details.length; i++) {
                var index = $.inArray(this.getColumnByField(isString ? details[i] : details[i]["field"]), this.model.columns);
                if (action == "add" || ej.isNullOrUndefined(action)) {
                    if (index == -1)
                        this.model.columns.push(isString ? { field: details[i] } : details[i]);
                    else
                        this.model.columns[index] = isString ? { field: details[i] } : details[i];
                }
                else {
                    if (index != -1)
                        this.model.columns.splice(index, 1);
                }
            }
            var $header = this.element.find(".e-gridheader");
            $header.find("div").first().empty().append(this._renderGridHeader().find("table"));
            this._headerCellgDragDrop();
            this.refreshGridContent(true);
        },
        /**
			* It is used to get the data of currently edited cell value in "batch" edit mode
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getCurrentEditCellData(); // Get data of currently edited cell value
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Get data of currently edited cell value
			* $("#Grid").ejGrid("getCurrentEditCellData");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getCurrentEditCellData: function () {
            if (this.model.isEdit && $("#" + this._id + "EditForm").length) {
                var $element = $("#" + this._id + this._bulkEditCellDetails.fieldName), cellValue;
                switch (this._bulkEditCellDetails.cellEditType) {
                    case ej.Grid.editingType.String:
                        cellValue = $element.val();
                        break;
                    case ej.Grid.editingType.Numeric:
                        cellValue = $element.ejNumericTextbox("getValue");
                        $element.ejNumericTextbox("getValue");
                        break;
                    case ej.Grid.editingType.Dropdown:
                        cellValue = this._bulkEditCellDetails.isForeignKey ? { "value": $element.ejDropDownList("getSelectedValue"), "text": $element.ejDropDownList("getSelectedItem") } : $element.ejDropDownList("getSelectedValue");
                        break;
                    case ej.Grid.editingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;
                    case ej.Grid.editingType.DatePicker:
                        cellValue = $element.ejDatePicker("model.value");
                        break;
                    case ej.Grid.editingType.DateTimePicker:
                        cellValue = $element.ejDateTimePicker("model.value");
                        break;
                }
                if (typeof cellValue == "string" && cellValue.length && this.model.columns[this._bulkEditCellDetails.columnIndex].type == "number")
                    cellValue = parseFloat(cellValue);
                return cellValue;
            }
            return null;
        },
        cancelEditCell: function () {
            if (this.model.isEdit) {
                var tr = this.getGridRows()[this._bulkEditCellDetails.rowIndex], cellData = {}, cell;
                cellData[this._bulkEditCellDetails.fieldName] = this._bulkEditCellDetails.cellValue;
                if ($(tr).hasClass("e-insertedrow"))
                    cell = tr.cells[this._bulkEditCellDetails.columnIndex + this.model.groupSettings.groupedColumns.length];
                else
                    cell = tr.cells[this._bulkEditCellDetails.columnIndex];
                $(cell).removeClass("e-editedbatchcell").empty().html($($.templates.Grid_JSONTemplate.render(cellData))[0].cells[this._bulkEditCellDetails.columnIndex].innerHTML);
                this.model.isEdit = false;
                this.element.focus();
            }
        },
        saveCell: function (preventSaveEvent) {
            if (this.model.isEdit) {
                if (!this.editFormValidate())
                    return true;
                var $form = $("#" + this._id + "EditForm"), $targetTR = $form.closest("tr"), $targetTD = $form.closest("td"), $toolBar, formattedValue
                , args = {}, column = this.model.columns[this._bulkEditCellDetails.columnIndex], $element = $("#" + this._id + this._bulkEditCellDetails.fieldName);
                args = {
                    columnName: column.field,
                    value: this.getCurrentEditCellData(),
                    rowData: this._bulkEditCellDetails.rowData,
                    previousValue: this._bulkEditCellDetails.cellValue,
                    columnObject: column,
                    cell: $targetTD,
                    isForeignKey: this._bulkEditCellDetails.isForeignKey
                };
                if (!preventSaveEvent && this._trigger("cellSave", args)) {
                    this._focusElements(args.cell);
                    this._bulkEditCellDetails.cancelSave = true;
                    return;
                }
                if (this._bulkEditCellDetails.cellEditType == "datetimepicker" || this._bulkEditCellDetails.cellEditType == "dropdownedit" || this._bulkEditCellDetails.cellEditType == "datepicker")
                    $element[$element.data("ejWidgets")[0]]("destroy");
                if (!ej.isNullOrUndefined(column.format)) {
                    formattedValue = this.formatting(column.format, args.isForeignKey ?
                                              (!isNaN(parseFloat(args.value.text)) && isFinite(args.value.text)
                                              ? parseFloat(args.value.text)
                                              : args.value.text) : args.value);
                    args.cell.empty().html(formattedValue);
                }
                else {
                    if (args.columnObject.type == "boolean") {
                        var cellData = {};
                        cellData[args.columnObject.field] = args.value;
                        args.cell.empty().html($($.templates.Grid_JSONTemplate.render(cellData))[0].cells[this._bulkEditCellDetails.columnIndex].innerHTML);
                    }
                    else
                        args.cell.empty().html(args.isForeignKey ? args.value.text : args.value);
                }
                var isValueModified = ((this._bulkEditCellDetails.cellEditType == "datepicker" || this._bulkEditCellDetails.cellEditType == "datetimepicker") && args.value instanceof Date && args.previousValue instanceof Date) ? (args.value.getTime() !== args.previousValue.getTime()) : (args.value !== args.previousValue);
                this.model.isEdit = false;
                if (isValueModified) {
                    this._enableSaveCancel();
                    args.cell.addClass("e-updatedtd e-icon e-gupdatenotify");
                    if (typeof args.rowData[this._bulkEditCellDetails["fieldName"]] == "string" && args.rowData[this._bulkEditCellDetails["fieldName"]].length)
                        args.rowData[this._bulkEditCellDetails["fieldName"]] = args.isForeignKey ? args.value.value.toString() : args.value.toString();
                    else
                        args.rowData[this._bulkEditCellDetails["fieldName"]] = args.isForeignKey ? (!isNaN(parseInt(args.value.value)) ? parseInt(args.value.value) : args.value.value) : args.value;
                    if ($.inArray(args.rowData, this._batchChanges.changed) == -1 && $.inArray(args.rowData, this._batchChanges.added) == -1)
                        this._batchChanges.changed.push(args.rowData);
                }
                $targetTR.removeClass("e-editedrow");
                args.cell.removeClass("e-editedbatchcell");
            }
        },
        _enableSaveCancel: function () {
            if (this.model.toolBar.allowToolBar) {
                $toolBar = this.element.find("#" + this._id + "_toolbarItems");
                $toolBar.ejToolbar("enableItembyID", this._id + "_update");
                $toolBar.ejToolbar("enableItembyID", this._id + "_cancel");
            }
        },

        setCellValue: function (index, fieldName, cellValue) {
            var data = this.getDataByIndex(index), tr = this.getGridRows()[index], dataIndex
                , columnIndex = this.getColumnIndexByField(fieldName);
            data[fieldName] = cellValue;
            if ($(tr).hasClass("e-insertedrow"))
                tr.cells[columnIndex + this.model.groupSettings.groupedColumns.length].innerHTML = cellValue;
            else {
                $.inArray(data, this._batchChanges.changed) == -1 && this._batchChanges.changed.push(data);
                tr.cells[columnIndex].innerHTML = cellValue;
            }
            this._enableSaveCancel();
        },
        setDefaultData: function (defaultData) {
            if (ej.isNullOrUndefined(defaultData)) {
                var defaultData = $.extend(true, {}, this._bulkEditCellDetails._data[0]), fieldNames = this.getColumnFieldNames();
                for (var field in defaultData) {
                    var index = $.inArray(field, fieldNames);
                    index == -1 && delete defaultData[field];
                    if (defaultData.hasOwnProperty(field)) {
                        if (!ej.isNullOrUndefined(this.model.columns[index]["defaultValue"])) {
                            defaultData[field] = this.model.columns[index]["defaultValue"];
                        }
                        else {
                            switch (typeof defaultData[field]) {
                                case "number":
                                    defaultData[field] = 0;
                                    break;
                                case "string":
                                    defaultData[field] = "";
                                    break;
                                case "boolean":
                                    defaultData[field] = false;
                                    break;
                                case "object":
                                    if ($.isArray(defaultData[field]))
                                        defaultData[field] = new Array();
                                    else
                                        defaultData[field] = "";
                            }
                        }
                    }
                }
            }
            this._bulkEditCellDetails.defaultData = defaultData;
        },
        /**
			* It is used to delete row in grid based on row index in "Batch" edit mode.
			* @return jQuery
			* @param { number } rowIndex Pass row index to be deleted in grid.
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.bulkDelete(2); // Delete particular row in grid based on row index
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Delete particular row in grid based on row index
			* $("#Grid").ejGrid("bulkDelete", 2);
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        bulkDelete: function (index) {
            if (ej.isNullOrUndefined(index))
                index = this._selectedRow();
            var tr = this.getGridRows()[index], $tr = $(tr), index, data = this.getDataByIndex(index), args = {};
            args = {
                primaryKey: this._primaryKeys,
                rowIndex: index,
                rowData: data,
                row: $tr
            };
            if (this._trigger("beforeBulkDelete", args))
                return;
            if ($tr.hasClass("e-insertedrow")) {
                $tr.remove();
                index = $.inArray(tr, this._bulkEditCellDetails.insertedTrCollection);
                if (index != -1) {
                    this._bulkEditCellDetails.insertedTrCollection.splice(index, 1);
                    this._batchChanges.added.splice(index, 1);
                }
            }
            else {
                $tr.hide();
                this._batchChanges.deleted.push(data);
            }
            this._enableSaveCancel();
            this._selectedRow(-1);
            args = {};
            this._trigger("bulkDelete", args);
        },
        /**
			* It is used to add a  row in grid when edit mode is "Batch".
			* @return jQuery
			* @param { object } defaultData defaultData is an optional parameter. If Defaults did not assign then it will take values for add row based on type of columns. Defaults parameter value type is an object with field name as properties.
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.bulkAddRow(); // To add a row to a grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // To add a row to a grid
			* $("#Grid").ejGrid("bulkAddRow");
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        bulkAddRow: function (defaultData) {
            var args = {}, $tr, editCellIndex, rows = this.getGridRows();
            if (!ej.isNullOrUndefined(defaultData))
                this._bulkEditCellDetails.defaultData = defaultData;
            ej.isNullOrUndefined(this._bulkEditCellDetails.defaultData) && this.setDefaultData();
            args = {
                defaultData: $.extend(true, {}, this._bulkEditCellDetails.defaultData),
                primaryKey: this._primaryKeys,
            };
            if (this._trigger("beforeBulkAdd", args))
                return;
            $tr = $($.render[this._id + "_JSONTemplate"](args.defaultData)).addClass("e-insertedrow");
            rows != null && $(rows[0]).hasClass("e-alt_row") && $tr.removeClass("e-alt_row");
            this.getGridContentTable().find("tbody").first().prepend($tr);
            this._gridRows = this.getGridContentTable().find("td.e-rowcell").closest("tr").toArray();
            for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                $tr.prepend(ej.buildTag("td.e-indentcell"));
            this._bulkEditCellDetails.insertedTrCollection.push($tr.get(0));
            this._batchChanges.added.push(args.defaultData);
            this._enableSaveCancel();
            editCellIndex = this._findNextEditableCell(0);
            this.selectRows(0);
            this.bulkEdit(0, this.model.columns[editCellIndex].field);
            $tr.find(".e-rowcell").addClass("e-updatedtd e-icon e-gupdatenotify");
            args = { defaultData: args.defaultData };
            $.extend(args, {
                columnObject: this.model.columns[editCellIndex],
                columnIndex: editCellIndex,
                row: $tr,
                primaryKey: this._primaryKeys,
                cell: $($tr[0].cells[editCellIndex])
            });
            this._trigger("bulkAdd", args);
        },
        getDataByIndex: function (rowIndex) {
            var $tr = $(this.getGridRows()[rowIndex]), insertedRowIndex, currentRowData, index;
            if ($tr.hasClass("e-insertedrow")) {
                insertedRowIndex = $.inArray($tr[0], this._bulkEditCellDetails.insertedTrCollection);
                return this._batchChanges.added[insertedRowIndex];
            }
            else
                return this._bulkEditCellDetails._data[rowIndex - this._bulkEditCellDetails.insertedTrCollection.length];

        },
        /**
			* It is used to refresh and reset the changes made in "batch" edit mode
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.refreshBatchEditChanges(); // It is used to refresh and reset the changes made in batch edit mode
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // It is used to refresh and reset the changes made in batch edit mode
			* $("#Grid").ejGrid("refreshBatchEditChanges");
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        refreshBatchEditChanges: function () {
            this._bulkEditCellDetails = {
                cellValue: null,
                rowIndex: -1,
                _data: null,
                columnIndex: -1,
                fieldName: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                rowData: null,
                isForeignKey: false
            };
            this._batchChanges = {
                added: [],
                deleted: [],
                changed: []
            };
        },
        /**
			* It is used to cancel the modified changes in grid control when edit mode is "Batch".
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.bulkCancel(); // Cancel added, edited, and deleted changes made in grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Cancel added, edited, and deleted changes made in grid
			* $("#Grid").ejGrid("bulkCancel");
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        bulkCancel: function () {
            this.sendCancelRequest();
        },
        /**
			* It is used to save the modified changes to data source in grid control when edit mode is "Batch".
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.bulkSave(); // Save added, edited, and deleted changes to source of data
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Save added, edited, and deleted changes to source of data
			* $("#Grid").ejGrid("bulkSave");
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        bulkSave: function () {
            var args = {}, deferedObject, gridObject = this;
            this.saveCell();
            args["batchChanges"] = this.getBatchChanges();
            if (this._trigger("beforeBulkSave", args))
                return;
            args = {};
            args.requestType = "bulksave";
            var deferedObject = this._dataManager.saveChanges(this.getBatchChanges(), this._primaryKeys[0], this.model.query._fromTable);
            if (this._dataManager instanceof ej.DataManager && !this._dataManager.dataSource.offline) {
                deferedObject.done(function (e) {
                    gridObject._processBindings(args);
                });
            }
            else
                this._processBindings(args);

        },
        /**
			* It is used to get the batch changes of edit, delete and add operations of grid control.
			* @return jQuery
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.getBatchChanges(); // Gets the edit, delete, and add changes of a grid
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Gets the edit, delete, and add changes of a grid
			* $("#Grid").ejGrid("getBatchChanges");        
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        getBatchChanges: function () {
            return this._batchChanges;
        },
        /**
			* It is used to edit a particular cell based on the row index and field name provided in "batch" edit mode.
			* @return jQuery
			* @param { number } Index Pass row index to edit particular cell
			* @param { string } fieldName Pass the field name of the column to perform bulk edit 
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Create grid object.
			* var gridObj = $("#Grid").data("ejGrid");
			* gridObj.bulkEdit(2, "OrderID"); // Edit particular cell based on row index and column field name
			* &lt;/script&gt;
			* @example 
			* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* // Edit particular cell based on row index and column field name
			* $("#Grid").ejGrid("bulkEdit", 2, "OrderID");
			* &lt;/script&gt;
			*@memberof ejGrid
			* @instance
        */
        bulkEdit: function (index, fieldName) {
            var $form = $("#" + this._id + "EditForm");
            this.model.isEdit && this.saveCell();
            if ($.isFunction($.validator) && $form.length && $form.validate().errorList.length)
                return;
            var $targetTR = $(this.getGridRows()[index]), columnIndex = this.getColumnIndexByField(fieldName), $targetTd = $targetTR.find(".e-rowcell").eq(columnIndex), column = this.model.columns[columnIndex], rowData = this.getDataByIndex(index);
            var args = {
                validationRules: ej.isNullOrUndefined(column.validationRules) ? {} : $.extend(true, {}, column.validationRules),
                columnName: column.field,
                value: rowData[fieldName],
                rowData: rowData,
                primaryKey: this._primaryKeys,
                columnObject: column,
                cell: $targetTd,
                isForeignKey: !ej.isNullOrUndefined(column.foreignKeyValue) && this.model.edit.editMode == "batch" ? true : false,
            }, isEditable = true;
            if (this._trigger("cellEdit", args))
                return;
            if ($.inArray(fieldName, this._primaryKeys) != -1 || args.columnObject.allowEditing === false || args.columnObject.columnTemplate || args.columnObject.isUnBound) {
                $.extend(this._bulkEditCellDetails, {
                    cellValue: args.value,
                    rowIndex: index,
                    fieldName: fieldName,
                    rowData: args.rowData,
                    columnIndex: columnIndex,
                    isForeignKey: ej.isNullOrUndefined(args.columnObject.foreignKeyValue) ? false : true
                });
                isEditable = false;
            }
            if ($targetTR.hasClass("e-insertedrow") && (!args.columnObject.columnTemplate && !args.columnObject.isUnBound))
                isEditable = true;
            if (isEditable) {
                $.extend(this._bulkEditCellDetails, {
                    rowIndex: index,
                    cellValue: args.value,
                    columnIndex: columnIndex,
                    fieldName: fieldName,
                    cellEditType: args.columnObject.columnEditType,
                    rowData: rowData,
                    isForeignKey: ej.isNullOrUndefined(args.columnObject.foreignKeyValue) ? false : true
                });
                this._renderBulkEditObject(args, $targetTd);
                $targetTR.addClass("e-editedrow");
                args.cell.addClass("e-editedbatchcell");
            }
        },
        _findNextEditableCell: function (columnIndex) {
            var endIndex = this.model.columns.length;
            for (var i = columnIndex; i < endIndex; i++) {
                if (!this.model.columns[i].columnTemplate && !this.model.columns[i].isUnBound)
                    return i;
            }
            return -1;
        },
        _findNextCell: function (columnIndex, direction) {
            var splittedColumn, visibleColumns = [], predicate, rows = this.getGridRows();
            predicate = ej.Predicate("visible", "equal", true).and("columnTemplate", "notequal", true);
            if (!$(rows[this._bulkEditCellDetails.rowIndex]).hasClass("e-insertedrow"))
                predicate = predicate.and("allowEditing", "notequal", false);
            splittedColumn = direction == "right" ? this.model.columns.slice(columnIndex) : this.model.columns.slice(0, columnIndex + 1).reverse();
            visibleColumns = ej.DataManager(splittedColumn).executeLocal(ej.Query().where(predicate));
            if (visibleColumns.length == 0 && (!(direction == "left" && this._bulkEditCellDetails.rowIndex == 0) && !(direction == "right" && this._bulkEditCellDetails.rowIndex + 1 == this.getGridRows().length))) {
                splittedColumn = direction == "right" ? this.model.columns.slice(0, columnIndex) : this.model.columns.slice(columnIndex).reverse();
                visibleColumns = ej.DataManager(splittedColumn).executeLocal(ej.Query().where(predicate));
                this._bulkEditCellDetails.rowIndex = visibleColumns.length && direction == "right" ? this._bulkEditCellDetails.rowIndex + 1 : this._bulkEditCellDetails.rowIndex - 1;
            }
            return visibleColumns.length ? $.inArray(visibleColumns[0], this.model.columns) : -1;
        },
        _moveCurrentCell: function (direction) {
            var editCellIndex, rowIndex = this._bulkEditCellDetails.rowIndex, currentRow, $form = $("#" + this._id + "EditForm");
            if (this._bulkEditCellDetails.rowIndex == -1 && this._bulkEditCellDetails.columnIndex == -1)
                return true;
            switch (direction) {
                case "right":
                    if ((this._bulkEditCellDetails.rowIndex == this.getGridRows().length - 1 && this._bulkEditCellDetails.columnIndex == this.model.columns.length - 1) || (!this.element.is(document.activeElement) && $form.length == 0))
                        return true;
                    if (this._bulkEditCellDetails.columnIndex == this.model.columns.length - 1) {
                        editCellIndex = 0;
                        this._bulkEditCellDetails.rowIndex = this._bulkEditCellDetails.rowIndex + 1;
                    }
                    else
                        editCellIndex = this._bulkEditCellDetails.columnIndex + 1;
                    if (this.model.columns[editCellIndex].columnTemplate === true || this.model.columns[editCellIndex].visible === false || this.model.columns[editCellIndex].allowEditing === false)
                        editCellIndex = this._findNextCell(editCellIndex, direction);
                    this._bulkEditCellDetails.rowIndex != rowIndex && this.selectRows(this._bulkEditCellDetails.rowIndex);
                    editCellIndex != -1 && this.bulkEdit(this._bulkEditCellDetails.rowIndex, this.model.columns[editCellIndex].field);
                    break;
                case "left":
                    if ((this._bulkEditCellDetails.rowIndex == 0 && this._bulkEditCellDetails.columnIndex == 0) || (!this.element.is(document.activeElement) && $form.length == 0))
                        return true;
                    if (this._bulkEditCellDetails.columnIndex == 0) {
                        editCellIndex = this.model.columns.length - 1;
                        this._bulkEditCellDetails.rowIndex = this._bulkEditCellDetails.rowIndex - 1;
                        this.selectRows(this._bulkEditCellDetails.rowIndex);
                    }
                    else
                        editCellIndex = this._bulkEditCellDetails.columnIndex - 1;
                    if (this.model.columns[editCellIndex].columnTemplate === true || this.model.columns[editCellIndex].visible === false || this.model.columns[editCellIndex].allowEditing === false)
                        editCellIndex = this._findNextCell(editCellIndex, direction);
                    this._bulkEditCellDetails.rowIndex != rowIndex && this.selectRows(this._bulkEditCellDetails.rowIndex);
                    editCellIndex != -1 && this.bulkEdit(this._bulkEditCellDetails.rowIndex, this.model.columns[editCellIndex].field);
                    break;
                case "up":
                    if (this._bulkEditCellDetails.rowIndex == 0)
                        return;
                    editCellIndex = this._bulkEditCellDetails.columnIndex;
                    this.selectRows(this._bulkEditCellDetails.rowIndex - 1);
                    this.bulkEdit(this._bulkEditCellDetails.rowIndex - 1, this.model.columns[this._bulkEditCellDetails.columnIndex].field);
                    break;
                case "down":
                    if (this._bulkEditCellDetails.rowIndex == this.getGridRows().length - 1)
                        return;
                    editCellIndex = this._bulkEditCellDetails.columnIndex;
                    this.selectRows(this._bulkEditCellDetails.rowIndex + 1);
                    this.bulkEdit(this._bulkEditCellDetails.rowIndex + 1, this.model.columns[this._bulkEditCellDetails.columnIndex].field);
                    break;

            }
            addedRow = !$(this.getGridRows()[this._bulkEditCellDetails.rowIndex]).hasClass("e-insertedrow");
            if (editCellIndex != -1 && (this.model.columns[editCellIndex].isUnbound || (this.model.columns[editCellIndex].key && addedRow) || this.model.columns[editCellIndex].columnTemplate))
                this.element.focus();
            return false;
        },
        _renderBulkEditObject: function (cellEditArgs, $td) {
            var $form = ej.buildTag("form", "", {}, { id: this._id + "EditForm" }), $bulkEditTemplate = this._bulkEditTemplate, mappingName = this._id + cellEditArgs.columnObject.field, $element, htmlString, cellData = {};
            cellData[cellEditArgs.columnObject.field] = cellEditArgs.value;
            $td.empty();
            htmlString = $bulkEditTemplate.find("#" + cellEditArgs.columnObject.field + "_BulkEdit").html();
            $element = $($.templates(htmlString).render(cellData));
            if ($element.get(0).tagName == "SELECT") {
                $element.val(cellData[cellEditArgs.columnObject.field]);
                $element.val() == null && $element.val($element.find("option").first().val());
            }
            $form.append($element);
            $td.append($form);
            this._setoffsetWidth();
            this._refreshEditForm();
            if ($.isFunction($.validator) && !$.isEmptyObject(cellEditArgs.validationRules)) {
                this._initValidator();
                this.setValidationToField(cellEditArgs.columnObject.field, cellEditArgs.validationRules);
            }
            this.model.isEdit = true;
        },

        // events

        _wireEvents: function () {
            var $menuFilter = this.model.allowFiltering && this.model.filterSettings.filterType == "menu";
            this._on(this.element, $.isFunction($.fn.tap) ? "tap" : "click", this._clickHandler);
            //if (this.model.allowSorting || $menuFilter)
            this._on(this.element, $.isFunction($.fn.tap) ? "tap" : "click", ".e-gridheader", this._mouseClickHandler);
            this._on(this.element, "dblclick", ".e-gridcontent", this._recorddblClickHandler);
            this._on(this.element, "contextmenu", this._rightClickHandler);
            this._on(this.element, "click", ".e-gridcontent", this._recordClick);
            this._enableEditingEvents();
            this._enableRowHover();
            this._on(this.element, "click", ".e-gridcontent .e-unboundcelldiv", this._unboundClickHandler);
            this._enableGroupingEvents();
            this._enableFilterEvents();
            if (this.model.allowKeyboardNavigation) {
                this.element[0].tabIndex = 0;
                this.element[0].accessKey = "e";
                this._on(this.element, "keyup", this._keyDownHandler);
            }
            this._on(this.element, "mousedown", ".e-gridheader", this._headerMouseDown);
            this._on(this.element, "mouseenter mouseleave", ".e-groupdroparea,.e-groupheadercell", this._dropAreaHover);
            this._on(this.element, "mouseover mouseleave", ".e-gridheader", this._headerHover);
            this._on(this.element, ej.eventType.mouseMove, ".e-gridheader", this._headerHover);
            this.model.allowResizeToFit && this._on(this.element, "dblclick", ".e-gridheader", this._columnResizeToFit);
            if (this.model.allowResizing) {
                this._on(this.element, ej.eventType.mouseMove, this._mouseMove);
                this._on(this.element, "mouseup", this._mouseUp);
            }
            this._on(this.element, "swipeleft swiperight", ".e-gridcontent .e-table", $.proxy(this._touchGrid, this));
        },
        _touchGrid: function (e) {
            var curPage = this._currentPage();
            var lastPage = this._gridRecordsCount / this.model.pageSettings.pageSize;
            var totPage = lastPage == 0 ? lastPage : (parseInt(lastPage, 10) + 1);
            switch (e.type) {
                case "swipeleft":
                    if (this.model.allowPaging && curPage != totPage && !this.model.isEdit)
                        this.element.ejGrid("sendPagingRequest", curPage + 1);
                    break;
                case "swiperight":
                    if (this.model.allowPaging && curPage > 1 && !this.model.isEdit)
                        this.element.ejGrid("sendPagingRequest", curPage - 1);
                    break;
            }
        },
        _enableGroupingEvents: function () {
            if (this.model.allowGrouping) {
                this._on(this.element, "click", ".e-groupdroparea,.e-groupheadercell", this._groupHeaderCellClick);
                //to avoid ungrouping issue.
                this._on(this.element, "mousedown", ".e-groupheadercell", function (e) {
                    return false;
                });
            }

        },
        _enableEditingEvents: function () {
            if (this.model.edit.allowEditing || this.model.edit.allowAdding) {
                if (this.model.edit.allowEditing && this.model.edit.editMode != "batch" && (this.model.edit.editEvent == "doubleclick"))
                    this._on(this.element, "dblclick", ".e-gridcontent", this._editdblClickHandler);
                else
                    this._off(this.element, "dblclick", ".e-gridcontent");
                this._off($("#" + this._id + "_dialogEdit"), "click keypress", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel");
                if (this.model.edit.editMode == "dialog" || this.model.edit.editMode == "dialogtemplate") {
                    this._on($("#" + this._id + "_dialogEdit"), "click keypress", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel", this._buttonClick);
                }
                else if (this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate") {
                    this._on($("#" + this._id + "_externalEdit"), "click keypress", "#EditExternalForm_" + this._id + "_Save ,#EditExternalForm_" + this._id + "_Cancel", this._buttonClick);
                    $(this.element).on("click", ".e-form-toggle-icon", $.proxy(this._buttonClick, this));
                }
                else if (this.model.edit.editMode == "batch")
                    this._on($(document), "mousedown", this._saveCellHandler);

                else if (this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate")
                    $(this.element).on("click keypress", "#InlineEditForm_" + this._id + "_Save ,#InlineEditForm_" + this._id + "_Cancel", $.proxy(this._buttonClick, this));

            } else {
                this._off($("#" + this._id + "_dialogEdit"), "click", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel");
                $(this.element).off("click", ".e-icon");
                this._off($("#" + this._id + "_externalEdit"), "click", "#EditExternalForm_" + this._id + "_Save ,#EditExternalForm_" + this._id + "_Cancel");
                $(this.element).off("click", ".e-form-toggle-icon");
                $(this.element).off("click", "#InlineEditForm_" + this._id + "_Save ,#InlineEditForm_" + this._id + "_Cancel");
            }
        },
        _enableRowHover: function () {
            if (this.model.rowHover)
                this._on(this.element, "mouseenter mouseleave", ".e-gridcontent tr td", this._rowHover);
            else
                this._off(this.element, "mouseenter mouseleave", ".e-gridcontent tr td");
        },
        _enableFilterEvents: function () {
            if (this.model.allowMultiSorting || this.model.selectionType == "multiple" || this.model.allowFiltering)
                this._on($(document), "click", this._docClickHandler);
            if (this.model.allowFiltering) {
                var proxy = this, $target;
                this._off(this.element, "keyup", ".e-filterbar input")._on(this.element, "keyup", ".e-filterbar input", this._filterBarHandler);
                this._on(this.element, "focus click", ".e-filterbar", this._filterBarClose);
            }
        },

        // Handlers
        _filterBarClose: function (e) {
            var $target = $(e.target);
            if (e.type == "click" && $target.hasClass("e-cancel")) {
                $targetText = $target.prev();
                $targetText.focus().val("");
                $targetText.trigger("keyup");
                e.stopPropagation();
            }
            if (e.type == "focusin" && $target.hasClass("e-filtertext")) {
                var $target = $(e.target).next();
                this.getGridFilterBar().find(".e-cancel").addClass("e-hide");
                $target.removeClass("e-hide");
            }
        },
        _saveCellHandler: function (e) {
            var $target = $(e.target);
            e.stopPropagation();
            if ($target.closest(".e-popup").length == 0 && $target.closest(".e-rowcell").find("#" + this._id + "EditForm").length == 0)
                this.saveCell();
        },
        _dropAreaHover: function (e) {
            var $target = $(e.target), proxy = this;
            if (e.type == "mouseenter") {
                if (this._dragActive) {
                    if ($target.hasClass("e-groupdroparea"))
                        $target.addClass("e-hover");
                } else
                    $target.removeClass("e-hover");
                $target.hasClass("e-groupheadercell") && this.model.groupSettings.showUngroupButton && $target.find(".e-ungroupbutton").show(150);
            } else if (e.type == "mouseleave") {
                if ($target.hasClass("e-groupdroparea")) {
                    $target.find(".e-ungroupbutton").hide(150);
                    $target.removeClass("e-hover");
                }
                $target.hasClass("e-groupheadercell") && this.model.groupSettings.showUngroupButton && $target.find(".e-ungroupbutton").hide(150);
            }
            return false;
        },
        _mouseUp: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseUp(e);
        },

        _mouseMove: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseMove(e);
        },

        _getContentWidth: function (cellindx) {
            var contentWidth = 0;
            $span = ej.buildTag('span', {}, {});
            proxy = this;
            $.each(proxy._gridRows, function (indx, row) {
                var td = $(row).find('td.e-rowcell').eq(cellindx);
                var content = $(td).html();
                $span.html(content);
                $(td).html($span);
                tdWidth = td.find('span:first').width();
                if (tdWidth > contentWidth)
                    contentWidth = tdWidth;
                $(td).html(content);
            });
            return contentWidth;
        },

        _getHeaderContentWidth: function ($cellDiv) {
            var headerWidth = 0;
            $span = ej.buildTag('span', {}, {});
            var content = $cellDiv.html();
            $span.html(content);
            $cellDiv.html($span);
            headerWidth = $cellDiv.find('span:first').width();
            $cellDiv.html(content);
            return headerWidth;
        },

        _columnResizeToFit: function (e) {
            if (this.model.allowResizeToFit && this.getGridHeaderTable().find(".e-columnheader").css('cursor') == 'col-resize') {
                if ($(e.target).is(".e-headercelldiv"))
                    e.target = e.target.parentNode;
                var $target = $(e.target);
                if ($target.hasClass("e-headercell")) {
                    var targetCell = e.target;
                    var hCellIndex = targetCell.cellIndex;
                    var cellIndex = hCellIndex;
                    this._resizer._orgX = e.pageX;
                    this._resizer._getResizableCell();
                    if (hCellIndex != this._resizer._currentCell) {
                        hCellIndex = cellIndex = this._resizer._currentCell;
                        targetCell = e.target.previousSibling;
                    }
                    var $cellDiv = $(targetCell).children('.e-headercelldiv');
                    if (this.model.groupSettings.groupedColumns.length) {
                        cellIndex = hCellIndex - this.model.groupSettings.groupedColumns.length;
                    }
                    var finalWidth = 0, headerWidth = 0, contentWidth = 0;
                    contentWidth = this._getContentWidth(cellIndex);
                    headerWidth = this._getHeaderContentWidth($cellDiv);
                    finalWidth = headerWidth > contentWidth ? headerWidth : contentWidth;
                    finalWidth = finalWidth + (finalWidth * 0.2);
                    var oldWidth = this.getGridHeaderTable().find('col').eq(hCellIndex).width();
                    var args = { columnIndex: cellIndex, column: this.model.columns[cellIndex], target: $target, oldWidth: oldWidth };
                    this._trigger("resizeStart", args);
                    this.getGridHeaderTable().find('col').eq(hCellIndex).width(finalWidth);
                    if (this.model.groupSettings.groupedColumns.length) {
                        var $colGroups = this.getGridContentTable().find('.e-recordtable').find('colgroup');
                        $.each($colGroups, function (indx, colgroup) {
                            $(colgroup).find('col').eq(cellIndex).width(finalWidth);
                        });
                    }
                    else
                        this.getGridContentTable().find('col').eq(cellIndex).width(finalWidth);
                    var newWidth = this.getGridHeaderTable().find('col').eq(hCellIndex).width();
                    this._colgroupRefresh();
                    args = { columnIndex: cellIndex, column: this.model.columns[cellIndex], target: $target, oldWidth: oldWidth, newWidth: newWidth, extra: Math.abs(newWidth - oldWidth) };
                    this._trigger("resizeEnd", args);
                    this._columnsWidthCollection[cellIndex] = finalWidth;
                    this.model.columns[cellIndex]["width"] = finalWidth;
                    args = { columnIndex: cellIndex, column: this.model.columns[cellIndex], target: $target, oldWidth: oldWidth, newWidth: newWidth };
                    this._trigger("resized", args);
                }
            }
        },

        _colgroupRefresh: function () {
            var GridheaderCol = $(this.getGridHeaderTable()).find('colgroup')[0];
            var GridcontentCol = $(this.getGridContentTable()).find('colgroup')[0];
            var headerColClone = $(GridheaderCol).clone();
            var contentColClone = $(GridcontentCol).clone();
            $(GridcontentCol).remove();
            $(GridheaderCol).remove();
            $(headerColClone).prependTo(this.getGridHeaderTable());
            $(contentColClone).prependTo(this.getGridContentTable());
        },

        _headerMouseDown: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseDown(e);
            if (($(e.target).hasClass("e-headercelldiv") && !$(e.target).parent().hasClass("e-grouptopleftcell")) || $(e.target).hasClass("e-headercell")) {
                var $headercell = $(e.target).hasClass("e-headercelldiv") ? $(e.target).parent() : $(e.target);
                this.model.headerEffects && $headercell.removeClass("e-hover e-headercell-hover").addClass("e-headercellactive e-active");
            }
        },
        _headerHover: function (e) {
            var $target = $(e.target);
            if (e.type == "mouseover" || e.type == "mousemove" || e.type == "touchmove" || e.type == "MSPointerMove") {
                if (this.model.allowResizing || this.model.allowResizeToFit)
                    this._resizer._mouseHover(e);

                if (this.model.headerEffects && !this._dragActive && (($target.hasClass("e-headercelldiv") && !$target.parent().hasClass("e-grouptopleftcell")) || $target.hasClass("e-headercell"))) {
                    if ($target.hasClass("e-headercelldiv"))
                        $target = $target.parent();
                    this.getGridHeaderTable().find(".e-columnheader").find(".e-headercell-hover").removeClass("e-headercell-hover").removeClass("e-hover");
                    $target.addClass("e-headercell-hover e-hover");
                }

            } else
                this.model.headerEffects && this.getGridHeaderTable().find(".e-columnheader").find(".e-headercell-hover").removeClass("e-headercell-hover").removeClass("e-hover");
        },
        _groupHeaderCellClick: function (e) {
            var $target = $(e.target);
            if ($target.hasClass("e-ungroupbutton")) {
                var columnName = $.trim($target.parent().text());
                var field = this.getFieldNameByHeaderText(columnName);
                this.sendUngroupingRequest(field);
            } else if ($target.hasClass("e-togglegroupbutton")) {
                var columnName = $.trim($target.parent().text()), field;
                field = this.getFieldNameByHeaderText(columnName);
                $target.hasClass("e-toggleungroup") && this.sendUngroupingRequest(field);
            } else if ($target.hasClass("e-animatebutton")) {
                if (!this.model.enableEffects) {
                    this.model.enableEffects = true;
                    this.collapseGroupDropArea();
                } else {
                    this.expandGroupDropArea();
                    this.model.enableEffects = false;
                }
            }

            return false;
        },

        _rowHover: function (e) {
            if (!this.model.rowHover)
                return;
            var $target = $(e.target);
            var $gridRows = $(this.getGridRows());
            if (($target.hasClass("e-rowcell") && $target.closest("#" + this._id + "EditForm").length) || !$target.hasClass("e-rowcell"))
                return;
            if (e.type == "mouseenter" && !this._dragActive) {
                if (this.model.scrolling.frozenColumns > 0 && !ej.isNullOrUndefined($gridRows[0]) && !ej.isNullOrUndefined($gridRows[1]))
                    $gridRows = $($gridRows[0]).add($gridRows[1]);
                $gridRows.removeClass("e-hover");
                var index = this.getIndexByRow($target.parent());
                index != -1 && this.getRowByIndex(index).addClass("e-hover");
            } else {
                if (this.model.scrolling.frozenColumns > 0 && !ej.isNullOrUndefined($gridRows[0]) && !ej.isNullOrUndefined($gridRows[1]))
                    $gridRows = $($gridRows[0]).add($gridRows[1]);
                $gridRows.removeClass("e-hover");
            }
            return false;
        },
        _unboundClickHandler: function (e) {
            var $target = $(e.target);
            if ($target.hasClass("e-unboundcelldiv"))
                return;
            this.model.allowSelection && this.selectRows(this.getIndexByRow($target.closest("tr")));
            $.isFunction($.fn.ejDatePicker) && $("#" + this._id + "EditForm").find(".e-datepicker").ejDatePicker("hide");
            if ($target.hasClass("e-editbutton")) {
                var index = this.getIndexByRow($target.closest("tr"));
                if (this.model.isEdit)
                    this.sendCancelRequest();
                var $tr = this.getRowByIndex(index);
                this.sendEditingRequest($tr);
            } else if ($target.hasClass("e-deletebutton"))
                this.sendDeleteRequest($target.closest("tr"));
            else if ($target.hasClass("e-savebutton"))
                this.sendSaveRequest();
            else if ($target.hasClass("e-cancelbutton"))
                this.sendCancelRequest();
        },
        _docClickHandler: function (e) {
            var details = this.getBrowserDetails();
            if (this._customPop != null && this.element.find(e.target).length == 0)
                this._customPop.hide();
            if (this.model.allowFiltering) {
                if (this.model.filterSettings.filterType == "menu") {
                    if (this._$colType && $(e.target).find(".e-grid.e-dlgcontainer").length > 1)
                        if (details.browser == "msie")
                            e.target.tagName != "BODY" && this._closeFilterDlg();
                        else
                            this._closeFilterDlg();
                }
                else
                    if (!$(e.target).hasClass("e-filtertext"))
                        this.getGridFilterBar().find(".e-cancel").addClass("e-hide");
            }

        },
        _closeFilterDlg: function () {
            $("#" + this._id + "_" + this._$colType + "Dlg").ejDialog("close");
            this._$fDlgIsOpen = false;
        },
        _clickHandler: function (e) {
            var $target = $(e.target), fieldName, $form = $("#" + this._id + "EditForm"), index;
            if ($target.closest("#" + this._id + "EditForm").length)
                return;
            if ($target.hasClass("e-rowcell") || $target.parent().hasClass("e-rowcell")) {
                if (this._bulkEditCellDetails.cancelSave) {
                    this._bulkEditCellDetails.cancelSave = false;
                    return;
                }
                if (this.model.edit.editMode == "batch" && ($.isFunction($.validator) && $form.length && $form.validate().errorList.length > 0))
                    return;
                this.model.edit.editMode == "batch" && this.element.focus();
                index = $target.closest("tr").hasClass("e-insertedrow") ? this.model.groupSettings.groupedColumns.length : 0;
                this._bulkEditCellDetails.columnIndex = $target.hasClass("e-rowcell") ? $target.index() : $target.parent().index() - index;
                this._bulkEditCellDetails.rowIndex = $(this.getGridRows()).index($target.closest('tr'));
                if (this.model.allowSelection) {
                    if (this.model.selectionType == ej.Grid.selectionType.Multiple) {
                        if (e.ctrlKey || this._enableSelectMultiTouch) {
                            this._multiSelectCtrlRequest = true;

                        }
                        if (e.shiftKey) {
                            this._multiSelectShiftRequest = true;
                            this.selectRows(this._previousIndex, this.getIndexByRow($target.closest('tr')));
                        }
                        if (e["pointerType"] == "touch" && this._customPop != null && !this._customPop.is(":visible") && this._customPop.find(".e-rowselect").hasClass("e-spanclicked"))
                            this._customPop.show();
                        if (e["pointerType"] == "touch" && this._customPop != null && (this._customPop.find(".e-sortdirect").is(":visible") || !this._customPop.find(".e-rowselect").hasClass("e-spanclicked")) && this.model.selectionType == "multiple") {
                            this._customPop.removeAttr("style");
                            var offset = $target.offset();
                            this._customPop.offset({ top: 0, left: 0 }).offset({ left: offset.left, top: offset.top - this.getGridRowHeight() }).find(".e-sortdirect").hide().end()
                                .find(".e-rowselect").show().end().show();
                        }
                    }
                    !this._multiSelectShiftRequest && this.selectRows(this.getIndexByRow($target.closest('tr')));
                    this._multiSelectShiftRequest = false;
                }
                fieldName = this.model.columns[this._bulkEditCellDetails.columnIndex]["field"];
                if (fieldName)
                    this.model.edit.allowEditing && this.model.edit.editMode == ej.Grid.editMode.Batch && this.bulkEdit($.inArray($target.closest("tr").get(0), this.getGridRows()), fieldName);
            }
            if ($target.hasClass("e-rowselect") || $target.hasClass("e-sortdirect")) {
                if (!$target.hasClass("e-spanclicked")) {
                    $target.addClass("e-spanclicked");
                    if ($target.hasClass("e-rowselect"))
                        this._enableSelectMultiTouch = true;
                    if ($target.hasClass("e-sortdirect"))
                        this._enableSortMultiTouch = true;
                }
                else {
                    $target.removeClass("e-spanclicked");
                    if ($target.hasClass("e-rowselect"))
                        this._enableSelectMultiTouch = false;
                    if ($target.hasClass("e-sortdirect"))
                        this._enableSortMultiTouch = false;
                    this._customPop.hide();
                }
            }
            this.expandCollapse($target);
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "menu" && !$target.is(".e-filtericon") && $target.closest(".e-dlgcontainer").length != 1)
                this._closeFilterDlg();
        },

        _editdblClickHandler: function (e) {
            var $target = $(e.target);
            if ($target.hasClass("e-rowcell")) {
                if (!this.model.isEdit) {
                    this._$currentTr = (this.model.scrolling.frozenColumns > 0 || this.model.scrolling.frozenRows > 0)
                        ? this.getRowByIndex($target.closest('tr').index())
                        : $target.closest('tr');
                    this.sendEditingRequest(this._$currentTr);
                }
            }
            return false;
        },

        _rightClickHandler: function (e) {
            e.preventDefault();
            if (e.which == 3) {
                var args = {};
                $target = $(e.target);
                $gridRow = $(this.getGridRows());
                if (this.getGridContentTable().has($target).length) {
                    var index = $gridRow.index($target.parent());
                    if (index == -1)
                        return;
                    args = { currentRowIndex: index, currrentRow: this.getRowByIndex(index), currentData: this.model.currentViewData[index], currentCellIndex: $target.index(), currentCellValue: $target.html(), currentCell: $target };
                }
                else if (this.getGridHeaderTable().has($target).length) {
                    var index = 0
                    $th = this.getGridHeaderTable().find('th').not('.e-detailheadercell,.e-grouptopleftcell,.e-filterbarcell');
                    if ($target.is('.e-headercelldiv'))
                        index = $th.index($target.closest('.e-headercell'));
                    else
                        index = $th.index($target);
                    if (index == -1)
                        return;
                    args = { headerIndex: index, headerText: this.getColumnFieldNames()[index], headerCell: $th.eq(index), column: this.getColumnByIndex(index) }
                }
                else if ($target.is('.e-pager') || (this.getGridPager() != null && this.getGridPager().has($target).length)) {
                    args = { pager: this.model.pageSettings }
                }
                this._trigger("gridRightClick", args);
            }
        },

        _recorddblClickHandler: function () {
            var args = {};
            args = { currentRowIndex: this._selectedRow(), currrentRow: this.getRowByIndex(this._selectedRow()), currentData: this.model.currentViewData[this._selectedRow()] };
            this._trigger("recordDoubleClick", args);
        },

        _recordClick: function (e) {
            var args = {};
            $target = $(e.target);
            if (!$target.is('.e-rowcell'))
                return;
            var index = this.getIndexByRow($target.closest('tr'));
            args = { currentRowIndex: index, currrentRow: this.getRowByIndex(index), currentData: this.model.currentViewData[index] };
            this._trigger("recordClick", args);
        },

        _keyPressed: function (action, target, e) {
            var $target = $(target);
            if (!this.model.allowKeyboardNavigation || (target.tagName == 'INPUT' && e.code != 40 && e.code != 38 && e.code != 13 && e.code != 27 && e.code != 9) || String.fromCharCode(e.code).toLowerCase() == this.element[0].accessKey.toLowerCase())
                return true;
            if ((this.model.edit.editMode == "dialog" || this.model.edit.editMode == "dialogtemplate") && $(target).closest("#" + this._id + "EditForm").length)
                return true;
            else if ($(target).parent().siblings("#" + this._id + "EditForm").length)
                return true;
            if (e.code == 13 && $target.parent().hasClass("e-unboundcelldiv"))
                return true;
            if (e.code == 13 && target.tagName == 'INPUT' && $target.parent().is("#" + this._id + "_search"))
                action = "searchRequest";
            if (this.getGridPager() != null)
                var pager = this.getGridPager().ejPager("model"), pageIndex = pager.currentPage;
            var returnValue = false, curEl, $target = $(target);

            switch (action) {
                case "insertRecord":
                    this._toolbarOperation(this._id + "_add");
                    break;
                case "searchRequest":
                    this.sendSearchRequest($target.val());
                    break;
                case "saveRequest":
                    if (this.model.edit.editMode == "batch")
                        this._moveCurrentCell("down");
                    else
                        this._toolbarOperation(this._id + "_update");
                    break;
                case "cancelRequest":
                    this._toolbarOperation(this._id + "_cancel");
                    break;
                case "deleteRecord":
                    this._toolbarOperation(this._id + "_delete");
                    break;
                case "editRecord":
                    this._toolbarOperation(this._id + "_edit");
                    break;
                case "totalGroupCollapse":
                    this.collapseAll();
                    break;
                case "totalGroupExpand":
                    this.expandAll();
                    break;
                case "selectedGroupExpand":
                    this._$currentTr = $(this.getGridRows()).eq(this._selectedRow());
                    curEl = this._$currentTr.parents("tr").first().prev().find(".e-recordpluscollapse");
                    this.expandCollapse(curEl);
                    break;
                case "selectedGroupCollapse":
                    this._$currentTr = $(this.getGridRows()).eq(this._selectedRow());
                    curEl = this._$currentTr.parents("tr").first().prev().find(".e-recordplusexpand");
                    this.expandCollapse(curEl);
                    break;
                case "firstRowSelection":
                    this.selectRows(0);
                    break;
                case "lastRowSelection":
                    this.selectRows($(this.getGridRows()).length - 1);
                    break;
                case "upArrow":
                    if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.edit.editMode != "batch")
                        return true;
                    if (this._selectedRow() != 0) {
                        this.selectRows(this._selectedRow() - 1);
                        if (this.model.edit.editMode == "batch")
                            this._moveCurrentCell("up");
                    }
                    break;
                case "downArrow":
                    if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.edit.editMode != "batch")
                        return true;
                    var lastRow = $(this.getGridRows()).length - 1;
                    if (this.model.scrolling.frozenColumns > 0 && !ej.isNullOrUndefined(this.getGridRows()[0]))
                        lastRow = this.getGridRows()[0].length - 1;
                    if (this._selectedRow() != lastRow) {
                        this.selectRows(this._selectedRow() + 1);
                        if (this.model.edit.editMode == "batch")
                            this._moveCurrentCell("down");
                    }
                    break;
                case "nextPage":
                    if (this.getGridPager() != null)
                        pageIndex = pageIndex + 1;
                    break;
                case "previousPage":
                    if (this.getGridPager() != null)
                        pageIndex = pageIndex - 1;
                    break;
                case "lastPage":
                    if (this.getGridPager() != null)
                        pageIndex = pager.totalPages;
                    break;
                case "firstPage":
                    if (this.getGridPager() != null)
                        pageIndex = 1;
                    break;
                case "nextPager":
                    if (this.getGridPager() != null)
                        pageIndex = Math.ceil(pager.currentPage / pager.pageCount) * pager.pageCount + 1;
                    break;
                case "previousPager":
                    if (this.getGridPager() != null)
                        pageIndex = (Math.floor(pager.currentPage / pager.pageCount) - 1) * pager.pageCount + 1;
                    break;
                case "moveCellLeft":
                    if (this.model.edit.editMode == "batch")
                        returnValue = this._moveCurrentCell("left");
                    else
                        returnValue = true;
                    break;
                case "moveCellRight":
                    if (this.model.edit.editMode == "batch" && $target)
                        returnValue = this._moveCurrentCell("right");
                    else
                        returnValue = true;
                    break;
                default:
                    returnValue = true;
            }
            if (this.getGridPager() != null && pager.currentPage !== pageIndex)
                this.getGridPager().ejPager("goToPage", pageIndex);
            return returnValue;
        },

        _mouseClickHandler: function (e) {
            if (this.getGridHeaderTable().find('.e-columnheader').css('cursor') == "col-resize")
                return;
            if ($(e.target).is(".e-ascending, .e-descending"))
                e.target = e.target.parentNode;
            var $target = $(e.target);
            if (this._$fDlgIsOpen && this.model.allowFiltering && this.model.filterSettings.filterType == "menu") {
                $.fx.off = true;
                $("#" + this._id + "_" + this._$prevColType + "Dlg").ejDialog("close");
                $.fx.off = false;
            }
            this.getGridHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive").removeClass("e-active");
            if ($target.hasClass("e-headercelldiv") || (!$target.hasClass("e-togglegroupbutton") && $target.closest(".e-headercelldiv").length)) {
                if (!this.model.allowSorting)
                    return;
                $target = $target.hasClass("e-headercelldiv") ? $target : $target.closest(".e-headercelldiv");
                var columnName = $target.attr("ej-mappingname");
                var columnSortDirection = ej.sortOrder.Ascending;
                this._$prevSElementTarget = this._$curSElementTarget;
                this._$curSElementTarget = $target;
                if ($target.find('span').hasClass("e-ascending"))
                    var columnSortDirection = ej.sortOrder.Descending;
                else
                    var columnSortDirection = ej.sortOrder.Ascending;
                if (e["pointerType"] == "touch" && this._customPop != null && !this._customPop.is(":visible") && this._customPop.find(".e-sortdirect").hasClass("e-spanclicked"))
                    this._customPop.show();
                if (e["pointerType"] == "touch" && this._customPop != null && (this._customPop.find(".e-rowselect").is(":visible") || !this._customPop.find(".e-sortdirect").hasClass("e-spanclicked")) && this.model.allowMultiSorting) {
                    var $offset = $target.offset();
                    this._customPop.removeAttr("style");
                    this._customPop.offset({ left: $offset.left, top: $offset.top - this.getGridHeaderTable().find(".e-columnheader").height() }).find(".e-sortdirect").show().end()
                        .find(".e-rowselect").hide().end().show();
                }
                if (this.model.allowMultiSorting && (e.ctrlKey || this._enableSortMultiTouch))
                    this._multiSortRequest = true;
                if (e.shiftKey && $.inArray(columnName, this.model.groupSettings.groupedColumns) == -1) {
                    this._removeSortedColumnFromCollection(columnName);
                    this._multiSortRequest = true;
                    columnName = null;
                }
                this.sendSortingRequest(columnName, columnSortDirection);
            } else if ($target.hasClass("e-togglegroupbutton")) {
                var field = $target.parent().attr("ej-mappingname");
                $target.hasClass("e-togglegroup") && this.sendGroupingRequest(field);
                $target.hasClass("e-toggleungroup") && this.sendUngroupingRequest(field);
            } else if ($target.hasClass("e-filtericon") || $target.hasClass("e-filteredicon")) {
                var columnName = $target.parent().find(".e-headercelldiv").attr("ej-mappingname");
                this._$prevFieldName = this._$curFieldName;
                if (this.model.allowFiltering) {
                    var proxy = this;
                    $.each(this.model.columns, function (indx, col) {
                        if (col.field == columnName) {
                            proxy._$colType = col.type;
                            proxy._$curFieldName = col.field;
                            proxy._$colFormat = col.format;
                            proxy._$colForeignKeyField = col.foreignKeyField ? col.foreignKeyField : col.field;
                            proxy._$colForeignKeyValue = col.foreignKeyValue;
                            proxy._$colDropdownData = col.dataSource;
                        }
                    });
                    var $id = "#" + this._id + "_" + this._$colType + "Dlg";
                    if (this._$colType == "string") {
                        if (this._$colForeignKeyValue && this._$colDropdownData)
                            $("#" + this._id + "_acString").ejAutocomplete({ fields: { text: proxy._$colForeignKeyValue, id: proxy._$colForeignKeyField }, dataSource: proxy._$colDropdownData });
                        else
                            $("#" + this._id + "_acString").ejAutocomplete({ fields: { text: proxy._$curFieldName, id: this._getIdField() }, dataSource: this.model.dataSource });
                    }
                    else if (this._$colType == "date") {
                        this._$colFormat = this._$colFormat.replace("{0:", "").replace("}", "");
                        $($id).find(".e-datewidget input").ejDatePicker({ dateFormat: this._$colFormat.replace("{0:", "").replace("}", "") });
                    }
                    this._setFilterFieldValues($id);
                    var docWidth = $(document).width(), dlgWidth = 250, xPos = e.clientX;
                    if (docWidth - e.clientX <= dlgWidth)
                        xPos = docWidth - dlgWidth;
                    $($id).ejDialog({ position: { X: xPos, Y: $target.parent().offset().top + 32 } })
                        .ejDialog("open");
                    if (this._$colType == "number")
                        $($id).find(".e-numerictextbox").ejNumericTextbox({ width: "230px" });
                    this._$prevColType = this._$colType;
                    this._$fDlgIsOpen = true;
                }
            }

            e.preventDefault();

        },
        _setFilterFieldValues: function (id) {
            var $fVal = "", proxy = this;
            if (this._$curFieldName != this._$prevFieldName) {
                $.each(this.model.filterSettings.filteredColumns, function (indx, col) {
                    if (col.field == proxy._$curFieldName) {
                        $fVal = col.actualFilterValue != null ? col.actualFilterValue : col.value;
                        return false;
                    }
                });
                if (this._$colType == "boolean") {
                    if ($fVal && $fVal != "")
                        $(id).find(".e-value input").attr("checked", true);
                    else
                        $(id).find(".e-value input").attr("checked", false);
                } else
                    $(id).find(".e-value input").val($fVal);
            }
        },
        _getIdField: function () {
            var $key;
            $.each(this.model.columns, function (indx, col) {
                if (col.key) {
                    $key = col.field;
                    return false;
                }
            });
            return $key;
        },
        _gPagerClickHandler: function (sender) {
            if (this._prevPageNo == sender.currentPage)
                return;
            this.model.masterObject.sendPagingRequest(sender.currentPage);
            return false;
        },
        _toolBarClick: function (Sender) {
            var $gridEle = $(this.itemsContainer).closest(".e-grid"), gridInstance = $gridEle.ejGrid("instance"), gridId = $gridEle.attr('id');
            if (Sender.event == undefined && Sender.target.tagName == "INPUT" && Sender.currentTarget.id == gridId + "_search")
                return;
            $.isFunction($.fn.ejDatePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDatePicker("hide");
            if ($gridEle.ejGrid("instance")._trigger("toolBarClick", Sender))
                return;
            switch (Sender.currentTarget.id) {
                case gridId + "_add":
                    gridInstance._toolbarOperation(gridId + "_add");
                    break;
                case gridId + "_edit":
                    gridInstance._toolbarOperation(gridId + "_edit");
                    break;
                case gridId + "_delete":
                    gridInstance._toolbarOperation(gridId + "_delete");
                    break;
                case gridId + "_update":
                    gridInstance._toolbarOperation(gridId + "_update");
                    break;
                case gridId + "_cancel":
                    if (gridInstance.model.edit.editMode == "batch")
                        gridInstance.sendCancelRequest();
                    else
                        gridInstance._toolbarOperation(gridId + "_cancel");
                    break;
                case gridId + "_search":
                    gridInstance._toolbarOperation(gridId + "_search", $(Sender.currentTarget).find("input").val());
                    break;
            }
            return false;
        },

        _toolbarOperation: function (operation, searchEle) {
            var $gridEle = this.element, gridObject = $gridEle.ejGrid("instance"), batchEnable = gridObject.model.edit.editMode == "batch", gridId = $gridEle.attr('id'), fieldName;
            switch (operation) {
                case gridId + "_add":
                    if (batchEnable)
                        gridObject.bulkAddRow();
                    else
                        gridObject.sendAddRequest();
                    break;
                case gridId + "_edit":
                    if (batchEnable) {
                        fieldName = gridObject.model.columns[gridObject._bulkEditCellDetails.columnIndex].field;
                        fieldName && gridObject.bulkEdit(gridObject._bulkEditCellDetails.rowIndex, fieldName);
                    }
                    else
                        gridObject.sendEditingRequest();
                    break;
                case gridId + "_delete":
                    if (batchEnable)
                        gridObject.bulkDelete();
                    else
                        gridObject.sendDeleteRequest();
                    break;
                case gridId + "_update":
                    if (batchEnable)
                        this._confirmDialog.find(".e-content").html(this._getLocalizedLabels("BulkSaveConifrm")).end().ejDialog("open");
                    else
                        gridObject.sendSaveRequest();
                    break;
                case gridId + "_cancel":
                    if (batchEnable) {
                        if ($("#" + gridId + "ConfirmDialog").ejDialog("isOpened") === true)
                            this._triggerConfirm();
                        else
                            gridObject.cancelEditCell();
                    }
                    else
                        gridObject.sendCancelRequest();
                    break;
                case gridId + "_search":
                    $gridEle.ejGrid("sendSearchRequest", searchEle);
                    break;
            }
            return false;
        },

        _filterBarHandler: function (e) {
            var keycode = e.keyCode;
            if ((this.model.filterSettings["filterBarMode"] == "immediate" || keycode == 13) && keycode != 9) {
                var $target = $(e.target);
                this.filterStatusMsg = "";
                var fieldName = $target.prop("id").replace("_filterBarcell", "");
                var column = this.getColumnByField(fieldName);
                if (column == null)
                    return;
                this._currentFilterColumn = column;
                if (this._currentFilterColumn != this._oldFilterColumn)
                    this.filterValueOldLength = 0;
                this._currentFilterbarValue = $target.val();
                this.filterValueCurrentLength = this._currentFilterbarValue.length;
                if (((this.filterValueCurrentLength == 0 && this.filterValueOldLength == 0) || this._currentFilterbarValue == this.OldfilterValue) && this._currentFilterColumn == this._oldFilterColumn) {
                    this._showFilterMsg();
                    return;
                }
                this._skipFilterProcess = this._checkForSkipInput();
                if (!this._skipFilterProcess) {
                    this._processFilter();
                } else {
                    if (this._currentFilterColumn.type == "string") {
                        this.filterStatusMsg = "Invalid Filter Data";
                        this._showFilterMsg();
                    } else {
                        this._skipFilterProcess = false;
                        this._showFilterMsg();
                        return;
                    }
                }
            }
        },

        //event models

        editFormValidate: function () {
            if ($.isFunction($.validator)) {
                if (this.model.scrolling.frozenColumns > 0) {
                    var forms = this.element.find(".gridform");
                    return forms.length > 1 ? forms.eq(0).validate().form() && forms.eq(1).validate().form() : forms.validate().form();
                }
                return $("#" + this._id + "EditForm").validate().form();
            }
            return true;
        },
        _checkEinHeader: function (field) {
            var $headerCell = this.element.children(".e-gridheader").find("thead").find(".e-columnheader").find(".e-headercell");
            for (var i = 0; i < $headerCell.length; i++) {
                if ($.trim($headerCell.eq(i).text()) == field)
                    return $headerCell.eq(i);
            }
            return null;

        },

        _checkEinGroupDrop: function (field) {
            var $groupHeaderCell = this.element.children(".e-groupdroparea").find(".e-grid-icon");
            for (var i = 0; i < $groupHeaderCell.length; i++) {
                if ($.trim($groupHeaderCell.eq(i).text()) == field)
                    return $groupHeaderCell.eq(i);
            }
            return null;
        },

        _addColumnToGroupDrop: function (field) {
            var $groupedColumn = ej.buildTag("div.e-grid-icon e-groupheadercell"), $groupDropArea = this.element.find(".e-groupdroparea").first();
            var $childDiv = ej.buildTag("div"), imageDirection = "e-rarrowup-2x";
            var column = this.getColumnByField(field)
            $groupedColumn.append($childDiv.html(column.headerText));
            var $headerCell = this._checkEinHeader(column.headerText);
            if (this.model.groupSettings.toggleGrouping) {
                $childDiv.append(this._getToggleButton().addClass("e-toggleungroup"));
                $headerCell.find(".e-headercelldiv").find(".e-togglegroupbutton").remove().end().append(this._getToggleButton().addClass("e-toggleungroup"));
            }
            var direction = "ascending";
            if ($headerCell.find(".e-ascending,.e-descending").length) {
                direction = $headerCell.find(".e-ascending,.e-descending").hasClass("e-ascending") ? "ascending" : "descending";
                imageDirection = direction == "ascending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
            }
            $childDiv.append(this._createSortElement().addClass("e-" + direction + " " + imageDirection));
            this.model.groupSettings.showUngroupButton && $childDiv.append(ej.buildTag("span.e-ungroupbutton e-icon e-cancel", " ", {}, { title: "Click here to ungroup" }));
            $groupDropArea.append($groupedColumn).css("height", "auto");
            var left = $groupedColumn.offset().left, $cloned = $groupedColumn.clone().css("position", "absolute"), proxy = this;
            $groupedColumn.css("visibility", "hidden")
            $groupDropArea.append($cloned).dequeue();
            $cloned.css({ "left": left + 150 }).animate({ left: left }, 150, function (e) {
                $groupedColumn.css("visibility", "visible");
                $cloned.remove();
            });
        },
        _removeColumnFromDrop: function (field) {
            var headerText = this.getHeaderTextByFieldName(field), proxy = this, $groupDropArea = this.element.children(".e-groupdroparea");
            var $groupHeaderCell = $groupDropArea.css("height", "30px").find(".e-grid-icon");
            for (var i = 0; i < $groupHeaderCell.length; i++) {
                if ($.trim($groupHeaderCell.eq(i).text()) == headerText) {
                    if (this.model.groupSettings.groupedColumns.length == 0) {
                        //this.model.groupSettings.groupedColumns.length == 0 && this.model.enableEffects && $groupDropArea.animate({ height: "0px" }, 200, function () {
                        //    if (proxy.model.groupSettings.groupedColumns.length == 0) {
                        //        $(this).html("");
                        //        $(this).dequeue().css("height", "auto");
                        //    }
                        //});
                        this.collapseGroupDropArea();
                    } else
                        $groupHeaderCell.eq(i).remove();
                }
            }
        },
        _addSortElementToColumn: function (field, direction) {
            var column = this.getColumnByField(field), imageDirection;
            var index = $.inArray(column, this.model.columns);
            var $headerCellDiv = this.getGridHeaderTable().find("thead").find(".e-headercell").not(".e-detailheadercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            imageDirection = direction == "ascending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
            $headerCellDiv.append(this._createSortElement().addClass("e-" + direction + " " + imageDirection));
            $headerCellDiv.parent().attr("aria-sort", direction);

        },
        _removeSortElementFromColumn: function (field) {
            var column = this.getColumnByField(field);
            var index = $.inArray(column, this.model.columns);
            var $headerCellDiv = this.getGridHeaderTable().find("thead").find(".e-headercell").not(".e-detailheadercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            $headerCellDiv.parent().removeAttr("aria-sort");
        },
        _refreshEditForm: function () {
            var form = this.model.scrolling.frozenColumns > 0 ? this.element.find(".gridform") : document.getElementById(this._id + "EditForm");
            var elementFocused = false;
            var $formElement = $(form).find("input,select"), percent = 86;

            for (var i = 0; i < $formElement.length; i++) {
                var $element = $formElement.eq(i);
                if ($element.closest("td").hasClass("e-unboundcell"))
                    continue;
                var inputWidth;
                if (this.model.edit.editMode == "externalform" || this.model.edit.editMode == "externalformtemplate" || this.model.edit.editMode == "inlineform" || this.model.edit.editMode == "inlineformtemplate") {
                    $element.parent().css("width", ej.dataUtil.max(this._tdsOffsetWidth) + "px");
                }
                if (this.model.edit.editMode == "batch")
                    percent = 95;
                else if (this.model.edit.editMode == "normal")
                    percent = 96;
                inputWidth = this._tdsOffsetWidth[i] * (percent / 100);
                if ($element.hasClass("e-numerictextbox")) {
                    var width = inputWidth;
                    var params = {};
                    var value = $element.val();
                    params.width = width;
                    params.rtl = this.model.rtl;
                    params.showSpinButton = true;
                    params.cssClass = this.model.cssClass;
                    if (value.length)
                        params.value = parseFloat(value);
                    var customParams = this.getColumnByField($element.prop("name"));
                    if (!ej.isNullOrUndefined(customParams["editParams"]))
                        $.extend(params, customParams["editParams"]);
                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(this._id, ""));
                } else if ($element.hasClass("e-datepicker")) {
                    var width = inputWidth;
                    var params = {};
                    params.width = width;
                    params.rtl = this.model.rtl;
                    params.cssClass = this.model.cssClass;
                    params.displayDefaultDate = true;
                    params.showDateIcon = false;
                    if ($element.val().length)
                        params.value = new Date($element.val());
                    var column = this.getColumnByField($element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        var formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDatePicker(params);
                }
                else if ($element.hasClass("e-datetimepicker")) {
                    var width = inputWidth;
                    var params = {
                        width: width,
                        rtl: this.model.rtl,
                        cssClass: this.model.cssClass,
                        showButton: false
                    };
                    if ($element.val().length)
                        params.value = new Date($element.val());
                    var column = this.getColumnByField($element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        var formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDateTimePicker(params);
                }
                else if ($element.hasClass("e-dropdownlist")) {
                    $element.ejDropDownList({
                        width: inputWidth + "px",
                        rtl: this.model.rtl
                    });
                    $element.ejDropDownList("selectItemByValue", $element.val());

                }
                    //else if ($element.hasClass("checkbox"))
                    //{
                    //    var value = $element.prop("checked");
                    //    $element.ejCheckBox({
                    //        id: $element[0].id,
                    //        checked: value
                    //    });
                    //}
                else {
                    switch ($element.prop('tagName')) {
                        case "INPUT":
                            if ($element.attr("type") != "checkbox") {
                                $element.css("text-align", $element.attr("name") != null && this.getColumnByField($element.attr("name")) != null ?
                                this.getColumnByField($element.attr("name")).textAlign : "center");
                                if (this.model.edit.editMode == "batch")
                                    $element.width('100%').height(28);
                                else if (this.model.edit.editMode == "normal")
                                    $element.css('width', '100%').css("height", "30px");
                                else
                                    $element.outerWidth(inputWidth).height(28);

                            }
                            else
                                $element.width(inputWidth > 0 ? ($element.width() > inputWidth ? inputWidth : $element.width()) : 1);
                            break;
                        case "SELECT":
                            $element.width(inputWidth).height(28);
                            break;
                    }
                }
                if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {
                    this._focusElements($element.closest('td'));
                    elementFocused = true;
                }
            }

        },
        _focusElements: function ($currentCell) {
            if ($currentCell.length) {
                var $childElem = $currentCell.children();
                if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {
                    $childElem.focus().select();
                    $childElem[0].focus();
                }
                else if ($childElem.find(".e-field.e-dropdownlist").length)
                    $childElem.find(".e-ddl").focus();
                else
                    $childElem.find('input,select').first().select().focus();
            }
        },

        _processFilter: function () {
            if (!this._alreadyFilterProcessed) {
                this._alreadyFilterProcessed = true;
                this._startTimer();
            } else {
                this._stopTimer();
                this._startTimer();
            }
        },
        _startTimer: function () {
            var proxy = this;
            this._timer = window.setTimeout(
                function () {
                    proxy._onTimerTick();
                },
                1500);
        },
        _stopTimer: function () {
            this._oldFilterColumn = this._currentFilterColumn;
            this.filterValueOldLength = this.filterValueCurrentLength;
            if (this._timer != null)
                window.clearTimeout(this._timer);
        },

        _onTimerTick: function () {
            this.OldfilterValue = this._currentFilterbarValue;
            this._findPredicate();
            var result = null;
            var matchcase = this._currentFilterColumn.type == "string" ? false : true;
            if (this._currentFilterColumn.foreignKeyValue && this._currentFilterColumn.dataSource && this._currentFilterbarValue != "")
                result = this._fltrForeignKeyValue(this._operator, this._currentFilterbarValue, matchcase,
                         this._currentFilterColumn.dataSource, this._currentFilterColumn.foreignKeyField,
                         this._currentFilterColumn.foreignKeyValue, this._currentFilterColumn.type);
            if (!this._skipFilterProcess) {
                if (ej.isNullOrUndefined(result))
                    this.sendFilteringRequest(this._currentFilterColumn.field, this._operator, this._currentFilterbarValue, this._predicate, matchcase);
                else
                    this.sendFilteringRequest(this._currentFilterColumn.field, result.operator, result.value, result.value.length > 1 ? "or" : this._predicate, matchcase, result.filterValue);
            }
            else
                this.filterStatusMsg = "Invalid Filter Data";
            this._showFilterMsg();
            this._stopTimer();
        },

        _findPredicate: function () {
            var _value = this._currentFilterbarValue.replace(/ && /i, " and ").replace(" || ", " or ");
            var _predicateFinder = _value.split(' ');
            this._predicate = "and";
            if (_predicateFinder.length != 0) {
                if ($.isFunction(ej.Predicate[_predicateFinder[1]])) {
                    this._skipFilterProcess = false;
                    this._predicate = _predicateFinder[1];
                    var valuesArray = _value.split(" " + _predicateFinder[1] + " ");
                    var tempOperator = [];
                    var filterValues = [];
                    for (var i = 0; i < valuesArray.length; i++) {
                        this._validateFilterValue(valuesArray[i]);
                        tempOperator.push(this._operator);
                        if (this._currentFilterColumn.type == "number")
                            filterValues.push(this._currentFilterbarValue);
                        else if (this._currentFilterColumn.type == "string")
                            filterValues.push(valuesArray[i]);
                    }
                    this._currentFilterbarValue = filterValues;
                    this._operator = tempOperator;
                } else
                    this._validateFilterValue($.trim(this._currentFilterbarValue));
            } else
                this._validateFilterValue($.trim(this._currentFilterbarValue));
        },

        _validateFilterValue: function (_value) {
            switch (this._currentFilterColumn.type) {
                case "number":
                    this._operator = ej.FilterOperators.equal;
                    var stringSkipInput = new Array(">", "<", "=", "!");
                    for (var i = 0; i < _value.length; i++) {
                        if (jQuery.inArray(_value[i], stringSkipInput) != -1) {
                            break;
                        }
                    }
                    if (i != _value.length) {
                        this._getOperator(_value.substring(i));
                        if (i != 0)
                            this._currentFilterbarValue = _value.substring(0, i);
                    }
                    if (this._currentFilterbarValue != "" && _value.length > 1)
                        this._currentFilterbarValue = parseFloat(this._currentFilterbarValue);
                    else
                        this._currentFilterbarValue = _value.length > 1 ? parseFloat(_value) : _value;
                    break;
                case "date":
                    this._operator = ej.FilterOperators.equal;
                    this._getOperator(_value);
                    var _format;
                    if (this._currentFilterColumn.format == "")
                        _format = Globalize.culture().calendar.patterns.d; //System Date format
                    else
                        _format = this._currentFilterColumn.format.split(':')[1].replace('}', "");
                    if (this._currentFilterbarValue != "")
                        this._currentFilterbarValue = Globalize.parseDate(this._currentFilterbarValue, _format);
                    break;
                case "string":
                    this._operator = ej.FilterOperators.startsWith;
                    break;
                case "boolean":
                    if (this._currentFilterbarValue.toLowerCase() == "true" || this._currentFilterbarValue == "1")
                        this._currentFilterbarValue = true;
                    else if (this._currentFilterbarValue.toLowerCase() == "false" || this._currentFilterbarValue == "0")
                        this._currentFilterbarValue = false;
                    this._operator = ej.FilterOperators.equal;
                    break;
                default:
                    this._operator = ej.FilterOperators.equal;
            }
        },
        _getOperator: function (_value) {
            if (_value.charAt(0) == "=") {
                this._operator = ej.FilterOperators.equal;
                this._currentFilterbarValue = _value.substring(1);
            }
            if (ej.data.operatorSymbols[_value.charAt(0)] !== undefined || ej.data.operatorSymbols[_value.slice(0, 2)] !== undefined) {
                this._operator = ej.data.operatorSymbols[_value.charAt(0)];
                this._currentFilterbarValue = _value.substring(1);
                if (this._operator === undefined) {
                    this._operator = ej.data.operatorSymbols[_value.slice(0, 2)];
                    this._currentFilterbarValue = _value.substring(2);
                }
            }
            if (this._operator == ej.FilterOperators.lessThan || this._operator == ej.FilterOperators.greaterThan) {
                if (this._currentFilterbarValue.charAt(0) == "=") {
                    this._operator = this._operator + "orequal";
                    this._currentFilterbarValue = this._currentFilterbarValue.substring(1);
                }
            }

        },

        _checkForSkipInput: function () {
            var isSkip = false;
            var skipInput = new Array("=", " ", "!");
            var context = this;
            if (this._currentFilterColumn.type == "number") {
                if (ej.data.operatorSymbols[this._currentFilterbarValue] !== undefined || $.inArray(this._currentFilterbarValue, skipInput) != -1)
                    isSkip = true;
            }
            if (this._currentFilterColumn.type == "string") {
                var stringSkipInput = new Array(">", "<", "=", "!");
                for (var i = 0; i < this._currentFilterbarValue.length; i++) {
                    if (jQuery.inArray(this._currentFilterbarValue[i], stringSkipInput) != -1)
                        isSkip = true;
                }
            }
            return isSkip;
        },
        _showFilterMsg: function () {
            var index = jQuery.inArray(this._currentFilterColumn, this.filterColumnCollection);
            if (this._currentFilterbarValue !== "" && index == -1)
                this.filterColumnCollection.push(this._currentFilterColumn);
            if (this._currentFilterbarValue == "" && index != -1) {
                this.filterColumnCollection.splice(index, 1);
            }
            if ((!this._skipFilterProcess || this.filterColumnCollection.length > 0) && this.filterStatusMsg != "Invalid Filter Data") {
                for (var index = 0; index < this.filterColumnCollection.length; index++) {
                    if (index > 0)
                        this.filterStatusMsg += " && ";
                    this.filterStatusMsg += this.filterColumnCollection[index].headerText + ": " + $("#" + this.filterColumnCollection[index].field + "_filterBarcell").val();
                }
            }

            if (this.model.allowPaging)
                this.getGridPager().ejPager("model.externalMessage", this.filterStatusMsg);
            else {
                this.$pagerStatusBarDiv.find("div").html(this.filterStatusMsg);
                if (this.filterStatusMsg.length)
                    this.$pagerStatusBarDiv.css("display", "block");
                else
                    this.$pagerStatusBarDiv.hide();
            }
            if (this.filterStatusMsg == "Invalid Filter Data") {
                index = jQuery.inArray(this._currentFilterColumn, this.filterColumnCollection);
                this.filterColumnCollection.splice(index, 1);
            }
            this.filterStatusMsg = "";

        },

        formatting: function (formatstring, str) {
            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
            var s = formatstring;
            var frontHtmlidx, FrontHtml, RearHtml, lastidxval;
            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            FrontHtml = frontHtmlidx[0];
            RearHtml = lastidxval[1];
            if (formatstring.indexOf("{0:") != -1) {
                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                var formatVal = toformat.exec(formatstring);
                if (formatVal != null && str != null) {
                    if (FrontHtml != null && RearHtml != null)
                        str = FrontHtml + Globalize.format(str, formatVal[2]) + RearHtml;
                    else
                        str = Globalize.format(str, formatVal[2]);
                } else if (str != null)
                    str = str;
                else
                    str = "";
                return str;
            } else if (this.data != null && this.data.Value == null) {
                $.each(this.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });
                return s;
            } else {
                return this.data.Value;
            }
        },

        /**
        * destroy the grid widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* var gridObj = $("#Grid").data("ejGrid");
		* gridObj.destroy(); // destroy the Grid
		* &lt;/script&gt;
		* @example 
		* &lt;div id="Grid"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* // destroy the Grid
		* $("#Grid").ejGrid("destroy");        
		* &lt;/script&gt;
		* @memberof ejGrid
		* @instance
         */
        _destroy: function () {
            /// <summary>This function is  used to destroy the Grid Object</summary>
            this.element.find(".e-gridheader").find(".e-headercontent,.e-movableheader")
                .add(this.getGridContent().find(".e-content,.e-movablecontent")).unbind('scroll');
            var editForm = $("#" + this._id + "EditForm");
            if (editForm.length) {
                $formEle = editForm.find('.e-field');
                for (var i = 0; i < $formEle.length; i++) {
                    $element = $($formEle[i]);
                    if ($element.hasClass('e-datetimepicker'))
                        $element.ejDateTimePicker("destroy");
                    else if ($element.hasClass('e-datepicker'))
                        $element.ejDatePicker("destroy");
                    else if ($element.hasClass('e-dropdownlist'))
                        $element.ejDropDownList("destroy");
                }
                editForm.remove();
            }
            if (this._confirmDialog)
                this._confirmDialog.ejDialog("destroy");
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "menu") {
                proxy = this;
                $.each(this.model.columns, function (indx, col) {
                    $colType = col.type;
                    $("#" + proxy._id + $colType + "_ddinput_popup_wrapper").remove();
                    if ($colType == "string")
                        $("#" + proxy._id + "_stringDlg").find('.e-autocomplete').ejAutocomplete("destroy");
                    else if ($colType == "date")
                        $("#" + proxy._id + "_dateDlg").find('.e-datepicker').ejDatePicker("destroy");
                    else if ($colType == "number")
                        $("#" + proxy._id + "_numberDlg").find('.e-numerictextbox').ejNumericTextbox("destroy");
                });
            }
            this.element.empty().removeClass("e-grid " + this.model.cssClass);
            this.element.ejWaitingPopup("destroy");
        }
    });

    ej.Grid.localization = {};

    ej.Grid.localization["en-US"] = {
        EmptyRecord: "No records to display",
        GroupDropArea: "Drag a column header here to group its column",
        DeleteOperationAlert: "No records selected for delete operation",
        EditOperationAlert: "No records selected for edit operation",
        SaveButton: "Save",
        OkButton: "OK",
        CancelButton: "Cancel",
        EditFormTitle: "Details of ",
        AddFormTitle: "Add New Record",
        Notactionkeyalert: "This Key-Combination is not avaiable",
        Keyconfigalerttext: "This Key-Combination has already been assigned to ",
        GroupCaptionFormat: "items",
        BulkSaveConifrm: "Are you sure you want to save changes?",
        BulkSaveLostChanges: "Unsaved changes will be lost. Are you sure you want to continue?",
        PagerInfo: "{0} of {1} pages ({2} items)",
        frozenColumnsViewAlert: "Frozen columns should be in grid view area",
        frozenColumnsScrollAlert: "Enable allowScrolling while using frozen Columns",
        frozenNotSupportedException: "Frozen Columns and Rows are not supported for Grouping, Row Template, Detail Template and Batch Editing",
        Add: "Add",
        Edit: "Edit",
        Delete: "Delete",
        Update: "Update",
        Cancel: "Cancel"
    };

    /**
	 * Enum for specifying various filter bar mode in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.filterBarMode = {
        /** Used to set filter bar mode as Immediate mode */
        Immediate: "immediate",
        /** Used to set filter bar mode as OnEnter mode */
        OnEnter: "onenter"
    };

    /**
	 * Enum for specifying various actions types in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.actions = {
        /** Used to specify paging action in grid   */
        Paging: "paging",
        /** Used to specify sorting action in grid   */
        Sorting: "sorting",
        /** Used to specify filtering action in grid   */
        Filtering: "filtering",
        /** Used to specify begin edit action in grid   */
        BeginEdit: "beginedit",
        /** Used to specify saving action in grid   */
        Save: "save",
        /** Used to specify adding action in grid   */
        Add: "add",
        /** Used to specify deleting action in grid   */
        Delete: "delete",
        /** Used to specify cancelling action in grid   */
        Cancel: "cancel",
        /** Used to specify grouping action in grid   */
        Grouping: "grouping",
        /** Used to specify un-grouping action in grid   */
        Ungrouping: "ungrouping",
        /** Used to specify refresh action in grid   */
        Refresh: "refresh",
        /** Used to specify reordering action in grid   */
        Reorder: "reorder",
        /** Used to specify searching action in grid   */
        Search: "searching",
        /** Used to specify bulk save action in grid   */
        BulkSave: "bulksave",
        /** Used to specify virtual scroll action in grid   */
        VirtualScroll: "virtualscroll"
    };

    /**
	 * Enum for specifying various tool bar items in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.toolBarItems = {
        /** Used to add toolbar item for adding records    */
        Add: "add",
        /** Used to add toolbar item for editing records    */
        Edit: "edit",
        /** Used to add toolbar item for deleting records    */
        Delete: "delete",
        /** Used to add toolbar item for updating records    */
        Update: "update",
        /** Used to add toolbar item for cancelling records    */
        Cancel: "cancel",
        /** Used to add toolbar item for searching records    */
        Search: "search"
    };

    /**
	 * Enum for specifying various editing type in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.editingType = {
        /**  Allows to set edit type as string edit type */
        String: "stringedit",
        /**  Allows to set edit type as boolean edit type */
        Boolean: "booleanedit",
        /**  Allows to set edit type as numeric edit type */
        Numeric: "numericedit",
        /**  Allows to set edit type as drop down edit type */
        Dropdown: "dropdownedit",
        /**  Allows to set edit type as date picker edit type */
        DatePicker: "datepicker",
        /**  Allows to set edit type as date time picker edit type */
        DateTimePicker: "datetimepicker",
    };

    /**
	 * Enum for specifying various unbound type in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.unboundType = {
        /** Used to specify unbound type as Edit   */
        Edit: "edit",
        /** Used to specify unbound type as Save   */
        Save: "save",
        /** Used to specify unbound type as Delete   */
        Delete: "delete",
        /** Used to specify unbound type as Cancel   */
        Cancel: "cancel"
    };

    /**
	 * Enum for various selection type in grid 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.selectionType = {
        /**  Support for Single selection only in grid */
        Single: "single",
        /**  Support for multiple selections in grid */
        Multiple: "multiple"
    };

    /**
	 * Enum for specifying edit mode in grid 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.editMode = {
        /**  Creates grid with editMode as Normal */
        Normal: "normal",
        /**  Creates grid with editMode as Dialog */
        Dialog: "dialog",
        /**  Creates grid with editMode as DialogTemplate */
        DialogTemplate: "dialogtemplate",
        /**  Creates grid with editMode as Batch */
        Batch: "batch",
        /**  Creates grid with editMode as ExternalForm */
        ExternalForm: "externalform",
        /**  Creates grid with editMode as ExternalFormTemplate */
        ExternalFormTemplate: "externalformtemplate",
        /**  Creates grid with editMode as InlineForm */
        InlineForm: "inlineform",
        /**  Creates grid with editMode as InlineTemplateForm */
        InlineTemplateForm: "inlineformtemplate"
    };

    /**
	 * Enum for specifying form position in grid 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.formPosition = {
        /**  Creates grid with formPosition as BottomLeft */
        BottomLeft: "bottomleft",
        /**  Creates grid with formPosition as TopRight */
        TopRight: "topright"
    };

    /**
	 * Enum for edit event	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.editEvent = {
        /**  Supports to set edit event as double click for grid  */
        DoubleClick: "doubleclick",
        /**  Supports to set edit event as none for grid */
        None: "none"
    };

    /**
	 * Enum for various grid filtering type	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.filterType = {
        /**  Creates grid with filtering type as Menu */
        Menu: "menu",
        /**  Creates grid with filtering type as FilterBar */
        FilterBar: "filterbar"
    };

    /**
	 * Enum for various grid summary type	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.summaryType = {
        /**  Creates grid with summary type as Average */
        Average: "average",
        /**  Creates grid with summary type as Minimum */
        Minimum: "minimum",
        /**  Creates grid with summary type as Maximum */
        Maximum: "maximum",
        /**  Creates grid with summary type as Count */
        Count: "count",
        /**  Creates grid with summary type as Sum */
        Sum: "sum",
        /**  Creates grid with summary type as TrueCount */
        TrueCount: "truecount",
        /**  Creates grid with summary type as FalseCount */
        FalseCount: "falsecount",
        /**  Creates grid with summary type as Custom */
        Custom: "custom"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin for schedule control
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejSchedule
    * @requires jQuery
    * @requires jsrender
    * @requires ej.web.all
    * @classdesc Design for Schedule control.
    * @example 
    * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create Schedule
    * $('#schedule1').ejSchedule({
    *       fields:{
    *               dataSource: datamanager, //collection of object from datasource
    *               idField: "Id",
    *               subjectField: "Subject",
    *               startTimeField: "StartTime",
    *               endTimeField: "EndTime",
    *               alldayField: "AllDay",
    *               recurrenceField: "Recurrence",
    *               recurrenceRuleField: "RecurrenceRule"
    *           }
    *   }); 	
    * &lt;/script&gt;
    */
    var monthCellsTemplate = function () {
        this._monthCellsTemplate = ("<table class='e-table e-workcellstab' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for cellrows ~cols1=cols1}}" +
        "<tr>{{for ~cols1}}" +
        "<td class='e-monthcells'>" +
        "<div class='e-monthheader'></div><div class='e-appointwrapper'>" +
        "</td>{{/for}}" +
        "</tr>{{/for}}" +
        "</tbody></table>");
        return this._monthCellsTemplate;
    };

    var appointmentTemplate = function () {
        this._appTemplate = ("<div class='{{:appClass}}' id='Appointment_{{:id}}' role='presentation' style='{{if rtl == true}}right{{else}}left{{/if}}: {{:left}}; width: {{:appWidth}}; height: {{:appHeight}}px; top: {{:top}}px;' >"
        + "{{if appClass == 'e-appointment'}}" +
        "<div class='e-tophandle'></div><div class='e-bottomhandle'></div>" +
        "{{else}}<div class='e-lefthandle'></div><div class='e-righthandle'></div>" +
        "{{/if}}{{if userAppTemplId == false}}" +
        "<div class='e-appointinnertext'><div class='e-apptext'>{{:subject}} {{if subject == '' }}{{else}},{{/if}}</div><div class='e-apptime'> {{:startTime}} - {{:endTime}}</div></div>" +
        "{{else}}{{:userTemplate}}{{/if}}</div>");
        return this._appTemplate;
    }

    var headerTemplate = function () {
        this._headerTemplate = (
        "{{for cols ~view=view}}" +
        "{{for ~index=#index}} {{if ~index == '0' }} {{if ~view == 'month'}} <td class='e-monthleftcells'> {{else}} <td class='e-settingsindent'> {{/if}} {{else}} <td class='e-headercells'> {{/if}} {{/for}}" +
        "{{if #index == '0' && ~view !='month'}} {{else}} {{if ~view == 'day' }} <div class='e-dateheadercell e-dayheadercell'> <span class='e-headerdaydisplay'>{{:currentDay}}</span></div> {{else}} <div class='e-dateheadercell'> <span class='e-headerdaydisplay'>{{:currentDay}}</span></div>{{/if}} {{/if}}" +
        "</td>" +
        "{{/for}}<td class='e-emptytd'></td>");
        return this._headerTemplate;
    };

    var alldayTemplate = function () {
        this._alldayTemplate = (
        "{{for cols}}" +
        "{{for ~index=#index}} {{if ~index == '0'}} <td class='e-leftindent'> {{else}} <td class='e-alldaycells'> {{/if}} {{/for}}" +
        "{{if #index == '0'}} <div class='e-schedulesettings e-textellipsis'> </div> {{else}}  {{/if}}" +
        "</td>" +
        "{{/for}}<td class='e-emptyalldaytd'></td>");
        return this._alldayTemplate;
    };

    var workCellsTemplate = function () {
        this._workCellsTemplate = ("<table class='e-table e-workcellstab' height='100%' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for cellrows ~cols1=cols1}}" +
        "<tr>{{for ~cols1 ~index=#index}}" +
        "{{if (~index+1) % 2 == 0 }}<td class='e-workcells'>{{else}}<td class='e-workcells e-alternatecells'>{{/if}}<div class='e-appointwrapper'></div></td>" +
        "{{/for}}" +
        "</tr>" +
        "{{/for}}</tbody></table>");
        return this._workCellsTemplate;
    };

    var timeCellsTemplate = function () {
        this._timeCellsTemplate = ("<div class='e-scrolltimecells'><table class='e-table e-timecelltable' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for timerows ~cols1=cols1 ~view1=viewclass}}" +
        "<tr>{{for ~name=time ~id=id ~view=~view1 }} " +
        "{{if ~view == 0}}<td class='e-timecells'><div class='e-timecelldivs'>{{:~name.slice(0,2)}}</div>" +
        "</td> <td class='e-timecells' rowspan='1'>{{for ~cols1}}{{if (#index+1)%2 ==0}}<div class='e-ampmdisplay'></div>{{else}}<div id='{{:~id}}' class='e-ampmdisplay e-ampmstyle'>{{:~name.slice(3,5)}}</div> {{/if}}{{/for}} </td>{{else}}<td class='e-timecells' style='width:50px;'><div class='e-timecelldivs'></div></td>{{/if}}{{/for}}</tr>{{/for}}</tbody></table></div>");
        return this._timeCellsTemplate;
    };

    ej.widget("ejSchedule", "ej.Schedule", {
        // Root Css Class
        _rootCSS: "e-schedule",
        // widget element will be automatically set in this
        element: null,
        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,

        // default model
        defaults: {
            // time zone collection object to set the list of time zones.
            timezoneCollection: [
            { text: "UTC -12:00", value: "UTC -12:00" },
            { text: "UTC -11:00", value: "UTC -11:00" },
            { text: "UTC -10:00", value: "UTC -10:00" },
            { text: "UTC -09:00", value: "UTC -09:00" },
            { text: "UTC -08:00", value: "UTC -08:00" },
            { text: "UTC -07:00", value: "UTC -07:00" },
            { text: "UTC -06:00", value: "UTC -06:00" },
            { text: "UTC -05:00", value: "UTC -05:00" },
            { text: "UTC -04:30", value: "UTC -04:30" },
            { text: "UTC -04:00", value: "UTC -04:00" },
            { text: "UTC -03:30", value: "UTC -03:30" },
            { text: "UTC -03:00", value: "UTC -03:00" },
            { text: "UTC -02:00", value: "UTC -02:00" },
            { text: "UTC -01:00", value: "UTC -01:00" },
            { text: "UTC +00:00", value: "UTC +00:00" },
            { text: "UTC +01:00", value: "UTC +01:00" },
            { text: "UTC +02:00", value: "UTC +02:00" },
            { text: "UTC +03:00", value: "UTC +03:00" },
            { text: "UTC +03:30", value: "UTC +03:30" },
            { text: "UTC +04:00", value: "UTC +04:00" },
            { text: "UTC +04:30", value: "UTC +04:30" },
            { text: "UTC +05:00", value: "UTC +05:00" },
            { text: "UTC +05:30", value: "UTC +05:30" },
            { text: "UTC +05:45", value: "UTC +05:45" },
            { text: "UTC +06:00", value: "UTC +06:00" },
            { text: "UTC +06:30", value: "UTC +06:30" },
            { text: "UTC +07:00", value: "UTC +07:00" },
            { text: "UTC +08:00", value: "UTC +08:00" },
            { text: "UTC +09:00", value: "UTC +09:00" },
            { text: "UTC +09:30", value: "UTC +09:30" },
            { text: "UTC +10:00", value: "UTC +10:00" },
            { text: "UTC +11:00", value: "UTC +11:00" },
            { text: "UTC +12:00", value: "UTC +12:00" },
            { text: "UTC +13:00", value: "UTC +13:00" },
            ],
            /**		
            * Specifies the custom views of the Schedule.
            * @default [ "Day" , "Week" , "WorkWeek" , "Month" ]
            * @type {Array.string}
            * @example 
            * //To set the views API value during initialization  
            * 	$("#schedule1").ejSchedule({  views: ["Day","Month"]});
            * @example 
            * //Get or set the views API, after initialization:<br>
            *	//Gets the views value  
            *	$("#schedule1").ejSchedule("option", "views");<br>			
            *	//Sets the views value 
            *	$("#schedule1").ejSchedule("option", "views", ["Day","Month"] ); 
            * @memberof ejSchedule
            * @instance
            */
            views: ["Day", "Week", "WorkWeek", "Month"],
            /**		
            * Specifies the view of the Schedule.
            * @default "week"
            * @type {enum}
            * @example 
            * //To set the currentView API value during initialization  
            * 	$("#schedule1").ejSchedule({  currentView: ej.Schedule.CurrentView.Day});
            * @example 
            * //Get or set the currentView API, after initialization:<br>
            *	//Gets the currentView value  
            *	$("#schedule1").ejSchedule("option", "currentView");<br>			
            *	//Sets the currentView value 
            *	$("#schedule1").ejSchedule("option", "currentView", ej.Schedule.CurrentView.Day ); 
            * @memberof ejSchedule
            * @instance
            */
            currentView: "week",
            /**		
            * Specifies the time mode of the Schedule.
            * @default "12"
            * @type {enum}
            * @example 
            * //To set the timeMode API value during initialization  
            * 	$("#schedule1").ejSchedule({  timeMode: ej.Schedule.TimeMode.Hour12});
            * @example 
            * //Get or set the timeMode API, after initialization:<br>
            *	//Gets the timeMode value  
            *	$("#schedule1").ejSchedule("option", "timeMode");<br>			
            *	//Sets the timeMode value 
            *	$("#schedule1").ejSchedule("option", "timeMode", ej.Schedule.TimeMode.Hour12 ); 
            * @memberof ejSchedule
            * @instance
            */
            timeMode: "12",
            /**		
            * Specifies the time zone of the Schedule.
            * @default null
            * @type {string}
            * @example 
            * //To set the timeZone API value during initialization  
            * 	$("#schedule1").ejSchedule({  timeZone: "UTC +2:00"});
            * @example 
            * //Get or set the timeZone API, after initialization:<br>
            *	//Gets the timeZone value  
            *	$("#schedule1").ejSchedule("option", "timeZone");<br>			
            *	//Sets the timeZone value 
            *	$("#schedule1").ejSchedule("option", "timeZone", "UTC +2:00" ); 
            * @memberof ejSchedule
            * @instance
            */
            timeZone: null,
            /**		
            * Specifies the start hour of the Schedule.
            * @default 0
            * @type {number}
            * @example 
            * //To set the startHour API value during initialization  
            * 	$("#schedule1").ejSchedule({  startHour: 9});
            * @example 
            * //Get or set the startHour API, after initialization:<br>
            *	//Gets the startHour value  
            *	$("#schedule1").ejSchedule("option", "startHour");<br>			
            *	//Sets the startHour value 
            *	$("#schedule1").ejSchedule("option", "startHour", 9 ); 
            * @memberof ejSchedule
            * @instance
            */
            startHour: 0,
            /**		
            * Specifies the end hour of the Schedule.
            * @default 24
            * @type {number}
            * @example 
            * //To set the endHour API value during initialization  
            * 	$("#schedule1").ejSchedule({  endHour: 18});
            * @example 
            * //Get or set the endHour API, after initialization:<br>
            *	//Gets the endHour value  
            *	$("#schedule1").ejSchedule("option", "endHour");<br>			
            *	//Sets the endHour value 
            *	$("#schedule1").ejSchedule("option", "endHour", 18 ); 
            * @memberof ejSchedule
            * @instance
            */
            endHour: 24,
            /**		
            * Specifies the highlight business hours of the Schedule.
            * @default true
            * @type {boolean}
            * @example 
            * //To set the highlightBusinessHours API value during initialization  
            * 	$("#schedule1").ejSchedule({  highlightBusinessHours: false});
            * @example 
            * //Get or set the highlightBusinessHours API, after initialization:<br>
            *	//Gets the highlightBusinessHours value  
            *	$("#schedule1").ejSchedule("option", "highlightBusinessHours");<br>			
            *	//Sets the highlightBusinessHours value 
            *	$("#schedule1").ejSchedule("option", "highlightBusinessHours", false ); 
            * @memberof ejSchedule
            * @instance
            */
            highlightBusinessHours: true,
            /**		
            * Enable the quick window of the Schedule.
            * @default true
            * @type {boolean}
            * @example 
            * //To set the showQuickWindow API value during initialization  
            * 	$("#schedule1").ejSchedule({  showQuickWindow: false});
            * @example 
            * //Get or set the showQuickWindow API, after initialization:<br>
            *	//Gets the showQuickWindow value  
            *	$("#schedule1").ejSchedule("option", "showQuickWindow");<br>			
            *	//Sets the showQuickWindow value 
            *	$("#schedule1").ejSchedule("option", "showQuickWindow", false ); 
            * @memberof ejSchedule
            * @instance
            */
            showQuickWindow: true,
            /**		
            * Specifies the business start hour of the Schedule.
            * @default 9
            * @type {number}
            * @example 
            * //To set the businessStartHour API value during initialization  
            * 	$("#schedule1").ejSchedule({  businessStartHour: 7});
            * @example 
            * //Get or set the businessStartHour API, after initialization:<br>
            *	//Gets the businessStartHour value  
            *	$("#schedule1").ejSchedule("option", "businessStartHour");<br>			
            *	//Sets the businessStartHour value 
            *	$("#schedule1").ejSchedule("option", "businessStartHour", 7 ); 
            * @memberof ejSchedule
            * @instance
            */
            businessStartHour: 9,
            /**		
            * Specifies the business end hour of the Schedule.
            * @default 18
            * @type {number}
            * @example 
            * //To set the businessEndHour API value during initialization  
            * 	$("#schedule1").ejSchedule({  businessEndHour: 16});
            * @example 
            * //Get or set the businessEndHour API, after initialization:<br>
            *	//Gets the businessEndHour value  
            *	$("#schedule1").ejSchedule("option", "businessEndHour");<br>			
            *	//Sets the businessEndHour value 
            *	$("#schedule1").ejSchedule("option", "businessEndHour", 16 ); 
            * @memberof ejSchedule
            * @instance
            */
            businessEndHour: 18,
            /**		
            * Specifies the width of the Schedule.
            * @default "800px"
            * @type {string}
            * @example 
            * //To set the width API value during initialization  
            * 	$("#schedule1").ejSchedule({  width: "700px"});
            * @example 
            * //Get or set the width API, after initialization:<br>
            *	//Gets the width value  
            *	$("#schedule1").ejSchedule("option", "width");<br>			
            *	//Sets the width value 
            *	$("#schedule1").ejSchedule("option", "width", "700px" ); 
            * @memberof ejSchedule
            * @instance
            */
            width: "800px",
            /**		
            * Specifies the height of the Schedule.
            * @default "800px"
            * @type {string}
            * @example 
            * //To set the height API value during initialization  
            * 	$("#schedule1").ejSchedule({  height: "700px"});
            * @example 
            * //Get or set the height API, after initialization:<br>
            *	//Gets the height value  
            *	$("#schedule1").ejSchedule("option", "height");<br>			
            *	//Sets the height value 
            *	$("#schedule1").ejSchedule("option", "height", "700px" ); 
            * @memberof ejSchedule
            * @instance
            */
            height: "800px",
            /**		
            * Specifies the current date of the Schedule.
            * @default new Date()
            * @type {date}
            * @example 
            * //To set the currentDate API value during initialization  
            * 	$("#schedule1").ejSchedule({  currentDate: new Date()});
            * @example 
            * //Get or set the currentDate API, after initialization:<br>
            *	//Gets the currentDate value  
            *	$("#schedule1").ejSchedule("option", "currentDate");<br>			
            *	//Sets the currentDate value 
            *	$("#schedule1").ejSchedule("option", "currentDate", new Date() ); 
            * @memberof ejSchedule
            * @instance
            */
            currentDate: new Date(),
            /**		
            * Specify the CSS class to schedule to achieve custom theme.
            * @default ""
            * @type {string}
            * @example 
            * // Set the CSS class during initialization. 			
            * 	$("#schedule1").ejSchedule({  cssClass : "gradient-lime" });
            * @example 
            * //Get or set the css Class after initialization:<br>
            *	// Gets the css Class style of schedule.		
            *	$("#schedule1").ejSchedule("option", "cssClass");			
            *	// Sets the css Class style to schedule
            *	$("#schedule1").ejSchedule("option", "cssClass",  "gradient-lime" );
            * @memberof ejSchedule
            * @instance
            */
            cssClass: "",
            /**		
            * Specifies the localization of the Schedule.
            * @default "en-US"
            * @type {string}
            * @example 
            * //To set the localization API value during initialization  
            * 	$("#schedule1").ejSchedule({  localization: "fr-FR"});
            * @example 
            * //Get or set the localization API, after initialization:<br>
            *	//Gets the localization value  
            *	$("#schedule1").ejSchedule("option", "localization");<br>			
            *	//Sets the localization value 
            *	$("#schedule1").ejSchedule("option", "localization", "fr-FR");
            * @memberof ejSchedule
            * @instance
            */
            localization: "en-US",
            /**		
            * Specifies the resizing option of the Schedule.
            * @default false
            * @type {boolean}
            * @example 
            * //To set the canResize API value during initialization  
            * 	$("#schedule1").ejSchedule({  canResize: true});
            * @example 
            * //Get or set the canResize API, after initialization:<br>
            *	//Gets the canResize value  
            *	$("#schedule1").ejSchedule("option", "canResize");<br>			
            *	//Sets the canResize value 
            *	$("#schedule1").ejSchedule("option", "canResize", true);
            * @memberof ejSchedule
            * @instance
            */
            canResize: false,
            /**		
            * Specifies the rtl of the Schedule.
            * @default false
            * @type {boolean}
            * @example 
            * //To set the rtl API value during initialization  
            * 	$("#schedule1").ejSchedule({  rtl: true});
            * @example 
            * //Get or set the rtl API, after initialization:<br>
            *	//Gets the rtl value  
            *	$("#schedule1").ejSchedule("option", "rtl");<br>			
            *	//Sets the rtl value 
            *	$("#schedule1").ejSchedule("option", "rtl", true);
            * @memberof ejSchedule
            * @instance
            */
            rtl: false,
            /**		
            * Specifies the appointment template of the Schedule.
            * @default null
            * @type {string}
            * @example 
            * //To set the appointmentTemplateId API value during initialization  
            * 	$("#schedule1").ejSchedule({  appointmentTemplateId: "#templatename"});
            * @example 
            * //Get or set the appointmentTemplateId API, after initialization:<br>
            *	//Gets the appointmentTemplateId value  
            *	$("#schedule1").ejSchedule("option", "appointmentTemplateId");<br>			
            *	//Sets the appointmentTemplateId value 
            *	$("#schedule1").ejSchedule("option", "appointmentTemplateId", "#templatename");
            * @memberof ejSchedule
            * @instance
            */
            appointmentTemplateId: null,
            /**		
            * Specifies the appointments drag and drop of the Schedule.
            * @default true
            * @type {boolean}
            * @example 
            * //To set the allowDragDrop API value during initialization  
            * 	$("#schedule1").ejSchedule({  allowDragDrop: false});
            * @example 
            * //Get or set the allowDragDrop API, after initialization:<br>
            *	//Gets the allowDragDrop value  
            *	$("#schedule1").ejSchedule("option", "allowDragDrop");<br>			
            *	//Sets the allowDragDrop value 
            *	$("#schedule1").ejSchedule("option", "allowDragDrop", false);
            * @memberof ejSchedule
            * @instance
            */
            allowDragDrop: true,
            /**		
            * Specifies the appointments resizing of the Schedule.
            * @default true
            * @type {boolean}
            * @example 
            * //To set the allowResizing API value during initialization  
            * 	$("#schedule1").ejSchedule({  allowResizing: true});
            * @example 
            * //Get or set the allowResizing API, after initialization:<br>
            *	//Gets the allowResizing value  
            *	$("#schedule1").ejSchedule("option", "allowResizing");<br>			
            *	//Sets the allowResizing value 
            *	$("#schedule1").ejSchedule("option", "allowResizing", true);
            * @memberof ejSchedule
            * @instance
            */
            allowResizing: true,
            /**		
            * Specifies the current time indicator shown in the Schedule.
            * @default true
            * @type {boolean}
            * @example 
            * //To set the showCurrentTimeIndicator API value during initialization  
            * 	$("#schedule1").ejSchedule({  showCurrentTimeIndicator: false});
            * @example 
            * //Get or set the showCurrentTimeIndicator API, after initialization:<br>
            *	//Gets the showCurrentTimeIndicator value  
            *	$("#schedule1").ejSchedule("option", "showCurrentTimeIndicator");<br>			
            *	//Sets the showCurrentTimeIndicator value 
            *	$("#schedule1").ejSchedule("option", "showCurrentTimeIndicator", false);
            * @memberof ejSchedule
            * @instance
            */
            showCurrentTimeIndicator: true,
            /**		
            * Specifies the keyboard navigation of the Schedule.
            * @default true
            * @type {boolean}
            * @example 
            * //To set the allowKeyboardNavigation API value during initialization  
            * 	$("#schedule1").ejSchedule({  allowKeyboardNavigation: false});
            * @example 
            * //Get or set the allowKeyboardNavigation API, after initialization:<br>
            *	//Gets the allowKeyboardNavigation value  
            *	$("#schedule1").ejSchedule("option", "allowKeyboardNavigation");<br>			
            *	//Sets the allowKeyboardNavigation value 
            *	$("#schedule1").ejSchedule("option", "allowKeyboardNavigation", false);
            * @memberof ejSchedule
            * @instance
            */
            allowKeyboardNavigation: true,
            /**		
            * Specifies the fields of the Schedule.
            * @default object
            * @type {object}
            * @example 
            * //To set the fields API value during initialization  
            * 	$("#schedule1").ejSchedule({  fields: {dataSource:datamanager,idField:"Id",subjectField:"Subject",startTimeField:"StartTime",endTimeField:"EndTime"}});
            * @example 
            * //Get or set the fields API, after initialization:<br>
            *	//Gets the fields value  
            *	$("#schedule1").ejSchedule("option", "fields");<br>			
            *	//Sets the fields value 
            *	$("#schedule1").ejSchedule("option", "fields", {dataSource:datamanager,idField:"Id",subjectField:"Subject",startTimeField:"StartTime",endTimeField:"EndTime"});
            * @memberof ejSchedule
            * @instance
            */
            fields: {
                /**		
                * Specifies the datasource of the Schedule.
                * @default null
                * @type {object}
                * @example 
                * //To set the dataSource API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {dataSource:datamanager}}); //collection of object from datasource. 
                * @example 
                * //Get or set the dataSource API, after initialization:<br>
                *	//Gets the dataSource value  
                *	$("#schedule1").ejSchedule("option", "fields.dataSource");<br>			
                *	//Sets the dataSource value 
                *	$("#schedule1").ejSchedule("option", "fields.dataSource", datamanager});
                * @memberof ejSchedule
                * @instance
                */
                dataSource: null,
                /**		
                * Specifies the query of the Schedule.
                * @default null
                * @type {object}
                * @example 
                * //To set the query API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {query:ej.Query().from("tablename").select("id,Subject,StartTime,EndTime")}});
                * @example 
                * //Get or set the query API, after initialization:<br>
                *	//Gets the query value  
                *	$("#schedule1").ejSchedule("option", "fields.query");<br>			
                *	//Sets the query value 
                *	$("#schedule1").ejSchedule("option", "fields.query", ej.Query().from("tablename").select("id,Subject,StartTime,EndTime")});
                * @memberof ejSchedule
                * @instance
                */
                query: null,
                /**		
                * Specifies the dataSource table name of the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the tableName API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {tableName:"tablename"}});
                * @example 
                * //Get or set the tableName API, after initialization:<br>
                *	//Gets the tableName value  
                *	$("#schedule1").ejSchedule("option", "fields.tableName");<br>			
                *	//Sets the tableName value 
                *	$("#schedule1").ejSchedule("option", "fields.tableName", "tablename"});
                * @memberof ejSchedule
                * @instance
                */
                tableName: null,
                /**		
                * Specifies the id field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the idField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {idField:"Id"}});
                * @example 
                * //Get or set the idField API, after initialization:<br>
                *	//Gets the idField value  
                *	$("#schedule1").ejSchedule("option", "fields.idField");<br>			
                *	//Sets the idField value 
                *	$("#schedule1").ejSchedule("option", "fields.idField", "Id"});
                * @memberof ejSchedule
                * @instance
                */
                idField: null,
                /**		
                * Specifies the subject field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the subjectField API value during initialization  
                * 	$("#schedule1").ejSchedule({ fields: {subjectField:"Subject"}});
                * @example 
                * //Get or set the subjectField API, after initialization:<br>
                *	//Gets the subjectField value  
                *	$("#schedule1").ejSchedule("option", "fields.subjectField");<br>			
                *	//Sets the subjectField value 
                *	$("#schedule1").ejSchedule("option", "fields.subjectField", "Subject"});
                * @memberof ejSchedule
                * @instance
                */
                subjectField: null,
                /**		
                * Specifies the description field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the descriptionField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {descriptionField:"Description"}});
                * @example 
                * //Get or set the descriptionField API, after initialization:<br>
                *	//Gets the descriptionField value  
                *	$("#schedule1").ejSchedule("option", "fields.descriptionField");<br>			
                *	//Sets the descriptionField value 
                *	$("#schedule1").ejSchedule("option", "fields.descriptionField", "Description"});
                * @memberof ejSchedule
                * @instance
                */
                descriptionField: null,
                /**		
                * Specifies the start time field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the startTimeField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {startTimeField:"StartTime"}});
                * @example 
                * //Get or set the startTimeField API, after initialization:<br>
                *	//Gets the startTimeField value  
                *	$("#schedule1").ejSchedule("option", "fields.startTimeField");<br>			
                *	//Sets the startTimeField value 
                *	$("#schedule1").ejSchedule("option", "fields.startTimeField", "StartTime"});
                * @memberof ejSchedule
                * @instance
                */
                startTimeField: null,
                /**		
                * Specifies the end time field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the endTimeField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {endTimeField:"EndTime"}});
                * @example 
                * //Get or set the endTimeField API, after initialization:<br>
                *	//Gets the endTimeField value  
                *	$("#schedule1").ejSchedule("option", "fields.endTimeField");<br>			
                *	//Sets the endTimeField value 
                *	$("#schedule1").ejSchedule("option", "fields.endTimeField", "EndTime"});
                * @memberof ejSchedule
                * @instance
                */
                endTimeField: null,
                /**		
                * Specifies the recurrence field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the recurrenceField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {recurrenceField:"Recurrence"}});
                * @example 
                * //Get or set the recurrenceField API, after initialization:<br>
                *	//Gets the recurrenceField value  
                *	$("#schedule1").ejSchedule("option", "fields.recurrenceField");<br>			
                *	//Sets the recurrenceField value 
                *	$("#schedule1").ejSchedule("option", "fields.recurrenceField", "Recurrence"});
                * @memberof ejSchedule
                * @instance
                */
                recurrenceField: null,
                /**		
                * Specifies the recurrencerule field from datsource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the recurrenceRuleField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {recurrenceRuleField:"RecurrenceRule"}});
                * @example 
                * //Get or set the recurrenceRuleField API, after initialization:<br>
                *	//Gets the recurrenceRuleField value  
                *	$("#schedule1").ejSchedule("option", "fields.recurrenceRuleField");<br>			
                *	//Sets the recurrenceRuleField value 
                *	$("#schedule1").ejSchedule("option", "fields.recurrenceRuleField", "RecurrenceRule"});
                * @memberof ejSchedule
                * @instance
                */
                recurrenceRuleField: null,
                /**		
                * Specifies the reminder field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the reminderField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {reminderField:"Reminder"}});
                * @example 
                * //Get or set the reminderField API, after initialization:<br>
                *	//Gets the reminderField value  
                *	$("#schedule1").ejSchedule("option", "fields.reminderField");<br>			
                *	//Sets the reminderField value 
                *	$("#schedule1").ejSchedule("option", "fields.reminderField", "Reminder"});
                * @memberof ejSchedule
                * @instance
                */
                reminderField: null,
                /**		
                * Specifies the category field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the categorizeField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {categorizeField:"Categorize"}});
                * @example 
                * //Get or set the categorizeField API, after initialization:<br>
                *	//Gets the categorizeField value  
                *	$("#schedule1").ejSchedule("option", "fields.categorizeField");<br>			
                *	//Sets the categorizeField value 
                *	$("#schedule1").ejSchedule("option", "fields.categorizeField", "Categorize"});
                * @memberof ejSchedule
                * @instance
                */
                categorizeField: null,
                /**		
                * Specifies the allday field from datasource for the Schedule.
                * @default null
                * @type {string}
                * @example 
                * //To set the alldayField API value during initialization  
                * 	$("#schedule1").ejSchedule({  fields: {alldayField:"AllDay"}});
                * @example 
                * //Get or set the alldayField API, after initialization:<br>
                *	//Gets the alldayField value  
                *	$("#schedule1").ejSchedule("option", "fields.alldayField");<br>			
                *	//Sets the alldayField value 
                *	$("#schedule1").ejSchedule("option", "fields.alldayField", "AllDay"});
                * @memberof ejSchedule
                * @instance
                */
                alldayField: null
            },
            /**     
            * Fires after click on any cell in the schedule.
            * @event
            * @name ejSchedule#cellClick		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @param {Object}  argument.starttime return the selected cell start date and time
            * @param {Object}  argument.endtime return the selected cell end date and time
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    cellClick: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            cellClick: null,
            /**     
            * Fires after click on any appointment in the schedule.
            * @event
            * @name ejSchedule#appointmentClick		
            * @param {Object} argument.e Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the Schedule model
            * @param {string}  argument.type returns the name of the event
            * @param {Object}  argument.appObj returns the appointment object
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    appointmentClick: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            appointmentClick: null,
            /**     
            * Fires after double click on any cell in the schedule.
            * @event
            * @name ejSchedule#cellDoubleClick		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @param {Object}  argument.starttime return the selected cell start date and time
            * @param {Object}  argument.endtime return the selected cell end date and time
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    cellDoubleClick: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            cellDoubleClick: null,
            /**     
            * Fires while the appointment window open  in the schedule.
            * @event
            * @name ejSchedule#appointmentWindowOpen		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @param {Object}  argument.starttime returns the selected cell start date and time
            * @param {Object}  argument.endtime returns the selected cell end date and time
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    appointmentWindowOpen: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            appointmentWindowOpen: null,
            /**     
            * Fires after new appointment saved in the schedule.
            * @event
            * @name ejSchedule#appointmentSaved		
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @param {Object}  argument.appObj return the saved appointment object.
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    appointmentSaved: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            appointmentSaved: null,
            /**     
            * Fires after edit the appointment in the schedule.
            * @event
            * @name ejSchedule#appointmentEdited		
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @param {Object}  argument.appObj return the edited appointment object.
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    appointmentEdited: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            appointmentEdited: null,
            /**     
            * Fires after delete the appointment in the schedule.
            * @event
            * @name ejSchedule#appointmentDeleted		
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @param {Object}  argument.appObj return the deleted appointment object.
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    appointmentDeleted: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            appointmentDeleted: null,
            /**     
            * Fires after navigate any view or date in the schedule.
            * @event
            * @name ejSchedule#navigation		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    navigation: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            navigation: null,
            /**     
            * Fires while start dragging the appointment in the schedule.
            * @event
            * @name ejSchedule#dragStart		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    dragStart: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            dragStart: null,
            /**     
            * Fires while drag over the appointment in the schedule.
            * @event
            * @name ejSchedule#dragOver		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    dragOver: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            dragOver: null,
            /**     
            * Fires after drop the appointment in the schedule.
            * @event
            * @name ejSchedule#dragDrop		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    dragDrop: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            dragDrop: null,
            /**     
            * Fires when start resize the appointment in the schedule.
            * @event
            * @name ejSchedule#resizeStart		
            * @param {Object} argument Event parameters from schedule
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    resizeStart: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            resizeStart: null,
            /**     
            * Fires while we resizing the appointment in the schedule.
            * @event
            * @name ejSchedule#resizing		
            * @param {Object} argument Event parameters from schedule
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    resizing: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            resizing: null,
            /**     
            * Fires after stop the resizing of appointment in the schedule.
            * @event
            * @name ejSchedule#resizeStop		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    resizeStop: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            resizeStop: null,
            /**     
            * Fires when the action begins in the schedule.
            * @event
            * @name ejSchedule#actionBegin		
            * @param {Object} argument Event parameters from schedule    
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    actionBegin: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            actionBegin: null,
            /**     
            * Fires after complete the action in the schedule.
            * @event
            * @name ejSchedule#actionComplete		
            * @param {Object} argument Event parameters from schedule     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the schedule model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //create event for schedule
            * $("#schedule1").ejSchedule({
            *    actionComplete: function (args) {}
            * });
            * @memberof ejSchedule
            * @instance
            */
            actionComplete: null
        },
        observables: ["fields.dataSource", "currentView", "currentDate"],
        currentView: ej.util.valueFunction("currentView"),
        currentDate: ej.util.valueFunction("currentDate"),
        dataSource: ej.util.valueFunction("fields.dataSource"),

        // constructor function
        /**
        * Create the schedule widget
        * @private
        */
        _init: function () {
            this._trigger("load");
            this.element.addClass("e-schedule"); //While resize the window the root class has been removed. 
            this._initPrivateProperties();
            this._renderInitSchedule();
            this._wireEvents();
            this._bindAppointmentsData();
            !ej.isNullOrUndefined(this.model.fields) && (this._dataManager = this.dataSource() instanceof ej.DataManager ? this.dataSource() : this.dataSource() != null ? ej.DataManager(this.dataSource()) : null);
            this.model.rtl && this.element.addClass("e-rtl");
        },

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var flag = false;
            for (var prop in options) {
                switch (prop) {
                    case "views":
                        this.hTD.find("table.e-viewstable").remove();
                        this.hTD.find("div.e-viewsdiv").append(this._renderViewNavigation(options[prop]));
                        break;
                    case "currentView":
                        this._navView = true;
                        this._viewChange(null, null, ej.util.getVal(options[prop]));
                        break;
                    case "currentDate":
                        this._setCurrentDate = true;
                        this._navigateClick(null);
                        break;
                    case "timeMode":
                        this.model.timeMode = options[prop];
                        if (this.currentView() !== "month") {
                            var viewclass = this.currentView() == "month" ? 1 : 0; //"e-monthtimecells" : "e-timecelldivs";
                            var timeCellhtml = this.timeCellTemplate.render({ timerows: this._getStartEndTime(), cols1: [1, 2], viewclass: viewclass });
                            this.$TimeCellTD.children().remove();
                            this.$TimeCellTD.append(timeCellhtml); if (this.model.showCurrentTimeIndicator) $("#" + this._id + "_HighlightCurrentTimeline").remove();
                            this.element.find(".e-scrolltimecells").css("top", -(this.element.find(".e-draggableworkarea").scrollTop()) + "px");
                            this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                            this._renderAppointmentAll();
                        }
                        break;
                    case "timeZone":
                        this.model.timeZone = options[prop];
                        this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").remove();
                        this._bindAppointmentsData();
                        break;
                    case "startHour":
                        this.model.startHour = options[prop];
                        this._setNewStartEndHours();
                        break;
                    case "endHour":
                        this.model.endHour = options[prop];
                        this._setNewStartEndHours();
                        break;
                    case "highlightBusinessHours":
                        this.model.highlightBusinessHours = options[prop];
                        (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") && this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                        break;
                    case "businessStartHour":
                        this.model.businessStartHour = options[prop];
                        this.currentView() !== "month" ? this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") : this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                        this._highlightBusinessHours();
                        break;
                    case "businessEndHour":
                        this.model.businessEndHour = options[prop];
                        this.currentView() !== "month" ? this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") : this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                        this._highlightBusinessHours();
                        break;
                    case "height":
                        this.model.height = options[prop];
                        if (this.model.showCurrentTimeIndicator) $("#" + this._id + "_HighlightCurrentTimeline").remove();
                        flag = true;
                        break;
                    case "width":
                        this.model.width = options[prop];
                        if (this.model.showCurrentTimeIndicator) $("#" + this._id + "_HighlightCurrentTimeline").remove();
                        flag = true;
                        break;
                    case "cssClass":
                        this.element.removeClass(this.model.cssClass).addClass(options[prop]);
                        break;
                    case "localization":
                        this.model.localization = options[prop];
                        var model = this.model;
                        this.element.ejSchedule("destroy").ejSchedule(model);
                        break;
                    case "canResize":
                        this.model.canResize = options[prop];
                        if (this.model.canResize) {
                            !this._isDevice() ? this._on($(window), "resize", this._scheduleResize) : this._on($(window), "orientationchange", this._scheduleResize);
                            this._wireResizeEvents();
                        }
                        break;
                    case "showCurrentTimeIndicator":
                        this.model.showCurrentTimeIndicator = options[prop];
                        this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                        if (!this.model.showCurrentTimeIndicator) this.currentView() !== "month" ? $("#" + this._id + "_HighlightCurrentTimeline").remove() : $(this.element.find(".e-monthcells")).removeClass("e-monthcurrenttime");
                        break;
                    case "rtl":
                        options[prop] ? this.element.addClass("e-rtl") : this.element.removeClass("e-rtl");
                        this.model.rtl = options[prop];
                        var model = this.model;
                        this.element.ejSchedule("destroy").ejSchedule(model);
                        break;
                    case "appointmentTemplateId":
                        this.model.appointmentTemplateId = options[prop];
                        this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").remove();
                        this._renderAppointmentAll();
                        break;
                    case "allowKeyboardNavigation":
                        this.model.allowKeyboardNavigation = options[prop];
                        if (this.model.allowKeyboardNavigation) {
                            this._on(this.element, "keydown", $.proxy(this._onKeyDown, this));
                            this._on($(document), "keydown", $.proxy(this._documentOnKeyDown, this));
                        }
                        else {
                            this._off(this.element, "keydown");
                            this._off($(document), "keydown");
                        }
                        break;
                    case "fields":
                        $.extend(this.model.fields, options[prop]);
                        this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").remove();
                        this._bindAppointmentsData();
                        break;
                }
            }
            if (flag) {
                this.element.empty();
                this._renderInitSchedule();
                this._renderAppointmentAll();
            }
        },

        //Touch Schedule:
        /**
        * To configure the touch for schedule		
        * @private
        */
        _touchSchedule: function (e) {
            var event;
            switch (e.type) {
                case "swipeleft":
                    event = { currentTarget: this.element.find('.e-navigatenext') };
                    this._navigateClick(event);
                    break;
                case "swiperight":
                    event = { currentTarget: this.element.find('.e-navigateprevious') };
                    this._navigateClick(event);
                    break;
            }
        },

        /**
        * To configure the private properties of schedule		
        * @private
        */
        _initPrivateProperties: function () {
            this._reminderCount = 0;
            this._appId = 0;
            this._parentId = 0;
            this._currentAction = "";
            this._reminderClose = false;
            this._processed = [];
            this._dateRender = [];
            this._rRule = {};
            this._dayNames = ["SU", "MO", "TU", "WE", "TH", "FR", "SA"];
            var culture = Globalize.culture(this.model.localization);
            this._dayShortNames = culture.calendar.days.namesAbbr;
            this._dayFullNames = culture.calendar.days.names;
            this._monthNames = culture.calendar.months.names;
            this._pattern = culture.calendar.patterns;
            this._navView = false;
            this._editKey = false;
            this._cellClick = false;
            this._recurChecked = false;
            this._currentRowIndex = 0;
            this._currentColIndex = 0;
            this._setCurrentDate = false;
            this._browserInfo = ej.browserInfo();
            this._leftResizing = false;
            this._rightResizing = false;
            this._topResizing = false;
            this._bottomResizing = false;
        },
        /**
        * destroy the schedule widget
        * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
        * @alias destroy
        * @return jQuery
        * @example 
        * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create Schedule
        * var schObj = $("#schedule1").data("ejSchedule");
        * schObj.destroy(); // destroy the schedule
        * &lt;/script&gt;
        * @example 
        * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // enable the schedule
        * $("#schedule1").ejSchedule("destroy");	
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        _destroy: function () {
            //Appointment window and Edit window controls destory
            this._destorySubControls($("#" + this._id + "_AddEditForm"));
            this._destorySubControls($("#" + this._id + "_RecurForm"));
            this.element.find("#" + this._id + "_navigator").ejDatePicker("destroy");
            //Clear  set interval
            window.clearInterval(this._intervalID);
            //Remove dialogs
            $("#" + this._id + "AppointmentAddEditWindow_wrapper,#" + this._id + "AppointmentQuickWindow_wrapper,#" + this._id + "AppDetailsWindow_wrapper").remove();
            //Remove templates
            $("#" + this._id + "_WorkCells_Template,#" + this._id + "_MonthCells_Template,#" + this._id + "_TimeCells_Template,#" + this._id + "_HeaderDate_Template,#" + this._id + "_Appointment_Template,#" + this._id + "_AlldayCells_Template").remove();
            this.element.empty().removeClass("e-schedule");
        },

        /**
        * Destroy subcontrols.
        * @private
        */
        _destorySubControls: function (formelement) {
            $formEle = formelement.find('.e-timepicker,.e-datepicker,.e-dropdownlist');
            for (var index = 0; index < $formEle.length; index++) {
                $element = $($formEle[index]);
                if ($element.hasClass('e-timepicker'))
                    $element.ejTimePicker("destroy");
                else if ($element.hasClass('e-datepicker'))
                    $element.ejDatePicker("destroy");
                else if ($element.hasClass('e-dropdownlist'))
                    $element.ejDropDownList("destroy");
            }
        },

        /**
        * Wiring the events to schedule control		
        * @private
        */
        _wireEvents: function () {
            this._on((this.element, $(window)), "mousedown", $.proxy(this._onScheduleMouseDown, this));
            this._on(this.element, "click", "div.e-navigateprevious,div.e-navigatenext", $.proxy(this._navigateClick, this));
            this._on(this.element, "click", ".e-commonviewbutton", $.proxy(this._viewChange, this));
            this._on(this.element, "click", ".e-datedisplay", $.proxy(this._viewCalendar, this));
            this._on(this.element, "click", "div.e-appsoverflow", $.proxy(this._appOverflowClick, this));
            //Work Cell events
            this._on(this.element, "click", ".e-headercells,.e-timecells,td.e-monthcells div.e-monthheader", $.proxy(this._navigateView, this));
            this._on(this.element, "mouseover", "td.e-workcells,td.e-monthcells", $.proxy(this._onCellMouseOver, this));
            this._on(this.element, "mouseout", "td.e-workcells,td.e-monthcells", $.proxy(this._onCellMouseOver, this));
            this._on(this.element, $.isFunction($.fn.tap) ? "tap" : "click", ".e-workcells,.e-alldaycells,.e-alldayappointment,.e-monthcells,div.e-monthappointment", $.proxy(this._cellSelection, this));
            // Appointments events
            this._on(this.element, "dblclick", ".e-workcells,.e-alldaycells,.e-alldayappointment,div.e-monthappointment,.e-monthcells", $.proxy(this._appointmentWindow, this));
            this._on(this.element, "mouseover", ".e-appointment,.e-alldayappointment,.e-monthappointment", $.proxy(this._onAppointMouseOver, this));
            this._on(this.element, "mouseout", ".e-appointment,.e-alldayappointment,.e-monthappointment", $.proxy(this._onAppointMouseOver, this));
            // Keydown events
            if (this.model.allowKeyboardNavigation) {
                this._on(this.element, "keydown", $.proxy(this._onKeyDown, this));
                this._on($(document), "keydown", $.proxy(this._documentOnKeyDown, this));
            }
            //Touch event.
            this._on(this.element, "swipeleft swiperight", ".e-contentarearow .e-workcellstab", $.proxy(this._touchSchedule, this));
            // Rezize events
            if (this.model.canResize)
                !this._isDevice() ? this._on($(window), "resize", this._scheduleResize) : this._on($(window), "orientationchange", this._scheduleResize);
        },
        /**
        * Section For handling mouse down action on the schedule control.
        * @private
        */
        _onScheduleMouseDown: function (e) {
            var _targ = $(e.target);
            var appointdiv = _targ.hasClass("e-appointment,e-alldayappointment,e-monthappointment") ? _targ : _targ.parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
            if (!_targ.hasClass("e-workcells") && !appointdiv.hasClass("e-appointment") && !appointdiv.hasClass("e-alldayappointment") && !appointdiv.hasClass("e-monthappointment") && !_targ.hasClass("e-alldaycells") && !_targ.hasClass("e-monthcells") && !(_targ).parents(".e-scheduledialog").hasClass("e-scheduledialog")) {
                this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") || this._quickAppDetailsWindow.ejDialog("close"));
                this.element.find(".e-workcells").removeClass("e-mousehover");
            }
        },
        /**
        * Section For handle mouse hover action in the schedule cells.
        * @private
        */
        _onCellMouseOver: function (e) {
            var element = $(e.currentTarget);
            if (e.type === "mouseover") {
                this.element.find(".e-mousehover").removeClass("e-mousehover");
                if (element.hasClass("e-workcells") || element.hasClass("e-monthcells"))
                    element.addClass("e-mousehover");
            }
            else if (e.type === "mouseout") {
                if (element.hasClass("e-workcells") || element.hasClass("e-monthcells"))
                    element.removeClass("e-mousehover");
            }
        },
        /**
        * Section For handle mouse hover action in the schedule appointments.
        * @private
        */
        _onAppointMouseOver: function (e) {
            if (this.model.allowResizing) {
                var appointdiv = $(e.target).hasClass("e-appointment,e-alldayappointment,e-monthappointment") ? $(e.target) : $(e.target).parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
                if (appointdiv.hasClass("e-appointment")) {
                    if (e.type === "mouseover") {
                        appointdiv.find(".e-bottomhandle,.e-tophandle").addClass("e-icon e-nsexpand");
                        appointdiv.find(".e-bottomhandle").addClass("e-appointresizer-bottom");
                        appointdiv.find(".e-tophandle").addClass("e-appointresizer-top");
                    }
                    else if (e.type === "mouseout") {
                        appointdiv.find(".e-bottomhandle,.e-tophandle").removeClass("e-icon e-nsexpand");
                    }

                }
                else {
                    if (e.type === "mouseover") {
                        appointdiv.find(".e-lefthandle,.e-righthandle").addClass("e-icon e-ewexpand");
                        appointdiv.find(".e-lefthandle").addClass("e-appointresizer-left");
                        appointdiv.find(".e-righthandle").addClass("e-appointresizer-right");
                    }
                    else if (e.type === "mouseout") {
                        appointdiv.find(".e-lefthandle,.e-righthandle").removeClass("e-icon e-ewexpand");
                    }
                }
            }
        },
        /**
        * Section For handle cell selection action of the schedule.
        * @private
        */
        _cellSelection: function (e) {
            this.element.find("e.workcells,e.alldaycells,.e-monthcells").removeClass("e-mousehover");
            this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
            var localizeDays = Globalize.culture(this.model.localization);
            var _target = !ej.isNullOrUndefined(e.target) ? $(e.target) : $(e);
            if (_target.hasClass("e-workcells") || _target.hasClass("e-alldaycells") || _target.hasClass("e-monthcells")) {
                this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
                var index = _target.hasClass("e-workcells") ? _target.index() : _target.hasClass("e-alldaycells") ? _target.index() - 1 : (_target.parent().index() * 7) + _target.index();
                var curDate = new Date(this._dateRender[index]);
                if (_target.hasClass("e-workcells")) {
                    var time = this.model.startHour + (_target.parent().index() / 2);
                    var timemin = time.toString().split(".");
                    this.cur_StartTime = new Date(curDate).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                    this.cur_EndTime = new Date(new Date(this.cur_StartTime).setMinutes(new Date(this.cur_StartTime).getMinutes() + 30));
                    this._appointmentAddWindow.find(".allday").ejCheckBox({ check: false });
                }
                else {
                    this.cur_StartTime = new Date(new Date(curDate).setHours(0, 0));
                    this.cur_EndTime = new Date(new Date(curDate).setHours(23, 59));
                    this._appointmentAddWindow.find(".allday").ejCheckBox({ check: true });
                }
                if (this.model.showQuickWindow) {
                    this._quickAppointWindow.find("#" + this._id + "_quickAppwindow").css("display", "block");
                    //add the dialog code to display while clicking on the cells
                    this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(curDate).getDay()] + ", " + Globalize.format(new Date(curDate), "MMMM") + " " + new Date(curDate).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), "hh.mm tt") + " - " + Globalize.format(new Date(this.cur_EndTime), "hh.mm tt"));
                    var topElmt = $(e).hasClass("e-workcells") || $(e).hasClass("e-alldaycells") || $(e).hasClass("e-monthcells") ? $(e) : $(e.target);
                    this._quickAppointWindow.ejDialog("open");
                    this._quickAppointWindow.find('.subject').val(""); var x = this.currentView() === "day" ? this.cellwidth / 2 - this.element.find(".e-leftindent").width() : topElmt.offset().left - this.element.find(".e-scrolltimecells").width(); var y = topElmt.offset().top - this._quickAppointWindow.height() - 10;
                    this._quickAppointWindow.ejDialog({ position: { X: x, Y: y } });
                }
                var event = { starttime: new Date(this.cur_StartTime), endtime: this.cur_EndTime, target: e };
                if (this._trigger("cellClick", event)) return false;
                _target.addClass("e-selectedCell").attr("tabIndex", 0).focus();
                this.model.showQuickWindow && this._quickAppointWindow.find('.subject').focus();
                this._cellClick = true;
                this._appointmentAddWindow.ejDialog({ title: this._getLocalizedLabels("CreateAppointmentTitle") });
            }
            else {
                this._cellClick = false;
                this._quickAppDetailsWindow && this._quickAppDetailsWindow.find("" + "#" + this._id + "_quickAppDetailswindow").css("display", "block");
                this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close");
                var appoint = _target.hasClass("e-appointment") || _target.hasClass("e-monthappointment") || _target.hasClass("e-alldayappointment") ? _target : _target.parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
                if (appoint.length <= 0) return false;
                appoint.attr("tabIndex", 0).focus();
                appoint.addClass("e-selectedAppointment");

                this._appId = parseInt(appoint[0].id.split("_")[1]);
                var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, this._appId);
                var templist = new ej.DataManager(this._processed).executeLocal(query);

                this._quickAppDetailsWindow && this._quickAppDetailsWindow.find('.e-quicksubject').html(templist[0][this.model.fields["subjectField"]]);
                var temp = $.extend({}, templist[0]);
                if (this.model.timeZone != null) {
                    var timezone = this.model.timeZone.split(" ")[1].split(":");
                    var hour = parseInt(timezone[0]);
                    var mins = parseInt(timezone[1]);
                    dateMinutes = new Date().getTimezoneOffset();
                    var obj = this._minToHour(dateMinutes + ((hour * 60) + mins));
                    var newStartDate = new Date(temp.StartTime); var newEndDate = new Date(temp.EndTime);
                    temp.StartTime = new Date(newStartDate.setHours(newStartDate.getHours() + obj.h, newStartDate.getMinutes() + obj.m));
                    temp.EndTime = new Date(newEndDate.setHours(newEndDate.getHours() + obj.h, newEndDate.getMinutes() + obj.m));
                }
                var startTimeDisp = this.model.timeMode == "12" ? Globalize.format(new Date(temp[this.model.fields["startTimeField"]]), this._pattern.t) : Globalize.format(new Date(temp[this.model.fields["startTimeField"]]), "HH:mm");
                var endTimeDisp = this.model.timeMode == "12" ? Globalize.format(new Date(temp[this.model.fields["endTimeField"]]), this._pattern.t) : Globalize.format(new Date(temp[this.model.fields["endTimeField"]]), "HH:mm");

                var diffTime = new Date().getTime() - (new Date(temp[this.model.fields["startTimeField"]]).getTime());
                var diffMinute = 60 * 1000;
                var diffHour = 60 * diffMinute;
                var diffDays = 24 * diffHour;

                var result = Math.abs(new Date() - new Date(temp[this.model.fields["startTimeField"]]));
                var h = Math.floor(result / 1000 / 60 / 60); h = (h <= 9 ? '0' + h : h);
                var m = new Date(result).getUTCMinutes(); m = (m <= 9 ? '0' + m : m);

                var dueIn = Math.ceil(diffTime / diffMinute - 1) < 0 && h < 24 ? this._getLocalizedLabels("EventBeginsin") + " " + h + " " + this._getLocalizedLabels("Hours") + " " + m + " " + this._getLocalizedLabels("Minutes") :
                Math.ceil(diffTime / diffMinute - 1) < 0 && h >= 24 ? this._getLocalizedLabels("EventBeginsin") + " " + Math.abs(Math.ceil(diffTime / diffDays)) + " " + this._getLocalizedLabels("Days") : this._getLocalizedLabels("Overdue");
                if (this.model.showQuickWindow) {
                    this._quickAppDetailsWindow.find('.e-rightfields').html(dueIn);
                    this._quickAppDetailsWindow.find('.e-quickstartendtime').html(localizeDays.calendar.days.names[new Date(temp[this.model.fields["startTimeField"]]).getDay()] + ", " + Globalize.format(new Date(temp[this.model.fields["startTimeField"]]), "MMMM") + " " + new Date(temp[this.model.fields["startTimeField"]]).getDate() + ", " + startTimeDisp + " - " + endTimeDisp);

                    if (temp[this.model.fields["recurrenceField"]] == 1)
                        this._quickAppDetailsWindow.find('.e-editseries').removeClass("e-disable").on('click', $.proxy(this._editClick, this));

                    else
                        this._quickAppDetailsWindow.find('.e-editseries').addClass("e-disable").off('click');
                    this._quickAppDetailsWindow.ejDialog("open");
                    var left = this.currentView() === "day" ? $(appoint).width() / 2 : $(appoint).offset().left - this.element.find(".e-scrolltimecells").width();
                    var top = this.currentView() === "month" ? $(appoint).offset().top - this._quickAppDetailsWindow.height() - 10 : $(appoint).parent().offset().top - this._quickAppDetailsWindow.height() - 10;
                    this._quickAppDetailsWindow.ejDialog({ position: { X: left, Y: top } });
                    this._quickAppDetailsWindow.focus();
                }
                var data = templist[0];
                if (this._trigger("appointmentClick", data))
                    return false;
                this._appointmentAddWindow.ejDialog({ title: this._getLocalizedLabels("Editevent") });
            }
            if (!$(_target).hasClass("e-monthcells"))
                this.element.find(".e-scrolltimecells").css("top", (-this.element.find('.e-draggableworkarea').scrollTop()) + "px");
        },
        /**
        * Section For handle current time indicator shown in the schedule.
        * @private
        */
        _highlightCurrentTime: function (_showIndicator) {
            if (_showIndicator) {
                var flag = false, index = 0;
                for (var i = 0; i < this._dateRender.length; i++) {
                    if (Globalize.format(new Date(this._dateRender[i]), this._pattern.d) === Globalize.format(new Date(), this._pattern.d)) {
                        flag = true;
                        index = i;
                    }
                }
                if (this.currentView() !== "month") {
                    if (flag) {
                        var temp = this;
                        var view = this.currentView() === "week" ? 7 : this.currentView() === "workweek" ? 5 : 1;
                        var timeWidth = this.element.find('.e-draggableworkarea')[0].offsetWidth / view;
                        if (new Date().getHours() >= temp.model.startHour && new Date().getHours() <= temp.model.endHour) {
                            this.element.find('.e-draggableworkarea').append('<div id="' + this._id + '_HighlightCurrentTimeline" class="e-highlightcurrenttimeline"></div>');
                            $('#' + this._id + '_HighlightCurrentTimeline').append('<div class="e-currenttime"></div>').append(ej.buildTag("div.e-icon e-timeidicatorcircle e-lefttimeindicator")).append(ej.buildTag("div.e-icon e-timeidicatorcircle e-righttimeindicator"));
                            $('#' + this._id + '_HighlightCurrentTimeline').css("width", timeWidth + "px").css("left", (index * timeWidth) + "px");
                            $('#' + this._id + '_HighlightCurrentTimeline').find('.e-righttimeindicator').css("margin-left", (timeWidth - 3) + "px");
                        }
                        function ChangeCurrentTimePosition() {
                            var now = new Date();
                            now.setMinutes(now.getMinutes());
                            var position;
                            if (now.getHours() >= temp.model.startHour && now.getHours() <= temp.model.endHour) {
                                var topVal = ((((now.getHours() - temp.model.startHour) * 60) + now.getMinutes()) * (temp.element.find(".e-workcells").height() + 1) * 2) / 60;
                                $('#' + temp._id + '_HighlightCurrentTimeline').css("top", topVal - 1);
                                var currentTime = temp.model.timeMode == "12" ? Globalize.format(new Date(), "hh:mm tt") : Globalize.format(new Date(), "HH:mm");
                                $('#' + temp._id + '_HighlightCurrentTimeline').find(".e-currenttime").html(currentTime);
                                if (temp.model.timeMode == "12") $('#' + temp._id + '_HighlightCurrentTimeline').find(".e-currenttime").addClass("e-timewidth");
                            }
                        }
                        ChangeCurrentTimePosition();
                        this._intervalID = window.setInterval(ChangeCurrentTimePosition, 60000);
                    }
                    else {
                        if ($("#" + this._id + "_HighlightCurrentTimeline")) {
                            clearInterval(this._intervalID);
                            $("#" + this._id + "_HighlightCurrentTimeline").remove();
                        }
                    }
                }
                else {
                    if (flag)
                        $(this.element.find(".e-monthcells")[index]).addClass("e-monthcurrenttime");
                    else
                        this.element.find(".e-monthcells").removeClass("e-monthcurrenttime");
                }
            }
        },

        /**
        * Section For handle view navigate action of the schedule.
        * @private
        */
        _navigateView: function (e) {
            var newView, oldview = this.currentView();
            var _target = $(e.target);
            if (this.currentView() === "month") {
                if (_target.hasClass("e-monthheader")) {
                    if (this.model.views.indexOf("Day") == -1) return false;
                    this.currentDate(this._getRenderDates()[((_target.parent().parent().index() * 7) + _target.parent().index())]);
                    newView = "day";
                }
                else if (_target.hasClass("e-timecells")) {
                    var target = _target.hasClass("e-timecells") ? _target : _target.parent();
                    if (this.model.views.indexOf("Week") == -1) return false;
                    this.currentDate(this._getRenderDates()[(target.parent().index() * 7)]);
                    newView = "week";
                }
            }
            else if (this.currentView() === "week" || this.currentView() === "workweek") {
                var dayTarget = _target.hasClass("e-headercells") ? _target : _target.parent().hasClass("e-headercells") ? _target.parent() : _target.parent().parent();
                if (this.model.views.indexOf("Day") == -1 || !dayTarget.hasClass("e-headercells")) return false;
                this.currentDate(this._getRenderDates()[dayTarget.index() - 1]);
                newView = "day";
            }
            this._navView = true;
            this._viewChange(e, oldview, newView);
        },
        /**
        * Section For handle over flow icon click of the schedule.
        * @private
        */
        _appOverflowClick: function (e) {
            //to stop the day view navigation while we click overflow app
            if (this.model.views.indexOf("Day") !== -1) {
                var overflowId = $(e.target)[0].id.toString().split("_");
                this.currentDate(new Date(overflowId[3], overflowId[1] - 1, overflowId[2]));
                this._navView = true;
                this._viewChange(e, this.currentView(), "day");
            }
            else return false;
        },
        /**
        * Section For handle calendar navigation of the schedule.
        * @private
        */
        _viewCalendar: function (e) {
            var obj = this.element.find("#" + this._id + "_navigator").data("ejDatePicker");
            if (obj) {
                obj.show();
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                var controlmargin = this.element.position();
                if (this.model.rtl)
                    $("div.e-datepicker").css("top", (controlmargin.top + 47) + "px").css("left", (controlmargin.left + this.element.width() - 280) + "px");
                else
                    $("div.e-datepicker").css("top", (controlmargin.top + 47) + "px").css("left", (controlmargin.left + 55) + "px");
            }
            this.element.find(".e-workcells").removeClass("e-selectedCell");
            this.element.find(".e-workcells").removeClass("e-mousehover");
            this.element.find(".e-appointment,.e-alldayappointment,.e-monthappointment").removeClass("e-selectedAppointment");
        },
        /**
        * Section For change the current view of the schedule.
        * @private
        */
        _viewChange: function (e, oldview, newView) {
            var flag = true;
            if (oldview !== newView || (!oldview && !newView)) {
                if (!this._navView) {
                    flag = (this.currentView() == (e.currentTarget.className.split("e-")[2]).split(" ")[0]) ? false : true;
                    this.currentView($(e.currentTarget).hasClass("e-activeview") ? this.currentView() : (e.currentTarget.className.split("e-")[2]).split(" ")[0]);
                }
                else
                    this.currentView(newView);
                if (flag) this._renderCurrentView();
            }
            var data = { oldView: oldview, newView: this.currentView(), Date: this.currentDate(), target: e, type: "Scheduleview" };
            if (this._trigger("navigation", data))
                return false;
        },
        /**
        * Section For render the current view of the schedule.
        * @private
        */
        _renderCurrentView: function () { //clear the old view and render with newly selected view
            //clear old view
            this.element.find(".e-commonviewbutton").blur();
            this.element.find(".e-commonviewbutton").removeClass("e-activeview");
            this.element.find(".e-headerdaydisplay,.e-datedisplay,.e-headrealldaytable").children().remove(); this.wTR.children().remove();
            this.aTR.children().remove();
            this._getRenderDates();
            //set new view
            this.element.find(".e-" + this.currentView()).addClass("e-activeview").attr("tabIndex", 0).focus();
            this._renderHeaderAllDayTemplate();
            this._renderContentAreaTemplate();
            this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
            this._renderScroller();
            this._renderHeaderDays();
            this.element.find(".e-ampmstyle").height(this.element.find(".e-workcells").height());
            this._dataProcessing(this.dataSource());
            this._renderAppointmentAll();
            (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
            this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
            this._navView = false;
        },
        /**
        * Section For render the header day cells of the schedule.
        * @private
        */
        _renderHeaderDays: function () {
            var dateRender = this._getRenderDates();
            var localizeDays = Globalize.culture(this.model.localization);
            var dateVal = this._getCurrentDateHeaderValue();
            this.element.find("div.e-datedisplay")[0].innerHTML = dateVal;
            if (this.currentView() !== "month") {
                for (var i = 0; i < dateRender.length; i++) {
                    var curday = (this.currentView() === "day") ? localizeDays.calendar.days.names[new Date(dateRender[i]).getDay()] : localizeDays.calendar.days.names[new Date(dateRender[i]).getDay()].slice(0, 3);
                    temp = this.currentView() === "day" ? curday + " " + Globalize.format(new Date(dateRender[i]), "dd MMMM yyyy") : curday + " " + new Date(dateRender[i]).getDate();
                    this.element.find(".e-headerdaydisplay")[i].innerHTML = temp;
                }
                this.element.find(".e-alldaycells").css('height', "27px");
                this.element.find(".e-headercells").addClass("e-dateheaderselect");
            }
            else
                this._renderMonthTimeCells();
        },
        /**
        * Section For render the month time cells of the schedule.
        * @private
        */
        _renderMonthTimeCells: function () {
            var j = 0; var dateRender = this._dateRender;
            for (var i = 0; i < dateRender.length; i++) {
                $(this.element.find("div.e-monthheader"))[i].innerHTML = (Globalize.format(dateRender[i], "dd")).toString();
                if ((i + 1) % 7 == 0) {
                    start = Globalize.format(dateRender[i - 6], "MMM dd");
                    end = Globalize.format(dateRender[i], "MMM dd");
                    $(this.element.find(".e-timecelldivs"))[j++].innerHTML = start + " - " + end;
                }
                if (new Date(dateRender[i]).getMonth() !== new Date(this.currentDate()).getMonth())
                    $(this.element.find("div.e-monthheader")[i]).addClass("e-monthdateshighlight");
                else
                    $(this.element.find("div.e-monthheader")[i]).removeClass("e-monthdateshighlight");
            }
            this.element.find(".e-timecells").addClass("e-dateheaderselect").height(this.element.find(".e-monthcells").height());
            if (j != 6) this.element.find(".e-timecells").addClass("e-timecellpadding");
        },
        /**
        * Section For handle the date navigation of the schedule.
        * @private
        */
        _navigateClick: function (e) {
            var data;
            if (e != null && !this._setCurrentDate) {
                var dat = new Date(this.currentDate());
                var day = this.currentView() === "day" ? 1 : 7;
                if ($(e.currentTarget).hasClass("e-navigateprevious")) {
                    this.currentDate(this.currentView() === "month" ? new Date(dat.setMonth(new Date(this.currentDate()).getMonth() - 1)) : new Date(dat.setDate(new Date(this.currentDate()).getDate() - day)));
                    this.element.find(".e-navigateprevious").attr("tabIndex", 0).focus();
                }
                else if ($(e.currentTarget).hasClass("e-navigatenext")) {
                    this.currentDate(this.currentView() === "month" ? new Date(dat.setMonth(new Date(this.currentDate()).getMonth() + 1)) : new Date(dat.setDate(new Date(this.currentDate()).getDate() + day)));
                    this.element.find(".e-navigatenext").attr("tabIndex", 0).focus();
                }
                if ($(e.currentTarget).hasClass("e-navigateprevious") || $(e.currentTarget).hasClass("e-navigatenext"))
                    data = { prevDate: new Date(dat.getDate() - 1), newDate: dat, target: e, type: "navigatedate" };
                if (this._trigger("navigation", data))
                    return false;
            }
            this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment"); this.element.find(".e-appsoverflow").remove();
            this.element.find(".e-monthcells").removeClass("e-monthcurrenttime");
            if (this.model.showCurrentTimeIndicator) $("#" + this._id + "_HighlightCurrentTimeline").remove();
            this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") || this._quickAppDetailsWindow.ejDialog("close"));
            //  this.element.find("td.e-workcells").removeClass("icon-AddGrad");
            if (this.currentView() === "month")
                this._renderCurrentView();
            else {
                this._renderHeaderDays();
                this._dataProcessing(this.dataSource());
                this._renderAppointmentAll();
                (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
            }
            this._setCurrentDate = false;
        },
        /**
        * Section For handle the device type.
        * @private
        */
        _isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
        * Section For handle the resize of the schedule.
        * @private
        */
        _scheduleResize: function (e) {
            this.model.showQuickWindow && (this._quickAppDetailsWindow.ejDialog("close") || this._quickAppointWindow.ejDialog("close"));
            if (this.model.canResize) {
                var schedule = this;
                if (this.resizeTO) clearTimeout(this.resizeTO);
                this.resizeTO = setTimeout(function () {
                    if (!schedule._isDevice()) {
                        var scheduleWidth = $($(schedule.element.find("div")[0]).parent()).width() + 20; // $(this.element.find("table")[0]).width + 20;
                        var scheduleHeight = $($(schedule.element.find("div")[0]).parent()).height(); // this.element.find("table")[0].offsetHeight;
                        if (scheduleWidth > $(this).width()) {
                            schedule._scWidth = $(this).width() - 20;
                        }
                        else if (scheduleWidth < $(this).width()) {
                            schedule._scWidth = schedule.model.width;
                        }
                        if (scheduleHeight > $(this).height()) {
                            schedule._scHeight = $(this).height() - 20;
                        }
                        else if (scheduleHeight < $(this).height()) {
                            schedule._scHeight = schedule.model.height;
                        }
                    }
                    else {
                        schedule._scHeight = schedule.element.height();
                        schedule._scWidth = schedule.element.width();
                    }
                    schedule.element.find(".e-highlightcurrenttimeline").remove();
                    schedule.element.children().remove();
                    schedule._renderInitSchedule();
                    schedule._renderAppointmentAll();
                }, 500);
            }
        },
        /**
        * Section For render the templates used in the schedule.
        * @private
        */
        _renderTemplates: function () {
            //content area template
            this.workcellTemp = $.templates(this._createTemplate(workCellsTemplate(), "_WorkCells"));
            this.monthcellTemp = $.templates(this._createTemplate(monthCellsTemplate(), "_MonthCells"));

            //timecells template rendering
            this.timeCellTemplate = $.templates(this._createTemplate(timeCellsTemplate(), "_TimeCells"));

            //header day template rendering
            this.headTemplate = $.templates(this._createTemplate(headerTemplate(), "_HeaderDate"));

            // appointment template rendering
            this.appTemplate = $.templates(this._createTemplate(appointmentTemplate(), "_Appointment"));

            //allday template rendering
            this.alldayTemp = $.templates(this._createTemplate(alldayTemplate(), "_AlldayCells"));
        },
        /**
        * To configure start and end hour of the schedule		
        * @private
        */
        _setNewStartEndHours: function () {
            if (this.currentView() !== "month") {
                this.wTR.children().remove();
                this._renderContentAreaTemplate();
                this._renderScroller();
                this.element.find(".e-ampmstyle").height(this.element.find(".e-workcells").height());
                (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").addClass("e-businesshighlightworkcells");
                if (this.model.showCurrentTimeIndicator) $("#" + this._id + "_HighlightCurrentTimeline").remove();
                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                this._renderAppointmentAll();
            }
        },
        /**
        * Section For render the schedule control initially.
        * @private
        */
        _renderInitSchedule: function () {
            this.model.width = this.model.width ? this.element.width(this.model.width).width() : this.element.width();
            this.model.height = this.model.height ? this.element.height(this.model.height).height() : this.element.height();
            this.scWidth = this._scWidth ? parseInt(this._scWidth) : parseInt(this.model.width);
            this.scHeight = this._scHeight ? parseInt(this._scHeight) : parseInt(this.model.height);

            this.element.addClass("e-scheduleouter").height(this.scHeight).width(this.scWidth).attr("role", "presentation");
            $(this.element).attr("tabIndex", 1);
            var $scheduleDiv = ej.buildTag('div', "", {}, { height: this.scHeight, width: this.scWidth });
            var $scheduleTable = ej.buildTag('table.e-table', "", {}, { cellpadding: "0px", cellspacing: "0px" }).addClass("e-scheduleoutertable");
            var $tBody = ej.buildTag('tbody');
            var hTR = ej.buildTag("tr.e-scheduleheader");
            this.aTR = ej.buildTag("tr");
            this._getRenderDates();
            this.wTR = ej.buildTag("tr.e-contentarearow");
            this._renderTemplates();
            this._renderHeaderBar(hTR); //header dates div(prev & next)
            this._renderHeaderAllDayTemplate(); //header and all-day div
            this._renderContentAreaTemplate(); //contentarea div

            $scheduleDiv.append($scheduleTable.append($tBody.append(hTR).append(this.aTR).append(this.wTR)));
            this.element.append($scheduleDiv);
            this.element.find(".e-" + this.currentView()).addClass("e-activeview");
            this.element.find("#" + this._id + "_navigator").ejDatePicker({ rtl: this.model.rtl, buttonText: this._getLocalizedLabels("Today"), select: $.proxy(this._onselect, this), localize: this.model.localization, cssClass: this.model.cssClass });
            this.element.find("span.e-datewidget").css("display", "none");

            (this.currentView() === "month") ? this._renderMonthTimeCells() : this.element.find(".e-headercells").addClass("e-dateheaderselect");
            (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
            this._renderAppointmentWindow();
            //Render Recurrence edit dialog.
            this._renderRecurEditWindow();
            this.model.showQuickWindow && (this._renderQuickAppWindow() || this._renderQuickAppDetailsView());
            this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
            // render scroller
            this._renderScroller();
            this.element.find(".e-ampmstyle").height($(this.element.find(".e-workcells")).height());
            this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
        },
        /**
        * Section For render the scroller of the schedule.
        * @private
        */
        _renderScroller: function () {
            var hghtdiff = this.element.find(".e-scheduleheader").height() + this.element.find(".e-headrealldaytable").height(); var proxy = this; var wdthdiff = (this.currentView() === "month") ? 0 : 4;
            var _scrollDiv = $('div#' + this._id + "_scroller");
            _scrollDiv.ejScroller({
                rtl: this.model.rtl, height: this.scHeight - hghtdiff, width: this.scWidth - this.element.find(".e-scrolltimecells").width() + wdthdiff, cssClass: this.model.cssClass,
                thumbStart: function (e) {
                    if ($(e.originalEvent.target).parents(".e-appointment").hasClass("e-appointment")) return false;
                },
                scroll: function (e) {
                    //  if (e.source == "thumb" && $(e.target.target).parents(".e-appointment").hasClass("e-appointment")) return false;
                    proxy.element.find(".e-scrolltimecells").css("top", (-e.scrollTop) + "px");

                    //changing AM/PM label while scrolling
                    if (proxy.model.timeMode === "12" && proxy.currentView() !== "month") {
                        var top = (e.scrollTop / (proxy.element.find(".e-workcells").height() * 2));
                        top = (Math.round(top) < top) ? Math.round(top) + 1 : Math.round(top);
                        var localizeAmPm = Globalize.culture(proxy.model.localization).calendar;
                        if (localizeAmPm.AM && localizeAmPm.PM) {
                            proxy.element.find('.e-ampmstyle').each(function (i, val) {
                                if (i == (top))
                                    (parseInt($(val)[0].id) < 12) ? $(val).text(localizeAmPm.AM[0]) : $(val).text(localizeAmPm.PM[0]);
                                else if (($(val).text() == localizeAmPm.AM[0] && i > 0) || ($(val).text() == localizeAmPm.PM[0] && parseInt($(val)[0].id) > 12))
                                    $(val).text("00");
                            });
                        }
                    }
                }
            });
            _scrollDiv.ejScroller("isVScroll") ? this.element.find(".e-emptytd,.e-emptyalldaytd").show() : this.element.find(".e-emptytd,.e-emptyalldaytd").hide() && this.element.find(".e-draggableworkarea").css("height", "100%");
            //(this._browserInfo.name == "webkit" || this._browserInfo.name == "chrome") ? this.element.find(".e-headrealldaytable").width(dragWidth + timecellWidth + 1) : this.element.find(".e-headrealldaytable").width(dragWidth + timecellWidth);
        },
        /**
        * Section For handle the business hour highlight of the schedule.
        * @private
        */

        _highlightBusinessHours: function () {
            if (this.currentView() !== "month") {
                this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
                var renderDates = this._dateRender;
                var target = this.element.find(".e-workcells");
                var b_startHour = this.model.businessStartHour >= this.model.startHour ? this.model.businessStartHour - this.model.startHour : 0;
                var b_endHour = this.model.businessEndHour <= this.model.endHour ? this.model.businessEndHour - this.model.startHour : this.model.endHour - this.model.startHour;
                var timeRowsCount = 60 / 30;
                var colCount = this.currentView() === "week" ? 7 : this.currentView() === "workweek" ? 5 : 1;
                for (var i = (b_startHour * timeRowsCount) ; i < (b_endHour * timeRowsCount) ; i++) {
                    for (var j = 0; j < colCount; j++) {
                        if (new Date(renderDates[j]).getDay() !== 0 && new Date(renderDates[j]).getDay() !== 6)
                            $(target[(i * colCount) + j]).addClass("e-businesshighlightworkcells");
                    }
                }
            }
            else {

                var monthDays = this._dateRender;
                var target = this.element.find(".e-monthcells"); target.removeClass("e-monthcellhighlight");
                for (var i = 0; i < monthDays.length; i++) {
                    if (new Date(monthDays[i]).getDay() !== 0 && new Date(monthDays[i]).getDay() !== 6)
                        $(target[i]).addClass("e-monthcellhighlight");
                }
            }
        },
        /**
        * Section For handle the calendar date select and change the current date of the schedule.
        * @private
        */
        _onselect: function (e) {
            var proxy = this;
            var currentdate = new Date(this.currentDate());
            var selectdate = new Date(e.value);
            this.currentDate(selectdate);
            var data = { prevDate: currentdate, newDate: this.currentDate(), target: e }
            if (this._trigger("navigation", data))
                return false;
            proxy._navigateClick(e);
        },
        /**
        * Section For render the header bar of the schedule.
        * @private
        */
        _renderHeaderBar: function (hTR) {
            this.hTD = ej.buildTag('td', "", {}, {});
            var commonDiv = ej.buildTag('div.e-datecommondiv').append(ej.buildTag('div.e-icon e-navigateprevious', "", {}, { title: this._getLocalizedLabels("Previous") })).append(ej.buildTag('div', "", {}, { "aria-live": "assertive", tabindex: 0 }).html(this._getCurrentDateHeaderValue()).addClass("e-datedisplay")).append(ej.buildTag("input#" + this._id + "_navigator").css("display", "none").addClass("dateCalendar")).append(ej.buildTag('div.e-icon e-navigatenext', "", {}, { title: this._getLocalizedLabels("Next") }));
            var buttonsDiv = ej.buildTag('div.e-viewsdiv');
            hTR.append(this.hTD.append(commonDiv).append(buttonsDiv.append(this._renderViewNavigation(this.model.views))));
        },
        /**
        * Section For render the views of the schedule.
        * @private
        */
        _renderViewNavigation: function (views) {
            var $viewTable = ej.buildTag('table.e-table e-viewstable', "", {}, { cellspacing: '0px', cellpadding: '0px' });
            var $viewBody = ej.buildTag('tbody');
            var $viewTR = ej.buildTag('tr');
            var _viewCollection = views; //this.model.views; // ["Day", "Week", "WorkWeek", "Month", "Agenda"];
            for (var i = 0; i < _viewCollection.length; i++) {
                var viewClassname = (_viewCollection.length == 1) ? "e-" + _viewCollection[i].toLowerCase() + " e-singleview" : (i == 0) ? "e-" + _viewCollection[i].toLowerCase() + " e-firstview" : i == (_viewCollection.length - 1) ? "e-" + _viewCollection[i].toLowerCase() + " e-lastview" : "e-" + _viewCollection[i].toLowerCase();
                $viewTR.append(ej.buildTag('td').append(ej.buildTag('div', this._getLocalizedLabels(_viewCollection[i]), {}, {}).addClass("e-commonviewbutton").addClass(viewClassname)));
            }
            $viewTable.append($viewBody.append($viewTR))
            return $viewTable;
        },
        /**
        * Section For render the recurrence window of the schedule.
        * @private
        */
        _recurrenceWindow: function () {
            var recurtype = ["Daily", "Weekly", "Monthly", "Yearly", "EveryWeekDay"];
            var culture = Globalize.culture(this.model.localization);
            var weekshort = this._dayShortNames;
            var weekfull = this._dayFullNames;
            var months = this._monthNames;
            var startday = ["First", "Second", "Third", "Fourth", "Last"];
            var $recurWin = "<div class='e-recurwindow e-windowmargin' style='display:none'>" +
                    "<form id='" + this._id + "_RecurForm'><table class='e-table' width='100%' cellpadding='7'><tbody>";
            $recurWin += "<tr><td width='16%' class='e-textlabel'>" + this._getLocalizedLabels("Recurrence") + ":</td>" +
                    "<td><input class='recurtype' id='recurtype' type='text' name='RecurrenceType' value='" + this._getLocalizedLabels(recurtype[0]) + "' /><div id='recurtypelist'><ul>";
            for (var type = 0; type < recurtype.length; type++) {
                $recurWin += "<li>" + this._getLocalizedLabels(recurtype[type]) + "</li>";
            }
            $recurWin += "</ul></div></td></tr>";
            $recurWin += "<tr id='" + this._id + "repeatEvery'><td class='e-textlabel'>" + this._getLocalizedLabels("Every") + ":</td><td><table><tr><td><div class='e-floatleft'><input id='recurevery' class='e-recurevery' type='text' /></div></td>" +
                    "<td><div id='" + this._id + "recurtypes' class='e-appcheckbox'>" + this._getLocalizedLabels("RecurrenceDay") + "</div></td></tr></table></td></tr>";

            $recurWin += "<tr id='" + this._id + "weeklyrecur' style='display:none'><td id='recurweek' class='e-textlabel'><div>" + this._getLocalizedLabels("RepeatOn") + ":</div></td>" +
                    "<td id='recurweekcount'><table class='e-table' cellpadding='3'><tr>";
            for (var weekcount = 0; weekcount < weekshort.length; weekcount++) {
                $recurWin += "<td><div class='e-weekday e-_" + weekshort[weekcount] + "'><input id='" + weekshort[weekcount] + "' class='weekdays e-weekly" + this._dayNames[weekcount] + "'name='weekdays' type='checkbox'/><label class='e-textmargin' for='" + weekshort[weekcount] + "'>" + weekshort[weekcount] + "</label></div></td>";
            }
            $recurWin += "</tr></table></td></tr>";
            $recurWin += "<tr id='" + this._id + "monthlyrecur' style='display:none'><td id='recurmonthtext' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("RepeatBy") + ":</div></td>" +
                    "<td id='recurmonthcount'><table class='e-table' cellpadding='3'><tr><td><div><input id='" + this._id + "monthday' class='e-monthdaytype' name='monthday' type='radio'/><label class='e-textmargin' for='" + this._id + "monthday'>" + this._getLocalizedLabels("Day") + "</label></div></td>" +
                    "<td><div><input id='monthdate' class='e-monthdate' type='text'/></div></td></tr>" +
                    "<tr><td><div><input id='" + this._id + "monthon' class='e-monthposition' name='monthday' type='radio'/><label class='e-textmargin' for='" + this._id + "monthon'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                    "<td><div><input id='monthsrt' class='monthsrt' type='text' name='monthsrt' value=''/><div id='monthsrtlist'><ul>";
            for (var day = 0; day < startday.length; day++) {
                $recurWin += "<li>" + this._getLocalizedLabels(startday[day]) + "</li>";
            }
            $recurWin += "</div></div></td>";
            $recurWin += "<td><div class='e-appcheckbox'><input id='monthsrtday' class='e-monthsrtday' type='text' name='monthsrtday' value=''/>" +
            "<div id='monthsrtdaylist'><ul>";
            for (var monthday = 0; monthday < weekfull.length; monthday++) {
                $recurWin += "<li>" + weekfull[monthday] + "</li>";
            }
            $recurWin += "</div></div></td></tr></table></td></tr>";
            $recurWin += "<tr id='" + this._id + "yearlyrecur' style='display:none'><td id='recuryeartext' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("RepeatBy") + ":</div></td>" +
                    "<td id='recuryearcount' ><table class='e-table' cellpadding='3'><tr>" +
                    "<td><div><input id='" + this._id + "yearday' class='e-yearrecurday' name='yearday' type='radio'/><label class='e-textmargin' for='" + this._id + "yearday'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                    "<td><div class='e-controlalign'><input id='yearmonth' class='yearmonth' type='text' name='yearmonth' value=''/></div></td>" +
                    "<div id='yearmonthlist'><ul>";
            for (var monthcount = 0; monthcount < (months.length - 1) ; monthcount++) {
                $recurWin += "<li>" + months[monthcount] + "</li>";
            }
            $recurWin += "</ul></div><td><input id='yeardate' class='yeardate' text='text'/></td></tr>" +
                    "<tr><td><div><input id='" + this._id + "yearother' class='e-yearrecurposi' name='yearday' type='radio'/><label class='e-textmargin' for='" + this._id + "yearother'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                    "<td><div><input id='yearsrt' class='yearsrt' type='text' name='yearsrt' value=''/><div id='yearsrtlist'><ul>";
            for (var yearstartday = 0; yearstartday < startday.length; yearstartday++) {
                $recurWin += "<li>" + this._getLocalizedLabels(startday[yearstartday]) + "</li>";
            }
            $recurWin += "</ul></div></div></td><td><div class='e-controlalign'><input id='yearsrtday' class='yearsrtday' type='text' name='yearsrtday' value=''/>" +
            "<div id='yearsrtdaylist'><ul>";
            for (var yearweek = 0; yearweek < weekfull.length; yearweek++) {
                $recurWin += "<li>" + weekfull[yearweek] + "</li>";
            }
            $recurWin += "</ul></div></div></td><td><div><span>" + this._getLocalizedLabels("OfEvery").split(" ")[0] + "</span></div></td><td><div><input id='yearsrtmonth' class='yearsrtmonth' type='text' name='yearsrtmonth' value=''/>" +
                    "<div id='yearsrtmonthlist'><ul>";
            for (var yearmonth = 0; yearmonth < (months.length - 1) ; yearmonth++) {
                $recurWin += "<li>" + months[yearmonth] + "</li>";
            }
            $recurWin += "</ul></div></div></td></tr></table></td></tr>";
            $recurWin += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("StartsOn") + ":</td>" +
                    "<td><input id='recurstartdate' class='recurstartdate' type='text' name='RecurStartDate' value=''/></td></tr>";
            $recurWin += "<tr><td class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("Ends") + ":</div></td>" +
                    "<td><table class='e-table' cellpadding='3'><tr><td><div><input id='repeatendnever' class='recurends e-recurnoend' type='radio' name='repeatend' value='Never'/><label class='e-textmargin' for='repeatendnever'>" + this._getLocalizedLabels("Never") + "</label></div></td></tr>" +
                    "<tr><td><div><input id='" + this._id + "repeatendafter' class='recurends e-recurafter' type='radio' name='repeatend'/><label class='e-textmargin' for='" + this._id + "repeatendafter'>" + this._getLocalizedLabels("After") + "</label>" +
                    "</div></td><td><div><input id='dailyoccurance' class='e-recurcount' type='text'/></div></td>" +
                    "<td><span style='margin-left: -80px;'>" + this._getLocalizedLabels("Occurence") + "</span></td></tr>" +
                    "<td><div><input id='" + this._id + "repeatendon' class='recurends e-recuruntil' type='radio' name='repeatend'/><label class='e-textmargin' for='" + this._id + "repeatendon'>" + this._getLocalizedLabels("On") + "</label></div></td>" +
                    "<td><input id='daily' class='e-until' type='text' name='daily' value=''/></td></tr></table></td></tr>";
            $recurWin += "<tr style='display:none'><td><span class='e-textlabel'>" + this._getLocalizedLabels("Summary") + ":</span></td><td><span class=e-recurRule></span></td></tr></tbody></table></form>";
            $recurWin += "<div class='e-floatright'><table class='e-table' cellpadding='5'><tr><td><div class='e-btnmargin e-floatleft'><button id='donerecur' class='recurbutton'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                    "<td><div class='e-btnmargin e-floatleft'><button id='cancelrecur' class='e-cancelrecur recurbutton'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></tabled></div></div>";
            return $recurWin;
        },
        /**
        * Section For render the appointment window of the schedule.
        * @private
        */
        _renderAppointmentWindow: function () {
            this._appointmentAddWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppointmentAddEditWindow");
            var $appWindow = "<div class='e-appwindow e-windowmargin'>" +
                            "<form id='" + this._id + "_AddEditForm'><table class='e-table' width='100%' cellpadding='7'><tbody>";
            $appWindow += "<tr><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("AppointmentSubject") + ":</td><td colspan='3' class='e-rightfields'>" +
                          "<input id='" + this._id + "subject' tabIndex='0' class='subject e-inputtext' name='" + this.model.fields["subjectField"] + "' type='text' /><input id='id' type='text' name='" + this.model.fields["idField"] + "' value='' style='display: none'/></td></tr>";
            $appWindow += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("StartTime") + ":</td><td width='25%'><input id='" + this._id + "startdate' class='startenddate' type='text' name='StartDate' value='' /></td><td width=22%'><input type='text' id='" + this._id + "starttime' class='startendtime' name='StartTime'/></td>" +
                          "<td width='28%'><input id='startutc' class='startutc' type='text' value=''/><div id='Timezoneliststr'></div></td></tr>";
            $appWindow += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("EndTime") + ":</td><td><input id='" + this._id + "enddate' class='startenddate' type='text' name='EndDate' value='' /></td><td><input type=''text' id='" + this._id + "endtime' class='startendtime' name='EndTime'/></td>" +
                          "<td><input id='endutc' class='endutc' type='text' value=''/><div id='Timezonelistend'></div></td></tr>";
            $appWindow += "<tr><td colspan='4'><table class='e-table' cellpadding='3'><tr><td><div class='e-allday e-checkboxmargin'><input id='" + this._id + "allday' tabIndex='0' class='allday' type='checkbox' name='" + this.model.fields["alldayField"] + "'/></div></td><td><div class='e-alldaytext'>" +
                          "<label for='" + this._id + "allday'>" + this._getLocalizedLabels("AllDay") + "</label></div></td><td><div class='e-checkboxmargin e-recurrence'>" +
                          "<input id='" + this._id + "recurrence' tabIndex='0' class='recurrence' type='checkbox' name='" + this.model.fields["recurrenceField"] + "' value='recurrence'/></div></td><td><label class='e-repeatlbl' for='" + this._id + "recurrence'>" +
                          this._getLocalizedLabels("Recurrence") + "</label></td></tr></table></td></tr>";
            $appWindow += "<tr id='" + this._id + "summary' style='display:none'><td class='e-textlabel'><label>" + this._getLocalizedLabels("Summary") + ":</label></td><td colspan='3'><table><tr><td><label class='e-recurFinalRule'></label></td><td><div class='e-recuredit'><a class='e-recuredit' href='#'>" + this._getLocalizedLabels("Edit") + "</a><span class='e-icon e-scheduleedit'></span></div></td></tr></table></td></tr>";
            $appWindow += "<tr><td colspan='4'><textarea class='e-description e-inputtext' id='description' name='" + this.model.fields["descriptionField"] + "'></textarea></td></tr></tbody></table></form>";
            $appWindow += "<div class='e-floatright'><table class='e-table' cellpadding='5'><tr><td><div class='e-btnmargin e-floatleft'><button id='done' class='e-btndone e-appsave'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                          "<td><div class='e-btnmargin e-floatleft'><button id='cancel' class='e-appointcancel cancel'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></table></div></div>";
            this._appointmentAddWindow.append($appWindow).append(this._recurrenceWindow());
            this.element.append(this._appointmentAddWindow);
            this._renderControls();
            this._appointmentAddWindow.ejDialog({
                width: 600,
                cssClass: this.model.cssClass,
                autoOpen: false,
                rtl: this.model.rtl, modal: true,
                title: this._getLocalizedLabels("CreateAppointmentTitle"),
                contentContainer: "#" + this._id,
                resizable: false,
                allowKeyboardNavigation: false,
                close: $.proxy(this._onclose, this)
            });
            $("#" + this._id + "AppointmentAddEditWindow_wrapper").appendTo("body").addClass("e-scheduledialog").find(".e-titlebar").addClass("e-dialogheader");
            this._on(this._appointmentAddWindow, "click", "a.e-recuredit", $.proxy(this._editRecurrence, this));
        },
        /**
        * Section For render the quick window.
        * @private
        */
        _renderQuickAppWindow: function () {
            this._quickAppointWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppointmentQuickWindow");
            var $quickWin = "<div id='" + this._id + "_quickAppwindow' class='e-tableAppoint'>" +
                    "<table cellpadding='7' class='e-qappwindow'><tbody>";
            $quickWin += "<tr><td colspan='2'><div class='e-quickstartend'></div> <div title='" + this._getLocalizedLabels("Close") + "'class='e-icon e-scheduleclose e-schedclosemargin e-pointercursor'></div></td></tr><tr><td class='e-subjectlabel'>" + this._getLocalizedLabels("Event") + ":</td><td colspan='2' class='e-rightfields'>" +
                          "<input id='" + this._id + "subject' class='subject e-inputtext' style='height:22px;' type='text' /></td></tr></tbody></table>";
            $quickWin += "<div style='margin-top:5px; padding-bottom: 30px;'><div tabIndex='0' class='e-detailedapp e-detailedwindow'> " + this._getLocalizedLabels("Detailed") + " >></div><div style='margin-right:-3px;' class='e-floatright'><div class='e-quickAppDone'><button id='done' class='e-btndone qAppDone' style='font-size:12px !important;'>" + this._getLocalizedLabels("CreateAppointmentTitle") + "</button></div>" +
                          "</div></div></div>";
            this._quickAppointWindow.append($quickWin);
            this.element.append(this._quickAppointWindow);
            this._quickAppointWindow.append(ej.buildTag("div.e-quickarrow").append("<div class='e-outerarrowline'></div>").append("<div class='e-innerarrow'></div>"));
            var control = this._quickAppointWindow;
            //control.find('.subject').ejMaskEdit({ name: "Subject", inputMode: ej.inputMode.Text, width: "100%", height: "25px", cssClass: this.model.cssClass });
            control.find('.e-btndone').ejButton({ roundCorner: true, width: "130px", click: $.proxy(this._saveAppointment, this), cssClass: this.model.cssClass });
            this._on(control.find('.e-scheduleclose'), "click", $.proxy(this._quickWindowClose, this));
            this._on(control.find('.e-detailedapp'), "click", $.proxy(this._appointmentWindow, this));
            this._quickAppointWindow.ejDialog({
                width: 330,
                cssClass: this.model.cssClass,
                rtl: this.model.rtl,
                autoOpen: false,
                resizable: false,
                draggable: false, showHeader: false,
                allowKeyboardNavigation: false
            });
            $("#" + this._id + "AppointmentQuickWindow_wrapper").addClass("e-scheduledialog").find("#" + this._id + "AppointmentQuickWindow").addClass("e-dialogouter");
        },
        /**
        * Section For handle the details of quick app window.
        * @private
        */
        _renderQuickAppDetailsView: function () {
            this._quickAppDetailsWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppDetailsWindow");
            var $quickAppWin = "<div id='" + this._id + "_quickAppDetailswindow' class='e-tableAppoint'>" +
                          "<table cellpadding='1' class='e-qappwindow'><tbody>";
            $quickAppWin += "<tr><td colspan='2'><div class='e-quicksubject' style='float:left;'></div> <div title='" + this._getLocalizedLabels("Close") + "' class='e-icon e-scheduleclose e-schedclosemargin e-pointercursor'></div></td></tr><tr><td colspan='2' class='e-rightfields'>" +
                          "</td></tr><tr><td colspan='2'><div class='e-quickstartendtime'></div></td></tr></tbody></table>";
            $quickAppWin += "<div style='margin-top:5px;'><div title='" + this._getLocalizedLabels("Delete") + "' class='e-icon e-scheduledelete e-pointercursor' tabIndex='0'></div><div style='margin-right:-15px;' class='e-floatright'><div class='e-editevents'><div class='e-editevent' tabIndex='0'>" + this._getLocalizedLabels("Editevent") + " | " + "</div><div class='e-editseries' tabIndex='0'>" + this._getLocalizedLabels("Editseries") + " >></div> </div>" +
                          "</div></div></div>";
            this._quickAppDetailsWindow.append($quickAppWin);
            this.element.append(this._quickAppDetailsWindow);
            this._quickAppDetailsWindow.append(ej.buildTag("div.e-quickarrow").append("<div class='e-outerarrowline'></div>").append("<div class='e-innerarrow'></div>"));
            this._on(this._quickAppDetailsWindow.find('.e-scheduleclose'), "click", $.proxy(this._quickAppDetailWindowClose, this));
            this._on(this._quickAppDetailsWindow.find('.e-scheduledelete'), "click", $.proxy(this._deleteAppoint, this));
            this._on(this._quickAppDetailsWindow.find('.e-editevent,.e-editseries'), "click", $.proxy(this._editClick, this));
            this._quickAppDetailsWindow.ejDialog({
                width: 270,
                cssClass: this.model.cssClass,
                rtl: this.model.rtl,
                autoOpen: false,
                resizable: false,
                draggable: false, showHeader: false,
                allowKeyboardNavigation: false
            });
            $("#" + this._id + "AppDetailsWindow_wrapper").addClass("e-scheduledialog").find("#" + this._id + "AppDetailsWindow").addClass("e-dialogouter");
        },
        /**
        * Section For edit click in quick window of the schedule.
        * @private
        */
        _editClick: function (e) {
            var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, this._appId);
            var templist = new ej.DataManager(this._processed).executeLocal(query);
            if ($(e.target).hasClass("e-editevent")) {
                if (templist[0][this.model.fields["recurrenceField"]] == 1) {
                    this._parentId = templist[0]["ParentId"];
                    this._currentAction = ej.Schedule.actions.EditOcurrence;
                    this._showAppointmentDetails(this._appId, true);
                }
                else {
                    this._currentAction = ej.Schedule.actions.Save;
                    this._showAppointmentDetails(this._appId);
                }
            }
            else if ($(e.target).hasClass("e-editseries")) {
                if (templist[0][this.model.fields["recurrenceField"]] == 1) {
                    this._parentId = templist[0]["ParentId"];
                    this._currentAction = ej.Schedule.actions.EditSeries;
                    this._showAppointmentDetails(this._parentId);
                }

            }
            this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
        },
        /**
        * Section For handle the delete appointments of the schedule.
        * @private
        */
        _deleteAppoint: function () {
            if (this._appId) {
                this._currentAction = ej.Schedule.actions.Delete;
                var parentData = this._getAppointmentByParentId(this._appId);
                if (ej.isNullOrUndefined(parentData[this.model.fields["recurrenceRuleField"]]))
                    this._deleteApp(parseInt(this._appId));
                else {
                    this._parentId = parseInt(parentData[this.model.fields["idField"]]);
                    this._recurEditWindow.find(".e-editonly").ejRadioButton({ check: true });
                    this._recurEditWindow.ejDialog("open");
                    this._recurEditWindow.focus();
                }
            }
            this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
        },
        /**
        * Section For handle the quick window close of the schedule.
        * @private
        */
        _quickAppDetailWindowClose: function () {
            this._quickAppDetailsWindow.ejDialog("close");
            this._processFocus();
        },
        /**
        * Section For handle the all day option check in the appoinment window.
        * @private
        */
        _alldayCheck: function () {
            if (this._appointmentAddWindow.find(".allday").data("ejCheckBox").isChecked) {
                this._appointmentAddWindow.find(".startendtime").ejTimePicker({ enabled: false });
                this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList({ enabled: false });
            }
            else {
                this._appointmentAddWindow.find(".startendtime").ejTimePicker({ enabled: true });
                this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList({ enabled: true });
            }
        },
        /**
        * Section For render the recurrence edit window of the schedule .
        * @private
        */
        //Render Recurrence window 
        _renderRecurEditWindow: function () {
            this._recurEditWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "RecurrenceEdit")
            var title = ej.buildTag('div.e-recurtitle', this._getLocalizedLabels("RecurrenceEditTitle"));
            title.append(ej.buildTag('div.e-icon e-scheduleclose e-schedclosemargin e-pointercursor', "", {}, { title: this._getLocalizedLabels("Close") }));
            this._recurEditWindow.append(title);
            var msgDiv = ej.buildTag('div.e-recureditmsg', this._getLocalizedLabels("RecurrenceEditMessage"));
            //<input type="radio" value="Never" name="repeatend" class="recurends e-radiobtn e-js e-input" id="repeatendnever" tabindex="">
            var editOnly = ej.buildTag('div.e-recureditopt').append(ej.buildTag('input.e-editonly', "", {}, { type: "radio", name: "recurEdit", id: this._id + "_editOnly" })).append(ej.buildTag('label.e-recureditlbl', this._getLocalizedLabels("RecurrenceEditOnly"), {}, { "for": this._id + "_editOnly" }));
            var editSer = ej.buildTag('div.e-recureditopt').append(ej.buildTag('input.e-editseries', "", {}, { type: "radio", name: "recurEdit", id: this._id + "_editSeries" })).append(ej.buildTag('label.e-recureditlbl', this._getLocalizedLabels("RecurrenceEditSeries"), {}, { "for": this._id + "_editSeries" }));
            var btns = ej.buildTag('div.e-recureditbtns').append(ej.buildTag('button.e-recurdone', this._getLocalizedLabels("Done"), {}, { name: "done", value: this._getLocalizedLabels("Done") })).append(ej.buildTag('button.e-recurcancel', this._getLocalizedLabels("Cancel"), {}, { name: "cancel", value: this._getLocalizedLabels("Cancel") }));
            this._recurEditWindow.append(msgDiv).append(editOnly).append(editSer).append(btns);
            this._recurEditWindow.find('.e-editonly,.e-editseries').ejRadioButton({ cssClass: this.model.cssClass });
            btns.find('.e-recurdone,.e-recurcancel').ejButton({ cssClass: this.model.cssClass });
            this._on(this._recurEditWindow, "click", ".e-recurdone,.e-recurcancel", $.proxy(this._recurBtnClick, this));
            this._on(this._recurEditWindow, "click", "div.e-scheduleclose", $.proxy(this._recrConformClose, this));
            this._recurEditWindow.appendTo(this.element);
            this._recurEditWindow.ejDialog({
                width: 360,
                cssClass: this.model.cssClass,
                rtl: this.model.rtl,
                autoOpen: false,
                modal: true,
                resizable: false,
                showHeader: false,
                contentContainer: "#" + this._id,
                allowKeyboardNavigation: false,
                close: $.proxy(this._processFocus, this)
            });
        },
        /**
        * Section For handle the recurrence window close.
        * @private
        */
        // Recurrence Conform window close
        _recrConformClose: function () {
            this._recurEditWindow.ejDialog("close");
            this._processFocus();
        },
        /**
        * Section For handle the recurrence edit click in the appointment window.
        * @private
        */
        //Recurrence edit button click events
        _recurBtnClick: function (args) {
            if ($(args.target).hasClass("e-recurdone")) {
                if (this._recurEditWindow.find(".e-editonly").ejRadioButton("option", "check")) {
                    if (this._currentAction == ej.Schedule.actions.Delete) {
                        var _newAddList = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, parseInt(this._appId)))[0];
                        this._deleteOcurrence(this._parentId, _newAddList[this.model.fields["startTimeField"]]);
                        var dele = new ej.DataManager(this._processed).remove(this.model.fields["idField"], parseInt(this._appId));
                        this.element.find("#Appointment_" + this._appId).remove();
                    }
                    else {
                        this._currentAction = ej.Schedule.actions.EditOcurrence;
                        this._showAppointmentDetails(this._appId, true);
                    }
                }
                else {
                    if (this._currentAction == ej.Schedule.actions.Delete)
                        this._deleteApp(parseInt(this._parentId));
                    else {
                        this._currentAction = ej.Schedule.actions.EditSeries;
                        this._showAppointmentDetails(this._parentId);
                    }
                }
            }
            this._recurEditWindow.ejDialog("close");
        },
        /**
        * Section For handle the recurrence select in the appointment window.
        * @private
        */
        _repeat: function (e) {
            if (e.isChecked || this._recurChecked) {
                if (this._currentAction == ej.Schedule.actions.EditSeries || this._appointmentAddWindow.find(".e-recurFinalRule").html() != "") {
                    this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "table-row");
                }
                else {
                    this._appointmentAddWindow.find(".e-appwindow").toggle();
                    this._appointmentAddWindow.find(".e-recurwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 400);
                    this._recurClearFields();
                    var strDate = new Date(this._appointmentAddWindow.find('.startenddate').ejDatePicker("option", "value"));
                    this._appointmentAddWindow.find('.recurtype').ejDropDownList({ selectedItem: 0 });
                    this._appointmentAddWindow.find('.recurtype').ejDropDownList({ selectedItem: 1 });
                    this._appointmentAddWindow.find(".recurstartdate").ejDatePicker({ value: strDate });
                    this._appointmentAddWindow.find('.e-until').ejDatePicker({ value: new Date(strDate.setDate(strDate.getDate() + 7 * 10)) });
                    this._appointmentAddWindow.find(".e-recurnoend").ejRadioButton({ check: true });
                }
            }
            else 
                (this._currentAction == ej.Schedule.actions.EditSeries || this._appointmentAddWindow.find(".e-recurFinalRule").html() != "") && this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "none");
            this._recurChecked = false;
        },
        /**
        * Section For handle the recurrence type change in the recurrence window.
        * @private
        */
        _recurtype: function (e) {
            this._rRuleFreq = "";
            var _rule = "";
            var strDate = new Date(this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value"));
            if (e.itemId != null) {
                this._appointmentAddWindow.find("#" + this._id + "repeatEvery").css("display", "table-row");
                if (e.itemId == 0) {
                    this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceDay");
                    this._appointmentAddWindow.find("#" + this._id + "weeklyrecur,#" + this._id + "monthlyrecur,#" + this._id + "yearlyrecur").css("display", "none");
                    this._appointmentAddWindow.find("#" + this._id + "dailyrecur").css("display", "table-row");
                    this._rRuleFreq = "DAILY";
                    _rule = this._getLocalizedLabels("RecurrenceDay").toLowerCase();
                }
                else if (e.itemId == 1) {
                    //_rule = this._weeklyRecurrnce();
                    this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceWeek");
                    var curDay = this._dayNames[strDate.getDay()];
                    this._appointmentAddWindow.find(".e-weekly" + curDay).ejCheckBox({ check: true });
                    _rule = this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._dayFullNames[strDate.getDay()];
                    this._rRuleFreq = "WEEKLY";
                    this._appointmentAddWindow.find("#" + this._id + "monthlyrecur,#" + this._id + "yearlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                    this._appointmentAddWindow.find("#" + this._id + "weeklyrecur").css("display", "table-row");
                }
                else if (e.itemId == 2) {
                    this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceMonth");
                    this._appointmentAddWindow.find("#" + this._id + "weeklyrecur,#" + this._id + "yearlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                    this._appointmentAddWindow.find("#" + this._id + "monthlyrecur").css("display", "table-row");
                    this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox({ value: strDate.getDate() });
                    this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList({ selectedItem: strDate.getDay() });
                    this._appointmentAddWindow.find('.monthsrt').ejDropDownList({ selectedItem: this._getWeekIndex(strDate) });
                    this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton({ check: true });
                    //var index = this._getWeekIndex(strDate); 
                    this._rRuleFreq = "MONTHLY";
                    _rule = this._getLocalizedLabels("RecurrenceMonth").toLowerCase();
                }
                else if (e.itemId == 3) {
                    this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceYear");
                    this._appointmentAddWindow.find("#" + this._id + "weeklyrecur,#" + this._id + "monthlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                    this._appointmentAddWindow.find("#" + this._id + "yearlyrecur").css("display", "table-row");
                    this._appointmentAddWindow.find('.yearmonth').ejDropDownList({ selectedItem: strDate.getMonth() });
                    this._appointmentAddWindow.find('.yeardate').ejNumericTextbox({ value: strDate.getDate() });
                    this._appointmentAddWindow.find('.yearsrtday').ejDropDownList({ selectedItem: strDate.getDay() });
                    this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList({ selectedItem: strDate.getMonth() });
                    this._appointmentAddWindow.find('.yearsrt').ejDropDownList({ selectedItem: this._getWeekIndex(strDate) });
                    this._appointmentAddWindow.find('.e-yearrecurday').ejRadioButton({ check: true });
                    this._rRuleFreq = "YEARLY";
                    _rule = this._getLocalizedLabels("RecurrenceYear").toLowerCase();
                }
                else if (e.itemId == 4) {
                    this._rRuleFreq = "WEEKDAYS";
                    this._appointmentAddWindow.find("#" + this._id + "weeklyrecur,#" + this._id + "yearlyrecur,#" + this._id + "dailyrecur,#" + this._id + "repeatEvery,#" + this._id + "monthlyrecur").css("display", "none");
                    _rule = e.selectedText;
                }
                this._appointmentAddWindow.find(".e-recurRule").html(_rule);
            }
        },
        /**
        * Section For generate the dates for all the view of the schedule.
        * @private
        */
        _getRenderDates: function () {
            this._dateRender = []; var firstDay;
            if (this.currentView() === "week" || this.currentView() === "workweek") {
                firstDay = this._firstweekdate();
                for (var i = 0; i < 7; i++) {
                    firstDay.setDate(this._firstweekdate().getDate() + i);
                    this._dateRender.push(new Date(firstDay.getFullYear(), firstDay.getMonth(), firstDay.getDate()).getTime());
                }
                if (this.currentView() === "workweek")
                    this._dateRender = this._dateRender.slice(1, 6);
            }
            else if (this.currentView() === "month") {
                this._dateRender = this._monthdates();
            }
            else {
                this._dateRender.push(this.currentDate());
            }
            return this._dateRender;
        },
        /**
        * Section For render the current date/date period in the header bar.
        * @private
        */
        _getCurrentDateHeaderValue: function () {
            var localizedDay = Globalize.culture(this.model.localization);
            var dateValue, value = this._dateRender;
            if (this.currentView() === "day")
                dateValue = localizedDay.calendar.days.names[new Date(value[0]).getDay()].slice(0, 3) + " " + new Date(value[0]).getDate() + " " + Globalize.format(new Date(value[0]), "MMM") + " " + new Date(value[0]).getFullYear(); //  Globalize.format(new Date(value[0]), "MMMM dd yyyy");
            else if (this.currentView() === "week" || this.currentView() === "workweek") {
                if (new Date(value[0]).getYear() === new Date(value[value.length - 1]).getYear()) {
                    if (new Date(value[0]).getMonth() === new Date(value[value.length - 1]).getMonth()) {
                        dateValue = Globalize.format(new Date(value[0]), "dd") + " - " + Globalize.format(new Date(value[value.length - 1]), "dd MMM yyyy");
                    }
                    else if (new Date(value[0]).getMonth() !== new Date(value[value.length - 1]).getMonth()) {
                        dateValue = this.model.rtl ? Globalize.format(new Date(value[value.length - 1]), "MMM dd") + " - " + Globalize.format(new Date(value[0]), "dd MMM yyyy") :
                           Globalize.format(new Date(value[0]), "dd MMM") + " - " + Globalize.format(new Date(value[value.length - 1]), "dd MMM yyyy");
                    }
                }
                else {
                    dateValue = Globalize.format(new Date(value[0]), "MMM dd yyyy") + " - " + Globalize.format(new Date(value[value.length - 1]), "MMM dd yyyy");
                }
                //  dateValue = new Date(value[0]).getDate() + " " + Globalize.format(new Date(value[0]), "MMM") + (new Date(value[0]).getFullYear) + "/" + " - " + localizedDay.calendar.days.names[new Date(value[value.length - 1]).getDay()].slice(0, 3) + " " + (new Date(value[value.length - 1]).getMonth() + 1) + "/" + new Date(value[value.length - 1]).getDate();
            }
            else {
                dateValue = Globalize.format(this.currentDate(), "MMMM yyyy");
            }
            return dateValue;
        },

        /**
        * Section For find the first day of the week.
        * @private
        */
        _firstweekdate: function () {
            var firstdayofweek = new Date(this.currentDate());
            var day = firstdayofweek.getDay();
            if (day != 0) {
                firstdayofweek.setDate(firstdayofweek.getDate() - day);
            }
            return firstdayofweek;
        },
        /**
        * Section For generate the allday cell counts of the schedule.
        * @private
        */
        _getHeaderAllDayCellsCount: function () {
            var weekcols = Globalize.culture(this.model.localization);
            var temp, count, cols, dateRender = this._dateRender;
            cols = [1];
            count = this._getColumnCount().length; // this.model.currentView === "month" ? dateRender.length / 5 : dateRender.length;
            for (var i = 0; i < count; i++) {
                var curday = (this.currentView() === "month" || this.currentView() === "day") ? weekcols.calendar.days.names[new Date(dateRender[i]).getDay()] : weekcols.calendar.days.names[new Date(dateRender[i]).getDay()].slice(0, 3);
                temp = this.currentView() === "month" ? { currentDay: curday } : this.currentView() === "day" ? { currentDay: curday + " " + Globalize.format(new Date(dateRender[i]), "dd MMMM yyyy") } : { currentDay: curday + " " + new Date(dateRender[i]).getDate() };
                cols.push(temp);
            }
            return cols;
        },
        /**
        * Section For render all day header template.
        * @private
        */
        _renderHeaderAllDayTemplate: function () {
            var headerDiv = ej.buildTag('div', "", {}, { height: "100%", width: "100%" }).css("position", "relative");
            var cols = this._getHeaderAllDayCellsCount();
            var table = ej.buildTag('table.e-table e-headrealldaytable', "", {}, { cellspacing: "0px", cellpadding: "0px" });
            var tbody = ej.buildTag('tbody');
            var headTR = ej.buildTag('tr.e-dateheader');

            var view = this.currentView();
            var html = this.headTemplate.render({ cols: cols, view: view });
            this.aTR.append(ej.buildTag('td').append(headerDiv.append(table.append(tbody.append(headTR.append(html))))));

            if (this.currentView() !== "month") {
                var alldayTR = ej.buildTag('tr');
                var allHtml = this.alldayTemp.render({ cols: cols });
                tbody.append(alldayTR.append(allHtml));
            }
        },
        /**
        * Section For generate the current month dates with previous and next month exceeding dates.
        * @private
        */
        _monthdates: function () {
            var currentDate = new Date(this.currentDate());
            var header = new Date(currentDate.getFullYear(), currentDate.getMonth(), 1);
            var day = header.getDay() + 1;
            header.setDate(header.getDate() - day);
            //nextmonth days.
            var daysInMonth = new Date(currentDate.getFullYear(), currentDate.getMonth() + 1, 0).getDate();
            var daysInNextMonth = (day + daysInMonth) % 7 != 0 ? (7 - (day + daysInMonth) % 7) : 0;
            var rows = parseInt((day + daysInMonth + daysInNextMonth) / 7);
            var dates = [];
            for (var row = 0; row < (rows * 7) ; row++) {
                dates.push(new Date(header.setDate(header.getDate() + 1)));
            }
            return dates;
        },
        /**
        * Section For get the start and end time of the schedule.
        * @private
        */
        _getStartEndTime: function () {
            var timerows, startendTimes = [];
            var localizeAmPm = Globalize.culture(this.model.localization).calendar;
            if (this.currentView() !== "month") {
                for (var i = this.model.startHour; i < this.model.endHour; i++) {
                    if (this.model.timeMode === "12") {
                        var timelist = ((i === this.model.startHour || i === 12) && (localizeAmPm.AM && localizeAmPm.PM)) ? Globalize.format(new Date(new Date().setHours(i)), "hh tt") : Globalize.format(new Date(new Date().setHours(i)), "hh 00");
                        startendTimes.push({ time: timelist, id: i });
                    }
                    else
                        startendTimes.push({ time: (i <= 9 ? "0" + i : i) + " 00" });
                }
            }
            else {
                var row = this._dateRender.length;
                for (var i = 0; i < (row / 7) ; i++) {
                    startendTimes.push(i);
                }
            }
            return startendTimes;
        },
        /**
        * Section For get the work cells count of the schedule.
        * @private
        */
        _getWorkCellsCount: function () {
            var workcells = [];
            if (this.currentView() !== "month") {
                var strEndTime = this._getStartEndTime();
                for (var i = 0; i < (2 * strEndTime.length) ; i++) {
                    workcells.push(i);
                }
            }
            else {
                var row = this._dateRender.length;
                for (var i = 0; i < (row / 7) ; i++) {
                    workcells.push(i);
                }
            }
            return workcells;
        },
        /**
        * Section For render the content area template of the schedule.
        * @private
        */
        _renderContentAreaTemplate: function () {
            var $contentAreaDiv = ej.buildTag('div.e-workareadiv');
            var $contentTable = ej.buildTag('table.e-table e-worktable', "", {}, { height: "100%", cellpadding: "0px", cellspacing: "0px", width: "100%" });
            var $contentBody = ej.buildTag('tbody');
            var $contentTR = ej.buildTag('tr');
            var timeCellClassName = this.currentView() === "month" ? "e-monthleftindent" : "e-workleftindent";
            this.$TimeCellTD = ej.buildTag('td', "", {}, {}).addClass(timeCellClassName);
            var $WorkCellTD = ej.buildTag('td');

            var colspan = [];
            var cols = this.currentView() === "day" ? 1 : this.currentView() === "week" ? 7 : 5;
            for (var i = 0; i < cols; i++) colspan.push(i);

            //time cells
            var timeCellhtml;
            var viewclass = this.currentView() == "month" ? 1 : 0; //"e-monthtimecells" : "e-timecelldivs";

            timeCellhtml = this.timeCellTemplate.render({ timerows: this._getStartEndTime(), cols1: [1, 2], viewclass: viewclass });

            //Work cells
            var $scrollerbarDiv = ej.buildTag('div#' + this._id + "_scroller");
            this.$WorkCellDiv = ej.buildTag('div').css('position', "relative").attr('class', 'e-draggableworkarea').css("height", "100%");
            var workCellhtml;
            if (this.currentView() !== "month")
                workCellhtml = this.workcellTemp.render({ cellrows: this._getWorkCellsCount(), cols1: colspan });
            else {
                //         this.$WorkCellDiv.css("top", "-1px");
                workCellhtml = this.monthcellTemp.render({ cellrows: this._getWorkCellsCount(), cols1: this._getColumnCount() });
            }
            this.wTR.append(ej.buildTag('td').append($contentAreaDiv.append($contentTable.append($contentBody.append($contentTR.append(this.$TimeCellTD.append(timeCellhtml)).append($WorkCellTD.append($scrollerbarDiv.append(this.$WorkCellDiv.append(workCellhtml)))))))));
        },
        /**
        * Section For get the column count of the schedule.
        * @private
        */
        _getColumnCount: function () {
            var weekcells = [];
            var daysCount = this.currentView() === "week" || this.currentView() === "month" ? 7 : this.currentView() === "workweek" ? 5 : 1;
            for (var i = 0; i < daysCount; i++) {
                weekcells.push(i);
            }
            return weekcells;
        },
        /**
        * To configure the dataSource values
        * @private
        */
        _bindAppointmentsData: function () {

            if (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null) {
                if (this.dataSource() instanceof ej.DataManager) {
                    this._initDataSource(); //Code to bind fields from database
                }
                else {
                    this._dataCount = this.dataSource().length;
                    this._dataProcessing(this.dataSource());
                    this._renderAppointmentAll();
                }
            }
        },
        /**
        * To configure the dataSource values to the fields.
        * @private
        */
        _initDataSource: function () {
            var query = this._columnToSelect(this.model.fields);
            var proxy = this;
            var queryPromise = this.dataSource().executeQuery(query);
            queryPromise.done(function (e) {
                //utc timezone conversion
                var tempstarttime, tempendtime, localutc;
                localutc = new Date().getTimezoneOffset();
                var obj = proxy._minToHour(localutc);
                for (var a = 0; a < e.result.length; a++) {
                    tempstarttime = e.result[a].StartTime;
                    tempendtime = e.result[a].EndTime;
                    tempstarttime.setHours(tempstarttime.getHours() - obj.h, tempstarttime.getMinutes() - obj.m);
                    tempendtime.setHours(tempendtime.getHours() - obj.h, tempendtime.getMinutes() - obj.m);
                }
                proxy.model.fields["dataSource"] = e.result;
                proxy._dataManager = ej.DataManager(e.result);
                proxy._dataCount = e.result.length;
                proxy._overlapList = [];
                proxy._dataProcessing(e.result);
                proxy._renderAppointmentAll();
            });
        },
        /**
        * To configure the column values of the table
        * @private
        */
        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "query" && col !== "dataSource")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.dataSource().dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },
        /**
        * Section For sort the appointments based on the appointment id.
        * @private
        */
        _sortAppById: function (appointments) {

            var proxy = this;
            appointments.sort(function (a, b) {
                var x = a[proxy.model.fields["idField"]]; var y = b[proxy.model.fields["idField"]];
                return ((x < y) ? -1 : ((x > y) ? 1 : 0));
            });
            return appointments;
        },
        /**
        * Section For handle the datasource dataprocessing.
        * @private
        */
        _dataProcessing: function (appointments) {
            this._processed = [];
            var appId = this._sortAppById(appointments);
            this._maxId = appId[appId.length - 1][this.model.fields["idField"]] + 1;
            var recurEndDate;
            for (var app = 0; app < appointments.length; app++) {
                this._appointmentProcessing(appointments[app]);
            }
            //this._maxId = null;
            return this._processed;
        },
        /**
        * Section For processing the appointments.
        * @private
        */
        _appointmentProcessing: function (appointment) {
            var day_start = new Date(appointment[this.model.fields["startTimeField"]]);
            this._rRule = {}, _count = 0, _lastDay = false;
            (!ej.isNullOrUndefined(appointment[this.model.fields["recurrenceRuleField"]])) && this.recurrenceRuleSplit(appointment[this.model.fields["recurrenceRuleField"]]);
            if (appointment[this.model.fields["recurrenceField"]] == 1 && ej.isNullOrUndefined(this._rRule.recurEditId)) {
                var byDay = "", _interval = ej.isNullOrUndefined(this._rRule.interval) ? 1 : this._rRule.interval;
                !ej.isNullOrUndefined(this._rRule.weekDays) && (byDay = this._rRule.weekDays.split(','));
                // Find Recurrence End date:
                if (!ej.isNullOrUndefined(this._rRule.until)) {
                    recurEndDate = new Date(this._rRule.until);
                    recurEndDate.setDate(recurEndDate.getDate() + 1);
                }
                else if (!ej.isNullOrUndefined(this._rRule.count))
                    switch (this._rRule.freq) {
                        case "DAILY":
                            recurEndDate = new Date(new Date(day_start).setDate(day_start.getDate() + this._rRule.count * (_interval)));
                            break;
                        case "WEEKLY":
                            recurEndDate = new Date(new Date(day_start).setDate(day_start.getDate() + (((this._rRule.count / byDay.length) * 7)) * (_interval)));
                            break;
                        case "MONTHLY":
                            recurEndDate = new Date(new Date(day_start).setMonth(day_start.getMonth() + this._rRule.count * (_interval)));
                            break;
                        case "YEARLY":
                            recurEndDate = new Date(new Date(day_start).setFullYear(day_start.getFullYear() + this._rRule.count * (_interval)));
                            break;
                    }
                else {
                    recurEndDate = new Date(new Date(this.currentDate()).setDate(day_start.getDate() + 42 * (_interval)));
                }
                switch (this._rRule.freq) {
                    case "WEEKLY":
                        byDay = this._rRule.weekDays.split(',');
                        if (byDay.indexOf(this._dayNames[new Date(day_start).getDay()]) == -1)
                            day_start = this._weeklyDayFind(day_start, byDay);
                        break;
                    case "MONTHLY":
                        if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                            _lastDay = new Date(day_start).getDate() == new Date(day_start.getFullYear(), day_start.getMonth() + 1, 0).getDate() ? true : false;
                            while (new Date(day_start).getDate() != this._rRule.monthDay) {
                                day_start = new Date(day_start.setDate(day_start.getDate() + 1));
                            }
                        }
                        else
                            day_start = new Date((this._getWeekIndex(day_start) + 1) > this._rRule.setPositions && this._rRule.setPositions != -1 ? new Date(day_start.getFullYear(), day_start.getMonth() + 1, 1) : day_start);
                        break;
                    case "YEARLY":
                        if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                            day_start = (day_start.getMonth() + 1 >= this._rRule.month) && day_start.getDate() > this._rRule.monthDay ? new Date(day_start.getFullYear() + 1, this._rRule.month - 1, this._rRule.monthDay) : new Date(day_start.getFullYear(), this._rRule.month - 1, this._rRule.monthDay);
                        }
                        else if (!ej.isNullOrUndefined(this._rRule.setPositions)) {
                            day_start = new Date(day_start.getFullYear(), this._rRule.month - 1, day_start.getDate());
                            day_start = new Date((this._getWeekIndex(day_start) + 1) > this._rRule.setPositions ? new Date(day_start.getFullYear(), day_start.getMonth() + 1, 1) : day_start);
                        }
                        break;
                }

                while (day_start < recurEndDate) {
                    var orgStrTime = new Date(appointment[this.model.fields["startTimeField"]]);
                    var newAppointment = {}, orgEndTime = new Date(appointment[this.model.fields["endTimeField"]]);
                    ((this._rRule.freq == "MONTHLY" || this._rRule.freq == "YEARLY") && !ej.isNullOrUndefined(this._rRule.setPositions)) && (day_start = this._dayOfWeekInMonth(day_start, this._rRule.weekDays, this._rRule.setPositions));
                    var strTime = new Date(day_start.getFullYear(), day_start.getMonth(), day_start.getDate(), orgStrTime.getHours(), orgStrTime.getMinutes(), 0);
                    var endTime = new Date(day_start.getFullYear(), day_start.getMonth(), day_start.getDate(), orgEndTime.getHours(), orgEndTime.getMinutes(), 0);
                    if (!ej.isNullOrUndefined(this._rRule.exDate)) {
                        if (this._rRule.exDate.indexOf(Globalize.format(new Date(day_start), this._pattern.d)) == -1) {
                            newAppointment = this._addAppData(appointment, this._maxId, strTime, endTime, appointment[this.model.fields["idField"]]);
                        }
                    }
                    else
                        newAppointment = this._addAppData(appointment, this._maxId, strTime, endTime, appointment[this.model.fields["idField"]]);
                    switch (this._rRule.freq) {
                        case "DAILY":
                            day_start = new Date(day_start.setDate(day_start.getDate() + _interval));
                            break;
                        case "WEEKLY":
                            var curDay = this._dayNames[new Date(day_start).getDay()];
                            if (byDay.length - 1 != byDay.indexOf(curDay)) {
                                day_start = this._weeklyDayFind(day_start, byDay);
                            }
                            else {
                                day_start = new Date(day_start.setDate(day_start.getDate() + (day_start.getDay() * -1)));
                                day_start = new Date(day_start.setDate(day_start.getDate() + (_interval * 7) - 1));
                                day_start = this._weeklyDayFind(day_start, byDay);
                            }
                            break;
                        case "MONTHLY":
                            day_start = _lastDay ? new Date(day_start.getFullYear(), (day_start.getMonth() + _interval) + 1, 0) :
                            ej.isNullOrUndefined(this._rRule.setPositions) ? new Date(day_start.setMonth(day_start.getMonth() + _interval)) : new Date(day_start.getFullYear(), day_start.getMonth() + _interval, 1);
                            break;
                        case "YEARLY":
                            day_start = new Date(day_start.setFullYear(day_start.getFullYear() + _interval));
                            break;
                    }
                    if ((Globalize.format(new Date(appointment[this.model.fields["startTimeField"]]), this._pattern.d) < Globalize.format(new Date(appointment[this.model.fields["endTimeField"]]), this._pattern.d)) && (((new Date(appointment[this.model.fields["endTimeField"]]) - new Date(appointment[this.model.fields["startTimeField"]])) / 3600000) < 24) && (appointment[this.model.fields["alldayField"]] !== true)) {
                        endTime.setDate(endTime.getDate() + 1);
                        this._multipleDayApp(newAppointment, this._maxId, new Date(newAppointment[this.model.fields["startTimeField"]]), endTime);
                    }
                    else
                        (!$.isEmptyObject(newAppointment)) && this._processed.push(newAppointment);
                    this._maxId++;
                    _count++;
                    if (!ej.isNullOrUndefined(this._rRule.count) && _count == this._rRule.count)
                        break;
                }
            }

            if ((Globalize.format(new Date(appointment[this.model.fields["startTimeField"]]), this._pattern.d) < Globalize.format(new Date(appointment[this.model.fields["endTimeField"]]), this._pattern.d)) && (((new Date(appointment[this.model.fields["endTimeField"]]) - new Date(appointment[this.model.fields["startTimeField"]])) / 3600000) < 24) && (appointment[this.model.fields["alldayField"]] !== true) && appointment[this.model.fields["recurrenceField"]] != 1) {
                this._multipleDayApp(appointment, this._maxId, new Date(appointment[this.model.fields["startTimeField"]]), new Date(appointment[this.model.fields["endTimeField"]]));
            }
            else if (appointment[this.model.fields["recurrenceField"]] == 0 || !ej.isNullOrUndefined(this._rRule.recurEditId)) {
                appointment.ParentId = appointment[this.model.fields["idField"]];
                this._processed.push(appointment);
            }
        },
        /**
        * Section For split the multiple day appointment.
        * @private
        */
        //Split two appointments for multiple day.
        _multipleDayApp: function (app, maxId, startTime, endTime) {
            var day_start = new Date(startTime);
            var nextDay = new Date(new Date(startTime).getFullYear(), new Date(startTime).getMonth(), new Date(startTime).getDate());
            var day_end = new Date(new Date(nextDay).setHours(new Date(nextDay).getHours() + 24));
            day_end = new Date(new Date(day_end).setMinutes(new Date(day_end).getMinutes() - 1));

            while (day_start >= new Date(startTime) && day_start <= day_end && day_end <= new Date(endTime)) {
                appointment = this._addAppData(app, maxId, day_start, day_end, app[this.model.fields["idField"]]);
                this._processed.push(appointment);
                day_start = new Date(new Date(day_end).setMinutes(new Date(day_end).getMinutes() + 1));
                day_end = new Date(app[this.model.fields["endTimeField"]]);
                maxId++;
                this._maxId = maxId;
            }
        },
        /**
        * Section For handle the recurrence appointment delete.
        * @private
        */
        //Delete ocurrence 
        _deleteOcurrence: function (appId, date) {
            var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, appId);
            var _editApp = this._dataManager.executeLocal(query);
            if (!ej.isNullOrUndefined(_editApp) && !ej.isNullOrUndefined(_editApp[0][this.model.fields["recurrenceRuleField"]])) {
                var _rRule = _editApp[0][this.model.fields["recurrenceRuleField"]];
                this.recurrenceRuleSplit(_rRule);
                if (ej.isNullOrUndefined(this._rRule.exDate)) {
                    _editApp[0][this.model.fields["recurrenceRuleField"]] = _rRule + ";EXDATE=" + Globalize.format(new Date(date), this._pattern.d);
                }
                else {
                    var _rules = _rRule.split(';');
                    recurRule = "";
                    for (var i = 0; i < _rules.length; i++) {
                        if (recurRule != "")
                            recurRule += ";";
                        var keyValuePair = _rules[i].replace(/ /g, '').split('=');
                        if (keyValuePair[0] == "EXDATE") {
                            _rules[i] = _rules[i] + "," + Globalize.format(new Date(date), this._pattern.d);
                        }
                        recurRule += _rules[i];
                    }
                    _editApp[0][this.model.fields["recurrenceRuleField"]] = recurRule;
                }
                this._dataManager.update(this.model.fields["idField"], _editApp);
                return _editApp;
            }
        },
        /**
        * Section For get the week index of the month.
        * @private
        */
        //Find week index in a month.
        _getWeekIndex: function (curDate) {
            var index = 0;
            var firstDate = new Date(curDate.getFullYear(), curDate.getMonth(), 1);
            var dayIndex = curDate.getDay();
            if (dayIndex != firstDate.getDay())
                do {
                    firstDate = new Date(firstDate.setDate(firstDate.getDate() + 1));
                } while (dayIndex != firstDate.getDay())
            while (firstDate.getDate() != curDate.getDate()) {
                firstDate = new Date(firstDate.setDate(firstDate.getDate() + 7));
                index++;
            }
            return index;
        },
        /**
        * create the appointment data with JSON object.
        * @private
        */
        //create the appointment data with JSON object.
        _addAppData: function (app, id, startTime, endTime, parentId) {
            var data = {};
            for (var item in this.model.fields) {
                if (item != "dataSource" && item != "query" && item != "tableName")
                    data[this.model.fields[item]] = app[this.model.fields[item]];
            }
            (!ej.isNullOrUndefined(id)) && (data[this.model.fields["idField"]] = id);
            (!ej.isNullOrUndefined(startTime)) && (data[this.model.fields["startTimeField"]] = startTime);
            (!ej.isNullOrUndefined(endTime)) && (data[this.model.fields["endTimeField"]] = endTime);
            (!ej.isNullOrUndefined(parentId)) && (data["ParentId"] = parentId);
            return data;
        },
        /**
        * Section for find the number of week and the day name.
        * @private
        */
        // find the number week with day name.
        _dayOfWeekInMonth: function (strDate, dayName, weekIndex) {
            var dayIndex = this._dayNames.indexOf(dayName);
            var curDate = weekIndex == -1 ? new Date(strDate.getFullYear(), strDate.getMonth() + 1, 0) : new Date(strDate.getFullYear(), strDate.getMonth(), 1);
            while (curDate.getDay() != dayIndex) {
                curDate = new Date(weekIndex == -1 ? curDate.setDate(curDate.getDate() - 1) : curDate.setDate(curDate.getDate() + 1));
            }
            return new Date(weekIndex == 1 || weekIndex == -1 ? curDate : curDate.setDate(curDate.getDate() + ((weekIndex - 1) * 7)));
        },
        /**
        * Section for find the day name.
        * @private
        */
        // Find the day name 
        _weeklyDayFind: function (strDate, days) {
            var newdayStr = strDate;
            do {
                newdayStr = new Date(newdayStr.setDate(newdayStr.getDate() + 1));
                curDay = this._dayNames[new Date(newdayStr).getDay()];
            } while (days.indexOf(curDay) == -1);
            return newdayStr;
        },
        /**
        * Section for render the appointments.
        * @private
        */
        _renderAppointments: function () {
            var dateCount = this._dateRender;
            this._tempapp = [];
            this.cellwidth = this.element.find(".e-workcells")[0].offsetWidth;
            //var cellHeight = this.element.find(".e-workcells")[0].offsetHeight;
            //var timeScaleRowCount = (60 / 30);
            var appointments = this.dataSource();
            this.leftPosition = 0;
            //var startHour = parseInt(this.model.startHour) < parseInt(this.model.endHour) ? parseInt(this.model.startHour) : 0;
            //var endHour = parseInt(this.model.startHour) < parseInt(this.model.endHour) ? parseInt(this.model.endHour) : 24;
            if (this.model.timeZone != null) {
                var timezone = this.model.timeZone.split(" ")[1].split(":");
                var hour = parseInt(timezone[0]);
                var mins = parseInt(timezone[1]);
                dateMinutes = new Date().getTimezoneOffset();
                var obj = this._minToHour(dateMinutes + ((hour * 60) + mins));
                for (var a = 0; a < appointments.length; a++) {
                    temp = $.extend({}, appointments[a]);
                    sHour = new Date(temp.StartTime);
                    eHour = new Date(temp.EndTime);
                    temp.StartTime = new Date(sHour.setHours(sHour.getHours() + obj.h, sHour.getMinutes() + obj.m));
                    temp.EndTime = new Date(eHour.setHours(eHour.getHours() + obj.h, eHour.getMinutes() + obj.m));
                    this._tempapp.push(temp);
                }
                this._dataProcessing(this._tempapp);
            }
            //else
            //    this._dataProcessing(this.dataSource());
            var appointmentsList = this._processed;

            for (var day = 0; day < dateCount.length; day++) {
                this._appList = [];
                for (var app = 0; app < appointmentsList.length; app++) {
                    if ((Globalize.format(new Date(appointmentsList[app][this.model.fields["startTimeField"]]), this._pattern.d) <= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.fields["endTimeField"]]), this._pattern.d) >= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.fields["startTimeField"]]), this._pattern.d) == Globalize.format(new Date(appointmentsList[app][this.model.fields["endTimeField"]]), this._pattern.d)) && (appointmentsList[app][this.model.fields["alldayField"]] === false) && (Globalize.format(new Date(appointmentsList[app][this.model.fields["startTimeField"]]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.fields["endTimeField"]]), this._pattern.d))) {
                        this._appList.push(appointmentsList[app]);
                    }

                }
                for (var i = 0; i < this._appList.length; i++) {
                    this._renderApp(this._appList[i], day);
                }
            }
        },
        /**
        * Section for handle the utc drop down.
        * @private
        */
        _utc: function (e) {
            if (this.element.hasClass("startutc"))
                $(".endutc").val(e.selectedText);
            else
                $(".startutc").val(e.selectedText);
        },
        /**
        * Section to create handle for appointments in row wise.
        * @private
        */
        _rowHandlesOver: function (e) {
            $(e.target).addClass("e-rowcursor");
        },
        /**
        * Section to create handle for appointments in column wise.
        * @private
        */
        _colHandlesOver: function (e) {
            $(e.target).addClass("e-columncursor");
        },
        /**
        * Section for render the appointments from dataSource.
        * @private
        */
        _renderApp: function (record, day) {
            var cellHeight = this.element.find(".e-workcells")[0].offsetHeight;
            var timeScaleRowCount = (60 / 30);
            var startHour = parseInt(this.model.startHour) < parseInt(this.model.endHour) ? parseInt(this.model.startHour) : 0;
            var endHour = parseInt(this.model.startHour) < parseInt(this.model.endHour) ? parseInt(this.model.endHour) : 24;
            var dateCount = this._dateRender;
            var appWidth, appHeight;
            var rStartTime = new Date(record[this.model.fields["startTimeField"]]);
            var rEndTime = new Date(record[this.model.fields["endTimeField"]]);

            if (rStartTime <= rEndTime) {
                var startTime = ((new Date(rStartTime).getHours()) < startHour) ? (new Date(rStartTime).getHours() + (-new Date(rStartTime).getHours()) + (-new Date(rStartTime).getMinutes())) : (new Date(rStartTime).getHours() + (-startHour));
                var endTime = ((new Date(rEndTime).getHours()) > endHour) ? (new Date(rEndTime).getHours() + (-new Date(rEndTime).getHours() + (endHour - startHour)) + (-new Date(rEndTime).getMinutes())) : (new Date(rEndTime).getHours() + (-startHour));
                endTime = (new Date(rEndTime).getHours() == endHour) ? (new Date(endTime).getMinutes() + (-new Date(rEndTime).getMinutes())) : endTime;

                if (endHour <= rEndTime.getHours()) {
                    var endDate = new Date(rEndTime.setHours(endHour, 00, 00));
                    appHeight = (parseFloat(((new Date(endDate).getHours() * 60) + (new Date(endDate).getMinutes())) - ((new Date(rStartTime).getHours() * 60) + (new Date(rStartTime).getMinutes()))) * ((cellHeight) * timeScaleRowCount) / 60) - 3;
                }
                else
                    appHeight = (parseFloat(((new Date(rEndTime).getHours() * 60) + (new Date(rEndTime).getMinutes())) - ((new Date(rStartTime).getHours() * 60) + (new Date(rStartTime).getMinutes()))) * ((cellHeight) * timeScaleRowCount) / 60) - 2;
                var topPosition = parseFloat((new Date(rStartTime).getMinutes())) * (cellHeight) * timeScaleRowCount / 60;

                if (this.model.startHour <= new Date(rStartTime).getHours()) {

                    var overlapList = this._overlapApp(record[this.model.fields["idField"]], record[this.model.fields["startTimeField"]], record[this.model.fields["endTimeField"]]);
                    var proxy = this;
                    if (overlapList.length > 1) {
                        //overlapList.push(record);
                        overlapList.sort(function (a, b) {
                            var d1 = new Date(a[proxy.model.fields["startTimeField"]]);
                            var d2 = new Date(b[proxy.model.fields["startTimeField"]]);
                            return d1 - d2;
                        });
                        var index = overlapList.indexOf(record);
                        appWidth = (100 / overlapList.length) + 2;
                        this.currentView() === "day" ? appWidth -= 7 : appWidth;
                        this.leftPosition = ((100 / overlapList.length) * (index)) + "%";
                    }
                    else {
                        this.leftPosition = "0";
                        appWidth = 95;
                    }

                    appWidth = (this.currentView() === "week" || this.currentView() === "workweek") ? (appWidth - 10) + "%" : appWidth + "%";

                    //user-defined Template data rendering
                    var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                    var userTempHtml = this._getUserAppointmentTemplate(record);
                    var startTimeDisp = this.model.timeMode == "12" ? Globalize.format(new Date(record[this.model.fields["startTimeField"]]), this._pattern.t) : Globalize.format(new Date(record[this.model.fields["startTimeField"]]), "HH:mm");
                    var endTimeDisp = this.model.timeMode == "12" ? Globalize.format(new Date(record[this.model.fields["endTimeField"]]), this._pattern.t) : Globalize.format(new Date(record[this.model.fields["endTimeField"]]), "HH:mm");

                    var category = "e-" + record[this.model.fields["categorizeField"]];
                    var timeCellhtml = this.appTemplate.render({ id: record[this.model.fields["idField"]], appHeight: appHeight, rtl: this.model.rtl, subject: record[this.model.fields["subjectField"]], startTime: startTimeDisp, endTime: endTimeDisp, left: this.leftPosition, top: topPosition, appWidth: appWidth, category: category, userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-appointment" });

                    $(this.element.find("div.e-appointwrapper")[(startTime * 2 * dateCount.length) + day]).append(timeCellhtml);
                }
                //                $(this.element.find("td.e-workcells")[(startTime * 2 * dateCount.length) + day]).addClass("icon-AddGrad");
            }
        },
        /**
        * Section for get the appointments template from user.
        * @private
        */
        _getUserAppointmentTemplate: function (record) {
            var userTemp = $.templates($(this.model.appointmentTemplateId).html());
            var userTempHtml = userTemp.render(record);
            return userTempHtml;
        },
        /**
        * Section for handle the overlap appointments.
        * @private
        */
        _overlapApp: function (id, appStartTime, appEndTime) {
            this.overlapList = [];
            for (var i = 0; i < this._appList.length; i++) {

                if ((new Date(this._appList[i][this.model.fields["endTimeField"]]) > new Date(appStartTime)) && (new Date(this._appList[i][this.model.fields["startTimeField"]]) < new Date(appEndTime))
                && (Globalize.format(new Date(this._appList[i][this.model.fields["startTimeField"]]), this._pattern.d) === Globalize.format(new Date(appStartTime), this._pattern.d))) {
                    this.overlapList.push(this._appList[i]);
                }
            }
            return this.overlapList;
        },
        /**
        * Section for render allday appointments of the schedule.
        * @private
        */
        _renderAllDayAppointments: function () {
            var appHeight = 16;
            var appWidth = this.cellwidth - 20;
            var renderDate = this._dateRender;
            var level = 1; this.level = 0;
            for (var day = 0; day < renderDate.length; day++) {
                for (var i = 0; i < this._processed.length; i++) {
                    if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(this._processed[i][this.model.fields["startTimeField"]]), this._pattern.d)) &&
                    (this._processed[i][this.model.fields["alldayField"]] === true || (Globalize.format(new Date(this._processed[i][this.model.fields["startTimeField"]]), this._pattern.d) < Globalize.format(new Date(this._processed[i][this.model.fields["endTimeField"]]), this._pattern.d)))) {
                        //level = this._getOverlapCount(this._processed[i], day, renderDate);
                        this._setAllDayPos(this._processed[i], day);
                    }
                }
            }
        },
        /**
        * Section for get the overlap appointments count of the schedule.
        * @private
        */
        _getOverlapCount: function (record, day, dateRender) {
            var len = dateRender.length;
            var appointments = this._processed;
            var filterData = [];
            for (var i = 0; i < appointments.length; i++) {
                if ((Globalize.format(new Date(appointments[i][this.model.fields["startTimeField"]]), this._pattern.d) <= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
            (Globalize.format(new Date(appointments[i][this.model.fields["endTimeField"]]), this._pattern.d) >= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
            (appointments[i][this.model.fields["alldayField"]] == true || (Globalize.format(new Date(appointments[i][this.model.fields["startTimeField"]]), this._pattern.d) < Globalize.format(new Date(appointments[i][this.model.fields["endTimeField"]]), this._pattern.d))) &&
            (Globalize.format(new Date(appointments[i][this.model.fields["startTimeField"]]), this._pattern.d) <= Globalize.format(new Date(appointments[i][this.model.fields["endTimeField"]]), this._pattern.d))) {
                    filterData.push(appointments[i]);
                }
            }
            var proxy = this;
            filterData.sort(function (a, b) {
                var d1 = new Date(a[proxy.model.fields["startTimeField"]]);
                var d2 = new Date(b[proxy.model.fields["startTimeField"]]);
                return d1 - d2;
            });
            var index = filterData.length > 0 ? filterData.indexOf(record) : 0;
            return index == -1 ? 0 : index;
        },
        /**
        * Section for handle the allday appointments position.
        * @private
        */
        _setAllDayPos: function (record, day) {
            var level = this._getOverlapCount(record, day, this._dateRender);
            var start = Globalize.format(new Date(record[this.model.fields["startTimeField"]]), this._pattern.d);
            var end = Globalize.format(new Date(record[this.model.fields["endTimeField"]]), this._pattern.d);
            if (new Date(start) <= new Date(end)) {
                var dateRender = this._dateRender;
                var appWidth, left = 0, top = 15, appHeight = 15;
                var r_start = Globalize.format(new Date(dateRender[0]), this._pattern.d);
                var r_end = Globalize.format(new Date(dateRender[dateRender.length - 1]), this._pattern.d);
                end = new Date(end) > new Date(r_end) ? r_end : end;
                var noOfDays = Math.ceil((new Date(start) - new Date(r_start)) / (1000 * 3600) / 24);
                var diffInDays = Math.ceil((new Date(end) - new Date(start)) / (1000 * 3600) / 24) + 1;
                var cellWidth = this.element.find(".e-alldaycells").width();
                left = this.currentView() !== "day" ? (noOfDays * cellWidth) + this.element.find(".e-settingsindent").width() + new Date(start).getDay() : (noOfDays * cellWidth) + this.element.find(".e-settingsindent").width();
                left = this.currentView() !== "day" ? (new Date(start).getDay() > 2 && new Date(start).getDay() < 7) ? left + new Date(start).getDay() * ((this._browserInfo.name == "msie" && this._browserInfo.version == 8) ? 3 : 1) : left + 1 : left;
                var colRightSpace = this.currentView() == "day" ? 20 : this.currentView() === "week" ? 17 : 24;
                appWidth = (diffInDays * cellWidth) - colRightSpace;
                top = (level * top) + (3 * (level + 1)) + this.element.find(".e-headercells").height() + 3;

                //user-defined Template data rendering
                var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                var userTempHtml = this._getUserAppointmentTemplate(record);

                var category = "e-" + record[this.model.fields["categorizeField"]];
                var alldayCellhtml = this.appTemplate.render({ id: record[this.model.fields["idField"]], top: top, appHeight: appHeight, rtl: this.model.rtl, subject: record[this.model.fields["subjectField"]], startTime: Globalize.format(new Date(record[this.model.fields["startTimeField"]]), this._pattern.t), endTime: Globalize.format(new Date(record[this.model.fields["endTimeField"]]), this._pattern.t), appWidth: appWidth + "px", allday: true, left: left + "px", category: category, userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-alldayappointment" });
                if (this.level < level) {
                    var cellHeight = (level + 1) * (appHeight + 3) + 3;
                    this.element.find(".e-alldaycells").css('height', cellHeight + "px");
                }
                this.element.find(".e-headrealldaytable").parent().append(alldayCellhtml);
                this.level = this.level < level ? level : this.level;
            }
        },
        /**
        * Section for render the appointments in the month view of schedule.
        * @private
        */
        _renderMonthAppointment: function () {
            var filterData = [];
            var renderDate = this._dateRender;
            // var appointments = this.model.fields["dataSource"];
            var appointments = this._processed; //this._dataProcessing(this.dataSource());
            for (var day = 0; day < renderDate.length; day++) {
                var filterData = [];
                for (var app = 0; app < appointments.length; app++) {
                    if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(appointments[app][this.model.fields["startTimeField"]]), this._pattern.d)) &&
                (Globalize.format(new Date(renderDate[day]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.fields["endTimeField"]]), this._pattern.d)) &&
                (Globalize.format(new Date(appointments[app][this.model.fields["startTimeField"]]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.fields["endTimeField"]]), this._pattern.d))) {
                        filterData.push(appointments[app]);
                    }
                }
                for (var i = 0; i < filterData.length; i++) {
                    var level = this._getIndex(day, filterData[i], renderDate, appointments);
                    this._renderAppointmentMonth(filterData[i], day, renderDate, level);
                }
            }
        },
        /**
        * Wiring the events to resize appointments .
        * @private
        */
        _wireResizeEvents: function () {
            if (this.model.allowResizing) {
                this._on(this.element.find("div.e-lefthandle,div.e-righthandle,div.e-tophandle,div.e-bottomhandle"), "mousedown", $.proxy(this._appointmentResizeStart, this));
                this._on(this.element.find("div.e-lefthandle,div.e-righthandle"), "mouseover", $.proxy(this._colHandlesOver, this));
                this._on(this.element.find("div.e-tophandle,div.e-bottomhandle"), "mouseover", $.proxy(this._rowHandlesOver, this));
            }
        },
        /**
        * Section for get the index to render the appointment.
        * @private
        */
        _getIndex: function (day, record, dateRender, appointments) {
            var proxy = this;
            appointments.sort(function (a, b) {
                var d1 = new Date(a[proxy.model.fields["startTimeField"]]);
                var d2 = new Date(b[proxy.model.fields["startTimeField"]]);
                return d1 - d2;
            });
            var len = dateRender.length;
            var data = [];
            for (var app = 0; app < appointments.length; app++) {
                if ((Globalize.format(new Date(appointments[app][this.model.fields["startTimeField"]]), this._pattern.d) <= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
            (Globalize.format(new Date(appointments[app][this.model.fields["endTimeField"]]), this._pattern.d) >= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
            (Globalize.format(new Date(appointments[app][this.model.fields["startTimeField"]]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.fields["endTimeField"]]), this._pattern.d))) {
                    data.push(appointments[app]);
                }
            }

            var index = data.length > 0 ? data.indexOf(record) : 0;
            return index;
        },
        /**
        * Render the appointments in the month view.
        * @private
        */
        _renderAppointmentMonth: function (record, day, renderDate, overlapCount) {
            var startTime = new Date(record[this.model.fields["startTimeField"]]);
            var endTime = new Date(record[this.model.fields["endTimeField"]]);
            if (startTime <= endTime) {
                var appHeight = 18;
                var appWidth, topPosition = 0;
                this.leftPosition = 0;
                this.CellWidth = this.element.find(".e-monthcells").width();
                this.CellHeight = this.element.find(".e-monthcells").height();
                var rowPosition = Math.ceil((day + 1) / 7);

                if (Globalize.format(startTime, this._pattern.d) != Globalize.format(endTime, this._pattern.d)) {
                    var r_start = new Date(renderDate[0]);
                    var r_end = new Date(renderDate[renderDate.length - 1]);
                    var start = (Globalize.format(startTime, this._pattern.d));
                    var end = (Globalize.format(endTime, this._pattern.d));
                    end = new Date(end) > new Date(r_end) ? r_end : end;
                    var maxDays = 7 - (new Date(start).getDay());
                    var diffInDays = Math.ceil((new Date(end) - new Date(start)) / (1000 * 3600) / 24) + 1;
                    diffInDays = diffInDays >= maxDays ? maxDays : diffInDays;
                    appWidth = (diffInDays * this.CellWidth) - 15 - diffInDays;
                }
                else
                    appWidth = (this.CellWidth * 6) / 7;
                var cellId = (7 * (rowPosition - 1) - 1) + (new Date(startTime).getDay() + 1);

                topPosition = (overlapCount * appHeight);
                this.leftPosition = 0;

                if ((overlapCount * (appHeight + 21)) < (this.CellHeight)) {
                    appHeight -= 2;
                    if (topPosition > 1) topPosition -= 1;
                    if (this.leftPosition > 1) this.leftPosition -= 1;
                    //user-defined Template data rendering
                    var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                    var userTempHtml = this._getUserAppointmentTemplate(record);
                    var category = "e-" + record[this.model.fields["categorizeField"]];
                    var timeCellhtml = this.appTemplate.render({ id: record[this.model.fields["idField"]], appHeight: appHeight, rtl: this.model.rtl, subject: record[this.model.fields["subjectField"]], startTime: Globalize.format(new Date(record[this.model.fields["startTimeField"]]), this._pattern.t), endTime: Globalize.format(new Date(record[this.model.fields["endTimeField"]]), this._pattern.t), left: this.leftPosition + "px", top: topPosition, appWidth: appWidth + "px", category: category, userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-monthappointment" });
                    $(this.element.find("div.e-appointwrapper")[cellId]).append(timeCellhtml);
                    //    this.element.find("div.e-draggableworkarea").append(timeCellhtml);
                    timeCellhtml = "";
                }
                else {
                    if ($("#" + this._id + "_" + Globalize.format(new Date(record[this.model.fields["startTimeField"]]), "MM_dd_yyyy")).length == 0) {
                        $(this.element.find("div.e-appointwrapper")[cellId]).append('<div id="' + this._id + "_" + Globalize.format(new Date(record[this.model.fields["startTimeField"]]), "MM_dd_yyyy") + '" class="e-icon e-appsoverflow"></div>');
                        this.element.find("#" + this._id + "_" + Globalize.format(new Date(record[this.model.fields["startTimeField"]]), "MM_dd_yyyy")).css(this.model.rtl ? "right" : "left", (this.CellWidth + this.leftPosition - 25) + "px").css("top", topPosition + "px");
                    }
                }
            }
        },
        /**
        * Section for handle the quick window close.
        * @private
        */
        _quickWindowClose: function () {
            this._quickAppointWindow.ejDialog("close");
            this._processFocus();
        },
        /**
        * Section for create the appointment window.
        * @private
        */
        _appointmentWindow: function (e) {
            var event = { starttime: new Date(this.cur_StartTime), endtime: this.cur_EndTime, target: e };
            if (this._trigger("cellDoubleClick", event))
                return false;
            var startTime, endTime, _target = !ej.isNullOrUndefined(e.target) ? $(e.target) : $(e);
            if (_target.hasClass("e-workcells") || _target.hasClass("e-alldaycells") || _target.hasClass("e-monthcells") || this._cellClick || _target.hasClass("e-detailedapp")) {
                this._currentAction = ej.Schedule.actions.Add;
                this._appointmentAddWindow.find(".e-recurwindow").css("display", "none");
                this._appointmentAddWindow.find(".e-appwindow").css("display", "block");
                if (!this._cellClick) {
                    var index = _target.hasClass("e-workcells") ? _target.index() : _target.hasClass("e-alldaycells") ? _target.index() - 1 : (_target.parent().index() * 7) + _target.index();
                    this.currentDate(new Date(this._dateRender[index]));

                    if (_target.hasClass("e-workcells")) {
                        var time = this.model.startHour + (_target.parent().index() / 2);
                        var timemin = time.toString().split(".");
                        startTime = new Date(this.currentDate()).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                        endTime = new Date(new Date(startTime).setMinutes(new Date(startTime).getMinutes() + 30));
                    }
                    else if (_target.hasClass("e-monthcells")) {
                        startTime = new Date(new Date(this.currentDate()).setHours(9));
                        startTime = new Date(startTime).setMinutes(00); //.setHours(00)).setMinutes(00);
                        endTime = new Date(new Date(startTime).setMinutes(30)); //.setHours(23)).setMinutes(59));
                    }
                    else {
                        startTime = new Date(this.currentDate()); //.setHours(00)).setMinutes(00);
                        endTime = new Date(startTime); //.setHours(23)).setMinutes(59));
                        this._appointmentAddWindow.find(".allday ").ejCheckBox({ check: true });
                    }
                } else {
                    startTime = new Date(this.cur_StartTime);
                    endTime = new Date(this.cur_EndTime);
                }
                if (this._trigger("appointmentWindowOpen", event))
                    return false;
                this._appointmentAddWindow.find("#" + this._id + "subject").val(this.model.showQuickWindow ? this._quickAppointWindow.find(".subject").val() : "");
                this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.d));
                this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.d));
                this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.t));
                this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(endTime), this._pattern.t));
                ($(e.currentTarget).hasClass("e-alldaycells") || this.currentView() == "month") && this._appointmentAddWindow.find(".allday").ejCheckBox({ check: true });
                this._alldayCheck();
                if (this.model.timeZone == null) {
                    document.getElementById('startutc').value = "UTC " + Globalize.format(new Date(), "zzz");
                    document.getElementById('endutc').value = "UTC " + Globalize.format(new Date(), "zzz");
                }
                else {
                    document.getElementById('startutc').value = this.model.timeZone;
                    document.getElementById('endutc').value = this.model.timeZone;
                }
                this._appointmentAddWindow.ejDialog("open");
                this._appointmentAddWindow.find(".subject").focus();
                this._cellClick = false;
            }
            else {
                var target = (this._editKey) ? $(e) : _target.hasClass("e-appointment") || _target.hasClass("e-monthappointment") || _target.hasClass("e-alldayappointment") ? _target : _target.parents('.e-monthappointment,.e-appointment,.e-alldayappointment');
                if (!ej.isNullOrUndefined(target[0].id)) {
                    this._currentAction = ej.Schedule.actions.Save;
                    this._appId = parseInt(target[0].id.split("_")[1]);
                    var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, this._appId);
                    var templist = new ej.DataManager(this._processed).executeLocal(query);
                    if (templist[0][this.model.fields["recurrenceField"]] == 1) {
                        this._parentId = templist[0]["ParentId"];
                        this._recurEditWindow.find(".e-editonly").ejRadioButton({ check: true });
                        this._recurEditWindow.ejDialog("open");
                        this._recurEditWindow.focus();
                    }
                    else {
                        this._showAppointmentDetails(this._appId);
                    }
                    this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
                }
                this._editKey = false;
            }
            this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close");
        },
        /**
        * Section for toggle the appointment window.
        * @private
        */
        _editRecurrence: function (args) {
            this._appointmentAddWindow.find(".e-appwindow").toggle();
            this._appointmentAddWindow.find(".e-recurwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 300);
        },
        /**
        * Section for show all the required details to the window while editing.
        * @private
        */
        _showAppointmentDetails: function (appId, editOcurrence) {
            var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, parseInt(appId));
            var templist = editOcurrence == true ? new ej.DataManager(this._processed).executeLocal(query) : this._dataManager.executeLocal(query);
            this._appointmentAddWindow.find('.subject').val(templist[0][this.model.fields["subjectField"]]);
            this._appointmentAddWindow.find('.id').val(templist[0][this.model.fields["idField"]]);
            this._appointmentAddWindow.find('.e-description').val(templist[0][this.model.fields["descriptionField"]] ? templist[0][this.model.fields["descriptionField"]] : "");
            var temp = $.extend({}, templist[0]);
            if (this.model.timeZone != null && !editOcurrence) {
                var timezone = this.model.timeZone.split(" ")[1].split(":");
                var hour = parseInt(timezone[0]);
                var mins = parseInt(timezone[1]);
                dateMinutes = new Date().getTimezoneOffset();
                var obj = this._minToHour(dateMinutes + ((hour * 60) + mins));
                var newStartDate = new Date(temp.StartTime); var newEndDate = new Date(temp.EndTime);
                temp.StartTime = new Date(newStartDate.setHours(newStartDate.getHours() + obj.h, newStartDate.getMinutes() + obj.m));
                temp.EndTime = new Date(newEndDate.setHours(newEndDate.getHours() + obj.h, newEndDate.getMinutes() + obj.m));
            }
            this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(new Date(temp[this.model.fields["startTimeField"]]), this._pattern.d));
            this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(new Date(temp[this.model.fields["endTimeField"]]), this._pattern.d));
            this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(temp[this.model.fields["startTimeField"]]), this._pattern.t));
            this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(temp[this.model.fields["endTimeField"]]), this._pattern.t));
            if (this.model.timeZone == null) {
                this._appointmentAddWindow.find("#startutc").val("UTC " + Globalize.format(new Date(temp[this.model.fields["startTimeField"]]), "zzz"));
                this._appointmentAddWindow.find("#endutc").val("UTC " + Globalize.format(new Date(temp[this.model.fields["endTimeField"]]), "zzz"));
            }
            else {
                this._appointmentAddWindow.find("#startutc").val(this.model.timeZone);
                this._appointmentAddWindow.find("#endutc").val(this.model.timeZone);
            }
            templist[0][this.model.fields["alldayField"]] == 1 && (this._appointmentAddWindow.find(".allday ").ejCheckBox({ check: true }));
            if (templist[0][this.model.fields["recurrenceField"]] == 1) {
                if (editOcurrence == true) {
                    this._appointmentAddWindow.find(".recurrence").ejCheckBox({ check: false, enabled: false });
                    this._appointmentAddWindow.find(".e-repeatlbl").addClass('e-disable');
                    this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "none");
                    this._appointmentAddWindow.find(".e-recurFinalRule").html("");
                }
                else {
                    this.recurrenceRuleSplit(templist[0][this.model.fields["recurrenceRuleField"]]);
                    this._appointmentAddWindow.find(".recurstartdate").ejDatePicker({ value: Globalize.format(new Date(temp[this.model.fields["startTimeField"]]), this._pattern.d) });
                    this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "table-row");
                    this._appointmentAddWindow.find(".e-recurFinalRule").html(this._showRecurrenceFields(appId));
                    this._appointmentAddWindow.find(".recurrence").ejCheckBox({ check: true });
                    this._appointmentAddWindow.find(".e-recuredit").css("display", "inline");
                }
            }
            var data = { appointment: templist[0], edit: editOcurrence };
            if (this._trigger("appointmentWindowOpen", data))
                return false;
            this._alldayCheck();
            this._appointmentAddWindow.ejDialog("open");
            this._appointmentAddWindow.find(".subject").focus();
        },
        /**
        * Section for show the recurrence details in the recurrence window.
        * @private
        */
        _showRecurrenceFields: function (appId) {
            var _recurRule = "", _recurOpt = "", _recurType = this._appointmentAddWindow.find('.recurtype');
            _recurRule = ej.isNullOrUndefined(this._rRule.interval) ? this._getLocalizedLabels("Every") + " " : this._getLocalizedLabels("Every") + " " + this._rRule.interval + " ";
            switch (this._rRule.freq) {
                case "DAILY":
                    _recurType.ejDropDownList({ selectedItem: 0 });
                    _recurRule += this._getLocalizedLabels("RecurrenceDay").toLowerCase();
                    break;
                case "WEEKLY":
                    _recurType.ejDropDownList({ selectedItem: 1 });
                    this._appointmentAddWindow.find('.weekdays').ejCheckBox({ check: false });
                    var _days = this._rRule.weekDays.split(',');
                    for (var i = 0; i < _days.length; i++) {
                        this._appointmentAddWindow.find('.e-weekly' + _days[i]).ejCheckBox({ check: true });
                        _recurOpt += this._dayShortNames[this._dayNames.indexOf(_days[i])] + (i == _days.length - 1 ? "" : ", ");
                    }
                    _recurRule += this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                    break;
                case "MONTHLY":
                    _recurType.ejDropDownList({ selectedItem: 2 });
                    if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                        this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton({ check: true });
                        this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox({ value: parseInt(this._rRule.monthDay) });
                        _recurOpt = parseInt(this._rRule.monthDay);
                    }
                    else {
                        this._appointmentAddWindow.find('.e-monthposition').ejRadioButton({ check: true });
                        this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList({ selectedItem: this._dayNames.indexOf(this._rRule.weekDays) });
                        this._appointmentAddWindow.find('.monthsrt').ejDropDownList({ selectedItem: this._rRule.setPositions - 1 });
                        _recurOpt = this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[this._dayNames.indexOf(this._rRule.weekDays)];
                    }
                    _recurRule += this._getLocalizedLabels("RecurrenceMonth").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                    break;
                case "YEARLY":
                    _recurType.ejDropDownList({ selectedItem: 3 });
                    if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                        this._appointmentAddWindow.find('.e-yearrecurday').ejRadioButton({ check: true });
                        this._appointmentAddWindow.find('.yearmonth').ejDropDownList({ selectedItem: this._rRule.month - 1 });
                        this._appointmentAddWindow.find('.yeardate').ejNumericTextbox({ value: parseInt(this._rRule.monthDay) });
                        _recurOpt = this._monthNames[this._rRule.month - 1] + " " + parseInt(this._rRule.monthDay);
                    }
                    else {
                        this._appointmentAddWindow.find('.e-yearrecurposi').ejRadioButton({ check: true });
                        this._appointmentAddWindow.find('.yearsrtday').ejDropDownList({ selectedItem: this._dayNames.indexOf(this._rRule.weekDays) });
                        this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList({ selectedItem: this._rRule.month - 1 });
                        this._appointmentAddWindow.find('.yearsrt').ejDropDownList({ selectedItem: this._rRule.setPositions - 1 });
                        _recurOpt = this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[this._dayNames.indexOf(this._rRule.weekDays)] + " " + this._monthNames[this._rRule.month - 1];
                    }
                    _recurRule += this._getLocalizedLabels("RecurrenceYear").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                    break;
            }
            // Recurrence interavel
            this._appointmentAddWindow.find('.e-recurevery').ejNumericTextbox({ value: ej.isNullOrUndefined(this._rRule.interval) ? 1 : this._rRule.interval });

            //Recurrence end 
            if (!ej.isNullOrUndefined(this._rRule.count)) {
                this._appointmentAddWindow.find(".e-recurafter").ejRadioButton({ check: true });
                this._appointmentAddWindow.find('.e-recurcount').ejNumericTextbox({ value: this._rRule.count });
                _recurRule += ", " + this._rRule.count + this._getLocalizedLabels("Times").toLowerCase();
            }
            else if (!ej.isNullOrUndefined(this._rRule.until)) {
                this._appointmentAddWindow.find(".e-recuruntil").ejRadioButton({ check: true });
                this._appointmentAddWindow.find('.e-until').ejDatePicker({ value: new Date(this._rRule.until) });
                _recurRule += " " + this._getLocalizedLabels("Until").toLowerCase() + " " + Globalize.format(new Date(this._rRule.until), this._pattern.d);
            }
            else
                this._appointmentAddWindow.find(".e-recurnoend").ejRadioButton({ check: true });

            return _recurRule;
        },
        /**
        * Section for handle the closing of recurrence window.
        * @private
        */
        _closerecur: function (args) {
            if (args.model.text == "Cancel") {
                this._appointmentAddWindow.find(".e-recurFinalRule").html() == "" && this._appointmentAddWindow.find("#" + this._id + "recurrence").ejCheckBox({ check: false });
            }
            else {
                var interval, monthDate, endRecur = "", recurOption = "";
                interval = this._findInterval(this._appointmentAddWindow.find('.e-recurevery'));
                switch (this._rRuleFreq) {
                    case "DAILY":
                        this._recRule = "FREQ=" + this._rRuleFreq + ";INTERVAL=" + interval;
                        break;
                    case "WEEKDAYS":
                        this._recRule = "FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR";
                        break;
                    case "WEEKLY":
                        var _weekChkList = this._appointmentAddWindow.find("input[name='weekdays']");
                        var _weekRule = "", _dayName = "", _byDay = "";
                        for (var i = 0; i < _weekChkList.length; i++) {
                            $(_weekChkList[i]).ejCheckBox("option", "check") == true ? _weekRule += (_weekRule != "" ? "," : "") + this._dayNames[i].toUpperCase() : "";
                            $(_weekChkList[i]).ejCheckBox("option", "check") == true ? _dayName += (_dayName != "" ? ", " : "") + this._dayShortNames[i] : "";
                        }
                        var _rule = this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " on " + _dayName;
                        this._appointmentAddWindow.find(".e-recurRule").html(_rule);
                        this._recRule = "FREQ=" + this._rRuleFreq + ";INTERVAL=" + interval + ";" + "BYDAY=" + _weekRule;
                        break;
                    case "MONTHLY":
                        if (this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton("option", "check")) {
                            monthDate = this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox("option", "value");
                            this._recRule = "FREQ=" + this._rRuleFreq + ";BYMONTHDAY=" + monthDate + ";INTERVAL=" + interval
                            recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + monthDate;
                        }
                        else {
                            var _setPos = this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "selectedItem");
                            var dayName = this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList("option", "selectedItem");
                            _setPos = _setPos < 4 ? (_setPos + 1) : -1;
                            this._recRule = "FREQ=" + this._rRuleFreq + ";BYDAY=" + this._dayNames[dayName] + ";BYSETPOS=" + _setPos;
                            recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[dayName];
                        }
                        break;
                    case "YEARLY":
                        if (this._appointmentAddWindow.find("#" + this._id + "yearday").ejRadioButton("option", "check")) {
                            var _monthIndex = this._appointmentAddWindow.find('.yearmonth').ejDropDownList("option", "selectedItem");
                            var _monthDate = this._findInterval(this._appointmentAddWindow.find('.yeardate'));
                            this._recRule = "FREQ=" + this._rRuleFreq + ";BYMONTHDAY=" + _monthDate + ";BYMONTH=" + (_monthIndex + 1) + ";INTERVAL=" + interval;
                            recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._monthNames[_monthIndex] + " " + _monthDate;
                        }
                        else {
                            var _setPos = this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "selectedItem");
                            var _monthIndex = this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList("option", "selectedItem");
                            var dayName = this._appointmentAddWindow.find('.yearsrtday').ejDropDownList("option", "selectedItem");
                            _setPos = _setPos < 4 ? (_setPos + 1) : -1;
                            this._recRule = "FREQ=" + this._rRuleFreq + ";BYDAY=" + this._dayNames[dayName] + ";BYMONTH=" + (_monthIndex + 1) + ";BYSETPOS=" + _setPos + ";INTERVAL=" + interval;
                            recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[dayName] + " " + this._monthNames[_monthIndex];
                        }
                        break;
                }
                if (this._appointmentAddWindow.find('.e-recurafter').ejRadioButton("option", "check")) {
                    var _count = this._findInterval(this._appointmentAddWindow.find('.e-recurcount'));
                    this._recRule += ";COUNT=" + _count;
                    endRecur = _count ? ", " + _count + " " + this._getLocalizedLabels("Times") : ", ";
                }
                else if (this._appointmentAddWindow.find('.e-recuruntil').ejRadioButton("option", "check")) {
                    var date = Globalize.format(new Date(this._appointmentAddWindow.find('.e-until').ejDatePicker("option", "value")), this._pattern.d);
                    this._recRule += ";UNTIL=" + date;
                    endRecur = " " + this._getLocalizedLabels("Until").toLowerCase() + " " + date;
                }
                var _every = (this._rRuleFreq != "WEEKDAYS" ? this._getLocalizedLabels("Every") : "") + " " + (interval > 1 ? interval + " " : "");
                var _rRuleStr = _every + this._appointmentAddWindow.find(".e-recurRule").html() + recurOption + endRecur;
                this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "table-row");
                this._appointmentAddWindow.find(".e-recurFinalRule").html(_rRuleStr);
                this._appointmentAddWindow.find(".e-recuredit").css("display", "inline");
                recurOption = "";
            }
            this._appointmentAddWindow.find(".e-recurwindow").toggle();
            this._appointmentAddWindow.find(".e-appwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 300);
            this._appointmentAddWindow.find('.done').focus();
        },
        /**
        Recurrence startdate value change
        @private
        */
        _recurDateChange: function (args) {
            this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", new Date(args.value));
            this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", new Date(args.value));
        },

        /**
        * Section to find the interval.
        * @private
        */
        _findInterval: function (numericElement) {
            var interval;
            interval = numericElement.ejNumericTextbox("option", "value");
            return interval = ej.isNullOrUndefined(interval) ? 0 : interval;
        },
        /**
        * Section for cancel the appointment window.
        * @private
        */
        _cancel: function () {
            this._appointmentAddWindow.ejDialog("close");
        },
        /**
        * Section for handle the start and end time in appointment window.
        * @private
        */
        _timeChange: function (e) {
            if (this._currentAction != ej.Schedule.actions.Add) return false;
            var obj = this._appointmentAddWindow.find("#" + this._id + "starttime").data("ejTimePicker");
            if (obj.showDropdown && obj._id == this._id + "starttime") {
                var endtime = $("#" + this._id + "starttime_popup").find("li.e-active").next("li").html();
                if (!ej.isNullOrUndefined(endtime)) document.getElementById(this._id + "endtime").value = endtime;
                else document.getElementById(this._id + "endtime").value = e.value;
            }
        },
        /**
        * Section for render all the controls used in the schedule.
        * @private
        */
        _renderControls: function (e) {
            var control = this._appointmentAddWindow;
            //control.find('.subject').ejMaskEdit({ rtl: this.model.rtl, name: this.model.fields["subjectField"], inputMode: ej.inputMode.Text, width: "100%", cssClass: this.model.cssClass });
            control.find('.allday').ejCheckBox({ rtl: this.model.rtl, cssClass: this.model.cssClass, change: $.proxy(this._alldayCheck, this) });
            control.find('.startenddate').ejDatePicker({ rtl: this.model.rtl, width: '100%', value: this.currentDate(), localize: this.model.localization, cssClass: this.model.cssClass });
            control.find('.startendtime').ejTimePicker({ rtl: this.model.rtl, width: "100%", value: this.currentDate(), localize: this.model.localization, change: $.proxy(this._timeChange, this), cssClass: this.model.cssClass });
            control.find('.startutc').ejDropDownList({ rtl: this.model.rtl, dataSource: this.model.timezoneCollection, fields: { text: "text", value: "value" }, width: "100%", change: this._utc, cssClass: this.model.cssClass });
            control.find('.endutc').ejDropDownList({ rtl: this.model.rtl, dataSource: this.model.timezoneCollection, fields: { text: "text", value: "value" }, width: "100%", change: this._utc, cssClass: this.model.cssClass });
            control.find('.recurrence').ejCheckBox({ rtl: this.model.rtl, change: $.proxy(this._repeat, this), cssClass: this.model.cssClass });
            control.find('.e-btndone').ejButton({ rtl: this.model.rtl, roundCorner: true, width: "100px", click: $.proxy(this._saveAppointment, this), cssClass: this.model.cssClass });
            control.find('.cancel').ejButton({ rtl: this.model.rtl, roundCorner: true, width: "100px", click: $.proxy(this._cancel, this), cssClass: this.model.cssClass });
            control.find('.recurtype').ejDropDownList({ rtl: this.model.rtl, targetId: "recurtypelist", width: "100%", select: $.proxy(this._recurtype, this), cssClass: this.model.cssClass });
            control.find('.dailyrecurtype').ejRadioButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            control.find('.e-dailyeveryday').ejRadioButton({ rtl: this.model.rtl, check: true, cssClass: this.model.cssClass });
            control.find('.everydaycount').ejNumericTextbox({ rtl: this.model.rtl, name: "everydaycount", minValue: 1, value: 1, width: "100px", float: "left", cssClass: this.model.cssClass });
            control.find('.recurends').ejRadioButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            control.find('.e-recurcount').ejNumericTextbox({ rtl: this.model.rtl, name: "dailyoccurance", minValue: 1, value: 10, width: "65px", cssClass: this.model.cssClass, change: $.proxy(this._endAfterChange, this) });
            control.find('.weekcount').ejNumericTextbox({ rtl: this.model.rtl, name: "weekcount", minValue: 1, value: 1, width: "100px", cssClass: this.model.cssClass });
            control.find('.e-recurevery').ejNumericTextbox({ rtl: this.model.rtl, name: "recurevery", minValue: 1, value: 1, width: "100px", cssClass: this.model.cssClass });
            control.find('.weekdays').ejCheckBox({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            control.find('.e-monthposition').ejRadioButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            control.find('.e-monthdaytype').ejRadioButton({ rtl: this.model.rtl, check: true, cssClass: this.model.cssClass });
            control.find('.e-monthdate').ejNumericTextbox({ rtl: this.model.rtl, name: "monthdate", minValue: 1, maxValue: 31, value: 1, width: "70px", cssClass: this.model.cssClass, change: $.proxy(this._monthDateChange, this) });
            control.find('.monthnumber').ejNumericTextbox({ rtl: this.model.rtl, name: "monthnumber", minValue: 1, value: 1, width: "70px", cssClass: this.model.cssClass });
            control.find('.yearcount').ejNumericTextbox({ rtl: this.model.rtl, name: "yearcount", minValue: 1, value: 1, width: "50px", cssClass: this.model.cssClass });
            control.find('.e-yearrecurposi,.e-yearrecurday').ejRadioButton({ rtl: this.model.rtl, cssClass: this.model.cssClass });
            control.find('.yeardate').ejNumericTextbox({ rtl: this.model.rtl, name: "yeardate", minValue: 1, maxValue: 31, value: 1, width: "70px", cssClass: this.model.cssClass, change: $.proxy(this._yearDateChange, this) });
            control.find('.recurbutton').ejButton({ rtl: this.model.rtl, roundCorner: true, width: "100px", click: $.proxy(this._closerecur, this), cssClass: this.model.cssClass });
            control.find('.monthsrt').ejDropDownList({ rtl: this.model.rtl, targetId: "monthsrtlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._monthDayChange, this) });
            control.find('.e-monthsrtday').ejDropDownList({ rtl: this.model.rtl, targetId: "monthsrtdaylist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._monthDayChange, this) });
            control.find('.yearsrt').ejDropDownList({ rtl: this.model.rtl, targetId: "yearsrtlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
            control.find('.yearmonth').ejDropDownList({ rtl: this.model.rtl, targetId: "yearmonthlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDateChange, this) });
            control.find('.yearsrtday').ejDropDownList({ rtl: this.model.rtl, targetId: "yearsrtdaylist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
            control.find('.yearsrtmonth').ejDropDownList({ rtl: this.model.rtl, targetId: "yearsrtmonthlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
            control.find('.recurstartdate').ejDatePicker({ rtl: this.model.rtl, localize: this.model.localization, cssClass: this.model.cssClass, change: $.proxy(this._recurDateChange, this) });
            control.find('.e-until').ejDatePicker({ rtl: this.model.rtl, localize: this.model.localization, cssClass: this.model.cssClass, change: $.proxy(this._endOnChange, this) });
        },
        /**
        * To check the radio button while change the year date.
        * @private
        */
        _yearDateChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "yearday").ejRadioButton("option", "check", true);
        },
        /**
        * To check the radio button while change the year day.
        * @private
        */
        _yearDayChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "yearother").ejRadioButton("option", "check", true);
        },
        /**
        * To check the radio button while change the month date.
        * @private
        */
        _monthDateChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "monthday").ejRadioButton("option", "check", true);
        },
        /**
        * To check the radio button while change the month day.
        * @private
        */
        _monthDayChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "monthon").ejRadioButton("option", "check", true);
        },
        /**
        * To check the radio button while change the end after value.
        * @private
        */
        _endAfterChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "repeatendafter").ejRadioButton("option", "check", true);
        },
        /**
        * To check the radio button while change the end date in recurrence window.
        * @private
        */
        _endOnChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "repeatendon").ejRadioButton("option", "check", true);
        },
        /**
        * Section to convert the minutes to hours.
        * @private
        */
        _minToHour: function (mins) {
            var hours = parseInt(mins / 60);
            var mins = mins % 60;
            var obj = {
                "h": hours,
                "m": mins
            };
            return obj;
        },
        /**
        * Section for handle the closing of appointment window.
        * @private
        */
        _onclose: function (args) {
            this._clearFields();
            this._processFocus();
        },
        /**
        * Section for handle the appointment save of schedule.
        * @private
        */
        _saveAppointment: function () {
            var appointments = this._sortAppById(this._processed), maxId;
            if (this._cellClick && this.model.showQuickWindow) {
                var obj = {};
                maxId = appointments[appointments.length - 1][this.model.fields["idField"]] + 1;
                obj[this.model.fields["idField"]] = maxId;
                obj[this.model.fields["subjectField"]] = this._quickAppointWindow.find('.subject').val();
                obj[this.model.fields["startTimeField"]] = new Date(this.cur_StartTime);
                obj[this.model.fields["endTimeField"]] = new Date(this.cur_EndTime);
                if (this.currentView() === "month" || (new Date(this.cur_StartTime).getHours() == 0 && new Date(this.cur_EndTime).getHours() == 23 && new Date(this.cur_EndTime).getMinutes() == 59))
                    obj[this.model.fields["alldayField"]] = true;
                else
                    obj[this.model.fields["alldayField"]] = false;
                obj[this.model.fields["recurrenceField"]] = false;
                var args = { data: obj };
                if (this._trigger("appointmentSaved", obj)) {
                    this._quickAppointWindow.ejDialog("close");
                    return false;
                }
                this._currentAction == ej.Schedule.actions.Add;
                this._dataManager.insert(args.data); // Add new appointment
                this._processed.push(args.data);
                this._cellClick = false;
                this._quickAppointWindow.ejDialog("close");
            }
            else {
                var formelement;
                obj = {};
                temp = {};
                //var appId = this._sortAppById(appointments);
                maxId = (this._currentAction == ej.Schedule.actions.Add || this._currentAction == ej.Schedule.actions.EditOcurrence) && appointments.length > 0 ? appointments[appointments.length - 1][this.model.fields["idField"]] + 1 : parseInt(this._appId);
                formelement = $("#" + this._id + "AppointmentAddEditWindow").find("#" + this._id + "_AddEditForm").get(0);
                for (var index = 0; index < formelement.length; index++) {
                    var columnName = formelement[index].name, $element = $(formelement[index]);
                    if (columnName != undefined) {
                        if (columnName == "")
                            columnName = formelement[index].id.replace(this._id, "");
                        if (columnName != "" && obj[columnName] == null) {
                            var value = formelement[index].value;
                            if (columnName == "Id") {
                                if (value == "")
                                    value = parseInt(maxId);
                                else
                                    value = parseInt(value);
                            }
                            if ($(formelement[index]).hasClass("e-datepicker"))
                                value = $element.ejDatePicker("option", "value");
                            else if ($(formelement[index]).hasClass("e-datepicker"))
                                value = $element.ejTimePicker("option", "value");
                            else if ($element.data("ejDropDownList"))
                                value = $element.ejDropDownList("model.value");
                            if (columnName != "StartDate" && columnName != "EndDate" && columnName != "StartTime" && columnName != "EndTime") {
                                if (formelement[index].type != "checkbox")
                                    obj[columnName] = value;
                                else
                                    obj[columnName] = $(formelement[index]).is(':checked');
                            }
                            else {
                                temp[columnName] = value;
                            }
                        }
                    }
                }
                var starttime = this._timeFormat(temp.StartTime).split(":");
                var endtime = this._timeFormat(temp.EndTime).split(":");
                obj[this.model.fields["startTimeField"]] = new Date(temp.StartDate.setHours(parseInt(starttime[0]), parseInt(starttime[1])));
                obj[this.model.fields["endTimeField"]] = new Date(temp.EndDate.setHours(parseInt(endtime[0]), parseInt(endtime[1])));
                (obj[this.model.fields["recurrenceField"]]) && (obj[this.model.fields["recurrenceRuleField"]] = this._recRule);
                // timezone calculation
                if (obj.startutc != "") {
                    var utc = obj.startutc.split(" ")[1].split(":");
                    var hour = parseInt(utc[0]);
                    var minutes = parseInt(utc[1]);
                    var localtime = new Date().getTimezoneOffset();
                    var local = this._minToHour(localtime + ((hour * 60) + minutes));
                    new Date(obj.StartTime.setHours(obj.StartTime.getHours() + local.h, obj.StartTime.getMinutes() + local.m));
                    new Date(obj.EndTime.setHours(obj.EndTime.getHours() + local.h, obj.EndTime.getMinutes() + local.m));
                }
                //Update the edited date
                if (this._currentAction == ej.Schedule.actions.EditOcurrence) {
                    var _editApp = this._deleteOcurrence(this._parentId, new Date(temp["StartDate"]));
                    new ej.DataManager(this._processed).remove(this.model.fields["idField"], this._appId);
                    obj[this.model.fields["recurrenceField"]] = 1;
                    obj[this.model.fields["recurrenceRuleField"]] = _editApp[0][this.model.fields["recurrenceRuleField"]] + ";RECUREDITID=" + _editApp[0][this.model.fields["idField"]];
                }
                else if (this._currentAction == ej.Schedule.actions.EditSeries) {
                    var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, this._parentId);
                    var _editApp = this._dataManager.executeLocal(query);
                    obj[this.model.fields["idField"]] = _editApp[0][this.model.fields["idField"]];
                    obj[this.model.fields["recurrenceRuleField"]] = this._recRule == "" ? _editApp[0][this.model.fields["recurrenceRuleField"]] : this._recRule;
                }
                if (this._currentAction != ej.Schedule.actions.Add) {
                    if (this._trigger("appointmentEdited", { appObj: obj })) {
                        this._appointmentAddWindow.ejDialog("close");
                        return false;
                    }
                }
                else {
                    if (this._trigger("appointmentSaved", { appObj: obj })) {
                        this._appointmentAddWindow.ejDialog("close");
                        return false;
                    }
                }
                var args = { data: obj };
                if (this._currentAction == ej.Schedule.actions.Add || this._currentAction == ej.Schedule.actions.EditOcurrence) {
                    this._dataManager.insert(args.data); // Add new appointment
                    this._appointmentProcessing(args.data);
                    //this._dataProcessing(this.model.fields["dataSource"]); //this._appointmentProcessing(args.data);
                }
                else if (this._currentAction == ej.Schedule.actions.Save || this._currentAction == ej.Schedule.actions.EditSeries) {
                    this._dataManager.update(this.model.fields["idField"], args.data); // Update edit appointmet
                    (obj[this.model.fields["recurrenceField"]] || (Globalize.format(new Date(obj[this.model.fields["startTimeField"]]), this._pattern.d) < Globalize.format(new Date(obj[this.model.fields["endTimeField"]]), this._pattern.d)) && (((new Date(obj[this.model.fields["endTimeField"]]) - new Date(obj[this.model.fields["startTimeField"]])) / 3600000) < 24) && (obj[this.model.fields["alldayField"]] !== true)) ? this._dataProcessing(this.dataSource()) : new ej.DataManager(this._processed).update(this.model.fields["idField"], args.data) && this.element.find("#Appointment_" + this._appId).remove();
                }
            }
            if (obj[this.model.fields["recurrenceField"]])
                this._renderAppointmentAll();
            else {
                var strDate = new Date(obj[this.model.fields["startTimeField"]]);
                var index = $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this._dateRender);
                index >= 0 && (obj[this.model.fields["alldayField"]] ? this._setAllDayPos(args.data, index) : this._renderApp(args.data, index));
            }
            this._clearFields();
            this._appointmentAddWindow.ejDialog("close");
            if (this.element.find('div.e-draggableworkarea tr').find('td').hasClass("e-selectedCell"))
                this._processFocus();
            this._recRule = "";

        },
        /**
        * Convert timeformat into 24 hour mode.
        * @private
        */
        _timeFormat: function (time) {
            var culture = Globalize.culture(this.model.localization);
            var hours = Number(time.match(/^(\d+)/)[1]);
            var minutes = Number(time.match(/:(\d+)/)[1]);
            if (!ej.isNullOrUndefined(time.split(" ")[1])) {
                var AMPM = time.match(/\s(.*)$/)[1];
                if (AMPM == culture.calendar.PM[0] && hours < 12) hours = hours + 12;
                if (AMPM == culture.calendar.AM[0] && hours == 12) hours = hours - 12;
            }
            var sHours = hours.toString();
            var sMinutes = minutes.toString();
            if (hours < 10) sHours = "0" + sHours;
            if (minutes < 10) sMinutes = "0" + sMinutes;
            return sHours + ":" + sMinutes;
        },
        /**
        * Section for handle the escape key process.
        * @private
        */
        _processEscapeKey: function () {
            this._processFocus();
            this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
        },
        /**
        * Section for handle the focus process of the schedule.
        * @private
        */
        _processFocus: function () {
            var selectApp = this.element.find("div.e-selectedAppointment");
            if (selectApp.length != 0)
                selectApp.focus();
            if (this.element.find("td.e-selectedCell").hasClass("e-workcells")) {
                var oRowCell = this.element.find('div.e-draggableworkarea tr');
                var rowcount = oRowCell.length - 1; // this.get_RowCount();
                var omincolIndex, omaxcolIndex, rowInx, maxrowInx, checkcolindex;
                var curr_View = this.currentView();
                var proxy = this;
                this.element.find('td.e-selectedCell').each(function (index, tdItem) {
                    var targetId = $(tdItem).index();
                    if (targetId != null && !isNaN(targetId)) {
                        if (omincolIndex == null) {
                            omincolIndex = targetId;
                            rowInx = $(tdItem).parent()[0].rowIndex;
                        }
                        else if (omincolIndex < targetId) {
                            omincolIndex = omincolIndex;
                        }
                        else {
                            if (omincolIndex != targetId) {
                                omincolIndex = targetId;
                                rowInx = $(tdItem).parent()[0].rowIndex;
                            }
                        }
                        if (omaxcolIndex == null) {
                            omaxcolIndex = targetId;
                            if (curr_View != "month") {
                                checkcolindex = parseInt(omaxcolIndex);
                                for (var i = 0; i <= rowcount; i++) {
                                    if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                        maxrowInx = i;

                                    }
                                }
                            }
                        }
                        else if (omaxcolIndex > targetId) {
                            omaxcolIndex = omaxcolIndex;
                        }
                        else {
                            if (omaxcolIndex != targetId) {
                                omaxcolIndex = targetId;
                                if (curr_View != "month") {
                                    checkcolindex = parseInt(omaxcolIndex);
                                    for (var i = 0; i <= rowcount; i++) {
                                        if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                            maxrowInx = i;

                                        }
                                    }
                                }
                            }
                        }
                    }
                });
                if (curr_View != "month") {
                    $(oRowCell[maxrowInx].cells[omaxcolIndex]).attr("tabIndex", 0).focus();
                }
                else {
                    var tdsel = oRowCell.find('td.e-selectedCell');
                    var tdlen = parseInt(tdsel.length) - 1;
                    $(tdsel[tdlen]).attr("tabIndex", 0).focus();
                }
            }
            else {
                this.element.focus();
                this.element.find("td.e-selectedCell").attr("tabIndex", 0).focus();
            }
        },
        /**
        * Section to clear all the fields of apointment window.
        * @private
        */
        _clearFields: function () {
            this._appointmentAddWindow.find('.subject').val("");
            this._appointmentAddWindow.find('.e-description').val("");
            this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(new Date(), this._pattern.d));
            this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(new Date(), this._pattern.d));
            this._appointmentAddWindow.find('.startendtime').ejTimePicker({ enabled: true });
            this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(), this._pattern.t));
            this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(), this._pattern.t));
            this._appointmentAddWindow.find(".recurrence,.weekdays").ejCheckBox({ check: false, enabled: true });
            this._appointmentAddWindow.find(".e-repeatlbl").removeClass('e-disable');
            this._appointmentAddWindow.find(".allday").ejCheckBox({ check: false });
            this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList("clearText");
            this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "none");
            this._appointmentAddWindow.find(".e-recurFinalRule").html("");
            this._appointmentAddWindow.find(".e-recuredit").hide();

        },
        /**
        * To configure the model for numeric text box.
        * @private
        */
        _getNumericModel: function () {
            return numericModel = {
                width: "100px",
                minValue: 1,
                value: 1,
                cssClass: this.model.cssClass,
                rtl: this.model.rtl,
                localize: this.model.localization
            };
        },
        /**
        * Section to clear all the recurrence fields.
        * @private
        */
        _recurClearFields: function () {
            var list = this._appointmentAddWindow.find("input[name='weekdays']");
            for (var i = 0; i < list.length; i++) {
                $(list[i]).ejCheckBox({ check: false });
            }
            this._appointmentAddWindow.find('.e-recurevery').ejNumericTextbox({ value: 1 });
            this._appointmentAddWindow.find('.e-recurcount').ejNumericTextbox({ value: 10 });
            this._appointmentAddWindow.find(".e-recurRule").html("");
        },
        /**
        * Section to delete the appointment.
        * @private
        */
        _deleteApp: function (id) {
            var args;
            var removeItem = this._dataManager.remove(this.model.fields["idField"], id);
            (removeItem[0][this.model.fields["recurrenceField"]]) ? this._dataProcessing(this.dataSource()) && this._renderAppointmentAll() : new ej.DataManager(this._processed).remove(this.model.fields["idField"], id) && this.element.find("#Appointment_" + id).remove();
            args = { data: removeItem };
            this._trigger("appointmentDeleted", args);
        },
        /**
        * Section for handle the appointment resize start process.
        * @private
        */
        _appointmentResizeStart: function (e) {
            if (this.model.allowResizing) {
                var data = { element: $(e.target).parent() };
                if (this._trigger("resizeStart", data))
                    return false;
                e.preventDefault();
                this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close");
                this.resizeStart = true;
                if ($(e.target).hasClass("e-lefthandle") || $(e.target).hasClass("e-righthandle")) {
                    this.resize = e.pageX;
                    this.appoint_width = $(e.target).parent().width();
                    this.appoint_left = parseInt($(e.target).parent()[0].style.left);
                    $(e.target).hasClass("e-lefthandle") ? this._leftResizing = true : this._rightResizing = true;
                }
                else if ($(e.target).hasClass("e-tophandle") || $(e.target).hasClass("e-bottomhandle")) {
                    this.resize = e.pageY;
                    this.appoint_height = $(e.target).parent().height();
                    this.appoint_top = parseInt($(e.target).parent()[0].style.top);
                    $(e.target).hasClass("e-tophandle") ? this._topResizing = true : this._bottomResizing = true;
                }
                this._curElmt = $(e.target).parent();
                this.element.find(".e-workcells").removeClass("e-selectedCell");
                this.element.find(".e-workcells").removeClass("e-mousehover");
                this._on($(document), ej.eventType.mouseMove, $.proxy(this._appointmentResizing, this));
                this._on($(document), ej.eventType.mouseUp, $.proxy(this._appointmentResizeStop, this));
                if ($(e.target).hasClass("e-rowcursor") || $(e.target).hasClass("e-columncursor")) // to prevent the drag n drop functionality
                    return false;
            }
        },
        /**
        * Section for handle the appointments while resizing.
        * @private
        */
        _appointmentResizing: function (e) {
            var target = $(e.target);
            var data = { element: target.parent() };
            if (this._trigger("resizing", data))
                return false;
            if (this._topResizing) {
                this._curElmt.height(this.appoint_height + this.resize - e.pageY + 3);
                this._curElmt.css("top", this.appoint_top - (this.resize - e.pageY) - 3);
            }
            else if (this._bottomResizing) {
                this._curElmt.height(this.appoint_height + (e.pageY - this.resize) + 2);
            }
            else if (this._rightResizing) {
                this._curElmt.width(this.appoint_width + (e.pageX - this.resize) + 2);
            }
            else if (this._leftResizing) {
                this._curElmt.width(this.appoint_width + this.resize - e.pageX + 3);
                this._curElmt.css("left", this.appoint_left - (this.resize - e.pageX) - 3);
            }
        },
        /**
        * Section for handle the appointment after stop the resize.
        * @private
        */
        _appointmentResizeStop: function (e) {
            try {
                this.resizeStart = false;
                $(document).unbind(ej.eventType.mouseMove);
                $(document).unbind(ej.eventType.mouseUp);
                var model = this.model.fields;
                var appList = this.dataSource();
                var appointDiv = $(e.target).hasClass("e-appointment") || $(e.target).hasClass("e-monthappointment") || $(e.target).hasClass("e-alldayappointment") ? $(e.target) : $(e.target).parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
                var appId = appointDiv[0].id.split("_")[1];
                var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, appId));
                var appoint = (tempApp[0][model["recurrenceField"]] && tempApp[0][model["recurrenceRuleField"]].toString().indexOf("RECUREDITID") == -1) ? this._getAppointmentByParentId(appId) : tempApp[0]
                //var appoint = this._getAppointmentByParentId(appId);

                if (appoint != null || appoint !== undefined) {
                    var bigApps = new Date(appoint[this.model.fields["startTimeField"]]).getDate() < new Date(appoint[this.model.fields["endTimeField"]]).getDate() ? true : false;
                    var newStartTime = new Date(appoint[this.model.fields["startTimeField"]]);
                    var newEndTime = new Date(appoint[this.model.fields["endTimeField"]]);
                    var resizeElmntHeight = $(appointDiv[0]).height();

                    if (this.currentView() !== "month") {
                        var appTop = $(appointDiv[0]).offset().top; var dragArea = this.element.find(".e-draggableworkarea");
                        var cellHeight = this.element.find(".e-workcells").height();
                        var resizeMax = dragArea.height() + dragArea.scrollTop();
                        var topPos = appTop - dragArea.offset().top + dragArea.scrollTop();
                        var resizeMaxTop = (topPos - (cellHeight / ((60 / 30)))) < resizeMax ? topPos : resizeMax - resizeElmntHeight;
                    }

                    if (this._topResizing) {

                        if (resizeMaxTop > 0) {
                            var newTopTime;
                            newTopTime = resizeMaxTop * 60 / ((60 / 30) * (cellHeight + 1));
                            newStartTime.setHours(parseInt(this.model.startHour) + parseInt(newTopTime / 60));
                            newStartTime.setMinutes(parseInt(newTopTime % 60));
                            newStartTime.setMinutes((Math.round(newStartTime.getMinutes() / 10.0) * 10));
                        }
                    }
                    else if (this._bottomResizing) {
                        if (resizeMaxTop > 0) {
                            var newTopTime;
                            var newcellCount = ($(appointDiv[0]).height() / cellHeight);
                            newTopTime = (resizeMaxTop + (newcellCount * cellHeight)) * 60 / ((60 / 30) * (cellHeight + 1));
                            newEndTime.setHours(parseInt(this.model.startHour) + parseInt(newTopTime / 60));
                            newEndTime.setMinutes(parseInt(newTopTime % 60));
                            newEndTime.setMinutes((Math.round(newEndTime.getMinutes() / 10.0) * 10));
                        }
                    }
                    else if (this._leftResizing) {

                        if (this.currentView() !== "month") {
                            newStartTime = new Date(this._dateRender[0]);
                            var diffLeft = $(appointDiv[0]).offset().left - $(this.element.find(".e-alldaycells")[0]).offset().left;
                            var diffDays = Math.round(diffLeft / (this.element.find(".e-alldaycells").width() + 1));
                            newStartTime.setDate(new Date(newStartTime).getDate() + diffDays);
                        }
                        else {
                            newStartTime = new Date(new Date(this.model.currentDate).getFullYear(),
                                                      new Date(this.model.currentDate).getMonth(), 1);
                            var daysInPrevMonth = newStartTime.getDay();
                            newStartTime.setDate(newStartTime.getDate() - daysInPrevMonth);

                            diffDays = Math.round($(appointDiv[0]).offset().left / (this.element.find(".e-monthcells").width()) - 1);
                            diffDays = (diffDays % 7);
                            newStartTime.setDate(newStartTime.getDate() + parseInt(diffDays));

                            var newTopDate = $(appointDiv[0]).offset().top / (this.element.find(".e-monthcells").height()) - 1;
                            newStartTime.setDate(newStartTime.getDate() + (parseInt(newTopDate) * 7));
                        }
                    }
                    else if (this._rightResizing) {

                        if (this.currentView() !== "month") {
                            newEndTime = new Date(this._dateRender[0]);
                            var diffLeft = ($(appointDiv[0]).offset().left - $(this.element.find(".e-alldaycells")[0]).offset().left) + $(appointDiv[0]).width();
                            var diffDays = Math.round(diffLeft / (this.element.find(".e-alldaycells").width())) - 1;
                        }
                        else {
                            var diffWidth = $(appointDiv[0]).width() - this.appoint_width;
                            var diffDays = Math.round(diffWidth / (this.element.find(".e-monthcells").width()));
                        }
                        newEndTime.setDate(new Date(newEndTime).getDate() + diffDays);

                    }

                    //Recurrence applointment
                    if (appoint[model["recurrenceField"]] == 1 && appoint[model["recurrenceRuleField"]].toString().indexOf("RECUREDITID") == -1) {
                        var _newAddList = {};
                        var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, appId);
                        var _newAddList = new ej.DataManager(this._processed).executeLocal(query)[0];
                        var _currStr = new Date(_newAddList[model["startTimeField"]]);
                        var _currEnd = new Date(_newAddList[model["endTimeField"]]);
                        var _resizeStrTime = new Date(_currStr.getFullYear(), _currStr.getMonth(), _currStr.getDate()).setHours(newStartTime.getHours(), newStartTime.getMinutes(), newStartTime.getMilliseconds()); //newStartTime.setTime(new Date(_newAddList[model["startTimeField"]]).getTime());
                        var _resizeEndTime = new Date(_currEnd.getFullYear(), _currEnd.getMonth(), _currEnd.getDate()).setHours(newEndTime.getHours(), newEndTime.getMinutes(), newEndTime.getMilliseconds()); //newStartTime.setTime(new Date(_newAddList[model["startTimeField"]]).getTime());
                        $.extend(_newAddList, appoint);
                        var appId = parseInt(appoint[model["idField"]]);
                        this._deleteOcurrence(appId, _currStr);
                        var maxId = this._maxId;
                        _newAddList[model["recurrenceRuleField"]] = appoint[model["recurrenceRuleField"]] + ";RECUREDITID=" + _newAddList[model["idField"]];
                        _newAddList[model["startTimeField"]] = new Date(_resizeStrTime);
                        _newAddList[model["endTimeField"]] = new Date(_resizeEndTime);
                        _newAddList[model["idField"]] = maxId;
                        var args = { data: _newAddList };
                        this._dataManager.insert(args.data); // Add new appointment
                        this._processed.splice(this._processed.indexOf(_newAddList), 1);
                        this._processed.push(args.data);
                    }
                    else {
                        var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, appoint[this.model.fields["idField"]]);
                        var object = new ej.DataManager(appList).executeLocal(query);
                        object[0][this.model.fields["startTimeField"]] = newStartTime;
                        object[0][this.model.fields["endTimeField"]] = newEndTime;
                        this._dataManager.update(this.model.fields["idField"], appList);
                        new ej.DataManager(this._processed).update(this.model.fields["idField"], appList);
                    }
                    var data = { appointment: appoint, target: e };
                    if (this._trigger("resizeStop", data))
                        return false;
                    this._topResizing = false;
                    this._bottomResizing = false;
                    this._leftResizing = false;
                    this._rightResizing = false;

                    //this._dataProcessing(appList);
                    this._renderAppointmentAll();
                }
                this.element.find("div.e-tophandle,div.e-bottomhandle").removeClass("e-rowcursor");
                this.element.find("div.e-lefthandle,div.e-righthandle").removeClass("e-columncursor");
            } catch (e) {

            }
        },
        /**
        * Split the Recurrence rule string.
        * @private
        */
        // Split the Recurrence rule string 
        recurrenceRuleSplit: function (rRule) {
            var _rule, _rRuleSplit = rRule.split(";");
            for (var r = 0; r < _rRuleSplit.length; r++) {
                _rule = _rRuleSplit[r].split("=");
                switch ($.trim(_rule[0]).toUpperCase()) {
                    case "FREQ":
                        this._rRule.freq = $.trim(_rule[1]).toUpperCase();
                        break;
                    case "INTERVAL":
                        this._rRule.interval = parseInt(_rule[1], 10);
                        break;
                    case "COUNT":
                        this._rRule.count = parseInt(_rule[1], 10);
                        break;
                    case "UNTIL":
                        this._rRule.until = new Date(_rule[1]);
                        break;
                    case "BYDAY":
                        this._rRule.weekDays = _rule[1];
                        break;
                    case "BYMONTHDAY":
                        this._rRule.monthDay = parseInt(_rule[1], 10);
                        break;
                    case "BYMONTH":
                        this._rRule.month = parseInt(_rule[1], 10);
                        break;
                    case "BYSETPOS":
                        this._rRule.setPositions = parseInt(_rule[1], 10);
                        break;
                    case "WKST":
                        this._rRule.weekStart = _rule[1];
                        break;
                    case "EXDATE":
                        var _exDates = _rule[1].split(",");
                        this._rRule.exDate = [];
                        for (var i = 0; i < _exDates.length; i++) {
                            this._rRule.exDate[i] = Globalize.format(new Date(_exDates[i]), this._pattern.d);
                        }
                        //this._rRule.exDate = _rule[1];
                        break;
                    case "RECUREDITID":
                        this._rRule.recurEditId = parseInt(_rule[1]);
                        break;
                }
            }
            return this._rRule;
        },
        /**
        * Section for handle the drag and drop appointments.
        * @private
        */
        _dragdropAppointments: function () {
            if (this.model.allowDragDrop) {
                var proxy = this;

                // normal appointments
                var $dragableElements = this.element.find(".e-appointment");
                $dragableElements.ejDraggable({
                    handle: ".e-appointment",
                    dragArea: proxy.element.find("div.e-draggableworkarea"),
                    cursorAt: { top: 20, left: 0 },
                    dragStart: function (args) {
                        proxy._dragStart(args);
                    },
                    drag: function (args) {
                        proxy._dragOver(args);
                    },
                    dragStop: function (args) {
                        proxy._dragStop(args);
                    },
                    helper: function (event, ui) {
                        var appointdiv = $(event.sender.target).hasClass("e-appointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-appointment");
                        if (appointdiv.hasClass("e-appointment"))
                            return appointdiv.addClass("e-draggableAppointment").css("width", appointdiv[0].offsetWidth + "px").appendTo(proxy.element);
                    }

                });

                // allday appointments
                var $dragableElements = this.element.find(".e-alldayappointment");
                $dragableElements.ejDraggable({
                    handle: ".e-alldayappointment",
                    dragArea: proxy.element.find(".e-headrealldaytable tr")[2],
                    cursorAt: { top: 17, left: 0 },
                    dragStart: function (args) {
                        proxy._dragStart(args);
                    },
                    drag: function (args) {
                        proxy._dragOver(args);
                    },
                    dragStop: function (args) {
                        proxy._dragStop(args);
                    },
                    helper: function (event, ui) {
                        var appointdiv = $(event.sender.target).hasClass("e-alldayappointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-alldayappointment");
                        if (appointdiv.hasClass("e-alldayappointment"))
                            return appointdiv.addClass("e-draggableAppointment").css("width", appointdiv[0].offsetWidth + "px").appendTo(proxy.element);
                    }

                });

                // month appointments
                var $dragableElements = this.element.find(".e-monthappointment");
                $dragableElements.ejDraggable({
                    handle: ".e-monthappointment",
                    dragArea: proxy.element.find(".e-workareadiv"),
                    cursorAt: { top: 17, left: 0 },
                    dragStart: function (args) {
                        proxy._dragStart(args);
                    },
                    drag: function (args) {
                        proxy._dragOver(args);
                    },
                    dragStop: function (args) {
                        proxy._dragStop(args);
                    },
                    helper: function (event, ui) {
                        var appointdiv = $(event.sender.target).hasClass("e-monthappointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-monthappointment");
                        if (appointdiv.hasClass("e-monthappointment"))
                            return appointdiv.addClass("e-draggableAppointment").css("width", appointdiv[0].offsetWidth + "px").appendTo(proxy.element);
                    }

                });
            }
        },
        /**
        * Section for handle the drag start of appointment.
        * @private
        */
        _dragStart: function (args) {
            var target = args.element;
            var data = { target: target };
            this._trigger("dragStart", data);
            this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") || this._quickAppDetailsWindow.ejDialog("close"));
        },
        /**
        * Section for handle the drag over of appointment.
        * @private
        */
        _dragOver: function (args) {
            var $target = $(args.element);
            var data = { target: $target };
            this._trigger("dragOver", data);
            this.element.find(".e-workcells").removeClass("e-selectedCell");
            this.element.find(".e-workcells").removeClass("e-mousehover");
        },
        /**
        * Section for handle the drag stop of appointment.
        * @private
        */
        _dragStop: function (args) {
            var proxy = this; var model = proxy.model.fields; var dragArea = proxy.element.find(".e-draggableworkarea");;
            var appId = args.element[0].id.split("_")[1];
            var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, appId));
            var appoint = (tempApp[0][model["recurrenceField"]] && tempApp[0][model["recurrenceRuleField"]].toString().indexOf("RECUREDITID") == -1) ? this._getAppointmentByParentId(appId) : tempApp[0]
            //var appoint = proxy._getAppointmentByParentId(appId);
            var appList = this.dataSource();
            if ($(args.element[0]).hasClass("e-appointment")) {
                var dragMax = dragArea.height() + dragArea.scrollTop();
                var dragElmntHeight = args.element[0].offsetHeight;
                var cellHeight = this.element.find(".e-workcells").height();
                var topPos = $(args.element[0]).offset().top - dragArea.offset().top + dragArea.scrollTop();
                var dragMaxTop = (topPos - (cellHeight / (60 / 30))) < dragMax ? topPos : dragMax - dragElmntHeight;
                if (dragMaxTop > 0) {
                    var newTopTime;
                    newTopTime = dragMaxTop * 60 / ((60 / 30) * (cellHeight + 1));

                    var twoDayApps = new Date(appoint[proxy.model.fields["startTimeField"]]).getDate() < new Date(appoint[proxy.model.fields["endTimeField"]]).getDate() ? true : false;
                    var startTime = new Date(appoint[proxy.model.fields["startTimeField"]]);
                    var endTime = new Date(appoint[proxy.model.fields["endTimeField"]]);
                    var duration = (endTime - startTime) / 1000;
                    var hours = Math.floor(duration / 3600);
                    var minutes = Math.floor((duration % 3600) / 60);
                    var newStartTime, newEndTime, cellstrtime, cellendtime;

                    if (proxy.model.currentView === "week" || proxy.model.currentView === "workweek") {
                        var firstDayOfWeek = new Date(proxy._firstweekdate());
                        if (proxy.model.currentView === "workweek") {
                            var _day = new Date(firstDayOfWeek).getDay();
                            var _dayAdd = _day == 0 ? 1 : 0;
                            firstDayOfWeek.setDate(firstDayOfWeek.getDate() + _dayAdd);
                        }
                        newStartTime = firstDayOfWeek;

                        //finding cellindex by left calculation
                        var dragAreaLength = dragArea.width();
                        var noOfCols = proxy.model.currentView === "week" ? 7 : 5;
                        var cellWidth = Math.round(dragAreaLength / noOfCols);
                        var originalLeft = args.element[0].offsetLeft - dragArea.offset().left;
                        var cellIndex = Math.round(originalLeft / cellWidth);

                        newStartTime = new Date(new Date(newStartTime).setDate(new Date(firstDayOfWeek).getDate() + cellIndex));
                        newEndTime = newStartTime;
                    }
                    else {
                        newStartTime = new Date(appoint[proxy.model.fields["startTimeField"]]);
                        newEndTime = new Date(newStartTime);
                    }

                    if (twoDayApps) {
                        newStartTime = proxy._getNewStartTime(newTopTime, newStartTime);
                        cellstrtime = new Date(newStartTime);
                        newEndTime = proxy._getNewEndTime(newStartTime, hours, minutes);
                        cellendtime = new Date(newEndTime);
                    }
                    else {
                        newStartTime.setHours(parseInt(proxy.model.startHour) + Math.round(newTopTime / 60));
                        newStartTime.setMinutes(parseInt(newTopTime % 60));
                        newStartTime = (new Date(newStartTime).getMinutes() >= 30) ? new Date(new Date(newStartTime).setHours(new Date(newStartTime).getHours() - 1)) : new Date(newStartTime);
                        newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
                        cellstrtime = new Date(newStartTime);

                        var totMin = (new Date(appoint[proxy.model.fields["endTimeField"]]).getHours() * 60 + new Date(appoint[proxy.model.fields["endTimeField"]]).getMinutes()) -
             (new Date(appoint[proxy.model.fields["startTimeField"]]).getHours() * 60 + new Date(appoint[proxy.model.fields["startTimeField"]]).getMinutes());


                        newEndTime.setHours(new Date(newStartTime).getHours() + parseInt(totMin / 60));
                        newEndTime.setMinutes(new Date(newStartTime).getMinutes() + parseInt(totMin % 60));
                        newEndTime.setMinutes(Math.round(newEndTime.getMinutes() / 10.0) * 10);
                        cellendtime = new Date(newEndTime);
                    }
                }
            }
            else if ($(args.element[0]).hasClass("e-alldayappointment")) {
                newStartTime = new Date(proxy._dateRender()[0]);
                newEndTime = newStartTime;
                var diffLeft = args.element[0].offsetLeft - this.element.find(".e-alldaycells").offset().left; // ui.position.left - ui.originalPosition.left;
                var diffDays = Math.round(diffLeft / (this.element.find(".e-alldaycells").width() + 1));
                newStartTime.setDate(new Date(newStartTime).getDate() + diffDays);
                cellstrtime = new Date(newStartTime);
                var endInd = Math.round($(args.element[0]).width() / (this.element.find(".e-alldaycells").width() + 1));
                newEndTime.setDate(new Date(newEndTime).getDate() + endInd - 1);
                cellendtime = new Date(newEndTime);
            }

            else if ($(args.element[0]).hasClass("e-monthappointment")) {
                var oldStartTime = new Date(appoint[proxy.model.fields["startTimeField"]]);
                var oldEndTime = new Date(appoint[proxy.model.fields["endTimeField"]]);
                newStartTime = new Date(new Date(proxy.model.currentDate).getFullYear(),
                                      new Date(proxy.model.currentDate).getMonth(), 1);
                var daysInPrevMonth = newStartTime.getDay();
                newStartTime.setDate(newStartTime.getDate() - daysInPrevMonth);

                diffDateLeft = Math.round(args.element[0].offsetLeft / (this.element.find(".e-monthcells").width()));
                diffDateLeft = diffDateLeft % 7;
                newStartTime.setDate(newStartTime.getDate() + parseInt(diffDateLeft));

                var newTopDate = args.element[0].offsetTop / (this.element.find(".e-monthcells").height()) - 1;
                newStartTime.setDate(newStartTime.getDate() + (parseInt(newTopDate) * 7));
                newStartTime.setHours(oldStartTime.getHours(), oldStartTime.getMinutes(), oldStartTime.getSeconds());
                cellstrtime = new Date(newStartTime);
                var diffTime = newStartTime.getTime() - new Date(oldStartTime).getTime();
                newEndTime = new Date(new Date(oldEndTime).getTime() + diffTime);
                cellendtime = new Date(newEndTime);
            }

            //Recurrence applointment
            if (appoint[model["recurrenceField"]] == 1 && appoint[model["recurrenceRuleField"]].indexOf("RECUREDITID") == -1) {
                var query = new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, appId);
                var _appList = new ej.DataManager(this._processed).executeLocal(query)[0];
                var _newAddList = {};
                $.extend(_newAddList, appoint);
                proxy._deleteOcurrence(parseInt(appoint[model["idField"]]), _appList[model["startTimeField"]]);
                var maxId = this._processed[appList.length - 1][model["idField"]] + 1;
                _newAddList[model["recurrenceRuleField"]] = appoint[model["recurrenceRuleField"]] + ";RECUREDITID=" + _newAddList[model["idField"]];
                _newAddList[model["startTimeField"]] = cellstrtime;
                _newAddList[model["endTimeField"]] = cellendtime;
                _newAddList[model["idField"]] = maxId;
                var argsAdd = { data: _newAddList };
                proxy._dataManager.insert(argsAdd.data); // Add new appointment
                proxy._processed.splice(proxy._processed.indexOf(_appList), 1);
                proxy._processed.push(argsAdd.data);
            }
            else {
                var query = new ej.Query().where(proxy.model.fields["idField"], ej.FilterOperators.equal, appoint[proxy.model.fields["idField"]]);
                var object = new ej.DataManager(appList).executeLocal(query);
                object[0][proxy.model.fields["startTimeField"]] = cellstrtime;
                object[0][proxy.model.fields["endTimeField"]] = cellendtime;
                proxy._dataManager.update(proxy.model.fields["idField"], appList);
                new ej.DataManager(proxy._processed).update(proxy.model.fields["idField"], appList);
            }
            $(args.element[0]).remove();
            // proxy._dataProcessing(appList);
            proxy._renderAppointmentAll();
            proxy._trigger("dragStop", object);
        },
        /**
        * To get the appointments parent Id.
        * @private
        */
        _getAppointmentByParentId: function (appId) {
            var tempApp, appoint, appList = this.dataSource(); // _processed;
            var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, appId));
            var appoint = this._dataManager.executeLocal(new ej.Query().where(this.model.fields["idField"], ej.FilterOperators.equal, parseInt(tempApp[0]["ParentId"])));
            return !$.isEmptyObject(appoint) ? appoint[0] : appoint;
        },
        /**
        * To render all the appointments.
        * @private
        */
        _renderAppointmentAll: function () {
            this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").remove();
            if (this.currentView() !== "month") {
                this._renderAppointments();
                this._renderAllDayAppointments();
            }
            else
                this._renderMonthAppointment();
            this._wireResizeEvents();
            this._dragdropAppointments();
        },
        /**
        * Section to get the new end time after drag and drop the appointment.
        * @private
        */
        _getNewEndTime: function (newStartTime, hours, minutes) {
            var newEndTime = new Date(newStartTime);
            newEndTime.setHours(newEndTime.getHours() + hours);
            newEndTime.setMinutes(newEndTime.getMinutes() + minutes);
            return newEndTime;
        },
        /**
        * Section to get the new start time after drag and drop the appointment.
        * @private
        */
        _getNewStartTime: function (newTopTime, newStartTime) {
            newStartTime.setHours((newStartTime.getHours()) + parseInt(this.model.startHour) + parseInt(newTopTime / 60));
            newStartTime.setMinutes((newStartTime.getMinutes()) + parseInt(newTopTime % 60));
            newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
            return newStartTime;
        },

        /**
        * Section to create the templates used in the schedule.
        * @private
        */
        _createTemplate: function (elt, templateId) {
            var scriptElt = document.createElement("script");
            scriptElt.id = (this._id + templateId + "_Template");
            scriptElt.type = "text/x-jsrender";
            scriptElt.text = elt;
            $("body").append(scriptElt);
            return scriptElt;
        },
        /**
        * Section for handle the keyboard support of the schedule.
        * @private
        */
        _documentOnKeyDown: function (event) {
            if (this.model.allowKeyboardNavigation) {
                var oEl = event.target || document.activeElement || event.srcElement;
                if (event.which == 27) {
                    this._processEscapeKey();
                }
                else if (event.which == 13) {
                    this._processEnterKey(oEl);
                }
                else if (event.shiftKey == false && event.which == 9) {
                    this._processWindowTab(oEl);
                }
            }
        },
        /**
        * Section for handle the window tabbing process.
        * @private
        */
        _processWindowTab: function (oEl) {
            if ($(oEl).hasClass("qAppDone"))
                this._quickAppointWindow.find(".subject").focus();
            else if ($(oEl).hasClass("e-appointcancel") || $(oEl).hasClass("e-cancelrecur"))
                this._appointmentAddWindow.find(".subject").focus();
            else if ($(oEl).hasClass("e-editseries"))
                this.model.showQuickWindow && this._quickAppDetailsWindow.focus();
        },
        /**
        * Section for handle all the keyboard navigation process.
        * @private
        */
        _onKeyDown: function (event) {
            if (this.model.allowKeyboardNavigation) {
                this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") || this._quickAppDetailsWindow.ejDialog("close"));
                var oRowCell = this.element.find('div.e-draggableworkarea tr');
                var oEl = event.target || document.activeElement || event.srcElement;
                if (event.shiftKey == false && event.which == 38) { //up arrow navigation
                    this._stopDefaultAction(event);
                    this._processUpArrow(oEl, oRowCell);
                }
                else if (event.shiftKey == false && event.which == 40) { //down arrow for cell navigation
                    this._stopDefaultAction(event);
                    this._processDown(oEl, oRowCell);
                }
                else if (event.ctrlKey == false && event.shiftKey == false && event.which == 39) { //right arrow cell navigation
                    this._stopDefaultAction(event);
                    this._processRight(oEl, oRowCell);
                }
                else if (event.ctrlKey == false && event.shiftKey == false && event.which == 37) { //left arrow cell navigation
                    this._stopDefaultAction(event);
                    this._processLeft(oEl, oRowCell);
                }
                else if (event.altKey == true && event.which == 67) { //Alt+C To show/hide the calendar pane
                    this._stopDefaultAction(event);
                    this.element.find(".e-datedisplay").click();
                }
                else if (event.ctrlKey == true && event.which == 37) { //Previous button
                    this._stopDefaultAction(event);
                    this.element.find("div.e-navigateprevious").click();
                    this.element.find('div.e-navigateprevious').attr("tabIndex", 0).focus();
                    this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                }
                else if (event.ctrlKey == true && event.which == 39) { //Next button
                    this._stopDefaultAction(event);
                    this.element.find("div.e-navigatenext").click();
                    this.element.find('div.e-navigatenext').attr("tabIndex", 0).focus();
                    this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                }
                else if ((event.altKey == true && event.which == 107) || (event.altKey == true && event.which == 187) || (event.altKey == true && event.which == 61)) { //Alt + "+" to navigate through the toolbar items
                    this._stopDefaultAction(event);
                    this._navigateToolbarRight(event);
                }
                else if ((event.altKey == true && event.which == 189) || (event.altKey == true && event.which == 109) || (event.altKey == true && event.which == 45) || (event.altKey == true && event.which == 173)) { // Alt + "-" to navigate back through the toolbar items
                    this._stopDefaultAction(event);
                    this._navigateToolbarLeft(event);
                }
                else if (event.ctrlKey == true && event.which == 67) { //Cancel Ctrl+C
                    this._stopDefaultAction(event);
                    this._cancel();
                }
                else if (event.which == 32) { //spacebar
                    this._stopDefaultAction(event);
                    if ($(document.activeElement)[0] === this.element.find("div.e-navigateprevious")[0]) {
                        this.element.find("div.e-navigateprevious").click();
                        this.element.find('div.e-navigateprevious').attr("tabIndex", 0).focus();
                    }
                    else if ($(document.activeElement)[0] === this.element.find("div.e-navigatenext")[0]) {
                        this.element.find("div.e-navigatenext").click();
                        this.element.find('div.e-navigatenext').attr("tabIndex", 0).focus();
                    }
                    this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                }
                else if (event.altKey == true && event.which == 78) { //New appointment window
                    this._stopDefaultAction(event);
                    this._newAppWindow(oEl, oRowCell);
                    this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                }
                else if (event.ctrlKey == true && event.which == 69) { //(E)Edit appointment window
                    this._stopDefaultAction(event);
                    var elmt = this.element.find(".e-selectedAppointment");
                    this._editKey = true;
                    this._cellClick = false;
                    this._appointmentWindow(elmt);
                    this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                }
                else if (event.which == 46) { //Delete
                    this._stopDefaultAction(event);
                    var delap = this.element.find('.e-selectedAppointment');
                    var appId = delap[0].id.split("_")[1];
                    var parentId = this._getAppointmentByParentId(appId);
                    if (delap.length != 0 && ej.isNullOrUndefined(parentId[this.model.fields["recurrenceRuleField"]]))
                        this._deleteApp(parseInt(appId));
                    else
                        this._deleteApp(parseInt(parentId[this.model.fields["idField"]]));
                }
                else if (event.shiftKey == true && event.which == 9) { //Shift + tab for reverse navigation of appointments
                    this._stopDefaultAction(event);
                    this._reverseTab(oEl, event, oRowCell);
                    this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                }
                else if (event.shiftKey == false && event.which == 9) { // For tab navigation of appointments
                    this._stopDefaultAction(event);
                    this._processTab(oEl, event, oRowCell);
                    this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                }
                else if (event.shiftKey == true && event.which == 38) { //shift + up arrow
                    this._stopDefaultAction(event);
                    this._processShiftUp(oEl, event, oRowCell);
                }
                else if (event.shiftKey == true && event.which == 40) { //shift + down arrow
                    this._stopDefaultAction(event);
                    this._processShiftDown(oEl, event, oRowCell);
                }
                else if (event.shiftKey == true && event.which == 37) { //shift+left
                    this._stopDefaultAction(event);
                    this._processShiftLeft(oEl, event, oRowCell);
                }
                else if (event.shiftKey == true && event.which == 39) { //shift+right
                    this._stopDefaultAction(event);
                    this._processShiftRight(oEl, event, oRowCell);
                }
                if (!$(oEl).hasClass("e-monthcells"))
                    this.element.find(".e-scrolltimecells").css("top", (-this.element.find('.e-draggableworkarea').scrollTop()) + "px");
            }
        },
        /**
        * Section for handle the shift up key process.
        * @private
        */
        _processShiftUp: function (oEl, event, oRowCell) {
            if ($(oEl).hasClass('e-workcells')) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    var newcur = parseInt(currrowindex) - 1;
                    if (newcur >= 0) {
                        if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                            $(oRowCell[newcur].cells[currcolindex]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                        }
                        else {
                            $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell");
                            $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                    this._currentRowIndex = newcur;
                    this._currentColIndex = currcolindex;
                }
                else {
                    $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
                    this._currentRowIndex = currrowindex;
                    this._currentColIndex = currcolindex;
                }

            }
            else if ($(oEl).hasClass('e-monthcells')) {
                var rowcount = oRowCell.length - 1, resmaxcolindex = 7, mincolindex = 0;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    var newcur = parseInt(currrowindex) - 1;
                    if (newcur >= 0) {
                        if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                            for (var i = currcolindex; i >= mincolindex; i--) {
                                $(oRowCell[currrowindex].cells[i]).attr("tabIndex", 0).focus();
                                $(oRowCell[currrowindex].cells[i]).removeClass("e-selectedCell");
                            }
                            for (var j = resmaxcolindex - 1; j >= currcolindex; j--) {
                                if (newcur >= 0) {
                                    $(oRowCell[newcur].cells[j]).attr("tabIndex", 0).focus();
                                    $(oRowCell[newcur].cells[j]).removeClass("e-selectedCell");
                                }
                            }
                            $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell");
                        }
                        else {
                            for (var i = currcolindex; i >= mincolindex; i--)
                                $(oRowCell[currrowindex].cells[i]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            for (var j = resmaxcolindex - 1; j >= currcolindex; j--) {
                                if (newcur >= 0)
                                    $(oRowCell[newcur].cells[j]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                        }
                    }
                    else
                        $(oRowCell[newcur + 1].cells[currcolindex]).attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
            else
                return false;

        },
        /**
        * Section for handle the shift down key process.
        * @private
        */
        _processShiftDown: function (oEl, event, oRowCell) {

            if ($(oEl).hasClass('e-workcells')) {
                var rowcount = oRowCell.length - 1; // this.get_RowCount();
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    var newcur = parseInt(currrowindex) + 1;
                    if (newcur <= rowcount) {
                        if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                            $(oRowCell[newcur].cells[currcolindex]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                        }
                        else {
                            $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell");
                            $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                        this._currentRowIndex = newcur;
                        this._currentColIndex = currcolindex;
                    }
                    else {
                        $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
                        this._currentRowIndex = currrowindex;
                        this._currentColIndex = currcolindex;
                    }

                }
            }
            else if ($(oEl).hasClass('e-monthcells')) {
                var rowcount = oRowCell.length - 1, resmaxcolindex = 7, mincolindex = 0;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    var newcur = parseInt(currrowindex) + 1;
                    if (newcur <= 5) {
                        if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                            for (var i = currcolindex; i <= resmaxcolindex - 1; i++) {
                                $(oRowCell[currrowindex].cells[i]).attr("tabIndex", 0).focus();
                                $(oRowCell[currrowindex].cells[i]).removeClass("e-selectedCell");
                            }
                            for (var j = mincolindex; j <= currcolindex; j++) {
                                if (newcur <= 5) {
                                    $(oRowCell[newcur].cells[j]).attr("tabIndex", 0).focus();
                                    $(oRowCell[newcur].cells[j]).removeClass("e-selectedCell");
                                }
                            }
                            $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell");
                        }
                        else {
                            for (var i = currcolindex; i <= resmaxcolindex - 1; i++)
                                $(oRowCell[currrowindex].cells[i]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            for (var j = mincolindex; j <= currcolindex; j++) {
                                if (newcur <= 5)
                                    $(oRowCell[newcur].cells[j]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                        }
                    }
                    else
                        $(oRowCell[newcur - 1].cells[currcolindex]).attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
            else
                return false;
        },
        /**
        * Section for handle the shift left key process.
        * @private
        */
        _processShiftLeft: function (oEl, event, oRowCell) {
            var rowcount = oRowCell.length - 1;
            if ($(oEl).hasClass('e-workcells')) {
                var targetElement = oEl;
                var resmaxcolindex = 0; // this.model.currentView == "day" ? 1 : this.model.currentView == "workweek" ? 5 : 7;
                var currColInx = $(targetElement)[0].cellIndex;
                var currRowInx = $(targetElement).parent()[0].rowIndex;
                var leftColInx = currColInx - 1;
                if (currColInx > 0 && leftColInx > parseInt(resmaxcolindex) - 1 && currColInx != "undefined") {
                    if ($(oRowCell[currRowInx].cells[leftColInx]).hasClass("e-selectedCell")) {
                        for (var i = 0; i <= parseInt(currRowInx) ; i++)
                            $(oRowCell[i].cells[currColInx]).removeClass("e-selectedCell");
                        for (var j = rowcount; j >= parseInt(currRowInx) ; j--)
                            $(oRowCell[j].cells[leftColInx]).removeClass("e-selectedCell");
                        $(oRowCell[currRowInx].cells[leftColInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else {
                        for (var i = 0; i <= parseInt(currRowInx) ; i++)
                            $(oRowCell[i].cells[currColInx]).addClass("e-selectedCell");
                        for (var j = rowcount; j >= parseInt(currRowInx) ; j--)
                            $(oRowCell[j].cells[leftColInx]).addClass("e-selectedCell");
                        $(oRowCell[currRowInx].cells[leftColInx]).attr("tabIndex", 0).focus();
                    }
                    this._currentRowIndex = currRowInx;
                    this._currentColIndex = leftColInx;
                }
                else {
                    $(oRowCell[currRowInx].cells[leftColInx + 1]).attr("tabIndex", 0).focus();
                    this._currentRowIndex = currRowInx;
                    this._currentColIndex = leftColInx + 1;
                }
            }
            else if ($(oEl).hasClass('e-monthcells')) {
                var rowcount = oRowCell.length - 1, mcolind = resmaxcolindex = 7, mincolindex = 0;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currcolindex >= mincolindex && currcolindex <= mcolind && currcolindex != "undefined") {
                    newcur = parseInt(currcolindex) - 1;
                    if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                        if (currcolindex == mincolindex && currrowindex >= 1) {
                            var newrow = parseInt(currrowindex) - 1;
                            var newcol = parseInt(mcolind) - 1;
                            if (newrow >= 0) {
                                $(oRowCell[newrow].cells[newcol]).attr("tabIndex", 0).focus();
                                $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell");
                                $(oRowCell[currrowindex].cells[mincolindex]).removeClass("e-selectedClass");
                            }
                        }
                        else if (currcolindex < mcolind) {
                            newcur = parseInt(currcolindex) - 1;
                            $(oRowCell[currrowindex].cells[newcur]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                        }
                    }
                    else {
                        if (currcolindex == mincolindex && currrowindex >= 1) {
                            var newrow = parseInt(currrowindex) - 1;
                            var newcol = parseInt(mcolind) - 1;
                            if (newrow >= 0)
                                $(oRowCell[newrow].cells[newcol]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            else
                                $(oRowCell[newrow + 1].cells[currcolindex]).attr("tabIndex", 0).focus();
                        }
                        else if (currcolindex < mcolind) {
                            newcur = parseInt(currcolindex) - 1;
                            $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                }
                else
                    return;
            }
            else
                return false;
        },
        /**
        * Section for handle the shift right key process.
        * @private
        */
        _processShiftRight: function (oEl, event, oRowCell) {
            if ($(oEl).hasClass('e-workcells')) {
                var rowcount = oRowCell.length - 1;
                var resmaxcolindex = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? 5 : 7;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                if (currcolindex >= 0 && currcolindex < parseInt(resmaxcolindex) - 1 && currcolindex != "undefined") {
                    if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                        for (var i = parseInt(currrowindex) ; i <= rowcount; i++)
                            $(oRowCell[i].cells[currcolindex]).removeClass("e-selectedCell");
                        for (var j = 0; j <= currrowindex; j++)
                            $(oRowCell[j].cells[newcur]).removeClass("e-selectedCell").attr("tabIndex", 0).focus();
                        $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell");
                    }
                    else {
                        for (var i = parseInt(currrowindex) ; i <= rowcount; i++)
                            $(oRowCell[i].cells[currcolindex]).addClass("e-selectedCell");
                        for (var j = 0; j <= currrowindex; j++)
                            $(oRowCell[j].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    this._currentRowIndex = currrowindex;
                    this._currentColIndex = newcur;
                }
                else
                    $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass('e-monthcells')) {
                var rowcount = oRowCell.length - 1, mcolind = resmaxcolindex = 7, mincolindex = 0;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                if (currcolindex >= 0 && currcolindex <= parseInt(resmaxcolindex) - 1 && currcolindex != "undefined") {
                    if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                        if (newcur < mcolind) {
                            $(oRowCell[currrowindex].cells[newcur]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                        }
                        if (currcolindex == parseInt(resmaxcolindex) - 1 && currrowindex <= 5) {
                            var newrow = parseInt(currrowindex) + 1;
                            currcolindex = mincolindex;
                            if (newrow <= 5) {
                                $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell").attr("tabIndex", 0).focus();
                                if (newcur < mcolind) {
                                    $(oRowCell[newrow].cells[newcur]).attr("tabIndex", 0).focus();
                                    $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell");
                                }
                            }
                        }
                    }
                    else {
                        if (newcur < mcolind)
                            $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        if (currcolindex == parseInt(resmaxcolindex) - 1 && currrowindex <= 5) {
                            var newrow = parseInt(currrowindex) + 1;
                            currcolindex = mincolindex;
                            if (newrow <= 5) {
                                $(oRowCell[newrow].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                                if (newcur < mcolind)
                                    $(oRowCell[newrow].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                            else
                                $(oRowCell[newrow - 1].cells[newcur - 1]).attr("tabIndex", 0).focus();
                        }
                    }
                }
                else
                    return false;
            }
            else
                return false;
        },
        /**
        * Section for handle the enter key process.
        * @private
        */
        _processEnterKey: function (oEl) {
            if ($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-monthcells") || $(oEl).hasClass("e-alldaycells")) {
                var tds = this.element.find('td.e-selectedCell');
                var nt = tds.length - 1;
                if (nt == 0) {
                    this._cellSelection(oEl);
                }
                else {
                    this._multipleCellAppointCreation(oEl);
                }
            }
            else if ($(oEl).hasClass("e-detailedapp e-detailedwindow")) {
                this._appointmentWindow(oEl);
            }
            else if ($(oEl).parent().hasClass("e-allday")) {
                var e = this._appointmentAddWindow.find("#" + this._id + "allday").data("ejCheckBox");
                var checkedVal = e.isChecked ? false : true;
                this._appointmentAddWindow.find(".allday").ejCheckBox({ check: checkedVal });
            }
            else if ($(oEl).parent().hasClass("e-recurrence")) {
                var e = this._appointmentAddWindow.find("#" + this._id + "recurrence").data("ejCheckBox");
                var checkedVal = e.isChecked ? false : true;
                this._appointmentAddWindow.find(".recurrence").ejCheckBox({ check: checkedVal });
                this._recurChecked = checkedVal ? true : false;
                this._repeat(oEl);
            }
            else if ($(oEl).hasClass("e-scheduledelete")) {
                this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-scheduledelete').click();
            }
            else if ($(oEl).hasClass("e-editevent"))
                this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-editevent').click();
            else if ($(oEl).hasClass(".e-editseries"))
                this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-editseries').click();
            else if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment"))
                this._cellSelection(oEl);
            else if ($(oEl).parent().hasClass("e-weekday")) {
                var e = this._appointmentAddWindow.find("#" + $(oEl).children()[0].id).data("ejCheckBox");
                var checkedVal = e.isChecked ? false : true;
                this._appointmentAddWindow.find("#" + $(oEl).children()[0].id).ejCheckBox({ check: checkedVal });
            }
        },
        /**
        * To find the minimum and maximum cell index of the schedule.
        * @private
        */
        _findMinMaxCellIndex: function () {
            var oRowCell = this.element.find('div.e-draggableworkarea tr');
            var rowcount = oRowCell.length - 1;
            var omincolIndex, omaxcolIndex, rowInx, maxrowInx, checkcolindex;
            var curr_View = this.currentView();
            var proxy = this;
            this.element.find('td.e-selectedCell').each(function (index, tdItem) {
                var targetId = $(tdItem).index();
                if (targetId != null && !isNaN(targetId)) {
                    if (omincolIndex == null) {
                        omincolIndex = targetId;
                        rowInx = $(tdItem).parent()[0].rowIndex;
                    }
                    else if (omincolIndex < targetId) {
                        omincolIndex = omincolIndex;
                    }
                    else {
                        if (omincolIndex != targetId) {
                            omincolIndex = targetId;
                            rowInx = $(tdItem).parent()[0].rowIndex;
                        }
                    }
                    if (omaxcolIndex == null) {
                        omaxcolIndex = targetId;
                        if (curr_View != "month") {
                            checkcolindex = parseInt(omaxcolIndex);
                            for (var i = 0; i <= rowcount; i++) {
                                if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                    maxrowInx = i;

                                }
                            }
                        }
                    }
                    else if (omaxcolIndex > targetId) {
                        omaxcolIndex = omaxcolIndex;
                    }
                    else {
                        if (omaxcolIndex != targetId) {
                            omaxcolIndex = targetId;
                            if (curr_View != "month") {
                                checkcolindex = parseInt(omaxcolIndex);
                                for (var i = 0; i <= rowcount; i++) {
                                    if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                        maxrowInx = i;

                                    }
                                }
                            }
                        }
                    }
                }
            });
            var _minMaxIndexes = { rowInx: rowInx, maxrowInx: maxrowInx, omincolIndex: omincolIndex, omaxcolIndex: omaxcolIndex };
            return _minMaxIndexes;
            //}

        },
        /**
        * Section for handle the multiple cell appointment creation.
        * @private
        */
        _multipleCellAppointCreation: function (target) {
            var indexes = this._findMinMaxCellIndex(); var renderDates = this._dateRender;
            var omincolIndex = indexes.omincolIndex, omaxcolIndex = indexes.omaxcolIndex, rowInx = indexes.rowInx, maxrowInx = indexes.maxrowInx;
            var minStartDate = renderDates[omincolIndex]; var maxEndDate = renderDates[omaxcolIndex];
            var localizeDays = Globalize.culture(this.model.localization);
            if ($(target).hasClass("e-workcells")) {
                var minStartTime = (this.model.startHour + (rowInx / 2)).toString().split(".");
                var maxEndTime = (this.model.startHour + (maxrowInx / 2)).toString().split(".");
                this.cur_StartTime = new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00);
                this.cur_EndTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                this.cur_EndTime = new Date(new Date(this.cur_EndTime).setMinutes(new Date(this.cur_EndTime).getMinutes() + 30));
            }
            else if ($(target).hasClass("e-monthcells")) {
                var selCell = this.element.find('td.e-selectedCell');
                var nsc = selCell.length - 1;
                var startIndex = ($(selCell[0]).parent().index() * 7) + $(selCell[0]).index();
                var endIndex = ($(selCell[nsc]).parent().index() * 7) + $(selCell[nsc]).index();
                var startDate = new Date(renderDates[startIndex]);
                var endDate = new Date(renderDates[endIndex]);
                this.cur_StartTime = new Date(new Date(startDate).setHours(9));
                this.cur_StartTime = new Date(this.cur_StartTime).setMinutes(00); //.setHours(00)).setMinutes(00);
                this.cur_EndTime = new Date(new Date(endDate).setHours(9)); //.setHours(23)).setMinutes(59)); 
                this.cur_EndTime = new Date(new Date(this.cur_EndTime).setMinutes(30));
            }
            if (this.model.showQuickWindow) {
                this._quickAppointWindow.find("#" + this._id + "_quickAppwindow").css("display", "block");

                //add the dialog code to display while clicking on the cells
                if (((new Date(this.cur_EndTime) - new Date(this.cur_StartTime)) / 3600000) < 24)
                    this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(this.cur_StartTime).getDay()] + ", " + Globalize.format(new Date(this.cur_StartTime), "MMMM") + " " + new Date(this.cur_StartTime).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), "hh.mm tt") + " - " + Globalize.format(new Date(this.cur_EndTime), "hh.mm tt"));
                else
                    this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(this.cur_StartTime).getDay()] + ", " + Globalize.format(new Date(this.cur_StartTime), "MMMM") + " " + new Date(this.cur_StartTime).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), "hh.mm tt") + " - " + localizeDays.calendar.days.names[new Date(this.cur_EndTime).getDay()] + ", " + Globalize.format(new Date(this.cur_EndTime), "MMMM") + " " + new Date(this.cur_EndTime).getDate() + ", " + Globalize.format(new Date(this.cur_EndTime), "hh.mm tt"));
                this._quickAppointWindow.find('.subject').val(""); var x = target.offsetLeft; var y = target.offsetTop;
                this._quickAppointWindow.ejDialog({ position: { X: x, Y: y } });
                this._quickAppointWindow.ejDialog("open");
                this._quickAppointWindow.find(".subject").focus();
            }
            this._cellClick = true;

        },
        /**
        * Section for handle the cell selection process using keyboard.
        * @private
        */
        _addCellSelection: function (oRowCell) {
            var tds = this.element.find('td.e-selectedCell');
            var nt = tds.length - 1;
            if (nt > 0) {
                this.element.find('td.e-selectedCell').removeClass("e-selectedCell");
                $(oRowCell[this._currentRowIndex].cells[this._currentColIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
        },
        /**
        * Section for handle the up arrow key process.
        * @private
        */
        _processUpArrow: function (oEl, oRowCell) {
            if ($(oEl).hasClass("e-selectedAppointment")) {
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-selectedCell"))
                this._addCellSelection(oRowCell);
            var allda = this.element.find(".e-alldaycells");
            if ($(oEl).hasClass("e-workcells")) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currrowindex) - 1;
                if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (currrowindex == 0) {
                    var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                    var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(this.element).attr("tabIndex", 0).focus();
                    $(allda[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
            }
            else if ($(oEl).hasClass("e-alldaycells")) {
                return false;
            }

            else if ($(oEl).hasClass("e-monthcells")) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currrowindex) - 1;
                if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    if (newcur >= 0) {
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[0].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
            else
                return false;

        },
        /**
        * Section for handle the down arrow key process.
        * @private
        */
        _processDown: function (oEl, oRowCell) {
            if ($(oEl).hasClass("e-selectedAppointment")) {
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-selectedCell"))
                this._addCellSelection(oRowCell);
            if ($(oEl).hasClass("e-workcells")) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currrowindex) + 1;
                if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    if (newcur <= rowcount) {
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
                }
            }
            else if ($(oEl).hasClass("e-alldaycells")) {
                var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex - 1;
                var allda = this.element.find(".e-alldaycells");
                $(allda[allDayInx]).removeClass("e-selectedCell").blur();
                $(oRowCell[0].cells[allDayInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }

            else if ($(oEl).hasClass("e-monthcells")) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currrowindex) + 1;
                if (currrowindex >= 0 && currrowindex < rowcount && currrowindex != "undefined") {
                    if (newcur <= 7) {
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[newcur - 1].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
            }
            else
                return false;
        },
        /**
        * Section for handle the right arrow key process.
        * @private
        */
        _processRight: function (oEl, oRowCell) {
            if ($(oEl).hasClass("e-selectedAppointment")) {
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-selectedCell"))
                this._addCellSelection(oRowCell);
            if ($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-monthcells")) {
                var maxcolindex = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? 5 : 7;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                if (currcolindex >= 0 && currcolindex < parseInt(maxcolindex) - 1 && currcolindex != "undefined") {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (currcolindex == parseInt(maxcolindex) - 1) {
                    var maxrowInx1 = $(oEl).parent()[0].rowIndex;
                    this.element.find("div.e-navigatenext").click();
                    $(oRowCell[maxrowInx1].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
            else if ($(oEl).hasClass("e-alldaycells")) {
                var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex - 1;
                var allda = this.element.find(".e-alldaycells");
                var nall = parseInt(allda.length) - 1;
                var newallDayInx = parseInt(allDayInx) + 1;
                if (allDayInx >= 0 && allDayInx < nall) {
                    $(allda[allDayInx]).removeClass("e-selectedCell").blur();
                    $(allda[newallDayInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (allDayInx == nall) {
                    this.element.find("div.e-navigatenext").click();
                    $(allda[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }

        },
        /**
        * Section for handle the left arrow key process.
        * @private
        */
        _processLeft: function (oEl, oRowCell) {
            if ($(oEl).hasClass("e-selectedAppointment")) {
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-selectedCell"))
                this._addCellSelection(oRowCell);
            if ($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-monthcells")) {
                var maxcolindex = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? 5 : 7;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currcolindex) - 1;
                if (currcolindex > 0 && currcolindex <= parseInt(maxcolindex) - 1 && currcolindex != "undefined") {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (currcolindex == 0) {
                    var maxrowInx1 = $(oEl).parent()[0].rowIndex;
                    this.element.find("div.e-navigateprevious").click();
                    $(oRowCell[maxrowInx1].cells[maxcolindex - 1]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
            else if ($(oEl).hasClass("e-alldaycells")) {
                var maxcolindex = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? 5 : 7;
                var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex - 1;
                var allda = this.element.find(".e-alldaycells");
                var nall = parseInt(allda.length) - 1;
                var newallDayInx = parseInt(allDayInx) - 1;
                if (allDayInx > 0 && allDayInx <= nall) {
                    $(allda[allDayInx]).removeClass("e-selectedCell").blur();
                    $(allda[newallDayInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (allDayInx == 0) {
                    this.element.find("div.e-navigateprevious").click();
                    $(allda[maxcolindex - 1]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
        },
        /**
        * Section for handle the navigate toolbar right key process.
        * @private
        */
        _navigateToolbarRight: function (event) {
            var views = this.model.views;
            var curViewIndex = this.element.find(".e-" + this.currentView()).parent().index();
            if (curViewIndex < views.length - 1) {
                var newView = views[curViewIndex + 1].toLowerCase(); this._navView = true;
                this._viewChange(event, this.currentView(), newView);
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
            }
        },
        /**
        * Section for handle the navigate toolbar left key process.
        * @private
        */
        _navigateToolbarLeft: function (event) {
            var curViewIndex = this.element.find(".e-" + this.currentView()).parent().index();
            if (curViewIndex > 0) {
                var newView = this.model.views[curViewIndex - 1].toLowerCase(); this._navView = true;
                this._viewChange(event, this.currentView(), newView);
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
            }
        },
        /**
        * Section to open the new appointment window using keyboard.
        * @private
        */
        _newAppWindow: function (contextElement, oRowCell) {
            if ($(contextElement).hasClass("e-selectedCell")) {
                this._currentAction = ej.Schedule.actions.Add;
                var targetElement = contextElement == null ? event : contextElement;
                var rowIndex = $(targetElement).parent().index();
                var columnIndex = $(targetElement).index();
                var resourceIndex = 0, renderDates = this._dateRender;
                var s = this.element.find('td.e-selectedCell');
                var ns = parseInt(s.length) - 1;
                if (ns == 0) {
                    var index = $(targetElement).hasClass("e-workcells") ? $(targetElement).index() : $(targetElement).hasClass("e-alldaycells") ? $(targetElement).index() - 1 : ($(targetElement).parent().index() * 7) + $(targetElement).index();
                    startdate = new Date(renderDates[index]);
                    if ($(targetElement).hasClass("e-workcells")) {
                        var time = this.model.startHour + (rowIndex / 2);
                        var timemin = time.toString().split(".");
                        startTime = new Date(startdate).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                        endTime = new Date(new Date(startTime).setMinutes(new Date(startTime).getMinutes() + 30));
                    }
                    else if ($(targetElement).hasClass("e-alldaycells")) {
                        startTime = new Date(startdate.setHours(0, 0));
                        endTime = new Date(startTime.setHours(0, 0));
                        this._appointmentAddWindow.find(".allday ").ejCheckBox({ check: true });
                    }
                    else if ($(targetElement).hasClass("e-monthcells")) {
                        startTime = new Date(new Date(startdate).setHours(9));
                        startTime = new Date(startTime).setMinutes(00); //.setHours(00)).setMinutes(00);
                        endTime = new Date(new Date(startTime).setMinutes(30)); //.setHours(23)).setMinutes(59));
                    }
                }
                else if (ns > 0) {
                    var indexes = this._findMinMaxCellIndex();
                    var omincolIndex = indexes.omincolIndex, omaxcolIndex = indexes.omaxcolIndex, rowInx = indexes.rowInx, maxrowInx = indexes.maxrowInx;
                    if ($(contextElement).hasClass('e-workcells')) {
                        var minStartDate = renderDates[omincolIndex]; var maxEndDate = renderDates[omaxcolIndex];
                        var minStartTime = (this.model.startHour + (rowInx / 2)).toString().split(".");
                        var maxEndTime = (this.model.startHour + (maxrowInx / 2)).toString().split(".");
                        startTime = new Date(new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00));
                        endTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                        endTime = new Date(new Date(endTime).setMinutes(new Date(endTime).getMinutes() + 30));
                    }
                    else if ($(contextElement).hasClass('e-monthcells')) {
                        var selCell = this.element.find('td.e-selectedCell');
                        var nsc = selCell.length - 1;
                        var startIndex = ($(selCell[0]).parent().index() * 7) + $(selCell[0]).index();
                        var endIndex = ($(selCell[nsc]).parent().index() * 7) + $(selCell[nsc]).index();
                        var startDate = new Date(renderDates[startIndex]);
                        var endDate = new Date(renderDates[endIndex]);
                        startTime = new Date(new Date(startDate).setHours(9));
                        startTime = new Date(new Date(startTime).setMinutes(00)); //.setHours(00)).setMinutes(00);
                        endTime = new Date(new Date(endDate).setHours(9)); //.setHours(23)).setMinutes(59)); 
                        endTime = new Date(new Date(endTime).setMinutes(30));
                    }
                }
                document.getElementById(this._id + "startdate").value = Globalize.format(new Date(startTime), this._pattern.d);
                document.getElementById(this._id + "enddate").value = Globalize.format(new Date(endTime), this._pattern.d);
                document.getElementById(this._id + "starttime").value = Globalize.format(new Date(startTime), this._pattern.t);
                document.getElementById(this._id + "endtime").value = Globalize.format(new Date(endTime), this._pattern.t);
                if (this.model.timeZone == null) {
                    this._appointmentAddWindow.find("#startutc").val("UTC " + Globalize.format(new Date(startTime), "zzz"));
                    this._appointmentAddWindow.find("#endutc").val("UTC " + Globalize.format(new Date(endTime), "zzz"));
                }
                else {
                    this._appointmentAddWindow.find("#startutc").val(this.model.timeZone);
                    this._appointmentAddWindow.find("#endutc").val(this.model.timeZone);
                }
                this._alldayCheck();
                this._appointmentAddWindow.ejDialog("open");
                this._appointmentAddWindow.find('.subject').focus();
                this._cellClick = false;
            }

        },
        /**
        * Section for handle the tab key process.
        * @private
        */
        _processTab: function (oEl, event, oRowCell) {
            var appo = this.element.find('div.e-appointment,div.e-alldayappointment,div.e-monthappointment');
            if (oEl.id == this._id || $(oEl).hasClass("e-datedisplay")) {
                if (appo.length != 0)
                    $(appo[0]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                else
                    $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            if ($(oEl).hasClass("e-selectedCell")) {
                if (appo.length != 0) {
                    var selcel = this.element.find('td.e-selectedCell');
                    var n = parseInt(selcel.length) - 1;
                    if (n > 0) {
                        for (var i = 0; i <= n; i++) {
                            selcel[i].blur();
                            $(selcel[i]).removeClass("e-selectedCell");
                        }
                    }
                    else
                        $(oEl).removeClass("e-selectedCell").blur();
                    $(appo[0]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                }
            }
            if ($(oEl).hasClass("e-navigateprevious")) {
                this.element.find('.e-navigatenext').attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-navigatenext"))
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment")) {
                var appcount = appo.length;
                var appindex = this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").index($(".e-selectedAppointment"));
                if (appindex < appcount && appindex >= 0) {
                    $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                    var newappindex = parseInt(appindex) + 1;
                    if (newappindex < appcount)
                        $(appo[newappindex]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                    else if (newappindex == appcount)
                        this.element.find('.e-navigateprevious').attr("tabIndex", 0).focus();
                }
                else if (appindex == appcount)
                    this.element.find('.e-navigateprevious').attr("tabIndex", 0).focus();
            }
        },
        /**
        * Section for handle the shift tab key process.
        * @private
        */
        _reverseTab: function (oEl, event, oRowCell) {
            var appo = this.element.find('div.e-appointment,div.e-alldayappointment,div.e-monthappointment');
            if (oEl.id == this._id || $(oEl).hasClass("e-datedisplay")) {
                var len = parseInt(appo.length) - 1;
                if (appo.length != 0)
                    $(appo[len]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
            }
            var appcount = appo.length;
            if ($(oEl).hasClass("e-selectedCell")) {
                var len = appo.length - 1;
                if (appo.length != 0) {
                    var selcel = this.element.find('td.e-selectedCell');
                    var n = parseInt(selcel.length) - 1;
                    if (n > 0) {
                        for (var i = 0; i <= n; i++)
                            $(selcel[i]).removeClass("e-selectedCell").blur();
                    }
                    else
                        $(oEl).removeClass("e-selectedCell").blur();
                    $(appo[len]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                }
            }
            if ($(oEl).hasClass("e-navigatenext"))
                $(this.element).find('.e-navigateprevious').attr("tabIndex", 0).focus();
            else if ($(oEl).hasClass("e-navigateprevious"))
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment")) {
                var appindex = this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").index($(".e-selectedAppointment"));
                if (appindex < appcount && appindex > 0) {
                    $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                    var newappindex = parseInt(appindex) - 1;
                    if (newappindex < appcount && appindex >= 0)
                        $(appo[newappindex]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                    else if (newappindex == 0)
                        $(this.element).find('.e-navigatenext').attr("tabIndex", 0).focus();
                }
                else if (appindex == 0) {
                    $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                    $(this.element).find('.e-navigatenext').attr("tabIndex", 0).focus();
                }
            }
        },
        /**
        * Section to stop the default action.
        * @private
        */
        _stopDefaultAction: function (e) {
            e.cancelBubble = true; e.returnValue = false; e.stopPropagation(); e.preventDefault();
        },
        /**
        * To get the localized labels.
        * @private
        */
        //get Localization text
        _getLocalizedLabels: function (property) {
            return ej.Schedule.localization[this.model.localization][property] === undefined ? ej.Schedule.localization["en-US"][property] : ej.Schedule.localization[this.model.localization][property];
        },
        /**
        * To save the appointment  
        * @return jQuery
        * @param {object} argument.obj obj of the appointment
        * @example 
        * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var schObj = $("#schedule1").data("ejSchedule");
        * schObj.saveAppointment(obj); //obj contains collection of appointment. 
        * // save the appointment
        * &lt;/script&gt;
        * @example
        * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#schedule1").ejSchedule("saveAppointment(obj)");
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        //public methods
        saveAppointment: function (obj) {
            var appointments = this._sortAppById(this.dataSource()), maxId;
            maxId = appointments[appointments.length - 1][this.model.fields["idField"]] + 1;
            if (ej.isNullOrUndefined(obj[this.model.fields["idField"]]) || ej.isNullOrUndefined(obj[this.model.fields["idField"]])) {
                obj[this.model.fields["idField"]] = maxId;
                this._dataManager.insert(obj);
                this._appointmentProcessing(obj);
            }
            else {
                this._dataManager.update(this.model.fields["idField"], obj);
                obj[this.model.fields["recurrenceField"]] ? this._dataProcessing(this.dataSource()) : new ej.DataManager(this._processed).update(this.model.fields["idField"], obj);
            }
            this._renderAppointmentAll();
        },
        /**
        * To delete the appointment  
        * @return jQuery
        * @param {number} argument.id id value of the appointment
        * @example 
        * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var schObj = $("#schedule1").data("ejSchedule");
        * schObj.deleteAppointment(105); //Appointments id number. 
        * // delete the appointment
        * &lt;/script&gt;
        * @example
        * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $('#schedule1').ejSchedule("deleteAppointment(105)");
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        deleteAppointment: function (id) {
            this._deleteApp(id);
            //this._renderAppointmentAll();
        },
        /**
        * To get the appointment list  
        * @return object
        * @example 
        * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var schObj = $("#schedule1").data("ejSchedule");
        * var appointments=schObj.getAppointments();
        * // get the appointments list
        * &lt;/script&gt;
        * @example 
        * &lt;div id="schedule1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#schedule1").ejSchedule("getAppointments()");
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        getAppointments: function () {
            var appList = this.dataSource();
            return appList;
        }
    });

    //Schedule Localization 
    ej.Schedule.localization = {};

    //Default english localized values
    ej.Schedule.localization["en-US"] = {
        ReminderWindowTitle: "Reminder window",
        CreateAppointmentTitle: "Create Appointment",
        RecurrenceEditTitle: "Edit repeat appointment",
        RecurrenceEditMessage: "How would you like to change the event in the series?",
        RecurrenceEditOnly: "Only this appointment",
        RecurrenceEditSeries: "Entire series",
        AppointmentSubject: "Subject",
        StartTime: "Start Time",
        EndTime: "End Time",
        AllDay: "All day",
        Today: "Today",
        Recurrence: "Repeat",
        Done: "Done",
        Cancel: "Cancel",
        RepeatBy: "Repeat by",
        RepeatEvery: "Repeat every",
        RepeatOn: "Repeat on",
        StartsOn: "Starts on",
        Ends: "Ends",
        Summary: "Summary",
        Daily: "Daily",
        Weekly: "Weekly",
        Monthly: "Monthly",
        Yearly: "Yearly",
        Every: "Every",
        EveryWeekDay: "Every weekday",
        Never: "Never",
        After: "After",
        Occurence: "Occurrence(s)",
        On: "On",
        Edit: "Edit",
        RecurrenceDay: "Day(s)",
        RecurrenceWeek: "Week(s)",
        RecurrenceMonth: "Month(s)",
        RecurrenceYear: "Year(s)",
        The: "The",
        OfEvery: "of every",
        First: "First",
        Second: "Second",
        Third: "Third",
        Fourth: "Fourth",
        Last: "Last",
        WeekDay: "Weekday",
        WeekEndDay: "Weekendday",
        Subject: "Subject",
        DueIn: "Due In",
        DismissAll: "Dismiss All",
        Dismiss: "Dismiss",
        OpenItem: "Open Item",
        Snooze: "Snooze",
        Day: "Day",
        Week: "Week",
        WorkWeek: "Work Week",
        Month: "Month",
        Agenda: "Agenda",
        Detailed: "Detailed",
        EventBeginsin: "Appointment Begins in",
        Editevent: "Edit Appointment",
        Editseries: "Edit series",
        Times: "times",
        Until: "until",
        Eventwas: "Appointment was",
        Hours: "hrs",
        Minutes: "mins",
        Overdue: "Overdue Appointment",
        Days: "day(s)",
        Event: "Subject",
        Select: "select",
        Previous: "prev",
        Next: "next",
        Close: "close",
        Delete: "delete"
    };
    /**
    * Enum for various schedule actions while create/edit appointments	 
    * @enum {string}
    * @global 
    */
    ej.Schedule.actions = {
        /**  support to know the add action while create appointment in the schedule */
        Add: "add",
        /**  support to know the save action while create appointment in the schedule */
        Save: "save",
        /**  support to know the delete action while delete appointment in the schedule */
        Delete: "delete",
        /**  support to know the edit occurance action while edit occurrence appointment in the schedule */
        EditOcurrence: "editOcurrence",
        /**  support to know the edit series action while edit series appointment in the schedule */
        EditSeries: "editSeries"
    };
    /**
    * Enum for various schedule view	 
    * @enum {string}
    * @global 
    */
    ej.Schedule.CurrentView = {
        /**  Creates schedule with default day view */
        Day: "day",
        /**  Creates schedule with default week view */
        Week: "week",
        /**  Creates schedule with default work week view */
        Workweek: "workweek",
        /**  Creates schedule with default month view */
        Month: "month"
    };
    /**
    * Enum for various schedule timemode	 
    * @enum {string}
    * @global 
    */
    ej.Schedule.TimeMode = {
        /**  Creates schedule with default twelve hour time mode */
        Hour12: "12",
        /**  Creates schedule with default twenty four hour time mode */
        Hour24: "24"
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.Grid = ej.Grid || {};
    ej.Grid.gridResize = function (instance) {
        this.$headerTable = instance.getGridHeaderTable();
        this.gridInstance = instance;
        this._colMinWidth = 15;
        this._$visualElement = $();
        this._currentCell = -1;
        this._allowStart = false;
        this._oldWidth = null;
        this._orgX = null;
        this._orgY = null;
        this._extra = null;
        this._expand = false;
    }

    ej.Grid.gridResize.prototype = {
        _mouseHover: function (e) {
            if (this._$visualElement.is(":visible"))
                return;
            if ($(e.target).is(".e-headercelldiv"))
                e.target = e.target.parentNode;
            var $target = $(e.target);
            if ($target.hasClass("e-headercell")) {
                var _resizableCell = e.target;
                var location = _resizableCell.getBoundingClientRect(), _x = 0, _y = 0;
                if (e.type = "mousemove") {
                    _x = e.clientX;
                    _y = e.clientY;
                }
                else if (e.type = "touchmove") {
                    _x = evt.originalEvent.changedTouches[0].clientX;
                    _y = evt.originalEvent.changedTouches[0].clientY;
                }
                else if (e.type = "MSPointerMove") {
                    _x = e.originalEvent.clientX;
                    _y = e.originalEvent.clientY;
                }
                if (this.gridInstance.model.scrolling && this.gridInstance.model.scrolling.frozenColumns)
                    var _nlx = this.gridInstance.getGridHeaderContent().width() + this.gridInstance.element.children(".e-gridheader").find(".e-columnheader").offset().left;
                else
                    var _nlx = this.gridInstance.getGridHeaderTable().width() + this.gridInstance.element.children(".e-gridheader").find(".e-columnheader").offset().left;
                if (((_x >= (location.left + document.documentElement.scrollLeft + _resizableCell.offsetWidth - 5)) || ((_x <= (location.left + 3)))) && (_x < _nlx) && (_x >= location.left) && (_y <= location.top + document.documentElement.scrollTop + e.target.offsetHeight)) {
                    $target.parent().css("cursor", "col-resize");
                    this._currentCell = this.gridInstance.getGridHeaderContent().find(".e-headercell").index(_resizableCell);
                    this._allowStart = true;
                }
                else {
                    $target.parent().css("cursor", "pointer");
                    this._allowStart = false;
                    this._currentCell = -1;
                }
            }
        },
        _start: function (_x, _y) {
            var _myrow = this.gridInstance.getGridHeaderTable().find(".e-columnheader");
            var _cells = _myrow.find(".e-headercell"), _mycel;
            if (this._currentCell != -1 && this._currentCell < _cells.length)
                _mycel = _cells[this._currentCell];
            if (typeof (_mycel) == 'undefined')
                return;
            var _j = _mycel.getBoundingClientRect();
            this._tableY = _j.top + parseInt(navigator.userAgent.indexOf("WebKit") != -1 ? document.body.scrollTop : document.documentElement.scrollTop);
            if (this._allowStart) {
                this._$visualElement = $(document.createElement('div'));
                _height = this.gridInstance.element.find(".e-gridcontent").first().height() + this.gridInstance.element.find(".e-gridheader").height();
                this._$visualElement.addClass("e-reSizeColbg").appendTo(this.gridInstance.element).css({ height: _height + 'px', cursor: 'col-resize' });
                this._$visualElement.css({ left: _x, top: this._tableY });
                this._oldWidth = _mycel.offsetWidth;
                this._orgX = _x;
                this._orgY = _y;
                this._extra = _x - this._orgX;
                this._expand = true;
            }
            else {
                this._currentCell = -1;
            }
        },
        _mouseMove: function (e) {
            if (this._expand) {
                var _x = 0, _y = 0;
                if (e.type = "mousemove") {
                    _x = e.clientX;
                    _y = e.clientY;
                }
                else if (e.type = "touchmove") {
                    _x = evt.originalEvent.changedTouches[0].clientX;
                    _y = evt.originalEvent.changedTouches[0].clientY;
                }
                else if (e.type = "MSPointerMove") {
                    _x = e.originalEvent.clientX;
                    _y = e.originalEvent.clientY;
                }
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY;
                }
                _x += document.documentElement.scrollLeft;
                e.preventDefault();
                this._moveVisual(_x);
            }
            else
                this._mouseHover(e);
        },
        _reSize: function (_x, _y) {
            // Function used for Resizing the column
            var proxy = this;
            var resized = false, $content;
            if (this.gridInstance.model.scrolling && this.gridInstance.model.scrolling.frozenColumns)
                this._initialTableWidth = this.gridInstance.getGridHeaderTable().first().parent().width() + this.gridInstance.getGridHeaderTable().last().parent().width();
            else
                this._initialTableWidth = this.gridInstance.getGridHeaderTable().parent().width();
            !this.gridInstance.model.rtl && this._getResizableCell();
            var _rowobj = this.gridInstance.getGridHeaderTable().find(".e-columnheader");
            if (this._currentCell != -1 && this._expand) {
                this._expand = false;
                var _childTH = _rowobj.find(".e-headercell").filter(":visible");
                var _outerCell = _childTH[this._currentCell];
                var _oldWidth = _outerCell.offsetWidth;
                var _extra = _x - this._orgX;
                //Check whether the column minimum width reached
                if (parseInt(_extra) + parseInt(_oldWidth) > this._colMinWidth) {
                    if (_extra != 0)
                        _rowobj.css("cursor", 'default');
                    this._resizeColumnUsingDiff(_oldWidth, _extra);
                    $content = this.gridInstance.element.find(".e-gridcontent").first();
                    this.gridInstance.model.allowScrolling && $content.ejScroller("refresh");
                    var browser = this.gridInstance.getBrowserDetails();
                    if (browser.browser == "msie") {
                        var oldWidth = this.gridInstance.getGridContentTable().width(), newwidth = this.gridInstance._calculateWidth();
                        if (this.gridInstance.model.scrolling && this.gridInstance.model.scrolling.frozenColumns > 0) {
                            this.gridInstance.getGridHeaderTable().last().width(newwidth - this.gridInstance.getGridHeaderContent().find(".e-frozenheaderdiv").width());
                            this.gridInstance.getGridContentTable().last().width(newwidth - this.gridInstance.getGridContent().find(".e-frozencontentdiv").width());
                            this.gridInstance.model.allowSummary && this.gridInstance.getGridFooterTable().last().width(newwidth - this.gridInstance.getGridFooterContent().find(".e-frozenfootertdiv").width());
                        }
                        else {
                            this.gridInstance.getGridHeaderTable().width(newwidth);
                            this.gridInstance.model.allowSummary && this.gridInstance.getGridFooterTable().width(newwidth);
                        }
                        if (parseInt(browser.version, 10) > 8 && this.gridInstance.model.groupSettings && this.gridInstance.model.groupSettings.groupedColumns.length) {
                            if (newwidth > oldWidth) {
                                this.gridInstance.getGridContentTable().width(newwidth);
                                this.gridInstance.getGridContentTable().children("colgroup").find("col").first().css("width", (20 / $content.find("table").first().width()) * 100 + "%");
                            }
                            else {
                                this.gridInstance.getGridContentTable().css("width", "100%");
                                this.gridInstance._groupingAction();
                                this.gridInstance.getGridContentTable().children("colgroup").find("col").first().css("width", ((this.gridInstance.getGridHeaderTable().find("colgroup").find("col").first().width() / $content.find("table").first().width()) * 100).toFixed(2) + "%");
                            }
                        }
                        this.gridInstance.getGridHeaderTable().parent().scrollLeft($content.find(".e-content").scrollLeft() - 1);
                    }
                }

            }
            this._$visualElement.remove();
            this._expand = false;
            this._currentCell = -1;
            this._allowStart = false;

        },
        _getFrozenResizeWidth: function () {
            var $frozenColumnsCol = this.gridInstance.getGridHeaderTable().find('colgroup').find("col").slice(0, this.gridInstance.model.scrolling ? this.gridInstance.model.scrolling.frozenColumn : 0), width = 0;
            for (var i = 0; i < $frozenColumnsCol.length; i++) {
                if ($frozenColumnsCol.eq(i).css("display") != "none")
                    width += parseInt($frozenColumnsCol[i].style.width.replace("px", ""));
            }
            return width;
        },
        _diaplayFinder: function () {
            return $(this).css('display') != 'none';
        },
        _resizeColumnUsingDiff: function (_oldWidth, _extra) {
            var proxy = this;
            var _newWidth = parseInt(_extra) + parseInt(_oldWidth);
            if (_newWidth > 0 && _extra != 0) {
                if (_newWidth < this._colMinWidth)
                    _newWidth = this._colMinWidth;
                var _extra = _newWidth - _oldWidth;
                var $headerCols = this.gridInstance.getGridHeaderTable().find('colgroup').find("col"), $headerCol = $headerCols.filter(this._diaplayFinder).eq(this._currentCell)
                    , $ContentCol, $footerCol, $frozenCols = $headerCols.slice(0, this.gridInstance.model.scrolling ? this.gridInstance.model.scrolling.frozenColumns : 0);
                if (this.gridInstance.model.scrolling && this.gridInstance.model.scrolling.frozenColumns) {
                    if (this._currentCell >= 0 && this._currentCell < this.gridInstance.model.scrolling.frozenColumns && this._getFrozenResizeWidth() + _extra > this.gridInstance.element.find(".e-headercontent").first().width())
                        return;
                    $ContentCol = this.gridInstance.getGridContentTable().find('colgroup').find("col").filter(this._diaplayFinder).eq(this._currentCell);
                }
                else
                    $ContentCol = this.gridInstance.getGridContentTable().find('colgroup').first().find("col").filter(this._diaplayFinder).eq(this._currentCell);
                if (this.gridInstance.model.allowSummary) {
                    $footerCol = this.gridInstance.getGridFooterTable().find('colgroup').find("col").filter(this._diaplayFinder).eq(this._currentCell);
                    $footerCol.outerWidth(_newWidth);
                }
                $headerCol.outerWidth(_newWidth);
                if (this.gridInstance.model.isEdit) {
                    var $editCol = this.gridInstance.getGridContentTable().find(".e-editedrow,.e-addedrow").find("table").find("colgroup");
                    $($editCol[0].children[this._currentCell - this.gridInstance.model.groupSettings ? this.gridInstance.model.groupSettings.groupedColumns.length : 0]).width(_newWidth);
                }
                if (this.gridInstance.model.groupSettings && this.gridInstance.model.groupSettings.groupedColumns.length) {
                    var $tables = this.gridInstance.getGridContentTable().find(".e-recordtable");
                    var $colGroup = $tables.find("colgroup");
                    for (var i = 0 ; i < $colGroup.length; i++)
                        $($colGroup[i].children[this._currentCell - this.gridInstance.model.groupSettings.groupedColumns.length]).width(_newWidth);
                }
                $ContentCol.width(_newWidth);
                this.gridInstance._findColumnsWidth();
                if (this.gridInstance.model.scrolling && this.gridInstance.model.scrolling.frozenColumns) {
                    var frozenColumns = this.gridInstance.getGridContentTable().find('colgroup').find("col").slice(0, this.gridInstance.model.scrolling.frozenColumns)
                        , width = 0, direction;
                    for (i = 0; i < frozenColumns.length; i++)
                        width += parseInt(frozenColumns[i].style.width.replace("px", ""));
                    this.gridInstance.getGridHeaderContent().find(".e-frozenheaderdiv").width(width);
                    direction = this.gridInstance.model.rtl ? "margin-right" : "margin-left";
                    this.gridInstance.getGridContent().find(".e-frozencontentdiv").width(width).next().css(direction, width + "px");
                    this.gridInstance.getGridHeaderContent().find(".e-frozenheaderdiv").width(width).next().css(direction, width + "px");
                    this.gridInstance.model.allowSummary && this.gridInstance.getGridFooterContent().find(".e-frozenfooterdiv").width(width);
                }
                this.gridInstance.getGridHeaderTable().find(".e-columnheader").css("cursor", "default");
            }
        },
        _mouseUp: function (e) {
            if (this._expand) {
                var _x = e.clientX, _y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY;
                }
                _x += document.documentElement.scrollLeft;
                this._reSize(_x, _y);
            }
        },
        _getResizableCell: function () {
            var row = this.gridInstance.getGridHeaderTable().find(".e-columnheader");
            var cell = row.find(".e-headercell").not(".e-hide");
            var scrollLeft = navigator.userAgent.indexOf("WebKit") != -1 ? document.body.scrollLeft : document.documentElement.scrollLeft;
            for (var i = 0; i < cell.length; i++) {
                point = cell[i].getBoundingClientRect();
                var xlimit = point.left + scrollLeft + 5;
                if (xlimit > this._orgX) {
                    this._currentCell = i - 1;
                    return;
                }
            }
        },
        _moveVisual: function (_x) {
            /// Used to move the visual element in mouse move
            var _bounds = this.gridInstance.getGridHeaderContent().find("div").first()[0].getBoundingClientRect();
            if ((_bounds.left + document.documentElement.scrollLeft + _bounds.width < _x) || (_x < _bounds.left + document.documentElement.scrollLeft))
                this._$visualElement.remove();
            else if (this._currentCell != -1)
                this._$visualElement.css({ left: _x, top: this._tableY });
        },
        _mouseDown: function (e) {
            if (this._allowStart && ($(e.target).closest("tr").css("cursor") == 'col-resize')) {
                var _x = e.clientX, _y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY;
                }
                _x += document.documentElement.scrollLeft;
                if (e.button != 2)
                    this._start(_x, _y);
                e.preventDefault();
            }
            return false;
        }

    };
})(jQuery, Syncfusion);
/**
* @fileOverview Plugin to create the Gantt
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
* @namespace ej
* @class ejGantt
* @requires jQuery
* @requires ej.web.all
* @classdesc Custom Design for Html Gantt control.
* @example 
* &lt;gantt id="gantt1"&gt;Gantt&lt;/gantt&gt; <br> 
* &lt;script&gt;
* // Create Gantt
* $('#gantt1').ejGantt(); 	
* &lt;/script&gt;
*/

    ej.widget("ejGantt", "ej.Gantt", {

        // widget element will be automatically set in this
        _rootCSS: "e-gantt",

        element: null,

        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,

        defaults: {
            /**		
            * Enables or disables sorting. When enabled, we can sort the column by clicking on the column.
            * @default false
            * @type {boolean}
            * @example 
            * // Enable sorting on initialization. 
            * 	//To set sorting API value 
            * 	$("#gantt1").ejGantt({ allowSorting:  true });			* 
            * @example 
            * //Get or set the allowSorting state, after initialization:<br>
            *	// Gets the allowSorting API value.		
            *	$("#gantt1").ejGantt("option", "allowSorting");			
            *	// Sets the allowSorting API
            *	$("#gantt1").ejGantt("option", "allowSorting",  true );
            * @memberof ejGantt
            * @instance
            */
            allowSorting: false,

            /**		
            * Enables or disables the ability to resize column.
            * @default false
            * @type {boolean}
            * @example 
            * // Enable allowColumnResize on initialization. 
            * 	//To set allowColumnResize API value 
            * 	$("#gantt").ejGantt({ allowColumnResize:  true });			* 
            * @example 
            * //Get or set the allowColumnResize state, after initialization:<br>
            *	// Gets the allowColumnResize API value.		
            *	$("#gantt").ejGantt("option", "allowColumnResize");			
            *	// Sets the allowColumnResize API
            *	$("#gantt").ejGantt("option", "allowColumnResize",  false );
            * @memberof ejGantt
            * @instance
            */
            allowColumnResize: false,

            /**		
            * Enables or disables the interactive selection of a row.
            * @default true
            * @type {boolean}
            * @example 
            * // Enable allowSelection on initialization. 
            * 	//To set allowSelection API value 
            * 	$("#gantt").ejGantt({ allowSelection:  true });			* 
            * @example 
            * //Get or set the allowSelection state, after initialization:<br>
            *	// Gets the allowSelection API value.		
            *	$("#gantt").ejGantt("option", "allowSelection");			
            *	// Sets the allowSelection API
            *	$("#gantt").ejGantt("option", "allowSelection",  false );
            * @memberof ejGantt
            * @instance
            */
            allowSelection: true,

            /**		
            * Collection of data or hierarchical data  to represent in gantt
            * @default 
            * @type {array}
            * @example 
            * // Set the dataSource during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     dataSource:[{Id:2,TaskName:"Testing",startDate:"12/1/2000",Duration:5 }]	
            *  });		
            * @example 
            * //Get or set the dataSource after initialization:<br>
            *	// Gets the dataSource.		
            *	$("#gantt1").ejGantt("option", "dataSource");			
            *	// Sets the dataSource
            *	$("#gantt1").ejGantt("option", "dataSource",  "[{Id:2,TaskName:"Implmentation",startDate:"12/1/2000",Duration:5}]" );
            * @memberof ejGantt
            * @instance
            */
            dataSource: [],

            /**		
           * Specifies the mapping property path for task Id in datasource
           * @default ""
           * @type {string}
           * @example 
           * // Set the taskIdMapping during initialization. 			
           * 	$("#gantt1").ejGantt({  taskIdMapping : "ID" });	
           * @example 
           * //Get or set the taskIdMapping state, after initialization:<br>
           *	// Gets the taskIdMapping API value.		
           *	$("#gantt1").ejGantt("option", "taskIdMapping");			
           *	// Sets the taskIdMapping API
           *	$("#gantt1").ejGantt("option", "taskIdMapping",  "ID" );
           * @memberof ejGantt
           * @instance
           */
            taskIdMapping: "",

            /**		
            * Specifies the mapping property path for task name in datasource
            * @default ""
            * @type {string}
            * @example 
            * // Set the taskNameMapping during initialization. 			
            * 	$("#gantt1").ejGantt({  taskNameMapping : "Name" });	
            * @example 
            * //Get or set the taskNameMapping state, after initialization:<br>
            *	// Gets the taskNameMapping API value.		
            *	$("#gantt1").ejGantt("option", "taskNameMapping");			
            *	// Sets the taskIdMapping API
            *	$("#gantt1").ejGantt("option", "taskNameMapping",  "Name" );
            * @memberof ejGantt
            * @instance
            */
            taskNameMapping: "",

            /**		
            * Specifies the mapping property path for start date of a task  in datasource
            * @default ""
            * @type {string}
            * @example 
            * // Set the startDateMapping during initialization. 			
            * 	$("#gantt1").ejGantt({  startDateMapping : "startdate" });	         
            * @example 
            * //Get or set the startDateMapping state, after initialization:<br>
            *	// Gets the startDateMapping API value.		
            *	$("#gantt1").ejGantt("option", "startDateMapping");			
            *	// Sets the taskNameMapping API
            *	$("#gantt1").ejGantt("option", "startDateMapping",  "startdate" );
            * @memberof ejGantt
            * @instance
            */
            startDateMapping: "",

            /**		
           * Specifies the mapping property path for end Date of a task  in datasource
           * @default ""
           * @type {string}
           * @example 
           * // Set the endDateMapping during initialization. 			
           * 	$("#gantt1").ejGantt({  endDateMapping : "EndDate" });	
           * @example 
           * //Get or set the endDateMapping state, after initialization:<br>
           *	// Gets the endDateMapping API value.		
           *	$("#gantt1").ejGantt("option", "endDateMapping");			
           *	// Sets the endDateMapping API
           *	$("#gantt1").ejGantt("option", "endDateMapping",  "EndDate" );
           * @memberof ejGantt
           * @instance
           */
            endDateMapping: "",

            /**		
           * Specifies the mapping property path for baseline start date of a task  in datasource
           * @default ""
           * @type { string}
           * @example 
           * // Set the baselineStartDateMapping during initialization. 			
           * 	$("#gantt1").ejGantt({  baselineStartDateMapping : "BaselineStartDate" });		
           * @example 
           * //Get or set the baselineStartDateMapping state, after initialization:<br>
           *	// Gets the baselineStartDateMapping API value.		
           *	$("#gantt1").ejGantt("option", "baselineStartDateMapping");			
           *	// Sets the baselineStartDateMapping API
           *	$("#gantt1").ejGantt("option", "baselineStartDateMapping",  "BaselineStartDate" );
           * @memberof ejGantt
           * @instance
           */
            baselineStartDateMapping: "",

            /**		
            *Specifies the mapping property path for baseline end date in datasource
            * @default ""
            * @type {string}
            * @example 
            * // Set the baselineEndDateMapping during initialization. 			
            * 	$("#gantt1").ejGantt({  baselineEndDateMapping : "BaselineEndDate" });
            * @example 
            * //Get or set the baselineEndDateMapping state, after initialization:<br>
            *	// Gets the baselineEndDateMapping API value.		
            *	$("#gantt1").ejGantt("option", "baselineEndDateMapping");			
            *	// Sets the baselineEndDateMapping API
            *	$("#gantt1").ejGantt("option", "baselineEndDateMapping",  "BaselineEndDate" );
            * @memberof ejGantt
            * @instance
            */
            baselineEndDateMapping: "",

            /**		
            * Specifies the mapping property path for sub tasks in datasource 
            * @default ""
            * @type {string}
            * @example 
            * // Set the childMapping during initialization. 			
            * 	$("#gantt1").ejGantt({  childMapping : "Children" });
            * @example 
            * //Get or set the childMapping state, after initialization:<br>
            *	// Gets the childMapping API value.		
            *	$("#gantt1").ejGantt("option", "childMapping");			
            *	// Sets the childMapping API
            *	$("#gantt1").ejGantt("option", "childMapping",  "Children" );
            * @memberof ejGantt
            * @instance
            */
            childMapping: "",

            /**		
            * Specifies the mapping property path for duration of a task in datasource
            * @default ""
            * @type {string}
            * @example 
            * // Set the durationMapping during initialization. 			
            * 	$("#gantt1").ejGantt({  durationMapping : "Duration" });
            * @example 
            * //Get or set the durationMapping state, after initialization:<br>
            *	// Gets the durationMapping API value.		
            *	$("#gantt1").ejGantt("option", "durationMapping");			
            *	// Sets the durationMapping API
            *	$("#gantt1").ejGantt("option", "durationMapping",  "Duration" );
            * @memberof ejGantt
            * @instance
            */
            durationMapping: "",

            /**		
           * Specifies the mapping property path for milestone in datasource
           * @default ""
           * @type {string}
           * @example 
           * // Set the mileStoneMapping during initialization. 			
           * 	$("#gantt1").ejGantt({  mileStoneMapping : "milestone" });	
           * @example 
           * //Get or set the mileStoneMapping state, after initialization:<br>
           *	// Gets the mileStoneMapping API value.		
           *	$("#gantt1").ejGantt("option", "mileStoneMapping");			
           *	// Sets the mileStoneMapping API
           *	$("#gantt1").ejGantt("option", "mileStoneMapping",  "milestone" );
           * @memberof ejGantt
           * @instance
           */
            mileStoneMapping: "",

            /**		
            *Specifies the mapping property path for  progress percentage  of a task in datasource
            * @default ""
            * @type {string}
            * @example 
            * // Set the progressMapping during initialization. 			
            * 	$("#gantt1").ejGantt({  progressMapping : "progress" });			
            * @example 
            * //Get or set the progressMapping state, after initialization:<br>
            *	// Gets the progressMapping API value.		
            *	$("#gantt1").ejGantt("option", "progressMapping");			
            *	// Sets the progressMapping API
            *	$("#gantt1").ejGantt("option", "progressMapping",  "progress" );
           * @memberof ejGantt
           * @instance
           */
            progressMapping: "",

            /**		
           *Specifies the mapping property path for predecessors  of a task in datasource
           * @default ""
           * @type {string}
           * @example 
           * // Set the predecessorMapping during initialization. 			
           * 	$("#gantt1").ejGantt({  predecessorMapping : "predecessor" });		
           * @example 
           * //Get or set the predecessorMapping state, after initialization:<br>
           *	// Gets the predecessorMapping API value.		
           *	$("#gantt1").ejGantt("option", "predecessorMapping");			
           *	// Sets the predecessorMapping API
           *	$("#gantt1").ejGantt("option", "predecessorMapping",  "predecessor" );
           * @memberof ejGantt
           * @instance
           */
            predecessorMapping: "",

            /**		
            * Specifies the mapping property path for resources of a task in datasource
            * @default ""
            * @type {string}
            * @example 
            * // Set the resourceInfoMapping during initialization. 			
            * 	$("#gantt1").ejGantt({  resourceInfoMapping : "resources" });	
            * @example 
            * //Get or set the resourceInfoMapping state, after initialization:<br>
            *	// Gets the resourceInfoMapping API value.		
            *	$("#gantt1").ejGantt("option", "resourceInfoMapping");			
            *	// Sets the resourceInfoMapping API
            *	$("#gantt1").ejGantt("option", "resourceInfoMapping",  "resources" );
            * @memberof ejGantt
            * @instance
            */
            resourceInfoMapping: "",

            /**		
           * Collection of data  regarding resources involved in entire project       
           * @default []
           * @type {array}
           * @example 
           * // Set the resourceCollection during initialization. 			
           * 	$("#gantt1").ejGantt(
           *  {
           *    resourceCollection:[{id:1; name:"jack" }]	
           *  });		
           * @example 
           * //Get or set the resourceCollection state, after initialization:<br>
           *	// Gets the resourceCollection API value.		
           *	$("#gantt1").ejGantt("option", "resourceCollection");			
           *	// Sets the resourceCollection API
           *	$("#gantt1").ejGantt("option", "resourceCollection",  "[{id:1; name:"jack"}]" );
           * @memberof ejGantt
           * @instance
           */
            resourceCollection: [],

            /**		
           * Collection of holidays with date, background and label information to be displayed in gantt. 
           * @default []
           * @type {array}
           * @example 
           * // Set the holidays during initialization. 			
           * 	$("#gantt1").ejGantt(
           *  {
           *    holidays:[{day:"12/2/2000";background:"cyan";label:"local holiday" }]	
           *  });		
           * @example 
           * //Get or set the resourceCollection state, after initialization:<br>
           *	// Gets the resourceCollection API value.		
           *	$("#gantt1").ejGantt("option", "resourceCollection");			
           *	// Sets the resourceCollection API
           *	$("#gantt1").ejGantt("option", "resourceCollection",  "[{day:"12/2/2000";background:"cyan";label:"local holiday"}]" );
           * @memberof ejGantt
           * @instance
           */
            holidays: [],


            /**		
           * Specifies whether to highlight the weekends in gantt .
           * @default true
           * @type {boolean}
           * @example 
           * // Enable highlightWeekEnds on initialization. 
           * 	//To set allowSelection API value 
           * 	$("#gantt1").ejGantt({ highlightWeekEnds:  true });			* 
           * @example 
           * //Get or set the highlightWeekEnds state, after initialization:<br>
           *	// Gets the highlightWeekEnds API value.		
           *	$("#gantt1").ejGantt("option", "highlightWeekEnds");			
           *	// Sets the highlightWeekEnds API
           *	$("#gantt1").ejGantt("option", "highlightWeekEnds",  "false" );
           * @memberof ejGantt
           * @instance
           */
            highlightWeekEnds: true,

            /**		
           * Specifies start date of the gantt schedule. By default, start date will be rounded to its previous Sunday.
           * @default null
           * @type {string}
           * @example 
           * // Set the scheduleStartDate during initialization. 			
           * 	$("#gantt1").ejGantt(
           *  {
           *     scheduleStartDate:"12/2/2000"
           *  });		
           * @example 
           * //Get or set the scheduleStartDate state, after initialization:<br>
           *	// Gets the scheduleStartDate API value.		
           *	$("#gantt1").ejGantt("option", "scheduleStartDate");			
           *	// Sets the scheduleStartDate API
           *	$("#gantt1").ejGantt("option", "scheduleStartDate",  "12/2/2000" );
           * @memberof ejGantt
           * @instance
           */
            scheduleStartDate: null,


            /**		
           * Specifies end date of the gantt schedule. By default, end date will be rounded to its next Saturday.
           * @default null
           * @type {string}
           * @example 
           * // Set the scheduleEndDate during initialization. 			
           * 	$("#gantt1").ejGantt(
           *  {
           *     scheduleEndDate:"12/2/2000"
           *  });		
           * @example 
           * //Get or set the scheduleEndDate state, after initialization:<br>
           *	// Gets the scheduleEndDate API value.		
           *	$("#gantt1").ejGantt("option", "scheduleEndDate");			
           *	// Sets the scheduleEndDate API
           *	$("#gantt1").ejGantt("option", "scheduleEndDate",  "12/2/2000" );
          * @memberof ejGantt
          * @instance
          */
            scheduleEndDate: null,

            /**		
           * Indicates whether we can edit the progress of a task interactively in gantt chart.
           * @default true
           * @type {boolean}
           * @example 
           * // Enable canResizeProgressBar on initialization. 
           * 	//To set canResizeProgressBar API value 
           * 	$("#gantt1").ejGantt({ canResizeProgressBar:  true });			
           * @example 
           * //Get or set the canResizeProgressBar state, after initialization:<br>
           *	// Gets the canResizeProgressBar API value.		
           *	$("#gantt1").ejGantt("option", "canResizeProgressBar");			
           *	// Sets the canResizeProgressBar API
           *	$("#gantt1").ejGantt("option", "canResizeProgressBar",  false );
           * @memberof ejGantt
           * @instance
           */
            canResizeProgressBar: true,

            /**		
           * Specifies the height of a single row in gantt. Also, we need to set same height in the CSS style with class name e-rowcell. 
           * @default "30"
           * @type {number}
           * @example 
           * // Set rowHeight during initialization. 			
           * 	$("#gantt1").ejGantt({  
           *			rowHeight : 30,
           * 			});
           * @example 
           * //Get or set the rowHeight state, after initialization:<br>
           *	// Gets the rowHeight API value.		
           *	$("#gantt1").ejGantt("option", "rowHeight");			
           *	// Sets the rowHeight API
           *	$("#gantt1").ejGantt("option", "rowHeight",  "30" );
          * @memberof ejGantt
          * @instance
          */
            rowHeight: 30,

            /**		
            * Specifies whether to include weekends while calculating the duration of a task. 
            * @default true
            * @type {boolean}
            * @example 
            * // Enable includeWeekend on initialization. 
            * 	//To set includeWeekend API value 
            * 	$("#gantt1").ejGantt({ includeWeekend:  true });			
            * @example 
            * //Get or set the includeWeekend state, after initialization:<br>
            *	// Gets the includeWeekend API value.		
            *	$("#gantt1").ejGantt("option", "includeWeekend");			
            *	// Sets the includeWeekend API
            *	$("#gantt1").ejGantt("option", "includeWeekend",  "false" );
            * @memberof ejGantt
            * @instance
            */
            includeWeekend: true,

            /**		
			* Specifies the toolbar options.
			* @type {object}
			* @memberof ejGantt
			* @instance
			*/
            toolBar: {

                /**		
                * Specifies the state of enabling or disabling toolbar
                * @default true
                * @type {boolean}
                * @example 
                * // Enable includeWeekend on initialization. 
                * 	//To set includeWeekend API value 
                * 	$("#gantt1").ejGantt({ allowToolBar:  true });			 
                * @example 
                * //Get or set the allowToolBar state, after initialization:<br>
                *	// Gets the allowToolBar API value.		
                *	$("#gantt1").ejGantt("option", "allowToolBar");			
                *	// Sets the allowToolBar API
                *	$("#gantt1").ejGantt("option", "allowToolBar",  "true" );
                * @instance
                * @alias ejGantt#toolBar->allowToolBar
                */
                allowToolBar: false,


                /**		
                * Specifies the list of toolbar items to rendered in toolbar
                * @default []
                * @type {array}
                * @example 
                * // Enable includeWeekend on initialization. 
                * 	//To set includeWeekend API value 
                * 	$("#gantt1").ejGantt({ includeWeekend:  true });			 
                * @example 
                * //Get or set the includeWeekend state, after initialization:<br>
                *	// Gets the toolBarItems API value.		
                *	$("#gantt1").ejGantt("option", "toolBarItems");			
                *	// Sets the toolBarItems API
                *	$("#gantt1").ejGantt("option", "toolBarItems",  "["Add","Delete"]" );
                * @instance
                * @alias ejGantt#toolBar->toolBarItems
                */
                toolBarItems: []
            },

            /**		
            * Specifies the options for striplines
            * @default "[]"
            * @type {string}
            * @example 
            * // Set interval during initialization. 			
            * 	$("#gantt1").ejGantt(stripLines: [
            *    {
            *   day: "12/2/2000,
            *  label: "Project Release",
            *   lineStyle: "dotted",
            *   lineColor: "Darkblue",
            *  lineWidth: 2
            *   }
            *   ]); 
            * @example 
            * //Get or set the stripLines for repeat gantt after initialization:<br>
            *	// Gets the stripLines value.		
            *	$("#gantt1").ejGantt("option", "stripLines");			
            *	// Sets the stripLines for repeat gantt
            *	$("#gantt1").ejGantt("option", "stripLines",  "[{ day: "12/2/2000,label: "Project Release", lineStyle: "solid", lineColor: "cyan",
            lineWidth: 2 }]" );
            * @memberof ejGantt
            * @instance
            */
            stripLines: [],

            /**		
			* Specifies the options for customizing schedule header. 
			* @type {object}
			 * @memberof ejGantt
			* @instance
			*/
            scheduleHeader:
            {
                /**		
                * Specified the format for weekview for schedule header
                 * @default "MMM dd , yyyy"
                        * @type {string}
                * @example 
                * // Set weekHeaderFormat during initialization. 			
                * 	$("#gantt1").ejGantt({  
                *			weekHeaderFormat : MMM dd , yyyy});
                * @example 
                * //Get or set the weekHeaderFormat:<br>
                *	// Gets the weekHeaderFormat value.		
                *	$("#gantt1").ejGantt("option", "scheduleHeader.weekHeaderFormat");			
                *	// Sets the weekHeaderFormat for gantt
                *	$("#gantt1").ejGantt("option", "scheduleHeader.weekHeaderFormat",  "MMM dd yy" );
                * @instance
                * @alias ejGantt#scheduleHeader->weekHeaderFormat
                */
                weekHeaderFormat: "MMM dd , yyyy",

                /**		
                    * Specified the format for day view in schedule header
                * @default "ddd"
                * @type {string}
                * @example 
                * // Set dayHeaderFormat during initialization. 			
                * 	$("#gantt1").ejGantt({  
                   *			dayHeaderFormat : ddd});
                * @example 
                * //Get or set the dayHeaderFormat:<br>
                *	// Gets the dayHeaderFormat value.		
                *	$("#gantt1").ejGantt("option", "scheduleHeader.dayHeaderFormat");			
                *	// Sets the dayHeaderFormat for gantt
                *	$("#gantt1").ejGantt("option", "scheduleHeader.dayHeaderFormat",  "dd" );
                * @instance
                * @alias ejGantt#scheduleHeader->dayHeaderFormat
                */
                dayHeaderFormat: "ddd",

                /**		
                * Specified the background for weekends in gantt
                * @default "#F2F2F2"
                * @type {string}
                * @example 
                * // Set weekendBackground during initialization. 			
                * 	$("#gantt1").ejGantt({  
                *			weekendBackground : #F2F2F2});
                * @example 
                * //Get or set the weekendBackground:<br>
                *	// Gets the weekendBackground value.		
                *	$("#gantt1").ejGantt("option", "scheduleHeader.weekendBackground");			
                *	// Sets the weekendBackground for gantt
                *	$("#gantt1").ejGantt("option", "scheduleHeader.weekendBackground",  "FF88FF" );
                * @instance
                * @alias ejGantt#scheduleHeader->weekendBackground
                */
                weekendBackground: '#F2F2F2'
            },

            /**		
            * Specifies the background of the taskbar in gantt
            * @default ""
            * @type {string}
            * @example 
            * // Set taskbarBackground during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			taskbarBackground : #F2F2F2});
            * @example 
            * //Get or set the taskbarBackground:<br>
            *	// Gets the taskbarBackground value.		
            *	$("#gantt1").ejGantt("option", "taskbarBackground");			
            *	// Sets the taskbarBackground for gantt
            *	$("#gantt1").ejGantt("option", "taskbarBackground",  "FF88FF" );
            * @memberof ejGantt
            * @instance
            */
            taskbarBackground: "",

            /**		
            *Specifies the background of progressbar in gantt
            * @default ""
            * @type {string}
            * @example 
            * // Set progressbarBackground during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			progressbarBackground : #F2F2F2});
            * @example 
            * //Get or set the progressbarBackground:<br>
            *	// Gets the progressbarBackground value.		
            *	$("#gantt1").ejGantt("option", "progressbarBackground");			
            *	// Sets the progressbarBackground for gantt
            *	$("#gantt1").ejGantt("option", "progressbarBackground",  "FF88FF" );
            * @memberof ejGantt
            * @instance
            */
            progressbarBackground: "",

            /**		
            * Specifies the background of connector lines in Gantt
            * @default ""
            * @type {string}
            * @example 
            * // Set connectorLineBackground during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			connectorLineBackground : #F2F2F2});
            * @example 
            * //Get or set the connectorLineBackground:<br>
            *	// Gets the connectorLineBackground value.		
            *	$("#gantt1").ejGantt("option", "connectorLineBackground");			
            *	// Sets the connectorLineBackground for gantt
            *	$("#gantt1").ejGantt("option", "connectorLineBackground",  "FF88FF" );
            * @memberof ejGantt
            * @instance
            */
            connectorLineBackground: "",

            /**		
            * Specifies the background of parent taskbar in gantt
            * @default ""
            * @type {string}
            * @example 
            * // Set parentTaskbarBackground during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			parentTaskbarBackground : #F2F2F2});
            * @example 
            * //Get or set the parentTaskbarBackground:<br>
            *	// Gets the parentTaskbarBackground value.		
            *	$("#gantt1").ejGantt("option", "parentTaskbarBackground");			
            *	// Sets the parentTaskbarBackground for gantt
            *	$("#gantt1").ejGantt("option", "parentTaskbarBackground",  "FF88FF" );
            * @memberof ejGantt
            * @instance
            */
            parentTaskbarBackground: "",
            /**		
            *Specifies the background of parent progressbar in gantt
            * @default ""
            * @type {string}
            * @example 
            * // Set parentProgressbarBackground during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			parentProgressbarBackground : #F2F2F2});
            * @example 
            * //Get or set the parentProgressbarBackground:<br>
            *	// Gets the parentProgressbarBackground value.		
            *	$("#gantt1").ejGantt("option", "parentProgressbarBackground");			
            *	// Sets the parentProgressbarBackground for gantt
            *	$("#gantt1").ejGantt("option", "parentProgressbarBackground",  "FF88FF" );
            * @memberof ejGantt
            * @instance
            */

            parentProgressbarBackground: "",

            /**		
            * Specifies the background of parent progressbar in gantt
            * @default "1"
            * @type {number}
            * @example 
            * // Set connectorlineWidth during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			connectorlineWidth : 1 });
            * @example 
            * //Get or set the connectorlineWidth:<br>
            *	// Gets the connectorlineWidth value.		
            *	$("#gantt1").ejGantt("option", "connectorlineWidth");			
            *	// Sets the connectorlineWidth
            *	$("#gantt1").ejGantt("option", "connectorlineWidth",  "5" );
             * @memberof ejGantt
            * @instance
            */
            connectorlineWidth: 1,

            /**		
            *Specifies whether to display task name beside task bar.
            * @default "true"
            * @type {boolean}
            * @example 
            * // Set interval during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			showTaskNames : true});
            * @example 
            * //Get or set the showTaskNames<br>
            *	// Gets the showTaskNames value.		
            *	$("#gantt1").ejGantt("option", "showTaskNames");			
            *	// Sets the showTaskNames
            *	$("#gantt1").ejGantt("option", "showTaskNames",  "false" );
            * @memberof ejGantt
            * @instance
            */
            showTaskNames: true,

            /**		
            * Specifies whether display task progress inside taskbar.
            * @default "true"
            * @type {boolean}
            * @example 
            * // Set interval during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			showProgressStatus : true});
            * @example 
            * //Get or set the showProgressStatus<br>
            *	// Gets the showProgressStatus value.		
            *	$("#gantt1").ejGantt("option", "showProgressStatus");			
            *	// Sets the showProgressStatus
            *	$("#gantt1").ejGantt("option", "showProgressStatus",  "false" );
            * @memberof ejGantt
            * @instance
            */
            showProgressStatus: true,

            /**		
            * Specifies whether to display resource names for a task beside taskbar.
            * @default "true"
            * @type {boolean}
            * @example 
            * // Set showResourceNames during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			showResourceNames : true});
            * @example 
            * //Get or set the showResourceNames<br>
            *	// Gets the showResourceNames value.		
            *	$("#gantt1").ejGantt("option", "showResourceNames");			
            *	// Sets the showResourceNames
            *	$("#gantt1").ejGantt("option", "showResourceNames",  "false" );
            * @memberof ejGantt
            * @instance
            */
            showResourceNames: true,

            /**		
            * Enables or disables tooltip while dragging/resizing the taskbar.
            * @default "true"
            * @type {boolean}
            * @example 
            * // Set showTaskbarDragTooltip during initialization. 			
            * 	$("#gantt1").ejGantt({  
            *			showTaskbarDragTooltip : true});
            * @example 
            * //Get or set the showTaskbarDragTooltip<br>
            *	// Gets the showTaskbarDragTooltip value.		
            *	$("#gantt1").ejGantt("option", "showTaskbarDragTooltip");			
            *	// Sets the showTaskbarDragTooltip
            *	$("#gantt1").ejGantt("option", "showTaskbarDragTooltip",  "false" );
            * @memberof ejGantt
            * @instance
            */
            showTaskbarDragTooltip: true,

            /**		
            * Enables or disables tooltip for taskbar.
            * @default true
            * @type {boolean}
            * @example 
            * // Create showTaskbarTooltip during initialization. 			
            * 	$("#gantt1").ejGantt({  showTaskbarTooltip : true });			* 
            * @example 
            * //Enable or disable the showTaskbarTooltip behaviour after initialization:<br>
            *	// Gets the showTaskbarTooltip status.		
            *	$("#gantt1").ejGantt("option", "showTaskbarTooltip");			
            *	// Sets the repeat showTaskbarTooltip
            *	$("#gantt1").ejGantt("option", "showTaskbarTooltip",  true );
            * @memberof ejGantt
            * @instance
            */
            showTaskbarTooltip: true,

            /**		
            * Specifies the edit options in gantt. 
            * @type {object}
            * @memberof ejGantt
            * @instance
            */
            edit: {

                /**		
                * specifies the option for enabling / disablig editing in gantt grid part
                * @default false
                * @type {boolean}
                * @example 
                * // Create edit during initialization. 			
                * 	$("#gantt1").ejGantt({  edit:{allowEditing : true} });			* 
                * @example 
                * //Enable or disable the edit behaviour after initialization:<br>
                *	// Gets the edit.		
                *	$("#gantt1").ejGantt("option", "allowEditing");			
                *	// Sets the edit
                *	$("#gantt1").ejGantt("option", "allowEditing",  {  edit:{allowEditing : true} } );
                * @instance
                * @alias ejGantt#edit->allowEditing
                */
                allowEditing: false,

                /**		
                * enables/disables add gantt in toolbar
                * @default false
                * @type {boolean}
                * @example 
                * // Create allowAdding during initialization. 			
                * 	$("#gantt1").ejGantt({  edit:{allowAdding : true} });			* 
                * @example 
                * //Enable or disable the allowAdding behaviour after initialization:<br>
                *	// Gets the edit.		
                *	$("#gantt1").ejGantt("option", "edit.allowAdding");			
                *	// Sets the edit
                *	$("#gantt1").ejGantt("option", "edi",  {  edit:{allowAdding : true} } );
                * @instance
                * @alias ejGantt#edit->allowAdding
                */
                allowAdding: false,

                /**		
                * enables/disables delete gantt in toolbar
                * @default false
                * @type {boolean}
                * @example 
                * // Create allowDeleting during initialization. 			
                * 	$("#gantt1").ejGantt({  edit:{allowDeleting : true} });			* 
                * @example 
                * //Enable or disable the allowAdding behaviour after initialization:<br>
                *	// Gets the edit.		
                *	$("#gantt1").ejGantt("option", "edit.allowDeleting");			
                *	// Sets the edit
                *	$("#gantt1").ejGantt("option", "edit.allowDeleting",  {  edit:{allowDeleting : true} } );
                * @instance
                * @alias ejGantt#edit->allowDeleting
                */
                allowDeleting: false,


                /**		
                * specifies the editmode in gantt , "normal" is for dialog editing ,"cellEditing" is for cell type editing
                * @default normal
                * @type {string}
                * @example 
                * // Create editMode during initialization. 			
                * 	$("#gantt1").ejGantt({  edit:{editMode : true} });			* 
                * @example 
                * //Enable or disable the allowAdding behaviour after initialization:<br>
                *	// Gets the edit.		
                *	$("#gantt1").ejGantt("option", "edit.editMode");			
                *	// Sets the edit
                *	$("#gantt1").ejGantt("option", "edit.editMode",  {  edit:{editMode : "cellEditing"} } );
                * @instance
                * @alias ejGantt#edit->editMode
                */
                editMode: "normal",
            },

            ganttRecords: [],
            parentRecords: [],

            /**		
            * Enables/Disables Keyboard navigation in gantt
            * @default true
            * @type {boolean}
            * @example 
            * // Create allowKeyboardNavigation during initialization. 			
            * 	$("#gantt1").ejGantt({ allowKeyboardNavigation : true});			
            * @example 
            * //Enable or disable the allowKeyboardNavigation behaviour after initialization:<br>
            *	// Gets the allowKeyboardNavigation.		
            *	$("#gantt1").ejGantt("option", "allowKeyboardNavigation");			
            *	// Sets the allowKeyboardNavigation
            *	$("#gantt1").ejGantt("option", "allowKeyboardNavigation",  {  allowKeyboardNavigation: true}  );
            * @memberof ejGantt
            * @instance
            */
            allowKeyboardNavigation: true,

            /**		
             * Specify the CSS class to gantt to achieve custom theme.
             * @default ""
             * @type {string}
             * @example 
             * // Set the CSS class during initialization. 			
             * 	$("#gantt1").ejGantt({  cssClass : "gradient-lime" });			* 
             * @example 
             * //Get or set the rounded corner after initialization:<br>
             *	// Gets the rounded corner value.		
             *	$("#gantt1").ejGantt("option", "cssClass");			
             *	// Sets the rounded corner to gantt
             *	$("#gantt1").ejGantt("option", "cssClass",  "gradient-lime" );
             * @memberof ejGantt
             * @instance
             */
            cssClass: "",

            /**		
            * Specify the localization for gantt
            * @default "en-US"
            * @type {string}
            * @example 
            * // Set the localization during initialization. 			
            * 	$("#gantt1").ejGantt({  localization : "it-IT" });			* 
            * @example 
            * //Get or set the localization after initialization:<br>
            *	// Gets the localization value.		
            *	$("#gantt1").ejGantt("option", "localization");			
            *	// Sets the localization to gantt
            *	$("#gantt1").ejGantt("option", "localization",  "it-IT" );
            * @memberof ejGantt
            * @instance
            */
            localization: "en-US",

            /**		
            * Specifies enabling / diabling multiple soting for gantt columns
            * @default false
            * @type {boolean}
            * @example 
            * // Create allowMultiSorting during initialization. 			
            * 	$("#gantt1").ejGantt({ allowMultiSorting : true});			
            * @example 
            * //Enable or disable the allowMultiSorting behaviour after initialization:<br>
            *	// Gets the allowMultiSorting.		
            *	$("#gantt1").ejGantt("option", "allowMultiSorting");			
            *	// Sets the allowKeyboardNavigation
            *	$("#gantt1").ejGantt("option", "allowMultiSorting",  {  allowMultiSorting: true}  );
            * @memberof ejGantt
            * @instance
            */
            allowMultiSorting: false,

            /**		
            *  Specifes the sorting options for gantt.
            * @type {object}
            * @memberof ejGantt
            * @instance
            */
            sortSettings: {
                /**		
           * Specifes the sorted columns columns for gantt
           * @default []
           * @type {array}
           * @example 
           * // Create sortSettings during initialization. 			
           * 	$("#gantt1").ejGantt({ sortSettings{sortedColumns : []}});			
           * @example 
           * //Enable or disable the sortSettings behaviour after initialization:<br>
           *	// Gets the sortSettings.		
           *	$("#gantt1").ejGantt("option", "sortSettings.sortedColumns");			
           *	// Sets the sortSettings
           *	$("#gantt1").ejGantt("option", "sortSettings",  { sortSettings{sortedColumns : []}}  );
           * @alias ejGantt#sortSettings->sortedColumns
           * @instance
           */
                sortedColumns: []
            },

            /**		
            * Specifies the action during rendering each row in gantt
            * @default null
            * @type {object}
            * @example 
            * // Set the rowDataBound during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     rowDataBound: ""
            *  });		
            * @example 
            * //Get or set the rowDataBound after initialization:<br>
            *	// Gets the rowDataBound		
            *	$("#gantt1").ejGantt("option", "rowDataBound");			
            *	// Sets the rowDataBound
            *	$("#gantt1").ejGantt("option", "rowDataBound",  "" );
            * @memberof ejGantt
            * @instance
            */
            rowDataBound: null,


            /**		
            * Enables/Disables alternate row effect in gantt
            * @default true
            * @type {boolean}
            * @example 
            * // Create altRow during initialization. 			
            * 	$("#gantt1").ejGantt({ altRow : true});			
            * @example 
            * //Enable or disable the altRow behaviour after initialization:<br>
            *	// Gets the allowMultiSorting.		
            *	$("#gantt1").ejGantt("option", "altRow");			
            *	// Sets the allowKeyboardNavigation
            *	$("#gantt1").ejGantt("option", "altRow",  {  altRow: true}  );
            * @memberof ejGantt
            * @instance
            */
            altRow: true,

            /**		
            * Enables/Disables virtualization for rendering gantt items.
            * @default false
            * @type {boolean}
            * @example 
            * // Create enableVirtualization during initialization. 			
            * 	$("#gantt1").ejGantt({ enableVirtualization : true});			
            * @example 
            * //Enable or disable the altRow behaviour after initialization:<br>
            *	// Gets the allowMultiSorting.		
            *	$("#gantt1").ejGantt("option", "enableVirtualization");			
            *	// Sets the enableVirtualization
            *	$("#gantt1").ejGantt("option", "enableVirtualization",  {  enableVirtualization: true}  );
            * @memberof ejGantt
            * @instance
            */
            enableVirtualization: false,

            /**		
             * Specified the height of the progressbar in taskbar
             * @default "100"
             * @type {number}
             * @example 
             * // Set interval during initialization. 			
             * 	$("#gantt1").ejGantt({  
             *			progressbarHeight : 100 });
             * @example 
             * //Get or set the progressbarHeight for repeat gantt after initialization:<br>
              *	// Gets the progressbarHeight value.		
              *	$("#gantt1").ejGantt("option", "progressbarHeight");			
              *	// Sets the interval for repeat gantt
             *	$("#gantt1").ejGantt("option", "progressbarHeight",  "70" );
              * @memberof ejGantt
            * @instance
             */
            progressbarHeight: 100,

            /**		
             * Specifies the template for tooltip on mouseaction on taskbars
             * @default null
             * @type {string}
             * @example 
             * // Set the taskbarTooltipTemplate during initialization. 			
             * 	$("#gantt1").ejGantt(
             *  {
             *     taskbarTooltipTemplate: ""
             *  });		
             * @example 
             * //Get or set the taskbarTooltipTemplate after initialization:<br>
             *	// Gets the taskbarTooltipTemplate		
             *	$("#gantt1").ejGantt("option", "taskbarTooltipTemplate");			
             *	// Sets the taskbarTooltipTemplate
             *	$("#gantt1").ejGantt("option", "taskbarTooltipTemplate",  "" );
              * @memberof ejGantt
             * @instance
             */
            taskbarTooltipTemplate: null,

            /**		
            * Specifies the dateFormat for gantt , given format is displayed in tooltip , grid .
            * @default ""
            * @type {string}
            * @example 
            * // Set the dateFormat during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     dateFormat: "dd/MM/yyyy"
            *  });		
            * @example 
            * //Get or set the dateFormat after initialization:<br>
            *	// Gets the dateFormat		
            *	$("#gantt1").ejGantt("option", "dateFormat");			
            *	// Sets the dateFormat
            *	$("#gantt1").ejGantt("option", "dateFormat",  "MM/dd/yyyy" );
           * @memberof ejGantt
           * @instance
           */
            dateFormat: "",

            /**		
            * Specifies the mapping property name for resource ID in resource Collection in gantt
            * @default ""
            * @type {string}
            * @example 
            * // Set the resourceIdMapping during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     resourceIdMapping: "peopleID"
            *  });		
            * @example 
            * //Get or set the resourceIdMapping after initialization:<br>
            *	// Gets the dateFormat		
            *	$("#gantt1").ejGantt("option", "resourceIdMapping");			
            *	// Sets the dateFormat
            *	$("#gantt1").ejGantt("option", "resourceIdMapping",  "resourceID" );
            * @memberof ejGantt
            * @instance
            */
            resourceIdMapping: "",

            /**		
            * Specifies the mapping property path for resource name of a task in gantt
            * @default ""
            * @type {string}
            * @example 
            * // Set the resourceNameMapping during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     resourceNameMapping: "EmployeeName"
            *  });		
            * @example 
            * //Get or set the resourceNameMapping after initialization:<br>
            *	// Gets the resourceNameMapping		
            *	$("#gantt1").ejGantt("option", "resourceNameMapping");			
            *	// Sets the resourceNameMapping
            *	$("#gantt1").ejGantt("option", "resourceNameMapping",  "ResouurceName" );
            * @memberof ejGantt
            * @instance
            */
            resourceNameMapping: "",

            /**		
            * Specifies the template ID for customized tooltip for progressbar editing in gantt
            * @default null
            * @type {string}
            * @example 
            * // Set the progressbarTooltipTemplateID during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     progressbarTooltipTemplateID: "tooltiptemplateID"
            *  });		
            * @example 
            * //Get or set the progressbarTooltipTemplateID after initialization:<br>
            *	// Gets the progressbarTooltipTemplateID		
            *	$("#gantt1").ejGantt("option", "progressbarTooltipTemplateID");			
            *	// Sets the progressbarTooltipTemplateID
            *	$("#gantt1").ejGantt("option", "progressbarTooltipTemplateID",  "tooltipID" );
            * @memberof ejGantt
            * @instance
            */
            progressbarTooltipTemplateID: null,

            /**		
            * Specifies the template Id for customized tooltip for taskbar editing in gantt
            * @default null
            * @type {string}
            * @example 
            * // Set the taskbarEditingTooltipTemplateID during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     taskbarEditingTooltipTemplateID: "tooltiptemplateID"
            *  });		
            * @example 
            * //Get or set the taskbarEditingTooltipTemplateID after initialization:<br>
            *	// Gets the taskbarEditingTooltipTemplateID		
            *	$("#gantt1").ejGantt("option", "taskbarEditingTooltipTemplateID");			
            *	// Sets the taskbarEditingTooltipTemplateID
            *	$("#gantt1").ejGantt("option", "taskbarEditingTooltipTemplateID",  "tooltipID" );
            * @memberof ejGantt
            * @instance
            */
            taskbarEditingTooltipTemplateID: null,

            /**		
            * Specifies the selected row Index in gantt , the row with given index is highlighted
            * @default  "-1"
            * @type {number}
            * @example 
            * // Set the selected row Index of gantt during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     selectedRowIndex:2
            *  });		
            * @example 
            * //Get or set the selectedRowIndex of gantt after initialization:<br>
            *	// Gets the selectedRowIndex
            *	$("#gantt1").ejGantt("option", "selectedRowIndex");			
            *	// Sets the selectedRowIndex
            *	$("#gantt1").ejGantt("option", "selectedRowIndex",  "4" );
            * @memberof ejGantt
            * @instance
            */
            selectedRowIndex: -1,



            /**		
            * specifies event action to be triggered during gridtree row rendering or loading in  gantt
            * @default  "null"
            * @type {}
            * @example 
            * // Set the queryCellInfo of gantt during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     queryCellInfo:queryCellInfo(args)
            *  });		
            * function queryCellInfo(args) {
            *           jQuery.addEventLog("Gantt <span class='eventTitle'>queryCellInfo</span> event called");
            * }
            * @example 
            * //Get or set the queryCellInfo of gantt after initialization:<br>
            *	// Gets the queryCellInfo	
            *	$("#gantt1").ejGantt("option", "queryCellInfo");			
            *	// Sets the queryCellInfo
            *	$("#gantt1").ejGantt("option", "queryCellInfo",  "queryInfo(args)" );
             * function queryInfo(args) {
            *           jQuery.addEventLog("Gantt <span class='eventTitle'>queryCellInfo</span> event called");
            *         
            *  }
            * @memberof ejGantt
            * @instance
            */
            queryCellInfo: null,

            /**		
            * specifies event action to be triggered during taskbar rendering or loading in  gantt
            * @default  "null"
            * @type {}
            * @example 
            * // Set the queryTaskbarInfo of gantt during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     queryTaskbarInfo:queryTaskbarInfo(args)
            *  });		
            * function queryTaskbarInfo(args) {
            *           jQuery.addEventLog("Gantt <span class='eventTitle'>queryTaskbarInfo</span> event called");
            *          if (args.data.status > 40) {
            *             args.taskbarBackground = "#5AC1E0";
            *            args.progressbarBackground = "#169DD7";
            *       }
            *  }
            * @example 
            * //Get or set the queryTaskbarInfo of gantt after initialization:<br>
            *	// Gets the queryTaskbarInfo	
            *	$("#gantt1").ejGantt("option", "queryTaskbarInfo");			
            *	// Sets the queryTaskbarInfo
            *	$("#gantt1").ejGantt("option", "queryTaskbarInfo",  "queryInfo(args)" );
            * function queryInfo(args) {
            *           jQuery.addEventLog("Gantt <span class='eventTitle'>queryTaskbarInfo</span> event called");
            *          if (args.data.status >= 60) {
            *             args.taskbarBackground = "#5AC1E0";
            *       }
            *  }
            * @memberof ejGantt
            * @instance
            */
            queryTaskbarInfo: null,

            /**		
            * Enables or Disables gantt chart editing in gantt
            * @default  "true"
            * @type {boolean}
            * @example 
            * // Set the chart editing state of gantt during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     allowGanttChartEditing:true
            *  });		
            * @example 
            * //Get or set the GanttChartEditing state of gantt after initialization:<br>
            *	// Gets the chart editing state	
            *	$("#gantt1").ejGantt("option", "allowGanttChartEditing");			
            *	// Sets the chart editing state
            *	$("#gantt1").ejGantt("option", "allowGanttChartEditing",  "false" );
            * @memberof ejGantt
            * @instance
            */
            allowGanttChartEditing: true,

            /**		
            * Specifies the size option of gantt control.
            * @type {object}
            * @memberof ejGantt
            * @instance
            */
            size: {
                /**		
                * Specifies the height of gantt control
                * @default  "450px"
                * @type {string}
                * @example 
                * // Set the height of gantt during initialization. 			
                * 	$("#gantt1").ejGantt(
                *  {
                *     size:{height: "700px"}
                *  });		
                * @example 
                * //Get or set the height of gantt after initialization:<br>
                *	// Gets the height	
                *	$("#gantt1").ejGantt("option", "size.height");			
                *	// Sets the height
                *	$("#gantt1").ejGantt("option", "height",  "900px" );
                * @memberof size
                * @instance
                * @alias ejGantt#size->height
                */
                height: "450px",

                /**		
                * Specifies the width of gantt control
                * @default  "1000px"
                * @type {string}
                *  @example 
                * // Set the width of gantt during initialization. 			
                * 	$("#gantt1").ejGantt(
                *  {
                *     size:{width: "700px"}
                *  });		
                * @example 
                *  //Get or set the width of gantt after initialization:<br>
                *	// Gets the Width	
                * 	$("#gantt1").ejGantt("option", "size.width");			
                *	// Sets the Width
                *	$("#gantt1").ejGantt("option", "size:{width}",  "900px" );
                * @instance
                * @alias ejGantt#size->width
                */
                width: "1000px"
            },

            /**		
          * Indicates the current selected item in gantt , the selected item row is displayed as highlighted row
          * @default  null
          * @type {}
          * @example 
          * // Set the selected Item during initialization. 			
          * 	$("#gantt1").ejGantt(
          *  {
          *     selectedItem: {Id :12 , Name:"task 1" , startDate:"12/2/2000" ,Duration:"5"}
          *  });		
          * @example 
          * //Get or set the selected Item after initialization:<br>
          *	// Gets the Selected Item	
          *	$("#gantt1").ejGantt("option", "selectedItem");			
          *	// Sets the selected Item
          *	$("#gantt1").ejGantt("option", "selectedItem",  {Id :1 , Name:"New child task" , startDate:"12/2/2000" ,Duration:"10"} );
          * @memberof ejGantt
          * @instance
          */
            selectedItem: null,

            /**		
          * Specifies the weekendBackground color in gantt
          * @default  "#F2F2F2"
          * @type {string}
          * @example 
          * // Set the weekendBackground during initialization. 			
          * 	$("#gantt1").ejGantt(
          *  {
          *     weekendBackground: "blue"
          *  });		
          * @example 
          * //Get or set the weekendBackground after initialization:<br>
          *	// Gets the weekendBackground	
          *	$("#gantt1").ejGantt("option", "weekendBackground");			
          *	// Sets the weekendBackground
          *	$("#gantt1").ejGantt("option", "weekendBackground",  "cyan" );
          * @memberof ejGantt
          * @instance
          */
            weekendBackground: '#F2F2F2',

            /**		
           * Specifies the baseline background color in gantt
           * @default  "#fba41c"
           * @type {string}
           * @example 
           * // Set the baselines background during initialization. 			
           * 	$("#gantt1").ejGantt(
           *  {
           *     baselineColor: "blue"
           *  });		
           * @example 
           * //Get or set the baseline background after initialization:<br>
           *	// Gets the baselines background		
           *	$("#gantt1").ejGantt("option", "baselineColor");			
           *	// Sets the baselines background
           *	$("#gantt1").ejGantt("option", "baselineColor",  "cyan" );
           * @memberof ejGantt
           * @instance
           */
            baselineColor: '#fba41c',


            /**		
            * Enables or Disables rendering baseines in Gantt , when enabled baseline in rendered in gantt
            * @default  "false"
            * @type {boolean}
            * @example 
            * // Set the rendering baselines state during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     renderBaseline: false
            *  });		
            * @example 
            * //Get or set the rendering baseline state after initialization:<br>
            *	// Gets the renderbaseline state		
            *	$("#gantt1").ejGantt("option", "renderBaseline");			
            *	// Sets the renderbaseline state
            *	$("#gantt1").ejGantt("option", "renderBaseline",  "true" );
            * @memberof ejGantt
            * @instance
            */
            renderBaseline: false,

            /**		
            * Enables or disables the contextmenu for gantt , when enabled contextmenu appears on right clicking gantt
            * @default  "true"
            * @type {boolean}
            * @example 
            * // Enables contextmenu during initialization. 			
            * 	$("#gantt1").ejGantt(
            *  {
            *     enableContextMenu: true
            *  });		
            * @example 
            * //Get or set the contextMenu state after initialization:<br>
            *	// Gets the ContextMenu state		
            *	$("#gantt1").ejGantt("option", "enableContextMenu");			
            *	// Sets the ContextMenu state
            *	$("#gantt1").ejGantt("option", "enableContextMenu",  "enableContextMenu: false" );
            * @memberof ejGantt
            * @instance
            */
            enableContextMenu: true,
        },

        dataTypes: {
            allowSorting: "boolean",
            allowColumnResize: "boolean",
            allowSelection: "boolean",
            dataSource: "data", //inline or remote data
            taskIdMapping: "string",
            taskNameMapping: "string",
            startDateMapping: "string",
            endDateMapping: "string",
            baselineStartDateMapping: "string",
            baselineEndDateMapping: "string",
            childMapping: "string",
            durationMapping: "string",
            mileStoneMapping: "string",
            progressMapping: "string",
            predecessorMapping: "string",
            resourceInfoMapping: "string",
            resourceCollection: "array",
            holidays: "array",
            highlightWeekEnds: "boolean",
            canResizeProgressBar: "boolean",
            rowHeight: "number",
            includeWeekend: "boolean",
            toolBar: {
                toolBarItems: "array"
            },
            stripLines: "array",
            scheduleHeader: "data",
            taskbarBackground: "string",
            progressbarBackground: "string",
            connectorLineBackground: "string",
            parentTaskbarBackground: "string",
            parentProgressbarBackground: "string",
            connectorlineWidth: "number",
            showTaskNames: "boolean",
            showProgressStatus: "boolean",
            showResourceNames: "boolean",
            showTaskbarDragTooltip: "boolean",
            showTaskbarTooltip: "boolean",
            edit: "data",
            ganttRecords: "array",
            parentRecords: "array",
            allowKeyboardNavigation: "boolean",
            cssClass: "string",
            localization: "string",
            allowMultiSorting: "boolean",
            sortSettings: {
                sortedColumns: "array"
            },
            altRow: "boolean",
            enableVirtualization: "boolean",
            progressbarHeight: "number",
            dateFormat: "string",
            resourceIdMapping: "string",
            resourceNameMapping: "string",
            selectedRowIndex: "number",
            allowGanttChartEditing: "boolean",
            size: {
                height: "string",
                width: "string"
            },
            selectedItem: "data",
            weekendBackground: "string",
            baselineColor: "string",
            renderBaseline: "boolean",
        },

        observables: ["selectedItem"],
        selectedItem: ej.util.valueFunction("selectedItem"),

        getColumns: function () {
            return this._columns;
        },

        setColumns: function (value) {
            this._columns = value;
        },

        _init: function () {
            var proxy = this,
                model = proxy.model, startDate, endDate;
            proxy.element.addClass("e-gantt-core");

            proxy._initPrivateProperties();
            proxy._setCultureInfo();
            proxy._initialize();
            if (model.scheduleStartDate != null && model.scheduleEndDate != null) {
                startDate = proxy._getDateFromFormat(model.scheduleStartDate);
                endDate = proxy._getDateFromFormat(model.scheduleEndDate);
            }
            else {
                startDate = proxy._getDateFromFormat(new Date());
                endDate = proxy._getDateFromFormat(new Date(startDate));
                endDate.setDate(endDate.getDate() + 20);
            }

            proxy._calculateWeekSplit(startDate, endDate);
            if (model.dataSource !== null && model.dataSource.length > 0) {
                proxy._createGanttRecords(model.dataSource);
                proxy.setColumns(proxy.createGridTreeColumns());
                proxy._updatePredecessors();
            }
            else {
                proxy.setColumns(proxy.createGridTreeColumns());
            }


            proxy._trigger("load");
            proxy._render();
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            proxy._connectorLinesCollection = [];
            proxy._createConnectorLinesCollection();
            proxy._$gridtreeHelper.ejGanttGridTree("renderGanttRecords");
            proxy._renderGanttChart();
            proxy._$ganttchartHelper && proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines", proxy._connectorLinesCollection);
            proxy._isRenddered = true;
            proxy._gridRows = proxy.getGridRows();
            proxy._ganttChartRows = proxy.getGanttChartRows();
            proxy._createSubContextMenuTemplate();
            proxy._createContextMenuTemplate();
            proxy._wireEvents();
        },

        reRenderChart: function (zoomDelta) {
            var proxy = this, model = proxy.model, zoomOffset;
            if (proxy._perDayWidth == proxy._prevDayWidth && zoomDelta > 0) {
                return;
            }
            proxy._prevDayWidth = proxy._perDayWidth;
            zoomOffset = (-zoomDelta) * proxy._perDayWidth * 15 / 100;
            proxy._perDayWidth += zoomOffset;
            proxy._perDayWidth = proxy._perDayWidth < 15 ? 15 : proxy._perDayWidth;
            proxy._perDayWidth = proxy._perDayWidth > 120 ? 120 : proxy._perDayWidth;
            if (proxy._perDayWidth >= 15 && proxy._perDayWidth < 120) {
                proxy._updateGanttRecords();
                $(".ejGanttChart").empty();
                proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                proxy._createConnectorLinesCollection();
                proxy._$ganttchartHelper.ejGanttChart("reRenderGanttChart", proxy._perDayWidth, model.ganttRecords);
                proxy._$ganttchartHelper.ejGanttChart("renderGanttRecords");
                proxy._connectorLinesCollection = [];
            }
        },

        _updateGanttRecords: function () {
            var proxy = this, model = proxy.model, count = model.ganttRecords.length;
            for (var i = 0; i < count; i++) {
                model.ganttRecords[i].left = (model.ganttRecords[i].left / proxy._prevDayWidth) * proxy._perDayWidth;
                model.ganttRecords[i].width = (model.ganttRecords[i].width / proxy._prevDayWidth) * proxy._perDayWidth;
                model.ganttRecords[i].width = model.ganttRecords[i].isMilestone == true ? 26 : model.ganttRecords[i].width,
                model.ganttRecords[i].progressWidth = model.ganttRecords[i]._calculateProgressWidth(model.ganttRecords[i].width, model.ganttRecords[i].status);
            }

        },

        _initPrivateProperties: function () {
            var proxy = this;
            proxy._ganttHeight = proxy.element.height() ? proxy.element.height() + "px" : proxy.model.size.height,
            proxy._ganttWidth = proxy.element.width() ? proxy.element.width() + "px" : proxy.model.size.width,
            proxy._disabledToolItems = [],
            proxy._id = proxy.element.attr('id'),
            proxy._scheduleWeeks = [],
            proxy._columns = [],
            proxy._rows = [],
            proxy._storedIndex = -1,
            proxy._perDayWidth = 30,
            proxy._prevDayWidth = 0,
            proxy._$gridtreePane = null,
            proxy._$ganttchartPane = null,
            proxy._$gridtree = null,
            proxy._$ganttchart = null,
            proxy._$ejGantt = null,
            proxy._$gridtreeHelper = null,
            proxy._$ganttchartHelper = null;
            proxy._tdsOffsetWidth = [];
            proxy._ids = [];
            proxy._predecessorsCollection = [];
            proxy._connectorLinesCollection = [];
            proxy._updatedConnectorLineCollection = [];
            proxy._connectorlineIds = [];
            proxy._updatedGanttRecords = [];
            proxy._currentViewData = [];
            proxy._gridRows = null;

            //variables used for ExpandCollapse action in non Virtualization 
            proxy._collapsedRecordCount = 0;
            proxy._expandedRecordsCount = 0;
            proxy._totalCollapseRecordCount = 0;
            proxy._isInExpandCollapse = false;

            proxy._hiddenRecordCount = 0;

            proxy.prevPredecessorTO = null;
            proxy.prevPredecessorFrom = null;
            proxy.prevPredecessorcount = 0;
            proxy._isValidationEnabled = true;
            proxy._isRenddered = false;
            proxy._holidays = proxy._getHoliday();
            proxy._stringHolidays = proxy._getStringHolidays();
            proxy._isinBeginEdit = false;
            proxy._isMileStoneEdited = false;
            proxy._cellEditType = null;
            proxy._isinAddnewRecord = false;

            //variables for context menu items
            proxy._contextMenuItems = proxy._getContextMenuItems();
            proxy._subContextMenuItems = proxy._getsubContextMenuItems();

            //private variable decalartion for localization
            proxy._columnHeaderTexts = null;
            proxy._editDialogTexts = null;
            proxy._months = null;
            proxy._days = null;
            proxy._emptyRecordText = null;
            proxy._commonColumnWidth = 150;
            proxy._seacrhString = "",
            proxy._emptyDataColumns = [];
        },

        _setCultureInfo: function () {
            var proxy = this,
                model = proxy.model,
                culture = model.localization;

            model.dateFormat = !model.dateFormat ? ((ej.Gantt.localization[culture] && ej.Gantt.localization[culture]["calendars"] &&
                    ej.Gantt.localization[culture]["calendars"]["standard"]["patterns"]) ?
                (ej.Gantt.localization[culture]["calendars"]["standard"]["patterns"]["d"]) :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["patterns"]["d"] :
                    ej.Gantt.localization["en-US"]["calendars"]["standard"]["patterns"]["d"]) : model.dateFormat;


            //EmptyRecord text to be displayed with dataSource is null
            proxy._emptyRecordText = ej.Gantt.localization[culture] ?
                ej.Gantt.localization[culture]["emptyRecord"] :
                ej.Gantt.localization["en-US"]["emptyRecord"];

            //ColumnHeaderText to be displayed in ganttgridtree
            proxy._columnHeaderTexts = ej.Gantt.localization[culture] ?
                ej.Gantt.localization[culture]["columnHeaderTexts"] :
                ej.Gantt.localization["en-US"]["columnHeaderTexts"];

            //EditDialog Text to be displayed in popup window
            proxy._editDialogTexts = ej.Gantt.localization[culture] ?
                ej.Gantt.localization[culture]["editDialogTexts"] :
                ej.Gantt.localization["en-US"]["editDialogTexts"];

            //Month names to be displayed in schedule
            proxy._months = (ej.Gantt.localization[culture] && ej.Gantt.localization[culture]["calendars"]) ?
                ej.Gantt.localization[culture]["calendars"]["standard"]["months"]["namesAbbr"] :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["months"]["namesAbbr"] :
                ej.Gantt.localization["en-US"]["calendars"]["standard"]["months"]["namesAbbr"];

            //Day names to displayed in schedule
            proxy._days = (ej.Gantt.localization[culture] && ej.Gantt.localization[culture]["calendars"]) ?
                ej.Gantt.localization[culture]["calendars"]["standard"]["months"]["namesAbbr"] :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["days"]["namesAbbr"] :
                ej.Gantt.localization["en-US"]["calendars"]["standard"]["days"]["namesAbbr"];

        },



        //populate the context menu with items
        _getContextMenuItems: function () {
            var contextMenuItems = [];

            contextMenuItems = [{
                iconPath: "url(../images/MenuItem_Details.png)",
                headerText: "Task Details...",
                eventHandler: null,
                isDefault: true,

            }, {
                iconPath: "url(../images/MenuItem_Add.png)",
                headerText: "Add New Task",
                eventHandler: null,
                isDefault: true
            }
            , {
                iconPath: "url(../images/MenuItem_Indent.png)",
                headerText: "Indent",
                eventHandler: null,
                isDefault: true
            }, {
                iconPath: "url(../images/MenuItem_Outdent.png)",
                headerText: "Outdent",
                eventHandler: null,
                isDefault: true
            }, {
                iconPath: "url(../images/MenuItem_Delete.png)",
                headerText: "Delete",
                eventHandler: null,
                isDefault: true
            },



            ];
            return contextMenuItems;
        },

        //populate sub context menu items
        _getsubContextMenuItems: function () {
            var contextMenuItems = [];

            contextMenuItems = [{
                iconPath: null,
                headerText: "Above",
                eventHandler: null
            }, {
                iconPath: null,
                headerText: "Below",
                eventHandler: null
            }
            ];
            return contextMenuItems;
        },


        _initialize: function () {
            var proxy = this;
            proxy._$gridtreePane = ej.buildTag('div.e-gridtreePane');
            proxy._$ganttchartPane = ej.buildTag('div.e-ganttchartPane');
            proxy._$ejGantt = ej.buildTag('div.e-gantt-Spliter#e-ejSpliter' + proxy._id, "", {}, { 'unselectable': "on" });
        },

        _render: function () {
            var proxy = this, element = proxy.element, model = proxy.model;
            if (proxy.model.toolBar.allowToolBar) {
                element.append(proxy._renderToolBar());
            }
            proxy._ganttHeight = parseInt(proxy._ganttHeight) - proxy._getToolBarHeight();
            proxy._$gridtree = ej.buildTag("div.ejGridTree#ejGridTree" + proxy._id, "", { "height": proxy._ganttHeight }, {});
            proxy._$ganttchart = ej.buildTag("div.ejGanttChart#ejGanttChart" + proxy._id, "", { "height": proxy._ganttHeight }, {});
            proxy._$ganttchartPane.append(proxy._$ganttchart);
            proxy._$gridtreePane.append(proxy._$gridtree);
            proxy._$ejGantt.append(proxy._$gridtreePane);
            proxy._$ejGantt.append(proxy._$ganttchartPane);

            element.append(proxy._$ejGantt);
            proxy._renderGantt();
            proxy._renderGridTree();
            proxy._$gridtreePane.css({ 'overflow': 'hidden' });
            if (model.edit.allowEditing || model.edit.allowAdding || model.edit.editMode == "normal" || model.edit.editMode == "dialogTemplate") {
                proxy.addDialogEditingTemplate();
                element.append(proxy._renderDialog());
            }
        },

        _renderDialog: function () {
            var $dialog = ej.buildTag("div.e-dialog e-dialog-content e-shadow e-widget-content", "", { display: "none" }, { id: this._id + "_dialogEdit" });
            return $dialog;
        },

        addDialogEditingTemplate: function () {
            var proxy = this,
                columns = proxy._columns,
                length = columns.length,
                emptyDataColumns = [];
            if (length == 0)
                return;

            for (var i = 0; i < columns.length; i++) {
                emptyDataColumns.push(columns[i].field);
            }

            var $tbody = ej.buildTag('div', "", "", { 'unselectable': 'on' }),
             $form = ej.buildTag('form', "", { 'height': 'auto', 'width': 'auto', 'font-family': 'Segoe UI', 'font-size': '14px' }, { id: proxy._id + "EditForm" }),
             $table = ej.buildTag('table', "", {}, { cellSpacing: 4 }),
             $innerTr,
             $labelTd,
             $valueTd,
             column,
             model = proxy.model;

            $innerTr = ej.buildTag('tr');


            if (model.taskIdMapping || emptyDataColumns.indexOf("taskId") != -1) {

                $labelTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0' }, { "colSpan": 1 });
                $valueTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0' }, { "colSpan": 1 }).addClass("e-rowcell");
                $labelTd.append("<label for=taskId  style='font-weight:normal;'>" + proxy._columnHeaderTexts["taskId"] + "</label>");
                column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", "taskId");
                proxy.initCellEditType(column, $valueTd);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
            }

            if (model.taskNameMapping || emptyDataColumns.indexOf("taskName") != -1) {
                $labelTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0', 'font-weight': 'normal' }, { "colSpan": 1 });
                $valueTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0' }, { "colSpan": 1 }).addClass("e-rowcell");
                $labelTd.append("<label for=taskId  style='font-weight:normal;'>" + proxy._columnHeaderTexts["taskName"] + "</label>");
                column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", "taskName");
                proxy.initCellEditType(column, $valueTd);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
            }

            $table.append($innerTr);

            if (model.resourceInfoMapping || emptyDataColumns.indexOf("resourceInfo") != -1) {
                $innerTr = ej.buildTag('tr');
                $labelTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0', 'font-weight': 'normal' });
                $valueTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0' }, { "colSpan": 3 }).addClass("e-rowcell");
                $labelTd.append("<label for=resourceInfo  style='font-weight:normal;'>" + proxy._columnHeaderTexts["resourceInfo"] + "</label>");
                column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", "resourceInfo");
                proxy.initCellEditType(column, $valueTd);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
            }

            $table.append($innerTr);

            $innerTr = ej.buildTag('tr');
            if (model.startDateMapping || emptyDataColumns.indexOf("startDate") != -1) {
                $labelTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0', 'font-weight': 'normal' });
                $valueTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0', 'padding-top': '13px', 'padding-right': '10px' }).addClass("e-rowcell");
                $labelTd.append("<label for=endDate   style='font-weight:normal;'>" + proxy._columnHeaderTexts["startDate"] + "</label>");
                column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", "startDate");
                proxy.initCellEditType(column, $valueTd);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
                if (model.endDateMapping || emptyDataColumns.indexOf("endDate") != -1) {
                    column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", "endDate");
                } else {
                    column = {
                        columnEditType: ej.Gantt.editingType.DatePicker,
                        field: "endDate",
                        headerText: proxy._columnHeaderTexts["endDate"]
                    };
                }

                $labelTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0', 'font-weight': 'normal' });
                $valueTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0' }).addClass("e-rowcell");
                $labelTd.append("<label for=taskId   style='font-weight:normal;'>" + proxy._columnHeaderTexts["endDate"] + "</label>");
                proxy.initCellEditType(column, $valueTd);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
            }

            $table.append($innerTr);
            $innerTr = ej.buildTag('tr');
            if (model.durationMapping || emptyDataColumns.indexOf("duration") != -1) {
                $labelTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0', 'font-weight': 'normal' });
                $valueTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0', 'padding-top': '13px', 'padding-right': '10px' }, { colSpan: 1 }).addClass("e-rowcell");
                $labelTd.append("<label for=endDate   style='font-weight:normal;'>" + proxy._columnHeaderTexts["duration"] + "</label>");
                column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", "duration");
                proxy.initCellEditType(column, $valueTd);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
            }

            if (model.progressMapping || emptyDataColumns.indexOf("status") != -1) {
                $labelTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0', 'font-weight': 'normal' });
                $valueTd = ej.buildTag('td', "", { "text-align": "left", 'padding': '20px 15px 0' }, { colSpan: 1 }).addClass("e-rowcell");
                $labelTd.append("<label for=endDate   style='font-weight:normal;'>" + proxy._columnHeaderTexts["status"] + "</label>");
                column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", "status");
                proxy.initCellEditType(column, $valueTd);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
            }

            $table.append($innerTr);

            $form.append($table);
            $form.appendTo($tbody);


            $tbody = proxy.renderDiaglogButton($form, $tbody);
            $.templates(proxy._id + "_JSONDialogEditingTemplate", $tbody.html());
        },

        _clearContextMenu: function () {
            //variables for context menu items

            $('.e-contextmenu').remove();
            $('.e-innerContextmenu').remove();
        },

        //JSRender Template for ContextMenu
        _createContextMenuTemplate: function () {
            var proxy = this,
                helpers = {
                    getHeaderName: proxy._getHeaderName
                };
            $.views.helpers(helpers);
            var menuItemList = "<li style='list-style-type:none;width:150px;height:25px;display:inline-block;'>";
            var listChild = "<div class='e-menuitem' id={{:headerText}}  style='left:0px;padding-top:3px;height:25px;width:150px;cursor:pointer;display:block;position:absolute;'  {{if !isDefault}}onClick='{{:eventHandler}}'{{/if}}>" +
                            "<div class='e-icon' style='left:10px;top:4px;height:20px;position:absolute;" +
                            "background-image:{{:iconPath}};background-repeat:no-repeat;'/>" +
                            "<div style='left:35px;position:absolute;'>" +
                            "<span style='font-family:Segoe UI;font-size:12px;padding-left:5px;'>{{:headerText}}</span></div>" +
                            "{{if ~getHeaderName()}}<span class='e-icon e-expander' style='height:16px;width:16px;right:5px;top:7px;position:absolute;'></span>{{/if}}";

            menuItemList += listChild;
            menuItemList += "</div></li>";
            var templates = {};
            templates[proxy._id + "contextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },

        //helper methods for context menu template
        _getHeaderName: function () {
            if (this.data.headerText == "Add New Task") {
                return true;
            }
            return false;
        },

        //JSRender Template for Inner Context Menu
        _createSubContextMenuTemplate: function () {
            var proxy = this,
                helpers = {
                    getHeaderName: proxy._getHeaderName
                };
            $.views.helpers(helpers);

            var menuItemList = "<li style='list-style-type:none;width:120px;height:25px;display:inline-block;'>";
            var listChild = "<div class='e-menuitem' id={{:headerText}} style='left:0px;padding-top:3px;" +
                "height:25px;width:120px;cursor: pointer;display:block;position:absolute;'>" +
                "<div style='left:10px;height:20px;width:16px;position:absolute;padding-top:5px;" +
                "background-image:{{:iconPath}};background-repeat:no-repeat;'/>" +
               "<div style='left:10px;position:absolute;'>" +
                "<span style='font-family:Segoe UI;font-size:12px;padding-left:5px;'>{{:headerText}}" +
                "</span></div></div>";


            menuItemList += listChild;
            menuItemList += "</li>";

            var templates = {};
            templates[proxy._id + "subContextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },

        initCellEditType: function (column, element) {

            var proxy = this,
                model = proxy.model,

                helpers = {};
            if (ej.isNullOrUndefined(column["columnEditType"]))
                column["columnEditType"] = "stringedit";

            switch (column["columnEditType"]) {
                case "stringedit":
                    if (column.mappingName === model.predecessorMapping) {
                        helpers["_" + proxy._id + "predecessor"] = proxy._getPredecessorsValue;
                        $.views.helpers(helpers);
                        element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{:~_" + proxy._id + "predecessor('" + proxy._id + "Object','" + model.predecessorMapping + "','" + column.field + "')}}", id: proxy._id + column.field, name: column.field }));
                    } else {
                        element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field, name: column.field }));
                    }
                    break;
                case "booleanedit":
                    element.html('{{if ' + column.field + '}} <input class="e-field e-checkbox" type ="checkbox" id=' + proxy._id + column.field + ' name=' + column.field + ' checked="checked"></input>{{else}} <input class="e-field e-checkbox" type ="checkbox" id=' + proxy._id + column.field + ' name=' + column.field + ' > {{/if}}');
                    break;
                case "numericedit":
                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field, name: column.field });
                    element.append($numericText);
                    break;
                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + column["columnEditType"] + ' e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field, name: column.field });
                    element.append($datePicker);
                    break;
                case "dropdownedit":
                    var $dropDownList = ej.buildTag('input.e-field e-dropdownlist' + ' e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field, name: column.field });
                    element.append($dropDownList);
                    break;
            }
        },


        _getPredecessorsValue: function (gridObject, mappingName) {
            return this.data.item ? this.data.item[mappingName] : "";
        },


        _refreshEditForm: function (args) {
            var proxy = this, form = document.getElementById(proxy._id + "EditForm"),
                elementFocused = false,
                $formElement = $(form).find("input,select"),
                percent = 86,
                i = 0,
                length = $formElement.length,
                $element,
                inputWidth = "165px",
                model = proxy.model,
                width,
                params = {},
                value,
                column,
                customParams,
                toformat,
                formatVal;

            for (i; i < length; i++) {
                $element = $formElement.eq(i);

                inputWidth = proxy._tdsOffsetWidth[i] * (percent / 100);

                if ($element.hasClass("e-numerictextbox")) {
                    width = inputWidth;
                    value = $element.val();
                    params.width = "auto";//width;                    
                    params.showSpinButton = true;
                    params.cssClass = model.cssClass;
                    if (value.length)
                        params.value = parseFloat(value);
                    customParams = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", $element.prop("name"));
                    if (!ej.isNullOrUndefined(customParams["editParams"]))
                        $.extend(params, customParams["editParams"]);
                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(proxy._id, ""));
                }
                else if ($element.hasClass("e-datepicker")) {
                    width = inputWidth;
                    params.width = width;
                    params.cssClass = model.cssClass;
                    params.dateFormat = model.dateFormat;
                    params.displayDefaultDate = true;
                    if ($element.val().length)
                        params.value = proxy._getDateFromFormat($element.val());
                    column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", $element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDatePicker(params);
                }
                else if ($element.hasClass("e-datetimepicker")) {
                    width = inputWidth;
                    params = {
                        width: "auto",//width;
                        rtl: model.rtl,
                        cssClass: model.css,
                        showButton: false
                    };
                    if ($element.val().length)
                        params.value = proxy._getDateFromFormat($element.val());
                    column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", $element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDateTimePicker(params);
                }
                else if ($element.hasClass("e-dropdownlist")) {
                    column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", $element.prop("name"));
                    var dataSource = column.dropdownData;
                    $element.ejDropDownList({
                        width: "auto",//inputWidth + "px",                        
                        showCheckbox: true,
                        dataSource: dataSource,
                        fields: { id: model.resourceIdMapping, text: model.resourceNameMapping, value: model.resourceNameMapping },
                        multiSelectedItemsIndex: proxy.getIndexofresourceInfo(dataSource, args.data)
                    });
                    $element.ejDropDownList("selectItemByValue", $element.val());
                } else {
                    switch ($element.prop('tagName')) {
                        case "INPUT":
                            if ($element.attr("type") != "checkbox") {
                                $element.css("text-align", $element.attr("name") != null && proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", $element.prop("name")) != null ?
                                    proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", $element.attr("name")).textAlign : "center");
                                $element.outerWidth(inputWidth).height(30);
                            } else
                                $element.width("165px");//(inputWidth > 0 ? ($element.width() > inputWidth ? inputWidth : $element.width()) : 1);
                            break;

                        case "SELECT":
                            $element.width(inputWidth).height(23);
                            break;
                    }
                }
                if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {
                    if (!proxy._isEnterKeyPressed) {
                        proxy._focusElements($element.closest('td'));
                        elementFocused = true;
                    }
                }
            }
        },

        getColumnIndexByField: function (field) {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                column = 0;

            for (column; column < length; column++) {

                if (columns[column]["field"] == field) break;

            }

            return column;
        },

        getIndexofresourceInfo: function (dataSource, data) {
            var resourceInfo = data.resourceInfo,
                count = 0, length = resourceInfo && resourceInfo.length, resourceIndex = [];

            for (count; count < length; count++)
                resourceIndex.push(dataSource.indexOf(resourceInfo[count]));

            return resourceIndex;

        },

        _focusElements: function ($currentCell) {
            if ($currentCell.length) {
                $currentCell.focus();
                var $childElem = $currentCell.children();
                if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {
                    $childElem.focus().select();
                    $childElem[0].focus();
                }
                else if ($childElem.find(".e-field.e-dropdownlist").length)
                    $childElem.find(".e-ddl").focus();
                else
                    $childElem.find('input,select').select().focus();
            }
        },

        renderDiaglogButton: function (form, tbody) {

            var btnId = "EditDialog_",
                proxy = this,
                model = proxy.model,
                savebtn = ej.buildTag('input', "",
                    {
                        'border-radius': '3px'
                    },
                    { type: "button", id: btnId + proxy._id + "_Save" }),
                saveText = proxy._editDialogTexts["saveButton"];

            savebtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: saveText,
                    width: "70"
                });

            var cancelText = proxy._editDialogTexts["cancelButton"],
             cancelbtn = ej.buildTag('input', "",
                 {
                     'margin-left': '20px',
                     'border-radius': '3px'
                 },
                 {
                     type: "button",
                     id: btnId + proxy._id + "_Cancel"
                 });

            cancelbtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: cancelText,
                    width: "70"
                });

            var btnDiv = ej.buildTag('div', "", { 'margin-top': '20px', 'float': 'right', 'margin-bottom': '15px', 'margin-right': '15px' }, { 'class': "e-editform-btn" });

            btnDiv.append(savebtn);
            btnDiv.append(cancelbtn);
            form.appendTo(tbody);
            btnDiv.appendTo(tbody);

            return tbody;
        },

        _renderToolBar: function () {
            var proxy = this,
                model = proxy.model,
                $div = ej.buildTag("div.e-gantttoolbar#e-gantttoolbar" + proxy._id, "", { 'height': '50px' }, { id: proxy._id + "_toolbarItems" }),
                $ul = ej.buildTag("ul", "", { 'width': proxy._ganttWidth }, {});
            (!ej.isNullOrUndefined(model.toolBar.toolBarItems) && model.toolBar.toolBarItems.length) && proxy._renderLi($ul);
            $div.append($ul);
            var $customUl = ej.buildTag("ul");
            $div.append($customUl);
            var helper = {};
            helper.click = proxy._toolBarClick;
            helper.cssClass = "";
            helper.rtl = false;
            helper.itemSeparator = false;
            helper.fields = {
                id: "",
                tooltipText: "",
                imageUrl: "",
                text: "",
                imageAttributes: "",
                spriteCSS: "",
                htmlAttributes: ""
            };
            $div.ejToolbar(helper);
            $div.ejToolbar("disableItem", proxy._disabledToolItems);
            proxy._disabledToolItems = $();
            return $div;
        },

        _renderLi: function ($ul) {
            var proxy = this,
                toolBarItems = proxy.model.toolBar.toolBarItems,
                length = toolBarItems.length,
                i = 0,
                $li;
            for (i; i < length; i++) {

                $li = ej.buildTag("li", "", {}, { id: proxy._id + "_" + toolBarItems[i] });

                if (toolBarItems[i] !== "search")
                    $li.css({ 'width': '30px', 'height': '24px' });

                proxy._renderLiContent($li, toolBarItems[i]);
                $ul.append($li);
            }
        },

        _renderLiContent: function ($li, item) {
            var $a, $input;
            switch (item) {

                case "add":
                    $a = ej.buildTag("span.e-addnewitem e-toolbaricons e-icon e-gantt-add", "", {});
                    if (!this.model.edit.allowAdding)
                        this._disabledToolItems.push($li.get(0));
                    break;

                case "edit":
                    $a = ej.buildTag("span.e-edititem e-toolbaricons e-icon e-gantt-edit", "", {});

                    if (!this.selectedItem()) {
                        this._disabledToolItems.push($li.get(0));
                    }

                    break;

                case "delete":
                    $a = ej.buildTag("span.e-deleteitem e-toolbaricons e-icon e-gantt-delete", "", {});

                    if (!this.selectedItem()) {
                        this._disabledToolItems.push($li.get(0));
                    }

                    break;

                case "update":
                    $a = ej.buildTag("span.e-saveitem e-toolbaricons e-icon e-gantt-save e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "cancel":
                    $a = ej.buildTag("span.e-cancel e-toolbaricons e-icon e-gantt-cancel e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "indent":
                    $input = ej.buildTag("span.e-indent e-toolbaricons e-icon e-disabletool e-gantt-indent", "", {}, {});
                    $li.append($input);
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "outdent":
                    $input = ej.buildTag("span.e-outdent e-toolbaricons e-icon e-disabletool e-gantt-outdent", "", {}, {});
                    $li.append($input);
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "expandAll":
                    $input = ej.buildTag("span.e-expandall e-toolbaricons e-icon e-gantt-expandall", "", {}, {});
                    $li.append($input);
                    break;

                case "collapseAll":
                    $input = ej.buildTag("span.e-collapseall e-toolbaricons e-icon e-gantt-collapseall", "", {}, {});
                    $li.append($input);
                    break;

                case "search":
                    $input = ej.buildTag("input.e-ejinputtext", "", { 'margin': '4px', 'text-indent': '4px' }, { type: "text", placeholder: "Search" }, { 'color': "#E8E9E9" });
                    $li.css({ 'float': 'right', 'padding': '2px' });
                    $li.addClass('e-search');
                    $li.append($input);
                    this.model.allowSearching = true;

                    $input.keydown($.proxy(this._keyDown, this));

                    break;
            }
            $li.append($a);
        },


        _keyDown: function (e) {

            if (e.which === 13) { //13 is key code for Enter Key 
                this._toolbarOperation(this._id + "_search", e.currentTarget.value);
            }
        },


        _updateToolbarOptions: function (searchString) {

            var proxy = this,
                model = proxy.model,
                $addElement = $($('.e-addnewitem')[0]).parent(),
                $editElement = $($('.e-edititem')[0]).parent(),
                $deleteElement = $($('.e-deleteitem')[0]).parent(),
                $indentElement = $($('.e-indent')[0]).parent(),
                $outdentElement = $($('.e-outdent')[0]).parent();


            if (searchString.length) {

                var disabledItems = [];

                if (model.edit.allowAdding) {
                    $addElement && disabledItems.push($addElement.removeClass('e-disable')[0]);
                }

                if (model.edit.allowEditing) {
                    $editElement && disabledItems.push($editElement.removeClass('e-disable')[0]);
                }

                if (model.edit.allowDeleting) {
                    $deleteElement && disabledItems.push($deleteElement.removeClass('e-disable')[0]);
                }

                $indentElement && disabledItems.push($indentElement.removeClass('e-disable')[0]);

                $outdentElement && disabledItems.push($outdentElement.removeClass('e-disable')[0]);

                $('.e-gantttoolbar').ejToolbar('disableItem', disabledItems);

            } else {
                var enabledItems = [];

                if (model.edit.allowAdding) {
                    $addElement && enabledItems.push($addElement[0]);
                }

                if (model.selectedItem) {

                    if (model.edit.allowEditing) {
                        $editElement && enabledItems.push($editElement[0]);
                    }

                    if (model.edit.allowDeleting) {
                        $deleteElement && enabledItems.push($deleteElement[0]);
                    }

                    proxy.updateIndentOutdentOption(model.selectedItem);
                }

                $('.e-gantttoolbar').ejToolbar('enableItem', enabledItems);
            }
        },

        _renderGantt: function () {

            var proxy = this,
                leftPaneSize,
                rightPaneSize,
                ganttWidth = parseInt(proxy._ganttWidth);

            //calculate gridtree pane size
            leftPaneSize = (2 * proxy._commonColumnWidth + 3) / ganttWidth * 100;
            leftPaneSize = leftPaneSize < 30 ? leftPaneSize : 30;

            //Here 7 is the splitter span width
            rightPaneSize = 100 - leftPaneSize - (7 / ganttWidth * 100);


            $("#e-ejSpliter" + proxy._id).ejSplitter({
                height: proxy._ganttHeight,
                width: proxy._ganttWidth,
                orientation: ej.orientation.Horizontal,
                properties: [{ paneSize: leftPaneSize + "%", collapsible: false }, { paneSize: rightPaneSize + "%", collapsible: false }],
                windowResizing: false
            });
        },

        _renderGridTree: function () {
            var proxy = this, model = proxy.model;
            proxy._$gridtreeHelper = $("#ejGridTree" + proxy._id).ejGanttGridTree({
                altRow: model.altRow,
                allowColumnResize: model.allowColumnResize,
                enableVirtualization: model.enableVirtualization,
                allowSorting: model.allowSorting,
                allowMultiSorting: model.allowMultiSorting,
                sortSettings: model.sortSettings,
                allowSelection: model.allowSelection,
                selectionType: "single",
                selectedRowIndex: model.selectedRowIndex,
                allowSearching: model.allowSearching,
                allowKeyboardNavigation: model.allowKeyboardNavigation,
                cssClass: model.cssClass,
                localization: model.localization,
                edit: model.edit,
                rowTemplate: model.rowTemplate,
                rowDataBound: model.rowDataBound,
                queryCellInfo: model.queryCellInfo,
                dataSource: model.dataSource,
                flatRecords: model.ganttRecords,
                parentRecords: model.parentRecords,
                records: proxy._rows,
                ids: proxy._ids,
                columns: proxy._columns,
                dateFormat: model.dateFormat,
                resourceInfoMapping: model.resourceInfoMapping,
                resourceNameMapping: model.resourceNameMapping,
                resourceIdMapping: model.resourceIdMapping,
                taskIdMapping: model.taskIdMapping,
                taskNameMapping: model.taskNameMapping,
                startDateMapping: model.startDateMapping,
                endDateMapping: model.endDateMapping,
                childMapping: model.childMapping,
                durationMapping: model.durationMapping,
                progressMapping: model.progressMapping,
                predecessorMapping: model.predecessorMapping,
                rowHeight: model.rowHeight,
                emptyRecordText: proxy._emptyRecordText
            });
        },

        _renderGanttChart: function () {
            var proxy = this,
                model = this.model;
            proxy._$ganttchartHelper = $("#ejGanttChart" + proxy._id).ejGanttChart(
                {
                    dataSource: model.dataSource,
                    scheduleStartDate: model.scheduleStartDate,
                    scheduleEndDate: model.scheduleEndDate,
                    startDateMapping: model.startDateMapping,
                    endDateMapping: model.endDateMapping,
                    progressMapping: model.progressMapping,
                    durationMapping: model.durationMapping,
                    childPropertyName: model.childMapping,
                    predecessorMapping: model.predecessorMapping,
                    enableVirtualization: model.enableVirtualization,
                    highlightWeekEnds: model.highlightWeekEnds,
                    mileStoneMapping: model.mileStoneMapping,
                    enableRowHover: model.enableRowHover,
                    perDayWidth: proxy._perDayWidth,
                    includeWeekend: model.includeWeekend,
                    connectorlineWidth: model.connectorlineWidth,
                    scheduleHeader: model.scheduleHeader,
                    taskbarBackground: model.taskbarBackground,
                    progressbarBackground: model.progressbarBackground,
                    connectorLineBackground: model.connectorLineBackground,
                    parentTaskbarBackground: model.parentTaskbarBackground,
                    parentProgressbarBackground: model.parentProgressbarBackground,
                    holidays: model.holidays,
                    dateFormat: model.dateFormat,
                    localization: model.localization,
                    ganttRecords: model.ganttRecords,
                    parentRecords: model.parentRecords,
                    rows: proxy._rows,
                    scheduleWeeks: proxy._scheduleWeeks,
                    progressbarHeight: model.progressbarHeight,
                    tooltipTemplate: model.taskbarTooltipTemplate,
                    queryTaskbarInfo: model.queryTaskbarInfo,
                    showTaskNames: model.showTaskNames,
                    showProgressStatus: model.showProgressStatus,
                    showResourceNames: model.showResourceNames,
                    resourceInfoMapping: model.resourceInfoMapping,
                    resourceNameMapping: model.resourceNameMapping,
                    canResizeProgressBar: model.canResizeProgressBar,
                    allowGanttChartEditing: model.allowGanttChartEditing,
                    progressbarTooltipTemplateID: model.progressbarTooltipTemplateID,
                    taskbarEditingTooltipTemplateID: model.taskbarEditingTooltipTemplateID,
                    _ids: proxy._ids,
                    stripLines: model.stripLines,
                    _predecessorCollection: proxy._predecessorsCollection,
                    weekendBackground: model.weekendBackground,
                    allowKeyboardNavigation: model.allowKeyboardNavigation,
                    updatedGanttRecords: proxy._updatedGanttRecords,
                    renderBaseline: model.renderBaseline,
                    baselineColor: model.baselineColor,
                    edit: model.edit,
                    columnHeaderTexts: proxy._columnHeaderTexts,
                    localizedDays: proxy._days,
                    localizedMonths: proxy._months,
                    rowHeight: model.rowHeight
                });
        },

        createGridTreeColumns: function () {

            var proxy = this, model = proxy.model, column, columns = [];

            var mapping = (model.taskIdMapping) ? model.taskIdMapping : "Task Id";

            if (mapping.length) {
                column = {
                    field: "taskId",
                    headerText: proxy._columnHeaderTexts["taskId"],
                    width: 30,
                    columnEditType: ej.Gantt.editingType.String,
                    mappingName: mapping
                };

                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }

            mapping = (model.taskNameMapping) ? model.taskNameMapping : "Task Name";

            if (mapping.length) {
                column = {
                    field: "taskName",
                    headerText: proxy._columnHeaderTexts["taskName"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.String,
                    mappingName: mapping
                };

                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }

            mapping = (model.startDateMapping) ? model.startDateMapping : "Start Date";

            if (mapping.length) {
                column = {
                    field: "startDate",
                    headerText: proxy._columnHeaderTexts["startDate"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.DatePicker,
                    mappingName: mapping
                };

                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }


            mapping = (model.endDateMapping) ? model.endDateMapping : "End Date";

            if (mapping.length) {
                column = {
                    field: "endDate",
                    headerText: proxy._columnHeaderTexts["endDate"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.DatePicker,
                    mappingName: mapping
                };

                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }

            mapping = (model.resourceInfoMapping) ? model.resourceInfoMapping : "Resources";

            if (mapping.length) {
                column = {
                    field: "resourceInfo",
                    headerText: proxy._columnHeaderTexts["resourceInfo"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.Dropdown,
                    mappingName: mapping,
                    dropdownData: model.resourceCollection
                };
                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }

            mapping = (model.durationMapping) ? model.durationMapping : "Duration";

            if (mapping.length) {
                column = {
                    field: "duration",
                    headerText: proxy._columnHeaderTexts["duration"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.Numeric,
                    mappingName: mapping
                };

                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }

            mapping = (model.progressMapping) ? model.progressMapping : "Progress";

            if (mapping.length) {
                column = {
                    field: "status",
                    headerText: proxy._columnHeaderTexts["status"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.Numeric,
                    mappingName: mapping
                };
                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }
            mapping = (model.predecessorMapping) ? model.predecessorMapping : "Predecessor";

            if (mapping.length) {
                column = {
                    field: "predecessor",
                    headerText: proxy._columnHeaderTexts["predecessor"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.String,
                    mappingName: mapping
                };
                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }

            mapping = (model.baselineStartDateMapping) ? model.baselineStartDateMapping : "Baseline StartDate";

            if (mapping.length) {
                column = {
                    field: "baselineStartDate",
                    headerText: proxy._columnHeaderTexts["baselineStartDate"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.DatePicker,
                    mappingName: mapping
                };
                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }
            mapping = (model.baselineEndDateMapping) ? model.baselineEndDateMapping : "Baseline EndDate";

            if (mapping.length) {
                column = {
                    field: "baselineEndDate",
                    headerText: proxy._columnHeaderTexts["baselineEndDate"],
                    width: 150,
                    columnEditType: ej.Gantt.editingType.DatePicker,
                    mappingName: mapping
                };
                columns.push(column);
                proxy._emptyDataColumns.push(column.field);
            }
            return columns;
        },


        _createGanttRecords: function (data) {
            var proxy = this,
                model = proxy.model,
                ganttRecords = model.ganttRecords,
                length = data.length,
                count = 0,
                parentGanttRecord,
                parentRecords = model.parentRecords,
                enableSorting = model.allowSorting,
                enableAltRow = model.altRow,
                rows = proxy._rows,
                ids = proxy._ids;


            for (count; count < length; count++) {
                parentGanttRecord = proxy._createGanttRecord(data[count], 0, null);
                proxy._storedIndex++;
                if (enableAltRow)
                    parentGanttRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                rows[proxy._storedIndex] = parentGanttRecord;
                ids[proxy._storedIndex] = parentGanttRecord["taskId"].toString();
                ganttRecords.push(parentGanttRecord);
                if (enableSorting)
                    parentRecords.push(parentGanttRecord);
                parentGanttRecord.childGanttRecords && proxy._addNestedGanttRecords(parentGanttRecord.childGanttRecords);
            }

            for (var i = 0; i < ganttRecords.length; i++) {

                if (ganttRecords[i].isMilestone) {
                    if (ganttRecords[i].parentItem) {
                        ganttRecords[i].startDate = ganttRecords[i].parentItem.endDate;
                    } else {
                        ganttRecords[i].startDate = ganttRecords[i - 1].endDate;
                    }
                    ganttRecords[i].left = ganttRecords[i]._calculateLeft(proxy._scheduleWeeks[0], proxy._perDayWidth,
                        model.includeWeekend, model.dateFormat, model.endDateMapping, proxy._holidays);
                }

                if (ganttRecords[i].parentItem) {

                    if (!ganttRecords[i].predecessor) {
                        proxy._updateParentItem(ganttRecords[i]);
                    }

                    if (!ganttRecords[i].hasChildGanttRecords)
                        ganttRecords[i]._updateParentProgress(ganttRecords[i].parentItem);
                }

            }
        },


        //AddNested Gantt Records
        _addNestedGanttRecords: function (data) {
            var proxy = this,
                rows = proxy._rows,
                ids = proxy._ids,
                model = proxy.model,
                ganttRecords = model.ganttRecords,
                count = 0,
                length = data.length,
                records = [],
                record,
                enableAltRow = model.altRow;

            for (count; count < length; count++) {
                record = data[count];
                proxy._storedIndex++;
                if (enableAltRow)
                    record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                rows[proxy._storedIndex] = record;
                ids[proxy._storedIndex] = record["taskId"].toString();
                ganttRecords.push(record);
                records = record.childGanttRecords;
                if (records) {
                    var j = 0, recordlength = records.length, childGanttRecord = null;
                    for (j; j < recordlength; j++) {
                        childGanttRecord = records[j];
                        proxy._storedIndex++;
                        if (enableAltRow)
                            childGanttRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        rows[proxy._storedIndex] = childGanttRecord;
                        ids[proxy._storedIndex] = childGanttRecord["taskId"].toString();
                        ganttRecords.push(childGanttRecord);
                        childGanttRecord.childGanttRecords && proxy._addNestedGanttRecords(childGanttRecord.childGanttRecords);
                    }
                }

            }

        },

        _createGanttRecord: function (data, level, parentItem, expanded) {
            var proxy = this, ganttRecord, model = proxy.model, column = proxy._emptyDataColumns,
                child = data[(model.childMapping) ? model.childMapping : ""],
                startDate = data[(model.startDateMapping) ? model.startDateMapping : "Start Date"],
                endDate = data[(model.endDateMapping) ? model.endDateMapping : "End Date"],
                duration = data[(model.durationMapping) ? model.durationMapping : "Duration"],
                status = (model.progressMapping) ? data[model.progressMapping] ? data[model.progressMapping] : 0 : data["Progress"],
                startDateType = proxy._getDateFromFormat(startDate),
                endDateType = proxy._getDateFromFormat(endDate),
                resourceInfo = data[(model.resourceInfoMapping) ? model.resourceInfoMapping : "Resources"],
                predecessors = data[(model.predecessorMapping) ? model.predecessorMapping : ""],
                baselineStartDate = data[(model.baselineStartDateMapping) ? model.baselineStartDateMapping : "Start Date"],
                baselineStartDateType = proxy._getDateFromFormat(baselineStartDate, model.dateFormat),
                baselineEndDate = data[(model.baselineEndDateMapping) ? model.baselineEndDateMapping : "End Date"],
                baselineEndDateType = proxy._getDateFromFormat(baselineEndDate, model.dateFormat);

            ganttRecord = new ej.Gantt.GanttRecord();
            ganttRecord.taskId = data[(model.taskIdMapping) ? model.taskIdMapping : "Task Id"],
            ganttRecord.taskName = data[(model.taskNameMapping) ? model.taskNameMapping : "Task Name"],
            ganttRecord.startDate = proxy.getFormatedDate(startDateType),
            ganttRecord.endDate = proxy.getFormatedDate(endDateType),

            ganttRecord.baselineStartDate = proxy.getFormatedDate(baselineStartDateType),
            ganttRecord.baselineEndDate = proxy.getFormatedDate(baselineEndDateType),

            ganttRecord.duration = duration;
            if (!duration && endDateType) {
                ganttRecord._calculateDuration(startDateType, endDateType, model.includeWeekend, proxy._holidays, data[model.mileStoneMapping]);
                duration = ganttRecord.duration;
            }
            ganttRecord.isMilestone = duration == 0 ? true : (data[model.mileStoneMapping]) ? false : data[model],
            ganttRecord.duration = (ganttRecord.isMilestone && duration != 0) ? 0 : duration,
            ganttRecord.status = status,
            ganttRecord.predecessor = predecessors && ganttRecord._calculatePredecessor(predecessors),
            ganttRecord.resourceInfo = resourceInfo && ganttRecord._setResourceInfo(data[(model.resourceInfoMapping) ? model.resourceInfoMapping : "Resources"], model.resourceIdMapping, model.resourceNameMapping, model.resourceCollection),
            ganttRecord.parentItem = parentItem,
            ganttRecord.item = data;
            ganttRecord.isSelected = proxy.selectedItem() ? proxy.selectedItem()[(model.taskIdMapping) ? model.taskIdMapping : "Task Id"] == ganttRecord.taskId ? true : false : false,
            ganttRecord.level = level,
            ganttRecord.left = ganttRecord._calculateLeft(proxy._scheduleWeeks[0], proxy._perDayWidth, model.includeWeekend, model.dateFormat, (model.endDateMapping) ? model.endDateMapping : "End Date", proxy._holidays),
            ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, model.includeWeekend, proxy._holidays, model.dateFormat),
            ganttRecord.width = ganttRecord.width == 0 ? 26 : ganttRecord.width,
            ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, status),
            ganttRecord.childGanttRecords = child && proxy._createChildGanttRecords(child, level + 1, ganttRecord),
            ganttRecord.hasChildGanttRecords = child ? true : false,
            ganttRecord.expanded = expanded !== undefined ? expanded : child ? child.length > 0 : false;
            if (model.renderBaseline) {
                ganttRecord.baselineLeft = ganttRecord._calculateBaselineLeft(proxy._scheduleWeeks[0], proxy._perDayWidth, model.includeWeekend, model.dateFormat, (model.endDateMapping) ? model.endDateMapping : "End Date", proxy._holidays);
                ganttRecord.baselineWidth = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model.dateFormat);
            }


            if (predecessors)
                proxy._predecessorsCollection.push(ganttRecord);

            return ganttRecord;
        },

        _getHoliday: function () {
            if (this.model.holidays != null) {
                var holidayList = this.model.holidays,
                    holidays = [],
                    holidayLength = holidayList.length;
                if (holidayLength > 0) {
                    for (var i = 0; i < holidayLength; i++) {
                        holidays[i] = this.getDateType(holidayList[i].day);
                    }
                    return holidays;
                }
            }
            return false;
        },

        _getStringHolidays: function () {
            if (this._holidays != null && this._holidays.length > 0) {
                var holidayList = this._holidays,
                    holidays = [],
                    holidayLength = holidayList.length;
                if (holidayLength > 0) {
                    for (var i = 0; i < holidayLength; i++) {
                        holidays[i] = Globalize.format(holidayList[i], this.model.dateFormat);
                    }
                    return holidays;
                }
            }
            return false;
        },

        //create childGanttRecords for particular GanttRecord
        _createChildGanttRecords: function (data, level, parentItem) {
            var ganttRecords = [],
                proxy = this,
                i = 0,
                length = data.length;

            for (i = 0; i < length; i++) {
                if (data[i]) {
                    var parentRecord = proxy._createGanttRecord(data[i], level, parentItem);
                    ganttRecords.push(parentRecord);
                }
            }
            return ganttRecords;
        },

        _createResourceInfoCollection: function (data) {
            var proxy = this,
                count = 0,
                length = data.length;
            for (count; count < length; count++) {
                proxy._resourceInfoCollection.push(proxy._createResourceInfo(data[count]));
            }
        },

        _createResourceInfo: function (data) {
            var model = this.model, resourceInfo = {};
            resourceInfo.resourceId = data[model.resourceIdMapping];
            resourceInfo.resourceName = data[model.resourceNameMapping];
            return resourceInfo;
        },

        _calculateWeekSplit: function (startdate, enddate) {
            var proxy = this;
            var roundOffStartDate = startdate.getDay() == 0 ? (startdate.getDate()) - 7 :
                (startdate.getDate()) - startdate.getDay();
            startdate.setDate(roundOffStartDate);
            do {
                proxy._scheduleWeeks.push(new Date(startdate));
                startdate.setDate(startdate.getDate() + 7);
            } while (!(startdate >= enddate));
        },

        _updatePredecessors: function () {
            var proxy = this,
                count = 0,
                predecessorsCollection = proxy._predecessorsCollection,
                length = predecessorsCollection.length,
                ganttRecord,
                connectorsCollection,
            i = 0,
            successorGanttRecord,
            connectorCount;
            for (count; count < length; count++) {
                ganttRecord = predecessorsCollection[count];
                connectorsCollection = ganttRecord["predecessor"];
                connectorCount = connectorsCollection.length;
                for (i = 0; i < connectorCount; i++) {
                    var connector = connectorsCollection[i];
                    successorGanttRecord = proxy.model.ganttRecords[proxy._ids.indexOf(connector.from)];
                    connector.to = ganttRecord['taskId'].toString();
                    if (successorGanttRecord) {
                        if (successorGanttRecord.predecessor) {
                            successorGanttRecord.predecessor.push(connector);
                        } else {
                            successorGanttRecord.predecessor = [];
                            successorGanttRecord.predecessor.push(connector);
                            predecessorsCollection.push(successorGanttRecord);
                        }
                    }

                }
            }
        },

        getDateType: function (date) {
            if (date != null) {
                if (typeof (date) == "object") {
                    return date;
                }
                else {
                    var hyphenRegex = new RegExp("\\-", "g"),
                    dotRegex = new RegExp("\\.", "g");
                    date = date.replace(hyphenRegex, "/");
                    date = date.replace(dotRegex, "/");
                    return new Date(date);
                }
            }
            return null;
        },

        getFormatedDate: function (date) {
            return Globalize.format(date, this.model.dateFormat);
        },

        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },


        _wireEvents: function () {
            var proxy = this,
                model = proxy.model,
                gridTree = $("#ejGridTree" + proxy._id),
                ganttChart = $("#ejGanttChart" + proxy._id);

            proxy._on(proxy.element, "click", proxy.clickHandler);
            proxy._on(proxy.element, "contextmenu", proxy._rightClick);
            gridTree.ejGanttGridTree({ rowSelecting: $.proxy(proxy.rowSelecting, proxy) });
            gridTree.ejGanttGridTree({ rowSelected: $.proxy(proxy.rowSelected, proxy) });
            //gridTree.ejGanttGridTree({ rowHover: $.proxy(proxy._gridTreeRowHover, proxy) });
            gridTree.ejGanttGridTree({ actionBegin: $.proxy(proxy.actionBegin, proxy) });
            gridTree.ejGanttGridTree({ actionComplete: $.proxy(proxy.actionComplete, proxy) });

            if (model.queryCellInfo) {
                gridTree.ejGanttGridTree({ queryCellInfo: $.proxy(proxy._queryCellInfo, proxy) });
            }

            if (model.rowDataBound) {
                gridTree.ejGanttGridTree({ rowDataBound: $.proxy(proxy._rowDataBound, proxy) });
            }

            gridTree.ejGanttGridTree({ beginEdit: $.proxy(proxy._beginEdit, proxy) });
            gridTree.ejGanttGridTree({ endEdit: $.proxy(proxy._endEdit, proxy) });

            gridTree.ejGanttGridTree({ expanding: $.proxy(proxy.expanding, proxy) });
            gridTree.ejGanttGridTree({ collapsing: $.proxy(proxy.collapsing, proxy) });
            gridTree.ejGanttGridTree({ expanded: $.proxy(proxy.expanded, proxy) });
            gridTree.ejGanttGridTree({ collapsed: $.proxy(proxy.collapsed, proxy) });
            gridTree.ejGanttGridTree({ refreshRow: $.proxy(proxy.refreshRow, proxy) });
            gridTree.ejGanttGridTree({ cancelEditCell: $.proxy(proxy.cancelEditCell, proxy) });


            ganttChart.ejGanttChart({ rowSelecting: $.proxy(proxy.rowSelecting, proxy) });
            ganttChart.ejGanttChart({ rowSelected: $.proxy(proxy.rowSelected, proxy) });
            ganttChart.ejGanttChart({ rowHover: $.proxy(proxy._ganttChartRowHover, proxy) });
            ganttChart.ejGanttChart({ actionBegin: $.proxy(proxy.actionBegin, proxy) });
            ganttChart.ejGanttChart({ actionComplete: $.proxy(proxy.chartactionComplete, proxy) });
            ganttChart.ejGanttChart({ refreshRow: $.proxy(proxy.refreshRowData, proxy) });
            ganttChart.ejGanttChart({ zooming: $.proxy(proxy.zoomingChart, proxy) });
            ganttChart.ejGanttChart({ queryTaskbarInfo: $.proxy(proxy.queryTaskbarInfo, proxy) });
            ganttChart.ejGanttChart({ expanding: $.proxy(proxy.expanding, proxy) });
            ganttChart.ejGanttChart({ collapsing: $.proxy(proxy.collapsing, proxy) });

            ganttChart.ejGanttChart({ taskbarEditing: $.proxy(proxy.taskbarEditing, proxy) });
            ganttChart.ejGanttChart({ taskbarEdited: $.proxy(proxy.taskbarEdited, proxy) });

            proxy._enableEditingEvents();
        },

        clickHandler: function (e) {
            var proxy = this;
            proxy._clearContextMenu();
            if (e.toElement) {
                e.toElement.tabIndex = 0;
                e.toElement.focus();
            }

            if (proxy._isinBeginEdit) {
                if (!$(e.target).hasClass('e-field')) {
                    if (!$(e.target).hasClass('e-editedcell')) {
                        proxy._$gridtreeHelper.ejGanttGridTree("cancelEditCell");
                        proxy._isinBeginEdit = false;
                    }
                }
            }

        },

        // handler for mouse right click
        _rightClick: function (e) {
            var proxy = this,
                $target = $(e.target),
                div = $target[0].parentNode,
                $ganttGridRows,
                row,
                recordIndexr,
                item,
                args = {};

            if ($(div).hasClass("e-chartcell") || $(div).hasClass("e-ganttrowcell")) {
                $ganttGridRows = $(proxy._$ganttchartHelper.ejGanttChart("getGanttChartRows"));
                args.target = "ejGanttGridTree";
            } else {
                $ganttGridRows = $(proxy._$gridtreeHelper.ejGanttGridTree("getGridRows"));
                args.target = "ejGanttChart";
            }
            row = $target.closest('tr');
            recordIndexr = $ganttGridRows.index(row);
            item = proxy._currentViewData[recordIndexr];
            args.data = item;
            args.recordIndex = recordIndexr;


            if (item && proxy.model.enableContextMenu) {
                proxy._contextMenuItems = [];
                proxy._contextMenuItems = proxy._getContextMenuItems();
                proxy._updateIndentOutdentContextmenuOption(recordIndexr, item);
                proxy._renderContextMenu(e, recordIndexr, item);
                proxy.rowSelected(args);
                proxy._$gridtreeHelper.ejGanttGridTree("selectRows", args.recordIndex);
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
            }
        },

        _updateIndentOutdentContextmenuOption: function (recordIndex, ganttRecord) {
            var proxy = this;
            if (recordIndex === 0) {;
                proxy._contextMenuItems.splice(2, 2);
            }
            else {
                if (ganttRecord.level === 0) {
                    proxy._contextMenuItems.splice(3, 1);
                }
                else {
                    var previousGanttRecord = proxy._updatedGanttRecords[recordIndex - 1];
                    if (ganttRecord.level - previousGanttRecord.level === 1) {
                        proxy._contextMenuItems.splice(2, 1);
                    }
                    else {

                    }
                }
            }
        },

        queryTaskbarInfo: function (args) {
            this._trigger("queryTaskbarInfo", args);
        },


        zoomingChart: function (args) {
            var proxy = this;
            proxy.reRenderChart(args.delta);
        },

        refreshRowData: function (args) {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("refreshRow", args.index);
        },

        expanding: function (args) {
            var proxy = this;
            if (!proxy._trigger("expanding", args)) {
                if (proxy.model.enableVirtualization) {
                    if (args.source === "ganttChart")
                        proxy._$gridtreeHelper.ejGanttGridTree("sendExpandCollapseRequest", args);
                } else {
                    proxy._isInExpandCollapse = true;
                    proxy.sendExpandCollapseRequest(args);
                    proxy._isInExpandCollapse = false;
                }
            }
        },

        collapsing: function (args) {
            var proxy = this;
            if (!proxy._trigger("collapsing", args)) {
                if (proxy.model.enableVirtualization) {
                    if (args.source === "ganttChart")
                        proxy._$gridtreeHelper.ejGanttGridTree("sendExpandCollapseRequest", args);
                } else {
                    proxy._gridRows = proxy.getGridRows();
                    proxy._isInExpandCollapse = true;
                    proxy.sendExpandCollapseRequest(args);
                    proxy._isInExpandCollapse = false;
                }
            }
        },

        expanded: function (args) {
            var proxy = this;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            //if (proxy.model.enableVirtualization) {
            if (proxy.model.predecessorMapping) {
                proxy._isValidationEnabled = false;
                proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                proxy._connectorlineIds = [];
                proxy._connectorLinesCollection = [];
                proxy._createConnectorLinesCollection();
                //  }
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);
            }
            proxy._trigger("expanded", args);
        },

        collapsed: function (args) {
            var proxy = this;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            if (proxy.model.enableVirtualization) {
                if (proxy.model.predecessorMapping) {
                    proxy._isValidationEnabled = false;
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._createConnectorLinesCollection();
                }
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);
            }
            proxy._trigger("collapsed", args);
        },

        refreshRow: function (args) {
            var proxy = this;
            proxy._$ganttchartHelper.ejGanttChart("refreshRow", args.recordIndex);
        },

        _enableEditingEvents: function () {
            var proxy = this,
                model = proxy.model;

            if (model.edit.editMode == "normal" || model.edit.editMode == "dialogTemplate" ||
                (model.toolBar.allowToolBar && ((model.edit.allowAdding && model.toolBar.toolBarItems.indexOf("add") !== -1) ||
                    (model.edit.allowEditing && model.toolBar.toolBarItems.indexOf("index") !== -1)))) {

                proxy._on($("#" + proxy._id + "_dialogEdit"),
                    "click keypress", "#EditDialog_" + proxy._id + "_Save ,#EditDialog_" + proxy._id + "_Cancel", proxy._buttonClick);

                $(proxy.element).on("click", ".e-icon", $.proxy(proxy._buttonClick, proxy));
            }
        },

        cancelEditCell: function () {

            var proxy = this;

            if (proxy.model.toolBar.toolBarItems.indexOf("cancel") !== -1) {
                proxy._disabledToolItems.push($($('.e-cancel')[0]).parent()[0]);
            }

            if (proxy.model.toolBar.toolBarItems.indexOf("save") !== -1) {
                proxy._disabledToolItems.push($($('.e-saveitem')[0]).parent()[0]);
            }

            $('.e-gantttoolbar').ejToolbar('disableItem', proxy._disabledToolItems);
        },

        _buttonClick: function (e) {

            if (e.keyCode !== undefined && e.keyCode != 13)
                return true;

            var proxy = this,
                model = proxy.model;

            if (model.edit.editMode == "normal" || model.edit.editMode == "dialogTemplate" ||
                (model.toolBar.allowToolBar && ((model.edit.allowAdding && model.toolBar.toolBarItems.indexOf("add") !== -1) ||
                    (model.edit.allowEditing && model.toolBar.toolBarItems.indexOf("index") !== -1)))) {

                if (e.target.id == "EditDialog_" + proxy._id + "_Save") {
                    if (!proxy._sendSaveRequest())
                        $("#" + proxy._id + "_dialogEdit").ejDialog("close");
                } else if (e.target.id == "EditDialog_" + proxy._id + "_Cancel") {
                    //proxy._sendCancelRequest();
                    $("#" + proxy._id + "_dialogEdit").ejDialog("close");
                }
            }
            else {
                proxy._sendCancelRequest();
            }

            return false;
        },

        rowSelecting: function (args) {
            var proxy = this;
            if (proxy.model.rowSelecting) {
                if (proxy._trigger("rowSelecting", args))
                    return true;
            }
            return true;
        },

        rowSelected: function (args) {
            var proxy = this;
            if (args.target == "ejGanttGridTree")
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
            if (args.target == "ejGanttChart")
                proxy._$gridtreeHelper.ejGanttGridTree("selectRows", args.recordIndex);
            proxy.model.selectedRowIndex = args.recordIndex;
            this.selectedItem(args.data);
            if (proxy.model.toolBar.allowToolBar) {

                if (proxy.model.toolBar.toolBarItems.indexOf("indent") !== -1 || proxy.model.toolBar.toolBarItems.indexOf("outdent") !== -1) {

                    proxy.updateIndentOutdentOption(args.data);
                }

                var enableItems = [];

                if (proxy.model.edit.allowDeleting) {
                    if (proxy.model.toolBar.toolBarItems.indexOf("delete") !== -1)
                        enableItems.push($($('.e-deleteitem')[0]).parent()[0]);
                }

                if (proxy.model.edit.allowEditing) {
                    if (proxy.model.toolBar.toolBarItems.indexOf("edit") !== -1)
                        enableItems.push($($('.e-edititem')[0]).parent()[0]);
                }

                $('.e-gantttoolbar').ejToolbar('enableItem', enableItems);
            }

            if (proxy.model.rowSelected)
                proxy._trigger("rowSelected", args);
        },

        _gridTreeRowHover: function (args) {
            var proxy = this;
            if (args.reason == "mouseenter")
                proxy._$ganttchartHelper.ejGanttChart("addRowHover", args.index);
            else if (args.reason == "mouseleave")
                proxy._$ganttchartHelper.ejGanttChart("removeRowHover");
        },

        updateIndentOutdentOption: function (ganttRecord) {
            var proxy = this;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            var recordIndex = proxy._updatedGanttRecords.indexOf(ganttRecord),
                previousGanttRecord;

            var indentIconElement = $($('.e-indent')[0]).parent()[0],
                outentIconElement = $($('.e-outdent')[0]).parent()[0],
                disabledItems = [];

            if (recordIndex === 0) {
                if (indentIconElement)
                    $('.e-gantttoolbar').ejToolbar('disableItem', indentIconElement);

                if (outentIconElement)
                    $('.e-gantttoolbar').ejToolbar('disableItem', outentIconElement);

                $('.e-gantttoolbar').ejToolbar('disableItem', disabledItems);
            }
            else {
                if (ganttRecord.level === 0) {
                    if (indentIconElement)
                        $('.e-gantttoolbar').ejToolbar('enableItem', indentIconElement);
                    if (outentIconElement)
                        $('.e-gantttoolbar').ejToolbar('disableItem', outentIconElement);
                }
                else {
                    previousGanttRecord = proxy._updatedGanttRecords[recordIndex - 1];
                    if (ganttRecord.level - previousGanttRecord.level === 1) {
                        if (outentIconElement)
                            $('.e-gantttoolbar').ejToolbar('enableItem', outentIconElement);
                        if (indentIconElement)
                            $('.e-gantttoolbar').ejToolbar('disableItem', indentIconElement);
                    }
                    else {
                        if (indentIconElement)
                            $('.e-gantttoolbar').ejToolbar('enableItem', indentIconElement);
                        if (outentIconElement)
                            $('.e-gantttoolbar').ejToolbar('enableItem', outentIconElement);
                    }
                }
            }
        },

        _ganttChartRowHover: function (args) {
            var proxy = this;
            if (args.reason == "mouseenter")
                proxy._$gridtreeHelper.ejGanttGridTree("addRowHover", args.index);
            else if (args.reason == "mouseleave")
                proxy._$gridtreeHelper.ejGanttGridTree("removeRowHover");
        },

        actionComplete: function (args) {
            var proxy = this;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            proxy._gridRows = proxy.getGridRows();
            proxy._ganttChartRows = proxy.getGanttChartRows();

            if (args.requestType === "searching" || args.requestType === "sorting") {

                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);
                proxy._isValidationEnabled = false;
                proxy._connectorlineIds = [];
                proxy._connectorLinesCollection = [];
                proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                proxy._createConnectorLinesCollection();
            }

            if (args.requestType === "delete") {
                //proxy._$ganttchartHelper.ejGanttChart("deleteChartRows", args.startIndex, args.count);
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);
                proxy._isValidationEnabled = false;

                if (args.data.parentItem) {
                    args.data.parentItem.hasChildGanttRecords = args.data.parentItem.childGanttRecords ? args.data.parentItem.childGanttRecords.length > 0 : false;
                    args.data.parentItem.expanded = args.data.parentItem.hasChildGanttRecords;
                    proxy.refreshGanttRecord(args.data.parentItem);
                }

                if (proxy.model.predecessorMapping) {
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._createConnectorLinesCollection();
                }

                if (proxy.model.toolBar.allowToolBar) {
                    if (proxy.model.toolBar.toolBarItems.indexOf("delete") !== -1) {
                        var toolBarDisableItems = [];
                        toolBarDisableItems.push($($('.e-deleteitem')[0]).parent()[0]);
                        $('.e-gantttoolbar').ejToolbar('disableItem', toolBarDisableItems);
                    }
                }
                proxy.updateAltRow(proxy.getUpdatedRecords, 0, 1);//for set e-alt-row style after deletion of element
            }

            if (args.requestType === "selection") {
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
            }

            if (args.requestType === "rowHover") {
                proxy._$ganttchartHelper.ejGanttChart("addRowHover", args.index);
            }

            if (args.requestType === "scroll") {

                proxy._$ganttchartHelper.ejGanttChart("onScrollHelper", args.delta);
            }

            if (args.requestType === "save") {

                if (args._cAddedRecord) {

                    proxy.model.ganttRecords = proxy._$gridtreeHelper.ejGanttGridTree("getUpdatedRecords");
                    proxy._rows = proxy._$gridtreeHelper.ejGanttGridTree("getFlatGanttRecords");
                    proxy._$ganttchartHelper.ejGanttChart("setUpdatedRecords", proxy._currentViewData, proxy._updatedGanttRecords, proxy._rows);
                    proxy._ids = proxy._$gridtreeHelper.ejGanttGridTree("getUpdatedIds");

                    proxy._$gridtreeHelper.ejGanttGridTree("renderRecord", args);
                    proxy._$ganttchartHelper.ejGanttChart("renderRecord", args);

                    //var addedGridRow = proxy._$gridtreeHelper.ejGanttGridTree("renderRecord", args),
                    //    addedchartRow = proxy._$ganttchartHelper.ejGanttChart("renderRecord", args);

                    //if (args.recordIndex == 0) {
                    //    $(proxy._gridRows).eq(args.recordIndex).before(addedGridRow);
                    //    $(proxy._ganttChartRows).eq(args.recordIndex).before(addedchartRow);
                    //} else {
                    //    $(proxy._gridRows).eq(args.recordIndex).after(addedGridRow);
                    //    $(proxy._ganttChartRows).eq(args.recordIndex).after(addedchartRow);
                    //}


                } else if (args._cModifiedData) {

                    proxy.refreshGanttRecord(args._cModifiedData);
                    if (args._cModifiedData.predecessor) {
                        if (args.previousValue) {
                            proxy._removeConnectorLine(args.previousValue, args._cModifiedData);
                        }
                    }
                }

                if (proxy.model.predecessorMapping) {

                    if ((args._cAddedRecord && args._cAddedRecord.predecessor)
                        || (args._cModifiedData && args._cModifiedData.predecessor)) {
                        proxy._isValidationEnabled = true;
                    }
                    if (args._cAddedRecord) {
                        proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                        proxy._connectorlineIds = [];
                        proxy._connectorLinesCollection = [];
                        proxy._createConnectorLinesCollection();
                        proxy._addConnectorLine(args._cAddedRecord);
                    } else {
                        if (args._cModifiedData) {
                            if (args.previousValue) {
                                proxy._removeConnectorLine(args.previousValue, args._cModifiedData);
                            }
                            proxy._addConnectorLine(args._cModifiedData);
                            if (args._cModifiedData.predecessor) {
                                proxy._isMileStoneEdited = args._cModifiedData.isMilestone;
                                proxy._updatedConnectorLineCollection = [];
                                proxy._connectorlineIds = [];
                                proxy._validatePredecessor(args._cModifiedData);
                                if (proxy._updatedConnectorLineCollection.length > 0) {
                                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                                }
                            }
                            if (args._cModifiedData.parentItem)
                                proxy._updateParentItem(args._cModifiedData);
                        }
                    }
                }
            }

            proxy._ganttActionCompleteTrigger(args);
        },

        chartactionComplete: function (args) {
            var proxy = this;

            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();

            if (args.requestType === "rowHover") {
                proxy._$gridtreeHelper.ejGanttGridTree('addRowHover', args.index);
            }

            if (args.requestType === "scroll") {

                proxy._isRenddered = false;

                if (proxy.model.enableVirtualization) {

                    if (proxy.model.predecessorMapping) {

                        if (proxy._connectorLinesCollection.length > 0) {
                            proxy._clearConnectorLines();
                        }

                        proxy._connectorlineIds = [];
                        proxy._connectorLinesCollection = [];
                        proxy._createConnectorLinesCollection();
                    }
                }

                proxy._$gridtreeHelper.ejGanttGridTree('onScrollHelper', args.delta);

                proxy._isRenddered = true;
            }

            if (args.requestType === "selection") {
                proxy._$gridtreeHelper.ejGanttGridTree("selectRows", args.recordIndex);
            }

            proxy._ganttActionCompleteTrigger(args);
        },

        _clearConnectorLines: function () {
            var proxy = this,
                count = 0,
                length = proxy._connectorLinesCollection.length,
                connectorLine,
                childId,
                childGanttRecord,
                predecessors,
                predecessorslength,
                i = 0,
                predecessor;

            for (count; count < length; count++) {

                connectorLine = proxy._connectorLinesCollection[count];
                childId = connectorLine && connectorLine.ConnectorLineId.split("child")[1];
                childGanttRecord = proxy._rows[proxy._ids.indexOf(childId)];
                predecessors = childGanttRecord.predecessor;
                predecessorslength = predecessors.length;

                for (i = 0; i < predecessorslength; i++) {

                    predecessor = predecessors[i];

                    if (predecessor.to === childId && predecessor.isdrawn) {
                        predecessor.isdrawn = false;
                    }
                }
            }

            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
        },


        taskbarEditing: function (args) {

            this._trigger("taskbarEditing", args);
        },

        _refreshParent: function (item) {
            this.refreshGanttRecord(item);
            if (item.parentItem) {
                this._refreshParent(item.parentItem);
            }
        },

        taskbarEdited: function (args) {
            var proxy = this,
                ganttRecord = args.data, model = proxy.model,
                projectStartDate = new Date(proxy._scheduleWeeks[0]),
                remainDays,
                day,
                offset = 0;

            proxy._isValidationEnabled = false;

            if (args.dragging) {
                if (args.previousData.left !== ganttRecord.left) {
                    remainDays = ganttRecord.left % proxy._perDayWidth,
                           day = (ganttRecord.left - remainDays) / proxy._perDayWidth,
                           offset = 0;

                    if (remainDays > 0) {
                        offset = remainDays >= proxy._perDayWidth / 2 ? 1 : 0;
                    }

                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);

                    if (remainDays >= (proxy._perDayWidth / 2)) {
                        ganttRecord.left = ganttRecord.left + proxy._perDayWidth - remainDays;
                    } else {
                        ganttRecord.left = ganttRecord.left - remainDays;
                    }

                    if (this._stringHolidays &&
                     this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }

                    if (this.model.includeWeekend == false) {
                        if (projectStartDate.getDay() === 0) {
                            projectStartDate.setDate(projectStartDate.getDate() + 1);
                            ganttRecord.left += proxy._perDayWidth;
                        } else if (projectStartDate.getDay() === 6) {
                            projectStartDate.setDate(projectStartDate.getDate() + 2);
                            ganttRecord.left += (2 * proxy._perDayWidth);
                        }
                    }

                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }

                    ganttRecord.startDate = ganttRecord._calculateFormatedDate(projectStartDate, model.dateFormat);
                    ganttRecord._calculateEndDate(model.includeWeekend, this._holidays, model.dateFormat);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model.dateFormat);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }

            } else if (args.rightResizing) {

                if (args.previousData.width !== ganttRecord.width) {
                    day = ganttRecord.width / proxy._perDayWidth;
                    var rem = ganttRecord.width % proxy._perDayWidth;
                    offset = rem >= (proxy._perDayWidth / 2) ? 1 : 0;
                    var startDate = proxy._getDateFromFormat(ganttRecord.startDate),
                    endDate = ganttRecord._dateUpdate(startDate, day - 1 + offset);
                    if (model.holidays && model.holidays.length) {
                        var holidayIndex = 0,
                            holidayslength = model.holidays.length,
                            holiday;
                        for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                            holiday = new Date(this._holidays[holidayIndex]);
                            if (endDate === holiday) {
                                endDate.setDate(endDate.getDate() - 1);
                            }
                        }
                    }

                    if (this.model.includeWeekend == false) {
                        if (endDate.getDay() == 0) {
                            endDate.setDate(endDate.getDate() - 2);
                        } else if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() - 1);
                        }
                    }

                    if (this._stringHolidays &&
                   this._stringHolidays.indexOf(ganttRecord._getFormatedDate(endDate, model.dateFormat)) != -1) {
                        endDate.setDate(endDate.getDate() + 1);
                        if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() + 2);
                        }
                    }

                    ganttRecord.endDate = ganttRecord._calculateFormatedDate(endDate, model.dateFormat);
                    ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model.dateFormat);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }

            } else if (args.leftResizing) {
                if (args.previousData.left !== ganttRecord.left) {
                    remainDays = ganttRecord.left % proxy._perDayWidth;
                    day = (ganttRecord.left - remainDays) / proxy._perDayWidth;
                    offset = 0;
                    if (remainDays > 0) {
                        offset = remainDays >= proxy._perDayWidth / 2 ? 1 : 0;
                    }
                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);

                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }

                    if (this.model.includeWeekend == false) {
                        if (projectStartDate.getDay() === 0) {
                            projectStartDate.setDate(projectStartDate.getDate() + 1);
                            ganttRecord.left += proxy._perDayWidth;
                        } else if (projectStartDate.getDay() === 6) {
                            projectStartDate.setDate(projectStartDate.getDate() + 2);
                            ganttRecord.left += (2 * proxy._perDayWidth);
                        }
                    }

                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }

                    if (remainDays >= (proxy._perDayWidth / 2)) {
                        ganttRecord.left = ganttRecord.left + proxy._perDayWidth - remainDays;
                    } else {
                        ganttRecord.left = ganttRecord.left - remainDays;
                    }
                    ganttRecord.startDate = ganttRecord._calculateFormatedDate(projectStartDate, model.dateFormat);
                    ganttRecord._calculateDuration(projectStartDate, proxy._getDateFromFormat(ganttRecord.endDate), model.includeWeekend, model.holidays);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model.dateFormat);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }
            } else if (args.progressResizing) {
                ganttRecord.status = ganttRecord._getProgressPercent(ganttRecord.width, ganttRecord.progressWidth);
                if (ganttRecord.parentItem) {
                    ganttRecord._updateParentProgress(ganttRecord.parentItem);
                }
                proxy._refreshParent(ganttRecord);
                //proxy.refreshGanttRecord(ganttRecord);
                //proxy.refreshGanttRecord(ganttRecord.parentItem);
            }
            if (proxy._isValidationEnabled) {
                proxy._updateEditedGanttRecord(ganttRecord);
                if (ganttRecord.predecessor) {
                    proxy._isMileStoneEdited = ganttRecord.isMilestone;
                    proxy._updatedConnectorLineCollection = [];
                    proxy._connectorlineIds = [];
                    proxy._validatePredecessor(ganttRecord);
                    if (proxy._updatedConnectorLineCollection.length > 0) {
                        proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                    }
                }
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord);
                proxy.refreshGanttRecord(ganttRecord);
            }
            proxy._trigger("taskbarEdited", args);
        },

        actionBegin: function (args) {

            var proxy = this;

            if (!proxy._trigger("actionBegin", args)) {

                if (args.requestType === "beginedit") {
                    proxy._sendEditRequest();
                }

            }

        },

        _ganttActionCompleteTrigger: function (args) {
            this._trigger("actionComplete", args);
        },

        _queryCellInfo: function (args) {
            this._trigger("queryCellInfo", args);
        },

        _beginEdit: function (args) {
            var proxy = this;
            proxy._trigger("beginEdit", args);
            proxy._isinBeginEdit = true;
            if (proxy.model.toolBar.allowToolBar) {
                var toolBarEnabledItems = [];
                toolBarEnabledItems.push($($('.e-cancel')[0]).parent()[0]);
                toolBarEnabledItems.push($($('.e-saveitem')[0]).parent()[0]);
                $('.e-gantttoolbar').ejToolbar('enableItem', toolBarEnabledItems);
            }
        },

        _endEdit: function (args) {
            var proxy = this;

            proxy._isinBeginEdit = false;

            if (args.isModified) {
                proxy._currentViewData = proxy._$gridtreeHelper.ejGanttGridTree("getCurrentViewData");
                proxy._isValidationEnabled = true;
                proxy._updateEditedGanttRecords(args);
            }

            proxy.refreshGanttRecord(args.data);

            delete args.isModified;
            if (proxy.model.toolBar.allowToolBar) {
                var toolBarDisableItems = [];

                if (proxy.model.toolBar.toolBarItems.indexOf("cancel") !== -1) {
                    toolBarDisableItems.push($($('.e-cancel')[0]).parent()[0]);
                }

                if (proxy.model.toolBar.toolBarItems.indexOf("delete") !== -1) {
                    toolBarDisableItems.push($($('.e-saveitem')[0]).parent()[0]);
                }

                $('.e-gantttoolbar').ejToolbar('disableItem', toolBarDisableItems);
            }
            proxy._trigger("endEdit", args);
        },



        _rowDataBound: function (args) {
            this._trigger("rowDataBound", args);
        },

        _toolBarClick: function (sender) {
            var proxy = this,
                $gridEle = $(this.itemsContainer).closest(".e-gantt"),
                gridInstance = $gridEle.ejGantt("instance"),
                gridId = $gridEle.attr('id');

            gridInstance._clearContextMenu();

            if (sender.event == undefined && sender.target.tagName == "INPUT" && sender.currentTarget.id == gridId + "_search")
                return false;

            $.isFunction($.fn.ejDatePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDatePicker("hide");

            if ($gridEle.ejGantt("instance")._trigger("toolBarClick", sender))
                return false;

            switch (sender.currentTarget.id) {

                case gridId + "_add":
                    gridInstance._toolbarOperation(gridId + "_add");
                    break;

                case gridId + "_edit":
                    gridInstance._toolbarOperation(gridId + "_edit");
                    break;

                case gridId + "_delete":
                    gridInstance._toolbarOperation(gridId + "_delete");
                    break;

                case gridId + "_update":
                    gridInstance._toolbarOperation(gridId + "_update");
                    break;

                case gridId + "_cancel":
                    gridInstance._toolbarOperation(gridId + "_cancel");
                    break;

                case gridId + "_search":
                    gridInstance._toolbarOperation(gridId + "_search", $(sender.currentTarget).find("input").val());
                    break;

                case gridId + "_indent":
                    gridInstance._toolbarOperation(gridId + "_indent");
                    break;

                case gridId + "_outdent":
                    gridInstance._toolbarOperation(gridId + "_outdent");
                    break;

                case gridId + "_expandAll":
                    gridInstance._toolbarOperation(gridId + "_expandAll");
                    break;

                case gridId + "_collapseAll":
                    gridInstance._toolbarOperation(gridId + "_collapseAll");
                    break;
            }
            return false;
        },

        _toolbarOperation: function (operation, searchEle) {
            var $gridEle = this.element, gridObject = $gridEle.ejGantt("instance"), gridId = $gridEle.attr('id'), proxy = this;
            switch (operation) {

                case gridId + "_add":
                    proxy._sendAddRequest();
                    break;

                case gridId + "_edit":
                    proxy._sendEditRequest();
                    break;

                case gridId + "_delete":
                    proxy._$gridtreeHelper.ejGanttGridTree("sendDeleteRequest");
                    break;

                case gridId + "_update":
                    proxy._$gridtreeHelper.ejGanttGridTree("sendSaveRequest");
                    //gridObject.sendSaveRequest();

                    if (proxy.model.toolBar.toolBarItems.indexOf("cancel") !== -1) {
                        proxy._disabledToolItems.push($($('.e-cancel')[0]).parent()[0]);
                    }

                    if (proxy.model.toolBar.toolBarItems.indexOf("save") !== -1) {
                        proxy._disabledToolItems.push($($('.e-saveitem')[0]).parent()[0]);
                    }
                    $('.e-gantttoolbar').ejToolbar('disableItem', proxy._disabledToolItems);

                    proxy._disabledToolItems = [];

                    break;

                case gridId + "_cancel":
                    proxy._$gridtreeHelper.ejGanttGridTree("cancelEditCell");

                    if (proxy.model.toolBar.toolBarItems.indexOf("cancel") !== -1) {
                        proxy._disabledToolItems.push($($('.e-cancel')[0]).parent()[0]);
                    }

                    if (proxy.model.toolBar.toolBarItems.indexOf("save") !== -1) {
                        proxy._disabledToolItems.push($($('.e-saveitem')[0]).parent()[0]);
                    }

                    $('.e-gantttoolbar').ejToolbar('disableItem', proxy._disabledToolItems);
                    proxy._disabledToolItems = [];
                    break;

                case gridId + "_search":
                    proxy._$gridtreeHelper.ejGanttGridTree("sendSearchRequest", searchEle);
                    //proxy.model.enableContextMenu = searchEle.length == 0 ? true : false;
                    proxy._seacrhString = searchEle;

                    proxy._updateToolbarOptions(searchEle);

                    break;

                case gridId + "_indent":
                    proxy._sendIndentRequest();
                    break;

                case gridId + "_outdent":
                    proxy._sendOutdentRequest();
                    break;

                case gridId + "_expandAll":
                    proxy._expandAll();
                    break;

                case gridId + "_collapseAll":
                    proxy._collapseAll();
                    break;
            }
            return false;
        },

        _setModel: function (options) {

            var proxy = this, option,
                model = proxy.model;

            for (option in options) {
                switch (option) {
                    case "sortSettings":
                        proxy._sendSortingRequest(options[option]);
                        break;
                    case "destroy":
                        proxy.destroy();
                        break;
                    case "allowGanttChartEditing":
                        proxy._ganttChartEditing(options[option]);
                        break;
                    case "canResizeProgressBar":
                        proxy._resizeProgressbar(options[option]);
                        break;
                    case "showTaskbarTooltip":
                        proxy._showTooltip(options[option]);
                        break;
                    case "showTaskbarDragTooltip":
                        proxy._showEditingTooltip(options[option]);
                        break;
                    case "altRow":
                        proxy._updateAltRow(options[option]);
                        break;
                    case "localization":
                        model.ganttRecords = [];
                        model.parentRecords = [];
                        model.dateFormat = "";
                        proxy._clearContextMenu();
                        model.localization = options[option];
                        proxy.element.ejGantt("destroy").ejGantt(model);
                        break;
                    case "allowSelection":
                        proxy._updateRowSelection(options[option]);
                        break;
                    case "edit":
                        $.extend(model.edit, options[option]);
                        proxy._updateEditMode(options[option]);
                        break;
                    case "taskbarBackground":
                        proxy._updateTaskbarBackground(options[option]);
                        break;
                    case "progressbarBackground":
                        proxy._updateProgressbarBackground(options[option]);
                        break;
                    case "parentTaskbarBackground":
                        proxy._updateParentTaskbarBackground(options[option]);
                        break;
                    case "parentProgressbarBackground":
                        proxy._updateParentProgressbarBackground(options[option]);
                        break;
                    case "showTaskNames":
                        proxy._updateRenderTaskNames(options[option]);
                        break;
                    case "showProgressStatus":
                        proxy._updateRendeProgressStatus(options[option]);
                        break;
                    case "showResourceNames":
                        proxy._updateRenderResourceNames(options[option]);
                        break;
                    case "progressbarHeight":
                        proxy._updateProgressbarHeight(options[option]);
                        break;
                    case "baselineColor":
                        proxy._updateBaselineColor(options[option]);
                        break;
                    case "renderBaseline":
                        proxy._updateRenderBaseline(options[option]);
                        break;
                    case "allowColumnResize":
                        proxy._updateAllowColumnResize(options[option]);
                        break;
                    case "allowKeyboardNavigation":
                        proxy._updateAllowKeyboardNavigation(options[option]);
                        break;
                    case "selectedItem":
                        proxy._updateSelectedItem(options[option]);
                        break;
                    case "dataSource":
                        model.dataSource = [];
                        model.ganttRecords = [];
                        model.parentRecords = [];
                        model.dataSource = (options[option]);
                        proxy.element.ejGantt("destroy").ejGantt(model);
                }
            }
        },

        _updateSelectedItem: function (item) {
            var proxy = this,
                args = {};
            if (item[proxy.model.taskIdMapping] != null) {


                args.recordIndex = proxy._ids.indexOf((item[proxy.model.taskIdMapping]).toString());
                args.item = proxy.model.ganttRecords[args.recordIndex];
                args.target = "ejGanttGridTree";
                proxy.rowSelected(args);
                proxy._$gridtreeHelper.ejGanttGridTree("selectRows", args.recordIndex);
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
                proxy._$ganttchartHelper.ejGanttChart("updateSelectedItem", args.recordIndex);
            }
        },

        _updateAllowKeyboardNavigation: function (bool) {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("updateAllowKeyboardNavigation", bool);
            proxy._$ganttchartHelper.ejGanttChart("updateAllowKeyboardNavigation", height);
        },

        _updateAllowColumnResize: function (bool) {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("updateAllowColumnResize", bool);
        },

        _updateProgressbarHeight: function (height) {
            var proxy = this;
            this.model.progressbarHeight = height;
            proxy._$ganttchartHelper.ejGanttChart("updateProgressbarHeight", height);
        },

        _updateBaselineColor: function (color) {
            var proxy = this;
            this.model.baselineColor = color;
            proxy._$ganttchartHelper.ejGanttChart("updateBaselineColor", color);
        },

        _updateRenderBaseline: function (bool) {
            var proxy = this;
            this.model.renderBaseline = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderBaseline", bool);
            proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines",
                proxy._connectorLinesCollection);
        },

        _updateRenderTaskNames: function (bool) {
            var proxy = this;
            this.model.showTaskNames = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderTaskNames", bool);
        },

        _updateRendeProgressStatus: function (bool) {
            var proxy = this;
            this.model.showProgressStatus = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRendeProgressStatus", bool);
        },

        _updateRenderResourceNames: function (bool) {
            var proxy = this;
            this.model.showResourceNames = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderResourceNames", bool);
        },

        _updateTaskbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.taskbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateTaskbarBackground", bgcolor);
        },

        _updateProgressbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.progressbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateProgressbarBackground", bgcolor);
        },

        _updateParentTaskbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.parentTaskbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateParentTaskbarBackground", bgcolor);
        },

        _updateParentProgressbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.parentProgressbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateParentProgressbarBackground", bgcolor);
        },

        _updateEditMode: function (edit) {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("updateEditMode", edit);

        },

        _updateRowSelection: function (bool) {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("updateRowSelection", bool);
        },

        _updateAltRow: function (bool) {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("updateAltRow", bool);
        },

        /**
       * To Open the dialog to add new task to the gantt
       * @return jQuery
       * @example 
       * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // Create Gantt
       * var ganttObj = $("#gantt").data("ejGantt");
       * gantObj.ejGantt("openAddDialog"); // To open the add dialog
       * &lt;/script&gt;
       * @example 
       *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // open Add dialog
       * $("#gantt").ejGantt("openAddDialog");	
       * &lt;/script&gt;
       * @memberof ejGantt
       * @instance
        */
        openAddDialog: function () {
            this._sendAddRequest();
        },

        /**
        * To Open the dialog to edit existing task to the gantt
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("openEditDialog"); // To open the add dialog
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // open Add dialog
        * $("#gantt").ejGantt("openEditDialog");	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        openEditDialog: function () {
            this._sendEditRequest();
        },

        /**
        * To search an item with search string provided at the run time
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("searchItem",$("#text").val()); // To search a task
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // To search a task
        * $("#gantt").ejGantt("searchItem",$("#text").val());	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        searchItem: function (searchString) {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("sendSearchRequest", searchString);
        },


        /**
        * To delete a selected item in gantt
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("deleteItem"); // To delete a task
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // To delete an item
        * $("#gantt").ejGantt("deleteItem");	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        deleteItem: function () {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("sendDeleteRequest");
        },


        /**
          * To Expand all the parent items in gantt
          * @return jQuery
          * @example 
          * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // Create Gantt
          * var ganttObj = $("#gantt").data("ejGantt");
          * gantObj.ejGantt("expandAllItems"); // To expand all parent items in gantt
          * &lt;/script&gt;
          * @example 
          *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // To expand all items
          * $("#gantt").ejGantt("expandAllItems");	
          * &lt;/script&gt;
          * @memberof ejGantt
          * @instance
           */

        expandAllItems: function () {
            this._expandAll();
        },

        /**
          * To collapse all the parent items in gantt
          * @return jQuery
          * @example 
          * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // Create Gantt
          * var ganttObj = $("#gantt").data("ejGantt");
          * gantObj.ejGantt("collapseAllItems"); // To collapse all parent items in gantt
          * &lt;/script&gt;
          * @example 
          *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // To expand all items
          * $("#gantt").ejGantt("collapseAllItems");	
          * &lt;/script&gt;
          * @memberof ejGantt
          * @instance
           */
        collapseAllItems: function () {
            this._collapseAll();
        },

        /**
        * To indent a selected item in gantt
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("indentItem"); // To indent a selected item in gantt
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // To indent a selected item in gantt
        * $("#gantt").ejGantt("indentItem");	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        indentItem: function () {
            this._sendIndentRequest();
        },

        /**
       * To outdent a selected item in gantt
       * @return jQuery
       * @example 
       * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // Create Gantt
       * var ganttObj = $("#gantt").data("ejGantt");
       * gantObj.ejGantt("outdentItem"); // To outdent a selected item in gantt
       * &lt;/script&gt;
       * @example 
       *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // To outdent a selected item in gantt
       * $("#gantt").ejGantt("outdentItem");	
       * &lt;/script&gt;
       * @memberof ejGantt
       * @instance
        */
        outdentItem: function () {
            this._sendOutdentRequest();
        },


        /**
        * To cancel the edited state of an item in gantt
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("cancelEdit"); // To cancel edited
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // To outdent a selected item in gantt
        * $("#gantt").ejGantt("cancelEdit");	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        cancelEdit: function () {
            this.sendCancelRequest();
        },


        /**
       * To expand and collapse an item in gantt using item's ID
       * @return jQuery
       * @example 
       * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // Create Gantt
       * var ganttObj = $("#gantt").data("ejGantt");
       * gantObj.ejGantt("expandCollapseRecord" , "23"); // To expand collapse an item
       * &lt;/script&gt;
       * @example 
       *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // To expand collapse an item
       * $("#gantt").ejGantt("expandCollapseRecord" , "23");	
       * &lt;/script&gt;
       * @memberof ejGantt
       * @instance
        */
        expandCollapseRecord: function (taskId) {
            var proxy = this, args = {},
                record = proxy.model.ganttRecords[proxy._ids.indexOf(taskId)],
                expand = record.expanded == true ? false : true;
            args.data = record;
            args.recordIndex = proxy._ids.indexOf(taskId);
            args.expanded = expand;
            args.requestType = "expandCollapse";
            proxy._$gridtreeHelper.ejGanttGridTree("sendExpandCollapseRequest", args);
        },



        /**
       * To save the edited state of an item in gantt
       * @return jQuery
       * @example 
       * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // Create Gantt
       * var ganttObj = $("#gantt").data("ejGantt");
       * gantObj.ejGantt("saveEdit"); // To save edited state of an item
       * &lt;/script&gt;
       * @example 
       *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // To expand collapse an item
       * $("#gantt").ejGantt("saveEdit");	
       * &lt;/script&gt;
       * @memberof ejGantt
       * @instance
        */
        saveEdit: function () {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("sendSaveRequest");
        },

        _showTooltip: function (boolValue) {
            var proxy = this;
            this.model.showTaskbarTooltip = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("showTooltip", boolValue);
        },

        _showEditingTooltip: function (boolValue) {
            var proxy = this;
            this.model.showTaskbarDragTooltip = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("showEditingTooltip", boolValue);
        },

        _ganttChartEditing: function (boolValue) {
            var proxy = this;
            proxy.model.allowGanttChartEditing = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("ganttChartEditing", boolValue);
        },

        _resizeProgressbar: function (boolValue) {
            var proxy = this;
            proxy.model.canResizeProgressBar = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("resizeProgressbar", boolValue);

        },

        _sendSortingRequest: function (sortSettings) {
            var proxy = this, columnName, columnSortDirection;
            columnName = sortSettings.sortedColumns[0].field;
            columnSortDirection = sortSettings.sortedColumns[0].direction;
            proxy._$gridtreeHelper.ejGanttGridTree("sendSortingRequest", columnName, columnSortDirection);
        },

        sendCancelRequest: function () {
            var proxy = this;
            proxy._$gridtreeHelper.ejGanttGridTree("cancelEditCell");

            if (proxy.model.toolBar.toolBarItems.indexOf("cancel") !== -1) {
                proxy._disabledToolItems.push($($('.e-cancel')[0]).parent()[0]);
            }

            if (proxy.model.toolBar.toolBarItems.indexOf("save") !== -1) {
                proxy._disabledToolItems.push($($('.e-saveitem')[0]).parent()[0]);
            }

            $('.e-gantttoolbar').ejToolbar('disableItem', proxy._disabledToolItems);
            proxy._disabledToolItems = [];
        },

        _sendAddRequest: function () {

            var proxy = this,
                columns = proxy._columns,
                length = columns.length,
                columnCount = 0,
                cloneData = {},
                args = {};
            proxy._isinAddnewRecord = true;

            for (columnCount; columnCount < length; columnCount++)
                cloneData[columns[columnCount].mappingName] = "";

            args.data = cloneData;
            args.requestType = "add";
            proxy._$gridtreeHelper.ejGanttGridTree("cleanUpSelection");
            proxy.renderedEditDialog(args);
        },

        _sendEditRequest: function ($tr) {
            var proxy = this,
                model = proxy.model,
                args,
                data;

            if (ej.isNullOrUndefined($tr)) {
                proxy._currentTrIndex = proxy.model.selectedRowIndex;
                proxy._$currentTr = $(proxy._$gridtreeHelper.ejGanttGridTree("getGridRows")).eq(proxy.model.selectedRowIndex);
            } else {
                proxy._currentTrIndex = $tr.index();
                proxy._$currentTr = $tr;
            }
            data = proxy._$gridtreeHelper.ejGanttGridTree("getCurrentViewData")[model.selectedRowIndex];

            args = { rowElement: proxy._$currentTr, data: data };
            proxy._trigger("beginEdit", args);
            args.requestType = "beginedit";
            proxy.renderedEditDialog(args);
        },

        _sendSaveRequest: function () {
            var obj = {},
                doc = document,
                proxy = this,
                index = 0,
                formelement = doc.getElementById(proxy._id + "EditForm"),
                length = formelement && formelement.length,
                editedRowWrap = $(formelement).closest('div'),
                columnName,
                $element,
                column,
                value,
                args = {};
            if (!this.editFormValidate())
                return true;

            for (index; index < length; index++) {
                columnName = formelement[index].name;
                $element = $(formelement[index]);
                if (columnName != undefined) {
                    if (columnName == "")
                        columnName = formelement[index].id.replace(proxy._id, "");
                    if (columnName != "" && obj[columnName] == null)
                        column = proxy._$gridtreeHelper.ejGanttGridTree("getColumnByField", columnName);

                    if (column) {
                        proxy._cellEditType = column.columnEditType;
                        value = proxy.getCurrentEditCellData($element, column.dropdownData); // formelement[index].value;
                        if (proxy._isinAddnewRecord === false && column.mappingName === proxy.model.predecessorMapping) {
                            args.previousValue = proxy._updatePredecessorValue(obj, value);
                        }
                        if (formelement[index].type != "checkbox")
                            obj[column.mappingName] = value;
                    }
                }
            }
            args.data = obj;
            args.requestType = ej.GanttGridTree.actions.Save;
            if (proxy._trigger("actionBegin", args))
                return true;
            if (editedRowWrap.hasClass("e-editedrow"))
                args._cModifiedData = proxy._updateRecord(obj);
            else if (editedRowWrap.hasClass("e-addedrow"))
                args._cAddedRecord = proxy._createGanttRecord(obj, 0, null);
            proxy._$gridtreeHelper.ejGanttGridTree("processBindings", args);
            proxy._cModifiedData = null;
            proxy._cAddedRecord = null;
            proxy._primaryKeyValues = [];
            proxy._isinAddnewRecord = false;
            return false;
        },

        _sendCancelRequest: function () {
            var proxy = this,
                args = {};
            $("#" + proxy._id + "EditForm").find(".e-datepicker").ejDatePicker("destroy");
            args.requestType = ej.GanttGridTree.actions.Cancel;
            proxy.processBindings(args);
            proxy._primaryKeyValues = [];
            proxy.model.currentData = null;
        },

        _updatePredecessorValue: function (obj, value) {
            var proxy = this,
                prevPredecessors,
                modifiedPredecessors,
                length = 0,
                predecessor,
                ganttPredecessor = [],
                count = 0;
            prevPredecessors = proxy.selectedItem()["predecessor"];

            if (value.length > 0) {
                modifiedPredecessors = proxy.selectedItem()._calculatePredecessor(value);
            }
            if (modifiedPredecessors)
                length = modifiedPredecessors.length;

            var index = 0,
                prevPrdecessorlength = prevPredecessors.length;

            for (index = 0; index < prevPrdecessorlength; index++) {
                predecessor = prevPredecessors[index];
                if (predecessor.from === proxy.selectedItem().taskId.toString()) {
                    ganttPredecessor.push(predecessor);
                }
            }
            for (count = 0; count < length; count++) {
                ganttPredecessor.push(modifiedPredecessors[count]);
            }
            proxy.selectedItem().predecessor = ganttPredecessor;
            proxy.selectedItem().item[proxy.model.predecessorMapping] = value;

            return prevPredecessors;
        },


        getCurrentEditCellData: function ($element, dropdownData) {
            var proxy = this, model = proxy.model, date;
            if ($("#" + proxy._id + "EditForm").length) {
                var cellValue = "";// $element = $("#" + proxy._id + proxy._columnFieldName), 
                switch (proxy._cellEditType) {
                    case ej.GanttGridTree.editingType.String:
                        cellValue = $element.val();
                        break;
                    case ej.GanttGridTree.editingType.Numeric:
                        cellValue = $element.ejNumericTextbox("getValue");
                        $element.ejNumericTextbox("getValue");
                        break;
                    case ej.GanttGridTree.editingType.Dropdown:
                        cellValue = proxy._getSelectedItem($element.ejDropDownList("model.multiSelectedItemsIndex"), dropdownData);
                        break;
                    case ej.GanttGridTree.editingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.GanttGridTree.editingType.DatePicker:
                        date = $element.ejDatePicker("model.value");
                        cellValue = Globalize.format(date, model.dateFormat);
                        break;
                    case ej.GanttGridTree.editingType.DateTimePicker:
                        cellValue = $element.ejDateTimePicker("model.value");
                        break;
                }

                //if (typeof cellValue == "string" && cellValue.length )//&& model.columns[proxy._cellEditingDetails.columnIndex].type == "number")
                //    cellValue = parseFloat(cellValue);
                return cellValue;
            }
            return null;
        },


        _getSelectedItem: function (indexArray, dropDownData) {
            var count = 0,
                length = indexArray.length,
                    selectedItems = [];
            if (dropDownData) {
                for (count = 0; count < length; count++) {
                    $.each(dropDownData, function (index, resourceInfo) {
                        if (indexArray[count] === index) {
                            selectedItems.push(resourceInfo);
                        }
                    });
                }
            }

            return selectedItems;
        },
        _updateRecord: function (data) {
            var proxy = this,
                columns = proxy._columns,
                length = columns.length,
                count = 0,
                ganttRecord = proxy.getCurrentViewData()[proxy.model.selectedRowIndex];

            for (count; count < length; count++) {

                if (columns[count].field.toString() === "predecessor")
                    continue;

                ganttRecord[columns[count].field] = data[columns[count].mappingName];
            }

            return ganttRecord;
        },

        _sendIndentRequest: function () {

            var proxy = this,
                model = proxy.model,
                data = proxy.getCurrentData();

            if (ej.isNullOrUndefined(data)) {
                alert("There is no gantt Record was not selected to perform the Indent ");
            }
            else {

                proxy._updatedGanttRecords = proxy.getUpdatedRecords();
                var previousGanttRecord = proxy._updatedGanttRecords[model.selectedRowIndex - 1];

                if (data.parentItem) {
                    var childIndex = data.parentItem.childGanttRecords.indexOf(data);
                    data.parentItem.childGanttRecords.splice(childIndex, 1);
                    data.parentItem.item[model.childMapping].splice(childIndex, 1);
                } else {
                    var dataSourceIndex = model.dataSource.indexOf(data.item);
                    model.dataSource.splice(dataSourceIndex, 1);
                }
                if (data.level === previousGanttRecord.level) {
                    if (previousGanttRecord.childGanttRecords) {
                        if (!previousGanttRecord.expanded) {
                            previousGanttRecord.childGanttRecords.splice(previousGanttRecord.childGanttRecords.length, 0, data);
                            previousGanttRecord.item[model.childMapping].splice(previousGanttRecord.childGanttRecords.length, 0, data.item);
                        }
                    } else {
                        previousGanttRecord.childGanttRecords = [];
                        previousGanttRecord.childGanttRecords.push(data);
                        data.parentItem = previousGanttRecord;
                        previousGanttRecord.item[model.childMapping] = [];
                        previousGanttRecord.item[model.childMapping].push(data.item);
                    }
                } else if (previousGanttRecord.level - data.level === 1) {
                    previousGanttRecord.parentItem.childGanttRecords.splice(previousGanttRecord.parentItem.childGanttRecords.length, 0, data);
                    previousGanttRecord.parentItem.item[model.childMapping].splice(previousGanttRecord.parentItem.childGanttRecords.length, 0, data.item);
                    data.parentItem = previousGanttRecord.parentItem;
                } else if (previousGanttRecord.level - data.level > 1) {

                    var parentItem = proxy.getParentItem(previousGanttRecord, data.level);
                    parentItem.childGanttRecords.splice(parentItem.childGanttRecords.length, 0, data);
                    parentItem.item[model.childMapping].splice(parentItem.childGanttRecords.length, 0, data.item);
                    data.parentItem = parentItem;

                }
            }
            previousGanttRecord.hasChildGanttRecords = previousGanttRecord.childGanttRecords ? previousGanttRecord.childGanttRecords.length > 0 : false;
            previousGanttRecord.expanded = previousGanttRecord.childGanttRecords ? previousGanttRecord.childGanttRecords.length > 0 : false;
            data.level += 1;
            proxy.updateLevel(data, +1);
            data._updateParentProgress(data);
            proxy.refreshGanttRecord(data);
            proxy.refreshGanttRecord(previousGanttRecord);
            proxy.updateIndentOutdentOption(data);
        },

        //get the parentItem of selected ganttRecord to perform the indent operation

        getParentItem: function (ganttRecord, level) {

            var proxy = this;

            if (ganttRecord.parentItem.level === level)
                return ganttRecord.parentItem;

            return proxy.getParentItem(ganttRecord.parentItem, level);
        },

        _sendOutdentRequest: function () {
            var proxy = this,
                model = proxy.model,
                data = proxy.getCurrentData();
            if (ej.isNullOrUndefined(data)) {
                alert("There is no gantt Record was not selected to perform the Outdent ");
            } else {
                if (data.level === 0)
                    return;
                else {

                    proxy._updatedGanttRecords = proxy.getUpdatedRecords();


                    //Remove the Gantt Record from it's parent GanttRecord
                    if (data.parentItem.childGanttRecords) {

                        var childIndex = data.parentItem.childGanttRecords.indexOf(data);

                        data.parentItem.childGanttRecords.splice(childIndex, 1);
                        data.parentItem.item[model.childMapping].splice(childIndex, 1);

                        if (data.parentItem.childGanttRecords.length == 0) {

                            data.parentItem.childGanttRecords = null;
                            data.parentItem.expanded = false;
                            data.parentItem.hasChildGanttRecords = false;
                        }
                        proxy.refreshGanttRecord(data.parentItem);
                    }

                    //Add Gantt Record to ParentItem of it's parent item
                    if (data.parentItem.parentItem) {

                        var parentIndex = data.parentItem.parentItem.childGanttRecords.indexOf(data.parentItem);

                        data.parentItem.parentItem.childGanttRecords.splice(parentIndex + 1, 0, data);
                        data.parentItem.parentItem.item[model.childMapping].splice(parentIndex + 1, 0, data.item);

                        proxy.refreshGanttRecord(data.parentItem.parentItem);


                    } else {
                        parentIndex = model.dataSource.indexOf(data.parentItem.item);
                        model.dataSource.splice(parentIndex + 1, 0, data.item);
                    }

                    if (data.hasChildGanttRecords) {
                        proxy._updateChildLevel(data);
                    }
                    data.level -= 1; //Decrease the Indent level of the Gantt Record                               
                    childIndex !== -1 && proxy._removeChildGanttRecords(data, childIndex);
                    data.parentItem = data.parentItem.parentItem;
                    if (data.hasChildGanttRecords) {
                        proxy._refreshChildLevel(data);
                    }

                    data.hasChildGanttRecords = data.childGanttRecords ? data.childGanttRecords.length > 0 : false;
                    data.expanded = data.childGanttRecords ? data.childGanttRecords.length > 0 : false;


                    if (data.hasChildGanttRecords) {
                        var count = 0,
                            length = data.childGanttRecords.length,
                            record = null;
                        for (count; count < length; count++) {
                            record = data.childGanttRecords[count];
                            if (record) {
                                proxy.refreshGanttRecord(record);
                            }
                        }
                    }
                    data._updateParentProgress(data);
                    proxy.refreshGanttRecord(data);
                }
            }
            proxy.updateIndentOutdentOption(data);
        },

        _refreshChildLevel: function (ganttRecord) {

            var proxy = this,
                count = 0,
                childGanttRecords = ganttRecord.childGanttRecords,
                length = childGanttRecords.length,
                record;

            for (count = 0; count < length; count++) {
                record = childGanttRecords[count];
                proxy.refreshGanttRecord(record);
                if (record.hasChildGanttRecords) {
                    proxy._refreshChildLevel(record);
                }
            }
        },

        _updateChildLevel: function (ganttRecord) {

            var proxy = this,
                count = 0,
                childGanttRecords = ganttRecord.childGanttRecords,
                length = childGanttRecords.length,
                record;

            for (count = 0; count < length; count++) {
                record = childGanttRecords[count];
                record.level = record.level - 1;
                // proxy.refreshGanttRecord(record);
                if (record.hasChildGanttRecords) {
                    proxy._updateChildLevel(record);
                }
            }
        },

        _removeChildGanttRecords: function (ganttRecord, index) {

            if (ganttRecord.parentItem) {
                var proxy = this,
                    model = proxy.model,
                    childIndex = 0,
                    record,
                    addedChildGanttRecordCount = 0,
                    length = 0;
                parentChildGanttRecords = ganttRecord.parentItem.childGanttRecords,
                parentChildGanttRecordsLength = parentChildGanttRecords && parentChildGanttRecords.length;

                for (childIndex = index; childIndex < parentChildGanttRecordsLength; childIndex++) {

                    record = parentChildGanttRecords[childIndex];


                    if (record) {

                        if (ganttRecord.childGanttRecords) {

                            length = ganttRecord.childGanttRecords.length;
                            ganttRecord.childGanttRecords.splice(length, 0, record);
                            ganttRecord.item[model.childMapping].splice(length, 0, record.item);

                        } else {
                            ganttRecord.childGanttRecords = [];
                            ganttRecord.item[model.childMapping] = [];
                            ganttRecord.childGanttRecords.push(record);
                            ganttRecord.item[model.childMapping].push(record.item);
                        }
                        record.parentItem = ganttRecord;
                        addedChildGanttRecordCount++;
                    }
                }

                if (ganttRecord.parentItem.childGanttRecords) {

                    ganttRecord.parentItem.childGanttRecords.splice(index, addedChildGanttRecordCount);

                    if (ganttRecord.parentItem.childGanttRecords.length == 0) {
                        ganttRecord.parentItem.childGanttRecords = null;
                        ganttRecord.parentItem.expanded = false;
                        ganttRecord.parentItem.hasChildGanttRecords = false;
                        proxy.refreshGanttRecord(ganttRecord.parentItem);
                    }
                }
            }
        },

        updateChildGanttRecordLevel: function (data, index) {
            var childIndex = 0, proxy = this, model = proxy.model,
                parentChildGanttRecords = data.parentItem && data.parentItem.parentItem && data.parentItem.parentItem.childGanttRecords,
                parentChildGanttRecordsLength = parentChildGanttRecords && parentChildGanttRecords.length,
                addedChildGanttRecordCount = 0;
            for (childIndex = index; childIndex < parentChildGanttRecordsLength; childIndex++) {
                var childGanttRecord = parentChildGanttRecords[childIndex];
                if (childGanttRecord.childGanttRecords) {
                    childGanttRecord.childGanttRecords.splice(childGanttRecord.childGanttRecords.length, 0, childGanttRecord);
                    childGanttRecord.item[model.childMapping].splice(childGanttRecord.childGanttRecords.length, 0, childGanttRecord.item);
                }
                else {
                    childGanttRecord.childGanttRecords = [];
                    childGanttRecord.item[model.childMapping] = [];
                    if (childGanttRecord) {
                        childGanttRecord.childGanttRecords.push(childGanttRecord);
                        childGanttRecord.item[model.childMapping].push(childGanttRecord.item);
                    }
                }
                proxy.refreshGanttRecord(data.parentItem);
                data.parentItem = childGanttRecord;
                ++addedChildGanttRecordCount;
            }
            data.parentItem.childGanttRecords.splice(index, addedChildGanttRecordCount);
            if (data.parentItem.childGanttRecords.length == 0) {
                data.parentItem.childGanttRecords = null;
                data.parentItem.expanded = false;
                data.parentItem.hasChildGanttRecords = false;
            }

            proxy.refreshGanttRecord(data.parentItem);

        },

        updateLevel: function (childGanttRecord, offset) {
            var proxy = this,
                count = 0,
                childGanttRecords = childGanttRecord.childGanttRecords,
                length = childGanttRecords && childGanttRecords.length,
                ganttRecord;
            for (count; count < length; count++) {
                ganttRecord = childGanttRecords[count];
                ganttRecord.level += offset;
                proxy.refreshGanttRecord(ganttRecord);
                if (ganttRecord.childGanttRecords)
                    proxy.updateLevel(ganttRecord, offset);
            }
        },

        addNewGanttRecord: function () {
            var proxy = this, eventArgs = {}, newRecord;
            newRecord = this.model.ganttRecords[this.model.ganttRecords.length].item;
            eventArgs.requestType = "save";
            eventArgs._cAddedRecord = jQuery.extend(true, {}, newRecord);
            eventArgs.recordIndex = index + 1;
            proxy.subContextMenuAction(eventArgs);
            proxy._clearContextMenu();
        },

        subContextMenuAction: function (args) {
            var proxy = this, eventArgs = {}, model = proxy.model, selectionIndex;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            proxy._gridRows = proxy.getGridRows();
            proxy._ganttChartRows = proxy.getGanttChartRows();
            proxy._$gridtreeHelper.ejGanttGridTree("cleanUpSelection");
            if (args.requestType === "save") {

                if (args._cAddedRecord) {

                    args._cAddedRecord.taskId = proxy._getNewTaskId(args._cAddedRecord.taskId);
                    args._cAddedRecord.item.Id = proxy._getNewTaskId(args._cAddedRecord.taskId);
                    args._cAddedRecord.taskName = "New Task" + args._cAddedRecord.taskId;
                    args._cAddedRecord.item.Name = "New Task" + args._cAddedRecord.taskId;
                    args._cAddedRecord.predecessor = null;
                    args._cAddedRecord.item.Predecessor = null;

                    eventArgs.data = args._cAddedRecord.item;
                    eventArgs.requestType = ej.GanttGridTree.actions.Save;
                    eventArgs.recordIndex = args.recordIndex;
                    eventArgs._cAddedRecord = proxy._createGanttRecord(eventArgs.data, args._cAddedRecord.level, null);

                    if (args._cAddedRecord.parentItem) {
                        eventArgs._cAddedRecord.parentItem = args._cAddedRecord.parentItem;
                        eventArgs._cAddedRecord.parentItem.childGanttRecords.push(eventArgs._cAddedRecord);
                    }
                    eventArgs._cAddedRecord.hasChildGanttRecords = false;
                    eventArgs._cAddedRecord.childGanttRecords = null;
                    eventArgs.actionRequestType = ej.GanttGridTree.actions.ContextMenuAdd;
                    proxy._$gridtreeHelper.ejGanttGridTree("processBindings", eventArgs);

                }
            }
            proxy._connectorlineIds = [];
            proxy._connectorLinesCollection = [];
            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
            proxy._createConnectorLinesCollection();
            proxy._ganttActionCompleteTrigger(args);

            if (args.recordIndex != 0)
                selectionIndex = args.recordIndex + 1;
            else
                selectionIndex = args.recordIndex;
            proxy._$gridtreeHelper.ejGanttGridTree("selectRows", selectionIndex);
            proxy._$ganttchartHelper.ejGanttChart("selectRows", selectionIndex);
        },

        _getNewTaskId: function (taskId) {
            var proxy = this, tempTaskId;
            if (proxy._ids.indexOf(taskId.toString()) == -1) {
                proxy._ids.push(taskId);
                return taskId;
            } else {
                taskId++;
                tempTaskId = proxy._getNewTaskId(taskId);
                return tempTaskId;
            }
        },

        contextMenuAction: function (args) {
            var proxy = this;
            if (args.requestType == "contextMenuAdd")
                proxy._sendAddRequest();
            if (args.requestType == "contextMenuIndent")
                proxy._sendOutdentRequest();
            if (args.requestType == "contextMenuOutdent")
                proxy._sendIndentRequest();
            if (args.requestType == "contextMenuDelete")
                proxy._$gridtreeHelper.ejGanttGridTree("sendDeleteRequest");
            if (args.requestType == "contextMenuTaskDetails")
                proxy._sendEditRequest();
        },

        _renderContextMenu: function (e, index, item) {

            var proxy = this,
                eventArgs = {},
                args = {},
                posx,
                posy,
                contextMenu,
                contextMenuUList,
                subContextMenu,
                subContextMenuUList;
            //Initialize the contextmenu item to avoid duplicates
            //proxy._contextMenuItems = proxy._getContextMenuItems();
            //proxy._subContextMenuItems = proxy._getsubContextMenuItems();
            proxy._clearContextMenu();

            if (!e) e = window.event;
            if (e.pageX || e.pageY) {
                posx = e.pageX;
                posy = e.pageY;
            } else if (e.clientX || e.clientY) {
                posx = e.clientX + document.body.scrollLeft
                    + document.documentElement.scrollLeft;
                posy = e.clientY + document.body.scrollTop
                    + document.documentElement.scrollTop;
            }

            args.requestType = "ContextMenuOpen";
            args.contextMenuItems = proxy._contextMenuItems;
            args.innerContextMenuItems = proxy._subContextMenuItems;
            proxy._trigger("contextMenuOpen", args);
            contextMenu = ej.buildTag("div.e-contextmenu", "", {
                'height': 'auto',
                'width': '150px',
                'position': 'absolute',
                'left': posx + 'px',
                'top': posy + 'px',
                'background-color': '#F0F0F0',
                'border': '1px solid #99bce8',
                'z-index': '10',
            }, {});

            contextMenuUList = ej.buildTag("ul", $.render[proxy._id + "contextMenuTemplate"](args.contextMenuItems), {
                'height': 'auto',
                'width': 'auto',
                'top': '0px',
                'display': 'inline'
            }, {
                'data-icon': false,
                'type': 'none',
                'data-role': 'list-divider'
            });

            subContextMenu = ej.buildTag("div.e-innerContextmenu", "", {
                'height': 'auto',
                'width': '120px',
                'position': 'absolute',
                'left': (posx + 152) + 'px',
                'top': (posy + 30) + 'px',
                'background-color': '#F0F0F0',
                'border': '1px solid #99bce8',
                'z-index': '10',
                'visibility': 'hidden',
            }, {});

            subContextMenuUList = ej.buildTag("ul", $.render[proxy._id + "subContextMenuTemplate"](args.innerContextMenuItems), {
                'height': 'auto',
                'width': 'auto',
                'top': '0px',
                'display': 'inline'
            }, {
                'data-icon': false,
                'type': 'none',
                'data-role': 'list-divider'
            });
            contextMenu.append(contextMenuUList);
            $(document.body).append(contextMenu);
            subContextMenu.append(subContextMenuUList);
            $(document.body).append(subContextMenu);



            $(contextMenu).css({ 'height': 'auto' });
            proxy._on(contextMenu, "contextmenu", function (argsE) {
                argsE.preventDefault();
            });

            //click event handlers for contextmenu items and subcontextmenu items

            $('.e-menuitem').click(function () {
                var choice = $(this).attr('id');
                switch (choice) {
                    case "Task":
                        eventArgs.requestType = "contextMenuTaskDetails";
                        proxy.contextMenuAction(eventArgs);
                        proxy._clearContextMenu();
                        break;
                    case "Indent": eventArgs.requestType = "contextMenuIndent";
                        proxy.contextMenuAction(eventArgs);
                        proxy._clearContextMenu();
                        break;
                    case "Outdent": eventArgs.requestType = "contextMenuOutdent";
                        proxy.contextMenuAction(eventArgs);
                        proxy._clearContextMenu();
                        break;
                    case "Delete": eventArgs.requestType = "contextMenuDelete";
                        proxy.contextMenuAction(eventArgs);
                        proxy._clearContextMenu();
                        break;
                    case "Above": eventArgs.requestType = "save";
                        eventArgs._cAddedRecord = jQuery.extend(true, {}, item);
                        eventArgs.recordIndex = index == 0 ? 0 : index - 1;
                        proxy.subContextMenuAction(eventArgs);
                        proxy._clearContextMenu();
                        break;
                    case "Below": eventArgs.requestType = "save";
                        eventArgs._cAddedRecord = jQuery.extend(true, {}, item);
                        eventArgs.recordIndex = index + 1;
                        proxy.subContextMenuAction(eventArgs);
                        proxy._clearContextMenu();
                        break;
                    case "Open":
                        eventArgs.requestType = "contextMenuAdd";
                        proxy.contextMenuAction(eventArgs);
                        proxy._clearContextMenu();
                        break;
                    default:
                        proxy._clearContextMenu();
                        break;
                }
            });
            //Mouseenter and mouse leave handler for contextmenu items and Subcontextmenuitems
            $(".e-menuitem").mouseenter(function () {
                $(this).addClass("e-contextmenu-mouseover");
                if ($(this).attr('id') == "Add" || $(this).attr('id') == "Above" || $(this).attr('id') == "Below" || $(this).attr('id') == "Open") {
                    $('.e-innerContextmenu').css({ 'visibility': 'visible' });
                    $('#Add').addClass("e-contextmenu-mouseover");
                }
                else {
                    $('.e-innerContextmenu').css({ 'visibility': 'hidden' });
                }
            });
            $(".e-menuitem").mouseleave(function () {
                $(this).removeClass("e-contextmenu-mouseover");
            });
            e.preventDefault();
        },

        sendExpandCollapseRequest: function (args) {

            var proxy = this,
                ganttRecord = args.data,
                model = proxy.model,
                index = proxy._currentViewData.indexOf(ganttRecord),
                $row = $($(proxy._gridRows)[index]),
                height = 0,
                isModified = ganttRecord.expanded !== args.expanded;

            if (args.expanded) {
                ganttRecord.expanded = args.expanded;
                proxy._expandedRecordsCount = 0;
                proxy.expandGanttRecord(ganttRecord, args.expanded, args);
                if (isModified)
                    proxy._totalCollapseRecordCount -= proxy._expandedRecordsCount;
                //$row.find(".e-gridtreecollapse").removeClass('e-gridtreecollapse').addClass('e-gridtreeexpand');
                //$row.removeClass('e-gridtreerowcollapse').addClass('e-gridtreerowexpand');                
                if (model.altRow)
                    proxy.updateAltRow(ganttRecord, index, 1);

            }
            else {
                proxy._collapsedRecordCount = 0;
                proxy.collapseGanttRecord(ganttRecord, args.expanded);
                if (isModified)
                    proxy._totalCollapseRecordCount += proxy._collapsedRecordCount;
                ganttRecord.expanded = args.expanded;
                $row.find(".e-gridtreeexpand").removeClass('e-gridtreeexpand').addClass('e-gridtreecollapse');
                $row.removeClass('e-gridtreerowexpand').addClass('e-gridtreerowcollapse');

                if (model.altRow)
                    proxy.updateAltRow(ganttRecord, index, proxy._collapsedRecordCount + 1);
            }
            height = (proxy._updatedGanttRecords.length - proxy._totalCollapseRecordCount) * model.rowHeight;
            proxy._$ganttchart.ejGanttChart("updateHeight", height);
            if (args.expanded)
                proxy._trigger("expanded", args);
            else
                proxy._trigger("collapsed", args);
        },

        expandGanttRecord: function (ganttRecord, expanded, args) {

            var proxy = this,
                rowClassName = ".gridrowtaskId" + ganttRecord.taskId.toString() + "level" + (ganttRecord.level + 1).toString(),
                $rows = $(rowClassName),
                count = 0,
                length = $rows.length,
                row,
                index,
                record,
                $expandedRows = $(rowClassName),
                index = proxy._currentViewData.indexOf(ganttRecord),
                $row = $($(proxy._gridRows)[index]);

            if (ganttRecord.hasChildGanttRecords && ganttRecord.childGanttRecords.length > 0) {
                $row.find(".e-gridtreecollapse").removeClass('e-gridtreecollapse').addClass('e-gridtreeexpand');
                $row.removeClass('e-gridtreerowcollapse').addClass('e-gridtreerowexpand');
            }

            proxy._expandedRecordsCount += $expandedRows.length / 2;
            if ($expandedRows.length === 0) {
                if (proxy.model.allowSearching && proxy._seacrhString.length > 0) {
                    args && proxy._$gridtreeHelper.ejGanttGridTree("sendExpandCollapseRequest", args);
                }
            }

            $expandedRows.css({ 'display': 'table-row' });
            proxy.updateConnectorLines($expandedRows, expanded);
            for (count; count < length / 2; count++) {
                row = $rows[count];
                index = $(proxy._gridRows).index(row);
                if (index !== -1) {
                    record = proxy._updatedGanttRecords[index];
                    record.isExpanded = true;
                    if (record.expanded)
                        proxy.expandGanttRecord(record, expanded);
                }
            }
        },

        updateConnectorLines: function ($gridRows, expanded) {
            var proxy = this,
                length = $gridRows.length / 2,
                count = 0,
                row,
                index,
                record;

            for (count = 0; count < length; count++) {
                row = $gridRows[count];
                index = $(proxy._gridRows).index(row);
                if (index !== -1) {
                    record = proxy._currentViewData[index];
                    record.isExpanded = expanded;
                }
            }
            proxy._connectorlineIds = [];
            proxy._connectorLinesCollection = [];
            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
            proxy._createConnectorLinesCollection();
        },

        collapseGanttRecord: function (ganttRecord, expanded) {
            var proxy = this,
                rowClassName = ".gridrowtaskId" + ganttRecord.taskId.toString() + "level" + (ganttRecord.level + 1).toString(),
                $rows = $(rowClassName),
                count = 0,
                length = $rows.length,
                row,
                index,
                record,
                $collapsedRows = $(rowClassName);
            proxy._collapsedRecordCount += $collapsedRows.length / 2;
            $collapsedRows.css({ 'display': 'none' });

            proxy.updateConnectorLines($collapsedRows, expanded);

            for (count; count < length / 2; count++) {
                row = $rows[count];
                index = $(proxy._gridRows).index(row);
                //if (index === -1) {
                //    index = $(proxy._ganttChartRows).index($rows[count + $collapsedRows.length / 2]);
                //    $(row).addClass('e-hide');
                //}
                if (index !== -1) {
                    record = proxy.getUpdatedRecords()[index];
                    if (record.expanded)
                        proxy.collapseGanttRecord(record, expanded);
                }
            }
        },

        updateAltRow: function (ganttRecord, recordIndex, offset) {
            var proxy = this,
                count = 0,
                currentViewData = proxy._currentViewData,
                length = currentViewData.length,
                isAltRow = ganttRecord.isAltRow,
                record,
                $gridRows = $(proxy._gridRows);
            for (count = recordIndex + offset; count < length; count++) {
                if ($($gridRows[count]).css('display') === "none")
                    continue;
                record = currentViewData[count];
                record.isAltRow = !isAltRow;
                isAltRow = record.isAltRow;
                if (isAltRow)
                    $($gridRows[count]).addClass('e-alt-row');
                else
                    $($gridRows[count]).removeClass('e-alt-row');
            }
        },

        renderedEditDialog: function (args) {
            var proxy = this;
            switch (args.requestType) {
                case ej.GanttGridTree.actions.Add:
                    var temp = document.createElement('div');
                    $(temp).addClass("e-addedrow");
                    temp.innerHTML = $.render[proxy._id + "_JSONDialogEditingTemplate"](args.data);
                    $("#" + proxy._id + "_dialogEdit").html($(temp));
                    var model = {};
                    model.cssClass = this.model.cssClass,
                    model.modal = true,
                    model.width = "auto",
                    model.resizable = false,
                    model.contentContainer = "#" + proxy._id,
                    model.modal = true,
                    model.rtl = proxy.model.rtl,
                    model.allowKeyboardNavigation = false,
                    title = proxy._editDialogTexts["addFormTitle"];
                    model.title = title;
                    $("#" + proxy._id + "_dialogEdit").ejDialog(model);
                    $("#" + proxy._id + "_dialogEdit").ejDialog("open");
                    break;
                case ej.GanttGridTree.actions.BeginEdit:
                    temp = document.createElement('div');
                    $(temp).addClass("e-editedrow");
                    temp.innerHTML = $.render[proxy._id + "_JSONDialogEditingTemplate"](args.data);
                    $("#" + proxy._id + "_dialogEdit").html($(temp));
                    model = {};
                    model.cssClass = proxy.model.cssClass;
                    model.rtl = model.rtl;
                    model.width = "auto";
                    model.resizable = false;
                    model.contentContainer = "#" + proxy._id;
                    model.modal = true;
                    model.allowKeyboardNavigation = false;
                    var title = proxy._editDialogTexts["editFormTitle"] + " " + proxy.selectedItem()["taskName"];
                    model.title = title; //args.Data[proxy._primaryKeys[0]];
                    $("#" + proxy._id + "_dialogEdit").ejDialog(model);
                    $("#" + proxy._id + "_dialogEdit").ejDialog("open");
                    break;
            }
            proxy._refreshEditForm(args);
            //proxy.setValidation();
        },

        editFormValidate: function () {
            if ($.isFunction($.validator))
                return $("#" + this._id + "EditForm").validate().form();
            return true;
        },

        getCurrentData: function () {
            var proxy = this,
                currentViewData = proxy.getCurrentViewData();
            return currentViewData && currentViewData[proxy.model.selectedRowIndex];
        },

        getCurrentViewData: function () {
            return this._$gridtreeHelper.ejGanttGridTree("getCurrentViewData");
        },

        getGridRows: function () {
            return this._$gridtreeHelper.ejGanttGridTree("getGridRows");
        },

        getGanttChartRows: function () {
            return this._$ganttchartHelper.ejGanttChart("getGanttChartRows");
        },

        getUpdatedRecords: function () {
            return this._$gridtreeHelper.ejGanttGridTree("getUpdatedRecords");
        },

        _getToolBarHeight: function () {
            var toolbarH = 0;
            if (this.model.toolBar.allowToolBar) {
                toolbarH = $(".e-gantttoolbar").height();
            }
            return toolbarH;
        },

        _initValidator: function () {
            var gridObject = this;
            $("#" + this._id + "EditForm").validate({
                errorClass: 'e-field-validation-error',
                errorElement: 'div',
                wrapper: "div",
                errorPlacement: function (error, element) {
                    var $td = element.closest("td"), $container = $(error).addClass("e-error")
                    , $tail = ej.buildTag("div.e-errortail e-toparrow");
                    $td.find(".e-error").remove();
                    element.hasClass("e-numerictextbox") ? $container.insertAfter(element.closest(".e-numeric")) : $container.insertAfter(element);
                    $container.prepend($tail);
                    gridObject.model.edit.editMode != "normal" && $container.offset({ left: element.offset().left });
                    $container.fadeIn("slow");
                },

            });
        },

        setValidation: function () {
            this._initValidator();
            for (var i = 0; i < this._columns.length; i++) {
                if (!ej.isNullOrUndefined(this._columns[i]["validationRules"])) {
                    this.setValidationToField(this._columns[i].field, this._columns[i].validationRules);
                }
            }
        },

        setValidationToField: function (name, rules) {
            var ele = $("#" + this._id + name).length > 0 ? $("#" + this._id + name) : $("#" + name);
            ele.rules("add", rules);
            var validator = $("#" + this._id + "EditForm").validate();
            if (!ej.isNullOrUndefined(rules["required"])) {
                validator.settings.messages["Name"] = {};
                validator.settings.messages["Name"]["required"] = this._$gridtreeHelper.ejGanttGridTree("getColumnByField", name).headerText + " is required";
            }
        },

        _createConnectorLinesCollection: function () {
            var proxy = this,
                recordLength = proxy._currentViewData.length,
                count,
                prdecessorIndex = -1,
                predecessorsCollection,
                ganttRecord,
                connectorLinesCollection = proxy._predecessorsCollection,
                connectorLinesLength = connectorLinesCollection.length,
                x = connectorLinesLength / 2,
                connectorLinesCount,
                enableVirtualization = proxy.model.enableVirtualization,
                record;
            for (count = 0; count < recordLength; count++) {
                ganttRecord = proxy._currentViewData[count];
                if (enableVirtualization === false && ganttRecord.isExpanded === false)
                    continue;
                if (proxy._currentViewData.indexOf(ganttRecord) !== -1) {
                    predecessorsCollection = ganttRecord["predecessor"];
                    if (predecessorsCollection) {
                        record = ganttRecord;
                        proxy._addPredecessorsCollection(ganttRecord, predecessorsCollection);
                    }
                }
            }


            prdecessorIndex = record ? connectorLinesCollection.indexOf(record) : -1;

            if (prdecessorIndex === -1) {

                for (connectorLinesCount = 0; connectorLinesCount < connectorLinesLength; connectorLinesCount++) {

                    ganttRecord = connectorLinesCollection[connectorLinesCount];

                    if (enableVirtualization === false && ganttRecord.isExpanded === false)
                        continue;

                    predecessorsCollection = ganttRecord["predecessor"];

                    if (predecessorsCollection)
                        proxy._addPredecessorsCollection(ganttRecord, predecessorsCollection);

                }
            }

            else if (prdecessorIndex >= 0 && prdecessorIndex <= x) {

                for (connectorLinesCount = 0; connectorLinesCount < prdecessorIndex; connectorLinesCount++) {

                    ganttRecord = connectorLinesCollection[connectorLinesCount];

                    if (enableVirtualization === false && ganttRecord.isExpanded === false)
                        continue;

                    predecessorsCollection = ganttRecord["predecessor"];

                    if (predecessorsCollection)
                        proxy._addPredecessorsCollection(ganttRecord, predecessorsCollection);

                }
            } else {

                for (connectorLinesCount = prdecessorIndex; connectorLinesCount < connectorLinesLength; connectorLinesCount++) {

                    ganttRecord = connectorLinesCollection[connectorLinesCount];

                    if (enableVirtualization === false && ganttRecord.isExpanded === false)
                        continue;

                    predecessorsCollection = ganttRecord["predecessor"];

                    if (predecessorsCollection)
                        proxy._addPredecessorsCollection(ganttRecord, predecessorsCollection);

                }
            }

            if (proxy._connectorLinesCollection.length > 0) {

                proxy._$ganttchartHelper && proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines", proxy._connectorLinesCollection);

            }
        },

        _addPredecessorsCollection: function (ganttRecord, predecessorsCollection) {

            var proxy = this,
                predecessorsLength,
                predecessorCount,
                predecessor,
                parentGanttRecord,
                childGanttRecord,
                model = proxy.model,
                connectorObj = {},
                predecessorValue = [];


            if (predecessorsCollection) {

                predecessorsLength = predecessorsCollection.length;

                for (predecessorCount = 0; predecessorCount < predecessorsLength; predecessorCount++) {

                    predecessor = predecessorsCollection[predecessorCount];

                    if (predecessor.isdrawn) {
                        predecessor.isdrawn = false;
                    }
                    else {

                        parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                        childGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.to)];

                        if (proxy._updatedGanttRecords.indexOf(parentGanttRecord) === -1 || proxy._updatedGanttRecords.indexOf(childGanttRecord) == -1)
                            continue;

                        if (!model.enableVirtualization) {
                            if (parentGanttRecord.isExpanded === false || childGanttRecord.isExpanded === false)
                                continue;
                        }

                        if (proxy._isValidationEnabled && !proxy._isInExpandCollapse)
                            proxy._validateChildGanttRecord(parentGanttRecord, childGanttRecord, predecessor);

                        if (parentGanttRecord && childGanttRecord) {

                            connectorObj = proxy._createConnectorLineObject(parentGanttRecord, childGanttRecord, predecessor);

                            if (connectorObj) {

                                if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId) == -1) {

                                    proxy._connectorLinesCollection.push(connectorObj);
                                    proxy._connectorlineIds.push(connectorObj.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.length == 0) {
                                    proxy._connectorLinesCollection.push(connectorObj);
                                    proxy._connectorlineIds.push(connectorObj.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId) != -1) {
                                    var index = proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId);
                                    proxy._connectorLinesCollection[index] = connectorObj;
                                }
                                predecessor.isdrawn = true;
                            }
                        }
                    }
                }

            }

        },

        _createConnectorLineObject: function (parentGanttRecord, childGanttRecord, predecessor) {
            var connectorObj = {},
                proxy = this,
                model = proxy.model,
                parentTop,
                childTop,
                parentIndex = proxy._updatedGanttRecords.indexOf(parentGanttRecord),
                childIndex = proxy._updatedGanttRecords.indexOf(childGanttRecord),
                count = 0,
                index = 0;

            if (parentIndex === -1 || childIndex === -1) {
                return;
            }
            if (!model.enableVirtualization) {
                var collapsedCount = 0;
                if (childIndex > parentIndex) {

                    for (count = 0; count < parentIndex; count++) {
                        if (proxy._updatedGanttRecords[count].isExpanded === false)
                            ++collapsedCount;
                    }
                    index = parentIndex;
                    parentIndex -= collapsedCount;

                    for (index; index < childIndex; index++) {
                        if (proxy._updatedGanttRecords[index].isExpanded === false)
                            ++collapsedCount;
                    }
                    childIndex -= collapsedCount;
                }
                else if (parentIndex > childIndex) {
                    for (count = 0; count < childIndex; count++) {
                        if (proxy._updatedGanttRecords[count].isExpanded === false)
                            ++collapsedCount;
                    }
                    index = childIndex;
                    childIndex -= collapsedCount;

                    for (index ; index < parentIndex; index++) {
                        if (proxy._updatedGanttRecords[index].isExpanded === false)
                            ++collapsedCount;
                    }
                    parentIndex -= collapsedCount;
                }
            }

            if (parentIndex != -1 && childIndex != -1) {
                parentTop = parentIndex * model.rowHeight;
                childTop = childIndex * model.rowHeight;
                connectorObj["ParentLeft"] = parentGanttRecord['left'];
                connectorObj["ChildLeft"] = childGanttRecord['left'];
                connectorObj["ParentWidth"] = parentGanttRecord['width'] == 0 ? 26 : parentGanttRecord['width'];
                connectorObj["ChildWidth"] = childGanttRecord['width'] == 0 ? 26 : childGanttRecord['width'];
                connectorObj["ParentTop"] = parentTop;
                connectorObj["ChildTop"] = childTop;
                connectorObj["Height"] = parentTop > childTop ? parentTop - childTop : childTop - parentTop;
                connectorObj["Type"] = predecessor.predecessorsType;
                connectorObj["ConnectorLineId"] = "parent" + parentGanttRecord['taskId'] + "child" + childGanttRecord['taskId'];
                connectorObj["milestoneParent"] = parentGanttRecord.isMilestone ? true : false;
                return connectorObj;
            }
        },

        //Validate the child Gantt Record

        _validateChildGanttRecord: function (parentGanttRecord, childGanttRecord, predecessor) {

            var proxy = this,
                model = proxy.model,
                childStartDate,
                parentStartDate,
                childEndDate,
                parentEndDate,
                offset,
                predecessorOffset = 0;
            predecessorOffset = parseInt(predecessor.offset);

            switch (predecessor.predecessorsType) {

                case 'SS':

                    parentStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                    childStartDate = proxy._getDateFromFormat(childGanttRecord.startDate);

                    if (childStartDate.getTime() !== parentStartDate.getTime()) {

                        if (childGanttRecord.isMilestone) {
                            parentStartDate.setDate(parentStartDate.getDate() - 1);
                        }

                        if (proxy.model.includeWeekend === false) {
                            proxy._setBeforeWeekend(parentStartDate);
                        }

                        if (predecessorOffset) {

                            parentStartDate.setDate(parentStartDate.getDate() + (Math.floor(predecessorOffset)));

                            if (proxy.model.includeWeekend === false) {

                                if (predecessorOffset < 0) {
                                    proxy._setBeforeWeekend(parentStartDate);
                                }

                                if (predecessorOffset > 0) {
                                    proxy._setAfterWeekend(parentStartDate);
                                }
                            }
                        }

                        if (proxy._stringHolidays &&
                          proxy._stringHolidays.indexOf(childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat)) != -1) {
                            parentStartDate.setDate(parentStartDate.getDate() + 1);
                        }

                        childGanttRecord.startDate = childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat);
                        //if (!predecessorOffset) {
                        var currentTaskId = childGanttRecord['taskId'].toString(),
                             predecessorsCollection = childGanttRecord['predecessor'],
                           childPredecessor = predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                        if (childPredecessor.length > 1) {

                            var minStartDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);


                            if (childGanttRecord.isMilestone) {
                                minStartDate.setDate(minStartDate.getDate() - 1);
                            }

                            if (model.includeWeekend === false) {
                                proxy._setBeforeWeekend(minStartDate);
                            }

                            if ((proxy._getDateFromFormat(childGanttRecord.startDate)).getTime() !== minStartDate.getTime()) {

                                var dateDiff = 0;

                                dateDiff = proxy._daydiff(proxy._getDateFromFormat(childGanttRecord.startDate), minStartDate);


                                var enddate = proxy._getDateFromFormat(minStartDate),
                                 weekends = proxy._calculateWeekendCount(enddate, dateDiff);

                                predecessor.offset = (dateDiff - weekends).toString();

                            }

                            childGanttRecord.startDate = childGanttRecord._getFormatedDate(minStartDate, model.dateFormat);

                        }
                        //}

                        childGanttRecord.left = childGanttRecord._calculateLeft(proxy._scheduleWeeks[0],
                                    proxy._perDayWidth, model.includeWeekend, model.dateFormat, model.endDateMapping, proxy._holidays);
                        childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, model.includeWeekend, model.holidays, model.dateFormat);
                    }
                    break;

                case 'SF':

                    parentStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                    childEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);

                    if ((proxy.dateUpdate(childEndDate, 1)).getTime() !== parentStartDate.getTime()) {

                        parentStartDate.setDate(parentStartDate.getDate() - 1);

                        if (proxy.model.includeWeekend === false) {
                            proxy._setBeforeWeekend(parentStartDate);
                        }

                        if (predecessorOffset) {

                            parentStartDate.setDate(parentStartDate.getDate() + (Math.floor(predecessorOffset)));

                            if (proxy.model.includeWeekend === false) {

                                if (predecessorOffset < 0) {
                                    proxy._setBeforeWeekend(parentStartDate);
                                }

                                if (predecessorOffset > 0) {
                                    proxy._setAfterWeekend(parentStartDate);
                                }
                            }
                        }


                        if (proxy._stringHolidays &&
                            proxy._stringHolidays.indexOf(childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat)) != -1) {
                            parentStartDate.setDate(parentStartDate.getDate() + 1);
                        }

                        childGanttRecord.endDate = childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat);

                        //if (!predecessorOffset) {
                        var currentTaskId = childGanttRecord['taskId'].toString(),
                                   predecessorsCollection = childGanttRecord['predecessor'],
                                 childPredecessor = predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                        if (childPredecessor.length > 0) {

                            var minStartDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);

                            minStartDate.setDate(minStartDate.getDate() - 1);

                            if (proxy.model.includeWeekend === false) {
                                proxy._setBeforeWeekend(minStartDate);
                            }

                            if ((proxy._getDateFromFormat(childGanttRecord.endDate)).getTime() !== minStartDate.getTime()) {

                                var dateDiff = 0;

                                dateDiff = proxy._daydiff(proxy._getDateFromFormat(childGanttRecord.endDate), minStartDate);

                                var enddate = proxy._getDateFromFormat(minStartDate),
                                 weekends = proxy._calculateWeekendCount(enddate, dateDiff);

                                predecessor.offset = (dateDiff - weekends).toString();

                            }

                            childGanttRecord.endDate = childGanttRecord._getFormatedDate(minStartDate, model.dateFormat);
                        }
                        //}
                        childGanttRecord.left = childGanttRecord._calculateLeft(proxy._scheduleWeeks[0],
                                    proxy._perDayWidth, model.includeWeekend, model.dateFormat, model.endDateMapping, proxy._holidays);
                        childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, model.includeWeekend, model.holidays, model.dateFormat);

                    }
                    break;

                case 'FS':

                    parentEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                    childStartDate = proxy._getDateFromFormat(childGanttRecord.startDate);

                    if ((proxy.dateUpdate(parentEndDate, 1)).getTime() !== childStartDate.getTime()) {

                        if (!childGanttRecord.isMilestone) {
                            parentEndDate.setDate(parentEndDate.getDate() + 1);
                        }

                        if (predecessorOffset < 0) {
                            predecessorOffset *= -1;
                            predecessorOffset -= 1;
                            offset = -1;
                            if (model.includeWeekend === false) {
                                proxy._setBeforeWeekend(parentEndDate);
                            }
                        } else {

                            if (model.includeWeekend === false) {
                                proxy._setAfterWeekend(parentEndDate);
                            }
                            offset = 1;
                        }


                        if (predecessorOffset) {

                            for (var i = 0; i < predecessorOffset; i++) {

                                if (proxy.model.includeWeekend === false) {
                                    if (offset < 0) {
                                        proxy._setBeforeWeekend(parentEndDate);
                                    }

                                    if (offset > 0) {
                                        proxy._setAfterWeekend(parentEndDate);
                                    }
                                }
                                parentEndDate.setDate(parentEndDate.getDate() + offset);
                            }

                            if (proxy.model.includeWeekend === false) {
                                proxy._setAfterWeekend(parentEndDate);
                            }
                        }

                        if (proxy._stringHolidays &&
                            proxy._stringHolidays.indexOf(childGanttRecord._getFormatedDate(parentEndDate, model.dateFormat)) != -1) {
                            parentEndDate.setDate(parentEndDate.getDate() + 1);
                        }

                        childGanttRecord.startDate = childGanttRecord._getFormatedDate(parentEndDate, model.dateFormat);

                        //if (!predecessorOffset) {
                        var predecessorsCollection = childGanttRecord['predecessor'],
                            currentTaskId = childGanttRecord['taskId'].toString(),
                            childPredecessor = predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                        if (childPredecessor.length > 0) {

                            var maxEndDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);

                            if (!childGanttRecord.isMilestone) {
                                maxEndDate.setDate(maxEndDate.getDate() + 1);
                            }

                            if (model.includeWeekend === false) {
                                proxy._setAfterWeekend(maxEndDate);
                            }

                            if ((proxy._getDateFromFormat(childGanttRecord.startDate)).getTime() !== maxEndDate.getTime()) {

                                var dateDiff = 0;

                                dateDiff = proxy._daydiff(proxy._getDateFromFormat(childGanttRecord.startDate), maxEndDate);


                                var enddate = proxy._getDateFromFormat(maxEndDate),
                                 weekends = proxy._calculateWeekendCount(enddate, dateDiff);

                                predecessor.offset = (dateDiff - weekends).toString();

                            }

                            childGanttRecord.startDate = childGanttRecord._getFormatedDate(maxEndDate, model.dateFormat);
                        }
                        //}

                        childGanttRecord.left = childGanttRecord._calculateLeft(proxy._scheduleWeeks[0], proxy._perDayWidth,
                            model.includeWeekend, model.dateFormat, model.endDateMapping);
                        childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, model.includeWeekend, model.holidays, model.dateFormat);
                    }

                    break;

                case 'FF':
                    parentEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                    childEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);

                    if (parentEndDate.getTime() !== childEndDate.getTime()) {
                        if (proxy.model.includeWeekend === false) {
                            proxy._setAfterWeekend(parentEndDate);
                        }

                        if (predecessorOffset) {
                            parentEndDate.setDate(parentEndDate.getDate() + (Math.floor(predecessorOffset)));

                            if (proxy.model.includeWeekend === false) {
                                if (predecessorOffset < 0) {
                                    proxy._setBeforeWeekend(parentEndDate);
                                }

                                if (predecessorOffset > 0) {
                                    proxy._setAfterWeekend(parentEndDate);
                                }
                            }
                        }

                        if (proxy._stringHolidays &&
                           proxy._stringHolidays.indexOf(childGanttRecord._getFormatedDate(parentEndDate, model.dateFormat)) != -1) {
                            parentEndDate.setDate(parentEndDate.getDate() + 1);
                        }

                        childGanttRecord.endDate = childGanttRecord._getFormatedDate(maxEndDate, model.dateFormat);
                        //if (!predecessorOffset) {
                        var currentTaskId = childGanttRecord['taskId'].toString(),
                                    predecessorsCollection = childGanttRecord['predecessor'],
                                  childPredecessor = predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                        if (childPredecessor.length > 1) {

                            var maxEndDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);

                            if (model.includeWeekend === false) {
                                proxy._setAfterWeekend(maxEndDate);
                            }

                            if ((proxy._getDateFromFormat(childGanttRecord.endDate)).getTime() !== maxEndDate.getTime()) {

                                var dateDiff = 0;

                                dateDiff = proxy._daydiff(proxy._getDateFromFormat(childGanttRecord.endDate), maxEndDate);


                                var enddate = proxy._getDateFromFormat(maxEndDate),
                                 weekends = proxy._calculateWeekendCount(enddate, dateDiff);

                                predecessor.offset = (dateDiff - weekends).toString();

                            }

                            childGanttRecord.endDate = childGanttRecord._getFormatedDate(maxEndDate, model.dateFormat);
                        }
                        //}
                        childGanttRecord.left = childGanttRecord._calculateLeft(proxy._scheduleWeeks[0], proxy._perDayWidth, model.includeWeekend, model.dateFormat, model.endDateMapping);
                        childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, model.includeWeekend, model.holidays, model.dateFormat);
                    }
                    break;
            }

            if (childGanttRecord.parentItem)
                proxy._updateParentItem(childGanttRecord);

            if (proxy._isRenddered) {
                proxy.refreshGanttRecord(childGanttRecord);
            }

        },

        _getPredecessorDate: function (ganttRecord, predecessorsCollection, predecessorType) {
            var minStartDate = null, proxy = this,
                    maxEndDate = null,
                    length,
                    i,
                    dateDiff = 0,
                    predecessorOffset,
                    connectorLineObject,
                    offset,
                    validatedpredecessor = predecessorsCollection.filter(function (data) {
                        return data.to === ganttRecord.taskId.toString();
                    });


            if (validatedpredecessor) {

                length = validatedpredecessor.length;



                for (i = 0; i < length; i++) {

                    predecessor = validatedpredecessor[i];

                    parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                    childGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.to)];

                    if (minStartDate === null) {
                        minStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                    }

                    if (maxEndDate === null) {
                        maxEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                    }

                    if (proxy._getDateFromFormat(parentGanttRecord.startDate).getTime() < minStartDate.getTime()) {
                        minStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                    }

                    if (proxy._getDateFromFormat(parentGanttRecord.endDate).getTime() > maxEndDate.getTime()) {
                        maxEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                    }

                }
            }
            if (predecessorType == "FS" || predecessorType == "FF") {
                return maxEndDate;
            }
            else if (predecessorType == "SF" || predecessorType == "SS") {
                return minStartDate;
            }

        },

        _renderPredecessor: function (predecessor, item) {
            //var proxy = this,
            //model = proxy.model,
            //length = predecessor.length, predecessorValues = [];

            //for (var i = 0; i < length; i++) {
            //    if (predecessor[i].offset < 0) {
            //        predecessorValues.push(predecessor[i].from + "" + predecessor[i].predecessorsType + ""
            //            + predecessor[i].offset + "d");
            //    }
            //    else if (predecessor[i].offset >= 1) {
            //        predecessorValues.push(predecessor[i].from + "" + predecessor[i].predecessorsType + "+"
            //            + predecessor[i].offset + "d");
            //    }
            //    else {
            //        predecessorValues.push(predecessor[i].from);
            //    }
            //}
            //if (predecessorValues.length > 0) {
            //    item.item[model.predecessorMapping] = predecessorValues.join(',');
            //    //this.refreshRow(itemIndex, false);
            //}
        },

        _setBeforeWeekend: function (date) {
            if (date.getDay() == 0)
                date.setDate(date.getDate() - 2);
            else if (date.getDay() == 6)
                date.setDate(date.getDate() - 1);
        },

        _setAfterWeekend: function (date) {
            if (date.getDay() == 0)
                date.setDate(date.getDate() + 1);
            else if (date.getDay() == 6)
                date.setDate(date.getDate() + 2);
        },

        _updateEditedGanttRecords: function (args) {
            var proxy = this, model = proxy.model,
                ganttRecord = args.data,
                startDate = proxy._getDateFromFormat(ganttRecord.startDate),
                isPredecessorsModified = false;

            if (args.columnName === "startDate") {

                if (this._stringHolidays &&
                      this._stringHolidays.indexOf(ganttRecord._getFormatedDate(startDate, model.dateFormat)) != -1) {
                    startDate.setDate(startDate.getDate() + 1);
                }

                if (proxy.model.includeWeekend === false) {
                    this._setAfterWeekend(startDate);
                    if (this._stringHolidays &&
                      this._stringHolidays.indexOf(ganttRecord._getFormatedDate(startDate, model.dateFormat)) != -1) {
                        startDate.setDate(startDate.getDate() + 1);
                    }
                }
                ganttRecord.startDate = proxy.getFormatedDate(startDate);
                ganttRecord.left = ganttRecord._calculateGanttRecordLeft(startDate, proxy._scheduleWeeks[0], proxy._perDayWidth);
                ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, model.includeWeekend, model.holidays, model.dateFormat);
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord);
            }

            else if (args.columnName === "endDate") {
                var endDate = proxy._getDateFromFormat(ganttRecord.endDate);

                //if (model.holidays && model.holidays.length) {
                //    holidayIndex = 0;
                //    holidayslength = model.holidays.length;
                //    holiday = null;
                //    for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                //        holiday = new Date(model.holidays[holidayIndex]);
                //        if (endDate === holiday) {
                //            endDate.setDate(endDate.getDate() - 1);
                //        }
                //    }
                //}

                if (this._stringHolidays &&
                      this._stringHolidays.indexOf(ganttRecord._getFormatedDate(endDate, model.dateFormat)) != -1) {
                    endDate.setDate(endDate.getDate() + 1);
                    if (endDate.getDay() == 6) {
                        endDate.setDate(endDate.getDate() + 2);
                    }
                }

                if (model.includeWeekend === false) {
                    if (endDate.getDay() == 0) {
                        endDate.setDate(endDate.getDate() - 2);
                    } else if (endDate.getDay() == 6) {
                        endDate.setDate(endDate.getDate() - 1);
                    }
                }
                ganttRecord.endDate = ganttRecord._calculateFormatedDate(endDate, model.dateFormat);
                ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays);
                ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model.dateFormat);
            }

            else if (args.columnName == "duration") {
                var currentDuration = ganttRecord.duration,
                    offset = currentDuration - args.previousValue,
                modifiedEndDate = proxy._calculateUpdatedDate(proxy._getDateFromFormat(ganttRecord.endDate), offset);

                if (currentDuration === 0) {
                    ganttRecord.isMilestone = true;
                } else {
                    ganttRecord.isMilestone = false;
                }

                ganttRecord.endDate = modifiedEndDate;
                ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, proxy.model.includeWeekend, proxy.model.holidays, model.dateFormat);
                ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, ganttRecord.status);
                ganttRecord.item[model.endDateMapping] = modifiedEndDate;
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord);
            }

            else if (args.columnName == "predecessor") {
                isPredecessorsModified = true;
                if (args.previousValue) {
                    proxy._removeConnectorLine(args.previousValue, ganttRecord);
                }
                proxy._addConnectorLine(ganttRecord);
            }

            else if (args.columnName == "status") {
                ganttRecord.status = ganttRecord.status > 100 ? 100 : ganttRecord.status;
                ganttRecord.progressWidth = proxy._getProgressWidth(ganttRecord.width, ganttRecord.status);
                if (ganttRecord.parentItem) {
                    ganttRecord._updateParentProgress(ganttRecord.parentItem);
                    proxy._refreshParent(ganttRecord);
                    //proxy.refreshGanttRecord(ganttRecord.parentItem);
                }
            }

            //proxy._updateEditedGanttRecord(ganttRecord);
            if (ganttRecord.predecessor) {
                if (!isPredecessorsModified)
                    proxy._$ganttchartHelper.ejGanttChart("updateConnectorLineOffset", ganttRecord);
                proxy._isMileStoneEdited = ganttRecord.isMilestone;
                proxy._updatedConnectorLineCollection = [];
                proxy._connectorlineIds = [];
                proxy._validatePredecessor(ganttRecord);
                if (proxy._updatedConnectorLineCollection.length > 0) {
                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                }
            }
            proxy._isValidationEnabled = false;

            proxy.refreshGanttRecord(ganttRecord);
        },

        _removeConnectorLine: function (predecessor, ganttRecord) {
            var proxy = this,
                connectorLineId,
                parentGanttRecord,
                record,
                i = 0,
                length = predecessor && predecessor.length,
                index = -1;

            for (i = 0; i < length; i++) {
                parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor[i].from)];
                record = proxy._rows[proxy._ids.indexOf(predecessor[i].to)];

                if (ganttRecord.taskId.toString() === predecessor[i].to.toString()) {

                    index = parentGanttRecord.predecessor.indexOf(predecessor[i]);
                    index >= 0 && parentGanttRecord.predecessor.splice(index, 1);
                }

                if (parentGanttRecord && record) {
                    connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                    proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                }
            }
        },

        _addConnectorLine: function (ganttRecord) {
            var proxy = this,
                count = 0,
                predecessors = ganttRecord.predecessor,
                length,
                predecessor,
                successorGanttRecord;

            if (predecessors) {

                length = predecessors.length;

                for (count = 0; count < length; count++) {

                    predecessor = predecessors[count];

                    if (predecessor.to === ganttRecord.taskId.toString()) {
                        successorGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                    }

                    if (successorGanttRecord) {
                        if (successorGanttRecord.predecessor) {
                            successorGanttRecord.predecessor.push(predecessor);
                        } else {
                            successorGanttRecord.predecessor = [];
                            successorGanttRecord.predecessor.push(predecessor);
                            proxy._predecessorsCollection.push(successorGanttRecord);
                        }
                    }
                }
            }
        },

        _calculateUpdatedDate: function (modifiedDate, offset) {
            var proxy = this,
                i = 0;
            if (offset > 0) {
                for (i; i < offset; i++) {
                    modifiedDate = proxy.dateUpdate(modifiedDate, 1);
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() + 1);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() + 2);
                }
            } else {
                var dateOffset = (-1) * offset;
                for (i = 0; i < dateOffset; i++) {
                    modifiedDate = proxy.dateUpdate(modifiedDate, -1);
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() - 2);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() - 1);
                }
            }
            modifiedDate = proxy.getFormatedDate(modifiedDate);
            return modifiedDate;
        },

        _updateEditedGanttRecord: function (ganttRecord) {
            var proxy = this;
            if (ganttRecord.predecessor) {
                var predecessorsCollection = ganttRecord.predecessor,
                    count = 0,
                    length = predecessorsCollection.length,
                    predecessor,
                    parentGanttRecord,
                    record,
                    connectorLineId,
                    connectorLineObject;
                for (count = 0; count < length; count++) {
                    proxy._updatedConnectorLineCollection = [];
                    proxy._connectorlineIds = [];
                    predecessor = predecessorsCollection[count];
                    if (predecessor.to === ganttRecord['taskId'].toString()) {

                        parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                        record = proxy._rows[proxy._ids.indexOf(predecessor.to)];

                        if (parentGanttRecord && record) {
                            connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                            proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                            connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                            if (connectorLineObject) {
                                if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                                    proxy._updatedConnectorLineCollection.push(connectorLineObject);
                                    proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.length == 0) {
                                    proxy._updatedConnectorLineCollection.push(connectorLineObject);
                                    proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                                }
                                proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                            }
                        }

                    }
                }
            }
        },


        _updateGanttRecord: function (ganttRecord) {
            var proxy = this,
                model = proxy.model;
            ganttRecord.left = ganttRecord._calculateLeft(proxy._scheduleWeeks[0], proxy._perDayWidth, model.includeWeekend, model.dateFormat, model.endDateMapping, model.holidays);
            ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, model.includeWeekend, model.holidays, model.dateFormat);
            proxy.refreshGanttRecord(ganttRecord);
            if (ganttRecord.parentItem)
                proxy._updateParentItem(ganttRecord);
        },

        refreshGanttRecord: function (ganttRecord) {
            var proxy = this;
            proxy._currentViewData = proxy.getCurrentViewData();
            var index = proxy._currentViewData.indexOf(ganttRecord);
            proxy._$ganttchartHelper.ejGanttChart("refreshRow", index);
            proxy._$gridtreeHelper.ejGanttGridTree("refreshRow", index);

        },

        _updateParentItem: function (ganttRecord) {

            var proxy = this,
                parentGanttRecord = ganttRecord.parentItem,
                model = proxy.model,
                prevStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate),
                currentStartDate = proxy._getDateFromFormat(ganttRecord.startDate),
                prevEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate),
                currentEndDate = proxy._getDateFromFormat(ganttRecord.endDate);


            if (currentStartDate.getTime() < prevStartDate.getTime()) {
                parentGanttRecord.startDate = ganttRecord.startDate;
                parentGanttRecord.item[model.startDateMapping] = parentGanttRecord.startDate;
            }

            if (currentEndDate.getTime() > prevEndDate.getTime()) {
                dateDiff = proxy._daydiff(prevEndDate, currentEndDate);
                parentGanttRecord.endDate = ganttRecord.endDate;
                parentGanttRecord.item[model.endDateMapping] = parentGanttRecord.endDate;
            }

            var count = 0,
                childGanttRecords = parentGanttRecord.childGanttRecords,
                length = childGanttRecords && childGanttRecords.length,
                childGanttRecord,
                minStartDate = null,
                maxEndDate = null,
                parentProgress = 0,
                milestoneCount = 0;

            for (count = 0; count < length; count++) {

                childGanttRecord = childGanttRecords[count];

                if (minStartDate === null) {
                    minStartDate = proxy._getDateFromFormat(childGanttRecord.startDate);
                }
                if (maxEndDate === null) {
                    maxEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);
                }
                if (proxy._getDateFromFormat(childGanttRecord.endDate).getTime() > maxEndDate.getTime()) {
                    maxEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);
                }
                if (proxy._getDateFromFormat(childGanttRecord.startDate).getTime() < minStartDate.getTime()) {
                    minStartDate = proxy._getDateFromFormat(childGanttRecord.startDate);
                }

                if (!childGanttRecord.isMilestone) {
                    parentProgress += Math.ceil(childGanttRecord.status);
                }
                else
                    milestoneCount++;
            }

            parentProgress = parentProgress / (length - milestoneCount);
            parentGanttRecord.progressWidth = proxy._getProgressWidth(parentGanttRecord.width, parentProgress);
            parentGanttRecord.status = Math.ceil(parentProgress);

            if (prevStartDate.getTime() !== minStartDate.getTime()) {
                parentGanttRecord.startDate = parentGanttRecord._getFormatedDate(minStartDate, model.dateFormat);
                parentGanttRecord.item[model.startDateMapping] = parentGanttRecord.startDate;
            }

            if (prevEndDate.getTime() != maxEndDate.getTime()) {
                parentGanttRecord.endDate = parentGanttRecord._getFormatedDate(maxEndDate, model.dateFormat);
                parentGanttRecord.item[model.endDateMapping] = parentGanttRecord.endDate;
            }
            parentGanttRecord._calculateDuration(proxy._getDateFromFormat(parentGanttRecord.startDate), proxy._getDateFromFormat(parentGanttRecord.endDate), model.includeWeekend, model.holidays, parentGanttRecord[model.mileStoneMapping]);

            var diffDate = Math.ceil((proxy._getDateFromFormat(parentGanttRecord.startDate) - proxy._scheduleWeeks[0]) / 86400000);
            parentGanttRecord.left = (diffDate * proxy._perDayWidth);
            parentGanttRecord.width = parentGanttRecord._calculateWidth(proxy._perDayWidth, model.includeWeekend, model.holidays, model.dateFormat);



            if (proxy._isRenddered)
                proxy.refreshGanttRecord(parentGanttRecord);

            if (parentGanttRecord.parentItem) {
                proxy._updateParentItem(parentGanttRecord);
            }


        },

        _validatePredecessor: function (childGanttRecord) {

            var proxy = this;

            if (childGanttRecord.predecessor) {

                var predecessorsCollection = childGanttRecord['predecessor'],
                    count,
                    parentGanttRecord,
                    record = null,
                    connectorLineId,
                    isValidated = false,
                    predecessor,
                    connectorLineObject,
                    lowestOffset;

                var currentTaskId = childGanttRecord['taskId'].toString(),

                    predecessors = predecessorsCollection.filter(function (data) { return data.to === currentTaskId }),
                    successors = predecessorsCollection.filter(function (data) { return data.from === currentTaskId });
                var predecessorLength = predecessors.length;

                if (predecessors.length > 1) {
                    for (count = 0; count < predecessors.length; count++) {
                        var tempOffset = parseInt(predecessors[count].offset.toString().replace(/[^0-9]/g, ''));
                        if (tempOffset < lowestOffset || ej.isNullOrUndefined(lowestOffset)) {
                            lowestOffset = tempOffset;
                        }
                    }

                    for (count = 0; count < predecessors.length; count++) {
                        predecessors[count].offset = predecessors[count].offset.toString().replace(/\d/g, lowestOffset);
                    }
                }

                for (count = 0; count < predecessorLength; count++) {
                    predecessor = predecessors[count];
                    parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                    record = proxy._rows[proxy._ids.indexOf(predecessor.to)];
                    connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                    if (proxy._$ganttchartHelper)
                        proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);

                    if (childGanttRecord.taskId.toString() === predecessor.to)
                        proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);


                    if (childGanttRecord.taskId.toString() === predecessor.from)
                        proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);
                    connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                    if (connectorLineObject) {
                        if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.length == 0) {
                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) != -1) {
                            var index = proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId);
                            proxy._updatedConnectorLineCollection[index] = connectorLineObject;
                        }
                    }

                }

                var length = successors.length;

                for (count = 0; count < length; count++) {

                    predecessor = successors[count];
                    parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                    record = proxy._rows[proxy._ids.indexOf(predecessor.to)];
                    connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                    if (proxy._$ganttchartHelper)
                        proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                    proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);
                    connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                    if (connectorLineObject) {
                        if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.length == 0) {
                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) != -1) {
                            var index = proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId);
                            proxy._updatedConnectorLineCollection[index] = connectorLineObject;
                        }
                    }

                    record && proxy._validatePredecessor(record);
                }

            }
        },

        _calculateDateDifference: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                offset = 0;

            if (dateDiff > 0)
                offset = differenceCount - this._calculateWeekendCount(emptyDate, dateDiff);
            else
                offset = dateDiff + this._calculateWeekendCount(emptyDate, dateDiff);
            return offset;
        },

        _calculateWeekendCount: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                weekendCount = 0,
                i = 0;

            if (dateDiff > 0) {
                for (i = 0; i < dateDiff; i++) {
                    if (emptyDate.getDay() == 0) {
                        weekendCount++;
                        emptyDate.setDate(emptyDate.getDate() + 1);
                    }
                    else if (emptyDate.getDay() == 6) {
                        weekendCount += 2;
                        ++i;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    } else
                        emptyDate.setDate(emptyDate.getDate() + 1);
                }
            }
            else {
                for (i = 0; i < differenceCount; i++) {
                    emptyDate.setDate(emptyDate.getDate() - 1);
                    if (emptyDate.getDay() == 0 || emptyDate.getDay() == 6) {
                        weekendCount += 1;
                    }
                }
            }
            return weekendCount;
        },

        _daydiff: function (first, second) {
            return ((new Date(second) - new Date(first)) / (1000 * 60 * 60 * 24));
        },




        dateUpdate: function (date, days) {
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours(),
                date.getMinutes(),
                date.getSeconds(),
                date.getMilliseconds()
            );
        },

        //Method for collapse all the GanttRecords

        _collapseAll: function () {
            var proxy = this,
                model = proxy.model,
                count = 0,
                ganttRecord,
                args = {},
                enableVirtualization = model.enableVirtualization,
                $gridRows = proxy.getGridRows(),
                $rowElement = null;

            proxy._isInExpandCollapse = true;

            //loop excecute for iterate the GanttRecords preseent in the Gantt Control

            for (count = 0; count < proxy._updatedGanttRecords.length; count++) {
                ganttRecord = proxy._updatedGanttRecords[count];
                args.data = ganttRecord;
                args.recordIndex = count;
                args.expanded = false;

                if (enableVirtualization) {
                    proxy._$gridtreeHelper.ejGanttGridTree("sendExpandCollapseRequest", args);
                } else {
                    $rowElement = $($gridRows[count]);

                    if ($rowElement.css('display') === 'none')
                        continue;

                    proxy.sendExpandCollapseRequest(args);
                    count += proxy._collapsedRecordCount;
                }
            }

            proxy._isInExpandCollapse = false;
        },


        //Method for expand all the GanttRecords

        _expandAll: function () {

            var proxy = this,
                model = proxy.model,
                count = 0,
                ganttRecord,
                args = {},
                enableVirtualization = model.enableVirtualization;

            proxy._isInExpandCollapse = true;

            //loop excecute for iterate the GanttRecords preseent in the Gantt Control

            for (count = 0; count < proxy._updatedGanttRecords.length; count++) {
                ganttRecord = proxy._updatedGanttRecords[count];
                args.data = ganttRecord;
                args.recordIndex = count;
                args.expanded = true;

                if (enableVirtualization) {
                    proxy._$gridtreeHelper.ejGanttGridTree("sendExpandCollapseRequest", args);
                    count += proxy._$gridtreeHelper.ejGanttGridTree("getExpandCollapseRecordsCount");
                } else {

                    if (ganttRecord.hasChildGanttRecords) {
                        proxy.expandAllChildGanttRecords(ganttRecord);
                    }
                    proxy.sendExpandCollapseRequest(args);
                    count += proxy._expandedRecordsCount;
                }
            }

            proxy._isInExpandCollapse = false;
        },


        expandAllChildGanttRecords: function (ganttRecord) {
            for (var i = 0; i < ganttRecord.childGanttRecords.length; i++) {
                var data = ganttRecord.childGanttRecords[i];
                if (data.hasChildGanttRecords) {
                    data.expanded = true;
                    this.expandAllChildGanttRecords(data);
                }
            }
        },
        _getDateFromFormat: function (date) {

            if (typeof date === "object") {
                return new Date(date);
            }

            return Globalize.parseDate(date, this.model.dateFormat);
        },

        /**
        * destroy the button widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // Create Button
		* var btnObj = $("#button1").data("ejButton");
		* btnObj.destroy(); // destroy the button
		* &lt;/script&gt;
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // enable the button
		* $("#button1").ejButton("destroy");	
		* &lt;/script&gt;
		* @memberof ejButton
		* @instance
         */
        _destroy: function () {
            var proxy = this;
            proxy.element.empty().removeClass("e-gantt-core e-gantt " + proxy.model.cssClass);
        }



    });

    ej.Gantt.editingType = {
        String: "stringedit",
        Boolean: "booleanedit",
        Numeric: "numericedit",
        Dropdown: "dropdownedit",
        DatePicker: "datepicker",
        DateTimePicker: "datetimepicker",
    };

    ej.Gantt.toolBarItems = {
        Add: "add",
        Edit: "edit",
        Delete: "delete",
        Update: "update",
        Cancel: "cancel",
        Search: "search",
        Indent: "indent",
        Outdent: "outdent",
        ExpandAll: "expandAll",
        CollapseAll: "collapseAll"
    };

    ej.Gantt.localization = {};

    ej.Gantt.localization["en-US"] = {

        //string to display with dataSource contains 0 objects
        emptyRecord: "No records to display",

        //headerText to be displayed in gridtree
        columnHeaderTexts: {
            taskId: "ID",
            taskName: "Task Name",
            startDate: "Start Date",
            endDate: "End Date",
            resourceInfo: "Resources",
            duration: "Duration",
            status: "Progress",
            predecessor: "Predecessor",
            baselineStartDate: "Baseline Start Date",
            baselineEndDate: "Baseline End Date"
        },

        //string to display in dialog 
        editDialogTexts: {
            addFormTitle: "New Task",
            editFormTitle: "Edit Task",
            saveButton: "Save",
            cancelButton: "Cancel"
        },

        calendars: {
            standard: {
                days: {
                    // full name of days
                    names: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
                    // abbreviated names of days
                    namesAbbr: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
                },
                months: {
                    // full name of months
                    names: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
                    // abbreviated name of months
                    namesAbbr: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
                },

                // set of predefined date and time patterns used by the culture.
                patterns: {
                    d: "M/d/yyyy",
                    D: "dddd, MMMM dd, yyyy",
                    F: "dddd, MMMM dd, yyyy h:mm:ss tt",
                    g: "M/d/yyyy h:mm tt",
                    G: "M/d/yyyy h:mm:ss tt",
                    m: "MMMM dd",
                    M: "MMMM dd",
                    s: "yyyy'-'MM'-'ddTHH':'mm':'ss",
                    t: "h:mm tt",
                    T: "h:mm:ss tt",
                    u: "yyyy'-'MM'-'dd HH':'mm':'ss'Z'",
                    y: "MMMM, yyyy",
                    Y: "MMMM, yyyy"
                }
            }
        }
    };

    /*-----Initialize the GanttRecord object--------------*/

    ej.Gantt.GanttRecord = function () {
        var proxy = this;
        proxy.taskId = null,
        proxy.taskName = null,
        proxy.startDate = null,
        proxy.endDate = null,
        proxy.duration = null,
        proxy.isMilestone = false,
        proxy.status = null,
        proxy.predecessor = null,
        proxy.resourceInfo = null,
        proxy.parentItem = null,
        proxy.isSelected = false,
        proxy.childGanttRecords = null,
        proxy.hasChildGanttRecords = false,
        proxy.expanded = false,
        proxy.level = 0,
        proxy.left = 0,
        proxy.width = 0,
        proxy.progressWidth = 0,
        proxy.item = null,
        proxy.baselineLeft = 0,
        proxy.baselineWidth = 0,
        proxy.baselineStartDate = null,
        proxy.baselineEndDate = null,
        proxy.isReadOnly = false;
    }

    ej.Gantt.GanttRecord.prototype = {

        //calculate the left position of the taskbar
        _calculateLeft: function (projectStartDate, dayWidth, includeWeekend, dateformat, endDateMappingName, holidays) {

            var proxy = this,
                diffDate,
                enddatecount = 0,
                beginDate = proxy._getDateFromFormat(proxy.startDate, dateformat),
                endDate = proxy.endDate,
                duration = parseInt(proxy.duration);

            if (endDate == null) {

                endDate = proxy._getDateFromFormat(proxy.startDate, dateformat);
                endDate.setDate(endDate.getDate() + duration);
            }


            if (includeWeekend == false) {

                if (beginDate.getDay() == 0) {
                    beginDate.setDate(beginDate.getDate() + 1);
                }
                if (beginDate.getDay() == 6) {
                    beginDate.setDate(beginDate.getDate() - 1);
                }

                proxy._updateHoliday(beginDate, holidays, includeWeekend);

                var emptyDate = new Date(beginDate);
                for (var i = 0; i < duration; i++) {
                    if (emptyDate.getDay() == 0) {
                        enddatecount++;
                    } else if (emptyDate.getDay() == 6) {
                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    }
                    emptyDate.setDate(emptyDate.getDate() + 1);
                }
                endDate = new Date(beginDate);
                enddatecount = duration === 0 ? 1 : enddatecount;
                endDate.setDate(endDate.getDate() + (duration + enddatecount - 1));


                if (endDate.getDay() == 0) {
                    endDate.setDate(endDate.getDate() + 1);
                }
                if (endDate.getDay() == 6) {
                    endDate.setDate(endDate.getDate() + 2);
                }
            }
            proxy.startDate = proxy._getFormatedDate(beginDate, dateformat);
            proxy.endDate = proxy._getFormatedDate(endDate, dateformat);

            if (endDateMappingName)
                proxy.item[endDateMappingName] = proxy.endDate;

            diffDate = Math.ceil((beginDate - projectStartDate) / 86400000);

            return (diffDate * dayWidth);
        },

        //calculate the left margin of the baseLine element.
        _calculateBaselineLeft: function (projectStartDate, dayWidth, includeWeekend, dateformat) {

            var diffDate,
                proxy = this,
                beginDate = Globalize.parseDate(proxy.startDate, dateformat),
                duration = parseInt(proxy.duration),
                enddate = proxy.endDate;

            if (enddate == null) {
                enddate = new Date(beginDate);
                enddate.setDate(enddate.getDate() + duration);
            }
            diffDate = Math.ceil((beginDate - projectStartDate) / 86400000);
            return (diffDate * dayWidth);
        },

        _updateHoliday: function (date, holidays, includeWeekend) {
            var j = 0,
                length = holidays && holidays.length;

            if (length > 0) {

                for (j = 0; j < length; j++) {

                    if (holidays[j].getDate() == date.getDate()) {
                        date.setDate(date.getDate() + 1);
                    }

                }

                if ((includeWeekend == false) && (date.getDay() == 6)) {
                    date.setDate(date.getDate() + 2);
                }
            }
        },

        _calculateGanttRecordLeft: function (startDate, projectStartDate, dayWidth) {
            var proxy = this,
                diffDate = proxy._daydiff(projectStartDate, startDate);
            return (diffDate * dayWidth);
        },

        _calculateReverseLeft: function (projectStartDate, dayWidth, includeWeekend, dateformat) {
            var diffDate,
                proxy = this,
                enddatecount = 0,
                startdate = proxy.startDate,
                duration = proxy.duration && parseInt(proxy.duration);

            if (startdate == null) {

                startdate = proxy._getDateFromFormat(proxy.endDate);
                startdate.setDate(startdate.getDate() - duration);
            }
            if (includeWeekend == false) {

                var emptyDate = proxy._getDateFromFormat(proxy.endDate, dateformat);

                for (var i = 0; i < duration; i++) {

                    if (emptyDate.getDay() == 0) {
                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() - 2);
                    }
                    else if (emptyDate.getDay() == 6) {
                        enddatecount++;
                    }
                    emptyDate.setDate(emptyDate.getDate() - 1);
                }
                startdate = proxy._getDateFromFormat(proxy.endDate, dateformat);
                enddatecount = duration === 0 ? 1 : enddatecount;
                startdate.setDate(startdate.getDate() - (duration + enddatecount - 1));
            }
            proxy.startDate = proxy._getFormatedDate(startdate, dateformat);
            diffDate = Math.ceil((startdate - projectStartDate) / 86400000);
            return (diffDate * dayWidth);
        },

        _calculateReverseWidth: function (dayWidth, dateFormat, includeWeekend, holidays) {

            var proxy = this,
                enddatecount = 0,
                diffDate,
                emptyDate,
                i = 0,
                startdate = proxy.startDate,
                duration = proxy.duration && parseInt(proxy.duration),
                enddate = proxy.endDate;

            if (startdate == null) {
                startdate = proxy._getDateFromFormat(enddate, dateFormat);
                startdate.setDate(startdate.getDate() - duration + 1);

            }

            if (includeWeekend == false) {

                emptyDate = proxy._getDateFromFormat(enddate, dateFormat);

                for (i = 0; i < duration; i++) {

                    if (emptyDate.getDay() == 0) {

                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() - 2);

                    } else if (emptyDate.getDay() == 6) {
                        enddatecount++;
                    }

                    emptyDate.setDate(emptyDate.getDate() - 1);

                }

                startdate = proxy._getDateFromFormat(enddate, dateFormat);
                startdate.setDate(startdate.getDate() - (duration + enddatecount - 1));
            }

            if (holidays != null && holidays.length > 0) {

                emptyDate = proxy._getDateFromFormat(startdate, dateFormat);

                for (i = 0; i < duration; i++) {

                    emptyDate.setDate(emptyDate.getDate() + 1);

                    for (var j = 0; j < holidays.length; j++) {

                        if (new Date(holidays[j].day).getDate() == emptyDate.getDate()) {
                            enddatecount++;
                        }

                    }

                    if ((includeWeekend == false) && (emptyDate.getDay() == 0 || emptyDate.getDay() == 6)) {
                        enddatecount++;
                    }

                }

                enddate = proxy._getDateFromFormat(proxy.startDate, dateFormat);
                enddate.setDate(enddate.getDate() + (duration + enddatecount - 1));
            }

            diffDate = Math.ceil((proxy._getDateFromFormat(enddate, dateFormat) - proxy._getDateFromFormat(proxy.startDate, dateFormat)) / 86400000) + 1;

            return (diffDate) * dayWidth;
        },

        _calculateWidth: function (dayWidth, includeWeekend, holidays, dateFormat) {

            var proxy = this,
                diffDate,
                startdate = proxy.startDate,
                enddate = proxy.endDate,
                duration = proxy.duration;

            if (enddate == null) {
                duration = parseInt(duration);
                enddate = proxy._getFormatedDate(startdate, dateFormat);
                enddate.setDate(enddate.getDate() + duration - 1);
            }

            proxy._calculateEndDate(includeWeekend, holidays, dateFormat);

            diffDate = Math.ceil((proxy._getDateFromFormat(proxy.endDate, dateFormat) - proxy._getDateFromFormat(proxy.startDate, dateFormat)) / 86400000) + 1;

            return (diffDate) * dayWidth;
        },


        _calculateTaskbarWidth: function (dayWidth, dateFormat) {
            var proxy = this,
                diffDate = proxy._daydiff(proxy._getDateFromFormat(proxy.startDate, dateFormat), proxy._getDateFromFormat(proxy.endDate, dateFormat)) + 1;
            return (diffDate * dayWidth);
        },

        _getFormatedDate: function (dateObject, dateformat) {
            return Globalize.format(dateObject, dateformat);
        },

        _getDateFromFormat: function (dateString, dateformat) {
            return Globalize.parseDate(dateString, dateformat);
        },

        _calculateProgressWidth: function (parentWidth, percent) {
            return (parentWidth * percent) / 100;
        },

        _setResourceInfo: function (resourceIdcollection, resourceIdMapping, resourceNameMapping, resourcecollection) {

            var count = 0,
            length = resourceIdcollection.length,
            resources = [];

            for (count; count < length; count++) {

                var resource = resourcecollection.filter(function (resourceInfo) {
                    return resourceIdcollection[count] === resourceInfo[resourceIdMapping];
                });

                resource.length && resources.push(resource[0]);
            }
            return resources;
        },

        _updateEndDate: function (prevStartDate, includeWeekend, dateFormat) {
            var proxy = this,
                formatedPrevStartDate = new Date(prevStartDate),
                formatedStartDate = proxy._getDateFromFormat(proxy.startDate, dateFormat),
                diff = proxy._daydiff(formatedPrevStartDate, formatedStartDate),
            endDate = proxy._getDateFromFormat(proxy.endDate, dateFormat),
            weekendCount = 0;

            if (includeWeekend == false) {
                var count = 0;
                for (count; count <= diff; count++) {
                    endDate.setDate(endDate.getDate() + 1);
                    if (endDate.getDay() == 0 || endDate.getDay() == 6)
                        weekendCount += 1;
                }
            }

            var modifiedEndDate = proxy._getDateFromFormat(proxy.endDate, dateFormat);
            modifiedEndDate.setDate(modifiedEndDate.getDate() + diff + weekendCount);
            proxy.endDate = proxy._getFormatedDate(modifiedEndDate, dateFormat);
        },

        _calculateFormatedDate: function (date, format) {
            return Globalize.format(date, format);
        },

        _updateParentProgress: function (parent) {
            var parentProgress = 0,
                childCount = parent.childGanttRecords && parent.childGanttRecords.length,
                milestoneCount = 0;
            if (parent.childGanttRecords) {
                for (var i = 0; i < childCount; i++) {
                    if (!parent.childGanttRecords[i].isMilestone) {
                        parentProgress += Math.ceil(parent.childGanttRecords[i].status);
                    }
                    else
                        milestoneCount++;
                }

                parentProgress = parentProgress / (parent.childGanttRecords.length - milestoneCount);
                parent.progressWidth = this._getProgressWidth(parent.width, parentProgress);
                parent.status = Math.ceil(parentProgress);
            }
            if (parent.parentItem) {
                this._updateParentProgress(parent.parentItem);
            }
        },


        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },

        _calculatePredecessor: function (string) {
            var collection = [],
                match,
                obj = {},
                values,
                to = this.taskId.toString(),
                offsetvalue;
            string.split(',').forEach(function (el) {
                values = el.split('+');
                offsetvalue = '+';
                if (el.indexOf('-') >= 0) {
                    values = el.split('-');
                    offsetvalue = '-';
                }
                match = values[0].match(/(\d+|[A-z]+)/g);
                obj = {
                    from: match[0],
                    predecessorsType: match.length > 1 ? match[1].toUpperCase() : "FS",
                    offset: values.length > 1 ? offsetvalue + "" + values[1] : "",
                    isdrawn: false,
                    to: to
                };
                collection.push(obj);
            });
            return collection;
        },

        _updateGanttRecord: function (startDateMappingName, endDateMappingName, durationMappingName, includeWeekend, scheduleStartDay, perDayWidth, dateFormat, holidays) {
            var proxy = this,
                remainDays = proxy.left % perDayWidth,
                day = (proxy.left - remainDays) / perDayWidth,
                startDate = new Date(scheduleStartDay);

            startDate.setDate(scheduleStartDay.getDate() + day);

            if (startDate.getDay() == 0)
                startDate.setDate(startDate.getDate() + 1);
            else if (startDate.getDay() == 6)
                startDate.setDate(startDate.getDate() + 2);

            proxy.startDate = proxy._getFormatedDate(startDate, dateFormat);
            proxy.left = proxy._calculateLeft(scheduleStartDay, perDayWidth, includeWeekend, dateFormat, endDateMappingName);
            proxy.width = proxy._calculateWidth(perDayWidth, includeWeekend, holidays, dateFormat);
            if (proxy.parentItem)
                proxy._updateParentItem(proxy, startDateMappingName, endDateMappingName, durationMappingName, includeWeekend, scheduleStartDay, perDayWidth, dateFormat, holidays);
        },

        _calculateUpdatedDate: function (modifiedDate, offset) {
            var proxy = this,
                i = 0;
            if (offset > 0) {
                for (i; i < offset; i++) {
                    modifiedDate = proxy._dataSubtract(modifiedDate, 1);
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() + 1);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() + 2);
                }
            } else {
                var dateOffset = (-1) * offset;
                for (i = 0; i < dateOffset; i++) {
                    modifiedDate = proxy._dataSubtract(modifiedDate, -1);
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() - 2);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() - 1);
                }
            }
            modifiedDate = proxy._getFormatedDate(modifiedDate);
            return modifiedDate;
        },

        _dataSubtract: function (date, days) {
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours(),
                date.getMinutes(),
                date.getSeconds(),
                date.getMilliseconds()
            );
        },

        _calculateEndDate: function (includeWeekend, holidays, dateFormat) {
            var proxy = this,
                count = 0,
                startDate = proxy._getDateFromFormat(proxy.startDate, dateFormat),
                duration = proxy.duration && parseInt(proxy.duration);


            if (duration === 0) {
                proxy.endDate = proxy.startDate;
                return;
            }

            if (includeWeekend === false) {
                for (count = 1; count < duration; count++) {
                    if (startDate.getDay() == 0)
                        startDate.setDate(startDate.getDate() + 1);
                    else if (startDate.getDay() == 6)
                        startDate.setDate(startDate.getDate() + 2);

                    startDate.setDate(startDate.getDate() + 1);
                }
            } else {
                startDate.setDate(startDate.getDate() + duration - 1);
            }

            if (includeWeekend === false) {
                if (startDate.getDay() == 0)
                    startDate.setDate(startDate.getDate() + 1);
                else if (startDate.getDay() == 6)
                    startDate.setDate(startDate.getDate() + 2);
            }

            if (holidays && holidays.length) {
                var holidayIndex = 0,
                    holidayslength = holidays.length,
                    holiday;
                for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                    holiday = new Date(holidays[holidayIndex]);
                    if (holiday >= proxy._getDateFromFormat(proxy.startDate, dateFormat) && holiday <= startDate) {
                        startDate.setDate(startDate.getDate() + 1);
                        if (includeWeekend === false) {
                            if (startDate.getDay() == 0) {
                                startDate.setDate(startDate.getDate() + 1);
                            } else if (startDate.getDay() == 6) {
                                startDate.setDate(startDate.getDate() + 2);
                            }
                        }
                    }
                }
            }

            if (includeWeekend === false) {
                if (startDate.getDay() == 0)
                    startDate.setDate(startDate.getDate() + 1);
                else if (startDate.getDay() == 6)
                    startDate.setDate(startDate.getDate() + 2);
            }
            proxy.endDate = proxy._calculateFormatedDate(startDate, dateFormat);
        },

        _dateUpdate: function (date, days) {
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours(),
                date.getMinutes(),
                date.getSeconds(),
                date.getMilliseconds()
            );
        },


        _calculateDuration: function (startDate, endDate, includeWeekend, holidays, milestone) {
            var proxy = this,
            dateDiff = proxy._daydiff(startDate, endDate);

            if (includeWeekend === false)
                offset = proxy._calculateDateDifference(new Date(startDate), dateDiff);
            else
                offset = dateDiff + 1;

            for (var count = 0; count < holidays.length; count++) {
                if (new Date(holidays[count].day) >= startDate && new Date(holidays[count].day) <= endDate) {
                    offset--;
                }
            }

            if (ej.isNullOrUndefined(milestone))
                proxy.duration = offset;
            else
                proxy.duration = (offset == 0 && milestone == false) ? 1 : offset;
        },

        _calculateDateDifference: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                offset = 0;

            if (dateDiff > 0)
                offset = differenceCount - this._calculateWeekendCount(emptyDate, dateDiff) + 1;
            else
                offset = dateDiff + this._calculateWeekendCount(emptyDate, dateDiff);
            return offset;
        },

        _calculateWeekendCount: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                weekendCount = 0,
                i = 0;

            if (dateDiff > 0) {
                for (i = 0; i < dateDiff; i++) {
                    if (emptyDate.getDay() == 0) {
                        weekendCount++;
                        emptyDate.setDate(emptyDate.getDate() + 1);
                    }
                    else if (emptyDate.getDay() == 6) {
                        weekendCount += 2;
                        ++i;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    } else
                        emptyDate.setDate(emptyDate.getDate() + 1);
                }
            }
            else {
                for (i = 0; i < differenceCount; i++) {
                    emptyDate.setDate(emptyDate.getDate() - 1);
                    if (emptyDate.getDay() == 0 || emptyDate.getDay() == 6) {
                        weekendCount += 1;
                    }
                }
            }
            return weekendCount;
        },

        _daydiff: function (first, second) {
            return ((second - first) / (1000 * 60 * 60 * 24));
        },

        _getProgressPercent: function (parentwidth, progresswidth) {
            return Math.ceil(((progresswidth / parentwidth) * 100).toFixed(2));
        },

    };

})(jQuery, Syncfusion);

;
/*ganttgridtree implementation of gantt control */
(function ($, ej, undefined) {

    ej.widget("ejGanttGridTree", "ej.GanttGridTree", {
        _rootCSS: "e-ganttgridtree",

        element: null,

        model: null,

        /* keyConfigurations for keyboard interaction in ganttgridtree */
        keyConfigs: {
            focus: "e",

            //Insert
            insertRecord: "45",

            // delete
            deleteRecord: "46",

            //F2
            editRecord: "113",

            // Enter
            saveRequest: "13",

            //Esc   
            cancelRequest: "27",

            //Home,
            firstRowSelection: "36",

            //End,
            lastRowSelection: "35",

            //Up arrow
            upArrow: "38",

            //Down arrow
            downArrow: "40",

            //tab
            moveCellRight: "9",

            //shifttab
            moveCellLeft: "shift+9",

            //AltPlusDownArrow
            selectedRowExpand: "alt+40",

            //CtrlPlusDownArrow
            totalRowExpand: "ctrl+40",

            //AltPlusUpArrow
            selectedRowCollapse: "alt+38",

            //CtrlPlusUpArrow
            totalRowCollapse: "ctrl+38"
        },

        /* default model */
        defaults: {

            //enable the altRow
            altRow: true,

            //enable the columnresize
            allowColumnResize: false,

            //enable the virtualization
            enableVirtualization: false,

            /* sorting API's */
            //enable the sorting
            allowSorting: false,
            //enable the multicolumn sorting
            allowMultiSorting: false,
            //define sortcolumns while loading
            sortSettings: {
                sortedColumns: []
            },

            /* selection API's */
            //enable the selection
            allowSelection: true,
            //enable the single/multiple row selection
            selectionType: "single",
            //slection based on the rowIndex
            selectedRowIndex: -1,
            //selected ganttRecord object
            selectedItem: null,

            //enable the searching 
            allowSearching: false,

            //enable the keynavigation to select rows
            allowKeyboardNavigation: true,

            //apply multiple themes to gridtree
            cssClass: "",

            //culture setting to apply localization
            localization: "en-US",

            /* editing API's */
            edit: {
                allowEditing: false,
                allowAdding: false,
                allowDeleting: false,
                editMode: "normal",
                dialogEditorTemplateId: null
            },

            isEdit: false,

            /* event function */
            rowTemplate: null,
            rowDataBound: null,
            queryCellInfo: null,

            /* ganttRecords object */
            dataSource: null,
            currentViewData: null,
            flatRecords: null,
            parentRecords: null,
            records: null,
            ids: null,

            columns: null,
            commonWidth: 150,

            /* datamanager API's */
            query: ej.Query(),

            dateFormat: "MM/dd/yyyy",

            /* resource mapping details */
            resourceIdMapping: "",
            resourceNameMapping: "",
            resourceInfoMapping: "",


            /* task details mapping */
            taskIdMapping: "",
            taskNameMapping: "",
            startDateMapping: "",
            endDateMapping: "",
            childMapping: "",
            durationMapping: "",
            progressMapping: "",
            predecessorMapping: "",

            //rowHeight of the ganttgridtree
            rowHeight: 30,

            emptyRecordText: null
        },

        dataTypes: {
            columns: "array",
            sortSettings: {
                sortedColumns: "array"
            },
            dataSource: "array",
            currentViewData: "array",
            flatGanttRecords: "array",
            parentGanttRecords: "array",
            records: "array",
            ids: "array",

        },

        //get the grid header table for columnresize
        getGridHeaderTable: function () {
            return this._$gridHeaderTable;
        },

        //set the grid header table
        setGridHeaderTable: function (value) {
            this._$gridHeaderTable = value;
        },

        //get the gridheadercontent
        getGridHeaderContent: function () {
            return this._gridHeaderContent;
        },

        //set the gridheadercontent
        setGridHeaderContent: function (value) {
            this._gridHeaderContent = value;
        },

        //get the gridContent table
        getGridContentTable: function () {
            return this._$gridContentTable;
        },

        //set the gridContent table
        setGridContentTable: function (value) {
            this._$gridContentTable = value;
        },

        //get gridtree rows
        getGridRows: function () {
            return this._gridRows;
        },

        //get the currentViewPort data
        getCurrentViewData: function () {
            return this.model.currentViewData;
        },

        //get the FlatGanttRecords
        getFlatGanttRecords: function () {
            return this.model.records;
        },

        //set the height based on the records count
        setHeight: function (value) {
            this._totalHeight = value;
        },

        //get the updatedRecords( sorted,filtered,expandcollapsed)
        getUpdatedRecords: function () {
            return this._updatedRecords;
        },

        //get the ids collection after perform the addition/delete operation
        getUpdatedIds: function () {
            return this.model.ids;
        },

        //get the records count in calculate VisibleRange
        getRecordsCount: function () {
            return this._recordsCount;
        },

        //set the records count
        setRecordsCount: function (value) {
            this._recordsCount = value;
        },

        //get the expandCollapse records count
        getExpandCollapseRecordsCount: function () {
            return this._expandCollapseRecordCount;
        },


        //constructor function
        _init: function () {

            var proxy = this;

            proxy._initPrivateProperties();

            if (proxy.model.edit.allowEditing) {
                proxy._processEditing();
            }

            proxy._checkDataBinding();

        },

        //Method for update the GanttGridTree in run time.
        _setModel: function (options) {
            var proxy = this;
            var model = proxy.model;

            for (var prop in options) {

                switch (prop) {

                    case "cssClass":
                        proxy.element.removeClass(model.cssClass).addClass(options[prop]);
                        break;

                        //case "enableRowHover":
                        //    model.enableRowHover = options[prop];
                        //    //proxy._enableRowHover();
                        //    break;
                    case "localization":
                        model.localization = options[prop];
                        proxy.element.ejGanttGridTree("destroy").ejGanttGridTree(model);
                        break;

                    case "dataSource":
                        proxy.resetModelCollections();
                        proxy.refreshGridContent();
                        break;

                    case "selectedRowIndex":
                        var index = options[prop];
                        if (!proxy._rowSelectingEventTrigger()) {
                            proxy.selectRows(options[prop]);
                            proxy._rowSelectedEventTrigger(index);
                        }
                        break;

                    case "sortSettings":
                        $.extend(model.sortSettings, options[prop]);
                        proxy.refreshGridContent();
                        break;

                    case "allowSelection":
                        if (options[prop]) proxy._off(proxy.element, "click")._on(proxy.element, 'click', proxy._clickHandler);
                        break;

                    case "altRow":
                        model.altRow = options[prop];
                        proxy.addInitTemplate();
                        proxy.refreshGridContent();
                        break;
                    case "edit":
                        $.extend(model.edit, options[prop]);
                        proxy._processEditing();
                        proxy.refreshCellEditMode();
                        proxy._enableEditingEvents();
                        break;
                }
            }
        },

        updateAllowColumnResize: function (bool) {

            var proxy = this, model = proxy.model;
            model.allowColumnResize = bool;
            if (model.allowColumnResize) {
                proxy._on(proxy.element, "mousemove", proxy._mouseMove);
                proxy._on(proxy.element, "mouseup", proxy._mouseUp);
            } else {
                proxy._off(proxy.element, "mousemove", proxy._mouseMove);
                proxy._off(proxy.element, "mouseup", proxy._mouseUp);
            }
        },

        updateAllowKeyboardNavigation: function (bool) {
            this.model.allowKeyboardNavigation = bool;
        },

        updateRowSelection: function (bool) {
            if (bool)
                proxy._off(proxy.element, "click")._off(proxy.element, 'click', proxy._clickHandler);
        },

        updateAltRow: function (bool) {
            var proxy = this, model = proxy.model;
            model.altRow = bool;
            for (var i = 0; i < proxy._updatedRecords.length; i++) {
                proxy._updatedRecords[i].isAltRow = model.altRow;
            }
            //proxy.addInitTemplate();
            proxy.refreshGridContent();
        },

        //toggle between edit modes dynamically
        updateEditMode: function (edit) {
            var proxy = this, model = proxy.model;
            $.extend(model.edit, edit);
            proxy._processEditing();
            proxy.refreshCellEditMode();
            proxy._enableEditingEvents();
        },

        refreshGridContent: function (templateRefresh) {

            var proxy = this, args = {};
            templateRefresh && proxy.templateRefresh();
            args.requestType = ej.GanttGridTree.actions.Refresh;
            proxy.processBindings(args);
        },

        templateRefresh: function () {
            var proxy = this;
            proxy.addInitTemplate();
        },

        resetModelCollections: function () {
            this.model.sortSettings.sortedColumns = [];
        },

        //refresh the editMode in runtime
        refreshCellEditMode: function () {

            var proxy = this;

            if (proxy.model.edit.editMode == "cellEditing") {
                proxy.refreshCellEditChanges();
                proxy._cellEditingDetails._data = $.extend(true, [], proxy.getCurrentViewData());
            }

        },

        //refresh the cellEditing details while perform cellEditing
        refreshCellEditChanges: function () {

            var proxy = this;

            proxy._cellEditingDetails = {
                cellValue: null,
                rowIndex: -1,
                _data: null,
                fieldName: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                editedTr: [],
                data: null
            };

            proxy._cellEditingChanges = {
                addedData: null,
                deletedData: null,
                changedData: null
            };

        },

        //declare the private variables
        _initPrivateProperties: function () {
            var proxy = this;
            proxy._columnsWidthCollection = [];
            proxy._cSortedColumn = null;
            proxy._cSortedDirection = null;
            proxy._dataManager = null;
            proxy._disabledSortableColumns = [];
            proxy._filteredGanttRecords = [];
            proxy._fieldColumnNames = [];
            proxy._id = proxy.element.attr("id");
            proxy._multiSortRequest = null;
            proxy._$gridContainer = null;
            proxy._$gridContent = null;
            proxy._$gridContentTable = null;
            proxy._$gridHeader = null;
            proxy._$gridHeaderTable = null;
            proxy._gridHeight = proxy.element.height();
            proxy._recordsCount = null;
            proxy._gridRows = null;
            proxy._gridWidth = proxy.element.width();
            proxy._headerColumnNames = [];
            proxy._hiddenColumns = [];
            proxy._multiSelectCtrlRequest = false;
            proxy._multiSelectShiftRequest = false;
            proxy._offset = 0;
            proxy._previousIndex = -1;
            proxy._prevRBottom = 0;
            proxy._prevRTop = 0;
            proxy._prevScrollTop = 0;
            proxy._prevScrollLeft = 0;
            proxy._prevVBottom = 0;
            proxy._prevVTop = 0;
            proxy._primaryKeyValues = [];
            proxy._renderedRange = null;
            proxy._resizer = null;
            proxy._scrollTop = 0;
            proxy._searchString = "";
            proxy._selectedRowsIndexes = [];
            proxy._sortedGanttRecords = [];
            proxy._storedIndex = -1;
            proxy._$tableContent = null;
            proxy._tempsortedrecords = [];
            proxy._tempfilteredrecords = [];
            proxy._totalHeight = 0;
            proxy._viewPortHeight = 0;
            proxy._visibleColumns = [];
            proxy._visibleRange = null;
            proxy._vScrollDir = 1;
            proxy._vScrollDist = 0;
            proxy._primaryKeys = [];
            proxy._dropDownManager = null;
            proxy._cellEditTemplate = $();
            proxy._dropDownTemplate = proxy._compiledDropDownTemplate();
            proxy._cellEditingDetails = {
                cellValue: null,
                rowIndex: -1,
                columnIndex: -1,
                fieldName: null,
                _data: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                data: null
            };
            proxy._cellEditingChanges = {
                added: null,
                deleted: null,
                changed: null
            };
            proxy._tdsOffsetWidth = [];
            proxy._isEnterKeyPressed = false;
            proxy._updatedRecords = [];
            proxy._scrollBarHeight = 0;
            proxy._expandCollapseRecordCount = 0;
            proxy._removedCount = 0;
            proxy._tempRecordIndex = -1;
            proxy._$currentTr = null;
            proxy._currentTrIndex = -1;
            proxy._gridHeaderContent = null;
        },

        _checkDataBinding: function () {

            var proxy = this,
                model = proxy.model;

            proxy.element.addClass('e-ganttgridtree-core');

            if (model.flatRecords.length === 0 && model.dataSource !== null) {
                proxy._createRecords(model.dataSource);
            }

            proxy._dataManager = new ej.DataManager(model.flatRecords);
            model.cssClass && proxy.element.addClass(model.cssClass);
            model.columns.length && proxy._renderAfterColumnInitialize();
            proxy._$gridHeader = $("#" + proxy._id + "e-gridheader");
            proxy._viewPortHeight = proxy.getViewPortHeight();
            proxy._initGridRender();
            proxy._$gridContent = $("#" + proxy._id + "e-gridcontent");
            proxy._scrollBarHeight = proxy._$gridContent[0].offsetHeight - proxy._$gridContent[0].clientHeight;
            proxy._$gridContainer = $("#" + proxy._id + "e-gridcontainer");
            proxy._$tableContent = $("#" + proxy._id + "e-table");
            proxy._wireEvents();
            proxy._ensureDataSource();
            proxy.setWidthToColumns();
            proxy._addEmptyColumntoGrid();
        },

        _ensureDataSource: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns,
                isupdatedHeight = false,
                index,
                childIndex;

            if (model.dataSource == null) {
                return;
            }

            model.query.requiresCount();
            proxy._queryManagar = model.query;

            if ((model.edit.allowEditing || model.edit.allowAdding) &&
                args && args.requestType == ej.GanttGridTree.actions.Save &&
                (!ej.isNullOrUndefined(args._cModifiedData) || !ej.isNullOrUndefined(args._cAddedRecord))) {

                if (ej.isNullOrUndefined(args._cAddedRecord)) {
                    $.extend(model.flatRecords[$.inArray(model.selectedItem, model.flatRecords)], args._cModifiedData);
                } else {

                    var parentItem;

                    if (model.selectedRowIndex === -1) {

                        args._cAddedRecord.hasChildGanttRecords = false;
                        model.flatRecords.unshift(args._cAddedRecord);
                        model.records.unshift(args._cAddedRecord);
                        model.dataSource.unshift(args._cAddedRecord.item);
                        model.ids.unshift(args._cAddedRecord.taskId);
                        args.recordIndex = 0;

                    } else {

                        args._cAddedRecord.level = model.selectedItem.level;
                        index = model.selectedItem ? model.flatRecords.indexOf(model.selectedItem) : 0;

                        if (args._cAddedRecord.level === 0) {
                            index = index > 0 ? index - 1 : index;
                        }

                        if (args.actionRequestType != ej.GanttGridTree.actions.ContextMenuAdd) {
                            args.recordIndex = index > 0 ? index - 1 : 0;
                        }
                        model.flatRecords.splice(index, 0, args._cAddedRecord);

                        var rowIndex = model.records.indexOf(model.selectedItem);
                        model.records.splice(rowIndex, 0, args._cAddedRecord);
                        model.ids.splice(rowIndex, 0, args._cAddedRecord.taskId.toString());

                        if (args._cAddedRecord.level > 0) {

                            parentItem = model.selectedItem.parentItem;
                            if (parentItem) {
                                childIndex = parentItem.childGanttRecords ?
                                    parentItem.childGanttRecords.indexOf(model.selectedItem) : -1;

                                if (childIndex !== -1) {
                                    childIndex = childIndex === 0 ? 0 : childIndex - 1;
                                }

                                parentItem.childGanttRecords.splice(childIndex, 0, args._cAddedRecord);
                            }
                        }

                        proxy.setRecordsCount(model.flatRecords.length);
                        proxy.setHeight(model.flatRecords.length * model.rowHeight);
                        proxy._updateHeight();
                        isupdatedHeight = true;
                    }

                    proxy._queryManagar.queries = [];
                    model.isEdit = false;
                    proxy._updatedGanttRecords = model.flatRecords;
                }
            }
            if (args && args.requestType === ej.GanttGridTree.actions.ExpandCollapse) {

                isupdatedHeight = true;
                proxy.cleanUpSelection(proxy.model.selectedRowIndex);
                proxy._filteredGanttRecords = [];
                proxy._expandCollapseRecordCount = 0;
                proxy.updateExpandCollapseGanttRecords(args.data, args.expanded, args.recordIndex);
                proxy.setRecordsCount(model.flatRecords.length);
                proxy.setHeight(model.flatRecords.length * model.rowHeight);
                proxy._updatedRecords = model.flatRecords;

                if (model.allowSearching && proxy._searchString.length) {
                    proxy._updateFilteredGanttRecords(proxy._tempfilteredrecords);
                    proxy.setRecordsCount(proxy._filteredGanttRecords.length);
                    proxy.setHeight(proxy._filteredGanttRecords * model.rowHeight);
                    proxy._updatedRecords = proxy._filteredGanttRecords;
                }

                if (model.allowSorting && model.sortSettings.sortedColumns.length) {
                    proxy._updateExpandCollapseForSortedGanttRecords(args.data, args.expanded);
                    proxy.setHeight(proxy._sortedGanttRecords * model.rowHeight);
                    proxy._updatedRecords = proxy._sortedGanttRecords;
                }

                proxy._updateHeight();

                if (model.altRow) {
                    proxy._updateAltRow();
                }

            }

            if (model.edit.allowDeleting && args && args.requestType === ej.GanttGridTree.actions.Delete) {

                var deletedRow = args.data,
                    recordIndex,
                    rowindex;
                proxy._removedCount = 0;
                var childGanttRecordsCount = proxy._getFilteredChildRecordsCount(deletedRow);
                childGanttRecordsCount += 1;
                proxy._removedCount = 0;

                if (deletedRow.parentItem) {

                    var childGanttRecords = deletedRow.parentItem.childGanttRecords;
                    childIndex = childGanttRecords.indexOf(deletedRow);

                    deletedRow.parentItem.childGanttRecords.splice(childIndex, 1);
                    deletedRow.parentItem.item[proxy.model.childMapping].splice(childIndex, 1);
                }

                recordIndex = proxy.model.flatRecords.indexOf(deletedRow);
                proxy.model.flatRecords.splice(recordIndex, childGanttRecordsCount);
                proxy.setRecordsCount(model.flatRecords.length);
                proxy.setHeight(model.flatRecords.length * model.rowHeight);
                proxy._updateHeight();
                isupdatedHeight = true;
                index = model.records.indexOf(deletedRow);
                model.records.splice(index, childGanttRecordsCount);

                proxy._updatedRecords = model.records;
                //args.count = childGanttRecordsCount;
                //args.startIndex = recordIndex;
                //proxy.deleteRows(recordIndex, childGanttRecordsCount);//remove Gantt tree rows

                rowindex = model.dataSource.indexOf(deletedRow.Item);
                proxy.model.dataSource.splice(rowindex, 1);
                proxy.cleanUpSelection(proxy.model.selectedRowIndex);
            }

            if (model.allowSearching && proxy._searchString.length) {

                proxy._queryManagar.search(proxy._searchString, proxy.getColumnFieldNames(), ej.FilterOperators.contains, true);

                proxy._filteredGanttRecords = model.records.filter(function (record) {

                    return record.hasChildGanttRecords === false;
                });

                var dataManager = new ej.DataManager(proxy._filteredGanttRecords);

                proxy._tempfilteredrecords = dataManager.executeLocal(proxy._queryManagar).result;
                proxy._updateFilteredGanttRecords(proxy._tempfilteredrecords);
                proxy.setRecordsCount(proxy._filteredGanttRecords.length);
                proxy._updatedRecords = proxy._filteredGanttRecords;

                if (model.allowSorting && model.sortSettings.sortedColumns.length > 0) {

                    proxy._sortingGanttRecords();
                }

                proxy.setHeight(proxy._updatedRecords.length * model.rowHeight);
                proxy._updateHeight();
                isupdatedHeight = true;

                if (model.altRow) {

                    proxy._updateAltRow();
                }

            } else if (model.allowSorting && sortcolumns.length > 0) {

                isupdatedHeight = proxy._sortingGanttRecords();
            }

            if (!isupdatedHeight) {

                proxy.setRecordsCount(model.flatRecords.length);
                proxy.setHeight(model.flatRecords.length * model.rowHeight);
                proxy._updatedRecords = model.flatRecords;
                proxy._updateHeight();

            }

            if (!args) {
                args = {};
                args.requestType = ej.GanttGridTree.actions.Refresh;

            }

            proxy._updateCurrentViewData();

        },
        //Delete Gantt tree rows without rerendering of whole element 
        deleteRows: function (startIndex, count) {
            var proxy = this;
            var $gridRows = $(proxy.getGridRows());
            for (var i = startIndex; i < startIndex + count; i++) {
                $gridRows.splice(i, 1);
            }
            //$(".e-gridtreerows:first").addClass("e-alt-row");
        },
        _updateAltRow: function () {

            var proxy = this,
                count = 0,
                currentViewData = proxy._updatedRecords,
                length = currentViewData.length,
                record;

            for (count = 0; count < length; count++) {

                record = currentViewData[count];
                record.isAltRow = count % 2 == 0 ? true : false;

            }

        },

        renderGanttRecords: function (args) {

            if (!args) {

                args = {};
                args.requestType = ej.GanttGridTree.actions.Refresh;

            }

            this.sendDataRenderingRequest(args);
        },

        _sortingGanttRecords: function () {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns;

            proxy._queryManagar.queries = [];

            if (sortcolumns.length) {
                var length = sortcolumns.length - 1;
                for (var i = length; i >= 0; i--)
                    proxy._queryManagar.sortBy(sortcolumns[i].field, sortcolumns[i].direction);
            }

            var dataManager = new ej.DataManager(model.parentRecords),
                records = dataManager.executeLocal(proxy._queryManagar).result;

            proxy._sortedGanttRecords = [];
            proxy._storedIndex = -1;

            proxy._createSortedGanttRecords(records);
            proxy._tempsortedrecords = proxy._sortedGanttRecords;
            proxy._updateSortedGanttRecords(proxy._tempsortedrecords);
            proxy.setRecordsCount(proxy._sortedGanttRecords.length);
            proxy._updatedRecords = proxy._sortedGanttRecords;

            return true;
        },

        _updateExpandCollapseForFilteredGanttRecords: function (ganttRecord, expanded) {

            var proxy = this,
                index = proxy._filteredGanttRecords.indexOf(ganttRecord),
                count,
                length = proxy._filteredGanttRecords.length,
                record,
                level = ganttRecord.level,
                removecount = 0,
                model = proxy.model;

            if (expanded) {

                index = model.flatRecords.indexOf(ganttRecord);

                var records = [],
                    dataManager,
                    childGanttRecords;

                for (count = index + 1; count < model.flatRecords.length; count++) {

                    record = model.flatRecords[count];

                    if (ganttRecord.level === record.level) {
                        break;
                    } else if (record.level > ganttRecord.level && !record.hasChildGanttRecords) {
                        records.push(record);
                    }

                }

                dataManager = ej.DataManager(records);
                childGanttRecords = dataManager.executeLocal(proxy._queryManagar).result;
                length = childGanttRecords.length;

                for (count = 0; count < length; count++) {

                    proxy._filteredGanttRecords.splice(index + 1 + count, 0, childGanttRecords[count]);

                }
            } else {

                for (count = index + 1; count < length; count++) {

                    record = proxy._filteredGanttRecords[count];

                    if (record.level == level) {

                        break;

                    } else {

                        removecount++;
                    }
                }

                proxy._filteredGanttRecords.splice(index + 1, removecount);

            }
        },

        _updateExpandCollapseForSortedGanttRecords: function (ganttRecord, expanded) {

            var proxy = this,
                count = 0,
                length = proxy._sortedGanttRecords.length,
                record,
                index = proxy._sortedGanttRecords.indexOf(ganttRecord);

            if (expanded) {

                proxy._addNestedSortGanttRecords(ganttRecord, index);

            } else {

                var removedCount = 0;

                for (count = index + 1; count < length; count++) {

                    record = proxy._sortedGanttRecords[count];

                    if (record.level === ganttRecord.level) {
                        break;
                    } else if (!proxy.getExpandStatus(record)) {
                        removedCount++;
                    }

                }

                proxy._sortedGanttRecords.splice(index + 1, removedCount);

            }
        },

        _addNestedSortGanttRecords: function (record, index) {

            var proxy = this,
                childGanttRecords,
                sortColumns = proxy.model.sortSettings.sortedColumns,
                length;

            if (sortColumns.length) {

                length = sortColumns.length - 1;
                proxy._queryManagar.queries = [];

                for (var i = length; i >= 0; i--) {

                    proxy._queryManagar.sortBy(sortColumns[i].field, sortColumns[i].direction);

                }

            }
            if (record.hasChildGanttRecords) {

                var dataManager = ej.DataManager(record.childGanttRecords),
                    count,
                    childrecord;

                childGanttRecords = dataManager.executeLocal(proxy._queryManagar).result;
                length = childGanttRecords.length;

                for (count = 0; count < length; count++) {

                    childrecord = childGanttRecords[count];

                    if (proxy._filteredGanttRecords.length) {

                        if ($.inArray(childrecord, proxy._filteredGanttRecords) !== -1) {

                            proxy._sortedGanttRecords.splice(++index, 0, childrecord);

                        }

                    } else {

                        proxy._sortedGanttRecords.splice(++index, 0, childrecord);

                    }

                    if (childrecord.hasChildGanttRecords && childrecord.expanded) {

                        proxy._addNestedSortGanttRecords(childrecord, index);

                    }
                }
            }
        },

        processBindings: function (args) {

            var proxy = this,
                model = proxy.model;

            model.query = new ej.Query();

            if (proxy._trigger("actionBegin", args)) {

                return true;

            }

            proxy._ensureDataSource(args);

            if (args.requestType === "save") {

                return proxy._trigger("actionComplete", args);

            }

            //for avoid rerendering of elements
            //if (args.requestType === "delete") {
            //    return proxy._trigger("actionComplete", args);
            //} else {
            proxy.renderGanttRecords(args);
            //}
            return true;
        },

        _processEditing: function () {

            var proxy = this;

            if (proxy.model.edit.editMode == "cellEditing") {

                proxy._addCellEditTemplate();

            }

        },

        //create ParentGrid Records
        _createRecords: function (data) {

            var proxy = this,
                model = proxy.model,
                flatRecords = model.flatRecords,
                count = 0,
                length = data.length,
                parentGanttRecord,
                parentRecords = model.parentRecords,
                enableSorting = model.allowSorting,
                rows = model.records,
                enableAltRow = model.altRow;

            for (count = 0; count < length; count++) {

                parentGanttRecord = proxy._createGanttRecord(data[count], 0, null);
                proxy._storedIndex++;

                if (enableAltRow) {

                    parentGanttRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;

                }

                rows[proxy._storedIndex] = parentGanttRecord;
                flatRecords.push(parentGanttRecord);

                if (enableSorting) {

                    parentRecords.push(parentGanttRecord);

                }

                parentGanttRecord.childGanttRecords && proxy._addNestedGanttRecords(parentGanttRecord.childGanttRecords);

            }
        },

        _addNestedGanttRecords: function (data) {

            var proxy = this,
                model = proxy.model,
                flatRecords = model.flatRecords,
                count = 0,
                length = data.length,
                records = [],
                record,
                enableAltRow = model.altRow,
                rows = model.records;

            for (count; count < length; count++) {

                record = data[count];
                proxy._storedIndex++;

                if (enableAltRow) {

                    record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;

                }

                rows[proxy._storedIndex] = record;
                flatRecords.push(record);
                records = record.childGanttRecords;

                if (records) {

                    var j = 0,
                        recordlength = records.length,
                        childGanttRecord = null;

                    for (j; j < recordlength; j++) {

                        childGanttRecord = records[j];
                        proxy._storedIndex++;

                        if (enableAltRow) {

                            childGanttRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;

                        }

                        rows[proxy._storedIndex] = childGanttRecord;
                        flatRecords.push(childGanttRecord);
                        childGanttRecord.childGanttRecords && proxy._addNestedGanttRecords(childGanttRecord.childGanttRecords);

                    }
                }
            }
        },

        //create GanttRecord
        _createGanttRecord: function (data, level, parentItem, expanded) {

            var proxy = this,
                ganttRecord = {},
                child = data[proxy.model.childMapping];

            ganttRecord = data;
            ganttRecord.Item = data;
            ganttRecord.parentItem = parentItem;
            ganttRecord.isSelected = false,
                ganttRecord.childGanttRecords = child ? proxy._createChildGanttRecords(child, level + 1, data) : [];
            ganttRecord.hasChildGanttRecords = ganttRecord.childGanttRecords.length > 0;
            ganttRecord.expanded = expanded !== undefined ? expanded : child ? child.length > 0 : false;
            ganttRecord.level = level;

            return ganttRecord;
        },

        //create childGanttRecords for particular GanttRecord
        _createChildGanttRecords: function (data, level, parentItem) {

            var ganttRecords = [],
                proxy = this,
                i = 0,
                length = data.length;

            for (i = 0; i < length; i++) {

                var parentRecord = proxy._createGanttRecord(data[i], level, parentItem);
                ganttRecords.push(parentRecord);

            }

            return ganttRecords;
        },

        _createSortedGanttRecords: function (data) {

            var proxy = this,
                sortedGanttRecords = proxy._sortedGanttRecords,
                dataManager,
                enableAltRow = proxy.model.altRow;

            $.each(data, function (index, record) {

                if (proxy._searchString.length > 0) {

                    if (proxy._filteredGanttRecords.length > 0 && proxy._filteredGanttRecords.indexOf(record) !== -1) {

                        proxy._storedIndex++;

                        if (enableAltRow) {

                            record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;

                        }

                        sortedGanttRecords[proxy._storedIndex] = record;

                    }
                } else {

                    proxy._storedIndex++;

                    if (enableAltRow) {

                        record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;

                    }

                    sortedGanttRecords[proxy._storedIndex] = record;

                }

                if (record.hasChildGanttRecords && record.expanded) {

                    dataManager = ej.DataManager(record.childGanttRecords);
                    var childGanttRecords = dataManager.executeLocal(proxy._queryManagar).result;
                    proxy._createSortedGanttRecords(childGanttRecords);

                }
            });
        },

        _renderAfterColumnInitialize: function () {

            var proxy = this;

            proxy.element.append(proxy._renderGridHeader());
        },

        //Method for rendering GridHeader
        _renderGridHeader: function () {

            var proxy = this,
                model = proxy.model,
                doc = document,
                $innerDiv = ej.buildTag('div.e-gridheadercontainer'),
                $table = ej.buildTag('table.e-table', "", {},
                {
                    "cellspacing": "0.25px",
                    "border-spacing": "0px"

                }),
                $thead = ej.buildTag('thead'),
                $tbody = ej.buildTag('tbody.e-hide'),
                $columnHeader = ej.buildTag('tr.e-columnheader'),
                $colGroup = $(doc.createElement('colgroup')),
                $rowBody = $(doc.createElement('tr')),
                columnCount = 0,
                columns = model.columns,
                length = columns.length,
                $headerCell,
                bodyCell,
                $headerCellDiv,
                col,
                $div = ej.buildTag('div.e-gridheader#' + proxy._id + "e-gridheader"),
                browserName = proxy.getBrowserDetails().browser;

            proxy.setGridHeaderContent($div);

            proxy._hiddenColumns = [];

            if (model.allowSorting) {
                $columnHeader.css({ 'cursor': 'pointer' });
            }

            for (columnCount; columnCount < length; columnCount++) {

                $headerCell = ej.buildTag('th.e-headercell', "", { "height": browserName === "opera" ? "49px" : "45px" }, {});
                bodyCell = doc.createElement('td');

                $headerCellDiv = ej.buildTag(
                    'div.e-headercelldiv',
                    columns[columnCount]["headerText"] == undefined ?
                    columns[columnCount]["headerText"] = columns[columnCount]["field"] :
                    columns[columnCount]["headerText"]
                );

                $headerCell.append($headerCellDiv);

                col = doc.createElement('col');

                $rowBody.append(bodyCell);
                $columnHeader.append($headerCell);
                $colGroup.append(col);

                if (columns[columnCount]["visible"] === false) {
                    $headerCell.addClass("e-hide") && $(col).css("display", "none") && proxy._hiddenColumns.push(columns[columnCount].headerText);

                } else {

                    proxy._visibleColumns.push(columns[columnCount].headerText);
                    columns[columnCount]["visible"] = true;

                }

                if (columns[columnCount]["allowSorting"] === false) {
                    proxy._disabledSortableColumns.push(columns[columnCount].headerText);
                    $headerCell.css({ 'cursor': 'normal' });
                }

                if (!ej.isNullOrUndefined(columns[columnCount]["headerTemplateId"])) {

                    $headerCellDiv.html($(columns[columnCount]["headerTemplateId"]).hide().html()).parent().addClass("headertemplate");

                }

                if (columns[columnCount]["width"] == undefined && model.commonWidth !== undefined) {

                    proxy._columnsWidthCollection[columnCount] = model.commonWidth;

                } else {

                    proxy._columnsWidthCollection.push(columns[columnCount]["width"]);
                }

                proxy._fieldColumnNames[columns[columnCount].headerText] = columns[columnCount].field;
                proxy._headerColumnNames[columns[columnCount].field] = columns[columnCount].headerText;

            }
            $thead.append($columnHeader);
            $tbody.append($rowBody);
            $table.append($colGroup).append($thead).append($tbody);

            $innerDiv.html($table);
            $div.html($innerDiv);

            proxy.setGridHeaderTable($table);

            return $div;
        },

        _initGridRender: function () {
            var proxy = this;
            proxy.addInitTemplate();
            proxy.renderGrid();

            if (proxy.model.allowColumnResize) {
                proxy._resizer = new ej.Grid.gridResize(proxy);
            }
        },

        //Template for rendering GridContent
        addInitTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                columnCount = 0,
                length = columns.length,
                columnName,
                td,
                helpers = {};

            if (columns.length === 0) {

                return false;
            }

            helpers["_" + proxy._id + "rowClassName"] = proxy._getrowClassName;
            helpers["_" + proxy._id + "expandStatus"] = $.proxy(proxy._getExpandStatusRecord, proxy);
            helpers["_" + proxy._id + "cellValue"] = proxy._getCellValue;
            helpers["_" + proxy._id + "itemDetails"] = $.proxy(proxy._convertItemToString, proxy);
            helpers["_" + proxy._id + "browserDetails"] = $.proxy(proxy._getRowHeight, proxy);
            $.views.helpers(helpers);

            var tr = "<tr class='e-gridtreerows {{:~_" + proxy._id + "rowClassName()}} " +
                "{{if isAltRow}}e-alt-row{{/if}} " +
                "{{if expanded}}e-gridtreerowexpand{{else hasChildGanttRecords}}e-gridtreerowcollapse{{/if}}' " +
                "style='display:{{:~_" + proxy._id + "expandStatus(#data)}}' role='row'>";

            for (columnCount; columnCount < length; columnCount++) {

                columnName = columns[columnCount].field;

                if (columns[columnCount]["columnTemplate"]) {
                    var templateId = columns[columnCount]["templateId"],
                        template = columns[columnCount]["template"];

                    if (templateId) {
                        td = "<td style='height:{{:~_" + proxy._id + "browserDetails()}};'class='{{if isSelected}}e-rowcell e-selectionbackground e-intend {{else}}e-rowcell e-intend {{/if}}'" +
                            "role='gridcell'tabindex='-1' aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                            $("#" + templateId)[0].innerHTML +
                            //"{{:#data['" + columnName + "']}}"+
                            "</td>";
                    } else if (template) {
                        td = "<td style='height:{{:~_" + proxy._id + "browserDetails()}};'class='{{if isSelected}}e-rowcell e-selectionbackground e-intend {{else}}e-rowcell e-intend {{/if}}'" +
                            "role='gridcell' tabindex='-1' aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                            template +
                            "</td>";
                    }
                } else if (columns[columnCount].mappingName === model.taskNameMapping) {

                    td = "<td style='height:{{:~_" + proxy._id + "browserDetails()}};' class='{{if isSelected}}e-rowcell e-selectionbackground e-intend {{else}}e-rowcell e-intend {{/if}}'" +
                        "role='gridcell' tabindex='-1' aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                        "<div class='intend' style='height:20px;'>" +//class name intend is added to div for css reference for outline problem in chrome
                        "{{if hasChildGanttRecords}}" +
                        "<div class='intend' " +
                        "style='height:1px; float:left; width:{{:level*20}}" + "px; display:inline-block;'>" +
                        "</div>" +
                        "{{else !hasChildGanttRecords}}" +
                        "<div class='intend'" +
                        "style='height:1px; float:left; width:{{:(level+1)*20}}" + "px; display:inline-block;'>" +
                        "</div>" +
                        "{{/if}}" +
                        "<div class='{{if expanded}}e-gridtreeexpand e-icon{{else hasChildGanttRecords}}e-gridtreecollapse e-icon{{/if}}'" +
                        " style='float: left;display:inline-block;'>" +
                        "</div>" +
                        "<div class='e-cell'" +
                        "style='display:inline-block;width:100%'>" +
                        "{{:#data['" + columnName + "']}}" +
                        "</div>" +
                        "</div>" +
                        "</td>";
                } else if (columns[columnCount].mappingName === model.resourceInfoMapping) {

                    helpers["_" + proxy._id + "resourceName"] = proxy._getResourceName;
                    $.views.helpers(helpers);

                    td = "<td  class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}}'" +
                        "role='gridcell' tabindex='-1' aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                        "{{:~_" + proxy._id + "resourceName('" + proxy._id + "Object','" +
                        model.resourceNameMapping + "','" + columnName + "')}}" +
                        "</td>";

                } else if (columns[columnCount].mappingName === model.predecessorMapping) {

                    helpers["_" + proxy._id + "predecessor"] = proxy._getPredecessorsValue;
                    $.views.helpers(helpers);

                    td = "<td style='height:{{:~_" + proxy._id + "browserDetails()}};' class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}}' " +
                        "role='gridcell' tabindex='-1' aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                        "{{:~_" + proxy._id + "predecessor('" + proxy._id + "Object','"
                        + model.predecessorMapping + "','" + columnName + "')}}" +
                        "</td>";

                } else {

                    td = "<td style='height:{{:~_" + proxy._id + "browserDetails()}};' class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}}'" +
                        "role='gridcell' tabindex='-1' aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                        //"{{:#data['" + columnName + "']}}" +
                        "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" +
                        "</td>";

                }

                tr += td;
            }
            tr += "<td class='e-extendcolumn'></td></tr>";

            var templates = {};
            templates[proxy._id + "_Template"] = tr;
            $.templates(templates);

            return true;
        },

        _gridTemplate: function (templateId) {

            return $("#" + templateId).render(this.data);

        },

        _convertItemToString: function (data) {
            var str = '',
                labelObject = {},
                columns = this.model.columns;

            for (var i = 0; i < columns.length; i++) {
                labelObject[columns[i].field] = data.item[columns[i].field];
            }

            for (var p in labelObject) {
                if (labelObject.hasOwnProperty(p)) {
                    str += p + '' + labelObject[p] + '\n';
                }
            }
            return str;
        },

        _getRowHeight: function () {

            var proxy = this,
                model = proxy.model,
                browserName = proxy.getBrowserDetails().browser;

            if (browserName === "safari")
                return (model.rowHeight - 1) + "px";

            else
                return model.rowHeight + "px";
        },

        _getCellValue: function (columnName) {

            var cellValue = this.data[columnName];

            if (cellValue) {
                return cellValue;
            } else {
                return this.data.item && this.data.item[columnName];
            }


        },

        _getResourceName: function (gridObject, mappingName, columnName) {

            var proxy = this,
                data = proxy.data[columnName],
                count = 0,
                length = data && data.length,
                resourceName = [];

            if (data && data.length > 0) {

                for (count; count < length; count++) {

                    resourceName.push(data[count][mappingName]);
                }

            }

            return resourceName;
        },

        _getPredecessorsValue: function (gridObject, mappingName) {

            return this.data.item[mappingName];

        },

        _getExpandStatusRecord: function (data) {

            var proxy = this;

            if (proxy.getExpandStatus(data)) {

                return 'table-row';

            }

            return 'none';
        },

        _getrowClassName: function () {

            var rowClass = "gridrowtaskId",
                proxy = this;

            if (proxy.data.parentItem) {

                rowClass += proxy.data.parentItem.taskId.toString();

            }

            rowClass += "level";
            rowClass += proxy.data.level.toString();

            return rowClass;
        },

        renderGrid: function () {
            var proxy = this;

            proxy._renderGridContent().insertAfter(proxy.element.children(".e-gridheader"));
            proxy._initialEndRendering();

        },

        _renderGridContent: function () {

            var proxy = this,
                doc = document,
                model = proxy.model,
                $tbody = $(doc.createElement('tbody')),
                $div = ej.buildTag("div.e-gridcontent#" + proxy._id + "e-gridcontent", "",
                {
                    "overflow": "auto",
                    "overflow-y": "hidden",
                    "height": proxy._viewPortHeight
                },
                {}),
                $innderDiv = ej.buildTag('div.e-gridcontainer#' + proxy._id + "e-gridcontainer", "", {},
                {});

            if (model.enableVirtualization) {

                $innderDiv.css({
                    'height': proxy._totalHeight
                });

            }

            var $table = ej.buildTag('table.e-table#' + proxy._id + "e-table", "", {
                position: "relative",
                top: "0px"
            },
            {
                'cellspacing': '0px'
            });

            $table.append(proxy.getGridHeaderTable().find('colgroup').clone()).append($tbody);

            $innderDiv.html($table);
            $div.html($innderDiv);

            proxy.setGridContentTable($table);

            if (model.dataSource === null) {

                var $emptyTd = ej.buildTag('td', model.emptyRecordText, {},
                {
                    colSpan: model.columns.length
                });

                $tbody.append($(doc.createElement("tr")).append($emptyTd));

            }

            return $div;
        },

        _initialEndRendering: function () {

            var proxy = this,
                index = proxy.model.selectedRowIndex;

            if (index != -1) {

                if (!proxy._rowSelectingEventTrigger()) {

                    proxy.selectRows(index);
                    proxy._rowSelectedEventTrigger(index);

                }
            }
        },

        _addCellEditTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                columnCount = 0,
                $outerDiv = ej.buildTag('div', "", {
                    display: "none"
                },
                {
                    id: proxy._id + "_CellEditTemplate"
                }),
                $innerDiv,
                length = columns.length;

            for (columnCount; columnCount < length; columnCount++) {


                $innerDiv = ej.buildTag('div', "", {},
                {
                    id: columns[columnCount].field + "_CellEdit"
                });

                proxy._initCellEditType(columnCount, $innerDiv);
                $outerDiv.append($innerDiv);


            }

            if ($outerDiv.children().length) {

                proxy._cellEditTemplate = $outerDiv;

            }
        },

        _initCellEditType: function (columnCount, element) {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                helpers = {};

            if (ej.isNullOrUndefined(columns[columnCount]["columnEditType"])) {

                columns[columnCount]["columnEditType"] = "stringedit";

            }
            helpers["_" + proxy._id + "cellValue"] = proxy._getCellValue;
            $.views.helpers(helpers);
            switch (columns[columnCount]["columnEditType"]) {

                case "stringedit":
                    element.html(ej.buildTag('input.e-field e-ejinputtext', "", {},
                    {
                        value: "{{:~_" + proxy._id + "cellValue('" + columns[columnCount].field + "')}}",
                        id: proxy._id + columns[columnCount].field,
                        name: columns[columnCount].field
                    }));

                    break;

                case "booleanedit":
                    element.html('{{if ' + columns[columnCount].field + '}} ' +
                        '<input class="e-field e-checkbox" type ="checkbox" ' +
                        'id=' + proxy._id + columns[columnCount].field +
                        'name=' + columns[columnCount].field +
                        'checked="checked"></input>' +
                        '{{else}} ' +
                        '<input class="e-field e-checkbox"' +
                        ' type ="checkbox" ' +
                        'id=' + proxy._id + columns[columnCount].field +
                        'name=' + columns[columnCount].field + ' > ' +
                        '{{/if}}');

                    break;

                case "numericedit":

                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {},
                    {
                        type: "text",
                        value: "{{:#data['" + columns[columnCount].field + "']}}",
                        id: proxy._id + columns[columnCount].field,
                        name: columns[columnCount].field
                    });
                    element.append($numericText);

                    break;

                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + columns[columnCount]["columnEditType"] + ' e-field', "", {},
                    {
                        type: "text",
                        value: "{{:#data['" + columns[columnCount].field + "']}}",
                        id: proxy._id + columns[columnCount].field,
                        name: columns[columnCount].field
                    });
                    element.append($datePicker);

                    break;

                case "dropdownedit":
                    var $dropDownList = ej.buildTag('input.e-field e-dropdownlist' + ' e-field', "", {},
                    {
                        type: "text",
                        value: "{{:#data['" + columns[columnCount].field + "']}}",
                        id: proxy._id + columns[columnCount].field,
                        name: columns[columnCount].field
                    });
                    element.append($dropDownList);

                    break;

            }
        },

        _compiledDropDownTemplate: function () {

            var $select = ej.buildTag('select'),
                $option = ej.buildTag("option", "{{:text}}", {},
                {
                    value: "{{:value}}"
                });

            $select.append($option);

            return $.templates($select.html());
        },

        setValidation: function () {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                count = 0;

            proxy._initValidator();

            for (count; count < length; count++) {

                if (!ej.isNullOrUndefined(columns[count]["validationRules"])) {

                    proxy.setValidationToField(columns[count].field, columns[count].validationRules);

                }
            }
        },

        _initValidator: function () {

            var proxy = this;

            $("#" + proxy._id + "EditForm").validate({
                onclick: false,
                onfocusout: false,
                errorClass: 'e-field-validation-error',
                errorElement: 'div',
                wrapper: "div",

                errorPlacement: function (error, element) {
                    var $td = element.closest("td"),
                        $container = $(error).addClass("e-error"),
                        $tail = ej.buildTag("div.e-errortail e-toparrow");

                    $td.find(".e-error").remove();
                    element.hasClass("e-numerictextbox") ? $container.insertAfter(element.closest(".e-numeric")) : $container.insertAfter(element);
                    $container.prepend($tail);
                    proxy.model.edit.editMode != "normal" && $container.offset({
                        left: element.offset().left
                    });

                    $container.fadeIn("slow");
                }
            });
        },

        setValidationToField: function (name, rules) {

            var proxy = this,
                ele = $("#" + proxy._id + name).length > 0 ? $("#" + proxy._id + name) : $("#" + name);
            ele.rules("add", rules);

            var validator = $("#" + proxy._id + "EditForm").validate({
                onsubmit: false

            });

            if (!ej.isNullOrUndefined(rules["required"])) {

                validator.settings.messages[name] = {};
                validator.settings.messages[name]["required"] = proxy.getColumnByField(name).headerText + " is required";

            }
        },

        sendSaveRequest: function () {

            var proxy = this;

            if (proxy.model.edit.editMode == "cellEditing")
                return proxy.saveCell();

            return true;

        },

        sendDeleteRequest: function ($tr) {

            var proxy = this,
                deletedRowIndex,
                deletedRow,
                args = {};


            if (ej.isNullOrUndefined($tr)) {

                $tr = $(proxy.getGridRows()).eq(proxy.model.selectedRowIndex);

            }

            deletedRowIndex = $(proxy.getGridRows()).index($tr);
            deletedRow = proxy.model.currentViewData[deletedRowIndex];

            args.tr = $tr;
            args.data = deletedRow;
            args.requestType = ej.GanttGridTree.actions.Delete;

            if (proxy._trigger("actionBegin", args)) {

                return true;

            }

            return proxy.processBindings(args);

        },


        sendSearchRequest: function (searchString) {

            var proxy = this,
                args = {};

            args.requestType = ej.GanttGridTree.actions.Searching;
            args.keyValue = searchString;

            proxy._searchString = searchString;

            proxy.processBindings(args);
        },

        sendSortingRequest: function (columnName, columnSortDirection) {

            var proxy = this,
                model = proxy.model,
                sortColumns = model.sortSettings.sortedColumns,
                args = {},
                count = 0,
                length = sortColumns.length;

            if (!model.allowSorting || $.inArray(columnName, proxy._disabledSortableColumns) != -1 || columnName.length == 0) {

                return;

            }

            if (!proxy._multiSortRequest) {

                model.sortSettings.sortedColumns = [];

            }

            args.requestType = ej.GanttGridTree.actions.Sorting;
            args.columnName = columnName;

            proxy._cSortedColumn = proxy.getFieldNameByHeaderText(columnName);
            proxy._cSortedDirection = args.columnSortDirection = columnSortDirection === undefined ?
                ej.sortOrder.Ascending : columnSortDirection;

            for (count; count < length; count++) {

                if (sortColumns[count].field == proxy._cSortedColumn) {

                    sortColumns.splice(count, 1);
                    break;

                }

            }

            model.sortSettings.sortedColumns.push({
                field: proxy._cSortedColumn,
                direction: proxy._cSortedDirection

            });

            var returnValue = proxy.processBindings(args);

            if (returnValue) {

                proxy._cSortedDirection = proxy._cSortedColumn = null;

            }
        },

        sendExpandCollapseRequest: function (args) {

            var proxy = this;

            args.requestType = ej.GanttGridTree.actions.ExpandCollapse;

            proxy.processBindings(args);

            if (args.expanded) {

                proxy._trigger("expanded", args);

            } else {

                proxy._trigger("collapsed", args);

            }
        },

        sendDataRenderingRequest: function (args) {

            var proxy = this,
                temp;

            if (proxy.model.currentViewData.length) {

                switch (args.requestType) {
                    case ej.GanttGridTree.actions.Delete:
                    case ej.GanttGridTree.actions.Refresh:
                    case ej.GanttGridTree.actions.Save:
                    case ej.GanttGridTree.actions.Sorting:
                    case ej.GanttGridTree.actions.Searching:
                    case ej.GanttGridTree.actions.ExpandCollapse:
                        temp = document.createElement('div');
                        proxy.getGridContentTable().find("colgroup").first().replaceWith(proxy._getMetaColGroup());
                        var $tbody = proxy.getGridContentTable().children('tbody');
                        $tbody.empty();
                        temp.innerHTML = ['<table>', $.render[proxy._id + "_Template"](proxy.model.currentViewData), '</table>'].join("");
                        proxy.getGridContentTable().get(0).replaceChild(temp.firstChild.firstChild, proxy.getGridContentTable().get(0).lastChild);
                        proxy._gridRows = proxy.getGridContentTable().get(0).rows;
                        proxy._eventBindings();
                        break;
                }

            } else {

                proxy.getGridContentTable().find('tbody').empty().append(proxy._getEmptyTbody());

            }

            proxy._completeAction(args);

        },

        _eventBindings: function () {

            var proxy = this,
                model = proxy.model,
                count = 0,
                gridRows = proxy._gridRows,
                length = gridRows.length;

            if (model.queryCellInfo != null || model.rowDataBound != null) {

                for (count; count < length; count++) {

                    proxy._rowEventTrigger(gridRows[count], model.flatRecords[count]);

                }
            }
        },

        _rowEventTrigger: function (row, data) {

            var proxy = this;

            proxy._trigger("rowDataBound", {
                rowElement: row,
                data: data
            });

            var tdCells = row.cells,
                $tdRowcells = $(row).find(".e-rowcell"),
                cellIndex = 0,
                length = tdCells.length,
                column,
                columns = proxy.model.columns;

            for (cellIndex; cellIndex < length; cellIndex++) {

                if ($(tdCells[cellIndex]).hasClass("e-rowcell")) {

                    column = columns[$tdRowcells.index(tdCells[cellIndex])];

                }

                if (column) {

                    proxy._cellEventTrigger(tdCells[cellIndex], data, column);

                }
            }
        },

        _cellEventTrigger: function (cell, data, column) {

            var args = {
                cellElement: cell,
                data: data,
                column: column,

            };

            args.cellValue = args.column && data[args.column.field];

            this._trigger("queryCellInfo", args);
        },

        selectRows: function (rowIndex, toIndex) {

            var proxy = this,
                model = proxy.model,
                $gridRows = $(proxy.getGridRows()),
                selectedItem;

            if ((model.edit.allowEditing || model.edit.allowAdding) && model.autoSaveOnRowSelection) {

                if ($("#" + proxy._id + "EditForm").length > 0 && proxy.sendSaveRequest()) {

                    return true;

                }

                $gridRows = $(proxy.getGridRows());
            }

            if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(rowIndex)) {

                rowIndex = ej.isNullOrUndefined(rowIndex) ? toIndex : rowIndex;

                if (model.currentViewData && model.currentViewData.length > 0) {

                    switch (model.selectionType) {

                        case ej.GanttGridTree.selectionType.Multiple:

                            if (proxy._multiSelectCtrlRequest) {
                                var selectedRowIndex = $.inArray(rowIndex, proxy._selectedRowsIndexes);

                                if (selectedRowIndex != -1 && cleanUpSelection(rowIndex)) {

                                    proxy._selectedRowsIndexes.splice(selectedRowIndex, 0);

                                }

                                if (selectedRowIndex == -1) {

                                    proxy._selectedRowsIndexes.push(rowIndex);
                                    $($gridRows[rowIndex]).find('.e-rowcell').addClass("e-selectionbackground e-active").attr("tabindex", "0");
                                    $($($gridRows[rowIndex])).attr("aria-selected", "true");
                                    model.currentViewData[rowIndex].isSelected = true;

                                }
                            }

                            break;

                        case ej.GanttGridTree.selectionType.Single:

                            if (rowIndex <= model.currentViewData.length - 1) {

                                proxy.cleanUpSelection();
                                proxy._selectedRowsIndexes = [];
                                proxy._selectedRowsIndexes.push(rowIndex);
                                $($($gridRows[rowIndex]).find('.e-rowcell')).addClass("e-selectionbackground e-active").attr("tabindex", "0");
                                $($($gridRows[rowIndex])).attr("aria-selected", "true");
                                model.currentViewData[rowIndex].isSelected = true;

                            } else {

                                return true;

                            }

                            break;
                    }
                }
            } else {

                proxy.cleanUpSelection();
                proxy._selectedRowsIndexes = [];

                var rows = rowIndex - toIndex < 0 ? $gridRows.slice(rowIndex, toIndex + 1) : $gridRows.slice(toIndex, rowIndex + 1),
                    count = 0,
                    length = rows.length;

                for (count; count < length; count++) {

                    proxy._selectedRowsIndexes.push(rows[count].rowIndex);
                    model.currentViewData[rows[i].rowIndex].isSelected = true;

                }

                $(rows).find('.e-rowcell').addClass("e-selectionbackground e-active").attr("tabindex", "0");
                $(rows).attr("aria-selected", "true");

            }

            proxy._previousIndex = proxy._selectedRowsIndexes.length ? rowIndex : undefined;
            model.selectedRowIndex = rowIndex;
            if (model.currentViewData !== null)// for avoid null error during change the localization after select event.
            {
                selectedItem = model.currentViewData[model.selectedRowIndex];
                model.selectedItem = selectedItem;
            }
            proxy._multiSelectCtrlRequest = false;

            return false;
        },

        _convertToString: function (obj) {
            var str = '',
                labelObject = {};

            for (var i = 0; i < this.model.columns.length; i++) {
                labelObject[this.model.columns[i].mappingName] = obj[this.model.columns[i].mappingName];
            }

            for (var p in labelObject) {
                if (labelObject.hasOwnProperty(p)) {
                    str += p + '' + labelObject[p] + '\n';
                }
            }
            return str;
        },

        _rowSelectingEventTrigger: function () {
            var proxy = this,
                model = proxy.model,
                args = {},
                selectedItem, $gridRows;
            if (model.selectedRowIndex !== -1 && model.currentViewData) {
                $gridRows = $(proxy.getGridRows());
                selectedItem = model.currentViewData[model.selectedRowIndex];
                model.selectedItem = selectedItem;
                args = {
                    rowElement: $gridRows.eq(model.selectedRowIndex),
                    data: selectedItem,
                    recordIndex: model.selectedRowIndex
                };
            }
            return proxy._trigger('rowSelecting', args);
        },

        _rowSelectedEventTrigger: function (index) {
            var proxy = this,
                args = {},
                $gridRows = $(proxy.getGridRows());
            args = {
                rowElement: $gridRows.eq(index),
                data: proxy.model.selectedItem,
                target: "ejGanttGridTree",
                recordIndex: index
            };
            proxy._trigger("rowSelected", args);
        },

        cleanUpSelection: function (index) {

            var proxy = this,
                model = proxy.model;

            if (model.selectedRowIndex != -1 && model.selectedRowIndex <= model.currentViewData.length - 1) {

                var $gridRows = $(proxy.getGridRows());

                if (!ej.isNullOrUndefined(index)) {
                    $($gridRows[index].cells).removeClass("e-selectionbackground").removeClass("e-active").attr("tabindex", "-1");
                    $($gridRows[index]).removeAttr("aria-selected");
                    model.currentViewData[index].isSelected = false;
                    index = $.inArray(index, proxy._selectedRowsIndexes);

                    if (index != -1) {

                        proxy._selectedRowsIndexes.splice(index, 1);

                    }

                } else {

                    $.each($gridRows, function (i, row) {

                        if ($(row.cells[0]).hasClass('e-selectionbackground')) {

                            model.currentViewData[i].isSelected = false;

                        }

                    });

                    $gridRows.find(".e-rowcell").removeClass("e-selectionbackground").removeClass("e-active").attr("tabindex", "-1");
                    $($gridRows).attr("aria-selected", "true");
                }

                if (proxy._selectedRowsIndexes.length == 0) {

                    model.selectedRow = -1;

                }
            }
            return true;
        },

        //set the columnWidth to each column in ganttgridtree
        setWidthToColumns: function () {

            var proxy = this,
                model = proxy.model,
                $cols1 = proxy.getGridContentTable().children("colgroup").find("col"),
                $cols2 = proxy.getGridHeaderTable().children("colgroup").find("col"),
                width = proxy._gridWidth,
                colCount = model.columns.length;

            if ($cols1.length > colCount) $cols1.splice(0, ($cols1.length - colCount));
            if ($cols2.length > colCount) $cols2.splice(0, ($cols2.length - colCount));

            for (var i = 0; i < $cols2.length; i++) {

                if (!ej.isNullOrUndefined(proxy._columnsWidthCollection[i])) {

                    $cols1.eq(i).width(proxy._columnsWidthCollection[i]);
                    $cols2.eq(i).width(proxy._columnsWidthCollection[i]);

                } else {

                    var colWidth = (width / colCount).toFixed(2) + "px";
                    $cols1.eq(i).css("width", colWidth);
                    $cols2.eq(i).css("width", colWidth);
                    proxy._columnsWidthCollection[i] = (width / colCount).toFixed(2);

                }
            }
        },

        //addParentRecord for filtered 0th level items

        _addParentRecord: function (record) {

            var proxy = this;

            if (record.parentItem) {
                proxy._addParentRecord(record.parentItem);

                if ($.inArray(record, proxy._filteredGanttRecords) === -1) {

                    if (proxy.getExpandStatus(record)) proxy._filteredGanttRecords.push(record);

                }
            } else {

                if ($.inArray(record, proxy._filteredGanttRecords) === -1) {

                    if (proxy.getExpandStatus(record)) proxy._filteredGanttRecords.push(record);

                }
            }
        },

        //add sorticon to the columnheader
        _addSortElementToColumn: function (field, direction) {

            var proxy = this,
                column = proxy.getColumnByField(field),
                index = $.inArray(column, proxy.model.columns),
                $headerCellDiv = proxy.getGridHeaderTable().find("thead").find(".e-headercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            $headerCellDiv.append(proxy._createSortElement().addClass("e-" + direction).addClass("e-icon"));
            $headerCellDiv.parent().attr("aria-sort", direction);
        },

        //remove sorticon to the column
        _removeSortElementFromColumn: function (field) {

            var proxy = this,
                column = proxy.getColumnByField(field),
                index = $.inArray(column, proxy.model.columns),
                $headerCellDiv = proxy.getGridHeaderTable().find("thead").find(".e-headercell").eq(index).find(".e-headercelldiv");

            $headerCellDiv.find(".ascending,.descending").remove();
            $headerCellDiv.parent().removeAttr("aria-sort");
        },

        //create the sortElement
        _createSortElement: function () {

            return ej.buildTag('span', "&nbsp;");

        },

        _addEmptyColumntoGrid: function () {

            $('.e-gridheadercontainer table colgroup').append(ej.buildTag('col', "", {
                "width": "auto"
            },
            {}));

            $('.e-gridheadercontainer table thead tr').append(ej.buildTag('th.e-headercell'));
        },

        _completeAction: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns;

            model.isEdit = false;

            if ((ej.GanttGridTree.actions.Sorting == args.requestType && model.allowSorting)
                || (ej.GanttGridTree.actions.Refresh == args.requestType)) {

                if (!proxy._multiSortRequest) {
                    proxy.getGridHeaderTable().find(".e-columnheader").
                        find(".e-headercelldiv").find(".e-ascending,.e-descending").remove();

                    proxy.getGridHeaderTable().find("[aria-sort]").removeAttr("aria-sort");
                }
                for (var i = 0; i < sortcolumns.length; i++) {

                    proxy._addSortElementToColumn(sortcolumns[i].field, sortcolumns[i].direction);

                }

                proxy._multiSortRequest = false;
            } else if (ej.GanttGridTree.actions.BeginEdit == args.requestType || ej.GanttGridTree.actions.Add == args.requestType) {

                var $form = $("#" + proxy._id + "EditForm");

                proxy.model.isEdit = true;
                proxy.setWidthToColumns();

                if (ej.GanttGridTree.actions.Add == args.requestType) {

                    $form.find(".e-field:disabled").not(".e-identity").removeAttr("disabled").removeClass("e-disable");

                }

            }

            proxy._trigger("actionComplete", args);
        },

        //Calculate the viewPortHeight
        getViewPortHeight: function () {

            var proxy = this,
                height = 0;

            height = proxy._gridHeight -
                proxy._$gridHeader.height() -
                parseFloat(proxy._$gridHeader.css("border-top-width")) -
                parseFloat(proxy._$gridHeader.css("border-bottom-width"));

            return height;
        },

        //Count the number of child records present in the ganttRecord

        _getFilteredChildRecordsCount: function (ganttRecord) {
            var proxy = this,
                length = ganttRecord.childGanttRecords && ganttRecord.childGanttRecords.length,
                i = 0,
                record;

            for (i = 0; i < length; i++) {
                record = ganttRecord.childGanttRecords[i];
                proxy._removedCount++;
                if (record.childGanttRecords) {
                    proxy._getFilteredChildRecordsCount(record);
                }
            }

            return proxy._removedCount;
        },

        //Count the childrecords of parentItem
        getChildGanttRecordsCount: function (record) {

            var proxy = this,
                childGanttRecords = record.childGanttRecords,
                count = 0,
                length = childGanttRecords ? childGanttRecords.length : 0;

            for (count; count < length; count++) {

                if (proxy._filteredGanttRecords.length > 0) {

                    var recordIndex = proxy._filteredGanttRecords.indexOf(childGanttRecords[count]);

                    if (recordIndex != -1) {

                        ++proxy._removedCount;

                        if (childGanttRecords[count].hasChildGanttRecords && childGanttRecords[count].expanded) { proxy.getChildGanttRecordsCount(childGanttRecords[count]); }

                    }
                } else {

                    ++proxy._removedCount;

                    if (childGanttRecords[count].hasChildGanttRecords && childGanttRecords[count].expanded) {
                        proxy.getChildGanttRecordsCount(childGanttRecords[count]);
                    }

                }
            }

            return proxy._removedCount;
        },

        //return the column field Names
        getColumnFieldNames: function () {

            var columnNames = [],
                columns = this.model.columns,
                count = 0,
                length = columns.length;

            for (count; count < length; count++) {

                if (columns[count]["field"]) columnNames.push(columns[count]["field"]);

            }

            return columnNames;
        },

        getFormatedDate: function (date) {

            return Globalize.format(date, this.model.dateFormat);

        },

        //get the Expand status of GridRecord
        getExpandStatus: function (record) {

            var parentRecord = record.parentItem;

            if (parentRecord != null) {

                if (parentRecord.expanded === false) {
                    return false;
                } else if (parentRecord.parentItem) {

                    if (parentRecord.parentItem.expanded === false) {
                        return false;
                    } else {
                        return this.getExpandStatus(parentRecord.parentItem);
                    }

                } else return true;
            } else return true;
        },

        //calculate the range of records to display in the current view port
        getVisibleRange: function () {

            var proxy = this,
                top = proxy._scrollTop / proxy.model.rowHeight,
                coeff = top - Math.floor(top),
                topIndex, bottomIndex,
                model = proxy.model;

            proxy._offset = coeff * model.rowHeight;
            topIndex = Math.floor(top);
            bottomIndex = proxy._getRowPosition(proxy._scrollTop + proxy._viewPortHeight);
            topIndex = Math.max(0, topIndex);
            bottomIndex = Math.min(proxy.getRecordsCount(), bottomIndex);

            proxy._visibleRange = {
                top: topIndex,
                bottom: bottomIndex
            };

            return proxy._visibleRange;

        },

        //get the FieldName of column by using headerText
        getFieldNameByHeaderText: function (headerText) {

            var proxy = this;

            if (ej.isNullOrUndefined(proxy._fieldColumnNames[headerText])) return null;

            return proxy._fieldColumnNames[headerText];
        },

        //get the column by using field name
        getColumnByField: function (field) {

            var columns = this.model.columns,
                column = 0;

            for (column; column < columns.length; column++) {

                if (columns[column]["field"] == field) break;

            }

            return column == columns.length ? null : columns[column];
        },

        //get the columnIndex by using field Name
        getColumnIndexByField: function (field) {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                column = 0;

            for (column; column < length; column++) {

                if (columns[column]["field"] == field) break;

            }

            return column;
        },

        //get the browser details
        getBrowserDetails: function () {
            var b = navigator.userAgent.match(/(firefox|chrome|opera|msie|safari)\s?\/?(\d+(.\d+)*)/i);
            if (!!navigator.userAgent.match(/Trident\/7\./))
                return { browser: "msie", version: jQuery.uaMatch(navigator.userAgent).version };
            return { browser: b[1].toLowerCase(), version: b[2] };
        },

        //get the ganttRecord position
        _getRowPosition: function (y) {
            return Math.ceil((y) / this.model.rowHeight);
        },

        //get columgroup of the headerTable
        _getMetaColGroup: function () {

            var $colgroup = this.getGridHeaderTable().find('colgroup').clone();
            return $colgroup;

        },

        //create empty row while the datasource object as null
        _getEmptyTbody: function () {

            var proxy = this,
                $emptyTd = ej.buildTag('td', proxy.model.emptyRecordText, {},
            {
                colSpan: proxy.model.columns.length
            });

            return $(document.createElement("tr")).append($emptyTd);
        },

        //Update the sortedGanttRecords:
        _updateSortedGanttRecords: function (data) {

            var proxy = this,
                count = 0,
                length = data.length,
                record;

            for (count; count < length; count++) {

                record = data[count];

                if (proxy.getExpandStatus(record)) {

                    if (proxy._filteredGanttRecords.length > 0) {

                        if ($.inArray(record, proxy._filteredGanttRecords) !== -1 && $.inArray(record, proxy._sortedGanttRecords) === -1) {

                            proxy._sortedGanttRecords.push(record);

                        }

                    } else {

                        if ($.inArray(record, proxy._sortedGanttRecords) === -1) proxy._sortedGanttRecords.push(record);

                    }

                    if (record.hasChildGanttRecords && record.childGanttRecords.length > 0 && record.expanded) {

                        var dataManager = ej.DataManager(record.childGanttRecords),
                            sortedChildGanttRecords = dataManager.executeLocal(proxy._queryManagar).result;
                        proxy._updateSortedGanttRecords(sortedChildGanttRecords);

                    }
                }
            }
        },

        //Update the filteredGanttRecords for adding ParentRecords.
        _updateFilteredGanttRecords: function (data) {

            var proxy = this,
                count = 0,
                length = data.length,
                record;

            proxy._filteredGanttRecords = [];

            for (count; count < length; count++) {

                record = data[count];

                if (record.parentItem != null) {

                    proxy._addParentRecord(record.parentItem);

                    if ($.inArray(record, proxy._filteredGridRecords) === -1) {
                        if (proxy.getExpandStatus(record)) proxy._filteredGanttRecords.push(record);
                    }

                } else {

                    if ($.inArray(record, proxy._filteredGridRecords) === -1) {
                        if (proxy.getExpandStatus(record)) proxy._filteredGanttRecords.push(record);
                    }
                }
            }
        },

        //Update the ExpandCollapsed records
        updateExpandCollapseGanttRecords: function (record, expand, index) {

            var proxy = this;

            record.expanded = expand;
            proxy._removedCount = 0;
            proxy._expandCollapseRecordCount = proxy.getChildGanttRecordsCount(record);

            if (expand) {

                proxy._tempRecordIndex = proxy.model.records.indexOf(record);

                if (record.hasChildGanttRecords) {
                    proxy._addNestedExpandCollapseGanttRecords(record.childGanttRecords, record.level, record, index);
                }

            } else {

                proxy.model.flatRecords.splice(index + 1, proxy._expandCollapseRecordCount);

            }
        },

        //add the expanded childGanttRecords to FlatRecords
        _addNestedExpandCollapseGanttRecords: function (data, level, parentItem, index) {

            var proxy = this,
                model = proxy.model,
                ganttRecords = model.flatRecords,
                rows = model.records,
                count = 0,
                length = data.length,
                parentGanttRecord;


            for (count; count < length; count++) {

                index++;
                proxy._tempRecordIndex++;
                parentGanttRecord = rows[proxy._tempRecordIndex];

                if ($.inArray(parentGanttRecord, ganttRecords) === -1) ganttRecords.splice(index, 0, parentGanttRecord);

                if (parentGanttRecord.isExpanded === false) parentGanttRecord.isExpanded = true;

                if (parentGanttRecord.hasChildGanttRecords) {

                    if (parentGanttRecord.expanded) {

                        var len = parentGanttRecord.childGanttRecords.length,
                            k = 0,
                            childGanttRecord;

                        for (k; k < len; k++) {

                            index++;
                            proxy._tempRecordIndex++;
                            childGanttRecord = rows[proxy._tempRecordIndex];
                            ganttRecords.splice(index, 0, childGanttRecord);

                            if (childGanttRecord.hasChildGanttRecords) {

                                if (childGanttRecord.expanded) {
                                    proxy._addNestedExpandCollapseGanttRecords(childGanttRecord.childGanttRecords, level + 1, childGanttRecord, index);
                                } else proxy._tempRecordIndex += childGanttRecord.childGanttRecords.length;

                            }
                        }
                    } else proxy._tempRecordIndex += parentGanttRecord.childGanttRecords.length;
                }
            }
        },

        //update the currentViewData from flatRecords/updatedRecords
        _updateCurrentViewData: function () {

            var proxy = this,
                model = proxy.model,
                args = {},
                tMargin,
                isRangeModified = false;

            if (model.enableVirtualization) {

                proxy.getVisibleRange();
                model.currentViewData = proxy._updatedRecords.slice(proxy._visibleRange.top, proxy._visibleRange.bottom);

                if (proxy._vScrollDist !== 0) {

                    tMargin = proxy._scrollTop - proxy._offset;

                    if ((proxy._prevRTop - proxy._visibleRange.top !== 0) || (proxy._prevRBottom - proxy._visibleRange.bottom !== 0)) {

                        args.requestType = ej.GanttGridTree.actions.Refresh;
                        proxy.sendDataRenderingRequest(args);
                        isRangeModified = true;

                    }

                    proxy._$tableContent.css({
                        "top": tMargin
                    });

                    proxy._prevRTop = proxy._visibleRange.top;
                    proxy._prevRBottom = proxy._visibleRange.bottom;

                }

            } else {

                model.currentViewData = proxy._updatedRecords;

            }

            if (proxy._isEnterKeyPressed) {

                proxy._isEnterKeyPressed = false;
                if (model.allowSelection && model.edit.allowEditing) {

                    if (!isRangeModified) proxy._cellEditingDetails.rowIndex += 1;

                    if (!proxy._rowSelectingEventTrigger()) {

                        proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                        proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);

                    }

                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, proxy.model.columns[proxy._cellEditingDetails.columnIndex].field);

                }
            }
        },

        //update the gridcontainer Height
        _updateHeight: function () {

            var proxy = this;

            if (this.model.enableVirtualization) {

                proxy._$gridContainer.css({
                    "height": proxy._totalHeight + "px"
                });

            }
        },

        //update the scrollTop of the gridContainer
        _updateScrollTop: function (y) {

            var args = {},
                proxy = this;

            args.requestType = "scroll";
            args.delta = y;

            proxy._completeAction(args);
            proxy._$gridContent.scrollTop(y);
        },

        _wireEvents: function () {

            var proxy = this,
                model = proxy.model;

            proxy._$gridContent.bind("scroll", $.proxy(proxy._onScroll, proxy));
            proxy._on(proxy.element, "click", proxy._onClick);
            proxy._on(proxy.element, "click", "#" + proxy._id + "e-gridheader", proxy._onHeaderClick);
            proxy._on(proxy.element, "mousedown", "#" + proxy._id + "e-gridheader", proxy._headerMouseDown);

            if (model.allowColumnResize) {
                proxy._on(proxy.element, "mousemove", proxy._mouseMove);
                proxy._on(proxy.element, "mouseup", proxy._mouseUp);
            }

            proxy._on(proxy.element, "mousewheel DOMMouseScroll", proxy._mouseWheel);

            proxy._enableEditingEvents();
        },


        _enableRowHover: function () {

            var proxy = this;

            if (proxy.model.enableRowHover) {

                proxy._on(proxy.element, "mouseenter mouseleave", "#" + proxy._id + "e-gridcontent tr td", proxy._rowHover);

            } else {

                proxy._off(proxy.element, "mouseenter mouseleave", "#" + proxy._id + "e-gridcontent tr td");

            }
        },

        _enableEditingEvents: function () {

            var proxy = this,
                model = proxy.model;

            if (model.edit.allowEditing) {

                if (model.edit.editMode == "cellEditing") {

                    proxy._on(proxy.element, "mousedown", ".e-gridcontent", proxy._saveCellHandler);

                }
                proxy._on(this.element, "dblclick", ".e-gridcontent", proxy._editdblClickHandler);
            }
        },

        //key pressed event for key board interaction in gridtree control
        _keyPressed: function (action, target, e) {

            var proxy = this,
                $gridtreeEle = proxy.element,
                gridObject = $gridtreeEle.ejGanttGridTree("instance"),
                model = gridObject.model;

            //check the allowkeyboardNavigation
            if (!model.allowKeyboardNavigation ||
                (target.tagName == 'INPUT' && e.code != 40 && e.code != 38 && e.code != 13 && e.code != 27 && e.code != 9)) {
                return true;
            }

            var returnValue = false,
                $target = $(target);

            switch (action) {

                //save the current edited cellValue on Enter key press
                case "saveRequest":
                    if (model.edit.editMode == "cellEditing") {
                        gridObject._moveCurrentCell("down");
                    }

                    break;

                    //cancel the cell editing in the current cell Esc key press
                case "cancelRequest":
                    if (model.edit.editMode == "cellEditing") {
                        gridObject.cancelEditCell();
                    }

                    break;

                    //edit the cell in gridtree by F2 key press
                case "editRecord":
                    if (model.edit.editMode == "cellEditing") {

                        gridObject._cellEditingDetails.rowIndex = model.selectedRowIndex;
                        gridObject._cellEditingDetails.columnIndex = proxy.getCellIndex(e, target);
                        fieldName = gridObject.model.columns[gridObject._cellEditingDetails.columnIndex].field;
                        fieldName && gridObject.cellEdit(gridObject._cellEditingDetails.rowIndex, fieldName);

                    }

                    break;

                    //select the first row while press the Home key
                case "firstRowSelection":

                    if (!proxy._rowSelectingEventTrigger()) {

                        proxy.selectRows(0);

                        recordIndex = proxy._updatedRecords.indexOf(model.selectedItem);
                        rowTop = recordIndex !== -1 && recordIndex * model.rowHeight;
                        scrollTop = proxy._$gridContent.scrollTop();

                        if (scrollTop > rowTop) {

                            args = {
                                requestType: "scroll",
                                delta: rowTop
                            };

                            proxy._completeAction(args);
                            proxy._$gridContent.scrollTop(rowTop);

                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                            }
                        }

                        proxy._rowSelectedEventTrigger(0);
                    }

                    break;

                case "lastRowSelection":

                    if (!proxy._rowSelectedEventTrigger()) {

                        lastRowIndex = proxy._updatedRecords.length - 1;
                        proxy.selectRows(lastRowIndex);

                        recordIndex = model.selectedItem && proxy._updatedRecords.indexOf(model.selectedItem);
                        rowTop = lastRowIndex !== -1 && recordIndex * model.rowHeight;

                        if (rowTop + proxy.model.rowHeight >= proxy._viewPortHeight) {

                            scrollTop = rowTop + proxy.model.rowHeight - (proxy._viewPortHeight - proxy._scrollBarHeight);

                            args = {
                                requestType: "scroll",
                                delta: scrollTop
                            };

                            proxy._completeAction(args); //actionComplete event trigger for perform the scrolling gantt chartcontrol.

                            proxy._$gridContent.scrollTop(scrollTop);

                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                            }

                        }

                        proxy._rowSelectedEventTrigger(lastRowIndex);
                    }

                    break;

                    //set the edit mode to previous cell by press shift +tab key
                case "moveCellLeft":

                    if (model.edit.editMode == "cellEditing") {
                        returnValue = proxy._moveCurrentCell("left");
                    }

                    break;

                    //set the edit mode to next cell by tab key
                case "moveCellRight":
                    if (model.edit.editMode == "cellEditing" && $target) {
                        returnValue = proxy._moveCurrentCell("right");
                    }

                    break;

                    //select the next row by press down arrow key
                case "downArrow":
                    var lastRowIndex = proxy._updatedRecords.length - 1; //get the last row index of the gridtree control

                    if (proxy.model.selectedRowIndex != lastRowIndex) {

                        proxy.selectRows(proxy.model.selectedRowIndex + 1);

                        var recordIndex = model.selectedItem && proxy._updatedRecords.indexOf(model.selectedItem),
                            rowTop = lastRowIndex !== -1 && recordIndex * model.rowHeight;

                        //check if the selected row present in  out of view port of the grid tree
                        if (rowTop + proxy.model.rowHeight >= proxy._viewPortHeight) {
                            var scrollTop = rowTop + proxy.model.rowHeight - (proxy._viewPortHeight - proxy._scrollBarHeight),
                                args = {
                                    requestType: "scroll",
                                    delta: scrollTop
                                };

                            //actionComplete event trigger for perform the scrolling gantt chartcontrol.
                            proxy._completeAction(args);

                            proxy._$gridContent.scrollTop(scrollTop);

                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                            }

                        }

                        proxy._rowSelectedEventTrigger(proxy.model.selectedRowIndex);
                    }

                    break;

                    //select the previous row by press down arrow key
                case "upArrow":

                    if (proxy.model.selectedRowIndex !== 0) {
                        proxy.selectRows(proxy.model.selectedRowIndex - 1);
                    }

                    recordIndex = proxy._updatedRecords.indexOf(model.selectedItem);
                    rowTop = recordIndex !== -1 && recordIndex * model.rowHeight;

                    scrollTop = proxy._$gridContent.scrollTop();

                    if (scrollTop > rowTop) {

                        args = {
                            requestType: "scroll",
                            delta: rowTop
                        };

                        proxy._completeAction(args);
                        proxy._$gridContent.scrollTop(rowTop);
                        if (model.enableVirtualization) {
                            proxy.element[0].focus();
                        }
                    }

                    proxy._rowSelectedEventTrigger(proxy.model.selectedRowIndex);

                    break;

                default:
                    returnValue = true;
            }

            return returnValue;
        },

        onScrollHelper: function (scrolltop) {
            var proxy = this;
            proxy._$gridContent.scrollTop(scrolltop);
        },

        _onScroll: function () {

            var proxy = this,
                hScrollDist;

            proxy._scrollLeft = proxy._$gridContent.scrollLeft();
            hScrollDist = Math.abs(proxy._scrollLeft - proxy._prevScrollLeft);

            if (hScrollDist) {

                proxy._$gridHeader.scrollLeft(proxy._scrollLeft);
                proxy._prevScrollLeft = proxy._scrollLeft;

            }

            if (proxy.model.enableVirtualization) {

                proxy._scrollTop = proxy._$gridContent.scrollTop();
                proxy._vScrollDir = proxy._prevScrollTop <= proxy._scrollTop ? 1 : -1;
                proxy._vScrollDist = Math.abs(proxy._scrollTop - proxy._prevScrollTop);

                if (proxy._vScrollDist) {

                    proxy._updateCurrentViewData();
                    proxy._prevScrollTop = proxy._scrollTop;

                }
            }
        },

        //click Handler for perform rowSelection and Expand Collapse the GanttRecord
        _onClick: function (e) {

            var proxy = this,
                model = proxy.model,
                $target = $(e.target);

            if ($target.hasClass('e-gridtreeexpand') || $target.hasClass("e-gridtreecollapse")) {

                var $tr = $target.closest('tr'),
                    $gridRows = $(proxy._gridRows),
                    rowIndex = $gridRows.index($tr),
                    record = model.currentViewData && model.currentViewData[rowIndex],
                    recordIndex = model.flatRecords.indexOf(record),
                    args = {},
                    isExpandCollapseEnabeled;

                args.data = record;
                args.recordIndex = recordIndex;
                args.expanded = $tr.hasClass("e-gridtreerowexpand") ? false : true;

                if (args.expanded) {

                    isExpandCollapseEnabeled = proxy._trigger("expanding", args);

                } else {

                    isExpandCollapseEnabeled = proxy._trigger("collapsing", args);

                }

                if (!isExpandCollapseEnabeled && model.enableVirtualization) {

                    //perform expand collapse in the particular GanttRecord
                    proxy.sendExpandCollapseRequest(args);

                }

            } else if ($target.hasClass("e-rowcell") || $target.parent().hasClass("e-rowcell")
                || ($target.closest("td").hasClass("e-rowcell")) && $target.hasClass("e-cell")) {

                if (!model.allowSelection) return;

                if (model.selectionType == ej.GanttGridTree.selectionType.Multiple) {

                    if (e.ctrlKey) proxy._multiSelectCtrlRequest = true;

                    if (e.shiftKey) {

                        proxy._multiSelectShiftRequest = true;
                        proxy.selectRows(proxy._previousIndex, $(proxy.getGridRows()).index($target.closest('tr')));

                    }
                }

                if (!proxy._multiSelectShiftRequest) {

                    recordIndex = $(proxy.getGridRows()).index($target.closest('tr'));

                    if (recordIndex != -1) {

                        record = model.currentViewData[recordIndex];
                        model.selectedItem = record;
                        model.selectedRowIndex = recordIndex;

                        if (!proxy._rowSelectingEventTrigger()) {

                            proxy.selectRows(recordIndex);
                            proxy._rowSelectedEventTrigger(recordIndex);

                        }
                    }
                }

                proxy._multiSelectShiftRequest = false;
            }
        },

        _editdblClickHandler: function (e) {

            var proxy = this,
                model = proxy.model,
                $form = $("#" + proxy._id + "EditForm"),
                args = {};

            if (model.edit.editMode === "normal") {
                args.requestType = ej.GanttGridTree.actions.BeginEdit;
                proxy._trigger("actionBegin", args);
            }

            if (proxy._cellEditingDetails.cancelSave) {

                proxy._cellEditingDetails.cancelSave = false;
                return;

            }

            if ($form.length === 0) {

                model.edit.editMode == "cellEditing" && proxy.element.focus();
                proxy._cellEditingDetails.columnIndex = proxy.getCellIndex(e);
                proxy._cellEditingDetails.rowIndex = proxy.getRowIndex(e);
                fieldName = proxy._cellEditingDetails.columnIndex >= 0 && model.columns[proxy._cellEditingDetails.columnIndex].field;
                if (fieldName) {
                    model.edit.allowEditing &&
                        model.edit.editMode == ej.GanttGridTree.editMode.CellEditing &&
                        proxy.cellEdit(proxy._cellEditingDetails.rowIndex, fieldName);
                }
            }
        },

        _onHeaderClick: function (e) {

            var proxy = this,
                model = proxy.model;

            if ($(e.target).is(".e-ascending, .e-descending")) e.target = e.target.parentNode;

            var $target = $(e.target),
                columnName, columnSortDirection;

            proxy.getGridHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive");

            if ($target.hasClass("e-headercelldiv")) {

                if (!model.allowSorting) return false;

                columnName = $.trim($target.text());

                if ($target.find('span').hasClass("e-ascending")) {
                    columnSortDirection = ej.sortOrder.Descending;
                } else {
                    columnSortDirection = ej.sortOrder.Ascending;
                }

                if (model.allowMultiSorting && e.ctrlKey) {
                    proxy._multiSortRequest = true;
                }

                proxy.sendSortingRequest(columnName, columnSortDirection);
            }

            return false;
        },

        _saveCellHandler: function (e) {

            var $target = $(e.target),
                proxy = this;

            e.stopPropagation();

            if ($target.closest(".e-popup").length == 0 &&
                $target.closest(".e-rowcell").find("#" + proxy._id + "EditForm").length == 0) {

                proxy.saveCell();

            }
        },

        //cancel the editMode of the cell
        cancelEditCell: function () {

            var proxy = this,
                tr = proxy.getGridRows()[proxy._cellEditingDetails.rowIndex],
                cellData = {},
                cell;

            cellData[proxy._cellEditingDetails.fieldName] = proxy._cellEditingDetails.cellValue,
            data = proxy.model.selectedItem;

            if ($(tr).hasClass("e-insertedrow")) cell = tr.cells[proxy._cellEditingDetails.columnIndex];
            else cell = tr.cells[proxy._cellEditingDetails.columnIndex];

            $(cell).removeClass("e-editedcell").empty().html(
                $($.render[proxy._id + "_Template"](data))[0].cells[proxy._cellEditingDetails.columnIndex].innerHTML
            );

            proxy.element.focus();
            proxy._trigger("cancelEditCell");
        },

        //reRendered the edited Row
        refreshRow: function (index) {
            var proxy = this,
                $tr = $(proxy.getGridRows()[index]),
                data = proxy.model.currentViewData[index];

            if (data) $tr.replaceWith($.render[proxy._id + "_Template"](data));
        },

        //render the added new record
        renderRecord: function (args) {
            var proxy = this,
                addedGridRow = $.render[this._id + "_Template"](args._cAddedRecord);

            if (args.recordIndex == 0) {
                $(proxy._gridRows).eq(args.recordIndex).before(addedGridRow);
            } else {
                $(proxy._gridRows).eq(args.recordIndex).after(addedGridRow);
            }

            if (proxy._gridRows == null) {
                proxy._gridRows = addedGridRow;
            }

        },

        //save the edited cell
        saveCell: function () {

            var proxy = this,
                model = proxy.model;

            if ($("#" + proxy._id + "EditForm").length > 0) {

                if (!proxy.editFormValidate()) return true;

                if (proxy._cellEditingDetails.columnIndex >= 0) {

                    var $form = $("#" + proxy._id + "EditForm"),
                        $targetTr = $form.closest("tr"),
                        $targetTd = $form.closest("td"),
                        formattedValue, args = {},
                        column = model.columns[proxy._cellEditingDetails.columnIndex],
                        $element = $("#" + proxy._id + proxy._cellEditingDetails.fieldName),
                        cellElement;

                    args = {
                        columnName: column.field,
                        value: proxy.getCurrentEditCellData(),
                        data: proxy._cellEditingDetails.data,
                        previousValue: proxy._cellEditingDetails.cellValue,
                        columnObject: column,
                        cellElement: $targetTd,
                        rowElement: $targetTr,
                    };

                    if (proxy._cellEditingDetails.cellEditType == "datetimepicker" ||
                        proxy._cellEditingDetails.cellEditType == "dropdownedit" ||
                        proxy._cellEditingDetails.cellEditType == "datepicker") {

                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
                        args.value = proxy.getFormatedDate(args.value);

                    }

                    if (!ej.isNullOrUndefined(column.format)) {

                        formattedValue = proxy.formatting(column.format, args.value);
                        args.cellElement.empty().html(formattedValue);

                    } else {

                        if (args.columnObject.type == "boolean") {

                            var cellData = {};
                            cellData[args.columnObject.field] = args.value;
                            args.cellElement.empty().html(
                                $($.templates.Grid_JSONTemplate.render(cellData))[0].cells[proxy._cellEditingDetails.columnIndex].innerHTML
                            );

                        } else args.cellElement.empty().html(args.value);
                    }

                    var isValueModified = ((proxy._cellEditingDetails.cellEditType == "datepicker" ||
                            proxy._cellEditingDetails.cellEditType == "datetimepicker") &&
                        args.value instanceof Date && args.previousValue instanceof Date) ?
                        (args.value.getTime() !== args.previousValue.getTime()) :
                        (args.value !== args.previousValue);

                    model.isEdit = false;

                    var mappingName = proxy._getMappingName(proxy._cellEditingDetails["fieldName"]);

                    if (isValueModified || mappingName === model.predecessorMapping) {

                        var ganttPredecessor = [],
                            prevPredecessors = [],
                            modifiedPredecessors = [],
                            count = 0,
                            length, predecessor;

                        if (mappingName === model.predecessorMapping) {

                            args.previousValue = args.data.predecessor;
                            prevPredecessors = args.previousValue;

                            if (args.value.length > 0) {
                                modifiedPredecessors = args.data._calculatePredecessor(args.value);
                            }

                            length = modifiedPredecessors.length;

                            var index = 0,
                                prevPrdecessorlength = args.previousValue && args.previousValue.length;

                            for (index = 0; index < prevPrdecessorlength; index++) {

                                predecessor = prevPredecessors[index];

                                if (predecessor.from === args.data.taskId.toString()) {
                                    ganttPredecessor.push(predecessor);
                                }

                            }

                            for (count = 0; count < length; count++) {

                                ganttPredecessor.push(modifiedPredecessors[count]);

                            }

                            args.data[proxy._cellEditingDetails["fieldName"]] = ganttPredecessor;
                            args.data.item[mappingName] = args.value;

                        } else {
                            if (mappingName) {
                                args.data[proxy._cellEditingDetails["fieldName"]] = args.value;
                                args.data.item[mappingName] = args.value;
                            } else {
                                args.data.item[proxy._cellEditingDetails["fieldName"]] = args.value;
                            }
                        }

                        proxy._cellEditingChanges.changed = args.data;
                    }

                    if (args.columnName === "startDate" ||
                        args.columnName === "endDate" ||
                        args.columnName === "duration" ||
                        args.columnName === "predecessor" ||
                        args.columnName == "status") {

                        args.isModified = true;

                    }

                    if (proxy._cellEditingDetails.columnIndex === 0) {

                        cellElement = $($.render[proxy._id + "_Template"](args.data))[0].cells[proxy._cellEditingDetails.columnIndex];
                        args.cellElement.removeClass("e-editedcell").replaceWith(cellElement);

                    } else {

                        cellElement = $($.render[proxy._id + "_Template"](args.data))[0].cells[proxy._cellEditingDetails.columnIndex];
                        args.cellElement.removeClass("e-editedcell").replaceWith(cellElement);

                    }
                    proxy._trigger("endEdit", args);
                    proxy._cellEventTrigger(args.rowElement[0].cells[proxy._cellEditingDetails.columnIndex], args.data, args.columnObject);
                    $targetTr.removeClass("e-editedrow");
                }
            }

            return false;
        },

        //validate the modified cellValue
        editFormValidate: function () {

            if ($.isFunction($.validator))
                return $("#" + this._id + "EditForm").validate({
                    onsubmit: false
                }).form();

            return true;
        },

        //get the currentCellData while perform editing operation

        getCurrentEditCellData: function () {

            var proxy = this,
                model = proxy.model;

            if ($("#" + proxy._id + "EditForm").length) {

                var $element = $("#" + proxy._id + proxy._cellEditingDetails.fieldName),
                    cellValue;

                switch (proxy._cellEditingDetails.cellEditType) {

                    case ej.GanttGridTree.editingType.String:
                        cellValue = $element.val();
                        break;

                    case ej.GanttGridTree.editingType.Numeric:
                        cellValue = $element.ejNumericTextbox("getValue");
                        $element.ejNumericTextbox("getValue");
                        break;

                    case ej.GanttGridTree.editingType.Dropdown:
                        cellValue = proxy._getSelectedItem($element.ejDropDownList("model.multiSelectedItemsIndex"));
                        break;

                    case ej.GanttGridTree.editingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.GanttGridTree.editingType.DatePicker:
                        cellValue = $element.ejDatePicker("model.value");
                        break;

                    case ej.GanttGridTree.editingType.DateTimePicker:
                        cellValue = $element.ejDateTimePicker("model.value");
                        break;

                }
                if (typeof cellValue == "string" && cellValue.length &&
                    model.columns[proxy._cellEditingDetails.columnIndex].type == "number") {

                    cellValue = parseFloat(cellValue);

                }

                return cellValue;
            }

            return null;
        },

        //get the selectedItem of DropDownListControl for resourceInfo column

        _getSelectedItem: function (indexArray) {

            var proxy = this,
                count = 0,
                length = indexArray.length,
                dropDownData = proxy.model.columns[proxy._cellEditingDetails.columnIndex].dropdownData,
                selectedItems = [];

            if (dropDownData) {

                for (count = 0; count < length; count++) {

                    $.each(dropDownData, function (index, resourceInfo) {
                        if (indexArray[count] === index) {
                            selectedItems.push(resourceInfo);
                        }
                    });

                }
            }

            return selectedItems;
        },

        _focusElements: function ($currentCell) {

            if ($currentCell.length) {

                $currentCell.focus();
                var $childElem = $currentCell.children();

                if ($childElem[0]) {

                    if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {

                        $childElem.focus().select();
                        $childElem[0].focus();

                    } else if ($childElem.find(".e-field.e-dropdownlist").length) {

                        $childElem.find(".e-ddl").focus();

                    } else {

                        $childElem.find('input,select').select().focus();

                    }
                }
            }
        },

        formatting: function (formatstring, str) {

            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");

            var proxy = this,
                s = formatstring,
                frontHtmlidx, frontHtml, rearHtml, lastidxval;

            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            frontHtml = frontHtmlidx[0];
            rearHtml = lastidxval[1];

            if (formatstring.indexOf("{0:") != -1) {

                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm"),
                    formatVal = toformat.exec(formatstring);

                if (formatVal != null && str != null) {

                    if (frontHtml != null && rearHtml != null) str = frontHtml + Globalize.format(str, formatVal[2]) + rearHtml;
                    else str = Globalize.format(str, formatVal[2]);

                } else if (str != null) str = str;
                else str = "";

                return str;
            } else if (proxy.data != null && proxy.data.Value == null) {

                $.each(proxy.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });

                return s;

            } else return proxy.data.Value;

        },

        //enter editMode to the cell
        cellEdit: function (index, fieldName) {

            var proxy = this,
                $form;

            proxy.model.isEdit && proxy.saveCell();

            $form = $("#" + proxy._id + "EditForm");

            if ($form.length > 0) return true;

            if (!proxy.model.selectedItem) return false;

            if (((fieldName === "status" ||
                fieldName === "startDate" ||
                fieldName === "endDate" ||
                fieldName === "duration" ||
                fieldName === "predecessor") &&
                proxy.model.selectedItem.hasChildGanttRecords) ||
                (fieldName === "baselineStartDate" ||
                    fieldName === "baselineEndDate")) {

                return false;

            }

            var $targetTr = $(proxy.getGridRows()[index]),
                columnIndex = proxy.getColumnIndexByField(fieldName),
                $targetTd = $targetTr.find(".e-rowcell").eq(columnIndex),
                column = proxy.model.columns[columnIndex],
                data = proxy.model.selectedItem,
                beginEditArgs = {},
                args = {
                    validationRules: ej.isNullOrUndefined(column.validationRules) ? {} : $.extend(true, {},
                        column.validationRules),
                    columnName: column.field,
                    value: data[fieldName] ? data[fieldName] : data.item && data.item[fieldName],
                    data: data,
                    columnObject: column,
                    cell: $targetTd
                },
                isEditable = true;

            if (args.columnName === "predecessor") {

                var predecessor = args.data.item[proxy.model.predecessorMapping];
                args.value = predecessor ? predecessor : "";

            }

            beginEditArgs.data = data;
            beginEditArgs.columnIndex = columnIndex;
            beginEditArgs.rowElement = $targetTr;
            beginEditArgs.cellElement = $targetTd;

            proxy._cellEventTrigger($targetTr[0].cells[proxy._cellEditingDetails.columnIndex], data, args.columnObject);

            if (proxy._trigger("beginEdit", beginEditArgs)) return false;

            if ($.inArray(fieldName, proxy._primaryKeys) != -1 ||
                args.columnObject.allowEditing === false) {

                $.extend(proxy._cellEditingDetails, {
                    cellValue: args.value,
                    rowIndex: index,
                    fieldName: fieldName,
                    data: args.data,
                    columnIndex: columnIndex
                });

                isEditable = false;
            }

            if ($targetTr.hasClass("e-insertedrow") && (!args.columnObject.columnTemplate)) isEditable = true;

            if (isEditable) {

                $.extend(proxy._cellEditingDetails, {
                    rowIndex: index,
                    cellValue: args.value,
                    columnIndex: columnIndex,
                    fieldName: fieldName,
                    cellEditType: args.columnObject.columnEditType,
                    data: data
                });
            }

            proxy._renderCellEditObject(args, $targetTd);
            $targetTr.addClass("e-editedrow");

            if (proxy._cellEditingDetails.columnIndex === 0) args.cell.addClass("e-editedfirstcell");
            else args.cell.addClass("e-editedcell");

            return true;
        },

        //render edited Cell
        _renderCellEditObject: function (cellEditArgs, $td) {

            var proxy = this,
                $form = ej.buildTag("form.#" + proxy._id + "EditForm", "", {},
                    {}),
                $cellEditTemplate = proxy._cellEditTemplate,
                $element,
                htmlString,
                cellData = {};

            cellData[cellEditArgs.columnObject.field] = cellEditArgs.value,
            columnIndex = proxy.getColumnIndexByField(cellEditArgs.columnObject.field);

            if (cellEditArgs.columnObject.field === "taskName") {
                $td.find('.e-cell').empty();
            } else $td.empty();

            htmlString = $cellEditTemplate.find("#" + cellEditArgs.columnObject.field + "_CellEdit").html();
            $element = $($.templates(htmlString).render(cellData));

            if ($element.get(0).tagName == "SELECT") {

                $element.val(cellData[cellEditArgs.columnObject.field]);
                $element.val() == null && $element.val($element.find("option").first().val());

            }

            $form.append($element);

            if (cellEditArgs.columnObject.field === "taskName") {
                $td.find('.e-cell').append($form);
            } else $td.append($form);

            proxy._setoffsetWidth();
            proxy._refreshEditForm();

            if ($.isFunction($.validator) && !$.isEmptyObject(cellEditArgs.validationRules)) {

                proxy._initValidator();
                proxy.setValidationToField(cellEditArgs.columnObject.field, cellEditArgs.validationRules);

            }
            proxy.model.isEdit = true;

        },

        //set with for editing cellElement
        _setoffsetWidth: function () {

            var proxy = this,
                tds, $form = $("#" + proxy._id + "EditForm"),
                i = 0,
                length, child;

            if (proxy.model.edit.editMode == "cellEditing") {
                tds = $form.closest("td");
            } else {
                tds = $form.find("tr").find(".e-rowcell");
            }

            length = tds.length;

            for (i; i < length; i++) {

                child = $(tds.children()[0]).children();

                if (child.length > 1) {
                    proxy._tdsOffsetWidth[i] = tds.get(i).offsetWidth - child[0].offsetWidth - child[1].offsetWidth;
                } else {
                    proxy._tdsOffsetWidth[i] = tds.get(i).offsetWidth;
                }
            }

        },

        //refresh the edited form
        _refreshEditForm: function () {

            var proxy = this,
                form = document.getElementById(proxy._id + "EditForm"),
                elementFocused = false,
                $formElement = $(form).find("input,select"),
                percent = 86,
                i = 0,
                length = $formElement.length,
                $element, inputWidth, model = proxy.model,
                width, params = {},
                value,
                column,
                customParams,
                toformat,
                formatVal;

            for (i; i < length; i++) {

                $element = $formElement.eq(i);

                if (model.edit.editMode == "cellEditing") percent = 95;

                inputWidth = proxy._tdsOffsetWidth[i] * (percent / 100);

                if ($element.hasClass("e-numerictextbox")) {

                    width = inputWidth;
                    value = $element.val();
                    params.width = width;
                    params.height = "23px";
                    params.showSpinButton = true;
                    params.cssClass = model.cssClass;

                    if (value.length) params.value = parseFloat(value);

                    customParams = proxy.getColumnByField($element.prop("name"));

                    if (!ej.isNullOrUndefined(customParams["editParams"])) $.extend(params, customParams["editParams"]);

                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(proxy._id, ""));

                } else if ($element.hasClass("e-datepicker")) {

                    inputWidth = proxy._tdsOffsetWidth[i] * (90 / 100);
                    width = inputWidth;
                    params.width = width;
                    params.height = "23px";
                    params.cssClass = model.cssClass;
                    params.displayDefaultDate = true;
                    params.dateFormat = model.dateFormat;

                    if ($element.val().length) params.value = new Date($element.val());

                    column = proxy.getColumnByField($element.prop("name"));

                    if (column["format"] !== undefined && column.format.length > 0) {

                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }

                    if (!ej.isNullOrUndefined(column["editParams"])) $.extend(params, column["editParams"]);

                    $element.ejDatePicker(params);
                } else if ($element.hasClass("e-datetimepicker")) {

                    width = inputWidth;
                    params = {
                        width: width,
                        height: "23px",
                        cssClass: model.css,
                        showButton: false
                    };

                    if ($element.val().length) params.value = new Date($element.val());

                    column = proxy.getColumnByField($element.prop("name"));

                    if (column["format"] !== undefined && column.format.length > 0) {

                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];

                    }

                    if (!ej.isNullOrUndefined(column["editParams"])) $.extend(params, column["editParams"]);

                    $element.ejDateTimePicker(params);

                } else if ($element.hasClass("e-dropdownlist")) {

                    var dataSource = model.columns[proxy._cellEditingDetails.columnIndex].dropdownData;

                    $element.ejDropDownList({
                        width: inputWidth + "px",
                        height: "23px",
                        showCheckbox: true,
                        dataSource: dataSource,
                        fields: {
                            id: model.resourceIdMapping,
                            text: model.resourceNameMapping,
                            value: model.resourceNameMapping
                        },
                        multiSelectedItemsIndex: proxy.getIndexofresourceInfo(dataSource)
                    });

                    $element.ejDropDownList("selectItemByValue", $element.val());

                } else {

                    switch ($element.prop('tagName')) {
                        case "INPUT":

                            if ($element.attr("type") != "checkbox") {

                                $element.css("text-align", $element.attr("name") != null &&
                                    proxy.getColumnByField($element.attr("name")) != null ?
                                    proxy.getColumnByField($element.attr("name")).textAlign :
                                    "center");

                                $element.outerWidth(inputWidth).height(19);

                            } else $element.width(inputWidth > 0 ? ($element.width() > inputWidth ? inputWidth : $element.width()) : 1);

                            break;

                        case "SELECT":
                            $element.width(inputWidth).height(20);
                            break;
                    }
                }

                if (!$element.is(":disabled") && !elementFocused &&
                    (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {

                    if (!proxy._isEnterKeyPressed) {

                        proxy._focusElements($element.closest('td'));
                        elementFocused = true;

                    }
                }
            }
        },

        //get Index of resourceInfo
        getIndexofresourceInfo: function (dataSource) {

            var proxy = this,
                data = proxy._cellEditingDetails.data,
                resourceInfo = data.resourceInfo,
                count = 0,
                length = resourceInfo && resourceInfo.length,
                resourceIndex = [];

            for (count; count < length; count++)
                resourceIndex.push(dataSource.indexOf(resourceInfo[count]));

            return resourceIndex;

        },

        //move the currentCell based on the keyboard interaction
        _moveCurrentCell: function (direction) {

            var proxy = this,
                rowIndex = proxy._cellEditingDetails.rowIndex,
                columnIndex = proxy._cellEditingDetails.columnIndex,
                model = proxy.model;

            if (rowIndex == -1 && columnIndex == -1) return true;

            switch (direction) {

                case "right":

                    if (!proxy._rowSelectingEventTrigger()) {

                        if (!proxy.selectRows(proxy._cellEditingDetails.rowIndex)) {

                            if (rowIndex == proxy.getGridRows().length - 1 && columnIndex == model.columns.length - 1) return true;

                            if (columnIndex == model.columns.length - 1) {

                                proxy._cellEditingDetails.columnIndex = 0;
                                proxy._cellEditingDetails.rowIndex = rowIndex + 1;

                            } else proxy._cellEditingDetails.columnIndex = columnIndex + 1;

                            proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);
                        }
                    }

                    break;

                case "left":

                    if ((rowIndex == 0 && columnIndex == 0)) return true;

                    if (columnIndex == 0) {

                        proxy._cellEditingDetails.columnIndex = model.columns.length - 1;
                        proxy._cellEditingDetails.rowIndex = rowIndex - 1;

                    } else {
                        proxy._cellEditingDetails.columnIndex = columnIndex - 1;
                    }

                    proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);

                    break;

                case "up":

                    if (rowIndex == 0) return true;

                    proxy._cellEditingDetails.rowIndex = rowIndex - 1;
                    proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                    proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);

                    break;

                case "down":

                    if (rowIndex + 1 == proxy.getGridRows().length - 1) {

                        if (proxy.model.enableVirtualization) {

                            proxy._isEnterKeyPressed = true;
                            proxy._updateScrollTop(proxy._$gridContent.scrollTop() + model.rowHeight - proxy._offset);

                        }
                    }

                    if (!proxy._isEnterKeyPressed) {

                        if (!proxy.selectRows(rowIndex + 1)) {

                            proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
                            proxy._cellEditingDetails.rowIndex = rowIndex + 1;
                            proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);

                        }
                    }

                    break;
            }

            return false;

        },


        //get the currentRowIndex
        getRowIndex: function (e) {

            var proxy = this,
                $target = $(e.target),
                $gridRows = $(proxy.getGridRows()),
                index = $gridRows.index($target.parent());

            if (index == -1) {

                var row = $target.closest('tr');
                index = $gridRows.index(row);

            }

            return index;
        },


        //get the currentCellIndex
        getCellIndex: function (e, target) {

            var $target = $(e.target);

            if (target) $target = $(target);

            if ($target.prop("tagName") == "TD") {
                return $target.index();
            } else if ($target.prop('tagName') == "DIV" && $target.hasClass('e-cell')) {

                var $td = $target.closest('td');
                return $td.index();

            }

            return -1;
        },


        //mousehover event for adding rowHover background

        _rowHover: function (e) {

            var proxy = this,
                $target = $(e.target),
                $gridRows, args = {},
                index, row;

            if (!proxy.model.enableRowHover || $target.closest('td').hasClass('e-extendcolumn')) return true;

            $gridRows = $(proxy.getGridRows());
            index = $gridRows.index($target.parent());

            if (index == -1) {

                row = $target.closest('tr');
                index = $gridRows.index(row);

            }

            args.index = index;

            if (e.type == "mouseenter") {

                args.reason = "mouseenter";
                proxy.addRowHover(index);

            } else {

                args.reason = "mouseleave";
                proxy.removeRowHover();

            }

            proxy._trigger("rowHover", args);

            return false;
        },

        //addRowHover to add rowHover background
        addRowHover: function (index) {

            var $gridRows = $(this.getGridRows());

            if (index != null) index != -1 && $($gridRows[index]).addClass("e-enablerowhover");
        },

        //remove the rowHover background
        removeRowHover: function () {

            var $gridRows = $(this.getGridRows());

            $gridRows.removeClass("e-enablerowhover");
        },

        _headerMouseDown: function (e) {

            var proxy = this,
                model = proxy.model;

            if (model.allowColumnResize) proxy._resizer._mouseDown(e);

            if (($(e.target).hasClass("headercelldiv") && $(e.target).hasClass("headercell"))) {

                var $headercell = $(e.target).hasClass("headercelldiv") ? $(e.target).parent() : $(e.target);
                model.headerEffects && $headercell.addClass("headercellactive");

            }
        },

        //perform the scrolling in ganttgridtree
        _mouseWheel: function (e) {

            var proxy = this,
                delta = null;

            e = e.originalEvent;
            try {
                e.preventDefault();
            }
            catch (err) {
            }

            if (e.wheelDelta) {
                delta = -e.wheelDelta / 120;
            } else if (e.detail) {
                delta = e.detail / 3;
            }

            if (!delta) return false;

            var scrollTop = proxy._$gridContent.scrollTop(),
                change = scrollTop + (delta * 57);

            proxy._updateScrollTop(change);

            return true;

        },

        _mouseMove: function (e) {

            var proxy = this;

            if (proxy.model.allowColumnResize) proxy._resizer._mouseMove(e);
        },

        _mouseUp: function (e) {

            var proxy = this;

            if (proxy.model.allowColumnResize) {
                if (proxy.model.isEdit && $(e.target).hasClass('e-headercelldiv')) {
                    proxy.saveCell();
                }

                proxy._resizer._mouseUp(e);
            }
        },

        //find the column width for gridColumns
        _findColumnsWidth: function () {

            var proxy = this,
                length = proxy.model.columns.length,
                j = proxy.getGridHeaderTable().find("colgroup").find("col");

            for (var i = 0; i < length; i++)
                proxy._columnsWidthCollection[i] = j.eq(i).width();

        },

        //calculte the total width of the columns
        _calculateWidth: function () {

            var j = this.getGridHeaderTable().find("colgroup").find("col"),
                width = 0;

            for (var i = 0; i < j.length; i++)
                width += j.eq(i).width();

            return width;
        },

        _getMappingName: function (header) {

            var proxy = this,
                columns = proxy.model.columns,
                count = 0,
                length = columns.length,
                model = proxy.model;

            for (count = 0; count < length; count++) {

                if (columns[count].field === header) return columns[count].mappingName;

            }

            return false;

        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {

            var proxy = this;
            proxy.element.empty().removeClass("e-ganttgridtree-core e-ganttgridtree " + proxy.model.cssClass);

        }

    });

    /* enum for each action in the gantt */
    ej.GanttGridTree.actions = {
        Sorting: "sorting",
        BeginEdit: "beginedit",
        Save: "save",
        Add: "add",
        Delete: "delete",
        Cancel: "cancel",
        Refresh: "refresh",
        Searching: "searching",
        ExpandCollapse: "expandcollapse",
        Selection: "selection",
        rowHover: "rowHover",
        Scroll: "scroll",
        ContextMenuAdd: "contextMenuAdd"

    };

    /* enum for selection Mode in the ganttgridtree */
    ej.GanttGridTree.selectionType = {
        Single: "single",
        Multiple: "multiple"
    };

    /* enum for three different editMode in the ganttgridtree */
    ej.GanttGridTree.editMode = {
        Normal: "normal",
        CellEditing: "cellEditing"
    };

    /* enum for five different columnEditType for ganttgridtree */
    ej.GanttGridTree.editingType = {
        String: "stringedit",
        Boolean: "booleanedit",
        Numeric: "numericedit",
        Dropdown: "dropdownedit",
        DatePicker: "datepicker",
        DateTimePicker: "datetimepicker"
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    // ejGanttChart is the plugin name 
    // "ej.GanttChart" is "namespace.className" will hold functions and properties

    ej.widget("ejGanttChart", "ej.GanttChart", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        _rootCSS: "e-ganttchart",

        /* keyConfigurations for keyboard interaction in ganttgridtree */
        keyConfigs: {
            focus: "e",

            //Insert
            insertRecord: "45",

            // delete
            deleteRecord: "46",

            //F2
            editRecord: "113",

            // Enter
            saveRequest: "13",

            //Esc   
            cancelRequest: "27",

            //Home,
            firstRowSelection: "36",

            //End,
            lastRowSelection: "35",

            //Up arrow
            upArrow: "38",

            //Down arrow
            downArrow: "40",

            //tab
            moveCellRight: "9",

            //shifttab
            moveCellLeft: "shift+9",

            //AltPlusDownArrow
            selectedRowExpand: "alt+40",

            //CtrlPlusDownArrow
            totalRowExpand: "ctrl+40",

            //AltPlusUpArrow
            selectedRowCollapse: "alt+38",

            //CtrlPlusUpArrow
            totalRowCollapse: "ctrl+38"
        },

        //default model
        defaults: {
            childPropertyName: "",
            taskIdMapping: "",
            taskNameMapping: "",
            startDateMapping: "",
            endDateMapping: "",
            childMapping: "",
            finishDateMapping: "",
            durationMapping: "",
            mileStoneMapping: "",
            progressMapping: "",
            predecessorMapping: "",
            resourceInfoMapping: "",
            resourceNameMapping: "",
            resourceCollection: [], /*--------------Array containing the resource names */

            holidays: [
                {
                    day: null,
                    label: null,
                    background: null
                }
            ],

            stripLines: [
                {
                    day: null,
                    label: null,
                    lineStyle: "dotted",
                    lineColor: "#169dd7",
                    lineWidth: 2
                }
            ],

            showTaskbarTooltip: true, /*------------showing tooltip on taskbar mouse over */
            showTaskbarDragTooltip: true, /*--------showing taskbar while dragging taskbar */

            highlightWeekEnds: true, /*-------------Boolean value for highlighting weekends */

            scheduleStartDate: null, /*-------------start date of entire project */
            scheduleEndDate: null, /*---------------end date of entire project */

            canResizeProgressBar: true, /*----------Boolean value for resizing progress bar */

            rowHeight: 30,
            progressbarHeight: 100,
            connectorlineWidth: 1,
            dateFormat: "MM/dd/yyyy",
            localization: "en-US",

            //background style
            taskbarBackground: "#DE8080", //"#5AC1E0",
            progressbarBackground: "#C44647", //"#169DD7",
            weekendBackground: '#F2F2F2',
            connectorLineBackground: "#383838",
            parentTaskbarBackground: "#383838",
            parentProgressbarBackground: "#1C1C1C",

            selectedItem: null,
            selectedRowIndex: -1,
            perDayWidth: null,
            queryTaskbarInfo: null,

            taskbarTemplate: null,
            progressbarTemplate: null,
            parenttaskbarTemplate: null,
            parentprogressbarTemplate: null,

            //internal collections
            rows: [],
            ganttRecords: [],
            scheduleWeeks: [],
            updatedGanttRecords: [],

            //tooltip templates
            tooltipTemplate: null,
            progressbarTooltipTemplateID: null,
            taskbarEditingTooltipTemplateID: null,

            //boolean values
            showTaskNames: true,
            showProgressStatus: true,
            showResourceNames: true,
            enableRowHover: false,
            includeWeekend: false,
            allowZooming: false,
            allowGanttChartEditing: true,
            allowKeyboardNavigation: true,

            //baselines
            renderBaseline: false,
            baselineColor: '#fba41c ',

            /* editing API's */
            edit: {
                allowEditing: false,
                allowAdding: false,
                allowDeleting: false,
                editMode: "normal",
                dialogEditorTemplateId: null
            },

            scheduleHeader:
            {
                weekHeaderFormat: "MMM dd , yyyy",
                dayHeaderFormat: "ddd",
                weekendBackground: '#F2F2F2'
            },

            localizedDays: null,
            localizedMonths: null,
            //localized string to be displayed in toolTip
            columnHeaderTexts: null
        },

        showTooltip: function (boolValue) {
            var proxy = this;
            proxy.model.showTaskbarTooltip = boolValue;
        },

        showEditingTooltip: function (boolValue) {
            var proxy = this;
            proxy.model.showTaskbarDragTooltip = boolValue;
        },

        updateTaskbarBackground: function (bgcolor) {
            var proxy = this;
            proxy.model.taskbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateProgressbarBackground: function (bgcolor) {
            this.model.progressbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateParentTaskbarBackground: function (bgcolor) {
            this.model.parentTaskbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateParentProgressbarBackground: function (bgcolor) {
            this.model.parentProgressbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateRenderTaskNames: function (bool) {
            var proxy = this;
            proxy.model.showTaskNames = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRendeProgressStatus: function (bool) {
            var proxy = this;
            proxy.model.showProgressStatus = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRenderResourceNames: function (bool) {
            var proxy = this;
            proxy.model.showResourceNames = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        ganttChartEditing: function (boolValue) {
            var proxy = this;
            proxy.model.allowGanttChartEditing = boolValue;
        },

        resizeProgressbar: function (boolValue) {
            var proxy = this;
            proxy.model.canResizeProgressBar = boolValue;

        },

        updateAllowKeyboardNavigation: function (bool) {
            this.model.allowKeyboardNavigation = bool;
        },

        updateProgressbarHeight: function (height) {
            var proxy = this;
            this.model.progressbarHeight = height;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateBaselineColor: function (color) {
            var proxy = this;
            this.model.baselineColor = color;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRenderBaseline: function (bool) {
            var proxy = this;
            this.model.renderBaseline = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
            proxy._createConnectorLineTemplate();
        },

        //returns the schedule week width
        _getWeekWidth: function () {
            return (this.model.perDayWidth * 7);
        },

        //returns the ganttchart container height
        _getHeight: function () {
            return this._containerHeight;
        },

        //sets the height of the ganttchart container
        _setHeight: function (value) {
            this._containerHeight = value;
        },

        //returns the rows in the ganttchart
        getGanttChartRows: function () {
            return this._ganttChartRows;
        },

        //returns the gantt record collection
        getUpdatedRecords: function () {
            return this.model.ganttRecords;
        },

        //sets value for currentviewdata , ganttrecords and row collections
        setUpdatedRecords: function (currentViewData, updatedGanttRecords, rows) {
            var proxy = this;
            proxy._currentViewData = currentViewData;
            proxy.model.updatedGanttRecords = updatedGanttRecords;
            proxy.model.rows = rows;
        },

        //returns the string date type 
        _getFormatedDate: function (date, format) {
            return Globalize.format(date, format);
        },

        //returns the viewport height
        _getViewportHeight: function () {
            var proxy = this,
                headerdiv = proxy._$headerContainer,
                height;

            height = proxy.element.height() - headerdiv.height() -
                    parseFloat(headerdiv.css("border-bottom-width"));
            return height;
        },

        //constructor function
        _init: function () {

            var proxy = this, model = proxy.model;
            proxy.element.addClass("e-ganttchart-core");
            proxy._initPrivateProperties();
            proxy._initialize();
            proxy._renderGanttChart();
            proxy._$bodyContainer.append(proxy._$bodyContent);
            proxy._$ganttChartContainer.append(proxy._$bodyContainer);

            $(proxy._$bodyContainer).css({ "height": proxy._viewportHeight + "px" });
            if (proxy.model.enableVirtualization)
                $(proxy._$bodyContent).css({ "height": (model.rowHeight * model.ganttRecords.length) + "px" });
            $(proxy._$bodyContent).css({ "width": (model.scheduleWeeks.length * proxy._scheduleWeekWidth) + "px" });

            proxy.renderGanttRecords();
            proxy._scrollBarHeight = proxy._$bodyContainer[0].offsetHeight - proxy._$bodyContainer[0].clientHeight;
            proxy._wireEvents();
        },

        //rendering gantt chart again after zoomingvalue changed
        reRenderGanttChart: function (daywidth, ganttRecords) {

            var proxy = this;
            proxy._wireEvents();
            proxy.model.perDayWidth = daywidth;
            proxy.model.ganttRecords = ganttRecords;
            proxy._scheduleWeekWidth = proxy._getWeekWidth();
            $(proxy._$secondaryCanvas).empty();
            $(proxy._$headerContainer).empty();
            $(proxy._$scheduleDiv).empty();
            proxy._renderGanttChart();
            proxy._$bodyContainer.append(proxy._$bodyContent);
            proxy._$ganttChartContainer.append(proxy._$bodyContainer);
            $(proxy._$bodyContent).css({ "width": (proxy.model.scheduleWeeks.length * proxy._scheduleWeekWidth) + "px" });
        },

        //declaring private properties
        _initPrivateProperties: function () {
            var proxy = this;
            proxy._viewportHeight = 0,
                proxy._prevScrollTop = 0,
                proxy._scrollTop = 0,
                proxy._vScrollDir = 1,
                proxy._$ganttChartContainer = null,
                proxy._$headerContainer = null,
                proxy._$bodyContainer = null,
                proxy._$bodyContent = null,
                proxy._$dependencyViewContainer = null,
                proxy._$secondaryCanvas = null,
                proxy._$ganttViewTable = null,
                proxy._$ganttViewTablebody = null,
                proxy._weekdays = [],
                proxy._rowMargin = 0,
                proxy._renderedRange = null,
                proxy._visibleRange = null,
                proxy._vTop = null,
                proxy._vBottom = null,
                proxy._rTop = null,
                proxy._rBottom = null,
                proxy._vscrollDist = 0,
                proxy._containerHeight = 0,
                proxy._currentViewData = [],
                proxy._id = this.element.attr("id"),
                proxy._ganttChartRows = null,
                proxy._projectStartDate = new Date(),
                proxy._projectEndDate = new Date(null),
                proxy._filteredRecords = [],
                proxy._sortedRecords = [],
                proxy._tempSortedRecords = [],
                proxy._scheduleWeekWidth = proxy._getWeekWidth(),
                proxy._tempFilteredRecords = [],
                proxy._prevLeft = 0,
                proxy._allowDragging = false,
                proxy._leftResizing = false,
                proxy._rightResizing = false,
                proxy._progressResizing = false,
                proxy._editingContainer = null,
                proxy._allowExpandCollapse = true,
                proxy._leftResizer = null,
                proxy._taskbarItem = null,
                proxy._progressbarItem = null,
                proxy._progressResizer = null,
                proxy._rightResizer = null,
                proxy._tasknameContainer = null,
                proxy._mousePosX = 0,
                proxy._currMousePosX = 0,
                proxy._currMousePosY = 0,
                proxy._prevItem = {},
                proxy._mouseTarget = null,
                proxy._mouseDown = false,
                proxy._$scheduleDiv = null,
                proxy._leftResizerGripper = null,
                proxy._rightResizerGripper = null,
                proxy._progressHandle = null,
                proxy._progressHandleChild = null,
                proxy._mouseOverTooltip = document.getElementById(this.model.tooltipTemplate),
                proxy._progressBarTooltipID = document.getElementById(this.model.progressbarTooltipTemplateID),
                proxy._taskbarEditingTooltipID = document.getElementById(this.model.taskbarEditingTooltipTemplateID),
                proxy._ids = null,
                proxy._scrollBarHeight = 0,
                proxy._windowWidth = $(window).width(),
                proxy._windowHeight = $(window).height(),
                proxy._editingItem = null,
                proxy._editingTarget = null;

        },

        //Initialize ganttchart
        _initialize: function () {
            var proxy = this;
            proxy._$ganttChartContainer = ej.buildTag("div.e-ganttviewercontainer#ganttviewercontainer" + proxy._id, "", { width: "100%" }, {});
            proxy._$headerContainer = ej.buildTag("div.e-ganttviewerheaderContainer#ganttviewerheaderContainer" + proxy._id, "", {}, {});
            proxy._$bodyContainer = ej.buildTag("div.e-ganttviewerbodyContianer#ganttviewerbodyContianer" + proxy._id, "", {
            }, {});
            proxy._$bodyContent = ej.buildTag("div.e-ganttviewerbodyContent#ganttviewerbodyContent" + proxy._id, "", {
            }, {});

            proxy._$dependencyViewContainer = ej.buildTag("div.e-ganttviewerbodyContianer-dependencyViewContainer#dependencyViewContainer" + proxy._id,
                "", { 'z-index': '2' }, {});
            proxy._$secondaryCanvas = ej.buildTag("div.e-ganttviewerbodyContianer-secondaryCanvas", "", {}, {});
            proxy._$ganttViewTable = ej.buildTag("table.e-ganttviewerbodyContianer-ganttViewTable#ganttViewTable" + proxy._id, "", {
                'z-index': '4'
            }, { 'cellspacing': '0px' });
            proxy._$scheduleDiv = ej.buildTag("div.e-gantt-schedule-container#gantt-schedule" + proxy._id + "", "", {}, {});
        },

        //Rendering GanttChart
        _renderGanttChart: function () {
            var proxy = this, model = proxy.model,
                scheduleLength, $colsgroup, $column;

            proxy._createSchedule(proxy._getDateType(model.scheduleStartDate),
                proxy._getDateType(model.scheduleEndDate));

            proxy._viewportHeight = proxy._getViewportHeight();
            scheduleLength = model.scheduleWeeks.length;

            proxy._$ganttViewTable.css("width", (scheduleLength * proxy._scheduleWeekWidth) + 'px');
            $colsgroup = ej.buildTag("colgroup", "", {}, {});
            $column = $(document.createElement("col"));
            $column.css("width", (scheduleLength * this._scheduleWeekWidth) + 'px');
            $colsgroup.append($column);
            proxy._$ganttViewTable.append($colsgroup);

            proxy._createTooltipTemplate();
            proxy._createTaskbarTemplate();
            proxy._createConnectorLineTemplate();

            proxy._updateCurrentViewData();

            //Display weekends
            if (proxy.model.highlightWeekEnds == true)
                proxy._renderWeekends();

            //Display striplines
            if (proxy.model.stripLines != null) {
                this._renderStripLines(proxy.model.stripLines);
            }

            //Display Holidays
            if (proxy.model.holidays != null) {
                var holidayCount = proxy.model.holidays.length;
                for (var i = 0; i < holidayCount; i++) {
                    proxy._renderHoliday(proxy.model.holidays[i]);
                }
            }
        },

        //Rendering connectorlines from Gantt
        renderConnectorLines: function (connectorLinesCollection) {
            var proxy = this,
                connectorLineContainer = $.render[proxy._id + "ConnectorLineTemplate"](connectorLinesCollection);

            $(proxy._$dependencyViewContainer).empty();
            proxy._$dependencyViewContainer.append(connectorLineContainer);
            proxy._$bodyContent.append(proxy._$dependencyViewContainer);
        },

        //Empty the entire connectorline collection
        clearConnectorLines: function () {
            $(this._$dependencyViewContainer).empty();
        },

        //append single connectorline
        appendConnectorLine: function (connectorLineCollection) {
            var proxy = this,
            connectorLineContainer = $.render[proxy._id + "ConnectorLineTemplate"](connectorLineCollection);
            proxy._$dependencyViewContainer.append(connectorLineContainer);
        },

        //remove a particular connector line
        removeConnectorline: function (lineId) {
            $("#ConnectorLine" + lineId).remove();
        },

        //Events
        _wireEvents: function () {
            var proxy = this;
            $(proxy._$bodyContainer).bind("scroll", $.proxy(proxy._handleScroll, proxy));

            proxy._on(proxy.element, "mouseenter", ".e-gantt-milestone-container,.e-gantt-childtaskbar,.e-gantt-childtaskbar-progress,.e-gantt-parenttaskbar-innerdiv,.e-gantt-parenttaskbar-progress", this._mouseHover);//.gantt-viewer-bodyContianer tr td
            proxy._on(proxy.element, "mouseleave", ".e-gantt-milestone-container,.e-gantt-childtaskbar,.e-gantt-childtaskbar-progress,.e-gantt-parenttaskbar-innerdiv,.e-gantt-parenttaskbar-progress", this._mouseLeave);//.gantt-viewer-bodyContianer tr td
            proxy._on(proxy.element, "click", proxy._cellClickHandler);

            proxy._on(proxy.element, "mouseenter", ".taskbarresizer-left", proxy._resizerleftOver);
            proxy._on(proxy.element, "mouseleave", ".taskbarresizer-left", proxy._resizerleftLeave);

            proxy._on(proxy.element, "mouseenter", ".taskbarresizer-right", proxy._resizerrightOver);
            proxy._on(proxy.element, "mouseleave", ".taskbarresizer-right", proxy._resizerrightLeave);

            proxy._on(proxy.element, "mouseenter", ".progressbarresizer-right", proxy._progressresizerOver);
            proxy._on(proxy.element, "mouseleave", ".progressbarresizer-right", proxy._progressresizerLeave);

            proxy._on(proxy.element, "mousedown", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
            $(proxy._$bodyContainer).bind("mouseup", $.proxy(proxy._handleMouseUp, proxy));
            $(document).bind("mouseup", $.proxy(proxy._tooltipMouseup, proxy));

            //$(document).bind("mousedown", $.proxy(proxy._clearContextMenu, proxy));
            //proxy._on(proxy.element, "contextmenu", proxy._rightClick);
            //proxy._on(proxy.element, "mousedown", proxy._contextMenuHandler);


            proxy._on(proxy.element, "mouseenter", ".e-stripline", proxy._stripLineMouseOver);
            proxy._on(proxy.element, "mouseleave", ".e-stripline", proxy._stripLineMouseLeave);
            proxy._enableEditingEvents();
        },

        //Event append for editDialogOpen while perform double click action
        _enableEditingEvents: function () {

            var proxy = this,
                model = proxy.model;

            if (model.edit.allowEditing) {
                if (model.edit.editMode === "normal") {
                    proxy._on(this.element, "dblclick", ".e-ganttviewerbodyContent", proxy._editdblClickHandler);
                }
            }
        },

        //Double click event handler for editing with editMode as normal
        _editdblClickHandler: function () {

            var proxy = this,
                args = {};

            if (proxy.model.edit.editMode === "normal") {
                args.requestType = ej.GanttGridTree.actions.BeginEdit;
                proxy._trigger("actionBegin", args);
            }

        },


        //Event to append RowHover effect
        _enableRowHover: function () {
            var proxy = this;
            if (proxy.model.enableRowHover)
                proxy._on(proxy.element, "mouseleave mouseenter",
                    "#ganttviewerbodyContent,.e-ganttviewerbodyContent tr,.e-ganttviewerbodyContent" + proxy._id,
                    proxy._rowHover);
            else
                proxy._on(proxy.element, "mouseleave mouseenter",
                    "#ganttviewerbodyContent,.e-ganttviewerbodyContent tr,.e-ganttviewerbodyContent" + proxy._id);
        },

        //MouseLeave event for striplines to remove tooltip
        _stripLineMouseLeave: function () {
            $(this._mouseOverTooltip).remove();
        },

        //mouseHover event for stripline to render tooltip
        _stripLineMouseOver: function (e) {

            e.target.style.cursor = "pointer";
            var proxy = this, model = proxy.model,
                $target = $(e.target),
                parentNode, match,
                tooltiptable,
                tooltipbody,
                posx = 0, posy = 0;

            if ($target.hasClass("e-striplinespan")) {
                parentNode = $target[0].parentNode;
            } else
                parentNode = $target[0];

            match = parentNode.id.match(/(\d+|[A-z]+)/g);

            if (e.type == "mouseenter" || e.type == "mousemove") {
                if (!e) e = window.event;
                if (e.pageX || e.pageY) {
                    posx = e.pageX;
                    posy = e.pageY;
                }
                else if (e.clientX || e.clientY) {
                    posx = e.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft;
                    posy = e.clientY + document.body.scrollTop
                        + document.documentElement.scrollTop;
                }

                tooltiptable = ej.buildTag("table.e-tooltiptable", "", { "width": "auto" },
                    { 'cellspacing': '2px', 'cellpadding': '2px' });
                tooltipbody = ej.buildTag("tbody",
                    "<tr><td>" + model.stripLines[match[1]].day + "</td></tr><tr><td>" +
                        model.stripLines[match[1]].label + "</td></tr>", {}, {});

                tooltiptable.append(tooltipbody);
                if (model.tooltipTemplate == null) {
                    proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", tooltiptable,
                        { "top": (posy) + "px", "left": (posx + 15) + "px", 'position': 'absolute', 'z-index': '4' }, {});
                } else {
                    if (typeof (model.tooltipTemplate) != "object") {
                        var parser = $.parseHTML(model.tooltipTemplate);
                        proxy._mouseOverTooltip = parser[0];
                    }
                    else {
                        var markup = $(model.tooltipTemplate)[0].innerHTML;
                        parser = $.parseHTML(markup);
                        proxy._mouseOverTooltip = parser[1];
                    }

                    $(proxy._mouseOverTooltip).css({
                        "left": (posx + 15) + "px", "top": (posy) + "px",
                        "position": "absolute"
                    });
                    $(proxy._mouseOverTooltip).addClass("customTooltip");
                    proxy._mouseOverTooltip.innerHTML = "<table>" + tooltiptable[0].innerHTML + "</table>";
                }
                $(document.body).append(proxy._mouseOverTooltip);
            }
        },

        //Mouseup over tooltip 
        _tooltipMouseup: function () {
            var proxy = this;
            if (proxy._mouseTarget != null) {
                proxy._handleMouseUp(proxy._mouseTarget);
            }
        },

        //removes all the tooltip in ganttchart
        _clearTooltip: function () {
            var proxy = this;
            $(proxy._mouseOverTooltip).remove();
            $(proxy._taskbarEditingTooltipID).remove();
            $(proxy._progressBarTooltipID).remove();
            $('.e-editingtooltip').remove();
            $(".e-progressbartooltip").remove();
        },

        //mousewheel action for zooming ganttchart
        _mouseWheel: function (e) {
            if (this.model.allowZooming) {
                var proxy = this, args = {},
                    delta = null;

                e = e.originalEvent;
                e.preventDefault();

                if (e.wheelDelta)
                    delta = -e.wheelDelta / 120;
                else if (e.detail)
                    delta = e.detail / 3;

                if (!delta)
                    return;
                args.requestType = "scroll";
                args.delta = delta;
                proxy._enbleZooming(args);
            }
        },


        //mouseHover event on taskbar right helper element
        _resizerrightOver: function (e) {

            var proxy = this,
                $target = e.target,
                div = $target.parentNode;

            if (proxy.model.allowGanttChartEditing == true) {
                if (this._mouseDown == false)
                    e.target.style.cursor = "w-resize";
                proxy._setEditingElements(div);
            }
        },


        //mouseLeave event for taskbar right helper element
        _resizerrightLeave: function () {

            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        //mouseHover event on taskbar left helper element
        _resizerleftOver: function (e) {

            var proxy = this,
                $target = e.target,
                div = $target.parentNode;
            if (proxy.model.allowGanttChartEditing == true) {
                if (this._mouseDown == false)
                    e.target.style.cursor = "e-resize";
                proxy._setEditingElements(div);
            }
        },


        //mouseLeave event for taskbar left helper element
        _resizerleftLeave: function () {
            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        //mouseHover event on taskbar progressbar helper element
        _progressresizerOver: function (e) {
            var proxy = this,
                $target = e.target,
                div = $target.parentNode;
            if (proxy.model.allowGanttChartEditing == true) {
                if (proxy._mouseDown == false)
                    $target.style.cursor = "col-resize";
                proxy._setEditingElements(div);
            }
        },


        //mouseLeave event for progressbar helper element
        _progressresizerLeave: function () {
            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        //adding styles for taskbar helper elements
        _setEditingElements: function (div) {
            var proxy = this, progressHandler;

            if ($(div).hasClass("e-childContainer")) {
                proxy._leftResizerGripper = div.childNodes[0],
                proxy._rightResizerGripper = div.childNodes[4];
                progressHandler = div.childNodes[3];
                proxy._progressHandle = progressHandler.firstChild;
                proxy._progressHandleChild = progressHandler.childNodes[1];
            }

            $(proxy._leftResizerGripper).addClass("gripper");
            $(proxy._rightResizerGripper).addClass("gripper");

            if (proxy.model.progressMapping) {
                $(proxy._progressHandle).addClass("e-progresshandle");
                $(proxy._progressHandleChild).addClass("e-progresshandleafter");
            }
        },


        //removing styles for taskbar helper elements
        _removeEditingElements: function () {
            var proxy = this;
            $(proxy._leftResizerGripper).removeClass("gripper");
            $(proxy._rightResizerGripper).removeClass("gripper");

            if (proxy.model.progressMapping) {
                $(proxy._progressHandle).removeClass("e-progresshandle");
                $(proxy._progressHandleChild).removeClass("e-progresshandleafter");
            }
        },


        //mousedown event for ganttchart
        _mousedown: function (e) {

            var proxy = this;
            proxy._clearTooltip();
            proxy._mouseDown = true;

            //proxy._clearContextMenu();
            var $target = $(e.target),
            $ganttGridRows = $(proxy.getGanttChartRows()),
            row = $target.closest('tr'),
            recordIndexr = $ganttGridRows.index(row),
            item = proxy._currentViewData[recordIndexr],
            div = $target[0].parentNode;

            proxy._editingItem = item;
            proxy._editingTarget = $target;

            if ($(div).hasClass("e-gantt-childtaskbar-progress") ||
                $(div).hasClass("progressbarresizer-right")) {
                div = div.parentNode;
            }

            if ($(div).hasClass("e-childContainer") ||
                ($(div).hasClass("e-gantt-milestone"))) {

                $(row).addClass("e-editmode");
                proxy._mouseTarget = e;
            }

            if (proxy.model.allowGanttChartEditing && ($(div).hasClass("e-childContainer")
                || ($(div).hasClass("e-gantt-milestone")))) {

                if (e.target.style.cursor == "e-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))) {
                    proxy._updateEditingType();
                    proxy._leftResizing = true;
                }

                if (e.target.style.cursor == "w-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))) {
                    proxy._updateEditingType();
                    proxy._rightResizing = true;

                }
                if (e.target.style.cursor == "col-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))) {
                    proxy._updateEditingType();
                    proxy._progressResizing = true;

                }
                if (e.target.style.cursor == "move" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))) {
                    proxy._updateEditingType();
                    proxy._allowDragging = true;
                }

                if (!e) e = window.event;
                if (e.pageX || e.pageY) {
                    proxy._currMousePosX = e.pageX;
                    proxy._currMousePosY = e.pageY;
                    proxy.dragPosX = e.pageX - 400;
                }
                else if (e.clientX || e.clientY) {
                    proxy.dragPosX = e.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft;
                }

                proxy._prevItem.left = item.left;
                proxy._prevItem.status = item.status;
                proxy._prevItem.duration = item.duration;
                proxy._prevItem.startDate = item.startDate;
                proxy._prevItem.endDate = item.endDate;
                proxy._prevItem.width = item.width;
                $(proxy._$bodyContainer).bind("mousemove", $.proxy(proxy.handleMouseMove, proxy));
            }

            if (($(div).hasClass("e-parentContainer"))) {
                if (proxy._allowExpandCollapse && item && item.hasChildGanttRecords) {
                    proxy._expandCollapse(recordIndexr);
                }
            }
            proxy._appendTooltip($target, recordIndexr);
        },


        handleMouseMove: function (e) {
            this._updateOnmouseMove(e);
        },


        //Handler for MouseUp event in ganttchart
        _handleMouseUp: function () {

            var proxy = this,
                args = {},
                $ganttGridRows = $(proxy.getGanttChartRows()),
                $target, row, recordIndexr = 0, item;

            if (proxy._editingTarget != null && proxy._editingItem != null) {
                $target = proxy._editingTarget,
                row = $target.closest('tr'),
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem;
                if ($(row).hasClass("e-editmode")) {
                    $(row).removeClass("e-editmode");
                    proxy._mouseTarget = null;
                }
            }


            proxy._editingTarget = null;
            proxy._editingItem = null;

            proxy._clearTooltip();
            proxy._$ganttChartContainer[0].style.cursor = "auto";

            proxy._allowExpandCollapse = true;

            if (item != null) {
                proxy.updateConnectorLineOffset(item);
            }

            if (item != null && (proxy._allowDragging || proxy._leftResizing || proxy._rightResizing || proxy._progressResizing)) {
                args.editingFields = { startdate: item.startDate, enddate: item.endDate, progress: item.status, duration: item.duration };
                args.data = item;
                args.previousData = proxy._prevItem;
                args.dragging = proxy._allowDragging;
                args.leftResizing = proxy._leftResizing;
                args.rightResizing = proxy._rightResizing;
                args.progressResizing = proxy._progressResizing;
                proxy._taskbarEdited(args);
            }

            if (args.cancel) {
                proxy._getPrevItem(item, recordIndexr);
            }

            $("#ganttviewerbodyContianer" + proxy._id).unbind("mousemove");
            proxy._updateEditingType();
            proxy._mouseDown = false;
        },

        //keyPressed event for perform the keynavigate selection in gantt
        _keyPressed: function (action, target, arg, e) {

            var proxy = this,
                $ganttChartEle = proxy.element,
                ganttChartObject = $ganttChartEle.ejGanttChart("instance"),
                model = ganttChartObject.model;

            e.stopImmediatePropagation();
            e.preventDefault();

            if (!model.allowKeyboardNavigation)
                return true;

            switch (action) {

                case "downArrow":
                    if (!proxy._rowSelectingEventTrigger()) {
                        //get the last row index of the ganttchart control
                        var lastRowIndex = proxy.model.updatedGanttRecords.length - 1;

                        if (proxy.model.selectedRowIndex != lastRowIndex) {

                            proxy.selectRows(proxy.model.selectedRowIndex + 1);

                            var recordIndex = model.selectedItem &&
                                proxy.model.updatedGanttRecords.indexOf(model.selectedItem),
                                rowTop = lastRowIndex !== -1 && recordIndex * model.rowHeight;

                            //check if the selected row present in  out of view port of the ganttchart
                            if (rowTop + proxy.model.rowHeight >= proxy._viewportHeight) {
                                var scrollTop = rowTop + proxy.model.rowHeight -
                                    (proxy._viewportHeight - proxy._scrollBarHeight),
                                    args = {
                                        requestType: "scroll",
                                        delta: scrollTop
                                    };

                                //actionComplete event trigger for perform the scrolling gantt chartcontrol.
                                proxy._completeAction(args);
                                proxy._$bodyContainer.scrollTop(scrollTop);

                                if (model.enableVirtualization) {
                                    proxy.element[0].focus();
                                }
                            }

                            proxy._rowSelectedEventTrigger(proxy.model.selectedRowIndex);
                        }
                    }
                    break;

                    //select the previous row by press down arrow key
                case "upArrow":
                    if (!proxy._rowSelectingEventTrigger()) {
                        if (proxy.model.selectedRowIndex !== 0) {
                            proxy.selectRows(proxy.model.selectedRowIndex - 1);
                        }

                        recordIndex = proxy.model.updatedGanttRecords.indexOf(model.selectedItem);
                        rowTop = recordIndex !== -1 && recordIndex * model.rowHeight;
                        scrollTop = proxy._$bodyContainer.scrollTop();

                        if (scrollTop > rowTop) {

                            args = {
                                requestType: "scroll",
                                delta: rowTop
                            };

                            proxy._completeAction(args);
                            proxy._$bodyContainer.scrollTop(rowTop);
                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                            }
                        }

                        proxy._rowSelectedEventTrigger(proxy.model.selectedRowIndex);
                    }

                    break;

                    //select the first row while press the Home key
                case "firstRowSelection":

                    if (!proxy._rowSelectingEventTrigger()) {

                        proxy.selectRows(0);

                        recordIndex = model.updatedGanttRecords.indexOf(model.selectedItem);
                        rowTop = recordIndex !== -1 && recordIndex * model.rowHeight;
                        scrollTop = proxy._$bodyContainer.scrollTop();

                        if (scrollTop > rowTop) {

                            args = {
                                requestType: "scroll",
                                delta: rowTop
                            };

                            proxy._completeAction(args);
                            proxy._$bodyContainer.scrollTop(rowTop);

                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                            }
                        }

                        proxy._rowSelectedEventTrigger(0);
                    }

                    break;

                case "lastRowSelection":

                    if (!proxy._rowSelectedEventTrigger()) {

                        lastRowIndex = model.updatedGanttRecords.length - 1;
                        proxy.selectRows(lastRowIndex);

                        recordIndex = model.selectedItem && model.updatedGanttRecords.indexOf(model.selectedItem);
                        rowTop = lastRowIndex !== -1 && recordIndex * model.rowHeight;

                        if (rowTop + proxy.model.rowHeight >= proxy._viewportHeight) {

                            scrollTop = rowTop + proxy.model.rowHeight - (proxy._viewportHeight - proxy._scrollBarHeight);

                            args = {
                                requestType: "scroll",
                                delta: scrollTop
                            };
                            //actionComplete event trigger for perform the scrolling gantt chartcontrol.
                            proxy._completeAction(args);
                            proxy._$bodyContainer.scrollTop(scrollTop);

                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                            }

                        }

                        proxy._rowSelectedEventTrigger(lastRowIndex);
                    }

                    break;
            }

            return true;
        },

        _rowSelectingEventTrigger: function () {
            var proxy = this,
                model = proxy.model,
                args = {},
                selectedItem,
                $gridRows;

            if (model.selectedRowIndex !== -1) {

                $gridRows = $(proxy.getGanttChartRows());
                selectedItem = proxy._currentViewData[model.selectedRowIndex];
                model.selectedItem = selectedItem;
                args = {
                    rowElement: $gridRows.eq(model.selectedRowIndex),
                    data: selectedItem,
                    recordIndex: model.selectedRowIndex
                };

            }

            return proxy._trigger('rowSelecting', args);
        },

        _rowSelectedEventTrigger: function (index) {
            var proxy = this,
                args,
                $gridRows = $(proxy.getGanttChartRows());
            args = {
                rowElement: $gridRows.eq(index),
                data: proxy.model.selectedItem,
                target: "ejGanttChart",
                recordIndex: index
            };
            proxy._trigger("rowSelected", args);
        },

        _updateEditingType: function () {
            var proxy = this;
            proxy._leftResizing = false;
            proxy._allowDragging = false;
            proxy._rightResizing = false;
            proxy._progressResizing = false;
        },


        //update offset values for predecessor
        updateConnectorLineOffset: function (item) {

            var parentGanttRecord,
                model = this.model,
                predecessor = item.predecessor,
                predecessorValue = [],
                parentStartDate,
                parentEndDate,
                weekEndOffset, childGanttRecord, dayOffset;

            if (predecessor) {
                for (var i = 0; i < predecessor.length; i++) {
                    if ((predecessor[i].to === item['taskId'].toString())) {

                        parentGanttRecord = model.rows[model._ids.indexOf(predecessor[i].from)];
                        childGanttRecord = model.rows[model._ids.indexOf(predecessor[i].to)];
                        parentStartDate = new Date(parentGanttRecord.startDate);
                        parentEndDate = new Date(parentGanttRecord.endDate);

                        if (predecessor[i].predecessorsType == "FS") {

                            if (parentEndDate.getDay() >= 5) {
                                weekEndOffset = 2;
                            }
                            if (childGanttRecord.left < (parentGanttRecord.left + parentGanttRecord.width)) {

                                dayOffset = ((parentGanttRecord.left + parentGanttRecord.width)
                                    - childGanttRecord.left) / this.model.perDayWidth;

                                dayOffset = childGanttRecord.isMilestone == true ? (dayOffset - 1) : dayOffset;
                                predecessor[i].offset = Math.floor(-dayOffset);
                            }
                            else {
                                dayOffset = (childGanttRecord.left - (parentGanttRecord.left + parentGanttRecord.width))
                                    / this.model.perDayWidth;

                                if (weekEndOffset && this.model.includeWeekend == false)
                                    dayOffset = dayOffset - weekEndOffset;
                                if (dayOffset >= 0)
                                    predecessor[i].offset = Math.floor(dayOffset);
                            }
                        }

                        if (predecessor[i].predecessorsType == "SS") {
                            if (childGanttRecord.left < parentGanttRecord.left) {
                                dayOffset = ((parentGanttRecord.left) - childGanttRecord.left) / this.model.perDayWidth;
                                predecessor[i].offset = Math.floor(-dayOffset);
                            }
                            else if (childGanttRecord.left > parentGanttRecord.left) {
                                dayOffset = (childGanttRecord.left - (parentGanttRecord.left)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.floor(dayOffset);
                            }
                        }

                        if (predecessor[i].predecessorsType == "SF") {

                            if (parentStartDate.getDay() == 5) {
                                weekEndOffset = 2 * this.model.perDayWidth;
                            }
                            if ((childGanttRecord.left + childGanttRecord.width) < parentGanttRecord.left) {

                                dayOffset = ((parentGanttRecord.left) - (childGanttRecord.left + childGanttRecord.width))
                                    / this.model.perDayWidth;
                                if (weekEndOffset && this.model.includeWeekend == false)
                                    dayOffset = dayOffset - weekEndOffset;
                                if (dayOffset > 0)
                                    predecessor[i].offset = Math.floor(-dayOffset);
                            } else if ((childGanttRecord.left + childGanttRecord.width) > parentGanttRecord.left) {

                                dayOffset = ((childGanttRecord.left + childGanttRecord.width) -
                                    (parentGanttRecord.left)) / this.model.perDayWidth;
                                if (weekEndOffset && this.model.includeWeekend == false)
                                    dayOffset = dayOffset - weekEndOffset;
                                if (dayOffset > 0)
                                    predecessor[i].offset = Math.floor(dayOffset);
                            }
                        }

                        if (predecessor[i].predecessorsType == "FF") {

                            if ((childGanttRecord.left + childGanttRecord.width) < (parentGanttRecord.left + parentGanttRecord.width)) {
                                dayOffset = ((parentGanttRecord.left + parentGanttRecord.width) -
                                    (childGanttRecord.left + childGanttRecord.width)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.floor(-dayOffset);
                            }
                            else if ((childGanttRecord.left + childGanttRecord.width) > (parentGanttRecord.left + parentGanttRecord.width)) {
                                dayOffset = ((childGanttRecord.left + childGanttRecord.width) -
                                    (parentGanttRecord.left + parentGanttRecord.width)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.floor(dayOffset);
                            }
                        }


                        if (predecessor[i].offset < 0) {
                            predecessorValue.push(predecessor[i].from + "" + predecessor[i].predecessorsType + ""
                                + predecessor[i].offset + "d");
                        }
                        else if (predecessor[i].offset >= 1) {
                            predecessorValue.push(predecessor[i].from + "" + predecessor[i].predecessorsType + "+"
                                + predecessor[i].offset + "d");
                        }
                        else {
                            predecessorValue.push(predecessor[i].from);
                        }
                    }
                }
                if (predecessorValue.length > 0) {
                    item.item[model.predecessorMapping] = predecessorValue.join(',');
                    //this.refreshRow(itemIndex, false);
                }
            }
        },


        //Returns the previous item properties
        _getPrevItem: function (item, index) {
            var proxy = this;
            item.left = proxy._prevItem.left;
            item.width = proxy._prevItem.width;
            item.startDate = proxy._prevItem.startDate;
            item.endDate = proxy._prevItem.endDate;
            item.duration = proxy._prevItem.duration;
            item.status = proxy._prevItem.status;
            item.progressWidth = this._getProgressWidth(item.width, item.status);
            this.refreshRow(index);
        },

        //handler for left resizing taskbar item
        _enableLeftResizing: function (e, item) {

            var proxy = this;
            e.target.style.cursor = "e-resize";
            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    if ((item.left + 30) < (item.left + item.width)) {
                        item.left -= proxy._mousePosX;
                    }
                    item.width += proxy._mousePosX;
                }
            }
            else {
                proxy._mousePosX -= proxy.dragPosX;
                if ((item.left + 30) < (item.left + item.width)) {
                    item.left += proxy._mousePosX;
                }
                item.width -= proxy._mousePosX;
            }

            proxy.dragPosX = e.pageX - 400;
            item.width = item.width < proxy.model.perDayWidth ? proxy.model.perDayWidth : item.width;
            item.progressWidth = proxy._getProgressWidth(item.width, item.status);

            proxy._setItemPosition(item);

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },

        //handler for progressbar resizing
        _enableProgressResizing: function (e, item) {

            var proxy = this,
                diff,
                radius;
            e.target.style.cursor = "col-resize";
            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy._progressResizer) {
                proxy._progressHandle = proxy._progressResizer.firstChild;
                proxy._progressHandleChild = proxy._progressResizer.childNodes[1];
            }

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.progressWidth -= proxy._mousePosX;
                }
            }
            else {
                proxy._mousePosX -= proxy.dragPosX;
                item.progressWidth += proxy._mousePosX;
            }

            proxy.dragPosX = e.pageX - 400;
            item.progressWidth = item.progressWidth > item.width ? item.width : item.progressWidth;
            item.progressWidth = item.progressWidth < 0 ? 0 : item.progressWidth;

            diff = item.width - item.progressWidth;

            if (diff <= 4) {
                radius = 4 - diff;
            }
            else
                radius = 0;

            $(proxy._progressbarItem).css({
                "width": (item.progressWidth) + "px",
                "left": (item.left) + "px",
                "border-top-right-radius": radius + "px",
                "border-bottom-right-radius": radius + "px"
            });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });

            $(proxy._progressHandle).addClass("e-progresshandle");
            $(proxy._progressHandleChild).addClass("e-progresshandleafter");
        },


        //handler for right resizing taskbar
        _enableRightResizing: function (e, item) {

            var proxy = this;
            e.target.style.cursor = "w-resize";

            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.width -= proxy._mousePosX;
                }
            } else {
                proxy._mousePosX -= proxy.dragPosX;
                item.width += proxy._mousePosX;
            }

            proxy.dragPosX = e.pageX - 400;
            item.width = item.width < proxy.model.perDayWidth ? proxy.model.perDayWidth : item.width;
            item.progressWidth = proxy._getProgressWidth(item.width, item.status);

            $(proxy._taskbarItem).css({ "left": (item.left) + "px", "width": (item.width) + "px" });
            $(proxy._progressbarItem).css({ "width": (item.progressWidth) + "px", "left": (item.left) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({ "left": (item.left) + "px", "width": (item.width) + "px" });

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },


        //Handler for dragging taskbar
        _enableDragging: function (e, item) {

            var proxy = this;
            e.target.style.cursor = "move";

            if ($(e.target.parentNode).hasClass("e-parentContainer")) {
                proxy._allowExpandCollapse = false;
            }

            if (this.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.left -= proxy._mousePosX;
                }
            } else {
                proxy._mousePosX -= proxy.dragPosX;
                item.left += proxy._mousePosX;
            }

            this.dragPosX = e.pageX - 400;
            item.left = item.left < 0 ? 0 : item.left;

            $(proxy._progressbarItem).css({ "left": (item.left) + "px" });
            $(proxy._taskbarItem).css({ "left": (item.left) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._leftResizer).css({ "left": (item.left + 2) + "px" });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },


        //Append tooltip while editing taskbar
        _appendTooltip: function (target, index) {

            var proxy = this, tooltipElement, tooltiptable, tooltipbody,
                $target = $(target),
                item = proxy._currentViewData[index],
                columnHeaderTexts = proxy.model.columnHeaderTexts;

            if (item != null) {
                if ((proxy._currMousePosX + 150) > proxy._windowWidth) {
                    proxy._currMousePosX -= (proxy._currMousePosX + 150) - proxy._windowWidth;
                }

                var div = $target[0];

                if ($(div).hasClass("e-parentContainer") || $(div).hasClass("e-childContainer")) {
                    proxy._editingContainer = div;
                    proxy._leftResizer = proxy._editingContainer.childNodes[0];
                    proxy._taskbarItem = proxy._editingContainer.childNodes[1];
                    proxy._progressbarItem = proxy._editingContainer.childNodes[2];
                    proxy._progressResizer = proxy._editingContainer.childNodes[3];
                    proxy._rightResizer = proxy._editingContainer.childNodes[4];
                }

                //Don't need to display the drag tooltip in Parent Taskbar
                if (this.model.showTaskbarDragTooltip && (!$target.hasClass('e-parentContainer'))) {

                    if (this._progressResizing == true) {

                        proxy._$ganttChartContainer[0].style.cursor = "col-resize";

                        tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '2px', 'cellpadding': '2px' });
                        tooltipbody = ej.buildTag("tbody", "", {}, {});
                        tooltipbody[0].innerHTML = "<tr><td>" + columnHeaderTexts["status"] + " : " + item.status + "%</td></tr>";
                        tooltiptable.append(tooltipbody);
                        if (proxy.model.progressbarTooltipTemplateID == null) {
                            proxy._progressBarTooltipID = ej.buildTag("div.e-progressbartooltip#progressbartooltip" + proxy._id + "",
                                tooltiptable,
                                {
                                    "top": (proxy._currMousePosY - 65) + "px",
                                    "left": proxy._currMousePosX + "px",
                                    'position': 'absolute',
                                    'z-index': '4'
                                }, { 'onselectstart': 'return false' });
                        } else {
                            proxy._progressBarTooltipID = proxy._getTemplate(proxy.model.progressbarTooltipTemplateID);
                            $(proxy._progressBarTooltipID).css({
                                "left": proxy._currMousePosX + "px",
                                "top": (proxy._currMousePosY - 65) + "px",
                                "position": "absolute"
                            });
                            $(proxy._progressBarTooltipID).addClass("customTooltip");
                            proxy._progressBarTooltipID.innerHTML = "<table>" + tooltiptable[0].innerHTML + "</table>";
                        }
                        $(document.body).append(proxy._progressBarTooltipID);
                    }
                    else {
                        if (proxy._leftResizing == true) {
                            proxy._$ganttChartContainer[0].style.cursor = "e-resize";

                            tooltipElement = {
                                ttipstartDate: item.startDate,
                                ttipduration: item.duration,
                            };
                        } else if (proxy._rightResizing == true) {

                            proxy._$ganttChartContainer[0].style.cursor = "w-resize";

                            tooltipElement = {
                                ttipendDate: item.endDate,
                                ttipduration: item.duration,
                            };
                        } else if (proxy._allowDragging == true) {
                            proxy._$ganttChartContainer[0].style.cursor = "move";

                            tooltipElement = {
                                ttipstartDate: item.startDate,
                                ttipendDate: item.endDate,
                            };
                        }
                        if (tooltipElement) {
                            tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '2px', 'cellpadding': '2px' }),
                            tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](tooltipElement), {}, {});
                            tooltiptable.append(tooltipbody);
                        }

                        if (proxy.model.taskbarEditingTooltipTemplateID) {
                            proxy._taskbarEditingTooltipID = proxy._getTemplate(proxy.model.taskbarEditingTooltipTemplateID);
                            $(proxy._taskbarEditingTooltipID).css({
                                "left": proxy._currMousePosX + "px",
                                "top": (proxy._currMousePosY - 85) + "px",
                                "position": "absolute"
                            });
                            $(proxy._taskbarEditingTooltipID).addClass("customTooltip");
                            proxy._taskbarEditingTooltipID.innerHTML = "<table>" + tooltiptable[0].innerHTML + "</table>";
                        }
                        else if (tooltiptable) {
                            proxy._taskbarEditingTooltipID = ej.buildTag("div.e-editingtooltip#editingtooltip" + proxy._id + "", tooltiptable,
                                {
                                    "top": (proxy._currMousePosY - 85) + "px",
                                    "left": (proxy._currMousePosX) + "px",
                                    'position': 'absolute',
                                    'z-index': '4'
                                }, {});
                        }
                        if (proxy._taskbarEditingTooltipID) {
                            $(document.body).append(proxy._taskbarEditingTooltipID);
                        }
                    }
                }
            }
        },

        _setItemPosition: function (item) {
            var proxy = this;
            $(proxy._leftResizer).css({ "left": (item.left + 2) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._taskbarItem).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });
            $(proxy._progressbarItem).css({
                "left": (item.left) + "px",
                "width": (item.progressWidth) + "px"
            });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });
        },

        //returns the custom template for tooltip
        _getTemplate: function (template) {
            if (typeof (template) != "object") {
                var parser = $.parseHTML(this.model.tooltipTemplate);
                return parser[0];
            } else {
                var markup = $(template)[0].innerHTML;
                parser = $.parseHTML(markup);
                return parser[1];
            }
        },


        //Handles the mousemove after mousedown event
        _updateOnmouseMove: function (e) {

            var proxy = this, args = {}, posy,
                $target = proxy._editingTarget,
                $ganttGridRows = $(this.getGanttChartRows()),
                row = $target.closest('tr'),
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem,
                div = $target[0].parentNode;

            if ($(div).hasClass("progressbarresizer-right") || $(div).hasClass("e-gantt-childtaskbar-progress")) {
                div = div.parentNode;
            }

            if (item != null) {

                args.editingFields = {
                    startdate: item.startDate,
                    enddate: item.endDate,
                    progress: item.status,
                    duration: item.duration
                };
                args.rowData = { rowData: item };

                proxy._taskbarEditing(args);

                if (!args.cancel) {

                    if (!e) e = window.event;
                    if (e.pageX || e.pageY) {
                        proxy._mousePosX = e.pageX - 400;
                        posy = e.pageY;
                    } else if (e.clientX || e.clientY) {
                        proxy._mousePosX = e.clientX + document.body.scrollLeft
                            + document.documentElement.scrollLeft - 400;
                        posy = e.clientY + document.body.scrollTop
                            + document.documentElement.scrollTop;
                    }

                    if ($target.hasClass("e-gantt-milestone")) {
                        if ($(row).hasClass("e-editmode")) {
                            this._editMilestone(e);
                        }
                    } else {

                        if ($(div).hasClass("e-childContainer")) {
                            proxy._editingContainer = div;
                            proxy._leftResizer = proxy._editingContainer.childNodes[0];
                            proxy._taskbarItem = proxy._editingContainer.childNodes[1];
                            proxy._progressbarItem = proxy._editingContainer.childNodes[2];
                            proxy._progressResizer = proxy._editingContainer.childNodes[3];
                            proxy._rightResizer = proxy._editingContainer.childNodes[4];
                            proxy._tasknameContainer = proxy._editingContainer.childNodes[5];
                        }

                        if ($(row).hasClass("e-editmode") &&
                            !($(div).hasClass("e-parentContainer"))) {

                            if (proxy._leftResizing == true) {
                                proxy._enableLeftResizing(e, item);
                                proxy._updateEditedItem(item, "leftResizing");

                            } else if (proxy._progressResizing == true
                                && proxy.model.canResizeProgressBar
                                && proxy.model.progressMapping) {
                                proxy._enableProgressResizing(e, item);
                                proxy._updateEditedItem(item, "progressResizing");

                            } else if (proxy._rightResizing == true) {
                                proxy._enableRightResizing(e, item);
                                proxy._updateEditedItem(item, "rightResizing");

                            } else if (proxy._allowDragging == true) {
                                proxy._enableDragging(e, item);
                                proxy._updateEditedItem(item, "dragging");
                            }
                        }
                    }
                    proxy._clearTooltip();
                    proxy._appendTooltip($target, recordIndexr);
                }
            }
        },


        //Handler for milestone editing
        _editMilestone: function (e) {

            var proxy = this,
                $target = proxy._editingTarget,
                div = $target[0].parentNode,
                $ganttGridRows = $(this.getGanttChartRows()),
                row = $target.closest('tr'),
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem;

            if (item != null) {

                if (!e) e = window.event;
                if (e.pageX || e.pageY) {
                    proxy._mousePosX = e.pageX - 400;
                } else if (e.clientX || e.clientY) {
                    proxy._mousePosX = e.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft - 400;
                }

                e.target.style.cursor = "move";
                if (proxy.dragPosX > proxy._mousePosX) {
                    proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                    if (item.left > 0) {
                        item.left -= proxy._mousePosX;
                    }
                } else {
                    proxy._mousePosX -= proxy.dragPosX;
                    item.left += proxy._mousePosX;
                }

                this.dragPosX = e.pageX - 400;
                item.left = item.left < 0 ? 0 : item.left;

                $(div).css({ "left": (item.left) + "px" });

                proxy._updateEditedItem(item, "dragging");
                proxy._appendTooltip($target, recordIndexr);
            }
        },


        //Updates the edited item's values
        _updateEditedItem: function (ganttRecord, reason) {

            var proxy = this,
                model = proxy.model,
                projectStartDate = new Date(model.scheduleWeeks[0]),
                remainDays, day, offset, rem, startDate, endDate;

            switch (reason) {

                case 'dragging':
                    remainDays = ganttRecord.left % model.perDayWidth,
                    day = (ganttRecord.left - remainDays) / model.perDayWidth,
                    offset = 0;
                    if (remainDays > 0) {
                        offset = remainDays >= model.perDayWidth / 2 ? 1 : 0;
                    }

                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                    ganttRecord.startDate = ganttRecord._calculateFormatedDate(projectStartDate, model.dateFormat);
                    ganttRecord._calculateEndDate(model.includeWeekend, model.holidays, model.dateFormat);
                    break;

                case 'leftResizing':
                    remainDays = ganttRecord.left % model.perDayWidth,
                    day = (ganttRecord.left - remainDays) / model.perDayWidth,
                    offset = 0;
                    if (remainDays > 0) {
                        offset = remainDays >= model.perDayWidth / 2 ? 1 : 0;
                    }

                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                    //if (projectStartDate < new Date(ganttRecord.endDate)) {
                    ganttRecord.startDate = ganttRecord._calculateFormatedDate(projectStartDate, model.dateFormat);
                    ganttRecord._calculateDuration(projectStartDate, new Date(ganttRecord.endDate),
                        model.includeWeekend, model.holidays, false);
                    //}
                    if (Globalize.parseDate(ganttRecord.startDate, this.model.dateFormat).getTime() == Globalize.parseDate(ganttRecord.endDate, this.model.dateFormat).getTime()
                        && ej.isNullOrUndefined(ganttRecord.isMilestone) && ganttRecord.isMilestone == false && ganttRecord.duration == 0) {
                        ganttRecord.duration = 1;
                    }
                    break;

                case 'rightResizing':
                    day = ganttRecord.width / model.perDayWidth,
                    rem = ganttRecord.width % model.perDayWidth,
                    offset = rem >= (model.perDayWidth / 2) ? 1 : 0,
                    startDate = new Date(ganttRecord.startDate),
                    endDate = ganttRecord._dateUpdate(startDate, day - 1 + offset);
                    ganttRecord.endDate = ganttRecord._calculateFormatedDate(endDate, model.dateFormat);

                    if (model.holidays && model.holidays.length) {
                        var holidayIndex = 0,
                            holidayslength = model.holidays.length,
                            holiday;
                        for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                            holiday = new Date(model.holidays[holidayIndex]);
                            if (endDate === holiday) {
                                endDate.setDate(endDate.getDate() - 1);
                            }
                        }
                    }
                    if (this.model.includeWeekend == false) {
                        if (endDate.getDay() == 0)
                            endDate.setDate(endDate.getDate() - 2);
                        else if (endDate.getDay() == 6)
                            endDate.setDate(endDate.getDate() - 1);
                    }
                    ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays, false);
                    break;

                case 'progressResizing':
                    ganttRecord.status = proxy._getProgressPercent(ganttRecord.width, ganttRecord.progressWidth);
                    break;
            }
        },

        _mouseOverTooltip: function () {

            $(proxy._mouseOverTooltip).remove();

            if (this._mouseDown == false) {
                $(proxy._leftResizerGripper).removeClass("gripper");
                $(proxy._rightResizerGripper).removeClass("gripper");

                if (proxy.model.progressMapping) {
                    $(proxy._progressHandle).removeClass("e-progresshandle");
                    $(proxy._progressHandleChild).removeClass("e-progresshandleafter");
                }
            }
        },

        _mouseLeave: function () {

            var proxy = this;
            $(".e-tooltipgantt").remove();
            $(".progressbarresizer-right").css({ "background-image": "none" });
            $(proxy._mouseOverTooltip).remove();

            if (this._mouseDown == false) {
                $(proxy._leftResizerGripper).removeClass("gripper");
                $(proxy._rightResizerGripper).removeClass("gripper");

                if (proxy.model.progressMapping) {
                    $(proxy._progressHandle).removeClass("e-progresshandle");
                    $(proxy._progressHandleChild).removeClass("e-progresshandleafter");
                }
            }
        },

        _mouseHover: function (e) {

            var $target = $(e.target), proxy = this,
                posx = 0,
                posy = 0,
                $ganttGridRows = $(proxy.getGanttChartRows()),
                row = $target.closest('tr'),
                div = $target[0].parentNode,
                progressHandler,
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._currentViewData[recordIndexr],
                tooltipElement, tooltiptable, tooltipbody;

            if (proxy._rightResizing == false && proxy._leftResizing == false &&
                proxy._progressResizing == false && proxy._mouseDown == false) {

                if ($target.hasClass('e-gantt-parenttaskbar-innerdiv')
                    || $target.hasClass('e-gantt-parenttaskbar-progress')) {
                } else if (proxy.model.allowGanttChartEditing) {
                    e.target.style.cursor = "move";
                }


                if (e.type == "mouseenter" || e.type == "mousemove") {

                    if (!e) e = window.event;
                    if (e.pageX || e.pageY) {
                        posx = e.pageX;
                        posy = e.pageY;
                    }
                    else if (e.clientX || e.clientY) {
                        posx = e.clientX + document.body.scrollLeft
                            + document.documentElement.scrollLeft;
                        posy = e.clientY + document.body.scrollTop
                            + document.documentElement.scrollTop;
                    }

                    if ((posx + 200) > proxy._windowWidth) {
                        posx -= (posx + 200) - proxy._windowWidth;
                    }

                    if ((posy + 150) > proxy._windowHeight) {
                        posy -= (posy + 150) - proxy._windowHeight;
                    }

                    if ($(div).hasClass("e-childContainer") && proxy.model.allowGanttChartEditing) {
                        proxy._leftResizerGripper = div.childNodes[0],
                        proxy._rightResizerGripper = div.childNodes[4];
                        progressHandler = div.childNodes[3];
                        proxy._progressHandle = progressHandler.firstChild;
                        proxy._progressHandleChild = progressHandler.childNodes[1];

                        $(proxy._leftResizerGripper).addClass("gripper");
                        $(proxy._rightResizerGripper).addClass("gripper");

                        if (proxy.model.progressMapping) {
                            $(proxy._progressHandle).addClass("e-progresshandle");
                            $(proxy._progressHandleChild).addClass("e-progresshandleafter");
                        }
                    }
                    if (proxy.model.showTaskbarTooltip) {
                        tooltipElement = {
                            ttipstartDate: item.startDate,
                            ttipendDate: item.endDate,
                            ttipduration: item.duration,
                            ttipprogress: item.status,
                            ttiptaskname: item.taskName,
                        };

                        tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '5' });
                        tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](tooltipElement), {}, {});
                        tooltiptable.append(tooltipbody);

                        if (proxy.model.tooltipTemplate == null) {
                            proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", tooltiptable,
                                { "top": (posy + 10) + "px", "left": (posx + 15) + "px", 'position': 'absolute', 'z-index': '4', 'border-radius': '5px' }, {});
                        } else {
                            if (typeof (proxy.model.tooltipTemplate) != "object") {
                                var parser = $.parseHTML(proxy.model.tooltipTemplate);
                                proxy._mouseOverTooltip = parser[0];
                            } else {
                                var markup = $(proxy.model.tooltipTemplate)[0].innerHTML;
                                parser = $.parseHTML(markup);
                                proxy._mouseOverTooltip = parser[1];
                            }

                            $(proxy._mouseOverTooltip).css({
                                "left": (posx + 15) + "px",
                                "top": (posy) + "px",
                                "position": "absolute"
                            });
                            $(proxy._mouseOverTooltip).addClass("customTooltip");
                            proxy._mouseOverTooltip.innerHTML = "<table>" + tooltiptable[0].innerHTML + "</table>";
                        }
                        $(document.body).append(proxy._mouseOverTooltip);
                    }
                }
            }
        },


        //Template to create tooltip
        _createTooltipTemplate: function () {

            var proxy = this,
                td,
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                helpers = {
                    _getStartDate: proxy._tooltipStartDate,
                    _getEndDate: proxy._tooltipEndDate,
                    _getDuration: proxy._tooltipDuration,
                    _getTaskName: proxy._tooltipTaskName,
                    _getProgress: proxy._tooltipProgress
                };

            $.views.helpers(helpers);
            var parentTr = "<tr class='e-tooltip_rowcell'>";

            td = "{{if ~_getTaskName()}}" +
                "<td class='e-tooltiptaskname' style='width:40px;text-align:left;' colspan='3';>{{:ttiptaskname}}</td>{{/if}}" +
                "</tr><tr>" +
                "{{if ~_getStartDate()}}" +
                "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["startDate"] + "</td>" +
                "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipstartDate}}</td> {{/if}}" +
                "</tr><tr>" +
                "{{if ~_getEndDate()}}" +
                "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["endDate"] + "</td>" +
                "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipendDate}}</td>{{/if}}" +
                "</tr><tr>" +
                "{{if ~_getDuration()}}" +
                "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["duration"] + "</td>" +
                "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipduration}} days</td>{{/if}}" +
                "</tr><tr>" +
                "{{if ~_getProgress()}}" +
                "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["status"] + "</td>" +
                "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipprogress}}%</td>{{/if}}";

            parentTr += td;
            parentTr += "</tr>";

            var templates = {};
            templates[proxy._id + "tooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        _tooltipTaskName: function () {
            return this.data.ttiptaskname;
        },

        _tooltipProgress: function () {
            return this.data.ttipprogress;
        },

        _tooltipStartDate: function () {
            return this.data.ttipstartDate;
        },

        _tooltipEndDate: function () {
            return this.data.ttipendDate;
        },

        _tooltipDuration: function () {
            return this.data.ttipduration;
        },

        _cellClickHandler: function (e) {
            var $target = $(e.target),
                row = $target.closest('tr'),
                proxy = this,
                args = {},
                selectedItem,
                index,
                $ganttGridRows = $(proxy.getGanttChartRows());

            //proxy._clearContextMenu();
            if (proxy.model.selectedRowIndex !== -1) {

                selectedItem = proxy._currentViewData && proxy._currentViewData[proxy.model.selectedRowIndex];
                args = {
                    rowElement: $ganttGridRows.eq(proxy.model.selectedRowIndex),
                    data: selectedItem,
                    recordIndex: proxy.model.selectedRowIndex
                };
            }

            if (proxy._trigger('rowSelecting', args))  //check the cancel arguements in rowSelecting event
                return;

            index = $ganttGridRows.index($target.parent());

            if (index == -1) {
                index = $ganttGridRows.index(row);
            }

            if (index !== -1) {
                proxy.selectRows(index);
                proxy.model.selectedRowIndex = index;
                args.target = "ejGanttChart";
                selectedItem = proxy._currentViewData[index];
                args.data = selectedItem;
                args.recordIndex = index;
                proxy._trigger("rowSelected", args);

                if (proxy.prevSelectedItem != null) {
                    proxy.prevSelectedItem.IsSelected = false;
                    proxy._currentViewData[index].IsSelected = true;
                }

                proxy.prevSelectedItem = selectedItem;

                proxy.element[0].focus();
            }

        },

        selectRows: function (index) {
            var proxy = this,
                $ganttGridRows = $(proxy.getGanttChartRows()),
                selectedItem;

            $ganttGridRows.removeClass("e-gantt-mouseclick");
            var row = $($ganttGridRows[index]);
            row.addClass("e-gantt-mouseclick");
            proxy.model.selectedRowIndex = index;
            selectedItem = proxy._currentViewData[index];
            proxy.model.selectedItem = selectedItem;
        },

        updateSelectedItem: function (recordIndex) {
            var proxy = this,
                $ganttGridRows = $(proxy.getGanttChartRows());

            $ganttGridRows.removeClass("e-gantt-mouseclick");
            proxy.model.ganttRecords[recordIndex].isSelected = true;
            proxy.model.selectedItem = proxy.model.ganttRecords[recordIndex];
            this._refresh();
            proxy._rowSelectedEventTrigger(recordIndex);

        },


        ganttrowHover: function (index) {
            var proxy = this;
            if (!proxy.model.enableRowHover)
                return;
            var $gridRows = $(this.getGanttChartRows());
            $gridRows.removeClass("e-gantt_mousehover").addClass("e-gantt_mouseleave");;
            $($gridRows[index]).addClass("e-gantt_mousehover").removeClass("e-gantt_mouseleave");
        },

        _rowHover: function (e) {
            var proxy = this,
                $target = $(e.target),
                $gridRows,
                args = {},
                index,
                row;
            if (!proxy.model.enableRowHover)
                return false;
            $gridRows = $(proxy.getGanttChartRows());
            index = $gridRows.index($target.parent());
            if (index == -1) {
                row = $target.closest('tr');
                index = $gridRows.index(row);
            }
            args.index = index;

            if (e.type == "mouseenter") {
                args.reason = "mouseenter";
                proxy.addRowHover(index);
            } else {
                args.reason = "mouseleave";
                proxy.removeRowHover();
            }
            proxy._trigger("rowHover", args);
            return false;
        },

        addRowHover: function (index) {
            var $gridRows = $(this.getGanttChartRows());
            if (index != null)
                index != -1 && $($gridRows[index]).addClass("e-enablerowhover");
        },

        removeRowHover: function () {
            var $gridRows = $(this.getGanttChartRows());
            $gridRows.removeClass("e-enablerowhover");
        },

        updateHeight: function (height) {
            var proxy = this,
                model = proxy.model;

            if (model.stripLines) {
                $(".e-stripline,.e-stripLines").css({ height: height + "px" });
            }

            if (model.holidays) {
                $(".e-holidays,.e-holiday").css({ height: height + "px" });
            }
            $(".e-secondary_canvas").css({ height: height + "px" });
            $(".e-weekends").css({ height: height + "px" });
        },


        //method to trigger actioncomplete event
        _completeAction: function (args) {
            var proxy = this;
            proxy._trigger("actionComplete", args);
        },


        //method to trigger taskbarediting event
        //to apply changes during taskbar editing
        _taskbarEditing: function (args) {
            var proxy = this;
            proxy._trigger("taskbarEditing", args);
        },


        //method to trigger taskbaredited event
        //to apply changes after taskbar edited
        _taskbarEdited: function (args) {
            var proxy = this;
            proxy._trigger("taskbarEdited", args);
        },


        //method to trigger querytaskbarinfo 
        //to apply changes at load time
        _queryTaskbarinfo: function (args) {
            this._trigger("queryTaskbarInfo", args);
            this._updateTaskbarItems(args);
        },


        //method to trigger refresh row
        _refreshRow: function (args) {
            var proxy = this;
            proxy._trigger("refreshRow", args);
        },

        _enbleZooming: function (args) {
            var proxy = this;
            proxy._trigger("zooming", args);
        },


        //Updates taskbar items styles when chart loads
        _updateTaskbarItems: function (args) {

            var $ganttChartRows = $(this.getGanttChartRows()),
                index = this.model.rows.indexOf(args.data),
                taskbarContainer = $ganttChartRows[index], taskElement;

            if (taskbarContainer) {
                if (taskbarContainer.childNodes.length > 1)
                    taskElement = taskbarContainer.childNodes[1].firstChild;
                else
                    taskElement = taskbarContainer.childNodes[0].firstChild;

                if ($(taskElement).hasClass("e-parentContainer")) {

                    var parentTaskbar = taskElement.firstChild,
                        parentProgressbar = taskElement.childNodes[1],
                        parentTaskbarStyle = parentTaskbar.style,
                        parentProgressbarStyle = parentProgressbar.style;

                    if (parentTaskbarStyle.backgroundColor != args.parentTaskbarBackground) {
                        parentTaskbarStyle.backgroundColor = args.parentTaskbarBackground;
                    }
                    if (parentProgressbarStyle.backgroundColor != args.parentProgressbarBackground) {
                        parentProgressbarStyle.backgroundColor = args.parentProgressbarBackground;
                    }

                } else if ($(taskElement).hasClass("e-childContainer")) {

                    var taskbar = taskElement.childNodes[1],
                        progressbar = taskElement.childNodes[2],
                        taskbarStyle = taskbar.style,
                        progressbarStyle = progressbar.style;

                    if (taskbarStyle.backgroundColor != args.taskbarBackground) {
                        taskbarStyle.backgroundColor = args.taskbarBackground;
                    }
                    if (progressbarStyle.backgroundColor != args.progressbarBackground) {
                        progressbarStyle.backgroundColor = args.progressbarBackground;
                    }
                }
            }
        },

        //to perform expand collapse operation from ganttchart
        _expandCollapse: function (recordIndex) {

            var proxy = this,
                args = {},
                data = proxy._currentViewData[recordIndex];

            args.data = data;
            args.recordIndex = recordIndex;
            args.source = "ganttChart";
            args.expanded = !data.expanded;
            if (data) {
                if (!args.expanded) {
                    proxy._trigger("collapsing", args);
                }
                else {
                    proxy._trigger("expanding", args);
                }
            }
        },


        //get the Expand status of GridRecord
        _getExpandStatus: function (record) {
            var parentRecord = record.parentItem;
            if (parentRecord != null) {
                if (parentRecord.expanded === false)
                    return false;
                else if (parentRecord.parentItem) {
                    if (parentRecord.parentItem.expanded === false)
                        return false;
                    else
                        return this._getExpandStatus(parentRecord.parentItem);
                }
                else
                    return true;
            } else
                return true;
        },



        //method to scroll ganttchart from gridtree scroll event
        onScrollHelper: function (scrolltop) {

            var div = $("#ganttviewerbodyContianer" + this._id);
            div.scrollTop(scrolltop);
            this._handleScroll();
        },


        //Handler for scroll event from ganttchart
        _handleScroll: function () {
            var proxy = this,
                args = {},
                headerdiv = proxy._$headerContainer,
                contentdiv = proxy._$bodyContainer;

            proxy._scrollTop = contentdiv.scrollTop();

            var left = contentdiv.scrollLeft(),
                diff = (left - proxy._prevLeft),
                vDiff = proxy._scrollTop - proxy._prevScrollTop;

            if (diff)
                headerdiv.scrollLeft(left);
            if (proxy.model.enableVirtualization) {
                proxy._vscrollDist = Math.abs(proxy._scrollTop - proxy._prevScrollTop);
                if (proxy._vscrollDist) {
                    proxy._vScrollDir = proxy._prevScrollTop <= proxy._scrollTop ? 1 : -1;
                    proxy._updateCurrentViewData();
                    proxy.renderGanttRecords();
                    proxy._prevScrollTop = proxy._scrollTop;
                }
            }
            args.requestType = "scroll";
            args.delta = proxy._scrollTop;
            proxy._prevLeft = left;
            proxy._prevScrollTop = proxy._scrollTop;
            if (diff === 0 && vDiff !== 0)
                proxy._completeAction(args);
        },


        //helper method to refresh the ganttchart content from gantt
        refreshHelper: function (currentviewdata, records) {

            var proxy = this;
            proxy._currentViewData = currentviewdata;
            proxy.model.updatedGanttRecords = records;
            if (records) {
                proxy._setHeight(records.length * proxy.model.rowHeight);
                if (proxy.model.enableVirtualization) {
                    $("#ganttviewerbodyContent" + proxy._id + "").css({
                        "height": proxy._containerHeight + "px"
                    });
                    proxy.updateHeight(proxy._containerHeight);
                }
                proxy._refresh();
            }
        },


        //to refresh the content of the ganttchart
        _refresh: function () {

            var proxy = this,
                contentdiv = $("#ganttViewTablebody" + proxy._id + ""),
                data = $.render[proxy._id + "_CustomTemplate2"](proxy._currentViewData);

            contentdiv.empty().append(data);
            if (contentdiv[0] != null) {
                this._ganttChartRows = contentdiv[0].childNodes;
            }
            else {
                proxy._$ganttViewTablebody = ej.buildTag("tbody.e-gantt-viewer-ganttViewTablebody#ganttViewTablebody" + proxy._id + "",
                    data, {}, {});
                proxy._ganttChartRows = proxy._$ganttViewTablebody[0].childNodes;
            }
            proxy._eventBindings();
        },


        //to rerender a particular ganttchart row
        refreshRow: function (index) {

            var proxy = this,
                $tr = $(proxy.getGanttChartRows()[index]),
                selectedItem = proxy._currentViewData[index];
            //rowClass = "gridrowtaskId";

            if (index != -1 && selectedItem) {
                //$tr.empty().html($($.render[proxy._id + "_CustomTemplate2"](selectedItem))[0].innerHTML);
                //if (selectedItem.parentItem)
                //    rowClass += selectedItem.parentItem.taskId.toString();
                //rowClass += "level";
                //rowClass += selectedItem.level.toString();
                //$tr.addClass(rowClass);
                $tr.replaceWith($($.render[proxy._id + "_CustomTemplate2"](selectedItem)));
            }
        },

        renderRecord: function (args) {
            var proxy = this,
                addedchartRow = $.render[this._id + "_CustomTemplate2"](args._cAddedRecord);

            if (args.recordIndex == 0) {
                $(proxy._ganttChartRows).eq(args.recordIndex).before(addedchartRow);
            } else {
                $(proxy._ganttChartRows).eq(args.recordIndex).after(addedchartRow);
            }

            if (proxy._ganttChartRows == null) {
                proxy._ganttChartRows = addedchartRow;
                proxy._refresh();
            }

            proxy._eventBindings();
        },
        //delete gantt chart rows without rerendering of whole page
        deleteChartRows: function (startIndex, count) {
            var proxy = this;
            var $chartRows = $(proxy.getGanttChartRows());
            for (var i = startIndex; i < startIndex + count; i++) {
                $chartRows.splice(i, 1);
            }
        },


        //helper method to bind querytaskbarinfo events to ganttchart rows
        _eventBindings: function () {

            var $ganttChartRows = $(this.getGanttChartRows()),
                rowsLength = $ganttChartRows.length,
                taskbarContainer,
                args = {}, progressbar, taskbar,
                parentProgressbar, parentTaskbar, taskElement;

            if (rowsLength != 0) {

                for (var row = 0; row < rowsLength; row++) {

                    taskbarContainer = $ganttChartRows[row];

                    if ($(taskbarContainer).hasClass("e-ganttrowcell")) {

                        if (taskbarContainer.childNodes.length > 1)
                            taskElement = taskbarContainer.childNodes[1].firstChild;
                        else
                            taskElement = taskbarContainer.childNodes[0].firstChild;

                        if ($(taskElement).hasClass("e-parentContainer")) {

                            parentTaskbar = taskElement.firstChild;
                            parentProgressbar = taskElement.childNodes[1];
                        } else if ($(taskElement).hasClass("e-childContainer")) {

                            taskbar = taskElement.childNodes[1];
                            progressbar = taskElement.childNodes[2];
                        }

                        args.data = this.model.ganttRecords[row];
                        args.taskbar = taskbarContainer;

                        args.taskbarBackground = taskbar != null ?
                            taskbar.style.backgroundColor : null;

                        args.progressbarBackground = progressbar != null ?
                            progressbar.style.backgroundColor : null;

                        args.parentTaskbarBackground = parentTaskbar != null ?
                            parentTaskbar.style.backgroundColor : null;

                        args.parentProgressbarBackground = parentProgressbar != null ?
                            parentProgressbar.style.backgroundColor : null;
                        this._queryTaskbarinfo(args);
                    }
                }
            }
        },


        //to update the currentviewdata of ganttchart
        _updateCurrentViewData: function () {
            var proxy = this, margin = 0;

            if (!this.model.enableVirtualization) {
                this._currentViewData = this.model.updatedGanttRecords;
            }
            else {
                this._getRenderedRowRange();
                this._currentViewData = this.model.updatedGanttRecords.slice(proxy._renderedRange.top,
                    proxy._renderedRange.bottom);
                if (proxy._vscrollDist !== 0) {
                    margin = proxy._scrollTop;
                    margin -= proxy._rowMargin;
                }
                $("#ganttViewTable" + proxy._id + "").css({ "top": margin + "px" });

                proxy._vTop = proxy._visibleRange.top;
                proxy._vBottom = proxy._visibleRange.bottom;
                proxy._rTop = proxy._renderedRange.top;
                proxy._rBottom = proxy._renderedRange.bottom;
            }

        },

        //to render ganttchart content
        renderGanttRecords: function () {

            var proxy = this;
            proxy._refresh();
            proxy._$ganttViewTable.append(proxy._$ganttViewTablebody);
            proxy._$bodyContent.append(proxy._$ganttViewTable);
        },

        _getRenderedRowRange: function () {

            var proxy = this;
            proxy._getVisibleRowRange();
            var topIndex = proxy._visibleRange.top;
            var bottomIndex = proxy._visibleRange.bottom;
            proxy._renderedRange = { top: topIndex, bottom: bottomIndex };
            return proxy._renderedRange;
        },

        _getVisibleRowRange: function () {

            var proxy = this,
                top = proxy._scrollTop / proxy.model.rowHeight,
                coeff = top - Math.floor(top),
                topIndex,
                bottomIndex, length;

            proxy._rowMargin = coeff * proxy.model.rowHeight;
            topIndex = Math.floor(top);
            length = proxy.model.ganttRecords.length;
            bottomIndex = proxy._getrowposition(proxy._scrollTop + proxy._viewportHeight);
            topIndex = Math.max(0, topIndex);
            bottomIndex = Math.min(length, bottomIndex);
            proxy._visibleRange = { top: topIndex, bottom: bottomIndex };
        },

        _getrowposition: function (y) {
            return Math.ceil((y) / this.model.rowHeight);
        },


        //returns the date object
        _getDateType: function (date) {

            if (date != null) {
                if (typeof (date) == "object") {
                    return date;
                } else {
                    var re = new RegExp("\\-", "g");
                    var re1 = new RegExp("\\.", "g");
                    date = date.replace(re, "/");
                    date = date.replace(re1, "/");

                    return new Date(date);
                }
            }
            return null;
        },


        //Template to render schedule week view
        _createScheduleWeekTemplate: function () {

            var proxy = this, td,
                model = proxy.model,
                dateFormat = model.scheduleHeader.weekHeaderFormat,
                helpers = {
                    renderWeek: $.proxy(proxy._getWeeksplit, proxy)
                };

            $.views.helpers(helpers);
            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>";

            td = "<div class='e-schedule-week-headercell-content'  style='width:"
                + (proxy._scheduleWeekWidth - 1) + "px;margin:auto;white-space:nowrap;'>"
                + "{{:~renderWeek('" + dateFormat + "',#data)}}" + "</div>";

            parentTr += td;
            parentTr += "</th>";

            var templates = {};
            templates[proxy._id + "ScheduleWeekTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper method for _createScheduleWeekTemplate
        _getWeeksplit: function (dateFormat, data) {

            var date = new Date(data),
                proxy = this,
                model = proxy.model,
                formatedDate;
            //var values = dateFormat.split(' '),
            //    scheduleUnits = values[0].match(/(\d+|[A-z]+)/g),
            //    month = date.getMonth(),
            //    day = date.getDay() == 1 ? proxy._days("mon") : "",
            //    weekTemplate = [];

            //switch (month) {
            //    case 0:
            //        month = proxy._getLocalizedLabels("jan");
            //        break;
            //    case 1:
            //        month = proxy._getLocalizedLabels("feb");
            //        break;
            //    case 2:
            //        month = proxy._getLocalizedLabels("mar");
            //        break;
            //    case 3:
            //        month = proxy._getLocalizedLabels("apr");
            //        break;
            //    case 4:
            //        month = proxy._getLocalizedLabels("may");
            //        break;
            //    case 5:
            //        month = proxy._getLocalizedLabels("jun");
            //        break;
            //    case 6:
            //        month = proxy._getLocalizedLabels("jul");
            //        break;
            //    case 7:
            //        month = proxy._getLocalizedLabels("aug");
            //        break;
            //    case 8:
            //        month = proxy._getLocalizedLabels("sep");
            //        break;
            //    case 9:
            //        month = proxy._getLocalizedLabels("oct");
            //        break;
            //    case 10:
            //        month = proxy._getLocalizedLabels("nov");
            //        break;
            //    case 11:
            //        month = proxy._getLocalizedLabels("dec");
            //        break;
            //}

            //for (var i = 0; i < scheduleUnits.length; i++) {
            //    switch (scheduleUnits[i]) {//('0' + currentDate.getHours()).slice(-2)
            //        case "d":
            //        case "D":
            //            weekTemplate.push(('0' + date.getDate()).slice(-2));
            //            break;
            //        case "dd":
            //        case "DD":
            //            weekTemplate.push(date.getDate());
            //            break;
            //        case "ddd":
            //        case "DDD":
            //            weekTemplate.push(day.substring(0, 3));
            //            break;
            //        case "dddd":
            //        case "DDDD":
            //            weekTemplate.push(day);
            //            break;
            //        case "M":
            //        case "m":
            //            weekTemplate.push(('0' + date.getMonth() + 1).slice(-2));
            //            break;
            //        case "MM":
            //        case "mm":
            //            weekTemplate.push(date.getMonth() + 1);
            //            break;
            //        case "MMM":
            //        case "mmm":
            //            weekTemplate.push(month.substring(0, 3));
            //            break;
            //        case "MMMM":
            //        case "mmmm":
            //            weekTemplate.push(month);
            //            break;
            //        case "YY":
            //        case "yy":
            //            weekTemplate.push(('0' + date.getFullYear()).slice(-2));
            //            break;
            //        case "YYYY":
            //        case "yyyy":
            //            weekTemplate.push(date.getFullYear());
            //            break;
            //    }
            //}
            formatedDate = Globalize.format(date, dateFormat, model.localization);

            return formatedDate.toLowerCase().replace(/(-(\w)|(^| )(\w)|:(\w)|,(\w)|\/(\w)|\.(\w))/g, function (x) {
                return x.toUpperCase();
            });
        },


        //Template to render schedule day view
        _createScheduleDayTemplate: function () {

            var proxy = this, td, model = proxy.model,
                dayFormat = model.scheduleHeader.dayHeaderFormat,
                helpers = {
                    renderDay: $.proxy(proxy._getDayTemplate, proxy)
                };

            $.views.helpers(helpers);

            var parentTr = "<th  class='e-schedule-day-headercell  e-headercell-weekend' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;" +
                "background-color:" + this.model.scheduleHeader.weekendBackground + "'>";
            td = "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;'>" +
                "{{:~renderDay('" + dayFormat + "',0,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;'>" +
                "{{:~renderDay('" + dayFormat + "',1,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;'>" +
                "{{:~renderDay('" + dayFormat + "',2,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;'>" +
                "{{:~renderDay('" + dayFormat + "',3,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell'" +
                " style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;'>" +
                "{{:~renderDay('" + dayFormat + "',4,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell'" +
                " style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;'>" +
                "{{:~renderDay('" + dayFormat + "',5,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell  e-headercell-weekend' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;" +
                "background-color:" + this.model.scheduleHeader.weekendBackground + "'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;'>" +
                "{{:~renderDay('" + dayFormat + "',6,#data)}}</div>";

            parentTr += td;
            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "_CustomTemplate1"] = parentTr;
            $.templates(templates);
        },

        _getDayTemplate: function (dayFormat, dayLength, data) {

            var date = new Date(data),
                proxy = this, model = proxy.model;
            date.setDate(date.getDate() + dayLength);

            switch (dayFormat) {
                case "D":
                case "d":
                    return (('0' + date.getDate()).slice(-2));
                case "DD":
                case "dd":
                    return date.getDate();
                case "DDD":
                case "ddd":
                case "DDDD":
                case "dddd":
                    return Globalize.format(date, dayFormat, model.localization)[0].toUpperCase();
                default: return Globalize.format(date, dayFormat, model.localization)[0].toUpperCase();
            }
        },


        //Method to render schedule header
        _createSchedule: function (startdate, enddate) {

            var proxy = this,
                scheduleLength = proxy.model.scheduleWeeks.length,
                $colsgroup, $column, $thead,
                $tr, $td, $div, $table;

            startdate = startdate == null ? proxy._projectStartDate : startdate;
            enddate = enddate == null ? proxy._projectEndDate : enddate;
            proxy._createScheduleWeekTemplate();

            $table = ej.buildTag("table.e-schedule-headerrow-week", "",
                { 'display': 'block' },
                { 'cell-spacing': '0' }
            );

            $colsgroup = ej.buildTag("colgroup", "", {}, {});
            $column = $(document.createElement("col"));
            $column.css("width", (scheduleLength * this._scheduleWeekWidth) + 'px');
            $colsgroup.append($column);

            $thead = ej.buildTag("thead.e-ejganttschedule", "",
                { 'display': 'block', 'border-collapse': 'collapse' }, {});
            $tr = ej.buildTag("tr", $.render[this._id + "ScheduleWeekTemplate"](proxy.model.scheduleWeeks), {}, {});
            $td = ej.buildTag("th.schedule", "", {}, {});
            $div = ej.buildTag("div.schedule", "", { 'width': 20 + 'px' }, {});
            $td.append($div);
            $tr.append($td);

            $thead.append($tr);
            $table.append($colsgroup);
            $table.append($thead);
            proxy._$scheduleDiv.append($table);

            $tr = null;
            proxy._createScheduleDayTemplate();
            $table = ej.buildTag("table.e-schedule-headerrow-day", "",
                {
                    'display': 'block',
                    'border-collapse': 'collapse'
                }, { 'cell-spacing': '0' });
            $thead = ej.buildTag("thead.e-ejganttschedule", "",
                { 'display': 'block', 'border-collapse': 'collapse' }, {});
            $tr = ej.buildTag("tr", $.render[this._id + "_CustomTemplate1"](proxy.model.scheduleWeeks), {}, {});
            $td = ej.buildTag("th.schedule", "", {}, {});
            $div = ej.buildTag("div.schedule", "", { 'width': 20 + 'px' }, {});
            $td.append($div);
            $tr.append($td);

            $thead.append($tr);
            $table.append($colsgroup);
            $table.append($thead);
            proxy._$scheduleDiv.append($table);

            proxy._$headerContainer.append(proxy._$scheduleDiv);
            proxy._$ganttChartContainer.append(proxy._$headerContainer);
            $("#" + proxy._id).append(proxy._$ganttChartContainer);
        },


        //Template to render taskbar and milestone items
        _createTaskbarTemplate: function () {

            var proxy = this,
                tdWidth = proxy.model.scheduleWeeks.length * proxy._scheduleWeekWidth,
                model = proxy.model,
                tr, baselineHeight = 4, taskbarHeight,
                shrinkedTaskbarHeight = 0,
                baselineColor = model.baselineColor,
                helpers = {
                    _getType: proxy._getTypedata,
                    _expander: proxy._addRecordExpandCollapse,
                    _milestoneMapping: proxy._isMilestone,
                    _resources: $.proxy(proxy._getResourceInfo, proxy),
                    _rowClassName: proxy._getrowClassName,
                    _expandStatus: $.proxy(proxy._getExpandStatusRecord, proxy),
                    _borderRadius: proxy._getBorderRadius,
                };
            if (model.renderBaseline) {
                //taskbarHeight = model.rowHeight - 10;
                taskbarHeight = model.rowHeight - baselineHeight - 5;
                shrinkedTaskbarHeight = 0;
            } else {
                taskbarHeight = model.rowHeight - 10;
            }

            $.views.helpers(helpers);

            if (!model.progressMapping) {


                this.model.showProgressStatus = false;
            }

            if (!model.resourceInfoMapping) {
                model.showResourceNames = false;
            }

            var parentTr = "<tr class='{{:~_rowClassName()}}  {{if isSelected}}e-ganttrowcell {{:~_expander()}} e-gantt-mouseclick{{else}}e-ganttrowcell {{:~_expander()}}{{/if}}' id='chartrowcell"
                + proxy._id + "'style='display:{{:~_expandStatus(#data)}}''>";
            var radius = "{{:~_borderRadius()}}";

            tr = "{{if ~_getType()}}" +

                //Parent taskbar Item

                "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width}}px;width:{{:left+width}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;" +
                "font-weight:bold'>{{:taskName}}</span></div></td>{{/if}}" +

                "<td  class='e-chartcell' vertical-align='middle' style='width:" + tdWidth + "px;'>" +

               "<div class='e-parentContainer' unselectable='on' style='height:29px;z-index:3;'>" +
                "<div class='e-gantt-parenttaskbar-innerdiv {{:~_expander()}}' style='left:{{:left}}px;margin-top:5px;width:{{:width}}px;height:"
                + (taskbarHeight) + "px;background-color:" + this.model.parentTaskbarBackground + " ;'></div>" +
                "<div  class='e-gantt-parenttaskbar-progress {{:~_expander()}}  progressbar' id='progressbar' " +
                "style='margin-top:5px;border-style:{{if progressWidth}}solid{{else}}none{{/if}}; left:{{:left}}px;width:{{:progressWidth}}px;height:" + (taskbarHeight)
                + "px;position:absolute;background-color:" + this.model.parentProgressbarBackground + ";z-index:3;" +
                "border-top-right-radius:" + radius + "px;border-bottom-right-radius:" + radius + "px;'></div>" +
                "</div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div  class='e-baselinebar' style='margin-top:" + (taskbarHeight) + "px;left:{{:baselineLeft}}px;" +
                "width:{{:baselineWidth}}px;height:" + baselineHeight + "px;background-color: " + baselineColor + ";" +
                "position:absolute;z-index:3;box-sizing:border-box;-moz-box-sizing:border-box;'></div>{{/if}}" +

                "</td>" +

               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='margin-top:5px;left:{{:left+width+30}}px;" +
                "margin:0;margin-left:5px;margin-top:-9px;position:absolute;background-color:transparent;font-weight:bold'>" +
                "<span class='e-resourcename'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>"
                + "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}" +


               //Milestone Item

                "{{else ~_milestoneMapping()}}" +

                "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width+150}}px;width:{{:left+width+150}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:taskName}}</span></div></td>{{/if}}" +

               "<td class='e-chartcell' style='width:" + tdWidth + "px;'>" +
                "<div class='e-gantt-milestone-container' style=height:29px;>" +
                "<div class='e-gantt-milestone' style='position:absolute;left:{{:left}}px;'>" +
                "<div class='e-gantt-milestone milestone-top'  style='top:" + (3) + "px;'></div>" +
                "<div class='e-gantt-milestone milestone-bottom'  style='top:" + (15) + "px;'></div>" +
                "</div></div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div class='e-baseline-gantt-milestone-container' >" +
                "<div class='e-baseline-gantt-milestone' style='position:absolute;left:{{:baselineLeft}}px;z-index:2;'>" +
                "<div class='e-baseline-gantt-milestone baseline-milestone-top'  " +
                "style=' border-bottom-color: " + baselineColor + ";z-index:2;top:" + (6) + "px;'></div>" +
                "<div class='e-baseline-gantt-milestone baseline-milestone-bottom'  " +
                "style=' border-top-color: " + baselineColor + ";z-index:2;top:" + (18) + "px;'></div>" +
                "</div></div>{{/if}}" +

                "</td>" +

               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='margin-top:5px;left:{{:left+56}}px;margin:0;margin-left:5px;margin-top:-9px;" +
                "position:absolute;background-color:transparent;'><span class='e-resourcename'  style='margin-top:"
                + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" + "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}" +

                //Child Taskbar Item

                "{{else}}" +

                //left label
               "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width}}px;width:{{:left+width}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:taskName}}</span></div></td>{{/if}}" +

                //taskbar
               "<td class='e-chartcell' style='width:" + tdWidth + "px;'>" +

               "<div  class='e-childContainer'  unselectable='on' style='height:29px;'>" +

              "<div class='taskbarresizer-left e-icon' style='left:{{:left+2}}px;width:10px;margin-top:5px;height:" + (taskbarHeight) + "px;'></div>" +

               "<div  class='e-gantt-childtaskbar' style='left:{{:left}}px;width:{{:width}}px;height:"
                + (taskbarHeight) + "px;background-color:" + this.model.taskbarBackground + ";margin-top:5px;'></div>" +

               "<div  class='e-gantt-childtaskbar-progress  progressbar' id='progressbar' style='border-style:{{if progressWidth}}solid{{else}}none{{/if}}; left:{{:left}}px;" +
                "width:{{:progressWidth}}px;text-align:right;line-height:" + (taskbarHeight) + "px;margin-top:5px;" +
                //"margin-top:" + (((taskbarHeight) - (this.model.progressbarHeight * ((taskbarHeight) / 100))) / 2) + "px;" +
                "height:" + (this.model.progressbarHeight * ((taskbarHeight) / 100)) + "px;" +
                "position:absolute;background-color:" + this.model.progressbarBackground + ";" +
                "border-top-right-radius:" + radius + "px;border-bottom-right-radius:" + radius + "px;'>" +
                "{{if " + this.model.showProgressStatus + "}}<span class='e-tasklabel' style=''>{{:status}}%</span>{{/if}}</div>" +

               "<div class='progressbarresizer-right' style='left:{{:left+progressWidth-10}}px;width:20px;margin-top:5px;" +
                "height:" + (taskbarHeight) + "px;position:absolute;z-index:4;'>" +
               "<div class='progressbarresizer-right' style='top:" + (22 - (shrinkedTaskbarHeight / 2)) + "px;margin-top:5px;position:absolute;'><div class='e-progresshandler-element'></div></div>" +//-" + (shrinkedTaskbarHeight/2) + "
               "<div class='progressbarresizer-right' style='top:" + (17 - (shrinkedTaskbarHeight / 2)) + "px;margin-top:5px;position:absolute;'><div class='e-progresshandlerafter-element'></div></div></div>" +

               "<div class='taskbarresizer-right e-icon' style='left:{{:left+width-12}}px;width:10px;" +
                "height:" + (taskbarHeight) + "px;position:absolute;z-index:2;margin-top:5px;'></div>" +
                "</div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div  class='e-baselinebar' style='margin-top:" + (taskbarHeight) + "px;left:{{:baselineLeft}}px;" +
                "width:{{:baselineWidth}}px;height:" + baselineHeight + "px;background-color: " + baselineColor + ";" +
                "position:absolute;z-index:3;box-sizing:border-box;-moz-box-sizing:border-box;'></div>{{/if}}" +

                "</td>" +

                //right label
               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='left:{{:left+width+30}}px;margin:0;margin-left:5px;margin-top:-9px;" +
                "position:absolute;text-align:left;background-color:transparent;'><span class='e-resourcename'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}{{/if}}";

            parentTr += tr;
            parentTr += "</tr>";

            var templates = {};
            templates[this._id + "_CustomTemplate2"] = parentTr;
            $.templates(templates);

        },

        _getResourceInfo: function (data) {
            var resource = null;
            if (data.resourceInfo != null) {
                var resourcelength = data.resourceInfo.length;
                if (resourcelength > 0) {
                    for (var i = 0; i < resourcelength; i++) {
                        if (resource == null)
                            resource = data.resourceInfo[i][this.model.resourceNameMapping];
                        else
                            resource += " , " + data.resourceInfo[i][this.model.resourceNameMapping];
                    }
                    return resource;
                } else
                    return data.resourceInfo[0][this.model.resourceNameMapping];
            }
            return String.empty;
        },

        _getrowClassName: function () {
            var rowClass = "gridrowtaskId",
                proxy = this;
            if (proxy.data.parentItem)
                rowClass += proxy.data.parentItem.taskId.toString();
            rowClass += "level";
            rowClass += proxy.data.level.toString();
            return rowClass;
        },

        _getExpandStatusRecord: function (data) {
            var proxy = this;
            if (proxy._getExpandStatus(data)) {
                return 'table-row';
            }
            return 'none';
        },


        _isMilestone: function () {
            if (this.data.isMilestone)
                return true;
            else
                return false;
        },

        _addRecordExpandCollapse: function () {
            if (this.data.expanded)
                return 'e-chartexpand';
            else if (!this.data.expanded && this.data.hasChildGanttRecords) {
                return 'e-chartcollapse';
            }
            return false;
        },

        _getTypedata: function () {
            return this.data.hasChildGanttRecords;
        },


        _getBorderRadius: function () {

            var proxy = this,
                diff = proxy.data.width - proxy.data.progressWidth;

            if (diff <= 4) {
                return 4 - diff;
            }
            else
                return 0;

        },


        _createConnectorLineTemplate: function () {
            var proxy = this,
                lineStroke = this.model.connectorlineWidth,
                shrinkedTaskbarHeight = 0,
                deltaTop, taskbarMidpoint,
                lineColor = this.model.connectorLineBackground,
                helpers = {
                    _getWidthVal: proxy._getWidth,
                    _getTop: proxy._getPredecessorTop,
                    _getPosition: proxy._getParentPosition,
                    _setContainerWidthSSType2: proxy._getContainerWidthSSType2,
                    _setInnerChildWidthSSType2: proxy._getInnerChildWidthSSType2,
                    _setInnerElementLeftSSType2: proxy._getInnerElementLeftSSType2,
                    _setInnerElementWidthSSType2: proxy._getInnerElementWidthSSType2,
                };

            if (proxy.model.renderBaseline) {
                deltaTop = shrinkedTaskbarHeight / 2;
                taskbarMidpoint = ((this.model.rowHeight) / 2) - deltaTop;
            } else {
                taskbarMidpoint = ((this.model.rowHeight) / 2);
            }

            $.views.helpers(helpers);
            var connectorContainer = "<div  id='ConnectorLine{{:ConnectorLineId}}' style='background-color=black'>";

            var div = "{{if ~_getPosition()=='FSType1'}}"//FS
            + "<div style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:(ChildLeft - (ParentLeft + ParentWidth + 20))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{if milestoneParent}}-7px{{else}}0px{{/if}};width:{{if milestoneParent}}{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-2}}px{{else}}{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px{{/if}};" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px;" +
                "width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='FSType2'}}"//FS
            + "<div  style='left:{{:ParentLeft}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:~_getWidthVal()}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{if milestoneParent}}{{:ParentWidth-7}}px{{else}}{{:ParentWidth}}px{{/if}};width:{{if milestoneParent}}17px{{else}}10px{{/if}};height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:ParentWidth+10-" + lineStroke + "}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-14-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div><div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;" +
                "width:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30 )}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;width:" + lineStroke + "px;" +
                "height:{{:14-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;top:{{:-6-" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FSType3'}}"
            + "<div style='left:{{:ChildLeft-20}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentWidth+ChildWidth}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;" +
                "top:{{:-6-" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative;top:-13px;'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-15-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative;top:-13px;'></div>"
            + "<div class='e-line' style='width:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30)}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative;top:-13px;'></div>"
            + "<div class='e-line' style='left:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30)}}px;width:" + lineStroke + "px;" +
                "height:{{:12-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative;" +
                "top:-13px;'></div>"
            + "<div class='e-line' style='left:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 20)}}px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative;top:-13px;'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FSType4'}}"
            + "<div  style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:(ChildLeft - (ParentLeft + ParentWidth + 20))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth) - 20)}}px;top:-13px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;left:{{:(ChildLeft - (ParentLeft + ParentWidth) - 20)}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:(ChildLeft-(ParentLeft + ParentWidth + 20))+" + lineStroke + "}}px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType4'}}"
            + "<div  style='left:{{:ParentLeft-10}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ChildLeft-ParentLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:{{:ChildLeft-ParentLeft}}px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:ChildLeft-ParentLeft}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType3'}}"
            + "<div  style='left:{{:ChildLeft-20}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentLeft-ChildLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:ParentLeft-ChildLeft+20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType2'}}"
            + "<div  style='left:{{:~_setInnerElementLeftSSType2()}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:~_setContainerWidthSSType2()}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:{{:~_setInnerChildWidthSSType2()}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:{{:~_setInnerElementWidthSSType2()}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:~_setInnerElementWidthSSType2()}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;" +
                "height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType1'}}"
            + "<div  style='left:{{:ChildLeft-20}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentLeft-ChildLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:{{:ParentLeft-ChildLeft+20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType1'}}"
            + "<div style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))}}px;width:20px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:10px;width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+10}}px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType2'}}"
            + "<div  style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+10}}px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))}}px;" +
                "border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType3'}}"
            + "<div  style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='top:-12.5px;border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;top:-12.5px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;top:-12.5px;" +
                "width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))}}px;top:-12.5px;width:20px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType4'}}"
            + "<div  style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div  class='connectorline-leftarrow' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))}}px;" +
                "border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+10}}px;top:-13px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;top:-13px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;top:-13px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='SFType4'}}"//SF
            + "<div style='left:{{:ParentLeft-10}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;" +
                "height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+10}}px;" +
                "top:-13px;border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;top:-13px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;top:-13px;width:" + lineStroke + "px;" +
                "height:{{:Height-15-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:12-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SFType3'}}"//SF
            + "<div style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft)-(ChildLeft + ChildWidth))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='top:-13px;border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='left:10px;top:-13px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:20px;top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:20px;top:-13px;width:{{:(ChildLeft-(ParentLeft + ParentWidth + 20))+" + lineStroke + "}}px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='SFType1'}}"//SF
            + "<div style='left:{{:ParentLeft-10}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ChildLeft+ChildWidth+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-15-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;width:" + lineStroke + "px;" +
                "height:{{:12-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+10}}px;" +
                "border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;" +
                "width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SFType2'}}"//SF
            + "<div  style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft) - (ChildLeft + ChildWidth))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{:((ParentLeft) - (ChildLeft + ChildWidth))-10}}px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft) - (ChildLeft + ChildWidth))-10}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:10px;width:{{:((ParentLeft) - (ChildLeft + ChildWidth))-20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{/if}}";

            connectorContainer += div;
            connectorContainer += "</div>";

            var templates = {};
            templates[this._id + "ConnectorLineTemplate"] = connectorContainer;
            $.templates(templates);
        },

        _getInnerChildWidthSSType2: function () {
            if ((this.data.ParentLeft + this.data.ParentWidth) < this.data.ChildLeft) {
                return 10;
            }
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 20;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },

        _getInnerElementLeftSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return (this.data.ParentLeft - 20);
            }
            return (this.data.ParentLeft - 10);
        },

        _getInnerElementWidthSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 10;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },


        _getContainerWidthSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 20;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },

        _getPredecessorTop: function () {
            if (this.data.ParentTop > this.data.ChildTop)
                return true;
            else
                return false;
        },

        _getParentPosition: function () {
            if (this.data.ParentTop < this.data.ChildTop) {

                if (this.data.Type == "FF") {
                    if (this.data.ChildLeft > this.data.ParentLeft) {
                        return "FFType2";
                    }
                    else {
                        return "FFType1";
                    }
                }

                else if ((this.data.ParentLeft < this.data.ChildLeft) && (this.data.ChildLeft > (this.data.ParentLeft + this.data.ParentWidth + 25))) {
                    if (this.data.Type == "FS") return "FSType1";
                    if (this.data.Type == "SF") return "SFType1";
                    else if (this.data.Type == "SS") return "SSType2";
                    else if (this.data.Type == "FF") return "FFType2";
                }
                else if ((this.data.ParentLeft < this.data.ChildLeft && (this.data.ChildLeft < (this.data.ParentLeft + this.data.ParentWidth)))
                    || (this.data.ParentLeft == this.data.ChildLeft || this.data.ParentLeft > this.data.ChildLeft)) {

                    if (this.data.ParentLeft > (this.data.ChildLeft + this.data.ChildWidth + 25)) {
                        if (this.data.Type == "SF") return "SFType2";
                    }

                    if (this.data.ParentLeft > this.data.ChildLeft) {
                        if (this.data.Type == "SS") return "SSType1";
                        if (this.data.Type == "SF") return "SFType1";
                        if (this.data.Type == "FF") return "FFType1";
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType2";
                    }
                    if (this.data.Type == "FS") return "FSType2";
                    else if (this.data.Type == "SS") return "SSType2";
                    else if (this.data.Type == "FF") return "FFType1";
                    else if (this.data.Type == "SF") return "SFType1";
                }
                else if ((this.data.ParentLeft + 25) < this.data.ChildLeft) {

                    if (this.data.Type == "FS") return "FSType2";
                    else if (this.data.Type == "FF") return "FFType2";

                }

            }
            else if (this.data.ParentTop > this.data.ChildTop) {
                if ((this.data.ParentLeft < this.data.ChildLeft) && (this.data.ChildLeft > (this.data.ParentLeft + this.data.ParentWidth))) {
                    if (this.data.Type == "FS") return "FSType4";
                    if (this.data.ParentLeft < this.data.ChildLeft || ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth))) {
                        if (this.data.Type == "SS") return "SSType4";
                        if (this.data.Type == "FF") return "FFType4";
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType4";
                    }
                }
                else if ((this.data.ParentLeft < this.data.ChildLeft && (this.data.ChildLeft < (this.data.ParentLeft + this.data.ParentWidth)))
                    || (this.data.ParentLeft == this.data.ChildLeft || this.data.ParentLeft > this.data.ChildLeft)) {
                    if ((this.data.ChildLeft + this.data.ChildWidth) <= (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType3";
                        if (this.data.Type == "SF") return "SFType3";
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType4";
                        if (this.data.Type == "SF") return "SFType4";
                    }
                    if (this.data.Type == "FS") return "FSType3";
                    else if (this.data.Type == "SS") return "SSType3";
                }
                else if (this.data.ParentLeft < this.data.ChildLeft) {

                    if (this.data.Type == "FS") return "FSType3";

                }
            }
            return false;
        },

        _getWidth: function () {
            return ((this.data.ChildLeft + this.data.ChildWidth) < (this.data.ParentLeft + this.data.ParentWidth)) ?
                this.data.ParentWidth : (this.data.ChildWidth + this.data.ParentWidth);
        },


        //renders Weekends in chart
        _renderWeekends: function () {
            var proxy = this,
                left = 0,
                width = (proxy.model.perDayWidth) + 1,
                height = ((proxy.model.ganttRecords.length) * this.model.rowHeight),
                count = proxy.model.scheduleWeeks.length,
                $sunday, $saturday,
                $weekEndContainer = ej.buildTag("div.e-secondary_canvas", "", { 'height': height + 'px' }, {});

            for (var i = 0; i < count; i++) {
                $sunday = ej.buildTag("div.e-weekends", "", {
                    'left': left + 'px',
                    'width': width + 'px',
                    'height': height + 'px',
                    'background-color': this.model.weekendBackground
                }, {});
                $saturday = ej.buildTag("div.e-weekends", "", {
                    'left': (left + (proxy.model.perDayWidth * 6)) + 'px',
                    'width': width + 'px',
                    'height': height + 'px',
                    'background-color': this.model.weekendBackground
                }, {});
                $weekEndContainer.append($sunday);
                $weekEndContainer.append($saturday);
                left = left + this._scheduleWeekWidth;
            }

            proxy._$secondaryCanvas.append($weekEndContainer);
            proxy._$bodyContent.append(proxy._$secondaryCanvas);
        },



        //Renders StripLines for given striplines collection object
        _renderStripLines: function (stripLines) {

            var proxy = this,
                height = ((proxy.model.ganttRecords.length) * this.model.rowHeight),
                stripLineCount = stripLines.length, day, left,
                projectStartDate = proxy.model.scheduleWeeks[0],
                dayWidth = this.model.perDayWidth,
                stripLineContainer = ej.buildTag("div.e-stripLines", "", { 'height': height + 'px' }, {});

            for (var i = 0; i < stripLineCount; i++) {

                day = this._getDateType(stripLines[i].day);
                left = ((day - this._getDateType(projectStartDate)) / 86400000) * dayWidth;
                stripLines[i].lineWidth = stripLines[i].lineWidth > 30 ? 30 : stripLines[i].lineWidth;

                var stripline = ej.buildTag("div.e-stripline#stripline" + [i] + "", "", {
                    'left': left + 'px',
                    'width': '1px',
                    'height': height + 'px',
                    'border-left-width': stripLines[i].lineWidth + 'px',
                    'border-left-color': stripLines[i].lineColor,
                    'border-left-style': stripLines[i].lineStyle
                }, {});

                var innerspan = ej.buildTag("span.e-striplinespan", stripLines[i].label, {
                    'background-color': stripLines[i].lineColor,
                }, {});

                if (stripLines[i].day && stripLines[i].day.length > 0) {
                    if (stripLines[i].label && stripLines[i].label.length > 0) {
                        stripline.append(innerspan);
                    }
                    stripLineContainer.append(stripline);
                }
            }

            proxy._$secondaryCanvas.append(stripLineContainer);
            proxy._$bodyContent.append(proxy._$secondaryCanvas);
        },


        //returns progress width for the taskbar
        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },


        //returns progress percentage of the taskbar
        _getProgressPercent: function (parentwidth, progresswidth) {
            return Math.ceil(((progresswidth / parentwidth) * 100).toFixed(2));
        },


        //renders holidays for holiday collection object
        _renderHoliday: function (holiday) {

            var proxy = this,
                date = this._getDateType(holiday.day),
                weekCount = proxy.model.scheduleWeeks.length,
                prevDate = 0,
                left = 0, textLength, text, $holidayContainer, $holidaySpan,
                ganttRecords = proxy.model.ganttRecords.length, height;

            for (var i = 0; i < weekCount; i++) {
                var givendate = (date - this._getDateType(proxy.model.scheduleWeeks[i])) / 86400000;
                if (givendate < 0) {
                    left = prevDate * this.model.perDayWidth;
                    //prevDate = 0;
                    break;
                }
                if (prevDate == 0)
                    prevDate = givendate;
            }

            height = (ganttRecords * this.model.rowHeight);
            textLength = holiday.label != null ? holiday.label.length : 0;
            if (holiday.background == null)
                holiday.background = "white";
            text = ej.buildTag("div.e-holiday-label", holiday.label, {
                'top': (150 + textLength) + 'px'
            }, {});
            $holidayContainer = ej.buildTag("div.e-secondary-canvas-holiday");
            $holidaySpan = ej.buildTag("div.e-holidays", "", {
                'left': left + 'px',
                'width': proxy.model.perDayWidth + 'px',
                'height': height + 'px',
                'background-color': holiday.background
            }, {});

            if (holiday.label != null)
                $holidaySpan.append(text);
            $holidayContainer.append($holidaySpan);

            proxy._$secondaryCanvas.append($holidayContainer);
            proxy._$bodyContent.append(proxy._$secondaryCanvas);
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {

            var proxy = this;
            proxy.element.empty().removeClass("e-ganttchart-core e-ganttchart" + proxy.model.cssClass);

        },
    });

    ej.GanttChart.contextMenuItem = {
        taskDetails: "Task Details....",
        add: "Add new task",
        indent: "Indent",
        outdent: "Outdent",
        Delete: "Delete",
    };

})(jQuery, Syncfusion);;
ej.EjVmlRender = function (element) {
    var doc = document;
    this.vmlNamespace = "urn:schemas-microsoft-com:vml";
    this.vmStyle = "#default#VML";
    doc.namespaces.add('v', 'urn:schemas-microsoft-com:vml');
    doc.createStyleSheet().cssText =
  'v\\:fill, v\\:path, v\\:polyline, v\\:line, v\\:rect,v\\:shape,v\\:oval, v\\:stroke' +
  '{ behavior:url(#default#VML); display: inline-block; } ';
    this._rootId = jQuery(element).attr("id");
    this.svgObj = doc.createElement("div");
    this.svgObj.style.position = 'relative';
    this.svgObj.setAttribute('id', this._rootId + '_vml');
    this.isdocmode8 = doc.documentMode === 8;
    this.changeOptions = {
        'id': 'id',
        'fill': 'fillcolor', 'stroke': 'strokecolor', 'stroke-width': 'strokeweight', 'd': 'path', 'font-size': 'font-size', 'font-family': 'font-family', 'font-style': 'font-style',
        'font-weight': 'font-weight', 'points': 'points'

    };
};
(function ($) {
    ej.EjVmlRender.prototype = {

        drawPath: function (options, element) {
            var darray = options.d.split(" ");
            if ($.inArray('A', darray) != -1) {
                var w = parseFloat(darray[4]);
                var h = parseFloat(darray[5]);
                options.d = this.drawArc(w, h, options);
            }
            if ($("#" + options.id).length > 0) {
                options.d = this.changePathValue(options.d);
                this.applyVMLStyle($("#" + options.id), options);
            }
            else {
                options.d = this.changePathValue(options.d);
                var shape = document.createElement("v:shape");
                var $shapeEle = $(shape);
                this.applyVMLStyle($shapeEle, options);
                $shapeEle.appendTo(element);
            }
        },
        drawPolyline: function (options, element) {
            var darray = options.points.split(" ");

            if ($("#" + options.id).length > 0) {
                options.points = this.changePathValue(options.points);
                this.applyVMLStyle($("#" + options.id), options);
            }
            else {
                options.points = this.changePathValue(options.points);
                var shape = document.createElement("v:polyline");
                var $shapeEle = $(shape);
                this.applyVMLStyle($shapeEle, options);
                $shapeEle.appendTo(element);
            }
        },
        setFillAttribute: function (element, options) {
            var fill = document.createElement("v:fill");
            var opacity = options["fill-opacity"] ? options["fill-opacity"] : options["opacity"];
            opacity = (opacity) ? opacity : (options["opacity"]) ? options["opacity"] : 1;
            var fillcolor = options["fill"];
            if (fillcolor != "none" && !ej.util.isNullOrUndefined(fillcolor)) {

                if (!(fillcolor).match('gradient'))
                    $(element).attr("fillcolor", fillcolor);
                else {
                    var colorName = ((fillcolor).replace('gradient', ""));
                    fill.setAttribute('type', "gradient");
                    fill.setAttribute("colors", colorName);
                }
            }
            else
                opacity = 0.1;


            fill.setAttribute('opacity', opacity);
            $(fill).appendTo(element);
        },
        setStrokeAttribute: function (element, options) {
            var stroke = options['stroke'];
            if (stroke == "transparent")
                options['opacity'] = 0.1;


            var opacity = options["opacity"];
            if (opacity) {
                var fill = document.createElement("v:stroke");



                fill.setAttribute('opacity', opacity);
                $(fill).appendTo(element);
            }
        },
        changePathValue: function (options) {
            if (!options["d"]) {
                var pathArray = options.split(" ");
                for (var i = 0; i < pathArray.length; i++) {
                    var matches = pathArray[i].match(/\d+/g);
                    if (matches != null) {
                        pathArray[i] = Math.round(parseFloat(pathArray[i]));
                    }
                }
                options = pathArray.join(' ');
                return options;
            }
        },


        drawArc: function (w, h, options) {
            var x = 0, y = 0;
            var startAngle = options.start - 1.571,
            endAngle = options.end - 1.571,
            radius = options.r || w || h,
            cosStart = Math.cos(startAngle),
            sinStart = Math.sin(startAngle),
            cosEnd = Math.cos(endAngle),
            sinEnd = Math.sin(endAngle),
            innerRadius = options.innerR,
            circleCorrection = 0.08 / radius,
            innerCorrection = (innerRadius && 0.1 / innerRadius) || 0,
            path;

            if (endAngle - startAngle === 0) {
                return ['x'];

            } else if (2 * Math.PI - endAngle + startAngle < circleCorrection) {
                cosEnd = -circleCorrection;
            } else if (endAngle - startAngle < innerCorrection) {
                cosEnd = Math.cos(startAngle + innerCorrection);
            }

            path = [
        'wa', // To draw clockwise arc
        x - radius,
        y - radius,
        x + radius,
        y + radius,
        x + radius * cosStart,
        y + radius * sinStart,
        x + radius * cosEnd,
        y + radius * sinEnd
            ];

            if (options.open && !innerRadius) {
                path.push(
            'e',
            M,
            x,
            y
        );
            }

            path.push(
                // To draw anti clockwise arc
        'at',
        x - innerRadius,
        y - innerRadius,
        x + innerRadius,
        y + innerRadius,
        x + innerRadius * cosEnd,
        y + innerRadius * sinEnd,
        x + innerRadius * cosStart,
        y + innerRadius * sinStart,
        'x',
        'e'
    );
            path = path.join(" ");
            return path;

        },
        changeVMLStyle: function ($element, options) {
            var chart = this;
            var strokeColor = options.stroke;
            $element.css("width", options.width).css("visibility", "visible").css("height", options.height).css("position", "absolute").css('left', options.x).css('top', options.y);
            $.each(options, function (index, val) {
                if (chart.changeOptions[index] != null || chart.changeOptions[index] != undefined) {
                    if (index == 'd' || index == 'points') {
                        val = chart.changePathValue(options[index]).toLowerCase();
                    }
                    if (val == 'transparent') {
                        options[index] = val = 'white';
                        if (index == "fill" && (!options.hasOwnProperty('opacity') || !options['opacity']))
                            options['opacity'] = 0.1;
                    }
                    if ($element[0].tagName === "SPAN") {
                        if (index == "fill") $element.css('color', val);
                        else {
                            if (index == "id") $element.attr(chart.changeOptions[index], val);
                            else
                                $element.css(chart.changeOptions[index], val);
                        }
                    } else {

                        if (chart.isdocmode8) { // IE8 setAttribute bug
                            $element.get(0)[chart.changeOptions[index]] = val;
                        } else {
                            $element.get(0).setAttribute(chart.changeOptions[index], val);
                        }
                    }
                }
            });
            if (options.hasOwnProperty('fill-opacity') || options.hasOwnProperty('opacity') || options.hasOwnProperty('fill'))
                if (!($element[0].tagName === "SPAN")) {
                    chart.setFillAttribute($element, options);
                    if (options.hasOwnProperty('stroke')) {
                        options.stroke = strokeColor;
                        chart.setStrokeAttribute($element, options);
                    }
                }
        },
        applyVMLStyle: function ($element, options) {

            if ($element[0].tagName !== "SPAN") {
                if (ej.util.isNullOrUndefined(options.width)) options.width = "1000px";
                if (ej.util.isNullOrUndefined(options.height)) options.height = "1000px";
            }
            if (ej.util.isNullOrUndefined(options.x)) options.x = "0px";
            if (ej.util.isNullOrUndefined(options.y)) options.y = "0px";

            this.changeVMLStyle($element, options);
        },
        drawLine: function (options, element) {
            if ($("#" + options.id).length > 0) {
                var from = (options.x1) + ',' + (options.y1);
                var to = (options.x2) + ',' + (options.y2);
                $(options).attr("from", from).attr("to", to);
                this.applyVMLStyle($("#" + options.id), options);
            }
            else {
                var line = document.createElement("v:line");
                var $lineEle = $(line);
                var from = (options.x1) + ',' + (options.y1);
                var to = (options.x2) + ',' + (options.y2);
                $lineEle.attr("from", from).attr("to", to);
                this.applyVMLStyle($lineEle, options);
                $lineEle.appendTo(element);
            }
        },
        //drawPolygon: function (options, element) {

        //},
        drawCircle: function (options, element) {
            if ($("#" + options.id).length > 0) {
                options.x = options.cx - options.r;
                options.y = options.cy - options.r;
                options.width = options.height = (2 * options.r);
                this.applyVMLStyle($("#" + options.id), options);
            }
            else {
                var circle = document.createElement("v:oval");
                var $circleEle = $(circle);
                options.x = options.cx - options.r;
                options.y = options.cy - options.r;
                options.width = options.height = (2 * options.r);
                this.applyVMLStyle($circleEle, options);
                if ($("#" + options.id).length == 0)
                    $circleEle.appendTo(element);
            }
        },

        drawEllipse: function (options, element) {
            if ($("#" + options.id).length > 0) {
                options.x = options.cx - options.rx;
                options.y = options.cy - options.ry;
                options.width = (2 * options.rx);
                options.height = (2 * options.ry);
                this.applyVMLStyle($("#" + options.id), options);
            }
            else {
                var ellipse = document.createElement("v:oval");
                var $ellipseEle = $(ellipse);
                options.x = options.cx - options.rx;
                options.y = options.cy - options.ry;
                options.width = (2 * options.rx);
                options.height = (2 * options.ry);
                this.applyVMLStyle($ellipseEle, options);
                $ellipseEle.appendTo(element);
            }
        },
        drawRect: function (options, element) {
            if ($("#" + options.id).length > 0) {
                this.applyVMLStyle($("#" + options.id), options);
            } else {
                var rect = document.createElement("v:rect");
                var $rectEle = $(rect);
                this.applyVMLStyle($rectEle, options);
                $rectEle.appendTo(element);
            }
        },
        _getAttrVal: function (ele, val, option) {
            val = (val == "x") ? "left" : val;
            val = (val == "y") ? "top" : val;
            var value1 = $(ele).attr(val);
            var value = value1 ? value1 : ($(ele).css(val));

            if (value != null)
                return value;
            else
                return option;
        },
        _setAttr: function (element, attribute) {
            if (attribute.transform) {
                var attrTrans = attribute.transform;
                var trans = attrTrans.slice(attrTrans.indexOf('(') + 1, attrTrans.indexOf(')')).split(',');
                attribute.x = trans[0];
                attribute.y = trans[1];
            }
            var $ele = $(element);
            if ($ele.length > 0)
                this.changeVMLStyle($ele, attribute);
        },
        createGradientElement: function (name, colors) {
            var colorName;

            if (Object.prototype.toString.call(colors) == '[object Array]') {
                colorName = "gradient" + " ";
                var appendString = "";
                for (var j = 0, i = colors.length - 1 ; i >= 0, j < colors.length ; j++, i--) {
                    colorName += appendString + colors[j]["colorStop"] + " " + colors[i]["color"];
                    appendString = ",";
                }
            }
            else {
                colorName = colors;
            }
            return colorName;
        },

        //drawGradient: function (options, gradientEle, element) {


        //},
        setSpanAttr: function (options, label, element) {
            var fontSize = (options["font-size"] > 0) ? (options["font-size"] + "px") : options["font-size"];
            var font = { size: fontSize, fontStyle: options["font-style"], fontFamily: options["font-weight"] };
            var bounds = ej.EjSvgRender.utils._measureText(label, null, font);

            $(element).css("white-space", "nowrap");

            if (options["text-anchor"] === "middle")
                options.x = Math.round(options.x - bounds.width / 2);
            else if (options["text-anchor"] === "end")
                options.x = Math.round(options.x - bounds.width);
            options.y = Math.round(options.y - (font.size.replace("px", "")));
            this.applyVMLStyle(element, options);


        },
        drawText: function (options, label, groupEle, font) {
            var textDoc = document;
            var $Ele = $("#" + options.id);
            if ($Ele.length > 0) {
                this._textAttrReplace(options, label, font, $Ele);
            } else {
                var span = textDoc.createElement("span");
                var $spanEle = $(span);
                if (jQuery.type(label) == "array") {
                    this.setSpanAttr(options, label, $spanEle);
                    for (var i = 0; i < label.length; i++) {
                        var textspan = textDoc.createElement("span");
                        $(textspan).html(label[i]);
                        $(textspan).appendTo(span);
                        var brtag = textDoc.createElement("br");
                        $(brtag).appendTo(span);

                    }
                } else {
                    this.setSpanAttr(options, label, $spanEle);
                    $spanEle.html(label);
                }
                $spanEle.appendTo(groupEle);
            }
        },
        //text element attributes values are replaced 
        _textAttrReplace: function (options, label, font, $Ele) {
            if (jQuery.type(label) == "array") {
                var elements = $("#" + options.id).children("span");
                this.setSpanAttr(options, label, $Ele);
                if (elements.length > 0 && elements.length == label.length) {
                    for (var i = 0; i < elements.length; i++) {
                        var element = elements[i];
                        $(element).text(label[i]);
                    }
                }
                else {
                    $("#" + options.id).remove();
                    this.drawText(options, label, this.gTransToolEle, font);

                }
            } else {
                this.setSpanAttr(options, label, $Ele);
                $Ele.text(label);
            }

        },

        drawImage: function (options, element) {
            var imgElement = document.createElement("img");
            var $imgElement = $(imgElement);
            $imgElement.css({ 'position': 'absolute' });
            $.each(options, function (index, val) {
                if (index == "href") index = "src";
                $imgElement.attr(index, val);
            });
            $imgElement.appendTo(element);

        },

        createDefs: function () {

        },

        //createClipPath: function (options) {

        //},
        //createForeignObject: function (options) {

        //},
        createGroup: function (options) {
            var group = document.createElement("div");
            if (!ej.util.isNullOrUndefined(options.transform)) {
                var trans = options.transform.slice(options.transform.indexOf('(') + 1, options.transform.indexOf(')')).split(',');
                $(group).css({ 'position': 'absolute', 'left': trans[0] + 'px', 'top': trans[1] + 'px' });
            }
            if (options.cursor)
                $(group).css({ 'cursor': options.cursor });

            $.each(options, function (index, val) {
                if (index !== 'transform' && index != "clip-path")
                    $(group).attr(index, val);
            });

            return group;
        },

        createText: function (options, label) {
            var text = document.createElement("span");
            $(text).attr(options);
            if (!ej.util.isNullOrUndefined(label))
                text.textContent = label;
            return text;


        },

        //createPath: function (options) {

        //},

        //createCircle: function (options) {

        //},

        //createLine: function (options) {

        //},


        //Draw clip path for each series to avoid series overlap in multiple axes zooming
        drawAxesBoundsClipPath: function (gSeriesGroupEle, options, invertedAxis) {
            var clipOptions;
            var element = $(gSeriesGroupEle);
            element.removeAttr("clip-path");
            var trans = this._getTransform(options.xAxis, options.yAxis, invertedAxis);
            clipOptions = {
                'id': gSeriesGroupEle.id + '_ClipRect',
                'x': 0,
                'y': 0,
                'width': (trans.width),
                'height': (trans.height),
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'transparent'
            };
            this.drawClipPath(clipOptions, gSeriesGroupEle);
            element.attr('clip-path', 'url(#' + clipOptions.id + ')');

        },

        getAttrVal: function (ele, val, option) {
            var value = $(ele).attr(val);
            if (value != null)
                return value;
            else
                return option;
        },

        round: function (value, div, up) {

            return div * (up ? Math.ceil(value / div) : Math.floor(value / div));
        },




        hexFromRGB: function (color) {
            var r = color.R;
            var g = color.G;
            var b = color.B;
            var hex = [r.toString(16), g.toString(16), b.toString(16)];
            $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
            return hex.join("").toUpperCase();
        },

        createDelegate: function (context, handler) {
            return function (e) {
                handler.apply(context, [e, this]);
            };
        },

        drawClipPath: function (options, element) {
            if (options.id.indexOf("ChartAreaClipRect") == -1) {
                var left = Math.round(options.x) + 'px';
                var top = Math.round(options.y) + 'px';
                var right = Math.round(options.width) + 'px';
                var bottom = Math.round(options.height) + 'px';
                var cssClip = 'rect(' + top + ' ' + right + ' ' + bottom + ' ' + left + ')';
                $(element).css("position", "absolute").css('clip', cssClip);
            }
        },

        append: function (childEle, parentEle) {
            $(childEle).appendTo(parentEle);
        }
    };
})(jQuery);;
ej.EjSvgRender = function (element) {

    this.svgLink = "http://www.w3.org/2000/svg";
    this.svgObj = document.createElementNS(this.svgLink, "svg");
    this._rootId = jQuery(element).attr("id");
    this.svgObj.setAttribute('id', this._rootId + '_svg');

};
(function ($) {
    ej.EjSvgRender.prototype = {
        drawPath: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var path = document.createElementNS(this.svgLink, "path");
                $(path).attr(options).appendTo(element);
            }

        },
        drawPolyline: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var polyline = document.createElementNS(this.svgLink, "polyline");
                $(polyline).attr(options).appendTo(element);
            }

        },

        drawLine: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var path = document.createElementNS(this.svgLink, "line");
                $(path).attr(options);
                $(path).appendTo(element);
            }
        },
        drawPolygon: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var polygon = document.createElementNS(this.svgLink, "polygon");
                $(polygon).attr(options);
                $(polygon).appendTo(element);
            }
        },
        drawCircle: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var circle = document.createElementNS(this.svgLink, "circle");
                $(circle).attr(options).appendTo(element);
            }
        },
        drawEllipse: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var ellipse = document.createElementNS(this.svgLink, "ellipse");
                $(ellipse).attr(options).appendTo(element);
            }
        },

        drawRect: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var rect = document.createElementNS(this.svgLink, "rect");
                $(rect).attr(options).appendTo(element);
            }
        },

        createGradientElement: function (name, colors, x1, y1, x2, y2, element) {
            var colorName;
            if (Object.prototype.toString.call(colors) == '[object Array]') {
                var options = {
                    'id': this.svgObj.id + '_' + name + 'Gradient',
                    'x1': x1 + '%',
                    'y1': y1 + '%',
                    'x2': x2 + '%',
                    'y2': y2 + '%'
                };
                var cName = '#' + this.svgObj.id + '_' + name + 'Gradient';
                this.drawGradient(options, colors, element);
                colorName = 'url(#' + this.svgObj.id + '_' + name + 'Gradient)';
            }
            else {
                colorName = colors;
            }
            return colorName;
        },

        drawGradient: function (options, gradientEle, element) {

            var defs = this.createDefs();
            var linerGradient = document.createElementNS(this.svgLink, "linearGradient");
            $(linerGradient).attr(options);
            for (var i = 0; i < gradientEle.length; i++) {
                var stop = document.createElementNS(this.svgLink, "stop");
                $(stop).attr({
                    'offset': gradientEle[i].colorStop,
                    'stop-color': gradientEle[i].color,
                    'stop-opacity': 1
                });
                $(stop).appendTo(linerGradient);
            }

            $(linerGradient).appendTo(defs);
            $(defs).appendTo(element);
        },

        drawText: function (options, label, groupEle, font) {

            if ($("#" + options.id).length > 0)
                this._textAttrReplace(options, label, font);
            else {
                var text = document.createElementNS(this.svgLink, "text");
                var $text = $(text);


                if (jQuery.type(label) == "array") {
                    var j = 0;
                    $text.attr(options);
                    for (var i = 0; i < label.length; i++) {
                        var textspan = document.createElementNS(this.svgLink, "tspan");
                        textspan.textContent = label[i];
                        $(textspan).attr({ "x": options.x, "dy": j });
                        $(textspan).appendTo(text);
                        var bounds = ej.EjSvgRender.utils._measureText(label[i], null, font);
                        j = bounds.height + 2;
                    }
                } else {
                    text.textContent = label;
                    $text.attr(options);
                }
                $text.appendTo(groupEle);


            }
        },
        //text element attributes values are replaced 
        _textAttrReplace: function (options, label, font) {
            $("#" + options.id).attr(options);

            if (jQuery.type(label) == "array") {
                var elements = $("#" + options.id).children("tspan");
                var j = 0;
                if (elements.length > 0 && elements.length == label.length) {
                    for (var i = 0; i < elements.length; i++) {
                        var element = elements[i];
                        $(element).attr({ "x": options.x, "dy": j });
                        element.textContent = label[i];
                        var bounds = ej.EjSvgRender.utils._measureText(label[i], null, font);
                        j = bounds.height + 2;
                    }
                }
                else {
                    $("#" + options.id).remove();
                    this.drawText(options, label, this.gTransToolEle, font);

                }
            } else {
                $("#" + options.id).text(label);
            }

        },

        drawImage: function (options, element) {

            var img = document.createElementNS(this.svgLink, 'image');
            img.setAttributeNS(null, 'height', options.height);
            img.setAttributeNS(null, 'width', options.width);
            img.setAttributeNS('http://www.w3.org/1999/xlink', 'href', options.href);
            img.setAttributeNS(null, 'x', options.x);
            img.setAttributeNS(null, 'y', options.y);
            img.setAttributeNS(null, 'id', options.id);
            img.setAttributeNS(null, 'visibility', options.visibility);
            if (!ej.util.isNullOrUndefined(options.clippath) || !ej.util.isNullOrUndefined(options.preserveAspectRatio)) {
                img.setAttributeNS(null, 'clip-path', options.clippath);
                img.setAttributeNS(null, "preserveAspectRatio", options.preserveAspectRatio);
            }
            $(img).appendTo(element);

        },

        createDefs: function () {
            var defs = document.createElementNS(this.svgLink, "defs");
            return defs;
        },

        createClipPath: function (options) {
            var clipPath = document.createElementNS(this.svgLink, "clipPath");
            $(clipPath).attr(options);
            return clipPath;
        },
        createForeignObject: function (options) {
            var object = document.createElementNS(this.svgLink, "foreignObject");
            $(object).attr(options);
            return object;
        },
        createGroup: function (options) {
            var group = document.createElementNS(this.svgLink, "g");
            $(group).attr(options);
            return group;
        },

        createText: function (options, label) {
            var text = document.createElementNS(this.svgLink, "text");
            $(text).attr(options);
            if (!ej.util.isNullOrUndefined(label))
                text.textContent = label;
            return text;
        },

        createPath: function (options) {
            var path = document.createElementNS(this.svgLink, "path");
            $(path).attr(options);
            return path;
        },

        createCircle: function (options) {
            var circle = document.createElementNS(this.svgLink, "circle");
            $(circle).attr(options);
            return circle;
        },

        createLine: function (options) {
            var line = document.createElementNS(this.svgLink, "line");
            $(line).attr(options);
            return line;
        },


        _getAttrVal: function (ele, val, option) {
            var value = $(ele).attr(val);
            if (value != null)
                return value;
            else
                return option;
        },


        hexFromRGB: function (color) {
            var r = color.R;
            var g = color.G;
            var b = color.B;
            var hex = [r.toString(16), g.toString(16), b.toString(16)];
            $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
            return hex.join("").toUpperCase();
        },

        createDelegate: function (context, handler) {
            return function (e) {
                handler.apply(context, [e, this]);
            };
        },

        drawClipPath: function (options, element) {

            var defs = this.createDefs();
            var clipPath = this.createClipPath({ 'id': options.id });
            this.drawRect(options, clipPath);

            this.append(clipPath, defs);

            this.append(defs, element);

        },

        append: function (childEle, parentEle) {
            $(childEle).appendTo(parentEle);
        },
        _setAttr: function (element, attribute) {
            $(element).attr(attribute);
        }
    };

    ej.EjSvgRender.commonChartEventArgs = {
        Cancel: false,
        Data: null
    };
    ej.EjSvgRender.utils = {



        _decimalPlaces: function (num) {
            var match = ('' + num).match(/(?:\.(\d+))?(?:[eE]([+-]?\d+))?$/);
            if (!match) { return 0; }
            return Math.max(
           0,
            // Number of digits right of decimal point.
           (match[1] ? match[1].length : 0)
            // Adjust for scientific notation.
           - (match[2] ? +match[2] : 0));
        },
        _getLabelContent: function (pos, axis) {
            switch (axis._categoryValueType) {
                case 'number':
                    var customFormat = (ej.util.isNullOrUndefined(axis.labelFormat)) ? null : axis.labelFormat.match('{value}');
                    return (ej.util.isNullOrUndefined(axis.labelFormat)) ? axis.labels[Math.floor(pos)] : (customFormat != null) ? axis.labelFormat.replace('{value}', axis.labels[Math.floor(pos)]) : (Globalize.format(axis.labels[Math.floor(pos)], axis.labelFormat));
                case 'date':
                    return (Globalize.format(new Date(axis.labels[Math.floor(pos)]), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? "dd/MM/yyyy" : axis.labelFormat)));
                case 'string':
                    return axis.labels[Math.floor(pos)];
                default:
                    return "";
            }
        },

        //Calculation for label size in template
        _getSeriesTemplateSize: function (point, pointIndex, series, isLeft, sender) {
            var point;
            var pointIndex;
            var areaType = sender.model.AreaType;

            var chartContainer = sender._id;
            var seriesIndex = $.inArray(series, sender.model._visibleSeries);

            if ($('#template_group_' + chartContainer).length != 0)
                var templateContainer = $('#template_group_' + chartContainer);
            else
                templateContainer = $("<div></div>").attr('id', "template_group_" + chartContainer);

            templateContainer.css('position', 'relative').css('z-index', 1000);
            var cloneNode = $("#" + series.marker.dataLabel.template).clone();
            $(cloneNode).attr("id", series.marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            var $cloneNode = $(cloneNode);
            $cloneNode.css("position", "absolute");

            point.count = 1;
            var data = { series: series, point: point };
            $cloneNode.html($cloneNode.html().parseTemplate(data));

            var display = (areaType == "cartesianaxes" || !series.animation || series.type.toLowerCase() == "pyramid") ? "block" : "none";
            $cloneNode.css("display", display).appendTo($(templateContainer));
            $(templateContainer).appendTo('#' + chartContainer);
            point.size = { height: $cloneNode.height(), width: $cloneNode.width() };

            if (isLeft) {
                if (ej.util.isNullOrUndefined(series.LeftLabelMaxHeight) || series.LeftLabelMaxHeight < point.size.height) {
                    series.LeftLabelMaxHeight = point.size.height;
                }
                if (ej.util.isNullOrUndefined(series.LeftLabelMaxWidth) || series.LeftLabelMaxWidth < point.size.width) {
                    series.LeftLabelMaxWidth = point.size.width;
                }
            }
            else {
                if (ej.util.isNullOrUndefined(series.RightLabelMaxHeight) || series.RightLabelMaxHeight < point.size.height) {
                    series.RightLabelMaxHeight = point.size.height;
                }
                if (ej.util.isNullOrUndefined(series.RightLabelMaxWidth) || series.RightLabelMaxWidth < point.size.width) {
                    series.RightLabelMaxWidth = point.size.width;
                }
            }

        },
        getMinPointsDelta: function (currentseries, chartObj, start) {
            var chart = chartObj;
            if (isNaN(chart.m_minPointsDelta) || chart.m_minPointsDelta == 1) {
                chart.m_minPointsDelta = Number.MAX_VALUE;
                $.each(chart.model._visibleSeries, function (index, series) {
                    if (series.visibility.toLowerCase() == 'visible' && currentseries.xAxis == series.xAxis) {
                        var xValues = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
                        if (xValues.length == 1) {
                            var minValue = (start) ? start : series.xAxis.visibleRange.min;
                            var delta = xValues[0].X - minValue;
                            if (delta != 0) {
                                chart.m_minPointsDelta = Math.min(chart.m_minPointsDelta, delta);
                            }
                        }
                        else {
                            $.each(xValues, function (pointIndex, point) {
                                if (pointIndex > 0) {
                                    var deltaValue = point.X - xValues[pointIndex - 1].X;

                                    if (deltaValue != 0) {
                                        chart.m_minPointsDelta = Math.min(chart.m_minPointsDelta, deltaValue);
                                    }
                                }
                            });
                        }
                    }
                });

                if (chart.m_minPointsDelta == Number.MAX_VALUE) {
                    chart.m_minPointsDelta = 1;
                }
            }

            return chart.m_minPointsDelta;
        },
        //Calculation for label size
        _getSeriesMaxLabel: function (series) {
            var maxTxtDim = { width: 0, height: 0 };
            var width = $(this.svgObject).width();
            var labels = [];
            if (series.labels.length > 0) {
                for (var j = 0; j < series.labels.length; j++) {
                    var dim = this._measureText(series.visibleLabels[j], width, series.marker.dataLabel.font);
                    if (maxTxtDim.width < dim.width) {
                        maxTxtDim.width = dim.width;
                    }
                    if (maxTxtDim.height < dim.height) {
                        maxTxtDim.height = dim.height;
                    }
                }
                series.LabelMaxHeight = maxTxtDim.height;
                series.LabelMaxWidth = maxTxtDim.width;
            }
            else {
                labels.push(series.rightsidePoints);
                labels.push(series.leftsidePoints);

                for (var k = 0; k < labels.length; k++) {
                    for (var j = 0; j < labels[k].length; j++) {
                        var text = (labels[k][j].text) ? labels[k][j].text : labels[k][j].y;
                        var dim = this._measureText(text, width, series.marker.dataLabel.font);
                        if (maxTxtDim.width < dim.width) {
                            maxTxtDim.width = dim.width;
                        }
                        if (maxTxtDim.height < dim.height) {
                            maxTxtDim.height = dim.height;
                        }
                    }
                    if (k == 0) {
                        series.RightLabelMaxHeight = maxTxtDim.height;
                        series.RightLabelMaxWidth = maxTxtDim.width;
                    }
                    else {
                        series.LeftLabelMaxHeight = maxTxtDim.height;
                        series.LeftLabelMaxWidth = maxTxtDim.width;
                    }
                }
            }


        },
        _getMaxLabelWidth: function (axis, sender) {
            var maxTxtDim = { width: 0, height: 0 };
            var rotateLabel = '';
            var currentRow = 1;
            var vmlrendering = sender.svgRenderer.vmlNamespace;
            var range = axis.visibleRange;
            var intersectAction = axis.labelIntersectAction ? axis.labelIntersectAction.toLowerCase() : "";
            var labelPlacement = axis.labelPlacement;
            var opposedPosition = axis.opposedPosition;
            var roundingPlaces = axis.roundingPlaces;
            var labelRotation = axis.labelRotation;
            var orientation = axis.orientation;
            var maxHeight = 0;
            var prevLabels = [];

            if (axis.visible) {
                var visibleLabels = axis.visibleLabels;
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    axis.visibleLabels[j].y = 0;
                    var dim = this._measureText(axis.visibleLabels[j].Text, $(this.svgObject).width(), axis.font, axis.labelRotation);
                    if (maxTxtDim.width < dim.width) {
                        maxTxtDim.width = dim.width;
                        rotateLabel = axis.visibleLabels[j].Text;
                    }
                    if (maxTxtDim.height < dim.height)
                        maxTxtDim.height = dim.height;
                }

                // initialize rows to 1
                maxTxtDim.rows = 1;
                maxTxtDim.maxWidth = maxTxtDim.width;
                maxTxtDim.maxHeight = maxTxtDim.height;

                if (!vmlrendering && labelRotation) {
                    maxHeight = this.rotatedLabel(axis, sender, labelRotation, rotateLabel);
                    if (axis.labelIntersectAction && axis.labelIntersectAction.toLowerCase() == 'multiplerows')
                        maxHeight = maxTxtDim.height + maxHeight;
                    maxTxtDim.height = maxHeight;
                }

                if (axis.labelIntersectAction) {
                    if (!vmlrendering) {
                        if (intersectAction == 'rotate90')
                            maxTxtDim.height = this.rotatedLabel(axis, sender, 90, rotateLabel);
                        else if (intersectAction == 'rotate45')
                            maxTxtDim.height = this.rotatedLabel(axis, sender, 45, rotateLabel);
                    }
                    if (intersectAction == 'wrap') {
                        // pointX calculation to find the gap between the ticks
                        labelPlacement = (!(labelPlacement)) ? ej.Chart.labelPlacement.BetweenTicks : labelPlacement;
                        var ticksbwtLabel = ((axis.labels.length > 1) && (labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
                        var tempInterval = ((axis.labels.length > 1) && (ticksbwtLabel < 0)) ? 1 + ticksbwtLabel : visibleLabels[1].Value;
                        tempInterval = (!(roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(range.interval) == 0 ?
                            1 : ej.EjSvgRender.utils._decimalPlaces(range.interval)))) : parseFloat(tempInterval.toFixed(roundingPlaces));
                        var pointX = Math.ceil(((tempInterval - range.min) / (range.max - range.min)) * (axis.length));
                        var row = Math.round(maxTxtDim.width / pointX);
                        maxTxtDim.rows = labelRotation ? row - currentRow : row + currentRow;
                        maxTxtDim.height = (maxTxtDim.height + (row) * maxTxtDim.height);
                    }
                    else if (intersectAction == 'multiplerows') {
                        var spaceValue = sender._getLegendSpace();
                        var chartBorderWidth = sender.model.border.width;

                        var verticalaxis = sender.model.axes[1];
                        var realWidth = $(sender.svgObject).width() - sender.model.margin.left - sender.model.margin.right;
                        var axisTitleHeight = (axis.title.text == "" || !(axis.visible)) ? 0 : (this._measureText(axis.title.text, realWidth, axis.title.font).height + (2 * sender.model.elementSpacing));
                        var vAxesWidth = sender.model.elementSpacing + axisTitleHeight + axis.majorTickLines.size + axis.axisLine.width;
                        var yLabels = sender._getYValues(sender.model._visibleSeries[0]);
                        var largest = Math.max.apply(Math, yLabels);
                        var vaxis = sender.model.axes[1];
                        var dim = this._measureText(largest, $(this.svgObject).width(), vaxis.font, vaxis.labelRotation);

                        var rightSpacing = spaceValue.rightLegendWidth + vAxesWidth + dim.width + sender.model.margin.right + sender.model.margin.left + (2 * chartBorderWidth);
                        var boundsWidth = $(sender.svgObject).width() - (rightSpacing);
                        if (orientation.toLowerCase() == 'horizontal') {
                            var addedMaxHeight = 0;
                            //loop to get current label
                            for (j = 0; j < visibleLabels.length; j++) {
                                //declaration
                                var currentLabel = visibleLabels[j];
                                var flag = false;
                                maxHeight = Math.max(maxHeight, addedMaxHeight)
                                addedMaxHeight = currentLabel.y = maxTxtDim.height;
                                var text = this._measureText(currentLabel.Text, $(this.svgObject).width(), axis.font);
                                var textWidth = text.width;
                                var textHeight = text.height;
                                var currentPoint = Math.abs(Math.floor(((currentLabel.Value - range.min) / (range.delta)) * (boundsWidth)));
                                // loop to get previous labels                         
                                for (var k = 0; k < prevLabels.length && !flag; k++) {
                                    for (var l = 0; prevLabels[k] && l < prevLabels[k].length; l++) {
                                        var prevLabel = prevLabels[k][l];
                                        var prevPoint = Math.abs(Math.floor(((prevLabel.Value - range.min) / (range.delta)) * (boundsWidth)));
                                        var preTextWidth = this._measureText(prevLabel.Text, $(this.svgObject).width(), axis.font).width;
                                        var value = prevPoint + preTextWidth / 2;
                                        if (value > currentPoint - textWidth / 2 && !ej.util.isNullOrUndefined(currentLabel.y)) {
                                            addedMaxHeight = currentLabel.y + textHeight;
                                            currentLabel.y += textHeight;
                                            if (k + 1 == prevLabels.length)
                                                flag = true;
                                        }
                                        else {
                                            if (l + 1 == prevLabels[k].length) {
                                                flag = true;
                                                break;
                                            }
                                        }
                                    }

                                }
                                currentLabel.y = addedMaxHeight;
                                row = (addedMaxHeight / textHeight) - 1;
                                if (prevLabels[row] == undefined)
                                    prevLabels[row] = [];
                                prevLabels[row].push(currentLabel);

                            }
                            maxHeight = (prevLabels.length * textHeight);
                            //   maxHeight = opposedPosition ? maxHeight + textHeight : maxHeight;
                            maxTxtDim.height = maxTxtDim.height > maxHeight ? maxTxtDim.height : maxHeight;
                            //calculating rows
                            if (opposedPosition)
                                maxTxtDim.rows = Math.round((maxHeight + textHeight) / textHeight);
                            else
                                maxTxtDim.rows = Math.round((maxHeight + textHeight) / textHeight);
                            maxTxtDim.rows = labelRotation ? maxTxtDim.rows - 1 : maxTxtDim.rows + 1;
                            if (maxTxtDim.rows < 1) maxTxtDim.rows = 1;
                        }
                        if (orientation.toLowerCase() == 'vertical') {
                            var addedMaxWidth = 0;
                            //loop to get current label
                            for (j = 0; j < axis.visibleLabels.length; j++) {
                                currentLabel = axis.visibleLabels[j];
                                text = this._measureText(currentLabel.Text, $(this.svgObject).width(), axis.font);
                                textWidth = text.width;
                                textHeight = text.height;
                                currentPoint = Math.abs(Math.floor(((currentLabel.Value - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.length)));
                                for (i = 0; i < j; i++) {
                                    // loop to get previous labels
                                    prevLabel = axis.visibleLabels[i];
                                    prevPoint = Math.abs(Math.floor(((prevLabel.Value - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.length)));
                                    var prevTextHeight = this._measureText(prevLabel.Text, $(this.svgObject).width(), axis.font).height;
                                    value = prevPoint + prevTextHeight / 2;
                                    if (value > currentPoint - textHeight / 2 && axis.visibleLabels[i].y == currentLabel.y) {
                                        addedMaxWidth = currentLabel.y + textWidth;
                                        currentLabel.y = addedMaxWidth;
                                    }
                                }
                                var maxheight = addedMaxWidth;
                            }
                            // label width + multiple rows width + gap between the rows
                            maxTxtDim.width = maxTxtDim.width + maxheight + 5;
                        }
                    }
                }

            }
            return maxTxtDim;
        },

        rotatedLabel: function (axis, sender, value, rotatedLabel) {
            // to get height of rotated labels
            var rotatedOptions = {
                'font-size': axis.font.size,
                'transform': 'rotate(' + value + ',0,0)',
                'font-family': axis.font.fontFamily,
                'font-style': axis.font.fontStyle,
                'text-anchor': 'middle'
            };
            var text = sender.svgRenderer.createText(rotatedOptions, rotatedLabel);
            var height = Math.ceil((this._measureBounds(text, sender).height));
            return height;
        },

        _getTransform: function (xAxis, yAxis, invertedAxis) {
            var x, y, width, height;
            if (invertedAxis) {
                x = yAxis.x;
                y = xAxis.y;
                width = yAxis.width;
                height = xAxis.height;
            } else {
                {
                    x = xAxis.x;
                    y = yAxis.y;
                    width = xAxis.width;
                    height = yAxis.height;
                }
            }
            return { x: x, y: y, width: width, height: height };
        },
        _measureText: function (text, maxwidth, font) {
            var textObj = document.createElement('text');
            var style = null, size = null, family = null;
            textObj.innerHTML = text;
            if (font != undefined && font.size == undefined) {
                var fontarray = font;
                fontarray = fontarray.split(" ");
                style = fontarray[0];
                size = fontarray[1];
                family = fontarray[2];
            }

            if (font != null) {
                textObj.style.fontSize = (font.size > 0) ? (font.size + "px") : font.size ? font.size : size;
                textObj.style.fontStyle = font.fontStyle ? font.fontStyle : style;
                textObj.style.fontFamily = font.fontFamily ? font.fontFamily : family;
            }
            textObj.style.backgroundColor = 'white';
            textObj.style.position = 'absolute';
            textObj.style.top = -100;
            textObj.style.left = 0;
            if (maxwidth)
                textObj.style.maxwidth = maxwidth + "px";

            document.body.appendChild(textObj);
            var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
            $(textObj).remove();
            return bounds;
        },
        _measureBounds: function (element, sender) {
            sender.svgRenderer.append(element, sender.svgObject);
            sender.svgRenderer.append(sender.svgObject, sender.element);
            var box = element.getBoundingClientRect();
            var bounds = { left: box.left, right: box.right, top: box.top, bottom: box.bottom, width: (box.right - box.left), height: (box.bottom - box.top) };
            $(element).remove();
            return bounds;
        },
        //Draw clip path for each series to avoid series overlap in multiple axes zooming
        _drawAxesBoundsClipPath: function (gSeriesGroupEle, options, sender) {
            var clipOptions;
            var element = $(gSeriesGroupEle);
            var trans = this._getTransform(options.xAxis, options.yAxis, sender.model.requireInvertedAxes);
            clipOptions = {
                'id': gSeriesGroupEle.id + '_ClipRect',
                'x': 0,
                'y': 0,
                'width': (trans.width),
                'height': (trans.height),
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'transparent'
            };

            sender.svgRenderer.drawClipPath(clipOptions, gSeriesGroupEle);
            element.attr('clip-path', 'url(#' + clipOptions.id + ')');

        },
        _getStringBuilder: function () {

            var data = [];
            var counter = 0;

            return {
                // adds string s to the stringbuilder

                append: function (s) {
                    data[counter++] = s;
                    return this;
                },

                // removes j elements starting at i, or 1 if j is omitted

                remove: function (i, j) {
                    data.splice(i, j || 1);
                    return this;
                },

                // inserts string s at i

                insert: function (i, s) {
                    data.splice(i, 0, s);
                    return this;
                },

                // builds the string

                toString: function (s) { return data.join(s || ""); }
            };


        },
        _addRegion: function (chart, bounds, series, point, pointIndex) {
            var type = series.type;
            var seriesIndex = $.inArray(series, chart.model._visibleSeries);
            var regionItem = { SeriesIndex: seriesIndex, Region: { PointIndex: pointIndex, Bounds: bounds }, type: type };

            chart.model.chartRegions.push(regionItem);
        },

        AddRegion: function (chart, bounds, isStripLine) {

            if (isStripLine) {
                var regionItem = { isStripLine: isStripLine, Region: { Bounds: bounds } };
                chart.model.chartRegions.push(regionItem);
            }
        },


        _getSvgXY: function (x, y, series, sender) {
            var svgX, svgY;
            if (!(sender.model.requireInvertedAxes)) {
                svgX = x + series.xAxis.x;
                svgY = y + series.yAxis.y;
            } else {
                svgX = x + series.yAxis.x;
                svgY = y + series.xAxis.y;
            }
            return { X: svgX, Y: svgY };
        },
        _getPoint: function (point, series) {
            var x = point.X;
            var y = point.YValues[0];

            var xvalue = (series.xAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((x == 0 ? 1 : x), series.xAxis.logBase) : x;
            var yvalue = (series.yAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((y == 0 ? 1 : y), series.xAxis.logBase) : y;

            xvalue = this._getPointXY(xvalue, series.xAxis.visibleRange, series.xAxis.isInversed) * (series.xAxis.width);
            yvalue = (1 - this._getPointXY(yvalue, series.yAxis.visibleRange, series.yAxis.isInversed)) * (series.yAxis.height);

            return { X: xvalue, Y: yvalue };
        },
        _getPointXY: function (value, Range, isInversed) {

            var result = 0;
            result = (value - Range.min) / (Range.delta);

            return (isInversed) ? (1 - result) : result;

        },

        _dateTimeLabelFormat: function (intervalType) {
            var format = 'd';
            if (intervalType.toLowerCase() == 'years')
                format = 'MMM, yyyy';
            else if (intervalType.toLowerCase() == 'months')
                format = 'dd, MMM';
            else if (intervalType.toLowerCase() == 'days')
                format = 'MM/dd/yyyy';
            else if (intervalType.toLowerCase() == 'hours')
                format = 'dd, hh:mm';
            else if (intervalType.toLowerCase() == 'seconds' || intervalType.toLowerCase() == 'minutes')
                format = 'hh:mm:ss';
            else if (intervalType.toLowerCase() == 'milliseconds')
                format = 'hh:mm:ss:tt';
            return format;
        },
        _getFontString: function (fontObj) {
            if (fontObj == null)
                fontObj = {};
            if (!fontObj.FontFamily)
                fontObj.FontFamily = "Arial";
            if (!fontObj.FontStyle)
                fontObj.FontStyle = "Normal";
            if (!fontObj.Size)
                fontObj.Size = "12px";

            return fontObj.FontStyle + " " + fontObj.Size + " " + fontObj.FontFamily;
        },
        _valueToCoefficient: function (axis, value) {

            var yvalue = (axis._valueType.toLowerCase() == "logarithmic") ?
              ej.EjSvgRender.utils._logBase((value == 0 ? 1 : value), axis.logBase) : value;

            yvalue = (yvalue - axis.visibleRange.min) / (axis.visibleRange.delta);

            return (axis.isInversed) ? 1 - yvalue : yvalue;
        },
        _getBoundingClientRect: function (element, sender, series, invertedAxes) {
            var box = element.getBoundingClientRect();
            var position = $("#" + (sender.svgObject.id))[0].getBoundingClientRect();
            var xSeries, ySeries;
            if (invertedAxes) {
                xSeries = this._getTransform(series.xAxis, series.yAxis, true).x;
                ySeries = this._getTransform(series.xAxis, series.yAxis, true).y;
            } else {
                xSeries = this._getTransform(series.xAxis, series.yAxis, false).x;
                ySeries = this._getTransform(series.xAxis, series.yAxis, false).y;
            }
            var x = box.left - (xSeries + position.left);
            var y = box.top - (ySeries + position.top);
            return { x: x, y: y, width: (box.right - box.left), height: (box.bottom - box.top) };
        },
        _minMax: function (value, min, max) {
            return value > max ? max : (value < min ? min : value);
        },
        _inside: function (value, range) {
            if (value === "")
                return false;
            return (value <= range.max) && (value >= range.min);
        },
        _logBase: function (val, base) {
            return Math.log(val) / Math.log(base);
        },
        _correctRect: function (x1, y1, x2, y2) {
            return { X: Math.min(x1, x2), Y: Math.min(y1, y2), Width: Math.abs(x2 - x1), Height: Math.abs(y2 - y1) };
        },
        _getValuebyPoint: function (x, y, series) {



            var xValue = (series.xAxis.isInversed) ? (1 - (x / series.xAxis.width)) : (x / series.xAxis.width);
            var yValue = (series.yAxis.isInversed) ? (1 - (y / series.yAxis.height)) : (y / series.yAxis.height);

            xValue = xValue * (series.xAxis.visibleRange.delta) + series.xAxis.visibleRange.min;

            yValue = yValue * (series.yAxis.visibleRange.delta) + series.yAxis.visibleRange.min;

            xValue = (series.xAxis._valueType == "logarithmic") ? Math.pow(series.xAxis.logBase, xValue) : xValue;

            yValue = (series.yAxis._valueType == "logarithmic") ? Math.pow(series.yAxis.logBase, yValue) : yValue;

            return { PointX: xValue, PointY: yValue };

        }

    };
    ej.EjSvgRender.chartSymbol =
    {
        _drawSeriesType: function (location, symbolStyle, sender) {
            switch (sender.model.series[symbolStyle.SeriesIndex].type) {
                case ej.Chart.type.Pie:
                case ej.Chart.type.Bubble:
                    this._drawCircle(location, symbolStyle, sender, sender.gLegendItemEle);
                    break;
                case ej.Chart.type.Column:
                    this._drawRectangle(location, symbolStyle, sender, sender.gLegendItemEle);
                    break;
                default:
                    this._drawRectangle(location, symbolStyle, sender, sender.gLegendItemEle);
                    break;
            }

        },

        _drawCircle: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;

            var options = {
                'id': symbolStyle.ID, 'cx': location.startX, 'cy': location.startY, 'r': radius,
                'fill': symbolStyle.Style.Color, 'stroke-width': symbolStyle.Style.BorderWidth, 'stroke': symbolStyle.Style.BorderColor, 'opacity': symbolStyle.Style.Opacity, 'visibility': symbolStyle.Style.Visibility
            };

            svgRender.drawCircle(options, element);
        },

        _drawLeftArrow: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;
            var path;
            path = "M" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4));
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);

        },


        _drawRightArrow: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;
            var path;
            path = "M" + " " + ((location.startX + (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + style.width) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2))) + " " + (location.startY) + " " + "L" + " " + ((location.startX + (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4));
            var x = ((location.startX + (style.width / 2)));
            var y = location.startY;
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'transform': 'rotate(180,' + x + ',' + y + ')',
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };


            svgRender.drawPath(options, element);


        },


        _drawUpArrow: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;
            var path;
            path = "M" + " " + ((location.startX) + style.width) + " " + (location.startY - (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((location.startX) + style.width) + " " + (location.startY - (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY - (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY - (style.height / 2) + (-style.height / 2)) + " " + "L" + " " + ((location.startX)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY - (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((location.startX) + style.width) + " " + (location.startY - (style.height / 2) + (style.height / 4));
            var x = ((location.startX));
            var y = (location.startY - (style.height / 2));
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'transform': 'rotate(90,' + x + ',' + y + ')',
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };


            svgRender.drawPath(options, element);


        },

        _drawDownArrow: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;
            var path;
            path = "M" + " " + ((location.startX) + style.width) + " " + (location.startY + (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((location.startX) + style.width) + " " + (location.startY + (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (-style.height / 2)) + " " + "L" + " " + ((location.startX)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((location.startX) + style.width) + " " + (location.startY + (style.height / 2) + (style.height / 4));


            var x = ((location.startX));
            var y = (location.startY + (style.height / 2));
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'transform': 'rotate(-90,' + x + ',' + y + ')',
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };


            svgRender.drawPath(options, element);

        },

        _drawCross: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " +
            "M" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2));
            var options = {
                'id': symbolStyle.ID,
                'opacity': symbolStyle.Style.Opacity,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.Color,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);

        },

        _drawHorizLine: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY);
            var options = {
                'id': symbolStyle.ID,
                'opacity': symbolStyle.Style.Opacity,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.Color,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);


        },
        _drawVertLine: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;
            var path;
            path = "M" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2));
            var options = {
                'id': symbolStyle.ID,
                'opacity': symbolStyle.Style.Opacity,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.Color,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };


            svgRender.drawPath(options, element);

        },

        _drawTriangle: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;
            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2));
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);

        },

        _drawInvertedTriangle: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2));
            var x = location.startX;
            var y = location.startY;
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'transform': 'rotate(180,' + x + ',' + y + ')',
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);


        },

        _drawHexagon: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY);
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);

        },

        _drawWedge: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + style.width) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (3 * (style.width / 4))) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY);
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);

        },

        _drawPentagon: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var eq = 72;
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            var sb = svgRender.utils._getStringBuilder();
            for (var i = 0; i <= 5; i++) {
                var deg = i * eq;
                var rad = (Math.PI / 180) * deg;
                var x1 = radius * Math.cos(rad);
                var y1 = radius * Math.sin(rad);
                if (i == 0)
                    sb.append("M" + " " + (location.startX + x1) + " " + (location.startY + y1) + " ");

                else
                    sb.append("L" + " " + (location.startX + x1) + " " + (location.startY + y1) + " ");


            }
            var path = sb.toString();
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);


        },

        _drawStar: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var style = symbolStyle.ShapeSize;
            var svgObj = sender.svgObject;

            var path;
            path = "M" + " " + (location.startX + (style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 6)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 6)) + " " + "L" + " " + (location.startX + (-style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 3)) + " " + (location.startY + (-style.height / 2));
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);


        },

        _drawRectangle: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2));
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);



        },

        _drawTrapezoid: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY);
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);

        },

        _drawDiamond: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var path;
            path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY);

            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path
            };

            svgRender.drawPath(options, element);


        },
        _drawEllipse: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;

            var x = location.startX;
            var y = location.startY;
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth,
                'stroke': symbolStyle.Style.BorderColor,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'cx': x,
                'cy': y,
                'rx': style.width,
                'ry': style.height / 2
            };


            svgRender.drawEllipse(options, element);


        },
        _drawImage: function (location, symbolStyle, sender, element) {
            var svgRender = sender.svgRenderer;
            var svgObj = sender.svgObject;
            var style = symbolStyle.ShapeSize;
            var x = location.startX + (-style.width / 2);
            var y = location.startY + (-style.width / 2);
            var width = style.width;
            var height = style.height;
            var options = {
                'id': svgObj.id + '_image' + symbolStyle.PointIndex, 'height': height, 'width': width, 'href': symbolStyle.Imageurl,
                'x': x, 'y': y, 'visibility': 'visible'
            };

            svgRender.drawImage(options, element);


        }
    };
    ej.EjSvgRender.seriesPalette = {
        defaultMetro: ["#E94649", "#F6B53F", "#6FAAB0", "#C4C24A", "#FB954F", "#005277", "#8BC652", "#69D2E7", "#E27F2D", "#6A4B82"],
        defaultGradient:
            {
                borderColors: ["#F34649", "#F6D321", "#6EB9B0", "#CBC26A", "#FBAF4F", "#E2CDB1", "#FFC0B7", "#68E1E6", "#E1A62D", "#9C6EBF"],
                seriesColors: [[{ color: "#F34649", colorStop: "0%" }, { color: "#B74143", colorStop: "100%" }],
         [{ color: "#F6D321", colorStop: "0%" }, { color: "#F6AE26", colorStop: "100%" }],
         [{ color: "#6EB9B0", colorStop: "0%" }, { color: "#3F77BD", colorStop: "100%" }],
         [{ color: "#CBC26A", colorStop: "0%" }, { color: "#9AAD21", colorStop: "100%" }],
         [{ color: "#FBAF4F", colorStop: "0%" }, { color: "#F07542", colorStop: "100%" }],
         [{ color: "#E2CDB1", colorStop: "0%" }, { color: "#AAA089", colorStop: "100%" }],
         [{ color: "#8BC652", colorStop: "0%" }, { color: "#6F9E41", colorStop: "100%" }],
         [{ color: "#68E1E6", colorStop: "0%" }, { color: "#3D9CBE", colorStop: "100%" }],
         [{ color: "#E1A62D", colorStop: "0%" }, { color: "#B66824", colorStop: "100%" }],
         [{ color: "#9C6EBF", colorStop: "0%" }], [{ color: "#593F6D", colorStop: "100%" }]]
            },
        blueMetro: ["#005378", "#006691", "#007EB5", "#0D97D4", "#00AEFF", "#14B9FF", "#54CCFF", "#87DBFF", "#ADE5FF", "#C5EDFF"],
        blueGradient:
         {
             seriesColors: [[{ color: "#005277", colorStop: "0%" }, { color: "#00304F", colorStop: "100%" }],
                [{ color: "#006590", colorStop: "0%" }, { color: "#004068", colorStop: "100%" }],
                [{ color: "#007DB4", colorStop: "0%" }, { color: "#00558B", colorStop: "100%" }],
                [{ color: "#0D97D4", colorStop: "0%" }, { color: "#057FC7", colorStop: "100%" }],
                [{ color: "#00ADFE", colorStop: "0%" }, { color: "#008BE9", colorStop: "100%" }],
                [{ color: "#14B8FE", colorStop: "0%" }, { color: "#0798EB", colorStop: "100%" }],
                [{ color: "#53CBFF", colorStop: "0%" }, { color: "#35AFEB", colorStop: "100%" }],
                [{ color: "#86DAFF", colorStop: "0%" }, { color: "#64C0EC", colorStop: "100%" }],
                [{ color: "#ACE5FF", colorStop: "0%" }, { color: "#8DCEED", colorStop: "100%" }],
                [{ color: "#C4ECFF", colorStop: "0%" }], [{ color: "#A3D1E6", colorStop: "100%" }]],
             borderColors: ["#005277", "#006590", "#007DB4", "#0D97D4", "#00ADFE", "#14B8FE", "#53CBFF", "#86DAFF", "#ACE5FF", "#C4ECFF"]
         },
        greenMetro: ["#496612", "#597B15", "#709A1B", "#87B62A", "#9AD926", "#A6DC37", "#BCE654", "#C8E780", "#D5EFA5", "#E2F3BE"],
        greenGradient:
         {

             seriesColors: [[{ color: "#5C7F16", colorStop: "0%" }, { color: "#384C08", colorStop: "100%" }],
                [{ color: "#6A9319", colorStop: "0%" }, { color: "#486009", colorStop: "100%" }],
                [{ color: "#739D1C", colorStop: "0%" }, { color: "#57760B", colorStop: "100%" }],
                [{ color: "#90B546", colorStop: "0%" }, { color: "#6E9215", colorStop: "100%" }],
                [{ color: "#9AD826", colorStop: "0%" }, { color: "#75A010", colorStop: "100%" }],
                [{ color: "#A5DB36", colorStop: "0%" }, { color: "#8EB91D", colorStop: "100%" }],
                [{ color: "#BBE554", colorStop: "0%" }, { color: "#A4C849", colorStop: "100%" }],
                [{ color: "#C8E780", colorStop: "0%" }, { color: "#B4D072", colorStop: "100%" }],
                [{ color: "#D4EEA5", colorStop: "0%" }, { color: "#BFD593", colorStop: "100%" }],
                [{ color: "#E1F2BD", colorStop: "0%" }], [{ color: "#C8D7A8", colorStop: "100%" }]],
             borderColors: ["#5C7F16", "#6A9319", "#739D1C", "#90B546", "#9AD826", "#A5DB36", "#BBE554", "#C8E780", "#D4EEA5", "#E1F2BD"]
         },

        sandleMetro: ["#6C450C", "#82520D", "#A36812", "#C07F1F", "#E69719", "#E89A2B", "#EEB564", "#F3CB93", "#F7DEB4", "#F9E6CA"],
        sandleGradient:
         {


             seriesColors: [[{ color: "#7F602F", colorStop: "0%" }, { color: "#512D04", colorStop: "100%" }],
                [{ color: "#986827", colorStop: "0%" }, { color: "#673803", colorStop: "100%" }],
                [{ color: "#A16C1F", colorStop: "0%" }, { color: "#8A4B05", colorStop: "100%" }],
                [{ color: "#BF812A", colorStop: "0%" }, { color: "#AD630D", colorStop: "100%" }],
                [{ color: "#E49519", colorStop: "0%" }, { color: "#B86607", colorStop: "100%" }],
                [{ color: "#E7992A", colorStop: "0%" }, { color: "#D7780D", colorStop: "100%" }],
                [{ color: "#EDB463", colorStop: "0%" }, { color: "#D98F31", colorStop: "100%" }],
                [{ color: "#F2CA92", colorStop: "0%" }, { color: "#DAAC6F", colorStop: "100%" }],
                [{ color: "#F6DDB3", colorStop: "0%" }, { color: "#DABE8F", colorStop: "100%" }],
                [{ color: "#F8E5C9", colorStop: "0%" }], [{ color: "#DDBE92", colorStop: "100%" }]],
             borderColors: ["#7F602F", "#986827", "#A16C1F", "#BF812A", "#E49519", "#E7992A", "#EDB463", "#F2CA92", "#F6DDB3", "#F8E5C9"]

         }
    };

    ej.EjSvgRender.themes = {
        flatlight:
        {
            background: 'transparent',
            legend:
            {
                font: { color: "#282828" }
            },
            title:
            {
                font: { color: '#565656' }
            },
            primaryXAxis:
            {
                majorGridLines:
                {
                    color: "#DFDFDF"
                },
                majorTickLines:
                {
                    color: "#8E8E8E"
                },
                minorGridLines:
                {
                    color: "#DFDFDF"
                },
                minorTickLines:
                {
                    color: "#8E8E8E"
                },
                axisLine: { color: '#8E8E8E' },
                font: { color: '#282828' },
                title:
                {
                    font: { color: '#282828' }
                },
                crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
            },
            secondaryX:
            {
                majorGridLines:
                {
                    color: "#DFDFDF"
                },
                majorTickLines:
                {
                    color: "#8E8E8E"
                },
                minorGridLines:
                {
                    color: "#DFDFDF"
                },
                minorTickLines:
                {
                    color: "#8E8E8E"
                },
                axisLine: { color: '#8E8E8E' },
                font: { color: '#282828' },
                title:
                {
                    font: { color: '#282828' }
                },
                crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
            },

            primaryYAxis:
            {
                majorGridLines:
                {
                    color: "#DFDFDF"
                },
                majorTickLines:
                {
                    color: "#8E8E8E"
                },
                minorGridLines:
                {
                    color: "#DFDFDF"
                },
                minorTickLines:
                {
                    color: "#8E8E8E"
                },
                axisLine: { color: '#8E8E8E' },
                font: { color: '#282828' },
                title:
                {
                    font: { color: '#282828' }
                },
                crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', borderWidth: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
            },
            secondaryY:
            {
                majorGridLines:
                {
                    color: "#DFDFDF"
                },
                majorTickLines:
                {
                    color: "#8E8E8E"
                },
                minorGridLines:
                {
                    color: "#DFDFDF"
                },
                minorTickLines:
                {
                    color: "#8E8E8E"
                },
                axisLine: { color: '#8E8E8E' },
                font: { color: '#282828' },
                title:
                {
                    font: { color: '#282828' }
                },
                crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
            },
            crosshair:
            {
                line:
                {
                    width: 1,
                    color: 'Black'
                }
            },

            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMetro,
            colors: ej.EjSvgRender.seriesPalette.defaultMetro

        },

        flatdark:
            {
                background: '#111111',

                legend:
                {
                    font: { color: "#C9C9C9" }
                },
                title:
                {
                    font: { color: '#C9C9C9' }
                },
                primaryXAxis:
                 {
                     majorGridLines:
                     {
                         color: "#333333"
                     },
                     majorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     minorGridLines:
                      {
                          color: "#333333"
                      },
                     minorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     axisLine: { color: '#AAAAAA' },
                     font: { color: '#C9C9C9' },
                     title:
                     {

                         font: { color: '#C9C9C9' }
                     },
                     crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
                 },
                secondaryX:
                 {
                     majorGridLines:
                     {
                         color: "#333333"
                     },
                     majorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     minorGridLines:
                      {
                          color: "#333333"
                      },
                     minorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     axisLine: { color: '#AAAAAA' },
                     font: { color: '#C9C9C9' },
                     title:
                     {

                         font: { color: '#C9C9C9' }
                     },
                     crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
                 },
                primaryYAxis:
                 {
                     majorGridLines:
                     {
                         color: "#333333"
                     },
                     majorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     minorGridLines:
                      {
                          color: "#333333"
                      },
                     minorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     axisLine: { color: '#AAAAAA' },
                     font: { color: '#C9C9C9' },
                     title:
                     {

                         font: { color: '#C9C9C9' }
                     },
                     crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
                 },
                secondaryY:
                 {
                     majorGridLines:
                     {
                         color: "#333333"
                     },
                     majorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     minorGridLines:
                      {
                          color: "#333333"
                      },
                     minorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     axisLine: { color: '#AAAAAA' },
                     font: { color: '#C9C9C9' },
                     title:
                     {

                         font: { color: '#C9C9C9' }
                     },
                     crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
                 },
                crosshair:
                 {
                     line:
                     {
                         width: 1,
                         color: 'White'
                     }
                 },

                seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMetro,
                colors: ej.EjSvgRender.seriesPalette.defaultMetro

            },


        gradientlight:
            {
                background: 'transparent',
                legend:
                {
                    font: { color: "#282828" }
                },
                title:
                {
                    font: { color: '#565656' }
                },
                primaryXAxis:
                 {
                     majorGridLines:
                     {
                         color: "#DFDFDF"
                     },
                     majorTickLines:
                      {
                          color: "#8E8E8E"
                      },
                     minorGridLines:
                      {
                          color: "#DFDFDF"
                      },
                     minorTickLines:
                      {
                          color: "#8E8E8E"
                      },
                     axisLine: { color: '#8E8E8E' },
                     font: { color: '#282828' },
                     title:
                     {

                         font: { color: '#282828' }
                     },
                     crosshairLabel: { rx: 3, ry: 3, fill: '#3D3D3D', border: { color: '#3D3D3D', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
                 },
                secondaryX:
                 {
                     majorGridLines:
                     {
                         color: "#DFDFDF"
                     },
                     majorTickLines:
                      {
                          color: "#8E8E8E"
                      },
                     minorGridLines:
                      {
                          color: "#DFDFDF"
                      },
                     minorTickLines:
                      {
                          color: "#8E8E8E"
                      },
                     axisLine: { color: '#8E8E8E' },
                     font: { color: '#282828' },
                     title:
                     {

                         font: { color: '#282828' }
                     },
                     crosshairLabel: { rx: 3, ry: 3, fill: '#3D3D3D', border: { color: '#3D3D3D', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
                 },
                primaryYAxis:
                 {
                     majorGridLines:
                     {
                         color: "#DFDFDF"
                     },
                     majorTickLines:
                      {
                          color: "#8E8E8E"
                      },
                     minorGridLines:
                      {
                          color: "#DFDFDF"
                      },
                     minorTickLines:
                      {
                          color: "#8E8E8E"
                      },
                     axisLine: { color: '#8E8E8E' },
                     font: { color: '#282828' },
                     title:
                     {

                         font: { color: '#282828' }
                     },
                     crosshairLabel: { rx: 3, ry: 3, fill: '#3D3D3D', border: { color: '#3D3D3D', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
                 },
                secondaryY:
                 {
                     majorGridLines:
                     {
                         color: "#DFDFDF"
                     },
                     majorTickLines:
                      {
                          color: "#8E8E8E"
                      },
                     minorGridLines:
                      {
                          color: "#DFDFDF"
                      },
                     minorTickLines:
                      {
                          color: "#8E8E8E"
                      },
                     axisLine: { color: '#8E8E8E' },
                     font: { color: '#282828' },
                     title:
                     {

                         font: { color: '#282828' }
                     },
                     crosshairLabel: { rx: 3, ry: 3, fill: '#3D3D3D', border: { color: '#3D3D3D', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
                 },
                crosshair:
                 {
                     line:
                     {
                         width: 1,
                         color: 'Black'
                     }
                 },

                seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultGradient.borderColors,
                colors: ej.EjSvgRender.seriesPalette.defaultGradient.seriesColors

            },

        gradientdark:
            {
                background: '#111111',
                legend:
                {
                    font: { color: "#C9C9C9" }
                },
                title:
                {
                    font: { color: '#C9C9C9' }
                },
                primaryXAxis:
                 {
                     majorGridLines:
                     {
                         color: "#333333"
                     },
                     majorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     minorGridLines:
                      {
                          color: "#333333"
                      },
                     minorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     axisLine: { color: '#AAAAAA' },
                     font: { color: '#C9C9C9' },
                     title:
                     {

                         font: { color: '#C9C9C9' }
                     },
                     crosshairLabel: { rx: 3, ry: 3, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
                 },
                secondaryX:
                 {
                     majorGridLines:
                     {
                         color: "#333333"
                     },
                     majorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     minorGridLines:
                      {
                          color: "#333333"
                      },
                     minorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     axisLine: { color: '#AAAAAA' },
                     font: { color: '#C9C9C9' },
                     title:
                     {

                         font: { color: '#C9C9C9' }
                     },
                     crosshairLabel: { rx: 3, ry: 3, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
                 },
                primaryYAxis:
                 {
                     majorGridLines:
                     {
                         color: "#333333"
                     },
                     majorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     minorGridLines:
                      {
                          color: "#333333"
                      },
                     minorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     axisLine: { color: '#AAAAAA' },
                     font: { color: '#C9C9C9' },
                     title:
                     {

                         font: { color: '#C9C9C9' }
                     },
                     crosshairLabel: { rx: 3, ry: 3, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
                 },
                secondaryY:
                 {
                     majorGridLines:
                     {
                         color: "#333333"
                     },
                     majorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     minorGridLines:
                      {
                          color: "#333333"
                      },
                     minorTickLines:
                      {
                          color: "#AAAAAA"
                      },
                     axisLine: { color: '#AAAAAA' },
                     font: { color: '#C9C9C9' },
                     title:
                     {

                         font: { color: '#C9C9C9' }
                     },
                     crosshairLabel: { rx: 3, ry: 3, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
                 },
                crosshair:
                 {
                     line:
                     {
                         width: 1,
                         color: 'White'
                     }
                 },

                seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultGradient.borderColors,
                colors: ej.EjSvgRender.seriesPalette.defaultGradient.seriesColors

            }
    };
    $.extend(ej.EjSvgRender.themes, {

        "azure":
        {

            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueMetro,
            colors: ej.EjSvgRender.seriesPalette.blueMetro
        },

        "azuredark":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueMetro,
            colors: ej.EjSvgRender.seriesPalette.blueMetro
        },
        "gradient-azure":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.blueGradient.seriesColors
        },

        "gradient-azuredark":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.blueGradient.seriesColors
        },

        "lime":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenMetro,
            colors: ej.EjSvgRender.seriesPalette.greenMetro
        },

        "limedark":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenMetro,
            colors: ej.EjSvgRender.seriesPalette.greenMetro
        },
        "gradient-lime":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.greenGradient.seriesColors
        },

        "gradient-limedark":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.greenGradient.seriesColors
        },
        "saffron":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleMetro,
            colors: ej.EjSvgRender.seriesPalette.sandleMetro
        },

        "saffrondark":
        {
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleMetro,
            colors: ej.EjSvgRender.seriesPalette.sandleMetro
        },
        "gradient-saffron":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.sandleGradient.seriesColors
    },

        "gradient-saffrondark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.sandleGradient.seriesColors
    }
    });
})(jQuery);;
ej.axisTypes = {};

ej.EjAxisRenderer = function () {
};
var _sideBySeriesPadding = function (chartObj, start, end, axis) {
    var visibleSeries = ej.DataManager(chartObj.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
    var isHorizontal = false;
    var isVertical = false;
    for (var i = 0; i < visibleSeries.length; i++) {
        var type = visibleSeries[i].type.toLowerCase();
        if (type == "column" || type == "rangecolumn" || type == "stackingcolumn" || type == "candle" || type == "hilo" || type == "hiloopenclose") {
            isHorizontal = true;
            break;
        } else
            if (type == "bar" || type == "stackingbar") {
                isVertical = true;
                break;
            }
    }
    if ((isHorizontal && axis.orientation.toLowerCase() == 'horizontal') || (isVertical && axis.orientation.toLowerCase() == 'vertical')) {
        var padding = ej.EjSvgRender.utils.getMinPointsDelta(visibleSeries[0], chartObj, start) * 0.5;
        start = start - padding;
        end = end + padding;
        axis.padding = padding;
    }
    var data, startPadding, endPadding;
    return data = { startPadding: start, endPadding: end };
};
ej.EjStripline = function (chartobj) {
    this.chart = chartobj;
};
(function ($) {
    ej.EjAxisRenderer.prototype = {

        //Calculate min/max value for series
        _seriesMinMax: function (chartObj, axis, seriesCollection, type) {
            if (seriesCollection.length > 0 && seriesCollection[0].points.length > 0) {

                var maxX, minY, minX, maxY;
                var maxMinXValue = { maxX: null, minX: null };
                var maxMinYValue = { maxY: null, minY: null };

                for (var i = 0; i < seriesCollection.length; i++) {
                    if (seriesCollection.length > 0 && seriesCollection[0].points.length > 0) {
                        if (seriesCollection[i].points.length > 0) {
                            if (chartObj.model.AreaType == "cartesianaxes" && seriesCollection[i].visibility.toLowerCase() === 'visible' && seriesCollection[i].points.length > 0) {
                                minX = maxX = seriesCollection[i].points[0].X;
                                maxY = minY = seriesCollection[i].points[0].YValues[0];
                                for (var j = 0; j < seriesCollection[i].points.length; j++) {
                                    var currentPoint = seriesCollection[i].points[j];
                                    if (minX > currentPoint.X)
                                        minX = currentPoint.X;
                                    if (maxX < currentPoint.X)
                                        maxX = currentPoint.X;
                                    if (seriesCollection[i]._hiloTypes) {
                                        for (var y = 0; y < 2; y++) {
                                            if (minY > currentPoint.YValues[y])
                                                minY = currentPoint.YValues[y];
                                            if (maxY < currentPoint.YValues[y])
                                                maxY = currentPoint.YValues[y];
                                        }
                                    } else {
                                        if (minY > currentPoint.YValues[0])
                                            minY = currentPoint.YValues[0];
                                        if (maxY < currentPoint.YValues[0])
                                            maxY = currentPoint.YValues[0];
                                    }
                                }
                                seriesCollection[i].minX = minX;
                                seriesCollection[i].maxX = maxX;
                                seriesCollection[i].minY = minY;
                                seriesCollection[i].maxY = maxY;


                            }
                        }


                    }
                    if (i == 0) {
                        maxMinXValue.maxX = seriesCollection[0].maxX;
                        maxMinXValue.minX = seriesCollection[0].minX;
                        maxMinYValue.maxY = seriesCollection[0].maxY;
                        maxMinYValue.minY = seriesCollection[0].minY;
                    }

                    if (maxMinXValue.minX > seriesCollection[i].minX) {
                        maxMinXValue.minX = seriesCollection[i].minX;
                    }
                    if (maxMinXValue.maxX < seriesCollection[i].maxX) {
                        maxMinXValue.maxX = seriesCollection[i].maxX;
                    }
                    if (maxMinYValue.minY > seriesCollection[i].minY) {
                        maxMinYValue.minY = seriesCollection[i].minY;
                    }
                    if (maxMinYValue.maxY < seriesCollection[i].maxY) {
                        maxMinYValue.maxY = seriesCollection[i].maxY;
                    }

                }
                // Find min/max for stacked series
                if (this.chartObj.model.stackedValue[axis.axisName]) {
                    var value = this.chartObj.model.stackedValue[axis.axisName];
                    if (maxMinYValue.minY > value.min)
                        maxMinYValue.minY = value.min;
                    if (maxMinYValue.maxY < value.max)
                        maxMinYValue.maxY = value.max;
                }

                if (maxMinXValue.maxX == maxMinXValue.minX)
                    maxMinXValue.maxX += 1;
                if (maxMinYValue.maxY == maxMinYValue.minY)
                    maxMinYValue.maxY += 1;

                var deltaX = maxMinXValue.maxX - maxMinXValue.minX;
                var deltaY = maxMinYValue.maxY - maxMinYValue.minY;

                //Set range for each axis
                if (axis.orientation.toLowerCase() == "horizontal") {
                    axis.range = (!(chartObj.model.requireInvertedAxes)) ?
                     (type == "double") ? this._getDoubleAutoRange(axis, maxMinXValue.minX, maxMinXValue.maxX, deltaX)
                         : this._getLogAutoRange(axis, maxMinXValue.minX, maxMinXValue.maxX, deltaX)
                    : (type == "double") ? this._getDoubleAutoRange(axis, maxMinYValue.minY, maxMinYValue.maxY, deltaY)
                        : this._getLogAutoRange(axis, maxMinYValue.minY, maxMinYValue.maxY, deltaY);
                }
                if (axis.orientation.toLowerCase() == "vertical") {
                    axis.range = (!(chartObj.model.requireInvertedAxes)) ?
                         (type == "double") ? this._getDoubleAutoRange(axis, maxMinYValue.minY, maxMinYValue.maxY, deltaY)
                             : this._getLogAutoRange(axis, maxMinYValue.minY, maxMinYValue.maxY, deltaY)
                       : (type == "double") ? this._getDoubleAutoRange(axis, maxMinXValue.minX, maxMinXValue.maxX, deltaX)
                           : this._getLogAutoRange(axis, maxMinXValue.minX, maxMinXValue.maxX, deltaX);
                }
            } else {
                if (!axis.setRange)
                    axis.range = { min: 0, max: 5, interval: 1, Delta: 4 };
            }
        },

        AlignRangeStart: function (sDate, intervalSize, intervalType) {
            var sResult = new Date(sDate);
            if (intervalType.toLowerCase() == "days") {
                var day = Math.floor(Math.floor((sDate.getDate()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), day, 0, 0, 0);
            } else if (intervalType.toLowerCase() == "hours") {
                var hour = Math.floor(Math.floor((sDate.getHours()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), hour, 0, 0);
            } else if (intervalType.toLowerCase() == "milliseconds") {
                var milliseconds = Math.floor(Math.floor((sDate.getMilliseconds()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), sDate.getMinutes(), sDate.getSeconds(), milliseconds);
            } else if (intervalType.toLowerCase() == "seconds") {
                var seconds = Math.floor(Math.floor((sDate.getSeconds()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), sDate.getMinutes(), seconds, 0);
            } else if (intervalType.toLowerCase() == "minutes") {
                var minutes = Math.floor(Math.floor((sDate.getMinutes()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), minutes, 0, 0);
            } else if (intervalType.toLowerCase() == "months") {
                var month = Math.floor(Math.floor((sDate.getMonth()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), month, sDate.getDate(), 0, 0, 0);
            } else if (intervalType.toLowerCase() == "years") {
                var year = Math.floor(Math.floor(sDate.getFullYear() / intervalSize) * intervalSize);
                sResult = new Date(year, sDate.getMonth(), sDate.getDate(), 0, 0, 0);
            }
            return sResult;
        },
        _increaseDateTimeInterval: function (date, interval, intervalType) {
            var result = new Date(date);
            if (intervalType.toLowerCase() == "days") {
                result.setDate(date.getDate() + interval);
            } else if (intervalType.toLowerCase() == "hours") {
                result.setHours(date.getHours() + interval);
            } else if (intervalType.toLowerCase() == "milliseconds") {
                result.setMilliseconds(date.getMilliseconds() + interval);
            } else if (intervalType.toLowerCase() == "seconds") {
                result.setSeconds(date.getSeconds() + interval);
            } else if (intervalType.toLowerCase() == "minutes") {
                result.setMinutes(date.getMinutes() + interval);
            } else if (intervalType.toLowerCase() == "months") {
                result.setMonth(date.getMonth() + interval);
            } else if (intervalType.toLowerCase() == "years") {
                result.setYear(date.getFullYear() + interval);
            }

            return result;
        },

        _calculatePadding: function (axis) {

            var start = (typeof axis.range.min == "string" && !isNaN(Date.parse(axis.range.min))) ? Date.parse(axis.range.min) : (axis.range.min).getTime();
            var end = (typeof axis.range.max == "string" && !isNaN(Date.parse(axis.range.max))) ? Date.parse(axis.range.max) : (axis.range.max).getTime();
            var interval = ((this._increaseDateTimeInterval(new Date(start), axis.range.interval, axis.intervalType)).getTime()) - start;
            if (!this.chartObj.zoomed && !axis.setRange) {
                start = new Date(start);
                end = new Date(end);
                var intervalType = axis.intervalType.toLowerCase();
                if (axis.rangePadding.toLowerCase() == 'none') {
                    start = start.getTime();
                    end = end.getTime();
                } else if (axis.rangePadding.toLowerCase() == 'additional' || axis.rangePadding.toLowerCase() == 'round') {
                    switch (intervalType) {
                        case 'years':
                            var startYear = (start.getFullYear() / axis.range.interval) * axis.range.interval;
                            var endYear = end.getFullYear() + (start.getFullYear() - startYear);
                            if (startYear <= 0) {
                                startYear = 1;
                            }
                            if (endYear <= 0) {
                                endYear = 1;
                            }
                            if (axis.rangePadding.toLowerCase() == 'additional') {
                                start = (new Date(startYear - axis.range.interval, 1, 1, 0, 0, 0)).getTime();
                                end = (new Date(endYear + axis.range.interval, 1, 1, 0, 0, 0)).getTime();
                            } else {
                                start = (new Date(startYear, 1, 1, 0, 0, 0)).getTime();
                                end = (new Date(endYear, 1, 1, 0, 0, 0)).getTime();
                            }
                            break;
                        case 'months':
                            var month = (start.getMonth() / axis.range.interval) * axis.range.interval;
                            if (month <= 0) {
                                month = 1;
                            }
                            var endMonth = end.getMonth() + (start.getMonth() - month);
                            if (endMonth <= 0) {
                                endMonth = 1;
                            }
                            if (axis.rangePadding.toLowerCase() == 'round') {
                                start = (new Date(start.getFullYear(), month, 1, 0, 0, 0)).getTime();
                                end = (new Date(end.getFullYear(), endMonth, endMonth == 2 ? 28 : 30, 0, 0, 0)).getTime();
                            } else {
                                start = (new Date(start.getFullYear(), month + (-axis.range.interval), 1, 0, 0, 0)).getTime();
                                end = (new Date(end.getFullYear(), endMonth + (axis.range.interval), endMonth == 2 ? 28 : 30, 0, 0, 0)).getTime();
                            }
                            break;
                        case 'days':
                            var day = (start.getDate() / axis.range.interval) * axis.range.interval;
                            if (day <= 0) {
                                day = 1;
                            }
                            var endDay = end.getDate() + (start.getDate() - day);
                            if (endDay <= 0) {
                                endDay = 1;
                            }
                            if (axis.rangePadding.toLowerCase() == 'round') {
                                start = (new Date(start.getFullYear(), start.getMonth(), day, 0, 0, 0)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), endDay, 0, 0, 0)).getTime();
                            } else {
                                start = (new Date(start.getFullYear(), start.getMonth(), day + (-axis.range.interval), 0, 0, 0)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), endDay + (axis.range.interval), 0, 0, 0)).getTime();
                            }
                            break;
                        case 'hours':
                            var hour = (start.getHours() / axis.range.interval) * axis.range.interval;
                            var endHour = end.getHours() + (start.getHours() - hour);
                            if (axis.rangePadding.toLowerCase() == 'round') {
                                start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), hour, 0, 0)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), endHour, 0, 0)).getTime();
                            } else {
                                start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), hour + (-axis.range.interval), 0, 0)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), endHour + (axis.range.interval), 0, 0)).getTime();
                            }
                            break;
                        case 'minutes':
                            var minute = (start.getMinutes() / axis.range.interval) * axis.range.interval;
                            var endMinute = end.getMinutes() + (start.getMinutes() - minute);
                            if (axis.rangePadding.toLowerCase() == 'round') {
                                start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHour(), minute, 0)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHour(), endMinute, 0)).getTime();
                            } else {
                                start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), minute + (-axis.range.interval), 0)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), minute + (axis.range.interval), 0)).getTime();
                            }
                            break;
                        case 'seconds':
                            var second = (start.getSeconds() / axis.range.interval) * axis.range.interval;
                            var endSecond = end.getSeconds() + (start.getSeconds() - second);
                            if (axis.rangePadding.toLowerCase() == 'round') {
                                start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), second, 0)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHour(), end.getMinutes(), endSecond, 0)).getTime();
                            } else {
                                start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), second + (-axis.range.interval), 0)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), endSecond + (axis.range.interval), 0)).getTime();
                            }
                            break;
                        case 'milliseconds':
                            var milliSecond = (start.getMilliseconds() / axis.range.interval) * axis.range.interval;
                            var endMilliSecond = end.getMilliseconds() + (start.getMilliseconds() - milliSecond);
                            if (axis.rangePadding.toLowerCase() == 'round') {
                                start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), start.getSeconds(), milliSecond)).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), end.getSeconds(), endMilliSecond)).getTime();
                            } else {
                                start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), start.getSeconds(), milliSecond + (-axis.range.interval))).getTime();
                                end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), end.getSeconds(), endMilliSecond + (axis.range.interval))).getTime();
                            }
                            break;
                    }

                }

            }
            var data = _sideBySeriesPadding(this.chartObj, start, end, axis);
            start = data.startPadding;
            end = data.endPadding;
            axis.actualRange.interval = interval;
            axis.actualRange.min = start;
            axis.actualRange.max = end;
            axis._intervalType = axis.intervalType;
            axis._interval = axis.range.interval;
            axis.actualRange.delta = (axis.actualRange.max - axis.actualRange.min);
            axis.range.min = new Date(start);
            axis.range.max = new Date(end);
            this._calculateVisibleRange(axis);

        },
        _calculateVisibleRange: function (axis) {
            axis.visibleRange = $.extend(true, {}, axis.actualRange);
            axis._intervalType = axis.intervalType;
            if (axis.zoomFactor < 1 || axis.zoomPosition > 0) {

                var baseRange = axis.actualRange;
                if (axis.isInversed) {
                    var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                    var end = start - axis.zoomFactor * axis.actualRange.delta;
                }
                else {
                    var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                    var end = start + axis.zoomFactor * axis.actualRange.delta;
                }

                if (start < baseRange.min) {
                    end = end + (baseRange.min - start);
                    start = baseRange.min;
                }

                if (end > baseRange.max) {
                    start = start - (end - baseRange.max);
                    end = baseRange.max;
                }

                var startDate = new Date(start);
                var endDate = new Date(end);
                //Use below code to find min,max and interval for visible range
                var intervalX = this.calculateDateTimeNiceInterval(axis, startDate, endDate);
                axis.visibleRange.min = Math.min(startDate.getTime(), endDate.getTime());
                axis.visibleRange.max = Math.max(startDate.getTime(), endDate.getTime());
                var interval = ((this._increaseDateTimeInterval(startDate, intervalX.interval, intervalX.intervalType)).getTime()) - axis.visibleRange.min;
                axis._intervalType = intervalX.intervalType;
                axis.visibleRange.interval = interval;
                axis.visibleRange.delta = Math.abs(axis.visibleRange.max - axis.visibleRange.min);
                axis._interval = intervalX.interval;
            }

        },
        _calculateAxisLabels: function (axis) {
            var position;
            if (this.chartObj.zoomed || axis.zoomed || !axis.setRange)
                position = (this.AlignRangeStart(new Date(axis.visibleRange.min), axis._interval, axis._intervalType)).getTime();

            else
                position = axis.visibleRange.min;
            while (position <= axis.visibleRange.max) {
                if (position >= axis.visibleRange.min && position <= axis.visibleRange.max) {

                    axis.visibleLabels.push({ Value: position, Text: (Globalize.format(new Date(position), ((!(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat))) });
                }
                position = (this._increaseDateTimeInterval(new Date(position), axis._interval, axis._intervalType)).getTime();
                var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxesEventArgs.Data = { Label: axis.visibleLabels[axis.visibleLabels.length - 1], Axis: axis };
                this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs.Data);
                axis.visibleLabels[axis.visibleLabels.length - 1] = commonAxesEventArgs.Data.Label;
            }
            axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
        },
        calculateDateTimeNiceInterval: function (axis, startDate, endDate) {
            var oneDay = 24 * 60 * 60 * 1000;
            //var axisInterval ;
            var totalDays = (Math.abs((startDate.getTime() - endDate.getTime()) / (oneDay)));
            var interval = this.calculateNumericNiceInterval(totalDays / 365, axis);

            if (interval >= 1) {
                return { interval: interval, intervalType: "Years" };
            }

            interval = this.calculateNumericNiceInterval(totalDays / 30, axis);

            if (interval >= 1) {
                return { interval: interval, intervalType: "Months" };
            }

            interval = this.calculateNumericNiceInterval(totalDays, axis);

            if (interval >= 1) {


                return { interval: interval, intervalType: "Days" };
            }

            var totalHours = totalDays * 24;

            interval = this.calculateNumericNiceInterval(totalHours, axis);

            if (interval >= 1) {

                return { interval: interval, intervalType: "Hours" };
            }

            var totalMinutes = totalDays * 24 * 60;

            interval = this.calculateNumericNiceInterval(totalMinutes, axis);

            if (interval >= 1) {

                return { interval: interval, intervalType: "Minutes" };
            }

            var totalSeconds = totalDays * 24 * 60 * 60;

            interval = this.calculateNumericNiceInterval(totalSeconds, axis);

            if (interval >= 1) {

                return { interval: interval, intervalType: "Seconds" };
            }

            var totalMilliseconds = totalDays * 24 * 60 * 60 * 1000;

            interval = this.calculateNumericNiceInterval(totalMilliseconds, axis);

            if (interval >= 1) {

                return { interval: interval, intervalType: "Milliseconds" };
            }

        },
        calculateNumericNiceInterval: function (delta, axis) {
            var desiredIntervalsCount = this.GetActualDesiredIntervalsCount(axis, axis.length);
            var niceInterval = delta / desiredIntervalsCount;
            var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
            var intervalDivs = [10, 5, 2, 1];

            for (var i = 0; i < intervalDivs.length; i++) {
                var currentInterval = minInterval * intervalDivs[i];
                if (desiredIntervalsCount < (delta / currentInterval)) {
                    return niceInterval;
                }

                niceInterval = currentInterval;
            }

            return niceInterval;
        },
        GetActualDesiredIntervalsCount: function (axis, size) {
            if (ej.util.isNullOrUndefined(axis.desiredIntervals)) {
                var desiredIntervalsCount = (axis.orientation.toLowerCase() == "horizontal" ? 0.533 : 1) * axis.maximumLabels;
                desiredIntervalsCount = Math.max((size * (desiredIntervalsCount / 100)), 1);
                axis.desiredIntervals = desiredIntervalsCount;
                return desiredIntervalsCount;
            } else {
                return axis.desiredIntervals;
            }
        },
        _calculateRange: function (axis, seriesCollection) {
            if (!this.chartObj.zoomed) {
                var defaultDate = new Date();
                if (seriesCollection.length > 0 && seriesCollection[0].points.length > 0) {
                    var maxX, minY;
                    var maxY, minX;
                    var maxMinXValue = { maxX: null, minX: null };
                    var maxMinYValue = { maxY: null, minY: null };
                    for (var i = 0; i < seriesCollection.length; i++) {
                        if (seriesCollection[i].points.length > 0) {
                            if (seriesCollection[i].visibility.toLowerCase() === 'visible' && seriesCollection[i].points.length > 0) {
                                minX = maxX = seriesCollection[i].points[0].X;
                                maxY = minY = seriesCollection[i].points[0].YValues[0];
                                for (var j = 0; j < seriesCollection[i].points.length; j++) {
                                    if (minX > seriesCollection[i].points[j].X)
                                        minX = seriesCollection[i].points[j].X;
                                    if (maxX < seriesCollection[i].points[j].X)
                                        maxX = seriesCollection[i].points[j].X;
                                    if (minY > seriesCollection[i].points[j].YValues[0])
                                        minY = seriesCollection[i].points[j].YValues[0];
                                    if (maxY < seriesCollection[i].points[j].YValues[0])
                                        maxY = seriesCollection[i].points[j].YValues[0];
                                }
                                seriesCollection[i].minX = minX;
                                seriesCollection[i].maxX = maxX;
                                seriesCollection[i].minY = minY;
                                seriesCollection[i].maxY = maxY;
                            }
                        }

                        if ((maxMinXValue.minX = (maxMinXValue.minX == null && typeof seriesCollection[i].minX != "number") ? seriesCollection[i].minX : maxMinXValue.minX) > seriesCollection[i].minX) {
                            maxMinXValue.minX = (typeof seriesCollection[i].minX != "number") ? seriesCollection[i].minX : maxMinXValue.minX;
                        }
                        if ((maxMinXValue.maxX = (maxMinXValue.maxX == null && typeof seriesCollection[i].maxX != "number") ? seriesCollection[i].maxX : maxMinXValue.maxX) < seriesCollection[i].maxX) {
                            maxMinXValue.maxX = (typeof seriesCollection[i].maxX != "number") ? seriesCollection[i].maxX : maxMinXValue.maxX;
                        }
                        if (maxMinYValue.minY > seriesCollection[i].minY) {
                            maxMinYValue.minY = seriesCollection[i].minY;
                        }
                        if (maxMinYValue.maxY < seriesCollection[i].maxY) {
                            maxMinYValue.maxY = seriesCollection[i].maxY;
                        }

                    }
                    maxMinXValue.minX = (maxMinXValue.minX == null) ? defaultDate : maxMinXValue.minX;
                    maxMinXValue.maxX = (maxMinXValue.maxX == null) ? new Date(new Date().setMonth(defaultDate.getMonth() + 5)) : maxMinXValue.maxX;
                    if (maxMinXValue.minX == maxMinXValue.maxX) {
                        maxMinXValue.minX = new Date(Date.parse(maxMinXValue.minX) - 2592000000);
                        maxMinXValue.maxX = new Date(Date.parse(maxMinXValue.maxX) + 2592000000);
                    }
                    var intervalX = this.calculateDateTimeNiceInterval(axis, maxMinXValue.minX, maxMinXValue.maxX);


                    if ((!axis.setRange)) {
                        axis.range = { min: maxMinXValue.minX, max: maxMinXValue.maxX, interval: intervalX.interval };
                        axis.intervalType = intervalX.intervalType;

                    } else {
                        if (ej.util.isNullOrUndefined(axis.actual_Range)) {
                            if (ej.util.isNullOrUndefined(axis.range.interval))
                                axis.range.interval = intervalX.interval;
                            if (!axis.range.max)
                                axis.range.max = maxMinXValue.maxX;
                            if (!axis.range.min)
                                axis.range.min = maxMinXValue.minX;
                            if (!axis.intervalType)
                                axis.intervalType = intervalX.intervalType;
                            if (typeof axis.range.min == "string" && axis.range.min.indexOf("/Date(") != -1)
                                axis.range.min = new Date(parseInt(axis.range.min.substr(6)));
                            if (typeof axis.range.max == "string" && axis.range.max.indexOf("/Date(") != -1)
                                axis.range.max = new Date(parseInt(axis.range.max.substr(6)));
                            axis.actual_Range = $.extend(true, {}, axis.range);
                            axis.interval_Type = axis.intervalType;
                            axis.setRange = true;
                        }
                        axis.range = { min: axis.actual_Range.min, max: axis.actual_Range.max, interval: axis.actual_Range.interval };
                        axis.intervalType = axis.interval_Type;
                    }
                } else {
                    if (!axis.setRange) {
                        axis.range = { min: defaultDate, max: new Date(new Date().setMonth(defaultDate.getMonth() + 5)), interval: 1 };
                        axis.intervalType = 'Months';
                    }

                }
            }
        },

        _calculateRanges: function (chartObj, axis, seriesCollection) {

            this.chartObj = chartObj;

            this._calculateRange(axis, seriesCollection);

            this._calculatePadding(axis);

            this._calculateAxisLabels(axis);
        }
    };

    function ejExtendClass(parent, members) {
        var object = function () {
        };
        object.prototype = new parent();
        $.extend(object.prototype, members);
        return object;
    }

    //DateTime calculation
    var ejDateTimeValue = ejExtendClass(ej.EjAxisRenderer);
    ej.axisTypes.datetime = ejDateTimeValue;

    var ejDoubleValue = ejExtendClass(ej.EjAxisRenderer, {


        _calculateRange: function (chartObj, axis, seriesCollection) {
            if (!chartObj.zoomed) {
                this._seriesMinMax(chartObj, axis, seriesCollection, "double");
            }
        },

        _getDoubleAutoRange: function (axis, min, max, delta) {
            var interval = this.calculateNumericNiceInterval(delta, axis);
            if (!axis.setRange) {
                return axis.range = { min: min, max: max, interval: interval, Delta: delta };

            } else {
                if (ej.util.isNullOrUndefined(axis.range.interval))
                    axis.range.interval = interval;
                if (ej.util.isNullOrUndefined(axis.range.max))
                    axis.range.max = max;
                if (ej.util.isNullOrUndefined(axis.range.min))
                    axis.range.min = min;
                if (ej.util.isNullOrUndefined(axis.actual_Range)) {
                    axis.actual_Range = $.extend(true, {}, axis.range);
                    axis.setRange = true;
                }
                return axis.range = { min: axis.actual_Range.min, max: axis.actual_Range.max, interval: axis.actual_Range.interval, delta: (axis.actual_Range.max - axis.actual_Range.min) };
            }
        },
        _calculatePadding: function (chartObj, axis, baseRange) {
            var start = baseRange.min;
            var end = baseRange.max;
            var rangePadding = axis.rangePadding.toLowerCase();
            var interval = baseRange.interval;
            if ((!axis.setRange) && (!chartObj.zoomed)) {
                var data = _sideBySeriesPadding(chartObj, start, end, axis);
                start = data.startPadding;
                end = data.endPadding;
                if (rangePadding == 'normal') {
                    var minimum = 0, remaining;
                    if (start < 0) {
                        start = 0;
                        minimum = baseRange.min + (baseRange.min / 20);

                        remaining = interval + (minimum % interval);

                        if ((0.365 * interval) >= remaining) {
                            minimum -= interval;
                        }

                        if (minimum % interval < 0) {
                            minimum = (minimum - interval) - (minimum % interval);
                        }
                    } else {
                        minimum = start < ((5.0 / 6.0) * end)
                            ? 0
                            : (start - (end - start) / 2);
                        if (minimum % interval > 0) {
                            minimum -= (minimum % interval);
                        }
                    }
                    var maximum = (end + (end - start) / 20);

                    remaining = interval - (maximum % interval);

                    if ((0.365 * interval) >= remaining) {
                        maximum += interval;
                    }

                    if (maximum % interval > 0) {
                        maximum = (maximum + interval) - (maximum % interval);
                    }
                    if (minimum <= 0) {
                        interval = this.calculateNumericNiceInterval(maximum - minimum, axis);
                        maximum = Math.ceil(maximum / interval) * interval;
                    }
                    start = (!(axis.roundingPlaces)) ? parseFloat(minimum.toFixed((ej.EjSvgRender.utils._decimalPlaces(interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(interval)))) : parseFloat(minimum.toFixed(axis.roundingPlaces));
                    end = (!(axis.roundingPlaces)) ? parseFloat(maximum.toFixed((ej.EjSvgRender.utils._decimalPlaces(interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(interval)))) : parseFloat(maximum.toFixed(axis.roundingPlaces));

                } else if (rangePadding == 'additional' || rangePadding == 'round') {
                    start = Math.floor((start / interval) * interval);
                    end = Math.ceil((end / interval) * interval);
                    if (rangePadding == 'additional') {
                        start -= interval;
                        end += interval;
                    }
                }


            }
            axis.actualRange.min = start;
            axis.actualRange.max = end;
            axis.actualRange.interval = interval;
            axis.range.min = start;
            axis.range.max = end;
            axis.range.interval = interval;
            axis.actualRange.delta = end - start;
            this._calculateVisibleRange(axis);
        },
        _calculateVisibleRange: function (axis) {
            axis.visibleRange = $.extend(true, {}, axis.actualRange);
            if (axis.zoomFactor < 1 || axis.zoomPosition > 0) {
                var baseRange = axis.actualRange;

                if (axis.isInversed) {
                    var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                    var end = start - axis.zoomFactor * axis.actualRange.delta;
                }
                else {
                    var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                    var end = start + axis.zoomFactor * axis.actualRange.delta;
                }

                if (start < baseRange.min) {
                    end = end + (baseRange.min - start);
                    start = baseRange.min;
                }

                if (end > baseRange.max) {
                    start = start - (end - baseRange.max);
                    end = baseRange.max;
                }
                var delta = Math.abs(end - start);
                axis.visibleRange.interval = (this.calculateNumericNiceInterval(delta, axis));
                var factor = (ej.util.isNullOrUndefined(axis.roundingPlaces)) ? (ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval)) : axis.roundingPlaces;
                axis.visibleRange.interval = parseFloat(axis.visibleRange.interval.toFixed(factor));
                axis.visibleRange.min = Math.min(start, end);
                axis.visibleRange.max = Math.max(start, end);
                axis.visibleRange.delta = delta;
                axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
            }

        },
        _calculateAxisLabels: function (chartObj, currentAxis) {

            var tempInterval;
            if (chartObj.zoomed || currentAxis.zoomed || currentAxis.padding)
                tempInterval = currentAxis.visibleRange.min - (currentAxis.visibleRange.min % currentAxis.visibleRange.interval);
            else
                tempInterval = currentAxis.visibleRange.min;
            for (; tempInterval <= currentAxis.visibleRange.max; tempInterval += currentAxis.visibleRange.interval) {
                tempInterval = (!(currentAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(currentAxis.roundingPlaces));

                if (ej.EjSvgRender.utils._inside(tempInterval, currentAxis.visibleRange)) {


                    var customFormat = (!(currentAxis.labelFormat)) ? null : currentAxis.labelFormat.match('{value}');
                    var labelText = (!(currentAxis.labelFormat)) ? tempInterval : (customFormat != null) ? currentAxis.labelFormat.replace('{value}', tempInterval) : (Globalize.format(tempInterval, currentAxis.labelFormat));
                    currentAxis.visibleLabels.push({ Value: tempInterval, Text: labelText });
                    // customize label by event
                    var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonAxesEventArgs.Data = { Label: currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1], Axis: currentAxis };
                    chartObj._trigger("axesLabelRendering", commonAxesEventArgs.Data);
                    currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1] = commonAxesEventArgs.Data.Label;
                }
            }

            currentAxis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(currentAxis, this.chartObj);

        },

        _calculateRanges: function (chartObj, axis, seriesCollection) {
            this.chartObj = chartObj;

            this._calculateRange(chartObj, axis, seriesCollection);


            this._calculatePadding(chartObj, axis, axis.range);


            this._calculateAxisLabels(chartObj, axis);

        }
    });
    ej.axisTypes.double = ejDoubleValue;
    //log axis
    var ejLogValue = ejExtendClass(ej.EjAxisRenderer, {

        // Get value for log min and max
        _getRoundValue: function (x, div, up) {
            return (up ? Math.ceil(x / div) : Math.floor(x / div)) * div;
        },

        // Caculate log nice interval
        calculateLogNiceInterval: function (delta, axis) {
            var desiredIntervalsCount = this.GetActualDesiredIntervalsCount(axis, axis.length);
            var niceInterval = delta;
            var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
            var intervalDivs = [10, 5, 2, 1];

            for (var i = 0; i < intervalDivs.length; i++) {
                var currentInterval = minInterval * intervalDivs[i];
                if (desiredIntervalsCount < (delta / currentInterval)) {
                    return niceInterval;
                }

                niceInterval = currentInterval;
            }

            return niceInterval;
        },

        // Get log value for auto range
        _getLogAutoRange: function (axis, min, max, delta) {

            var logStart = ej.EjSvgRender.utils._logBase(min, axis.logBase);
            logStart = $.isNumeric(logStart) ? logStart : min;
            var logEnd = ej.EjSvgRender.utils._logBase(max, axis.logBase);
            logEnd = $.isNumeric(logEnd) ? logEnd : max;

            var mulS = this._getRoundValue(logStart, 1, false);
            var mulE = this._getRoundValue(logEnd, 1, true);
            delta = mulE - mulS;

            var interval = this.calculateLogNiceInterval(delta, axis);
            if (!axis.setRange) {
                return axis.range = { min: mulS, max: mulE, interval: interval, Delta: delta };

            } else {
                if (ej.util.isNullOrUndefined(axis.range.interval))
                    axis.range.interval = interval;
                if (ej.util.isNullOrUndefined(axis.range.max))
                    axis.range.max = mulE;
                if (ej.util.isNullOrUndefined(axis.range.min))
                    axis.range.min = mulS;
                if (ej.util.isNullOrUndefined(axis.actual_Range)) {
                    axis.actual_Range = $.extend(true, {}, axis.range);
                    axis.setRange = true;
                }
                return axis.range = {
                    min: axis.actual_Range.min,
                    max: axis.actual_Range.max,
                    interval: axis.actual_Range.interval,
                    delta: (axis.actual_Range.max - axis.actual_Range.min)
                };
            }
        },

        //Calcualte auto log range from series
        _calculateLogRange: function (chartObj, axis, seriesCollection) {
            if (!chartObj.zoomed) {
                this._seriesMinMax(chartObj, axis, seriesCollection, "logarithmic");
            }
        },

        //Assign base range values to actual range
        _calculateLogPadding: function (chartObj, axis, baseRange) {
            // No padding support for log axis
            var start = baseRange.min;
            var end = baseRange.max;
            var interval = baseRange.interval;
            axis.actualRange.min = start;
            axis.actualRange.max = end;
            axis.actualRange.interval = interval;
            axis.range.min = start;
            axis.range.max = end;
            axis.range.interval = interval;
            axis.actualRange.delta = end - start;
            this._calculateVisibleRange(axis);
        },

        // Calculate visible range for zooming
        _calculateVisibleRange: function (axis) {
            axis.visibleRange = $.extend(true, {}, axis.actualRange);
            if (axis.zoomFactor < 1 || axis.zoomPosition > 0) {
                var baseRange = axis.actualRange;
                if (axis.isInversed) {
                    var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                    var end = start - axis.zoomFactor * axis.actualRange.delta;
                }
                else {
                    var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                    var end = start + axis.zoomFactor * axis.actualRange.delta;
                }

                if (start < baseRange.min) {
                    end = end + (baseRange.min - start);
                    start = baseRange.min;
                }

                if (end > baseRange.max) {
                    start = start - (end - baseRange.max);
                    end = baseRange.max;
                }
                var delta = Math.abs(end - start);
                axis.visibleRange.interval = (this.calculateLogNiceInterval(delta, axis));
                var factor = (ej.util.isNullOrUndefined(axis.roundingPlaces)) ?
                    (ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval) == 0 ? 1
                    : ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval)) : axis.roundingPlaces;
                axis.visibleRange.interval = parseFloat(axis.visibleRange.interval.toFixed(factor));
                axis.visibleRange.min = Math.min(start, end);
                axis.visibleRange.max = Math.max(start, end);
                axis.visibleRange.delta = delta;
                axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
            }

        },

        //Method implementation for Generate Labels in ChartAxis
        _calculateAxisLabels: function (chartObj, currentAxis) {

            var tempInterval;
            if (chartObj.zoomed || currentAxis.zoomed)
                tempInterval = currentAxis.visibleRange.min - (currentAxis.visibleRange.min % currentAxis.visibleRange.interval);
            else
                tempInterval = currentAxis.visibleRange.min;
            for (; tempInterval <= currentAxis.visibleRange.max; tempInterval += currentAxis.visibleRange.interval) {


                if (ej.EjSvgRender.utils._inside(tempInterval, currentAxis.visibleRange)) {
                    var customFormat = (!(currentAxis.labelFormat)) ? null : currentAxis.labelFormat.match('{value}');
                    var tempIntervaltext = Math.pow(currentAxis.logBase, tempInterval);
                    tempIntervaltext = (!(currentAxis.roundingPlaces)) ?
                        parseFloat(tempIntervaltext.toFixed((ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval) == 0 ? 1
                            : ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval))))
                            : parseFloat(tempIntervaltext.toFixed(currentAxis.roundingPlaces));
                    var labelText = (!(currentAxis.labelFormat)) ? tempIntervaltext : (customFormat != null) ?
                             currentAxis.labelFormat.replace('{value}', tempIntervaltext) :
                              (Globalize.format(tempIntervaltext, currentAxis.labelFormat));
                    currentAxis.visibleLabels.push({ Value: tempInterval, Text: labelText });
                    // customize label by event
                    var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonAxesEventArgs.Data = { Label: currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1], Axis: currentAxis };
                    chartObj._trigger("axesLabelRendering", commonAxesEventArgs.Data);
                    currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1] = commonAxesEventArgs.Data.Label;
                }
            }

            currentAxis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(currentAxis, this.chartObj);

        },


        //Category axis calculation
        _calculateRanges: function (chartObj, axis, seriesCollection) {
            this.chartObj = chartObj;
            this._calculateLogRange(chartObj, axis, seriesCollection);
            this._calculateLogPadding(chartObj, axis, axis.range);
            this._calculateAxisLabels(chartObj, axis);
        }


    });
    ej.axisTypes.logarithmic = ejLogValue;

    //Category Axes calculation
    var ejCategoryValue = ejExtendClass(ej.EjAxisRenderer, {
        //axis range calculation and assign indexed value to x value of points
        _calculateCategoryRange: function (axis, seriesCollection) {
            if (!this.chartObj.zoomed) {
                var maxXValue = { maxX: null, minX: 0 };
                if (seriesCollection.length > 0 && seriesCollection[0].points.length > 0) {
                    for (var l = 0; l < seriesCollection.length; l++) {
                        seriesCollection[l]._pointCollection = [];
                        for (var k = 0; k < seriesCollection[l].points.length; k++) {
                            seriesCollection[l]._pointCollection.push(seriesCollection[l].points[k].x);
                            seriesCollection[l].points[k].X = k;
                            seriesCollection[l].minX = 0;
                            seriesCollection[l].maxX = (seriesCollection[l]._pointCollection.length - 1);
                        }
                        // To find maximum x value of axis from series collection
                        if (l == 0) {
                            maxXValue.maxX = (seriesCollection[l]._pointCollection.length - 1);
                            axis.labels = seriesCollection[l]._pointCollection;
                            axis._categoryValueType = seriesCollection[l]._xAxisValueType;
                        }
                        if (maxXValue.maxX < (seriesCollection[l].maxX)) {
                            maxXValue.maxX = seriesCollection[l].maxX;
                            axis.labels = [];
                            axis.labels = seriesCollection[l]._pointCollection;
                        }
                    }
                    var deltaX = maxXValue.maxX - maxXValue.minX;
                    var intervalX = this._calculateActualInterval(deltaX, axis);
                    axis.range = { min: maxXValue.minX, max: maxXValue.maxX, interval: intervalX, Delta: deltaX };
                }
                else {
                    if (axis.labels.length > 0)
                        axis.range = { min: 0, max: axis.labels.length - 1, interval: 1, Delta: axis.labels.length - 1 };
                }
            }
        },

        //Calculates actual interval
        _calculateActualInterval: function (delta, axis) {
            if (axis.categoryInterval == null)
                return Math.max(1, Math.floor(delta / this.GetActualDesiredIntervalsCount(axis, axis.length)));
            else
                return axis.categoryInterval;
        },

        //Apply padding based on labelPlacement
        _applyRangePadding: function (axis) {
            if (!this.chartObj.zoomed) {
                var ticks = ((axis.labelPlacement === null || axis.labelPlacement === undefined)) ? -0.5 : (axis.labelPlacement.toLowerCase() == "betweenticks") ? (-0.5) : 0;
                if (ticks < 0) {
                    axis.range.min = axis.range.min + ticks;
                    axis.range.max = axis.range.max - ticks;
                    axis.range.delta = axis.range.max - axis.range.min;
                    axis.actualRange = axis.range;
                } else {
                    var data = _sideBySeriesPadding(this.chartObj, axis.range.min, axis.range.max, axis);
                    axis.range.min = data.startPadding;
                    axis.range.max = data.endPadding;
                    axis.range.delta = axis.range.max - axis.range.min;
                    axis.actualRange = axis.range;
                }
            }
            this._calculateVisibleRange(axis);
        },

        //Calculates the visible range  
        _calculateVisibleRange: function (axis) {
            axis.visibleRange = $.extend(true, {}, axis.actualRange);
            if (axis.zoomFactor < 1 || axis.zoomPosition > 0) {
                var baseRange = axis.actualRange;
                if (axis.isInversed) {
                    var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                    var end = start - axis.zoomFactor * axis.actualRange.delta;
                }
                else {
                    var start = axis.actualRange.min + (axis.zoomPosition * axis.actualRange.delta);
                    var end = start + axis.zoomFactor * axis.actualRange.delta;
                }

                if (start < baseRange.min) {
                    end = end + (baseRange.min - start);
                    start = baseRange.min;
                }

                if (end > baseRange.max) {
                    start = start - (end - baseRange.max);
                    end = baseRange.max;
                }
                var delta = Math.abs(end - start);
                axis.visibleRange.interval = this._calculateActualInterval(delta, axis);
                axis.visibleRange.min = Math.min(start, end);
                axis.visibleRange.max = Math.max(start, end);
                axis.visibleRange.delta = delta;
                axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
            }

        },

        //Generate category label for axis
        _calculateAxisLabels: function (axis) {
            var interval = axis.visibleRange.interval;
            var position = axis.visibleRange.min - (axis.visibleRange.min % axis.range.interval);
            for (; position <= axis.visibleRange.max; position += interval) {
                if (ej.EjSvgRender.utils._inside(position, axis.visibleRange)) {
                    var pos = Math.round(position);
                    axis.visibleLabels.push({ Value: pos, Text: ej.EjSvgRender.utils._getLabelContent(pos, axis) });

                    // customize label by event
                    var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonAxesEventArgs.Data = { Label: axis.visibleLabels[axis.visibleLabels.length - 1], Axis: axis };
                    this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs.Data);
                    axis.visibleLabels[axis.visibleLabels.length - 1] = commonAxesEventArgs.Data.Label;
                }
            }
            axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
        },

        //Category axis calculation
        _calculateRanges: function (chartObj, axis, seriesCollection) {
            this.chartObj = chartObj;

            this._calculateCategoryRange(axis, seriesCollection);
            this._applyRangePadding(axis);

            this._calculateAxisLabels(axis);
        }
    });
    ej.axisTypes.category = ejCategoryValue;

    ej.EjAxisRenderer.prototype = {
        _drawAxes: function (axisIndex, axis) {

            if (axis.orientation.toLowerCase() == "horizontal") {
                this._drawXAxisGridLine(axisIndex, axis);
                if (axis.visible) {
                    this._drawXAxisLabels(axisIndex, axis);

                    this._drawXTitle(axisIndex, axis);
                }
            }

            if (axis.orientation.toLowerCase() == "vertical") {
                this._drawYAxisGridLine(axisIndex, axis);
                if (axis.visible) {
                    this._drawYAxisLabels(axisIndex, axis);

                    this._drawYTitle(axisIndex, axis);
                }
            }

        },
        _getSharpPath: function (width) {
            var value = ((width % 2) == 0) ? 0 : 0.5;
            return value;
        },
        _drawXAxisGridLine: function (axisIndex, xAxis) {
            var xMajorGridDir, xMajorTicksDir, xMinorTicksDir, xMinorGridDir;
            var sbXMinorTicks = ej.EjSvgRender.utils._getStringBuilder();
            var sbXMinorGrid = ej.EjSvgRender.utils._getStringBuilder();
            var sbXMajorTicks = ej.EjSvgRender.utils._getStringBuilder();
            var sbXMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
            var minorPointX;

            var x1 = Math.floor(xAxis.x);
            var x2 = Math.floor(xAxis.x + xAxis.width);
            var y1 = Math.floor(xAxis.y);
            var y2 = Math.floor(xAxis.y);

            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisGrid' + '_' + axisIndex });

            if (xAxis.axisLine.visible) {
                var offset = xAxis.axisLine.offset;
                var val = this._getSharpPath(xAxis.axisLine.width);
                var optionsLine = {
                    'id': this.svgObject.id + '_XAxisLine',
                    x1: (x1 - xAxis.plotOffset) + val + offset,
                    y1: y1 + val,
                    x2: (x2 + xAxis.plotOffset) - offset + val,
                    y2: y2 + val,
                    'stroke-dasharray': xAxis.axisLine.dashArray,
                    'stroke-width': xAxis.axisLine.width,
                    'stroke': xAxis.axisLine.color,
                    'opacity': (this.vmlRendering) ? 0.1 : xAxis.axisLine.opacity
                };

                this.svgRenderer.drawLine(optionsLine, gEle);
            }

            xAxis.labelPlacement = (!(xAxis.labelPlacement)) ? ej.Chart.labelPlacement.BetweenTicks : xAxis.labelPlacement;
            var ticksbwtLabel = ((xAxis.labels.length > 1) && (xAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
            var collectionLength = xAxis.visibleLabels.length;
            //For between ticks and gridlines
            if ((xAxis._valueType.toLowerCase() == ej.Chart.valueType.Category) && ticksbwtLabel < 0) {
                collectionLength = xAxis.labels.length + 1;
            }
            for (var i = 0; i < collectionLength; i++) {

                var tempInterval = ((xAxis.labels.length > 1) && (ticksbwtLabel < 0)) ? i + ticksbwtLabel : xAxis.visibleLabels[i].Value;

                tempInterval = (!(xAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(xAxis.roundingPlaces));

                if (ej.EjSvgRender.utils._inside(tempInterval, xAxis.visibleRange)) {

                    if ((xAxis.hidePartialLabels) && ((i == 0) || (i == (collectionLength - 1))))
                        continue;

                    //Calculate Minor Ticks and Grid lines
                    if (xAxis.minorGridLines.visible || xAxis.minorTickLines.visible) {
                        var ticksVal = this._getSharpPath(xAxis.minorTickLines.width);
                        var linesVal = this._getSharpPath(xAxis.minorGridLines.width);
                        var yTickPosition;
                        if (xAxis._valueType == "logarithmic") {
                            minorTicks = tempInterval;
                            var logmax = xAxis.visibleRange.max;
                            var logmin = xAxis.visibleRange.min;

                            var logtickstart = Math.pow(xAxis.logBase, minorTicks - xAxis.visibleRange.interval);
                            var logtickend = Math.pow(xAxis.logBase, minorTicks);
                            var logtickInterval = (logtickend - logtickstart) / (xAxis.minorTicksPerInterval + 1);
                            var logtickPos = logtickstart + logtickInterval;
                            minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                            for (var j = 0; j < xAxis.minorTicksPerInterval; j++) {


                                if (minorTicks < logmax && minorTicks > logmin) {

                                    minorPointX = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (xAxis.width));

                                    //Calculate Minor Ticks
                                    if ((xAxis.minorTickLines) && xAxis.minorTickLines.visible) {
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 - xAxis.minorTickLines.size) : Math.floor(xAxis.minorTickLines.size + y1);
                                        var xTickPosition = Math.floor(minorPointX + x1);
                                        sbXMinorTicks.append("M" + " " + (xTickPosition + ticksVal) + " " + (yTickPosition + ticksVal) + " " + "L" + " " + (xTickPosition + ticksVal) + " " + (y1 + ticksVal) + " ");
                                    }

                                    //Calculate Minor Gridlines
                                    if ((xAxis.minorGridLines) && xAxis.minorGridLines.visible) {
                                        var tickPosition = Math.floor(minorPointX + x1);
                                        sbXMinorGrid.append("M" + " " + (tickPosition + linesVal) + " " + (y1 + linesVal) + " " + "L" + " " + (tickPosition + linesVal) + " " + (Math.floor(this.model.m_AreaBounds.Y) + linesVal) + " ");
                                    }

                                }
                                logtickPos += logtickInterval;
                                minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                            }
                        } else {
                            for (var j = 0, minorTicks = tempInterval ; j < xAxis.minorTicksPerInterval; j++) {
                                minorTicks += xAxis.visibleRange.interval / (xAxis.minorTicksPerInterval + 1);
                                if (minorTicks < xAxis.visibleRange.max && minorTicks > xAxis.visibleRange.min) {
                                    minorPointX = Math.ceil(((minorTicks - xAxis.visibleRange.min) / (xAxis.visibleRange.max - xAxis.visibleRange.min)) * (xAxis.width));

                                    //Calculate Minor Ticks
                                    if ((xAxis.minorTickLines) && xAxis.minorTickLines.visible) {
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 - xAxis.minorTickLines.size) : Math.floor(xAxis.minorTickLines.size + y1);
                                        var points = Math.floor(minorPointX + x1);
                                        sbXMinorTicks.append("M" + " " + (points + ticksVal) + " " + (yTickPosition + ticksVal) + " " + "L" + " " + (points + ticksVal) + " " + (y1 + ticksVal) + " ");
                                    }

                                    //Calculate Minor Gridlines
                                    if ((xAxis.minorGridLines) && xAxis.minorGridLines.visible)
                                        sbXMinorGrid.append("M" + " " + (Math.floor(minorPointX + x1) + linesVal) + " " + (y1 + linesVal) + " " + "L" + " " + (Math.floor(minorPointX + x1) + linesVal) + " " + (Math.floor(this.model.m_AreaBounds.Y) + linesVal) + " ");
                                }
                            }
                        }
                    }

                    if (xAxis.majorTickLines.visible || xAxis.majorGridLines.visible) {
                        var mtVal = this._getSharpPath(xAxis.majorTickLines.width);
                        var mlVal = this._getSharpPath(xAxis.majorGridLines.width);
                        //Calculate Major Ticks and Grid lines
                        var pointX = this.pointX = Math.ceil(ej.EjSvgRender.utils._getPointXY(tempInterval, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width));
                        //Calculate Major Ticks
                        if (xAxis.majorTickLines.visible) {
                            var lineStart = xAxis.majorTickLines.size + xAxis.axisLine.width / 2;
                            var yPosition = (xAxis.opposedPosition) ? y1 - lineStart : lineStart + y1;
                            sbXMajorTicks.append("M" + " " + (Math.floor(pointX + x1) + mtVal) + " " + (Math.floor(yPosition) + mtVal) + " " + "L" + " " + (Math.floor(pointX + x1) + mtVal) + " " + (Math.floor(y1 + ((xAxis.opposedPosition) ? (-xAxis.axisLine.width / 2) : (xAxis.axisLine.width / 2))) + mtVal) + " ");
                        }
                        var mX1 = (Math.floor(pointX + x1));
                        if (i == 0) {
                            if (mX1 == Math.floor(this.model.m_AreaBounds.X))
                                continue;
                        }
                        if ((i == (collectionLength - 1)) && (this.model.chartArea.border.color != "transparent" && this.model.chartArea.border.width > 0)) {
                            if (mX1 == Math.floor(this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width))
                                continue;

                        }

                        //Calculate Major Gridlines
                        if (xAxis.majorGridLines.visible)
                            sbXMajorGrid.append("M" + " " + (mX1 + mlVal) + " " + (Math.floor(this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + mlVal) + " " + "L" + " " + (mX1 + mlVal) + " " + (Math.floor(this.model.m_AreaBounds.Y) + mlVal) + " ");
                    }

                }

            }
            //Drawing Minor Ticks
            if (xAxis.minorTickLines.visible && xAxis.visible) {
                xMinorTicksDir = sbXMinorTicks.toString();
                var optionsMinorTicks = {
                    'id': this.svgObject.id + '_XAxisMinorTicks',
                    'fill': 'none',
                    'stroke-width': xAxis.minorTickLines.width,
                    'stroke': xAxis.minorTickLines.color,
                    'd': xMinorTicksDir
                };

                this.svgRenderer.drawPath(optionsMinorTicks, gEle);

            }
            if (xAxis.minorGridLines.visible) {
                xMinorGridDir = sbXMinorGrid.toString();
                //Drawing Minor Grid Lines
                var optionsMinorGrid = {
                    'id': this.svgObject.id + '_XAxisMinorGridLines',
                    'fill': 'none',
                    'stroke-width': xAxis.minorGridLines.width,
                    'stroke': xAxis.minorGridLines.color,
                    'stroke-dasharray': xAxis.minorGridLines.dashArray,
                    'd': xMinorGridDir
                };
                this.svgRenderer.drawPath(optionsMinorGrid, gEle);


            }

            //Drawing Major Ticks

            if (xAxis.majorTickLines.visible && xAxis.visible) {
                xMajorTicksDir = sbXMajorTicks.toString();
                var optionsMajorTicks = {
                    'id': this.svgObject.id + '_XAxisMajorTicks',
                    'fill': 'none',
                    'stroke-width': xAxis.majorTickLines.width,
                    'stroke': xAxis.majorTickLines.color,
                    'd': xMajorTicksDir
                };
                this.svgRenderer.drawPath(optionsMajorTicks, gEle);
            }

            xMajorGridDir = sbXMajorGrid.toString();
            if (xAxis.majorGridLines.visible) {
                var options = {
                    'id': this.svgObject.id + '_XAxisMajorGridLines',
                    'fill': 'none',
                    'stroke-width': xAxis.majorGridLines.width,
                    'stroke': xAxis.majorGridLines.color,
                    'opacity': xAxis.majorGridLines.opacity,
                    'stroke-dasharray': xAxis.majorGridLines.dashArray,
                    'd': xMajorGridDir
                };

                //Drawing Major Grid Lines
                this.svgRenderer.drawPath(options, gEle);

            }
            $(gEle).appendTo(this.gXaxisEle);
        },

        _drawYAxisGridLine: function (axisIndex, yAxis) {

            var yMajorGridDir = "", yMajorTicksDir = "", yMinorTicksDir = "", yMinorGridDir = "";
            var pointY = 0;
            var x1 = Math.floor(yAxis.x);
            var y1 = Math.floor(yAxis.y + yAxis.height);
            var x2 = Math.floor(yAxis.x);
            var y2 = Math.floor(yAxis.y);

            var sbYMinorTicks = ej.EjSvgRender.utils._getStringBuilder();
            var sbYMinorGrid = ej.EjSvgRender.utils._getStringBuilder();
            var sbYMajorTicks = ej.EjSvgRender.utils._getStringBuilder();
            var sbYMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisGrid' + '_' + axisIndex });

            if (yAxis.axisLine.visible) {
                var offset = yAxis.axisLine.offset;
                var val = this._getSharpPath(yAxis.axisLine.width);

                var optionsLine = {
                    'id': this.svgObject.id + '_YAxisLine',
                    x1: x1 + val,
                    y1: (y2 - yAxis.plotOffset) + offset + val,
                    x2: x2 + val,
                    y2: (y1 + yAxis.plotOffset) - offset + val,
                    'stroke-width': yAxis.axisLine.width,
                    'stroke-dasharray': yAxis.axisLine.dashArray,
                    'stroke': yAxis.axisLine.color,
                    'opacity': (this.vmlRendering) ? 0.1 : yAxis.axisLine.opacity
                };
                this.svgRenderer.drawLine(optionsLine, gEle);
            }

            if (yAxis.axisBottomLine) {
                if (yAxis.axisBottomLine.visible) {
                    var optionsBottomLine = {
                        'id': this.svgObject.id + '_YAxisBottomLine',
                        x1: this.model.m_AreaBounds.X,
                        y1: y1,
                        x2: this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width,
                        y2: y1,
                        'stroke-width': yAxis.axisBottomLine.width,
                        'stroke': yAxis.axisBottomLine.color
                    };
                    this.svgRenderer.drawLine(optionsBottomLine, gEle);
                }
            }

            for (var i = 0; i < yAxis.visibleLabels.length; i++) {
                var tempInterval = yAxis.visibleLabels[i].Value;
                tempInterval = (!(yAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(yAxis.roundingPlaces));

                if (ej.EjSvgRender.utils._inside(tempInterval, yAxis.visibleRange)) {

                    if ((yAxis.hidePartialLabels) && ((i == 0) || (i == (yAxis.visibleLabels.length - 1))))
                        continue;

                    // Calculate Minor Ticks and Grid lines      
                    if (yAxis.minorGridLines.visible || yAxis.minorTickLines.visible) {
                        var minorPointY;
                        var tickVal = this._getSharpPath(yAxis.minorTickLines.width);
                        var lineVal = this._getSharpPath(yAxis.minorGridLines.width);
                        if (yAxis._valueType == "logarithmic") {
                            minorTicks = tempInterval;
                            var logmax = yAxis.visibleRange.max;
                            var logmin = yAxis.visibleRange.min;

                            var logtickstart = Math.pow(yAxis.logBase, minorTicks - yAxis.visibleRange.interval);
                            var logtickend = Math.pow(yAxis.logBase, minorTicks);
                            var logtickInterval = (logtickend - logtickstart) / (yAxis.minorTicksPerInterval + 1);
                            var logtickPos = logtickstart + logtickInterval;
                            minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                            for (var j = 0; j < yAxis.minorTicksPerInterval; j++) {


                                if (minorTicks < logmax && minorTicks > logmin) {

                                    minorPointY = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (yAxis.height));

                                    //Calculate Minor Ticks
                                    if ((yAxis.minorTickLines) && yAxis.minorTickLines.visible) {
                                        ytickPosition = (yAxis.opposedPosition) ? yAxis.minorTickLines.size : -yAxis.minorTickLines.size;
                                        sbYMinorTicks.append("M" + " " + (Math.floor(ytickPosition + x1) + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " " + "L" + " " + (x1 + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " ");
                                    }

                                    //Calculate Minor Gridlines
                                    if (yAxis.minorGridLines.visible)
                                        sbYMinorGrid.append("M" + " " + (Math.floor(this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " " + "L" + " " + (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " ");

                                }
                                logtickPos += logtickInterval;
                                minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                            }
                        } else {
                            for (var j = 0, minorTicks = tempInterval; j < yAxis.minorTicksPerInterval; j++) {

                                minorTicks += yAxis.visibleRange.interval / (yAxis.minorTicksPerInterval + 1);
                                if (minorTicks < yAxis.visibleRange.max && minorTicks > yAxis.visibleRange.min) {
                                    minorPointY = ((minorTicks - yAxis.visibleRange.min) / (yAxis.visibleRange.delta)) * (yAxis.height);
                                    // Calculate Minor Ticks   
                                    if (yAxis.minorTickLines.visible) {
                                        var ytickPosition = (yAxis.opposedPosition) ? yAxis.minorTickLines.size : -yAxis.minorTickLines.size;
                                        sbYMinorTicks.append("M" + " " + (Math.floor(ytickPosition + x1) + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " " + "L" + " " + (x1 + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " ");
                                    }
                                    // Calculate Minor Grid lines   
                                    if (yAxis.minorGridLines.visible)
                                        sbYMinorGrid.append("M" + " " + (Math.floor(this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " " + "L" + " " + (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " ");
                                }
                            }
                        }
                    }
                    // Calculate Major Ticks and Grid lines
                    if (yAxis.majorTickLines.visible || yAxis.majorGridLines.visible) {
                        var mtVal = this._getSharpPath(yAxis.minorTickLines.width);
                        var mlVal = this._getSharpPath(yAxis.majorGridLines.width);
                        // Calculate Major Ticks
                        if (yAxis.majorTickLines.visible) {
                            pointY = ej.EjSvgRender.utils._getPointXY(tempInterval, yAxis.visibleRange, yAxis.isInversed) * (yAxis.height);
                            var xPosition = (yAxis.opposedPosition) ? yAxis.majorTickLines.size + yAxis.axisLine.width / 2 : -(yAxis.majorTickLines.size + yAxis.axisLine.width / 2);
                            sbYMajorTicks.append("M" + " " + (Math.floor(xPosition + x1) + mtVal) + " " + (Math.floor((pointY * -1) + y1) + mtVal) + " " + "L" + " " + (Math.floor(x1 + ((yAxis.opposedPosition) ? (yAxis.axisLine.width / 2) : (yAxis.axisLine.width / 2))) + mtVal) + " " + (Math.floor((pointY * -1) + y1) + mtVal) + " ");
                        }


                        // Calculate Major Grid lines
                        if (yAxis.majorGridLines.visible) {
                            pointY = ej.EjSvgRender.utils._getPointXY(tempInterval, yAxis.visibleRange, yAxis.isInversed) * (yAxis.height);
                            var mX1 = (Math.floor(this.model.m_AreaBounds.X) + mlVal);
                            var mY1 = (Math.floor((pointY * -1) + y1));
                            var mX2 = (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + mlVal);
                            if (i == 0) {
                                if (mY1 == Math.floor(this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height))
                                    continue;
                            }
                            if ((i == (yAxis.visibleLabels.length - 1)) && (this.model.chartArea.border.color != "transparent" && this.model.chartArea.border.width > 0)) {
                                if (mY1 == (Math.floor(this.model.m_AreaBounds.Y)))
                                    continue;

                            }

                            sbYMajorGrid.append("M" + " " + mX1 + " " + (mY1 + mlVal) + " " + "L" + " " + mX2 + " " + (mY1 + mlVal) + " ");
                        }
                    }
                }
            }

            //Drawning Minor Ticks
            if (yAxis.minorTickLines.visible && yAxis.visible) {
                yMinorTicksDir = sbYMinorTicks.toString();
                var optionsMinotTick = {
                    'id': this.svgObject.id + '_YAxisMinorTicks',
                    'fill': 'none',
                    'stroke-width': yAxis.minorTickLines.width,
                    'stroke': yAxis.minorTickLines.color,
                    'd': yMinorTicksDir
                };

                this.svgRenderer.drawPath(optionsMinotTick, gEle);
            }
            if (yAxis.minorGridLines.visible) {
                //Drawning Minor Grid Lines
                yMinorGridDir = sbYMinorGrid.toString();
                var optionsMinorGrid = {
                    'id': this.svgObject.id + '_YAxisMinorGridLines',
                    'fill': 'none',
                    'stroke-width': yAxis.minorGridLines.width,
                    'stroke': yAxis.minorGridLines.color,
                    'stroke-dasharray': yAxis.minorGridLines.dashArray,
                    'd': yMinorGridDir
                };
                this.svgRenderer.drawPath(optionsMinorGrid, gEle);

            }

            if (yAxis.majorTickLines.visible && yAxis.visible) {
                yMajorTicksDir = sbYMajorTicks.toString();
                //Drawning Major Ticks
                var optionsMajorTick = {
                    'id': this.svgObject.id + '_YAxisMajorTicks',
                    'fill': 'none',
                    'stroke-width': yAxis.majorTickLines.width,
                    'stroke': yAxis.majorTickLines.color,
                    'd': yMajorTicksDir
                };
                this.svgRenderer.drawPath(optionsMajorTick, gEle);

            }
            //Drawing Major Grid Lines
            yMajorGridDir = sbYMajorGrid.toString();
            if (yAxis.majorGridLines.visible) {
                var options = {
                    'id': this.svgObject.id + '_YAxisMajorGridLines',
                    'fill': 'none',
                    'stroke-width': yAxis.majorGridLines.width,
                    'opacity': yAxis.majorGridLines.opacity,
                    'stroke-dasharray': yAxis.majorGridLines.dashArray,
                    'stroke': yAxis.majorGridLines.color,
                    'd': yMajorGridDir
                };

                this.svgRenderer.drawPath(options, gEle);

            }
            $(gEle).appendTo(this.gYaxisEle);
        },

        _drawXAxisLabels: function (axisIndex, xAxis) {
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels' + '_' + axisIndex, 'cursor': 'default' });
            var prevLabels = [];

            // loop to draw labels
            for (var i = 0; i < xAxis.visibleLabels.length; i++) {
                var label = xAxis.visibleLabels[i];
                var labelText = label.displayText = label.Text;
                if (ej.EjSvgRender.utils._inside(label.Value, xAxis.visibleRange)) {
                    //Declaration
                    var gap = xAxis.width / xAxis.visibleLabels.length;       //space between ticks
                    var range = xAxis.visibleRange;
                    var areaBoundWidth = this.model.m_AreaBounds.Width;
                    var pointX = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(label.Value, range, xAxis.isInversed) * (xAxis.width)));
                    var textSize = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font);
                    var textWidth = textSize.width;
                    var textHeight = textSize.height;
                    var labelMaxWidth = xAxis._LableMaxWidth;
                    var labelMaxHeight = labelMaxWidth.height;
                    var labelMaxRow = labelMaxWidth.rows;
                    var vmlrendering = this.svgRenderer.vmlNamespace;
                    var opposedPosition = xAxis.opposedPosition;
                    var lineWidth = xAxis.axisLine.width;
                    var font = xAxis.font;
                    var labelRotation = xAxis.labelRotation;
                    var hidePartialLabels = xAxis.hidePartialLabels;
                    var count = 0;
                    var flag = false;

                    if (!ej.util.isNullOrUndefined(label.Text)) {
                        var x = pointX + xAxis.x;
                        var y = (opposedPosition) ? (xAxis.y - xAxis.majorTickLines.size + textHeight / 2 - (lineWidth / 2)
                            - (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y + xAxis.majorTickLines.size + (lineWidth / 2)
                            + (labelMaxHeight / (2 * labelMaxRow)) + 5);
                        var options = {
                            'id': this.svgObject.id + '_' + xAxis.axisName + '_XLabel_' + i,
                            'x': x,
                            'y': y,
                            'fill': font.color,
                            'font-size': font.size,
                            'font-family': font.fontFamily,
                            'font-style': font.fontStyle,
                            'font-weight': font.fontWeight,
                            'opacity': font.opacity,
                            'text-anchor': 'middle'
                        };

                        //To hide partial labels
                        var length = xAxis.visibleLabels.length;
                        if ((!this.zoomed) && xAxis._valueType == 'string' && (hidePartialLabels) && xAxis.labelPlacement == 'onTicks' && ((i == 0) || (i == (length - 1))))
                            continue;
                        else if ((!this.zoomed) && (hidePartialLabels) && ((i == 0) || (i == (length - 1))))
                            continue;

                        //To perform rotation 
                        if (!vmlrendering && labelRotation && labelRotation != 0)
                            this.labelRotation(xAxis, x, y, options, label, labelRotation);

                        if (xAxis.labelIntersectAction) {
                            //To perform label intersect action
                            var intersectAction = xAxis.labelIntersectAction.toLowerCase();
                            if (intersectAction == 'trim') {
                                for (var t = 4; t > 0; t--) {
                                    if (textWidth > gap) {
                                        labelText = labelText.substring(0, t) + '...';
                                        textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, font).width;
                                        label.displayText = labelText;
                                    } else
                                        break;
                                }
                            } else if (intersectAction == 'wrap') {
                                options.y = opposedPosition ? options.y + textHeight : options.y - textHeight;
                                var text = labelText;
                                var textLength = text.length;
                                if (textWidth > gap) {
                                    for (var w = 1; w <= text.length; w++) {
                                        labelText = text.substring(0, w);
                                        textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, font).width;
                                        if (textWidth > gap) {
                                            var count = +1;
                                            labelText = text.substring(0, w - 1);
                                            text = text.slice(w - 1, textLength);
                                            options.y = opposedPosition ? options.y - (textHeight * count - 1) : options.y + (textHeight * count - 1);
                                            this.svgRenderer.drawText(options, labelText, gEle);
                                            w = 0;
                                        }
                                    }
                                }
                                options.y = opposedPosition ? options.y - textHeight : options.y + textHeight;
                                this.svgRenderer.drawText(options, labelText, gEle);
                                continue;
                            } else if (labelRotation != 0 && !vmlrendering && intersectAction == 'rotate90')
                                this.labelRotation(xAxis, x, y, options, label, 90);
                            else if (labelRotation != 0 && !vmlrendering && intersectAction == 'rotate45')
                                this.labelRotation(xAxis, x, y, options, label, 45);
                            else if (intersectAction == 'hide') {
                                for (var j = 0; j < i; j++) {
                                    // loop to get previous label
                                    var preLabel = xAxis.visibleLabels[j];
                                    var temp = 0;
                                    opposedPosition = xAxis.opposedPosition;
                                    areaBoundWidth = this.model.m_AreaBounds.Width;

                                    var prePoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(preLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                                    var preTextWidth = ej.EjSvgRender.utils._measureText(preLabel.displayText, areaBoundWidth, xAxis.font).width;
                                    var value = prePoint + ((xAxis.isInversed) ? -preTextWidth / 2 : preTextWidth / 2);
                                    if (intersectAction == 'hide' && ((xAxis.isInversed) ? (value < pointX + textWidth / 2) : (value > pointX - textWidth / 2)))
                                        label.displayText = labelText = '';
                                }
                            }
                            else {
                                for (var k = 0; k < prevLabels.length && !flag; k++) {
                                    for (var l = 0; prevLabels[k] && l < prevLabels[k].length; l++) {
                                        var preLabel = prevLabels[k][l];
                                        var temp = 0;
                                        opposedPosition = xAxis.opposedPosition;
                                        areaBoundWidth = this.model.m_AreaBounds.Width;

                                        var prePoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(preLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                                        var preTextWidth = ej.EjSvgRender.utils._measureText(preLabel.displayText, areaBoundWidth, xAxis.font).width;
                                        var value = prePoint + ((xAxis.isInversed) ? -preTextWidth / 2 : preTextWidth / 2);
                                        if ((xAxis.isInversed) ? (value < pointX + textWidth / 2) : (value > pointX - textWidth / 2)) {
                                            if (opposedPosition) {
                                                temp = options.y - textHeight;
                                                options.y = temp;
                                                if (k + 1 == prevLabels.length)
                                                    flag = true;
                                                count++;
                                                break;
                                            } else {
                                                temp = options.y + textHeight;
                                                options.y = temp;
                                                if (k + 1 == prevLabels.length)
                                                    flag = true;
                                                count++;
                                                break;

                                            }
                                        }
                                        else {
                                            if (l + 1 == prevLabels[k].length) {
                                                flag = true;
                                                break;
                                            }
                                        }
                                    }
                                }
                                if (temp == undefined) {
                                    if (prevLabels[0] == undefined)
                                        prevLabels[0] = [];
                                    prevLabels[0].push(label);
                                }
                                else {
                                    if (prevLabels[count] == undefined)
                                        prevLabels[count] = [];
                                    prevLabels[count].push(label);
                                }
                            }
                        }
                        if (!this.zoomed && !xAxis.zoomed && !(labelRotation) && (((pointX + xAxis.x) + (textWidth / 2)) > ($(this.svgObject).width()))) {
                            $(options).attr('x', ($(this.svgObject).width() - 2));
                            $(options).attr('text-anchor', 'end');
                        }
                        xAxis.visibleLabels[i].y = options.y;
                        this.svgRenderer.drawText(options, labelText, gEle);
                    }
                }
            }
            $(gEle).appendTo(this.gXaxisEle);
        },

        labelRotation: function (axis, x, y, options, label, degree) {
            // To rotate axis labels
            var labelText = label.Text;
            var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
            var rotate = 'rotate(' + angle + ',' + (x) + ',' + y + ')';
            $(options).attr('transform', rotate);
            var textElement = this.svgRenderer.createText(options, labelText);
            var diffHeight = Math.ceil(ej.EjSvgRender.utils._measureBounds(textElement, this).height - ej.EjSvgRender.utils._measureText(labelText, null, axis.font).height);
            diffHeight = axis._LableMaxWidth.height - diffHeight - ej.EjSvgRender.utils._measureText(labelText, null, axis.font).height;
            var yLocation = (axis.opposedPosition) ? (diffHeight / 2) : (-diffHeight / 2);
            rotate = 'rotate(' + angle + ',' + (x) + ',' + (y + yLocation) + ')';
            $(options).attr({
                'transform': rotate,
                'y': (y + yLocation)
            });
        },

        _drawYAxisLabels: function (axisIndex, yAxis) {
            // method to draw y axis labels
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLabels' + '_' + axisIndex });
            for (var i = 0; i < yAxis.visibleLabels.length; i++) {
                var label = yAxis.visibleLabels[i];
                var visibleRange = yAxis.visibleRange;
                if (ej.EjSvgRender.utils._inside(label.Value, visibleRange)) {
                    var pointY = Math.abs(ej.EjSvgRender.utils._getPointXY(label.Value, visibleRange, yAxis.isInversed) * (yAxis.height));
                    var text = ej.EjSvgRender.utils._measureText(label.Text, this.model.m_AreaBounds.Height, yAxis.font);
                    var textHeight = text.height;
                    var opposedPosition = yAxis.opposedPosition;
                    var font = yAxis.font;
                    var range = yAxis.visibleRange;
                    var lineWidth = yAxis.axisLine.width;
                    var x = (opposedPosition) ? ((yAxis.majorTickLines.size) + (lineWidth / 2) + yAxis.x + 5)
                        : (yAxis.x - (lineWidth / 2) - 5 - yAxis.majorTickLines.size);
                    var y = ((pointY * -1) + (yAxis.y + yAxis.height) + 3);
                    var options = {
                        'id': this.svgObject.id + '_YLabel_' + i,
                        'x': x,
                        'y': y,
                        'fill': font.color,
                        'font-size': font.size,
                        'font-family': font.fontFamily,
                        'font-style': font.fontStyle,
                        'font-weight': font.fontWeight,
                        'opacity': font.opacity,
                        'text-anchor': (opposedPosition) ? 'start' : 'end'
                    };

                    // To hide partial labels
                    if ((yAxis.hidePartialLabels) && ((i == 0) || (i == (yAxis.visibleLabels.length - 1))))
                        continue;

                    //Label intersection
                    if (yAxis.labelIntersectAction) {
                        var intersectAction = yAxis.labelIntersectAction.toLowerCase();
                        for (var j = 0; j < i; j++) {
                            var prevLabel = yAxis.visibleLabels[j];
                            var temp = 0;
                            var prevPoint = Math.abs((ej.EjSvgRender.utils._getPointXY(prevLabel.Value, range, yAxis.isInversed) * (yAxis.height)));
                            var prevTextHeight = ej.EjSvgRender.utils._measureText(prevLabel.Text, this.model.m_AreaBounds.Height, font).height;
                            var value = ((prevPoint * -1) + (yAxis.y + yAxis.height) + 3);
                            value = value + ((yAxis.isInversed) ? prevTextHeight / 2 : -prevTextHeight / 2);
                            if (intersectAction == 'hide' && ((yAxis.isInversed) ? value > options.y - textHeight / 2 : value < options.y + textHeight / 2))
                                label.Text = '';
                            else if (intersectAction == 'multiplerows' && ((yAxis.isInversed) ? value > options.y - textHeight / 2 : value < options.y + textHeight / 2) && prevLabel.x == options.x) {
                                if (opposedPosition) {
                                    temp = prevLabel.x + yAxis._LableMaxWidth.maxWidth + 5; // gap between the rows
                                    options.x = temp;
                                }
                                else {
                                    temp = prevLabel.x - yAxis._LableMaxWidth.maxWidth - 5; // gap between the rows
                                    options.x = temp;
                                }
                            }
                        }
                    }

                    yAxis.visibleLabels[i].x = options.x;
                    this.svgRenderer.drawText(options, label.Text, gEle);
                }
            }
            $(gEle).appendTo(this.gYaxisEle);
        },

        _drawXTitle: function (axisIndex, axis) {

            if (axis.title.text != "" && axis.title.text) {
                var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTitle' + '_' + axisIndex });
                var xtitleLocation = (this.model.elementSpacing) + axis._LableMaxWidth.height;
                var titlesize = (ej.EjSvgRender.utils._measureText(axis.title.text, (axis.width), axis.title.font).height / 2);
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.Data = { Title: axis.title.text, Location: { X: (axis.width) / 2, Y: (xtitleLocation + titlesize + axis.majorTickLines.size + axis.axisLine.width) }, Axes: axis };
                this._trigger("axesTitleRendering", commonEventArgs);
                var options = {
                    'id': this.svgObject.id + '_XAxisTitle' + '_' + axisIndex,
                    'x': commonEventArgs.Data.Location.X + axis.x,
                    'y': (axis.opposedPosition) ? (axis.y - commonEventArgs.Data.Location.Y) : (commonEventArgs.Data.Location.Y + axis.y),
                    'fill': axis.title.font.color,
                    'font-size': axis.title.font.size,
                    'font-family': axis.title.font.fontFamily,
                    'font-style': axis.title.font.fontStyle,
                    'font-weight': axis.title.font.fontWeight,
                    'opacity': axis.title.font.opacity,
                    'text-anchor': 'middle',
                    'dominant-baseline': 'middle'
                };
                if ($(this.svgObject).find("#" + this.svgObject.id + '_XAxisTitle' + '_' + axisIndex).length > 0)
                    $(this.svgObject).find("#" + this.svgObject.id + '_XAxisTitle' + '_' + axisIndex).attr(options);
                else
                    this.svgRenderer.drawText(options, commonEventArgs.Data.Title, gEle);
                $(gEle).appendTo(this.gXaxisEle);
            }

        },
        _drawYTitle: function (axisIndex, yAxis) {

            if (yAxis.title.text != "" && yAxis.title.text) {
                var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTitle' + '_' + axisIndex });
                var titleSize = ej.EjSvgRender.utils._measureText(yAxis.title.text, (yAxis.height), yAxis.title.font);
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.majorTickLines.size + yAxis.axisLine.width) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.majorTickLines.size + yAxis.axisLine.width);

                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.Data = { Title: yAxis.title.text, Location: { X: titleSize.width / 2, Y: 0 }, Axes: yAxis };
                this._trigger("axesTitleRendering", commonEventArgs);

                var y = commonEventArgs.Data.Location.Y + (yAxis.y + yAxis.height) + (((yAxis.height) / 2) * -1);

                var labelRotation = (yAxis.opposedPosition) ? 90 : -90;

                var options = {
                    'id': this.svgObject.id + '_YAxisTitle' + '_' + axisIndex,
                    'x': x,
                    'y': y,
                    'fill': yAxis.title.font.color,
                    'transform': 'rotate(' + labelRotation + ',' + (x) + ',' + y + ')',
                    'font-size': yAxis.title.font.size,
                    'font-family': yAxis.title.font.fontFamily,
                    'font-style': yAxis.title.font.fontStyle,
                    'font-weight': yAxis.title.font.fontWeight,
                    'opacity': yAxis.title.font.opacity,
                    'text-anchor': 'middle',
                    'dominant-baseline': 'middle'
                };
                if ($(this.svgObject).find("#" + this.svgObject.id + '_YAxisTitle' + '_' + axisIndex).length > 0)
                    $(this.svgObject).find("#" + this.svgObject.id + '_YAxisTitle' + '_' + axisIndex).attr(options);
                else
                    this.svgRenderer.drawText(options, commonEventArgs.Data.Title, gEle);
                $(gEle).appendTo(this.gYaxisEle);
            }

        }
    },
    ej.EjStripline.prototype = {
        _drawStripline: function (axis, stripline) {

            var x = 0, height = 0, angle = 0;
            var y = 0, pointYstart = 0, yHeight = 0;
            var width = 0, widthValue = 0, textX = 0, textY = 0;
            if (stripline.visible) {
                var striplineid = (stripline.zOrder == 'over') ? this.gStriplineOver : this.gStriplineBehind;
                var isVisible = true;
                if (axis.orientation.toLowerCase() == "horizontal") {
                    var pointstart = 0;

                    if (stripline.startFromAxis) {

                        var point = (ej.util.isNullOrUndefined(stripline.offset)) ? axis.visibleRange.min : axis.visibleRange.min + stripline.offset;
                        pointstart = (((point - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);

                        widthValue = (((point + (ej.util.isNullOrUndefined(stripline.width) ? 0 : stripline.width) - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.width));

                    } else {
                        if (ej.util.isNullOrUndefined(stripline.start) && (ej.util.isNullOrUndefined(stripline.end) || ej.util.isNullOrUndefined(stripline.width))) {
                            isVisible = false;
                        } else {
                            stripline.start = (jQuery.type(stripline.start) == "date") ? (stripline.start).getTime() : stripline.start;
                            if (!ej.util.isNullOrUndefined(stripline.end) && (jQuery.type(stripline.end) == "date")) {
                                stripline.end = (stripline.end).getTime();
                            }
                            pointstart = (ej.EjSvgRender.utils._getPointXY(stripline.start, axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);
                            var pointend = (ej.EjSvgRender.utils._getPointXY(stripline.end, axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);
                            var widthMax = (ej.EjSvgRender.utils._getPointXY((stripline.start + (ej.util.isNullOrUndefined(stripline.width) ? 0 : stripline.width)), axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x;

                            widthValue = (axis.isInversed ? (pointend < widthMax) : (pointend > widthMax)) ? Math.abs(pointstart - pointend) : Math.abs(pointstart - widthMax);

                        }
                    }
                    width = widthValue;
                    height = this.chart.model.m_AreaBounds.Height;
                    x = (axis.isInversed) ? (pointstart - widthValue) : pointstart;
                    y = this.chart.model.m_AreaBounds.Y;
                    angle = -90;
                    var textWidth = ej.EjSvgRender.utils._measureText(stripline.text, null, stripline.font).height;
                    if (stripline.textAlignment == 'middletop') {
                        textX = x - (textWidth / 2);
                        textY = Math.abs(y + height / 2);
                    } else if (stripline.textAlignment == 'middlecenter') {
                        textX = (Math.floor(x + width / 2) + (textWidth / 3));
                        textY = Math.abs(y + height / 2);
                    } else if (stripline.textAlignment == 'middlebottom') {
                        textX = x + width + (textWidth);
                        textY = Math.abs(y + height / 2);
                    }


                } else {
                    if (stripline.startFromAxis) {
                        var pointY = (ej.util.isNullOrUndefined(stripline.offset)) ? axis.visibleRange.min : axis.visibleRange.min + stripline.offset;
                        pointYstart = (axis.y + axis.height) - (((pointY - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.height));
                        widthValue = (((pointY + (ej.util.isNullOrUndefined(stripline.width) ? 0 : stripline.width) - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.height));
                        yHeight = widthValue;
                    } else {

                        if (ej.util.isNullOrUndefined(stripline.start) && (ej.util.isNullOrUndefined(stripline.end) || ej.util.isNullOrUndefined(stripline.width))) {
                            isVisible = false;
                        } else {
                            stripline.start = (jQuery.type(stripline.start) == "date") ? (stripline.start).getTime() : stripline.start;
                            if (!ej.util.isNullOrUndefined(stripline.end) && (jQuery.type(stripline.end) == "date")) {
                                stripline.end = (stripline.end).getTime();
                            }
                            pointYstart = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY(stripline.start, axis.visibleRange, axis.isInversed) * (axis.height));
                            var pointYend = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY((ej.util.isNullOrUndefined(stripline.end) ? 0 : stripline.end), axis.visibleRange, axis.isInversed) * (axis.height));
                            var widthYValue = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY((stripline.start + (ej.util.isNullOrUndefined(stripline.width) ? 0 : stripline.width)), axis.visibleRange, axis.isInversed) * (axis.height));
                            yHeight = (axis.isInversed ? (pointYend > widthYValue) : (pointYend < widthYValue)) ? Math.abs(pointYstart - pointYend) : Math.abs(pointYstart - widthYValue);
                        }
                    }
                    height = yHeight;
                    x = this.chart.model.m_AreaBounds.X;
                    width = this.chart.model.m_AreaBounds.Width;
                    y = pointYstart + ((axis.isInversed) ? 0 : -yHeight);
                    var textHeight = ej.EjSvgRender.utils._measureText(stripline.text, width, stripline.font).height;

                    if (stripline.textAlignment == 'middletop') {
                        textX = x + width / 2;
                        textY = y - (textHeight / 3);
                    } else if (stripline.textAlignment == 'middlecenter') {
                        textX = x + width / 2;
                        textY = (Math.abs(y + (height / 2)) + (textHeight / 3));
                    } else if (stripline.textAlignment == 'middlebottom') {
                        textX = x + width / 2;
                        textY = y + height + (textHeight);
                    }


                }
                if (isVisible) {
                    if (!ej.util.isNullOrUndefined(stripline.imageUrl)) {
                        var imgOptions = {
                            'height': height,
                            'width': width,
                            'href': stripline.imageUrl,
                            'x': x,
                            'y': y,
                            'id': striplineid.id + '_backImage',
                            'visibility': 'visible',
                            'preserveAspectRatio': 'none'
                        };

                        this.svgRenderer.drawImage(imgOptions, striplineid);
                    } else {
                        var options = {
                            'id': striplineid.id + '_striplineRect',
                            'x': x,
                            'y': y,
                            'width': width,
                            'height': height,
                            'fill': stripline.color,
                            'opacity': stripline.opacity,
                            'stroke-width': stripline.borderWidth,
                            'stroke': (stripline.borderWidth == 0) ? "transparent" : stripline.borderColor
                        };
                        var textOptions = {
                            'id': striplineid.id + '_striplineRectText',
                            'x': textX,
                            'y': textY,
                            'fill': stripline.font.color,
                            'transform': 'rotate(' + angle + ',' + (textX) + ',' + textY + ')',
                            'font-size': stripline.font.size,
                            'font-family': stripline.font.fontFamily,
                            'font-style': stripline.font.fontStyle,
                            'font-weight': stripline.font.fontWeight,
                            'text-anchor': 'middle'
                        };

                        var bounds = { X: x, Y: y, Width: width, Height: height };
                        var isStripline = stripline.zOrder == "over" ? true : false;
                        ej.EjSvgRender.utils.AddRegion(this.chart, bounds, isStripline)

                        this.svgRenderer.drawRect(options, striplineid);
                        this.svgRenderer.drawText(textOptions, stripline.text, striplineid);
                    }
                }

            }
        }
    };
})(jQuery);
ej.EjLegendRender = function (chartobj) {

    this.chartObj = chartobj;
};


(function ($) {
    ej.EjLegendRender.prototype =
    {

        drawLegendItem: function (legendItem, x, y) {
            var shapeSize = this.chartObj.model.legend.itemSize;
            var location = { startX: x + shapeSize.width / 2, startY: (y + shapeSize.height / 2) };

            var style = { ShapeSize: shapeSize, Style: legendItem.LegendStyle, Gradient: legendItem.Gradient, SeriesIndex: legendItem.SeriesIndex };
            var symbolName;
            for (name in this.chartObj.model.symbolShape) {
                if (legendItem.Shape.toLowerCase() == name.toLowerCase()) {
                    symbolName = name;
                    break;
                }
            }
            if (symbolName == "None")
                symbolName = "SeriesType";

            var index = (ej.util.isNullOrUndefined(legendItem.PointIndex)) ? legendItem.SeriesIndex : legendItem.PointIndex;
            var cursorStyle = (this.chartObj.model.AreaType == "cartesianaxes" && !this.chartObj.zoomed) ? 'pointer' : 'default';
            this.chartObj.gLegendItemEle = this.chartObj.svgRenderer.createGroup({ 'id': this.chartObj.svgObject.id + '_Legend' + index, 'cursor': cursorStyle });

            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.Data = { LegendItem: legendItem, Location: location, Legend: true, model: this.chartObj.model, svgRenderer: this.chartObj.svgRenderer, svgObject: this.chartObj.svgObject, SymbolShape: symbolName, Style: style, gLegendItemEle: this.chartObj.gLegendItemEle };

            this.chartObj._trigger("legendItemRendering", commonEventArgs);


            ej.EjSvgRender.chartSymbol["_draw" + commonEventArgs.Data.SymbolShape](commonEventArgs.Data.Location, commonEventArgs.Data.Style, commonEventArgs.Data, this.chartObj.gLegendItemEle);
            var color;
            if (legendItem.LegendStyle.Color && legendItem.LegendStyle.Color.toLowerCase() !== 'gray')
                color = this.chartObj.model.legend.font.color;
            else
                color = 'gray';

            var fontLegend = this.chartObj.model.legend.font;
            var itemInfo = commonEventArgs.Data.LegendItem;
            var heightText = ej.EjSvgRender.utils._measureText(itemInfo.Text, this.chartObj.model.LegendBounds.Width, fontLegend).height;

            var options = {
                'id': this.chartObj.svgObject.id + '_LegendItemText',
                'x': shapeSize.width + x + this.chartObj.model.legend.itemPadding / 2,
                'y': commonEventArgs.Data.Location.startY,
                'fill': color,
                'font-size': fontLegend.size,
                'font-style': fontLegend.fontStyle,
                'font-family': fontLegend.fontFamily,
                'font-weight': fontLegend.fontWeight,
                'text-anchor': 'start'
            };
            var matched = jQuery.uaMatch(navigator.userAgent);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (matched.browser.toLowerCase() == "msie" || isIE11)
                options.y = (commonEventArgs.Data.Location.startY + (heightText / 3));
            else
                this.chartObj.svgRenderer._setAttr($(options), { 'dominant-baseline': 'middle' });

            this.chartObj.svgRenderer.drawText(options, commonEventArgs.Data.LegendItem.Text, this.chartObj.gLegendItemEle);
            this.chartObj.svgRenderer.append(this.chartObj.gLegendItemEle, this.chartObj.gLegendEle);
            var legendbound = { X: this.chartObj.model.LegendBounds.X, Y: this.chartObj.model.LegendBounds.Y };
            var itembound = { X: (x), Y: (y) };
            var bounds = { LegendBound: legendbound, ItemBound: itembound };
            var legendRegion = { LegendItem: legendItem, Location: location, SymbolShape: symbolName, Style: style, Bounds: bounds };
            this.chartObj.model.legendRegion.push(legendRegion);
        },

        drawLegend: function () {

            if (!ej.util.isNullOrUndefined(this.chartObj.model.legend) && this.chartObj.model.legend.visible) {
                // draw legend.

                if (this.chartObj.model.legend.position.toLowerCase() == this.chartObj.model.legendPosition.bottom) {
                    this.chartObj.model.LegendBounds.Y = $(this.chartObj.svgObject).height() - (this.chartObj.model.elementSpacing * 1.5) - this.chartObj.model.LegendBounds.Height;

                    this.chartObj.model.LegendBounds.X = ($(this.chartObj.svgObject).width() / 2) - (this.chartObj.model.LegendBounds.Width / 2);

                }
                else if (this.chartObj.model.legend.position.toLowerCase() == this.chartObj.model.legendPosition.top) {
                    this.chartObj.model.LegendBounds.Y = ((this.chartObj.model._titleLocation) ? this.chartObj.model._titleLocation.Y : 0) + this.chartObj.model.elementSpacing;
                    this.chartObj.model.LegendBounds.X = ($(this.chartObj.svgObject).width() / 2) - (this.chartObj.model.LegendBounds.Width / 2);
                }

                else if (this.chartObj.model.legend.position.toLowerCase() == this.chartObj.model.legendPosition.right) {
                    this.chartObj.model.LegendBounds.Y = ($(this.chartObj.svgObject).height()) / 2 - (this.chartObj.model.LegendBounds.Height / 2);
                    this.chartObj.model.LegendBounds.X = $(this.chartObj.svgObject).width() - this.chartObj.model.elementSpacing - this.chartObj.model.LegendBounds.Width;
                }

                else if (this.chartObj.model.legend.position.toLowerCase() == this.chartObj.model.legendPosition.left) {
                    this.chartObj.model.LegendBounds.Y = ($(this.chartObj.svgObject).height()) / 2 - (this.chartObj.model.LegendBounds.Height / 2);
                    this.chartObj.model.LegendBounds.X = this.chartObj.model.elementSpacing;
                }
                else {
                    this.chartObj.model.LegendBounds.Y = (this.chartObj.model.legend.location.Y < $(this.chartObj.svgObject).height()) ? this.chartObj.model.legend.location.Y : 0;
                    this.chartObj.model.LegendBounds.X = (this.chartObj.model.legend.location.X < $(this.chartObj.svgObject).width()) ? this.chartObj.model.legend.location.X : 0;
                }
                if (this.chartObj.model.legend.position.toLowerCase() == 'left' || this.chartObj.model.legend.position.toLowerCase() == 'right') {
                    if (this.chartObj.model.legend.alignment.toLowerCase() == 'center') {
                        this.chartObj.model.LegendBounds.Y = ($(this.chartObj.svgObject).height()) / 2 - (this.chartObj.model.LegendBounds.Height / 2);
                    } else if (this.chartObj.model.legend.alignment.toLowerCase() == 'near') {
                        this.chartObj.model.LegendBounds.Y = this.chartObj.model.m_Spacing.Top + this.chartObj.model.elementSpacing + (this.chartObj.model.legend.itemPadding / 2);
                    } else {
                        this.chartObj.model.LegendBounds.Y = $(this.chartObj.svgObject).height() - this.chartObj.model.m_Spacing.Bottom - this.chartObj.model.LegendBounds.Height - this.chartObj.model.elementSpacing - 5;
                    }
                }
                else {
                    if (this.chartObj.model.legend.position.toLowerCase() != 'custom') {
                        if (this.chartObj.model.legend.alignment.toLowerCase() == 'center') {
                            if (this.chartObj.model.AreaType == "cartesianaxes")
                                this.chartObj.model.LegendBounds.X = (this.chartObj.model.m_AreaBounds.Width / 2) - (this.chartObj.model.LegendBounds.Width / 2)
                                + this.chartObj.model.m_AreaBounds.X;
                            else
                                this.chartObj.model.LegendBounds.X = ($(this.chartObj.svgObject).width() / 2) - (this.chartObj.model.LegendBounds.Width / 2);
                        } else if (this.chartObj.model.legend.alignment.toLowerCase() == 'near') {
                            this.chartObj.model.LegendBounds.X = this.chartObj.model.m_Spacing.Left + this.chartObj.model.elementSpacing + (this.chartObj.model.legend.itemPadding / 2);
                        } else {
                            this.chartObj.model.LegendBounds.X = $(this.chartObj.svgObject).width() - this.chartObj.model.m_Spacing.Right - this.chartObj.model.LegendBounds.Width - this.chartObj.model.elementSpacing - 5;
                        }
                    }
                }

                var legnTx = this.chartObj.model.LegendBounds.X;

                var legnTy = this.chartObj.model.LegendBounds.Y;
                if (ej.util.isNullOrUndefined(this.chartObj.svgRenderer.vmlNamespace))
                    this.chartObj.svgRenderer._setAttr($(this.chartObj.gLegendEle), { 'transform': 'translate(' + legnTx + ',' + legnTy + ')' });
                else {
                    $(this.chartObj.gLegendEle).css({ 'position': 'absolute', 'left': legnTx, 'top': legnTy });
                }

                var options = {
                    'id': this.chartObj.svgObject.id + '_LegendArea', 'x': 0, 'y': 0, 'width': this.chartObj.model.LegendBounds.Width, 'height': this.chartObj.model.LegendBounds.Height,
                    'fill': 'none', 'stroke-width': this.chartObj.model.legend.border.width, 'stroke': this.chartObj.model.legend.border.color
                };

                this.chartObj.svgRenderer.drawRect(options, this.chartObj.gLegendEle);


                var currentX, currentY;
                var startX, startY;
                currentX = startX = this.chartObj.model.legend.itemPadding;
                currentY = startY = this.chartObj.model.legend.itemPadding;

                for (var k = 0; k < this.chartObj.model.legendCollection.length; k++) {
                    var position = this.chartObj.model.legend.position.toLowerCase();
                    if (position == this.chartObj.model.legendPosition.top || position == this.chartObj.model.legendPosition.bottom || position == this.chartObj.model.legendPosition.custom || this.chartObj.model.legend.rowCount || this.chartObj.model.legend.columnCount) {
                        if ((currentX + this.chartObj.model.legendCollection[k].Bounds.Width) > this.chartObj.model.LegendBounds.Width + startX) {
                            currentX = this.chartObj.model.legend.itemPadding;
                            currentY += this.chartObj.model.legendCollection[k].Bounds.Height;
                        }
                        this.drawLegendItem(this.chartObj.model.legendCollection[k], currentX, (currentY));
                        currentX += this.chartObj.model.legendCollection[k].Bounds.Width;
                    }
                    else {
                        if (((currentY + this.chartObj.model.legendCollection[k].Bounds.Height) > this.chartObj.model.LegendBounds.Height + startY)) {
                            currentY = this.chartObj.model.legend.itemPadding;
                            currentX = (this.chartObj.model.legendCollection[k].Bounds.Width) + (this.chartObj.model.legend.itemPadding);

                        }
                        this.drawLegendItem(this.chartObj.model.legendCollection[k], (currentX), (currentY));
                        currentY += this.chartObj.model.legendCollection[k].Bounds.Height;
                    }
                }

                this.chartObj.svgRenderer.append(this.chartObj.gLegendEle, this.chartObj.svgObject);

            }
        }
    };
})(jQuery);;

ej.seriesTypes = {};

ej.EjSeriesRender = function () {

};
(function ($) {

    ej.EjSeriesRender.prototype = {

        setLineSeriesStyle: function (currentSeries) {

            var seriesIndex = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
            var serColor = this.chartObj.model.seriesColors[seriesIndex];


            var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(currentSeries) + seriesIndex, serColor, 0, 0, 0, $(this.chartObj.svgObject).height(), this.chartObj.gSeriesEle);
            var seriesBorder = this.chartObj.model.seriesBorderColors[seriesIndex];
            return { SeriesInterior: seriesInterior, SeriesBorder: seriesBorder };

        },
        getSeriesName: function (series) {
            var serName;
            if (series.name)
                serName = series.name.replace(/\s/g, '');
            else
                serName = "series";

            return serName;
        },

        drawAreaPath: function (series, style, direction) {
            var seriesIndex = $.inArray(series, this.chartObj.model.series);
            var transX = series.xAxis.x;

            var transY = series.yAxis.y;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

            var options = {
                'id': this.chartObj.svgObject.id + "_" + this.getSeriesName(series) + "_" + seriesIndex,
                'fill': style.SeriesInterior,
                'fill-opacity': series.opacity,
                'stroke-width': series.border.width,
                'stroke': series.border.color,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'd': direction
            };
            this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);
        },

        setAreaSeriesStyle: function (series) {

            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
            var maxY = Math.max.apply(0, $.map(series.points, function (v) {
                return v.YValues[0];
            }));
            var minY = Math.min.apply(0, $.map(series.points, function (v) {
                return v.YValues[0];
            }));
            var minX = Math.min.apply(0, $.map(series.points, function (v) {
                return v.X;
            }));

            var point1Xy = ej.EjSvgRender.utils._getPointXY(minX, minY, series);
            var point2Xy = ej.EjSvgRender.utils._getPointXY(minX, maxY, series);
            var serColor = this.chartObj.model.seriesColors[seriesIndex];

            var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(series) + seriesIndex, serColor, point1Xy.X, point1Xy.Y, point2Xy.X, point2Xy.Y, this.chartObj.gSeriesEle);

            return { SeriesInterior: seriesInterior };

        },





        naturalSpline: function (points) {
            var count = points.length;
            //constant
            var yIndex = 0, a = 6;
            var ySpline = [], ySplineDuplicate = [];

            // assigning the first and last value as zero
            ySpline[0] = ySplineDuplicate[0] = 0;
            ySpline[points.length - 1] = 0;

            for (var i = 1; i < count - 1; i++) {
                var d1 = points[i].X - points[i - 1].X;
                var d2 = points[i + 1].X - points[i - 1].X;
                var d3 = points[i + 1].X - points[i].X;
                var dy1 = points[i + 1].YValues[yIndex] - points[i].YValues[yIndex];
                var dy2 = points[i].YValues[yIndex] - points[i - 1].YValues[yIndex];

                if (d1 == 0 || d2 == 0 || d3 == 0) {
                    ySpline[i] = 0;
                    ySplineDuplicate[i] = 0;
                }
                else {
                    var p = 1 / (d1 * ySpline[i - 1] + 2 * d2);

                    ySpline[i] = -p * d3;
                    ySplineDuplicate[i] = p * (a * (dy1 / d3 - dy2 / d1) - d1 * ySplineDuplicate[i - 1]);
                }
            }

            for (var k = count - 2; k >= 0; k--)
                ySpline[k] = ySpline[k] * ySpline[k + 1] + ySplineDuplicate[k];


            return ySpline;

        },
        getBezierControlPoints: function (point1, point2, ySpline1, ySpline2, yIndex) {
            var controlPoint1;
            var controlPoint2;
            var one3 = 1 / 3.0;
            var deltaX2 = (point2.X - point1.X);

            deltaX2 = deltaX2 * deltaX2;

            var dx1 = 2 * point1.X + point2.X;
            var dx2 = point1.X + 2 * point2.X;

            var dy1 = 2 * point1.YValues[yIndex] + point2.YValues[yIndex];
            var dy2 = point1.YValues[yIndex] + 2 * point2.YValues[yIndex];

            var y1 = one3 * (dy1 - one3 * deltaX2 * (ySpline1 + 0.5 * ySpline2));
            var y2 = one3 * (dy2 - one3 * deltaX2 * (0.5 * ySpline1 + ySpline2));

            controlPoint1 = { X: dx1 * one3, YValues: [y1] };
            controlPoint2 = { X: dx2 * one3, YValues: [y2] };

            return { controlPoint1: controlPoint1, controlPoint2: controlPoint2 };


        },

        improveChartPerformance: function (chartSeries) {
            var tempPoints = [];
            var prevXValue = 0;
            var prevYValue = 0;
            var xDelta = chartSeries.xAxis.visibleRange.max - chartSeries.xAxis.visibleRange.min;
            var yDelta = chartSeries.yAxis.visibleRange.max - chartSeries.yAxis.visibleRange.min;

            var xTolerance = Math.abs((xDelta * 1) / this.chartObj.model.m_AreaBounds.Width);
            var yTolerance = Math.abs((yDelta * 1) / this.chartObj.model.m_AreaBounds.Height);

            var xVal = 0;
            for (var k = 0; k < chartSeries.points.length; k++) {
                var isEmpty = (chartSeries.points[k].isEmpty === null || chartSeries.points[k].isEmpty === undefined) ? false : chartSeries.points[k].isEmpty;
                chartSeries.points[k].visible = (chartSeries.points[k].visible === null || chartSeries.points[k].visible === undefined) ? true : chartSeries.points[k].visible;
                if (chartSeries.points[k].X === null || chartSeries.points[k].X === undefined) {
                    chartSeries.points[k].isEmpty = true;
                    chartSeries.points[k].visible = false;
                    chartSeries.points[k].X = chartSeries.xAxis.visibleRange.min;
                } else
                    chartSeries.points[k].isEmpty = isEmpty;
                if ((chartSeries.points[k].YValues[0] == null) || (chartSeries.points[k].YValues[0] == 'undefined')) {
                    chartSeries.points[k].isEmpty = true;
                    chartSeries.points[k].visible = false;
                    chartSeries.points[k].YValues[0] = chartSeries.yAxis.visibleRange.min;
                } else {
                    chartSeries.points[k].isEmpty = chartSeries.points[k].isEmpty;
                    chartSeries.points[k].visible = (chartSeries.points[k].isEmpty) ? !chartSeries.points[k].isEmpty : chartSeries.points[k].visible;
                }

                var xVal = chartSeries.points[k].X;
                var yVal = chartSeries.points[k].YValues[0];
                if (Math.abs(prevXValue - xVal) >= xTolerance || Math.abs(prevYValue - yVal) >= yTolerance) {

                    tempPoints.push(chartSeries.points[k]);
                    prevXValue = xVal;
                    prevYValue = yVal;
                }
            }
            return tempPoints;

        },
        _isVisiblePoints: function (chartSeries) {
            for (var k = 0; k < chartSeries.points.length; k++) {
                var isEmpty = (ej.util.isNullOrUndefined(chartSeries.points[k].isEmpty)) ? false : chartSeries.points[k].isEmpty;
                chartSeries.points[k].visible = (ej.util.isNullOrUndefined(chartSeries.points[k].visible)) ? true : chartSeries.points[k].visible;
                if (ej.util.isNullOrUndefined(chartSeries.points[k].X)) {
                    chartSeries.points[k].isEmpty = true;
                    chartSeries.points[k].visible = false;
                    chartSeries.points[k].X = chartSeries.xAxis.visibleRange.min;
                } else
                    chartSeries.points[k].isEmpty = isEmpty;
                if ((chartSeries.points[k].YValues[0] == null) || (chartSeries.points[k].YValues[0] == 'undefined')) {
                    chartSeries.points[k].isEmpty = true;
                    chartSeries.points[k].visible = false;
                    if (ej.util.isNullOrUndefined(chartSeries.points[k].YValues))
                        chartSeries.points[k].YValues = [];
                    chartSeries.points[k].YValues[0] = chartSeries.yAxis.visibleRange.min;
                } else {
                    chartSeries.points[k].isEmpty = chartSeries.points[k].isEmpty;
                    chartSeries.points[k].visible = (chartSeries.points[k].isEmpty) ? !chartSeries.points[k].isEmpty : chartSeries.points[k].visible;
                }
                if (jQuery.type(chartSeries.points[k].X) == "date")
                    chartSeries.points[k].X = (chartSeries.points[k].X).getTime();

            }
            return chartSeries.points;
        },
        _pieVisiblePoints: function (currentseries) {
            var visiblePoints = [];
            var visibleLabels = [];
            var point;
            for (var i = 0; i < currentseries.points.length; i++) {
                point = currentseries.points[i];
                if (ej.util.isNullOrUndefined(point.visible))
                    point.visible = true;
                if (point.y < 0) {
                    point.YValues[0] = Math.abs(point.YValues[0])
                }
                if (point.visible == true && !ej.util.isNullOrUndefined(point.y)) {
                    visiblePoints.push(point);
                    visibleLabels.push(point.text ? point.text : point.y);
                }
            }
            currentseries.visiblePoints = visiblePoints;
            currentseries.visibleLabels = visibleLabels;
            return visiblePoints;
        },
        calculatingSliceAngle: function (currentseries) {
            var visiblePoints = this._pieVisiblePoints(currentseries);
            var legXSpace = 0;
            var legYSpace = 0;
            this.chartObj.model.arcData = [];
            currentseries.rightsidePoints = [];
            currentseries.leftsidePoints = [];
            currentseries.labels = [];
            this.chartObj.model.chartRegions = [];
            this.chartObj.model.sumofYValues = 0;
            this.chartObj.model.midPoint = 0;
            this.chartObj.model.centerX = 0, this.chartObj.model.centerY = 0;
            this.chartObj.model.startX = [], this.chartObj.model.startY = [];
            var point;
            this.chartObj.model.Radius = 0;
            var itemCurrentXPos = 0;

            for (j = 0; j < visiblePoints.length; j++) {
                this.chartObj.model.sumofYValues += visiblePoints[j].YValues[0];
            }
            if (currentseries.startAngle) {
                if (currentseries.startAngle > 360 || currentseries.startAngle == 360)
                    currentseries.startAngle = currentseries.startAngle % 360;
                else if (currentseries.startAngle < 0) {
                    if (currentseries.startAngle < -360 || currentseries.startAngle == -360)
                        currentseries.startAngle = currentseries.startAngle % 360;
                    currentseries.startAngle += 360;
                }
                itemCurrentXPos = (currentseries.startAngle) ? ((this.chartObj.model.sumofYValues / 360) * currentseries.startAngle) : 0;
                itemCurrentXPos = itemCurrentXPos / this.chartObj.model.sumofYValues;
                this.chartObj.model.itemCurrentXPos = itemCurrentXPos;
            }

            var legend = this.chartObj.model.legend;
            if (legend.visible && legend.position.toLowerCase() != "custom") {
                if (legend.position.toLowerCase() == "right" || legend.position.toLowerCase() == "left")
                    legXSpace = ((legend.position.toLowerCase() == "right") ? this.chartObj.model.margin.right : this.chartObj.model.margin.left) + this.chartObj.model.LegendBounds.Width;
                else
                    legYSpace = ((legend.position.toLowerCase() == "top") ? this.chartObj.model.margin.top : this.chartObj.model.margin.bottom) + this.chartObj.model.LegendBounds.Height;

            }

            var yOffset = ((this.chartObj.model.title.text) ? this.chartObj.model._titleLocation.Y : 0) + legYSpace;

            var actualWidth = $(this.chartObj.svgObject).width() - legXSpace;
            var actualHeight = $(this.chartObj.svgObject).height() - yOffset;

            this.chartObj.model.centerX = actualWidth * 0.5 + ((legend.position.toLowerCase() === "left") ? legXSpace : 0);
            this.chartObj.model.centerY = actualHeight * 0.5 + ((legend.position.toLowerCase() === "top") ? yOffset : ((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y) : 0));

            for (var l = 0; l < currentseries.visiblePoints.length; l++) {
                point = currentseries.visiblePoints[l];
                var pointIndex = l;
                point.startAngle = 2 * Math.PI * itemCurrentXPos;
                point.endAngle = 2 * Math.PI * (itemCurrentXPos + (point.YValues[0] / this.chartObj.model.sumofYValues));
                point.midAngle = (point.endAngle + point.startAngle) / 2;
                point.pointIndex = pointIndex;
                var chartStartingAngle = -.5 * Math.PI;

                point.radian = (point.midAngle) % (2 * Math.PI)

                if (currentseries.labelPosition.toLowerCase() == "outsideextended") {

                    if (point.radian < Math.PI)
                        currentseries.rightsidePoints.push(point);
                    else
                        currentseries.leftsidePoints.push(point);
                }
                itemCurrentXPos += point.YValues[0] / this.chartObj.model.sumofYValues;

                var midAngle = point.midAngle + chartStartingAngle;

                if ((pointIndex == currentseries.explodeIndex || currentseries.explodeAll) && !this.chartObj.vmlRendering) {

                    this.chartObj.model.startX[pointIndex] = this.chartObj.model.centerX + Math.cos(midAngle) * currentseries.explodeOffset;
                    this.chartObj.model.startY[pointIndex] = this.chartObj.model.centerY + Math.sin(midAngle) * currentseries.explodeOffset;
                }
                else {
                    this.chartObj.model.startX[pointIndex] = this.chartObj.model.centerX;
                    this.chartObj.model.startY[pointIndex] = this.chartObj.model.centerY;
                }

            }

            if (currentseries.labelPosition.toLowerCase() == "outsideextended") {
                currentseries.rightsidePoints = ej.DataManager(currentseries.rightsidePoints, ej.Query().sortBy("radian")).executeLocal();
                currentseries.leftsidePoints = ej.DataManager(currentseries.leftsidePoints, ej.Query().sortBy("radian")).executeLocal();
            }
            if (currentseries.marker.dataLabel.template) {
                if (currentseries.labelPosition.toLowerCase() == "outsideextended") {
                    for (var i = 0; i < currentseries.rightsidePoints.length; i++) {
                        ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.rightsidePoints[i], currentseries.rightsidePoints[i].pointIndex, currentseries, false, this.chartObj);
                    }
                    for (var k = 0; k < currentseries.leftsidePoints.length; k++) {
                        ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.leftsidePoints[k], currentseries.leftsidePoints[k].pointIndex, currentseries, true, this.chartObj);
                    }

                }
                else {
                    for (var j = 0; j < currentseries.visiblePoints.length; j++) {
                        ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.visiblePoints[j], currentseries.visiblePoints[j].pointIndex, currentseries, true, this.chartObj);
                    }
                }

            }
            else {
                ej.EjSvgRender.utils._getSeriesMaxLabel(currentseries);
            }


            currentseries.finalSize = { width: actualWidth, height: actualHeight };
            return currentseries.finalSize;
        },
        _drawHiloPath: function (series, style, interior, direction, i, pointbounds) {


            var options = {
                'id': this.chartObj.svgObject.id + "_" + this.getSeriesName(series) + "_" + i,
                'fill': "none",
                'stroke-dasharray': series.dashArray,
                'stroke-width': style.borderWidth,
                'stroke': interior,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'opacity': series.opacity,
                'd': direction
            };
            this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);
            var valWidth, valHeight;
            x = (Math.min(pointbounds.point2.X, pointbounds.point1.X) + series.xAxis.x) - (style.borderWidth);
            y = (Math.min(pointbounds.point2.Y, pointbounds.point1.Y) + series.yAxis.y);
            if (pointbounds.point1.Y != pointbounds.point2.Y) {
                valWidth = style.borderWidth + (style.borderWidth / 2);
                valHeight = Math.abs(pointbounds.point1.Y - pointbounds.point2.Y);
            }
            else {
                valWidth = Math.abs(pointbounds.point2.X - pointbounds.point1.X);
                valHeight = style.borderWidth + (style.borderWidth / 2);
            }
            var bounds = { X: x, Y: y, Width: valWidth, Height: valHeight };
            ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, i);

        },
        isIntersec: function (other, currBounds) {
            if (currBounds.Bottom < other.Top || currBounds.Right < other.Left
                  || currBounds.Top > other.Bottom || currBounds.Left > other.Right) {
                return false;
            }
            return true;
        },
        IntersectWith: function (sender, point, currBounds) {
            var chartObj = this;
            var intersect = false;
            var bounds;
            if (point.radian > Math.PI && (sender.model.bounds["left"])) {
                bounds = (sender.model.bounds["left"]);
                if (bounds.length > 0) {
                    for (var i = point.index + 1; i < bounds.length; i++) {
                        var other = bounds[i];
                        if (other && this.isIntersec(other, currBounds)) {
                            return true;
                        }
                    }
                }

            }
            else {
                bounds = (sender.model.bounds["right"]) ? sender.model.bounds["right"] : sender.model.bounds;
                var chartObj = this;
                if (bounds.length > 0) {
                    for (var i = point.index - 1; i >= 0; i--) {
                        var other = bounds[i];
                        if (other && this.isIntersec(other, currBounds)) {
                            return true;
                        }
                    }
                }
            }
            return intersect;

        },


        _calculateArcData: function (sender, pointIndex, point, series) {
            var visiblePoints = series.visiblePoints;
            var chartObj = sender;
            var chartStartAngle = -.5 * Math.PI;
            var currBounds;
            var startAngle = point.startAngle + chartStartAngle;
            var endAngle = point.endAngle + chartStartAngle - 0.000001;

            var longArc = endAngle - startAngle < Math.PI ? 0 : 1;
            var midAngle = (startAngle + endAngle) / 2;
            var direction;
            var startX;
            var startY

            if ((pointIndex == series.explodeIndex || series.explodeAll) && !chartObj.vmlRendering) {

                startX = chartObj.model.centerX + Math.cos(midAngle) * series.explodeOffset;
                startY = chartObj.model.centerY + Math.sin(midAngle) * series.explodeOffset;
            }
            else {
                startX = chartObj.model.centerX;
                startY = chartObj.model.centerY;
            }


            var x1 = startX + chartObj.model.Radius * Math.cos(startAngle);
            var y1 = startY + chartObj.model.Radius * Math.sin(startAngle);

            var x2 = startX + chartObj.model.Radius * Math.cos(endAngle);
            var y2 = startY + chartObj.model.Radius * Math.sin(endAngle);

            if (series.type.toLowerCase() == "doughnut") {
                var dStartX = startX + chartObj.model.dRadius * Math.cos(startAngle);
                var dStartY = startY + chartObj.model.dRadius * Math.sin(startAngle);

                var dEndX = startX + chartObj.model.dRadius * Math.cos(endAngle);
                var dEndY = startY + chartObj.model.dRadius * Math.sin(endAngle);

                direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + chartObj.model.Radius + " " + chartObj.model.Radius + " " + "0" + " " + longArc + " " + "1" + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "A" + " " + chartObj.model.dRadius + " " + chartObj.model.dRadius + " " + "1" + " " + longArc + " " + "0" + " " + dStartX + " " + dStartY + " " + "z";
            }

            else {
                direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + chartObj.model.Radius + " " + chartObj.model.Radius + " " + "0" + " " + longArc + " " + "1" + " " + x2 + " " + y2 + " " + "L" + " " + startX + " " + startY + " " + "z";
            }

            // display label for pie and doughnut chart


            return { Direction: direction };

        },

        getXCordinate: function (x, radius, angle) {
            var x1 = x + radius * (Math.cos(angle));
            return x1;
        },

        getYCordinate: function (y, radius, angle) {
            var y1 = y + radius * (Math.sin(angle));
            return y1;
        },
        getDoubleRange: function (start, end) {
            var mstart;
            var mend;
            if (start > end) {
                mstart = end;
                mend = start;
            }
            else {
                mstart = start;
                mend = end;
            }

            var mdelta = mend - mstart;
            var mmedian = (mstart + mend) / 2;
            var misEmpty = isNaN(mstart) || isNaN(mend);

            return { Start: mstart, End: mend, Delta: mdelta, Median: mmedian, IsEmpty: misEmpty };

        },
        isDependentSeries: function (series) {
            var type = series.type.toLowerCase();
            if (!this.chartObj.model.requireInvertedAxes)
                return (type == ej.Chart.type.Candle || type == ej.Chart.type.Column || type == ej.Chart.type.Hilo || type == ej.Chart.type.HiloOpenClose || type == ej.Chart.type.StackingColumn || type == ej.Chart.type.RangeColumn) ? true : false;
            else
                return (type == ej.Chart.type.Bar || type == ej.Chart.type.StackingBar) ? true : false;
        },

        getPointXYOrgin: function (x, y, series) {
            var xvalue = ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x) * (series.xAxis.width);
            var yvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.height);
            return { X: xvalue, Y: yvalue };
        },
        getRectangle: function (x1, y1, x2, y2, series) {
            var pt1 = this.getPointXYOrgin(x1, y1, series);
            var pt2 = this.getPointXYOrgin(x2, y2, series);
            return ej.EjSvgRender.utils._correctRect(pt1.X, pt1.Y, pt2.X, pt2.Y);
        },
        calculateSides: function (point, sidebysideinfo) {
            var x1 = point.X + sidebysideinfo.Start;
            var x2 = point.X + sidebysideinfo.End;
            return { x1: x1, x2: x2 };
        },
        getSideBySidePositions: function (currentSeries) {

            var chart = this;
            var stackingGroup = [];
            var model = chart.chartObj.model;

            if (ej.util.isNullOrUndefined(currentSeries.position) || model.legendCollapsed) {
                var hAxes = (model.requireInvertedAxes) ? model.vAxes : model.hAxes;
                var vAxes = (model.requireInvertedAxes) ? model.hAxes : model.vAxes;

                $.each(hAxes, function (index, hAxis) {
                    var all = 0;
                    var seriesCollection = [];
                    for (j = 0; j < model._visibleSeries.length; j++) {
                        var series = model._visibleSeries[j];
                        if (series.visibility.toLowerCase() === 'visible') {
                            if (chart.isDependentSeries(series)) {
                                if (hAxis.axisName == series.xAxisName) {
                                    seriesCollection.push(series);
                                }
                            }
                        }
                    }
                    var vAxeseries = [];
                    for (var j = 0; j < seriesCollection.length; j++) {
                        var index = (model.requireInvertedAxes) ? seriesCollection[j].yAxis.columnIndex : seriesCollection[j].yAxis.rowIndex;
                        if (!vAxeseries[index])
                            vAxeseries[index] = [];
                        vAxeseries[index].push(seriesCollection[j]);
                    }

                    for (var i = 0; i < vAxeseries.length; i++) {
                        if (!ej.util.isNullOrUndefined(vAxeseries[i])) {
                            vAxeseries[i].all = 0;
                            for (var j = 0; j < vAxeseries[i].length; j++) {
                                series = vAxeseries[i][j];
                                if (series.type.toLowerCase().indexOf("stacking") != -1) {
                                    if (series.stackingGroup) {
                                        if (ej.util.isNullOrUndefined(stackingGroup[series.stackingGroup])) {
                                            vAxeseries[i].all++;
                                            series.position = vAxeseries[i].all;
                                            stackingGroup[series.stackingGroup] = vAxeseries[i].all;
                                        }
                                        else
                                            series.position = stackingGroup[series.stackingGroup];
                                    }
                                    else {
                                        if (ej.util.isNullOrUndefined(series.yAxis.position) || model.legendCollapsed) {
                                            vAxeseries[i].all++;
                                            series.position = vAxeseries[i].all;
                                            series.yAxis.position = vAxeseries[i].all;
                                            model.legendCollapsed = false;
                                        }
                                        else
                                            series.position = series.yAxis.position;
                                    }
                                }
                                else {
                                    vAxeseries[i].all++;
                                    series.position = vAxeseries[i].all;
                                }
                            }
                        }
                    }

                    for (var k = 0; k < seriesCollection.length; k++) {
                        var index = (seriesCollection[k].type.toLowerCase().indexOf("bar") != -1) ? seriesCollection[k].yAxis.columnIndex : seriesCollection[k].yAxis.rowIndex;
                        if (seriesCollection[k].type.toLowerCase().indexOf("stacking") != -1)
                            seriesCollection[k].all = vAxeseries[index].all;
                        else
                            seriesCollection[k].all = vAxeseries[index].length;
                    }

                });

            }
            return { all: currentSeries.all, pos: currentSeries.position };
        },

        getSideBySideInfo: function (series) {
            var pos;
            var all;

            var data = this.getSideBySidePositions(series);
            pos = data.pos;
            all = data.all;

            var seriesSpacing = 0;
            var pointSpacing = 0.3 - 0;
            var width = ej.EjSvgRender.utils.getMinPointsDelta(series, this.chartObj) * (1 - pointSpacing);
            var div = 1 / all;
            var loc = 0.5 - div * (pos - 1);

            var range = this.getDoubleRange(-loc, -loc + div);

            // multiplying with width  and scaling
            if (!range.IsEmpty) {
                range = this.getDoubleRange(range.Start * width, range.End * width);
                var radius = 0.5 * (1 - seriesSpacing) * range.Delta;
                var median = range.Median;
                range = this.getDoubleRange(median - radius, median + radius);
            }

            return range;

        },



        draw: function (chart, options) {

            this.chartObj = chart;

            //DrawLineGraph
            var lDirection;
            var sb = ej.EjSvgRender.utils._getStringBuilder();
            var currentseries = options;
            var style = this.setLineSeriesStyle(currentseries);
            if (currentseries.sorting)
                currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
            var visiblePoints = this.improveChartPerformance(currentseries);

            var firstPoint = null;
            var secondPoint;
            var firstIndex = -1;
            for (var i = 0; i < visiblePoints.length; i++) {
                secondPoint = visiblePoints[i];
                if (secondPoint.visible) {
                    if (firstPoint != null) {
                        if (visiblePoints.length > firstIndex + 1) {
                            var nextpoint = visiblePoints[firstIndex + 1];
                            var point1 = ej.EjSvgRender.utils._getPoint(firstPoint, currentseries);
                            var point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentseries);
                            sb.append("M" + " " + (point1.X) + " " + ((point1.Y)) + " " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " ");

                        }

                    }
                    firstPoint = secondPoint;
                    firstIndex = i;

                }
                else {
                    firstPoint = null;
                }

            }

            lDirection = sb.toString();

            this._drawLinePath(currentseries, style, lDirection);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);



        },
        _doLineSymbol: function (element, sec, val, series, invertedAxes) {
            var beginTime = parseInt(val * sec);
            var chartObj = this.chartObj;
            var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
            var centerX = box.x + (box.width / 2);
            var centerY = box.y + (box.height / 2);
            var scaleVal;
            var $ele = $(element);
            var rotate = chartObj.svgRenderer._getAttrVal($ele, "transform");
            rotate = !ej.util.isNullOrUndefined(rotate) ? rotate : " ";
            chartObj.svgRenderer._setAttr($ele, { "transform": "scale(0)" });
            $ele.delay(beginTime).animate(
                {
                    scale: 1
                },
                {
                    duration: 200,

                    step: function (now) {
                        scaleVal = now;
                        chartObj.svgRenderer._setAttr($ele, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" + rotate });

                    }
                }
            );
        },
        animateSymbol: function (element, delayInterval, series, invertedAxes) {
            var chartObj = this.chartObj;
            var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
            var centerX = box.x + (box.width / 2);
            var centerY = box.y + (box.height / 2);
            var scaleVal;
            var $ele = $(element);
            var rotate = chartObj.svgRenderer._getAttrVal($ele, "transform");
            rotate = !ej.util.isNullOrUndefined(rotate) ? rotate : " ";
            chartObj.svgRenderer._setAttr($ele, { "transform": "scale(0)" });
            $ele.delay(delayInterval).each(function () { this.scale = 0.5; }).animate(
                {
                    scale: 1
                },
                {
                    duration: 500,

                    step: function (now) {
                        scaleVal = now;
                        chartObj.svgRenderer._setAttr($ele, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" + (rotate) });
                    }
                }
            );
        },
        animateStackingRect: function (element, series, invertedAxes) {
            var chartObj = this.chartObj;

            var boxY = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, 0)) * (series.yAxis.height);
            var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);

            var centerX, centerY;
            var $element = $(element);
            if (chartObj.svgRenderer._getAttrVal($element, "plot")) {
                centerX = box.x;
                centerY = boxY;
            }
            var scaleVal;
            $element.animate(
                {
                    scale: 1
                },
                {
                    duration: 1000,

                    step: function (now) {
                        scaleVal = now;
                        chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + ") scale(1," + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" });
                    }
                });
        },
        animateRect: function (element, series) {
            var chartObj = this.chartObj;
            var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, false);
            var centerX, centerY;
            var $element = $(element);
            if (chartObj.svgRenderer._getAttrVal($element, "plot")) {
                if (chartObj.svgRenderer._getAttrVal($element, "plot") === "negative") {
                    centerX = box.x;
                    centerY = box.y;
                } else {
                    centerX = (box.x + box.width);
                    centerY = (box.y + box.height);
                }
            } else {
                centerX = (box.x + (box.width / 2));
                centerY = box.y + (box.height / 2);
            }
            var scaleVal;
            $element.animate(
                {
                    scale: 1
                },
                {
                    duration: 1000,

                    step: function (now) {
                        scaleVal = now;
                        chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + ") scale(1," + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" });
                    }
                });

        },
        animateBarRect: function (element, series, invertedAxes) {
            var chartObj = this.chartObj;
            var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
            var centerX, centerY;
            var $element = $(element);
            if (chartObj.svgRenderer._getAttrVal($element, "plot") === "negative") {
                centerX = box.x + box.width;
                centerY = box.y + box.height;
            } else {

                centerX = box.x;
                centerY = box.y;
            }
            var scaleVal;
            $element.animate(
                {
                    scale: 1
                },
                {
                    duration: 1000,

                    step: function (now) {
                        scaleVal = now;
                        chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ",1) translate(" + (-centerX) + " " + (-centerY) + ")" });
                    }
                });

        },
        animateStackingBar: function (element, series, invertedAxes) {
            var chartObj = this.chartObj;

            var boxX = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, 0)) * (series.yAxis.width);
            var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);

            var centerX, centerY;
            var $element = $(element);

            centerX = boxX;
            centerY = box.y;

            var scaleVal;
            $element.animate(
                {
                    scale: 1
                },
                {
                    duration: 1000,

                    step: function (now) {
                        scaleVal = now;
                        chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ",1) translate(" + (-centerX) + " " + (-centerY) + ")" });
                    }
                });
        },
        _doLineAnimation: function (series) {
            var chartObj = this.chartObj;
            var clipRect = $(chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id + "_ClipRect").children();
            $(clipRect).animate(
                { width: parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id + "_ClipRect").children(), "width")) },
                {
                    duration: 2000,

                    step: function (now) {
                        chartObj.svgRenderer._setAttr($(clipRect), { "width": now });
                    }
                });
            var elements = $(chartObj.gSymbolGroupEle).children().not("defs");
            var secondsPerPoint = 2000 / elements.length;
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
            }


        },
        doAnimation: function (series) {

            this._doLineAnimation(series);
        },

        _drawLinePath: function (series, style, direction) {

            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
            var transX = series.xAxis.x;
            var transY = series.yAxis.y;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var options = {
                'id': this.chartObj.svgObject.id + "_" + this.getSeriesName(series) + "_" + seriesIndex,
                'fill': 'none',
                'stroke-dasharray': series.dashArray,
                'stroke-width': series.width,
                'stroke': style.SeriesInterior,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'opacity': series.opacity,
                'd': direction
            };

            this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);


        },





        drawDataLabel: function (series, index, point, x, y) {
            // method for data label symbols - cartesianaxes
            var labelFormat = series.yAxis.labelFormat ? series.yAxis.labelFormat : "";
            var pointText = (point.text) ? point.text : point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
            var seriesIndex = $.inArray(series, this.chartObj.model.series);
            var textAlign, textBaseline;
            var marker = point.marker ? point.marker : series.marker;
            marker = $.extend(true, {}, series.marker, marker);
            var dataLabelFont = marker.dataLabel.font;
            var size, fontStyle, fontFamily;
            var type = this.chartObj.model.chartRegions[index] ? this.chartObj.model.chartRegions[index].type : "";
            size = ej.util.isNullOrUndefined(dataLabelFont.size) ? "11px" : dataLabelFont.size;
            fontStyle = ej.util.isNullOrUndefined(dataLabelFont.fontStyle) ? "Normal" : dataLabelFont.fontStyle;
            fontFamily = ej.util.isNullOrUndefined(dataLabelFont.fontFamily) ? "Segoe UI" : dataLabelFont.fontFamily;
            var labelfont = { size: size, fontStyle: fontStyle, fontFamily: fontFamily };
            var textOffset = ej.EjSvgRender.utils._measureText(pointText, null, labelfont);

            var position = this.textPosition(series, index, textOffset, type, x, y);

            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.Data = { Text: pointText, Location: { X: position.x, Y: position.y } };
            this.chartObj._trigger("displayTextRendering", commonEventArgs);

            var dataLabelOffset = marker.dataLabel.offset;
            if (series.marker.dataLabel.offset) {
                if (series.marker.dataLabel.textPosition == 'bottom')
                    commonEventArgs.Data.Location.Y = commonEventArgs.Data.Location.Y + dataLabelOffset;
                else
                    commonEventArgs.Data.Location.Y = commonEventArgs.Data.Location.Y - dataLabelOffset;
            }

            if (!commonEventArgs.Cancel) {
                var options = {
                    'id': this.chartObj.svgObject.id + '_SeriesText' + index + seriesIndex,
                    'x': commonEventArgs.Data.Location.X,
                    'y': commonEventArgs.Data.Location.Y,
                    'fill': dataLabelFont.color,
                    'font-size': dataLabelFont.size,
                    'font-family': dataLabelFont.fontFamily,
                    'font-style': dataLabelFont.fontStyle,
                    'font-weight': dataLabelFont.fontWeight,
                    'text-anchor': 'middle'
                };
                var margin = marker.dataLabel.margin;
                var width = textOffset.width + margin.left + margin.right;
                var height = textOffset.height + margin.top + margin.bottom;
                var location = { X: commonEventArgs.Data.Location.X, Y: commonEventArgs.Data.Location.Y };

                if (ej.util.isNullOrUndefined(series.marker.dataLabel.template)) {
                    if (marker.dataLabel.shape)
                        var element = marker.dataLabel.shape;
                    else if (series.marker.dataLabel.shape)
                        element = marker.dataLabel.shape;
                    else
                        element = "None";

                    $.each(this.chartObj.model.symbolShape, function (name) {
                        if (element.toLowerCase() == name.toLowerCase())
                            symbolName = name;
                    });

                    var xXalue = commonEventArgs.Data.Location.X - (margin.left) / 2 + (margin.right) / 2;
                    var yValue = commonEventArgs.Data.Location.Y - (margin.top) / 2 - (height / margin.top) + (margin.bottom) / 2;
                    this.dataLabelSymbol(seriesIndex, series, index, xXalue, yValue, width, height, symbolName, this.chartObj);
                    this.chartObj.svgRenderer.drawText(options, commonEventArgs.Data.Text, this.chartObj.gSeriesTextEle);
                }

                else {
                    this.drawLabelTemplate(series, point, index, location, this.chartObj);
                }

            }
        },

        //smart labels for pie and doughnut

        updateSmartLabelPosition: function (currentseries, point, pointIndex, bounds, midAngle, startPoint, size, sender) {
            var renderingPoints = [];
            var chartObj = sender;
            var seriesType = new ej.seriesTypes[currentseries.type]();
            var connectorDirection = "";

            var connectorX = bounds.connector.connectorX;
            var connectorY = bounds.connector.connectorY;
            var midX = bounds.midPoint.midX;
            var midY = bounds.midPoint.midY;
            var dMidX = bounds.doughnutMidPoint.dMidX;
            var dMidY = bounds.doughnutMidPoint.dMidY;
            var startX = startPoint.startX;
            var startY = startPoint.startY;
            var textGap = (currentseries.marker.dataLabel.template) ? 0 : 4;

            var outerX;
            var textOffset = currentseries.marker.dataLabel.connectorLine.height;
            var radius = chartObj.model.Radius;

            var isIntersectedLabel;
            var angle = midAngle;

            //calculation for inside labels
            if (currentseries.labelPosition.toLowerCase() == 'inside') {
                positionX = (midX + startX) / 2;
                positionY = (midY + startY) / 2;

                positionX = (midX + positionX) / 2;
                positionY = (midY + positionY) / 2;

                if (point.isIntersected && chartObj.model.bounds[pointIndex]) {
                    currBounds = chartObj.model.bounds[pointIndex];
                }
                else {
                    if (midX > startX)
                        currBounds = { X: positionX, Y: positionY, Left: positionX, Top: positionY, Right: positionX + size.width, Bottom: positionY + size.height };
                    else
                        currBounds = { X: positionX, Y: positionY, Left: positionX - size.width, Top: positionY, Right: positionX, Bottom: positionY + size.height };
                }

                do {
                    isIntersectedLabel = false;
                    if (!this.IntersectWith(chartObj, point, currBounds)) continue;
                    angle += 0.01;
                    point.isIntersected = true;
                    currBounds.Angle = angle;
                    if (midX > startX) {
                        currBounds.X = currBounds.Left = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                        currBounds.Right = currBounds.X + size.width;
                    }
                    else {
                        currBounds.X = currBounds.Right = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                        currBounds.Left = currBounds.Right - size.width;
                    }
                    currBounds.Y = currBounds.Top = seriesType.getYCordinate(startY, (radius) + textOffset, angle);
                    currBounds.Bottom = currBounds.Y + size.height;
                    isIntersectedLabel = true;
                } while (isIntersectedLabel)
                chartObj.model.bounds[pointIndex] = currBounds;

                if (point.isIntersected && chartObj.model.bounds[pointIndex].Angle) {

                    positionX = seriesType.getXCordinate(startX, (radius) + textOffset, chartObj.model.bounds[pointIndex].Angle);
                    positionY = seriesType.getYCordinate(startY, (radius) + textOffset, chartObj.model.bounds[pointIndex].Angle);

                    var connectorMidX = seriesType.getXCordinate(startX, (radius) + textOffset / 3, midAngle);
                    var connectorMidY = seriesType.getYCordinate(startY, (radius) + textOffset / 3, midAngle);
                    connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorMidX + " " + connectorMidY + "M" + " " + connectorMidX + " " + connectorMidY + "L" + " " + positionX + " " + positionY;

                }
            }

                //calculation for outside labels
            else if (currentseries.labelPosition.toLowerCase() == 'outside') {

                if (midX > startX)
                    currBounds = { X: connectorX, Y: connectorY, Left: connectorX, Top: connectorY, Right: connectorX + size.width, Bottom: connectorY + size.height };
                else
                    currBounds = { X: connectorX, Y: connectorY, Left: connectorX - size.width, Top: connectorY, Right: connectorX, Bottom: connectorY + size.height };

                do {
                    isIntersectedLabel = false;
                    if (!this.IntersectWith(chartObj, point, currBounds)) continue;
                    angle += 0.01;
                    point.isIntersected = true;
                    if (midX > startX) {
                        currBounds.X = currBounds.Left = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                        currBounds.Right = currBounds.X + size.width;
                    }
                    else {
                        currBounds.X = currBounds.Right = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                        currBounds.Left = currBounds.Right - size.width;
                    }

                    currBounds.Y = currBounds.Top = seriesType.getYCordinate(startY, (radius) + textOffset, angle);
                    currBounds.Bottom = currBounds.Y + size.height;
                    isIntersectedLabel = true;
                } while (isIntersectedLabel)

                chartObj.model.bounds[pointIndex] = currBounds;

                var connectorMidX = seriesType.getXCordinate(startX, (radius) + textOffset / 3, midAngle);
                var connectorMidY = seriesType.getYCordinate(startY, (radius) + textOffset / 3, midAngle);

                renderingPoints.push({ X: midX, Y: midY });
                renderingPoints.push({ X: connectorMidX, Y: connectorMidY });
                renderingPoints.push({ X: currBounds.X, Y: currBounds.Y });
                positionX = currBounds.X;
                positionY = currBounds.Y;
                if (textOffset > 0) {
                    if (midX < startX) {
                        outerX = currBounds.X - chartObj.model.elementSpacing;
                        positionX = outerX - textGap;
                    }
                    else {
                        outerX = currBounds.X + chartObj.model.elementSpacing;
                        positionX = outerX + textGap;
                    }
                    renderingPoints.push({ X: outerX, Y: currBounds.Y });
                }
                if (currentseries.marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                    seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
                }
                else {
                    connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorMidX + " " + connectorMidY + "M" + " " + connectorMidX + " " + connectorMidY + "L" + " " + currBounds.X + " " + currBounds.Y + "M" + " " + currBounds.X + " " + currBounds.Y + "L" + " " + outerX + " " + currBounds.Y;
                }

            }


                //calculation for outsideExtended labels
            else {
                var labelX = midX;
                var labelY = midY;

                midX = midX + (Math.cos((midAngle)) * (-(radius / 10)));
                midY = midY + (Math.sin((midAngle)) * (-(radius / 10)));

                var connectorHeight = radius / 4;

                renderingPoints.push({ X: midX, Y: midY });
                midX = midX + (Math.cos((midAngle)) * connectorHeight);
                midY = midY + (Math.sin((midAngle)) * connectorHeight);

                renderingPoints.push({ X: midX, Y: midY });

                var legendWidth = (chartObj.model.legend.visible && chartObj.model.legend.position.toLowerCase() == "left") ? (chartObj.model.margin.left + chartObj.model.LegendBounds.Width) : 0;
                var centerX = chartObj.model.centerX;
                var labelEdge;

                if (midX < startX) {

                    midX = ((centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0)
                                     ? centerX - radius - (connectorHeight * 2) + legendWidth
                                     : (currentseries.LeftLabelMaxWidth > labelX) ? labelX : currentseries.LeftLabelMaxWidth) - size.width / 2;

                    labelEdge = renderingPoints[1].X - currentseries.LeftLabelMaxWidth;

                    var height = (centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0) ? connectorHeight
                                 : (currentseries.LeftLabelMaxWidth > labelX) ? 0
                                 : (labelEdge > 3 * (connectorHeight / 2)) ? connectorHeight : labelEdge / 2;

                    connectorLineEdge = +size.width / 2;

                }
                else {
                    midX = ((centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width())
                   ? centerX + radius + (connectorHeight * 2) - legendWidth
                   : ((centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? labelX : $(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth) + size.width / 2;


                    labelEdge = ($(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth) - renderingPoints[1].X;

                    var height = (centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width()) ? connectorHeight
                                : ((centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? 0
                                : (labelEdge > 3 * (connectorHeight / 2)) ? connectorHeight : labelEdge / 2;
                    connectorLineEdge = -size.width / 2;
                }

                var distanceFromOrigin = (Math.sqrt(Math.pow(labelX - midX, 2) + Math.pow(labelY - midY, 2))) / 10;

                var isLeft = (midX < startX) ? true : false;
                midX = isLeft ? midX + distanceFromOrigin : midX - distanceFromOrigin;
                var index = isLeft ? 1 : -1;

                var bounds = isLeft ? chartObj.model.bounds["left"] : chartObj.model.bounds["right"];
                var currBounds = { X: midX, Y: midY, Left: midX, Top: midY, Right: midX + size.width, Bottom: midY + size.height, index: pointIndex };
                if (this.IntersectWith(chartObj, point, currBounds)) {
                    renderingPoints.push(midX < startX
                           ? { X: midX + height + connectorLineEdge, Y: midY }
                           : { X: midX - height + connectorLineEdge, Y: midY });
                    midY = bounds[point.index + index].Bottom + 2;

                    currBounds.Y = midY;
                    currBounds.Top = midY;
                    currBounds.Bottom = midY + size.height;

                }

                renderingPoints.push({ X: midX + connectorLineEdge, Y: midY });

                if (!isLeft)
                    chartObj.model.bounds["right"][point.index] = (currBounds);
                else
                    chartObj.model.bounds["left"][point.index] = (currBounds);


                connectorDirection = ej.EjSvgRender.utils._getStringBuilder();
                if (currentseries.marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                    seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
                }
                else {
                    for (k = 0; k < renderingPoints.length; k++) {
                        if (k == renderingPoints.length - 1)
                            connectorDirection.append(" ");
                        else
                            connectorDirection.append("M" + " " + (renderingPoints[k].X) + " " + (renderingPoints[k].Y) + " " + "L" + " " + (renderingPoints[k + 1].X) + " " + (renderingPoints[k + 1].Y) + " ");
                    }
                    connectorDirection = connectorDirection.toString();
                }
                positionX = renderingPoints[renderingPoints.length - 1].X + ((midX < startX) ? -textGap : textGap);
                positionY = renderingPoints[renderingPoints.length - 1].Y;
            }
            return { positionX: positionX, positionY: positionY, connectorDirection: connectorDirection };
        },

        updateLabelPosition: function (currentseries, point, pointIndex, bounds, midAngle, startPoint, size, sender) {

            var renderingPoints = [];
            var chartObj = sender;
            var seriesType = new ej.seriesTypes[currentseries.type]();
            var connectorDirection = "";

            var connectorX = bounds.connector.connectorX;
            var connectorY = bounds.connector.connectorY;
            var midX = bounds.midPoint.midX;
            var midY = bounds.midPoint.midY;
            var dMidX = bounds.doughnutMidPoint.dMidX;
            var dMidY = bounds.doughnutMidPoint.dMidY;
            var startX = startPoint.startX;
            var startY = startPoint.startY;
            var textGap = (currentseries.marker.dataLabel.template) ? 0 : 4;

            var outerX;
            var textOffset = currentseries.marker.dataLabel.connectorLine.height;
            var radius = chartObj.model.Radius;

            //calculation for outside labels

            renderingPoints.push({ X: midX, Y: midY });
            renderingPoints.push({ X: connectorX, Y: connectorY });
            positionX = connectorX;
            positionY = connectorY;

            if (currentseries.labelPosition.toLowerCase() == 'outside') {
                if (textOffset > 0) {
                    if (connectorX < startX) {
                        outerX = connectorX - chartObj.model.elementSpacing;
                        positionX = outerX - textGap;
                    }
                    else {
                        outerX = connectorX + chartObj.model.elementSpacing;
                        positionX = outerX + textGap;
                    }
                    renderingPoints.push({ X: outerX, Y: connectorY });
                }

                if (currentseries.marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                    seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
                }
                else {
                    connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorX + " " + connectorY + " " + "M" + " " + connectorX + " " + connectorY + "L" + " " + outerX + " " + connectorY;
                }

            }


                //calculation for outsideextended labels
            else if (currentseries.labelPosition.toLowerCase() == 'outsideextended') {

                labelX = midX;
                labelY = midY;

                midX = midX + (Math.cos((midAngle)) * (-(radius / 10)));
                midY = midY + (Math.sin((midAngle)) * (-(radius / 10)));


                var renderingPoints = [];

                var connectorHeight = radius / 4;

                renderingPoints.push({ X: midX, Y: midY });
                connectorX = midX + (Math.cos((midAngle)) * connectorHeight);
                connectorY = midY + (Math.sin((midAngle)) * connectorHeight);

                renderingPoints.push({ X: connectorX, Y: connectorY });

                var legendWidth = (chartObj.model.legend.visible && (chartObj.model.legend.position.toLowerCase() == "left" || chartObj.model.legend.position.toLowerCase() == "right")) ? (chartObj.model.elementSpacing) : 0;

                if (midX < startX) {
                    connectorX = (chartObj.model.centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0)
                                         ? chartObj.model.centerX - radius - (connectorHeight * 2) + legendWidth
                                         : (currentseries.LeftLabelMaxWidth > labelX) ? labelX : currentseries.LeftLabelMaxWidth;
                }
                else {
                    connectorX = (chartObj.model.centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width())
                         ? chartObj.model.centerX + radius + (connectorHeight * 2) - legendWidth
                         : ((chartObj.model.centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? labelX : $(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth;

                }

                renderingPoints.push({ X: connectorX, Y: connectorY });

                connectorDirection = ej.EjSvgRender.utils._getStringBuilder();
                if (currentseries.marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                    seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
                }
                else {
                    for (k = 0; k < renderingPoints.length; k++) {
                        if (k == renderingPoints.length - 1)
                            connectorDirection.append(" ");
                        else
                            connectorDirection.append("M" + " " + (renderingPoints[k].X) + " " + (renderingPoints[k].Y) + " " + "L" + " " + (renderingPoints[k + 1].X) + " " + (renderingPoints[k + 1].Y) + " ");
                    }
                    connectorDirection = connectorDirection.toString();
                }
                positionX = renderingPoints[renderingPoints.length - 1].X + ((midX < startX) ? -textGap : textGap);;
                positionY = renderingPoints[renderingPoints.length - 1].Y;
            }

                //calculation for inside labels

            else {
                if (currentseries.type.toLowerCase() == "doughnut") {
                    positionX = (midX + dMidX) / 2;
                    positionY = (midY + dMidY) / 2;
                }
                else {
                    positionX = (midX + startX) / 2;
                    positionY = (midY + startY) / 2;

                    positionX = (midX + positionX) / 2;
                    positionY = (midY + positionY) / 2;
                }
            }

            return { positionX: positionX, positionY: positionY, connectorDirection: connectorDirection };
        },

        drawDataLabelAcc: function (sender, currentseries, pointIndex, point) {
            // method for data label symbols - accumulation series
            var chartObj = sender;
            var seriesType = new ej.seriesTypes[currentseries.type]();
            var outerX;
            var connectorDirection;
            var positionX = 0;
            var positionY = 0;
            var pointColor;
            var numberToFixed = ej.util.isNullOrUndefined(chartObj.model.roundingPlaces) ? 2 : chartObj.model.roundingPlaces;

            var _labelPosition = currentseries.labelPosition.toLowerCase();


            var text = (Math.round(point.YValues[0]) == point.YValues[0]) ? point.YValues[0] : point.YValues[0].toFixed(numberToFixed);
            var labelfont = { size: currentseries.marker.dataLabel.font.size, fontStyle: currentseries.marker.dataLabel.font.fontStyle, fontFamily: currentseries.marker.dataLabel.font.fontFamily };
            var textsize = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(point.text) ? text : point.text), null, labelfont);

            var margin = currentseries.marker.dataLabel.margin;

            var textWidth = textsize.width + margin.left + margin.right;
            var textHeight = textsize.height + margin.top + margin.bottom;

            if (currentseries.type.toLowerCase() == "pyramid") {
                var measureTitle = (chartObj.model.title.text) ? ej.EjSvgRender.utils._measureText(chartObj.model.title.text, $(this.svgObject).width() - chartObj.model.margin.left - chartObj.model.margin.right, chartObj.model.title.font) : 0;
                var pyrX = chartObj.model.elementSpacing + chartObj.model.margin.left +
                           (chartObj.model.legend.position.toLowerCase() === "left") ?
                           (chartObj.model.LegendBounds.Width) : 0;
                var pyrY = ((chartObj.model.legend.position.toLowerCase() === "top")
                            ? (chartObj.model.LegendBounds.Height) : 0) +
                            ((chartObj.model.title.text) ? (chartObj.model._titleLocation.Y + measureTitle.height) : (chartObj.model.margin.top + chartObj.model.elementSpacing));
                textWidth = textWidth + 10;
                var Location = { X: point.xLocation + pyrX, Y: point.yLocation + pyrY };
                positionX = point.xLocation;
                positionY = point.yLocation;
                var marker = point.marker ? point.marker : currentseries.marker;
                marker = $.extend(true, {}, currentseries.marker, marker);
                connectorDirection = point.connectorLine;

            }

            if (currentseries.type.toLowerCase() != "pyramid") {
                var textOffset;
                var radius = chartObj.model.Radius;

                if (ej.util.isNullOrUndefined(currentseries.marker.dataLabel.connectorLine.height)) {
                    text = ej.util.isNullOrUndefined(point.text) ? point.y : point.text;
                    currentseries.marker.dataLabel.connectorLine.height = ej.EjSvgRender.utils._measureText(text, null, currentseries.marker.dataLabel.font).height;
                }

                textOffset = currentseries.marker.dataLabel.connectorLine.height;

                var chartStartingAngle = -.5 * Math.PI; // to move the arc to 90 degree from the start.
                var renderingPoints = [];

                var startX, startY;

                var midAngle = point.midAngle + chartStartingAngle;
                if ((pointIndex == currentseries.explodeIndex || currentseries.explodeAll) && !chartObj.vmlRendering) {

                    startX = chartObj.model.centerX + Math.cos(midAngle) * currentseries.explodeOffset;
                    startY = chartObj.model.centerY + Math.sin(midAngle) * currentseries.explodeOffset;
                }
                else {
                    startX = chartObj.model.startX[pointIndex];
                    startY = chartObj.model.startY[pointIndex];
                }
                var midX = seriesType.getXCordinate(startX, (radius), midAngle);
                var midY = seriesType.getYCordinate(startY, (radius), midAngle);

                var dMidX = seriesType.getXCordinate(startX, (chartObj.model.dRadius), midAngle);
                var dMidY = seriesType.getYCordinate(startY, (chartObj.model.dRadius), midAngle);

                var connectorX = this.getXCordinate(startX, (radius) + textOffset, midAngle);
                var connectorY = this.getYCordinate(startY, (radius) + textOffset, midAngle);

                if (currentseries.marker.dataLabel.template) {
                    size = point.size;
                }
                else {
                    var marker = point.marker ? point.marker : currentseries.marker;
                    marker = $.extend(true, {}, currentseries.marker, marker);
                    var shape = marker.dataLabel.shape;
                    size = ej.EjSvgRender.utils._measureText(currentseries.visibleLabels[pointIndex], $(chartObj.svgObject).width(), currentseries.marker.dataLabel.font);
                    if (shape.toLowerCase() != "none") {
                        var margin = currentseries.marker.dataLabel.margin;
                        size.height = size.height + margin.top + margin.bottom;
                        size.width = size.width + margin.left + margin.right;
                    }
                }

                var midPoint = { midX: midX, midY: midY };
                var doughnutMidPoint = { dMidX: dMidX, dMidY: dMidY };
                var connector = { connectorX: connectorX, connectorY: connectorY };
                var bounds = { midPoint: midPoint, doughnutMidPoint: doughnutMidPoint, connector: connector };
                var startPoint = { startX: startX, startY: startY };

                if (!currentseries.enableSmartLabels) {
                    position = seriesType.updateLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, size, sender);
                    positionX = position.positionX;
                    positionY = position.positionY;
                    connectorDirection = position.connectorDirection;
                }
                    // smart labels

                else {

                    position = seriesType.updateSmartLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, size, sender);
                    positionX = position.positionX;
                    positionY = position.positionY;
                    connectorDirection = position.connectorDirection;
                }



            }
            var textAnchor = ((_labelPosition != 'inside' && _labelPosition != 'insidenooverlap') || (currentseries.type.toLowerCase() == "pyramid" && _labelPosition == 'outside')) ? ((positionX < chartObj.model.centerX) ? 'end' : 'start') : 'middle';
            var width = (_labelPosition == 'inside' || _labelPosition == 'insidenooverlap') ? 0 : (positionX < chartObj.model.centerX) ? -textsize.width : textsize.width;
            if (!currentseries.marker.dataLabel.template) {
                var textOptions = {
                    'id': chartObj.svgObject.id + currentseries.name + "text" + pointIndex,
                    'x': positionX,
                    'y': positionY + textsize.height / 4,
                    'fill': currentseries.marker.dataLabel.font.color,
                    'font-size': currentseries.marker.dataLabel.font.size,
                    'font-family': currentseries.marker.dataLabel.font.fontFamily,
                    'font-style': currentseries.marker.dataLabel.font.fontStyle,
                    'font-weight': currentseries.marker.dataLabel.font.fontWeight,
                    'opacity': currentseries.marker.dataLabel.font.opacity,
                    'text-anchor': textAnchor
                };
                chartObj.svgRenderer.drawText(textOptions, ((ej.util.isNullOrUndefined(point.text) ? text : point.text)), chartObj.gSeriesTextEle);
            }

            if (currentseries.marker.dataLabel.template) {
                Location = ej.util.isNullOrUndefined(Location) ? { X: positionX, Y: positionY } : Location;
                seriesType.drawLabelTemplate(currentseries, point, pointIndex, Location, sender);
            }

            else {
                $.each(chartObj.model.symbolShape, function (name) {
                    var marker = point.marker ? point.marker : currentseries.marker;
                    marker = $.extend(true, {}, currentseries.marker, marker);
                    var shape = marker.dataLabel.shape;
                    if (shape.toLowerCase() == name.toLowerCase())
                        symbolName = name;
                });

                positionX = positionX - (margin.left) / 2 + (margin.right) / 2 + width / 2;
                positionY = positionY - (margin.top) / 2 + (margin.bottom) / 2;

                seriesType.dataLabelSymbol(0, currentseries, pointIndex, positionX, positionY, textWidth, textHeight, symbolName, chartObj);

            }
            pointColor = jQuery.type(chartObj.model.pointColors[pointIndex]) == "array" ? chartObj.model.pointColors[pointIndex][0].color : chartObj.model.pointColors[pointIndex];
            if ((currentseries.marker.dataLabel.connectorLine.type.toLowerCase() != 'bezier' || chartObj.vmlRendering || currentseries.type.toLowerCase() == "pyramid") && (currentseries.labelPosition != 'inside')) {
                var connectorOptions = {
                    'id': chartObj.svgObject.id + currentseries.name + "line" + pointIndex,
                    'stroke': (currentseries.marker.dataLabel.connectorLine.color) ? currentseries.marker.dataLabel.connectorLine.color : pointColor,
                    'stroke-width': currentseries.marker.dataLabel.connectorLine.width,
                    'd': connectorDirection
                };
                chartObj.svgRenderer.drawPath(connectorOptions, chartObj.gSeriesConnectorEle);
            }

        },
        drawBezierSegment: function (controlpoints, currentseries, pointIndex, sender) {
            var polyLine = ej.EjSvgRender.utils._getStringBuilder();
            var seriesType = new ej.seriesTypes[currentseries.type]();
            for (var i = 0; i <= 16; i++) {
                var t = i / 16;
                var points = seriesType.GetBezierPoint(t, controlpoints, 0, controlpoints.length, currentseries);
                polyLine.append(points.X + "," + points.Y + " ");
            }
            var connectorOptions = {
                'id': sender.svgObject.id + currentseries.name + "line" + pointIndex,
                'stroke': (currentseries.marker.dataLabel.connectorLine.color) ? currentseries.marker.dataLabel.connectorLine.color : sender.model.pointColors[pointIndex],
                'stroke-width': currentseries.marker.dataLabel.connectorLine.width,
                'fill': 'none',
                'points': polyLine.toString()
            };
            sender.svgRenderer.drawPolyline(connectorOptions, sender.gSeriesConnectorEle);
        },
        GetBezierPoint: function (t, controlPoints, index, count, currentseries) {
            var seriesType = new ej.seriesTypes[currentseries.type]();
            if (count == 1)
                return controlPoints[index];
            var p0 = seriesType.GetBezierPoint(t, controlPoints, index, count - 1, currentseries);
            var p1 = seriesType.GetBezierPoint(t, controlPoints, index + 1, count - 1, currentseries);

            var X = (1 - t) * p0.X + t * p1.X;
            var Y = (1 - t) * p0.Y + t * p1.Y;

            return { X: X, Y: Y };


            // return new Point((1 - t) * p0.X + t * p1.X, (1 - t) * p0.Y + t * p1.Y);
        },

        drawLabelTemplate: function (series, point, pointIndex, location, sender) {
            // method for data label template
            if (sender)
                this.chartObj = sender;
            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
            var areaType = this.chartObj.model.AreaType;
            var currentSeries = this.chartObj.model.series[seriesIndex];
            var xPosition = 0, yPosition = 0;
            var chartContainer = this.chartObj._id;
            var marker = point.marker ? point.marker : series.marker;
            marker = $.extend(true, {}, series.marker, marker);
            var style = { 'interior': series.marker.dataLabel.fill, 'opacity': series.marker.dataLabel.opacity, 'borderColor': series.marker.dataLabel.border.color, 'borderWidth': series.marker.dataLabel.border.width };
            var color, width, height;
            var type = this.chartObj.model.chartRegions[pointIndex] ? this.chartObj.model.chartRegions[pointIndex].type : "";
            if (areaType == "cartesianaxes") {

                if ($('#template_group_' + chartContainer).length != 0)
                    var templateContainer = $('#template_group_' + chartContainer);
                else
                    templateContainer = $("<div></div>").attr('id', "template_group_" + chartContainer);

                templateContainer.css('position', 'relative').css('z-index', 1000);
                var cloneNode = $("#" + series.marker.dataLabel.template).clone();
                $(cloneNode).attr("id", series.marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
                var $cloneNode = $(cloneNode);
                $cloneNode.css("position", "absolute");

                if (series.xAxis._valueType.toLowerCase() == "datetime")
                    point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
                if (series.xAxis._valueType.toLowerCase() == "category")
                    point.x = ej.EjSvgRender.utils._getLabelContent(pointIndex, series.xAxis);

                point.count = 1;
                var data = { series: series, point: point };
                $cloneNode.html($cloneNode.html().parseTemplate(data));

                var display = (areaType == "cartesianaxes" || !series.animation || series.type.toLowerCase() == "pyramid") ? "block" : "none";
                $cloneNode.css("display", display).appendTo($(templateContainer));
                $(templateContainer).appendTo('#' + chartContainer);

                var areaBound = this.chartObj.model.m_AreaBounds;
                xPosition = areaBound.X;
                yPosition = currentSeries.yAxis.y;
                width = $cloneNode.width() / 2;
                height = $cloneNode.height();
                if (ej.util.isNullOrUndefined(style) && style.interior)
                    color = style.interior;
                else
                    color = this.chartObj.model.seriesColors[seriesIndex];
            }
            else {
                if (ej.util.isNullOrUndefined(style) && style.interior)
                    color = style.interior;
                else
                    color = this.chartObj.model.pointColors[pointIndex];
                width = this.chartObj.model.centerX > location.X ? point.size.width : 0;
                $cloneNode = $('#template_group_' + chartContainer).find("#" + series.marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
                height = point.size.height / 2;
            }

            if (currentSeries.type == 'hilo' || currentSeries.type == 'hiloopenclose' || currentSeries.type == 'candle')
                color = currentSeries.points[pointIndex].interior;
            color = jQuery.type(color) == "array" ? color[0].color : color;

            if (type == "bar") {
                var left = location.X + xPosition - width;
                var top = location.Y + areaBound.Y - height;
            } else {
                var left = location.X + xPosition - width;
                var top = yPosition + location.Y - height;
            }

            $cloneNode.css("left", left).css("top", top).css("background-color", color);
            if (areaType == "cartesianaxes") {
                if ((left > xPosition + areaBound.Width || top > yPosition + areaBound.Height) || (this.chartObj.zoomed && (left < xPosition || top < yPosition || left > xPosition + areaBound.Width || top > yPosition + areaBound.Height)))
                    $cloneNode.css("display", "none");
            }
        },

        animateLabelTemplate: function (options) {
            // method to animate data label template
            var seriesIndex = $.inArray(options, this.chartObj.model._visibleSeries);
            var length = options.points.length;
            var secondsPerPoint = 1000;

            for (var i = 0; i < length; i++) {
                if (options.points[i].marker.dataLabel.visible) {
                    var ele = options.marker.dataLabel.template + '_' + seriesIndex + '_' + i + '_' + this.chartObj._id;
                    ele = $('#' + ele);
                    ele.css("display", "none");
                    ele.delay(secondsPerPoint).fadeIn(300);
                }
                else
                    continue;
            }
        },

        textPosition: function (series, index, textOffset, type, x, y) {
            var horizontalTextAlignment = series.marker.dataLabel.horizontalTextAlignment;
            var verticalTextAlignment = series.marker.dataLabel.verticalTextAlignment;
            var lineHeight = 10;

            if (type == 'column' || type == 'stackingcolumn') {
                var length = this.chartObj.model.chartRegions.length - series.points.length;
                length = length + index;
                var pointHeight = this.chartObj.model.chartRegions[length].Region.Bounds.Height;
                if (series.marker.dataLabel.textPosition.toLowerCase() == "bottom")
                    y += pointHeight + textOffset.height / 4;
                else if (series.marker.dataLabel.textPosition.toLowerCase() == "middle")
                    y += pointHeight / 2 + (textOffset.height) / 4;
                else
                    y += (textOffset.height) / 4;

                if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                    x += textOffset.width + lineHeight;
                else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                    x -= ((textOffset.width) + lineHeight);

                if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                    y += textOffset.height / 3;
                else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                    y -= textOffset.height / 3;
            }

            else if (type == 'bar' || type == 'stackingbar') {
                length = this.chartObj.model.chartRegions.length - series.points.length;
                length = length + index;
                var pointWidth = this.chartObj.model.chartRegions[length].Region.Bounds.Width;
                y = y + textOffset.width / 3;
                if (series.marker.dataLabel.textPosition.toLowerCase() == "bottom")
                    x -= pointWidth + textOffset.width / 4;
                else if (series.marker.dataLabel.textPosition.toLowerCase() == "middle")
                    x -= pointWidth / 2 + textOffset.width / 2;
                else
                    x = x - textOffset.width / 4;

                if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                    x += textOffset.width - textOffset.width / 4;
                else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                    x -= textOffset.width / 4;

                if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                    y += textOffset.height;
                else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                    y -= textOffset.height;
            }
            else {
                if (series.marker.dataLabel.textPosition.toLowerCase() == "bottom")
                    y += (2 * textOffset.height + lineHeight);
                else if (series.marker.dataLabel.textPosition.toLowerCase() == "top")
                    y -= (lineHeight + (2 * textOffset.height) / 2);

                if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                    x += textOffset.width + lineHeight;
                else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                    x -= ((textOffset.width) + lineHeight);

                if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                    y += (textOffset.height + lineHeight);
                else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                    y -= (lineHeight + (textOffset.height) / 2);
            }
            var location = { x: x, y: y };
            return location;
        },

        drawSymbolImage: function (series, pointIndex) {
            var point = series.points[pointIndex];
            if (point.visible) {

                if (point.marker) {
                    if (!(point.marker.visible || series.marker.visible) || point.marker.shape != this.chartObj.model.symbolShape.image)
                        return;
                }


                var element = (point.marker) ? point : series;
                element = $.extend(true, {}, series.marker, element);
                if (element.shape.toLowerCase() == "image") {
                    var img = new Image();
                    img.src = element.imageUrl;
                    element.image = img;
                }
            }

        },

        changeCrossHairSymbol: function (element, track, ptIndex, serIndex, series) {
            if (!ej.util.isNullOrUndefined(track)) {
                var trackMarker = {};
                element = $.extend(true, {}, series.marker, element);
                trackMarker = $.extend(true, {}, this.chartObj.model.crosshair.marker, trackMarker);

                if (ej.util.isNullOrUndefined(trackMarker.shape)) {
                    trackMarker.shape = element.shape;
                }

                if ((!this.chartObj.model.crosshair.visible || ej.util.isNullOrUndefined(trackMarker.fill)) && series.type.toLowerCase() != "bubble" && series.type.toLowerCase() != "scatter") {
                    if (trackMarker.shape.toLowerCase() == "cross" || trackMarker.shape.toLowerCase() == "horizline" || trackMarker.shape.toLowerCase() == "vertline")
                        trackMarker.fill = jQuery.type(this.chartObj.model.seriesColors[serIndex]) == "array" ? this.chartObj.model.seriesColors[serIndex][0].color : this.chartObj.model.seriesColors[serIndex];
                    else
                        trackMarker.fill = element.border.color;
                }
                if (series.type.toLowerCase() == "bubble") {
                    var box = $(this.chartObj.gSeriesEle).find("#" + this.chartObj.svgObject.id + '_circlesymbol_' + ptIndex + "_" + serIndex)[0].getBoundingClientRect();
                    trackMarker.size.width = (box.right - box.left) / 2 + 15;
                    trackMarker.size.height = (box.bottom - box.top) / 2 + 15;
                } else {
                    if (trackMarker.size.width <= element.size.width)
                        trackMarker.size.width = element.size.width + 2;
                    if (trackMarker.size.height <= element.size.height)
                        trackMarker.size.height = element.size.height + 2;

                    if (ej.util.isNullOrUndefined(trackMarker.border.color) || !this.chartObj.model.crosshair.visible) {
                        if (!ej.util.isNullOrUndefined(element.fill))
                            trackMarker.border.color = jQuery.type(element.fill) == "array" ? element.fill._gradientStop[0].color : element.fill;
                        else
                            trackMarker.border.color = jQuery.type(this.chartObj.model.seriesColors[serIndex]) == "array" ? this.chartObj.model.seriesColors[serIndex][0].color : this.chartObj.model.seriesColors[serIndex];
                    }
                }
            }
            series._trackMarker = trackMarker;
            return trackMarker;


        },

        dataLabelSymbol: function (seriesIndex, series, pointIndex, x, y, width, height, symbolName, sender) {
            var chartObj = sender;
            if (symbolName == 'None')
                return;
            var point = series.points[pointIndex];
            var location = { startX: x, startY: y };
            var symbolEle = chartObj.gSymbolGroupEle;
            var element = point.marker ? point.marker : series.marker;
            var gradientName = "symbol";
            var style = { 'interior': series.marker.dataLabel.fill, 'opacity': series.marker.dataLabel.opacity, 'borderColor': series.marker.dataLabel.border.color, 'borderWidth': series.marker.dataLabel.border.width };
            var options = {
                'style': style,
                'gradientName': gradientName,
                'symbolEle': symbolEle,
                'width': width,
                'height': height,
                'point': point,
                'location': location,
                'seriesIndex': seriesIndex,
                'pointIndex': pointIndex,
                'element': element,
                'symbolName': symbolName,
                'id': chartObj.svgObject.id + '_dataLabel_' + seriesIndex + "_" + pointIndex
            };
            this.drawSymbolStyle(options, chartObj);
        },

        drawSymbolStyle: function (options, sender) {
            var chartObj = sender;
            var borderColor, borderWidth, opacity;
            var style = options.style, gradientName = options.gradientName, symbolEle = options.symbolEle, width = options.width;
            var height = options.height, point = options.point, location = options.location, seriesIndex = options.seriesIndex, visibility = options.visibility;
            var pointIndex = options.pointIndex, element = options.element, symbolName = options.symbolName;
            if (style != null) {
                if (style.borderColor != "")
                    borderColor = style.borderColor ? style.borderColor : "transparent";

                opacity = style.opacity;
                if (style.borderWidth)
                    borderWidth = style.borderWidth;
            }
            var colors = null;
            if (style.interior) {
                colors = ej.util.isNullOrUndefined(style.interior._gradientStop) ? style.interior : style.interior._gradientStop;
            }
            else {
                if (point.fill)
                    colors = ej.util.isNullOrUndefined(point.fill._gradientStop) ? point.fill : point.fill._gradientStop;
                else
                    colors = (chartObj.model.seriesColors[seriesIndex]) ? chartObj.model.seriesColors[seriesIndex] : chartObj.model.pointColors[pointIndex];
            }
            chartObj.symbolColorName = chartObj.svgRenderer.createGradientElement(gradientName + seriesIndex, colors, 0, 0, 0, $(chartObj.svgObject).height(), symbolEle);

            var symbolStyle = { ShapeSize: { width: width, height: height }, Style: { BorderColor: borderColor, BorderWidth: borderWidth, Opacity: opacity, Visibility: visibility, Color: chartObj.symbolColorName }, PointIndex: pointIndex, SeriesIndex: seriesIndex, Imageurl: element.imageUrl, Image: element.image, ID: options.id };

            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.Data = { Location: location, Style: symbolStyle };
            chartObj._trigger("symbolRendering", commonEventArgs);

            if (!commonEventArgs.Cancel)
                ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.Data.Location, commonEventArgs.Data.Style, chartObj, symbolEle);

            if (chartObj.model.AreaType == "none" && $(symbolEle).children().not("defs").length > chartObj.model._visibleSeries[seriesIndex].visiblePoints.length) {
                $(symbolEle.childNodes[options.pointIndex]).replaceWith($(symbolEle.childNodes[$(symbolEle.childNodes).length - 1]));
            }

        },

        drawSymbol: function (seriesIndex, series, pointIndex, x, y, chart, trackSymbol) {

            if (ej.util.isNullOrUndefined(this.chartObj)) this.chartObj = chart;


            var point = series.points[pointIndex];

            //if (point.marker) {
            //    if (!(point.marker.visible || series.marker.visible) || point.marker.shape == this.chartObj.model.symbolShape.None)
            //        return;
            //}

            var symbolName = "None";
            var tracker = this.changeCrossHairSymbol(point.marker ? point.marker : series.marker, trackSymbol, pointIndex, seriesIndex, series);
            var element = tracker ? tracker : point.marker ? point.marker : series.marker;
            //element = $.extend(true, {}, series.marker, element);
            //var element = point.marker ? point.marker : trackSymbol ? tracker : series.marker;

            $.each(this.chartObj.model.symbolShape, function (name) {
                if (element.shape.toLowerCase() == name.toLowerCase())
                    symbolName = name;
            });
            if (symbolName == "None")
                return;
            var location = { startX: x, startY: y };
            var style = { 'interior': element.fill, 'opacity': element.opacity, 'borderColor': element.border.color, 'borderWidth': element.border.width };

            var symbolEle = (trackSymbol) ? this.chartObj.gTrackerEle : this.chartObj.gSymbolGroupEle;
            var id = (trackSymbol) ? (this.chartObj.svgObject.id + '_trackSymbol_' + seriesIndex + "_" + pointIndex) : (this.chartObj.svgObject.id + '_symbol_' + seriesIndex + "_" + pointIndex);
            var gradientName = trackSymbol ? "TrackSymbol" : "symbol";
            var width = element.size.width;
            var height = element.size.height;
            var options = {
                'style': style,
                'gradientName': gradientName,
                'symbolEle': symbolEle,
                'width': width,
                'height': height,
                'point': point,
                'location': location,
                'seriesIndex': seriesIndex,
                'pointIndex': pointIndex,
                'element': element,
                'visibility': (point.visible) ? 'visible' : 'hidden',
                'symbolName': symbolName,
                'id': id
            };
            this.drawSymbolStyle(options, this.chartObj);

        },

        chartAreaType: "cartesianAxes",
        requireInvertedAxes: false,
        stackingSeries: false,
        hiloTypes: false

    };


    function ejExtendClass(parent, members) {
        var object = function () { };
        object.prototype = new parent();
        $.extend(object.prototype, members);
        return object;
    }

    ej.ejLineSeries = ejExtendClass(ej.EjSeriesRender);

    ej.seriesTypes.line = ej.ejLineSeries;


    ej.ejStepLineSeries = ejExtendClass(ej.EjSeriesRender, {

        draw: function (chart, options) {
            this.chartObj = chart;
            var lDirection;
            var sb = ej.EjSvgRender.utils._getStringBuilder();
            var currentseries = options;
            var style = this.setLineSeriesStyle(currentseries);
            if (currentseries.sorting)
                currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
            var visiblePoints = this.improveChartPerformance(currentseries);

            var firstPoint = null;
            var secondPoint;
            var firstIndex = -1;
            var xOffset = 0;
            if (currentseries.xAxis._valueType.toLowerCase() == "category" && currentseries.xAxis.labelPlacement.toLowerCase() != "onticks")
                xOffset = 0.5;
            for (var i = 0; i <= visiblePoints.length; i++) {
                //secondPoint = visiblePoints[i];
                if (i < visiblePoints.length && visiblePoints[i].visible) {
                    secondPoint = { X: visiblePoints[i].X - xOffset, YValues: [visiblePoints[i].y] };
                    point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                    if (firstPoint != null) {
                        if (visiblePoints.length > firstIndex + 1) {
                            var step = { X: secondPoint.X, YValues: [firstPoint.YValues[0]] };
                            var stepPoint = ej.EjSvgRender.utils._getPoint(step, currentseries);
                            sb.append("M" + " " + (point1.X) + " " + ((point1.Y)) + " " + "L" + " " + (stepPoint.X) + " " + (stepPoint.Y) + " " + "L" + " " + (point2.X) + " " + (point2.Y) + " ");
                        }
                    }
                    firstPoint = secondPoint;
                    point2 = point1;
                }
                else {
                    if (i == visiblePoints.length && xOffset > 0) {
                        secondPoint = { X: visiblePoints[i - 1].X + xOffset, YValues: [visiblePoints[i - 1].y] };
                        var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                        sb.append("M" + " " + (point1.X) + " " + ((point1.Y)) + " " + "L" + " " + (point2.X) + " " + (point2.Y) + " ");
                    }
                    else
                        firstPoint = null;
                }
            }
            lDirection = sb.toString();

            this._drawLinePath(currentseries, style, lDirection);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);




        },
        doAnimation: function (series) {

            this._doLineAnimation(series);
        }

    });

    ej.ejStepAreaSeries = ejExtendClass(ej.EjSeriesRender, {

        draw: function (chart, options) {
            this.chartObj = chart;
            var aDirection;
            var sb = ej.EjSvgRender.utils._getStringBuilder();
            var currentseries = options;
            var style = this.setAreaSeriesStyle(currentseries);
            if (currentseries.sorting)
                currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
            var visiblePoints = this.improveChartPerformance(currentseries);
            var xOffset = 0;
            var firstPoint = null;
            var secondPoint;
            if (currentseries.xAxis._valueType.toLowerCase() == "category" && currentseries.xAxis.labelPlacement.toLowerCase() != "onticks")
                xOffset = 0.5;
            var origin = Math.max(options.yAxis.visibleRange.min, 0);
            var startPoint = { X: visiblePoints[0].X, YValues: [origin] };
            var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentseries);
            for (var i = 0; i <= visiblePoints.length; i++) {
                if (i < visiblePoints.length && visiblePoints[i].visible) {
                    secondPoint = { X: visiblePoints[i].X - xOffset, YValues: [visiblePoints[i].y] };
                    point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                    if (firstPoint != null) {
                        var step = { X: secondPoint.X, YValues: [firstPoint.YValues[0]] };
                        var stepPoint = ej.EjSvgRender.utils._getPoint(step, currentseries);
                        if (i == 1) {
                            sb.append("M" + " " + (point2.X) + " " + (startLoc.Y) + " " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " ");
                        }
                        sb.append("L" + " " + (stepPoint.X) + " " + ((stepPoint.Y)) + " " + "L" + " " + (point1.X) + " " + (point1.Y) + " ");
                        if (xOffset == 0 && i == visiblePoints.length - 1) {
                            sb.append("L" + " " + point1.X + " " + (startLoc.Y) + " ");
                        }
                    }
                    point2 = point1;
                    firstPoint = secondPoint;

                }
                else {
                    if (i == visiblePoints.length && xOffset > 0) {
                        secondPoint = { X: visiblePoints[i - 1].X + xOffset, YValues: [visiblePoints[i - 1].y] };
                        var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                        sb.append("L" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (point1.X) + " " + (startLoc.Y) + " ");
                    }
                    else {
                        firstPoint = null;
                    }
                }
            }
            aDirection = sb.toString();

            this.drawAreaPath(currentseries, style, aDirection);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);



        },

        doAnimation: function (series) {

            this._doLineAnimation(series);
        }

    });
    ej.seriesTypes.steparea = ej.ejStepAreaSeries;


    ej.seriesTypes.stepline = ej.ejStepLineSeries;


    ej.ejColumnSeries = ejExtendClass(ej.EjSeriesRender, {



        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
            var origin = Math.max(options.yAxis.visibleRange.min, 0);

            var sidebysideinfo = this.getSideBySideInfo(series);
            var visiblePoints = this._isVisiblePoints(series);

            var transX = series.xAxis.x;
            var transY = series.yAxis.y;

            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

            var cSer = this;
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                var point = visiblePoints[i];

                var y1 = point.YValues[0];
                var y2 = origin;
                if (point.visible) {
                    //calculate sides
                    var data = cSer.calculateSides(point, sidebysideinfo);
                    var x1 = data.x1;
                    var x2 = data.x2;

                    var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                    var rect = cSer.getRectangle(x1, y1, x2, y2, series);

                    if (rect.X < 0) {
                        rect.Width = rect.Width + rect.X;
                        rect.X = 0;
                    }

                    rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
                    //drawing part
                    var xr = Math.min(0, rect.Width);
                    var yr = Math.min(0, rect.Height);

                    var bounds;
                    if (xr == 0 || yr == 0) {
                        options = {
                            'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                            'x': (rect.X),
                            'y': (rect.Y),
                            'width': rect.Width,
                            'height': rect.Height,
                            'fill': styleOptions.interior,
                            'stroke-width': styleOptions.borderWidth,
                            'plot': y1 < 0 ? "negative" : "positive",
                            'opacity': styleOptions.opacity,
                            'stroke': styleOptions.borderColor
                        };

                        cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                        var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                    }
                    if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                        point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                    }
                }
            }

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);


        },

        doAnimation: function (series) {
            var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this.animateRect(element, series);
            }
            var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
            for (var j = 0; j < symbolElements.length; j++) {
                var symbol = symbolElements[j];
                this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
            }
        }

    });
    ej.seriesTypes.column = ej.ejColumnSeries;

    ej.ejStackingColumnSeries = ejExtendClass(ej.EjSeriesRender, {

        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
            //var origin = Math.max(options.yAxis.visibleRange.min, 0);

            var sidebysideinfo = this.getSideBySideInfo(series);
            var visiblePoints = this._isVisiblePoints(series);

            var transX = series.xAxis.x;
            var transY = series.yAxis.y;

            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var cSer = this;
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                var point = visiblePoints[i];

                var y2 = series.stackedValue.StartValues[i];
                var y1 = series.stackedValue.EndValues[i];
                if (point.visible) {
                    //calculate sides
                    var data = cSer.calculateSides(point, sidebysideinfo);
                    var x1 = data.x1;
                    var x2 = data.x2;

                    var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                    var rect = cSer.getRectangle(x1, y1, x2, y2, series);
                    if (rect.X < 0) {
                        rect.Width = rect.Width + rect.X;
                        rect.X = 0;
                    }

                    rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
                    //drawing part
                    var xr = Math.min(0, rect.Width);
                    var yr = Math.min(0, rect.Height);

                    var bounds;
                    if (xr == 0 || yr == 0) {
                        options = {
                            'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                            'x': (rect.X),
                            'y': (rect.Y),
                            'width': rect.Width,
                            'height': rect.Height,
                            'fill': styleOptions.interior,
                            'stroke-width': styleOptions.borderWidth,
                            'plot': y1 < 0 ? "negative" : "positive",
                            'opacity': styleOptions.opacity,
                            'stroke': styleOptions.borderColor
                        };

                        cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                        var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                    }
                    if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                        point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                    }
                }
            }

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);



        },
        doAnimation: function (series) {
            var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this.animateStackingRect(element, series, this.chartObj.model.requireInvertedAxes);
            }
            var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
            for (var j = 0; j < symbolElements.length; j++) {
                var symbol = symbolElements[j];
                this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
            }
        },

        stackingSeries: true

    });
    ej.seriesTypes.stackingcolumn = ej.ejStackingColumnSeries;

    ej.seriesTypes.rangecolumn = ej.ejRangeColumnSeries;

    ej.ejRangeColumnSeries = ejExtendClass(ej.EjSeriesRender, {

        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);

            var sidebysideinfo = this.getSideBySideInfo(series);
            var visiblePoints = this._isVisiblePoints(series);

            var transX = series.xAxis.x;
            var transY = series.yAxis.y;

            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var cSer = this;
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                var point = visiblePoints[i];

                var y1 = point.YValues[0];
                var y2 = point.YValues[1];
                if (point.visible) {
                    //calculate sides
                    var data = cSer.calculateSides(point, sidebysideinfo);
                    var x1 = data.x1;
                    var x2 = data.x2;

                    var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                    var rect = cSer.getRectangle(x1, y1, x2, y2, series);
                    if (rect.X < 0) {
                        rect.Width = rect.Width + rect.X;
                        rect.X = 0;
                    }
                    var width = cSer.chartObj.model.m_AreaBounds.Width;
                    rect.Width = ((rect.X + rect.Width) > width) ? width - rect.X : rect.Width;
                    //drawing part
                    var xr = Math.min(0, rect.Width);
                    var yr = Math.min(0, rect.Height);

                    var bounds;
                    if (xr == 0 || yr == 0) {
                        options = {
                            'id': cSer.chartObj.svgObject.id + '_series' + seriesIndex + '_Point' + pointIndex,
                            'x': (rect.X),
                            'y': (rect.Y),
                            'width': rect.Width,
                            'height': rect.Height,
                            'fill': styleOptions.interior,
                            'stroke-width': styleOptions.borderWidth,
                            'opacity': styleOptions.opacity,
                            'stroke': styleOptions.borderColor
                        };

                        cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                        //Add region for each rect
                        var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                    }
                    //Add rect location details for symbol
                    if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                        point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                    }
                }
            }

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);



        },

        doAnimation: function (series) {
            var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this.animateRect(element, series);
            }
            var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
            for (var j = 0; j < symbolElements.length; j++) {
                var symbol = symbolElements[j];
                this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
            }
        },
        hiloTypes: true

    });
    ej.seriesTypes.rangecolumn = ej.ejRangeColumnSeries;

    ej.ejStackingBarSeries = ejExtendClass(ej.EjSeriesRender, {
        getPointXYOrgin: function (x, y, series) {
            var xvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x)) * (series.xAxis.height);
            var yvalue = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.width);
            return { X: yvalue, Y: xvalue };
        },
        getRectangle: function (x1, y1, x2, y2, series) {
            var pt1 = this.getPointXYOrgin(x1, y1, series);
            var pt2 = this.getPointXYOrgin(x2, y2, series);
            return ej.EjSvgRender.utils._correctRect(pt1.X, pt1.Y, pt2.X, pt2.Y);
        },
        calculateSides: function (point, sidebysideinfo) {
            var x1 = point.X + sidebysideinfo.Start;
            var x2 = point.X + sidebysideinfo.End;
            return { x1: x1, x2: x2 };
        },
        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
            //var origin = Math.max(options.yAxis.visibleRange.min, 0);
            var sidebysideinfo = this.getSideBySideInfo(series);
            var visiblePoints = this._isVisiblePoints(series);
            var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.chartObj.model.requireInvertedAxes);

            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var cSer = this;
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                var point = visiblePoints[i];
                var y1 = series.stackedValue.EndValues[i];
                var y2 = series.stackedValue.StartValues[i];
                if (point.visible) {
                    //calculate sides
                    var data = cSer.calculateSides(point, sidebysideinfo);
                    var x1 = data.x1;
                    var x2 = data.x2;

                    var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                    var rect = cSer.getRectangle(x1, y1, x2, y2, series);

                    var xr = Math.min(0, rect.Width);
                    var yr = Math.min(0, rect.Height);

                    if (xr == 0 || yr == 0) {
                        var bounds;
                        options = {
                            'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                            'x': (rect.X),
                            'y': (rect.Y),
                            'width': rect.Width,
                            'height': rect.Height,
                            'fill': styleOptions.interior,
                            'stroke-width': styleOptions.borderWidth,
                            'opacity': styleOptions.opacity,
                            'stroke': styleOptions.borderColor
                        };

                        cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                        var svgXy = ej.EjSvgRender.utils._getSvgXY((rect.X), (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                    }

                    if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                        point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                    }
                }
            }

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);


        },
        doAnimation: function (series) {
            var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this.animateStackingBar(element, series, this.chartObj.model.requireInvertedAxes);
            }
            var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
            for (var j = 0; j < symbolElements.length; j++) {
                var symbol = symbolElements[j];
                this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
            }
        },

        stackingSeries: true,
        requireInvertedAxes: true

    });
    ej.seriesTypes.stackingbar = ej.ejStackingBarSeries;

    ej.ejBarSeries = ejExtendClass(ej.EjSeriesRender, {
        getPointXYOrgin: function (x, y, orginX, orginY, series) {
            var xvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x)) * (series.xAxis.height);
            var yvalue = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.width);
            return { X: yvalue, Y: xvalue };
        },
        getRectangle: function (x1, y1, x2, y2, series) {
            var orginX = series.xAxis.visibleRange.min;
            var orginY = Math.max(series.yAxis.visibleRange.min, 0);
            var pt1 = this.getPointXYOrgin(x1, y1, orginX, orginY, series);
            var pt2 = this.getPointXYOrgin(x2, y2, orginX, orginY, series);
            return ej.EjSvgRender.utils._correctRect(pt1.X, pt1.Y, pt2.X, pt2.Y);
        },
        calculateSides: function (point, sidebysideinfo) {
            var x1 = point.X + sidebysideinfo.Start;
            var x2 = point.X + sidebysideinfo.End;
            return { x1: x1, x2: x2 };
        },
        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
            var origin = Math.max(options.yAxis.visibleRange.min, 0);
            var style = $.extend(true, {}, this.chartObj.model.seriesStyle, series.style);
            var sidebysideinfo = this.getSideBySideInfo(series);
            var visiblePoints = this._isVisiblePoints(series);
            var colors;
            var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.chartObj.model.requireInvertedAxes);

            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var cSer = this;
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                var point = visiblePoints[i];
                var y1 = point.YValues[0];
                var y2 = origin;
                if (point.visible) {
                    //calculate sides
                    var data = cSer.calculateSides(point, sidebysideinfo);
                    var x1 = data.x1;
                    var x2 = data.x2;

                    var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                    var rect = cSer.getRectangle(x1, y1, x2, y2, series);

                    var xr = Math.min(0, rect.Width);
                    var yr = Math.min(0, rect.Height);

                    if (xr == 0 || yr == 0) {
                        var bounds;
                        options = {
                            'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                            'x': (rect.X),
                            'y': (rect.Y),
                            'width': rect.Width,
                            'height': rect.Height,
                            'fill': styleOptions.interior,
                            'stroke-width': styleOptions.borderWidth,
                            'plot': y1 < 0 ? "negative" : "positive",
                            'opacity': styleOptions.opacity,
                            'stroke': styleOptions.borderColor
                        };

                        cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                        var svgXy = ej.EjSvgRender.utils._getSvgXY((rect.X), (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                    }


                    if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                        point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                    }
                }
            }

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);


        },
        doAnimation: function (series) {
            var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this.animateBarRect(element, series, this.chartObj.model.requireInvertedAxes);
            }
            var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
            for (var j = 0; j < symbolElements.length; j++) {
                var symbol = symbolElements[j];
                this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
            }
        },
        requireInvertedAxes: true
    });
    ej.seriesTypes.bar = ej.ejBarSeries;


    ej.ejStackingAreaSeries = ejExtendClass(ej.EjSeriesRender, {

        draw: function (chart, options) {
            this.chartObj = chart;
            var aDirection;
            var areasb = ej.EjSvgRender.utils._getStringBuilder();
            var currentSeries = options;
            var style = this.setAreaSeriesStyle(currentSeries);

            currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("X")).executeLocal();



            var visiblePoints = this._isVisiblePoints(currentSeries);

            var origin = Math.max(options.yAxis.visibleRange.min, 0);

            var startPoint = { X: visiblePoints[0].X, YValues: [origin] };
            var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
            areasb.append("M" + " " + (startLoc.X) + " " + (startLoc.Y) + " ");
            //var index = $.inArray(currentSeries, this.chartObj.model._visibleSeries);

            for (j = 0; j < currentSeries.stackedValue.EndValues.length; j++) {
                if (visiblePoints[j].visible) {
                    if (visiblePoints.length > j + 1) {
                        currentPoint = { X: visiblePoints[j].X, YValues: [currentSeries.stackedValue.EndValues[j]] };
                        nextpoint = { X: visiblePoints[j + 1].X, YValues: [currentSeries.stackedValue.EndValues[j + 1]] };

                        point1 = ej.EjSvgRender.utils._getPoint(currentPoint, currentSeries);
                        point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentSeries);

                        areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)) + " " + " " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " ");

                    }
                }
            }



            for (var j = currentSeries.stackedValue.StartValues.length - 1; j > 0; j--) {
                if (visiblePoints[j].visible) {

                    var currentPoint = { X: visiblePoints[j].X, YValues: [currentSeries.stackedValue.StartValues[j]] };
                    var nextpoint = { X: visiblePoints[j - 1].X, YValues: [currentSeries.stackedValue.StartValues[j - 1]] };

                    var point1 = ej.EjSvgRender.utils._getPoint(currentPoint, currentSeries);
                    var point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentSeries);

                    areasb.append((point1.X) + " " + ((point1.Y)) + " " + (point2.X) + " " + ((point2.Y)) + " ");
                }
            }


            aDirection = areasb.toString();

            this.drawAreaPath(currentSeries, style, aDirection);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);




        },

        doAnimation: function (series) {

            this._doLineAnimation(series);
        },
        stackingSeries: true

    });
    ej.seriesTypes.stackingarea = ej.ejStackingAreaSeries;


    ej.ejAreaSeries = ejExtendClass(ej.EjSeriesRender, {


        draw: function (chart, options) {
            this.chartObj = chart;
            var aDirection;
            var areasb = ej.EjSvgRender.utils._getStringBuilder();
            var currentSeries = options;
            var style = this.setAreaSeriesStyle(currentSeries);

            currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("X")).executeLocal();

            var visiblePoints = this._isVisiblePoints(currentSeries);
            var origin = Math.max(options.yAxis.visibleRange.min, 0);

            var startPoint = { X: visiblePoints[0].X, YValues: [origin] };
            var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
            areasb.append("M" + " " + (startLoc.X) + " " + (startLoc.Y) + " ");

            for (var i = 0; i < visiblePoints.length; i++) {
                if (visiblePoints[i].visible) {
                    if (visiblePoints.length > i + 1) {
                        if (!startPoint) {
                            startPoint = { X: visiblePoints[i].X, YValues: [origin] }
                            var point1 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                            areasb.append(" " + "M" + " " + (point1.X) + " " + ((point1.Y)));
                        }


                        var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);

                        areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)));

                        if (!visiblePoints[i + 1].visible) {
                            point = { X: visiblePoints[i].X, YValues: [origin] }
                            var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                            var point3 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                            areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " " + "L" + " " + (point3.X) + " " + ((point3.Y)));
                            startPoint = null;
                        }
                    }
                    else {
                        var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                        areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)));
                    }
                }
            }

            var endPoint = { X: visiblePoints[visiblePoints.length - 1].X, YValues: [origin] };
            var endLoc = ej.EjSvgRender.utils._getPoint(endPoint, currentSeries);

            areasb.append(" " + "L" + " " + (endLoc.X) + " " + ((endLoc.Y)) + " ");

            aDirection = areasb.toString();

            this.drawAreaPath(currentSeries, style, aDirection);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);




        },

        doAnimation: function (series) {

            this._doLineAnimation(series);
        }

    });

    ej.seriesTypes.area = ej.ejAreaSeries;

    ej.ejSplineSeries = ejExtendClass(ej.EjSeriesRender, {

        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            var spDirection = "";
            var splinesb = ej.EjSvgRender.utils._getStringBuilder();
            var style = this.setLineSeriesStyle(series);
            var yIndex = 0;
            var ySpline = this.naturalSpline(series.points);
            var firstPoint = null;
            var secondPoint = null;
            var firstIndex = -1;
            //Removed spline sorting behavior based on the 'X' points.
            //series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
            var visiblePoints = this._isVisiblePoints(series);
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                secondPoint = visiblePoints[i];
                if (secondPoint.visible) {
                    if (firstPoint != null) {
                        var controlPoint1 = null;
                        var controlPoint2 = null;
                        var data = this.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                        controlPoint1 = data["controlPoint1"];
                        controlPoint2 = data["controlPoint2"];
                        var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, series);
                        var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, series);
                        var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, series);
                        var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, series);
                        splinesb.append("M" + " " + (pt1.X) + " " + (pt1.Y) + " " + "C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");

                    }
                    firstPoint = secondPoint;
                    firstIndex = pointIndex;
                } else {
                    firstPoint = null;
                }
            }

            spDirection = splinesb.toString();
            this._drawLinePath(series, style, spDirection);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);



        },

        doAnimation: function (series) {

            this._doLineAnimation(series);
        }

    });

    ej.seriesTypes.spline = ej.ejSplineSeries;

    ej.ejSplineAreaSeries = ejExtendClass(ej.EjSeriesRender, {


        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            var spDirection = "";
            var splinesb = ej.EjSvgRender.utils._getStringBuilder();
            var style = this.setAreaSeriesStyle(series);
            var yIndex = 0;
            var ySpline = this.naturalSpline(series.points);
            var firstPoint = null;
            var secondPoint = null;
            var firstIndex = -1;
            //Removed splineArea sorting behavior based on the 'X' points.
            series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
            var visiblePoints = this._isVisiblePoints(series);
            var origin = Math.max(options.yAxis.visibleRange.min, 0);
            var startPoint = { X: visiblePoints[0].X, YValues: [origin] };
            var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, series);
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                secondPoint = visiblePoints[i];
                if (secondPoint.visible) {
                    if (firstPoint != null) {
                        var controlPoint1 = null;
                        var controlPoint2 = null;
                        var data = this.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                        controlPoint1 = data["controlPoint1"];
                        controlPoint2 = data["controlPoint2"];
                        var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, series);
                        var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, series);
                        var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, series);
                        var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, series);
                        if (pointIndex == 1) {
                            splinesb.append("M" + " " + (pt1.X) + " " + (startLoc.Y) + " " + "L" + " " + (pt1.X) + " " + ((pt1.Y)) + " ");
                        }
                        splinesb.append("C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");
                        if (pointIndex == visiblePoints.length - 1)
                            splinesb.append("L" + " " + (pt2.X) + " " + (startLoc.Y));
                    }
                    firstPoint = secondPoint;
                    firstIndex = pointIndex;
                } else {
                    firstPoint = null;
                }
            }

            spDirection = splinesb.toString();

            this.drawAreaPath(series, style, spDirection);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);




        },
        doAnimation: function (series) {

            this._doLineAnimation(series);
        }

    });

    ej.seriesTypes.splinearea = ej.ejSplineAreaSeries;



    ej.ejScatterSeries = ejExtendClass(ej.EjSeriesRender, {



        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            this._isVisiblePoints(series);
            if (!series.marker.visible)
                series.marker.visible = true;

        },

        doAnimation: function (series) {

            var elements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
            for (var i = 0; i < elements.length; i++) {
                var ubound = 20;
                var lbound = 0;
                var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
                var delayInterval = parseInt(randomValue * 50);
                this.animateSymbol(elements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
            }


        }

    });

    ej.seriesTypes.scatter = ej.ejScatterSeries;


    ej.ejBubbleSeries = ejExtendClass(ej.EjSeriesRender, {

        createBubbleGroup: function (series) {

            var seriesIndex = $.inArray(series, this.chartObj.model.series);
            var transX = series.xAxis.x;

            var transY = series.yAxis.y;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

        },
        draw: function (chart, options) {
            this.chartObj = chart;
            var series = options;
            series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
            var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
            var visiblePoints = this._isVisiblePoints(series);
            var sizeValueIndex = 1;
            var minRadius = (options.bubbleOptions && options.bubbleOptions.minRadius) ? options.bubbleOptions.minRadius : 10;
            var maxradius = (options.bubbleOptions && options.bubbleOptions.maxRadius) ? options.bubbleOptions.maxRadius : 30;
            var radius = maxradius - minRadius;
            var segmentRadius;
            var maximumSize = Math.max.apply(0, $.map(visiblePoints, function (v) { return v.YValues[sizeValueIndex]; }));
            this.createBubbleGroup(series);
            for (var i = 0; i < visiblePoints.length; i++) {
                var point = visiblePoints[i];
                var pointIndex = i;
                if (point.visible) {
                    segmentRadius = minRadius + radius * Math.abs(point.YValues[sizeValueIndex] / maximumSize);
                    var location = ej.EjSvgRender.utils._getPoint(point, series);

                    var styleOptions = this.chartObj.setStyle(this, series, seriesIndex, pointIndex);

                    var bubbleOptions = {
                        'id': this.chartObj.svgObject.id + '_circlesymbol_' + i + '_' + seriesIndex,
                        'cx': location.X,
                        'cy': (location.Y),
                        'r': segmentRadius,
                        'fill': styleOptions.interior,
                        'stroke-width': styleOptions.borderWidth,
                        'opacity': styleOptions.opacity,
                        'stroke': styleOptions.borderColor
                    };

                    this.chartObj.svgRenderer.drawCircle(bubbleOptions, this.gSeriesGroupEle);



                    var cx = location.X;
                    var cy = location.Y;
                    var r = segmentRadius;


                    var valwidth, x, y, valheight;
                    x = ((cx - r) + series.xAxis.x);
                    y = ((cy - r) + series.yAxis.y);

                    valheight = 2 * r;
                    valwidth = 2 * r;

                    var bounds = { X: x, Y: y, Width: valwidth, Height: valheight };
                    ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, i);
                }
            }
            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);



        },


        doAnimation: function (series) {
            var elements = $(this.gSeriesGroupEle).children().not("defs");

            for (var i = 0; i < elements.length; i++) {
                var ubound = 20;
                var lbound = 0;
                var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
                var delayInterval = parseInt(randomValue * 50);
                this.animateSymbol(elements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
            }
        }
    });

    ej.seriesTypes.bubble = ej.ejBubbleSeries;
    ej.ejhiloSeries = ejExtendClass(ej.EjSeriesRender, {
        draw: function (chart, options) {

            this.chartObj = chart;

            //DrawHiloGraph      

            var currentseries = options;
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
            var visiblePoints = this._isVisiblePoints(currentseries);
            var sidebysideinfo = this.getSideBySideInfo(currentseries);
            var point = null;

            var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
            var transX = currentseries.xAxis.x;
            var transY = currentseries.yAxis.y;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);


            for (var i = 0; i < visiblePoints.length; i++) {
                point = visiblePoints[i];
                var pointIndex = i;
                if (point.visible) {

                    var lDirection = "";
                    if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]))
                        continue;

                    var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex);

                    var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex, seriesIndex);


                    var lowvalue = {}, highvalue = {};
                    if (point.YValues[0] < point.YValues[1]) {
                        lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                        highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                    } else {
                        lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                        highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                    }
                    var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
                    var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                    lDirection = "M" + " " + (point1.X) + " " + ((point1.Y)) + " " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " ";
                    var pointbounds = { point1: point1, point2: point2 };
                    this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                }
            }

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);




        },

        doAnimation: function (series) {

            var elements = $(this.gSeriesGroupEle).children().not("defs");
            var secondsPerPoint = 2000 / elements.length;
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
            }
        },
        hiloTypes: true
    });

    ej.seriesTypes.hilo = ej.ejhiloSeries;

    //Draw HiloOpenClose
    ej.ejhiloopencloseSeries = ejExtendClass(ej.EjSeriesRender, {
        draw: function (chart, options) {

            this.chartObj = chart;

            //DrawHiloGraph      

            var currentseries = options;
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
            var visiblePoints = this._isVisiblePoints(currentseries);
            var sidebysideinfo = this.getSideBySideInfo(currentseries);
            var point = null;

            var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
            var transX = currentseries.xAxis.x;
            var transY = currentseries.yAxis.y;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

            var interior;




            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                point = visiblePoints[i];
                if (point.visible) {

                    var lDirection = "";
                    var drawOpenWing = (ej.util.isNullOrUndefined(currentseries.drawMode) ? true : (currentseries.drawMode.toLowerCase() == "both" || currentseries.drawMode.toLowerCase() == "open") ? true : false);
                    var drawCloseWing = (ej.util.isNullOrUndefined(currentseries.drawMode) ? true : (currentseries.drawMode.toLowerCase() == "both" || currentseries.drawMode.toLowerCase() == "close") ? true : false);


                    var lowvalue = {}, highvalue = {}, openvalue = {}, closevalue = {};
                    if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]) || ej.util.isNullOrUndefined(point.YValues[2]) || ej.util.isNullOrUndefined(point.YValues[3]))
                        continue;


                    if (point.YValues[0] < point.YValues[1]) {
                        lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                        highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                        openvalue = { X: point.X, YValues: [point.YValues[2]] };
                        closevalue = { X: point.X, YValues: [point.YValues[3]] };

                    } else {
                        lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                        highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                        openvalue = { X: point.X, YValues: [point.YValues[2]] };
                        closevalue = { X: point.X, YValues: [point.YValues[3]] };

                    }
                    if (point.YValues[2] < point.YValues[3])
                        interior = (ej.util.isNullOrUndefined(currentseries.bearFillColor)) ? "#339933" : currentseries.bearFillColor;

                    else
                        interior = (ej.util.isNullOrUndefined(currentseries.bullFillColor)) ? "#E51400" : currentseries.bullFillColor;


                    var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex, interior);

                    var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex, seriesIndex, interior);


                    if (point.fill) {
                        interior = jQuery.type(point.interior) == "array" ? point.fill[0] : point.fill;
                    }
                    else {
                        point.fill = interior;
                    }

                    //Draw open points
                    if (drawOpenWing) {

                        var startOpenValue = { X: openvalue.X + sidebysideinfo.Median, YValues: openvalue.YValues };
                        var pto1 = ej.EjSvgRender.utils._getPoint(startOpenValue, currentseries);
                        var endOpenValue = { X: openvalue.X + sidebysideinfo.Start, YValues: openvalue.YValues };
                        var pto2 = ej.EjSvgRender.utils._getPoint(endOpenValue, currentseries);
                        lDirection = "M" + " " + (pto1.X) + " " + ((pto1.Y)) + " " + "L" + " " + (pto2.X) + " " + ((pto2.Y)) + " ";
                        var openbounds = { point1: pto1, point2: pto2 };
                        this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, openbounds);
                        lDirection = "";

                    }
                    //Draw close points
                    if (drawCloseWing) {
                        var startCloseValue = { X: closevalue.X + sidebysideinfo.Median, YValues: closevalue.YValues };
                        var ptc1 = ej.EjSvgRender.utils._getPoint(startCloseValue, currentseries);
                        var endCloseValue = { X: closevalue.X + sidebysideinfo.End, YValues: closevalue.YValues };
                        var ptc2 = ej.EjSvgRender.utils._getPoint(endCloseValue, currentseries);
                        lDirection = "M" + " " + (ptc1.X) + " " + ((ptc1.Y)) + " " + "L" + " " + (ptc2.X) + " " + ((ptc2.Y)) + " ";
                        var pointbounds = { point1: ptc1, point2: ptc2 };
                        this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                        lDirection = "";

                    }
                    // Draw High low points
                    var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
                    var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                    lDirection = "M" + " " + (point1.X) + " " + ((point1.Y)) + " " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " ";
                    var bounds = { point1: point1, point2: point2 };
                    this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, bounds);
                }

            }
            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        },

        doAnimation: function (series) {

            var elements = $(this.gSeriesGroupEle).children().not("defs");
            var secondsPerPoint = 2000 / elements.length;
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
            }
        },
        hiloTypes: true

    });

    ej.seriesTypes.hiloopenclose = ej.ejhiloopencloseSeries;

    //Draw candle
    ej.ejCandleSeries = ejExtendClass(ej.EjSeriesRender, {
        draw: function (chart, options) {

            this.chartObj = chart;

            //DrawCandleGraph      

            var currentseries = options;
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
            var visiblePoints = this._isVisiblePoints(currentseries);
            var sidebysideinfo = this.getSideBySideInfo(currentseries);
            var point = null;

            var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
            var transX = currentseries.xAxis.x;
            var transY = currentseries.yAxis.y;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var interior;

            for (var i = 0; i < visiblePoints.length; i++) {
                point = visiblePoints[i];
                var pointIndex = i;
                if (point.visible) {

                    var lDirection = "";

                    var lowvalue = {}, highvalue = {}, openvalue = {}, closevalue = {};
                    if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]) || ej.util.isNullOrUndefined(point.YValues[2]) || ej.util.isNullOrUndefined(point.YValues[3]))
                        continue;

                    if (point.YValues[0] < point.YValues[1]) {
                        lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                        highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                        openvalue = { X: point.X, YValues: [point.YValues[2]] };
                        closevalue = { X: point.X, YValues: [point.YValues[3]] };

                    } else {
                        lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                        highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                        openvalue = { X: point.X, YValues: [point.YValues[2]] };
                        closevalue = { X: point.X, YValues: [point.YValues[3]] };

                    }
                    if (point.YValues[2] < point.YValues[3]) {

                        openvalue = { X: point.X, YValues: [point.YValues[2]] };
                        closevalue = { X: point.X, YValues: [point.YValues[3]] };
                        interior = (ej.util.isNullOrUndefined(currentseries.bearFillColor)) ? "#339933" : currentseries.bearFillColor;

                    } else {

                        openvalue = { X: point.X, YValues: [point.YValues[3]] };
                        closevalue = { X: point.X, YValues: [point.YValues[2]] };
                        interior = (ej.util.isNullOrUndefined(currentseries.bullFillColor)) ? "#E51400" : currentseries.bullFillColor;

                    }

                    var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex, interior);

                    var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex, seriesIndex, interior);

                    if (point.style && point.style.interior) {
                        interior = jQuery.type(point.style.interior) == "array" ? point.style.interior[0] : point.style.interior;
                    }
                    else {

                        point.fill = interior;
                    }

                    // Draw High low points
                    var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
                    var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                    lDirection = "M" + " " + (point1.X) + " " + ((point1.Y)) + " " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " ";
                    var pointbounds = { point1: point1, point2: point2 };
                    this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                    //Draw open and close points

                    var startRect = { X: closevalue.X + sidebysideinfo.Start, YValues: closevalue.YValues };
                    var orginRect = ej.EjSvgRender.utils._getPoint(startRect, currentseries);

                    var rectSize = { X: openvalue.X + sidebysideinfo.End, YValues: openvalue.YValues };
                    var sizeRect = ej.EjSvgRender.utils._getPoint(rectSize, currentseries);
                    var rect = ej.EjSvgRender.utils._correctRect(orginRect.X, orginRect.Y, sizeRect.X, sizeRect.Y);

                    var candleOptions = {
                        'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + i,
                        'x': rect.X,
                        'y': rect.Y,
                        'width': rect.Width,
                        'height': rect.Height,
                        'fill': styleOptions.interior,
                        'stroke-width': styleOptions.borderWidth,
                        'stroke': styleOptions.borderColor
                    };

                    this.chartObj.svgRenderer.drawRect(candleOptions, this.gSeriesGroupEle);

                    var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, rect.Y, currentseries, this.chartObj);
                    var bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                    ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, point, i);

                }

            }
            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);


        },

        doAnimation: function (series) {
            var elements = $(this.gSeriesGroupEle).children().not("defs");
            var secondsPerPoint = 2000 / elements.length;
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
            }

        },
        hiloTypes: true
    });
    ej.seriesTypes.candle = ej.ejCandleSeries;

    ej.ejPieSeries = ejExtendClass(ej.EjSeriesRender, {

        draw: function (chart, options) {

            this.chartObj = chart;
            var currentseries = options;
            var internalRegion = [];

            this.chartObj.model.bounds = [];

            var size = this.calculatingSliceAngle(currentseries);

            this.chartObj.model.Radius = 0.5 * currentseries.pieCoefficient * Math.min(size.width, size.height);

            var text;
            var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

            var visibility = (currentseries.animation) ? 'hidden' : 'visible';

            if (currentseries.marker.dataLabel.visible) {
                var txtOptions = { 'id': this.chartObj.svgObject.id + '_textGroup' + '_' + seriesIndex, 'visibility': visibility };
                this.chartObj.gSeriesTextEle = this.chartObj.svgRenderer.createGroup(txtOptions);

                var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'visibility': visibility };
                this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

                var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'visibility': visibility };
                this.chartObj.gSeriesConnectorEle = this.chartObj.svgRenderer.createGroup(connectorOptions);

            }

            for (var j = 0; j < currentseries.visiblePoints.length; j++) {
                pointIndex = j;
                var point = currentseries.visiblePoints[j];
                var result = this._calculateArcData(this.chartObj, pointIndex, point, currentseries);

                var styleOptions = this.chartObj.setStyle(this, currentseries, 0, pointIndex);

                options = {
                    'id': this.chartObj.svgObject.id + currentseries.name + "path" + pointIndex,
                    'fill': styleOptions.interior,
                    'stroke-width': styleOptions.borderWidth,
                    'stroke': styleOptions.borderColor,
                    'stroke-linecap': currentseries.lineCap,
                    'stroke-linejoin': currentseries.lineJoin,
                    'd': result.Direction,
                    'opacity': styleOptions.opacity,
                    'start': point.startAngle,
                    'end': point.endAngle,
                    'innerR': 0,
                    'x': this.chartObj.model.centerX,
                    'y': this.chartObj.model.centerY
                };
                this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

                region = { PointIndex: pointIndex, StartAngle: point.startAngle, EndAngle: point.endAngle, StartX: this.chartObj.model.startX[pointIndex], StartY: this.chartObj.model.startY[pointIndex] };
                internalRegion.push(region);

            }

            var seriesData = { Radius: this.chartObj.model.Radius, CenterX: this.chartObj.model.centerX, CenterY: this.chartObj.model.centerY };
            var region = { Series: currentseries, SeriesData: seriesData, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
            this.chartObj.model.chartRegions.push(region);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
        },
        doAnimation: function (series) {
            var chartObj = this.chartObj;
            var seriesRender = this;
            var angle = (series.points[0]) ? series.points[0].startAngle : 0;
            $.each(series.visiblePoints, function (pointIndex, point) {
                var startAngle = point.startAngle;
                var endAngle = point.endAngle;
                chartObj.model._radius = chartObj.model.Radius;

                point.startAngle = 0;
                point.endAngle = point.startAngle;
                chartObj.model.Radius = 0;
                var piePath = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[pointIndex];
                $(piePath).each(function () { this.StartAngle = angle, this.EndAngle = angle; }).animate(
                    { StartAngle: startAngle, EndAngle: endAngle, pieRadius: chartObj.model._radius },

                    {
                        duration: 1200, queue: false, step: function (now, fx) {

                            if (fx.prop.toString() === "StartAngle") {
                                point.startAngle = now;
                                point.endAngle = point.startAngle;
                            }
                            else if (fx.prop.toString() == "pieRadius") {
                                chartObj.model.Radius = now;
                            }
                            else {
                                point.endAngle = now;
                            }
                            var result = seriesRender._calculateArcData(chartObj, pointIndex, point, series);
                            chartObj.svgRenderer._setAttr($(piePath), { "d": result.Direction });
                            chartObj.model.Radius = chartObj.model._radius;
                        },
                        complete: function () {
                            if (pointIndex == series.visiblePoints.length - 1) {
                                if (series.marker.dataLabel.visible) {
                                    chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle.id), { "visibility": "visible" });
                                    if (chartObj.gSymbolGroupEle)
                                        chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle.id), { "visibility": "visible" });
                                    if (series.marker.dataLabel.template) {
                                        var elements = $(chartObj.element[0].childNodes[0].childNodes);
                                        for (var i = 0; i < elements.length; i++)
                                            $(elements[i]).css('display', 'block');
                                    }
                                    if (chartObj.gSeriesConnectorEle)
                                        chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesConnectorEle.id), { "visibility": "visible" });
                                }
                                chartObj.model.AnimationComplete = true;
                                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                                commonEventArgs.Data = { Series: series };
                                chartObj._trigger("animationComplete", commonEventArgs);
                            }
                        }
                    });
            });
        },
        chartAreaType: "None"

    });

    ej.seriesTypes.pie = ej.ejPieSeries;

    ej.ejDoughnutSeries = ejExtendClass(ej.EjSeriesRender, {
        draw: function (chart, options) {
            this.chartObj = chart;
            var currentseries = options;
            var internalRegion = [];

            this.chartObj.model.bounds = [];

            var size = this.calculatingSliceAngle(currentseries);

            this.chartObj.model.Radius = 0.5 * currentseries.doughnutSize * Math.min(size.width, size.height);
            this.chartObj.model.dRadius = currentseries.doughnutCoefficient * this.chartObj.model.Radius;


            var numberToFixed = ej.util.isNullOrUndefined(this.chartObj.model.roundingPlaces) ? 2 : this.chartObj.model.roundingPlaces;

            var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

            var visibility = (currentseries.animation) ? 'hidden' : 'visible';

            if (currentseries.marker.dataLabel.visible) {
                var txtOptions = { 'id': this.chartObj.svgObject.id + '_textGroup' + '_' + seriesIndex, 'visibility': visibility };
                this.chartObj.gSeriesTextEle = this.chartObj.svgRenderer.createGroup(txtOptions);

                var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'visibility': visibility };
                this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

                var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'visibility': visibility };
                this.chartObj.gSeriesConnectorEle = this.chartObj.svgRenderer.createGroup(connectorOptions);

            }

            for (var j = 0; j < currentseries.visiblePoints.length; j++) {
                var pointIndex = j;
                var point = currentseries.visiblePoints[j];
                var result = this._calculateArcData(this.chartObj, pointIndex, point, currentseries);


                var styleOptions = this.chartObj.setStyle(this, currentseries, 0, pointIndex);

                options = {
                    'id': this.chartObj.svgObject.id + "_" + this.getSeriesName(currentseries) + "_" + "path" + pointIndex,
                    'fill': styleOptions.interior,
                    'stroke-width': styleOptions.borderWidth,
                    'stroke': styleOptions.borderColor,
                    'stroke-linecap': currentseries.lineCap,
                    'opacity': styleOptions.opacity,
                    'stroke-linejoin': currentseries.lineJoin,
                    'd': result.Direction,
                    'start': point.startAngle,
                    'end': point.endAngle,
                    'innerR': this.chartObj.model.dRadius,
                    'x': this.chartObj.model.centerX,
                    'y': this.chartObj.model.centerY
                };
                this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);


                region = { PointIndex: pointIndex, StartAngle: point.startAngle, EndAngle: point.endAngle, StartX: this.chartObj.model.startX[pointIndex], StartY: this.chartObj.model.startY[pointIndex] };
                internalRegion.push(region);

            }

            var seriesData = { Radius: this.chartObj.model.Radius, DRadius: this.chartObj.model.dRadius, CenterX: this.chartObj.model.centerX, CenterY: this.chartObj.model.centerY };
            var region = { Series: currentseries, SeriesData: seriesData, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
            this.chartObj.model.chartRegions.push(region);

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        },


        doAnimation: function (series) {
            var chartObj = this.chartObj;
            var seriesRender = this;
            var angle = (series.points[0]) ? series.points[0].startAngle : 0;
            $.each(series.visiblePoints, function (pointIndex, point) {
                var startAngle = point.startAngle;
                var endAngle = point.endAngle;

                point.startAngle = 0;
                point.endAngle = point.startAngle;
                var doughPath = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[pointIndex];
                $(doughPath).each(function () { this.StartAngle = angle, this.EndAngle = angle; }).animate(

                { StartAngle: startAngle, EndAngle: endAngle },

                {
                    duration: 1200, queue: false, step: function (now, fx) {
                        if (fx.prop.toString() === "StartAngle") {
                            point.startAngle = now;
                            point.endAngle = point.startAngle;
                        }
                        else
                            point.endAngle = now;

                        var result = seriesRender._calculateArcData(chartObj, pointIndex, point, series);
                        chartObj.svgRenderer._setAttr($(doughPath), { "d": result.Direction });
                    },
                    complete: function () {
                        if (pointIndex == series.visiblePoints.length - 1) {
                            if (series.marker.dataLabel.visible) {
                                chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle.id), { "visibility": "visible" });
                                if (chartObj.gSymbolGroupEle)
                                    chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle.id), { "visibility": "visible" });
                                if (series.marker.dataLabel.template) {
                                    var elements = $(chartObj.element[0].childNodes[0].childNodes);
                                    for (var i = 0; i < elements.length; i++)
                                        $(elements[i]).css('display', 'block');
                                }
                                if (chartObj.gSeriesConnectorEle)
                                    chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesConnectorEle.id), { "visibility": "visible" });
                            }
                            chartObj.model.AnimationComplete = true;
                            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                            commonEventArgs.Data = { Series: series };
                            chartObj._trigger("animationComplete", commonEventArgs);
                        }
                    }
                });
            });
        },

        chartAreaType: "None"
    });

    ej.seriesTypes.doughnut = ej.ejDoughnutSeries;

    ej.ejPyramidSeries = ejExtendClass(ej.EjSeriesRender, {
        draw: function (chart, options) {

            this.chartObj = chart;
            var point;
            var seriesRender = this;
            var currentseries = options;
            var visiblePoints = this._pieVisiblePoints(currentseries);
            var internalRegion = [];

            this.chartObj.model.chartRegions = [];
            this.chartObj.model.sumofYValues = 0;

            //var explodeValue = 0;
            //var itemCurrentXPos = 0;
            var textSize = 0;
            var text;

            var measureTitle = (this.chartObj.model.title.text) ? ej.EjSvgRender.utils._measureText(this.chartObj.model.title.text, $(this.svgObject).width() - this.chartObj.model.margin.left - this.chartObj.model.margin.right, this.chartObj.model.title.font) : 0;

            var xOffset = (this.chartObj.model.margin.left + this.chartObj.model.elementSpacing + this.chartObj.model.margin.right + this.chartObj.model.elementSpacing + ((this.chartObj.model.legend.position.toLowerCase() === "right" || this.chartObj.model.legend.position.toLowerCase() === "left") ? this.chartObj.model.LegendBounds.Width : 0));
            var yOffset = (((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y + measureTitle.height) : this.chartObj.model.elementSpacing) + this.chartObj.model.margin.top + this.chartObj.model.elementSpacing + this.chartObj.model.margin.bottom + this.chartObj.model.elementSpacing + ((this.chartObj.model.legend.position.toLowerCase() === "top" || this.chartObj.model.legend.position.toLowerCase() === "bottom") ? this.chartObj.model.LegendBounds.Height : 0));

            if (currentseries.labelPosition.toLowerCase() === 'outside') {
                for (var i = 0; i < visiblePoints.length; i++) {
                    var textWidth = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(visiblePoints[i].text) ? visiblePoints[i].y : visiblePoints[i].text), null, currentseries.marker.dataLabel.font).width;
                    if (textSize < textWidth)
                        textSize = textWidth;
                }
                this.chartObj.model.textSize = textSize;
            }

            this.chartObj.model.actualWidth = $(chart.svgObject).width() - xOffset - textSize;
            this.chartObj.model.actualHeight = $(chart.svgObject).height() - yOffset;

            var pyrX = ((this.chartObj.model.legend.position.toLowerCase() === "left") ? (this.chartObj.model.LegendBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;
            var pyrY = ((this.chartObj.model.legend.position.toLowerCase() === "top") ? (this.chartObj.model.LegendBounds.Height) : 0) + ((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y + measureTitle.height) : (this.chartObj.model.margin.top + this.chartObj.model.elementSpacing));

            for (var j = 0; j < visiblePoints.length; j++) {
                this.chartObj.model.sumofYValues += visiblePoints[j].YValues[0];
            }


            var numberToFixed = ej.util.isNullOrUndefined(this.chartObj.model.roundingPlaces) ? 2 : this.chartObj.model.roundingPlaces;

            var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

            if (currentseries.marker.dataLabel.visible) {
                var txtOptions = { 'id': this.chartObj.svgObject.id + '_textGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
                this.chartObj.gSeriesTextEle = this.chartObj.svgRenderer.createGroup(txtOptions);
                if (currentseries.labelPosition.toLowerCase() === 'outside') {

                    var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
                    this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

                    var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
                    this.chartObj.gSeriesConnectorEle = this.chartObj.svgRenderer.createGroup(connectorOptions);
                }
            }

            if (currentseries.pyramidMode.toLowerCase() == "linear")
                this.calculateLinearSegments(currentseries);
            else
                this.calculateSurfaceSegments(currentseries);

            for (j = 0; j < visiblePoints.length; j++) {
                var pointIndex = j;
                point = visiblePoints[j];
                var result = this._getPyramidData(currentseries, pointIndex);

                var styleOptions = this.chartObj.setStyle(this, currentseries, 0, pointIndex);

                options = {
                    'id': this.chartObj.svgObject.id + "_" + this.getSeriesName(currentseries) + "_" + "path" + pointIndex,
                    'fill': styleOptions.interior,
                    'stroke-width': styleOptions.borderWidth,
                    'stroke': styleOptions.borderColor,
                    'stroke-linecap': currentseries.lineCap,
                    'stroke-linejoin': currentseries.lineJoin,
                    'opacity': styleOptions.opacity,
                    'pointIndex': pointIndex,
                    'd': result.Direction
                };
                this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

                if (currentseries.marker.dataLabel.template) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(point, pointIndex, currentseries, true, this.chartObj);
                }

                point.xLocation = result.PositionX;
                point.yLocation = result.PositionY;
                point.connectorLine = result.Connector;

                region = { PointIndex: pointIndex };
                internalRegion.push(region);
            }
            var region = { Series: currentseries, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
            this.chartObj.model.chartRegions.push(region);

            $(this.gSeriesGroupEle.childNodes[pointIndex]).bind('mousemove', function (evt) {
                evt.target = seriesRender.gSeriesGroupEle.childNodes[pointIndex];
                seriesRender.chartObj.chartInteractiveBehavior(evt);
            });

            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);



        },



        doAnimation: function (series) {

        },
        _getPyramidData: function (series, index) {

            var model = this.chartObj.model;
            var offset = ((series.explodeIndex == index) || (series.explodeAll) ? series.explodeOffset : 0);

            var top = model.pyramidData[index].CurrY;
            var bottom = model.pyramidData[index].CurrY + model.pyramidData[index].Height;
            var topRadius = 0.5 * (1 - model.pyramidData[index].CurrY);
            var bottomRadius = 0.5 * (1 - bottom);

            var line1 = { x: offset + topRadius * model.actualWidth, y: top * model.actualHeight };
            var line2 = { x: offset + (1 - topRadius) * model.actualWidth, y: top * model.actualHeight };
            var line3 = { x: offset + (1 - bottomRadius) * model.actualWidth, y: bottom * model.actualHeight };
            var line4 = { x: offset + bottomRadius * model.actualWidth, y: bottom * model.actualHeight };

            var direction = "M " + " " + (line1.x) + " " + (line1.y) + " " + "L" + " " + (line2.x) + " " + (line2.y) + " " + "L" + " " + (line3.x) + " " + (line3.y) + " " + "L" + " " + (line4.x) + " " + (line4.y) + " " + "z";

            if (series.marker.dataLabel.visible) {
                var positionX;
                var positionY;

                if (series.labelPosition.toLowerCase() === 'outside') {

                    positionX = (line2.x + line3.x) / 2;
                    positionY = (line2.y + line3.y) / 2;

                    var pyrX = ((this.chartObj.model.legend.position.toLowerCase() == "left" || this.chartObj.model.legend.position.toLowerCase() == "right") ? (this.chartObj.model.LegendBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;

                    if (ej.util.isNullOrUndefined(series.marker.dataLabel.connectorLine.height)) {
                        var textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + this.chartObj.model.textSize + this.chartObj.model.elementSpacing);
                    }
                    else
                        textOffset = series.marker.dataLabel.connectorLine.height;

                    var connectorX = this.getXCordinate(positionX, textOffset, 0);
                    var connectorY = this.getYCordinate(positionY, textOffset, 0);

                    var connectorDirection = "M" + " " + positionX + " " + positionY + " " + "L" + " " + connectorX + " " + connectorY;

                    positionX = connectorX;
                    positionY = connectorY;


                }
                else {
                    positionX = (line1.x + line2.x) / 2;
                    positionY = (line3.y + line1.y) / 2;
                }
            }
            return { Direction: direction, PositionX: positionX, PositionY: positionY, Connector: connectorDirection };
        },

        calculateLinearSegments: function (series) {
            var currY = 0;
            var model = this.chartObj.model;
            model.pyramidData = [];
            var coef = 1 / (model.sumofYValues * (1 + series.gapRatio / (1 - series.gapRatio)));
            var count = series.points.length;
            for (var i = 0; i < series.points.length; i++) {
                var height = coef * series.points[i].YValues[0];
                model.pyramidData[i] = { CurrY: currY, Height: height };
                currY += (series.gapRatio / (count - 1)) + height;
            }
        },

        calculateSurfaceSegments: function (series) {
            var count = series.points.length;
            var model = this.chartObj.model;
            model.pyramidData = [];
            var currY = 0;
            var y = [];
            var height = [];
            var gapHeight = series.gapRatio / (count - 1);
            var preSum = this.getSurfaceHeight(0, model.sumofYValues);
            for (i = 0; i < count; i++) {
                y[i] = currY;
                height[i] = this.getSurfaceHeight(currY, Math.abs(series.points[i].YValues[0]));
                currY += height[i] + gapHeight * preSum;
            }
            var coef = 1 / (currY - gapHeight * preSum);
            for (var i = 0; i < count; i++) {
                var currHeight = coef * y[i];
                model.pyramidData[i] = { CurrY: currHeight, Height: coef * height[i] };
            }
        },

        getSurfaceHeight: function (y, surface) {
            var result = this.solveQuadraticEquation(1, 2 * y, -surface);
            if (result) {
                return result;
            }
            return 0;
        },
        solveQuadraticEquation: function (a, b, c) {
            var root1;
            var root2;

            if (a != 0) {
                var d = b * b - 4 * a * c;

                if (d >= 0) {
                    var sd = Math.sqrt(d);

                    root1 = (-b - sd) / (2 * a);
                    root2 = (-b + sd) / (2 * a);
                    return Math.max(root1, root2);

                }
            }
            else if (b != 0) {
                root1 = -c / b;
                root2 = -c / b;
                return Math.max(root1, root2);
            }

            return false;
        },

        chartAreaType: "None"
    });

    ej.seriesTypes.pyramid = ej.ejPyramidSeries;
})(jQuery);;
ej.ejChart = {};
(function ($) {
    String.prototype.parseTemplate = function () {
        var str = this;
        $.each(arguments[0], function (dataIndex, dataValue) {
            if (dataValue.count > 0) {
                $.each(dataValue, function (datachildIndex, datachildValue) {
                    str = str.replace(new RegExp('\\#' + dataIndex + '.' + datachildIndex + '\\#', 'gm'), datachildValue);
                });
            } else {
                str = str.replace(new RegExp('\\#' + dataIndex + '\\#', 'gm'), dataValue);
            }

        });
        return str;
    };


    var Gradient = function (colors) {
        this._gradientStop = [];
        for (var j = 0; j < colors.length; j++) {
            this._gradientStop.push(colors[j]);
        }
    };



    ej.ejChart = {

        _renderSfChart: function () {

            var svgSupport = (window.SVGSVGElement) ? true : false;
            if (svgSupport) {
                this.svgRenderer = new ej.EjSvgRender(this.element);
                this._createChart();
                $(this.svgObject).appendTo(this.element);
            }
            else {

                this.svgRenderer = new ej.EjVmlRender(this.element);
                this._createChart();
                $(this.svgObject).appendTo(this.element);
            }
            return true;
        },

        setModelProperties: function () {

            //set size for the svgObject
            var containerHeight = $(this.element).height();
            var height = 450; //set default height if size not specified to chart or container
            $(this.svgObject).width(this.model.size.width ? this.model.size.width : $(this.element).width());

            if (this.model.size.height)
                height = this.model.size.height;
            else if (containerHeight > 0)
                height = containerHeight;
            else
                $(this.svgObject).css("display", "block"); //Added css to chart due to 5px difference for container and svg (http://jsfiddle.net/Me5Zd/)       
            $(this.svgObject).height(height);

            this.svgWidth = $(this.svgObject).width();
            this.svgHeight = $(this.svgObject).height();

            this.chartCross = {
                visible: this.model.crosshair.visible,
                mArea: this.model.crosshair.type
            };

            // extend series with commonSeriesOptions
            this.serAnimation = [];
            if (this.model.series) {
                for (var j = 0; j < this.model.series.length; j++) {
                    this.model.series[j] = ej.copyObject({}, this.model.commonSeriesOptions, this.model.series[j]);
                    this.serAnimation.push((this.model.series[j].animation === null || this.model.series[j].animation === undefined) ? this.model.commonSeriesOptions.animation : this.model.series[j].animation);
                }
            }
        },

        _createChart: function () {
            this.svgObject = this.svgRenderer.svgObj;

            var commonlaodEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            this._trigger("load", commonlaodEventArgs);
            this.setTheme(ej.EjSvgRender.themes, this.model.theme);
            var model = this.model;
            if (!(model.series) && !(model.dataSource)) {
                model.series = [];
                var randomPoints = [];
                for (var i = 1; i <= 10; i++) {
                    randomPoints.push({ x: i, y: (Math.floor(Math.random() * (10 - 40)) + 40) });

                }
                model.series.push({ points: randomPoints });

            }
            this.vmlRendering = (this.svgRenderer.vmlNamespace) ? true : false;
            if ($(this.element).is(":visible") || $(this.element).css("visibility") != "hidden" || $(this.element).css('display') != 'none') {
                this.bindEvents();
                this.bindTo();
            }
        },
        setTheme: function (jsonObj, themeName) {
            var name = themeName.toLowerCase();
            if ((name.indexOf("azure") >= 0 || name.indexOf("lime") >= 0 || name.indexOf("saffron") >= 0) && (name.indexOf("gradient") == -1) && (name.indexOf("dark") == -1))
                jsonObj[name] = $.extend(true, {}, jsonObj["flatlight"], jsonObj[name]);
            else if ((name.indexOf("azuredark") >= 0 || name.indexOf("limedark") >= 0 || name.indexOf("saffrondark")) >= 0 && (name.indexOf("gradient") == -1))
                jsonObj[name] = $.extend(true, {}, jsonObj["flatdark"], jsonObj[name]);
            else if ((name.indexOf("azure") >= 0 || name.indexOf("lime") >= 0 || name.indexOf("saffron") >= 0) && (name.indexOf("gradient") >= 0) && (name.indexOf("dark") == -1))
                jsonObj[name] = $.extend(true, {}, jsonObj["gradientlight"], jsonObj[name]);
            else if ((name.indexOf("azuredark") >= 0 || name.indexOf("limedark") >= 0 || name.indexOf("saffrondark") >= 0) && (name.indexOf("gradient") >= 0))
                jsonObj[name] = $.extend(true, {}, jsonObj["gradientdark"], jsonObj[name]);
            else if (!(jsonObj[name])) name = "flatlight";

            if (this.model._themeChanged)
                this.model = ej.copyObject({}, this.model, jsonObj[name]);
            else
                this.model = ej.copyObject({}, jsonObj[name], this.model);


        },
        _drawBackInterior: function () {
            if (this.model.backInterior && this.model.backInterior._gradientStop) {
                var options = {
                    'id': this.svgObject.id + '_backGradient',
                    'x1': '0%',
                    'y1': '0%',
                    'x2': '0%',
                    'y2': $(this.svgObject).height() + '%'
                };
                this.svgRenderer.drawGradient(options, this.model.backInterior._gradientStop);
            }
        },



        _drawBackImage: function () {
            var options = {
                'height': $(this.svgObject).height(),
                'width': $(this.svgObject).width(),
                'href': this.model.backGroundImageUrl,
                'x': 0,
                'y': 0,
                'id': this.svgObject.id + '_backImage',
                'visibility': 'visible',
                'preserveAspectRatio': 'none'
            };

            this.svgRenderer.drawImage(options, this.svgObject);

        },
        _getDataType: function (val) {
            var type = typeof (val);
            if (type === "number" || type === "string")
                return type;
            else if (jQuery.type(val) == "date")
                return "date";
            else
                return false;

        },
        addedXYValues: function () {
            if (!this.zoomed) {
                for (var i = 0; i < this.model.series.length; i++) {
                    var setXValueType = false, pointX, currentSeries;

                    if (!this.model.series[i].dataSource) {
                        this.model.series[i] = $.extend(true, {}, this.model.commonSeriesOptions, this.model.series[i]);
                        currentSeries = this.model.series[i];
                        for (k = 0; k < currentSeries.points.length; k++) {
                            currentSeries.points[k].YValues = [];
                            currentSeries.points[k].X = currentSeries.points[k].x;
                            if (!setXValueType) {
                                pointX = currentSeries.points[k].x;
                                if (pointX) {
                                    currentSeries._xAxisValueType = this._getDataType(pointX);
                                    setXValueType = true;
                                }
                            }
                            if (!(currentSeries._hiloTypes)) {
                                currentSeries.points[k].YValues[0] = currentSeries.points[k].y;

                                if (currentSeries.points[k].size)
                                    currentSeries.points[k].YValues[1] = currentSeries.points[k].size;

                            }
                            else {
                                if (currentSeries.points[k].high) {
                                    currentSeries.points[k].YValues[0] = currentSeries.points[k].high;
                                    currentSeries.points[k].y = currentSeries.points[k].high;
                                }
                                if (currentSeries.points[k].low)
                                    currentSeries.points[k].YValues[1] = currentSeries.points[k].low;
                                if (currentSeries.points[k].open)
                                    currentSeries.points[k].YValues[2] = currentSeries.points[k].open;
                                if (currentSeries.points[k].close)
                                    currentSeries.points[k].YValues[3] = currentSeries.points[k].close;
                            }
                        }
                    }
                    else {
                        currentSeries = this.model.series[i];
                        for (var j = 0, length = currentSeries.points.length; j < length; j++) {
                            pointX = currentSeries.points[j].X = currentSeries.points[j].x;
                            if (ej.util.isNullOrUndefined(currentSeries._xAxisValueType)) {
                                if (!ej.util.isNullOrUndefined(pointX)) currentSeries._xAxisValueType = this._getDataType(pointX);
                            }
                        }
                    }
                }
                this._setInternalValues();
            }
        },
        _setInternalValues: function () {
            var type = this.model.series[0].type.toLowerCase();
            if (ej.seriesTypes[type].prototype.chartAreaType == "cartesianAxes") {
                this.model.AreaType = "cartesianaxes";
            }
            else if (ej.seriesTypes[type].prototype.chartAreaType == "None")
                this.model.AreaType = "none";

            this.model.requireInvertedAxes = ej.seriesTypes[type].prototype.requireInvertedAxes;

            this.model._visibleSeries = [];
            for (var k = 0; k < this.model.series.length; k++) {
                var series = this.model.series[k];
                if (this.model.AreaType == "cartesianaxes" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "cartesianAxes") {
                    series._zOrder = (series.zOrder) ? series.zOrder : 0;
                    this.model._visibleSeries.push(series);
                }
                else if (this.model.AreaType == "none" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "None") {
                    series._zOrder = (series.zOrder) ? series.zOrder : 0;
                    this.model._visibleSeries.push(series);
                }

                if (series.yAxisName || series.xAxisName) {
                    if (this.model.axes) {
                        for (var t = 0; t < this.model.axes.length; t++) {
                            if (series.yAxisName && (series.yAxisName == this.model.axes[t].axisName)) {
                                this.model.axes[t].orientation = (this.model.axes[t].orientation) ? this.model.axes[t].orientation : (this.model.requireInvertedAxes) ? "horizontal" : "vertical";
                            }
                            if (series.xAxisName && (series.xAxisName == this.model.axes[t].axisName)) {
                                this.model.axes[t].orientation = (this.model.axes[t].orientation) ? this.model.axes[t].orientation : (this.model.requireInvertedAxes) ? "vertical" : "horizontal";
                            }
                        }
                    }
                }
            }
        },
        setHiloStyle: function (series, pointIndex, seriesIndex, interiorColor) {
            var point = series.points[pointIndex];

            if (point.fill)
                interior = point.fill;
            else
                interior = series.fill ? series.fill : interiorColor;

            // setting default series color		
            var colors = interior ? interior : this.model.seriesColors[seriesIndex];

            // applying gradient color		  

            seriesInterior = jQuery.type(colors) == "array" ? colors[0].color : colors;

            return seriesInterior;

        },
        setStyle: function (chart, series, seriesIndex, pointIndex, interiorColor) {

            var point = series.points[pointIndex];
            // var pointStyle = point.style ? point.style : " ";
            var interior, borderColor, borderWidth, opacity;

            // setting interior
            if (chart.chartObj.model.AreaType == "cartesianaxes") {
                if (point.fill)
                    interior = point.fill;
                else
                    interior = series.fill ? series.fill : interiorColor;

                // setting default series color		
                var colors = interior ? interior : this.model.seriesColors[seriesIndex];

                // applying gradient color		  
                if (chart.chartObj.model.requireInvertedAxes)
                    var seriesInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + "_" + seriesIndex + pointIndex, colors, 0, 0, 100, 0, chart.gSeriesGroupEle);
                else
                    seriesInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + "_" + seriesIndex + pointIndex, colors, 0, 0, 0, 100, chart.gSeriesGroupEle);
            }

                // interior for pie type series
            else {
                interior = point.fill ? point.fill : this.model.pointColors[pointIndex];
                var pointInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + pointIndex, interior, 0, 0, 0, $(chart.svgObject).height(), chart.chartObj.gSeriesEle);
            }

            // setting borderColor
            borderColor = ((point.border) && point.border.color) ? point.border.color : series.border.color;
            borderWidth = ((point.border) && point.border.width) ? point.border.width : series.border.width;
            opacity = point.opacity ? point.opacity : series.opacity;


            var options = {
                'interior': seriesInterior ? seriesInterior : pointInterior,
                'borderColor': borderColor,
                'borderWidth': borderWidth,
                'opacity': opacity
            };
            return options;
        },

        colorNameToHex: function (colour) {
            var color = colour;
            var colours = {
                "aliceblue": "#f0f8ff", "antiquewhite": "#faebd7", "aqua": "#00ffff", "aquamarine": "#7fffd4", "azure": "#f0ffff",
                "beige": "#f5f5dc", "bisque": "#ffe4c4", "black": "#000000", "blanchedalmond": "#ffebcd", "blue": "#0000ff", "blueviolet": "#8a2be2", "brown": "#a52a2a", "burlywood": "#deb887",
                "cadetblue": "#5f9ea0", "chartreuse": "#7fff00", "chocolate": "#d2691e", "coral": "#ff7f50", "cornflowerblue": "#6495ed", "cornsilk": "#fff8dc", "crimson": "#dc143c", "cyan": "#00ffff",
                "darkblue": "#00008b", "darkcyan": "#008b8b", "darkgoldenrod": "#b8860b", "darkgray": "#a9a9a9", "darkgreen": "#006400", "darkkhaki": "#bdb76b", "darkmagenta": "#8b008b", "darkolivegreen": "#556b2f",
                "darkorange": "#ff8c00", "darkorchid": "#9932cc", "darkred": "#8b0000", "darksalmon": "#e9967a", "darkseagreen": "#8fbc8f", "darkslateblue": "#483d8b", "darkslategray": "#2f4f4f", "darkturquoise": "#00ced1",
                "darkviolet": "#9400d3", "deeppink": "#ff1493", "deepskyblue": "#00bfff", "dimgray": "#696969", "dodgerblue": "#1e90ff",
                "firebrick": "#b22222", "floralwhite": "#fffaf0", "forestgreen": "#228b22", "fuchsia": "#ff00ff",
                "gainsboro": "#dcdcdc", "ghostwhite": "#f8f8ff", "gold": "#ffd700", "goldenrod": "#daa520", "gray": "#808080", "green": "#008000", "greenyellow": "#adff2f",
                "honeydew": "#f0fff0", "hotpink": "#ff69b4",
                "indianred ": "#cd5c5c", "indigo ": "#4b0082", "ivory": "#fffff0", "khaki": "#f0e68c",
                "lavender": "#e6e6fa", "lavenderblush": "#fff0f5", "lawngreen": "#7cfc00", "lemonchiffon": "#fffacd", "lightblue": "#add8e6", "lightcoral": "#f08080", "lightcyan": "#e0ffff", "lightgoldenrodyellow": "#fafad2",
                "lightgrey": "#d3d3d3", "lightgreen": "#90ee90", "lightpink": "#ffb6c1", "lightsalmon": "#ffa07a", "lightseagreen": "#20b2aa", "lightskyblue": "#87cefa", "lightslategray": "#778899", "lightsteelblue": "#b0c4de",
                "lightyellow": "#ffffe0", "lime": "#00ff00", "limegreen": "#32cd32", "linen": "#faf0e6",
                "magenta": "#ff00ff", "maroon": "#800000", "mediumaquamarine": "#66cdaa", "mediumblue": "#0000cd", "mediumorchid": "#ba55d3", "mediumpurple": "#9370d8", "mediumseagreen": "#3cb371", "mediumslateblue": "#7b68ee",
                "mediumspringgreen": "#00fa9a", "mediumturquoise": "#48d1cc", "mediumvioletred": "#c71585", "midnightblue": "#191970", "mintcream": "#f5fffa", "mistyrose": "#ffe4e1", "moccasin": "#ffe4b5",
                "navajowhite": "#ffdead", "navy": "#000080",
                "oldlace": "#fdf5e6", "olive": "#808000", "olivedrab": "#6b8e23", "orange": "#ffa500", "orangered": "#ff4500", "orchid": "#da70d6",
                "palegoldenrod": "#eee8aa", "palegreen": "#98fb98", "paleturquoise": "#afeeee", "palevioletred": "#d87093", "papayawhip": "#ffefd5", "peachpuff": "#ffdab9", "peru": "#cd853f", "pink": "#ffc0cb", "plum": "#dda0dd", "powderblue": "#b0e0e6", "purple": "#800080",
                "red": "#ff0000", "rosybrown": "#bc8f8f", "royalblue": "#4169e1",
                "saddlebrown": "#8b4513", "salmon": "#fa8072", "sandybrown": "#f4a460", "seagreen": "#2e8b57", "seashell": "#fff5ee", "sienna": "#a0522d", "silver": "#c0c0c0", "skyblue": "#87ceeb", "slateblue": "#6a5acd", "slategray": "#708090", "snow": "#fffafa", "springgreen": "#00ff7f", "steelblue": "#4682b4",
                "tan": "#d2b48c", "teal": "#008080", "thistle": "#d8bfd8", "tomato": "#ff6347", "turquoise": "#40e0d0",
                "violet": "#ee82ee",
                "wheat": "#f5deb3", "white": "#ffffff", "whitesmoke": "#f5f5f5",
                "yellow": "#ffff00", "yellowgreen": "#9acd32"
            };

            if (Object.prototype.toString.call(color) == '[object Array]')
                return color;

            if (typeof colours[color.toLowerCase()] != 'undefined')
                return colours[color.toLowerCase()];

            return color;
        },

        // set culture for chart
        _setCulture: function (culture) {
            this.culture = Globalize.culture(culture);
        },


        draw: function () {

            this._drawBackInterior();
            this.model.chartRegions = [];
            this.model.stackedValue = {};
            this._setCulture(this.model.localization);
            this.addedXYValues();


            var commonpreRenderEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonpreRenderEventArgs.Data = {};
            this._trigger("preRender", commonpreRenderEventArgs);

            this.gLegendEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_Legend' });

            this.gTitleEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ChartTitle' });

            this._createAxisLabelAndRange();

            this._initializeSeriesColors();

            for (i = 0; i < this.model.series.length; i++) {
                if (!ej.util.isNullOrUndefined(this.model.series[i].style)) {
                    if (!ej.util.isNullOrUndefined(this.model.series[i].style.interior)) {
                        var interior = this.model.series[i].style.interior;
                        interior = this.colorNameToHex(interior);
                        this.model.series[i].style.interior = interior;
                    }
                }
            }

            this._calculateLegendBounds();

            this.legendRender = new ej.EjLegendRender(this);

            var commonLegendEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLegendEventArgs.Data = { LegendBound: this.model.LegendBounds };
            this._trigger("lengendBoundsCalculate", commonLegendEventArgs);
            for (var m = 0; m < this.model._visibleSeries.length; m++) {
                var options = this.model._visibleSeries[m];
                for (i = 0; i < options.points.length; i++) {
                    if (typeof options.points[i].X == "string" && options.points[i].X.indexOf("/Date(") != -1)
                        options.points[i].X = new Date(parseInt(options.points[i].X.substr(6)));
                }
            }
            this.model.hAxes = [];
            this.model.vAxes = [];
            var chartobj = this;
            var isStriplineOver = false, isStriplineBehind = false;
            this._arrangeAxis();
            for (var k = 0; k < this.model.axes.length; k++) {
                var axis = this.model.axes[k];
                var seriesCollection = [];
                for (var slCount = 0; slCount < axis.stripline.length; slCount++) {
                    axis.stripline[slCount] = $.extend(true, {}, this.model.striplineDefault, axis.stripline[slCount]);
                    if (axis.stripline[slCount].zOrder.toLowerCase() == 'over') {
                        isStriplineOver = true;
                    } else {
                        isStriplineBehind = true;
                    }
                }
                var axisOrientation = axis.orientation.toLowerCase();
                for (j = 0; j < this.model._visibleSeries.length; ++j) {
                    if (this.model._visibleSeries[j].visibility.toLowerCase() == 'visible') {
                        var series = this.model._visibleSeries[j];
                        series.xAxisName = (!(series.xAxisName)) ? chartobj.model.axes[0].axisName : series.xAxisName;
                        series.yAxisName = (!(series.yAxisName)) ? chartobj.model.axes[1].axisName : series.yAxisName;
                        var xAxisName = series.xAxisName.toLowerCase();
                        var yAxisName = series.yAxisName.toLowerCase();
                        var axisName = axis.axisName.toLowerCase();
                        if (xAxisName || yAxisName) {
                            if (xAxisName == axisName || yAxisName == axisName) {
                                if (axisOrientation == "horizontal" && axis.valueType && axis.valueType.toString() == "datetime") {
                                    for (var m = 0; m < series.points.length; m++) {
                                        if (typeof series.points[m].X == "string" && !isNaN(Date.parse(series.points[m].X)))
                                            series.points[m].X = new Date(Date.parse(series.points[m].X));
                                    }
                                }
                                else {
                                    if (series._xAxisValueType == "string" && !(axis.valueType))
                                        axis._valueType = (axisOrientation == 'horizontal' && !this.model.requireInvertedAxes) ? 'category' : (axisOrientation == 'vertical' && this.model.requireInvertedAxes) ? 'category' : 'double';

                                    if (series._xAxisValueType == "date" && !(axis.valueType))
                                        axis._valueType = (axisOrientation == 'horizontal' && !this.model.requireInvertedAxes) ? "datetime" : (axisOrientation == 'vertical' && this.model.requireInvertedAxes) ? 'datetime' : 'double';
                                }
                                seriesCollection.push(series);
                            }
                        }
                    }
                }
                if (axisOrientation == ((!this.model.requireInvertedAxes) ? "vertical" : "horizontal")) {
                    for (var i = 0; i < seriesCollection.length; i++) {
                        var type = seriesCollection[i].type.toLowerCase();
                        if (ej.seriesTypes[type].prototype.stackingSeries) {
                            this._calculateStackingValues(seriesCollection, axis);
                            break;
                        }
                    }
                }
                axis._valueType = (!(axis._valueType)) ? (!(axis.valueType) ? "double" : axis.valueType) : axis._valueType;
                var axisRange = new ej.axisTypes[axis._valueType.toLowerCase()]();
                axisRange._calculateRanges(chartobj, axis, seriesCollection);
                var commonAxisEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxisEventArgs.Data = { range: axis.range };
                this._trigger("axesRangeCalculate", commonAxisEventArgs);
            }







            var commonAxesInitEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAxesInitEventArgs.Data = { Axes: this.model.axes };
            this._trigger("axesLabelsInitialize", commonAxesInitEventArgs);

            this._calculateAreaBounds();

            if (this.model.AreaType == "cartesianaxes") {

                var commonAreaEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAreaEventArgs.Data = { AreaBound: this.model.m_AreaBounds };
                this._trigger("chartAreaBoundsCalculate", commonAreaEventArgs);

                this._drawChartAreaRect();

                this._calculateAxisSize();

                this.gXaxisEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxis' });

                this.gYaxisEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxis' });

                $.extend(ej.EjAxisRenderer.prototype, this);
                this.chartaxis = new ej.EjAxisRenderer(this);
                var chartaxis = this.chartaxis;
                for (var l = 0; l < this.model.axes.length; l++) {
                    chartaxis._drawAxes(l, this.model.axes[l]);
                }

                this.svgRenderer.append(this.gXaxisEle, this.svgObject);

                this.svgRenderer.append(this.gYaxisEle, this.svgObject);

                var clipRectOptions = {
                    'id': this.svgObject.id + '_ChartAreaClipRect',
                    'x': this.model.m_AreaBounds.X,
                    'y': this.model.m_AreaBounds.Y,
                    'width': this.model.m_AreaBounds.Width,
                    'height': this.model.m_AreaBounds.Height,
                    'fill': 'white',
                    'stroke-width': 1,
                    'stroke': 'Gray'
                };

                this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);

                this._setZoomProperties();
            }

            else {
                this._drawChartAreaRect();

                clipRectOptions = {
                    'id': this.svgObject.id + '_ChartAreaClipRect',
                    'x': 0,
                    'y': 0,
                    'width': $(this.svgObject).width(),
                    'height': $(this.svgObject).height(),
                    'fill': 'white',
                    'stroke-width': 1,
                    'stroke': 'Gray'
                };

                this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);
            }

            if (this.model.title.text != "" && this.model.title.text)
                this._drawTitle();

            if (isStriplineBehind && this.model.AreaType == "cartesianaxes") {
                this.gStriplineBehind = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_StriplineBehind', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' });
                this._renderStripline('behind');
            }

            if (this.model.initSeriesRender)
                this.seriesRender();



            if (isStriplineOver && this.model.AreaType == "cartesianaxes") {
                this.gStriplineOver = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_StriplineOver', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' });
                this._renderStripline('over');
            }



            var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLoadedEventArgs.Data = { Model: this.model };
            this._trigger("loaded", commonLoadedEventArgs);


        },

        // set value to private variable for two way binding
        _setZoomProperties: function () {
            for (var l = 0; l < this.model.axes.length; l++) {
                var axis = this.model.axes[l];

                var _zf = axis.zoomFactor.toFixed(3);
                var _zp = axis.zoomPosition.toFixed(3);
                if (axis.axisName == this.model.primaryXAxis.axisName && (_zf != this._xZoomFactor() || _zp != this._xZoomPosition())) {
                    this._xZoomFactor(axis.zoomFactor.toFixed(3));
                    this._xZoomPosition(axis.zoomPosition.toFixed(3));
                }
                if (axis.axisName == this.model.primaryYAxis.axisName && (_zf != this._yZoomFactor() || _zp != this._yZoomPosition())) {
                    this._yZoomFactor(axis.zoomFactor.toFixed(3));
                    this._yZoomPosition(axis.zoomPosition.toFixed(3));
                }

            }
        },
        seriesRender: function () {
            var chartObj = this;
            if ($(this.svgObject).find("#" + this.svgObject.id + "_SeriesCollection").length > 0) {
                $(this.svgObject).find("#" + this.svgObject.id + "_SeriesCollection").remove();
            }
            var serOptions = { 'id': this.svgObject.id + '_SeriesCollection', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' };

            this.gSeriesEle = this.svgRenderer.createGroup(serOptions);
            var visibleSeries = ej.DataManager(this.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
            for (var i = 0; i < visibleSeries.length; i++) {
                if (visibleSeries[i].visibility == "visible") {
                    if (!ej.util.isNullOrUndefined(visibleSeries[i].xAxis)) {
                        visibleSeries[i].xAxis = null;
                        visibleSeries[i].yAxis = null;
                    }
                    for (var j = 0; j < this.model.axes.length; j++) {
                        var axis = this.model.axes[j];
                        var index = i;
                        if (axis.axisName && axis.orientation.toLowerCase() == 'horizontal' && axis.axisName.toLowerCase() == visibleSeries[i].xAxisName.toLowerCase()) {
                            visibleSeries[i].xAxis = axis;
                        }
                        else if (axis.axisName && axis.orientation.toLowerCase() == 'vertical' && axis.axisName.toLowerCase() == visibleSeries[i].yAxisName.toLowerCase()) {
                            visibleSeries[i].yAxis = axis;
                        }
                        if (this.model.requireInvertedAxes) {
                            if (axis.axisName && axis.orientation.toLowerCase() == 'vertical' && axis.axisName.toLowerCase() == visibleSeries[i].xAxisName.toLowerCase()) {
                                visibleSeries[i].xAxis = axis;
                            }
                            else if (axis.axisName && axis.orientation.toLowerCase() == 'horizontal' && axis.axisName.toLowerCase() == visibleSeries[i].yAxisName.toLowerCase()) {
                                visibleSeries[i].yAxis = axis;
                            }
                        }

                    }
                }
            }
            for (var m = 0; m < visibleSeries.length; m++) {
                var index = m;
                var seriesOptions = visibleSeries[m];
                if (this.model.AreaType == "none" && seriesOptions.visibility.toLowerCase() === 'visible') {
                    if (index === 0) {

                        var type = seriesOptions.type.toLowerCase();
                        var seriesIndex = $.inArray(visibleSeries[0], this.model._visibleSeries);
                        this.model._visibleSeries[seriesIndex] = visibleSeries[0] = $.extend(true, seriesOptions, visibleSeries[0]);
                        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                        commonEventArgs.Data = { series: seriesOptions };
                        this._trigger("seriesRendering", commonEventArgs);
                        var seriesType = new ej.seriesTypes[type]();
                        seriesType.draw(chartObj, seriesOptions);
                        if (seriesOptions.animation && !this.vmlRendering) {
                            seriesType.doAnimation(seriesOptions);
                        }
                        this.drawAccDisplayText(seriesType, seriesOptions);
                    }
                }
                else {
                    if (seriesOptions.visibility.toLowerCase() === 'visible')
                        this.renderSeries(seriesOptions);
                }
            }
            if (this.svgRenderer.vmlNamespace && (this.model.AreaType != "none"))
                this.svgRenderer.append(this.gSeriesEle, this.svgObject);
            if ($(this.svgObject).find("#" + this.svgObject.id + "_StriplineOver").length > 0) {
                var detachEle = $(this.svgObject).find("#" + this.svgObject.id + "_StriplineOver").detach();
                detachEle.appendTo(this.svgObject);
            }

            this.legendRender.drawLegend();

            chartObj.model.initSeriesRender = true;
        },
        _renderStripline: function (zorder) {
            $.extend(ej.EjStripline.prototype, this);
            this.chartstripline = new ej.EjStripline(this);
            var stripline = this.chartstripline;
            for (var sl = 0; sl < this.model.axes.length; sl++) {
                var slAxis = this.model.axes[sl];
                for (var slCount = 0; slCount < slAxis.stripline.length; slCount++) {
                    if (slAxis.stripline[slCount].zOrder.toLowerCase() == zorder)
                        stripline._drawStripline(slAxis, slAxis.stripline[slCount]);
                }
            }
            if (zorder == 'over') {

                this.svgRenderer.append(this.gStriplineOver, this.svgObject);
            } else {
                this.svgRenderer.append(this.gStriplineBehind, this.svgObject);
            }
        },

        renderSeries: function (options) {

            var chart = this;
            var axis;
            var index;
            var seriesIndex = $.inArray(options, this.model._visibleSeries);

            for (var i = 0; i < options.points.length; i++) {
                if (typeof options.points[i].x == "string" && options.points[i].x.indexOf("/Date(") != -1)
                    options.points[i].x = new Date(parseInt(options.points[i].x.substr(6)));
            }
            options.xAxis = (options.xAxis === null || options.xAxis === undefined) ? this.model.axes[0] : options.xAxis;
            options.yAxis = (options.yAxis === null || options.yAxis === undefined) ? this.model.axes[1] : options.yAxis;
            var type = options.type.toLowerCase();
            var symbolOptions;


            var trans = ej.EjSvgRender.utils._getTransform(options.xAxis, options.yAxis, this.model.requireInvertedAxes);
            var txtOptions = { 'id': this.svgObject.id + '_TextGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
            if (type != "scatter")
                symbolOptions = { 'id': this.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
            else
                symbolOptions = { 'id': this.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
            this.gSymbolGroupEle = this.svgRenderer.createGroup(symbolOptions);
            this.gSeriesTextEle = this.svgRenderer.createGroup(txtOptions);

            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.Data = { series: options };
            this._trigger("seriesRendering", commonEventArgs);

            var series = new ej.seriesTypes[type]();
            series.draw(this, options);

            //Add axes bounds clip path for series
            var element = series.gSeriesGroupEle;
            if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);

            // Draw Symbols

            if (options.marker && options.marker.visible) {
                if (!(type == "candle" || type == "hilo" || type == "hiloopenclose")) {
                    $.each(options.points, function (pointIndex, point) {
                        series.drawSymbolImage(options, pointIndex);
                        var point1 = (point.symbolLocation) ? point.symbolLocation : ej.EjSvgRender.utils._getPoint(point, options);
                        point.marker = $.extend(true, {}, options.marker, point.marker);
                        if (point.marker.visible)
                            series.drawSymbol(seriesIndex, options, pointIndex, point1.X, point1.Y);
                    });
                    if (type != "scatter") {
                        this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);
                        //Add axes bounds clip path for marker
                        if (options.marker.visible) {
                            element = this.gSymbolGroupEle;
                            ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                        }
                        //   Add axes bounds clip path for marker text
                        if (options.marker.dataLabel.visible) {
                            element = this.gSeriesTextEle;
                            ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                        }
                    } else {
                        element = this.gSymbolGroupEle;
                        ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                        this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);

                        if (options.tooltip.visible || type == "scatter") {
                            var elements = $(this.gSymbolGroupEle).children().not("defs");
                            if (this.vmlRendering) {
                                this.cloneSeriesEle = $(this.gSeriesEle).clone();
                                this.cloneobj = $(this.element).clone();
                                this.svgclone = $(this.svgObject).clone();
                                $(document.body).append(this.cloneobj);
                                this.svgRenderer.append(this.cloneSeriesEle, this.svgclone);
                                this.svgRenderer.append(this.svgclone, this.cloneobj);
                            }
                            else {
                                this.svgRenderer.append(this.gSeriesEle, this.svgObject);
                                this.svgRenderer.append(this.svgObject, this.element);
                            }


                            for (var j = 0; j < elements.length; j++) {
                                var elementNode = (this.vmlRendering) ? ($("#" + (elements[j].id))[0]) : elements[j];
                                var bound = ej.EjSvgRender.utils._getBoundingClientRect(elementNode, this, options, false);

                                var valwidth, x, y, valheight;
                                x = (bound.x + options.xAxis.x);
                                y = (bound.y + options.yAxis.y);

                                valheight = bound.height;
                                valwidth = bound.width;

                                var bounds = { X: x, Y: y, Width: valwidth, Height: valheight };
                                ej.EjSvgRender.utils._addRegion(this, bounds, options, null, j);
                            }
                            if (this.vmlRendering) {
                                $(this.cloneobj).remove();
                                $(this.svgclone).remove();
                            }
                        }
                    }
                }

            }

            //Draw Series Text

            this.drawDisplayText(series, options);

            if (!this.vmlRendering) {
                this.svgRenderer.append(this.gSeriesEle, this.svgObject);
                this.svgRenderer.append(this.svgObject, this.element);
                if (options.animation) {
                    var templateContainer = $('#template_group_' + this._id);
                    var dataLabel = options.marker.dataLabel;
                    if (templateContainer.children().length != 0 && (dataLabel.visible && dataLabel.template))
                        series.animateLabelTemplate(options);
                    series.doAnimation(options);
                    this.model.AnimationComplete = true;
                    var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonEventArgs.Data = { Series: options };
                    this._trigger("animationComplete", commonEventArgs);
                }
            }

        },

        drawDisplayText: function (series, options) {
            // method to draw text 
            $.each(options.points, function (pointIndex, point) {
                var point1 = (point.symbolLocation) ? point.symbolLocation : ej.EjSvgRender.utils._getPoint(point, options);
                point.marker = $.extend(true, {}, options.marker, point.marker);
                if (point.visible && point.marker.dataLabel.visible != false && (point.marker.dataLabel.visible || (options.marker.dataLabel.visible)))
                    series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y);
            });

            this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);
            this.svgRenderer.append(this.gSeriesTextEle, this.gSeriesEle);
        },

        drawAccDisplayText: function (series, options) {
            // method to draw text 
            var chartObj = this;

            if (options.type.toLowerCase() != "pyramid" && options.leftsidePoints.length > 0) {
                chartObj.model.bounds["left"] = [];
                chartObj.model.bounds["right"] = [];

                for (var i = 0; i < options.rightsidePoints.length; i++) {
                    options.rightsidePoints[i].index = i;
                    series.drawDataLabelAcc(chartObj, options, options.rightsidePoints[i].pointIndex, options.rightsidePoints[i])
                }
                for (var j = options.leftsidePoints.length - 1; j >= 0; j--) {
                    options.leftsidePoints[j].index = j;
                    series.drawDataLabelAcc(chartObj, options, options.leftsidePoints[j].pointIndex, options.leftsidePoints[j])
                }
            }
            else {
                for (var i = 0; i < options.visiblePoints.length; i++) {
                    options.visiblePoints[i].index = i;
                    series.drawDataLabelAcc(chartObj, options, i, options.visiblePoints[i])
                }
            }


            this.svgRenderer.append(this.gSeriesConnectorEle, this.gSeriesEle);

            this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);

            this.svgRenderer.append(this.gSeriesTextEle, this.gSeriesEle);

            this.svgRenderer.append(this.gSeriesEle, this.svgObject);

            if (!this.vmlRendering)
                this.svgRenderer.append(this.svgObject, this.element);
        },

        _processOData: function (series) {
            var chart = this;
            var queryPromise = series.dataSource.data.executeQuery(series.dataSource.query);
            queryPromise.done(function (e) {
                chart._processJsonData(e.result, series);
                series.visibility = 'visible';
                $(chart.svgObject).empty();
                var redrawChart = true;
                for (var i = 0; i < chart.model.series.length; i++)
                    if (chart.model.series[i].dataSource) {
                        if (chart.model.series[i].visibility == 'hidden') {
                            redrawChart = false;
                            break;
                        }
                    }
                if (redrawChart) {
                    chart.draw();
                }
            });
            queryPromise.fail(function (e) {
                series.visibility = 'visible';
                $(chart.svgObject).empty();
                var redrawChart = true;
                for (var i = 0; i < chart.model.series.length; i++)
                    if (chart.model.series[i].dataSource) {
                        if (chart.model.series[i].visibility == 'hidden') {
                            redrawChart = false;
                            break;
                        }
                    }
                if (redrawChart) {
                    chart.draw();
                }
            });
        },
        _processJsonData: function (jsonObj, series) {
            if (series.points.length > 0)
                series.points.length = 0;
            var xdata = [];
            var dataSource = series.dataSource;
            var xName = dataSource.xName;
            var yNames = dataSource.yName;

            for (var j = 0; j < jsonObj.length; j++) {
                if (typeof jsonObj[j][xName] == "string" && jsonObj[j][xName].indexOf("/Date(") != -1) {
                    jsonObj[j][xName] = new Date(parseInt(jsonObj[j][xName].substr(6)));
                    xdata.push((jsonObj[j][xName]) ? jsonObj[j][xName] : j);
                }
                else
                    xdata.push((jsonObj[j][xName]) ? jsonObj[j][xName] : j);
            }
            var yValues = [];
            var index = 0;
            if (!yNames) {
                if (series._hiloTypes) {
                    var yVal = [];
                    //For high
                    for (var s = 0; s < jsonObj.length; s++) {
                        yVal.push(jsonObj[s][dataSource.high] === null || jsonObj[s][dataSource.high] === undefined ? jsonObj[s][dataSource.high] : parseFloat(jsonObj[s][dataSource.high]));

                    }
                    yValues[index] = yVal;
                    index = index + 1;
                    //For low
                    yVal = [];
                    for (var l = 0; l < jsonObj.length; l++) {
                        yVal.push(jsonObj[l][dataSource.low] === null || jsonObj[l][dataSource.low] === undefined ? jsonObj[l][dataSource.low] : parseFloat(jsonObj[l][dataSource.low]));
                    }
                    yValues[index] = yVal;
                    index = index + 1;
                    if (series.type.toLowerCase() !== 'hilo') {
                        //For open
                        yVal = [];
                        for (var h = 0; h < jsonObj.length; h++) {
                            yVal.push(jsonObj[h][dataSource.open] === null || jsonObj[h][dataSource.open] === undefined ? jsonObj[h][dataSource.open] : parseFloat(jsonObj[h][dataSource.open]));
                        }
                        yValues[index] = yVal;
                        index = index + 1;

                        //For close
                        yVal = [];
                        for (var mI = 0; mI < jsonObj.length; mI++) {
                            yVal.push((jsonObj[mI][dataSource.close] === null || jsonObj[mI][dataSource.close] === undefined) ? jsonObj[mI][dataSource.close] : parseFloat(jsonObj[mI][dataSource.close]));
                        }
                        yValues[index] = yVal;
                    }
                }
            }
            if (yNames) {

                var yNVal = [];
                for (var yI = 0; yI < jsonObj.length; yI++) {
                    yNVal.push((jsonObj[yI][yNames]) ? parseFloat(jsonObj[yI][yNames]) : jsonObj[yI][yNames]);
                }
                yValues[index] = yNVal;
                index = index + 1;


                if (series.type.toLowerCase() == 'bubble') {
                    yVal = [];
                    for (var b = 0; b < jsonObj.length; b++) {
                        yVal.push(jsonObj[b][dataSource.size] === null || jsonObj[b][dataSource.size] === undefined ? jsonObj[b][dataSource.size] : parseFloat(jsonObj[b][dataSource.size]));
                    }
                    yValues[index] = yVal;
                }
            }


            for (var m = 0; m < xdata.length; m++) {
                series.points[m] = { x: xdata[m], X: xdata[m], y: yValues[0][m], YValues: [], visible: true };
                if (series.type.toLowerCase() == 'bubble') {
                    series.points[m].YValues.push(yValues[0][m]);
                    series.points[m].size = series.points[m].YValues[1] = yValues[1][m];
                }
                else if (series._hiloTypes) {
                    series.points[m].high = series.points[m].YValues[0] = yValues[0][m];
                    series.points[m].low = series.points[m].YValues[1] = yValues[1][m];
                    if (series.type.toLowerCase() != 'hilo' && series.type.toLowerCase() != 'rangecolumn') {
                        series.points[m].open = series.points[m].YValues[2] = yValues[2][m];
                        series.points[m].close = series.points[m].YValues[3] = yValues[3][m];
                    }
                }
                else {
                    series.points[m].YValues.push(yValues[0][m]);
                }
            }
        },
        bindTo: function () {
            if (this.model.series) {
                var series = this.model.series;
                if (!this.zoomed && !this.panning) {
                    this.setModelProperties();
                    for (var i = 0; i < series.length; i++) {
                        if (series[i].visibility == null || series[i].visibility == undefined) {
                            series[i].visibility = 'visible';
                            if (series[i].type) series[i]._hiloTypes = ej.seriesTypes[series[i].type.toLowerCase()].prototype.hiloTypes;
                        }
                        if (!(series[i].points))
                            series[i].points = [];
                        if (series[i].dataSource) {
                            if (series[i].dataSource.data instanceof ej.DataManager) {
                                series[i].visibility = 'hidden';
                                this._processOData(series[i]);
                            } else if (series[i].dataSource.data != null && series[i].dataSource.data.length > 0) {

                                this._processJsonData(series[i].dataSource.data, series[i]);

                            }
                        }
                    }
                }
                this.draw();
            }



        },

        chartResize: function () {
            var chart = this;
            var $svgObj = $(chart.svgObject);
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                if (!chart.isDevice()) {
                    $svgObj.width($(chart.element).width());
                    if (chart.svgHeight > $(this).height()) {
                        $svgObj.height($(this).height());
                    } else {
                        $svgObj.height(chart.svgHeight);
                    }
                }
                else {
                    $svgObj.width($(chart.element).width());
                    $svgObj.height($(chart.element).height());
                }
                $(chart.svgObject).empty();
                $('#template_group_' + chart._id).remove();
                chart.disableAnimation();
                chart.draw();
                if (chart.zoomed)
                    chart._enableZoomingButtons();
                chart.enableAnimation();
            }, 500);

        },
        isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            //  return (/mobile|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        bindEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);

            this._on($(this.svgObject), "touchstart", this.chartMouseDown);
            this._on(this.element, "touchmove", this.chartMouseMove);
            this._on($(document), "touchend", this.chartMouseUp);
            this._on(this.element, 'touchleave', this.chartMouseLeave);

            this._on($(this.svgObject), "mousedown", this.chartMouseDown);
            this._on(this.element, 'mouseleave', this.chartMouseLeave);
            this._on($(document), "mouseup", this.chartMouseUp);

            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (this.isDevice() && !isIE11 && !this.vmlRendering) {
                if (window.navigator.msPointerEnabled) {
                    this._on(this.element, 'MSPointerMove', this.chartMouseMove);
                    this._on(this.element, 'MSPointerDown', this.chartMouseDown);
                    this._on(this.element, 'MSPointerOut', this.chartMouseLeave);
                    this._on($(document), "MSPointerUp", this.chartMouseUp);
                    this._on($(this.svgObject), 'click', this.chartMouseClick);
                    if (this.model.zooming.enable)
                        $(this.svgObject).css('-ms-touch-action', 'none');
                }
                else {
                    this._on(this.element, 'touchmove', this.chartTouchMove);
                    this._on($(this.svgObject), 'touchstart', this.chartTouchClick);
                }
            }
            else {
                this._on(this.element, 'mousemove', this.chartMouseMove);
                this._on($(this.svgObject), 'click', this.chartMouseClick);
            }
            if (this.model.canResize) {
                if (!this.isDevice() || window.navigator.msPointerEnabled)
                    this._on($(window), "resize", this.chartResize);
                else
                    this._on($(window), "orientationchange", this.chartResize);
            }

            var eventName = matched.browser.toLowerCase() == "mozilla" ? "DOMMouseScroll" : "mousewheel";
            this._on(this.element, eventName, this.chartMouseWheel);

        },
        calZoomFactors: function (cumulativeScale, origin, currentZoomFactor, currentZoomPos) {
            var calcZoomFactorVal, calcZoomPosition;
            if (cumulativeScale == 1) {
                calcZoomFactorVal = 1;
                calcZoomPosition = 0;
            } else {
                calcZoomFactorVal = ej.EjSvgRender.utils._minMax(1 / cumulativeScale, 0, 1);
                calcZoomPosition = currentZoomPos + ((currentZoomFactor - calcZoomFactorVal) * origin);
            }
            return { zoomMFactor: calcZoomFactorVal, zoomMPosition: calcZoomPosition };
        },
        doMouseWheelZoom: function (cumulativeScale, origin, axis) {
            if (cumulativeScale >= 1) {
                var calZoomVal = this.calZoomFactors(cumulativeScale, origin, axis.zoomFactor, axis.zoomPosition);
                //calcZoomPos = Math.Round(calcZoomPos, 2);
                if (axis.zoomPosition != calZoomVal.zoomMPosition) {
                    this.zoomed = true;
                    axis.zoomPosition = calZoomVal.zoomMPosition;
                    axis.zoomFactor = (calZoomVal.zoomMPosition + calZoomVal.zoomMFactor) > 1 ? (1 - calZoomVal.zoomMPosition) : calZoomVal.zoomMFactor;
                    return true;
                }
            }

            return false;

        },
        enableAnimation: function () {
            for (var m = 0; m < this.model._visibleSeries.length; m++) {
                var series = this.model._visibleSeries[m];
                series.animation = this.serAnimation[m];
            }
        },
        disableAnimation: function () {
            var chart = this;
            for (var m = 0; m < chart.model._visibleSeries.length; m++) {
                chart.model.series[m].animation = false;

            }
        },
        _removeTrackBall: function () {
            var id = "#" + this.svgObject.id;
            $(this.svgObject).find(id + "_Tracker," + id + "_TrackAxisToolTip," + id + "_CrosshairVertical," + id + "_CrosshairHorizontal," + id + "_AxisCrossToolTip").remove();
            $(this.svgObject).find('[id*="_TrackSymbol_"],[id*="_TrackToolTip"]').remove();
            $(document).find('[id*="_TrackToolTipTemplate_"]').remove();
        },
        enableTrackBall: function () {
            this.model.crosshair.visible = this.chartCross.visible;
            this.model.crosshair.type = this.chartCross.mArea;
        },
        disableTrackBall: function () {

            this.model.crosshair.visible = false;

            this._removeTrackBall();
        },
        _enableZoomingButtons: function () {
            var chart = this;
            if ($(chart.svgObject).find("#" + this.svgObject.id + '_ResetZoom').length == 0) {
                this.resetZoomButton();
                this.panButton();
                this.zoomButton();
            }
        },
        chartMouseWheel: function (e) {
            if (this.model.zooming.enableMouseWheel) {
                var chart = this;
                var chartZoomed;
                this.zoomed = false;
                var canUpdate = false;
                var id = "#" + chart.svgObject.id;
                var matched = jQuery.uaMatch(navigator.userAgent);
                var direction = matched.browser.toLowerCase() == "mozilla" ? (-(e.originalEvent.detail) / 3 > 0 ? 1 : -1) : ((e.originalEvent.wheelDelta / 120) > 0 ? 1 : -1);
                for (var i = 0; i < this.model.axes.length; i++) {
                    var axis = this.model.axes[i];
                    if ((axis.orientation.toLowerCase() == "vertical" && (chart.model.zooming.type.toLowerCase() == "y" || chart.model.zooming.type.trim().toLowerCase() == "x,y")) || (axis.orientation.toLowerCase() == "horizontal" && (chart.model.zooming.type.toLowerCase() == "x" || chart.model.zooming.type.trim().toLowerCase() == "x,y"))) {

                        var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                        var cumulativeScale = Math.max(currentScale + (0.25 * direction), 1);
                        canUpdate = canUpdate | chart.doMouseWheelZoom(cumulativeScale, 0.5, axis);
                    }
                    if (axis.zoomFactor != 1 && axis.zoomPosition != 0)
                        chartZoomed = true;
                }

                this.disableAnimation();

                this.disableTrackBall();
                if (canUpdate) {
                    this.zoomed = true;
                    chart.redraw();
                }

                this._enableZoomingButtons();
                this.mouseWheelCoords = { x: e.originalEvent.pageX, y: e.originalEvent.pageY };
                if (!chartZoomed) {
                    $(chart.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').remove();
                    this.enableAnimation();
                    this.panning = false;
                }
                var matched = jQuery.uaMatch(navigator.userAgent);
                if (e.preventDefault && matched.browser == 'chrome' && this.model.zooming.enable)
                    e.preventDefault();
                e.returnValue = false;
            }

            $(".tooltipDiv").remove();
        },
        getLegendData: function (evt) {
            var mouseMoveCords = this.calMousePosition(evt);
            this.mousemoveX = mouseMoveCords.X;
            this.mousemoveY = mouseMoveCords.Y;

            {
                var currentX = this.mousemoveX;
                var currentY = this.mousemoveY;
                var region;
                if (!ej.util.isNullOrUndefined(evt.target.id) && this.svgRenderer._getAttrVal($(evt.target).parents(':eq(1)'), 'id') == this.svgObject.id + '_Legend') {
                    $.each(this.model.legendRegion, function (index, regionItem) {

                        if ((currentX >= regionItem.Bounds.LegendBound.X + regionItem.Bounds.ItemBound.X) && (currentX <= regionItem.Bounds.LegendBound.X + regionItem.Bounds.ItemBound.X + regionItem.LegendItem.Bounds.Width)) {
                            if ((currentY >= regionItem.Bounds.LegendBound.Y + regionItem.Bounds.ItemBound.Y - (regionItem.LegendItem.Bounds.Height / 2)) && (currentY <= regionItem.Bounds.LegendBound.Y + regionItem.Bounds.ItemBound.Y + regionItem.LegendItem.Bounds.Height))
                                region = regionItem;
                        }
                    });
                    if (!ej.util.isNullOrUndefined(region)) {
                        return { LegendItem: region, Series: this.model._visibleSeries[region.LegendItem.SeriesIndex] };
                    }
                }

            }
            return false;
        },



        GetSeriesPoint: function (evt) {
            var mouseX;
            var mouseY;
            var currentX = this.mousemoveX;
            var currentY = this.mousemoveY;
            var region;
            if (this.model.AreaType == 'cartesianaxes') {
                if (currentX > this.model.m_AreaBounds.X && currentX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && currentY > this.model.m_AreaBounds.Y && currentY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height)) {
                    $.each(this.model.chartRegions, function (index, regionItem) {
                        if ((((currentX >= regionItem.Region.Bounds.X) && (currentX <= (regionItem.Region.Bounds.X + (regionItem.Region.Bounds.Width))))) && ((currentY > regionItem.Region.Bounds.Y) && (currentY < (regionItem.Region.Bounds.Y) + (regionItem.Region.Bounds.Height)))) {
                            region = regionItem;
                            return false;
                        }

                    });
                }
                if (region) {
                    return { Region: region, location: { X: currentX, Y: currentY } };
                }
            }

            else {
                var chartObj = this;
                var distanceFromCenter;
                var regionValue;
                if (chartObj.model.chartRegions[0].Series.type.toLowerCase() == "pyramid") {
                    var nodeName = $(evt.target)[0].nodeName;
                    if ((nodeName == "path" || nodeName == "shape") && $($(evt.target).parent())[0].id == chartObj.svgObject.id + '_SeriesGroup' + '_' + chartObj.model.chartRegions[0].SeriesIndex) {
                        regionValue = chartObj.model.chartRegions[0];
                        index = parseInt(chartObj.svgRenderer._getAttrVal($(evt.target)[0], "pointIndex"));
                        regionValue.Region.PointIndex = index;
                        var pointData = {};
                        pointData.pointIndex = index;
                        region = { Region: regionValue, PointData: pointData, location: { X: currentX, Y: currentY } };
                        return region;
                    }
                }
                else {
                    // ReSharper disable DuplicatingLocalDeclaration
                    $.each(chartObj.model.chartRegions, function (regionIndex, regionValue) {
                        // ReSharper restore DuplicatingLocalDeclaration
                        var chartStartAngle = -.5 * Math.PI;
                        var seriesData = regionValue.SeriesData; var regionData = regionValue.Region;
                        var innerRadius = (chartObj.model._visibleSeries[regionValue.SeriesIndex].type.toLowerCase() == 'doughnut') ? seriesData.DRadius : 0;
                        var fromCenterX;
                        var fromCenterY;
                        fromCenterX = (currentX) - (seriesData.CenterX);
                        fromCenterY = (currentY) - (seriesData.CenterY);
                        var series = chartObj.model._visibleSeries[regionValue.SeriesIndex];
                        var arcAngle = (series.startAngle) ? 2 * Math.PI * chartObj.model.itemCurrentXPos : 0;
                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle);
                        if (clickAngle < 0) clickAngle = 2 * Math.PI + clickAngle;

                        pointData = [];
                        for (var i = 0; i < regionData.length; i++) {
                            var startAngle = regionData[i].StartAngle - arcAngle;
                            var endAngle = regionData[i].EndAngle - arcAngle;
                            if (clickAngle >= startAngle && clickAngle <= endAngle) {
                                pointData.push(regionData[i]);
                                break;
                            }
                        }
                        if (pointData.length > 0) {
                            var isExploded = (series.explodeAll || (series.explodeIndex == pointData[0].PointIndex));
                            if (isExploded) {

                                var fromCenterX1 = currentX - pointData[0].StartX;
                                var fromCenterY1 = currentY - pointData[0].StartY;

                                clickAngle = Math.atan2(fromCenterY1, fromCenterX1) - chartStartAngle;
                                if (clickAngle < 0) clickAngle = 2 * Math.PI + clickAngle;
                                pointData = [];
                                for (var j = 0; j < regionData.length; j++) {
                                    if (clickAngle >= regionData[j].StartAngle && clickAngle <= regionData[j].EndAngle)
                                        pointData.push(regionData[j]);
                                }
                                if (pointData.length > 0) {

                                    distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX1), 2) + Math.pow(Math.abs(fromCenterY1), 2));
                                    if (distanceFromCenter <= seriesData.Radius && distanceFromCenter > innerRadius) {
                                        regionData.PointIndex = pointData[0].PointIndex;
                                        region = { Region: regionValue, PointData: pointData, location: { X: currentX, Y: currentY } };
                                    }
                                }
                            }
                            else {
                                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                                if (distanceFromCenter <= seriesData.Radius && distanceFromCenter > innerRadius) {
                                    regionData.PointIndex = pointData[0].PointIndex;
                                    region = { Region: regionValue, PointData: pointData, location: { X: currentX, Y: currentY } };
                                }
                            }
                        }

                    });

                    return region;
                }
            }
        },

        mousePosition: function (evt) {
            if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX > 0)
                return { x: evt.pageX, y: evt.pageY };
            else if (!ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX > 0)
                return { x: evt.originalEvent.pageX, y: evt.originalEvent.pageY };
            else if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                return { x: evt.originalEvent.changedTouches[0].pageX, y: evt.originalEvent.changedTouches[0].pageY };
        },


        calMousePosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseposition = this.mousePosition(e);
            e.pageX = mouseposition.x;
            e.pageY = mouseposition.y;
            var mouseX, mouseY;
            var browser = matched.browser.toLowerCase();
            if (browser == "webkit" || browser == "mozilla") {
                mouseX = (e.pageX) - $(this.svgObject).parent().offset().left;
                mouseY = (e.pageY) - $(this.svgObject).parent().offset().top;
            } else {
                mouseX = (e.pageX) - $(this.svgObject).offset().left;
                mouseY = (e.pageY) - $(this.svgObject).offset().top;
            }

            return { X: mouseX, Y: mouseY };

        },

        cancelEvent: function (e) {
            e.stopPropagation();
            var matched = jQuery.uaMatch(navigator.userAgent);
            if (matched.browser == 'chrome' && this.model.zooming.enable) {
                e.preventDefault();
            }
            e.returnValue = false;
            e.cancelBubble = true;
            return false;
        },
        resetZoom: function () {
            var chartobj = this;
            var id = "#" + chartobj.svgObject.id;
            if (chartobj.zoomed) {
                $.each(this.model.axes, function (index, axis) {
                    axis.zoomFactor = 1;
                    axis.zoomPosition = 0;
                });
                $(chartobj.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').remove();
                chartobj.zoomed = false;
                chartobj.panning = false;
                chartobj.redraw();
                chartobj.enableAnimation();
                chartobj.enableTrackBall();
                chartobj.svgRenderer._setAttr($(chartobj.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection"), { "cursor": "default" });
            }
        },
        startZoom: function () {
            var chartObj = this;
            var id = "#" + chartObj.svgObject.id;
            var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
            chartObj.svgRenderer._setAttr($(id + '_ZoomBtn'), { 'selected': 'true' });
            chartObj.svgRenderer._setAttr($(id + '_ZoomPath'), { "fill": "#14B9FF" });
            chartObj.svgRenderer._setAttr($(id + '_ZoomIcon'), { "fill": '#FFFFFF' });
            chartObj.svgRenderer._setAttr($(id + '_PanPath'), { "fill": '#B7B7B7' });
            chartObj.svgRenderer._setAttr($(id + '_panIcon'), { "fill": iconColor });
            this._cursorToDefault();
            chartObj.panning = false;
            chartObj.enableTrackBall();
        },
        startPan: function () {
            var chartObj = this;
            var id = "#" + chartObj.svgObject.id;
            var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
            chartObj.svgRenderer._setAttr($(id + '_ZoomBtn'), { 'selected': 'false' });
            chartObj.svgRenderer._setAttr($(id + '_PanBtn'), { 'selected': 'true' });
            chartObj.svgRenderer._setAttr($(id + '_ZoomPath'), { "fill": "#B7B7B7" });
            chartObj.svgRenderer._setAttr($(id + '_ZoomIcon'), { "fill": iconColor });
            chartObj.svgRenderer._setAttr($(id + '_PanPath'), { "fill": '#14B9FF' });
            chartObj.svgRenderer._setAttr($(id + '_panIcon'), { "fill": '#FFFFFF' });
            this._cursorToPointer();
            chartObj.panning = true;
            chartObj.disableTrackBall();

        },
        zoomButton: function () {
            var padding = 20;
            var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (3 * 32) - padding); //32 path bouding box for pan,reset, zoom button 
            var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
            var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ZoomBtn', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'true' });

            var pathOptions = {
                'id': this.svgObject.id + '_ZoomPath',
                'd': "M32,27c0,2.75-2.25,5-5,5H5c-2.75,0-5-2.25-5-5V5c0-2.75,2.25-5,5-5h22c2.75,0,5,2.25,5,5V27z",
                'fill': this.panning ? '#b7b7b7' : '#14B9FF'
            };

            this.svgRenderer.drawPath(pathOptions, gEle);

            var options = {
                'id': this.svgObject.id + '_ZoomIcon',
                'd': "M26.101,22.893l-6.605-6.174c1.414-2.781,0.897-6.267-1.496-8.504c-2.901-2.711-7.448-2.56-10.161,0.341    c-2.712,2.9-2.56,7.45,0.341,10.163c2.426,2.266,6,2.523,8.694,0.853l6.579,6.151L26.101,22.893z M10.186,16.571    c-1.715-1.604-1.805-4.293-0.203-6.009c1.605-1.715,4.295-1.805,6.009-0.201c1.715,1.603,1.805,4.293,0.202,6.007    C14.59,18.084,11.901,18.175,10.186,16.571",
                'fill': this.panning ? iconColor : '#FFFFFF'
            };
            this.svgRenderer.drawPath(options, gEle);
            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.startZoom);

        },
        panButton: function () {

            var padding = 15;
            var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (2 * 32) - padding); //32 path bounding box for pan and resetzoom button
            var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
            var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_PanBtn', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'false' });

            var pathOptions = {
                'id': this.svgObject.id + '_PanPath',
                'd': "M 32 27 c 0 2.75 -2.25 5 -5 5 H 5 c -2.75 0 -5 -2.25 -5 -5 V 5 c 0 -2.75 2.25 -5 5 -5 h 22 c 2.75 0 5 2.25 5 5 V 27 Z",
                'fill': !this.panning ? '#b7b7b7' : '#14B9FF'
            };

            this.svgRenderer.drawPath(pathOptions, gEle);

            var options = {
                'id': this.svgObject.id + '_panIcon',
                'points': "26.105,16 21.053,12.211 21.053,14.737 17.263,14.737 17.263,10.947 19.834,10.947 16.044,5.895 12.255,10.947 14.737,10.947 14.737,14.737 10.947,14.737 10.947,12.211 5.895,16 10.947,19.789 10.947,17.263 14.737,17.263 14.737,21.053 12.255,21.053 16.044,26.105 19.834,21.053 17.263,21.053 17.263,17.263 21.053,17.263 21.053,19.789",
                'fill': !this.panning ? iconColor : '#FFFFFF'
            };
            this.svgRenderer.drawPolygon(options, gEle);
            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.startPan);


        },
        resetZoomButton: function () {

            var padding = 10;
            var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (32 + padding)); //32 path bouding box
            var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ResetZoom', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'false' });
            var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
            var options = {
                'id': this.svgObject.id + '_ResetZoomPath',
                'd': "M 32 27 c 0 2.75 -2.25 5 -5 5 H 5 c -2.75 0 -5 -2.25 -5 -5 V 5 c 0 -2.75 2.25 -5 5 -5 h 22 c 2.75 0 5 2.25 5 5 V 27 Z",
                'fill': '#b7b7b7'
            };
            this.svgRenderer.drawPath(options, gEle);

            var ptOptions = {
                'id': this.svgObject.id + '_ptIcon',
                'points': "11.895,18.398 8.061,22.23 5.796,19.967 5.796,26.283 12.112,26.283 9.848,24.018 13.682,20.186",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(ptOptions, gEle);
            var pt1Options = {
                'id': this.svgObject.id + '_ptIcon1',
                'points': "19.691,6.072 21.955,8.337 18.121,12.172 19.908,13.959 23.742,10.123 26.007,12.389 26.007,6.072",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt1Options, gEle);

            var pt2Options = {
                'id': this.svgObject.id + '_ptIcon2',
                'points': "11.895,13.958 13.682,12.172 9.848,8.337 12.112,6.072 5.796,6.072 5.796,12.389 8.061,10.123",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt2Options, gEle);

            var pt3Options = {
                'id': this.svgObject.id + '_ptIcon3',
                'points': "19.908,18.396 18.121,20.186 21.955,24.018 19.691,26.283 26.007,26.283 26.007,19.967 23.741,22.23",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt3Options, gEle);

            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.resetZoom);


        },

        doZoom: function (zoomRect, zoomRectWidth, zoomRectHeight) {
            var chart = this;
            $("#" + this.svgObject.id + "_ZoomArea").remove();
            this.drag = false;
            this.zoomed = true;
            for (var k = 0; k < this.model.axes.length; k++) {
                var axis = this.model.axes[k];
                var previousZoomFactor = axis.zoomFactor;
                var previousZoomPosition = axis.zoomPosition;
                if (axis.orientation.toLowerCase() == "horizontal") {
                    axis.zoomFactor = chart.model.zooming.type.toLowerCase() != "y"
                        ? previousZoomFactor * (zoomRectWidth / (chart.model.m_AreaBounds.Width)) : 1;
                    axis.zoomPosition = chart.model.zooming.type.toLowerCase() != "y"
                        ? previousZoomPosition + Math.abs((parseFloat(chart.svgRenderer._getAttrVal($(zoomRect), "x")) - chart.model.m_AreaBounds.X) / (chart.model.m_AreaBounds.Width)) * previousZoomFactor : 0;
                    if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                        axis.zoomFactor = previousZoomFactor;
                        axis.zoomPosition = previousZoomPosition;

                    }
                } else {
                    axis.zoomFactor = chart.model.zooming.type.toLowerCase() != "x"
                        ? previousZoomFactor * zoomRectHeight / chart.model.m_AreaBounds.Height : 1;
                    axis.zoomPosition = chart.model.zooming.type.toLowerCase() != "x"
                        ? previousZoomPosition + (1 - Math.abs((zoomRectHeight + (parseFloat(chart.svgRenderer._getAttrVal($(zoomRect), "y")) - chart.model.m_AreaBounds.Y)) / (chart.model.m_AreaBounds.Height))) * previousZoomFactor : 0;
                    if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                        axis.zoomFactor = previousZoomFactor;
                        axis.zoomPosition = previousZoomPosition;

                    }
                }
            }

            this.disableAnimation();
            this.redraw();

            this._enableZoomingButtons();
        },
        _cursorToDefault: function () {
            var id = "#" + this.svgObject.id;
            this.svgRenderer._setAttr($(this.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection," + id + "_StriplineBehind," + id + '_StriplineOver'), { "cursor": "default" });
        },
        _cursorToPointer: function () {
            var id = "#" + this.svgObject.id;
            this.svgRenderer._setAttr($(this.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection," + id + "_StriplineBehind," + id + '_StriplineOver'), { "cursor": "pointer" });
        },
        chartMouseUp: function (evt) {
            // this.cancelEvent(evt);
            var chart = this;
            if (this.model.zooming.enable && this.model.AreaType == "cartesianaxes") {
                var zoomRect = $("#" + this.svgObject.id + "_ZoomArea");
                var $zoomRect = $(zoomRect);
                var zoomRectWidth = parseFloat(this.svgRenderer._getAttrVal($zoomRect, "width"));
                var zoomRectHeight = parseFloat(this.svgRenderer._getAttrVal($zoomRect, "height"));
                if (zoomRectWidth > 0 && zoomRectHeight > 0) {
                    chart.doZoom(zoomRect, zoomRectWidth, zoomRectHeight);
                }
                else
                    this.drag = false;
                if (!chart.panning)
                    this.enableTrackBall();
            }

            if (this.doPan)
                this.doPan = false;
        },

        chartMouseDown: function (e) {
            this.cancelEvent(e);
            if (!this.svgRenderer.vmlNamespace) {
                this.oPreviousCoords = {};
                if (this.svgRenderer._getAttrVal($(e.target).parent(), "id") != this.svgObject.id + "_ResetZoom" && this.svgRenderer._getAttrVal($(e.target).parent(), "id") != this.svgObject.id + "_PanBtn" && this.model.zooming.enable && !(this.panning) && this.model.AreaType == "cartesianaxes") {
                    var mouseDownCords = this.calMousePosition(e);
                    this.mouseDownX = mouseDownCords.X;
                    this.mouseDownY = mouseDownCords.Y;
                    if (this.mouseDownX >= this.model.m_AreaBounds.X && this.mouseDownX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mouseDownY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mouseDownY >= this.model.m_AreaBounds.Y) {
                        this.drag = true;
                        this.disableTrackBall();
                    }


                }
                else if (this.panning) {
                    var mousePanCords = this.calMousePosition(e);
                    this.mousePanX = mousePanCords.X;
                    this.mousePanY = mousePanCords.Y;
                    if (this.mousePanX >= this.model.m_AreaBounds.X && this.mousePanX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mousePanY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mousePanY >= this.model.m_AreaBounds.Y) {
                        this.doPan = true;
                        this.oPreviousCoords = {
                            'x': e.pageX,
                            'y': e.pageY
                        };
                    }
                }


            }
        },
        chartMouseLeave: function (evt) {
            this.cancelEvent(evt);
            $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackToolTip").remove();
            if (!($(".tooltipDiv").hasClass(this.svgObject.id)))
                $(".tooltipDiv").remove();
            this._removeTrackBall();

        },
        chartTouchClick: function (evt) {
            this.cancelEvent(evt);
            evt = evt.originalEvent.touches[0];
            this._doClick(evt);
        },
        _doClick: function (evt) {
            var seriesIndex;
            var chartSeries;
            for (var i = 0; i < this.model._visibleSeries.length; i++) {
                seriesIndex = i;
                chartSeries = this.model._visibleSeries[i];
                if (this.model.AreaType == "cartesianaxes" && chartSeries.visibility.toLowerCase() == 'visible' && chartSeries.type.toLowerCase() !== "scatter" && chartSeries.type.toLowerCase() !== "bubble" && chartSeries.type.toLowerCase() !== "column" && chartSeries.type.toLowerCase() !== "bar" && chartSeries.type.toLowerCase() !== "stackingbar" && chartSeries.type.toLowerCase() !== "stackingcolumn" && !chartSeries._hiloTypes) {
                    var serY = [];
                    var serX = [];
                    var location = null;
                    var mouseMoveCords = this.calMousePosition(evt);
                    this.mousemoveX = mouseMoveCords.X;
                    this.mousemoveY = mouseMoveCords.Y;
                    var closestXyPoint = this.getClosesPointXY(serX, serY, chartSeries, this.mousemoveX, this.mousemoveY);
                    if (!ej.util.isNullOrUndefined(closestXyPoint.point)) {
                        var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                        commonPointEventArgs.Data = { location: { X: this.mousemoveX, Y: this.mousemoveY }, Region: { SeriesIndex: i, Region: { PointIndex: closestXyPoint.index } } };
                        this._trigger("pointRegionClick", commonPointEventArgs);
                    }
                }
            }
            var data = this.GetSeriesPoint(evt);
            if (data) {
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.Data = data;
                this._trigger("pointRegionClick", commonEventArgs);
            }
            var legenddata = this.getLegendData(evt);
            if (legenddata) {
                $('#template_group_' + this._id).remove();
                var commonEventArgsCommonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgsCommonEventArgs.Data = legenddata;
                this._trigger("legendItemClick", commonEventArgsCommonEventArgs);
                if (this.model.AreaType == "cartesianaxes" && !this.zoomed) {
                    legenddata.Series.visibility = legenddata.Series.visibility.toLowerCase() === 'visible' ? 'hidden' : 'visible';
                    $(this.svgObject).empty();
                    this.model.legendCollapsed = true;
                    this.disableAnimation();
                    this.draw();
                    this.enableAnimation();
                }
            }
        },
        chartMouseClick: function (evt) {

            this.cancelEvent(evt);
            if (window.navigator.msPointerEnabled) {
                evt = evt.originalEvent;
            }
            this._doClick(evt);
        },

        drawTrackerSymbol: function (series, seriesIndex, ptIndex, tracker, point) {

            var type = series.type.toLowerCase();
            var chartSeriesObj = new ej.seriesTypes[type]();

            var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
            if (this.model.AreaType == "cartesianaxes") {
                if (((point.X + trans.x) <= (trans.x + trans.width)) && ((point.X + trans.x) >= trans.x || point.X == 0) &&
                     (Math.abs(point.Y + trans.y) >= (trans.y) || point.Y == 0)) {
                    if (this.model.crosshair.visible || type == "bubble" || type == "scatter") {
                        if ($(this.svgObject).find("#" + this.svgObject.id + '_TrackSymbolClipRect').length == 0 && !this.vmlRendering) {
                            var clipRectOptions = {
                                'id': this.svgObject.id + '_TrackSymbolClipRect',
                                'x': 0,
                                'y': 0,
                                'width': this.model.m_AreaBounds.Width,
                                'height': this.model.m_AreaBounds.Height,
                                'fill': 'white',
                                'stroke-width': 1,
                                'stroke': 'Gray'
                            };

                            this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);
                        }
                        var transSymbolOptions = { 'clip-path': 'url(#' + this.svgObject.id + '_TrackSymbolClipRect)', 'id': this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                        this.gTrackerEle = this.svgRenderer.createGroup(transSymbolOptions);
                    }
                    if (this.model.crosshair.marker.visible)
                        chartSeriesObj.drawSymbol(seriesIndex, series, ptIndex, (point.X), (point.Y), this, tracker);
                    this.svgRenderer.append(this.gTrackerEle, this.svgObject);
                }
            }
        },
        getClosesPointXY: function (serX, serY, series, x, y) {
            var closestPoint;
            var valAxis = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
            var mousePoint = ej.EjSvgRender.utils._getValuebyPoint(Math.abs(x - valAxis.x), Math.abs(y - (valAxis.y + valAxis.height)), series);
            var pointX = mousePoint.PointX;
            var pointY = mousePoint.PointY;
            var ptIndex;
            var chartPoint;
            var pointIndex;
            if ((y <= (valAxis.y + valAxis.height) && valAxis.y <= y) && (valAxis.x <= x && x <= (valAxis.x + valAxis.width))) {
                for (var i = 0; i < series.points.length; i++) {
                    chartPoint = series.points[i];
                    pointIndex = i;
                    serX.push(chartPoint.X);
                    serY.push(chartPoint.YValues[0]);
                }
                var closestX = this.getClosest(serX, pointX);
                var closestY = this.getClosest(serY, pointY);
                if (!ej.util.isNullOrUndefined(closestX) && !ej.util.isNullOrUndefined(closestY)) {
                    for (var i = 0; i < series.points.length; i++) {
                        chartPoint = series.points[i];
                        pointIndex = i;
                        // To get the location of the point
                        var loc = ej.EjSvgRender.utils._getPoint(chartPoint, series);
                        var boundsY = this.model.m_AreaBounds.Y;
                        // px difference between the point and mouse position
                        var markerSize = (chartPoint.marker.size.height + chartPoint.marker.border.width) * 3;
                        var width = series.width;
                        var diff = markerSize > width ? markerSize : width;
                        if (chartPoint.X == closestX && chartPoint.YValues[0] == closestY && loc.Y + boundsY - diff < y && y < loc.Y + boundsY + diff) {
                            closestPoint = chartPoint;
                            ptIndex = pointIndex;

                        }
                    }

                }
            }
            return { point: closestPoint, index: ptIndex };
        },
        getClosestPointX: function (serX, series, x, y) {
            var closestPoint = [];
            var valAxis = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
            var mousePoint = ej.EjSvgRender.utils._getValuebyPoint(Math.abs(x - valAxis.x), Math.abs(y - (valAxis.y + valAxis.height)), series);
            var pointX = mousePoint.PointX;
            var ptIndex = [];
            $.each(series.points, function (pointIndex, chartPoint) {
                serX.push(chartPoint.X);
            });
            var closest = this.getClosest(serX, pointX);
            if (!ej.util.isNullOrUndefined(closest)) {
                $.each(series.points, function (pointIndex, cPoint) {
                    if (cPoint.X == closest) {
                        closestPoint.push(cPoint);
                        ptIndex.push(pointIndex);

                    }
                });

            }
            return { point: closestPoint, index: ptIndex };
        },
        getClosest: function (obj, val) {
            var closest = null;

            // Work out min and max
            var min = Math.min.apply(null, obj);
            var max = Math.max.apply(null, obj);

            // Only calculate closest if point is within array
            if (val >= min && val <= max + 1) {
                var i;
                for (i = 0; i < obj.length; i++) {
                    if (closest == null || Math.abs(obj[i] - val) < Math.abs(closest - val)) {
                        closest = obj[i];
                    }
                }
            }
            return closest;
        },
        createTooltip: function (region, evt) {
            var series = this.model._visibleSeries[region.SeriesIndex];
            var seriesPoint = series.points[region.Region.PointIndex];
            var point = $.extend(true, {}, seriesPoint);
            var tooltipMargin = 10;
            if (point.visible) {
                var tooltip = null;
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.Data = { currentText: tooltip, seriesIndex: region.SeriesIndex, pointIndex: region.Region.PointIndex, Series: series };
                this._trigger("toolTipInitialize", commonEventArgs);
                var tooltipdiv = $('.tooltipDiv');
                if ($('.tooltipDiv').length == 0) {
                    this.tooltipFirst = true;
                    tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 13000; display: block;'></div>");
                    $(document.body).append(tooltipdiv);
                    $('.tooltipDiv').addClass(this.svgObject.id);
                } else {
                    this.tooltipFirst = false;
                }
                if (this.model.AreaType == "cartesianaxes") {
                    if (series.xAxis._valueType.toLowerCase() == "datetime")
                        point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
                    if (series.xAxis._valueType.toLowerCase() == "category")
                        point.x = ej.EjSvgRender.utils._getLabelContent(region.Region.PointIndex, series.xAxis);
                }
                if (series.tooltip.template != null) {
                    var cloneNode = $("#" + series.tooltip.template).clone();
                    $('.tooltipDiv')[0].innerHTML = "";
                    $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                    series.count = 1;
                    point.count = 1;
                    var seriesColor = this.getSeriesColor(point, region.SeriesIndex);
                    if (seriesColor)
                        $(tooltipdiv).css("background-color", jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor);
                    else
                        $(tooltipdiv).css("background-color", this.model.pointColors[region.Region.PointIndex]);
                    var data = { series: series, point: point };
                    if (!ej.util.isNullOrUndefined(commonEventArgs.Data.currentText))
                        $(tooltipdiv).html(commonEventArgs.Data.currentText);
                    else
                        $(tooltipdiv).html($(tooltipdiv).html().parseTemplate(data));
                } else {
                    $(tooltipdiv).html(commonEventArgs.Data.ToolTip);
                }
                var xPos = evt.pageX + tooltipMargin;
                var yPos = evt.pageY + tooltipMargin;
                xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
                yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());

                if (xPos === undefined || xPos === null)
                    xPos = evt.pageX + tooltipMargin;
                if (yPos === undefined || yPos === null)
                    yPos = evt.pageY + tooltipMargin;
                $(tooltipdiv).css("left", xPos);
                $(tooltipdiv).css("top", yPos);
                var tooltipOptions = series.tooltip;
                if (tooltipOptions.animation) {
                    $(tooltipdiv).css({
                        'transition-property': 'left,top',
                        '-moz-transition-property': 'left,top', /* Firefox 4 */
                        '-webkit-transition-property': 'left,top', /* Safari and Chrome */
                        '-o-transition-property': 'left,top',
                        'transition-duration': tooltipOptions.duration,
                        '-moz-transition-duration': tooltipOptions.duration, /* Firefox 4 */

                        '-webkit-transition-duration': this.tooltipFirst ? '0s' : tooltipOptions.duration, /* Safari and Chrome */
                        '-o-transition-duration': tooltipOptions.duration /* Opera */
                    });
                }
            }
        },

        translate: function (axis, translateX, translateY, currentScale) {
            var offset = axis.orientation.toLowerCase() == "horizontal"
                                                  ? translateX / this.model.m_AreaBounds.Width / currentScale
                                                  : translateY / this.model.m_AreaBounds.Height / currentScale;

            axis.zoomPosition = axis.orientation.toLowerCase() == "horizontal"
                ? ej.EjSvgRender.utils._minMax(axis.zoomPosition + offset, 0, (1 - axis.zoomFactor))
                : ej.EjSvgRender.utils._minMax(axis.zoomPosition - offset, 0, (1 - axis.zoomFactor));
        },
        tooltip: function (chart, evt) {
            var tooltipShowing = false;
            var chartSeries;
            var seriesIndex;
            var data = this.GetSeriesPoint(evt);
            if (!this.model.crosshair.visible && this.model.AreaType == "cartesianaxes") {
                for (var i = 0; i < this.model._visibleSeries.length; i++) {
                    chartSeries = this.model._visibleSeries[i];
                    seriesIndex = i;
                    if (chartSeries.visibility.toLowerCase() == 'visible' && chartSeries.type.toLowerCase() !== "scatter" && chartSeries.type.toLowerCase() !== "bubble" && chartSeries.type.toLowerCase() !== "column" && chartSeries.type.toLowerCase() !== "bar" && chartSeries.type.toLowerCase() !== "stackingbar" && chartSeries.type.toLowerCase() !== "stackingcolumn" && !chartSeries._hiloTypes) {
                        $("#" + chart.svgObject.id + "_TrackToolTipTemplate_" + seriesIndex).remove();
                        if (!($(".tooltipDiv").hasClass(chart.svgObject.id)))
                            $(".tooltipDiv").remove();
                        $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackSymbol" + '_' + seriesIndex).remove();
                        var serY = [];
                        var serX = [];
                        location = null;
                        var closestXyPoint = chart.getClosesPointXY(serX, serY, chartSeries, chart.mousemoveX, chart.mousemoveY);
                        if (closestXyPoint.point)
                            location = ej.EjSvgRender.utils._getPoint(closestXyPoint.point, chartSeries);
                        var pointData = this.model.prevPoint;
                        if (pointData && pointData.seriesIndex == i && !closestXyPoint.point) {
                            var pointLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                            chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, pointLocation);
                            this.model.prevPoint = null;
                        }

                        if (chartSeries.marker.visible && closestXyPoint.point) {
                            if (closestXyPoint.point.marker.visible)
                                chart.drawTrackerSymbol(chartSeries, seriesIndex, closestXyPoint.index, false, location);

                            if (pointData && closestXyPoint.point != pointData.point) {
                                var prevLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                                chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, prevLocation);
                            }
                            if (closestXyPoint.point.marker.visible)
                                this.model.prevPoint = { point: closestXyPoint.point, pointIndex: closestXyPoint.index, series: chartSeries, seriesIndex: seriesIndex };
                        }


                        if (chartSeries.tooltip.visible && closestXyPoint.point && (!data || (i > data.Region.SeriesIndex && !(chartSeries._zOrder < this.model._visibleSeries[data.Region.SeriesIndex]._zOrder)))) {
                            trans = ej.EjSvgRender.utils._getTransform(chartSeries.xAxis, chartSeries.yAxis, chart.model.requireInvertedAxes);
                            if ((location.X + trans.x) <= (trans.x + trans.width) && ((location.X + trans.x) >= trans.x || location.X == 0) && (Math.abs(location.Y - (trans.y + trans.height)) <= (trans.y + trans.height))
                                         && (Math.abs(location.Y - (trans.y + trans.height)) >= trans.y || location.Y == 0)) {
                                if ($(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip").length == 0) {
                                    var transOptions = { 'id': chart.svgObject.id + '_TrackToolTip', 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                                    chart.gTransToolEle = chart.svgRenderer.createGroup(transOptions);
                                }
                                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip"), { 'transform': 'translate(' + trans.x + ',' + trans.y + ')' });
                                if (ej.util.isNullOrUndefined(chartSeries.tooltip.template)) {
                                    tooltipShowing = true;
                                    $(".tooltipDiv").remove();
                                    chart.displayShowTooltip(location, closestXyPoint.point, chartSeries, closestXyPoint.index);
                                }
                                else {
                                    tooltipShowing = true;
                                    $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").remove();
                                    var region = { SeriesIndex: seriesIndex, Region: { PointIndex: closestXyPoint.index } };
                                    chart.createTooltip(region, evt);
                                }

                                //Comment the below code for VML -The line series tooltip color not displayed due to attach and detach the element to DOM
                                //  var detachEle = $(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip").detach();
                                // detachEle.appendTo(chart.svgObject);

                                var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                                commonPointEventArgs.Data = { location: { X: this.mousemoveX, Y: this.mousemoveX }, Region: { SeriesIndex: i, Region: { PointIndex: closestXyPoint.index } } };
                                chart._trigger("pointRegionMouseMove", commonPointEventArgs);
                            }
                        }
                    }
                }
            }

            if (!($(".tooltipDiv").hasClass(this.svgObject.id)))
                $(".tooltipDiv").remove();
            if (data && ej.util.isNullOrUndefined(data.Region.isStripLine)) {
                if (this.model.AreaType == "none" && this.model._visibleSeries[data.Region.SeriesIndex].explode) {
                    if (this.model._visibleSeries[data.Region.SeriesIndex].animation) {
                        if (this.model.AnimationComplete)
                            this.pieExplosion(data, evt);
                    }
                    else
                        this.pieExplosion(data, evt);
                }
                series = this.model._visibleSeries[data.Region.SeriesIndex];
                var seriesPoint = this.model._visibleSeries[data.Region.SeriesIndex].points[data.Region.Region.PointIndex];

                if (seriesPoint && (series.type.toLowerCase() == "bubble" || series.type.toLowerCase() == "scatter")) {
                    for (var i = 0; i < this.model._visibleSeries.length; i++) {
                        $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackSymbol" + '_' + i).remove();
                    }
                    var pointlocation = ej.EjSvgRender.utils._getPoint(seriesPoint, series);

                    this.drawTrackerSymbol(series, data.Region.SeriesIndex, data.Region.Region.PointIndex, true, pointlocation);
                }

                if (series.tooltip.visible && !tooltipShowing) {

                    if (series.visibility.toLowerCase() == 'visible') {
                        if (ej.util.isNullOrUndefined(series.tooltip.template)) {
                            var location;
                            if (this.model.AreaType == "cartesianaxes") {

                                var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
                                if ($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").length == 0) {
                                    var transToolOptions = { 'id': this.svgObject.id + '_TrackToolTip', 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                                    this.gTransToolEle = this.svgRenderer.createGroup(transToolOptions);
                                }
                                this.svgRenderer._setAttr($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip"), { 'transform': 'translate(' + trans.x + ',' + trans.y + ')' });
                                location = { X: (data.location.X - trans.x), Y: Math.abs(data.location.Y - trans.y) };
                            } else {
                                if ($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").length == 0) {
                                    var transToolTipOptions = { 'id': this.svgObject.id + '_TrackToolTip' };
                                    this.gTransToolEle = this.svgRenderer.createGroup(transToolTipOptions);
                                }
                                location = { X: (data.location.X), Y: Math.abs(data.location.Y) };
                            }


                            $(".tooltipDiv").remove();
                            this.displayShowTooltip(location, seriesPoint, series, data.Region.Region.PointIndex);
                            //Comment the below code for VML -The line series tooltip color not displayed due to attach and detach the element to DOM
                            // if (series.type.toLowerCase() == "bubble" || series.type.toLowerCase() == "scatter") {
                            //     var detachEle = $(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").detach();
                            //    detachEle.appendTo(this.svgObject);
                            // }
                        } else {
                            $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").remove();
                            this.createTooltip(data.Region, evt);
                        }
                    }
                }
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.Data = data;
                this._trigger("pointRegionMouseMove", commonEventArgs);
            }
            else {
                if (!ej.util.isNullOrUndefined(this.model.explodeValue)) {
                    var type = this.model.series[0].type;
                    var series = this.model._visibleSeries[0];
                    var index = this.model.explodeValue;
                    var seriesType = new ej.seriesTypes[type]();
                    var targetElement;
                    var id;
                    var _labelPosition = series.labelPosition.toLowerCase();
                    var result = seriesType._calculateArcData(this, index, series.points[index], series);
                    //var explodeId = this.model.series[0].name + "group" + index;
                    id = this.svgObject.id + '_SeriesGroup' + '_0';
                    targetElement = $(this.gSeriesEle).children('#' + id)[0];
                    this.svgRenderer._setAttr($(targetElement.childNodes[index]), { 'd': result.Direction });
                    if (this.model._visibleSeries[0].marker.dataLabel.visible) {
                        seriesType.drawDataLabelAcc(this, series, index, series.points[index]);
                    }
                    this.model.explodeValue = null;
                }
            }


        },

        chartCrossHair: function (chart, mouseLocation) {
            if (this.model.crosshair.visible && this.model.crosshair.type.toLowerCase() == "trackball") {

                var point;
                var seriesIndex;
                var series;
                $("#" + chart.svgObject.id).find(chart.gTrackAxisEle).remove();
                if (this.model.crosshair.tooltipTemplate != null) {
                    $(document).find('[id*="_TrackToolTipTemplate_"]').remove();
                }
                if (chart.gTransToolEle && chart.gTrackerEle) {
                    $(this.svgObject).find('[id*="_TrackSymbol_"],[id*="_TrackToolTip"]').remove();
                }
                for (var i = 0; i < this.model._visibleSeries.length; i++) {
                    seriesIndex = i;
                    series = this.model._visibleSeries[i];
                    if (series.visibility.toLowerCase() == 'visible') {
                        var serX = [];
                        var closestPoint;

                        closestPoint = chart.getClosestPointX(serX, series, chart.mousemoveX, chart.mousemoveY);

                        if (closestPoint.point) {
                            for (var t = 0; t < closestPoint.point.length; t++) {
                                if (closestPoint.point[t].visible) {
                                    point = ej.EjSvgRender.utils._getPoint(closestPoint.point[t], series);

                                    var dir = "M" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y) + " " + "L" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y + chart.model.m_AreaBounds.Height);

                                    var pathOptions = {
                                        'id': chart.svgObject.id + "_Tracker",
                                        'fill': 'none',
                                        'stroke-width': chart.model.crosshair.line.width,
                                        'clip-path': 'url(#' + chart.svgObject.id + '_ChartAreaClipRect)',
                                        'stroke': chart.model.crosshair.line.color,
                                        'd': dir
                                    };
                                    if ($(chart.svgObject).find("#" + chart.svgObject.id + "_Tracker").length == 0)
                                        chart.svgRenderer.drawPath(pathOptions, chart.svgObject);
                                    else
                                        chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_Tracker"), { "d": dir });

                                    var crossHairTrans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, chart.model.requireInvertedAxes);
                                    var tToolOptions = { 'transform': 'translate(' + crossHairTrans.x + ',' + crossHairTrans.y + ')', 'id': chart.svgObject.id + '_TrackToolTip' + '_' + seriesIndex };

                                    chart.gTransToolEle = chart.svgRenderer.createGroup(tToolOptions);

                                    chart.drawTrackerSymbol(series, seriesIndex, closestPoint.index[t], true, point);

                                    if (!chart.model.crosshair.tooltipTemplate)
                                        chart.displayTooltip(point, closestPoint.point[t], series, closestPoint.index[t]);
                                    else {
                                        var tX, tY;
                                        chart.displayTemplateTooltip(tX, tY, point, closestPoint.point[t], series, seriesIndex, closestPoint.index[t]);
                                    }
                                }
                            }

                        } else {
                            $(chart.svgObject).find('[id*="_Tracker"]').remove();
                        }
                    }
                }
                var gTrackAxisOptions = { 'id': chart.svgObject.id + '_TrackAxisToolTip' };
                chart.gTrackAxisEle = chart.svgRenderer.createGroup(gTrackAxisOptions);
                if (point) {
                    $.each(chart.model.axes, function (axisIndex, axis) {
                        var areaX = chart.model.m_AreaBounds.X;
                        if (axis.orientation.toLowerCase() == "horizontal" && axis.crosshairLabel.visible) {
                            if ((point.X + areaX) <= (axis.x + axis.width) && axis.x <= (point.X + areaX)) {
                                var valueType = axis._valueType.toLowerCase();
                                var xVal = ((point.X / (axis.width)) * (axis.visibleRange.delta) + axis.visibleRange.min).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                                if (valueType == "logarithmic") xVal = Math.pow(axis.logBase, xVal);
                                if (valueType == "datetime") xVal = (Globalize.format(new Date(Math.floor(xVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? EjSvgRender.utils._dateTimeLabelFormat(axis.intervalType) : axis.labelFormat)));
                                if (valueType == "category") xVal = ej.EjSvgRender.utils._getLabelContent(Math.floor(xVal), axis);
                                var pointLocation = { X: (point.X + areaX), Y: (axis.opposedPosition) ? (axis.y + axis.height - axis.majorTickLines.size) : (axis.y + axis.height + axis.majorTickLines.size + 10) };
                                chart.displayAxisTooltip(pointLocation, xVal, axis, axisIndex, mouseLocation);
                            }
                        }

                    });

                    if ($(chart.gTrackAxisEle).children().length > 0)
                        chart.svgRenderer.append(chart.gTrackAxisEle, chart.svgObject);
                }


            }
        },
        chartAreaCrossHair: function (chart, mouseLocation) {
            if (this.model.crosshair.visible && this.model.crosshair.type.toLowerCase() == "crosshair") {


                $("#" + chart.svgObject.id).find(chart.gTrackAxisEle).remove();
                var d = "M" + " " + (this.mousemoveX) + " " + (this.model.m_AreaBounds.Y) + " " + "L" + " " + (this.mousemoveX) + " " + (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height);
                var dhor = "M" + " " + (this.model.m_AreaBounds.X) + " " + (this.mousemoveY) + " " + "L" + " " + (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) + " " + (this.mousemoveY);
                var options = {
                    'id': chart.svgObject.id + "_CrosshairVertical",
                    'fill': 'none',
                    'stroke-width': chart.model.crosshair.line.width,
                    'stroke': chart.model.crosshair.line.color,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'd': d
                };
                var optionsHor = {
                    'id': chart.svgObject.id + "_CrosshairHorizontal",
                    'fill': 'none',
                    'stroke-width': chart.model.crosshair.line.width,
                    'stroke': chart.model.crosshair.line.color,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'd': d
                };
                if ($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairVertical").length == 0)
                    chart.svgRenderer.drawPath(options, chart.svgObject);
                else
                    chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairVertical"), { "d": d });
                if ($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairHorizontal").length == 0)
                    chart.svgRenderer.drawPath(optionsHor, chart.svgObject);
                else
                    chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairHorizontal"), { "d": dhor });
                var gAxisCrossOptions = { 'id': chart.svgObject.id + '_AxisCrossToolTip' };
                chart.gTrackAxisEle = chart.svgRenderer.createGroup(gAxisCrossOptions);

                $.each(chart.model.axes, function (axisIndex, axis) {
                    var valueType = axis._valueType.toLowerCase();
                    if (axis.orientation.toLowerCase() == "horizontal" && axis.crosshairLabel.visible) {
                        if ((chart.mousemoveX) <= (axis.x + axis.width) && axis.x <= (chart.mousemoveX)) {
                            var labelplacement = (axis._valueType == "category" && axis.labelPlacement.toLowerCase() == 'betweenticks') ? 0.5 : 0;
                            var xVal = (((chart.mousemoveX - axis.x) / (axis.width)) * (axis.visibleRange.delta) + axis.visibleRange.min + labelplacement).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                            if (valueType == "logarithmic") xVal = Math.pow(axis.logBase, xVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                            if (valueType == "datetime") xVal = (Globalize.format(new Date(Math.floor(xVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis.intervalType) : axis.labelFormat)));
                            if (valueType == "category") xVal = ej.EjSvgRender.utils._getLabelContent(xVal, axis);
                            var xPointLocation = { X: (chart.mousemoveX), Y: (axis.y) };
                            chart.displayAxisTooltip(xPointLocation, xVal, axis, axisIndex, mouseLocation);
                        }
                    } else if (axis.orientation.toLowerCase() == "vertical" && axis.crosshairLabel.visible) {
                        if ((chart.mousemoveY) <= (axis.y + axis.height) && axis.y <= (chart.mousemoveY)) {
                            var yVal = ((Math.abs(1 - (chart.mousemoveY - (axis.y)) / (axis.height)) * (axis.visibleRange.delta) + axis.visibleRange.min).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), ""));
                            if (valueType == "logarithmic") yVal = Math.pow(axis.logBase, yVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                            if (valueType == "datetime") yVal = (Globalize.format(new Date(Math.floor(yVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis.intervalType) : axis.labelFormat)));
                            if (valueType == "category") yVal = ej.EjSvgRender.utils._getLabelContent(yVal, axis);
                            var ypointLocation = { X: axis.x, Y: (chart.mousemoveY) };
                            chart.displayAxisTooltip(ypointLocation, yVal, axis, axisIndex, mouseLocation);
                        }
                    }
                });
                if ($(chart.gTrackAxisEle).children().length > 0)
                    chart.svgRenderer.append(chart.gTrackAxisEle, chart.svgObject);

            }
        },
        chartTouchMove: function (evt) {
            this.cancelEvent(evt);
            evt = evt.originalEvent.touches[0];
            this.chartInteractiveBehavior(evt);

        },

        chartInteractiveBehavior: function (evt) {
            var mouseMoveCords = this.calMousePosition(evt);
            this.mousemoveX = mouseMoveCords.X;
            this.mousemoveY = mouseMoveCords.Y;

            var id = "#" + this.svgObject.id;
            if (this.mouseWheelCoords) {
                var changeX = Math.abs(this.mouseWheelCoords.x - evt.pageX);
                var changeY = Math.abs(this.mouseWheelCoords.y - evt.pageY);
                if ((changeX > 0 || changeY > 0) && !(this.panning)) {
                    this.enableTrackBall();
                } else {
                    this.disableTrackBall();
                }
            }

            var chart = this;

            if (chart.model.AreaType == "cartesianaxes" && this.mousemoveX >= this.model.m_AreaBounds.X && this.mousemoveX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mousemoveY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mousemoveY >= this.model.m_AreaBounds.Y && this.model.initSeriesRender) {
                var mouseLocation = { x: chart.mousemoveX, y: chart.mousemoveY };
                var targetId = chart.svgRenderer._getAttrVal($(evt.target).parent(), "id");
                if ((targetId != undefined) && (targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_ZoomBtn'), "id") || targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_ResetZoom'), "id") || targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_PanBtn'), "id"))) {
                    this.disableTrackBall();
                }
                else {
                    this.enableTrackBall();
                }

                //show tooltip and track ball
                if (!this.model.crosshair.visible)
                    this.tooltip(chart, evt);

                //crossHair of chart
                this.chartCrossHair(chart, mouseLocation);

                //crossHair for chart area
                this.chartAreaCrossHair(chart, mouseLocation);

            } else {
                if (chart.model.AreaType == "none")
                    this.tooltip(chart, evt);
                else {
                    $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").remove();
                    $(".tooltipDiv").remove();
                    this._removeTrackBall();
                }
            }
        },

        chartMouseMove: function (evt) {

            this.cancelEvent(evt);

            var chart = this;
            this.calMousePosition(evt)
            $('#tooltip').remove();
            if (!this.panning) {
                if (!this.drag)
                    this.chartInteractiveBehavior(evt);
                var id = "#" + this.svgObject.id;
                if (this.model.zooming.enable && this.model.AreaType == "cartesianaxes") {
                    if (this.drag) {
                        this.disableTrackBall();
                        $('#' + this.svgObject.id + '_ZoomArea').remove();
                        var width, height, x, y;
                        if (this.model.zooming.type.toLowerCase() == "x") {
                            width = Math.abs(this.mousemoveX - this.mouseDownX);
                            height = this.model.m_AreaBounds.Height;
                            x = this.mousemoveX > this.mouseDownX ? this.mouseDownX : this.mousemoveX;
                            y = this.model.m_AreaBounds.Y;
                        } else if (this.model.zooming.type.toLowerCase() == "y") {

                            width = this.model.m_AreaBounds.Width;
                            height = Math.abs(this.mousemoveY - this.mouseDownY);
                            x = this.model.m_AreaBounds.X;
                            y = this.mousemoveY > this.mouseDownY ? this.mouseDownY : this.mousemoveY;

                        } else {
                            width = Math.abs(this.mousemoveX - this.mouseDownX);
                            height = Math.abs(this.mousemoveY - this.mouseDownY);
                            x = this.mousemoveX > this.mouseDownX ? this.mouseDownX : this.mousemoveX;
                            y = this.mousemoveY > this.mouseDownY ? this.mouseDownY : this.mousemoveY;

                        }

                        $(this.svgObject).css({
                            '-moz-user-select': '-moz-none',
                            '-khtml-user-select': 'none',
                            '-webkit-user-select': 'none',
                            '-ms-user-select': 'none',
                            'user-select': 'none'
                        });

                        this.zooming = true;

                        var rectOptions = {
                            'id': this.svgObject.id + '_ZoomArea',
                            'x': x,
                            'y': y,
                            'width': width,
                            'height': height,
                            'fill': 'rgba(69,114,167,0.25)',
                            'stroke-width': 1,
                            'stroke': 'rgba(69,114,167,0.25)',
                            'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                        };

                        this.svgRenderer.drawRect(rectOptions, this.svgObject);
                    }

                }
            }
            if (this.panning && this.doPan && this.model.AreaType == "cartesianaxes") {

                var oDelta;
                oDelta = {
                    'x': this.oPreviousCoords.x - evt.pageX,
                    'y': this.oPreviousCoords.y - evt.pageY
                };

                this.oPreviousCoords = {
                    'x': evt.pageX,
                    'y': evt.pageY
                };
                $.each(this.model.axes, function (index, axis) {

                    var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                    chart.translate(axis, (oDelta.x), (oDelta.y), currentScale);
                });

                this.redraw();
                this._cursorToPointer();
                this._enableZoomingButtons();

            }

            //Declaration
            var targetid = evt.target.id;
            var id = this.svgObject.id;
            var parentId = evt.target.parentNode.id;

            //condition to find the buttons and display tooltip on hover
            if (parentId == id + "_ResetZoom")
                createBtnTooltip("Reset");
            else if (parentId == id + '_PanBtn')
                createBtnTooltip("Pan");
            else if (parentId == id + '_ZoomBtn')
                createBtnTooltip("Zoom");

            // method to create tooltip for zooming and panning buttons
            function createBtnTooltip(text) {
                var tooltipdiv = $("<div id = 'tooltip'></div>");
                $(tooltipdiv).html("&nbsp" + text + "&nbsp");
                $(document.body).append(tooltipdiv);
                // adding css prop to the div
                $(tooltipdiv).css({
                    "left": evt.pageX + 10,
                    "top": evt.pageY + 10,
                    "display": "block",
                    "position": "absolute",
                    "z-index": "13000",
                    "cursor": "default",
                    "font-family": "Segoe UI",
                    "color": "#707070",
                    "font-size": "12px",
                    "background-color": "#FFFFFF",
                    "border": "1px solid #707070"
                });
            }
            var svgId = this.svgObject.id;
            var hAxes = this.model.hAxes;

            if (targetid.indexOf("_XLabel_") >= 0) {
                // Loop to find the axis
                for (var j = 0; j < hAxes.length; j++) {
                    if (targetid.indexOf(svgId + '_' + hAxes[j].axisName + '_XLabel_') >= 0)
                        break;
                }
                // Loop to find the labels in the axis
                for (var i = 0; i < hAxes[j].visibleLabels.length; i++) {
                    if (targetid == svgId + '_' + hAxes[j].axisName + '_XLabel_' + i) {
                        var ele = hAxes[j].visibleLabels[i];
                        var tooltipdiv = $("<div id = 'tooltip'></div>");
                        var str = ele.Text == ele.displayText ? "" : ele.Text;
                        if (str == "") break;
                        $(tooltipdiv).html("&nbsp" + str + "&nbsp");
                        $(document.body).append(tooltipdiv);
                        // adding css prop to the div
                        $(tooltipdiv).css({
                            "left": evt.pageX + 10,
                            "top": evt.pageY + 10,
                            "display": "block",
                            "position": "absolute",
                            "z-index": "13000",
                            "cursor": "default",
                            "font-family": "Segoe UI",
                            "color": "#707070",
                            "font-size": "12px",
                            "background-color": "#FFFFFF",
                            "border": "1px solid #707070"
                        });
                    }
                }
            }

            if ($(this.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').length > 0) {
                $(this.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').appendTo(this.svgObject);
            }

            var legenddata = this.getLegendData(evt);
            if (legenddata) {
                var commonLegendMoveEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonLegendMoveEventArgs.Data = legenddata;
                this._trigger("legendItemMouseMove", commonLegendMoveEventArgs);
            }
        },
        pieExplosion: function (data) {
            var region = data;
            var id;
            var targetElement;
            var symbolName;
            var series = region.Region.Series;
            var seriesType = new ej.seriesTypes[series.type]();
            var explodePoint = series.explodeIndex;
            var _labelPosition = series.labelPosition.toLowerCase();
            if (explodePoint != region.PointData[0].PointIndex && !region.Region.Series.explodeAll) {
                series.explodeIndex = region.PointData[0].PointIndex;

                if (!ej.util.isNullOrUndefined(this.model.explodeValue)) {
                    var result = seriesType._calculateArcData(this, this.model.explodeValue, series.points[this.model.explodeValue], region.Region.Series);
                    id = this.svgObject.id + '_SeriesGroup' + '_' + region.Region.SeriesIndex;
                    targetElement = $(this.gSeriesEle).children('#' + id)[0];
                    this.svgRenderer._setAttr($(targetElement.childNodes[this.model.explodeValue]), { 'd': result.Direction });

                    if (series.marker.dataLabel.visible) {
                        seriesType.drawDataLabelAcc(this, series, this.model.explodeValue, series.points[this.model.explodeValue])
                    }
                }

                result = seriesType._calculateArcData(this, region.PointData[0].PointIndex, this.model.chartRegions[0].Series.points[region.PointData[0].PointIndex], region.Region.Series);
                id = this.svgObject.id + '_SeriesGroup' + '_' + region.Region.SeriesIndex;
                targetElement = $(this.gSeriesEle).children('#' + id)[0];
                this.svgRenderer._setAttr($(targetElement.childNodes[region.PointData[0].PointIndex]), { 'd': result.Direction });
                if (series.marker.dataLabel.visible) {
                    seriesType.drawDataLabelAcc(this, series, region.PointData[0].PointIndex, series.points[region.PointData[0].PointIndex])
                }
                this.model.explodeValue = region.PointData[0].PointIndex;

            }
            region.Region.Series.explodeIndex = explodePoint;
        },
        displayTemplateTooltip: function (x, y, location, seriesPoint, series, serIndex, ptIndex) {
            var point = $.extend(true, {}, seriesPoint);
            point.x = point.X;
            if (series.xAxis._valueType.toLowerCase() == "datetime")
                point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
            if (series.xAxis._valueType.toLowerCase() == "category") point.x = ej.EjSvgRender.utils._getLabelContent(ptIndex, series.xAxis);
            var xLoc = location.X + x + 15;
            var yLoc = (location.Y) + y + 15;
            var tooltipdiv = $("<div style='position: absolute; z-index: 13000; display: block;'></div>");
            $(tooltipdiv).addClass("TrackToolTipTemplate");
            this.svgRenderer._setAttr($(tooltipdiv), { "id": this.svgObject.id + "_TrackToolTipTemplate_" + serIndex });
            if (this.model.crosshair.tooltipTemplate != null) {
                var cloneNode = $("#" + this.model.crosshair.tooltipTemplate).clone();
                $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                series.count = 1;
                point.count = 1;
                var seriesColor = this.getSeriesColor(point, serIndex);
                $(tooltipdiv).css("background-color", jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor);
                var data = { series: series, point: point };
                $(tooltipdiv).html($(tooltipdiv).html().parseTemplate(data));
            }
            $(document.body).append(tooltipdiv);
            $(tooltipdiv).css("left", xLoc + $(tooltipdiv).width());
            $(tooltipdiv).css("top", yLoc + $(tooltipdiv).height());

        },

        displayAxisTooltip: function (location, text, axis, index, mouseLoc) {
            var maxTickSize = 0;
            for (var i = 0; i < this.model.axes.length; i++) {
                if (this.model.axes[i].majorTickLines.size > maxTickSize) {
                    maxTickSize = this.model.axes[i].majorTickLines.size;
                }
            }
            var maxLocation = (axis.opposedPosition) ? ((maxTickSize + 10) + (axis.x + axis.width)) : (axis.x + axis.width);
            if (location.X >= 0 && location.X <= maxLocation) {
                var x = (location.X);
                var y = location.Y;
                var padding = 5;
                var trackAxisText = text;
                var textOffset = ej.EjSvgRender.utils._measureText(text, null, axis.crosshairLabel.font);
                if (axis.orientation.toLowerCase() == 'horizontal') {
                    x = x - textOffset.width / 2;
                    if (axis.opposedPosition == true) {
                        y = axis.Location.Y1 - textOffset.height / 2 + padding - maxTickSize;
                    }

                    if (axis.opposedPosition == false) {
                        y = axis.Location.Y1 + textOffset.height - padding + maxTickSize;
                    }
                }
                if (!(axis.opposedPosition) && (x + textOffset.width) > (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) {
                    x = ((this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) - textOffset.width) + padding + this.model.elementSpacing;
                }
                if (axis.orientation.toLowerCase() == 'vertical') {
                    y = location.Y + textOffset.height / 4;
                    if (axis.opposedPosition == true) {
                        x = axis.x + 2 + padding - 2;
                    }
                    if (axis.opposedPosition == false) {
                        x = axis.x - textOffset.width - maxTickSize;
                    }
                }


                var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonTrackTextArgs.Data = { axisIndex: index, chartAxis: axis, currentTrackText: trackAxisText, location: mouseLoc };
                this._trigger("trackAxisToolTip", commonTrackTextArgs);

                var textAxisOptions = {
                    'id': this.svgObject.id + '_AxisToolTipText' + '_' + index,
                    'x': x,
                    'y': y,
                    'fill': axis.crosshairLabel.font.color,
                    'font-size': axis.crosshairLabel.font.size,
                    'font-family': axis.crosshairLabel.font.fontFamily,
                    'font-style': axis.crosshairLabel.font.fontStyle,
                    'font-weight': axis.crosshairLabel.font.fontWeight,
                    'text-anchor': 'start',
                    'opacity': axis.crosshairLabel.font.opacity,

                };

                var fontSize = ej.EjSvgRender.utils._measureText(commonTrackTextArgs.Data.currentTrackText, this.model.m_AreaBounds.Width, axis.crosshairLabel.font);

                var toolAxisRectOptions = {
                    'id': this.svgObject.id + '_AxisToolTipRect' + '_' + index,
                    'x': (x - padding),
                    'y': (y - (fontSize.height)),
                    'width': (fontSize.width) + (2 * padding),
                    'height': (2 * fontSize.height) - (2 * padding),
                    'rx': axis.crosshairLabel.rx,
                    'ry': axis.crosshairLabel.ry,
                    'fill': axis.crosshairLabel.fill,
                    'stroke-width': axis.crosshairLabel.border.width,
                    'stroke': axis.crosshairLabel.border.color
                };

                this.svgRenderer.drawRect(toolAxisRectOptions, this.gTrackAxisEle);
                this.svgRenderer.drawText(textAxisOptions, commonTrackTextArgs.Data.currentTrackText, this.gTrackAxisEle);

            }
        },
        displayTooltip: function (location, point, series, pointIndex) {
            if ((location.X + this.model.m_AreaBounds.X > this.model.m_AreaBounds.X || location.X == 0) && ((location.X + this.model.m_AreaBounds.X) <= (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width)) &&
         (location.Y + this.model.m_AreaBounds.Y) > this.model.m_AreaBounds.Y && (location.Y + this.model.m_AreaBounds.Y) < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height)) {
                var seriesIndex = $.inArray(series, this.model._visibleSeries);
                var padding = 5;
                var x = location.X + ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) + padding + 1;
                var y = (location.Y);
                var format = series.tooltip.format;
                var numberToFixed = ej.util.isNullOrUndefined((series.yAxis.roundingPlaces) ? 2 : series.yAxis.roundingPlaces);
                var trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);
                var textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText);
                var seriesColor = this.getSeriesColor(point, seriesIndex);
                var toolTipOptions = this.getTooltipOptions(seriesColor, series);
                var rectBorderColor = toolTipOptions.rectBColor;
                var rectFillColor = toolTipOptions.rectFColor;
                var textColor = toolTipOptions.rectTextColor;
                var rX = toolTipOptions.rectX;
                var rY = toolTipOptions.rectY;
                var rectOptions = series.tooltip;
                var toolRectOptions = {
                    'id': this.svgObject.id + '_ToolTipRect' + '_' + seriesIndex + '_' + pointIndex,
                    'x': x,
                    'y': y,
                    'width': textOffset.width,
                    'height': textOffset.height,
                    'fill': rectOptions.fill ? rectOptions.fill : rectFillColor,
                    'stroke-width': rectOptions.border.width ? rectOptions.border.width : 1,
                    'stroke': rectOptions.border.color ? rectOptions.border.color : rectBorderColor,
                    'fill-opacity': rectOptions.opacity ? rectOptions.opacity : 0.85,
                    'rx': rectOptions.rx ? rectOptions.rx : rX,
                    'ry': rectOptions.ry ? rectOptions.ry : rY
                };

                this.svgRenderer.drawRect(toolRectOptions, this.gTransToolEle);

                var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonTrackTextArgs.Data = { serIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText };
                this._trigger("trackToolTip", commonTrackTextArgs);
                var fontSize = 0;
                var tooltip = series.tooltip;
                var font = tooltip.font ? $.extend(false, series.font, {}, tooltip.font) : series.font;
                fontSize = ej.EjSvgRender.utils._measureText(text, series.xAxis.width, font).height;
                var textOptions = {
                    'id': this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + '_' + pointIndex,
                    'x': x,
                    'y': y + fontSize / 4,
                    'fill': (tooltip.font && tooltip.font.color) ? font.color : textColor,
                    'font-size': font.size,
                    'font-family': font.fontFamily,
                    'font-style': font.fontStyle,
                    'font-weight': font.fontWeight,
                    'text-anchor': 'start'

                };
                var text = commonTrackTextArgs.Data.currentText;
                if (commonTrackTextArgs.Data.currentText.indexOf("<br/>") >= 0) {
                    text = commonTrackTextArgs.Data.currentText.split("<br/>");
                }
                this.svgRenderer.drawText(textOptions, text, this.gTransToolEle, font);
                this.svgRenderer.append(this.gTransToolEle, this.svgObject);
                fontSize = ej.EjSvgRender.utils._measureText(text, series.xAxis.width, font).height;

                var box = $("#" + textOptions.id)[0].getBoundingClientRect();
                if ((x + (box.right - box.left)) >= this.model.m_AreaBounds.Width) {
                    var diff = x - (((box.right - box.left) + ((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ? series.marker.size.width : 0) : series._trackMarker.size.width)) + (4 * padding));
                    this.svgRenderer._setAttr("#" + textOptions.id, { 'x': diff });
                    var elements = $("#" + textOptions.id).children("tspan");
                    for (var i = 0; i < elements.length; i++) {
                        var element = elements[i];
                        this.svgRenderer._setAttr(element, { 'x': diff });
                    }
                    x = diff;
                }

                this.svgRenderer._setAttr("#" + toolRectOptions.id, { 'x': (x - padding), 'y': ((y - (fontSize / 2)) - padding / 2), 'width': (box.right - box.left) + 2 * padding, 'height': (box.bottom - box.top) + padding });




            }

        },
        getTooltipOptions: function (seriesColor, series) {
            var rectBorderColor, rectFillColor, rX, rY, textColor;
            if (this.model.theme.indexOf("gradient") >= 0) {
                rectBorderColor = jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor;
                rectFillColor = 'white';
                textColor = "#333333";
                rX = 5;
                rY = 5;

            } else {
                var type = series.type.toLowerCase();
                if (type != "column" && type != "bar" && type != "rangecolumn" && type != "stackingcolumn" && type != "stackingbar" && this.model.AreaType != "none") {
                    rectFillColor = jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor;
                    rectBorderColor = 'transparent';
                    textColor = "white";
                } else {
                    rectBorderColor = "#333333";
                    rectFillColor = 'white';
                    textColor = "#333333";
                }
                rX = 0;
                rY = 0;
            }
            return { rectBColor: rectBorderColor, rectFColor: rectFillColor, rectTextColor: textColor, rectX: rX, rectY: rY };
        },
        getSeriesColor: function (point, seriesIndex) {
            var seriesColor;
            if (point.fill) {
                if (!point.fill._gradientStop)
                    seriesColor = point.fill;
                else {
                    seriesColor = point.fill._gradientStop[0].color;
                }
            } else {
                seriesColor = this.model.seriesColors[seriesIndex];
            }
            return seriesColor;
        },
        getTooltipFormat: function (seriesPoint, series, serIndex, ptIndex, format, numberToFixed) {
            var point = $.extend(true, {}, seriesPoint);
            series.count = 1;
            point.count = 1;
            if (this.model.AreaType == "cartesianaxes") {
                if (series.xAxis._valueType.toLowerCase() == "datetime")
                    point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
                if (series.xAxis._valueType.toLowerCase() == "category")
                    point.x = ej.EjSvgRender.utils._getLabelContent(ptIndex, series.xAxis);
            }
            else {
                if (series._xAxisValueType == "date")
                    point.x = (Globalize.format(new Date(point.X), (ej.EjSvgRender.utils._dateTimeLabelFormat('days'))));
            }
            point.y = (ej.EjSvgRender.utils._decimalPlaces(point.y) == 0) ? point.y : point.y.toFixed(numberToFixed);
            var data = { series: series, point: point };
            format = format.parseTemplate(data);
            return format;
        },
        displayShowTooltip: function (location, point, series, pointIndex) {
            if (point.visible) {
                var seriesIndex = $.inArray(series, this.model._visibleSeries),
                seriesColor,
                textOffset,
                padding = 7,
                x,
                y,
                trackTooltipText,
                format = series.tooltip.format;

                //Set tooltip position ,text and color
                if (this.model.AreaType == "cartesianaxes") {
                    var numberToFixed = (series.yAxis.roundingPlaces === null || series.yAxis.roundingPlaces === undefined) ? 2 : series.yAxis.roundingPlaces;
                    x = location.X + ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) + padding + 1;
                    y = (location.Y);
                    trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);
                    seriesColor = this.getSeriesColor(point, seriesIndex);
                    textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText);
                } else {
                    x = location.X + series.marker.size.width + padding;
                    y = (location.Y);
                    numberToFixed = ej.util.isNullOrUndefined(this.model.roundingPlaces) ? 2 : chartObj.model.roundingPlaces;
                    trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);;
                    textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText);
                    seriesColor = this.model.pointColors[pointIndex];
                }
                var toolTipOptions = this.getTooltipOptions(seriesColor, series);
                var rectBorderColor = toolTipOptions.rectBColor;
                var rectFillColor = toolTipOptions.rectFColor;
                var textColor = toolTipOptions.rectTextColor;
                var rX = toolTipOptions.rectX;
                var rY = toolTipOptions.rectY;
                var rectOptions = series.tooltip;
                //draw tooltip rectangle
                var toolRectOptions = {
                    'id': this.svgObject.id + '_ToolTipRect',
                    'x': x,
                    'y': y,
                    'width': textOffset.width,
                    'height': textOffset.height,
                    'fill': (rectOptions.fill) ? rectOptions.fill : rectFillColor,
                    'stroke-width': (rectOptions.border.width) ? rectOptions.border.width : 1,
                    'stroke': (rectOptions.border.color) ? rectOptions.border.color : rectBorderColor,
                    'fill-opacity': rectOptions.opacity,
                    'rx': (rectOptions.rx) ? rectOptions.rx : rX,
                    'ry': (rectOptions.ry) ? rectOptions.ry : rY
                };

                this.svgRenderer.drawRect(toolRectOptions, this.gTransToolEle);

                //event for tooltip text
                var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonTrackTextArgs.Data = { seriesIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText };
                this._trigger("toolTipInitialize", commonTrackTextArgs);

                //draw tooltip text
                var fontSize = 0;
                var tooltip = series.tooltip;
                var font = (tooltip.font) ? $.extend(false, series.font, {}, tooltip.font) : series.font;
                var textOptions = {
                    'id': this.svgObject.id + '_ToolTipText',
                    'x': x,
                    'y': y,
                    'fill': (tooltip.font && tooltip.font.color) ? font.color : textColor,
                    'font-size': font.size,
                    'font-family': font.fontFamily,
                    'font-style': font.fontStyle,
                    'font-weight': font.fontWeight,
                    'text-anchor': 'start'
                };

                // Add string in text array for tooltip format
                var text = commonTrackTextArgs.Data.currentText;
                if (commonTrackTextArgs.Data.currentText.indexOf("<br/>") >= 0) {
                    text = commonTrackTextArgs.Data.currentText.split("<br/>");
                }
                fontSize = (this.model.AreaType == "cartesianaxes") ? ej.EjSvgRender.utils._measureText(text, series.xAxis.width, font).height : ej.EjSvgRender.utils._measureText(text, null, font).height;
                var maxWidth = (this.model.AreaType == "cartesianaxes") ? (this.model.requireInvertedAxes) ? series.yAxis.width : series.xAxis.width : $(this.svgObject).width();
                this.svgRenderer.drawText(textOptions, text, this.gTransToolEle, font);
                if ($(this.svgObject).find("#" + this.svgObject.id + +"_TrackToolTip").length == 0)
                    this.svgRenderer.append(this.gTransToolEle, this.svgObject);
                //Adjust x-position to display tooltip within chart area 
                var box = $("#" + textOptions.id)[0].getBoundingClientRect();
                if ((x + (box.right - box.left)) >= maxWidth) {
                    var diff = x - (((box.right - box.left) + ((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ? series.marker.size.width : 0) : series._trackMarker.size.width)) + (4 * padding));
                    this.svgRenderer._setAttr("#" + textOptions.id, { 'x': diff });
                    var elements = $("#" + textOptions.id).children("tspan");
                    for (var i = 0; i < elements.length; i++) {
                        var element = elements[i];
                        this.svgRenderer._setAttr(element, { 'x': diff });
                    }
                    x = diff;
                }

                //adjust rectangle size based on text length  
                this.svgRenderer._setAttr("#" + toolRectOptions.id, { 'x': (x - padding), 'y': ((y - (fontSize / 2)) - padding), 'width': (box.right - box.left) + 2 * padding, 'height': (box.bottom - box.top) + padding });
            }
        },
        _initializeSeriesColors: function () {
            var chartObj = this;
            chartObj.model.colors = (chartObj.model.palette) ? chartObj.model.palette : chartObj.model.colors;
            var count = chartObj.model.colors.length;
            for (var i = 0; i < this.model._visibleSeries.length; i++) {
                var series = this.model._visibleSeries[i];
                if (series.visibility.toLowerCase() === 'visible' && chartObj.model.AreaType == "cartesianaxes") {
                    if (!ej.util.isNullOrUndefined(series.fill)) {
                        chartObj.model.seriesColors[i] = ej.util.isNullOrUndefined(series.fill._gradientStop) ? series.fill : series.fill._gradientStop;
                    } else {

                        chartObj.model.seriesColors[i] = chartObj.model.colors[i % count];
                    }
                    if (!ej.util.isNullOrUndefined(series.border.color)) {
                        chartObj.model.seriesBorderColors[i] = series.border.color;
                    } else {
                        chartObj.model.seriesBorderColors[i] = chartObj.model.seriesBorderDefaultColors[i % 10];
                    }
                } else {
                    if (series.visibility.toLowerCase() === 'visible') {
                        for (var j = 0; j < series.points.length; j++) {
                            var point = series.points[j];
                            if (!ej.util.isNullOrUndefined(point.fill)) {
                                chartObj.model.pointColors[j] = ej.util.isNullOrUndefined(point.fill._gradientStop) ? point.fill : point.fill._gradientStop;
                            } else {
                                chartObj.model.pointColors[j] = chartObj.model.colors[j % count];
                            }
                            if (!ej.util.isNullOrUndefined(point.border) && !ej.util.isNullOrUndefined(point.border.color)) {
                                chartObj.model.pointBorderColors[j] = point.border.color;
                            } else {
                                chartObj.model.pointBorderColors[j] = chartObj.model.seriesBorderDefaultColors[i % 10];
                            }

                        }
                    }

                }

            }
        },



        _createAxisLabelAndRange: function () {
            var unsift = false;
            this.model.axes = (!this.model.axes) ? [] : this.model.axes;
            for (var i = 0; i < this.model.axes.length; i++) {
                var axis = this.model.axes[i];
                axis.orientation = (!axis.orientation) ? "vertical" : axis.orientation;
                if (axis.orientation.toLowerCase() == "horizontal")
                    this.model.axes[i] = $.extend(true, {}, this.model.secondaryX, axis);
                else
                    this.model.axes[i] = $.extend(true, {}, this.model.secondaryY, axis);
            }


            this.model.primaryYAxis.axisName = "SecondaryAxis";
            this.model.primaryXAxis.axisName = "PrimaryAxis";

            for (var j = 0; j < this.model.axes.length; j++) {
                var axis = this.model.axes[j];
                if (axis.axisName === this.model.primaryXAxis.axisName || axis.axisName === this.model.primaryYAxis.axisName)
                    unsift = true;
            }

            if (!unsift) {
                this.model.axes.unshift(this.model.primaryYAxis);
                this.model.axes.unshift(this.model.primaryXAxis);
            }
            for (var k = 0; k < this.model.axes.length; k++) {
                var axis = this.model.axes[k];
                if (!unsift) {
                    if (this.model.requireInvertedAxes) {
                        if (axis.axisName === this.model.primaryYAxis.axisName || axis.axisName === this.model.primaryXAxis.axisName) {
                            if (axis.orientation.toLowerCase() == "horizontal") {
                                axis.orientation = "vertical";

                            } else {
                                axis.orientation = "horizontal";
                                axis.labelPlacement = (!(axis.labelPlacement)) ? ej.Chart.labelPlacement.OnTicks : axis.labelPlacement;
                            }
                        }
                    }
                }

                axis.visibleLabels = [];
                axis.setRange = (!(axis.range)) ? false : (ej.util.isNullOrUndefined(axis.setRange)) ? true : axis.setRange;
                axis.setRange = ((axis.actual_Range) || (axis.setRange != false)) ? true : false;
                axis.actualRange = (!(axis.actualRange)) ? (axis.range == null) ? {} : axis.range : axis.actualRange;
                axis.visibleRange = (!(axis.visibleRange)) ? (axis.range == null) ? {} : axis.range : axis.visibleRange;
                axis.axisName = !(axis.axisName) ? k.toString() : axis.axisName;
            }
        },






        _drawTitle: function () {
            var measureTitle = ej.EjSvgRender.utils._measureText(this.model.title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.font);
            var modelTitleHeight = (this.model.title.text == "") ? 0 : (measureTitle.height + this.model.elementSpacing);


            // Drawing the chart title
            var legendSpace = (this.model.legend.position.toLowerCase() == "left") ? this.model.LegendBounds.Width / 2 : ((this.model.legend.position.toLowerCase() == "right" ? (-this.model.LegendBounds.Width / 2) : 0))
            var x = (this.model.AreaType == "none") ? legendSpace : this.model.m_AreaBounds.X;
            if (this.model.title.text != "" && this.model.title.text != null) {
                var titleLocation = this.model.margin.top + (modelTitleHeight / 2) + (this.model.elementSpacing);
                var locX = ((this.model.AreaType == "none") ? ($(this.svgObject).width() / 2) : (this.model.m_AreaBounds.Width / 2)) - measureTitle.width / 2;

                if (this.model.title.textAlignment.toLowerCase() == "near") {
                    locX = 0;
                }
                else if (this.model.title.textAlignment.toLowerCase() == "far") {
                    locX = $(this.svgObject).width() - (this.model.margin.right + this.model.elementSpacing + this.model.m_AreaBounds.X) - measureTitle.width;
                }


                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.Data = { Title: this.model.title.text, Location: { X: locX, Y: titleLocation }, Size: measureTitle };
                this._trigger("titleRendering", commonEventArgs);
                this.model._titleLocation = { X: commonEventArgs.Data.Location.X + x, Y: commonEventArgs.Data.Location.Y };
                var options = {
                    'id': this.svgObject.id + '_ChartTitle',
                    'x': commonEventArgs.Data.Location.X + x,
                    'y': commonEventArgs.Data.Location.Y,
                    'fill': this.model.title.font.color,
                    'font-size': this.model.title.font.size,
                    'font-family': this.model.title.font.fontFamily,
                    'font-style': this.model.title.font.fontStyle,
                    'font-weight': this.model.title.font.fontWeight,
                    'text-anchor': 'start'
                };

                this.svgRenderer.drawText(options, commonEventArgs.Data.Title, this.gTitleEle);

                this.svgRenderer.append(this.gTitleEle, this.svgObject);
            }
        },



        _getLegendSize: function (series) {

            var font = this.model.legend.font;
            var textSize = ej.EjSvgRender.utils._measureText(series.name, null, font);
            var symbolSize = this.model.legend.itemSize;
            var padding = this.model.legend.itemPadding;


            var width = padding + symbolSize.width + padding + textSize.width;
            var height = Math.max(symbolSize.height + padding, textSize.height + padding);

            return { Width: width, Height: height };

        },

        _getYValues: function (series) {
            var values = [];
            for (var i = 0; i < series.points.length; i++)
                values.push(series.points[i].y);
            return values;
        },
        _getXValues: function () {
            var values = [];
            for (var i = 0; i < series.points.length; i++)
                values.push(series.points[i].X);
            return values;
        },


        _calculateStackingValues: function (seriesCollection, axis) {

            this.model.stackedValue[axis.axisName] = [];
            this.model.stackedValue[axis.axisName].min = 0;
            this.model.stackedValue[axis.axisName].max = 0;
            var stackAxes = (!this.model.requireInvertedAxes) ? this.model.hAxes : this.model.vAxes;

            for (var k = 0; k < stackAxes.length; k++) {
                j = 0;
                var lastPosValue = [];
                var lastNegValue = [];
                for (var i = 0; i < seriesCollection.length; i++) {
                    if (seriesCollection[i].xAxisName == stackAxes[k].axisName) {
                        if (seriesCollection[i].type.toLowerCase().indexOf("stacking") != -1) {
                            var values = {};
                            values.StartValues = [];
                            values.EndValues = [];
                            if (seriesCollection[i].stackingGroup && seriesCollection[i].type.toLowerCase() != "stackingarea") {
                                if (ej.util.isNullOrUndefined(lastPosValue[seriesCollection[i].stackingGroup])) {
                                    lastPosValue[seriesCollection[i].stackingGroup] = [];
                                    lastNegValue[seriesCollection[i].stackingGroup] = [];
                                }
                            }

                            else {
                                seriesCollection[i].stackingGroup = "";
                                if (ej.util.isNullOrUndefined(lastPosValue[seriesCollection[i].stackingGroup])) {
                                    lastPosValue[seriesCollection[i].stackingGroup] = [];
                                    lastNegValue[seriesCollection[i].stackingGroup] = [];
                                }
                            }
                            series = seriesCollection[i];
                            series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
                            var yValues = this._getYValues(series);
                            for (var j = 0; j < yValues.length; j++) {
                                var lastValue = 0;
                                var currentValue = yValues[j];
                                if (lastPosValue[seriesCollection[i].stackingGroup].length <= j)
                                    lastPosValue[seriesCollection[i].stackingGroup].push(0);
                                if (lastNegValue[seriesCollection[i].stackingGroup].length <= j)
                                    lastNegValue[seriesCollection[i].stackingGroup].push(0);
                                if (values.StartValues.length <= j) {
                                    values.StartValues.push(0);
                                    values.EndValues.push(0);
                                }
                                if (currentValue > 0) {
                                    lastValue = lastPosValue[seriesCollection[i].stackingGroup][j];
                                    lastPosValue[seriesCollection[i].stackingGroup][j] += currentValue;
                                }
                                else {
                                    lastValue = lastNegValue[seriesCollection[i].stackingGroup][j];
                                    lastNegValue[seriesCollection[i].stackingGroup][j] += currentValue;
                                }

                                values.StartValues[j] = lastValue;
                                values.EndValues[j] = currentValue + lastValue;
                                series.points[j].YValues = [values.EndValues[j]];
                            }
                            values.stackedSeries = false;
                            this.model.stackedValue[axis.axisName].push(values);
                            if (this.model.stackedValue[axis.axisName].min > Math.min.apply(0, values.StartValues))
                                this.model.stackedValue[axis.axisName].min = Math.min.apply(0, values.StartValues);
                            if (this.model.stackedValue[axis.axisName].max < Math.max.apply(0, values.EndValues))
                                this.model.stackedValue[axis.axisName].max = Math.max.apply(0, values.EndValues);
                            series.stackedValue = values;
                        }
                    }
                }
            }
        },
        _legendItemBounds: function (itemCount, legendItemWidth, legendItemHeight) {
            var legend = this.model.legend;
            var legnedHeightIncr, column, legendWidth = 0, legendHeight = 0;
            if (ej.util.isNullOrUndefined(legend.columnCount) && legend.rowCount) {
                legnedHeightIncr = legend.rowCount;
                column = Math.ceil(itemCount / legnedHeightIncr);
                legendWidth = legendItemWidth * column;
                legendHeight = legendItemHeight * legnedHeightIncr;
            }
            else if (ej.util.isNullOrUndefined(legend.rowCount) && legend.columnCount) {
                legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
                legendWidth = legendItemWidth * this.model.legend.columnCount;
                legendHeight = legendItemHeight * legnedHeightIncr;
            }
            else if ((legend.rowCount) && (legend.columnCount)) {
                if (legend.columnCount < legend.rowCount) {
                    legnedHeightIncr = legend.rowCount;
                    column = Math.ceil(itemCount / legnedHeightIncr);
                    legendWidth = legendItemWidth * column;
                }
                else {
                    legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
                    legendWidth = legendItemWidth * legend.columnCount;
                }
                legendHeight = legendItemHeight * legnedHeightIncr;
            }
            legendHeight += this.model.elementSpacing;
            return { LegendWidth: legendWidth, LegendHeight: legendHeight };
        },

        _calculateLegendBounds: function () {
            var chartObj = this;
            this.model.legendCollection = [];
            this.model.legendRegion = [];
            var width = 0, height = 0, legendItemWidth = 0, legendItemHeight = 0, legendWidth = 0, legendHeight = 0, legnedHeightIncr = 1;
            if (this.model._visibleSeries && this.model.legend.visible) {

                //calculating legend Position
                var legendsize;
                var style;
                var opacity;
                var colors = null;
                var colorGradName;
                if (this.model.AreaType == "none") {
                    series = this.model._visibleSeries[0];
                    var drawSeries = new ej.seriesTypes[series.type]();
                    var visiblePoints = drawSeries._pieVisiblePoints(series);
                    var point;
                    var pointindex;
                    var tempSeries;
                    var legendFont;
                    var legendItem;
                    for (j = 0; j < visiblePoints.length; j++) {
                        point = visiblePoints[j];
                        pointindex = j;
                        tempSeries = { name: point.X, Font: series.font };
                        legendsize = chartObj._getLegendSize(tempSeries);
                        legendItemWidth = Math.max(legendsize.Width, legendItemWidth);
                        legendItemHeight = Math.max(legendsize.Height, legendItemHeight);

                    }
                    if (chartObj.model.legend.columnCount || chartObj.model.legend.rowCount) {
                        legendBounds = this._legendItemBounds(visiblePoints.length, legendItemWidth, legendItemHeight);
                        legendWidth = legendBounds.LegendWidth;
                        legendHeight = legendBounds.LegendHeight;
                    }

                    for (var k = 0; k < visiblePoints.length; k++) {
                        point = visiblePoints[k];
                        pointindex = k;
                        tempSeries = { name: point.X, Font: series.font };
                        legendsize = chartObj._getLegendSize(tempSeries);
                        if (!(chartObj.model.legend.rowCount) && !(chartObj.model.legend.columnCount)) {
                            if ((chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.top || chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.bottom || chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.custom)) {
                                width += legendsize.Width;
                                if (width > $(chartObj.svgObject).width()) {
                                    width -= legendsize.Width;
                                    legendWidth = Math.max(legendWidth, width);
                                    width = legendsize.Width;
                                    legnedHeightIncr++;
                                    legendHeight = legendItemHeight * legnedHeightIncr;
                                    legendHeight += this.model.elementSpacing;
                                }
                                else
                                    legendWidth = Math.max(legendWidth, width);

                                height = Math.max(height, legendItemHeight);
                            }
                            else {
                                height += legendItemHeight;
                                legendWidth = Math.max(width, legendItemWidth);
                            }
                        }

                        var legend = (chartObj.model.legend);
                        opacity = legend.opacity;


                        if (legend.fill) {
                            colors = !(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop;
                        }
                        else {
                            colors = chartObj.model.pointColors[pointindex];
                        }


                        colorGradName = chartObj.svgRenderer.createGradientElement('legend' + pointindex, colors, 0, 0, 0, $(chartObj.svgObject).height(), chartObj.gLegendEle);



                        if (series.font)

                            legendFont = series.font;

                        else
                            legendFont = chartObj.model.legend.font;

                        var bounds = (chartObj.model.legend.rowCount || chartObj.model.legend.columnCount) ? { Width: legendItemWidth, Height: legendItemHeight } : legendsize;
                        legendItem = { Text: point.X, Shape: chartObj.model.legend.shape, LegendStyle: { BorderColor: chartObj.model.legend.itemSize.border.color, BorderWidth: chartObj.model.legend.itemSize.border.width, Opacity: opacity, Color: colorGradName, Font: legendFont }, Bounds: bounds, SeriesIndex: 0, PointIndex: pointindex };
                        chartObj.model.legendCollection.push(legendItem);


                    }
                }
                else {
                    var series;
                    for (var i = 0; i < this.model._visibleSeries.length; i++) {
                        series = this.model._visibleSeries[i];
                        legendsize = chartObj._getLegendSize(series);
                        legendItemWidth = Math.max(legendsize.Width, legendItemWidth);
                        legendItemHeight = Math.max(legendsize.Height, legendItemHeight);
                    }
                    if (chartObj.model.legend.columnCount || chartObj.model.legend.rowCount) {
                        var legendBounds = this._legendItemBounds(this.model._visibleSeries.length, legendItemWidth, legendItemHeight);
                        legendWidth = legendBounds.LegendWidth;
                        legendHeight = legendBounds.LegendHeight;
                    }
                    for (var j = 0; j < this.model._visibleSeries.length; j++) {
                        index = j;
                        series = this.model._visibleSeries[j];
                        legendsize = chartObj._getLegendSize(series);
                        if (!chartObj.model.legend.rowCount && !chartObj.model.legend.columnCount) {
                            if (chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.top || chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.bottom || chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.custom) {
                                width += legendsize.Width;
                                if (width > $(chartObj.svgObject).width()) {
                                    width -= legendsize.Width;
                                    legendWidth = Math.max(legendWidth, width);
                                    width = legendsize.Width;
                                    legnedHeightIncr++;
                                    legendHeight = legendItemHeight * legnedHeightIncr;
                                    legendHeight += this.model.elementSpacing;
                                }
                                height = Math.max(height, legendsize.Height);
                            }


                            else {

                                height += legendItemHeight;
                                width = Math.max(width, legendItemWidth);
                            }
                        }

                        var legend = (chartObj.model.legend);
                        opacity = legend.opacity;


                        //                    // calculate gradient. 

                        if (legend.fill) {
                            colors = !(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop;
                        }
                        else {
                            colors = chartObj.model.seriesColors[index];
                        }

                        if (series.visibility.toLowerCase() === 'visible')
                            colorGradName = chartObj.svgRenderer.createGradientElement('legend' + index, colors, 0, 0, 0, $(chartObj.svgObject).height(), chartObj.gLegendEle);
                        else
                            colorGradName = 'gray';

                        if (series.font)

                            legendFont = series.font;

                        else
                            legendFont = chartObj.model.legend.font;

                        bounds = (chartObj.model.legend.rowCount || chartObj.model.legend.columnCount) ? { Width: legendItemWidth, Height: legendItemHeight } : legendsize;
                        legendItem = { Text: (series.name) ? series.name : "series" + i, Shape: chartObj.model.legend.shape, LegendStyle: { BorderColor: chartObj.model.legend.itemSize.border.color, BorderWidth: chartObj.model.legend.itemSize.border.width, Opacity: opacity, Color: colorGradName, Font: legendFont }, Bounds: bounds, SeriesIndex: index };
                        chartObj.model.legendCollection.push(legendItem);
                    }
                }
                if (this.model.legend.position.toLowerCase() === this.model.legendPosition.top || this.model.legend.position.toLowerCase() === this.model.legendPosition.bottom || this.model.legend.position.toLowerCase() === this.model.legendPosition.custom) {
                    width += chartObj.model.legend.itemPadding;
                    height += chartObj.model.legend.itemPadding;
                    this.model.LegendBounds = { Width: Math.max(legendWidth, width), Height: Math.max(legendHeight, height), Rows: legnedHeightIncr };
                }
                else {
                    width += chartObj.model.legend.itemPadding;
                    height += chartObj.model.legend.itemPadding;
                    this.model.LegendBounds = { Width: Math.max(legendWidth, width), Height: Math.max(legendHeight, height), Columns: legnedHeightIncr };
                }

            }
            else {
                this.model.LegendBounds = { Width: 0, Height: 0 };
            }

        },



        GetPointXYOrgin: function (x, y, orginX, orginY) {

            var xvalue = ((x - orginX) / (this.model.axes.PrimaryXaxis.visibleRange.max - orginX)) * (this.model.m_AreaBounds.Width);
            var yvalue = ((y - orginY) / (this.model.axes.PrimaryYaxis.visibleRange.max - orginY)) * (this.model.m_AreaBounds.Height - this.GetPointXY(this.model.axes.PrimaryXaxis.visibleRange.min, Math.max(this.model.axes.PrimaryYaxis.visibleRange.min, 0)).Y);

            return { X: xvalue, Y: yvalue };
        },









        _drawChartAreaRect: function () {

            var fillColor = !(this.model.background && this.model.background._gradientStop) ? this.model.background : 'url(#' + this.svgObject.id + '_backGradient)';

            if (this.model.backGroundImageUrl || this.model.AreaType == 'none')
                fillColor = 'transparent';

            var chartBorder = this.model.border;

            var options = {
                'id': this.svgObject.id + '_SvgRect',
                'x': (chartBorder.width > 0) ? chartBorder.width / 2 : 0,
                'y': (chartBorder.width > 0) ? chartBorder.width / 2 : 0,
                'width': $(this.svgObject).width() - (2 * chartBorder.width),
                'height': $(this.svgObject).height() - (2 * chartBorder.width),
                'fill': fillColor,
                'opacity': chartBorder.opacity,
                'stroke-width': chartBorder.width,
                'stroke': chartBorder.color
            };
            this.svgRenderer.drawRect(options, this.svgObject);



            if (this.model.backGroundImageUrl)
                this._drawBackImage();

            if (this.model.AreaType == 'cartesianaxes') {
                var borderOptions = {
                    'id': this.svgObject.id + '_ChartArea',
                    'x': this.model.m_AreaBounds.X,
                    'y': this.model.m_AreaBounds.Y,
                    'width': this.model.m_AreaBounds.Width,
                    'height': this.model.m_AreaBounds.Height,
                    'fill': this.model.chartArea.background,
                    'stroke-width': this.model.chartArea.border.width,
                    'opacity': this.model.chartArea.border.opacity,
                    'stroke': this.model.chartArea.border.color
                };

                this.svgRenderer.drawRect(borderOptions, this.svgObject);

            }

        },
        axesIndexCount: function (axis, index) {
            var vRowcount = [];
            for (var k = 0; k < axis.length; k++) {
                var currentAxis = axis[k];
                if (currentAxis.orientation.toLowerCase() == "vertical") {
                    if (currentAxis.rowIndex == index)
                        vRowcount.push(currentAxis);
                }
                else {
                    if (currentAxis.columnIndex == index)
                        vRowcount.push(currentAxis);
                }
            }
            return vRowcount;
        },
        axesCount: function (axis) {
            var vRowcount = [], indexValue = [], definitionsLength = 0;
            var chartModel = this.model;
            $.each(axis, function (index, currentAxis) {
                if (currentAxis.orientation.toLowerCase() == "vertical") {
                    definitionsLength = (chartModel.rowDefinitions) ? chartModel.rowDefinitions.length : 0;
                    if (index == 0) vRowcount.push({ axis: currentAxis, index: currentAxis.rowIndex });
                    else if (currentAxis.rowIndex != vRowcount[vRowcount.length - 1].axis.rowIndex)
                        vRowcount.push({ axis: currentAxis, index: currentAxis.rowIndex });
                }
                else {
                    definitionsLength = (chartModel.columnDefinitions) ? chartModel.columnDefinitions.length : 0;
                    if (index == 0) vRowcount.push({ axis: currentAxis, index: currentAxis.columnIndex });
                    else if (currentAxis.columnIndex != vRowcount[vRowcount.length - 1].axis.columnIndex)
                        vRowcount.push({ axis: currentAxis, index: currentAxis.columnIndex });
                }
            });
            var length = vRowcount.length;
            if (definitionsLength > length) {
                length = definitionsLength;
                for (var l = 0; l < definitionsLength; l++)
                    indexValue.push(l);
            } else {
                for (var i = 0; i < length; i++)
                    indexValue.push(vRowcount[i].index);
            }

            return { length: length, indexValue: indexValue };

        },

        //Calculate position for each vertical axis
        _calRowSize: function () {
            var start = 0;
            var yaxisOrign = [];
            var totalRealLength = 0;
            var orginY = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;
            //Caluculate size for rowdefinitions values
            if (this.model._rowDefinitions[0].rowDefinitions != null) {
                for (var i = 0, rowMax = this.model._rowDefinitions.length; i < rowMax; i++) {
                    var rowDef = this.model._rowDefinitions[i].rowDefinitions;

                    var realLength = 0;
                    if (rowDef.unit.toLowerCase() == "percentage")
                        realLength = Math.floor((this.model.m_AreaBounds.Height) * (rowDef.rowHeight / 100));
                    else
                        realLength = rowDef.rowHeight;
                    if (i == 0) yaxisOrign.push(orginY);

                    yaxisOrign.push((orginY - realLength - totalRealLength));
                    totalRealLength += realLength;


                }

            }
                //Calculate size based on chart area size
            else {
                var height = Math.floor((this.model.m_AreaBounds.Height) / this.model._rowDefinitions.length);

                for (start = (orginY) ; start >= this.model.m_AreaBounds.Y; start = start - height) {
                    yaxisOrign.push(start);
                }
            }
            return yaxisOrign;
        },

        //Calculate position for each horizontal axis
        _calColumnSize: function () {
            var xaxisOrign = [];
            var orginX = this.model.m_AreaBounds.X;
            var areaWidth = this.model.m_AreaBounds.Width;
            var totalRealLength = 0;
            var columnDef = this.model._columnDefinitions;
            //Caluculate size for columndefinitions values
            if (columnDef[0].columnDefinitions != null) {
                for (var i = 0, max = columnDef.length; i < max; i++) {
                    var rowDef = columnDef[i].columnDefinitions;

                    var realLength;
                    if (rowDef.unit.toLowerCase() == "percentage")
                        realLength = Math.floor((areaWidth) * (rowDef.columnWidth / 100));
                    else
                        realLength = rowDef.columnWidth;
                    if (i == 0) xaxisOrign.push(orginX);

                    xaxisOrign.push((orginX + realLength + totalRealLength));
                    totalRealLength += realLength;
                }
            }
                //Caluculate size based on chart area size
            else {
                var width = Math.floor((areaWidth) / columnDef.length);
                for (var start = orginX, maxStart = orginX + areaWidth; start <= maxStart; start = start + width) {
                    xaxisOrign.push(start);

                }
            }
            return xaxisOrign;
        },

        //Calculate position and size for each axis
        _calculateAxisSize: function () {

            //For Horizontal
            var isInitAxes = true, isOppInitAxes = true;
            var xaxisOrign = this._calColumnSize();
            var columnDefinition = this.model._columnDefinitions;
            for (var i = 0, maxLength = columnDefinition.length; i < maxLength; i++) {
                var x1 = xaxisOrign[i];
                var x2 = xaxisOrign[i + 1];
                var nearIndexVal = 0;
                var farIndexVal = 0;;
                var isColFirstAxes = true, isColOppFirstAxes = true;
                var oppY = this.model.m_AreaBounds.Y;
                var norY = oppY + this.model.m_AreaBounds.Height;
                for (var j = 0, maxCol = columnDefinition[i].axis.length; j < maxCol; j++) {
                    var currentColAxis = columnDefinition[i].axis[j];
                    var realY = 0, axisColumnSpace = 0;
                    if (!currentColAxis.opposedPosition) {
                        if (!(isColFirstAxes && (isInitAxes || nearIndexVal == 0))) {
                            axisColumnSpace = columnDefinition.nearSizes[nearIndexVal];
                            realY = norY = (norY + axisColumnSpace);
                            nearIndexVal++;
                        }
                        else {
                            realY = norY;
                            isColFirstAxes = false;
                        }

                    } else {
                        if (!(isColOppFirstAxes && (isOppInitAxes || farIndexVal == 0))) {
                            axisColumnSpace = columnDefinition.farSizes[farIndexVal];
                            realY = oppY = (oppY - axisColumnSpace);
                            farIndexVal++;
                        }
                        else {
                            realY = oppY;
                            isColOppFirstAxes = false;
                        }
                    }
                    if (currentColAxis.columnSpan) {
                        //Skip the axis size calculation for duplicate span axis
                        if (!(columnDefinition[i].index == currentColAxis.columnIndex))
                            continue;
                        var axisSize = (maxLength >= (i + currentColAxis.columnSpan)) ? i + currentColAxis.columnSpan : i + 1;
                        x2 = xaxisOrign[axisSize];
                        if (!currentColAxis.opposedPosition && nearIndexVal == 0) isInitAxes = isColFirstAxes;
                        if (currentColAxis.opposedPosition && farIndexVal == 0) isOppInitAxes = isColOppFirstAxes;
                    } else {
                        x2 = xaxisOrign[i + 1];
                        isInitAxes = true;
                        isOppInitAxes = true;
                    }

                    currentColAxis.Location = {};
                    currentColAxis.Location.X1 = x1 + currentColAxis.plotOffset;
                    currentColAxis.Location.Y1 = realY;
                    currentColAxis.Location.X2 = x2 - currentColAxis.plotOffset;
                    currentColAxis.Location.Y2 = realY;
                    currentColAxis.x = currentColAxis.Location.X1;
                    currentColAxis.y = currentColAxis.Location.Y1;
                    currentColAxis.width = (currentColAxis.Location.X2 - currentColAxis.Location.X1);
                    currentColAxis.height = currentColAxis.Location.Y2 - currentColAxis.Location.Y1;

                }
            }

            //For Vertical
            isInitAxes = true, isOppInitAxes = true;
            var yaxisOrign = this._calRowSize();
            var rowDefinition = this.model._rowDefinitions;
            for (var i = 0, rowLength = rowDefinition.length; i < rowLength; i++) {
                var y1 = yaxisOrign[i];
                var y2 = yaxisOrign[i + 1];
                var nearIndex = 0;
                var farIndex = 0;
                var isFirstAxes = true, isOppFirstAxes = true;
                var x = this.model.m_AreaBounds.X;
                var norX = x, oppX = x + this.model.m_AreaBounds.Width;
                for (var j = 0, maxRow = rowDefinition[i].axis.length; j < maxRow; j++) {
                    var axisSpace = 0;
                    var realX = 0;

                    var currentAxis = rowDefinition[i].axis[j];

                    if (!currentAxis.opposedPosition) {
                        if (!(isFirstAxes && (isInitAxes || nearIndex == 0))) {
                            axisSpace = rowDefinition.nearSizes[nearIndex];
                            realX = norX = (norX - axisSpace);
                            nearIndex++;
                        } else {
                            realX = x;
                            isFirstAxes = false;
                        }

                    } else {
                        if (!(isOppFirstAxes && (isOppInitAxes || isOppInitAxes == 0))) {
                            axisSpace = rowDefinition.farSizes[farIndex];
                            realX = oppX = (oppX + axisSpace);
                            farIndex++;
                        } else {
                            realX = oppX;
                            isOppFirstAxes = false;
                        }
                    }
                    if (currentAxis.rowSpan) {
                        //Skip the axis size calculation for duplicate span axis
                        if (!(rowDefinition[i].index == currentAxis.rowIndex))
                            continue;
                        var axisRowSize = (rowLength >= (i + currentAxis.rowSpan)) ? i + currentAxis.rowSpan : i + 1;
                        y2 = yaxisOrign[axisRowSize];
                        if (!currentAxis.opposedPosition && nearIndex == 0) isInitAxes = isFirstAxes;
                        if (currentAxis.opposedPosition && farIndex == 0) isOppInitAxes = isOppFirstAxes;
                    } else {
                        y2 = yaxisOrign[i + 1];
                        isInitAxes = true;
                        isOppInitAxes = true;
                    }
                    currentAxis.Location = {};
                    currentAxis.Location.X1 = realX;
                    currentAxis.Location.Y1 = y1 - currentAxis.plotOffset;
                    currentAxis.Location.X2 = realX;
                    currentAxis.Location.Y2 = y2 + currentAxis.plotOffset;
                    currentAxis.x = currentAxis.Location.X1;
                    currentAxis.y = currentAxis.Location.Y2;
                    currentAxis.height = (currentAxis.Location.Y1 - currentAxis.Location.Y2);
                    currentAxis.width = currentAxis.axisLine.width;

                }
            }
        },
        _arrangeAxis: function () {
            var chartobj = this;
            var axis;
            var customRow;
            //Generate columnIndex/RowIndex value if it is not specify in sample  
            for (var i = 0; i < this.model.axes.length; i++) {
                axis = this.model.axes[i];
                if (axis.orientation.toLowerCase() == "horizontal") {
                    chartobj.model.hAxes.push(axis);
                    axis.columnIndex = (ej.util.isNullOrUndefined(axis.columnIndex)) ? 0 : (axis.columnIndex);
                    chartobj.model.hAxes[chartobj.model.hAxes.length - 1].columnIndex = axis.columnIndex;
                } else if (axis.orientation.toLowerCase() == "vertical") {

                    chartobj.model.vAxes.push(axis);
                    axis.rowIndex = (ej.util.isNullOrUndefined(axis.rowIndex)) ? 0 : (axis.rowIndex);
                    chartobj.model.vAxes[chartobj.model.vAxes.length - 1].rowIndex = axis.rowIndex;
                    customRow = (!ej.util.isNullOrUndefined(chartobj.model.rowDefinitions)) ? chartobj.model.rowDefinitions[axis.rowIndex] : null;
                    if (customRow) {
                        axis.axisBottomLine = {};
                        axis.axisBottomLine.visible = true;
                        axis.axisBottomLine.lineWidth = customRow.lineWidth;
                        axis.axisBottomLine.color = customRow.lineColor;
                    }
                }
            }

            this.model.hAxes = ej.DataManager(this.model.hAxes, ej.Query().sortBy("columnIndex")).executeLocal();

            this.model.vAxes = ej.DataManager(this.model.vAxes, ej.Query().sortBy("rowIndex")).executeLocal();

            //column axis information push into _rowDefinitions
            var rowCount = this.axesCount(chartobj.model.vAxes);
            chartobj.model._rowDefinitions = [];
            if (chartobj.model.vAxes.length > 0) {
                for (var j = 0; j < rowCount.length; j++) {
                    var rowCollection = chartobj.model.rowDefinitions;
                    var rowDefinitions = ((rowCollection)) ? (rowCollection[j] ? rowCollection[j] : rowCollection[0]) : null;
                    chartobj.model._rowDefinitions.push({
                        axis: this.axesIndexCount(chartobj.model.vAxes, rowCount.indexValue[j])
                                    , index: rowCount.indexValue[j], rowDefinitions: rowDefinitions
                    });
                }
            }

            var rowDefinition = chartobj.model._rowDefinitions;
            //Using filter to get spanning row axis collection
            var spanningRow = this._axisFilter(rowDefinition, "rowSpan");
            //Arrange the entire axis for row span
            this._SpanningAxes(rowDefinition, spanningRow, true);

            //column axis information push into _columnDefinitions
            var columnCount = this.axesCount(chartobj.model.hAxes);
            chartobj.model._columnDefinitions = [];
            if (chartobj.model.hAxes.length > 0) {
                for (var k = 0; k < columnCount.length; k++) {
                    var columnCollection = chartobj.model.columnDefinitions;
                    var columnDefinitions = (columnCollection) ? (columnCollection[k] ? columnCollection[k] : columnCollection[0]) : null;
                    chartobj.model._columnDefinitions.push({
                        axis: this.axesIndexCount(chartobj.model.hAxes, columnCount.indexValue[k])
                        , index: columnCount.indexValue[k], columnDefinitions: columnDefinitions
                    });
                }
            }

            var columnDefinition = chartobj.model._columnDefinitions;
            //Using filter to get spanning column axis collection
            var spanningColumn = this._axisFilter(columnDefinition, "columnSpan");
            //Arrange the entire axis for column span
            this._SpanningAxes(columnDefinition, spanningColumn, false);

            this._axisSize();


        },

        //Arrange the entire axis for spanning
        _SpanningAxes: function (definition, spanningCollection, isRow) {
            for (var spanIndex = 0, spanMax = spanningCollection.length; spanIndex < spanMax; spanIndex++) {
                var currentAxis = spanningCollection[spanIndex];
                if (currentAxis == null) break;
                var span = isRow ? currentAxis.rowSpan : currentAxis.columnSpan;
                var index = isRow ? currentAxis.rowIndex : currentAxis.columnIndex;
                var axisIndex = definition[index].axis.indexOf(currentAxis);
                for (var k = 1, m = index + 1; k < span && m < definition.length; k++, m++) {
                    if (definition[m].axis.length >= axisIndex) {
                        //Insert duplicate span row to appropriate rowdefinition
                        definition[m].axis.splice(axisIndex, 0, currentAxis);
                    }
                }
            }
        },

        //Filter span axis from axes
        _axisFilter: function (definition, fieldName) {
            var axisResult = [];
            definition.filter(function (axes) {
                //Execute query to get spanning axis from axis array
                var axisCollection = ej.DataManager(axes["axis"], ej.Query().where(fieldName, ">", 1)).executeLocal();
                for (var i = 0, max = axisCollection.length; i < max; i++)
                    axisResult.push(axisCollection[i]);
            });
            return axisResult;
        },

        _axisSize: function () {
            //The below calcultion to get approximate length of axis
            var spaceValue = this._getLegendSpace();
            var hSpace = $(this.svgObject).width() - this.model.margin.left - this.model.margin.right - (this.model.elementSpacing * 2) - spaceValue.leftLegendWidth - spaceValue.rightLegendWidth;
            var hWidth = Math.floor(hSpace / this.axesCount(this.model.hAxes).length);

            var vSpace = $(this.svgObject).height() - this.model.margin.top - this.model.margin.bottom - (this.model.elementSpacing * 2) - spaceValue.topLegendHeight - spaceValue.bottomLegendHeight - spaceValue.modelTitleHeight;
            var vWidth = Math.floor(vSpace / this.axesCount(this.model.vAxes).length);

            for (var j = 0; j < this.model.axes.length; j++) {
                axis = this.model.axes[j];
                if (axis.orientation.toLowerCase() == "horizontal") {
                    var customColumn = (!ej.util.isNullOrUndefined(this.model.columnDefinitions)) ? this.model.columnDefinitions[axis.columnIndex] : null;
                    var columnLength = hWidth;
                    if (customColumn) {
                        if (customColumn.unit.toLowerCase() == "percentage")
                            columnLength = Math.floor((hWidth) * (customColumn.columnWidth / 100));
                        else
                            columnLength = customColumn.columnWidth;

                    }
                    axis.length = axis.columnSpan ? axis.columnSpan * columnLength : columnLength;;
                } else if (axis.orientation.toLowerCase() == "vertical") {
                    var customRow = (!ej.util.isNullOrUndefined(this.model.rowDefinitions)) ? this.model.rowDefinitions[axis.rowIndex] : null;
                    var realLength = vWidth;
                    if (customRow) {
                        if (customRow.unit.toLowerCase() == "percentage")
                            realLength = Math.floor((vWidth) * (customRow.rowHeight / 100));
                        else
                            realLength = customRow.rowHeight;

                    }
                    axis.length = axis.rowSpan ? (axis.rowSpan * realLength) : realLength;
                }
            }
        },
        _axisMeasure: function (axis, realWidth, orientation) {
            var axisTitleHeight = 0;
            var labelSize = 0;
            if (orientation == "vertical") {
                labelSize = axis._LableMaxWidth.width;
                axisTitleHeight = (axis.title.text == "" || !(axis.visible)) ? 0 : (ej.EjSvgRender.utils._measureText(axis.title.text, realWidth, axis.title.font).height + (2 * this.model.elementSpacing));
            }
            else {
                labelSize = (axis._LableMaxWidth.height);
                axisTitleHeight = (axis.title.text == "" || !(axis.visible)) ? 0 : (ej.EjSvgRender.utils._measureText(axis.title.text, realWidth, axis.title.font).height + (this.model.elementSpacing));
            }
            axis.AxisMaxWidth = this.model.elementSpacing + axisTitleHeight + labelSize + axis.majorTickLines.size + axis.axisLine.width;
            return axis.AxisMaxWidth;
        },

        _arraySum: function (val) {
            var total = 0;
            for (var i = 0; i < val.length; i++) {
                total += parseFloat(val[i]) || 0;
            }
            return total;
        },
        _calSpace: function (realWidth, definitions, orientation) {
            var nearSizes = [];
            var farSizes = [];
            for (var i = 0; i < definitions.length; i++) {
                var nearIndex = 0, farIndex = 0;
                for (var j = 0; j < definitions[i].axis.length; j++) {
                    var currentaxis = definitions[i].axis[j];
                    var measureValue = this._axisMeasure(currentaxis, realWidth, orientation);
                    if (!currentaxis.opposedPosition) {
                        if (nearSizes.length <= nearIndex) {
                            nearSizes.push(measureValue);
                        } else if (nearSizes[nearIndex] < (measureValue)) {
                            nearSizes[nearIndex] = measureValue;
                        }
                        nearIndex++;
                    }
                    else {
                        if (farSizes.length <= farIndex) {
                            farSizes.push(measureValue);
                        } else if (farSizes[farIndex] < (measureValue)) {
                            farSizes[farIndex] = measureValue;
                        }
                        farIndex++;
                    }

                }
            }
            return { nearSizes: nearSizes, farSizes: farSizes };
        },

        _getLegendSpace: function () {
            var position = this.model.legend.position.toLowerCase();
            var legendPosition = this.model.legendPosition;
            var space = (this.model.LegendBounds.Width + (this.model.legend.itemPadding / 2) + this.model.elementSpacing);
            var leftLegendWidth = position == legendPosition.left ? space : 0;
            var rightLegendWidth = position == legendPosition.right ? space : 0;

            var topLegendHeight = position == legendPosition.top ? (this.model.LegendBounds.Height) : 0;
            var bottomLegendHeight = position == legendPosition.bottom ? (this.model.LegendBounds.Height) : 0;
            var modelTitleHeight = (this.model.title.text == "") ? 0 : (ej.EjSvgRender.utils._measureText(this.model.title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.font).height + this.model.elementSpacing);

            return { leftLegendWidth: leftLegendWidth, rightLegendWidth: rightLegendWidth, topLegendHeight: topLegendHeight, bottomLegendHeight: bottomLegendHeight, modelTitleHeight: modelTitleHeight };


        },
        _calculateAreaBounds: function () {
            var chartobj = this;
            var realWidth = $(this.svgObject).width() - this.model.margin.left - this.model.margin.right;
            // Calculate area bounds X and width
            var chartBorderWidth = chartobj.model.border.width;

            //Calcultion for multiple axes(vertical)
            var sizes = this._calSpace(realWidth, chartobj.model._rowDefinitions, "vertical");
            chartobj.model._rowDefinitions.nearSizes = sizes.nearSizes;
            chartobj.model._rowDefinitions.farSizes = sizes.farSizes;
            var vAxesWidth = this._arraySum(sizes.nearSizes);
            var vAxesOppWidth = this._arraySum(sizes.farSizes);

            var spaceValue = this._getLegendSpace();

            var x = vAxesWidth + spaceValue.leftLegendWidth + chartobj.model.margin.left + chartBorderWidth;
            var rightSpacing = vAxesOppWidth + spaceValue.rightLegendWidth + chartobj.model.margin.right + chartobj.model.margin.left + (2 * chartBorderWidth);
            var boundsWidth = $(this.svgObject).width() - (x + rightSpacing);

            // Calculate area bounds Y and Height   

            //Calcultion for multiple axes(vertical)
            var columnSizes = this._calSpace(realWidth, chartobj.model._columnDefinitions, "horizontal");
            chartobj.model._columnDefinitions.nearSizes = columnSizes.nearSizes;
            chartobj.model._columnDefinitions.farSizes = columnSizes.farSizes;
            var hAxesWidth = this._arraySum(columnSizes.nearSizes);
            var hAxesOppWidth = this._arraySum(columnSizes.farSizes);

            var y = hAxesOppWidth + this.model.margin.top + this.model.elementSpacing + spaceValue.modelTitleHeight + spaceValue.topLegendHeight + chartBorderWidth;
            var bottomSpacing = hAxesWidth + this.model.margin.bottom + spaceValue.bottomLegendHeight + (2 * chartBorderWidth);
            var boundsHeight = Math.abs($(this.svgObject).height() - (y + bottomSpacing));

            this.model.m_AreaBounds = { X: x, Y: y, Width: boundsWidth, Height: boundsHeight };
            this.model.m_Spacing = { Left: x, Top: y, Right: rightSpacing, Bottom: bottomSpacing };
        }
    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };



})(jQuery);;
/**
* @fileOverview Plugin to style the Html Chart elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej) {
    /**
     * @namespace ej
     * @class ejChart
     * @requires jQuery
     * @requires ej.chart
     * @classdesc Defined values to render a chart.
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; <br> 
     * &lt;script&gt;
     * // Create Chart
     * $('#container').ejChart(); 	
     * &lt;/script&gt;
     */

    ej.widget("ejChart", "ej.Chart", {

        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,

        validTags: ["div"],

        // default model
        defaults: {
            /**		
			* Specifies the border fot the chart. By specifying the border properties we can draw border around the chart. And also we can customize the color and width of the border. 
			* @type {object}
			 * @memberof ejChart
			* @instance
			*/
            border: {
                /**		
			* Specifies the color of the border. The user can set desired color to chart border.
              @default null
			* @type {string}
			* @example 
			* //To set border color API value during initialization  
			* 	$("#container").ejChart({border:{color:"#ff0000"}});					
			* @example 
			* //Gets or sets the border color API, after initialization:<br>
			*	//Gets the border color value  
			*	$("#container").ejChart("option", "border.color");<br>			
			*	//Sets the border color value 
			*	$("#container").ejChart("option", {border:{color:"#ff0000"}} ); 
			 * @alias ejChart#border->color
			* @instance
			*/
                color: 'transparent',
                /**		
			* Specifies the width of the border. The user can set desired width to chart border.
			* @default 0
			* @type {number}
			* @example 
			* //To set border width API value during initialization  
			* 	$("#container").ejChart({border:{width:2}});				 
			* @example 
			* //Gets or sets the border width API, after initialization:<br>
			*	//Gets the border width value  
			*	$("#container").ejChart("option", "border.width");<br>			
			*	//Sets the border width value 
			*	$("#container").ejChart("option", {border:{width:2}} ); 
			 * @alias ejChart#border->width
			* @instance
			*/
                width: 0,
                /**		
			* Specifies the opacity of the border. The user can set desired opacity to chart border.
			* @default 0.3
			* @type {number}
			* @example 
			* //To set border opacity API value during initialization  
			* 	$("#container").ejChart({border:{width:0}});				
			* @example 
			* //Gets or sets the border opacity API, after initialization:<br>
			*	//Gets the border opacity value  
			*	$("#container").ejChart("option", "border.opacity");<br>			
			*	//Sets the border opacity value 
			*	$("#container").ejChart("option", {border:{opacity:0}} ); 
			 * @alias ejChart#border->opacity
			* @instance
			*/
                opacity: 0.3
            },
            /**		
			* Specifies the chartArea. ChartArea represents the area where the chart is rendered. 
			* @type {object}
			 * @memberof ejChart
			* @instance
			*/
            chartArea:
            {
                /**		
			* Specifies the border of chartArea. By using these properties we can customize the border of chartArea. 
			* @type {object}
			 * @alias ejChart#chartArea->border
			* @instance
			*/
                border: {
                    /**		
          * Specifies the border color of chart area. The user can set desired color to chartArea border.
          * @default Gray
          * @type {string}
          * @example 
          * //To set color of chart area value during initialization  
          * 	$("#container").ejChart({chartArea:{border:{ color:"Green"}}});				
          * @example 
          * //Gets or sets the border color API, after initialization:<br>
          *	//Gets the border color of chart area  
          *	$("#container").ejChart("option", "chartArea.border.color");<br>			
          *	//Sets the border color of chart area
          *	$("#container").ejChart("option", {chartArea:{border:{ color:"Green"}}} ); 
           * @alias ejChart#chartArea->border->color
          * @instance
          */
                    color: 'Gray',
                    /**		
          * Specifies the border width of chart area. The user can set desired width to chartArea border.
          * @default 0.5
          * @type {number}
          * @example 
          * //To set border width of chart area value during initialization  
          * 	$("#container").ejChart({chartArea:{border:{width:2}}});				
          * @example 
          * //Gets or sets the border width API, after initialization:<br>
          *	//Gets the border width of chart area  
          *	$("#container").ejChart("option", "chartArea.border.width");<br>			
          *	//Sets the border width of chart area
          *	$("#container").ejChart("option", {chartArea:{border:{width:2}}} ); 
           * @alias ejChart#chartArea->border->width
          * @instance
          */
                    width: 0.5,
                    /**		
          * Specifies the boder opacity for chart area.  The user can set desired opacity to chartArea border.
          * @default 0.3
          * @type {number}
          * @example 
          * //To set border opacity of chart area value during initialization  
          * 	$("#container").ejChart({chartArea:{border:{ opacity:1}}});				
          * @example 
          * //Gets or sets the border opacity API, after initialization:<br>
          *	//Gets the border opacity of chart area  
          *	$("#container").ejChart("option", "chartArea.border.opacity");<br>			
          *	//Sets the border opacity of chart area
          *	$("#container").ejChart("option", {chartArea:{border:{ opacity:1}}} ); 
           * @alias ejChart#chartArea->border->opacity
          * @instance
          */
                    opacity: 0.3
                },
                /**		
           * Specifies the background for chart area. By applying this we can change the background color of the chartArea.
           * @default transparent
           * @type {string}
           * @example 
           * //To set background of chart area value during initialization  
           * 	$("#container").ejChart({chartArea:{background:"#ff0000"}});				
           * @example 
           * //Gets or sets the border opacity API, after initialization:<br>
           *	//Gets the background of chart area  
           *	$("#container").ejChart("option", "chartArea.background");<br>			
           *	//Sets the background of chart area
           *	$("#container").ejChart("option", chartArea:{{background:"#ff0000"}} ); 
            * @alias ejChart#chartArea->background
           * @instance
           */
                background: 'transparent'

            },
            /**		
			* Specifies the primaryXAxis. This includes the properties relate to the primary x axis. 
			* @type {object}
			 * @memberof ejChart
			* @instance
			*/
            primaryXAxis:
     {
         /**		
     * Specifies the majorGridLines of primaryXAxis. These are properties involved in customizing the majorGridLines of chart. 
     * @type {object}
     * @alias ejChart#primaryXAxis->majorGridLines
     * @instance
     */
         majorGridLines:
{
    /**		
* Specifies the width of the majorGridLines. The user can modify the width of the majorGridLines in primaryXAxis.
* @default 1
* @type {number}
* @example 
* //To set width of majorGridLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{majorGridLines:{width:0.5}}});				
* @example 
* //Gets or sets the width of the majorGridLines, after initialization:<br>
*	//Gets the width of the majorGridLines  
*	$("#container").ejChart("option", "primaryXAxis.majorGidLines.width");<br>			
*	//Sets the width of the majorGridLines
*	$("#container").ejChart("option", {primaryXAxis:{majorGridLines:{width:0.5}}} ); 
 * @alias ejChart#primaryXAxis->majorGridLines->width
* @instance
*/
    width: 1,
    /**		
* Specifies the dashArray of the majorGridLines. The user can modify the dashArray of the majorGridLines in primaryXAxis.
* @default null
* @type {string}
* @example 
* //To set dashArray of majorGridLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{majorGridLines:{dashArray:"2,3"}}});				
* @example 
* //Gets or sets the dashArray of the majorGridLines, after initialization:<br>
*	//Gets the dashArray of the majorGridLines  
*	$("#container").ejChart("option", "primaryXAxis.majorGidLines.dashArray");<br>			
*	//Sets the dashArray of the majorGridLines
*	$("#container").ejChart("option", {primaryXAxis:{majorGridLines:{dashArray:"2,3"}}} ); 
 * @alias ejChart#primaryXAxis->majorGridLines->dashArray
* @instance
*/
    dashArray: "",
    /**		
* Specifies the visibility of the majorGridLines. The user can modify the visibility of the majorGridLines in primaryXAxis.
* @default true
* @type {boolean}
* @example 
* //To set visibility of majorGridLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{majorGridLines:{visible:false}}});				
* @example 
* //Gets or sets the visibility of the majorGridLines, after initialization:<br>
*	//Gets the visibility of the majorGridLines  
*	$("#container").ejChart("option", "primaryXAxis.majorGidLines.visible");<br>			
*	//Sets the visibility of the majorGridLines
*	$("#container").ejChart("option", {primaryXAxis:{majorGridLines:{visible:false}}} ); 
 * @alias ejChart#primaryXAxis->majorGridLines->visible
* @instance
*/
    visible: true,
    /**		
* Specifies the opacity of the majorGridLines. The user can modify the opacity of the majorGridLines in primaryXAxis.
* @default 1
* @type {number}
* @example 
* //To set opacity of majorGridLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{majorGridLines:{opacity:0}}});				
* @example 
* //Gets or sets the opacity of the majorGridLines, after initialization:<br>
*	//Gets the opacity of the majorGridLines  
*	$("#container").ejChart("option", "primaryXAxis.majorGidLines.opacity");<br>			
*	//Sets the opacity of the majorGridLines
*	$("#container").ejChart("option", {primaryXAxis:{majorGridLines:{opacity:0}}} ); 
 * @alias ejChart#primaryXAxis->majorGridLines->opacity
* @instance
*/
    opacity: 1

},
         /**		
     * Specifies the majorTickLines of primaryXAxis.These are properties involved in customizing the majorTickLines of chart.
     * @type {object}
 * @alias ejChart#primaryXAxis->majorTickLines
     * @instance
     */
         majorTickLines:
 {
     /**		
* Specifies the width of the majorTickLines. The user can modify the width of the majorTickLines in primaryXAxis.
* @default 1
* @type {number}
* @example 
* //To set width of majorTickLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{majorTickLines:{width:0}}});				
* @example 
* //Gets or sets the width of the majorTickLines, after initialization:<br>
*	//Gets the width of the majorTickLines  
*	$("#container").ejChart("option", "primaryXAxis.majorTickLines.width");<br>			
*	//Sets the width of the majorTickLines
*	$("#container").ejChart("option", {primaryXAxis:{majorTickLines:{width:0}}} ); 
 * @alias ejChart#primaryXAxis->majorTickLines->width
* @instance
*/
     width: 1,
     /**		
* Specifies the size of the majorTickLines. The user can modify the size of the majorTickLines in primaryXAxis.
* @default 5
* @type {number}
* @example 
* //To set size of majorTickLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{majorTickLines:{size:3}}});				
* @example 
* //Gets or sets the size of the majorTickLines, after initialization:<br>
*	//Gets the size of the majorTickLines  
*	$("#container").ejChart("option", "primaryXAxis.majorTickLines.size");<br>			
*	//Sets the size of the majorTickLines
*	$("#container").ejChart("option", {primaryXAxis:{majorTickLines:{size:3}}} ); 
 * @alias ejChart#primaryXAxis->majorTickLines->size
* @instance
*/
     size: 5,
     /**		
* Specifies the visibility of the majorTickLines. The user can modify the visibility of the majorTickLines in primaryXAxis.
* @default true
* @type {boolean}
* @example 
* //To set visibility of majorTickLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{majorTickLines:{visible:false}}});				
* @example 
* //Gets or sets the visibility of the majorTickLines, after initialization:<br>
*	//Gets the visibility of the majorTickLines  
*	$("#container").ejChart("option", "primaryXAxis.majorTickLines.visible");<br>			
*	//Sets the visibility of the majorTickLines
*	$("#container").ejChart("option", {primaryXAxis:{majorTickLines:{visible:false}}} ); 
 * @alias ejChart#primaryXAxis->majorTickLines->visible
* @instance
*/
     visible: true
 },
         /**		
     * Specifies the minorGridLines of primaryXAxis. These are properties involved in customizing the minorGridLines of chart. 
     * @type {object}
      * @alias ejChart#primaryXAxis->minorGridLines
     * @instance
     */
         minorGridLines:
 {
     /**		
* Specifies the width of the minorGridLines. The user can modify the width of the minorGridLines in primaryXAxis.
* @default 1
* @type {number}
* @example 
* //To set width of minorGridLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{minorGridLines:{width:2}}});				
* @example 
* //Gets or sets the width of the minorGridLines, after initialization:<br>
*	//Gets the width of the minorGridLines  
*	$("#container").ejChart("option", "primaryXAxis.minorGridLines.width");<br>			
*	//Sets the width of the minorGridLines
*	$("#container").ejChart("option", {primaryXAxis:{minorGridLines:{width:2}}} ); 
* @alias ejChart#primaryXAxis->minorGridLines->width
* @instance
*/
     width: 1,
     /**		
* Specifies the dashArray of the minorGridLines. The user can modify the dashArray of the minorGridLines in primaryXAxis.
* @default null
* @type {string}
* @example 
* //To set dashArray of minorGridLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{minorGridLines:{dashArray:"2,3"}}});				
* @example 
* //Gets or sets the dashArray of the minorGridLines, after initialization:<br>
*	//Gets the dashArray of the minorGridLines  
*	$("#container").ejChart("option", "primaryXAxis.minorGridLines.dashArray");<br>			
*	//Sets the dashArray of the minorGridLines
*	$("#container").ejChart("option", {primaryXAxis:{minorGridLines:{dashArray:"2,3"}}} ); 
* @alias ejChart#primaryXAxis->minorGridLines->dashArray
* @instance
*/
     dashArray: "",
     /**		
* Specifies the visibility of the minorGridLines. The user can modify the visibility of the minorGridLines in primaryXAxis.
* @default false
* @type {boolean}
* @example 
* //To set visibility of minorGridLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{minorGridLines:{visible:true}}});				
* @example 
* //Gets or sets the visibility of the minorGridLines, after initialization:<br>
*	//Gets the visibility of the minorGridLines  
*	$("#container").ejChart("option", "primaryXAxis.minorGridLines.visible");<br>			
*	//Sets the visibility of the minorGridLines
*	$("#container").ejChart("option", {primaryXAxis:{minorGridLines:{visible:true}}} ); 
* @alias ejChart#primaryXAxis->minorGridLines->visible
* @instance
*/
     visible: false

 },
         /**		
     * Specifies the minorTickLines of primaryXAxis. These are properties involved in customizing the minorTickLines of chart.
     * @type {object}
     * @alias ejChart#primaryXAxis->minorTickLines
     * @instance
     */
         minorTickLines:
{
    /**		
* Specifies the width of the minorTickLines. The user can modify the width of the minorTickLines in primaryXAxis.
* @default 1
* @type {number}
* @example 
* //To set width of minorTickLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{minorTickLines:{width:2}}});				
* @example 
* //Gets or sets the width of the minorTickLines, after initialization:<br>
*	//Gets the width of the minorTickLines  
*	$("#container").ejChart("option", "primaryXAxis.minorTickLines.width");<br>			
*	//Sets the width of the minorTickLines
*	$("#container").ejChart("option", {primaryXAxis:{minorTickLines:{width:2}}} ); 
* @alias ejChart#primaryXAxis->minorTickLines->width
* @instance
*/
    width: 1,
    /**		
* Specifies the size of the minorTickLines.  The user can modify the size of the minorTickLines in primaryXAxis.
* @default 5
* @type {number}
* @example 
* //To set size of minorTickLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{minorTickLines:{size:2}}});				
* @example 
* //Gets or sets the size of the minorTickLines, after initialization:<br>
*	//Gets the size of the minorTickLines  
*	$("#container").ejChart("option", "primaryXAxis.minorTickLines.size");<br>			
*	//Sets the size of the minorTickLines
*	$("#container").ejChart("option", {primaryXAxis:{minorTickLines:{size:2}}} ); 
* @alias ejChart#primaryXAxis->minorTickLines->size
* @instance
*/
    size: 5,
    /**		
* Specifies the visibility of the minorTickLines.  The user can modify the visibility of the minorTickLines in primaryXAxis.
* @default false
* @type {boolean}
* @example 
* //To set visibility of minorTickLines value during initialization  
* 	$("#container").ejChart({primaryXAxis:{minorTickLines:{visible:true}}});				
* @example 
* //Gets or sets the visibility of the minorTickLines, after initialization:<br>
*	//Gets the visibility of the minorTickLines  
*	$("#container").ejChart("option", "primaryXAxis.minorTickLines.visible");<br>			
*	//Sets the visibility of the minorTickLines
*	$("#container").ejChart("option", {primaryXAxis:{minorTickLines:{visible:true}}} ); 
* @alias ejChart#primaryXAxis->minorTickLines->visible
* @instance
*/
    visible: false
},
         /**		
      * Specifies the minorTicksPerInterval of primaryXAxis. This property is to set the number of minor ticks to render per interval
      * @default null
      * @type {number}
      * @example 
      * //To set minorTicksPerInterval of primaryXAxis value during initialization  
      * 	$("#container").ejChart({primaryXAxis:{minorTicksPerInterval:3}});				
      * @example 
      * //Gets or sets the minorTicksPerInterval of primaryXAxis, after initialization:<br>
      *	//Gets the minorTicksPerInterval of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.minorTicksPerInterval");<br>			
      *	//Sets the minorTicksPerInterval of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{minorTicksPerInterval:3}} ); 
* @alias ejChart#primaryXAxis->minorTicksPerInterval
      * @instance
      */
         minorTicksPerInterval: null,
         /**		
     * Specifies the columnIndex of primaryXAxis. This is to specify the index of the column.
     * @default null
     * @type {number}
     * @example 
     * //To set columnIndex of primaryXAxis value during initialization  
     * 	$("#container").ejChart({primaryXAxis:{columnIndex:2}});				
     * @example 
     * //Gets or sets the columnIndex of primaryXAxis, after initialization:<br>
     *	//Gets the columnIndex of primaryXAxis  
     *	$("#container").ejChart("option", "primaryXAxis.columnIndex");<br>			
     *	//Sets the columnIndex of primaryXAxis
     *	$("#container").ejChart("option", {primaryXAxis:{columnIndex:2}} ); 
* @alias ejChart#primaryXAxis->columnIndex
     * @instance
     */
         columnIndex: null,
         /**		
      * Specifies the columnSpan of primaryXAxis. EjChart provides option to span the column. This is achieved by using this property.
      * @default null
      * @type {number}
      * @example 
      * //To set columnSpan of primaryXAxis value during initialization  
      * 	$("#container").ejChart({primaryXAxis:{columnSpan:2}});				
      * @example 
      * //Gets or sets the columnSpan of primaryXAxis, after initialization:<br>
      *	//Gets the columnSpan of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.columnSpan");<br>			
      *	//Sets the columnSpan of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{columnSpan:2}} ); 
* @alias ejChart#primaryXAxis->columnSpan
      * @instance
      */
         columnSpan: null,
         /**		
      * Specifies the rotation of labels. The user can rotate the labels to desired angle. The axis labels are rendered in the degree, specified for the label rotation property.
      * @default null
      * @type {number}
      * @example 
      * //To set rotation of labels  
      * 	$("#container").ejChart({primaryXAxis:{labelRotation:45}});				
      * @example 
      * //Gets or sets the rotation of labels, after initialization:<br>
      *	//Gets the label rotation value  
      *	$("#container").ejChart("option", "primaryXAxis.labelRotation");<br>			
      *	//Sets the rotation of labels
      *	$("#container").ejChart("option", {primaryXAxis:{labelRotation:45}} ); 
      * @alias ejChart#primaryXAxis->labelRotation
      * @instance
      */
         labelRotation: null,
         /**		
     * Specifies the valueType of primaryXAxis. It denotes the type of axis. It can be double, datetime, etc. See {@link valueType}
     * @default null
     * @type {enum}
     * @example 
     * //To set valueType of primaryXAxis
     * 	$("#container").ejChart({primaryXAxis:{valueType:"Double"}});				
     * @example 
     * //Gets or sets the valueType of primaryXAxis, after initialization:<br>
     *	//Gets the valueType of primaryXAxis  
     *	$("#container").ejChart("option", "primaryXAxis.valueType");<br>			
     *	//Sets the valueType of primaryXAxis
     *	$("#container").ejChart("option", {primaryXAxis:{valueType:"Double"}} ); 
     * @alias ejChart#primaryXAxis->valueType
     * @instance
     */
         valueType: null,
         /**		
       * Specifies the axisName of primaryXAxis. This is the name to represent the axis in chart.
       * @default null
       * @type {string}
       * @example 
       * //To set axisName of primaryXAxis
       * 	$("#container").ejChart({primaryXAxis:{axisName:"xaxis"}});				
       * @example 
       * //Gets or sets the axisName of primaryXAxis, after initialization:<br>
       *	//Gets the axisName of primaryXAxis  
       *	$("#container").ejChart("option", "primaryXAxis.axisName");<br>			
       *	//Sets the axisName of primaryXAxis
       *	$("#container").ejChart("option", {primaryXAxis:{axisName:"xaxis"}} ); 
       * @alias ejChart#primaryXAxis->axisName
       * @instance
       */
         axisName: null,
         /**		
      * Specifies the labelFormat of primaryXAxis. The axis labels render in the format, which is specified here.
      * @default null
      * @type {string}
      * @example 
      * //To set labelFormat of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{labelFormat:"${value}"}});				
      * @example 
      * //Gets or sets the labelFormat of primaryXAxis, after initialization:<br>
      *	//Gets the labelFormat of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.labelFormat");<br>			
      *	//Sets the labelFormat of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{labelFormat:"${value}"}} ); 
      * @alias ejChart#primaryXAxis->labelFormat
      * @instance
      */
         labelFormat: null,
         /**		
     * Specifies the desiredIntervals of primaryXAxis. By this property we can display the labels after appropriate intervals.
     * @default null
     * @type {number}
     * @example 
     * //To set desiredIntervals of primaryXAxis
     * 	$("#container").ejChart({primaryXAxis:{desiredIntervals:5}});				
     * @example 
     * //Gets or sets the desiredIntervals of primaryXAxis, after initialization:<br>
     *	//Gets the desiredIntervals of primaryXAxis  
     *	$("#container").ejChart("option", "primaryXAxis.desiredIntervals");<br>			
     *	//Sets the desiredIntervals of primaryXAxis
     *	$("#container").ejChart("option", {primaryXAxis:{desiredIntervals:5}} ); 
     * @alias ejChart#primaryXAxis->desiredIntervals
     * @instance
     */
         desiredIntervals: null,
         /**		
      * Specifies the intervalType of primaryXAxis. This is to set the type of intervals to chart. IntervalType may be days, years, etc. See {@link intervalType}
      * @default null
      * @type {enum}
      * @example 
      * //To set desiredIntervintervalTypeals of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{intervalType:"days"}});				
      * @example 
      * //Gets or sets the intervalType of primaryXAxis, after initialization:<br>
      *	//Gets the intervalType of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.intervalType");<br>			
      *	//Sets the intervalType of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{intervalType:"days"}} ); 
       * @alias ejChart#primaryXAxis->intervalType
      * @instance
      */
         intervalType: null,
         /**		
      * Specifies the roundingPlaces of primaryXAxis. This property is to truncate the decimal values after the specified position.
      * @default null
      * @type {number}
      * @example 
      * //To set roundingPlaces of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{roundingPlaces:3}});				
      * @example 
      * //Gets or sets the roundingPlaces of primaryXAxis, after initialization:<br>
      *	//Gets the roundingPlaces of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.roundingPlaces");<br>			
      *	//Sets the roundingPlaces of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{roundingPlaces:3}} ); 
        * @alias ejChart#primaryXAxis->roundingPlaces
      * @instance
      */
         roundingPlaces: null,
         /**		
      * Specifies the logBase of primaryXAxis. LogBase is used in log axis to render certain values.
      * @default 10
      * @type {number}
      * @example 
      * //To set logBase of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{logBase:5}});				
      * @example 
      * //Gets or sets the logBase of primaryXAxis, after initialization:<br>
      *	//Gets the logBase of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.logBase");<br>			
      *	//Sets the logBase of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{logBase:5}} ); 
       * @alias ejChart#primaryXAxis->logBase
      * @instance
      */
         logBase: 10,
         /**		
      * Specifies the plotOffset of primaryXAxis. This specifies the offset.
      * @default 10
      * @type {number}
      * @example 
      * //To set plotOffset of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{plotOffset:0}});				
      * @example 
      * //Gets or sets the plotOffset of primaryXAxis, after initialization:<br>
      *	//Gets the plotOffset of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.plotOffset");<br>			
      *	//Sets the plotOffset of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{plotOffset:0}} ); 
       * @alias ejChart#primaryXAxis->plotOffset
      * @instance
      */
         plotOffset: 0,
         labels: [],
         stripline: [],
         /**		
    * Specifies the title. This is to be rendered as title for primaryXAxis.
    * @type {object}
      * @alias ejChart#primaryXAxis->title
    * @instance
    */
         title:
 {

     /**		
   * Specifies the title text of primaryXAxis. This is the text which is to be rendered as title for primaryXAxis.
   * @default ""
   * @type {string}
   * @example 
   * //To set title text of primaryXAxis
   * 	$("#container").ejChart({primaryXAxis:{ title:{ text:"Power production"}}});				
   * @example 
   * //Gets or sets the title text of primaryXAxis, after initialization:<br>
   *	//Gets the title text of primaryXAxis  
   *	$("#container").ejChart("option", "primaryXAxis.title.text");<br>			
   *	//Sets the title text of primaryXAxis
   *	$("#container").ejChart("option", {primaryXAxis:{ title:{text:"Power production"}}} ); 
     * @alias ejChart#primaryXAxis->title->text
   * @instance
   */
     text: "",
     /**		
* Specifies the font. These font properties are customization are applied for title text. 
* @type {object}
 * @alias ejChart#primaryXAxis->title->font
* @instance
*/
     font:
{
    /**		
  * Specifies the title fontfamily of primaryXAxis. This fontFamily is applied to the title text of primaryXAxis.
  * @default "Segoe UI"
  * @type {string}
  * @example 
  * //To set title fontfamily of primaryXAxis
  * 	$("#container").ejChart({primaryXAxis:{ title:{ font:{fontFamily:"Algerian"}}}});			 
  * @example 
  * //Gets or sets the title fontfamily of primaryXAxis, after initialization:<br>
  *	//Gets the title fontfamily of primaryXAxis  
  *	$("#container").ejChart("option", "primaryXAxis.title.font.fontFamily");<br>			
  *	//Sets the title fontfamily of primaryXAxis
  *	$("#container").ejChart("option", {primaryXAxis:{ title:{font:{fontFamily:"Algerian"}}}} ); 
  * @alias ejChart#primaryXAxis->title->font->fontFamily
  * @instance
  */
    fontFamily: 'Segoe UI',
    /**		
    * Specifies the title fontStyle of primaryXAxis. This fontStyle is applied to the title text of primaryXAxis.
    * @default ej.Chart.fontStyle.Normal
    * @type {enum}
    * @example 
    * //To set title fontStyle of primaryXAxis
    * 	$("#container").ejChart({primaryXAxis:{ title:{ font:{fontStyle:"Italic"}}}});				 
    * @example 
    * //Gets or sets the title fontStyle of primaryXAxis, after initialization:<br>
    *	//Gets the title fontStyle of primaryXAxis  
    *	$("#container").ejChart("option", "primaryXAxis.title.font.fontStyle");<br>			
    *	//Sets the title fontStyle of primaryXAxis
    *	$("#container").ejChart("option", {primaryXAxis:{ title:{font:{fontStyle:"Italic"}}}} ); 
    * @alias ejChart#primaryXAxis->title->font->fontStyle
    * @instance
    */
    fontStyle: 'Normal',
    /**		
 * Specifies the title size of primaryXAxis. This font size is applied to the title text of primaryXAxis.
 * @default "16px"
 * @type {string}
 * @example 
 * //To set title size of primaryXAxis
 * 	$("#container").ejChart({primaryXAxis:{ title:{ font:{size:"13px"}}}});				
 * @example 
 * //Gets or sets the title size of primaryXAxis, after initialization:<br>
 *	//Gets the title size of primaryXAxis  
 *	$("#container").ejChart("option", "primaryXAxis.title.font.size");<br>			
 *	//Sets the title size of primaryXAxis
 *	$("#container").ejChart("option", {primaryXAxis:{ title:{font:{size:"13px"}}}} ); 
  * @alias ejChart#primaryXAxis->title->font->size
 * @instance
 */
    size: '16px',
    /**		
 * Specifies the title opacity of primaryXAxis. This opacity value is applied to the title text of primaryXAxis.
 * @default 1
 * @type {number}
 * @example 
 * //To set title opacity of primaryXAxis
 * 	$("#container").ejChart({primaryXAxis:{ title:{ font:{opacity:0.5}}}});				
 * @example 
 * //Gets or sets the title opacity of primaryXAxis, after initialization:<br>
 *	//Gets the title opacity of primaryXAxis  
 *	$("#container").ejChart("option", "primaryXAxis.title.font.opacity");<br>			
 *	//Sets the title opacity of primaryXAxis
 *	$("#container").ejChart("option", {primaryXAxis:{ title:{font:{opacity:0.5}}}} ); 
  * @alias ejChart#primaryXAxis->title->font->opacity
 * @instance
 */
    opacity: 1,
    /**		
 * Specifies the title fontWeight of primaryXAxis. This fontWeight is applied to the title text of primaryXAxis. See {@link fontWeight}
 * @default ej.Chart.fontWeight.Regular
 * @type {enum}
 * @example 
 * //To set title fontWeight of primaryXAxis
 * 	$("#container").ejChart({primaryXAxis:{ title:{ font:{fontWeight:"lighter"}}}});				 
 * @example 
 * //Gets or sets the title fontWeight of primaryXAxis, after initialization:<br>
 *	//Gets the title fontWeight of primaryXAxis  
 *	$("#container").ejChart("option", "primaryXAxis.title.font.fontWeight");<br>			
 *	//Sets the title fontWeight of primaryXAxis
 *	$("#container").ejChart("option", {primaryXAxis:{ title:{font:{fontWeight:"lighter"}}}} ); 
  * * @alias ejChart#primaryXAxis->title->font->fontWeight
 * @instance
 */
    fontWeight: 'regular'
}
 },

         /**		
      * Specifies the rangePadding of primaryXAxis. Specify the spacing of the chart axis for auto range calculation
None: Range of axis is calculated from the minimum value to the maximum value in datasource
Normal: Range of axis is calculated based on range calculation,(for numerical axis only)
Additional: Range of axis is calculated by adding interval to minimum and  maximum value in datasource.
Round: Round the range of axis to the nearest possible value. See {@link rangePadding}
      * @default ej.Chart.rangePaddding.None
      * @type {enum}
      * @example 
      * //To set rangePadding of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{rangePadding:"Normal"}});				
      * @example 
      * //Gets or sets the rangePadding of primaryXAxis, after initialization:<br>
      *	//Gets the rangePadding of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.rangePadding");<br>			
      *	//Sets the rangePadding of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{rangePadding:"Normal"}} ); 
       * @alias ejChart#primaryXAxis->rangePadding
      * @instance
      */
         rangePadding: 'None',
         additionalPadding: [1, 1],
         /**		
      * Specifies the orientation of primaryXAxis. This is to set the orientation for chart axis. The user can set either vertical or horizontal. See {@link orientation}
      * @default ej.Chart.orientation.Horizontal
      * @type {enum}
      * @example 
      * //To set orientation of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{orientation:"Vertical"}});				
      * @example 
      * //Gets or sets the orientation of primaryXAxis, after initialization:<br>
      *	//Gets the orientation of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.orientation");<br>			
      *	//Sets the orientation of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{orientation:"Vertical"}} ); 
       * @alias ejChart#primaryXAxis->orientation
      * @instance
      */
         orientation: 'Horizontal',
         /**		
      * Specifies the maximumLabels of primaryXAxis. This specifies the maximum number of labels to be rendered on the axis.
      * @default 3
      * @type {number}
      * @example 
      * //To set maximumLabels of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{maximumLabels:2}});				
      * @example 
      * //Gets or sets the maximumLabels of primaryXAxis, after initialization:<br>
      *	//Gets the maximumLabels of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.maximumLabels");<br>			
      *	//Sets the maximumLabels of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{maximumLabels:2}} ); 
      * @alias ejChart#primaryXAxis->maximumLabels
      * @instance
      */
         maximumLabels: 3,
         /**		
     * Specifies the opposedPosition of primaryXAxis. This is to render the axis in opposed position or not. 
     * @default false
     * @type {boolean}
     * @example 
     * //To set opposedPosition of primaryXAxis
     * 	$("#container").ejChart({primaryXAxis:{opposedPosition:true}});				
     * @example 
     * //Gets or sets the opposedPosition of primaryXAxis, after initialization:<br>
     *	//Gets the opposedPosition of primaryXAxis  
     *	$("#container").ejChart("option", "primaryXAxis.opposedPosition");<br>			
     *	//Sets the opposedPosition of primaryXAxis
     *	$("#container").ejChart("option", {primaryXAxis:{opposedPosition:true}} ); 
      * @alias ejChart#primaryXAxis->opposedPosition
     * @instance
     */
         opposedPosition: false,
         /**		
       * Specifies the axisLine.This line is drawn in chart to represent the end of the axis. 
       * @type {object}
       * @alias ejChart#primaryXAxis->axisLine
       * @instance
       */
         axisLine:
  {
      /**		
     * Specifies the visibility of axisLine in primaryXAxis. The user can change the visibility of the axisLine.
     * @default true
     * @type {boolean}
     * @example 
     * //To set visibility of axisLine in primaryXAxis
     * 	$("#container").ejChart({primaryXAxis:{axisLine:{visible:false}}});				
     * @example 
     * //Gets or sets the visibility of axisLine in primaryXAxis, after initialization:<br>
     *	//Gets the visibility of axisLine in primaryXAxis  
     *	$("#container").ejChart("option", "primaryXAxis.axisLine.visible");<br>			
     *	//Sets the visibility of axisLine in primaryXAxis
     *	$("#container").ejChart("option", {primaryXAxis:{axisLine:{visible:false}}} ); 
     * @alias ejChart#primaryXAxis->axisLine->visible
     * @instance
     */
      visible: true,
      /**		
     * Specifies the width of axisLine in primaryXAxis. The user can change the width of the axisLine.
     * @default 1
     * @type {number}
     * @example 
     * //To set width of axisLine in primaryXAxis
     * 	$("#container").ejChart({primaryXAxis:{axisLine:{width:0.5}}});				
     * @example 
     * //Gets or sets the width of axisLine in primaryXAxis, after initialization:<br>
     *	//Gets the width of axisLine in primaryXAxis  
     *	$("#container").ejChart("option", "primaryXAxis.axisLine.width");<br>			
     *	//Sets the width of axisLine in primaryXAxis
     *	$("#container").ejChart("option", {primaryXAxis:{axisLine:{width:0.5}}} ); 
     * @alias ejChart#primaryXAxis->axisLine->width
     * @instance
     */
      width: 1,
      /**		
 * Specifies the dashArray of axisLine in primaryXAxis. The user can change the dashArray of the axisLine.
 * @default null
 * @type {string}
 * @example 
 * //To set dashArray of axisLine in primaryXAxis
 * 	$("#container").ejChart({primaryXAxis:{axisLine:{dashArray:"2,3"}}});				
 * @example 
 * //Gets or sets the dashArray of axisLine in primaryXAxis, after initialization:<br>
 *	//Gets the dashArray of axisLine in primaryXAxis  
 *	$("#container").ejChart("option", "primaryXAxis.axisLine.dashArray");<br>			
 *	//Sets the dashArray of axisLine in primaryXAxis
 *	$("#container").ejChart("option", {primaryXAxis:{axisLine:{dashArray:"2,3"}}} ); 
  * @alias ejChart#primaryXAxis->axisLine->dashArray
 * @instance
 */
      dashArray: "",
      /**		
* Specifies the offset of axisLine in primaryXAxis. The user can change the offset values of the axisLine.
* @default null
* @type {number}
* @example 
* //To set offset of axisLine in primaryXAxis
* 	$("#container").ejChart({primaryXAxis:{axisLine:{offset:1}}});				
* @example 
* //Gets or sets the offset of axisLine in primaryXAxis, after initialization:<br>
*	//Gets the offset of axisLine in primaryXAxis  
*	$("#container").ejChart("option", "primaryXAxis.axisLine.offset");<br>			
*	//Sets the offset of axisLine in primaryXAxis
*	$("#container").ejChart("option", {primaryXAxis:{axisLine:{offset:1}}} ); 
 * @alias ejChart#primaryXAxis->axisLine->offset
* @instance
*/
      offset: 0
  },
         /**		
      * Specifies the hidePartialLabels of primaryXAxis. By applying this property, the user can hide the partially visible labels in the axis.
      * @default false
      * @type {boolean}
      * @example 
      * //To set hidePartialLabels of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{hidePartialLabels:true}});			 
      * @example 
      * //Gets or sets the hidePartialLabels of primaryXAxis, after initialization:<br>
      *	//Gets the hidePartialLabels of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.hidePartialLabels");<br>			
      *	//Sets the hidePartialLabels of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{hidePartialLabels:true}} ); 
       * @alias ejChart#primaryXAxis->hidePartialLabels
      * @instance
      */
         hidePartialLabels: false,
         /**		
    * Specifies the font of primaryXAxis. These font properties are applied for the axis labels.
    * @type {object}
    * @alias ejChart#primaryXAxis->font
    * @instance
    */
         font:
  {
      /**		
      * Specifies the fontfamily of primaryXAxis. The axis labels render with this specified fontFamily.
      * @default "Segoe UI"
      * @type {string}
      * @example 
      * //To set fontfamily of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{ font:{fontFamily:"Algerian"}}});				
      * @example 
      * //Gets or sets the fontfamily of primaryXAxis, after initialization:<br>
      *	//Gets the fontfamily of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.font.fontFamily");<br>			
      *	//Sets the fontfamily of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{ font:{fontFamily:"Algerian"}}} ); 
       * @alias ejChart#primaryXAxis->font->fontFamily
      * @instance
      */
      fontFamily: 'Segoe UI',
      /**		
     * Specifies the fontStyle of primaryXAxis. The axis labels render with this specified fontStyle. See {@link fontStyle}
     * @default ej.Chart.fontStyle.Normal
     * @type {enum}
     * @example 
     * //To set fontStyle of primaryXAxis
     * 	$("#container").ejChart({primaryXAxis:{ font:{fontStyle:"Italic"}}});				
     * @example 
     * //Gets or sets the fontStyle of primaryXAxis, after initialization:<br>
     *	//Gets the fontStyle of primaryXAxis  
     *	$("#container").ejChart("option", "primaryXAxis.font.fontStyle");<br>			
     *	//Sets the fontStyle of primaryXAxis
     *	$("#container").ejChart("option", {primaryXAxis:{font:{fontStyle:"Italic"}}} ); 
     * @alias ejChart#primaryXAxis->font->fontStyle
     * @instance
     */
      fontStyle: 'Normal',
      /**		
      * Specifies the title size of primaryXAxis. The axis labels render with this specified size.
      * @default "13px"
      * @type {string}
      * @example 
      * //To set size of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{font:{size:"11px"}}});				
      * @example 
      * //Gets or sets the size of primaryXAxis, after initialization:<br>
      *	//Gets the size of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.font.size");<br>			
      *	//Sets the size of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{font:{size:"11px"}}} ); 
      * @alias ejChart#primaryXAxis->font->size
      * @instance
      */
      size: '13px',
      /**		
      * Specifies the fontWeight of primaryXAxis. The axis labels render with this specified fontWeight.See {@link fontWeight}
      * @default ej.Chart.fontWeight.Regular
      * @type {enum}
      * @example 
      * //To set fontWeight of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{font:{fontWeight:"lighter"}}});				
      * @example 
      * //Gets or sets the fontWeight of primaryXAxis, after initialization:<br>
      *	//Gets the fontWeight of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.font.fontWeight");<br>			
      *	//Sets the fontWeight of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{font:{fontWeight:"lighter"}}} ); 
      * @alias ejChart#primaryXAxis->font->fontWeight
      * @instance
      */
      fontWeight: 'regular',
      /**		
     * Specifies the title opacity of primaryXAxis. The axis labels render with this specified opacity.
     * @default 1
     * @type {number}
     * @example 
     * //To set title opacity of primaryXAxis
     * 	$("#container").ejChart({primaryXAxis:{ title:{ font:{opacity:0.5}}}});				
     * @example 
     * //Gets or sets the title opacity of primaryXAxis, after initialization:<br>
     *	//Gets the title opacity of primaryXAxis  
     *	$("#container").ejChart("option", "primaryXAxis.title.font.opacity");<br>			
     *	//Sets the title opacity of primaryXAxis
     *	$("#container").ejChart("option", {primaryXAxis:{ title:{font:{opacity:0.5}}}} ); 
     * @alias ejChart#primaryXAxis->font->opacity
     * @instance
     */
      opacity: 1
  },
         /**		
      * Specifies the visibility of primaryXAxis. The user can change the visibility of the axis labels.
      * @default true
      * @type {boolean}
      * @example 
      * //To set visibility of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{visible:false}});				
      * @example 
      * //Gets or sets the visibility of primaryXAxis, after initialization:<br>
      *	//Gets the visibility of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.visible");<br>			
      *	//Sets the visibility of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{visible:false}} ); 
      * @alias ejChart#primaryXAxis->visible
      * @instance
      */
         visible: true,
         /**		
    * Specifies the crosshairLabel of primaryXAxis. These are the properties relate to crosshair Label. 
    * @type {object}
     * @alias ejChart#primaryXAxis->crosshairLabel
    * @instance
    */
         crosshairLabel:
   {
       /**		
      * Specifies the visibility of crosshairLabel. This is the property to change the visibility of the axis labels. 
      * @default false
      * @type {boolean}
      * @example 
      * //To set visibility of crosshairLabel
      * 	$("#container").ejChart({primaryXAxis:{crosshairLabel:{visible:true}}});				
      * @example 
      * //Gets or sets the visibility of crosshairLabel, after initialization:<br>
      *	//Gets the visibility of crosshairLabel  
      *	$("#container").ejChart("option", "primaryXAxis.crosshairLabel.visible");<br>			
      *	//Sets the visibility of crosshairLabel
      *	$("#container").ejChart("option", {primaryXAxis:{crosshairLabel:{visible:true}}} ); 
       * @alias ejChart#primaryXAxis->crosshairLabel->visible
      * @instance
      */
       visible: false
   },
         /**		
       * Specifies the zoomFactor of primaryXAxis. This is to specify zoom factor value for axis zooming. The user can specify value from 0 to 1.
       * @default 1
       * @type {number}
       * @example 
       * //To set zoomFactor of primaryXAxis
       * 	$("#container").ejChart({primaryXAxis:{zoomFactor:0}});				
       * @example 
       * //Gets or sets the zoomFactor of primaryXAxis, after initialization:<br>
       *	//Gets the zoomFactor of primaryXAxis  
       *	$("#container").ejChart("option", "primaryXAxis.zoomFactor");<br>			
       *	//Sets the zoomFactor of primaryXAxis
       *	$("#container").ejChart("option", {primaryXAxis:{zoomFactor:0}} ); 
       * @alias ejChart#primaryXAxis->zoomFactor
       * @instance
       */
         zoomFactor: 1,
         /**		
      * Specifies the zoomPosition of primaryXAxis. To specify zoom position value for axis zooming. The user can specify value from 0 to 1.
      * @default 0
      * @type {number}
      * @example 
      * //To set zoomPosition of primaryXAxis
      * 	$("#container").ejChart({primaryXAxis:{zoomPosition:1}});				
      * @example 
      * //Gets or sets the zoomPosition of primaryXAxis, after initialization:<br>
      *	//Gets the zoomPosition of primaryXAxis  
      *	$("#container").ejChart("option", "primaryXAxis.zoomPosition");<br>			
      *	//Sets the zoomPosition of primaryXAxis
      *	$("#container").ejChart("option", {primaryXAxis:{zoomPosition:1}} ); 
     * @alias ejChart#primaryXAxis->zoomPosition
      * @instance
      */
         zoomPosition: 0
     },

            /**		
           * Specifies the primaryYAxis. This includes the properties relate to the primary y axis.
           * @type {object}
            * @memberof ejChart
           * @instance
           */
            primaryYAxis:
      {
          /**		
    * Specifies the majorGridLines. These are properties involved in customizing the majorGridLines of chart. 
    * @type {object}
     * @alias ejChart#primaryYAxis->majorGridLines
    * @instance
    */
          majorGridLines:
     {
         /**		
* Specifies the width of the majorGridLines. The user can modify the width of the majorGridLines in primaryYAxis.
* @default 1
* @type {number}
* @example 
* //To set width of majorGridLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{majorGridLines:{width:2}}});				
* @example 
* //Gets or sets the width of the majorGridLines, after initialization:<br>
*	//Gets the width of the majorGridLines  
*	$("#container").ejChart("option", "primaryYAxis.majorGidLines.width");<br>			
*	//Sets the width of the majorGridLines
*	$("#container").ejChart("option", {primaryYAxis:{majorGridLines:{width:2}}} ); 
* @alias ejChart#primaryYAxis->majorGridLines->width
* @instance
*/
         width: 1,
         /**		
* Specifies the dashArray of the majorGridLines. The user can modify the dashArray of the majorGridLines in primaryYAxis.
* @default null
* @type {string}
* @example 
* //To set dashArray of majorGridLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{majorGridLines:{dashArray:"2,3"}}});				
* @example 
* //Gets or sets the dashArray of the majorGridLines, after initialization:<br>
*	//Gets the dashArray of the majorGridLines  
*	$("#container").ejChart("option", "primaryYAxis.majorGidLines.dashArray");<br>			
*	//Sets the dashArray of the majorGridLines
*	$("#container").ejChart("option", {primaryYAxis:{majorGridLines:{dashArray:"2,3"}}} ); 
* @alias ejChart#primaryYAxis->majorGridLines->dashArray
* @instance
*/
         dashArray: "",
         /**		
* Specifies the visibility of the majorGridLines. The user can modify the visibility of the majorGridLines in primaryYAxis.
* @default true
* @type {boolean}
* @example 
* //To set visibility of majorGridLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{majorGridLines:{visible:false}}});				
* @example 
* //Gets or sets the visibility of the majorGridLines, after initialization:<br>
*	//Gets the visibility of the majorGridLines  
*	$("#container").ejChart("option", "primaryYAxis.majorGidLines.visible");<br>			
*	//Sets the visibility of the majorGridLines
*	$("#container").ejChart("option", {primaryYAxis:{majorGridLines:{visible:false}}} ); 
* @alias ejChart#primaryYAxis->majorGridLines->visible
* @instance
*/
         visible: true,
         /**		
* Specifies the opacity of the majorGridLines. The user can modify the opacity of the majorGridLines in primaryYAxis.
* @default 1
* @type {number}
* @example 
* //To set opacity of majorGridLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{majorGridLines:{opacity:0.5}}});				
* @example 
* //Gets or sets the opacity of the majorGridLines, after initialization:<br>
*	//Gets the opacity of the majorGridLines  
*	$("#container").ejChart("option", "primaryYAxis.majorGidLines.opacity");<br>			
*	//Sets the opacity of the majorGridLines
*	$("#container").ejChart("option", {primaryYAxis:{majorGridLines:{opacity:0.5}}} ); 
* @alias ejChart#primaryYAxis->majorGridLines->opacity
* @instance
*/
         opacity: 1

     },
          /**		
* Specifies the majorTickLines.These are properties involved in customizing the majorTickLines of chart.
* @type {object}
* @alias ejChart#primaryYAxis->majorTickLines
* @instance
*/
          majorTickLines:
     {
         /**		
* Specifies the width of the majorTickLines. The user can modify the width of the majorTickLines in primaryYAxis.
* @default 1
* @type {number}
* @example 
* //To set width of majorTickLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{majorTickLines:{width:2}}});				
* @example 
* //Gets or sets the width of the majorTickLines, after initialization:<br>
*	//Gets the width of the majorTickLines  
*	$("#container").ejChart("option", "primaryYAxis.majorTickLines.width");<br>			
*	//Sets the width of the majorTickLines
*	$("#container").ejChart("option", {primaryYAxis:{majorTickLines:{width:2}}} ); 
* @alias ejChart#primaryYAxis->majorTickLines->width
* @instance
*/
         width: 1,
         /**		
* Specifies the size of the majorTickLines. The user can modify the size of the majorTickLines in primaryYAxis.
* @default 5
* @type {number}
* @example 
* //To set size of majorTickLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{majorTickLines:{size:3}}});				
* @example 
* //Gets or sets the size of the majorTickLines, after initialization:<br>
*	//Gets the size of the majorTickLines  
*	$("#container").ejChart("option", "primaryYAxis.majorTickLines.size");<br>			
*	//Sets the size of the majorTickLines
*	$("#container").ejChart("option", {primaryYAxis:{majorTickLines:{size:3}}} ); 
* @alias ejChart#primaryYAxis->majorTickLines->size
* @instance
*/
         size: 5,
         /**		
* Specifies the visibility of the majorTickLines. The user can modify the visibility of the majorTickLines in primaryYAxis.
* @default true
* @type {boolean}
* @example 
* //To set visibility of majorTickLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{majorTickLines:{visible:false}}});				
* @example 
* //Gets or sets the visibility of the majorTickLines, after initialization:<br>
*	//Gets the visibility of the majorTickLines  
*	$("#container").ejChart("option", "primaryYAxis.majorTickLines.visible");<br>			
*	//Sets the visibility of the majorTickLines
*	$("#container").ejChart("option", {primaryYAxis:{majorTickLines:{visible:false}}} ); 
* @alias ejChart#primaryYAxis->majorTickLines->visible
* @instance
*/
         visible: true
     },
          /**		
     * Specifies the minorGridLines. These are properties involved in customizing the minorGridLines of chart. 
     * @type {object}
  * @alias ejChart#primaryYAxis->minorGridLines
     * @instance
     */
          minorGridLines:
     {
         /**		
* Specifies the width of the minorGridLines. The user can modify the width of the minorGridLines in primaryYAxis.
* @default 1
* @type {number}
* @example 
* //To set width of minorGridLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{minorGridLines:{width:2}}});				
* @example 
* //Gets or sets the width of the minorGridLines, after initialization:<br>
*	//Gets the width of the minorGridLines  
*	$("#container").ejChart("option", "primaryYAxis.minorGridLines.width");<br>			
*	//Sets the width of the minorGridLines
*	$("#container").ejChart("option", {primaryYAxis:{minorGridLines:{width:2}}} ); 
* @alias ejChart#primaryYAxis->minorGridLines->width
* @instance
*/
         width: 1,
         /**		
* Specifies the dashArray of the minorGridLines. The user can modify the dashArray of the minorGridLines in primaryYAxis.
* @default null
* @type {string}
* @example 
* //To set dashArray of minorGridLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{minorGridLines:{dashArray:"2,3"}}});				
* @example 
* //Gets or sets the dashArray of the minorGridLines, after initialization:<br>
*	//Gets the dashArray of the minorGridLines  
*	$("#container").ejChart("option", "primaryYAxis.minorGridLines.dashArray");<br>			
*	//Sets the dashArray of the minorGridLines
*	$("#container").ejChart("option", {primaryYAxis:{minorGridLines:{dashArray:"2,3"}}} ); 
* @alias ejChart#primaryYAxis->minorGridLines->dashArray
* @instance
*/
         dashArray: "",
         /**		
* Specifies the visibility of the minorGridLines. The user can modify the visibility of the minorGridLines in primaryYAxis.
* @default false
* @type {boolean}
* @example 
* //To set visibility of minorGridLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{minorGridLines:{visible:true}}});				
* @example 
* //Gets or sets the visibility of the minorGridLines, after initialization:<br>
*	//Gets the visibility of the minorGridLines  
*	$("#container").ejChart("option", "primaryYAxis.minorGridLines.visible");<br>			
*	//Sets the visibility of the minorGridLines
*	$("#container").ejChart("option", {primaryYAxis:{minorGridLines:{visible:true}}} ); 
* @alias ejChart#primaryYAxis->minorGridLines->visible
* @instance
*/
         visible: false

     },
          /**		
* Specifies the minorTickLines. These are properties involved in customizing the minorTickLines of chart. 
* @type {object}
* @alias ejChart#primaryYAxis->minorTickLines
* @instance
*/
          minorTickLines:
     {
         /**		
* Specifies the width of the minorTickLines. The user can modify the width of the minorTickLines in primaryYAxis.
* @default 1
* @type {number}
* @example 
* //To set width of minorTickLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{minorTickLines:{width:2}}});				
* @example 
* //Gets or sets the width of the minorTickLines, after initialization:<br>
*	//Gets the width of the minorTickLines  
*	$("#container").ejChart("option", "primaryYAxis.minorTickLines.width");<br>			
*	//Sets the width of the minorTickLines
*	$("#container").ejChart("option", {primaryYAxis:{minorTickLines:{width:2}}} ); 
* @alias ejChart#primaryYAxis->minorTickLines->width
* @instance
*/
         width: 1,
         /**		
* Specifies the size of the minorTickLines.  The user can modify the size of the minorTickLines in primaryYAxis.
* @default 5
* @type {number}
* @example 
* //To set size of minorTickLines value during initialization  
* 	$("#container").ejChart({primaryYAxis:{minorTickLines:{size:3}}});				
* @example 
* //Gets or sets the size of the minorTickLines, after initialization:<br>
*	//Gets the size of the minorTickLines  
*	$("#container").ejChart("option", "primaryYAxis.minorTickLines.size");<br>			
*	//Sets the size of the minorTickLines
*	$("#container").ejChart("option", {primaryYAxis:{minorTickLines:{size:3}}} ); 
* @alias ejChart#primaryYAxis->minorTickLines->size
* @instance
*/
         size: 5,
         /**		
* Specifies the visibility of the minorTickLines.  The user can modify the visibility of the minorTickLines in primaryYAxis.
* @default false
* @type {boolean}
* @example 
* //To set visibility of minorTickLines value during initialization 
* 	$("#container").ejChart({primaryYAxis:{minorTickLines:{visible:true}}});				
* @example 
* //Gets or sets the visibility of the minorTickLines, after initialization:<br>
*	//Gets the visibility of the minorTickLines  
*	$("#container").ejChart("option", "primaryYAxis.minorTickLines.visible");<br>			
*	//Sets the visibility of the minorTickLines
*	$("#container").ejChart("option", {primaryYAxis:{minorTickLines:{visible:true}}} ); 
* @alias ejChart#primaryYAxis->minorTickLines->visible
* @instance
*/
         visible: false
     },
          /**		
* Specifies the minorTicksPerInterval of primaryYAxis. This property is to set the number of minor ticks to render per interval
* @default null
* @type {number}
* @example 
* //To set minorTicksPerInterval of primaryYAxis value during initialization  
* 	$("#container").ejChart({primaryYAxis:{minorTicksPerInterval:3}});				
* @example 
* //Gets or sets the minorTicksPerInterval of primaryYAxis, after initialization:<br>
*	//Gets the minorTicksPerInterval of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.minorTicksPerInterval");<br>			
*	//Sets the minorTicksPerInterval of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{minorTicksPerInterval:3}} ); 
* @alias ejChart#primaryYAxis->minorTicksPerInterval
* @instance
*/
          minorTicksPerInterval: null,
          /**		
* Specifies the rowIndex of primaryYAxis. This is to specify the index of the row.
* @default null
* @type {number}
* @example 
* //To set rowIndex of primaryYAxis value during initialization  
* 	$("#container").ejChart({primaryYAxis:{rowIndex:3}});				
* @example 
* //Gets or sets the rowIndex of primaryYAxis, after initialization:<br>
*	//Gets the rowIndex of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.rowIndex");<br>			
*	//Sets the rowIndex of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{rowIndex:3}} ); 
* @alias ejChart#primaryYAxis->rowIndex
* @instance
*/
          rowIndex: null,
          /**		
 * Specifies the rowSpan of primaryYAxis. EjChart provides option to span the row. This is achieved by using this property.
 * @default null
 * @type {number}
 * @example 
 * //To set rowSpan of primaryYAxis value during initialization  
 * 	$("#container").ejChart({primaryYAxis:{rowSpan:2}});				
 * @example 
 * //Gets or sets the rowSpan of primaryYAxis, after initialization:<br>
 *	//Gets the rowSpan of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.rowSpan");<br>			
 *	//Sets the rowSpan of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{rowSpan:2}} ); 
* @alias ejChart#primaryYAxis->rowSpan
 * @instance
 */
          rowSpan: null,
          /**		
* Specifies the valueType of primaryYAxis. It denotes the type of axis. It can be double, datetime, etc.
* @default null
* @type {enum}
* @example 
* //To set valueType of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{valueType:"Double"}});				
* @example 
* //Gets or sets the valueType of primaryYAxis, after initialization:<br>
*	//Gets the valueType of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.valueType");<br>			
*	//Sets the valueType of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{valueType:"Double"}} ); 
* @alias ejChart#primaryYAxis->valueType
* @instance
*/
          valueType: null,
          /**		
  * Specifies the axisName of primaryYAxis. This is the name to represent the axis in chart.
  * @default null
  * @type {string}
  * @example 
  * //To set axisName of primaryYAxis
  * 	$("#container").ejChart({primaryYAxis:{axisName:"yaxis"}});				
  * @example 
  * //Gets or sets the axisName of primaryYAxis, after initialization:<br>
  *	//Gets the axisName of primaryYAxis  
  *	$("#container").ejChart("option", "primaryYAxis.axisName");<br>			
  *	//Sets the axisName of primaryYAxis
  *	$("#container").ejChart("option", {primaryYAxis:{axisName:"yaxis"}} ); 
* @alias ejChart#primaryYAxis->axisName
  * @instance
  */
          axisName: null,
          /**		
 * Specifies the labelFormat of primaryYAxis. The axis labels render in the format, which is specified here.
 * @default null
 * @type {string}
 * @example 
 * //To set labelFormat of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{labelFormat:"${value}"}});				
 * @example 
 * //Gets or sets the labelFormat of primaryYAxis, after initialization:<br>
 *	//Gets the labelFormat of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.labelFormat");<br>			
 *	//Sets the labelFormat of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{labelFormat:"${value}"}} ); 
 * @alias ejChart#primaryYAxis->labelFormat
 * @instance
 */
          labelFormat: null,
          /**		
* Specifies the desiredIntervals of primaryYAxis. By this property we can display the labels after appropriate intervals.
* @default null
* @type {number}
* @example 
* //To set desiredIntervals of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{desiredIntervals:2}});				
* @example 
* //Gets or sets the desiredIntervals of primaryYAxis, after initialization:<br>
*	//Gets the desiredIntervals of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.desiredIntervals");<br>			
*	//Sets the desiredIntervals of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{desiredIntervals:2}} ); 
 * @alias ejChart#primaryYAxis->desiredIntervals
* @instance
*/
          desiredIntervals: null,
          /**		
 * Specifies the intervalType of primaryYAxis. This is to set the type of intervals to chart. IntervalType may be days, years, etc. See {@link intervalType}
 * @default null
 * @type {enum}
 * @example 
 * //To set desiredIntervintervalTypeals of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{intervalType:"days"}});				
 * @example 
 * //Gets or sets the intervalType of primaryYAxis, after initialization:<br>
 *	//Gets the intervalType of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.intervalType");<br>			
 *	//Sets the intervalType of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{intervalType:"days"}} ); 
  * @alias ejChart#primaryYAxis->intervalType
 * @instance
 */
          intervalType: null,
          /**		
 * Specifies the roundingPlaces of primaryYAxis. This property is to truncate the decimal values after the specified position.
 * @default null
 * @type {number}
 * @example 
 * //To set roundingPlaces of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{roundingPlaces:2}});				
 * @example 
 * //Gets or sets the roundingPlaces of primaryYAxis, after initialization:<br>
 *	//Gets the roundingPlaces of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.roundingPlaces");<br>			
 *	//Sets the roundingPlaces of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{roundingPlaces:2}} ); 
  * @alias ejChart#primaryYAxis->roundingPlaces
 * @instance
 */
          roundingPlaces: null,
          labels: [],
          /**		
* Specifies the title.This is to be rendered as title for primaryYAxis. 
* @type {object}
* @alias ejChart#primaryYAxis->title
* @instance
*/
          title:
    {
        /**		
* Specifies the title text of primaryYAxis.  This is the text which is to be rendered as title for primaryYAxis.
* @default ""
* @type {string}
* @example 
* //To set title text of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ title:{ text:"Sales report"}}});				
* @example 
* //Gets or sets the title text of primaryYAxis, after initialization:<br>
*	//Gets the title text of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.title.text");<br>			
*	//Sets the title text of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{ title:{text:"Sales report"}}} ); 
* @alias ejChart#primaryYAxis->title->text
* @instance
*/
        text: "",
        /**		
* Specifies the font. These font properties are customization are applied for title text.
* @type {object}
* @alias ejChart#primaryYAxis->title->font
* @instance
*/
        font:
     {
         /**		
* Specifies the title fontfamily of primaryYAxis. This fontFamily is applied to the title text of primaryYAxis.
* @default "Segoe UI"
* @type {string}
* @example 
* //To set title fontfamily of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ title:{ font:{fontFamily:"Algerian"}}}});				
* @example 
* //Gets or sets the title fontfamily of primaryYAxis, after initialization:<br>
*	//Gets the title fontfamily of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.title.font.fontFamily");<br>			
*	//Sets the title fontfamily of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{ title:{font:{fontFamily:"Algerian"}}}} ); 
* @alias ejChart#primaryYAxis->title->font->fontFamily
* @instance
*/
         fontFamily: 'Segoe UI',
         /**		
* Specifies the title fontStyle of primaryYAxis. This fontStyle is applied to the title text of primaryYAxis. See {@link fontStyle}
* @default ej.Chart.fontStyle.Normal
* @type {enum}
* @example 
* //To set title fontStyle of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ title:{ font:{fontStyle:"Italic"}}}});				 
* @example 
* //Gets or sets the title fontStyle of primaryYAxis, after initialization:<br>
*	//Gets the title fontStyle of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.title.font.fontStyle");<br>			
*	//Sets the title fontStyle of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{ title:{font:{fontStyle:"Italic"}}}} ); 
* @alias ejChart#primaryYAxis->title->font->fontStyle
* @instance
*/
         fontStyle: 'Normal',
         /**		
* Specifies the title opacity of primaryYAxis. This font size is applied to the title text of primaryYAxis.
* @default 1
* @type {number}
* @example 
* //To set title opacity of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ title:{ font:{opacity:0.5}}}});				 
* @example 
* //Gets or sets the title opacity of primaryYAxis, after initialization:<br>
*	//Gets the title opacity of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.title.font.opacity");<br>			
*	//Sets the title opacity of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{ title:{font:{opacity:0.5}}}} ); 
* @alias ejChart#primaryYAxis->title->font->opacity
* @instance
*/
         opacity: 1,
         /**		
* Specifies the title size of primaryYAxis. This opacity value is applied to the title text of primaryYAxis.
* @default "16px"
* @type {string}
* @example 
* //To set title size of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ title:{ font:{size:"13px"}}}});				 
* @example 
* //Gets or sets the title size of primaryYAxis, after initialization:<br>
*	//Gets the title size of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.title.font.size");<br>			
*	//Sets the title size of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{ title:{font:{size:"13px"}}}} ); 
* @alias ejChart#primaryYAxis->title->font->size
* @instance
*/
         size: '16px',
         /**		
* Specifies the title fontWeight of primaryYAxis. This fontWeight is applied to the title text of primaryYAxis. See {@link fontWeight}
* @default ej.Chart.fontWeight.Regular
* @type {enum}
* @example 
* //To set title fontWeight of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ title:{ font:{fontWeight:"lighter"}}}});				
* @example 
* //Gets or sets the title fontWeight of primaryYAxis, after initialization:<br>
*	//Gets the title fontWeight of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.title.font.fontWeight");<br>			
*	//Sets the title fontWeight of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{ title:{font:{fontWeight:"lighter"}}}} ); 
* @alias ejChart#primaryYAxis->title->font->fontWeight
* @instance
*/
         fontWeight: 'regular'
     }
    },
          /**		
* Specifies the rangePadding of primaryYAxis.  Specify the spacing of the chart axis for auto range calculation
None: Range of axis is calculated from the minimum value to the maximum value in datasource
Normal: Range of axis is calculated based on range calculation,(for numerical axis only)
Additional: Range of axis is calculated by adding interval to minimum and  maximum value in datasource.
Round: Round the range of axis to the nearest possible value.  See {@link rangePadding}
* @default ej.Chart.rangePadding.None
* @type {enum}
* @example 
* //To set rangePadding of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{rangePadding:"none"}});				
* @example 
* //Gets or sets the rangePadding of primaryYAxis, after initialization:<br>
*	//Gets the rangePadding of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.rangePadding");<br>			
*	//Sets the rangePadding of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{rangePadding:"none"}} ); 
* @alias ejChart#primaryYAxis->rangePadding
* @instance
*/
          rangePadding: 'Normal',

          stripline: [],
          /**		
 * Specifies the logBase of primaryYAxis. LogBase is used in log axis to render certain values.
 * @default 10
 * @type {number}
 * @example 
 * //To set logBase of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{logBase:5}});				
 * @example 
 * //Gets or sets the logBase of primaryYAxis, after initialization:<br>
 *	//Gets the logBase of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.logBase");<br>			
 *	//Sets the logBase of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{logBase:5}} ); 
 * @alias ejChart#primaryYAxis->logBase
 * @instance
 */
          logBase: 10,
          /**		
 * Specifies the plotOffset of primaryYAxis. This specifies the offset.
 * @default 10
 * @type {number}
 * @example 
 * //To set plotOffset of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{plotOffset:1}});				 
 * @example 
 * //Gets or sets the plotOffset of primaryYAxis, after initialization:<br>
 *	//Gets the plotOffset of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.plotOffset");<br>			
 *	//Sets the plotOffset of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{plotOffset:1}} ); 
  * @alias ejChart#primaryYAxis->plotOffset
 * @instance
 */
          plotOffset: 0,
          /**		
 * Specifies the orientation of primaryYAxis.  This is to set the orientation for chart axis. The user can set either vertical or horizontal. See {@link orientation}
 * @default ej.Chart.orientation.Horizontal
 * @type {enum}
 * @example 
 * //To set orientation of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{orientation:"Horizontal"}});				
 * @example 
 * //Gets or sets the orientation of primaryYAxis, after initialization:<br>
 *	//Gets the orientation of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.orientation");<br>			
 *	//Sets the orientation of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{orientation:"Horizontal"}} ); 
  * @alias ejChart#primaryYAxis->orientation
 * @instance
 */
          orientation: 'Vertical',
          /**		
* Specifies the maximumLabels of primaryYAxis.  This specifies the maximum number of labels to be rendered on the axis.
* @default 3
* @type {number}
* @example 
* //To set maximumLabels of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{maximumLabels:2}});				 
* @example 
* //Gets or sets the maximumLabels of primaryYAxis, after initialization:<br>
*	//Gets the maximumLabels of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.maximumLabels");<br>			
*	//Sets the maximumLabels of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{maximumLabels:2}} ); 
 * @alias ejChart#primaryYAxis->maximumLabels
* @instance
*/
          maximumLabels: 3,
          /**		
 * Specifies the hidePartialLabels of primaryYAxis. By applying this property the user can hide the partially visible labels. 
 * @default false
 * @type {boolean}
 * @example 
 * //To set hidePartialLabels of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{hidePartialLabels:true}});				
 * @example 
 * //Gets or sets the hidePartialLabels of primaryYAxis, after initialization:<br>
 *	//Gets the hidePartialLabels of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.hidePartialLabels");<br>			
 *	//Sets the hidePartialLabels of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{hidePartialLabels:true}} ); 
  * @alias ejChart#primaryYAxis->hidePartialLabels
 * @instance
 */
          hidePartialLabels: false,
          /**		
* Specifies the crosshairLabel of primaryYAxis. These are the properties relate to crosshair Label.
* @type {object}
* @alias ejChart#primaryYAxis->crosshairLabel
* @instance
*/
          crosshairLabel:
   {
       /**		
* Specifies the visibility of crosshairLabel. This is the property to change the visibility of the axis labels. 
* @default false
* @type {boolean}
* @example 
* //To set visibility of crosshairLabel
* 	$("#container").ejChart({primaryYAxis:{crosshairLabel:{visible:true}}});				 
* @example 
* //Gets or sets the visibility of crosshairLabel, after initialization:<br>
*	//Gets the visibility of crosshairLabel  
*	$("#container").ejChart("option", "primaryYAxis.crosshairLabel.visible");<br>			
*	//Sets the visibility of crosshairLabel
*	$("#container").ejChart("option", {primaryYAxis:{crosshairLabel:{visible:true}}} ); 
 * @alias ejChart#primaryYAxis->crosshairLabel->visible
* @instance
*/
       visible: false
   },
          /**		
 * Specifies the visibility of primaryYAxis. 
 * @default true
 * @type {boolean}
 * @example 
 * //To set visibility of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{visible:false}});				
 * @example 
 * //Gets or sets the visibility of primaryYAxis, after initialization:<br>
 *	//Gets the visibility of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.visible");<br>			
 *	//Sets the visibility of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{visible:false}} ); 
  * @alias ejChart#primaryYAxis->visible
 * @instance
 */
          visible: true,
          /**		
* Specifies the opposedPosition of primaryYAxis. This is to render the axis in opposed position or not. 
* @default false
* @type {boolean}
* @example 
* //To set opposedPosition of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{opposedPosition:true}});				
* @example 
* //Gets or sets the opposedPosition of primaryYAxis, after initialization:<br>
*	//Gets the opposedPosition of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.opposedPosition");<br>			
*	//Sets the opposedPosition of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{opposedPosition:true}} ); 
 * @alias ejChart#primaryYAxis->opposedPosition
* @instance
*/
          opposedPosition: false,
          /**		
* Specifies the font of primaryYAxis.  These font properties are applied for the axis labels. 
* @type {object}
* @alias ejChart#primaryYAxis->font
* @instance
*/
          font:
   {
       /**		
* Specifies the fontfamily of primaryYAxis. The axis labels render with this specified fontFamily.
* @default "Segoe UI"
* @type {string}
* @example 
* //To set fontfamily of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ font:{fontFamily:"Algerian"}}});				
* @example 
* //Gets or sets the fontfamily of primaryYAxis, after initialization:<br>
*	//Gets the fontfamily of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.font.fontFamily");<br>			
*	//Sets the fontfamily of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{ font:{fontFamily:"Algerian"}}} ); 
 * @alias ejChart#primaryYAxis->font->fontFamily
* @instance
*/
       fontFamily: 'Segoe UI',
       /**		
* Specifies the fontStyle of primaryYAxis. The axis labels render with this specified fontStyle.  See {@link fontStyle}
* @default ej.Chart.fontStyle.Normal
* @type {enum}
* @example 
* //To set fontStyle of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ font:{fontStyle:"Italic"}}});				
* @example 
* //Gets or sets the fontStyle of primaryYAxis, after initialization:<br>
*	//Gets the fontStyle of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.font.fontStyle");<br>			
*	//Sets the fontStyle of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{font:{fontStyle:"Italic"}}} ); 
 * @alias ejChart#primaryYAxis->font->fontStyle
* @instance
*/
       fontStyle: 'Normal',
       /**		
 * Specifies the title size of primaryYAxis. The axis labels render with this specified size.
 * @default "13px"
 * @type {string}
 * @example 
 * //To set size of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{font:{size:"12px"}}});				
 * @example 
 * //Gets or sets the size of primaryYAxis, after initialization:<br>
 *	//Gets the size of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.font.size");<br>			
 *	//Sets the size of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{font:{size:"12px"}}} ); 
  * @alias ejChart#primaryYAxis->font->size
 * @instance
 */
       size: '13px',
       /**		
* Specifies the title opacity of primaryYAxis. The axis labels render with this specified opacity.
* @default 1
* @type {number}
* @example 
* //To set title opacity of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{ title:{ font:{opacity:0}}}});			 
* @example 
* //Gets or sets the title opacity of primaryYAxis, after initialization:<br>
*	//Gets the title opacity of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.title.font.opacity");<br>			
*	//Sets the title opacity of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{ title:{font:{opacity:0}}}} ); 
* @alias ejChart#primaryYAxis->font->opacity
* @instance
*/
       opacity: 1,
       /**		
* Specifies the fontWeight of primaryYAxis. The axis labels render with this specified fontWeight.  {@link fontWeight}
* @default ej.Chart.fontWeight.Regular
* @type {enum}
* @example 
* //To set fontWeight of primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{font:{fontWeight:"lighter"}}});				
* @example 
* //Gets or sets the fontWeight of primaryYAxis, after initialization:<br>
*	//Gets the fontWeight of primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.font.fontWeight");<br>			
*	//Sets the fontWeight of primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{font:{fontWeight:"lighter"}}} ); 
 * @alias ejChart#primaryYAxis->font->fontWeight
* @instance
*/
       fontWeight: 'regular'
   },
          /**		
  * Specifies the axisLine. This line is drawn in chart to represent the end of the axis. 
  * @type {object}
  * @alias ejChart#primaryYAxis->axisLine
  * @instance
  */
          axisLine:
   {
       /**		
* Specifies the visibility of axisLine in primaryYAxis. The user can change the visibility of the axisLine.
* @default true
* @type {boolean}
* @example 
* //To set visibility of axisLine in primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{axisLine:{visible:false}}});				
* @example 
* //Gets or sets the visibility of axisLine in primaryYAxis, after initialization:<br>
*	//Gets the visibility of axisLine in primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.axisLine.visible");<br>			
*	//Sets the visibility of axisLine in primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{axisLine:{visible:false}}} ); 
 * @alias ejChart#primaryYAxis->axisLine->visible
* @instance
*/
       visible: true,
       /**		
* Specifies the width of axisLine in primaryYAxis. The user can change the width of the axisLine.
* @default 1
* @type {number}
* @example 
* //To set width of axisLine in primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{axisLine:{width:2}}});				 
* @example 
* //Gets or sets the width of axisLine in primaryYAxis, after initialization:<br>
*	//Gets the width of axisLine in primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.axisLine.width");<br>			
*	//Sets the width of axisLine in primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{axisLine:{width:2}}} ); 
* @alias ejChart#primaryYAxis->axisLine->width
* @instance
*/
       width: 1,
       /**		
* Specifies the dashArray of axisLine in primaryYAxis.  The user can change the dashArray of the axisLine.
* @default null
* @type {string}
* @example 
* //To set dashArray of axisLine in primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{axisLine:{dashArray:"2,3"}}});				 
* @example 
* //Gets or sets the dashArray of axisLine in primaryYAxis, after initialization:<br>
*	//Gets the dashArray of axisLine in primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.axisLine.dashArray");<br>			
*	//Sets the dashArray of axisLine in primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{axisLine:{dashArray:"2,3"}}} ); 
* @alias ejChart#primaryYAxis->axisLine->dashArray
* @instance
*/
       dashArray: "",
       /**		
* Specifies the offset of axisLine in primaryYAxis. The user can change the offset values of the axisLine.
* @default null
* @type {number}
* @example 
* //To set offset of axisLine in primaryYAxis
* 	$("#container").ejChart({primaryYAxis:{axisLine:{offset:1}}});				
* @example 
* //Gets or sets the offset of axisLine in primaryYAxis, after initialization:<br>
*	//Gets the offset of axisLine in primaryYAxis  
*	$("#container").ejChart("option", "primaryYAxis.axisLine.offset");<br>			
*	//Sets the offset of axisLine in primaryYAxis
*	$("#container").ejChart("option", {primaryYAxis:{axisLine:{offset:1}}} ); 
* @alias ejChart#primaryYAxis->axisLine->offset
* @instance
*/
       offset: 0
   },

          /**		
  * Specifies the zoomFactor of primaryYAxis. The user can specify value from 0 to 1.
  * @default 1
  * @type {number}
  * @example 
  * //To set zoomFactor of primaryYAxis
  * 	$("#container").ejChart({primaryYAxis:{zoomFactor:0}});				
  * @example 
  * //Gets or sets the zoomFactor of primaryYAxis, after initialization:<br>
  *	//Gets the zoomFactor of primaryYAxis  
  *	$("#container").ejChart("option", "primaryYAxis.zoomFactor");<br>			
  *	//Sets the zoomFactor of primaryYAxis
  *	$("#container").ejChart("option", {primaryYAxis:{zoomFactor:0}} ); 
   * @alias ejChart#primaryYAxis->zoomFactor
  * @instance
  */
          zoomFactor: 1,
          /**		
 * Specifies the zoomPosition of primaryYAxis. The user can specify value from 0 to 1.
 * @default 0
 * @type {number}
 * @example 
 * //To set zoomPosition of primaryYAxis
 * 	$("#container").ejChart({primaryYAxis:{zoomPosition:1}});				 
 * @example 
 * //Gets or sets the zoomPosition of primaryYAxis, after initialization:<br>
 *	//Gets the zoomPosition of primaryYAxis  
 *	$("#container").ejChart("option", "primaryYAxis.zoomPosition");<br>			
 *	//Sets the zoomPosition of primaryYAxis
 *	$("#container").ejChart("option", {primaryYAxis:{zoomPosition:1}} ); 
 * @alias ejChart#primaryYAxis->zoomPosition
 * @instance
 */
          zoomPosition: 0
      },

            /**		
           * Specifies the secondaryX. This includes the properties relate to the secondary x axis. 
           * @type {object}
            * @memberof ejChart
           * @instance
           */
            secondaryX:
             {
                 /**		
             * Specifies the majorGridLines. These are properties involved in customizing the majorGridLines of chart.
             * @type {object}
              * @alias ejChart#secondaryX->majorGridLines
             * @instance
             */
                 majorGridLines:
        {
            /**		
        * Specifies the width of the majorGridLines. The user can modify the width of the majorGridLines in secondaryX.
        * @default 1
        * @type {number}
        * @example 
        * //To set width of majorGridLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{majorGridLines:{width:2}}});				
        * @example 
        * //Gets or sets the width of the majorGridLines, after initialization:<br>
        *	//Gets the width of the majorGridLines  
        *	$("#container").ejChart("option", "secondaryX.majorGidLines.width");<br>			
        *	//Sets the width of the majorGridLines
        *	$("#container").ejChart("option", {secondaryX:{majorGridLines:{width:2}}} ); 
        * @alias ejChart#secondaryX->majorGridLines->width
        * @instance
        */
            width: 1,
            /**		
        * Specifies the dashArray of the majorGridLines. The user can modify the dashArray of the majorGridLines in secondaryX.
        * @default null
        * @type {string}
        * @example 
        * //To set dashArray of majorGridLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{majorGridLines:{dashArray:"2,3"}}});				
        * @example 
        * //Gets or sets the dashArray of the majorGridLines, after initialization:<br>
        *	//Gets the dashArray of the majorGridLines  
        *	$("#container").ejChart("option", "secondaryX.majorGidLines.dashArray");<br>			
        *	//Sets the dashArray of the majorGridLines
        *	$("#container").ejChart("option", {secondaryX:{majorGridLines:{dashArray:"2,3"}}} ); 
        * @alias ejChart#secondaryX->majorGridLines->dashArray
        * @instance
        */
            dashArray: "",
            /**		
        * Specifies the visibility of the majorGridLines.  The user can modify the visibility of the majorGridLines in secondaryX.
        * @default true
        * @type {boolean}
        * @example 
        * //To set visibility of majorGridLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{majorGridLines:{visible:false}}});				
        * @example
        * //Gets or sets the visibility of the majorGridLines, after initialization:<br>
        *	//Gets the visibility of the majorGridLines  
        *	$("#container").ejChart("option", "secondaryX.majorGidLines.visible");<br>			
        *	//Sets the visibility of the majorGridLines
        *	$("#container").ejChart("option", {secondaryX:{majorGridLines:{visible:false}}} ); 
        * @alias ejChart#secondaryX->majorGridLines->visible
        * @instance
        */
            visible: true,
        },
                 /**		
             * Specifies the majorTickLines. These are properties involved in customizing the majorTickLines of chart. 
             * @type {object}
              * @alias ejChart#secondaryX->majorTickLines
             * @instance
             */
                 majorTickLines:
         {
             /**		
        * Specifies the width of the majorTickLines. The user can modify the width of the majorTickLines in secondaryX.
        * @default 1
        * @type {number}
        * @example 
        * //To set width of majorTickLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{majorTickLines:{width:2}}});				
        * @example 
        * //Gets or sets the width of the majorTickLines, after initialization:<br>
        *	//Gets the width of the majorTickLines  
        *	$("#container").ejChart("option", "secondaryX.majorTickLines.width");<br>			
        *	//Sets the width of the majorTickLines
        *	$("#container").ejChart("option", {secondaryX:{majorTickLines:{width:2}}} ); 
         * @alias ejChart#secondaryX->majorTickLines->width
        * @instance
        */
             width: 1,
             /**		
        * Specifies the size of the majorTickLines. The user can modify the size of the majorTickLines in secondaryX.
        * @default 5
        * @type {number}
        * @example 
        * //To set size of majorTickLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{majorTickLines:{size:3}}});			 
        * @example 
        * //Gets or sets the size of the majorTickLines, after initialization:<br>
        *	//Gets the size of the majorTickLines  
        *	$("#container").ejChart("option", "secondaryX.majorTickLines.size");<br>			
        *	//Sets the size of the majorTickLines
        *	$("#container").ejChart("option", {secondaryX:{majorTickLines:{size:3}}} ); 
        * @alias ejChart#secondaryX->majorTickLines->size
        * @instance
        */
             size: 5,
             /**		
        * Specifies the visibility of the majorTickLines. The user can modify the visibility of the majorTickLines in secondayX.
        * @default true
        * @type {boolean}
        * @example 
        * //To set visibility of majorTickLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{majorTickLines:{visible:false}}});				
        * @example 
        * //Gets or sets the visibility of the majorTickLines, after initialization:<br>
        *	//Gets the visibility of the majorTickLines  
        *	$("#container").ejChart("option", "secondaryX.majorTickLines.visible");<br>			
        *	//Sets the visibility of the majorTickLines
        *	$("#container").ejChart("option", {secondaryX:{majorTickLines:{visible:false}}} ); 
        * @alias ejChart#secondaryX->majorTickLines->visible
        * @instance
        */
             visible: true
         },
                 /**		
             * Specifies the minorGridLines. These are properties involved in customizing the minorGridLines of chart. 
             * @type {object}
             * @alias ejChart#secondaryX->minorGridLines
             * @instance
             */
                 minorGridLines:
         {
             /**		
        * Specifies the width of the minorGridLines. The user can modify the width of the minorGridLines in secondaryX.
        * @default 1
        * @type {number}
        * @example 
        * //To set width of minorGridLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{minorGridLines:{width:2}}});				
        * @example 
        * //Gets or sets the width of the minorGridLines, after initialization:<br>
        *	//Gets the width of the minorGridLines  
        *	$("#container").ejChart("option", "secondaryX.minorGridLines.width");<br>			
        *	//Sets the width of the minorGridLines
        *	$("#container").ejChart("option", {secondaryX:{minorGridLines:{width:2}}} ); 
         * @alias ejChart#secondaryX->minorGridLines->width
        * @instance
        */
             width: 1,
             /**		
        * Specifies the dashArray of the minorGridLines. The user can modify the dashArray of the minorGridLines in secondaryX.
        * @default null
        * @type {string}
        * @example 
        * //To set dashArray of minorGridLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{minorGridLines:{dashArray:"2,3"}}});				
        * @example 
        * //Gets or sets the dashArray of the minorGridLines, after initialization:<br>
        *	//Gets the dashArray of the minorGridLines  
        *	$("#container").ejChart("option", "secondaryX.minorGridLines.dashArray");<br>			
        *	//Sets the dashArray of the minorGridLines
        *	$("#container").ejChart("option", {secondaryX:{minorGridLines:{dashArray:"2,3"}}} ); 
        * @alias ejChart#secondaryX->minorGridLines->dashArray
        * @instance
        */
             dashArray: "",
             /**		
        * Specifies the visibility of the minorGridLines. The user can modify the visibility of the minorGridLines in secondaryX.
        * @default false
        * @type {boolean}
        * @example 
        * //To set visibility of minorGridLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{minorGridLines:{visible:true}}});				 
        * @example 
        * //Gets or sets the visibility of the minorGridLines, after initialization:<br>
        *	//Gets the visibility of the minorGridLines  
        *	$("#container").ejChart("option", "secondaryX.minorGridLines.visible");<br>			
        *	//Sets the visibility of the minorGridLines
        *	$("#container").ejChart("option", {secondaryX:{minorGridLines:{visible:true}}} ); 
        * @alias ejChart#secondaryX->minorGridLines->visible
        * @instance
        */
             visible: false

         },
                 /**		
             * Specifies the minorTickLines. These are properties involved in customizing the minorTickLines of chart.
             * @type {object}
              * @alias ejChart#secondaryX->minorTickLines
             * @instance
             */
                 minorTickLines:
        {
            /**		
        * Specifies the width of the minorTickLines. The user can modify the width of the minorTickLines in secondaryX.
        * @default 1
        * @type {number}
        * @example 
        * //To set width of minorTickLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{minorTickLines:{width:0}}});				
        * @example 
        * //Gets or sets the width of the minorTickLines, after initialization:<br>
        *	//Gets the width of the minorTickLines  
        *	$("#container").ejChart("option", "secondaryX.minorTickLines.width");<br>			
        *	//Sets the width of the minorTickLines
        *	$("#container").ejChart("option", {secondaryX:{minorTickLines:{width:0}}} ); 
        * @alias ejChart#secondaryX->minorTickLines->width
        * @instance
        */
            width: 1,
            /**		
        * Specifies the size of the minorTickLines. The user can modify the size of the minorTickLines in secondaryX.
        * @default 5
        * @type {number}
        * @example 
        * //To set size of minorTickLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{minorTickLines:{size:3}}});				 
        * @example 
        * //Gets or sets the size of the minorTickLines, after initialization:<br>
        *	//Gets the size of the minorTickLines  
        *	$("#container").ejChart("option", "secondaryX.minorTickLines.size");<br>			
        *	//Sets the size of the minorTickLines
        *	$("#container").ejChart("option", {secondaryX:{minorTickLines:{size:3}}} ); 
        * @alias ejChart#secondaryX->minorTickLines->size
        * @instance
        */
            size: 5,
            /**		
        * Specifies the visibility of the minorTickLines. The user can modify the visibility of the minorTickLines in secondaryX.
        * @default false
        * @type {boolean}
        * @example 
        * //To set visibility of minorTickLines value during initialization  
        * 	$("#container").ejChart({secondaryX:{minorTickLines:{visible:true}}});				
        * @example 
        * //Gets or sets the visibility of the minorTickLines, after initialization:<br>
        *	//Gets the visibility of the minorTickLines  
        *	$("#container").ejChart("option", "secondaryX.minorTickLines.visible");<br>			
        *	//Sets the visibility of the minorTickLines
        *	$("#container").ejChart("option", {secondaryX:{minorTickLines:{visible:true}}} ); 
        * @alias ejChart#secondaryX->minorTickLines->visible
        * @instance
        */
            visible: false
        },
                 /**		
              * Specifies the minorTicksPerInterval of secondaryX. This property is to set the number of minor ticks to render per interval
              * @default null
              * @type {number}
              * @example 
              * //To set minorTicksPerInterval of secondaryX value during initialization  
              * 	$("#container").ejChart({secondaryX:{minorTicksPerInterval:2}});				
              * @example 
              * //Gets or sets the minorTicksPerInterval of secondaryX, after initialization:<br>
              *	//Gets the minorTicksPerInterval of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.minorTicksPerInterval");<br>			
              *	//Sets the minorTicksPerInterval of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{minorTicksPerInterval:2}} ); 
              * @alias ejChart#secondaryX->minorTicksPerInterval
              * @instance
              */
                 minorTicksPerInterval: null,
                 /**		
             * Specifies the columnIndex of secondaryX. This is to specify the index of the column.
             * @default null
             * @type {number}
             * @example 
             * //To set columnIndex of secondaryX value during initialization  
             * 	$("#container").ejChart({secondaryX:{columnIndex:2}});				
             * @example 
             * //Gets or sets the columnIndex of secondaryX, after initialization:<br>
             *	//Gets the columnIndex of secondaryX  
             *	$("#container").ejChart("option", "secondaryX.columnIndex");<br>			
             *	//Sets the columnIndex of secondaryX
             *	$("#container").ejChart("option", {secondaryX:{columnIndex:2}} ); 
              * @alias ejChart#secondaryX->columnIndex
             * @instance
             */
                 columnIndex: null,
                 /**		
              * Specifies the columnSpan of secondaryX. EjChart provides option to span the column. This is achieved by using this property.
              * @default null
              * @type {number}
              * @example 
              * //To set columnSpan of secondaryX value during initialization  
              * 	$("#container").ejChart({secondaryX:{columnSpan:2}});				 
              * @example 
              * //Gets or sets the columnSpan of secondaryX, after initialization:<br>
              *	//Gets the columnSpan of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.columnSpan");<br>			
              *	//Sets the columnSpan of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{columnSpan:2}} ); 
               * @alias ejChart#secondaryX->columnSpan
              * @instance
              */
                 columnSpan: null,
                 /**		
              * Specifies the rotation of labels.The user can rotate the labels to desired angle. The axis labels are rendered in the degree, specified for the label rotation property.
              * @default null
              * @type {number}
              * @example 
              * //To set rotation of labels  
              * 	$("#container").ejChart({secondaryX:{labelRotation:45}});				
              * @example 
              * //Gets or sets the rotation of labels, after initialization:<br>
              *	//Gets the label rotation value  
              *	$("#container").ejChart("option", "secondaryX.labelRotation");<br>			
              *	//Sets the rotation of labels
              *	$("#container").ejChart("option", {secondaryX:{labelRotation:45}} ); 
              * @alias ejChart#secondaryX->labelRotation
              * @instance
              */
                 labelRotation: null,
                 /**		
             * Specifies the valueType of secondaryX. It denotes the type of axis. It can be double, datetime, etc.  See {@link valueType}
             * @default null
             * @type {enum}
             * @example 
             * //To set valueType of secondaryX
             * 	$("#container").ejChart({secondaryX:{valueType:"Double"}});				 
             * @example 
             * //Gets or sets the valueType of secondaryX, after initialization:<br>
             *	//Gets the valueType of secondaryX  
             *	$("#container").ejChart("option", "secondaryX.valueType");<br>			
             *	//Sets the valueType of secondaryX
             *	$("#container").ejChart("option", {secondaryX:{valueType:"Double"}} ); 
              * @alias ejChart#secondaryX->valueType
             * @instance
             */
                 valueType: null,
                 /**		
               * Specifies the axisName of secondaryX. This is the name to represent the axis in chart.
               * @default null
               * @type {string}
               * @example 
               * //To set axisName of secondaryX
               * 	$("#container").ejChart({secondaryX:{axisName:"secondaryX"}});				 
               * @example 
               * //Gets or sets the axisName of secondaryX, after initialization:<br>
               *	//Gets the axisName of secondaryX  
               *	$("#container").ejChart("option", "secondaryX.axisName");<br>			
               *	//Sets the axisName of secondaryX
               *	$("#container").ejChart("option", {secondaryX:{axisName:"secondaryX"}} ); 
                * @alias ejChart#secondaryX->axisName
               * @instance
               */
                 axisName: null,
                 /**		
              * Specifies the labelFormat of secondaryX. The axis labels render in the format, which is specified here.
              * @default null
              * @type {string}
              * @example 
              * //To set labelFormat of secondaryX
              * 	$("#container").ejChart({secondaryX:{labelFormat:"${value}"}});				 
              * @example 
              * //Gets or sets the labelFormat of secondaryX, after initialization:<br>
              *	//Gets the labelFormat of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.labelFormat");<br>			
              *	//Sets the labelFormat of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{labelFormat:"${value}"}} ); 
               * @alias ejChart#secondaryX->labelFormat
              * @instance
              */
                 labelFormat: null,
                 /**		
             * Specifies the desiredIntervals of secondaryX. By this property we can display the labels after appropriate intervals.
             * @default null
             * @type {number}
             * @example 
             * //To set desiredIntervals of secondaryX
             * 	$("#container").ejChart({secondaryX:{desiredIntervals:5}});				 
             * @example 
             * //Gets or sets the desiredIntervals of secondaryX, after initialization:<br>
             *	//Gets the desiredIntervals of secondaryX  
             *	$("#container").ejChart("option", "secondaryX.desiredIntervals");<br>			
             *	//Sets the desiredIntervals of secondaryX
             *	$("#container").ejChart("option", {secondaryX:{desiredIntervals:5}} ); 
              * @alias ejChart#secondaryX->desiredIntervals
             * @instance
             */
                 desiredIntervals: null,
                 /**		
              * Specifies the intervalType of secondaryX. This is to set the type of intervals to chart. IntervalType may be days, years, etc. See {@link intervalType}
              * @default null
              * @type {enum}
              * @example 
              * //To set desiredIntervintervalTypeals of secondaryX
              * 	$("#container").ejChart({secondaryX:{intervalType:"days"}});				 
              * @example 
              * //Gets or sets the intervalType of secondaryX, after initialization:<br>
              *	//Gets the intervalType of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.intervalType");<br>			
              *	//Sets the intervalType of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{intervalType:"days"}} ); 
               * @alias ejChart#secondaryX->intervalType
              * @instance
              */
                 intervalType: null,
                 /**		
              * Specifies the roundingPlaces of secondaryX. This property is to truncate the decimal values after the specified position.
              * @default null
              * @type {number}
              * @example 
              * //To set roundingPlaces of secondaryX
              * 	$("#container").ejChart({secondaryX:{roundingPlaces:2}});				
              * @example 
              * //Gets or sets the roundingPlaces of secondaryX, after initialization:<br>
              *	//Gets the roundingPlaces of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.roundingPlaces");<br>			
              *	//Sets the roundingPlaces of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{roundingPlaces:2}} ); 
               * @alias ejChart#secondaryX->roundingPlaces
              * @instance
              */
                 roundingPlaces: null,
                 /**		
              * Specifies the logBase of secondaryX. LogBase is used in log axis to render certain values.
              * @default 10
              * @type {number}
              * @example 
              * //To set logBase of secondaryX
              * 	$("#container").ejChart({secondaryX:{logBase:8}});				
              * @example 
              * //Gets or sets the logBase of secondaryX, after initialization:<br>
              *	//Gets the logBase of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.logBase");<br>			
              *	//Sets the logBase of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{logBase:8}} ); 
               * @alias ejChart#secondaryX->logBase
              * @instance
              */
                 logBase: 10,
                 /**		
              * Specifies the plotOffset of secondaryX.  This specifies the offset.
              * @default 10
              * @type {number}
              * @example 
              * //To set plotOffset of secondaryX
              * 	$("#container").ejChart({secondaryX:{plotOffset:1}});				
              * @example 
              * //Gets or sets the plotOffset of secondaryX, after initialization:<br>
              *	//Gets the plotOffset of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.plotOffset");<br>			
              *	//Sets the plotOffset of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{plotOffset:1}} ); 
               * @alias ejChart#secondaryX->plotOffset
              * @instance
              */
                 plotOffset: 0,
                 labels: [],
                 stripline: [],
                 /**		
            * Specifies the title. This is to be rendered as title for secondaryX.
            * @type {object}
             * @alias ejChart#secondaryX->title
            * @instance
            */
                 title:
         {

             /**		
           * Specifies the title text of secondaryX. This is the text which is to be rendered as title for secondaryX.
           * @default ""
           * @type {string}
           * @example 
           * //To set title text of secondaryX
           * 	$("#container").ejChart({secondaryX:{ title:{ text:"Sales comparison"}}});				
           * @example 
           * //Gets or sets the title text of secondaryX, after initialization:<br>
           *	//Gets the title text of secondaryX  
           *	$("#container").ejChart("option", "secondaryX.title.text");<br>			
           *	//Sets the title text of secondaryX
           *	$("#container").ejChart("option", {secondaryX:{ title:{text:"Sales comparison"}}} ); 
            * @alias ejChart#secondaryX->title->text
           * @instance
           */
             text: "",
             /**		
        * Specifies the font. These font properties are customization are applied for title text.
        * @type {object}
        * @alias ejChart#secondaryX->title->font
        * @instance
        */
             font:
        {
            /**		
          * Specifies the title fontfamily of secondaryX. This fontFamily is applied to the title text of secondaryX.
          * @default "Segoe UI"
          * @type {string}
          * @example 
          * //To set title fontfamily of secondaryX
          * 	$("#container").ejChart({secondaryX:{ title:{ font:{fontFamily:"Algerian"}}}});				 
          * @example 
          * //Gets or sets the title fontfamily of secondaryX, after initialization:<br>
          *	//Gets the title fontfamily of secondaryX  
          *	$("#container").ejChart("option", "secondaryX.title.font.fontFamily");<br>			
          *	//Sets the title fontfamily of secondaryX
          *	$("#container").ejChart("option", {secondaryX:{ title:{font:{fontFamily:"Algerian"}}}} ); 
           * @alias ejChart#secondaryX->title->font->fontFamily
          * @instance
          */
            fontFamily: 'Segoe UI',
            /**		
            * Specifies the title fontStyle of secondaryX. This fontStyle is applied to the title text of secondaryX. See {@link fontStyle}
            * @default ej.Chart.fontStyle.Normal
            * @type {enum}
            * @example 
            * //To set title fontStyle of secondaryX
            * 	$("#container").ejChart({secondaryX:{ title:{ font:{fontStyle:"Italic"}}}});				 
            * @example 
            * //Gets or sets the title fontStyle of secondaryX, after initialization:<br>
            *	//Gets the title fontStyle of secondaryX  
            *	$("#container").ejChart("option", "secondaryX.title.font.fontStyle");<br>			
            *	//Sets the title fontStyle of secondaryX
            *	$("#container").ejChart("option", {secondaryX:{ title:{font:{fontStyle:"Italic"}}}} ); 
             * @alias ejChart#secondaryX->title->font->fontStyle
            * @instance
            */
            fontStyle: 'Normal',
            /**		
         * Specifies the title size of secondaryX. This font size is applied to the title text of secondaryX.
         * @default "16px"
         * @type {string}
         * @example 
         * //To set title size of secondaryX
         * 	$("#container").ejChart({secondaryX:{ title:{ font:{size:"16px"}}}});				
         * @example 
         * //Gets or sets the title size of secondaryX, after initialization:<br>
         *	//Gets the title size of secondaryX  
         *	$("#container").ejChart("option", "secondaryX.title.font.size");<br>			
         *	//Sets the title size of secondaryX
         *	$("#container").ejChart("option", {secondaryX:{ title:{font:{size:"16px"}}}} ); 
          * @alias ejChart#secondaryX->title->font->size
         * @instance
         */
            size: '16px',
            /**		
         * Specifies the title opacity of secondaryX. This opacity value is applied to the title text of secondaryX.
         * @default 1
         * @type {number}
         * @example 
         * //To set title opacity of secondaryX
         * 	$("#container").ejChart({secondaryX:{ title:{ font:{opacity:1}}}});				 
         * @example 
         * //Gets or sets the title opacity of secondaryX, after initialization:<br>
         *	//Gets the title opacity of secondaryX  
         *	$("#container").ejChart("option", "secondaryX.title.font.opacity");<br>			
         *	//Sets the title opacity of secondaryX
         *	$("#container").ejChart("option", {secondaryX:{ title:{font:{opacity:1}}}} ); 
          * @alias ejChart#secondaryX->title->font->opacity
         * @instance
         */
            opacity: 1,
            /**		
         * Specifies the title fontWeight of secondaryX. This fontWeight value is applied to the title text of secondaryX. See {@link fontWeight}
         * @default ej.Chart.fontWeight.Regular
         * @type {enum}
         * @example 
         * //To set title fontWeight of secondaryX
         * 	$("#container").ejChart({secondaryX:{ title:{ font:{fontWeight:"lighter"}}}});				 
         * @example 
         * //Gets or sets the title fontWeight of secondaryX, after initialization:<br>
         *	//Gets the title fontWeight of secondaryX  
         *	$("#container").ejChart("option", "secondaryX.title.font.fontWeight");<br>			
         *	//Sets the title fontWeight of secondaryX
         *	$("#container").ejChart("option", {secondaryX:{ title:{font:{fontWeight:"lighter"}}}} ); 
          * @alias ejChart#secondaryX->title->font->fontWeight
         * @instance
         */
            fontWeight: 'regular'
        }
         },

                 /**		
              * Specifies the rangePadding of secondaryX. Specify the spacing of the chart axis for auto range calculation
None: Range of axis is calculated from the minimum value to the maximum value in datasource
Normal: Range of axis is calculated based on range calculation,(for numerical axis only)
Additional: Range of axis is calculated by adding interval to minimum and  maximum value in datasource.
Round: Round the range of axis to the nearest possible value. See {@link rangePadding}
              * @default ej.Chart.rangePadding.None
              * @type {enum}
              * @example 
              * //To set rangePadding of secondaryX
              * 	$("#container").ejChart({secondaryX:{rangePadding:"normal"}});				
              * @example 
              * //Gets or sets the rangePadding of secondaryX, after initialization:<br>
              *	//Gets the rangePadding of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.rangePadding");<br>			
              *	//Sets the rangePadding of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{rangePadding:"normal"}} ); 
               * @alias ejChart#secondaryX->rangePadding
              * @instance
              */
                 rangePadding: 'None',
                 additionalPadding: [1, 1],
                 /**		
              * Specifies the orientation of secondaryX. This is to set the orientation for chart axis. The user can set either vertical or horizontal. See {@link orientation}
              * @default ej.Chart.orientation.Horizontal
              * @type {enum}
              * @example 
              * //To set orientation of secondaryX
              * 	$("#container").ejChart({secondaryX:{orientation:"vertical"}});				
              * @example 
              * //Gets or sets the orientation of secondaryX, after initialization:<br>
              *	//Gets the orientation of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.orientation");<br>			
              *	//Sets the orientation of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{orientation:"vertical"}} ); 
               * @alias ejChart#secondaryX->orientation
              * @instance
              */
                 orientation: 'Horizontal',
                 /**		
              * Specifies the maximumLabels of secondaryX. This specifies the maximum number of labels to be rendered on the axis. 
              * @default 3
              * @type {number}
              * @example 
              * //To set maximumLabels of secondaryX
              * 	$("#container").ejChart({secondaryX:{maximumLabels:5}});			 
              * @example 
              * //Gets or sets the maximumLabels of secondaryX, after initialization:<br>
              *	//Gets the maximumLabels of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.maximumLabels");<br>			
              *	//Sets the maximumLabels of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{maximumLabels:5}} ); 
               * @alias ejChart#secondaryX->maximumLabels
              * @instance
              */
                 maximumLabels: 3,
                 /**		
             * Specifies the opposedPosition of secondaryX. This is to render the axis in opposed position or not. 
             * @default false
             * @type {boolean}
             * @example 
             * //To set opposedPosition of secondaryX
             * 	$("#container").ejChart({secondaryX:{opposedPosition:true}});				 
             * @example 
             * //Gets or sets the opposedPosition of secondaryX, after initialization:<br>
             *	//Gets the opposedPosition of secondaryX  
             *	$("#container").ejChart("option", "secondaryX.opposedPosition");<br>			
             *	//Sets the opposedPosition of secondaryX
             *	$("#container").ejChart("option", {secondaryX:{opposedPosition:true}} ); 
              * @alias ejChart#secondaryX->opposedPosition
             * @instance
             */
                 opposedPosition: false,
                 /**		
               * Specifies the axisLine. This line is drawn in chart to represent the end of the axis.  
               * @type {object}
               * @alias ejChart#secondaryX->axisLine
               * @instance
               */
                 axisLine:
          {
              /**		
             * Specifies the visibility of axisLine in secondaryX. The user can change the visibility of the axisLine.
             * @default true
             * @type {boolean}
             * @example 
             * //To set visibility of axisLine in secondaryX
             * 	$("#container").ejChart({secondaryX:{axisLine:{visible:false}}});				
             * @example 
             * //Gets or sets the visibility of axisLine in secondaryX, after initialization:<br>
             *	//Gets the visibility of axisLine in secondaryX  
             *	$("#container").ejChart("option", "secondaryX.axisLine.visible");<br>			
             *	//Sets the visibility of axisLine in secondaryX
             *	$("#container").ejChart("option", {secondaryX:{axisLine:{visible:false}}} ); 
              * @alias ejChart#secondaryX->axisLine->visible
             * @instance
             */
              visible: true,
              /**		
             * Specifies the width of axisLine in secondaryX. The user can change the width of the axisLine.
             * @default 1
             * @type {number}
             * @example 
             * //To set width of axisLine in secondaryX
             * 	$("#container").ejChart({secondaryX:{axisLine:{width:0.5}}});				
             * @example 
             * //Gets or sets the width of axisLine in secondaryX, after initialization:<br>
             *	//Gets the width of axisLine in secondaryX  
             *	$("#container").ejChart("option", "secondaryX.axisLine.width");<br>			
             *	//Sets the width of axisLine in secondaryX
             *	$("#container").ejChart("option", {secondaryX:{axisLine:{width:0.5}}} ); 
              * @alias ejChart#secondaryX->axisLine->width
             * @instance
             */
              width: 1,
              /**		
         * Specifies the dashArray of axisLine in secondaryX. The user can change the dashArray of the axisLine.
         * @default null
         * @type {string}
         * @example 
         * //To set dashArray of axisLine in secondaryX
         * 	$("#container").ejChart({secondaryX:{axisLine:{dashArray:"2,3"}}});				 
         * @example 
         * //Gets or sets the dashArray of axisLine in secondaryX, after initialization:<br>
         *	//Gets the dashArray of axisLine in secondaryX  
         *	$("#container").ejChart("option", "secondaryX.axisLine.dashArray");<br>			
         *	//Sets the dashArray of axisLine in secondaryX
         *	$("#container").ejChart("option", {secondaryX:{axisLine:{dashArray:"2,3"}}} ); 
         * @alias ejChart#secondaryX->axisLine->dashArray
         * @instance
         */
              dashArray: "",
              /**		
        * Specifies the offset of axisLine in secondaryX. The user can change the offset of the axisLine.
        * @default null
        * @type {number}
        * @example 
        * //To set offset of axisLine in secondaryX
        * 	$("#container").ejChart({secondaryX:{axisLine:{offset:1}}});				
        * @example 
        * //Gets or sets the offset of axisLine in secondaryX, after initialization:<br>
        *	//Gets the offset of axisLine in secondaryX  
        *	$("#container").ejChart("option", "secondaryX.axisLine.offset");<br>			
        *	//Sets the offset of axisLine in secondaryX
        *	$("#container").ejChart("option", {secondaryX:{axisLine:{offset:1}}} ); 
         * @alias ejChart#secondaryX->axisLine->offset
        * @instance
        */
              offset: 0
          },
                 /**		
              * Specifies the hidePartialLabels of secondaryX. By applying this property, the user can hide the partially visible labels in the axis.
              * @default false
              * @type {boolean}
              * @example 
              * //To set hidePartialLabels of secondaryX
              * 	$("#container").ejChart({secondaryX:{hidePartialLabels:true}});				
              * @example 
              * //Gets or sets the hidePartialLabels of secondaryX, after initialization:<br>
              *	//Gets the hidePartialLabels of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.hidePartialLabels");<br>			
              *	//Sets the hidePartialLabels of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{hidePartialLabels:true}} ); 
               * @alias ejChart#secondaryX->hidePartialLabels
              * @instance
              */
                 hidePartialLabels: false,
                 /**		
            * Specifies the font of secondaryX. These font properties are applied for the axis labels.
            * @type {object}
            * @alias ejChart#secondaryX->font
            * @instance
            */
                 font:
          {
              /**		
              * Specifies the fontfamily of secondaryX. The axis labels render with this specified fontFamily.
              * @default "Segoe UI"
              * @type {string}
              * @example 
              * //To set fontfamily of secondaryX
              * 	$("#container").ejChart({secondaryX:{ font:{fontFamily:"Algerian"}}});				
              * @example 
              * //Gets or sets the fontfamily of secondaryX, after initialization:<br>
              *	//Gets the fontfamily of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.font.fontFamily");<br>			
              *	//Sets the fontfamily of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{ font:{fontFamily:"Algerian"}}} ); 
               * @alias ejChart#secondaryX->font->fontFamily
              * @instance
              */
              fontFamily: 'Segoe UI',
              /**		
             * Specifies the fontStyle of secondaryX. The axis labels render with this specified fontStyle.  See {@link fontStyle}
             * @default ej.Chart.fontStyle.Normal
             * @type {enum}
             * @example 
             * //To set fontStyle of secondaryX
             * 	$("#container").ejChart({secondaryX:{ font:{fontStyle:"Italic"}}});				
             * @example 
             * //Gets or sets the fontStyle of secondaryX, after initialization:<br>
             *	//Gets the fontStyle of secondaryX  
             *	$("#container").ejChart("option", "secondaryX.font.fontStyle");<br>			
             *	//Sets the fontStyle of secondaryX
             *	$("#container").ejChart("option", {secondaryX:{font:{fontStyle:"Italic"}}} ); 
              * @alias ejChart#secondaryX->font->fontStyle
             * @instance
             */
              fontStyle: 'Normal',
              /**		
              * Specifies the title size of secondaryX. The axis labels render with this specified size.
              * @default "13px"
              * @type {string}
              * @example 
              * //To set size of secondaryX
              * 	$("#container").ejChart({secondaryX:{font:{size:"12px"}}});				
              * @example 
              * //Gets or sets the size of secondaryX, after initialization:<br>
              *	//Gets the size of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.font.size");<br>			
              *	//Sets the size of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{font:{size:"12px"}}} ); 
             * @alias ejChart#secondaryX->font->size
              * @instance
              */
              size: '13px',
              /**		
              * Specifies the fontWeight of secondaryX. The axis labels render with this specified fontWeight. See {@link fontWeight}
              * @default ej.Chart.fontWeight.Regular
              * @type {enum}
              * @example 
              * //To set fontWeight of secondaryX
              * 	$("#container").ejChart({secondaryX:{font:{fontWeight:"lighter"}}});				
              * @example 
              * //Gets or sets the fontWeight of secondaryX, after initialization:<br>
              *	//Gets the fontWeight of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.font.fontWeight");<br>			
              *	//Sets the fontWeight of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{font:{fontWeight:"lighter"}}} ); 
               * @alias ejChart#secondaryX->font->fontWeight
              * @instance
              */
              fontWeight: 'regular',
              /**		
             * Specifies the title opacity of secondaryX. The axis labels render with this specified opacity.
             * @default 1
             * @type {number}
             * @example 
             * //To set title opacity of secondaryX
             * 	$("#container").ejChart({secondaryX:{ title:{ font:{opacity:0.5}}}});				
             * @example 
             * //Gets or sets the title opacity of secondaryX, after initialization:<br>
             *	//Gets the title opacity of secondaryX  
             *	$("#container").ejChart("option", "secondaryX.title.font.opacity");<br>			
             *	//Sets the title opacity of secondaryX
             *	$("#container").ejChart("option", {secondaryX:{ title:{font:{opacity:0.5}}}} ); 
             * @alias ejChart#secondaryX->font->opacity
             * @instance
             */
              opacity: 1
          },
                 /**		
              * Specifies the visibility of secondaryX. The user can change the visibility of the axis labels.
              * @default true
              * @type {boolean}
              * @example 
              * //To set visibility of secondaryX
              * 	$("#container").ejChart({secondaryX:{visible:false}});			 
              * @example 
              * //Gets or sets the visibility of secondaryX, after initialization:<br>
              *	//Gets the visibility of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.visible");<br>			
              *	//Sets the visibility of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{visible:false}} ); 
               * @alias ejChart#secondaryX->visible
              * @instance
              */
                 visible: true,
                 /**		
            * Specifies the crosshairLabel of secondaryX. These are the properties relate to crosshair Label. 
            * @type {object}
             * @alias ejChart#secondaryX->crosshairLabel
            * @instance
            */
                 crosshairLabel:
           {
               /**		
              * Specifies the visibility of crosshairLabel.  This is the property to change the visibility of the axis labels. 
              * @default false
              * @type {boolean}
              * @example 
              * //To set visibility of crosshairLabel
              * 	$("#container").ejChart({secondaryX:{crosshairLabel:{visible:true}}});				 
              * @example 
              * //Gets or sets the visibility of crosshairLabel, after initialization:<br>
              *	//Gets the visibility of crosshairLabel  
              *	$("#container").ejChart("option", "secondaryX.crosshairLabel.visible");<br>			
              *	//Sets the visibility of crosshairLabel
              *	$("#container").ejChart("option", {secondaryX:{crosshairLabel:{visible:true}}} ); 
               * @alias ejChart#secondaryX->crosshairLabel->visible
              * @instance
              */
               visible: false
           },
                 /**		
               * Specifies the zoomFactor of secondaryX. This is to specify zoom factor value for axis zooming. The user can specify value from 0 to 1.
               * @default 1
               * @type {number}
               * @example 
               * //To set zoomFactor of secondaryX
               * 	$("#container").ejChart({secondaryX:{zoomFactor:0.5}});				
               * @example 
               * //Gets or sets the zoomFactor of secondaryX, after initialization:<br>
               *	//Gets the zoomFactor of secondaryX  
               *	$("#container").ejChart("option", "secondaryX.zoomFactor");<br>			
               *	//Sets the zoomFactor of secondaryX
               *	$("#container").ejChart("option", {secondaryX:{zoomFactor:0.5}} ); 
                * @alias ejChart#secondaryX->zoomFactor
               * @instance
               */
                 zoomFactor: 1,
                 /**		
              * Specifies the zoomPosition of secondaryX. To specify zoom position value for axis zooming.
              * @default 0
              * @type {number}
              * @example 
              * //To set zoomPosition of secondaryX
              * 	$("#container").ejChart({secondaryX:{zoomPosition:0.5}});				
              * @example 
              * //Gets or sets the zoomPosition of secondaryX, after initialization:<br>
              *	//Gets the zoomPosition of secondaryX  
              *	$("#container").ejChart("option", "secondaryX.zoomPosition");<br>			
              *	//Sets the zoomPosition of secondaryX
              *	$("#container").ejChart("option", {secondaryX:{zoomPosition:0.5}} ); 
                * @alias ejChart#secondaryX->zoomPosition
              * @instance
              */
                 zoomPosition: 0
             },
            /**		
           * Specifies the secondaryY. This includes the properties relate to the secondary y axis.
           * @type {object}
            * @memberof ejChart
           * @instance
           */
            secondaryY:
                {
                    /**		
            * Specifies the majorGridLines of secondaryYAxis. These are properties involved in customizing the majorGridLines of chart.
            * @type {object}
            * @alias ejChart#secondaryY->majorGridLines
            * @instance
            */
                    majorGridLines:
       {
           /**		
   * Specifies the width of the majorGridLines. The user can modify the width of the majorGridLines in secondaryY.
   * @default 1
   * @type {number}
   * @example 
   * //To set width of majorGridLines value during initialization  
   * 	$("#container").ejChart({secondaryY:{majorGridLines:{width:2}}});				
   * @example 
   * //Gets or sets the width of the majorGridLines, after initialization:<br>
   *	//Gets the width of the majorGridLines  
   *	$("#container").ejChart("option", "secondaryY.majorGidLines.width");<br>			
   *	//Sets the width of the majorGridLines
   *	$("#container").ejChart("option", {secondaryY:{majorGridLines:{width:2}}} ); 
   * @alias ejChart#secondaryY->majorGridLines->width
   * @instance
   */
           width: 1,
           /**		
   * Specifies the dashArray of the majorGridLines. The user can modify the dashArray of the majorGridLines in secondaryY.
   * @default null
   * @type {string}
   * @example 
   * //To set dashArray of majorGridLines value during initialization  
   * 	$("#container").ejChart({secondaryY:{majorGridLines:{dashArray:"2,3"}}});				
   * @example 
   * //Gets or sets the dashArray of the majorGridLines, after initialization:<br>
   *	//Gets the dashArray of the majorGridLines  
   *	$("#container").ejChart("option", "secondaryY.majorGidLines.dashArray");<br>			
   *	//Sets the dashArray of the majorGridLines
   *	$("#container").ejChart("option", {secondaryY:{majorGridLines:{dashArray:"2,3"}}} ); 
  * @alias ejChart#secondaryY->majorGridLines->dashArray
   * @instance
   */
           dashArray: "",
           /**		
   * Specifies the visibility of the majorGridLines. The user can modify the visibility of the majorGridLines in secondaryY.
   * @default true
   * @type {boolean}
   * @example 
   * //To set visibility of majorGridLines value during initialization  
   * 	$("#container").ejChart({secondaryY:{majorGridLines:{visible:false}}});				
   * @example 
   * //Gets or sets the visibility of the majorGridLines, after initialization:<br>
   *	//Gets the visibility of the majorGridLines  
   *	$("#container").ejChart("option", "secondaryY.majorGidLines.visible");<br>			
   *	//Sets the visibility of the majorGridLines
   *	$("#container").ejChart("option", {secondaryY:{majorGridLines:{visible:false}}} ); 
  * @alias ejChart#secondaryY->majorGridLines->visible
   * @instance
   */
           visible: true

       },
                    /**		
                * Specifies the majorTickLines. These are properties involved in customizing the majorTickLines of chart. 
                * @type {object}
                * @alias ejChart#secondaryY->majorTickLines
                * @instance
                */
                    majorTickLines:
                        {
                            /**		
                   * Specifies the width of the majorTickLines. The user can modify the width of the majorTickLines in secondaryY.
                   * @default 1
                   * @type {number}
                   * @example 
                   * //To set width of majorTickLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{majorTickLines:{width:2}}});				
                   * @example 
                   * //Gets or sets the width of the majorTickLines, after initialization:<br>
                   *	//Gets the width of the majorTickLines  
                   *	$("#container").ejChart("option", "secondaryY.majorTickLines.width");<br>			
                   *	//Sets the width of the majorTickLines
                   *	$("#container").ejChart("option", {secondaryY:{majorTickLines:{width:2}}} ); 
                    * @alias ejChart#secondaryY->majorTickLines->width
                   * @instance
                   */
                            width: 1,
                            /**		
                   * Specifies the size of the majorTickLines. The user can modify the size of the majorTickLines in secondaryY.
                   * @default 5
                   * @type {number}
                   * @example 
                   * //To set size of majorTickLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{majorTickLines:{size:4}}});				
                   * @example 
                   * //Gets or sets the size of the majorTickLines, after initialization:<br>
                   *	//Gets the size of the majorTickLines  
                   *	$("#container").ejChart("option", "secondaryY.majorTickLines.size");<br>			
                   *	//Sets the size of the majorTickLines
                   *	$("#container").ejChart("option", {secondaryY:{majorTickLines:{size:4}}} ); 
                    * @alias ejChart#secondaryY->majorTickLines->size
                   * @instance
                   */
                            size: 5,
                            /**		
                   * Specifies the visibility of the majorTickLines. The user can modify the visibility of the majorTickLines in secondaryY.
                   * @default true
                   * @type {boolean}
                   * @example 
                   * //To set visibility of majorTickLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{majorTickLines:{visible:false}}});				
                   * @example 
                   * //Gets or sets the visibility of the majorTickLines, after initialization:<br>
                   *	//Gets the visibility of the majorTickLines  
                   *	$("#container").ejChart("option", "secondaryY.majorTickLines.visible");<br>			
                   *	//Sets the visibility of the majorTickLines
                   *	$("#container").ejChart("option", {secondaryY:{majorTickLines:{visible:false}}} ); 
                    * @alias ejChart#secondaryY->majorTickLines->visible
                   * @instance
                   */
                            visible: true
                        },
                    /**		
                * Specifies the minorGridLines. These are properties involved in customizing the minorGridLines of chart. 
                * @type {object}
                 * @alias ejChart#secondaryY->minorGridLines
                * @instance
                */
                    minorGridLines:
                        {
                            /**		
                   * Specifies the width of the minorGridLines. The user can modify the width of the minorGridLines in secondaryY.
                   * @default 1
                   * @type {number}
                   * @example 
                   * //To set width of minorGridLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{minorGridLines:{width:0.5}}});				
                   * @example 
                   * //Gets or sets the width of the minorGridLines, after initialization:<br>
                   *	//Gets the width of the minorGridLines  
                   *	$("#container").ejChart("option", "secondaryY.minorGridLines.width");<br>			
                   *	//Sets the width of the minorGridLines
                   *	$("#container").ejChart("option", {secondaryY:{minorGridLines:{width:0.5}}} ); 
                   * @alias ejChart#secondaryY->minorGridLines->width
                   * @instance
                   */
                            width: 1,
                            /**		
                   * Specifies the dashArray of the minorGridLines. The user can modify the dashArray of the minorGridLines in secondaryY.
                   * @default null
                   * @type {string}
                   * @example 
                   * //To set dashArray of minorGridLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{minorGridLines:{dashArray:"2,3"}}});				
                   * @example 
                   * //Gets or sets the dashArray of the minorGridLines, after initialization:<br>
                   *	//Gets the dashArray of the minorGridLines  
                   *	$("#container").ejChart("option", "secondaryY.minorGridLines.dashArray");<br>			
                   *	//Sets the dashArray of the minorGridLines
                   *	$("#container").ejChart("option", {secondaryY:{minorGridLines:{dashArray:"2,3"}}} ); 
                   * @alias ejChart#secondaryY->minorGridLines->dashArray
                   * @instance
                   */
                            dashArray: "",
                            /**		
                   * Specifies the visibility of the minorGridLines. The user can visibility the width of the minorGridLines in secondaryY.
                   * @default false
                   * @type {boolean}
                   * @example 
                   * //To set visibility of minorGridLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{minorGridLines:{visible:true}}});				
                   * @example 
                   * //Gets or sets the visibility of the minorGridLines, after initialization:<br>
                   *	//Gets the visibility of the minorGridLines  
                   *	$("#container").ejChart("option", "secondaryY.minorGridLines.visible");<br>			
                   *	//Sets the visibility of the minorGridLines
                   *	$("#container").ejChart("option", {secondaryY:{minorGridLines:{visible:true}}} ); 
                   * @alias ejChart#secondaryY->minorGridLines->visible
                   * @instance
                   */
                            visible: false

                        },
                    /**		
                * Specifies the minorTickLines. These are properties involved in customizing the minorTickLines of chart. 
                * @type {object}
                * @alias ejChart#secondaryY->minorTickLines
                * @instance
                */
                    minorTickLines:
                        {
                            /**		
                   * Specifies the width of the minorTickLines. The user can modify the width of the minorTickLines in secondaryY.
                   * @default 1
                   * @type {number}
                   * @example 
                   * //To set width of minorTickLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{minorTickLines:{width:2}}});				
                   * @example 
                   * //Gets or sets the width of the minorTickLines, after initialization:<br>
                   *	//Gets the width of the minorTickLines  
                   *	$("#container").ejChart("option", "secondaryY.minorTickLines.width");<br>			
                   *	//Sets the width of the minorTickLines
                   *	$("#container").ejChart("option", {secondaryY:{minorTickLines:{width:2}}} ); 
                   * @alias ejChart#secondaryY->minorTickLines->width
                   * @instance
                   */
                            width: 1,
                            /**		
                   * Specifies the size of the minorTickLines. The user can modify the size of the minorTickLines in secondaryY.
                   * @default 5
                   * @type {number}
                   * @example 
                   * //To set size of minorTickLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{minorTickLines:{size:4}}});				
                   * @example 
                   * //Gets or sets the size of the minorTickLines, after initialization:<br>
                   *	//Gets the size of the minorTickLines  
                   *	$("#container").ejChart("option", "secondaryY.minorTickLines.size");<br>			
                   *	//Sets the size of the minorTickLines
                   *	$("#container").ejChart("option", {secondaryY:{minorTickLines:{size:4}}} ); 
                  * @alias ejChart#secondaryY->minorTickLines->size
                   * @instance
                   */
                            size: 5,
                            /**		
                   * Specifies the visibility of the minorTickLines. The user can modify the visibility of the minorTickLines in secondaryY.
                   * @default false
                   * @type {boolean}
                   * @example 
                   * //To set visibility of minorTickLines value during initialization  
                   * 	$("#container").ejChart({secondaryY:{minorTickLines:{visible:true}}});				
                   * @example 
                   * //Gets or sets the visibility of the minorTickLines, after initialization:<br>
                   *	//Gets the visibility of the minorTickLines  
                   *	$("#container").ejChart("option", "secondaryY.minorTickLines.visible");<br>			
                   *	//Sets the visibility of the minorTickLines
                   *	$("#container").ejChart("option", {secondaryY:{minorTickLines:{visible:true}}} ); 
                   * @alias ejChart#secondaryY->minorTickLines->visible
                   * @instance
                   */
                            visible: false
                        },
                    /**		
                * Specifies the minorTicksPerInterval of secondaryY. This property is to set the number of minor ticks to render per interval
                * @default null
                * @type {number}
                * @example 
                * //To set minorTicksPerInterval of secondaryY value during initialization  
                * 	$("#container").ejChart({secondaryY:{minorTicksPerInterval:2}});				
                * @example 
                * //Gets or sets the minorTicksPerInterval of secondaryY, after initialization:<br>
                *	//Gets the minorTicksPerInterval of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.minorTicksPerInterval");<br>			
                *	//Sets the minorTicksPerInterval of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{minorTicksPerInterval:2}} ); 
                * @alias ejChart#secondaryY->minorTicksPerInterval
                * @instance
                */
                    minorTicksPerInterval: null,
                    /**		
                * Specifies the rowIndex of secondaryY. This is to specify the index of the row.
                * @default null
                * @type {number}
                * @example 
                * //To set rowIndex of secondaryY value during initialization  
                * 	$("#container").ejChart({secondaryY:{rowIndex:2}});				
                * @example 
                * //Gets or sets the rowIndex of secondaryY, after initialization:<br>
                *	//Gets the rowIndex of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.rowIndex");<br>			
                *	//Sets the rowIndex of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{rowIndex:2}} ); 
                * @alias ejChart#secondaryY->rowIndex
                * @instance
                */
                    rowIndex: null,
                    /**		
                * Specifies the rowSpan of secondaryY. EjChart provides option to span the row. This is achieved by using this property.
                * @default null
                * @type {number}
                * @example 
                * //To set rowSpan of secondaryY value during initialization  
                * 	$("#container").ejChart({secondaryY:{rowSpan:2}});				
                * @example 
                * //Gets or sets the rowSpan of secondaryY, after initialization:<br>
                *	//Gets the rowSpan of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.rowSpan");<br>			
                *	//Sets the rowSpan of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{rowSpan:2}} ); 
                * @alias ejChart#secondaryY->rowSpan
                * @instance
                */
                    rowSpan: null,
                    /**		
                * Specifies the valueType of secondaryY. It denotes the type of axis. It can be double, datetime, etc. See {@link valueType}
                * @default null
                * @type {enum}
                * @example 
                * //To set valueType of secondaryY
                * 	$("#container").ejChart({secondaryY:{valueType:"Double"}});				
                * @example 
                * //Gets or sets the valueType of secondaryY, after initialization:<br>
                *	//Gets the valueType of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.valueType");<br>			
                *	//Sets the valueType of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{valueType:"Double"}} ); 
                * @alias ejChart#secondaryY->valueType
                * @instance
                */
                    valueType: null,
                    /**		
                * Specifies the axisName of secondaryY. This is the name to represent the axis in chart.
                * @default null
                * @type {string}
                * @example 
                * //To set axisName of secondaryY
                * 	$("#container").ejChart({secondaryY:{axisName:"xaxis"}});			 
                * @example 
                * //Gets or sets the axisName of secondaryY, after initialization:<br>
                *	//Gets the axisName of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.axisName");<br>			
                *	//Sets the axisName of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{axisName:"xaxis"}} ); 
                * @alias ejChart#secondaryY->axisName
                * @instance
                */
                    axisName: null,
                    /**		
                * Specifies the labelFormat of secondaryY. The axis labels render in the format, which is specified here.
                * @default null
                * @type {string}
                * @example 
                * //To set labelFormat of secondaryY
                * 	$("#container").ejChart({secondaryY:{labelFormat:"${value}"}});				
                * @example 
                * //Gets or sets the labelFormat of secondaryY, after initialization:<br>
                *	//Gets the labelFormat of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.labelFormat");<br>			
                *	//Sets the labelFormat of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{labelFormat:"${value}"}} ); 
                * @alias ejChart#secondaryY->labelFormat
                * @instance
                */
                    labelFormat: null,
                    /**		
                * Specifies the desiredIntervals of secondaryY. By this property we can display the labels after appropriate intervals.
                * @default null
                * @type {number}
                * @example 
                * //To set desiredIntervals of secondaryY
                * 	$("#container").ejChart({secondaryY:{desiredIntervals:5}});				
                * @example 
                * //Gets or sets the desiredIntervals of secondaryY, after initialization:<br>
                *	//Gets the desiredIntervals of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.desiredIntervals");<br>			
                *	//Sets the desiredIntervals of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{desiredIntervals:5}} ); 
                * @alias ejChart#secondaryY->desiredIntervals
                * @instance
                */
                    desiredIntervals: null,
                    /**		
                * Specifies the intervalType of secondaryY. This is to set the type of intervals to chart. IntervalType may be days, years, etc. See {@link intervalType}
                * @default null
                * @type {enum}
                * @example 
                * //To set desiredIntervintervalTypeals of secondaryY
                * 	$("#container").ejChart({secondaryY:{intervalType:"Years"}});				
                * @example 
                * //Gets or sets the intervalType of secondaryY, after initialization:<br>
                *	//Gets the intervalType of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.intervalType");<br>			
                *	//Sets the intervalType of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{intervalType:"Years"}} ); 
               * @alias ejChart#secondaryY->intervalType
                * @instance
                */
                    intervalType: null,
                    /**		
                * Specifies the roundingPlaces of secondaryY. This property is to truncate the decimal values after the specified position.
                * @default null
                * @type {number}
                * @example 
                * //To set roundingPlaces of secondaryY
                * 	$("#container").ejChart({secondaryY:{roundingPlaces:3}});				
                * @example 
                * //Gets or sets the roundingPlaces of secondaryY, after initialization:<br>
                *	//Gets the roundingPlaces of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.roundingPlaces");<br>			
                *	//Sets the roundingPlaces of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{roundingPlaces:3}} ); 
                * @alias ejChart#secondaryY->roundingPlaces
                * @instance
                */
                    roundingPlaces: null,
                    labels: [],
                    /**		
                * Specifies the title. This is to be rendered as title for secondaryY.
                * @type {object}
                * @alias ejChart#secondaryY->title
                * @instance
                */
                    title:
                    {
                        /**		
                * Specifies the title text of secondaryY. This is the text which is to be rendered as title for secondaryY.
                * @default ""
                * @type {string}
                * @example 
                * //To set title text of secondaryY
                * 	$("#container").ejChart({secondaryY:{ title:{ text:"Years"}}});				 
                * @example 
                * //Gets or sets the title text of secondaryY, after initialization:<br>
                *	//Gets the title text of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.title.text");<br>			
                *	//Sets the title text of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{ title:{text:"Years"}}} ); 
                * @alias ejChart#secondaryY->title->text
                * @instance
                */
                        text: "",
                        /**		
                * Specifies the font. These font properties are customization are applied for title text.
                * @type {object}
                * @alias ejChart#secondaryY->title->font
                * @instance
                */
                        font:
                        {
                            /**		
                   * Specifies the title fontfamily of secondaryY. This fontFamily is applied to the title text of secondaryY.
                   * @default "Segoe UI"
                   * @type {string}
                   * @example 
                   * //To set title fontfamily of secondaryY
                   * 	$("#container").ejChart({secondaryY:{ title:{ font:{fontFamily:"Algerian"}}}});				
                   * @example 
                   * //Gets or sets the title fontfamily of secondaryY, after initialization:<br>
                   *	//Gets the title fontfamily of secondaryY  
                   *	$("#container").ejChart("option", "secondaryY.title.font.fontFamily");<br>			
                   *	//Sets the title fontfamily of secondaryY
                   *	$("#container").ejChart("option", {secondaryY:{ title:{font:{fontFamily:"Algerian"}}}} ); 
                   * @alias ejChart#secondaryY->title->font->fontFamily
                   * @instance
                   */
                            fontFamily: 'Segoe UI',
                            /**		
                   * Specifies the title fontStyle of secondaryY. This fontStyle is applied to the title text of secondaryY. See {@link fontStyle}
                   * @default ej.Chart.fontStyle.Normal
                   * @type {enum}
                   * @example 
                   * //To set title fontStyle of secondaryY
                   * 	$("#container").ejChart({secondaryY:{ title:{ font:{fontStyle:"Italic"}}}});				 
                   * @example 
                   * //Gets or sets the title fontStyle of secondaryY, after initialization:<br>
                   *	//Gets the title fontStyle of secondaryY  
                   *	$("#container").ejChart("option", "secondaryY.title.font.fontStyle");<br>			
                   *	//Sets the title fontStyle of secondaryY
                   *	$("#container").ejChart("option", {secondaryY:{ title:{font:{fontStyle:"Italic"}}}} ); 
                   * @alias ejChart#secondaryY->title->font->fontStyle
                   * @instance
                   */
                            fontStyle: 'Normal',
                            /**		
                   * Specifies the title opacity of secondaryY.  This font size is applied to the title text of secondaryY.
                   * @default 1
                   * @type {number}
                   * @example 
                   * //To set title opacity of secondaryY
                   * 	$("#container").ejChart({secondaryY:{ title:{ font:{opacity:0.5}}}});				 
                   * @example 
                   * //Gets or sets the title opacity of secondaryY, after initialization:<br>
                   *	//Gets the title opacity of secondaryY  
                   *	$("#container").ejChart("option", "secondaryY.title.font.opacity");<br>			
                   *	//Sets the title opacity of secondaryY
                   *	$("#container").ejChart("option", {secondaryY:{ title:{font:{opacity:0.5}}}} ); 
                   * @alias ejChart#secondaryY->title->font->opacity
                   * @instance
                   */
                            opacity: 1,
                            /**		
                   * Specifies the title size of secondaryY. This opacity value is applied to the title text of secondaryY.
                   * @default "16px"
                   * @type {string}
                   * @example 
                   * //To set title size of secondaryY
                   * 	$("#container").ejChart({secondaryY:{ title:{ font:{size:"13px"}}}});				 
                   * @example 
                   * //Gets or sets the title size of secondaryY, after initialization:<br>
                   *	//Gets the title size of secondaryY  
                   *	$("#container").ejChart("option", "secondaryY.title.font.size");<br>			
                   *	//Sets the title size of secondaryY
                   *	$("#container").ejChart("option", {secondaryY:{ title:{font:{size:"13px"}}}} ); 
                  * @alias ejChart#secondaryY->title->font->size
                   * @instance
                   */
                            size: '16px',
                            /**		
                   * Specifies the title fontWeight of secondaryY.This fontWeight is applied to the title text of secondaryY. See {@link fontWeight}
                   * @default ej.Chart.fontWeight.Regular
                   * @type {enum}
                   * @example 
                   * //To set title fontWeight of secondaryY
                   * 	$("#container").ejChart({secondaryY:{ title:{ font:{fontWeight:"lighter"}}}});				 
                   * @example 
                   * //Gets or sets the title fontWeight of secondaryY, after initialization:<br>
                   *	//Gets the title fontWeight of secondaryY  
                   *	$("#container").ejChart("option", "secondaryY.title.font.fontWeight");<br>			
                   *	//Sets the title fontWeight of secondaryY
                   *	$("#container").ejChart("option", {secondaryY:{ title:{font:{fontWeight:"lighter"}}}} ); 
                   * @alias ejChart#secondaryY->title->font->fontWeight
                   * @instance
                   */
                            fontWeight: 'regular'
                        }
                    },
                    /**		
                * Specifies the rangePadding of secondaryY.  Specify the spacing of the chart axis for auto range calculation
None: Range of axis is calculated from the minimum value to the maximum value in datasource
Normal: Range of axis is calculated based on range calculation,(for numerical axis only)
Additional: Range of axis is calculated by adding interval to minimum and  maximum value in datasource.
Round: Round the range of axis to the nearest possible value. See {@link rangePadding}
                * @default ej.Chart.rangePadding.None
                * @type {enum}
                * @example 
                * //To set rangePadding of secondaryY
                * 	$("#container").ejChart({secondaryY:{rangePadding:"none"}});				 
                * @example 
                * //Gets or sets the rangePadding of secondaryY, after initialization:<br>
                *	//Gets the rangePadding of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.rangePadding");<br>			
                *	//Sets the rangePadding of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{rangePadding:"none"}} ); 
                * @alias ejChart#secondaryY->rangePadding
                * @instance
                */
                    rangePadding: 'Normal',

                    stripline: [],
                    /**		
                * Specifies the logBase of secondaryY. LogBase is used in log axis to render certain values.
                * @default 10
                * @type {number}
                * @example 
                * //To set logBase of secondaryY
                * 	$("#container").ejChart({secondaryY:{logBase:5}});				 
                * @example 
                * //Gets or sets the logBase of secondaryY, after initialization:<br>
                *	//Gets the logBase of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.logBase");<br>			
                *	//Sets the logBase of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{logBase:5}} ); 
                * @alias ejChart#secondaryY->logBase
                * @instance
                */
                    logBase: 10,
                    /**		
                * Specifies the plotOffset of secondaryY.  This specifies the offset.
                * @default 10
                * @type {number}
                * @example 
                * //To set plotOffset of secondaryY
                * 	$("#container").ejChart({secondaryY:{plotOffset:1}});				 
                * @example 
                * //Gets or sets the plotOffset of secondaryY, after initialization:<br>
                *	//Gets the plotOffset of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.plotOffset");<br>			
                *	//Sets the plotOffset of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{plotOffset:1}} ); 
               * @alias ejChart#secondaryY->plotOffset
                * @instance
                */
                    plotOffset: 0,
                    /**		
                * Specifies the orientation of secondaryY.  This is to set the orientation for chart axis. The user can set either vertical or horizontal. See {@link orientation}
                * @default ej.Chart.orientation.Horizontal
                * @type {enum}
                * @example 
                * //To set orientation of secondaryY
                * 	$("#container").ejChart({secondaryY:{orientation:"Horizontal"}});				 
                * @example 
                * //Gets or sets the orientation of secondaryY, after initialization:<br>
                *	//Gets the orientation of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.orientation");<br>			
                *	//Sets the orientation of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{orientation:"Horizontal"}} ); 
                * @alias ejChart#secondaryY->orientation
                * @instance
                */
                    orientation: 'Vertical',
                    /**		
                * Specifies the maximumLabels of secondaryY. This specifies the maximum number of labels to be rendered on the axis.
                * @default 3
                * @type {number}
                * @example 
                * //To set maximumLabels of secondaryY
                * 	$("#container").ejChart({secondaryY:{maximumLabels:5}});				 
                * @example 
                * //Gets or sets the maximumLabels of secondaryY, after initialization:<br>
                *	//Gets the maximumLabels of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.maximumLabels");<br>			
                *	//Sets the maximumLabels of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{maximumLabels:5}} ); 
                * @alias ejChart#secondaryY->maximumLabels
                * @instance
                */
                    maximumLabels: 3,
                    /**		
                * Specifies the hidePartialLabels of secondaryY.  By applying this property the user can hide the partially visible labels.
                * @default false
                * @type {boolean}
                * @example 
                * //To set hidePartialLabels of secondaryY
                * 	$("#container").ejChart({secondaryY:{hidePartialLabels:true}});				 
                * @example 
                * //Gets or sets the hidePartialLabels of secondaryY, after initialization:<br>
                *	//Gets the hidePartialLabels of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.hidePartialLabels");<br>			
                *	//Sets the hidePartialLabels of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{hidePartialLabels:true}} ); 
                * @alias ejChart#secondaryY->hidePartialLabels
                * @instance
                */
                    hidePartialLabels: false,
                    /**		
                * Specifies the crosshairLabel of secondaryY.  These are the properties relate to crosshair Label.
                * @type {object}
               * @alias ejChart#secondaryY->crosshairLabel
                * @instance
                */
                    crosshairLabel:
                    {
                        /**		
                 * Specifies the visibility of crosshairLabel. This is the property to change the visibility of the axis labels. 
                 * @default false
                 * @type {boolean}
                 * @example 
                 * //To set visibility of crosshairLabel
                 * 	$("#container").ejChart({secondaryY:{crosshairLabel:{visible:true}}});				
                 * @example 
                 * //Gets or sets the visibility of crosshairLabel, after initialization:<br>
                 *	//Gets the visibility of crosshairLabel  
                 *	$("#container").ejChart("option", "secondaryY.crosshairLabel.visible");<br>			
                 *	//Sets the visibility of crosshairLabel
                 *	$("#container").ejChart("option", {secondaryY:{crosshairLabel:{visible:true}}} ); 
                 * @alias ejChart#secondaryY->crosshairLabel->visible
                 * @instance
                 */
                        visible: false
                    },
                    /**		
                * Specifies the visibility of secondaryY. 
                * @default true
                * @type {boolean}
                * @example 
                * //To set visibility of secondaryY
                * 	$("#container").ejChart({secondaryY:{visible:false}});				 
                * @example 
                * //Gets or sets the visibility of secondaryY, after initialization:<br>
                *	//Gets the visibility of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.visible");<br>			
                *	//Sets the visibility of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{visible:false}} ); 
                * @alias ejChart#secondaryY->visible
                * @instance
                */
                    visible: true,
                    /**		
                * Specifies the opposedPosition of secondaryY. This is to render the axis in opposed position or not. 
                * @default false
                * @type {boolean}
                * @example 
                * //To set opposedPosition of secondaryY
                * 	$("#container").ejChart({secondaryY:{opposedPosition:true}});				
                * @example 
                * //Gets or sets the opposedPosition of secondaryY, after initialization:<br>
                *	//Gets the opposedPosition of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.opposedPosition");<br>			
                *	//Sets the opposedPosition of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{opposedPosition:true}} ); 
               * @alias ejChart#secondaryY->opposedPosition
                * @instance
                */
                    opposedPosition: false,
                    /**		
                * Specifies the font of secondaryY. These font properties are applied for the axis labels. 
                * @type {object}
                * @alias ejChart#secondaryY->font
                * @instance
                */
                    font:
                    {
                        /**		
                 * Specifies the fontfamily of secondaryY. The axis labels render with this specified fontFamily.
                 * @default "Segoe UI"
                 * @type {string}
                 * @example 
                 * //To set fontfamily of secondaryY
                 * 	$("#container").ejChart({secondaryY:{ font:{fontFamily:"Algerian"}}});				 
                 * @example 
                 * //Gets or sets the fontfamily of secondaryY, after initialization:<br>
                 *	//Gets the fontfamily of secondaryY  
                 *	$("#container").ejChart("option", "secondaryY.font.fontFamily");<br>			
                 *	//Sets the fontfamily of secondaryY
                 *	$("#container").ejChart("option", {secondaryY:{ font:{fontFamily:"Algerian"}}} ); 
                * @alias ejChart#secondaryY->font->fontFamily
                 * @instance
                 */
                        fontFamily: 'Segoe UI',
                        /**		
                 * Specifies the fontStyle of secondaryY. The axis labels render with this specified fontStyle. See {@link fontStyle}
                 * @default ej.Chart.fontStyle.Normal
                 * @type {enum}
                 * @example 
                 * //To set fontStyle of secondaryY
                 * 	$("#container").ejChart({secondaryY:{ font:{fontStyle:"Italic"}}});				 
                 * @example 
                 * //Gets or sets the fontStyle of secondaryY, after initialization:<br>
                 *	//Gets the fontStyle of secondaryY  
                 *	$("#container").ejChart("option", "secondaryY.font.fontStyle");<br>			
                 *	//Sets the fontStyle of secondaryY
                 *	$("#container").ejChart("option", {secondaryY:{font:{fontStyle:"Italic"}}} ); 
                  * @alias ejChart#secondaryY->font->fontStyle
                 * @instance
                 */
                        fontStyle: 'Normal',
                        /**		
                  * Specifies the title size of secondaryY. The axis labels render with this specified size.
                  * @default "13px"
                  * @type {string}
                  * @example 
                  * //To set size of secondaryY
                  * 	$("#container").ejChart({secondaryY:{font:{size:"12px"}}});				
                  * @example 
                  * //Gets or sets the size of secondaryY, after initialization:<br>
                  *	//Gets the size of secondaryY  
                  *	$("#container").ejChart("option", "secondaryY.font.size");<br>			
                  *	//Sets the size of secondaryY
                  *	$("#container").ejChart("option", {secondaryY:{font:{size:"12px"}}} ); 
                   * @alias ejChart#secondaryY->font->size
                  * @instance
                  */
                        size: '13px',
                        /**		
                 * Specifies the title opacity of secondaryY. The axis labels render with this specified opacity.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set title opacity of secondaryY
                 * 	$("#container").ejChart({secondaryY:{ title:{ font:{opacity:0.5}}}});				
                 * @example 
                 * //Gets or sets the title opacity of secondaryY, after initialization:<br>
                 *	//Gets the title opacity of secondaryY  
                 *	$("#container").ejChart("option", "secondaryY.title.font.opacity");<br>			
                 *	//Sets the title opacity of secondaryY
                 *	$("#container").ejChart("option", {secondaryY:{ title:{font:{opacity:0.5}}}} ); 
                 * @alias ejChart#secondaryY->font->opacity
                 * @instance
                 */
                        opacity: 1,
                        /**		
                 * Specifies the fontWeight of secondaryY. The axis labels render with this specified fontWeight. See {@link fontWeight}
                 * @default ej.Chart.fontWeight.Regular
                 * @type {enum}
                 * @example 
                 * //To set fontWeight of secondaryY
                 * 	$("#container").ejChart({secondaryY:{font:{fontWeight:"lighter"}}});				 
                 * @example 
                 * //Gets or sets the fontWeight of secondaryY, after initialization:<br>
                 *	//Gets the fontWeight of secondaryY  
                 *	$("#container").ejChart("option", "secondaryY.font.fontWeight");<br>			
                 *	//Sets the fontWeight of secondaryY
                 *	$("#container").ejChart("option", {secondaryY:{font:{fontWeight:"lighter"}}} ); 
                 * @alias ejChart#secondaryY->font->fontWeight
                 * @instance
                 */
                        fontWeight: 'regular'
                    },
                    /**		
                * Specifies the axisLine. This line is drawn in chart to represent the end of the axis.
                * @type {object}
                * @alias ejChart#secondaryY->axisLine
                * @instance
                */
                    axisLine:
                        {
                            /**		
                     * Specifies the visibility of axisLine in secondaryY. The user can change the visibility of the axisLine.
                     * @default true
                     * @type {boolean}
                     * @example 
                     * //To set visibility of axisLine in secondaryY
                     * 	$("#container").ejChart({secondaryY:{axisLine:{visible:false}}});				
                     * @example 
                     * //Gets or sets the visibility of axisLine in secondaryY, after initialization:<br>
                     *	//Gets the visibility of axisLine in secondaryY  
                     *	$("#container").ejChart("option", "secondaryY.axisLine.visible");<br>			
                     *	//Sets the visibility of axisLine in secondaryY
                     *	$("#container").ejChart("option", {secondaryY:{axisLine:{visible:false}}} ); 
                      * @alias ejChart#secondaryY->axisLine->visible
                     * @instance
                     */
                            visible: true,
                            /**		
                     * Specifies the width of axisLine in secondaryY. The user can change the width of the axisLine.
                     * @default 1
                     * @type {number}
                     * @example 
                     * //To set width of axisLine in secondaryY
                     * 	$("#container").ejChart({secondaryY:{axisLine:{width:2}}});				
                     * @example 
                     * //Gets or sets the width of axisLine in secondaryY, after initialization:<br>
                     *	//Gets the width of axisLine in secondaryY  
                     *	$("#container").ejChart("option", "secondaryY.axisLine.width");<br>			
                     *	//Sets the width of axisLine in secondaryY
                     *	$("#container").ejChart("option", {secondaryY:{axisLine:{width:2}}} ); 
                     * @alias ejChart#secondaryY->axisLine->width
                     * @instance
                     */
                            width: 1,
                            /**		
                     * Specifies the dashArray of axisLine in secondaryY. The user can change the dashArray of the axisLine.
                     * @default null
                     * @type {string}
                     * @example 
                     * //To set dashArray of axisLine in secondaryY
                     * 	$("#container").ejChart({secondaryY:{axisLine:{dashArray:"2,3"}}});			 
                     * @example 
                     * //Gets or sets the dashArray of axisLine in secondaryY, after initialization:<br>
                     *	//Gets the dashArray of axisLine in secondaryY  
                     *	$("#container").ejChart("option", "secondaryY.axisLine.dashArray");<br>			
                     *	//Sets the dashArray of axisLine in secondaryY
                     *	$("#container").ejChart("option", {secondaryY:{axisLine:{dashArray:"2,3"}}} ); 
                     * @alias ejChart#secondaryY->axisLine->dashArray
                     * @instance
                     */
                            dashArray: "",
                            /**		
                     * Specifies the offset of axisLine in secondaryY. The user can adjust the position of the axis line with the specified offset.
                     * @default null
                     * @type {string}
                     * @example 
                     * //To set offset of axisLine in secondaryY
                     * 	$("#container").ejChart({secondaryY:{axisLine:{offset:1}}});				
                     * @example 
                     * //Gets or sets the offset of axisLine in secondaryY, after initialization:<br>
                     *	//Gets the offset of axisLine in secondaryY  
                     *	$("#container").ejChart("option", "secondaryY.axisLine.offset");<br>			
                     *	//Sets the offset of axisLine in secondaryY
                     *	$("#container").ejChart("option", {secondaryY:{axisLine:{offset:1}}} ); 
                    * @alias ejChart#secondaryY->axisLine->offset
                     * @instance
                     */
                            offset: 0
                        },

                    /**		
                * Specifies the zoomFactor of secondaryY. This is to specify zoom factor value for axis zooming. The user can specify value from 0 to 1.
                * @default 1
                * @type {number}
                * @example 
                * //To set zoomFactor of secondaryY
                * 	$("#container").ejChart({secondaryY:{zoomFactor:0}});				
                * @example 
                * //Gets or sets the zoomFactor of secondaryY, after initialization:<br>
                *	//Gets the zoomFactor of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.zoomFactor");<br>			
                *	//Sets the zoomFactor of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{zoomFactor:0}} ); 
                * @alias ejChart#secondaryY->zoomFactor
                * @instance
                */
                    zoomFactor: 1,
                    /**		
                * Specifies the zoomPosition of secondaryY. This is to specify zoom position value for axis zooming. The user can specify value from 0 to 1.
                * @default 0
                * @type {number}
                * @example 
                * //To set zoomPosition of secondaryY
                * 	$("#container").ejChart({secondaryY:{zoomPosition:1}});				
                * @example 
                * //Gets or sets the zoomPosition of secondaryY, after initialization:<br>
                *	//Gets the zoomPosition of secondaryY  
                *	$("#container").ejChart("option", "secondaryY.zoomPosition");<br>			
                *	//Sets the zoomPosition of secondaryY
                *	$("#container").ejChart("option", {secondaryY:{zoomPosition:1}} ); 
                * @alias ejChart#secondaryY->zoomPosition
                * @instance
                */
                    zoomPosition: 0
                },
            /**		
           * Specifies the default values of stripline. These properties are involved in customizing the stripLine for chart.
           * @type {object}
            * @memberof ejChart
           * @instance
           */
            striplineDefault: {
                /**		
* Specifies the visibility of stripline. By this property the user can change the visibility of the stipline.
* @default true
* @type {boolean}
* @example 
* //To set visibility of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{visible: false }]}});				 
* @example 
* //Gets or sets the visibility of stripline, after initialization:<br>
*	//Gets the visibility of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].visible;		
*	//Sets the visibility of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{visible: false }]}} ); 
 * @alias ejChart#striplineDefault->visible
* @instance
*/
                visible: true,
                /**		
* Specifies the startFromAxis of stripline. Specifies that the stripLine will start from the orgin of the axis
* @default false
* @type {boolean}
* @example 
* //To set startFromAxis of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{startFromAxis: true }]}});				
* @example 
* //Gets or sets the startFromAxis of stripline, after initialization:<br>
*	//Gets the startFromAxis of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].startFromAxis;		
*	//Sets the startFromAxis of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{startFromAxis: true }]}} ); 
 * @alias ejChart#striplineDefault->startFromAxis
* @instance
*/
                startFromAxis: false,
                /**		
* Specifies the text of stripline. This defines the text of the stripline which is displayed on the stripline
* @default "stripline"
* @type {string}
* @example 
* //To set text of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{text: "striplines" }]}});				
* @example 
* //Gets or sets the text of stripline, after initialization:<br>
*	//Gets the text of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].text;		
*	//Sets the text of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{text: "striplines" }]}} ); 
 * @alias ejChart#striplineDefault->text
* @instance
*/
                text: "stripline",
                /**		
* Specifies the textAlignment of stripline. This property is to change the alignment of the stripline.
* @default "middlecenter"
* @type {enum}
* @example 
* //To set textAlignment of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{textAlignment: "middlecenter" }]}});				 
* @example 
* //Gets or sets the textAlignment of stripline, after initialization:<br>
*	//Gets the textAlignment of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].textAlignment;		
*	//Sets the textAlignment of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{textAlignment: "middlecenter" }]}} ); 
 * @alias ejChart#striplineDefault->textAlignment
* @instance
*/
                textAlignment: 'middlecenter',
                /**		
         * Specifies the  font of stripline. These are the font customization done on he text of the stripline.
         * @type {object}
           * @alias ejChart#striplineDefault->font
         * @instance
         */
                font:
                {
                    /**		
* Specifies the fontFamily of stripline. Stripline text render with this specified fontFamily.
* @default "Segoe UI"
* @type {string}
* @example 
* //To set fontFamily of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{font:{fontFamily: "Algerian" }}]}});				 
* @example 
* //Gets or sets the fontFamily of stripline, after initialization:<br>
*	//Gets the fontFamily of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].font.fontFamily;		
*	//Sets the fontFamily of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{font:{fontFamily: "Algrian" }}]}} ); 
* @alias ejChart#striplineDefault->font->fontFamily
* @instance
*/
                    fontFamily: 'Segoe UI',
                    /**		
* Specifies the fontStyle of stripline. Stripline text render with this specified fontStyle. See {@link fontStyle}
* @default ejChart.fontStyle.Normal
* @type {enum}
* @example 
* //To set fontStyle of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{font:{fontStyle: "Italic" }}]}});				 
* @example 
* //Gets or sets the fontStyle of stripline, after initialization:<br>
*	//Gets the fontStyle of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].font.fontStyle;		
*	//Sets the fontStyle of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{font:{fontStyle: "Italic" }}]}} ); 
* @alias ejChart#striplineDefault->font->fontStyle
* @instance
*/
                    fontStyle: 'Normal',
                    /**		
* Specifies the font size of stripline.  Stripline text render with this specified font size.
* @default "Segoe UI"
* @type {string}
* @example 
* //To set font size of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{font:{size: "10px" }}]}});				 
* @example 
* //Gets or sets the font size of stripline, after initialization:<br>
*	//Gets the font size of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].font.size;		
*	//Sets the font size of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{font:{size: "10px" }}]}} ); 
* @alias ejChart#striplineDefault->font->size
* @instance
*/
                    size: '12px',
                    /**		
* Specifies the font color of stripline.  Stripline text render with this specified font color.
* @default "Segoe UI"
* @type {string}
* @example 
* //To set color of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{font:{size: "10px" }}]}});				 
* @example 
* //Gets or sets the color of stripline, after initialization:<br>
*	//Gets the color of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].font.size;		
*	//Sets the color of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{font:{size: "10px" }}]}} ); 
* @alias ejChart#striplineDefault->font->color
* @instance
*/
                    color: 'black',
                    /**		
* Specifies the opacity of stripline.  Stripline text render with this specified opacity.
* @default 1
* @type {number}
* @example 
* //To set opacity of stripline
* 	$("#container").ejChart({primaryXAxis:{stripline:[{font:{opacity: 0.5 }}]}});				 
* @example 
* //Gets or sets the opacity of stripline, after initialization:<br>
*	//Gets the opacity of stripline 
*   $("#container").ejChart("option", "primaryXAxis.stripline")[0].font.opacity;		
*	//Sets the opacity of stripline
*	$("#container").ejChart("option", {primaryXAxis:{stripline:[{font:{opacity: 0.5 }}]}} ); 
* @alias ejChart#striplineDefault->font->opacity
* @instance
*/
                    opacity: 1
                },
                /**		
  * Specifies the start of stripline. This determine the start value of stripLine
  * @default null
  * @type {number}
  * @example 
  * //To set start of stripline
  * 	$("#container").ejChart({primaryXAxis:{stripline:[{start: 0.5 }]}});				
  * @example 
  * //Gets or sets the start of stripline, after initialization:<br>
  *	//Gets the start of stripline 
  *   $("#container").ejChart("option", "primaryXAxis.stripline")[0].start;		
  *	//Sets the start of stripline
  *	$("#container").ejChart("option", {primaryXAxis:{stripline:[{start: 0.5 }]}} ); 
  * @alias ejChart#striplineDefault->start
  * @instance
  */
                start: null,
                /**		
  * Specifies the end of stripline. This determine the end value of stripLine
  * @default null
  * @type {number}
  * @example 
  * //To set end of stripline
  * 	$("#container").ejChart({primaryXAxis:{stripline:[{end: 1 }]}});				 
  * @example 
  * //Gets or sets the end of stripline, after initialization:<br>
  *	//Gets the end of stripline 
  *   $("#container").ejChart("option", "primaryXAxis.stripline")[0].end;		
  *	//Sets the end of stripline
  *	$("#container").ejChart("option", {primaryXAxis:{stripline:[{end: 1 }]}} ); 
  * @alias ejChart#striplineDefault->end
  * @instance
  */
                end: null,
                /**		
        * Specifies the color of stripline. This determine the color of stripLine
        * @default "gray"
        * @type {string}
        * @example 
        * //To set color of stripline
        * 	$("#container").ejChart({primaryXAxis:{stripline:[{color: "green" }]}});				 
        * @example 
        * //Gets or sets the color of stripline, after initialization:<br>
        *	//Gets the color of stripline 
        *   $("#container").ejChart("option", "primaryXAxis.stripline")[0].color;		
        *	//Sets the color of stripline
        *	$("#container").ejChart("option", {primaryXAxis:{stripline:[{color: "green" }]}} ); 
       * @alias ejChart#striplineDefault->color
        * @instance
        */
                color: 'gray',
                /**		
       * Specifies the borderColor of stripline. This determines the border color of stripLine
       * @default "black"
       * @type {string}
       * @example 
       * //To set borderColor of stripline
       * 	$("#container").ejChart({primaryXAxis:{stripline:[{borderColor: "green" }]}});				 
       * @example 
       * //Gets or sets the borderColor of stripline, after initialization:<br>
       *	//Gets the borderColor of stripline 
       *   $("#container").ejChart("option", "primaryXAxis.stripline")[0].borderColor;		
       *	//Sets the borderColor of stripline
       *	$("#container").ejChart("option", {primaryXAxis:{stripline:[{borderColor: "green" }]}} ); 
       * @alias ejChart#striplineDefault->borderColor
       * @instance
       */
                borderColor: 'black',
                /**		
      * Specifies the zOrder of stripline. Determines the position of the stripline, whether it should render over the series or behind the series. See {@link zOrder}
      * @default ej.Chart.zOrder.over
      * @type {enum}
      * @example 
      * //To set zOrder of stripline
      * 	$("#container").ejChart({primaryXAxis:{stripline:[{zOrder: "below" }]}});				 
      * @example 
      * //Gets or sets the borderCozOrderlor of stripline, after initialization:<br>
      *	//Gets the zOrder of stripline 
      *   $("#container").ejChart("option", "primaryXAxis.stripline")[0].zOrder;		
      *	//Sets the zOrder of stripline
      *	$("#container").ejChart("option", {primaryXAxis:{stripline:[{zOrder: "below" }]}} ); 
      * @alias ejChart#striplineDefault->zOrder
      * @instance
      */
                zOrder: 'over',
                /**		
          * Specifies the borderWidth of stripline. This determines the border width of stripLine
          * @default 1
          * @type {number}
          * @example 
          * //To set borderWidth of stripline
          * 	$("#container").ejChart({primaryXAxis:{stripline:[{borderWidth: 2 }]}});				 
          * @example 
          * //Gets or sets the borderWidth of stripline, after initialization:<br>
          *	//Gets the borderWidth of stripline 
          *   $("#container").ejChart("option", "primaryXAxis.stripline")[0].borderWidth;		
          *	//Sets the borderWidth of stripline
          *	$("#container").ejChart("option", {primaryXAxis:{stripline:[{borderWidth: 2 }]}} ); 
          * @alias ejChart#striplineDefault->borderWidth
          * @instance
          */
                borderWidth: 1
            },
            /**		
         * Specifies the rowDefinitions. These are the properties specified for the rows in axis.
         * @type {object}
         * @example 
         * //To set values for rowDefinitions
         * 	$("#container").ejChart({ rowDefinitions: { unit: "percentage"} });			 
         * @example 
         * //Gets or sets the values for rowDefinitions, after initialization:<br>
         *	//Gets the values for rowDefinitions 
         *   $("#container").ejChart("option", "rowDefinitions.unit");		
         *	//Sets the values for rowDefinitions
         *	$("#container").ejChart("option", { rowDefinitions:{ unit: "percentage"} });
         * @memberof ejChart
         * @instance
         */
            rowDefinitions: null,
            /**		
        * Specifies the columnDefinitions. These are the properties specified for the columns in axis. 
        * @type {object}
        * @example 
        * //To set values for columnDefinitions
        * 	$("#container").ejChart({ columnDefinitions: { unit: "percentage"} });			 
        * @example 
        * //Gets or sets the values for columnDefinitions, after initialization:<br>
        *	//Gets the values for columnDefinitions 
        *   $("#container").ejChart("option", "columnDefinitions.unit");		
        *	//Sets the values for columnDefinitions
        *	$("#container").ejChart("option", { columnDefinitions:{ unit: "percentage"} });
        * @memberof ejChart
        * @instance
        */
            columnDefinitions: null,
            /**		
           * Specifies the chart title. This title property denotes the title of the chart. 
           * @type {object}
            * @memberof ejChart
           * @instance
           */
            title:
				{
				    /**		
       * Specifies the text of the chart title. The text which is to be render as chart title is set here.
       * @default ""
       * @type {string}
       * @example 
       * //To set text of the chart title
       * 	$("#container").ejChart({ title: { text: "Sales report"} });			 
       * @example 
       * //Gets or sets the text of the chart title, after initialization:<br>
       *	//Gets the text of the chart title
       *   $("#container").ejChart("option", "title.text");		
       *	//Sets the text of the chart title
       *	$("#container").ejChart("option", { title:{ text: "Sales report"} });
       * @alias ejChart#title->text
       * @instance
       */
				    text: "",
				    /**		
     * Specifies the textAlignment of the chart title. This modifies the alignment of the chart title. See {@link textAlignment}
     * @default ej.Chart.textAlignment.Center
     * @type {enum}
     * @example 
     * //To set textAlignment of the chart title
     * 	$("#container").ejChart({ title: { textAlignment: "far"} });			
     * @example 
     * //Gets or sets the textAlignment of the chart title, after initialization:<br>
     *	//Gets the textAlignment of the chart title
     *   $("#container").ejChart("option", "title.textAlignment");		
     *	//Sets the textAlignment of the chart title
     *	$("#container").ejChart("option", { title:{ textAlignment: "far"} });
     * @alias ejChart#title->textAlignment
     * @instance
     */
				    textAlignment: "center",
				    /**		
          * Specifies the chart title. These are the font properties involved in chart title customization.
          * @type {object}
           * @alias ejChart#title->font
          * @instance
          */
				    font: {
				        /**		
 * Specifies the title fontfamily of primaryXAxis. Chart title render with this specified fontFamily.
 * @default "Segoe UI"
 * @type {string}
 * @example 
 * //To set title fontfamily of primaryXAxis
 * 	$("#container").ejChart({primaryXAxis:{ title:{ font:{fontFamily:"Algerian"}}}});				 
 * @example 
 * //Gets or sets the title fontfamily of primaryXAxis, after initialization:<br>
 *	//Gets the title fontfamily of primaryXAxis  
 *	$("#container").ejChart("option", "primaryXAxis.title.font.fontFamily");<br>			
 *	//Sets the title fontfamily of primaryXAxis
 *	$("#container").ejChart("option", {primaryXAxis:{ title:{font:{fontFamily:"Algerian"}}}} ); 
  * @alias ejChart#title->font->fontFamily
 * @instance
 */
				        fontFamily: 'Segoe UI',
				        /**		
    * Specifies the title fontStyle of primaryXAxis. Chart title render with this specified fontStyle. See {@link fontStyle}
    * @default ej.Chart.fontStyle.Normal
    * @type {enum}
    * @example 
    * //To set title fontStyle of primaryXAxis
    * 	$("#container").ejChart({primaryXAxis:{ title:{ font:{fontStyle:"Italic"}}}});				 
    * @example 
    * //Gets or sets the title fontStyle of primaryXAxis, after initialization:<br>
    *	//Gets the title fontStyle of primaryXAxis  
    *	$("#container").ejChart("option", "primaryXAxis.title.font.fontStyle");<br>			
    *	//Sets the title fontStyle of primaryXAxis
    *	$("#container").ejChart("option", {primaryXAxis:{ title:{font:{fontStyle:"Italic"}}}} ); 
     * @alias ejChart#title->font->fontStyle
    * @instance
    */
				        fontStyle: 'Normal',
				        /**		
 * Specifies the title size. Chart title render with this specified font size.
 * @default "20px"
 * @type {string}
 * @example 
 * //To set title size
 * 	$("#container").ejChart(title:{ font:{size:"15px"}}});				
 * @example 
 * //Gets or sets the title size, after initialization:<br>
 *	//Gets the title size  
 *	$("#container").ejChart("option", "title.font.size");<br>			
 *	//Sets the title size
 *	$("#container").ejChart("option", { title:{font:{size:"15px"}}} ); 
  * @alias ejChart#title->font->size
 * @instance
 */
				        size: '20px',
				        /**		
* Specifies the title opacity. Chart title render with this specified font opacity.
* @default 0.5
* @type {number}
* @example 
* //To set title opacity
* 	$("#container").ejChart(title:{ font:{opacity:1}}});				 
* @example 
* //Gets or sets the title opacity, after initialization:<br>
*	//Gets the title opacity  
*	$("#container").ejChart("option", "title.font.opacity");<br>			
*	//Sets the title opacity
*	$("#container").ejChart("option", { title:{font:{opacity:1}}} ); 
 * @alias ejChart#title->font->opacity
* @instance
*/
				        opacity: 0.5,
				        /**		
 * Specifies the title fontWeight. Chart title render with this specified fontWeight.
 * @default ej.Chart.fontWeight.Regular
 * @type {enum}
 * @example 
 * //To set title fontWeight
 * 	$("#container").ejChart(title:{ font:{fontWeight:"lighter"}}});				 
 * @example 
 * //Gets or sets the title fontWeight, after initialization:<br>
 *	//Gets the title fontWeight  
 *	$("#container").ejChart("option", "title.font.fontWeight");<br>			
 *	//Sets the title fontWeight
 *	$("#container").ejChart("option", { title:{font:{fontWeight:"lighter"}}} ); 
 * @alias ejChart#title->font->fontWeight
 * @instance
 */
				        fontWeight: 'regular'
				    }
				},

            lineCap: { butt: 'butt', round: 'round', square: 'square' },
            lineJoin: { round: 'round', bevel: 'bevel', miter: 'miter' },
            legendAlignment: { near: 'near', center: 'center', far: 'far' },
            legendPosition: { top: 'top', bottom: 'bottom', right: 'right', left: 'left', custom: 'custom' },
            /**		
* Specifies the animation of chart. EjChart supports animation for series. When this property is set to true, then the series will animate from left to right on chart load.
* @default true
* @type {boolean}
* @example 
* //To set animation of chart
* 	$("#container").ejChart(animation:false);				 
* @example 
* //Gets or sets the animation of chart, after initialization:<br>
*	//Gets the animation of chart  
*	$("#container").ejChart("option", "animation");<br>			
*	//Sets the animation of chart
*	$("#container").ejChart("option", { animation:false} ); 
* @memberof ejChart
* @instance
*/
            animation: true,
            /**		
          * Specifies the chart legend. These are properties involved in customization of chart legend.
          * @type {object}
           * @memberof ejChart
          * @instance
          */
            legend:
            {
                /**		
          * Specifies the legend border. These are properties involved in customization of chart legend border.
          * @type {object}
           * @alias ejChart#legend->border
          * @instance
          */
                border:
                {
                    /**		
* Specifies the legend border color. Chart legend border render with this specified color.
* @default "transparent"
* @type {string}
* @example 
* //To set legend border color
* 	$("#container").ejChart({legend:{ border:{color:"red"}}});				
* @example 
* //Gets or sets the legend border color, after initialization:<br>
*	//Gets the legend border color  
*	$("#container").ejChart("option", "legend.border.color");<br>			
*	//Sets the legend border color
*	$("#container").ejChart("option", {legend:{ border:{color:"red"}}} ); 
* @alias ejChart#legend->border->color
* @instance
*/
                    color: 'transparent',

                    /**		
* Specifies the legend border width. Chart legend border render with this specified width.
* @default 1
* @type {number}
* @example 
* //To set legend border width
* 	$("#container").ejChart({legend:{ border:{width:0.5}}});				
* @example 
* //Gets or sets the legend border color, after initialization:<br>
*	//Gets the legend border color  
*	$("#container").ejChart("option", "legend.border.width");<br>			
*	//Sets the legend border color
*	$("#container").ejChart("option", {legend:{ border:{width:0.5}}} ); 
 * @alias ejChart#legend->border->width
* @instance
*/
                    width: 1
                },
                /**		
 * Specifies the legend visibility. This specifies the visibility of chart legend. 
 * @default true
 * @type {boolean}
 * @example 
 * //To set legend visibility
 * 	$("#container").ejChart({legend:{ visible:false}});				 
 * @example 
 * //Gets or sets the legend visibility, after initialization:<br>
 *	//Gets the legend visibility  
 *	$("#container").ejChart("option", "legend.visible");<br>			
 *	//Sets the legend visibility
 *	$("#container").ejChart("option", {legend:{visible:false}} ); 
  * @alias ejChart#legend->visible
 * @instance
 */
                visible: true,
                /**		
* Specifies the legend itemPadding. This is used to specify the gap/padding between the legend items.
* @default 10
* @type {number}
* @example 
* //To set legend itemPadding
* 	$("#container").ejChart({legend:{ itemPadding:5}});				
* @example 
* //Gets or sets the legend itemPadding, after initialization:<br>
*	//Gets the legend itemPadding  
*	$("#container").ejChart("option", "legend.itemPadding");<br>			
*	//Sets the legend itemPadding
*	$("#container").ejChart("option", {legend:{itemPadding:5}} ); 
 * @alias ejChart#legend->itemPadding
* @instance
*/
                itemPadding: 10,
                /**		
 * Specifies the legend item shape. This property specifies the shape of the legend items. It can be of any shape, that are specified in enum. See {@link shape}
 * @default ej.Chart.shape.None
 * @type {enum}
 * @example 
 * //To set legend item shape
 * 	$("#container").ejChart({legend:{ shape:"circle"}});				 
 * @example 
 * //Gets or sets the legend item shape, after initialization:<br>
 *	//Gets the legend item shape  
 *	$("#container").ejChart("option", "legend.shape");<br>			
 *	//Sets the legend item shape
 *	$("#container").ejChart("option", {legend:{shape:"circle"}} ); 
  * @alias ejChart#legend->shape
 * @instance
 */
                shape: 'None',
                /**		
 * Specifies the legend alignment. By this, we can adjust the alignment of the legend. See {@link alignment}
 * @default ej.Chart.alignment.Center
 * @type {enum}
 * @example 
 * //To set legend alignment
 * 	$("#container").ejChart({legend:{ alignment:"far"}});				 
 * @example 
 * //Gets or sets the legend alignment, after initialization:<br>
 *	//Gets the legend alignment  
 *	$("#container").ejChart("option", "legend.alignment");<br>			
 *	//Sets the legend alignment
 *	$("#container").ejChart("option", {legend:{alignment:"far"}} ); 
  * @alias ejChart#legend->alignment
 * @instance
 */
                alignment: 'Center',
                /**		
* Specifies the legend position. By this, we can change the position of the legend. See {@link position}
* @default ej.Chart.position.Bottom
* @type {enum}
* @example 
* //To set legend position
* 	$("#container").ejChart({legend:{ position:"top"}});				 
* @example 
* //Gets or sets the legend position, after initialization:<br>
*	//Gets the legend position  
*	$("#container").ejChart("option", "legend.position");<br>			
*	//Sets the legend position
*	$("#container").ejChart("option", {legend:{position:"top"}} ); 
* @alias ejChart#legend->position
* @instance
*/
                position: 'Bottom',
                /**		
          * Specifies the legend location.  By this, we can chnage the location of the legend.
          * @type {object}
          * @alias ejChart#legend->location
          * @instance
          */
                location:
                {
                    /**		
 * Specifies the legend location X. This property specifies the X position for rendering legend.
 * @default 0
 * @type {number}
 * @example 
 * //To set legend location X
 * 	$("#container").ejChart({legend:{ location:{ X:200}}});				 
 * @example 
 * //Gets or sets the legend location X, after initialization:<br>
 *	//Gets the legend location X  
 *	$("#container").ejChart("option", "legend.location.X");<br>			
 *	//Sets the legend location X
 *	$("#container").ejChart("option", {legend:{ location:{ X:200}}} ); 
 * @alias ejChart#legend->location->X
 * @instance
 */
                    X: 0,
                    /**		
* Specifies the legend location Y. This property specifies the Y position for rendering legend.
* @default 0
* @type {number}
* @example 
* //To set legend location Y
* 	$("#container").ejChart({legend:{ location:{ Y:300}}});				 
* @example 
* //Gets or sets the legend location Y, after initialization:<br>
*	//Gets the legend location Y  
*	$("#container").ejChart("option", "legend.location.Y");<br>			
*	//Sets the legend location Y
*	$("#container").ejChart("option", {legend:{ location:{ Y:300}}} ); 
* @alias ejChart#legend->location->Y
* @instance
*/
                    Y: 0
                },
                /**		
          * Specifies the legend item size.This property specifies the size of the items in legend.
          * @type {object}
           * @alias ejChart#legend->itemSize
          * @instance
          */
                itemSize:
                {
                    /**		
* Specifies the legend itemSize height. Legend item render with this specified height.
* @default 10
* @type {number}
* @example 
* //To set legend itemSize height
* 	$("#container").ejChart({legend:{ itemSize:{ height:5}}});				
* @example 
* //Gets or sets the legend itemSize height, after initialization:<br>
*	//Gets the legend itemSize height  
*	$("#container").ejChart("option", "legend.itemSize.height");<br>			
*	//Sets the legend itemSize height
*	$("#container").ejChart("option", {legend:{ itemSize:{ height:5}}} ); 
* @alias ejChart#legend->itemSize->height
* @instance
*/
                    height: 10,
                    /**		
 * Specifies the legend itemSize width. Legend item render with this specified width.
 * @default 10
 * @type {number}
 * @example 
 * //To set legend itemSize width
 * 	$("#container").ejChart({legend:{ itemSize:{ width:5}}});				 
 * @example 
 * //Gets or sets the legend itemSize width, after initialization:<br>
 *	//Gets the legend itemSize width  
 *	$("#container").ejChart("option", "legend.itemSize.width");<br>			
 *	//Sets the legend itemSize width
 *	$("#container").ejChart("option", {legend:{ itemSize:{ width:10}}} ); 
 * @alias ejChart#legend->itemSize->width
 * @instance
 */
                    width: 10,
                    /**		
          * Specifies the legend itemSize border. These are properties availble for customization of legend item border.
          * @type {object}
           * @alias ejChart#legend->itemSize->border
          * @instance
          */
                    border:
                    {
                        /**		
 * Specifies the legend itemSize border color. Legend item border render with this specified color.
 * @default "transparent"
 * @type {string}
 * @example 
 * //To set legend itemSize border color
 * 	$("#container").ejChart({legend:{ itemSize:{ border:{color:"green"}}}});				 
 * @example 
 * //Gets or sets the legend itemSize border color, after initialization:<br>
 *	//Gets the legend itemSize border color  
 *	$("#container").ejChart("option", "legend.itemSize.border.color");<br>			
 *	//Sets the legend itemSize border color
 *	$("#container").ejChart("option", {legend:{ itemSize:{ border:{color :"green"}}}} ); 
 * @alias ejChart#legend->itemSize->border->color
 * @instance
 */
                        color: 'transparent',
                        /**		
 * Specifies the legend itemSize border width. Legend item border render with this specified width.
 * @default 1
 * @type {number}
 * @example 
 * //To set legend itemSize border width
 * 	$("#container").ejChart({legend:{ itemSize:{ border:{width:0.5}}}});				 
 * @example 
 * //Gets or sets the legend itemSize border width, after initialization:<br>
 *	//Gets the legend itemSize border width  
 *	$("#container").ejChart("option", "legend.itemSize.border.width");<br>			
 *	//Sets the legend itemSize border width
 *	$("#container").ejChart("option", {legend:{ itemSize:{ border:{width :0.5}}}} ); 
 * @alias ejChart#legend->itemSize->border->width
 * @instance
 */
                        width: 1
                    }
                },
                /**		
 * Specifies the legend rowCount. This is the property which specifies the number of rows in which the legend item render.
 * @default null
 * @type {number}
 * @example 
 * //To set legend rowCount
 * 	$("#container").ejChart({legend:{rowCount:2 }});				
 * @example 
 * //Gets or sets the legend rowCount, after initialization:<br>
 *	//Gets the legend rowCount  
 *	$("#container").ejChart("option", "legend.rowCount");<br>			
 *	//Sets the legend rowCount
 *	$("#container").ejChart("option", {legend:{rowCount:2 }} ); 
 * @alias ejChart#legend->rowCount
 * @instance
 */
                rowCount: null,
                /**		
 * Specifies the legend columnCount. This is the property which specifies the number of columns in which the legend item render.
 * @default null
 * @type {number}
 * @example 
 * //To set legend columnCount
 * 	$("#container").ejChart({legend:{columnCount:2 }});				 
 * @example 
 * //Gets or sets the legend columnCount, after initialization:<br>
 *	//Gets the legend columnCount  
 *	$("#container").ejChart("option", "legend.columnCount");<br>			
 *	//Sets the legend columnCount
 *	$("#container").ejChart("option", {legend:{columnCount:2 }} ); 
 * @alias ejChart#legend->columnCount
 * @instance
 */
                columnCount: null,
                /**		
       * Specifies the legend opacity. Legend items render with this specified opacity.
       * @default 1
       * @type {number}
       * @example 
       * //To set legend opacity
       * 	$("#container").ejChart({legend:{opacity:0.5 }});				 
       * @example 
       * //Gets or sets the legend opacity, after initialization:<br>
       *	//Gets the legend opacity  
       *	$("#container").ejChart("option", "legend.opacity");<br>			
       *	//Sets the legend opacity
       *	$("#container").ejChart("option", {legend:{opacity:0.5 }} ); 
       * @alias ejChart#legend->opacity
       * @instance
       */
                opacity: 1,
                /**		
      * Specifies the legend fill color. Legend items render with this specified fill color.
      * @default null
      * @type {string}
      * @example 
      * //To set legend fill color
      * 	$("#container").ejChart({legend:{fill:"#ff0000" }});				 
      * @example 
      * //Gets or sets the legend fill color, after initialization:<br>
      *	//Gets the legend fill color  
      *	$("#container").ejChart("option", "legend.fill");<br>			
      *	//Sets the legend fill color
      *	$("#container").ejChart("option", {legend:{fill:"#ff0000" }} ); 
     * @alias ejChart#legend->fill
      * @instance
      */
                fill: null,
                /**		
          * Specifies the legend font. These font properties are used to customize the legend item text.
          * @type {object}
           * @alias ejChart#legend->font
          * @instance
          */
                font:
                {
                    /**		
    * Specifies the legend fontfamily color. Legend item text render with this specified font family.
    * @default "Segoe UI"
    * @type {string}
    * @example 
    * //To set legend fontFamily
    * 	$("#container").ejChart({legend:{font:{fontFamily:"Algerian" }}});				
    * @example 
    * //Gets or sets the legend fontFamily, after initialization:<br>
    *	//Gets the legend fontFamily  
    *	$("#container").ejChart("option", "legend.font.fontFamily");<br>			
    *	//Sets the legend fontFamily
    *	$("#container").ejChart("option", {legend:{font:{fontFamily:"Algerian"} }} ); 
   * @alias ejChart#legend->font->fontFamily
    * @instance
    */
                    fontFamily: 'Segoe UI',
                    /**		
    * Specifies the legend font style. Legend item text render with this specified font style. See {@link fontStyle}
    * @default ej.Chart.fontStyle.Normal
    * @type {enum}
    * @example 
    * //To set legend font style
    * 	$("#container").ejChart({legend:{font:{fontStyle:"Italic"} }});				
    * @example 
    * //Gets or sets the legend font style, after initialization:<br>
    *	//Gets the legend font style
    *	$("#container").ejChart("option", "legend.font.fontStyle");<br>			
    *	//Sets the legend font style
    *	$("#container").ejChart("option", {legend:{font:{fontStyle:"Italic"}} }); 
    * @alias ejChart#legend->font->fontStyle
    * @instance
    */
                    fontStyle: 'Normal',
                    /**		
    * Specifies the legend font weight.  Legend item text render with this specified font weight.{@link fontWeight}
    * @default ej.Chart.fontWeight.Regular
    * @type {enum}
    * @example 
    * //To set legend font weight
    * 	$("#container").ejChart({legend:{font:{fontWeight:"lighter"}}});				 
    * @example 
    * //Gets or sets the legend font weight, after initialization:<br>
    *	//Gets the legend font weight  
    *	$("#container").ejChart("option", "legend.font.fontFamily");<br>			
    *	//Sets the legend font weight
    *	$("#container").ejChart("option", {legend:{font:{fontWeight:"lighter"}}} ); 
    * @alias ejChart#legend->font->fontWeight
    * @instance
    */
                    fontWeight: 'Regular',
                    /**		
     * Specifies the legend font size.  Legend item text render with this specified size.
     * @default "12px"
     * @type {string}
     * @example 
     * //To set legend font size
     * 	$("#container").ejChart({legend:{font:{size:"10px"}}});				 
     * @example 
     * //Gets or sets the legend font size, after initialization:<br>
     *	//Gets the legend font size  
     *	$("#container").ejChart("option", "legend.font.fontFamily");<br>			
     *	//Sets the legend font size
     *	$("#container").ejChart("option", {legend:{font:{size:"10px"}}} ); 
     * @alias ejChart#legend->font->size
     * @instance
     */
                    size: '12px'
                }
            },
            /**		
         * Specifies the commonSeriesOptions. By specfying the properties in commonSeriesOptions, it is applied to all the series in chart.
         * @type {object}
          * @memberof ejChart
         * @instance
         */
            commonSeriesOptions: {
                /**		
    * Specifies the seriesType common to all series. This property specifies the series type for chart. See {@link type}
    * @default ej.Chart.type.Line
    * @type {enum}
    * @example 
    * //To set seriesType common to all series
    * 	$("#container").ejChart({commonSeriesOptions:{type:"spline"}});				 
    * @example 
    * //Gets or sets the seriesType common to all series, after initialization:<br>
    *	//Gets the seriesType common to all series 
    *	$("#container").ejChart("option", "commonSeriesOptions.type");<br>			
    *	//Sets the seriesType common to all series
    *	$("#container").ejChart("option", {commonSeriesOptions:{type:"spline"}} ); 
    * @alias ejChart#commonSeriesOptions->type
    * @instance
    */
                type: 'line',
                /**		
   * Specifies the series animation common to all series. By specifying this, we can enable/disable the animation of the series.
   * @default true
   * @type {boolean}
   * @example 
   * //To set series animation common to all series
   * 	$("#container").ejChart({commonSeriesOptions:{animation:false}});				 
   * @example 
   * //Gets or sets the series animation common to all series, after initialization:<br>
   *	//Gets the series animation common to all series 
   *	$("#container").ejChart("option", "commonSeriesOptions.animation");<br>			
   *	//Sets the series animation common to all series
   *	$("#container").ejChart("option", {commonSeriesOptions:{animation:false}} ); 
   * @alias ejChart#commonSeriesOptions->animation
   * @instance
   */
                animation: true,
                /**		
  * Specifies the doughnutCoefficient. The hole of the doughnut depends on this specified value. The user can specify a value from 0 to 1.
  * @default 0.4
  * @type {number}
  * @example 
  * //To set doughnutCoefficient to all series
  * 	$("#container").ejChart({commonSeriesOptions:{doughnutCoefficient:0.6}});				 
  * @example 
  * //Gets or sets the doughnutCoefficient to all series, after initialization:<br>
  *	//Gets the doughnutCoefficient to all series 
  *	$("#container").ejChart("option", "commonSeriesOptions.doughnutCoefficient");<br>			
  *	//Sets the doughnutCoefficient to all series
  *	$("#container").ejChart("option", {commonSeriesOptions:{doughnutCoefficient:0.6}} ); 
  * @alias ejChart#commonSeriesOptions->doughnutCoefficient
  * @instance
  */
                doughnutCoefficient: 0.4,
                /**		
   * Specifies the explodeOffset. This property represents the offset/gap of the slices of pie on explode.
   * @default 0.4
   * @type {number}
   * @example 
   * //To set explodeOffset to all series
   * 	$("#container").ejChart({commonSeriesOptions:{explodeOffset:15}});				
   * @example 
   * //Gets or sets the explodeOffset to all series, after initialization:<br>
   *	//Gets the explodeOffset to all series 
   *	$("#container").ejChart("option", "commonSeriesOptions.explodeOffset");<br>			
   *	//Sets the explodeOffset to all series
   *	$("#container").ejChart("option", {commonSeriesOptions:{explodeOffset:15}} ); 
  * @alias ejChart#commonSeriesOptions->explodeOffset
   * @instance
   */
                explodeOffset: 25,
                /**		
 * Specifies the pyramid mode. This is to set the mode of the pyramid. It has two modes, which can be selected from enum. See {@link pyramidMode}
 * @default ej.Chart.pyramidMode.Linear
 * @type {enum}
 * @example 
 * //To set pyramidMode to all series
 * 	$("#container").ejChart({commonSeriesOptions:{pyramidMode:"linear"}});				 
 * @example 
 * //Gets or sets the pyramidMode to all series, after initialization:<br>
 *	//Gets the pyramidMode to all series 
 *	$("#container").ejChart("option", "commonSeriesOptions.pyramidMode");<br>			
 *	//Sets the pyramidMode to all series
 *	$("#container").ejChart("option", {commonSeriesOptions:{pyramidMode:"linear"}} ); 
 * @alias ejChart#commonSeriesOptions->pyramidMode
 * @instance
 */
                pyramidMode: 'linear',
                /**		
 * Specifies the position of the labels. This specifies the position of the labels. We can render the chart with labels inside/outside.
 * @default ej.Chart.labelPosition.Inside
 * @type {enum}
 * @example 
 * //To set labelPosition to all series
 * 	$("#container").ejChart({commonSeriesOptions:{labelPosition:"outside"}});				 
 * @example 
 * //Gets or sets labelPosition to all series, after initialization:<br>
 *	//Gets the labelPosition to all series 
 *	$("#container").ejChart("option", "commonSeriesOptions.labelPosition");<br>			
 *	//Sets the labelPosition to all series
 *	$("#container").ejChart("option", {commonSeriesOptions:{labelPosition:"outside"}} ); 
 * @alias ejChart#commonSeriesOptions->labelPosition
 * @instance
 */
                labelPosition: 'inside',
                /**		
* Specifies the gapRatio. This property os relate to pyramid. slides in pyramid render with the specified gap.
* @default 0
* @type {number}
* @example 
* //To set gapRatio to all series
* 	$("#container").ejChart({commonSeriesOptions:{gapRatio:5}});				 
* @example 
* //Gets or sets gapRatio to all series, after initialization:<br>
*	//Gets the gapRatio to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.gapRatio");<br>			
*	//Sets the gapRatio to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{gapRatio:5}} ); 
* @alias ejChart#commonSeriesOptions->gapRatio
* @instance
*/
                gapRatio: 0,
                /**		
* Specifies the pieCoefficient. Pie Size depends on this specified value. The user can specify a value from 0 to 1.
* @default 0
* @type {number}
* @example 
* //To set pieCoefficient to all series
* 	$("#container").ejChart({commonSeriesOptions:{pieCoefficient:1}});				
* @example 
* //Gets or sets pieCoefficient to all series, after initialization:<br>
*	//Gets the pieCoefficient to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.pieCoefficient");<br>			
*	//Sets the gapRapieCoefficienttio to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{pieCoefficient:1}} ); 
* @alias ejChart#commonSeriesOptions->pieCoefficient
* @instance
*/
                pieCoefficient: 0.8,
                /**		
* Specifies the size of the doughnut. Doughnut Size depends on this specified value. The user can specify a value from 0 to 1.
* @default 0.8
* @type {number}
* @example 
* //To set doughnutSize to all series
* 	$("#container").ejChart({commonSeriesOptions:{doughnutSize:1}});				 
* @example 
* //Gets or sets doughnutSize to all series, after initialization:<br>
*	//Gets the doughnutSize to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.doughnutSize");<br>			
*	//Sets the doughnutSize to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{doughnutSize:1}} ); 
* @alias ejChart#commonSeriesOptions->doughnutSize
* @instance
*/
                doughnutSize: 0.8,

                dataSource: null,
                /**		
  * Specifies the startAngle. The chart will render from this specified degree.
  * @default null
  * @type {number}
  * @example 
  * //To set startAngle to all series
  * 	$("#container").ejChart({commonSeriesOptions:{startAngle:30}});				 
  * @example 
  * //Gets or sets startAngle to all series, after initialization:<br>
  *	//Gets the startAngle to all series 
  *	$("#container").ejChart("option", "commonSeriesOptions.startAngle");<br>			
  *	//Sets the startAngle to all series
  *	$("#container").ejChart("option", {commonSeriesOptions:{startAngle:30}} ); 
  * @alias ejChart#commonSeriesOptions->startAngle
  * @instance
  */
                startAngle: null,
                /**		
* Specifies the name of the xAxis. This specifies the name of the Xaxis. 
* @default null
* @type {string}
* @example 
* //To set xAxisName to all series
* 	$("#container").ejChart({commonSeriesOptions:{xAxisName:"xaxis"}});				
* @example 
* //Gets or sets xAxisName , after initialization:<br>
*	//Gets the xAxisName
*	$("#container").ejChart("option", "commonSeriesOptions.xAxisName");<br>			
*	//Sets the xAxisName 
*	$("#container").ejChart("option", {commonSeriesOptions:{xAxisName:"xaxis"}} ); 
* @alias ejChart#commonSeriesOptions->xAxisName
* @instance
*/
                xAxisName: null,
                /**		
* Specifies the name of the yAxis. This specifies the name of the Yaxis. 
* @default null
* @type {string}
* @example 
* //To set yAxisName to all series
* 	$("#container").ejChart({commonSeriesOptions:{yAxisName:"yaxis"}});				 
* @example 
* //Gets or sets yAxisName , after initialization:<br>
*	//Gets the yAxisName 
*	$("#container").ejChart("option", "commonSeriesOptions.yAxisName");<br>			
*	//Sets the yAxisName
*	$("#container").ejChart("option", {commonSeriesOptions:{yAxisName:"yaxis"}} ); 
* @alias ejChart#commonSeriesOptions->yAxisName
* @instance
*/
                yAxisName: null,
                /**		
* Specifies the explodeAll. By enabling this property we can explode all the slices of chart on chart load.
* @default false
* @type {boolean}
* @example 
* //To set explodeAll to all series
* 	$("#container").ejChart({commonSeriesOptions:{explodeAll:false}});				 
* @example 
* //Gets or sets explodeAll to all series, after initialization:<br>
*	//Gets the explodeAll to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.explodeAll");<br>			
*	//Sets the explodeAll to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{explodeAll:false}} ); 
* @alias ejChart#commonSeriesOptions->explodeAll
* @instance
*/
                explodeAll: null,
                /**		
* Specifies the explodeIndex. By enabling this property we can explode a particular slice of chart on chart load.
* @default null
* @type {number}
* @example 
* //To set explodeIndex to all series
* 	$("#container").ejChart({commonSeriesOptions:{explodeIndex:2}});				 
* @example 
* //Gets or sets explodeIndex to all series, after initialization:<br>
*	//Gets the explodeIndex to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.explodeIndex");<br>			
*	//Sets the explodeIndex to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{explodeIndex:2}} ); 
* @alias ejChart#commonSeriesOptions->explodeIndex
* @instance
*/
                explodeIndex: null,
                /**		
* Specifies the enableSmartLabels. By enabling this property we can enable/disable the smart labels.
* @default true
* @type {boolean}
* @example 
* //To set enableSmartLabels to all series
* 	$("#container").ejChart({commonSeriesOptions:{enableSmartLabels:false}});				 
* @example 
* //Gets or sets enableSmartLabels to all series, after initialization:<br>
*	//Gets the enableSmartLabels to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.enableSmartLabels");<br>			
*	//Sets the enableSmartLabels to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{enableSmartLabels:false}} ); 
* @alias ejChart#commonSeriesOptions->enableSmartLabels
* @instance
*/
                enableSmartLabels: true,
                /**		
         * Specifies the tooltip. These are the properties involved for customizing the tooltip.
         * @type {object}
         * @alias ejChart#commonSeriesOptions->tooltip
         * @instance
         */
                tooltip: {
                    /**		
* Specifies the visibility of tooltip. The user can change the visibility of the tooltip with this property.
* @default false
* @type {boolean}
* @example 
* //To set visibility of tooltip to all series
* 	$("#container").ejChart({commonSeriesOptions:{tooltip:{visible:true}}});				 
* @example 
* //Gets or sets visibility of tooltip to all series, after initialization:<br>
*	//Gets the visibility of tooltip to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.tooltip.visible");<br>			
*	//Sets the visibility of tooltip to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{tooltip:{visible:true}}} ); 
* @alias ejChart#commonSeriesOptions->tooltip->visible
* @instance
*/
                    visible: false,
                    /**		
* Specifies the format of tooltip.  The user can change the format of the tooltip with this property.
* @default "#point.x# : #point.y#"
* @type {string}
* @example 
* //To set format of tooltip to all series
* 	$("#container").ejChart({commonSeriesOptions:{tooltip:{format:"#point.y# : #point.x#"}}});				 
* @example 
* //Gets or sets format of tooltip to all series, after initialization:<br>
*	//Gets the format of tooltip to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.tooltip.format");<br>			
*	//Sets the format of tooltip to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{tooltip:{format:"#point.y# : #point.x#"}}} ); 
* @alias ejChart#commonSeriesOptions->tooltip->format
* @instance
*/
                    format: "#point.x# : #point.y#",
                    /**		
* Specifies the fill color of tooltip.  The user can change the fill color of the tooltip with this property.
* @default null
* @type {string}
* @example 
* //To set fill color of tooltip to all series
* 	$("#container").ejChart({commonSeriesOptions:{tooltip:{fill:"#ff0000"}}});				 
* @example 
* //Gets or sets fill color of tooltip to all series, after initialization:<br>
*	//Gets the fill color of tooltip to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.tooltip.fill");<br>			
*	//Sets the fill color of tooltip to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{tooltip:{fill:"#ff0000"}}} ); 
* @alias ejChart#commonSeriesOptions->tooltip->fill
* @instance
*/
                    fill: null,
                    /**		
         * Specifies the tooltip border. These properties are involved in customizing the tooltip border .
         * @type {object}
          * @alias ejChart#commonSeriesOptions->tooltip->border
         * @instance
         */
                    border: {
                        /**		
* Specifies the border color of tooltip. Tooltip border render with this specified color.
* @default null
* @type {string}
* @example 
* //To set border color of tooltip to all series
* 	$("#container").ejChart({commonSeriesOptions:{tooltip:{border:{color:"#00ff00"}}}});				 
* @example 
* //Gets or sets border color of tooltip to all series, after initialization:<br>
*	//Gets the border color of tooltip to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.tooltip.border.color");<br>			
*	//Sets the border color of tooltip to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{tooltip:{border:{color:"00ff00"}}}} ); 
* @alias ejChart#commonSeriesOptions->tooltip->border->color
* @instance
*/
                        color: null,
                        /**		
* Specifies the border width of tooltip. Tooltip border render with this specified width.
* @default 1
* @type {number}
* @example 
* //To set border width of tooltip to all series
* 	$("#container").ejChart({commonSeriesOptions:{tooltip:{border:{width:0.5}}}});				 
* @example 
* //Gets or sets border width of tooltip to all series, after initialization:<br>
*	//Gets the border width of tooltip to all series 
*	$("#container").ejChart("option", "commonSeriesOptions.tooltip.border.width");<br>			
*	//Sets the border width of tooltip to all series
*	$("#container").ejChart("option", {commonSeriesOptions:{tooltip:{border:{width:0.5}}}} ); 
* @alias ejChart#commonSeriesOptions->tooltip->border->width
* @instance
*/
                        width: 1
                    },
                    /**		
 * Specifies the animation of tooltip. This property is for specifying the animation for tooltip, when tooltip template is specified.
 * @default 1
 * @type {number}
 * @example 
 * //To set animation of tooltip to all series
 * 	$("#container").ejChart({commonSeriesOptions:{tooltip:{animation:false}}});				 
 * @example 
 * //Gets or sets animation of tooltip to all series, after initialization:<br>
 *	//Gets the animation of tooltip to all series 
 *	$("#container").ejChart("option", "commonSeriesOptions.tooltip.animation");<br>			
 *	//Sets the animation of tooltip to all series
 *	$("#container").ejChart("option", {commonSeriesOptions:{tooltip:{animation:false}}} ); 
 * @alias ejChart#commonSeriesOptions->tooltip->animation
 * @instance
 */
                    animation: true,
                    /**		
 * Specifies the duration of tooltip.  This property is for specifying the duration of animation for tooltip, when tooltip template is specified.
 * @default "500ms"
 * @type {string}
 * @example 
 * //To set duration of tooltip to all series
 * 	$("#container").ejChart({commonSeriesOptions:{tooltip:{duration:"600ms"}}});				 
 * @example 
 * //Gets or sets duration of tooltip to all series, after initialization:<br>
 *	//Gets the duration of tooltip to all series 
 *	$("#container").ejChart("option", "commonSeriesOptions.tooltip.duration");<br>			
 *	//Sets the duration of tooltip to all series
 *	$("#container").ejChart("option", {commonSeriesOptions:{tooltip:{duration:"600ms"}}} ); 
 * @alias ejChart#commonSeriesOptions->tooltip->duration
 * @instance
 */
                    duration: '500ms',
                    /**		
 * Specifies the opacity of tooltip.  This property is for specifying the opacity for tooltip, when tooltip template is specified.
 * @default "500ms"
 * @type {string}
 * @example 
 * //To set opacity of tooltip to all series
 * 	$("#container").ejChart({commonSeriesOptions:{tooltip:{opacity:1"}}});				
 * @example 
 * //Gets or sets opacity of tooltip to all series, after initialization:<br>
 *	//Gets the opacity of tooltip to all series 
 *	$("#container").ejChart("option", "commonSeriesOptions.tooltip.opacity");<br>			
 *	//Sets the opacity of tooltip to all series
 *	$("#container").ejChart("option", {commonSeriesOptions:{tooltip:{opacity:1}}} ); 
 * @alias ejChart#commonSeriesOptions->tooltip->opacity
 * @instance
 */
                    opacity: 0.95
                },
                /**		
         * Specifies the fill color of series. Series render with this specified color as background.
         * @default null
         * @type {string}
         * @example 
         * //To set fill color to all series
         * 	$("#container").ejChart({commonSeriesOptions:{fill:"#ff0000"}});				 
         * @example 
         * //Gets or sets fill color to all series, after initialization:<br>
         *	//Gets the fill color to all series 
         *	$("#container").ejChart("option", "commonSeriesOptions.fill");<br>			
         *	//Sets the fill color to all series
         *	$("#container").ejChart("option", {commonSeriesOptions:{fill:"#ff0000"}} ); 
        * @alias ejChart#commonSeriesOptions->fill
         * @instance
         */
                fill: null,
                /**		
        * Specifies the opacity of series.  Series render with this specified opacity.
        * @default 1
        * @type {number}
        * @example 
        * //To set opacity to all series
        * 	$("#container").ejChart({commonSeriesOptions:{opacity:0.5}});				 
        * @example 
        * //Gets or sets opacity to all series, after initialization:<br>
        *	//Gets the opacity to all series 
        *	$("#container").ejChart("option", "commonSeriesOptions.opacity");<br>			
        *	//Sets the opacity to all series
        *	$("#container").ejChart("option", {commonSeriesOptions:{opacity:0.5}} ); 
         * @alias ejChart#commonSeriesOptions->opacity
        * @instance
        */
                opacity: 1,
                /**		
       * Specifies the lineCap. This is for cutomizing the lines in line chart. See {@link lineCap}
       * @default ej.Chart.lineCap.Butt
       * @type {enum}
       * @example 
       * //To set lineCap to all series
       * 	$("#container").ejChart({commonSeriesOptions:{lineCap:"butt"}});				 
       * @example 
       * //Gets or sets lineCap to all series, after initialization:<br>
       *	//Gets the lineCap to all series 
       *	$("#container").ejChart("option", "commonSeriesOptions.lineCap");<br>			
       *	//Sets the lineCap to all series
       *	$("#container").ejChart("option", {commonSeriesOptions:{lineCap:"butt"}} ); 
        * @alias ejChart#commonSeriesOptions->lineCap
       * @instance
       */
                lineCap: 'butt',
                /**		
      * Specifies the lineJoin. This is for cutomizing the lines in line chart.  See {@link lineJoin}
      * @default ej.Chart.lineJoin.Round
      * @type {enum}
      * @example 
      * //To set lineJoin to all series
      * 	$("#container").ejChart({commonSeriesOptions:{lineJoin:"round"}});				 
      * @example 
      * //Gets or sets lineJoin to all series, after initialization:<br>
      *	//Gets the lineJoin to all series 
      *	$("#container").ejChart("option", "commonSeriesOptions.lineJoin");<br>			
      *	//Sets the lineJoin to all series
      *	$("#container").ejChart("option", {commonSeriesOptions:{lineJoin:"round"}} ); 
       * @alias ejChart#commonSeriesOptions->lineJoin
      * @instance
      */
                lineJoin: 'round',
                /**		
      * Specifies the dashArray.This is for cutomizing the lines in line chart. This renders a dashed line based on the value specified.
      * @default ""
      * @type {string}
      * @example 
      * //To set dashArray to all series
      * 	$("#container").ejChart({commonSeriesOptions:{dashArray:"2,3"}});				 
      * @example 
      * //Gets or sets dashArray to all series, after initialization:<br>
      *	//Gets the dashArray to all series 
      *	$("#container").ejChart("option", "commonSeriesOptions.dashArray");<br>			
      *	//Sets the dashArray to all series
      *	$("#container").ejChart("option", {commonSeriesOptions:{dashArray:"2,3"}} ); 
       * @alias ejChart#commonSeriesOptions->dashArray
      * @instance
      */
                dashArray: "",
                /**		
         * Specifies the border of the series. These are the properties involved in customizing the series border.
         * @type {object}
           * @alias ejChart#commonSeriesOptions->border
         * @instance
         */
                border: {
                    /**		
   * Specifies the border color of series. Series in chart render with this specified color as border color
   * @default "transparent"
   * @type {string}
   * @example 
   * //To set border color to all series
   * 	$("#container").ejChart({commonSeriesOptions:{border:{color:"#ff0000"}}});				 
   * @example 
   * //Gets or sets border color to all series, after initialization:<br>
   *	//Gets the border color to all series 
   *	$("#container").ejChart("option", "commonSeriesOptions.border.color");<br>			
   *	//Sets the border color to all series
   *	$("#container").ejChart("option", {commonSeriesOptions:{border:{color:"#ff0000"}}} ); 
   * @alias ejChart#commonSeriesOptions->border->color
   * @instance
   */
                    color: 'transparent',
                    /**		
    * Specifies the border width of series.Series in chart render with this specified value as border width
    * @default 1
    * @type {number}
    * @example 
    * //To set border width to all series
    * 	$("#container").ejChart({commonSeriesOptions:{border:{width:2}}});				 
    * @example 
    * //Gets or sets border width to all series, after initialization:<br>
    *	//Gets the border width to all series 
    *	$("#container").ejChart("option", "commonSeriesOptions.border.width");<br>			
    *	//Sets the border width to all series
    *	$("#container").ejChart("option", {commonSeriesOptions:{border:{width:2}}} ); 
     * @alias ejChart#commonSeriesOptions->border->width
    * @instance
    */
                    width: 1
                },
                width: 2,
                /**		
         * Specifies the marker of the series. These are properties involved in cutomization of markers in series. 
         * @type {object}
           * @alias ejChart#commonSeriesOptions->marker
         * @instance
         */
                marker:
                {
                    /**		
  * Specifies the shape of the marker. The marker will render with this specified shape. See {@link shape}
  * @default ej.Chart.shape.Circle
  * @type {enum}
  * @example 
  * //To set shape of the marker to all series
  * 	$("#container").ejChart({commonSeriesOptions:{marker:{shape:"rectangle"}}});				
  * @example 
  * //Gets or sets shape of the marker, after initialization:<br>
  *	//Gets the shape of the marker
  *	$("#container").ejChart("option", "commonSeriesOptions.marker.shape");<br>			
  *	//Sets the shape of the marker
  *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{shape:"rectangle"}}} ); 
   * @alias ejChart#commonSeriesOptions->marker->shape
  * @instance
  */
                    shape: 'circle',
                    /**		
         * Specifies the size of the marker. This property specifies the size of the marker.
         * @type {object}
          * @alias ejChart#commonSeriesOptions->marker->size
         * @instance
         */
                    size: {
                        /**		
  * Specifies the width of the marker. Marker will render with this specifies width.
  * @default 6
  * @type {number}
  * @example 
  * //To set width of the marker
  * 	$("#container").ejChart({commonSeriesOptions:{marker:{size:{width:5}}}});				 
  * @example 
  * //Gets or sets width of the marker, after initialization:<br>
  *	//Gets the width of the marker
  *	$("#container").ejChart("option", "commonSeriesOptions.marker.size.width");<br>			
  *	//Sets the width of the marker
  *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{size:{width:5}}}} ); 
  * @alias ejChart#commonSeriesOptions->marker->size->width
  * @instance
  */
                        width: 6,
                        /**		
* Specifies the height of the marker. Marker will render with this specifies height.
* @default 6
* @type {number}
* @example 
* //To set height of the marker
* 	$("#container").ejChart({commonSeriesOptions:{marker:{size:{height:5}}}});				 
* @example 
* //Gets or sets height of the marker, after initialization:<br>
*	//Gets the height of the marker
*	$("#container").ejChart("option", "commonSeriesOptions.marker.size.height");<br>			
*	//Sets the height of the marker
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{size:{height:5}}}} ); 
* @alias ejChart#commonSeriesOptions->marker->size->height
* @instance
*/
                        height: 6
                    },
                    /**		
* Specifies the visibility of the marker. The user can change the visibility of the marker by using this property.
* @default false
* @type {boolean}
* @example 
* //To set visibility of the marker
* 	$("#container").ejChart({commonSeriesOptions:{marker:{visible:true}}});				
* @example 
* //Gets or sets visibility of the marker, after initialization:<br>
*	//Gets the visibility of the marker
*	$("#container").ejChart("option", "commonSeriesOptions.marker.visible");<br>			
*	//Sets the visibility of the marker
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{visible:true}}} ); 
* @alias ejChart#commonSeriesOptions->marker->visible
* @instance
*/
                    visible: false,
                    /**		
* Specifies the opacity of the marker. The user can change the opacity of the marker by using this property.
* @default 1
* @type {number}
* @example 
* //To set opacity of the marker
* 	$("#container").ejChart({commonSeriesOptions:{marker:{opacity:0.5}}});				
* @example 
* //Gets or sets opacity of the marker, after initialization:<br>
*	//Gets the opacity of the marker
*	$("#container").ejChart("option", "commonSeriesOptions.marker.opacity");<br>			
*	//Sets the opacity of the marker
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{opacity:0.5}}} ); 
* @alias ejChart#commonSeriesOptions->marker->opacity
* @instance
*/
                    opacity: 1,
                    /**		
* Specifies the fill color of the marker. The user can change the fill color of the marker by using this property.
* @default null
* @type {string}
* @example 
* //To set fill color of the marker
* 	$("#container").ejChart({commonSeriesOptions:{marker:{fill:"#ff0000"}}});				 
* @example 
* //Gets or sets fill color of the marker, after initialization:<br>
*	//Gets the fill color of the marker
*	$("#container").ejChart("option", "commonSeriesOptions.marker.fill");<br>			
*	//Sets the fill color of the marker
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{fill:"#ff0000"}}} ); 
* @alias ejChart#commonSeriesOptions->marker->fill
* @instance
*/
                    fill: null,
                    /**		
       * Specifies the border of the marker. These are the properties involved in customization of border of the marker.
       * @type {object}
        * @alias ejChart#commonSeriesOptions->marker->border
       * @instance
       */
                    border: {
                        /**		
                        * Specifies the border color of the marker. The marker will render with the specified color as border color.
                        * @default "white"
                        * @type {string}
                        * @example 
                        * //To set border color of the marker
                        * 	$("#container").ejChart({commonSeriesOptions:{marker:{border:{color:"black"}}});				 
                        * @example 
                        * //Gets or sets border color of the marker, after initialization:<br>
                        *	//Gets the border color of the marker
                        *	$("#container").ejChart("option", "commonSeriesOptions.marker.border.color");<br>			
                        *	//Sets the border color of the marker
                        *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{border:{color:"black"}}}} ); 
                        * @alias ejChart#commonSeriesOptions->marker->border->color
                        * @instance
                        */
                        color: 'white',
                        /**		
* Specifies the border width of the marker. The marker will render with the specified value as border width.
* @default 3
* @type {number}
* @example 
* //To set border width of the marker
* 	$("#container").ejChart({commonSeriesOptions:{marker:{border:{width:2}}});				
* @example 
* //Gets or sets border width of the marker, after initialization:<br>
*	//Gets the border width of the marker
*	$("#container").ejChart("option", "commonSeriesOptions.marker.border.width");<br>			
*	//Sets the border width of the marker
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{border:{width:2}}}} ); 
* @alias ejChart#commonSeriesOptions->marker->border->width
* @instance
*/
                        width: 3
                    },
                    /**		
       * Specifies the dataLabel in the marker. These are the properties for customizing the dataLabel.
       * @type {object}
        * @alias ejChart#commonSeriesOptions->marker->dataLabel
       * @instance
       */
                    dataLabel:
                    {
                        /**		
* Specifies the visibility of dataLabel in the marker. The user can change the visibility of dataLabel by applying this property.
* @default false
* @type {boolean}
* @example 
* //To set visibility of dataLabel in the marker
* 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{visible:true}}});				 
* @example 
* //Gets or sets visibility of dataLabel in the marker, after initialization:<br>
*	//Gets the visibility of dataLabel in the marker
*	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.visible");<br>			
*	//Sets the visibility of dataLabel in the marker
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{visible:true}}}} ); 
* @alias ejChart#commonSeriesOptions->marker->dataLabel->visible
* @instance
*/
                        visible: false,
                        /**		
* Specifies the verticalTextAlignment of text in dataLabel. This property is used to change the alignment of the text in data label vertically. See {@link verticalTextAlignment}
* @default ej.Chart.verticalTextAlignment.Center
* @type {enum}
* @example 
* //To set verticalTextAlignment of text in dataLabel
* 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{verticalTextAlignment:"far"}}});				 
* @example 
* //Gets or sets verticalTextAlignment of text in dataLabel, after initialization:<br>
*	//Gets the verticalTextAlignment of text in dataLabel
*	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.verticalTextAlignment");<br>			
*	//Sets the verticalTextAlignment of text in dataLabel
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{verticalTextAlignment:"far"}}}} ); 
* @alias ejChart#commonSeriesOptions->marker->dataLabel->verticalTextAlignment
* @instance
*/
                        verticalTextAlignment: 'center',
                        /**		
* Specifies the horizontalTextAlignment of text in dataLabel. This property is used to change the alignment of the text in data label horizontally. See {@link horizontalTextAlignment}
* @default ej.Chart.horizontalTextAlignment.Center
* @type {enum}
* @example 
* //To set horizontalTextAlignment of text in dataLabel
* 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{horizontalTextAlignment:"far"}}});				 
* @example 
* //Gets or sets horizontalTextAlignment of text in dataLabel, after initialization:<br>
*	//Gets the horizontalTextAlignment of text in dataLabel
*	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.horizontalTextAlignment");<br>			
*	//Sets the horizontalTextAlignment of text in dataLabel
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{horizontalTextAlignment:"far"}}}} ); 
* @alias ejChart#commonSeriesOptions->marker->dataLabel->horizontalTextAlignment
* @instance
*/
                        horizontalTextAlignment: 'center',
                        /**		
* Specifies the textPosition of text in dataLabel. This property is used to change the position of the text in data label. See {@link textPosition}
* @default ej.Chart.textPosition.Top
* @type {enum}
* @example 
* //To set textPosition of text in dataLabel
* 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{textPosition:"bottom"}}});				
* @example 
* //Gets or sets textPosition of text in dataLabel, after initialization:<br>
*	//Gets the textPosition of text in dataLabel
*	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.textPosition");<br>			
*	//Sets the textPosition of text in dataLabel
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{textPosition:"bottom"}}}} ); 
* @alias ejChart#commonSeriesOptions->marker->dataLabel->textPosition
* @instance
*/
                        textPosition: 'top',
                        /**		
* Specifies the shape of dataLabel.This property is used to change the shape of the data label. The user can specify any shape mentioned in enum. See {@link shape}
* @default ej.Chart.shape.None
* @type {enum}
* @example 
* //To set shape of dataLabel
* 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{shape:"circle"}}});				
* @example 
* //Gets or sets shape of dataLabel, after initialization:<br>
*	//Gets the shape of dataLabel
*	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.shape");<br>			
*	//Sets the shape of dataLabel
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{shape:"circle"}}}} ); 
* @alias ejChart#commonSeriesOptions->marker->dataLabel->shape
* @instance
*/
                        shape: 'none',

                        /**		
* Specifies the opacity of dataLabel. This property is used to change the opacity of the data label.
* @default 1
* @type {number}
* @example 
* //To set opacity of dataLabel
* 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{opacity:0.5}}});				 
* @example 
* //Gets or sets opacity of dataLabel, after initialization:<br>
*	//Gets the opacity of dataLabel
*	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.opacity");<br>			
*	//Sets the opacity of dataLabel
*	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{opacity:0.5}}}} ); 
* @alias ejChart#commonSeriesOptions->marker->dataLabel->opacity
* @instance
*/
                        opacity: 1,
                        /**		
         * Specifies the fill color of dataLabel. This property is used to change the fill color of the data label.
         * @default null
         * @type {string}
         * @example 
         * //To set fill color of dataLabel
         * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{fill:"#ff0000"}}});				 
         * @example 
         * //Gets or sets fill color of dataLabel, after initialization:<br>
         *	//Gets the fill color of dataLabel
         *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.fill");<br>			
         *	//Sets the fill color of dataLabel
         *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{fill:"#ff0000"}}}} ); 
         * @alias ejChart#commonSeriesOptions->marker->dataLabel->fill
         * @instance
         */
                        fill: null,
                        /**		
 * Specifies the border of the dataLabel. These are the properties involved in customizing the border of the data label.
 * @type {object}
  * @alias ejChart#commonSeriesOptions->marker->dataLabel->border
 * @instance
 */
                        border: {
                            /**		
          * Specifies the border color of dataLabel. Data label render with the specified color as border color.
          * @default null
          * @type {string}
          * @example 
          * //To set border color of dataLabel
          * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{border:{color:"black"}}}});				 
          * @example 
          * //Gets or sets border color of dataLabel, after initialization:<br>
          *	//Gets the border color of dataLabel
          *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.border.color");<br>			
          *	//Sets the border color of dataLabel
          *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{border:{color:"black"}}}} ); 
          * @alias ejChart#commonSeriesOptions->marker->dataLabel->border->color
          * @instance
          */
                            color: 'white',
                            /**		
          * Specifies the border width of dataLabel. Data label render with the specified value as border width.
          * @default 0.1
          * @type {number}
          * @example 
          * //To set border width of dataLabel
          * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{border:{width:0.5}}}});				 
          * @example 
          * //Gets or sets border width of dataLabel, after initialization:<br>
          *	//Gets the border width of dataLabel
          *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.border.width");<br>			
          *	//Sets the border width of dataLabel
          *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{border:{width:0.5}}}} ); 
         * @alias ejChart#commonSeriesOptions->marker->dataLabel->border->width
          * @instance
          */
                            width: 0.1
                        },
                        /**		
      * Specifies the connectorLine of the dataLabel for accumulation series. These are properties for customizing the connector line in data label.
      * @type {object}
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->connectorLine
      * @instance
      */
                        connectorLine:
                        {
                            /**		
             * Specifies the connectorLine width of dataLabel. By this property, we can specify the width of the connector line.
             * @default 0.5
             * @type {number}
             * @example 
             * //To set connectorLine width of dataLabel
             * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{connectorLine:{width:1}}}});				 
             * @example 
             * //Gets or sets connectorLine width of dataLabel, after initialization:<br>
             *	//Gets the connectorLine width of dataLabel
             *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.connectorLine.width");<br>			
             *	//Sets the connectorLine width of dataLabel
             *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{connectorLine:{width:1}}}} ); 
             * @alias ejChart#commonSeriesOptions->marker->dataLabel->connectorLine->width
             * @instance
             */
                            width: 0.5,
                            /**		
             * Specifies the connectorLine type of dataLabel. By this property, we can specify the type of the connector line. See {@link type}
             * @default ej.Chart.type.Line
             * @type {enum}
             * @example 
             * //To set connectorLine type of dataLabel
             * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{connectorLine:{type:"spline"}}}});				 
             * @example 
             * //Gets or sets connectorLine type of dataLabel, after initialization:<br>
             *	//Gets the connectorLine type of dataLabel
             *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.connectorLine.type");<br>			
             *	//Sets the connectorLine type of dataLabel
             *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{connectorLine:{type:"spline"}}}} ); 
             * @alias ejChart#commonSeriesOptions->marker->dataLabel->connectorLine->type
             * @instance
             */
                            type: 'line'
                        },
                        /**		
      * Specifies the font of the dataLabel. These are the properties involved in customizing the data label font.
      * @type {object}
       * @alias ejChart#commonSeriesOptions->marker->dataLabel->font 
      * @instance
      */
                        font:
                        {
                            /**		
           * Specifies the fontFamily of dataLabel. Text in the data label render with the specified font family.
           * @default "Segoe UI"
           * @type {string}
           * @example 
           * //To set fontFamily of dataLabel
           * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{font:{fontFamily:"Algerian"}}}});				 
           * @example 
           * //Gets or sets fontFamily of dataLabel, after initialization:<br>
           *	//Gets the fontFamily of dataLabel
           *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.font.fontFamily");<br>			
           *	//Sets the fontFamily of dataLabel
           *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{font:{fontFamily:"Algerian"}}}} ); 
          * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->fontFamily
           * @instance
           */
                            fontFamily: 'Segoe UI',
                            /**		
          * Specifies the fontStyle of dataLabel. Text in the data label render with the specified font style. See {@link fontStyle}
          * @default ej.Chart.fontStyle.Normal
          * @type {enum}
          * @example 
          * //To set fontStyle of dataLabel
          * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{font:{fontStyle:"Italic"}}}});				 
          * @example 
          * //Gets or sets fontStyle of dataLabel, after initialization:<br>
          *	//Gets the fontStyle of dataLabel
          *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.font.fontStyle");<br>			
          *	//Sets the fontStyle of dataLabel
          *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{font:{fontStyle:"Italic"}}}} ); 
          * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->fontStyle
          * @instance
          */
                            fontStyle: 'Normal',
                            /**		
         * Specifies the fontWeight of dataLabel. Text in the data label render with the specified font weight. See {@link fontWeight}
         * @default ej.Chart.fontWeight.Regular
         * @type {enum}
         * @example 
         * //To set fontWeight of dataLabel
         * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{font:{fontWeight:"lighter"}}}});				 
         * @example 
         * //Gets or sets fontWeight of dataLabel, after initialization:<br>
         *	//Gets the fontWeight of dataLabel
         *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.font.fontWeight");<br>			
         *	//Sets the fontWeight of dataLabel
         *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{font:{fontWeight:"lighter"}}}} ); 
         * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->fontWeight
         * @instance
         */
                            fontWeight: 'Regular',
                            /**		
        * Specifies the font size of dataLabel. Text in the data label render with the specified font size.
        * @default "12px"
        * @type {string}
        * @example 
        * //To set font size of dataLabel
        * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{font:{size:"10px"}}}});				 
        * @example 
        * //Gets or sets font size of dataLabel, after initialization:<br>
        *	//Gets the font size of dataLabel
        *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.font.size");<br>			
        *	//Sets the font size of dataLabel
        *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{font:{size:"10px"}}}} ); 
        * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->size
        * @instance
        */
                            size: '12px',
                            /**		
        * Specifies the color of the text in dataLabel.  Text in the data label render with the specified font color.
        * @default "#707070"
        * @type {string}
        * @example 
        * //To set color of the text in dataLabel
        * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{font:{color:"#700000"}}}});				 
        * @example 
        * //Gets or sets color of the text in dataLabel, after initialization:<br>
        *	//Gets the color of the text in dataLabel
        *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.font.color");<br>			
        *	//Sets the color of the text in dataLabel
        *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{font:{color:"#700000"}}}} ); 
        * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->color
        * @instance
        */
                            color: '#707070',
                            /**		
       * Specifies the opacity of the text in dataLabel. Text in the data label render with the specified opacity.
       * @default 1
       * @type {number}
       * @example 
       * //To set opacity of the text in dataLabel
       * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{font:{opacity:0.5}}}});				 
       * @example 
       * //Gets or sets opacity of the text in dataLabel, after initialization:<br>
       *	//Gets the opacity of the text in dataLabel
       *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.font.opacity");<br>			
       *	//Sets the opacity of the text in dataLabel
       *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{font:{opacity:0.5}}}} ); 
       * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->opacity
       * @instance
       */
                            opacity: 1
                        },
                        /**		
      * Specifies the margin of the dataLabel. These are properties invloved in setting the margin for data label.
      * @type {object}
       * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin
      * @instance
      */
                        margin:
                        {
                            /**		
      * Specifies the left margin value in dataLabel. By specifying this property, we can adjust the text in data label to left.
      * @default 5
      * @type {number}
      * @example 
      * //To set left margin value in dataLabel
      * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{margin:{left:3}}}});				 
      * @example 
      * //Gets or sets left margin value in dataLabel, after initialization:<br>
      *	//Gets the left margin value in dataLabel
      *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.margin.left");<br>			
      *	//Sets the left magin value in dataLabel
      *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{margin:{left:3}}}} ); 
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin->left
      * @instance
      */
                            left: 5,
                            /**		
      * Specifies the top margin value in dataLabel. By specifying this property, we can adjust the text in data label to top.
      * @default 5
      * @type {number}
      * @example 
      * //To set top margin value in dataLabel
      * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{margin:{top:3}}}});				 
      * @example 
      * //Gets or sets top margin value in dataLabel, after initialization:<br>
      *	//Gets the top margin value in dataLabel
      *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.margin.top");<br>			
      *	//Sets the top magin value in dataLabel
      *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{margin:{top:3}}}} ); 
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin->top
      * @instance
      */
                            top: 5,
                            /**		
      * Specifies the bottom margin value in dataLabel. By specifying this property, we can adjust the text in data label to bottom.
      * @default 5
      * @type {number}
      * @example 
      * //To set bottom margin value in dataLabel
      * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{margin:{bottom:3}}}});				
      * @example 
      * //Gets or sets bottom margin value in dataLabel, after initialization:<br>
      *	//Gets the bottom margin value in dataLabel
      *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.margin.bottom");<br>			
      *	//Sets the bottom magin value in dataLabel
      *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{margin:{bottom:3}}}} ); 
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin->bottom
      * @instance
      */
                            bottom: 5,
                            /**		
      * Specifies the right margin value in dataLabel. By specifying this property, we can adjust the text in data label to right.
      * @default 5
      * @type {number}
      * @example 
      * //To set right margin value in dataLabel
      * 	$("#container").ejChart({commonSeriesOptions:{marker:{dataLabel:{margin:{right:3}}}});				
      * @example 
      * //Gets or sets right margin value in dataLabel, after initialization:<br>
      *	//Gets the right margin value in dataLabel
      *	$("#container").ejChart("option", "commonSeriesOptions.marker.dataLabel.margin.right");<br>			
      *	//Sets the right magin value in dataLabel
      *	$("#container").ejChart("option", {commonSeriesOptions:{marker:{dataLabel:{margin:{right:3}}}} ); 
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin->right
      * @instance
      */
                            right: 5
                        }
                    }
                },
                /**		
      * Specifies the font that is common to all series. These are the properties involved in customizing the font of the series in chart. 
      * @type {object}
       * @alias ejChart#commonSeriesOptions->font
      * @instance
      */
                font: {
                    /**		
      * Specifies the fontFamily, that is common to all series. Text in the series, render with the specified font family.
      * @default "Segoe UI"
      * @type {string}
      * @example 
      * //To set fontFamily to the series
      * 	$("#container").ejChart({commonSeriesOptions:{font:{fontFamily:"Algerian"}}});				 
      * @example 
      * //Gets or sets fontFamily to the series, after initialization:<br>
      *	//Gets the fontFamily to the series
      *	$("#container").ejChart("option", "commonSeriesOptions.font.fontFamily");<br>			
      *	//Sets the fontFamily to the series
      *	$("#container").ejChart("option", {commonSeriesOptions:{font:{fontFamily:"Algerian"}}} ); 
      * @alias ejChart#commonSeriesOptions->font->fontFamily
      * @instance
      */
                    fontFamily: 'Segoe UI',
                    /**		
      * Specifies the fontStyle, that is common to all series. Text in the series, render with the specified font style.
      * @default ej.Chart.fontStyle.Normal
      * @type {enum}
      * @example 
      * //To set fontStyle to the series
      * 	$("#container").ejChart({commonSeriesOptions:{font:{fontStyle:"Italic"}}});				
      * @example 
      * //Gets or sets fontStyle to the series, after initialization:<br>
      *	//Gets the fontStyle to the series
      *	$("#container").ejChart("option", "commonSeriesOptions.font.fontStyle");<br>			
      *	//Sets the fontStyle to the series
      *	$("#container").ejChart("option", {commonSeriesOptions:{font:{fontStyle:"Italic"}}} ); 
     * @alias ejChart#commonSeriesOptions->font->fontSyle
      * @instance
      */
                    fontStyle: 'Normal',
                    /**		
     * Specifies the fontWeight, that is common to all series. Text in the series, render with the specified font weight.
     * @default ej.Chart.fontWeight.Regular
     * @type {enum}
     * @example 
     * //To set fontWeight to the series
     * 	$("#container").ejChart({commonSeriesOptions:{font:{fontWeight:"lighter"}}});				 
     * @example 
     * //Gets or sets fontWeight to the series, after initialization:<br>
     *	//Gets the fontWeight to the series
     *	$("#container").ejChart("option", "commonSeriesOptions.font.fontWeight");<br>			
     *	//Sets the fontWeight to the series
     *	$("#container").ejChart("option", {commonSeriesOptions:{font:{fontWeight:"lighter"}}} ); 
     * @alias ejChart#commonSeriesOptions->font->fontWeight
     * @instance
     */
                    fontWeight: 'Regular',
                    /**		
     * Specifies the font size, that is common to all series. Text in the series, render with the specified font size.
     * @default "12px"
     * @type {string}
     * @example 
     * //To set font size to the series
     * 	$("#container").ejChart({commonSeriesOptions:{font:{size:"10px"}}});				 
     * @example 
     * //Gets or sets font size to the series, after initialization:<br>
     *	//Gets the font size to the series
     *	$("#container").ejChart("option", "commonSeriesOptions.font.size");<br>			
     *	//Sets the font size to the series
     *	$("#container").ejChart("option", {commonSeriesOptions:{font:{size:"10px"}}} ); 
     * @alias ejChart#commonSeriesOptions->font->size
     * @instance
     */
                    size: '12px',
                    /**		
    * Specifies the color, that is common to all series. Text in the series, render with the specified font color.
    * @default "#707070"
    * @type {string}
    * @example 
    * //To set font color to the series
    * 	$("#container").ejChart({commonSeriesOptions:{font:{color:"#700000"}}});				 
    * @example 
    * //Gets or sets font color to the series, after initialization:<br>
    *	//Gets the font color to the series
    *	$("#container").ejChart("option", "commonSeriesOptions.font.color");<br>			
    *	//Sets the font color to the series
    *	$("#container").ejChart("option", {commonSeriesOptions:{font:{color:"#700000"}}} ); 
    * @alias ejChart#commonSeriesOptions->font->color
    * @instance
    */
                    color: '#707070',
                    /**		
    * Specifies the opacity, that is common to all series. Text in the series, render with the specified font opacity.
    * @default 1
    * @type {number}
    * @example 
    * //To set font opacity to the series
    * 	$("#container").ejChart({commonSeriesOptions:{font:{opacity:0.5}}});				
	* @example 
    * //Gets or sets font opacity to the series, after initialization:<br>
    *	//Gets the font opacity to the series
    *	$("#container").ejChart("option", "commonSeriesOptions.font.opacity");<br>			
    *	//Sets the font opacity to the series
    *	$("#container").ejChart("option", {commonSeriesOptions:{font:{opacity:0.5}}} ); 
    * @alias ejChart#commonSeriesOptions->font->opacity
    * @instance
    */
                    opacity: 1
                }
            },
            /**		
     * Specifies the crosshair. These are the properties involved for customizing the crosshair.
     * @type {object}
      * @memberof ejChart
     * @instance
     */
            crosshair:
            {
                /**		
     * Specifies the marker of the crosshair. These are the properties involved for customizing the marker in crosshair. 
     * @type {object}
      * @alias ejChart#crosshair->marker
     * @instance
     */
                marker:
                {
                    /**		
    * Specifies the visibility of the marker in crosshair. By specifying this property we can change the visibility of marker in crosshair.
    * @default true
    * @type {boolean}
    * @example 
    * //To set visibility of the marker in crosshair
    * 	$("#container").ejChart({crosshair:{marker:{visible:false}}});				
    * @example 
    * //Gets or sets visibility of the marker in crosshair, after initialization:<br>
    *	//Gets the visibility of the marker in crosshair
    *	$("#container").ejChart("option", "crosshair.marker.visible");<br>			
    *	//Sets the visibility of the marker in crosshair
    *	$("#container").ejChart("option",{crosshair:{marker:{visible:false}}}); 
     * @alias ejChart#crosshair->marker->visible
    * @instance
    */
                    visible: true,
                    /**		
     * Specifies the size of the marker in crosshair. By specifying this property we can change the size of the marker in crosshair.
     * @type {object}
       * @alias ejChart#crosshair->marker->size
     * @instance
     */
                    size:
                    {
                        /**		
   * Specifies the width of the marker in crosshair. Marker in crosshair render with the specified width.
   * @default 10
   * @type {number}
   * @example 
   * //To set width of the marker in crosshair
   * 	$("#container").ejChart({crosshair:{marker:{size:{width:5}}}});				 
   * @example 
   * //Gets or sets width of the marker in crosshair, after initialization:<br>
   *	//Gets the width of the marker in crosshair
   *	$("#container").ejChart("option", "crosshair.marker.size.width");<br>			
   *	//Sets the width of the marker in crosshair
   *	$("#container").ejChart("option",{crosshair:{marker:{size:{width:5}}}}); 
    * @alias ejChart#crosshair->marker->size->width
   * @instance
   */
                        width: 10,
                        /**		
 * Specifies the height of the marker in crosshair. Marker in crosshair render with the specified height.
 * @default 10
 * @type {number}
 * @example 
 * //To set height of the marker in crosshair
 * 	$("#container").ejChart({crosshair:{marker:{size:{height:5}}}});				 
 * @example 
 * //Gets or sets height of the marker in crosshair, after initialization:<br>
 *	//Gets the height of the marker in crosshair
 *	$("#container").ejChart("option", "crosshair.marker.size.height");<br>			
 *	//Sets the height of the marker in crosshair
 *	$("#container").ejChart("option",{crosshair:{marker:{size:{height:5}}}}); 
 * @alias ejChart#crosshair->marker->size->height
 * @instance
 */
                        height: 10
                    },
                    /**		
 * Specifies the opacity of the marker in crosshair. Marker in crosshair render with the specified opacity.
 * @default true
 * @type {boolean}
 * @example 
 * //To set opacity of the marker in crosshair
 * 	$("#container").ejChart({crosshair:{marker:{size:{opacity:0.5}}}});				 
 * @example 
 * //Gets or sets opacity of the marker in crosshair, after initialization:<br>
 *	//Gets the opacity of the marker in crosshair
 *	$("#container").ejChart("option", "crosshair.marker.size.opacity");<br>			
 *	//Sets the opacity of the marker in crosshair
 *	$("#container").ejChart("option",{crosshair:{marker:{size:{opacity:0.5}}}}); 
  * @alias ejChart#crosshair->marker->opacity
 * @instance
 */
                    opacity: 1,
                    /**		
     * Specifies the border of the marker in crosshair. These are the properties involved in customizing the border of the marker in crosshair.
     * @type {object}
       * @alias ejChart#crosshair->marker->border
     * @instance
     */
                    border:
                    {
                        /**		
      * Specifies the border width of the marker in crosshair. Marker in crosshair render with the specified border width.
      * @default 3
      * @type {number}
      * @example 
      * //To set border width of the marker in crosshair
      * 	$("#container").ejChart({crosshair:{marker:{border:{ width:2}}}});				 
      * @example 
      * //Gets or sets border width of the marker in crosshair, after initialization:<br>
      *	//Gets the border width of the marker in crosshair
      *	$("#container").ejChart("option", "crosshair.marker.border width");<br>			
      *	//Sets the border width of the marker in crosshair
      *	$("#container").ejChart("option",{crosshair:{marker:{border:{ width:2}}}}); 
       * @alias ejChart#crosshair->marker->border->width
      * @instance
      */
                        width: 3
                    }
                },
                /**		
      * Specifies the visibility of the crosshair. This property is to change the visibility of the marker in dataLabel.
      * @default 3
      * @type {number}
      * @example 
      * //To set the visibility of the crosshair
      * 	$("#container").ejChart({crosshair:{visible:true}});				
      * @example 
      * //Gets or sets visibility of the crosshair, after initialization:<br>
      *	//Gets the visibility of the crosshair
      *	$("#container").ejChart("option", "crosshair.visible");<br>			
      *	//Sets the visibility of the crosshair
      *	$("#container").ejChart("option",{crosshair:{visible:true}}); 
       * @alias ejChart#crosshair->visible
      * @instance
      */
                visible: false,
                /**		
      * Specifies the type of the crosshair. The user can specify the type of the crosshair. It may be trackball or crosshair.
      * @default ej.Chart.crosshairType.Crosshair
      * @type {enum}
      * @example 
      * //To set the type of the crosshair
      * 	$("#container").ejChart({crosshair:{type:"trackBall"}});				 
      * @example 
      * //Gets or sets type of the crosshair, after initialization:<br>
      *	//Gets the type of the crosshair
      *	$("#container").ejChart("option", "crosshair.type");<br>			
      *	//Sets the type of the crosshair
      *	$("#container").ejChart("option",{crosshair:{type:"trackBall"}}); 
       * @alias ejChart#crosshair->type
      * @instance
      */
                type: "crosshair"
            },


            pointStyle:
            {
                lineCap: 'butt', lineJoin: 'round', opacity: 1, interior: null, borderColor: null, borderWidth: 1
            },
            textStyle:
            {
                marker: { textAlignment: 'center', textPosition: 'top' },
                font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', color: '#707070', opacity: 1 }
            },
            symbolShape: {
                None: 0,
                LeftArrow: 1,
                RightArrow: 2,
                Circle: 3,
                Cross: 4,
                HorizLine: 5,
                VertLine: 6,
                Diamond: 7,
                Rectangle: 8,
                Triangle: 9,
                InvertedTriangle: 10,
                Hexagon: 11,
                Pentagon: 12,
                Star: 13,
                Ellipse: 14,
                Wedge: 15,
                Trapezoid: 16,
                UpArrow: 17,
                DownArrow: 18,
                Image: 19
            },
            initSeriesRender: true,
            /**		
     * Specifies the theme for the chart. By specifying this property the user can change the theme of the chart. See {@link theme}
     * @default ej.Chart.theme.Flatlight
     * @type {enum}
     * @example 
     * //To set the theme for the chart
     * 	$("#container").ejChart({theme:"flatdark"});				 
     * @example 
     * //Gets or sets the theme for the chart, after initialization:<br>
     *	//Gets the theme for the chart
     *	$("#container").ejChart("option", "theme");<br>			
     *	//Sets the theme for the chart
     *	$("#container").ejChart("option",{theme:"flatdark"}); 
     * @memberof ejChart
     * @instance
     */
            theme: "flatlight",
            /**		
    * Specifies whether to enable the canResize or not. This property is used to make the chart responsive on resize.
    * @default false
    * @type {enum}
    * @example 
    * //To set resize for chart
    * 	$("#container").ejChart({canResize:true});				 
    * @example 
    * //Gets or sets the resize of chart, after initialization:<br>
    *	//Gets the resize of chart
    *	$("#container").ejChart("option", "canResize");<br>			
    *	//Sets the resize of chart
    *	$("#container").ejChart("option",{canResize:true}); 
    * @memberof ejChart
    * @instance
    */
            canResize: false,
            /**		
     * Specifies the zooming. These are the properties involved for zooming. 
     * @type {object}
      * @memberof ejChart
     * @instance
     */
            zooming:
            {
                /**		
   * Specifies the zooming enabled. By this the user can enable/disable the zooming for chart.
   * @default false
   * @type {boolean}
   * @example 
   * //To set zooming enabled
   * 	$("#container").ejChart({zooming:{enable:true}});				 
   * @example 
   * //Gets or sets the zooming enabled, after initialization:<br>
   *	//Gets the zooming enabled
   *	$("#container").ejChart("option", "zooming.enable");<br>			
   *	//Sets the zooming enabled
   *	$("#container").ejChart("option",{zooming:{enable:true}}); 
    * @alias ejChart#zooming->enable
   * @instance
   */
                enable: false,
                /**		
   * Specifies the type of zooming. By this property the user can zoom the selective or both the axis.
   * @default 'x,y'
   * @type {string}
   * @example 
   * //To set the type of zooming
   * 	$("#container").ejChart({zooming:{type:"y"}});				 
   * @example 
   * //Gets or sets the zooming tyoe, after initialization:<br>
   *	//Gets the zooming enabled
   *	$("#container").ejChart("option", "zooming.type");<br>			
   *	//Sets the zooming enabled
   *	$("#container").ejChart("option",{zooming:{type:"y"}}); 
   * @alias ejChart#zooming->type
   * @instance
   */
                type: 'x,y',
                /**		
   * Specifies the zooming is enabled in mouse wheel. By this property the user can enable/disable the zooming of chart through mouse wheel.
   * @default false
   * @type {boolean}
   * @example 
   * //To set enable mouse wheel zoom
   * 	$("#container").ejChart({zooming:{enableMouseWheel:true}});				 
   * @example 
   * //Gets or sets the enable mouse wheel zoom, after initialization:<br>
   *	//Gets enable mouse wheel zoom
   *	$("#container").ejChart("option", "zooming.enableMouseWheel");<br>			
   *	//Sets enable mouse wheel zoom
   *	$("#container").ejChart("option",{zooming:{enableMouseWheel:true}}); 
   * @alias ejChart#zooming->enableMouseWheel
   * @instance
   */
                enableMouseWheel: false
            },
            type: 'x,y',
            /**		
* Specifies the localization of the chart. This property is to specify the localization of chart. Bsed on this, the text in chart will render.
* @default "en-US"
* @type {string}
* @example 
* //To set localization of chart
* 	$("#container").ejChart({localization : "en-US"});				 
* @example 
* //Gets or sets the enable mouse wheel zoom, after initialization:<br>
*	//Gets enable mouse wheel zoom
*	$("#container").ejChart("option", "localization");<br>			
*	//Sets enable mouse wheel zoom
*	$("#container").ejChart("option",{localization : "en-US"}); 
* @memberof ejChart
* @instance
*/
            localization: "en-US",

            xZoomFactor: 1,

            yZoomFactor: 1,

            xZoomPosition: 0,

            yZoomPosition: 0,
            /**     
              * Fires on load of chart.
              * @event
              * @name ejChart#load	
              * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
              * @example 
              * //load event for chart
              * $("#container").ejChart({
              *    load: function (args) {}
              * });
              * @memberof ejChart
              * @instance
              */
            load: "",
            /**     
              * Fires on rendering the axis labels.
              * @event
              * @name ejChart#axesLabelRendering
              * @param {Object} argument.Axis parameters from chart   
              * @param {Object} argument.Label parameters from chart for processing of labels
			  * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			  * @param {object}  argument.model returns the chart model
			  * @param {string}  argument.type returns the name of the event
              * @example 
              * //axesLabelRendering event for chart
              * $("#container").ejChart({
              *    axesLabelRendering: function (args) {}
              * });
              * @memberof ejChart
              * @instance
              */
            axesLabelRendering: "",
            /**     
             * Fires on axes range calculation.
             * @event
             * @name ejChart#axesRangeCalculate	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //axesRangeCalculate event for chart
             * $("#container").ejChart({
             *    axesRangeCalculate: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            axesRangeCalculate: "",
            /**     
             * Fires on rendering the axes title.
             * @event
             * @name ejChart#axesTitleRendering	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //axesTitleRendering event for chart
             * $("#container").ejChart({
             *    axesTitleRendering: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            axesTitleRendering: "",
            /**     
             * Fires on chart area bounds calculation.
             * @event
             * @name ejChart#chartAreaBoundsCalculate	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //chartAreaBoundsCalculate event for chart
             * $("#container").ejChart({
             *    chartAreaBoundsCalculate: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            chartAreaBoundsCalculate: "",
            /**     
             * Fires on rendering of legend item.
             * @event
             * @name ejChart#legendItemRendering
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //legendItemRendering event for chart
             * $("#container").ejChart({
             *    legendItemRendering: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            legendItemRendering: "",
            /**     
             * Fires on calculating the legend bounds
             * @event
             * @name ejChart#lengendBoundsCalculate
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //lengendBoundsCalculate event for chart
             * $("#container").ejChart({
             *    lengendBoundsCalculate: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            lengendBoundsCalculate: "",
            /**     
             * Fires on pre render.
             * @event
             * @name ejChart#preRender	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //preRender event for chart
             * $("#container").ejChart({
             *    preRender: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            preRender: "",
            /**     
             * Fires on rendering the series.
             * @event
             * @name ejChart#seriesRendering
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //seriesRendering event for chart
             * $("#container").ejChart({
             *    seriesRendering: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            seriesRendering: "",
            /**     
             * Fires on rendering the symbols.
             * @event
             * @name ejChart#symbolRendering
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //symbolRendering event for chart
             * $("#container").ejChart({
             *    symbolRendering: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            symbolRendering: "",
            /**     
             * Fires on rendering the title.
             * @event
             * @name ejChart#titleRendering	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //titleRendering event for chart
             * $("#container").ejChart({
             *    titleRendering: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            titleRendering: "",
            /**     
             * Fires on initializing the axis labels.
             * @event
             * @name ejChart#axesLabelsInitialize
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //axesLabelsInitialize event for chart
             * $("#container").ejChart({
             *    axesLabelsInitialize: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            axesLabelsInitialize: "",
            /**     
             * Fires on click of point region.
             * @event
             * @name ejChart#pointRegionClick	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //pointRegionClick event for chart
             * $("#container").ejChart({
             *    pointRegionClick: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            pointRegionClick: "",
            /**     
             * Fires on mousemove over the point region.
             * @event
             * @name ejChart#pointRegionMouseMove	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //pointRegionMouseMove event for chart
             * $("#container").ejChart({
             *    pointRegionMouseMove: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            pointRegionMouseMove: "",
            /**     
             * Fires on click of legend item
             * @event
             * @name ejChart#legendItemClick	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //legendItemClick event for chart
             * $("#container").ejChart({
             *    legendItemClick: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            legendItemClick: "",
            /**     
             * Fires on mouse move over legend item.
             * @event
             * @name ejChart#legendItemMouseMove	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //legendItemMouseMove event for chart
             * $("#container").ejChart({
             *    legendItemMouseMove: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            legendItemMouseMove: "",
            /**     
             * Fires on rendering the display text.
             * @event
             * @name ejChart#displayTextRendering		
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //displayTextRendering event for chart
             * $("#container").ejChart({
             *    displayTextRendering: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            displayTextRendering: "",
            /**     
             * Fires on initialize the tooltip.
             * @event
             * @name ejChart#toolTipInitialize	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //toolTipInitialize event for chart
             * $("#container").ejChart({
             *    toolTipInitialize: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            toolTipInitialize: "",
            /**     
             * Fires on track the axis tooltip.
             * @event
             * @name ejChart#trackAxisToolTip	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //trackAxisToolTip event for chart
             * $("#container").ejChart({
             *    trackAxisToolTip: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            trackAxisToolTip: "",
            /**     
             * Fires on tracking the tooltip.
             * @event
             * @name ejChart#trackToolTip	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //trackToolTip event for chart
             * $("#container").ejChart({
             *    trackToolTip: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            trackToolTip: "",
            /**     
             * Fires on completion of animation.
             * @event
             * @name ejChart#animationComplete
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //animationComplete event for chart
             * $("#container").ejChart({
             *    animationComplete: function (args) {}
             * });
             * @memberof ejChart
             * @instance
             */
            animationComplete: "",
            /**     
             * Fires when chart is destroyed completely.
             * @event
             * @name ejChart#destroy
             * @example 
             * //animationComplete event for chart
             * $("#container").ejChart("destroy");
             * @memberof ejChart
             * @instance
             */
            destroy: "",
            /**     
            * Fires after chart is created.
            * @event
            * @name ejChart#create		
            * @example 
            * //create event for chart
            * $("#container").ejChart("create");
            * @memberof ejChart
            * @instance
            */
            create: "",

            margin: { left: 10, right: 10, top: 10, bottom: 10 },
            size: {},
            elementSpacing: 10,

            seriesColors: [],
            seriesBorderColors: [],
            dateStart: 25568.791666666668,
            pointColors: [],
            pointBorderColors: []
        },
        /**
          * Specify the data types for default properties 
          * @private
          */
        dataTypes: {
            dataSource: "data",
            axes: "array",
            series: "array",
            seriesColors: "array",

            rowDefinitions: "array",
            columnDefinitions: "array",
            seriesBorderColors: "array",
            pointColors: "array",
            pointBorderColors: "array",
            initSeriesRender: "boolean",
            theme: "enum",
            canResize: "boolean",
            elementSpacing: "number",
            primaryXAxis: {
                labels: "array",
                stripline: "array",
                orientation: "enum",
                rangePadding: "enum",

                hidePartialLabels: "boolean",
                opposedPosition: "boolean",
                zoomFactor: "number",
                zoomPosition: "number"
            },
            primaryYAxis: {
                labels: "array",
                stripline: "array",
                orientation: "enum",
                rangePadding: "enum",

                hidePartialLabels: "boolean",
                opposedPosition: "boolean",
                zoomFactor: "number",
                zoomPosition: "number"
            },
            legend: {

                shape: "enum",
                alignment: "enum",
                position: "enum",
                itemPadding: "number"
            },
            zooming: {
                enable: "boolean",
                type: "string",
                enableMouseWheel: "boolean"
            },
            size: {
                width: "number",
                height: "number"
            }

        },

        observables: ["xZoomFactor", "yZoomFactor", "xZoomPosition", "yZoomPosition"],
        _xZoomFactor: ej.util.valueFunction("xZoomFactor"),
        _yZoomFactor: ej.util.valueFunction("yZoomFactor"),
        _xZoomPosition: ej.util.valueFunction("xZoomPosition"),
        _yZoomPosition: ej.util.valueFunction("yZoomPosition"),

        /**
        * Create the chart widget
        * @private
        */
        _init: function () {

            this._renderSfChart();

        },
        /**
        * destroy the chart widget
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Destroy Chart
		* var chartObj = $("#container").data("ejChart");
		* chartObj.destroy(); // destroy the chart
		* &lt;/script&gt;
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#container").ejChart("destroy");	
		* &lt;/script&gt;
		* @memberof ejChart
		* @instance
         */
        _destroy: function () {
            $(this.element).removeClass("e-chart e-js").find("#" + this.svgObject.id).remove();
        },
        /**
        * redraw the chart widget
		* @alias redraw
		* @return jQuery
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Redraw Chart
		* var chartObj = $("#container").data("ejChart");
		* chartObj.redraw(); // redraw the chart
		* &lt;/script&gt;
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#container").ejChart("redraw");	
		* &lt;/script&gt;
		* @memberof ejChart
		* @instance
         */
        redraw: function () {
            $('#template_group_' + this.instance._id).remove();
            $(this.svgObject).empty();
            this.bindTo();
        },
        /**
                 * To configure the properties at runtime using SetModel		
                 * @private
                 */
        _setModel: function (options) {
            for (var prop in options) {
                this.disableAnimation();
                switch (prop) {
                    case "theme":
                        this.model._themeChanged = true;
                        this.model.theme = options[prop];
                        this.setTheme(ej.EjSvgRender.themes, this.model.theme);
                        $.extend(true, this.model.axes[0], ej.EjSvgRender.themes[this.model.theme].primaryXAxis);
                        $.extend(true, this.model.axes[1], ej.EjSvgRender.themes[this.model.theme].primaryYAxis);
                        for (var k = 2; k < this.model.axes.length; k++) {
                            if (this.model.axes[k].orientation.toLowerCase() == "horizontal")
                                $.extend(true, this.model.axes[k], ej.EjSvgRender.themes[this.model.theme].secondaryX);
                            else
                                $.extend(true, this.model.axes[k], ej.EjSvgRender.themes[this.model.theme].secondaryY);
                        }
                        break;
                    case "series":
                        $.extend(true, this.model.series, {}, options[prop]);
                        break;
                    case "legend":
                        $.extend(true, this.model.legend, {}, options[prop]);
                        break;
                    case "axes":
                        $.extend(true, this.model.axes, {}, options[prop]);
                        break;
                    case "primaryXAxis":
                        this.model.axes[0].setRange = (options[prop].range) ? true : false;
                        this.model.axes[0].actual_Range = (this.model.axes[0].setRange) ? null : this.model.axes[0].actual_Range;
                        $.extend(true, this.model.axes[0], {}, options[prop]);
                        break;
                    case "primaryYAxis":
                        this.model.axes[1].setRange = (options[prop].range) ? true : false;
                        this.model.axes[1].actual_Range = (this.model.axes[1].setRange) ? null : this.model.axes[1].actual_Range;
                        $.extend(true, this.model.axes[1], {}, options[prop]);
                        break;
                    case "xZoomFactor":
                        this.model.axes[0].zoomFactor = this._xZoomFactor();
                        break;
                    case "yZoomFactor":
                        this.model.axes[1].zoomFactor = this._yZoomFactor();
                        break;
                    case "xZoomPosition":
                        this.model.axes[0].zoomPosition = this._xZoomPosition();
                        break;
                    case "yZoomPosition":
                        this.model.axes[1].zoomPosition = this._yZoomPosition();
                        break;
                    case "drilldown":
                        this.model.series = [];
                        this.model.explodeValue = null;
                        $.extend(true, this.model, {}, options[prop]);
                        break;
                    case "datasource":
                        for (var i = 0; i < this.model.series.length; i++) {
                            this.model.series[i].dataSource = null;
                            this.model.series[i].points = null;
                        }
                        $.extend(true, this.model, {}, options[prop]);
                        break;

                    default:
                        $.extend(true, this.model, {}, options[prop]);
                }
            }
            $(this.svgObject).empty();
            this.bindTo();
        }
    });

    /**
	 * Enum for chart crosshairType.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.crosshairType = {
        /**  set crosshair for chart. */
        Crosshair: 'crosshair',
        /**  set trackBall for chart. */
        TrackBall: 'trackBall',
    };
    /**
	 * Enum for chart valueType.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.valueType = {
        /**  set valueType for chart to double. */
        Double: 'double',
        /**  set valueType for chart to dateTime. */
        DateTime: 'datetime',
        /**  set valueType for chart to category. */
        Category: 'category',
        /**  set valueType for chart to logarithmic. */
        Logarithmic: 'logarithmic'
    };
    /**
	 * Enum for chart series type.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.type = {
        /**  sets the seriesType of chart to line. */
        Line: 'line',
        /**  sets the seriesType of chart to spline. */
        Spline: 'spline',
        /**  sets the seriesType of chart to column. */
        Column: 'column',
        /**  sets the seriesType of chart to area. */
        Area: 'area',
        /**  sets the seriesType of chart to splinearea. */
        SplineArea: 'splinearea',
        /**  sets the seriesType of chart to stepline. */
        StepLine: 'stepline',
        /**  sets the seriesType of chart to steparea. */
        StepArea: 'steparea',
        /**  sets the seriesType of chart to pie. */
        Pie: 'pie',
        /**  sets the seriesType of chart to hilo. */
        Hilo: 'hilo',
        /**  sets the seriesType of chart to hiloopenclose. */
        HiloOpenClose: 'hiloopenclose',
        /**  sets the seriesType of chart to candle. */
        Candle: 'candle',
        /**  sets the seriesType of chart to bubble. */
        Bubble: 'bubble',
        /**  sets the seriesType of chart to bar. */
        Bar: 'bar',
        /**  sets the seriesType of chart to stackingarea. */
        StackingArea: 'stackingarea',
        /**  sets the seriesType of chart to rangecolumn. */
        RangeColumn: 'rangecolumn',
        /**  sets the seriesType of chart to stackingcolumn. */
        StackingColumn: 'stackingcolumn',
        /**  sets the seriesType of chart to stackingbar. */
        StackingBar: 'stackingbar',
        /**  sets the seriesType of chart to pyramid. */
        Pyramid: 'pyramid'
    };
    /**
	 * Enum for chart labelPlacement.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.labelPlacement = {
        /**  specifies the placement of labels between the ticks. */
        BetweenTicks: "betweenTicks",
        /**  specifies the placement of labels on the ticks. */
        OnTicks: "onTicks"
    };
    /**
	 * Enum for chart themes.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.theme = {
        /**  Render chart in azure theme. */
        Azure: 'azure',
        /**  Render chart in flatlight theme. */
        FlatLight: 'flatlight',
        /**  Render chart in azuredark theme. */
        Azuredark: 'azuredark',
        /**  Render chart in lime theme. */
        Lime: 'lime',
        /**  Render chart in limedark theme. */
        LimeDark: 'limedark',
        /**  Render chart in saffron theme. */
        Saffron: 'saffron',
        /**  Render chart in saffrondark theme. */
        SaffronDark: 'saffrondark',
        /**  Render chart in gradientlight theme. */
        GradientLight: 'gradientlight',
        /**  Render chart in gradientdark theme. */
        GradientDark: 'gradientdark'
    };
    /**
	 * Enum for chart style of font.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.fontStyle = {
        /**  Render normal font. */
        Normal: 'normal',
        /**  Render bold font. */
        Bold: 'bold',
        /**  Render italic font. */
        Italic: 'italic'
    };
    /**
	 * Enum for weight of font.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.fontWeight = {
        /**  Render regular font. */
        Regular: 'regular',
        /**  Render lighter font. */
        Lighter: 'lighter'
    };
    /**
	 * Enum for chart orientation.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.orientation = {
        /**  Sets chart orientation to horizontal. */
        Horizontal: 'horizontal',
        /**  Sets chart orientation to vertical. */
        Vertical: 'vertical'
    };
    /**
	 * Enum for chart interval type.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.intervalType = {
        /**  Sets chart interval type to days. */
        Days: 'days',
        /**  Sets chart interval type to hours. */
        Hours: 'hours',
        /**  Sets chart interval type to seconds. */
        Seconds: 'seconds',
        /**  Sets chart interval type to milliseconds. */
        Milliseconds: 'milliseconds',
        /**  Sets chart interval type to minutes. */
        Minutes: 'minutes',
        /**  Sets chart interval type to months. */
        Months: 'months',
        /**  Sets chart interval type to years. */
        Years: 'years'
    };
    /**
	 * Enum for range padding.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.rangePadding = {
        /**  Sets chart range padding as additional. */
        Additional: 'additional',
        /**  Sets chart range padding as normal. */
        Normal: 'normal',
        /**  Sets chart range padding as none. */
        None: 'none',
        /**  Sets chart range padding as round. */
        Round: 'round'
    };
    /**
	 * Enum for text alignment.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.textAlignment = {
        /**  Support for changing text alignment to middleTop. */
        MiddleTop: 'middletop',
        /**  Support for changing text alignment to middleCenter. */
        MiddleCenter: 'middlecenter',
        /**  Support for changing text alignment to middleBottom. */
        MiddleBottom: 'middlebottom'
    };
    /**
	 * Enum for label position.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.labelPosition = {
        /**  Support for changing the labelPosition to inside. */
        Inside: 'inside',
        /**  Support for changing the labelPosition to outside. */
        Outside: 'outside'
    };
    /**
	 * Enum for zOrder.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.zOrder = {
        /**  Support for changing the zOrder to over. */
        Over: 'over',
        /**  Support for changing the labelPosition to behind. */
        Behind: 'behind'
    };
    /**
	 * Enum for chart unit.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.unit = {
        /**  Specifies the unit in percentage. */
        percentage: 'percentage',
        /**  Specifies the unit in pixel. */
        pixel: 'pixel'
    };
    /**
	 * Enum for chart pyramid mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.pyramidMode = {
        /**  Specifies the mode of the pyramid as linear. */
        Linear: 'linear',
        /**  Specifies the mode of the pyramid as surface. */
        Surface: 'Surface'
    };
    /**
	 * Enum for chart shape.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.shape = {
        /**  Sets the shape to none. */
        None: 'none',
        /**  Sets the shape to leftarrow. */
        LeftArrow: 'leftarrow',
        /**  Sets the shape to rightarrow. */
        RightArrow: 'rightarrow',
        /**  Sets the shape to circle. */
        Circle: 'circle',
        /**  Sets the shape to cross. */
        Cross: 'cross',
        /**  Sets the shape to horizline. */
        HorizLine: 'horizline',
        /**  Sets the shape to vertline. */
        VertLine: 'vertLine',
        /**  Sets the shape to diamond. */
        Diamond: 'diamond',
        /**  Sets the shape to rectangle. */
        Rectangle: 'rectangle',
        /**  Sets the shape to triangle. */
        Triangle: 'triangle',
        /**  Sets the shape to invertedtriangle. */
        InvertedTriangle: 'invertedtriangle',
        /**  Sets the shape to hexagon. */
        Hexagon: 'hexagon',
        /**  Sets the shape to pentagon. */
        Pentagon: 'pentagon',
        /**  Sets the shape to star. */
        Star: 'star',
        /**  Sets the shape to ellipse. */
        Ellipse: 'ellipse',
        /**  Sets the shape to wedge. */
        Wedge: 'wedge',
        /**  Sets the shape to trapezoid. */
        Trapezoid: 'trapezoid',
        /**  Sets the shape to uparrow. */
        UpArrow: 'uparrow',
        /**  Sets the shape to downarrow. */
        DownArrow: 'downarrow',
        /**  Sets the shape to image. */
        Image: 'image'
    };
    /**
	 * Enum for hilopenclose drawmode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.drawmode = {
        /**  Specifies the draw mode as both. */
        Both: 'both',
        /**  Specifies the draw mode as open. */
        Open: 'open',
        /**  Specifies the draw mode as close */
        Close: 'close'
    };
    /**
	 * Enum for linecap.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.lineCap = {
        /**  Sets the linecap as butt. */
        Butt: 'butt',
        /**  Sets the linecap as round. */
        Round: 'round',
        /**  Sets the linecap as square. */
        Square: 'square'
    };
    /**
	 * Enum for line join.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.lineJoin = {
        /**  Sets the lineJoin to round. */
        Round: 'round',
        /**  Sets the lineJoin to bevel. */
        Bevel: 'bevel',
        /**  Sets the lineJoin to miter. */
        Miter: 'miter'
    };
    /**
	 * Enum for position.	 
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.position = {
        /**  Sets the position to top. */
        Top: 'top',
        /**  Sets the position to middle. */
        Middle: 'middle',
        /**  Sets the position to bottom. */
        Bottom: 'bottom'
    };
    /**
	 * Enum for alignment.	 12
	 * @enum {string}
	 * @global 
	 */
    ej.Chart.alignment = {
        /**  Sets the lineJoin to center. */
        Center: 'center',
        /**  Sets the lineJoin to near. */
        Near: 'near',
        /**  Sets the lineJoin to far. */
        Far: 'far'
    };
    $.extend(ej.Chart.prototype, ej.ejChart);
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin for Circular Gauge
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 11.2 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    /**
    * @namespace ej
    * @class ejCircularGauge
    * @requires jQuery
    * @requires ej.common.all
    * @classdesc Design for Circular Gauge.2
    * @example 
    * &lt;div id="CoreCircularGauge"&gt;
    * &lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create Circular Gauge
    * $(function () {
        $("#CoreCircularGauge").ejCircularGauge({
        });
    });
    * &lt;/script&gt;
    */

    ej.widget({ "ejCircularGauge": "ej.CircularGauge" /** @lends ejCircularGauge# */ }, {
        validTags: ["div", "span"],
        // default model
        defaults: /** @lends ejCircularGauge# */{
            /**		
			* Specifies the value of circular gauge.
			* @default 0
            * @alias ejCircularGauge#value
			* @type {number}
			* @example 
			* //To set API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ value: 0 });
			* @example 
			* //Get or set the API, after initialization:<br>
			*	//Gets the value  
			*	$("#CoreCircularGauge").ejCircularGauge("option", "value");<br>			
			*	//Sets the value 
			*	$("#CoreCircularGauge").ejCircularGauge("option", "value",  0 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            value: 0,
            /**		
			* Specifies the minimum value of circular gauge.
			* @default 0
            * @alias ejCircularGauge#minimum
			* @type {number}
			* @example 
			* //To set minimum API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ minimum: 0 });
			* @example 
			* //Get or set the minimum API, after initialization:<br>
			*	//Gets the minimum value  
			*	$("#CoreCircularGauge").ejCircularGauge("option", "minimum");<br>			
			*	//Sets the minimum value 
			*	$("#CoreCircularGauge").ejCircularGauge("option", "minimum",  0 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            minimum: 0,
            /**		
			* Specifies the maximum value of circular gauge.
			* @default 100
            * @alias ejCircularGauge#maximum
			* @type {number}
			* @example 
			* //To set maximum API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ maximum: 100 });
			* @example 
			* //Get or set the maximum API, after initialization:<br>
			*	//Gets the maximum value  
			*	$("#CoreCircularGauge").ejCircularGauge("option", "maximum");<br>			
			*	//Sets the maximum value 
			*	$("#CoreCircularGauge").ejCircularGauge("option", "maximum",  100 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            maximum: 100,
            /**		
			* Specifies the radius of circular gauge.
			* @default 180
            * @alias ejCircularGauge#radius
			* @type {number}
			* @example 
			* //To set radius API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ radius: 180 });
			* @example 
			* //Get or set the radius API, after initialization:<br>
			*	//Gets the radius value  
			*	$("#CoreCircularGauge").ejCircularGauge("option", "radius");<br>			
			*	//Sets the radius value 
			*	$("#CoreCircularGauge").ejCircularGauge("option", "radius",  180 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            radius: 180,
            /**		
			* Specifies the width of circular gauge.
			* @default 360
            * @alias ejCircularGauge#width
			* @type {number}
			* @example 
			* //To set width API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ width: 360 });
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value  
			*	$("#CoreCircularGauge").ejCircularGauge("option", "width");<br>			
			*	//Sets the width value 
			*	$("#CoreCircularGauge").ejCircularGauge("option", "width",  360 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            width: 360, // Width Should be 2*radious
            /**		
			* Specifies the height of circular gauge.
            * @alias ejCircularGauge#height
			* @default 360
			* @type {number}
			* @example 
			* //To set height API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ height: 360 });
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value  
			*	$("#CoreCircularGauge").ejCircularGauge("option", "height");<br>			
			*	//Sets the height value 
			*	$("#CoreCircularGauge").ejCircularGauge("option", "height",  360 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            height: 360, //Height Should be 2*radious
            /**		
			* Specifies the frameType of circular gauge. See {@link frame}
			* @default ej.CircularGauge.frame.fullCircle
            * @alias ejCircularGauge#frameType
			* @type {enum}
			* @example 
			* // Set the gauge frameType on initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({  frameType : "fullCircle" });
			* @example 
			* //Get or set the gauge frameType, after initialization:<br>
			*	// Gets the gauge frameType.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "frameType");			
			*	// Sets the gauge frameType.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "frameType",  "fullCircle" );
			 * @memberof ejCircularGauge
			* @instance
			*/
            frameType: "fullCircle", // ["fullCircle", "halfCircle"],
            /**		
			* Specifies the backgroundcolor of circular gauge.
			* @default null
            * @alias ejCircularGauge#backgroundColor
			* @type {string}
			* @example 
			* // Set the gauge backgroundcolor on initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({  backgroundColor : "#FFFFFF" });
			* @example 
			* //Get or set the gauge backgroundcolor, after initialization:<br>
			*	// Gets the gauge backgroundcolor.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "backgroundColor");			
			*	// Sets the gauge backgroundcolor.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "backgroundColor",  "#1A1A1A" );
			 * @memberof ejCircularGauge
			* @instance
			*/
            backgroundColor: null,
            /**		
			* Specifies the interiorGradient of circular gauge.
			* @default null
            * @alias ejCircularGauge#interiorGradient
			* @type {string}
			* @example 
			* // Set the gauge interiorGradient on initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({ interiorGradient: { colorInfo:[{colorStop : 0, color:"Green"},{colorStop : 1, color:"Yellow"}] } });
			* @example 
			* //Get or set the gauge interiorGradient after initialization:<br>
			*	// Gets the gauge interiorGradient.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "interiorGradient");			
			*	// Sets the gauge interiorGradient.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "interiorGradient",  { colorInfo:[{colorStop : 0, color:"Green"},{colorStop : 1, color:"Yellow"}] } );
			 * @memberof ejCircularGauge
			* @instance
			*/
            interiorGradient: null, //{colorInfo:[{colorStop : 0, color:"Green"},{colorStop : 1, color:"Yellow"}] },
            /**		
			* Specify readonly value of circular gauge
			* @default true
            * @alias ejCircularGauge#readOnly
			* @type {boolean}
			* @example 
			* // Set the readonly value during initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({  readOnly : true });
			* //Get or set the readonly value after initialization:<br>
			*	// Gets the readonly value.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "readOnly");			
			*	// Sets the readonly value.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "readOnly",  true );
			 * @memberof ejCircularGauge
			* @instance
			*/
            readOnly: true,
            /**		
			* Specifies the start angle for the half circular frame.
			* @default 180
            * @alias ejCircularGauge#halfCircleFrameStartAngle
			* @type {number}
			* @example 
			* //To set start angle for the half circular frame API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ halfCircleFrameStartAngle: 180 });
			* @example 
			* //Get or set the start angle for the half circular frame API, after initialization:<br>
			*	//Gets the start angle for the half circular frame value  
			*	$("#CoreCircularGauge").ejCircularGauge("option", "halfCircleFrameStartAngle");<br>			
			*	//Sets the start angle for the half circular frame value 
			*	$("#CoreCircularGauge").ejCircularGauge("option", "halfCircleFrameStartAngle", 180 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            halfCircleFrameStartAngle: 180,
            /**		
			* Specifies the end angle for the half circular frame.
			* @default 360
            * @alias ejCircularGauge#halfCircleFrameEndAngle
			* @type {number}
			* @example 
			* //To set end angle for the half circular frame API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ halfCircleFrameEndAngle: 360 });
			* @example 
			* //Get or set the end angle for the half circular frame API, after initialization:<br>
			*	//Gets the end angle for the half circular frame value  
			*	$("#CoreCircularGauge").ejCircularGauge("option", "halfCircleFrameEndAngle");<br>			
			*	//Sets the end angle for the half circular frame value 
			*	$("#CoreCircularGauge").ejCircularGauge("option", "halfCircleFrameEndAngle", 360 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            halfCircleFrameEndAngle: 360,
            /**		
			* Specify animate value of circular gauge
			* @default true
            * @alias ejCircularGauge#animate
			* @type {boolean}
			* @example 
			* // Set the animate value during initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({ animate : true });
			* @example 
			* //Get or set the animate value after initialization:<br>
			*	// Gets the animate value.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "animate");			
			*	// Sets the animate value.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "animate", true );
			 * @memberof ejCircularGauge
			* @instance
			*/
            animate: true,
            /**		
			* Specifies animationSpeed of circular gauge
			* @default 500
            * @alias ejCircularGauge#animationSpeed
			* @type {number}
			* @example 
			* //To set animationSpeed API value during initialization  
			* 	$("#CoreCircularGauge").ejCircularGauge({ animationSpeed: 500 });
			* @example 
			* //Get or set animationSpeed API, after initialization:<br>
			*	//Gets animationSpeed value.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "animationSpeed");<br>			
			*	//Sets animationSpeed value.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "animationSpeed", 500 ); 
			 * @memberof ejCircularGauge
			* @instance
			*/
            animationSpeed: 500,
            /**		
			* Specify the theme of circular gauge.
			* @default "flatlight"
            * @alias ejCircularGauge#theme
			* @type {string}
			* @example 
			* // Set the theme during initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({  theme : "flatlight" });
			* @example 
			* //Get or set the theme after initialization:<br>
			*	// Gets the theme.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "theme");			
			*	// Sets the theme.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "theme", "flatlight" );
			 * @memberof ejCircularGauge
			* @instance
			*/
            theme: "flatlight",
            /**		
			* Specify isRadialGradient value of circular gauge
			* @default true
            * @alias ejCircularGauge#isRadialGradient
			* @type {boolean}
			* @example 
			* // Set the isRadialGradient value during initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({  isRadialGradient : true });
			* @example 
			* //Get or set the isRadialGradient value after initialization:<br>
			*	// Gets the isRadialGradient value.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "isRadialGradient");			
			*	// Sets the isRadialGradient value.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "isRadialGradient",  true );
			 * @memberof ejCircularGauge
			* @instance
			*/
            isRadialGradient: false,
            /**		
			* Specify canResize value of circular gauge
			* @default true
            * @alias ejCircularGauge#canResize
			* @type {boolean}
			* @example 
			* // Set the canResize value during initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({  canResize : true });
			* @example 
			* //Get or set the canResize value after initialization:<br>
			*	// Gets the canResize value.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "canResize");			
			*	// Sets the canResize value.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "canResize",  true );
			 * @memberof ejCircularGauge
			* @instance
			*/
            canResize: false,
            /**		
			* Specify the url of the frame background image for circular gauge
			* @default ""
            * @alias ejCircularGauge#frameBackgroundImageUrl
			* @type {boolean}
			* @example 
			* // Set the frameBackgroundImageUrl value during initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({  frameBackgroundImageUrl : "Sun.jpg" });
			* @example 
			* //Get or set the frameBackgroundImageUrl value after initialization:<br>
			*	// Gets the frameBackgroundImageUrl value.		
			*	$("#CoreCircularGauge").ejCircularGauge("option", "frameBackgroundImageUrl");			
			*	// Sets the frameBackgroundImageUrl value.
			*	$("#CoreCircularGauge").ejCircularGauge("option", "frameBackgroundImageUrl",  "Moon.png" );
			 * @memberof ejCircularGauge
			* @instance
			*/
            frameBackgroundImageUrl: "",

            //Events

            /**     
			 * Triggers while the ticks are being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawTicks		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.context returns the context element
            * @param {Object} args.location returns the startX and startY of the ticks
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.scaleElement returns the options of the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the tick belongs.
            * @param {string} args.style returns the ticks style
            * @param {number} args.tickAngle returns the angle of the tick.
            * @param {Object} args.tickElement returns the current tick element.
            * @param {number} args.tickIndex returns the index of the tick.
            * @param {number} args.tickValue returns the label value of the tick.
            * @param {string} args.type returns the name of the event
			 * @example 
			 * //create event for drawTicks
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawTicks: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawTicks: null,
            /**     
			 * Triggers while the labels are being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawLabels		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.context returns the context element
            * @param {Object} args.location returns the startX and startY of the labels
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.scaleElement returns the options of the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the label belongs.
            * @param {string} args.style returns the label style
            * @param {number} args.labelAngle returns the angle of the labels.
            * @param {Object} args.labelElement returns the current label element.
            * @param {number} args.labelIndex returns the index of the label.
            * @param {number} args.labelValue returns the value of the label.
            * @param {string} args.type returns the name of the event
			 * @example 
			 * //create event for drawLabels
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawLabels: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawLabels: null,
            /**     
			 * Triggers while the pointers are being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawPointers		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.context returns the context element
            * @param {Object} args.location returns the startX and startY of the pointer
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.scaleElement returns the options of the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
            * @param {string} args.style returns the pointer style
            * @param {number} args.pointerAngle returns the angle of the pointer.
            * @param {Object} args.pointerElement returns the current pointer element.
            * @param {number} args.pointerIndex returns the index of the pointer.
            * @param {number} args.pointerValue returns the value of the pointer.
            * @param {string} args.type returns the name of the event
			 * @example 
			 * //create event for drawPointers
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawPointers: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawPointers: null,
            /**     
			 * Triggers when the ranges begin to be getting drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawRange		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.context returns the context element
            * @param {Object} args.location returns the startX and startY of the range
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.scaleElement returns the options of the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the range belongs.
            * @param {string} args.style returns the range style
            * @param {Object} args.rangeElement returns the current range element.
            * @param {number} args.rangeIndex returns the index of the range.
            * @param {string} args.type returns the name of the event
			 * @example 
			 * //create event for drawRange
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawRange: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawRange: null,
            /**     
			 * Triggers while the custom labels are being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawCustomLabel		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.context returns the context element
            * @param {Object} args.location returns the startX and startY of the custom label
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.scaleElement returns the options of the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the custom label belongs.
            * @param {string} args.style returns the custom label style
            * @param {Object} args.customLabelElement returns the current custom label element.
            * @param {number} args.customLabelIndex returns the index of the custom label.
            * @param {string} args.type returns the name of the event
			 * @example 
			 * //create event for drawCustomLabel
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawCustomLabel: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawCustomLabel: null,
            /**     
			 * Triggers while the indicators are being started to drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawIndicators		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.context returns the context element
            * @param {Object} args.location returns the startX and startY of the indicator
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.scaleElement returns the options of the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the indicator belongs.
            * @param {string} args.style returns the indicator style
            * @param {Object} args.indicatorElement returns the current indicator element.
            * @param {number} args.indicatorIndex returns the index of the indicator.
            * @param {string} args.type returns the name of the event
			 * @example 
			 * //create event for drawIndicators
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawIndicators: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawIndicators: null,
            /**     
			 * Triggers while the pointer cap is being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawPointerCap		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.context returns the context element
            * @param {Object} args.scaleElement returns the options of the scale element.
            * @param {Object} args.location returns the startX and startY of the pointer cap.
            * @param {Object} args.model returns the gauge model
            * @param {string} args.style returns the pointer cap style
            * @param {string} args.type returns the name of the event
			 * @example 
			 * //create event for drawPointerCap
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawPointerCap: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawPointerCap: null,
            /**     
			 * Triggers when the rendering of the gauge is completed.
			 * @event
			 * @name ejCircularGauge#renderComplete		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.context returns the context element
            * @param {Object} args.scaleElement returns the entire scale element.
            * @param {Object} args.model returns the gauge model
            * @param {string} args.type returns the name of the event
			 * @example 
			 * //create event for renderComplete
             * $("#CoreCircularGauge").ejCircularGauge({
             *    renderComplete: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            renderComplete: null,
            /**     
			 * Triggers when the left mouse button is clicked.
			 * @event
			 * @name ejCircularGauge#mouseClick		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.type returns the name of the event
            * @param {Object} args.scaleElement returns the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
            * @param {number} args.pointerIndex returns the pointer Index
            * @param {Object} args.pointerElement returns the pointer element.
            * @param {Object} args.context returns the context element
            * @param {number} args.pointerValue returns the value of the pointer.
            * @param {number} args.pointerAngle returns the angle of the pointer.
            * @param {string} args.style returns the pointer style
            * @param {Object} args.location returns the startX and startY of the pointer.
			 * @example 
			 * //create event for mouseClick
             * $("#CoreCircularGauge").ejCircularGauge({
             *    mouseClick: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            mouseClick: null,
            /**     
			 * Triggers when clicking and dragging the mouse pointer over the gauge pointer.
			 * @event
			 * @name ejCircularGauge#mouseClickMove		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.type returns the name of the event
            * @param {Object} args.scaleElement returns the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
            * @param {number} args.pointerIndex returns the pointer Index
            * @param {Object} args.pointerElement returns the pointer element.
            * @param {Object} args.context returns the context element
            * @param {number} args.pointerValue returns the value of the pointer.
            * @param {number} args.pointerAngle returns the angle of the pointer.
            * @param {string} args.style returns the pointer style
            * @param {Object} args.location returns the startX and startY of the pointer.
			 * @example 
			 * //create event for mouseClickMove
             * $("#CoreCircularGauge").ejCircularGauge({
             *    mouseClickMove: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            mouseClickMove: null,
            /**     
			 * Triggers when the mouse click is released.
			 * @event
			 * @name ejCircularGauge#mouseClickUp		
            * @param {Object} args.Object returns the object of the gauge.
            * @param {boolean} args.cancel returns the cancel option value
            * @param {Object} args.model returns the gauge model
            * @param {Object} args.type returns the name of the event
            * @param {Object} args.scaleElement returns the scale element.
            * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
            * @param {number} args.pointerIndex returns the pointer Index
            * @param {Object} args.pointerElement returns the pointer element.
            * @param {Object} args.context returns the context element
            * @param {number} args.pointerValue returns the value of the pointer.
            * @param {number} args.pointerAngle returns the angle of the pointer.
            * @param {string} args.style returns the pointer style
            * @param {Object} args.location returns the startX and startY of the pointer.
			 * @example 
			 * //create event for mouseClickUp
             * $("#CoreCircularGauge").ejCircularGauge({
             *    mouseClickUp: function (args) {}
             * });
			 * @memberof ejCircularGauge
			 * @instance
			 */
            mouseClickUp: null,

            //Scale Collection
            /**		
			* Specify the pointers, ticks, labels, indicators, ranges  of circular gauge
			* @default [Array.object] scale = [{ scaleBarSize: 6, scaleBorderWidth: 1.5, capBorderColor: null, capBackgroundColor: null }]
			* @type {object}
			* @example 
			* // Set the pointers, ticks, labels, indicators, ranges value during initialization. 			
			* 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ scaleBarSize: 6, scaleBorderWidth: 1.5 }] });
			* @example 
			* //Get or set the pointers, ticks, labels, indicators, ranges value after initialization:<br>
			*   //Gets the pointers, ticks, labels, indicators, ranges value.
            *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
            *   // index refers the index value of the scale
            *   var scalesize = scale[index];
			*   //Sets the pointers, ticks, labels, indicators, ranges value.
			*   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ scaleBarSize: 6, scaleBorderWidth: 1.5 }]);
			    * @memberof ejCircularGauge
			* @instance
			*/
            scales: null
        },
        // * @memberof ejCircularGauge

        _defaultScaleValues: function () {
            return {
                /**		
			    * Specify scaleBarSize of circular gauge
			    * @default [Array.number] scale.scaleBarSize = 6
                * @alias ejCircularGauge#scales->scaleBarSize
			    * @type {number}
			    * @example 
			    * // Set the scaleBarSize value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ scaleBarSize: 6 }] });
			    * @example 
			    * //Get or set the scaleBarSize value after initialization:<br>
			    *   //Gets the scaleBarSize value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var scalesize = scale[index].scaleBarSize;
			    *   //Sets the scaleBarSize value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ scaleBarSize: 6 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                scaleBarSize: 6,
                /**		
			    * Specify scaleBorderWidth of circular gauge
			    * @default [Array.double] scale.scaleBorderWidth = 1.5
                * @alias ejCircularGauge#scales->scaleBorderWidth
			    * @type {double}
			    * @example 
			    * // Set the scaleBorderWidth value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ scaleBorderWidth: 1.5 }] });
			    * @example 
			    * //Get or set the scaleBorderWidth value after initialization:<br>
			    *   //Gets the scaleBorderWidth value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var scaleborder = scale[index].scaleBorderWidth;
			    *   //Sets the scaleBorderWidth value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ scaleBorderWidth: 1.5 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                scaleBorderWidth: 1.5,
                /**		
			    * Specify capBorderColor of circular gauge
			    * @default [Array.string] scale.capBorderColor = null
                * @alias ejCircularGauge#scales->capBorderColor
			    * @type {string}
			    * @example 
			    * // Set the capBorderColor during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ capBorderColor: "#FFFFFF" }] });
			    * @example 
			    * //Get or set the capBorderColor after initialization:<br>
			    *   //Gets the capBorderColor.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var capborder = scale[index].capBorderColor;
			    *   //Sets the capBorderColor.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ capBorderColor: "#FFFFFF" }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                capBorderColor: null,
                /**		
			    * Specify capBackgroundColor of circular gauge
			    * @default [Array.string] scale.capBackgroundColor = null
                * @alias ejCircularGauge#scales->capBackgroundColor
			    * @type {string}
			    * @example 
			    * // Set the capBackgroundColor during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ capBackgroundColor: "#1A1B2C" }] });
			    * @example 
			    * //Get or set the capBackgroundColor after initialization:<br>
			    *   //Gets the capBackgroundColor.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var capbackground = scale[index].capBackgroundColor;
			    *   //Sets the capBackgroundColor.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ capBackgroundColor: "#1A1B2C" }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                capBackgroundColor: null,
                /**		
			    * Specify showScaleBar of circular gauge
			    * @default [Array.boolean] scale.showScaleBar = false
                * @alias ejCircularGauge#scales->showScaleBar
			    * @type {boolean}
			    * @example 
			    * // Set the showScaleBar value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showScaleBar: false }] });
			    * @example 
			    * //Get or set the showScaleBar value after initialization:<br>
			    *   //Gets the showScaleBar value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var showscale = scale[index].showScaleBar;
			    *   //Sets the showScaleBar value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ showScaleBar: false }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showScaleBar: false,
                /**		
			    * Specify sweepAngle of circular gauge
			    * @default [Array.number] scale.sweepAngle = 310
                * @alias ejCircularGauge#scales->sweepAngle
			    * @type {number}
			    * @example 
			    * // Set the sweepAngle value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ sweepAngle: 310 }] });
			    * @example 
			    * //Get or set the sweepAngle value after initialization:<br>
			    *   //Gets the sweepAngle value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var sweepangle = scale[index].sweepAngle;
			    *   //Sets the sweepAngle value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ sweepAngle: 310 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                sweepAngle: 310,
                /**		
			    * Specify scaleRadius of circular gauge
			    * @default [Array.number] scale.scaleRadius = 170
                * @alias ejCircularGauge#scales->scaleRadius
			    * @type {number}
			    * @example 
			    * // Set the scaleRadius value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ scaleRadius: 170 }] });
			    * @example 
			    * //Get or set the scaleRadius value after initialization:<br>
			    *   //Gets the scaleRadius value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var radius = scale[index].scaleRadius;
			    *   //Sets the scaleRadius value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ scaleRadius: 170 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                scaleRadius: 170,
                /**		
			    * Specify startAngle of circular gauge
			    * @default [Array.number] scale.startAngle = 115
                * @alias ejCircularGauge#scales->startAngle
			    * @type {number}
			    * @example 
			    * // Set the startAngle value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ startAngle: 115 }] });
			    * @example 
			    * //Get or set the startAngle value after initialization:<br>
			    *   //Gets the startAngle value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var angle = scale[index].startAngle;
			    *   //Sets the startAngle value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ startAngle: 115 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                startAngle: 115,
                /**		
			    * Specify majorIntervalValue of circular gauge
			    * @default [Array.number] scale.majorIntervalValue = 10
                * @alias ejCircularGauge#scales->majorIntervalValue
			    * @type {number}
			    * @example 
			    * // Set the majorIntervalValue value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ majorIntervalValue: 10 }] });
			    * @example 
			    * //Get or set the majorIntervalValue value after initialization:<br>
			    *   //Gets the majorIntervalValue value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var interval = scale[index].majorIntervalValue;
			    *   //Sets the majorIntervalValue value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ majorIntervalValue: 10 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                majorIntervalValue: 10,
                /**		
			    * Specify minorIntervalValue of circular gauge
			    * @default [Array.number] scale.minorIntervalValue = 2
                * @alias ejCircularGauge#scales->minorIntervalValue
			    * @type {number}
			    * @example 
			    * // Set the minorIntervalValue value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ minorIntervalValue: 2 }] });
			    * @example 
			    * //Get or set the minorIntervalValue value after initialization:<br>
			    *   //Gets the minorIntervalValue value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var interval = scale[index].minorIntervalValue;
			    *   //Sets the minorIntervalValue value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ minorIntervalValue: 2 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                minorIntervalValue: 2,
                /**		
			    * Specify maximum scale value of circular gauge
			    * @default [Array.number] scale.maximum = null
                * @alias ejCircularGauge#scales->maximum
			    * @type {number}
			    * @example 
			    * // Set the maximum scale value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ maximum: 100 }] });
			    * @example 
			    * //Get or set the maximum scale value after initialization:<br>
			    *   //Gets the maximum scale value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var maxVal = scale[index].maximum;
			    *   //Sets the maximum scale value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ maximum: 100 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                maximum: null,
                /**		
			    * Specify minimum scale value of circular gauge
			    * @default [Array.number] scale.minimum = null
                * @alias ejCircularGauge#scales->minimum
			    * @type {number}
			    * @example 
			    * // Set the minimum scale value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ minimum: 100 }] });
			    * @example 
			    * //Get or set the minimum scale value after initialization:<br>
			    *   //Gets the minimum scale value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var minVal = scale[index].minimum;
			    *   //Sets the minimum scale value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ minimum: 100 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                minimum: null,
                /**		
			    * Specify borderColor for scales of circular gauge
			    * @default [Array.string] scale.borderColor = null
                * @alias ejCircularGauge#scales->borderColor
			    * @type {string}
			    * @example 
			    * // Set the scale borderColor during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ borderColor: "#1A1A1A" }] });
			    * @example 
			    * //Get or set scale the borderColor after initialization:<br>
			    *   //Gets the scale borderColor.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var border = scale[index].borderColor;
			    *   //Sets the scale borderColor.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ borderColor: "#1A1A1A" }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                borderColor: null,
                /**		
			    * Specify backgroundColor for the scale of circular gauge
			    * @default [Array.string] scale.backgroundColor = null
                * @alias ejCircularGauge#scales->backgroundColor
			    * @type {string}
			    * @example 
			    * // Set the scale backgroundColor during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ backgroundColor: "#1A1A1A" }] });
			    * @example 
			    * //Get or set the scale backgroundColor after initialization:<br>[Array.string] scale.backgroundColor
			    *   //Gets the scale backgroundColor.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var background = scale[index].backgroundColor;
			    *   //Sets the scale backgroundColor.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ backgroundColor: "#1A1A1A" }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                backgroundColor: null,
                /**		
			    * Specify labelAutoAngle value of circular gauge
			    * @default [Array.boolean] scale.labelAutoAngle = false
                * @alias ejCircularGauge#scales->labelAutoAngle
			    * @type {boolean}
			    * @example 
			    * // Set the labelAutoAngle value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labelAutoAngle: false }] });
			    * @example 
			    * //Get or set the labelAutoAngle value after initialization:<br>
			    *   //Gets the labelAutoAngle value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var anglevalue = scale[index].labelAutoAngle;
			    *   //Sets the labelAutoAngle value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labelAutoAngle: false }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                labelAutoAngle: false,
                /**		
			    * Specify scaleDirection of circular gauge. See {@link scaleDirection}
			    * @default [Array.enum] scale.scaleDirection = "Clockwise"
                * @alias ejCircularGauge#scales->scaleDirection
			    * @type {enum}
			    * @example 
			    * // Set the scaleDirection during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ scaleDirection: "Clockwise" }] });
			    * @example 
			    * //Get or set the scaleDirection after initialization:<br>
			    *   //Gets the scaleDirection.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var direction = scale[index].scaleDirection;
			    *   //Sets the scaleDirection.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ scaleDirection: "Clockwise" }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                scaleDirection: "Clockwise", //["Clockwise", "CounterClockwise"]
                /**		
			    * Specify showPointers of circular gauge
			    * @default [Array.boolean] scale.showPointers = true
                * @alias ejCircularGauge#scales->showPointers
			    * @type {boolean}
			    * @example 
			    * // Set the showPointers during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showPointers: true }] });
			    * @example 
			    * //Get or set the showPointers after initialization:<br>
			    *   //Gets the showPointers.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var pointer = scale[index].showPointers;
			    *   //Sets the showPointers.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ showPointers: true }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showPointers: true,
                /**		
			    * Specify showRanges of circular gauge
			    * @default [Array.boolean] scale.showRanges = false
                * @alias ejCircularGauge#scales->showRanges
			    * @type {boolean}
			    * @example 
			    * // Set the showRanges during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showRanges: false }] });
			    * @example 
			    * //Get or set the showRanges after initialization:<br>
			    *   //Gets the showRanges.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var range = scale[index].showRanges;
			    *   //Sets the showRanges.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ showRanges: false }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showRanges: false,
                /**		
			    * Specify showTicks of circular gauge
			    * @default [Array.boolean] scale.showTicks = true
                * @alias ejCircularGauge#scales->showTicks
			    * @type {boolean}
			    * @example 
			    * // Set the showTicks during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showTicks: true }] });
			    * @example 
			    * //Get or set the showTicks after initialization:<br>
			    *   //Gets the showTicks.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var tick = scale[index].showTicks;
			    *   //Sets the showTicks.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ showTicks: true }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showTicks: true,
                /**		
			    * Specify showLabels of circular gauge
			    * @default [Array.boolean] scale.showLabels = true
                * @alias ejCircularGauge#scales->showLabels
			    * @type {boolean}
			    * @example 
			    * // Set the showLabels during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showLabels: true }] });
			    * @example 
			    * //Get or set the showLabels after initialization:<br>
			    *   //Gets the showLabels.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var label = scale[index].showLabels;
			    *   //Sets the showLabels.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ showLabels: true }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showLabels: true,
                /**		
			    * Specify showIndicators of circular gauge
			    * @default [Array.boolean] scale.showIndicators = false
                * @alias ejCircularGauge#scales->showIndicators
			    * @type {boolean}
			    * @example 
			    * // Set the showIndicators during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showIndicators: false }] });
			    * @example 
			    * //Get or set the showIndicators after initialization:<br>
			    *   //Gets the showIndicators.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var indicator = scale[index].showIndicators;
			    *   //Sets the showIndicators.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ showIndicators: false }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showIndicators: false,
                /**		
			    * Specify pointerCapRadius value of circular gauge
			    * @default [Array.number] scale.pointerCapRadius = 7
                * @alias ejCircularGauge#scales->pointerCapRadius
			    * @type {number}
			    * @example 
			    * // Set the pointerCapRadius value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointerCapRadius: 10 }] });
			    * @example 
			    * //Get or set the pointerCapRadius value after initialization:<br>
			    *   //Gets the pointerCapRadius value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var capradius = scale[index].pointerCapRadius;
			    *   //Sets the pointerCapRadius value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointerCapRadius: 10 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                pointerCapRadius: 7,
                /**		
			    * Specify pointerCapBorderWidth value of circular gauge
			    * @default [Array.double] scale.pointerCapBorderWidth = 3
                * @alias ejCircularGauge#scales->pointerCapBorderWidth
			    * @type {double}
			    * @example 
			    * // Set the pointerCapBorderWidth value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointerCapBorderWidth: 3 }] });
			    * @example 
			    * //Get or set the pointerCapBorderWidth value after initialization:<br>
			    *   //Gets the pointerCapBorderWidth value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var capborder = scale[index].pointerCapBorderWidth;
			    *   //Sets the pointerCapBorderWidth value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointerCapBorderWidth: 3 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                pointerCapBorderWidth: 3,
                /**		
			    * Specify capInteriorGradient value of circular gauge
			    * @default [Array.string] scale.capInteriorGradient = null
                * @alias ejCircularGauge#scales->capInteriorGradient
			    * @type {string}
			    * @example 
			    * // Set the capInteriorGradient value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ capInteriorGradient: {colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }}] });
			    * @example 
			    * //Get or set the capInteriorGradient value after initialization:<br>
			    *   //Gets the capInteriorGradient value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var capgradient = scale[index].capInteriorGradient;
			    *   //Sets the capInteriorGradient value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{capInteriorGradient: {colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}]} }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                capInteriorGradient: null, //{colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }
                /**		
			    * Specify opacity value of circular gauge
			    * @default [Array.number] scale.opacity = null
                * @alias ejCircularGauge#scales->opacity
			    * @type {string}
			    * @example 
			    * // Set the opacity value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ opacity:0.5 }] });
			    * @example 
			    * //Get or set the opacity value after initialization:<br>
			    *   //Gets the opacity value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var capgradient = scale[index].opacity;
			    *   //Sets the opacity value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{opacity:1 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                opacity: 1,
                /**		
			    * Specify shadowOffset value of circular gauge
			    * @default [Array.string] scale.shadowOffset = 0
                * @alias ejCircularGauge#scales->shadowOffset
			    * @type {string}
			    * @example 
			    * // Set the shadowOffset value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ shadowOffset: 1}] });
			    * @example 
			    * //Get or set the shadowOffset value after initialization:<br>
			    *   //Gets the shadowOffset value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // index refers the index value of the scale
                *   var capgradient = scale[index].shadowOffset;
			    *   //Sets the shadowOffset value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{shadowOffset: 1 }]);
			     * @memberof ejCircularGauge
			    * @instance
			    */
                shadowOffset: 0,
                // Pointers Collection
                /**		
			    * Specify pointers value of circular gauge
			    * @default [Array.object] scale.pointers = [{ distanceFromScale: 0, showBackNeedle: false, backNeedleLength: 10, pointerLength: 150 }]
                * @alias ejCircularGauge#scales->pointers
			    * @type {object}
			    * @example 
			    * // Set the pointers value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ distanceFromScale: 0, showBackNeedle: false }] }] });
			    * @example 
			    * //Get or set the pointers value after initialization:<br>
			    *   //Gets the pointers value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // sindex - scale index value
                *   // pindex - pointer index value
                *   var result = scale[sindex].pointers[pindex];
			    *   //Sets the pointers value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ distanceFromScale: 0, showBackNeedle: false }] }]);
			        * @memberof ejCircularGauge
			    * @instance
			    */
                pointers: [{
                    /**		
			        * Specify distanceFromScale value for pointers of circular gauge
			        * @default [Array.number] scale.pointers.distanceFromScale = 0
                    * @alias ejCircularGauge#scales->pointers->distanceFromScale
			        * @type {number}
			        * @example 
			        * // Set the distanceFromScale value for pointers during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ distanceFromScale: 10 }] }] });
			        * @example 
			        * //Get or set the distanceFromScale value for pointers after initialization:<br>
			        *   //Gets the distanceFromScale value for pointers.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].distanceFromScale;
			        *   //Sets the distanceFromScale value for pointers.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ distanceFromScale: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    distanceFromScale: 0,
                    /**		
			        * Specify showBackNeedle value of circular gauge
			        * @default [Array.boolean] scale.pointers.showBackNeedle = false
                    * @alias ejCircularGauge#scales->pointers->showBackNeedle
			        * @type {boolean}
			        * @example 
			        * // Set the showBackNeedle value during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ showBackNeedle: false }] }] });
			        * @example 
			        * //Get or set the showBackNeedle value after initialization:<br>
			        *   //Gets the showBackNeedle value.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].showBackNeedle;
			        *   //Sets the showBackNeedle value.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ showBackNeedle: false }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    showBackNeedle: false,
                    /**		
			        * Specify backNeedleLength of circular gauge
			        * @default [Array.number] scale.pointers.backNeedleLength = 10
                    * @alias ejCircularGauge#scales->pointers->backNeedleLength
			        * @type {number}
			        * @example 
			        * // Set the backNeedleLength during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ backNeedleLength: 10 }] }] });
			        * @example 
			        * //Get or set the backNeedleLength after initialization:<br>
			        *   //Gets the backNeedleLength.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].backNeedleLength;
			        *   //Sets the backNeedleLength.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ backNeedleLength: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    backNeedleLength: 10,
                    /**		
			        * Specify pointerLength of circular gauge
			        * @default [Array.number] scale.pointers.pointerLength = 150
                    * @alias ejCircularGauge#scales->pointers->pointerLength
			        * @type {number}
			        * @example 
			        * // Set the pointerLength during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ pointerLength: 150 }] }] });
			        * @example 
			        * //Get or set the pointerLength after initialization:<br>
			        *   //Gets the pointerLength.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].pointerLength;
			        *   //Sets the pointerLength.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ pointerLength: 150 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    pointerLength: 150,
                    /**		
			        * Specify pointerPosition value of circular gauge. See {@link pointerPosition}
			        * @default [Array.enum] scale.pointers.pointerPosition = "Near"
                    * @alias ejCircularGauge#scales->pointers->pointerPosition
			        * @type {enum}
			        * @example 
			        * // Set the pointerPosition value during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ pointerPosition: "Near" }] }] });
			        * @example 
			        * //Get or set the pointerPosition value after initialization:<br>
			        *   //Gets the pointerPosition value.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].pointerPosition;
			        *   //Sets the pointerPosition value.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ pointerPosition: "Near" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    pointerPosition: "Near", //[Near,Far,Center],
                    /**		
			        * Specify pointerWidth of circular gauge
			        * @default [Array.number] scale.pointers.pointerWidth = 7
                    * @alias ejCircularGauge#scales->pointers->pointerWidth
                    * @type {number}
			        * @example 
			        * // Set the pointerWidth during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ pointerWidth: 7 }] }] });
			        * @example 
			        * //Get or set the pointerWidth after initialization:<br>
			        *   //Gets the pointerWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].pointerWidth;
			        *   //Sets the pointerWidth.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ pointerWidth: 7 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    pointerWidth: 7,
                    /**		
			        * Specify opacity value for pointer of circular gauge
			        * @default [Array.double] scale.pointers.opacity = 1
                    * @alias ejCircularGauge#scales->pointers->opacity
                    * @type {double}
			        * @example 
			        * // Set the pointer opacity during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ opacity: 1 }] }] });
			        * @example 
			        * //Get or set the pointer opacity after initialization:<br>
			        *   //Gets the pointer opacity.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].opacity;
			        *   //Sets the pointer opacity.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ opacity: 1 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    opacity: 1,
                    /**		
			        * Specify borderWidth for scales of circular gauge
			        * @default [Array.double] scale.pointers.borderWidth = 1.5
                    * @alias ejCircularGauge#scales->pointers->borderWidth
                    * @type {double}
			        * @example 
			        * // Set the scale borderWidth during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ borderWidth: 1.5 }] }] });
			        * @example 
			        * //Get or set the scale borderWidth after initialization:<br>
			        *   //Gets the scale pointerWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].borderWidth;
			        *   //Sets the scale borderWidth.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ borderWidth: 1.5 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    borderWidth: 1.5,
                    /**		
			        * Specify value of the pointer of circular gauge
			        * @default [Array.number] scale.pointers.value = null
                    * @alias ejCircularGauge#scales->pointers->value
                    * @type {number}
			        * @example 
			        * // Set the value of the pointer during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ value: 50 }] }] });
			        * @example 
			        * //Get or set the value of the pointer after initialization:<br>
			        *   //Gets the value of the pointer.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].value;
			        *   //Sets the value of the pointer.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ value: 50 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    value: null,
                    /**		
			        * Specify borderColor for pointer of circular gauge
			        * @default [Array.string] scale.pointers.borderColor = null
                    * @alias ejCircularGauge#scales->pointers->borderColor
                    * @type {string}
			        * @example 
			        * // Set the pointer borderColor during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ borderColor: "#1A1A1A" }] }] });
			        * @example 
			        * //Get or set pointer borderColor after initialization:<br>
			        *   //Gets the pointer borderColor.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].borderColor;
			        *   //Sets the pointer borderColor.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ borderColor: [{ value: "#1A1A1A" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    borderColor: null,
                    /**		
			        * Specify backgroundColor for the pointer of circular gauge
			        * @default [Array.string] scale.pointers.backgroundColor = null
                    * @alias ejCircularGauge#scales->pointers->backgroundColor
                    * @type {string}
			        * @example 
			        * // Set the pointer backgroundColor during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ backgroundColor: "#1A1A1A" }] }] });
			        * @example 
			        * //Get or set pointer backgroundColor after initialization:<br>
			        *   //Gets the pointer backgroundColor.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].backgroundColor;
			        *   //Sets the pointer backgroundColor.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ backgroundColor: [{ value: "#1A1A1A" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    backgroundColor: null,
                    /**		
			        * Specify pointerGradient of circular gauge
			        * @default [Array.string] scale.pointers.pointerGradient = null
                    * @alias ejCircularGauge#scales->pointers->pointerGradient
                    * @type {string}
			        * @example 
			        * // Set the pointerGradient during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ pointerGradient: {colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }}] }] });
			        * @example 
			        * //Get or set pointerGradient after initialization:<br>
			        *   //Gets the pointerGradient.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].pointerGradient;
			        *   //Sets the pointerGradient.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ pointerGradient: {colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}]} }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    pointerGradient: null, //{colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }
                    /**		
			        * Specify pointerType value of circular gauge. See {@link pointerType}
			        * @default [Array.enum] scale.pointers.pointerType = "Needle"
                    * @alias ejCircularGauge#scales->pointers->pointerType
			        * @type {enum}
			        * @example 
			        * // Set the pointerType value during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ pointerType: "Needle" }] }] });
			        * @example 
			        * //Get or set the pointerType value after initialization:<br>
			        *   //Gets the pointerType value.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].pointerType;
			        *   //Sets the pointerType value.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ pointerType: "Needle" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    pointerType: "Needle", //["Needle", "Marker"]
                    /**		
			        * Specify needleStyle value of circular gauge. See {@link needleStyle}
			        * @default [Array.enum] scale.pointers.needleStyle = "Triangle"
                    * @alias ejCircularGauge#scales->pointers->needleStyle
			        * @type {enum}
			        * @example 
			        * // Set the needleStyle value during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ needleStyle: "Triangle" }] }] });
			        * @example 
			        * //Get or set the needleStyle value after initialization:<br>
			        *   //Gets the needleStyle value.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].needleStyle;
			        *   //Sets the needleStyle value.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ needleStyle: "Triangle" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    needleStyle: "Triangle", //["Triangle", "Rectangle", "Trapezoid", "Arrow"],
                    /**		
			        * Specify markerStyle value of circular gauge. See {@link markerStyle}
			        * @default [Array.enum] scale.pointers.markerStyle = "Rectangle"
                    * @alias ejCircularGauge#scales->pointers->markerStyle
			        * @type {enum}
			        * @example 
			        * // Set the markerStyle value during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ markerStyle: "Rectangle" }] }] });
			        * @example 
			        * //Get or set the markerStyle value after initialization:<br>
			        *   //Gets the markerStyle value.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].markerStyle;
			        *   //Sets the markerStyle value.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ pointers: [{ markerStyle: "Rectangle" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    markerStyle: "Rectangle" //["Rectangle", "Triangle", "Ellipse", "Diamond", "Pentagon", "Circle", "Slider", "Pointer", "Wedge", "Trapezoid", "RoundedRectangle"],
                }],
                /**		
			    * Specify ranges value of circular gauge
			    * @default [Array.object] scale.ranges = [{ distanceFromScale: 25, size: 5, rangePosition: "Near", startValue: null, endValue: null }]
                * @alias ejCircularGauge#scales->ranges
                * @type {object}
			    * @example 
			    * // Set the ranges value during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ distanceFromScale: 25, size: 5}] }] });
			    * @example 
			    * //Get or set the ranges value after initialization:<br>
			    *   //Gets the ranges value.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // sindex - scale index value
                *   // pindex - pointer index value
                *   var result = scale[sindex].pointers[pindex];
			    *   //Sets the ranges value.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ distanceFromScale: 25, size: 5 }] }]);
			        * @memberof ejCircularGauge
			    * @instance
			    */
                ranges: [{
                    /**		
			        * Specify distanceFromScale value for ranges of circular gauge
			        * @default [Array.number] scale.ranges.distanceFromScale = 25
                    * @alias ejCircularGauge#scales->ranges->distanceFromScale
                    * @type {number}
			        * @example 
			        * // Set the distanceFromScale value for ranges during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ distanceFromScale: 25 }] }] });
			        * @example 
			        * //Get or set the distanceFromScale value for ranges after initialization:<br>
			        *   //Gets the distanceFromScale value for ranges.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].distanceFromScale;
			        *   //Sets the distanceFromScale value for ranges.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ distanceFromScale: 25 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    distanceFromScale: 25,
                    /**		
			        * Specify size of the range value of circular gauge
			        * @default [Array.number] scale.ranges.size = 5
                    * @alias ejCircularGauge#scales->ranges->size
                    * @type {number}
			        * @example 
			        * // Set the size of the range value during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ size: 5 }] }] });
			        * @example 
			        * //Get or set the size of the range value after initialization:<br>
			        *   //Gets the size of the range value.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].size;
			        *   //Sets the size of the range value.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ size: 5 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    size: 5,
                    /**		
			        * Specify rangePosition of circular gauge. See {@link rangePosition}
			        * @default [Array.enum] scale.ranges.rangePosition = "Near"
                    * @alias ejCircularGauge#scales->ranges->rangePosition
                    * @type {enum}
			        * @example 
			        * // Set the rangePosition during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ rangePosition: [{ rangePosition: "Near" }] }] });
			        * @example 
			        * //Get or set the rangePosition after initialization:<br>
			        *   //Gets the rangePosition.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].rangePosition;
			        *   //Sets the rangePosition.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ rangePosition: [{ rangePosition: "Near" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    rangePosition: "Near", //["Near", "Far", "Center"]
                    /**		
			        * Specify startValue for ranges of circular gauge
			        * @default [Array.number] scale.ranges.startValue = null
                    * @alias ejCircularGauge#scales->ranges->startValue
                    * @type {number}
			        * @example 
			        * // Set the range startValue during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ startValue: 10 }] }] });
			        * @example 
			        * //Get or set the range startValue after initialization:<br>
			        *   //Gets the range startValue.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].startValue;
			        *   //Sets the range startValue.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ startValue: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    startValue: null,
                    /**		
			        * Specify endValue for ranges of circular gauge
			        * @default [Array.number] scale.ranges.endValue = null
                    * @alias ejCircularGauge#scales->ranges->endValue
                    * @type {number}
			        * @example 
			        * // Set the range endValue during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ endValue: 100 }] }] });
			        * @example 
			        * //Get or set the range endValue after initialization:<br>
			        *   //Gets the range endValue.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].endValue;
			        *   //Sets the range endValue.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ endValue: 100 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    endValue: null,
                    /**		
			        * Specify startWidth of circular gauge
			        * @default [Array.number] scale.ranges.startWidth = 10
                    * @alias ejCircularGauge#scales->ranges->startWidth
                    * @type {number}
			        * @example 
			        * // Set the startWidth during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ startWidth: 10 }] }] });
			        * @example 
			        * //Get or set the startWidth after initialization:<br>
			        *   //Gets the startWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].startWidth;
			        *   //Sets the startWidth.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ startWidth: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    startWidth: 10,
                    /**		
			        * Specify endWidth for ranges of circular gauge
			        * @default [Array.number] scale.ranges.endWidth = 10
                    * @alias ejCircularGauge#scales->ranges->endWidth
                    * @type {number}
			        * @example 
			        * // Set the range endWidth during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ endWidth: 10 }] }] });
			        * @example 
			        * //Get or set the range endWidth after initialization:<br>
			        *   //Gets the range endWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].endWidth;
			        *   //Sets the range endWidth.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ endWidth: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    endWidth: 10,
                    /**		
			        * Specify rangeGradient of circular gauge
			        * @default [Array.string] scale.ranges.rangeGradient = null
                    * @alias ejCircularGauge#scales->ranges->rangeGradient
                    * @type {string}
			        * @example 
			        * // Set the rangeGradient during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ rangeGradient: { colorInfo:[{ colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }}]  }] });
			        * @example 
			        * //Get or set the rangeGradient after initialization:<br>
			        *   //Gets the rangeGradient.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].rangeGradient;
			        *   //Sets the rangeGradient.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ rangeGradient: { colorInfo:[{ colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}]} }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    rangeGradient: null, //{colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }
                    /**		
			        * Specify borderWidth for ranges of circular gauge
			        * @default [Array.double] scale.ranges.borderWidth = 1.5
                    * @alias ejCircularGauge#scales->ranges->borderWidth
                    * @type {double}
			        * @example 
			        * // Set the range borderWidth during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ borderWidth: 1.5 }] }] });
			        * @example 
			        * //Get or set the range borderWidth after initialization:<br>
			        *   //Gets the range borderWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].borderWidth;
			        *   //Sets the range borderWidth.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ borderWidth: 1.5 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    borderWidth: 1.5,
                    /**		
			        * Specify opacity value for ranges of circular gauge
			        * @default [Array.double] scale.ranges.opacity = null
                    * @alias ejCircularGauge#scales->ranges->opacity
                    * @type {double}
			        * @example 
			        * // Set the range opacity during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ opacity: 0.5 }] }] });
			        * @example 
			        * //Get or set the range opacity after initialization:<br>
			        *   //Gets the range opacity.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].opacity;
			        *   //Sets the range opacity.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ opacity: 0.5 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    opacity: null,
                    /**		
			        * Specify backgroundColor for the ranges of circular gauge
			        * @default [Array.string] scale.ranges.backgroundColor = "#32b3c6"
                    * @alias ejCircularGauge#scales->ranges->backgroundColor
                    * @type {string}
			        * @example 
			        * // Set the range backgroundColor during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ backgroundColor: "#32b3c6" }] }] });
			        * @example 
			        * //Get or set the range backgroundColor after initialization:<br>
			        *   //Gets the range backgroundColor.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].backgroundColor;
			        *   //Sets the range backgroundColor.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ backgroundColor: "#32b3c6" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    backgroundColor: "#32b3c6",
                    /**		
			        * Specify borderColor for ranges of circular gauge
			        * @default [Array.string] scale.ranges.borderColor = "#32b3c6"
                    * @alias ejCircularGauge#scales->ranges->borderColor
                    * @type {string}
			        * @example 
			        * // Set the range borderColor during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges: [{ borderColor: "#32b3c6" }] }] });
			        * @example 
			        * //Get or set the range borderColor after initialization:<br>
			        *   //Gets the range borderColor.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].pointers[pindex].borderColor;
			        *   //Sets the range borderColor.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ranges: [{ borderColor: "#32b3c6" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    borderColor: "#32b3c6"
                }],
                /**		
			    * Specify ticks of circular gauge
			    * @default [Array.object] scale.ticks = [ { angle: 10, distanceFromScale: 10, tickColor: null, tickStyle: "Major", tickPosition: "Near" } ]
                * @alias ejCircularGauge#scales->ticks
                * @type {object}
			    * @example 
			    * // Set the ticks during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ angle: 10, distanceFromScale: 10 }] }] });
			    * @example 
			    * //Get or set the ticks after initialization:<br>
			    *   //Gets the ticks.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // sindex - scale index value
                *   // pindex - pointer index value
                *   var result = scale[sindex].ticks[pindex];
			    *   //Sets the ticks.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ angle: 10, distanceFromScale: 10 }] }]);
			        * @memberof ejCircularGauge
			    * @instance
			    */
                ticks: [{
                    /**		
			        * Specify the angle for the ticks of circular gauge
			        * @default [Array.number] scale.ticks.angle = 0
                    * @alias ejCircularGauge#scales->ticks->angle
                    * @type {number}
			        * @example 
			        * // Set the angle during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ angle: 10 }] }] });
			        * @example 
			        * //Get or set the angle after initialization:<br>
			        *   //Gets the angle.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].angle;
			        *   //Sets the angle.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ angle: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    angle: 0,
                    /**		
			        * Specify distanceFromScale value for ticks of circular gauge
			        * @default [Array.number] scale.ticks.distanceFromScale = 0
                    * @alias ejCircularGauge#scales->ticks->distanceFromScale
                    * @type {number}
			        * @example 
			        * // Set the distanceFromScale value for ticks during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ distanceFromScale: 10 }] }] });
			        * @example 
			        * //Get or set the distanceFromScale value for ticks after initialization:<br>
			        *   //Gets the distanceFromScale value for ticks.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].distanceFromScale;
			        *   //Sets the distanceFromScale value for ticks.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ distanceFromScale: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    distanceFromScale: 0,
                    /**		
			        * Specify tickColor of circular gauge
			        * @default [Array.string] scale.ticks.tickColor = null
                    * @alias ejCircularGauge#scales->ticks->tickColor
                    * @type {string}
			        * @example 
			        * // Set the tickColor during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickColor: "#777777" }] }] });
			        * @example 
			        * //Get or set the tickColor after initialization:<br>
			        *   //Gets the tickColor.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickColor;
			        *   //Sets the tickColor.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickColor: "#777777" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickColor: null,
                    /**		
			        * Specify tickStyle of circular gauge. See {@link tickStyle}
			        * @default [Array.enum] scale.ticks.tickStyle = "Major"
                    * @alias ejCircularGauge#scales->ticks->tickStyle
                    * @type {enum}
			        * @example 
			        * // Set the tickStyle during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickStyle: "Major" }] }] });
			        * @example 
			        * //Get or set the tickStyle after initialization:<br>
			        *   //Gets the tickStyle.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickStyle;
			        *   //Sets the tickStyle.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickStyle: "Major" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickStyle: "Major", //["Major", "Minor",]
                    /**		
			        * Specify tickPosition of circular gauge. See {@link tickPosition}
			        * @default [Array.enum] scale.ticks.tickPosition = "Near"
                    * @alias ejCircularGauge#scales->ticks->tickPosition
                    * @type {enum}
			        * @example 
			        * // Set the tickPosition during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickPosition: "Near" }] }] });
			        * @example 
			        * //Get or set the tickPosition after initialization:<br>
			        *   //Gets the tickPosition.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickPosition;
			        *   //Sets the tickPosition.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickPosition: "Near" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickPosition: "Near", //[Near,Far,Center],
                    /**		
			        * Specify tickHeight of circular gauge
			        * @default [Array.number] scale.ticks.tickHeight = 16
                    * @alias ejCircularGauge#scales->ticks->tickHeight
                    * @type {number}
			        * @example 
			        * // Set the tickHeight during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickHeight: 16 }] }] });
			        * @example 
			        * //Get or set the tickHeight after initialization:<br>
			        *   //Gets the tickHeight.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickHeight;
			        *   //Sets the tickHeight.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickHeight: 16 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickHeight: 16,
                    /**		
			        * Specify tickWidth of circular gauge
			        * @default [Array.number] scale.ticks.tickWidth = 3
                    * @alias ejCircularGauge#scales->ticks->tickWidth
                    * @type {number}
			        * @example 
			        * // Set the tickWidth during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickWidth: 3 }] }] });
			        * @example 
			        * //Get or set the tickWidth after initialization:<br>
			        *   //Gets the tickWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickWidth;
			        *   //Sets the tickWidth.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickWidth: 3 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickWidth: 3
                }, {
                    /**		
			        * Specify the angle for the ticks of circular gauge
			        * @default [Array.number] scale.ticks.angle = 0
                    * @alias ejCircularGauge#scales->ticks->angle
                    * @type {number}
			        * @example 
			        * // Set the angle during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ angle: 10 }] }] });
			        * @example 
			        * //Get or set the angle after initialization:<br>
			        *   //Gets the angle.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].angle;
			        *   //Sets the angle.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ angle: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    angle: 0,
                    /**		
                   * Specify distanceFromScale value for ticks of circular gauge
                   * @default [Array.number] scale.ticks.distanceFromScale = 0
                   * @alias ejCircularGauge#scales->ticks->distanceFromScale
                   * @type {number}
                   * @example 
                   * // Set the distanceFromScale value for ticks during initialization. 			
                   * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ distanceFromScale: 10 }] }] });
                   * @example 
                   * //Get or set the distanceFromScale value for ticks after initialization:<br>
                   *   //Gets the distanceFromScale value for ticks.
                   *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                   *   // sindex - scale index value
                   *   // pindex - pointer index value
                   *   var result = scale[sindex].ticks[pindex].distanceFromScale;
                   *   //Sets the distanceFromScale value for ticks.
                   *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ distanceFromScale: 10 }] }]);
                    * @memberof ejCircularGauge
                   * @instance
                   */
                    distanceFromScale: 0,
                    /**		
			        * Specify tickColor of circular gauge
			        * @default [Array.string] scale.ticks.tickColor = null
                    * @alias ejCircularGauge#scales->ticks->tickColor
                    * @type {string}
			        * @example 
			        * // Set the tickColor during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickColor: "#777777" }] }] });
			        * @example 
			        * //Get or set the tickColor after initialization:<br>
			        *   //Gets the tickColor.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickColor;
			        *   //Sets the tickColor.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickColor: "#777777" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickColor: null,
                    /**		
			        * Specify tickStyle of circular gauge. See {@link tickStyle}
			        * @default [Array.enum] scale.ticks.tickStyle = "Minor"
                    * @alias ejCircularGauge#scales->ticks->tickStyle
                    * @type {enum}
			        * @example 
			        * // Set the tickStyle during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickStyle: "Minor" }] }] });
			        * @example 
			        * //Get or set the tickStyle after initialization:<br>
			        *   //Gets the tickStyle.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickStyle;
			        *   //Sets the tickStyle.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickStyle: "Minor" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickStyle: "Minor", //["Major", "Minor",]
                    /**		
			        * Specify tickPosition of circular gauge. See {@link tickPosition}
			        * @default [Array.enum] scale.ticks.tickPosition = "Near"
                    * @alias ejCircularGauge#scales->ticks->tickPosition
                    * @type {enum}
			        * @example 
			        * // Set the tickPosition during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickPosition: "Near" }] }] });
			        * @example 
			        * //Get or set the tickPosition after initialization:<br>
			        *   //Gets the tickPosition.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickPosition;
			        *   //Sets the tickPosition.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickPosition: "Near" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickPosition: "Near", //[Near,Far,Center],
                    /**		
			        * Specify tickHeight of circular gauge
			        * @default [Array.number] scale.ticks.tickHeight = 7
                    * @alias ejCircularGauge#scales->ticks->tickHeight
                    * @type {number}
			        * @example 
			        * // Set the tickHeight during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickHeight: 16 }] }] });
			        * @example 
			        * //Get or set the tickHeight after initialization:<br>
			        *   //Gets the tickHeight.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickHeight;
			        *   //Sets the tickHeight.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickHeight: 16 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickHeight: 7,
                    /**		
			        * Specify tickWidth of circular gauge
			        * @default [Array.number] scale.ticks.tickWidth = 1
                    * @alias ejCircularGauge#scales->ticks->tickWidth
                    * @type {number}
			        * @example 
			        * // Set the tickWidth during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ tickWidth: 3 }] }] });
			        * @example 
			        * //Get or set the tickWidth after initialization:<br>
			        *   //Gets the tickWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].ticks[pindex].tickWidth;
			        *   //Sets the tickWidth.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ ticks: [{ tickWidth: 3 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    tickWidth: 1
                }],
                /**		
			    * Specify the text values displayed in a meaningful manner alongside the ticks of circular gauge
			    * @default [Array.object] scale.labels = [{ angle: 10, opacity: 0.4, font: { size: "12px", fontFamily: "Segou", fontStyle: "Bold" }, labelColor: "#212A23" }]
                * @alias ejCircularGauge#scales->labels
                * @type {object}
			    * @example 
			    * // Set the labels during initialization. 			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ angle: 10, opacity: 0.4 }] }] });
			    * @example 
			    * //Get or set the labels after initialization:<br>
			    *   //Gets the labels.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // sindex - scale index value
                *   // pindex - pointer index value
                *   var result = scale[sindex].labels[pindex];
			    *   //Sets the labels.
			    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ angle: 10, opacity: 0.4 }] }]);
			        * @memberof ejCircularGauge
			    * @instance
			    */
                labels: [{
                    /**		
			        * Specify the angle for the labels of circular gauge
			        * @default [Array.number] scale.labels.angle = 0
                    * @alias ejCircularGauge#scales->labels->angle
                    * @type {number}
			        * @example 
			        * // Set the angle during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ angle: 10 }] }] });
			        * @example 
			        * //Get or set the angle after initialization:<br>
			        *   //Gets the angle.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].angle;
			        *   //Sets the angle.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ angle: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    angle: 0,
                    /**		
			        * Specify opacity value for labels of circular gauge
			        * @default [Array.double] scale.labels.opacity = null
                    * @alias ejCircularGauge#scales->labels->opacity
                    * @type {double}
			        * @example 
			        * // Set the label opacity during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ opacity: 0.4 }] }] });
			        * @example 
			        * //Get or set the label opacity after initialization:<br>
			        *   //Gets the label opacity.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].opacity;
			        *   //Sets the label opacity.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ opacity: 0.4 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    opacity: null,
                    /**		
			        * Specify font for labels of circular gauge
			        * @default [Array.object] scale.labels.font = { size: "11px", fontFamily: "Arial", fontStyle: "Bold" }
                    * @alias ejCircularGauge#scales->labels->font
                    * @type {object}
			        * @example 
			        * // Set the label font during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ font: { size: "12px", fontFamily: "Segou", fontStyle: "Bold" } }] }] });
			        * @example 
			        * //Get or set the label font after initialization:<br>
			        *   //Gets the label font.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].font;
			        *   //Sets the label font.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ font: { size: "12px", fontFamily: "Segou", fontStyle: "Bold" } }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    font: {
                        /**		
                        * Specify font for labels of circular gauge
                        * @default [Array.object] scale.labels.font = { size: "11px" }
                        * @alias ejCircularGauge#scales->labels->font->size
                        * @type {object}
                        * @example 
                        * // Set the label font during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ font: { size: "12px" } }] }] });
                        * @example 
                        * //Get or set the label font after initialization:<br>
                        *   //Gets the label font.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].labels[pindex].font.size;
                        *   //Sets the label font.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ font: { size: "12px" } }] }]);
                         * @memberof ejCircularGauge
                        * @instance
                        */
                        size: "11px",
                        /**		
                        * Specify font for labels of circular gauge
                        * @default [Array.object] scale.labels.font = { fontFamily: "Arial" }
                        * @alias ejCircularGauge#scales->labels->font->fontFamily
                        * @type {object}
                        * @example 
                        * // Set the label font during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ font: { fontFamily: "Arial" } }] }] });
                        * @example 
                        * //Get or set the label font after initialization:<br>
                        *   //Gets the label font.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].labels[pindex].font.fontFamily;
                        *   //Sets the label font.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ font: { fontFamily: "Arial" } }] }]);
                         * @memberof ejCircularGauge
                        * @instance
                        */
                        fontFamily: "Arial",
                        /**		
                        * Specify font for labels of circular gauge
                        * @default [Array.object] scale.labels.font = { fontStyle: "Bold" }
                        * @alias ejCircularGauge#scales->labels->font->fontStyle
                        * @type {object}
                        * @example 
                        * // Set the label font during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ font: { fontStyle: "Bold" } }] }] });
                        * @example 
                        * //Get or set the label font after initialization:<br>
                        *   //Gets the label font.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].labels[pindex].font.fontStyle;
                        *   //Sets the label font.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ font: { fontStyle: "Bold" } }] }]);
                         * @memberof ejCircularGauge
                        * @instance
                        */
                        fontStyle: "Bold"
                    },
                    /**		
			        * Specify labelColor of circular gauge
			        * @default [Array.string] scale.labels.labelColor = null
                    * @alias ejCircularGauge#scales->labels->labelColor
                    * @type {string}
			        * @example 
			        * // Set the labelColor during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ labelColor: "#212A23" }] }] });
			        * @example 
			        * //Get or set the labelColor after initialization:<br>
			        *   //Gets the labelColor.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].labelColor;
			        *   //Sets the labelColor.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ labelColor: "#212A23" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    labelColor: null,
                    /**		
			        * Specify distanceFromScale value for labels of circular gauge
			        * @default [Array.number] scale.labels.distanceFromScale = 0
                    * @alias ejCircularGauge#scales->labels->distanceFromScale
                    * @type {number}
			        * @example 
			        * // Set the distanceFromScale value for labels during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ distanceFromScales: 10 }] }] });
			        * @example 
			        * //Get or set the distanceFromScale value for labels after initialization:<br>
			        *   //Gets the distanceFromScale value for labels.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].distanceFromScale;
			        *   //Sets the distanceFromScale value for labels.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ distanceFromScale: 10 }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    distanceFromScale: 0,
                    /**		
			        * Specify includeFirstValue of circular gauge
			        * @default [Array.boolean] scale.labels.includeFirstValue = true
                    * @alias ejCircularGauge#scales->labels->includeFirstValue
                    * @type {boolean}
			        * @example 
			        * // Set the includeFirstValue during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ includeFirstValue: true }] }] });
			        * @example 
			        * //Get or set the includeFirstValue after initialization:<br>
			        *   //Gets the includeFirstValue.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].includeFirstValue;
			        *   //Sets the includeFirstValue.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ includeFirstValue: true }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    includeFirstValue: true,
                    /**		
			        * Specify labelPosition of circular gauge. See {@link labelPosition}
			        * @default [Array.enum] scale.labels.labelPosition = "Near"
                    * @alias ejCircularGauge#scales->labels->labelPosition
                    * @type {enum}
			        * @example 
			        * // Set the labelPosition during initialization. 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ labelPosition: "Near" }] }] });
			        * @example 
			        * //Get or set the labelPosition after initialization:<br>
			        *   //Gets the labelPosition.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].labelPosition;
			        *   //Sets the labelPosition.
			        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ labelPosition: "Near" }] }]);
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    labelPosition: "Near", //[Near,Far,Center],
                    /**		
                    * Specify labelStyle of circular gauge.	See {@link labelStyle}
                    * @default [Array.enum] scale.labels.labelStyle = "Major"
                    * @alias ejCircularGauge#scales->labels->labelStyle
                    * @type {enum}
                    * @example 
                    * // Set the labelStyle during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ labelStyle: "Major" }] }] });
                    * @example 
                    * //Get or set the labelStyle after initialization:<br>
                    *   //Gets the labelStyle.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].labelStyle;
                    *   //Sets the labelStyle.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ labelStyle: "Major" }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    labelStyle: "Major",//["Major", "Minor",]
                    /**		
                    * Specify unitText of circular gauge
                    * @default [Array.string] scale.labels.unitText = ""
                    * @alias ejCircularGauge#scales->labels->unitText
                    * @type {string}
                    * @example 
                    * // Set the unitText during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ unitText: "kmph" }] }] });
                    * @example 
                    * //Get or set the unitText after initialization:<br>
                    *   //Gets the unitText.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].unitText;
                    *   //Sets the unitText.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ unitText: "kmph" }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    unitText: "",
                    /**		
                    * Specify unitTextPosition of circular gauge.	See {@link unitTextPosition}
                    * @default [Array.enum] scale.labels.unitTextPosition = "Back"
                    * @alias ejCircularGauge#scales->labels->unitTextPosition
                    * @type {enum}
                    * @example 
                    * // Set the unitTextPosition during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ unitTextPosition: "Back" }] }] });
                    * @example 
                    * //Get or set the unitTextPosition after initialization:<br>
                    *   //Gets the unitTextPosition.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].labels[pindex].unitTextPosition;
                    *   //Sets the unitTextPosition.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ labels: [{ unitTextPosition: "Back" }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    unitTextPosition: "Back",//["Front","Back"]
                }],
                /**		
                * Specify representating state of circular gauge
                * @default [Array.object] scale.indicators = [{ indicatorHeight: 15, indicatorWidth: 15, indicatorStyle: "Circle", indicatorValue: 0 }]
                * @alias ejCircularGauge#scales->indicators
                * @type {object}
                * @example 
                * // Set the representating state during initialization. 			
                * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ indicatorHeight: 15, indicatorWidth: 15 }] }] });
                * @example 
                * //Get or set the representating state after initialization:<br>
                *   //Gets the representating state.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // sindex - scale index value
                *   // pindex - pointer index value
                *   var result = scale[sindex].indicators[pindex];
                *   //Sets the representating state.
                *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ indicatorHeight: 15, indicatorWidth: 15 }] }]);
                * @memberof ejCircularGauge
                * @instance
                */
                indicators: [{
                    /**		
                    * Specify indicatorHeight of circular gauge
                    * @default [Array.int] scale.indicators.indicatorHeight = 15
                    * @alias ejCircularGauge#scales->indicators->indicatorHeight
                    * @type {int}
                    * @example 
                    * // Set the indicatorHeight during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ indicatorHeight: 15 }] }] });
                    * @example 
                    * //Get or set the indicatorHeight after initialization:<br>
                    *   //Gets the indicatorHeight.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].indicators[pindex].indicatorHeight;
                    *   //Sets the indicatorHeight.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ indicatorHeight: 15 }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    indicatorHeight: 15,
                    /**		
                    * Specify indicatorWidth of circular gauge
                    * @default [Array.int] scale.indicators.indicatorWidth = 15
                    * @alias ejCircularGauge#scales->indicators->indicatorWidth
                    * @type {int}
                    * @example 
                    * // Set the indicatorWidth during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ indicatorWidth: 15 }] }] });
                    * @example 
                    * //Get or set the indicatorWidth after initialization:<br>
                    *   //Gets the indicatorWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].indicators[pindex].indicatorWidth;
                    *   //Sets the indicatorWidth.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ indicatorWidth: 15 }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    indicatorWidth: 15,
                    /**		
                    * Specify indicatorStyle of circular gauge.	See {@link indicatorStyle}
                    * @default [Array.enum] scale.indicators.indicatorStyle = "Circle"
                    * @alias ejCircularGauge#scales->indicators->indicatorStyle
                    * @type {enum}
                    * @example 
                    * // Set the indicatorStyle during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ indicatorStyle: "Circle" }] }] });
                    * @example 
                    * //Get or set the indicatorStyle after initialization:<br>
                    *   //Gets the indicatorStyle.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].indicators[pindex].indicatorStyle;
                    *   //Sets the indicatorStyle.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ indicatorStyle: "Circle" }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    indicatorStyle: "Circle", //["Rectangle", "Circle","RoundedRectangle", "Text"]
                    /**		
                    * Specify indicatorValue of circular gauge
                    * @default [Array.int] scale.indicators.indicatorValue = 0
                    * @alias ejCircularGauge#scales->indicators->indicatorValue
                    * @type {int}
                    * @example 
                    * // Set the indicatorValue during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ indicatorValue: 0 }] }] });
                    * @example 
                    * //Get or set the indicatorValue after initialization:<br>
                    *   //Gets the indicatorValue.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].indicators[pindex].indicatorValue;
                    *   //Sets the indicatorValue.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ indicatorValue: 0 }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    indicatorValue: 0,
                    /**		
                    * Specify imageUrl of circular gauge
                    * @default [Array.string] scale.indicators.imageUrl = null
                    * @alias ejCircularGauge#scales->indicators->imageUrl
                    * @type {string}
                    * @example 
                    * // Set the imageUrl during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ imageUrl: "../Images/sample.jpg" }] }] });
                    * @example 
                    * //Get or set the imageUrl after initialization:<br>
                    *   //Gets the imageUrl.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].indicators[pindex].imageUrl;
                    *   //Sets the imageUrl.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ imageUrl: "../Images/sample.jpg" }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    imageUrl: null,
                    /**		
                    * Specify indicatorLocation of circular gauge
                    * @default [Array.object] scale.indicators.indicatorLocation = { x: 0, y: 0 }
                    * @alias ejCircularGauge#scales->indicators->indicatorLocation
                    * @type {object}
                    * @example 
                    * // Set the indicatorLocation during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ indicatorLocation: { x: 50, y: 40 } }] }] });
                    * @example 
                    * //Get or set the indicatorLocation after initialization:<br>
                    *   //Gets the indicatorLocation.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].indicators[pindex].indicatorLocation;
                    *   //Sets the indicatorLocation.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ indicatorLocation: { x: 50, y: 40 } }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    indicatorLocation: {
                        /**		
                        * Specify x-axis of indicatorLocation of circular gauge
                        * @default [Array.object] scale.indicators.indicatorLocation = { x: 0 }
                        * @alias ejCircularGauge#scales->indicators->indicatorLocation->x
                        * @type {object}
                        * @example 
                        * // Set the x-axis of indicatorLocation during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ indicatorLocation: { x: 50 } }] }] });
                        * @example 
                        * //Get or set the x-axis of indicatorLocation after initialization:<br>
                        *   //Gets the x-axis of indicatorLocation.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[pindex].indicatorLocation;
                        *   //Sets the x-axis of indicatorLocation.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ indicatorLocation: { x: 50 } }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        x: 0,
                        /**		
                        * Specify y-axis of indicatorLocation of circular gauge
                        * @default [Array.object] scale.indicators.indicatorLocation = { y: 0 }
                        * @alias ejCircularGauge#scales->indicators->indicatorLocation->y
                        * @type {object}
                        * @example 
                        * // Set the y-axis of indicatorLocation during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ indicatorLocation: { y: 40 } }] }] });
                        * @example 
                        * //Get or set the y-axis of indicatorLocation after initialization:<br>
                        *   //Gets the y-axis of indicatorLocation.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[pindex].indicatorLocation;
                        *   //Sets the y-axis of indicatorLocation.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ indicatorLocation: { y: 40 } }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        y: 0
                    },
                    /**		
                    * Specify the various states of circular gauge
                    * @default [Array.object] scale.indicators.stateRanges = [{ endValue: 0, startValue: 0, backgroundColor: null, borderColor: null }]
                    * @alias ejCircularGauge#scales->indicators->stateRanges
                    * @type {object}
                    * @example 
                    * // Set the various states during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ stateRanges: [{ endValue: 50, startValue: 50 }]}]  }] });
                    * @example 
                    * //Get or set the various states after initialization:<br>
                    *   //Gets the the various states.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].indicators[index].stateRanges[rindex];
                    *   //Sets the the various states.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ stateRanges: [{ endValue: 50, startValue: 50  }] }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    stateRanges: [{
                        /**		
                        * Specify end value for each specified state of circular gauge
                        * @default [Array.int] scale.indicators.stateRanges.indicatorLocation = 0
                        * @alias ejCircularGauge#scales->indicators->stateRanges->endValue
                        * @type {int}
                        * @example 
                        * // Set the end value for each specified state during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ stateRanges: [{ endValue: 50 }]}]  }] });
                        * @example 
                        * //Get or set the end value for each specified state after initialization:<br>
                        *   //Gets the end value for each specified state.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[index].stateRanges[rindex].endValue;
                        *   //Sets the end value for each specified state.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ stateRanges: [{ endValue: 50  }] }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        endValue: 0,
                        /**		
                        * Specify start value for each specified state of circular gauge
                        * @default [Array.int] scale.indicators.stateRanges.startValue = 0
                        * @alias ejCircularGauge#scales->indicators->stateRanges->startValue
                        * @type {int}
                        * @example 
                        * // Set the start value for each specified state during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ stateRanges: [{ startValue: 50 }]}]  }] });
                        * @example 
                        * //Get or set the start value for each specified state after initialization:<br>
                        *   //Gets the start value for each specified state.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[index].stateRanges[rindex].startValue;
                        *   //Sets the start value for each specified state.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ stateRanges: [{ startValue: 50  }] }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        startValue: 0,
                        /**		
                        * Specify backgroundColor for indicator of circular gauge
                        * @default [Array.string] scale.indicators.stateRanges.backgroundColor = null
                        * @alias ejCircularGauge#scales->indicators->stateRanges->backgroundColor
                        * @type {string}
                        * @example 
                        * // Set the backgroundColor for indicator during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ stateRanges: [{ backgroundColor: "#1E1A1E" }]}]  }] });
                        * @example 
                        * //Get or set the backgroundColor for indicator after initialization:<br>
                        *   //Gets the backgroundColor for indicator.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[index].stateRanges[rindex].backgroundColor;
                        *   //Sets the backgroundColor for indicator.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ stateRanges: [{ backgroundColor: "#1E1A1E"  }] }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        backgroundColor: null,
                        /**		
                        * Specify borderColor for indicator of circular gauge
                        * @default [Array.string] scale.indicators.stateRanges.borderColor = null
                        * @alias ejCircularGauge#scales->indicators->stateRanges->borderColor
                        * @type {string}
                        * @example 
                        * // Set borderColor for indicator during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ stateRanges: [{ borderColor: "#1A1A1A" }]}]  }] });
                        * @example 
                        * //Get or set the borderColor for indicator after initialization:<br>
                        *   //Gets the borderColor for indicator.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[index].stateRanges[rindex].borderColor;
                        *   //Sets the borderColor for indicator.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ stateRanges: [{ borderColor: "#1A1A1A"  }] }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        borderColor: null,
                        /**		
                        * Specify value of the text as the indicator when the indicator style is set with the value "Text" of circular gauge
                        * @default [Array.string] scale.indicators.stateRanges.text = ""
                        * @alias ejCircularGauge#scales->indicators->stateRanges->text
                        * @type {string}
                        * @example 
                        * // Set text for indicator during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ stateRanges: [{ text: "Hello" }]}]  }] });
                        * @example 
                        * //Get or set the text for indicator after initialization:<br>
                        *   //Gets the text for indicator.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[index].stateRanges[rindex].text;
                        *   //Sets the text for indicator.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ stateRanges: [{ text: "Hello"  }] }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        text: "",
                        /**		
                        * Specify value of the textColor as the indicator when the indicator style is set with the value "Text" of circular gauge
                        * @default [Array.string] scale.indicators.stateRanges.textColor = ""
                        * @alias ejCircularGauge#scales->indicators->stateRanges->textColor
                        * @type {string}
                        * @example 
                        * // Set textColor for indicator during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ stateRanges: [{ textColor: "#FF111F" }]}]  }] });
                        * @example 
                        * //Get or set the textColor for indicator after initialization:<br>
                        *   //Gets the textColor for indicator.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[index].stateRanges[rindex].textColor;
                        *   //Sets the textColor for indicator.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ stateRanges: [{ textColor: "#FF111F"  }] }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        textColor: null,
                        /**		
                        * Specify value of the font as the indicator when the indicator style is set with the value "Text" of circular gauge
                        * @default [Array.string] scale.indicators.stateRanges.font = ""
                        * @alias ejCircularGauge#scales->indicators->stateRanges->font
                        * @type {string}
                        * @example 
                        * // Set font for indicator during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ indicators: [{ stateRanges: [{ font: { size: "11px", fontFamily: "Arial", fontStyle: "Bold" } }]}]  }] });
                        * @example 
                        * //Get or set the font for indicator after initialization:<br>
                        *   //Gets the font for indicator.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].indicators[index].stateRanges[rindex].font;
                        *   //Sets the font for indicator.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ indicators: [{ stateRanges: [{ font: { size: "11px", fontFamily: "Arial", fontStyle: "Bold" }  }] }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        font: null
                    }]
                }],

                //subGuageHeight,subGuageWidth,location:{X:5,Y:10},controlID
                /**		
                * Specify subGuage of circular gauge
                * @default [Array.object] scale.subGuage = [{ subGaugeHeight: 150, subGaugeWidth: 150, location: { x: 0, y: 0 } }]
                * @alias ejCircularGauge#scales->subGauge
                * @type {object}
                * @example 
                * // Set the subGuage during initialization. 			
                * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ subGuage: [{ subGaugeHeight: 150, subGaugeWidth: 150 }] }] });
                * @example 
                * //Get or set the subGuage after initialization:<br>
                *   //Gets the subGuage.
                *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                *   // sindex - scale index value
                *   // pindex - pointer index value
                *   var result = scale[sindex].subGuage[pindex];
                *   //Sets the subGuage.
                *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ subGuage: [{ subGaugeHeight: 150, subGaugeWidth: 150 }] }]);
                * @memberof ejCircularGauge
                * @instance
                */
                subGuage: [{
                    /**		
                    * Specify subGaugeHeight of circular gauge
                    * @default [Array.int] scale.subGuage.subGaugeHeight = 150
                    * @alias ejCircularGauge#scales->subGauge->subGaugeHeight
                    * @type {int}
                    * @example 
                    * // Set the subGaugeHeight during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ subGuage: [{ subGaugeHeight: 150 }] }] });
                    * @example 
                    * //Get or set the subGaugeHeight after initialization:<br>
                    *   //Gets the subGaugeHeight.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].subGuage[pindex].subGaugeHeight;
                    *   //Sets the subGaugeHeight.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ subGuage: [{ subGaugeHeight: 150 }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    subGaugeHeight: 150,
                    /**		
                    * Specify subGaugeWidth of circular gauge
                    * @default [Array.int] scale.subGuage.subGaugeWidth = 150
                    * @alias ejCircularGauge#scales->subGauge->subGaugeWidth
                    * @type {int}
                    * @example 
                    * // Set the subGaugeWidth during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ subGuage: [{ subGaugeWidth: 150 }] }] });
                    * @example 
                    * //Get or set the subGaugeWidth after initialization:<br>
                    *   //Gets the subGaugeWidth.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].subGuage[pindex].subGaugeWidth;
                    *   //Sets the subGaugeWidth.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ subGuage: [{ subGaugeWidth: 150 }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    subGaugeWidth: 150,
                    /**		
                    * Specify location for sub-gauge of circular gauge
                    * @default [Array.object] scale.subGuage.location = { x: 0, y: 0 }
                    * @alias ejCircularGauge#scales->subGauge->location
                    * @type {object}
                    * @example 
                    * // Set the location during initialization. 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ subGuage: [{ location: { x: 0, y: 0 } }] }] });
                    * @example 
                    * //Get or set the location after initialization:<br>
                    *   //Gets the location.
                    *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                    *   // sindex - scale index value
                    *   // pindex - pointer index value
                    *   var result = scale[sindex].subGuage[pindex].location;
                    *   //Sets the location.
                    *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ subGuage: [{ location: { x: 0, y: 0 } }] }]);
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    location: {
                        /**		
                        * Specify x-axis location for sub-gauge of circular gauge
                        * @default [Array.object] scale.subGuage.location = { x: 0 }
                        * @alias ejCircularGauge#scales->subGauge->location->x
                        * @type {object}
                        * @example 
                        * // Set the x-axis location during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ subGuage: [{ location: { x: 0 } }] }] });
                        * @example 
                        * //Get or set the x-axis location after initialization:<br>
                        *   //Gets the x-axis location.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].subGuage[pindex].location;
                        *   //Sets the x-axis location.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ subGuage: [{ location: { x: 0 } }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        x: 0,
                        /**		
                        * Specify y-axis location for sub-gauge of circular gauge
                        * @default [Array.object] scale.subGuage.location = { y: 0 }
                        * @alias ejCircularGauge#scales->subGauge->location->y
                        * @type {object}
                        * @example 
                        * // Set the y-axis location during initialization. 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ subGuage: [{ location: { y: 0 } }] }] });
                        * @example 
                        * //Get or set the y-axis location after initialization:<br>
                        *   //Gets the y-axis location.
                        *   var scale = $("#CoreCircularGauge").ejCircularGauge("option", "scales");
                        *   // sindex - scale index value
                        *   // pindex - pointer index value
                        *   var result = scale[sindex].subGuage[pindex].location;
                        *   //Sets the y-axis location.
                        *   $("#CoreCircularGauge").ejCircularGauge("option", "scales", [{ subGuage: [{ location: { y: 0 } }] }]);
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        y: 0
                    }
                }],
                //textAngle ,labelValue:"CustomText", location:{X:5,Y:10}, labelColor, font: { size: "11px", fontFamily: "Arial", fontStyle: "Bold" },
                customLabel: []
            };
        },
        dataTypes: {
            scales: "data"
        },
        observables: ["value", "minimum", "maximum"],
        value: ej.util.valueFunction("value"),
        minimum: ej.util.valueFunction("minimum"),
        maximum: ej.util.valueFunction("maximum"),

        // set model function
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "radius": this.model.radius = options[option]; break;
                    case "frameType": this.model.frameType = options[option]; break;
                    case "backgroundColor": this.model.backgroundColor = options[option]; break;
                    case "interiorGradient": this.model.interiorGradient = options[option]; break;
                    case "readOnly": this.model.readOnly = options[option]; break;
                    case "halfCircleFrameStartAngle": this.model.halfCircleFrameStartAngle = options[option]; break;
                    case "theme": this.model.theme = options[option]; break;
                    case "isRadialGradient": this.model.isRadialGradient = options[option]; break;
                    case "value":
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (j = 0; this.model.scales[i].pointers[j] != null; j++) {
                                this.model.scales[i].pointers[j].value = parseFloat(this.value());
                            }
                        }
                        break;
                    case "minimum":
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            this.model.scales[i].minimum = parseInt(this.minimum());
                        }
                        break;
                    case "maximum":
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            this.model.scales[i].maximum = parseInt(this.maximum());
                        }
                        break;
                    case "scales":
                        $.extend(true, this.model.scales, options[option]);
                        break;
                }
            }
            this._init();
        },

        /**
           * destroy the circular gauge widget.
           * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
           * @alias destroy
           * @return jQuery
           * @example 
           * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
           * &lt;script&gt;
           * //Create the CircularGauge
           * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
           * circulargaugeObj.destroy(); //destroy the circular gauge
           * &lt;/script&gt;
           * @example 
           * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
           * &lt;script&gt;
           * //Enable the CircularGauge
           * $("#CoreCircularGauge").ejCircularGauge("destroy");	
           * &lt;/script&gt;
           * @memberof ejCircularGauge
           * @instance
        */
        _destroy: function () {
            this._unWireEvents();
            this.element.remove();
        },


        // constructor function
        /**
         * Constructor to initialize the gauge	
		 * @private
         */
        _init: function () {
            //this._isHalfCircle = false;
            this._scalesInitialize();
            this._trigger("load");
            this._setTheme();
            this._initialize();
            if (!this.model.readOnly)
                this._wireEvents();
            this._onWindowResize();
        },
        /**
         * Invoke when window size changes 
		 * @private
         */
        _onWindowResize: function () {
            if (this.model.canResize) {
                proxy = this;
                if (!this.isDevice())
                    this._on($(window), "resize", proxy.resizeCanvas);
                else
                    this._on($(window), "orientationchange", proxy.resizeCanvas);
            }
        },
        /**
         * To check whether it is a device or not
		 * @private
         */
        isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
         * To initialize the gauge object
		 * @private
         */
        _initialize: function () {
            this._initObject(this);
            this._drawOuterLayer();
            this._drawScales();
            if (this.model.renderComplete)
                this._onRenderComplete();
        },
        /**
         * To initialize the scales
		 * @private
         */
        _scalesInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },
        /**
         * To check the each array element to load
		 * @private
         */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },
        /**
         * To Load individual default values
		 * @private
         */
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._LoadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
        /**
         * To initialize the gauge object
		 * @private
         */
        _initObject: function (element) {
            this.element.addClass("e-widget");
            element.GaugeEl = this.element;
            for (var i = 0; this.model.scales[i] != null; i++) {
                if (this.model.scales[i].minimum == null)
                    this.model.scales[i].minimum = this.minimum();
                if (this.model.scales[i].maximum == null)
                    this.model.scales[i].maximum = this.maximum();
                for (j = 0; this.model.scales[i].pointers[j] != null; j++) {
                    if (this.model.scales[i].pointers[j].value == null)
                        this.model.scales[i].pointers[j].value = this.value();
                }
            }
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.GaugeEl.append(element.canvasEl);
            element.canvasEl.attr("role", "presentation");
            initialDivWidth = element.GaugeEl.width();
            initialDivHeight = element.GaugeEl.height();
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            if (this._isHalfCircle && this._isHalfCircle()) {
                if ((element.model.halfCircleFrameEndAngle - element.model.halfCircleFrameStartAngle) >= 180) {
                    if ((element.model.halfCircleFrameStartAngle == 180) && (element.model.halfCircleFrameEndAngle == 360)) {
                        element.centerY = element.canvasEl[0].height;
                    }
                    else if ((element.model.halfCircleFrameStartAngle == 0) && (element.model.halfCircleFrameEndAngle == 180)) {
                        element.centerY = 0;
                    }
                    else if ((element.model.halfCircleFrameStartAngle == 90) && (element.model.halfCircleFrameEndAngle == 270)) {
                        element.centerX = element.canvasEl[0].width;
                    }
                    else if ((element.model.halfCircleFrameStartAngle == 270) && (element.model.halfCircleFrameEndAngle == 90)) {
                        element.centerX = 0;
                    }
                }
                else if ((element.model.halfCircleFrameEndAngle - element.model.halfCircleFrameStartAngle) <= 90) {
                    if ((element.model.halfCircleFrameStartAngle >= 0) && (element.model.halfCircleFrameStartAngle <= 180)) {
                        element.centerX = 0;
                        element.centerY = 0;
                    }
                    if ((element.model.halfCircleFrameStartAngle == 90)) {
                        element.centerX = element.canvasEl[0].width;
                    }
                    if (element.model.halfCircleFrameStartAngle == 0 || element.model.halfCircleFrameStartAngle == 270) {
                        element.centerX = 0;
                    }
                    if ((element.model.halfCircleFrameStartAngle == 180) && (element.model.halfCircleFrameEndAngle == 270)) {
                        element.centerX = element.canvasEl[0].width;
                        element.centerY = element.canvasEl[0].height / 2;
                    }
                }
            }
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
        /**
         * Wiring the events to Circular gauge control		
		 * @private
         */
        _wireEvents: function () {
            this.startEv = 'mousedown';
            this.moveEv = 'mousemove';
            this.endEv = 'mouseup';
            this.onMouseMoveHandler = $.proxy(this._onMouseMove, this);
            this.onMouseUpHandler = $.proxy(this._onMouseUp, this);
            this.onMouseDownHandler = $.proxy(this._onMouseDown, this);
            $(this.canvasEl).bind(this.startEv, this.onMouseDownHandler);
        },
        /**
         * To unWiring the events to Circular gauge control		
		 * @private
         */
        _unWireEvents: function () {
            $(this.canvasEl).unbind(this.startEv, this.onMouseDownHandler);
        },
        /**
         * Section For handle the Mouse Down event
		 * @private
         */
        _onMouseDown: function (e) {
            this._blockDefaultActions(e);
            var startPoint = { "x": this.centerX, "y": this.centerY };
            var endPoint = { "x": e.pageX - $(this.canvasEl).offset().left, "y": e.pageY - $(this.canvasEl).offset().top };
            var indegree = 180 * this._getCirucumferenceAngle(startPoint, endPoint) / Math.PI;
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.pointers != null) {
                    self.pointerEl = element.pointers;
                    $.each(element.pointers, function (pointerIndex, pointerElement) {
                        if (self._isHalfCircle())
                            element.showBackNeedle = false;
                        angle = self._getAngle(pointerElement.value);
                        if (angle > 360)
                            angle = angle - 360;
                        var greatervalue = angle + pointerElement.pointerWidth;
                        var lesservalue = angle - pointerElement.pointerWidth;
                        radius = Math.sqrt((endPoint.x - startPoint.x) * (endPoint.x - startPoint.x) + (endPoint.y - startPoint.y) * (endPoint.y - startPoint.y));
                        if (self._isBetween(lesservalue, greatervalue, indegree) && radius <= pointerElement.pointerLength) {
                            if (self.model.mouseClick)
                                self._onMouseClick(angle, element.value);
                            self.activeElement = pointerElement;
                            $(document).bind(self.moveEv, self.onMouseMoveHandler);
                            $(document).bind(self.endEv, self.onMouseUpHandler);
                        }
                    });
                }
            });
        },
        /**
         * Section For handle the Mouse Up event
		 * @private
         */
        _onMouseUp: function (e) {
            $(document).unbind(self.moveEv, self.onMouseMoveHandler);
            $(document).unbind(self.endEv, self.onMouseUpHandler);
            if (this.model.mouseClickUp)
                this._onMouseClickUp(this._getAngle(this.activeElement.value), this.activeElement.value);
        },
        /**
         * Section For handle the Mouse Move event
		 * @private
         */
        _onMouseMove: function (e) {
            this._blockDefaultActions(e);
            var startPoint = { "x": this.centerX, "y": this.centerY };
            var endPoint = { "x": e.pageX - $(this.canvasEl).offset().left, "y": e.pageY - $(this.canvasEl).offset().top };
            var indegree = 180 * this._getCirucumferenceAngle(startPoint, endPoint) / Math.PI;
            if (indegree < this.scaleEl[this.scaleIndex].startAngle && !this._isHalfCircle())
                indegree = indegree + 360;
            //if ((this._isHalfCircle() && this._isBetween(this.scaleEl[this.scaleIndex].startAngle, this.scaleEl[this.scaleIndex].sweepAngle, indegree)) || !this._isHalfCircle())
            //    this.activeElement.value = this._getValue(indegree);
            if (this.model.mouseClickMove)
                this._onMouseClickMove(this._getAngle(this.activeElement.value), this.activeElement.value);
            if (this._getValue(indegree) <= this.scaleEl[this.scaleIndex].maximum)
                this.activeElement.value = this._getValue(indegree);
            if (this.contextEl.putImageData)
                this._reDrawPointer();
            else
                this._init();
        },

        /**
         * To check whether gauge is half circle or not
		 * @private
         */
        _isHalfCircle: function () {
            if (this.model.frameType == "halfCircle")
                return true;
        },
        /**
         * To get the Y position of semi circular gauge
		 * @private
         */
        _getHalfCircleYPosition: function () {
            return this._getYCordinate(this.centerY, 0, (this.model.halfCircleFrameStartAngle + this.model.halfCircleFrameEndAngle) / 2);
        },
        /**
         * To get the X position of semi circular gauge
		 * @private
         */
        _getHalfCircleXPosition: function () {
            return this._getXCordinate(this.centerX, 0, (this.model.halfCircleFrameStartAngle + this.model.halfCircleFrameEndAngle) / 2);
        },
        /**
         * To get the X Co-ordinate of semi circular gauge
		 * @private
         */
        _getXCordinate: function (x, radius, angle) {
            var x1 = x + radius * (Math.cos(Math.PI * (angle / 180)));
            return x1;
        },
        /**
         * To get the Y Co-ordinate of semi circular gauge
		 * @private
         */
        _getYCordinate: function (y, radius, angle) {
            var y1 = y + radius * (Math.sin(Math.PI * (angle / 180)));
            return y1;
        },
        /**
         * To get the angle circular gauge
		 * @private
         */
        _getAngle: function (value) {
            var tempValue, angle;
            if (value >= this.scaleEl[this.scaleIndex].minimum && value <= this.scaleEl[this.scaleIndex].maximum)
                tempValue = this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise" ? value - this.scaleEl[this.scaleIndex].minimum : this.scaleEl[this.scaleIndex].maximum - value;
            else {
                if (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise") {
                    if (value <= this.scaleEl[this.scaleIndex].minimum)
                        tempValue = this.scaleEl[this.scaleIndex].minimum;
                    else
                        tempValue = this.scaleEl[this.scaleIndex].maximum;
                }
                else {
                    if (value <= this.scaleEl[this.scaleIndex].minimum)
                        tempValue = this.scaleEl[this.scaleIndex].maximum;
                    else
                        tempValue = this.scaleEl[this.scaleIndex].minimum;
                }
            }
            angle = (tempValue * ((this.scaleEl[this.scaleIndex].sweepAngle) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum))) + (this.scaleEl[this.scaleIndex].startAngle);
            return angle;
        },
        /**
         * To Handle the Decimal value manupulation
		 * @private
         */
        _subtractDecimal: function (value, interval) {
            var strValue = value.toString();
            var strInterval = interval.toString();
            var vDecimal;
            var iDecimal;
            if (strValue.indexOf('.') > -1)
                vDecimal = strValue.length - strValue.indexOf('.') - 1;
            else
                vDecimal = 0;
            if (strInterval.indexOf('.') > -1)
                iDecimal = strInterval.length - strInterval.indexOf('.') - 1;
            else
                iDecimal = 0;
            var decimal = vDecimal > iDecimal ? vDecimal : iDecimal;
            var correctValue = (value * Math.pow(10, decimal) - interval * Math.pow(10, decimal)) / Math.pow(10, decimal);
            return correctValue;
        },
        /**
         * To get the circumference angle for the circular gauge
		 * @private
         */
        _getCirucumferenceAngle: function (startPoint, endPoint) {
            if (endPoint.x > startPoint.x) {
                if (endPoint.y > startPoint.y)
                    return this._tangent(startPoint, endPoint); //quadrant2
                else {
                    if (endPoint.y == startPoint.y)
                        return 0;
                    else
                        return 2 * Math.PI + this._tangent(startPoint, endPoint); //quadrant1
                }
            }
            else {
                if (endPoint.x == startPoint.x) {
                    if (endPoint.y == startPoint.y)
                        return 0;   //0 degree
                    else {
                        if (endPoint.y > startPoint.y)
                            return Math.PI / 2;  // 90 degree
                        else
                            return 1.5 * Math.PI; // 270 degree
                    }
                }
                else {
                    if (endPoint.y == startPoint.y)
                        return Math.PI; // 180 degree
                    else {
                        if (endPoint.y > startPoint.y)
                            return Math.PI + this._tangent(startPoint, endPoint);  // quadrant3
                        else
                            return Math.PI + this._tangent(startPoint, endPoint); //quadrant4
                    }
                }
            }
        },
        /**
         * To Return the Tangent value in radians
		 * @private
         */
        _tangent: function (startPoint, endPoint) {
            var distance = (endPoint.y - startPoint.y) / (endPoint.x - startPoint.x);
            var inradians = Math.atan(distance);
            //indegrees=180*inradians/Math.PI
            return inradians;
        },
        /**
         * To get the Image for Indicator
		 * @private
         */
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                this._getPointerImage();
        },
        /**
         * To get the Image for Pointer
		 * @private
         */
        _getPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
        /**
         * To get the Image for SubGauge
		 * @private
         */
        _getSubGaugeImage: function () {
            if (this.rangeImage)
                return this.rangeImage;
            else
                return this._getRangeImage();
        },
        /**
         * To get the Image for CustomLabel
		 * @private
         */
        _getCustomLabelImage: function () {
            if (this.subGaugeImage)
                return this.subGaugeImage;
            else
                return this._getSubGaugeImage();
        },
        /**
         * To get the Image for Range
		 * @private
         */
        _getRangeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
        /**
         * To get the Image for Label
		 * @private
         */
        _getLabelImage: function () {
            if (this.tickImage)
                return this.tickImage;
            else
                return this._getTickImage();
        },
        /**
         * To get the Image for Tick
		 * @private
         */
        _getTickImage: function () {
            if (this.scaleImage)
                return this.scaleImage;
            else
                return this.outerImage;
        },
        /**
         * To get the value from angle given
		 * @private
         */
        _getValue: function (angle) {
            return ((angle - this.scaleEl[this.scaleIndex].startAngle) / this.scaleEl[this.scaleIndex].sweepAngle) * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum);
        },
        /**
         * To draw scales
		 * @private
         */
        _drawScales: function () {
            var self = this;
            this.scaleEl = this.model.scales;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.showScaleBar)
                    self._setScaleCordinates(element);
            });
            this._setTicks();
            this._setLabels();
            this._setRanges();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            $.each(this.model.scales, function (index, element) {
                if (element.showPointers) {
                    if (element.pointers.length > 1)
                        self.model.animate = false;
                }
            });
            if (!this.contextEl.putImageData)
                this.model.animate = false;
            if (this.model.animationSpeed != null && this.model.animationSpeed > 0) {
                var delay = this.model.animationSpeed / 25;
                if (delay >= 0) {
                    if (this.model.animate) {
                        this._onAnimate(delay);
                    }
                }
            }
            this._setIndicators();
        },
        /**
         * To set the value for ticks coordinates
		 * @private
         */
        _setTicks: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showTicks) {
                    self.scaleIndex = index;
                    if (element.ticks != null) {
                        self.tickEl = element.ticks;
                        $.each(element.ticks, function (tickIndex, tickElement) {
                            self.tickIndex = tickIndex;
                            self._setTicksCordinates(tickIndex, tickElement);
                        });
                    }
                }
            });
        },
        /**
         * To set the Label coordnates
		 * @private
         */
        _setLabels: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showLabels) {
                    self.scaleIndex = index;
                    if (element.labels != null) {
                        self.labelEl = element.labels;
                        $.each(element.labels, function (labelIndex, labelElement) {
                            self.labelIndex = labelIndex;
                            self._setLabelCoridnates(labelIndex, labelElement);
                        });
                    }
                }
            });
        },
        /**
         * To set the Indicators coordnates
		 * @private
         */
        _setIndicators: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showIndicators) {
                    self.scaleIndex = index;
                    if (element.indicators != null) {
                        self.indicatorEl = element.indicators;
                        $.each(element.indicators, function (indIndex, indElement) {
                            self.indicatorIndex = indIndex;
                            self._drawIndicator(indIndex, indElement);
                        });
                    }
                }
            });
        },
        /**
         * To set the Pointers coordnates
		 * @private
         */
        _setPointers: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showPointers) {
                    self.scaleIndex = index;
                    if (element.pointers != null) {
                        self.pointerEl = element.pointers;
                        $.each(element.pointers, function (pointerIndex, pointerElement) {
                            if (self._isHalfCircle())
                                element.showBackNeedle = false;
                            self.pointerIndex = pointerIndex;
                            self._drawPointers(pointerIndex, pointerElement);
                            //  setInterval( function(){ self._drawPointers(pointerIndex,pointerElement);}, 30 ); 
                        });
                    }
                }

            });
        },
        /**
         * To handle values while Animating
		 * @private
         */
        _onAnimate: function (delay) {
            var self = this, timer;
            var currentValue = self.model.scales[0].minimum;
            var pointerValue = self.model.scales[0].pointers[0].value;
            timer = setInterval(function pointerchan() {
                if (pointerValue > currentValue) {
                    currentValue = currentValue + 1;
                    self.setPointerValue(0, 0, currentValue);
                }
                else if (currentValue == pointerValue) {
                    window.clearInterval(timer);
                }
            }, delay);
        },
        /**
         * To set the Ranges coordnates
		 * @private
         */
        _setRanges: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showRanges) {
                    self.scaleIndex = index;
                    if (element.ranges != null) {
                        self.rangeEl = element.ranges;
                        $.each(element.ranges, function (rangeIndex, rangeElement) {
                            self.rangeIndex = rangeIndex;
                            self._setRangeCordinates(rangeIndex, rangeElement);
                        });
                    }
                }
            });
        },
        /**
         * To set the CustomLabel coordnates
		 * @private
         */
        _setCustomLabel: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.customLabel != null) {
                    self.customLabelEl = element.customLabel;
                    $.each(element.customLabel, function (labelIndex, labelElement) {
                        self.customLabelIndex = index;
                        self._setCustomLabelCordinates(labelIndex, labelElement);
                    });
                }
            });
        },
        /**
         * To Invoke the subgauge elements
		 * @private
         */
        _subGauge: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.subGauge != null) {
                    self.subGaugeEl = element.subGauge;
                    $.each(element.subGauge, function (subGaugeIndex, subGaugeElement) {
                        self.subGaugeIndex = index;
                        self._setSubGuage(subGaugeIndex, subGaugeElement);
                    });
                }
            });
        },
        /**
         * To set the Scale coordnates
		 * @private
         */
        _setScaleCordinates: function (element) {
            this.location = {
                "centerX": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "centerY": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY,
                "startAngle": element.startAngle, "endAngle": (element.startAngle + element.sweepAngle),
                "startRadius": this.scaleEl[this.scaleIndex].scaleRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2, "endRadius": this.scaleEl[this.scaleIndex].scaleRadius + this.scaleEl[this.scaleIndex].scaleBarSize / 2
            };
            this.style = {
                "radius": element.scaleRadius - element.scaleBarSize / 2,
                "strokeStyle": (element.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.borderColor), // this._getColor(element, element.borderColor),
                "lineWidth": element.scaleBorderWidth,
                "size": element.scaleBarSize,
                "isFill": true,
                "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                "isStroke": true,
                "shadowOffset": element.shadowOffset,
                "fillStyle": (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor), //this._getColor(element, element.backgroundColor),
                "counterClockwise": element.scaleDirection == "Clockwise" ? false : true
            };
            this.maximum(element.maximum);
            this.minimum(element.minimum);
            this._drawScaleBar(this.location, this.style);
            if (this.contextEl.getImageData)
                this.scaleImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
        /**
         * To draw Outer layer for the circuar gauge
		 * @private
         */
        _drawOuterLayer: function () {
            var interiorGradient, startPoint, endPoint, hcFrameInnerAngle;
            if (this._isHalfCircle()) {
                startPoint = { "x": this.centerX, "y": this.centerY };
                endPoint = { "x": this.centerX + this.model.radius, "y": this.centerY / 2 };
                hcFrameInnerAngle = this._getCirucumferenceAngle(startPoint, endPoint);
            }
            if (!this.model.isRadialGradient) {
                interiorGradient = this.contextEl.createLinearGradient(0, 0, 0, this.canvasEl[0].height);
            }
            else {
                interiorGradient = this.contextEl.createRadialGradient(this.canvasEl[0].width / 2, this.canvasEl[0].height / 2, 0, this.canvasEl[0].width / 2, this.canvasEl[0].height / 2, this.model.radius);
            }
            if (!ej.isNullOrUndefined(this.model.interiorGradient))
                this._setGradientColor(this, interiorGradient, this.model.interiorGradient.colorInfo);

            this.frameOuterLocation = { "centerX": this.centerX, "hcCenterX": this.centerX, "hcCenterY": this.centerY, "centerY": this.centerY, "startAngle": this._isHalfCircle() ? Math.PI * ((this.model.halfCircleFrameStartAngle) / 180) : 0, "endAngle": this._isHalfCircle() ? Math.PI * ((this.model.halfCircleFrameEndAngle) / 180) : 2 * Math.PI };
            if (this._isHalfCircle())
                this.frameInnerLocation = { "centerX": this.centerX, "hcCenterX": this._getXCordinate(this.centerX, 0, (this.model.halfCircleFrameStartAngle + this.model.halfCircleFrameEndAngle) / 2), "hcCenterY": this._getYCordinate(this.centerX, 0, (this.model.halfCircleFrameStartAngle + this.model.halfCircleFrameEndAngle) / 2), "centerY": this.centerY, "startAngle": Math.PI * ((this.model.halfCircleFrameStartAngle) / 180) + hcFrameInnerAngle, "endAngle": Math.PI * ((this.model.halfCircleFrameEndAngle) / 180) - hcFrameInnerAngle };
            this.frameInnerStyle = {
                "radius": this.model.radius,
                "isStroke": false, "isFill": true,
                "fillStyle": (this.model.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(null, this.model.backgroundColor),
                "counterClockwise": false
            };
            if (this.model.frameType == "fullCircle") {
                this._drawCircleFrame(this.frameOuterLocation, this.frameInnerStyle);
            }
            else if (this.model.frameType == "halfCircle") {
                this._drawHalfCircle(this.frameInnerLocation, this.frameInnerStyle);
            }
            if (this.contextEl.getImageData)
                this.outerImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
        /**
         * To set Ticks Cordinates for the circuar gauge
		 * @private
         */
        _setTicksCordinates: function (index, element) {
            var startX, startY, lineHeight, minimumMeet, interval, radius, angle;
            if (element.tickStyle == "Major") {
                interval = this.scaleEl[this.scaleIndex].majorIntervalValue;
                this.majorTickHeight = element.tickHeight;
            }
            else {
                interval = this.scaleEl[this.scaleIndex].minorIntervalValue;
            }
            if (element.tickPosition == "Far")
                radius = this.scaleEl[this.scaleIndex].scaleRadius + this.scaleEl[this.scaleIndex].scaleBarSize / 2 + this.scaleEl[this.scaleIndex].scaleBorderWidth / 2 + element.distanceFromScale;
            if (element.tickPosition == "Center")
                radius = this.scaleEl[this.scaleIndex].scaleRadius - element.tickHeight / 2 - this.scaleEl[this.scaleIndex].scaleBorderWidth / 2 - element.distanceFromScale;
            if (element.tickPosition == "Near")
                radius = this.scaleEl[this.scaleIndex].scaleRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - this.scaleEl[this.scaleIndex].scaleBorderWidth / 2 - element.distanceFromScale;
            lineHeight = element.tickPosition == "Near" ? -element.tickHeight : element.tickHeight;
            for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value -= interval) {
                if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                    if (value == this.scaleEl[this.scaleIndex].minimum)
                        minimumMeet = true;
                    angle = this._getAngle(value);
                    angle = angle > 360 ? angle - 360 : angle;
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                    this.location = { "startX": startX, "startY": startY };
                    this.style = { "angle": element.angle + angle, "isStroke": true, "isFill": false, "lineHeight": lineHeight, "lineWidth": element.tickWidth, "strokeStyle": (element.tickColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.tickColor) };
                    if (this.model.drawTicks)
                        this._onDrawTicks(angle, value);
                    this._drawTickMark(this.location, this.style);
                }
            }
            if (!minimumMeet) {
                angle = this._getAngle(this.scaleEl[this.scaleIndex].minimum);
                angle = angle > 360 ? angle - 360 : angle;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                this.location = { "startX": startX, "startY": startY };
                this.style = { "angle": element.angle + angle, "isStroke": true, "isFill": false, "lineHeight": lineHeight, "lineWidth": element.tickWidth, "strokeStyle": (element.tickColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.tickColor) };
                //if (this.model.drawTicks)
                //    this._onDrawTicks(angle, value);
                //this._drawTickMark(this.location, this.style);
            }

            if (this.contextEl.getImageData)
                this.tickImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
        /**
         * To set Label Cordinates for the circuar gauge
		 * @private
         */
        _setLabelCoridnates: function (index, element) {
            var minimumMeet;
            var startX, startY, lineHeight, incrementAngle, interval, radius;
            interval = element.labelStyle == "Major" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
            incrementAngle = element.labelStyle == "Major" ? (this.majorIntervalAngle = this.scaleEl[this.scaleIndex].sweepAngle / ((this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) / interval)) : this.scaleEl[this.scaleIndex].sweepAngle / ((this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) / interval);
            if (element.labelPosition == "Far")
                radius = this.scaleEl[this.scaleIndex].scaleRadius + this.scaleEl[this.scaleIndex].scaleBarSize / 2 + this.majorTickHeight + element.distanceFromScale;
            if (element.labelPosition == "Center")
                radius = this.scaleEl[this.scaleIndex].scaleRadius - 10 - element.distanceFromScale;
            if (element.labelPosition == "Near")
                radius = this.scaleEl[this.scaleIndex].scaleRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - 10 - this.majorTickHeight - element.distanceFromScale;
            for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value = this._subtractDecimal(value, interval)) {
                if (value == this.scaleEl[this.scaleIndex].minimum)
                    minimumMeet = true;
                if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                    if (value == this.scaleEl[this.scaleIndex].minimum && !element.includeFirstValue) {
                        continue;
                    }
                    angle = this._getAngle(value);
                    angle = angle > 360 ? angle - 360 : angle;
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                    this.location = { "startX": startX, "startY": startY };
                    this.style = { "angle": this.scaleEl[this.scaleIndex].labelAutoAngle ? angle + element.angle : element.angle, "isStroke": false, "isFill": true, "textValue": value, "opacity": !(element.opacity) ? 1 : element.opacity, "font": this._getFontString(element, element.font), "fillStyle": (element.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.labelColor) };
                    if (this.model.drawLabels)
                        this._onDrawLabels(this.scaleEl[this.scaleIndex].labelAutoAngle ? angle + element.angle : element.angle, value);
                    this._drawLabel(this.location, this.style, false);
                }
            }
            if (!minimumMeet) {
                angle = this._getAngle(this.scaleEl[this.scaleIndex].minimum);
                angle = angle > 360 ? angle - 360 : angle;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                this.location = { "startX": startX, "startY": startY };
                this.style = { "angle": this.scaleEl[this.scaleIndex].labelAutoAngle ? angle + element.angle : element.angle, "isStroke": false, "isFill": true, "textValue": this.scaleEl[this.scaleIndex].minimum, "opacity": !(element.opacity) ? 1 : element.opacity, "font": this._getFontString(element, element.font), "lineHeight": lineHeight, "lineWidth": element.tickWidth, "fillStyle": (element.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.labelColor) };
                if (this.model.drawLabels)
                    this._onDrawLabels(angle, value);
                this._drawLabel(this.location, this.style, false);
                angle = this._getAngle(value);
                angle = angle > 360 ? angle - 360 : angle;
            }
            if (this.contextEl.getImageData)
                this.labelImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
        /**
         * To set Range Cordinates for the circuar gauge
		 * @private
         */
        _setRangeCordinates: function (index, element) {
            if (element.startValue >= this.scaleEl[this.scaleIndex].minimum && element.endValue <= this.scaleEl[this.scaleIndex].maximum) {
                var startX, startY, endX, endY, startAngle, radius, endAngle, startRadius, endRadius, grd, controlAngle;
                controlAngle = this._getAngle((element.startValue + element.endValue) / 2);
                startAngle = this._getAngle(element.startValue);
                endAngle = this._getAngle(element.endValue);
                radius = this.scaleEl[this.scaleIndex].scaleRadius - element.distanceFromScale - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.size - this.majorTickHeight;
                startRadius = this.scaleEl[this.scaleIndex].scaleRadius - element.distanceFromScale - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - this.majorTickHeight;
                endRadius = this.scaleEl[this.scaleIndex].scaleRadius - element.distanceFromScale - element.size - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - this.majorTickHeight;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, startAngle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, startAngle);
                endX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, endAngle);
                endY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, endAngle);
                startAngle = (180 * this._getCirucumferenceAngle({ "x": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "y": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY }, { "x": startX, "y": startY })) / Math.PI;
                endAngle = (180 * this._getCirucumferenceAngle({ "x": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "y": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY }, { "x": endX, "y": endY })) / Math.PI;

                if (!ej.isNullOrUndefined(element.rangeGradient)) {
                    grd = this.contextEl.createRadialGradient(this.centerX, this.centerY, endRadius, this.centerX, this.centerY, startRadius);
                    this._setGradientColor(this, grd, element.rangeGradient.colorInfo);
                } else
                    grd = (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor); // this._getColor(element, element.backgroundColor);
                this.location = { "startX": startX, "startY": startY, "endX": endX, "endY": endY, "startAngle": startAngle, "endAngle": endAngle };
                this.style = {
                    "startWidth": element.startWidth,
                    "isFill": true,
                    "fillStyle": grd,
                    "strokeStyle": (element.borderColor == "transparent") ? "rgba(0,0,0,0)" : element.borderColor, // element.borderColor,
                    "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                    "counterClockwise": this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise" ? false : true,
                    "startRadius": startRadius, "endRadius": endRadius, "endWidth": element.endWidth, "lineWidth": element.borderWidth, "isStroke": true
                };
                if (this.model.drawRange)
                    this._onDrawRange(this.location, this.style);
                this._drawRange(this.location, this.style);
            }
            if (this.contextEl.getImageData)
                this.rangeImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
        /**
         * To set the SubGuage
		 * @private
         */
        _setSubGuage: function (index, element) {
            var subGaugeDiv = $('div[id=' + element.controlID + ']');
            if (subGaugeDiv.length > 0 && subGaugeDiv.find('canvas').length) {
                var sourceCanvas = subGaugeDiv.find('canvas')[0].getContext('2d');
                this.contextEl.drawImage(sourceCanvas.canvas, element.location.x, element.location.y, element.subGaugeWidth, element.subGaugeHeight);
                subGaugeDiv.css('display', 'none');
                if (this.contextEl.getImageData)
                    this.subGaugeImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
            }
        },
        /**
         * To set Custom Label Cordinates for the circuar gauge
		 * @private
         */
        _setCustomLabelCordinates: function (index, element) {
            element.labelColor = element.labelColor ? (element.labelColor) : "#282828";
            this.location = element.location ? ({ "startX": element.location.x, "startY": element.location.y }) : { "startX": 0, "startY": 0 };
            this.style = { "angle": element.textAngle, "textValue": element.labelValue, "fillStyle": (element.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.labelColor), "font": this._getFontString(element, element.font) };
            if (this.model.drawCustomLabel)
                this._onDrawCustomLabel(this.location, this.style);
            this._drawLabel(this.location, this.style, true);
            if (this.contextEl.getImageData)
                this.customLabelImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
        /**
         * To Draw Indicators
		 * @private
         */
        _drawIndicator: function (index, element) {
            var self = this, isInStateRange = false;
            this.location = { "centerX": element.indicatorLocation.x - element.indicatorWidth / 2, "textLocation": element.indicatorsLocation, "centerY": element.indicatorLocation.y - element.indicatorHeight / 2, "startAngle": 0, "endAngle": 2 * Math.PI };
            this.style = {
                "radius": (element.indicatorHeight + element.indicatorWidth) / 2,
                "strokeStyle": "#2BA104",
                "cornerRadius": element.indicatorStyle == "RoundedRectangle" ? 2 : 0,
                "height": element.indicatorHeight,
                "width": element.indicatorWidth,
                "lineWidth": element.indicatorBorderWidth,
                "fillStyle": "#2BA104",
                "isStroke": true,
                "isFill": true,
                "indicatorText": element.indicatorText,
                "textColor": null,
                "font": null,
                "counterClockwise": false
            };
            if (this.model.drawIndicators)
                this._onDrawIndicators(this.style, this.location);
            if (element.indicatorStyle == "Image") {
                var image = new Image();
                image.onload = function () {
                    self.contextEl.drawImage(this, element.indicatorLocation.x, element.indicatorLocation.y);
                };
                image.src = element.imageUrl;
            }
            else if (element.stateRanges != null) {
                $.each(element.stateRanges, function (index, srEl) {//srEl = StateRange Element
                    if (self.pointerEl[self.pointerIndex].value >= srEl.startValue && self.pointerEl[self.pointerIndex].value <= srEl.endValue) {
                        isInStateRange = true;
                        if (srEl.text.length > 0) {
                            self.style.indicatorText = srEl.text;
                            self.style.textColor = (srEl.textColor == "transparent") ? "rgba(0,0,0,0)" : srEl.textColor; // srEl.textColor;
                            self.style.font = element.font;
                        }
                        self.style.strokeStyle = (srEl.borderColor == "transparent") ? "rgba(0,0,0,0)" : srEl.borderColor;
                        self.style.fillStyle = (srEl.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : srEl.backgroundColor;
                        if (element.indicatorStyle == "RoundedRectangle")
                            self._drawRectangleFrame(self.location, self.style);
                        else
                            self["_draw" + element.indicatorStyle + "Frame"](self.location, self.style);
                    }
                });
            }
            if (!isInStateRange && element.indicatorStyle != "Image") {
                if (element.indicatorStyle == "RoundedRectangle")
                    self._drawRectangleFrame(self.location, self.style);
                else
                    self["_draw" + element.indicatorStyle + "Frame"](self.location, self.style);
            }
            if (this.contextEl.getImageData && element.indicatorStyle != "Image")
                this.indicatorImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
        /**
         * To Draw Scale Bar
		 * @private
         */
        _drawScaleBar: function (location, style) {
            this.contextEl.shadowColor = (style.strokeStyle == "transparent") ? "rgba(0,0,0,0)" : style.strokeStyle;
            if (style.shadowOffset)
                this.contextEl.shadowBlur = style.shadowOffset;
            this._contextOpenPath(style, this);
            if ((location.endAngle - location.startAngle) == 0)
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (0 / 180), Math.PI * (0 / 180), false);
            else if ((location.endAngle - location.startAngle) % 360 == 0) {
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (0 / 180), Math.PI * (360 / 180), false);
                this.contextEl.arc(location.centerX, location.centerY, location.endRadius, Math.PI * (360 / 180), Math.PI * (0 / 180), true);
            }
            else {
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), false);
                this.contextEl.arc(location.centerX, location.centerY, location.endRadius, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), true);
            }
            this._contextClosePath(style, this);
        },
        /**
         * To Draw TickMark
		 * @private
         */
        _drawTickMark: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.moveTo(0, 0);
            this.contextEl.rotate((Math.PI * (style.angle / 180)));
            this.contextEl.lineTo(style.lineHeight, 0);
            this._contextClosePath(style, this);
        },
        /**
         * To Draw Label
		 * @private
         */
        _drawLabel: function (location, style, isCustomLabel) {
            this._contextOpenPath(style, this);
            this.contextEl.textAlign = "center";
            this.contextEl.textBaseline = "middle";
            this.contextEl.font = style.font;
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.moveTo(0, 0);
            this.scaleEl[this.scaleIndex].labelAutoAngle ? this.contextEl.rotate((Math.PI * ((style.angle - 270) / 180))) : this.contextEl.rotate((Math.PI * (style.angle / 180)));
            if (!ej.isNullOrUndefined(isCustomLabel) && !isCustomLabel) {
                var textPostion = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitTextPosition;
                if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "Back")
                    style.textValue = style.textValue + this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText;
                else if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "Front")
                    style.textValue = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText + style.textValue;
            }
            this.contextEl.fillText(style.textValue, 0, 0);
            this._contextClosePath(style, this);
        },
        /**
         * To Draw Circle Frame
		 * @private
         */
        _drawCircleFrame: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.arc(location.centerX, location.centerY, style.radius, location.startAngle, location.endAngle, style.counterClockwise);
            this._contextClosePath(style, this);
            if (style.indicatorText)
                this._drawText(location, style);
        },
        /**
         * To Draw Half Circle
		 * @private
         */
        _drawHalfCircle: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.lineJoin = "round";
            this.contextEl.arc(location.centerX, location.centerY, style.radius, location.startAngle, location.endAngle, false);
            this.contextEl.lineTo(location.hcCenterX, location.hcCenterY);
            this._contextClosePath(style, this);
        },
        /**
         * To Draw Rectangular Frame
		 * @private
         */
        _drawRectangleFrame: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.centerX, location.centerY - style.height / 2);
            this.contextEl.moveTo(style.cornerRadius, 0);
            this.contextEl.lineTo(style.width - style.cornerRadius, 0);
            this.contextEl.quadraticCurveTo(style.width, 0, style.width, style.cornerRadius);
            this.contextEl.lineTo(style.width, style.height - style.cornerRadius);
            this.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.cornerRadius, style.height);
            this.contextEl.lineTo(style.cornerRadius, style.height);
            this.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.cornerRadius);
            this.contextEl.lineTo(0, style.cornerRadius);
            this.contextEl.quadraticCurveTo(0, 0, style.cornerRadius, 0);
            this._contextClosePath(style, this);
            if (style.indicatorText)
                this._drawText(location, style);
        },
        /**
         * To Draw Text for Indicators
		 * @private
         */
        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.textAlign = "center";
            this.contextEl.fillStyle = (style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor;
            this.contextEl.font = style.font;
            this.contextEl.fillText(style.indicatorText, location.textlocation.x, location.textlocation.y);
            this.contextEl.closePath();
        },
        /**
         * To Draw Range
		 * @private
         */
        _drawRange: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.startRadius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), style.counterClockwise);
            this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.endRadius, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), !style.counterClockwise);
            this._contextClosePath(style, this);
        },
        /**
         * To Draw Text for Pointers
		 * @private
         */
        _drawPointers: function (index, element) {
            var startX, startY, angle, height, grd;
            angle = this._getAngle(element.value);
            angle = angle > 360 ? angle - 360 : angle;
            if (element.pointerType == "Needle") {
                if (element.pointerPosition == "Far")
                    height = element.pointerLength + this.scaleEl[this.scaleIndex].scaleBarSize / 2;
                if (element.pointerPosition == "Center")
                    height = element.pointerLength;
                if (element.pointerPosition == "Near")
                    height = element.pointerLength - this.scaleEl[this.scaleIndex].scaleBarSize / 2;
            }
            else {
                height = element.pointerLength;
                if (element.pointerPosition == "Far") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].scaleRadius + this.scaleEl[this.scaleIndex].scaleBarSize / 2 + element.distanceFromScale, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].scaleRadius + this.scaleEl[this.scaleIndex].scaleBarSize / 2 + element.distanceFromScale, angle);
                }
                if (element.pointerPosition == "Center") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].scaleRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.distanceFromScale, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].scaleRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.distanceFromScale, angle);
                }
                if (element.pointerPosition == "Near") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].scaleRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.distanceFromScale - this.majorTickHeight - 15, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].scaleRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.distanceFromScale - this.majorTickHeight - 15, angle);
                }
            }
            if (height > this.model.radius)
                height = this.model.radius;

            this.location = { "startX": element.pointerType == "Needle" ? this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX : startX, "startY": element.pointerType == "Needle" ? this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY : startY };
            if (!ej.isNullOrUndefined(element.pointerGradient)) {
                grd = this.contextEl.createLinearGradient(0, 0, 0, element.pointerWidth);
                this._setGradientColor(this, grd, element.pointerGradient.colorInfo);
            } else
                grd = (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor);
            this.style = {
                "width": element.pointerWidth,
                "isFill": true,
                "isStroke": true,
                "radius": 0,
                "showBackNeedle": element.showBackNeedle,
                "backNeedleLength": element.backNeedleLength,
                "angle": element.pointerType == "Needle" ? angle : element.pointerPosition == "Far" ? angle : angle + 180,
                "height": height, "lineWidth": element.borderWidth, "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                "strokeStyle": (element.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.borderColor),
                "fillStyle": grd
            };
            if (this.model.drawPointers)
                this._onDrawPointers(angle, element.value);
            if (element.pointerType == "Needle") {
                this._drawNeedlePointer(this.location, this.style, element);
                this._setPointerCap(element);
            }
            else {
                if (element.markerStyle == "RoundedRectangle")
                    this.style.radius = 5;
                this["_draw" + element.markerStyle](this.location, this.style, this);
            }
            if (this.contextEl.getImageData)
                this.pointerImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
            this.value(element.value);
        },
        /**
         * To Draw Needle Pointer
		 * @private
         */
        _drawNeedlePointer: function (location, style, element) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.rotate(Math.PI * ((style.angle) / 180));
            this.contextEl.moveTo(0, -style.width / 2);
            if (this.pointerEl[this.pointerIndex].needleStyle == "Triangle") {
                this.contextEl.lineTo(style.height, 0);
            }
            else if (this.pointerEl[this.pointerIndex].needleStyle == "Rectangle") {
                this.contextEl.lineTo(style.height, -style.width / 2);
                this.contextEl.lineTo(style.height, style.width / 2);
            }
            else if (this.pointerEl[this.pointerIndex].needleStyle == "Trapezoid") {
                this.contextEl.lineTo(style.height, -style.width / 4);
                this.contextEl.lineTo(style.height, style.width / 4);
            }
            else if (this.pointerEl[this.pointerIndex].needleStyle == "Arrow") {
                this.contextEl.lineTo(style.height - style.height / 4, -style.width / 6);
                this.contextEl.lineTo(style.height - style.height / 4, -style.width / 2);
                this.contextEl.lineTo(style.height, 0);
                this.contextEl.lineTo(style.height - style.height / 4, style.width / 2);
                this.contextEl.lineTo(style.height - style.height / 4, style.width / 6);
            }
            this.contextEl.lineTo(0, style.width / 2);
            if (style.showBackNeedle) {
                this.contextEl.lineTo(-(style.backNeedleLength + this.scaleEl[this.scaleIndex].pointerCapRadius / 2), style.width / 2);
                this.contextEl.lineTo(-(style.backNeedleLength + this.scaleEl[this.scaleIndex].pointerCapRadius / 2), -style.width / 2);
            }
            this.canvasEl.attr("aria-label", this.model.scales[this.scaleIndex].pointers[this.pointerIndex].value);
            this._contextClosePath(style, this);
        },
        /**
         * To set the pointer cap position
		 * @private
         */
        _setPointerCap: function (element) {
            var grd;
            //var gradient = [{ "ColorStop": 0.19, "Color": "#F2DFDF" }, { "ColorStop": 0.71, "Color": "#FFFFFF" }, { "ColorStop": 0.77, "Color": "#836B33" }];
            grd = this.contextEl.createRadialGradient(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.canvasEl[0].height / 2, 0, this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.canvasEl[0].height / 2, this.scaleEl[this.scaleIndex].pointerCapRadius);
            if (!ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].capInteriorGradient))
                this._setGradientColor(this, grd, this.scaleEl[this.scaleIndex].capInteriorGradient.colorInfo);
            else
                grd = (this.scaleEl[this.scaleIndex].capBackgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].capBackgroundColor)); // this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].capBackgroundColor));
            this.location = { "centerX": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "centerY": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, "startAngle": 0, "endAngle": 2 * Math.PI };
            this.style = { "isStroke": true, "isFill": true, "strokeStyle": (this.scaleEl[this.scaleIndex].capBorderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].capBorderColor)), "radius": this.scaleEl[this.scaleIndex].pointerCapRadius, "lineWidth": this.scaleEl[this.scaleIndex].pointerCapBorderWidth, "fillStyle": grd };
            if (this.model.drawPointerCap)
                this._onDrawPointerCap();
            this._drawCircleFrame(this.location, this.style);
        },
        /**
         * To Redraw the sclae
		 * @private
         */
        _reDrawScale: function () {
            this.contextEl.putImageData(this.outerImage, 0, 0);
            this._drawScales();

        },
        /**
         * To Redraw the Pointers
		 * @private
         */
        _reDrawPointer: function () {
            this.contextEl.putImageData(this._getPointerImage(), 0, 0);
            this._setPointers();
            this._setIndicators();
        },
        /**
         * To redraw the Custom Label
		 * @private
         */
        _reDrawCustomLabel: function () {
            this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },
        /**
         * To Redraw the Ranges
		 * @private
         */
        _reDrawRange: function () {
            this.contextEl.putImageData(this._getRangeImage(), 0, 0);
            this._setRanges();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },
        /**
         * To Redraw the Label
		 * @private
         */
        _reDrawLabel: function () {
            this.contextEl.putImageData(this._getLabelImage(), 0, 0);
            this._setLabels();
            this._setRanges();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },
        /**
         * To Redraw the TickMark
		 * @private
         */
        _reDrawTickMark: function () {
            this.contextEl.putImageData(this._getTickImage(), 0, 0);
            this._setTicks();
            this._setLabels();
            this._setRanges();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },
        /**
         * To Redraw the SubGauge
		 * @private
         */
        _reDrawSubGauge: function () {
            this.contextEl.putImageData(this._getSubGaugeImage(), 0, 0);
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },

        // ************************************* Client side Methods ***********************************
        /**
         * To check whether the device is andriod or not
		 * @private
         */
        isAndroid: function () {
            return (/android/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
          * Switching the redraw option for the gauge
          * @return jQuery
          * @param {string} argument.value redraw value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the redraw value
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.reDraw("scale");
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the redraw value
          * $("#CoreCircularGauge").ejCircularGauge("reDraw('scale')"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        reDraw: function (value) {
            switch (value) {
                case "scale":
                    this._reDrawScale(); break;
                case "pointer":
                    this._reDrawPointer(); break;
                case "range":
                    this._reDrawRange(); break;
                case "label":
                    this._reDrawLabel(); break;
                case "tickMark":
                    this._reDrawTickMark();
                    break;
                case "subGauge":
                    this._reDrawSubGauge();
                    break;
                case "CustomLabel":
                    this._reDrawCustomLabel();
                    break;
            }
        },
        /**
         * To get Indicator Image
		 * @private
         */
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                this._getPointerImage();
        },
        /**
         * To get Pointer Image
		 * @private
         */
        _getPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
        /**
         * To get sub gauge Image
		 * @private
         */
        _getSubGaugeImage: function () {
            if (this.rangeImage)
                return this.rangeImage;
            else
                return this._getRangeImage();
        },
        /**
         * To get Custom Label Image
		 * @private
         */
        _getCustomLabelImage: function () {
            if (this.subGaugeImage)
                return this.subGaugeImage;
            else
                return this._getSubGaugeImage();
        },
        /**
         * To get Range Image
		 * @private
         */
        _getRangeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
        /**
         * To get Label Image
		 * @private
         */
        _getLabelImage: function () {
            if (this.tickImage)
                return this.tickImage;
            else
                return this._getTickImage();
        },
        /**
         * To get Tick Image
		 * @private
         */
        _getTickImage: function () {
            if (this.scaleImage)
                return this.scaleImage;
            else
                return this.outerImage;
        },
        /**
        * To get IndicatorValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.indicatorIndex indicatorIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the IndicatorValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getIndicatorValue(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the IndicatorValue
          * $("#CoreCircularGauge").ejCircularGauge("getIndicatorValue(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getIndicatorValue: function (scaleIndex, indicatorIndex) {
            if (scaleIndex < this.model.scales.length && indicatorIndex < this.model.scales[scaleIndex].indicators.length)
                return this.scaleEl[scaleIndex].indicators[indicatorIndex].value;
        },
        /**
          * To set IndicatorValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.indicatorIndex indicatorIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the IndicatorValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setIndicatorValue(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the IndicatorValue 
          * $("#CoreCircularGauge").ejCircularGauge("setIndicatorValue(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setIndicatorValue: function (scaleIndex, indicatorIndex, value) {
            if (scaleIndex < this.model.scales.length && indicatorIndex < this.model.scales[scaleIndex].indicators.length && value != null) {
                this.scaleEl[scaleIndex].indicators[indicatorIndex].value = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        /**
        * To get PointerValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerValue(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerValue
          * $("#CoreCircularGauge").ejCircularGauge("getPointerValue(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].value;
        },
        /**
          * To set MarkerDistanceFromScale 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MarkerDistanceFromScale 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMarkerDistanceFromScale(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MarkerDistanceFromScale 
          * $("#CoreCircularGauge").ejCircularGauge("setMarkerDistanceFromScale(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMarkerDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        /**
        * To get MarkerDistanceFromScale
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MarkerDistanceFromScale
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMarkerDistanceFromScale(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MarkerDistanceFromScale
          * $("#CoreCircularGauge").ejCircularGauge("getMarkerDistanceFromScale(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMarkerDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].distanceFromScale;
        },
        /**
          * To set PointerLength 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerLength 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerLength(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerLength 
          * $("#CoreCircularGauge").ejCircularGauge("setPointerLength(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerLength: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].pointerLength = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        /**
        * To get PointerLength
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerLength
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerLength(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerLength
          * $("#CoreCircularGauge").ejCircularGauge("getPointerLength(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerLength: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].pointerLength;
        },
        /**
          * To set PointerWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerWidth 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerWidth(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerWidth 
          * $("#CoreCircularGauge").ejCircularGauge("setPointerWidth(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerWidth: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].pointerWidth = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        /**
        * To get PointerWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerWidth
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerWidth(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerWidth
          * $("#CoreCircularGauge").ejCircularGauge("getPointerWidth(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].pointerWidth;
        },

        /**
          * To set BackNeedleLength 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the BackNeedleLength 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setBackNeedleLength(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the BackNeedleLength 
          * $("#CoreCircularGauge").ejCircularGauge("setBackNeedleLength(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setBackNeedleLength: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].backNeedleLength = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        /**
        * To get BackNeedleLength
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the BackNeedleLength
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getBackNeedleLength(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the BackNeedleLength
          * $("#CoreCircularGauge").ejCircularGauge("getBackNeedleLength(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getBackNeedleLength: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].backNeedleLength;
        },
        /**
          * To set NeedleStyle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the NeedleStyle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setNeedleStyle(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the NeedleStyle 
          * $("#CoreCircularGauge").ejCircularGauge("setNeedleStyle(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setNeedleStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].needleStyle = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        /**
        * To get NeedleStyle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the NeedleStyle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getNeedleStyle(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the NeedleStyle
          * $("#CoreCircularGauge").ejCircularGauge("getNeedleStyle(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getNeedleStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.model.needleStyle[this.scaleEl[scaleIndex].pointers[pointerIndex].needleStyle];
        },
        /**
          * To set PointerPlacement 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerPlacement 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerPlacement(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerPlacement 
          * $("#CoreCircularGauge").ejCircularGauge("setPointerPlacement(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerPlacement: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].pointerPlacement = $.inArray(value, this.model.scalePlacement);
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        /**
        * To get PointerPlacement
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerPlacement
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerPlacement(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerPlacement
          * $("#CoreCircularGauge").ejCircularGauge("getPointerPlacement(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerPlacement: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.model.scalePlacement[this.scaleEl[scaleIndex].pointers[pointerIndex].pointerPlacement];
        },
        /**
          * To set PointerNeedleType 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerNeedleType 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerNeedleType(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerNeedleType 
          * $("#CoreCircularGauge").ejCircularGauge("setPointerNeedleType(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerNeedleType: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].pointerNeedleType = $.inArray(value, this.model.pointerNeedleType);
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else this._initialize();
            }
        },
        /**
        * To get PointerNeedleType
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerNeedleType
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerNeedleType(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerNeedleType
          * $("#CoreCircularGauge").ejCircularGauge("getPointerNeedleType(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerNeedleType: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.model.pointerNeedleType[this.scaleEl[scaleIndex].pointers[pointerIndex].pointerNeedleType];
        },
        /**
          * To set MarkerStyle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MarkerStyle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMarkerStyle(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MarkerStyle 
          * $("#CoreCircularGauge").ejCircularGauge("setMarkerStyle(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMarkerStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                var Exist = false;
                if (!ej.isNullOrUndefined(ej.markerStyle[value])) {
                    this.scaleEl[scaleIndex].pointers[pointerIndex].markerStyle = value;
                    if (this.contextEl.putImageData && !this.isAndroid())
                        this._reDrawCustomLabel();
                    else this._initialize();
                }
            }
        },
        /**
        * To get MarkerStyle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MarkerStyle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMarkerStyle(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MarkerStyle
          * $("#CoreCircularGauge").ejCircularGauge("getMarkerStyle(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMarkerStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.model.markerStyle[this.scaleEl[scaleIndex].pointers[pointerIndex].markerStyle];
        },
        /**
          * To set CustomLabelValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.customLabelIndex customLabelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the CustomLabelValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setCustomLabelValue(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the CustomLabelValue 
          * $("#CoreCircularGauge").ejCircularGauge("setCustomLabelValue(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setCustomLabelValue: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabel.length && value != null) {
                this.scaleEl[scaleIndex].customLabel[customLabelIndex].labelValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else
                    this._initialize();
            }
        },
        /**
        * To get CustomLabelValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.customLabelIndex customLabelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the CustomLabelValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getCustomLabelValue(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the CustomLabelValue
          * $("#CoreCircularGauge").ejCircularGauge("getCustomLabelValue(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getCustomLabelValue: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabel.length)
                return this.scaleEl[scaleIndex].customLabel[customLabelIndex].labelValue;
        },

        /**
          * To set SubGaugeLocation 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.GaugeIndex GaugeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the SubGaugeLocation 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setSubGaugeLocation(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the SubGaugeLocation 
          * $("#CoreCircularGauge").ejCircularGauge("setSubGaugeLocation(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setSubGaugeLocation: function (scaleIndex, GaugeIndex, value) {
            if (scaleIndex < this.model.scales.length && GaugeIndex < this.model.scales[scaleIndex].subGauge.length && value != null) {
                this.scaleEl[scaleIndex].subGauge[GaugeIndex].location.x = value.X;
                this.scaleEl[scaleIndex].subGauge[GaugeIndex].location.y = value.Y;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawSubGauge();
                else
                    this._initialize();
            }
        },
        /**
        * To get SubGaugeLocation
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.GaugeIndex GaugeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the SubGaugeLocation
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getSubGaugeLocation(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the SubGaugeLocation
          * $("#CoreCircularGauge").ejCircularGauge("getSubGaugeLocation(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getSubGaugeLocation: function (scaleIndex, GaugeIndex) {
            if (scaleIndex < this.model.scales.length && GaugeIndex < this.model.scales[scaleIndex].subGauge.length)
                return this.scaleEl[scaleIndex].subGauge[GaugeIndex].Location;
        },
        /**
          * To set CustomLabelAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.customLabelIndex customLabelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the CustomLabelAngle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setCustomLabelAngle(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the CustomLabelAngle 
          * $("#CoreCircularGauge").ejCircularGauge("setCustomLabelAngle(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setCustomLabelAngle: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabel.length && value != null) {
                this.scaleEl[scaleIndex].customLabel[customLabelIndex].textAngle = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else
                    this._initialize();
            }
        },
        /**
        * To get CustomLabelAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.customLabelIndex customLabelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the CustomLabelAngle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getCustomLabelAngle(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the CustomLabelAngle
          * $("#CoreCircularGauge").ejCircularGauge("getCustomLabelAngle(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getCustomLabelAngle: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabel.length)
                return this.scaleEl[scaleIndex].customLabel[customLabelIndex].textAngle;
        },
        /**
          * To set RangeStartValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeStartValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeStartValue(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeStartValue 
          * $("#CoreCircularGauge").ejCircularGauge("setRangeStartValue(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeStartValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeStartValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeStartValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeStartValue(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeStartValue
          * $("#CoreCircularGauge").ejCircularGauge("getRangeStartValue(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeStartValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startValue;
        },
        /**
          * To set RangeEndValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeEndValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeEndValue(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeEndValue 
          * $("#CoreCircularGauge").ejCircularGauge("setRangeEndValue(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeEndValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeEndValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeEndValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeEndValue(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeEndValue
          * $("#CoreCircularGauge").ejCircularGauge("getRangeEndValue(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeEndValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endValue;
        },
        /**
          * To set RangeSize 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeSize 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeSize(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeSize 
          * $("#CoreCircularGauge").ejCircularGauge("setRangeSize(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeSize: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].size = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeSize
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeSize
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeSize(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeSize
          * $("#CoreCircularGauge").ejCircularGauge("getRangeSize(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeSize: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].size;
        },
        /**
          * To set RangeDistanceFromScale 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeDistanceFromScale 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeDistanceFromScale(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeDistanceFromScale 
          * $("#CoreCircularGauge").ejCircularGauge("setRangeDistanceFromScale(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeDistanceFromScale: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeRangeDistanceFromScale
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeRangeDistanceFromScale
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeRangeDistanceFromScale(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeRangeDistanceFromScale
          * $("#CoreCircularGauge").ejCircularGauge("getRangeRangeDistanceFromScale(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeRangeDistanceFromScale: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale;
        },
        /**
          * To set RangePosition 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangePosition 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangePosition(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangePosition 
          * $("#CoreCircularGauge").ejCircularGauge("setRangePosition(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangePosition: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].RangePosition = $.inArray(value, this.model.scalePlacement);
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangePosition
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangePosition
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangePosition(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangePosition
          * $("#CoreCircularGauge").ejCircularGauge("getRangePosition(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangePosition: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.model.scalePlacement[this.scaleEl[scaleIndex].ranges[rangeIndex].RangePosition];
        },
        /**
          * To set RangeBorderWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeBorderWidth 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeBorderWidth(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the RangeBorderWidth 
          * $("#CoreCircularGauge").ejCircularGauge("setRangeBorderWidth(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeBorderWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].borderWidth = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeBorderWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeBorderWidth
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeBorderWidth(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the RangeBorderWidth
          * $("#CoreCircularGauge").ejCircularGauge("getRangeBorderWidth(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeBorderWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].borderWidth;
        },
        /**
          * To set PointerValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerValue(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerValue 
          * $("#CoreCircularGauge").ejCircularGauge("setPointerValue(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].value = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        /**
          * To set LabelAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.angle angle value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the LabelAngle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setLabelAngle(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the LabelAngle 
          * $("#CoreCircularGauge").ejCircularGauge("setLabelAngle(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setLabelAngle: function (scaleIndex, labelIndex, angle) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && angle != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].angle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
        /**
        * To get LabelAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the LabelAngle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getLabelAngle(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the LabelAngle
          * $("#CoreCircularGauge").ejCircularGauge("getLabelAngle(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getLabelAngle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].angle;
        },
        /**
          * To set LabelDistanceFromScale 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the LabelDistanceFromScale 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setLabelDistanceFromScale(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the LabelDistanceFromScale 
          * $("#CoreCircularGauge").ejCircularGauge("setLabelDistanceFromScale(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setLabelDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
        /**
        * To get LabelDistanceFromScale
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the LabelDistanceFromScale
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getLabelDistanceFromScale(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the LabelDistanceFromScale
          * $("#CoreCircularGauge").ejCircularGauge("getLabelDistanceFromScale(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getLabelDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale;
        },
        /**
          * To set LabelStyle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the LabelStyle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setLabelStyle(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the LabelStyle 
          * $("#CoreCircularGauge").ejCircularGauge("setLabelStyle(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setLabelStyle: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].labelStyle = $.inArray(value, this.model.interval);
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
        /**
        * To get LabelStyle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the LabelStyle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getLabelStyle(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the LabelStyle
          * $("#CoreCircularGauge").ejCircularGauge("getLabelStyle(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getLabelStyle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.model.interval[this.scaleEl[scaleIndex].labels[labelIndex].labelStyle];
        },
        /**
          * To set LabelPlacement 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the LabelPlacement 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setLabelPlacement(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the LabelPlacement 
          * $("#CoreCircularGauge").ejCircularGauge("setLabelPlacement(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setLabelPlacement: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].LabelPlacement = $.inArray(value, this.model.scalePlacement);
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
        /**
        * To get LabelPlacement
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the LabelPlacement
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getLabelPlacement(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the LabelPlacement
          * $("#CoreCircularGauge").ejCircularGauge("getLabelPlacement(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getLabelPlacement: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.model.scalePlacement[this.scaleEl[scaleIndex].labels[labelIndex].LabelPlacement];
        },
        /**
          * To set TickAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickAngle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickAngle(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickAngle 
          * $("#CoreCircularGauge").ejCircularGauge("setTickAngle(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickAngle: function (scaleIndex, tickIndex, angle) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && angle != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].angle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else
                    this._initialize();
            }
        },
        /**
        * To get TickAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickAngle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickAngle(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickAngle
          * $("#CoreCircularGauge").ejCircularGauge("getTickAngle(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickAngle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length) {
                return this.scaleEl[scaleIndex].ticks[tickIndex].angle;
            }
        },
        /**
          * To set TickStyle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickStyle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickStyle(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickStyle 
          * $("#CoreCircularGauge").ejCircularGauge("setTickStyle(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickStyle: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].tickStyle = $.inArray(value, this.model.interval);
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickStyle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickStyle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickStyle(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickStyle
          * $("#CoreCircularGauge").ejCircularGauge("getTickStyle(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickStyle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.model.interval[this.scaleEl[scaleIndex].ticks[tickIndex].tickStyle];
        },
        /**
          * To set TickPlacement 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickPlacement 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickPlacement(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickPlacement 
          * $("#CoreCircularGauge").ejCircularGauge("setTickPlacement(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickPlacement: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].tickPlacement = $.inArray(value, this.model.scalePlacement);
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickPlacement
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickPlacement
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickPlacement(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickPlacement
          * $("#CoreCircularGauge").ejCircularGauge("getTickPlacement(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickPlacement: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.model.scalePlacement[this.scaleEl[scaleIndex].ticks[tickIndex].tickPlacement];
        },
        /**
          * To set TickWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickWidth 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickWidth(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickWidth 
          * $("#CoreCircularGauge").ejCircularGauge("setTickWidth(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickWidth: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].tickWidth = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickWidth
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickWidth(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickWidth
          * $("#CoreCircularGauge").ejCircularGauge("getTickWidth(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickWidth: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].tickWidth;
        },
        /**
          * To set TickHeight 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickHeight 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickHeight(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickHeight 
          * $("#CoreCircularGauge").ejCircularGauge("setTickHeight(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickHeight: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].tickHeight = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickHeight
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickHeight
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickHeight(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickHeight
          * $("#CoreCircularGauge").ejCircularGauge("getTickHeight(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickHeight: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].tickHeight;
        },
        /**
          * To set TickDistanceFromScale 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickDistanceFromScale 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickDistanceFromScale(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the TickDistanceFromScale 
          * $("#CoreCircularGauge").ejCircularGauge("setTickDistanceFromScale(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickDistanceFromScale: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickDistanceFromScale
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickDistanceFromScale
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickDistanceFromScale(0, 0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the TickDistanceFromScale
          * $("#CoreCircularGauge").ejCircularGauge("getTickDistanceFromScale(0, 0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickDistanceFromScale: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale;
        },
        /**
          * To set StartAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the StartAngle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setStartAngle(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the StartAngle 
          * $("#CoreCircularGauge").ejCircularGauge("setStartAngle(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setStartAngle: function (scaleIndex, angle) {
            if (scaleIndex < this.model.scales.length && angle != null) {
                this.scaleEl[scaleIndex].startAngle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get StartAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the StartAngle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getStartAngle(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the StartAngle
          * $("#CoreCircularGauge").ejCircularGauge("getStartAngle(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getStartAngle: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length) {
                return this.scaleEl[scaleIndex].startAngle;
            }
        },
        /**
          * To set SweepAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the SweepAngle 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setSweepAngle(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the SweepAngle 
          * $("#CoreCircularGauge").ejCircularGauge("setSweepAngle(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setSweepAngle: function (scaleIndex, angle) {
            if (scaleIndex < this.model.scales.length && angle != null) {
                this.scaleEl[scaleIndex].sweepAngle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get SweepAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the SweepAngle
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getSweepAngle(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the SweepAngle
          * $("#CoreCircularGauge").ejCircularGauge("getSweepAngle(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getSweepAngle: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length) {
                return this.scaleEl[scaleIndex].sweepAngle;
            }
        },
        /**
          * To set MinimumValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MinimumValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMinimumValue(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MinimumValue 
          * $("#CoreCircularGauge").ejCircularGauge("setMinimumValue(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMinimumValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minimum = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get MinimumValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MinimumValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMinimumValue(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MinimumValue
          * $("#CoreCircularGauge").ejCircularGauge("getMinimumValue(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMinimumValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].minimum;
        },
        /**
          * To set MaximumValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MaximumValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMaximumValue(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MaximumValue 
          * $("#CoreCircularGauge").ejCircularGauge("setMaximumValue(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMaximumValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].maximum = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get MaximumValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MaximumValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMaximumValue(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MaximumValue
          * $("#CoreCircularGauge").ejCircularGauge("getMaximumValue(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMaximumValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].maximum;
        },
        /**
          * To set ScaleBarSize 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the ScaleBarSize 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setScaleBarSize(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the ScaleBarSize 
          * $("#CoreCircularGauge").ejCircularGauge("setScaleBarSize(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setScaleBarSize: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].scaleBarSize = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get ScaleBarSize
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the ScaleBarSize
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getScaleBarSize(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the ScaleBarSize
          * $("#CoreCircularGauge").ejCircularGauge("getScaleBarSize(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getScaleBarSize: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].scaleBarSize;
        },
        /**
          * To set ScaleRadius 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the ScaleRadius 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setScaleRadius(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the ScaleRadius 
          * $("#CoreCircularGauge").ejCircularGauge("setScaleRadius(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setScaleRadius: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].scaleRadius = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get ScaleRadius
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the ScaleRadius
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getScaleRadius(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the ScaleRadius
          * $("#CoreCircularGauge").ejCircularGauge("getScaleRadius(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getScaleRadius: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].scaleRadius;
        },
        /**
          * To set MajorIntervalValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MajorIntervalValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMajorIntervalValue(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MajorIntervalValue 
          * $("#CoreCircularGauge").ejCircularGauge("setMajorIntervalValue(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMajorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].majorIntervalValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get MajorIntervalValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MajorIntervalValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMajorIntervalValue(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MajorIntervalValue
          * $("#CoreCircularGauge").ejCircularGauge("getMajorIntervalValue(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMajorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].majorIntervalValue;
        },
        /**
          * To set MinorIntervalValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MinorIntervalValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMinorIntervalValue(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the MinorIntervalValue 
          * $("#CoreCircularGauge").ejCircularGauge("setMinorIntervalValue(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMinorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minorIntervalValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get MinorIntervalValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MinorIntervalValue
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMinorIntervalValue(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the MinorIntervalValue
          * $("#CoreCircularGauge").ejCircularGauge("getMinorIntervalValue(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMinorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].minorIntervalValue;
        },
        /**
          * To set PointerCapRadius 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerCapRadius 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerCapRadius(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerCapRadius 
          * $("#CoreCircularGauge").ejCircularGauge("setPointerCapRadius(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerCapRadius: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].pointerCapRadius = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get PointerCapRadius
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerCapRadius
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerCapRadius(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerCapRadius
          * $("#CoreCircularGauge").ejCircularGauge("getPointerCapRadius(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerCapRadius: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].pointerCapRadius;
        },
        /**
          * To set ScaleBorderWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the ScaleBorderWidth 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setScaleBorderWidth(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the ScaleBorderWidth 
          * $("#CoreCircularGauge").ejCircularGauge("setScaleBorderWidth(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setScaleBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].scaleBorderWidth = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get ScaleBorderWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the ScaleBorderWidth
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getScaleBorderWidth(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the ScaleBorderWidth
          * $("#CoreCircularGauge").ejCircularGauge("getScaleBorderWidth(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getScaleBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].scaleBorderWidth;
        },
        /**
          * To set PointerCapBorderWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerCapBorderWidth 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerCapBorderWidth(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the PointerCapBorderWidth 
          * $("#CoreCircularGauge").ejCircularGauge("setPointerCapBorderWidth(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerCapBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].capBorderWidth = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get PointerCapBorderWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerCapBorderWidth
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerCapBorderWidth(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the PointerCapBorderWidth
          * $("#CoreCircularGauge").ejCircularGauge("getPointerCapBorderWidth(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerCapBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].capBorderWidth;
        },
        /**
          * To set ScaleDirection 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the ScaleDirection 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setScaleDirection(1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the ScaleDirection 
          * $("#CoreCircularGauge").ejCircularGauge("setScaleDirection(1, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setScaleDirection: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].scaleDirection = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get ScaleDirection
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the ScaleDirection
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getScaleDirection(0);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //get the ScaleDirection
          * $("#CoreCircularGauge").ejCircularGauge("getScaleDirection(0)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getScaleDirection: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.model.scaleDirection[this.scaleEl[scaleIndex].scaleDirection];
        },
        /**
          * To set includeFirstValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the includeFirstValue 
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.includeFirstValue(1, 1, 10);
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //set the includeFirstValue 
          * $("#CoreCircularGauge").ejCircularGauge("includeFirstValue(1, 8, 20)"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        includeFirstValue: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].includeFirstValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawLabel();
                else this.initialize();
            }
        },
        /**
         * To Redraw scale
		 * @private
         */
        _reDrawScale: function () {
            if (this.model.frameBackgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this.outerImage, 0, 0);
                this._drawScales();

            }
        },
        /**
         * To Redraw Pointer
		 * @private
         */
        _reDrawPointer: function () {
            if (this.model.frameBackgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getPointerImage(), 0, 0);
                this._setPointers();
                this._setIndicators();
            }
        },
        /**
         * To Redraw Custom Label
		 * @private
         */
        _reDrawCustomLabel: function () {
            if (this.model.frameBackgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
        /**
         * To Redraw Range
		 * @private
         */
        _reDrawRange: function () {
            if (this.model.frameBackgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getRangeImage(), 0, 0);
                this._setRanges();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
        /**
         * To Redraw Label
		 * @private
         */
        _reDrawLabel: function () {
            if (this.model.frameBackgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getLabelImage(), 0, 0);
                this._setLabels();
                this._setRanges();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
        /**
         * To Redraw TickMark
		 * @private
         */
        _reDrawTickMark: function () {
            if (this.model.frameBackgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getTickImage(), 0, 0);
                this._setTicks();
                this._setLabels();
                this._setRanges();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
        /**
         * To Redraw SubGauge
		 * @private
         */
        _reDrawSubGauge: function () {
            if (this.model.frameBackgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getSubGaugeImage(), 0, 0);
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
        /**
         * To refresh SubGauge
		 * @private
         */
        refreshSubGauge: function () {
            if (this.contextEl.putImageData)
                this._reDrawSubGauge();
            else
                this._initialize();
        },
        /**
         * To refresh the Entire gauge
		 * @private
         */
        refresh: function () {
            this._initialize();
        },
        /**
        * To export Image
          * @return jQuery
          * @param {string} argument.fileName fileName for the Image
          * @param {string} argument.fileType fileType for the Image
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //To Export Image
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.exportImage("myImage","jpeg");
          * &lt;/script&gt;
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;CircularGauge&lt;/div&gt; <br> 
          * &lt;script&gt;
          * //To Export Image
          * $("#CoreCircularGauge").ejCircularGauge("exportImage('myImage','png')"); 
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            var lnk = document.createElement('a'), e;
            lnk.download = fileName + "." + fileType;
            lnk.href = this.canvasEl[0].toDataURL("image/" + fileType).replace("image/" + fileType, "image/octet-stream");
            if (document.createEvent) {
                e = document.createEvent("MouseEvents");
                e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                lnk.dispatchEvent(e);
            }
            else if (lnk.fireEvent) {
                lnk.fireEvent("onclick");
            }
        },
        /**
         * To Resize the gauge while resizing the window
		 * @private
         */
        resizeCanvas: function () {
            if (proxy.element.width() != initialDivWidth || proxy.element.height() != initialDivHeight) {
                var canvasWidth = proxy.element.width() / initialDivWidth;
                var canvasHeight = proxy.element.height() / initialDivHeight;
                var ratio;
                if (canvasWidth != 1 && canvasHeight != 1)
                    ratio = canvasWidth;
                else
                    ratio = canvasHeight != 1 ? canvasHeight : canvasWidth;
                proxy.model.width *= ratio;
                for (var i = 0; proxy.model.scales[i] != null; i++) {
                    proxy.model.scales[i].scaleRadius *= ratio;
                    for (var j = 0; proxy.model.scales[i].pointers[j] != null; j++) {
                        proxy.model.scales[i].pointers[j].pointerLength *= ratio;
                        proxy.model.scales[i].pointers[j].pointerCapRadius *= ratio;
                        proxy.model.scales[i].pointers[j].pointerWidth *= ratio;
                        proxy.model.scales[i].pointers[j].backNeedleLength *= ratio;
                    }
                }
                proxy.refresh();
                initialDivWidth = proxy.element.width();
                initialDivHeight = proxy.element.height();
            }
        },

        // ********************************* ClientSide Events ************************************
        /**
         * Section For handle the DrawTicks event
		 * @private
         */
        _onDrawTicks: function (tickAngle, pointerValue) {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, tickIndex: this.tickIndex, tickElement: this.tickEl[this.tickIndex], context: this.contextEl, tickValue: pointerValue, tickAngle: parseInt(tickAngle), style: this.style, location: this.location };
            this._trigger("drawTicks", data);
        },
        /**
         * Section For handle the DrawLabels event
		 * @private
         */
        _onDrawLabels: function (labelAngle, pointerValue) {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, labelIndex: this.labelIndex, labelElement: this.labelEl[this.labelIndex], context: this.contextEl, labelValue: pointerValue, labelAngle: parseInt(labelAngle), style: this.style, location: this.location };
            this._trigger("drawLabels", data);
        },
        /**
         * Section For handle the DrawPointers event
		 * @private
         */
        _onDrawPointers: function (pointerAngle, pointerValue) {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, pointerIndex: this.pointerIndex, pointerElement: this.pointerEl[this.pointerIndex], context: this.contextEl, pointerValue: pointerValue, pointerAngle: parseInt(pointerAngle), style: this.style, location: this.location };
            this._trigger("drawPointers", data);
        },
        /**
         * Section For handle the DrawRange event
		 * @private
         */
        _onDrawRange: function () {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, rangeElement: this.rangeEl[this.rangeEl], context: this.contextEl, style: this.style, location: this.location };
            this._trigger("drawRange", data);
        },
        /**
         * Section For handle the DrawCustomLabel event
		 * @private
         */
        _onDrawCustomLabel: function () {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, customLabelIndex: this.customLabelIndex, customLabelElement: this.customLabelEl[this.customLabelIndex], context: this.contextEl, style: this.style, location: this.location };
            this._trigger("drawCustomLabel", data);
        },
        /**
         * Section For handle the DrawIndicators event
		 * @private
         */
        _onDrawIndicators: function () {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, indicatorIndex: this.indicatorIndex, indicatorEl: this.indicatorEl[this.indicatorIndex], context: this.contextEl, style: this.style, location: this.location };
            this._trigger("drawIndicators", data);
        },
        /**
         * Section For handle the DrawPointerCap event
		 * @private
         */
        _onDrawPointerCap: function () {
            var data = { Object: this, scaleElement: this.model.scales, location: this.location, style: this.style, context: this.contextEl };
            this._trigger("drawPointerCap", data);
        },
        /**
         * Section For handle the RenderComplete event
		 * @private
         */
        _onRenderComplete: function () {
            var data = { Object: this, scaleElement: this.model.scales, context: this.contextEl };
            this._trigger("renderComplete", data);
        },
        /**
         * Section For handle the MouseClick event
		 * @private
         */
        _onMouseClick: function (pointerAngle, pointerValue) {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, pointerIndex: this.pointerIndex, pointerElement: this.pointerEl[this.pointerIndex], context: this.contextEl, pointerValue: pointerValue, pointerAngle: parseInt(pointerAngle), style: this.style, location: this.location };
            this._trigger("mouseClick", data);
        },
        /**
         * Section For handle the MouseClickMove event
		 * @private
         */
        _onMouseClickMove: function (pointerAngle, pointerValue) {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, pointerIndex: this.pointerIndex, pointerElement: this.pointerEl[this.pointerIndex], context: this.contextEl, pointerValue: pointerValue, pointerAngle: parseInt(pointerAngle), style: this.style, location: this.location };
            this._trigger("mouseClickMove", data);
        },
        /**
         * Section For handle the MouseClickUp event
		 * @private
         */
        _onMouseClickUp: function (pointerAngle, pointerValue) {
            var data = { Object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, pointerIndex: this.pointerIndex, pointerElement: this.pointerEl[this.pointerIndex], context: this.contextEl, pointerValue: pointerValue, pointerAngle: parseInt(pointerAngle), style: this.style, location: this.location };
            this._trigger("mouseClickUp", data);
        },

        //************************* Common Functions **********************************
        /**
         * To set the theme
		 * @private
         */
        _setTheme: function () {
            selectedTheme = ej.CircularGauge.themes[this.model.theme];
            this._setThemeColors(selectedTheme);
        },
        /**
         * To set the theme color
		 * @private
         */
        _setThemeColors: function (selectedTheme) {
            var result = [], jsonObj = ej.CircularGauge.themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                this.model.backgroundColor = ((!this.model.backgroundColor || this.model.backgroundColor == jsonObj[result[th]].backgroundColor) ? selectedTheme.backgroundColor : this.model.backgroundColor);
                for (var i = 0; i < this.model.scales.length; i++) {
                    this.model.scales[i].backgroundColor = (!this.model.scales[i].backgroundColor || this.model.scales[i].backgroundColor == jsonObj[result[th]].scales.backgroundColor) ? selectedTheme.scales.backgroundColor : this.model.scales[i].backgroundColor;
                    this.model.scales[i].borderColor = (!this.model.scales[i].borderColor || this.model.scales[i].borderColor == jsonObj[result[th]].scales.borderColor) ? selectedTheme.scales.borderColor : this.model.scales[i].borderColor;
                    for (var j = 0; j < this.model.scales[i].pointers.length; j++) {
                        this.model.scales[i].pointers[j].backgroundColor = (!this.model.scales[i].pointers[j].backgroundColor || this.model.scales[i].pointers[j].backgroundColor == jsonObj[result[th]].scales.pointers.backgroundColor) ? selectedTheme.scales.pointers.backgroundColor : this.model.scales[i].pointers[j].backgroundColor;
                        this.model.scales[i].pointers[j].borderColor = (!this.model.scales[i].pointers[j].borderColor || this.model.scales[i].pointers[j].borderColor == jsonObj[result[th]].scales.pointers.borderColor) ? selectedTheme.scales.pointers.borderColor : this.model.scales[i].pointers[j].borderColor;
                        this.model.scales[i].capBackgroundColor = (!this.model.scales[i].capBackgroundColor || this.model.scales[i].capBackgroundColor == jsonObj[result[th]].scales.capBackgroundColor) ? selectedTheme.scales.capBackgroundColor : this.model.scales[i].capBackgroundColor;
                        this.model.scales[i].capBorderColor = (!this.model.scales[i].capBorderColor || this.model.scales[i].capBorderColor == jsonObj[result[th]].scales.capBorderColor) ? selectedTheme.scales.capBorderColor : this.model.scales[i].capBorderColor;
                    }
                    for (var k = 0; k < this.model.scales[i].ticks.length; k++) {
                        this.model.scales[i].ticks[k].tickColor = (!this.model.scales[i].ticks[k].tickColor || this.model.scales[i].ticks[k].tickColor == jsonObj[result[th]].scales.ticks.tickColor) ? selectedTheme.scales.ticks.tickColor : this.model.scales[i].ticks[k].tickColor;
                    }
                    for (var l = 0; l < this.model.scales[i].labels.length; l++) {
                        this.model.scales[i].labels[l].labelColor = (!this.model.scales[i].labels[l].labelColor || this.model.scales[i].labels[l].labelColor == jsonObj[result[th]].scales.labels.labelColor) ? selectedTheme.scales.labels.labelColor : this.model.scales[i].labels[l].labelColor;
                    }
                }
            }
        },
        /**
         * To get the FontString
		 * @private
         */
        _getFontString: function (element, font) {
            return (font ? (font.fontStyle ? font.fontStyle : "") + " " + ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily : "");
        },
        /**
         * To draw Triangle
		 * @private
         */
        _drawTriangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Pointer
		 * @private
         */
        _drawPointer: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(style.width / 2, style.height / 4);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Wedge
		 * @private
         */
        _drawWedge: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(3 * style.width / 4, 0);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Slider
		 * @private
         */
        _drawSlider: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 4, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(style.width / 4, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Star
		 * @private
         */
        _drawStar: function (location, style, element) {
            this._contextOpenPath(style, element);
            if (element.model.Orientation == "Horizontal" && element.markerPlacement == "Near") {
                element.contextEl.moveTo(location.startX + style.width - (style.width / 6), location.startY); // 1
                element.contextEl.lineTo(location.startX, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width / 6, location.startY);
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY + style.height);
            }
            else {
                element.contextEl.moveTo(location.startX + (style.width / 6), location.startY + style.height); // 1
                element.contextEl.lineTo(location.startX + style.width, location.startY + (style.height / 3)); // 2
                element.contextEl.lineTo(location.startX, location.startY + (style.height / 3)); // 3
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY + style.height); // 4
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY); // 5
            }
            this._contextClosePath(style, element);
        },
        /**
         * To draw Pentagon
		 * @private
         */
        _drawPentagon: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 3, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width / 3, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Diamond
		 * @private
         */
        _drawDiamond: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(style.width, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(0, 0);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Circle
		 * @private
         */
        _drawCircle: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Trapezoid
		 * @private
         */
        _drawTrapezoid: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 4);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Rectangle
		 * @private
         */
        _drawRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Rounded Rectangle
		 * @private
         */
        _drawRoundedRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY - style.height / 2);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.radius, 0);
            element.contextEl.lineTo(style.width - style.radius, 0);
            element.contextEl.quadraticCurveTo(style.width, 0, style.width, style.radius);
            element.contextEl.lineTo(style.width, style.height - style.radius);
            element.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.radius, style.height);
            element.contextEl.lineTo(style.radius, style.height);
            element.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.radius);
            element.contextEl.lineTo(0, style.radius);
            element.contextEl.quadraticCurveTo(0, 0, style.radius, 0);
            this._contextClosePath(style, element);
        },
        /**
         * To draw Custom Image
		 * @private
         */
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            var self = this;
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.model.Width, element.model.Height);
                if (element.model.scales != null)
                    element._drawScales();
                if (element.model.items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
        /**
         * To draw Ellipse
		 * @private
         */
        _drawEllipse: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.scale(2, 1);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
        /**
         * To set Pointer Dimension
		 * @private
         */
        _setPointerDimension: function (style, element) {
            if (!ej.isNullOrUndefined(element.model.Orientation)) {
                if (element.model.Orientation == "Horizontal") {
                    var tempWidth = style.width;
                    var tempHeight = style.height;
                    style.height = tempWidth;
                    style.width = tempHeight;
                }
            }
            return style;
        },
        /**
         * To set Context Rotation
		 * @private
         */
        _setContextRotation: function (style, element) {
            element.contextEl.rotate(Math.PI * (style.angle / 180));
        },
        /**
         * To Manupulate the Context open path.
		 * @private
         */
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },
        /**
         * To Manupulate the Context Close path.
		 * @private
         */
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },
        /**
         * To block default actions.
		 * @private
         */
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
        /**
         * To check whether the given value is between the first and last value
		 * @private
         */
        _isBetween: function (first, last, number) {
            return (first < last ? number >= first && number <= last : number >= last && number <= first);
        },
        /**
         * To get the color
		 * @private
         */
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.r + ", " + option.g + "," + option.b + ", " + option.a / 255 + ")");
            }
        },
        /**
         * To set the GradientColor
		 * @private
         */
        _setGradientColor: function (element, gradient, options) {
            var self = element;
            if (options.Name || typeof (options) === "string") {
                gradient.addColorStop(0, this._getColor(element, options));
                gradient.addColorStop(1, this._getColor(element, options));
            }
            else
                $.each(options, function (index, colorElement) {
                    gradient.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : self._getColor(element, colorElement.color));
                });
        }
    });
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.frame = {
        /**  To set the CircularGauge frame as fullCircle.*/
        FullCircle: "fullCircle",
        /**  To set the CircularGauge frame as halfCircle.*/
        HalfCircle: "halfCircle"
    };
    /**
	 * Enum for gauge scaleDirection	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.scaleDirection = {
        /**  To set the scaleDirection Clockwise. */
        Clockwise: "Clockwise",
        /**  To set the scaleDirection CounterClockwise. */
        CounterClockwise: "CounterClockwise"
    };
    /**
	 * Enum for gauge pointerPosition	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.pointerPosition = {
        /**  To set the pointerPosition Near. */
        Near: "Near",
        /**  To set the pointerPosition Far. */
        Far: "Far",
        /**  To set the pointerPosition Center. */
        Center: "Center"
    };
    /**
	 * Enum for gauge pointerType	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.pointerType = {
        /**  To set the pointerType Needle. */
        Needle: "Needle",
        /**  To set the pointerType Marker. */
        Marker: "Marker"
    };
    /**
	 * Enum for gauge pointerType	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.needleStyle = {
        /**  To set the needleStyle Triangle. */
        Triangle: "Triangle",
        /**  To set the needleStyle Rectangle. */
        Rectangle: "Rectangle",
        /**  To set the needleStyle Trapezoid. */
        Trapezoid: "Trapezoid",
        /**  To set the needleStyle Arrow. */
        Arrow: "Arrow"
    };
    /**
	 * Enum for gauge markerStyle	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.markerStyle = {
        /**  To set the markerStyle Rectangle. */
        Rectangle: "Rectangle",
        /**  To set the markerStyle Triangle. */
        Triangle: "Triangle",
        /**  To set the markerStyle Ellipse. */
        Ellipse: "Ellipse",
        /**  To set the markerStyle Diamond. */
        Diamond: "Diamond",
        /**  To set the markerStyle Pentagon. */
        Pentagon: "Pentagon",
        /**  To set the markerStyle Circle. */
        Circle: "Circle",
        /**  To set the markerStyle Slider. */
        Slider: "Slider",
        /**  To set the markerStyle Pointer. */
        Pointer: "Pointer",
        /**  To set the markerStyle Wedge. */
        Wedge: "Wedge",
        /**  To set the markerStyle Trapezoid. */
        Trapezoid: "Trapezoid",
        /**  To set the markerStyle RoundedRectangle. */
        RoundedRectangle: "RoundedRectangle"
    };
    /**
	 * Enum for gauge rangePosition	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.rangePosition = {
        /**  To set the rangePosition Near. */
        Near: "Near",
        /**  To set the rangePosition Far. */
        Far: "Far",
        /**  To set the rangePosition Center. */
        Center: "Center"
    };
    /**
	 * Enum for gauge tickStyle	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.tickStyle = {
        /**  To set the tickStyle Major. */
        Major: "Major",
        /**  To set the tickStyle Minor. */
        Minor: "Minor"
    };
    /**
	 * Enum for gauge tickPosition	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.tickPosition = {
        /**  To set the tickPosition Near. */
        Near: "Near",
        /**  To set the tickPosition Far. */
        Far: "Far",
        /**  To set the tickPosition Center. */
        Center: "Center"
    };
    /**
	 * Enum for gauge labelPosition	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.labelPosition = {
        /**  To set the labelPosition Near. */
        Near: "Near",
        /**  To set the labelPosition Far. */
        Far: "Far",
        /**  To set the labelPosition Center. */
        Center: "Center"
    };
    /**
	 * Enum for gauge labelStyle	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.labelStyle = {
        /**  To set the labelStyle Major. */
        Major: "Major",
        /**  To set the labelStyle Minor. */
        Minor: "Minor"
    };
    /**
	 * Enum for gauge unitTextPosition	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.unitTextPosition = {
        /**  To set the unitTextPosition Back. */
        Back: "Back",
        /**  To set the unitTextPosition Front. */
        Front: "Front"
    };
    /**
	 * Enum for gauge indicatorStyle	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.indicatorStyle = {
        /**  To set the indicatorStyle Rectangle. */
        Rectangle: "Rectangle",
        /**  To set the indicatorStyle Circle. */
        Circle: "Circle",
        /**  To set the indicatorStyle RoundedRectangle. */
        RoundedRectangle: "RoundedRectangle",
        /**  To set the indicatorStyle Text. */
        Text: "Text"
    };
    /**
	 * Enum for gauge themes	 
	 * @enum {string}
	 * @global 
	 */
    ej.CircularGauge.themes = {
        /**  To set the themes flatlight. */
        flatlight: {
            backgroundColor: "#FFFFFF",
            scales: {
                backgroundColor: "#777777",
                borderColor: "#777777",
                capBackgroundColor: "#424242",
                capBorderColor: "#424242",
                pointers: {
                    backgroundColor: "#424242",
                    borderColor: "#424242"

                },
                ticks: {
                    tickColor: "#777777"
                },
                labels: {
                    labelColor: "#282828"
                }
            }
        },
        /**  To set the themes flatdark. */
        flatdark: {
            backgroundColor: "#1F1F1F",
            scales: {
                backgroundColor: "#7a7a7a",
                borderColor: "#7a7a7a",
                capBackgroundColor: "#686868",
                capBorderColor: "#686868",
                pointers: {
                    backgroundColor: "#686868",
                    borderColor: "#686868"

                },
                ticks: {
                    tickColor: "#7a7a7a"
                },
                labels: {
                    labelColor: "#d3d3d3"
                }
            }
        }
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejLinearGauge
	* @requires jQuery
	* @requires ej.common.all
	* @classdesc Custom Design for Linear Gauge
	* @example 
	* &lt;div id="LinearGauge1"&gt;div&lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create Linear Gauge
    * $('#LinearGauge1').ejLinearGauge(); 	
	* &lt;/script&gt;
	*/
    // ejLinearGauge is the plugin name 
    // "ej.LinearGauge" is "namespace.className" will hold functions and properties

    ej.widget({ "ejLinearGauge": "ej.LinearGauge" /** @lends ejLinearGauge# */ }, {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],

        // default model
        defaults:/** @lends ejLinearGauge# */{
            /**		
			* Specifies the value of the Gauge.
			* @default 0
            * @alias ejLinearGauge#value
			* @type {number}
			* @example 
			* //To set value API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({  size: 5.5});					 
			* @example 
			* //Get or set the value API, after initialization:<br>
			*	//Gets the value  
			*	$("#LinearGauge1").ejLinearGauge("option", "value");<br>			
			*	//Sets the value 
			*	$("#LinearGauge1").ejLinearGauge("option", "value",  5.5 ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            value: 0,
            /**		
			* Specifies the minimum value of Linear gauge.
			* @default 0
            * @alias ejLinearGauge#minimum
			* @type {number}
			* @example 
			* //To set minimum API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ minimum: 0 });   
			* @example 
			* //Get or set the minimum API, after initialization:<br>
			*	//Gets the minimum value  
			*	$("#LinearGauge1").ejLinearGauge("option", "minimum");<br>			
			*	//Sets the minimum value 
			*	$("#LinearGauge1").ejLinearGauge("option", "minimum",  0 ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            minimum: 0,
            /**		
			* Specifies the maximum value of Linear gauge.
			* @default 100
            * @alias ejLinearGauge#maximum
			* @type {number}
			* @example 
			* //To set maximum API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ maximum: 100 });   
			* @example 
			* //Get or set the maximum API, after initialization:<br>
			*	//Gets the maximum value  
			*	$("#LinearGauge1").ejLinearGauge("option", "maximum");<br>			
			*	//Sets the maximum value 
			*	$("#LinearGauge1").ejLinearGauge("option", "maximum",  100 ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            maximum: 100,
            /**		
			* Specifies the width of Linear gauge.
			* @default 360
            * @alias ejLinearGauge#width
			* @type {number}
			* @example 
			* //To set width API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ width: 360 });   
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width value  
			*	$("#LinearGauge1").ejLinearGauge("option", "width");<br>			
			*	//Sets the width value 
			*	$("#LinearGauge1").ejLinearGauge("option", "width",  360 ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            width: 150,
            /**		
			* Specifies the height of Linear gauge.
			* @default 360
            * @alias ejLinearGauge#height
			* @type {number}
			* @example 
			* //To set height API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ height: 360 });   
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height value  
			*	$("#LinearGauge1").ejLinearGauge("option", "height");<br>			
			*	//Sets the height value 
			*	$("#LinearGauge1").ejLinearGauge("option", "height",  360 ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            height: 400,
            /**		
			* Specifies the theme for Linear gauge. See {@link themes}
			* @default ej.themes.flatlight
            * @alias ejLinearGauge#theme
			* @type {enum}
			* @example 
			* //To set theme API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ theme: ej.theme.flatdark });  
			* @example 
			* //Get or set the theme API, after initialization:<br>
			*	//Gets the theme value  
			*	$("#LinearGauge1").ejLinearGauge("option", "theme");<br>			
			*	//Sets the theme value 
			*	$("#LinearGauge1").ejLinearGauge("option", "theme", ej.theme.flatdark  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            theme: "flatlight",
            /**		
			* Specifies the orientation for Linear gauge.
			* @default "Vertical"
            * @alias ejLinearGauge#orientation
			* @type {string}
			* @example 
			* //To set orientation API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ theme: "vertical" });   
			* @example 
			* //Get or set the orientation API, after initialization:<br>
			*	//Gets the orientation value  
			*	$("#LinearGauge1").ejLinearGauge("option", "orientation");<br>			
			*	//Sets the orientation value 
			*	$("#LinearGauge1").ejLinearGauge("option", "orientation", "vertical"  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            orientation: "Vertical",
            /**		
			* Specifies the pointerGradient1 for Linear gauge.
			* @default null
            * @alias ejLinearGauge#pointerGradient1
			* @type {string}
			* @example 
			* //To set pointerGradient1 API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ pointerGradient1: "Blue" });   
			* @example 
			* //Get or set the pointerGradient1 API, after initialization:<br>
			*	//Gets the pointerGradient1 value  
			*	$("#LinearGauge1").ejLinearGauge("option", "pointerGradient1");<br>			
			*	//Sets the pointerGradient1 value 
			*	$("#LinearGauge1").ejLinearGauge("option", "pointerGradient1", "Blue"  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            pointerGradient1: null,
            /**		
			* Specifies the pointerGradient2 for Linear gauge.
			* @default null
            * @alias ejLinearGauge#pointerGradient2
			* @type {string}
			* @example 
			* //To set pointerGradient2 API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ pointerGradient2: "Blue" });   
			* @example 
			* //Get or set the pointerGradient2 API, after initialization:<br>
			*	//Gets the pointerGradient2 value  
			*	$("#LinearGauge1").ejLinearGauge("option", "pointerGradient2");<br>			
			*	//Sets the pointerGradient2 value 
			*	$("#LinearGauge1").ejLinearGauge("option", "pointerGradient2", "Blue"  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            pointerGradient2: null,
            /**		
			* Specifies the backgroundColor for Linear gauge.
			* @default null
            * @alias ejLinearGauge#backgroundColor
			* @type {string}
			* @example 
			* //To set backgroundColor API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ backgroundColor: "Red" });   
			* @example 
			* //Get or set the backgroundColor API, after initialization:<br>
			*	//Gets the backgroundColor value  
			*	$("#LinearGauge1").ejLinearGauge("option", "backgroundColor");<br>			
			*	//Sets the backgroundColor value 
			*	$("#LinearGauge1").ejLinearGauge("option", "backgroundColor", "Red"  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            backgroundColor: null,
            /**		
			* Specifies the borderColor for Linear gauge.
			* @default null
            * @alias ejLinearGauge#borderColor
			* @type {string}
			* @example 
			* //To set borderColor API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ borderColor: "Red" });   
			* @example 
			* //Get or set the borderColor API, after initialization:<br>
			*	//Gets the borderColor value  
			*	$("#LinearGauge1").ejLinearGauge("option", "borderColor");<br>			
			*	//Sets the borderColor value 
			*	$("#LinearGauge1").ejLinearGauge("option", "borderColor", "Red"  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            borderColor: null,
            /**		
			* Specifies the labelColor for Linear gauge.
			* @default null
            * @alias ejLinearGauge#labelColor
			* @type {string}
			* @example 
			* //To set labelColor API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ labelColor: "Red" });   
			* @example 
			* //Get or set the labelColor API, after initialization:<br>
			*	//Gets the labelColor value  
			*	$("#LinearGauge1").ejLinearGauge("option", "labelColor");<br>			
			*	//Sets the labelColor value 
			*	$("#LinearGauge1").ejLinearGauge("option", "labelColor", "Red"  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            labelColor: null,
            /**		
			* Specifies the tickColor for Linear gauge.
			* @default null
            * @alias ejLinearGauge#tickColor
			* @type {string}
			* @example 
			* //To set tickColor API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ tickColor: "Red" });   
			* @example 
			* //Get or set the tickColor API, after initialization:<br>
			*	//Gets the tickColor value  
			*	$("#LinearGauge1").ejLinearGauge("option", "tickColor");<br>			
			*	//Sets the tickColor value 
			*	$("#LinearGauge1").ejLinearGauge("option", "tickColor", "Red"  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            tickColor: null,
            /**		
			* Specifies the read only state.
			* @default true
            * @alias ejLinearGauge#readOnly
			* @type {boolean}
			* @example 
			* //To set readOnly API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ readOnly: false });   
			* @example 
			* //Get or set the readOnly API, after initialization:<br>
			*	//Gets the readOnly value  
			*	$("#LinearGauge1").ejLinearGauge("option", "readOnly");<br>			
			*	//Sets the readOnly value 
			*	$("#LinearGauge1").ejLinearGauge("option", "readOnly", false  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            readOnly: true,
            /**		
			* Specifies the can resize state.
			* @default false
            * @alias ejLinearGauge#canResize
			* @type {boolean}
			* @example 
			* //To set canResize API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ canResize: true });   
			* @example 
			* //Get or set the canResize API, after initialization:<br>
			*	//Gets the canResize value  
			*	$("#LinearGauge1").ejLinearGauge("option", "canResize");<br>			
			*	//Sets the canResize value 
			*	$("#LinearGauge1").ejLinearGauge("option", "canResize", true  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            canResize: false,
            /**		
			* Specifies the frame Background ImageUrl
			* @default null
            * @alias ejLinearGauge#frameBackgroundImageUrl
			* @type {string}
			* @example 
			* //To set frameBackgroundImageUrl API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ frameBackgroundImageUrl: "bg.png" });  
			* @example 
			* //Get or set the frameBackgroundImageUrl API, after initialization:<br>
			*	//Gets the frameBackgroundImageUrl value  
			*	$("#LinearGauge1").ejLinearGauge("option", "frameBackgroundImageUrl");<br>			
			*	//Sets the frameBackgroundImageUrl value 
			*	$("#LinearGauge1").ejLinearGauge("option", "frameBackgroundImageUrl","bg.png"  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            frameBackgroundImageUrl: null, //"bg.png",
            /**		
			* Specifies the frameOuterWidth
			* @default 12
            * @alias ejLinearGauge#frameOuterWidth
			* @type {number}
			* @example 
			* //To set frameOuterWidth API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ frameOuterWidth: 13 });   
			* @example 
			* //Get or set the frameOuterWidth API, after initialization:<br>
			*	//Gets the frameOuterWidth value  
			*	$("#LinearGauge1").ejLinearGauge("option", "frameOuterWidth");<br>			
			*	//Sets the frameOuterWidth value 
			*	$("#LinearGauge1").ejLinearGauge("option", "frameOuterWidth",13  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            frameOuterWidth: 12,
            /**		
			* Specifies the frameInnerWidth
			* @default 12
            * @alias ejLinearGauge#frameInnerWidth
			* @type {number}
			* @example 
			* //To set frameInnerWidth API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ frameInnerWidth: 9});   
			* @example 
			* //Get or set the frameInnerWidth API, after initialization:<br>
			*	//Gets the frameInnerWidth value  
			*	$("#LinearGauge1").ejLinearGauge("option", "frameInnerWidth");<br>			
			*	//Sets the frameInnerWidth value 
			*	$("#LinearGauge1").ejLinearGauge("option", "frameInnerWidth",9  ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            frameInnerWidth: 8,
            /**		
			* Specifies the scales
			* @default 12
            * @alias ejLinearGauge#scales
			* @type {number}
			* @example 
			* //To set scales API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ scales: {[]}});   
			* @example 
			* //Get or set the scales API, after initialization:<br>
			*	//Gets the scales value  
			*	$("#LinearGauge1").ejLinearGauge("option", "scales");<br>			
			*	//Sets the scales value 
			*	$("#LinearGauge1").ejLinearGauge("option", "scales",scales[0] ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            scales: null,
            /**		
			* Specifies the animate state
			* @default true
            * @alias ejLinearGauge#animate
			* @type {boolean}
			* @example 
			* //To set animate API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ animate: true});  
			* @example 
			* //Get or set the animate API, after initialization:<br>
			*	//Gets the animate value  
			*	$("#LinearGauge1").ejLinearGauge("option", "animate");<br>			
			*	//Sets the animate value 
			*	$("#LinearGauge1").ejLinearGauge("option", "animate",false ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            animate: true,
            /**		
			* Specifies the animationSpeed
			* @default 500
            * @alias ejLinearGauge#animationSpeed
			* @type {number}
			* @example 
			* //To set animationSpeed API value during initialization  
			* 	$("#LinearGauge1").ejLinearGauge({ animationSpeed: 600});   
			* @example 
			* //Get or set the animationSpeed API, after initialization:<br>
			*	//Gets the animationSpeed value  
			*	$("#LinearGauge1").ejLinearGauge("option", "animationSpeed");<br>			
			*	//Sets the animationSpeed value 
			*	$("#LinearGauge1").ejLinearGauge("option", "animationSpeed",600 ); 
			* @memberof ejLinearGauge
			* @instance
			*/
            animationSpeed: 500,
            /**     
			 * Triggers while the ticks are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawTicks		
             * @param {Object} args.Object  returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
			 * @param {context} args.context returns the context element
			 * @param {location} args.location returns the startX and startY of the ticks
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
 			 * @param {scaleIndex} args.scaleIndex returns the scaleIndex to which the tick belongs.
   			 * @param {style} args.style returns the ticks style
			 * @param {tickAngle} args.tickAngle returns the angle of the tick.
			 * @param {tickElement} args.tickElement returns the current tick element.
			 * @param {tickIndex} args.tickIndex returns the index of the tick.
			 * @param {tickValue} args.tickValue returns the tick value of the tick.
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    drawTicks: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawTicks: null,
            /**     
			 * Triggers while the label are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawLabels		
             * @param {Object} args.Object  returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
			 * @param {context} args.context returns the context element
			 * @param {location} args.location returns the startX and startY of the label
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
 			 * @param {scaleIndex} args.scaleIndex returns the scaleIndex to which the label belongs.
   			 * @param {style} args.style returns the label style
			 * @param {labelAngle} args.labelAngle returns the angle of the label.
			 * @param {labelElement} args.labelElement returns the current label element.
			 * @param {labelIndex} args.labelIndex returns the index of the label.
			 * @param {labelValue} args.labelValue returns the label value of the label.
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    drawLabels: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawLabels: null,
            /**     
			 * Triggers while the bar pointer are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawBarPointers		
             * @param {Object} args.Object returns the object of the gauge.
			 * @param {cancel }  args.cancel returns the cancel option value
			 * @param {context} args.context returns the context element
			 * @param {location} args.location returns the startX and startY of the pointer
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
 			 * @param {scaleIndex} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {style} args.style returns the pointer style
			 * @param {barElement} args.barElement returns the current Bar pointer element.
			 * @param {barPointerIndex} args.barPointerIndex returns the index of the bar pointer.
			 * @param {PointerValue} args.PointerValue returns the value of the bar pointer.
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    drawBarPointers: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawBarPointers: null,
            /**     
			 * Triggers while the marker are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawMarkerPointers		
             * @param {Object} args.Object returns the object of the gauge.
			 * @param {cancel }  args.cancel returns the cancel option value
			 * @param {context} args.context returns the context element
			 * @param {location} args.location returns the startX and startY of the pointer
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
 			 * @param {scaleIndex} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {style} args.style returns the ticks style
			 * @param {markerElement} args.markerElement returns the current marker pointer element.
			 * @param {markerPointerIndex} args.markerPointerIndex returns the index of the marker pointer.
			 * @param {pointerValue} args.pointerValue returns the value of the marker pointer.
			 * @param {pointerAngle} args.pointerAngle returns the angle of the marker pointer.
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    drawMarkerPointers: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawMarkerPointers: null,
            /**     
			 * Triggers while the range are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawRange		
             * @param {Object} args.Object  returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
			 * @param {context} args.context returns the context element
			 * @param {location} args.location returns the startX and startY of the range
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
 			 * @param {scaleIndex} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {style} args.style returns the range style
			 * @param {rangeElement} args.rangeElement returns the current range element.
			 * @param {rangeIndex} args.rangeIndex returns the index of the range.
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    drawRange: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawRange: null,
            /**     
			 * Triggers while the customLabel are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawCustomLabel		
             * @param {Object} args.Object returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
			 * @param {context} args.context returns the context element
			 * @param {location} args.location returns the startX and startY of the customLabel
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
 			 * @param {scaleIndex} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {style} args.style returns the customLabel style
			 * @param {customLabelElement} args.customLabelElement returns the current customLabel element.
			 * @param {customLabelrangeIndex} args.customLabelrangeIndex returns the index of the customLabel.
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    drawCustomLabel: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawCustomLabel: null,
            /**     
			 * Triggers while the Indicator are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawIndicators		
             * @param {Object} args.Object returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
			 * @param {context} args.context returns the context element
			 * @param {location} args.location returns the startX and startY of the Indicator
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
 			 * @param {scaleIndex} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {style} args.style returns the Indicator style
			 * @param {IndicatorElement} args.IndicatorElement  returns the current Indicator element.
			 * @param {IndicatorIndex} args.IndicatorIndex returns the index of the Indicator.
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    drawIndicators: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawIndicators: null,
            /**     
			 * Triggers while the gauge start to Load.
			 * @event
			 * @name ejLinearGauge#load		
             * @param {Object} args.Object returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the entire scale element.
			 * @param {context} args.context returns the context element
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    load: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            load: null,
            /**     
			 * Triggers when the gauge is initialized.
			 * @event
			 * @name ejLinearGauge#init		
             * @param {Object} args.Object returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the entire scale element.
			 * @param {context} args.context returns the context element
			 * @param {type} args.type eturns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    init: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            init: null,
            /**     
			 * Triggers while the rendering of the gauge completed.
			 * @event
			 * @name ejLinearGauge#renderComplete		
             * @param {Object} args.Object returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the entire scale element.
			 * @param {context} args.context returns the context element
			 * @param {type} args.type returns the name of the event
			 * @example 
			 * //create event for button
             * $("#LinearGauge1").ejLinearGauge({
             *    renderComplete: function (args) {}
             * });
			 * @memberof ejLinearGauge
			 * @instance
			 */
            renderComplete: null
        },
        /**		
        * Specifies the scales.
        * @default object
        * @alias ejLinearGauge#scales
        * @type {object}
        * @example 
        * //To set Scale API value during initialization  
        * 	$("#LinearGauge1").ejLinearGauge({  scales:{minimum:5}});					 
        * @example 
        * //Get or set the Scale API, after initialization:<br>
        *	//Gets the Scale  
        *	$("#LinearGauge1").ejLinearGauge("option", "scales.minimum");<br>			
        *	//Sets the Scale 
        *	$("#LinearGauge1").ejLinearGauge("option", "scales.minimum",  5 ); 
        * @memberof ejLinearGauge
        * @instance
        */
        _defaultScaleValues: function () {
            return {
                /**		
				* Specifies the minimum of the Scale.
				* @default [Array.number] scales.minimum=null
                * @alias ejLinearGauge#scales->minimum
				* @type {number}
				* @example 
				* //To set minimum API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{minimum:5}});					 
				* @example 
				* //Get or set the minimum API, after initialization:<br>
				*	//Gets the minimum  
				*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                *var result = scale[index].minimum; 
				*	//Sets the minimum 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales",  [{minimum:5}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                minimum: null,
                /**		
				* Specifies the maximum of the Scale.
				* @default [Array.number] scales.maximum=null
                * @alias ejLinearGauge#scales->maximum
				* @type {number}
				* @example 
				* //To set maximum API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{maximum:100}});					 
				* @example 
				* //Get or set the maximum API, after initialization:<br>
				*	//Gets the maximum  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].maximum; 			
				*	//Sets the maximum 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{maximum:100}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                maximum: null,
                /**		
				* Specifies the majorIntervalValue of the Scale.
				* @default [Array.number] scales.majorIntervalValue=10
                * @alias ejLinearGauge#scales->majorIntervalValue
				* @type {number}
				* @example 
				* //To set majorIntervalValue API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{majorIntervalValue:10}});					 
				* @example 
				* //Get or set the majorIntervalValue API, after initialization:<br>
				*	//Gets the majorIntervalValue  
				*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].majorIntervalValue; 			
				*	//Sets the majorIntervalValue 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{majorIntervalValue:10}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                majorIntervalValue: 10,
                /**		
				* Specifies the minorIntervalValue of the Scale.
				* @default [Array.number] scales.minorIntervalValue=2
                * @alias ejLinearGauge#scales->minorIntervalValue
				* @type {number}
				* @example 
				* //To set minorIntervalValue API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{minorIntervalValue:1}});					 
				* @example 
				* //Get or set the minorIntervalValue API, after initialization:<br>
				*	//Gets the minorIntervalValue  
				*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].minorIntervalValue; 		
				*	//Sets the minorIntervalValue 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales",  {minorIntervalValue:1} ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                minorIntervalValue: 2,
                /**		
				* Specifies the scaleDirection of the Scale. See {@link scaleDirection}
				* @default [Array.enum] scales.scaleDirection=ej.scaleDirection.CounterClockwise
                * @alias ejLinearGauge#scales->scaleDirection
				* @type {enum}
				* @example 
				* //To set scaleDirection API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{scaleDirection:ej.scaleDirection.Clockwise}});					 
				* @example 
				* //Get or set the scaleDirection API, after initialization:<br>
				*	//Gets the scaleDirection  
				*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].scaleDirection; 		
				*	//Sets the scaleDirection 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{scaleDirection:ej.scaleDirection.Clockwise}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                scaleDirection: "CounterClockwise",
                /**		
				* Specifies the backgroundColor of the Scale.
				* @default [Array.string] scales.backgroundColor=null
                * @alias ejLinearGauge#scales->backgroundColor
				* @type {string}
				* @example 
				* //To set backgroundColor API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{backgroundColor:"red"}});					 
				* @example 
				* //Get or set the backgroundColor API, after initialization:<br>
				*	//Gets the backgroundColor  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].backgroundColor; 		
				* //Sets the backgroundColor 
				* $("#LinearGauge1").ejLinearGauge("option", "scales", [{backgroundColor:"red"}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                backgroundColor: null,
                /**		
				* Specifies the borderColor of the Scale.
				* @default [Array.string] scales.borderColor=null
                * @alias ejLinearGauge#scales->borderColor
				* @type {string}
				* @example 
				* //To set borderColor API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{borderColor:"red"}});					 
				* @example 
				* //Get or set the borderColor API, after initialization:<br>
				*	//Gets the borderColor  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].borderColor; 			
				*	//Sets the borderColor 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{borderColor:"red"}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                borderColor: null,
                /**		
				* Specifies the borderWidth of the Scale.
				* @default [Array.number] scales.borderWidth=1.5
                * @alias ejLinearGauge#scales->borderWidth
				* @type {number}
				* @example 
				* //To set borderWidth API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{borderWidth:2}});					 
				* @example 
				* //Get or set the borderWidth API, after initialization:<br>
				*	//Gets the borderWidth  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].borderWidth; 			
				*	//Sets the borderWidth 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{borderWidth:2}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                borderWidth: 1.5,
                /**		
				* Specifies the opacity of the Scale.
				* @default [Array.number] scales.opacity=NaN
                * @alias ejLinearGauge#scales->opacity
				* @type {number}
				* @example 
				* //To set opacity API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{opacity:0.5}});					 
				* @example 
				* //Get or set the opacity API, after initialization:<br>
				*	//Gets the opacity  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].opacity; 				
				*	//Sets the opacity 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{opacity:0.5}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                opacity: NaN,
                /**		
				* Specifies the scaleBarSize.
				* @default [Array.number] scales.scaleBarSize=30
                * @alias ejLinearGauge#scales->scaleBarSize
				* @type {number}
				* @example 
				* //To set scaleBarSize API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{scaleBarSize:25}});					 
				* @example 
				* //Get or set the scaleBarSize API, after initialization:<br>
				*	//Gets the scaleBarSize  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].scaleBarSize; 				
				*	//Sets the scaleBarSize 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{scaleBarSize:25}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                scaleBarSize: 30,
                /**		
				* Specifies the shadowOffset.
				* @default [Array.number] scales.shadowOffset= 0
                * @alias ejLinearGauge#scales->shadowOffset
				* @type {number}
				* @example 
				* //To set shadowOffset API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{shadowOffset:1}});					 
				* @example 
				* //Get or set the shadowOffset API, after initialization:<br>
				*	//Gets the shadowOffset  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].shadowOffset; 				
				*	//Sets the shadowOffset 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{shadowOffset:1}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                shadowOffset: 0,
                /**		
				* Specifies the scaleBarLength.
				* @default [Array.number] scales.scaleBarLength=290
                * @alias ejLinearGauge#scales->scaleBarLength
				* @type {number}
				* @example 
				* //To set scaleBarLength API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{scaleBarLength:300}});					 
				* @example 
				* //Get or set the scaleBarLength API, after initialization:<br>
				* //Gets the scaleBarLength  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].scaleBarLength; 				
				* //Sets the scaleBarLength 
				* $("#LinearGauge1").ejLinearGauge("option", "scales", [{scaleBarLength:300}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                scaleBarLength: 290,
                /**		
				* Specifies the scaleBarLength.See {@link scaleStyle}
				* @default [Array.enum] scales.scaleStyle=ej.scaleStyle.Rectangle
                * @alias ejLinearGauge#scales->scaleStyle
				* @type {enum}
				* @example 
				* //To set scaleStyle API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{scaleStyle:ej.scaleStyle.Rectangle}});					 
				* @example 
				* //Get or set the scaleStyle API, after initialization:<br>
				*	//Gets the scaleStyle  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].scaleStyle; 				
				*	//Sets the scaleStyle 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{scaleStyle:ej.scaleStyle.Rectangle}]); 
				* @memberof ejLinearGauge
				* @instance
				*/
                scaleStyle: "Rectangle",
                /**		
				* Specifies the Horizontal Location
				* @default [Array.object] scales.location={x:10,y:10}
                * @alias ejLinearGauge#scales->location
				* @type {object}
				* @example 
				* //To set Location API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{location:{x:30}}});					 
				* @example 
				* //Get or set the Location API, after initialization:<br>
				*	//Gets the Location  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].location; 
                * // index refers the index value of the scale
                * var result = scale[index].Location; 				
				*	//Sets the Location 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{location{x:30}}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                location: {
                    /**		
                    * Specifies the Horizontal Location
                    * @default [Array.Array.number] scales.location.x=50
                    * @alias ejLinearGauge#scales->location->x
                    * @type {number}
                    * @example 
                    * //To set x API value during initialization  
                    * 	$("#LinearGauge1").ejLinearGauge({  scales:{location:{x:30}}});					 
                    * @example 
                    * //Get or set the x API, after initialization:<br>
                    *	//Gets the x  location		
                    * var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // index refers the index value of the scale
                    * var result = scale[index].location.x; 
                    *	//Sets the x location
                    *	$("#LinearGauge1").ejLinearGauge("option", "scales, [{location{x:30}}] ); 
                    * @memberof ejLinearGauge
                    * @instance
                    */
                    x: 50,
                    /**		
                    * Specifies the vertical Location
                    * @default [Array.Array.number] scales.location.y=50
                    * @alias ejLinearGauge#scales->location->y
                    * @type {number}
                    * @example 
                    * //To set y API value during initialization  
                    * 	$("#LinearGauge1").ejLinearGauge({  scales:{location:{y:30}}});					 
                    * @example 
                    * //Get or set the y API, after initialization:<br>
                    *	//Gets the y  location
                     * var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // index refers the index value of the scale
                    * var result = scale[index].location.y; 			
                    *	//Sets the y location
                    *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{location{y:30}}] ); 
                    * @memberof ejLinearGauge
                    * @instance
                    */
                    y: 50
                },
                /**		
				* Specifies the showRanges state.
				* @default [Array.boolean] scales.showRanges=false
                * @alias ejLinearGauge#scales->showRanges
				* @type {boolean}
				* @example 
				* //To set showRanges API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{showRanges:false}});					 
				* @example 
				* //Get or set the showRanges API, after initialization:<br>
				*	//Gets the showRanges  
				 * var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].showRanges; 			
				*	//Sets the showRanges 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{showRanges:false}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                showRanges: false,
                /**		
				* Specifies the showIndicators state.
				* @default [Array.boolean] scales.showIndicators=false
                * @alias ejLinearGauge#scales->showIndicators
				* @type {boolean}
				* @example 
				* //To set showIndicators API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{showIndicators:true}});					 
				* @example 
				* //Get or set the showIndicators API, after initialization:<br>
				*	//Gets the showIndicators  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].showIndicators; 				
				*	//Sets the showIndicators 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{showIndicators:true}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                showIndicators: false,
                /**		
				* Specifies the showCustomLabel state.
				* @default  [Array.boolean] scales.showCustomLabel=false
                * @alias ejLinearGauge#scales->showCustomLabel
				* @type {boolean}
				* @example 
				* //To set showCustomLabel API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{showCustomLabel:true}});					 
				* @example 
				* //Get or set the showCustomLabel API, after initialization:<br>
				*	//Gets the showCustomLabel  
				 * var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].showCustomLabel; 		
				*	//Sets the showCustomLabel 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{showCustomLabel:true}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                showCustomLabel: false,
                /**		
				* Specifies the showLabels state.
				* @default  [Array.boolean] scales.showLabels=true
                * @alias ejLinearGauge#scales->showLabels
				* @type {boolean}
				* @example 
				* //To set showLabels API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{showLabels:false}});					 
				* @example 
				* //Get or set the showLabels API, after initialization:<br>
				*	//Gets the showLabels  
				 * var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].showLabels; 		
				*	//Sets the showLabels 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{showLabels:false}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                showLabels: true,
                /**		
				* Specifies the showTicks state.
				* @default  [Array.boolean] scales.showTicks=true
                * @alias ejLinearGauge#scales->showTicks
				* @type {boolean}
				* @example 
				* //To set showTicks API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{showTicks:false}});					 
				* @example 
				* //Get or set the showTicks API, after initialization:<br>
				*	//Gets the showTicks  
				 * var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].showTicks; 			
				*	//Sets the showTicks 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{showTicks:false}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                showTicks: true,
                /**		
				* Specifies the showBarPointers state.
				* @default  [Array.boolean] scales.showBarPointers=true
                * @alias ejLinearGauge#scales->showBarPointers
				* @type {boolean}
				* @example 
				* //To set showBarPointers API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{showBarPointers:false}});					 
				* @example 
				* //Get or set the showBarPointers API, after initialization:<br>
				*	//Gets the showBarPointers  
				 * var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].showBarPointers; 			
				*	//Sets the showBarPointers 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{showBarPointers:false}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                showBarPointers: true,
                /**		
				* Specifies the showMarkerPointers state.
				* @default  [Array.boolean] scales.showMarkerPointers=true
                * @alias ejLinearGauge#scales->showMarkerPointers
				* @type {boolean}
				* @example 
				* //To set showMarkerPointers API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{showMarkerPointers:false}});					 
				* @example 
				* //Get or set the showMarkerPointers API, after initialization:<br>
				*	//Gets the showMarkerPointers  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].showMarkerPointers; 			
				*	//Sets the showMarkerPointers 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{showMarkerPointers:false}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                showMarkerPointers: true,
                /**		
				* Specifies the ticks in the scale.
				* @default  [Array.object] scales.ticks={xDistanceFromScale: 0}
                * @alias ejLinearGauge#scales->ticks
				* @type {object}
				* @example 
				* //To set ticks API value during initialization  
				* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{angle:30}}});					 
				* @example 
				* //Get or set the ticks API, after initialization:<br>
				*	//Gets the ticks  
				* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // index refers the index value of the scale
                * var result = scale[index].ticks.angle; 			
				*	//Sets the ticks 
				*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{angle:30}}] ); 
				* @memberof ejLinearGauge
				* @instance
				*/
                ticks: [{
                    /**		
					* Specifies the xDistanceFromScale in the tick.
					* @default [Array.Array.number] scales.ticks.xDistanceFromScale=0
                    * @alias ejLinearGauge#scales->ticks->xDistanceFromScale
					* @type {number}
					* @example 
					* //To set xDistanceFromScale API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{xDistanceFromScale:10}}});					 
					* @example 
					* //Get or set the xDistanceFromScale API, after initialization:<br>
					*	//Gets the xDistanceFromScale  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].xDistanceFromScale; 			
					*	//Sets the xDistanceFromScale 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{xDistanceFromScale:10}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    xDistanceFromScale: 0,
                    /**		
					* Specifies the yDistanceFromScale in the tick.
					* @default [Array.Array.number] scales.ticks.yDistanceFromScale=0
                    * @alias ejLinearGauge#scales->ticks->yDistanceFromScale
					* @type {number}
					* @example 
					* //To set yDistanceFromScale API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{yDistanceFromScale:10}}});					 
					* @example 
					* //Get or set the yDistanceFromScale API, after initialization:<br>
					*	//Gets the yDistanceFromScale  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].yDistanceFromScale; 			
					*	//Sets the yDistanceFromScale 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{yDistanceFromScale:10}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    yDistanceFromScale: 0,
                    /**		
					* Specifies the angle in the tick.
					* @default [Array.Array.number] scales.ticks.angle=0
                    * @alias ejLinearGauge#scales->ticks->angle
					* @type {number}
					* @example 
					* //To set angle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{angle:0}}});					 
					* @example 
					* //Get or set the angle API, after initialization:<br>
					*	//Gets the angle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].angle;		
					*	//Sets the angle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{angle:0}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    angle: 0,
                    /**		
					* Specifies the tickColor in the tick.
					* @default [Array.Array.string] scales.ticks.tickColor=null
                    * @alias ejLinearGauge#scales->ticks->tickColor
					* @type {number}
					* @example 
					* //To set tickColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickColor:"Blue"}}});					 
					* @example 
					* //Get or set the tickColor API, after initialization:<br>
					*	//Gets the tickColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickColor;		
					*	//Sets the tickColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales",[{ticks{tickColor:"Blue"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickColor: null,
                    /**		
					* Specifies the tickStyle in the tick. See {@link tickStyle}
					* @default [Array.Array.enum] scales.ticks.tickStyle=ej.tickStyle.MajorInterval
                    * @alias ejLinearGauge#scales->ticks->tickStyle
					* @type {enum}
					* @example 
					* //To set tickStyle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickStyle:ej.tickStyle.MajorInterval}}});					 
					* @example 
					* //Get or set the tickStyle API, after initialization:<br>
					*	//Gets the tickStyle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickStyle;			
					*	//Sets the tickStyle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{tickStyle:ej.tickStyle.MajorInterval}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickStyle: "MajorInterval",
                    /**		
					* Specifies the tickPlacement in the tick. See {@link tickPlacement}
					* @default [Array.Array.enum] scales.ticks.tickPlacement=ej.tickPlacement.Near
                    * @alias ejLinearGauge#scales->ticks->tickPlacement
					* @type {enum}
					* @example 
					* //To set tickPlacement API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickPlacement:ej.tickPlacement.Near}}});					 
					* @example 
					* //Get or set the tickPlacement API, after initialization:<br>
					*	//Gets the tickPlacement  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickPlacement;			
					*	//Sets the tickPlacement 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{tickPlacement:ej.tickPlacement.Near}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickPlacement: "Near",
                    /**		
					* Specifies the opacity in the tick.
					* @default [Array.Array.number] scales.ticks.opacity=0
                    * @alias ejLinearGauge#scales->ticks->opacity
					* @type {number}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{opacity:0.7}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].opacity;		
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{opacity:0.7}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 0,
                    /**		
					* Specifies the tickHeight in the tick.
					* @default [Array.Array.number] scales.ticks.tickHeight=10
                    * @alias ejLinearGauge#scales->ticks->tickHeight
					* @type {number}
					* @example 
					* //To set tickHeight API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickHeight:8}}});					 
					* @example 
					* //Get or set the tickHeight API, after initialization:<br>
					*	//Gets the tickHeight  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickHeight;			
					*	//Sets the tickHeight 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{tickHeight:8}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickHeight: 10,
                    /**		
					* Specifies the tickWidth in the tick.
					* @default [Array.Array.number] scales.ticks.tickWidth=3
                    * @alias ejLinearGauge#scales->ticks->tickWidth
					* @type {number}
					* @example 
					* //To set tickWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickWidth:4}}});					 
					* @example 
					* //Get or set the tickWidth API, after initialization:<br>
					*	//Gets the tickWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickWidth;		
					*	//Sets the tickWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{tickWidth:4}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickWidth: 3
                },
                {
                    /**		
                        * Specifies the xDistanceFromScale in the tick.
                        * @default [Array.Array.number] scales.ticks.xDistanceFromScale=0
                        * @alias ejLinearGauge#scales->ticks->xDistanceFromScale
                        * @type {number}
                        * @example 
                        * //To set xDistanceFromScale API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{xDistanceFromScale:10}}});					 
                        * @example 
                        * //Get or set the xDistanceFromScale API, after initialization:<br>
                        *	//Gets the xDistanceFromScale  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // tindex - tick index value
                        * var result = scale[sindex].ticks[tindex].xDistanceFromScale;			
                        *	//Sets the xDistanceFromScale 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales",[{ticks{xDistanceFromScale:10}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                    xDistanceFromScale: 0,
                    /**		
					* Specifies the yDistanceFromScale in the tick.
					* @default [Array.Array.number] scales.ticks.yDistanceFromScale=0
                    * @alias ejLinearGauge#scales->ticks->yDistanceFromScale
					* @type {number}
					* @example 
					* //To set yDistanceFromScale API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{yDistanceFromScale:10}}});					 
					* @example 
					* //Get or set the yDistanceFromScale API, after initialization:<br>
					*	//Gets the yDistanceFromScale  
					* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // index refers the index value of the scale
                    * var result = scale[index].ticks.yDistanceFromScale; 			
					*	//Sets the yDistanceFromScale 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{yDistanceFromScale:10}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    yDistanceFromScale: 0,
                    /**		
					* Specifies the angle in the tick.
					* @default [Array.Array.number] scales.ticks.angle=0
                    * @alias ejLinearGauge#scales->ticks->angle
					* @type {number}
					* @example 
					* //To set angle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{angle:0}}});					 
					* @example 
					* //Get or set the angle API, after initialization:<br>
					*	//Gets the angle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].angle;			
					*	//Sets the angle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{angle:0}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    angle: 0,
                    /**		
					* Specifies the tickColor in the tick.
					* @default [Array.Array.string] scales.ticks.tickColor=null
                    * @alias ejLinearGauge#scales->ticks->tickColor
					* @type {number}
					* @example 
					* //To set tickColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickColor:"Blue"}}});					 
					* @example 
					* //Get or set the tickColor API, after initialization:<br>
					*	//Gets the tickColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickColor;		
					*	//Sets the tickColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales",[{ticks{tickColor:"Blue"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickColor: null,
                    /**		
					* Specifies the tickStyle in the tick.  See {@link tickStyle}
					* @default [Array.Array.enum] scales.ticks.tickStyle=ej.tickStyle.MajorInterval
                    * @alias ejLinearGauge#scales->ticks->tickStyle
					* @type {enum}
					* @example 
					* //To set tickStyle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickStyle:ej.tickStyle.MajorInterval}}});					 
					* @example 
					* //Get or set the tickStyle API, after initialization:<br>
					*	//Gets the tickStyle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickStyle;			
					*	//Sets the tickStyle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{tickStyle:ej.tickStyle.MajorInterval}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickStyle: "MinorInterval",
                    /**		
					* Specifies the tickPlacement in the tick.  See {@link tickPlacement}
					* @default [Array.Array.enum] scales.ticks.tickPlacement=ej.tickPlacement.Near
                    * @alias ejLinearGauge#scales->ticks->tickPlacement
					* @type {enum}
					* @example 
					* //To set tickPlacement API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickPlacement:ej.tickPlacement.Near}}});					 
					* @example 
					* //Get or set the tickPlacement API, after initialization:<br>
					*	//Gets the tickPlacement  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickPlacement;			
					*	//Sets the tickPlacement 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{tickPlacement:ej.tickPlacement.Near}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickPlacement: "Near",
                    /**		
					* Specifies the opacity in the tick.
					* @default [Array.Array.number] scales.ticks.opacity= 0
                    * @alias ejLinearGauge#scales->ticks->opacity
					* @type {number}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{opacity:0.7}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].opacity;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{opacity:0.7}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 0,
                    /**		
					* Specifies the tickHeight in the tick.
					* @default [Array.Array.number] scales.ticks.tickHeight=5
                    * @alias ejLinearGauge#scales->ticks->tickHeight
					* @type {number}
					* @example 
					* //To set tickHeight API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickHeight:8}}});					 
					* @example 
					* //Get or set the tickHeight API, after initialization:<br>
					*	//Gets the tickHeight  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickHeight;		
					*	//Sets the tickHeight 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales",[{ticks{tickHeight:8}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickHeight: 5,
                    /**		
					* Specifies the tickWidth in the tick.
					* @default [Array.Array.number] scales.ticks.tickWidth=2
                    * @alias ejLinearGauge#scales->ticks->tickWidth
					* @type {number}
					* @example 
					* //To set tickWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ticks:{tickWidth:4}}});					 
					* @example 
					* //Get or set the tickWidth API, after initialization:<br>
					*	//Gets the tickWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // tindex - tick index value
                    * var result = scale[sindex].ticks[tindex].tickWidth;			
					*	//Sets the tickWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ticks{tickWidth:4}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    tickWidth: 2
                }],
                /**		
					* Specifies the ranges in the tick.
					* @default [Array.object] scales.ranges;
                    * @alias ejLinearGauge#scales->ranges->tickWidth
					* @type {number}
					* @example 
					* //To set ranges API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{endWidth:4}}});					 
					* @example 
					* //Get or set the ranges API, after initialization:<br>
					*	//Gets the ranges  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the ranges 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{endWidth:4}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                ranges: [{
                    /**		
					* Specifies the endWidth in the ranges.
					* @default [Array.Array.number] scales.ranges.endWidth=10
                    * @alias ejLinearGauge#scales->ranges->endWidth
					* @type {number}
					* @example 
					* //To set endWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{endWidth:4}}});					 
					* @example 
					* //Get or set the endWidth API, after initialization:<br>
					*	//Gets the endWidth  
					* var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the endWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{endWidth:4}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    endWidth: 10,
                    /**		
					* Specifies the rangePosition in the ranges. See {@link rangePosition}
					* @default [Array.Array.enum] scales.ranges.rangePosition= ej.rangePosition.Near
                    * @alias ejLinearGauge#scales->ranges->rangePosition
					* @type {enum}
					* @example 
					* //To set rangePosition API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{rangePosition:ej.rangePosition.Near}}});					 
					* @example 
					* //Get or set the rangePosition API, after initialization:<br>
					*	//Gets the rangePosition  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the rangePosition 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{rangePosition:ej.rangePosition.Near}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    rangePosition: "Center",
                    /**		
					* Specifies the startWidth in the ranges.
					* @default [Array.Array.number] scales.ranges.startWidth= 10
                    * @alias ejLinearGauge#scales->ranges->startWidth
					* @type {number}
					* @example 
					* //To set startWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{startWidth:10}}});					 
					* @example 
					* //Get or set the startWidth API, after initialization:<br>
					*	//Gets the startWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the startWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{startWidth:15}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    startWidth: 10,
                    /**		
					* Specifies the distanceFromScale in the ranges.
					* @default [Array.Array.number] scales.ranges.distanceFromScale= 0
                    * @alias ejLinearGauge#scales->ranges->distanceFromScale
					* @type {number}
					* @example 
					* //To set distanceFromScale API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{distanceFromScale:50}}});					 
					* @example 
					* //Get or set the distanceFromScale API, after initialization:<br>
					*	//Gets the distanceFromScale  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the distanceFromScale 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{distanceFromScale:5}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    distanceFromScale: 0,
                    /**		
					* Specifies the endValue in the ranges.
					* @default [Array.Array.number] scales.ranges.endValue= 60
                    * @alias ejLinearGauge#scales->ranges->endValue
					* @type {number}
					* @example 
					* //To set endValue API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{endValue:60}}});					 
					* @example 
					* //Get or set the endValue API, after initialization:<br>
					*	//Gets the endValue  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the endValue 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{endValue:60}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    endValue: 60,
                    /**		
					* Specifies the startValue in the ranges.
					* @default [Array.Array.number] scales.ranges.startValue= 20
                    * @alias ejLinearGauge#scales->ranges->startValue
					* @type {number}
					* @example 
					* //To set startValue API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{startValue:30}}});					 
					* @example 
					* //Get or set the startValue API, after initialization:<br>
					*	//Gets the startValue  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the startValue 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{startValue:30}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    startValue: 20,
                    /**		
					* Specifies the rangeGradient in the ranges.
					* @default [Array.Array.string] scales.ranges.rangeGradient= null
                    * @alias ejLinearGauge#scales->ranges->rangeGradient
					* @type {string}
					* @example 
					* //To set rangeGradient API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{rangeGradient:"red"}}});					 
					* @example 
					* //Get or set the rangeGradient API, after initialization:<br>
					*	//Gets the rangeGradient  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the rangeGradient 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{rangeGradient:"red"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    rangeGradient: null,
                    /**		
					* Specifies the backgroundColor in the ranges.
					* @default [Array.Array.string] scales.ranges.backgroundColor= null
                    * @alias ejLinearGauge#scales->ranges->backgroundColor
					* @type {string}
					* @example 
					* //To set backgroundColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{backgroundColor:"red"}}});					 
					* @example 
					* //Get or set the backgroundColor API, after initialization:<br>
					*	//Gets the backgroundColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the backgroundColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{backgroundColor:"red"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    backgroundColor: null,
                    /**		
					* Specifies the borderColor in the ranges.
					* @default [Array.Array.string] scales.ranges.borderColor= null
                    * @alias ejLinearGauge#scales->ranges->borderColor
					* @type {string}
					* @example 
					* //To set borderColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{borderColor:"red"}}});					 
					* @example 
					* //Get or set the borderColor API, after initialization:<br>
					*	//Gets the borderColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the borderColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{borderColor:"red"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    borderColor: null,
                    /**		
					* Specifies the borderWidth in the ranges.
					* @default [Array.Array.number] scales.ranges.borderWidth= 1.5
                    * @alias ejLinearGauge#scales->ranges->borderWidth
					* @type {number}
					* @example 
					* //To set borderWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{borderWidth:2}}});					 
					* @example 
					* //Get or set the borderWidth API, after initialization:<br>
					*	//Gets the borderWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the borderWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{borderWidth:2}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    borderWidth: 1.5,
                    /**		
					* Specifies the opacity in the ranges.
					* @default [Array.Array.number] scales.ranges.opacity= nul
                    * @alias ejLinearGauge#scales->ranges->opacity
					* @type {number}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{ranges:{opacity:1}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // rindex - Range index value
                    * var result = scale[sindex].ranges[rindex].endWidth;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{ranges{opacity:1}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: null
                }],
                /**		
                * Specifies the labels.
                * @default [Array.object] scales.labels={yDistanceFromScale: 0}
                * @alias ejLinearGauge#scales->labels
                * @type {number}
                * @example 
                * //To set labels API value during initialization  
                * 	$("#LinearGauge1").ejLinearGraph({  scales:{ranges:{yDistanceFromScale:1}}});					* 
                * @example 
                * //Get or set the labels API, after initialization:<br>
                *	//Gets the labels  
                *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // sindex - scale index value
                * // lindex - label index value
                * var result = scale[sindex].labels[lindex].endWidth;			
                *	//Sets the labels 
                *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{yDistanceFromScale:1}}]); 
                * @memberof ejLinearGauge
                * @instance
                */
                labels: [{
                    /**		
					* Specifies the yDistanceFromScale of labels.
					* @default [Array.Array.number] scales.labels.yDistanceFromScale=0
                    * @alias ejLinearGauge#scales->labels->yDistanceFromScale
					* @type {number}
					* @example 
					* //To set yDistanceFromScale API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{yDistanceFromScale:1}}});					 
					* @example 
					* //Get or set the yDistanceFromScale API, after initialization:<br>
					*	//Gets the yDistanceFromScale  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].yDistanceFromScale;			
					*	//Sets the yDistanceFromScale 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{yDistanceFromScale:1}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    yDistanceFromScale: 0,
                    /**		
					* Specifies the xDistanceFromScale of labels.
					* @default [Array.Array.number] scales.labels.xDistanceFromScale=-10
                    * @alias ejLinearGauge#scales->labels->xDistanceFromScale
					* @type {number}
					* @example 
					* //To set xDistanceFromScale API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{xDistanceFromScale:1}}});					 
					* @example 
					* //Get or set the xDistanceFromScale API, after initialization:<br>
					*	//Gets the xDistanceFromScale  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].xDistanceFromScale;			
					*	//Sets the xDistanceFromScale 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{xDistanceFromScale:1}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    xDistanceFromScale: -10,
                    /**		
					* Specifies the angle of labels.
					* @default [Array.Array.number] scales.labels.angle=0
                    * @alias ejLinearGauge#scales->labels->angle
					* @type {number}
					* @example 
					* //To set angle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{angle:1}}});					 
					* @example 
					* //Get or set the angle API, after initialization:<br>
					*	//Gets the angle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].angle;			
					*	//Sets the angle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{angle:1}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    angle: 0,
                    /**		
					* Specifies the font of labels.
					* @default [Array.Array.object] scales.labels.font
                    * @alias ejLinearGauge#scales->labels->font
					* @type {object}
					* @example 
					* //To set font API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{font:{size: "11px"}}}});					 
					* @example 
					* //Get or set the font API, after initialization:<br>
					*	//Gets the font  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].font;			
					*	//Sets the font 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{font:{size: "11px"}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    font: {
                        /**		
                        * Specifies the size of font.
                        * @default [Array.Array.Array.string] scales.labels.font.size="11px"
                        * @alias ejLinearGauge#scales->labels->font->size
                        * @type {string}
                        * @example 
                        * //To set size API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{font:{size: "11px"}}}});					 
                        * @example 
                        * //Get or set the size API, after initialization:<br>
                        *	//Gets the size  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // lindex - label index value
                        * var result = scale[sindex].labels[lindex].font.size;			
                        *	//Sets the size 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{font:{size: "11px"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        size: "11px",
                        /**		
                        * Specifies the fontFamily of font.
                        * @default [Array.Array.Array.string] scales.labels.font.fontFamily="Arial"
                        * @alias ejLinearGauge#scales->labels->font->fontFamily
                        * @type {string}
                        * @example 
                        * //To set fontFamily API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{font:{fontFamily: "Arial"}}}});					
                        * @example 
                        * //Get or set the fontFamily API, after initialization:<br>
                        *	//Gets the fontFamily  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // lindex - label index value
                        * var result = scale[sindex].labels[lindex].font.fontFamily;			
                        *	//Sets the fontFamily 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{font:{fontFamily: "Arial"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        fontFamily: "Arial",
                        /**		
                        * Specifies the fontStyle of font.See {@link fontStyle}
                        * @default [Array.Array.Array.enum] scales.labels.font.fontStyle=ej.fontStyle.Bold
                        * @alias ejLinearGauge#scales->labels->font->fontStyle
                        * @type {enum}
                        * @example 
                        * //To set fontStyle API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{font:{fontStyle: ej.fontStyle.Bold}}}});					 
                        * @example 
                        * //Get or set the fontStyle API, after initialization:<br>
                        *	//Gets the fontStyle  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // lindex - label index value
                        * var result = scale[sindex].labels[lindex].font.fontStyle;			
                        *	//Sets the fontStyle 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{font:{fontStyle: ej.fontStyle.Bold}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        fontStyle: "Bold"
                    },
                    /**		
					* Specifies the textColor of font.
					* @default [Array.Array.string] scales.labels.textColor=null
                    * @alias ejLinearGauge#scales->labels->textColor
					* @type {string}
					* @example 
					* //To set textColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{textColor: "green"}}});					 
					* @example 
					* //Get or set the textColor API, after initialization:<br>
					*	//Gets the textColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].textColor;			
					*	//Sets the textColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels{textColor: "green"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    textColor: null,
                    /**		
					* Specifies the opacity of label.
					* @default [Array.Array.number] scales.labels.opacity=0
                    * @alias ejLinearGauge#scales->labels->opacity
					* @type {number}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{opacity: 0.5}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].opacity;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels:{opacity: 0.5}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 0,
                    /**		
					* Specifies the labelStyle of label. See {@link labelStyle}
					* @default [Array.Array.enum] scales.labels.labelStyle=ej.labelStyle.Major
                    * @alias ejLinearGauge#scales->labels->labelStyle
					* @type {enum}
					* @example 
					* //To set labelStyle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{labelStyle: ej.labelStyle.Major}}});					 
					* @example 
					* //Get or set the labelStyle API, after initialization:<br>
					*	//Gets the labelStyle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].labelStyle;			
					*	//Sets the labelStyle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels:{labelStyle: ej.labelStyle.Major}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    labelStyle: "Major",
                    /**		
					* Specifies the labelPlacement of label. See {@link labelPlacement}
					* @default [Array.Array.enum] scales.labels.labelPlacement=ej.labelPlacement.Near
                    * @alias ejLinearGauge#scales->labels->labelPlacement
					* @type {enum}
					* @example 
					* //To set labelPlacement API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{labelPlacement: ej.labelPlacement.Far}}});					 
					* @example 
					* //Get or set the labelPlacement API, after initialization:<br>
					*	//Gets the labelPlacement  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].labelPlacement;			
					*	//Sets the labelPlacement 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels:{labelPlacement: ej.labelPlacement.Far}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    labelPlacement: "Near",
                    /**		
					* need to includeFirstValue.
					* @default [Array.Array.boolean] scales.labels.labelPlacement=true
                    * @alias ejLinearGauge#scales->labels->includeFirstValue
					* @type {boolean}
					* @example 
					* //To set includeFirstValue API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{includeFirstValue: false}}});					 
					* @example 
					* //Get or set the includeFirstValue API, after initialization:<br>
					*	//Gets the includeFirstValue  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].includeFirstValue;			
					*	//Sets the includeFirstValue 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels:{includeFirstValue: false}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    includeFirstValue: true,
                    /**		
					* Specifies the unitText of label.  
					* @default [Array.Array.string] scales.labels.unitText=""
                    * @alias ejLinearGauge#scales->labels->unitText
					* @type {string}
					* @example 
					* //To set unitText API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{unitText: "MpH"}}});					 
					* @example 
					* //Get or set the unitText API, after initialization:<br>
					*	//Gets the unitText  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].unitText;			
					*	//Sets the unitText 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels:{unitText: "MpH"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    unitText: "",
                    /**		
					* Specifies the unitTextPosition of label.See {@link unitTextPosition}
					* @default [Array.Array.enum] scales.labels.unitTextPosition="Back"
                    * @alias ejLinearGauge#scales->labels->unitTextPosition
					* @type {enum}
					* @example 
					* //To set unitTextPosition API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{labels:{unitTextPosition: "Front"}}});					 
					* @example 
					* //Get or set the unitTextPosition API, after initialization:<br>
					*	//Gets the unitTextPosition  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // lindex - label index value
                    * var result = scale[sindex].labels[lindex].unitTextPosition;			
					*	//Sets the unitTextPosition 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{labels:{unitTextPosition: "Front"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    unitTextPosition: "Back"//["Front","Back"]
                }],
                /**		
                * Specifies the markerPointers
                * @default [Array.object] scales.markerPointers
                * @alias ejLinearGauge#scales->markerPointers
                * @type {object}
                * @example 
                * //To set markerPointers API value during initialization  
                * 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{markerStyle: "Triangle"}}});					 
                * @example 
                * //Get or set the markerPointers API, after initialization:<br>
                *	//Gets the markerPointers  
                *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // sindex - scale index value
                * // mindex - markerPointers index value
                * var result = scale[sindex].markerPointers[mindex];			
                *	//Sets the markerPointers 
                *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{markerStyle: "Triangle"}}]); 
                * @memberof ejLinearGauge
                * @instance
                */
                markerPointers: [{
                    /**		
					* Specifies the markerStyle of marker pointerSee {@link markerStyle}
					* @default [Array.Array.enum] scales.markerPointers.markerStyle=ej.markerStyle.Triangle
                    * @alias ejLinearGauge#scales->markerPointers->markerStyle
					* @type {enum}
					* @example 
					* //To set markerStyle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{markerStyle: ej.markerStyle.Rectangle}}});					 
					* @example 
					* //Get or set the markerStyle API, after initialization:<br>
					*	//Gets the markerStyle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex - markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].markerStyle;			
					*	//Sets the markerStyle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{markerStyle: ej.markerStyle.Rectangle}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    markerStyle: "Triangle",
                    /**		
					* Specifies the pointerLength of marker pointer
					* @default [Array.Array.number] scales.markerPointers.pointerLength=30
                    * @alias ejLinearGauge#scales->markerPointers->pointerLength
					* @type {number}
					* @example 
					* //To set pointerLength API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{pointerLength: 25}}});					 
					* @example 
					* //Get or set the pointerLength API, after initialization:<br>
					*	//Gets the pointerLength  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex - markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].pointerLength;			
					*	//Sets the pointerLength 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{pointerLength: 25}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    pointerLength: 30,
                    /**		
					* Specifies the pointerPlacement of marker pointer  See {@link pointerPlacement}
					* @default [Array.Array.enum] scales.markerPointers.pointerPlacement=ej.pointerPlacement.Far
                    * @alias ejLinearGauge#scales->markerPointers->pointerPlacement
					* @type {enum}
					* @example 
					* //To set pointerPlacement API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{pointerPlacement: ej.pointerPlacement.Near}}});					
					* @example 
					* //Get or set the pointerPlacement API, after initialization:<br>
					*	//Gets the pointerPlacement  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex - markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].pointerPlacement;			
					*	//Sets the pointerPlacement 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{pointerPlacement: ej.pointerPlacement.Near}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    pointerPlacement: "Far",
                    /**		
					* Specifies the pointerGradient of marker pointer
					* @default [Array.Array.string] scales.markerPointers.pointerGradient=null
                    * @alias ejLinearGauge#scales->markerPointers->pointerGradient
					* @type {string}
					* @example 
					* //To set pointerGradient API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{pointerGradient: "red"}}});					 
					* @example 
					* //Get or set the pointerGradient API, after initialization:<br>
					*	//Gets the pointerGradient  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex - markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].pointerGradient;			
					*	//Sets the pointerGradient 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{pointerGradient: "red"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    pointerGradient: null,
                    /**		
					* Specifies the distanceFromScale of marker pointer
					* @default [Array.Array.number] scales.markerPointers.distanceFromScale=0
                    * @alias ejLinearGauge#scales->markerPointers->distanceFromScale
					* @type {number}
					* @example 
					* //To set distanceFromScale API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{distanceFromScale: 2}}});					 
					* @example 
					* //Get or set the distanceFromScale API, after initialization:<br>
					*	//Gets the distanceFromScale  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex - markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].distanceFromScale;			
					*	//Sets the distanceFromScale 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{distanceFromScale: 2}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    distanceFromScale: 0,
                    /**		
					* Specifies the pointerWidth of marker pointer
					* @default [Array.Array.number] scales.markerPointers.pointerWidth=30
                    * @alias ejLinearGauge#scales->markerPointers->pointerWidth
					* @type {number}
					* @example 
					* //To set pointerWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{pointerWidth: 25}}});					 
					* @example 
					* //Get or set the pointerWidth API, after initialization:<br>
					*	//Gets the pointerWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex - markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].pointerWidth;			
					*	//Sets the pointerWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{pointerWidth: 25}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    pointerWidth: 30,
                    /**		
					* Specifies the value of marker pointer
					* @default [Array.Array.number] scales.markerPointers.value=null
                    * @alias ejLinearGauge#scales->markerPointers->value
					* @type {number}
					* @example 
					* //To set value API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{value: 25}}});					 
					* @example 
					* //Get or set the value API, after initialization:<br>
					*	//Gets the value  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex - markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].value;			
					*	//Sets the value 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{value: 25}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    value: null,
                    /**		
					* Specifies the backgroundColor of marker pointer
					* @default [Array.Array.string] scales.markerPointers.backgroundColor=null
                    * @alias ejLinearGauge#scales->markerPointers->backgroundColor
					* @type {string}
					* @example 
					* //To set backgroundColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{backgroundColor: "blue"}}});					 
					* @example 
					* //Get or set the backgroundColor API, after initialization:<br>
					*	//Gets the backgroundColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex -  markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].backgroundColor;			
					*	//Sets the backgroundColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{backgroundColor: "blue"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    backgroundColor: null,
                    /**		
					* Specifies the borderColor of marker pointer
					* @default [Array.Array.string] scales.markerPointers.borderColor=null
                    * @alias ejLinearGauge#scales->markerPointers->borderColor
					* @type {string}
					* @example 
					* //To set borderColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{borderColor: "blue"}}});					 
					* @example 
					* //Get or set the borderColor API, after initialization:<br>
					*	//Gets the borderColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex -  markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].borderColor;			
					*	//Sets the borderColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{borderColor: "blue"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    borderColor: null,
                    /**		
					* Specifies the borderWidth of marker pointer
					* @default [Array.Array.number] scales.markerPointers.borderWidth=1.5
                    * @alias ejLinearGauge#scales->markerPointers->borderWidth
					* @type {number}
					* @example 
					* //To set borderWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{borderWidth: 1.5}}});					 
					* @example 
					* //Get or set the borderWidth API, after initialization:<br>
					*	//Gets the borderWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex -  markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].borderWidth;			
					*	//Sets the borderWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{borderWidth: 2}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    borderWidth: 1.5,
                    /**		
					* Specifies the opacity of marker pointer
					* @default [Array.Array.number] scales.markerPointers.opacity=1
                    * @alias ejLinearGauge#scales->markerPointers->opacity
					* @type {number}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{markerPointers:{opacity: 0.5}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // mindex -  markerPointers index value
                    * var result = scale[sindex].markerPointers[mindex].opacity;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{markerPointers:{opacity: 0.5}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 1
                }],
                /**		
                * Specifies the scaleBarGradient of bar pointer
                * @default [Array.Array] scales.barPointers
                * @alias ejLinearGauge#scales->barPointers
                * @type {object}
                * @example 
                * //To set scaleBarGradient API value during initialization  
                * 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{}}});					 
                * @example 
                * //Get or set the scaleBarGradient API, after initialization:<br>
                *	//Gets the scaleBarGradient  
                *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // sindex - scale index value
                * // bindex -  barPointers index value
                * var result = scale[sindex].barPointers[bindex].scaleBarGradient;			
                *	//Sets the scaleBarGradient 
                *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{scaleBarGradient: "red"}}]); 
                * @memberof ejLinearGauge
                * @instance
                */
                barPointers: [{
                    /**		
					* Specifies the scaleBarGradient of bar pointer
					* @default [Array.Array.string] scales.barPointers.scaleBarGradient=null
                    * @alias ejLinearGauge#scales->barPointers->scaleBarGradient
					* @type {string}
					* @example 
					* //To set scaleBarGradient API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{scaleBarGradient: "red"}}});					
					* @example 
					* //Get or set the scaleBarGradient API, after initialization:<br>
					*	//Gets the scaleBarGradient  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // bindex -  barPointers index value
                    * var result = scale[sindex].barPointers[bindex].scaleBarGradient;			
					*	//Sets the scaleBarGradient 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{scaleBarGradient: "red"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    scaleBarGradient: null,
                    /**		
					* Specifies the distanceFromScale of bar pointer
					* @default [Array.Array.number] scales.barPointers.distanceFromScale=0
                    * @alias ejLinearGauge#scales->barPointers->distanceFromScale
					* @type {number}
					* @example 
					* //To set distanceFromScale API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{distanceFromScale: 20}}});					 
					* @example 
					* //Get or set the distanceFromScale API, after initialization:<br>
					*	//Gets the distanceFromScale  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // bindex -  barPointers index value
                    * var result = scale[sindex].barPointers[bindex].distanceFromScale;			
					*	//Sets the distanceFromScale 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{distanceFromScale: 20}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    distanceFromScale: 0,
                    /**		
					* Specifies the pointerWidth of bar pointer
					* @default [Array.Array.number] scales.barPointers.pointerWidth=30
                    * @alias ejLinearGauge#scales->barPointers->pointerWidth
					* @type {number}
					* @example 
					* //To set pointerWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{pointerWidth: 25}}});					 
					* @example 
					* //Get or set the pointerWidth API, after initialization:<br>
					*	//Gets the pointerWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // bindex -  barPointers index value
                    * var result = scale[sindex].barPointers[bindex].pointerWidth;			
					*	//Sets the pointerWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{pointerWidth: 25}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    pointerWidth: 30,
                    /**		
					* Specifies the value of bar pointer
					* @default [Array.Array.number] scales.barPointers.value=50
                    * @alias ejLinearGauge#scales->barPointers->value
					* @type {number}
					* @example 
					* //To set value API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{value: 100}}});					 
					* @example 
					* //Get or set the value API, after initialization:<br>
					*	//Gets the value  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // bindex -  barPointers index value
                    * var result = scale[sindex].barPointers[bindex].value;			
					*	//Sets the value 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{value: 100}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    value: 50,
                    /**		
					* Specifies the backgroundColor of bar pointer
					* @default [Array.Array.string] scales.barPointers.backgroundColor=null
                    * @alias ejLinearGauge#scales->barPointers->backgroundColor
					* @type {string}
					* @example 
					* //To set backgroundColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{backgroundColor: "red"}}});					 
					* @example 
					* //Get or set the backgroundColor API, after initialization:<br>
					*	//Gets the backgroundColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // bindex -  barPointers index value
                    * var result = scale[sindex].barPointers[bindex].backgroundColor;			
					*	//Sets the backgroundColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{backgroundColor: "red"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    backgroundColor: null,
                    /**		
					* Specifies the borderColor of bar pointer
					* @default [Array.Array.string] scales.barPointers.borderColor=null
                    * @alias ejLinearGauge#scales->barPointers->borderColor
					* @type {string}
					* @example 
					* //To set borderColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{borderColor: "red"}}});					 
					* @example 
					* //Get or set the borderColor API, after initialization:<br>
					*	//Gets the borderColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // bindex -  barPointers index value
                    * var result = scale[sindex].barPointers[bindex].borderColor;			
					*	//Sets the borderColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{borderColor: "red"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    borderColor: null,
                    /**		
					* Specifies the borderWidth of bar pointer
					* @default [Array.Array.number] scales.barPointers.borderWidth=1.5
                    * @alias ejLinearGauge#scales->barPointers->borderWidth
					* @type {number}
					* @example 
					* //To set borderWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{borderWidth: 2}}});					 
					* @example 
					* //Get or set the borderWidth API, after initialization:<br>
					*	//Gets the borderWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // bindex -  barPointers index value
                    * var result = scale[sindex].barPointers[bindex].borderWidth;			
					*	//Sets the borderWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{borderWidth: 2}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    borderWidth: 1.5,
                    /**		
					* Specifies the opacity of bar pointer
					* @default [Array.Array.number] scales.barPointers.opacity=1
                    * @alias ejLinearGauge#scales->barPointers->opacity
					* @type {number}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{barPointers:{opacity: 0.5}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // bindex -  barPointers index value
                    * var result = scale[sindex].barPointers[bindex].opacity;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{barPointers:{opacity: 0.5}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 1
                }],
                /**		
                * Specifies the indicator
                * @default [Array.Array.object] scales.indicators
                * @alias ejLinearGauge#scales->indicators
                * @type {object}
                * @example 
                * //To set indicatorWidth API value during initialization  
                * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{indicatorWidth: 25}}}});					 
                * @example 
                * //Get or set the indicatorWidth API, after initialization:<br>
                *	//Gets the indicatorWidth  
                *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // sindex - scale index value
                * // iindex -  indicators index value
                * var result = scale[sindex].indicators[bindex].indicatorWidth;			
                *	//Sets the indicatorWidth 
                *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{indicatorWidth: 25}}}]); 
                * @memberof ejLinearGauge
                * @instance
                */
                indicators: [{
                    /**		
					* Specifies the font of bar indicators
					* @default [Array.Array.object] scales.indicators.font={size: "11px"}
                    * @alias ejLinearGauge#scales->indicators->font
					* @type {object}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{font:{size: "11px"}}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].opacity;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{font:{size: "11px"}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    font: {
                        /**		
                        * Specifies the size of font in bar indicators
                        * @default [Array.Array.Array.number] scales.indicators.font.size="11px"
                        * @alias ejLinearGauge#scales->indicators->font->size
                        * @type {number}
                        * @example 
                        * //To set size API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{font:{size: "11px"}}}});					 
                        * @example 
                        * //Get or set the size API, after initialization:<br>
                        *	//Gets the size  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].font.size;			
                        *	//Sets the size 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{font:{size: "11px"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        size: "11px",
                        /**		
                        * Specifies the fontFamily of font in bar indicators
                        * @default [Array.Array.Array.string] scales.indicators.font.fontFamily="Arial"
                        * @alias ejLinearGauge#scales->indicators->font->fontFamily
                        * @type {string}
                        * @example 
                        * //To set fontFamily API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{font:{fontFamily: "Arial"}}}});					 
                        * @example 
                        * //Get or set the fontFamily API, after initialization:<br>
                        *	//Gets the fontFamily  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].font.fontFamily;			
                        *	//Sets the fontFamily 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{font:{fontFamily: "Arial"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        fontFamily: "Arial",
                        /**		
                        * Specifies the fontStyle of font in bar indicators.  See {@link pointerPlacement}
                        * @default [Array.Array.Array.enum] scales.indicators.font.fontStyle=ej.fontStyle.Bold
                        * @alias ejLinearGauge#scales->indicators->font->fontStyle
                        * @type {enum}
                        * @example 
                        * //To set fontStyle API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{font:{fontStyle: ej.fontStyle.Bold}}}});					 
                        * @example 
                        * //Get or set the fontStyle API, after initialization:<br>
                        *	//Gets the fontStyle  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].font.fontStyle;			
                        *	//Sets the fontStyle 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{font:{fontStyle: ej.fontStyle.Bold}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        fontStyle: "Bold"
                    },
                    /**		
					* Specifies the indicatorHeight of bar indicators
					* @default [Array.Array.number] scales.indicators.indicatorHeight=30
                    * @alias ejLinearGauge#scales->indicators->indicatorHeight
					* @type {number}
					* @example 
					* //To set indicatorHeight API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{indicatorHeight: 35}}}});					 
					* @example 
					* //Get or set the indicatorHeight API, after initialization:<br>
					*	//Gets the indicatorHeight  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].font.indicatorHeight;			
					*	//Sets the indicatorHeight 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{indicatorHeight: 35}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    indicatorHeight: 30,
                    /**		
					* Specifies the indicatorStyle of font in bar indicators
					* @default [Array.Array.Array.number] scales.indicators.font.indicatorStyle=ej.indicatorStyle.Rectangle
                    * @alias ejLinearGauge#scales->indicators->indicatorStyle
					* @type {number}
					* @example 
					* //To set indicatorStyle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{font:{indicatorStyle: ej.indicatorStyle.Rectangle}}}});					 
					* @example 
					* //Get or set the indicatorStyle API, after initialization:<br>
					*	//Gets the indicatorStyle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].font.indicatorStyle;			
					*	//Sets the indicatorStyle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{font:{indicatorStyle: ej.indicatorStyle.Rectangle}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    indicatorStyle: "Rectangle",
                    /**		
					* Specifies the indicatorWidth in bar indicators
					* @default [Array.Array.Array.number] scales.indicators.indicatorWidth=30
                    * @alias ejLinearGauge#scales->indicators->indicatorWidth
					* @type {number}
					* @example 
					* //To set indicatorWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{indicatorWidth: 25}}}});					 
					* @example 
					* //Get or set the indicatorWidth API, after initialization:<br>
					*	//Gets the indicatorWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].indicatorWidth;			
					*	//Sets the indicatorWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{indicatorWidth: 25}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    indicatorWidth: 30,
                    /**		
                * Specifies the location position in bar indicators
                * @default [Array.Array.Array.number] scales.indicators.location={x:0,y:0}
                * @alias ejLinearGauge#scales->indicators->location
                * @type {number}
                * @example 
                * //To set location API value during initialization  
                * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{location: {y:10}}}}});					 
                * @example 
                * //Get or set the location API, after initialization:<br>
                *	//Gets the location  
                *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // sindex - scale index value
                * // iindex -  indicators index value
                * var result = scale[sindex].indicators[bindex].location.y;			
                *	//Sets the location 
                *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{loction: {y:10}}}}]); 
                * @memberof ejLinearGauge
                * @instance
                */
                    location: {
                        /**		
                        * Specifies the x position in bar indicators
                        * @default [Array.Array.Array.number] scales.indicators.location.x=0
                        * @alias ejLinearGauge#scales->indicators->location->x
                        * @type {number}
                        * @example 
                        * //To set x API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{location: {x:10}}}}});					 
                        * @example 
                        * //Get or set the x API, after initialization:<br>
                        *	//Gets the x  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].location.x;			
                        *	//Sets the x 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{loction: {x:10}}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        x: 0,
                        /**		
                        * Specifies the y position in bar indicators
                        * @default [Array.Array.Array.number] scales.indicators.location.y=0
                        * @alias ejLinearGauge#scales->indicators->location->y
                        * @type {number}
                        * @example 
                        * //To set y API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{location: {y:10}}}}});					 
                        * @example 
                        * //Get or set the y API, after initialization:<br>
                        *	//Gets the y  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].location.y;			
                        *	//Sets the y 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{loction: {y:10}}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        y: 0
                    },
                    /**		
					* Specifies the textLocation in bar indicators
					* @default [Array.Array.object] scales.indicators.textLocation{x:0,y:0}
                    * @alias ejLinearGauge#scales->indicators->textLocation
					* @type {object}
					* @example 
					* //To set textLocation API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{textLocation: {x:0,y:10}}}}});					 
					* @example 
					* //Get or set the textLocation API, after initialization:<br>
					*	//Gets the textLocation  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].textLocation;			
					*	//Sets the textLocation 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{textLocation: {x:0,y:10}}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    textLocation: {
                        /**		
                        * Specifies the textLocation position in bar indicators
                        * @default [Array.Array.Array.number] scales.indicators.textLocation.x=0
                        * @alias ejLinearGauge#scales->indicators->textLocation->x
                        * @type {number}
                        * @example 
                        * //To set x API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{textLocation: {x:10}}}}});					 
                        * @example 
                        * //Get or set the x API, after initialization:<br>
                        *	//Gets the x  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].textLocation.x;			
                        *	//Sets the x 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{textLocation: {x:10}}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        x: 0,
                        /**		
                        * Specifies the Y position in bar indicators
                        * @default [Array.Array.Array.number] scales.indicators.textLocation.y=0
                        * @alias ejLinearGauge#scales->indicators->textLocation->y
                        * @type {number}
                        * @example 
                        * //To set y API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{textLocation: {y:10}}}}});					 
                        * @example 
                        * //Get or set the y API, after initialization:<br>
                        *	//Gets the y  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].textLocation.y;			
                        *	//Sets the y 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{textLocation: {y:10}}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        y: 0
                    },
                    /**		
					* Specifies the opacity in bar indicators
					* @default [Array.Array.number] scales.indicators.stateRanges
                    * @alias ejLinearGauge#scales->indicators->stateRanges
					* @type {number}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{stateRanges: {opacity:1}}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].stateRanges.opacity;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{stateRanges: {opacity:1}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    stateRanges: [{
                        /**		
                        * Specifies the endValue in bar indicators
                        * @default [Array.Array.Array.number] scales.indicators.stateRanges.endValue=60
                        * @alias ejLinearGauge#scales->indicators->stateRanges->endValue
                        * @type {number}
                        * @example 
                        * //To set endValue API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{stateRanges: {endValue:10}}}}});					 
                        * @example 
                        * //Get or set the endValue API, after initialization:<br>
                        *	//Gets the endValue  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].stateRanges.endValue;			
                        *	//Sets the endValue 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{stateRanges: {endValue:10}}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        endValue: 60,
                        /**		
                        * Specifies the startValue in bar indicators
                        * @default [Array.Array.Array.number] scales.indicators.stateRanges.startValue=50
                        * @alias ejLinearGauge#scales->indicators->stateRanges->startValue
                        * @type {number}
                        * @example 
                        * //To set startValue API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{stateRanges: {startValue:20}}}}});					 
                        * @example 
                        * //Get or set the startValue API, after initialization:<br>
                        *	//Gets the startValue  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].stateRanges.startValue;			
                        *	//Sets the startValue 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{stateRanges: {startValue:20}}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        startValue: 50,
                        /**		
                        * Specifies the backgroundColor in bar indicators
                        * @default [Array.Array.Array.string] scales.indicators.stateRanges.backgroundColor=null
                        * @alias ejLinearGauge#scales->indicators->stateRanges->backgroundColor
                        * @type {string}
                        * @example 
                        * //To set backgroundColor API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{stateRanges: {backgroundColor:"red"}}}}});					 
                        * @example 
                        * //Get or set the backgroundColor API, after initialization:<br>
                        *	//Gets the backgroundColor  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].stateRanges.backgroundColor;			
                        *	//Sets the backgroundColor 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{stateRanges: {backgroundColor:"red"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        backgroundColor: null,
                        /**		
                        * Specifies the borderColor in bar indicators
                        * @default [Array.Array.Array.string] scales.indicators.stateRanges.borderColor=null
                        * @alias ejLinearGauge#scales->indicators->stateRanges->borderColor
                        * @type {string}
                        * @example 
                        * //To set borderColor API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{stateRanges: {borderColor:"red"}}}}});					 
                        * @example 
                        * //Get or set the borderColor API, after initialization:<br>
                        *	//Gets the borderColor  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].stateRanges.borderColor;			
                        *	//Sets the borderColor 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{stateRanges: {borderColor:"red"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        borderColor: null,
                        /**		
                        * Specifies the text in bar indicators
                        * @default [Array.Array.Array.string] scales.indicators.stateRanges.text=null
                        * @alias ejLinearGauge#scales->indicators->stateRanges->text
                        * @type {string}
                        * @example 
                        * //To set text API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{stateRanges: {text:"welcome"}}}});					 
                        * @example 
                        * //Get or set the text API, after initialization:<br>
                        *	//Gets the text  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].stateRanges.text;			
                        *	//Sets the text 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{stateRanges: {text:"welcome"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        text: "",
                        /**		
                        * Specifies the textColor in bar indicators
                        * @default [Array.Array.Array.string] scales.indicators.stateRanges.textColor=null
                        * @alias ejLinearGauge#scales->indicators->stateRanges->textColor
                        * @type {string}
                        * @example 
                        * //To set textColor API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{stateRanges: {textColor:"red"}}}});					 
                        * @example 
                        * //Get or set the textColor API, after initialization:<br>
                        *	//Gets the textColor  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // iindex -  indicators index value
                        * var result = scale[sindex].indicators[bindex].stateRanges.textColor;			
                        *	//Sets the textColor 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{stateRanges: {textColor:"red"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        textColor: null
                    }],
                    /**		
					* Specifies the value in bar indicators
					* @default [Array.Array.number] scales.indicators.value=0
                    * @alias ejLinearGauge#scales->indicators->value
					* @type {number}
					* @example 
					* //To set value API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators: {value:0}}});					 
					* @example 
					* //Get or set the value API, after initialization:<br>
					*	//Gets the value  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].stateRanges.value;			
					*	//Sets the value 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators: {value:0}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    value: 0,
                    /**		
					* Specifies the backgroundColor in bar indicators
					* @default [Array.Array.string] scales.indicators.backgroundColor=null
                    * @alias ejLinearGauge#scales->indicators->backgroundColor
					* @type {string}
					* @example 
					* //To set backgroundColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators: {backgroundColor:"Blue"}}});					 
					* @example 
					* //Get or set the backgroundColor API, after initialization:<br>
					*	//Gets the backgroundColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].stateRanges.backgroundColor;			
					*	//Sets the backgroundColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators: {backgroundColor:"Blue"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    backgroundColor: null,
                    /**		
					* Specifies the borderColor in bar indicators
					* @default [Array.Array.string] scales.indicators.borderColor=null
                    * @alias ejLinearGauge#scales->indicators->borderColor
					* @type {string}
					* @example 
					* //To set borderColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators: {borderColor:"Blue"}}});					 
					* @example 
					* //Get or set the borderColor API, after initialization:<br>
					*	//Gets the borderColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].stateRanges.borderColor;			
					*	//Sets the borderColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators: {borderColor:"Blue"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    borderColor: null,
                    /**		
					* Specifies the borderWidth in bar indicators
					* @default [Array.Array.number] scales.indicators.borderWidth=1.5
                    * @alias ejLinearGauge#scales->indicators->borderWidth
					* @type {number}
					* @example 
					* //To set borderWidth API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators: {borderWidth:2}}}});					 
					* @example 
					* //Get or set the borderWidth API, after initialization:<br>
					*	//Gets the borderWidth  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].stateRanges.borderWidth;			
					*	//Sets the borderWidth 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators: {borderWidth:2}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    borderWidth: 1.5,
                    /**		
					* Specifies the opacity in bar indicators
					* @default [Array.Array.number] scales.indicators.opacity=NaN
                    * @alias ejLinearGauge#scales->indicators->opacity
					* @type {number}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{indicators:{stateRanges: {opacity:1}}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // iindex -  indicators index value
                    * var result = scale[sindex].indicators[bindex].stateRanges.opacity;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{indicators:{stateRanges: {opacity:1}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: NaN
                }],
                /**		
                * Specifies the  customLabel
                * @default [Array.object] scales.customLabel={font{size:"11px"}}
                * @alias ejLinearGauge#scales->customLabel
                * @type {object}
                * @example 
                * //To set font API value during initialization  
                * 	$("#LinearGauge1").ejLinearGauge({  scales:{customLabel: {font:{size: "11px"}}}}});					 
                * @example 
                * //Get or set the font API, after initialization:<br>
                *	//Gets the font  
                *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                * // sindex - scale index value
                * // cindex -  customLabel index value
                * var result = scale[sindex].customLabel[cindex].font;			
                *	//Sets the font 
                *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {font:{size: "11px"}}}]); 
                * @memberof ejLinearGauge
                * @instance
                */
                customLabel: [{
                    /**		
					* Specifies the font in  customLabel
					* @default [Array.Array.Array] scales.customLabel.font={size: "11px"}
                    * @alias ejLinearGauge#scales->customLabel->font
					* @type {object}
					* @example 
					* //To set font API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:{customLabel: {font:{size: "11px"}}}});					 
					* @example 
					* //Get or set the font API, after initialization:<br>
					*	//Gets the font  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // cindex -  customLabel index value
                    * var result = scale[sindex].customLabel[cindex].font;			
					*	//Sets the font 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {font:{size: "11px"}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    font: {
                        /**		
                        * Specifies the font size in  customLabel
                        * @default [Array.Array.Array.string] scales.customLabel.font.size="11px"
                        * @alias ejLinearGauge#scales->customLabel->font->size
                        * @type {string}
                        * @example 
                        * //To set size API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{customLabel: {font:{size: "11px"}}}});					 
                        * @example 
                        * //Get or set the size API, after initialization:<br>
                        *	//Gets the size  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // cindex -  customLabel index value
                        * var result = scale[sindex].customLabel[cindex].font.size;			
                        *	//Sets the size 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {font:{size: "11px"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        size: "11px",
                        /**		
                        * Specifies the fontFamily in  customLabel
                        * @default [Array.Array.Array.string] scales.customLabel.font.fontFamily="Arial"
                        * @alias ejLinearGauge#scales->customLabel->font->fontFamily
                        * @type {string}
                        * @example 
                        * //To set fontFamily API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{customLabel: {font:{fontFamily: "Arial"}}}});					 
                        * @example 
                        * //Get or set the fontFamily API, after initialization:<br>
                        *	//Gets the fontFamily  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // cindex -  customLabel index value
                        * var result = scale[sindex].customLabel[cindex].font.fontFamily;			
                        *	//Sets the fontFamily 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {font:{fontFamily: "Arial"}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        fontFamily: "Arial",
                        /**		
                        * Specifies the fontStyle in  customLabel. See {@link fontStyle}
                        * @default [Array.Array.Array.enum] scales.customLabel.font.fontStyle=ej.fontStyle.Bold
                        * @alias ejLinearGauge#scales->customLabel->font->fontStyle
                        * @type {enum}
                        * @example 
                        * //To set fontStyle API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales:{customLabel: {font:{fontStyle: ej.fontStyle.Normal}}}});					 
                        * @example 
                        * //Get or set the fontStyle API, after initialization:<br>
                        *	//Gets the fontStyle  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // cindex -  customLabel index value
                        * var result = scale[sindex].customLabel[cindex].font.fontStyle;			
                        *	//Sets the fontStyle 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {font:{fontStyle: ej.fontStyle.Normal}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        fontStyle: "Bold"
                    },
                    /**		
					* Specifies the labelColor in  customLabel
					* @default [Array.Array.number] scales.customLabel.labelColor=null
                    * @alias ejLinearGauge#scales->customLabel->labelColor
					* @type {number}
					* @example 
					* //To set labelColor API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales{customLabel: {labelColor:"Red"}}});					 
					* @example 
					* //Get or set the labelColor API, after initialization:<br>
					*	//Gets the labelColor  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // cindex -  customLabel index value
                    * var result = scale[sindex].customLabel[cindex].labelColor;			
					*	//Sets the labelColor 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {labelColor:"Red"}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    labelColor: null,
                    /**		
					* Specifies the opacity in  customLabel
					* @default [Array.Array.string] scales.customLabel.opacity=0
                    * @alias ejLinearGauge#scales->customLabel->opacity
					* @type {string}
					* @example 
					* //To set opacity API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales{customLabel: {opacity:0.5}}}});					 
					* @example 
					* //Get or set the opacity API, after initialization:<br>
					*	//Gets the opacity  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // cindex -  customLabel index value
                    * var result = scale[sindex].customLabel[cindex].opacity;			
					*	//Sets the opacity 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {opacity:0.5}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 0,
                    /**		
					* Specifies the labelValue in  customLabel
					* @default [Array.Array.string] scales.customLabel.labelValue=""
                    * @alias ejLinearGauge#scales->customLabel->labelValue
					* @type {string}
					* @example 
					* //To set labelValue API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales{customLabel: {labelValue:0.5}}});					 
					* @example 
					* //Get or set the labelValue API, after initialization:<br>
					*	//Gets the labelValue  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // cindex -  customLabel index value
                    * var result = scale[sindex].customLabel[cindex].labelValue;			
					*	//Sets the labelValue 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {labelValue:0.5}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    labelValue: "",
                    /**		
					* Specifies the textAngle in  customLabel
					* @default [Array.Array.number] scales.customLabel.textAngle=0
                    * @alias ejLinearGauge#scales->customLabel->textAngle
					* @type {number}
					* @example 
					* //To set textAngle API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales{customLabel: {textAngle:30}}});					 
					* @example 
					* //Get or set the textAngle API, after initialization:<br>
					*	//Gets the textAngle  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // cindex -  customLabel index value
                    * var result = scale[sindex].customLabel[cindex].textAngle;			
					*	//Sets the textAngle 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {textAngle:30}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    textAngle: 0,
                    /**		
					* Specifies the location in  customLabel
					* @default [Array.Array.object] scales.customLabel.location
                    * @alias ejLinearGauge#scales->customLabel->location
					* @type {object}
					* @example 
					* //To set location API value during initialization  
					* 	$("#LinearGauge1").ejLinearGauge({  scales{customLabel: {location:{x:10,y:10}}}});					 
					* @example 
					* //Get or set the location API, after initialization:<br>
					*	//Gets the location  
					*var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                    * // sindex - scale index value
                    * // cindex -  customLabel index value
                    * var result = scale[sindex].customLabel[cindex].location;			
					*	//Sets the location 
					*	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {location:{x:0}}}]); 
					* @memberof ejLinearGauge
					* @instance
					*/
                    location: {
                        /**		
                        * Specifies the location x in  customLabel
                        * @default [Array.Array.Array.number] scales.customLabel.location.x
                        * @alias ejLinearGauge#scales->customLabel->location->x
                        * @type {number}
                        * @example 
                        * //To set x API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales{customLabel: {location:{x:10}}}});					
                        * @example 
                        * //Get or set the x API, after initialization:<br>
                        *	//Gets the x  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // cindex -  customLabel index value
                        * var result = scale[sindex].customLabel[cindex].location.x;			
                        *	//Sets the x 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel: {location:{x:0}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        x: 0,
                        /**		
                        * Specifies the y in  customLabel
                        * @default [Array.Array.number] scales.customLabel.y=0
                        * @alias ejLinearGauge#scales->customLabel->location->y
                        * @type {number}
                        * @example 
                        * //To set y API value during initialization  
                        * 	$("#LinearGauge1").ejLinearGauge({  scales{customLabel: {location:{y:30}}}});					
                        * @example 
                        * //Get or set the y API, after initialization:<br>
                        *	//Gets the y  
                        *var scale = $("#LinearGauge").ejLinearGauge("option", "scales");
                        * // sindex - scale index value
                        * // cindex -  customLabel index value
                        * var result = scale[sindex].customLabel[cindex].location.y;			
                        *	//Sets the y 
                        *	$("#LinearGauge1").ejLinearGauge("option", "scales", [{customLabel:{location: {y:30}}}]); 
                        * @memberof ejLinearGauge
                        * @instance
                        */
                        y: 0
                    }
                }]
            };
        },

        dataTypes: {
            scales: "data"
        },
        observables: ["value", "minimum", "maximum"],
        value: ej.util.valueFunction("value"),
        minimum: ej.util.valueFunction("minimum"),
        maximum: ej.util.valueFunction("maximum"),

        // constructor function
        /**
         * To initiate the gauge		
		 * @private
         */
        _init: function () {
            this._initialize();
            this._trigger("load");
            this._setTheme();
            this._render();
            if (!this.model.readOnly)
                this.wireEvents();
            this._onWindowResize();
        },
        /**
         * Invoke when window size changes 
		 * @private
         */
        _onWindowResize: function () {
            if (this.model.canResize) {
                proxy = this;
                if (!this.isDevice())
                    this._on($(window), "resize", proxy.resizeCanvas);
                else
                    this._on($(window), "orientationchange", proxy.resizeCanvas);
            }
        },
        /**
         * To check whether it is a device or not
		 * @private
         */
        isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "theme": this.model.theme = options[option]; this._init(); break;
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "orientation": this.model.orientation = options[option]; break;
                    case "pointerGradient1": this.model.pointerGradient1 = options[option]; break;
                    case "pointerGradient2": this.model.pointerGradient2 = options[option]; break;
                    case "labelColor": this.model.labelColor = options[option]; break;
                    case "tickColor": this.model.tickColor = options[option]; break;
                    case "backgroundColor": this.model.backgroundColor = options[option]; break;
                    case "borderColor": this.model.borderColor = options[option]; break;
                    case "frameBackgroundImageUrl": this.model.frameBackgroundImageUrl = options[option]; break;
                    case "frameOuterWidth": this.model.frameOuterWidth = options[option]; break;
                    case "frameInnerWidth": this.model.frameInnerWidth = options[option]; break;
                    case "readOnly": this.model.readOnly = options[option]; break;
                    case "value":
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (j = 0; this.model.scales[i].markerPointers[j] != null; j++) {
                                this.model.scales[i].markerPointers[j].value = parseFloat(this.value());
                            }
                        }
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (j = 0; this.model.scales[i].barPointers[j] != null; j++) {
                                this.model.scales[i].barPointers[j].value = parseFloat(this.value());
                            }
                        }
                        break;
                    case "minimum":
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            this.model.scales[i].minimum = parseInt(this.minimum());
                        }
                        break;
                    case "maximum":
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            this.model.scales[i].maximum = parseInt(this.maximum());
                        }
                        break;
                    case "scales":
                        this.model.scales = options[option];
                        this._itemInitialize();
                        break;
                }
            }
            this._render();
        },
        /**
        * destroy the linear gauge
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="linearGauge1"&gt;div&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Create LinearGauge
		* var linearGaugeobj = $("#linearGauge1").data("ejButton");
		* linearGaugeobj.destroy(); // destroy the LinearGauge
		* &lt;/script&gt;
		* @example 
		* &lt;div id="button1"&gt;Lineargauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // enable the LinearGauge
		* $("#linearGauge1").ejButton("destroy");	
		* &lt;/script&gt;
		* @memberof ejLinearGauge
		* @instance
         */
        _destroy: function () {
            this.activeElement = null;
            this.canvasEl = null;
            this.contextEl = null;
            this.unWireEvents();
            this.element.remove();
        },
        /**
         * To initialize the gauge object
		 * @private
         */
        _initialize: function () {
            this.GaugeEl = this.element;
            this.scaleStartX = [];
            this.scaleStartY = [];
            this.isScaleModified = false;
            this.target = this.element[0];
            this._itemInitialize();
            this.Model = this.model;
        },
        /**
         * To render the gauge object
		 * @private
         */
        _render: function () {
            this.initialize();
        },
        /**
         * To Initialize the items
		 * @private
         */
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },
        /**
         * To check the each array element to load
		 * @private
         */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    innerElement = proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    innerElement = proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },
        /**
         * To Load individual default values
		 * @private
         */
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, Ele) {
                if (Ele instanceof Array) {
                    Ele = proxy._checkArrayObject(Ele, name);
                }
                else if (Ele != null && typeof Ele == "object") {
                    Ele = proxy._LoadIndividualDefaultValues(Ele, defaultObj, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
        /**
         * To Initialize the gauge
		 * @private
         */
        initialize: function () {
            this._initObject(this);

            if (this.Model.frameBackgroundImageUrl)
                this._drawCustomImage(this, this.Model.frameBackgroundImageUrl);
            else {
                if (this.Model.scales != null)
                    this._drawScales();
            }
        },
        /**
         * To Initialize the object
		 * @private
         */
        _initObject: function (element) {
            this.element.addClass("e-widget");
            element.GaugeEl = element.element;
            for (var i = 0; this.model.scales[i] != null; i++) {
                if (this.model.scales[i].minimum == null)
                    this.model.scales[i].minimum = this.minimum();
                if (this.model.scales[i].maximum == null)
                    this.model.scales[i].maximum = this.maximum();
                for (var j = 0; this.model.scales[i].markerPointers[j] != null; j++) {
                    if (this.model.scales[i].markerPointers[j].value == null)
                        this.model.scales[i].markerPointers[j].value = this.value();
                }
                for (var j = 0; this.model.scales[i].barPointers[j] != null; j++) {
                    if (this.model.scales[i].barPointers[j].value == null)
                        this.model.scales[i].barPointers[j].value = this.value();
                }
            }
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.GaugeEl.append(element.canvasEl);
            element.canvasEl.attr("role", "presentation");
            initialDivWidth = element.GaugeEl.width();
            initialDivHeight = element.GaugeEl.height();
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
        /**
         * To draw circular frame
		 * @private
         */
        _drawFrameCircle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.arc(location.startX, location.startY, style.circleRadius, 0, 2 * Math.PI, true);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                element._drawText(location, style);
        },
        /**
         * To draw Rectangular frame
		 * @private
         */
        _drawFrameRectangle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.moveTo(location.startX + style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width - style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.radius);
            element.contextEl.lineTo(location.startX + style.radius, location.startY + style.height);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                element._drawText(location, style);
        },
        /**
         * To draw Thermometer frame
		 * @private
         */
        _drawFrameThermometer: function (location, style, element) {
            var radius = element.Model.orientation == "Vertical" ? Math.sqrt(style.width * style.width + style.width * style.width) / 2 : Math.sqrt(style.height * style.height + style.height * style.height) / 2; //radius of the bottom circle
            this._contextOpenPath(style, element);
            if (element.Model.orientation == "Vertical") {
                if (element.scaleEl[element.scaleIndex].scaleDirection == "CounterClockwise") {
                    element.contextEl.arc(location.startX + Math.cos(Math.PI * (45 / 180)) * radius,
                                       location.startY + style.height - Math.sin(Math.PI * (45 / 180)) * radius,
                                       radius, Math.PI * (-45 / 180), Math.PI * (225 / 180), false);
                    element.contextEl.lineTo(location.startX, location.startY + style.width / 2);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width / 2, location.startY + style.width / 2, style.width / 2, -Math.PI, 0, false);
                    else
                        element.contextEl.lineTo(location.startX + style.width, location.startY + style.width / 2);
                }
                else {
                    element.contextEl.arc(location.startX + Math.cos(Math.PI * (45 / 180)) * radius,
                                       location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                                       radius, Math.PI * (45 / 180), Math.PI * (-225 / 180), true);
                    element.contextEl.lineTo(location.startX, location.startY + style.height - style.width / 2);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width / 2, location.startY + style.height - style.width / 2, style.width / 2, -Math.PI, 0, true);
                    else
                        element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.width / 2);
                }
            }
            else {
                if (element.scaleEl[element.scaleIndex].scaleDirection == "CounterClockwise") {
                    element.contextEl.arc(location.startX + style.width - radius / 4 - Math.cos(Math.PI * (45 / 180)) * radius,
                               location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                               radius, Math.PI * (135 / 180), Math.PI * (225 / 180), true);
                    element.contextEl.lineTo(location.startX + style.height / 2, location.startY);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.height / 2, location.startY + style.height / 2, style.height / 2, Math.PI * (270 / 180), Math.PI * (90 / 180), true);
                    else
                        element.contextEl.lineTo(location.startX + style.height / 2, location.startY + style.height);

                }
                else {
                    element.contextEl.arc(location.startX + radius / 4 + Math.cos(Math.PI * (45 / 180)) * radius,
                               location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                               radius, Math.PI * (45 / 180), Math.PI * (315 / 180), false);
                    element.contextEl.lineTo(location.startX + style.width - style.height / 2, location.startY);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width - style.height / 2, location.startY + style.height / 2, style.height / 2, Math.PI * (270 / 180), Math.PI * (90 / 180), false);
                    else
                        element.contextEl.lineTo(location.startX + style.width - style.height / 2, location.startY + style.height);
                }
            }
            this._contextClosePath(style, element);
        },
        /**
         * To draw RoundedRectangle frame
		 * @private
         */
        _drawFrameRoundedRectangle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.moveTo(location.startX + style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width - style.radius, location.startY);
            element.contextEl.quadraticCurveTo(location.startX + style.width, location.startY, location.startX + style.width, location.startY + style.radius);
            element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.radius);
            element.contextEl.quadraticCurveTo(location.startX + style.width, location.startY + style.height, location.startX + style.width - style.radius, location.startY + style.height);
            element.contextEl.lineTo(location.startX + style.radius, location.startY + style.height);
            element.contextEl.quadraticCurveTo(location.startX, location.startY + style.height, location.startX, location.startY + style.height - style.radius);
            element.contextEl.lineTo(location.startX, location.startY + style.radius);
            element.contextEl.quadraticCurveTo(location.startX, location.startY, location.startX + style.radius, location.startY);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                this._drawText(location, style);
        },
        /**
         * To styling context open path
		 * @private
         */
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },
        /**
         * To styling context Close path
		 * @private
         */
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },
        /**
         * To Draw scales
		 * @private
         */
        _drawScales: function () {
            var self = this;
            this.scaleEl = this.Model.scales;
            this.contextEl.save();
            this.contextEl.translate(this.Model.frameOuterWidth + this.Model.frameInnerWidth, this.Model.frameOuterWidth + this.Model.frameInnerWidth);
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                self._setScaleCordinates(element, element.scaleStyle);
            });
            this._setTicks();
            this._setLabels();
            this._setRange();
            this._setCustomLabel();
            this._setIndicators();
            this._setBarPointers();
            this._setMarkerPointers();
            $.each(this.Model.scales, function (ind, elemt) {
                if (elemt.showBarPointers) {
                    if (elemt.barPointers.length > 1)
                        self.model.animate = false;
                }
                if (elemt.showMarkerPointers) {
                    if (elemt.markerPointers.length > 1)
                        self.model.animate = false;
                }
            });
            if (!this.contextEl.putImageData)
                this.model.animate = false;
            if (this.model.animationSpeed != null && this.model.animationSpeed > 0) {
                var delay = this.model.animationSpeed / 25;
                if (delay >= 0) {
                    if (this.model.animate) {
                        this._onAnimate(delay);
                    }
                }
            }
        },
        /**
         * To set ticks coordinates
		 * @private
         */
        _setTicks: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showTicks) {
                    self.scaleIndex = index;
                    if (element.ticks != null) {
                        self.tickEl = element.ticks;
                        $.each(element.ticks, function (index, element) {
                            self.tickIndex = index;
                            self._setTicksCordinates(element, index);
                        });
                    }
                }
            });
        },
        /**
         * To set Label coordinates
		 * @private
         */
        _setLabels: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showLabels) {
                    self.scaleIndex = index;
                    if (element.labels != null) {
                        self.labelEl = element.labels;
                        $.each(element.labels, function (index, element) {
                            self.labelIndex = index;
                            self._setLabelCordinates(element, index);
                        });
                    }
                }
            });
        },
        /**
         * To set Indicator coordinates
		 * @private
         */
        _setIndicators: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.indicators != null && element.showIndicators) {
                    self.indicatorEl = element.indicators;
                    $.each(element.indicators, function (index, element) {
                        self.indicatorIndex = index;
                        self._drawIndicator(index, element);
                    });
                }
            });
        },
        /**
         * To set Bar Pointer coordinates
		 * @private
         */
        _setBarPointers: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showBarPointers) {
                    self.scaleIndex = index;
                    if (element.barPointers != null) {
                        self.barPointerEl = element.barPointers;
                        $.each(element.barPointers, function (index, element) {
                            self.barPointerIndex = index;
                            self._drawScaleBarPointer(element, index);
                        });
                    }
                }
            });
        },
        /**
         * To set Marker Pointer coordinates
		 * @private
         */
        _setMarkerPointers: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showMarkerPointers) {
                    self.scaleIndex = index;
                    if (element.markerPointers != null) {
                        self.markerPointerEl = element.markerPointers;
                        $.each(element.markerPointers, function (index, element) {
                            self.markerPointerIndex = index;
                            self._drawMarkerPointer(element, index);
                            self.canvasEl.attr("aria-label", self.model.scales[self.scaleIndex].markerPointers[self.markerPointerIndex].value);
                        });
                    }
                }
            });
        },
        /**
         * To set delay value
		 * @private
         */
        _onAnimate: function (delay) {
            var self = this, timer, timer1;
            var currentValue = self.model.scales[0].minimum;
            var barPointerValue = self.model.scales[0].barPointers[0].value;
            var markerPointerValue = self.model.scales[0].markerPointers[0].value;
            timer = setInterval(function pointerchan() {
                if (barPointerValue > currentValue) {
                    currentValue = currentValue + 1;
                    self.setBarPointerValue(0, 0, currentValue);
                }
                else if (currentValue == barPointerValue) {
                    window.clearInterval(timer);
                }
            }, delay);
            timer1 = setInterval(function pointerchan() {
                if (markerPointerValue > currentValue) {
                    currentValue = currentValue + 1;
                    self.setPointerValue(0, 0, currentValue);
                }
                else if (currentValue == markerPointerValue) {
                    window.clearInterval(timer1);
                }
            }, delay);
        },
        /**
         * To set range coordinates
		 * @private
         */
        _setRange: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.ranges != null && element.showRanges) {
                    self.rangeEl = element.ranges;
                    $.each(element.ranges, function (index, element) {
                        self.rangeIndex = index;
                        self._drawRange(element);
                    });
                }
            });
        },
        /**
         * To set CustomLabel coordinates
		 * @private
         */
        _setCustomLabel: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.customLabel != null && element.showCustomLabel) {
                    self.customLabelEl = element.customLabel;
                    $.each(element.customLabel, function (index, element) {
                        self.customLabelIndex = index;
                        self._setCustomLabelCordinates(index, element);
                    });
                }
            });
        },
        /**
         * To set ScaleCordinates
		 * @private
         */
        _setScaleCordinates: function (element, scaleStyle) {
            var bottomRadius, location, style, radius;
            this.opacity = 1;
            this.bottomRadius = Math.sqrt(element.scaleBarSize * element.scaleBarSize + element.scaleBarSize * element.scaleBarSize) / 2;

            this.bounds = {
                height: this.canvasEl[0].height - 2 * (this.Model.frameOuterWidth + this.Model.frameInnerWidth),
                width: this.canvasEl[0].width - 2 * (this.Model.frameOuterWidth + this.Model.frameInnerWidth)
            };
            if (this.Model.orientation == "Vertical") {
                this.scaleStartX[this.scaleIndex] = (this.bounds.width - element.scaleBarSize) * ((element.location.x) / 100);
                this.scaleStartY[this.scaleIndex] = (this.bounds.height - element.scaleBarLength) * ((element.location.y) / 100);
            }
            else {
                this.scaleStartX[this.scaleIndex] = (this.bounds.width - element.scaleBarLength) * ((element.location.x) / 100);
                this.scaleStartY[this.scaleIndex] = (this.bounds.height - element.scaleBarSize) * ((element.location.y) / 100);
            }
            radius = scaleStyle == "RoundedRectangle" ? 5 : 0;
            location = { "startX": this.scaleStartX[this.scaleIndex], "startY": this.scaleStartY[this.scaleIndex] };
            style = {
                "width": this.Model.orientation == "Vertical" ? element.scaleBarSize : element.scaleBarLength, "isStroke": true,
                "topRounded": true,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((this.Model.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.backgroundColor)),
                "lineWidth": element.borderWidth,
                "radius": radius,
                "height": this.Model.orientation == "Vertical" ? element.scaleBarLength : element.scaleBarSize,
                "isStroke": true,
                "isFill": true,
                "strokeStyle": element.borderColor ? ((element.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.borderColor)) : ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor))
            };
            this.minimum(element.minimum);
            this.maximum(element.maximum);
            if (element.shadowOffset) {
                this.contextEl.shadowBlur = element.shadowOffset;
                this.contextEl.shadowColor = (style.fillStyle == "transparent") ? "rgba(0,0,0,0)" : style.fillStyle;
            }
            this["_drawFrame" + scaleStyle](location, style, this);
            if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer" && !this.isScaleModified) {
                this._modifyWidth();
                this.isScaleModified = true;
            }
            if (this.contextEl.getImageData)
                this.scaleImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
        /**
         * To set Ticks Cordinates
		 * @private
         */
        _setTicksCordinates: function (element, index) {
            var location, style, staticPosition, height, tempVal, lineChangePosition, interval;
            if (this.scaleEl[this.scaleIndex].majorIntervalValue > this.scaleEl[this.scaleIndex].minorIntervalValue) {
                interval = element.tickStyle == "MajorInterval" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
                if (element.tickPlacement == "Near") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] : this.scaleStartY[this.scaleIndex];
                }
                else if (element.tickPlacement == "Far") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize;
                }
                else if (element.tickPlacement == "Center") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2;
                }
                height = element.tickPlacement == "Near" ? -element.tickHeight : element.tickHeight;
                for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value -= interval) {
                    if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                        lineChangePosition = this._getClockwiseLinePosition(value);
                        this.location = {
                            "lineChangePosition": lineChangePosition + (this.Model.orientation == "Horizontal" ? element.xDistanceFromScale : (element.yDistanceFromScale)),
                            "lineStaticPosition": staticPosition + (this.Model.orientation == "Horizontal" ? element.yDistanceFromScale : (element.xDistanceFromScale))
                        };
                        this.style = {
                            "lineHeight": height,
                            "angle": this.Model.orientation == "Vertical" ? element.angle : element.angle + 270,
                            "tickShape": element.TickShape,
                            "strokeStyle": element.tickColor ? ((element.tickColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.tickColor)) : ((this.Model.tickColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.tickColor)),
                            "lineWidth": element.tickWidth
                        }
                        if (this.Model.drawTicks)
                            this._onDrawTicks(this.Model.orientation == "Vertical" ? element.angle : element.angle + 270, value);
                        this._drawTickMark(this.location, this.style);
                    }
                }

                if (this.contextEl.getImageData)
                    this.tickImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
            }
        },
        /**
         * To draw Ticks Mark
		 * @private
         */
        _drawTickMark: function (location, style) {
            var length = this.Model.orientation == "Vertical" ? style.lineHeight - location.lineStaticPosition : location.lineStaticPosition;
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.lineWidth = style.lineWidth;
            this.contextEl.strokeStyle = style.strokeStyle;
            if (this.Model.orientation == "Vertical")
                this.contextEl.translate(location.lineStaticPosition, location.lineChangePosition);
            else
                this.contextEl.translate(location.lineChangePosition, location.lineStaticPosition);
            this.contextEl.moveTo(0, 0);
            if (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise")
                this.contextEl.rotate((Math.PI * (style.angle / 180)));
            else
                this.contextEl.rotate(-(Math.PI * (style.angle / 180)));
            this.contextEl.lineTo(style.lineHeight, 0);
            this.contextEl.stroke();
            this.contextEl.restore();
            this.contextEl.closePath();
        },
        /**
         * To manupulate decimal values
		 * @private
         */
        _addDecimal: function (_value, _interval) {
            var value = _value.toString();
            var interval = _interval.toString();
            var vDecimal;
            var iDecimal;
            if (value.indexOf('.') > -1)
                vDecimal = value.length - value.indexOf('.') - 1;
            else
                vDecimal = 0;
            if (interval.indexOf('.') > -1)
                iDecimal = interval.length - interval.indexOf('.') - 1;
            else
                iDecimal = 0;
            var Decimal = vDecimal > iDecimal ? vDecimal : iDecimal
            var correctValue = (_value * Math.pow(10, Decimal) + _interval * Math.pow(10, Decimal)) / Math.pow(10, Decimal);
            return correctValue;
        },
        /**
         * To set label coordinates
		 * @private
         */
        _setLabelCordinates: function (element, index) {

            var location, style, xDistanceFromScale, minimumMeet, yDistanceFromScale, staticPosition, tempVal, height, lineChangePosition, textValue, labelLocation, labelStyle, interval;
            if (this.scaleEl[this.scaleIndex].majorIntervalValue > this.scaleEl[this.scaleIndex].minorIntervalValue) {

                if (this.Model.orientation == "Vertical") {
                    xDistanceFromScale = element.xDistanceFromScale;
                    yDistanceFromScale = element.yDistanceFromScale;
                }
                else {
                    xDistanceFromScale = element.yDistanceFromScale;
                    yDistanceFromScale = element.xDistanceFromScale;
                }
                interval = element.labelStyle == "Major" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
                if (element.labelPlacement == "Near") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] - this.scaleEl[this.scaleIndex].borderWidth / 2 : this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].borderWidth - 5;
                    this.contextEl.textAlign = this.Model.orientation == "Vertical" ? "right" : "center";
                }
                else if (element.labelPlacement == "Far") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize + this.scaleEl[this.scaleIndex].borderWidth / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize + this.scaleEl[this.scaleIndex].borderWidth + 5;
                    this.contextEl.textAlign = this.Model.orientation == "Vertical" ? "left" : "center"
                }
                else {
                    this.contextEl.textAlign = "center";
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 + this.scaleEl[this.scaleIndex].borderWidth / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 + this.scaleEl[this.scaleIndex].borderWidth / 2;
                }
                for (var value = this.scaleEl[this.scaleIndex].minimum; value <= this.scaleEl[this.scaleIndex].maximum; value = this._addDecimal(value, interval)) {
                    if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                        lineChangePosition = this.scaleEl[this.scaleIndex].scaleDirection == "CounterClockwise" ? this._getCounterClockwiseLinePosition(value) : this._getClockwiseLinePosition(value);
                        this.labelValue = value;
                        this.location = {
                            "lineChangePosition": lineChangePosition + (yDistanceFromScale),
                            "lineStaticPosition": staticPosition + (xDistanceFromScale)
                        };
                        this.style = {
                            "angle": this.Model.orientation == "Vertical" ? element.angle : element.angle + 270,
                            "fillStyle": element.textColor ? ((element.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.textColor)) : ((this.Model.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.labelColor)), "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                            "font": this._getFontString(this, element.font),
                            "textValue": this.labelValue
                        };

                        if (this.Model.drawLabels)
                            this._onDrawLabels(this.Model.orientation == "Vertical" ? element.angle : element.angle + 270);
                        this._drawLabel(this.location, this.style, false);
                    }
                }
            }

            if (this.contextEl.getImageData)
                this.labelImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);

        },
        /**
         * To draw Label
		 * @private
         */
        _drawLabel: function (location, style, isCustomLabel) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.textBaseline = "middle";
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.font = style.font;
            if (style.opacity)
                this.contextEl.globalAlpha = style.opacity;
            if (this.Model.orientation == "Vertical")
                this.contextEl.translate(location.lineStaticPosition, location.lineChangePosition);
            else
                this.contextEl.translate(location.lineChangePosition, location.lineStaticPosition);
            this.contextEl.moveTo(0, 0);
            if (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise")
                this.contextEl.rotate((Math.PI * (style.angle / 180)));
            else
                this.contextEl.rotate(-(Math.PI * (style.angle / 180)));
            if (!ej.isNullOrUndefined(isCustomLabel) && !isCustomLabel) {
                var textPostion = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitTextPosition;
                if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "Back")
                    style.textValue = style.textValue + this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText;
                else if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "Front")
                    style.textValue = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText + style.textValue;
            }
            this.contextEl.fillText(style.textValue, 0, 0);
            this.contextEl.fill();
            this.contextEl.restore();

        },
        /**
         * To draw Scale bar pointer
		 * @private
         */
        _drawScaleBarPointer: function (element, index) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var grd, location, style, radius, height, width, startY, backgroundColor, startX, gradient;
            gradient = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2 }];
            radius = this.scaleEl[this.scaleIndex].scaleStyle == "RoundedRectangle" ? 5 : 0;
            if (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise") {
                lineYPosition = this._getClockwiseLinePosition(element.value);
                if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer" && this.isScaleModified) {
                    this._restoreWidth();
                    this.isModify = true;
                }
                if (this.Model.orientation == "Vertical") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerWidth / 2 + element.distanceFromScale;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex], startX + element.pointerWidth, this.scaleStartY[this.scaleIndex]);
                }
                else {
                    startX = this.scaleStartX[this.scaleIndex];
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerWidth / 2, startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 + element.pointerWidth / 2);
                }
            }
            else {
                lineYPosition = this._getCounterClockwiseLinePosition(element.value);
                if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer" && this.isScaleModified) {
                    this._restoreWidth();
                    this.isModify = true;
                }
                if (this.Model.orientation == "Vertical") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerWidth / 2 + element.distanceFromScale;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarLength - lineYPosition, startX + element.pointerWidth, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarLength - lineYPosition);
                }
                else {
                    startX = this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer" ? lineYPosition - this.scaleEl[this.scaleIndex].scaleBarSize / 2 - this.scaleEl[this.scaleIndex].borderWidth / 2 : lineYPosition - this.scaleEl[this.scaleIndex].borderWidth / 2;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerWidth / 2, startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 + element.pointerWidth / 2);
                }
            }
            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.scaleBarGradient)
                this._setGradientColor(this, grd, element.scaleBarGradient.ColorInfo);
            else if (this.Model.ScaleInterior)
                this._setGradientColor(this, grd, this.Model.ScaleInterior.ColorInfo);
            else
                this._setGradientColor(this, grd, gradient);
            if (this.Model.orientation == "Vertical") {
                startY = this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise" ? this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].borderWidth / 2 : lineYPosition;
                if (this.scaleEl[this.scaleIndex].scaleDirection == "CounterClockwise" && this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                    startY = startY - this.scaleEl[this.scaleIndex].scaleBarSize / 2;
                height = this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise" ? lineYPosition - this.scaleStartY[this.scaleIndex] : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarLength - lineYPosition - this.scaleEl[this.scaleIndex].borderWidth / 2;
                width = element.pointerWidth;
            }
            else {
                startY = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerWidth / 2 + this.scaleEl[this.scaleIndex].borderWidth / 2 + element.distanceFromScale;
                height = element.pointerWidth;
                width = this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise" ? lineYPosition - this.scaleStartX : this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarLength - lineYPosition;
            }
            this.location = { "startX": startX + this.scaleEl[this.scaleIndex].borderWidth / 2, "startY": startY };
            this.style = {
                "width": (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer" && this.Model.orientation == "Horizontal") ? width + height / 2 - this.scaleEl[this.scaleIndex].borderWidth / 2 : width,
                "lineWidth": element.borderWidth,
                "radius": radius,
                "topRounded": element.value == this.scaleEl[this.scaleIndex].maximum ? true : false,
                "isStroke": false,
                "isFill": true,
                "height": (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer" && this.Model.orientation == "Vertical") ? height + width / 2 : height,
                "strokeStyle": element.borderColor == null ? ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor)) : ((element.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.borderColor)),
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd)),
                "opacity": isNaN(element.opacity) ? 0.4 : element.opacity
            };
            this.value(element.value);
            if (this.Model.drawBarPointers)
                this._onDrawBarPointers(element.value);
            this["_drawFrame" + this.scaleEl[this.scaleIndex].scaleStyle](this.location, this.style, this);
            if (this.contextEl.getImageData)
                this.barPointerImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
        /**
         * To draw Marker Pointer
		 * @private
         */
        _drawMarkerPointer: function (element, index) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var location, style, startX, startY, radius, linePosition, angle, grd, backgroundColor;
            var gradient = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2 }];
            this.markerPlacement = element.pointerPlacement;
            radius = Math.sqrt(element.pointerWidth * element.pointerWidth + element.pointerLength * element.pointerLength) / 2;
            if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer" && this.isModify)
                this._modifyWidth();
            if (this.Model.orientation == "Vertical") {
                if (this.markerPlacement == "Far") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize + this.scaleEl[this.scaleIndex].borderWidth / 2 + element.distanceFromScale;
                    angle = 0;
                }
                if (this.markerPlacement == "Near") {
                    if (element.markerStyle == "Star")
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale - element.pointerWidth;
                    else
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    angle = 180;
                }
                if (this.markerPlacement == "Center") {
                    if (element.markerStyle == "Circle")
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerWidth / 2 + element.distanceFromScale + (element.borderWidth);
                    else
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerWidth / 2 + element.distanceFromScale;
                    angle = 0;
                }
            }
            else {
                if (this.markerPlacement == "Far") {
                    startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize + this.scaleEl[this.scaleIndex].borderWidth / 2 + element.distanceFromScale;
                    angle = 90;
                }
                if (this.markerPlacement == "Near") {
                    if (element.markerStyle == "Star")
                        startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].borderWidth / 2 + element.distanceFromScale - element.pointerLength;
                    else
                        startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].borderWidth / 2 + element.distanceFromScale;
                    angle = 270;
                }
                if (this.markerPlacement == "Center") {
                    if (element.markerStyle == "Circle")
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerLength / 2 + element.distanceFromScale + (element.borderWidth);
                    else
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerLength / 2 + element.distanceFromScale;
                    angle = 90;
                }
            }
            linePosition = this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise" ? this._getClockwiseLinePosition(element.value) : this._getCounterClockwiseLinePosition(element.value);
            if (element.markerStyle == "Star") {
                if (this.Model.orientation == "Vertical") {
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex], startX + element.pointerWidth, this.scaleStartY[this.scaleIndex]);
                    startY = linePosition - element.pointerLength / 3;
                }
                else {
                    grd = this.contextEl.createLinearGradient(linePosition, startX, linePosition, startX + element.pointerLength);
                    startY = linePosition - element.pointerWidth / 2;
                }
            }
            else {
                grd = this.contextEl.createLinearGradient(0, 0, element.pointerWidth, 0);
                startY = linePosition;
            }
            if (element.markerStyle == "RoundedRectangle") {
                if (this.Model.orientation == "Vertical" && this.markerPlacement == "Near")
                    startY += element.pointerLength;
                else if (this.Model.orientation == "Horizontal") {
                    if (this.markerPlacement == "Near")
                        startY -= (element.pointerWidth);
                    startX += element.pointerWidth / 2;
                }
            }
            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.pointerGradient)
                this._setGradientColor(this, grd, element.pointerGradient.ColorInfo);
            else if (this.Model.PointerInterior)
                this._setGradientColor(this, grd, this.Model.PointerInterior.ColorInfo);
            else
                this._setGradientColor(this, grd, gradient);

            this.location = { "startX": this.Model.orientation == "Vertical" ? startX : startY, "startY": this.Model.orientation == "Vertical" ? startY : startX };
            this.style = {
                "width": element.pointerWidth,
                "radius": element.markerStyle == "Rectangle" ? 0 : radius,
                "height": element.pointerLength,
                "lineWidth": element.borderWidth,
                "isFill": true,
                "isStroke": true,
                "angle": angle,
                "strokeStyle": ((element.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.borderColor)),
                "markerPlacement": this.markerPlacement,
                "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd))
            };
            this.value(element.value);
            if (this.Model.drawMarkerPointers)
                this._onDrawMarkerPointers(angle, element.value);
            if (element.markerStyle == "RoundedRectangle")
                this.style.radius = 5;
            this["_draw" + element.markerStyle](this.location, this.style, this);
            if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer" && this.isModify) {
                this._restoreWidth();
                this.isScaleModified = false;
            }
            if (this.contextEl.getImageData)
                this.markerPointerImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
        /**
         * To draw Range
		 * @private
         */
        _drawRange: function (element) {
            this.rangePosition = element.rangePosition;
            var startLinePosition, endLinePosition, location, style, startX, startY, grd, backgroundColor;
            var gradient = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2 }];
            startLinePosition = (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise") ? this._getClockwiseLinePosition(element.startValue) : this._getCounterClockwiseLinePosition(element.startValue);
            endLinePosition = (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise") ? this._getClockwiseLinePosition(element.endValue) : this._getCounterClockwiseLinePosition(element.endValue);
            if (this.Model.orientation == "Vertical") {
                if (element.rangePosition == "Far")
                    startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].scaleBarSize + this.scaleEl[this.scaleIndex].borderWidth;
                if (element.rangePosition == "Near")
                    startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                if (element.rangePosition == "Center")
                    startX = (element.startWidth > element.endWidth) ? this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.startWidth / 2 : this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.endWidth / 2;
                grd = this.contextEl.createLinearGradient(endLinePosition, endLinePosition, endLinePosition, startLinePosition);
                this.location = { "startX": startX, "startY": startLinePosition, "endY": endLinePosition };
            }
            else {
                if (element.rangePosition == "Far")
                    startY = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize + element.distanceFromScale + this.scaleEl[this.scaleIndex].borderWidth;
                if (element.rangePosition == "Near")
                    startY = this.scaleStartY[this.scaleIndex] + element.distanceFromScale;
                if (element.rangePosition == "Center")
                    startY = (element.startWidth > element.endWidth) ? this.scaleStartY[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.startWidth / 2 : this.scaleStartY[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.endWidth / 2;
                grd = this.contextEl.createLinearGradient(endLinePosition, startY, startLinePosition, startY);
                this.location = { "startX": startLinePosition, "startY": startY, "endX": endLinePosition };
            }

            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.rangeGradient)
                this._setGradientColor(this, grd, element.rangeGradient.ColorInfo);
            else if (this.Model.RangeInterior)
                this._setGradientColor(this, grd, this.Model.RangeInterior.ColorInfo);
            else
                this._setGradientColor(this, grd, gradient);

            this.style = {
                "startWidth": element.startWidth, "lineWidth": element.borderWidth,
                "isStoke": true, "isFill": true, "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "endWidth": element.endWidth,
                "fillStyle": element.backgroundColor ? ((backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd)),
                "strokeStyle": element.borderColor ? ((element.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.borderColor)) : ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor))
            };
            if (this.Model.drawRange)
                this._onDrawRange();
            this._drawRangeBar(this.location, this.style);
            if (this.contextEl.getImageData)
                this.rangeImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
        /**
         * To draw Range bar
		 * @private
         */
        _drawRangeBar: function (location, style) {
            this._contextOpenPath(style, this);
            if (this.Model.orientation == "Vertical") {
                this.contextEl.moveTo(location.startX, location.startY);
                this.contextEl.lineTo(location.startX, location.endY);
                if (this.rangePosition == "Near") {
                    this.contextEl.lineTo(location.startX - style.endWidth, location.endY);
                    this.contextEl.lineTo(location.startX - style.startWidth, location.startY);
                }
                else {
                    this.contextEl.lineTo(location.startX + style.endWidth, location.endY);
                    this.contextEl.lineTo(location.startX + style.startWidth, location.startY);
                }
            }
            else {
                this.contextEl.moveTo(location.startX, location.startY);
                this.contextEl.lineTo(location.endX, location.startY);
                if (this.rangePosition == "Near") {
                    this.contextEl.lineTo(location.endX, location.startY - style.endWidth);
                    this.contextEl.lineTo(location.startX, location.startY - style.startWidth);
                }
                else {
                    this.contextEl.lineTo(location.endX, location.startY + style.endWidth);
                    this.contextEl.lineTo(location.startX, location.startY + style.startWidth);
                }
            }
            this._contextClosePath(style, this);
        },
        /**
         * To set Custom Label Cordinates
		 * @private
         */
        _setCustomLabelCordinates: function (index, element) {
            var location, style, startX, startY;
            this.contextEl.textAlign = "center";
            if (this.Model.orientation == "Vertical") {
                startX = (this.bounds.width) * ((element.location.x) / 100);
                startY = (this.bounds.height) * ((element.location.y) / 100);
            }
            else {
                startX = (this.bounds.width) * ((element.location.x) / 100);
                startY = (this.bounds.height) * ((element.location.y) / 100);
            }
            this.location = { "lineStaticPosition": this.Model.orientation == "Vertical" ? startX : startY, "lineChangePosition": this.Model.orientation == "Vertical" ? startY : startX }
            this.style = { "angle": element.textAngle, "textValue": element.labelValue, "fillStyle": element.labelColor ? ((element.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.labelColor)) : ((this.Model.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.labelColor)), "font": element.font };
            if (this.Model.drawCustomLabel)
                this._onDrawCustomLabel();
            this._drawLabel(this.location, this.style, true);

            if (this.contextEl.getImageData)
                this.customLabelImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
        /**
         * To draw Indicator
		 * @private
         */
        _drawIndicator: function (index, element) {
            var self = this, xlocation, ylocation, txtLocation, txtLocationX, txtLocationY, isInStateRange = false;
            xlocation = (this.bounds.width - 2 * element.indicatorWidth) * ((element.location.x) / 100);
            ylocation = (this.bounds.height - 2 * element.indicatorHeight) * ((element.location.y) / 100);
            txtLocation = { "x": (this.bounds.width) * ((element.textLocation.x) / 100), "y": (this.bounds.height) * ((element.textLocation.y) / 100) };
            self.location = { "startX": element.indicatorStyle == "Circle" ? xlocation + element.indicatorWidth : xlocation, "textLocation": txtLocation, "startY": element.indicatorStyle == "Circle" ? ylocation + element.indicatorHeight : ylocation, "startAngle": 0, "endAngle": 2 * Math.PI };
            self.style = {
                "radius": element.indicatorStyle == "RoundedRectangle" ? 2 : 0,
                "strokeStyle": element.borderColor ? ((element.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.borderColor)) : this._getColor(element, "#FFFFFF"),
                "angle": 0,
                "circleRadius": (element.indicatorHeight + element.indicatorWidth) / 2,
                "height": element.indicatorHeight,
                "width": element.indicatorWidth,
                "lineWidth": element.borderWidth,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : this._getColor(element, "#FFFFFF"),
                "isStroke": true,
                "isFill": true,
                "indicatorText": null,
                "textColor": null,
                "font": null,
                "counterClockwise": false
            };
            if (this.Model.drawIndicators)
                this._onDrawIndicators(self.style, self.location);
            if (element.stateRanges != null) {
                $.each(element.stateRanges, function (index, srEl) {//srEl = StateRange Element
                    if (element.value >= srEl.startValue && element.value <= srEl.endValue) {
                        isInStateRange = true;
                        if (srEl.text.length > 0) {
                            self.style.indicatorText = srEl.text;
                            self.style.textColor = ((srEl.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, srEl.textColor));
                            self.style.font = this._getFontString(self, element.font);
                        }
                        self.style.strokeStyle = ((srEl.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, srEl.borderColor));
                        self.style.fillStyle = ((srEl.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, srEl.backgroundColor));
                        this["_drawFrame" + element.indicatorStyle](self.location, self.style, self);
                    }
                });
            }
            if (!isInStateRange)
                this["_drawFrame" + element.indicatorStyle](self.location, self.style, self);
            if (this.contextEl.getImageData)
                this.indicatorImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
        /**
         * To draw the text in the gauge
		 * @private
         */
        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.textAlign = "center";
            this.contextEl.fillStyle = ((style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor);
            this.contextEl.font = style.font;
            this.contextEl.fillText(style.indicatorText, location.textLocation.x, location.textLocation.y);
            this.contextEl.closePath();
        },
        /**
         * To draw the triangle
		 * @private
         */
        _drawTriangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Pointer
		 * @private
         */
        _drawPointer: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(style.width / 2, style.height / 4);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Wedge
		 * @private
         */
        _drawWedge: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(3 * style.width / 4, 0);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Slider
		 * @private
         */
        _drawSlider: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 4, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(style.width / 4, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Star
		 * @private
         */
        _drawStar: function (location, style, element) {
            this._contextOpenPath(style, element);
            if (element.Model.orientation == "Horizontal" && element.markerPlacement == "Near") {
                element.contextEl.moveTo(location.startX + style.width - (style.width / 6), location.startY); // 1
                element.contextEl.lineTo(location.startX, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width / 6, location.startY);
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY + style.height);
            }
            else {
                element.contextEl.moveTo(location.startX + (style.width / 6), location.startY + style.height); // 1
                element.contextEl.lineTo(location.startX + style.width, location.startY + (style.height / 3)); // 2
                element.contextEl.lineTo(location.startX, location.startY + (style.height / 3)); // 3
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY + style.height); // 4
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY); // 5
            }
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Pentagon
		 * @private
         */
        _drawPentagon: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 3, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width / 3, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Diamond
		 * @private
         */
        _drawDiamond: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(style.width, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(0, 0);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Circle
		 * @private
         */
        _drawCircle: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Trapezoid
		 * @private
         */
        _drawTrapezoid: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 4);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Rectangle
		 * @private
         */
        _drawRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 2);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Rounded Rectangle
		 * @private
         */
        _drawRoundedRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY - style.height / 2);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.radius, 0);
            element.contextEl.lineTo(style.width - style.radius, 0);
            element.contextEl.quadraticCurveTo(style.width, 0, style.width, style.radius);
            element.contextEl.lineTo(style.width, style.height - style.radius);
            element.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.radius, style.height);
            element.contextEl.lineTo(style.radius, style.height);
            element.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.radius);
            element.contextEl.lineTo(0, style.radius);
            element.contextEl.quadraticCurveTo(0, 0, style.radius, 0);
            this._contextClosePath(style, element);
        },
        /**
         * To draw the Custom image 
		 * @private
         */
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            var self = this;
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.Model.width, element.Model.height);
                if (element.Model.scales != null)
                    element._drawScales();
                if (element.Model.Items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
        /**
         * To draw the Ellipse
		 * @private
         */
        _drawEllipse: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.scale(2, 1);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
        //Client-Side Methods
        /**
         * To get Indicator Image
		 * @private
         */
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                this._getMarkerPointerImage();
        },
        /**
         * To get Bar Pointer Image
		 * @private
         */
        _getBarPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
        /**
         * To get Marker Pointer Image
		 * @private
         */
        _getMarkerPointerImage: function () {
            if (this.barPointerImage)
                return this.barPointerImage;
            else
                return this._getCustomLabelImage();
        },
        /**
         * To get Custom Label Image
		 * @private
         */
        _getCustomLabelImage: function () {
            if (this.rangeImage)
                return this.rangeImage;
            else
                return this._getRangeImage();
        },
        /**
         * To get Range Image
		 * @private
         */
        _getRangeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
        /**
         * To get Label Image
		 * @private
         */
        _getLabelImage: function () {

            if (this.tickImage)
                return this.tickImage;
            else
                return this._getTickImage();
        },
        /**
         * To get Tick Image
		 * @private
         */
        _getTickImage: function () {
            if (this.scaleImage)
                return this.scaleImage;
            else
                return this.outerImage;
        },
        /**
        * To get the Indicator Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.indicatorIndex indicatorIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the IndicatorValue in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getIndicatorValue(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the IndicatorValue
		* $("#linearGauge1").ejLinearGauge("getIndicatorValue(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getIndicatorValue: function (scaleIndex, indicatorIndex) {
            if (scaleIndex < this.Model.scales.length && indicatorIndex < this.Model.scales[scaleIndex].indicators.length)
                return this.scaleEl[scaleIndex].indicators[indicatorIndex].value;
        },
        /**
        * To set IndicatorValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.indicatorIndex indicatorIndex value for the Gauge
		* @param {number} arguemnt.value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setIndicatorValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setIndicatorValue(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setIndicatorValue
		* $("#linearGauge1").ejLinearGauge("setIndicatorValue(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setIndicatorValue: function (scaleIndex, indicatorIndex, value) {
            if (scaleIndex < this.Model.scales.length && indicatorIndex < this.Model.scales[scaleIndex].indicators.length && value != null) {
                this.scaleEl[scaleIndex].indicators[indicatorIndex].value = value;
                this.initialize();
            }
        },
        /**
        * To set PointerValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value Pointer value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set PointerValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerValue(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setPointerValue
		* $("#linearGauge1").ejLinearGauge("setPointerValue(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].value = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get PointerValue in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get PointerValue in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerValue(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the getPointerValue
		* $("#linearGauge1").ejLinearGauge("getPointerValue(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length) {
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].value;
            }
        },
        /**
        * To set PointerWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex  scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.width Pointer width for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setPointerWidth
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerWidth(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setPointerWidth
		* $("#linearGauge1").ejLinearGauge("setPointerWidth(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerWidth: function (scaleIndex, pointerIndex, width) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && width != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].pointerWidth = width;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get PointerWidth in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get PointerWidth in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerWidth(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the PointerValue
		* $("#linearGauge1").ejLinearGauge("getPointerWidth(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length) {
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].pointerWidth;
            }
        },
        /**
        * To set PointerHeight 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.height for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setPointerHeight
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerHeight(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setPointerHeight
		* $("#linearGauge1").ejLinearGauge("setPointerHeight(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerHeight: function (scaleIndex, pointerIndex, height) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && height != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].pointerLength = height;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get PointerHeight in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get PointerHeight in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerHeight(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the PointerHeight
		* $("#linearGauge1").ejLinearGauge("getPointerHeight(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerHeight: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].pointerLength;
        },
        /**
         * To get color
		 * @private
         */
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.r + ", " + option.g + "," + option.b + ", " + option.a / 255 + ")");
            }
        },
        /**
        * To set setPointerDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setIndicatorValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setIndicatorValue(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setFeatureMeasureBarValue
		* $("#linearGauge1").ejLinearGauge("setFeatureMeasureBarValue(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Pointer Distance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Pointer Distance From Scale in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerDistanceFromScale(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Pointer Distance From Scale
		* $("#linearGauge1").ejLinearGauge("getPointerDistanceFromScale(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].distanceFromScale;
        },
        /**
        * To set setPointerPlacement 
		* @return jQuery
		* @param {number} argument.scaleIndex  scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value  pointer placement for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setPointerPlacement
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerPlacement(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setPointerPlacement
		* $("#linearGauge1").ejLinearGauge("setPointerPlacement(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerPlacement: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].pointerPlacement = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Pointer Placement in String
		* @return string
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Pointer Placement in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerPlacement(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Pointer Placement
		* $("#linearGauge1").ejLinearGauge("getPointerPlacement(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerPlacement: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].MarkerPointer[pointerIndex].pointerPlacement;
        },
        /**
        * To set setMarkerStyle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndexvalue for the Gauge
		* @param {string} arguemnt.value marker Style for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setMarkerStyle
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMarkerStyle(0,0,"triangle");
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setMarkerStyle
		* $("#linearGauge1").ejLinearGauge("setMarkerStyle(0,0,"trinagle")");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMarkerStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].markerStyle = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get MarkerStyle in number
		* @return number
		* @param {number} argument.scaleIndex vscaleIndex alue for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get MarkerStyle in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMarkerStyle(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the MarkerStyle
		* $("#linearGauge1").ejLinearGauge("getMarkerStyle(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMarkerStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].markerStyle;
        },
        /**
        * To set setBarPointerValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value Bar Pointer Value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setBarPointerValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setBarPointerValue(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setBarPointerValue
		* $("#linearGauge1").ejLinearGauge("setBarPointerValue(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setBarPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].value = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Bar Pointer Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Bar Pointer Value in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getBarPointerValue(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Bar Pointer Value
		* $("#linearGauge1").ejLinearGauge("getBarPointerValue(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getBarPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].barPointers[pointerIndex].value;
        },
        /**
        * To set setBarWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndexvalue for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value Bar Width for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setBarWidth
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setBarWidth(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setBarWidth
		* $("#linearGauge1").ejLinearGauge("setBarWidth(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setBarWidth: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].pointerWidth = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Bar Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Bar Width in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getBarWidth(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Bar Width
		* $("#linearGauge1").ejLinearGauge("getBarWidth(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getBarWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].BarPointer[pointerIndex].pointerWidth;
        },
        /**
        * To set setBarDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex,value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value Bar DistanceFromScale value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setBarDistanceFromScale
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setBarDistanceFromScale(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setBarDistanceFromScale
		* $("#linearGauge1").ejLinearGauge("setBarDistanceFromScale(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setBarDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].scaleStyle == "Thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Bar Distance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Bar Distance From Scale in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getBarDistanceFromScale(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Bar Distance From Scale
		* $("#linearGauge1").ejLinearGauge("getBarDistanceFromScale(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getBarDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].BarPointer[pointerIndex].distanceFromScale;
        },
        /**
        * To set setCustomLabelValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.customLabelIndex customLabelIndex value for the Gauge
		* @param {number} arguemnt.value CustomLabel value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setCustomLabelValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setCustomLabelValue(0,0,"Text");
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setCustomLabelValue
		* $("#linearGauge1").ejLinearGauge("setCustomLabelValue(0,0,"Text")");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setCustomLabelValue: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabel.length && value != null) {
                this.scaleEl[scaleIndex].customLabel[customLabelIndex].labelValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawCustomLabel();
                else
                    this.initialize();
            }
        },
        /**
        * To get CustomLabel Value in string
		* @return string
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.customLabelIndex customLabelIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get CustomLabel Value in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getCustomLabelValue(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the CustomLabel Value
		* $("#linearGauge1").ejLinearGauge("getCustomLabelValue(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getCustomLabelValue: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabel.length)
                return this.scaleEl[scaleIndex].customLabel[customLabelIndex].labelValue;
        },
        /**
        * To set setCustomLabelAngle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.customLabelIndex customLabelIndex value for the Gauge
		* @param {number} arguemnt.value Custom Label Angle for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setCustomLabelAngle
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setCustomLabelAngle(0,0,30);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setCustomLabelAngle
		* $("#linearGauge1").ejLinearGauge("setCustomLabelAngle(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setCustomLabelAngle: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabel.length && value != null) {
                this.scaleEl[scaleIndex].customLabel[customLabelIndex].textAngle = value;
                if (this.contextEl.putImageData)
                    this._reDrawCustomLabel();
                else
                    this.initialize();
            }
        },
        /**
        * To get CustomLabel Angle in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.customLabelIndex customLabelIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get CustomLabel Angle in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getCustomLabelAngle(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the CustomLabel Angle
		* $("#linearGauge1").ejLinearGauge("getCustomLabelAngle(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getCustomLabelAngle: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabel.length)
                return this.scaleEl[scaleIndex].customLabel[customLabelIndex].textAngle;
        },
        /**
        * To set setRangeStartValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value range start value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setRangeStartValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeStartValue(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setRangeStartValue
		* $("#linearGauge1").ejLinearGauge("setRangeStartValue(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeStartValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
        /**
        * To get Range Start Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Range Start Value in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeStartValue(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Range Start Value
		* $("#linearGauge1").ejLinearGauge("getRangeStartValue(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeStartValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startValue;
        },
        /**
        * To set setRangeEndValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range end value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setRangeEndValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeEndValue(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setRangeEndValue
		* $("#linearGauge1").ejLinearGauge("setRangeEndValue(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeEndValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
        /**
        * To get Range End Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Range End Value in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeEndValue(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Range End Value
		* $("#linearGauge1").ejLinearGauge("getRangeEndValue(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeEndValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endValue;
        },
        /**
        * To set setRangeStartWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range Start Width for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setRangeStartWidth
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeStartWidth(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setRangeStartWidth
		* $("#linearGauge1").ejLinearGauge("setRangeStartWidth(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeStartWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
        /**
        * To get Range Start Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Range Start Width in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeStartWidth(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Range Start Width
		* $("#linearGauge1").ejLinearGauge("getRangeStartWidth(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeStartWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startWidth;
        },
        /**
        * To set setRangeEndWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range End Width for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setRangeEndWidth
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeEndWidth(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setRangeEndWidth
		* $("#linearGauge1").ejLinearGauge("setRangeEndWidth(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeEndWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
        /**
        * To get Range End Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Range End Width in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeEndWidth(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Range End Width
		* $("#linearGauge1").ejLinearGauge("getRangeEndWidth(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeEndWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endWidth;
        },
        /**
        * To set setRangeDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range Distance FromScale for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setRangeDistanceFromScale
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setRangeDistanceFromScale
		* $("#linearGauge1").ejLinearGauge("setRangeDistanceFromScale(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeDistanceFromScale: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
        /**
        * To get Range Distance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Range Distance From Scale in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeDistanceFromScale(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Range Distance From Scale
		* $("#linearGauge1").ejLinearGauge("getRangeDistanceFromScale(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeDistanceFromScale: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale;
        },
        /**
        * To set setRangePosition 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range Position for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setRangePosition
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangePosition(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setRangePosition
		* $("#linearGauge1").ejLinearGauge("setRangePosition(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangePosition: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].rangePosition = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
        /**
        * To get Range Position in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Range Position in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangePosition(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Range Position
		* $("#linearGauge1").ejLinearGauge("getRangePosition(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangePosition: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].rangePosition;
        },
        /**
        * To set setRangeBorderWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range Border Width for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setRangeBorderWidth
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeBorderWidth(0,0,2);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setRangeBorderWidth
		* $("#linearGauge1").ejLinearGauge("setRangeBorderWidth(0,0,2)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeBorderWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].borderWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
        /**
        * To get Range Border Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Range Border Width in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeBorderWidth(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Range Border Width
		* $("#linearGauge1").ejLinearGauge("getRangeBorderWidth(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeBorderWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].borderWidth;
        },
        /**
        * To set setLabelAngle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {number} arguemnt.angle Label Angle for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setLabelAngle
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelAngle(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setLabelAngle
		* $("#linearGauge1").ejLinearGauge("setLabelAngle(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelAngle: function (scaleIndex, labelIndex, angle) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && angle != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].angle = angle;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
        /**
        * To get Label Angle in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Label Angle in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelAngle(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Label Angle
		* $("#linearGauge1").ejLinearGauge("getLabelAngle(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelAngle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].angle;
        },
        /**
        * To set setLabelStyle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {string} arguemnt.value Label Style for  Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setLabelStyle
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelStyle(0,0,"Major");
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setLabelStyle
		* $("#linearGauge1").ejLinearGauge("setLabelStyle(0,0,"Major")");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelStyle: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].labelStyle = $.inArray(value, this.Model.tickStyle);
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
        /**
        * To get LabelStyle in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Label Style in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelStyle(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Label Style
		* $("#linearGauge1").ejLinearGauge("getLabelStyle(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelStyle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].barPointers[labelIndex].labelStyle;
        },
        /**
        * To set setLabelPlacement 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {number} arguemnt.value Label Placement for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setLabelPlacement
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelPlacement(0,0,"Far");
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setLabelPlacement
		* $("#linearGauge1").ejLinearGauge("setLabelPlacement(0,0,"Far")");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelPlacement: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].labelPlacement = $.inArray(value, this.Model.labelPlacement);
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
        /**
        * To get LabelPlacement in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get LabelPlacement in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelPlacement(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the LabelPlacement
		* $("#linearGauge1").ejLinearGauge("getLabelPlacement(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelPlacement: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].barPointers[labelIndex].labelPlacement;
        },
        /**
        * To set setLabelXDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {number} arguemnt.value Label XDistance From Scale for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setLabelXDistanceFromScale
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelXDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setLabelXDistanceFromScale
		* $("#linearGauge1").ejLinearGauge("setLabelXDistanceFromScale(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelXDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].xDistanceFromScale = value;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
        /**
        * To get Label XDistance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get PointerValue in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelXDistanceFromScale(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Label XDistance From Scale
		* $("#linearGauge1").ejLinearGauge("getLabelXDistanceFromScale(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelXDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].xDistanceFromScale;
        },
        /**
        * To set setLabelYDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {number} arguemnt.value Label YDistance From Scale for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setLabelYDistanceFromScale
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelYDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setLabelYDistanceFromScale
		* $("#linearGauge1").ejLinearGauge("setLabelYDistanceFromScale(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelYDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].yDistanceFromScale = value;
                if (this.contextEl.putImageData)
                    this._reDrawLabel();
                else
                    this.initialize();
            }
        },
        /**
        * To get PointerValue in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get PointerValue in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelYDistanceFromScale(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Label YDistance From Scale
		* $("#linearGauge1").ejLinearGauge("getLabelYDistanceFromScale(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelYDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].yDistanceFromScale;
        },
        /**
        * To set setTickAngle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.angle Tick Angle for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setTickAngle
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickAngle(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setTickAngle
		* $("#linearGauge1").ejLinearGauge("setTickAngle(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickAngle: function (scaleIndex, TickIndex, angle) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && angle != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].angle = angle;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Tick Angle in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get TickAngle in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickAngle(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the TickAngle
		* $("#linearGauge1").ejLinearGauge("getTickAngle(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickAngle: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[TickIndex].angle;
        },
        /**
        * To set setTickWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick Width for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setTickWidth
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickWidth(0,0,5);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setTickWidth
		* $("#linearGauge1").ejLinearGauge("setTickWidth(0,0,5)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickWidth: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].tickWidth = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Tick Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Tick Width in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickWidth(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Tick Width
		* $("#linearGauge1").ejLinearGauge("getTickWidth(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickWidth: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].labels[TickIndex].tickWidth;
        },
        /**
        * To set setTickHeight 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick Height for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setTickHeight
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickHeight(0,0,10);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setTickHeight
		* $("#linearGauge1").ejLinearGauge("setTickHeight(0,0,10)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickHeight: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].tickHeight = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Tick Height in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Tick Height in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickHeight(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Tick Height
		* $("#linearGauge1").ejLinearGauge("getTickHeight(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickHeight: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].labels[TickIndex].tickHeight;
        },
        /**
        * To set setTickStyle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {string} arguemnt.value Tick Stylefor Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setTickStyle
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickStyle(0,0,"Major");
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setTickStyle
		* $("#linearGauge1").ejLinearGauge("setTickStyle(0,0,"Major")");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickStyle: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].tickStyle = $.inArray(value, this.Model.tickStyle);
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Tick Style in string
		* @return string
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Tick Style in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickStyle(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Tick Style
		* $("#linearGauge1").ejLinearGauge("getTickStyle(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickStyle: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].barPointers[TickIndex].tickStyle;
        },
        /**
        * To set setTickPlacement 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick Placement for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setTickPlacement
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickPlacement(0,0,"Far");
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setTickPlacement
		* $("#linearGauge1").ejLinearGauge("setTickPlacement(0,0,"Far")");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickPlacement: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].tickPlacement = $.inArray(value, this.Model.tickPlacement);
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get getTickPlacement in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get getTickPlacement in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickPlacement(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Tick Placement
		* $("#linearGauge1").ejLinearGauge("getTickPlacement(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickPlacement: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].barPointers[TickIndex].tickPlacement;
        },
        /**
        * To set setTickXDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick XDistance From Scale for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setTickXDistanceFromScale
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickXDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setTickXDistanceFromScale
		* $("#linearGauge1").ejLinearGauge("setTickXDistanceFromScale(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickXDistanceFromScale: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].xDistanceFromScale = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get get Tick XDistance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get getTick XDistance From Scale in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickXDistanceFromScale(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Tick XDistance From Scale
		* $("#linearGauge1").ejLinearGauge("getTickXDistanceFromScale(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickXDistanceFromScale: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[TickIndex].xDistanceFromScale;
        },
        /**
        * To set setTickYDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick YDistance From Scalefor Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setTickYDistanceFromScale
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickYDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setTickYDistanceFromScale
		* $("#linearGauge1").ejLinearGauge("setTickYDistanceFromScale(0,0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickYDistanceFromScale: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].yDistanceFromScale = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
        /**
        * To get Tick YDistance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get PointerValue in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickYDistanceFromScale(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Tick YDistance From Scale
		* $("#linearGauge1").ejLinearGauge("getTickYDistanceFromScale(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickYDistanceFromScale: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[TickIndex].yDistanceFromScale;
        },
        /**
        * To set setScaleLocation 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {object} arguemnt.value Scale Location for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setScaleLocation
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleLocation(0,[x.20,y.20]);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setScaleLocation
		* $("#linearGauge1").ejLinearGauge("setScaleLocation(0,[x.20,y.20])");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleLocation: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].location.x = value.x;
                this.scaleEl[scaleIndex].location.y = value.y;
                this.initialize();
            }
        },
        /**
        * To get Scale Location in object
		* @return object
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Scale Location in object
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleLocation(0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Scale Location
		* $("#linearGauge1").ejLinearGauge("getScaleLocation(0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleLocation: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return { "x": this.scaleEl[scaleIndex].location.x, "y": this.scaleEl[scaleIndex].location.y }
        },
        /**
        * To set setMaximumValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value MaximumValue for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setMaximumValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMaximumValue(0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setMaximumValue
		* $("#linearGauge1").ejLinearGauge("setMaximumValue(0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMaximumValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].maximum = value;
                this.initialize();
            }
        },
        /**
        * To get Maximum Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Maximum Value in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMaximumValue(0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Maximum Value
		* $("#linearGauge1").ejLinearGauge("getMaximumValue(0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMaximumValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[this.scaleIndex].maximum;
        },
        /**
        * To set setMinimumValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value MinimumValue for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setMinimumValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMinimumValue(0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setMinimumValue
		* $("#linearGauge1").ejLinearGauge("setMinimumValue(0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMinimumValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minimum = value;
                this.initialize();
            }
        },
        /**
        * To get PointerValue in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get PointerValue in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMinimumValue(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the MinimumValue
		* $("#linearGauge1").ejLinearGauge("getMinimumValue(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMinimumValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[this.scaleIndex].minimum;
        },
        /**
        * To set setScaleBarSize 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value ScaleBarSize for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setScaleBarSize
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleBarSize(0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setScaleBarSize
		* $("#linearGauge1").ejLinearGauge("setScaleBarSize(0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleBarSize: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].scaleBarSize = value;
                this.initialize();
            }
        },
        /**
        * To get Scale Bar Size in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.pointerIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Scale Bar Size in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleBarSize(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Scale Bar Size
		* $("#linearGauge1").ejLinearGauge("getScaleBarSize(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleBarSize: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].scaleBarSize;
        },
        /**
        * To set setScaleBarLength 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Scale Bar Length for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setScaleBarLength
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleBarLength(0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setScaleBarLength
		* $("#linearGauge1").ejLinearGauge("setScaleBarLength(0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleBarLength: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].scaleBarLength = value;
                this.initialize();
            }
        },
        /**
        * To set setScaleStyle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setScaleStyle
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleStyle(0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setScaleStyle
		* $("#linearGauge1").ejLinearGauge("setScaleStyle(0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleStyle: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].scaleStyle = value;
                this.initialize();
            }
        },
        /**
        * To get Scale Style in string
		* @return string
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Scale Style in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleStyle(0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the scaleStyle
		* $("#linearGauge1").ejLinearGauge("getScaleStyle(0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleStyle: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].scaleStyle;
        },
        /**
        * To get ScaleBarLength in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get ScaleBarLength in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleBarLength(0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the ScaleBarLength
		* $("#linearGauge1").ejLinearGauge("getScaleBarLength(0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleBarLength: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].scaleBarLength;
        },
        /**
        * To set setScaleBorderWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Scale Border Width for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setScaleBorderWidth
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleBorderWidth(0,20);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setScaleBorderWidth
		* $("#linearGauge1").ejLinearGauge("setScaleBorderWidth(0,20)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].borderWidth = value;
                this.initialize();
            }
        },
        /**
        * To set setScaleDirection 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Scale Direction for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setScaleDirection
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleDirection(0,"CounterClockwise");
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setScaleDirection
		* $("#linearGauge1").ejLinearGauge("setScaleDirection(0,"CounterClockwise")");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleDirection: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].scaleDirection = value;
                this.initialize();
            }
        },
        /**
        * To get Scale Direction in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Scale Direction in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleDirection(0,0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the Scale Direction
		* $("#linearGauge1").ejLinearGauge("getScaleDirection(0,0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleDirection: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].scaleDirection;
        },
        /**
        * To get Scale Border Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Scale Border Width in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleBorderWidth(0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Scale Border Width
		* $("#linearGauge1").ejLinearGauge("getScaleBorderWidth(0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].borderWidth;
        },
        /**
        * To set setMajorIntervalValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Major Interval Value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setMajorIntervalValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMajorIntervalValue(0,5);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setMajorIntervalValue
		* $("#linearGauge1").ejLinearGauge("setMajorIntervalValue(0,5)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMajorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].majorIntervalValue = value;
                this.initialize();
            }
        },
        /**
        * To get Major Interval Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get Major Interval Value in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMajorIntervalValue(0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Major Interval Value
		* $("#linearGauge1").ejLinearGauge("getMajorIntervalValue(0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMajorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].majorIntervalValue;
        },
        /**
        * To set setMinorIntervalValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Minor Interval Value for Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setMinorIntervalValue
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMinorIntervalValue(0,2);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setMinorIntervalValue
		* $("#linearGauge1").ejLinearGauge("setMinorIntervalValue(0,2)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMinorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minorIntervalValue = value;
                this.initialize();
            }
        },
        /**
        * To get Minor Interval Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get PointerValue in number
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMinorIntervalValue(0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // get the Minor Interval Value
		* $("#linearGauge1").ejLinearGauge("getMinorIntervalValue(0)");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMinorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].minorIntervalValue;
        },
        /**
         * To redraw the bar pointer
		 * @private
         */
        _reDrawBarPointer: function () {
            if (this.Model.frameBackgroundImageUrl)
                this.initialize();
            else {
                if (this.contextEl.putImageData != "undefined") {
                    this.contextEl.putImageData(this._getBarPointerImage(), 0, 0);
                    this._setBarPointers();
                    this._setMarkerPointers();
                }

            }
        },
        /**
         * To redraw the Marker pointer
		 * @private
         */
        _reDrawMarkerPointer: function () {
            if (this.Model.frameBackgroundImageUrl)
                this.initialize();
            else {
                if (this.contextEl.putImageData != "undefined") {
                    this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                    this._setMarkerPointers();
                }

            }
        },
        /**
         * To redraw the custom label
		 * @private
         */
        _reDrawCustomLabel: function () {
            if (this.Model.frameBackgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
        /**
         * To redraw the Range
		 * @private
         */
        _reDrawRange: function () {
            if (this.Model.frameBackgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getRangeImage(), 0, 0);
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
        /**
         * To redraw the Label
		 * @private
         */
        _reDrawLabel: function () {
            if (this.Model.frameBackgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getLabelImage(), 0, 0);
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
        /**
         * To redraw the TickMark
		 * @private
         */
        _reDrawTickMark: function () {
            if (this.Model.frameBackgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getTickImage(), 0, 0);
                this._setTicks();
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
        /**
         * To refresh the gauge
		 * @private
         */
        refresh: function () {
            this._init();
        },
        /**
        * To export Image
		* @return jQuery
		* @param {number} argument.fileName for the Image
		* @param {number}  argument.fileType for the Image
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // To Export Image
		* var LinearGaugeObj = $("#linearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.exportImage("myImage","jpeg");
		* &lt;/script&gt;
		* @example 
		* &lt;div id="linearGauge1"&gt;LinearGauge&lt;/div&gt; <br> 
		* &lt;script&gt;
		* //To Export Image
		* $("#linearGauge1").ejLinearGauge("exportImage("myImage","png")");	
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            var lnk = document.createElement('a'), e;
            lnk.download = fileName + "." + fileType;
            lnk.href = this.canvasEl[0].toDataURL("image/" + fileType).replace("image/" + fileType, "image/octet-stream");
            if (document.createEvent) {
                e = document.createEvent("MouseEvents");
                e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                lnk.dispatchEvent(e);
            }
            else if (lnk.fireEvent) {
                lnk.fireEvent("onclick");
            }
        },
        /**
         * To resize canvas while resizing the window
		 * @private
         */
        resizeCanvas: function () {
            if (proxy.element.width() != initialDivWidth || proxy.element.height() != initialDivHeight) {
                var canvasWidth = proxy.element.width() / initialDivWidth;
                var canvasHeight = proxy.element.height() / initialDivHeight;
                proxy.model.width *= canvasWidth;
                proxy.model.height *= canvasHeight;
                for (var i = 0; proxy.model.scales[i] != null; i++) {
                    if (proxy.element.data("ejLinearGauge").model.orientation.toString() == "Vertical")
                        proxy.element.data("ejLinearGauge").model.scales[i].scaleBarLength *= canvasHeight;
                    else
                        proxy.element.data("ejLinearGauge").model.scales[i].scaleBarLength *= canvasWidth;
                    for (var j = 0; proxy.model.scales[i].markerPointers[j] != null; j++) {
                        proxy.model.scales[i].markerPointers[j].pointerLength *= canvasWidth;
                        proxy.model.scales[i].markerPointers[j].pointerWidth *= canvasWidth;
                    }
                }
                proxy.refresh();
                initialDivWidth = proxy.element.width();
                initialDivHeight = proxy.element.height();
            }
        },
        //Client-Side Events
        /**
         * Section For handle the DrawTicks event
		 * @private
         */
        _onDrawTicks: function (tickAngle, tickValue) {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, tickIndex: this.tickIndex, tickElement: this.tickEl[this.tickIndex], context: this.contextEl, style: this.style, location: this.location, tickValue: tickValue, tickAngle: parseInt(tickAngle) };
            this._trigger("drawTicks", data);
        },
        /**
         * Section For handle the DrawLabels event
		 * @private
         */
        _onDrawLabels: function (labelAngle) {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, labelIndex: this.labelIndex, labelElement: this.labelEl[this.labelIndex], context: this.contextEl, style: this.style, location: this.location, labelValue: this.labelValue, labelAngle: parseInt(labelAngle) };
            this._trigger("drawLabels", data);
        },
        /**
         * Section For handle the DrawBarPointers event
		 * @private
         */
        _onDrawBarPointers: function (pointerValue) {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, barPointerIndex: this.barPointerIndex, barElement: this.barPointerEl[this.barPointerIndex], context: this.contextEl, style: this.style, location: this.location, pointerValue: pointerValue };
            this._trigger("drawBarPointers", data);
        },
        /**
         * Section For handle the DrawMarkerPointers event
		 * @private
         */
        _onDrawMarkerPointers: function (pointerAngle, pointerValue) {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, markerPointerIndex: this.markerPointerIndex, markerElement: this.markerPointerEl[this.markerPointerIndex], context: this.contextEl, style: this.style, location: this.location, pointerValue: pointerValue, pointerAngle: parseInt(pointerAngle) };
            this._trigger("drawMarkerPointers", data);
        },
        /**
         * Section For handle the DrawRange event
		 * @private
         */
        _onDrawRange: function () {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, rangeElement: this.rangeEl[this.rangeIndex], context: this.contextEl, style: this.style, location: this.location };
            this._trigger("drawRange", data);
        },
        /**
         * Section For handle the DrawCustomLabel event
		 * @private
         */
        _onDrawCustomLabel: function () {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, customLabelIndex: this.customLabelIndex, customLabelElement: this.customLabelEl[this.customLabelIndex], context: this.contextEl, style: this.style, location: this.location };
            this._trigger("drawCustomLabel", data);
        },
        /**
         * Section For handle the DrawIndicators event
		 * @private
         */
        _onDrawIndicators: function (location, style) {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, indicatorIndex: this.indicatorIndex, indicatorEl: this.indicatorEl[this.indicatorIndex], context: this.contextEl, style: this.style, location: this.location };
            this._trigger("drawIndicators", data);
        },
        /**
         * Invoke while onload
		 * @private
         */
        onLoad: function () {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("load", data);
        },
        /**
         * Invoke while initialize
		 * @private
         */
        _onInit: function () {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("init", data);
        },
        /**
         * Invoke while render complete
		 * @private
         */
        _onRenderComplete: function () {
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("renderComplete", data);
        },

        //_trigger: function (type, event, data) {
        //    var fn = this.Model[type];
        //    event = $.Event(event);
        //    event.type = type;
        //    $(this.GaugeEl).trigger(event, data);
        //    return !($.isFunction(fn) &&
        //        fn(this, data) === false ||
        //        event.isDefaultPrevented());
        //},

        //trigger: function (element, eventName, e) {
        //    e = $.extend(e || {}, new $.Event(eventName));
        //    e.stopPropagation();
        //    $(element).trigger(e);
        //    return e.isDefaultPrevented();
        //},
        /**
         * To restore width
		 * @private
         */
        _restoreWidth: function () {
            this.scaleEl[this.scaleIndex].scaleBarLength = this.scaleEl[this.scaleIndex].scaleBarLength + this.bottomRadius + this.scaleEl[this.scaleIndex].scaleBarSize;
            if (this.Model.orientation == "Vertical") {
                if (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise")
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] - this.bottomRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2;
                else
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].scaleBarSize / 2;
            }
            else {
                if (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise")
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] - this.bottomRadius - this.scaleEl[this.scaleIndex].scaleBarSize / 2;
                else
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] - this.scaleEl[this.scaleIndex].scaleBarSize / 2;
            }
        },
        /**
         * To modify width
		 * @private
         */
        _modifyWidth: function () {
            this.scaleEl[this.scaleIndex].scaleBarLength = this.scaleEl[this.scaleIndex].scaleBarLength - this.bottomRadius - this.scaleEl[this.scaleIndex].scaleBarSize;
            if (this.Model.orientation == "Vertical") {
                if (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise")
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] + this.bottomRadius + this.scaleEl[this.scaleIndex].scaleBarSize / 2;
                else
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2;
            }
            else {
                if (this.scaleEl[this.scaleIndex].scaleDirection == "Clockwise")
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] + this.bottomRadius + this.scaleEl[this.scaleIndex].scaleBarSize / 2;
                else
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2;
            }
        },
        // return y positions for vertical gauge
        /**
         * To get Clockwise LinePosition
		 * @private
         */
        _getClockwiseLinePosition: function (value) {
            var tempVal, linePosition;
            tempVal = (value - this.scaleEl[this.scaleIndex].minimum) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) * 100;
            linePosition = this.Model.orientation == "Vertical" ? this.scaleStartY[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].scaleBarLength) / 100) : this.scaleStartX[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].scaleBarLength) / 100);
            return linePosition;
        },
        /**
         * To get Counter Clockwise LinePosition
		 * @private
         */
        _getCounterClockwiseLinePosition: function (value) {
            var tempVal, linePosition;
            tempVal = this.scaleEl[this.scaleIndex].maximum - value + this.scaleEl[this.scaleIndex].minimum;
            tempVal = (tempVal - this.scaleEl[this.scaleIndex].minimum) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) * 100;
            linePosition = this.Model.orientation == "Vertical" ? this.scaleStartY[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].scaleBarLength) / 100) : this.scaleStartX[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].scaleBarLength) / 100);
            return linePosition;
        },
        /**
         * To get value
		 * @private
         */
        _getValue: function (position) {
            if (this.Model.orientation == "Vertical")
                tempVal = ((position.y - this.scaleStartY[this.scaleIndex]) / this.scaleEl[this.scaleIndex].scaleBarLength) * 100;
            else
                tempVal = (((position.x - this.scaleStartX[this.scaleIndex])) / this.scaleEl[this.scaleIndex].scaleBarLength) * 100;
            value = ((tempVal * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum)) + this.scaleEl[this.scaleIndex].minimum) / 100;
            if (this.scaleEl[this.scaleIndex].scaleDirection == "CounterClockwise")
                value = this.scaleEl[this.scaleIndex].maximum + this.scaleEl[this.scaleIndex].minimum - value;
            return value;
        },
        /**
         * To get Pointer X Position
		 * @private
         */
        _getPointerXPosition: function (element) {
            if (this.Model.orientation == "Vertical") {
                if (this.markerPlacement == "Far") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize + this.scaleEl[this.scaleIndex].borderWidth / 2 + element.distanceFromScale;
                    angle = 0;
                }
                if (this.markerPlacement == "Near") {
                    startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    angle = 180;
                }
                if (this.markerPlacement == "Center") {
                    if (element.markerStyle == "Circle")
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - radius + element.distanceFromScale;
                    else
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerWidth / 2 + element.distanceFromScale;
                    angle = 0;
                }
            }
            else {
                if (this.markerPlacement == "Far") {
                    startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize + this.scaleEl[this.scaleIndex].borderWidth / 2 + element.distanceFromScale;
                    angle = 90;
                }
                if (this.markerPlacement == "Near") {
                    startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].borderWidth / 2 + element.distanceFromScale;
                    angle = 270;
                }
                if (this.markerPlacement == "Center") {
                    if (element.markerStyle == "Circle")
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - radius + element.distanceFromScale;
                    else
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].scaleBarSize / 2 - element.pointerLength / 2 + element.distanceFromScale;
                    angle = 90;
                }
            }
            return { "startx": startX, "angle": angle };
        },
        /**
         * To get hexa decimal value from RGB combination
		 * @private
         */
        _hexFromRGB: function (r, g, b) {
            var hex = [r.toString(16), g.toString(16), b.toString(16)];
            $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
            return hex.join("").toUpperCase();
        },
        /**
         * To set GradientColor
		 * @private
         */
        _setGradientColor: function (element, gradient, options) {
            var self = element;
            if (options.Name || typeof (options) === "string") {
                gradient.addColorStop(0, options);
                gradient.addColorStop(1, options);
            }
            else
                $.each(options, function (index, colorElement) {
                    gradient.addColorStop(colorElement.ColorStop != NaN ? colorElement.ColorStop : 0, typeof (colorElement.Color) === "string" ? colorElement.Color : colorElement.Color);
                });
        },
        /**
         * To get Font String
		 * @private
         */
        _getFontString: function (element, font) {
            return font.fontStyle + " " + ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily;
        },
        /**
         * To get  Pointer Dimension
		 * @private
         */
        _setPointerDimension: function (style, element) {
            if (element.Model.orientation) {
                if (element.Model.orientation == "Horizontal") {
                    var tempWidth = style.width;
                    var tempHeight = style.height;
                    style.height = tempWidth;
                    style.width = tempHeight;
                }
            }
            return style;
        },
        /**
         * To set  cntext Rotation
		 * @private
         */
        _setContextRotation: function (style, element) {
            element.contextEl.rotate(Math.PI * (style.angle / 180));
        },
        /**
         * To wire  the events
		 * @private
         */
        wireEvents: function () {
            this.startEv = 'mousedown';
            this.moveEv = 'mousemove';
            this.endEv = 'mouseup';
            //this.onMouseMoveHandler = $.proxy(this._onMouseMove, this);
            //this.onMouseUpHandler = $.proxy(this._onMouseUp, this);
            this.onMouseDownHandler = $.proxy(this._onMouseDown, this);
            this.element.bind(this.startEv, this.onMouseDownHandler);
        },
        /**
         * To Unwire  the events
		 * @private
         */
        unWireEvents: function () {
            this.element.unbind(this.startEv, this.onMouseDownHandler);
        },
        /**
         * To block Default Actions
		 * @private
         */
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
        /**
         * Section For handle the Mouse Down event
		 * @private
         */
        _onMouseDown: function (e) {
            var startPoint, endPoint, value, position, greaterValue, lesserValue, startX, isOffset;
            this._blockDefaultActions(e);
            startPoint = { "x": this.centerX, "y": this.centerY };
            endPoint = { "x": e.pageX - $(this.canvasEl).offset().left - (this.Model.frameOuterWidth + this.Model.frameInnerWidth), "y": e.pageY - $(this.canvasEl).offset().top - (this.Model.frameOuterWidth + this.Model.frameInnerWidth) };
            offsetPoint = { "x": e.pageX - $(this.canvasEl).offset().left, "y": e.pageY - $(this.canvasEl).offset().top };
            value = this._getValue(endPoint);
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.markerPointers != null) {
                    self.markerPointerEl = element.markerPointers;
                    $.each(element.markerPointers, function (index, element) {
                        if (self.scaleEl[self.scaleIndex].scaleDirection == "Clockwise")
                            position = self._getClockwiseLinePosition(element.value);
                        else
                            position = self._getCounterClockwiseLinePosition(element.value);
                        greaterValue = position + element.pointerWidth;
                        lesserValue = position - element.pointerWidth;
                        startX = self._getPointerXPosition(element).startx;
                        var isOffset = self._isBetween(((self.Model.orientation == "Horizontal" ? endPoint.y : endPoint.x) - element.pointerWidth), ((self.Model.orientation == "Horizontal" ? endPoint.y : endPoint.x) + element.pointerWidth), startX);
                        if ((self.Model.orientation == "Horizontal" ? self._isBetween(lesserValue, greaterValue, endPoint.x) : self._isBetween(lesserValue, greaterValue, endPoint.y)) && isOffset)
                            self.activeElement = element;
                        if (self.Model.scales[self.scaleIndex].barPointers[index] != null)
                            self.activeBarElement = self.Model.scales[self.scaleIndex].barPointers[index];
                        self.onMouseMoveHandler = $.proxy(self._onMouseMove, self);
                        self.onMouseUpHandler = $.proxy(self._onMouseUp, self);
                        $(document).bind(self.moveEv, self.onMouseMoveHandler);
                        $(document).bind(self.endEv, self.onMouseUpHandler);
                    });
                }
            });
        },
        /**
         * To check whether the given value is in between first and Last arguemnet
		 * @private
         */
        _isBetween: function (first, last, number) {
            return (first < last ? number >= first && number <= last : number >= last && number <= first);
        },
        /**
         * Section For handle the Mouse up event
		 * @private
         */
        _onMouseUp: function () {
            this.mouseMove = false;
            $(document).unbind(self.moveEv, self.onMouseMoveHandler);
            $(document).unbind(self.endEv, self.onMouseUpHandler);
        },
        /**
         * Section For handle the Mouse move event
		 * @private
         */
        _onMouseMove: function (e) {
            this._blockDefaultActions(e);
            startPoint = { "x": this.centerX, "y": this.centerY };
            endPoint = { "x": e.pageX - $(this.canvasEl).offset().left - (this.Model.frameOuterWidth + this.Model.frameInnerWidth), "y": e.pageY - $(this.canvasEl).offset().top - (this.Model.frameOuterWidth + this.Model.frameInnerWidth) };
            this.activeElement.value = this._getValue(endPoint);
            if (this.activeBarElement)
                this.activeBarElement.value = this._getValue(endPoint);
            if (this.contextEl.putImageData)
                this._reDrawBarPointer();
            else
                this._init();
        },

        //************************* Skin Functions **********************************//
        /**
         * To set theme
		 * @private
         */
        _setTheme: function () {
            selectedTheme = ej.LinearGauge.themes[this.model.theme];
            this._setThemeColors(selectedTheme);
        },
        /**
         * To set theme color
		 * @private
         */
        _setThemeColors: function (selectedTheme) {
            var result = [], jsonObj = ej.LinearGauge.themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                this.model.backgroundColor = ((!this.model.backgroundColor || this.model.backgroundColor == jsonObj[result[th]].scales.backgroundColor) ? selectedTheme.scales.backgroundColor : this.model.backgroundColor);
                this.model.borderColor = ((!this.model.borderColor || this.model.borderColor == jsonObj[result[th]].scales.borderColor) ? selectedTheme.scales.borderColor : this.model.borderColor);
                this.model.labelColor = ((!this.model.labelColor || this.model.labelColor == jsonObj[result[th]].scales.labels.labelColor) ? selectedTheme.scales.labels.labelColor : this.model.labelColor);
                this.model.tickColor = ((!this.model.tickColor || this.model.tickColor == jsonObj[result[th]].scales.ticks.tickColor) ? selectedTheme.scales.ticks.tickColor : this.model.tickColor);

                for (var i = 0; i < this.model.scales.length; i++) {
                    for (var m = 0; m < this.model.scales[i].markerPointers.length; m++) {
                        this.model.scales[i].markerPointers[m].backgroundColor = (!this.model.scales[i].markerPointers[m].backgroundColor || this.model.scales[i].markerPointers[m].backgroundColor == jsonObj[result[th]].scales.markerPointers.backgroundColor) ? selectedTheme.scales.markerPointers.backgroundColor : this.model.scales[i].markerPointers[m].backgroundColor;
                        this.model.scales[i].markerPointers[m].borderColor = (!this.model.scales[i].markerPointers[m].borderColor || this.model.scales[i].markerPointers[m].borderColor == jsonObj[result[th]].scales.markerPointers.borderColor) ? selectedTheme.scales.markerPointers.borderColor : this.model.scales[i].markerPointers[m].borderColor;
                    }
                    for (var b = 0; b < this.model.scales[i].barPointers.length; b++) {
                        this.model.scales[i].barPointers[b].backgroundColor = (!this.model.scales[i].barPointers[b].backgroundColor || this.model.scales[i].barPointers[b].backgroundColor == jsonObj[result[th]].scales.barPointers.backgroundColor) ? selectedTheme.scales.barPointers.backgroundColor : this.model.scales[i].barPointers[b].backgroundColor;
                        this.model.scales[i].barPointers[b].borderColor = (!this.model.scales[i].barPointers[b].borderColor || this.model.scales[i].barPointers[b].borderColor == jsonObj[result[th]].scales.barPointers.borderColor) ? selectedTheme.scales.barPointers.borderColor : this.model.scales[i].barPointers[b].borderColor;
                    }
                }
            }
        }
    });
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.tickStyle = {
        /**  To set the tickStyle MajorInterval. */
        MajorInterval: "MajorInterval",
        /**  To set the tickStyle MinorInterval. */
        MinorInterval: "MinorInterval"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.labelStyle = {
        /**  To set the labelStyle Major. */
        Major: "Major",
        /**  To set the labelStyle Minor. */
        Minor: "Minor"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.fontStyle = {
        /**  To set the fontStyle Bold. */
        Bold: "Bold",
        /**  To set the fontStyle Italic. */
        Italic: "Italic",
        /**  To set the fontStyle Regular. */
        Regular: "Regular",
        /**  To set the fontStyle Strikeout. */
        Strikeout: "Strikeout",
        /**  To set the fontStyle Underline. */
        Underline: "Underline"
    };


    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.pointerPlacement = {
        /**  To set the pointerPlacement Near. */
        Near: "Near",
        /**  To set the pointerPlacement Far. */
        Far: "Far",
        /**  To set the pointerPlacement Center. */
        Center: "Center"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.tickPlacement = {
        /**  To set the tickPlacement Near. */
        Near: "Near",
        /**  To set the tickPlacement Far. */
        Far: "Far",
        /**  To set the tickPlacement Center. */
        Center: "Center"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.labelPlacement = {
        /**  To set the labelPlacement Near. */
        Near: "Near",
        /**  To set the labelPlacement Far. */
        Far: "Far",
        /**  To set the labelPlacement Center. */
        Center: "Center"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.rangePosition = {
        /**  To set the rangePosition Near. */
        Near: "Near",
        /**  To set the rangePosition Far. */
        Far: "Far",
        /**  To set the rangePosition Center. */
        Center: "Center"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */

    ej.unitTextPosition = {
        /**  To set the unitTextPosition Front. */
        Front: "Front",
        /**  To set the unitTextPosition Back. */
        Back: "Back"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */

    ej.scaleDirection = {
        /**  To set the scaleDirection Clockwise. */
        Clockwise: "Clockwise",
        /**  To set the scaleDirection CounterClockwise. */
        CounterClockwise: "CounterClockwise"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.scaleStyle = {
        /**  To set the scaleStyle Rectangle. */
        Rectangle: "Rectangle",
        /**  To set the scaleStyle RoundedRectangle. */
        RoundedRectangle: "RoundedRectangle",
        /**  To set the scaleStyle Thermometer. */
        Thermometer: "Thermometer"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.indicatorStyle = {
        /**  To set the indicatorStyle Rectangle. */
        Rectangle: "Rectangle",
        /**  To set the indicatorStyle Circle. */
        Circle: "Circle",
        /**  To set the indicatorStyle RoundedRectangle. */
        RoundedRectangle: "RoundedRectangle",
        /**  To set the indicatorStyle text. */
        text: "text"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.markerStyle = {
        /**  To set the markerStyle Rectangle. */
        Rectangle: "Rectangle",
        /**  To set the markerStyle Triangle. */
        Triangle: "Triangle",
        /**  To set the markerStyle Ellipse. */
        Ellipse: "Ellipse",
        /**  To set the markerStyle Diamond. */
        Diamond: "Diamond",
        /**  To set the markerStyle Pentagon. */
        Pentagon: "Pentagon",
        /**  To set the markerStyle Circle. */
        Circle: "Circle",
        /**  To set the markerStyle Star. */
        Star: "Star",
        /**  To set the markerStyle Slider. */
        Slider: "Slider",
        /**  To set the markerStyle Pointer. */
        Pointer: "Pointer",
        /**  To set the markerStyle Wedge. */
        Wedge: "Wedge",
        /**  To set the markerStyle Trapezoid. */
        Trapezoid: "Trapezoid",
        /**  To set the markerStyle RoundedRectangle. */
        RoundedRectangle: "RoundedRectangle"
    };
    /**
	 * Enum for gauge frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.LinearGauge.themes = {
        /**  To set the themes flatlight. */
        flatlight: {
            scales: {
                backgroundColor: "#FFFFFF",
                borderColor: "#1d1e1e",
                barPointers: {
                    backgroundColor: "#8abc3b",
                    borderColor: "#8abc3b"
                },
                markerPointers: {
                    backgroundColor: "#212121",
                    borderColor: "#212121"
                },
                ticks: {
                    tickColor: "#1d1e1e"
                },
                labels: {
                    labelColor: "#222222"
                }
            }
        },
        /**  To set the themes flatdark. */
        flatdark: {
            scales: {
                backgroundColor: "#808080",
                borderColor: "#808080",
                barPointers: {
                    backgroundColor: "#8abc3b",
                    borderColor: "#8abc3b"
                },
                markerPointers: {
                    backgroundColor: "#CCCCCC",
                    borderColor: "#CCCCCC"
                },
                ticks: {
                    tickColor: "#808080"
                },
                labels: {
                    labelColor: "#CCCCCC"
                }
            }
        }
    };

})(jQuery, Syncfusion);
;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/


(function ($, ej, undefined) {
    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    /**
    * @namespace ej
    * @class ejDigitalGauge
    * @requires jQuery
    * @requires ej.common.all
    * @classdesc Custom Design for Digital Gauge control.
    * @example 
    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create Digital Gauge
    * $('#DigitalCore').ejDigitalGauge();         
    * &lt;/script&gt;
    */



    ej.widget({ "ejDigitalGauge": "ej.DigitalGauge" }, {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        // default model
        defaults: /** @lends ejDigitalGauge# */{
            /**		
			* Specifies the segmentData for the DigitalGauge.
			*/

            segmentData: {
                "0": [0, 1, 2, 3, 4, 5, 14, 15], "1": [1, 2], "2": [0, 14, 1, 6, 8, 4, 3, 15], "3": [0, 1, 2, 3, 6, 8, 14, 15], "4": [1, 2, 5, 6, 8], "5": [0, 2, 3, 5, 6, 8, 14, 15], "6": [0, 2, 3, 4, 5, 6, 8, 14, 15],
                "7": [0, 1, 2, 14], "8": [0, 1, 2, 3, 4, 5, 6, 8, 14, 15], "9": [0, 1, 2, 3, 5, 6, 8, 14, 15], "A": [0, 1, 2, 4, 5, 6, 8, 14], "B": [0, 1, 2, 3, 7, 9, 8, 14, 15], "C": [0, 3, 4, 5, 14, 15],
                "D": [0, 1, 2, 3, 7, 9, 14, 15], "E": [0, 3, 4, 5, 6, 8, 14, 15], "F": [0, 4, 5, 6, 8, 14], "G": [0, 2, 3, 4, 5, 8, 14, 15], "H": [1, 2, 4, 5, 6, 8], "I": [0, 3, 7, 9, 14, 15],
                "J": [1, 2, 3, 4, 15], "K": [4, 5, 6, 10, 11], "L": [3, 4, 5, 15], "M": [1, 2, 4, 5, 10, 13], "N": [1, 2, 4, 5, 11, 13], "O": [0, 1, 2, 3, 4, 5, 14, 15], "P": [0, 1, 4, 5, 6, 8, 14],
                "Q": [0, 1, 2, 3, 4, 5, 11, 14, 15], "R": [0, 1, 4, 5, 6, 8, 11, 14], "S": [0, 2, 3, 5, 6, 8, 14, 15], "T": [0, 7, 9, 14], "U": [1, 2, 3, 4, 5, 15], "V": [4, 5, 10, 12], "W": [1, 2, 4, 5, 11, 12],
                "X": [10, 11, 12, 13], "Y": [1, 5, 6, 7, 8], "Z": [0, 3, 10, 12, 14, 15]
            },
            /**		
			* Specifies the matrixSegmentData for the DigitalGauge.
			*/
            matrixSegmentData: {
                "1": [0, 3, 0, 4, 1, 1, 1, 2, 1, 3, 1, 4, 2, 3, 2, 4, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "2": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 1, 5, 1, 6, 2, 5, 2, 6, 3, 4, 3, 5, 4, 3, 4, 2, 5, 2, 5, 1, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "3": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 1, 5, 1, 6, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5],
                "4": [0, 3, 0, 4, 0, 5, 1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 2, 4, 2, 5, 3, 0, 3, 1, 3, 4, 3, 5, 4, 0, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 4, 5, 5, 6, 4, 6, 5],
                "5": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "6": [0, 3, 0, 4, 0, 5, 0, 6, 1, 2, 1, 3, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 2, 5, 3, 5, 6, 5, 7, 6, 3, 6, 4, 6, 5, 6, 6],
                "7": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 6, 1, 7, 2, 5, 2, 6, 3, 4, 3, 5, 4, 3, 4, 4, 5, 2, 5, 3, 6, 1, 6, 2],
                "8": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "9": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 4, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 4, 6, 5],
                "0": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "a": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 6, 1, 7, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "b": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "c": [1, 3, 1, 4, 1, 5, 1, 6, 2, 2, 2, 3, 3, 1, 3, 2, 4, 1, 4, 2, 5, 2, 5, 3, 6, 3, 6, 4, 6, 5, 6, 6],
                "d": [0, 6, 0, 7, 1, 6, 1, 7, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 5, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "e": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "f": [0, 4, 0, 5, 0, 6, 0, 7, 1, 3, 1, 4, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "g": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 6, 4, 7, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "h": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "i": [0, 3, 0, 4, 2, 1, 2, 2, 2, 3, 2, 4, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "j": [1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "k": [0, 1, 0, 2, 1, 1, 1, 2, 1, 4, 1, 5, 2, 1, 2, 2, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 5, 1, 5, 2, 5, 5, 5, 6, 6, 1, 6, 2, 6, 6, 6, 7],
                "l": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 5, 6, 6],
                "m": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 0, 1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 2, 0, 2, 1, 2, 3, 2, 4, 2, 6, 2, 7, 3, 0, 3, 1, 3, 3, 3, 4, 3, 6, 3, 7, 4, 0, 4, 1, 4, 3, 4, 4, 4, 6, 4, 7, 5, 0, 5, 1, 5, 3, 5, 4, 5, 6, 5, 7, 6, 0, 6, 1, 6, 3, 6, 4, 6, 6, 6, 7],
                "n": [1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 0, 2, 1, 2, 2, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 6, 4, 7, 5, 0, 5, 1, 5, 6, 5, 7, 6, 0, 6, 1, 6, 6, 6, 7],
                "o": [1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 2, 5, 2, 6, 3, 1, 3, 2, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "p": [1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 1, 2, 2, 2, 3, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 1, 5, 2, 6, 1, 6, 2],
                "q": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 5, 3, 6, 3, 7, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 4, 7, 5, 6, 5, 7, 6, 6, 6, 7],
                "r": [0, 1, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 1, 2, 2, 2, 6, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "s": [1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 3, 1, 3, 2, 4, 3, 4, 4, 5, 4, 5, 5, 6, 1, 6, 2, 6, 3, 6, 4],
                "t": [0, 3, 0, 4, 1, 3, 1, 4, 2, 1, 2, 2, 2, 3, 2, 4, 2, 5, 2, 6, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 4, 6, 5, 6, 6, 6, 7],
                "u": [1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "v": [1, 1, 1, 2, 1, 6, 1, 7, 2, 2, 2, 3, 2, 5, 2, 6, 3, 2, 3, 3, 3, 5, 3, 6, 4, 3, 4, 4, 4, 5, 5, 4, 5, 5, 6, 4],
                "w": [0, 0, 0, 1, 0, 6, 0, 7, 1, 0, 1, 1, 1, 6, 1, 7, 2, 0, 2, 1, 2, 3, 2, 4, 2, 6, 2, 7, 3, 0, 3, 1, 3, 3, 3, 4, 3, 6, 3, 7, 4, 0, 4, 1, 4, 3, 4, 4, 4, 6, 4, 7, 5, 0, 5, 1, 5, 2, 5, 3, 5, 4, 5, 5, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "x": [1, 1, 1, 2, 1, 6, 1, 7, 2, 2, 2, 3, 2, 5, 2, 6, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 4, 5, 5, 2, 5, 3, 5, 5, 5, 6, 6, 1, 6, 2, 6, 6, 6, 7],
                "y": [1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 5, 2, 5, 3, 6, 1, 6, 2],
                "z": [1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 2, 6, 2, 7, 3, 5, 3, 6, 4, 4, 4, 5, 5, 3, 5, 4, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "A": [0, 3, 0, 4, 1, 2, 1, 3, 1, 4, 1, 5, 2, 2, 2, 3, 2, 4, 2, 5, 3, 1, 3, 2, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 0, 6, 1, 6, 6, 6, 7],
                "B": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "C": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 0, 2, 1, 3, 0, 3, 1, 4, 0, 4, 1, 5, 1, 5, 2, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "D": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "E": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "F": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "G": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 0, 2, 1, 3, 0, 3, 1, 3, 4, 3, 5, 3, 6, 4, 0, 4, 1, 4, 6, 5, 1, 5, 2, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "H": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "I": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "J": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "K": [0, 1, 0, 2, 0, 5, 0, 6, 1, 1, 1, 2, 1, 4, 1, 5, 2, 1, 2, 2, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 4, 1, 4, 2, 4, 3, 4, 4, 5, 1, 5, 2, 5, 4, 5, 5, 6, 1, 6, 2, 6, 5, 6, 6],
                "L": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "M": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 3, 1, 5, 1, 6, 1, 7, 2, 1, 2, 2, 2, 4, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "N": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 4, 2, 6, 2, 7, 3, 1, 3, 2, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "O": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 0, 2, 1, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "P": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "Q": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 0, 2, 1, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 4, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "R": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 5, 1, 5, 5, 6, 2, 6, 1, 6, 2, 6, 6],
                "S": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 2, 3, 3, 3, 4, 3, 5, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "T": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4],
                "U": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "V": [0, 0, 0, 1, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 5, 3, 6, 4, 3, 4, 4, 4, 5, 4, 6, 5, 4, 5, 5, 6, 4],
                "W": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 4, 4, 6, 4, 7, 5, 1, 5, 2, 5, 3, 5, 5, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "X": [0, 0, 0, 1, 0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 2, 4, 3, 4, 4, 4, 5, 5, 1, 5, 2, 5, 5, 5, 6, 6, 0, 6, 1, 6, 6, 6, 7],
                "Y": [0, 0, 0, 1, 0, 6, 0, 7, 1, 0, 1, 1, 1, 6, 1, 7, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "Z": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 6, 1, 7, 2, 5, 2, 6, 3, 4, 3, 5, 4, 2, 4, 3, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                ",": [5, 3, 5, 4, 5, 5, 6, 4, 6, 5, 7, 3, 7, 4],
                ":": [1, 3, 1, 4, 1, 5, 2, 3, 2, 4, 2, 5, 4, 3, 4, 4, 4, 5, 5, 3, 5, 4, 5, 5],
                "%": [0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 4, 2, 5, 3, 3, 3, 4, 4, 2, 4, 3, 5, 1, 5, 2, 5, 4, 5, 5, 6, 0, 6, 1, 6, 4, 6, 5],
                "!": [0, 3, 0, 4, 0, 5, 1, 3, 1, 4, 1, 5, 2, 3, 2, 4, 2, 5, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 4, 5, 5, 3, 5, 4, 5, 5, 7, 4],
                "(": [0, 2, 0, 3, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 7, 2, 7, 3],
                ")": [0, 5, 0, 6, 1, 6, 1, 7, 2, 6, 2, 7, 3, 6, 3, 7, 4, 6, 4, 7, 5, 6, 5, 7, 6, 6, 6, 7, 7, 5, 7, 6],
                ".": [5, 3, 5, 4, 5, 5, 6, 3, 6, 4, 6, 5, 7, 3, 7, 4, 7, 5]
            },
            /**		
           * Specifies the url of an image to be displayed as background of the Digital gauge.
           * @default null
           * @type {string}
           * @example 
           * //To initialize the url of the background image  
           * 	$("#DigitalCore").ejDigitalGauge({ frameBackgroundImageUrl: "Car.png"  }); 
           * @example 
           * //To get/set the url of the background image for the Digital gauge, after initialization:<br>
           *	//Gets the frameInnerWidth value
           *	$("#DigitalCore").ejDigitalGauge("option", "frameBackgroundImageUrl");			
           *	//Sets the frameInnerWidth value
           *	$("#DigitalCore").ejDigitalGauge("option", "frameBackgroundImageUrl", "Car.png" );
            * @memberof ejDigitalGauge
           * @instance
           */
            frameBackgroundImageUrl: null,
            /**		
			* Specifies the inner width for the frame, when the background image has been set for the Digital gauge..
			* @default 6
			* @type {number}
			* @example 
			* //To initialize the inner width of the frame  
			* 	$("#DigitalCore").ejDigitalGauge({ frameInnerWidth: 30 });
			* @example 
			* //To get/set the value of frameInnerWidth, after initialization:<br>
			*	//Gets the frameInnerWidth value
			*	$("#DigitalCore").ejDigitalGauge("option", "frameInnerWidth");			
			*	//Sets the frameInnerWidth value
			*	$("#DigitalCore").ejDigitalGauge("option", "frameInnerWidth", 30 );
			 * @memberof ejDigitalGauge
			* @instance
			*/
            frameInnerWidth: 6,
            /**		
           * Specifies the outer width of the frame, when the background image has been set for the Digital gauge.
           * @default 10
           * @type {number}
           * @example 
           * //To initialize the outer width of the frame  
           * 	$("#DigitalCore").ejDigitalGauge({ frameOuterWidth: 30 });
           * @example 
           * //To get/set the value of frameOuterWidth, after initialization:<br>
           *	//Gets the frameInnerWidth value
           *	$("#DigitalCore").ejDigitalGauge("option", "frameOuterWidth");			
           *	//Sets the frameInnerWidth value
           *	$("#DigitalCore").ejDigitalGauge("option", "frameOuterWidth", 30 );
            * @memberof ejDigitalGauge
           * @instance
           */
            frameOuterWidth: 10,
            /**		
           * Specifies the height of the DigitalGauge.
           * @default 150
           * @type {number}
           * @example 
           * //To initialize the height of the gauge  
           * 	$("#DigitalCore").ejDigitalGauge({ height: 30 }); 
           * @example 
           * //To get/set the value of height, after initialization:<br>
           *	//Gets the height value
           *	$("#DigitalCore").ejDigitalGauge("option", "height");			
           *	//Sets the height value
           *	$("#DigitalCore").ejDigitalGauge("option", "height", 30 );
            * @memberof ejDigitalGauge
           * @instance
           */
            height: 150,
            /**		
          * Specifies the width for the Digital gauge.
          * @default 400
          * @type {number}
          * @example 
          * //To initialize the width of the gauge  
          * $("#DigitalCore").ejDigitalGauge({ width: 30 });
          * @example 
          * //To get/set the value of width, after initialization:<br>
          *	//Gets the height value
          *	$("#DigitalCore").ejDigitalGauge("option", "width");			
          *	//Sets the height value
          *	$("#DigitalCore").ejDigitalGauge("option", "width", 30 );
           * @memberof ejDigitalGauge
          * @instance
          */
            width: 400,
            /**		
            * Specifies the resize option of the DigitalGauge.
            * @default false
            * @type {boolean}
            * @example 
            * //To set resize option value during initialization  
            * $("#DigitalCore").ejDigitalGauge({ canResize: true });
            * @example 
            * //To get/set the value of canResize, after initialization:<br>
            *	//Gets the canResize value
            *	$("#DigitalCore").ejDigitalGauge("option", "canResize");			
            *	//Sets the canResize value
            *	$("#DigitalCore").ejDigitalGauge("option", "canResize", true );
             * @memberof ejDigitalGauge
            * @instance
            */
            canResize: false,
            /**		
            * Specifies the color of the text segments.
            * @default #836B33
            * @type {string}
            * @example 
            * //To initialize the segment color  
            * $("#DigitalCore").ejDigitalGauge({ segmentColor: #FF1F2F });
            * @example 
            * //To get/set the color of the segments, after initialization:<br>
            * //Gets the segment Color
            *   var item = $("#DigitalCore").ejDigitalGauge("option", "items");
            * //index refers to the index value of the items.
            *   var result = item[index].segmentColor; 
            * //Set the color of the segments
            *   $("#DigitalCore").ejDigitalGauge("option", "items", [{ segmentColor: "#FF1F2F" }]);
            * @memberof ejDigitalGauge
            * @instance
            */
            segmentColor: "#836B33",
            /**		
            * Specifies the theme theme for the Digital gauge. See {@link themes} 
            * @default flatlight
            * @type {string}
            * @example 
            * //To initialize the theme for the gauge  
            * $("#DigitalCore").ejDigitalGauge({ theme: flatlight });
            * @example 
            * //To get/set the theme, after initialization:<br>
            *	//Get the theme of the gauge
            *	$("#DigitalCore").ejDigitalGauge("option", "theme");			
            *	//Set the theme of the gauge
            *	$("#DigitalCore").ejDigitalGauge("option", "theme", flatlight );
            * @memberof ejDigitalGauge
            * @instance
            */
            theme: "flatlight",
            /**		
           * Specifies the items for the DigitalGauge.
           * @default null                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
           * @type {int}
           * @example 
           * //To set items during initialization  
           * $("#DigitalCore").ejDigitalGauge({items: [{ characterCount: 2 }] });
           * @example 
           * //Get or set the items, after initialization:<br>
           * //Gets the items value
           * $("#DigitalCore").ejDigitalGauge("option", "items");			
           * //Sets the items value
           * //Set the value for characterCount
           * //$("#DigitalCore").ejDigitalGauge("option", "items", [{ characterCount: 4 }]);
           * @memberof ejDigitalGauge
           * @instance
           */
            items: null,
            /**     
            * Triggers when the gauge is initialized.
            * @event
            * @name ejDigitalGauge#init 	
            * @param {Object} args.Object  returns the object of the gauge.
            * @param {boolean} args.cancel  returns the cancel option value
            * @param {Object} args.items   returns the all the options of the items.
            * @param {Object} args.context returns the context element
            * @param {Object} args.model   returns the gauge model
            * @param {String} args.type    returns the name of the event
            * @param {}
            * @example 
            * //init event for Digital Gauge
            * $("#DigitalCore").ejDigitalGauge({
            *    init: function (args) {}
            * });      
            * @memberof ejDigitalGauge
            * @instance
            */
            init: null,
            /**     
            * Triggers when the gauge is start to load.
            * @event
            * @name ejDigitalGauge#load 	
            * @param {Object} args.Object  returns the object of the gauge.
            * @param {boolean} args.cancel  returns the cancel option value
            * @param {Object} args.items   returns the all the options of the items.
            * @param {Object} args.context returns the context element
            * @param {Object} args.model   returns the gauge model
            * @param {String} args.type    returns the name of the event
            * @example 
            * //load event for Digital Gauge
            * $("#DigitalCore").ejDigitalGauge({
            *    load: function (args) {}
            * });      
            * @memberof ejDigitalGauge
            * @instance
            */
            load: null,
            /**     
            * Triggers when the gauge render is completed.
            * @event
            * @name ejDigitalGauge#renderComplete 	
            * @param {Object} args.Object  returns the object of the gauge.
            * @param {boolean} args.cancel  returns the cancel option value
            * @param {Object} args.items   returns the all the options of the items.
            * @param {Object} args.context returns the context element
            * @param {Object} args.model   returns the gauge model
            * @param {String} args.type    returns the name of the event
            * @example 
            * //renderComplete event for Digital Gauge
            * $("#DigitalCore").ejDigitalGauge({
            *    renderComplete: function (args) {}
            * });      
            * @memberof ejDigitalGauge
            * @instance
            */
            renderComplete: null,
            /**		
           * Specifies the value to the DigitalGauge.
           * @default text                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
           * @type {string}
           * @example 
           * //To set value API value during initialization  
           * $("#DigitalCore").ejDigitalGauge({items: [{ value: "Welcome" }] });
           * @example 
           * //Get or set the text value API, after initialization:<br>
           * //Gets the value
           * $("#DigitalCore").ejDigitalGauge("option", "items");			
           * //Sets the text value
           * //Set the text value 
           * //$("#DigitalCore").ejDigitalGauge("option", "items", [{ value: "Welcome" }]);
           * @memberof ejDigitalGauge
           * @instance
           */
            value: "text"
        },

        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            frameBackgroundImageUrl: "string",
            segmentColor: "string",
            segmentData: "data",
            matrixSegmentData: "data",
            items: "array"
        },

        /**
        * To configure set values of the Digital Gauge		
        * @private
        */
        _setValues: function () {
            this.gaugeEl = this.element;
            this.segmentCount = null;
            this.contextEl = null;
            this.style = null;
            this._value = null;
            this.location = null;
            this.canvasEl = null;
            this.segement16X = null;
            this.segment16Y = null;
            this.segmentHeight = null;
            this.segmentAngle = null;
            this.startX = 5;
            this.startY = 5;
            this.gradient = null;
            this.itemIndex = null;
            this.characterSpace = null;
            this.outerImage = null;
            this.radius = null;
            this.frameOuterLocation = null;
            this.frameInnerLocation = null;
            this.glassFrameLocation = null;
            this.glassFrameStyle = null;
            this.frameOuterStyle = null;
            this.character = null;
            this.frameInnerStyle = null;
            this._itemInitialize();
        },
        observables: ["value"],
        value: ej.util.valueFunction("value"),


        /**
       * To destroy the digital gauge 
       * @alias destroy
       * @return jQuery
       * @example 
       * &lt;div id="DigitalGauge1"&gt;Digital Gauge&lt;/div&gt; <br> 
       * &lt;script&gt;
       * // Destroy Digital Gauge
       * var gphObj = $("#DigitalGauge1").data("ejDigitalGauge");
       * gphObj.destroy(); // destroy the digital gauge
       * &lt;/script&gt;
       * @example 
       * &lt;div id="DigitalGauge1"&gt;DigitalGauge&lt;/div&gt; <br> 
       * &lt;script&gt;
       * // destroy the Digital Gauge
       * $("#DigitalGauge1").ejDigitalGauge("destroy"); 
       * &lt;/script&gt;
       *@memberof ejDigitalGauge
       * @instance
         */
        _destroy: function () {
            this.element.empty();
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height":
                        this.model.height = options[option];
                        break;
                    case "width":
                        this.model.width = options[option];
                        break;
                    case "items":
                        this.model.items = options[option];
                        this._itemInitialize();
                        break;
                    case "frameBackgroundImageUrl":
                        this.model.frameBackgroundImageUrl = options[option];
                        break;
                    case "frameInnerWidth":
                        this.model.frameInnerWidth = options[option];
                        break;
                    case "frameOuterWidth":
                        this.model.frameOuterWidth = options[option];
                        break;
                    case "theme": this.model.theme = options[option]; break;
                    case "value":
                        for (var i = 0; this.model.items[i] != null; i++)
                            this.model.items[i].value = this.value();
                }
            }
            this.refresh();
        },
        /**
        * To initialize the Digital Gauge		
        * @private
        */
        _init: function () {
            this._setValues();
            this._trigger("load");
            this._setTheme();
            this._initialize();
            this._onWindowResize();
        },
        /**
        * To window resize for the Digital Gauge		
        * @private
        */
        _onWindowResize: function () {
            if (this.model.canResize) {
                proxy = this;
                if (!this.isDevice())
                    this._on($(window), "resize", proxy.resizeCanvas);
                else
                    this._on($(window), "orientationchange", proxy.resizeCanvas);
            }
        },
        /**
        * To check the Device of the Digital Gauge rendered	
        * @private
        */
        isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
        * To set theme for the Digital Gauge		
        * @private
        */
        _setTheme: function () {
            selectedThem = ej.DigitalGauge.themes[this.model.theme];
            this._setThemeColors(selectedThem);
        },
        /**
        * To apply theme colors to the selected theme for the Digital Gauge		
        * @private
        */
        _setThemeColors: function (selectedThem) {
            var result = [], jsonObj = ej.DigitalGauge.themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                for (var i = 0; i < this.model.items.length; i++) {
                    this.model.items[i].segmentColor = (!this.model.items[i].segmentColor || this.model.items[i].segmentColor == jsonObj[result[th]].items.segmentColor) ? selectedThem.items.segmentColor : this.model.items[i].segmentColor;
                    this.model.items[i].shadowColor = (!this.model.items[i].shadowColor || this.model.items[i].shadowColor == jsonObj[result[th]].items.shadowColor) ? selectedThem.items.shadowColor : this.model.items[i].shadowColor;
                    this.model.items[i].textColor = (!this.model.items[i].textColor || this.model.items[i].textColor == jsonObj[result[th]].items.textColor) ? selectedThem.items.textColor : this.model.items[i].textColor;
                }
            }
        },
        /**
        * To initialize the Digital Gauge		
        * @private
        */
        _initialize: function () {
            if (this.model.init)
                this._clientSideOnInit();
            this._initObject(this);
            if (this.model.load)
                this._clientSideOnLoad();
            if (this.model.frameBackgroundImageUrl != null)
                this._drawCustomImage(this, this.model.frameBackgroundImageUrl);
            else
                this._renderItems();
            if (this.model.renderComplete)
                this._clientSideOnRenderComplete();
        },
        /**
        * To initialize the items of Digital Gauge		
        * @private
        */
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.items != null) {
                $.each(this.model.items, function (index, element) {
                    var obj = proxy._sendDefaultItem();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.items = [this._sendDefaultItem()];
            }
        },
        /**
        * To set the default items values for the Digital Gauge		
        * @private
        */
        _sendDefaultItem: function () {
            var defaultItems;
            return defaultItems = {
                /**		
                * Specifies the CharacterCount value for the DigitalGauge.
                * @alias ejDigitalGauge#items->characterCount
                * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {int}
                * @example 
                * //To set CharacterCount during initialization  
                * $("#DigitalCore").ejDigitalGauge({items: [{ characterCount: 2 }] });
                * @example 
                * //Get or set the CharacterCount, after initialization:<br>             
                *  //Get the characterCount value
                *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                *    var result = item[index].characterCount;
                *  //index refers to the index value of the items.
                *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                *  //Sets the CharacterCount value
                *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ characterCount: 4 }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                characterCount: 1,
                /**		
                * Specifies the opacity value for the characters..
        		* @alias ejDigitalGauge#items->characterOpacity
                * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {int}
                * @example 
                * //To initialize the value for the characterOpacity 
                * $("#DigitalCore").ejDigitalGauge({items: [{ characterOpacity: 2 }] });
                * @example 
                * //To get/set the characterOpacity, after initialization:<br>             
                *  //Get the characterOpacity value
                *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                *    var result = item[index].characterOpacity;
                *  //index refers to the index value of the items.
                *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                *  //Sets the characterOpacity value
                *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ characterOpacity: 0.4  }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                characterOpacity: 1,
                /**		
                * Specifies the value for spacing between the characters
         		* @alias ejDigitalGauge#items->characterSpacing
                * @default 2                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {int}
                * @example 
                * //To initialize the value for the characterSpacing, 
                * $("#DigitalCore").ejDigitalGauge({items: [{ characterSpacing: 4 }] });
                * @example 
                * //To get/set the characterSpacing, after initialization:<br>             
                *  //Get the characterSpacing value
                *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                *    var result = item[index].characterSpacing;
                *  //index refers to the index value of the items.
                *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                *  //Set the characterSpacing value
                *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ characterSpacing: 2  }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                characterSpacing: 2,
                /**		
                * Specifies the character type for the text to be displayed.  See {@link characterType}
                * @alias ejDigitalGauge#items->characterType
                * @default ej.characterType.EightCrossEightDotMatrix                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {enum}
                * @example 
                * //To initialize the gauge with the characterType, 
                * $("#DigitalCore").ejDigitalGauge({items: [{ characterType: ej.characterType.EightCrossEightDotMatrix }] });
                * @example 
                * //To get/set the characterType, after initialization:<br>             
                *  //Get the type of the character
                *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                *    var result = item[index].characterType;
                *  //index refers to the index value of the items.
                *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                *  //Set the value for characterType
                *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ characterType: ej.characterType.SevenSegment  }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                characterType: ej.characterType.EightCrossEightDotMatrix,
                /**		
               * Enable/Disable the custom font to be applied to the text in the gauge. <br>
    	       * @alias ejDigitalGauge#items->enableCustomFont
               * @default false                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
               * @type {boolean}
               * @example 
               * //To initialize the gauge with the enableCustomFont, 
               * $("#DigitalCore").ejDigitalGauge({items: [{ enableCustomFont: true }] });
               * @example 
               * //To get/set the enableCustomFont, after initialization:<br>             
               *  //Get the enableCustomFont value
               *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
               *    var result = item[index].enableCustomFont;
               *  //index refers to the index value of the items.
               *    $("#DigitalCore").ejDigitalGauge("option", "items");			
               *  //Set the value for enableCustomFont
               *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ enableCustomFont: true  }]);
               * @memberof ejDigitalGauge
               * @instance
               */
                enableCustomFont: false,
                /**		
              * Set the length for the text segments. <br>
    	      * @alias ejDigitalGauge#items->segmentLength
              * @default 2                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
              * @type {int}
              * @example 
              * //To initialize the length of the segments, 
              * $("#DigitalCore").ejDigitalGauge({items: [{ segmentLength: 4 }] });
              * @example 
              * //To get/set the segmentLength, after initialization:<br>             
              *  //Get the segmentLength value
              *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
              *    var result = item[index].segmentLength;
              *  //index refers to the index value of the items.
              *    $("#DigitalCore").ejDigitalGauge("option", "items");			
              *  //Set the value for segmentLength
              *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ segmentLength: 2  }]);
              * @memberof ejDigitalGauge
              * @instance
              */
                segmentLength: 2,
                /**		
             * Set the opacity level for the text segments <br>
    	     * @alias ejDigitalGauge#items->segmentOpacity
             * @default 0                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
             * @type {double}
             * @example 
             * //To initialize the opacity level of the segments, 
             * $("#DigitalCore").ejDigitalGauge({items: [{ segmentOpacity: 1 }] }); 
             * @example 
             * //To get/set the segmentOpacity, after initialization:<br>             
             *  //Get the segment Opacity value
             *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
             *    var result = item[index].segmentOpacity;
             *  //index refers to the index value of the items.
             *    $("#DigitalCore").ejDigitalGauge("option", "items");			
             *  //Set the Opacity of the segment
             *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ segmentOpacity: 0.3   }]);
             * @memberof ejDigitalGauge
             * @instance
             */
                segmentOpacity: 0,
                /**		
             * Set the spacing between the text segments <br>
    	     * @alias ejDigitalGauge#items->segmentSpacing
             * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
             * @type {int}
             * @example 
             * //To initialize the spacing between the segments, 
             * $("#DigitalCore").ejDigitalGauge({items: [{ segmentSpacing: 5 }] });
             * @example 
             * //To get/set the segmentSpacing, after initialization:<br>             
             *  //Get the segment spacing value
             *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
             *    var result = item[index].segmentSpacing;
             *  //index refers to the index value of the items.
             *    $("#DigitalCore").ejDigitalGauge("option", "items");			
             *  //Set the spacing between the segment
             *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ segmentSpacing: 3   }]);
             * @memberof ejDigitalGauge
             * @instance
             */
                segmentSpacing: 1,
                /**		
                * Set the width for the text segments <br>
        		* @alias ejDigitalGauge#items->segmentWidth
                * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {int}
                * @example 
                * //To initialize the width of the segments, 
                * $("#DigitalCore").ejDigitalGauge({items: [{ segmentWidth:  4 }] }); 
                * @example 
                * //To get/set the segmentWidth, after initialization:<br>             
                *  //Get the segment width
                *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                *    var result = item[index].segmentSpacing;
                *  //index refers to the index value of the items.
                *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                *  //Set the width of the segment
                *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ segmentWidth: 3   }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                segmentWidth: 1,
                /**		
              * Set the value for enabling/disabling the blurring effect for the shadows of the text <br>
              * @alias ejDigitalGauge#items->shadowBlur
              * @default 0                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
              * @type {int}
              * @example 
              * //To initialize the value for shadowBlur, 
              * $("#DigitalCore").ejDigitalGauge({items: [{ shadowBlur:  1 }] });
              * @example 
              * //To get/set the value of shadowBlur, after initialization:<br>             
              *  //Get the value of shadowBlur
              *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
              *    var result = item[index].shadowBlur;
              *  //index refers to the index value of the items.
              *    $("#DigitalCore").ejDigitalGauge("option", "items");			
              *  //Set the value for shadowBlur
              *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ shadowBlur: 0   }]);
              * @memberof ejDigitalGauge
              * @instance
              */
                shadowBlur: 0,
                /**		
                * Set the x offset value for the shadow of the text, indicating the location where it needs to be displayed. <br>
         		* @alias ejDigitalGauge#items->shadowOffsetX
                * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {int}
                * @example 
                * //To initialize the value of shadowOffsetX, 
                * $("#DigitalCore").ejDigitalGauge({items: [{ shadowOffsetX:  2 }] });
                * @example 
                * //To get/set the value of shadowOffsetX, after initialization:<br>             
                *  //Get the value of shadowOffsetX
                *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                *    var result = item[index].shadowOffsetX;
                *  //index refers to the index value of the items.
                *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                *  //Set the value for shadowOffsetX
                *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ shadowOffsetX: 10   }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                shadowOffsetX: 1,
                /**		
                * Set the y offset value for the shadow of the text, indicating the location where it needs to be displayed. <br>
        		* @alias ejDigitalGauge#items->shadowOffsetY
                * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {int}
                * @example 
                * //To initialize the value of shadowOffsetY, 
                * $("#DigitalCore").ejDigitalGauge({items: [{ shadowOffsetY:  12 }] });	
                * @example 
                * //To get/set the value of shadowOffsetY, after initialization:<br>             
                *  //Get the value of shadowOffsetY
                *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                *    var result = item[index].shadowOffsetY;
                *  //index refers to the index value of the items.
                *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                *  //Set the value for shadowOffsetY
                *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ shadowOffsetY: 20   }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                shadowOffsetY: 1,
                /**		
                * Set the alignment of the text that is displayed within the gauge.See {@link textAlignment}       
        		* @alias ejDigitalGauge#items->textAlign		
                * @default Left                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {string}
                * @example 
                * //To initialize the text alignment of the Digital gauge, 
                * $("#DigitalCore").ejDigitalGauge({items: [{ textAlign:  "Right" }] });
                * @example 
                * //To get/set the text alignment, after initialization<br>             
                *  //Get the text alignment of the gauge
                *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                *    var result = item[index].textAlign;
                *  //index refers to the index value of the items.
                *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                *  //Set the text alignment of the gauge 
                *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ textAlign: "Right"   }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                textAlign: "Left",
                /**		
               * Set the specific font for the text, when the enableCustomFont is set to true <br>
    	       * @alias ejDigitalGauge#items->font
               * @memberof ejDigitalGauge
               * @instance
               */
                font:/** @lends ejDigitalGauge# */ {
                    /**		
                    * Set the font size value <br>
	        	    * @alias ejDigitalGauge#items->font->size
                    * @default 11px                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {string}
                    * @example 
                    * //To initialize the font for the text, 
                    * $("#DigitalCore").ejDigitalGauge({items: [{font: { size: "12px"}});
                    * @example 
                    * //To get/set the value of font size, after initialization:<br>             
                    *  //Get the value of font size
                    *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                    *    var result = item[index].font;
                    *  //index refers to the index value of the items.
                    *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                    *  //Set the size value of font
                    *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ font: { size: "12px"} }]);
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    size: '11px',
                    /**		
                    * Set the font family value <br>
        		    * @alias ejDigitalGauge#items->font->fontFamily
                    * @default Arial                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {string}
                    * @example 
                    * //To initialize the font family for the text, 
                    * $("#DigitalCore").ejDigitalGauge({items: [{font: { fontFamily: "Segou"}});
                    * @example 
                    * //To get/set the value of font family, after initialization:<br>             
                    *  //Get the value of font family
                    *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                    *    var result = item[index].font;
                    *  //index refers to the index value of the items.
                    *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                    *  //Set the font family value of font
                    *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ font: { fontFamily: "Segou"} }]);
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    fontFamily: "Arial",
                    /**		
                    * Set the font style for the font. See {@link fontStyle}<br>
				    * @alias ejDigitalGauge#items->font->fontStyle
                    * @default Italic                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {string}
                    * @example 
                    * //To initialize the font style for the text, 
                    * $("#DigitalCore").ejDigitalGauge({items: [{font: { fontStyle: "Bold" }});
                    * @example 
                    * //To get/set the value of font style, after initialization:<br>             
                    *  //Get the value of font style
                    *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                    *    var result = item[index].font;
                    *  //index refers to the index value of the items.
                    *    $("#DigitalCore").ejDigitalGauge("option", "items");			
                    *  //Set the font style value of font
                    *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ font: { fontStyle: "Bold" } }]);
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    fontStyle: "Italic"
                },
                /**		
             * Set the location for the text, where it needs to be placed within the gauge. <br>
		     * @alias ejDigitalGauge#items->location
             * @default { x: 0, y: 0 }                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
             * @type {object}
             * @example 
             * //To initialize the value for location, 
             * $("#DigitalCore").ejDigitalGauge({items: [{location: { x: 10, y: 20 } }});
             * @example 
             * //To get/set the value of the text location, after initialization:<br>             
             *  //Get the text location
             *    var item = $("#DigitalCore").ejDigitalGauge("option", "items");
             *    var result = item[index].location;
             *  //index refers to the index value of the items.
             *    $("#DigitalCore").ejDigitalGauge("option", "items");			
             *  //Set the location for the text.
             *    $("#DigitalCore").ejDigitalGauge("option", "items", [{ location: { x: 10, y: 20 }  }]);
             * @memberof ejDigitalGauge
             * @instance
             */
                location: { x: 0, y: 0 },
                segmentColor: null,
                /**		
                * Specifies the color of the text shadow.
                * @default null
                * @type {string}
                * @example 
                * //To initialize the shadow color  
                * $("#DigitalCore").ejDigitalGauge({ shadowColor: #FF1F2F });
                * @example 
                * //To get/set the color of the shadows, after initialization:<br>
                * //Gets the shadow Color
                *   var item = $("#DigitalCore").ejDigitalGauge("option", "items");
                * //index refers to the index value of the items.
                *   var result = item[index].shadowColor; 
                * //Set the color of the shadows
                *   $("#DigitalCore").ejDigitalGauge("option", "items", [{ shadowColor: "#FF1F2F" }]);
                * @memberof ejDigitalGauge
                * @instance
                */
                shadowColor: null,
                /**		
         * Specifies the color of the text.
         * @default null
         * @type {string}
         * @example 
         * //To initialize the text color  
         * $("#DigitalCore").ejDigitalGauge({ textColor: #FF1F2F });
         * @example 
         * //To get/set the color of the shadows, after initialization:<br>
         * //Gets the text Color
         *   var item = $("#DigitalCore").ejDigitalGauge("option", "items");
         * //index refers to the index value of the items.
         *   var result = item[index].textColor; 
         * //Set the color of the text
         *   $("#DigitalCore").ejDigitalGauge("option", "items", [{ textColor: "#FF1F2F" }]);
         * @memberof ejDigitalGauge
         * @instance
         */
                textColor: null,
                value: null
            };
        },
        /**
        * To init the object for the Digital Gauge		
        * @private
        */
        _initObject: function (element) {
            this.element.addClass("e-widget");
            element.gaugeEl = this.element;
            for (var i = 0; this.model.items[i] != null; i++) {
                if (this.model.items[i].value == null)
                    this.model.items[i].value = this.value();
            }
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.gaugeEl.append(element.canvasEl);
            element.canvasEl.attr("role", "presentation");
            initialDivWidth = element.gaugeEl.width();
            initialDivHeight = element.gaugeEl.height();
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
        /**
        * To draw the custom image for the Digital Gauge		
        * @private
        */
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            var self = this;
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.model.width, element.model.height);
                if (element.model.Scales != null)
                    element._drawScales();
                if (element.model.items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
        /**
        * To set the SegmentCount value to the Digital Gauge		
        * @private
        */
        _setSegmentCount: function (element) {
            switch (element) {
                case "SevenSegment": this._SegmentCount = 7; break;
                case "FourteenSegment": this._SegmentCount = 14; break;
                case "SixteenSegment": this._SegmentCount = 16; break;
                case "EightCrossEightDotMatrix": this._SegmentCount = [8, 8]; break;
                case "EightCrossEightSquareMatrix": this._SegmentCount = [8, 8]; break;
                default:
                    this._SegmentCount = 7;
            }

        },
        /**
        * To set the InnerPosition of the Digital Gauge		
        * @private
        */
        _setInnerPosition: function () {
            this.contextEl.save();
            this.contextEl.translate(this.model.frameOuterWidth + this.model.frameInnerWidth, this.model.frameOuterWidth + this.model.frameInnerWidth);
            this.bounds = {
                height: this.canvasEl[0].height - 2 * (this.model.frameOuterWidth + this.model.frameInnerWidth),
                width: this.canvasEl[0].width - 2 * (this.model.frameOuterWidth + this.model.frameInnerWidth)
            };
        },
        /**
        * To set the width to the Digital Gauge		
        * @private
        */
        _setWidth: function () {
            var characterCount = [];
            if (this.model.items != null) {
                var self = this;
                $.each(this.model.items, function (index, element) {
                    characterCount.push(element.characterCount);
                });
            }
        },

        /**
        * Render the items for Digital Gauge		
        * @private
        */
        _renderItems: function () {
            if (this.model.items != null) {
                this._setInnerPosition();
                var self = this;
                $.each(this.model.items, function (index, element) {
                    self._setSegmentCount(element.characterType);
                    self.itemIndex = index;
                    self.canvasEl.attr("aria-label", element.value);
                    self._setShadow(index, element);
                    if (element.enableCustomFont)
                        self._setCustomFont(index, element);
                    else if (element.characterType.indexOf("Matrix") != -1)
                        self._drawMatrixSegments(index, element);
                    else
                        self._drawSegments(index, element);
                });
            }
        },

        /**
        * To set the GradientColor to Digital Gauge		
        * @private
        */
        _setGradientColor: function (element, gradient, options) {
            var self = element;
            if (options.Name || typeof (options) === "string") {
                gradient.addColorStop(0, this._getColor(element, options));
                gradient.addColorStop(1, this._getColor(element, options));
            }
            else
                $.each(options, function (index, colorElement) {
                    gradient.addColorStop(colorElement.ColorStop != NaN ? colorElement.ColorStop : 0, typeof (colorElement.Color) === "string" ? colorElement.Color : this._getColor(element, colorElement.Color));
                });
        },

        /**
        * To get the color of the Digital Gauge		
        * @private
        */
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.R + ", " + option.G + "," + option.B + ", " + option.A / 255 + ")");
            }
        },

        /**
        * To draw the matrix segments to the Digital Gauge		
        * @private
        */
        _drawMatrixSegments: function (index, element) {
            var segmentXCollection = [], segmentCollection = [];
            if (element.value) {
                this._value = element.value.toString().split('');
                if (element.characterCount < this._value.length)
                    element.characterCount = this._value.length;
            }
            else
                this._value = "";
            this.radius = (element.characterType.indexOf("Dot") != -1) ? (element.segmentLength + element.segmentWidth) / 2 : element.segmentWidth / 2;
            var controlStartX = this.startX = (this.bounds.width - element.characterCount * (this._SegmentCount[0] * (2 * this.radius) + element.characterSpacing + this._SegmentCount[0] * element.segmentSpacing)) * (element.location.x / 100);
            var controlStartY = this.startY = (this.bounds.height - (this._SegmentCount[1] * ((element.characterType.indexOf("Dot") != -1) ? 2 * this.radius : element.segmentLength) + this._SegmentCount[1] * element.segmentSpacing)) * (element.location.y / 100);
            for (var character = 0; character < element.characterCount; character++) {
                if (this._value) {
                    this.character = element.textAlign == "Right" ? this._value[this._value.length - element.characterCount + character] : this._value[character];
                    segmentCollection = this.model.matrixSegmentData[this.character];
                }
                if (character != 0) {
                    controlStartX = this.startX = this.startX + element.characterSpacing + element.segmentSpacing + (2 * this.radius);
                    this.startY = controlStartY;
                }
                for (var dotY = 0; dotY < this._SegmentCount[1]; dotY++) {
                    if (dotY != 0) {
                        this.startY = ((element.characterType.indexOf("Dot") != -1) ? (2 * this.radius) : element.segmentLength) + this.startY + element.segmentSpacing;
                        this.startX = controlStartX;
                    }
                    if (segmentCollection) {
                        $.each(segmentCollection, function (index) {
                            if (index % 2 == 0) {
                                if (segmentCollection[index] > dotY)
                                    return false;
                                if (segmentCollection[index] == dotY)
                                    segmentXCollection.push(parseInt(segmentCollection[index + 1]));
                            }
                        });
                    }
                    for (var dotX = 0; dotX < this._SegmentCount[0]; dotX++) {
                        if (dotX != 0)
                            this.startX = this.startX + 2 * this.radius + element.segmentSpacing;
                        if (element.characterType.indexOf("Dot") != -1)
                            this.gradient = this.contextEl.createRadialGradient(0, 0, 0, 0, 0, this.radius);
                        else
                            this.gradient = this.contextEl.createLinearGradient(0, 0, element.segmentWidth, 0);
                        if (element.SegmentGradient)
                            this._setGradientColor(this, this.gradient, element.SegmentGradient.ColorInfo);
                        this.location = { "startX": this.startX, "startY": this.startY };
                        this.style = {
                            "opacity": (segmentXCollection && ($.inArray(dotX, segmentXCollection)) != -1) ? element.characterOpacity : element.segmentOpacity,
                            "height": element.segmentLength,
                            "width": element.segmentWidth,
                            "fillStyle": ((element.segmentColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.segmentColor)),
                            "skewX": element.SkewAngleX,
                            "skewY": element.SkewAngleX
                        };
                        if (this.model.onItemRendering)
                            this._clientSideOnItemRendering(true, dotX, dotY);
                        if (element.characterType.indexOf("Dot") != -1)
                            this._drawDot(this.location, this.style);
                        else
                            this._drawSquare(this.location, this.style);
                    }
                    segmentXCollection = [];
                }
            }
        },

        /**
        * Draw the segments of Digital Gauge		
        * @private
        */
        _drawSegments: function (index, element) {
            var segmentCollection = [], characterSpace, self = this;
            if (element.value) {
                this._value = element.value.toUpperCase().toString().split('');
                if (element.characterCount < this._value.length)
                    element.characterCount = this._value.length;
            }
            this.characterSpace = element.characterType == "SevenSegment" ? 2 * element.segmentWidth : 4 * element.segmentWidth;
            this._renderSegmentCalculation(element);
            this.gradient = this.contextEl.createLinearGradient(0, 0, 0, element.segmentWidth);
            if (element.segmentColor)
                this._setGradientColor(this, this.gradient, element.segmentColor);
            else if (element.SegmentGradient)
                this._setGradientColor(this, this.gradient, element.SegmentGradient.ColorInfo);
            else
                this._setGradientColor(this, this.gradient, this.model.segmentColor);
            for (var character = 0; character < element.characterCount; character++) {
                if (element.value)
                    segmentCollection = this.model.segmentData[element.textAlign == "Right" ? this._value[this._value.length - element.characterCount + character] : this._value[character]];
                for (var segment = 0; segment < this._SegmentCount; segment++) {
                    if (character != 0)
                        this.segment16X[segment] = this.segment16X[segment] + element.segmentLength + this.characterSpace + element.characterSpacing;
                    if (this._value)
                        this.character = element.textAlign == "Right" ? this._value[this._value.length - element.characterCount + character] : this._value[character]
                    this.location = { "startX": this.segment16X[segment], "startY": this.segment16Y[segment] };
                    this.style = {
                        "angle": this.angle[segment],
                        "fillStyle": this.gradient,
                        "isStroke": false,
                        "isFill": true,
                        "characterHeight": element.characterType == "SevenSegment" ? element.segmentLength : this.segmentHeight[segment],
                        "segmentWidth": element.segmentWidth,
                        "opacity": (segmentCollection && ($.inArray(segment, segmentCollection) != -1)) ? element.characterOpacity : element.segmentOpacity
                    };
                    if (this.model.onItemRendering)
                        this._clientSideOnItemRendering(false, segment);
                    this._drawSegmentLayers(this.location, this.style);
                }
                this.value(element.value);
                segmentCollection = [];
            }
        },

        /**
        * To set the CustomFont style to Digital Gauge		
        * @private
        */
        _setCustomFont: function (index, element) {
            this.startX = (this.bounds.width - this._measureText(element.value, 0, this._getFontString(this, element.font)).width) * (element.location.x / 100);
            this.startY = (this.bounds.height - this._measureText(element.value, 0, this._getFontString(this, element.font)).height) * (element.location.y / 100);
            this.location = { "startX": this.startX, "startY": this.startY };
            this.style = { "font": this._getFontString(this, element.font), "text": element.value, "textColor": element.textColor ? ((element.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.textColor)) : ((this.model.segmentColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this.model.segmentColor)) };
            if (this.model.onItemRendering)
                this._clientSideOnItemRendering(false, segment);
            this._drawText(this.location, this.style);
        },

        /**
        * To get the Font String value of Digital Gauge		
        * @private
        */
        _getFontString: function (element, font) {
            return ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily + " " + (font.fontStyle ? font.fontStyle : "");
        },

        /**
        * Rendering the SegmentCalculation of Digital Gauge		
        * @private
        */
        _renderSegmentCalculation: function (element) {
            this.startX = (this.bounds.width - element.characterCount * (element.segmentLength + this.characterSpace + element.characterSpacing)) * (element.location.x / 100);
            this.startY = (this.bounds.height - 2 * element.segmentLength - element.segmentWidth) * (element.location.y / 100);
            var tempLength = element.characterType == "SevenSegment" ? element.segmentLength : element.segmentLength / 2;
            this.segment16X = [
          this.startX + element.segmentWidth / 2,
          this.startX + element.segmentLength + 4 * element.segmentWidth,
          this.startX + element.segmentLength + 4 * element.segmentWidth,
          this.startX + element.segmentWidth / 2,
          this.startX,
          this.startX,
          this.startX + element.segmentWidth / 2,
          this.startX + tempLength + 2 * element.segmentWidth,
          this.startX + 2.5 * element.segmentWidth + tempLength,
          this.startX + tempLength + 2 * element.segmentWidth,
          this.startX + element.segmentLength + 2.5 * element.segmentWidth,
          this.startX + tempLength + 2.5 * element.segmentWidth,
          this.startX + tempLength + 1.5 * element.segmentWidth,
          this.startX + 1.5 * element.segmentWidth,
          this.startX + 5 * element.segmentWidth / 2 + tempLength,
          this.startX + 2.5 * element.segmentWidth + tempLength
            ];
            this.segment16Y = [
          this.startY,
          this.startY,
          this.startY + element.segmentLength + element.segmentWidth,
          this.startY + 2 * element.segmentLength + 2 * element.segmentWidth,
          this.startY + element.segmentLength + element.segmentWidth,
          this.startY,
          this.startY + element.segmentLength + element.segmentWidth,
          this.startY + element.segmentLength + element.segmentWidth,
          this.startY + element.segmentLength + element.segmentWidth,
          this.startY,
          this.startY + element.segmentWidth,
          this.startY + element.segmentLength + element.segmentWidth,
          this.startY + element.segmentLength + element.segmentWidth,
          this.startY + element.segmentWidth,
          this.startY,
          this.startY + 2 * element.segmentLength + 2 * element.segmentWidth
            ];
            this.segmentHeight = [
          element.segmentLength / 2,
          element.segmentLength,
          element.segmentLength,
          element.segmentLength / 2,
          element.segmentLength,
          element.segmentLength,
          element.segmentLength / 2,
          element.segmentLength,
          element.segmentLength / 2,
          element.segmentLength,
          element.segmentLength,
          element.segmentLength,
          element.segmentLength,
          element.segmentLength,
          element.segmentLength / 2,
          element.segmentLength / 2
            ];
            this.angle = [-90, 0, 0, -90, 0, 0, -90, 0, -90, 0, 27, -27, 27, -27, -90, -90];
            if (element.characterType == "SevenSegment")
                this.segment16X[2] = this.segment16X[1] = this.startX + element.segmentLength + 2 * element.segmentWidth;
            if (element.characterType == "FourteenSegment")
                this.segmentHeight[3] = this.segmentHeight[0] = element.segmentLength + 2 * element.segmentWidth;
        },

        /**
        * Drawing the Segment Layers of the Digital Gauge		
        * @private
        */
        _drawSegmentLayers: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.rotate(Math.PI * (style.angle / 180));
            this.contextEl.moveTo(0, 0);
            this.contextEl.lineTo(-style.segmentWidth, style.segmentWidth);
            this.contextEl.lineTo(-style.segmentWidth, style.characterHeight);
            this.contextEl.lineTo(0, style.characterHeight + style.segmentWidth);
            this.contextEl.lineTo(style.segmentWidth, style.characterHeight);
            this.contextEl.lineTo(style.segmentWidth, style.segmentWidth);
            this._contextClosePath(style, this);
        },

        /**
        * Drawing the Dot of the Digital Gauge		
        * @private
        */
        _drawDot: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.globalAlpha = style.opacity;
            this.contextEl.arc(0, 0, this.radius, 0, 2 * Math.PI, true);
            this.contextEl.fill();
            this.contextEl.closePath();
            this.contextEl.restore();
        },

        /**
        * To set the Shadow of the Digital Gauge		
        * @private
        */
        _setShadow: function (index, element) {
            this.contextEl.save();
            this.contextEl.shadowColor = ((element.shadowColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.shadowColor));
            this.contextEl.shadowOffsetY = element.shadowOffsetY;
            this.contextEl.shadowOffsetY = element.shadowOffsetX;
            this.contextEl.shadowBlur = element.shadowBlur;
        },

        /**
       * Drawing the Square to Digital Gauge		
       * @private
       */
        _drawSquare: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.translate(location.startX, location.startY);
            // this.contextEl.setTransform(1, style.skewY, style.skewX, 1, location.startX, location.startY);
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.globalAlpha = style.opacity;
            this.contextEl.rect(0, 0, style.width, style.height);
            this.contextEl.fill();
            this.contextEl.closePath();
            this.contextEl.restore();
        },

        /**
       * Drawing the Text value of Digital Gauge		
       * @private
       */

        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.font = style.font;
            this.contextEl.textBaseline = "hanging";
            this.contextEl.fillStyle = ((style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor);
            this.contextEl.fillText(style.text, location.startX, location.startY);
            this.contextEl.closePath();
            this.contextEl.restore();
        },

        /**
       * ClientSideMethod SetValue
       * Sets the value of an item to be displayed in the gauge.
       * @alias setValue 
       * @param {int} itemIndex  Index value of the digital gauge 
       * @param {string} value  Text value to be displayed in the gauge
       * @example 
       * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
       * &lt;script&gt;
       * $("#DigitalGauge1").ejDigitalGauge("setValue", 0, "Welcome");	
       * // The above code sets the value as "Welcome" on an item with the index 0.
       * &lt;/script&gt;
       * @memberof ejDigitalGauge
       * @instance
        */

        setValue: function (itemIndex, value) {
            if (itemIndex < this.model.items.length && value != null) {
                this.model.items[itemIndex].value = value;
                if (this.model.frameBackgroundImageUrl)
                    this._initialize();
                else
                    this._initialize();
            }
        },

        /**
         * ClientSideMethod getValue
         * Gets the value of an item that is displayed on the gauge
         * @alias getValue       
         * @param {int}  itemIndex Index value of an item that displayed on the gauge      
         * @example 
         * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * $("#DigitalGauge1").ejDigitalGauge("getValue", 0);	
         * &lt;/script&gt;
         * //The above code retrieves the value of an item with the index 0.
         * @memberof ejDigitalGauge
         * @instance
         */
        getValue: function (itemIndex) {
            return this.model.items[itemIndex].value;
        },

        /**
      * ClientSideMethod SetLocation
      * Sets the location of an item to be displayed in the gauge
      * @alias setLocation       
      * @param {int} itemIndex  Index value of the digital gauge 
      * @param {object} value  Location value of the digital gauge
      * @example 
      * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
      * $("#DigitalGauge1").ejDigitalGauge("setLocation", 0, { x:50, y:40 });	
      * &lt;/script&gt;
      * //The above code sets the location for an item with the index 0.
      * @memberof ejDigitalGauge
      * @instance
       */

        setLocation: function (itemIndex, value) {
            if (itemIndex < this.model.items.length && value != null) {
                this.model.items[itemIndex].location.x = value.x;
                this.model.items[itemIndex].location.y = value.y;
                if (this.model.frameBackgroundImageUrl)
                    this._initialize();
                else if (this.contextEl.putImageData) {
                    this.contextEl.putImageData(this.outerImage, 0, 0);
                    this._renderItems();
                }
                else
                    this._initialize();
            }
        },


        /**
        * Gets the location of an item that is displayed on the gauge.
        * @alias getLocation       
        * @param {int}  itemIndex Location value of an item that is displayed on the gauge.     
        * @example 
        * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#DigitalGauge1").ejDigitalGauge("getLocation", 0);	
        * &lt;/script&gt;
        * //The above code retrieves the location of an item with the index 0.
        * @memberof ejDigitalGauge
        * @instance
        */

        getLocation: function (itemIndex) {
            if (itemIndex < this.model.items.length)
                return { "x": this.model.items[itemIndex].location.x, "y": this.model.items[itemIndex].location.y };
        },

        /**
      * Refresh the digital gauge widget
      * @alias refresh
      * @example 
      * &lt;div id="DigitalGauge"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
      * //initialize the DigitalGauge object
      *   var GaugeObj = $("#DigitalGauge").data("ejDigitalGauge");
      * //calls the refresh method of Digital gauge
      *   GaugeObj.refresh();
      * //An alternative way of refreshing the gauge without creating object is as follows
      * $("#DigitalGauge").ejDigitalGauge("refresh"); // refresh the Digital Gauge
      * &lt;/script&gt;     
      * @memberof ejDigitalGauge     
      * @instance      
      */
        refresh: function () {
            this._setTheme();
            this._initialize();
        },

        /**
         * To export Digital Gauge as Image
         * @return jQuery
         * @param {string} argument.fileName fileName for the Image
         * @param {string} argument.fileType fileType for the Image
         * @example 
         * &lt;div id="DigitalGauge"&gt;DigitalGauge&lt;/div&gt; <br> 
         * &lt;script&gt;
         * //To Export Image
         * var DigitalGaugeObj = $("#DigitalGauge").data("ejDigitalGauge");
         * DigitalGaugeObj.exportImage("myImage","jpeg");
         * &lt;/script&gt;
         * @example 
         * &lt;div id="DigitalGauge"&gt;DigitalGauge&lt;/div&gt; <br> 
         * &lt;script&gt;
         * //To Export Image
         * $("#DigitalGauge").ejDigitalGauge("exportImage('myImage','png')"); 
         * &lt;/script&gt;
         *@memberof ejDigitalGauge
         * @instance
       */
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            var lnk = document.createElement('a'), e;
            lnk.download = fileName + "." + fileType;
            lnk.href = this.canvasEl[0].toDataURL("image/" + fileType).replace("image/" + fileType, "image/octet-stream");
            if (document.createEvent) {
                e = document.createEvent("MouseEvents");
                e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                lnk.dispatchEvent(e);
            }
            else if (lnk.fireEvent) {
                lnk.fireEvent("onclick");
            }
        },

        /**
       * ClientSideMethod ResizeCanvas		
       * @private
       */
        resizeCanvas: function () {
            if (proxy.element.width() != initialDivWidth || proxy.element.height() != initialDivHeight) {
                var canvasHeight = proxy.element.height() / initialDivHeight;
                var canvasWidth = proxy.element.width() / initialDivWidth;
                proxy.model.width *= canvasWidth;
                proxy.model.height *= canvasHeight;
                for (var i = 0; proxy.model.items[i] != null; i++) {
                    proxy.model.items[i].segmentWidth *= canvasWidth;
                    proxy.model.items[i].segmentSpacing *= canvasWidth;
                }
                proxy.refresh();
                initialDivWidth = proxy.element.width();
                initialDivHeight = proxy.element.height();
            }
        },

        //ClientSide events
        /**
       * ClientSide events _clientSideOnLoad		
       * @private
       */
        _clientSideOnLoad: function () {
            var data = { Object: this, items: this.model.items, context: this.contextEl };
            this._trigger("load", data);
        },

        /**
       * ClientSide events _clientSideOnItemRendering		
       * @private
       */
        _clientSideOnItemRendering: function (isMatrix, row, column) {
            var data;
            if (isMatrix)
                data = { Object: this, items: this.model.items, character: this.character, context: this.contextEl, location: this.location, style: this.style, row: row, column: column };
            else
                data = { Object: this, model: this.model, id: this.model.ClientId, items: this.model.items, character: this.character, context: this.contextEl, location: this.location, style: this.style, segment: row };
            this._trigger("onItemRendering", data);
        },

        /**
        * ClientSide events _clientSideOnInit		
        * @private
        */
        _clientSideOnInit: function () {
            var data = { Object: this, items: this.model.items, context: this.contextEl };
            this._trigger("init", data);
        },

        /**
        * ClientSide events _clientSideOnRenderComplete		
        * @private
        */
        _clientSideOnRenderComplete: function () {
            var data = { Object: this, items: this.model.items, context: this.contextEl };
            this._trigger("renderComplete", data);
        },


        /*--------common js ----------*/
        /**
        * ClientSide events _contextOpenPath		
        * @private
        */
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },

        /**
       * ClientSide events _contextClosePath		
       * @private
       */
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },

        /**
       * ClientSide events _measureText		
       * @private
       */
        _measureText: function (text, maxwidth, font) {
            var textObj = document.createElement('DIV');
            textObj.innerHTML = text;
            if (font != null)
                textObj.style.font = font;
            textObj.style.backgroundColor = 'white';
            textObj.style.position = 'absolute';
            textObj.style.top = -100;
            textObj.style.left = 0;
            if (maxwidth)
                textObj.style.maxwidth = maxwidth + "px";
            document.body.appendChild(textObj);
            var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
            $(textObj).remove();
            return bounds;
        }

    });

    /**
	 * Enum for gauge characterType	 
	 * @enum {string}
	 * @global 
	 */
    ej.characterType = {
        /**  To set the characterType SevenSegment. */
        SevenSegment: "SevenSegment",
        /**  To set the characterType FourteenSegment. */
        FourteenSegment: "FourteenSegment",
        /**  To set the characterType SixteenSegment. */
        SixteenSegment: "SixteenSegment",
        /**  To set the characterType EigthtCrossEightDotMatrixSegment. */
        EightCrossEightDotMatrix: "EightCrossEightDotMatrix",
        /**  To set the characterType EightCrossEightSquareMatrixSegment. */
        EightCrossEightSquareMatrix: "EightCrossEightSquareMatrix"
    };
    /**
	 * Enum for gauge textAlignment	 
	 * @enum {string}
	 * @global 
	 */
    ej.textAlignment = {
        /**  To set the textAlignment Left. */
        Left: "Left",
        /**  To set the textAlignment Right. */
        Right: "Right"
    };

    /**
	 * Enum for gauge fontStyle	 
	 * @enum {string}
	 * @global 
	 */
    ej.fontStyle = {
        /**  To set the fontStyle Normal. */
        Normal: "Normal",
        /**  To set the fontStyle Bold. */
        Bold: "Bold",
        /**  To set the fontStyle Italic. */
        Italic: "Italic",
        /**  To set the fontStyle Underline. */
        Underline: "Underline",
        /**  To set the fontStyle Strikeout. */
        Strikeout: "Strikeout"
    };

    /**
	 * Enum for gauge themes	 
	 * @enum {string}
	 * @global 
	 */
    ej.DigitalGauge.themes = {
        /**  To set the themes flatlight. */
        flatlight: {
            items: {
                segmentColor: "#232323",
                shadowColor: "#232323",
                textColor: "#232323"
            }
        },
        /**  To set the themes flatdark. */
        flatdark: {
            items: {
                segmentColor: "#b1b0b0",
                shadowColor: "#b1b0b0",
                textColor: "#b1b0b0"
            }
        }
    };

})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
       * @namespace ej
       * @class ejBulletGraph
       * @requires jQuery
       * @requires ej.common.all
       * @classdesc Custom Design for Bullet graph.
       * @example 
       * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; <br> 
       * &lt;script&gt;
       * // Create BulletGraph
       * $('#bulletGraph1').ejBulletGraph(); 	
       * &lt;/script&gt;
       */
    ej.widget("ejBulletGraph", "ej.BulletGraph", /** @lends ejBulletGraph# */ {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div"],
        // default model
        defaults:/** @lends ejBulletGraph# */ {
            /**		
			* Specifies the value of the Graph.
			* @default 0
            * @alias ejBulletGraph#value
			* @type {number}
			* @example 
			* //To set value API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  size: 5.5});
			* @example 
			* //Get or set the value API, after initialization:<br>
			*	//Gets the value  
			*	$("#bulletGraph1").ejBulletGraph("option", "value");<br>			
			*	//Sets the value 
			*	$("#bulletGraph1").ejBulletGraph("option", "value",  5.5 ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            value: 0,
            /**		
			* Specifies the comparativeMeasureValue of the Graph.
			* @default 0
            * @alias ejBulletGraph#comparativeMeasureValue
			* @type {number}
			* @example 
			* //To set comparativeMeasureValue API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  comparativeMeasureValue: 5.5});	
			* @example 
			* //Get or set the comparativeMeasureValue API, after initialization:<br>
			*	//Gets the comparativeMeasureValue  
			*	$("#bulletGraph1").ejBulletGraph("option", "comparativeMeasureValue");<br>			
			*	//Sets the comparativeMeasureValue 
			*	$("#bulletGraph1").ejBulletGraph("option", "comparativeMeasureValue",  5.5 ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            comparativeMeasureValue: 0,
            /**		
			* Specifies the width of the Graph.
			* @default 595
            * @alias ejBulletGraph#width
			* @type {number}
			* @example 
			* //To set width API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  width: 595});
			* @example 
			* //Get or set the width API, after initialization:<br>
			*	//Gets the width  
			*	$("#bulletGraph1").ejBulletGraph("option", "width");<br>			
			*	//Sets the width 
			*	$("#bulletGraph1").ejBulletGraph("option", "width",  595 ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            width: 595,
            /**		
			* Specifies the height of the Graph.
			* @default 90
            * @alias ejBulletGraph#height
			* @type {number}
			* @example 
			* //To set height API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  height: 595});	
			* @example 
			* //Get or set the height API, after initialization:<br>
			*	//Gets the height  
			*	$("#bulletGraph1").ejBulletGraph("option", "height");<br>			
			*	//Sets the height 
			*	$("#bulletGraph1").ejBulletGraph("option", "height",  595 ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            height: 90,
            /**		
			* Specifies the theme of the Graph.
			* @default "flatlight"
            * @alias ejBulletGraph#theme
			* @type {string}
			* @example 
			* //To set theme API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  theme: "flatlight"});	
			* @example 
			* //Get or set the theme API, after initialization:<br>
			*	//Gets the theme  
			*	$("#bulletGraph1").ejBulletGraph("option", "theme");<br>			
			*	//Sets the theme 
			*	$("#bulletGraph1").ejBulletGraph("option", "theme",  "flatlight" ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            theme: "flatlight",
            /**		
			* Specifies the orientation of the Graph.
			* @default "horizontal"
            * @alias ejBulletGraph#orientation
			* @type {enum}
			* @example 
			* //To set orientation API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  orientation: ej.BulletGraph.orientation.horizontal});
			* @example 
			* //Get or set the orientation API, after initialization:<br>
			*	//Gets the orientation  
			*	$("#bulletGraph1").ejBulletGraph("option", "orientation");<br>			
			*	//Sets the orientation 
			*	$("#bulletGraph1").ejBulletGraph("option", "orientation",  ej.BulletGraph.orientation.horizontal ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            orientation: "horizontal",
            /**		
			* Specifies the flowDirection of the Graph.See {@link flowDirection}
			* @default "forward"
            * @alias ejBulletGraph#flowDirection
			* @type {enum}
			* @example 
			* //To set flowDirection API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  flowDirection: ej.BulletGraph.flowDirection.forward});	 
			* @example 
			* //Get or set the flowDirection API, after initialization:<br>
			*	//Gets the flowDirection  
			*	$("#bulletGraph1").ejBulletGraph("option", "flowDirection");<br>			
			*	//Sets the flowDirection 
			*	$("#bulletGraph1").ejBulletGraph("option", "flowDirection",  ej.BulletGraph.flowDirection.forward ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            flowDirection: "forward",
            /**		
			* Specifies the qualitativeRangeSize of the Graph.
			* @default 32
            * @alias ejBulletGraph#qualitativeRangeSize
			* @type {number}
			* @example 
			* //To set qualitativeRangeSize API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  qualitativeRangeSize: 32});	
			* @example 
			* //Get or set the qualitativeRangeSize API, after initialization:<br>
			*	//Gets the qualitativeRangeSize  
			*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRangeSize");<br>			
			*	//Sets the qualitativeRangeSize 
			*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRangeSize", 32 ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            qualitativeRangeSize: 32,
            /**		
			* Specifies the quantitativeScaleLength of the Graph.
			* @default 475
            * @alias ejBulletGraph#quantitativeScaleLength
			* @type {number}
			* @example 
			* //To set quantitativeScaleLength API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScaleLength: 475});	 
			* @example 
			* //Get or set the quantitativeScaleLength API, after initialization:<br>
			*	//Gets the quantitativeScaleLength  
			*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScaleLength");<br>			
			*	//Sets the quantitativeScaleLength 
			*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScaleLength", 475 ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            quantitativeScaleLength: 475,
            /**		
			* Specifies the Tooltip state
			* @default true
            * @alias ejBulletGraph#showTooltip
			* @type {boolean}
			* @example 
			* //To set showTooltip API value during initialization 
			* 	$("#bulletGraph1").ejBulletGraph({  showTooltip: true});
			* @example 
			* //Get or set the showTooltip API, after initialization:<br>
			*	//Gets the showTooltip  API
			*	$("#bulletGraph1").ejBulletGraph("option", "showTooltip");<br>			
			*	//Sets the showTooltip  API
			*	$("#bulletGraph1").ejBulletGraph("option", "showTooltip", true ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            showTooltip: true,
            /**		
			* Specifies the Template ID for Tooltip 
			* @default null
            * @alias ejBulletGraph#tooltipTemplateId
			* @type {string}
			* @example 
			* //Enable Tooltip on initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  tooltipTemplateId: "Tooltip"});	 
			* @example 
			* //Get or set the tooltipTemplateId API, after initialization:<br>
			*	//Gets the tooltipTemplateId  API
			*	$("#bulletGraph1").ejBulletGraph("option", "tooltipTemplateId");<br>			
			*	//Sets the tooltipTemplateId  API
			*	$("#bulletGraph1").ejBulletGraph("option", "tooltipTemplateId", Tooltip ); 
			 * @memberof ejBulletGraph
			* @instance
			*/
            tooltipTemplateId: null,
            /**		
			* Specifies the scale values for the Graph.
			* @default [Array]
            * @alias ejBulletGraph#quantitativeScale
			* @type {object}
			* @example 
			* //To set quantitativeScale API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{minimum: 2}});	
			* @example 
			* //Get or set the quantitativeScale API, after initialization:<br>
			*	//Gets the quantitativeScale attributes 
			*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale");<br>			
			*	//Sets the quantitativeScale attributes
			*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale, [{minimum: 2}] ); 
			* @memberof ejBulletGraph
			* @instance
			*/
            quantitativeScale: {
                /**		
				* Specifies the Location in the graph
				* @default [Array.number] quantitativeScale.location={x:10,y:10}
                * @alias ejBulletGraph#quantitativeScale->location
				* @type {object}
				* @example 
				* //To set the Location API values during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{location:{x:10}}});	 
				*@example
                * //Get the location of scale
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *
                *var x = scale.location.x;
                *var y = scale.location.y;
                *
				*	//Sets the location  API
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{location:{x:10}}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                location: {
                    /**		
                    * Specifies the Location in the graph
                    * @default [Array.Array.number] quantitativeScale.location.x=10
                    * @alias ejBulletGraph#quantitativeScale->location->x
                    * @type {number}
                    * @example 
                    * //To set the Location API values during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{location:{x:10}}});	
                    *@example
                    * //Get the location of scale
                    *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    *
                    *var x = scale.location.x;
                    *var y = scale.location.y;
                    *
                    *	//Sets the location  API
                    *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{location:{x:10}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    x: 10,
                    /**		
                       * Specifies the Location in the graph
                       * @default [Array.Array.number] quantitativeScale.location.y=10
                       * @alias ejBulletGraph#quantitativeScale->location->y
                       * @type {number}
                       * @example 
                       * //To set the Location API values during initialization  
                       * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{location:{y:10}}});	 
                       *@example
                       * //Get the location of scale
                       *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                       *
                       *var x = scale.location.x;
                       *var y = scale.location.y;
                       *
                       *	//Sets the location  API
                       *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{location:{y:10}}] ); 
                       * @memberof ejBulletGraph
                       * @instance
                       */
                    y: 10
                },
                /**		
				* Specifies the minimum of the Graph.
				* @default [Array.number] quantitativeScale.minimum=0
                * @alias ejBulletGraph#quantitativeScale->minimum
				* @type {number}
				* @example 
				* //To set minimum API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{minimum: 2}});
				*@example
                * //Get the minimum value
                * var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                * var minimumVal = scale.minimum;
                *			
				*	//Sets the minimum 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{minimum: 2}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                minimum: 0,
                /**		
				* Specifies the maximum of the Graph.
				* @default [Array.number] quantitativeScale.maximum=0
                * @alias ejBulletGraph#quantitativeScale->maximum
				* @type {number}
				* @example 
				* //To set maximum API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{maximum: 20}});	
				* @example 
				* //Get or set the maximum API, after initialization:<br>
				* //Get the minimum value
                * var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                * var maximumVal = scale.maximum;
                *					
				*	//Sets the maximum 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{maximum: 20}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                maximum: 10,
                /**		
				* Specifies the interval for the Graph.
				* @default [Array.number] quantitativeScale.interval=1
                * @alias ejBulletGraph#quantitativeScale->interval
				* @type {number}
				* @example 
				* //To set interval API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{interval: 20}});	
				* @example 
				* //Get the interval value of major ticks
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var tickInterval = scale.interval;
                *
				*	//Sets the interval 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{interval: 20}]); 
				* @memberof ejBulletGraph
				* @instance
				*/
                interval: 1,
                /**		
				* Specifies the minorTicksPerInterval for the Graph.
				* @default [Array.number] quantitativeScale.minorTicksPerInterval=4
                * @alias ejBulletGraph#quantitativeScale->minorTicksPerInterval
				* @type {number}
				* @example 
				* //To set minorTicksPerInterval API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{minorTicksPerInterval: 5}});	 
				* @example 
				* //Get the minor interval value
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var minorTickInterval = scale.minorTicksPerInterval;
				*	//Sets the minorTicksPerInterval 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{minorTicksPerInterval: 5}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                minorTicksPerInterval: 4,
                /**		
				* Specifies the majorTickSize of the Graph.
				* @default [Array.number] quantitativeScale.majorTickSize=13
                * @alias ejBulletGraph#quantitativeScale->majorTickSize
				* @type {number}
				* @example 
				* //To set majorTickSize API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{majorTickSize: 15}});
				* @example 
				* //Get or set the majorTickSize API, after initialization:<br>
				*	//Gets the majorTickSize  
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale");<br>			
				*	//Sets the majorTickSize 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{majorTickSize: 15}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                majorTickSize: 13,
                /**		
				* Specifies the minorTickSize of the Graph.
				* @default [Array.number] quantitativeScale.minorTickSize=7
                * @alias ejBulletGraph#quantitativeScale->minorTickSize
				* @type {number}
				* @example 
				* //To set minorTickSize API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{minorTickSize: 5}});	 
				* @example 
				* //Get the major tick size
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var size = scale.majorTickSize;			
				*	//Sets the minorTickSize 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{minorTickSize: 5}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                minorTickSize: 7,
                /**		
				* Specifies the majorTickStroke color of the Graph.
				* @default [Array.string] quantitativeScale.majorTickStroke=null
                * @alias ejBulletGraph#quantitativeScale->majorTickStroke
				* @type {string}
				* @example 
				* //To set majorTickStroke API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{majorTickStroke: "red"}});	 
				* @example 
				* //Get or set the majorTickStroke API, after initialization:<br>
				* //Get the major tick stroke
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var stroke = scale.majorTickStroke;		
				*	//Sets the majorTickStroke 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{majorTickStroke: "red"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                majorTickStroke: null,
                /**		
				* Specifies the minorTickStroke color of the Graph.
				* @default [Array.string] quantitativeScale.minorTickStroke=null
                * @alias ejBulletGraph#quantitativeScale->minorTickStroke
				* @type {string}
				* @example 
				* //To set minorTickStroke API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{minorTickStroke: "green"}});	 
				* @example 
				* //Get or set the minorTickStroke API, after initialization:<br>
				* //Get the minor tick stroke
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var stroke = scale.minorTickStroke;			
				*	//Sets the minorTickStroke 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{minorTickStroke: "green"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                minorTickStroke: null,
                /**		
				* Specifies the majorTickStroke thickness  of the Graph.
				* @default [Array.number] quantitativeScale.majorTickStrokeThickness=2
                * @alias ejBulletGraph#quantitativeScale->majorTickStrokeThickness
				* @type {number}
				* @example 
				* //To set majorTickStrokeThickness API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{majorTickStrokeThickness: 3}});	
				* @example 
				* //Get or set the majorTickStrokeThickness API, after initialization:<br>
				* //Get the major tick stroke thickness
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var thickness = scale.majorTickStrokeThickness;			
				*	//Sets the majorTickStrokeThickness 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{majorTickStrokeThickness: 3}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                majorTickStrokeThickness: 2,
                /**		
				* Specifies the minorTickStroke thickness  of the Graph.
				* @default [Array.number] quantitativeScale.majorTickStrokeThickness=2
                * @alias ejBulletGraph#quantitativeScale->minorTickStrokeThickness
				* @type {number}
				* @example 
				* //To set majorTickStrokeThickness API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{majorTickStrokeThickness: 3}});	 
				* @example 
				* //Get or set the majorTickStrokeThickness API, after initialization:<br>
				* //Get the minor tick stroke thickness
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var thickness = scale.minorTickStrokeThickness;			
				*	//Sets the majorTickStrokeThickness 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{majorTickStrokeThickness: 3}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                minorTickStrokeThickness: 2,
                /**		
				* Specifies the tickPosition in the Graph.See {@link tickPosition}
				* @default [Array.string] quantitativeScale.tickPosition="below"
                * @alias ejBulletGraph#quantitativeScale->tickPosition
				* @type {enum}
				* @example 
				* //To set majorTickStrokeThickness API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{tickPosition: ej.tickPosition.above}});	 
				* @example 
				* //Get or set the majorTickStrokeThickness API, after initialization:<br>
				*	//Gets the majorTickStrokeThickness  
				*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var placement = scale.tickPosition;			
				*	//Sets the majorTickStrokeThickness 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{tickPosition: ej.tickPosition.above}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                tickPosition: "below",
                /**		
                * Specifies the label properties of the Graph.
                * @default [Array.object] quantitativeScale.labels={labelStroke:null}
                * @alias ejBulletGraph#quantitativeScale->labels
                * @type {object}
                * @example 
                * //To set labels API value during initialization  
                * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:{labelStroke:"Red"}}});
                * @example 
                * //Get or set the label API, after initialization:<br>
                *	//Gets the labels  
                *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                * // lindex - label index value
                *var result = scale.labels[lindex].font;		
                *	//Sets the labels
                *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{labels:{labelStroke:"Red"}}] ); 
                * @memberof ejBulletGraph
                * @instance
                */
                labels: {
                    /**		
					* Specifies the labelStroke color of the Graph.
					* @default [Array.Array.number] quantitativeScale.labels.labelStroke=null
                    * @alias ejBulletGraph#quantitativeScale->labels->labelStroke
					* @type {string}
					* @example 
					* //To set labelStroke API value during initialization  
					* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:{labelStroke:"Red"}}});
					* @example 
					* //Get or set the labelStroke API, after initialization:<br>
					*	//Gets the labelStroke  
					*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    * // lindex - label index value
                    *var result = scale.labels[lindex].labelStroke;
					*	//Sets the labelStroke 
					*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[ {labels:{labelStroke:"Red"}}] ); 
					* @memberof ejBulletGraph
					* @instance
					*/
                    labelStroke: null,
                    /**		
					* Specifies the labelSize of the Graph.
					* @default [Array.Array.number] quantitativeScale.labels.labelSize=12
                    * @alias ejBulletGraph#quantitativeScale->labels->labelSize
					* @type {number}
					* @example 
					* //To set labelSize API value during initialization  
					* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:{labelSize:12}}}); 
					* @example 
					* //Get or set the labelSize API, after initialization:<br>
					*	//Gets the labelSize  
					*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    * // lindex - label index value
                    *var result = scale.labels[lindex].labelSize;
					*	//Sets the labelSize 
					*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{labels:{labelSize:12}}] ); 
					* @memberof ejBulletGraph
					* @instance
					*/
                    labelSize: 12,
                    /**		
					* Specifies the labelOffset in the Graph.
					* @default [Array.Array.number] quantitativeScale.labels.labelOffset=20
                    * @alias ejBulletGraph#quantitativeScale->labels->labelOffset
					* @type {number}
					* @example 
					* //To set labelOffset API value during initialization  
					* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:{labelOffset:25}}});	 
					* @example 
					* //Get or set the labelOffset API, after initialization:<br>
					*	//Gets the labelOffset  
					*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    * // lindex - label index value
                    *var result = scale.labels[lindex].labelOffset;
					*	//Sets the labelOffset 
					*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{labels:{labelOffset:25}} ]); 
					* @memberof ejBulletGraph
					* @instance
					*/
                    labelOffset: 20,
                    /**		
					* Specifies the fonts in the Graph.
					* @default [Array.Array.object] quantitativeScale.labels.font={fontFamily: "Segoe UI"}
                    * @alias ejBulletGraph#quantitativeScale->labels->font
					* @type {object}
					* @example 
					* //To set font API value during initialization  
					* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:font{fontFamily:"Segoe UI"}}});	
					* @example 
					* //Get or set the font API, after initialization:<br>
					*	//Gets the font  
					*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    * // lindex - label index value
                    *var result = scale.labels[lindex].font;		
					*	//Sets the font
					*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{labels:font{fontFamily:"Segoe UI"}}] ); 
					* @memberof ejBulletGraph
					* @instance
					*/
                    font: {
                        /**		
                        * Specifies the fontFamily in the Graph.
                        * @default [Array.Array.Array.string] quantitativeScale.labels.font.fontFamily= "Segoe UI"
                        * @alias ejBulletGraph#quantitativeScale->labels->font->fontFamily
                        * @type {string}
                        * @example 
                        * //To set fontFamily API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:font{fontFamily:"Segoe UI"}}});	
                        * @example 
                        * //Get or set the fontFamily API, after initialization:<br>
                        *	//Gets the fontFamily  
                        *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                        * // lindex - label index value
                        *var result = scale.labels[lindex].font.fontFamily;			
                        *	//Sets the fontFamily 
                        *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{labels:font{fontFamily:"Segoe UI"}}] ); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        fontFamily: 'Segoe UI',
                        /**		
                        * Specifies the fontStyle in the Graph.See {@link fontStyle}
                        * @default [Array.Array.Array.string] quantitativeScale.labels.font.fontStyle= "Normal"
                        * @alias ejBulletGraph#quantitativeScale->labels->font->fontStyle
                        * @type {enum}
                        * @example 
                        * //To set fontStyle API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:{font:{fontStyle:ej.BulletGraph.fontStyle.Italic}}}}); 
                        * @example 
                        * //Get or set the fontStyle API, after initialization:<br>
                        *	//Gets the fontStyle  
                        *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                        * // lindex - label index value
                        *var result = scale.labels[lindex].font.fontStyle;				
                        *	//Sets the fontStyle 
                        *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{labels:{font:{fontStyle:ej.BulletGraph.fontStyle.Italic}}}] ); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        fontStyle: 'Normal ',
                        /**		
                        * Specifies the fontWeight in the Graph.See {@link fontWeight}
                        * @default [Array.Array.Array.string] quantitativeScale.labels.font.fontWeight= "regular"
                        * @alias ejBulletGraph#quantitativeScale->labels->font->fontWeight
                        * @type {enum}
                        * @example 
                        * //To set fontWeight API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:{font:{fontWeight:ej.BulletGraph.fontWeight.Bold}}}}); 
                        * @example 
                        * //Get or set the fontWeight API, after initialization:<br>
                        *	//Gets the fontWeight  
                        *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                        * // lindex - label index value
                        *var result = scale.labels[lindex].font.fontWeight;				
                        *	//Sets the fontWeight 
                        *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{labels:{font:{fontWeight:ej.BulletGraph.fontWeight.Bold}}}] ); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        fontWeight: 'regular',
                        /**		
                        * Specifies the opacity of font in the Graph.
                        * @default [Array.Array.Array.number] quantitativeScale.labels.font.opacity= 1
                        * @alias ejBulletGraph#quantitativeScale->labels->font->opacity
                        * @type (number)
                        * @example 
                        * //To set opacity API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:{font:{opacity:0.5}}}});	
                        * @example 
                        * //Get or set the opacity API, after initialization:<br>
                        *	//Gets the opacity  
                        *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                        * // lindex - label index value
                        *var result = scale.labels[lindex].font.opacity;			
                        *	//Sets the opacity 
                        *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{labels:{font:{opacity:0.5}}}]); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        opacity: 1
                    },
                    /**		
					* Specifies the labelPosition of font in the Graph.See {@link labelPosition}
					* @default [Array.Array.number] quantitativeScale.labels.labelPosition= "below"
                    * @alias ejBulletGraph#quantitativeScale->labels->labelPosition
					* @type {enum}
					* @example 
					* //To set labelPosition API value during initialization  
					* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{labels:{labelPosition:ej.BulletGraph.labelPosition.above}}});
					* @example 
					* //Get or set the labelPosition API, after initialization:<br>
					*	//Gets the labelPosition  
					*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    * // lindex - label index value
                    *var result = scale.labels[lindex].labelPosition;			
					*	//Sets the labelPosition 
					*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{labels:{labelPosition:ej.BulletGraph.labelPosition.above}}] ); 
					* @memberof ejBulletGraph
					* @instance
					*/
                    labelPosition: "below"
                },
                /**		
				* Specifies the featuredMeasureBarStrokeThickness of the Graph.
				* @default [Array.number] quantitativeScale.featuredMeasureBarStrokeThickness= 6
                * @alias ejBulletGraph#quantitativeScale->featuredMeasureBarStrokeThickness
				* @type {number}
				* @example 
				* //To set featuredMeasureBarStrokeThickness API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{featuredMeasureBarStrokeThickness:7}});	
				* @example 
				* //Get or set the featuredMeasureBarStrokeThickness API, after initialization:<br>
				*	//Gets the featuredMeasureBarStrokeThickness  
				*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var thickness = scale.featuredMeasureBarStrokeThickness;			
				*	//Sets the featuredMeasureBarStrokeThickness 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{featuredMeasureBarStrokeThickness:7}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                featuredMeasureBarStrokeThickness: 6,
                /**		
				* Specifies the featuredMeasureBarStroke of the Graph.
				* @default [Array.string] quantitativeScale.featuredMeasureBarStroke= null
                * @alias ejBulletGraph#quantitativeScale->featuredMeasureBarStroke
				* @type {string}
				* @example 
				* //To set featuredMeasureBarStroke API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{featuredMeasureBarStroke:"green"}});	 
				* @example 
				* //Get or set the featuredMeasureBarStroke API, after initialization:<br>
				*	//Gets the featuredMeasureBarStroke  
				*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var stroke = scale.featuredMeasureBarStroke;		
				*	//Sets the featuredMeasureBarStroke 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{featuredMeasureBarStroke:"green"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                featuredMeasureBarStroke: null,
                /**		
				* Specifies the comparativeMeasureSymbolStrokeThickness of the Graph.
				* @default * @default [Array.number] quantitativeScale.comparativeMeasureSymbolStrokeThickness= 5
                * @alias ejBulletGraph#quantitativeScale->comparativeMeasureSymbolStrokeThickness
				* @type {number}
				* @example 
				* //To set comparativeMeasureSymbolStrokeThickness API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{comparativeMeasureSymbolStrokeThickness:6}}); 
				* @example 
				* //Get or set the comparativeMeasureSymbolStrokeThickness API, after initialization:<br>
				*	//Gets the comparativeMeasureSymbolStrokeThickness  
				*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var thickness = scale.comparativeMeasureSymbolStrokeThickness;			
				*	//Sets the comparativeMeasureSymbolStrokeThickness 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale", [{comparativeMeasureSymbolStrokeThickness:6}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                comparativeMeasureSymbolStrokeThickness: 5,
                /**		
				* Specifies the comparativeMeasureSymbolStroke of the Graph.
				* @default [Array.number] quantitativeScale.comparativeMeasureSymbolStroke= null
                * @alias ejBulletGraph#quantitativeScale->comparativeMeasureSymbolStroke
				* @type {string}
				* @example 
				* //To set comparativeMeasureSymbolStroke API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{comparativeMeasureSymbolStroke:"yellow"}});	 
				* @example 
				* //Get or set the comparativeMeasureSymbolStroke API, after initialization:<br>
				*	//Gets the comparativeMeasureSymbolStroke  
				*var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                *var thickness = scale.comparativeMeasureSymbolStroke;			
				*	//Sets the comparativeMeasureSymbolStroke 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{comparativeMeasureSymbolStroke:"yellow"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                comparativeMeasureSymbolStroke: null,
                /**		
				* Specifies the Feature Measure value of the Graph.
				* @default [Array.object] quantitativeScale.featureMeasure= {value:10}
                * @alias ejBulletGraph#quantitativeScale->featureMeasure
				* @type {object}
				* @example 
				* //To set featureMeasure API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{featureMeasure:{value:8}}});	 
				* @example 
				* //Get or set the featureMeasure API, after initialization:<br>
				*	//Gets the featureMeasure  
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale");<br>			
				*	//Sets the featureMeasure 
				*	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{featureMeasure:{value:8}}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                featureMeasure: [{
                    /**		
                    * Specifies the Feature Measure value of the Graph.
                    * @default [Array.Array.number] quantitativeScale.featureMeasure.value=null
                    * @alias ejBulletGraph#quantitativeScale->featureMeasure->value
                    * @type {number}
                    * @example 
                    * //To set value API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{featureMeasure:{value:8}}});	 
                    * @example 
                    * //Get or set the value API, after initialization:<br>
                    *	//Gets the value  
                    *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    * // findex - feature measure bar index value
                    *var featureValue = scale.featureMeasure[findex].value;			
                    *	//Sets the value 
                    *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{featureMeasure:{value:8}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    value: null,
                    /**		
                    * Specifies the Comparative Measure Value  of the Graph.
                    * @default [Array.Array.number] quantitativeScale.featureMeasure.comparativeMeasureValue=null
                     * @alias ejBulletGraph#quantitativeScale->featureMeasure->comparativeMeasureValue
                    * @type {number}
                    * @example 
                    * //To set comparativeMeasureValue API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{featureMeasure:{comparativeMeasureValue:8}}});	 
                    * @example 
                    * //Get or set the comparativeMeasureValue API, after initialization:<br>
                    *	//Gets the comparativeMeasureValue  
                    *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    * // findex - feature measure bar index value
                    *var compartiveValue = scale.featureMeasure[findex].comparativeMeasureValue;		
                    *	//Sets the comparativeMeasureValue 
                    *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{featureMeasure:{comparativeMeasureValue:8}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    comparativeMeasureValue: null,
                    /**		
                    * Specifies the feature measure catagory  of the Graph.
                    * @default [Array.Array.string] quantitativeScale.featureMeasure.category=null
                    * @alias ejBulletGraph#quantitativeScale->featureMeasure->category
                    * @type {string}
                    * @example 
                    * //To set category API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  quantitativeScale:{featureMeasure:{category:"category"}}});	 
                    * @example 
                    * //Get or set the category API, after initialization:<br>
                    *	//Gets the category  
                    *var scale = $("#SampleBullet").ejBulletGraph("option", "quantitativeScale");
                    * // findex - feature measure bar index value
                    *var categ = scale.featureMeasure[findex].category;			
                    *	//Sets the category 
                    *	$("#bulletGraph1").ejBulletGraph("option", "quantitativeScale",[{featureMeasure:{category:"category"}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    category: null
                }]
            },
            /**		
            * Specifies the fields for the Graph.
            * @default [Array.Array] quantitativeScale.fields={datasource=null}
            * @alias ejBulletGraph#quantitativeScale->fields
            * @type {object}
            * @example 
            *	//To set fields API value during initialization  
            * 	$("#bulletGraph1").ejBulletGraph({ fields:{dataSource:ej.DataManager({url: "http://mvc.syncfusion.com/Services/Northwnd.svc/" }}); 
            * @example 
            * //Get or set the fields API, after initialization:<br>
            *	//Gets the fields
            *	$("#bulletGraph1").ejBulletGraph("option", "fields");<br>			
            *	//Sets the fields 
            *	$("#bulletGraph1").ejBulletGraph("option", "fields",[{dataSource{ej.DataManager({url: "http://mvc.syncfusion.com/Services/Northwnd.svc/" }}}]);	 
            * @memberof ejBulletGraph
            * @instance
            */
            fields: {
                /**		
				* Specifies the dataSource for the Graph.
				* @default [Array.Array.Object] quantitativeScale.fields.dataSource=null
                * @alias ejBulletGraph#quantitativeScale->fields->dataSource
				* @type {object}
				* @example 
				* // To initialize with local Json data to the dataSource
                *$("#SampleBullet").ejBulletGraph({
                *fields: {
                *dataSource: localData,
                *category: “Category”,
                *featureMeasure: “FeatureValue”,
                *comparativeMeasure: “ComparativeValue”
                *}
                *});
                * // where local Json data assigned to the dataSource is as the below one
                *var localData = [{
                *                    FeatureValue: 8, ComparativeValue: 7.5,
                *                    Category: 1
                *                },
                *                {
                *                    FeatureValue: 9, ComparativeValue: 5,
                *                    Category: 2
                *                }];
                * // To initialize with remote data to the dataSource using DataManager
                *$("#SampleBullet").ejBulletGraph({
                *      fields: {
                *           dataSource: dataManager,
                *           query: queryString,
                *           category: "ProductID",
                *           featureMeasure: "UnitPrice",
                *           comparativeMeasure: "Quantity"
                *      }
                *});
                * // Data Manager creation
                *            var dataManager = ej.DataManager({
                *                url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"
                *            });
                *
                * 
				* @example 
				* //Get or set the dataSource API, after initialization:<br>
				*	//Gets the dataSource
				*	$("#bulletGraph1").ejBulletGraph("option", "fields");<br>			
				*	//Sets the dataSource 
				*	$("#bulletGraph1").ejBulletGraph("option", "fields",[{datasource{ej.DataManager({url: "http://mvc.syncfusion.com/Services/Northwnd.svc/" }}}]); 
				* @memberof ejBulletGraph
				* @instance
				*/
                dataSource: null,
                /**		
				* Specifies the query for getting value to the Graph.
				* @default [Array.Array.string] quantitativeScale.fields.query=null
                * @alias ejBulletGraph#quantitativeScale->fields->query
				* @type {string}
				* @example 
				*	//To set query API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ fields:{query: "ej.Query().from('Categories').select('CategoryID,CategoryName').take(3)"}});	
				* @example 
				* //Get or set the query API, after initialization:<br>
				*	//Gets the query
				*	$("#bulletGraph1").ejBulletGraph("option", "fields");<br>			
				*	//Sets the query 
				*	$("#bulletGraph1").ejBulletGraph("option", "fields",[{"query{ej.Query().from('Categories').select('CategoryID,CategoryName').take(3)"}}]); 
				* @memberof ejBulletGraph
				* @instance
				*/
                query: null,
                /**		
				* Specifies the tableName for the Graph.
				* @default [Array.Array.string] quantitativeScale.fields.tableName=null
                * @alias ejBulletGraph#quantitativeScale->fields->tableName
				* @type {string}
				* @example 
				* // Query creation
                * var queryString = ej.Query().from("Order_Details").select("ProductID,UnitPrice,Quantity").take(7);
				* @example 
				* //Get or set the tableName API, after initialization:<br>
				*	//Gets the tableName
				*	var field = $("#SampleBullet").ejBulletGraph("option", "fields");
	            *var tableName = field.tableName;	
				*	//Sets the tableName 
				*	$("#bulletGraph1").ejBulletGraph("option", "fields",[{tableName{"Product"}}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                tableName: null,
                /**		
				* Specifies the category for the Graph.
				* @default [Array.Array.string] quantitativeScale.fields.category=null
                * @alias ejBulletGraph#quantitativeScale->fields->category
				* @type {string}
				* @example 
				*	//To set category API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ fields:{category: "ProductID"}});	
				* @example 
				* //Get or set the category API, after initialization:<br>
				*	//Gets the category
				*	var field = $("#SampleBullet").ejBulletGraph("option", "fields");
	            *var catagory = field.catagory;		
				*	//Sets the category 
				*	$("#bulletGraph1").ejBulletGraph("option", "fields",[{category: "ProductID"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                category: null,
                /**		
				* Specifies the featureMeasure for the Graph.
				* @default [Array.Array.number] quantitativeScale.fields.featureMeasure=null
                * @alias ejBulletGraph#quantitativeScale->fields->featureMeasure
				* @type {number}
				* @example 
				*	//To set featureMeasure API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ fields:{featureMeasure:"UnitPrice" }});	 
				* @example 
				* //Get or set the featureMeasure API, after initialization:<br>
				*	//Gets the featureMeasure
				*	var field = $("#SampleBullet").ejBulletGraph("option", "fields");
	            * var featureMeasure = field.featureMeasure;					
				*	//Sets the featureMeasure 
				*	$("#bulletGraph1").ejBulletGraph("option", "fields",[{featureMeasure:"UnitPrice" }] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                featureMeasure: null,
                /**		
				* Specifies the comparativeMeasure for the Graph.
				* @default [Array.Array.number] quantitativeScale.fields.comparativeMeasure=null
                * @alias ejBulletGraph#quantitativeScale->fields->comparativeMeasure
				* @type {number}
				* @example 
				*	//To set comparativeMeasure API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ fields:{comparativeMeasure:"Quantity" }});	
				* @example 
				* //Get or set the comparativeMeasure API, after initialization:<br>
				*	//Gets the comparativeMeasure
				*	var field = $("#SampleBullet").ejBulletGraph("option", "fields");
	            *  var comparativeMeasure = field.comparativeMeasure;				
				*	//Sets the comparativeMeasure 
				*	$("#bulletGraph1").ejBulletGraph("option", "fields",[{comparativeMeasure:"Quantity" }] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                comparativeMeasure: null
            },
            /**		
			* Specifies the Animation property for the Graph.
			* @default [boolean] enableAnimation=true
            * @alias ejBulletGraph#enableAnimation
			* @type {boolean}
			* @example 
			*	//To set enableAnimation API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({ enableAnimation:false });
			* @example 
			* //Get or set the enableAnimation API, after initialization:<br>
			*	//Gets the enableAnimation
			*	$("#bulletGraph1").ejBulletGraph("option", "enableAnimation");<br>			
			*	//Sets the enableAnimation 
			*	$("#bulletGraph1").ejBulletGraph("option", "enableAnimation",false ); 
			* @memberof ejBulletGraph
			* @instance
			*/
            enableAnimation: true,
            /**		
			* Specifies the canResize property for the Graph.
			* @default [boolean] canResize=true
            * @alias ejBulletGraph#canResize
			* @type {boolean}
			* @example 
			*	//To set canResize API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({ canResize:false });	
			* @example 
			* //Get or set the canResize API, after initialization:<br>
			*	//Gets the canResize
			*	$("#bulletGraph1").ejBulletGraph("option", "canResize");<br>			
			*	//Sets the canResize 
			*	$("#bulletGraph1").ejBulletGraph("option", "canResize",false ); 
			* @memberof ejBulletGraph
			* @instance
			*/
            canResize: true,
            /**		
			* Specifies the bindRangeStrokeToTicks property for the Graph.
			* @default [boolean] bindRangeStrokeToTicks=false
            * @alias ejBulletGraph#bindRangeStrokeToTicks
			* @type {boolean}
			* @example 
			*	//To set bindRangeStrokeToTicks API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({ bindRangeStrokeToTicks:false });	 
			* @example 
			* //Get or set the bindRangeStrokeToTicks API, after initialization:<br>
			*	//Gets the bindRangeStrokeToTicks
			*	$("#bulletGraph1").ejBulletGraph("option", "bindRangeStrokeToTicks");<br>			
			*	//Sets the bindRangeStrokeToTicks 
			*	$("#bulletGraph1").ejBulletGraph("option", "bindRangeStrokeToTicks",false ); 
			* @memberof ejBulletGraph
			* @instance
			*/
            bindRangeStrokeToTicks: false,
            /**		
			* Specifies the bindRangeStrokeToLabels property for the Graph.
			* @default [Array.boolean] bindRangeStrokeToLabels=false
            * @alias ejBulletGraph#bindRangeStrokeToLabels
			* @type {boolean}
			* @example 
			*	//To set bindRangeStrokeToLabels API value during initialization  
			* 	$("#bulletGraph1").ejBulletGraph({ bindRangeStrokeToLabels:false });	
			* @example 
			* //Get or set the bindRangeStrokeToLabels API, after initialization:<br>
			*	//Gets the bindRangeStrokeToLabels
			*	$("#bulletGraph1").ejBulletGraph("option", "bindRangeStrokeToLabels");<br>			
			*	//Sets the bindRangeStrokeToLabels 
			*	$("#bulletGraph1").ejBulletGraph("option", "bindRangeStrokeToLabels",false ); 
			* @memberof ejBulletGraph
			* @instance
			*/
            bindRangeStrokeToLabels: false,
            /**		
            * Specifies the qualitativeRanges property for the Graph.
            * @default [Array.object] qualitativeRanges={rangeEnd: 4.3}
            * @alias ejBulletGraph#qualitativeRanges
            * @type {object}
            * @example 
            *	//To set qualitativeRanges API value during initialization  
            * 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeEnd:4.3} });		 
            * @example 
            * //Get or set the qualitativeRanges API, after initialization:<br>
            *	//Gets the qualitativeRanges
            *	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges");<br>			
            *	//Sets the qualitativeRanges 
            *	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeEnd:4.3}]); 
            * @memberof ejBulletGraph
            * @instance
            */
            qualitativeRanges: [{
                /**		
				* Specifies the rangeEnd property for the Graph.
				* @default [Array.Array.number] qualitativeRanges.rangeEnd=3
                * @alias ejBulletGraph#qualitativeRanges->rangeEnd
				* @type {number}
				* @example 
				*	//To set rangeEnd API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeEnd:4.3} });	
				* @example 
				* //Get or set the rangeEnd API, after initialization:<br>
				*	//Gets the rangeEnd
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var result = range[rindex].rangeEnd;		
				*	//Sets the rangeEnd 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges.rangeEnd",[{rangeEnd:4.3}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeEnd: 4.3,
                /**		
				* Specifies the rangeStroke property for the Graph.
				* @default [Array.Array.number] qualitativeRanges.rangeStroke=null
                * @alias ejBulletGraph#qualitativeRanges->rangeStroke
				* @type {string}
				* @example 
				*	//To set rangeStroke API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeStroke:"yellow"} });	 
				* @example 
				* //Get or set the rangeStroke API, after initialization:<br>
				*	//Gets the rangeStroke
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var rangeColor = range[rindex].rangeStroke;	
				*	//Sets the rangeStroke 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeStroke:"yellow"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeStroke: null,
                /**		
				* Specifies the rangeOpacity property for the Graph.
				* @default [Array.Array.number] qualitativeRanges.rangeOpacity=1
                * @alias ejBulletGraph#qualitativeRanges->rangeOpacity
				* @type {number}
				* @example 
				*	//To set rangeOpacity API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeOpacity:0.5} });	 
				* @example 
				* //Get or set the rangeOpacity API, after initialization:<br>
				*	//Gets the rangeOpacity
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var RangeOpac = range[rindex].rangeOpacity;			
				*	//Sets the rangeOpacity 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeOpacity:0.5}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeOpacity: 1
            }, {
                /**		
				* Specifies the rangeEnd property for the Graph.
				* @default [Array.Array.number] qualitativeRanges.rangeEnd=7.3
                * @alias ejBulletGraph#qualitativeRanges->rangeEnd
				* @type {number}
				* @example 
				*	//To set rangeEnd API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeEnd:4.3} });	 
				* @example 
				* //Get or set the rangeEnd API, after initialization:<br>
				*	//Gets the rangeEnd
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var result = range[rindex].rangeEnd;				
				*	//Sets the rangeEnd 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeEnd:4.3}]); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeEnd: 7.3,
                /**		
				* Specifies the rangeStroke property for the Graph.
				* @default [Array.number] qualitativeRanges.rangeStroke=null
                * @alias ejBulletGraph#qualitativeRanges->rangeStroke
				* @type {string}
				* @example 
				*	//To set rangeStroke API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeStroke:"yellow"} });	 
				* @example 
				* //Get or set the rangeStroke API, after initialization:<br>
				*	//Gets the rangeStroke
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var rangeColor = range[rindex].rangeStroke;			
				*	//Sets the rangeStroke 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeStroke:"yellow"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeStroke: null,
                /**		
				* Specifies the rangeOpacity property for the Graph.
				* @default [Array.number] qualitativeRanges.rangeOpacity=1
                * @alias ejBulletGraph#qualitativeRanges->rangeOpacity
				* @type {number}
				* @example 
				*	//To set rangeOpacity API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeOpacity:0.5} });	 
				* @example 
				* //Get or set the rangeOpacity API, after initialization:<br>
				*	//Gets the rangeOpacity
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var RangeOpac = range[rindex].rangeOpacity;				
				*	//Sets the rangeOpacity 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeOpacity:0.5}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeOpacity: 1
            }, {
                /**		
				* Specifies the rangeEnd property for the Graph.
				* @default [Array.number] qualitativeRanges.rangeEnd=10
                * @alias ejBulletGraph#qualitativeRanges->rangeEnd
				* @type {number}
				* @example 
				*	//To set rangeEnd API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeEnd:4.3} });	 
				* @example 
				* //Get or set the rangeEnd API, after initialization:<br>
				*	//Gets the rangeEnd
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var result = range[rindex].rangeEnd;		
				*	//Sets the rangeEnd 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeEnd:4.3}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeEnd: 10,
                /**		
				* Specifies the rangeStroke property for the Graph.
				* @default [Array.number] qualitativeRanges.rangeStroke=null
                * @alias ejBulletGraph#qualitativeRanges->rangeStroke
				* @type {string}
				* @example 
				*	//To set rangeStroke API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeStroke:"yellow"} });	 
				* @example 
				* //Get or set the rangeStroke API, after initialization:<br>
				*	//Gets the rangeStroke
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var rangeColor = range[rindex].rangeStroke;			
				*	//Sets the rangeStroke 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeStroke:"yellow"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeStroke: null,
                /**		
				* Specifies the rangeOpacity property for the Graph.
				* @default [Array.number] qualitativeRanges.rangeOpacity=1
                * @alias ejBulletGraph#qualitativeRanges->rangeOpacity
				* @type {number}
				* @example 
				*	//To set rangeOpacity API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ qualitativeRanges:{rangeOpacity:0.5} });	 
				* @example 
				* //Get or set the rangeOpacity API, after initialization:<br>
				*	//Gets the rangeOpacity
				*var range = $("#SampleBullet").ejBulletGraph("option", "qualitativeRanges");
                * // rindex - range index value
                *var RangeOpac = range[rindex].rangeOpacity;			
				*	//Sets the rangeOpacity 
				*	$("#bulletGraph1").ejBulletGraph("option", "qualitativeRanges",[{rangeOpacity:0.5}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                rangeOpacity: 1
            }],
            /**		
            * Specifies the caption property for the Graph.
            * @default [Array.object] quantitativeScale.caption={textangle:0}
            * @alias ejBulletGraph#caption
            * @type {number}
            * @example 
            *	//To set caption API value during initialization  
            * 	$("#bulletGraph1").ejBulletGraph({ caption:{textAngle:0}} ); 
            * @example 
            * //Get or set the caption API, after initialization:<br>
            *	//Gets the caption
            *var title = $("#SampleBullet").ejBulletGraph("option", "caption");
            *var result = title.font;	
            *	//Sets the caption 
            *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{textAngle:0}] ); 
            * @memberof ejBulletGraph
            * @instance
            */
            caption: {
                /**		
				* Specifies the textAngle property for the Graph.
				* @default [Array.number] caption.textangle=0
                * @alias ejBulletGraph#caption->textAngle
				* @type {number}
				* @example 
				*	//To set textAngle API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ caption:{textAngle:0}} );
				* @example 
				* //Get or set the textAngle API, after initialization:<br>
				*	//Gets the textAngle
				*var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                *var result = title.textAngle;		
				*	//Sets the textAngle 
				*	$("#bulletGraph1").ejBulletGraph("option", "caption",[caption:{textAngle:0}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                textAngle: 0,
                /**		
				* Specifies the Location in graph
				* @default [Array.object] caption.location={x:10;y:10}
                * @alias ejBulletGraph#caption->location
				* @type {object}
				* @example 
				* //To set the Location before initialization
				* 	$("#bulletGraph1").ejBulletGraph({  caption:{location:{x:10}}}); 
				* @example 
				* //Get or set the Location API, after initialization:<br>
				*	//Gets the Location  API
				*var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                *var x = title.location.x;
                *var y = title.location.y;		
				*	//Sets the Location  API
				*	$("#bulletGraph1").ejBulletGraph("option", "caption", [{location:{x:10}}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                location: {
                    /**		
                    * Specifies the Location in horizontal direction
                    * @default [Array.Array.number] caption.location.x=17
                    * @alias ejBulletGraph#caption->location->x
                    * @type {number}
                    * @example 
                    * //To set the Location before initialization
                    * 	$("#bulletGraph1").ejBulletGraph({  caption:{location:{x:10}}}); 
                    * @example 
                    * //Get or set the x API, after initialization:<br>
                    *	//Gets the x  API
                    *var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var x = title.location.x;	
                    *	//Sets the x  API
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption", [{location:{x:10}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    x: 17,
                    /**		
                    * Specifies the Location in horizontal direction
                    * @default [Array.Array.number] caption.location.y=30
                    * @alias ejBulletGraph#caption->location->y
                    * @type {number}
                    * @example 
                    * //Enable Tooltip on initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  caption:{location:{y:10}}}); 
                    * @example 
                    * //Get or set the y API, after initialization:<br>
                    *	//Gets the y  API
                    *var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var y = title.location.y;		
                    *	//Sets the y  API
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption", [{location:{y:10}}]); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    y: 30
                },
                /**		
				* Specifies the text property for the Graph.
				* @default [Array.string] caption.text=""
                * @alias ejBulletGraph#caption->text
				* @type {string}
				* @example 
				*	//To set text API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ caption:{text:"Hello"}} ); 
				* @example 
				* //Get or set the text API, after initialization:<br>
				*	//Gets the text
				*var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                *var text = title.subtitle.text;	
				*	//Sets the text 
				*	$("#bulletGraph1").ejBulletGraph("option", "caption",[{text:"Hello"}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                text: "",
                /**		
				* Specifies the font
				* @default [Array.object] caption.Font={color=null}
                * @alias ejBulletGraph#caption->font
				* @type {object}
				* @example 
				* //To set the font before initialization  
				* 	$("#bulletGraph1").ejBulletGraph({  caption:{font:{color:"yellow"}}});	 
				* @example 
				* //Get or set the font API, after initialization:<br>
				*	//Gets the font  API
				*	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
	            *var result = title.font;		
				*	//Sets the font  API
				*	$("#bulletGraph1").ejBulletGraph("option", "caption", [{font:{color:"yellow"}}] ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                font: {
                    /**		
                    * Specifies the color of the font
                    * @default [Array.Array.string] caption.Font.color=null
                    * @alias ejBulletGraph#caption->font->color
                    * @type {string}
                    * @example 
                    * //To set the font befor initialization 
                    * 	$("#bulletGraph1").ejBulletGraph({  caption:{font:{color:"yellow"}}});	 
                    * @example 
                    * //Get or set the color API, after initialization:<br>
                    *	//Gets the color  API
                    *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var result = title.font.color;		
                    *	//Sets the color  API
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption", [{font:{color:"yellow"}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    color: null,
                    /**		
                    * Specifies the fontFamily of the font
                    * @default [Array.Array.string] caption.Font.fontFamily="Segoe UI"
                    * @alias ejBulletGraph#caption->font->fontFamily
                    * @type {string}
                    * @example 
                    * //Specify the font family 
                    * 	$("#bulletGraph1").ejBulletGraph({  caption:{font:{fontFamily:"Segoe UI"}}});	 
                    * @example 
                    * //Get or set the fontFamily API, after initialization:<br>
                    *	//Gets the fontFamily  API
                    *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var result = title.font.fontFamily;		
                    *	//Sets the fontFamily  API
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption", [{font:{fontFamily:"Segoe UI"}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    fontFamily: 'Segoe UI',
                    /**		
                    * Specifies the fontStyle in the Graph.See {@link fontStyle}
                    * @default [Array.Array.enum] caption.Font.fontStyle="Normal"
                    * @alias ejBulletGraph#caption->font->fontStyle
                    * @type {enum}
                    * @example 
                    * //To set fontStyle API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  caption:{font:{fontStyle:ej.BulletGraph.fontStyle.Italic}}}); 
                    * @example 
                    * //Get or set the fontStyle API, after initialization:<br>
                    *	//Gets the fontStyle  
                    *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var result = title.font.fonStyle;			
                    *	//Sets the fontStyle 
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption", [{font:{fontStyle:ej.BulletGraph.fontStyle.Italic}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    fontStyle: 'Normal',
                    /**		
                    * Specifies the size in the Graph.
                    * @default [Array.Array.string] caption.Font.size="12px"
                    * @alias ejBulletGraph#caption->font->size
                    * @type {string}
                    * @example 
                    * //To set size API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  caption:{font:{size:"10px"}}});	 
                    * @example 
                    * //Get or set the size API, after initialization:<br>
                    *	//Gets the size  
                    *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var result = title.font.size;		
                    *	//Sets the size 
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption", [{font:{size:"10px"}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    size: '12px',
                    /**		
                    * Specifies the fontWeight in the Graph.See {@link fontWeight}
                    * @default [Array.Array.enum] caption.Font.fontWeight="regular"
                    * @alias ejBulletGraph#caption->font->fontWeight
                    * @type {enum}
                    * @example 
                    * //To set fontWeight API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  caption:{font:{fontWeight:ej.BulletGraph.fontWeight.Normal}}});	 
                    * @example 
                    * //Get or set the fontWeight API, after initialization:<br>
                    *	//Gets the fontWeight  
                    *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var result = title.font.fontWeight;			
                    *	//Sets the fontWeight 
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption", [{font:{fontWeight:ej.BulletGraph.fontWeight.Normal}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    fontWeight: 'regular',
                    /**		
                    * Specifies the opacity in the Graph.
                    * @default [Array.Array.number] caption.Font.opacity=1
                    * @alias ejBulletGraph#caption->font->opacity
                    * @type {number}
                    * @example 
                    * //To set opacity API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({  caption:{font:{opacity:1}}});
                    * @example 
                    * //Get or set the opacity API, after initialization:<br>
                    *	//Gets the opacity  
                    *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var result = title.font.opacity;		
                    *	//Sets the opacity 
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption", [{font:{opacity:1}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    opacity: 1
                },
                /**		
				* Specifies the subtitle property for the Graph.
				* @default [Array.number] caption.subtitle=null
                * @alias ejBulletGraph#caption->subtitle
				* @type {number}
				* @example 
				*	//To set subtitle API value during initialization  
				* 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:{textAngle:0}}} );	 
				* @example 
				* //Get or set the subtitle API, after initialization:<br>
				*	//Gets the subtitle
				*	$("#bulletGraph1").ejBulletGraph("option", "caption");<br>			
				*	//Sets the subtitle 
				*	$("#bulletGraph1").ejBulletGraph("option", "caption",{subtitle:{textAngle:0}} ); 
				* @memberof ejBulletGraph
				* @instance
				*/
                subtitle: {
                    /**		
                    * Specifies the textAngle property for the Graph.
                    * @default  [Array.Array.number] caption.subtitle.textAngle=0
                    * @alias ejBulletGraph#caption->subtitle.textAngle
                    * @type {number}
                    * @example 
                    *	//To set textAngle API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:{textAngle:0}}} );	 
                    * @example 
                    * //Get or set the textAngle API, after initialization:<br>
                    *	//Gets the textAngle
                    *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var angle = title.subtitle
                    *	//Sets the textAngle 
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:{textAngle:0}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    textAngle: 0,
                    /**		
                    * Specifies the text property for the Graph.
                    * @default [Array.Array.string] caption.subtitle.text=""
                    * @alias ejBulletGraph#caption->subtitle.text
                    * @type {string}
                    * @example 
                    *	//To set text API value during initialization  
                    * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:{text:"Product"}}} );	 
                    * @example 
                    * //Get or set the text API, after initialization:<br>
                    *	//Gets the text
                    *var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                    *var text = title.subtitle.text;	
                    *	//Sets the text 
                    *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:{text:"Product"}}] ); 
                    * @memberof ejBulletGraph
                    * @instance
                    */
                    text: "",
                    /**		
					* Specifies the  location of the text.
					* @default [Array.Array.object] caption.subtitle.location={x:10,y:10}
                    * @alias ejBulletGraph#caption->subtitle.location
					* @type {object}
					* @example 
					*	//To set location API value during initialization  
					* 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:location:{x:10}}} ); 
					* @example 
					* //Get or set the location API, after initialization:<br>
					*	//Gets the location
					*	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
	                * var locateX = title.subtitle.location
					*	//Sets the location 
					*	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:location:{x:10}}] ); 
					* @memberof ejBulletGraph
					* @instance
					*/
                    location: {
                        /**		
                        * Specifies the horizontal position of the text.
                        * @default [Array.Array.Array.number] caption.subtitle.location.x=10
                        * @alias ejBulletGraph#caption->subtitle.location.x
                        * @type {number}
                        * @example 
                        *	//To set x API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:location:{x:10}}} ); 
                        * @example 
                        * //Get or set the x API, after initialization:<br>
                        *	//Gets the x
                        *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                        *var locateX = title.subtitle.location.x;		
                        *	//Sets the x 
                        *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:location:{x:10}}] ); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        x: 10,
                        /**		
                        * Specifies the vertical position of the text.
                        * @default [Array.Array.Array.number] caption.subtitle.location.y=45
                        * @alias ejBulletGraph#caption->subtitle.location.y
                        * @type {number}
                        * @example 
                        *	//To set y API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:location:{y:45}}} ); 
                        * @example 
                        * //Get or set the y API, after initialization:<br>
                        *	//Gets the y
                        *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                        *var locate = title.subtitle.location.y;	
                        *	//Sets the y 
                        *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:location:{y:45}} ]); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        y: 45
                    },
                    /**		
					* Specifies the font.
					* @default [Array.Array.object] caption.subtitle.font={color:null}
                    * @alias ejBulletGraph#caption->subtitle.font
					* @type {object}
					* @example 
					*	//To set font API value during initialization  
					* 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:font:{color:"black"}}} );
					* @example 
					* //Get or set the font API, after initialization:<br>
					*	//Gets the font
					*	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
	                *var textFont = title.subtitle.font;		
					*	//Sets the font 
					*	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:font:{color:"black"}}]); 
					* @memberof ejBulletGraph
					* @instance
					*/
                    font: {
                        /**		
                        * Specifies the color of the text.
                        * @default [Array.Array.Array.string] caption.subtitle.font.color=null
                        * @alias ejBulletGraph#caption->subtitle.font.color
                        * @type {string}
                        * @example 
                        *	//To set color API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:font:{color:"black"}}} ); 
                        * @example 
                        * //Get or set the color API, after initialization:<br>
                        *	//Gets the color
                        *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                        *var textFont = title.subtitle.font.color;				
                        *	//Sets the color 
                        *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:font:{color:"black"}}]); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        color: null,
                        /**		
                        * Specifies the fontFamily of the text.
                        * @default [Array.Array.Array.string] caption.subtitle.font.fontFamily="Segoe UI"
                        * @alias ejBulletGraph#caption->subtitle.font.fontFamily
                        * @type {string}
                        * @example 
                        *	//To set fontFamily API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:font:{fontFamily:"Segoe UI"}}} );	 
                        * @example 
                        * //Get or set the fontFamily API, after initialization:<br>
                        *	//Gets the fontFamily
                        *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                        *var textFont = title.subtitle.fontFamily;				
                        *	//Sets the fontFamily 
                        *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:font:{fontFamily:"Segoe UI"}}]); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        fontFamily: 'Segoe UI',
                        /**		
                        * Specifies the fontStyle of the text.See {@link fontStyle}
                        * @default [Array.Array.Array.enum] caption.subtitle.font.fontStyle="Normal"
                        * @alias ejBulletGraph#caption->subtitle.font.fontStyle
                        * @type {enum}
                        * @example 
                        *	//To set fontStyle API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:font:{fontStyle:ej.BulletGraph.fontStyle.Italic}}} ); 
                        * @example 
                        * //Get or set the fontStyle API, after initialization:<br>
                        *	//Gets the fontStyle
                        *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                        *var textFont = title.subtitle.fontStyle;			
                        *	//Sets the fontStyle 
                        *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:font:{fontStyle:ej.BulletGraph.fontStyle.Italic}}]); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        fontStyle: 'Normal ',
                        /**		
                        * Specifies the size of the text.
                        * @default [Array.Array.Array.string] caption.subtitle.font.size="12px"
                        * @alias ejBulletGraph#caption->subtitle.font.size
                        * @type {string}
                        * @example 
                        *	//To set size API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:font:{size:"12px"}}} ); 
                        * @example 
                        * //Get or set the size API, after initialization:<br>
                        *	//Gets the size
                        *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                        *var textFont = title.subtitle.size;				
                        *	//Sets the size 
                        *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:font:{size:"12px"}}]); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        size: '12px',
                        /**		
                        * Specifies the fontWeight of the text.See {@link fontWeight}
                        * @default [Array.Array.Array.enum] caption.subtitle.font.fontWeight="regular"
                        * @alias ejBulletGraph#caption->subtitle.font.fontWeight
                        * @type {enum}
                        * @example 
                        *	//To set fontWeight API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:font:{fontWeight:ej.BulletGraph.fontWeight.Normal}}} ); 
                        * @example 
                        * //Get or set the fontWeight API, after initialization:<br>
                        *	//Gets the fontWeight
                        *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                        *var textFont = title.subtitle.fontWeight;			
                        *	//Sets the fontWeight 
                        *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:font:{fontWeight:ej.BulletGraph.fontWeight.Normal}}]); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        fontWeight: 'regular',
                        /**		
                        * Specifies the opacity of the text.
                        * @default [Array.Array.Array.number] caption.subtitle.font.opacity=1
                        * @alias ejBulletGraph#caption->subtitle.font.opacity
                        * @type {number}
                        * @example 
                        *	//To set opacity API value during initialization  
                        * 	$("#bulletGraph1").ejBulletGraph({ caption:{subtitle:font:{opacity:0.5}}} ); 
                        * @example 
                        * //Get or set the opacity API, after initialization:<br>
                        *	//Gets the opacity
                        *	var title = $("#SampleBullet").ejBulletGraph("option", "caption");
                        *var textFont = title.subtitle.font.opacity;			
                        *	//Sets the opacity 
                        *	$("#bulletGraph1").ejBulletGraph("option", "caption",[{subtitle:font:{opacity:0.5}}]); 
                        * @memberof ejBulletGraph
                        * @instance
                        */
                        opacity: 1
                    }
                }
            },

            //Events
            /**     
			 * Fires after tick has been drawn.
			 * @event
			 * @name ejBulletGraph#drawTicks		
			 * @param{Object} args.Object returns the object of the bullet graph.
			 * @param{scaleElement}	args.scaleElement returns the options of the scale element.
			 * @param{tickElement} args.tickElement returns the current tick element.
			 * @param{tickType}	args.tickType returns the current tick element’s type.
			 * @example 
			 * //drawTicks event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawTicks: function (args) {}
             * });
			 * @memberof ejBulletGraph
			 * @instance
			 */
            drawTicks: null,
            /**     
			 * Fires after labels has been drawn.
			 * @event
			 * @name ejBulletGraph#drawLabels		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {labelElement} args.tickElement returns the current label element.
			 * @param {labelType} args.labelType returns the label type.
			 * @example 
			 * //drawLabels event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawLabels: function (args) {}
             * });
			 * @memberof ejBulletGraph
			 * @instance
			 */
            drawLabels: null,
            /**     
			 * Fires after caption has been drawn.
			 * @event
			 * @name ejBulletGraph#drawCaption		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {captionElement} args.captionElement returns the current caption element.
			 * @param {captionType} args.captionType returns the type of the caption.
			 * @example 
			 * //drawCaption event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawCaption: function (args) {}
             * });
			 * @memberof ejBulletGraph
			 * @instance
			 */
            drawCaption: null,
            /**     
			 * Fires after Qualitative Ranges has been drawn.
			 * @event
			 * @name ejBulletGraph#drawQualitativeRanges		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {captionElement} args.captionElement returns the current caption element.
			 * @param {captionType} args.captionType returns the type of the caption.
			 * @example 
			 * //drawQualitativeRanges event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawQualitativeRanges: function (args) {}
             * });
			 * @memberof ejBulletGraph
			 * @instance
			 */
            drawQualitativeRanges: null,
            /**     
			 * Fires after Feature Measure Bar has been drawn.
			 * @event
			 * @name ejBulletGraph#drawFeatureMeasureBar		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {currentElement} args.currentElement returns the options of feature measure element.
			 * @param {Value} args.Value returns the value of the feature measure bar.
			 * @example 
			 * //drawFeatureMeasureBar event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawFeatureMeasureBar: function (args) {}
             * });
			 * @memberof ejBulletGraph
			 * @instance
			 */
            drawFeatureMeasureBar: null,
            /**     
			 * Fires after catagory has been drawn.
			 * @event
			 * @name ejBulletGraph#drawCategory		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {categoryElement} args.categoryElement returns the options of category element.
			 * @param {Value}  args.Value returns the text value of the category that is drawn.
			 * @example 
			 * //drawCategory event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawCategory: function (args) {}
             * });
			 * @memberof ejBulletGraph
			 * @instance
			 */
            drawCategory: null,
            /**     
			 * Fires comparitive Measure symbol has been drawn.
			 * @event
			 * @name ejBulletGraph#drawComparativeMeasureSymbol		
			 * @param {Object}  args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {targetElement} args.targetElement returns the options of comparative measure element.
			 * @param {Value} args.Value returns the value of the comparative measure symbol.
			 * @example 
			 * //drawComparativeMeasureSymbol event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawComparativeMeasureSymbol: function (args) {}
             * });
			 * @memberof ejBulletGraph
			 * @instance
			 */
            drawComparativeMeasureSymbol: null
        },

        observables: ["value", "comparativeMeasureValue"],
        value: ej.util.valueFunction("value"),
        comparativeMeasureValue: ej.util.valueFunction("comparativeMeasureValue"),
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            quantitativeScale: {
                labels: "data",
                featureMeasure: "array"
            },
            fields: { dataSource: "data", query: "data" },
            qualitativeRanges: "array",
            caption: "data"
        },

        // constructor function
        /**
         * Create the bullet graph widget
		 * @private
         */
        _init: function () {
            this._renderBulletGraph();
        },
        /**
         * check whether SVG element
		 * @private
         */
        _isSVG: function () {
            if (window.SVGSVGElement)
                return true;
            else
                return false;
        },
        /**
         * rendering the bullet graph
		 * @private
         */
        _renderBulletGraph: function () {
            if (this._isSVG()) {
                this.svgRenderer = new ej.EjSvgRender(this.element);
                this.svgObject = this.svgRenderer.svgObj;
                this._trigger("load");
                for (var i = 0; this.model.quantitativeScale.featureMeasure[i] != null; i++) {
                    if (this.model.quantitativeScale.featureMeasure[i].value == null)
                        this.model.quantitativeScale.featureMeasure[i].value = this.value();
                    if (this.model.quantitativeScale.featureMeasure[i].comparativeMeasureValue == null)
                        this.model.quantitativeScale.featureMeasure[i].comparativeMeasureValue = this.comparativeMeasureValue();
                }
                this._setTheme(ej.BulletGraph.themes, this.model.theme);
                this._renderBulletElements();
                this.bindEvents();
                if (this.model.enableAnimation)
                    this._animateMeasures();
            }
        },
        /**
         * checking the animate state
		 * @private
         */
        _animateMeasures: function () {
            this._doAnimation();
            this._doLineAnimation();
        },
        /**
         * rendering the bullet Element
		 * @private
         */
        _renderBulletElements: function () {
            this.svgObject.setAttribute('width', this.model.width);
            this.svgObject.setAttribute('height', this.model.height);
            this.svgObject.setAttribute('viewBox', "0 0 " + this.model.width + " " + this.model.height);
            this.svgObject.setAttribute('preserveAspectRatio', "xMinYMin");
            this.svgWidth = this.model.width;
            this.svgHeight = this.model.height;
            var captionGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_captionGroup' });
            var scaleGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_scaleGroup' });
            var bulletGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_outerWrap' });

            this._initializeValues();
            var intervalValue = ((this.model.quantitativeScaleLength) / ((this._scale.maximum - this._scale.minimum) / this._scale.interval));

            //render caption
            this._drawCaption(captionGroup);

            //render scale
            this._scaleGroup = scaleGroup;
            this._drawScale(captionGroup, scaleGroup);

            //draw ranges
            this._drawQualitativeRanges(scaleGroup);

            //render ticks
            this._drawMajorTicks(intervalValue, scaleGroup);
            this._drawMinorTicks(intervalValue, scaleGroup);

            //render labels
            this._drawLabels(intervalValue, scaleGroup);

            // data binding
            this._bindData();

            $(captionGroup).appendTo(bulletGroup);
            $(scaleGroup).appendTo(bulletGroup);

            this._changeOrientation(bulletGroup);

            $(bulletGroup).appendTo(this.svgObject);
            $(this.svgObject).appendTo(this.element);
        },
        /**
         * Initializing the control values
		 * @private
         */
        _initializeValues: function () {
            this._scale = this.model.quantitativeScale;
            this._labelPosition = this._scale.labels.labelPosition;
            this._tickPosition = this._scale.tickPosition;
            this._flowDirection = this.model.flowDirection;
            this._orientation = this.model.orientation;
        },
        /**
         * checking the orientation
		 * @private
         */
        _changeOrientation: function (bulletGroup) {
            if (this._orientation == ej.BulletGraph.orientation.Vertical) {
                bulletGroup.setAttribute("transform", "translate(" + (this.model.height - this.model.width) + ", " + (-(this.model.width + (2 * this._scale.labels.labelOffset) + this._scale.labels.labelSize)) + ") scale(1) rotate(-90 " + this.model.width + " " + this.model.height + ")");
            }
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            var isFlag = true;
            for (var key in options) {
                this.model.enableAnimation = false;
                switch (key) {
                    case "height":
                        this.model.height = options[key];
                        break;
                    case "width":
                        this.model.width = options[key];
                        break;
                    case "theme":
                        this.model.theme = options[key];
                        this._setTheme(ej.BulletGraph.themes, this.model.theme);
                        break;
                    case "orientation":
                        this.model.orientation = options[key];
                        break;
                    case "flowDirection":
                        this.model.flowDirection = options[key];
                        break;
                    case "qualitativeRangeSize":
                        this.model.qualitativeRangeSize = options[key];
                        break;
                    case "quantitativeScaleLength":
                        this.model.quantitativeScaleLength = options[key];
                        break;
                    case "quantitativeScale":
                        $.extend(true, this.model.quantitativeScale, {}, options[key]);
                        break;
                    case "bindRangeStrokeToTicks":
                        this.model.bindRangeStrokeToTicks = options[key];
                        break;
                    case "bindRangeStrokeToLabels":
                        this.model.bindRangeStrokeToLabels = options[key];
                        break;
                    case "qualitativeRanges":
                        $.extend(true, this.model.qualitativeRanges, {}, options[key]);
                        break;
                    case "caption":
                        $.extend(true, this.model.caption, {}, options[key]);
                        break;
                    case "dataSource":
                        $.extend(true, this.model.fields, {}, options[key]);
                        break;
                    case "value":
                        for (var i = 0; this.model.quantitativeScale.featureMeasure[i] != null; i++) {
                            this.model.quantitativeScale.featureMeasure[i].value = parseFloat(this.value());
                        }
                        break;
                    case "comparativeMeasureValue":
                        for (var i = 0; this.model.quantitativeScale.featureMeasure[i] != null; i++) {
                            this.model.quantitativeScale.featureMeasure[i].comparativeMeasureValue = parseFloat(this.comparativeMeasureValue());
                        }
                        break;
                    case "enableAnimation":
                        this.model.enableAnimation = options[key];
                        if (this.model.enableAnimation) {
                            $(this.svgObject).empty();
                            this._renderBulletElements();
                            this._animateMeasures();
                        }
                        isFlag = false;
                        break;
                }
            }
            if (isFlag) {
                $(this.svgObject).empty();
                this._renderBulletElements();
            }
        },
        /**
         * To bind the datasource value		
		 * @private
         */
        _bindData: function () {

            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) {
                if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                    this._initDataSource();
                }
                else {
                    this._dataCount = this.model.fields["dataSource"].length;
                    this._drawMeasures();
                }
            }
            else {      //this.model.quantitativeScale.featureMeasure != null
                this._dataCount = this._scale.featureMeasure.length;
                this._drawMeasures();
            }
        },
        /**
         * To draw measures value		
		 * @private
         */
        _drawMeasures: function () {
            this._drawFeatureMeasureBar();
            this._drawComparativeMeasureSymbol();
        },
        /**
         * To draw measures value		
		 * @private
         */
        _initDataSource: function () {
            var query = this._columnToSelect(this.model.fields);
            var proxy = this;
            var queryPromise = this.model.fields["dataSource"].executeQuery(query);
            queryPromise.done(function (e) {
                proxy.model.fields["dataSource"] = e.result;
                proxy._dataCount = e.result.length;
                proxy._drawFeatureMeasureBar();
                proxy._drawComparativeMeasureSymbol();
            });
        },
        /**
         * To select column name from the datasource		
		 * @private
         */
        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "query" && col !== "dataSource")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },
        /**
         * To draw the caption for the graph	
		 * @private
         */
        _drawCaption: function (captionGroup) {
            if (this.model.drawCaption)
                this._onDrawCaption(this._textOptions(this.model.caption), "MainTitle", this.model.caption.text);
            this.svgRenderer.drawText(this._textOptions(this.model.caption), this.model.caption.text, captionGroup);
            if (this.model.caption.subtitle.text != "") {
                if (this.model.drawCaption)
                    this._onDrawCaption(this._textOptions(this.model.caption.subtitle), "SubTitle", this.model.caption.subtitle.text);
                this.svgRenderer.drawText(this._textOptions(this.model.caption.subtitle), this.model.caption.subtitle.text, captionGroup);
            }
        },
        /**
         * To give the font option to the text	
		 * @private
         */
        _textOptions: function (text) {
            var y = (text.text === this.model.caption.text) ? text.location.y : text.location.y - 15;
            var textOptions = {
                'x': text.location.x,
                'y': text.location.y,
                'fill': text.font.color,
                'font-size': text.font.size,
                'font-family': text.font.fontFamily,
                'font-style': text.font.fontStyle,
                'font-weight': text.font.fontWeight,
                'text-anchor': 'start',
                'opacity': text.font.opacity,
                'dominant-baseline': 'middle',
                'transform': 'rotate(' + text.textAngle + ',' + text.location.x + ',' + y + ')'
            };
            return textOptions;
        },
        /**
         * To draw the scales
		 * @private
         */
        _drawScale: function (captionGroup, scaleGroup) {
            var pointY, pointX;
            if (this._tickPosition == ej.BulletGraph.tickPosition.Below || this._tickPosition == ej.BulletGraph.tickPosition.Cross)
                pointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y : this._scale.location.y + this._scale.labels.labelOffset + this._scale.labels.labelSize;
            else
                pointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + this._scale.majorTickSize : this._scale.location.y + this._scale.labels.labelOffset + this._scale.labels.labelSize + this._scale.majorTickSize;

            if (this._flowDirection == ej.BulletGraph.flowDirection.Forward)
                pointX = this._scale.location.x;
            else
                pointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x - this._scale.labels.labelOffset : this._scale.location.x;

            var scaleOptions = {
                'id': this.svgObject.id + '_SvgScale',
                'x': pointX,
                'y': pointY,
                'width': this.model.quantitativeScaleLength,
                'height': this.model.qualitativeRangeSize,
                'fill': "transparent",
                'stroke-width': 0
            };
            this.svgRenderer.drawRect(scaleOptions, scaleGroup);
        },
        /**
         * To draw the major ticks
		 * @private
         */
        _drawMajorTicks: function (intervalValue, scaleGroup) {
            var majorPointX, y1, tickIndex = 0;
            if (this._flowDirection == ej.BulletGraph.flowDirection.Forward)
                majorPointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x + (this._scale.majorTickStrokeThickness / 2) : this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labels.labelOffset + (this._scale.majorTickStrokeThickness / 2);
            else
                majorPointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labels.labelOffset - (this._scale.majorTickStrokeThickness / 2) : this._scale.location.x + (this._scale.majorTickStrokeThickness / 2);

            if (this._tickPosition == ej.BulletGraph.tickPosition.Below)
                y1 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + this.model.qualitativeRangeSize : this._scale.location.y + this.model.qualitativeRangeSize + this._scale.labels.labelOffset + this._scale.labels.labelSize;
            else if (this._tickPosition == ej.BulletGraph.tickPosition.Above)
                y1 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y : this._scale.location.y + this._scale.labels.labelOffset + this._scale.labels.labelSize;
            else
                y1 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + ((this.model.qualitativeRangeSize) / 2) - (this._scale.majorTickSize / 2) : this._scale.location.y + ((this.model.qualitativeRangeSize) / 2) - (this._scale.majorTickSize / 2) + this._scale.labels.labelOffset + this._scale.labels.labelSize;

            for (var i = this._scale.minimum; i <= this._scale.maximum; i += this._scale.interval) {
                tickIndex++;
                if (i >= this._scale.maximum)
                    majorPointX -= this._scale.majorTickStrokeThickness;
                if (this._orientation == ej.BulletGraph.orientation.Vertical && this._flowDirection == ej.BulletGraph.flowDirection.Forward)
                    majorPointX = (i >= this._scale.maximum) ? majorPointX + (this._scale.majorTickStrokeThickness) : (i == this._scale.minimum) ? majorPointX - (this._scale.majorTickStrokeThickness) : majorPointX;

                if (this.model.bindRangeStrokeToTicks)
                    strokeColor = this._bindingRangeStrokes(majorPointX - (this._scale.majorTickStrokeThickness / 2));
                var options = {
                    'x1': majorPointX,
                    'y1': y1,
                    'x2': majorPointX,
                    'y2': y1 + this._scale.majorTickSize,
                    'stroke-width': this._scale.majorTickStrokeThickness,
                    'stroke': (this.model.bindRangeStrokeToTicks) ? strokeColor : this._scale.majorTickStroke
                };
                if (this._orientation == ej.BulletGraph.orientation.Vertical && this._flowDirection == ej.BulletGraph.flowDirection.Forward && i == this._scale.minimum)
                    majorPointX = majorPointX + this._scale.majorTickStrokeThickness;
                if (this.model.drawTicks)
                    this._onDrawTicks(options, "MajorTick", tickIndex);
                var majorTicks = this.svgRenderer.createLine(options);
                majorPointX = ((this._flowDirection == ej.BulletGraph.flowDirection.Backward && this._orientation == ej.BulletGraph.orientation.Horizontal) || (this._flowDirection == ej.BulletGraph.flowDirection.Forward && this._orientation == ej.BulletGraph.orientation.Vertical)) ? majorPointX - intervalValue : majorPointX + intervalValue;
                scaleGroup.appendChild(majorTicks);
            }
        },
        /**
         * To bind the stroke in forward direction
		 * @private
         */
        _forwardStrokeBinding: function (majorPointX) {
            if (majorPointX >= this._scale.location.x && majorPointX <= (this._rangeCollection[0] + this._scale.location.x)) {
                return this.model.qualitativeRanges[0].rangeStroke;
            }
            for (var k = 0; k <= this._rangeCollection.length - 1; k++) {
                if (majorPointX >= (this._rangeCollection[k] + this._scale.location.x) && majorPointX <= (this._rangeCollection[k + 1] + this._scale.location.x))
                    return this.model.qualitativeRanges[k + 1].rangeStroke;
            }
        },
        /**
         * To bind the stroke in backward direction
		 * @private
         */
        _backwardStrokeBinding: function (majorPointX) {
            if (majorPointX >= this._rangeCollection[this._rangeCollection.length - 1]) {
                return this.model.qualitativeRanges[0].rangeStroke;
            }
            for (k = 0; k <= this._rangeCollection.length - 1; k++) {
                if (majorPointX >= this._rangeCollection[k] && majorPointX < this._rangeCollection[k + 1]) {
                    var index = $.inArray(this._rangeCollection[k], this._rangeCollection)
                    return this.model.qualitativeRanges[(this._rangeCollection.length - 1) - index].rangeStroke;
                }
            }
        },
        /**
         * To bind the range stroke
		 * @private
         */
        _bindingRangeStrokes: function (majorPointX) {
            if (this._rangeCollection.length == 1) {
                if (majorPointX >= this._scale.location.x && majorPointX <= (this._rangeCollection[0] + this._scale.location.x))
                    return this.model.qualitativeRanges[0].rangeStroke;
            }
            else {
                if ((this.model.orientation == ej.BulletGraph.orientation.Horizontal && this._flowDirection == ej.BulletGraph.flowDirection.Forward) || (this.model.orientation == ej.BulletGraph.orientation.Vertical && this._flowDirection == ej.BulletGraph.flowDirection.Backward)) {
                    return this._forwardStrokeBinding(majorPointX);
                }
                else {
                    return this._backwardStrokeBinding(majorPointX);
                }
            }
        },
        /**
         * To draw minor ticks
		 * @private
         */
        _drawMinorTicks: function (intervalValue, scaleGroup) {
            var majorPointX, minorPointX, y1, y2, x, tickIndex = 0;

            if (this._flowDirection == ej.BulletGraph.flowDirection.Forward)
                majorPointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x : this._scale.location.x - this._scale.labels.labelOffset - (2 * this._scale.minorTickStrokeThickness);
            else
                majorPointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labels.labelOffset - (this._scale.minorTickStrokeThickness / 2) : this._scale.location.x;

            if (this._tickPosition == ej.BulletGraph.tickPosition.Below) {
                y1 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + this.model.qualitativeRangeSize : this._scale.location.y + this.model.qualitativeRangeSize + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                y2 = y1 + this._scale.minorTickSize;
            }
            else if (this._tickPosition == ej.BulletGraph.tickPosition.Above) {
                y1 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + this._scale.minorTickSize : this._scale.location.y + this._scale.labels.labelOffset + this._scale.labels.labelSize + this._scale.minorTickSize;
                y2 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + this._scale.majorTickSize : this._scale.location.y + this._scale.labels.labelOffset + this._scale.labels.labelSize + this._scale.majorTickSize;
            }
            else {
                y1 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + ((this.model.qualitativeRangeSize) / 2) - (this._scale.minorTickSize / 2) : this._scale.location.y + ((this.model.qualitativeRangeSize) / 2) - (this._scale.minorTickSize / 2) + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                y2 = y1 + this._scale.minorTickSize;
            }
            var maxLimit = (this._flowDirection == ej.BulletGraph.flowDirection.Backward && this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.maximum + 1 : this._scale.maximum;
            for (var i = this._scale.minimum; i <= maxLimit; i += this._scale.interval) {
                minorPointX = intervalValue / this._scale.minorTicksPerInterval;

                for (var j = 1; j <= this._scale.minorTicksPerInterval; j++) {
                    tickIndex++;
                    x = majorPointX + minorPointX - (minorPointX / (this._scale.minorTicksPerInterval + 1));

                    if (this._flowDirection == ej.BulletGraph.flowDirection.Backward && this._orientation == ej.BulletGraph.orientation.Horizontal) {
                        if (tickIndex >= ((this._scale.maximum - this._scale.minimum) * this._scale.minorTicksPerInterval))
                            x += (this._scale.minorTickStrokeThickness / 2);
                    }

                    if (this.model.bindRangeStrokeToTicks)
                        strokeColor = this._bindingRangeStrokes(x);
                    var options = {
                        'x1': x,
                        'y1': y1,
                        'x2': x,
                        'y2': y2,
                        'stroke-width': this._scale.minorTickStrokeThickness,
                        'stroke': (this.model.bindRangeStrokeToTicks) ? strokeColor : this._scale.minorTickStroke
                    };
                    if (this.model.drawTicks)
                        this._onDrawTicks(options, "MinorTick", tickIndex);
                    var chkMax = ((this._flowDirection == ej.BulletGraph.flowDirection.Backward && this._orientation == ej.BulletGraph.orientation.Horizontal) || (this._flowDirection == ej.BulletGraph.flowDirection.Forward && this._orientation == ej.BulletGraph.orientation.Vertical)) ? (this.model.quantitativeScaleLength + this._scale.location.x - this._scale.labels.labelOffset + 1) : (this.model.quantitativeScaleLength + this._scale.location.x + 1);
                    if (x <= chkMax) {
                        var minorTicks = this.svgRenderer.createLine(options);
                        scaleGroup.appendChild(minorTicks);
                    }
                    minorPointX = (intervalValue / this._scale.minorTicksPerInterval) * (j + 1);
                }
                majorPointX = (this._flowDirection == ej.BulletGraph.flowDirection.Backward && this._orientation == ej.BulletGraph.orientation.Horizontal) ? majorPointX - intervalValue : majorPointX + intervalValue;
            }
        },
        /**
         * To Draw labels
		 * @private
         */
        _drawLabels: function (intervalValue, scaleGroup) {
            var labelX, pointY;
            if (this._flowDirection == ej.BulletGraph.flowDirection.Forward)
                labelX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x : this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labels.labelOffset - (this._scale.majorTickStrokeThickness / 2);
            else
                labelX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labels.labelOffset - this._scale.majorTickStrokeThickness : this._scale.location.x - (this._scale.majorTickStrokeThickness / 2);

            if (this._tickPosition == ej.BulletGraph.tickPosition.Below || this._tickPosition == ej.BulletGraph.tickPosition.Cross)
                pointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + this.model.qualitativeRangeSize + this._scale.majorTickSize + this._scale.labels.labelOffset : this._scale.location.y + this._scale.labels.labelSize;
            else
                pointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + this.model.qualitativeRangeSize + this._scale.labels.labelOffset + this._scale.majorTickSize : this._scale.location.y + this._scale.labels.labelSize;

            for (var i = this._scale.minimum; i <= this._scale.maximum; i += this._scale.interval) {
                if (this._orientation == ej.BulletGraph.orientation.Vertical)
                    transformText = 'rotate(' + 90 + ',' + (labelX - 4) + ',' + pointY + ')';
                else
                    transformText = 'rotate(' + 0 + ',' + labelX + ',' + pointY + ')';
                labelX = (i == this._scale.maximum) ? labelX - (this._scale.majorTickStrokeThickness) : labelX;
                if (this._orientation == ej.BulletGraph.orientation.Vertical)
                    labelX = (this._flowDirection == ej.BulletGraph.flowDirection.Forward) ? (i == this._scale.maximum || i == this._scale.minimum) ? labelX + (this._scale.majorTickStrokeThickness) : labelX : (i == this._scale.minimum) ? labelX + (this._scale.majorTickStrokeThickness / 2) : labelX;

                if (this.model.bindRangeStrokeToLabels)
                    strokeColor = this._bindingRangeStrokes(labelX);

                var labelOptions = {
                    'x': labelX,
                    'y': pointY,
                    'text-anchor': 'middle',
                    'fill': (this.model.bindRangeStrokeToLabels) ? strokeColor : this._scale.labels.labelStroke,
                    'font-size': this._scale.labels.labelSize + "px",
                    'font-family': this._scale.labels.font.fontFamily,
                    'font-style': this._scale.labels.font.fontStyle,
                    'font-weight': this._scale.labels.font.fontWeight,
                    'opacity': this._scale.labels.font.opacity,
                    'transform': transformText
                };
                if (this._flowDirection == ej.BulletGraph.flowDirection.Forward)
                    labelX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? labelX + intervalValue : labelX - intervalValue;
                else
                    labelX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? labelX - intervalValue : labelX + intervalValue;

                if (this.model.drawLabels)
                    this._onDrawLabels(labelOptions, "AxisLabel", i);
                this.svgRenderer.drawText(labelOptions, i, scaleGroup);
            }
        },
        /**
         * To qualititative ranges
		 * @private
         */
        _drawQualitativeRanges: function (scaleGroup) {
            var pointX, pointY, width;
            this._rangeCollection = [];
            if (this._tickPosition == ej.BulletGraph.tickPosition.Below || this._tickPosition == ej.BulletGraph.tickPosition.Cross)
                pointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y : this._scale.location.y + this._scale.labels.labelOffset + this._scale.labels.labelSize;
            else
                pointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + this._scale.majorTickSize : this._scale.location.y + this._scale.labels.labelOffset + this._scale.labels.labelSize + this._scale.majorTickSize;

            for (var i = this.model.qualitativeRanges.length - 1; i >= 0; i--) {
                if (this._flowDirection == ej.BulletGraph.flowDirection.Forward) {
                    pointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x : this._scale.location.x - this._scale.labels.labelOffset + (this.model.quantitativeScaleLength - this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - this.model.qualitativeRanges[i].rangeEnd))));
                    width = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - this.model.qualitativeRanges[i].rangeEnd))) : this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - this.model.qualitativeRanges[i].rangeEnd)));
                }
                else {
                    pointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x - this._scale.labels.labelOffset + (this.model.quantitativeScaleLength - this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - this.model.qualitativeRanges[i].rangeEnd)))) : this._scale.location.x;
                    width = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - this.model.qualitativeRanges[i].rangeEnd))) : this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - this.model.qualitativeRanges[i].rangeEnd)));
                }
                var rangeOptions = {
                    'x': pointX,
                    'y': pointY,
                    'height': this.model.qualitativeRangeSize,
                    'width': width,
                    'fill': this.model.qualitativeRanges[i].rangeStroke,
                    'opacity': this.model.qualitativeRanges[i].rangeOpacity
                };
                ((this.model.orientation == ej.BulletGraph.orientation.Horizontal && this._flowDirection == ej.BulletGraph.flowDirection.Forward) || (this.model.orientation == ej.BulletGraph.orientation.Vertical && this._flowDirection == ej.BulletGraph.flowDirection.Backward)) ? this._rangeCollection.push(width) : this._rangeCollection.push(pointX);
                if (this.model.drawQualitativeRanges)
                    this._onDrawQualitativeRanges(rangeOptions, this.model.qualitativeRanges[i].rangeEnd, i);
                this.svgRenderer.drawRect(rangeOptions, scaleGroup);
            }
            this._rangeCollection.sort(this._sortRangeCollection);
        },
        /**
         * To return the range value
		 * @private
         */
        _sortRangeCollection: function (a, b) {
            return (a - b);
        },
        /**
         * To draw Feature Measure Bar
		 * @private
         */
        _drawFeatureMeasureBar: function () {
            var j = 1, count, value, categoryValue;
            var pointY, lPointY, pointX, width, lPointX;
            if (this._dataCount > 0) {
                for (var i = this._dataCount - 1; i >= 0; i--) {
                    value = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][i][this.model.fields.featureMeasure] : this._scale.featureMeasure[i].value;
                    categoryValue = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][i][this.model.fields.category] : this._scale.featureMeasure[i].category;

                    if (this._tickPosition == ej.BulletGraph.tickPosition.Below || this._tickPosition == ej.BulletGraph.tickPosition.Cross) {
                        pointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                        lPointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureBarStrokeThickness / 2) - 1 : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureBarStrokeThickness / 2) - 1 + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                    }
                    else {
                        pointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) + this._scale.majorTickSize : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) + this._scale.majorTickSize + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                        lPointY = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureBarStrokeThickness / 2) - 1 + this._scale.majorTickSize : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureBarStrokeThickness / 2) - 1 + this._scale.majorTickSize + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                    }

                    if (this._flowDirection == ej.BulletGraph.flowDirection.Forward) {
                        var stringLength = (!ej.isNullOrUndefined(categoryValue)) ? categoryValue.toString().length : 0;
                        pointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x : this._scale.location.x - this._scale.labels.labelOffset + (this.model.quantitativeScaleLength - this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))));
                        width = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))) : this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)));
                        lPointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? pointX - this._scale.labels.labelOffset - stringLength : this._scale.location.x + this.model.quantitativeScaleLength;
                    }
                    else {
                        pointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x - this._scale.labels.labelOffset + (this.model.quantitativeScaleLength - this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))) : this._scale.location.x;
                        width = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))) : this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)));
                        lPointX = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x + this.model.quantitativeScaleLength + this._scale.labels.labelOffset : this._scale.location.x - this._scale.labels.labelOffset;
                    }
                    var featureBarOptions = {
                        'class': this.svgObject.id + "_FeatureMeasure",
                        'id': this.svgObject.id + "_FeatureMeasure_" + i,
                        'x': pointX,
                        'y': pointY,
                        'height': this._scale.featuredMeasureBarStrokeThickness,
                        'width': width,
                        'fill': this._scale.featuredMeasureBarStroke,
                        'onmouseover': "evt.target.setAttribute('opacity', '0.7')",
                        'onmouseout': "evt.target.setAttribute('opacity', '1')"
                    };
                    if (this.model.drawFeatureMeasureBar)
                        this._onDrawFeatureMeasureBar(featureBarOptions, value);
                    this.svgRenderer.drawRect(featureBarOptions, this._scaleGroup);

                    if (this._orientation == ej.BulletGraph.orientation.Vertical)
                        transformText = 'rotate(' + 90 + ',' + lPointX + ',' + (lPointY - 4) + ')';
                    else
                        transformText = 'rotate(' + 0 + ',' + lPointX + ',' + lPointY + ')';

                    var categoryOptions = {
                        'x': lPointX,
                        'y': lPointY + (this._scale.featuredMeasureBarStrokeThickness / 2),
                        'text-anchor': 'middle',
                        'fill': this._scale.labels.labelStroke,
                        'font-size': this._scale.labels.labelSize + "px",
                        'font-family': this._scale.labels.font.fontFamily,
                        'font-style': this._scale.labels.font.fontStyle,
                        'font-weight': this._scale.labels.font.fontWeight,
                        'opacity': this._scale.labels.font.opacity,
                        'transform': transformText
                    };
                    if (this.model.drawCategory)
                        this._onDrawCategoryText(categoryOptions, categoryValue);
                    if (!ej.isNullOrUndefined(categoryValue))
                        this.svgRenderer.drawText(categoryOptions, categoryValue, this._scaleGroup);
                    j += 2;
                    this.value(value);
                }

            }
        },
        /**
         * To draw Comparitive Measure symbol
		 * @private
         */
        _drawComparativeMeasureSymbol: function () {
            var j = 1, value;
            var y1, y2, x1;
            if (this._dataCount > 0) {
                for (var i = this._dataCount - 1; i >= 0; i--) {
                    value = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][i][this.model.fields.comparativeMeasure] : this._scale.featureMeasure[i].comparativeMeasureValue;

                    if (this._tickPosition == ej.BulletGraph.tickPosition.Below || this._tickPosition == ej.BulletGraph.tickPosition.Cross) {
                        y1 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) - this._scale.featuredMeasureBarStrokeThickness : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) - this._scale.featuredMeasureBarStrokeThickness + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                        y2 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) + (2 * this._scale.featuredMeasureBarStrokeThickness) : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) + (2 * this._scale.featuredMeasureBarStrokeThickness) + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                    }
                    else {
                        y1 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) - this._scale.featuredMeasureBarStrokeThickness + this._scale.majorTickSize : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) - this._scale.featuredMeasureBarStrokeThickness + this._scale.majorTickSize + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                        y2 = (this._labelPosition == ej.BulletGraph.labelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) + (2 * this._scale.featuredMeasureBarStrokeThickness) + this._scale.majorTickSize : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureBarStrokeThickness / 2) + (2 * this._scale.featuredMeasureBarStrokeThickness) + this._scale.majorTickSize + this._scale.labels.labelOffset + this._scale.labels.labelSize;
                    }

                    if (this._flowDirection == ej.BulletGraph.flowDirection.Forward)
                        x1 = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x + (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))) : this._scale.location.x - this._scale.labels.labelOffset + (this.model.quantitativeScaleLength - (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))));
                    else
                        x1 = (this._orientation == ej.BulletGraph.orientation.Horizontal) ? this._scale.location.x - this._scale.labels.labelOffset + (this.model.quantitativeScaleLength - (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))))) : this._scale.location.x + (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))));

                    var compareMeasureOptions = {
                        'class': this.svgObject.id + "_ComparativeMeasure",
                        'id': this.svgObject.id + "_ComparativeMeasure_" + i,
                        'x1': (value == this._scale.maximum) ? x1 - (this._scale.comparativeMeasureSymbolStrokeThickness / 2) : (value == this._scale.minimum) ? x1 + (this._scale.comparativeMeasureSymbolStrokeThickness / 2) : x1,
                        'y1': y1,
                        'x2': (value == this._scale.maximum) ? x1 - (this._scale.comparativeMeasureSymbolStrokeThickness / 2) : (value == this._scale.minimum) ? x1 + (this._scale.comparativeMeasureSymbolStrokeThickness / 2) : x1,
                        'y2': y2,
                        'stroke-width': this._scale.comparativeMeasureSymbolStrokeThickness,
                        'stroke': this._scale.comparativeMeasureSymbolStroke
                    };
                    if (this.model.drawComparativeMeasureSymbol)
                        this._onDrawComparativeMeasure(compareMeasureOptions, value);
                    this.svgRenderer.drawLine(compareMeasureOptions, this._scaleGroup);
                    j += 2;
                    this.comparativeMeasureValue(value);
                }
            }
        },
        /**
         * To set the theme for the bullet graph
		 * @private
         */
        _setTheme: function (jsonObj, themeName) {
            var result = [];
            this._scale = this.model.quantitativeScale;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var i = 0; i < result.length; i++) {
                this._scale.majorTickStroke = ((!this._scale.majorTickStroke || this._scale.majorTickStroke == jsonObj[result[i]].quantitativeScale.majorTickStroke) ? jsonObj[themeName].quantitativeScale.majorTickStroke : this._scale.majorTickStroke);
                this._scale.minorTickStroke = ((!this._scale.minorTickStroke || this._scale.minorTickStroke == jsonObj[result[i]].quantitativeScale.minorTickStroke) ? jsonObj[themeName].quantitativeScale.minorTickStroke : this._scale.minorTickStroke);
                this._scale.labels.labelStroke = ((!this._scale.labels.labelStroke || this._scale.labels.labelStroke == jsonObj[result[i]].quantitativeScale.labels.labelStroke) ? jsonObj[themeName].quantitativeScale.labels.labelStroke : this._scale.labels.labelStroke);
                this._scale.featuredMeasureBarStroke = ((!this._scale.featuredMeasureBarStroke || this._scale.featuredMeasureBarStroke == jsonObj[result[i]].quantitativeScale.featuredMeasureBarStroke) ? jsonObj[themeName].quantitativeScale.featuredMeasureBarStroke : this._scale.featuredMeasureBarStroke);
                this._scale.comparativeMeasureSymbolStroke = ((!this._scale.comparativeMeasureSymbolStroke || this._scale.comparativeMeasureSymbolStroke == jsonObj[result[i]].quantitativeScale.comparativeMeasureSymbolStroke) ? jsonObj[themeName].quantitativeScale.comparativeMeasureSymbolStroke : this._scale.comparativeMeasureSymbolStroke);
                this.model.caption.font.color = ((!this.model.caption.font.color || this.model.caption.font.color == jsonObj[result[i]].caption.font.color) ? jsonObj[themeName].caption.font.color : this.model.caption.font.color);
                this.model.caption.subtitle.font.color = ((!this.model.caption.subtitle.font.color || this.model.caption.subtitle.font.color == jsonObj[result[i]].caption.subtitle.font.color) ? jsonObj[themeName].caption.subtitle.font.color : this.model.caption.subtitle.font.color);
                for (var j = 0; j < this.model.qualitativeRanges.length; j++)
                    this.model.qualitativeRanges[j].rangeStroke = ((!this.model.qualitativeRanges[j].rangeStroke) ? (ej.isNullOrUndefined(jsonObj[themeName].qualitativeRanges[j]) ? jsonObj[themeName].qualitativeRanges[0].rangeStroke : jsonObj[themeName].qualitativeRanges[j].rangeStroke) : this.model.qualitativeRanges[j].rangeStroke);
            }
            //            this.model = $.extend(true, this.model, jsonObj[themeName], this.model);
        },

        // ********************************* ClientSide Events ************************************
        /**
         * Section For handle the DrawTicks event
		 * @private
         */
        _onDrawTicks: function (tickOptions, tickType, tickIndex) {
            var data = { Object: this, scaleElement: this.model.quantitativeScale, tickIndex: tickIndex, tickElement: tickOptions, tickType: tickType };
            this._trigger("drawTicks", data);
        },
        /**
         * Section For handle the DrawLabels event
		 * @private
         */
        _onDrawLabels: function (labelOptions, labelType, labelValue) {
            var data = { Object: this, scaleElement: this.model.quantitativeScale, labelValue: labelValue, labelElement: labelOptions, labelType: labelType };
            this._trigger("drawLabels", data);

        },
        /**
         * Section For handle the DrawCaption event
		 * @private
         */
        _onDrawCaption: function (captionOptions, titleType, captionValue) {
            var data = { Object: this, scaleElement: this.model.quantitativeScale, captionText: captionValue, captionElement: captionOptions, captionType: titleType };
            this._trigger("drawCaption", data);
        },
        /**
         * Section For handle the DrawQualitativeRanges event
		 * @private
         */
        _onDrawQualitativeRanges: function (rangeOptions, rangeEnd, rangeIndex) {
            var data = { Object: this, scaleElement: this.model.quantitativeScale, rangeIndex: rangeIndex, rangeElement: rangeOptions, rangeEndValue: rangeEnd };
            this._trigger("drawQualitativeRanges", data);
        },
        /**
         * Section For handle the DrawFeatureMeasureBar event
		 * @private
         */
        _onDrawFeatureMeasureBar: function (featureOptions, value) {
            var data = { Object: this, scaleElement: this.model.quantitativeScale, currentElement: featureOptions, Value: value };
            this._trigger("drawFeatureMeasureBar", data);
        },
        /**
         * Section For handle the DrawCategoryText event
		 * @private
         */
        _onDrawCategoryText: function (categoryOptions, textValue) {
            var data = { Object: this, scaleElement: this.model.quantitativeScale, categoryElement: categoryOptions, Value: textValue };
            this._trigger("drawCategory", data);
        },
        /**
         * Section For handle the DrawComparativeMeasure event
		 * @private
         */
        _onDrawComparativeMeasure: function (compareMeasureOptions, value) {
            var data = { Object: this, scaleElement: this.model.quantitativeScale, targetElement: compareMeasureOptions, Value: value };
            this._trigger("drawComparativeMeasureSymbol", data);

        },
        /**
         * To animate the graph
		 * @private
         */
        _doAnimation: function () {
            var elements = $("." + this.svgObject.id + "_FeatureMeasure");
            for (var i = elements.length - 1; i >= 0; i--) {
                var element = elements[i];
                this._animateFeatureBar(element);
            }
        },
        /**
         * To animate the Line in the graph
		 * @private
         */
        _doLineAnimation: function () {
            var compareElements = $("." + this.svgObject.id + "_ComparativeMeasure");
            var secondsPerPoint = 2000 / compareElements.length;
            for (var i = compareElements.length - 1; i >= 0; i--) {
                var element = compareElements[i];
                $(element).attr("transform", "scale(0)");
                this._doLineSymbol(element, secondsPerPoint, i);
            }
        },
        /**
         * To animate the FeatureBar in the graph
		 * @private
         */
        _animateFeatureBar: function (element) {
            var box = element.getBBox();
            var left, top;
            if (((this._orientation == ej.BulletGraph.orientation.Horizontal) && (this._flowDirection == ej.BulletGraph.flowDirection.Backward)) || ((this._orientation == ej.BulletGraph.orientation.Vertical) && (this._flowDirection == ej.BulletGraph.flowDirection.Forward))) {
                left = box.x + box.width;
                top = box.y + box.height;
            }
            else {
                left = box.x;
                top = box.y;
            }
            $(element).animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now;
                    $(element).attr("transform", "translate(" + left + " " + top + ") scale(" + now + ",1) translate(" + (-left) + " " + (-top) + ")");
                }
            });
        },
        /**
         * To manupulate the line symbol
		 * @private
         */
        _doLineSymbol: function (element, sec, val) {
            var beginTime = parseInt(val * sec);
            var box = element.getBBox();
            var centerX = box.x + (box.width / 2);
            var centerY = box.y + (box.height / 2);
            $(element).delay(beginTime).animate(
                {
                    scale: 1
                },
                {
                    duration: 200,
                    step: function (now) {
                        $(element).attr("transform", "translate(" + centerX + " " + centerY + ") scale(" + now + ") translate(" + (-centerX) + " " + (-centerY) + ")");
                    }
                });
        },
        /**
         * Section For Binding event
		 * @private
         */
        bindEvents: function () {
            if (this.model.showTooltip)
                this._on($(this.svgObject), "mousemove", this._bulletMouseMove);
            if (this.model.canResize) {
                if (!this.isDevice())
                    this._on($(window), "resize", this._bulletResize);
                else
                    this._on($(window), "orientationchange", this._bulletResize);
            }
        },
        /**
         * To check whether it is a device or not
		 * @private
         */
        isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
         * To resize the bullet
		 * @private
         */
        _bulletResize: function () {
            var bullet = this;
            var $svgObj = $(bullet.svgObject);
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                if (!bullet.isDevice()) {
                    if (bullet.svgWidth > $(this).width()) {
                        $svgObj.width($(this).width());
                    } else {
                        $svgObj.width(bullet.svgWidth);
                    }
                    if (bullet.svgHeight > $(this).height()) {
                        $svgObj.height($(this).height());
                    } else {
                        $svgObj.height(bullet.svgHeight);
                    }

                }
                else {
                    $svgObj.width($(bullet.element).width());
                    $svgObj.height($(bullet.element).height());
                }
                bullet.redraw();
            }, 500);
        },
        /**
         * Section For handle the Mouse move event
		 * @private
         */
        _bulletMouseMove: function (evt) {
            var currentVal, targetVal, categoryVal, measureId, targetId, targetClass, tooltipdiv = $('.tooltipDiv');
            targetId = $(evt.target).attr("id");
            targetClass = $(evt.target).attr("class");
            if (targetClass != "undefined" && (targetClass == this.svgObject.id + '_FeatureMeasure' || targetClass == this.svgObject.id + '_ComparativeMeasure')) {
                measureId = targetId.substring(targetId.lastIndexOf("_") + 1);
                currentVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][measureId][this.model.fields.featureMeasure] : this._scale.featureMeasure[measureId].value;
                targetVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][measureId][this.model.fields.comparativeMeasure] : this._scale.featureMeasure[measureId].comparativeMeasureValue;
                categoryVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? (ej.isNullOrUndefined(this.model.fields["dataSource"][measureId][this.model.fields.category])) ? null : this.model.fields["dataSource"][measureId][this.model.fields.category] : (ej.isNullOrUndefined(this._scale.featureMeasure[measureId].category)) ? null : this._scale.featureMeasure[measureId].category;
                var data = { currentValue: currentVal, targetValue: targetVal, category: categoryVal };
                if ($('.tooltipDiv').length == 0) {
                    tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 105; display: block;'></div>");
                    $(document.body).append(tooltipdiv);
                    $('.tooltipDiv').addClass(this.svgObject.id);
                }
                if (this.model.tooltipTemplateId != "" && this.model.tooltipTemplateId != null) {
                    var cloneNode = $("#" + this.model.tooltipTemplateId).clone();
                    $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                    $(tooltipdiv).html($(cloneNode).render(data));
                } else {
                    var category = (!ej.isNullOrUndefined(data.category)) ? ("<br/> Category : " + data.category) : "";
                    $(tooltipdiv).html("Current : " + data.currentValue + "<br/> Target : " + data.targetValue + category);
                }
                $(tooltipdiv).css("font-size", "12px");
                var xPos = evt.clientX + $(document).scrollLeft() + 10;
                var yPos = evt.clientY + $(document).scrollTop() + 10;
                xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
                yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());

                if (xPos === undefined || xPos === null)
                    xPos = evt.clientX + $(document).scrollLeft() + 10;
                if (yPos === undefined || yPos === null)
                    yPos = evt.clientY + $(document).scrollTop() + 10;
                $(tooltipdiv).css("left", xPos);
                $(tooltipdiv).css("top", yPos);
                $(tooltipdiv).css({
                    '-webkit-border-radius': "5px 5px 5px 5px",
                    '-moz-border-radius': "5px 5px 5px 5px",
                    '-o-border-radius': "5px 5px 5px 5px",
                    'border-radius': "5px 5px 5px 5px",
                    'background-color': "White",
                    'border': "1px Solid Black",
                    'padding-bottom': "5px",
                    'padding-left': "5px",
                    'padding-right': "5px",
                    'padding-top': "5px"
                });
            }
            else if (targetClass != this.svgObject.id + '_FeatureMeasure' && targetClass != this.svgObject.id + '_ComparativeMeasure') {
                $(".tooltipDiv").remove();
            }
        },

        //-----Public methods----------//
        /**
        * To redraw the bulet graph  
		* @return jQuery
		* @example 
		* &lt;div id="bulletgraph1"&gt;Bullet Graph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Create Bullet graph
		* var gphObj = $("#bulletGraph1").data("ejBulletGraph");
		* gphObj.redraw(); // redraw the graph
		* &lt;/script&gt;
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // redraw the Bullet graph
		* $("#butbulletGraph1ton1").ejButton("redraw");	
		* &lt;/script&gt;
		*@memberof ejBulletGraph
		* @instance
        */
        redraw: function () {
            $(this.svgObject).empty();
            this._renderBulletElements();
        },
        /**
        * To destroy the bullet graph  
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="bulletgraph1"&gt;Bullet Graph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Destroy Bullet graph
		* var gphObj = $("#bulletGraph1").data("ejBulletGraph");
		* gphObj.destroy(); // destroy the graph
		* &lt;/script&gt;
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // destroy the Bullet graph
		* $("#bulletGraph1").ejButton("destroy");	
		* &lt;/script&gt;
		*@memberof ejBulletGraph
		* @instance
        */
        _destroy: function () {
            $(this.element).removeClass("e-bulletgraph e-js").find("#" + this.svgObject.id).remove();
        },
        /**
        * To set FeatureMeasureBarValue 
		* @return jQuery
		* @param {number} argument.Index value for the graph 
		* @param {number}  argument.Measure value for the graph
		* @example 
		* &lt;div id="bulletGraph1"&gt;Bulletgraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setFeatureMeasureBarValue
		* var btnObj = $("#bulletGraph1").data("ejBulletGraph");
		* btnObj.setFeatureMeasureBarValue(1,8); // set the value
		* &lt;/script&gt;
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setFeatureMeasureBarValue
		* $("#bulletGraph1").ejBulletGraph("setFeatureMeasureBarValue(1,8)");	
		* &lt;/script&gt;
		*@memberof ejBulletGraph
		* @instance
        */
        setFeatureMeasureBarValue: function (index, measureValue) {
            this._scale = this.model.quantitativeScale;
            var length = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"].length : this._scale.featureMeasure.length; //(this._scale.featureMeasure != null) ? this._scale.featureMeasure.length : this.model.fields["dataSource"].length;
            if (index < length && measureValue != null) {
                if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                    this.model.fields["dataSource"][index][this.model.fields.featureMeasure] = measureValue;
                else
                    this._scale.featureMeasure[index].value = measureValue;
                this.redraw();
                if (this.model.enableAnimation)
                    this._doAnimation();
            }
        },
        /**
        * To set setComparativeMeasureSymbol 
		* @return jQuery
		* @param {number} argument.Index value for the graph 
		* @param {number}  argument.Measure value for the graph
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set setFeatureMeasureBarValue
		* var btnObj = $("#bulletGraph1").data("ejBulletGraph");
		* btnObj.setFeatureMeasureBarValue(1,7); // set the value
		* &lt;/script&gt;
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the setComparativeMeasureSymbol
		* $("#bulletGraph1").ejBulletGraph("setFeatureMeasureBarValue(1,7)");	
		* &lt;/script&gt;
		*@memberof ejBulletGraph
		* @instance
        */
        setComparativeMeasureSymbol: function (index, measureValue) {
            this._scale = this.model.quantitativeScale;
            var length = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"].length : this._scale.featureMeasure.length; //(this._scale.featureMeasure != null) ? this._scale.featureMeasure.length : this.model.fields["dataSource"].length;
            if (index < length && measureValue != null) {
                if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                    this.model.fields["dataSource"][index][this.model.fields.comparativeMeasure] = measureValue;
                else
                    this._scale.featureMeasure[index].comparativeMeasureValue = measureValue;
                this.redraw();
                if (this.model.enableAnimation)
                    this._doLineAnimation();
            }
        }

    });
    /**
	 * Enum for Bullet Graph Orientation	 
	 * @enum {string}
	 * @global 
	 */
    ej.BulletGraph.orientation = {
        /**  support for orientation in horizontal direction. */
        Horizontal: "horizontal",
        /**  support for orientation in vertical direction. */
        Vertical: "vertical"
    };
    /**
	 * Enum for Bullet Graph Tick Position 
	 * @enum {string}
	 * @global 
	 */
    ej.BulletGraph.tickPosition = {
        /**  To set the Tick position below. */
        Below: "below",
        /**  To set the Tick position above. */
        Above: "above",
        /**  To set the Tick position cross. */
        Cross: "cross"
    };
    /**
	 * Enum for Bullet Graph Label Position 
	 * @enum {string}
	 * @global 
	 */
    ej.BulletGraph.labelPosition = {
        /**  To set the label position below. */
        Below: "below",
        /**  To set the label position above. */
        Above: "above"
    };
    /**
	 * Enum for Bullet Graph Flow Direction 
	 * @enum {string}
	 * @global 
	 */
    ej.BulletGraph.flowDirection = {
        /**  To set the flow direction forward. */
        Forward: "forward",
        /**  To set the flow direction backward. */
        Backward: "backward"
    };
    /**
	 * Enum for Font Style  
	 * @enum {string}
	 * @global 
	 */
    ej.BulletGraph.fontStyle = {
        /**  To set the fontStyle Normal. */
        Normal: "Normal",
        /**  To set the fontStyle Italic. */
        Italic: "Italic",
        /**  To set the fontStyle Oblique. */
        Oblique: "Oblique"
    };
    /**
	 * Enum for Font Weight 
	 * @enum {string}
	 * @global 
	 */
    ej.BulletGraph.fontWeight = {
        /**  To set the fontWeight Normal. */
        Normal: "Normal",
        /**  To set the fontWeight Bold. */
        Bold: "Bold",
        /**  To set the fontWeight Bolder. */
        Bolder: "Bolder",
        /**  To set the fontWeight Lighter. */
        Lighter: "Lighter"
    };
    /**
	 * Enum for Bullet Graph Theme3
	 * @enum {string}
	 * @global 
	 */
    ej.BulletGraph.themes = {
        /**  To set the theme flatlight. */
        flatlight: {
            quantitativeScale: {
                majorTickStroke: "#191919",
                minorTickStroke: "#191919",
                labels: {
                    labelStroke: "#191919"
                },
                featuredMeasureBarStroke: "#191919",
                comparativeMeasureSymbolStroke: "#191919"
            },
            qualitativeRanges: [{
                rangeStroke: "#ebebeb"
            }, {
                rangeStroke: "#d8d8d8"
            }, {
                rangeStroke: "#7f7f7f"
            }],
            caption: {
                font: { color: "#191919" },
                subtitle: {
                    font: { color: "#191919" }
                }
            }
        },
        /**  To set the theme flatdark. */
        flatdark: {
            /**  To set the theme quantitativeScale. */
            quantitativeScale: {
                majorTickStroke: "#ffffff",
                minorTickStroke: "#ffffff",
                labels: {
                    labelStroke: "#ffffff"
                },
                featuredMeasureBarStroke: "#ffffff",
                comparativeMeasureSymbolStroke: "#ffffff"
            },
            qualitativeRanges: [{
                rangeStroke: "#b3b3b3"
            }, {
                rangeStroke: "#999999"
            }, {
                rangeStroke: "#4d4d4d"
            }],
            caption: {
                font: { color: "#ffffff" },
                subtitle: {
                    font: { color: "#ffffff" }
                }
            }
        }
    };

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html RangeNavigator elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
         * @namespace ej
         * @class ejRangeNavigator
         * @requires jQuery
         * @classdesc Defined values to render a RangeNavigator.
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Create RangeNavigator
         * $('#container').RangeNavigator(); 	
         * &lt;/script&gt;
         */


    ej.widget("ejRangeNavigator", "ej.RangeNavigator", {

        ejChart: "",

        validTags: ["div"],
        // default model
        defaults: {
            /**		
               * Rangenavigator allows us to customize and set the theme colors as an background of RangeNavigator. We can set our wish theme as a rangenavigator backgournd using theme property of rangeNavigator.
                 @default null
               * @type {string}
               * @example 
               * //To set theme for RangeNavigator during initialization  
               * 	$("#container").ejRangeNavigator({theme:"flatlight"});					
               * @example 
               * //Gets or sets the theme for RangeNavigator, after initialization:<br>
               *	//Gets the theme for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", "theme");<br>			
               *	//Sets the theme for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", {theme:"flatlight"} ); 
                * @memberof ejRangeNavigator
               * @instance
               */
            theme: "",
            /**		
               * Padding is used to set the slider position at the start and end position of the rangenaivgator. which is used to display the slider without ovelapping.
                 @default "0"
               * @type {string}
               * @example 
               * //To set padding for RangeNavigator during initialization  
               * 	$("#container").ejRangeNavigator({padding:"15"});					
               * @example 
               * //Gets or sets the padding for RangeNavigator, after initialization:<br>
               *	//Gets the padding for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", "padding");<br>			
               *	//Sets the padding for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", {padding:"15"} ); 
                * @memberof ejRangeNavigator
               * @instance
               */
            padding: "0",
            /**		
               * canReSize property is used to set the height and width to rangenavigator dynamically when the browser or parent element div size dynamically changed. default value is false.
                 @default false
               * @type {boolean}
               * @example 
               * //To set canResize for RangeNavigator during initialization  
               * 	$("#container").ejRangeNavigator({canResize:false});					
               * @example 
               * //Gets or sets the canResize for RangeNavigator, after initialization:<br>
               *	//Gets the canResize for RangeNavigator 
               *	$("#container").ejRangeNavigator( "canResize");<br>			
               *	//Sets the canResize for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", {canResize:false} ); 
                * @memberof ejRangeNavigator
               * @instance
               */
            canResize: false,
            /**		
               * snappingMode specifies whether the slider should always place on the nearest range interval when drag the slider, or the slider should place dropped position exactly in the RangeNavigator.
                 @default false
               * @type {boolean}
               * @example 
               * //To set snappingMode for RangeNavigator during initialization  
               * 	$("#container").ejRangeNavigator({snappingMode:false});					
               * @example 
               * //Gets or sets the snappingMode for RangeNavigator, after initialization:<br>
               *	//Gets the snappingMode for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", "snappingMode");<br>			
               *	//Sets the snappingMode for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", {snappingMode:false} ); 
                * @memberof ejRangeNavigator
               * @instance
               */
            snappingMode: false,

            /**		
           * Specifies the size. Rangenavigator contains the default height and width. If we want to set height and width to rangenavigator explicitly we can set using size property, 
           * @type {object}
            * @memberof ejRangeNavigator
           * @instance
           */
            size: {
                /**		
               * Specifies width for RangeNavigator
                 @default null
               * @type {string}
               * @example 
               * //To set width for RangeNavigator during initialization  
               * 	$("#container").ejRangeNavigator({size:{width: "600"}});					
               * @example 
               * //Gets or sets the width for RangeNavigator, after initialization:<br>
               *	//Gets the width for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", "size.width");<br>			
               *	//Sets the width for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", {size:{width: "600"}} ); 
                * @alias ejRangeNavigator#size->width
               * @instance
               */
                width: "",
                /**		
               * Specifies height for RangeNavigator
                 @default null
               * @type {string}
               * @example 
               * //To set height for RangeNavigator during initialization  
               * 	$("#container").ejRangeNavigator({size:{height:"200"}});					
               * @example 
               * //Gets or sets the height for RangeNavigator, after initialization:<br>
               *	//Gets the height for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", "size.height");<br>			
               *	//Sets the height for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", {size:{height:"200"}} ); 
                * @alias ejRangeNavigator#size->height
               * @instance
               */
                height: ""
            },
            _size: {

            },
            /**		
               * Localization  is used to display the rangenavigator labels and toolTip values in the given localization format. EjRangeNavigator gives the globalize language support in the control. Default it supports 'en-US' language as localization.
                 @default "en-US"
               * @type {string}
               * @example 
               * //To set localization for RangeNavigator during initialization  
               * 	$("#container").ejRangeNavigator({localization:"en-US"});					
               * @example 
               * //Gets or sets the localization for RangeNavigator, after initialization:<br>
               *	//Gets the localization for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", "localization");<br>			
               *	//Sets the localization for RangeNavigator 
               *	$("#container").ejRangeNavigator("option", {localization:"en-US"} ); 
                * @memberof ejRangeNavigator
               * @instance
               */
            localization: "en-US",
            /**		
              * Rangenavigator supports two types of value type dateTime and numeric value type. by default it will support 'dateTime' value type.
                @default "datetime"
              * @type {enum}
              * @example 
              * //To set valueType for RangeNavigator during initialization  
              * 	$("#container").ejRangeNavigator({valueType:"double"});					
              * @example 
              * //Gets or sets the valueType for RangeNavigator, after initialization:<br>
              *	//Gets the valueType for RangeNavigator 
              *	$("#container").ejRangeNavigator("option", "valueType");<br>			
              *	//Sets the valueType for RangeNavigator 
              *	$("#container").ejRangeNavigator("option", {valueType:"double"} ); 
               * @memberof ejRangeNavigator
              * @instance
              */
            valueType: "datetime",
            /**		
           * valueAxisSettings is used to customize the rangenavigator axisLines, gridLines, tickLines and Fonts and used to set rangePadding type.
           * @type {object}
            * @memberof ejRangeNavigator
           * @instance
           */
            valueAxisSettings: {
                rangePadding: 'none',
                axisLine: { visible: false },
                font: { size: '0px', },
                majorTickLines:
                {
                    width: 0, size: 0, visible: true
                },
                majorGridLines: { visible: false },
                visible: false
            },
            /**		
              * rangePadding is used to set range in the specified range and it is used to avoid the label ovelapping. For numeric type, it have normal and none padding type. for datetime it have 'round' padding.
                @default "none"
              * @type {enum}
              * @example 
              * //To set rangePadding for RangeNavigator during initialization  
              * 	$("#container").ejRangeNavigator({rangePadding:"normal"});					
              * @example 
              * //Gets or sets the rangePadding for RangeNavigator, after initialization:<br>
              *	//Gets the rangePadding for RangeNavigator 
              *	$("#container").ejRangeNavigator("option", "rangePadding");<br>			
              *	//Sets the rangePadding for RangeNavigator 
              *	$("#container").ejRangeNavigator("option", {rangePadding:"normal"} ); 
               * @memberof ejRangeNavigator
              * @instance
              */
            rangePadding: "none",
            /**		
         * isRTL is used to change the rendering format "Right to Left". default it will render always "Left to Right" in RangeNavigator. 
           @default false
         * @type {boolean}
         * @example 
         * //To set isRTL for RangeNavigator during initialization  
         * 	$("#container").ejRangeNavigator({isRTL:true});					
         * @example 
         * //Gets or sets the isRTL for RangeNavigator, after initialization:<br>
         *	//Gets the isRTL for RangeNavigator 
         *	$("#container").ejRangeNavigator("option", "isRTL");<br>			
         *	//Sets the isRTL for RangeNavigator 
         *	$("#container").ejRangeNavigator("option", {isRTL:true} ); 
          * @memberof ejRangeNavigator
         * @instance
         */
            isRTL: false,
            /**		
           * dataSource property is used to set data to rangenavigator. The customer can set JSON data and local datasource to rangenavigator using this property.
           * @type {object}
            * @memberof ejRangeNavigator
           * @instance
           */
            dataSource: {
                /**		
         * Specifies data in dataSource for RangeNavigator. 
           @default null
         * @type {string}
         * @example 
         * //To set data in dataSource for RangeNavigator during initialization  
         * 	$("#container").ejRangeNavigator({dataSource:{data:"data.open"}});					
         * @example 
         * //Gets or sets the data in dataSource for RangeNavigator, after initialization:<br>
         *	//Gets the data in dataSource for RangeNavigator 
         *	$("#container").ejRangeNavigator("option", "dataSource.data");<br>			
         *	//Sets the data in dataSource for RangeNavigator 
         *	$("#container").ejRangeNavigator("option", {dataSource:{data:"data.open"}} ); 
         * @alias ejRangeNavigator#dataSource->data
         * @instance
         */
                data: "",
                /**		
        * Specifies xName in dataSource for RangeNavigator. 
          @default null
        * @type {string}
        * @example 
        * //To set xName in dataSource for RangeNavigator during initialization  
        * 	$("#container").ejRangeNavigator({dataSource:{xName:"field"}});					
        * @example 
        * //Gets or sets the xName in dataSource for RangeNavigator, after initialization:<br>
        *	//Gets the xName in dataSource for RangeNavigator 
        *	$("#container").ejRangeNavigator("option", "dataSource.xName");<br>			
        *	//Sets the xName in dataSource for RangeNavigator 
        *	$("#container").ejRangeNavigator("option", {dataSource:{xName:"field"}} ); 
         * @alias ejRangeNavigator#dataSource->xName
        * @instance
        */
                xName: "",
                /**		
       * Specifies yName in dataSource for RangeNavigator. 
         @default null
       * @type {string}
       * @example 
       * //To set yName in dataSource for RangeNavigator during initialization  
       * 	$("#container").ejRangeNavigator({dataSource:{yName:"field"}});					
       * @example 
       * //Gets or sets the yName in dataSource for RangeNavigator, after initialization:<br>
       *	//Gets the yName in dataSource for RangeNavigator 
       *	$("#container").ejRangeNavigator("option", "dataSource.yName");<br>			
       *	//Sets the yName in dataSource for RangeNavigator 
       *	$("#container").ejRangeNavigator("option", {dataSource:{yName:"field"}} ); 
        * @alias ejRangeNavigator#dataSource->yName
       * @instance
       */
                yName: ""
            },
            /**		
           * Specifies the tooltip display in the RangeNavigator when slider move.
           * @type {object}
            * @memberof ejRangeNavigator
           * @instance
           */
            tooltip: {
                /**		
      * Specifies the visibility of tooltip for RangeNavigator. 
        @default true
      * @type {boolean}
      * @example 
      * //To set visibility of tooltip for RangeNavigator during initialization  
      * 	$("#container").ejRangeNavigator({tooltip:{visible:false}});					
      * @example 
      * //Gets or sets the visibility of tooltip for RangeNavigator, after initialization:<br>
      *	//Gets the visibility of tooltip for RangeNavigator 
      *	$("#container").ejRangeNavigator("option", "tooltip.visible");<br>			
      *	//Sets the visibility of tooltip for RangeNavigator 
      *	$("#container").ejRangeNavigator("option", {tooltip:{visible:false}} ); 
      * @alias ejRangeNavigator#tooltip->visible
      * @instance
      */
                visible: true,
                /**		
   * Specifies the labelFormat of tooltip for RangeNavigator. 
     @default "MM/dd/yyyy"
   * @type {string}
   * @example 
   * //To set labelFormat of tooltip for RangeNavigator during initialization  
   * 	$("#container").ejRangeNavigator({tooltip:{labelFormat:"yyyy"}});					
   * @example 
   * //Gets or sets the labelFormat of tooltip for RangeNavigator, after initialization:<br>
   *	//Gets the labelFormat of tooltip for RangeNavigator 
   *	$("#container").ejRangeNavigator("option", "tooltip.labelFormat");<br>			
   *	//Sets the labelFormat of tooltip for RangeNavigator 
   *	$("#container").ejRangeNavigator("option", {tooltip:{labelFormat:"yyyy"}} ); 
   * @alias ejRangeNavigator#tooltip->labelFormat
   * @instance
   */
                labelFormat: "MM/dd/yyyy",
                /**		
  * tooltipDisplayMode is used to specify whether the slider toolTip should always display on the rangenavigator or only it should display when drag the sliderbar. default value of "tooltipDisplayMode" is 'always'. 
    @default null
  * @type {string}
  * @example 
  * //To set tooltipDisplayMode of tooltip for RangeNavigator during initialization  
  * 	$("#container").ejRangeNavigator({tooltip:{tooltipDisplayMode:"always"}});					
  * @example 
  * //Gets or sets the tooltipDisplayMode of tooltip for RangeNavigator, after initialization:<br>
  *	//Gets the tooltipDisplayMode of tooltip for RangeNavigator 
  *	$("#container").ejRangeNavigator("option", "tooltip.tooltipDisplayMode");<br>			
  *	//Sets the tooltipDisplayMode of tooltip for RangeNavigator 
  *	$("#container").ejRangeNavigator("option", {tooltip:{tooltipDisplayMode:"always"}} ); 
  * @alias ejRangeNavigator#tooltip->tooltipDisplayMode
  * @instance
  */
                tooltipDisplayMode: "always",
                /**		
  * Specifies the backgroundColor of tooltip for RangeNavigator. 
    @default "#303030"
  * @type {string}
  * @example 
  * //To set backgroundColor of tooltip for RangeNavigator during initialization  
  * 	$("#container").ejRangeNavigator({tooltip:{backgroundColor:"#ff0000"}});					
  * @example 
  * //Gets or sets the backgroundColor of tooltip for RangeNavigator, after initialization:<br>
  *	//Gets the backgroundColor of tooltip for RangeNavigator 
  *	$("#container").ejRangeNavigator("option", "tooltip.backgroundColor");<br>			
  *	//Sets the backgroundColor of tooltip for RangeNavigator 
  *	$("#container").ejRangeNavigator("option", {tooltip:{backgroundColor:"#ff0000"}} ); 
   * @alias ejRangeNavigator#tooltip->backgroundColor
  * @instance
  */
                backgroundColor: "#303030",
                /**		
           * Specifies the labelStyle properties of RangeNavigator. 
           * @type {object}
            * @alias ejRangeNavigator#tooltip->labelstyles
           * @instance
           */
                labelstyles: {
                    /**		
           * Specifies the font properties of RangeNavigator.  
           * @type {object}
           * @alias ejRangeNavigator#tooltip->labelstyles->font
           * @instance
           */
                    font:
                    {
                        /**		
 * Specifies the color of the text in tooltip for RangeNavigator. 
   @default "#FFFFFF"
 * @type {string}
 * @example 
 * //To set the color of the text in tooltip for RangeNavigator during initialization  
 * 	$("#container").ejRangeNavigator({tooltip:{labelStyles:{ font:{color:"#ff0000"}}}});					
 * @example 
 * //Gets or sets the color of the text in tooltip for RangeNavigator, after initialization:<br>
 *	//Gets the color of the text in tooltip for RangeNavigator 
 *	$("#container").ejRangeNavigator("option", "tooltip.labelStyles.font.color");<br>			
 *	//Sets the color of the text in tooltip RangeNavigator 
 *	$("#container").ejRangeNavigator("option", {tooltip:{labelStyles:{ font:{color:"#ff0000"}}}} ); 
  * @alias ejRangeNavigator#tooltip->labelstyles->font->color
 * @instance
 */
                        color: '#FFFFFF',
                        /**		
* Specifies the fontFamily of the text in tooltip for RangeNavigator. 
  @default "Segoe UI"
* @type {string}
* @example 
* //To set the fontFamily of the text in tooltip for RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({tooltip:{labelStyles:{ font:{fontFamily:"Arial"}}}});					
* @example 
* //Gets or sets the fontFamily of the text in tooltip for RangeNavigator, after initialization:<br>
*	//Gets the fontFamily of the text in tooltip for RangeNavigator 
*	$("#container").ejRangeNavigator("option", "tooltip.labelStyles.font.fontFamily");<br>			
*	//Sets the fontFamily of the text in tooltip RangeNavigator 
*	$("#container").ejRangeNavigator("option", {tooltip:{labelStyles:{ font:{fontFamily:"Arial"}}}} ); 
 * @alias ejRangeNavigator#tooltip->labelstyles->font->fontFamily
* @instance
*/
                        fontFamily: 'Segoe UI',
                        /**		
* Specifies the fontStyle of the text in tooltip for RangeNavigator.
 @default ej.RangeNavigator.fontStyle.Normal
* @type {string}
* @example 
* //To set the fontStyle of the text in tooltip for RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({tooltip:{labelStyles:{ font:{fontStyle:"Normal"}}}});					
* @example 
* //Gets or sets the fontStyle of the text in tooltip for RangeNavigator, after initialization:<br>
*	//Gets the fontStyle of the text in tooltip for RangeNavigator 
*	$("#container").ejRangeNavigator("option", "tooltip.labelStyles.font.fontStyle");<br>			
*	//Sets the fontStyle of the text in tooltip RangeNavigator 
*	$("#container").ejRangeNavigator("option", {tooltip:{labelStyles:{ font:{fontStyle:"Normal"}}}} ); 
* @alias ejRangeNavigator#tooltip->labelstyles->font->fontStyle
* @instance
*/
                        fontStyle: 'Normal',
                        /**		
* Specifies the size of the text in tooltip for RangeNavigator. 
 @default "10px"
* @type {string}
* @example 
* //To set the size of the text in tooltip for RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({tooltip:{labelStyles:{ font:{size:"12px"}}}});					
* @example 
* //Gets or sets the size of the text in tooltip for RangeNavigator, after initialization:<br>
*	//Gets the size of the text in tooltip for RangeNavigator 
*	$("#container").ejRangeNavigator("option", "tooltip.labelStyles.font.size");<br>			
*	//Sets the size of the text in tooltip RangeNavigator 
*	$("#container").ejRangeNavigator("option", {tooltip:{labelStyles:{ font:{size:"12px"}}}} ); 
* @alias ejRangeNavigator#tooltip->labelstyles->font->size
* @instance
*/
                        size: '10px',
                        /**		
* Specifies the opacity of the text in tooltip for RangeNavigator. 
 @default 1
* @type {number}
* @example 
* //To set the opacity of the text in tooltip for RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({tooltip:{labelStyles:{ font:{opacity: 0.5}}}});					
* @example 
* //Gets or sets the opacity of the text in tooltip for RangeNavigator, after initialization:<br>
*	//Gets the opacity of the text in tooltip for RangeNavigator 
*	$("#container").ejRangeNavigator("option", "tooltip.labelStyles.font.opacity");<br>			
*	//Sets the opacity of the text in tooltip RangeNavigator 
*	$("#container").ejRangeNavigator("option", {tooltip:{labelStyles:{ font:{opacity:0.5}}}} ); 
* @alias ejRangeNavigator#tooltip->labelstyles->font->opacity
* @instance
*/opacity: 1,
                        /**		
* Specifies the fontWeight of the text in tooltip for RangeNavigator.  
 @default ej.RangeNavigator.fontWeight.Regular
* @type {string}
* @example 
* //To set the fontWeight of the text in tooltip for RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({tooltip:{labelStyles:{ font:{fontWeight:"regular"}}}});					
* @example 
* //Gets or sets the fontWeight of the text in tooltip for RangeNavigator, after initialization:<br>
*	//Gets the fontWeight of the text in tooltip for RangeNavigator 
*	$("#container").ejRangeNavigator("option", "tooltip.labelStyles.font.fontWeight");<br>			
*	//Sets the fontWeight of the text in tooltip RangeNavigator 
*	$("#container").ejRangeNavigator("option", {tooltip:{labelStyles:{ font:{fontWeight:"regular"}}}} ); 
* @alias ejRangeNavigator#tooltip->labelstyles->font->fontWeight
* @instance
*/
                        fontWeight: 'regular'
                    }
                },
            },
            /**		
          * Specifies the zoomCordinates properties of RangeNavigator. 
          * @type {object}
           * @memberof ejRangeNavigator
          * @instance
          */
            zoomCordinates: {

                /**		
* Specifies the zoomPosition of RangeNavigator. 
@default "0"
* @type {string}
* @example 
* //To set the zoomPosition of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({zoomCordinates:{zoomPosition:"0"}});					
* @example 
* //Gets or sets the zoomPosition of RangeNavigator, after initialization:<br>
*	//Gets the zoomPosition of RangeNavigator 
*	$("#container").ejRangeNavigator("option", "zoomCordinates.zoomPosition");<br>			
*	//Sets the zoomPosition of RangeNavigator 
*	$("#container").ejRangeNavigator("option", {zoomCordinates:{zoomPosition:"0"}} ); 
* @alias ejRangeNavigator#zoomCordinates->zoomPosition
* @instance
*/
                zoomPosition: "0",
                /**		
* Specifies the zoomFactor of primaryXAxis. This is to specify zoom factor value for axis zooming. The user can specify value from 0 to 1.
@default "1"
* @type {string}
* @example 
* //To set the zoomFactor of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({zoomCordinates:{zoomFactor:"1"}});					
* @example 
* //Gets or sets the zoomFactor of RangeNavigator, after initialization:<br>
*	//Gets the zoomFactor of RangeNavigator 
*	$("#container").ejRangeNavigator("option", "zoomCordinates.zoomFactor");<br>			
*	//Sets the zoomFactor of RangeNavigator 
*	$("#container").ejRangeNavigator("option", {zoomCordinates:{zoomFactor:"1"}} ); 
* @alias ejRangeNavigator#zoomCordinates->zoomFactor
* @instance
*/
                zoomFactor: "1"
            },
            /**		
          * selectedRange is used to select the specific range in the rangenavigator. based on this selectedRange info, the chart is draw its points and displaying in the chart area. It contains the start range and end range property.
          * @type {object}
           * @memberof ejRangeNavigator
          * @instance
          */
            selectedRange: {
                /**		
* Specifies the start range of RangeNavigator. 
@default null
* @type {string}
* @example 
* //To set the start range  of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({selectedRange:{start:"01/05/1992"}});					
* @example 
* //Gets or sets the start range  of RangeNavigator, after initialization:<br>
*	//Gets the start range  of RangeNavigator 
*	$("#container").ejRangeNavigator("option", "selectedRange.start");<br>			
*	//Sets the start range  of RangeNavigator 
*	$("#container").ejRangeNavigator("option", {selectedRange:{start:"01/05/1992"}} ); 
* @alias ejRangeNavigator#selectedRange->start
* @instance
*/
                start: "",
                /**		
* Specifies the end range of RangeNavigator. 
@default null
* @type {string}
* @example 
* //To set the end range  of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({selectedRange:{end:"01/05/1993"}});					
* @example 
* //Gets or sets the end range  of RangeNavigator, after initialization:<br>
*	//Gets the end range  of RangeNavigator 
*	$("#container").ejRangeNavigator("option", "selectedRange.end");<br>			
*	//Sets the end range  of RangeNavigator 
*	$("#container").ejRangeNavigator("option", {selectedRange:{end:"01/05/1993"}} ); 
* @alias ejRangeNavigator#selectedRange->end
* @instance
*/
                end: ""
            },

            selectedData: "",
            /**		
          * Specifies the range properties of RangeNavigator.
          * @type {object}
           * @memberof ejRangeNavigator
          * @instance
          */
            range: {
                /**		
* range property is used to set the range to RangeNavigator explicitly. range property contains the start range and end range. Based on this info rangenavigator display the labels in the higher level and lower level. 
@default null
* @type {string}
* @example 
* //To set the start range of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({range:{start:"01/05/1993"}});					
* @example 
* //Gets or sets the start range  of RangeNavigator, after initialization:<br>
*	//Gets the start range  of RangeNavigator 
*	$("#container").ejRangeNavigator("option", "range.start");<br>			
*	//Sets the start range  of RangeNavigator 
*	$("#container").ejRangeNavigator("option", {range:{start:"01/05/1993"}} ); 
* @alias ejRangeNavigator#range->start
* @instance
*/
                start: "",
                /**		
* Specifies the end range of RangeNavigator. 
@default null
* @type {string}
* @example 
* //To set the end range  of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({range:{end:"01/05/1993"}});					
* @example 
* //Gets or sets the end range  of RangeNavigator, after initialization:<br>
*	//Gets the end range  of RangeNavigator 
*	$("#container").ejRangeNavigator("option", "range.end");<br>			
*	//Sets the end range  of RangeNavigator 
*	$("#container").ejRangeNavigator("option", {range:{end:"01/05/1993"}} ); 
* @alias ejRangeNavigator#range->end
* @instance
*/
                end: ""
            },
            /**		
         * deferredUpdate is used to set whether chart is need to redraw again and again when slider is moving or need to draw only once when the slider is dropped. Default value is "true"
           @default true
         * @type {boolean}
         * @example 
         * //To set deferredUpdate for RangeNavigator during initialization  
         * 	$("#container").ejRangeNavigator({deferredUpdate:false});					
         * @example 
         * //Gets or sets the deferredUpdate for RangeNavigator, after initialization:<br>
         *	//Gets the deferredUpdate for RangeNavigator 
         *	$("#container").ejRangeNavigator("option", "deferredUpdate");<br>			
         *	//Sets the deferredUpdate for RangeNavigator 
         *	$("#container").ejRangeNavigator("option", {deferredUpdate:false} ); 
          * @alias ejRangeNavigator#range->deferredUpdate
         * @instance
         */
            deferredUpdate: true,

            series: "",

            seriesSettings: "",
            /**		
         * labelSettings property is used to customize the RangeNavigator labels colors, font, labelStyles, size, horizontalAlignment and opacity.
         * @type {object}
          * @memberof ejRangeNavigator
         * @instance
         */
            labelSettings: {
                /**		
         * Specifies the labelstyles properties of RangeNavigator.
         * @type {object}
          * @alias ejRangeNavigator#labelSettings->labelstyles
         * @instance
         */
                labelstyles: {
                    /**		
        * Specifies the font properties of RangeNavigator.
        * @type {object}
        * @alias ejRangeNavigator#labelSettings->labelstyles->font
        * @instance
        */
                    font: {
                        /**		
* Specifies the color of the text in RangeNavigator. 
  @default "#FFFFFF"
* @type {string}
* @example 
* //To set the color of the text during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{labelstyles:{ font:{color:"#ff0000"}}}});					
* @example 
* //Gets or sets the color of the text, after initialization:<br>
*	//Gets the color of the text 
*	$("#container").ejRangeNavigator("option", "labelSettings.labelstyles.font.color");<br>			
*	//Sets the color of the text
*	$("#container").ejRangeNavigator("option", {labelSettings:{labelstyles:{ font:{color:"#ff0000"}}}} ); 
 * @alias ejRangeNavigator#labelSettings->labelstyles->font->color
* @instance
*/
                        color: '#333333',
                        /**		
* Specifies the fontFamily of the text in RangeNavigator. 
  @default "Segoe UI"
* @type {string}
* @example 
* //To set the fontFamily of the text during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{labelstyles:{ font:{fontFamily:"Arial"}}}});					
* @example 
* //Gets or sets the fontFamily of the text, after initialization:<br>
*	//Gets the fontFamily of the text 
*	$("#container").ejRangeNavigator("option", "labelSettings.labelstyles.font.fontFamily");<br>			
*	//Sets the fontFamily of the text
*	$("#container").ejRangeNavigator("option", {labelSettings:{labelstyles:{ font:{fontFamily:"Arial"}}}} ); 
 * @alias ejRangeNavigator#labelSettings->labelstyles->font->fontFamily
* @instance
*/
                        fontFamily: 'Segoe UI',
                        /**		
  * Specifies the fontStyle of the text in RangeNavigator.
    @default "Normal"
  * @type {enum}
  * @example 
  * //To set the fontStyle of the text during initialization  
  * 	$("#container").ejRangeNavigator({labelSettings:{labelstyles:{ font:{fontStyle:"Normal"}}}});					
  * @example 
  * //Gets or sets the fontStyle of the text, after initialization:<br>
  *	//Gets the fontStyle of the text 
  *	$("#container").ejRangeNavigator("option", "labelSettings.labelstyles.font.fontStyle");<br>			
  *	//Sets the fontStyle of the text
  *	$("#container").ejRangeNavigator("option", {labelSettings:{labelstyles:{ font:{fontStyle:"Noraml"}}}} ); 
   * @alias ejRangeNavigator#labelSettings->labelstyles->font->fontStyle
  * @instance
  */
                        fontStyle: 'Normal',
                        /**		
* Specifies the size of the text in RangeNavigator. 
  @default "1px"
* @type {string}
* @example 
* //To set the size of the text during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{labelstyles:{ font:{size:"12px"}}}});					
* @example 
* //Gets or sets the size of the text, after initialization:<br>
*	//Gets the size of the text 
*	$("#container").ejRangeNavigator("option", "labelSettings.labelstyles.font.size");<br>			
*	//Sets the size of the text
*	$("#container").ejRangeNavigator("option", {labelSettings:{labelstyles:{ font:{size:"12px"}}}} ); 
* @alias ejRangeNavigator#labelSettings->labelstyles->font->size
* @instance
*/
                        size: '10px',
                        /**		
                        * Specifies the opacity of the text in RangeNavigator. 
                          @default 1
                        * @type {number}
                        * @example 
                        * //To set the opacity of the text during initialization  
                        * 	$("#container").ejRangeNavigator({labelSettings:{labelstyles:{ font:{opacity:0.5}}}});					
                        * @example 
                        * //Gets or sets the opacity of the text, after initialization:<br>
                        *	//Gets the opacity of the text 
                        *	$("#container").ejRangeNavigator("option", "labelSettings.labelstyles.font.opacity");<br>			
                        *	//Sets the opacity of the text
                        *	$("#container").ejRangeNavigator("option", {labelSettings:{labelstyles:{ font:{opacity:0.5}}}} ); 
                        * @alias ejRangeNavigator#labelSettings->labelstyles->font->opacity
                        * @instance
                        */
                        opacity: 1,
                        /**		
* Specifies the fontWeight of the text in RangeNavigator.
  @default "regular"
* @type {enum}
* @example 
* //To set the fontWeight of the text during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{labelstyles:{ font:{fontWeight:"Regular"}}}});					
* @example 
* //Gets or sets the fontWeight of the text, after initialization:<br>
*	//Gets the fontWeight of the text 
*	$("#container").ejRangeNavigator("option", "labelSettings.labelstyles.font.fontWeight");<br>			
*	//Sets the fontWeight of the text
*	$("#container").ejRangeNavigator("option", {labelSettings:{labelstyles:{ font:{fontWeight:"Regular"}}}} ); 
 * @alias ejRangeNavigator#labelSettings->labelstyles->font->fontWeight
* @instance
*/
                        fontWeight: 'regular'
                    },
                    /**		
* Specifies the horizontalAlignment of the text in RangeNavigator.
@default "center"
* @type {enum}
* @example 
* //To set the horizontalAlignment of the text during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{labelstyles:{ horizontalAlignment:"center"}}});					
* @example 
* //Gets or sets the horizontalAlignment of the text, after initialization:<br>
*	//Gets the horizontalAlignment of the text 
*	$("#container").ejRangeNavigator("option", "labelSettings.labelstyles.horizontalAlignment");<br>			
*	//Sets the horizontalAlignment of the text
*	$("#container").ejRangeNavigator("option", {labelSettings:{labelstyles:{ horizontalAlignment:"center"}}} ); 
* @alias ejRangeNavigator#labelSettings->labelstyles->horizontalAlignment
* @instance
*/
                    horizontalAlignment: "center"
                },

                /**		
       * higherLevel property is used to  display the labels in both top and bottom of the RangeNavigator. We mostly used this property to display the years in rangenavigator and top position.
       * @type {object}
        * @alias ejRangeNavigator#labelSettings->higherLevel
       * @instance
       */
                higherLevel: {
                    /**		
* Specifies the intervalType of RangeNavigator.
@default "years"
* @type {enum}
* @example 
* //To set the intervalType of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ intervalType:"days"}}});					
* @example 
* //Gets or sets  intervalType of RangeNavigator, after initialization:<br>
*	//Gets the intervalType of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.intervalType");<br>			
*	//Sets the intervalType of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ intervalType:"days"}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->intervalType
* @instance
*/
                    intervalType: 'years',
                    labelstyles: "",
                    /**		
       * gridLineStyle specfies the griLine colors, width, strokedDashArray, border properties to customize the RangeNavigator gridLines.
       * @type {object}
       * @alias ejRangeNavigator#labelSettings->higherLevel->gridLineStyle
       * @instance
       */
                    gridLineStyle: {
                        /**		
* Specifies the color of gridLines in higherLevel.
@default "#B5B5B5"
* @type {string}
* @example 
* //To set the color of gridLines in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ gridLineStyle :{color:"#ff0000"}}}});					
* @example 
* //Gets or sets  color of gridLines in RangeNavigator, after initialization:<br>
*	//Gets the color of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.gridLineStyle.color");<br>			
*	//Sets the color of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ gridLineStyle :{color:"#ff0000"}}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->gridLineStyle->color
* @instance
*/
                        color: "#B5B5B5",
                        /**		
* Specifies the width of gridLines in higherLevel.
@default "#B5B5B5"
* @type {string}
* @example 
* //To set the width of gridLines in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ gridLineStyle :{width:1}}}});					
* @example 
* //Gets or sets  width of gridLines in RangeNavigator, after initialization:<br>
*	//Gets the width of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.gridLineStyle.width");<br>			
*	//Sets the width of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ gridLineStyle :{width:1}}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->gridLineStyle->width
* @instance
*/
                        width: 1,
                        /**		
* Specifies the strokedasharray of gridLines in higherLevel.
@default "20 5 0"
* @type {string}
* @example 
* //To set the strokedasharray of gridLines in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ gridLineStyle :{strokedasharray:"20 10 5"}}}});					
* @example 
* //Gets or sets  strokedasharray of gridLines in RangeNavigator, after initialization:<br>
*	//Gets the strokedasharray of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.gridLineStyle.strokedasharray");<br>			
*	//Sets the strokedasharray of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ gridLineStyle :{strokedasharray:"20 10 5"}}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->gridLineStyle->strokedasharray
* @instance
*/
                        strokedasharray: "20 5 0"
                    },

                    /**		
       * Specifies the border properties of higherLevel.
       * @type {object}
        * @alias ejRangeNavigator#labelSettings->higherLevel->border
       * @instance
       */
                    border:
                    {
                        /**		
* Specifies the color of border in higherLevel.
@default "transparent"
* @type {string}
* @example 
* //To set the color of border in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ border :{color:"#ff0000"}}}});					
* @example 
* //Gets or sets  color of border in RangeNavigator, after initialization:<br>
*	//Gets the color of border in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.border.color");<br>			
*	//Sets the color of border in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ border :{color:"#ff0000"}}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->border->color
* @instance
*/
                        color: "transparent",
                        /**		
* Specifies the width of border in higherLevel.
@default "0.5"
* @type {string}
* @example 
* //To set the width of border in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ border :{width:1}}}});					
* @example 
* //Gets or sets  width of border in RangeNavigator, after initialization:<br>
*	//Gets the width of border in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.border.width");<br>			
*	//Sets the width of border in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ border :{width:1}}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->border->width
* @instance
*/
                        width: 0.5
                    },
                    /**		
* Specifies the fill color of higherLevel.
@default "transparent"
* @type {string}
* @example 
* //To set the fill color of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ fill:"days"}}});					
* @example 
* //Gets or sets  fill color of RangeNavigator, after initialization:<br>
*	//Gets the fill color of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.fill");<br>			
*	//Sets the fill color of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ fill:"days"}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->fill
* @instance
*/
                    fill: "transparent",
                    /**		
* Specifies the position of higherLevel.
@default "top"
* @type {enum}
* @example 
* //To set the position of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ position:"bottom"}}});					
* @example 
* //Gets or sets  position of RangeNavigator, after initialization:<br>
*	//Gets the position of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.position");<br>			
*	//Sets the position of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ position:"bottom"}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->position
* @instance
*/
                    position: "top",
                    /**		
* Specifies the visibility of higherLevel.
@default true
* @type {boolean}
* @example 
* //To set the visibility of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ visible:false}}});					
* @example 
* //Gets or sets  visibility of RangeNavigator, after initialization:<br>
*	//Gets the visibility of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.visible");<br>			
*	//Sets the visibility of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ visible:false}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->visible
* @instance
*/
                    visible: true,
                    /**		
* Specifies the labelPlacement of higherLevel.
@default "outside"
* @type {enum}
* @example 
* //To set the labelPlacement of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{higherLevel:{ labelPlacement:"inside"}}});					
* @example 
* //Gets or sets  labelPlacement of RangeNavigator, after initialization:<br>
*	//Gets the labelPlacement of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.higherLevel.labelPlacement");<br>			
*	//Sets the intervalabelPlacementlType of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higherLevel:{ labelPlacement:"inside"}}} ); 
* @alias ejRangeNavigator#labelSettings->higherLevel->labelPlacement
* @instance
*/
                    labelPlacement: "outside"
                },
                /**		
      * lowerLevel property is used to  display the labels in both top and bottom of the RangeNavigator. We mostly used this property to display the months and quarters in rangenavigator and bottom position.
      * @type {object}
       * @alias ejRangeNavigator#labelSettings->lowerLevel
      * @instance
      */
                lowerLevel: {
                    /**		
* Specifies the intervalType of lowerLevel.
@default "years"
* @type {enum}
* @example 
* //To set the intervalType of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ intervalType:"days"}}});					
* @example 
* //Gets or sets  intervalType of RangeNavigator, after initialization:<br>
*	//Gets the intervalType of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.intervalType");<br>			
*	//Sets the intervalType of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ intervalType:"days"}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->intervalTypes
* @instance
*/
                    intervalType: 'quarters',
                    labelstyles: "",
                    /**		
      * Specifies the gridLineStyle properties of lowerLevel.
      * @type {object}
       * @alias ejRangeNavigator#labelSettings->lowerLevel->gridLineStyle
      * @instance
      */
                    gridLineStyle:
                    {  /**		
* Specifies the color of gridLines in lowerLevel.
@default "#B5B5B5"
* @type {string}
* @example 
* //To set the color of gridLines in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ gridLineStyle :{color:"#ff0000"}}}});					
* @example 
* //Gets or sets  color of gridLines in RangeNavigator, after initialization:<br>
*	//Gets the color of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.gridLineStyle.color");<br>			
*	//Sets the color of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{higlowerLevelherLevel:{ gridLineStyle :{color:"#ff0000"}}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->gridLineStyle->color
* @instance
*/
                        color: "#B5B5B5",
                        /**		
* Specifies the width of gridLines in lowerLevel.
@default "#B5B5B5"
* @type {string}
* @example 
* //To set the width of gridLines in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ gridLineStyle :{width:1}}}});					
* @example 
* //Gets or sets  width of gridLines in RangeNavigator, after initialization:<br>
*	//Gets the width of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.gridLineStyle.width");<br>			
*	//Sets the width of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ gridLineStyle :{width:1}}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->gridLineStyle->width
* @instance
*/
                        width: 1,
                        /**		
* Specifies the strokedasharray of gridLines in lowerLevel.
@default "20 5 0"
* @type {string}
* @example 
* //To set the strokedasharray of gridLines in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ gridLineStyle :{strokedasharray:"20 10 5"}}}});					
* @example 
* //Gets or sets  strokedasharray of gridLines in RangeNavigator, after initialization:<br>
*	//Gets the strokedasharray of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.gridLineStyle.strokedasharray");<br>			
*	//Sets the strokedasharray of gridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ gridLineStyle :{strokedasharray:"20 10 5"}}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->gridLineStyle->strokedasharray
* @instance
*/
                        strokedasharray: ""
                    },
                    /**		
      * Specifies the border properties of lowerLevel. 
      * @type {object}
     * @alias ejRangeNavigator#labelSettings->lowerLevel->border
      * @instance
      */
                    border:
                    {
                        /**		
* Specifies the color of border in lowerLevel.
@default "transparent"
* @type {string}
* @example 
* //To set the color of border in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ border :{color:"#ff0000"}}}});					
* @example 
* //Gets or sets  color of border in RangeNavigator, after initialization:<br>
*	//Gets the color of border in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.border.color");<br>			
*	//Sets the color of border in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ border :{color:"#ff0000"}}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->border->color
* @instance
*/
                        color: "transparent",
                        /**		
* Specifies the width of border in lowerLevel.
@default "0.5"
* @type {string}
* @example 
* //To set the width of border in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ border :{width:1}}}});					
* @example 
* //Gets or sets  width of border in RangeNavigator, after initialization:<br>
*	//Gets the width of border in RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.border.width");<br>			
*	//Sets the width of border in RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ border :{width:1}}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->border->width
* @instance
*/
                        width: 0.5
                    },
                    /**		
* Specifies the fill color of lowerLevel.
@default "transparent"
* @type {string}
* @example 
* //To set the fill color of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ fill:"days"}}});					
* @example 
* //Gets or sets  fill color of RangeNavigator, after initialization:<br>
*	//Gets the fill color of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.fill");<br>			
*	//Sets the fill color of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ fill:"days"}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->fill
* @instance
*/
                    fill: "transparent",
                    /**		
* Specifies the position of lowerLevel.
@default "bottom"
* @type {enum}
* @example 
* //To set the position of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ position:"bottom"}}});					
* @example 
* //Gets or sets  position of RangeNavigator, after initialization:<br>
*	//Gets the position of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.position");<br>			
*	//Sets the position of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ position:"bottom"}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->position
* @instance
*/
                    position: "bottom",
                    /**		
* Specifies the visibility of lowerLevel.
@default true
* @type {boolean}
* @example 
* //To set the visibility of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ visible:false}}});					
* @example 
* //Gets or sets  visibility of RangeNavigator, after initialization:<br>
*	//Gets the visibility of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.visible");<br>			
*	//Sets the visibility of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ visible:false}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->visible
* @instance
*/

                    visible: true,
                    /**		
* Specifies the labelPlacement of lowerLevel.
@default "outside"
* @type {enum}
* @example 
* //To set the labelPlacement of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({labelSettings:{lowerLevel:{ labelPlacement:"inside"}}});					
* @example 
* //Gets or sets  labelPlacement of RangeNavigator, after initialization:<br>
*	//Gets the labelPlacement of RangeNavigator
*	$("#container").ejRangeNavigator("option", "labelSettings.lowerLevel.labelPlacement");<br>			
*	//Sets the intervalabelPlacementlType of RangeNavigator
*	$("#container").ejRangeNavigator("option", {labelSettings:{lowerLevel:{ labelPlacement:"inside"}}} ); 
* @alias ejRangeNavigator#labelSettings->lowerLevel->labelPlacement
* @instance
*/
                    labelPlacement: "outside"
                }
            },
            /**		
     * navigatorStyle property is used to customize the selected and unselected RegionColor, thumbColor, thumbRadius, background, border and opacity of navigatorStyle.
     * @type {object}
      * @memberof ejRangeNavigator
     * @instance
     */
            navigatorStyle: {
                /**		
* Specifies the selectedRegionColor of RangeNavigator.
@default "#EFEFEF"
* @type {string}
* @example 
* //To set the selectedRegionColor of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({navigatorStyle:{ selectedRegionColor:"#ff0000"}});					
* @example 
* //Gets or sets  selectedRegionColor of RangeNavigator, after initialization:<br>
*	//Gets the selectedRegionColor of RangeNavigator
*	$("#container").ejRangeNavigator("option", "navigatorStyle.selectedRegionColor");<br>			
*	//Sets the selectedRegionColor of RangeNavigator
*	$("#container").ejRangeNavigator("option", {navigatorStyle:{ selectedRegionColor:"#ff0000"}} ); 
* @alias ejRangeNavigator#navigatorStyle->selectedRegionColor
* @instance
*/
                selectedRegionColor: "#EFEFEF",
                /**		
* Specifies the unselectedRegionColor of RangeNavigator.
@default "#5EABDE"
* @type {string}
* @example 
* //To set the unselectedRegionColor of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({navigatorStyle:{ unselectedRegionColor:"#ff0000"}});					
* @example 
* //Gets or sets unselectedRegionColor of RangeNavigator, after initialization:<br>
*	//Gets the unselectedRegionColor of RangeNavigator
*	$("#container").ejRangeNavigator("option", "navigatorStyle.unselectedRegionColor");<br>			
*	//Sets the unselectedRegionColor of RangeNavigator
*	$("#container").ejRangeNavigator("option", {navigatorStyle:{ unselectedRegionColor:"#ff0000"}} ); 
* @alias ejRangeNavigator#navigatorStyle->unselectedRegionColor
* @instance
*/
                unselectedRegionColor: "#5EABDE",
                /**		
* Specifies the thumbColor of RangeNavigator.
@default "#2382C3"
* @type {string}
* @example 
* //To set the thumbColor of RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({navigatorStyle:{ thumbColor:"#ff0000"}});					
* @example 
* //Gets or sets thumbColor of RangeNavigator, after initialization:<br>
*	//Gets the thumbColor of RangeNavigator
*	$("#container").ejRangeNavigator("option", "navigatorStyle.thumbColor");<br>			
*	//Sets the thumbColor of RangeNavigator
*	$("#container").ejRangeNavigator("option", {navigatorStyle:{ thumbColor:"#ff0000"}} ); 
* @alias ejRangeNavigator#navigatorStyle->thumbColor
* @instance
*/
                thumbColor: "#2382C3",
                /**		
 * Specifies the thumbRadius of RangeNavigator.
 @default 10
 * @type {number}
 * @example 
 * //To set the thumbRadius of RangeNavigator during initialization  
 * 	$("#container").ejRangeNavigator({navigatorStyle:{ thumbRadius:15}});					
 * @example 
 * //Gets or sets thumbRadius of RangeNavigator, after initialization:<br>
 *	//Gets the thumbRadius of RangeNavigator
 *	$("#container").ejRangeNavigator("option", "navigatorStyle.thumbRadius");<br>			
 *	//Sets the thumbRadius of RangeNavigator
 *	$("#container").ejRangeNavigator("option", {navigatorStyle:{ thumbRadius:15}} ); 
 * @alias ejRangeNavigator#navigatorStyle->thumbRadius
 * @instance
 */
                thumbRadius: 10,
                /**		
 * Specifies the thumbStroke of RangeNavigator.
 @default "#303030"
 * @type {string}
 * @example 
 * //To set the thumbStroke of RangeNavigator during initialization  
 * 	$("#container").ejRangeNavigator({navigatorStyle:{ thumbStroke:"#ff0000"}});					
 * @example 
 * //Gets or sets thumbStroke of RangeNavigator, after initialization:<br>
 *	//Gets the thumbStroke of RangeNavigator
 *	$("#container").ejRangeNavigator("option", "navigatorStyle.thumbStroke");<br>			
 *	//Sets the thumbStroke of RangeNavigator
 *	$("#container").ejRangeNavigator("option", {navigatorStyle:{ thumbStroke:"#ff0000"}} ); 
 * @alias ejRangeNavigator#navigatorStyle->thumbStroke
 * @instance
 */
                thumbStroke: "#303030",
                /**		
 * Specifies the background of RangeNavigator.
 @default "#dddddd"
 * @type {string}
 * @example 
 * //To set the background of RangeNavigator during initialization  
 * 	$("#container").ejRangeNavigator({navigatorStyle:{ background:"#ff0000"}});					
 * @example 
 * //Gets or sets background of RangeNavigator, after initialization:<br>
 *	//Gets the background of RangeNavigator
 *	$("#container").ejRangeNavigator("option", "navigatorStyle.background");<br>			
 *	//Sets the background of RangeNavigator
 *	$("#container").ejRangeNavigator("option", {navigatorStyle:{ background:"#ff0000"}} ); 
* @alias ejRangeNavigator#navigatorStyle->background
 * @instance
 */
                background: "#dddddd",
                /**		
    * Specifies the border properties of RangeNavigator.
    * @type {object}
    * @alias ejRangeNavigator#navigatorStyle->border
    * @instance
    */
                border:
                {
                    /**		
* Specifies the border color.
@default "transparent"
* @type {string}
* @example 
* //To set the border color during initialization  
* 	$("#container").ejRangeNavigator({navigatorStyle:{ border:{color:"#ff0000"}} });					
* @example 
* //Gets or sets border color, after initialization:<br>
*	//Gets the border color of RangeNavigator
*	$("#container").ejRangeNavigator("option", "navigatorStyle.border.color");<br>			
*	//Sets the border color of RangeNavigator
*	$("#container").ejRangeNavigator("option", {navigatorStyle:{ border:{color:"#ff0000"}} }); 
* @alias ejRangeNavigator#navigatorStyle->border->color
* @instance
*/
                    color: "transparent",
                    /**		
* Specifies the border width.
@default 0.5
* @type {number}
* @example 
* //To set the border width during initialization  
* 	$("#container").ejRangeNavigator({navigatorStyle:{ border:{width:1}} });					
* @example 
* //Gets or sets border width, after initialization:<br>
*	//Gets the border width of RangeNavigator
*	$("#container").ejRangeNavigator("option", "navigatorStyle.border.width");<br>			
*	//Sets the border width of RangeNavigator
*	$("#container").ejRangeNavigator("option", {navigatorStyle:{ border:{width:1}} }); 
* @alias ejRangeNavigator#navigatorStyle->border->width
* @instance
*/
                    width: 0.5,
                    /**		
* Specifies the dash array.
@default null
* @type {string}
* @example 
* //To set thedash array during initialization  
* 	$("#container").ejRangeNavigator({navigatorStyle:{ border:{strokedasharray:"2,3"}} });					
* @example 
* //Gets or sets dash array, after initialization:<br>
*	//Gets the dash array of RangeNavigator
*	$("#container").ejRangeNavigator("option", "navigatorStyle.border.strokedasharray");<br>			
*	//Sets the dash array of RangeNavigator
*	$("#container").ejRangeNavigator("option", {navigatorStyle:{ border:{strokedasharray:"2,3"}} }); 
* @alias ejRangeNavigator#navigatorStyle->border->width
* @instance
*/
                    strokedasharray: ""
                },
                /**		
 * Specifies the thumbRadius of RangeNavigator.
 @default 10
 * @type {number}
 * @example 
 * //To set the thumbRadius of RangeNavigator during initialization  
 * 	$("#container").ejRangeNavigator({navigatorStyle:{ thumbRadius:15}});					
 * @example 
 * //Gets or sets thumbRadius of RangeNavigator, after initialization:<br>
 *	//Gets the thumbRadius of RangeNavigator
 *	$("#container").ejRangeNavigator("option", "navigatorStyle.thumbRadius");<br>			
 *	//Sets the thumbRadius of RangeNavigator
 *	$("#container").ejRangeNavigator("option", {navigatorStyle:{ thumbRadius:15}} ); 
  * @alias ejRangeNavigator#navigatorStyle->opacity
 * @instance
 */
                opacity: 1,
                /**		
    * Specifies the majorGridLineStyle properties of RangeNavigator.
    * @type {object}
     * @alias ejRangeNavigator#navigatorStyle->majorGridLineStyle
    * @instance
    */
                majorGridLineStyle:
                {
                    /**		
 * Specifies the color of majorGridLines of RangeNavigator.
 @default "#B5B5B5"
 * @type {string}
 * @example 
 * //To set the color of majorGridLines in RangeNavigator during initialization  
 * 	$("#container").ejRangeNavigator({navigatorStyle:{ majorGridLineStyle:{color:"#ff0000"}}});					
 * @example 
 * //Gets or sets color of majorGridLines in RangeNavigator, after initialization:<br>
 *	//Gets the color of majorGridLines in RangeNavigator
 *	$("#container").ejRangeNavigator("option", "navigatorStyle.majorGridLineStyle.color");<br>			
 *	//Sets the color of majorGridLines in RangeNavigator
 *	$("#container").ejRangeNavigator("option", {navigatorStyle:{ majorGridLineStyle:{color:"#ff0000"}}} ); 
   * @alias ejRangeNavigator#navigatorStyle->majorGridLineStyle->color
 * @instance
 */
                    color: "#B5B5B5",
                    /**		
* Specifies the visibility of majorGridLines of RangeNavigator.
@default true
* @type {boolean}
* @example 
* //To set the visibility of majorGridLines in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({navigatorStyle:{ majorGridLineStyle:{visible:false}}});					
* @example 
* //Gets or sets visibility of majorGridLines in RangeNavigator, after initialization:<br>
*	//Gets the visibility of majorGridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", "navigatorStyle.majorGridLineStyle.visible");<br>			
*	//Sets the visibility of majorGridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", {navigatorStyle:{ majorGridLineStyle:{visible:false}}} ); 
  * @alias ejRangeNavigator#navigatorStyle->majorGridLineStyle->visible
* @instance
*/
                    visible: true
                },
                /**		
    * Specifies the minorGridLineStyle properties of RangeNavigator.
    * @type {object}
       * @alias ejRangeNavigator#navigatorStyle->minorGridLineStyle
    * @instance
    */
                minorGridLineStyle:
                {
                    /**		
 * Specifies the color of majorGridLines of RangeNavigator.
 @default "#B5B5B5"
 * @type {string}
 * @example 
 * //To set the color of majorGridLines in RangeNavigator during initialization  
 * 	$("#container").ejRangeNavigator({navigatorStyle:{ majorGridLineStyle:{color:"#ff0000"}}});					
 * @example 
 * //Gets or sets color of majorGridLines in RangeNavigator, after initialization:<br>
 *	//Gets the color of majorGridLines in RangeNavigator
 *	$("#container").ejRangeNavigator("option", "navigatorStyle.majorGridLineStyle.color");<br>			
 *	//Sets the color of majorGridLines in RangeNavigator
 *	$("#container").ejRangeNavigator("option", {navigatorStyle:{ majorGridLineStyle:{color:"#ff0000"}}} ); 
 * @alias ejRangeNavigator#navigatorStyle->minorGridLineStyle->color
 * @instance
 */
                    color: "#B5B5B5",
                    /**		
* Specifies the visibility of majorGridLines of RangeNavigator.
@default true
* @type {boolean}
* @example 
* //To set the visibility of majorGridLines in RangeNavigator during initialization  
* 	$("#container").ejRangeNavigator({navigatorStyle:{ majorGridLineStyle:{visible:false}}});					
* @example 
* //Gets or sets visibility of majorGridLines in RangeNavigator, after initialization:<br>
*	//Gets the visibility of majorGridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", "navigatorStyle.majorGridLineStyle.visible");<br>			
*	//Sets the visibility of majorGridLines in RangeNavigator
*	$("#container").ejRangeNavigator("option", {navigatorStyle:{ majorGridLineStyle:{visible:false}}} ); 
* @alias ejRangeNavigator#navigatorStyle->minorGridLineStyle->visible
* @instance
*/
                    visible: true
                },
            },

            /**     
                         * Fires after RangeNavigator is loaded.
                         * @event
                         * @name ejRangeNavigator#loaded	
                         * @param {Object} argument.Data parameters from RangeNavigator     
                         * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
                         * @param {object}  argument.model returns the RangeNavigator model
                         * @param {string}  argument.type returns the name of the event
                         * @example 
                         * //loaded event for chart
                         * $("#container").ejRangeNavigator({
                         *    loaded: function (args) {}
                         * });
                         * @memberof ejRangeNavigator
                         * @instance
                         */
            loaded: "",
            /**     
                        * Fires on load of RangeNavigator.
                        * @event
                        * @name ejRangeNavigator#load	
                        * @param {Object} argument.Data parameters from RangeNavigator     
                        * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
                        * @param {object}  argument.model returns the RangeNavigator model
                        * @param {string}  argument.type returns the name of the event
                        * @example 
                        * //load event for chart
                        * $("#container").ejRangeNavigator({
                        *    load: function (args) {}
                        * });
                        * @memberof ejRangeNavigator
                        * @instance
                        */
            load: "",
            /**     
                       * Fires on changing the range of RangeNavigator.
                       * @event
                       * @name ejRangeNavigator#rangeChanged	
                       * @param {Object} argument.Data parameters from RangeNavigator     
                       * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
                       * @param {object}  argument.model returns the RangeNavigator model
                       * @param {string}  argument.type returns the name of the event
                       * @example 
                       * //rangeChanged event for chart
                       * $("#container").ejRangeNavigator({
                       *    rangeChanged: function (args) {}
                       * });
                       * @memberof ejRangeNavigator
                       * @instance
                       */
            rangeChanged: ""

        },
        /**
          * Rangenavigator contains the default values. dataTypes property is used to set the default value to rangenavigator
          * @private
          */
        dataTypes: {
            //  theme:"string",
            // padding:"string",
            canResize: "boolean",
            snappingMode: "boolean",
            //  size:{
            //      width:"string",
            //      height:"string"
            //  },
            dataSource: {
                data: "data",
                query: "data"
            },
            tooltip: {
                visible: "boolean",
                labelFormat: "string",
                tooltipDisplayMode: "string",
            },
            zoomCordinates: {
                zoomPosition: "string",
                zoomFactor: "string"
            },
            selectedData: "string",
            deferredUpdate: "boolean",
            series: "array",
            //seriesSettings: "string",
            labelSettings: {
                higherLevel: {
                    intervalType: 'string',
                    position: "string",
                    visible: "boolean",
                    labelPlacement: "string"
                },
                lowerLevel: {
                    intervalType: 'string',
                    position: "string",
                    visible: "boolean",
                    labelPlacement: "string"
                }
            },
            navigatorStyle: {
                thumbRadius: "number",
            },
            //	loaded: "string",
            //rangeChanged: "string" 

        },
        observables: ["dataSource.data"],

        _themes: {
            flatlight: {
                tooltip: {
                    backgroundColor: "#303030",
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            azurelight: {
                tooltip: {
                    backgroundColor: "#303030",
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            limelight: {
                tooltip: {
                    backgroundColor: "#303030",
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#A9CA44 ",
                    thumbColor: "#AECF49",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            saffronlight: {
                tooltip: {
                    backgroundColor: "#303030",
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#FAA113",
                    thumbColor: "#F9920B",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientlight: {
                tooltip: {
                    backgroundColor: "#303030",
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientazure: {
                tooltip: {
                    backgroundColor: "#303030",
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientlime: {
                tooltip: {
                    backgroundColor: "#303030",
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientsaffron: {
                tooltip: {
                    backgroundColor: "#303030",
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            flatdark: {
                tooltip: {
                    backgroundColor: "#FFFFFF",
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            azuredark: {
                tooltip: {
                    backgroundColor: "#FFFFFF",
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            limedark: {
                tooltip: {
                    backgroundColor: "#FFFFFF",
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#A9CA44",
                    thumbColor: "#AECF49",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            saffrondark: {
                tooltip: {
                    backgroundColor: "#FFFFFF",
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#FAA113",
                    thumbColor: "#F9920B",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientdark: {
                tooltip: {
                    backgroundColor: "#FFFFFF",
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientazuredark: {
                tooltip: {
                    backgroundColor: "#FFFFFF",
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientlimedark: {
                tooltip: {
                    backgroundColor: "#FFFFFF",
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientsaffrondark: {
                tooltip: {
                    backgroundColor: "#FFFFFF",
                    labelstyles: { font: { color: '#1E1E1E', fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '10px', opacity: 1, fontWeight: 'regular' } },
                },
                labelSettings: {
                    labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                    higherLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, strokedasharray: "15 5 0" },
                    },
                    lowerLevel: {
                        labelstyles: { font: { color: '#FFFFFF', fontFamily: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, fontWeight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyle: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

        },
        /**
                * To configure the properties at runtime using SetModel		
                * @private
                */
        _setModel: function (options) {
            for (var prop in options) {
                //  this.disableAnimation();
                switch (prop) {
                    case "theme":
                        this.model._themeChanged = true;
                        this.model.theme = options[prop];
                        this._setTheme(this._themes, this.model.theme);
                    default:
                        $.extend(true, this.model, {}, options[prop]);
                }
            }
            //  $(this.svgDocument).empty();		  
            this.bindTo();
            this.rendernavigator(this);
            this._bindevents();
        },
        /**
        * destroy the RangeNavigator widget
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Destroy RangeNavigator
		* var obj = $("#container").data("ejRangeNavigator");
		* obj.destroy(); // destroy the RangeNavigator
		* &lt;/script&gt;
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#container").ejRangeNavigator("destroy");	
		* &lt;/script&gt;
		* @memberof ejRangeNavigator
		* @instance
         */
        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },


        _createSvg: function (element) {

            this.svgSupport = (window.SVGSVGElement) ? true : false;
            var bwr = jQuery.uaMatch(navigator.userAgent);
            this.brow = bwr.browser.toLowerCase();
            if (this.svgSupport) {
                //rendering object created
                this.renderer = new ej.EjSvgRender(this.element);
                //Creating svg document
                this.svgns = this.renderer.svgLink;
                this.svgDocument = this.renderer.svgObj;
            }
            else {
                //rendering object created
                this.renderer = new ej.EjVmlRender(this.element);
                //Creating VML document
                this.svgDocument = this.renderer.svgObj;
            }

        },

        _setTheme: function (jsonObj, themeName) {
            var name = themeName.toLowerCase();
            this.model = ej.copyObject({}, this.model, jsonObj[name]);
        },

        _setPositions: function () {

            var higherposition = this.model.labelSettings.higherLevel.position;
            var lowerposition = this.model.labelSettings.lowerLevel.position;
            var highervisible = this.model.labelSettings.higherLevel.visible;
            var lowervisible = this.model.labelSettings.lowerLevel.visible;
            var higherlabelplace = this.model.labelSettings.higherLevel.labelPlacement;
            var lowerlabelplace = this.model.labelSettings.lowerLevel.labelPlacement;
            //to initialize the positions of the sliders, labelbars and chart
            if (higherposition === "top" && lowerposition === "bottom") {
                this.centerPosition = this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minHighHeight + this.centerHeight + 1;
            }
            else if ((higherposition === "bottom" && lowerposition === "bottom")) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minLowHeight + this.centerHeight + 1;

            }
            else if (higherposition === "bottom" && lowerposition === "top") {
                this.centerPosition = this.minLowHeight + 1;
                topPosition = 0;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minLowHeight + this.centerHeight + 1;
            }
            else if (higherposition === "top" && lowerposition === "top") {
                this.centerPosition = this.minLowHeight + this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.newHeight;
            }

            if (highervisible === false && lowervisible === false) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight;
                this.bottomPostion = this.newHeight;
            }
            else if (highervisible === false) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight - this.minLowHeight;
                this.bottomPosition = this.newHeight;
            }
            else if (lowervisible === false) {
                this.centerPosition = this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight;
                this.bottomPosition = this.minHighHeight + this.centerHeight + 1;
            }

            if (higherlabelplace === "outside" && lowerlabelplace === "outside") {
                this.sliderPosition = this.centerPosition;
                this.sliderHeight = this.centerHeight;
            }
            else if (higherlabelplace === "outside") {
                this.sliderPosition = this.centerPosition;
                this.sliderHeight = this.newHeight;
            }
            else if (lowerlabelplace === "outside") {
                this.sliderPosition = 0;
                this.sliderHeight = this.centerHeight + this.minHighHeight;
            }
            else {
                this.sliderPosition = 0;
                this.sliderHeight = this.newHeight;
            }

        },

        rendernavigator: function () {
            this.culture = Globalize.culture(this.model.localization);
            this._initializeVariables.call(this, this.model.dataSource, this.model.series, this.model.seriesSettings);

            this._rootId = $(this.element).attr("id");
            this.leftPadding = parseFloat($("#" + this._rootId).css('padding-left'));

            //initializing minimum height and width for the control
            var fontsize = parseFloat(this.model.labelSettings.higherLevel.labelstyles.font.size);
            var type = this.model.valueType;
            if (type == "datetime") {
                this.minHighHeight = fontsize < 20 ?
                    20 :
                    fontsize + 2;
                fontsize = parseFloat(this.model.labelSettings.lowerLevel.labelstyles.font.size);
            }
            else
                this.minHighHeight = 0;
            this.minLowHeight = fontsize < 20 ?
                20 :
                fontsize + 2;
            if (type == "datetime") {
                var ht = ($(this.element).height() && this.newHeight == undefined) ? $(this.element).height() : 120;
                this.minCenterHeight = parseFloat(ht) - parseFloat(this.minLowHeight) - parseFloat(this.minHighHeight);
            } else {
                var ht = ($(this.element).height()) && this.newHeight == undefined ? $(this.element).height() - this.minLowHeight : 100;
                this.minCenterHeight = ht;
            }
            this.minHeight = this.minHighHeight + this.minCenterHeight + this.minLowHeight;
            this.padding = parseFloat(this.model.padding);


            //calculating the height and width of the control
            this.newWidth = this.model.size.width ? this.model.size.width : $(this.element).width();
            this.newHeight = this.model.size.height ? this.model.size.height : ($(this.element).height() && this.newHeight == undefined) > this.minHeight ? $(this.element).height() : this.minHeight;


            //initializing the higherlevel and lowerlevel collections
            this._higherTextNode = [];
            this._higherTotalValues = [];
            this._higherLineLeft = [];
            this._higherTextLeft = [];
            this._lowerTextNode = [];
            this._lowerTotalValues = [];
            this._lowerLineLeft = [];
            this._lowerTextLeft = [];

            //calculating positions
            this.centerPosition = 0;
            this.bottomPosition = 0;
            this.centerHeight = 0;
            this.sliderPosition = 0;
            this.sliderHeight = 0;

            //set positions
            this._setPositions();

            //set initial width and height for the svg document
            $(this.svgDocument).empty();
            this.svgDocument.setAttribute("position", "relative");
            this.svgDocument.setAttribute("overflow", "visible");
            if (this.svgSupport) {
                this.svgDocument.setAttribute("height", this.newHeight);
                this.svgDocument.setAttribute("width", this.newWidth);
            }
            else {
                $(this.svgDocument).width(this.newWidth);
                $(this.svgDocument).height(this.newHeight);
            }
            this.model._size.width = this.newWidth;
            this.model._size.height = this.newHeight;
            var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLoadedEventArgs.Data = { Model: this.model };
            this._trigger("loaded", commonLoadedEventArgs);
            if (this.model.theme != "")
                this._setTheme(this._themes, this.model.theme);
            //create gradient
            var uncolor = this.model.navigatorStyle.unselectedRegionColor;
            this.unselectgrad = this.renderer.createGradientElement('unselected', uncolor, 150, 0, 150, 100, this.svgDocument);
            var navibackcolor = this.model.navigatorStyle.background;
            this.naviback = this.renderer.createGradientElement('naviback', navibackcolor, 150, 0, 150, 100, this.svgDocument);
            var selcolor = this.model.navigatorStyle.selectedRegionColor;
            this.selectedgrad = this.renderer.createGradientElement('selected', selcolor, 150, 0, 150, 100, this.svgDocument);
            var tbColor = this.model.navigatorStyle.thumbColor;
            this.tbClr = this.renderer.createGradientElement('thumbClr', tbColor, 150, 0, 150, 100, this.svgDocument);

            var areaHeight;
            if (type == "numeric")
                areaHeight = this.newHeight - this.model.navigatorStyle.border.width - this.minLowHeight;
            else
                areaHeight = this.newHeight - this.model.navigatorStyle.border.width - this.minLowHeight - this.minHighHeight;
            //Creating border for navigator
            var naviBorderOptions = {
                'id': this._rootId + '_Border',
                'x': this.padding,
                'y': this.sliderPosition,
                'width': this.newWidth - this.padding * 2,
                'height': this.sliderHeight,
                'fill': this.naviback,
                'stroke-width': this.model.navigatorStyle.border.width,
                'stroke': this.model.navigatorStyle.border.color,
                'opacity': this.model.navigatorStyle.opacity,
                'stroke-dasharray': this.model.navigatorStyle.border.strokedasharray,

            }

            this.renderer.drawRect(naviBorderOptions, this.svgDocument);

            //creating g tags to group higherlevel and lowerlevel elements(txt,line).
            this.higherLevel = this.renderer.createGroup({ 'id': this._rootId + 'higherLevel' });
            this.lowerLevel = this.renderer.createGroup({ 'id': this._rootId + 'lowerLevel' });
            this.centerLevel = this.renderer.createGroup({ 'id': this._rootId + 'centerLelvel' });
            this.higherLevel.height = this.minHighHeight;
            this.lowerLevel.height = this.minLowHeight;

            //creating g tags to group tooltip
            if (this.model.tooltip.visible && this.svgSupport) {
                this.leftTooltip = this.renderer.createGroup({ 'id': this._rootId + 'leftTooltip' });
                this.rightTooltip = this.renderer.createGroup({ 'id': this._rootId + 'rightTooltip' });

                var leftToolOptions = {
                    'fill': this.model.tooltip.backgroundColor,
                    'd': 'M 0 0 L 0 21 L 64 21 L 64 6 L 70 0 Z',
                };
                this.renderer.drawPath(leftToolOptions, this.leftTooltip);
                var tooltipoptions = this.model.tooltip.labelstyles.font;


                var textOptions = {
                    'id': this._rootId + '_LeftToolText',
                    'x': 5,
                    'fill': tooltipoptions.color,
                    'font-size': tooltipoptions.size,
                    'font-family': tooltipoptions.fontFamily,
                    'font-style': tooltipoptions.fontStyle,
                    'font-weight': tooltipoptions.fontWeight,
                    'text-anchor': 'start',
                    'opacity': tooltipoptions.opacity,
                    'dominant-baseline': 'middle',


                };
                this.renderer.drawText(textOptions, Globalize.format(new Date(1 / 1 / 2000), "MMMM, yyyy"), this.leftTooltip);
                this.leftTxt = this.leftTooltip.childNodes[1];
                var rightToolOptions = {
                    'fill': this.model.tooltip.backgroundColor,
                    'd': 'M 0 0 L 70 0 L 70 21 L 6 21 L 6 6 Z',
                    'horizontal-alignment': 'stretch'
                };
                this.renderer.drawPath(rightToolOptions, this.rightTooltip);
                this.renderer.drawText(textOptions, Globalize.format(new Date(1 / 1 / 2000), "MMMM, yyyy"), this.rightTooltip);
                this.rightTxt = this.rightTooltip.childNodes[1];

            }
            //creating g tag for holding chart
            this.chartGView = this.renderer.createGroup({ 'id': this._rootId + 'charView' });

            this.renderer.append(this.chartGView, this.svgDocument);
            if (this.model.valueType == "datetime")
                this.renderer.append(this.higherLevel, this.svgDocument);
            this.renderer.append(this.lowerLevel, this.svgDocument);
            this.renderer.append(this.centerLevel, this.svgDocument);

            //creating g tag for sliders
            this.leftUnSelected = this.renderer.createGroup({ 'id': 'unselectleft' });
            this.gLeftSlider = this.renderer.createGroup({ 'id': 'leftslider' });
            this.gRightSlider = this.renderer.createGroup({ 'id': 'rightslider' });
            this.gCenterSlider = this.renderer.createGroup({ 'id': 'centerslider' });
            this.rightUnSelected = this.renderer.createGroup({ 'id': 'unselectright' });

            var unLeftSliderOptions = {
                'width': parseFloat(this.newWidth - 4 - this.padding),
                'height': this.sliderHeight,
                'fill': this.unselectgrad,
                'opacity': '0.3',
                'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')'
            };
            this.renderer.drawRect(unLeftSliderOptions, this.leftUnSelected);
            this.leftUnArea = this.leftUnSelected.firstChild;
            this.renderer.append(this.leftUnSelected, this.svgDocument);

            var centerSliderOptions = {
                'width': parseFloat(this.newWidth - 4),
                'height': this.sliderHeight,
                'fill': this.selectedgrad,
                'opacity': '0',
                'transform': 'translate(2,' + this.sliderPosition + ')'
            };

            this.renderer.drawRect(centerSliderOptions, this.gCenterSlider);
            this.centerSlider = this.gCenterSlider.firstChild;
            $(this.centerSlider).css('cursor', 'pointer');

            this.renderer.append(this.gCenterSlider, this.svgDocument);
            var leftSliderOptions = {
                'width': '2',
                'height': this.sliderHeight,
                'fill': this.model.navigatorStyle.thumbStroke,
                'opacity': '1',
                'y': this.sliderPosition
            };
            var lcircleOptions = {
                'cx': 0,
                'cy': this.sliderPosition + (this.sliderHeight / 2),
                'r': this.model.navigatorStyle.thumbRadius,
                'stroke-width': 2,
                'stroke': this.model.navigatorStyle.thumbStroke,
                'fill': this.tbClr
            }


            this.renderer.drawRect(leftSliderOptions, this.gLeftSlider);
            this.renderer.drawCircle(lcircleOptions, this.gLeftSlider);
            this.leftCircle = this.gLeftSlider.lastChild;
            $(this.leftCircle).css('cursor', 'pointer');
            this.leftSlider = this.gLeftSlider.firstChild;
            $(this.leftSlider).css('cursor', 'w-resize');

            this.renderer.append(this.gLeftSlider, this.svgDocument);

            var unRightSliderOptions = {
                'width': parseFloat(this.newWidth - 4 - this.padding),
                'height': this.sliderHeight,
                'fill': this.unselectgrad,
                'opacity': '0.3',
                'transform': 'translate(0,' + this.sliderPosition + ')'
            };
            this.renderer.drawRect(unRightSliderOptions, this.rightUnSelected);
            this.rightUnArea = this.rightUnSelected.firstChild;
            this.renderer.append(this.rightUnSelected, this.svgDocument);
            var rightSliderOptions = {
                'width': '2',
                'height': this.sliderHeight,
                'fill': this.model.navigatorStyle.thumbStroke,
                'opacity': '1',
                'transform': 'translate(' + parseFloat(this.newWidth - 4) + ', ' + this.sliderPosition + ' )'
            };
            var rcircleOptions = {
                'cx': 0,
                'cy': this.sliderPosition + (this.sliderHeight / 2),
                'r': this.model.navigatorStyle.thumbRadius,
                'stroke-width': 2,
                'stroke': this.model.navigatorStyle.thumbStroke,
                'fill': this.tbClr
            }
            this.renderer.drawRect(rightSliderOptions, this.gRightSlider);
            this.renderer.drawCircle(rcircleOptions, this.gRightSlider);
            this.rightCircle = this.gRightSlider.lastChild;
            $(this.rightCircle).css('cursor', 'pointer');
            this.rightSlider = this.gRightSlider.firstChild;
            $(this.rightSlider).css('cursor', 'w-resize');

            this.renderer.append(this.gRightSlider, this.svgDocument);


            this.renderer.append(this.leftTooltip, this.svgDocument);
            this.renderer.append(this.rightTooltip, this.svgDocument);

            this.element.append(this.svgDocument);
            if (this.svgSupport) {
                this.trueCoords = this.svgDocument.createSVGPoint();

                this.grabPoint = this.svgDocument.createSVGPoint();
            }

            this.backDrop = this.element;
            this.dragTarget = null;

            this._renderChart.call(this, this.model);

            if ((this.startDateTime != undefined && this.endDateTime != undefined) || (this.startValue != undefined && this.endValue != undefined))
                this.calculateInterval.call(this, this.model.labelSettings.higherLevel
                    , this.model.labelSettings.lowerLevel);

            this.setSliders();

            //this._trigger("loaded", this.model);
        },

        eachInterval: "",

        //Rendering chart
        _renderChart: function (model) {
            if (model.dataSource.data != "" && model.dataSource.xName != "" && model.dataSource.yName != "" && model.series == "" && model.seriesSettings == "") {
                $(this.chartGView).ejChart(
                {
                    localization: this.model.localization,
                    border: { width: 0, color: "transparent" },
                    margin: { left: 0, right: 0, top: 0, bottom: 0 },
                    elementSpacing: 0,
                    chartArea:
                    {
                        background: "transparent",
                        border: { width: 0 }
                    },
                    primaryXAxis: {
                        axisLine: { visible: false },
                        font: { size: '0px', },
                        majorTickLines:
                        {
                            lineWidth: 0,
                            size: 0,
                            visible: true
                        },
                        majorGridLines: { visible: false },
                        rangePadding: this.model.rangePadding,
                        visible: false,
                        isInversed: this.model.isRTL

                    },
                    primaryYAxis: this.model.valueAxisSettings,
                    series: [{
                        dataSource: { data: model.dataSource.data, xName: model.dataSource.xName, yName: [model.dataSource.yName] },
                        type: 'line',
                        width: 1.5

                    },
                    ],
                    size: { height: parseFloat(this.sliderHeight), width: parseFloat(this.newWidth - (2 * this.padding)), padding: -10 },
                    legend: {
                        visible: false,
                        itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                    }
                });
            }
            else if (model.dataSource.data != "" && model.dataSource.xName != "" && model.dataSource.yName != "" && model.series != "" && model.series[0].dataSource == undefined || model.seriesSettings != "" && model.seriesSettings.dataSource == undefined) {
                model.seriesSettings.dataSource = model.dataSource;
                $(this.chartGView).ejChart(
            {
                localization: this.model.localization,
                border: { width: 0, color: "transparent" },
                margin: { left: 0, right: 0, top: 0, bottom: 0 },
                elementSpacing: 0,
                chartArea:
                {
                    background: "transparent",
                    border: { width: 0 }
                },
                primaryXAxis: {

                    axisLine: { visible: false },
                    font: { size: '0px', },
                    majorTickLines:
                    {
                        lineWidth: 0, size: 0, visible: true
                    },
                    majorGridLines: { visible: false },
                    rangePadding: this.model.rangePadding,
                    visible: false,
                    isInversed: this.model.isRTL

                },
                primaryYAxis: this.model.valueAxisSettings,
                series: model.series,
                commonSeriesOptions: model.seriesSettings,
                size: { height: parseFloat(this.sliderHeight), width: parseFloat(this.newWidth - (2 * this.padding)), padding: -10 },
                legend: {
                    visible: false,
                    itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                }
            });
            }
            else if (model.series != "" && model.series[0].dataSource != undefined || model.seriesSettings != "" && model.seriesSettings.dataSource != undefined) {
                if (model.seriesSettings == "") {
                    model.seriesSettings = {
                        type: 'line'
                    };
                }
                $(this.chartGView).ejChart(
            {
                localization: this.model.localization,
                border: { width: 0, color: "transparent" },
                margin: { left: 0, right: 0, top: 0, bottom: 0 },
                elementSpacing: 0,
                chartArea:
                {
                    background: "transparent",
                    border: { width: 0 }
                },
                primaryXAxis: {

                    axisLine: { visible: false },
                    font: { size: '0px', },
                    majorTickLines:
                    {
                        lineWidth: 0, size: 0, visible: true
                    },
                    majorGridLines: { visible: false },
                    rangePadding: this.model.rangePadding,
                    visible: false,
                    isInversed: this.model.isRTL

                },
                primaryYAxis: this.model.valueAxisSettings,
                series: model.series,
                commonSeriesOptions: model.seriesSettings,
                size: { height: parseFloat(this.sliderHeight), width: parseFloat(this.newWidth - (2 * this.padding)), padding: -10 },
                legend: {
                    visible: false,
                    itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                }
            });
            }

            this.ejChart = $(this.chartGView).data("ejChart");
            this.renderer._setAttr($(this.chartGView), { 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')', 'height': this.sliderHeight });

        },

        _init: function () {
            this._createSvg(this);
            var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLoadedEventArgs.Data = { Model: this.model };
            this._trigger("load", commonLoadedEventArgs);
            this.rendernavigator(this);
            if (this.model.canResize) {
                this._on($(window), "resize", this.rendernavigator);
            }
            this._bindevents();
        },

        isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },

        _bindevents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (window.navigator.msPointerEnabled && !isIE11) {
                this._on($(this.svgDocument), "MSPointerDown", this._grab);
                this._on($(this.svgDocument), "MSPointerMove", this._drag);
                this._on($(this.svgDocument), "MSPointerUp", this._drop);
                this._on($(window), "MSPointerUp", this.mouseup);
                $(this.svgDocument).css('-ms-touch-action', 'none');
            }
            else if (this.isDevice()) {
                this._on($(this.svgDocument), "touchstart", this._grab);
                this._on($(this.svgDocument), "touchmove", this._drag);
                this._on($(this.svgDocument), "touchend", this._drop);
            }
            else if (matched.browser.toLowerCase() == "chrome") {
                this._on($(this.svgDocument), "touchstart", this._grab);
                this._on($(this.svgDocument), "touchmove", this._drag);
                this._on($(this.svgDocument), "touchend", this._drop);
                this._on($(this.svgDocument), "mousedown", this._grab);
                this._on($(this.svgDocument), "mousemove", this._drag);
                this._on($(this.svgDocument), "mouseup", this._drop);
            }
            else {
                this._on($(this.svgDocument), "mousedown", this._grab);
                this._on($(this.svgDocument), "mousemove", this._drag);
                this._on($(this.svgDocument), "mouseup", this._drop);
            }
            if (isIE11)
                $(this.svgDocument).css('touch-action', 'none');

            this._on($(window), "mouseup", this.mouseup);
        },

        _initializeVariables: function (dataSource, series, sSettings) {
            //intializing ismouseup flag
            this.ismouseup = false;
            var labelStyle = this.model.labelSettings.labelstyles;
            var lowLabelStyle = this.model.labelSettings.lowerLevel.labelstyles;
            var highLabelStyle = this.model.labelSettings.higherLevel.labelstyles;
            //checking labelstyle
            if (highLabelStyle == "")
                this.model.labelSettings.higherLevel.labelstyles = labelStyle;
            if (lowLabelStyle == "") {
                this.model.labelSettings.lowerLevel.labelstyles = labelStyle;
            }
            else if (lowLabelStyle != undefined && lowLabelStyle.horizontalAlignment == undefined) {
                this.model.labelSettings.lowerLevel.labelstyles.horizontalAlignment = "center";
            }
            if (this.model.valueType == "numeric" && this.model.tooltip.labelFormat == "MM/dd/yyyy") {
                this.model.tooltip.labelFormat = "d";
            }
            //checking for datasource and finding start and end values
            if (dataSource.data != "" && dataSource.xName != "" && dataSource.yName != "") {
                this._processJsonData(dataSource.data, this.model);
            }
            else if (series != "" && series != undefined && series[0].dataSource != undefined && series[0].dataSource.xName != undefined && series[0].dataSource.yName != undefined) {
                this._processJsonData(series[0].dataSource.data, series[0]);
            }
            else if (sSettings.dataSource != undefined && sSettings.dataSource.xName != undefined && sSettings.dataSource.yName != undefined) {
                this._processJsonData(sSettings.dataSource.data, sSettings);
            }
            else if (this.model.range.start != "" && this.model.range.end != "") {
                if (this.model.valueType == "datetime") {
                    this.startDateTime = new Date(this.model.range.start);
                    this.endDateTime = new Date(this.model.range.end);
                }
                else {
                    this.startValue = this.model.range.start;
                    this.endValue = this.model.range.end;
                }
            }
            // initializing start year and end year
            if (this.startDateTime != undefined && this.endDateTime != undefined) {
                if (this.model.valueType == "datetime" && this.model.rangePadding == "round") {
                    var start = new Date(this.startValue);
                    var end = new Date(this.endValue);
                    this.startDateTime = new Date(start.getFullYear(), start.getMonth(), start.getDate(), 0, 0, 0);
                    this.endDateTime = new Date(end.getFullYear(), end.getMonth(), end.getDate(), 23, 59, 59);
                }
                this.startYear = this.startDateTime.getFullYear();
                this.startMonth = this.startDateTime.getMonth();
                this.endYear = this.endDateTime.getFullYear();
                this.endMonth = this.endDateTime.getMonth();
            }
        },

        setSliders: function () {
            var wholeSize = this.newWidth - this.padding * 2;
            var type = this.model.valueType;
            var selectedRangeStart = this.model.selectedRange.start;
            var selectedRangeEnd = this.model.selectedRange.end;
            // var leftSliderPosition, rightSliderPosition;
            if (type == "datetime")
                this.eachInterval = wholeSize / parseFloat(this.endDateTime.getTime() - this.startDateTime.getTime());
            else
                this.eachInterval = wholeSize / parseFloat(this.endValue - this.startValue);
            if (selectedRangeStart === "" && selectedRangeEnd === "") {
                if (type == "datetime") {
                    selectedRangeStart = this.startDateTime;
                    selectedRangeEnd = this.endDateTime;
                }
                else {
                    selectedRangeStart = this.startValue;
                    selectedRangeEnd = this.endValue;
                }

            }

            if (type == "datetime") {
                this.leftSliderPosition = (new Date(selectedRangeStart).getTime() - this.startDateTime.getTime()) * this.eachInterval + this.padding;
                this.rightSliderPosition = (new Date(selectedRangeEnd).getTime() - this.startDateTime.getTime()) * this.eachInterval + this.padding;
            }
            else {
                this.leftSliderPosition = (selectedRangeStart - this.startValue) * this.eachInterval + this.padding;
                this.rightSliderPosition = (selectedRangeEnd - this.startValue) * this.eachInterval + this.padding;
            }
            if (this.model.tooltip.tooltipDisplayMode == "always")
                this.setopacity = 1;
            else
                this.setopacity = 0;
            this.setSliderPositions(this.leftSliderPosition, null, null);
            this.setSliderPositions(null, null, this.rightSliderPosition);
            this.setopacity = 1;
            if (this.model.deferredUpdate) {
                this._calculateSelectedData();
                if (this.startDateTime !== new Date(selectedRangeStart) && this.endDateTime !== new Date(selectedRangeEnd)) {
                    this._trigger("rangeChanged", this.model);
                }
            }

        },

        centerSliderWidth: "",
        zoomp: 0,
        zoomf: 1,

        setSliderPositions: function (leftValue, centerValue, rightValue) {
            var doubledate;
            var type = this.model.valueType;
            var cPosition;
            var labelPlacement = this.model.labelSettings.higherLevel.labelPlacement;
            cPosition = labelPlacement == "inside" ? 0 : this.centerPosition;
            var rtlValue;
            if (leftValue != null && leftValue >= this.padding) {
                this.leftSliderPosition = leftValue;
                rtlValue = this.model.isRTL ? (this.newWidth - leftValue) : leftValue;
                if (type == "datetime")
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                if (this.model.tooltip.visible && this.svgSupport) {
                    this.leftTooltip.setAttribute('opacity', this.setopacity);
                    if (type == "datetime")
                        this.leftTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltip.labelFormat);
                    else
                        this.leftTxt.textContent = Globalize.format(doubledate, this.model.tooltip.labelFormat);
                    var txtwidth = this.leftTooltip.lastChild.getBBox().width + 15;
                    var txtheight = this.leftTooltip.lastChild.getBBox().height;
                    var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                    this.leftTxt.setAttribute('y', txty);
                    this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                    this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";

                    if (leftValue - txtwidth >= 0) {
                        this.leftTooltip.firstChild.setAttribute('d', this.leftd);
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue - txtwidth) + ',' + cPosition + ')' });
                        this.leftTxt.setAttribute('x', 5);
                    }
                    else if (leftValue + txtwidth < this.rightTooltip.getCTM().e) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue) + ',' + cPosition + ')' });
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                        this.leftTxt.setAttribute('x', 10);
                    }
                    else if (this.rightTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue) + ',' + (cPosition + 30) + ')' });
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                    }
                    else {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue) + ',' + (cPosition) + ')' });
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                    }
                    if (leftValue > this.newWidth - this.rightTooltip.lastChild.getBBox().width - 15 && this.rightTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue - txtwidth) + ',' + (cPosition + 30) + ')' });
                    }
                    if (this.rightTooltip.getCTM().f >= 30 && leftValue < this.rightTooltip.getCTM().e) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + this.rightTooltip.getCTM().e + ',' + (cPosition) + ')' });
                    }
                }
                if (this.ismouseup == false) {
                    // this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding});
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    if (!this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + this.sliderPosition + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + 0 + ')' });
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftValue });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftValue - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                    }
                    this.renderer._setAttr($(this.centerSlider), { 'transform': 'translate(' + parseFloat(leftValue) + ',' + this.sliderPosition + ')' });
                    var width = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                    this.renderer._setAttr($(this.centerSlider), { 'width': width });
                }
                else {
                    this.ismouseup = false;
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    if (!this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + this.sliderPosition + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + 0 + ')' });
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftValue });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftValue - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                    }
                    this.renderer._setAttr($(this.centerSlider), { 'transform': 'translate(' + parseFloat(leftValue) + ',' + this.sliderPosition + ')' });
                    var width = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                    this.renderer._setAttr($(this.centerSlider), { 'width': width });
                }
                this.model.zoomCordinates.zoomPosition = (leftValue - this.padding) / (this.newWidth - this.padding * 2);
                this.model.zoomCordinates.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
                var rightposX = this.rightSlider.getBoundingClientRect().left;
                if (type == "datetime") {
                    this.model.selectedRange.start = new Date(doubledate);
                    this.model.selectedRange.end = new Date((rightposX / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval));
                }
                else {
                    this.model.selectedRange.start = doubledate;
                    this.model.selectedRange.end = (rightposX / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                }
            }
            else if (rightValue != null && rightValue <= this.newWidth - this.padding + 1) {
                this.rightSliderPosition = rightValue;
                var startDateTime = new Date(this.model.range.start);
                rtlValue = this.model.isRTL ? (this.newWidth - rightValue) : rightValue;
                if (type == "datetime")
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                this.model.selectedRange.end = new Date(doubledate);

                if (this.model.tooltip.visible && this.svgSupport) {
                    this.rightTooltip.setAttribute('opacity', this.setopacity);
                    if (type == "datetime")
                        this.rightTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltip.labelFormat);
                    else
                        this.rightTxt.textContent = Globalize.format(doubledate, this.model.tooltip.labelFormat);
                    var txtwidth = this.rightTooltip.lastChild.getBBox().width + 15;
                    var txtheight = this.rightTooltip.lastChild.getBBox().height;
                    var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                    this.rightTxt.setAttribute('y', txty);
                    this.rightTxt.setAttribute('x', 10);
                    this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                    this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                    if (rightValue + txtwidth <= this.newWidth) {
                        this.rightTooltip.firstChild.setAttribute('d', this.rightd);
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue) + ',' + cPosition + ')' });
                    }
                    else if (rightValue + txtwidth >= this.newWidth && rightValue - txtwidth > this.centerSlider.getCTM().e) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + cPosition + ')' });
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                        this.rightTxt.setAttribute('x', 5);
                    }
                    else if (this.leftTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + (cPosition + 30) + ')' });
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                    }
                    else {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + cPosition + ')' });
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                    }
                    if (rightValue < this.leftTooltip.getCTM().e + txtwidth && this.leftTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue) + ',' + (cPosition + 30) + ')' });
                    }
                    if (this.leftTooltip.getCTM().f >= 30 && rightValue > this.leftTooltip.getCTM().e + txtwidth && this.rightTooltip.getCTM().e > this.leftTooltip.getCTM().e + txtwidth) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + this.leftTooltip.getCTM().e + ',' + (cPosition) + ')' });
                        this.rightTxt.setAttribute('x', 5);
                    }
                }

                this.renderer._setAttr($(this.rightUnArea), { 'width': this.newWidth - rightValue - this.padding - 2, 'transform': 'translate(' + parseFloat(rightValue + 2) + ',' + this.sliderPosition + ')' });
                this.renderer._setAttr($(this.rightSlider), { 'transform': 'translate(' + rightValue + ',' + this.sliderPosition + ')' });
                if (this.rightCircle != undefined) {
                    if (this.svgSupport)
                        this.renderer._setAttr($(this.rightCircle), { 'cx': rightValue });
                    else
                        this.renderer._setAttr($(this.rightCircle), { 'transform': 'translate(' + parseFloat(rightValue - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                }
                this.centerSliderWidth = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                this.renderer._setAttr($(this.centerSlider), { 'width': this.centerSliderWidth });

                //  this.model.zoomCordinates.zoomFactor = (rightValue - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
                this.model.zoomCordinates.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
            }
            else if (centerValue != null) {
                var rectwidth = this.renderer._getAttrVal($(this.leftSlider), 'width');
                var leftPosX = centerValue;

                var rightPosX = centerValue + parseFloat(this.renderer._getAttrVal($(this.centerSlider), 'width'));
                var startDateTime = new Date(this.model.range.start);
                rtlValue = this.model.isRTL ? (this.newWidth - leftPosX) : leftPosX;
                if (type == "datetime")
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                this.model.selectedRange.start = new Date(doubledate);
                if (leftPosX >= this.padding && rightPosX <= this.newWidth - this.padding) {
                    if (this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftPosX + ',' + 0 + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftPosX + ',' + parseFloat(this.sliderPosition) + ')' });
                    this.leftSliderPosition = leftPosX;
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftPosX });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftPosX - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' });
                    }
                    if (this.model.tooltip.visible && this.svgSupport) {
                        this.leftTooltip.setAttribute('opacity', this.setopacity);
                        if (type == "datetime")
                            this.leftTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltip.labelFormat);
                        else
                            this.leftTxt.textContent = Globalize.format(doubledate, this.model.tooltip.labelFormat);
                        var txtwidth = this.leftTooltip.lastChild.getBBox().width + 15;
                        var txtheight = this.leftTooltip.lastChild.getBBox().height;
                        var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                        this.leftTxt.setAttribute('y', txty);
                        this.rightTxt.setAttribute('y', txty);
                        this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                        this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                        if (leftPosX - txtwidth >= 0) {
                            this.leftTooltip.firstChild.setAttribute('d', this.leftd);
                            this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX - txtwidth) + ',' + cPosition + ')' });
                            this.leftTxt.setAttribute('x', 5);
                        }
                        else {
                            this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX) + ',' + cPosition + ')' });
                            this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                            this.leftTxt.setAttribute('x', 10);
                            if (this.leftSliderPosition + txtwidth > this.rightSlider.getCTM().e) {
                                this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX) + ',' + (cPosition + 30) + ')' });
                            }

                        }
                    }
                    rtlValue = this.model.isRTL ? (this.newWidth - rightPosX) : rightPosX;
                    var doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                    this.model.selectedRange.end = new Date(doubledate);
                    this.renderer._setAttr($(this.rightSlider), { 'transform': 'translate(' + rightPosX + ',' + this.sliderPosition + ')' });
                    this.rightSliderPosition = rightPosX;
                    if (this.rightCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.rightCircle), { 'cx': rightPosX });
                        else
                            this.renderer._setAttr($(this.rightCircle), { 'transform': 'translate(' + parseFloat(rightPosX - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' });
                    }
                    if (this.model.tooltip.visible && this.svgSupport) {
                        this.rightTooltip.setAttribute('opacity', this.setopacity);
                        if (type == "datetime")
                            this.rightTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltip.labelFormat);
                        else
                            this.rightTxt.textContent = Globalize.format(doubledate, this.model.tooltip.labelFormat);
                        var txtwidth = this.rightTooltip.lastChild.getBBox().width + 15;
                        var txtheight = this.rightTooltip.lastChild.getBBox().height;
                        this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                        this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                        if (rightPosX + txtwidth <= this.newWidth) {
                            this.rightTooltip.firstChild.setAttribute('d', this.rightd);
                            this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX) + ',' + cPosition + ')' });
                            this.rightTxt.setAttribute('x', 10);
                        }
                        else {
                            this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX - txtwidth) + ',' + cPosition + ')' });
                            this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                            this.rightTxt.setAttribute('x', 5);
                            if (this.rightTooltip.getBoundingClientRect().left < this.leftSlider.getBoundingClientRect().left) {
                                this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX - txtwidth) + ',' + (cPosition + 30) + ')' });
                            }
                        }
                    }
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftPosX - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    this.renderer._setAttr($(this.rightUnArea), { 'transform': 'translate(' + parseFloat(rightPosX + 2) + ',' + this.sliderPosition + ')', 'width': this.newWidth - rightPosX - this.padding - 2 });

                    this.model.zoomCordinates.zoomPosition = (this.leftSlider.getBoundingClientRect().left - this.padding) / (this.newWidth - this.padding * 2);

                    this.renderer._setAttr($(this.centerSlider), { 'transform': 'translate(' + centerValue + ',' + this.sliderPosition + ')' });
                    this.model.zoomCordinates.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
                }

            }

            if (!this.model.deferredUpdate && this.leftSlider.getBoundingClientRect().left < this.rightSlider.getBoundingClientRect().left) {
                this._calculateSelectedData();
                if (this.zoomp != this.model.zoomCordinates.zoomPosition || this.zoomf != this.model.zoomCordinates.zoomFactor) {
                    this._trigger("rangeChanged", this.model);
                    this.zoomp = this.model.zoomCordinates.zoomPosition;
                    this.zoomf = this.model.zoomCordinates.zoomFactor;
                }
            }

        },

        _calculateSelectedData: function () {
            var j = [];
            var count = 0;
            if (this.model.dataSource.data != "")
                for (var i = 0; i < this.model.dataSource.data.length; i++) {
                    if (Date.parse(this.model.selectedRange.start) <= Date.parse(this.model.dataSource.data[i][this.model.dataSource.xName]) && Date.parse(this.model.selectedRange.end) >= Date.parse(this.model.dataSource.data[i][this.model.dataSource.xName])) {
                        j[count] = this.model.dataSource.data[i];
                        count++;
                    }
                }
            else if (this.model.series != "" && this.model.series[0].dataSource != "" && this.model.series[0].dataSource.data != "") {
                for (var i = 0; i < this.model.series[0].dataSource.data.length; i++) {
                    if (Date.parse(this.model.selectedRange.start) <= Date.parse(this.model.series[0].dataSource.data[i][this.model.series[0].dataSource.xName]) && Date.parse(this.model.selectedRange.end) >= Date.parse(this.model.series[0].dataSource.data[i][this.model.series[0].dataSource.xName])) {
                        j[count] = this.model.series[0].dataSource.data[i];
                        count++;
                    }
                }
            }
            this.model.selectedData = j;
        },

        calculateInterval: function (higherLevel, lowerLevel) {
            if (higherLevel.visible === true && this.model.valueType == "datetime") {
                if (higherLevel.intervalType === "years")
                    this.setYearInterval(this, "higher");
                else if (higherLevel.intervalType === "quarters")
                    this.setQuarterInterval(this, "higher", 0);
                else if (higherLevel.intervalType === "months")
                    this.setMonthInterval(this, "higher", 0);
                else if (higherLevel.intervalType == "weeks")
                    this.setWeekInterval(this, "higher", 0);
                else if (higherLevel.intervalType === "days")
                    this.setDayInterval(this, "higher", 0);
                else if (higherLevel.intervalType === "hours")
                    this.setHourInterval(this, "higher", 0);
            }
            if (lowerLevel.visible === true && this.model.valueType == "datetime") {

                if (lowerLevel.intervalType === "years")
                    this.setYearInterval(this, "lower");
                else if (lowerLevel.intervalType === "quarters")
                    this.setQuarterInterval(this, "lower", 0);
                else if (lowerLevel.intervalType === "months")
                    this.setMonthInterval(this, "lower", 0);
                else if (lowerLevel.intervalType == "weeks")
                    this.setWeekInterval(this, "lower", 0);
                else if (lowerLevel.intervalType === "days")
                    this.setDayInterval(this, "lower", 0);
                else if (lowerLevel.intervalType === "hours")
                    this.setHourInterval(this, "lower", 0);
            }

            if (lowerLevel.visible === true && this.model.valueType == "numeric") {
                this.setInterval(this);
            }


        },

        setInterval: function (naviobj) {
            var currentValue, lastValue;
            var delta = this.endValue - this.startValue;
            var rangePadding = naviobj.model.rangePadding;
            var start = this.startValue;
            var numericInterval = this.calculateNumericInterval(naviobj, delta);
            if (rangePadding == "additional") {
                this.endValue = this.endValue + numericInterval;
                this.startValue = this.startValue - numericInterval;
                delta = this.endValue - this.startValue;
                //  numericInterval = this.calculateNumericInterval(naviobj, delta);
            } else if (rangePadding == "normal") {
                var minimum = 0, remaining;
                if (start < 0) {
                    start = 0;
                    minimum = this.startValue + (this.startValue / 20);

                    remaining = numericInterval + (minimum % numericInterval);

                    if ((0.365 * numericInterval) >= remaining) {
                        minimum -= numericInterval;
                    }

                    if (minimum % numericInterval < 0) {
                        minimum = (minimum - numericInterval) - (minimum % numericInterval);
                    }
                } else {
                    minimum = start < ((5.0 / 6.0) * this.endValue)
                        ? 0
                        : (start - (this.endValue - start) / 2);
                    if (minimum % numericInterval > 0) {
                        minimum -= (minimum % numericInterval);
                    }
                }
                var maximum = (this.endValue + (this.endValue - start) / 20);

                remaining = numericInterval - (maximum % numericInterval);

                if ((0.365 * numericInterval) >= remaining) {
                    maximum += numericInterval;
                }

                if (maximum % numericInterval > 0) {
                    maximum = (maximum + numericInterval) - (maximum % numericInterval);
                }
                numericInterval = this.calculateNumericInterval(naviobj, maximum - minimum);
                this.startValue = minimum;
                this.endValue = maximum;

            } else if (rangePadding == "round") {
                this.startValue = Math.floor((this.startValue / numericInterval) * numericInterval);
                this.endValue = Math.ceil((this.endValue / numericInterval) * numericInterval);
            }

            currentValue = naviobj.startValue;
            count = 0;
            while (currentValue <= naviobj.endValue) {
                this._insertNumericText(naviobj, currentValue);
                count++;
                currentValue = currentValue + numericInterval;
            }
            this.insertLabels(naviobj, "lower");

        },

        calculateNumericInterval: function (naviobj, delta) {
            var desiredIntervalsCount = this.GetDesiredIntervalsCount(naviobj.newWidth);
            var niceInterval = delta / desiredIntervalsCount;
            var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
            var intervalDivs = [10, 5, 2, 1];

            for (var i = 0; i < intervalDivs.length; i++) {
                var currentInterval = minInterval * intervalDivs[i];
                if (desiredIntervalsCount < (delta / currentInterval)) {
                    return niceInterval;
                }

                niceInterval = currentInterval;
            }

            return niceInterval;
        },

        GetDesiredIntervalsCount: function (size) {
            // if (ej.util.isNullOrUndefined(axis.desiredIntervals)) {
            var desiredIntervalsCount = 0.8 * 2;
            desiredIntervalsCount = Math.max((size * (desiredIntervalsCount / 100)), 1);
            return desiredIntervalsCount;
            //} else {
            //    return axis.desiredIntervals;
            //  }
        },

        setHourInterval: function (naviobj, position, level) {

            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addHours = function (hours) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0) {
                    dat.setMilliseconds(dat.getMilliseconds() + (1000 - dat.getMilliseconds()));
                }
                if (dat.getSeconds() !== 0) {
                    dat.setSeconds(dat.getSeconds() + (60 - dat.getSeconds()));
                }
                if (dat.getMinutes() !== 0) {
                    dat.setMinutes(dat.getMinutes() + (60 - dat.getMinutes()));
                }
                else
                    dat.setHours(dat.getHours() + hours);
                return dat;
            }

            var hourArray = [];
            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hhtt", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addHours(1);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setHourInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hht", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addHours(1);
                    }
                    this.insertLabels(naviobj, position);
                    break;
            }
        },

        // Empty the labelbars
        _emptyLabelBars: function (naviobj, position) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();
        },

        //to insert week text as there is no globalized format for week
        _insertWeekText: function (naviobj, position, textValue, currentDate, count, bwidth) {
            var hiLabelFont = naviobj.model.labelSettings.higherLevel.labelstyles.font;
            var loLabelFont = naviobj.model.labelSettings.lowerLevel.labelstyles.font;
            if (position === "higher") {
                if (count == 0)
                    naviobj._higherTotalValues[count] = 0;
                else
                    naviobj._higherTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_higLabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": hiLabelFont.size,
                    "font-family": hiLabelFont.fontFamily,
                    "font-style": hiLabelFont.fontStyle,
                    "font-weight": hiLabelFont.fontWeight,
                    "fill": hiLabelFont.color
                };
                naviobj.renderer.drawText(textOptions, textValue, naviobj.higherLevel);
                naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.higherLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            else {
                if (count == 0)
                    naviobj._lowerTotalValues[count] = 0;
                else
                    naviobj._lowerTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_LabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": loLabelFont.size,
                    "font-family": loLabelFont.fontFamily,
                    "font-style": loLabelFont.fontStyle,
                    "font-weight": loLabelFont.fontWeight,
                    "fill": loLabelFont.color
                };

                naviobj.renderer.drawText(textOptions, textValue, naviobj.lowerLevel);
                naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.lowerLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }

            return bwidth;
        },

        //To calculate Week Interval
        setWeekInterval: function (naviobj, position, level) {

            this._emptyLabelBars(naviobj, position);
            //get week number of a year
            Date.prototype.getWeek = function () {
                var date = new Date(this.getTime());
                date.setHours(0, 0, 0, 0);
                // Thursday in current week decides the year.
                date.setDate(date.getDate() + 3 - (date.getDay() + 6) % 7);
                // January 4 is always in week 1.
                var week1 = new Date(date.getFullYear(), 0, 4);
                // Adjust to Thursday in week 1 and count number of weeks from date to week1.
                return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000
                                      - 3 + (week1.getDay() + 6) % 7) / 7);
            }

            Date.prototype.dayOfWeek = function () {
                var days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                return days[this.getDay()];
            }

            Date.prototype.addDays = function (noOfDays) {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() + noOfDays);
                return dat;
            }

            Date.prototype.subDays = function (noOfDays) {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() - noOfDays);
                return dat;
            }

            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            var culture = this.model.localization;
            var locale = ej.RangeNavigator.localization[culture] ? ej.RangeNavigator.localization[culture] : ej.RangeNavigator.localization["en-US"];


            while (currentDate.dayOfWeek() != "Monday") {
                currentDate = currentDate.subDays(1);
            }
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + " " + currentDate.getWeek() + Globalize.format(currentDate, " MMMM, yyyy"), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + " " + currentDate.getWeek() + Globalize.format(currentDate, " MMM, yy"), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + " " + currentDate.getWeek(), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].shortWeeks + " " + currentDate.getWeek(), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    this.insertLabels(naviobj, position);
                    break;
            }
        },

        setDayInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addDays = function () {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() + 1);
                return dat;
            }

            var hourArray = [];
            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            var bBox;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dddd, MMMM d, yyyy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "ddd, MMM d, yy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dddd, d", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dd", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();
                    }
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;

            }

        },

        setMonthInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            var currentDate = naviobj.startDateTime;

            var count = 0;
            var date = new Date(naviobj.startDateTime);
            var txtBlockWidth = 0;
            var bwidth = 0;
            var bBox;

            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMMM, yyyy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMMM", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM", currentDate, count, bwidth, true);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;

            }

        },

        _quarter: function (naviobj, position, quarterValue, yearString, date, txtBlockWidth, count) {

            for (var i = naviobj.startYear; i <= naviobj.endYear; i++) {
                var stringDate = yearString != null ? Globalize.format(date, yearString) : "";
                if (i === naviobj.startYear) {
                    for (var j = naviobj.startMonth; j <= 11; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.labelstyles.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontWeight,
                                "fill": naviobj.model.labelSettings.higherLevel.labelstyles.font.color
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.labelstyles.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontWeight,
                                "fill": naviobj.model.labelSettings.lowerLevel.labelstyles.font.color
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            if (j === naviobj.startMonth)
                                naviobj._lowerTotalValues[count] = naviobj.startDateTime.getTime() - naviobj.startDateTime.getTime();
                            else
                                naviobj._lowerTotalValues[count] = new Date(naviobj.startYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            if (j === naviobj.startMonth)
                                naviobj._higherTotalValues[count] = naviobj.startDateTime.getTime() - naviobj.startDateTime.getTime();
                            else
                                naviobj._higherTotalValues[count] = new Date(naviobj.startYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }
                else if (i === naviobj.endYear) {
                    for (var j = 0; j <= naviobj.endMonth; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.labelstyles.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontWeight
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.labelstyles.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontWeight
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            naviobj._lowerTotalValues[count] = new Date(naviobj.endYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            naviobj._higherTotalValues[count] = new Date(naviobj.endYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }
                else {
                    for (var j = 0; j <= 11; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.labelstyles.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontWeight
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.labelstyles.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontWeight
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            naviobj._lowerTotalValues[count] = new Date(i + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            naviobj._higherTotalValues[count] = new Date(i + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }

            }
            return txtBlockWidth;
        },

        setQuarterInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();
            var date = new Date(naviobj.startDateTime);
            var count = 0;
            var k = 0;
            var txtBlockWidth = 0;
            var culture = this.model.localization;
            var locale = ej.RangeNavigator.localization[culture] ? ej.RangeNavigator.localization[culture] :
                                                                               ej.RangeNavigator.localization["en-US"];
            switch (level) {
                case 0:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].longQuarters, "yyyy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setQuarterInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].longQuarters, "yy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setQuarterInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].shortQuarters, "yyyy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setQuarterInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].shortQuarters, null, date, txtBlockWidth, count);
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;
            }
        },

        _insertText: function (naviobj, position, txtValue, currentDate, count, bwidth, setsubstring) {

            Date.prototype.addYears = function (years) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0 || dat.getSeconds() !== 0
                 || dat.getMinutes() !== 0 || dat.getHours() !== 0 || dat.getDate() !== 0
                 || dat.getMonth() !== 0) {
                    dat = new Date(dat.getFullYear() + 1, 0, 1);
                }
                else
                    dat.setYear(dat.getFullYear() + years);
                return dat;
            }

            Date.prototype.addMonths = function () {
                var dat = new Date(this.valueOf())
                dat = new Date(dat.getFullYear(), dat.getMonth() + 1, 1);
                return dat;
            }

            if (position === "higher") {
                naviobj._higherTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_higLabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": naviobj.model.labelSettings.higherLevel.labelstyles.font.size,
                    "font-family": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily,
                    "font-style": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle,
                    "font-weight": naviobj.model.labelSettings.higherLevel.labelstyles.font.fontWeight,
                    "fill": naviobj.model.labelSettings.higherLevel.labelstyles.font.color
                };
                if (setsubstring == false)
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue), naviobj.higherLevel);
                else
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue).toString().substring(0, 1), naviobj.higherLevel);
                naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.higherLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            else {
                naviobj._lowerTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_LabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": naviobj.model.labelSettings.lowerLevel.labelstyles.font.size,
                    "font-family": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily,
                    "font-style": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle,
                    "font-weight": naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontWeight,
                    "fill": naviobj.model.labelSettings.lowerLevel.labelstyles.font.color
                };
                if (setsubstring == false)
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue), naviobj.lowerLevel);
                else
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue).substring(0, 1), naviobj.lowerLevel);
                naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.lowerLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            return bwidth;
        },

        _insertNumericText: function (naviobj, currentValue) {
            naviobj._lowerTotalValues[count] = currentValue - naviobj.startValue;
            var font = naviobj.model.labelSettings.labelstyles.font;
            var textOptions = {
                'id': this._rootId + '_LabelBarText' + '_' + count,
                'x': 0,
                'y': 0,
                'text-anchor': 'start',
                'dominant-baseline': 'middle',
                "font-size": font.size,
                "font-family": font.fontFamily,
                "font-style": font.fontStyle,
                "font-weight": font.fontWeight,
                "fill": font.color
            };
            naviobj.renderer.drawText(textOptions, currentValue, naviobj.lowerLevel);
            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily };
            var textOffset;
            if (this.svgSupport) {
                textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
            } else {
                textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
            }
        },

        setYearInterval: function (naviobj, position) {
            var currentDate = naviobj.startDateTime;
            count = 0;
            while (currentDate <= naviobj.endDateTime) {
                this._insertText(naviobj, position, "yyyy", currentDate, count, 0, false);
                count++;
                currentDate = currentDate.addYears(1);
            }
            this.insertLabels(naviobj, position);
        },

        _applyPadding: function () {
            var i;
            for (i = 0; i < this._higherTotalValues.length; i++) {
                this._higherTotalValues[i] += this.padding;
            }
            for (i = 0; i < this._lowerTotalValues.length; i++) {
                this._lowerTotalValues[i] += this.padding;
            }
        },

        insertLabels: function (naviobj, level) {
            // this._applyPadding();
            var matched = jQuery.uaMatch(navigator.userAgent);
            var lowgridColor = this.model.labelSettings.lowerLevel.gridLineStyle.color;
            this.lgClr = this.renderer.createGradientElement('lgColor', lowgridColor, 150, 0, 150, 100, this.svgDocument);
            var type = this.model.valueType;
            var wholeSize = parseFloat(this.renderer._getAttrVal($(naviobj.svgDocument), 'width'));
            if (type == "datetime")
                this.eachInterval = (wholeSize - this.padding * 2) / parseFloat(this.endDateTime.getTime() - this.startDateTime.getTime());
            else
                this.eachInterval = (wholeSize - this.padding * 2) / parseFloat(this.endValue - this.startValue);
            var count = 0;
            var labelcount = 0;
            var txtwidth = 0;
            var rectoptions;
            var lineleft;
            var newleft;
            var textOffset;
            var labelfont;
            var i;
            if (level === "higher" && naviobj.model.labelSettings.higherLevel.visible === true) {

                if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                    rectoptions = {
                        x: this.padding,
                        y: 1,
                        width: naviobj.newWidth - naviobj.padding * 2,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }
                else if (naviobj.model.labelSettings.higherLevel.position == "top") {
                    rectoptions = {
                        x: this.padding,
                        y: 1,
                        width: naviobj.newWidth - naviobj.padding * 2,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }
                else {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.bottomPosition,
                        width: naviobj.newWidth - naviobj.padding * 2,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }

                naviobj._higherLineLeft = [];

                if (!naviobj.model.isRTL) {
                    lineleft = (naviobj._higherTotalValues[count] * this.eachInterval) + this.padding;
                } else {
                    lineleft = naviobj.newWidth - ((naviobj._higherTotalValues[naviobj._higherTextNode.length - 1] * this.eachInterval) + this.padding);
                }
                var isrtl;
                if (naviobj.model.isRTL) {
                    i = naviobj._higherTextNode.length - 1;
                    isrtl = true;
                }
                else {
                    i = 0;
                    isrtl = false;
                }
                var setLeft = 0;

                for (; isrtl ? i >= 0 : i < naviobj._higherTextNode.length; isrtl ? i-- : i++) {
                    naviobj._higherLineLeft[i] = lineleft;
                    //  var txt =  naviobj._higherTextNode[i].getBBox();
                    labelfont = { size: naviobj.model.labelSettings.higherLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.labelstyles.font.fontFamily };
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                    txtwidth = textOffset.width;
                    newleft = naviobj._higherTotalValues[i + 1] === undefined
                        ? isrtl ? 0 : naviobj.newWidth :
                        isrtl ? naviobj.newWidth - ((naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding)) :
                        (naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding);
                    if (naviobj.model.labelSettings.higherLevel.labelstyles.horizontalAlignment === "center") {
                        setleft = (newleft - lineleft) / 2 + lineleft - txtwidth / 2;
                    }
                    else if (naviobj.model.labelSettings.higherLevel.labelstyles.horizontalAlignment === "left") {
                        setleft = lineleft + 10;
                    }
                    else
                        setleft = newleft - txtwidth - 2;
                    naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'x': setleft });
                    naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'fill': naviobj.model.labelSettings.higherLevel.labelstyles.font.color });
                    naviobj._higherTextLeft[i] = setleft;

                    if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                        naviobj._higherTextNode[i].setAttribute("y", naviobj.minHighHeight - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: '0',
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.strokedasharray,

                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    else if (naviobj.model.labelSettings.higherLevel.position == "top") {
                        if (this.svgSupport) {
                            if (matched.browser.toLowerCase() == "msie")
                                naviobj._higherTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                            else
                                naviobj._higherTextNode[i].setAttribute("y", naviobj.centerPosition - 9);
                        }
                        else
                            naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'y': (naviobj.centerPosition / 2) - 5 });
                        rectoptions = {
                            x1: lineleft,
                            y1: '0',
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.strokedasharray,

                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    else {
                        naviobj._higherTextNode[i].setAttribute("y", naviobj.newHeight - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: this.bottomPosition,
                            x2: lineleft,
                            y2: naviobj.newHeight,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.strokedasharray,

                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    if (!this.model.isRTL) {
                        lineleft = naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding;
                    } else {
                        lineleft = naviobj.newWidth - (naviobj._higherTotalValues[i - 1] * this.eachInterval + this.padding);
                    }
                    if ((setleft < this.padding || setleft + txtwidth > naviobj.newWidth - this.padding) && type == "datetime") {
                        naviobj.higherLevel.removeChild(naviobj._higherTextNode[i]);
                        continue;
                    }
                    count++;
                }



            }
            else if (naviobj.model.labelSettings.lowerLevel.visible === true) {

                naviobj._lowerLineLeft = [];
                if (!naviobj.model.isRTL) {
                    lineleft = (naviobj._lowerTotalValues[count] * this.eachInterval) + this.padding;
                } else {
                    lineleft = naviobj.newWidth - ((naviobj._lowerTotalValues[naviobj._lowerTextNode.length - 1] * this.eachInterval) + this.padding);
                }
                var hy = naviobj.model.labelSettings.higherLevel.visible ? naviobj.bottomPosition : naviobj.centerHeight;

                if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "top") {
                    rectoptions = {
                        x: this.padding,
                        y: hy,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    if (type != "numeric")
                        naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.centerHeight,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.lowerLevel.position === "top" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                    rectoptions = {
                        x: this.padding,
                        y: 0,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.higherLevel.position === "top" && naviobj.model.labelSettings.lowerLevel.position === "top") {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.minHighHeight - 1,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }

                if (naviobj.model.isRTL) {
                    i = naviobj._lowerTextNode.length - 1;
                    isrtl = true;
                }
                else {
                    i = 0;
                    isrtl = false;
                }


                for (; isrtl ? i >= 0 : i < naviobj._lowerTextNode.length; isrtl ? i-- : i++) {
                    naviobj._lowerLineLeft[i] = lineleft;
                    labelfont = { size: naviobj.model.labelSettings.lowerLevel.labelstyles.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.labelstyles.font.fontFamily };
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont); // var txt = naviobj._lowerTextNode[i].getBBox();
                    txtwidth = textOffset.width;// txt.width;
                    var setleft = 0;
                    newleft = naviobj._lowerTotalValues[i + 1] === undefined
                        ? isrtl ? 0 : naviobj.newWidth :
                        isrtl ? naviobj.newWidth - ((naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding)) :
                        (naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding);
                    if (naviobj.model.labelSettings.lowerLevel.labelstyles.horizontalAlignment === "center") {
                        setleft = (newleft - lineleft) / 2 + lineleft - txtwidth / 2;
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.labelstyles.horizontalAlignment === "left") {
                        setleft = lineleft + 2;
                    }
                    else
                        setleft = newleft - txtwidth - 2;
                    if (type == "numeric")
                        setleft = lineleft - txtwidth / 2;
                    naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'x': setleft });
                    naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'fill': naviobj.model.labelSettings.lowerLevel.labelstyles.font.color });

                    naviobj._lowerTextLeft[i] = setleft;

                    if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "top") {
                        if (!this.svgSupport)
                            naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'y': naviobj.newHeight - textOffset.height });
                        else {
                            if (matched.browser.toLowerCase() == "msie")
                                naviobj._lowerTextNode[i].setAttribute("y", naviobj.newHeight - 5);
                            else
                                naviobj._lowerTextNode[i].setAttribute("y", naviobj.newHeight - 9);

                        }
                        var ht = naviobj.model.labelSettings.higherLevel.visible ? parseFloat(naviobj.minHighHeight + naviobj.centerHeight) : naviobj.minHighHeight;
                        rectoptions = {
                            x1: lineleft,
                            y1: ht,
                            x2: lineleft,
                            y2: naviobj.newHeight,
                            'stroke': this.lgClr,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.strokedasharray,
                        };
                        if (i != 0 && type != "numeric")
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.bottomPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: naviobj.centerHeight,
                            x2: lineleft,
                            y2: naviobj.bottomPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.strokedasharray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.position === "top" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: 0,
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.strokedasharray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.higherLevel.position === "top" && naviobj.model.labelSettings.lowerLevel.position === "top") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: naviobj.minHighHeight,
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.strokedasharray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    if (!this.model.isRTL)
                        lineleft = naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding;
                    else
                        lineleft = naviobj.newWidth - (naviobj._lowerTotalValues[i - 1] * this.eachInterval + this.padding);
                    if ((setleft < this.padding || setleft + txtwidth > naviobj.newWidth - this.padding) && type == "datetime") {
                        naviobj.lowerLevel.removeChild(naviobj._lowerTextNode[i]);
                        continue;
                    }
                    count++;
                }



            }
            naviobj._setGridlines(naviobj);
            //this.higherLevel.setAttributeNS(null, 'transform', 'translate(' + 20 + ',' + 0 + ')');
            //this.lowerLevel.setAttributeNS(null, 'transform', 'translate(' + 20 + ',' + 0 + ')');

        },

        _setGridlines: function (naviobj) {

            if (naviobj.model.navigatorStyle.minorGridLineStyle.visible === true) {

                for (var i = 1; i < naviobj._lowerTextNode.length; i++) {
                    if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                        var rectoptions = {
                            x1: naviobj._lowerLineLeft[i],
                            y1: naviobj.centerHeight + naviobj.minLowHeight + naviobj.minHighHeight,
                            x2: naviobj._lowerLineLeft[i],
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.navigatorStyle.minorGridLineStyle.color,
                            'stroke-width': '1'
                        };
                        naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                    }
                    else {
                        var rectoptions = {
                            x1: naviobj._lowerLineLeft[i],
                            y1: naviobj.centerPosition,
                            x2: naviobj._lowerLineLeft[i],
                            y2: naviobj.centerPosition + naviobj.centerHeight,
                            'stroke': naviobj.model.navigatorStyle.minorGridLineStyle.color,
                            'stroke-width': '1'
                        };
                        naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                    }

                }
            }
            else {
                for (var i = 0; i < naviobj._higherTextNode.length; i++) {
                    // var lineLeft = naviobj._lowerLineLeft[i];
                    var rectoptions = {
                        x1: naviobj._higherLineLeft[i],
                        y1: naviobj.centerHeight + naviobj.minLowHeight,
                        x2: naviobj._higherLineLeft[i],
                        y2: naviobj.centerPosition,
                        'stroke': naviobj.model.navigatorStyle.majorGridLineStyle.color,
                        'stroke-width': '1'
                    };
                    naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                }
            }

        },

        bindTo: function () {
            if (!ej.util.isNullOrUndefined(this.model.dataSource)) {
                if (this.model.dataSource.data != null && this.model.dataSource.data.length > 0) {

                    this._processJsonData(this.model.dataSource.data, this.model);
                }
            }

        },

        _processJsonData: function (jsonObj, newmodel) {

            var xdata = [];
            var yValues = [];
            var index = 0;

            for (var j = 0; j < jsonObj.length; j++) {
                if (typeof jsonObj[j][newmodel.dataSource.xName] == "string" && jsonObj[j][newmodel.dataSource.xName].indexOf("/Date(") != -1)
                    jsonObj[j][newmodel.dataSource.xName] = new Date(parseInt(jsonObj[j][newmodel.dataSource.xName].substr(6)));
                if (jQuery.type(jsonObj[j][newmodel.dataSource.xName]) != "string" && jQuery.type(jsonObj[j][newmodel.dataSource.xName]) != "date")
                    xdata.push(parseFloat(!ej.util.isNullOrUndefined(jsonObj[j][newmodel.dataSource.xName]) ? jsonObj[j][newmodel.dataSource.xName] : j));
                else {
                    xdata.push(!ej.util.isNullOrUndefined(jsonObj[j][newmodel.dataSource.xName]) ? jsonObj[j][newmodel.dataSource.xName] : j);
                }
            }

            if (!ej.util.isNullOrUndefined(newmodel.dataSource.yName)) {
                for (var k = 0; k < newmodel.dataSource.yName.length; k++) {
                    var yNVal = [];
                    for (var yI = 0; yI < jsonObj.length; yI++) {
                        yNVal.push(jsonObj[yI][newmodel.dataSource.yName[k]] === null || jsonObj[yI][newmodel.dataSource.yName[k]] === undefined ? jsonObj[yI][newmodel.dataSource.yName[k]] : parseFloat(jsonObj[yI][newmodel.dataSource.yName[k]]));
                    }
                    yValues[k] = yNVal;
                    index = k + 1;
                }
            }

            this.endDateTime = new Date(Math.max.apply(null, xdata));
            this.startDateTime = new Date(Math.min.apply(null, xdata));
            this.endValue = Math.max.apply(null, xdata);
            this.startValue = Math.min.apply(null, xdata);
        },

        calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;

            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else if (matched.browser.toLowerCase() == "msie") {

                mouseX = (e.originalEvent.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.originalEvent.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }
            else {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        calMousePosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;
            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                mouseX = (e.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (e.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = 0;//$(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else {
                mouseX = (e.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;

            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else if (matched.browser.toLowerCase() == "msie") {

                mouseX = (e.originalEvent.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.originalEvent.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }
            else {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        //calMousePosition: function (e) {
        //    var matched = jQuery.uaMatch(navigator.userAgent);
        //    var mouseX, mouseY;
        //    if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
        //        mouseX = (e.pageX) - $(this.svgDocument).parent().offset().left;
        //        mouseY = (e.pageY) - $(this.svgDocument).parent().offset().top;
        //        this.leftPadding = $(this.svgDocument).offset().left;
        //        this.grabPadding = 0;
        //    } else {
        //        mouseX = (e.pageX) - $(this.svgDocument).offset().left;
        //        mouseY = (e.pageY) - $(this.svgDocument).offset().top;
        //        this.leftPadding = $(this.svgDocument).offset().left;
        //    }

        //    return { X: mouseX, Y: mouseY };

        //},

        _grab: function (evt) {
            evt.preventDefault();
            var matched = jQuery.uaMatch(navigator.userAgent);
            var naviobj = this;// evt.data.Param1;
            var mousedownCords;
            if ((this.svgSupport && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") {
                mousedownCords = naviobj.calMousePosition(evt);
            } else if (!this.svgSupport) {
                mousedownCords = naviobj.calMousePosition(evt);
            } else {
                mousedownCords = naviobj.calTouchPosition(evt);
            } // naviobj.calMousePosition(evt);
            naviobj.mouseDownX = mousedownCords.X;
            naviobj.mouseDownY = mousedownCords.Y;
            // find out which element we moused down on

            var targetElement;
            if (naviobj.edge == true) {
                targetElement = naviobj.target;
            }
            else {
                targetElement = evt.target;
            }

            //$(targetElement).css('cursor', 'w-resize');
            // you cannot drag the background itself, so ignore any attempts to mouse down on it
            if (targetElement.parentNode.id === "leftslider" || targetElement.parentNode.id === "rightslider" || targetElement.parentNode.id === "centerslider" || targetElement.parentNode.id === this._rootId + "higherLevel" || targetElement.parentNode.id === this._rootId + "lowerLevel") {
                //set the item moused down on as the element to be dragged
                naviobj.dragTarget = targetElement;
                naviobj.grabbed = true;
                naviobj.edge = false;
                if (this.svgSupport) {
                    var transMatrix = naviobj.dragTarget.getCTM();
                    if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                        naviobj.grabPoint.x = naviobj.mouseDownX + this.grabPadding - Number(transMatrix.e);
                    }
                    else
                        naviobj.grabPoint.x = naviobj.mouseDownX + this.leftPadding - Number(transMatrix.e);
                    naviobj.grabPoint.y = naviobj.trueCoords.y - Number(transMatrix.f);
                }
                var txtPosition;
                var i;
                var flag = false;
                var isrtl = naviobj.model.isRTL;
                if (isrtl) {
                    naviobj._higherLineLeft.sort(function (a, b) { return a - b });
                    naviobj._lowerLineLeft.sort(function (a, b) { return a - b });
                }
                if (naviobj.dragTarget.parentNode.id === this._rootId + "higherLevel" && naviobj.model.valueType == "datetime") {
                    naviobj.grabbed = false;
                    txtPosition = naviobj.mouseDownX;
                    for (i = 0; i < naviobj._higherLineLeft.length; i++) {
                        if (i + 1 < naviobj._higherLineLeft.length && txtPosition > naviobj._higherLineLeft[i] && txtPosition < naviobj._higherLineLeft[i + 1]) {
                            naviobj.setSliderPositions(naviobj._higherLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj._higherLineLeft[i + 1]);
                            flag = true;
                        }
                        else if (i + 1 === naviobj._higherLineLeft.length && txtPosition > naviobj._higherLineLeft[i]) {
                            naviobj.setSliderPositions(naviobj._higherLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                            flag = true;
                        }
                    }
                    if (isrtl && !flag) {
                        naviobj.setSliderPositions(0 + naviobj.padding, null, null);
                        naviobj.setSliderPositions(null, null, naviobj._higherLineLeft[0]);
                        flag = false;
                    }
                }
                else if (naviobj.dragTarget.parentNode.id === this._rootId + "lowerLevel" && naviobj.model.valueType == "datetime") {
                    naviobj.grabbed = false;
                    txtPosition = naviobj.mouseDownX;
                    for (i = 0; i < naviobj._lowerLineLeft.length; i++) {
                        if (i + 1 < naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i] && txtPosition < naviobj._lowerLineLeft[i + 1]) {
                            naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1]);
                            flag = true;
                        }
                        else if (i + 1 === naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i]) {
                            naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                            flag = true;
                        }
                    }
                    if (isrtl && !flag) {
                        naviobj.setSliderPositions(0 + naviobj.padding, null, null);
                        naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[0]);
                        flag = false;
                    }
                }

            }
        },

        _drag: function (evt) {

            evt.preventDefault();
            var matched = jQuery.uaMatch(navigator.userAgent);

            var naviobj = this;//evt.data.Param1;
            if ((this.svgSupport && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") {
                var mousedownCords = naviobj.calMousePosition(evt);
            }
            else if (!this.svgSupport)
                var mousedownCords = naviobj.calMousePosition(evt);
            else
                var mousedownCords = naviobj.calTouchPosition(evt);//naviobj.calMousePosition(evt);
            naviobj.mouseDownX = mousedownCords.X;
            naviobj.mouseDownY = mousedownCords.Y;
            if (this.svgSupport)
                naviobj.GetTrueCoords(evt);
            // if we don't currently have an element in tow, don't do anything
            if (naviobj.dragTarget) {
                // account for the offset between the element's origin and the
                //    exact place we grabbed it... this way, the drag will look more natural
                if (!naviobj.edge && naviobj.dragTarget === naviobj.leftSlider || naviobj.dragTarget === naviobj.rightSlider || naviobj.dragTarget === naviobj.centerSlider) {
                    if (this.svgSupport) {
                        if (matched.browser.toLowerCase() == "webkit" || matched.browser.toLowerCase() == "mozilla") {
                            var newX = naviobj.mouseDownX - naviobj.grabPoint.x;
                            var newY = naviobj.mouseDownY - naviobj.grabPoint.y;
                        }
                        else {
                            var newX = naviobj.trueCoords.x - naviobj.grabPoint.x;
                            var newY = naviobj.trueCoords.y - naviobj.grabPoint.y;
                        }
                    }
                    else {
                        var newX = naviobj.mouseDownX;
                        var newY = naviobj.mouseDownY;
                    }
                }
                else {
                    var newX = naviobj.mouseDownX;// naviobj.trueCoords.x - naviobj.leftPadding;
                    //  var newX = naviobj.mouseDownX;// naviobj.trueCoords.x - naviobj.leftPadding;
                    var newY = naviobj.mouseDownY;// naviobj.trueCoords.y;
                }
                naviobj.diff = naviobj.leftSlider.getBoundingClientRect().left - naviobj.rightSlider.getBoundingClientRect().left;
                naviobj.leftdiff = naviobj.rightSlider.getBoundingClientRect().left - this.padding;
                naviobj.rightdiff = naviobj.leftSlider.getBoundingClientRect().left - this.padding;
                if (naviobj.dragTarget.parentNode.id === "leftslider") {
                    naviobj.grabbed = false;
                    if (naviobj.diff >= -1 && naviobj.diff <= 1 && naviobj.switched == false && naviobj.rightdiff < naviobj.newWidth - (2 * this.padding)) {
                        naviobj.target = naviobj.rightSlider;
                        naviobj.switched = true;
                        naviobj.edge = true;
                        naviobj._grab(evt);
                        naviobj._drag(evt);
                    }
                    else {
                        if (newX <= this.rightSliderPosition) {
                            naviobj.setSliderPositions(newX, null, null);
                            this.leftSliderPosition = newX;
                            this.leftset = false;
                        }
                        else {
                            naviobj.setSliderPositions(this.rightSliderPosition, null, null);
                            this.leftset = true;
                            this.leftSliderPosition = this.rightSliderPosition;
                        }
                        if (naviobj.diff >= 1 || naviobj.diff <= -1)
                            naviobj.switched = false;
                    }
                    naviobj.left = newX;
                }
                else if (naviobj.dragTarget.parentNode.id === "rightslider") {
                    naviobj.grabbed = false;
                    // if (((this.svgSupport) && naviobj.leftSlider.getCTM().e == naviobj.rightSlider.getCTM().e && newX < naviobj.newWidth-15 && naviobj.rightSlider.getCTM().e == (naviobj.newWidth - naviobj.padding)) || (naviobj.leftSlider.getBoundingClientRect().left == naviobj.rightSlider.getBoundingClientRect().left && newX < naviobj.newWidth - 15 && naviobj.rightSlider.getBoundingClientRect().left == (naviobj.newWidth - naviobj.padding))) {                                                                
                    if (naviobj.diff >= 0 && naviobj.diff <= 1 && naviobj.switched == false && naviobj.leftdiff > 0) {
                        naviobj.target = naviobj.leftSlider;
                        naviobj.switched = true;
                        naviobj.edge = true;
                        naviobj._grab(evt);
                        naviobj._drag(evt);
                    }
                    else {
                        //if (newX >= naviobj.leftSlider.getCTM().e)
                        if (newX >= this.leftSliderPosition) { // naviobj.leftSlider.getBoundingClientRect().left - this.leftPadding)
                            naviobj.setSliderPositions(null, null, newX);
                            this.rightSliderPosition = newX;
                        } else {
                            naviobj.setSliderPositions(null, null, this.leftSliderPosition);
                            this.rightSliderPosition = this.leftSliderPosition;
                        }
                        //naviobj.setSliderPositions(null, null, naviobj.leftSlider.getCTM().e);
                        if (naviobj.diff >= 1 || naviobj.diff <= -1)
                            naviobj.switched = false;
                    }
                    naviobj.right = newX;
                }
                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                    naviobj.grabbed = false;
                    naviobj.setSliderPositions(null, parseFloat(newX), null);
                    if (newX > newX + this.renderer._getAttrVal($(naviobj.dragTarget), 'width'))
                        naviobj.setSliderPositions(null, newX, null);
                    naviobj.center = newX;
                }
            }
        },


        _drop: function (evt) {
            // if we aren't currently dragging an element, don't do anything
            if (this.dragTarget) {
                // since the element currently being dragged has its pointer-events turned off,
                //    we are afforded the opportunity to find out the element it's being dropped on
                var targetElement = evt.target;

                var naviobj = this;
                // set the global variable to null, so nothing will be dragged until we
                //    grab the next element
                if (naviobj.model.snappingMode && parseInt(naviobj.leftSlider.getBoundingClientRect().left + 2) < parseInt(naviobj.rightSlider.getBoundingClientRect().left)) {
                    naviobj.ismouseup = true;
                    if (naviobj.dragTarget.parentNode.id === "leftslider")
                        var txtPosition = naviobj.left;
                    else if (naviobj.dragTarget.parentNode.id === "rightslider")
                        var txtPosition = naviobj.right;
                    else if (naviobj.dragTarget.parentNode.id === "centerslider")
                        var txtPosition = naviobj.center;
                    var pos = 0;

                    for (var i = 0; i < naviobj._lowerLineLeft.length; i++) {
                        if (i + 1 < naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i] && txtPosition < naviobj._lowerLineLeft[i + 1]) {
                            naviobj.snapValue = naviobj._lowerLineLeft[i];
                            pos = (naviobj._lowerLineLeft[i + 1] - naviobj.snapValue);
                            if (txtPosition >= (naviobj.snapValue + (pos / 2))) {
                                naviobj.snapValue = naviobj._lowerLineLeft[i + 1];
                                if (naviobj.dragTarget.parentNode.id === "leftslider")
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                else if (naviobj.dragTarget.parentNode.id === "rightslider")
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1]);
                                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1] + parseFloat(naviobj.centerSlider.getAttribute("width")));
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                }

                            }
                            else {
                                naviobj.snapValue = naviobj._lowerLineLeft[i];
                                if (naviobj.dragTarget.parentNode.id === "leftslider")
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                else if (naviobj.dragTarget.parentNode.id === "rightslider")
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i]);
                                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i] + parseFloat(naviobj.centerSlider.getAttribute("width")));
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                }

                            }
                        }
                        else if (i + 1 === naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i]) {
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                        }
                    }
                }
                this.dragTarget = null;

                if (this.leftTooltip != undefined && this.model.tooltip.tooltipDisplayMode == "always") {
                    this.leftTooltip.setAttribute('opacity', 1);
                    this.rightTooltip.setAttribute('opacity', 1);
                }
                else if (this.leftTooltip != undefined && this.model.tooltip.tooltipDisplayMode == "ondemand") {
                    this.leftTooltip.setAttribute('opacity', 0);
                    this.rightTooltip.setAttribute('opacity', 0);
                }
                if (this.model.deferredUpdate) {
                    this._calculateSelectedData();
                    if (!naviobj.grabbed) {
                        if (this.zoomp != this.model.zoomCordinates.zoomPosition || this.zoomf != this.model.zoomCordinates.zoomFactor) {
                            this._callRangeChange(this);
                            this.zoomp = this.model.zoomCordinates.zoomPosition;
                            this.zoomf = this.model.zoomCordinates.zoomFactor;
                        }

                    }
                }
            }
        },

        _callRangeChange: function (args) {
            args._trigger("rangeChanged", args.model);
        },

        mouseup: function (evt) {
            if (this.dragTarget != null) {
                this._drop(evt);
                if (this.dragTarget != null) {
                    this.dragTarget.setAttributeNS(null, 'pointer-events', 'all');
                    this.dragTarget = null;
                }
                if (this.model.deferredUpdate) {
                    this._calculateSelectedData();
                    this._trigger("rangeChanged", this.model);

                }
            }
        },

        GetTrueCoords: function (evt) {
            // find the current zoom level and pan setting, and adjust the reported
            //    mouse position accordingly
            var newScale = this.svgDocument.currentScale;
            var translation = this.svgDocument.currentTranslate;
            var matched = jQuery.uaMatch(navigator.userAgent);
            if (evt.originalEvent.toString() !== "[object TouchEvent]" && matched.browser.toLowerCase() != "msie") {
                this.trueCoords.x = (evt.clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.clientY - translation.y) / newScale;
            }
            else if (matched.browser.toLowerCase() == "msie") {
                this.trueCoords.x = (evt.originalEvent.clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.originalEvent.clientY - translation.y) / newScale;
            }
            else {
                this.trueCoords.x = (evt.originalEvent.touches[0].clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.originalEvent.touches[0].clientY - translation.y) / newScale;
            }
        }
    });

    ej.RangeNavigator.localization = {};
    ej.RangeNavigator.localization["en-US"] = {

        intervals: {
            quarter: { longQuarters: "Quarter", shortQuarters: "Q" },
            week: { longWeeks: "Week", shortWeeks: "W" }
        }
    };
})(jQuery, Syncfusion);

jQuery.uaMatch = function (ua) {
    ua = ua.toLowerCase();

    var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
        /(webkit)[ \/]([\w.]+)/.exec(ua) ||
        /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
        /(msie) ([\w.]+)/.exec(ua) ||
        ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
        [];

    return {
        browser: match[1] || "",
        version: match[2] || "0"
    };
};;
/**
* @fileOverview Plugin to style the Html Map elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejMap
    * @requires jQuery
    * @requires ej.Map
    * @requires jsrender
    * @requires properties
    * @classdesc Defined values to render a map.
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create Map
    * $('#container').ejMap(); 	
    * &lt;/script&gt;
    */

    ej.widget({ "ejMap": "ej.Map" }, {

        validTags: ["div"],
        _initPrivateProperties: function () {
            this._rootgroup = null;
            this._bubblegroup = null;
            this._scale = 1;
            this._prevDistance = 0;
            this._tileTranslatePoint = { x: 0, y: 0 };
            this._translatePoint = { x: 0, y: 0 };
            this._prevPoint = null;
            this._Tiles = [];
            this._prevScale = 0;
            this._tileDiv = null;
            this._containerWidth = 500;
            this._containerHeight = 400;
            this._baseTranslatePoint = { x: 0, y: 0 };
            this._isDragging = false;
            this._prevLevel = 1;
            this._browser = null;
            this._baseScale = 0;
            this._mapBounds = null;
            this._svgns = "http://www.w3.org/2000/svg";
            this._ispanning = false;
            this._dragStartX = 0;
            this._isNavigationPressed = false;
            this._isPolygonSelected = false;
            this._dragStartY = 0;
            this._width = 500;
            this._height = 500;
            this._svgDocument = null;
            this._tooltipElement = null;
            this._templateDiv = null;
            this._mapContainer = null;
            this._isSVG = true;
            this._VMLPathFractionCount = 0;
            this._sliderControl = null;
            this._isTileMap = false;
            this._isRendered = false;
            this._urlTemplate = null;
            this._polylineCount = 0;
            this._pointscount = 0;
            this._isPinching = false;
            this._groupSize = null;
            this._groupBorder = { x: 0, y: 0 };
        },


        defaults: {
            /**		
			* Specifies the background color of the map
            * @default "white"
			* @type {string}
			* @example 
            * //To set mapBackground API value during initialization 
            *   $("#container").ejMap({mapBackground:'white'});
            * @example 
			* //Get or set the mapBackground API, after initialization:<br>
			*	//Gets the mapBackground value 
            *   var property = $("#container").data("ejMap").model.mapBackground;<br>
            *	//Sets the mapBackground value 
			*	$("#container").data("ejMap").model.mapBackground="transparent";
			* @memberof ejMap
			* @instance
            */
            mapBackground: "white",

            /**		
			* Specifies the minimum zoom level of the map
            * @default 1
			* @type {number}
			* @example 
            * //To set minZoom API value during initialization 
            *   $("#container").ejMap({minZoom:1});
            * @example 
			* //Get or set the minZoom API, after initialization:<br>
			*	//Gets the minZoom value 
            * 	var property = $("#container").data("ejMap").model.minZoom;<br>
            *	//Sets the minZoom value 
			*	$("#container").data("ejMap").model.minZoom= 1;
			* @memberof ejMap
			* @instance
            */
            minZoom: 1,

            /**		
			* Specifies the maximum zoom level of the map
            * @default 100
			* @type {number}
			* @example 
            * //To set maxZoom API value during initialization 
            *   $("#container").ejMap({maxZoom:100});
            * @example 
			* //Get or set the maxZoom API, after initialization:<br>
			*	//Gets the maxZoom value 
            * 	var property = $("#container").data("ejMap").model.maxZoom;<br>
            *	//Sets the maxZoom value 
			*	$("#container").data("ejMap").model.maxZoom= 100;
			* @memberof ejMap
			* @instance
            */
            maxZoom: 100,

            /**		
			* Specifies the zoom factor of the map
            * @default 1
			* @type {number}
			* @example 
            * //To set zoomFactor API value during initialization 
            *   $("#container").ejMap({zoomFactor:1});
            * @example 
			* //Get or set the zoomFactor API, after initialization:<br>
			*	//Gets the zoomFactor value 
            * 	var property = $("#container").data("ejMap").model.zoomFactor;<br>
            *	//Sets the zoomFactor value 
			*	$("#container").data("ejMap").model.zoomFactor= 1;
			* @memberof ejMap
			* @instance
            */
            zoomFactor: 1,

            /**		
			* Specifies the zoom level of the map
            * @default 1
			* @type {number}
			* @example 
            * //To set zoomLevel API value during initialization 
            *   $("#container").ejMap({zoomLevel:1});
            * @example 
			* //Get or set the zoomLevel API, after initialization:<br>
			*	//Gets the zoomLevel value 
            * 	var property = $("#container").data("ejMap").model.zoomLevel;<br>
            *	//Sets the zoomLevel value 
			*	$("#container").data("ejMap").model.zoomLevel= 1;
			* @memberof ejMap
			* @instance
            */
            zoomLevel: 1,

            /**		
            * Specify center position of map.
            * @default [0,0]
            * @type {object} 
            * @example  
            * // Set the centerPosition during initialization. 			
            * 	 $("#container").ejMap({centerPosition: [38.5000, -98]});
            * @example 
            * //Get or set the centerPosition after initialization:<br>
            *   //Gets the centerPosition from map.
            *   var property =$("#container").data("ejMap").model.centerPosition;
            *   //Sets the centerPosition to map.
            *   $("#container").data("ejMap").model.centerPosition = [38.5000, -98];
            * @memberof ejMap
            * @instance
            */
            centerPosition: [0, 0],

            /**		
			 * Specifies whether map need to resize when container is resized 
			 * @default true
			 * @type {boolean}
			 * @example 
			 * //To set canResize API value during initialization 
			 * 	$("#container").ejMap({canResize:true});	
			 * @example 
			 * //Get or set the canResize API, after initialization:<br>
			 *	//Gets the canResize value 
			 *	var property = $("#container").data("ejMap").model.canResize;<br>		
			 *	//Sets the canResize value 
			 *	$("#container").data("ejMap").model.canResize= true }); 
			  * @memberof ejMap
			 * @instance
			 */
            canResize: true,

            /**		
			 * Specifies animation status of the map
			 * @default false
			 * @type {boolean}
			 * @example 
			 * //To set enableAnimation API value during initialization 
			 * 	$("#container").ejMap({enableAnimation:true});	
			 * @example 
			 * //Get or set the enableAnimation API, after initialization:<br>
			 *	//Gets the enableAnimation value 
			 *	var property = $("#container").data("ejMap").model.enableAnimation;<br>		
			 *	//Sets the enableAnimation value 
			 *	$("#container").data("ejMap").model.enableAnimation=true }); 
			  * @memberof ejMap
			 * @instance
			 */
            enableAnimation: false,

            /**		
			 * Specifies zoomin on Selection status of the map
			 * @default false
			 * @type {boolean}
			 * @example 
			 * //To set zoomOnSelection API value during initialization 
			 * 	$("#container").ejMap({zoomOnSelection:true});	
			 * @example 
			 * //Get or set the zoomOnSelection API, after initialization:<br>
			 *	//Gets the zoomOnSelection value 
			 *	var property = $("#container").data("ejMap").model.zoomOnSelection;<br>		
			 *	//Sets the zoomOnSelection value 
			 *	$("#container").data("ejMap").model.zoomOnSelection=true }); 
			  * @memberof ejMap
			 * @instance
			 */
            zoomOnSelection: false,

            /**		
			* Specifies the navigation control of the map
            * @default null 
			* @type {object}
			* @example 
            * //To set navigationControl during initialization 
            *   $("#container").ejMap(navigationControl:{enableNavigation:true,orientation:'vertical',absolutePosition:{x:5,y:20}});
            * @example 
			* //Get or set the navigationControl, after initialization:<br>
			*	//Gets the navigationControl  
            * 	var property = $("#container").data("ejMap").model.navigationControl;<br>
            *	//Sets the navigationControl  
			*	$("#container").data("ejMap").model.navigationControl={enableNavigation:true,orientation:'vertical',absolutePosition:{x:5,y:20}}); 
			* @memberof ejMap
			* @instance
            */
            navigationControl: null,

            /**		
			 * Specifies animation status while layer change, of the map
			 * @default false
			 * @type {boolean}
			 * @example 
			 * //To set enableLayerChangeAnimation API value during initialization 
			 * 	$("#container").ejMap({enableLayerChangeAnimation:true});	
			 * @example 
			 * //Get or set the enableLayerChangeAnimation API, after initialization:<br>
			 *	//Gets the enableLayerChangeAnimation value 
			 *	var property = $("#container").data("ejMap").model.enableLayerChangeAnimation;<br>		
			 *	//Sets the enableLayerChangeAnimation value 
			 *	$("#container").data("ejMap").model.enableLayerChangeAnimation=false }); 
			 * @memberof ejMap
			 * @instance
			 */
            enableLayerChangeAnimation: false,

            /**		
            * Specifies panning status of the map
            * @default true
            * @type {boolean}
            * @example 
            * //To set enablePan API value during initialization 
            * 	$("#container").ejMap({enablePan:true});	
            * @example 
            * //Get or set the enablePan API, after initialization:<br>
            *	//Gets the enablePan value 
            *	var property = $("#container").data("ejMap").model.enablePan;<br>		
            *	//Sets the enablePan value 
            *	$("#container").data("ejMap").model.enablePan=true }); 
            * @memberof ejMap
            * @instance
            */
            enablePan: true,

            /**		
			 * Specifies zooming status of the map
			 * @default true
			 * @type {boolean}
			 * @example 
			 * //To set enableZoom API value during initialization 
			 * 	$("#container").ejMap({enableZoom:true});	
			 * @example 
			 * //Get or set the enableZoom API, after initialization:<br>
			 *	//Gets the enableZoom value 
			 *	var property = $("#container").data("ejMap").model.enableZoom;<br>		
			 *	//Sets the enableZoom value 
			 *	$("#container").data("ejMap").model.enableZoom=true }); 
			 * @memberof ejMap
			 * @instance
			 */
            enableZoom: true,

            /**		
			* Specifies the base map-index of the map
            * @default 0 
			* @type {number}
			* @example 
            * //To set baseMapIndex API value during initialization 
            *   $("#container").ejMap({baseMapIndex:0});
            * @example 
			* //Get or set the baseMapIndex API, after initialization:<br>
			*	//Gets the baseMapIndex value 
            * 	var property = $("#container").data("ejMap").model.baseMapIndex;<br>
            *	//Sets the baseMapIndex value 
			*	$("#container").data("ejMap").model.baseMapIndex= 0;
			* @memberof ejMap
			* @instance
            */
            baseMapIndex: 0,

            /**     
            * Fires on shape selection in map.
            * @event
            * @name ejMap#shapeSelected		
            * @example 
            * //shapeSelected event for map
            * $("#container").ejMap({
            *    shapeSelected: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            shapeSelected: "",

            /**     
            * Fires on annotation selection in map.
            * @event
            * @name ejMap#annotationSelected		
            * @example 
            * //annotationSelected event for map
            * $("#container").ejMap({
            *    annotationSelected: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            annotationSelected: "",

            /**     
            * Fires on map zoomed-in.
            * @event
            * @name ejMap#zoomedIn		
            * @example 
            * //zoomedIn event for map
            * $("#container").ejMap({
            *    zoomedIn: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            zoomedIn: "",

            /**     
            * Fires once map render completed.
            * @event
            * @name ejMap#zoomedIn		
            * @example 
            * //zoomedIn event for map
            * $("#container").ejMap({
            *    zoomedIn: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            onRenderComplete: "",

            /**     
            * Fires during panning of map.
            * @event
            * @name ejMap#panning		
            * @example 
            * //panning event for map
            * $("#container").ejMap({
            *    panning: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            panning: "",

            /**     
            * Fires once panning ends.
            * @event
            * @name ejMap#panned		
            * @example 
            * //panned event for map
            * $("#container").ejMap({
            *    panned: function () {}
            * });
            * @memberof ejMap
            * @instance
            */

            panned: "",
            /**     
            * Fires on zoomed out of map.
            * @event
            * @name ejMap#zoomedOut		
            * @example 
            * //zoomedOut event for map
            * $("#container").ejMap({
            *    zoomedOut: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            zoomedOut: "",

            /**     
            * Fires on shape hovered in map.
            * @event
            * @name ejMap#shapeHovered		
            * @example 
            * //shapeHovered event for map
            * $("#container").ejMap({
            *    shapeHovered: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            shapeHovered: "",

            /**     
            * Fires on shape unhovered in map.
            * @event
            * @name ejMap#shapeUnHovered		
            * @example 
            * //shapeUnHovered event for map
            * $("#container").ejMap({
            *    shapeUnHovered: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            shapeUnHovered: "",


            // Layers Collection
            /**		
            * Specify layers of map.
            * @default []
            * @type {shapeFileLayer} 
            * @example  
            * // Set the layers during initialization. 			
            * 	$("#container").ejMap({layers: [{ mapType: "geometry", enableMouseHover: true, shapeSetting: { shapeStroke: "black", shapeFill: "#C3E6ED", mouseHoverColor: "#63B7B7", shapeSelectionColor: "#207BB2", shapeStrokeThickness: "0.5" }, data: Africa }]})
            * @example 
            * //Get or set the layer after initialization:<br>
            *   //Gets the layer from map.
            *   var layer =$("#container").data("ejMap").model.layers[layerIndex];
            *   //Sets the layer to map.
            *   $("#container").data("ejMap").model.layers[layerIndex]  = { mapType: "geometry", enableMouseHover: true, shapeSetting: { shapeStroke: "black", shapeFill: "#C3E6ED", mouseHoverColor: "#63B7B7", shapeSelectionColor: "#207BB2", shapeStrokeThickness: "0.5" }, data: Africa };
            * @memberof ejMap
            * @instance
            */
            layers: []
        },

        observables: ["dataSource"],

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            layers: "array"
        },

        /**
        * Destroy the map widget
        * @private
        */

        _destroy: function () {
            /// <summary>This function is  used to destroy the Map Object</summary>
            this._unWireEvents();
            if (_isSVG) {
                $(this.element).removeClass("e-map e-js").find("#svgDocument").remove();
            }
            else {
                $(this.element).removeClass("e-map e-js").find("#rootGroup").remove();
            }
        },

        /**
          * To configure the properties at runtime using SetModel		
          * @private
          */
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "zoomLevel":
                        if (this.model.enableZoom) {

                            this.zoom(options[prop]);

                        }
                        break;
                    case "baseMapIndex":
                        this.model.baseMapIndex = options[prop];
                        var map = this;
                        if (this.model.enableAnimation) {
                            this.model.enableLayerChangeAnimation = true;
                            $(this._mapContainer).animate({ opacity: 0 }, 500, function () {
                                map.refresh();
                            });

                        } else {
                            this.refresh();
                        }

                        break;
                    case "minZoom":
                        this.model.minZoom = options[prop];
                        if (this.model.zoomLevel < this.model.minZoom) {
                            this.zoom(this.model.minZoom);
                        }
                        break;
                    case "maxZoom":
                        this.model.maxZoom = options[prop];
                        if (this.model.zoomLevel > this.model.maxZoom) {
                            this.zoom(this.model.maxZoom);
                        }
                        break;
                    case "mapBackground":
                        this.model.background = options[prop];
                        $(this._mapContainer).css('background-image', this.model.background);
                }
            }
        },


        /**
        * Create the map widget
        * @private
        */
        _init: function () {
            var proxy = this;
            this._initPrivateProperties();
            $.each(this.model.layers, function (index) {
                proxy._layerInitialize(index);
            });
            this._mapContainer = this.element;
            $(this._mapContainer).css({ "overflow": "hidden" });
            var style = $('<style>.labelLegend {padding:3px;font-weight: normal;color:black;width:auto;height:20px;text-align:center;font-size: 14px;font-family:Segoe UI;}.labelLegend:hover{cursor:pointer;}</style>');
            $('html > head').append(style);
            this.refresh();
            this._trigger("onRenderComplete");
            this._isRendered = true;
            if (_isSVG) {
                $(this._svgDocument).pinchZoom(this._rootgroup, this);
            }
        },

        /**
      * Create layers for the map widget
      * @private
      */
        _layerInitialize: function (layerindex) {
            var proxy = this;
            if (this.model.layers[layerindex] != null) {
                $.each(this.model.layers, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = new shapeFileLayer();
                    $.extend(obj, element);
                    $.extend(element, obj);
                    $.each(element.subshapeFileLayers, function (subindex, subelement) {
                        subelement = proxy._checkArrayObject(subelement, subindex);
                        var subobj = new shapeFileLayer();
                        $.extend(subobj, subelement);
                        $.extend(subelement, subobj);
                    });
                });
            }
            else {

                this.layers[0] = new shapeFileLayer();

            }
        },

        /**
      * Merges the model values with default value
      * @private
      */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = new shapeFileLayer();
                    proxy._loadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

        /**
        * Loads individual default values
        * @private
        */
        _loadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._loadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },

        /**
        * Refresh the map with animation
        * @private
        */
        _refreshWithAnimation: function () {
            this.model.layers[this.model.baseMapIndex]._setMapItemsPositionWithAnimation(this);
            for (var key = 0; key < this.model.layers[this.model.baseMapIndex].subshapeFileLayers.length; key++) {
                this.model.layers[this.model.baseMapIndex].subshapeFileLayers[key]._setMapItemsPositionWithAnimation(this);
            }
        },

        /**
        * Reduces the shape stoke while zooming
        * @private
        */
        _resizeShape: function () {
            this.model.layers[this.model.baseMapIndex]._resizeShapes(this);
            for (var key = 0; key < this.model.layers[this.model.baseMapIndex].subshapeFileLayers.length; key++) {
                this.model.layers[this.model.baseMapIndex].subshapeFileLayers[key]._resizeShapes(this);
            }

        },

        /**
       * Refresh the shape file  layers
       * @private
       */
        _refrshLayers: function () {
            this.model.layers[this.model.baseMapIndex]._setMapItemsPosition(this);
            for (var key = 0; key < this.model.layers[this.model.baseMapIndex].subshapeFileLayers.length; key++) {
                this.model.layers[this.model.baseMapIndex].subshapeFileLayers[key]._setMapItemsPosition(this);
            }

        },

        /**
      * Pans the tile map
      * @private
      */
        _panTileMap: function (factorX, factorY, centerPosition) {

            var totalSize = Math.pow(2, this.model.zoomLevel) * 256;
            this._tileTranslatePoint.x = (factorX / 2) - (totalSize / 2);
            this._tileTranslatePoint.y = (factorY / 2) - (totalSize / 2);
            var position = this._convertTileLatLongtoPoint(centerPosition[0], centerPosition[1]);
            this._tileTranslatePoint.x -= position.x - (factorX / 2);
            this._tileTranslatePoint.y -= position.y - (factorY / 2);

        },

        /**
     * Generates tiles
     * @private
     */
        _generateTiles: function (zoomLevel) {
            this.Tiles = [];
            var xcount, ycount;
            xcount = ycount = Math.pow(2, zoomLevel);
            var endY = Math.min(ycount, ((-this._tileTranslatePoint.y + this._height) / 256) + 1);
            var endX = Math.min(xcount, ((-this._tileTranslatePoint.x + this._width) / 256) + 1);
            var startX = (-(this._tileTranslatePoint.x + 256) / 256);
            var startY = (-(this._tileTranslatePoint.y + 256) / 256);
            for (var i = Math.round(startX) ; i < Math.round(endX) ; i++) {
                for (var j = Math.round(startY) ; j < Math.round(endY) ; j++) {
                    var x = 256 * i + this._tileTranslatePoint.x;
                    var y = 256 * j + this._tileTranslatePoint.y;
                    if (x > -256 && x <= this._width && y > -256 && y < this._height) {
                        if (i >= 0 && j >= 0) {
                            var tile = new Tile(i, j);
                            tile.left = x;
                            tile.top = y;
                            tile.src = this._urlTemplate.replace("level", this.model.zoomLevel).replace('tileX', tile.X).replace('tileY', tile.Y);
                            this.Tiles.push(tile);
                        }
                    }


                }
            }
            var proxTiles = $.extend(true, [], this.Tiles);
            for (var layerIndex = 0; layerIndex < this.model.layers[this.model.baseMapIndex].subshapeFileLayers.length; layerIndex++) {
                var layer = this.model.layers[this.model.baseMapIndex].subshapeFileLayers[layerIndex];
                if (layer.mapType == 'osm') {
                    for (var tileIndex = 0; tileIndex < proxTiles.length; tileIndex++) {
                        var baseTile = proxTiles[tileIndex];
                        var subtile = $.extend(true, {}, baseTile);
                        subtile.src = layer.urlTemplate.replace("level", this.model.zoomLevel).replace('tileX', baseTile.X).replace('tileY', baseTile.Y);
                        this.Tiles.push(subtile);
                    }

                }
            }
            this._arrangeTiles();
        },

        /**
        * Arrange tiles
        * @private
        */
        _arrangeTiles: function () {
            var pathTemplate = "<div><div style='position:absolute;left:{{:left}}px;top:{{:top}}px;height:{{:height}}px;width:{{:width}}px;'><img src={{:src}}></img></div></div>";
            var tmpl2 = $.templates(pathTemplate);
            var htmlString = tmpl2.render(this.Tiles);
            var first = this._mapContainer[0].children[0];
            this._tileDiv.html(htmlString);
        },

        /**
        * Generated shape path
        * @private
        */
        _generatePath: function () {

            var baseLayer = this.model.layers[this.model.baseMapIndex];
            var pathCollection = '';
            this._polylineCount = 0;
            this._pointscount = 0;
            _isSVG = (window.SVGSVGElement) ? true : false;
            if (baseLayer.mapType == 'geometry') {
                if (baseLayer != undefined && baseLayer.data != null) {
                    baseLayer._isBaseLayer = true;
                    this._scale = this.model.zoomLevel;
                    pathCollection = this._getPathCollection(baseLayer);
                }
            }
            else {
                this._isTileMap = true;
                this._scale = Math.pow(2, this.model.zoomLevel - 1);
            }
            for (var key = 0; key < baseLayer.subshapeFileLayers.length; key++) {
                if (baseLayer.subshapeFileLayers[key].mapType == 'geometry')
                    pathCollection += this._getPathCollection(baseLayer.subshapeFileLayers[key]);
            }
            if (_isSVG) {
                var htmlstring = '<div style="position:relative;" id="tileDiv"/><svg xmlns="http://www.w3.org/2000/svg" style= "overflow:hidden;z-index:0;float:left;left:0,top:0"' +
                             'id="svgDocument"> <g id="rootGroup">' + pathCollection + '</g></svg>';
                this._mapContainer.html(htmlstring);
                this._svgDocument = this.element.find("#svgDocument")[0];
                this._tileDiv = this.element.find("#tileDiv");
                $(this._svgDocument).css({ height: this._height, width: this._width });
            }
            else {
                document.createStyleSheet().addRule(".vml", "behavior:url(#default#VML);display:inline-block");
                document.namespaces.add('v', 'urn:schemas-microsoft-com:vml', "#default#VML");
                var vmlGroup = '<div id="tileDiv"/><v:group   id = "rootGroup" style="position:absolute; width:' + this._width + 'px;height:' + this._height + 'px;" coordorigin = "0 0" coordsize="' + this._width + ' ' + this._height + '">' + pathCollection + '</v:group>';
                this._mapContainer.html(vmlGroup);
                this._tileDiv = this.element.find("#tileDiv");
                this._bubblegroup = this._createGroup(false, "bubbleGroup");
                this._bubblegroup.style.position = "absolute";
                this.element.append(this._bubblegroup);
            }
            if (baseLayer.mapType == 'osm') {
                this._urlTemplate = baseLayer.urlTemplate;
                this._panTileMap(this._width, this._height, this.model.centerPosition);
                this._generateTiles(this.model.zoomLevel);
            }
            $(this._mapContainer).css({ height: this._height, width: this._width });
            if (!_isSVG) {
                $(this._mapContainer).css("position", "relative");
            }
            this._rootgroup = this.element.find("#rootGroup")[0];
        },

        /**
        * Generates path collection
        * @private
        */
        _getPathCollection: function (layer) {
            var shapePath = "";
            var dataManager = new ej.DataManager(layer.data.features);
            if (layer._isBaseLayer) {
                this._mapBounds = layer.data.bbox;
            }
            var queryPolyLine = ej.Query().from(layer).where("geometry.type", ej.FilterOperators.equal, 'LineString');
            var queryPolygon = ej.Query().from(layer).where("geometry.type", ej.FilterOperators.equal, 'Polygon');
            var queryMulitiPolygon = ej.Query().from(layer).where("geometry.type", ej.FilterOperators.equal, 'MultiPolygon');
            var polylineDatas = dataManager.executeLocal(queryPolyLine);
            var polygonDatas = dataManager.executeLocal(queryPolygon);
            var queryPoint = ej.Query().from(layer).where("geometry.type", ej.FilterOperators.equal, 'Point');
            var pointData = dataManager.executeLocal(queryPoint);
            var multiPolygonDatas = dataManager.executeLocal(queryMulitiPolygon);
            this._polylineCount = this._polylineCount + polylineDatas.length;
            this._pointscount = this._pointscount + pointData.length;
            $.merge(polygonDatas, multiPolygonDatas)
            layer._newBounds = [];
            var pointHtml = "";
            if (pointData.length > 0) {
                for (var i = 0; i < pointData.length; i++) {
                    var currentData = pointData[i];
                    var check = '';
                    subData = currentData.geometry.coordinates;
                    var data = [];
                    var data_clean = [];
                    data.push({
                        lat: parseFloat(subData[1]),
                        lng: parseFloat(subData[0])
                    });
                    var l = 0;

                    var point = data[0];
                    latitude = point.lat; // (φ)
                    longitude = point.lng;   // (λ)                   
                    var p = this._convertTileLatLongtoPointForShapes(latitude, longitude, this._mapBounds);
                    if (_isSVG) {
                        data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                    }
                    else {
                        data_clean.push({ x: p.x - ((10 / this._scale) / 2), y: p.y - ((10 / this._scale) / 2), lat: point.lat, lng: point.lng });
                    }

                    if (_isSVG) {
                        pointHtml += '<circle cx="' + data_clean[0].x + '" cy="' + data_clean[0].y + '" r="' + 5 / this._scale + '" stroke="' + layer.shapeSetting.shapeStroke + '" stroke-width="' + layer.shapeSetting.shapeStrokeThickness + '0" fill="' + layer.shapeSetting.shapeFill + '" />';
                    }
                    else {
                        var pointHtmlString = '<v:oval display="block" fill="' + layer.shapeSetting.shapeFill + '" style="position:absolute;top: ' + data_clean[0].y + 'px;left:' + data_clean[0].x + 'px;width:' + 10 / this._scale + 'px;height:' + 10 / this._scale + 'px;stroke-width:' + layer.shapeSetting.shapeStrokeThickness + 'px;"></v:oval>';
                        pointHtml += pointHtmlString;
                    }
                }

            }

            if (polygonDatas.length > 0) {
                var minX, maxX, minY, maxY, flag = true;
                for (var i = 0; i < polygonDatas.length; i++) {
                    var currentData = polygonDatas[i];
                    var midCoordinate = 0;
                    var midCoordinateLength = currentData.geometry.coordinates[0][0].length;
                    var currentDataLength = currentData.geometry.coordinates.length;
                    if (currentDataLength > 1) {
                        var lst = currentData.geometry.coordinates;
                        for (var index = 0; index < lst.length; index++) {
                            if (midCoordinateLength < lst[index][0].length) {
                                midCoordinateLength = lst[index][0].length;
                                midCoordinate = index;
                            }
                        }
                    }
                    var path_code = '';
                    for (var j = 0; j < currentDataLength; j++) {
                        var subData = '';
                        if (currentDataLength > 1) {
                            subData = currentData.geometry.coordinates[j][0];
                            if (subData.length <= 2) {
                                subData = currentData.geometry.coordinates[j];
                            }
                        }
                        else {
                            subData = currentData.geometry.coordinates[j];
                        }
                        var data = [];
                        var data_clean = [];
                        if (subData.length > 2) {
                            for (var k = 0; k < subData.length; k++) {
                                data.push({
                                    lat: parseFloat(subData[k][1]),
                                    lng: parseFloat(subData[k][0])
                                });
                            }
                        }
                        else {
                            data.push({
                                lat: parseFloat(subData[1]),
                                lng: parseFloat(subData[0])
                            });
                        }
                        for (var l = 0; l < data.length - 1 || (data.length == 1 && l == 0) ; l++) {
                            var point = data[l];
                            latitude = point.lat; // (φ)
                            longitude = point.lng;   // (λ                           
                            var boundingBox = this._mapBounds;
                            if (this._isTileMap || (longitude >= boundingBox[0][0] && longitude <= boundingBox[1][0] && latitude >= boundingBox[0][1] && latitude <= boundingBox[1][1])) {

                                var p = this._convertTileLatLongtoPointForShapes(latitude, longitude, boundingBox);
                                if (flag) {
                                    minX = maxX = p.x;
                                    minY = maxY = p.y;
                                    flag = false;
                                }
                                else {
                                    minX = Math.min(minX, p.x);
                                    maxX = Math.max(maxX, p.x);
                                    minY = Math.min(minY, p.y);
                                    maxY = Math.max(maxY, p.y);
                                }
                                if (_isSVG) {
                                    data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                                }
                                else {
                                    data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                                }
                            }

                        }
                        if (j == midCoordinate) {
                            layer._newBounds.push(this._findMidPointofPoylgon(data_clean));
                        }

                        if (data_clean.length > 0) {
                            if (_isSVG) {
                                path_code += "M" + (data_clean[0].x) + "," + (data_clean[0].y);
                                for (m = 1; m < data_clean.length; m++) {
                                    path_code += "," + (data_clean[m].x) + "," + (data_clean[m].y);
                                }
                                path_code += " Z";
                            }
                            else {
                                path_code += "m" + parseInt(data_clean[0].x) + "," + parseInt(data_clean[0].y);
                                path_code += " l" + parseInt(data_clean[0].x) + "," + parseInt(data_clean[0].y);
                                for (m = 1; m < data_clean.length; m++) {
                                    path_code += "," + parseInt(data_clean[m].x) + "," + parseInt(data_clean[m].y);
                                }
                                path_code += " e";
                            }
                        }

                    }
                    if (path_code != "") {
                        if (_isSVG) {
                            shapePath += '<path class="mapShape" stroke=' + layer.shapeSetting.shapeStroke +
                                        ' stroke-width=' + (layer.shapeSetting.shapeStrokeThickness / this._scale) +
                                         ' fill= ' + layer.shapeSetting.shapeFill + ' d="' + path_code + '"  stroke-linejoin= round stroke-linecap= square></path>';
                        }
                        else {
                            shapePath += '<v:shape  style="width:' + this._width + 'px;height:' + this._height + 'px;"  coordsize="'
                                + this._width + ' ' + this._height + '" coordorigin="0 0" strokecolor=' + layer.shapeSetting.shapeStroke +
                                           ' stroke-width=' + (layer.shapeSetting.shapeStrokeThickness / this._scale) + 'px"' +
                                            ' fillcolor= ' + layer.shapeSetting.shapeFill + ' path="' + path_code + '"></v:shape>';
                        }
                    }
                    else {
                        polygonDatas.splice(i, 1);
                        i--;
                    }

                }

                this._groupSize = { minX: minX, maxX: maxX, minY: minY, maxY: maxY };

            }

            if (polylineDatas.length > 0) {

                for (var i = 0; i < polylineDatas.length; i++) {
                    var coordinates = polylineDatas[i].geometry.coordinates;
                    var linedata = '';
                    for (var k = 0; k < coordinates.length; k++) {
                        var points = this._convertTileLatLongtoPointForShapes(coordinates[k][1], coordinates[k][0], this._mapBounds);
                        linedata += points.x + "," + points.y;
                        if (k != coordinates.length - 1)
                            linedata += ",";
                    }

                    if (_isSVG) {
                        shapePath += '<polyline stroke=' + layer.shapeSetting.shapeStroke +
                                    ' stroke-width=' + (layer.shapeSetting.shapeStrokeThickness / this._scale) + ' fill="transparent" stroke-width="1" points="' + linedata + '" ></polyline>';

                    }
                    else {
                        shapePath += '<v:polyline coordsize="' + this._width + ',' + this._height + '" coordorigin="0 0" strokecolor=' + layer.shapeSetting.shapeStroke +
                                                               ' strokeweight=' + (layer.shapeSetting.shapeStrokeThickness / this._scale) + 'px"' +
                                                                ' fillcolor=' + layer.shapeSetting.shapeFill + '  points="' + linedata + '"/>';


                    }
                }

            }
            var lines = [].concat(polygonDatas, polylineDatas);
            layer._polygonData = $.merge(lines, pointData);
            return shapePath + pointHtml;
        },


        /**
         * Creates VML group
         * @private
         */
        _createGroup: function (isRoot, name) {

            var vmlGroup;
            vmlGroup = document.createElement('<v:group class=' + name + '>');
            vmlGroup.style.width = this._width + 'px';
            vmlGroup.style.height = this._height + 'px';
            vmlGroup.coordorigin = "0 0";
            vmlGroup.coordsize = this._width + ' ' + this._height;
            if (isRoot) {
                this._rootgroup = vmlGroup;
                vmlGroup.style.left = '20px';
                vmlGroup.style.top = '20px';
            }
            return vmlGroup;
        },

        /**
        * Generates palette colors for shape
        * @private
        */
        _generatePaletteColorsForShape: function (shape, shapelayer, palette, palettesettings) {

            if (palettesettings != null) {
                shapelayer.shapeSetting.mouseHoverColor = palettesettings.MouseHoverColor;
                shapelayer.shapeSetting.mouseHoverStroke = palettesettings.MouseHoverStroke;
                shapelayer.shapeSetting.mouseHoverWidth = palettesettings.MouseHoverStrokeWidth;
                shapelayer.shapeSetting.shapeSelectionColor = palettesettings.SelectionColor;
                shapelayer.shapeSetting.shapeSelectionStroke = palettesettings.SelectionStroke;
                shapelayer.shapeSetting.shapeSelectionStrokeWidth = palettesettings.SelectionStrokeWidth;
            }

            if (_isSVG) {
                shape.setAttribute("fill", palette[shapelayer._prevPaletteIndex]);
            }
            else {
                shape.fillcolor = palette[shapelayer._prevPaletteIndex];
            }
            shapelayer._prevPaletteIndex = shapelayer._prevPaletteIndex + 1;
            if (shapelayer._prevPaletteIndex > palette.length - 1)
                shapelayer._prevPaletteIndex = 0;
        },

        /**
        * Renders layers
        * @private
        */
        _renderLayers: function (layer, pathCount) {
            layer._initializeLocalValues();
            var proxyrootGroup = this._rootgroup;
            var proxySVG = this._svgDocument;
            var proxyControl = this;
            var shapelayer = layer;
            if (shapelayer.legendSetting != null && shapelayer.shapeSetting.colorMappings != null)
                shapelayer._generateLegends(this);
            if (shapelayer.shapeSetting.colorMappings != null && shapelayer.shapeSetting.colorMappings.rangeColorMapping != null) {

                shapelayer.shapeSetting.colorMappings.rangeColorMapping.sort(this._orderByNameAscending);
            }
            if (shapelayer.bubbleSetting.colorMappings != null && shapelayer.bubbleSetting.colorMappings.rangeColorMapping != null) {
                shapelayer.bubbleSetting.colorMappings.rangeColorMapping.sort(this._orderByNameAscending);
            }
            var minValue = 0;
            var maxValue = 0;
            var sum = 0;
            if (shapelayer.dataSource != null && shapelayer.bubbleSetting != null) {
                if (shapelayer.bubbleSetting.bubbleColorValuepath == null) {
                    shapelayer.bubbleSetting.bubbleColorValuepath = shapelayer.bubbleSetting.bubbleValuepath;
                }
                for (var i = 0; i < shapelayer.dataSource.length; i++) {
                    var bubbledata = parseFloat(this._reflection(shapelayer.dataSource[i], shapelayer.bubbleSetting.bubbleValuepath));
                    if (i != 0) {
                        if (bubbledata > maxValue) {
                            maxValue = bubbledata;
                        } else if (bubbledata < minValue) {
                            minValue = bubbledata;
                        }
                    } else {
                        minValue = maxValue = bubbledata;
                    }
                }
            }

            this._generateTooltipForLayers(shapelayer);
            if (shapelayer.data != null) {
                var shapesource = [];
                for (var key in shapelayer.dataSource) {
                    var shapeitem = shapelayer.dataSource[key];
                    var shapeItemValue = proxyControl._reflection(shapeitem, shapelayer.shapeIdPath);
                    shapesource.push(shapeItemValue);
                }
                for (var dataIndex = 0; dataIndex < shapelayer._polygonData.length; dataIndex++) {
                    var ValueObject = shapelayer._polygonData[dataIndex].properties;
                    var DBFValue = proxyControl._reflection(ValueObject, shapelayer.shapeIdTableField);
                    var shape = this._rootgroup.childNodes[dataIndex + pathCount];
                    var obj = {};
                    var bounds = shapelayer._newBounds[dataIndex];

                    if (shapelayer.shapeSetting.autoFill) {
                        switch (shapelayer.shapeSetting.shapeColorPalette) {
                            case 'palette1':
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette1, shapelayer._colorPaletteSettings.Palette1);
                                break;
                            case 'palette2':
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette2, shapelayer._colorPaletteSettings.Palette2);
                                break;
                            case 'palette3':
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette3, shapelayer._colorPaletteSettings.Palette3);
                                break;
                            case 'custompalette':
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.shapeSetting.customPalette, null);
                                break;
                        }
                    }

                    if (shapesource.indexOf(DBFValue) != -1) {
                        var item = shapelayer.dataSource[shapesource.indexOf(DBFValue)];
                        var ItemValue = DBFValue;
                        if (item != null) {
                            if (shape != null) {

                                var id = ItemValue;
                            }
                            layer._bounds.push(bounds);
                            if (shapelayer.showMapItems) {
                                var itemTemplateDiv = $('.itemTemplateDiv');
                                itemTemplateDiv = $("<div style='display:block;position:absolute;pointer-events: none;'></div>");
                                itemTemplateDiv[0].className = ItemValue;
                                proxyControl._templateDiv.append(itemTemplateDiv);
                                $(itemTemplateDiv).css({ "left": bounds.x, "top": bounds.y });
                                var htmlString;
                                if (shapelayer.mapItemsTemplate == null) {
                                    htmlString = $(' <div><label>{{:' + shapelayer.shapeSetting.shapeValuepath + '}}</label></div>').render(item);;
                                }
                                else {
                                    htmlString = $("#" + shapelayer.mapItemsTemplate).render(item);
                                }
                                $(itemTemplateDiv).html(htmlString);
                                layer._mapItems.push(itemTemplateDiv);
                            }

                            if (shapelayer.shapeSetting.colorMappings != null) {
                                if (shapelayer.shapeSetting.shapeColorValuepath == null) {
                                    shapelayer.shapeSetting.shapeColorValuepath = shapelayer.shapeSetting.shapeValuepath;
                                }
                                var shapeValue = proxyControl._reflection(item, shapelayer.shapeSetting.shapeColorValuepath);
                                if (shapeValue != null && !shapelayer.shapeSetting.autoFill) {
                                    if (((shapelayer.legendSetting != null && shapelayer.legendSetting.legendMode == 'interactive') || shapelayer.shapeSetting.enableGradient) && shapelayer.shapeSetting.colorMappings.rangeColorMapping != null) {
                                        obj["legendrect"] = shapelayer._updateLegendRange(shapeValue, shapelayer, shape);
                                        var shapeColor = shapelayer.shapeSetting.shapeFill;
                                        if (obj["legendrect"] != undefined) {
                                            shapeColor = obj["legendrect"].color;
                                        }
                                        if (_isSVG) {
                                            shape.setAttribute("fill", shapeColor);
                                        }
                                        else {
                                            shape.fillcolor = shapeColor;
                                        }
                                    } else {
                                        shapelayer._fillColors(shapeValue, shapelayer.shapeSetting.colorMappings, shape);
                                    }
                                }
                            }
                            else if (!shapelayer.shapeSetting.autoFill) {
                                if (_isSVG) {
                                    shape.setAttribute("fill", shapelayer.shapeSetting.shapeFill);
                                }
                                else {
                                    shape.fillcolor = shapelayer.shapeSetting.shapeFill;
                                    shape.strokecolor = shapelayer.shapeSetting.shapeStroke;
                                }
                            }

                            if (shapelayer.bubbleSetting != null && shapelayer.bubbleSetting.bubbleValuepath != null) {
                                if (_isSVG) {
                                    var circle = document.createElementNS(proxyControl._svgns, "circle");
                                }
                                else {
                                    var bubbleID = "bubble_" + id;
                                    var bubbleHtmlString = '<v:oval id=' + bubbleID + ' display="block" style="position:absolute;top: ' + 0 + 'px;left:' + 0 + 'px;width:100px;height:100px;stroke-width:' + 0 + 'px;"><v:fill opacity="0.9"/></v:oval>';
                                    this._bubblegroup.innerHTML = this._bubblegroup.innerHTML + bubbleHtmlString;
                                    var circle = document.getElementById("bubble_" + id);
                                }

                                var bubbleValue = proxyControl._reflection(item, shapelayer.bubbleSetting.bubbleValuepath);
                                var bubblecolorValue = proxyControl._reflection(item, shapelayer.bubbleSetting.bubbleColorValuepath);
                                if (shapelayer.bubbleSetting.colorMappings != null) {
                                    shapelayer._fillColors(bubblecolorValue, shapelayer.bubbleSetting.colorMappings, circle);
                                }
                                else {
                                    if (_isSVG) {
                                        circle.setAttribute("fill", shapelayer.bubbleSetting.bubbleColor);
                                    }
                                    else {
                                        circle.strokecolor = shapelayer.bubbleSetting.bubbleColor;
                                        circle.fillcolor = shapelayer.bubbleSetting.bubbleColor;
                                    }
                                }
                                var radius = proxyControl._getRatioOfBubble(shapelayer.bubbleSetting.minSize, shapelayer.bubbleSetting.maxSize, bubbleValue, minValue, maxValue);
                                if (_isSVG) {
                                    $(circle).attr({
                                        "cx": bounds.x, "cy": bounds.y,
                                        "fill-opacity": "0.9", "r": radius
                                    });
                                    if (proxyControl.model.enableAnimation) {
                                        $(circle).css("display", "none");
                                    }
                                }
                                else {
                                    $(circle).css({
                                        "height": +2 * radius + "px",
                                        "width": +2 * radius + "px"
                                    });
                                }
                                if (shapelayer.bubbleSetting.showToolTip == true) {
                                    var tooltipTemplateDiv = $('.tooltipTemplateDiv');
                                    tooltipTemplateDiv = $("<div style='position:absolute;display:none;z-index:9999'/>");
                                    var template_ID = null;
                                    if (shapelayer.bubbleSetting.tooltipTemplate != null) {
                                        template_ID = shapelayer.bubbleSetting.tooltipTemplate;
                                    } else {
                                        $(tooltipTemplateDiv).append('<div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">' + bubbleValue + '</label></p></div></div>');
                                    }
                                    $(circle).mouseenter({ htmlobj: tooltipTemplateDiv, templateID: template_ID, itemsrc: item }, proxyControl._bubbleEnterFunction);
                                    $(circle).mousemove({ htmlobj: tooltipTemplateDiv }, proxyControl._bubbleOverFunction);
                                    $(circle).mouseleave({ htmlobj: tooltipTemplateDiv }, proxyControl._bubbleleaveFunction);
                                } else {
                                    $(circle).css("pointer-events", "none");
                                }
                                if (_isSVG) {
                                    proxySVG.appendChild(circle);
                                    layer._bubbles.push(circle);
                                }
                                else {
                                    layer._bubbles.push(circle);
                                }
                            }
                            obj["data"] = item;
                        }
                    }
                    if (layer.labelSetting != null && layer.labelSetting.showLabels && layer.labelSetting.labelPath != null) {
                        var labelValue = proxyControl._reflection(ValueObject, layer.labelSetting.labelPath);
                        if (labelValue != null) {
                            if (bounds == null) {
                                //bounds = (shape);
                            }
                            var labelElement = $("<div class='labelStyle' style='font-family:Segoe UI;font-size:14px;white-space: nowrap;position:absolute;pointer-events:stroke;'></div>");
                            labelElement[0].innerHTML = labelValue;
                            this._templateDiv.append(labelElement);
                            $(labelElement).css({ "left": bounds.x, "top": bounds.y });
                            proxyControl._on($(labelElement), ej.eventType.mouseDown, { Param1: null, Param2: layer, Param3: shape }, proxyControl._polyClickFunction);
                            proxyControl._on($(labelElement), ej.eventType.mouseLeave, { Param1: layer, map: this }, proxyControl._polyLeaveFunction);
                            layer._labelCollection.push(labelElement);
                            layer._labelBounds.push(bounds);
                            layer._labelData.push(labelValue);

                        }
                    }
                    obj["shape"] = shape;
                    obj["shapeData"] = layer._polygonData[dataIndex];
                    if (shape != null) {
                        var shapefill = null;
                        if (_isSVG) {
                            shapefill = shape.getAttribute("fill");
                            if (shapefill == "undefined")
                                shapefill = layer.shapeSetting.shapeFill;
                            shape.setAttribute("nodeValue", shapefill);
                        }
                        else {
                            shapefill = shape.fillcolor.value;
                            if (shapefill == "undefined")
                                shapefill = layer.shapeSetting.shapeFill;
                            shape.style.behavior = shapefill;
                        }
                    }
                    layer._mapShapes.push(obj);
                    if (proxyControl.model.zoomOnSelection) {
                        $(shape).hover(function () {
                            $(this).css('cursor', 'pointer');
                        }, function () {
                            $(this).css('cursor', 'auto');
                        });
                    }
                    $(shape).mouseenter({ Param1: shapelayer, Param2: obj, map: proxyControl }, proxyControl._polyEnterFunction);
                    this._on($(shape), ej.eventType.mouseDown, { Param1: obj, Param2: shapelayer, Param3: shape }, proxyControl._polyClickFunction);
                    this._on($(shape), ej.eventType.mouseMove, { Param1: shapelayer, Param2: obj }, proxyControl._polyMoveFunction);
                    this._on($(shape), ej.eventType.mouseLeave, { Param1: shapelayer, Param2: obj, map: this }, proxyControl._polyLeaveFunction);
                    this._on($(shape), ej.eventType.mouseUp, { Param1: obj, Param2: shapelayer, Param3: shape }, proxyControl._polyUpFunction);

                }
            }
        },


        /**
        * Zooms in and out the map
        * @private
        */
        _zooming: function (delta, e) {
            var position = this._getCurrentPoint(e);
            if (delta >= 120) {
                this._zoomingIn(position.x, position.y, e);
            }
            else {
                this._zoomingOut(position.x, position.y, e);
            }
        },

        /**
        * Zooms in the map
        * @private
        */
        _zoomingIn: function (x, y, event) {
            var map = this;
            if (map.model.enableZoom && map.model.zoomLevel >= map.model.minZoom && map.model.zoomLevel <= map.model.maxZoom) {
                var scal = map._scale;
                var position = { x: x, y: y };
                map._prevScale = scal;
                if (!this._isTileMap) {
                    map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                    map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale + map.model.zoomFactor)) / (map._width / x);
                    map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale + map.model.zoomFactor)) / (map._height / y);
                    map._scale = scal + map.model.zoomFactor;
                    map.model.zoomLevel += 1;
                }
                else {
                    this._tileZoom(map.model.zoomLevel - 1, map.model.zoomLevel, position);
                    var prevLevel = map.model.zoomLevel;
                    map.model.zoomLevel += 1;
                    this._generateTiles(this.model.zoomLevel);
                    map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._scale = (Math.pow(2, prevLevel));
                }
                map._trigger("zoomedIn", { originalEvent: event, zoomLevel: map.model.zoomLevel });
                map._applyTransform(map._scale, map._translatePoint);
                map._refrshLayers();
                map._resizeShape();
                map._updateSliderValue();
            }
        },

        /**
        * Zooms out the map
        * @private
        */
        _zoomingOut: function (x, y, event) {
            var map = this;
            if (this.model.enableZoom) {
                if (this.model.zoomLevel > this.model.minZoom) {
                    var scal = map._scale;
                    var position = { x: x, y: y };
                    map._prevScale = scal;
                    if (!this._isTileMap) {
                        map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                        map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale - map.model.zoomFactor)) / (map._width / x);
                        map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale - map.model.zoomFactor)) / (map._height / y);
                        map._scale = map._scale - map.model.zoomFactor;
                        map.model.zoomLevel -= 1;
                    }
                    else {
                        this._tileZoom(map.model.zoomLevel + 1, map.model.zoomLevel, position);
                        var prevLevel = map.model.zoomLevel;
                        map.model.zoomLevel -= 1;
                        this._generateTiles(this.model.zoomLevel);
                        map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, this.model.zoomLevel - 1))) / (Math.pow(2, this.model.zoomLevel - 1));
                        map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, this.model.zoomLevel - 1))) / (Math.pow(2, this.model.zoomLevel - 1));
                        map._scale = (Math.pow(2, this.model.zoomLevel - 1));
                    }
                    map._applyTransform(map._scale, map._translatePoint);
                    map._refrshLayers();
                    map._resizeShape();

                    map._updateSliderValue();
                }
                else {
                    this.zoom(this.model.minZoom, false);
                }
                map._trigger("zoomedOut", { originalEvent: event, zoomLevel: map.model.zoomLevel });
            }
        },

        /**
        * Calculates the bubble size ratio
        * @private
        */
        _getRatioOfBubble: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var bubbleRadius = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
            return bubbleRadius;
        },

        /**
        * Gets property from object
        * @private
        */
        _reflection: function (Source, path) {
            var ShapeValues = Source;
            if (path != null && Source != null) {
                var parts = path.split(".");
                var i = 0;
                for (; i < parts.length; i++) {

                    if (Source != undefined) {
                        var hasProperty = ShapeValues.hasOwnProperty(parts[i]);
                        if (hasProperty) {
                            $.each(ShapeValues, function (prop, propval) {
                                if (prop == parts[i]) {
                                    ShapeValues = propval;
                                    return false;
                                }

                            });
                        }
                    }
                }
                return ShapeValues;
            }
            else {
                return null;
            }

        },

        /**
        * Generates annotations
        * @private
        */
        _generateAnnotations: function () {
            var index = this.model.baseMapIndex;
            this.model.layers[index]._generateAnnotationForLayers(this);
            for (var key = 0; key < this.model.layers[index].subshapeFileLayers.length; key++) {
                (this.model.layers[index].subshapeFileLayers[key])._generateAnnotationForLayers(this);
            }
        },

        /**
        * Generates tooltip for layers
        * @private
        */
        _generateTooltipForLayers: function (layer) {
            if (layer.showToolTip) {
                var tooltipTemplateDiv = $('.tooltipTemplateDiv');
                tooltipTemplateDiv = $("<div style='z-index:1000;display:none;position:absolute;pointer-events:none;'></div>");
                $(document.body).append(tooltipTemplateDiv);
                layer._tooltipElement = tooltipTemplateDiv;
                if (layer.tooltipTemplate == null) {
                    var path = layer.shapeSetting.shapeValuepath;
                    if (path != null) {
                        layer.tooltipTemplate = 'defaultTooltip';
                        this._mapContainer.append($('<div id="defaultTooltip" style="display:none;"><div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">{{:' + path + '}}</label></p></div></div></div>'));
                    }
                }
            }
        },

        /**
        * Sorts in ascending order
        * @private
        */
        _orderByNameAscending: function (a, b) {
            if (a.Range == b.Range) {
                return 0;
            } else if (a.Range > b.Range) {
                return 1;
            }
            return -1;
        },

        /**
        * Resizes the container
        * @private
        */
        _resizingContainer: function () {
            this._scale = this._isTileMap ? Math.pow(2, this.model.zoomLevel - 1) : this.model.zoomLevel;
            if (this._isTileMap) {
                this._translatePoint.x = this._tileTranslatePoint.x / this._scale;
                this._translatePoint.y = this._tileTranslatePoint.y / this._scale;
            }
            else {
                this._translatePoint.x = this._tileTranslatePoint.x / this._scale;
                this._translatePoint.y = this._tileTranslatePoint.y / this._scale;
            }
            this._applyTransform(this._scale, this._translatePoint);
            this._baseScale = this._scale;
            this._baseTranslatePoint = { x: (this._translatePoint.x * 2), y: (this._translatePoint.y * 2) };

        },

        /**
       * Calculates margin with conditions
       * @private
       */
        _calculateMarginConditions: function () {
            var trans = this._calculateMinMax();
            if (this._translatePoint.y > trans.maxY) {
                this._translatePoint.y = trans.maxY;
            }
            else if (this._translatePoint.y < trans.minY) {
                this._translatePoint.y = trans.minY;
            }
            if (this._translatePoint.x > trans.maxX) {
                this._translatePoint.x = trans.maxX;
            }
            else if (this._translatePoint.x < trans.minX) {
                this._translatePoint.x = trans.minX;
            }
        },

        /**
       * Calculates minimum and maximum of map boundaries
       * @private
       */
        _calculateMinMax: function () {
            var bounds = this._groupBorder;
            var maximumTransX, maximumTransY, minimumTransX, minimumTransY;
            if (this._containerHeight * this._scale <= this._height) {
                maximumTransY = (this._height - this._containerHeight * this._scale) / (2 * this._scale) - bounds.y;
                minimumTransY = (this._height - this._containerHeight * this._scale) / (2 * this._scale) - bounds.y;
            } else {
                maximumTransY = 0;
                minimumTransY = ((this._height - this._containerHeight * this._scale) / this._scale) - bounds.y;;
            }
            if (this._containerWidth * this._scale <= this._width) {
                maximumTransX = (this._width - this._containerWidth * this._scale) / (2 * this._scale) - bounds.x;
                minimumTransX = (this._width - this._containerWidth * this._scale) / (2 * this._scale) - bounds.x;
            } else {
                maximumTransX = 0;
                minimumTransX = ((this._width - this._containerWidth * this._scale) / this._scale) - bounds.x;
            }
            return { minX: minimumTransX, maxX: maximumTransX, minY: minimumTransY, maxY: maximumTransY };
        },


        /**
          * Calculates mid point of polygon
          * @private
          */
        _findMidPointofPoylgon: function (points) {
            var min = 0;
            var max = points.length;
            var initialPoint = points[0];
            var minX = initialPoint.x, maxX = initialPoint.x;
            var minY = initialPoint.y, maxY = initialPoint.y;
            var startX, startY, startX1, startY1, sum = 0, xSum = 0, ySum = 0;

            for (var i = min; i <= max - 1; i++) {
                var prevPoint = points[i];
                startX = prevPoint.x;
                startY = prevPoint.y;
                if (i == max - 1) {
                    startX1 = initialPoint.x;
                    startY1 = initialPoint.y;
                }
                else {
                    var nextPoint = points[i + 1];
                    startX1 = nextPoint.x;
                    startY1 = nextPoint.y;
                }
                sum = sum + Math.abs(((startX * startY1)) - (startX1 * startY));
                xSum = xSum + Math.abs(((startX + startX1) * (((startX * startY1) - (startX1 * startY)))));
                ySum = ySum + Math.abs(((startY + startY1) * (((startX * startY1) - (startX1 * startY)))));

            }
            sum = 0.5 * sum;
            xSum = (1 / (4 * sum)) * xSum;
            ySum = (1 / (4 * sum)) * ySum;
            return { x: xSum, y: ySum };


            //for (var i = min; i < max; i++) {
            //    minX = Math.min(minX, points[i].x);
            //    maxX = Math.max(maxX, points[i].x);
            //    minY = Math.min(minY, points[i].y);
            //    maxY= Math.max(maxY, points[i].y);
            //}          
            //return {x:minX+((maxX-minX)/2),y:(minY+(maxY-minY)/2)};            
        },


        /**
          * Converts tiles' latitude and longitude point for shapes
          * @private
          */
        _convertTileLatLongtoPointForShapes: function (lat, lng, bounds, vmlBound) {
            var Size;
            if (bounds == null) { bounds = [[-180, -85], [180, 85]] }
            if (this._isTileMap) {
                Size = Math.pow(2, 1) * 256;
            }
            else {
                var factor;
                if (this._height < this._width) {
                    var percent = ((bounds[1][1] - bounds[0][1]) / 180) * 100;
                    factor = this._height / ((percent * this._height) / 100);
                }
                else {
                    var percent = ((bounds[1][0] - bounds[0][0]) / 360) * 100;
                    factor = this._width / ((percent * this._width) / 100);
                }

                Size = Math.min(this._height, this._width) * factor;
            }

            var x = (lng + 180) / 360;
            var sinLatitude = Math.sin(lat * Math.PI / 180);
            if (sinLatitude == -1)
                sinLatitude = -0.5;
            var y = 0.5 - Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4 * Math.PI);
            var pixelX = (x * Size + 0.5);
            var pixelY = (y * Size + 0.5);
            if (vmlBound != null) {
                pixelX -= vmlBound.x;
                pixelY -= vmlBound.y;
            }
            return { x: pixelX, y: pixelY };


        },

        /**
          * Converts tiles' latitude and longitude point
          * @private
          */
        _convertTileLatLongtoPoint: function (lat, lng) {
            var Size = Math.pow(2, this.model.zoomLevel) * 256;
            var x = (lng + 180) / 360;
            var sinLatitude = Math.sin(lat * Math.PI / 180);
            var y = 0.5 - Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4 * Math.PI);
            var pixelX = (x * Size + 0.5) + this._tileTranslatePoint.x;
            var pixelY = (y * Size + 0.5) + this._tileTranslatePoint.y;
            return { x: pixelX, y: pixelY };
        },

        /**
         * Converts tiles' latitude and longitude point for Annotation
         * @private
         */
        _convertLatLongtoPointforAnnotation: function (lat, lng) {
            var point = this._convertTileLatLongtoPointForShapes(lat, lng, this._mapBounds);
            return { x: (point.x + this._translatePoint.x) * this._scale, y: (point.y + this._translatePoint.y) * this._scale };

        },

        /**
        * Animates map
        * @private
        */
        _animate: function (animateduration) {
            this._calculateMarginConditions();
            $(this._rootgroup).stop(true, false);
            if (this._sliderControl != null) {
                $(this._sliderControl).stop(true, false);
            }
            var map = this;
            var currentTranformation = { x: this._translatePoint.x, y: this._translatePoint.y };
            var startPoint = [this._prevPoint.x, this._prevPoint.y];
            var endPoint = [this._translatePoint.x, this._translatePoint.y];

            function slope(horizontal, vertical) {

                if (horizontal[0] == vertical[0]) {
                    return null;
                }
                return (vertical[1] - horizontal[1]) / (vertical[0] - horizontal[0]);
            }

            function intercept(point, slopeValue) {
                if (slopeValue === null) {
                    return point[0];
                }
                return point[1] - slopeValue * point[0];
            }

            var slopeFactor = slope(startPoint, endPoint);
            var slopeIntersection = intercept(startPoint, slopeFactor);
            var horizontalDifference = endPoint[0] - startPoint[0];
            var verticalDifference = endPoint[1] - startPoint[1];
            var scaleDifference = this._scale - this._prevScale;
            var currentScale = this._scale;
            this._updateSliderValue();
            $(this._rootgroup).animate(
                {
                    count: 10
                },
                {
                    duration: animateduration,
                    step: function (now, fx) {
                        var scaleX = map._prevScale + ((now * (scaleDifference / fx.end)));
                        var transX = startPoint[0] + (now * (horizontalDifference / fx.end)) / (scaleX / currentScale);
                        var transY;
                        if (slopeFactor == null) {
                            transY = startPoint[1] + (now * (verticalDifference / fx.end));
                        } else {
                            transY = ((slopeFactor * transX) + slopeIntersection);
                        }
                        if (_isSVG) {
                            map._rootgroup.setAttribute('transform', 'scale(' + scaleX + ') translate(' + transX + ', ' + transY + ')');
                        }
                        else {
                            map._rootgroup.coordorigin = -transX + ',' + -transY;
                            map._rootgroup.coordsize = (map._width / scaleX) + ',' + (map._height / scaleX);
                        }
                        map._translatePoint.x = transX;
                        map._translatePoint.y = transY;
                        map._scale = scaleX;
                        map._refrshLayers();
                        map._resizeShape();
                    },
                    complete: function () {
                        map._translatePoint.x = currentTranformation.x;
                        map._translatePoint.y = currentTranformation.y;
                        map._scale = currentScale;
                        this.count = 0;
                    }
                }
            );

        },

        /**
        * Applies transforms to rootgroup
        * @private
        */
        _applyTransform: function (scale, translatePoint) {
            this._scale = scale;
            this._translatePoint = translatePoint;
            if (!this._isTileMap)
                this._calculateMarginConditions();
            if (_isSVG) {
                this._rootgroup.setAttribute('transform', 'scale(' + this._scale + ') translate(' + this._translatePoint.x + ', ' + this._translatePoint.y + ')');
            }
            else {
                this._rootgroup.coordorigin = (-this._translatePoint.x) + ',' + (-this._translatePoint.y);
                this._rootgroup.coordsize = (this._width / scale) + ',' + (this._height / scale);
            }

        },

        /**
         * Section For handling the mouse move event
		 * @private
         */
        _mouseMove: function (event) {
            if (event.type == "touchmove" && event.originalEvent.touches.length > 1) {
                this._isPinching = true;
            }
            else {
                var map = this;
                if (map.ispanning && map.model.enablePan) {
                    event.preventDefault();
                    var position;
                    if (event.type == "mousemove")
                        position = { x: event.pageX, y: event.pageY };
                    else if (event.type == "touchmove")
                        position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                    else if (event.type == "MSPointerMove")
                        position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                    if (this._isTileMap) {
                        var curtileX = this._tileTranslatePoint.x - (this._dragStartX - position.x);
                        var curtileY = this._tileTranslatePoint.y - (this._dragStartY - position.y);
                        this._tileTranslatePoint.x = curtileX;
                        this._tileTranslatePoint.y = curtileY;
                        this._generateTiles(this.model.zoomLevel);
                    }
                    var curX = map._translatePoint.x - (map._dragStartX - position.x) / map._scale;
                    var curY = map._translatePoint.y - (map._dragStartY - position.y) / map._scale;
                    if (curX != map._translatePoint.x || curY != map._translatePoint.y) {
                        map._isDragging = true;
                    }
                    this._translatePoint.x = curX;
                    this._translatePoint.y = curY;
                    map._applyTransform(map._scale, map._translatePoint);
                    map._dragStartX = position.x;
                    map._dragStartY = position.y;
                    map._refrshLayers();
                    map._trigger("panning", { originalEvent: event });
                }
            }
        },

        /**
         * Section For handling the mouse leave event
		 * @private
         */
        _svgMouseLeave: function (event) {
            var map = this;
            map.ispanning = false;
        },

        /**
        * Section For handling the mouse wheel event
        * @private
        */
        _mouseWheel: function (event) {
            if (this.model.enableAnimation) {
                $(this._rootgroup).stop(true, false);
            }
            var e = event.originalEvent;
            var original = event;
            if (!_isSVG) {
                e = event;
            }
            e.preventDefault ? e.preventDefault() : original.preventDefault();
            if (_isSVG) {
                this._zooming(event.originalEvent.wheelDelta, event.originalEvent);
            }
            else {
                this._zooming(event.originalEvent.wheelDelta, e);
            }
        },

        /**
        * Section For handling the mouse button up event
        * @private
        */
        _mouseButtonUp: function (event) {

            var map = this;
            if (map.ispanning) {
                map._trigger("panned", { originalEvent: event });
            }
            map.ispanning = false;
        },

        /**
        * Section For handling the mouse up event
        * @private
        */
        _mouseUp: function (event) {
            this.ispanning = false;
            this._isDragging = false;
        },

        /**
        * Section For handling the mouse button down event
        * @private
        */
        _mouseButtonDown: function (event) {
            this._isPinching = false;
            if (event.target.className != "e-vhandle") {
                this._isNavigationPressed = false;
                if (this.model.enableAnimation) {
                    $(this._rootgroup).stop(true, false);
                    if (this._sliderControl != null) {
                        $(this._sliderControl).stop(true, false);
                    }
                }
                if (event.type == "touchstart" && event.originalEvent.touches.length > 1)
                { }
                else {
                    event.preventDefault();
                    var position;
                    if (event.type == "mousedown")
                        position = { x: event.pageX, y: event.pageY };
                    else if (event.type == "touchstart")
                        position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                    else if (event.type == "MSPointerDown")
                        position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                    var map = this;
                    map.ispanning = true;
                    map._dragStartX = position.x;
                    map._dragStartY = position.y;
                }
            }
        },

        /**
       * Section to get current point
       * @private
       */
        _getCurrentPoint: function (event) {
            var map = this._mapContainer;
            var xPos = event.pageX - map[0].offsetLeft;
            var yPos = event.pageY - map[0].offsetTop;
            return { x: xPos, y: yPos };
        },

        /**
        * Section For handling the double click event
        * @private
        */
        _doubleClick: function (event) {
            var map = this;
            if (!this._isNavigationPressed && event.target.className.toString().indexOf("e-icon1") == -1) {
                var position = this._getCurrentPoint(event);
                if (map.model.enableZoom && map.model.zoomLevel + 1 >= map.model.minZoom && map.model.zoomLevel + 1 <= map.model.maxZoom) {
                    var scal = map._scale;
                    map._prevScale = scal;
                    if (!this._isTileMap) {
                        map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                        map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale + map.model.zoomFactor)) / (map._width / position.x);
                        map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale + map.model.zoomFactor)) / (map._height / position.y);
                        map._scale = scal + map.model.zoomFactor;
                        map.model.zoomLevel += 1;
                    }
                    else {
                        this._tileZoom(map.model.zoomLevel - 1, map.model.zoomLevel, position);
                        var prevLevel = map.model.zoomLevel;
                        map.model.zoomLevel += 1;
                        this._generateTiles(this.model.zoomLevel);
                        map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                        map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                        map._scale = (Math.pow(2, prevLevel));
                    }
                    if (map.model.enableAnimation && !this._isTileMap) {
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                        map._refrshLayers();
                        map._resizeShape();
                    }
                    map._updateSliderValue();
                }
            }
        },

        /**
        * Section to zoom the tile
        * @private
        */
        _tileZoom: function (prevlevel, level, position) {

            var map = this;
            if (level > 0 && level < 20) {
                this._tileDiv.html("");
                var prevSize = Math.pow(2, prevlevel) * 256;
                if (position == undefined) {
                    position = { x: this._width / 2, y: this._height / 2 };
                }
                var totalSize = Math.pow(2, level) * 256;
                var percentX = ((position.x - this._tileTranslatePoint.x) / prevSize) * 100;
                var percentY = ((position.y - this._tileTranslatePoint.y) / prevSize) * 100;
                var pointX = (percentX * totalSize) / 100;
                var pointY = (percentY * totalSize) / 100;
                this._tileTranslatePoint.x = position.x - pointX;
                this._tileTranslatePoint.y = position.y - pointY;

            }
        },

        /**
        * Section For handling the mouse enter event for bubble
        * @private
        */
        _bubbleEnterFunction: function (event) {
            if (event.data.templateID != null) {
                var tooltiphtmlString = $("#" + event.data.templateID).render(event.data.itemsrc);
                $(event.data.htmlobj).html(tooltiphtmlString);
            }
            $(document.body).append(event.data.htmlobj);
            return $(event.data.htmlobj).css("display", "block").css({ "left": (event.pageX + 8) + "px", "top": (event.pageY + 6) + "px" });
        },

        /**
        * Section For handling the mouse leave event for bubble
        * @private
        */
        _bubbleleaveFunction: function (event) {
            $(event.data.htmlobj).remove();
            return $(event.data.htmlobj).css("display", "none");
        },

        /**
        * Section For handling the mouse over event for bubble
        * @private
        */
        _bubbleOverFunction: function (event) {
            return $(event.data.htmlobj).css("display", "block").css({ "left": (event.pageX + 8) + "px", "top": (event.pageY + 6) + "px" });
        },

        /**
       * Section For handling the mouse over event for path
       * @private
       */
        _polyEnterFunction: function (event) {
            var layer = event.data.Param1;
            var shape = event.data.Param2["shape"];
            var map = event.data.map;
            if (map != null && layer.enableMouseHover && map._browser != 'msie') {
                map._rootgroup.insertBefore(shape, map._rootgroup.childNodes[map._rootgroup.childNodes.length - (map._polylineCount + map._pointscount) - 1]);
            }
            var legend = jQuery.grep(layer._smartLabels, function (obj) { return obj.shape == shape });
            if (layer.enableMouseHover) {
                layer._clearShapeWidth(map._scale);
                if (layer._prevHoverdLegend != null && layer._prevHoverdLegend != layer._prevSelectedLegend) {
                    layer._prevHoverdLegend.css("background-color", _isSVG ? layer._prevHoveredShape.getAttribute("fill") : layer._prevHoveredShape.fillcolor.value);
                }
                if (_isSVG) {
                    shape.setAttribute("stroke-width", (layer.shapeSetting.mouseHoverWidth / map._scale));
                    shape.setAttribute('stroke', layer.shapeSetting.mouseHoverStroke);
                } else {
                    shape.strokeweight = layer.shapeSetting.mouseHoverWidth;
                    shape.strokecolor = layer.shapeSetting.mouseHoverStroke / map._scale;
                }
                if (layer.shapeSetting.mouseHoverColor != null) {
                    if (layer.shapeSetting.mouseHoverColor != "transparent" && layer._prevSelectedShape != shape) {
                        if (_isSVG) {
                            shape.setAttribute('fill', layer.shapeSetting.mouseHoverColor);
                        } else {
                            shape.fillcolor = layer.shapeSetting.mouseHoverColor;
                        }
                    }
                    if (legend.length > 0 && legend[0].legend != layer._prevSelectedLegend) {

                        legend[0].legend.css("background-color", layer.shapeSetting.mouseHoverColor);
                        layer._prevHoverdLegend = legend[0].legend;
                    }

                    layer._prevHoveredShape = shape;
                }

                if (layer.legendSetting != null && layer.legendSetting.legendMode == "interactive" && layer.shapeSetting.colorMappings != null && layer.shapeSetting.colorMappings.rangeColorMapping != null
                     && !layer.shapeSetting.autoFill && (layer.legendSetting.showLegend == undefined || layer.legendSetting.showLegend)) {
                    for (var i = 0; i < layer._mapShapes.length; i++) {
                        var mapshape = layer._mapShapes[i].shape;
                        if (mapshape == shape) {
                            var mappings = null;
                            if (layer.shapeSetting.colorMappings.rangeColorMapping != undefined) {
                                mappings = layer.shapeSetting.colorMappings.rangeColorMapping;
                            }
                            var _legendwidth = layer.legendSetting.legendWidth;
                            if (layer.legendSetting.legendWidth == undefined)
                                _legendwidth = 150;

                            var rectwidth = (_legendwidth / mappings.length) / 10;
                            var rectPosition = layer._mapShapes[i].legendrect.marginLeft;
                            $(layer._interactiveArrow).css({ "margin-left": rectPosition + Math.ceil(rectwidth) - layer._interactiveArrow.width() / 2, "display": "block" });
                            break;
                        }
                    }
                }
                map._trigger("shapeHovered", { originalEvent: event.data.Param2 });
            }
            var tooltipObject = event.data.Param2;
            if (tooltipObject.hasOwnProperty("data")) {
                var element = layer._tooltipElement;
                var template = layer.tooltipTemplate;
                if (element != null && template != null) {
                    $(element).css({ "left": event.pageX, "top": event.pageY, "display": "block" });
                    var htmlString = $("#" + template).render(tooltipObject["data"]);
                    $(element).html(htmlString);

                }
            }

        },

        /**
       * updates shape rect
       * @private
       */
        _updateShapeRect: function (layer) {

            for (var index = 0; index < layer._mapShapes.length; index++) {
                var obj = layer._mapShapes[index];
                if (obj.left == null) {
                    obj["left"] = layer._mapShapes[index].shape.getBoundingClientRect().left;
                    obj["right"] = layer._mapShapes[index].shape.getBoundingClientRect().right;
                    obj["top"] = layer._mapShapes[index].shape.getBoundingClientRect().top;
                    obj["bottom"] = layer._mapShapes[index].shape.getBoundingClientRect().bottom;
                    layer._mapShapes[index] = obj;
                }
            }
        },


        /**
       * Section For handling the mouse up event for path
       * @private
       */
        _polyUpFunction: function (event) {
            var layer = event.data.Param2;
            var shape = event.data.Param3;
            if (this.model.zoomOnSelection && !this._isDragging && !this._isTileMap && !this._isPinching) {
                if (_isSVG) {
                    var bounds = shape.getBBox();
                    this._zoomOnSelection(bounds);
                } else {
                    this._updateShapeRect(layer);

                    for (var index = 0; index < layer._mapShapes.length; index++) {
                        if (shape == layer._mapShapes[index].shape) {
                            var left = layer._mapShapes[index].left;
                            var top = layer._mapShapes[index].top;
                            var right = layer._mapShapes[index].right;
                            var bottom = layer._mapShapes[index].bottom;
                            var bound = { x: left, y: top, width: right - left, height: bottom - top };
                            this._zoomOnSelection(bound);
                            break;
                        }
                    }
                }
                this.ispanning = false;
            }

            layer._prevSelectedShape = shape;
        },

        /**
       * Section For handling the mouse click event for annotation
       * @private
       */
        _annotationPressed: function (event) {
            this._trigger("annotationSelected", { originalEvent: event.data });
        },

        /**
       * Section For handling the mouse click event for path
       * @private
       */
        _polyClickFunction: function (event) {
            var layer = event.data.Param2;
            var map = this;
            var shape = event.data.Param3;
            var legend = jQuery.grep(layer._smartLabels, function (obj) { return obj.shape == shape });
            if (layer.enableSelection && map._browser != 'msie') {
                map._rootgroup.insertBefore(shape, map._rootgroup.childNodes[map._rootgroup.childNodes.length - (map._polylineCount + map._pointscount) - 1]);
            }
            map._isPolygonSelected = true;
            if (layer._prevSelectedLegend != null && layer._prevSelectedShape != null) {
                layer._prevSelectedLegend.css("background-color", layer._prevSelectedShape.getAttribute("nodeValue"));
            }
            if (legend.length > 0) {
                legend[0].legend.css("background-color", layer.shapeSetting.shapeSelectionColor);
                layer._prevSelectedLegend = legend[0].legend;
            }
            if (layer._prevSelectedShape != null) {
                if (_isSVG) {
                    layer._prevSelectedShape.setAttribute("fill", layer._prevSelectedShape.getAttribute("nodeValue"));
                    layer._prevSelectedShape.setAttribute("stroke", layer.shapeSetting.shapeStroke);
                    layer._prevSelectedShape.setAttribute("stroke-width", layer.shapeSetting.shapeStrokeThickness / this._scale);
                }
                else {
                    layer._prevSelectedShape.fillcolor = layer._prevSelectedShape.style.behavior;
                    layer._prevSelectedShape.strokecolor = layer.shapeSetting.shapeStroke;
                    layer._prevSelectedShape.strokeweight = layer.shapeSetting.shapeStrokeThickness / this._scale;
                }
            }
            if (layer.enableSelection) {
                if (_isSVG) {
                    shape.setAttribute("fill", layer.shapeSetting.shapeSelectionColor);
                    shape.setAttribute("stroke", layer.shapeSetting.shapeSelectionStroke);
                    shape.setAttribute("stroke-width", (layer.shapeSetting.shapeSelectionStrokeWidth / map._scale));
                } else {
                    shape.fillcolor = layer.shapeSetting.shapeSelectionColor;
                    shape.strokecolor = layer.shapeSetting.shapeSelectionStroke;
                    shape.strokeweight = layer.shapeSetting.shapeSelectionStrokeWidth / map._scale;
                }
                layer._prevSelectedShape = shape;
            }
            map._trigger("shapeSelected", { originalEvent: event.data.Param1 });


        },

        /**
      * Zooms map on shape selection
      * @private
      */
        _zoomOnSelection: function (bounds) {
            var layerScale;
            if (!_isSVG) {
                bounds.x = (bounds.x - (this._baseTranslatePoint.x / 2)) - (bounds.width / 2);
                bounds.y = (bounds.y - (this._baseTranslatePoint.y / 2)) - (bounds.height / 2);
            }
            var boundwidth = bounds.width;
            var boundHeight = bounds.height;
            if ((this._width - 100) / (this._height - 100) > boundwidth / boundHeight) {
                layerScale = (this._height - 100) / boundHeight;
            } else {
                layerScale = (this._width - 100) / boundwidth;
            }
            this._prevScale = this._scale;
            this._scale = layerScale;
            this._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
            var level = this.model.zoomLevel = this._scale - this._baseScale + 1;
            if (!(level > this.model.minZoom && level < this.model.maxZoom)) {
                this.model.zoomLevel = level > (((this.model.maxZoom - this.model.minZoom) / 2) + this.model.minZoom) ? this.model.maxZoom : this.model.minZoom;
                this._scale = this.model.zoomLevel + this._baseScale - 1;
            }
            var leftMargin = (this._width / 2) - ((boundwidth * this._scale) / 2);
            var leftPos = leftMargin / this._scale;
            var topMargin = (this._height / 2) - ((boundHeight * this._scale) / 2);
            var topPos = topMargin / this._scale;
            this._translatePoint.x = -bounds.x + leftPos;
            this._translatePoint.y = -bounds.y + topPos;
            if (this.model.enableAnimation && !this._isTileMap) {
                this._animate(1200);
            }
            else {
                this._applyTransform(this._scale, this._translatePoint);
                this._updateSliderValue();
            }

            //this._refrshLayers();

        },

        /**
      * Section For handling the mouse move event for path
      * @private
      */
        _polyMoveFunction: function (event) {
            var element = event.data.Param1._tooltipElement;
            var tooltipObject = event.data.Param2;
            if (element != null && tooltipObject.hasOwnProperty("data")) {
                $(element).css({ "display": "block", "left": event.pageX + 10, "top": event.pageY + 10 });
            }
        },

        /**
      * Section For handling the mouse leave event for path
      * @private
      */
        _polyLeaveFunction: function (event) {
            var layer = event.data.Param1;
            var map = event.data.map;
            var element = layer._tooltipElement;
            if (element != null) {
                $(element).css("display", "none");
            }
            if (layer.legendSetting != null && layer.legendSetting.legendMode == "interactive" && layer._interactiveArrow != null)
                $(layer._interactiveArrow).css("display", "none");
            layer._clearShapeWidth(map._scale);
            map._trigger("shapeUnHovered", { originalEvent: event.data.Param2 });
        },

        /**
         * Wiring the events to map control		
		 * @private
         */
        _wireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            this._on($(this._mapContainer), ej.eventType.mouseDown, this._mouseButtonDown);
            this._on($(this._mapContainer), ej.eventType.mouseMove, this._mouseMove);
            this._on($(document), ej.eventType.mouseUp, this._mouseUp);
            if (window.navigator.msPointerEnabled) {
                this._on($(this._mapContainer), "MSPointerUp", this._mouseButtonUp);
                $(this._mapContainer).css('-ms-touch-action', 'none');
            }
            else {
                this._on($(this._mapContainer), "mouseup", this._mouseButtonUp);
            }
            var map = this;
            this._browser = browser;
            if (browser != "mozilla") {
                this._on($(this._mapContainer), "mousewheel", this._mouseWheel);
            }
            else {
                var elem = this._svgDocument;
                if (elem.addEventListener) {
                    // Mouse Scrolling event for firefox
                    elem.addEventListener("DOMMouseScroll", MouseWheel, false);
                }

            }
            function MouseWheel(event) {
                if (map.model.enableAnimation) {
                    $(map._rootgroup).stop(true, false);
                }
                event.preventDefault(event);
                map._zooming(-40 * event.detail, event);
            }
            this._on($(this._mapContainer), "dblclick", this._doubleClick);
            if (this.model.canResize) {
                if (!this._isDevice())
                    this._on($(window), "resize", this._mapResize);
                else
                    this._on($(window), "orientationchange", this._mapResize);
            }
        },

        /**
       * Section For handling the map resize event
       * @private
       */
        _mapResize: function (event) {
            event.preventDefault();
            event.stopPropagation();
            var map = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                map.refresh();
            }, 500);
        },

        /**
        * checks the device type
        * @private
        */
        _isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },

        /**
         * Unwiring the events to map control		
		 * @private
         */
        _unWireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            this._on($(document), ej.eventType.mouseUp, this._mouseUp);

            if (window.navigator.msPointerEnabled) {
                this._off($(this._mapContainer), "MSPointerDown", this._mouseButtonDown);
                this._off($(this._mapContainer), "MSPointerMove", this._mouseMove);
                this._off($(this._mapContainer), "MSPointerUp", this._mouseButtonUp);
                $(this._mapContainer).css('-ms-touch-action', 'none');
            }
            else if (browser == "webkit" || browser == "chrome" || browser == "mozilla") {
                this._off($(this._mapContainer), "mousedown", this._mouseButtonDown);
                this._off($(this._mapContainer), "mousemove", this._mouseMove);
                this._off($(this._mapContainer), "mouseup", this._mouseButtonUp);

            }
            else {
                this._off($(this._mapContainer), "mousedown", this._mouseButtonDown);
                this._off($(this._mapContainer), "mousemove", this._mouseMove);
                this._off($(this._mapContainer), "mouseup", this._mouseButtonUp);

            }
            this._off($(this._mapContainer), "dblclick", this._doubleClick);

        },

        /**     
        * Fires on shape unhovered in map.
        * @event
        * @name ejMap#shapeUnHovered	
        * @param {number} latitude parameter of map
	    * @param {number} longitude parameter of map
	    * @param {number} level parameter of map
        * @example 
        * //shapeUnHovered event for map
        * $("#container").ejMap({
        *    shapeUnHovered: function (lat, long, level) {}
        * });
        * @memberof ejMap
        * @instance
        */
        navigateTo: function (latitude, longitude, level) {

            level = parseFloat(level);
            if (level == undefined) level = this.model.zoomLevel;
            if (level > this.model.minZoom && level < this.model.maxZoom) {
                this.model.zoomLevel = level;
            }
            else {
                this.model.zoomLevel = level > (((this.model.maxZoom - this.model.minZoom) / 2) + this.model.minZoom) ? this.model.maxZoom : this.model.minZoom;

            }
            var translatePoint = this._convertTileLatLongtoPointForShapes(latitude, longitude, this._mapBounds);
            this._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
            this._prevScale = this._scale;
            this._scale = this._baseScale + ((level - 1) * this.model.zoomFactor);
            var leftPosition = ((this._containerWidth + this._baseTranslatePoint.x) / 2) / this._scale;
            var topPosition = ((this._containerHeight + this._baseTranslatePoint.y) / 2) / this._scale;
            this._translatePoint.x = -translatePoint.x + leftPosition;
            this._translatePoint.y = -translatePoint.y + topPosition;
            if (this.model.enableAnimation && !this._isTileMap) {
                this._animate(1200);
            }
            else {
                this._applyTransform(this._scale, this._translatePoint);
            }
            this._updateSliderValue();
            this._refrshLayers();
        },


        selectShape: function (obj, layer, isZoom) {
            if (obj != null) {
                if (layer == null) {
                    layer = this.model.layers[this.model.baseMapIndex];
                }
                for (var i = 0; i < layer._mapShapes.length; i++) {
                    var data = layer._mapShapes[i].data;
                    var shape = layer._mapShapes[i].shape;
                    if (data != null && obj == this._reflection(data, layer.shapeSetting.shapeValuepath)) {
                        if (layer._prevSelectedShape != null) {
                            if (_isSVG) {
                                layer._prevSelectedShape.setAttribute("fill", layer._prevSelectedShape.getAttribute("nodeValue"));
                                layer._prevSelectedShape.setAttribute("stroke", layer.shapeSetting.shapeStroke);
                                layer._prevSelectedShape.setAttribute("stroke-width", layer.shapeSetting.shapeStrokeThickness / this._scale);
                            }
                            else {
                                layer._prevSelectedShape.fillcolor = layer._prevSelectedShape.style.behavior;
                                layer._prevSelectedShape.strokecolor = layer.shapeSetting.shapeStroke;
                                layer._prevSelectedShape.strokeweight = layer.shapeSetting.shapeStrokeThickness / this._scale;
                            }
                        }
                        if (layer.enableSelection) {
                            if (_isSVG) {
                                shape.setAttribute("fill", layer.shapeSetting.shapeSelectionColor);
                                shape.setAttribute("stroke", layer.shapeSetting.shapeSelectionStroke);
                                shape.setAttribute("stroke-width", (layer.shapeSetting.shapeSelectionStrokeWidth / this._scale));
                            } else {
                                shape.fillcolor = layer.shapeSetting.shapeSelectionColor;
                                shape.strokecolor = layer.shapeSetting.shapeSelectionStroke;
                                shape.strokeweight = layer.shapeSetting.shapeSelectionStrokeWidth / this._scale;
                            }
                            layer._prevSelectedShape = shape;
                        }
                        if (isZoom && this.model.zoomOnSelection) {
                            this._zoomOnSelection(shape.getBBox());
                        }
                        this._trigger("shapeSelected", { originalEvent: data });
                        i = layer._mapShapes.length;
                    }
                }
            }

        },

        /**
        * Gets intersected elements
        * @private
        */
        _getIntersectedElements: function (evt, shapes) {

            if (_isSVG && this._browser != 'mozilla' && this._browser != 'webkit') {
                var rpos = this._svgDocument.createSVGRect();
                rpos.x = evt.left;
                rpos.y = evt.top;
                rpos.width = evt.width + 5;
                rpos.height = evt.height + 5;
                return this._svgDocument.getIntersectionList(rpos, null);
            }
            else {
                var elements = [];
                for (var i = 0; i < shapes.length; i++) {
                    var shape = shapes[i].shape;
                    var bounds = shape.getBoundingClientRect();
                    if (this._isIntersect(evt, { left: (bounds.left + this._translatePoint.x), top: (bounds.top + this._translatePoint.y), height: (bounds.bottom * this._scale) - (bounds.top * this._scale), width: (bounds.right * this._scale) - (bounds.left * this._scale) })) {
                        elements.push(shape);
                        return elements;
                    }

                }
                return elements;
            }

        },

        /**
        * Checks intersected elements
        * @private
        */
        _isIntersect: function (rect1, rect2) {

            if (rect1.left - 2 >= (rect2.left + rect2.width)
|| rect1.left - 2 + rect1.width <= rect2.left - 2
|| rect1.top - 2 >= rect2.top + rect2.height
|| rect1.top + rect1.height <= rect2.top - 2) {
                return false;
            }
            return true;

        },

        /**     
        * Fires on map pan.
        * @event
        * @name ejMap#pan	
        * @param {string} panning direction
        * @example 
        * //pan event for map
        * $("#container").ejMap({
        *    pan: function (direction) {}
        * });
        * @memberof ejMap
        * @instance
        */
        pan: function (direction) {
            var map = this;
            var Xdiff = 0;
            var Ydiff = 0;
            if (this.model.zoomLevel != 1) {
                switch (direction) {

                    case 'right':
                        {
                            Xdiff = this._width / 7;
                            Ydiff = 0;
                            break;
                        }
                    case 'top':
                        {
                            Xdiff = 0;
                            Ydiff = -(this._height / 7);
                            break;
                        } n
                    case 'left':
                        {
                            Xdiff = -(this._width / 7);
                            Ydiff = 0;
                            break;
                        }
                    case 'bottom':
                        {
                            Xdiff = 0;
                            Ydiff = this._height / 7;
                            break;
                        }
                }
                if (map.model.enablePan) {
                    if (this._isTileMap) {
                        var curtileX = this._tileTranslatePoint.x - Xdiff / map._scale;
                        var curtileY = this._tileTranslatePoint.y - Ydiff / map._scale;
                        this._tileTranslatePoint.x = curtileX;
                        this._tileTranslatePoint.y = curtileY;
                        this._generateTiles(this.model.zoomLevel);
                    }
                    var curX = map._translatePoint.x - Xdiff / map._scale;
                    map._prevScale = map._scale;
                    var curY = map._translatePoint.y - Ydiff / map._scale;
                    map._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
                    this._translatePoint.x = curX;
                    this._translatePoint.y = curY;
                    if (map.model.enableAnimation && !this._isTileMap) {
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                    }
                    map._refrshLayers();
                    map._trigger("panning", null);
                }
            }

        },

        /**     
        * Fires on map zoom. 
        * @event
        * @name ejMap#zoom	
        * @param {number} zoom level
        * @param {boolean} animation status
        * @example 
        * //zoom event for map
        * $("#container").ejMap({
        *    zoom: function (level,isAnimate) {}
        * });
        * @memberof ejMap
        * @instance
        */
        zoom: function (level, isAnimate) {
            var map = this;
            if (level <= this.model.maxZoom && level >= this.model.minZoom) {
                if (this._isTileMap) {
                    if (level > this.model.zoomLevel) {
                        this._zoomingIn(this._width / 2, this._height / 2, null);
                    }
                    else {
                        this._zoomingOut(this._width / 2, this._height / 2, null);
                    }
                }
                else {
                    var fac = (map._baseScale + (level * map.model.zoomFactor)) - 1;
                    map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                    map._prevScale = map._scale;
                    map._translatePoint.x -= (map._width / map._scale - map._width / fac) / 2;
                    map._translatePoint.y -= (map._height / map._scale - map._height / fac) / 2;
                    map._scale = fac;
                    map.model.zoomLevel = level;
                    if (map.model.enableAnimation && isAnimate) {
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                        map._refrshLayers();
                        map._resizeShape();
                    }
                    map._updateSliderValue();
                }


            }
            else if (this.model.zoomLevel <= this.model.minZoom) {
                this.model.zoomLevel = this.model.minZoom;
                this.zoom();
            }
            else if (this.model.zoomLevel >= this.model.maxZoom) {
                this.model.zoomLevel = this.model.maxZoom;
                this.zoom();
            }

        },

        /**     
       * Fires on map refresh.
       * @event
       * @name ejMap#refresh
       * @example 
       * //refresh event for map
       * $("#container").ejMap({
       *    refresh: function () {}
       * });
       * @memberof ejMap
       * @instance
       */
        refresh: function () {

            this._unWireEvents();
            $(this._svgDocument).empty();
            if (this._svgDocument != null) {
                this._svgDocument = null;
            }
            $(this._mapContainer).empty();
            this._scale = 1;
            this._translatePoint = { x: 0, y: 0 };
            this._height = this._mapContainer.height();
            this._width = this._mapContainer.width();
            if (this._height == 0) {
                this._height = this._mapContainer[0].parentElement.clientHeight;
            }
            if (this._width == 0) {
                this._width = this._mapContainer[0].parentElement.clientWidth;
            }
            if (this.model.baseMapIndex >= this.model.layers.length) {
                this.model.baseMapIndex = 0;
            }
            this._generatePath();
            if (this._groupSize != null) {
                this._containerHeight = this._groupSize.maxY - this._groupSize.minY;
                this._containerWidth = this._groupSize.maxX - this._groupSize.minX;
                this._groupBorder = { x: this._groupSize.minX, y: this._groupSize.minY };
            }
            this._resizingContainer();
            this._renderMapElements();
            if (this.model.enableLayerChangeAnimation) {
                $(this._mapContainer).animate({ opacity: 1 }, 500);
            }

        },


        /**
        * Updates slider value
        * @private
        */
        _updateSliderValue: function (isAnimate) {
            var slider = this._sliderControl;
            if (isAnimate == undefined) isAnimate = this.model.enableAnimation;
            if (slider != null) {
                var obj = slider.data("ejSlider");
                obj.option("value", this.model.zoomLevel);
            }
        },

        /**
        * Creates div element
        * @private
        */
        _createDivElement: function (parent, child, base, classname) {
            var parentElement = parent;
            var childElement = child;
            childElement.appendTo(parentElement);
            parentElement.appendTo(base);
            this._on(childElement, ej.eventType.mouseDown, { className: classname, map: this }, this._navigationControlPressed);

        },

        /**
      * Section For handling mouse click event for navigation control
      * @private
      */
        _navigationControlPressed: function (event) {

            event.stopPropagation();
            this._isNavigationPressed = true;
            var map = event.data.map;
            var isAnimate = map.model.enableAnimation;
            if (event.data.className == "zoomIn") {
                map.zoom(map.model.zoomLevel + 1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }
            else if (event.data.className == "zoomOut") {
                map.zoom(map.model.zoomLevel - 1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }
            else if (event.data.className == "panLeft") {
                map.pan("left");
            }
            else if (event.data.className == "panRight") {
                map.pan("right");
            }
            else if (event.data.className == "panTop") {
                map.pan("top");
            }
            else if (event.data.className == "panBottom") {
                map.pan("bottom");
            }
            else if (event.data.className == "home") {
                map.zoom(1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }

        },

        /**     
          * Fires on navigation control refresh. 
          * @event
          * @name ejMap#refreshNavigationControl	
          * @param {object} navigation control
          * @example 
          * //Refresh navigation control event for map
          * $("#container").ejMap({
          *    refreshNavigationControl: function (navigation) {}
          * });
          * @memberof ejMap
          * @instance
          */
        refreshNavigationControl: function (navigation) {

            var prevNav = this._mapContainer.find("#ejNavigation");
            if (prevNav.length > 0) {
                this._mapContainer[0].removeChild(prevNav[0]);
            }
            if (navigation == undefined) {
                navigation = this.model.navigationControl;
            }
            var navigationOrientation = ej.orientation.Vertical;
            var controlSize;
            var navigationHeight = 150;
            var navigationWidth = 12;
            controlSize = { width: 60, height: 350 };
            var navigationControl = $("<div id='ejNavigation' class='e-map-navigation e-orientation-vert'/>");
            if (navigation.orientation == 'horizontal') {
                navigationOrientation = ej.orientation.Horizontal;
                navigationHeight = 12;
                navigationWidth = 200;
                controlSize = { width: 350, height: 60 };
                navigationControl = $("<div id='ejNavigation' class='e-map-navigation e-orientation-horz' />");
            }
            var baseDiv = $("<div class='e-panContainer'/>");
            var isHor = navigationOrientation == 'horizontal' ? 'Horz' : '';
            var zoominOutDiv = $("<div/>");
            var slidercontrol = $("<div  />");
            baseDiv.appendTo(navigationControl);
            zoominOutDiv.appendTo(navigationControl);
            var sliderDiv = $("<div style='height:150px;width:10px;margin-top:-197px;margin-left: 34px;' />");
            sliderDiv.appendTo(slidercontrol);
            slidercontrol.appendTo(navigationControl);
            this._sliderControl = sliderDiv;
            var navPos = { x: 0, y: 0 };
            if (navigation.dockPosition == null || navigation.dockPosition == 'none') {
                navPos.x = (this._width * navigation.absolutePosition.x) / 100;
                navPos.y = (this._height * navigation.absolutePosition.y) / 100;
            }
            else {
                navPos = this._getDockPosition(navigation.dockPosition, controlSize);
            }
            navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
            var map = this;
            $(sliderDiv).ejSlider({
                orientation: navigationOrientation,
                sliderType: ej.sliderType.MinRange,
                value: 1,
                animationSpeed: 1200,
                startValue: this.model.minZoom,
                showTooltip: true,
                animate: false,
                endValue: this.model.maxZoom,
                step: this.model.zoomFactor,
                slide: onslide,
                change: onchange,
                height: navigationHeight,
                width: navigationWidth
            });
            navigationControl.appendTo(this._mapContainer);
            this._createDivElement($("<div class='e-icon1 e-incrementButton icon_margin1' />"), $("<div class='e-icon1 e-innerIncrement'/>"), zoominOutDiv, "zoomIn");
            this._createDivElement($("<div class='e-icon1 e-incrementButton icon_margin2'/>"), $("<div class='e-icon1 e-innerDecrement'/>"), zoominOutDiv, "zoomOut");
            this._createDivElement($("<div class='e-icon1 e-radialTop'/>"), $("<div class='e-icon1 e-arrowUp'/>"), baseDiv, "panTop");
            this._createDivElement($("<div class='e-icon1 e-radialLeft'/>"), $("<div class='e-icon1 e-arrowLeft'/>"), baseDiv, "panLeft");
            this._createDivElement($("<div class='e-icon1 e-radialRight'/>"), $("<div class='e-icon1 e-arrowRight'/>"), baseDiv, "panRight");
            this._createDivElement($("<div class='e-icon1 e-radialBottom'/>"), $("<div class='e-icon1 e-arrowDown'/>"), baseDiv, "panBottom");
            var homeDiv = $("<div class='e-icon1 e-home-bg'><div class='e-icon1 e-map-home'></div>");
            homeDiv.appendTo(baseDiv);
            homeDiv.mousedown({ className: "home", map: this }, this._navigationControlPressed);
            function onchange(args) {
                if (map != null && map._isRendered && map.model.zoomLevel != args.value) {
                    map.zoom(args.value, false);
                }

            }

            function onslide(args) {
                if (map != null && map._isRendered && map.model.zoomLevel != args.value) {
                    map.zoom(args.value, false);
                }
            }
        },

        /**
       * Gets dock position for legend and navigation control
       * @private
       */
        _getDockPosition: function (position, size) {
            var dockPosition = { x: 0, y: 0 };
            switch (position) {
                case 'topleft':
                    break;
                case 'topcenter':
                    dockPosition.x = (this._width / 2) - (size.width / 2);
                    break;
                case 'topright':
                    dockPosition.x = this._width - size.width;
                    break;
                case 'centerleft':
                    dockPosition.y = (this._height / 2) - (size.height / 2);
                    break;
                case 'center':
                    dockPosition.x = (this._width / 2) - (size.width / 2);
                    dockPosition.y = (this._height / 2) - (size.height / 2);
                    break;
                case 'centerright':
                    dockPosition.x = this._width - size.width;
                    dockPosition.y = (this._height / 2) - (size.height / 2);
                    break;
                case 'bottomleft':
                    dockPosition.y = this._height - size.height;
                    break;
                case 'bottomcenter':
                    dockPosition.x = (this._width / 2) - (size.width / 2);
                    dockPosition.y = this._height - size.height;
                    break;
                case 'bottomright':
                    dockPosition.x = this._width - size.width;
                    dockPosition.y = this._height - size.height;
                    break;

            }
            return dockPosition;
        },

        /**
       * Renders map elements
       * @private
       */
        _renderMapElements: function () {
            this._templateDiv = $("<div class='TemplateDiv'/>");
            this._templateDiv.appendTo(this._mapContainer);
            this._templateDiv.css({
                'pointer-events': 'none', 'overflow': 'hidden', "position": "absolute",
                'z-index': '1', 'height': this._height, 'width': this._width
            });
            this._wireEvents();
            this.refreshLayers();
            if (this.model.navigationControl != null && this.model.navigationControl.enableNavigation) {
                this.refreshNavigationControl(this.model.navigationControl);
            }
        },


        /**     
          * Fires on layers refresh. 
          * @event
          * @name ejMap#refreshLayers	
          * @example 
          * //refresh layers event for map
          * $("#container").ejMap({
          *    refreshLayers: function () {}
          * });
          * @memberof ejMap
          * @instance
          */
        refreshLayers: function () {
            $(this._templateDiv).empty();
            var prevPathCount = 0;
            var baseLayer = this.model.layers[this.model.baseMapIndex];
            if (baseLayer.mapType == 'geometry') {
                this._renderLayers(baseLayer, prevPathCount);
                if (baseLayer.data != null)
                    prevPathCount = baseLayer._polygonData.length;
            }
            else {
                baseLayer._initializeLocalValues();
            }
            for (var key = 0; key < this.model.layers[this.model.baseMapIndex].subshapeFileLayers.length; key++) {
                var sublayer = this.model.layers[this.model.baseMapIndex].subshapeFileLayers[key];
                sublayer._isBaseLayer = false;
                if (sublayer.mapType == 'geometry') {
                    this._renderLayers(sublayer, prevPathCount);
                    prevPathCount += sublayer._polygonData.length;
                }
                else {
                    sublayer._initializeLocalValues();
                }
            }
            this._generateAnnotations();
            if (_isSVG && this.model.enableAnimation && !this._isTileMap) {
                this._refreshWithAnimation();
            }
            else {
                this._refrshLayers();
            }

        }

    });



    var Tile = function (x, y) {
        this.X = x;
        this.Y = y;
        this.left = 0;
        this.top = 0;
        this.height = 256;
        this.width = 256;
        this.src = null;
    };

    /** 
    * // class for shape file layer
    * @memberof ejMap
    * @member
    * @instance
    */
    var shapeFileLayer = function () {


        /** 
        * Enables or disables the shape selection
         * @default true
         * @type {boolean} 
         * @example  
         * // Set the enableSelection during initialization. 			
         * 	$("#container").ejMap({layers: [{ enableSelection:true }]})
         * @example 
         * //Get or set the enableSelection after initialization:<br>
         *   //Gets the enableSelection from map.
         *   var property =$("#container").data("ejMap").model.layers[layerIndex].enableSelection;
         *   //Sets the enableSelection to map.
         *   $("#container").data("ejMap").model.layers[layerIndex].enableSelection  = true;
         * @alias ejMap#shapeFileLayer->enableSelection
         * @instance
         */
        this.enableSelection = true;

        /**
        * Enables or disables the shape mouse hover
            * @default false
            * @type {boolean} 
            * @example  
            * // Set the enableMouseHover during initialization. 			
            * 	$("#container").ejMap({layers: [{ enableMouseHover:false }]})
            * @example 
            * //Get or set the enableMouseHover after initialization:<br>
            *   //Gets the enableMouseHover from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].enableMouseHover;
            *   //Sets the enableMouseHover to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].enableMouseHover  = false;
            * @alias ejMap#shapeFileLayer->enableMouseHover
            * @instance
            */
        this.enableMouseHover = false;

        /** 
        * Specifies the shape data for the shape file layer
        * @type {object}
        * @example
        * // Set the data of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", data: mapShapeData }]})
        * @example 
        * //Get or set the data after initialization:<br>
        *   //Gets the data from map layer.
        *   var data =$("#container").data("ejMap").model.layers[layerIndex].data;
        *   //Sets the data to map layer.
        *   $("#container").data("ejMap").model.layers[layerIndex].data  = mapShapeData;
        * @alias ejMap#shapeFileLayer->data
        * @instance
        */
        this.data = null;

        /**		
            * Specify annotations for shape file layer.
            * @default []
            * @type {annotation} 
            * @example  
            * // Set the annotations during initialization. 			
            * 	$("#container").ejMap({layers: [{annotations:[{annotationLabel : "chennai",latitude : 13.08 ,longitude : 80.27}]}]})
            * @example 
            * //Get or set the annotations after initialization:<br>
            *   //Gets the annotations from map.
            *   var annotation =$("#container").data("ejMap").model.layers[layerIndex].annotations[annotationIndex];
            *   //Sets the annotation to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].annotations[annotationIndex]  = {annotations:[{annotationLabel : "chennai",latitude : 13.08 ,longitude : 80.27}]};
            * @alias ejMap#shapeFileLayer->annotations
            * @instance
            */
        this.annotations = [];

        /** 
        * Specifies the datasource for the shape file layer
        * @type {object}
        * @example
        * // Set the dataSource of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", dataSource: source,  data: mapShapeData }]})
        * @example 
        * //Get or set the dataSource after initialization:<br>
        *   //Gets the dataSource from map layer.
        *   var dataSource =$("#container").data("ejMap").model.layers[layerIndex].dataSource;
        *   //Sets the dataSource to map layer.
        *   $("#container").data("ejMap").model.layers[layerIndex].dataSource  = source;
        * @alias ejMap#shapeFileLayer->dataSource
        * @instance
        */
        this.dataSource = null;

        /** 
        * Specifies the url template for the OSM type map.
            * @default 'http://a.tile.openstreetmap.org/level/tileX/tileY.png'
            * @type {string} 
            * @example  
            * // Set the urlTemplate during initialization. 			
            * 	$("#container").ejMap({layers: [{ urlTemplate:'http://a.tile.openstreetmap.org/level/tileX/tileY.png' }]})
            * @example 
            * //Get or set the urlTemplate after initialization:<br>
            *   //Gets the urlTemplate from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].urlTemplate;
            *   //Sets the urlTemplate to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].urlTemplate  = 'http://a.tile.openstreetmap.org/level/tileX/tileY.png';
            * @alias ejMap#shapeFileLayer->urlTemplate
            * @instance
            */
        this.urlTemplate = 'http://a.tile.openstreetmap.org/level/tileX/tileY.png';

        /** 
        * Shows or hides the tooltip for shapes
            * @default false
            * @type {boolean} 
            * @example  
            * // Set the showToolTip during initialization. 			
            * 	$("#container").ejMap({layers: [{ showToolTip:false }]})
            * @example 
            * //Get or set the showToolTip after initialization:<br>
            *   //Gets the showToolTip from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].showToolTip;
            *   //Sets the showToolTip to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].showToolTip  = false;
            * @alias ejMap#shapeFileLayer->showToolTip
            * @instance
            */
        this.showToolTip = false;

        /** 
        * Specifies the tooltip template for shapes.
        * @type {string}
        * @example
        * // Set the tooltipTemplate of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", tooltipTemplate: "Template",  data: mapShapeData }]})
        * @example 
        * //Get or set the tooltipTemplate after initialization:<br>
        *   //Gets the tooltipTemplate from map.
        *   var tooltipTemplate =$("#container").data("ejMap").model.layers[layerIndex].tooltipTemplate;
        *   //Sets the tooltipTemplate to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].tooltipTemplate  ="Template";
        * @alias ejMap#shapeFileLayer->tooltipTemplate
        * @instance
        */
        this.tooltipTemplate = null;

        /**
        * Specifies the map items template for shapes.
        * @type {string}
        * @example
        * // Set the mapItemsTemplate of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", mapItemsTemplate: "Template",  data: mapShapeData }]})
        * @example 
        * //Get or set the mapItemsTemplate after initialization:<br>
        *   //Gets the mapItemsTemplate from map.
        *   var mapItemsTemplate =$("#container").data("ejMap").model.layers[layerIndex].mapItemsTemplate;
        *   //Sets the mapItemsTemplate to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].mapItemsTemplate  = "Template";
        * @alias ejMap#shapeFileLayer->mapItemsTemplate
        * @instance
        */
        this.mapItemsTemplate = null;

        /** 
        * Enables or disables the animation
            * @default false
            * @type {boolean} 
            * @example  
            * // Set the enableAnimation during initialization. 			
            * 	$("#container").ejMap({layers: [{ enableAnimation:false }]})
            * @example 
            * //Get or set the enableAnimation after initialization:<br>
            *   //Gets the enableAnimation from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].enableAnimation;
            *   //Sets the enableAnimation to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].enableAnimation  = false;
            * @alias ejMap#shapeFileLayer->enableAnimation
            * @instance
            */
        this.enableAnimation = false;

        /** 
        * Specifies the legend setting of the shapes
        * @type {object}
        * @example
        * // Set the legendSetting of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", legendSetting: {showLegend:true, legendPosition: "bottomleft", legendPositionX: 3, legendPositionY:80, legendHeight:18, legendWidth:190, legendType:"Layers", legendMode:"interactive", legendTitle: "Number of Students", legendLeftLabel:"0 M ", legendRightLabel:"1.4 M "},  data: mapShapeData }]})
        * @example 
        * //Get or set the legendSetting after initialization:<br>
        *   //Gets the legendSetting from map.
        *   var legendSetting =$("#container").data("ejMap").model.layers[layerIndex].legendSetting;
        *   //Sets the legendSetting to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].legendSetting  = {showLegend:true, legendPosition: "bottomleft", legendPositionX: 3, legendPositionY:80, legendHeight:18, legendWidth:190, legendType:"Layers", legendMode:"interactive", legendTitle: "Number of Students", legendLeftLabel:"0 M ", legendRightLabel:"1.4 M "};
        * @alias ejMap#shapeFileLayer->legendSetting
        * @instance
        */
        this.legendSetting = null;

        /** 
        * Specifies the label setting of the shapes
        * @type {object}
        * @example
        * // Set the labelSetting of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", labelSetting: { showLabels: true, labelPath: 'iso_3166_2',enableSmartLabel: true },  data: mapShapeData }]})
        * @example 
        * //Get or set the labelSetting after initialization:<br>
        *   //Gets the labelSetting from map.
        *   var labelSetting =$("#container").data("ejMap").model.layers[layerIndex].labelSetting;
        *   //Sets the labelSetting to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].labelSetting  =  { showLabels: true, labelPath: 'iso_3166_2',enableSmartLabel: true };
        * @alias ejMap#shapeFileLayer->labelSetting
        * @instance
        */
        this.labelSetting = null,

        /** 
        * Specifies the map annotation template for shapes.
        * @type {string}
        * @example
        * // Set the annotationTemplate of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", annotationTemplate: "Template",  data: mapShapeData }]})
        * @example 
        * //Get or set the annotationTemplate after initialization:<br>
        *   //Gets the annotationTemplate from map.
        *   var annotationTemplate =$("#container").data("ejMap").model.layers[layerIndex].annotationTemplate;
        *   //Sets the annotationTemplate to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].annotationTemplate  = "Template";
        * @alias ejMap#shapeFileLayer->annotationTemplate
        * @instance
        */
        this.annotationTemplate = null;

        /**
        * Specifies the annotation datasource.
        * @type {object}
        * @example
        * // Set the annotationDataSource of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", annotationDataSource: source,  data: mapShapeData }]})
        * @example 
        * //Get or set the annotationDataSource after initialization:<br>
        *   //Gets the annotationDataSource from map layer.
        *   var annotationDataSource =$("#container").data("ejMap").model.layers[layerIndex].annotationDataSource;
        *   //Sets the annotationDataSource to map layer.
        *   $("#container").data("ejMap").model.layers[layerIndex].annotationDataSource  = source;
        * @alias ejMap#shapeFileLayer->annotationDataSource
        * @instance
        */
        this.annotationDataSource = null;

        /** 
        * Shows or hides the map items.
            * @default false
            * @type {boolean} 
            * @example  
            * // Set the showMapItems during initialization. 			
            * 	$("#container").ejMap({layers: [{ showMapItems:false }]})
            * @example 
            * //Get or set the showMapItems after initialization:<br>
            *   //Gets the showMapItems from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].showMapItems;
            *   //Sets the showMapItems to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].showMapItems  = false;
            * @alias ejMap#shapeFileLayer->showMapItems
            * @instance
            */
        this.showMapItems = false;

        /** 
        * Specifies the map type.
            * @default 'geometry'
            * @type {string} 
            * @example  
            * // Set the mapType during initialization. 			
            * 	$("#container").ejMap({layers: [{ mapType:'geometry' }]})
            * @example 
            * //Get or set the mapType after initialization:<br>
            *   //Gets the mapType from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].mapType;
            *   //Sets the mapType to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].mapType  = 'geometry';
            * @alias ejMap#shapeFileLayer->mapType
            * @instance
            */
        this.mapType = 'geometry';
        /**
         * Specifies inbuilt color palette		
		 * @private
         */
        this._colorPaletteSettings = {
            'Palette1':
            {
                'MouseHoverColor': '#F7CD1C',
                'MouseHoverStroke': 'white',
                'MouseHoverStrokeWidth': '1',
                'SelectionColor': '#F96C0D',
                'SelectionStroke': 'white',
                'SelectionStrokeWidth': '2.5'
            },
            'Palette2':
            {
                'MouseHoverColor': '#68A3EA',
                'MouseHoverStroke': 'White',
                'MouseHoverStrokeWidth': '1',
                'SelectionColor': '#116EF4',
                'SelectionStroke': 'White',
                'SelectionStrokeWidth': '2.5'
            },
            'Palette3':
            {
                'MouseHoverColor': '#CCCCCC',
                'MouseHoverStroke': 'white',
                'MouseHoverStrokeWidth': '1',
                'SelectionColor': '#4D4D4D',
                'SelectionStroke': 'white',
                'SelectionStrokeWidth': '2.5'
            }
        };
        this.colorPalettes = {
            'Palette1': ['#4A3825', '#736F3D', '#F2DABD', '#BF9D7E', '#7F6039', '#7F715F', '#70845D', '#CC995C', '#736F3D', '#89541B'],
            'Palette2': ['#E51400', '#730202', '#EF6535', '#C63477', '#BF004D', '#F0A30B', '#CE1B1B', '#97993D', '#D6BF38', '#835A2C'],
            'Palette3': ['#A4C400', '#008B00', '#1BA0E2', '#0050EF', '#AA00FF', '#D90073', '#E51400', '#F96800', '#E3C800', '#A20026']
        };
        /**
         * Specifies previous palette index		
		 * @private
         */
        this._prevPaletteIndex = 0;
        /**
         * Specifies new map bounds	
		 * @private
         */
        this._newBounds = [];
        /** 
        * Specifies the sub shape file layers
            * @default []
            * @type {shapeFileLayer} 
            * @example  
            * // Set the subshapeFileLayers during initialization. 			
            * 	$("#container").ejMap({layers: [{ mapType: "geometry", enableMouseHover: true, shapeSetting: { shapeStroke: "black", shapeFill: "#C3E6ED", mouseHoverColor: "#63B7B7", shapeSelectionColor: "#207BB2", shapeStrokeThickness: "0.5" },subshapeFileLayers: [{shapeIdPath: "pathName", shapeIdTableField: "tableFieldName", mapItemsTemplate: 'Template', showMapItems: true, enableMouseHover: true, dataSource: mapDataSource, shapeSetting: { shapeFill: "#9FD0D3", shapeStrokeThickness: "0.2", shapeStroke: "white", mouseHoverColor: "#63B7B7", },  data: mapShapeData }, data: mapShapeData }]})
            * @example 
            * //Get or set the layer after initialization:<br>
            *   //Gets the layer from map.
            *   var layer =$("#container").data("ejMap").model.layers[layerIndex];
            *   //Sets the layer to map.
            *   $("#container").data("ejMap").model.layers[layerIndex]  = { mapType: "geometry", enableMouseHover: true, shapeSetting: { shapeStroke: "black", shapeFill: "#C3E6ED", mouseHoverColor: "#63B7B7", shapeSelectionColor: "#207BB2", shapeStrokeThickness: "0.5" },subshapeFileLayers: [{shapeIdPath: "pathName", shapeIdTableField: "tableFieldName", mapItemsTemplate: 'Template', showMapItems: true, enableMouseHover: true, dataSource: mapDataSource, shapeSetting: { shapeFill: "#9FD0D3", shapeStrokeThickness: "0.2", shapeStroke: "white", mouseHoverColor: "#63B7B7", },  data: mapSubShapeData }, data: mapShapeData };
            * @alias ejMap#shapeFileLayer->subshapeFileLayers
            * @instance
            */
        this.subshapeFileLayers = [];


        /** 
        * Specifies the shape settings
        * @type {object}
        * @example
        * // Set the shapeSetting of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", enableMouseHover: true, shapeSetting: { shapeStroke: "black", shapeFill: "#C3E6ED", mouseHoverColor: "#63B7B7", shapeSelectionColor: "#207BB2", shapeStrokeThickness: "0.5" }, data: Africa }]})
        * @example 
        * //Get or set the shapeSetting after initialization:<br>
        *   //Gets the shapeSetting from map.
        *   var shapeSetting =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting;
        *   //Sets the shapeSetting to map.
        *   $("#container").data("ejMap").model.layers[layerIndex]  = { mapType: "geometry", enableMouseHover: true, shapeSetting: { shapeStroke: "black", shapeFill: "#C3E6ED", mouseHoverColor: "#63B7B7", shapeSelectionColor: "#207BB2", shapeStrokeThickness: "0.5" }, data: Africa };
        * @alias ejMap#shapeFileLayer->shapeSetting
        * @instance
        */
        this.shapeSetting = {
            /**		
			* Specifies the mouse hover color of the shape file layer in map
            * @default "gray"
			* @type {string}
			* @example 
            * //To set mouseHoverColor API value during initialization 
			*   $("#container").ejMap({layers:{shapeSetting: {mouseHoverColor:'gray'}}});
            * @example 
			* //Get or set the mouseHoverColor API, after initialization:<br>
			*	//Gets the mouseHoverColor value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.mouseHoverColor;<br> 	
            *	//Sets the mouseHoverColor value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.mouseHoverColor='gray'; 
            * @alias ejMap#shapeFileLayer->shapeSetting->mouseHoverColor
			* @instance
            */
            mouseHoverColor: "gray",

            /**		
			* Specifies the mouse over width of the shape file layer in map
            * @default 1
			* @type {number}
			* @example 
            * //To set mouseHoverWidth API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {mouseHoverWidth:1}}});
            * @example 
			* //Get or set the mouseHoverWidth API, after initialization:<br>
			*	//Gets the mouseHoverWidth value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.mouseHoverWidth;<br> 	
            *	//Sets the mouseHoverWidth value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.mouseHoverWidth=1; 
			* @alias ejMap#shapeFileLayer->shapeSetting->mouseHoverWidth
			* @instance
            */
            mouseHoverWidth: 1,

            /**		
			* Specifies the shape selection color of the shape file layer in map
            * @default "gray"
			* @type {string}
			* @example 
            * //To set shapeSelectionColor API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeSelectionColor:'gray'}}});
            * @example 
			* //Get or set the shapeSelectionColor API, after initialization:<br>
			*	//Gets the shapeSelectionColor value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeSelectionColor;<br> 	
            *	//Sets the shapeSelectionColor value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeSelectionColor='gray'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeSelectionColor
			* @instance
            */
            shapeSelectionColor: "gray",

            /**		
			* Specifies the shape fill color of the shape file layer in map
            * @default "#E5E5E5"
			* @type {string}
			* @example 
            * //To set shapeFill API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeFill:'#E5E5E5'}}});
            * @example 
			* //Get or set the shapeFill API, after initialization:<br>
			*	//Gets the shapeFill value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeFill;<br> 	
            *	//Sets the shapeFill value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeFill='#E5E5E5'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeFill
			* @instance
            */
            shapeFill: "#E5E5E5",

            /**		
			* Specifies the shape stroke thickness value of the shape file layer in map
            * @default "0.2"
			* @type {string}
			* @example 
            * //To set shapeStrokeThickness API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeStrokeThickness:'0.2'}}});
            * @example 
			* //Get or set the shapeStrokeThickness API, after initialization:<br>
			*	//Gets the shapeStrokeThickness value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeStrokeThickness;<br> 	
            *	//Sets the shapeStrokeThickness value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeStrokeThickness='0.2'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeStrokeThickness
			* @instance
            */
            shapeStrokeThickness: "0.2",

            /**		
			* Specifies the shape stroke color of the shape file layer in map
            * @default "#C1C1C1"
			* @type {string}
			* @example 
            * //To set shapeStroke API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeStroke:'#C1C1C1'}}});
            * @example 
			* //Get or set the shapeStroke API, after initialization:<br>
			*	//Gets the shapeStroke value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeStroke;<br> 	
            *	//Sets the shapeStroke value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeStroke='#C1C1C1'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeStroke
			* @instance
            */
            shapeStroke: "#C1C1C1",

            /**		
			* Specifies the shape color palette value of the shape file layer in map
            * @default "palette1"
			* @type {string}
			* @example 
            * //To set shapeColorPalette API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeColorPalette:'palette1'}}});
            * @example 
			* //Get or set the shapeColorPalette API, after initialization:<br>
			*	//Gets the shapeColorPalette value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeColorPalette;<br> 	
            *	//Sets the shapeColorPalette value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeColorPalette='palette1'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeColorPalette
			* @instance
            */
            shapeColorPalette: 'palette1',

            /**		
			* Specifies the mouse over stroke color of the shape file layer in map
            * @default "#C1C1C1"
			* @type {string}
			* @example 
            * //To set mouseHoverStroke API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {mouseHoverStroke:'#C1C1C1'}}});
            * @example 
			* //Get or set the mouseHoverStroke API, after initialization:<br>
			*	//Gets the mouseHoverStroke value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.mouseHoverStroke;<br> 	
            *	//Sets the mouseHoverStroke value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.mouseHoverStroke='#C1C1C1'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->mouseHoverStroke
			* @instance
            */
            mouseHoverStroke: "#C1C1C1",

            /**		
			* Specifies the shape selection stroke color of the shape file layer in map
            * @default "#C1C1C1"
			* @type {string}
			* @example 
            * //To set shapeSelectionStroke API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeSelectionStroke:'#C1C1C1'}}});
            * @example 
			* //Get or set the shapeSelectionStroke API, after initialization:<br>
			*	//Gets the shapeSelectionStroke value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeSelectionStroke;<br> 	
            *	//Sets the shapeSelectionStroke value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeSelectionStroke='#C1C1C1'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeSelectionStroke
			* @instance
            */
            shapeSelectionStroke: "#C1C1C1",

            /**		
			* Specifies the shape selection stroke width of the shape file layer in map
            * @default 1
			* @type {number}
			* @example 
            * //To set shapeSelectionStrokeWidth API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeSelectionStrokeWidth:1}}});
            * @example 
			* //Get or set the shapeSelectionStrokeWidth API, after initialization:<br>
			*	//Gets the shapeSelectionStrokeWidth value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeSelectionStrokeWidth;<br> 	
            *	//Sets the shapeSelectionStrokeWidth value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeSelectionStrokeWidth=1; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeSelectionStrokeWidth
			* @instance
            */
            shapeSelectionStrokeWidth: 1,

            /**		
			* Specifies the shape color valuepath of the shape file layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set shapeColorValuepath API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeColorValuepath:'sales'}}});
            * @example 
			* //Get or set the shapeColorValuepath API, after initialization:<br>
			*	//Gets the shapeColorValuepath value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeColorValuepath;<br> 	
            *	//Sets the shapeColorValuepath value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeColorValuepath='sales'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeColorValuepath
			* @instance
            */
            shapeColorValuepath: null,


            /**		
			* Specifies the shape valuepath of the shape file layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set shapeValuepath API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {shapeValuepath:'name'}}});
            * @example 
			* //Get or set the shapeValuepath API, after initialization:<br>
			*	//Gets the shapeValuepath value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.shapeValuepath;<br> 	
            *	//Sets the shapeValuepath value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.shapeValuepath='name'; 
			* @alias ejMap#shapeFileLayer->shapeSetting->shapeValuepath
			* @instance
            */
            shapeValuepath: null,

            /**		
			* Specifies the gradient color status of the shape file layer in map
            * @default false
			* @type {boolean}
			* @example 
            * //To set enableGradient API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {enableGradient:false}}});
            * @example 
			* //Get or set the enableGradient API, after initialization:<br>
			*	//Gets the enableGradient value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.enableGradient;<br> 	
            *	//Sets the enableGradient value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.enableGradient=false; 
			* @alias ejMap#shapeFileLayer->shapeSetting->enableGradient
			* @instance
            */
            enableGradient: false,

            /**		
			* Specifies the colorMappings of the shape file layer in map
            * @default null
			* @type {object}
			* @example 
            * //To set colorMappings API value during initialization 
			*   $("#container").ejMap({layers:{ shapeSetting: {colorMappings:{rangeColorMapping:{from: 0,to: 100000,gradientColors: ["#9CBF4E", "#B8CE7B"]}}}}});
            * @example 
			* //Get or set the colorMappings API, after initialization:<br>
			*	//Gets the colorMappings value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.colorMappings;<br> 	
            *	//Sets the colorMappings value 
			*	$("#container").data("ejMap").model.layers[0].shapeSetting.colorMappings={rangeColorMapping:{from: 0,to: 100000,gradientColors: ["#9CBF4E", "#B8CE7B"]}}; 
			* @alias ejMap#shapeFileLayer->shapeSetting->colorMappings
			* @instance
            */
            colorMappings: null,

            /**		
			* Specifies the auto fill  status of the shape file layer in map
            * @default false
			* @type {boolean}
			* @example 
            * //To set autoFill API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {autoFill:false}}});
            * @example 
			* //Get or set the autoFill API, after initialization:<br>
			*	//Gets the autoFill value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSetting.autoFill;<br> 	
            *	//Sets the autoFill value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSetting.autoFill=false; 
			* @alias ejMap#shapeFileLayer->shapeSetting->autoFill
			* @instance
            */
            autoFill: false
        };

        /** 
        * Specifies the bubble settings.
        * @type {object}
        * @example 
        * // Set the bubbleSetting of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ mapType: "geometry", enableMouseHover: true, shapeSetting: { shapeStroke: "black", shapeFill: "#C3E6ED", mouseHoverColor: "#63B7B7", shapeSelectionColor: "#207BB2", shapeStrokeThickness: "0.5" },bubbleSetting:{ bubbleValuepath: "valuePath", minSize: 20, maxSize: 30, bubbleColor: "#379F64",}, data: mapShapeData }]})
        * @example 
        * //Get or set the bubbleSetting after initialization:<br>
        *   //Gets the bubbleSetting from map.
        *   var bubbleSetting =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting;
        *   //Sets the bubbleSetting to map.
        *   $("#container").data("ejMap").model.layers[layerIndex]  = { mapType: "geometry", enableMouseHover: true, shapeSetting: { shapeStroke: "black", shapeFill: "#C3E6ED", mouseHoverColor: "#63B7B7", shapeSelectionColor: "#207BB2", shapeStrokeThickness: "0.5" },bubbleSetting:{ bubbleValuepath: "valuePath", minSize: 20, maxSize: 30, bubbleColor: "#379F64",}, data: mapShapeData };
        * @instance
        * @alias ejMap#shapeFileLayer->bubbleSetting
        */
        this.bubbleSetting = {

            /**		
			* Specifies the minimum size value of bubbles in shape file layer in map
            * @default "10"
			* @type {string}
			* @example 
            * //To set minSize API value during initialization 
			*   $("#container").ejMap({layers: {bubbleSetting: {minSize:'10'}}});
            * @example 
			* //Get or set the minSize API, after initialization:<br>
			*	//Gets the minSize value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting.minSize;<br> 	
            *	//Sets the minSize value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSetting.minSize='10'; 
			* @alias ejMap#shapeFileLayer->bubbleSetting->minSize
			* @instance
            */
            minSize: "10",

            /**		
			* Specifies the maximum size value of bubbles in shape file layer in map
            * @default "20"
			* @type {string}
			* @example 
            * //To set maxSize API value during initialization 
			*   $("#container").ejMap({layers: {bubbleSetting: {maxSize:'20'}}});
            * @example 
			* //Get or set the maxSize API, after initialization:<br>
			*	//Gets the maxSize value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting.maxSize;<br> 	
            *	//Sets the maxSize value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSetting.maxSize='20'; 
			* @alias ejMap#shapeFileLayer->bubbleSetting->maxSize
			* @instance
            */
            maxSize: "20",

            /**		
			* Specifies the mouse hover color of the shape file layer in map
            * @default "gray"
			* @type {string}
			* @example 
            * //To set bubbleColor API value during initialization 
			*   $("#container").ejMap({layers:{bubbleSetting: {bubbleColor:'gray'}}});
            * @example 
			* //Get or set the bubbleColor API, after initialization:<br>
			*	//Gets the bubbleColor value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting.bubbleColor;<br> 	
            *	//Sets the bubbleColor value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSetting.bubbleColor='gray'; 
			* @alias ejMap#shapeFileLayer->bubbleSetting->bubbleColor
			* @instance
            */
            bubbleColor: "gray",

            /**		
			* Specifies the bubble color valuepath of the shape file layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set bubbleColorValuepath API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {bubbleColorValuepath:'sales'}}});
            * @example 
			* //Get or set the bubbleColorValuepath API, after initialization:<br>
			*	//Gets the bubbleColorValuepath value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting.bubbleColorValuepath;<br> 	
            *	//Sets the bubbleColorValuepath value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSetting.bubbleColorValuepath='sales'; 
			* @alias ejMap#shapeFileLayer->bubbleSetting->bubbleColorValuepath
			* @instance
            */
            bubbleColorValuepath: null,

            /**		
			* Specifies the bubble valuepath of the shape file layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set bubbleValuepath API value during initialization 
			*   $("#container").ejMap({layers: {shapeSetting: {bubbleValuepath:'name'}}});
            * @example 
			* //Get or set the bubbleValuepath API, after initialization:<br>
			*	//Gets the bubbleValuepath value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting.bubbleValuepath;<br> 	
            *	//Sets the bubbleValuepath value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSetting.bubbleValuepath='name'; 
			* @alias ejMap#shapeFileLayer->bubbleSetting->bubbleValuepath
			* @instance
            */
            bubbleValuepath: null,

            /**		
			* Specifies the colorMappings of the shape file layer in map
            * @default null
			* @type {object}
			* @example 
            * //To set colorMappings API value during initialization 
			*   $("#container").ejMap({layers:{ bubbleSetting: {colorMappings:{rangeColorMapping:{from: 0,to: 100000,gradientColors: ["#9CBF4E", "#B8CE7B"]}}}}});
            * @example 
			* //Get or set the colorMappings API, after initialization:<br>
			*	//Gets the colorMappings value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting.colorMappings;<br> 	
            *	//Sets the colorMappings value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSetting.colorMappings={rangeColorMapping:{from: 0,to: 100000,gradientColors: ["#9CBF4E", "#B8CE7B"]}}; 
			* @alias ejMap#shapeFileLayer->bubbleSetting->colorMappings
			* @instance
            */
            colorMappings: null,

            /**		
			* Specifies the tooltip visibility  status of the shape file layer in map
            * @default false
			* @type {boolean}
			* @example 
            * //To set showToolTip API value during initialization 
			*   $("#container").ejMap({layers: {bubbleSetting: {showToolTip:false}}});
            * @example 
			* //Get or set the showToolTip API, after initialization:<br>
			*	//Gets the showToolTip value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting.showToolTip;<br> 	
            *	//Sets the showToolTip value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSetting.showToolTip=false; 
			* @alias ejMap#shapeFileLayer->bubbleSetting->showToolTip
			* @instance
            */
            showToolTip: false,

            /**		
			* Specifies the bubble tooltip template of the shape file layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set tooltipTemplate API value during initialization 
			*   $("#container").ejMap({layers: {bubbleSetting: {tooltipTemplate:'template'}}});
            * @example 
			* //Get or set the tooltipTemplate API, after initialization:<br>
			*	//Gets the tooltipTemplate value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSetting.tooltipTemplate;<br> 	
            *	//Sets the tooltipTemplate value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSetting.tooltipTemplate='template'; 
			* @alias ejMap#shapeFileLayer->bubbleSetting->tooltipTemplate
			* @instance
            */
            tooltipTemplate: null
        };
    };

    shapeFileLayer.prototype =
    {
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            dataSource: "data",
            annotations: "array",
            subshapeFileLayers: "array",
            shapeSetting: {
                colorMappings: "array"
            },
            bubbleSetting: {
                colorMappings: "array"
            }

        },

        /**
       * Initializes local values of the shape file layer
       * @private
       */
        _initializeLocalValues: function () {
            this._svgns = "http://www.w3.org/2000/svg";
            this._bounds = [];
            this._prevSelectedShape = null;
            this._prevSelectedLegend = null;
            this._isBaseLayer = true;
            this._prevHoverdLegend = null;
            this._prevHoveredShape = null;
            this._labelCollection = [];
            this._scrollBar = null;
            this._mapShapes = [];
            this._bubbles = [];
            this._labelBounds = [];
            this._bubbleCount = 0;
            this._mapItems = [];
            this._mapannotations = [];
            this._smartLabels = [];
            this._labelData = [];
            this._interactiveArrow = null;
            this._legendRects = [];

        },

        /**
       * Generates annotation for layers
       * @private
       */
        _generateAnnotationForLayers: function (map) {
            var rootTop = map._rootgroup.getBoundingClientRect().top;
            var rootLeft = map._rootgroup.getBoundingClientRect().left;
            for (var key = 0; key < this.annotations.length; key++) {
                var annotationObeject = this.annotations[key];
                if (this.annotationTemplate != null) {
                    var annotationTemplateDiv = $('.annotationTemplateDiv');
                    annotationTemplateDiv = $("<div style='display:block;position:absolute;pointer-events: stroke;'></div>");
                    annotationTemplateDiv[0].className = key;
                    map._templateDiv.append(annotationTemplateDiv);
                    var htmlString = $("#" + this.annotationTemplate).render(annotationObeject);
                    $(annotationTemplateDiv).html(htmlString);
                    map._on($(annotationTemplateDiv), ej.eventType.mouseDown, { annotation: $(annotationTemplateDiv), data: annotationObeject }, map._annotationPressed);
                    this._mapannotations.push(annotationTemplateDiv);
                }
                else {
                    var annotationItem = $(' <div style="display:block;position:absolute;pointer-events: stroke;"><label>' + annotationObeject.annotationLabel + '</label></div>');// document.createElementNS(this._svgns, "text");
                    if (_isSVG) {
                        map._templateDiv.append(annotationItem);
                    }
                    else {
                        annotationItem.appendTo(map._templateDiv);
                    }
                    map._on($(annotationItem), ej.eventType.mouseDown, { annotation: $(annotationItem), data: annotationObeject }, map._annotationPressed);
                    this._mapannotations.push(annotationItem);

                }
            }
            if (this.annotationDataSource != null) {
                for (var key = 0; key < this.annotationDataSource.length; key++) {
                    var annotationObeject = this.annotationDataSource[key];
                    var annotationItem;
                    if (this.annotationTemplate != null) {
                        annotationItem = $("<div style='display:block;position:absolute;pointer-events: stroke;'></div>");
                        annotationItem[0].className = key;
                        var htmlString = $("#" + this.annotationTemplate).render(annotationObeject);
                        $(annotationItem).html(htmlString);
                    }
                    else {
                        annotationItem = $(' <div style="display:block;position:absolute;pointer-events: stroke;"><label>' + annotationObeject.annotationLabel + '</label></div>');
                    }
                    map._on($(annotationItem), ej.eventType.mouseDown, { annotation: $(annotationItem), data: annotationObeject }, map._annotationPressed);
                    map._templateDiv.append(annotationItem);
                    this._mapannotations.push(annotationItem);

                }
            }

        },

        /**
       * Clears shape width
       * @private
       */
        _clearShapeWidth: function (scale) {
            if (scale == null) scale = 1;
            for (var index = 0; index < this._mapShapes.length; index++) {
                var mapshape = this._mapShapes[index];
                if (this._prevSelectedShape != mapshape.shape) {
                    if (_isSVG) {
                        mapshape.shape.setAttribute("fill", mapshape.shape.getAttribute("nodeValue"));
                        mapshape.shape.setAttribute("stroke-width", (this.shapeSetting.shapeStrokeThickness / scale));
                        mapshape.shape.setAttribute("stroke", this.shapeSetting.shapeStroke);
                    }
                    else {
                        mapshape.shape.fillcolor = mapshape.shape.style.behavior;
                        mapshape.shape.strokeweight = this.shapeSetting.shapeStrokeThickness;
                        mapshape.shape.strokecolor = this.shapeSetting.shapeStroke;
                    }
                }
            }



        },

        /**
       * Converts RGB to Hex decimal values
        * @private
       */
        _rgbToHex: function (R, G, B) {
            return "#" + this._toHex(R) + this._toHex(G) + this._toHex(B);
        },

        /**
       * Converts to Hex decimal values
        * @private
       */
        _toHex: function (n) {
            var charstr = "0123456789ABCDEF";
            n = parseInt(n, 10);
            if (isNaN(n)) return "00";
            n = Math.max(0, Math.min(n, 255));
            return charstr.charAt((n - n % 16) / 16)
                + charstr.charAt(n % 16);
        },

        /**
       * Converts Hex decimal values to Red value
        * @private
       */
        _hexToR: function (h) {
            return parseInt((this._cropHex(h)).substring(0, 2), 16);
        },

        /**
       * Converts Hex decimal values to Green value
        * @private
       */
        _hexToG: function (h) {
            return parseInt((this._cropHex(h)).substring(2, 4), 16);
        },

        /**
      * Converts Hex decimal values to Blue value
       * @private
      */
        _hexToB: function (h) {
            return parseInt((this._cropHex(h)).substring(4, 6), 16);
        },

        /**
      * crops Hex value
       * @private
      */
        _cropHex: function (h) {
            return (h.charAt(0) == "#") ? h.substring(1, 7) : h;
        },

        /**
        * Gets range color values
        * @private
        */
        _getRangeColorValues: function (range, start, end) {
            var rangeColorValues = [];
            rangeColorValues.push(start);
            if (start > end) {
                var rangeValue = (start - end) / (range - 1);
                for (var i = range; i > 2; i--) {
                    start = start - rangeValue;
                    rangeColorValues.push(start);
                }
            }
            else {
                var rangeValue = (end - start) / (range - 1);
                for (var i = 2; i < range; i++) {
                    start = start + rangeValue;
                    rangeColorValues.push(start + rangeValue);
                }
            }
            rangeColorValues.push(end);
            return rangeColorValues;
        },

        /**
        * Generate gradient collection
        * @private
        */
        _generateGradientCollection: function (gradientColors) {
            var gradientCollection = [];
            var startRangeColor = gradientColors[0];
            var endRangeColor = gradientColors[gradientColors.length - 1];
            var startR = this._hexToR(startRangeColor);
            var startG = this._hexToG(startRangeColor);
            var startB = this._hexToB(startRangeColor);
            var endR = this._hexToR(endRangeColor);
            var endG = this._hexToG(endRangeColor);
            var endB = this._hexToB(endRangeColor);
            var range = 10;
            var rRange = this._getRangeColorValues(range, startR, endR);
            var gRange = this._getRangeColorValues(range, startG, endG);
            var bRange = this._getRangeColorValues(range, startB, endB);
            for (var i = 0; i < range; i++) {
                gradientCollection.push(this._rgbToHex(rRange[i], gRange[i], bRange[i]));
            }
            return gradientCollection;
        },

        /**
        * Creates labels
        * @private
        */
        _createLabel: function (content, xpos, ypos) {
            var label = $("<div></div>"); // document.createElementNS(this._svgns, "text");
            label[0].innerHTML = content;
            label.css({
                "font-family": "Segoe UI",
                "pointer-events": "none",
                "text-align": "center",
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute"
            });
            return label;
        },

        /**
        * Creates interactive arrow for interactive legend
        * @private
        */
        _createInteractiveArrow: function (xpos, ypos) {

            var interactiveElement = $("<div class='e-icon1 e-interactiveArrow'></div>");
            //temporary code
            interactiveElement[0].innerHTML = "&#9650";

            interactiveElement.css({
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute",
                //temporary code
                "color": "#444444"
            });
            return interactiveElement;
        },

        /**
        * Generates legend for maps
        * @private
        */
        _generateLegends: function (map) {
            if ((this.legendSetting.showLegend == undefined || this.legendSetting.showLegend) && !this.shapeSetting.autoFill) {
                var xpos = 10;
                var ypos = 10;
                var _legendheight = this.legendSetting.legendHeight;
                var _legendwidth = this.legendSetting.legendWidth;
                var legenddiv = $("<div/>");
                if (_isSVG) {
                    legenddiv.appendTo(map._templateDiv);
                }
                else {
                    map._templateDiv.append(legenddiv);
                }
                var totalwidth = 0;
                var totalheight = 0;
                var isRange = true;
                var mappings = null;
                if (this.shapeSetting.colorMappings.rangeColorMapping != undefined) {
                    mappings = this.shapeSetting.colorMappings.rangeColorMapping;
                } else if (this.shapeSetting.colorMappings.equalColorMapping != undefined) {
                    mappings = this.shapeSetting.colorMappings.equalColorMapping;
                    isRange = false;
                }

                if ((this.legendSetting.legendType == undefined || this.legendSetting.legendType == 'Layers') && this.shapeSetting.colorMappings != null
                   && (this.legendSetting.legendMode == undefined || this.legendSetting.legendMode == 'default' || (this.legendSetting.legendMode == 'interactive'
                   && this.shapeSetting.colorMappings.equalColorMapping != null))) {

                    if (this.legendSetting.legendHeight == undefined)
                        _legendheight = 20;
                    if (this.legendSetting.legendWidth == undefined)
                        _legendwidth = 20;

                    for (var key = 0; key < mappings.length; key++) {
                        var colorMapping = mappings[key];
                        if (!colorMapping.hideLegend) {
                            var rect = $("<div class='e-mapLegend'/>");
                            rect.css({
                                "height": _legendheight + "px",
                                "width": _legendwidth + "px",
                                "background-color": colorMapping.color,
                                "margin-left": xpos + "px",
                                "margin-top": ypos + "px",
                                "position": "absolute"
                            });

                            var textcon = this._createLabel(colorMapping.legendLabel, xpos + _legendwidth + 5, ypos);

                            if (colorMapping.legendLabel != null) {
                                textcon[0].innerText = colorMapping.legendLabel;

                            } else {
                                colorMapping.legendLabel = textcon[0].innerText = isRange ? colorMapping.from : colorMapping.value;
                            }
                            if (_isSVG) {
                                rect.appendTo(legenddiv);
                                textcon.appendTo(legenddiv);
                            }
                            else {
                                legenddiv.append(rect);
                                legenddiv.append(textcon);
                            }

                            if (totalwidth < (colorMapping.legendLabel.length * 10) + rect.width())
                                totalwidth = (colorMapping.legendLabel.length * 10) + rect.width();
                            var textHeight = textcon.height();
                            var rectHeight = rect.height();
                            if (textHeight > rectHeight) {
                                ypos += (textHeight + 5);
                            } else {
                                ypos += (rectHeight + 5);
                            }
                        }
                    }
                    totalheight = ypos;
                } else if ((this.legendSetting.legendType == undefined || this.legendSetting.legendType == 'Layers') && this.legendSetting.legendMode == 'interactive' && this.shapeSetting.colorMappings != null) {
                    var textcon = '';

                    if (this.legendSetting.legendHeight == undefined)
                        _legendheight = 18;
                    if (this.legendSetting.legendWidth == undefined)
                        _legendwidth = 150;

                    if (this.legendSetting.legendLeftLabel == null)
                        this.legendSetting.legendLeftLabel = '';
                    if (this.legendSetting.legendTitle != null) {

                        var newxpos = xpos + this.legendSetting.legendLeftLabel.length * 10;
                        var textcon = this._createLabel(this.legendSetting.legendTitle, newxpos, ypos);

                        if (_isSVG)
                            textcon.appendTo(legenddiv);
                        else
                            legenddiv.append(textcon);
                        ypos += 25;
                    }

                    if (this.legendSetting.legendLeftLabel != null) {
                        var textcon = this._createLabel(this.legendSetting.legendLeftLabel, xpos, ypos - 3);

                        if (_isSVG)
                            textcon.appendTo(legenddiv);
                        else
                            legenddiv.append(textcon);
                        xpos = xpos + this.legendSetting.legendLeftLabel.length * 10;

                        var interactiveElement = this._createInteractiveArrow(xpos, ypos + _legendheight);
                        interactiveElement.appendTo(legenddiv);
                        this._interactiveArrow = interactiveElement;
                    }

                    var _legendgroup = null;
                    if (!_isSVG && this.shapeSetting.enableGradient) {
                        _legendgroup = map._createGroup(false, "legendGroup");
                        _legendgroup.style.left = 0 + 'px';
                        _legendgroup.style.top = 0 + 'px';
                        _legendgroup.style.position = "relative";
                        legenddiv.append(_legendgroup);
                    }

                    for (var key = 0; key < mappings.length; key++) {
                        var colorMapping = mappings[key];
                        if (!colorMapping.hideLegend) {
                            var gradientCollection = [];
                            if (this.shapeSetting.enableGradient) {
                                gradientCollection = this._generateGradientCollection(colorMapping.gradientColors);
                            }

                            var obj = {};
                            if (this.shapeSetting.enableGradient) {
                                if (_isSVG) {
                                    var canvas = $("<canvas/>");
                                    var ctx = canvas[0].getContext("2d");
                                    var grd = ctx.createLinearGradient(0, 0, 300, 0);

                                    var temp = 0;
                                    for (var i = 0; i < 10; i++) {
                                        temp = temp + (1 / 10);
                                        if (i == 0) {
                                            grd.addColorStop(0, colorMapping.gradientColors[0]);
                                            grd.addColorStop(temp, gradientCollection[i]);
                                        } else if (i == gradientCollection.length - 1) {
                                            grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                            grd.addColorStop(temp, colorMapping.gradientColors[1]);
                                        } else {
                                            grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                            grd.addColorStop(temp, gradientCollection[i + 1]);
                                        }
                                    }
                                    ctx.fillStyle = grd;
                                    ctx.fillRect(0, 0, 300, 300);
                                    canvas.css({
                                        "height": _legendheight + "px",
                                        "width": (_legendwidth / mappings.length) + "px",
                                        "margin-left": xpos + "px",
                                        "margin-top": ypos + "px",
                                        "opacity": "0.9",
                                        "filter": "alpha(opacity=90)", /* For IE8 and earlier */
                                        "position": "absolute"
                                    });
                                    canvas.appendTo(legenddiv);
                                } else {
                                    var legendID = "legend" + key;
                                    var legendHtmlString = '<v:rect id=' + legendID + ' display="block" style="position:absolute;top: ' + (ypos - 2) + 'px;left:' + xpos + 'px;width:' + (_legendwidth / mappings.length) + 'px;height:' + _legendheight + 'px;"><v:fill opacity="0.9px" type="gradient" method="linear sigma" angle="270"/><v:stroke opacity="0px"/></v:rect>';
                                    _legendgroup.innerHTML = _legendgroup.innerHTML + legendHtmlString;
                                    var legendelement = document.getElementById(legendID);
                                    legendelement.fillcolor = colorMapping.gradientColors[0];
                                    legendelement.fill.color2 = colorMapping.gradientColors[1];
                                }
                            }
                            else {
                                var rect = $("<div/>");
                                rect.css({
                                    "height": _legendheight + "px",
                                    "width": (_legendwidth / mappings.length) + "px",
                                    "background-color": colorMapping.color,
                                    "margin-left": xpos + "px",
                                    "margin-top": ypos + "px",
                                    "opacity": "0.9",
                                    "filter": "alpha(opacity=90)",/* For IE8 and earlier */
                                    "position": "absolute"
                                });

                                if (_isSVG)
                                    rect.appendTo(legenddiv);
                                else
                                    legenddiv.append(rect);
                            }

                            for (var i = 0; i < 10; i++) {
                                obj = {};
                                obj["marginLeft"] = xpos;
                                this._legendRects.push(obj);
                                xpos = xpos + (_legendwidth / mappings.length) / 10;
                            }
                        }
                    }

                    if (this.legendSetting.legendRightLabel != null) {
                        var textcon = this._createLabel(this.legendSetting.legendRightLabel, xpos + 10, ypos - 3);
                        if (_isSVG)
                            textcon.appendTo(legenddiv);
                        else
                            legenddiv.append(textcon);
                        xpos = xpos + this.legendSetting.legendRightLabel.length * 10;
                    }

                    totalwidth = xpos + 10;
                    totalheight = ypos + _legendheight + this._interactiveArrow.height() + 10;
                }

                if (this.legendSetting.legendPosition == 'none') {
                    var posx = (map._width * this.legendSetting.legendPositionX) / 100;
                    var posy = (map._height * this.legendSetting.legendPositionY) / 100;
                    legenddiv.css({ "margin-left": posx + "px", "margin-top": posy + "px" });
                } else {
                    var legendPosition = (this.legendSetting.legendPosition == undefined) ? "topleft" : this.legendSetting.legendPosition;
                    var controlSize = { width: totalwidth, height: totalheight };
                    var navPos = map._getDockPosition(legendPosition, controlSize);
                    legenddiv.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                }
            }
        },

        /**
        * Animates the bubble while rendering
        * @private
        */
        _animateBubble: function (element, delayInterval) {
            var radius = element.getAttribute("r");
            var scaleVal;
            var $ele = $(element);
            var layer = this;
            $ele.delay(delayInterval).each(function () { }).animate(
                {
                    r: radius
                },
                {
                    duration: 700,

                    step: function (now) {
                        scaleVal = now;
                        if (_isSVG) {
                            $ele.attr("style", "display:block;pointer-events:stroke;");
                            $ele[0].setAttribute("r", scaleVal);
                        }
                        else {
                        }
                    },
                    complete: function () {
                        layer._bubbleCount++;
                        if (layer._bubbleCount == layer._bubbles.length) {
                            layer._setMapElements();
                        }
                    }
                }

            );
        },

        /**
       * Sets map elements
       * @private
       */
        _setMapElements: function () {
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                $(item).css({ "display": "block" });
            }
        },

        /**
      * Sets map items position with animation
      * @private
      */
        _setMapItemsPositionWithAnimation: function (map) {

            for (var key = 0; key < this._bubbles.length; key++) {
                var bubble = this._bubbles[key];
                var position = this._bounds[key];
                var xpos = ((position.x + map._translatePoint.x) * map._scale);
                var ypos = ((position.y + map._translatePoint.y) * map._scale);
                var ubound = 20;
                var lbound = 0;
                var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
                var delayInterval = parseInt(randomValue * 50);
                if (_isSVG) {
                    $(bubble).attr({ "cx": xpos, "cy": ypos });
                }
                this._animateBubble(bubble, delayInterval);

            }
            for (var key = 0; key < this._mapannotations.length; key++) {
                var item = this._mapannotations[key];
                var annotation;
                if (this.annotations.length > 0) {
                    annotation = this.annotations[key];
                }
                else if (this.annotationDataSource != null && this.annotationDataSource.length > 0) {
                    annotation = this.annotationDataSource[key];
                }
                var position;
                if (map._isTileMap) {
                    position = map._convertTileLatLongtoPoint(annotation.latitude, annotation.longitude);
                }
                else {
                    position = map._convertLatLongtoPointforAnnotation(annotation.latitude, annotation.longitude);
                }

                if (_isSVG) {
                    var xpos = position.x;
                }
                else {
                    var xpos = ((position.x + map._transformX) * map._scale);
                }
                var ypos = position.y;
                $(item).css({ "display": "block", "left": xpos, "top": ypos - 100 });
                $(item).delay(500).each(function () { }).animate({ "top": ypos }, 500);

            }
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                var position = this._bounds[key];
                var box = item[0].getBoundingClientRect();
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (box.width / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (box.height / 4);
                $(item).css({ "left": xpos, "top": ypos, "display": "none" });

            }
            for (var key = 0; key < this._labelCollection.length; key++) {
                var item = this._labelCollection[key];
                var position = this._labelBounds[key];
                $(item).css("display", "block");
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 2);
                if (item[0].className = "labelLegend") {
                    item[0].className = "labelStyle";
                }
                item[0].innerHTML = this._labelData[key];
                $(item).css({ "left": xpos, "top": ypos });

            }
            if (this.labelSetting != null && this.labelSetting.showLabels) {
                this._validateSmartLabel(map);
            }
        },

        /**
     * Resizes Shapes of the shape file layer
     * @private
     */
        _resizeShapes: function (map) {
            var thickness = this.shapeSetting.shapeStrokeThickness / map._scale;
            for (var i = 0; i < this._mapShapes.length; i++) {
                var element = this._mapShapes[i].shape;
                if (_isSVG) {
                    if (element.localName == 'circle') {
                        element.setAttribute("r", 5 / map._scale);
                    } else {
                        if (element == this._prevSelectedShape) {
                            element.setAttribute("stroke-width", this.shapeSetting.shapeSelectionStrokeWidth / map._scale);
                        } else {
                            element.setAttribute("stroke-width", thickness);
                        }
                    }
                } else {
                    if (element.nodeName == 'oval') {

                    } else {
                        if (element == this._prevSelectedShape) {
                            element.strokeweight = this.shapeSetting.shapeSelectionStrokeWidth / map._scale;
                        } else {
                            element.strokeweight = thickness;
                        }
                    }
                }
            }
        },
        /**
         * Sets map items position
         * @private
         */
        _setMapItemsPosition: function (map) {

            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                var position = this._bounds[key];
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 4);
                $(item).css({ "left": xpos, "top": ypos });
            }
            for (var key = 0; key < this._bubbles.length; key++) {
                var bubble = this._bubbles[key];
                var position = this._bounds[key];
                if (_isSVG) {
                    var xpos = ((position.x + map._translatePoint.x) * map._scale);
                    var ypos = ((position.y + map._translatePoint.y) * map._scale);
                    $(bubble).attr({ "cx": xpos, "cy": ypos });
                }
                else {
                    bubble = document.getElementById(bubble.id);
                    var xpos = ((position.x + map._translatePoint.x) * map._scale);
                    var ypos = ((position.y + map._translatePoint.y) * map._scale);
                    var bubbleTop = ypos - (bubble.getBoundingClientRect().bottom - bubble.getBoundingClientRect().top) / 2;
                    var bubbleLeft = xpos - (bubble.getBoundingClientRect().right - bubble.getBoundingClientRect().left) / 2;

                    $(bubble).css({
                        "left": bubbleLeft,
                        "top": bubbleTop
                    });
                }
            }

            for (var key = 0; key < this._mapannotations.length; key++) {
                var item = this._mapannotations[key];

                var annotation;
                if (this.annotations.length > 0) {
                    annotation = this.annotations[key];
                }
                else if (this.annotationDataSource != null && this.annotationDataSource.length > 0) {
                    annotation = this.annotationDataSource[key];
                }
                var position;
                if (map._isTileMap) {
                    position = map._convertTileLatLongtoPoint(annotation.latitude, annotation.longitude);
                }
                else {
                    position = map._convertLatLongtoPointforAnnotation(annotation.latitude, annotation.longitude);
                }
                var xpos = position.x;
                var ypos = position.y;
                $(item).css({ "left": xpos, "top": ypos });
            }
            for (var key = 0; key < this._labelCollection.length; key++) {
                var item = this._labelCollection[key];
                var position = this._labelBounds[key];
                $(item).css("display", "block");
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 2);
                if (item[0].className = "labelLegend") {
                    item[0].className = "labelStyle";
                }
                item[0].innerHTML = this._labelData[key];
                $(item).css({ "left": xpos, "top": ypos });

            }
            if (this.labelSetting != null && this.labelSetting.showLabels) {
                this._validateSmartLabel(map);
            }
        },

        /**
         * Validates smart labels
         * @private
         */
        _validateSmartLabel: function (map) {
            this._smartLabels = [];
            var filledRects = [];
            if (this._labelCollection.length > 0) {
                for (var index = 0; index < this._mapShapes.length; index++) {
                    var shapemidPoint = this._labelBounds[index];
                    var midPoint = { x: (shapemidPoint.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.y + map._translatePoint.y) * map._scale };
                    var labelElement = this._labelCollection[index];
                    if (midPoint.x > 0 && midPoint.x < map._width && midPoint.y > 0 && midPoint.y < map._height) {
                        var shape = this._mapShapes[index].shape;
                        var bounds = shape.getBoundingClientRect();
                        var flag = false;
                        var labelSize = labelElement[0].getBoundingClientRect();
                        var labelHeight = labelSize.height + 2;
                        var labelWidth = labelSize.width + 2;
                        if (!_isSVG) {
                            bounds = { width: (bounds.right * map._scale) - (bounds.left * map._scale), height: (bounds.bottom * map._scale) - (bounds.top * map._scale) };
                            labelHeight = labelSize.bottom - labelSize.top;
                            labelWidth = labelSize.right - labelSize.left;
                        }
                        var leftPosition = 0;
                        var topPosition = 0;
                        var factor = 20;
                        var xArray = [];
                        var yArray = [];
                        if (labelHeight > bounds.height || labelWidth > bounds.width) {

                            if (this.labelSetting.enableSmartLabel) {
                                if (this.labelSetting.smartLabelSize == 'fixed') {
                                    labelHeight = 25;
                                    labelWidth = 15 * this.labelSetting.labelLength;
                                }
                                else {
                                    labelWidth *= 1.3;
                                    labelHeight = 25;
                                }
                                while (!flag) {
                                    if (factor > 400) {
                                        flag = true;
                                    }
                                    xArray = pushToArray(factor);
                                    yArray = xArray;
                                    for (var xIndex = 0; xIndex < xArray.length; xIndex++) {
                                        for (var yIndex = 0; yIndex < xArray.length; yIndex++) {
                                            leftPosition = yArray[yIndex];
                                            topPosition = xArray[xIndex];
                                            if (midPoint.x + leftPosition + labelWidth > map._width && midPoint.x < map._width) {
                                                leftPosition -= (midPoint.x + leftPosition + labelWidth) - map._width;
                                            }
                                            if (midPoint.x + leftPosition < 0) {
                                                leftPosition = 0;
                                            }
                                            if (midPoint.y + topPosition < 0) {
                                                topPosition = 0;
                                            }
                                            if (midPoint.y + topPosition + labelHeight > map._height && midPoint.y < map._height) {
                                                topPosition -= (midPoint.y + topPosition + labelHeight) - map._height;
                                            }
                                            var rect = { left: midPoint.x + leftPosition, top: midPoint.y + topPosition, height: labelHeight, width: labelWidth };
                                            if (isEmpty(rect)) {
                                                var ele = map._getIntersectedElements(rect, this._mapShapes);
                                                if (ele != null && ele.length == 0) {
                                                    flag = true;
                                                    xIndex = xArray.length;
                                                    yIndex = xArray.length;
                                                    labelElement[0].className = 'labelLegend';
                                                    if (this.labelSetting.smartLabelSize == 'fixed') {
                                                        labelElement[0].innerHTML = labelElement[0].innerHTML.substring(0, this.labelSetting.labelLength);
                                                    }
                                                    labelElement.mouseenter({ Param1: this, Param2: this._mapShapes[index], map: map }, map._polyEnterFunction);
                                                    labelElement.mousemove({ Param1: this, Param2: this._mapShapes[index] }, map._polyMoveFunction);
                                                    $(labelElement).css({ left: midPoint.x + leftPosition, top: midPoint.y + topPosition, "background-color": _isSVG ? shape.getAttribute("fill") : shape.fillcolor.value });
                                                    filledRects.push(rect);
                                                    var labelObj = {};
                                                    labelObj["shape"] = shape;
                                                    labelObj["legend"] = labelElement;
                                                    this._smartLabels.push(labelObj);
                                                }
                                            }
                                        }
                                    }
                                    factor += 10;
                                }
                            } else {
                                labelElement.css("display", "none");
                            }
                        }
                    }
                }
            }

            function pushToArray(value) {
                var array = [];
                array.push(0);
                for (var j = 10; j <= value; j += 20) {
                    array.push(-j);
                    array.push(j);
                }

                return array;
            }

            function isEmpty(rect) {
                for (var i = 0; i < filledRects.length; i++) {
                    if (isIntersect(rect, filledRects[i]))
                        return false;
                }
                return true;
            }

            function isIntersect(rect1, rect2) {

                if (rect1.left - 2 >= (rect2.left + rect2.width)
|| rect1.left - 2 + rect1.width <= rect2.left - 2
|| rect1.top - 2 >= rect2.top + rect2.height
|| rect1.top + rect1.height <= rect2.top - 2) {
                    return false;
                }
                return true;
            }


        },

        /**
         * Fills colors for the shapes based on color mapping
         * @private
         */
        _fillColors: function (value, colorMapping, shape) {

            if (colorMapping.rangeColorMapping != null) {

                this._fillRangeColors(value, colorMapping.rangeColorMapping, shape);
            }
            else if (colorMapping.equalColorMapping != null) {
                this._fillEqualColors(value, colorMapping, shape);
            }
        },

        /**
         * Fills colors for the shapes based on equal color mapping
         * @private
         */
        _fillEqualColors: function (value, colorMapping, shape) {
            $.each(colorMapping.equalColorMapping, function (index, gValue) {
                if (gValue.value == value) {
                    if (_isSVG) {
                        shape.setAttribute("fill", gValue.color);
                    }
                    else {
                        shape.fillcolor = gValue.color;
                    }
                }
            });
        },
        /**
         * Updates legend range
         * @private
         */
        _updateLegendRange: function (value, shapelayer, shape) {
            var colormapping = shapelayer.shapeSetting.colorMappings.rangeColorMapping;
            for (var index = 0; index < colormapping.length; index++) {
                var gradientCollection = null;
                var mapping = colormapping[index];
                if (shapelayer.shapeSetting.enableGradient) {
                    gradientCollection = this._generateGradientCollection(mapping.gradientColors);
                }
                if (value >= mapping.from && value <= mapping.to) {
                    var minValue = index;
                    if (index != 0)
                        minValue = index * 10;
                    var minRange = mapping.from;
                    var maxRange = mapping.from + ((mapping.to - mapping.from) / 10);
                    for (var i = minValue; i < minValue + 10; i++) {
                        if (value >= minRange && value <= maxRange) {
                            var obj = {};
                            if (shapelayer._legendRects[i] != undefined)
                                obj = shapelayer._legendRects[i];
                            if (shapelayer.shapeSetting.enableGradient) {
                                if (index != 0) {
                                    obj["color"] = gradientCollection[i - index * 10];
                                } else {
                                    obj["color"] = gradientCollection[i];
                                }
                            } else
                                obj["color"] = mapping.color;
                            return obj;
                        }
                        minRange = maxRange;
                        maxRange = maxRange + ((mapping.to - mapping.from) / 10);
                    }
                }
            }
        },

        /**
        * Fills colors for the shapes based on range color mapping
        * @private
        */
        _fillRangeColors: function (value, colormapping, shape) {

            for (var index = 0; index < colormapping.length; index++) {
                var mapping = colormapping[index];
                if (value >= mapping.from && value <= mapping.to) {
                    var shapeOpacity = this._getColorRatio(0.7, 1, value, mapping.from, mapping.to);
                    if (_isSVG) {
                        shape.setAttribute("fill", mapping.color);
                    }
                    else {
                        shape.fillcolor = mapping.color;
                    }
                    $(shape).css({ "opacity": shapeOpacity });
                }
            }
        },

        /**
        * Gets color ratio for gradient color mapping
        * @private
        */
        _getColorRatio: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var colorCode = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
            return colorCode;
        }

    };

    $.fn.pinchZoom = function (rootgroup, map) {

        var stateOrigin,
                    stateTf,
                    root = this[0],
                    startTouches = [],

           getTouchEventPoint = function (evt) {

               var p = root.createSVGPoint(),
                   targetTouches = evt.targetTouches,
                   offsetX,
                   offsetY;

               if (targetTouches.length) {
                   var touch = targetTouches[0];
                   offsetX = touch.pageX;
                   offsetY = touch.pageY;
               }

               p.x = offsetX;
               p.y = offsetY;

               return p;
           };

        handleTouchStart = function (evt) {
            var g = rootgroup;
            stateTf = g.getCTM().inverse();
            stateOrigin = getTouchEventPoint(evt).matrixTransform(stateTf);
            var touches = evt.touches;
            if (touches.length >= 2) {
                for (var i = 0; i < touches.length; i++) {
                    var touch = touches[i]
                      , found = false;
                    for (var j = 0; j < startTouches.length; j++) {
                        var startTouch = startTouches[j];
                        if (touch.identifier === startTouch.identifier) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        var touchCopy = $.extend({}, touch);
                        startTouches.push(touchCopy);
                    }
                }
            }

            evt.preventDefault();
        };
        getDistance = function (touch1, touch2) {
            var x = touch2.pageX - touch1.pageX,
                y = touch2.pageY - touch1.pageY;
            return Math.sqrt((x * x) + (y * y));
        };

        setCTM = function (element, matrix) {
            var isZoomIn = null;
            var s = "matrix(" + matrix.a + "," + matrix.b + "," + matrix.c + "," + matrix.d + "," + matrix.e + "," + matrix.f + ")";
            if (matrix.a > map._baseScale && map.model.enableZoom && map.model.enablePan) {
                if (matrix.a > map._scale) {
                    isZoomIn = true;
                }
                else if (matrix.a < map._scale) {
                    isZoomIn = false;
                }
                map._scale = matrix.a;
                map._translatePoint.x = matrix.e / matrix.a;
                map._translatePoint.y = matrix.f / matrix.a;
                map._applyTransform(map._scale, map._translatePoint);
                map.model.zoomLevel = map._scale - map._baseScale + 1;
                if (isZoomIn != null && isZoomIn) {
                    map._trigger("zoomedIn", { originalEvent: null, zoomLevel: map.model.zoomLevel });
                    map._updateSliderValue(false);
                    map._resizeShape();
                }
                else if (isZoomIn != null) {
                    map._trigger("zoomedOut", { originalEvent: null, zoomLevel: map.model.zoomLevel });
                    map._updateSliderValue(false);
                    map._resizeShape();
                }
                map._isDragging = true;
                map._refrshLayers();
            }

        };

        handleTouchEnd = function (evt) {
            var changedTouches = evt.changedTouches;
            for (var i = 0; i < changedTouches.length; i++) {
                var changedTouch = changedTouches[i];
                for (var j = 0; j < startTouches.length; j++) {
                    var startTouch = startTouches[j];
                    if (startTouch.identifier === changedTouch.identifier) {
                        var idx = startTouches.indexOf(startTouch);
                        startTouches.splice(idx, 1);
                    }
                }
            }
            evt.preventDefault();
        };

        updateTouch = function (dstTouch, srcTouch) {
            dstTouch.pageX = srcTouch.pageX;
            dstTouch.pageY = srcTouch.pageY;
        };

        handleTouchMove = function (evt) {
            evt.preventDefault();

            var g = rootgroup,
                touches = evt.touches,
                z,
                p,
                k;

            if (touches.length >= 2) {
                var touch0 = touches[0]
                  , touch1 = touches[1]
                  , startTouch0 = startTouches[0]
                  , startTouch1 = startTouches[1];
                z = getDistance(touch0, touch1) / getDistance(startTouch0, startTouch1);
                p = root.createSVGPoint();
                p.x = (touch0.pageX + touch1.pageX) / 2;
                p.y = (touch0.pageY + touch1.pageY) / 2;
                p = p.matrixTransform(g.getCTM().inverse());
                k = root.createSVGMatrix().translate(p.x, p.y).scale(z).translate(-p.x, -p.y);
                setCTM(g, g.getCTM().multiply(k));
                if (typeof stateTf === "undefined") {
                    stateTf = g.getCTM().inverse();
                }
                stateTf = stateTf.multiply(k.inverse());
                updateTouch(startTouch0, touch0);
                updateTouch(startTouch1, touch1);
            } else if (!startTouches.length) {
                p = getTouchEventPoint(evt).matrixTransform(stateTf);
                setCTM(g, stateTf.inverse().translate(p.x - stateOrigin.x, p.y - stateOrigin.y));
            }
        };
        this[0].addEventListener('touchstart', handleTouchStart, false);
        this[0].addEventListener('touchend', handleTouchEnd, false);
        this[0].addEventListener('touchmove', handleTouchMove, false);


    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html TreeMap elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejTreeMap
    * @requires jQuery
    * @requires ej.TreeMap
    * @requires jsrender
    * @requires properties
    * @classdesc Defined values to render a Treemap.
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create TreeMap
    * $('#container').ejTreeMap(); 	
    * &lt;/script&gt;
    */

    ej.widget({ "ejTreeMap": "ej.TreeMap" }, {

        validTags: ["div"],

        defaults: {
            /**
            * Specifies the leaf settings of the treemap
            * @default {borderThickness: 1,borderBrush: "white",showLabels: false,labelPath: null,labelTemplate: null}
			* @type {object}
			* @example 
            * //To set datasource API value during initialization 
			*   $("#container").ejTreeMap({leafItemSetting:{ labelPath: "GameName" ,showItem: true, showLabels: true, labelTemplate: temp}});
            * @example 
            * //Get or set the datasource API, after initialization:<br>
            *	//Gets the datasource value 
            *   var property =$("#container").data("ejTreeMap").model.leafItemSetting;<br> 
            *	//Sets the datasource value 
            *	$("#container").data("ejTreeMap").model.leafItemSetting = { labelPath: "GameName" ,showItem: true, showLabels: true, labelTemplate: temp}; 
            * @memberof ejTreeMap
            * @instance
            */
            leafItemSetting: {
                /**
                * Specifies the border thickness of the leaf item.
                * @default null
			    * @type {object}
			    * @example 
                * //To set borderThickness API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSetting:{ borderThickness: 1}});
                * @example 
                * //Get or set the borderThickness API, after initialization:<br>
                *	//Gets the borderThickness value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSetting.borderThickness;<br> 
                *	//Sets the borderThickness value 
                *	$("#container").data("ejTreeMap").model.leafItemSetting.borderThickness = { borderThickness: 1}; 
                * @alias ejTreeMap#leafItemSetting->borderThickness
                * @instance
                */
                borderThickness: 1,

                /**
                * Specifies the border bruch color of the leaf item.
                * @default "white"
			    * @type {string}
			    * @example 
                * //To set borderBrush API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSetting:{ borderBrush: "white"}});
                * @example 
                * //Get or set the borderBrush API, after initialization:<br>
                *	//Gets the borderBrush value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSetting.borderBrush;<br> 
                *	//Sets the borderBrush value 
                *	$("#container").data("ejTreeMap").model.leafItemSetting.borderBrush = { borderBrush: "white"}; 
                * @alias ejTreeMap#leafItemSetting->borderBrush
                * @instance
                */
                borderBrush: "white",

                /**
                * Shows or hides the label of the leaf item.
                * @default "false"
			    * @type {boolean}
			    * @example 
                * //To set showItem API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSetting:{ showItem: "false"}});
                * @example 
                * //Get or set the showItem API, after initialization:<br>
                *	//Gets the showItem value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSetting.showItem;<br> 
                *	//Sets the showItem value 
                *	$("#container").data("ejTreeMap").model.leafItemSetting.showItem = { showItem: "false"}; 
                * @alias ejTreeMap#leafItemSetting->showItem
                * @instance
                */
                showItem: false,

                /**
                * Shows or hides the label of the leaf item.
                * @default "false"
			    * @type {boolean}
			    * @example 
                * //To set showLabels API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSetting:{ showLabels: "false"}});
                * @example 
                * //Get or set the showLabels API, after initialization:<br>
                *	//Gets the showLabels value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSetting.showLabels;<br> 
                *	//Sets the showLabels value 
                *	$("#container").data("ejTreeMap").model.leafItemSetting.showLabels = { showLabels: "false"}; 
                * @alias ejTreeMap#leafItemSetting->showLabels
                * @instance
                */
                showLabels: false,

                /**
                * Specifies the label path of the leaf item.
                * @default null
                * @type {string}
                * @example 
                * //To set labelPath API value during initialization 
                *   $("#container").ejTreeMap({leafItemSetting:{ labelPath: "GameName"}});
                * @example 
                * //Get or set the labelPath API, after initialization:<br>
                *	//Gets the labelPath value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSetting.labelPath;<br> 
                *	//Sets the labelPath value 
                *	$("#container").data("ejTreeMap").model.leafItemSetting.labelPath = { labelPath: "GameName"}; 
                * @alias ejTreeMap#leafItemSetting->labelPath
                * @instance
                */
                labelPath: null,

                /**
                * Specifies the label template of the leaf item.
                * @default null
			    * @type {string}
			    * @example 
                * //To set labelTemplate API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSetting:{ labelTemplate: "temp"}});
                * @example 
                * //Get or set the labelTemplate API, after initialization:<br>
                *	//Gets the labelTemplate value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSetting.labelTemplate;<br> 
                *	//Sets the labelTemplate value 
                *	$("#container").data("ejTreeMap").model.leafItemSetting.labelTemplate = { labelTemplate: "temp"}; 
                * @alias ejTreeMap#leafItemSetting->labelTemplate
                * @instance
                */
                labelTemplate: null
            },
            /**		
			* Specifies the datasource of the treemap
            * @default null
			* @type {object}
			* @example 
            * //To set datasource API value during initialization 
			*   $("#container").ejTreeMap({datasource:medal_data});
            * @example 
			* //Get or set the datasource API, after initialization:<br>
			*	//Gets the datasource value 
            *   var property =$("#container").data("ejTreeMap").model.datasource;<br> 
            *	//Sets the datasource value 
			*	$("#container").data("ejTreeMap").model.datasource = medal_data; 
			* @memberof ejTreeMap
			* @instance
            */
            dataSource: null,

            /**		
			* Specifies the color valuepath of the treemap
            * @default null
			* @type {string}
			* @example 
            * //To set colorValuePath API value during initialization 
			*   $("#container").ejTreeMap({colorValuePath:'GoldMedals'});
            * @example 
			* //Get or set the colorValuePath API, after initialization:<br>
			*	//Gets the colorValuePath value 
            *   var property =$("#container").data("ejTreeMap").model.colorValuePath;<br> 
            *	//Sets the colorValuePath value 
			*	$("#container").data("ejTreeMap").model.colorValuePath = 'GoldMedals'; 
			* @memberof ejTreeMap
			* @instance
            */
            colorValuePath: null,

            /**		
			* Specifies the weight valuepath of the treemap
            * @default null
			* @type {string}
			* @example 
            * //To set weightValuePath API value during initialization 
			*   $("#container").ejTreeMap({weightValuePath:'TotalMedals'});
            * @example 
			* //Get or set the weightValuePath API, after initialization:<br>
			*	//Gets the weightValuePath value 
            *   var property =$("#container").data("ejTreeMap").model.weightValuePath;<br> 
            *	//Sets the weightValuePath value 
			*	$("#container").data("ejTreeMap").model.weightValuePath = 'TotalMedals'; 
			* @memberof ejTreeMap
			* @instance
            */
            weightValuePath: null,


            treeMapItems: [],

            /**		
			* Specifies the legend visibility  status of the treemap
            * @default false
			* @type {boolean}
			* @example 
            * //To set showLegend API value during initialization 
			*   $("#container").ejTreeMap({showLegend:false});
            * @example 
			* //Get or set the showLegend API, after initialization:<br>
			*	//Gets the showLegend value 
            *   var property =$("#container").data("ejTreeMap").model.showLegend;<br> 
            *	//Sets the showLegend value 
			*	$("#container").data("ejTreeMap").model.showLegend = false; 
			* @memberof ejTreeMap
			* @instance
            */
            showLegend: false,

            /**		
			* Specifies the border brush color of the treemap
            * @default "white"
			* @type {string}
			* @example 
            * //To set borderBrush API value during initialization 
			*   $("#container").ejTreeMap( {borderBrush:'white'});
            * @example 
			* //Get or set the borderBrush API, after initialization:<br>
			*	//Gets the borderBrush value 
            *   var property =$("#container").data("ejTreeMap").model.borderBrush;<br> 
            *	//Sets the borderBrush value 
			*	$("#container").data("ejTreeMap").model.borderBrush = 'white'; 
			* @memberof ejTreeMap
			* @instance
            */
            borderBrush: "white",

            /**		
			* Specifies the border thickness of the treemap
            * @default 1
			* @type {number}
			* @example 
            * //To set borderThickness API value during initialization 
			*   $("#container").ejTreeMap({borderThickness:1});
            * @example 
			* //Get or set the borderThickness API, after initialization:<br>
			*	//Gets the borderThickness value 
            *   var property =$("#container").data("ejTreeMap").model.borderThickness;<br> 
            *	//Sets the borderThickness value 
			*	$("#container").data("ejTreeMap").model.borderThickness = 1; 
			* @memberof ejTreeMap
			* @instance
            */
            borderThickness: 1,

            /**		
			* Specifies whether treemap need to resize when container is resized
            * @default true
			* @type {boolean}
			* @example 
            * //To set canResize API value during initialization 
			*   $("#container").ejTreeMap({canResize:false});
            * @example 
			* //Get or set the canResize API, after initialization:<br>
			*	//Gets the canResize value 
            *   var property =$("#container").data("ejTreeMap").model.canResize;<br> 
            *	//Sets the canResize value 
			*	$("#container").data("ejTreeMap").model.canResize = false; 
			* @memberof ejTreeMap
			* @instance
            */
            canResize: true,

            /**		
			* Specifies the items layout mode of the treemap
            * @default "Squarified"
			* @type {string}
			* @example 
            * //To set itemsLayoutMode API value during initialization 
			*   $("#container").ejTreeMap({itemsLayoutMode:'Squarified'});
            * @example 
			* //Get or set the itemsLayoutMode API, after initialization:<br>
			*	//Gets the itemsLayoutMode value 
            *   var property =$("#container").data("ejTreeMap").model.itemsLayoutMode;<br> 
            *	//Sets the itemsLayoutMode value 
			*	$("#container").data("ejTreeMap").model.itemsLayoutMode = 'Squarified'; 
			* @memberof ejTreeMap
			* @instance
            */
            itemsLayoutMode: "Squarified",

            // Level Collection
            /**		
            * Specify levels of treemap.
            * @default []
            * @type {treeMapLevel} 
            * @example  
            * // Set the levels during initialization. 		
            *   $("#container").ejTreeMap({	levels: [{ groupPath: "Continent", groupGap: 5, headerHeight: 30, headerTemplate: 'headertemplate' }]})
            * @example 
            * //Get or set the levels after initialization:<br>
            *   //Gets the levels from map.
            *   var levels =$("#container").data("ejTreeMap").model.levels[levelIndex];
            *   //Sets the levels to map.
            *   $("#container").data("ejTreeMap").model.levels[levelIndex]  = { groupPath: "Continent", groupGap: 5, headerHeight: 30, headerTemplate: 'headertemplate' };
            * @memberof ejTreeMap
            * @instance
            */
            levels: [],

            /**		
			* Specifies the treeMapLegend of the treemap
            * @default null
			* @type {object}
			* @example 
            * //To set treeMapLegend API value during initialization 
			*   $("#container").ejTreeMap( {treeMapLegend:{ legendTemplate: null, legendIconHeight: 15, legendIconWidth: 25 }});
            * @example 
			* //Get or set the treeMapLegend API, after initialization:<br>
			*	//Gets the treeMapLegend value 
            *   var property =$("#container").data("ejTreeMap").model.treeMapLegend;<br> 
            *	//Sets the treeMapLegend value 
			*	$("#container").data("ejTreeMap").model.treeMapLegend = { legendTemplate: null, legendIconHeight: 15, legendIconWidth: 25 }; 
			* @memberof ejTreeMap
			* @instance
            */
            treeMapLegend: { legendTemplate: null, legendIconHeight: 15, legendIconWidth: 25 },

            /**		
			* Specifies the range color mapping of the treemap
            * @default []
			* @type {object}
			* @example 
            * //To set rangeColorMapping API value during initialization 
			*   $("#container").ejTreeMap( {rangeColorMapping:[{ color: "#77D8D8",legendLabel:"1% Growth", from: "0", to: "1" }]});
            * @example 
			* //Get or set the rangeColorMapping API, after initialization:<br>
			*	//Gets the rangeColorMapping value 
            *   var property =$("#container").data("ejTreeMap").model.rangeColorMapping;<br> 
            *	//Sets the rangeColorMapping value 
			*	$("#container").data("ejTreeMap").model.rangeColorMapping = [{ color: "#77D8D8",legendLabel:"1% Growth", from: "0", to: "1" }]; 
			* @memberof ejTreeMap
			* @instance
            */
            rangeColorMapping: [],
            /**		
			* Specifies the uniform color mapping of the treemap
            * @default { color: null }
			* @type {object}
			* @example 
            * //To set uniColorMapping API value during initialization 
			*   $("#container").ejTreeMap( {uniColorMapping{ color: null }});
            * @example 
			* //Get or set the uniColorMapping API, after initialization:<br>
			*	//Gets the uniColorMapping value 
            *   var property =$("#container").data("ejTreeMap").model.uniColorMapping;<br> 
            *	//Sets the uniColorMapping value 
			*	$("#container").data("ejTreeMap").model.uniColorMapping = { color: null }; 
			* @memberof ejTreeMap
			* @instance
            */
            uniColorMapping: { color: null },

            /**		
			* Specifies the desaturation color mapping of the treemap
            * @default { from: 0, to: 0, color: 'blue', rangeMinimum: 0, rangeMaximum: 0 }
			* @type {object}
			* @example 
            * //To set desaturationColorMapping API value during initialization 
			*   $("#container").ejTreeMap( {desaturationColorMapping{ from:1, to:0.5 ,color:"#41B8C4", rangeMinimum:2000,rangeMaximum:8000}});
            * @example 
			* //Get or set the desaturationColorMapping API, after initialization:<br>
			*	//Gets the desaturationColorMapping value 
            *   var property =$("#container").data("ejTreeMap").model.desaturationColorMapping;<br> 
            *	//Sets the desaturationColorMapping value 
			*	$("#container").data("ejTreeMap").model.desaturationColorMapping = { from:1, to:0.5 ,color:"#41B8C4", rangeMinimum:2000,rangeMaximum:8000}; 
			* @memberof ejTreeMap
			* @instance
            */
            desaturationColorMapping: { from: 0, to: 0, color: null, rangeMinimum: 0, rangeMaximum: 0 },

            /**		
			* Specifies the palette color mapping of the treemap
            * @default null
			* @type {object}
			* @example 
            * //To set paletteColorMapping API value during initialization 
			*   $("#container").ejTreeMap( {paletteColorMapping{colors: ["red","green","blue", "yellow"]}});
            * @example 
			* //Get or set the paletteColorMapping API, after initialization:<br>
			*	//Gets the paletteColorMapping value 
            *   var property =$("#container").data("ejTreeMap").model.paletteColorMapping;<br> 
            *	//Sets the paletteColorMapping value 
			*	$("#container").data("ejTreeMap").model.paletteColorMapping = {colors: ["red","green","blue", "yellow"]}; 
			* @memberof ejTreeMap
			* @instance
            */
            paletteColorMapping: null,

            /**		
			* Specifies whether treemap item need to highlighted on selection
            * @default false
			* @type {boolean}
			* @example 
            * //To set highlightOnSelection API value during initialization 
			*   $("#container").ejTreeMap({highlightOnSelection:false});
            * @example 
			* //Get or set the highlightOnSelection API, after initialization:<br>
			*	//Gets the highlightOnSelection value 
            *   var property =$("#container").data("ejTreeMap").model.highlightOnSelection;<br> 
            *	//Sets the highlightOnSelection value 
			*	$("#container").data("ejTreeMap").model.highlightOnSelection = false; 
			* @memberof ejTreeMap
			* @instance
            */
            highlightOnSelection: false,

            /**		
			* Specifies whether treemap tooltip need to be visible
            * @default false
			* @type {boolean}
			* @example 
            * //To set showTooltip API value during initialization 
			*   $("#container").ejTreeMap({showTooltip:false});
            * @example 
			* //Get or set the showTooltip API, after initialization:<br>
			*	//Gets the showTooltip value 
            *   var property =$("#container").data("ejTreeMap").model.showTooltip;<br> 
            *	//Sets the showTooltip value 
			*	$("#container").data("ejTreeMap").model.showTooltip = false; 
			* @memberof ejTreeMap
			* @instance
            */
            showTooltip: false,

            /**		
			* Specifies the tooltip template of the treemap
            * @default null
			* @type {string}
			* @example 
            * //To set tooltipTemplate API value during initialization 
			*   $("#container").ejTreeMap({tooltipTemplate:'template'});
            * @example 
			* //Get or set the tooltipTemplate API, after initialization:<br>
			*	//Gets the tooltipTemplate value 
            *   var property =$("#container").data("ejTreeMap").model.tooltipTemplate;<br> 
            *	//Sets the tooltipTemplate value 
			*	$("#container").data("ejTreeMap").model.tooltipTemplate = 'template'; 
			* @memberof ejTreeMap
			* @instance
            */
            tooltipTemplate: null,

            /**		
			* Specifies the border thickness when treemap items is highlighted in the treemap
            * @default 5
			* @type {number}
			* @example 
            * //To set highlightBorderThickness API value during initialization 
			*   $("#container").ejTreeMap({highlightBorderThickness:5});
            * @example 
			* //Get or set the highlightBorderThickness API, after initialization:<br>
			*	//Gets the highlightBorderThickness value 
            *   var property =$("#container").data("ejTreeMap").model.highlightBorderThickness;<br> 
            *	//Sets the highlightBorderThickness value 
			*	$("#container").data("ejTreeMap").model.highlightBorderThickness = 5; 
			* @memberof ejTreeMap
			* @instance
            */
            highlightBorderThickness: 5,

            /**		
			* Specifies the highlight border brush of treemap
            * @default "gray"
			* @type {string}
			* @example 
            * //To set highlightBorderBrush API value during initialization 
			*   $("#container").ejTreeMap({highlightBorderBrush:'gray'});
            * @example 
			* //Get or set the highlightBorderBrush API, after initialization:<br>
			*	//Gets the highlightBorderBrush value 
            *   var property =$("#container").data("ejTreeMap").model.highlightBorderBrush;<br> 
            *	//Sets the highlightBorderBrush value 
			*	$("#container").data("ejTreeMap").model.highlightBorderBrush = 'gray'; 
			* @memberof ejTreeMap
			* @instance
            */
            highlightBorderBrush: "gray"
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            dataSource: "data",
            treeMapItems: "array",
            levels: "array",
            rangeColorMapping: "array",
            paletteColorMapping: "array"
        },

        observables: ["dataSource"],

        _initPrivateProperties: function () {
            this._svgDocument = null;
            this._templateDiv = null;
            this._backgroundTile = null;
            this._height = 500;
            this._initDiv = null;
            this._width = 500;
            this._svgns = "http://www.w3.org/2000/svg";
            this._prevSelectedItem = null;
            /**     
        * Fires on treemap item selected. 
        * @event
        * @name ejTreeMap#treeMapItemSelected	
        * @example 
        * //treemap item selected event for treemap
        * $("#container").ejTreeMap({
        *    treeMapItemSelected: function () {}
        * });
        * @memberof ejTreeMap
        * @instance
        */
            this.treeMapItemSelected = null;
            this._toolTipElement = null;
            this._rootTreeMapItems = [];
            this._treeMapHeaders = [];
            this._treeMapLabels = [];
            this._labelTemplateElements = [];
            this._itemGroups = [];

        },

        /**
          * To configure the properties at runtime using SetModel		
          * @private
          */
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "itemsLayoutMode":
                        this.model.itemsLayoutMode = options[prop];
                        this.refresh();
                        break;
                }
            }
        },

        /**
        * Create the treemap widget
        * @private
        */
        _init: function () {

            this._initPrivateProperties();
            this._levelInitialize();
            this.refresh();

        },

        /**
        * Destroy the map widget
        * @private
        */
        _destroy: function () {

        },

        /**
        * Create level for treemap
        * @private
        */
        _levelInitialize: function () {
            var proxy = this;
            if (this.model.levels != null) {
                $.each(this.model.levels, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = new treeMapLevel();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.levels.treeMapLevel = new treeMapLevel();

            }
        },

        /**
       * Checks array object
       * @private
       */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = new treeMapLevel();
                    proxy._loadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

        /**
          * Loads individual default values
          * @private
          */
        _loadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._loadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },


        /**     
       * Fires on treemap refresh.
       * @event
       * @name ejTreeMap#refresh
       * @example 
       * //refresh event for treemap
       * $("#container").ejTreeMap({
       *    refresh: function () {}
       * });
       * @memberof ejTreeMap
       * @instance
       */
        refresh: function () {
            this._initPrivateProperties();
            $(this.element).empty();
            if (this._svgDocument != null) {
                $(this._svgDocument).empty();
            }
            this._height = this.element.height();
            this._width = this.element.width();
            if (this._height == 0) {
                this._height = this.element[0].parentElement.clientHeight;
            }
            if (this._width == 0) {
                this._width = this.element[0].parentElement.clientWidth;
            }
            $(this.element).css({ height: this._height, width: this._width });
            if (this._height != 0 && this._width != 0) {
                this._templateDiv = $("<div class='_templateDiv'></div>");
                this._templateDiv.css({
                    'pointer-events': 'none', 'overflow': 'hidden', 'float': 'left',
                    'z-index': '2', 'height': this._height, 'width': this._width, "position": "absolute"
                });
                if (!window.SVGSVGElement) {
                    this._templateDiv.css("position", "absolute");
                }
                this._backgroundTile = $('<div id="backgroundTile" style="overflow:hidden;position:absolute;z-index:0;"></div>');
                this._groupLevels();
                if (this.model.showLegend) {
                    this._generateLegend();
                }
                this._generateToolTip();
                this._renderLabels();
                if (this.model.canResize) {
                    this._on($(window), "resize", this._treeMapResize);
                }
            }
        },

        /**
      * Section For handling treemap resize
      * @private
      */
        _treeMapResize: function (event) {
            event.preventDefault();
            event.stopPropagation();
            var treemap = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                treemap.refresh();
            }, 500);
        },

        /**
          * Groups levels of treemap
          * @private
          */
        _groupLevels: function () {
            var subItems = null;
            var rootItems = this._getTopGroupitem(this.model.leafItemSetting.labelPath, this.model.dataSource, 0);
            subItems = rootItems;
            if (this.model.levels.length == 0) {
                var levelItem = this.model.leafItemSetting;
                this._rootTreeMapItems = rootItems;
                this._getTopLevels(rootItems[0].innerItems, levelItem);
                this._generateTreeMapItems(rootItems[0].innerItems, "gray", this.model.colorValuePath);
                if (levelItem.showLabels || levelItem.showItem) {
                    this._generateLabels(rootItems[0].innerItems, levelItem.labelTemplate, levelItem.showItem);
                }
            }
            if (this.model.levels.length > 0) {
                for (var i = 0; i <= this.model.levels.length; i++) {
                    var levelItem = this.model.levels[i];
                    if (i != 0) {
                        var cloneItems = subItems;
                        var PrevItem = this.model.levels[i - 1];
                        subItems = [];
                        if (levelItem == null) levelItem = this.model.leafItemSetting;
                        this._generateSubItems(cloneItems, levelItem, subItems, PrevItem);
                        this._rootTreeMapItems = subItems;
                        if (i == this.model.levels.length) {
                            this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);

                        }
                        if (levelItem.showLabels || levelItem.showItem) {
                            this._generateLabels(subItems, levelItem.labelTemplate, levelItem.showItem);
                        }
                    }
                    else {
                        var rootItems1 = this._getGroupitem(levelItem.groupPath, rootItems[0].Data, levelItem.headerHeight);
                        this._rootTreeMapItems = rootItems1;
                        this._getTopLevels(rootItems1, levelItem);
                        subItems = rootItems1;
                    }
                }

            }
        },

        /**
          * Gets top levels of treemap
          * @private
          */
        _getTopLevels: function (rootItems, level) {
            var gap = (level.groupGap != null) ? level.groupGap : 0;
            var layout = this.model.itemsLayoutMode;
            var legendHeight = 0;
            if (this.model.showLegend && this.model.treeMapLegend != null) {
                legendHeight = this.model.treeMapLegend.legendIconHeight + 20;
            }
            if (layout == 'SliceAndDiceHorizontal') {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, 0, true, legendHeight);
            }
            else if (layout == 'SliceAndDiceVertical') {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, 0, false, legendHeight);
            }
            else if (layout == 'SliceAndDiceAuto') {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, 0, null, legendHeight);
            }
            else {
                this._calculateSquarifiedItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, 0, legendHeight);
            }
        },


        /**
         * Generates sub treemap items
         * @private
         */
        _generateSubItems: function (cloneItems, levelItem, subItems, PrevItem) {
            var layout = this.model.itemsLayoutMode;
            var cloneItemsCount = cloneItems.length;
            var path = levelItem.groupPath;
            if (path == null) path = this.model.leafItemSetting.labelPath;
            if (path == null) path = this.model.weightValuePath;
            var gap = levelItem.groupGap != null ? levelItem.groupGap : 0;
            var headerHeight = (PrevItem.showHeader || PrevItem.showHeader == null) ? PrevItem.headerHeight : 0;
            var totalItemsCount = 0;
            for (var j = 0; j < cloneItemsCount; j++) {
                var treeItems = this._getGroupitem(path, cloneItems[j].Data, levelItem.headerHeight);
                if (layout == 'SliceAndDiceHorizontal') {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + (PrevItem.groupPadding), cloneItems[j].TopPosition + (PrevItem.groupPadding), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), gap, headerHeight, true, 0);
                }
                else if (layout == 'SliceAndDiceVertical') {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + (PrevItem.groupPadding), cloneItems[j].TopPosition + (PrevItem.groupPadding), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), gap, headerHeight, false, 0);
                }
                else if (layout == 'SliceAndDiceAuto') {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + (PrevItem.groupPadding), cloneItems[j].TopPosition + (PrevItem.groupPadding), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), gap, headerHeight, null, 0);
                }
                else {
                    this._calculateSquarifiedItemSize(treeItems, cloneItems[j].LeftPosition + (PrevItem.groupPadding), cloneItems[j].TopPosition + (PrevItem.groupPadding), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), gap, headerHeight, 0);
                }
                var headerItem = this._rootTreeMapItems[j];
                headerItem.headerHeight = PrevItem.headerHeight;
                headerItem.showHeader = PrevItem.showHeader;
                headerItem.headerWidth = cloneItems[j].ItemWidth;
                headerItem.headerTemplate = PrevItem.headerTemplate;
                headerItem.headerLeftPosition = cloneItems[j].LeftPosition;
                headerItem.headerTopPosition = cloneItems[j].TopPosition;
                if (PrevItem.showHeader || PrevItem.showHeader == null) {
                    this._createBackGround(headerItem, PrevItem);
                    this._generateHeaders(headerItem, PrevItem);
                }

                cloneItems[j].ChildtreeMapItems = treeItems;
                cloneItems[j].GroupingLevel = this.model.levels.indexOf(PrevItem) + 1;
                for (var k = 0; k < treeItems.length; k++) {
                    subItems[totalItemsCount] = treeItems[k];
                    subItems[totalItemsCount].ItemHeight -= PrevItem.groupPadding;
                    subItems[totalItemsCount].ItemWidth -= PrevItem.groupPadding;
                    subItems[totalItemsCount].LeftPosition += PrevItem.groupBorderThickness;
                    subItems[totalItemsCount].TopPosition += PrevItem.groupBorderThickness;
                    totalItemsCount++;
                }
            }
            if (PrevItem.showLabels || PrevItem.showItem) {
                this._generateLabels(cloneItems, PrevItem.labelTemplate, false);
            }
        },
        /**
        * create background for treemap items
        * @private
        */
        _createBackGround: function (header, level) {
            var rect = $("<div />");
            var height = (header.showHeader || header.showHeader == null) ? header.ItemHeight - (2 * level.groupBorderThickness) : 0;
            var top = (header.showHeader || header.showHeader == null) ? header.TopPosition : 0;
            rect.css({
                "height": height + "px",
                "width": header.ItemWidth - (2 * level.groupBorderThickness) + "px",
                "left": header.LeftPosition + "px",
                "top": top + "px",
                "border-style": "solid",
                "border-width": level.groupBorderThickness + "px",
                "border-color": level.groupBorderColor,
                "position": "absolute",
                "background-color": level.groupBackground
            });
            rect.appendTo(this._backgroundTile);
        },

        /**
        * Generates labels for treemap items
        * @private
        */
        _generateLabels: function (Items, labelTemplate, showItem) {
            for (var i = 0; i < Items.length; i++) {
                var item = Items[i];
                if (showItem) {
                    var textcon = $("<label></label>");
                    $(textcon).css({
                        "font-family": "Segoe UI", "font-size": "14px;", "color": "white", "position": "absolute",
                        "overflow": "hidden", "width": item.ItemWidth - 10 + "px", "left": item.LeftPosition + 10 + "px",
                        "top": item.TopPosition + 5 + "px", "height": item.ItemHeight - 10, "font-weight": "normal", "pointer-events": "none"
                    });
                    textcon[0].innerHTML = item.label;
                    this._treeMapLabels.push(textcon);
                    this._templateDiv.append(textcon);
                }
                if (labelTemplate != null) {
                    var OriginalNode = $("#" + labelTemplate);
                    var item_templateDiv = $("<div style='overflow:hidden;display:block;position:absolute;pointer-events: none;'></div>");
                    item_templateDiv[0].data = item;
                    var tmpl2 = $.templates(OriginalNode.html());
                    var htmlString = tmpl2.render(item);
                    $(item_templateDiv).html(htmlString);
                    $(item_templateDiv).css({
                        "left": item.LeftPosition + (item.ItemWidth / 2),
                        "top": item.TopPosition + (item.ItemHeight / 2)
                    });
                    this._templateDiv.append(item_templateDiv);
                    this._labelTemplateElements.push(item_templateDiv);
                }
            }

        },

        /**
        * Generates tooltip for treemap
        * @private
        */
        _generateToolTip: function () {

            if (this.model.showTooltip) {
                var tooltip_templateDiv = $("<div class='tooltipelement' style='display:none;position:absolute;z-index:1000;pointer-events:none;'></div>");
                $(document.body).append(tooltip_templateDiv);
                this._toolTipElement = tooltip_templateDiv;
                if (this.model.tooltipTemplate == null) {

                    var path;
                    if (this.model.leafItemSetting.labelPath != null)
                        path = this.model.leafItemSetting.labelPath;
                    else
                        path = this.model.weightValuePath;
                    if (path != null) {
                        this.model.tooltipTemplate = 'defaultTooltip';

                        this.element.append($('<div id="defaultTooltip" style="display:none;"><div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">{{:' + path + '}}</label></p></div></div></div>'));
                    }
                }
            }

        },

        /**
        * Generates legend for treemap
        * @private
        */
        _generateLegend: function () {
            if (this.model.rangeColorMapping != null) {
                var colormapping = this.model.rangeColorMapping;
                var xPos = 0;
                var yPos = 0;
                var treeMaplegend = this.model.treeMapLegend;
                var legendtype = treeMaplegend.legendTemplate;
                for (var i = 0; i < colormapping.length; i++) {
                    var legendItem;

                    if (legendtype == "Ellipse") {
                        legendItem = this._getEllipseLegend(treeMaplegend, xPos, yPos);
                    }
                    else {
                        legendItem = this._getRectLegend(treeMaplegend, xPos, yPos);
                    }

                    $(legendItem).css("background-color", colormapping[i].color);
                    legendItem.appendTo(this._svgDocument);
                    xPos += treeMaplegend.legendIconWidth + 5;
                    var legendText = $("<div/>");
                    legendText.css({
                        "left": xPos + "px",
                        "top": yPos + "px",
                        "position": "absolute"
                    });
                    $(legendText).css({ "font-size": "15px", "color": "gray" });
                    legendText[0].innerHTML = colormapping[i].legendLabel;
                    legendText.appendTo(this._svgDocument);
                    var bounds = legendText[0].getBoundingClientRect();
                    var boundwidth = window.SVGSVGElement ? bounds.width : bounds.right - bounds.left;
                    xPos += boundwidth + 18;

                }
            }
        },

        /**
        * Gets ellipse legend
        * @private
        */
        _getEllipseLegend: function (legengItem, xPos, yPos) {

            var rect = $("<div class='e-mapLegend'/>");
            rect.css({
                "height": this.model.treeMapLegend.legendIconHeight + "px",
                "width": this.model.treeMapLegend.legendIconWidth + "px",
                "border-radius": this.model.treeMapLegend.legendIconHeight / 2 + "px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;

        },

        /**
        * Gets rect legend
        * @private
        */
        _getRectLegend: function (legengItem, xPos, yPos) {
            var rect = $("<div />");
            rect.css({
                "height": this.model.treeMapLegend.legendIconHeight + "px",
                "width": this.model.treeMapLegend.legendIconWidth + "px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;
        },

        /**
        * Gets position from parent
        * @private
        */
        _getPositionFromParent: function (element, parentelement) {
            var child = element.getBoundingClientRect();
            var parent = parentelement.getBoundingClientRect();
            var width = window.SVGSVGElement ? child.width : child.right - child.left;
            var height = window.SVGSVGElement ? child.height : child.bottom - child.top;
            return { left: child.left - parent.left, top: child.top - parent.top, height: height, width: width };
        },

        /**
       * Renders labels
       * @private
       */
        _renderLabels: function () {

            for (var j = 0; j < this._labelTemplateElements.length; j++) {
                var label = this._labelTemplateElements[j];
                var bounds = this._getPositionFromParent(label[0], this._templateDiv[0]);
                var item = label[0].data;
                var leftPos = $(label).css('left');
                var topPos = $(label).css('top');
                $(label).css({
                    "left": parseInt(leftPos) - (bounds.width / 2),
                    "top": parseInt(topPos) - (bounds.height / 2),
                    "pointer-events": "none"
                });


            }

        },

        /**
       * Generates headers
       * @private
       */
        _generateHeaders: function (headerValue, level) {
            if (headerValue.headerTemplate == null) {
                var headerItem = $('<div style="display:block;position:absolute;pointer-events: stroke;overflow: hidden;"><label style="color:#262626;font-size:15px;font-weight:normal;font-family:Segoe UI;">' + headerValue.header + '</label></div>');
                $(headerItem).css({
                    "left": headerValue.headerLeftPosition + 2,
                    "top": headerValue.headerTopPosition + 2,
                    "width": headerValue.headerWidth - level.groupPadding,
                    "height": headerValue.headerHeight,
                    "margin-left": level.groupPadding
                });
                this._templateDiv.append(headerItem);
            }
            else {
                var item_templateDiv = $("<div style='display:block;position:absolute;pointer-events: none;overflow: hidden;'></div>");
                this._templateDiv.append(item_templateDiv);
                $(item_templateDiv).css({
                    "left": headerValue.headerLeftPosition,
                    "width": headerValue.headerWidth, "top": headerValue.headerTopPosition,
                    "height": headerValue.headerHeight
                });
                var htmlString = $("#" + headerValue.headerTemplate).render(headerValue);
                $(item_templateDiv).html(htmlString);

            }
        },

        /**
          * Generates treemap items
          * @private
          */
        _generateTreeMapItems: function (Items, fill, path) {
            this._getTreeMapItemTemplate(Items);
            if (Items != null) {
                for (var i = 0; i < Items.length; i++) {
                    var item = Items[i];
                    var rect = this._svgDocument[0].childNodes[i];
                    var ColorValue = item._generateColorMappings(this.model);
                    if (ColorValue != null) {
                        $(rect).css("background-color", ColorValue);
                    }
                    else {
                        $(rect).css("background-color", "#BBBDC0");
                    }
                    if (item.Data != null && item.Data.length > 0) {
                        item.Data = item.Data[0];
                    }
                    item.Rectangle = rect;
                    $(rect).mouseenter({ Param1: item.Data, Param2: this }, this._rectMouseEnter);
                    $(rect).mousemove({ Param1: item.Data, Param2: this }, this._rectMouseMove);
                    $(rect).mouseleave({ Param1: this }, this._rectMouseLeave);
                    $(rect).mouseup({ Param1: this }, this._rectMouseLeave);
                    $(rect).mousedown({ Param1: item.Data, Param2: this }, this._rectClickFunction);
                }
                if (this.model.rangeColorMapping.length == 0 && this.model.uniColorMapping.color == null && this.model.desaturationColorMapping.color != null) {
                    this._setDesaturationColor(Items, this.model.desaturationColorMapping);
                }
                else if (this.model.PaletteColorMapping != null) {
                    this._setPaletteColor(Items, this.model.PaletteColorMapping);
                }
            }
        },

        /**
          * Gets treemap item template
          * @private
          */
        _getTreeMapItemTemplate: function (Items) {

            var rects = "";
            for (var i = 0; i < Items.length; i++) {
                var item = Items[i];
                rects += '<div style="border-style:solid;position:absolute;left:' + item.LeftPosition + 'px;top:' + item.TopPosition + 'px;height:' + item.ItemHeight + 'px;width:' + item.ItemWidth + 'px;border-width:' + this.model.leafItemSetting.borderThickness + 'px;border-color:' + this.model.leafItemSetting.borderBrush + '"></div>';
            }
            this._svgDocument = $('<div style= "overflow:hidden;position:absolute' +
                              'z-index:1;"' +
                             'id="svgDocument">' + rects + '</div>');
            this._backgroundTile.appendTo(this.element);
            this._svgDocument.appendTo(this.element);
            this._svgDocument.css({
                "height": this._height + "px", "width": this._width + "px", "overflow": "hidden",
                "z-index": 1, "position": "absolute"
            });
            $(this._backgroundTile).css({
                "height": this._height + "px", "width": this._width + "px", "overflow": "hidden",
                "z-index": 0, "position": "absolute"
            });
            this._templateDiv.appendTo(this.element);

        },

        /**
         * Section For handling mouse click event for treemap item
         * @private
         */
        _rectClickFunction: function (event) {
            var treeMap = event.data.Param2;
            if (treeMap.model.highlightOnSelection) {
                if (treeMap._prevSelectedItem != null && treeMap._prevSelectedItem != this) {
                    $(treeMap._prevSelectedItem).css({ "border-width": treeMap.model.leafItemSetting.borderThickness, "border-color": treeMap.model.leafItemSetting.borderBrush });
                    $(this).css({ "border-width": treeMap.model.highlightBorderThickness, "border-color": treeMap.model.highlightBorderBrush });
                }
                else if (treeMap._prevSelectedItem == this) {
                    $(this).css({ "border-width": treeMap.model.leafItemSetting.borderThickness, "border-color": treeMap.model.leafItemSetting.borderBrush });
                }
                else {
                    $(this).css({ "border-width": treeMap.model.highlightBorderThickness, "border-color": treeMap.model.highlightBorderBrush });
                }
                treeMap._prevSelectedItem = this;
            }
            treeMap._trigger("treeMapItemSelected", event.data.Param1);
        },
        /**
         * Section For handling mouse enter event for  treemap item
         * @private
         */
        _rectMouseEnter: function (event) {
            var tooltipObject = event.data.Param1;
            if (tooltipObject != null) {
                var element = event.data.Param2._toolTipElement;
                var template = event.data.Param2.model.tooltipTemplate;
                if (element != null && template != null) {
                    $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });
                    var htmlString = $("#" + template).render(tooltipObject);
                    $(element).html(htmlString);

                }
            }
        },
        /**
         * Section For handling mouse move event for  treemap item
         * @private
         */
        _rectMouseMove: function (event) {
            var element = event.data.Param2._toolTipElement;
            if (element != null) {
                $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });
            }
        },

        /**
        * Section For handling mouse leave event for  treemap item
        * @private
        */
        _rectMouseLeave: function (event) {
            var element = event.data.Param1._toolTipElement;
            if (element != null) {
                $(element).css("display", "none");
            }
        },

        /**
        * Sets desaturation color for treemap items
        * @private
        */
        _setDesaturationColor: function (Items, desaturationColorMapping) {
            Items = Items.sort(this._orderBycolorWeight);
            var from = desaturationColorMapping.from;
            var to = desaturationColorMapping.to;
            if (from < 0 || from > 1)
                from = 1;
            if (to > 1 || to < 0)
                to = 0;

            var interval = (from - to) / Items.length;
            for (var i = 0; i < Items.length; i++) {
                $(Items[i].Rectangle).css("background-color", desaturationColorMapping.color);
                if (Items[i].colorWeight >= desaturationColorMapping.rangeMinimum && Items[i].colorWeight <= desaturationColorMapping.rangeMaximum) {
                    Items[i].Rectangle.setAttribute('opacity', from);
                    from = from - interval;
                }
            }
        },

        /**
        * Sets palette color for treemap items
        * @private
        */
        _setPaletteColor: function (Items, PaletteColors) {
            Items = Items.sort(this._orderBycolorWeight);
            for (var i = 0; i < Items.length; i++) {
                if (PaletteColors.colors.length > 0) {
                    Items[i].Rectangle.setAttribute("fill", PaletteColors.Colors[i % (PaletteColors.colors.length)]);
                }
            }
        },

        /**
        * Orders items by color weight
        * @private
        */
        _orderBycolorWeight: function (a, b) {
            if (a.colorWeight == b.colorWeight) {
                return 0;
            } else if (a.colorWeight < b.colorWeight) {
                return 1;
            }
            return -1;
        },
        /**
        * Orders items by area weight
        * @private
        */
        _orderByAreaWight: function (a, b) {
            if (a.AreaByWeight == b.AreaByWeight) {
                return 0;
            } else if (a.AreaByWeight < b.AreaByWeight) {
                return 1;
            }
            return -1;
        },

        /**
       * Calculates slice and dice item size 
       * @private
       */
        _calculateSliceAndDiceItemSize: function (Items, xPos, yPos, width, height, gap, headerHeight, isHorizontal, legendHeight) {

            var Gap = gap;
            var totalWeight = this._getTotalWeight(Items);
            if (isHorizontal == undefined) {
                isHorizontal = width > height ? true : false;
            }
            var AvailableSize = { "Width": width, "Height": height - legendHeight };
            var headerSize = headerHeight < AvailableSize.Height ? headerHeight : 0;
            AvailableSize.Height = AvailableSize.Height - headerSize;
            var AvailableArea = { "X": xPos, "Y": yPos + headerSize, "Width": AvailableSize.Width, "Height": AvailableSize.Height };
            var parentArea = AvailableSize.Height * AvailableSize.Width;
            var itemsCount = Items.length;
            if (isHorizontal) {
                var parentHeight = AvailableSize.Height;
                var allottedWidth = 0;
                for (var i = 0; i < itemsCount; i++) {
                    var childarea = (parentArea / totalWeight) * Items[i].weight;
                    var childWidth = childarea / parentHeight;
                    gap = (childWidth > Gap) ? Gap : 0;

                    if (allottedWidth <= AvailableSize.Width) {
                        Items[i].ItemWidth = (i != itemsCount - 1) ? childWidth - gap : childWidth;
                        Items[i].ItemHeight = parentHeight;
                        Items[i].LeftPosition = allottedWidth + xPos;
                        Items[i].TopPosition = yPos + headerHeight;
                        allottedWidth += childWidth;
                    }
                }
            }
            else {
                var parentWidth = AvailableSize.Width;
                var allottedHeight = 0;
                for (var i = 0; i < itemsCount; i++) {
                    var childarea = (parentArea / totalWeight) * Items[i].weight;
                    var childHeight = childarea / parentWidth;
                    gap = (childHeight > Gap) ? Gap : 0;
                    if (allottedHeight <= AvailableSize.Height) {
                        Items[i].ItemWidth = parentWidth;
                        Items[i].ItemHeight = (i != itemsCount - 1) ? childHeight - gap : childHeight;
                        Items[i].LeftPosition = xPos;
                        Items[i].TopPosition = allottedHeight + yPos + headerSize;
                        allottedHeight += childHeight;
                    }
                }
            }
        },

        /**
       * Calculates squarified item size
       * @private
       */
        _calculateSquarifiedItemSize: function (Items, xPos, yPos, width, height, Gap, headerHeight, legendHeight) {
            var totalweight = this._getTotalWeight(Items);
            var AvailableSize = { "Width": width, "Height": height - legendHeight };
            var headerSize = headerHeight < AvailableSize.Height ? headerHeight : 0;
            AvailableSize.Height = AvailableSize.Height - headerSize;
            for (var i = Items.length - 1; i >= 0; i--) {
                var item = Items[i];
                item["AreaByWeight"] = (AvailableSize.Height * AvailableSize.Width) * item.weight / totalweight;
                item.headerTopPosition = yPos;
            }
            var AvailableArea = { "X": xPos, "Y": yPos + headerSize, "Width": AvailableSize.Width, "Height": AvailableSize.Height };
            var OrderedItems = Items.sort(this._orderByAreaWight);
            var curX = 0, curY = 0;
            var j = 0;
            for (var i = 0; i < Items.length; i = j) {
                var firstTreemapItem = OrderedItems[i];
                weightObject = this._getGroupTotalWeight(firstTreemapItem, OrderedItems, AvailableArea, i);
                var GroupTotalWeight = weightObject.totalWeight;
                j = weightObject.index;
                var currentRect = {};
                var isHorizontal = (AvailableArea.Width > AvailableArea.Height) ? true : false;
                for (var k = i; k < j; k++) {
                    var item = OrderedItems[k];
                    var areaSum = GroupTotalWeight;
                    if (k == i) {
                        currentRect.X = AvailableArea.X;
                        currentRect.Y = AvailableArea.Y;
                        if (isHorizontal) {
                            currentRect.Width = areaSum / AvailableArea.Height;
                            currentRect.Height = AvailableArea.Height;
                            AvailableArea.X += currentRect.Width;
                            AvailableArea.Width = Math.max(0, AvailableArea.Width - currentRect.Width);
                        }
                        else {
                            currentRect.Width = AvailableArea.Width;
                            currentRect.Height = areaSum / AvailableArea.Width;
                            AvailableArea.Y += currentRect.Height;
                            AvailableArea.Height = Math.max(0, AvailableArea.Height - currentRect.Height);
                        }
                        curX = currentRect.X;
                        curY = currentRect.Y;
                    }
                    var rect = {};
                    if (OrderedItems.indexOf(item) != OrderedItems.length - 1) {
                        rect.X = 0;
                        rect.Y = 0;
                        rect.Width = (isHorizontal) ? currentRect.Width - Gap : item.AreaByWeight / currentRect.Height;
                        rect.Height = (isHorizontal) ? item.AreaByWeight / currentRect.Width : currentRect.Height - Gap;
                        if (j - k != 1) {
                            if (isHorizontal) {
                                rect.Height -= Gap;
                            }
                            else {
                                rect.Width -= Gap;
                            }
                        }
                    }
                    else {
                        rect.Width = (isHorizontal) ? currentRect.Width : item.AreaByWeight / currentRect.Height;
                        rect.Height = (isHorizontal) ? item.AreaByWeight / currentRect.Width : currentRect.Height;
                    }
                    item.ItemWidth = rect.Width;
                    item.ItemHeight = rect.Height;
                    item.LeftPosition = curX;
                    item.TopPosition = curY;
                    if (isHorizontal) {
                        curY += (j - k != 1) ? rect.Height + Gap : rect.Height;
                    }
                    else {
                        curX += (j - k != 1) ? rect.Width + Gap : rect.Width;
                    }
                }
            }
        },

        /**
      * Gets group total weight
      * @private
      */
        _getGroupTotalWeight: function (firstTreemapItem, OrderedItems, AvailableArea, j) {
            var GroupTotalWeight = 0;
            var GroupMaxAspectRatio = 0;
            for (; j < OrderedItems.length; j++) {
                var ShorterSideLength = this._getShortersideLength(AvailableArea.Width, AvailableArea.Height);
                var lastTreemapItem = OrderedItems[j];
                GroupTotalWeight += lastTreemapItem.AreaByWeight;
                var GroupWidth = GroupTotalWeight / ShorterSideLength;
                var firstitemheight = firstTreemapItem.AreaByWeight / GroupWidth;
                var lastitemheight = lastTreemapItem.AreaByWeight / GroupWidth;
                if (j == 0)
                    GroupMaxAspectRatio = this._aspectRatio(GroupWidth, ShorterSideLength);
                var TempAspectRatio = Math.max(this._aspectRatio(firstitemheight, GroupWidth), this._aspectRatio(lastitemheight, GroupWidth));
                if (GroupTotalWeight == lastTreemapItem.AreaByWeight || TempAspectRatio < GroupMaxAspectRatio) {
                    GroupMaxAspectRatio = TempAspectRatio;
                }
                else {
                    GroupTotalWeight -= lastTreemapItem.AreaByWeight;
                    GroupWidth = GroupTotalWeight / ShorterSideLength;
                    GroupMaxAspectRatio = Math.max(this._aspectRatio(firstitemheight, GroupWidth), this._aspectRatio(lastitemheight, GroupWidth));
                    break;
                }
            }
            return { totalWeight: GroupTotalWeight, index: j };
        },

        /**
      * Calculates aspect ratio
      * @private
      */
        _aspectRatio: function (x, y) {
            return (x > y) ? (x / y) : (y / x);
        },

        /**
     * Calculates shorter side length
     * @private
     */
        _getShortersideLength: function (width, height) {
            return width > height ? height : width;
        },

        /**
     * Calculates total weight
     * @private
     */

        _getTotalWeight: function (Items) {
            var total = 0;
            for (var i = 0; i < Items.length; i++) {
                total += Items[i].weight;
            }
            return total;

        },

        /**
     * Gets group item
     * @private
     */
        _getGroupitem: function (path, dataSource, headerHeight) {
            var obj = [];
            if (dataSource != null && this.model.weightValuePath != null) {
                for (var i in dataSource) {
                    var item = dataSource[i];
                    if (item.hasOwnProperty(path)) {
                        var itemValue = this._reflection(item, path);
                        if ($.inArray(itemValue, obj) == -1) {
                            obj.push(itemValue);
                        }
                    }
                }
                var objCollection = [];
                for (var i = 0; i < obj.length; i++) {
                    var Coll = {};
                    var groupObj = [];
                    var groupId = obj[i];
                    var colorWeight = 0;
                    var weight = 0;
                    var count = 0;
                    for (var j = 0; j < dataSource.length; j++) {
                        var item = dataSource[j];
                        var itemValue = this._reflection(item, path);
                        if (itemValue == groupId) {
                            var weightValue = this._reflection(item, this.model.weightValuePath);
                            var colorWeightValue = this._reflection(item, this.model.colorValuePath);
                            groupObj.push(item);
                            count++;
                            if (weightValue != null) {
                                weight += weightValue;
                            }
                            if (colorWeightValue != null) {
                                colorWeight += colorWeightValue;
                            }
                        }
                    }

                    Coll["header"] = groupId;
                    Coll["data"] = groupObj;
                    Coll["weight"] = weight;
                    Coll["colorWeight"] = colorWeight;
                    Coll["headerHeight"] = headerHeight;
                    if (weight > 0) {
                        var newTreemapItem = new treeMapItem(Coll);
                        objCollection.push(newTreemapItem);
                    }
                }
                return objCollection;
            }
            else { return null; }
        },

        /**
        * Gets top group item
        * @private
        */
        _getTopGroupitem: function (path, dataSource) {
            if (path == null) path = this.model.leafItemSetting.labelPath;
            if (path == null) path = this.model.weightValuePath;
            var obj = [];
            var Coll = {};
            var weight = 0, colorWeight = 0;
            var groupId = "";
            var innerobjCollection = [];
            var itemColl = [];
            for (var j = 0; j < dataSource.length; j++) {
                var item = dataSource[j];
                var itemValue = this._reflection(item, path);
                var innerColl = {};
                var weightValue = this._reflection(item, this.model.weightValuePath);
                var colorWeightValue = this._reflection(item, this.model.colorValuePath);
                if (weightValue != null) {
                    weight += weightValue;
                }
                if (colorWeightValue != null) {
                    colorWeight += colorWeightValue;
                }
                itemColl.push(item);
                innerColl["weight"] = weightValue;
                innerColl["data"] = item;
                innerColl["header"] = itemValue;
                innerColl["colorWeight"] = colorWeightValue;
                var newinnerTreemapItem = new treeMapItem(innerColl);
                innerobjCollection.push(newinnerTreemapItem);
            }
            Coll["header"] = groupId;
            Coll["data"] = itemColl;
            Coll["weight"] = weight;
            Coll["innerItem"] = innerobjCollection;
            Coll["colorWeight"] = colorWeight;
            var treemapColl = [];
            var treeitem = new treeMapItem(Coll);
            treemapColl.push(treeitem);
            return treemapColl;
        },



        /**
        * Gets property from object
        * @private
        */
        _reflection: function (Source, Path) {
            var ShapeValues = Source;
            if (Path != null && Source != null) {
                var parts = Path.split(".");
                var i = 0;
                for (; i < parts.length; i++) {
                    var hasProperty = ShapeValues.hasOwnProperty(parts[i]);
                    if (hasProperty) {
                        $.each(ShapeValues, function (prop, propval) {
                            if (prop == parts[i]) {
                                ShapeValues = propval;
                                return false;
                            }
                        });
                    }
                }
                return ShapeValues;
            }
            else {
                return null;
            }

        }

    });
    /**
    * // class for tree map levels
    *  @memberof ejTreeMap
    * @member
    * @instance
    */
    var treeMapLevel = function () {

        /** 
        * specifies the group background
			 * @default 'white'
             * @type {string} 
             * @example  
             * // Set the groupBackground during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupBackground:"white" }]})
             * @example 
             * //Get or set the groupBackground after initialization:<br>
             *   //Gets the groupBackground from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupBackground;
             *   //Sets the groupBackground to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupBackground  = "white";
             * @alias ejTreeMap#treeMapLevel->groupBackground
             */
        this.groupBackground = 'white';

        /** 
        * Specifies the group border color for tree map level.
         * @default '#58585B'
         * @type {string} 
         * @example  
         * // Set the groupBorderColor during initialization. 			
         * 	$("#container").ejTreeMap({levels: [{ groupBorderColor:"#58585B" }]})
         * @example 
         * //Get or set the groupBorderColor after initialization:<br>
         *   //Gets the groupBorderColor from map.
         *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupBorderColor;
         *   //Sets the groupBorderColor to map.
         *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupBorderColor  = "#58585B";
         * @alias ejTreeMap#treeMapLevel->groupBorderColor
         */
        this.groupBorderColor = '#58585B';

        /** 
        * Specifies the group border thickness for tree map level.
			 * @default 1
             * @type {number} 
             * @example  
             * // Set the groupBorderThickness during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupBorderThickness:1 }]})
             * @example 
             * //Get or set the groupBorderThickness after initialization:<br>
             *   //Gets the groupBorderThickness from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupBorderThickness;
             *   //Sets the groupBorderThickness to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupBorderThickness  = 1;
             * @alias ejTreeMap#treeMapLevel->groupBorderThickness
             */
        this.groupBorderThickness = 1;

        /** 
                * Specifies the group padding for tree map level.
                     * @default 4
                     * @type {number} 
                     * @example  
                     * // Set the groupPadding during initialization. 			
                     * 	$("#container").ejTreeMap({levels: [{ groupPadding:4 }]})
                     * @example 
                     * //Get or set the groupPadding after initialization:<br>
                     *   //Gets the groupPadding from map.
                     *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupPadding;
                     *   //Sets the groupPadding to map.
                     *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupPadding  = 4;
                     * @alias ejTreeMap#treeMapLevel->groupPadding
                     */
        this.groupPadding = 4;

        /** 
        * Specifies the group path for tree map level.
             * @type {string} 
             * @example  
             * // Set the groupPath during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupPath:"pathName" }]})
             * @example 
             * //Get or set the groupPath after initialization:<br>
             *   //Gets the groupPath from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupPath;
             *   //Sets the groupPath to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupPath  = "pathName";
             * @alias ejTreeMap#treeMapLevel->groupPath
             */
        this.groupPath = null;

        /** 
        * Shows or hides the tree map items
             * @default false
             * @type {boolean} 
             * @example  
             * // Set the showItem during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ showItem:false }]})
             * @example 
             * //Get or set the showItem after initialization:<br>
             *   //Gets the showItem from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].showItem;
             *   //Sets the showItem to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].showItem  = false;
             * @alias ejTreeMap#treeMapLevel->showItem
             */
        this.showItem = false;

        /**
        * Specifies the group gap for tree map level.
			 * @default 1
             * @type {number} 
             * @example  
             * // Set the groupGap during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupGap:1 }]})
             * @example 
             * //Get or set the groupGap after initialization:<br>
             *   //Gets the groupGap from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupGap;
             *   //Sets the groupGap to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupGap  = 1;
             * @alias ejTreeMap#treeMapLevel->groupGap
             */
        this.groupGap = 1;

        /**
        * Specifies the header height for tree map level.
			 * @default 0
             * @type {number} 
             * @example  
             * // Set the headerHeight during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ headerHeight:20 }]})
             * @example 
             * //Get or set the headerHeight after initialization:<br>
             *   //Gets the headerHeight from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].headerHeight;
             *   //Sets the headerHeight to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].headerHeight  = 1;
             * @alias ejTreeMap#treeMapLevel->headerHeight
             */
        this.headerHeight = 20;

        /** 
        * Shows or hides the header for tree map level.
            * @default false
            * @type {bool} 
            * @example  
            * // Set the shoeHeader during initialization. 			
            * 	$("#container").ejTreeMap({levels: [{ showHeader:false }]})
            * @example 
            * //Get or set the showHeader after initialization:<br>
            *   //Gets the showHeader from treemap.
            *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].showHeader;
            *   //Sets the headerHeight to map.
            *   $("#container").data("ejTreeMap").model.levels[levelIndex].showHeader  = true;
            * @alias ejTreeMap#treeMapLevel->showHeader
            */
        this.showHeader = true;

        /**
        * Shows or hides the labels for tree map level.
             * @default false
             * @type {boolean} 
             * @example  
             * // Set the showLabels during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ showLabels:false }]})
             * @example 
             * //Get or set the showLabels after initialization:<br>
             *   //Gets the showLabels from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].showLabels;
             *   //Sets the showLabels to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].showLabels  = false;
             * @alias ejTreeMap#treeMapLevel->showLabels
             */
        this.showLabels = false;

        /** 
        * Specifies the header template for tree map level.
         * @default null
             * @type {string} 
             * @example  
             * // Set the headerTemplate during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ headerTemplate:"template" }]})
             * @example 
             * //Get or set the headerTemplate after initialization:<br>
             *   //Gets the headerTemplate from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].headerTemplate;
             *   //Sets the headerTemplate to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].headerTemplate  = "template";
             * @alias ejTreeMap#treeMapLevel->headerTemplate
             */
        this.headerTemplate = null;
        /**
        * Specifies the label template for tree map level.
        * @default null
             * @type {string} 
             * @example  
             * // Set the labelTemplate during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ labelTemplate:"template" }]})
             * @example 
             * //Get or set the labelTemplate after initialization:<br>
             *   //Gets the labelTemplate from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].labelTemplate;
             *   //Sets the labelTemplate to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].labelTemplate  = "template";
             * @alias ejTreeMap#treeMapLevel->labelTemplate
             */
        this.labelTemplate = null;
        this.treeMapItems = [];
    };

    treeMapLevel.prototype = {

    };

    var treeMapItem = function (params) {
        this.weight = params.weight;
        this.colorWeight = params.colorWeight;
        this.Data = params.data;
        this.headerHeight = params.headerHeight;
        this.header = params.header;
        this.label = params.header;
        this.headerLeftPosition = 0;
        this.headerTopPosition = 0;
        this.innerItems = params.innerItem;
        this.headerWidth = 0;
        this.headerTemplate = null;

    };

    treeMapItem.prototype = {
        /**
        * Generates color mapping for treemap items
        * @private
        */
        _generateColorMappings: function (colorMappings) {
            if (colorMappings.rangeColorMapping.length > 0) {
                return this._getRangeBrushColor(colorMappings.rangeColorMapping);
            }
            if (colorMappings.uniColorMapping.color != null) {
                return this._getUniColor(colorMappings.uniColorMapping);
            }
        },

        /**
        * Get unicolor for treemap items
        * @private
        */
        _getUniColor: function (uniColorMapping) {

            return uniColorMapping.color;
        },

        /**
        * Get range brush color for treemap items
        * @private
        */
        _getRangeBrushColor: function (rangeColorMapping) {
            for (var j = 0; j < rangeColorMapping.length; j++) {
                var rangeBrush = rangeColorMapping[j];
                if (this.colorWeight >= rangeBrush.from && this.colorWeight <= rangeBrush.to) {
                    return rangeBrush.color;
                }
            }
        }

    };

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Barcode elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
* Use of this code is subject to the terms of our license.
* A copy of the current license can be obtained at any time by e-mailing
* licensing@syncfusion.com. Any infringement will be prosecuted under
* applicable laws.
* @version 12.1
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /** ejBarcode is the plugin name.*/
    /** "ej.Barcode" is "namespace.className" will hold functions and properties.*/
    /**
     * @namespace ej
     * @class ejBarcode
     * @requires jQuery
     * @requires ej.common.all
     * @classdesc Custom Design for Barcode control.
     * @example 
     * &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
     * &lt;script&gt;
     * // Create Barcode
     * $('#barcode1').ejBarcode();         
     * &lt;/script&gt;
     */

    ej.widget("ejBarcode", "ej.Barcode", {
        /** Widget element will be automatically set in this*/
        element: {
            target: null
        },

        /** User defined model will be automatically set in this*/
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-barcode",

        defaults: {
            /**		
			* Specifies whether to display original text. Applicable only for One dimensional barcode.
			* @default true
			* @type {boolean}
            * @memberof ejBarcode
			* @example 
			* //To set displayText API value during initialization  
			* 	$("#barcode").ejBarcode({ displayText: true});
			* @example 
			* //Get or set the displayText API, after initialization:<br>
			*	//Gets the displayText value  
			*	$("#barcode1").ejBarcode("option", "displayText");<br>
			*	//Sets the displayText value 
			*	$("#barcode1").ejBarcode("option", "displayText", true ); 
			* @instance
			*/
            displayText: true,
            /**		
			* Specifies the text to encode.
			* @default empty
			* @type {string}
            * @memberof ejBarcode
			* @example 
			* //To set text value during initialization  
			* 	$("#barcode1").ejBarcode({ text:"SYNCFUSION" });
			* @example 
			* //Get or set the text API, after initialization:<br>
			*	//Gets the text value  
			*	$("#barcode1").ejBarcode("option", "text");<br>
			*	//Sets the text value 
			*	$("#barcode1").ejBarcode("option", "text", "SYNCFUSION" ); 
			* @instance
			*/
            text: "",
            /**                
             * Specifies the symbology type of the Barcode.        See {@link symbologyType}
             * @default ej.barcode.symbologyType.qrbarcode
             * @type {enum}
             * @memberof ejBarcode
             * @example 
             * //To set symbology type API value during initialization  
             *         $("#barcode1").ejBarcode({ symbologyType: "code39" });
             * @example 
             * //Get or set the symbology type API, after initialization:<br>
             *        //Gets the symbology type value  
             *        $("#barcode1").ejBarcode("option", "symbologyType");<br>          
             *        //Sets the symbology type value 
             *        $("#barcode1").ejBarcode("option", "symbologyType", "code39" ); 
             * @instance
             */
            symbologyType: "qrbarcode",
            /**                
             * Specifies the display text color of the Barcode. Applicable only for One dimensional barcode.
             * @default black
             * @type {object}
             * @memberof ejBarcode
             * @example 
             * //To set text color API value during initialization  
             *         $("#barcode1").ejBarcode({ textColor: "black" });
             * @example 
             * //Get or set the text color API, after initialization:<br>
             *        //Gets the text color value  
             *        $("#barcode1").ejBarcode("option", "textColor");<br>
             *        //Sets the text color value 
             *        $("#barcode1").ejBarcode("option", "textColor", "black" ); 
             * @instance
             */
            textColor: 'black',
            /**                
             * Specifies the light bar color of the Barcode. Applicable only for One dimensional barcode.
             * @default white
             * @type {object}
             * @memberof ejBarcode
             * @example 
             * //To set light bar color API value during initialization  
             *         $("#barcode1").ejBarcode({ lightBarColor: "white" });
             * @example 
             * //Get or set the light bar color API, after initialization:<br>
             *        //Gets the light bar color value  
             *        $("#barcode1").ejBarcode("option", "lightBarColor");<br>
             *        //Sets the light bar color value 
             *        $("#barcode1").ejBarcode("option", "lightBarColor", "white" ); 
             * @instance
             */
            lightBarColor: 'white',
            /**                
             * Specifies the dark bar color of the Barcode. Applicable only for One dimensional barcode.
             * @default black
             * @type {object}
             * @memberof ejBarcode
             * @example 
             * //To set dark bar color API value during initialization  
             *         $("#barcode1").ejBarcode({ darkBarColor: "black" });
             * @example 
             * //Get or set the dark bar color API, after initialization:<br>
             *        //Gets the dark bar color value  
             *        $("#barcode1").ejBarcode("option", "darkBarColor");<br>
             *        //Sets the dark bar color value 
             *        $("#barcode1").ejBarcode("option", "darkBarColor", "black" ); 
             * @instance
             */
            darkBarColor: 'black',
            /**                
             * The blank margin on the side(s) which denotes the reader with the start and stop of the Barcode.
             * @type {object}
             * @memberof ejBarcode
             * @instance
             */
            quietZone: {
                /**                
                 * Specifies the left quiet zone of the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 *         $("#barcode1").ejBarcode({ quietZone: { left: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 *        //Gets the quiet zone value  
                 *        $("#barcode1").ejBarcode("option", "quietZone.left");<br>                        
                 *        //Sets the quiet zone value 
                 *        $("#barcode1").ejBarcode("option", "quietZone", {left: 5} ); 
                 * @alias ejBarcode#quietZone->left
                 * @instance
                 */
                left: 1,
                /**                
                 * Specifies the top quiet zone of the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 *         $("#barcode1").ejBarcode({ quietZone: { top: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 *        //Gets the quiet zone value  
                 *        $("#barcode1").ejBarcode("option", "quietZone.top");<br>                   
                 *        //Sets the quiet zone value 
                 *        $("#barcode1").ejBarcode("option", "quietZone", {top: 5} ); 
                 * @alias ejBarcode#quietZone->top
                 * @instance
                 */
                top: 1,
                /**                
                 * Specifies the right quiet zone of the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 *         $("#barcode1").ejBarcode({ quietZone: { right: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 *        //Gets the quiet zone value  
                 *        $("#barcode1").ejBarcode("option", "quietZone.right");<br>                      
                 *        //Sets the quiet zone value 
                 *        $("#barcode1").ejBarcode("option", "quietZone", {right: 5} ); 
                 * @alias ejBarcode#quietZone->right
                 * @instance
                 */
                right: 1,
                /**                
                 * Specifies the bottom quiet zone of the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 *         $("#barcode1").ejBarcode({ quietZone: { bottom: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 *        //Gets the quiet zone value  
                 *        $("#barcode1").ejBarcode("option", "quietZone.bottom");<br>                     
                 *        //Sets the quiet zone value 
                 *        $("#barcode1").ejBarcode("option", "quietZone", {bottom: 5} ); 
                 * @alias ejBarcode#quietZone->bottom
                 * @instance
                 */
                bottom: 1,
                /**                
                 * Specifies the quiet zone around the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 *         $("#barcode1").ejBarcode({ quietZone: { all: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 *        //Gets the quiet zone value  
                 *        $("#barcode1").ejBarcode("option", "quietZone.all");<br>                 
                 *        //Sets the quiet zone value 
                 *        $("#barcode1").ejBarcode("option", "quietZone", {all: 5} ); 
                 * @alias ejBarcode#quietZone->all
                 * @instance
                 */
                all: 1
            },
            /**                
             * Specifies the width of narrow bar in the Barcode. Applicable only for One dimensional barcode.
             * @default 1
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set narrowBarWidth API value during initialization  
             *         $("#barcode1").ejBarcode({ narrowBarWidth: 1 });
             * @example 
             * //Get or set the narrowBarWidth API, after initialization:<br>
             *        //Gets the width of the narrow bar value  
             *        $("#barcode1").ejBarcode("option", "narrowBarWidth");<br>
             *        //Sets the width of the narrow bar value 
             *        $("#barcode1").ejBarcode("option", "narrowBarWidth", 1 ); 
             * @instance
             */
            narrowBarWidth: 1,
            /**                
             * Specifies the width of wide bar in the Barcode. Applicable only for One dimensional barcode.
             * @default 3
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set wideBarWidth API value during initialization  
             *         $("#barcode1").ejBarcode({ wideBarWidth: 1 });
             * @example 
             * //Get or set the wideBarWidth API, after initialization:<br>
             *        //Gets the width of the wide bar value  
             *        $("#barcode1").ejBarcode("option", "wideBarWidth");<br>
             *        //Sets the width of the wide bar value 
             *        $("#barcode1").ejBarcode("option", "wideBarWidth", 3 ); 
             * @instance
             */
            wideBarWidth: 3,
            /**                
             * Specifies the height of bars in the Barcode. Applicable only for One dimensional barcode.
             * @default 150
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set barHeight API value during initialization  
             *         $("#barcode1").ejBarcode({ barHeight: 150 });
             * @example 
             * //Get or set the barHeight API, after initialization:<br>
             *        //Gets the height of the bars  
             *        $("#barcode1").ejBarcode("option", "barHeight");<br>
             *        //Sets the height of the bars 
             *        $("#barcode1").ejBarcode("option", "barHeight", 150 ); 
             * @instance
             */
            barHeight: 150,
            /**                
             * Specifies the gap distance between the bar and the text in the Barcode. Applicable only for One dimensional barcode.
             * @default 10
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set barcodeToTextGapHeight API value during initialization  
             *         $("#barcode1").ejBarcode({ barcodeToTextGapHeight: 10 });
             * @example 
             * //Get or set the barcodeToTextGapHeight API, after initialization:<br>
             *        //Gets the gap distance between the bar and the text
             *        $("#barcode1").ejBarcode("option", "barcodeToTextGapHeight");<br>
             *        //Sets the gap distance between the bar and the text
             *        $("#barcode1").ejBarcode("option", "barcodeToTextGapHeight", 10 ); 
             * @instance
             */
            barcodeToTextGapHeight: 10,
            /**                
             * Specifies the size of each block in the Barcode. Applicable only for Two dimensional barcode.
             * @default 4
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set xDimension API value during initialization  
             *         $("#barcode1").ejBarcode({ xDimension: 4 });
             * @example 
             * //Get or set the xDimension API, after initialization:<br>
             *        //Gets the size of each block
             *        $("#barcode1").ejBarcode("option", "xDimension");<br>
             *        //Sets the size of each block
             *        $("#barcode1").ejBarcode("option", "xDimension", 4 ); 
             * @instance
             */
            xDimension: 4,
            /**                
             * Specifies whether to encode start and stop symbol in the Barcode. Applicable only for One dimensional barcode.
             * @default 10
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set encodeStartStopSymbol API value during initialization  
             *         $("#barcode1").ejBarcode({ encodeStartStopSymbol: true });
             * @example 
             * //Get or set the encodeStartStopSymbol API, after initialization:<br>
             *        //Gets the encodeStartStopSymbol value
             *        $("#barcode1").ejBarcode("option", "encodeStartStopSymbol");<br>
             *        //Sets the whether to encode start and stop symbol value
             *        $("#barcode1").ejBarcode("option", "encodeStartStopSymbol", true ); 
             * @instance
             */
            encodeStartStopSymbol: true,
            /**     
             * Fires after Barcode control is loaded.
             * @event
             * @memberof ejBarcode
             * @param {Object} argument Event parameters from barcode
             * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {object}  argument.model returns the barcode model
             * @param {string}  argument.type returns the name of the event
             * @param {boolean}  argument.status return the barcode state
             * @example 
             * //create event for barcode
                         * $("#barcode1").ejBarcode({
                         *    load: function (args) {}
                         * });
             * @example 
             * //Bind create event using jquery "on"
                         * $("#barcode1").on("ejBarcodeload", function(e) {} );
             * @instance
             */
            load: null,
            /**		
            * Specifies whether the control is enabled.
            * @default true
            * @type {boolean}
            * @memberof ejBarcode
            * @example 
            * //To set enabled API value during initialization  
            * 	$("#barcode").ejBarcode({ enabled: true});
            * @example 
            * //Get or set the enabled API, after initialization:<br>
            *	//Gets the enabled value  
            *	$("#barcode1").ejBarcode("option", "enabled");<br>
            *	//Sets the enabled value 
            *	$("#barcode1").ejBarcode("option", "enabled", true ); 
            * @instance
            */
            enabled: true,
        },

        /**
         * Specify the data types for default properties 
         * @private
         */
        dataTypes: {
            displayText: "boolean",
            text: "string",
            enabled: "boolean",
            symbologyType: "enum",
            narrowBarWidth: "number",
            wideBarWidth: "number",
            barHeight: "number",
            encodeStartStopSymbol: "boolean"
        },

        /**
         * @private
         */
        _defaultScaleValues: function () {
            return {
                enabled: true,
                displayText: true,
                text: "",
                symbologyType: "qrbarcode",
                textColor: 'black',
                lightBarColor: 'white',
                darkBarColor: 'black',
                quietZone: { left: 1, top: 1, right: 1, bottom: 1 },
                narrowBarWidth: 1,
                wideBarWidth: 3,
                barHeight: 150,
                barcodeToTextGapHeight: 10,
                xDimension: 4,
                encodeStartStopSymbol: true
            };
        },

        /**
         * @constructor
         * @private
         */
        _init: function () {
            this._initialize();
            this._renderBarcode();
        },

        /**
         * @private
         */
        _destroy: function () {
            this.element.empty();
        },

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "enabled": this._disabled(!options[option]); break;
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "lightBarColor": this.model.lightBarColor = options[option]; break;
                    case "darkBarColor": this.model.darkBarColor = options[option]; break;
                    case "textColor": this.model.textColor = options[option]; break;
                    case "displayText": this.model.displayText = options[option]; break;
                    case "quietZone": this.model.quietZone = options[option]; break;
                    case "narrowBarWidth": this.model.narrowBarWidth = options[option]; break;
                    case "wideBarWidth": this.model.wideBarWidth = options[option]; break;
                    case "barHeight": this.model.barHeight = options[option]; break;
                    case "xDimension": this.model.xDimension = options[option]; break;
                    case "encodeStartStopSymbol": this.model.encodeStartStopSymbol = options[option]; break;
                    case "symbologyType":
                        this.model.symbologyType = options[option];
                        this._itemInitialize();
                        break;
                    case "text":
                        this.model.text = options[option];
                        this._itemInitialize();
                        break;
                }
            }
            this._renderBarcode();
        },

        /**
         * Initializes control.
         * @private
         */
        _initialize: function () {
            this.BarcodeEl = this.element;
            this.target = this.element[0];
            this._itemInitialize();
            this.Model = this.model;
        },

        /**
         * Renders barcode.
         * @private
         */
        _renderBarcode: function () {
            this.initialize();
        },

        /**
         * @private
         */
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },

        /**
         * @private
         */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    innerElement = proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    innerElement = proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

        /**
         * @private
         */
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, Ele) {
                if (Ele instanceof Array) {
                    Ele = proxy._checkArrayObject(Ele, name);
                }
                else if (Ele != null && typeof Ele == "object") {
                    Ele = proxy._LoadIndividualDefaultValues(Ele, defaultObj, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },

        /**
         * Initializes the control.
         * @private
         */
        initialize: function () {
            if (!this.model.enabled)
                return;

            this._initObject(this);

            if (this.Model.text != null)
                this._findBarcodeType();
        },

        /**
        * To enable the barcode 		
		* @return jQuery
		* @example 
		* &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
		* &lt;script&gt;
		* // Create Barcode
		* var btnObj = $("#barcode1").data("ejBarcode");
		* btnObj.enable(); // enable the barcode
		* &lt;/script&gt;
		* @example 
		* &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
		* &lt;script&gt;
		* // enable the barcode
		* $("#barcode1").ejBarcode("enable");	
		* &lt;/script&gt;
		* @memberof ejBarcode
		* @instance
         */
        enable: function () {
            if (!this.model.enabled) {
                this.element.removeClass("e-disable");
                this.model.enabled = true;
            }
        },

        /**
        * To disable the barcode  
		* @return jQuery
		* @example 
		* &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
		* &lt;script&gt;
		* // Create Barcode
		* var btnObj = $("#barcode1").data("ejBarcode");
		* btnObj.disable(); // disable the barcode
		* &lt;/script&gt;
		* @example 
		* &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
		* &lt;script&gt;
		* // disable the barcode
		* $("#barcode1").ejBarcode("disable");	
		* &lt;/script&gt;
		* @memberof ejBarcode
		* @instance
        */
        disable: function () {
            if (this.model.enabled) {
                this.element.addClass("e-disable");
                this.model.enabled = false;
            }
        },

        /**
         * @private
         */
        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        /**
         * Initializes rendering canvas.
         * @private
         */
        _initObject: function (element) {
            element.BarcodeEl = $("#" + element.target.id);
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.BarcodeEl.append(element.canvasEl);
            element.BarcodeEl.css("width", element.model.width);
            element.BarcodeEl.css("height", element.model.height);
            element.canvasEl[0].setAttribute("width", element.BarcodeEl.width());
            element.canvasEl[0].setAttribute("height", element.BarcodeEl.height());
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");

            if (element.contextEl == undefined) //IE 8
                return;
        },

        /**
         * Finds the barcode type and encodes text.
         * @private
         */
        _findBarcodeType: function () {
            if (this.contextEl == undefined) //IE 8
                return;

            if (this.Model.text == "")
                return;

            this.textFont = { size: "12px", fontFamily: "Segoe UI", fontStyle: "Regular" };

            if (this.model.quietZone.all > 1)
                this.model.quietZone.left = this.model.quietZone.right = this.model.quietZone.top = this.model.quietZone.bottom = this.model.quietZone.all;

            intercharacterGap = 1;
            extendedText = "";
            continuous = false;
            validatorExp = "";
            encodedText = this.Model.text;
            var startSymbol, stopSymbol, symbolTable, extendedCodes;

            var type = this.Model.symbologyType;
            var oneD = true;

            /** Apply symbology setting.*/
            if (ej.Barcode.symbologyType.dataMatrix == type || ej.Barcode.symbologyType.qrBarcode == type) {
                oneD = false;
            }
            else {
                startSymbol = ej.Barcode.symbologySettings[type].startSymbol;
                stopSymbol = ej.Barcode.symbologySettings[type].stopSymbol;
                validatorExp = ej.Barcode.symbologySettings[type].validatorExp;
                symbolTable = ej.Barcode.symbologySettings[type].symbolTable;
                extendedCodes = ej.Barcode.symbologySettings[type].extendedCodes;

                if (ej.Barcode.symbologyType.code39Extended == type) {
                    var cArray = encodedText.split("");
                    var builder = "";
                    for (var j = 0; j < cArray.length; j++) {
                        var item = cArray[j];
                        for (var k = 0; k < extendedCodes.length; k++) {
                            if (extendedCodes[k][0] == item) {
                                var chArray = extendedCodes[k][1];
                                if (!(chArray == undefined)) {
                                    for (var i = 0; i < chArray.length; i++) {
                                        builder += chArray[i];
                                    }
                                }
                            }
                        }
                    }
                    extendedText = builder;
                }
                else if (ej.Barcode.symbologyType.code93Extended == type) {
                    var cArray = encodedText.split("");
                    var builder = "";
                    for (var j = 0; j < cArray.length; j++) {
                        var item = cArray[j];
                        for (var k = 0; k < extendedCodes.length; k++) {
                            if (extendedCodes[k][0] == item) {
                                var chArray = extendedCodes[k][1];
                                if (!(chArray == undefined)) {
                                    for (var i = 0; i < chArray.length; i++) {
                                        builder += chArray[i];
                                    }
                                }
                            }
                        }
                    }
                    extendedText = builder;
                }
            }

            /** Process one dimensional barcode.*/
            if (oneD) {
                if (!this._validateText(this.Model.text, validatorExp)) {
                    console.log("Barcode Text contains characters that are not accepted by this barcode specification.");
                    return;
                }

                if (this.model.symbologyType == "code32" && this.model.text.length != 8) {
                    console.log("Barcode Text Length that are not accepted by this barcode specification.");
                    return;
                }

                if (extendedText.length > 0)
                    encodedText = extendedText;

                if (this.Model.encodeStartStopSymbol)
                    encodedText = startSymbol.toString() + encodedText + stopSymbol.toString();

                var w = this._getCharWidth(encodedText, symbolTable);

                if (continuous == false)
                    w -= (intercharacterGap * encodedText.length);
                else {
                    w -= (extendedText.length > 0) ?
                        intercharacterGap * (extendedText.length - this.Model.text.length) :
                        intercharacterGap;
                }

                var textHeight = parseInt(this.Model.displayText ? this.textFont.size.replace('px', '') : 0);
                var h = this.Model.quietZone.top + this.Model.quietZone.bottom + this.Model.barHeight + intercharacterGap + (this.Model.displayText ? (textHeight + this.Model.barcodeToTextGapHeight) : 0);
                w += this.Model.quietZone.left + this.Model.quietZone.right;

                /** Prepare the canvas.*/
                this.Model.width = w;
                this.canvasEl[0].setAttribute("width", this.Model.width);
                this.BarcodeEl.css("width", this.Model.width);

                this.contextEl.beginPath();
                this.contextEl.fillStyle = this.Model.lightBarColor;
                this.contextEl.fillRect(0, 0, w, h);
                this.contextEl.closePath();

                /** Draw one dimensional barcode.*/
                this._draw1DBarcode(encodedText, symbolTable);
            }
                /** Process two dimensional barcode.*/
            else {
                if (ej.Barcode.symbologyType.dataMatrix == type)
                    this._buildDataMatrix();
                else if (ej.Barcode.symbologyType.qrBarcode == type)
                    this._buildQRBarcode();
            }

            /** Raise load event*/
            this._raiseEvent("load");
        },

        /**
         * Load event.
         * @private
         * @event
         */
        _raiseEvent: function (event) {
            if (this.model[event])
                this._trigger(event);
        },

        /**
         * Generates QR barcode.
         * @private
         */
        _buildQRBarcode: function () {
            var txt = this.Model.text;
            this.m_EciAssignmentNumber = 3;
            this.mode = "numeric";
            this.isEci = false;
            this.m_Version = 1;
            var dataCapacityOfVersions = new Array(14, 26, 42, 62, 84, 106, 122, 152, 180, 213, 251, 287, 331, 362, 412, 450, 504, 560, 624, 666, 711, 779, 857, 911, 997, 1059, 1125, 1190, 1264, 1370, 1452, 1538, 1628, 1722, 1809, 1911, 1989, 2099, 2213, 2331);
            for (var i = 0; i < txt.length; i++) {
                var cCode = txt.charCodeAt(i);
                if (cCode < 58 && cCode > 47) {
                    /** numeric only*/
                }
                else if ((cCode < 91 && cCode > 64) || txt[i] == '$' || txt[i] == '%' || txt[i] == '*' || txt[i] == '+' || txt[i] == '-' || txt[i] == '.' || txt[i] == '/' || txt[i] == ':' || txt[i] == ' ') {
                    this.mode = "alphanumeric";
                }
                else if (cCode >= 65377 && cCode <= 65439) {
                    this.mode = "binary";
                    break;
                }
                else {
                    this.mode = "binary";
                    this.isEci = true;
                    break;
                }
            }
            if (this.mode == "numeric")
                dataCapacityOfVersions = new Array(34, 63, 101, 149, 202, 255, 293, 365, 432, 513, 604, 691, 796, 871, 991, 1082, 1212, 1346, 1500, 1600, 1708, 1872, 2059, 2188, 2395, 2544, 2701, 2857, 3035, 3289, 3486, 3693, 3909, 4134, 4343, 4588, 4775, 5039, 5313, 5596);
            else if (this.mode == "alphanumeric")
                dataCapacityOfVersions = new Array(20, 38, 61, 90, 122, 154, 178, 221, 262, 311, 366, 419, 483, 528, 600, 656, 734, 816, 909, 970, 1035, 1134, 1248, 1326, 1451, 1542, 1637, 1732, 1839, 1994, 2113, 2238, 2369, 2506, 2632, 2780, 2894, 3054, 3220, 3391);

            if (this.isEci == true) {
                this.m_EciAssignmentNumber = this._findECINumber(txt);
            }
            for (var i = 0; i < dataCapacityOfVersions.length; i++) {
                if (dataCapacityOfVersions[i] > txt.length) {
                    this.m_Version = i + 1;
                    break;
                }
            }

            this.qrbarcodeValues = { Version: this.m_Version, ErrorCorrectionLevel: 1, NumberOfDataCodeWord: this._getNumberOfDataCodeWord(), NumberOfErrorCorrectingCodeWords: this._getNumberOfErrorCorrectingCodeWords(), NumberOfErrorCorrectionBlocks: this._getNumberOfErrorCorrectionBlocks(), End: this._getEnd(), DataCapacity: this._getDataCapacity(), FormatInformation: this._getFormatInformation(), VersionInformation: this._getVersionInformation() };
            this.noOfModules = (this.m_Version - 1) * 4 + 21;

            this.moduleValue = this._create2DArray(this.noOfModules, this.noOfModules);

            for (var i = 0; i < this.noOfModules; i++) {
                for (var j = 0; j < this.noOfModules; j++) {
                    this.moduleValue[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                }
            }

            this._drawPDP(0, 0);
            this._drawPDP(this.noOfModules - 7, 0);
            this._drawPDP(0, this.noOfModules - 7);
            this._drawTimingPattern();

            if (this.m_Version != 1) {
                var allignCoOrdinates = this._getAlignmentPatternCoOrdinates();

                for (var i = 0; i < allignCoOrdinates.length; i++) {
                    for (var j = 0; j < allignCoOrdinates.length; j++) {
                        var x = allignCoOrdinates[i];
                        var y = allignCoOrdinates[j];
                        if (this.moduleValue[x][y].IsPDP != true)
                            this._drawAlignmentPattern(x, y);
                    }
                }
            }

            this._allocateFormatVersionInfo();
            this._encodeData();
            this._drawFormatInformation();
            this._addQuietZone();

            var w = this.moduleValue.length * this.Model.xDimension;
            var h = this.moduleValue[0].length * this.Model.xDimension;

            /** Prepare canvas.*/
            this.Model.width = w;
            this.canvasEl[0].setAttribute("width", this.Model.width);
            this.BarcodeEl.css("width", this.Model.width);

            this.contextEl.beginPath();
            this.contextEl.fillStyle = 'white';
            this.contextEl.fillRect(0, 0, w, h);
            this.contextEl.closePath();

            /** Draw QR barcode in canvas.*/
            this._drawQRBarcode(h);
        },

        /** 
         * Allocate format and version information.
         * @private
         */
        _allocateFormatVersionInfo: function () {
            for (var i = 0; i < 9; i++) {
                this.moduleValue[8][i].IsFilled = true;
                this.moduleValue[i][8].IsFilled = true;
            }
            for (var i = this.noOfModules - 8; i < this.noOfModules; i++) {
                this.moduleValue[8][i].IsFilled = true;
                this.moduleValue[i][8].IsFilled = true;
            }
            if (this.m_Version > 6) {
                var versionInformation = this.qrbarcodeValues.VersionInformation;
                var count = 0;
                for (var i = 0; i < 6; i++) {
                    for (var j = 2; j >= 0; j--) {
                        this.moduleValue[i][this.noOfModules - 9 - j].IsBlack = versionInformation[count] == 1 ? true : false;
                        this.moduleValue[i][this.noOfModules - 9 - j].IsFilled = true;

                        this.moduleValue[this.noOfModules - 9 - j][i].IsBlack = versionInformation[count++] == 1 ? true : false;
                        this.moduleValue[this.noOfModules - 9 - j][i].IsFilled = true;
                    }
                }
            }
        },

        /**
         * @private
         */
        _drawTimingPattern: function () {
            for (var i = 8; i < this.noOfModules - 8; i += 2) {
                this.moduleValue[i][6].IsBlack = true;
                this.moduleValue[i][6].IsFilled = true;

                this.moduleValue[i + 1][6].IsBlack = false;
                this.moduleValue[i + 1][6].IsFilled = true;

                this.moduleValue[6][i].IsBlack = true;
                this.moduleValue[6][i].IsFilled = true;

                this.moduleValue[6][i + 1].IsBlack = false;
                this.moduleValue[6][i + 1].IsFilled = true;
            }
            this.moduleValue[this.noOfModules - 8][8].IsBlack = true;
            this.moduleValue[this.noOfModules - 8][8].IsFilled = true;
        },

        /** 
         * Draws QR barcode.
         * @private
         */
        _drawQRBarcode: function (ht) {
            var quietZone = this.Model.quietZone.all;
            if (quietZone < 2)
                quietZone = 2;

            var whiteBrush = 'white';
            var blackBrush = 'black';

            this.Model.height = ht;
            this.canvasEl[0].setAttribute("height", this.Model.height);
            this.BarcodeEl.css("height", this.Model.height);
            var dim = parseInt(this.Model.xDimension);

            var y = 0;

            var w = this.noOfModules + 2 * quietZone, h = this.noOfModules + 2 * quietZone;

            this.contextEl.beginPath();

            for (var i = 0; i < w; i++) {
                var x = 0;

                for (var j = 0; j < h; j++) {
                    var colorBrush = null;
                    if (this.moduleValue[i][j].IsBlack)
                        colorBrush = 'black';
                    else
                        colorBrush = 'white';

                    if (this.dataAllocationValues[j][i].IsFilled)
                        if (this.dataAllocationValues[j][i].IsBlack)
                            colorBrush = 'black';

                    this.contextEl.fillStyle = colorBrush;
                    this.contextEl.fillRect(x, y, dim, dim)

                    x += dim;
                }

                y += dim;
            }

            this.contextEl.closePath();
            this.contextEl.stroke();
        },

        /** 
         * Adds quiet zone to the barcode.
         * @private
         */
        _addQuietZone: function () {
            var quietZone = this.Model.quietZone.all;
            if (quietZone < 2)
                quietZone = 2;
            var w = this.noOfModules + 2 * quietZone;
            var h = this.noOfModules + 2 * quietZone;
            var tempValue1 = this._create2DArray(w, h);
            var tempValue2 = this._create2DArray(w, h);

            /** Initialize array.*/
            for (var i = 0; i < w; i++) {
                for (var j = 0; j < h; j++) {
                    tempValue1[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                    tempValue2[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                }
            }

            for (var i = quietZone; i < w - quietZone; i++) {
                for (var j = quietZone; j < h - quietZone; j++) {
                    tempValue1[i][j] = this.moduleValue[i - quietZone][j - quietZone];
                    tempValue2[i][j] = this.dataAllocationValues[i - quietZone][j - quietZone];
                }
            }

            this.moduleValue = tempValue1;
            this.dataAllocationValues = tempValue2;
        },

        /** 
         * Adds format information to the barcode.
         * @private
         */
        _drawFormatInformation: function () {
            var formatInformation = this.qrbarcodeValues.FormatInformation;

            var count = 0;
            for (var i = 0; i < 7; i++) {
                /** Draw from 14 to 8.*/
                if (i == 6)
                    this.moduleValue[i + 1][8].IsBlack = formatInformation[count] == 1 ? true : false;
                else
                    this.moduleValue[i][8].IsBlack = formatInformation[count] == 1 ? true : false;

                this.moduleValue[8][this.noOfModules - i - 1].IsBlack = formatInformation[count++] == 1 ? true : false;
            }
            count = 14;
            for (var i = 0; i < 7; i++) {
                /** Draw from 0 to 6.*/
                if (i == 6)
                    this.moduleValue[8][i + 1].IsBlack = formatInformation[count] == 1 ? true : false;
                else
                    this.moduleValue[8][i].IsBlack = formatInformation[count] == 1 ? true : false;
                this.moduleValue[this.noOfModules - i - 1][8].IsBlack = formatInformation[count--] == 1 ? true : false;
            }

            /** Draw 7.*/
            this.moduleValue[8][8].IsBlack = formatInformation[7] == 1 ? true : false;
            this.moduleValue[8][this.noOfModules - 8].IsBlack = formatInformation[7] == 1 ? true : false;
        },

        /** 
         * @private
         */
        _dataAllocationAndMasking: function (Data) {
            var num = this.noOfModules;
            this.dataAllocationValues = this._create2DArray(num, num);
            var point = 0;

            for (var i = 0; i < num; i++)
                for (var j = 0; j < num; j++)
                    this.dataAllocationValues[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };

            for (var i = num - 1; i >= 0; i -= 2) {
                for (var j = num - 1; j >= 0; j--)
                    point = this._allocateValues(Data, i, j, point);

                i -= 2;
                if (i == 6)
                    i--;

                for (var j = 0; j < num; j++)
                    point = this._allocateValues(Data, i, j, point);
            }
            for (var i = 0; i < num; i++)
                for (var j = 0; j < num; j++)
                    if (!this.moduleValue[i][j].IsFilled) {
                        var flag = this.dataAllocationValues[i][j].IsBlack;
                        if (flag)
                            this.dataAllocationValues[i][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i][j].IsBlack = true;
                    }


        },

        /** 
         * @private
         */
        _allocateValues: function (Data, i, j, point) {
            if (!(this.moduleValue[i][j].IsFilled && this.moduleValue[i - 1][j].IsFilled)) {
                if (!this.moduleValue[i][j].IsFilled) {
                    if (point + 1 < Data.length)
                        this.dataAllocationValues[i][j].IsBlack = Data[point++];
                    if ((i + j) % 3 == 0) {
                        if (this.dataAllocationValues[i][j].IsBlack)
                            this.dataAllocationValues[i][j].IsBlack = true;
                        else
                            this.dataAllocationValues[i][j].IsBlack = false;
                    }
                    else {
                        if (this.dataAllocationValues[i][j].IsBlack)
                            this.dataAllocationValues[i][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i][j].IsBlack = true;
                    }

                    this.dataAllocationValues[i][j].IsFilled = true;
                }

                if (!this.moduleValue[i - 1][j].IsFilled) {
                    if (point + 1 < Data.length)
                        this.dataAllocationValues[i - 1][j].IsBlack = Data[point++];
                    if ((i - 1 + j) % 3 == 0) {
                        if (this.dataAllocationValues[i - 1][j].IsBlack)
                            this.dataAllocationValues[i - 1][j].IsBlack = true;
                        else
                            this.dataAllocationValues[i - 1][j].IsBlack = false;
                    }
                    else {
                        if (this.dataAllocationValues[i - 1][j].IsBlack)
                            this.dataAllocationValues[i - 1][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i - 1][j].IsBlack = true;
                    }
                    this.dataAllocationValues[i - 1][j].IsFilled = true;
                }
            }

            return point;
        },

        /** 
         * Encodes data.
         * @private
         */
        _encodeData: function () {
            var eData = new Array();
            var c = -1;

            switch (this.mode) {
                case "numeric":
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    break;

                case "alphanumeric":
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    break;

                case "binary":
                    if (this.isEci) {
                        /** Add ECI Mode Indicator.*/
                        eData[++c] = false;
                        eData[++c] = true;
                        eData[++c] = true;
                        eData[++c] = true;

                        /** Add ECI assignment number.*/
                        var numberInBool = this._stringToBoolArray(this.m_EciAssignmentNumber.toString(), 8);
                        for (var e = 0; e < numberInBool.length; e++)
                            eData[++c] = numberInBool[e];
                    }
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                    break;
            }

            var numberOfBitsInCharacterCountIndicator = 0;
            if (this.m_Version < 10) {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 10;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 9;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 8;
                        break;
                }
            }
            else if (this.m_Version < 27) {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 12;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 11;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 16;
                        break;
                }
            }
            else {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 14;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 13;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 16;
                        break;
                }
            }

            var dataStringArray = this.Model.text;
            var numberOfBitsInCharacterCountIndicatorInBool = this._intToBoolArray(dataStringArray.length, numberOfBitsInCharacterCountIndicator);

            for (var i = 0; i < numberOfBitsInCharacterCountIndicator; i++)
                eData[++c] = numberOfBitsInCharacterCountIndicatorInBool[i];

            if (this.mode == "numeric") {
                var number = "";
                for (var i = 0; i < dataStringArray.length; i++) {
                    var numberInBool = new Array();
                    number += dataStringArray[i];

                    if (i % 3 == 2 && i != 0 || i == dataStringArray.length - 1) {
                        if (number.length == 3)
                            numberInBool = this._stringToBoolArray(number, 10);
                        else if (number.length == 2)
                            numberInBool = this._stringToBoolArray(number, 7);
                        else
                            numberInBool = this._stringToBoolArray(number, 4);
                        number = "";
                        for (var j = 0; j < numberInBool.length; j++)
                            eData[++c] = numberInBool[j];
                    }
                }
            }
            else if (this.mode == "alphanumeric") {
                var numberInString = "";
                var number = 0;
                for (var i = 0; i < dataStringArray.length; i++) {
                    var numberInBool = new Array();
                    numberInString += dataStringArray[i];

                    if (i % 2 == 0 && i + 1 != dataStringArray.length)
                        number = 45 * this._getAlphanumericvalues(dataStringArray[i]);

                    if (i % 2 == 1 && i != 0) {
                        number += this._getAlphanumericvalues(dataStringArray[i]);
                        numberInBool = this._intToBoolArray(number, 11);
                        number = 0;
                        for (var n = 0; n < numberInBool.length; n++)
                            eData[++c] = numberInBool[n];
                        numberInString = "";
                    }
                    if (i != 1 && numberInString != null)
                        if (i + 1 == dataStringArray.length && numberInString.length == 1) {
                            number = this._getAlphanumericvalues(dataStringArray[i]);
                            numberInBool = this._intToBoolArray(number, 6);
                            number = 0;
                            for (var j = 0; j < numberInBool.length; j++)
                                eData[++c] = numberInBool[j];
                        }
                }
            }
            else if (this.mode = "binary") {
                {
                    for (var i = 0; i < dataStringArray.length; i++) {
                        var number = 0;
                        if ((dataStringArray.charCodeAt(i) >= 32 && dataStringArray.charCodeAt(i) <= 126) || (dataStringArray.charCodeAt(i) >= 161 && dataStringArray.charCodeAt(i) <= 255))
                            number = dataStringArray.charCodeAt(i);
                        else if (dataStringArray.charCodeAt(i) >= 65377 && dataStringArray.charCodeAt(i) <= 65439)
                            number = dataStringArray.charCodeAt(i) - 65216;
                        else {
                            console.log("Input text contains non-convertable characters");
                            return;
                        }
                        var numberInBool = this._intToBoolArray(number, 8);
                        for (var j = 0; j < numberInBool.length; j++)
                            eData[++c] = numberInBool[j];
                    }
                }
            }

            /** Terminator.*/
            for (var i = 0; i < 4; i++)
                if (eData.length == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else
                    eData[++c] = false;

            /** Encode to code words.*/
            for (; ;)              /** Add Padding Bits.*/ {
                if (eData.length % 8 == 0)
                    break;
                else
                    eData[++c] = false;
            }

            for (; ;) {
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else {
                    eData[++c] = true;       //11101100
                    eData[++c] = true;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                }
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else {
                    eData[++c] = false;       //00010001
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                }
            }

            var dataBits = this.qrbarcodeValues.NumberOfDataCodeWord;
            blocks = this.qrbarcodeValues.NumberOfErrorCorrectionBlocks;

            var totalBlockSize = blocks[0];
            if (blocks.length == 6)
                totalBlockSize = blocks[0] + blocks[3];

            var ds1 = new Array(totalBlockSize);

            var testEncodeData = eData;
            var n = -1;
            if (blocks.length == 6)     /** If seperated into two seperate blocks.*/ {
                var dataCodeWordLength = blocks[0] * blocks[2] * 8;
                testEncodeData = new Array(dataCodeWordLength);
                for (var i = 0; i < dataCodeWordLength; i++)
                    testEncodeData[++n] = eData[i];
            }
            var dsOne = this._create2DArray(blocks[0], testEncodeData.length / 8 / blocks[0]);
            dsOne = this._createBlocks(testEncodeData, blocks[0]);

            for (var i = 0; i < blocks[0]; i++) {
                ds1[i] = this._splitCodeWord(dsOne, i, testEncodeData.length / 8 / blocks[0]);
            }

            n = -1;
            if (blocks.length == 6) {
                testEncodeData = new Array();
                for (var i = parseInt(blocks[0] * blocks[2] * 8) ; i < eData.length; i++)
                    testEncodeData[++n] = eData[i];

                var dsTwo = this._create2DArray(blocks[0], testEncodeData.length / 8 / blocks[3]);
                dsTwo = this._createBlocks(testEncodeData, blocks[3]);

                for (var i = blocks[0], count = 0; i < totalBlockSize; i++)
                    ds1[i] = this._splitCodeWord(dsTwo, count++, testEncodeData.length / 8 / blocks[3]);
            }
            eData = new Array();
            c = -1;
            for (var i = 0; i < 125; i++) {
                for (var k = 0; k < totalBlockSize; k++)
                    for (var j = 0; j < 8; j++)
                        if (i < ds1[k].length) {
                            var temp = ds1[k][i][j];
                            eData[++c] = (temp == '1' ? true : false);
                        }
            }

            this.corCodeWords = { DataBits: this.qrbarcodeValues.NumberOfDataCodeWord, ECCW: this.qrbarcodeValues.NumberOfErrorCorrectingCodeWords, DC: 0 };

            var dataBits = this.corCodeWords.DataBits;
            var eccw = this.corCodeWords.ECCW;
            blocks = this.qrbarcodeValues.NumberOfErrorCorrectionBlocks;

            if (blocks.length == 6)
                this.corCodeWords.DataBits = (dataBits - blocks[3] * blocks[5]) / blocks[0];
            else
                this.corCodeWords.DataBits = dataBits / blocks[0];

            this.corCodeWords.ECCW = eccw / totalBlockSize;
            var polynomial = new Array(totalBlockSize);
            var count = 0;

            for (var i = 0; i < blocks[0]; i++) {
                this.corCodeWords.DC = ds1[count];
                polynomial[count++] = this._getERCW();
            }
            if (blocks.length == 6) {
                this.corCodeWords.DataBits = (dataBits - blocks[0] * blocks[2]) / blocks[3];

                for (var i = 0; i < blocks[3]; i++) {
                    this.corCodeWords.DC = ds1[count];
                    polynomial[count++] = this._getERCW();
                }
            }
            if (blocks.length != 6) {
                for (var i = 0; i < polynomial[0].length; i++) {
                    for (var k = 0; k < blocks[0]; k++)
                        for (var j = 0; j < 8; j++)
                            if (i < polynomial[k].length)
                                eData[++c] = (polynomial[k][i][j] == '1' ? true : false);
                }
            }
            else {
                for (var i = 0; i < polynomial[0].length; i++) {
                    for (var k = 0; k < totalBlockSize; k++)
                        for (var j = 0; j < 8; j++)
                            if (i < polynomial[k].length)
                                eData[++c] = (polynomial[k][i][j] == '1' ? true : false);
                }
            }


            this._dataAllocationAndMasking(eData);
        },

        /** 
         * @private
         */
        _getERCW: function () {
            var decimalRepresentation;
            var ecw;
            var decimalValue = new Array(this.corCodeWords.DataBits);
            var gx = new Array();

            switch (this.corCodeWords.ECCW) {
                case 7:
                    gx = new Array(0, 87, 229, 146, 149, 238, 102, 21);
                    gx = this._getElement(gx);
                    break;
                case 10:
                    gx = new Array(0, 251, 67, 46, 61, 118, 70, 64, 94, 32, 45);
                    gx = this._getElement(gx);
                    break;
                case 13:
                    gx = new Array(0, 74, 152, 176, 100, 86, 100, 106, 104, 130, 218, 206, 140, 78);
                    gx = this._getElement(gx);
                    break;
                case 15:
                    gx = new Array(0, 8, 183, 61, 91, 202, 37, 51, 58, 58, 237, 140, 124, 5, 99, 105);
                    gx = this._getElement(gx);
                    break;
                case 16:
                    gx = new Array(0, 120, 104, 107, 109, 102, 161, 76, 3, 91, 191, 147, 169, 182, 194, 225, 120);
                    gx = this._getElement(gx);
                    break;
                case 17:
                    gx = new Array(0, 43, 139, 206, 78, 43, 239, 123, 206, 214, 147, 24, 99, 150, 39, 243, 163, 136);
                    gx = this._getElement(gx);
                    break;
                case 18:
                    gx = new Array(0, 215, 234, 158, 94, 184, 97, 118, 170, 79, 187, 152, 148, 252, 179, 5, 98, 96, 153);
                    gx = this._getElement(gx);
                    break;
                case 20:
                    gx = new Array(0, 17, 60, 79, 50, 61, 163, 26, 187, 202, 180, 221, 225, 83, 239, 156, 164, 212, 212, 188, 190);
                    gx = this._getElement(gx);
                    break;
                case 22:
                    gx = new Array(0, 210, 171, 247, 242, 93, 230, 14, 109, 221, 53, 200, 74, 8, 172, 98, 80, 219, 134, 160, 105, 165, 231);
                    gx = this._getElement(gx);
                    break;
                case 24:
                    gx = new Array(0, 229, 121, 135, 48, 211, 117, 251, 126, 159, 180, 169, 152, 192, 226, 228, 218, 111, 0, 117, 232, 87, 96, 227, 21);
                    gx = this._getElement(gx);
                    break;
                case 26:
                    gx = new Array(0, 173, 125, 158, 2, 103, 182, 118, 17, 145, 201, 111, 28, 165, 53, 161, 21, 245, 142, 13, 102, 48, 227, 153, 145, 218, 70);
                    gx = this._getElement(gx);
                    break;
                case 28:
                    gx = new Array(0, 168, 223, 200, 104, 224, 234, 108, 180, 110, 190, 195, 147, 205, 27, 232, 201, 21, 43, 245, 87, 42, 195, 212, 119, 242, 37, 9, 123);
                    gx = this._getElement(gx);
                    break;
                case 30:
                    gx = new Array(0, 41, 173, 145, 152, 216, 31, 179, 182, 50, 48, 110, 86, 239, 96, 222, 125, 42, 173, 226, 193, 224, 130, 156, 37, 251, 216, 238, 40, 192, 180);
                    gx = this._getElement(gx);
                    break;
                case 32:
                    gx = new Array(0, 10, 6, 106, 190, 249, 167, 4, 67, 209, 138, 138, 32, 242, 123, 89, 27, 120, 185, 80, 156, 38, 69, 171, 60, 28, 222, 80, 52, 254, 185, 220, 241);
                    gx = this._getElement(gx);
                    break;
                case 34:
                    gx = new Array(0, 111, 77, 146, 94, 26, 21, 108, 19, 105, 94, 113, 193, 86, 140, 163, 125, 58, 158, 229, 239, 218, 103, 56, 70, 114, 61, 183, 129, 167, 13, 98, 62, 129, 51);
                    gx = this._getElement(gx);
                    break;
                case 36:
                    gx = new Array(0, 200, 183, 98, 16, 172, 31, 246, 234, 60, 152, 115, 0, 167, 152, 113, 248, 238, 107, 18, 63, 218, 37, 87, 210, 105, 177, 120, 74, 121, 196, 117, 251, 113, 233, 30, 120);
                    gx = this._getElement(gx);
                    break;
                case 40:
                    gx = new Array(0, 59, 116, 79, 161, 252, 98, 128, 205, 128, 161, 247, 57, 163, 56, 235, 106, 53, 26, 187, 174, 226, 104, 170, 7, 175, 35, 181, 114, 88, 41, 47, 163, 125, 134, 72, 20, 232, 53, 35, 15);
                    gx = this._getElement(gx);
                    break;
                case 42:
                    gx = new Array(0, 250, 103, 221, 230, 25, 18, 137, 231, 0, 3, 58, 242, 221, 191, 110, 84, 230, 8, 188, 106, 96, 147, 15, 131, 139, 34, 101, 223, 39, 101, 213, 199, 237, 254, 201, 123, 171, 162, 194, 117, 50, 96);
                    gx = this._getElement(gx);
                    break;
                case 44:
                    gx = new Array(0, 190, 7, 61, 121, 71, 246, 69, 55, 168, 188, 89, 243, 191, 25, 72, 123, 9, 145, 14, 247, 1, 238, 44, 78, 143, 62, 224, 126, 118, 114, 68, 163, 52, 194, 217, 147, 204, 169, 37, 130, 113, 102, 73, 181);
                    gx = this._getElement(gx);
                    break;
                case 46:
                    gx = new Array(0, 112, 94, 88, 112, 253, 224, 202, 115, 187, 99, 89, 5, 54, 113, 129, 44, 58, 16, 135, 216, 169, 211, 36, 1, 4, 96, 60, 241, 73, 104, 234, 8, 249, 245, 119, 174, 52, 25, 157, 224, 43, 202, 223, 19, 82, 15);
                    gx = this._getElement(gx);
                    break;
                case 48:
                    gx = new Array(0, 228, 25, 196, 130, 211, 146, 60, 24, 251, 90, 39, 102, 240, 61, 178, 63, 46, 123, 115, 18, 221, 111, 135, 160, 182, 205, 107, 206, 95, 150, 120, 184, 91, 21, 247, 156, 140, 238, 191, 11, 94, 227, 84, 50, 163, 39, 34, 108);
                    gx = this._getElement(gx);
                    break;
                case 50:
                    gx = new Array(0, 232, 125, 157, 161, 164, 9, 118, 46, 209, 99, 203, 193, 35, 3, 209, 111, 195, 242, 203, 225, 46, 13, 32, 160, 126, 209, 130, 160, 242, 215, 242, 75, 77, 42, 189, 32, 113, 65, 124, 69, 228, 114, 235, 175, 124, 170, 215, 232, 133, 205);
                    gx = this._getElement(gx);
                    break;
                case 52:
                    gx = new Array(0, 116, 50, 86, 186, 50, 220, 251, 89, 192, 46, 86, 127, 124, 19, 184, 233, 151, 215, 22, 14, 59, 145, 37, 242, 203, 134, 254, 89, 190, 94, 59, 65, 124, 113, 100, 233, 235, 121, 22, 76, 86, 97, 39, 242, 200, 220, 101, 33, 239, 254, 116, 51);
                    gx = this._getElement(gx);
                    break;
                case 54:
                    gx = new Array(0, 183, 26, 201, 87, 210, 221, 113, 21, 46, 65, 45, 50, 238, 184, 249, 225, 102, 58, 209, 218, 109, 165, 26, 95, 184, 192, 52, 245, 35, 254, 238, 175, 172, 79, 123, 25, 122, 43, 120, 108, 215, 80, 128, 201, 235, 8, 153, 59, 101, 31, 198, 76, 31, 156);
                    gx = this._getElement(gx);
                    break;
                case 56:
                    gx = new Array(0, 106, 120, 107, 157, 164, 216, 112, 116, 2, 91, 248, 163, 36, 201, 202, 229, 6, 144, 254, 155, 135, 208, 170, 209, 12, 139, 127, 142, 182, 249, 177, 174, 190, 28, 10, 85, 239, 184, 101, 124, 152, 206, 96, 23, 163, 61, 27, 196, 247, 151, 154, 202, 207, 20, 61, 10);
                    gx = this._getElement(gx);
                    break;
                case 58:
                    gx = new Array(0, 82, 116, 26, 247, 66, 27, 62, 107, 252, 182, 200, 185, 235, 55, 251, 242, 210, 144, 154, 237, 176, 141, 192, 248, 152, 249, 206, 85, 253, 142, 65, 165, 125, 23, 24, 30, 122, 240, 214, 6, 129, 218, 29, 145, 127, 134, 206, 245, 117, 29, 41, 63, 159, 142, 233, 125, 148, 123);
                    gx = this._getElement(gx);
                    break;
                case 60:
                    gx = new Array(0, 107, 140, 26, 12, 9, 141, 243, 197, 226, 197, 219, 45, 211, 101, 219, 120, 28, 181, 127, 6, 100, 247, 2, 205, 198, 57, 115, 219, 101, 109, 160, 82, 37, 38, 238, 49, 160, 209, 121, 86, 11, 124, 30, 181, 84, 25, 194, 87, 65, 102, 190, 220, 70, 27, 209, 16, 89, 7, 33, 240);
                    gx = this._getElement(gx);
                    break;
                case 62:
                    gx = new Array(0, 65, 202, 113, 98, 71, 223, 248, 118, 214, 94, 0, 122, 37, 23, 2, 228, 58, 121, 7, 105, 135, 78, 243, 118, 70, 76, 223, 89, 72, 50, 70, 111, 194, 17, 212, 126, 181, 35, 221, 117, 235, 11, 229, 149, 147, 123, 213, 40, 115, 6, 200, 100, 26, 246, 182, 218, 127, 215, 36, 186, 110, 106);
                    gx = this._getElement(gx);
                    break;
                case 64:
                    gx = new Array(0, 45, 51, 175, 9, 7, 158, 159, 49, 68, 119, 92, 123, 177, 204, 187, 254, 200, 78, 141, 149, 119, 26, 127, 53, 160, 93, 199, 212, 29, 24, 145, 156, 208, 150, 218, 209, 4, 216, 91, 47, 184, 146, 47, 140, 195, 195, 125, 242, 238, 63, 99, 108, 140, 230, 242, 31, 204, 11, 178, 243, 217, 156, 213, 231);
                    gx = this._getElement(gx);
                    break;
                case 66:
                    gx = new Array(0, 5, 118, 222, 180, 136, 136, 162, 51, 46, 117, 13, 215, 81, 17, 139, 247, 197, 171, 95, 173, 65, 137, 178, 68, 111, 95, 101, 41, 72, 214, 169, 197, 95, 7, 44, 154, 77, 111, 236, 40, 121, 143, 63, 87, 80, 253, 240, 126, 217, 77, 34, 232, 106, 50, 168, 82, 76, 146, 67, 106, 171, 25, 132, 93, 45, 105);
                    gx = this._getElement(gx);
                    break;
                case 68:
                    gx = new Array(0, 247, 159, 223, 33, 224, 93, 77, 70, 90, 160, 32, 254, 43, 150, 84, 101, 190, 205, 133, 52, 60, 202, 165, 220, 203, 151, 93, 84, 15, 84, 253, 173, 160, 89, 227, 52, 199, 97, 95, 231, 52, 177, 41, 125, 137, 241, 166, 225, 118, 2, 54, 32, 82, 215, 175, 198, 43, 238, 235, 27, 101, 184, 127, 3, 5, 8, 163, 238);
                    gx = this._getElement(gx);
                    break;
            }

            decimalValue = this._toDecimal(this.corCodeWords.DC, decimalValue);
            decimalRepresentation = this._divide(gx, decimalValue);
            ecw = this._toBinary(decimalRepresentation);
            return ecw;
        },

        /** 
         * Convert to binary.
         * @private
         */
        _toBinary: function (decimalRepresentation) {
            var toBinary = new Array(this.corCodeWords.ECCW);
            for (var i = 0; i < decimalRepresentation.length; i++) {
                var temp = decimalRepresentation[i].toString(2);
                var str = new String();
                for (var j = 0; j < (8 - temp.length) ; j++)
                    str += "0";
                toBinary[i] = str + temp;
            }
            return toBinary;
        },

        /** 
         * @private
         */
        _divide: function (gx, decimalValue) {
            var alpha = new Array(1, 2, 4, 8, 16, 32, 64, 128, 29, 58, 116, 232, 205, 135, 19, 38, 76, 152, 45, 90, 180, 117, 234, 201, 143, 3, 6, 12, 24, 48, 96, 192, 157, 39, 78, 156, 37, 74, 148, 53, 106, 212, 181, 119, 238, 193, 159, 35, 70, 140, 5, 10, 20, 40, 80, 160, 93, 186, 105, 210, 185, 111, 222, 161, 95, 190, 97, 194, 153, 47, 94, 188, 101, 202, 137, 15, 30, 60, 120, 240, 253, 231, 211, 187, 107, 214, 177, 127, 254, 225, 223, 163, 91, 182, 113, 226, 217, 175, 67, 134, 17, 34, 68, 136, 13, 26, 52, 104, 208, 189, 103, 206, 129, 31, 62, 124, 248, 237, 199, 147, 59, 118, 236, 197, 151, 51, 102, 204, 133, 23, 46, 92, 184, 109, 218, 169, 79, 158, 33, 66, 132, 21, 42, 84, 168, 77, 154, 41, 82, 164, 85, 170, 73, 146, 57, 114, 228, 213, 183, 115, 230, 209, 191, 99, 198, 145, 63, 126, 252, 229, 215, 179, 123, 246, 241, 255, 227, 219, 171, 75, 150, 49, 98, 196, 149, 55, 110, 220, 165, 87, 174, 65, 130, 25, 50, 100, 200, 141, 7, 14, 28, 56, 112, 224, 221, 167, 83, 166, 81, 162, 89, 178, 121, 242, 249, 239, 195, 155, 43, 86, 172, 69, 138, 9, 18, 36, 72, 144, 61, 122, 244, 245, 247, 243, 251, 235, 203, 139, 11, 22, 44, 88, 176, 125, 250, 233, 207, 131, 27, 54, 108, 216, 173, 71, 142);
            var m_length = this.corCodeWords.DataBits + this.corCodeWords.ECCW;
            var result = new Array(m_length);
            var fx = new Array(this.corCodeWords.ECCW);
            var a1 = new Array(gx.length);
            var a2 = new Array(m_length);

            for (var i = 0; i < gx.length; i++)
                a1[i] = this._findElement(gx[i], alpha);

            for (var i = 0; i < decimalValue.length; i++)
                result[i] = decimalValue[i];

            for (var i = 0; i < this.corCodeWords.DC.length; i++) {
                a2[i] = this._findElement(result[i], alpha);
                var g = i, z = i;
                for (var j = 0; j < gx.length; j++) {
                    var e = a1[j] + a2[i];
                    if (e >= 255)
                        e = e - 255;
                    result[z] = alpha[e] ^ result[g];
                    z++;
                    g++;
                }
            }
            var k = result.length - 1;
            for (var i = this.corCodeWords.ECCW - 1; i >= 0; i--) {
                fx[i] = result[k];
                k--;
            }
            return fx;
        },

        /** 
         * @private
         */
        _findElement: function (element, alpha) {
            var j = 0;
            for (; j < alpha.length; j++) {
                if (element == alpha[j])
                    break;
            }
            return j;
        },

        /** 
         * @private
         */
        _getElement: function (element) {
            var gx = new Array(element.length);
            var alpha = new Array(1, 2, 4, 8, 16, 32, 64, 128, 29, 58, 116, 232, 205, 135, 19, 38, 76, 152, 45, 90, 180, 117, 234, 201, 143, 3, 6, 12, 24, 48, 96, 192, 157, 39, 78, 156, 37, 74, 148, 53, 106, 212, 181, 119, 238, 193, 159, 35, 70, 140, 5, 10, 20, 40, 80, 160, 93, 186, 105, 210, 185, 111, 222, 161, 95, 190, 97, 194, 153, 47, 94, 188, 101, 202, 137, 15, 30, 60, 120, 240, 253, 231, 211, 187, 107, 214, 177, 127, 254, 225, 223, 163, 91, 182, 113, 226, 217, 175, 67, 134, 17, 34, 68, 136, 13, 26, 52, 104, 208, 189, 103, 206, 129, 31, 62, 124, 248, 237, 199, 147, 59, 118, 236, 197, 151, 51, 102, 204, 133, 23, 46, 92, 184, 109, 218, 169, 79, 158, 33, 66, 132, 21, 42, 84, 168, 77, 154, 41, 82, 164, 85, 170, 73, 146, 57, 114, 228, 213, 183, 115, 230, 209, 191, 99, 198, 145, 63, 126, 252, 229, 215, 179, 123, 246, 241, 255, 227, 219, 171, 75, 150, 49, 98, 196, 149, 55, 110, 220, 165, 87, 174, 65, 130, 25, 50, 100, 200, 141, 7, 14, 28, 56, 112, 224, 221, 167, 83, 166, 81, 162, 89, 178, 121, 242, 249, 239, 195, 155, 43, 86, 172, 69, 138, 9, 18, 36, 72, 144, 61, 122, 244, 245, 247, 243, 251, 235, 203, 139, 11, 22, 44, 88, 176, 125, 250, 233, 207, 131, 27, 54, 108, 216, 173, 71, 142);
            for (var i = 0; i < element.length; i++) {
                if (element[i] > 255)
                    element[i] = element[i] - 255;
                gx[i] = alpha[element[i]];
            }
            return gx;
        },

        /** 
         * @private
         */
        _toDecimal: function (inString) {
            var decimalValue = new Array(inString.length);
            for (var i = 0; i < inString.length; i++)
                decimalValue[i] = parseInt(inString[i], 2);

            return decimalValue;
        },

        /** 
         * @private
         */
        _splitCodeWord: function (ds, blk, count) {
            count = parseInt(count);
            var ds1 = new Array(count);
            for (var i = 0; i < count; i++)
                ds1[i] = ds[blk][i];
            return ds1;
        },

        /** 
         * Create blocks for datamatrix barcode.
         * @private
         */
        _createBlocks: function (data, noOfBlocks) {
            var ret = this._create2DArray(noOfBlocks, data.length / 8 / noOfBlocks);
            var stringValue = "";
            var j = 0, i = 0, blockNumber = 0;

            for (; j < data.length; j++) {
                if (j % 8 == 0 && j != 0) {
                    ret[blockNumber][i] = stringValue;
                    stringValue = "";
                    i++;

                    if (i == (data.length / noOfBlocks / 8)) {
                        blockNumber++;
                        i = 0;
                    }
                }

                stringValue += data[j] ? 1 : 0;
            }
            ret[blockNumber][i] = stringValue;
            return ret;
        },

        /** 
         * Returns alphanumeric values for the character.
         * @private
         */
        _getAlphanumericvalues: function (charValue) {
            var valueInInt = -1;
            var alphaNumArray = new Array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', ' ', '$', '%', '*', '+', '-', '.', '/', ':');
            valueInInt = alphaNumArray.indexOf(charValue);
            return valueInInt;
        },

        /** 
         * @private
         */
        _stringToBoolArray: function (numberInString, noOfBits) {
            var numberInBool = new Array(noOfBits);

            var number = 0;
            for (var i = 0; i < numberInString.length; i++)
                number = number * 10 + numberInString.charCodeAt(i) - 48;

            for (var i = 0; i < noOfBits; i++)
                numberInBool[noOfBits - i - 1] = ((number >> i) & 1) == 1;

            return numberInBool;
        },

        /** 
         * @private
         */
        _intToBoolArray: function (number, noOfBits) {
            var numberInBool = new Array(noOfBits);
            for (var i = 0; i < noOfBits; i++)
                numberInBool[noOfBits - i - 1] = ((number >> i) & 1) == 1;
            return numberInBool;
        },

        /** 
         * @private
         */
        _drawPDP: function (x, y) {
            var i, j;
            for (i = x, j = y; i < x + 7; i++, j++) {
                this.moduleValue[i][y].IsBlack = true;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 6].IsBlack = true;
                this.moduleValue[i][y + 6].IsFilled = true;
                this.moduleValue[i][y + 6].IsPDP = true;

                if (y + 7 < this.noOfModules) {
                    this.moduleValue[i][y + 7].IsBlack = false;
                    this.moduleValue[i][y + 7].IsFilled = true;
                    this.moduleValue[i][y + 7].IsPDP = true;
                }
                else if (y - 1 >= 0) {
                    this.moduleValue[i][y - 1].IsBlack = false;
                    this.moduleValue[i][y - 1].IsFilled = true;
                    this.moduleValue[i][y - 1].IsPDP = true;
                }

                this.moduleValue[x][j].IsBlack = true;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 6][j].IsBlack = true;
                this.moduleValue[x + 6][j].IsFilled = true;
                this.moduleValue[x + 6][j].IsPDP = true;

                if (x + 7 < this.noOfModules) {
                    this.moduleValue[x + 7][j].IsBlack = false;
                    this.moduleValue[x + 7][j].IsFilled = true;
                    this.moduleValue[x + 7][j].IsPDP = true;
                }
                else if (x - 1 >= 0) {
                    this.moduleValue[x - 1][j].IsBlack = false;
                    this.moduleValue[x - 1][j].IsFilled = true;
                    this.moduleValue[x - 1][j].IsPDP = true;
                }
            }

            if (x + 7 < this.noOfModules && y + 7 < this.noOfModules) {
                this.moduleValue[x + 7][y + 7].IsBlack = false;
                this.moduleValue[x + 7][y + 7].IsFilled = true;
                this.moduleValue[x + 7][y + 7].IsPDP = true;
            }
            else if (x + 7 < this.noOfModules && y + 7 >= this.noOfModules) {
                this.moduleValue[x + 7][y - 1].IsBlack = false;
                this.moduleValue[x + 7][y - 1].IsFilled = true;
                this.moduleValue[x + 7][y - 1].IsPDP = true;
            }
            else if (x + 7 >= this.noOfModules && y + 7 < this.noOfModules) {
                this.moduleValue[x - 1][y + 7].IsBlack = false;
                this.moduleValue[x - 1][y + 7].IsFilled = true;
                this.moduleValue[x - 1][y + 7].IsPDP = true;
            }

            x++;
            y++;
            for (i = x, j = y; i < x + 5; i++, j++) {
                this.moduleValue[i][y].IsBlack = false;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 4].IsBlack = false;
                this.moduleValue[i][y + 4].IsFilled = true;
                this.moduleValue[i][y + 4].IsPDP = true;

                this.moduleValue[x][j].IsBlack = false;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 4][j].IsBlack = false;
                this.moduleValue[x + 4][j].IsFilled = true;
                this.moduleValue[x + 4][j].IsPDP = true;
            }

            x++;
            y++;
            for (i = x, j = y; i < x + 3; i++, j++) {
                this.moduleValue[i][y].IsBlack = true;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 2].IsBlack = true;
                this.moduleValue[i][y + 2].IsFilled = true;
                this.moduleValue[i][y + 2].IsPDP = true;

                this.moduleValue[x][j].IsBlack = true;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 2][j].IsBlack = true;
                this.moduleValue[x + 2][j].IsFilled = true;
                this.moduleValue[x + 2][j].IsPDP = true;
            }
            this.moduleValue[x + 1][y + 1].IsBlack = true;
            this.moduleValue[x + 1][y + 1].IsFilled = true;
            this.moduleValue[x + 1][y + 1].IsPDP = true;
        },

        /** 
         * @private
         */
        _drawAlignmentPattern: function (x, y) {
            var i, j;
            for (i = x - 2, j = y - 2; i < x + 3; i++, j++) {
                this.moduleValue[i][y - 2].IsBlack = true;
                this.moduleValue[i][y - 2].IsFilled = true;

                this.moduleValue[i][y + 2].IsBlack = true;
                this.moduleValue[i][y + 2].IsFilled = true;

                this.moduleValue[x - 2][j].IsBlack = true;
                this.moduleValue[x - 2][j].IsFilled = true;

                this.moduleValue[x + 2][j].IsBlack = true;
                this.moduleValue[x + 2][j].IsFilled = true;
            }

            for (i = x - 1, j = y - 1; i < x + 2; i++, j++) {
                this.moduleValue[i][y - 1].IsBlack = false;
                this.moduleValue[i][y - 1].IsFilled = true;

                this.moduleValue[i][y + 1].IsBlack = false;
                this.moduleValue[i][y + 1].IsFilled = true;

                this.moduleValue[x - 1][j].IsBlack = false;
                this.moduleValue[x - 1][j].IsFilled = true;

                this.moduleValue[x + 1][j].IsBlack = false;
                this.moduleValue[x + 1][j].IsFilled = true;
            }
            this.moduleValue[x][y].IsBlack = true;
            this.moduleValue[x][y].IsFilled = true;
        },

        /** 
         * @private
         */
        _getAlignmentPatternCoOrdinates: function () {
            var allign = new Array();
            switch (this.m_Version) {
                case 02:
                    allign = new Array(6, 18);
                    break;
                case 03:
                    allign = new Array(6, 22);
                    break;
                case 04:
                    allign = new Array(6, 26);
                    break;
                case 05:
                    allign = new Array(6, 30);
                    break;
                case 06:
                    allign = new Array(6, 34);
                    break;
                case 07:
                    allign = new Array(6, 22, 38);
                    break;
                case 08:
                    allign = new Array(6, 24, 42);
                    break;
                case 09:
                    allign = new Array(6, 26, 46);
                    break;
                case 10:
                    allign = new Array(6, 28, 50);
                    break;
                case 11:
                    allign = new Array(6, 30, 54);
                    break;
                case 12:
                    allign = new Array(6, 32, 58);
                    break;
                case 13:
                    allign = new Array(6, 34, 62);
                    break;
                case 14:
                    allign = new Array(6, 26, 46, 66);
                    break;
                case 15:
                    allign = new Array(6, 26, 48, 70);
                    break;
                case 16:
                    allign = new Array(6, 26, 50, 74);
                    break;
                case 17:
                    allign = new Array(6, 30, 54, 78);
                    break;
                case 18:
                    allign = new Array(6, 30, 56, 82);
                    break;
                case 19:
                    allign = new Array(6, 30, 58, 86);
                    break;
                case 20:
                    allign = new Array(6, 34, 62, 90);
                    break;
                case 21:
                    allign = new Array(6, 28, 50, 72, 94);
                    break;
                case 22:
                    allign = new Array(6, 26, 50, 74, 98);
                    break;
                case 23:
                    allign = new Array(6, 30, 54, 78, 102);
                    break;
                case 24:
                    allign = new Array(6, 28, 54, 80, 106);
                    break;
                case 25:
                    allign = new Array(6, 32, 58, 84, 110);
                    break;
                case 26:
                    allign = new Array(6, 30, 58, 86, 114);
                    break;
                case 27:
                    allign = new Array(6, 34, 62, 90, 118);
                    break;
                case 28:
                    allign = new Array(6, 26, 50, 74, 98, 122);
                    break;
                case 29:
                    allign = new Array(6, 30, 54, 78, 102, 126);
                    break;
                case 30:
                    allign = new Array(6, 26, 52, 78, 104, 130);
                    break;
                case 31:
                    allign = new Array(6, 30, 56, 82, 108, 134);
                    break;
                case 32:
                    allign = new Array(6, 34, 60, 86, 112, 138);
                    break;
                case 33:
                    allign = new Array(6, 30, 58, 86, 114, 142);
                    break;
                case 34:
                    allign = new Array(6, 34, 62, 90, 118, 146);
                    break;
                case 35:
                    allign = new Array(6, 30, 54, 78, 102, 126, 150);
                    break;
                case 36:
                    allign = new Array(6, 24, 50, 76, 102, 128, 154);
                    break;
                case 37:
                    allign = new Array(6, 28, 54, 80, 106, 132, 158);
                    break;
                case 38:
                    allign = new Array(6, 32, 58, 84, 110, 136, 162);
                    break;
                case 39:
                    allign = new Array(6, 26, 54, 82, 110, 138, 166);
                    break;
                case 40:
                    allign = new Array(6, 30, 58, 86, 114, 142, 170);
                    break;
            }
            return allign;
        },

        /** 
         * @private
         */
        _findECINumber: function (text) {
            var num = 3;
            for (var i = 0; i < text.length; i++) {
                var cCode = text.charCodeAt(i);
                if (cCode >= 32 && cCode <= 255) {
                    continue;
                }

                var check = cCode.toString(16);
                var CP437CharSet = new Array("2591", "2592", "2593", "2502", "2524", "2561", "2562", "2556", "2555", "2563", "2551", "2557", "255D", "255C", "255B", "2510", "2514", "2534", "252C", "251C", "2500", "253C", "255E", "255F", "255A", "2554", "2569", "2566", "2560", "2550", "256C", "2567", "2568", "2564", "2565", "2559", "2558", "2552", "2553", "256B", "256A", "2518", "250C", "2588", "2584", "258C", "2590", "2580", "25A0");
                var latin2CharSet = new Array("104", "2D8", "141", "13D", "15A", "160", "15E", "164", "179", "17D", "17B", "105", "2DB", "142", "13E", "15B", "2C7", "161", "15F", "165", "17A", "2DD", "17E", "17C", "154", "102", "139", "106", "10C", "118", "11A", "10E", "110", "143", "147", "150", "158", "16E", "170", "162", "155", "103", "13A", "107", "10D", "119", "11B", "10F", "111", "144", "148", "151", "159", "16F", "171", "163", "2D9");
                var latin3CharSet = new Array("126", "124", "130", "15E", "11E", "134", "17B", "127", "125", "131", "15F", "11F", "135", "17C", "10A", "108", "120", "11C", "16C", "15C", "10B", "109", "121", "11D", "16D", "15D");
                var latin4CharSet = new Array("104", "138", "156", "128", "13B", "160", "112", "122", "166", "17D", "105", "2DB", "157", "129", "13C", "2C7", "161", "113", "123", "167", "14A", "17E", "14B", "100", "12E", "10C", "118", "116", "12A", "110", "145", "14C", "136", "172", "168", "16A", "101", "12F", "10D", "119", "117", "12B", "111", "146", "14D", "137", "173", "169", "16B");
                var windows1250CharSet = new Array("141", "104", "15E", "17B", "142", "105", "15F", "13D", "13E", "17C");
                var windows1251CharSet = new Array("402", "403", "453", "409", "40A", "40C", "40B", "40F", "452", "459", "45A", "45C", "45B", "45F", "40E", "45E", "408", "490", "401", "404", "407", "406", "456", "491", "451", "454", "458", "405", "455", "457");
                var windows1252CharSet = new Array("20AC", "201A", "192", "201E", "2026", "2020", "2021", "2C6", "2030", "160", "2039", "152", "17D", "2018", "2019", "201C", "201D", "2022", "2013", "2014", "2DC", "2122", "161", "203A", "153", "17E", "178");
                var windows1256CharSet = new Array("67E", "679", "152", "686", "698", "688", "6AF", "6A9", "691", "153", "6BA", "6BE", "6C1");

                /** Check for CP437*/
                if (CP437CharSet.indexOf(check) != -1) {
                    num = 2;
                    break;
                }
                    /** Check for ISO/IEC 8859-2*/
                else if (latin2CharSet.indexOf(check) != -1) {
                    num = 4;
                    break;
                }
                    /** Check for ISO/IEC 8859-3*/
                else if (latin3CharSet.indexOf(check) != -1) {
                    num = 5;
                    break;
                }
                    /** Check for ISO/IEC 8859-4*/
                else if (latin4CharSet.indexOf(check) != -1) {
                    num = 6;
                    break;
                }
                    /** Check for ISO/IEC 8859-5*/
                else if (check >= 1025 && check <= 1119 && check != 1037 && check != 1104 && check != 1117) {
                    num = 7;
                    break;
                }
                    /** Check for ISO/IEC 8859-6*/
                else if ((check >= 1569 && check <= 1594) || (check >= 1600 && check <= 1618) || check == 1567 || check == 1563 || check == 1548) {
                    num = 8;
                    break;
                }
                    /** Check for ISO/IEC 8859-7*/
                else if ((check >= 900 && check <= 974) || check == 890) {
                    num = 9;
                    break;
                }
                    /** Check for ISO/IEC 8859-8*/
                else if (check >= 1488 && check <= 1514) {
                    num = 10;
                    break;
                }
                    /** Check for ISO/IEC 8859-8*/
                else if (check >= 3585 && check <= 3675) {
                    num = 13;
                    break;
                }
                    /** Check for Windows1250*/
                else if ((windows1250CharSet.indexOf(check) != 1) || (check >= 1569 && check <= 1610)) {
                    num = 21;
                    break;
                }
                    /** Check for Windows1251*/
                else if ((windows1251CharSet.indexOf(check) != 1) || (check >= 1040 && check <= 1103)) {
                    num = 22;
                    break;
                }
                    /** Check for Windows1252*/
                else if (windows1252CharSet.indexOf(check) != -1) {
                    num = 23;
                    break;
                }
                    /** Check for Windows1256*/
                else if (windows1256CharSet.indexOf(check) != -1) {
                    num = 24;
                    break;
                }
            }
            return num;
        },

        /** 
         * @private
         */
        _getNumberOfDataCodeWord: function () {
            var countOfDataCodeWord = 0;

            switch (this.m_Version) {
                case 01:
                    countOfDataCodeWord = 16;
                    break;
                case 02:
                    countOfDataCodeWord = 28;
                    break;
                case 03:
                    countOfDataCodeWord = 44;
                    break;
                case 04:
                    countOfDataCodeWord = 64;
                    break;
                case 05:
                    countOfDataCodeWord = 86;
                    break;
                case 06:
                    countOfDataCodeWord = 108;
                    break;
                case 07:
                    countOfDataCodeWord = 124;
                    break;
                case 08:
                    countOfDataCodeWord = 154;
                    break;
                case 09:
                    countOfDataCodeWord = 182;
                    break;
                case 10:
                    countOfDataCodeWord = 216;
                    break;
                case 11:
                    countOfDataCodeWord = 254;
                    break;
                case 12:
                    countOfDataCodeWord = 290;
                    break;
                case 13:
                    countOfDataCodeWord = 334;
                    break;
                case 14:
                    countOfDataCodeWord = 365;
                    break;
                case 15:
                    countOfDataCodeWord = 415;
                    break;
                case 16:
                    countOfDataCodeWord = 453;
                    break;
                case 17:
                    countOfDataCodeWord = 507;
                    break;
                case 18:
                    countOfDataCodeWord = 563;
                    break;
                case 19:
                    countOfDataCodeWord = 627;
                    break;
                case 20:
                    countOfDataCodeWord = 669;
                    break;
                case 21:
                    countOfDataCodeWord = 714;
                    break;
                case 22:
                    countOfDataCodeWord = 782;
                    break;
                case 23:
                    countOfDataCodeWord = 860;
                    break;
                case 24:
                    countOfDataCodeWord = 914;
                    break;
                case 25:
                    countOfDataCodeWord = 1000;
                    break;
                case 26:
                    countOfDataCodeWord = 1062;
                    break;
                case 27:
                    countOfDataCodeWord = 1128;
                    break;
                case 28:
                    countOfDataCodeWord = 1193;
                    break;
                case 29:
                    countOfDataCodeWord = 1267;
                    break;
                case 30:
                    countOfDataCodeWord = 1373;
                    break;
                case 31:
                    countOfDataCodeWord = 1455;
                    break;
                case 32:
                    countOfDataCodeWord = 1541;
                    break;
                case 33:
                    countOfDataCodeWord = 1631;
                    break;
                case 34:
                    countOfDataCodeWord = 1725;
                    break;
                case 35:
                    countOfDataCodeWord = 1812;
                    break;
                case 36:
                    countOfDataCodeWord = 1914;
                    break;
                case 37:
                    countOfDataCodeWord = 1992;
                    break;
                case 38:
                    countOfDataCodeWord = 2102;
                    break;
                case 39:
                    countOfDataCodeWord = 2216;
                    break;
                case 40:
                    countOfDataCodeWord = 2334;
                    break;
            }

            return countOfDataCodeWord;
        },

        /** 
         * Returns the number of error correcting codewords based on QR version.
         * @private
         */
        _getNumberOfErrorCorrectingCodeWords: function () {
            var index = (this.m_Version - 1) * 4;
            index += 1;
            var numberOfErrorCorrectingCodeWords = new Array(7, 10, 13, 17, 10, 16, 22, 28, 15, 26, 36, 44, 20, 36, 52, 64, 26, 48, 72, 88, 36, 64, 96, 112, 40, 72, 108, 130, 48, 88, 132, 156, 60, 110, 160, 192, 72, 130, 192, 224, 80, 150, 224, 264, 96, 176, 260, 308, 104, 198, 288, 352, 120, 216, 320, 384, 132, 240, 360, 432, 144, 280, 408, 480, 168, 308, 448, 532, 180, 338, 504, 588, 196, 364, 546, 650, 224, 416, 600, 700, 224, 442, 644, 750, 252, 476, 690, 816, 270, 504, 750, 900, 300, 560, 810, 960, 312, 588, 870, 1050, 336, 644, 952, 1110, 360, 700, 1020, 1200, 390, 728, 1050, 1260, 420, 784, 1140, 1350, 450, 812, 1200, 1440, 480, 868, 1290, 1530, 510, 924, 1350, 1620, 540, 980, 1440, 1710, 570, 1036, 1530, 1800, 570, 1064, 1590, 1890, 600, 1120, 1680, 1980, 630, 1204, 1770, 2100, 660, 1260, 1860, 2220, 720, 1316, 1950, 2310, 750, 1372, 2040, 2430);
            return numberOfErrorCorrectingCodeWords[index];
        },

        /** 
         * Returns the number of error correction blocks based on QR version.
         * @private
         */
        _getNumberOfErrorCorrectionBlocks: function () {
            var numberOfErrorCorrectionBlocks = new Array();
            switch (this.m_Version) {
                case 01:
                case 02:
                case 03:
                    numberOfErrorCorrectionBlocks[0] = 1;
                    break;
                case 04:
                case 05:
                    numberOfErrorCorrectionBlocks[0] = 2;
                    break;
                case 06:
                case 07:
                    numberOfErrorCorrectionBlocks[0] = 4;
                    break;
                case 08:
                    numberOfErrorCorrectionBlocks = new Array(2, 60, 38, 2, 61, 39);
                    break;
                case 09:
                    numberOfErrorCorrectionBlocks = new Array(3, 58, 36, 2, 59, 37);
                    break;
                case 10:
                    numberOfErrorCorrectionBlocks = new Array(4, 69, 43, 1, 70, 44);
                    break;
                case 11:
                    numberOfErrorCorrectionBlocks = new Array(1, 80, 50, 4, 81, 51);
                    break;
                case 12:
                    numberOfErrorCorrectionBlocks = new Array(6, 58, 36, 2, 59, 37);
                    break;
                case 13:
                    numberOfErrorCorrectionBlocks = new Array(8, 59, 37, 1, 60, 38);
                    break;
                case 14:
                    numberOfErrorCorrectionBlocks = new Array(4, 64, 40, 5, 65, 41);
                    break;
                case 15:
                    numberOfErrorCorrectionBlocks = new Array(5, 65, 41, 5, 66, 42);
                    break;
                case 16:
                    numberOfErrorCorrectionBlocks = new Array(7, 73, 45, 3, 74, 46);
                    break;
                case 17:
                    numberOfErrorCorrectionBlocks = new Array(10, 74, 46, 1, 75, 47);
                    break;
                case 18:
                    numberOfErrorCorrectionBlocks = new Array(9, 69, 43, 4, 70, 44);
                    break;
                case 19:
                    numberOfErrorCorrectionBlocks = new Array(3, 70, 44, 11, 71, 45);
                    break;
                case 20:
                    numberOfErrorCorrectionBlocks = new Array(3, 67, 41, 13, 68, 42);
                    break;
                case 21:
                case 22:
                    numberOfErrorCorrectionBlocks[0] = 17;
                    break;
                case 23:
                    numberOfErrorCorrectionBlocks = new Array(4, 75, 47, 14, 76, 48);
                    break;
                case 24:
                    numberOfErrorCorrectionBlocks = new Array(6, 73, 45, 14, 74, 46);
                    break;
                case 25:
                    numberOfErrorCorrectionBlocks = new Array(8, 75, 47, 13, 76, 48);
                    break;
                case 26:
                    numberOfErrorCorrectionBlocks = new Array(19, 74, 46, 4, 75, 47);
                    break;
                case 27:
                    numberOfErrorCorrectionBlocks = new Array(22, 73, 45, 3, 74, 46);
                    break;
                case 28:
                    numberOfErrorCorrectionBlocks = new Array(3, 73, 45, 23, 74, 46);
                    break;
                case 29:
                    numberOfErrorCorrectionBlocks = new Array(21, 73, 45, 7, 74, 46);
                    break;
                case 30:
                    numberOfErrorCorrectionBlocks = new Array(19, 75, 47, 10, 76, 48);
                    break;
                case 31:
                    numberOfErrorCorrectionBlocks = new Array(2, 74, 46, 29, 75, 47);
                    break;
                case 32:
                    numberOfErrorCorrectionBlocks = new Array(10, 74, 46, 23, 75, 47);
                    break;
                case 33:
                    numberOfErrorCorrectionBlocks = new Array(14, 74, 46, 21, 75, 47);
                    break;
                case 34:
                    numberOfErrorCorrectionBlocks = new Array(14, 74, 46, 23, 75, 47);
                    break;
                case 35:
                    numberOfErrorCorrectionBlocks = new Array(12, 75, 47, 26, 76, 48);
                    break;
                case 36:
                    numberOfErrorCorrectionBlocks = new Array(6, 75, 47, 34, 76, 48);
                    break;
                case 37:
                    numberOfErrorCorrectionBlocks = new Array(29, 74, 46, 14, 75, 47);
                    break;
                case 38:
                    numberOfErrorCorrectionBlocks = new Array(13, 74, 46, 32, 75, 47);
                    break;
                case 39:
                    numberOfErrorCorrectionBlocks = new Array(40, 75, 47, 7, 76, 48);
                    break;
                case 40:
                    numberOfErrorCorrectionBlocks = new Array(18, 75, 47, 31, 76, 48);
                    break;
            }

            return numberOfErrorCorrectionBlocks;
        },

        /** 
         * Returns the end values based on qr version
         * @private
         */
        _getEnd: function () {
            var endValues = new Array(208, 359, 567, 807, 1079, 1383, 1568, 1936, 2336, 2768, 3232, 3728, 4256, 4651, 5243, 5867, 6523, 7211, 7931, 8683, 9252, 10068, 10916, 11796, 12708, 13652, 14628, 15371, 16411, 17483, 18587, 19723, 20891, 22091, 23008, 24272, 25568, 26896, 28256, 29648);
            return endValues[this.m_Version - 1];
        },

        /** 
         * Returns the data capacity based on QR version
         * @private
         */
        _getDataCapacity: function () {
            var dataCapacityValues = new Array(26, 44, 70, 100, 134, 172, 196, 242, 292, 346, 404, 466, 532, 581, 655, 733, 815, 901, 991, 1085, 1156, 1258, 1364, 1474, 1588, 1706, 1828, 1921, 2051, 2185, 2323, 2465, 2611, 2761, 2876, 3034, 3196, 3362, 3532, 3706);
            return dataCapacityValues[this.m_Version - 1];
        },

        /** 
         * @private
         */
        _getFormatInformation: function () {
            var formatInformation = new Array(1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1);
            return formatInformation;
        },

        /** 
         * Returns the QR version
         * @private
         */
        _getVersionInformation: function () {
            var versionInformation = new Array();
            switch (this.m_Version) {
                case 07:
                    versionInformation = new Array(0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0);
                    break;
                case 08:
                    versionInformation = new Array(0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0);
                    break;
                case 09:
                    versionInformation = new Array(1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0);
                    break;
                case 10:
                    versionInformation = new Array(1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0);
                    break;
                case 11:
                    versionInformation = new Array(0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0);
                    break;
                case 12:
                    versionInformation = new Array(0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0);
                    break;
                case 13:
                    versionInformation = new Array(1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0);
                    break;
                case 14:
                    versionInformation = new Array(1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0);
                    break;
                case 15:
                    versionInformation = new Array(0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0);
                    break;
                case 16:
                    versionInformation = new Array(0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0);
                    break;
                case 17:
                    versionInformation = new Array(1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0);
                    break;
                case 18:
                    versionInformation = new Array(1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0);
                    break;
                case 19:
                    versionInformation = new Array(0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0);
                    break;
                case 20:
                    versionInformation = new Array(0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0);
                    break;
                case 21:
                    versionInformation = new Array(1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0);
                    break;
                case 22:
                    versionInformation = new Array(1, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0);
                    break;
                case 23:
                    versionInformation = new Array(0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0);
                    break;
                case 24:
                    versionInformation = new Array(0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0);
                    break;
                case 25:
                    versionInformation = new Array(1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0);
                    break;
                case 26:
                    versionInformation = new Array(1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0);
                    break;
                case 27:
                    versionInformation = new Array(0, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0);
                    break;
                case 28:
                    versionInformation = new Array(0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0);
                    break;
                case 29:
                    versionInformation = new Array(1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0);
                    break;
                case 30:
                    versionInformation = new Array(1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0);
                    break;
                case 31:
                    versionInformation = new Array(0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0);
                    break;
                case 32:
                    versionInformation = new Array(1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1);
                    break;
                case 33:
                    versionInformation = new Array(0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1);
                    break;
                case 34:
                    versionInformation = new Array(0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1);
                    break;
                case 35:
                    versionInformation = new Array(1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1);
                    break;
                case 36:
                    versionInformation = new Array(1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0, 1);
                    break;
                case 37:
                    versionInformation = new Array(0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1);
                    break;
                case 38:
                    versionInformation = new Array(0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1);
                    break;
                case 39:
                    versionInformation = new Array(1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1);
                    break;
                case 40:
                    versionInformation = new Array(1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1);
                    break;
            }

            return versionInformation;
        },

        /** 
         * Returns the code points of the text
         * @private
         */
        _getStringCodePoints: (function () {
            function surrogatePairToCodePoint(charCode1, charCode2) {
                return ((charCode1 & 0x3FF) << 10) + (charCode2 & 0x3FF) + 0x10000;
            }

            /** Read string in character by character and create an array of code points.*/
            return function (str) {
                var codePoints = [], i = 0, charCode;
                while (i < str.length) {
                    charCode = str.charCodeAt(i);
                    if ((charCode & 0xF800) == 0xD800)
                        codePoints.push(surrogatePairToCodePoint(charCode, str.charCodeAt(++i)));
                    else
                        codePoints.push(charCode);
                    ++i;
                }
                return codePoints;
            }
        })(),

        /** 
         * Creates 2 dimensional array.
         * @private
         */
        _create2DArray: function (rows, columns) {
            rows = parseInt(rows);
            columns = parseInt(columns);
            var twoDArray = new Array(rows);

            for (var i = 0; i < rows; i++)
                twoDArray[i] = new Array(columns);

            return twoDArray;
        },

        /** 
         * Builds datamatrix barcode.
         * @private
         */
        _buildDataMatrix: function () {
            var symbolAttributes = [[10, 10, 1, 1, 3, 5, 1, 3], [12, 12, 1, 1, 5, 7, 1, 5], [14, 14, 1, 1, 8, 10, 1, 8], [16, 16, 1, 1, 12, 12, 1, 12], [18, 18, 1, 1, 18, 14, 1, 18], [20, 20, 1, 1, 22, 18, 1, 22], [22, 22, 1, 1, 30, 20, 1, 30], [24, 24, 1, 1, 36, 24, 1, 36], [26, 26, 1, 1, 44, 28, 1, 44], [32, 32, 2, 2, 62, 36, 1, 62], [36, 36, 2, 2, 86, 42, 1, 86], [40, 40, 2, 2, 114, 48, 1, 114], [44, 44, 2, 2, 144, 56, 1, 144], [48, 48, 2, 2, 174, 68, 1, 174], [52, 52, 2, 2, 204, 84, 2, 102], [64, 64, 4, 4, 280, 112, 2, 140], [72, 72, 4, 4, 368, 144, 4, 92], [80, 80, 4, 4, 456, 192, 4, 114], [88, 88, 4, 4, 576, 224, 4, 144], [96, 96, 4, 4, 696, 272, 4, 174], [104, 104, 4, 4, 816, 336, 6, 136], [120, 120, 6, 6, 1050, 408, 6, 175], [132, 132, 6, 6, 1304, 496, 8, 163], [144, 144, 6, 6, 1558, 620, 10, 156], [8, 18, 1, 1, 5, 7, 1, 5], [8, 32, 2, 1, 10, 11, 1, 10], [12, 26, 1, 1, 16, 14, 1, 16], [12, 36, 2, 1, 22, 18, 1, 22], [16, 36, 2, 1, 32, 24, 1, 32], [16, 48, 2, 1, 49, 28, 1, 49]];
            var codeWords = this._getStringCodePoints(this.Model.text);
            var correctionCodewords;
            var symAttr;

            /** ASCII Encoder.*/
            var encodedPoints = new Array(); var n = 0;
            for (var e = 0; e < codeWords.length; e++) {
                if (codeWords[e] < 127)
                    encodedPoints[n] = codeWords[e] + 1;
                else {
                    encodedPoints[n++] = 235;
                    encodedPoints[n] = codeWords[e] - 127;
                }
                n++;
            }

            /** Find symbol attribute.*/
            for (var e = 0; e < symbolAttributes.length; e++) {
                if (symbolAttributes[e][4] >= codeWords.length) {
                    symAttr = symbolAttributes[e];
                    break;
                }
            }

            if (symAttr[4] > encodedPoints.length) { //pad codewords
                var l = encodedPoints.length;
                var temp = new Array(symAttr[4]);
                var t = 0;

                for (var i = 0; i < l; i++)
                    temp[i] = encodedPoints[i];

                if (l < (symAttr[4] + 1))
                    temp[l] = 129; // denotes padding.

                l++;
                while (l < symAttr[4]) {	// more padding
                    var v = 129 + (((l + 1) * 149) % 253) + 1;
                    if (v > 254)
                        v -= 254;
                    temp[l++] = v;
                }

                encodedPoints = temp;
            }

            correctionCodewords = symAttr[5];
            errCorrArray = this._errorCorrection(correctionCodewords, encodedPoints);

            var finalCodeWord = encodedPoints.concat(errCorrArray);
            var dataMatrixArray = this._createMatrix(finalCodeWord, symAttr);

            var w = dataMatrixArray.length * this.Model.xDimension;
            var h = dataMatrixArray[0].length * this.Model.xDimension;

            /** Prepare the canvas.*/
            this.Model.width = w;
            this.canvasEl[0].setAttribute("width", this.Model.width);
            this.BarcodeEl.css("width", this.Model.width);

            this.contextEl.beginPath();
            this.contextEl.fillStyle = 'white';
            this.contextEl.fillRect(0, 0, w, h);
            this.contextEl.closePath();

            /** Draw DataMatrix.*/
            this._drawDataMatrix(dataMatrixArray, w, h);
        },

        /** 
         * Draws datamatrix barcode.
         * @private
         */
        _drawDataMatrix: function (dmArray, w, h) {
            this.Model.height = h;
            this.canvasEl[0].setAttribute("height", this.Model.height);
            this.BarcodeEl.css("height", this.Model.height);
            var dim = parseInt(this.Model.xDimension);
            var y = 0;

            this.contextEl.beginPath();

            for (var i = 0; i < dmArray.length; i++) {
                var x = 0;
                for (var j = 0; j < dmArray[i].length; j++) {
                    var colorBrush = 'white';
                    if (dmArray[i][j] == 1)
                        colorBrush = 'black';
                    this.contextEl.fillStyle = colorBrush;
                    this.contextEl.fillRect(x, y, dim, dim)
                    x += dim;
                }
                y += dim;
            }

            this.contextEl.closePath();
            this.contextEl.stroke();
        },

        /** 
         * Creates matrix.
         * @private
         */
        _createMatrix: function (codeWord, symAttr) {
            var x, y, NC, NR;
            var places = new Array();

            var W = symAttr[1], H = symAttr[0];
            var FW = W / symAttr[2], FH = H / symAttr[3];

            NC = W - 2 * (W / FW);
            NR = H - 2 * (H / FH);

            var places = new Array(NC * NR);
            this._ecc200placement(places, NR, NC);
            var matrix = new Array(W * H);

            for (var i = 0; i < matrix.length; i++)
                matrix[i] = 0;

            for (y = 0; y < H; y += FH) {
                for (x = 0; x < W; x++)
                    matrix[parseInt(y * W) + x] = 1;
                for (x = 0; x < W; x += 2)
                    matrix[parseInt((y + FH - 1) * W) + x] = 1;
            }
            for (x = 0; x < W; x += FW) {
                for (y = 0; y < H; y++)
                    matrix[parseInt(y * W) + x] = 1;
                for (y = 0; y < H; y += 2)
                    matrix[parseInt(y * W) + x + FW - 1] = 1;
            }
            for (y = 0; y < NR; y++) {
                for (x = 0; x < NC; x++) {
                    var v = places[((NR - y - 1) * NC) + x];
                    if (v == 1 || v > 7 && (codeWord[(v >> 3) - 1] & (1 << (v & 7))) != 0) {
                        var inde = parseInt((parseInt(1 + y + (2 * parseInt(y / (FH - 2)))) * W) + 1 + x + +(2 * parseInt(x / (FW - 2))));
                        matrix[inde] = 1;
                    }
                }
            }

            var _w = symAttr[1], _h = symAttr[0];

            var tempArray = this._create2DArray(_w, _h);

            for (var x1 = 0; x1 < _w; x1++) {
                for (var y1 = 0; y1 < _h; y1++) {
                    tempArray[x1][y1] = matrix[(_w * y1) + x1];
                }
            }

            /** Rotate the array left.*/
            var tempArray2 = this._create2DArray(_h, _w);

            for (var i = 0; i < _h; i++) {
                for (var j = 0; j < _w; j++) {
                    tempArray2[_h - 1 - i][j] = tempArray[j][i];
                }
            }

            var w = symAttr[0] + (this.Model.quietZone.top + this.Model.quietZone.bottom);
            var h = symAttr[1] + (this.Model.quietZone.left + this.Model.quietZone.right);
            var quietZone = this.Model.quietZone.all;

            var dataMatrixArray = this._create2DArray(w, h);

            for (var i = 0; i < w; i++)
                for (var j = 0; j < h; j++)
                    dataMatrixArray[i][j] = 0;

            for (var i = quietZone; i < w - quietZone; i++) {
                for (var j = quietZone; j < h - quietZone; j++) {
                    dataMatrixArray[i][j] = tempArray2[i - quietZone][j - quietZone];
                }
            }

            return dataMatrixArray;
        },

        /** 
         * @private
         */
        _ecc200placement: function (array, NR, NC) {
            var r, c, p;
            // invalidate
            for (r = 0; r < NR; r++)
                for (c = 0; c < NC; c++)
                    array[r * NC + c] = 0;
            // start
            p = 1;
            r = 4;
            c = 0;
            do {
                // check corner
                if (r == NR && !(c != 0))
                    this._ecc200placementcornerA(array, NR, NC, p++);
                if ((r == NR - 2) && !(c != 0) && ((NC % 4) != 0))
                    this._ecc200placementcornerB(array, NR, NC, p++);
                if (r == NR - 2 && !(c != 0) && (NC % 8) == 4)
                    this._ecc200placementcornerC(array, NR, NC, p++);
                if (r == NR + 4 && c == 2 && !((NC % 8) != 0))
                    this._ecc200placementcornerD(array, NR, NC, p++);
                // up/right
                do {
                    if (r < NR && c >= 0 && !(array[r * NC + c] != 0)) {
                        this._ecc200placementblock(array, NR, NC, r, c, p++);
                    }
                    r -= 2;
                    c += 2;
                }
                while (r >= 0 && c < NC);
                r++;
                c += 3;
                // down/left
                do {
                    if (r > -1) {
                        if (c < NC && !(array[r * NC + c] != 0)) {
                            this._ecc200placementblock(array, NR, NC, r, c, p++);
                        }
                    }
                    r += 2;
                    c -= 2;
                }
                while (r < NR && c >= 0);
                r += 3;
                c++;
            }
            while (r < NR || c < NC);
            // unfilled corner
            if (!(array[NR * NC - 1] != 0))
                array[NR * NC - 1] = array[NR * NC - NC - 2] = 1;
        },

        /** 
         * @private
         */
        _ecc200placementcornerA: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 1, 1, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 2, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 2);
            this._ecc200placementbit(array, NR, NC, 2, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 3, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerB: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 3, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 2, 0, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 4, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 3, p, 3);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 2);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerC: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 3, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 2, 0, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 2);
            this._ecc200placementbit(array, NR, NC, 2, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 3, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerD: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 1, NC - 1, p, 6);
            this._ecc200placementbit(array, NR, NC, 0, NC - 3, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 3, p, 2);
            this._ecc200placementbit(array, NR, NC, 1, NC - 2, p, 1);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementblock: function (array, NR, NC, r, c, p) {
            this._ecc200placementbit(array, NR, NC, r - 2, c - 2, p, 7);
            this._ecc200placementbit(array, NR, NC, r - 2, c - 1, p, 6);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 2, p, 5);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 1, p, 4);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 0, p, 3);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 2, p, 2);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 0, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementbit: function (array, NR, NC, r, c, p, b) {
            if (r < 0) {
                r += NR;
                c += 4 - ((NR + 4) % 8);
            }
            if (c < 0) {
                c += NC;
                r += 4 - ((NC + 4) % 8);
            }
            array[r * NC + c] = (p << 3) + b;
        },

        /** 
         * Adds error correction codeword.
         * @private
         */
        _errorCorrection: function (num, codeWords) {
            var i, k, index = 1;
            var rsPolynomial;
            var temp = new Array(num + 1);

            /** Create log array.*/
            {
                var m_log = new Array(256);
                var m_aLog = new Array(256);

                m_log[0] = -255;
                m_aLog[0] = 1;

                for (var i = 1; i <= 255; i++) {
                    m_aLog[i] = m_aLog[i - 1] * 2;

                    if (m_aLog[i] >= 256)
                        m_aLog[i] = m_aLog[i] ^ 301;

                    m_log[m_aLog[i]] = i;
                }
            }

            /** CreateRSPolynomial.*/
            temp[0] = 1;
            for (i = 1; i <= num; i++) {
                temp[i] = 1;
                for (k = i - 1; k > 0; k--) {
                    if (temp[k] != 0)
                        temp[k] = m_aLog[(m_log[temp[k]] + index) % 255];
                    temp[k] ^= temp[k - 1];
                }
                temp[0] = m_aLog[(m_log[temp[0]] + index) % 255];
                index++;
            }

            rsPolynomial = new Array(num);
            for (var e = 0; e < rsPolynomial.length; e++)
                rsPolynomial[e] = temp[e];

            var t = 0;

            /** Create error correction array.*/
            var errCorrArray = new Array(num);

            for (var i = 0; i < num; i++)
                errCorrArray[i] = 0;

            for (var i = 0; i < codeWords.length; i++) {
                t = errCorrArray[num - 1] ^ codeWords[i];
                for (var j = num - 1; j > 0; j--) {
                    if (t != 0 && rsPolynomial[j] != 0)
                        errCorrArray[j] = (errCorrArray[j - 1] ^ m_aLog[(m_log[t] + m_log[rsPolynomial[j]]) % 255]);
                    else
                        errCorrArray[j] = errCorrArray[j - 1];
                }
                if (t != 0 && rsPolynomial[0] != 0)
                    errCorrArray[0] = (m_aLog[(m_log[t] + m_log[rsPolynomial[0]]) % 255]);
                else
                    errCorrArray[0] = 0;
            }

            errCorrArray = errCorrArray.reverse();
            return errCorrArray;
        },

        /** 
         * Validates the text against the barcode type.
         * @private
         */
        _validateText: function (str, exp) {
            var patt = new RegExp(exp);
            return patt.test(str);
        },

        /** 
         * @private
         */
        _getCharWidth: function (encodedText, symbolTable) {
            var charW = 0;
            var charArray = encodedText.split("");
            for (var e = 0; e < charArray.length; e++)
                charW = charW + intercharacterGap + this._getWidth(symbolTable, charArray[e]);

            return charW;
        },

        /** 
         * @private
         */
        _getWidth: function (symbolTable, c) {
            var check = false;
            var wid = 0;
            for (var e = 0; e < symbolTable.length; e++) {
                if (symbolTable[e][0] == c) {
                    var bars = symbolTable[e][2];
                    for (var i = 0; i < bars.length; i++) {
                        if (check == false) {
                            if (bars.length % 2 != 0)
                                continuous = true;
                        }

                        wid = wid + (bars[i] * this.Model.narrowBarWidth);
                        check = true;
                    }
                    return wid;
                }
            }
            return wid;
        },

        /** 
         * Draws 1 dimensional barcode.
         * @private
         */
        _draw1DBarcode: function (encodedText, symbolTable) {
            if (symbolTable.length > 0) {
                var charArray = encodedText.split("");
                var left = this.Model.quietZone.left;
                var top = this.Model.quietZone.top;
                this.Model.height = top + this.Model.barHeight + this.Model.quietZone.bottom;

                var textSize = parseInt(this.textFont.size.replace('px', ''));
                if (this.Model.displayText) {
                    this.Model.height += (textSize + this.Model.barcodeToTextGapHeight);
                }
                this.canvasEl[0].setAttribute("height", this.Model.height);
                this.BarcodeEl.css("height", this.Model.height);
                var wid = this.Model.width;
                var hei = this.Model.height;
                this.contextEl.beginPath();

                this.contextEl.fillStyle = 'white';
                this.contextEl.fillRect(0, 0, wid, hei);

                for (var e = 0; e < charArray.length; e++) {
                    for (var h = 0; h < symbolTable.length; h++) {
                        if (symbolTable[h][0] == charArray[e]) {
                            var bars = symbolTable[h][2];
                            for (var i = 0; i < bars.length; i++) {
                                var barWidth = bars[i] * this.Model.narrowBarWidth;
                                if (i % 2 == 0) {
                                    this.contextEl.fillStyle = this.Model.darkBarColor;
                                    this.contextEl.fillRect(left, top, barWidth, this.Model.barHeight)
                                }
                                left += barWidth;
                            }
                            if (bars.length % 2 != 0)
                                left += intercharacterGap;
                        }
                    }
                }

                this.contextEl.closePath();
                this.contextEl.stroke();

                this.Model.width = left += (this.Model.quietZone.right);

                /** Draws text for 1D barcodes.*/
                if (this.Model.displayText) {
                    var txtF = (this.textFont.fontStyle.toLowerCase() == "regular") ? "" : +this.textFont.fontStyle.toLowerCase() + " ";
                    txtF += this.textFont.size + " " + this.textFont.fontFamily.toLowerCase() + ",sans-serif";
                    this.contextEl.font = txtF;

                    var textWidth = 0;
                    textWidth = this.contextEl.measureText(this.Model.text).width;
                    if (this.Model.width < textWidth) {
                        this.Model.width = textWidth;
                        this.canvasEl[0].setAttribute("width", this.Model.width);
                        this.BarcodeEl.css("width", this.Model.width);
                    }

                    this.contextEl.beginPath();
                    this.contextEl.fillStyle = this.Model.textColor;
                    this.contextEl.textAlign = "center";
                    hei = hei - textSize + textSize / 2;
                    this.contextEl.fillText(this.Model.text, (this.Model.width / 2), hei);
                    this.contextEl.closePath();
                    this.contextEl.stroke();
                }
            }
        }
    });

    /**
     * Enum for Symbol type.
	 * @enum {string}
     * @global
     */
    ej.Barcode.symbologyType = {
        /** Specifies Code 39 Barcode. */
        code39: "code39",
        /** Specifies Code 39 Extended Barcode. */
        code39Extended: "code39extended",
        /** Specifies Code 11 Barcode. */
        code11: "code11",
        /** Specifies Codabar Barcode. */
        codabar: "codabar",
        /** Specifies Code 32 Barcode. */
        code32: "code32",
        /** Specifies Code 93 Barcode. */
        code93: "code93",
        /** Specifies Code 93 Extended Barcode. */
        code93Extended: "code93extended",
        /** Specifies Code 128 A Barcode. */
        code128A: "code128a",
        /** Specifies Code 128 B Barcode. */
        code128B: "code128b",
        /** Specifies Code 128 C Barcode. */
        code128C: "code128c",
        /** Specifies DataMatrix Barcode. */
        dataMatrix: "datamatrix",
        /** Specifies QR Barcode. */
        qrBarcode: "qrbarcode"
    };

    /**
     * Enum for Symbol type.
     * @readonly
     * @private
     */
    ej.Barcode.symbologySettings = {
        /** Specifies symbology settinngs for the Code 39 Barcode. */
        code39: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\.\$\/\+\%\ ]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["A", 10, [3, 1, 1, 1, 1, 3, 1, 1, 3]], ["B", 11, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 12, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 13, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["E", 14, [3, 1, 1, 1, 3, 3, 1, 1, 1]], ["F", 15, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 16, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 17, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["I", 18, [1, 1, 3, 1, 1, 3, 3, 1, 1]], ["J", 19, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 20, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 21, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 22, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 23, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["O", 24, [3, 1, 1, 1, 3, 1, 1, 3, 1]], ["P", 25, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 26, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 27, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 28, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 29, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 30, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 31, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 32, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 33, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 34, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 35, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["-", 36, [1, 3, 1, 1, 1, 1, 3, 1, 3]], [".", 37, [3, 3, 1, 1, 1, 1, 3, 1, 1]], [" ", 38, [1, 3, 3, 1, 1, 1, 3, 1, 1]], ["$", 39, [1, 3, 1, 3, 1, 3, 1, 1, 1]], ["/", 40, [1, 3, 1, 3, 1, 1, 1, 3, 1]], ["+", 41, [1, 3, 1, 1, 1, 3, 1, 3, 1]], ["%", 42, [1, 1, 1, 3, 1, 3, 1, 3, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 39 Extended Barcode. */
        code39extended: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x00-\x7F]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["A", 10, [3, 1, 1, 1, 1, 3, 1, 1, 3]], ["B", 11, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 12, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 13, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["E", 14, [3, 1, 1, 1, 3, 3, 1, 1, 1]], ["F", 15, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 16, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 17, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["I", 18, [1, 1, 3, 1, 1, 3, 3, 1, 1]], ["J", 19, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 20, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 21, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 22, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 23, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["O", 24, [3, 1, 1, 1, 3, 1, 1, 3, 1]], ["P", 25, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 26, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 27, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 28, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 29, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 30, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 31, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 32, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 33, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 34, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 35, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["-", 36, [1, 3, 1, 1, 1, 1, 3, 1, 3]], [".", 37, [3, 3, 1, 1, 1, 1, 3, 1, 1]], [" ", 38, [1, 3, 3, 1, 1, 1, 3, 1, 1]], ["$", 39, [1, 3, 1, 3, 1, 3, 1, 1, 1]], ["/", 40, [1, 3, 1, 3, 1, 1, 1, 3, 1]], ["+", 41, [1, 3, 1, 1, 1, 3, 1, 3, 1]], ["%", 42, [1, 1, 1, 3, 1, 3, 1, 3, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]],
            extendedCodes: [['\0', ['%', 'U']], [String.fromCharCode(parseInt('0001', 16)), ['$', 'A']], [String.fromCharCode(parseInt('0002', 16)), ['$', 'B']], [String.fromCharCode(parseInt('0003', 16)), ['$', 'C']], [String.fromCharCode(parseInt('0004', 16)), ['$', 'D']], [String.fromCharCode(parseInt('0005', 16)), ['$', 'E']], [String.fromCharCode(parseInt('0006', 16)), ['$', 'F']], ['\a', ['$', 'G']], ['\b', ['$', 'H']], ['\t', ['$', 'I']], ['\n', ['$', 'J']], ['\v', ['$', 'K']], ['\f', ['$', 'L']], ['\r', ['$', 'M']], [String.fromCharCode(parseInt('000e', 16)), ['$', 'N']], [String.fromCharCode(parseInt('000f', 16)), ['$', 'O']], [String.fromCharCode(parseInt('0010', 16)), ['$', 'P']], [String.fromCharCode(parseInt('0011', 16)), ['$', 'Q']], [String.fromCharCode(parseInt('0012', 16)), ['$', 'R']], [String.fromCharCode(parseInt('0013', 16)), ['$', 'S']], [String.fromCharCode(parseInt('0014', 16)), ['$', 'T']], [String.fromCharCode(parseInt('0015', 16)), ['$', 'U']], [String.fromCharCode(parseInt('0016', 16)), ['$', 'V']], [String.fromCharCode(parseInt('0017', 16)), ['$', 'W']], [String.fromCharCode(parseInt('0018', 16)), ['$', 'X']], [String.fromCharCode(parseInt('0019', 16)), ['$', 'Y']], [String.fromCharCode(parseInt('001a', 16)), ['$', 'Z']], [String.fromCharCode(parseInt('001b', 16)), ['%', 'A']], [String.fromCharCode(parseInt('001c', 16)), ['%', 'B']], [String.fromCharCode(parseInt('001d', 16)), ['%', 'C']], [String.fromCharCode(parseInt('001e', 16)), ['%', 'D']], [String.fromCharCode(parseInt('001f', 16)), ['%', 'E']], [' ', [' ']], ['!', ['/', 'A']], ['"', ['/', 'B']], ['#', ['/', 'C']], ['$', ['/', 'D']], ['%', ['/', 'E']], ['&', ['/', 'F']], ['\'', ['/', 'G']], ['(', ['/', 'H']], [')', ['/', 'I']], ['*', ['/', 'J']], ['+', ['/', 'K']], [',', ['/', 'L']], ['-', ['/', 'M']], ['.', ['/', 'N']], ['/', ['/', 'O']], ['0', ['0']], ['1', ['1']], ['2', ['2']], ['3', ['3']], ['4', ['4']], ['5', ['5']], ['6', ['6']], ['7', ['7']], ['8', ['8']], ['9', ['9']], [':', ['/', 'Z']], [';', ['%', 'F']], ['<', ['%', 'G']], ['=', ['%', 'H']], ['>', ['%', 'I']], ['?', ['%', 'J']], ['@', ['%', 'V']], ['A', ['A']], ['B', ['B']], ['C', ['C']], ['D', ['D']], ['E', ['E']], ['F', ['F']], ['G', ['G']], ['H', ['H']], ['I', ['I']], ['J', ['J']], ['K', ['K']], ['L', ['L']], ['M', ['M']], ['N', ['N']], ['O', ['O']], ['P', ['P']], ['Q', ['Q']], ['R', ['R']], ['S', ['S']], ['T', ['T']], ['U', ['U']], ['V', ['V']], ['W', ['W']], ['X', ['X']], ['Y', ['Y']], ['Z', ['Z']], ['[', ['%', 'K']], ['\\', ['%', 'L']], [']', ['%', 'M']], ['^', ['%', 'N']], ['_', ['%', 'O']], ['`', ['%', 'W']], ['a', ['+', 'A']], ['b', ['+', 'B']], ['c', ['+', 'C']], ['d', ['+', 'D']], ['e', ['+', 'E']], ['f', ['+', 'F']], ['g', ['+', 'G']], ['h', ['+', 'H']], ['i', ['+', 'I']], ['j', ['+', 'J']], ['k', ['+', 'K']], ['l', ['+', 'L']], ['m', ['+', 'M']], ['n', ['+', 'N']], ['o', ['+', 'O']], ['p', ['+', 'P']], ['q', ['+', 'Q']], ['r', ['+', 'R']], ['s', ['+', 'S']], ['t', ['+', 'T']], ['u', ['+', 'U']], ['v', ['+', 'V']], ['w', ['+', 'W']], ['x', ['+', 'X']], ['y', ['+', 'Y']], ['z', ['+', 'Z']], ['{', ['%', 'P']], ['|', ['%', 'Q']], ['}', ['%', 'R']], ['~', ['%', 'S']], [String.fromCharCode(parseInt('007f', 16)), ['%', 'T']]]
        },
        /** Specifies symbology settings for the Code 11 Barcode. */
        code11: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[0-9\-]*$",
            symbolTable: [["0", 0, [1, 1, 1, 1, 2]], ["1", 1, [2, 1, 1, 1, 2]], ["2", 2, [1, 2, 1, 1, 2]], ["3", 3, [2, 2, 1, 1, 1]], ["4", 4, [1, 1, 2, 1, 2]], ["5", 5, [2, 1, 2, 1, 1]], ["6", 6, [1, 2, 2, 1, 1]], ["7", 7, [1, 1, 1, 2, 2]], ["8", 8, [2, 1, 1, 2, 1]], ["9", 9, [2, 1, 1, 1, 1]], ["-", 10, [1, 1, 2, 1, 1]], ["*", 0, [1, 1, 2, 2, 1]]]
        },
        /** Specifies symbology settings for the Codabar Barcode. */
        codabar: {
            startSymbol: 'A',
            stopSymbol: 'B',
            validatorExp: "^[0-9\-\.\$\/\:\+]*$",
            symbolTable: [["0", 0, [1, 1, 1, 1, 1, 2, 2]], ["1", 0, [1, 1, 1, 1, 2, 2, 1]], ["2", 0, [1, 1, 1, 2, 1, 1, 2]], ["3", 0, [2, 2, 1, 1, 1, 1, 1]], ["4", 0, [1, 1, 2, 1, 1, 2, 1]], ["5", 0, [2, 1, 1, 1, 1, 2, 1]], ["6", 0, [1, 2, 1, 1, 1, 1, 2]], ["7", 0, [1, 2, 1, 1, 2, 1, 1]], ["8", 0, [1, 2, 2, 1, 1, 1, 1]], ["9", 0, [2, 1, 1, 2, 1, 1, 1]], ["-", 0, [1, 1, 1, 2, 2, 1, 1]], ["$", 0, [1, 1, 2, 2, 1, 1, 1]], [":", 0, [2, 1, 1, 1, 2, 1, 2]], ["/", 0, [2, 1, 2, 1, 1, 1, 2]], [".", 0, [2, 1, 2, 1, 2, 1, 1]], ["+", 0, [1, 1, 2, 1, 2, 1, 2]], ["A", 0, [1, 1, 2, 2, 1, 2, 1]], ["B", 0, [1, 1, 1, 2, 1, 2, 2]]]
        },
        /** Specifies symbology settings for the Code 32 Barcode. */
        code32: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\*\.\$\/\+\%]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["B", 10, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 11, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 12, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["F", 13, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 14, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 15, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["J", 0x10, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 0x11, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 0x12, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 0x13, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 20, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["P", 0x15, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 0x16, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 0x17, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 0x18, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 0x19, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 0x1a, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 0x1b, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 0x1c, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 0x1d, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 30, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 0x1f, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 93 Barcode. */
        code93: {
            startSymbol: '*',
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\.\$\/\+\%\ ]+$",
            symbolTable: [["0", 0, [1, 3, 1, 1, 1, 2]], ["1", 1, [1, 1, 1, 2, 1, 3]], ["2", 2, [1, 1, 1, 3, 1, 2]], ["3", 3, [1, 1, 1, 4, 1, 1]], ["4", 4, [1, 2, 1, 1, 1, 2]], ["5", 5, [1, 2, 1, 2, 1, 2]], ["6", 6, [1, 2, 1, 3, 1, 1]], ["7", 7, [1, 1, 1, 1, 1, 4]], ["8", 8, [1, 3, 1, 2, 1, 1]], ["9", 9, [1, 4, 1, 1, 1, 1]], ["A", 10, [2, 1, 1, 1, 1, 3]], ["B", 11, [2, 1, 1, 2, 1, 2]], ["C", 12, [2, 1, 1, 3, 1, 1]], ["D", 13, [2, 2, 1, 1, 1, 2]], ["E", 14, [2, 2, 1, 2, 1, 1]], ["F", 15, [2, 3, 1, 1, 1, 1]], ["G", 0x10, [1, 1, 2, 1, 1, 3]], ["H", 0x11, [1, 1, 2, 2, 1, 2]], ["I", 0x12, [1, 1, 2, 3, 1, 1]], ["J", 0x13, [1, 2, 2, 1, 1, 2]], ["K", 20, [1, 3, 2, 1, 1, 1]], ["L", 0x15, [1, 1, 1, 1, 2, 3]], ["M", 0x16, [1, 1, 1, 2, 2, 2]], ["N", 0x17, [1, 1, 1, 3, 2, 1]], ["O", 0x18, [1, 2, 1, 1, 2, 2]], ["P", 0x19, [1, 3, 1, 1, 2, 1]], ["Q", 0x1a, [2, 1, 2, 1, 1, 2]], ["R", 0x1b, [2, 1, 2, 2, 1, 1]], ["S", 0x1c, [2, 1, 1, 1, 2, 2]], ["T", 0x1d, [2, 1, 1, 2, 2, 1]], ["U", 30, [2, 2, 1, 1, 2, 1]], ["V", 0x1f, [2, 2, 2, 1, 1, 1]], ["W", 0x20, [1, 1, 2, 1, 2, 2]], ["X", 0x21, [1, 1, 2, 2, 2, 1]], ["Y", 0x22, [1, 2, 2, 1, 2, 1]], ["Z", 0x23, [1, 2, 3, 1, 1, 1]], ["-", 0x24, [1, 2, 1, 1, 3, 1]], [".", 0x25, [3, 1, 1, 1, 1, 2]], [" ", 0x26, [3, 1, 1, 2, 1, 1]], ["$", 0x27, [3, 2, 1, 1, 1, 1]], ["/", 40, [1, 1, 2, 1, 3, 1]], ["+", 0x29, [1, 1, 3, 1, 2, 1]], ["%", 0x2a, [2, 1, 1, 1, 3, 1]], ["*", 0, [1, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00ff', 16)), 0, [1, 1, 1, 1, 4, 1, 1]], [String.fromCharCode(parseInt('00fb', 16)), 0x2b, [1, 2, 1, 2, 2, 0]], [String.fromCharCode(parseInt('00fc', 16)), 0x2c, [3, 1, 2, 1, 1, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x2d, [3, 1, 1, 1, 2, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x2e, [1, 2, 2, 2, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 93 Extended Barcode. */
        code93extended: {
            startSymbol: '*',
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x00-\x7F\x00fb\x00fd\x00fe\'�'\'�'\'�']+$",
            symbolTable: [["0", 0, [1, 3, 1, 1, 1, 2]], ["1", 1, [1, 1, 1, 2, 1, 3]], ["2", 2, [1, 1, 1, 3, 1, 2]], ["3", 3, [1, 1, 1, 4, 1, 1]], ["4", 4, [1, 2, 1, 1, 1, 2]], ["5", 5, [1, 2, 1, 2, 1, 2]], ["6", 6, [1, 2, 1, 3, 1, 1]], ["7", 7, [1, 1, 1, 1, 1, 4]], ["8", 8, [1, 3, 1, 2, 1, 1]], ["9", 9, [1, 4, 1, 1, 1, 1]], ["A", 10, [2, 1, 1, 1, 1, 3]], ["B", 11, [2, 1, 1, 2, 1, 2]], ["C", 12, [2, 1, 1, 3, 1, 1]], ["D", 13, [2, 2, 1, 1, 1, 2]], ["E", 14, [2, 2, 1, 2, 1, 1]], ["F", 15, [2, 3, 1, 1, 1, 1]], ["G", 0x10, [1, 1, 2, 1, 1, 3]], ["H", 0x11, [1, 1, 2, 2, 1, 2]], ["I", 0x12, [1, 1, 2, 3, 1, 1]], ["J", 0x13, [1, 2, 2, 1, 1, 2]], ["K", 20, [1, 3, 2, 1, 1, 1]], ["L", 0x15, [1, 1, 1, 1, 2, 3]], ["M", 0x16, [1, 1, 1, 2, 2, 2]], ["N", 0x17, [1, 1, 1, 3, 2, 1]], ["O", 0x18, [1, 2, 1, 1, 2, 2]], ["P", 0x19, [1, 3, 1, 1, 2, 1]], ["Q", 0x1a, [2, 1, 2, 1, 1, 2]], ["R", 0x1b, [2, 1, 2, 2, 1, 1]], ["S", 0x1c, [2, 1, 1, 1, 2, 2]], ["T", 0x1d, [2, 1, 1, 2, 2, 1]], ["U", 30, [2, 2, 1, 1, 2, 1]], ["V", 0x1f, [2, 2, 2, 1, 1, 1]], ["W", 0x20, [1, 1, 2, 1, 2, 2]], ["X", 0x21, [1, 1, 2, 2, 2, 1]], ["Y", 0x22, [1, 2, 2, 1, 2, 1]], ["Z", 0x23, [1, 2, 3, 1, 1, 1]], ["-", 0x24, [1, 2, 1, 1, 3, 1]], [".", 0x25, [3, 1, 1, 1, 1, 2]], [" ", 0x26, [3, 1, 1, 2, 1, 1]], ["$", 0x27, [3, 2, 1, 1, 1, 1]], ["/", 40, [1, 1, 2, 1, 3, 1]], ["+", 0x29, [1, 1, 3, 1, 2, 1]], ["%", 0x2a, [2, 1, 1, 1, 3, 1]], ["*", 0, [1, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00ff', 16)), 0, [1, 1, 1, 1, 4, 1, 1]], [String.fromCharCode(parseInt('00fb', 16)), 0x2b, [1, 2, 1, 2, 2, 0]], [String.fromCharCode(parseInt('00fc', 16)), 0x2c, [3, 1, 2, 1, 1, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x2d, [3, 1, 1, 1, 2, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x2e, [1, 2, 2, 2, 1, 1]]],
            extendedCodes: [['\0', [String.fromCharCode(parseInt('00fc', 16)), 'U']], [String.fromCharCode(parseInt('0001', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'A']], [String.fromCharCode(parseInt('0002', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'B']], [String.fromCharCode(parseInt('0003', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'C']], [String.fromCharCode(parseInt('0004', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'D']], [String.fromCharCode(parseInt('0005', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'E']], [String.fromCharCode(parseInt('0006', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'F']], ['\a', [String.fromCharCode(parseInt('00fb', 16)), 'G']], ['\b', [String.fromCharCode(parseInt('00fb', 16)), 'H']], ['\t', [String.fromCharCode(parseInt('00fb', 16)), 'I']], ['\n', [String.fromCharCode(parseInt('00fb', 16)), 'J']], ['\v', [String.fromCharCode(parseInt('00fb', 16)), 'K']], ['\f', [String.fromCharCode(parseInt('00fb', 16)), 'L']], ['\r', [String.fromCharCode(parseInt('00fb', 16)), 'M']], [String.fromCharCode(parseInt('000e', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'N']], [String.fromCharCode(parseInt('000f', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'O']], [String.fromCharCode(parseInt('0010', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'P']], [String.fromCharCode(parseInt('0011', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Q']], [String.fromCharCode(parseInt('0012', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'R']], [String.fromCharCode(parseInt('0013', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'S']], [String.fromCharCode(parseInt('0014', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'T']], [String.fromCharCode(parseInt('0015', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'U']], [String.fromCharCode(parseInt('0016', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'V']], [String.fromCharCode(parseInt('0017', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'W']], [String.fromCharCode(parseInt('0018', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'X']], [String.fromCharCode(parseInt('0019', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Y']], [String.fromCharCode(parseInt('001a', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Z']], [String.fromCharCode(parseInt('001b', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'A']], [String.fromCharCode(parseInt('001c', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'B']], [String.fromCharCode(parseInt('001d', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'C']], [String.fromCharCode(parseInt('001e', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'D']], [String.fromCharCode(parseInt('001f', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'E']], [' ', [' ']], ['!', [String.fromCharCode(parseInt('00fd', 16)), 'A']], ['"', [String.fromCharCode(parseInt('00fd', 16)), 'B']], ['#', [String.fromCharCode(parseInt('00fd', 16)), 'C']], ['$', [String.fromCharCode(parseInt('00fd', 16)), 'D']], ['%', [String.fromCharCode(parseInt('00fd', 16)), 'E']], ['&', [String.fromCharCode(parseInt('00fd', 16)), 'F']], ['\'', [String.fromCharCode(parseInt('00fd', 16)), 'G']], ['(', [String.fromCharCode(parseInt('00fd', 16)), 'H']], [')', [String.fromCharCode(parseInt('00fd', 16)), 'I']], ['*', [String.fromCharCode(parseInt('00fd', 16)), 'J']], ['+', [String.fromCharCode(parseInt('00fd', 16)), 'K']], [',', [String.fromCharCode(parseInt('00fd', 16)), 'L']], ['-', [String.fromCharCode(parseInt('00fd', 16)), 'M']], ['.', [String.fromCharCode(parseInt('00fd', 16)), 'N']], ['/', [String.fromCharCode(parseInt('00fd', 16)), 'O']], ['0', ['0']], ['1', ['1']], ['2', ['2']], ['3', ['3']], ['4', ['4']], ['5', ['5']], ['6', ['6']], ['7', ['7']], ['8', ['8']], ['9', ['9']], [':', [String.fromCharCode(parseInt('00fd', 16)), 'Z']], [';', [String.fromCharCode(parseInt('00fc', 16)), 'F']], ['<', [String.fromCharCode(parseInt('00fc', 16)), 'G']], ['=', [String.fromCharCode(parseInt('00fc', 16)), 'H']], ['>', [String.fromCharCode(parseInt('00fc', 16)), 'I']], ['?', [String.fromCharCode(parseInt('00fc', 16)), 'J']], ['@', [String.fromCharCode(parseInt('00fc', 16)), 'V']], ['A', ['A']], ['B', ['B']], ['C', ['C']], ['D', ['D']], ['E', ['E']], ['F', ['F']], ['G', ['G']], ['H', ['H']], ['I', ['I']], ['J', ['J']], ['K', ['K']], ['L', ['L']], ['M', ['M']], ['N', ['N']], ['O', ['O']], ['P', ['P']], ['Q', ['Q']], ['R', ['R']], ['S', ['S']], ['T', ['T']], ['U', ['U']], ['V', ['V']], ['W', ['W']], ['X', ['X']], ['Y', ['Y']], ['Z', ['Z']], ['[', [String.fromCharCode(parseInt('00fc', 16)), 'K']], ['\\', [String.fromCharCode(parseInt('00fc', 16)), 'L']], [']', [String.fromCharCode(parseInt('00fc', 16)), 'M']], ['^', [String.fromCharCode(parseInt('00fc', 16)), 'N']], ['_', [String.fromCharCode(parseInt('00fc', 16)), 'O']], ['`', [String.fromCharCode(parseInt('00fc', 16)), 'W']], ['a', [String.fromCharCode(parseInt('00fe', 16)), 'A']], ['b', [String.fromCharCode(parseInt('00fe', 16)), 'B']], ['c', [String.fromCharCode(parseInt('00fe', 16)), 'C']], ['d', [String.fromCharCode(parseInt('00fe', 16)), 'D']], ['e', [String.fromCharCode(parseInt('00fe', 16)), 'E']], ['f', [String.fromCharCode(parseInt('00fe', 16)), 'F']], ['g', [String.fromCharCode(parseInt('00fe', 16)), 'G']], ['h', [String.fromCharCode(parseInt('00fe', 16)), 'H']], ['i', [String.fromCharCode(parseInt('00fe', 16)), 'I']], ['j', [String.fromCharCode(parseInt('00fe', 16)), 'J']], ['k', [String.fromCharCode(parseInt('00fe', 16)), 'K']], ['l', [String.fromCharCode(parseInt('00fe', 16)), 'L']], ['m', [String.fromCharCode(parseInt('00fe', 16)), 'M']], ['n', [String.fromCharCode(parseInt('00fe', 16)), 'N']], ['o', [String.fromCharCode(parseInt('00fe', 16)), 'O']], ['p', [String.fromCharCode(parseInt('00fe', 16)), 'P']], ['q', [String.fromCharCode(parseInt('00fe', 16)), 'Q']], ['r', [String.fromCharCode(parseInt('00fe', 16)), 'R']], ['s', [String.fromCharCode(parseInt('00fe', 16)), 'S']], ['t', [String.fromCharCode(parseInt('00fe', 16)), 'T']], ['u', [String.fromCharCode(parseInt('00fe', 16)), 'U']], ['v', [String.fromCharCode(parseInt('00fe', 16)), 'V']], ['w', [String.fromCharCode(parseInt('00fe', 16)), 'W']], ['x', [String.fromCharCode(parseInt('00fe', 16)), 'X']], ['y', [String.fromCharCode(parseInt('00fe', 16)), 'Y']], ['z', [String.fromCharCode(parseInt('00fe', 16)), 'Z']], ['{', [String.fromCharCode(parseInt('00fc', 16)), 'P']], ['|', [String.fromCharCode(parseInt('00fc', 16)), 'Q']], ['}', [String.fromCharCode(parseInt('00fc', 16)), 'R']], ['~', [String.fromCharCode(parseInt('00fc', 16)), 'S']], [String.fromCharCode(parseInt('007f', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'T']]],
        },
        /** Specifies symbology settings for the Code 128 A Barcode. */
        code128a: {
            startSymbol: String.fromCharCode(parseInt('00f9', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[\0\x0001\x0002\x0003\x0004\x0005\x0006\a\b\t\n\v\f\r\x000e\x000f\x0010\x0011\x0012\x0013\x0014\x0015\x0016\x0017\x0018\x0019\x001a\x001b\x001c\x001d\x001e\x001f !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_\x00f0\x00f1\x00f2\x00f3\x00f4]",
            symbolTable: [["\0", 0x40, [1, 1, 1, 4, 2, 2]], [String.fromCharCode(parseInt('0001', 16)), 0x41, [1, 2, 1, 1, 2, 4]], [String.fromCharCode(parseInt('0002', 16)), 0x42, [1, 2, 1, 4, 2, 1]], [String.fromCharCode(parseInt('0003', 16)), 0x43, [1, 4, 1, 1, 2, 2]], [String.fromCharCode(parseInt('0004', 16)), 0x44, [1, 4, 1, 2, 2, 1]], [String.fromCharCode(parseInt('0005', 16)), 0x45, [1, 1, 2, 2, 1, 4]], [String.fromCharCode(parseInt('0006', 16)), 70, [1, 1, 2, 4, 1, 2]], ["\a", 0x47, [1, 2, 2, 1, 1, 4]], ["\b", 0x48, [1, 2, 2, 4, 1, 1]], ["\t", 0x49, [1, 4, 2, 1, 1, 2]], ["\n", 0x4a, [1, 4, 2, 2, 1, 1]], ["\v", 0x4b, [2, 4, 1, 2, 1, 1]], ["\f", 0x4c, [2, 2, 1, 1, 1, 4]], ["\r", 0x4d, [4, 1, 3, 1, 1, 1]], [String.fromCharCode(parseInt('000e', 16)), 0x4e, [2, 4, 1, 1, 1, 2]], [String.fromCharCode(parseInt('000f', 16)), 0x4f, [1, 3, 4, 1, 1, 1]], [String.fromCharCode(parseInt('0010', 16)), 80, [1, 1, 1, 2, 4, 2]], [String.fromCharCode(parseInt('0011', 16)), 0x51, [1, 2, 1, 1, 4, 2]], [String.fromCharCode(parseInt('0012', 16)), 0x52, [1, 2, 1, 2, 4, 1]], [String.fromCharCode(parseInt('0013', 16)), 0x53, [1, 1, 4, 2, 1, 2]], [String.fromCharCode(parseInt('0014', 16)), 0x54, [1, 2, 4, 1, 1, 2]], [String.fromCharCode(parseInt('0015', 16)), 0x55, [1, 2, 4, 2, 1, 1]], [String.fromCharCode(parseInt('0016', 16)), 0x56, [4, 1, 1, 2, 1, 2]], [String.fromCharCode(parseInt('0017', 16)), 0x57, [4, 2, 1, 1, 1, 2]], [String.fromCharCode(parseInt('0018', 16)), 0x58, [4, 2, 1, 2, 1, 1]], [String.fromCharCode(parseInt('0019', 16)), 0x59, [2, 1, 2, 1, 4, 1]], [String.fromCharCode(parseInt('001a', 16)), 90, [2, 1, 4, 1, 2, 1]], [String.fromCharCode(parseInt('001b', 16)), 0x5b, [4, 1, 2, 1, 2, 1]], [String.fromCharCode(parseInt('001c', 16)), 0x5c, [1, 1, 1, 1, 4, 3]], [String.fromCharCode(parseInt('001d', 16)), 0x5d, [1, 1, 1, 3, 4, 1]], [String.fromCharCode(parseInt('001e', 16)), 0x5e, [1, 3, 1, 1, 4, 1]], [String.fromCharCode(parseInt('001f', 16)), 0x5f, [1, 1, 4, 1, 1, 3]], [" ", 0, [2, 1, 2, 2, 2, 2]], ["!", 1, [2, 2, 2, 1, 2, 2]], ["\"", 2, [2, 2, 2, 2, 2, 1]], ["#", 3, [1, 2, 1, 2, 2, 3]], ["$", 4, [1, 2, 1, 3, 2, 2]], ["%", 5, [1, 3, 1, 2, 2, 2]], ["&", 6, [1, 2, 2, 2, 1, 3]], ["\'", 7, [1, 2, 2, 3, 1, 2]], ["(", 8, [1, 3, 2, 2, 1, 2]], [")", 9, [2, 2, 1, 2, 1, 3]], ["*", 10, [2, 2, 1, 3, 1, 2]], ["+", 11, [2, 3, 1, 2, 1, 2]], [",", 12, [1, 1, 2, 2, 3, 2]], ["-", 13, [1, 2, 2, 1, 3, 2]], [".", 14, [1, 2, 2, 2, 3, 1]], ["/", 15, [1, 1, 3, 2, 2, 2]], ["0", 0x10, [1, 2, 3, 1, 2, 2]], ["1", 0x11, [1, 2, 3, 2, 2, 1]], ["2", 0x12, [2, 2, 3, 2, 1, 1]], ["3", 0x13, [2, 2, 1, 1, 3, 2]], ["4", 20, [2, 2, 1, 2, 3, 1]], ["5", 0x15, [2, 1, 3, 2, 1, 2]], ["6", 0x16, [2, 2, 3, 1, 1, 2]], ["7", 0x17, [3, 1, 2, 1, 3, 1]], ["8", 0x18, [3, 1, 1, 2, 2, 2]], ["9", 0x19, [3, 2, 1, 1, 2, 2]], [":", 0x1a, [3, 2, 1, 2, 2, 1]], [";", 0x1b, [3, 1, 2, 2, 1, 2]], ["<", 0x1c, [3, 2, 2, 1, 1, 2]], ["=", 0x1d, [3, 2, 2, 2, 1, 1]], [">", 30, [2, 1, 2, 1, 2, 3]], ["?", 0x1f, [2, 1, 2, 3, 2, 1]], ["@", 0x20, [2, 3, 2, 1, 2, 1]], ["A", 0x21, [1, 1, 1, 3, 2, 3]], ["B", 0x22, [1, 3, 1, 1, 2, 3]], ["C", 0x23, [1, 3, 1, 3, 2, 1]], ["D", 0x24, [1, 1, 2, 3, 1, 3]], ["E", 0x25, [1, 3, 2, 1, 1, 3]], ["F", 0x26, [1, 3, 2, 3, 1, 1]], ["G", 0x27, [2, 1, 1, 3, 1, 3]], ["H", 40, [2, 3, 1, 1, 1, 3]], ["I", 0x29, [2, 3, 1, 3, 1, 1]], ["J", 0x2a, [1, 1, 2, 1, 3, 3]], ["K", 0x2b, [1, 1, 2, 3, 3, 1]], ["L", 0x2c, [1, 3, 2, 1, 3, 1]], ["M", 0x2d, [1, 1, 3, 1, 2, 3]], ["N", 0x2e, [1, 1, 3, 3, 2, 1]], ["O", 0x2f, [1, 3, 3, 1, 2, 1]], ["P", 0x30, [3, 1, 3, 1, 2, 1]], ["Q", 0x31, [2, 1, 1, 3, 3, 1]], ["R", 50, [2, 3, 1, 1, 3, 1]], ["S", 0x33, [2, 1, 3, 1, 1, 3]], ["T", 0x34, [2, 1, 3, 3, 1, 1]], ["U", 0x35, [2, 1, 3, 1, 3, 1]], ["V", 0x36, [3, 1, 1, 1, 2, 3]], ["W", 0x37, [3, 1, 1, 3, 2, 1]], ["X", 0x38, [3, 3, 1, 1, 2, 1]], ["Y", 0x39, [3, 1, 2, 1, 1, 3]], ["Z", 0x3a, [3, 1, 2, 3, 1, 1]], ["[", 0x3b, [3, 3, 2, 1, 1, 1]], ["\\", 60, [3, 1, 4, 1, 1, 1]], ["]", 0x3d, [2, 2, 1, 4, 1, 1]], ["^", 0x3e, [4, 3, 1, 1, 1, 1]], ["_", 0x3f, [1, 1, 1, 2, 2, 4]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00f1', 16)), 0x61, [4, 1, 1, 1, 1, 3]], [String.fromCharCode(parseInt('00f2', 16)), 0x60, [1, 1, 4, 3, 1, 1]], [String.fromCharCode(parseInt('00f3', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00f4', 16)), 0x62, [4, 1, 1, 3, 1, 1]], [String.fromCharCode(parseInt('00fc', 16)), 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00fb', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00f9', 16)), 0x67, [2, 1, 1, 4, 1, 2]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the Code 128 B Barcode. */
        code128b: {
            startSymbol: String.fromCharCode(parseInt('00fd', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x00-\x7F]",
            symbolTable: [[" ", 0, [2, 1, 2, 2, 2, 2]], ["!", 1, [2, 2, 2, 1, 2, 2]], ["\"", 2, [2, 2, 2, 2, 2, 1]], ["#", 3, [1, 2, 1, 2, 2, 3]], ["$", 4, [1, 2, 1, 3, 2, 2]], ["%", 5, [1, 3, 1, 2, 2, 2]], ["&", 6, [1, 2, 2, 2, 1, 3]], ["\'", 7, [1, 2, 2, 3, 1, 2]], ["(", 8, [1, 3, 2, 2, 1, 2]], [")", 9, [2, 2, 1, 2, 1, 3]], ["*", 10, [2, 2, 1, 3, 1, 2]], ["+", 11, [2, 3, 1, 2, 1, 2]], [",", 12, [1, 1, 2, 2, 3, 2]], ["-", 13, [1, 2, 2, 1, 3, 2]], [".", 14, [1, 2, 2, 2, 3, 1]], ["/", 15, [1, 1, 3, 2, 2, 2]], ["0", 0x10, [1, 2, 3, 1, 2, 2]], ["1", 0x11, [1, 2, 3, 2, 2, 1]], ["2", 0x12, [2, 2, 3, 2, 1, 1]], ["3", 0x13, [2, 2, 1, 1, 3, 2]], ["4", 20, [2, 2, 1, 2, 3, 1]], ["5", 0x15, [2, 1, 3, 2, 1, 2]], ["6", 0x16, [2, 2, 3, 1, 1, 2]], ["7", 0x17, [3, 1, 2, 1, 3, 1]], ["8", 0x18, [3, 1, 1, 2, 2, 2]], ["9", 0x19, [3, 2, 1, 1, 2, 2]], [":", 0x1a, [3, 2, 1, 2, 2, 1]], [";", 0x1b, [3, 1, 2, 2, 1, 2]], ["<", 0x1c, [3, 2, 2, 1, 1, 2]], ["=", 0x1d, [3, 2, 2, 2, 1, 1]], [">", 30, [2, 1, 2, 1, 2, 3]], ["?", 0x1f, [2, 1, 2, 3, 2, 1]], ["@", 0x20, [2, 3, 2, 1, 2, 1]], ["A", 0x21, [1, 1, 1, 3, 2, 3]], ["B", 0x22, [1, 3, 1, 1, 2, 3]], ["C", 0x23, [1, 3, 1, 3, 2, 1]], ["D", 0x24, [1, 1, 2, 3, 1, 3]], ["E", 0x25, [1, 3, 2, 1, 1, 3]], ["F", 0x26, [1, 3, 2, 3, 1, 1]], ["G", 0x27, [2, 1, 1, 3, 1, 3]], ["H", 40, [2, 3, 1, 1, 1, 3]], ["I", 0x29, [2, 3, 1, 3, 1, 1]], ["J", 0x2a, [1, 1, 2, 1, 3, 3]], ["K", 0x2b, [1, 1, 2, 3, 3, 1]], ["L", 0x2c, [1, 3, 2, 1, 3, 1]], ["M", 0x2d, [1, 1, 3, 1, 2, 3]], ["N", 0x2e, [1, 1, 3, 3, 2, 1]], ["O", 0x2f, [1, 3, 3, 1, 2, 1]], ["P", 0x30, [3, 1, 3, 1, 2, 1]], ["Q", 0x31, [2, 1, 1, 3, 3, 1]], ["R", 50, [2, 3, 1, 1, 3, 1]], ["S", 0x33, [2, 1, 3, 1, 1, 3]], ["T", 0x34, [2, 1, 3, 3, 1, 1]], ["U", 0x35, [2, 1, 3, 1, 3, 1]], ["V", 0x36, [3, 1, 1, 1, 2, 3]], ["W", 0x37, [3, 1, 1, 3, 2, 1]], ["X", 0x38, [3, 3, 1, 1, 2, 1]], ["Y", 0x39, [3, 1, 2, 1, 1, 3]], ["Z", 0x3a, [3, 1, 2, 3, 1, 1]], ["[", 0x3b, [3, 3, 2, 1, 1, 1]], ["\\", 60, [3, 1, 4, 1, 1, 1]], ["]", 0x3d, [2, 2, 1, 4, 1, 1]], ["^", 0x3e, [4, 3, 1, 1, 1, 1]], ["_", 0x3f, [1, 1, 1, 2, 2, 4]], ["`", 0x40, [1, 1, 1, 4, 2, 2]], ["a", 0x41, [1, 2, 1, 1, 2, 4]], ["b", 0x42, [1, 2, 1, 4, 2, 1]], ["c", 0x43, [1, 4, 1, 1, 2, 2]], ["d", 0x44, [1, 4, 1, 2, 2, 1]], ["e", 0x45, [1, 1, 2, 2, 1, 4]], ["f", 70, [1, 1, 2, 4, 1, 2]], ["g", 0x47, [1, 2, 2, 1, 1, 4]], ["h", 0x48, [1, 2, 2, 4, 1, 1]], ["i", 0x49, [1, 4, 2, 1, 1, 2]], ["j", 0x4a, [1, 4, 2, 2, 1, 1]], ["k", 0x4b, [2, 4, 1, 2, 1, 1]], ["l", 0x4c, [2, 2, 1, 1, 1, 4]], ["m", 0x4d, [4, 1, 3, 1, 1, 1]], ["n", 0x4e, [2, 4, 1, 1, 1, 2]], ["o", 0x4f, [1, 3, 4, 1, 1, 1]], ["p", 80, [1, 1, 1, 2, 4, 2]], ["q", 0x51, [1, 2, 1, 1, 4, 2]], ["r", 0x52, [1, 2, 1, 2, 4, 1]], ["s", 0x53, [1, 1, 4, 2, 1, 2]], ["t", 0x54, [1, 2, 4, 1, 1, 2]], ["u", 0x55, [1, 2, 4, 2, 1, 1]], ["v", 0x56, [4, 1, 1, 2, 1, 2]], ["w", 0x57, [4, 2, 1, 1, 1, 2]], ["x", 0x58, [4, 2, 1, 2, 1, 1]], ["y", 0x59, [2, 1, 2, 1, 4, 1]], ["z", 90, [2, 1, 4, 1, 2, 1]], ["{", 0x5b, [4, 1, 2, 1, 2, 1]], ["|", 0x5c, [1, 1, 1, 1, 4, 3]], ["}", 0x5d, [1, 1, 1, 3, 4, 1]], ["~", 0x5e, [1, 3, 1, 1, 4, 1]], [String.fromCharCode(parseInt('007f', 16)), 0x5f, [1, 1, 4, 1, 1, 3]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00f1', 16)), 0x61, [4, 1, 1, 1, 1, 3]], [String.fromCharCode(parseInt('00f2', 16)), 0x60, [1, 1, 4, 3, 1, 1]], [String.fromCharCode(parseInt('00f3', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00f4', 16)), 0x62, [4, 1, 1, 3, 1, 1]], [String.fromCharCode(parseInt('00fc', 16)), 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00fa', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x68, [2, 1, 1, 2, 1, 4]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the Code 128 C Barcode. */
        code128c: {
            startSymbol: String.fromCharCode(parseInt('00fe', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[0-9]",
            symbolTable: [["\0", 0, [2, 1, 2, 2, 2, 2]], [String.fromCharCode(parseInt('0001', 16)), 1, [2, 2, 2, 1, 2, 2]], [String.fromCharCode(parseInt('0002', 16)), 2, [2, 2, 2, 2, 2, 1]], [String.fromCharCode(parseInt('0003', 16)), 3, [1, 2, 1, 2, 2, 3]], [String.fromCharCode(parseInt('0004', 16)), 4, [1, 2, 1, 3, 2, 2]], [String.fromCharCode(parseInt('0005', 16)), 5, [1, 3, 1, 2, 2, 2]], [String.fromCharCode(parseInt('0006', 16)), 6, [1, 2, 2, 2, 1, 3]], ["\a", 7, [1, 2, 2, 3, 1, 2]], ["\b", 8, [1, 3, 2, 2, 1, 2]], ["\t", 9, [2, 2, 1, 2, 1, 3]], ["\n", 10, [2, 2, 1, 3, 1, 2]], ["\v", 11, [2, 3, 1, 2, 1, 2]], ["\f", 12, [1, 1, 2, 2, 3, 2]], ["\r", 13, [1, 2, 2, 1, 3, 2]], [String.fromCharCode(parseInt('000e', 16)), 14, [1, 2, 2, 2, 3, 1]], [String.fromCharCode(parseInt('000f', 16)), 15, [1, 1, 3, 2, 2, 2]], [String.fromCharCode(parseInt('0010', 16)), 0x10, [1, 2, 3, 1, 2, 2]], [String.fromCharCode(parseInt('0011', 16)), 0x11, [1, 2, 3, 2, 2, 1]], [String.fromCharCode(parseInt('0012', 16)), 0x12, [2, 2, 3, 2, 1, 1]], [String.fromCharCode(parseInt('0013', 16)), 0x13, [2, 2, 1, 1, 3, 2]], [String.fromCharCode(parseInt('0014', 16)), 20, [2, 2, 1, 2, 3, 1]], [String.fromCharCode(parseInt('0015', 16)), 0x15, [2, 1, 3, 2, 1, 2]], [String.fromCharCode(parseInt('0016', 16)), 0x16, [2, 2, 3, 1, 1, 2]], [String.fromCharCode(parseInt('0017', 16)), 0x17, [3, 1, 2, 1, 3, 1]], [String.fromCharCode(parseInt('0018', 16)), 0x18, [3, 1, 1, 2, 2, 2]], [String.fromCharCode(parseInt('0019', 16)), 0x19, [3, 2, 1, 1, 2, 2]], [String.fromCharCode(parseInt('001a', 16)), 0x1a, [3, 2, 1, 2, 2, 1]], [String.fromCharCode(parseInt('001b', 16)), 0x1b, [3, 1, 2, 2, 1, 2]], [String.fromCharCode(parseInt('001c', 16)), 0x1c, [3, 2, 2, 1, 1, 2]], [String.fromCharCode(parseInt('001d', 16)), 0x1d, [3, 2, 2, 2, 1, 1]], [String.fromCharCode(parseInt('001e', 16)), 30, [2, 1, 2, 1, 2, 3]], [String.fromCharCode(parseInt('001f', 16)), 0x1f, [2, 1, 2, 3, 2, 1]], [" ", 0x20, [2, 3, 2, 1, 2, 1]], ["!", 0x21, [1, 1, 1, 3, 2, 3]], ["\"", 0x22, [1, 3, 1, 1, 2, 3]], ["#", 0x23, [1, 3, 1, 3, 2, 1]], ["$", 0x24, [1, 1, 2, 3, 1, 3]], ["%", 0x25, [1, 3, 2, 1, 1, 3]], ["&", 0x26, [1, 3, 2, 3, 1, 1]], ["\'", 0x27, [2, 1, 1, 3, 1, 3]], ["[", 40, [2, 3, 1, 1, 1, 3]], [")", 0x29, [2, 3, 1, 3, 1, 1]], ["*", 0x2a, [1, 1, 2, 1, 3, 3]], ["+", 0x2b, [1, 1, 2, 3, 3, 1]], [",", 0x2c, [1, 3, 2, 1, 3, 1]], ["-", 0x2d, [1, 1, 3, 1, 2, 3]], [".", 0x2e, [1, 1, 3, 3, 2, 1]], ["/", 0x2f, [1, 3, 3, 1, 2, 1]], ["0", 0x30, [3, 1, 3, 1, 2, 1]], ["1", 0x31, [2, 1, 1, 3, 3, 1]], ["2", 50, [2, 3, 1, 1, 3, 1]], ["3", 0x33, [2, 1, 3, 1, 1, 3]], ["4", 0x34, [2, 1, 3, 3, 1, 1]], ["5", 0x35, [2, 1, 3, 1, 3, 1]], ["6", 0x36, [3, 1, 1, 1, 2, 3]], ["7", 0x37, [3, 1, 1, 3, 2, 1]], ["8", 0x38, [3, 3, 1, 1, 2, 1]], ["9", 0x39, [3, 1, 2, 1, 1, 3]], [":", 0x3a, [3, 1, 2, 3, 1, 1]], [";", 0x3b, [3, 3, 2, 1, 1, 1]], ["<", 60, [3, 1, 4, 1, 1, 1]], ["=", 0x3d, [2, 2, 1, 4, 1, 1]], [">", 0x3e, [4, 3, 1, 1, 1, 1]], ["?", 0x3f, [1, 1, 1, 2, 2, 4]], ["@", 0x40, [1, 1, 1, 4, 2, 2]], ["A", 0x41, [1, 2, 1, 1, 2, 4]], ["B", 0x42, [1, 2, 1, 4, 2, 1]], ["C", 0x43, [1, 4, 1, 1, 2, 2]], ["D", 0x44, [1, 4, 1, 2, 2, 1]], ["E", 0x45, [1, 1, 2, 2, 1, 4]], ["F", 70, [1, 1, 2, 4, 1, 2]], ["G", 0x47, [1, 2, 2, 1, 1, 4]], ["H", 0x48, [1, 2, 2, 4, 1, 1]], ["I", 0x49, [1, 4, 2, 1, 1, 2]], ["J", 0x4a, [1, 4, 2, 2, 1, 1]], ["K", 0x4b, [2, 4, 1, 2, 1, 1]], ["L", 0x4c, [2, 2, 1, 1, 1, 4]], ["M", 0x4d, [4, 1, 3, 1, 1, 1]], ["N", 0x4e, [2, 4, 1, 1, 1, 2]], ["O", 0x4f, [1, 3, 4, 1, 1, 1]], ["P", 80, [1, 1, 1, 2, 4, 2]], ["Q", 0x51, [1, 2, 1, 1, 4, 2]], ["R", 0x52, [1, 2, 1, 2, 4, 1]], ["S", 0x53, [1, 1, 4, 2, 1, 2]], ["T", 0x54, [1, 2, 4, 1, 1, 2]], ["U", 0x55, [1, 2, 4, 2, 1, 1]], ["V", 0x56, [4, 1, 1, 2, 1, 2]], ["W", 0x57, [4, 2, 1, 1, 1, 2]], ["X", 0x58, [4, 2, 1, 2, 1, 1]], ["Y", 0x59, [2, 1, 2, 1, 4, 1]], ["Z", 90, [2, 1, 4, 1, 2, 1]], ["[", 0x5b, [4, 1, 2, 1, 2, 1]], ["\\", 0x5c, [1, 1, 1, 1, 4, 3]], ["]", 0x5d, [1, 1, 1, 3, 4, 1]], ["^", 0x5e, [1, 3, 1, 1, 4, 1]], ["_", 0x5f, [1, 1, 4, 1, 1, 3]], ["`", 0x60, [1, 1, 4, 3, 1, 1]], ["a", 0x61, [4, 1, 1, 1, 1, 3]], ["b", 0x62, [4, 1, 1, 3, 1, 1]], ["c", 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00fa', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00fb', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x69, [2, 1, 1, 2, 3, 2]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    "use strict";
    //#region ej.Diagram widget
    ej.widget("ejDiagram", "ej.Diagram", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],

        //#region defaults
        defaults: {
            width: null,
            height: null,
            nodes: null,
            connectors: null,
            enableContextMenu: true,
            snapSettings: null,
            pageSettings: null,
            contextMenu: null,
            autoScroll: true,
            visualGuide: null,
            enableVisualGuide: true,
            autoScrollMargin: 20,
            layout: null,
            userHandles: null,
            drawingTools: {},
            backgroundImage: null,
            selectorConstraints: null,

            nodeCollectionChanged: null,
            connectorCollectionChanged: null,
            nodeCollectionChanging: null,
            connectorCollectionChanging: null,
            selectionChanging: null,
            selectionChanged: null,
            contextMenuBeforeOpen: null,
            contextMenuClick: null,
            mouseLeave: null,
            mouseEnter: null,
            mouseHover: null,
            click: null,
            doubleClick: null,
            drop: null,
            textChanged: null,
            nodeDragging: null,
            nodeDragged: null,
            connectorDragging: null,
            connectorDragged: null,
            nodeSizeChanged: null,
            nodeSizeChanging: null,
            connectionChanging: null,
            connectionChanged: null,
            connectorEndDragged: null,
            nodeRotationChanged: null
        },

        dataTypes: {
            width: "string",
            height: "string",
            zoomIncrement: "number",
            autoScroll: "boolean",
            autoScrollMargin: "number",
            enableContextMenu: "boolean",
            enableVisualGuide: "boolean"
        },

        //#endregion

        //#region Local members
        _canvas: null,
        _svg: null,
        _toolToActivate: null,
        _inAction: false,
        _isEditing: false,
        _nodeToHit: null,
        _currentCursor: null,
        _pasteIndex: null,
        _page: null,
        _view: null,
        _currentLabel: null,
        _zOrder: 0,
        //#endregion

        //#region public members
        activeTool: null,
        selectionList: null,
        tools: null,
        //#endregion

        //#region Initialization
        _init: function () {
            var pageSettings = this.model.pageSettings;
            var snapSettings = this.model.snapSettings;
            this.model.pageSettings = new ej.Diagram.PageSettings(pageSettings);
            this.model.snapSettings = new ej.Diagram.SnapSettings(snapSettings);
            this.model.selectorConstraints = this.model.selectorConstraints || ej.Diagram.SelectorConstraints.All;
            this._spatialsearch = new ej.Diagram.SpatialSearch(this);
            this._clipboard = new ej.Diagram.Clipboard(this);
            this._initDefaults();
            this._initNodeCollection();
            this._initConnectorCollection();
            this._initCanvas();
            this._initVisualGuide();
            this._wireEvents();
            this._initHandles();
            this._initContextMenu();
            this._drawingTool = false;
            this._isDropped = false;
        },

        _setModel: function (options) {
            for (var option in options) {
                switch (option) {
                    case "nodes":
                        this.model.nodes = options[option];
                        this._initNodeCollection();
                        this.element.remove(".node");
                        this._renderNodes();
                        break;
                    case "connectors":
                        this.model.connectors = options[option];
                        this._initConnectorCollection();
                        this.element.remove(".connector");
                        this._renderConnectors();
                        break;
                    case "enableContextMenu":
                        this.model.enableContextMenu = options[option];
                        this._initContextMenu();
                        break;
                    case "contextMenu":
                        this.model.contextMenu = options[option];
                        this._initContextMenu();
                        break;
                    case "visualGuide":
                        this.model.visualGuide = options[option];
                        this._initVisualGuide();
                        break;
                    case "enableVisualGuide":
                        this.model.enableVisualGuide = options[option];
                        break;
                    case "snapSettings":
                        this.model.snapSettings = new ej.Diagram.SnapSettings(options[option]);
                        this._renderGrid();
                        break;
                    case "selectorConstraints":
                        this._setSelectorConstraints(options[option]);
                        break;
                    case "userHandles":
                        this.model.userHandles = options[option];
                        this._initHandles();
                        break;
                    case "drawingTools":
                        this.model.drawingTools = options[option];
                        this._registerDrawingTools();
                        break;
                    case "backgroundImage":
                        this.model.backgroundImage = options[option];
                        this._renderBackground();
                        break;
                    case "autoScroll":
                        this.model.autoScroll = options[option];
                        break;
                    case "autoScrollMargin":
                        this.model.autoScrollMargin = options[option];
                        break;
                    case "layout":
                        this._setLayout(options[option]);
                        break;
                    case "pageSettings":
                        this.model.pageSettings = new ej.Diagram.PageSettings(options[option]);
                        this.model.pageSettings._updatePageSize(this);
                        break;
                }
            }
        },

        _loadDiagramFromFile: function () {
            if (this.model.filePath !== "") {
                var deserialize = new ej.Diagram.Deserializer();
                deserialize.deSerialize(this.model.filePath, this.model, this);
            }
        },

        _initNodeCollection: function () {
            if (!this.model.nodes) {
                this.model.nodes = new ej.Diagram.Collection();
            }
            else {
                var nodes = this.model.nodes;
                if (!(nodes.type === "collection") && nodes instanceof Array) {
                    var collection = new ej.Diagram.Collection();
                    for (var i = 0, len = nodes.length; i < len; i++) {
                        if (nodes[i].children === undefined) {
                            var node = new ej.Diagram.Node(nodes[i]);
                            collection.add(node);
                        }
                        else {
                            var group = new ej.Diagram.Group(nodes[i]);
                            collection.add(group);
                        }
                    }
                    this.model.nodes = collection;
                }
            }
            this.model.nodes.addEventHandler("collectionchanging", $.proxy(this._nodeCollectionChanging, this));
            this.model.nodes.addEventHandler("collectionchanged", $.proxy(this._nodeCollectionChanged, this));
            this._spatialsearch._initializeNodes();
        },

        _nodeCollectionChanging: function (args) {
            this._trigger("nodeCollectionChanging", args);
        },

        _nodeCollectionChanged: function (args) {
            if (!this._isUndo) {
                var entry;
                if (this.selectionList[0] && this.selectionList[0].type == "pseudoGroup") {
                    var obj = this.selectionList[0];
                    var children = [];
                    for (var i = 0; i < obj.children.length; i++) {
                        children.push(obj.children[i]);
                    }
                    entry = new ej.Diagram.HistoryEntry(new ej.Diagram.CollectionModifyCmd(args.changeType, children));
                    this.historyManager.addHistoryEntry(entry);
                }
                else if (!(args.element.type === "pseudoGroup")) {
                    entry = new ej.Diagram.HistoryEntry(new ej.Diagram.CollectionModifyCmd(args.changeType, args.element));
                    this.historyManager.addHistoryEntry(entry);
                }
            }
            this._trigger("nodeCollectionChanged", args);
        },

        _connectorCollectionChanging: function (args) {
            this._trigger("connectorCollectionChanging", args);
        },

        _connectorCollectionChanged: function (args) {
            if (!this._isUndo) {
                var entry;
                if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup") {
                    var obj = this.selectionList[0];
                    var children = [];
                    for (var i = 0; i < obj.children.length; i++) {
                        children.push(obj.children[i]);
                    }
                    entry = new ej.Diagram.HistoryEntry(new ej.Diagram.CollectionModifyCmd(args.changeType, children));
                    this.historyManager.addHistoryEntry(entry);
                }
                else if (!(args.element.type === "pseudoGroup")) {
                    entry = new ej.Diagram.HistoryEntry(new ej.Diagram.CollectionModifyCmd(args.changeType, args.element));
                    this.historyManager.addHistoryEntry(entry);
                }
            }
            this._trigger("connectorCollectionChanged", args);
        },

        _initConnectorCollection: function () {
            if (!this.model.connectors) {
                this.model.connectors = new ej.Diagram.Collection();
            }
            else {
                var connectors = this.model.connectors;
                if (!(connectors.type === "collection") && connectors instanceof Array) {
                    var collection = new ej.Diagram.Collection();
                    for (var i = 0, len = connectors.length; i < len; i++) {
                        var connector = new ej.Diagram.Connector(connectors[i]);
                        if (connector.targetNodeName && connector.targetNodeName !== "")
                            connector.targetNode = this._findNode(connector.targetNodeName);
                        if (connector.targetPortName && connector.targetPortName !== "")
                            connector.targetPort = this._findPort(connector.targetNode, connector.targetPortName);
                        if (connector.sourceNodeName && connector.sourceNodeName !== "")
                            connector.sourceNode = this._findNode(connector.sourceNodeName);
                        if (connector.sourcePortName && connector.sourcePortName !== "")
                            connector.sourcePort = this._findPort(connector.sourceNode, connector.sourcePortName);
                        collection.add(connector);
                    }
                    this.model.connectors = collection;
                }
            }
            this.model.connectors.addEventHandler("collectionchanging", $.proxy(this._connectorCollectionChanging, this));
            this.model.connectors.addEventHandler("collectionchanged", $.proxy(this._connectorCollectionChanged, this));
            this._spatialsearch._initializeConnectors();
        },

        _initDefaults: function () {
            this.selectionList = new ej.Diagram.Collection();
            this.selectionList.addEventHandler("collectionchanging", $.proxy(this._selectionChanging, this));
            this.selectionList.addEventHandler("collectionchanged", $.proxy(this._selectionChanged, this));
            this._initTools();
        },

        _initCanvas: function () {
            this._renderCanvas();
        },

        _initVisualGuide: function () {
            if (!this.model.visualGuide && this.model.enableVisualGuide) {
                var script = document.createElement("script");
                var attr = {
                    "id": this.element[0].id + "_defaultTooltip",
                    "type": "text/x-jsrender"
                };
                ej.Diagram.Util.attr(script, attr);
                script.innerHTML = "<span class=\"e-diagram-tooltipLabel-default\"> {{:type}}{{:x}}{{:type1}}{{:y}}</span>";
                this._canvas.appendChild(script);
                this.model.visualGuide = new ej.Diagram.VisualGuide(ej.Diagram.VisualGuideType.Tooltip, "");
            }
            else {
                this.element.remove("#" + this.element[0].id + "_defaultTooltip");
            }
        },

        _selectionChanging: function (args) {
            this._trigger("selectionChanging", args);
        },

        _selectionChanged: function (args) {
            this._trigger("selectionChanged", args);
        },

        _initTools: function () {
            this.tools = {
                "select": new ej.Diagram.SelectTool(this),
                "move": new ej.Diagram.MoveTool(this),
                "resize": new ej.Diagram.ResizeTool(this),
                "rotate": new ej.Diagram.RotateTool(this),
                //"pivot": new ej.Diagram.PivotTool(this),
                "endPoint": new ej.Diagram.EndPointTool(this),
                "panTool": new ej.Diagram.PanTool(this),
                "straightLine": new ej.Diagram.StraightLineTool(this),
                "orthogonalLine": new ej.Diagram.OrthogonalLineTool(this)
            };
            this.historyManager = new ej.Diagram.HistoryManager();
            this._registerHistoryEvents();
            this._toolToActivate = "select";
            this.activeTool = this.tools[this._toolToActivate];
            this._registerDrawingTools();
        },

        _initHandles: function () {
            var userHandles = this.model.userHandles;
            if (userHandles) {
                var collection = new ej.Diagram.Collection();
                var i;
                for (i = 0; i < userHandles.length; i++) {
                    collection.add(new ej.Diagram.UserHandle(userHandles[i]));
                }
                this.model.userHandles = collection;

                for (i = 0; i < userHandles.length; i++) {
                    var handle = userHandles[i];
                    if (handle.tool) {
                        handle.tool.diagram = this;
                        this.tools[handle.name] = handle.tool;
                    }
                }
            }
        },

        _initLayout: function () {
            if (this.model.layout) {
                this._doLayout();
            }
        },

        _destroy: function () {
            this.element.ejScroller("destroy");
            this.element.empty().removeClass("e-diagram");
        },
        //#endregion       

        //#region Public methods

        save: function () {
            var serializer = new ej.Diagram.Serializer();
            var serializedObj = serializer.serialize(this.model);
            return serializedObj;
        },

        load: function (jsonObj) {
            var deSerializer = new ej.Diagram.Deserializer();
            this.element.empty();
            deSerializer.deSerialize(jsonObj, this);
        },

        setNodeShape: function (node, shape) {
            if (shape) {
                var mNode = this._findNode(node.fullName());
                var parent;
                if (mNode && mNode !== node) {
                    parent = mNode;
                }
                if (mNode) {
                    node.shape = shape;
                    ej.Diagram.SvgContext.setNodeShape(node, this._svg, parent);
                }
            }
        },

        setLine: function (connector, line) {
            var con = this._findConnector(connector.fullName());
            var parent;
            if (con) {
                if (connector !== con) {
                    parent = con;
                }
                if (line.type === "straight") {
                    connector.line = new ej.Diagram.Lines.Straight(line);
                }
                else if (line.type === "orthogonal") {
                    connector.line = new ej.Diagram.Lines.Orthogonal(line);
                }
                else if (line.type === "bezier") {
                    connector.line = new ej.Diagram.Lines.Bezier(line);
                }
                ej.Diagram.SvgContext.setLine(connector, this._svg, parent);
            }
        },

        _setLayout: function (layout) {
            if (layout) {
                this.model.layout = layout;
                this._doLayout();
                this._clearSelection();
                this._updateNodes();
            }
        },

        addSelection: function (node) {
            this._addSelection(node);
        },

        updateSelection: function (isDragging) {
            this._updateSelectionHandle(isDragging);
        },

        activateTool: function (toolName, singleAction) {

            if (this.tools[toolName]) {
                if (this.activeTool instanceof ej.Diagram.LineTool) {
                    this.activeTool._showAllPorts(true);
                }
                this._toolToActivate = toolName;
                this.activeTool = this.tools[this._toolToActivate];
                if (singleAction != undefined)
                    this.activeTool.singleAction = singleAction;
            }
            if (this.activeTool instanceof ej.Diagram.ImageTool && this.activeTool.autoOpenUpload) {
                this.activeTool._showPopup();
            }
            if (this.activeTool instanceof ej.Diagram.PanTool) {
                this._currentCursor = "pointer";
            }
            if (this.activeTool instanceof ej.Diagram.LineTool) {
                this.activeTool._showAllPorts();
            }
            this._drawingTool = true;
        },

        deActivateTool: function () {
            if (this.activeTool instanceof ej.Diagram.LineTool) {
                this.activeTool._showAllPorts(true);
            }
            this._toolToActivate = "select";
            this.activeTool = this.tools[this._toolToActivate];
            this._drawingTool = false;
        },

        setTargetDecorator: function (name, value) {
            if (name) {
                var connector = this._findConnector(name);
                if (connector) {
                    connector.targetDecorator = value;
                    ej.Diagram.SvgContext.clearDecorators(connector, this._svg);
                    ej.Diagram.SvgContext.renderDecorators(connector, this._svg);
                }
            }
        },

        setTargetDecoratorStyle: function (name, value) {
            if (name && value) {
                var connector = this._findConnector(name);
                if (connector && connector.targetDecorator) {
                    var decorator = connector.targetDecorator;
                    if (value.borderColor) {
                        decorator.borderColor = value.borderColor;
                    }
                    if (value.fillColor) {
                        decorator.fillColor = value.fillColor;
                    }
                    ej.Diagram.SvgContext.updateTargetDecoratorStyle(connector, this._svg);
                }
            }
        },

        setTargetDecoratorSize: function (name, value) {
            if (name && value) {
                var connector = this._findConnector(name);
                if (connector && connector.targetDecorator) {
                    if (value.height)
                        connector.targetDecorator.height = value.height;
                    if (value.width)
                        connector.targetDecorator.width = value.width;
                    ej.Diagram.SvgContext.clearDecorators(connector, this._svg);
                    ej.Diagram.SvgContext.renderDecorators(connector, this._svg);
                }
            }
        },

        setTargetDecoratorShape: function (name, value) {
            if (name && value) {
                var connector = this._findConnector(name);
                if (connector) {
                    connector.targetDecorator.shape = value.shape;
                    ej.Diagram.SvgContext.clearDecorators(connector, this._svg);
                    ej.Diagram.SvgContext.renderDecorators(connector, this._svg);
                }
            }
        },

        setSourceDecoratorSize: function (name, value) {
            if (name && value) {
                var connector = this._findConnector(name);
                if (connector && connector.sourceDecorator) {
                    if (value.height)
                        connector.sourceDecorator.height = value.height;
                    if (value.width)
                        connector.sourceDecorator.width = value.width;
                    ej.Diagram.SvgContext.clearDecorators(connector, this._svg);
                    ej.Diagram.SvgContext.renderDecorators(connector, this._svg);
                }
            }
        },

        setSourceDecoratorShape: function (name, value) {
            if (name && value) {
                var connector = this._findConnector(name);
                if (connector && connector.sourceDecorator) {
                    connector.sourceDecorator.shape = value.shape;
                    ej.Diagram.SvgContext.clearDecorators(connector, this._svg);
                    ej.Diagram.SvgContext.renderDecorators(connector, this._svg);
                }
            }
        },

        setSourceDecorator: function (name, value) {
            if (name) {
                var connector = this._findConnector(name);
                if (connector) {
                    connector.sourceDecorator = value;
                    ej.Diagram.SvgContext.clearDecorators(connector, this._svg);
                    ej.Diagram.SvgContext.renderDecorators(connector, this._svg);
                }
            }
        },

        setSourceDecoratorStyle: function (name, value) {
            if (name && value) {
                var connector = this._findConnector(name);
                if (connector && connector.sourceDecorator) {
                    var decorator = connector.sourceDecorator;
                    if (value.borderColor) {
                        decorator.borderColor = value.borderColor;
                    }
                    if (value.fillColor) {
                        decorator.fillColor = value.fillColor;
                    }
                    ej.Diagram.SvgContext.updateSourceDecoratorStyle(connector, this._svg);
                }
            }
        },

        setNodeStyle: function (name, value) {
            if (name) {
                var mNode = this._findNode(name);
                if (mNode) {
                    if (value.fillColor) {
                        mNode.fillColor = value.fillColor;
                    }
                    if (value.borderColor) {
                        mNode.borderColor = value.borderColor;
                    }
                    if (value.borderWidth) {
                        mNode.borderWidth = value.borderWidth;
                    }
                    if (value.lineDashArray) {
                        mNode.lineDashArray = value.lineDashArray;
                    }

                    if (value.gradient !== undefined)
                        mNode.gradient = value.gradient;

                    ej.Diagram.SvgContext.updateNodeStyle(mNode, this._svg);
                }
            }
        },

        setLabelStyle: function (nodeName, label, obj) {
            if (nodeName) {
                var mNode = this._findNode(nodeName);
                if (!mNode)
                    mNode = this._findConnector(nodeName);
                if (mNode) {
                    for (var prop in obj)
                        if (label.hasOwnProperty(prop)) {
                            label[prop] = obj[prop];
                        }
                    if (mNode.type === "connector" || (mNode.shape && mNode.shape.type !== "text")) {
                        ej.Diagram.SvgContext.updateLabelStyle(mNode, label, this._svg);
                        ej.Diagram.SvgContext.updateLabel(mNode, label, this._svg);
                    }
                    else if (mNode.shape && mNode.shape.type === "text") {
                        ej.Diagram.SvgContext.updateTextBlock(mNode, label, this._svg);
                    }
                }
            }
        },

        startLabelEdit: function (node, label) {
            if (node) {
                //Disables the edit mode of labels in the selectionList
                for (var i = 0; i < this.selectionList.length; i++) {
                    var mNode = this.selectionList[i];
                    if (mNode !== node && typeof mNode.labels.length != "undefined") {
                        for (var j = 0; j < mNode.labels.length; j++)
                            mNode.labels[j].mode = ej.Diagram.LabelEditMode.View;
                    }
                }
                if (this._setLabelEditing(label))
                    this._startEdit(node);
                else {
                    this._endEdit();
                }
            }
        },

        setConstraints: function (name, value) {
            if (name && value) {
                var mNode = this._findNode(name);
                if (!mNode)
                    mNode = this._findConnector(name);
                if (mNode) {
                    mNode.constraints = value;
                    if (this.selectionList[0] && this.selectionList[0].fullName() === mNode.fullName())
                        ej.Diagram.SvgContext.renderSelector(mNode, this._svg, this._adornerLayer, this._zoomPan.currentZoom,
                        this.model.selectorConstraints);
                }
            }
        },

        setConnectorStyle: function (name, value) {
            if (name) {
                var mConnector = this._findConnector(name);
                if (mConnector) {
                    if (value.lineColor) {
                        mConnector.lineColor = value.lineColor;
                    }
                    if (value.lineWidth) {
                        mConnector.lineWidth = value.lineWidth;
                    }
                    if (value.lineDashArray) {
                        mConnector.lineDashArray = value.lineDashArray;
                    }
                    ej.Diagram.SvgContext.updateConnectorStyle(mConnector, this._svg);
                }
            }
        },

        _setSelectorConstraints: function (constraints) {
            this.model.selectorConstraints = constraints;
            if (this.selectionList[0]) {
                ej.Diagram.SvgContext.clearSelector(this._svg, this._adornerLayer);
                ej.Diagram.SvgContext.renderSelector(this.selectionList[0], this._svg, this._adornerLayer, this._zoomPan.currentZoom, constraints);
                if (this.model.selectorConstraints & ej.Diagram.SelectorConstraints.UserHandles)
                    ej.Diagram.SvgContext.renderUserHandles(this.model.userHandles, this.selectionList[0], this._svg, this.selectionList[0].type === "pseudoGroup",
                        this._zoomPan.currentZoom, this._adornerLayer);
            }
        },

        group: function () {
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup") {
                var selectionList = this.selectionList[0].children;
                var i;
                var group = new ej.Diagram.Group();
                this._clearSelection();
                for (i = 0; i < selectionList.length; i++) {
                    this._isUndo = true;
                    group.children.add(selectionList[i]);
                    this.selectionList.remove(selectionList[i]);
                    if (selectionList[i].type === "connector")
                        this.model.connectors.remove(selectionList[i]);
                    else
                        this.model.nodes.remove(selectionList[i]);
                    if (ej.browserInfo().name === "msie") {
                        if (selectionList[i].shape && selectionList[i].shape.type == "html") {
                            var element = document.getElementById(selectionList[i].fullName() + "_parentdiv");
                            element.parentNode.removeChild(element);
                        }
                        else {
                            if (selectionList[i].type == "group")
                                this._checkForHtmlNode(selectionList[i]);
                        }
                    }
                    // $("#" + selectionList[i].name)[0].parentNode.removeChild($("#" + selectionList[i].name)[0]);
                    var element = document.getElementById(selectionList[i].fullName());
                    element.parentNode.removeChild(element);
                }
                this.selectionList.clear();
                var gname = ej.Diagram.Util.randomId();
                group.name = gname;
                this.add(group);
                this._addSelection(group);
                var entry = new ej.Diagram.HistoryEntry(new ej.Diagram.GroupHandleCmd(group, "group"));
                this.historyManager.addHistoryEntry(entry);
                this._isUndo = false;
            }
        },

        _checkForHtmlNode: function (group) {
            for (var i = 0; i < group.children.length; i++) {
                var child = group.children[i];
                if (child.type == "node") {
                    if (child.shape.type == "html") {
                        var element = document.getElementById(child.fullName() + "_parentdiv");
                        element.parentNode.removeChild(element);
                    }
                }
                else if (child.type == "group") {
                    this._checkForHtmlNode(child);
                }
            }
        },
        ungroup: function () {
            if (this.selectionList[0] && this.selectionList[0].type === "group") {
                this._isUndo = true;
                var selectionList = this.selectionList[0].children;
                var i, node;
                node = this.selectionList[0];
                $("#" + this.selectionList[0].name)[0].parentNode.removeChild($("#" + this.selectionList[0].name)[0]);
                if (ej.browserInfo().name === "msie")
                    this._checkForHtmlNode(this.selectionList[0]);
                this._clearSelection();
                this.selectionList.clear();
                for (i = 0; i < selectionList.length; i++) {
                    selectionList[i].parent = null;
                    if (selectionList[i].type === "connector") {
                        var object = selectionList[i];
                        this.model.connectors.add(object);
                        this._updateEdges(object);
                        object.dock();
                        ej.Diagram.SvgContext.renderConnector(object, this._svg, this._diagramLayer, this);
                    } else {
                        this.model.nodes.add(selectionList[i]);
                        var object = selectionList[i];
                        if (object.type === "group") {
                            this._updateChildrenEdges(object);
                            ej.Diagram.SvgContext.renderGroup(object, this._svg, this._diagramLayer, this);
                        }
                        else
                            ej.Diagram.SvgContext.renderNode(object, this._svg, this._diagramLayer, this);
                    }

                }
                //this._clearSelection();
                this.model.nodes.remove(this._findNode(node.name));
                var entry = new ej.Diagram.HistoryEntry(new ej.Diagram.GroupHandleCmd(node, "ungroup"));
                this.historyManager.addHistoryEntry(entry);
                this._isUndo = false;

            }
        },

        nudgeUp: function () {
            if (this.selectionList[0]) {
                var node = this.selectionList[0];
                this._nudge(node, 0, -1);
            }
        },

        nudgeDown: function () {
            if (this.selectionList[0]) {
                var node = this.selectionList[0];
                this._nudge(node, 0, 1);
            }
        },

        nudgeLeft: function () {
            if (this.selectionList[0]) {
                var node = this.selectionList[0];
                this._nudge(node, -1, 0);
            }
        },

        nudgeRight: function () {
            if (this.selectionList[0]) {
                var node = this.selectionList[0];
                this._nudge(node, 1, 0);
            }
        },

        selectAll: function () {
            var collection, nodes, i, len;
            collection = new ej.Diagram.PseudoGroup();
            nodes = this.model.nodes;
            for (i = 0, len = nodes.length; i < len; i++) {
                if (nodes[i].canSelect())
                    collection.children.add(nodes[i]);
            }
            var connectors = this.model.connectors;
            for (i = 0, len = connectors.length; i < len; i++) {
                if (connectors[i].canSelect())
                    collection.children.add(connectors[i]);
            }
            this.selectionList.clear();
            this._clearSelection();
            if (!collection.children.isEmpty())
                this._addSelection(collection);
        },

        setNodeHeight: function (name, height) {
            if (name) {
                var mNode = this._findNode(name);
                if (mNode) {
                    var delheight = height / mNode.height;
                    mNode._scale(new ej.Diagram.Matrix()._createIdentity(), 1, delheight, new ej.Diagram.Point(mNode.offsetX, mNode.offsetY));
                    if (mNode.type == "node")
                        ej.Diagram.SvgContext.updateNode(mNode, this._svg);
                    else
                        ej.Diagram.SvgContext.updateGroup(mNode, this._svg);
                    if (this.selectionList[0] && this.selectionList[0].fullName() === mNode.fullName())
                        ej.Diagram.SvgContext.updateSelector(mNode, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
                }
            }

        },

        setNodeWidth: function (name, width) {
            if (name) {
                var mNode = this._findNode(name);
                if (mNode) {
                    var delwidth = width / mNode.width;
                    mNode._scale(new ej.Diagram.Matrix()._createIdentity(), delwidth, 1, new ej.Diagram.Point(mNode.offsetX, mNode.offsetY));
                    if (mNode.type == "node")
                        ej.Diagram.SvgContext.updateNode(mNode, this._svg);
                    else
                        ej.Diagram.SvgContext.updateGroup(mNode, this._svg);
                    if (this.selectionList[0] && this.selectionList[0].fullName() === mNode.fullName())
                        ej.Diagram.SvgContext.updateSelector(mNode, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
                }
            }
        },

        setNodeRotateAngle: function (name, angle) {
            if (name) {
                var mNode = this._findNode(name);
                if (mNode) {
                    var dif = angle - mNode.rotateAngle;
                    mNode._rotate(dif);
                    if (mNode.type == "node")
                        ej.Diagram.SvgContext.updateNode(mNode, this._svg);
                    else
                        ej.Diagram.SvgContext.updateGroup(mNode, this._svg);
                    if (this.selectionList[0] && this.selectionList[0].fullName() === mNode.fullName())
                        ej.Diagram.SvgContext.updateSelector(mNode, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
                }
            }
        },
        setNodeOffsetX: function (name, x) {
            if (name) {
                var mNode = this._findNode(name);
                if (mNode) {
                    var dif = x - mNode.offsetX;
                    mNode._translate(dif, 0);
                    if (mNode.type == "node")
                        ej.Diagram.SvgContext.updateNode(mNode, this._svg);
                    else
                        ej.Diagram.SvgContext.updateGroup(mNode, this._svg);
                    if (this.selectionList[0] && this.selectionList[0].fullName() === mNode.fullName())
                        ej.Diagram.SvgContext.updateSelector(mNode, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
                }
            }
        },

        setNativeNodeContent: function (name, content) {
            if (name) {
                var mNode = this._findNode(name);
                if (mNode && mNode.shape.type === "native") {
                    mNode.shape.content = content;
                    ej.Diagram.SvgContext.updateNode(mNode, this._svg);
                    if (this.selectionList[0] && this.selectionList[0].fullName() === mNode.fullName())
                        ej.Diagram.SvgContext.updateSelector(mNode, this._svg, this._zoomPan.currentZoom, this.model.selectorConstraints);
                }
            }
        },

        setNodeOffsetY: function (nodeName, y) {
            if (nodeName) {
                var mNode = this._findNode(nodeName);
                if (mNode) {
                    var dif = y - mNode.offsetY;
                    mNode._translate(0, dif);
                    if (mNode.type == "node")
                        ej.Diagram.SvgContext.updateNode(mNode, this._svg);
                    else
                        ej.Diagram.SvgContext.updateGroup(mNode, this._svg);
                    if (this.selectionList[0] && this.selectionList[0].fullName() === mNode.fullName())
                        ej.Diagram.SvgContext.updateSelector(mNode, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
                }
            }
        },

        alignLeft: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNode = this._findCornerNodes().left;
                var fNodeBounds = fNode.getBoundingBox(fNode);
                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    bounds = list[i].getBoundingBox();
                    var diffX = bounds.x - fNodeBounds.x;
                    this._moveNode(-diffX, 0, list[i], collection);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        alignRight: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNode = this._findCornerNodes().right;
                var fNodeBounds = fNode.getBoundingBox(fNode);
                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    bounds = list[i].getBoundingBox();
                    var diffX = (bounds.x + bounds.width) - (fNodeBounds.x + fNodeBounds.width);
                    this._moveNode(-diffX, 0, list[i], collection);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        alignCenter: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNodeBounds = this.selectionList[0].getBoundingBox();

                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    bounds = list[i].getBoundingBox();
                    var diffX = (bounds.x + bounds.width / 2) - (fNodeBounds.x + fNodeBounds.width / 2);
                    this._moveNode(-diffX, 0, list[i], collection);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        alignTop: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNode = this._findCornerNodes().top;
                var fNodeBounds = fNode.getBoundingBox(fNode);
                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    bounds = list[i].getBoundingBox();
                    var diffY = bounds.y - fNodeBounds.y;
                    this._moveNode(0, -diffY, list[i], collection);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        alignBottom: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNode = this._findCornerNodes().bottom;
                var fNodeBounds = fNode.getBoundingBox(fNode);
                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    bounds = list[i].getBoundingBox();
                    var diffY = (bounds.y + bounds.height) - (fNodeBounds.y + fNodeBounds.height);
                    this._moveNode(0, -diffY, list[i], collection);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        alignMiddle: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNodeBounds = this.selectionList[0].getBoundingBox();
                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    bounds = list[i].getBoundingBox();
                    var diffY = (bounds.y + bounds.height / 2) - (fNodeBounds.y + fNodeBounds.height / 2);
                    this._moveNode(-0, -diffY, list[i], collection);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        sameSize: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNode = this.selectionList[0].children[0];
                var fNodeBounds = fNode.getBoundingBox(fNode);
                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    if (!(list[i].type === "connector")) {
                        list[i].height = fNodeBounds.height;
                        list[i].width = fNodeBounds.width;
                        list[i].updatebounds();
                        if (list[i].type === "node")
                            ej.Diagram.SvgContext.updateNode(list[i], this._svg);
                        else if (list[i].type === "group")
                            ej.Diagram.SvgContext.updateGroup(list[i], this._svg);
                    }
                    collection.children.add(list[i]);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        sameHeight: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNode = this.selectionList[0].children[0];
                var fNodeBounds = fNode.getBoundingBox(fNode);
                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    if (!(list[i].type === "connector")) {
                        list[i].height = fNodeBounds.height;
                        list[i].updatebounds();
                        if (list[i].type === "node")
                            ej.Diagram.SvgContext.updateNode(list[i], this._svg);
                        else if (list[i].type === "group")
                            ej.Diagram.SvgContext.updateGroup(list[i], this._svg);
                    }
                    collection.children.add(list[i]);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        sameWidth: function () {
            var collection = new ej.Diagram.PseudoGroup();
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup" && this.selectionList[0].children && !this.selectionList[0].children.isEmpty()) {
                var list = this.selectionList[0].children;
                var fNode = this.selectionList[0].children[0];
                var fNodeBounds = fNode.getBoundingBox(fNode);
                var i, bounds;
                for (i = 0; i < list.length; i++) {
                    if (!(list[i].type === "connector")) {
                        list[i].width = fNodeBounds.width;
                        list[i].updatebounds();
                        if (list[i].type === "node")
                            ej.Diagram.SvgContext.updateNode(list[i], this._svg);
                        else if (list[i].type === "group")
                            ej.Diagram.SvgContext.updateGroup(list[i], this._svg);
                    }
                    collection.children.add(list[i]);
                }
                ej.Diagram.SvgContext.updateSelector(collection, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
            }
        },

        spaceDown: function () {
            var lstSelections = this.selectionList[0].children;
            this._sortByYvalue(this.selectionList[0].children);
            var selectionCount = lstSelections.length, fSpacing;
            if (lstSelections && selectionCount > 2) {
                var cornerNodes = this._findCornerNodes();
                var firstNode = cornerNodes.top;
                var lastNode = cornerNodes.bottom;
                if (firstNode && lastNode) {
                    var bFirst = firstNode.getBoundingBox();
                    var bLast = lastNode.getBoundingBox();
                    fSpacing = (bLast.y - bFirst.y) / (selectionCount - 1);
                    var y = 1;
                    for (var i = 0; i < lstSelections.length; i++) {
                        if (lstSelections[i] !== firstNode && lstSelections[i] !== lastNode) {
                            lstSelections[i].offsetY = bFirst.y + (y * fSpacing) + (lstSelections[i].height / 2);
                            lstSelections[i].updatebounds();
                            y += 1;
                            if (lstSelections[i].type === "connector")
                                ej.Diagram.SvgContext.updateConnector(lstSelections[i], this._svg);
                            else if (lstSelections[i].type === "node")
                                ej.Diagram.SvgContext.updateNode(lstSelections[i], this._svg);
                            else if (lstSelections[i].type === "group")
                                ej.Diagram.SvgContext.updateGroup(lstSelections[i], this._svg);
                        }
                    }
                }
            }
        },

        spaceAcross: function () {
            var lstSelections = this.selectionList[0].children;
            this._sortByXvalue(this.selectionList[0].children);
            var selectionCount = lstSelections.length, fSpacing;
            if (lstSelections && selectionCount > 2) {
                var units;
                var cornerNodes = this._findCornerNodes();
                var firstNode = cornerNodes.left;
                var lastNode = cornerNodes.right;
                if (firstNode && lastNode) {
                    var bFirst = firstNode.getBoundingBox();
                    var bLast = lastNode.getBoundingBox();
                    fSpacing = (bLast.x - bFirst.x) / (selectionCount - 1);
                    var x = 1;
                    for (var i = 0; i < lstSelections.length; i++) {
                        if (lstSelections[i] !== firstNode && lstSelections[i] !== lastNode) {
                            lstSelections[i].offsetX = bFirst.x + (x * fSpacing) + (lstSelections[i].width / 2);
                            lstSelections[i].updatebounds();
                            x += 1;
                            if (lstSelections[i].type === "connector")
                                ej.Diagram.SvgContext.updateConnector(lstSelections[i], this._svg);
                            else if (lstSelections[i].type === "node")
                                ej.Diagram.SvgContext.updateNode(lstSelections[i], this._svg);
                            else if (lstSelections[i].type === "group")
                                ej.Diagram.SvgContext.updateGroup(lstSelections[i], this._svg);
                        }
                    }
                }
            }
        },

        sendBackward: function () {
            var ovarLapNodes, i;
            var prevNode;
            if (this.selectionList[0]) {
                var node = this.selectionList[0];
                if (node) {
                    ovarLapNodes = this._findOverlapNode(node);
                    this._sortByZIndex(ovarLapNodes);
                    for (var k = 0; k < ovarLapNodes.length; k++)
                        if (node.zOrder > ovarLapNodes[k].zOrder) {
                            prevNode = ovarLapNodes[k];
                            break;
                        }
                    var start = node.zOrder;
                    if (prevNode) {
                        var end = prevNode.zOrder;
                        for (i = end; i < start; i++) {
                            if (node.parent)
                                this._findObjectByIndex(i, true).zOrder += 1;
                            else
                                this._findObjectByIndex(i).zOrder += 1;
                        }
                        if (node.parent) {
                            this._findChildren(node.parent, node.fullName()).zOrder = end;
                        } else
                            this._findObjectByName(node.name).zOrder = end;
                        this._svg.getElementById(prevNode.fullName()).parentNode.insertBefore(this._svg.getElementById(node.fullName()), this._svg.getElementById(prevNode.fullName()));
                        ej.Diagram.SvgContext.renderSelector(node, this._svg, this._adornerLayer, this._zoomPan.currentZoom,
                        this.model.selectorConstraints);
                    }
                }
            }
        },

        moveForward: function () {
            var ovarLapNodes, i;
            var prevNode;
            if (this.selectionList[0]) {
                var node = this.selectionList[0];
                if (node) {
                    ovarLapNodes = this._findOverlapNode(node);
                    this._sortByZIndex(ovarLapNodes);
                    for (var k = ovarLapNodes.length - 1; k >= 0; k--)
                        if (node.zOrder < ovarLapNodes[k].zOrder) {
                            prevNode = ovarLapNodes[k];
                            break;
                        }

                    var start = node.zOrder;
                    if (prevNode) {
                        var end = prevNode.zOrder;
                        for (i = end; i > start; i--) {
                            if (node.parent)
                                this._findObjectByIndex(i, true).zOrder -= 1;
                            else
                                this._findObjectByIndex(i).zOrder -= 1;
                        }
                        if (node.parent)
                            this._findChildren(node.parent, node.fullName()).zOrder = end;
                        else
                            this._findObjectByName(node.name).zOrder = end;
                        this._svg.getElementById(prevNode.fullName()).parentNode.insertBefore(this._svg.getElementById(node.fullName()), this._svg.getElementById(prevNode.fullName()).nextSibling);
                        ej.Diagram.SvgContext.renderSelector(node, this._svg, this._adornerLayer, this._zoomPan.currentZoom,
                        this.model.selectorConstraints);

                    }
                }
            }
        },

        sendToBack: function () {
            var node = this.selectionList[0];
            var prevNode;
            if (node) {
                var ovarLapNodes = this._findOverlapNode(node);
                var nodes = this._sortByZIndex(ovarLapNodes);
                var start = node.zOrder;
                var end = node.zOrder;
                for (var i = 0; i < nodes.length; i++) {
                    if (nodes[i].zOrder < end) {
                        end = nodes[i].zOrder;
                        prevNode = nodes[i];
                    }
                }
                if (prevNode) {
                    for (i = end; i < start; i++) {
                        if (node.parent)
                            this._findObjectByIndex(i, true).zOrder += 1;
                        else
                            this._findObjectByIndex(i).zOrder += 1;
                    }
                    if (node.parent) {
                        this._findChildren(node.parent, node.fullName()).zOrder = end;
                    } else
                        this._findObjectByName(node.name).zOrder = end;
                    this._svg.getElementById(prevNode.fullName()).parentNode.insertBefore(this._svg.getElementById(node.fullName()), this._svg.getElementById(prevNode.fullName()));
                    ej.Diagram.SvgContext.renderSelector(node, this._svg, this._adornerLayer, this._zoomPan.currentZoom,
                        this.model.selectorConstraints);
                }
            }
        },

        bringToFront: function () {
            var node = this.selectionList[0];
            var prevNode;
            if (node) {
                var ovarLapNodes = this._findOverlapNode(node);
                var nodes = this._sortByZIndex(ovarLapNodes);
                var start = node.zOrder;
                var end = node.zOrder;
                for (var i = 0; i < nodes.length; i++) {
                    if (nodes[i].zOrder > end) {
                        end = nodes[i].zOrder;
                        prevNode = nodes[i];
                    }
                }
                if (prevNode) {
                    for (i = 0; i < nodes.length; i++) {
                        if (nodes[i].zOrder > start) {
                            nodes[i].zOrder -= 1;
                        }
                    }
                    if (node.parent) {
                        this._findChildren(node.parent, node.fullName()).zOrder = end;
                    } else
                        this._findObjectByName(node.name).zOrder = end;

                    this._svg.getElementById(prevNode.fullName()).parentNode.insertBefore(this._svg.getElementById(node.fullName()), this._svg.getElementById(prevNode.fullName()).nextSibling);
                    ej.Diagram.SvgContext.renderSelector(node, this._svg, this._adornerLayer, this._zoomPan.currentZoom,
                        this.model.selectorConstraints);
                }
            }
        },

        undo: function () {
            this._isUndo = true;
            this.historyManager.undo();
            this._isUndo = false;
        },

        redo: function () {
            this._isUndo = true;
            this.historyManager.redo();
            this._isUndo = false;
        },

        cut: function () {
            this._clipboard.cut();
        },

        copy: function () {
            this._clipboard.copy();
        },

        paste: function () {
            this._isUndo = true;
            this._clipboard.paste();
            this._isUndo = false;
        },

        zoomTo: function (zoom) {
            if (!zoom) {
                zoom = new ej.Diagram.Zoom();
            }
            if (zoom.zoomCommand & ej.Diagram.ZoomCommand.ZoomIn) {
                this._zoomPan.zoomPanDelta(0, 0, (1 + zoom.zoomFactor), zoom.focusPoint, true);
            }
            else {
                this._zoomPan.zoomPanDelta(0, 0, (1 - zoom.zoomFactor), zoom.focusPoint, true);
            }
        },

        remove: function () {
            this._delete();
        },

        clear: function () {
            this._clearSelection();
            this.model.nodes.clear();
            this.model.connectors.clear();
            $("#" + this._id + "_canvas_diagramLayer").empty();
        },

        add: function (object) {
            var success;
            this._setZorder(object);
            if (object.type === "node" || object.type === "group") {
                success = this.model.nodes.add(object);
                if (success) {
                    if (object.type === "group") {
                        this._updateChildrenEdges(object);
                        ej.Diagram.SvgContext.renderGroup(object, this._svg, this._diagramLayer, this);
                    }
                    else
                        ej.Diagram.SvgContext.renderNode(object, this._svg, this._diagramLayer, this);
                    this._clearSelection();
                    this._addSelection(object);
                }
            }
            else if (object.type === "connector") {
                success = this.model.connectors.add(object);
                if (success) {
                    this._updateEdges(object);
                    object.dock();
                    ej.Diagram.SvgContext.renderConnector(object, this._svg, this._diagramLayer, this);
                    this._clearSelection();
                    this._addSelection(object);
                }
            }
            return success;
        },

        updateUserHandles: function (node) {
            ej.Diagram.SvgContext.updateUserHandles(this.model.userHandles, node, this._svg, false, false, this._zoomPan.currentZoom);
        },

        scrollToNode: function (node) {
            var nodeX = 0, nodeY = 0;
            var curZoomfactor = this._zoomPan.currentZoom;
            var elementBounds = this.element[0].getBoundingClientRect();
            var nodeBounds = node.getBoundingBox();
            var nodeX = (nodeBounds.x + nodeBounds.width / 2) * curZoomfactor;
            var nodeY = (nodeBounds.y + nodeBounds.height / 2) * curZoomfactor;

            if (elementBounds.width + this._scroller._horizontalOffset < nodeX + nodeBounds.width / 2) {
                this._scroller._horizontalOffset = (nodeX + (nodeBounds.width * curZoomfactor) - elementBounds.width);
            }
            if (elementBounds.height + this._scroller._verticalOffset < nodeY + nodeBounds.height / 2) {
                this._scroller._verticalOffset = (nodeY + (nodeBounds.height * curZoomfactor) - elementBounds.height);
            }
            this._scroller._updateOffset(this._scroller._horizontalOffset, this._scroller._verticalOffset);
        },
        //#endregion

        //#region support methods

        _registerDrawingTools: function () {
            for (var toolname in this.model.drawingTools) {
                var tool = this.model.drawingTools[toolname];
                tool.diagram = this;
                this.tools[tool.name] = tool;
            }
        },

        _sortByXvalue: function (nodes) {
            var temp, i, j;
            var fNode, lNode;
            for (i = 0; i < nodes.length; i++) {
                for (j = i + 1; j < nodes.length; j++) {
                    fNode = nodes[i].getBoundingBox();
                    lNode = nodes[j].getBoundingBox();
                    if (fNode.x > lNode.x) {
                        temp = nodes[i];
                        nodes[i] = nodes[j];
                        nodes[j] = temp;
                    }
                }
            }
            return nodes;
        },

        _sortByYvalue: function (nodes) {
            var temp, i, j;
            var fNode, lNode;
            for (i = 0; i < nodes.length; i++) {
                for (j = i + 1; j < nodes.length; j++) {
                    fNode = nodes[i].getBoundingBox();
                    lNode = nodes[j].getBoundingBox();
                    if (fNode.y > lNode.y) {
                        temp = nodes[i];
                        nodes[i] = nodes[j];
                        nodes[j] = temp;
                    }
                }
            }
            return nodes;
        },

        _sortByZIndex: function (nodes) {
            var temp, i, j;
            for (i = 0; i < nodes.length; i++) {
                for (j = i + 1; j < nodes.length; j++) {
                    if (nodes[i].zOrder < nodes[j].zOrder) {
                        temp = nodes[i];
                        nodes[i] = nodes[j];
                        nodes[j] = temp;
                    }
                }
            }
            return nodes;
        },

        _findCornerNodes: function () {
            var lNode, rNode, tNode, bNode, i, nBounds;
            var list = this.selectionList[0].children;
            var fnode = this.selectionList[0].children[0];
            lNode = fnode, rNode = fnode, tNode = fnode, bNode = fnode;

            if (list.length > 1) {
                for (i = 0; i < list.length; i++) {
                    nBounds = list[i].getBoundingBox();

                    var rNodeBounds = rNode.getBoundingBox();    //Right Node
                    if ((nBounds.x + nBounds.width) > (rNodeBounds.x + rNodeBounds.width))
                        rNode = list[i];

                    var bNodeBounds = bNode.getBoundingBox();    //Bottom Node
                    if ((nBounds.y + nBounds.height) > (bNodeBounds.y + bNodeBounds.height))
                        bNode = list[i];

                    var lNodeBounds = lNode.getBoundingBox();    //Left Node
                    if (nBounds.x < lNodeBounds.x)
                        lNode = list[i];

                    var tNodeBounds = tNode.getBoundingBox();    //top Node
                    if (nBounds.y < tNodeBounds.y)
                        tNode = list[i];

                }
            }
            return { left: lNode, right: rNode, top: tNode, bottom: bNode };
        },

        _moveNode: function (diffX, diffY, node, collection) {
            node._translate(diffX, diffY);
            if (node.type === "connector")
                ej.Diagram.SvgContext.updateConnector(node, this._svg);
            else if (node.type === "node")
                ej.Diagram.SvgContext.updateNode(node, this._svg);
            else if (node.type === "group")
                ej.Diagram.SvgContext.updateGroup(node, this._svg);
            collection.children.add(node);
        },

        _nudge: function (node, x, y) {
            node._translate(x, y);
            if (node.type === "connector")
                ej.Diagram.SvgContext.updateConnector(node, this._svg);
            else if (node.type === "node")
                ej.Diagram.SvgContext.updateNode(node, this._svg);
            else if (node.type === "group" || node.type === "pseudoGroup")
                ej.Diagram.SvgContext.updateGroup(node, this._svg);
            ej.Diagram.SvgContext.updateSelector(node, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
        },

        _findObjectByName: function (name) {
            var node = null;
            if (this._findNode(name))
                node = this._findNode(name);
            if (this._findConnector(name))
                node = this._findConnector(name);
            return node;
        },

        _findObjectByIndex: function (index, group) {
            var nodes, findNode, node, m, k;
            if (group) {
                nodes = this.selectionList[0].parent.children;
                for (k = 0; k < nodes.length; k++) {
                    if (nodes[k].zOrder === index)
                        findNode = nodes[k];
                }
            } else {
                for (m = 0; m < this.model.nodes.length; m++) {
                    node = this.model.nodes[m];
                    if (node.zOrder === index)
                        findNode = node;
                }
                for (m = 0; m < this.model.connectors.length; m++) {
                    node = this.model.connectors[m];
                    if (node.zOrder === index)
                        findNode = node;
                }
            }

            return findNode;
        },

        _findOverlapNode: function (node) {
            var oNodes = [], m, i, rNodes = [];
            if (!node.parent) {
                for (m = 0; m < this.model.nodes.length; m++) {
                    oNodes.push(this.model.nodes[m]);
                }
                for (m = 0; m < this.model.connectors.length; m++) {
                    oNodes.push(this.model.connectors[m]);
                }
                var nBound = node.getBoundingBox();
                if (node) {
                    for (i = 0; i < oNodes.length; i++) {
                        var chBound = oNodes[i].getBoundingBox();
                        if (node.name !== oNodes[i].name) {
                            if (this._intersect(nBound, chBound))
                                rNodes.push(oNodes[i]);
                        }
                    }
                }
            } else {
                for (m = 0; m < node.parent.children.length; m++) {
                    if (node.parent.children[m].name !== node.name)
                        rNodes.push(node.parent.children[m]);
                }
            }
            return rNodes;
        },

        _intersect: function (a, b) {
            return (a.x <= (b.x + b.width) &&
                b.x <= (a.x + a.width) &&
                a.y <= (b.y + b.height) &&
                b.y <= (a.y + a.height));
        },
        //#endregion 

        //#region Rendering
        _createTooltipDiv: function () {
            var div = document.createElement("div");
            var attr = {
                "id": this.element[0].id + "_tooltipDiv",
                "class": "e-diagram-tooltip-default", "style": "padding-top:3px; height: 24px;",
            };
            ej.Diagram.Util.attr(div, attr);
            this._canvas.appendChild(div);
            return div;
        },

        _renderTooltip: function (object) {
            if (this.model.visualGuide && this.model.enableVisualGuide && $.templates) {
                var tooltipDiv = $("#" + this.element[0].id + "_tooltipDiv")[0];
                if (!tooltipDiv) {
                    tooltipDiv = this._createTooltipDiv();
                }
                var template;
                var objectValues;
                var html;
                if (this.model.visualGuide.type & ej.Diagram.VisualGuideType.Tooltip) {
                    var curZoomfactor = this._zoomPan.currentZoom;
                    if (this.model.visualGuide.tooltipTemplateId === "") {
                        template = $.templates("#" + this.element[0].id + "_defaultTooltip");
                        if (this.activeTool instanceof ej.Diagram.MoveTool) {
                            tooltipDiv.style.width = "150px";
                            objectValues = [{ type: "X : ", type1: "Y : ", x: Math.round((object.offsetX - object.width / 2)) + "px  ", y: Math.round((object.offsetY - object.height / 2)) + "px" }];
                        }
                        else if (this.activeTool instanceof ej.Diagram.ResizeTool) {
                            tooltipDiv.style.width = "150px";
                            objectValues = [{ type: "W : ", type1: "H : ", x: Math.round(object.width) + "px  ", y: Math.round(object.height) + "px" }];
                        }
                        else if (this.activeTool instanceof ej.Diagram.RotateTool) {
                            tooltipDiv.style.width = "50px";
                            objectValues = [{ type: "", type1: "", x: Math.round(object.rotateAngle) + "&deg; ", y: "" }];
                        }
                        if (objectValues) {
                            tooltipDiv.style.top = -this._scroller._verticalOffset + ((object.offsetY * curZoomfactor - (object.height / 2) * curZoomfactor) + (object.height * curZoomfactor) + 15) + "px";
                            tooltipDiv.style.left = -this._scroller._horizontalOffset + ((object.offsetX * curZoomfactor - (object.width / 2) * curZoomfactor)) + (object.width / 2 * curZoomfactor - $(tooltipDiv).width() / 2) + "px";
                            html = template.render(objectValues);
                            $(tooltipDiv).html(html);
                        }
                        else {
                            this._removeTooltip();
                        }
                    }
                    else {
                        tooltipDiv.style.top = -this._scroller._verticalOffset + ((object.offsetY * curZoomfactor - (object.height / 2) * curZoomfactor) + (object.height * curZoomfactor) + 15) + "px";
                        tooltipDiv.style.left = -this._scroller._horizontalOffset + ((object.offsetX * curZoomfactor - (object.width / 2) * curZoomfactor)) + (object.width / 2 * curZoomfactor - $(tooltipDiv).width() / 2) + "px";
                        template = $.templates("#" + this.model.visualGuide.tooltipTemplateId);
                        objectValues = [{ name: this.activeTool.selectedObject.name, x: (object.offsetX - object.width / 2) + "px", y: (object.offsetY - object.height / 2) + "px", width: object.width + "px", height: object.height + "px", rotationAngle: object.rotateAngle }];
                        html = template.render(objectValues);
                        $(tooltipDiv).html(html);
                    }
                }
            }
        },

        _removeTooltip: function () {
            var tooltipDiv = document.getElementById(this.element[0].id + "_tooltipDiv");
            if (tooltipDiv !== null) {
                tooltipDiv.parentNode.removeChild(tooltipDiv);
            }
        },

        _renderCanvas: function () {
            this._scroller = new ej.Diagram.Scroller(this);
            this._snapping = new ej.Diagram.Snapping(this);
            this._canvas = document.createElement("div");
            this._canvas.setAttribute("id", this.element[0].id + "_canvas");
            this._canvas.setAttribute("class", "drawing");
            this._canvas.setAttribute("style", "position:relative; height:" + this.model.height + "; width:" + this.model.width + ";");
            this.element.css({
                overflow: "hidden", outline: "none", display: "block"
            });
            this.element[0].setAttribute("tabindex", 0);
            this.element.append(this._canvas);
            this._zoomPan = new ej.Diagram.ZoomAndPan(this);
            this._renderDiagram();
            this._scroller._render = false;
            this._scroller._renderScroller();
            this._svgWidth = this._scroller._viewPortWidth;
            this._svgHeight = this._scroller._viewportHeight;
            this._renderTooltipTemplate();
            this.model.pageSettings._updatePageSize(this);
            this.updateViewPort();
        },

        _getDigramBounds: function () {
            var left, top, right, bottom;
            left = Math.max(0, this._spatialsearch._pageLeft);
            top = Math.max(0, this._spatialsearch._pageTop);
            if (!this.model.pageSettings.pageWidth) {
                right = this._spatialsearch._pageRight;
            }
            if (!this.model.pageSettings.pageHeight) {
                bottom = this._spatialsearch._pageBottom;
            }
            if (this.model.pageSettings && this.model.pageSettings.multiplePage) {
                left = Math.max(0, this._spatialsearch._pageLeft);
                top = Math.max(0, this._spatialsearch._pageTop);
                if (this.model.pageSettings.pageWidth) {
                    left = Math.floor(left / this.model.pageSettings.pageWidth) * this.model.pageSettings.pageWidth;
                    right = Math.ceil(this._spatialsearch._pageRight / this.model.pageSettings.pageWidth) * this.model.pageSettings.pageWidth;
                }
                if (this.model.pageSettings.pageHeight) {
                    top = Math.floor(top / this.model.pageSettings.pageHeight) * this.model.pageSettings.pageHeight;
                    bottom = Math.ceil(this._spatialsearch._pageBottom / this.model.pageSettings.pageHeight) * this.model.pageSettings.pageHeight;
                }
                if (this._spatialsearch._pageRight === null) {
                    if (this.model.pageSettings.pageWidth !== 0) {
                        right = this.model.pageSettings.pageWidth;
                    }
                }
                if (this._spatialsearch._pageBottom === null) {
                    if (this.model.pageSettings.pageHeight !== 0) {
                        bottom = this.model.pageSettings.pageHeight;
                    }
                }
            }
            else {
                if (this.model.pageSettings.pageWidth) {
                    left = Math.min(0, this._spatialsearch._pageLeft);
                    right = this.model.pageSettings.pageWidth;
                }
                if (this.model.pageSettings.pageHeight) {
                    top = Math.min(0, this._spatialsearch._pageTop);

                    bottom = this.model.pageSettings.pageHeight;
                }
            }
            var svgBounds = new ej.Diagram.Rectangle();
            svgBounds.x = left;
            svgBounds.y = top;
            svgBounds.width = right;
            svgBounds.height = bottom;
            return svgBounds;
        },

        _initContextMenu: function () {
            if (this.model.enableContextMenu) {
                if (this.model.contextMenu && this.model.contextMenu.items.length > 0) {
                    this.model.contextMenu = new ej.Diagram.ContextMenu(this.model.contextMenu);
                    this._renderContextMenu();
                }
            }
            else {
                this.element.remove("#" + this.element[0].id + "_contextMenu");
            }
        },

        _renderContextMenuItem: function (item, ul) {
            var li = document.createElement("li");
            var a = document.createElement("a");
            a.innerHTML = item.text;
            li.setAttribute("id", this.element[0].id + "_contextMenu_" + item.name);

            var div = document.createElement("span");
            div.setAttribute("id", item.text + "_image");
            div.setAttribute("style", "margin-top:-5px;");
            div.setAttribute("class", "sf-d-menuitem " + item.name);
            a.appendChild(div);
            if (item.subItems.length > 0) {
                var subUl = document.createElement("ul");
                var i = 0, subli;
                for (i; i < item.subItems.length; i++) {
                    subli = this._renderContextMenuItem(item.subItems[i], subUl);
                    subUl.appendChild(subli);
                }
                li.appendChild(subUl);
            }
            li.appendChild(a);
            return li;
        },

        _renderContextMenu: function () {
            if (this.model.enableContextMenu) {
                var menuCollection = this.model.contextMenu.items;
                var i = 0, item, mItem;
                var ul = document.createElement("ul");
                ul.setAttribute("id", this.element[0].id + "_contextMenu");
                for (i; i < menuCollection.length; i++) {
                    item = menuCollection[i];
                    mItem = this._renderContextMenuItem(item, ul);
                    ul.appendChild(mItem);
                }
                this._canvas.appendChild(ul);
                $("#" + this.element[0].id + "_contextMenu").ejMenu({
                    menuType: ej.menuType.ContextMenu,
                    contextTargetId: this.element[0].id + "_canvas_svg",
                    click: $.proxy(this._onMenuItemClick, this),
                    beforeContextOpen: $.proxy(this._onContextMenuOpen, this),
                    mouseOver: $.proxy(this._onMenuItemMouseOver, this),
                    mouseOut: $.proxy(this._onMenuItemMouseOut, this),
                    contextClose: $.proxy(this._onContextMenuClose, this),
                });
            }
        },
        _onContextMenuClose: function (args) {
            this.element[0].focus();
        },

        _onContextMenuOpen: function (args) {
            var diagramId = this.element[0].id;
            if (this.model.contextMenu._isDefaultLoaded) {
                document.getElementById(diagramId + "_contextMenu_cut").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_copy").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_paste").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_undo").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_redo").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_selectAll").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_cut").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_grouping").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_order").style.display = "block";
                if (this.selectionList.isEmpty()) {
                    document.getElementById(diagramId + "_contextMenu_cut").style.display = "none";
                    document.getElementById(diagramId + "_contextMenu_copy").style.display = "none";
                    document.getElementById(diagramId + "_contextMenu_cut").style.display = "none";
                    document.getElementById(diagramId + "_contextMenu_grouping").style.display = "none";
                    document.getElementById(diagramId + "_contextMenu_order").style.display = "none";
                }
            }
            var obj = ({ diagram: this, contextmenu: args });
            this._trigger("contextMenuBeforeOpen", obj);
        },

        _onMenuItemClick: function (args) {
            var option = args.Events.menuText;
            switch (option) {
                case "Cut":
                    this.cut();
                    break;
                case "Copy":
                    this.copy();
                    break;
                case "Paste":
                    this.paste();
                    break;
                case "Undo":
                    this.undo();
                    break;
                case "Redo":
                    this.redo();
                    break;
                case "Select All":
                    this.selectAll();
                    break;
                case "Group":
                    this.group();
                    break;
                case "Ungroup":
                    this.ungroup();
                    break;
                case "Bring To Front":
                    this.bringToFront();
                    break;
                case "Send To Back":
                    this.sendToBack();
                    break;
                case "Move Forward":
                    this.moveForward();
                    break;
                case "Send Backward":
                    this.sendBackward();
                    break;
                default:
                    break;
            }
            this._trigger("contextMenuClick", args);
        },

        _onMenuItemMouseOver: function (args) {
            var t = args.Events.element.lastChild.childNodes[1];
            $(t).addClass("hover");
        },

        _onMenuItemMouseOut: function (args) {
            var t = args.Events.element.lastChild.childNodes[1];
            $(t).removeClass("hover");
        },

        _renderDiagram: function () {
            if (window.SVGSVGElement) {
                var attr = {
                    id: this._canvas.id + "_svg",
                    width: this.model.width,
                    height: this.model.height,
                    version: "1.1",
                    xlink: "http://www.w3.org/1999/xlink",
                    "style": "position:absolute"
                };

                this._svg = new ej.Diagram.Svg(attr);
                this._canvas.appendChild(this._svg.document);
                ej.Diagram.SvgContext._createScroller(this._canvas);
                this._renderView();
                this._renderPage();
                this._renderHtmlLayer();
                this._renderBackground();
                this._renderBackgroundLayer();
                this._renderGrid();
                this._renderDiagramLayer();
                this._renderAdornerLayer();
                this._renderNodes();
                this._renderConnectors();
                if (this.model.layout) {
                    this._initLayout();
                }
            }
        },

        _doLayout: function () {
            var layout;
            if (this.model.layout.type === "hierarchicalTree") {
                layout = this.model.layout = new ej.Diagram.HierarchicalLayout(this.model.layout);
                if (layout.fixedNode != null) {
                    layout.fixedNode = this.model.layout.fixedNode = this._findNode(layout.fixedNode.name);
                }
            }
            layout._setModel(this.model, this);
            this._updateEdgeCollection();
            layout.doLayout();
        },

        _updateNodes: function () {
            ej.Diagram.SvgContext.updateNodes(this.model.nodes, this._svg);
        },

        _updateConnectors: function () {
            ej.Diagram.SvgContext.updateConnectors(this.model.connectors, this._svg);
        },

        _updateEdgeCollection: function () {
            for (var i = 0; i < this.model.connectors.length; i++) {
                this._updateEdges(this.model.connectors[i]);
            }
        },

        updateViewPort: function () {
            this._scroller._updateViewPort();
            this.model.pageSettings._updatePageSize(this, true);
        },

        _renderView: function () {
            var svg = this._svg;
            var attr = { id: this._canvas.id + "_view" };
            this._view = svg.g(attr);
            this._svg.appendChild(this._view);

        },

        _renderHtmlLayer: function () {
            var div = document.createElement("div");
            var attr = { "id": this._canvas.id + "_htmlLayer", "class": "htmlLayer" };
            ej.Diagram.Util.attr(div, attr);
            this._htmlLayer = div;
            div.style.pointerEvents = "none";
            this._canvas.appendChild(this._htmlLayer);
        },

        _renderPage: function () {
            var svg = this._svg;
            var attr = { "id": this._canvas.id + "_page", "class": "page" };
            this._page = svg.g(attr);
            this._view.appendChild(this._page);
        },

        _renderDiagramLayer: function () {
            var svg = this._svg;
            var attr = { "id": this._canvas.id + "_diagramLayer", "class": "DiagramLayer" };
            this._diagramLayer = svg.g(attr);
            this._page.appendChild(this._diagramLayer);
        },

        _renderAdornerLayer: function () {
            var svg = this._svg;
            var attr = { "id": this._canvas.id + "_adornerLayer", "class": "AdornerLayer" };
            this._adornerLayer = svg.g(attr);
            this._page.appendChild(this._adornerLayer);
        },

        _renderBackgroundLayer: function () {
            var svg = this._svg;
            var attr = {
                "id": this._canvas.id + "_pageBackground",
                "class": "PageBackgroundLayer"
            };
            this._pageBackgroundLayer = svg.g(attr);
            this._page.appendChild(this._pageBackgroundLayer);
        },

        _renderTooltipTemplate: function () {
            if (!this.visualGuide) {
                this.visualGuide = ej.Diagram.VisualGuides.Tooltip | ej.Diagram.VisualGuides.Alignment;
            }
            if (this.visualGuide & ej.Diagram.VisualGuides.Tooltip) {
                if (!this.visualGuideTooltipId) {
                    var script = document.createElement("script");
                    var attr = {
                        "id": this.element[0].id + "_guideTooltip",
                        "type": "text/x-jsrender",
                    };
                    ej.Diagram.Util.attr(script, attr);
                    script.innerHTML = "<label class=\"e-diagram-tooltipLabel-default\"> {{:type}}:{{:x}}   {{:type1}}:{{:y}}</label>";
                    this._canvas.appendChild(script);
                }
            }
        },

        _renderBackground: function () {
            var svg = this._svg;
            svg.document.style.msTouchAction = "none";
            svg.document.style.display = "block";
            svg.document.style.backgroundColor = "white";
            if (this.model.backgroundImage) {
                var attr = { id: this._canvas.id + "_backgroundLayer" };
                var g = svg.g(attr);
                attr = { "id": this._canvas.id + "_background", "x": "0", "y": "0", "width": "100%", "height": "100%", "preserveAspectRatio": "none", "pointer-events": "none" };
                var image = svg.image(attr);
                image.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", this.model.backgroundImage);
                if (!this._svg.getElementById(this._canvas.id + "_background")) {
                    g.appendChild(image);
                    this._page.appendChild(g);
                }
            }
        },

        _renderGrid: function () {
            var svg = this._svg;
            var defs = svg.defs({ "id": this._canvas.id + "patterndefinition" });
            var g;
            svg.appendChild(defs);
            if (this.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.ShowHorizontalLines ||
                this.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.ShowVerticalLines) {
                var grid = document.getElementById(this._canvas.id + "_grid");
                var rect = document.getElementById(this._canvas.id + "pattern_gridline");
                var pattern = document.getElementById(this._canvas.id + "pattern_grid");
                if (pattern)
                    defs.removeChild(pattern);
                var attr = { id: this._canvas.id + "_grid", "pointer-events": "none" };
                g = svg.g(attr);
                var width = 0;
                if (this.model.snapSettings.horizontalGridlines.linesInterval) {
                    this.model.snapSettings.horizontalGridlines.linesInterval.forEach(function (entry) {
                        width += entry;
                    });
                }
                var height = 0;
                if (this.model.snapSettings.verticalGridlines.linesInterval) {
                    this.model.snapSettings.verticalGridlines.linesInterval.forEach(function (entry) {
                        height += entry;
                    });
                }
                var scale = this._zoomPan.currentZoom;
                attr = { id: this._canvas.id + "pattern_grid", x: 0, y: 0, width: width * scale, height: height * scale, patternUnits: "userSpaceOnUse" };
                var pattern = this._svg.pattern(attr);
                scale = this._scaleSnapInterval(scale);
                this._renderHorizontalGridlines(scale, pattern, width);
                this._renderVerticalGridlines(scale, pattern, height);
                defs.appendChild(pattern);
                var rect = svg.rect({ id: this._canvas.id + "pattern_gridline", "x": 0, "y": 0, "width": "100%", "height": "100%", "fill": "url(#" + this._canvas.id + "pattern_grid)" });
                g.appendChild(rect);
                if (!grid)
                    this._page.appendChild(g);
            }
            else {
                g = document.getElementById(this._canvas.id + "_grid");
                if (g) {
                    var grid = document.getElementById(this._canvas.id + "pattern_gridline");
                    if (grid)
                        g.removeChild(grid);
                }
            }
        },

        _updateGrid: function (hoffset, voffset, scale) {
            var grid = document.getElementById(this._canvas.id + "pattern_gridline");
            var i;
            if (grid) {
                var pattern = document.getElementById(this._canvas.id + "pattern_grid");
                var width = 0;
                if (this.model.snapSettings.horizontalGridlines.linesInterval) {
                    for (i = 0; i < this.model.snapSettings.horizontalGridlines.linesInterval.length; i++) {
                        width += this.model.snapSettings.horizontalGridlines.linesInterval[i];
                    }
                }
                var height = 0;
                if (this.model.snapSettings.verticalGridlines.linesInterval) {
                    for (i = 0; i < this.model.snapSettings.verticalGridlines.linesInterval.length; i++) {
                        height += this.model.snapSettings.verticalGridlines.linesInterval[i];
                    }
                }

                scale = this._scaleSnapInterval(scale);
                var svg = this._svg;
                var defs = svg.defs({ "id": this._canvas.id + "patterndefinition" });
                if (pattern)
                    defs.removeChild(pattern);
                var attr = { id: this._canvas.id + "pattern_grid", x: 0, y: 0, width: width * scale, height: height * scale, patternUnits: "userSpaceOnUse" };
                pattern = this._svg.pattern(attr);
                defs.appendChild(pattern);
                this._renderHorizontalGridlines(scale, pattern, width);
                this._renderVerticalGridlines(scale, pattern, height);

            }
        },

        _scaleSnapInterval: function (scale) {
            if (scale >= 2) {
                while (scale >= 2) {
                    scale /= 2;
                }
            }
            else if (scale <= 0.5) {
                while (scale <= 0.5) {
                    scale *= 2;
                }
            }
            var i;
            if (scale !== 1) {
                this.model.snapSettings.horizontalGridlines._snapinterval = [];
                for (i = 0; i < this.model.snapSettings.horizontalGridlines.snapInterval.length; i++) {
                    this.model.snapSettings.horizontalGridlines._snapinterval[i] =
                        this.model.snapSettings.horizontalGridlines.snapInterval[i] * scale;
                }
                this.model.snapSettings.verticalGridlines._snapinterval = [];
                for (i = 0; i < this.model.snapSettings.verticalGridlines.snapInterval.length; i++) {
                    this.model.snapSettings.verticalGridlines._snapinterval[i] =
                        this.model.snapSettings.verticalGridlines.snapInterval[i] * scale;
                }
            }
            return scale;
        },

        _renderHorizontalGridlines: function (scale, pattern, height) {
            var snapSettings = this.model.snapSettings;
            if (snapSettings.snapConstraints & ej.Diagram.SnapConstraints.ShowHorizontalLines) {
                var space = 0;
                var cnt = 0;
                for (var i = 0; i < snapSettings.horizontalGridlines.linesInterval.length; i = i + 2) {
                    var thickness = snapSettings.horizontalGridlines.linesInterval[i];
                    var line = this._svg.line({
                        "x1": space * scale, "y1": 0, "x2": space * scale, "y2": height * scale, "stroke": snapSettings.horizontalGridlines.lineColor,
                        "stroke-width": thickness, "stroke-dasharray": snapSettings.horizontalGridlines.lineDashArray
                    });
                    space += snapSettings.horizontalGridlines.linesInterval[i + 1] + thickness;
                    pattern.appendChild(line);
                    cnt++;
                }
            }
        },

        _renderVerticalGridlines: function (scale, pattern, width) {
            var snapSettings = this.model.snapSettings;
            if (snapSettings.snapConstraints & ej.Diagram.SnapConstraints.ShowVerticalLines) {
                var space = 0;
                var cnt = 0;
                for (var i = 0; i < snapSettings.verticalGridlines.linesInterval.length; i = i + 2) {
                    var thickness = this.model.snapSettings.verticalGridlines.linesInterval[i];
                    var line = this._svg.line({
                        "x1": 0, "y1": space * scale, "x2": width * scale, "y2": space * scale, "stroke": snapSettings.verticalGridlines.lineColor,
                        "stroke-width": thickness, "stroke-dasharray": snapSettings.verticalGridlines.lineDashArray
                    });
                    space += snapSettings.verticalGridlines.linesInterval[i + 1] + thickness;
                    pattern.appendChild(line);
                    cnt++;
                }
            }
        },


        _setZorder: function (node) {
            if (node) {
                node.zOrder = this._zOrder;
                this._zOrder++;
                if (node.type === "group") {
                    var i = 0;
                    for (i; i < node.children.length; i++) {
                        node.children[i].zOrder = i;
                    }
                }
            }
        },
        _renderNodes: function () {
            if (this.model.nodes) {
                var nodes = this.model.nodes;
                for (var i = 0, len = nodes.length; i < len; ++i) {
                    var node = nodes[i];
                    this._setZorder(node);
                    if (node.type === "group") {
                        ej.Diagram.SvgContext.renderGroup(node, this._svg, this._diagramLayer, this);
                    }
                    else {
                        ej.Diagram.SvgContext.renderNode(node, this._svg, this._diagramLayer, this);
                        node.updatebounds();
                    }
                }
            }
        },

        _renderConnectors: function () {
            if (this.model.connectors) {
                var connectors = this.model.connectors;
                var connector;
                for (var i = 0, len = connectors.length; i < len; ++i) {
                    connector = connectors[i];
                    this._setZorder(connector);
                    this._updateEdges(connector);
                    connector.dock();
                    ej.Diagram.SvgContext.renderConnector(connector, this._svg, this._diagramLayer, this);
                }
            }
        },

        //#endregion

        //#region Events
        _wireEvents: function () {
            //svg document events
            var svgDocument = $(this._canvas);
            this._on(svgDocument, ej.eventType.mouseDown, this._mousedown);
            this._on(svgDocument, ej.eventType.mouseMove, this._mousemove);
            this._on(svgDocument, ej.eventType.mouseUp, this._mouseup);
            this._on(svgDocument, ej.eventType.mouseLeave, this._documentmouseup);
            this._on(svgDocument, "dblclick", this._doubleclick);
            this._on(svgDocument, "doubletap", this._doubleclick);

            this._on(this.element, "keydown", this._keydown);
            //this._on(this.element, "contextmenu", this._contextmenu);
            this._on(svgDocument, "pinchin", this._pinchin);
            this._on(svgDocument, "pinchout", this._pinchout);
            this._on(this.element, "touchstart", this.handleTouchStart);
            if (ej.browserInfo().name === "mozilla") {
                this._on(this.element, "DOMMouseScroll", this._handleMouseWheel);
            }
            else
                this._on(this.element, "mousewheel", this._handleMouseWheel);
            //window document event
            this._on($(document), "mouseup", this._documentmouseup);

            if (window) {
                this._on($(window), "resize", this.updateViewPort);
            }
            var obj = this;
            $(svgDocument).ejDroppable({
                drop: function (event, ui) {
                    obj._svgdrop(event, ui);
                    obj._isDropped = true;
                }
            });
        },

        _mousePosition: function (evt, exOffset) {
            var e = this._isTouchEvent(evt);
            if (!e) {
                e = evt;
            }
            var scrollLeft = this._canvas.scrollLeft;
            var scrollTop = this._canvas.scrollTop;
            var controlBBox = this._canvas.getBoundingClientRect();
            var layerx = (e.clientX + scrollLeft) - controlBBox.left;
            var layery = (scrollTop + e.clientY) - controlBBox.top;
            if (!exOffset) {
                layerx = (layerx + this._scroller._horizontalOffset) / this._zoomPan.currentZoom;
                layery = (layery + this._scroller._verticalOffset) / this._zoomPan.currentZoom;
            }
            return new ej.Diagram.Point(layerx, layery);
        },

        _contextmenu: function (evt) {
            evt.preventDefault();
            evt.stopPropagation();
        },

        _pinchin: function (evt) {
            evt.preventDefault();
        },

        _pinchout: function (evt) {
            evt.preventDefault();
        },

        handleTouchStart: function (evt) {
            evt.preventDefault();
        },

        _mousedown: function (evt) {
            if (this._invoke(evt)) {
                evt.preventDefault();
                if (this.model.contextMenu && this.model.contextMenu._isContextMenuOpen) {
                    evt.cancel = true;
                }
                if (this._isEditing)
                    this._endEdit();
                if (evt.which == "1" || (evt.which == 0 && evt.originalEvent instanceof TouchEvent)) {
                    this.activateTool(this._toolToActivate);
                    this.activeTool.mousedown(evt);
                }
            }
        },

        _invoke: function (evt) {
            var className = evt.originalEvent.target.className;
            if (evt.originalEvent.target && (className == "edit" || className == "editBox" || className == "e-hhandle" || className == "e-hhandlespace" || className == "e-vhandle" || className == "e-vhandlespace")) {
                return false;
            }

            return true;
        },

        _handleMouseWheel: function (evt) {
            if (evt.originalEvent.preventDefault)
                evt.originalEvent.preventDefault();
            var position = new ej.Diagram.Point(0, 0);
            this._toolToActivate = "select";
            this.activeTool = this.tools[this._toolToActivate];
            if (evt.ctrlKey) {
                position.x = (evt.originalEvent.layerX + this._scroller._horizontalOffset) / this._zoomPan.currentZoom;
                position.y = (evt.originalEvent.layerY + this._scroller._verticalOffset) / this._zoomPan.currentZoom;
                if (evt.originalEvent.wheelDelta > 0 || -40 * evt.originalEvent.detail > 0) {
                    this._zoomPan.zoomPanDelta(0, 0, (1 + 0.2), position, true);
                }
                else {
                    this._zoomPan.zoomPanDelta(0, 0, (1 - 0.2), position, true);
                }
            }
            else {
                var offsetY = 0;
                if (evt.originalEvent.wheelDelta > 0 || -40 * evt.originalEvent.detail > 0) {
                    if ((this._scroller._verticalOffset - this._scroller._scrollPixel) >= 0) {
                        this._scroller._updateOffset(this._scroller._horizontalOffset, (this._scroller._verticalOffset - this._scroller._scrollPixel));
                        offsetY = -this._scroller._scrollPixel;
                    }
                    else {
                        this._scroller._updateOffset(this._scroller._horizontalOffset, 0);
                        offsetY = 0;
                    }

                }
                else {
                    if (document.getElementById(this.element[0].id).getElementsByClassName('e-vhandle').length > 0) {
                        if ((this._scroller._extendedHeight - this._scroller._viewportHeight) >= (this._scroller._verticalOffset + this._scroller._scrollPixel)) {
                            this._scroller._updateOffset(this._scroller._horizontalOffset, (this._scroller._verticalOffset + this._scroller._scrollPixel));
                            offsetY = this._scroller._scrollPixel;
                        }
                        else if ((this._scroller._extendedHeight - this._scroller._viewportHeight) >= (this._scroller._verticalOffset)) {
                            var v = (this._scroller._extendedHeight - this._scroller._viewportHeight);
                            var o = this._scroller._verticalOffset;
                            this._scroller._updateOffset(this._scroller._horizontalOffset, this._scroller._verticalOffset + (v - o));
                            offsetY = 0;
                        }
                    }
                }
                this._updateEditBox(0, offsetY);
            }
        },

        bringIntoCenter: function (rect) {
            var scale = this._zoomPan.currentZoom;
            var hoffset = rect.x + (rect.right() - rect.x) / 2 - this._scroller._viewPortWidth / 2;
            var voffset = rect.y + (rect.bottom() - rect.y) / 2 - this._scroller._viewPortHeight / 2;
            hoffset = Math.max(0, hoffset) * scale;
            voffset = Math.max(0, voffset) * scale;
            this._zoomPan._applyTransform(scale, new ej.Diagram.Point(-hoffset, -voffset), null);
        },

        bringIntoView: function (bounds) {
            var x, y;
            var hoffset = this._scroller._horizontalOffset;
            var voffset = this._scroller._verticalOffset;
            var view_width = this._scroller._viewPortWidth;
            var view_height = this._scroller._viewPortHeight;
            if (bounds.right() > (-hoffset + view_width)) {
                x = bounds.right() - view_width;
            }
            if (bounds.x < -hoffset) {
                x = bounds.x;
            }
            if (bounds.bottom() > (-voffset + view_height)) {
                y = bounds.bottom() - view_height;
            }
            if (bounds.y < -voffset) {
                y = bounds.y;
            }
            this._zoomPan._applyTransform(this._zoomPan.currentZoom, new ej.Diagram.Point(-x, -y), null);

        },

        _mousemove: function (evt) {
            var handleName = null;
            var node = null;
            var point = this._mousePosition(evt);
            var isUserHandle = this._isUserHandle(evt);
            if ((this._isInternalTool(this.activeTool) && !this.activeTool.inAction)) {
                if (!isUserHandle) {
                    node = this._findNodeUnderMouse(evt);
                    if (!node) {
                        this._raiseMouseEvents(node);
                    }
                }
                if (!node && this.selectionList[0]) {
                    node = this.selectionList[0];
                }
                this._checkToolToActivate(evt, node, point);
            }
            else {
                if (!this.activeTool.inAction && !this.activeTool.singleAction) {
                    this._toolToActivate = this.activeTool.name;
                    this._currentCursor = this.activeTool.cursor;
                }
                node = this._findNodeUnderMouse(evt);
                if (!node) {
                    this._raiseMouseEvents(node);
                }
            }
            this._updateCursor();
            this.activeTool.mousemove(evt);
            if (this.activeTool.inAction && this.model.autoScroll) {
                var scrollbarSize = 17;
                var canvas = this.element[0];
                var pt = this._mousePosition(evt, true);
                if (pt.x + 15 >= this._scroller._viewPortWidth - 15) {
                    this._beginAutoScroll("right", evt);
                    this._canAutoScroll = true;
                }
                else if (pt.x <= 15 && this._scroller._horizontalOffset > 0) {
                    this._beginAutoScroll("left", evt);
                    this._canAutoScroll = true;
                }
                else if (pt.y + 15 >= this._scroller._viewportHeight - 15) {
                    this._beginAutoScroll("bottom", evt);
                    this._canAutoScroll = true;
                }
                else if (pt.y <= 15 && this._scroller._verticalOffset > 0) {
                    this._beginAutoScroll("top", evt);
                    this._canAutoScroll = true;
                }
                else {
                    this._canAutoScroll = false;
                }
            }
        },

        _setActiveTool: function (evt) {
            var point = this._mousePosition(evt);
            if (!this.activeTool.inAction && (this._isUserHandleActive(this.activeTool.name) || !this.model.userHandles || !this.model.userHandles.length)) {
                if (this._isInternalTool(this.activeTool) || this.activeTool.singleAction) {
                    var node = null;
                    var success = false;
                    //var handle = this._isHandle(evt);
                    if (this.selectionList[0]) {
                        node = this.selectionList[0];
                        success = this._checkToolToActivate(evt, node, point);
                    }
                    if (!success) {
                        node = this._findNodeUnderMouse(evt);
                        if (this._isUserHandle(evt)) {
                            var handleName = $(evt.target).parents("g").first().context.id.split("_")[0];
                            this._checkToolToActivate(evt, node, point, handleName);
                            this.activeTool = this.tools[this._toolToActivate];
                        }
                        else {
                            this._checkToolToActivate(evt, node, point);
                            this._raiseMouseEvents(node);
                            if (this._isUserHandleActive(this.activateTool.name) && !this._drawingTool)
                                this.activeTool = this.tools[this._toolToActivate];
                        }
                    }
                }
            }
        },
        _beginAutoScroll: function (option, evt) {
            var callBack = this;
            var canvas = this.element[0];
            var left = 0, top = 0;
            if (this._canAutoScroll) {
                switch (option) {
                    case "right":
                        if (this._scroller._horizontalOffset + this._scroller._viewPortWidth < this._scroller._extendedWidth) {
                            left = 10;
                        }
                        break;
                    case "left":
                        if (this._scroller._horizontalOffset - 10 >= 0) {
                            left = -10;
                        }
                        break;
                    case "bottom":
                        if (this._scroller._verticalOffset + this._scroller._viewportHeight < this._scroller._extendedHeight) {
                            top = 10;
                        }
                        break;
                    case "top":
                        if (this._scroller._verticalOffset - 10 >= 0) {
                            top = -10;
                        }
                        break;
                }
                setTimeout(function () {
                    callBack._updateScrollBar(left, top, option, evt);
                }, 100);
            }
        },

        _updateScrollBar: function (x, y, option, evt) {
            if (x === 0 && y === 0) {
                this._canAutoScroll = false;
            }
            else if (this.activeTool.inAction) {
                if (this._enableAutoScroll(x, y)) {
                    this._scroller._horizontalOffset += (x * this._zoomPan.currentZoom);
                    this._scroller._verticalOffset += (y * this._zoomPan.currentZoom);
                    this._updateSelectionOnScroll(x, y);
                    this._scroller._updateOffset(this._scroller._horizontalOffset, this._scroller._verticalOffset);
                    this._beginAutoScroll(option, evt);
                }
            }
        },

        _enableAutoScroll: function (x, y) {
            var enableScroll = true;
            if (x < 0 && this._scroller._horizontalOffset + x < 0) {
                enableScroll = false;
            }
            if (y < 0 && this._scroller._verticalOffset + y < 0) {
                enableScroll = false;
            }
            if (this.activeTool.name === "select" && this._scroller._verticalOffset + this._scroller._viewPortHeight + y > this._scroller._extendedHeight) {
                enableScroll = false;
            }
            if (this.activeTool.name === "select" && this._scroller._horizontalOffset + this._scroller._viewPortWidth + x > this._scroller._extendedWidth) {
                enableScroll = false;
            }
            if (this.activeTool.name === "endPoint") {
                enableScroll = false;
            }
            return enableScroll;
        },

        _updateSelectionOnScroll: function (x, y) {
            var tool = this.activeTool;
            if (tool.name === "select") {
                tool.currentPoint = new ej.Diagram.Point(tool.currentPoint.x + x, tool.currentPoint.y + y);
                tool._updateHelper();
            }
            else if (!this.selectionList.isEmpty()) {
                var node = this.selectionList[0];
                tool.currentPoint.x = (tool.currentPoint.x + x);
                tool.currentPoint.y = (tool.currentPoint.y + y);
                if (tool.name === "move") {
                    node._translate(x, y);
                }
                else if (tool.name === "resize") {
                    this._snapping._removeGuidelines();
                    tool._updateSize(node, tool.previousPoint, tool.currentPoint, x, y);
                }
                if (node.type === "connector")
                    ej.Diagram.SvgContext.updateConnector(node, this._svg);
                else if (node.type === "node")
                    ej.Diagram.SvgContext.updateNode(node, this._svg);
                else if (node.type === "group" || node.type === "pseudoGroup")
                    ej.Diagram.SvgContext.updateGroup(node, this._svg);
                this._updateSelectionHandle();
                this._renderTooltip(node);
            }
        },

        _isUserHandle: function (evt) {
            if (evt.target.parentNode.className.animVal === "userHandle") {
                return true;
            }
            else {
                return false;
            }
        },

        _isUserHandleActive: function (name) {
            var userHandleActive = false;
            if (this.model.userHandles && this.model.userHandles.length > 0) {
                var i = 0;
                var length = this.model.userHandles.length;
                for (i; i < length; i++) {
                    if (this.model.userHandles[i].name === name) {
                        userHandleActive = true;
                        break;
                    }
                    else
                        userHandleActive = true;
                }
            }
            return userHandleActive;
        },

        _raiseMouseEvents: function (node) {
            var type = null;
            if (!node && this._nodeToHit) {
                this._trigger("mouseLeave", { element: this._nodeToHit });
            }
            if (node) {
                if (this._nodeToHit !== node) {
                    this._nodeToHit = node;
                    this._trigger("mouseEnter", { element: node });
                }
            }
            else {
                this._nodeToHit = null;
            }
            if (node) {
                this._trigger("mouseHover", { diagram: node });
            }
        },

        _isHandle: function (evt) {
            var handle = $(evt.target).parent("g.handle");
            if (handle.length > 0) {
                return true;
            }
            return false;
        },

        _findNodeUnderMouse: function (evt) {
            var node = null;
            var type;
            var parent = $(evt.target).parents(".ej-d-node,.ej-d-connector");
            if (parent) {
                type = parent.attr("class");
                if (type === "ej-d-node" || type === "ej-d-group") {
                    node = this._findNode(parent.attr("id"));
                }
                else if (type === "ej-d-connector") {
                    node = this._findConnector(parent.attr("id"));
                }
            }
            return node;
        },

        _mouseup: function (evt) {
            if (this._invoke(evt)) {
                evt.preventDefault();
                if (evt.target.id === this._id + "_canvas_svg")
                    this._trigger("click", { element: this });
                this.activeTool.mouseup(evt);
                if (this.activeTool instanceof ej.Diagram.PanTool) {
                    document.onmousemove = null;
                    document.onmouseup = null;
                }
                if ((!(this.activeTool instanceof ej.Diagram.PolygonTool) && !(this.activeTool instanceof ej.Diagram.TextTool)) && document.activeElement && document.activeElement.id !== this.element[0].id)
                    if (!this._isEditing)
                        this.element[0].focus();
                this._currentCursor = this.activeTool.cursor;
                this._updateCursor();
            }
        },

        _doubleclick: function (evt) {
            var args;
            if (!this.activeTool.inAction && !this.selectionList.isEmpty()) {
                args = this._trigger("doubleClick", { element: this.selectionList[0], cancel: false });
                var obj = this.selectionList[0];
                if (!(obj.type === "pseudoGroup") && !args.cancel) {
                    if (!this._isEditing && obj.shape && obj.shape.type === "text" && this._setLabelEditing(obj.shape.textBlock)) {
                        this.scrollToNode(obj);
                        this._startEdit(obj);
                    }
                    else if (!this._isEditing && ((obj.labels.length === 0 || typeof obj.labels.length === "undefined") || (obj.labels.length > 0 && this._setLabelEditing(obj.labels[0])))) {
                        this.scrollToNode(obj);
                        this._startEdit(obj);
                    }
                }
            }
            else if (this.activeTool instanceof ej.Diagram.PolygonTool) {
                this.activeTool.doubleclick(evt);
            }
        },

        _keydown: function (evt) {
            var keycode = evt.keyCode ? evt.keyCode : evt.which;
            if (keycode === 113) {
                if (!this.selectionList.isEmpty()) {
                    var shape = this.selectionList[0];
                    if (!(shape.type === "pseudoGroup")) {
                        if (shape.shape.type === "text" && this._setLabelEditing(shape.shape.textBlock))
                            this._startEdit(shape);
                        else if (!shape.labels.isEmpty() && this._setLabelEditing(shape.labels[0]))
                            this._startEdit(shape);
                    }
                }
            }
            else if (this._isEditing && keycode === 27) {
                this._endEdit();
            }
            else if (!this._isEditing && keycode === 46) {
                this._delete();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 88) {
                this.cut();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 67) {
                this.copy();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 86) {
                this.paste();
            }
            else if (evt.ctrlKey && keycode === 65) {
                if (this._isEditing)
                    this._endEdit();
                evt.preventDefault();
                this.selectAll();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 89) {
                this.redo();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 90) {
                this.undo();
            }
            if (!this._isEditing && (keycode === 37 || keycode === 38 || keycode === 39 || keycode === 40)) {
                evt.preventDefault();
                if (keycode === 37) { //left
                    this.nudgeLeft();
                } else if (keycode === 38) { //up
                    this.nudgeUp();
                } else if (keycode === 39) { //right
                    this.nudgeRight();
                } else if (keycode === 40) { //down
                    this.nudgeDown();
                }
            }
            this.activeTool.keydown(evt);
        },

        _registerHistoryEvents: function () {
            this.historyManager.addEventHandler("collectionchanged", $.proxy(this._recordCollectionChanged, this));
            this.historyManager.addEventHandler("positionchanged", $.proxy(this._recordPinPointChanged, this));
            this.historyManager.addEventHandler("sizechanged", $.proxy(this._recordResizeChanged, this));
            this.historyManager.addEventHandler("rotationchanged", $.proxy(this._recordRotationChanged, this));
            this.historyManager.addEventHandler("endpointchanged", $.proxy(this._recordEndPointChanged, this));
            this.historyManager.addEventHandler("labelchanged", $.proxy(this._recordLabelChanged, this));
            this.historyManager.addEventHandler("groupingchanged", $.proxy(this._recordGroupChanged, this));
        },

        _updatePoints: function (helper, startPoint, endPoint, isTarget) {
            var offset = new ej.Diagram.Point(endPoint.x - startPoint.x, endPoint.y - startPoint.y);
            if (!offset.isEmpty()) {
                helper._updateEndPoint(offset.x, offset.y, isTarget);
            }
        },

        _recordCollectionChanged: function (args) {
            this._clearSelection();
            this._isUndo = true;
            var changeType = args.changeType;
            if (args.isUndo) {
                if (args.changeType == "insert")
                    changeType = "remove";
                else
                    changeType = "insert";
            }
            if (changeType == "insert") {
                if (args.object.length == undefined) {
                    this.add(args.object);
                    this.addSelection(args.object);
                }
                else {
                    var pseudoGroup = new ej.Diagram.PseudoGroup({ "name": "multipleSelection" });
                    for (var i = 0; i < args.object.length; i++) {
                        this.add(args.object[i]);
                        pseudoGroup.children.add(args.object[i]);
                    }
                    this.addSelection(pseudoGroup);
                }
            }
            else if (changeType == "remove") {
                if (args.object.length == undefined) {
                    this._remove(args.object);
                }
                else {
                    for (var i = 0; i < args.object.length; i++) {
                        this._remove(args.object[i]);
                    }
                }
            }
            this._isUndo = false;
        },

        _recordPinPointChanged: function (args) {
            this._clearSelection();
            var object;
            var i, len;
            var childNode;
            if (args.isMultipleNode) {
                var pseudoGroup = new ej.Diagram.PseudoGroup({ "name": "multipleSelection" });
                for (i = 0, len = args.object.children.length; i < len; i++) {
                    if (args.object.children[i].line === undefined) {
                        childNode = this._findNode(args.object.children[i].name);
                        childNode._translate(args.object.children[i].offsetX - childNode.offsetX, args.object.children[i].offsetY - childNode.offsetY);
                        pseudoGroup.children.add(childNode);
                    }
                    else {
                        childNode = this._findConnector(args.object.children[i].name);
                        childNode.startPoint = args.object.children[i].line.startPoint;
                        childNode.endPoint = args.object.children[i].line.endPoint;
                        pseudoGroup.children.add(childNode);
                    }
                }
                this.model.nodes.add(pseudoGroup);
                object = pseudoGroup;
            }
            else {
                if (args.object.line === undefined) {
                    object = this._findNode(args.object.name);
                    object._translate(args.object.offsetX - object.offsetX, args.object.offsetY - object.offsetY);
                }
                else {
                    object = this._findConnector(args.object.name);
                    object.line.startPoint = args.object.line.startPoint;
                    object.line.endPoint = args.object.line.endPoint;
                    object.line.points = args.object.line.points;
                }
            }
            if (object.type === "connector") {
                var connector = object;
                if (!connector.isTargetConnected() && !connector.isSourceConnected()) {
                    ej.Diagram.SvgContext.updateConnector(connector, this._svg);
                }
            }
            else if (object.type === "node") {
                ej.Diagram.SvgContext.updateNode(object, this._svg);
            }
            else if (object.type === "pseudoGroup") {
                var child;
                for (i = 0, len = object.children.length; i < len; i++) {
                    child = object.children[i];
                    //child.parent = null;
                    if (child.type === "connector") {
                        ej.Diagram.SvgContext.updateConnector(child, this._svg);
                    }
                    else if (child.type === "node") {
                        ej.Diagram.SvgContext.updateNode(child, this._svg);
                    }
                    else if (child.type === "group") {
                        ej.Diagram.SvgContext.updateGroup(child, this._svg);
                    }
                    //child.parent = object;
                }
            }
            else if (object.type === "group") {
                ej.Diagram.SvgContext.updateGroup(object, this._svg);
            }
            this.addSelection(object);
        },

        _recordResizeChanged: function (args) {
            this._clearSelection();
            var node;
            if (args.isMultipleNode) {
                var pseudoGroup = new ej.Diagram.PseudoGroup({ "name": "multipleSelection" });
                for (var i = 0, len = args.object.children.length; i < len; i++) {
                    if (args.object.children[i].line === undefined) {
                        var childNode = this._findNode(args.object.children[i].name);
                        childNode._translate(args.object.children[i].offsetX - childNode.offsetX, args.object.children[i].offsetY - childNode.offsetY);
                        childNode._scale(new ej.Diagram.Matrix()._createIdentity(), args.object.children[i].width / childNode.width, args.object.children[i].height / childNode.height, new ej.Diagram.Point(childNode.offsetX, childNode.offsetY));
                        pseudoGroup.children.add(childNode);
                    }
                }
                this.model.nodes.add(pseudoGroup);
                node = pseudoGroup;
            }
            else {
                node = this._findNode(args.object.name);
                node._translate(args.object.offsetX - node.offsetX, args.object.offsetY - node.offsetY);
                node._scale(new ej.Diagram.Matrix()._createIdentity(), args.object.width / node.width, args.object.height / node.height, new ej.Diagram.Point(node.offsetX, node.offsetY));
            }
            if (node.type === "node")
                ej.Diagram.SvgContext.updateNode(node, this._svg, this._diagramLayer);
            else
                ej.Diagram.SvgContext.updateGroup(node, this._svg, this._diagramLayer);
            this.addSelection(node);
        },

        _recordRotationChanged: function (args) {
            this._clearSelection();
            var node;
            if (args.isMultipleNode) {
                var pseudoGroup = new ej.Diagram.PseudoGroup({ "name": "multipleSelection" });
                for (var i = 0, len = args.object.children.length; i < len; i++) {
                    if (args.object.children[i].line === undefined) {
                        var childNode = this._findNode(args.object.children[i].name);
                        childNode._translate(args.object.children[i].offsetX - childNode.offsetX, args.object.children[i].offsetY - childNode.offsetY);
                        childNode._rotate(args.object.children[i].rotateAngle - childNode.rotateAngle, args.object.children[i].pivot);
                        pseudoGroup.children.add(childNode);
                    }
                }
                this.model.nodes.add(pseudoGroup);
                node = pseudoGroup;
            }
            else {
                node = this._findNode(args.object.name);
                node._rotate(args.object.rotateAngle - node.rotateAngle, args.object.pivot);
            }
            if (node.type === "node")
                ej.Diagram.SvgContext.updateNode(node, this._svg);
            else
                ej.Diagram.SvgContext.updateGroup(node, this._svg);
            this.addSelection(node);
        },

        _recordEndPointChanged: function (args) {
            var connector, currentObject, previousObject;
            if (args.isUndo) {
                connector = currentObject = args.undoObject;
                previousObject = args.redoObject;
            }
            else {
                connector = currentObject = args.redoObject;
                previousObject = args.undoObject;
            }
            if (connector.parent) {

                connector = this._findChildren(connector.parent, connector.name);
            }
            else
                connector = this._findConnector(connector.name);

            if (previousObject.targetNode) {
                previousObject.targetNode.inEdges.remove(previousObject);
            }
            if (previousObject.sourceNode) {
                previousObject.sourceNode.outEdges.remove(previousObject);
            }
            if (currentObject.targetNode) {
                currentObject.targetNode.inEdges.add(currentObject);
            }
            if (currentObject.sourceNode) {
                currentObject.sourceNode.outEdges.add(currentObject);
            }
            connector.targetNode = currentObject.targetNode;
            connector.sourceNode = currentObject.sourceNode;
            connector.targetPort = currentObject.targetPort;
            connector.sourcePort = currentObject.sourcePort;
            connector.line.startPoint = currentObject.line.startPoint;
            connector.line.endPoint = currentObject.line.endPoint;
            connector.line.points = currentObject.line.points;
            this._updateEdges(connector);
            connector.dock();
            ej.Diagram.SvgContext.updateConnector(connector, this._svg);
            this._updateSelectionHandle();
        },

        _recordLabelChanged: function (args) {
            args.object.labels[0].text = args.label;
            ej.Diagram.SvgContext.updateLabel(args.object, args.object.labels[0], this._svg);
        },

        _recordGroupChanged: function (args) {
            this._clearSelection();
            var actionType = args.actionType;
            if (args.isUndo) {
                if (args.actionType === "group")
                    actionType = "ungroup";
                if (args.actionType === "ungroup")
                    actionType = "group";
            }
            var selectionList = args.object.children;
            if (actionType === "group") {
                var group = new ej.Diagram.Group();
                for (i = 0; i < selectionList.length; i++) {
                    group.children.add(selectionList[i]);
                    if (selectionList[i].type === "connector")
                        this.model.connectors.remove(selectionList[i]);
                    else
                        this.model.nodes.remove(selectionList[i]);
                    $("#" + selectionList[i].name)[0].parentNode.removeChild($("#" + selectionList[i].name)[0]);
                }
                group.name = args.object.name;
                this.add(group);
                this._addSelection(group);
            }
            else {
                var i, node;
                node = args.object;
                $("#" + node.name)[0].parentNode.removeChild($("#" + node.name)[0]);
                for (i = 0; i < node.children.length; i++) {
                    var childNode = node.children[i];
                    childNode.parent = null;
                    if (childNode.type === "connector") {
                        this.add(childNode);
                    } else {
                        this.add(childNode);
                    }
                }
                this.model.nodes.remove(this._findNode(node.name));
            }
        },

        _svgdrop: function (evt, ui) {
            var args = {};
            var e = evt.originalEvent;
            if (e.preventDefault)
                e.preventDefault();
            else
                e.returnValue = false;
            var scale = this._zoomPan.currentZoom;
            var offset = $("#" + this.element[0].id).offset();
            var coor = this._isTouchEvent(evt);
            var pageX;
            var pageY;
            if (coor) {
                pageX = coor.pageX;
                pageY = coor.pageY;
            }
            else {
                pageX = evt.pageX;
                pageY = evt.pageY;
            }
            var x = pageX + this._scroller._horizontalOffset - offset.left;
            var y = pageY + this._scroller._verticalOffset - offset.top;
            var symbolPaletteId = ui.helper[0].getAttribute("paletteId");
            var symbolPalette = $("#" + symbolPaletteId).ejSymbolPalette("instance");
            if (symbolPaletteId) {
                var paletteItem = ej.Diagram.Util.clone(symbolPalette.selectedItem);
                paletteItem.name += ej.Diagram.Util.randomId();
                if (paletteItem.type === "connector") {
                    var connector = paletteItem;
                    if (connector.line.points) {
                        var points = connector.line.points;
                        if (points.type === "collection") {
                            var bounds = ej.Diagram.Geometry.rect(points);
                            for (var i = 0, len = points.length; i < len; ++i) {
                                var point = ej.Diagram.Geometry.translate(points[i], -bounds.x, -bounds.y);
                                point = ej.Diagram.Geometry.translate(point, x, y);
                                points[i] = point;
                                var endpoint = new ej.Diagram.Point(point.x / scale, point.y / scale);
                                if (i === 0)
                                    connector.line._setEndPoint(endpoint, false);
                                if (i === points.length - 1)
                                    connector.line._setEndPoint(endpoint, true);
                            }
                        }
                    }
                    args = { element: paletteItem };
                    this._raiseDropEvent(args);
                    this.add(paletteItem);
                }
                else {
                    paletteItem.offsetX = x / scale;
                    paletteItem.offsetY = y / scale;
                    args = { element: paletteItem };
                    this._raiseDropEvent(args);
                    this.add(paletteItem);
                }
                this.scrollToNode(paletteItem);
                if (!paletteItem.labels.isEmpty() && paletteItem.labels[0].mode === "edit") {
                    this._isEditing = true;
                    this.startLabelEdit(paletteItem, paletteItem.labels[0]);
                }
                else
                    this.element[0].focus();
            }
            else {
                var node = null;
                node = this._findNodeUnderMouse(evt);
                args = { element: ui.helper, e: evt, draggable: ui.draggable, targetNode: node };
                this._raiseDropEvent(args);
            }
        },

        _isTouchEvent: function (evt) {
            var e = evt;
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend") {
                e = evt.originalEvent.changedTouches[0];
            }
            else {
                e = null;
            }
            return e;
        },

        _raiseDropEvent: function (args) {
            this._trigger("drop", args);
        },

        _documentmouseup: function (evt) {
            if (this.activeTool && this.activeTool.inAction && !(this.activeTool instanceof ej.Diagram.PolygonTool) && !(this.activeTool instanceof ej.Diagram.TextTool)) {
                this.activeTool.mouseup(evt);
            }
        },
        //#endregion

        //#region Helper methods
        _updateEdges: function (connector) {
            if (connector.isTargetConnected()) {
                connector.targetNode.inEdges.add(connector);
            }
            if (connector.isSourceConnected()) {
                connector.sourceNode.outEdges.add(connector);
            }
        },

        _updateChildrenEdges: function (object) {
            for (var i = 0; i < object.children.length; i++) {
                var child = object.children[i];
                if (child.type === "connector") {
                    this._updateEdges(child);
                    child.dock();
                }
                else if (child.type === "group") {
                    this._updateChildrenEdges(child);
                }
            }
        },

        _findPortAtPoint: function (point, node) {
            var ports;
            var port;
            var location = new ej.Diagram.Rectangle(0, 0, 0, 0);
            var size, pt, x, y;
            var bounds;
            if (node) {
                bounds = node.getBoundingBox();
                ports = node.ports;
                for (var i = 0, len = ports.length; i < len; ++i) {
                    port = ports[i];
                    size = new ej.Diagram.Size(port.size, port.size);
                    pt = port._getPosition(bounds);

                    var mat = new ej.Diagram.Matrix();
                    var matrix = mat._createIdentity();
                    matrix._rotateAt(node.rotateAngle, node.offsetX, node.offsetY);
                    pt = matrix._transform(pt);
                    x = pt.x - port.size / 2;
                    y = pt.y - port.size / 2;
                    location = new ej.Diagram.Rectangle(x, y, size.width, size.height);
                    if (location.contains(point)) {
                        return port;
                    }
                }
            }
            return null;
        },

        _findLabelAtPoint: function (point, node) {
            var labels;
            var label;
            var location = new ej.Diagram.Rectangle(0, 0, 0, 0);
            var size, pt, x, y, width, height;
            var bounds;
            if (node) {
                bounds = node.getBoundingBox();
                labels = node.labels;
                for (var i = 0, len = labels.length; i < len; ++i) {
                    label = labels[i];
                    var textBounds = this._svg.getElementById(node.fullName() + "_" + label.name).getBBox();
                    width = textBounds.width == 0 ? bounds.width : textBounds.width;
                    height = textBounds.height == 0 ? label.fontSize : textBounds.height;
                    size = new ej.Diagram.Size(width, height);
                    pt = label._getPosition(bounds);
                    pt.x = bounds.x + pt.x;
                    pt.y = bounds.y + pt.y;
                    var mat = new ej.Diagram.Matrix();
                    var matrix = mat._createIdentity();
                    if (node.type === "node" || node.type === "group")
                        matrix._rotateAt(node.rotateAngle, node.offsetX, node.offsetY);
                    else
                        matrix._rotateAt(0, bounds.x + bounds.width / 2, bounds.y + bounds.height / 2);
                    pt = matrix._transform(pt);
                    x = pt.x - size.width / 2;
                    y = pt.y - size.height / 2;
                    location = new ej.Diagram.Rectangle(x, y, size.width, size.height);
                    if (location.contains(point)) {
                        return label;
                    }
                    else {
                        return labels[0];
                    }
                }
            }
            return null;
        },

        _findNodeAtPoint: function (point) {
            var collection = this._findAllNodeAtPoint(point);
            if (!collection.isEmpty()) {
                return collection[collection.length - 1];
            }
            return null;
        },

        _findAllNodeAtPoint: function (point) {
            var nodes = this.model.nodes;
            var bounds;
            var node;
            var collection = new ej.Diagram.Collection();
            for (var i = 0, len = nodes.length; i < len; i++) {
                node = nodes[i];
                if (node.contains(point)) {
                    collection.add(node);
                    if (node.type === "group") {
                        var children = this._findChildrenAtPoint(node, point);
                        for (i = 0, len = children.length; i < len; i++) {
                            collection.add(children[i]);
                        }
                    }
                }
            }
            return collection;
        },

        _findChildrenAtPoint: function (group, point) {
            var collection = new ej.Diagram.Collection();
            var child;
            for (var i = 0, len = group.children.length; i < len; i++) {
                child = group.children[i];
                if (child.contains(point)) {
                    collection.add(child);
                }
                if (child.type === "group") {
                    var children = this._findChildrenAtPoint(child, point);
                    for (i = 0, len = children.length; i < len; i++) {
                        collection.add(children[i]);
                    }
                }
            }
            return collection;
        },

        _findNode: function (id) {
            var nodes = this.model.nodes;
            var mNode;
            var child = null;
            var exists = false;
            for (var i = 0, len = nodes.length; i < len; i++) {
                if (nodes[i].fullName() === id) {
                    mNode = nodes[i];
                    break;
                }
                else if (nodes[i].type === "group") {
                    child = this._findChildren(nodes[i], id);
                    if (child) {
                        mNode = child;
                        break;
                    }
                    else if (nodes[i].fullName() === id) {
                        mNode = nodes[i];
                        break;
                    }
                }
            }
            return mNode;
        },

        _findPort: function (node, portId) {
            var ports = node.ports;
            var mPort;
            for (var i = 0, len = ports.length; i < len; i++) {
                if (ports[i].name === portId) {
                    mPort = ports[i];
                    break;
                }
            }
            return mPort;
        },

        _findConnector: function (id) {
            var connectors = this.model.connectors;
            for (var i = 0, len = connectors.length; i < len; i++) {
                if (connectors[i].fullName() === id) {
                    return connectors[i];
                }
            }
            var parent = $(document.getElementById(id)).parents(".ej-d-group");
            if (parent && parent.length > 0) {
                var container = this._findNode(parent[parent.length - 1].id);
                return this._findChildren(container, id);
            }
            return null;
        },

        _findChildren: function (group, name) {
            var child;
            var children = group.children;
            for (var i = 0, len = children.length; i < len; i++) {
                if (children[i].fullName() === name) {
                    child = children[i];
                }
                else if (children[i].type === "group") {
                    var result = this._findChildren(children[i], name);
                    if (result) child = result;
                }
            }
            return child;
        },

        _delete: function () {
            if (!this.selectionList.isEmpty()) {
                var obj = this.selectionList[0];
                if (obj.type === "pseudoGroup") {
                    var children = obj.children;
                    for (var i = children.length - 1; i >= 0; i--) {
                        this._remove(children[i]);
                        this._isUndo = true;
                    }
                    this._isUndo = false;
                }
                else {
                    this._remove(obj);
                }
            }
            this._clearSelection();
        },

        _remove: function (obj) {
            if (obj.canDelete()) {
                if (obj.type === "node" || obj.type === "group") {
                    this._disConnect(obj);
                    this.model.nodes.remove(obj);
                }
                else {
                    this._removeEdges(obj);
                    this.model.connectors.remove(obj);
                }
                if (obj.parent && obj.parent.type === "group") {
                    obj.parent.children.remove(obj);
                }
                var element = this._svg.getElementById(obj.fullName());
                if (ej.browserInfo().name === "msie") {
                    if (obj.type == "node" && obj.shape && obj.shape.html) {
                        var htmlelement = document.getElementById(obj.fullName() + "_parentdiv");
                        htmlelement.parentNode.removeChild(htmlelement);
                    }
                    else if (obj.type == "group")
                        this._checkForHtmlNode(obj);
                }
                element.parentNode.removeChild(element);
                if (this.selectionList[0]) {
                    this._clearSelection();
                }
            }
        },

        _disConnect: function (node) {
            var edges;
            var connector;
            var i, len;
            if (!node.inEdges.isEmpty()) {
                edges = node.inEdges;
                for (i = 0, len = edges.length; i < len; i++) {
                    connector = edges[i];
                    connector.targetNode = null;
                    connector.targetPort = null;
                }
            }
            if (!node.outEdges.isEmpty()) {
                edges = node.outEdges;
                for (i = 0, len = edges.length; i < len; i++) {
                    connector = edges[i];
                    connector.sourceNode = null;
                    connector.sourcePort = null;
                }
            }
        },

        _removeEdges: function (connector) {
            if (connector.targetNode) {
                connector.targetNode.inEdges.remove(connector);
            }
            if (connector.sourceNode) {
                connector.sourceNode.outEdges.remove(connector);
            }
        },

        _addMultipleSelection: function (bounds) {
            if (bounds.width !== 0 || bounds.height !== 0) {
                var i;
                var len;
                var collection = new ej.Diagram.Collection();
                var nodes = this.model.nodes;
                for (i = 0, len = nodes.length; i < len; i++) {
                    var node = nodes[i];
                    if (node.canSelect()) {
                        if (bounds.contains(node._bounds))
                            collection.add(node);
                        else if (node.type === "group") {
                            this._checkGroupChildren(collection, node, bounds);
                        }
                    }
                }
                var connectors = this.model.connectors;
                for (i = 0, len = connectors.length; i < len; i++) {
                    var connector = connectors[i];
                    if (connector.canSelect() && bounds.contains(connector._bounds))
                        collection.add(connector);
                }
                if (this._hasSelection()) {
                    this._clearSelection();
                }
                if (collection.length > 0) {
                    var children = collection;
                    var selection;
                    if (children.length > 1) {
                        var pseudoGroup = new ej.Diagram.PseudoGroup({ "name": "multipleSelection" });
                        for (i = 0, len = children.length; i < len; i++) {
                            pseudoGroup.children.add(children[i]);
                        }
                        this.model.nodes.add(pseudoGroup);
                        selection = pseudoGroup;
                    }
                    else {
                        selection = collection[0];
                    }
                    this._addSelection(selection);
                }
            }
        },

        _checkGroupChildren: function (collection, group, bounds) {
            for (var i = 0; i < group.children.length; i++) {
                var child = group.children[i];
                if (bounds.contains(child._bounds)) {
                    collection.add(child);
                }
                else if (child.type === "group") {
                    this._checkGroupChildren(collection, child, bounds);
                }
            }
        },

        _addSelection: function (selectedShape) {
            if (selectedShape && selectedShape.canSelect()) {
                if (!this.selectionList.contains(selectedShape)) {
                    this.selectionList.add(selectedShape);
                    ej.Diagram.SvgContext.renderSelector(selectedShape, this._svg, this._adornerLayer, this._zoomPan.currentZoom,
                        this.model.selectorConstraints);
                    if ((this.model.userHandles !== null && this.model.userHandles.length > 0)) {
                        var isMultipleSelection = false;
                        if (selectedShape.type === "pseudoGroup")
                            isMultipleSelection = true;
                        if (this.model.selectorConstraints & ej.Diagram.SelectorConstraints.UserHandles)
                            ej.Diagram.SvgContext.renderUserHandles(this.model.userHandles, selectedShape, this._svg, isMultipleSelection, this._zoomPan.currentZoom, this._adornerLayer);
                    }
                }
            }
        },

        _clearSelection: function () {
            if (this._isEditing && !this.selectionList.isEmpty()) {
                this._endEdit();
            }
            if (this.selectionList[0] && this.selectionList[0].type === "pseudoGroup") {
                for (var i = 0, len = this.selectionList[0].children.length; i < len; i++) {
                    //this.selectionList[0].children[i].parent = null;
                }
                this._isUndo = true;
                this.model.nodes.remove(this.selectionList[0]);
                this._isUndo = false;
            }
            this.selectionList.clear();
            ej.Diagram.SvgContext.clearSelector(this._svg, this._adornerLayer);
        },

        _updateSelectionHandle: function (isDragging) {
            if (!this.selectionList.isEmpty()) {
                var shape = this.selectionList[0];
                ej.Diagram.SvgContext.updateSelector(shape, this._svg, this._zoomPan.currentZoom, this, this.model.selectorConstraints);
                if ((this.model.userHandles !== null && this.model.userHandles.length > 0)) {
                    var isMultipleSelection = false;
                    if (shape.name === "multipleSelection")
                        isMultipleSelection = true;
                    if (this.model.selectorConstraints & ej.Diagram.SelectorConstraints.UserHandles)
                        ej.Diagram.SvgContext.updateUserHandles(this.model.userHandles, shape, this._svg, isMultipleSelection, isDragging, this._zoomPan.currentZoom);
                }
            }
        },

        _hasSelection: function () {
            return !this.selectionList.isEmpty();
        },

        _checkToolToActivate: function (evt, node, point, handleName) {
            var success = false;
            if (node) {
                if (this.selectionList.contains(node)) {
                    if (node.type === "connector") {
                        success = this._canActivateEndPointTool(evt, node, point);
                    }
                    else {
                        success = this._canActivateResizeTool(evt, node, point);
                        if (!success) {
                            success = this._canActivateRotateTool(evt, node, point);
                        }
                    }
                    if (!success) {
                        success = this._canActivateUserHandle(evt, node, point);
                    }
                    if (!success) {
                        success = this._canActivateMoveTool(evt, node, point);
                    }
                }
                else {
                    if (!success) {
                        success = this._canActivateMoveTool(evt, node, point);
                    }
                }
            }
            if (!success) {
                this._toolToActivate = "select";
                this._currentCursor = "default";
            }
            if (this._toolToActivate != "rotate") {
                this._svg.document.removeAttribute("class");
            }
            return success;
        },

        _canActivateUserHandle: function (evt, node, point) {
            if (this._isUserHandle(evt)) {
                var handleName = $(evt.target).parents("g").first().context.id.split("_")[0];
                this._toolToActivate = handleName;
                this._currentCursor = this.tools[this._toolToActivate].cursor;
                return true;
            }
            return false;
        },

        _canActivateEndPointTool: function (evt, connector, point) {
            if (connector) {
                var endPointHandle = evt.target.getAttribute("class");
                if ((endPointHandle === "targetEndPoint" && connector.canDragTargetEnd()) || (endPointHandle === "sourceEndPoint" && connector.canDragSourceEnd()) ||
                (endPointHandle === "bezierpoint1" || endPointHandle === "bezierpoint2") && connector.canDragSegmentThumbs()) {
                    this._toolToActivate = "endPoint";
                    this._currentCursor = "move";
                    return true;
                }
            }
            return false;
        },

        _canActivateResizeTool: function (evt, node, point) {
            if (node.canResize() && (this.model.selectorConstraints & ej.Diagram.SelectorConstraints.Resizer)) {
                var dir = evt.target.getAttribute("class");
                if (dir === "nw-resize" || dir === "n-resize" || dir === "ne-resize" || dir === "w-resize" || dir === "e-resize" || dir === "sw-resize" || dir === "s-resize" || dir === "se-resize") {
                    this._toolToActivate = "resize";
                    this._currentCursor = dir;
                    return true;
                }
            }
            return false;
        },

        _canActivateMoveTool: function (evt, node, point) {
            var findNode = this._findNodeUnderMouse(evt);
            if (findNode && findNode.fullName() === node.fullName()) {
                if (node.canMove()) {
                    this._toolToActivate = "move";
                    if (this.selectionList.length > 0 && (this.selectionList.contains(node) || (this.selectionList[0].children && this.selectionList[0].children.contains(node)))) {
                        this._currentCursor = "move";
                    }
                    else {
                        this._currentCursor = "default";
                    }
                    return true;
                }
            }
            return false;
        },

        _canActivateRotateTool: function (evt, node, point) {
            if (node.canRotate() && (this.model.selectorConstraints & ej.Diagram.SelectorConstraints.Rotator)) {
                var rotateHandle = evt.target.getAttribute("class");
                if (rotateHandle === "rotate") {
                    this._toolToActivate = "rotate";
                    this._svg.document.setAttribute("class", "svg-rotate");
                    this._currentCursor = "";
                    return true;
                }
            }
            return false;
        },

        _canActivatePivotTool: function (evt, node, point) {
            var pivotHandle = evt.target.getAttribute("class");
            if (pivotHandle === "pivot") {
                this._toolToActivate = "pivot";
                this._currentCursor = "default";
                return true;
            }

            return false;
        },

        _updateCursor: function () {
            this._svg.document.style.cursor = this._currentCursor;
        },

        _getResizeCursor: function (resizeDirection) {
            var cursor;
            switch (resizeDirection) {
                case "topLeft": cursor = "nw-resize";
                    break;
                case "topCenter": cursor = "n-resize";
                    break;
                case "topRight": cursor = "ne-resize";
                    break;
                case "middleLeft": cursor = "w-resize";
                    break;
                case "middleRight": cursor = "e-resize";
                    break;
                case "bottomLeft": cursor = "sw-resize";
                    break;
                case "bottomCenter": cursor = "s-resize";
                    break;
                case "bottomRight": cursor = "se-resize";
                    break;
            }
            return cursor;
        },

        _startEdit: function (shape) {
            var label;
            var centerX, centerY;
            var bbox;
            var nodeBounds;
            var bounds;
            var curZoomfactor;
            if (shape.shape && shape.shape.type === "text") {
                if (this._svg.getElementById(shape.fullName() + "_shape") !== null)
                    bbox = this._svg.getElementById(shape.fullName() + "_shape").getBBox();
                else
                    bbox = { x: shape.offsetX - shape.width / 2, y: shape.offsetY - shape.height / 2, width: shape.width, height: shape.height };
                label = shape.shape.textBlock;
                if (label) {
                    nodeBounds = shape.getBoundingBox();
                    bounds = new ej.Diagram.Rectangle(bbox.x - label.fontSize, bbox.y - label.fontSize, bbox.width + label.fontSize / 2, bbox.height + label.fontSize / 2);
                    curZoomfactor = this._zoomPan.currentZoom;
                    centerX = shape.offsetX;
                    centerY = shape.offsetY;
                    if (bbox.width === 0 || bbox.height === 0) {
                        bbox = shape.getBoundingBox();
                        bounds.width = bbox.width == 0 ? 50 : nodeBounds.width * curZoomfactor;
                        bounds.x = (centerX * curZoomfactor - bounds.width / 2 * curZoomfactor) - this._scroller._horizontalOffset;

                        bounds.height = label.fontSize;
                        bounds.y = (centerY * curZoomfactor - bounds.height / 2 * curZoomfactor) - this._scroller._verticalOffset;
                    }
                    else {
                        bounds.width = 50;
                        bounds.height = label.fontSize;
                        bounds.x = (centerX * curZoomfactor - (bounds.width / 2 * curZoomfactor)) - this._scroller._horizontalOffset;
                        bounds.y = (centerY * curZoomfactor - (bounds.height / 2 * curZoomfactor)) - this._scroller._verticalOffset;
                    }
                    this._boundingBox = bounds;
                    this._createEditBox(label, bounds.x, bounds.y, bounds.width, bounds.height, shape);
                    this._isEditing = true;
                }
            }
            else {
                if (shape.labels.length > 0) {
                    if (this.activeTool.name === "textTool") {
                        label = this.activeTool.getLabelUnderMouse();
                        if (label)
                            label = shape.labels[0];
                    }
                    else
                        label = shape.labels[0];
                }
                else {
                    var mNode = this._findNode(shape.fullName());
                    label = new ej.Diagram.Label({ "name": "label" + ej.Diagram.Util.randomId() });
                    shape.labels.add(label);
                    var parent = (mNode !== shape) ? mNode : null;
                    ej.Diagram.SvgContext.addNodeLabel(shape, label, this._svg, parent);
                    if (this.activeTool.name === "textTool")
                        this.activateTool.activeLabel = label;
                }
                if (label && !label.readOnly) {
                    bbox = this._svg.getElementById(shape.fullName() + "_" + label.name).getBBox();
                    nodeBounds = shape.getBoundingBox();
                    bounds = new ej.Diagram.Rectangle(bbox.x, bbox.y, bbox.width, bbox.height);
                    curZoomfactor = this._zoomPan.currentZoom;
                    if (shape.type === "connector") {
                        centerX = nodeBounds.x + nodeBounds.width / 2;
                        centerY = nodeBounds.y + nodeBounds.height / 2;
                    }
                    else {
                        centerX = shape.offsetX;
                        centerY = shape.offsetY;
                    }
                    if (bbox.width === 0 || bbox.height === 0) {
                        bbox = shape.getBoundingBox();
                        if (!shape.shape) {
                            bounds.width = bbox.width < 50 ? 50 : nodeBounds.width * curZoomfactor;
                            bounds.x = (centerX * curZoomfactor - bounds.width / 2 * curZoomfactor) - this._scroller._horizontalOffset;
                        }
                        else {
                            bounds.width = bbox.width < 50 ? 50 : (nodeBounds.width * curZoomfactor);
                            bounds.x = (centerX * curZoomfactor - bounds.width / 2) - this._scroller._horizontalOffset;
                        }
                        bounds.height = label.fontSize;
                        bounds.y = (centerY * curZoomfactor - bounds.height / 2 * curZoomfactor) - this._scroller._verticalOffset;
                    }
                    else {
                        bounds.x = (centerX * curZoomfactor - (bounds.width / 2 * curZoomfactor)) - this._scroller._horizontalOffset;
                        bounds.y = (centerY * curZoomfactor - (bounds.height / 2 * curZoomfactor)) - this._scroller._verticalOffset;
                    }
                    this._boundingBox = bounds;
                    this._createEditBox(label, bounds.x, bounds.y, bounds.width, bounds.height, shape);
                    this._isEditing = true;
                }
            }
        },

        _createEditBox: function (label, x, y, width, height, shape) {
            var curZoomfactor = this._zoomPan.currentZoom;
            var nodeBounds = shape.getBoundingBox();
            var div = document.createElement("div");

            var attr = {
                "id": this.element[0].id + "_editBoxDiv",
                "class": "edit",
                "style": "top:" + y + "px;left:" + x + "px;width:" + width + "px;height:" + height + "px;" +
                    "font-family:" + label.fontFamily + ";font-size:" + label.fontSize + "px;min-width: 50px;min-height:" + label.fontSize + "px;",
            };
            ej.Diagram.Util.attr(div, attr);
            var textBox = document.createElement("textarea");
            attr = {
                "id": this.element[0].id + "_editBox",
                "class": "editBox",
                "style": "font-size:" + label.fontSize + "px;font-family:" + label.fontFamily + ";min-height:" + label.fontSize + "px;line-height:" + label.fontSize + "px;",
                "nodeWidth": nodeBounds.width, "nodeHeight": nodeBounds.height, "fontSize": label.fontSize, "nodeOffsetX": shape.offsetX, "nodeOffsetY": shape.offsetY, "containerName": shape.fullName()
            };
            ej.Diagram.Util.attr(textBox, attr);
            textBox.value = label.text;
            div.appendChild(textBox);
            $(this._canvas).append(div);

            if (label.text !== "") {
                this._updateEditor(null, div, textBox, nodeBounds.width, label.fontSize);
            }
            this._on($(textBox), "focusout", this._editboxfocusout);
            this._on($(textBox), "keyup", this._editBoxKeyUp);
            this._on($(textBox), "input", this._editBoxTextChange);
            textBox.focus();
            textBox.select();
        },

        _editBoxKeyUp: function (evt) {
            this._updateLabelPosition(evt);
        },

        _editBoxTextChange: function (evt) {
            this._updateLabelPosition(evt);
        },

        _updateEditBox: function (x, y) {
            var editBox = document.getElementById(this.element[0].id + "_editBoxDiv");
            if (editBox) {
                editBox.style.left = editBox.offsetLeft - x + "px";
                editBox.style.top = editBox.offsetTop - y + "px";
            }
        },

        _editboxfocusout: function (evt) {
            this._endEdit();
        },

        _updateLabelPosition: function (evt) {
            var minWidth = 50;
            var curZoomfactor = this._zoomPan.currentZoom;
            var parentBoundary = this._boundingBox;
            var fontSize = Number(evt.target.getAttribute("fontSize"));
            var nodeWidth = Number(evt.target.getAttribute("nodeWidth"));
            var conCenterX = parentBoundary.x + (parentBoundary.width / 2);
            var conCenterY = parentBoundary.y + (parentBoundary.height / 2);
            var bounds = this._textLength(evt);

            var width = minWidth;
            if (bounds.width === 0) {
                width = parentBoundary.width < minWidth ? minWidth : parentBoundary.width;
                evt.target.parentNode.style.width = width + "px";
                evt.target.parentNode.style.left = conCenterX - width / 2 + "px";
            }
            else {
                if (bounds.width <= (nodeWidth * curZoomfactor))
                    width = bounds.width < minWidth ? minWidth : bounds.width;
                else
                    width = nodeWidth < minWidth ? minWidth : (nodeWidth * curZoomfactor);
                evt.target.parentNode.style.width = width + "px";
                evt.target.parentNode.style.left = conCenterX - (width / 2) + "px";
            }
            if (bounds.height === 0) {
                evt.target.style.height = fontSize + "px";
                evt.target.parentNode.style.top = conCenterY - fontSize / 2 + 'px';
            }
            else {
                this._updateEditor(evt, evt.target.parentNode, evt.target, nodeWidth, fontSize);
            }
        },

        _updateEditor: function (evt, div, textBox, nodeWidth, fontSize) {
            var offsetX, offsetY;
            var elHeight = textBox.scrollHeight;
            var bounds = this._textLength(evt);
            var curZoomfactor = this._zoomPan.currentZoom;
            var minWidth = 50;
            var x = this._boundingBox.x;
            var y = this._boundingBox.y;
            var width = this._boundingBox.width;
            var height = this._boundingBox.height;

            var str = textBox.value;
            var line = str.split('\n');
            var lineHeight = fontSize;

            var temp = (nodeWidth * curZoomfactor) < bounds.width ? (nodeWidth * curZoomfactor) : bounds.width;
            var elWidth = temp < minWidth ? minWidth : temp;
            div.style.width = elWidth + "px";
            if (evt)
                offsetX = (elWidth - width) / 2;
            else
                offsetX = (elWidth - width * curZoomfactor) / 2;
            div.style.left = x - offsetX + "px";

            if (line.length > 1 || bounds.width >= nodeWidth) {
                if (ej.browserInfo().name === "mozilla")
                    elHeight = textBox.scrollHeight;
                else
                    elHeight = textBox.scrollHeight - 4;
                textBox.style.height = 'auto';
            }
            else
                elHeight = fontSize;

            textBox.style.height = elHeight + "px";
            if (evt)
                offsetY = (elHeight - height) / 2;
            else
                offsetY = (elHeight - height * curZoomfactor) / 2;
            this._boundingBox = new ej.Diagram.Rectangle(x - offsetX, y - offsetY, elWidth, elHeight);
            div.style.top = y - offsetY + "px";
        },

        _textLength: function (evt) {
            var temp = document.createElement("div");
            var textElement = document.getElementById(this.element[0].id + "_editBox");
            var attr = {
                "id": this.element[0].id + "_editBoxHiddenDiv",
                "style": "position:absolute; width:" + textElement.width + "px;height:" + textElement.height + "px;" +
                         "font-size:" + textElement.style.fontSize + "px;font-family:" + textElement.style.fontFamily + ";padding: 4px;word-wrap: break-word;white-space:pre-wrap;",
            };
            ej.Diagram.Util.attr(temp, attr);
            temp.innerHTML = evt === null ? textElement.value : evt.target.value;
            document.body.appendChild(temp);
            var bounds = temp.getBoundingClientRect();
            document.body.removeChild(temp);
            return bounds;
        },

        _endEdit: function () {
            var type, args;
            var entry;
            var editBox = document.getElementById(this.element[0].id + "_editBox");
            if (editBox) {
                var shape = this._findNode(editBox.getAttribute("containerName"));;
                if (this.activeTool.name === "textTool")
                    shape = this.activeTool.selectedObject;
                else if (!shape) {
                    for (var i = 0; i < this.selectionList.length; i++) {
                        var mNode = this.selectionList[i];
                        if (typeof mNode.labels.length != "undefined" && mNode.labels.length > 0 && mNode.labels[0].mode === ej.Diagram.LabelEditMode.Edit)
                            shape = this.selectionList[i];
                    }
                }

                if (shape) {
                    this._isEditing = false;
                    args = this._trigger("textChanged", { element: shape, value: editBox.value });
                    if (!args) {
                        if (shape && shape.shape !== undefined && shape.shape.type === "text") {
                            shape.shape.textBlock.text = editBox.value;
                            shape.shape.textBlock.mode = ej.Diagram.LabelEditMode.View;
                            if (!this._findNode(shape.name)) {
                                if (editBox.value !== "") {
                                    var height = Number(editBox.style.height.substring(0, editBox.style.height.length - 2));
                                    if (shape.height < height) {
                                        shape.offsetY = (shape.offsetY - (shape.height / 2)) + (height / 2);
                                        shape.height = height;
                                    }
                                    if (this.selectionList.length > 0)
                                        this._clearSelection();
                                    this.add(shape);
                                }
                                this.activeTool.selectedObject = null;
                            } else {
                                ej.Diagram.SvgContext.updateNode(shape, this._svg);
                                entry = new ej.Diagram.HistoryEntry(new ej.Diagram.TextChangeCmd(shape, shape.shape.textBlock.text, editBox.value));
                                this.historyManager.addHistoryEntry(entry);
                            }
                        } else if (shape != undefined && !(shape.type === "pseudoGroup")) {
                            var label;
                            if (this.activeTool.name === "textTool")
                                label = this.activeTool.getLabelUnderMouse();
                            else
                                label = shape.labels[0];
                            if (label != undefined) {
                                if (label.text != editBox.value) {
                                    entry = new ej.Diagram.HistoryEntry(new ej.Diagram.LabelChangeCmd(shape, label.text, editBox.value));
                                    this.historyManager.addHistoryEntry(entry);
                                }
                                label.text = editBox.value;
                                label.mode = ej.Diagram.LabelEditMode.View;
                                ej.Diagram.SvgContext.updateLabel(shape, label, this._svg);
                            }
                            else if (shape.labels.length == 0) {
                                var label = ej.Diagram.Label();
                                label.text = editBox.value;
                                label.mode = ej.Diagram.LabelEditMode.View;
                                shape.labels.add(label);
                                ej.Diagram.SvgContext.addNodeLabel(shape, label, this._svg, null);
                            }
                        }
                    }
                }

                this._off($(editBox), "focusout", this._editboxfocusout);
                this._off($(editBox), "keyup", this._editBoxKeyUp);
                this._off($(editBox), "input", this._editBoxTextChange);
                var element = $("#" + this.element[0].id + "_editBoxDiv")[0];
                if (element)
                    element.parentNode.removeChild(element);
            }
        },

        _setLabelEditing: function (label) {
            var success;
            if (label.readOnly) {
                label.mode = ej.Diagram.LabelEditMode.View;
                success = false;
            }
            else {
                if (this._currentLabel)
                    this._currentLabel.mode = ej.Diagram.LabelEditMode.View;
                label.mode = ej.Diagram.LabelEditMode.Edit;
                this._currentLabel = label;
                success = true;
            }
            return success;
        },

        _isInternalTool: function (tool) {
            if (tool instanceof ej.Diagram.SelectTool || tool instanceof ej.Diagram.EndPointTool || tool instanceof ej.Diagram.MoveTool || tool instanceof ej.Diagram.ResizeTool
                || tool instanceof ej.Diagram.RotateTool || tool instanceof ej.Diagram.PivotTool)
                return true;
            else
                return false;
        },
        //#endregion
    });
    //#endregion

})(jQuery, Syncfusion);;
(function ($, ej) {
    "use strict";

    ej.Diagram.Matrix = function (m11, m12, m21, m22, offsetX, offsetY) {
        this.MatrixTypes =
        {
            TRANSFORM_IS_IDENTITY: 0,
            TRANSFORM_IS_TRANSLATION: 1,
            TRANSFORM_IS_SCALING: 2,
            TRANSFORM_IS_UNKNOWN: 4
        }
        this.s_identity = this._createIdentity();
        this._m11 = m11;
        this._m12 = m12;
        this._m21 = m21;
        this._m22 = m22;
        this._offsetX = offsetX;
        this._offsetY = offsetY;
        this._type;
        this._padding;
        this.c_identityHashCode = 0;
        this.Identity = this.s_identity;
        this._deriveMatrixType();
    };
    /// <summary> Applies a rotation of the specified angle about the origin of this <see cref="T:System.Windows.Media.Matrix" /> structure. </summary>
    /// <param name="angle">The angle of rotation.</param>
    ej.Diagram.Matrix.prototype._rotate = function (angle) {
        angle %= 360.0;
        this._multiplyWith(this._createRotationRadians(angle * 0.017453292519943295));
        return this;
    };

    ej.Diagram.Matrix.prototype._scale = function (scaleX, scaleY) {
        this._multiplyWith(this._createScaling1(scaleX, scaleY));
    };
    ej.Diagram.Matrix.prototype._translate = function (offsetX, offsetY) {
        if (this._type & this.MatrixTypes.TRANSFORM_IS_IDENTITY) {
            this._setMatrix(1.0, 0.0, 0.0, 1.0, offsetX, offsetY, this.MatrixTypes.TRANSFORM_IS_TRANSLATION);
            return;
        }
        if (this._type & this.MatrixTypes.TRANSFORM_IS_UNKNOWN) {
            this._offsetX += offsetX;
            this._offsetY += offsetY;
            return;
        }
        this._offsetX += offsetX;
        this._offsetY += offsetY;
        this._type |= this.MatrixTypes.TRANSFORM_IS_TRANSLATION;
    };

    ej.Diagram.Matrix.prototype._createScaling1 = function (scaleX, scaleY) {
        var result = new ej.Diagram.Matrix();
        result._setMatrix(scaleX, 0.0, 0.0, scaleY, 0.0, 0.0, this.MatrixTypes.TRANSFORM_IS_SCALING);
        return result;
    };
    ej.Diagram.Matrix.prototype._deriveMatrixType = function () {
        this._type = this.MatrixTypes.TRANSFORM_IS_IDENTITY;
        if (this._m21 || this._m12) {
            this._type = this.MatrixTypes.TRANSFORM_IS_UNKNOWN;
            return;
        }
        if (this._m11 && this._m11 != 1.0 || this._m22 && this._m22 != 1.0) {
            this._type = this.MatrixTypes.TRANSFORM_IS_SCALING;
        }
        if (this._offsetX || this._offsetY) {
            this._type |= this.MatrixTypes.TRANSFORM_IS_TRANSLATION;
        }
        if ((this._type & (this.MatrixTypes.TRANSFORM_IS_TRANSLATION | this.MatrixTypes.TRANSFORM_IS_SCALING)) == this.MatrixTypes.TRANSFORM_IS_IDENTITY) {
            this._type = this.MatrixTypes.TRANSFORM_IS_IDENTITY;
        }
    };
    ej.Diagram.Matrix.prototype._createRotationRadians = function (angle) {
        return this._createRotationRadiansAt(angle, 0.0, 0.0);
    };
    ej.Diagram.Matrix.prototype._createRotationRadiansAt = function (angle, centerX, centerY) {
        var result = new ej.Diagram.Matrix();
        var num = Math.sin(angle);
        var num2 = Math.cos(angle);
        var offsetX = centerX * (1.0 - num2) + centerY * num;
        var offsetY = centerY * (1.0 - num2) - centerX * num;
        result._setMatrix(num2, num, -num, num2, offsetX, offsetY, this.MatrixTypes.TRANSFORM_IS_UNKNOWN);
        return result;
    };

    ej.Diagram.Matrix.prototype._setMatrix = function (m11, m12, m21, m22, offsetX, offsetY, type) {
        this._m11 = m11;
        this._m12 = m12;
        this._m21 = m21;
        this._m22 = m22;
        this._offsetX = offsetX;
        this._offsetY = offsetY;
        this._type = type;
    };
    ej.Diagram.Matrix.prototype._createIdentity = function () {
        var result = this;
        result._setMatrix(1.0, 0.0, 0.0, 1.0, 0.0, 0.0, this.MatrixTypes.TRANSFORM_IS_IDENTITY);
        return result;
    };
    /// <summary>Transforms the specified point by the <see cref="T:System.Windows.Media.Matrix" /> and returns the result.</summary>
    /// <returns>The result of transforming <paramref name="point" /> by this <see cref="T:System.Windows.Media.Matrix" />.</returns>
    /// <param name="point">The point to transform.</param>
    ej.Diagram.Matrix.prototype._transform = function (point) {
        var result = point;
        var x = result.x;
        var y = result.y;
        result = this._multiplyPoint(x, y);
        //result = new ej.Diagram.Point(x, y);
        return result;
    };

    ej.Diagram.Matrix.prototype._multiplyPoint = function (x, y) {
        switch (this._type) {
            case this.MatrixTypes.TRANSFORM_IS_IDENTITY:
                {
                    break;
                }
            case this.MatrixTypes.TRANSFORM_IS_TRANSLATION:
                {
                    x += this._offsetX;
                    y += this._offsetY;
                    break;
                }
            case this.MatrixTypes.TRANSFORM_IS_SCALING:
                {
                    x *= this._m11;
                    y *= this._m22;
                    break;
                }
            case this.MatrixTypes.TRANSFORM_IS_TRANSLATION | this.MatrixTypes.TRANSFORM_IS_SCALING:
                {
                    x *= this._m11;
                    x += this._offsetX;
                    y *= this._m22;
                    y += this._offsetY;
                    break;
                }
            default:
                {
                    var num = y * this._m21 + this._offsetX;
                    var num2 = x * this._m12 + this._offsetY;
                    x *= this._m11;
                    x += num;
                    y *= this._m22;
                    y += num2;
                    break;
                }
        }
        return new ej.Diagram.Point(x, y);
    };

    /// <summary>Rotates this matrix about the specified point.</summary>
    /// <param name="angle">The angle, in degrees, by which to rotate this matrix. </param>
    /// <param name="centerX">The x-coordinate of the point about which to rotate this matrix.</param>
    /// <param name="centerY">The y-coordinate of the point about which to rotate this matrix.</param>
    ej.Diagram.Matrix.prototype._rotateAt = function (angle, centerX, centerY) {
        angle %= 360.0;
        this._multiplyWith(this._createRotationRadiansAt(angle * 0.017453292519943295, centerX, centerY));
        return this;
    };
    ej.Diagram.Matrix.prototype._scaleAt = function (scaleX, scaleY, centerX, centerY) {
        this._multiplyWith(this._createScaling(scaleX, scaleY, centerX, centerY));
    };
    ej.Diagram.Matrix.prototype._createScaling = function (scaleX, scaleY, centerX, centerY) {
        var result = new ej.Diagram.Matrix();
        result._setMatrix(scaleX, 0.0, 0.0, scaleY, centerX - scaleX * centerX, centerY - scaleY * centerY, this.MatrixTypes.TRANSFORM_IS_TRANSLATION | this.MatrixTypes.TRANSFORM_IS_SCALING);
        return result;
    };
    /// <summary> Multiplies a <see cref="T:System.Windows.Media.Matrix" /> structure by another <see cref="T:System.Windows.Media.Matrix" /> structure. </summary>
    /// <returns>The result of multiplying <paramref name="trans1" /> by <paramref name="trans2" />.</returns>
    /// <param name="trans1">The first <see cref="T:System.Windows.Media.Matrix" /> structure to multiply.</param>
    /// <param name="trans2">The second <see cref="T:System.Windows.Media.Matrix" /> structure to multiply.</param>
    ej.Diagram.Matrix.prototype._multiplyWith = function (trans2) {
        ej.Diagram.MatrixUtil.prototype._multiplyMatrix(this, trans2);
        //return this;
    };
    /// <summary> Gets an identity <see cref="T:System.Windows.Media.Matrix" />. </summary>
    /// <returns>An identity matrix.</returns>
    //this.Identity = s_identity;

    ej.Diagram.MatrixUtil = function () { };
    //this.matrix = matrix;

    ej.Diagram.MatrixUtil.prototype._transformRect = function (rect, matrix) {
        if (rect.IsEmpty) {
            return;
        }
        var type = matrix._type;
        if (type == MatrixTypes.TRANSFORM_IS_IDENTITY) {
            return;
        }
        if ((type & MatrixTypes.TRANSFORM_IS_SCALING) != MatrixTypes.TRANSFORM_IS_IDENTITY) {
            rect.X *= matrix._m11;
            rect.Y *= matrix._m22;
            rect.Width *= matrix._m11;
            rect.Height *= matrix._m22;
            if (rect.Width < 0.0) {
                rect.X += rect.Width;
                rect.Width = -rect.Width;
            }
            if (rect.Height < 0.0) {
                rect.Y += rect.Height;
                rect.Height = -rect.Height;
            }
        }
        if ((type & MatrixTypes.TRANSFORM_IS_TRANSLATION) != MatrixTypes.TRANSFORM_IS_IDENTITY) {
            rect.X += matrix._offsetX;
            rect.Y += matrix._offsetY;
        }
        if (type == MatrixTypes.TRANSFORM_IS_UNKNOWN) {
            var point = matrix._transform(new ej.Diagram.Point(rect.Left, rect.Top));
            var point2 = matrix._transform(new ej.Diagram.Point(rect.Right, rect.Top));
            var point3 = matrix._transform(new ej.Diagram.Point(rect.Right, rect.Bottom));
            var point4 = matrix._transform(new ej.Diagram.Point(rect.Left, rect.Bottom));
            rect.X = Math.Min(Math.Min(point.X, point2.X), Math.Min(point3.X, point4.X));
            rect.Y = Math.Min(Math.Min(point.Y, point2.Y), Math.Min(point3.Y, point4.Y));
            rect.Width = Math.Max(Math.Max(point.X, point2.X), Math.Max(point3.X, point4.X)) - rect.X;
            rect.Height = Math.Max(Math.Max(point.Y, point2.Y), Math.Max(point3.Y, point4.Y)) - rect.Y;
        }
    };
    ej.Diagram.MatrixUtil.prototype._multiplyMatrix = function (matrix1, matrix2) {
        var type = matrix1._type;
        var type2 = matrix2._type;
        if (type2 == matrix1.MatrixTypes.TRANSFORM_IS_IDENTITY) {
            return;
        }
        if (type == matrix1.MatrixTypes.TRANSFORM_IS_IDENTITY) {
            //matrix1 = matrix2;
            matrix1._m11 = matrix2._m11;
            matrix1._m12 = matrix2._m12;
            matrix1._m21 = matrix2._m21;
            matrix1._m22 = matrix2._m22;
            matrix1._offsetX = matrix2._offsetX;
            matrix1._offsetY = matrix2._offsetY;
            matrix1._type = matrix2._type;
            return;
        }
        if (type2 == matrix1.MatrixTypes.TRANSFORM_IS_TRANSLATION) {
            matrix1._offsetX += matrix2._offsetX;
            matrix1._offsetY += matrix2._offsetY;
            if (type != matrix1.MatrixTypes.TRANSFORM_IS_UNKNOWN) {
                matrix1._type |= matrix1.MatrixTypes.TRANSFORM_IS_TRANSLATION;
            }
            return;
        }
        if (type != matrix1.MatrixTypes.TRANSFORM_IS_TRANSLATION) {
            var num = type << 4 | type2;
            var num2 = num;
            switch (num2) {
                case 34:
                    {
                        matrix1._m11 *= matrix2._m11;
                        matrix1._m22 *= matrix2._m22;
                        return;
                    }
                case 35:
                    {
                        matrix1._m11 *= matrix2._m11;
                        matrix1._m22 *= matrix2._m22;
                        matrix1._offsetX = matrix2._offsetX;
                        matrix1._offsetY = matrix2._offsetY;
                        matrix1._type = (this.MatrixTypes.TRANSFORM_IS_TRANSLATION | this.MatrixTypes.TRANSFORM_IS_SCALING);
                        return;
                    }
                case 36:
                    {
                        break;
                    }
                default:
                    {
                        switch (num2) {
                            case 50:
                                {
                                    matrix1._m11 *= matrix2._m11;
                                    matrix1._m22 *= matrix2._m22;
                                    matrix1._offsetX *= matrix2._m11;
                                    matrix1._offsetY *= matrix2._m22;
                                    return;
                                }
                            case 51:
                                {
                                    matrix1._m11 *= matrix2._m11;
                                    matrix1._m22 *= matrix2._m22;
                                    matrix1._offsetX = matrix2._m11 * matrix1._offsetX + matrix2._offsetX;
                                    matrix1._offsetY = matrix2._m22 * matrix1._offsetY + matrix2._offsetY;
                                    return;
                                }
                            case 52:
                                {
                                    break;
                                }
                            default:
                                {
                                    switch (num2) {
                                        case 66:
                                        case 67:
                                        case 68:
                                            {
                                                break;
                                            }
                                        default:
                                            {
                                                return;
                                            }
                                    }
                                    break;
                                }
                        }
                        break;
                    }
            }
            //var m11=matrix1._m11 * matrix2._m11 + matrix1._m12 * matrix2._m21;
            //var m12=matrix1._m11 * matrix2._m12 + matrix1._m12 * matrix2._m22;
            //var m21= matrix1._m21 * matrix2._m11 + matrix1._m22 * matrix2._m21;
            //var m22 = matrix1._offsetX * matrix2._m12 + matrix1._offsetY * matrix2._m22 + matrix2._offsetY;

            var result = new ej.Diagram.Matrix(matrix1._m11 * matrix2._m11 + matrix1._m12 * matrix2._m21, matrix1._m11 * matrix2._m12 + matrix1._m12 * matrix2._m22, matrix1._m21 * matrix2._m11 + matrix1._m22 * matrix2._m21, matrix1._m21 * matrix2._m12 + matrix1._m22 * matrix2._m22, matrix1._offsetX * matrix2._m11 + matrix1._offsetY * matrix2._m21 + matrix2._offsetX, matrix1._offsetX * matrix2._m12 + matrix1._offsetY * matrix2._m22 + matrix2._offsetY);
            matrix1._m11 = result._m11;
            matrix1._m12 = result._m12;
            matrix1._m21 = result._m21;
            matrix1._m22 = result._m22;
            matrix1._offsetX = result._offsetX;
            matrix1._offsetY = result._offsetY;
            matrix1._type = result._type;
            return;
        }
        var offsetX = matrix1._offsetX;
        var offsetY = matrix1._offsetY;
        matrix1._m11 = matrix2._m11;
        matrix1._m12 = matrix2._m12;
        matrix1._m21 = matrix2._m21;
        matrix1._m22 = matrix2._m22;
        matrix1._offsetX = matrix2._offsetX;
        matrix1._offsetY = matrix2._offsetY;
        matrix1._type = matrix2._type;
        matrix1._offsetX = offsetX * matrix2._m11 + offsetY * matrix2._m21 + matrix2._offsetX;
        matrix1._offsetY = offsetX * matrix2._m12 + offsetY * matrix2._m22 + matrix2._offsetY;
        if (type2 == matrix1.MatrixTypes.TRANSFORM_IS_UNKNOWN) {
            matrix1._type = matrix1.MatrixTypes.TRANSFORM_IS_UNKNOWN;
            return;
        }
        matrix1._type = (matrix1.MatrixTypes.TRANSFORM_IS_TRANSLATION | matrix1.MatrixTypes.TRANSFORM_IS_SCALING);
    };

    ej.Diagram.MatrixUtil.prototype._prependOffset = function (matrix, offsetX, offsetY) {
        if (matrix._type == MatrixTypes.TRANSFORM_IS_IDENTITY) {
            matrix = new MatrixExt(1.0, 0.0, 0.0, 1.0, offsetX, offsetY);
            matrix._type = matrix.MatrixTypes.TRANSFORM_IS_TRANSLATION;
            return;
        }
        matrix._offsetX += matrix._m11 * offsetX + matrix._m21 * offsetY;
        matrix._offsetY += matrix._m12 * offsetX + matrix._m22 * offsetY;
        if (matrix._type != matrix.MatrixTypes.TRANSFORM_IS_UNKNOWN) {
            matrix._type |= matrix.MatrixTypes.TRANSFORM_IS_TRANSLATION;
        }
    };


})(jQuery, Syncfusion);;
(function ($, ej) {
    "use strict";

    //#region HitTesting
    var HitTesting;
    (function (HitTesting) {
        function _getResizeHandleCenter(shape, position, scale) {
            if (!scale) scale = 1;
            var _corners = shape._corners;
            var center = new ej.Diagram.Point(0, 0);
            switch (position) {
                case "topLeft":
                    center = _corners._topLeft;
                    break;
                case "topCenter":
                    center = _corners._top;
                    break;
                case "topRight":
                    center = _corners._topRight;
                    break;
                case "middleLeft":
                    center = _corners._left;
                    break;
                case "middleRight":
                    center = _corners._right;
                    break;
                case "bottomLeft":
                    center = _corners._bottomLeft;
                    break;
                case "bottomCenter":
                    center = _corners._bottom;
                    break;
                case "bottomRight":
                    center = _corners._bottomRight;
                    break;
                case "pivot":
                    center = new ej.Diagram.Point(shape.offsetX, shape.offsetY);
                    break;
                case "rotate":
                    var thumb_height = 20;
                    var dist = 20 / scale; //distance between element top and rotator thumb
                    var pt = new ej.Diagram.Point(shape.offsetX, (shape.offsetY - shape.height * shape.pivot.y) - thumb_height / 2 - dist);
                    var matrix = new ej.Diagram.Matrix()._createIdentity();
                    matrix._rotateAt(shape.rotateAngle, shape.offsetX, shape.offsetY);
                    center = matrix._transform(pt);
                    break;
            }
            return center;
        }
        HitTesting._getResizeHandleCenter = _getResizeHandleCenter;
        function findResizeHandleAtPoint(node, point) {
            var resizeDirection = null;
            if (node) {
                //var bounds = node.getBoundingBox();
                var center;
                var success = false;
                var directions = ["topLeft", "topCenter", "topRight", "middleLeft", "middleRight", "bottomLeft", "bottomCenter", "bottomRight"];
                for (var i = 0, len = directions.length; i < len; i++) {
                    center = _getResizeHandleCenter(node, directions[i]);
                    if (center != new ej.Diagram.Point(0, 0)) {

                        success = ej.Diagram.Geometry.checkPointOnCircle(point, new ej.Diagram.Point(center.x, center.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                        if (success) {
                            resizeDirection = directions[i];
                            break;
                        }
                    }
                }
            }
            return resizeDirection;
        }
        HitTesting.findResizeHandleAtPoint = findResizeHandleAtPoint;
        function findRotationHandleAtPoint(node, point, scale) {
            var center = _getResizeHandleCenter(node, "rotate", scale);
            var success = ej.Diagram.Geometry.checkPointOnCircle(point, new ej.Diagram.Point(center.x, center.y), 10); // right now 7 is default handle radius size, need to give as constant variable
            if (success) {
                return true;

            }
            return false;
        }
        HitTesting.findRotationHandleAtPoint = findRotationHandleAtPoint;
        function findPivotHandleAtPoint(node, point) {
            var center = _getResizeHandleCenter(node, "pivot");
            var success = ej.Diagram.Geometry.checkPointOnCircle(point, new ej.Diagram.Point(center.x, center.y), 7); // right now 7 is default handle radius size, need to give as constant variable
            if (success) {
                return true;

            }
            return false;
        }
        HitTesting.findPivotHandleAtPoint = findPivotHandleAtPoint;
        function findEndPointHandleAtPoint(connector, point) {
            var endPointHandle = null;
            if (connector) {
                var success = false;
                var points = connector.line.points;
                var pt = points[points.length - 1];
                success = ej.Diagram.Geometry.checkPointOnCircle(point, new ej.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                if (success) {
                    endPointHandle = "targetEndPoint";
                }
                if (!success) {
                    pt = points[0];
                    success = ej.Diagram.Geometry.checkPointOnCircle(point, new ej.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                    if (success) {
                        endPointHandle = "sourceEndPoint";
                    }
                }
                if (!success && connector.line.type == "bezier") {
                    pt = connector.line._point1;
                    success = ej.Diagram.Geometry.checkPointOnCircle(point, new ej.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                    if (success) {
                        endPointHandle = "bezierpoint1";
                    }
                    if (!success) {
                        var pt1 = connector.line._point2;
                        success = ej.Diagram.Geometry.checkPointOnCircle(point, new ej.Diagram.Point(pt1.x, pt1.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                        if (success) {
                            endPointHandle = "bezierpoint2";
                        }
                    }
                }
            }
            return endPointHandle;
        }
        HitTesting.findEndPointHandleAtPoint = findEndPointHandleAtPoint;
    })(HitTesting || (HitTesting = {}));

    ej.Diagram.HitTesting = HitTesting;
    //#endregion

    //#region Util
    var Util;
    (function (Util) {
        function attr(element, attributes) {
            for (var atr in attributes) {
                element.setAttribute(atr.toString(), attributes[atr]);
            }
        }
        Util.attr = attr;
        function randomId() {
            var chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz";
            var id = "";
            //get 4 random chars
            for (var i = 0; i < 4; i++) {
                var num = Math.floor(Math.random() * chars.length);
                id += chars.substring(num, num + 1);
            }
            return id;
        }
        Util.randomId = randomId;
        function htmlEncode(html) {
            var html = html.replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&apos;/g, '\'');
            return html;
        }
        Util.htmlEncode = htmlEncode;

        function clone(object) {
            var clonedObject;
            if (object.type === "pseudoGroup") {
                clonedObject = new ej.Diagram.PseudoGroup(object);
            }
            else if (object.type === "group") {
                clonedObject = new ej.Diagram.Group(object);
            }
            else if (object.type === "node") {
                clonedObject = new ej.Diagram.Node(object);
            }
            else if (object.type === "connector") {
                clonedObject = new ej.Diagram.Connector(object);
            }
            else if (object.type === "point") {
                clonedObject = new ej.Diagram.Point(object.x, object.y);
            }
            else if (object.type === "collection") {
                clonedObject = new ej.Diagram.Collection();
                for (var i = 0; i < object.length; i++) {
                    if (object[i].type === "point")
                        clonedObject.add(this.clone(new ej.Diagram.Point(object[i].x, object[i].y)));
                    else
                        clonedObject.add(this.clone(object[i]));
                }
            }
            return clonedObject;
        };

        ;
        Util.clone = clone;
    })(Util || (Util = {}));

    ej.Diagram.Util = Util;
    //#endregion

    //#region Extends
    ej.Diagram.extend = function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() { this.constructor = d; }
        __.prototype = b.prototype;
        d.prototype = new __();
    }
    //#endregion

    //#region merge
    ej.Diagram.merge = function (d, b) {
        for (var p in b) {
            if (d.hasOwnProperty(p)) {
                if (b[p] instanceof Object && d.merge) {
                    d.merge(p, b[p]);
                }
                else
                    d[p] = b[p];
            }
        }
    }
    //#endregion

    //#region Geometry
    var Geometry;
    (function (Geometry) {
        function checkPointOnCircle(point, center, radius) {
            var r2 = Math.pow(radius, 2);
            var x2 = Math.pow((point.x - center.x), 2);
            var y2 = Math.pow((point.y - center.y), 2);
            if ((x2 + y2) <= r2) {
                return true;
            }
            return false;
        }
        Geometry.checkPointOnCircle = checkPointOnCircle;
        function rect(points) {
            var rectangle = new ej.Diagram.Rectangle();
            if (arguments.length === 1) {
                var left = Number.MAX_VALUE;
                var top = Number.MAX_VALUE;
                var right = Number.MIN_VALUE;
                var bottom = Number.MIN_VALUE;
                var len = points.length;
                var x, y;
                for (var i = 0; i < len; i++) {
                    x = points[i].x;
                    y = points[i].y;
                    if (x < left) {
                        left = x;
                    }
                    if (x > right) {
                        right = x;
                    }
                    if (y < top) {
                        top = y;
                    }
                    if (y > bottom) {
                        bottom = y;
                    }
                }
                rectangle = new ej.Diagram.Rectangle(left, top, right - left, bottom - top);
            }
            else if (arguments.length === 2) {
                var ptStart = new ej.Diagram.Point(arguments[0]);
                var ptEnd = new ej.Diagram.Point(arguments[1]);
                var swap;
                if (ptStart.x > ptEnd.x) {
                    swap = ptStart.x;
                    ptStart.x = ptEnd.x;
                    ptEnd.x = swap;
                }
                if (ptStart.y > ptEnd.y) {
                    swap = ptStart.y;
                    ptStart.y = ptEnd.y;
                    ptEnd.y = swap;
                }
                rectangle = new ej.Diagram.Rectangle(ptStart.x, ptStart.y, ptEnd.x - ptStart.x, ptEnd.y - ptStart.y);
            }
            return rectangle;
        }
        Geometry.rect = rect;
        function union(rect1, rect2) {
            var x = Math.min(rect1.x, rect2.x);
            var y = Math.min(rect1.y, rect2.y);
            var width = Math.max(rect1.x + rect1.width, rect2.x + rect2.width);
            var height = Math.max(rect1.y + rect1.height, rect2.y + rect2.height);
            return new ej.Diagram.Rectangle(x, y, width, height);
        }
        Geometry.union = union;
        function distance(pt1, pt2) {
            return Math.sqrt(Math.pow(pt2.x - pt1.x, 2) + Math.pow(pt2.y - pt1.y, 2));
        }
        Geometry.distance = distance;
        function translate(pt1, dx, dy) {
            return new ej.Diagram.Point(pt1.x + dx, pt1.y + dy);
        }
        Geometry.translate = translate;
        function intersect(segment1, segment2) {
            return segmentIntersect(segment1.Point1.x, segment1.Point1.y, segment1.Point2.x, segment1.Point2.y, segment2.Point1.x, segment2.Point1.y, segment2.Point2.x, segment2.Point2.y);
        }
        Geometry.intersect = intersect;
        function findAngle(point1, point2) {
            var angle = Math.atan2(point2.y - point1.y, point2.x - point1.x);
            angle = (180 * angle / Math.PI);
            angle %= 360;
            if (angle < 0) {
                angle += 360;
            }
            return angle;
        }
        Geometry.findAngle = findAngle;
        function segmentIntersect(x1, y1, x2, y2, x3, y3, x4, y4) {
            var a1, a2, b1, b2, c1, c2, x, y;
            var r1, r2, r3, r4;
            var denom, offset, num;
            a1 = y2 - y1;
            b1 = x1 - x2;
            c1 = (x2 * y1) - (x1 * y2);
            r3 = ((a1 * x3) + (b1 * y3) + c1);
            r4 = ((a1 * x4) + (b1 * y4) + c1);
            if ((r3 != 0) && (r4 != 0) && sameSign(r3, r4)) {
                return null;
            }
            a2 = y4 - y3;
            b2 = x3 - x4;
            c2 = (x4 * y3) - (x3 * y4);
            r1 = (a2 * x1) + (b2 * y1) + c2;
            r2 = (a2 * x2) + (b2 * y2) + c2;
            if ((r1 != 0) && (r2 != 0) && (sameSign(r1, r2))) {
                return null;
            }
            denom = (a1 * b2) - (a2 * b1);

            if (denom == 0) {
                return null;
            }

            if (denom < 0) {
                offset = -denom / 2;
            }
            else {
                offset = denom / 2;
            }
            num = (b1 * c2) - (b2 * c1);
            if (num < 0) {
                x = (num - offset) / denom;
            }
            else {
                x = (num + offset) / denom;
            }

            num = (a2 * c1) - (a1 * c2);
            if (num < 0) {
                y = (num - offset) / denom;
            }
            else {
                y = (num + offset) / denom;
            }
            return new ej.Diagram.Point(x, y);
        }
        function sameSign(a, b) {
            return ((a * b) >= 0);
        }
    })(Geometry || (Geometry = {}));

    ej.Diagram.Geometry = Geometry;
    //#endregion    

    //#region Segment
    var Segment = (function () {
        function Segment(point1, point2) {
            this.Point1 = point1;
            this.Point2 = point2;
        }
        Segment.prototype.length = function () {
            return ej.Diagram.Geometry.distance(this.Point1, this.Point2);
        };
        Segment.prototype.intersect = function (segment) {
            return ej.Diagram.Geometry.intersect(this, segment);
        };
        Segment.prototype.midPoint = function () {
            return new ej.Diagram.Point((this.Point1.x + this.Point2.x) / 2, (this.Point1.y + this.Point2.y) / 2);
        };
        return Segment;
    })();

    ej.Diagram.Segment = function (point1, point2) {
        return new Segment(point1, point2);
    };
    //#endregion

    //#region Point
    var Point = (function () {
        function Point(x, y) {
            this.x = Number(x) || 0;
            this.y = Number(y) || 0;
            this.type = "point";
        }
        Point.prototype.isEmpty = function () {
            return this.x === 0 && this.y === 0;
        };
        Point.prototype.toString = function () {
            return this.x + ", " + this.y;
        };
        return Point;
    })();
    ej.Diagram.Point = function (x, y) {
        return new Point(x, y)
    }
    //#endregion

    //#region Size
    var Size = (function () {
        function Size(width, height) {
            this.width = Number(width) || 0;
            this.height = Number(height) || 0;
        }
        Size.prototype.isEmpty = function () {
            return this.width === 0 && this.height === 0;
        };
        return Size;
    })();
    ej.Diagram.Size = function (width, height) {
        return new Size(width, height);
    };
    //#endregion

    //#region Rectangle
    var Rectangle = (function () {
        function Rectangle(x, y, width, height) {
            this.x = Number(x) || 0;
            this.y = Number(y) || 0;
            this.width = Number(width) || 0;
            this.height = Number(height) || 0;
            this.type = "rectangle";
        }
        Rectangle.prototype.left = function () {
            return this.x;
        };
        Rectangle.prototype.right = function () {
            return this.x + this.width;
        };
        Rectangle.prototype.top = function () {
            return this.y;
        };
        Rectangle.prototype.bottom = function () {
            return this.y + this.height;
        };
        Rectangle.prototype.center = function () {
            return new ej.Diagram.Point(this.x + this.width / 2, this.y + this.height / 2);
        };
        Rectangle.prototype.contains = function (value) {
            if (value && value.type === "point")
                return ((((this.x <= value.x) && (value.x < (this.x + this.width)))
                && (this.y <= value.y)) && (value.y < (this.y + this.height)));
            else if (value && value.type === "rectangle")
                return ((((this.x <= value.x) && ((value.x + value.width) <= (this.x + this.width)))
               && (this.y <= value.y)) && ((value.y + value.height) <= (this.y + this.height)));
            return false;
        };
        Rectangle.prototype.inflate = function (x, y) {
            this.x -= x;
            this.y -= y;
            this.width += 2 * x;
            this.height += 2 * y;
        };
        Rectangle.prototype.intersectsWith = function (rectangle) {
            return ((((rectangle.x < (this.x + this.width)) && (this.x < (rectangle.x + rectangle.width)))
                && (rectangle.y < (this.y + this.height))) && (this.y < (rectangle.y + rectangle.height)));
        };
        Rectangle.prototype.location = function () {
            return new ej.Diagram.Point(this.x, this.y);
        };
        Rectangle.prototype.size = function () {
            return new ej.Diagram.Size(this.width, this.height);
        };
        Rectangle.prototype.isEmpty = function () {
            return this.x === 0 && this.y === 0 && this.width === 0 && this.height === 0;
        };
        Rectangle.prototype.union = function (rectangle) {
            var x = Math.min(this.x, rectangle.x);
            var num2 = Math.max((this.x + this.width), (rectangle.x + rectangle.width));
            var y = Math.min(this.y, rectangle.y);
            var num4 = Math.max((this.y + this.height), (rectangle.y + rectangle.height));
            return new ej.Diagram.Rectangle(x, y, num2 - x, num4 - y);
        };
        Rectangle.prototype.toPoints = function () {
            var points = new ej.Diagram.Collection();
            points.add(new ej.Diagram.Point(this.x, this.y)); //topLeft
            points.add(new ej.Diagram.Point(this.x + this.width, this.y)); //topRight
            points.add(new ej.Diagram.Point(this.x + this.width, this.y + this.height)); //bottomRight
            points.add(new ej.Diagram.Point(this.x, this.y + this.height)); //bottomLeft
            return points;
        };
        Rectangle.prototype._toCorners = function () {
            var points = new ej.Diagram.Collection();
            points.add(new ej.Diagram.Point(this.x, this.y)); //topLeft
            points.add(new ej.Diagram.Point(this.x + this.width / 2, this.y)); //topCenter
            points.add(new ej.Diagram.Point(this.x + this.width, this.y)); //topRight
            points.add(new ej.Diagram.Point(this.x, this.y + this.height / 2)); //middleleft
            points.add(new ej.Diagram.Point(this.x + this.width, this.y + this.height / 2)); //middleright
            points.add(new ej.Diagram.Point(this.x, this.y + this.height)); //bottomLeft
            points.add(new ej.Diagram.Point(this.x + this.width / 2, this.y + this.height)); //bottomCenter
            points.add(new ej.Diagram.Point(this.x + this.width, this.y + this.height)); //bottomRight
            return points;
        };
        return Rectangle;
    })();
    ej.Diagram.Rectangle = function (x, y, width, height) {
        return new Rectangle(x, y, width, height);
    };
    //#endregion

    //#region Collection
    var Collection = (function (base) {
        ej.Diagram.extend(Collection, base);
        function Collection() {
            this._events = [];
            this.type = "collection";
        }
        Collection.prototype = {
            add: function (item) {
                if (!this.contains(item)) {
                    var args = { changeType: "insert", element: item, cancel: false };
                    this.raiseEvent("collectionchanging", args);
                    if (!args.cancel) {
                        base.prototype.push.call(this, item);
                        args = { changeType: "insert", element: item };
                        this.raiseEvent("collectionchanged", args);
                        return true;
                    }
                }
                return false;
            },
            remove: function (item) {
                if (this.contains(item)) {
                    var args = { changeType: "remove", element: item, cancel: false };
                    this.raiseEvent("collectionchanging", args);
                    if (!args.cancel) {
                        var index = base.prototype.indexOf.call(this, item);
                        base.prototype.splice.call(this, index, 1);
                        args = { changeType: "remove", element: item, cancel: false };
                        this.raiseEvent("collectionchanged", args);
                        return true;
                    }
                }
                return false;
            },
            contains: function (item) {
                if (item.hasOwnProperty("name")) {
                    for (var i = 0, len = this.length; i < len; i++) {
                        if (this[i].name === item.name) {
                            return true;
                        }
                    }
                    return false;
                }
                else
                    return this.indexOf(item) === -1 ? false : true;
            },
            indexOf: function (item) {
                return base.prototype.indexOf.call(this, item);
            },
            clear: function () {
                if (this.length > 0) {
                    var items = base.prototype.slice.call(this, 0, this.length);
                    var args = { changeType: "remove", element: items, cancel: false };
                    this.raiseEvent("collectionchanging", args);
                    if (!args.cancel) {
                        base.prototype.splice.call(this, 0, this.length);
                        args = { changeType: "remove", element: items };
                        this.raiseEvent("collectionchanged", args);
                        return true;
                    }
                    return false;
                } return true;
            },
            reverse: function () {
                base.prototype.reverse.call(this);
            },
            slice: function (start, end) {
                return this._slice(new ej.Diagram.Collection(), start, end);
            },
            isEmpty: function () {
                return !this.length || this.length === 0;
            },
            addEventHandler: function (type, handler) {
                this._events.push({ "type": type, "handler": handler });
            },
            removeEventHandler: function (type, handler) {
                for (var i = 0, len = this._events.length; i < len; i++) {
                    if (this._events[i].type == type && this._events[i].handler == handler) {
                        this._events.splice(i, 1);
                    }
                }
            },
            raiseEvent: function (type, args) {
                for (var i = 0, len = this._events.length; i < len; i++) {
                    var event = this._events[i];
                    if (event.type == type) {
                        if (event.handler)
                            event.handler(args);
                    }
                }
            },
            _slice: function (collection, start, end) {
                var len = this.length, s = 0, e = len;
                if (start) {
                    s = start;
                }
                if (end) {
                    e = end;
                }
                for (var i = s; i < e; i++)
                    collection.push(this[i]);
                return collection;
            },
        };
        return Collection;
    })(Array);
    ej.Diagram.Collection = function () {
        return new Collection();
    };
    //#endregion

    //#region Palette
    var Palette = (function () {
        function Palette(props) {
            this.name = "";
            this.expanded = false;
            this._events = [];
            this.subControlId = null;
            this.items = new ej.Diagram.Collection();
            ej.Diagram.merge(this, props);
            this._wireEvents();
        }
        Palette.prototype.add = function (item) {
            this.items.add(item);
        };
        Palette.prototype.remove = function (item) {
            this.items.remove(item);
        };
        Palette.prototype.addEventHandler = function (type, handler) {
            this._events.push({ "type": type, "handler": handler });
        };
        Palette.prototype.removeEventHandler = function (type, handler) {
            for (var i = 0, len = this._events.length; i < len; i++) {
                if (this._events[i].type == type && this._events[i].handler == handler) {
                    this._events.splice(i, 1);
                }
            }
        };
        Palette.prototype._wireEvents = function () {

        };
        Palette.prototype._raiseEvent = function (type, args) {
            for (var i = 0, len = this._events.length; i < len; i++) {
                var event = this._events[i];
                if (event.type == type) {
                    if (event.handler)
                        event.handler({ changeType: args.changeType, palette: this, item: args.element });
                }
            }
        };
        Palette.prototype._collectionChanged = function (evt) {
            var args = { changeType: evt.changeType, element: evt.element };
            this._raiseEvent("collectionchanged", args);
        };
        Palette.prototype.merge = function (prop, value) {
            this.items = new ej.Diagram.Collection();
            if (prop.toString() === "items") {
                for (var i = 0; i < value.length; i++) {
                    if (value[i].children != undefined) {
                        var group = ej.Diagram.Group(value[i]);
                        this.items.add(group);
                    }
                    else {
                        if (value[i].shape !== undefined) {
                            var node = ej.Diagram.Node(value[i]);
                            this.items.add(node);
                        }
                        else {
                            var node = ej.Diagram.Connector(value[i]);
                            this.items.add(node);
                        }
                    }
                }
            }
        };
        return Palette;
    })();
    ej.Diagram.Palette = function (props) {
        return new Palette(props);
    };
    //#endregion

    //#region Shapes
    var Shapes;
    (function (Shapes) {
        Shapes.Rectangle = (function () {
            function Rectangle(props) {
                this.type = "rectangle";
                this.cornerRadius = 0;
                ej.Diagram.merge(this, props);
            }
            return Rectangle;
        })();

        Shapes.Ellipse = (function () {
            function Ellipse() {
                this.type = "ellipse";
            }
            return Ellipse;
        })();

        Shapes.Image = (function () {
            function Image(props) {
                this.type = "image";
                this.src = props.src;
                ej.Diagram.merge(this, props);
            }
            return Image;
        })();

        Shapes.Path = (function () {
            function Path(props) {
                this.type = "path";
                this.pathData = "";
                ej.Diagram.merge(this, props);
            }
            return Path;
        })();

        Shapes.Text = (function () {
            function Text(props) {
                this.type = "text";
                this.textBlock = new ej.Diagram.TextBlock();
                ej.Diagram.merge(this, props);
            }
            Text.prototype.merge = function (prop, value) {
                if (prop.toString() === "textBlock") {
                    this.textBlock = new ej.Diagram.TextBlock(value);
                }
            };
            return Text;
        })();

        Shapes.Polygon = (function () {
            function Polygon(props) {
                this.type = "polygon";
                this.points = new ej.Diagram.Collection();
                ej.Diagram.merge(this, props);
            }
            Polygon.prototype.merge = function (prop, value) {
                if (prop.toString() === "points") {
                    for (var i = 0, len = value.length; i < len; i++) {
                        this.points.add(new ej.Diagram.Point(value[i].x, value[i].y));
                    }
                }
            };
            return Polygon;
        })();

        Shapes.Html = (function () {
            function Html(props) {
                this.type = "html";
                this.html = null;
                ej.Diagram.merge(this, props);
            }
            Html.prototype.merge = function (prop, value) {
                if (prop.toString() === "html") {
                    if (value instanceof HTMLElement) {
                        this.html = value.cloneNode(true);
                    }
                    else if (value instanceof String) {
                        this.html = value;
                    }
                }
            };
            return Html;
        })();

        Shapes.Native = (function () {
            function Native(props) {
                this.type = "native";
                this.content = null;
                ej.Diagram.merge(this, props);
            }
            Native.prototype.merge = function (prop, value) {
                if (prop.toString() === "content") {
                    if (value instanceof SVGElement) {
                        this.content = value.cloneNode(true);
                    }
                }
            };
            return Native;
        })();

    })(Shapes || (Shapes = {}));

    ej.Diagram.Shapes = {};
    ej.Diagram.Shapes.Rectangle = function (props) {
        return new Shapes.Rectangle(props);
    };
    ej.Diagram.Shapes.Ellipse = function () {
        return new Shapes.Ellipse();
    };
    ej.Diagram.Shapes.Image = function (props) {
        return new Shapes.Image(props);
    };
    ej.Diagram.Shapes.Path = function (props) {
        return new Shapes.Path(props);
    };
    ej.Diagram.Shapes.Text = function (props) {
        return new Shapes.Text(props);
    };
    ej.Diagram.Shapes.Polygon = function (props) {
        return new Shapes.Polygon(props);
    };
    ej.Diagram.Shapes.Html = function (props) {
        return new Shapes.Html(props);
    };
    ej.Diagram.Shapes.Native = function (props) {
        return new Shapes.Native(props);
    };
    //#endregion

    //#region Lines
    var Lines;
    (function (Lines) {
        var Straight = (function () {
            function Straight(props) {
                this.type = "straight";
                this.points = new ej.Diagram.Collection();
                this.startPoint = new ej.Diagram.Point(0, 0);
                this.endPoint = new ej.Diagram.Point(0, 0);
                this.points.add(this.startPoint);
                this.points.add(this.endPoint);
                this.parent = null;
                ej.Diagram.merge(this, props);
            }
            Straight.prototype.merge = function (prop, value) {
                if (prop.toString() === "startPoint") {
                    value = new ej.Diagram.Point(value.x, value.y);
                    this._setEndPoint(value, false);
                }
                if (prop.toString() === "endPoint") {
                    value = new ej.Diagram.Point(value.x, value.y);
                    this._setEndPoint(value, true);
                }
            };
            Straight.prototype._translate = function (dx, dy, parent) {
                if (this.points) {
                    var points = this.points;
                    if (points.type === "collection") {
                        var bounds = ej.Diagram.Geometry.rect(points);
                        for (var i = 0, len = points.length; i < len; ++i) {
                            var point = ej.Diagram.Geometry.translate(points[i], dx, dy);
                            points[i] = point;
                        }
                        this.startPoint = points[0];
                        this.endPoint = points[points.length - 1];
                    }
                }
            };
            Straight.prototype._dockNode = function (targetNode, sourceNode, parent) {
                if (parent.isTargetConnected() && parent.isSourceConnected() && !parent.targetPort && !parent.sourcePort) {
                    this._dock(targetNode, sourceNode, true);
                    this._dock(sourceNode, targetNode, false);
                }
                else if (parent.isTargetConnected() && !parent.targetPort) {
                    this._dock(targetNode, sourceNode, true);
                }
                else if (parent.isSourceConnected() && !parent.sourcePort) {
                    this._dock(targetNode, sourceNode, false);
                }
            };
            Straight.prototype._dock = function (targetNode, sourceNode, isTarget) {
                var mat = new ej.Diagram.Matrix();
                var matrix;
                var targetBounds;
                var sourceBounds;
                var points;
                var i, len;
                if (targetNode) {
                    targetBounds = targetNode.getBoundingBox();
                    points = targetBounds.toPoints();
                    matrix = mat._createIdentity();
                    matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                    for (i = 0, len = points.length; i < len; i++) {
                        points[i] = matrix._transform(points[i]);
                    }
                    targetBounds = ej.Diagram.Geometry.rect(points);
                    if (sourceNode) {
                        sourceBounds = sourceNode.getBoundingBox();
                        points = sourceBounds.toPoints();
                        matrix = mat._createIdentity();
                        matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                        for (i = 0, len = points.length; i < len; i++) {
                            points[i] = matrix._transform(points[i]);
                        }
                        sourceBounds = ej.Diagram.Geometry.rect(points);
                        this._setEndPoint(sourceBounds.center(), !isTarget);
                        this._setEndPoint(targetBounds.center(), isTarget);
                        this._dockBounds(sourceNode, !isTarget, parent);
                    }
                    this._setEndPoint(targetBounds.center(), isTarget);
                    this._dockBounds(targetNode, isTarget, parent);
                }
            };
            Straight.prototype._dockBounds = function (node, isTarget, parent) {
                var segmentPoints = node.getBoundingBox().toPoints();
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                var i, len;
                matrix._rotateAt(node.rotateAngle, node.offsetX, node.offsetY);
                for (i = 0, len = segmentPoints.length; i < len; i++) {
                    segmentPoints[i] = matrix._transform(segmentPoints[i]);
                }
                var thisSegment = new ej.Diagram.Segment(this.startPoint, this.endPoint);
                len = segmentPoints.length;
                var segment = new ej.Diagram.Segment(segmentPoints[len - 1], segmentPoints[0]);
                var intersection = thisSegment.intersect(segment);
                if (!intersection) {
                    for (i = 0; i < len - 1; i++) {
                        segment = new ej.Diagram.Segment(segmentPoints[i], segmentPoints[i + 1]);
                        intersection = thisSegment.intersect(segment);
                        if (intersection)
                            break;
                    }
                }
                if (intersection) {
                    this._setEndPoint(intersection, isTarget);
                }
            };
            Straight.prototype._dockPortToPort = function (targetNode, targetPort, sourceNode, sourcePort) {
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                var portLocation;
                if (targetPort) {
                    matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                    portLocation = matrix._transform(targetPort._getPosition(targetNode.getBoundingBox()));
                    this._setEndPoint(portLocation, true);
                }
                if (sourcePort) {
                    matrix = mat._createIdentity();
                    matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                    portLocation = matrix._transform(sourcePort._getPosition(sourceNode.getBoundingBox()));
                    this._setEndPoint(portLocation, false);
                }
            };
            Straight.prototype._dockPortToNode = function (sourceNode, sourcePort, isTarget, parent) {
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                var portLocation = matrix._transform(sourcePort._getPosition(sourceNode.getBoundingBox()));
                this._setEndPoint(portLocation, isTarget);
            };
            Straight.prototype._dockPortToEndPoint = function (targetNode, targetPort, isTarget) {
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                var portLocation = matrix._transform(targetPort._getPosition(targetNode.getBoundingBox()));
                this._setEndPoint(portLocation, isTarget);
            };
            Straight.prototype._updateEndPoint = function (dx, dy, isTarget) {
                var point;
                if (isTarget) {
                    point = ej.Diagram.Geometry.translate(this.points[this.points.length - 1], dx, dy);
                    this.endPoint = this.points[this.points.length - 1] = point;
                }
                else {
                    point = ej.Diagram.Geometry.translate(this.points[0], dx, dy);
                    this.startPoint = this.points[0] = point;
                }
            };
            Straight.prototype._setEndPoint = function (point, isTarget) {
                if (isTarget) {
                    this.endPoint = this.points[this.points.length - 1] = point;
                }
                else {
                    this.startPoint = this.points[0] = point;
                }
            };
            Straight.prototype._setPoints = function (points) {
                this.startPoint = this.points[0] = points[0];
                this.endPoint = this.points[this.points.length - 1] = points[points.length - 1];
            };
            return Straight;
        })();
        Lines.Straight = Straight;

        var Orthogonal = (function () {
            function Orthogonal(props) {
                this.type = "orthogonal";
                this.startPoint = new ej.Diagram.Point(0, 0);
                this.endPoint = new ej.Diagram.Point(0, 0);
                this.points = new ej.Diagram.Collection();
                this.points.add(this.startPoint);
                this._addControlPoints();
                this.points.add(this.endPoint);
                ej.Diagram.merge(this, props);
            }
            Orthogonal.prototype.merge = function (prop, value) {

                if (prop.toString() === "startPoint") {
                    value = new ej.Diagram.Point(value.x, value.y);
                    this._setEndPoint(value, false);
                }
                if (prop.toString() === "endPoint") {
                    value = new ej.Diagram.Point(value.x, value.y);
                    this._setEndPoint(value, true);
                }
                if (prop.toString() === "points") {
                    this.points = new ej.Diagram.Collection();
                    var points = value;
                    for (var i = 0; i < points.length; i++) {
                        this.points.add(new ej.Diagram.Point(points[i].x, points[i].y));
                    }

                }
            };
            Orthogonal.prototype.segments = function () {
                var segment;
                var segments = new ej.Diagram.Collection();
                var points = this.points;
                for (var i = 0, len = points.length - 1; i < len; i++) {
                    segments.add(new ej.Diagram.Segment(points[i], points[i + 1]));
                }
                return segments;
            };
            Orthogonal.prototype._resetControlPoints = function () {
                var points = new ej.Diagram.Collection();
                points.add(this.startPoint);
                points.add(this.endPoint);
                this.points.clear();
                this.startPoint = points[0];
                this.points.add(points[0]);
                var ctrlPoints = this._constructControlPoints();
                this.points.add(ctrlPoints[0]);
                this.points.add(ctrlPoints[1]);
                this.endPoint = points[1];
                this.points.add(points[1]);
            };
            Orthogonal.prototype._translate = function (dx, dy, parent) {
                if (parent) {
                    if (!parent.isTargetConnected() || !parent.isSourceConnected()) {
                        this._resetControlPoints();
                    }
                }
                var points = this.points;
                if (points.type === "collection") {
                    var bounds = ej.Diagram.Geometry.rect(points);
                    for (var i = 0, len = points.length; i < len; ++i) {
                        var point = ej.Diagram.Geometry.translate(points[i], dx, dy);
                        points[i] = point;
                    }
                    this.startPoint = points[0];
                    this.endPoint = points[points.length - 1];
                }
            };
            Orthogonal.prototype._addControlPoints = function () {
                var ctrlPoints = this._constructControlPoints();
                this.points.add(ctrlPoints[0]);
                this.points.add(ctrlPoints[1]);
            };
            Orthogonal.prototype._constructControlPoints = function (update) {
                var divideX = false;
                if (update != undefined) {
                    divideX = update;
                }
                else {
                    var dx = Math.abs(this.startPoint.x - this.endPoint.x);
                    var dy = Math.abs(this.startPoint.y - this.endPoint.y);
                    if (dx > dy) {
                        divideX = true;
                    }
                }
                var points = new ej.Diagram.Collection();
                var ctrlPoint1;
                var ctrlPoint2;
                if (divideX) {
                    ctrlPoint1 = new ej.Diagram.Point(this.startPoint.x + (this.endPoint.x - this.startPoint.x) / 2, this.startPoint.y);
                    ctrlPoint2 = new ej.Diagram.Point(this.startPoint.x + (this.endPoint.x - this.startPoint.x) / 2, this.endPoint.y);
                } else {
                    ctrlPoint1 = new ej.Diagram.Point(this.startPoint.x, this.startPoint.y + (this.endPoint.y - this.startPoint.y) / 2);
                    ctrlPoint2 = new ej.Diagram.Point(this.endPoint.x, this.startPoint.y + (this.endPoint.y - this.startPoint.y) / 2);
                }
                points.add(ctrlPoint1);
                points.add(ctrlPoint2);
                return points;
            };
            Orthogonal.prototype._updateControlPoints = function (side) {
                var ctrlPoints = this._constructControlPoints(side);
                this.points[1] = ctrlPoints[0];
                this.points[this.points.length - 2] = ctrlPoints[1];
            };
            Orthogonal.prototype._getIntersectSegment = function (bounds, isTarget) {
                var segment;
                var i;
                var len;
                var segments = this.segments();
                if (isTarget) {
                    for (i = segments.length - 1; i >= 0; i--) {
                        segment = segments[i];
                        if (bounds.contains(segment.Point1) && bounds.contains(segment.Point2))
                            continue;
                        else
                            break;
                    }
                }
                else {
                    for (i = 0, len = segments.length; i < len; i++) {
                        segment = segments[i];
                        if (bounds.contains(segment.Point1) && bounds.contains(segment.Point2))
                            continue;
                        else
                            break;
                    }
                }
                return segment;
            };
            Orthogonal.prototype._getDirection = function (targetNode, targetPort, sourceNode, sourcePort) {
                var bounds = targetNode.getBoundingBox();
                var points = bounds.toPoints();
                var center = bounds.center();
                var point = targetPort._getPosition(bounds);
                var direction;
                var angle = ej.Diagram.Geometry.findAngle(point, center);
                if (angle > ej.Diagram.Geometry.findAngle(points[0], center) && angle <= ej.Diagram.Geometry.findAngle(points[1], center)) {
                    direction = "top";
                }
                else if (angle >= ej.Diagram.Geometry.findAngle(points[1], center) && angle <= ej.Diagram.Geometry.findAngle(points[2], center)) {
                    direction = "right";
                }
                else if (angle >= ej.Diagram.Geometry.findAngle(points[3], center) || angle < ej.Diagram.Geometry.findAngle(points[0], center)) {
                    direction = "left";
                }
                else if (angle >= ej.Diagram.Geometry.findAngle(points[2], center) && angle <= ej.Diagram.Geometry.findAngle(points[3], center)) {
                    direction = "bottom";
                }
                else {
                    direction = "right";
                }
                return direction;
            };
            Orthogonal.prototype._rightToLeft = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, reverse) {
                var points;
                var point;
                var x, y;
                if (targetPortLocation.x > sourcePortLocation.x) {
                    points = new ej.Diagram.Collection();
                    points.add(sourcePortLocation);
                    if ((sourcePortLocation.x + 5) >= targetBounds.left()) {
                        x = sourceBounds.right() + 15;
                        y = sourcePortLocation.y;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        if (targetPortLocation.y > sourceBounds.top()) {
                            y = targetPortLocation.y;
                        }
                        else {
                            y = targetBounds.bottom() + 15;
                        }
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        if (targetPortLocation.y <= sourceBounds.top()) {
                            x = targetBounds.right() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            y = targetBounds.top() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            x = targetPortLocation.x;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                        }
                    }
                    else {
                        x = sourceBounds.right() + (targetBounds.left() - sourceBounds.right()) / 2;
                        y = sourcePortLocation.y;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        y = targetPortLocation.y;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                    }
                    //}
                }
                else {
                    //5 segment
                    points = new ej.Diagram.Collection();
                    points.add(sourcePortLocation);
                    if (Math.abs(targetBounds.top() - sourceBounds.bottom()) < 30 || Math.abs(targetBounds.bottom() - sourceBounds.top()) < 30) {
                        if ((sourcePortLocation.x + 15) > targetBounds.right()) {
                            x = sourcePortLocation.x + 15;
                            y = sourcePortLocation.y;
                        }
                        else {
                            x = targetBounds.right() + 15;
                            y = sourcePortLocation.y;
                        }
                    }
                    else {
                        x = sourceBounds.right() + 15;
                        y = sourcePortLocation.y;
                    }
                    point = new ej.Diagram.Point(x, y);
                    points.add(point);
                    if ((targetBounds.top() - sourceBounds.bottom()) >= 30) {
                        y = targetBounds.top() - 15;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                    }
                    else if (sourcePortLocation.y > targetPortLocation.y) {
                        if ((sourceBounds.top() - targetBounds.bottom()) < 30) {
                            y = targetBounds.top() - 15;
                        }
                        else {
                            y = targetBounds.bottom() + 15;
                        }
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                    }
                    else {
                        y = targetBounds.bottom() + 15;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                    }
                    x = targetBounds.left() - 15;
                    point = new ej.Diagram.Point(x, y);
                    points.add(point);
                    point = new ej.Diagram.Point(x, targetPortLocation.y);
                    points.add(point);
                }
                points.add(targetPortLocation);
                if (reverse) {
                    points.reverse();
                }
                this._setPoints(points);
            };
            Orthogonal.prototype._rightToRight = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (targetBounds.right() >= sourceBounds.left()) {
                    if (sourcePortLocation.y >= targetBounds.top() && sourcePortLocation.y <= targetBounds.bottom()) {
                        if (sourcePortLocation.y >= targetBounds.center().y) {
                            x = sourcePortLocation.x + 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            if (sourcePortLocation.x > targetBounds.left()) {
                                if (sourcePortLocation.y > targetPortLocation.y) {
                                    y = targetBounds.bottom() + 15;
                                }
                                else {
                                    y = targetBounds.top() - 15;
                                }
                            }
                            else {
                                y = targetBounds.top() - 15;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            x = targetBounds.right() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(x, targetPortLocation.y);
                            points.add(point);
                        }
                        else {
                            point = new ej.Diagram.Point(sourcePortLocation.x + 15, sourcePortLocation.y);
                            points.add(point);
                            point = new ej.Diagram.Point(sourcePortLocation.x + 15, targetBounds.bottom() + 15);
                            points.add(point);
                            point = new ej.Diagram.Point(targetBounds.right() + 15, targetBounds.bottom() + 15);
                            points.add(point);
                            point = new ej.Diagram.Point(targetBounds.right() + 15, targetPortLocation.y);
                            points.add(point);
                        }
                    }
                    else if (targetBounds.right() < sourceBounds.right()) {
                        point = new ej.Diagram.Point(sourcePortLocation.x + 15, sourcePortLocation.y);
                        points.add(point);
                        point = new ej.Diagram.Point(sourcePortLocation.x + 15, targetPortLocation.y);
                        points.add(point);
                    }
                    else {
                        point = new ej.Diagram.Point(targetBounds.right() + 15, sourcePortLocation.y);
                        points.add(point);
                        point = new ej.Diagram.Point(targetBounds.right() + 15, targetPortLocation.y);
                        points.add(point);
                    }
                }
                else {
                    if (targetPortLocation.y >= sourceBounds.top() && targetPortLocation.y <= sourceBounds.bottom()) {
                        if (targetPortLocation.y <= sourceBounds.center().y) {
                            point = new ej.Diagram.Point(sourcePortLocation.x + 15, sourcePortLocation.y);
                            points.add(point);
                            point = new ej.Diagram.Point(sourcePortLocation.x + 15, sourceBounds.top() - 15);
                            points.add(point);
                            point = new ej.Diagram.Point(targetBounds.right() + 15, sourceBounds.top() - 15);
                            points.add(point);
                            point = new ej.Diagram.Point(targetBounds.right() + 15, targetPortLocation.y);
                            points.add(point);
                        }
                        else {
                            point = new ej.Diagram.Point(sourcePortLocation.x + 15, sourcePortLocation.y);
                            points.add(point);
                            point = new ej.Diagram.Point(sourcePortLocation.x + 15, sourceBounds.bottom() + 15);
                            points.add(point);
                            point = new ej.Diagram.Point(targetBounds.right() + 15, sourceBounds.bottom() + 15);
                            points.add(point);
                            point = new ej.Diagram.Point(targetBounds.right() + 15, targetPortLocation.y);
                            points.add(point);
                        }
                    }
                    else {
                        point = new ej.Diagram.Point(sourcePortLocation.x + 15, sourcePortLocation.y);
                        points.add(point);
                        point = new ej.Diagram.Point(sourcePortLocation.x + 15, targetPortLocation.y);
                        points.add(point);
                    }
                }
                points.add(targetPortLocation);
                this._setPoints(points);
            };
            Orthogonal.prototype._rightToTop = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, reverse) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (targetPortLocation.x >= sourcePortLocation.x) {
                    if ((sourcePortLocation.y + 15) >= targetPortLocation.y && targetBounds.height != 0) {
                        if (targetBounds.left() <= sourceBounds.right()) {
                            x = targetBounds.right() + 15;
                        }
                        else if ((targetBounds.left() - sourceBounds.right()) > 30) {
                            x = sourceBounds.right() + 15;
                        }
                        else {
                            x = sourceBounds.right() + (targetBounds.left() - sourceBounds.right()) / 2;
                        }
                        point = new ej.Diagram.Point(x, sourcePortLocation.y);
                        points.add(point);
                        y = targetBounds.top() - 15;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        point = new ej.Diagram.Point(targetPortLocation.x, y);
                        points.add(point);
                    }
                    else {
                        point = new ej.Diagram.Point(targetPortLocation.x, sourcePortLocation.y);
                        points.add(point);
                    }
                }
                else {
                    if (sourceBounds.bottom() >= targetBounds.top()) {
                        if ((sourcePortLocation.x + 15) <= targetBounds.right()) {
                            x = targetBounds.right() + 15;
                        }
                        else {
                            x = sourceBounds.right() + 15;
                        }
                        point = new ej.Diagram.Point(x, sourcePortLocation.y);
                        points.add(point);

                        if (sourceBounds.top() >= targetBounds.top()) {
                            if (targetBounds.height != 0) {
                                y = targetBounds.top() - 15;
                            }
                            else {
                                y = targetPortLocation.y;
                            }
                        }
                        else {
                            y = sourceBounds.top() - 15;
                        }

                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        point = new ej.Diagram.Point(targetPortLocation.x, y);
                        points.add(point);
                    }
                    else {
                        if ((targetBounds.top() - sourceBounds.bottom()) < 30 && targetBounds.height != 0) {
                            if ((sourceBounds.left() - targetBounds.right()) >= 30) {
                                x = sourcePortLocation.x + 15;
                                point = new ej.Diagram.Point(x, sourcePortLocation.y);
                                points.add(point);
                                y = sourceBounds.bottom() + 15;
                                point = new ej.Diagram.Point(x, y);
                                points.add(point);
                                point = new ej.Diagram.Point(sourceBounds.left() - 15, y);
                                points.add(point);
                                y = targetBounds.top() - 15;
                                point = new ej.Diagram.Point(sourceBounds.left() - 15, y);
                                points.add(point);
                                point = new ej.Diagram.Point(targetPortLocation.x, y);
                                points.add(point);
                            }
                            else {
                                point = new ej.Diagram.Point(sourcePortLocation.x + 15, sourcePortLocation.y);
                                points.add(point);
                                y = targetBounds.top() - (targetBounds.top() - sourceBounds.bottom()) / 2;
                                point = new ej.Diagram.Point(sourcePortLocation.x + 15, y);
                                points.add(point);
                                point = new ej.Diagram.Point(targetPortLocation.x, y);
                                points.add(point);
                            }
                        }
                        else {
                            x = sourcePortLocation.x + 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            if (targetBounds.height != 0) {
                                y = targetBounds.top() - 15;
                            }
                            else {
                                y = targetPortLocation.y;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                }
                points.add(targetPortLocation);
                if (reverse) {
                    points.reverse();
                }
                this._setPoints(points);
            };
            Orthogonal.prototype._rightToBottom = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, reverse) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (targetPortLocation.x >= sourcePortLocation.x) {
                    if (sourcePortLocation.y <= targetBounds.bottom()) {
                        if (targetBounds.left() > (sourceBounds.right() + 15)) {
                            x = sourcePortLocation.x + 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = targetBounds.bottom() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            x = targetBounds.right() + 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = targetBounds.bottom() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                    else {
                        if ((sourceBounds.right() + 15) > targetPortLocation.x) {
                            x = sourcePortLocation.x + 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            if ((sourceBounds.top() - targetBounds.bottom()) > 30) {
                                y = targetBounds.bottom() + 15;
                            }
                            else {
                                y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            point = new ej.Diagram.Point(targetPortLocation.x, sourcePortLocation.y);
                            points.add(point);
                        }
                    }
                }
                else {
                    if (sourcePortLocation.y <= targetBounds.bottom()) {
                        if ((sourceBounds.right() + 15) > targetBounds.right()) {
                            x = sourceBounds.right() + 15;
                        }
                        else {
                            x = targetBounds.right() + 15;
                        }
                        point = new ej.Diagram.Point(x, sourcePortLocation.y);
                        points.add(point);
                        y = targetBounds.bottom() + 15;
                        if (y <= (sourceBounds.bottom() + 15)) {
                            y = sourceBounds.bottom() + 15;
                        }
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        point = new ej.Diagram.Point(targetPortLocation.x, y);
                        points.add(point);
                    }
                    else {
                        if ((targetBounds.bottom()) < sourceBounds.top()) {
                            x = sourceBounds.right() + 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            if ((sourceBounds.top() - targetBounds.bottom()) < 30) {
                                y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                            }
                            else {
                                y = targetBounds.bottom() + 15;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            x = sourceBounds.right() + 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = sourceBounds.bottom() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                }
                points.add(targetPortLocation);
                if (reverse) {
                    points.reverse();
                }
                this._setPoints(points);
            };
            Orthogonal.prototype._leftToLeft = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (targetPortLocation.x >= sourcePortLocation.x) {
                    if (sourcePortLocation.y > targetPortLocation.y) {
                        if ((targetPortLocation.y + 15) >= sourceBounds.top()) {
                            x = sourceBounds.left() - 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = sourceBounds.top() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            if ((targetBounds.left() - sourceBounds.right()) > 30) {
                                x = targetBounds.left() - 15;
                            }
                            else {
                                x = sourceBounds.right() + (targetBounds.left() - sourceBounds.right()) / 2;
                            }

                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(x, targetPortLocation.y);
                            points.add(point);
                        }
                        else {
                            x = sourceBounds.left() - 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = targetPortLocation.y;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                    else {
                        if ((sourceBounds.bottom() + 15) < targetPortLocation.y) {
                            x = sourceBounds.left() - 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = targetPortLocation.y;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            x = sourceBounds.left() - 15;
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = sourceBounds.bottom() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            if ((targetBounds.left() - sourceBounds.right()) > 30) {
                                x = targetBounds.left() - 15;
                            }
                            else {
                                x = sourceBounds.right() + (targetBounds.left() - sourceBounds.right()) / 2;
                            }

                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(x, targetPortLocation.y);
                            points.add(point);
                        }
                    }
                }
                else {
                    if (targetPortLocation.y > sourcePortLocation.y) {
                        if (sourcePortLocation.y > (targetBounds.top() - 15)) {
                            if ((sourceBounds.left() - targetBounds.right()) < 30) {
                                x = targetBounds.right() + (sourceBounds.left() - targetBounds.right()) / 2;
                            }
                            else {
                                x = sourceBounds.left() - 15;
                            }
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = targetBounds.top() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            x = targetBounds.left() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(x, targetPortLocation.y);
                            points.add(point);
                        }
                        else {
                            x = targetBounds.left() - 15;
                            y = sourcePortLocation.y;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(x, targetPortLocation.y);
                            points.add(point);
                        }
                    }
                    else {
                        if ((targetBounds.bottom() + 15) >= sourcePortLocation.y) {
                            if ((sourceBounds.left() - targetBounds.right()) < 30) {
                                x = targetBounds.right() + (sourceBounds.left() - targetBounds.right()) / 2;
                            }
                            else {
                                x = sourceBounds.left() - 15;
                            }
                            point = new ej.Diagram.Point(x, sourcePortLocation.y);
                            points.add(point);
                            y = targetBounds.bottom() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            x = targetBounds.left() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(x, targetPortLocation.y);
                            points.add(point);
                        }
                        else {
                            x = targetBounds.left() - 15;
                            y = sourcePortLocation.y;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(x, targetPortLocation.y);
                            points.add(point);
                        }
                    }
                }
                points.add(targetPortLocation);
                this._setPoints(points);
            };
            Orthogonal.prototype._leftToTop = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, reverse) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (sourcePortLocation.x <= targetPortLocation.x) {
                    if (sourcePortLocation.y > targetPortLocation.y && targetBounds.left() <= sourceBounds.left()) {
                        x = targetBounds.left() - 15;
                    }
                    else {
                        x = sourceBounds.left() - 15;
                    }
                    point = new ej.Diagram.Point(x, sourcePortLocation.y);
                    points.add(point);
                    if (sourceBounds.bottom() <= targetBounds.top()) {
                        if ((sourceBounds.bottom() + 15) < targetBounds.top()) {
                            if (targetBounds.height != 0) {
                                y = targetBounds.top() - 15;
                            }
                            else {
                                y = targetPortLocation.y;
                            }
                        }
                        else if (targetPortLocation.x < sourceBounds.right()) {
                            y = sourceBounds.bottom() + (targetBounds.top() - sourceBounds.bottom()) / 2;
                        }
                        else {
                            y = sourceBounds.top() - 15;
                        }
                    }
                    else {
                        if (sourceBounds.top() >= targetBounds.top()) {
                            if (targetBounds.height != 0) {
                                y = targetBounds.top() - 15;
                            }
                            else {
                                y = targetPortLocation.y;
                            }
                        }
                        else {
                            y = sourceBounds.top() - 15;
                        }
                    }
                    point = new ej.Diagram.Point(x, y);
                    points.add(point);
                    point = new ej.Diagram.Point(targetPortLocation.x, y);
                    points.add(point);
                }
                else {
                    if (targetPortLocation.y <= (sourcePortLocation.y + 15) && targetBounds.height != 0) {
                        if ((sourceBounds.left() - targetBounds.right()) > 30) {
                            x = sourceBounds.left() - 15;
                        }
                        else {
                            if (targetBounds.right() >= sourceBounds.left()) {
                                x = targetBounds.left() - 15;
                            }
                            else {
                                x = targetBounds.right() + (sourceBounds.left() - targetBounds.right()) / 2;
                            }
                        }
                        point = new ej.Diagram.Point(x, sourcePortLocation.y);
                        points.add(point);
                        y = targetBounds.top() - 15;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        point = new ej.Diagram.Point(targetPortLocation.x, y);
                        points.add(point);
                    }
                    else {
                        x = targetPortLocation.x;
                        point = new ej.Diagram.Point(x, sourcePortLocation.y);
                        points.add(point);
                    }
                }
                points.add(targetPortLocation);
                if (reverse) {
                    points.reverse();
                }
                this._setPoints(points);
            };
            Orthogonal.prototype._leftToBottom = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, reverse) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (sourcePortLocation.x <= targetPortLocation.x) {
                    if (targetBounds.left() <= sourcePortLocation.x && sourcePortLocation.y <= targetBounds.top()) {
                        x = targetBounds.left() - 15;
                    }
                    else {
                        x = sourceBounds.left() - 15;
                    }
                    y = sourcePortLocation.y;
                    point = new ej.Diagram.Point(x, y);
                    points.add(point);
                    if (targetBounds.bottom() < sourceBounds.top()) {
                        if ((sourceBounds.top() - targetBounds.bottom()) < 30) {
                            y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                        }
                        else {
                            y = targetBounds.bottom() + 15;
                        }
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        point = new ej.Diagram.Point(targetPortLocation.x, y);
                        points.add(point);
                    }
                    else {
                        if (targetBounds.bottom() < sourceBounds.bottom()) {
                            y = sourceBounds.bottom() + 15;
                        }
                        else {
                            y = targetBounds.bottom() + 15;
                        }
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        point = new ej.Diagram.Point(targetPortLocation.x, y);
                        points.add(point);
                    }
                }
                else {
                    if (sourcePortLocation.y > targetPortLocation.y) {
                        point = new ej.Diagram.Point(targetPortLocation.x, sourcePortLocation.y);
                        points.add(point);
                    }
                    else {
                        if (targetBounds.right() >= sourceBounds.left()) {
                            x = targetBounds.left() - 15;
                        }
                        else if ((sourceBounds.left() - targetBounds.right()) < 30) {
                            x = sourceBounds.left() - (sourceBounds.left() - targetBounds.right()) / 2;
                        }
                        else {
                            x = sourceBounds.left() - 15;
                        }
                        point = new ej.Diagram.Point(x, sourcePortLocation.y);
                        points.add(point);
                        y = targetBounds.bottom() + 15;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        point = new ej.Diagram.Point(targetPortLocation.x, y);
                        points.add(point);
                    }
                }
                points.add(targetPortLocation);
                if (reverse) {
                    points.reverse();
                }
                this._setPoints(points);
            };
            Orthogonal.prototype._topToTop = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (targetPortLocation.x >= sourcePortLocation.x) {
                    x = sourcePortLocation.x;
                    if (targetPortLocation.y >= sourcePortLocation.y) {
                        y = sourceBounds.top() - 15;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        if (targetPortLocation.x <= sourceBounds.right()) {
                            x = sourceBounds.right() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            if (targetBounds.height != 0) {
                                if ((targetBounds.top() - sourceBounds.bottom()) > 30) {
                                    y = targetBounds.top() - 15;
                                }
                                else {
                                    y = sourceBounds.bottom() + (targetBounds.top() - sourceBounds.bottom()) / 2;
                                }
                            }
                            else {
                                y = targetPortLocation.y;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                    else {
                        if (targetBounds.left() <= sourcePortLocation.x) {
                            if ((sourceBounds.top() - targetBounds.bottom()) > 30) {
                                y = sourceBounds.top() - 15;
                            }
                            else {
                                y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            x = targetBounds.left() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            y = targetBounds.top() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            if (targetBounds.height != 0) {
                                y = targetBounds.top() - 15;
                            }
                            else {
                                y = targetPortLocation.y;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                }
                else {
                    x = sourcePortLocation.x;
                    if (targetPortLocation.y >= sourcePortLocation.y) {
                        y = sourceBounds.top() - 15;
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        if (targetPortLocation.x >= sourceBounds.left()) {
                            x = sourceBounds.left() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            if (targetBounds.height != 0) {
                                if ((targetBounds.top() - sourceBounds.bottom()) > 30) {
                                    y = targetBounds.top() - 15;
                                }
                                else {
                                    y = sourceBounds.bottom() + (targetBounds.top() - sourceBounds.bottom()) / 2;
                                }
                            }
                            else {
                                y = targetPortLocation.y;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                    else {
                        if (targetBounds.right() >= sourcePortLocation.x) {
                            if ((sourceBounds.top() - targetBounds.bottom()) > 30) {
                                y = sourceBounds.top() - 15;
                            }
                            else {
                                y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            x = targetBounds.right() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            y = targetBounds.top() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            if (targetBounds.height != 0) {
                                y = targetBounds.top() - 15;
                            }
                            else {
                                y = targetPortLocation.y;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                }
                points.add(targetPortLocation);
                this._setPoints(points);
            };
            Orthogonal.prototype._topToBottom = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, reverse) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (targetPortLocation.x >= sourcePortLocation.x) {
                    if (sourceBounds.top() >= targetBounds.bottom()) {
                        if (sourceBounds.height != 0) {
                            if ((sourceBounds.top() - targetBounds.bottom()) >= 30) {
                                y = targetBounds.bottom() + 15;
                                point = new ej.Diagram.Point(sourcePortLocation.x, y);
                                points.add(point);
                                point = new ej.Diagram.Point(targetPortLocation.x, y);
                                points.add(point);
                            }
                            else {
                                y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                                point = new ej.Diagram.Point(sourcePortLocation.x, y);
                                points.add(point);
                                if ((targetBounds.left() - sourceBounds.right()) >= 30) {
                                    x = sourceBounds.right() + 15;
                                }
                                else {
                                    x = sourceBounds.right() + (targetBounds.left() - sourceBounds.right()) / 2;
                                }
                                point = new ej.Diagram.Point(x, y);
                                points.add(point);
                                y = targetBounds.bottom() + 15;
                                point = new ej.Diagram.Point(x, y);
                                points.add(point);
                                point = new ej.Diagram.Point(targetPortLocation.x, y);
                                points.add(point);
                            }
                        }
                        else {
                            point = new ej.Diagram.Point(targetPortLocation.x, sourcePortLocation.y);
                            points.add(point);
                        }
                    }
                    else {
                        if (sourceBounds.height != 0) {
                            if (targetBounds.top() < sourceBounds.top()) {
                                if (targetBounds.left() >= (sourceBounds.right() + 15)) {
                                    y = sourceBounds.top() - 15;
                                }
                                else {
                                    y = targetBounds.top() - 15;
                                }
                            }
                            else {
                                y = sourceBounds.top() - 15;
                            }
                        }
                        else {
                            y = targetBounds.bottom() + 15;
                        }
                        point = new ej.Diagram.Point(sourcePortLocation.x, y);
                        points.add(point);
                        if (sourceBounds.height != 0) {
                            if (targetBounds.left() >= (sourceBounds.right() + 15)) {
                                x = sourceBounds.right() + 15;
                            }
                            else {
                                x = targetBounds.right() + 15;
                            }
                        }
                        else {
                            x = targetPortLocation.x;
                        }
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        if (sourceBounds.height != 0) {
                            y = targetBounds.bottom() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                }
                else {
                    if (sourceBounds.top() >= targetBounds.bottom()) {
                        if (sourceBounds.height != 0) {
                            if ((sourceBounds.top() - targetBounds.bottom()) > 30) {
                                y = targetBounds.bottom() + 15;
                                point = new ej.Diagram.Point(sourcePortLocation.x, y);
                                points.add(point);
                                point = new ej.Diagram.Point(targetPortLocation.x, y);
                                points.add(point);
                            }
                            else {
                                y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                                point = new ej.Diagram.Point(sourcePortLocation.x, y);
                                points.add(point);
                                if ((sourceBounds.left() - targetBounds.right()) >= 30) {
                                    x = sourceBounds.left() - 15;
                                }
                                else {
                                    x = targetBounds.right() + (sourceBounds.left() - targetBounds.right()) / 2;
                                }
                                point = new ej.Diagram.Point(x, y);
                                points.add(point);
                                y = targetBounds.bottom() + 15;
                                point = new ej.Diagram.Point(x, y);
                                points.add(point);
                                point = new ej.Diagram.Point(targetPortLocation.x, y);
                                points.add(point);
                            }
                        }
                        else {
                            point = new ej.Diagram.Point(targetPortLocation.x, sourcePortLocation.y);
                            points.add(point);
                        }
                    }
                    else {
                        if (sourceBounds.height != 0) {
                            if (sourceBounds.left() >= (targetBounds.right() + 15)) {
                                y = sourceBounds.top() - 15;
                            }
                            else {
                                y = sourceBounds.top() - 15;
                            }
                        }
                        else {
                            y = targetBounds.bottom() + 15;
                        }
                        point = new ej.Diagram.Point(sourcePortLocation.x, y);
                        points.add(point);
                        if (sourceBounds.height != 0) {
                            if (sourceBounds.left() >= (targetBounds.right() + 15)) {
                                x = targetBounds.right() + 15;
                            }
                            else {
                                x = sourceBounds.right() + 15;
                            }
                        }
                        else {
                            x = targetPortLocation.x;
                        }
                        point = new ej.Diagram.Point(x, y);
                        points.add(point);
                        if (sourceBounds.height != 0) {
                            if (sourceBounds.left() >= (targetBounds.right() + 15)) {
                                y = targetBounds.bottom() + 15;
                            }
                            else {
                                if (targetBounds.bottom() <= sourceBounds.bottom()) {
                                    y = sourceBounds.bottom() + 15;
                                }
                                else {
                                    y = targetBounds.bottom() + 15;
                                }
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                }
                points.add(targetPortLocation);
                if (reverse) {
                    points.reverse();
                }
                this._setPoints(points);
            };
            Orthogonal.prototype._bottomToBottom = function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
                var points;
                var point;
                var x, y;
                points = new ej.Diagram.Collection();
                points.add(sourcePortLocation);
                if (targetPortLocation.x >= sourcePortLocation.x) {
                    if (sourceBounds.bottom() >= targetBounds.bottom()) {
                        y = sourceBounds.bottom() + 15;
                        point = new ej.Diagram.Point(sourcePortLocation.x, y);
                        points.add(point);
                        if ((targetPortLocation.x) < sourceBounds.right()) {
                            x = sourceBounds.right() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            if (sourceBounds.top() >= targetBounds.bottom()) {
                                if ((sourceBounds.top() - targetBounds.bottom()) >= 30) {
                                    y = targetBounds.bottom() + 15;
                                }
                                else {
                                    y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                                }
                            }
                            else {
                                y = targetBounds.bottom() + 15;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                    else {
                        if (sourcePortLocation.x > targetBounds.left()) {
                            if ((targetBounds.bottom() - sourceBounds.bottom()) >= 30) {
                                y = sourceBounds.bottom() + 15;
                            }
                            else {
                                y = sourceBounds.bottom() + (targetBounds.bottom() - sourceBounds.bottom()) / 2;
                            }
                            point = new ej.Diagram.Point(sourcePortLocation.x, y);
                            points.add(point);
                            x = targetBounds.left() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            y = targetBounds.bottom() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            y = targetBounds.bottom() + 15;
                            point = new ej.Diagram.Point(sourcePortLocation.x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                }
                else {
                    if (sourceBounds.bottom() >= targetBounds.bottom()) {
                        y = sourceBounds.bottom() + 15;
                        point = new ej.Diagram.Point(sourcePortLocation.x, y);
                        points.add(point);
                        if (targetPortLocation.x > sourceBounds.left()) {
                            x = sourceBounds.left() - 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            if (sourceBounds.top() >= targetBounds.bottom()) {
                                if ((sourceBounds.top() - targetBounds.bottom()) >= 30) {
                                    y = targetBounds.bottom() + 15;
                                }
                                else {
                                    y = targetBounds.bottom() + (sourceBounds.top() - targetBounds.bottom()) / 2;
                                }
                            }
                            else {
                                y = targetBounds.bottom() + 15;
                            }
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                    else {
                        if (sourcePortLocation.x < targetBounds.right()) {
                            if ((targetBounds.bottom() - sourceBounds.bottom()) >= 30) {
                                y = sourceBounds.bottom() + 15;
                            }
                            else {
                                y = sourceBounds.bottom() + (targetBounds.bottom() - sourceBounds.bottom()) / 2;
                            }
                            point = new ej.Diagram.Point(sourcePortLocation.x, y);
                            points.add(point);
                            x = targetBounds.right() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            y = targetBounds.bottom() + 15;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                        else {
                            y = targetBounds.bottom() + 15;
                            point = new ej.Diagram.Point(sourcePortLocation.x, y);
                            points.add(point);
                            point = new ej.Diagram.Point(targetPortLocation.x, y);
                            points.add(point);
                        }
                    }
                }
                points.add(targetPortLocation);
                this._setPoints(points);
            };
            Orthogonal.prototype._constructSegements = function (targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection) {
                this._setEndPoint(targetPortLocation, true);
                this._setEndPoint(sourcePortLocation, false);
                if (sourceDirection == "right") {
                    if (targetDirection == "left") {
                        this._rightToLeft(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                    else if (targetDirection == "right") {
                        this._rightToRight(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                    else if (targetDirection == "top") {
                        this._rightToTop(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                    else if (targetDirection == "bottom") {
                        this._rightToBottom(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                }
                if (sourceDirection == "left") {
                    if (targetDirection == "right") {
                        this._rightToLeft(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    }
                    if (targetDirection == "left") {
                        this._leftToLeft(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                    if (targetDirection == "top") {
                        this._leftToTop(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                    if (targetDirection == "bottom") {
                        this._leftToBottom(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                }
                if (sourceDirection == "top") {
                    if (targetDirection == "left") {
                        this._leftToTop(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    }
                    if (targetDirection == "right") {
                        this._rightToTop(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    }
                    if (targetDirection == "top") {
                        this._topToTop(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                    if (targetDirection == "bottom") {
                        this._topToBottom(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                }
                if (sourceDirection == "bottom") {
                    if (targetDirection == "left") {
                        this._leftToBottom(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    }
                    if (targetDirection == "right") {
                        this._rightToBottom(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    }
                    if (targetDirection == "top") {
                        this._topToBottom(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    }
                    if (targetDirection == "bottom") {
                        this._bottomToBottom(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                    }
                }
            };
            Orthogonal.prototype._swapEdges = function (rotateAngle, left, top, right, bottom) {
                var angle = rotateAngle % 360;
                if (angle < 0) {
                    angle += 360;
                }
                var temp;
                if (angle > 315 || angle < 45) {
                    //no need to swap
                }
                else if (angle < 135) {
                    temp = top;
                    top = left;
                    left = bottom;
                    bottom = right;
                    right = temp;
                }
                else if (angle < 225) {
                    temp = top;
                    top = bottom;
                    bottom = temp;
                    temp = left;
                    left = right;
                    right = temp;
                }
                else {
                    temp = bottom;
                    bottom = left;
                    left = top;
                    top = right;
                    right = temp;
                }
            };
            Orthogonal.prototype._dock = function (targetNode, sourceNode, isTarget) {
                this._resetControlPoints();
                var x, y;
                var sourceBounds;
                var dockSegment;
                var targetDirection;
                var sourceDirection;
                if (targetNode) {
                    var targetBounds = targetNode.getBoundingBox();
                    var points = targetBounds.toPoints();
                    var mat = new ej.Diagram.Matrix();
                    var matrix = mat._createIdentity();
                    matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                    for (var i = 0, len = points.length; i < len; i++) {
                        points[i] = matrix._transform(points[i]);
                    }
                    targetBounds = ej.Diagram.Geometry.rect(points);
                    if (sourceNode) {
                        sourceBounds = sourceNode.getBoundingBox();
                        points = sourceBounds.toPoints();
                        matrix = mat._createIdentity();
                        matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                        for (var i = 0, len = points.length; i < len; i++) {
                            points[i] = matrix._transform(points[i]);
                        }
                        sourceBounds = ej.Diagram.Geometry.rect(points);
                        this._setEndPoint(sourceBounds.center(), !isTarget);
                        this._setEndPoint(targetBounds.center(), isTarget);
                        dockSegment = this._getDockSegment(sourceNode);
                        if (dockSegment) {
                            this._setEndPoint(dockSegment.dockPoint, !isTarget);
                            sourceDirection = dockSegment.direction;;
                        }
                    }
                    this._setEndPoint(targetBounds.center(), isTarget);
                    dockSegment = this._getDockSegment(targetNode);
                    if (dockSegment) {
                        this._setEndPoint(dockSegment.dockPoint, isTarget);
                        targetDirection = dockSegment.direction;
                    }
                }
                var startPoint, endPoint;
                var points;
                var point;
                var temp;
                if (!isTarget && sourceDirection) {
                    temp = targetDirection;
                    targetDirection = sourceDirection;
                    sourceDirection = temp;
                }
                if (targetDirection && sourceDirection) {
                    if ((targetDirection == "left" || targetDirection == "right") && (sourceDirection == "right" || sourceDirection == "left")) {
                        this._updateControlPoints(true);
                    }
                    else if ((targetDirection == "top" || targetDirection == "bottom") && (sourceDirection == "bottom" || sourceDirection == "top")) {
                        this._updateControlPoints(false);
                    }
                    else {
                        if ((targetDirection == "top" || targetDirection == "bottom") && (sourceDirection == "right" || sourceDirection == "left")) {
                            points = new ej.Diagram.Collection();
                            points.add(this.startPoint);
                            x = this.endPoint.x;
                            y = this.startPoint.y;
                            point = new ej.Diagram.Point(x, y);
                            points.add(point);
                            points.add(this.endPoint);

                        }
                        else {
                            points = new ej.Diagram.Collection();
                            points.add(this.startPoint);
                            point = new ej.Diagram.Point(this.startPoint.x, this.endPoint.y);
                            points.add(point);
                            points.add(this.endPoint);
                        }
                        this._setPoints(points);
                    }
                }
                else if (targetDirection) {
                    points = new ej.Diagram.Collection();
                    points.add(this.startPoint);
                    if (isTarget) {
                        if (targetDirection == "left" || targetDirection == "right") {
                            x = this.startPoint.x;
                            y = this.endPoint.y;
                        }
                        else {
                            x = this.endPoint.x;
                            y = this.startPoint.y;
                        }
                    }
                    else {
                        if (targetDirection == "left" || targetDirection == "right") {
                            x = this.endPoint.x;
                            y = this.startPoint.y;
                        }
                        else {
                            x = this.startPoint.x;
                            y = this.endPoint.y;
                        }
                    }
                    point = new ej.Diagram.Point(x, y);
                    points.add(point);
                    points.add(this.endPoint);
                    this._setPoints(points);
                }
            };
            Orthogonal.prototype._dockNode = function (targetNode, sourceNode, parent) {
                if (parent.isTargetConnected() && parent.isSourceConnected() && !parent.targetPort && !parent.sourcePort) {
                    this._dock(targetNode, sourceNode, true);
                    this._dock(sourceNode, targetNode, false);
                }
                else if (parent.isTargetConnected() && !parent.targetPort) {
                    this._dock(targetNode, sourceNode, true);
                }
                else if (parent.isSourceConnected() && !parent.sourcePort) {
                    this._dock(targetNode, sourceNode, false);
                }
            };
            Orthogonal.prototype._getDockSegment = function (node) {
                var direction;
                var segmentPoints = node.getBoundingBox().toPoints();
                var segment;
                var intersecSeg;
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                matrix._rotateAt(node.rotateAngle, node.offsetX, node.offsetY);
                for (var i = 0, len = segmentPoints.length; i < len; i++) {
                    segmentPoints[i] = matrix._transform(segmentPoints[i]);
                }
                var leftSeg = new ej.Diagram.Segment(segmentPoints[3], segmentPoints[0]);
                var topSeg = new ej.Diagram.Segment(segmentPoints[0], segmentPoints[1]);
                var rightSeg = new ej.Diagram.Segment(segmentPoints[1], segmentPoints[2]);
                var bottomSeg = new ej.Diagram.Segment(segmentPoints[2], segmentPoints[3]);
                this._swapEdges(node.rotateAngle, leftSeg, topSeg, rightSeg, bottomSeg);
                intersecSeg = new ej.Diagram.Segment(this.startPoint, this.endPoint);
                if (intersecSeg.intersect(leftSeg)) {
                    segment = leftSeg;
                    direction = "left";
                }
                else if (intersecSeg.intersect(topSeg)) {
                    segment = topSeg;
                    direction = "top";
                }
                else if (intersecSeg.intersect(rightSeg)) {
                    segment = rightSeg;
                    direction = "right";
                }
                else if (intersecSeg.intersect(bottomSeg)) {
                    segment = bottomSeg;
                    direction = "bottom";
                }
                direction = this._swapDirection(node, direction);
                if (segment) {
                    return { "dockPoint": segment.midPoint(), "direction": direction };
                }
                return null;
            };
            Orthogonal.prototype._dockPortToPort = function (targetNode, targetPort, sourceNode, sourcePort) {
                var targetDirection = this._swapDirection(targetNode, this._getDirection(targetNode, targetPort, sourceNode, sourcePort));
                var sourceDirection = this._swapDirection(sourceNode, this._getDirection(sourceNode, sourcePort, targetNode, targetPort));
                var targetBounds = targetNode.getBoundingBox();
                var sourceBounds = sourceNode.getBoundingBox();
                var points = targetBounds.toPoints();
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                for (var i = 0, len = points.length; i < len; i++) {
                    points[i] = matrix._transform(points[i]);
                }
                var targetPortLocation = matrix._transform(targetPort._getPosition(targetBounds));
                targetBounds = ej.Diagram.Geometry.rect(points);
                points = sourceBounds.toPoints();
                mat = new ej.Diagram.Matrix();
                matrix = mat._createIdentity();
                matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                for (var i = 0, len = points.length; i < len; i++) {
                    points[i] = matrix._transform(points[i]);
                }
                var sourcePortLocation = matrix._transform(sourcePort._getPosition(sourceBounds));
                sourceBounds = ej.Diagram.Geometry.rect(points);
                this._constructSegements(targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);
            };
            Orthogonal.prototype._swapDirection = function (node, direction) {
                var dir;
                var angle = node.rotateAngle;
                angle = angle % 360;
                if (angle < 0) {
                    angle += 360;
                }
                var temp;
                if (angle > 315 || angle < 45) {
                    dir = direction;
                }
                else if (angle < 135) {
                    if (direction == "left") {
                        dir = "top";
                    }
                    else if (direction == "bottom") {
                        dir = "left";
                    }
                    else if (direction == "right") {
                        dir = "bottom";
                    }
                    else if (direction == "top") {
                        dir = "right";
                    }
                }
                else if (angle < 225) {
                    if (direction == "left") {
                        dir = "right";
                    }
                    else if (direction == "bottom") {
                        dir = "top";
                    }
                    else if (direction == "right") {
                        dir = "left";
                    }
                    else if (direction == "top") {
                        dir = "bottom";
                    }
                }
                else {
                    if (direction == "left") {
                        dir = "bottom";
                    }
                    else if (direction == "bottom") {
                        dir = "right";
                    }
                    else if (direction == "right") {
                        dir = "top";
                    }
                    else if (direction == "top") {
                        dir = "left";
                    }
                }
                return dir;
            };
            Orthogonal.prototype._dockPortToNode = function (sourceNode, sourcePort, isTarget, parent) {
                var sourceBounds = sourceNode.getBoundingBox();
                var points = sourceBounds.toPoints();
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                for (var i = 0, len = points.length; i < len; i++) {
                    points[i] = matrix._transform(points[i]);
                }
                var sourcePortLocation = matrix._transform(sourcePort._getPosition(sourceBounds));
                sourceBounds = ej.Diagram.Geometry.rect(points);
                var sourceDirection = this._swapDirection(sourceNode, this._getDirection(sourceNode, sourcePort, null, null));
                var targetNode;
                var targetBounds;
                var targetPortLocation;
                if (!isTarget) {
                    targetNode = parent.targetNode;
                    targetPortLocation = this.endPoint;
                }
                else {
                    targetNode = parent.sourceNode;
                    targetPortLocation = this.startPoint;
                }
                targetBounds = targetNode.getBoundingBox();
                points = targetBounds.toPoints();
                mat = new ej.Diagram.Matrix();
                matrix = mat._createIdentity();
                matrix._rotateAt(targetNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                for (var i = 0, len = points.length; i < len; i++) {
                    points[i] = matrix._transform(points[i]);
                }
                var targetDirection = this._swapDirection(targetNode, this._getEndPointDirection(targetNode, !isTarget));
                targetBounds = ej.Diagram.Geometry.rect(points);
                if (targetDirection) {
                    if (!isTarget) {
                        this._constructSegements(targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);
                    }
                    else {
                        this._constructSegements(sourceBounds, sourcePortLocation, sourceDirection, targetBounds, targetPortLocation, targetDirection);
                    }
                }
            };
            Orthogonal.prototype._dockPortToEndPoint = function (targetNode, targetPort, isTarget) {
                var targetDirection;
                var sourceDirection;
                var targetBounds;
                var sourceBounds;
                var sourcePortLocation;
                var targetPortLocation;
                var points;
                var mat;
                var matrix;
                if (isTarget) {
                    targetBounds = targetNode.getBoundingBox();
                    points = targetBounds.toPoints();
                    mat = new ej.Diagram.Matrix();
                    matrix = mat._createIdentity();
                    matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                    for (var i = 0, len = points.length; i < len; i++) {
                        points[i] = matrix._transform(points[i]);
                    }
                    targetPortLocation = matrix._transform(targetPort._getPosition(targetBounds));
                    targetBounds = ej.Diagram.Geometry.rect(points);
                    targetDirection = this._swapDirection(targetNode, this._getDirection(targetNode, targetPort, null, null));
                    sourcePortLocation = this.startPoint;
                    sourceBounds = new ej.Diagram.Rectangle(sourcePortLocation.x, sourcePortLocation.y, 0, 0);
                    sourceDirection = "top";
                }
                else {
                    sourceBounds = targetNode.getBoundingBox();
                    points = sourceBounds.toPoints();
                    mat = new ej.Diagram.Matrix();
                    matrix = mat._createIdentity();
                    matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                    for (var i = 0, len = points.length; i < len; i++) {
                        points[i] = matrix._transform(points[i]);
                    }
                    sourcePortLocation = matrix._transform(targetPort._getPosition(sourceBounds));
                    sourceBounds = ej.Diagram.Geometry.rect(points);
                    sourceDirection = this._swapDirection(targetNode, this._getDirection(targetNode, targetPort, null, null));
                    targetPortLocation = this.endPoint;
                    targetBounds = new ej.Diagram.Rectangle(targetPortLocation.x, targetPortLocation.y, 0, 0);
                    targetDirection = "top";
                }
                this._constructSegements(targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);
            }
            Orthogonal.prototype._getEndPointDirection = function (targetNode, isTarget) {
                var point;
                if (isTarget) {
                    point = this.endPoint;
                }
                else {
                    point = this.startPoint;
                }
                var bounds = targetNode.getBoundingBox();
                var points = bounds.toPoints();
                var center = bounds.center();
                var direction;
                var angle = ej.Diagram.Geometry.findAngle(point, center);
                if (angle > ej.Diagram.Geometry.findAngle(points[0], center) && angle <= ej.Diagram.Geometry.findAngle(points[1], center)) {
                    direction = "top";
                }
                else if (angle >= ej.Diagram.Geometry.findAngle(points[1], center) && angle <= ej.Diagram.Geometry.findAngle(points[2], center)) {
                    direction = "right";
                }
                else if (angle >= ej.Diagram.Geometry.findAngle(points[3], center) || angle < ej.Diagram.Geometry.findAngle(points[0], center)) {
                    direction = "left";
                }
                else if (angle >= ej.Diagram.Geometry.findAngle(points[2], center) && angle <= ej.Diagram.Geometry.findAngle(points[3], center)) {
                    direction = "bottom";
                }
                else {
                    direction = "right";
                }
                return direction;
            };
            Orthogonal.prototype._updateEndPoint = function (dx, dy, isTarget, parent) {
                if (parent) {
                    if (!parent.isTargetConnected() || !parent.isSourceConnected()) {
                        this._resetControlPoints();
                    }
                }
                var point;
                if (isTarget) {
                    point = ej.Diagram.Geometry.translate(this.points[this.points.length - 1], dx, dy);
                    this.endPoint = this.points[this.points.length - 1] = point;
                }
                else {
                    point = ej.Diagram.Geometry.translate(this.points[0], dx, dy);
                    this.startPoint = this.points[0] = point;
                }
                this.points[1] = new ej.Diagram.Point(this.startPoint.x, this.startPoint.y + (this.endPoint.y - this.startPoint.y) / 2);
                this.points[this.points.length - 2] = new ej.Diagram.Point(this.endPoint.x, this.startPoint.y + (this.endPoint.y - this.startPoint.y) / 2);
            };
            Orthogonal.prototype._setEndPoint = function (point, isTarget, dock, parent) {
                if (parent) {
                    if (!parent.isTargetConnected() || !parent.isSourceConnected()) {
                        this._resetControlPoints();
                    }
                }
                if (isTarget) {
                    this.endPoint = this.points[this.points.length - 1] = point;
                }
                else {
                    this.startPoint = this.points[0] = point;
                }
                this._updateControlPoints(dock);
            };
            Orthogonal.prototype._setPoints = function (points) {
                this.startPoint = this.points[0] = points[0];
                this.endPoint = this.points[this.points.length - 1] = points[points.length - 1];
                this.points = points;
                //this._updateControlPoints();
            };
            return Orthogonal;
        })();
        Lines.Orthogonal = Orthogonal;

        var Bezier = (function () {
            function Bezier(props) {
                this.type = "bezier";
                this.startPoint = new ej.Diagram.Point(0, 0);
                this.endPoint = this.point3 = new ej.Diagram.Point(0, 0);
                this.points = new ej.Diagram.Collection();
                this.points.add(this.startPoint);
                this.points.add(this.point3);
                this.point1 = null;
                this.point2 = null;
                this._point1 = new ej.Diagram.Point(0, 0);
                this._point2 = new ej.Diagram.Point(0, 0);
                this._updateControlPoints();
                this.parent = null;
                ej.Diagram.merge(this, props);
            }
            Bezier.prototype.merge = function (prop, value) {
                if (prop.toString() === "startPoint") {
                    value = new ej.Diagram.Point(value.x, value.y);
                    this._setEndPoint(value, false);
                }
                else if (prop.toString() === "endPoint") {
                    value = new ej.Diagram.Point(value.x, value.y);
                    this._setEndPoint(value, true);
                }
                else if (prop.toString() === "point1") {
                    this._setBezierPoint(value, "bezierpoint1");
                }
                else if (prop.toString() === "point2") {
                    this._setBezierPoint(value, "bezierpoint2");
                }
            };
            Bezier.prototype._setEndPoint = function (point, isTarget) {
                if (isTarget) {
                    this.endPoint = this.points[1] = this.point3 = point;
                }
                else {
                    this.startPoint = this.points[0] = point;
                }
                this._updateControlPoints();
            };
            Bezier.prototype._dockNode = function (targetNode, sourceNode, parent) {
                if (parent.isTargetConnected() && parent.isSourceConnected() && !parent.targetPort && !parent.sourcePort) {
                    this._dock(targetNode, sourceNode, true);
                    this._dock(sourceNode, targetNode, false);
                }
                else if (parent.isTargetConnected() && !parent.targetPort) {
                    this._dock(targetNode, sourceNode, true);
                }
                else if (parent.isSourceConnected() && !parent.sourcePort) {
                    this._dock(targetNode, sourceNode, false);
                }
            };
            Bezier.prototype._dock = function (targetNode, sourceNode, isTarget) {
                var mat = new ej.Diagram.Matrix();
                var matrix;
                var targetBounds;
                var sourceBounds;
                var points;
                var i, len;
                if (targetNode) {
                    targetBounds = targetNode.getBoundingBox();
                    points = targetBounds.toPoints();
                    matrix = mat._createIdentity();
                    matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                    for (i = 0, len = points.length; i < len; i++) {
                        points[i] = matrix._transform(points[i]);
                    }
                    targetBounds = ej.Diagram.Geometry.rect(points);
                    if (sourceNode) {
                        sourceBounds = sourceNode.getBoundingBox();
                        points = sourceBounds.toPoints();
                        matrix = mat._createIdentity();
                        matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                        for (i = 0, len = points.length; i < len; i++) {
                            points[i] = matrix._transform(points[i]);
                        }
                        sourceBounds = ej.Diagram.Geometry.rect(points);
                        this._setEndPoint(sourceBounds.center(), !isTarget);
                        this._setEndPoint(targetBounds.center(), isTarget);
                        this._dockBounds(sourceNode, !isTarget, parent);
                    }
                    this._setEndPoint(targetBounds.center(), isTarget);
                    this._dockBounds(targetNode, isTarget, parent);
                }
            };
            Bezier.prototype._dockBounds = function (node, isTarget, parent) {
                var segmentPoints = node.getBoundingBox().toPoints();
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                var i, len;
                matrix._rotateAt(node.rotateAngle, node.offsetX, node.offsetY);
                for (i = 0, len = segmentPoints.length; i < len; i++) {
                    segmentPoints[i] = matrix._transform(segmentPoints[i]);
                }
                if (isTarget) {
                    if (this.point2)
                        var thisSegment = new ej.Diagram.Segment(this._point2, this.endPoint);
                    else
                        var thisSegment = new ej.Diagram.Segment(this.startPoint, this.endPoint);
                } else {
                    if (this.point1) {
                        var thisSegment = new ej.Diagram.Segment(this.startPoint, this._point1);
                    }
                    else
                        var thisSegment = new ej.Diagram.Segment(this.startPoint, this.endPoint);
                }
                var len = segmentPoints.length;
                var segment = new ej.Diagram.Segment(segmentPoints[len - 1], segmentPoints[0]);
                var intersection = thisSegment.intersect(segment);
                if (!intersection) {
                    for (var i = 0; i < len - 1; i++) {
                        segment = new ej.Diagram.Segment(segmentPoints[i], segmentPoints[i + 1]);
                        intersection = thisSegment.intersect(segment);
                        if (intersection)
                            break;
                    }
                }
                if (intersection) {
                    this._setEndPoint(intersection, isTarget);
                    this._updateControlPoints();
                }
            };
            Bezier.prototype._dockPortToPort = function (targetNode, targetPort, sourceNode, sourcePort) {
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                var portLocation;
                if (targetPort) {
                    matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                    portLocation = matrix._transform(targetPort._getPosition(targetNode.getBoundingBox()));
                    this._setEndPoint(portLocation, true);
                }
                if (sourcePort) {
                    matrix = mat._createIdentity();
                    matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                    portLocation = matrix._transform(sourcePort._getPosition(sourceNode.getBoundingBox()));
                    this._setEndPoint(portLocation, false);
                }
                this._updateControlPoints(targetPort, sourcePort, targetNode, sourceNode);
            };
            Bezier.prototype._dockPortToNode = function (sourceNode, sourcePort, isTarget, parent) {
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                matrix._rotateAt(sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                var portLocation = matrix._transform(sourcePort._getPosition(sourceNode.getBoundingBox()));
                this._setEndPoint(portLocation, isTarget);
            };
            Bezier.prototype._dockPortToEndPoint = function (targetNode, targetPort, isTarget) {
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                matrix._rotateAt(targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                var portLocation = matrix._transform(targetPort._getPosition(targetNode.getBoundingBox()));
                this._setEndPoint(portLocation, isTarget);
            };
            Bezier.prototype._translate = function (dx, dy, parent) {
                if (this.points) {
                    this.points[0] = this.startPoint = ej.Diagram.Geometry.translate(this.startPoint, dx, dy);
                    this.points[1] = this.endPoint = this.point3 = ej.Diagram.Geometry.translate(this.endPoint, dx, dy);
                    this._point1.x += dx;
                    this._point1.y += dy;
                    this._point2.x += dx;
                    this._point2.y += dy;
                    if (this.point1) { this.point1 = this._point1; }
                    if (this.point2) { this.point2 = this._point2; }
                }
            };
            Bezier.prototype._setBezierPoint = function (point, whichpoint) {
                if (whichpoint == "bezierpoint1") {
                    this.point1 = this._point1 = point;
                }
                else {
                    this.point2 = this._point2 = point;
                }
            };
            Bezier.prototype._updateControlPoints = function (targetport, sourceport, targetnode, sourcenode) {
                if (this.point1 == null || this.point2 == null) {
                    var target = this.endPoint;
                    var source = this.startPoint;
                    var dir;
                    if (!sourceport)
                        dir = this._getBezierDirection(source, target);
                    else {
                        var source = sourceport._getPosition(sourcenode.getBoundingBox());
                        if (sourcenode.rotateAngle != 0) {
                            var mat = new ej.Diagram.Matrix();
                            var matrix = mat._createIdentity();
                            matrix._rotateAt(sourcenode.rotateAngle, sourcenode.offsetX, sourcenode.offsetY);
                            source = matrix._transform(source);
                        }
                        dir = this._getDirection(sourcenode, source);
                    }
                    this._point1 = this.point1 || this._getBezierAdjPoint(this.getAngle(dir), this.startPoint, this.endPoint);

                    if (!targetport) { var dir = this._getBezierDirection(target, source); }
                    else
                    {
                        var target = targetport._getPosition(targetnode.getBoundingBox());
                        if (targetnode.rotateAngle != 0) {
                            var mat = new ej.Diagram.Matrix();
                            var matrix = mat._createIdentity();
                            matrix._rotateAt(targetnode.rotateAngle, targetnode.offsetX, targetnode.offsetY);
                            target = matrix._transform(target);
                        }
                        dir = this._getDirection(targetnode, target);

                    }


                    this._point2 = this.point2 || this._getBezierAdjPoint(this.getAngle(dir), this.endPoint, this.startPoint);
                }
            };
            Bezier.prototype.getAngle = function (dir) {
                switch (dir) {
                    case "right":
                        return 0;
                    case "left":
                        return 180;
                    case "top":
                        return 270;
                    case "bottom":
                        return 90;
                }
            };
            Bezier.prototype._getBezierDirection = function (src, tar) {
                if (Math.abs(tar.x - src.x) > Math.abs(tar.y - src.y)) {
                    return src.x < tar.x ? "right" : "left";
                }
                else {
                    return src.y < tar.y ? "bottom" : "top";
                }
            };
            Bezier.prototype._getDirection = function (node, point) {
                var one, two, three, four, center;
                if (!node._corners)
                    node.updatebounds();
                var corners = node._corners;
                one = corners._topLeft;
                two = corners._topRight;
                three = corners._bottomRight;
                four = corners._bottomLeft;
                center = new ej.Diagram.Point(node.offsetX - node.width * node.pivot.x + node.width / 2, node.offsetY - node.height * node.pivot.y + node.height / 2);

                var angle = ej.Diagram.Geometry.findAngle(point, center);


                //Top
                if (angle > ej.Diagram.Geometry.findAngle(one, center) && angle <= ej.Diagram.Geometry.findAngle(two, center)) {
                    return "top";
                }
                    //Right
                else if (angle >= ej.Diagram.Geometry.findAngle(two, center) && angle <= ej.Diagram.Geometry.findAngle(three, center)) {
                    return "right";
                }
                else if (angle >= ej.Diagram.Geometry.findAngle(four, center) || angle < ej.Diagram.Geometry.findAngle(one, center)) {
                    return "left";
                }
                else if (angle >= ej.Diagram.Geometry.findAngle(three, center) && angle <= ej.Diagram.Geometry.findAngle(four, center)) {
                    return "bottom";
                }
                else {  //Somthing wrong.
                    return "right";
                }
            };
            Bezier.prototype._getBezierAdjPoint = function (angle, srcEnd, tarEnd) {
                var distance = 60;
                var endAdj = new ej.Diagram.Point(0, 0);
                var dir;
                if (angle > 45 && angle < 135) {
                    dir = "Bottom";
                }
                else if (angle > 135 && angle < 225) {
                    dir = "Left";
                }
                else if (angle > 225 && angle < 315) {
                    dir = "Top";
                }
                else {
                    dir = "Right";
                }

                //distance = Math.Min(srcEnd.FindLength(tarEnd) * 0.3, distance);
                //endAdj = srcEnd._transform(distance, dir);
                switch (dir) {
                    case "Right":
                        distance = Math.min(Math.abs(srcEnd.x - tarEnd.x) * 0.45, distance);
                        endAdj = new ej.Diagram.Point(srcEnd.x + distance, srcEnd.y);
                        break;
                    case "Bottom":
                        distance = Math.min(Math.abs(srcEnd.y - tarEnd.y) * 0.45, distance);
                        endAdj = new ej.Diagram.Point(srcEnd.x, srcEnd.y + distance);
                        break;
                    case "Left":
                        distance = Math.min(Math.abs(srcEnd.x - tarEnd.x) * 0.45, distance);
                        endAdj = new ej.Diagram.Point(srcEnd.x - distance, srcEnd.y);
                        break;
                    case "Top":
                        distance = Math.min(Math.abs(srcEnd.y - tarEnd.y) * 0.45, distance);
                        endAdj = new ej.Diagram.Point(srcEnd.x, srcEnd.y - distance);
                        break;
                }
                return endAdj;
            };
            Bezier.prototype._setPoints = function (points) {
                this.startPoint = this.points[0] = points[0];
                this.endPoint = this.point3 = this.points[1] = points[1];
                this.point1 = points[2] || this.point1;
                this.point2 = points[3] || this.point2;
                if (!this.point1)
                    this._point1 = this.point1;
                if (!this.point2)
                    this._point2 = this.point2;
                this._updateControlPoints();
            };
            return Bezier;
        })();
        Lines.Bezier = Bezier;
    })(Lines || (Lines = {}));

    ej.Diagram.Lines = {};
    ej.Diagram.Lines.Straight = function (props) {
        return new Lines.Straight(props);
    };
    ej.Diagram.Lines.Orthogonal = function (props) {
        return new Lines.Orthogonal(props);
    };
    ej.Diagram.Lines.Bezier = function (props) {
        return new Lines.Bezier(props);
    };

    var Margin = (function () {
        function Margin(props) {
            this.top = 5;
            this.left = 5;
            this.right = 5;
            this.bottom = 5;
            ej.Diagram.merge(this, props);
        }
        return Margin;
    })();

    ej.Diagram.Margin = function (props) {
        return new Margin(props);
    };
    //#endregion

    //#region Enums

    //#region DecoratorShapes
    ej.Diagram.DecoratorShapes = {
        Arrow: "arrow",
        OpenArrow: "openarrow",
        Circle: "circle",
        Diamond: "diamond",
        Path: "path"
    };
    //#endregion

    //#region PortShapes
    ej.Diagram.PortShapes = {
        X: "x",
        Circle: "circle",
        Square: "square",
        Path: "path"
    };
    //#endregion

    //#region PortVisibility
    ej.Diagram.PortVisibility = {
        Visible: 1 << 0,
        Hidden: 1 << 1,
        Hover: 1 << 2,
        Connection: 1 << 3,
        Default: 1 << 2 | 1 << 3
    };
    //#endregion

    //#region TextAlign
    ej.Diagram.TextAlign = {
        Left: "left",
        Center: "center",
        Right: "right"
    };
    //#endregion

    //#region HorizontalAignment
    ej.Diagram.HorizontalAlignment = {
        Left: "left",
        Center: "center",
        Right: "right"
    };

    ej.Diagram.LabelEditMode = {
        Edit: "edit",
        View: "view"
    };
    //#endregion

    //#region VerticalAlignment
    ej.Diagram.VerticalAlignment = {
        Top: "top",
        Center: "center",
        Bottom: "bottom"
    };
    //#endregion    

    //#region TextDecorations
    ej.Diagram.TextDecorations = {
        Underline: "underline",
        Overline: "overline",
        LineThrough: "line-through",
        None: "none"
    };
    //#endregion

    //#region Constraints
    ej.Diagram.NodeConstraints = {
        None: 1 << 0,
        Select: 1 << 1,
        Delete: 1 << 2,
        Resize: 1 << 3,
        Drag: 1 << 4,
        Rotate: 1 << 5,
        Connect: 1 << 6,
        Default: 1 << 1 | 1 << 2 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6
    };
    ej.Diagram.ConnectorConstraints = {
        None: 1 << 0,
        Select: 1 << 1,
        Delete: 1 << 2,
        Drag: 1 << 3,
        DragSourceEnd: 1 << 4,
        DragTargetEnd: 1 << 5,
        DragSegmentThumb: 1 << 6,
        Default: 1 << 1 | 1 << 2 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6
    };
    ej.Diagram.PortConstraints = {
        None: 1 << 0,
        Connect: 1 << 1 | 1 << 2
    };
    ej.Diagram.SelectorConstraints = {
        None: 1 << 0,
        Rotator: 1 << 1,
        Resizer: 1 << 2,
        UserHandles: 1 << 3,
        All: 1 << 1 | 1 << 2 | 1 << 3
    };
    //#endregion

    //#region Handle
    ej.Diagram.Position = {
        TopLeft: "topLeft",
        TopCenter: "topCenter",
        TopRight: "topRight",
        MiddleLeft: "middleLeft",
        MiddleRight: "middleRight",
        BottomLeft: "bottomLeft",
        BottomCenter: "bottomCenter",
        BottomRight: "bottomRight",
    };

    //#endregion

    //#endregion

    //#region Guides
    ej.Diagram.VisualGuides = {
        Tooltip: 1,
        Alignment: 2,
        None: 4
    };
    //#endregion

    //#region SnapSettings
    ej.Diagram.SnapConstraints = {
        None: 0,
        SnapToHorizontalLines: 1,
        SnapToVerticalLines: 2,
        SnapToLines: 1 | 2,
        ShowHorizontalLines: 4,
        ShowVerticalLines: 8,
        ShowLines: 4 | 8,
        All: 1 | 2 | 4 | 8
    };

    var GridLines = (function () {
        function GridLines(props) {
            this.linesInterval = [1.25, 18.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75];
            this.snapInterval = this._snapinterval = [20];
            this.lineDashArray = "0";
            this.lineColor = "lightgray";
            ej.Diagram.merge(this, props);
            this._snapinterval = this.snapInterval;
        }
        GridLines.prototype.merge = function (prop, value) {
            if (prop.toString() === "linesInterval") {
                this.linesInterval = value;
            }
            if (prop.toString() === "snapInterval") {
                this.snapInterval = value;
            }
        }
        return GridLines;
    })();
    ej.Diagram.GridLines = function (props) {
        return new GridLines(props);
    };

    var SnapSettings = (function () {
        function SnapSettings(props) {
            this.horizontalGridlines = new ej.Diagram.GridLines();
            this.verticalGridlines = new ej.Diagram.GridLines();
            this.snapConstraints = ej.Diagram.SnapConstraints.All;
            this.snapToObject = true;
            this.snapAngle = 5;
            ej.Diagram.merge(this, props);
        }
        SnapSettings.prototype.merge = function (prop, value) {
            if (prop.toString() === "horizontalGridlines") {
                this.horizontalGridlines = new ej.Diagram.GridLines(value);
            }
            if (prop.toString() === "verticalGridlines") {
                this.verticalGridlines = new ej.Diagram.GridLines(value);
            }
        }
        return SnapSettings;
    })();
    ej.Diagram.SnapSettings = function (props) {
        return new SnapSettings(props);
    };
    //#endregion

    //#region Path
    var Path = (function () {
        function Path() {
            this._path = "";
        }
        Path.prototype.moveTo = function (x, y) {
            this._path += "M" + x + "," + y;
        };
        Path.prototype.lineTo = function (x, y) {
            this._path += "L" + x + "," + y;
        };
        Path.prototype.cubicBezierTo = function (points, isSmooth) {

        };
        Path.prototype.quadraticBezierTo = function (points, isSmooth) {

        };
        Path.prototype.bezierCurveTo = function (points, isSmooth) {

        };
        Path.prototype.ellipticalArcTo = function (rx, ry, xAngle, arcFlag, sweepFlag, x, y) {

        };
        Path.prototype.close = function () {
            this._path += "Z";
        };
        Path.prototype.toString = function () {
            return this._path;
        };
        return Path;
    })();
    ej.Diagram.Path = function () {
        return new Path();
    };
    //#endregion

    //#region Decorator
    var Decorator = (function () {
        function Decorator(props) {
            this.shape = ej.Diagram.DecoratorShapes.Arrow;
            this.width = 8;
            this.height = 8;
            this.borderColor = "black";
            this.fillColor = "black";
            this.pathData = "";
            ej.Diagram.merge(this, props);
        }
        return Decorator;
    })();
    ej.Diagram.Decorator = function (props) {
        return new Decorator(props);
    };
    //#endregion

    //#region Label
    var TextBlock = (function () {
        function TextBlock(props) {
            this.readOnly = false;
            this.bold = false;
            this.italic = false;
            this.text = "";
            this.textDecoration = ej.Diagram.TextDecorations.None;
            this.fontSize = 12;
            this.fontFamily = "Arial";
            this.fontColor = "black";
            this.offset = new ej.Diagram.Point(0.5, 0.5);
            this.textAlign = ej.Diagram.TextAlign.Center;
            this.horizontalAlignment = ej.Diagram.HorizontalAlignment.Center;
            this.verticalAlignment = ej.Diagram.VerticalAlignment.Center;
            this.wrapText = true;
            this.margin = new ej.Diagram.Margin();
            this.mode = ej.Diagram.LabelEditMode.Edit;
            this.width = 50;
            ej.Diagram.merge(this, props);
        }
        TextBlock.prototype.merge = function (prop, value) {
            if (prop.toString() === "margin") {
                this.margin = new ej.Diagram.Margin(value);
            }
            else if (prop.toString() === "offset") {
                this.offset = new ej.Diagram.Point(value.x, value.y);
            }
        };
        TextBlock.prototype._getPosition = function (bounds) {
            var x = (bounds.width * this.offset.x);
            var y = (bounds.height * this.offset.y);
            return new ej.Diagram.Point(x, y);
        };
        return TextBlock;
    })();
    ej.Diagram.TextBlock = function (props) {
        return new TextBlock(props);
    };
    var Label = (function (base) {
        ej.Diagram.extend(Label, base);
        function Label(props) {
            this.name = "";
            this.visible = true;
            this.borderColor = "transparent";
            this.borderWidth = 0;
            this.fillColor = "transparent";
            ej.Diagram.merge(this, props);
            base.call(this, props);
        }
        Label.prototype.clone = function (src) {
            var label = src ? src : new ej.Diagram.Label();
            label.name = this.name;
            label.visible = this.visible;
            label.borderColor = this.borderColor;
            label.borderWidth = this.borderWidth;
            label.fillColor = this.fillColor;
            return base.prototype.clone.call(this, label);
        };
        return Label;
    })(TextBlock);
    ej.Diagram.Label = function (props) {
        return new Label(props);
    };
    //#endregion

    //#region Port
    var Port = (function () {
        function Port(props) {
            this.name = "";
            this.visibility = ej.Diagram.PortVisibility.Default;
            this.constraints = ej.Diagram.PortConstraints.Connect;
            this.size = 8;
            this.offset = new ej.Diagram.Point(0, 0);
            this.borderColor = "#1a1a1a";
            this.borderWidth = 1;
            this.fillColor = "white";
            this.shape = ej.Diagram.PortShapes.Square;
            this.pathData = "";
            ej.Diagram.merge(this, props);
        }
        Port.prototype.merge = function (prop, value) {
            if (prop.toString() === "offset") {
                this.offset = new ej.Diagram.Point(value.x, value.y);
            }
        };
        Port.prototype.canConnect = function () {
            return this.constraints & ej.Diagram.PortConstraints.Connect;
        };
        Port.prototype._getPosition = function (bounds, bExcludeBoundsLocation) {
            if (bExcludeBoundsLocation) {
                return new ej.Diagram.Point(bounds.width * this.offset.x, bounds.height * this.offset.y);
            }
            else {
                var x = bounds.x + (bounds.width * this.offset.x);
                var y = bounds.y + (bounds.height * this.offset.y);
                return new ej.Diagram.Point(x, y);
            }
        };
        return Port;
    })();
    ej.Diagram.Port = function (props) {
        return new Port(props);
    };
    //#endregion

    //#region Stop
    var Stop = (function () {
        function Stop(props) {
            this.color = "";
            this.offset = 0;
            this.opacity = 1;
            ej.Diagram.merge(this, props);
        }
        return Stop;
    })();
    ej.Diagram.Stop = function (props) {
        return new Stop(props);
    };
    //#endregion

    //#region LinearGradient
    var Gradient = (function () {
        function Gradient(props) {
            this.stops = new ej.Diagram.Collection();
            ej.Diagram.merge(this, props);
        }
        Gradient.prototype.merge = function (prop, value) {
            if (prop.toString() === "stops") {
                for (var i = 0, len = value.length; i < len; i++) {
                    this.stops.add(new ej.Diagram.Stop(value[i]));
                }
            }
        };
        return Gradient;
    })();
    ej.Diagram.Gradient = function (props) {
        return new Gradient(props);
    };
    var LinearGradient = (function (base) {
        ej.Diagram.extend(LinearGradient, base);
        function LinearGradient(props) {
            this.x1 = 0;
            this.x2 = 0;
            this.y1 = 0;
            this.y2 = 0;
            this.type = "linear";
            ej.Diagram.merge(this, props);
            base.call(this, props);
        }
        LinearGradient.prototype.clone = function (src) {
            var gradient = src ? src : new ej.Diagram.LinearGradient();
            gradient.x1 = this.x1;
            gradient.x2 = this.x2;
            gradient.y1 = this.y1;
            gradient.y2 = this.y2;
            gradient.stops = this.stops.clone();
            return gradient;
        };
        return LinearGradient;
    })(Gradient);
    ej.Diagram.LinearGradient = function (props) {
        return new LinearGradient(props);
    };
    //#endregion

    //#region RadialGradient
    var RadialGradient = (function (base) {
        ej.Diagram.extend(RadialGradient, base);
        function RadialGradient(props) {
            this.cx = 0;
            this.cy = 0;
            this.fx = 0;
            this.fy = 0;
            this.r = 0;
            this.type = "radial";
            ej.Diagram.merge(this, props);
            base.call(this, props);
        }
        return RadialGradient;
    })(ej.Diagram.Gradient);
    ej.Diagram.RadialGradient = function (props) {
        return new RadialGradient(props);
    };
    //#endregion

    //#region Node
    var NodeBase = (function () {
        function NodeBase(props) {
            this.name = "";
            this.width = 50;
            this.height = 50;
            this.offsetX = 0;
            this.offsetY = 0;
            this.visible = true;
            this.constraints = ej.Diagram.NodeConstraints.Default;
            this.parent = null;
            this.zOrder = 0;
            this.exclude = false;
            this.labels = ej.Diagram.Collection();
            this.ports = ej.Diagram.Collection();
            this.inEdges = ej.Diagram.Collection();
            this.outEdges = ej.Diagram.Collection();
            this.rotateAngle = 0;
            this.pivot = new ej.Diagram.Point(0.5, 0.5);
            this._diagram = null;
            this.addInfo = null;
            this._corners = null;
            this._bounds = null;
            ej.Diagram.merge(this, props);
        }
        NodeBase.prototype.merge = function (prop, value) {
            var i, len;
            if (prop.toString() === "labels") {
                for (i = 0, len = value.length; i < len; i++)
                    this.labels.add(new ej.Diagram.Label(value[i]));
            }
            else if (prop.toString() === "ports") {
                for (i = 0, len = value.length; i < len; i++)
                    this.ports.add(new ej.Diagram.Port(value[i]));
            }
            else if (prop.toString() === "pivot")
                this.pivot = new ej.Diagram.Point(value.x, value.y);
            else if (prop.toString() === "addInfo")
                this.addInfo = value;
            else if (prop.toString() === "parent" && !(value.type === "pseudoGroup"))
                this.parent = value;
            else if (prop.toString() === "gradient")
                if (value && value.r) {
                    this.gradient = new ej.Diagram.RadialGradient(value);
                } else {
                    this.gradient = new ej.Diagram.LinearGradient(value);
                }

        };
        NodeBase.prototype.getBoundingBox = function () {
            return new ej.Diagram.Rectangle(this.offsetX - this.width * this.pivot.x, this.offsetY - this.height * this.pivot.y, this.width, this.height);
        };
        NodeBase.prototype.fullName = function () {
            if (this.parent) {
                var parentName = this.parent.fullName();
                if (parentName) {
                    return parentName + "." + this.name;
                }
                return this.name;
            }
            else {
                return this.name;
            }
        };
        NodeBase.prototype.canResize = function () {
            return this.constraints & ej.Diagram.NodeConstraints.Resize;
        };
        NodeBase.prototype.canMove = function () {
            return this.constraints & ej.Diagram.NodeConstraints.Drag;
        };
        NodeBase.prototype.canRotate = function () {
            return this.constraints & ej.Diagram.NodeConstraints.Rotate;
        };
        NodeBase.prototype.canSelect = function () {
            return this.constraints & ej.Diagram.NodeConstraints.Select;
        };
        NodeBase.prototype.canDelete = function () {
            return this.constraints & ej.Diagram.NodeConstraints.Delete;
        };
        NodeBase.prototype.canInConnect = function () {
            return this.constraints & ej.Diagram.NodeConstraints.InConnect;
        };
        NodeBase.prototype.canOutConnect = function () {
            return this.constraints & ej.Diagram.NodeConstraints.OutConnect;
        };
        NodeBase.prototype.canConnect = function () {
            return this.constraints & ej.Diagram.NodeConstraints.Connect;
        };
        NodeBase.prototype.contains = function (point) {
            var bounds = this.getBoundingBox();
            if (bounds.contains(point)) {
                return true;
            }
            return false;
        };
        NodeBase.prototype._updateAssociatedConnectorEnds = function () {
            var i;
            var len;
            if (this.inEdges.length > 0) {
                for (i = 0, len = this.inEdges.length; i < len; i++) {
                    this.inEdges[i].dock();
                    this.inEdges[i].updatebounds();
                }
            }
            if (this.outEdges.length > 0) {
                for (i = 0, len = this.outEdges.length; i < len; i++) {
                    this.outEdges[i].dock();
                    this.outEdges[i].updatebounds();
                }
            }
        };
        return NodeBase;
    })();
    ej.Diagram.NodeBase = function (props) {
        return new NodeBase(props);
    };
    var Node = (function (base) {
        ej.Diagram.extend(Node, base);
        function Node(props) {
            this.type = "node";
            this.fillColor = "white";
            this.borderColor = "black";
            this.borderWidth = 1;
            this.borderDashArray = null;
            this.opacity = 1;
            this.gradient = null;
            this.shape = new ej.Diagram.Shapes.Rectangle();
            ej.Diagram.merge(this, props);
            base.call(this, props);
        }
        Node.prototype.merge = function (prop, value) {
            base.prototype.merge.call(this, prop, value);
            if (prop.toString() === "shape") {
                if (value.type) {
                    var type = value.type;
                    if (type === "rectangle")
                        this.shape = new ej.Diagram.Shapes.Rectangle(value);
                    else if (type === "circle")
                        this.shape = new ej.Diagram.Shapes.Circle(value);
                    else if (type === "polygon")
                        this.shape = new ej.Diagram.Shapes.Polygon(value);
                    else if (type === "image")
                        this.shape = new ej.Diagram.Shapes.Image(value);
                    else if (type === "path")
                        this.shape = new ej.Diagram.Shapes.Path(value);
                    else if (type === "text")
                        this.shape = new ej.Diagram.Shapes.Text(value);
                    else if (type === "html")
                        this.shape = new ej.Diagram.Shapes.Html(value);
                    else if (type === "native")
                        this.shape = new ej.Diagram.Shapes.Native(value);
                }
            }
        };
        Node.prototype._scale = function (mat, deltaWidth, deltaHeight, pivot) {
            var scaledMatrix = mat._createIdentity();
            scaledMatrix._rotateAt(-this.rotateAngle, pivot.x, pivot.y);
            scaledMatrix._scaleAt(deltaWidth, deltaHeight, pivot.x, pivot.y);
            scaledMatrix._rotateAt(this.rotateAngle, pivot.x, pivot.y);
            var newPosition = scaledMatrix._transform(new ej.Diagram.Point(this.offsetX, this.offsetY));
            var width = this.width * deltaWidth;
            var height = this.height * deltaHeight;
            if (width > 1) {
                this.width = width > 1 ? width : this.width;
                this.offsetX = newPosition.x;
            }
            if (height > 1) {
                this.height = height > 1 ? height : this.height;
                this.offsetY = newPosition.y;
            }
            this.updatebounds();
            this._updateAssociatedConnectorEnds();
            if (this.parent && this.parent.type === "group") {
                if (this._diagram && this.parent != this._diagram.selectionList[0]) {
                    this.parent._updateBoundingBox();
                }
            }
        };
        Node.prototype.updatebounds = function () {
            this._corners = new ej.Diagram.RectCorners(new ej.Diagram.Rectangle(this.offsetX, this.offsetY, this.width, this.height),
                false, this.pivot, this.rotateAngle);
            var bounds = this._corners._outerBounds;
            if (!this._bounds || bounds.x != this._bounds.x || bounds.y != this._bounds.y || bounds.width != this._bounds.width || bounds.height != this._bounds.height) {
                this._bounds = bounds;
                if (this._diagram) {
                    this._diagram._spatialsearch._updateQuad(this);
                }
            }
        };
        Node.prototype._translate = function (dx, dy) {
            this.offsetX += dx;
            this.offsetY += dy;
            this.updatebounds();
            this._updateAssociatedConnectorEnds();
            if (this.parent && this.parent.type === "group") {
                if (this._diagram && this.parent != this._diagram.selectionList[0]) {
                    this.parent._updateBoundingBox();
                }
            }
        };
        Node.prototype._rotate = function (angle, pivot) {
            this.rotateAngle += angle;
            this.rotateAngle %= 360;
            if (this.rotateAngle < 0) this.rotateAngle += 360;
            this.updatebounds();
            this._updateAssociatedConnectorEnds();
            if (this.parent && this.parent.type === "group") {
                if (this._diagram && this.parent != this._diagram.selectionList[0]) {
                    this.parent._updateBoundingBox();
                }
            }
        };
        Node.prototype.updateShape = function () {


        };
        return Node;
    })(NodeBase);
    ej.Diagram.Node = function (props) {
        return new Node(props);
    };
    //#endregion

    //#region Connector
    var Connector = (function () {
        function Connector(props) {
            this.type = "connector";
            this.name = "";
            this.visible = true;
            this.lineDashArray = null;
            this.targetDecorator = null;
            this.sourceDecorator = null;
            this.line = new ej.Diagram.Lines.Straight({ "startPoint": new ej.Diagram.Point(0, 0), "endPoint": new ej.Diagram.Point(0, 0) });
            this.lineColor = "black";
            this.lineWidth = 1;
            this.constraints = ej.Diagram.ConnectorConstraints.Default;
            this.parent = null;
            this.opacity = 1;
            this.labels = new ej.Diagram.Collection();
            this.tag = null;
            this.zOrder = 0;
            this.lineHitPadding = 10;
            this._diagram = null;
            this.targetNode = null;
            this.sourceNode = null;
            this.targetPort = null;
            this.sourcePort = null;
            this._bounds = null;
            this.targetNodeName = null;
            this.targetPortName = null;
            this.sourceNodeName = null;
            this.sourcePortName = null;
            this.addInfo = null;
            ej.Diagram.merge(this, props);
        }
        Connector.prototype.merge = function (prop, value) {
            if (prop.toString() === "line") {
                if (value.type) {
                    var type = value.type;
                    if (type === "straight")
                        this.line = new ej.Diagram.Lines.Straight(value);
                    else if (type === "orthogonal")
                        this.line = new ej.Diagram.Lines.Orthogonal(value);
                    else if (type === "bezier")
                        this.line = new ej.Diagram.Lines.Bezier(value);
                }
            }
            else if (prop.toString() === "labels") {
                for (var i = 0, len = value.length; i < len; i++)
                    this.labels.add(new ej.Diagram.Label(value[i]));
            }
            else if (prop.toString() === "targetDecorator" && Object.getOwnPropertyNames(value).length > 0)
                this.targetDecorator = new ej.Diagram.Decorator(value);
            else if (prop.toString() === "sourceDecorator" && Object.getOwnPropertyNames(value).length > 0)
                this.sourceDecorator = new ej.Diagram.Decorator(value);
            else if (prop.toString() === "targetNode" && value.type == "node")
                this.targetNode = value;
            else if (prop.toString() === "targetPort")
                this.targetPort = value;
            else if (prop.toString() === "sourceNode" && value.type === "node")
                this.sourceNode = value;
            else if (prop.toString() === "sourcePort")
                this.sourcePort = value;
            else if (prop.toString() === "addInfo")
                this.addInfo = value;
            else if (prop.toString() === "parent" && !(value.type === "pseudoGroup"))
                this.parent = value;
        };
        Connector.prototype.getBoundingBox = function () {
            var bounds = ej.Diagram.Geometry.rect(this.line.points);
            return bounds;
        };
        Connector.prototype.updatebounds = function () {
            var bounds = this.getBoundingBox();
            var pivot = new ej.Diagram.Point(.5, .5);
            this._corners = new ej.Diagram.RectCorners(new ej.Diagram.Rectangle((bounds.x + (bounds.width / 2)), (bounds.y + (bounds.height / 2)), bounds.width, bounds.height), false, pivot, 0);

            if (!this._bounds || bounds.x != this._bounds.x || bounds.y != this._bounds.y || bounds.width != this._bounds.width || bounds.height != this._bounds.height) {
                this._bounds = bounds;
                if (this._diagram)
                    this._diagram._spatialsearch._updateQuad(this);
                if (this.parent && this.parent.type === "group") {
                    if (this._diagram && this.parent != this._diagram.selectionList[0]) {
                        this.parent._updateBoundingBox();
                    }
                }
            }
        };
        Connector.prototype.fullName = function () {
            if (this.parent) {
                var parentName = this.parent.fullName();
                if (parentName) {
                    return parentName + "." + this.name;
                }
                return this.name;
            }
            else {
                return this.name;
            }
        };
        Connector.prototype.canMove = function () {
            return this.constraints & ej.Diagram.ConnectorConstraints.Drag;
        };
        Connector.prototype.canRotate = function () {
            return this.constraints & ej.Diagram.ConnectorConstraints.Rotate;
        };
        Connector.prototype.canSelect = function () {
            return this.constraints & ej.Diagram.ConnectorConstraints.Select;
        };
        Connector.prototype.canDelete = function () {
            return this.constraints & ej.Diagram.ConnectorConstraints.Delete;
        };
        Connector.prototype.canDragSourceEnd = function () {
            return this.constraints & ej.Diagram.ConnectorConstraints.DragSourceEnd;
        };
        Connector.prototype.canDragTargetEnd = function () {
            return this.constraints & ej.Diagram.ConnectorConstraints.DragTargetEnd;
        };

        Connector.prototype.canDragSegmentThumbs = function () {
            return this.constraints & ej.Diagram.ConnectorConstraints.DragSegmentThumb;
        };
        Connector.prototype.isTargetConnected = function () {
            return this.targetNode ? true : false;
        };
        Connector.prototype.isSourceConnected = function () {
            return this.sourceNode ? true : false;
        };
        Connector.prototype._dockNode = function (targetNode, sourceNode, sourcePort, isTarget) {
            var bounds;
            var center;
            if (targetNode) {
                //dock sourceNode
                if (sourceNode) {
                    bounds = sourceNode.getBoundingBox();
                    if (!sourcePort) {
                        center = bounds.center();
                        this._setEndPoint(center, !isTarget);
                        center = targetNode.getBoundingBox().center();
                        this._setEndPoint(center, isTarget);
                        this.line._dockNode(bounds, !isTarget, this);
                    }
                }
                //dock targetNode
                bounds = targetNode.getBoundingBox();
                center = bounds.center();
                this._setEndPoint(center, isTarget);
                this.line._dockNode(bounds, isTarget, this);
            }
        };
        Connector.prototype._dockPortToPort = function (targetNode, targetPort, sourceNode, sourcePort) {
            this.line._dockPortToPort(targetNode, targetPort, sourceNode, sourcePort);
        };
        Connector.prototype._dockPortToEndPoint = function (targetNode, targetPort, isTarget) {
            this.line._dockPortToEndPoint(targetNode, targetPort, isTarget);
        };
        Connector.prototype.dock = function () {
            if (this.isTargetConnected() && this.isSourceConnected() && this.targetPort && this.sourcePort) {
                this._dockPortToPort(this.targetNode, this.targetPort, this.sourceNode, this.sourcePort);
            }
            else if (this.isTargetConnected() && this.targetPort) {
                if (this.isSourceConnected()) {
                    if (!(this.line.type === "orthogonal")) {
                        this.line._dockPortToNode(this.targetNode, this.targetPort, true, this);
                        this.line._dockNode(this.sourceNode, null, this);
                    }
                    else {
                        this.line._dockNode(this.sourceNode, null, this);
                        this.line._dockPortToNode(this.targetNode, this.targetPort, true, this);
                    }
                }
                else {
                    this._dockPortToEndPoint(this.targetNode, this.targetPort, true);
                }
            }
            else if (this.isSourceConnected() && this.sourcePort) {
                if (this.isTargetConnected()) {
                    if (!(this.line.type == "orthogonal")) {
                        this.line._dockPortToNode(this.sourceNode, this.sourcePort, false, this);
                        this.line._dockNode(this.targetNode, null, this);
                    }
                    else {
                        this.line._dockNode(this.targetNode, null, this);
                        this.line._dockPortToNode(this.sourceNode, this.sourcePort, false, this);
                    }
                }
                else {
                    this._dockPortToEndPoint(this.sourceNode, this.sourcePort, false);
                }
            }
            else if (this.isTargetConnected() && this.isSourceConnected()) {
                this.line._dockNode(this.targetNode, this.sourceNode, this);
            }
            else if (this.isTargetConnected()) {
                this.line._dockNode(this.targetNode, null, this);
            }
            else if (this.isSourceConnected()) {
                this.line._dockNode(this.sourceNode, null, this);
            }
            this.updatebounds();
        };
        Connector.prototype.setPoints = function (points) {
            this.line._setPoints(points);
            this.updatebounds();
        };
        Connector.prototype._translate = function (dx, dy) {
            this.line._translate(dx, dy, this);
            this.dock();
            this.updatebounds();
        };

        Connector.prototype._scale = function (mat, deltaWidth, deltaHeight, pivot) {
            var scaledMatrix = mat._createIdentity();
            scaledMatrix._scaleAt(deltaWidth, deltaHeight, pivot.x, pivot.y);
            if (!this.sourceNode)
                this.line._setEndPoint(scaledMatrix._transform(this.line.startPoint), false);
            if (!this.targetNode)
                this.line._setEndPoint(scaledMatrix._transform(this.line.endPoint), true);
            this.dock();
            this.updatebounds();
        };

        Connector.prototype._updateEndPoint = function (dx, dy, isTarget) {
            this.line._updateEndPoint(dx, dy, isTarget, this);
            this.updatebounds();
        };
        Connector.prototype._setEndPoint = function (point, isTarget) {
            this.line._setEndPoint(point, isTarget, null, this);
            this.updatebounds();
        };
        return Connector;
    })();
    ej.Diagram.Connector = function (props) {
        return new Connector(props);
    };
    //#endregion

    //#region Group
    var Group = (function (base) {
        ej.Diagram.extend(Group, base);
        function Group(props) {
            base.call(this);
            this.type = "group";
            this._childBounds = null;
            this.children = new ej.Diagram.Collection();
            this.canUngroup = true;
            ej.Diagram.merge(this, props);
            base.call(this, props);
            this._wireEvents();
        }
        Group.prototype.merge = function (prop, value) {
            base.prototype.merge.call(this, prop, value);
            if (prop.toString() === "children") {
                if (value.type === "collection") {
                    this.children = ej.Diagram.Util.clone(value);
                }
                else {
                    for (var i = 0, len = value.length; i < len; i++) {
                        if (value.type) {
                            if (value.type === "node")
                                this.children.add(new ej.Diagram.Node(value[i]));
                            else if (value.type === "group")
                                this.children.add(new ej.Diagram.Group(value[i]));
                            else if (value.type === "connector")
                                this.children.add(new ej.Diagram.Connector(value[i]));
                        }
                    }
                }
            }
        };
        Group.prototype._translate = function (dx, dy) {
            this.offsetX += dx;
            this.offsetY += dy;
            for (var i = 0, len = this.children.length; i < len; i++) {
                this.children[i]._translate(dx, dy);
            }
            this._updateBoundingBox();
            this._updateAssociatedConnectorEnds();
        };
        Group.prototype._updateBoundingBox = function (child, childinteracting) {
            var bounds = this._childBounds ? this._childBounds : new ej.Diagram.Rectangle();
            var matrix = new ej.Diagram.Matrix()._createIdentity();
            if (child) {
                var bnds;
                if (this.children.length === 1 && !(child.type === "connector")) {
                    this.rotateAngle = child.rotateAngle;
                }

                if (child.type == "group") { bnds = child._updateBoundingBox(); }
                //else
                {
                    matrix._createIdentity();
                    bnds = this._findChildBounds(child, matrix);
                }
                if (this.children.length === 1) {
                    bounds = bnds;
                }
                else
                    bounds = bounds.union(bnds);


            }
            else {
                var children = this.children;
                bounds = new ej.Diagram.Rectangle();
                for (var i = 0, len = children.length; i < len; i++) {
                    var child = this.children[i];
                    var bnds = new ej.Diagram.Rectangle();
                    if (child.type === "group") {
                        if (!childinteracting) {
                            bnds = child._updateBoundingBox();
                            matrix._createIdentity();
                            bnds = this._findChildBounds(child, matrix);
                        }
                        else {
                            bnds = child._bounds;
                        }
                    }
                    else {
                        matrix._createIdentity();
                        bnds = this._findChildBounds(child, matrix);
                    }
                    if (i === 0)
                        bounds = bnds;
                    bounds = bounds.union(bnds);
                    child = null;
                }

            }
            var matrix1 = matrix._createIdentity();
            var x = bounds.x + bounds.width * this.pivot.x;
            var y = bounds.y + bounds.height * this.pivot.y;
            matrix1._rotate(this.rotateAngle);
            var newposition = matrix1._transform(new ej.Diagram.Point(x, y));
            this.offsetX = newposition.x;
            this.offsetY = newposition.y;
            this.width = bounds.width;
            this.height = bounds.height;
            this._childBounds = bounds;
            this.updatebounds();
            if (!child && this.parent && this.parent.type == "group") {
                if (this._diagram && this.parent != this._diagram.selectionList[0]) {
                    this.parent._updateBoundingBox(null, true);
                }
            }
            return bounds;
        };
        Group.prototype._findChildBounds = function (child, matrix) {
            var bnds = new ej.Diagram.Rectangle();
            matrix._rotate(-this.rotateAngle);
            if (child.type === "connector") {
                var topleft = matrix._transform(new ej.Diagram.Point(child._bounds.x, child._bounds.y));
                var topright = matrix._transform(new ej.Diagram.Point(child._bounds.right(), child._bounds.y));
                var bottomLeft = matrix._transform(new ej.Diagram.Point(child._bounds.x, child._bounds.bottom()));
                var bottomRight = matrix._transform(new ej.Diagram.Point(child._bounds.right(), child._bounds.bottom()));
            }
            else {
                var topleft = matrix._transform(child._corners._topLeft);
                var topright = matrix._transform(child._corners._topRight);
                var bottomLeft = matrix._transform(child._corners._bottomLeft);
                var bottomRight = matrix._transform(child._corners._bottomRight);
            }
            bnds.x = Math.min(topleft.x, topright.x, bottomLeft.x, bottomRight.x);
            bnds.width = Math.max(topleft.x, topright.x, bottomLeft.x, bottomRight.x) - bnds.x;
            bnds.y = Math.min(topleft.y, topright.y, bottomLeft.y, bottomRight.y);
            bnds.height = Math.max(topleft.y, topright.y, bottomLeft.y, bottomRight.y) - bnds.y;
            return bnds;
        };
        Group.prototype._childrenBounds = function () {
            var children = this.children;
            var bounds = new ej.Diagram.Rectangle();
            bounds = children[0]._bounds ? children[0]._bounds : children[0].getBoundingBox();
            for (var i = 1, len = children.length; i < len; i++) {
                bounds = bounds.union(children[i]._bounds ? children[i]._bounds : children[i].getBoundingBox());
            }
            return bounds;
        };
        Group.prototype._scale = function (mat, deltaWidth, deltaHeight, pivot) {

            for (var i = 0; i < this.children.length; i++) {
                this.children[i]._scale(mat, deltaWidth, deltaHeight, pivot);
            }
            var scaledMatrix = mat._createIdentity();
            scaledMatrix._rotateAt(-this.rotateAngle, pivot.x, pivot.y);
            scaledMatrix._scaleAt(deltaWidth, deltaHeight, pivot.x, pivot.y);
            scaledMatrix._rotateAt(this.rotateAngle, pivot.x, pivot.y);
            var newPosition = scaledMatrix._transform(new ej.Diagram.Point(this.offsetX, this.offsetY));
            var width = this.width * deltaWidth;
            var height = this.height * deltaHeight;
            if (width > 1) {
                this.width = width > 1 ? width : this.width;
                this.offsetX = newPosition.x;
            }
            if (height > 1) {
                this.height = height > 1 ? height : this.height;
                this.offsetY = newPosition.y;
            }
            this._updateBoundingBox();
            this._updateAssociatedConnectorEnds();
        };
        Group.prototype._rotate = function (angle, pivot) {
            this.rotateAngle += angle;
            this.rotateAngle %= 360;
            if (this.rotateAngle < 0) this.rotateAngle += 360;
            var mat = new ej.Diagram.Matrix()._createIdentity();
            mat._rotateAt(angle, this.offsetX, this.offsetY);
            for (var i = 0; i < this.children.length; i++) {
                var child = this.children[i];
                if (!(child.type === "connector")) {
                    child._rotate(angle);
                    var pinx = child.offsetX;
                    var piny = child.offsetY;
                    var actualpt = mat._transform(new ej.Diagram.Point(pinx, piny));
                    child._translate(actualpt.x - pinx, actualpt.y - piny);
                }
                else {
                    if (!child.sourceNode)
                        child._setEndPoint(mat._transform(child.line.startPoint), false);
                    if (!child.targetNode)
                        child._setEndPoint(mat._transform(child.line.endPoint), true);
                }
            }
            this._updateBoundingBox();
        };
        Group.prototype.updatebounds = function (grp) {
            this._corners = new ej.Diagram.RectCorners(new ej.Diagram.Rectangle(this.offsetX, this.offsetY, this.width, this.height),
                false, this.pivot, this.rotateAngle, grp);
            this._bounds = this._corners._outerBounds;
        };
        Group.prototype._wireEvents = function () {
            this.children.addEventHandler("collectionchanged", $.proxy(this._collectionChanged, this));
        };
        Group.prototype._collectionChanged = function (evt) {
            if (evt.changeType === "insert") {
                var child = evt.element;
                if (!(this.type === "pseudoGroup"))
                    child.parent = this;
                if (!child._bounds)
                    child.updatebounds();
                this._updateBoundingBox(child);
            }
            else {
                this._updateBoundingBox();
            }
        };
        return Group;
    })(NodeBase);
    ej.Diagram.Group = function (props) {
        return new Group(props);
    };
    //#endregion

    //#region PseudoGroup
    var PseudoGroup = (function (base) {
        ej.Diagram.extend(PseudoGroup, base);
        function PseudoGroup(props) {
            base.call(this, props);
            this.type = "pseudoGroup";
        }
        PseudoGroup.prototype.fullName = function () {
            return "";
        };
        return PseudoGroup;
    })(Group);
    ej.Diagram.PseudoGroup = function (props) {
        return new PseudoGroup(props);
    };
    //#endregion

    //#region ContextMenu
    var ContextMenu = (function () {
        function ContextMenu(props) {
            this.items = new ej.Diagram.Collection();
            this.onClick = "";
            this.onBeforeOpen = "";
            this.enable = true;
            this._isDefaultLoaded = false;
            ej.Diagram.merge(this, props);
        }
        ContextMenu.prototype.merge = function (prop, value) {
            if (prop.toString() === "items") {
                for (var i = 0, len = value.length; i < len; i++)
                    this.items.add(new ej.Diagram.ContextMenuItem(value[i]));
            }
        };
        ContextMenu.prototype.loadDefaults = function () {
            this._isDefaultLoaded = true;
            var item, subItem;
            this.items.add(this._createMenuItems("cut", "Cut"));
            this.items.add(this._createMenuItems("copy", "Copy"));
            this.items.add(this._createMenuItems("paste", "Paste"));
            this.items.add(this._createMenuItems("undo", "Undo"));
            this.items.add(this._createMenuItems("redo", "Redo"));
            this.items.add(this._createMenuItems("selectAll", "Select All"));

            item = this._createMenuItems("grouping", "Grouping");
            subItem = new ej.Diagram.Collection();
            subItem.add(this._createMenuItems("group", "Group"));
            subItem.add(this._createMenuItems("ungroup", "Ungroup"));
            item.subItems = (subItem);
            this.items.add(item);

            item = this._createMenuItems("order", "Order");
            subItem = new ej.Diagram.Collection();
            subItem.add(this._createMenuItems("bringToFront", "Bring To Front"));
            subItem.add(this._createMenuItems("sendToBack", "Send To Back"));
            subItem.add(this._createMenuItems("moveForward", "Move Forward"));
            subItem.add(this._createMenuItems("sendBackward", "Send Backward"));

            item.subItems = (subItem);
            this.items.add(item);
        };
        ContextMenu.prototype._createMenuItems = function (name, text) {
            var item = new ej.Diagram.ContextMenuItem();
            item.name = name;
            item.text = text;
            return item;
        };
        return ContextMenu;
    })();
    ej.Diagram.ContextMenu = function (props) {
        return new ContextMenu(props);
    };
    var ContextMenuItem = (function () {
        function ContextMenuItem(props) {
            this.name = "";
            this.text = "";
            this.subItems = new ej.Diagram.Collection();
            ej.Diagram.merge(this, props);
        }
        ContextMenuItem.prototype.merge = function (prop, value) {
            if (prop.toString() === "subItems") {
                for (var i = 0, len = value.length; i < len; i++)
                    this.subItems.add(new ej.Diagram.ContextMenuItem(value[i]));
            }
        };
        return ContextMenuItem;
    })();
    ej.Diagram.ContextMenuItem = function (props) {
        return new ContextMenuItem(props);
    };
    //#endregion

    //#region VisualGuide
    var VisualGuide = (function () {
        function VisualGuide(props) {
            this.type = ej.Diagram.VisualGuideType.Tooltip;
            this.tooltipTemplateId = "";
            ej.Diagram.merge(this, props);
        }
        return VisualGuide;
    })();
    ej.Diagram.VisualGuide = function (props) {
        return new VisualGuide(props);
    };

    ej.Diagram.VisualGuideType = {
        Tooltip: 1,
        Lines: 2,
        All: 1 | 2
    };
    //#endregion

    //#region ZoomCommand
    ej.Diagram.ZoomCommand = {
        ZoomIn: 1,
        ZoomOut: 2
    };

    var Zoom = (function () {
        function Zoom(props) {
            this.zoomFactor = 0.2;
            this.focusPoint = null;
            this.zoomCommand = ej.Diagram.ZoomCommand.ZoomIn;
            ej.Diagram.merge(this, props);
        }
        return Zoom;
    })();
    ej.Diagram.Zoom = function (props) {
        return new Zoom(props);
    };
    //#endregion

    //#region RectCorners
    ej.Diagram.RectCorners = function (rect, actual, pivot, angle, grp) {
        this._topLeft = null;
        this._top = null;
        this._topRight = null;
        this._left = null;
        this._right = null;
        this._bottomLeft = null;
        this._bottom = null;
        this._bottomRight = null;
        this._isActual = null;
        this._outerBounds = new ej.Diagram.Rectangle();
        this._matrix = null;
        this._center = null;
        this._origX = null;
        this._origY = null;
        this._isActual = actual;
        this._origX = rect.x;
        this._origY = rect.y;
        var _origW = rect.width;
        var _origH = rect.height;
        var w = _origW;
        this._origX = this._origX - pivot.x * w;
        var twoX = this._origX + w / 2;
        var threeX = this._origX + w;
        var h = _origH;
        this._origY = this._origY - pivot.y * h;
        var twoY = this._origY + h / 2;
        var threeY = this._origY + h;
        this._topLeft = new ej.Diagram.Point(this._origX, this._origY);
        this._top = new ej.Diagram.Point(twoX, this._origY);
        this._topRight = new ej.Diagram.Point(threeX, this._origY);
        this._left = new ej.Diagram.Point(this._origX, twoY);
        this._center = new ej.Diagram.Point(twoX, twoY);
        this._right = new ej.Diagram.Point(threeX, twoY);
        this._bottomLeft = new ej.Diagram.Point(this._origX, threeY);
        this._bottom = new ej.Diagram.Point(twoX, threeY);
        this._bottomRight = new ej.Diagram.Point(threeX, threeY);
        var mat = new ej.Diagram.Matrix();
        this._matrix = mat._createIdentity();
        if (angle != 0) {
            this._matrix._rotateAt(angle, rect.x, rect.y);
            this._topLeft = this._matrix._transform(this._topLeft);
            this._top = this._matrix._transform(this._top);
            this._topRight = this._matrix._transform(this._topRight);
            this._left = this._matrix._transform(this._left);
            this._center = this._matrix._transform(this._center);
            this._right = this._matrix._transform(this._right);
            this._bottomLeft = this._matrix._transform(this._bottomLeft);
            this._bottom = this._matrix._transform(this._bottom);
            this._bottomRight = this._matrix._transform(this._bottomRight);
            var minX = Math.min(this._topLeft.x, this._top.x, this._topRight.x, this._left.x, this._right.x, this._bottomLeft.x, this._bottom.x, this._bottomRight.x);
            var minY = Math.min(this._topLeft.y, this._top.y, this._topRight.y, this._left.y, this._right.y, this._bottomLeft.y, this._bottom.y, this._bottomRight.y);
            var maxX = Math.max(this._topLeft.x, this._top.x, this._topRight.x, this._left.x, this._right.x, this._bottomLeft.x, this._bottom.x, this._bottomRight.x);
            var maxY = Math.max(this._topLeft.y, this._top.y, this._topRight.y, this._left.y, this._right.y, this._bottomLeft.y, this._bottom.y, this._bottomRight.y);
            this._outerBounds = new ej.Diagram.Rectangle(minX, minY, maxX - minX, maxY - minY);
        }
        else {
            this._outerBounds = new ej.Diagram.Rectangle(this._left.x, this._top.y, this._right.x - this._left.x, this._bottom.y - this._top.y);
        }
    };

    ej.Diagram.RectCorners.prototype.Transform = function (point) {
        return _matrix._transform(new ej.Diagram.Point(_origX + point.x, _origY + point.y));
    };
    //#endregion

    //#region Clipboard
    ej.Diagram.Clipboard = (function () {
        function Clipboard(diagram) {
            this.diagram = diagram;
            this.pasteIndex = null;
            this.clipboardData = null;
        }

        Clipboard.prototype.cut = function () {
            this._setClipboard();
            this.diagram._delete();
            this.pasteIndex = 0;
        };
        Clipboard.prototype.copy = function () {
            this._setClipboard();
            this.pasteIndex = 1;
        };
        Clipboard.prototype.paste = function () {
            if (this.getData()) {
                var data = this.getData().Data;
                var clonedData = this._getClonedData(data);
                var node;
                if (data.length > 0) {
                    node = new ej.Diagram.PseudoGroup();
                    node.name = "multipleSelection";
                    for (var i = 0; i < data.length; i++) {
                        var childNode = data[i];
                        this._setNodeName(childNode);
                        childNode._translate(this.pasteIndex * 10, this.pasteIndex * 10);
                        this.diagram.add(childNode);
                        node.children.add(childNode);
                    }
                    this.diagram.model.nodes.add(node);
                }
                else {
                    node = data;
                    this._setNodeName(node);
                    node._translate(this.pasteIndex * 10, this.pasteIndex * 10);
                    this.diagram.add(node);
                }
                this.setData({ Data: clonedData });
                this.diagram._clearSelection();
                this.diagram._addSelection(node);
                var entry = new ej.Diagram.HistoryEntry(new ej.Diagram.CollectionModifyCmd("insert", node.type === "pseudoGroup" ? node.children : node));
                this.diagram.historyManager.addHistoryEntry(entry);
                this.pasteIndex++;
            }
        };
        Clipboard.prototype.getData = function () {
            if (this.clipboardData) {
                return this.clipboardData;
            }
            else
                return null;
        };
        Clipboard.prototype.setData = function (data) {
            if (data) {
                this.clipboardData = data;
            }
        };
        Clipboard.prototype.clearData = function () {
            this.clipboardData = null;
        };
        Clipboard.prototype._setNodeName = function (node) {
            node.name = node.name + ej.Diagram.Util.randomId();
            if (node.children && node.children.length > 0) {
                for (var j = 0; j < node.children.length; j++) {
                    var childNode = node.children[j];
                    if (childNode.children && childNode.children.length > 0) {
                        this._setNodeName(childNode);
                    }
                    else
                        childNode.name = childNode.name + ej.Diagram.Util.randomId();
                }
            }
        };
        Clipboard.prototype._setClipboard = function () {
            if (this.diagram.selectionList.length > 0) {
                var data, obj;
                obj = this.diagram.selectionList[0];
                if (obj.type === "pseudoGroup") {
                    var collection = new ej.Diagram.Collection();
                    for (var i = 0; i < obj.children.length; i++) {
                        collection.add(ej.Diagram.Util.clone(obj.children[i]));
                    }
                    for (var i = 0; i < obj.children.length; i++) {
                        this._preserveNodeConnection(obj.children[i], collection, this.diagram.selectionList);
                    }
                    data = { Data: collection };
                }
                else {
                    var cloneObject = ej.Diagram.Util.clone(obj);
                    if (cloneObject.type === "group") {
                        this._preserveNodeConnection(obj, cloneObject.children, this.diagram.selectionList);
                    }
                    data = { Data: cloneObject };
                }
                this.setData(data);
            }
        };
        Clipboard.prototype._preserveNodeConnection = function (curNode, nodesClipboard, selectionList) {
            var nodePortContainer;
            if (curNode.type === "connector") {
                var targetNode = curNode.targetNode;
                if (targetNode != null) {
                    if (this._contains(selectionList, targetNode)) {
                        var nodeCloned = this._findNodeByName(targetNode.fullName(), nodesClipboard);
                        var connectorCloned = this._findNodeByName(curNode.fullName(), nodesClipboard);
                        if (curNode.targetPort) {
                            var portCloned = this.diagram._findPort(nodeCloned, curNode.targetPort.name);
                            connectorCloned.targetPort = portCloned;
                        }
                        connectorCloned.targetNode = nodeCloned;

                    }
                }

                var sourceNode = curNode.sourceNode;
                if (sourceNode != null) {
                    if (this._contains(selectionList, sourceNode)) {
                        var nodeCloned = this._findNodeByName(sourceNode.fullName(), nodesClipboard);
                        var connectorCloned = this._findNodeByName(curNode.fullName(), nodesClipboard);
                        if (curNode.sourcePort) {
                            var portCloned = this.diagram._findPort(nodeCloned, curNode.sourcePort.name);
                            connectorCloned.sourcePort = portCloned;
                        }
                        connectorCloned.sourceNode = nodeCloned;

                    }
                }
            }
            else if (curNode.type === "group" || curNode.type === "pseudoGroup") {
                var node = curNode;
                var nLength = node.children.length;
                var nCount = 0;
                var childNode;
                while (nLength > nCount) {
                    childNode = node.children[nCount];
                    this._preserveNodeConnection(childNode, nodesClipboard, selectionList);
                    nCount++;
                }
            }
        };
        Clipboard.prototype._contains = function (nodeCollection, nodeToSearch) {
            var bSuccess = false;
            for (var i = 0; i < nodeCollection.length; i++) {
                var node = nodeCollection[i];
                if (node.fullName() === nodeToSearch.fullName()) {
                    bSuccess = true;
                    break;
                }
                if (node.type === "group" || node.type === "pseudoGroup") {
                    bSuccess = this._contains(node.children, nodeToSearch);
                    if (bSuccess)
                        break;
                }
            }
            return bSuccess;
        };
        Clipboard.prototype._findNodeByName = function (name, nodeCollection) {
            var nodeToReturn = null;
            for (var i = 0; i < nodeCollection.length; i++) {
                var node = nodeCollection[i];
                if (node.fullName() === name) {
                    nodeToReturn = node;
                    break;
                }
                if (node.type === "group" || node.type === "pseudoGroup") {
                    nodeToReturn = this._findNodeByName(name, node.children);
                    if (nodeToReturn)
                        break;
                }
            }
            return nodeToReturn;
        };
        Clipboard.prototype._getClonedData = function (selectionList) {
            var clonedList = ej.Diagram.Util.clone(selectionList);
            for (var i = 0; i < selectionList.length; i++) {
                this._preserveNodeConnection(selectionList[i], clonedList, selectionList);
            }
            return clonedList;
        };
        return Clipboard;
    })();
    //#endregion

    //#region HistoryManager
    ej.Diagram.HistoryManager = (function () {
        function HistoryManager() {
            this._historyList = new ej.Diagram.HistoryList();
            this._events = [];
        };
        HistoryManager.prototype.addHistoryEntry = function (entry) {
            this._historyList.addEntry(entry);
        };
        HistoryManager.prototype.canUndo = function () {
            return this._historyList.canUndo;
        };
        HistoryManager.prototype.canRedo = function () {
            return this._historyList.canRedo;
        };
        HistoryManager.prototype.addEventHandler = function (type, handler) {
            this._events.push({ "type": type, "handler": handler });
        };
        HistoryManager.prototype.raiseEvent = function (type, args) {
            for (var i = 0, len = this._events.length; i < len; i++) {
                var event = this._events[i];
                if (event.type == type) {
                    if (event.handler)
                        event.handler(args);
                }
            }
        };
        HistoryManager.prototype.removeEventHandler = function (type, handler) {
            for (var i = 0, len = this._events.length; i < len; i++) {
                if (this._events[i].type == type && this._events[i].handler == handler) {
                    this._events.splice(i, 1);
                }
            }
        };
        HistoryManager.prototype.undo = function () {
            var cmd = this._historyList.getUndoCmd();
            if (cmd != null) {
                cmd.undo(this);
            }
        };
        HistoryManager.prototype.redo = function () {
            var cmd = this._historyList.getRedoCmd();
            if (cmd != null) {
                cmd.redo(this);
            }
        };
        return HistoryManager;
    })();
    //#endregion

    //#region HistoryList
    ej.Diagram.HistoryList = (function () {
        function HistoryList() {
            this.CurrentEntry = null;
            this.canUndo = false;
            this.canRedo = false;
        }
        HistoryList.prototype.clearHistory = function () {
            this.CurrentEntry = null;
            this.canUndo = false;
            this.canRedo = false;
        };
        HistoryList.prototype.addEntry = function (entry) {
            if (this.CurrentEntry != null) {
                if (this.CurrentEntry.Next != undefined) {
                    if (this.CurrentEntry.Previous != undefined) {
                        var entry1 = this.CurrentEntry.Next;
                        entry1.Previous = null;
                        this.CurrentEntry.Next = entry;
                        entry.Previous = this.CurrentEntry;
                    }
                }
                else {
                    this.CurrentEntry.Next = entry;
                    entry.Previous = this.CurrentEntry;
                }
            }
            this.CurrentEntry = entry;
            this.canUndo = true;
            this.canRedo = false;
        };
        HistoryList.prototype.getUndoCmd = function () {
            var undoEntry = null;
            var currentEntry = null;
            if (this.canUndo) {
                undoEntry = this.CurrentEntry;
                currentEntry = this.CurrentEntry.Previous;
                if (currentEntry != undefined) {
                    this.CurrentEntry = currentEntry;
                    if (!this.canRedo)
                        this.canRedo = true;
                }
                else {
                    this.canRedo = true;
                    this.canUndo = false;
                }
            }
            return undoEntry ? undoEntry.command : undoEntry;
        };
        HistoryList.prototype.getRedoCmd = function () {
            var redoEntry = null;
            var entryCurrent = null;
            if (this.canRedo) {
                if (this.CurrentEntry.Previous == undefined && !this.canUndo) {
                    entryCurrent = this.CurrentEntry;
                }
                else
                    entryCurrent = this.CurrentEntry.Next;
                if (entryCurrent != undefined) {
                    this.CurrentEntry = entryCurrent;

                    if (!this.canUndo)
                        this.canUndo = true;
                    if (entryCurrent.Next == undefined) {
                        this.canRedo = false;
                        this.canUndo = true;
                    }
                }
                redoEntry = this.CurrentEntry;
            }
            return redoEntry ? redoEntry.command : redoEntry;
        };
        return HistoryList;
    })();
    //#endregion

    //#region HistoryEntry
    ej.Diagram.HistoryEntry = (function () {
        function HistoryEntry(command) {
            this.command = command;
        }
        return HistoryEntry;
    })();
    //#endregion

    //#region CollectionModifyCmd
    ej.Diagram.CollectionModifyCmd = (function () {
        function CollectionModifyCmd(type, object) {
            this.changeType = type;
            this.object = object;
        };
        CollectionModifyCmd.prototype.undo = function (historyManager) {
            var args = { object: this.object, changeType: this.changeType, isUndo: true };
            historyManager.raiseEvent("collectionchanged", args);
        };
        CollectionModifyCmd.prototype.redo = function (historyManager) {
            var args = { object: this.object, changeType: this.changeType, isUndo: false };
            historyManager.raiseEvent("collectionchanged", args);
        };
        return CollectionModifyCmd;
    })();
    //#endregion

    //#region MoveHandleCmd
    ej.Diagram.MoveHandleCmd = (function () {
        function MoveHandleCmd(undoObject, redoObject, isMultipleNode) {
            this.undoObject = undoObject;
            this.redoObject = redoObject;
            this.isMultipleNode = isMultipleNode;
        };
        MoveHandleCmd.prototype.undo = function (historyManager) {
            var args = { object: this.undoObject, isMultipleNode: this.isMultipleNode };
            historyManager.raiseEvent("positionchanged", args);
        };

        MoveHandleCmd.prototype.redo = function (historyManager) {
            var args = { object: this.redoObject, isMultipleNode: this.isMultipleNode };
            historyManager.raiseEvent("positionchanged", args);
        };
        return MoveHandleCmd;
    })();
    //#endregion

    //#region ResizeHandleCmd
    ej.Diagram.ResizeHandleCmd = (function () {
        function ResizeHandleCmd(undoObject, redoObject, isMultipleNode) {
            this.undoObject = undoObject;
            this.redoObject = redoObject;
            this.isMultipleNode = isMultipleNode;
        };
        ResizeHandleCmd.prototype.undo = function (historyManager) {
            var args = { object: this.undoObject, isMultipleNode: this.isMultipleNode };
            historyManager.raiseEvent("sizechanged", args);
        };

        ResizeHandleCmd.prototype.redo = function (historyManager) {
            var args = { object: this.redoObject, isMultipleNode: this.isMultipleNode };
            historyManager.raiseEvent("sizechanged", args);
        };
        return ResizeHandleCmd;
    })();
    //#endregion

    //#region LabelChangeCmd
    ej.Diagram.LabelChangeCmd = (function () {
        function LabelChangeCmd(object, previousLabelValue, updatedLabelValue) {
            this.object = object;
            this.previousLabelValue = previousLabelValue;
            this.updatedLabelValue = updatedLabelValue;
        };

        LabelChangeCmd.prototype.undo = function (historyManager) {
            var args = { object: this.object, label: this.previousLabelValue };
            historyManager.raiseEvent("labelchanged", args);
        };

        LabelChangeCmd.prototype.redo = function (historyManager) {
            var args = { object: this.object, label: this.updatedLabelValue };
            historyManager.raiseEvent("labelchanged", args);
        };

        return LabelChangeCmd;
    })();
    //#endregion

    //#region TextChangeCmd
    ej.Diagram.TextChangeCmd = (function () {
        function TextChangeCmd(object, previousValue, updatedValue) {
            this.object = object;
            this.previousValue = previousValue;
            this.updatedValue = updatedValue;
        };

        TextChangeCmd.prototype.undo = function (historyManager) {
            var args = { object: this.object, text: this.previousValue };
            historyManager.raiseEvent("textchanged", args);
        };

        TextChangeCmd.prototype.redo = function (historyManager) {
            var args = { object: this.object, text: this.updatedValue };
            historyManager.raiseEvent("textchanged", args);
        };
        return TextChangeCmd;
    })();
    //#endregion

    //#region EndPointHandleCmd
    ej.Diagram.EndPointHandleCmd = (function () {
        function EndPointHandleCmd(undoObject, redoObject) {
            this.undoObject = undoObject;
            this.redoObject = redoObject;
        };
        EndPointHandleCmd.prototype.undo = function (historyManager) {
            var args = { undoObject: this.undoObject, redoObject: this.redoObject, isUndo: true };
            historyManager.raiseEvent("endpointchanged", args);
        };
        EndPointHandleCmd.prototype.redo = function (historyManager) {
            var args = { undoObject: this.undoObject, redoObject: this.redoObject, isUndo: false };
            historyManager.raiseEvent("endpointchanged", args);
        };
        return EndPointHandleCmd;
    })();
    //#endregion

    //#region RotationHandleCmd
    ej.Diagram.RotationHandleCmd = (function () {
        function RotationHandleCmd(undoObject, redoObject, isMultipleNode) {
            this.undoObject = undoObject;
            this.redoObject = redoObject;
            this.isMultipleNode = isMultipleNode;
        };
        RotationHandleCmd.prototype.undo = function (historyManager) {
            var args = { object: this.undoObject, isMultipleNode: this.isMultipleNode };
            historyManager.raiseEvent("rotationchanged", args);
        };

        RotationHandleCmd.prototype.redo = function (historyManager) {
            var args = { object: this.redoObject, isMultipleNode: this.isMultipleNode };
            historyManager.raiseEvent("rotationchanged", args);
        };
        return RotationHandleCmd;
    })();
    //#endregion

    //#region GroupHandleCmd
    ej.Diagram.GroupHandleCmd = (function () {
        function GroupHandleCmd(object, actionType) {
            this.object = object;
            this.actionType = actionType;
        };
        GroupHandleCmd.prototype.undo = function (historyManager) {
            var args = { object: this.object, isUndo: true, actionType: this.actionType };
            historyManager.raiseEvent("groupingchanged", args);
        };

        GroupHandleCmd.prototype.redo = function (historyManager) {
            var args = { object: this.object, isUndo: false, actionType: this.actionType };
            historyManager.raiseEvent("groupingchanged", args);
        };
        return GroupHandleCmd;
    })();
    //#endregion

    //#region UserHandle
    var UserHandle = (function () {
        function UserHandle(props) {
            this.name = "";
            this.pathData = "";
            this.borderColor = "";
            this.borderWidth = 0;
            this.backgroundColor = "#2382c3";
            this.position = ej.Diagram.Position.BottomCenter;
            this.pathColor = "white";
            this.tool = null;
            this.size = 20;
            this.visibility = true;
            this.enableMultiSelection = false;
            ej.Diagram.merge(this, props);
        }
        return UserHandle;
    })();
    ej.Diagram.UserHandle = function (props) {
        return new UserHandle(props);
    };
    //#endregion

    //#region Serializer
    ej.Diagram.Serializer = (function () {
        function Serializer() {
            this.serializeObj = new Object();
        }
        Serializer.prototype.serialize = function (obj) {
            this._createCommonProperties(obj);
            this._createCanvasProperties(obj);
            this._createNodes(obj);
            this._createConnectors(obj);
            this._createPageSetting(obj);
            this._createSnapSetting(obj);
            this._createJson(obj);
            return this._createJson(this.serializeObj);
        };
        Serializer.prototype._createCommonProperties = function (obj) {
            this.serializeObj.autoScroll = obj.autoScroll ? obj.autoScroll : null;
            this.serializeObj.autoScrollMargin = obj.autoScrollMargin ? obj.autoScrollMargin : null;
            this.serializeObj.borderColor = obj.borderColor ? obj.borderColor : "#DADADA";
            this.serializeObj.borderWidth = obj.borderWidth ? obj.borderWidth : 1;
            this.serializeObj.enableContextMenu = obj.enableContextMenu ? obj.enableContextMenu : true;
            this.serializeObj.enableVisualGuide = obj.enableVisualGuide ? obj.enableVisualGuide : false;
            this.serializeObj.height = obj.height ? obj.height : "100%";
            this._createLayout(obj.layout);
            this.serializeObj.magnification = obj.magnification;
            this.serializeObj.snap = obj.snap;
            this.serializeObj.width = obj.width ? obj.width : "100%";
            this.zoomIncrement = obj.zoomIncrement ? obj.zoomIncrement : 25;
        };
        Serializer.prototype._createLayout = function (obj) {
            var objLayout = obj;
            var layout = null;
            if (objLayout) {
                layout = new Object();
                layout.type = objLayout.type;
                layout.orientation = objLayout.orientation;
                layout.horizontalSpacing = objLayout.horizontalSpacing;
                layout.verticalSpacing = objLayout.verticalSpacing;
                layout.marginX = objLayout.marginX;
                layout.marginY = objLayout.marginY;
                layout.fixedNode = null;
            }
            this.serializeObj.layout = layout;
        };
        Serializer.prototype._createCanvasProperties = function (obj) {
            var objCanvas = obj.canvas;
            var canvas = new Object();
            if (objCanvas) {
                canvas.backgroundColor = objCanvas.backgroundColor;
                canvas.backgroundImage = objCanvas.backgroundImage;
                canvas.height = objCanvas.height;
                canvas.readOnly = objCanvas.readOnly;
                canvas.width = objCanvas.width;
            }
            this.serializeObj.canvas = canvas;
        };
        Serializer.prototype._createGrid = function (obj) {
            var grid = new Object();
            if (obj.grid) {
                var objGrid = obj.grid;
                grid.color = objGrid.color;
                grid.horizontalSpacing = objGrid.horizontalSpacing;
                grid.type = objGrid.type;
                grid.verticalSpacing = objGrid.verticalSpacing;
                grid.visible = objGrid.visible;
            }
            this.serializeObj.grid = grid;
        };
        Serializer.prototype._createNodes = function (obj) {
            var i, node, nodes = [];
            if (obj.nodes.length > 0) {
                for (i = 0; i < obj.nodes.length; i++) {
                    node = this._createNode(obj.nodes[i]);
                    if (node)
                        nodes.push(node);
                }
                this.serializeObj.nodes = nodes;
            }
        };
        Serializer.prototype._createConnectors = function (obj) {
            var i, connector, connectors = [];
            if (obj.connectors.length > 0) {
                for (i = 0; i < obj.connectors.length; i++) {
                    connector = this._createConnector(obj.connectors[i]);
                    if (connector)
                        connectors.push(connector);
                }
                this.serializeObj.connectors = connectors;
            }
        };
        Serializer.prototype._createConnector = function (connector) {
            var sConnector = new Object();
            if (connector) {
                sConnector.name = connector.name || "";
                sConnector.visible = connector.visible || true;
                sConnector.lineDashArray = connector.lineDashArray || null;
                //sConnector.headPort = connector.headPort || null;
                //sConnector.tailPort = connector.tailPort || null;
                sConnector.sourceDecorator = connector.sourceDecorator ? connector.sourceDecorator : null;
                sConnector.targetDecorator = connector.targetDecorator ? connector.targetDecorator : null;
                sConnector.line = connector.line || {};
                sConnector.lineColor = connector.lineColor || "black";
                sConnector.navigateUrl = connector.navigateUrl || "";
                sConnector.lineWidth = connector.lineWidth || 1;
                sConnector.lineDashArray = connector.lineDashArray || null;
                sConnector.constraints = connector.constraints || ej.Diagram.ConnectorConstraints.Default;
                sConnector.parent = connector.parent ? connector.parent.fullName() : null;
                sConnector.opacity = connector.opacity || 1;
                sConnector.labels = connector.labels || {};
                sConnector.offsetX = connector.offsetX || 0;
                sConnector.offsetY = connector.offsetY || 0;
                sConnector.tag = connector.tag || null;
                sConnector.sourceNodeName = connector.sourceNode ? connector.sourceNode.name : null;
                sConnector.targetNodeName = connector.targetNode ? connector.targetNode.name : null;
                sConnector.sourcePortName = connector.sourcePort ? connector.sourcePort.name : null;
                sConnector.targetPortName = connector.targetPort ? connector.targetPort.name : null;
                sConnector.zOrder = connector.zOrder || 0;
                sConnector.lineHitPadding = connector.lineHitPadding || null;
                sConnector.addInfo = connector.addInfo || null;
            }
            return sConnector;
        };
        Serializer.prototype._createJson = function (obj) {
            return JSON.parse(JSON.stringify(this.serializeObj));
        };
        Serializer.prototype._createNode = function (node) {
            var sNode = new Object();
            sNode.canUngroup = node.canUngroup;
            if (node.children && node.children.length > 0) {
                var c, children = [];
                for (c = 0; c < node.children.length; c++) {
                    if (node.children[c].type == "connector")
                        children.push(this._createConnector(node.children[c]));
                    else
                        children.push(this._createNode(node.children[c]));
                }
                sNode.children = children;
            }
            sNode.borderColor = node.borderColor ? node.borderColor : null;
            sNode.borderDashArray = node.borderDashArray ? node.borderDashArray : null;
            sNode.borderWidth = node.borderWidth;
            sNode.fillColor = node.fillColor ? node.fillColor : null;
            sNode.gradient = node.gradient ? node.gradient : null;
            sNode.opacity = node.opacity ? node.opacity : null;
            sNode.exclude = node.exclude ? node.exclude : null;
            sNode.shape = node.shape ? node.shape : null;
            sNode.constraints = node.constraints;
            sNode.enableCentralPort = node.enableCentralPort;
            sNode.height = node.height;
            sNode.labels = node.labels;
            sNode.name = node.fullName();
            sNode.inEdges = null;
            sNode.outEdges = null;
            sNode.navigateUrl = node.navigateUrl;
            sNode.parent = node.parent ? node.parent.fullName() : null;
            sNode.offsetX = node.offsetX;
            sNode.offsetY = node.offsetY;
            sNode.ports = node.ports;
            sNode.tag = node.tag;
            sNode.visible = node.visible;
            sNode.width = node.width;
            sNode.zOrder = node.zOrder;
            sNode.rotateAngle = node.rotateAngle;
            sNode.pivot = node.pivot;
            if (node.addInfo && node.addInfo.isSerialize)
                sNode.addInfo = node.addInfo;
            return sNode;
        };
        Serializer.prototype._createPageSetting = function (obj) {
            var page = new Object();
            if (obj.pageSettings) {
                var objPage = obj.pageSettings;
                page.pageWidth = objPage.pageWidth;
                page.pageHeight = objPage.pageHeight;
                page.multiplePage = objPage.multiplePage;
                page.pageborderWidth = objPage.pageborderWidth;
                page.pagebackgroundColor = objPage.pagebackgroundColor;
                page.pageborderColor = objPage.pageborderColor;
                page.pageMargin = objPage.pageMargin;
                page.showPageBreaks = objPage.showPageBreaks;
                page.pageOrientation = objPage.pageOrientation;
            }
            this.serializeObj.pageSettings = page;
        };
        Serializer.prototype._createSnapSetting = function (obj) {
            var sSettings = new Object();
            var objSnap = obj.snapSettings;
            if (obj.snapSettings) {
                sSettings.horizontalGridlines = objSnap.horizontalGridlines;
                sSettings.snapConstraints = objSnap.snapConstraints;
                sSettings.snapAngle = objSnap.snapAngle;
                sSettings.snapToObject = objSnap.snapToObject;
                sSettings.verticalGridlines = objSnap.verticalGridlines;
            }
            this.serializeObj.snapSettings = sSettings;
        };
        return Serializer;
    })();
    //#endregion 

    //#region Deserializer
    ej.Diagram.Deserializer = (function () {

        function Deserializer() {
            this.deSerializeObj = new Object();
        }
        Deserializer.prototype.deSerialize = function (jsonObj, diagram) {
            if (jsonObj != null) {
                this._load(jsonObj, diagram);
            }
        };
        Deserializer.prototype._load = function (jsonObj, diagram) {

            this.deSerializeObj = jsonObj;
            this._createCommonProperties(this.deSerializeObj, diagram.model);
            this._createCanvasProperties(this.deSerializeObj, diagram.model);
            this._createNodes(this.deSerializeObj, diagram.model);
            this._createConnectors(this.deSerializeObj, diagram.model);
            this._createPageSetting(this.deSerializeObj, diagram.model);
            this._createSnapSetting(this.deSerializeObj, diagram.model);
            this._setConnectorsPort(diagram.model, diagram);
            diagram._init();
        };

        Deserializer.prototype._createCommonProperties = function (obj, model) {
            model.autoScroll = obj.autoScroll ? obj.autoScroll : null;
            model.autoScrollMargin = obj.autoScrollMargin ? obj.autoScrollMargin : null;
            model.borderColor = obj.borderColor ? obj.borderColor : "#DADADA";
            model.borderWidth = obj.borderWidth ? obj.borderWidth : 1;
            model.contextMenu = obj.contextMenu;
            model.enableVisualGuide = obj.enableVisualGuide ? obj.enableVisualGuide : false;
            model.height = obj.height ? obj.height : "100%";
            model.layout = obj.layout;
            model.magnification = obj.magnification;
            model.snapToGrid = obj.snapToGrid;
            model.visualGuide = obj.visualGuide;
            model.width = obj.width ? obj.width : "100%";
        };
        Deserializer.prototype._createCanvasProperties = function (obj, model) {
            var objCanvas = obj.canvas;
            //if (obj.canvas) {
            //    if (!model.canvas)
            //        model.canvas = new ej.Diagram.Canvas(objCanvas);
            //}
        };
        Deserializer.prototype._createNodes = function (obj, model) {
            if (!model.nodes)
                model.nodes = new ej.Diagram.Collection();
            model.nodes.clear();
            var i;
            if (obj.nodes && obj.nodes.length > 0) {
                for (i = 0; i < obj.nodes.length; i++) {
                    model.nodes.add(this._createNode(obj.nodes[i], model));
                }
            }
        };
        Deserializer.prototype._createConnectors = function (obj, model) {
            if (!model.connectors)
                model.connectors = new ej.Diagram.Collection();
            model.connectors.clear();
            var i;
            if (obj.connectors && obj.connectors.length > 0) {
                for (i = 0; i < obj.connectors.length; i++) {
                    model.connectors.add(this._createConnector(obj.connectors[i], model));
                }
            }
        };
        Deserializer.prototype._createConnector = function (obj, model) {
            var sConnector = new ej.Diagram.Connector(obj);
            return sConnector;
        };
        Deserializer.prototype._createNode = function (obj, model) {
            var sNode = null, childNode;
            if (obj.children && obj.children.length > 0) {
                sNode = new ej.Diagram.Group(obj);
                var k;
                for (k = 0; k < obj.children.length; k++) {
                    if (obj.children[k].line) {
                        childNode = this._createConnector(obj.children[k]);
                        sNode.children.add(childNode);
                    } else {
                        childNode = this._createNode(obj.children[k]);
                        sNode.children.add(childNode);
                    }
                }
            } else {
                sNode = new ej.Diagram.Node(obj);
            }
            if (!sNode.inEdges)
                sNode.inEdges = new ej.Diagram.Collection();
            if (!sNode.outEdges)
                sNode.outEdges = new ej.Diagram.Collection();

            return sNode;
        };
        Deserializer.prototype._setConnectorsPort = function (model, diagram) {
            if (model.connectors && model.connectors.length > 0) {
                for (var i = 0, len = model.connectors.length; i < len; i++) {
                    var connector = model.connectors[i];
                    if (connector.sourceNodeName != null && connector.sourceNodeName != "")
                        connector.sourceNode = diagram._findNode(connector.sourceNodeName);
                    if (connector.sourcePortName != null && connector.sourcePortName != "")
                        connector.sourcePort = diagram._findPort(connector.sourceNode, connector.sourcePortName);
                    if (connector.targetNodeName != null && connector.targetNodeName != "")
                        connector.targetNode = diagram._findNode(connector.targetNodeName);
                    if (connector.targetPortName != null && connector.targetPortName != "")
                        connector.targetPort = diagram._findPort(connector.targetNode, connector.targetPortName);
                }
            }
        };
        Deserializer.prototype._createPageSetting = function (obj, model) {
            var objpage = obj.pageSettings;
            if (obj.pageSettings) {
                if (!model.pageSettings)
                    model.pageSettings = new ej.Diagram.PageSettings(objpage);
            }

        };
        Deserializer.prototype._createSnapSetting = function (obj, model) {
            var objSnap = obj.snapSettings;
            if (obj.snapSettings) {
                if (!model.snapSettings)
                    model.snapSettings = new ej.Diagram.SnapSettings(objSnap);
            }
        };
        return Deserializer;
    })();
    //#endregion 

})(jQuery, Syncfusion);
;

(function ($, ej) {
    "use strict";

    //#region SVG
    var Svg = (function () {
        function Svg(attr) {
            this.xmlns = "http://www.w3.org/2000/svg";
            this.document = document.createElementNS(this.xmlns, "svg");
            this.id = attr.id;
            if (attr) {
                ej.Diagram.Util.attr(this.document, attr);
            }
            return this;
        }
        Svg.prototype.element = function (attr, tag) {
            var element;
            if (attr && attr.id) {
                element = this.document.getElementById(attr.id);
            }
            if (!element) {
                element = document.createElementNS(this.xmlns, tag);
            }
            if (attr)
                ej.Diagram.Util.attr(element, attr);
            return element;
        };
        Svg.prototype.g = function (attr) {
            return this.element(attr, "g");
        };
        Svg.prototype.defs = function (attr) {
            return this.element(attr, "defs");
        };
        Svg.prototype.image = function (attr) {
            return this.element(attr, "image");
        };
        Svg.prototype.path = function (attr) {
            return this.element(attr, "path");
        };
        Svg.prototype.text = function (attr) {
            return this.element(attr, "text");
        };
        Svg.prototype.textPath = function (attr) {
            return this.element(attr, "textPath");
        };
        Svg.prototype.tspan = function (attr) {
            return this.element(attr, "tspan");
        };
        Svg.prototype.rect = function (attr) {
            return this.element(attr, "rect");
        };
        Svg.prototype.ellipse = function (attr) {
            return this.element(attr, "ellipse");
        };
        Svg.prototype.circle = function (attr) {
            return this.element(attr, "circle");
        };
        Svg.prototype.line = function (attr) {
            return this.element(attr, "line");
        };
        Svg.prototype.polyline = function (attr) {
            return this.element(attr, "polyline");
        };
        Svg.prototype.polygon = function (attr) {
            return this.element(attr, "polygon");
        };
        Svg.prototype.foreignObject = function (attr) {
            return this.element(attr, "foreignObject");
        };
        Svg.prototype.linearGradient = function (attr) {
            return this.element(attr, "linearGradient");
        };
        Svg.prototype.radialGradient = function (attr) {
            return this.element(attr, "radialGradient");
        };
        Svg.prototype.stop = function (attr) {
            return this.element(attr, "stop");
        };
        Svg.prototype.pattern = function (attr) {
            return this.element(attr, "pattern");
        };
        Svg.prototype.appendChild = function (element) {
            this.document.appendChild(element);
        };
        Svg.prototype.removeChild = function (element, parent) {
            if (parent) {
                parent.removeChild(element);
            } else {
                this.document.removeChild(element);
            }
        };
        Svg.prototype.getElementById = function (id) {
            return this.document.getElementById(id);
        };
        Svg.prototype.getElementsByClassName = function (name) {
            return this.document.getElementsByClassName(name);
        };
        Svg.prototype.getElementsByTagName = function (name) {
            return this.document.getElementsByTagName(name);
        };
        Svg.prototype.pathBounds = function (data) {
            var attributes = { d: data };
            var path = this.path(attributes);
            this.document.appendChild(path);
            var bounds = path.getBBox();
            this.document.removeChild(path);
            return bounds;
        };
        Svg.prototype.textBounds = function (text, attr, textElement) {
            var bounds = null;
            if (text && attr) {
                var svgText = this.text(attr);
                svgText.appendChild(document.createTextNode(text));
                this.document.appendChild(svgText);
                bounds = svgText.getBBox();
                this.document.removeChild(svgText);
            }
            else if (textElement) {
                this.document.appendChild(textElement);
                bounds = textElement.getBBox();
                this.document.removeChild(textElement);
            }
            return bounds;
        };
        Svg.prototype.absolutePath = function (path) {
            var x0, y0, x1, y1, x2, y2, segs = path.pathSegList;
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if (/[MLHVCSQTA]/.test(char)) {
                    if ('x' in seg) x = seg.x;
                    if ('y' in seg) y = seg.y;
                } else {
                    if ('x1' in seg) x1 = x + seg.x1;
                    if ('x2' in seg) x2 = x + seg.x2;
                    if ('y1' in seg) y1 = y + seg.y1;
                    if ('y2' in seg) y2 = y + seg.y2;
                    if ('x' in seg) x += seg.x;
                    if ('y' in seg) y += seg.y;
                    switch (char) {
                        case 'm': segs.replaceItem(path.createSVGPathSegMovetoAbs(x, y), i); break;
                        case 'l': segs.replaceItem(path.createSVGPathSegLinetoAbs(x, y), i); break;
                        case 'h': segs.replaceItem(path.createSVGPathSegLinetoHorizontalAbs(x), i); break;
                        case 'v': segs.replaceItem(path.createSVGPathSegLinetoVerticalAbs(y), i); break;
                        case 'c': segs.replaceItem(path.createSVGPathSegCurvetoCubicAbs(x, y, x1, y1, x2, y2), i); break;
                        case 's': segs.replaceItem(path.createSVGPathSegCurvetoCubicSmoothAbs(x, y, x2, y2), i); break;
                        case 'q': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticAbs(x, y, x1, y1), i); break;
                        case 't': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticSmoothAbs(x, y), i); break;
                        case 'a': segs.replaceItem(path.createSVGPathSegArcAbs(x, y, seg.r1, seg.r2, seg.angle, seg.largeArcFlag, seg.sweepFlag), i); break;
                        case 'z': case 'Z': x = x0; y = y0; break;
                    }
                }
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }
            return path;
        };
        return Svg;
    })();

    ej.Diagram.Svg = Svg;
    //#endregion

    //#region Renderer
    ej.Diagram.SvgContext = {
        //#region Transformation

        updateTransformation: function (diagram, hOffset, vOffset) {
            var pageLayer = diagram._view;
            var attr = {
                "transform": "translate(" + hOffset + "," + vOffset + ")"
            };
            ej.Diagram.Util.attr(pageLayer, attr);
            if (ej.browserInfo().name === "msie") {
                if (diagram._htmlLayer && diagram._htmlLayer.children.length > 0) {
                    diagram._htmlLayer.style.left = hOffset + "px";
                    diagram._htmlLayer.style.top = vOffset + "px";
                    diagram._htmlLayer.style.position = "absolute";
                }
            }
        },

        updateNegativeLayer: function (diagram, x, y) {
            var mainLayer = diagram._page;
            var attr = {
                "transform": "translate(" + x + "," + y + ")"
            };
            ej.Diagram.Util.attr(mainLayer, attr);
        },

        updateScaling: function (diagram, scale) {
            var diagramLayer = diagram._diagramLayer;
            var attr = {
                "transform": "scale(" + scale + ")"
            };
            ej.Diagram.Util.attr(diagramLayer, attr);
            if (ej.browserInfo().name === "msie") {
                if (diagram._htmlLayer && diagram._htmlLayer.children.length > 0) {
                    diagram._htmlLayer.style.transform = "scale(" + scale + ")";
                }
            }
            if (diagram.selectionList.length > 0)
                this.updateSelector(diagram.selectionList[0], diagram._svg, scale, diagram, diagram.model.selectorConstraints);
            if (diagram.selectionList[0]) {
                if (!(diagram.selectionList[0] instanceof ej.Diagram.Connector) && (diagram.model.userHandles != null && diagram.model.userHandles.length > 0)) {
                    var isMultipleSelection = false;
                    if (diagram.selectionList[0].name == "multipleSelection")
                        isMultipleSelection = true;
                    if (diagram.model.selectorConstraints & ej.Diagram.SelectorConstraints.UserHandles)
                        ej.Diagram.SvgContext.updateUserHandles(diagram.model.userHandles, diagram.selectionList[0], diagram._svg, isMultipleSelection, false, scale);
                }
            }
        },

        updateSize: function (svg, width, height) {
            var attr = {
                width: width + "px",
                height: height + "px",
            };
            ej.Diagram.Util.attr(svg.document, attr);
        },

        _createScroller: function (canvas) {
            var scroller = document.createElement("div");
            scroller.setAttribute("id", canvas.id + "_scroller");
            var contentHolder = document.createElement("div");
            contentHolder.setAttribute("id", canvas.id + "_contentHolder");
            scroller.appendChild(contentHolder);
            var content = document.createElement("div");
            content.setAttribute("id", contentHolder.id + "_content");
            contentHolder.appendChild(content);
            canvas.appendChild(scroller);
        },

        _findIndex: function (diagram, node) {
            var lastNode, newIndex = 1, index, elements;
            if (node.parent)
                elements = $("#" + node.parent.name).find(">.node,>.connector,>.group");
            else
                elements = $("#DiagramContent_canvas_svg").find(">.node,>.connector,>.group");
            if (node.name != "helper") {
                lastNode = elements[elements.length - 1];
                if (lastNode && lastNode.id == "helper")
                    lastNode = elements[elements.length - 2];
                if (lastNode) {
                    index = 0;
                    if (lastNode.className.animVal == "ej-d-node" || lastNode.className.animVal == "ej-d-group") {
                        newIndex = index + 1;
                    }
                    else if (lastNode.className.animVal == "ej-d-connector") {
                        newIndex = index + 1;
                    }
                }
            }
            return newIndex;
        },
        //#endregion 

        //#region Render Node

        //#region render
        renderNode: function (node, svg, parent) {
            var g;
            // if (node.visible) 
            {
                var visible = node.visible ? "visible" : "hidden";
                var x = node.offsetX - node.width * node.pivot.x;
                var y = node.offsetY - node.height * node.pivot.y;
                g = svg.g({
                    "id": node.fullName(), "class": "ej-d-node", "transform": "rotate(" + node.rotateAngle + "," + node.offsetX + "," + node.offsetY +
                        "),translate(" + x + "," + y + ")", "visibility": visible
                });
                var backgroundrect = svg.rect({ "id": node.fullName() + "_rect", "fill": "transparent", "width": node.width, "height": node.height });
                g.appendChild(backgroundrect);
                if (parent) {
                    parent.appendChild(g);
                }
                else {
                    svg.appendChild(g);
                }
                this._renderNode(node, svg, g);
            }
            return g;
        },

        _renderNode: function (node, svg, g) {

            this._renderShape(node, svg, g);
            this._renderLabels(node, svg, g);
            this._renderPorts(node, svg, g);
        },

        renderGroup: function (group, svg, parent) {
            var g;
            var visible = group.visible ? "visible" : "hidden";
            //if (group.visible) 
            {
                g = svg.g({ "id": group.fullName(), "class": "ej-d-group", "visibility": visible });
                if (parent) {
                    parent.appendChild(g);
                }
                else {
                    svg.appendChild(g);
                }
                var children = group.children;
                var bounds = group._childrenBounds();
                for (var i = 0, len = children.length; i < len; i++) {
                    var child = children[i];
                    if (child.type === "node")
                        this.renderNode(child, svg, g);
                    else if (child.type === "connector")
                        this.renderConnector(child, svg, g);
                    else if (child.type === "group")
                        this.renderGroup(child, svg, g);
                }
                this._renderLabels(group, svg, g);
                this._renderPorts(group, svg, g);
            }
            return g;
        },

        _renderGradient: function (name, gradient, svg) {
            var fill;
            var id;
            if (gradient.type === "linear") {
                id = this._renderLinearGradient(name, gradient, svg);
                fill = "url(#" + id + ")";
            }
            else if (gradient.type === "radial") {
                id = this._renderRadialGradient(name, gradient, svg);
                fill = "url(#" + id + ")";
            }
            return fill;
        },

        _renderLinearGradient: function (name, gradient, svg) {
            var defs = svg.getElementsByTagName("defs")[0];
            if (defs) {
                var attr = { "id": name + "_gradient", "x1": gradient.x1 + "%", "y1": gradient.y1 + "%", "x2": gradient.x2 + "%", "y2": gradient.y2 + "%" };
                var linear = svg.linearGradient(attr);
                this._renderStops(gradient, svg, linear);
                defs.appendChild(linear);
                return linear.id;
            }
            return null;
        },

        _renderRadialGradient: function (name, gradient, svg) {
            var defs = svg.getElementsByTagName("defs")[0];
            if (defs) {
                var attr = {
                    "id": name + "_gradient", "cx": gradient.cx + "%", "cy": gradient.cy + "%",
                    "fx": gradient.fx + "%", "fy": gradient.fy + "%", "r": gradient.r + "%"
                };
                var radial = svg.radialGradient(attr);
                this._renderStops(gradient, svg, radial);
                defs.appendChild(radial);
                return radial.id;
            }
            return null;
        },

        _renderStops: function (gradient, svg, element) {
            if (svg.getElementById(element.getAttribute("id")) === element) {
                while (element.firstChild) {
                    element.removeChild(element.firstChild);
                }
            }
            var stops = gradient.stops;
            var stop;
            for (var i = 0, len = stops.length; i < len; i++) {
                stop = stops[i];
                element.appendChild(svg.stop({ "offset": stop.offset + "%", "stop-color": stop.color, "stop-opacity": stop.opacity }));
            }
        },

        _renderShape: function (node, svg, g) {
            switch (node.shape.type) {
                case "rectangle":
                    this._renderRect(node, svg, g);
                    break;
                case "ellipse":
                    this._renderEllipse(node, svg, g);
                    break;
                case "image":
                    this._renderImage(node, svg, g);
                    break;
                case "path":
                    this._renderPath(node, svg, g);
                    break;
                case "polygon":
                    this._renderPolygon(node, svg, g);
                    break;
                case "text":
                    this._renderTextElement(node, svg, g);
                    break;
                case "html":
                    this._renderHtmlElement(node, svg, g);
                    break;
                case "native":
                    this._rendercontent(node, svg, g);
                    break;
            }
        },

        _rendercontent: function (node, svg, g) {
            if (node.shape.content) {
                if (typeof (node.shape.content) == 'string') {
                    var content = document.getElementById(node.shape.content);
                    node.shape.content = content.cloneNode(true);
                }
                node.shape.content.setAttribute("preserveAspectRatio", "none");
                g.appendChild(node.shape.content);
                var bounds = node.shape.content.getBBox();
                var delwidth = bounds.width ? node.width / bounds.width : 1;
                var delheight = bounds.height ? node.height / bounds.height : 1;
                node.shape.content.setAttribute("transform", "scale(" + delwidth + "," + delheight + ")");
                node.shape.content.setAttribute("id", node.fullName() + "_shape");
            }
        },

        _fill: function (node, svg) {
            var fill;
            if (node.gradient) {
                fill = this._renderGradient(node.fullName(), node.gradient, svg);
            }
            if (!fill) {
                fill = node.fillColor;
            }
            return fill;
        },

        _renderRect: function (node, svg, g) {
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.fullName() + "_shape", "rx": node.shape.cornerRadius, "ry": node.shape.cornerRadius,
                "width": node.width, "height": node.height, "fill": fill, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "opacity": node.opacity, "stroke-dasharray": node.borderDashArray
            };
            g.appendChild(svg.rect(attr));
        },

        _renderEllipse: function (node, svg, g) {
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.fullName() + "_shape", "rx": node.width / 2, "ry": node.height / 2,
                "cx": node.width / 2, "cy": node.height / 2,
                "fill": fill, "stroke": node.borderColor, "stroke-width": node.borderWidth,
                "opacity": node.opacity, "stroke-dasharray": node.borderDashArray

            };
            g.appendChild(svg.ellipse(attr));
        },

        _renderImage: function (node, svg, g) {
            var backRect = this._renderBackgroundRect(node, svg);
            g.appendChild(backRect);
            var attr = {
                "id": node.fullName() + "_shape", "width": node.width, "height": node.height,
                "opacity": node.opacity, "preserveAspectRatio": "none"
            };
            var shape = svg.image(attr);
            shape.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", node.shape.src);
            g.appendChild(shape);
        },

        _renderPolygon: function (node, svg, g) {
            var fill = this._fill(node, svg);
            this._updatePolygonPoints(node);
            var points = this._convertToSVGPoints(node.shape.points);
            var attr = {
                "id": node.fullName() + "_shape", "fill": fill, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray, "points": points
            };
            g.appendChild(svg.polygon(attr));
        },

        _renderPath: function (node, svg, g) {
            var x = node.offsetX - node.width * node.pivot.x;
            var y = node.offsetY - node.height * node.pivot.y;
            var fill = this._fill(node, svg);

            var d;
            d = node.updateShape();
            if (!d) {
                d = this._updatePath(0, 0, node.width, node.height, node.shape.pathData, svg);
            }
            var attr = {
                "id": node.fullName() + "_shape", "width": node.width,
                "height": node.height, "d": d, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "fill": fill

            };
            g.appendChild(svg.path(attr));
        },

        _renderHtmlElement: function (node, svg, g) {
            var backRect = this._renderBackgroundRect(node, svg);
            g.appendChild(backRect);
            if (node.shape.html) {
                var x = node.offsetX - node.width * node.pivot.x;
                var y = node.offsetY - node.height * node.pivot.y;
                var div = document.createElement("div");
                var style = "width:" + node.width + "px;height:" + node.height + "px;padding:1px; opacity:" + node.opacity + ";left:" + x + "px; top:" + y + "px; ";
                if (ej.browserInfo().name === "msie")
                    style += "display: block; position: absolute; border:0px;";
                var attr = { "id": node.fullName() + "_html", "style": style };
                ej.Diagram.Util.attr(div, attr);
                if (node.shape.html instanceof HTMLElement) {
                    div.appendChild(node.shape.html);
                }
                else
                    div.innerHTML = node.shape.html.toString();
                if (ej.browserInfo().name === "msie") {
                    var parentdiv = document.createElement("div");
                    var attr1 = { "id": node.fullName() + "_parentdiv", "class": "ej-d-node" };
                    ej.Diagram.Util.attr(parentdiv, attr1);
                    parentdiv.appendChild(div);
                    var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                    if (htmlLayer)
                        htmlLayer.appendChild(parentdiv);
                    else
                        svg.document.parentNode.appendChild(div);
                }
                else {
                    attr = { "id": node.fullName() + "_shape", "width": node.width, "height": node.height, "style": "pointer-events:none" };
                    var shape = svg.foreignObject(attr);
                    shape.appendChild(div);
                    g.appendChild(shape);
                }
            }
        },

        _initializeGuidelines: function (svg, attr, parent) {
            var g = svg.g(attr);
            parent.appendChild(g);
            return g;
        },

        _renderSideAlignmentLines: function (start, end, g, svg, scale) {
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 1
            });
            g.appendChild(line);
        },

        _renderCenterAlignmentLines: function (start, end, g, svg, scale) {
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 1,
            });
            g.appendChild(line);
        },

        _renderSpacingLines: function (start, end, g, svg, scale) {
            var d, d1;
            var div1 = svg.g({});
            if (start.x == end.x) {
                d = "M" + (start.x * scale - 5) + " " + (start.y * scale + 5) + " L" + start.x * scale + " " + start.y * scale + " L" + (start.x * scale + 5) + " " + (start.y * scale + 5) + "z";
                d1 = "M" + (end.x * scale - 5) + " " + (end.y * scale - 5) + " L" + end.x * scale + " " + end.y * scale + " L" + (end.x * scale + 5) + " " + (end.y * scale - 5) + "z";
                div1.appendChild(svg.line({ "x1": start.x * scale - 8, "x2": start.x * scale + 8, "y1": start.y * scale - 1, "y2": start.y * scale - 1, "stroke": "#07EDE1", "stroke-width": 1 }));
                div1.appendChild(svg.line({ "x1": end.x * scale - 8, "x2": end.x * scale + 8, "y1": end.y * scale + 1, "y2": end.y * scale + 1, "stroke": "#07EDE1", "stroke-width": 1 }));
            }
            else {
                d = "M" + (start.x * scale + 5) + " " + (start.y * scale + 5) + " L" + start.x * scale + " " + start.y * scale + " L" + (start.x * scale + 5) + " " + (start.y * scale - 5) + "z";
                d1 = "M" + (end.x * scale - 5) + " " + (end.y * scale - 5) + " L" + end.x * scale + " " + end.y * scale + " L" + (end.x * scale - 5) + " " + (end.y * scale + 5) + "z";
                div1.appendChild(svg.line({ "x1": start.x * scale - 1, "x2": start.x * scale - 1, "y1": start.y * scale - 8, "y2": start.y * scale + 8, "stroke": "#07EDE1", "stroke-width": 1 }));
                div1.appendChild(svg.line({ "x1": end.x * scale + 1, "x2": end.x * scale + 1, "y1": end.y * scale - 8, "y2": end.y * scale + 8, "stroke": "#07EDE1", "stroke-width": 1 }));
            }
            div1.appendChild(svg.path({ "d": d, "fill": "#07EDE1" }));
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 0.6, "fill": "#07EDE1"
            });
            div1.appendChild(line);
            div1.appendChild(svg.path({ "d": d1, "fill": "#07EDE1" }));
            g.appendChild(div1);
        },

        _removeGuidelines: function (parent, g) {
            if (g != null) {
                parent.removeChild(g);
            }
        },

        //#endregion

        //#region update
        updateGroup: function (group, svg) {
            var children = group.children;
            var visible = group.visible ? "visible" : "hidden";
            svg.g({ "id": group.fullName(), "visibility": visible });
            for (var i = 0, len = children.length; i < len; i++) {
                var child = children[i];
                if (child.type === "node")
                    this.updateNode(child, svg);
                else if (child.type === "connector")
                    this.updateConnector(child, svg);
                else if (child.type === "group")
                    this.updateGroup(child, svg);
            }
            this._updateAssociatedConnector(group, svg);
            if (!group._diagram || group._diagram.activeTool.name != "move" && group._diagram.activeTool.name != "rotate") {
                this._updateLabels(group, svg);
                this._updatePorts(group, svg);
            }
        },

        _updateAssociatedConnector: function (node, svg) {
            var i;
            var len;
            if (node.inEdges.length > 0) {
                for (i = 0, len = node.inEdges.length; i < len; i++) {
                    this.updateConnector(node.inEdges[i], svg);
                }
            }
            if (node.outEdges.length > 0) {
                for (i = 0, len = node.outEdges.length; i < len; i++) {
                    this.updateConnector(node.outEdges[i], svg);
                }
            }
        },

        updateLabelStyle: function (node, label, svg) {
            var attr = {};
            if (node && svg && label) {
                if (label.bold)
                    attr["font-weight"] = "bold";
                else
                    attr["font-weight"] = "";

                if (label.italic)
                    attr["font-style"] = "italic";
                else
                    attr["font-style"] = "";

                if (label.textDecoration)
                    attr["text-decoration"] = label.textDecoration;
                attr["font-family"] = label.fontFamily;
                attr["font-size"] = label.fontSize;
                attr["fill"] = label.fontColor;
            }
            ej.Diagram.Util.attr(svg.getElementById(node.fullName() + "_" + label.name), attr);
        },

        updateNodeStyle: function (node, svg) {
            if (node && svg) {
                var attr = {};
                attr["fill"] = node.fillColor;
                attr["stroke"] = node.borderColor;
                attr["stroke-width"] = node.borderWidth;
                attr["stroke-dasharray"] = node.lineDashArray;
                if (node.gradient)
                    attr["fill"] = this._renderGradient(node.fullName(), node.gradient, svg);
                if (ej.browserInfo().name === "msie" && node.shape && node.shape.type == "html") {
                    //ej.Diagram.Util.attr(svg.getElementById(node.fullName() + "_html"), attr);
                }
                else
                    ej.Diagram.Util.attr(svg.getElementById(node.fullName() + "_shape"), attr);
            }
        },

        updateConnectorStyle: function (connector, svg) {
            if (connector && svg) {
                var attr = {};
                attr["stroke"] = connector.lineColor;
                attr["stroke-width"] = connector.lineWidth;
                attr["stroke-dasharray"] = connector.lineDashArray;
                ej.Diagram.Util.attr(svg.getElementById(connector.fullName()).firstChild, attr);
            }
        },

        updateTargetDecoratorStyle: function (connector, svg) {
            if (svg) {
                var attr = {};
                attr["stroke"] = connector.targetDecorator.borderColor;
                attr["fill"] = connector.targetDecorator.fillColor;
                ej.Diagram.Util.attr(svg.getElementById(connector.fullName() + "_targetDecorator"), attr);
            }
        },

        updateSourceDecoratorStyle: function (connector, svg) {
            if (svg) {
                var attr = {};
                attr["stroke"] = connector.sourceDecorator.borderColor;
                attr["fill"] = connector.sourceDecorator.fillColor;
                ej.Diagram.Util.attr(svg.getElementById(connector.fullName() + "_sourceDecorator"), attr);
            }
        },

        updateNode: function (node, svg) {
            var g, bounds;
            if (node.shape.type === "html" && ej.browserInfo().name === "msie") {
                g = document.getElementById(node.fullName() + "_html").getBoundingClientRect();
                bounds = { x: g.left, y: g.top, width: g.width, height: g.height };
            }
            else {
                g = svg.getElementById(node.fullName() + "_shape");
                bounds = g.getBBox();
            }
            var backgroundrect = svg.rect({ "id": node.fullName() + "_rect", "fill": "transparent", "width": node.width, "height": node.height });
            this._updateNode(node, svg);
            this._updateAssociatedConnector(node, svg);
            if (!node._diagram || node._diagram.activeTool.name != "move" && node._diagram.activeTool.name != "rotate") {
                this._updateLabels(node, svg);
                if (bounds.width != node.width || bounds.height != node.height) {
                    this._updatePorts(node, svg);
                }
            }
        },

        updateNodes: function (nodes, svg) {
            if (nodes) {
                for (var i = 0; i < nodes.length; i++) {
                    var node = nodes[i];
                    node._updateAssociatedConnectorEnds();
                    if (node.type === "group") {
                        var children = node.children;
                        var visible = node.visible ? "visible" : "hidden";
                        svg.g({ "id": node.fullName(), "visibility": visible });
                        for (var i = 0, len = children.length; i < len; i++) {
                            var child = children[i];
                            if (child.type === "node")
                                this.updateNode(child, svg);
                            else if (child.type === "connector")
                                this.updateConnector(child, svg);
                            else if (child.type === "group")
                                this.updateGroup(child, svg);
                        }
                        this._updateAssociatedConnector(node, svg);
                        this._updatePorts(node, svg);
                    }
                    else if (node.type === "node") {
                        var g, bounds;
                        if (node.shape.type === "html" && ej.browserInfo().name === "msie") {
                            g = document.getElementById(node.fullName() + "_html").getBoundingClientRect();
                            bounds = { x: g.left, y: g.top, width: g.width, height: g.height };
                        }
                        else {
                            g = svg.getElementById(node.fullName() + "_shape");
                            bounds = g.getBBox();
                        }
                        var backgroundrect = svg.rect({ "id": node.fullName() + "_rect", "fill": "transparent", "width": node.width, "height": node.height });
                        this._updateNode(node, svg);
                        this._updateAssociatedConnector(node, svg);
                        if (bounds.width != node.width || bounds.height != node.height) {
                            this._updatePorts(node, svg);
                        }
                    }
                }
            }
        },

        _updateNode: function (node, svg) {
            var x = node.offsetX - node.width * node.pivot.x;
            var y = node.offsetY - node.height * node.pivot.y;
            var angle = node.rotateAngle;
            var pt = new ej.Diagram.Point(node.offsetX, node.offsetY);
            var visible = node.visible ? "visible" : "hidden";
            var attr = ({
                "id": node.fullName(), "transform": "rotate(" + angle + "," + pt.x + "," + pt.y + "),translate(" + x + "," + y + ")",
                "visibility": visible
            });
            svg.g(attr);
            switch (node.shape.type) {
                case "rectangle":
                    svg.rect({
                        "id": node.fullName() + "_shape", "width": node.width, "height": node.height,
                    });
                    break;
                case "ellipse":
                    svg.ellipse({
                        "id": node.fullName() + "_shape", "rx": node.width / 2, "ry": node.height / 2,
                        "cx": node.width / 2, "cy": node.height / 2
                    });
                    break;
                case "image":
                    this._updateBackgroundRect(node, svg);
                    svg.image({
                        "id": node.fullName() + "_shape", "width": node.width, "height": node.height,
                    });
                    break;
                case "path":
                    var d;
                    d = node.updateShape();
                    if (!d) {
                        var d = this._updatePath(0, 0, node.width, node.height, node.shape.pathData, svg);
                    }
                    svg.path({
                        "id": node.fullName() + "_shape", "d": d,
                    });
                    break;
                case "polygon":
                    this._updatePolygon(node, svg);
                    break;
                case "text":
                    this._updateTextElement(node, svg);
                    break;
                case "html":
                    this._updateHtmlElement(node, svg);
                    break;
                case "native":
                    var bounds = node.shape.content.getBBox();
                    var delwidth = bounds.width ? node.width / bounds.width : 1;
                    var delheight = bounds.height ? node.height / bounds.height : 1;
                    svg.g({
                        "id": node.fullName() + "_shape", "width": node.width, "height": node.height,
                        "transform": "scale(" + delwidth + "," + delheight + ")"
                    });

                    break;
            }
        },

        _updateHtmlElement: function (node, svg) {
            this._updateBackgroundRect(node, svg);
            var div;
            if (ej.browserInfo().name === "msie") {
                var x = node.offsetX - node.width * node.pivot.x;
                var y = node.offsetY - node.height * node.pivot.y;
                div = document.getElementById(node.fullName() + "_html");
                div.style.left = x + "px";
                div.style.top = y + "px";
                div.style.transform = "rotate(" + node.rotateAngle + "deg)";
            }
            else {
                div = svg.getElementById(node.fullName() + "_shape");
                svg.foreignObject({
                    "id": node.fullName(), "width": node.width, "height": node.height
                });
            }
            div.style.width = node.width.toString() + "px";
            div.style.height = node.height.toString() + "px";
        },

        _updatePolygon: function (node, svg) {
            this._updatePolygonPoints(node);
            svg.polygon({
                "id": node.fullName() + "_shape", "points": this._convertToSVGPoints(node.shape.points)
            });
        },

        _updatePolygonPoints: function (node) {
            if (node.shape.points) {
                var points = node.shape.points;
                if (points.type === "collection") {
                    this._scalePoints(points, node);
                    var bounds = ej.Diagram.Geometry.rect(points);
                    for (var i = 0, len = points.length; i < len; ++i) {
                        var point = ej.Diagram.Geometry.translate(points[i], -bounds.x, -bounds.y);
                        point = ej.Diagram.Geometry.translate(point, 0, 0);
                        points[i] = point;
                    }
                }
            }
        },

        _convertToSVGPoints: function (points) {
            var pts = "";
            for (var i = 0, len = points.length; i < len; i++) {
                pts += points[i].toString() + " ";
            }
            return pts.trim();
        },

        _scalePoints: function (points, node) {
            var bounds = ej.Diagram.Geometry.rect(points);
            if (bounds.width > 0 && bounds.height > 0) {
                var scaleX = node.width / bounds.width;
                var scaleY = node.height / bounds.height;
                for (var i = 0, len = points.length; i < len; ++i) {
                    points[i].x = points[i].x * scaleX;
                    points[i].y = points[i].y * scaleY;
                }
            }
        },

        _renderTextElement: function (node, svg, g) {
            var textElement = node.shape.textBlock;
            if (textElement) {
                var backRect = this._renderBackgroundRect(node, svg);
                g.appendChild(backRect);
                var attr = {
                    "id": node.fullName() + "_shape", "class": "ej-d-label", "font-family": textElement.fontFamily,
                    "font-size": textElement.fontSize, "fill": textElement.fontColor,
                    "text-decoration": textElement.textDecoration, "opacity": node.opacity,
                    "alignment-baseline": "middle"
                };
                if (textElement.bold)
                    attr["font-weight"] = "bold";
                if (textElement.italic)
                    attr["font-style"] = "italic";
                var text = svg.text(attr);
                text.appendChild(document.createTextNode(textElement.text));
                g.appendChild(text);
                this._wrapText(node, text, node.shape.textBlock, svg);
                this._alignTextOnLabel(node.shape.textBlock, node, text, svg);
            }
        },

        _updateTextElement: function (node, svg) {
            this._updateBackgroundRect(node, svg);
            var text = svg.getElementById(node.fullName() + "_shape");
            this._wrapText(node, text, node.shape.textBlock, svg);
            this._alignTextOnLabel(node.shape.textBlock, node, text, svg);
        },

        _renderBackgroundRect: function (node, svg) {
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.fullName() + "_backRect", "class": "backrect", "x": 0, "y": 0,
                "width": node.width, "height": node.height, "fill": fill,
                "stroke": node.borderColor, "stroke-width": node.borderWidth, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray
            };
            return svg.rect(attr);
        },

        _updateBackgroundRect: function (node, svg) {
            var attr = {
                "id": node.fullName() + "_backRect", "x": 0, "y": 0,
                "width": node.width, "height": node.height,
            };
            svg.rect(attr);
        },

        updateTextBlock: function (node, label, svg) {
            var attr = {};
            var label = node.shape.textBlock;
            node.shape.textBlock = label;
            if (node && svg && label) {
                if (label.bold)
                    attr["font-weight"] = "bold";
                else
                    attr["font-weight"] = "";

                if (label.italic)
                    attr["font-style"] = "italic";
                else
                    attr["font-style"] = "";

                if (label.textDecoration)
                    attr["text-decoration"] = label.textDecoration;
                attr["font-family"] = label.fontFamily;
                attr["font-size"] = label.fontSize;
                attr["fill"] = label.fontColor;
            }
            ej.Diagram.Util.attr(svg.getElementById(node.fullName() + "_shape"), attr);
            this._updateTextElement(node, svg);
        },

        _updatePath: function (x, y, width, height, pathData, svg) {
            var bBox;
            var isInit = true;
            bBox = svg.pathBounds(pathData);
            var newX = 0, newY = 0;
            var isResize = false;
            if (x.toString() !== bBox.x || y.toString() !== bBox.y) {
                newX = x - Number(bBox.x);
                newY = y - Number(bBox.y);
            }
            if (width !== bBox.width || height !== bBox.height) {
                newX = width / Number(bBox.width);
                newY = height / Number(bBox.height);
                isResize = true;
            }
            return this._processPathData(pathData, newX, newY, isResize, bBox.x, bBox.y, x, y, isInit, svg);
        },

        _processPathData: function (path, newX, newY, isResizing, oldX, oldY, offsetX, offsetY, isInit, svg) {
            var shape = document.createElementNS('http://www.w3.org/2000/svg', "path");
            shape.setAttribute("d", path);
            if (isInit) {
                path = svg.absolutePath(shape).getAttribute('d');
                shape.setAttribute("d", path);
            }
            var x0, y0, x1, y1, x2, y2, segs = shape.pathSegList;
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;

                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;


                if (isResizing) {
                    if (x != undefined) {
                        x = this._scalePathData(x, newX, oldX, offsetX);
                    }
                    if (y != undefined) {
                        y = this._scalePathData(y, newY, oldY, offsetY);
                    }
                    if (x1 != undefined) {
                        x1 = this._scalePathData(x1, newX, oldX, offsetX);
                    }
                    if (y1 != undefined) {
                        y1 = this._scalePathData(y1, newY, oldY, offsetY);
                    }
                    if (x2 != undefined) {
                        x2 = this._scalePathData(x2, newX, oldX, offsetX);
                    }
                    if (y2 != undefined) {
                        y2 = this._scalePathData(y2, newY, oldY, offsetY);
                    }
                }

                else {
                    if (x != undefined) {
                        x = (x + newX);
                    }
                    if (y != undefined) {
                        y = (y + newY);
                    }
                    if (x1 != undefined) {
                        x1 = (x1 + newX);
                    }
                    if (y1 != undefined) {
                        y1 = (y1 + newY);
                    }
                    if (x2 != undefined) {
                        x2 = (x2 + newX);
                    }
                    if (y2 != undefined) {
                        y2 = (y2 + newY);
                    }
                }

                switch (char) {
                    case 'M':
                        segs.replaceItem(shape.createSVGPathSegMovetoAbs(x, y), i);
                        break;
                    case 'L':
                        segs.replaceItem(shape.createSVGPathSegLinetoAbs(x, y), i);
                        break;
                    case 'H':
                        segs.replaceItem(shape.createSVGPathSegLinetoHorizontalAbs(x), i);
                        break;
                    case 'V':
                        segs.replaceItem(shape.createSVGPathSegLinetoVerticalAbs(y), i);
                        break;
                    case 'C':
                        segs.replaceItem(shape.createSVGPathSegCurvetoCubicAbs(x, y, x1, y1, x2, y2), i);
                        break;
                    case 'S':
                        segs.replaceItem(shape.createSVGPathSegCurvetoCubicSmoothAbs(x, y, x2, y2), i);
                        break;
                    case 'Q':
                        segs.replaceItem(shape.createSVGPathSegCurvetoQuadraticAbs(x, y, x1, y1), i);
                        break;
                    case 'T':
                        segs.replaceItem(shape.createSVGPathSegCurvetoQuadraticSmoothAbs(x, y), i);
                        break;
                    case 'A':
                        var r1 = seg.r1, r2 = seg.r2;
                        if (isResizing) {
                            r1 = (r1 * newX);
                            r2 = (r2 * newY);
                        }
                        segs.replaceItem(shape.createSVGPathSegArcAbs(x, y, r1, r2, seg.angle, seg.largeArcFlag, seg.sweepFlag), i);
                        break;
                    case 'Z':
                        x = x0; y = y0;
                        break;
                }
                // Record the start of a subpath
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }
            var pathData = shape.getAttribute("d");
            return pathData;
        },

        _scalePathData: function (val, scaleFactor, oldOffset, newOffset) {
            if (val !== oldOffset) {
                if (newOffset !== oldOffset) {
                    val = (((val * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)))
                        + (newOffset - Number(oldOffset)));
                }
                else
                    val = ((Number(val) * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)));
            }
            else {
                if (newOffset !== oldOffset) {
                    val = newOffset;
                }
            }
            return val;
        },
        //#endregion

        setNodeShape: function (node, svg, parent) {
            var g;
            if (parent) {
                var parentg = svg.getElementById(parent.fullName());
                g = $(parentg).find("#" + node.fullName())[0];
            }
            else {
                g = svg.getElementById(node.fullName());
            }
            $(g).empty();
            this._renderNode(node, svg, g);
        },

        setLine: function (connector, svg, parent) {
            var g;
            if (parent) {
                var parentg = svg.getElementById(parent.fullName());
                g = $(parentg).find("#" + connector.fullName())[0];
            }
            else {
                g = svg.getElementById(connector.fullName());
            }
            $(g).empty();

            this.renderConnector(connector, svg, g);
        },

        addNodeLabel: function (node, label, svg, parent) {
            var g;
            if (parent) {
                var parentg = svg.getElementById(parent.fullName());
                g = $(parentg).find("#" + node.fullName())[0];
            }
            else {
                g = svg.getElementById(node.fullName());
            }
            this._renderLabel(node, label, svg, g);
        },
        //#endregion

        //#region Render Connector
        renderConnector: function (connector, svg, parent) {
            var g;
            var line;
            var visible = connector.visible ? "visible" : "hidden";
            //if (connector.visible) 
            {
                g = svg.g({ "id": connector.fullName(), "class": "ej-d-connector", "visibility": visible });
                if (parent) {
                    if (parent.id !== g.id) {
                        parent.appendChild(g);
                    }
                }
                else {
                    svg.appendChild(g);
                }
                var attr = {
                    "id": connector.fullName() + "_line", "fill": "none", "stroke": connector.lineColor,
                    "stroke-width": connector.lineWidth, "stroke-dasharray": connector.lineDashArray, "opacity": connector.opacity
                };
                if (connector.line.type != "bezier") {
                    var points = this._clipDecorators(connector);
                    attr["points"] = this._convertToSVGPoints(points);
                    line = svg.polyline(attr);
                    g.appendChild(line);
                    attr = { "id": attr.id + "_hitTest", "stroke-width": connector.lineHitPadding, "points": attr.points, "pointer-events": "stroke", "visibility": "hidden" };
                    line = svg.polyline(attr);
                    g.appendChild(line);
                }
                else {
                    var conn = connector.line;
                    var d = "M" + conn.startPoint.x + " " + conn.startPoint.y + " C" + conn._point1.x + " " + conn._point1.y + " " + conn._point2.x + " " + conn._point2.y + " " + conn.point3.x + " " + conn.point3.y;
                    attr = {
                        "id": connector.fullName() + "_line",
                        "d": d, "fill": "none", "stroke": connector.lineColor,
                        "stroke-width": connector.lineWidth, "stroke-dasharray": connector.lineDashArray, "opacity": connector.opacity
                    };
                    line = svg.path(attr);
                    g.appendChild(line);
                    attr = { "id": attr.id + "_hitTest", "stroke-width": connector.lineHitPadding, "d": d, "pointer-events": "stroke", "visibility": "hidden" };
                    line = svg.path(attr);
                    g.appendChild(line);
                }

                this._renderLabels(connector, svg, g);
                this._renderDecorators(connector, svg, g);
            }
            return g;
        },

        _clipDecorators: function (connector) {
            var start, end;
            var points = ej.Diagram.Util.clone(connector.line.points);
            if (connector.targetDecorator) {
                start = points[points.length - 1];
                end = points[points.length - 2];
                var len = ej.Diagram.Geometry.distance(start, end);
                len = (len == 0) ? 1 : len;
                var width = connector.targetDecorator.width;
                var newPoint = new ej.Diagram.Point();
                newPoint.x = start.x + width * (end.x - start.x) / len;
                newPoint.y = start.y + width * (end.y - start.y) / len;
                points[points.length - 1] = newPoint;
            }
            if (connector.sourceDecorator) {
                start = points[0];
                end = points[1];
                var len = ej.Diagram.Geometry.distance(start, end);
                len = (len == 0) ? 1 : len;
                var width = connector.sourceDecorator.width;
                var newPoint = new ej.Diagram.Point();
                newPoint.x = start.x + width * (end.x - start.x) / len;
                newPoint.y = start.y + width * (end.y - start.y) / len;
                points[0] = newPoint;
            }
            return points;
        },

        _updateConnector: function (connector, svg) {
            if (connector.line.type != "bezier") {
                var points = this._clipDecorators(connector);
                svg.polyline({ "id": connector.fullName() + "_line", "points": this._convertToSVGPoints(points) });
                svg.polyline({ "id": connector.fullName() + "_line_hitTest", "points": this._convertToSVGPoints(points) });
            }
            else {
                var conn = connector.line;
                var d = "M" + conn.startPoint.x + " " + conn.startPoint.y + " C" + conn._point1.x + " " + conn._point1.y + " " + conn._point2.x + " " + conn._point2.y + " " + conn.point3.x + " " + conn.point3.y;
                svg.path({ "id": connector.fullName() + "_line", "d": d });
                svg.path({ "id": connector.fullName() + "_line_hitTest", "d": d });
            }
        },

        updateConnector: function (connector, svg) {
            var visible = connector.visible ? "visible" : "hidden";
            svg.g({ "id": connector.fullName(), "visibility": visible });
            this._updateConnector(connector, svg);
            this._updateLabels(connector, svg);
            this._updateDecorators(connector, svg);
        },

        updateConnectors: function (connectors, svg) {
            for (var i = 0; i < connectors.length; i++) {
                var connector = connectors[i];
                connector.updatebounds();
                var visible = connector.visible ? "visible" : "hidden";
                svg.g({ "id": connector.fullName(), "visibility": visible });
                this._updateConnector(connector, svg);
                this._updateDecorators(connector, svg);
            }
        },

        renderDecorators: function (connector, svg) {
            var g = svg.getElementById(connector.fullName());
            this._renderDecorators(connector, svg, g);
        },
        clearDecorators: function (connector, svg) {
            this._updateConnector(connector, svg);
            var g = svg.getElementById(connector.fullName());
            var dec = svg.getElementById(connector.fullName() + "_targetDecorator");
            if (dec)
                svg.removeChild(dec, g);
            dec = svg.getElementById(connector.fullName() + "_sourceDecorator");
            if (dec)
                svg.removeChild(dec, g);
        },
        //#endregion

        //#region Render Decorator
        _renderDecorators: function (connector, svg, g) {
            var startPoint;
            var endPoint;
            if (connector.targetDecorator && connector.targetDecorator.shape) {
                startPoint = connector.line.points[connector.line.points.length - 2];
                endPoint = connector.line.endPoint;
                if (connector.line.type == "bezier") {
                    startPoint = connector.line._point2;
                }
                g.appendChild(this._renderDecorator(connector.fullName() + "_targetDecorator", endPoint,
                    startPoint, connector.targetDecorator, svg));
            }
            if (connector.sourceDecorator && connector.sourceDecorator.shape) {
                startPoint = connector.line.startPoint;
                endPoint = connector.line.points[1];
                if (connector.line.type == "bezier") {
                    endPoint = connector.line._point1;
                }
                g.appendChild(this._renderDecorator(connector.fullName() + "_sourceDecorator", startPoint,
                    endPoint, connector.sourceDecorator, svg));
            }
        },

        _renderDecorator: function (name, point1, point2, decorator, svg) {
            var shape;
            var attr;
            var d;
            var size = new ej.Diagram.Size(decorator.width, decorator.height);
            switch (decorator.shape) {
                case "arrow":
                    d = this._constructArrow(point1, size);
                    attr = { "id": name, "class": "decorator", "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor };
                    shape = svg.path(attr);
                    break;
                case "openarrow":
                    d = this._constructArrow(point1, size, true);
                    attr = { "id": name, "class": "decorator", "d": d, "fill": "transparent", "stroke": decorator.borderColor };
                    shape = svg.path(attr);
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = {
                        "id": name, "class": "decorator", "rx": rx, "ry": ry, "cx": (point1.x + size.width / 2), "cy": (point1.y),
                        "fill": decorator.fillColor, "stroke": decorator.borderColor
                    };
                    shape = svg.ellipse(attr);
                    break;
                case "diamond":
                    d = this._constructDiamond(point1, size, svg);
                    attr = {
                        "id": name, "class": "decorator", "x": point1.x, "y": point1.y, "width": size.width,
                        "height": size.height, "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor
                    };
                    shape = svg.path(attr);
                    break;
                case "path":
                    d = this._updatePath(point1.x, point1.y - size.height / 2, size.width, size.height, decorator.pathData, svg);
                    attr = {
                        "id": name, "class": "decorator", "x": point1.x, "y": point1.y, "width": size.width,
                        "height": size.height, "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor
                    };
                    shape = svg.path(attr);
                    break;
            }
            //apply translate matrix and rotate matrix here
            shape.setAttribute("transform", "rotate(" + ej.Diagram.Geometry.findAngle(point1, point2) + " " + point1.x + " " + point1.y + ")");
            return shape;
        },

        _updateDecorators: function (connector, svg) {
            var startPoint;
            var endPoint;
            if (connector.targetDecorator && connector.targetDecorator.shape !== ej.Diagram.DecoratorShapes.None) {
                startPoint = connector.line.points[connector.line.points.length - 2];
                endPoint = connector.line.endPoint;
                if (connector.line.type == "bezier") {
                    startPoint = connector.line._point2;
                }
                this._updateDecorator(connector.fullName() + "_targetDecorator", endPoint, startPoint, connector.targetDecorator, svg);
            }
            if (connector.sourceDecorator && connector.sourceDecorator.shape !== ej.Diagram.DecoratorShapes.None) {
                startPoint = connector.line.startPoint;
                endPoint = connector.line.points[1];
                if (connector.line.type == "bezier") {
                    endPoint = connector.line._point1;
                }
                this._updateDecorator(connector.fullName() + "_sourceDecorator", startPoint, endPoint, connector.sourceDecorator, svg);
            }
        },

        _updateDecorator: function (name, point1, point2, decorator, svg) {
            var shape;
            var attr;
            var d;
            var size = new ej.Diagram.Size(decorator.width, decorator.height);
            switch (decorator.shape) {
                case "arrow":
                    d = this._constructArrow(point1, size);
                    attr = { "id": name, "d": d };
                    shape = svg.path(attr);
                    break;
                case "openarrow":
                    d = this._constructArrow(point1, size, true);
                    attr = { "id": name, "d": d };
                    shape = svg.path(attr);
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = { "id": name, "rx": rx, "ry": ry, "cx": (point1.x + size.width / 2), "cy": (point1.y) };
                    shape = svg.ellipse(attr);
                    break;
                case "diamond":
                    d = this._constructDiamond(point1, size, svg);
                    attr = { "id": name, "x": point1.x, "y": point1.y, "width": size.width, "height": size.height, "d": d };
                    shape = svg.path(attr);
                    break;
                case "path":
                    d = this._updatePath(point1.x, point1.y - size.height / 2, size.width, size.height, decorator.pathData, svg);
                    attr = {
                        "id": name, "x": point1.x, "y": point1.y, "width": size.width, "height": size.height, "d": d
                    };
                    shape = svg.path(attr);
                    break;
            }
            //apply translate matrix and rotate matrix here
            shape.setAttribute("transform", "rotate(" + ej.Diagram.Geometry.findAngle(point1, point2) + " " + point1.x + " " + point1.y + ")");
        },

        _constructArrow: function (point, size, isOpen) {
            var path = new ej.Diagram.Path();
            path.moveTo(point.x + size.width, point.y + size.height / 2);
            path.lineTo(point.x, point.y);
            path.lineTo(point.x + size.width, point.y - size.height / 2);
            if (!isOpen) {
                path.close();
            }
            return path.toString();
        },

        _constructDiamond: function (point, size, svg) {
            var path = new ej.Diagram.Path();
            path.moveTo(point.x + size.width, point.y);
            path.lineTo(point.x + size.width / 2, point.y + size.height / 2);
            path.lineTo(point.x, point.y);
            path.lineTo(point.x + size.width / 2, point.y - size.height / 2);
            path.lineTo(point.x + size.width, point.y);
            var d = path.toString();
            return d;
        },
        //#endregion

        //#region Render Label
        _renderLabels: function (node, svg, g) {
            var labels = node.labels;
            for (var i = 0, len = labels.length; i < len; ++i) {
                this._renderLabel(node, labels[i], svg, g);
            }
        },

        _renderLabelBackground: function (label, node, text, svg) {
            var bounds;
            if (ej.browserInfo().name === "mozilla")
                bounds = text.getBoundingClientRect();
            else
                bounds = text.getBBox();
            var point = label._getPosition(node.getBoundingBox());
            var attr = {
                "id": node.fullName() + "_" + label.name + "_lblbg",
                "width": bounds.width, "height": bounds.height, "fill": label.fillColor,
                "stroke": label.borderColor, "stroke-width": label.borderWidth, "transform": "translate(" + 0 + "," + 0 + ")"
            };
            return svg.rect(attr);
        },

        _renderLabel: function (node, label, svg, g) {
            var attr = {
                "id": node.fullName() + "_" + label.name, "class": "ej-d-label", "font-family": label.fontFamily,
                "font-size": label.fontSize, "fill": label.fontColor, "text-decoration": label.textDecoration,
                "dominant-baseline": "middle", "pointer-events": "none"
            };
            if (label.bold)
                attr["font-weight"] = "bold";
            if (label.italic)
                attr["font-style"] = "italic";
            var text = svg.text(attr);
            text.textContent = label.text;
            g.appendChild(text);
            var bg = this._renderLabelBackground(label, node, text, svg);
            g.insertBefore(bg, text);
            this._wrapText(node, text, label, svg);
            this._alignTextOnLabel(label, node, text, svg);
        },

        _updateLabels: function (node, svg) {
            var labels = node.labels;
            for (var i = 0, len = labels.length; i < len; ++i) {
                var label = labels[i];
                var text = svg.getElementById(node.fullName() + "_" + label.name);
                text.setAttribute("fill", label.fontColor);
                this._wrapText(node, text, label, svg);
                this._alignTextOnLabel(label, node, text, svg);
            }
        },

        updateLabel: function (node, label, svg) {
            var text = svg.getElementById(node.fullName() + "_" + label.name);
            text.setAttribute("fill", label.fontColor);
            text.textContent = label.text;
            this._wrapText(node, text, label, svg);
            this._alignTextOnLabel(label, node, text, svg);
        },

        _alignTextOnLabel: function (label, node, text, svg) {
            var bounds = text.getBBox();
            var nodeBounds = node.getBoundingBox();
            var offset = label._getPosition(nodeBounds);
            var point = { x: 0, y: 0 };
            var y = 0;

            if (label.verticalAlignment == ej.Diagram.VerticalAlignment.Top)
                y = label.margin.top + offset.y;
            else if (label.verticalAlignment == ej.Diagram.VerticalAlignment.Center)
                y = offset.y - bounds.height / 2;
            else
                y = offset.y - bounds.height - label.margin.bottom;
            point.y = y;

            switch (label.horizontalAlignment) {
                case ej.Diagram.HorizontalAlignment.Left:
                    switch (label.textAlign) {
                        case ej.Diagram.TextAlign.Left:
                            point.x = offset.x + label.margin.left;
                            break;
                        case ej.Diagram.TextAlign.Center:
                            point.x = offset.x + bounds.width / 2 + label.margin.left;
                            break;
                        case ej.Diagram.TextAlign.Right:
                            point.x = offset.x + bounds.width + label.margin.left;
                            break;
                    }
                    break;
                case ej.Diagram.HorizontalAlignment.Center:
                    switch (label.textAlign) {
                        case ej.Diagram.TextAlign.Left:
                            point.x = offset.x - bounds.width / 2;
                            break;
                        case ej.Diagram.TextAlign.Center:
                            point.x = offset.x;
                            break;
                        case ej.Diagram.TextAlign.Right:
                            point.x = offset.x + bounds.width / 2;
                            break;
                    }
                    break;
                case ej.Diagram.HorizontalAlignment.Right:
                    switch (label.textAlign) {
                        case ej.Diagram.TextAlign.Left:
                            point.x = offset.x - bounds.width - label.margin.right;
                            break;
                        case ej.Diagram.TextAlign.Center:
                            point.x = offset.x - bounds.width / 2 - label.margin.right;
                            break;
                        case ej.Diagram.TextAlign.Right:
                            point.x = offset.x - label.margin.right;
                            break;
                    }
                    break;
            }
            if (node.type === "connector" || node.type === "group") {
                point.x = point.x + nodeBounds.x;
                point.y = point.y + nodeBounds.y;
            }
            text.setAttribute("transform", "translate(" + point.x + "," + point.y + ")");
            //Updates the label background
            var background = svg.getElementById(node.fullName() + "_" + label.name + "_lblbg");
            if (background) {
                var x = point.x + bounds.x;
                y = point.y;
                if (ej.browserInfo().name === "mozilla")
                    y = point.y + label.fontSize / 2;
                var attr = {
                    "width": bounds.width, "height": bounds.height, "transform": "translate(" + x + "," + y + ")", "fill": label.fillColor,
                    "stroke": label.borderColor, "stroke-width": label.borderWidth
                };
                ej.Diagram.Util.attr(background, attr);
            }
        },

        _wrapText: function (node, text, label, svg) {
            var str = label.text; var attr = null;
            var textBBox = node.getBoundingBox();
            while (text.hasChildNodes()) {
                text.removeChild(text.lastChild);
            }
            var bbWidth, bbHeight;
            if (node.line) {
                bbWidth = textBBox.width;
                bbHeight = textBBox.height;
            }
            else {
                bbWidth = textBBox.width - label.margin.left - label.margin.right;
                bbHeight = textBBox.height - label.margin.top - label.margin.bottom;
            }
            bbWidth = bbWidth < label.width ? label.width : bbWidth;
            var eachLine = str.split('\n');
            var x, y, tspan, j, string = "", posX, posY, childNodes, bounds, i;
            childNodes = svg.getElementById(text.id).childNodes;
            for (j = 0; j < eachLine.length; j++) {
                tspan = svg.tspan();
                tspan.style.fontSize = label.fontSize;
                text.appendChild(tspan);
                for (i = 0; i < eachLine[j].length; i++) {
                    bounds = childNodes[childNodes.length - 1].getComputedTextLength();
                    if (label.wrapText) {
                        if (bounds >= (bbWidth - label.fontSize)) {
                            tspan.textContent += eachLine[j][i];
                            posX = -bounds;
                            if (!(i == eachLine[j].length - 1)) {
                                tspan = svg.tspan();
                                text.appendChild(tspan);
                            }
                        }
                        else {
                            tspan.textContent += eachLine[j][i];
                        }
                    }
                    else {
                        tspan.textContent += eachLine[j][i];
                    }
                }
            }
            this._wrapTextAlign(text, childNodes, label.fontSize, label.textAlign);
        },

        _wrapTextAlign: function (text, childNodes, height, textAlign) {
            for (var i = 0; i < childNodes.length; i++) {
                var x = childNodes[i].getComputedTextLength();
                switch (textAlign) {
                    case "left":
                        x = 0;
                        break;
                    case "center":
                        x = -x / 2;
                        break;
                    case "right":
                        x = -x;
                        break;
                }
                var attr = { "x": Number(x), "dy": height };
                var tspan = childNodes[i];
                ej.Diagram.Util.attr(tspan, attr);
            }
        },
        //#endregion

        //#region Render Port
        _renderPorts: function (node, svg, g) {
            var ports = node.ports;
            var port;
            for (var i = 0, len = ports.length; i < len; ++i) {
                port = ports[i];
                var shape = this._renderPort(node, port, svg);
                g.appendChild(shape);
                if (port.visibility & ej.Diagram.PortVisibility.Hidden || port.visibility & ej.Diagram.PortVisibility.Hover || port.visibility & ej.Diagram.PortVisibility.Connect) {
                    shape.setAttribute("visibility", "hidden");
                }
            }
        },

        _renderPort: function (node, port, svg) {
            var shape;
            var attr;
            var size = new ej.Diagram.Size(port.size, port.size);
            var point = port._getPosition(node.getBoundingBox(), true);
            switch (port.shape) {
                case "x":
                    var d = this._constructX(point, port.size);
                    attr = {
                        "id": node.fullName() + "_" + port.name, "class": "ej-d-port", "fill": port.fillColor,
                        "stroke": port.borderColor, "stroke-width": port.borderWidth, "d": d
                    };
                    shape = svg.path(attr);
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = {
                        "id": node.fullName() + "_" + port.name, "class": "ej-d-port", "rx": rx, "ry": ry,
                        "cx": point.x, "cy": point.y, "fill": port.fillColor, "stroke": port.borderColor,
                        "stroke-width": port.borderWidth
                    };
                    shape = svg.ellipse(attr);
                    break;
                case "square":
                    attr = {
                        "id": node.fullName() + "_" + port.name, "class": "ej-d-port", "x": point.x - port.size / 2,
                        "y": point.y - port.size / 2, "width": size.width, "height": size.height,
                        "fill": port.fillColor, "stroke": port.borderColor, "stroke-width": port.borderWidth
                    };
                    shape = svg.rect(attr);
                    break;
            }
            return shape;
        },

        _updatePort: function (node, port, svg) {
            var size = new ej.Diagram.Size(port.size, port.size);
            var point = port._getPosition(node.getBoundingBox(), true);
            var id = node.fullName() + "_" + port.name;
            switch (port.shape) {
                case "x":
                    var d = this._constructX(point, port.size);
                    svg.path({ "id": id, "d": d });
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    svg.ellipse({ "id": id, "rx": rx, "ry": ry, "cx": point.x, "cy": point.y });
                    break;
                case "square":
                    svg.rect({ "id": id, "x": point.x - port.size / 2, "y": point.y - port.size / 2 });
                    break;
            }
        },

        _updatePorts: function (node, svg) {
            var ports = node.ports;
            for (var i = 0, len = ports.length; i < len; ++i) {
                var port = ports[i];
                this._updatePort(node, port, svg);
            }
        },

        _constructX: function (point, size) {
            var path = new ej.Diagram.Path();
            path.moveTo((point.x - size / 2), (point.y - size / 2));
            path.lineTo((point.x + size / 2), (point.y + size / 2));
            path.moveTo((point.x + size / 2), (point.y - size / 2));
            path.lineTo((point.x - size / 2), (point.y + size / 2));
            return path.toString();
        },

        _drawPortHighlighter: function (port, node, svg, parent, scale) {
            var shape, size, point, x, y, transform;
            var bounds = node.getBoundingBox();
            var fill = "#8CC63F";
            var border = "#1A1A1A";
            var borderWidth = 0.5
            if (!port) {
                size = bounds.size();
                point = bounds.location();
                x = point.x * scale;
                y = point.y * scale;
                border = fill;
                fill = "transparent";
                borderWidth = 2;
                var center = new ej.Diagram.Point(node.offsetX - node.width * node.pivot.x + node.width / 2, node.offsetY - node.height * node.pivot.y + node.height / 2);
                transform = "rotate(" + node.rotateAngle + "," + center.x * scale + "," + center.y * scale + ")"
            }
            else {
                size = new ej.Diagram.Size(port.size * 2, port.size * 2);
                point = port._getPosition(node.getBoundingBox());
                var mat = new ej.Diagram.Matrix();
                var matrix = mat._createIdentity();
                matrix._rotateAt(node.rotateAngle, node.offsetX, node.offsetY);
                point = matrix._transform(point);
                x = (point.x - port.size) * scale;
                y = (point.y - port.size) * scale;
                transform = "rotate(" + 0 + "," + (x + size.width / 2) + "," + (y + size.height / 2) + ")"
            }
            var attr = {
                "id": "portHighlighter", "class": "ej-d-port", "x": x,
                "y": y, "width": size.width * scale, "height": size.height * scale,
                "fill": fill, "stroke": border, "stroke-width": borderWidth, "pointer-events": "none", "transform": transform
            };
            shape = svg.rect(attr);
            parent.appendChild(shape);
            return shape;
        },

        _removePortHighlighter: function (svg, parent) {
            var highlighter = svg.getElementById("portHighlighter");
            if (highlighter) {
                svg.removeChild(highlighter, parent);
            }
        },
        //#endregion

        //#region Render Selector

        renderSelector: function (shape, svg, parent, scale, constraints) {
            if (shape.type === "node" || shape.type === "group" || shape.type === "pseudoGroup") {
                var g = svg.g({
                    "id": svg.document.id + "handle_g", "class": "handle",
                    "transform": "translate(" + (shape.offsetX - shape.width / 2) * scale + "," + (shape.offsetY - shape.height / 2) * scale + "),rotate(" + shape.rotateAngle + "," + (shape.width / 2) * scale + "," + (shape.height / 2) * scale + ")"
                });
                parent.appendChild(g);
                this._renderResizeHandle(shape, svg, scale, constraints);
            }
            else {
                var g = svg.g({
                    "id": svg.document.id + "handle_g", "class": "handle"
                });
                parent.appendChild(g);
                this._renderEndPointHandle(shape, svg, scale);
            }
        },

        _renderResizeHandle: function (node, svg, scale, constraints) {
            if (!node._corners) node.updatebounds();
            var rotate = new ej.Diagram.Point(node.width / 2, -20 / scale);
            if (constraints & ej.Diagram.SelectorConstraints.Rotator) {
                this._renderPivotLine(node, svg, scale);
                this._renderRotateThumb("rotate", node, rotate.x, rotate.y, scale, svg);
            }
            if (constraints & ej.Diagram.SelectorConstraints.Resizer) {
                this._renderResizeBorder("resizeBorder", node, svg, scale);
                this._renderResizeCorner("nw-resize", node, 0, 0, svg);
                this._renderResizeCorner("n-resize", node, (node.width / 2) * scale, 0, svg);
                this._renderResizeCorner("ne-resize", node, node.width * scale, 0, svg);
                this._renderResizeCorner("w-resize", node, 0, (node.height / 2) * scale, svg);
                this._renderResizeCorner("e-resize", node, node.width * scale, (node.height / 2) * scale, svg);
                this._renderResizeCorner("sw-resize", node, 0, node.height * scale, svg);
                this._renderResizeCorner("s-resize", node, (node.width / 2) * scale, node.height * scale, svg);
                this._renderResizeCorner("se-resize", node, node.width * scale, node.height * scale, svg);
            }
        },

        renderUserHandles: function (handles, shape, svg, isMultipleSelection, scale, parent) {
            for (var handleIndex = 0; handleIndex < handles.length; handleIndex++) {
                if ((isMultipleSelection && handles[handleIndex].enableMultiSelection) || !isMultipleSelection) {
                    if (handles[handleIndex].visibility)
                        this._renderUserHandle(handles[handleIndex], shape, svg, scale, parent);
                }
            }
        },

        _renderUserHandle: function (handle, node, svg, scale, parent) {
            var g = svg.g({ "id": svg.document.id + "userHandle_g", "class": "userHandle" });
            parent.appendChild(g);
            var position = this._getHandlePosition(handle, node, scale);
            var d = this._updatePath(position.x * scale - handle.size / 4, position.y * scale - handle.size / 4, (handle.size / 2), (handle.size / 2), handle.pathData, svg);
            var shape = this._getHandleShape(handle, position.x * scale, position.y * scale);
            var icon = this._getHandleIcon(handle, d, position.x, position.y);
            g.appendChild(svg.circle(shape));
            g.appendChild(svg.path(icon));
        },

        _getHandleIcon: function (handle, d) {
            var icon = {
                "id": handle.name + "_icon", "class": "userHandle-icon", "d": d, "fill": handle.pathColor
            };
            return icon;
        },

        _getHandlePosition: function (handle, node, scale) {
            var positionPoints = new ej.Diagram.Point();
            var bounds = node.getBoundingBox();
            var location = bounds.location();
            var position = handle.position;
            var distance = 10;
            var matrix = new ej.Diagram.Matrix()._createIdentity();
            if (node.rotateAngle)
                matrix._rotate(node.rotateAngle);
            var point = new ej.Diagram.Point();
            switch (position) {
                case "topLeft":
                    point = matrix._transform(new ej.Diagram.Point(distance + (handle.size / 2), distance + (handle.size / 2)));
                    positionPoints.x = node._corners._topLeft.x - (point.x / scale);
                    positionPoints.y = node._corners._topLeft.y - (point.y / scale);
                    break;
                case "topCenter":
                    point = matrix._transform(new ej.Diagram.Point(0, distance + (handle.size / 2)));
                    positionPoints.x = node._corners._top.x - (point.x / scale);
                    positionPoints.y = node._corners._top.y - point.y / scale;
                    break;
                case "topRight":
                    point = matrix._transform(new ej.Diagram.Point(distance + (handle.size / 2), (distance / 2) - (handle.size)));
                    positionPoints.x = node._corners._topRight.x + (point.x / scale);
                    positionPoints.y = node._corners._topRight.y + (point.y / scale);
                    break;
                case "middleLeft":
                    point = matrix._transform(new ej.Diagram.Point(-(distance + (handle.size / 2)), 0));
                    positionPoints.x = node._corners._left.x + (point.x / scale);
                    positionPoints.y = node._corners._left.y + (point.y / scale);
                    break;
                case "middleRight":
                    point = matrix._transform(new ej.Diagram.Point(distance + (handle.size / 2), 0));
                    positionPoints.x = node._corners._right.x + (point.x / scale);
                    positionPoints.y = node._corners._right.y + (point.y / scale);
                    break;
                case "bottomLeft":
                    point = matrix._transform(new ej.Diagram.Point(-(distance + (handle.size / 2)), distance + (handle.size / 2)));
                    positionPoints.x = node._corners._bottomLeft.x + (point.x / scale);
                    positionPoints.y = node._corners._bottomLeft.y + (point.y / scale);
                    break;
                case "bottomCenter":
                    point = matrix._transform(new ej.Diagram.Point(0, ((distance / 2) - (handle.size))));
                    positionPoints.x = node._corners._bottom.x - (point.x / scale);
                    positionPoints.y = node._corners._bottom.y - (point.y / scale);
                    break;
                case "bottomRight":
                    point = matrix._transform(new ej.Diagram.Point(distance + (handle.size / 2), distance + (handle.size / 2)));
                    positionPoints.x = node._corners._bottomRight.x + (point.x / scale);
                    positionPoints.y = node._corners._bottomRight.y + (point.y / scale);
                    break;
            }
            return positionPoints;
        },

        _getHandleShape: function (handle, cx, cy) {
            var shape = {
                "id": handle.name + "_shape", "class": "userHandle", "cx": cx, "cy": cy, "r": handle.size / 2, "fill": handle.backgroundColor,
                "stroke": handle.borderColor, "stroke-width": handle.borderWidth, "title": handle.name
            };
            return shape;
        },

        _updateHandleShape: function (handle, cx, cy, svg) {
            var attr = { "id": handle.name + "_shape", "cx": cx, "cy": cy };
            svg.circle(attr);
        },

        _updateHanleIcon: function (handle, d, svg) {
            var attr = { "id": handle.name + "_icon", "d": d };
            svg.path(attr);
        },

        _getHanleIcon: function (handle, d, cx, cy) {
            var icon = {
                "id": handle.name, "d": d, "stroke": handle.borderColor,
                "stroke-width": handle.borderWidth, "fill": handle.fillColor
            };
            return icon;
        },

        renderPivotPoint: function (node, scale, svg) {
            if (!svg.getElementById("pivot")) {
                ej.Diagram.SvgContext._renderResizeCorner("pivot", node,
                      node.offsetX * scale, node.offsetY * scale, svg);
                this._updatePivotLine(node, svg, scale);
            }

        },

        _renderPivotLine: function (node, svg, scale) {
            var rotateThumb = new ej.Diagram.Point(node.width / 2, -20 / scale);
            var pivot = new ej.Diagram.Point(node.width / 2, 0);
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var line = svg.line({
                "id": "pivotLine", "x1": rotateThumb.x * scale, "y1": rotateThumb.y * scale, "x2": pivot.x * scale, "y2": pivot.y * scale,
                "stroke": "black", "stroke-width": 1, "stroke-dasharray": "2,3", "fill": "none"
            });
            handle.appendChild(line);
        },

        _renderResizeBorder: function (id, node, svg, scale) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var rect = svg.rect({
                "id": id, "width": node.width * scale, "height": node.height * scale, "stroke": "#097F7F", "stroke-width": 0.6, "stroke-dasharray": "6,3", "fill": "none"
            });
            handle.appendChild(rect);
        },

        _renderResizeCorner: function (corner, node, cx, cy, svg) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var fill = "white";
            if (!(node.canResize()))
                fill = "darkgray";
            var attr = { "id": corner, "fill": fill, "stroke": "black", "stroke-width": 1, "cx": cx, "cy": cy, "r": 7 };
            handle.appendChild(svg.circle(attr));
            var attr = { "id": corner + "_transparent", "class": corner, "fill": "transparent", "cx": cx, "cy": cy, "r": 10 };
            handle.appendChild(svg.circle(attr));
        },

        _renderRotateThumb: function (corner, node, cx, cy, scale, svg) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var d = "M 16.856 10.239 L 18 3.438 l -2.189 0.817 c -0.974 -1.694 -2.482 -2.995 -4.326 -3.696 C 9.375 -0.245 7.071" +
                "-0.18 5.01 0.744 C 2.942 1.668 1.365 3.342 0.558 5.453 c -0.803 2.115 -0.738 4.414 0.185 6.478 c 0.925 2.064 " +
            "2.6 3.645 4.714 4.45 c 0.969 0.371 1.993 0.554 3.013 0.554 c 1.345 0 2.685 -0.317 3.897 -0.948 l -1.016 -1.962 c -1.584 0.822" +
            "-3.445 0.929 -5.114 0.293 c -1.56 -0.596 -2.793 -1.762 -3.479 -3.289 C 2.078 9.502 2.031 7.803 2.622 6.244 C 3.216 4.68 4.387" +
            " 3.443 5.914 2.761 C 7.437 2.079 9.137 2.03 10.7 2.626 c 1.246 0.475 2.271 1.328 2.986 2.424 L 11.54 5.851 L 16.856 10.239 Z";
            var fill = "#231f20";
            if (!(node.canRotate()))
                fill = "darkgray";
            var width = 20;
            var height = 20;
            var dist = 20;
            var attr = {
                "id": "rotatehandle", "class": corner, "fill": fill, "stroke": "black", "stroke-width": 0.5, "width": width, "height": height, "d": d,
                "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
            };
            var path = svg.path(attr);
            var circle = svg.circle({
                "id": "rotate", "class": corner, "fill": "transparent", "stroke": "none", "cx": width / 2, "cy": height / 2, "r": 13,
                "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
            });
            handle.appendChild(path);
            handle.appendChild(circle);
        },

        _renderEndPointHandle: function (connector, svg, scale) {

            if (connector.line.type == "bezier") {
                var line = connector.line;
                var enabled = connector.canDragSegmentThumbs();
                this._renderBezierLine("bezierline1", connector.line.startPoint, connector.line._point1, scale, svg);
                this._renderBezierLine("bezierline2", connector.line.endPoint, connector.line._point2, scale, svg);
                this._renderEndPointCorner("bezierpoint1", line._point1.x * scale, line._point1.y * scale, false, svg, enabled);
                this._renderEndPointCorner("bezierpoint2", line._point2.x * scale, line._point2.y * scale, false, svg, enabled);
            }
            var points = connector.line.points;
            this._renderEndPointCorner("targetEndPoint", points[points.length - 1].x * scale, points[points.length - 1].y * scale, connector.isTargetConnected(), svg,
                connector.canDragTargetEnd());
            this._renderEndPointCorner("sourceEndPoint", points[0].x * scale, points[0].y * scale, connector.isSourceConnected(), svg,
                connector.canDragSourceEnd());

        },

        _renderEndPointCorner: function (corner, x, y, isConnected, svg, isenabled) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var fill = isConnected ? "#8CC63F" : "white";
            if (!isenabled) fill = "darkgray";
            var attr = { "id": corner, "class": corner, "fill": fill, "stroke": "black", "stroke-width": 2, "cx": x, "cy": y, "r": 7 };
            handle.appendChild(svg.circle(attr));
        },

        _renderBezierLine: function (id, start, end, scale, svg) {
            var line = svg.line({
                "id": id, "x1": start.x * scale, "y1": start.y * scale, "x2": end.x * scale, "y2": end.y * scale,
                "stroke": "black", "stroke-width": 1, "stroke-dasharray": "3,3", "fill": "none"
            });
            var handle = svg.getElementById(svg.document.id + "handle_g");
            handle.appendChild(line);
        },
        _renderControlPointCorner: function (corner, point, index, svg) {
            var g = svg.g({ "class": corner });
            svg.getElementById(svg.document.id + "handle_g").appendChild(g);
            var attr = {
                "id": corner + "_" + index, "fill": "white", "stroke": "black", "stroke-width": 2,
                "cx": point.x, "cy": point.y, "r": 5
            };
            g.appendChild(svg.circle(attr));
        },

        _removePivotPoint: function (node, svg, scale) {
            var parent = svg.getElementById(svg.document.id + "handle_g");
            if (parent != null && svg.getElementById("pivot")) {
                parent.removeChild(svg.getElementById("pivot"));
                this._updatePivotLine(node, svg, scale);
            }
        },

        updateSelector: function (shape, svg, scale, diagram, constraints) {
            if (shape.type === "node" || shape.type === "group" || shape.type === "pseudoGroup") {
                var g = svg.g({
                    "id": svg.document.id + "handle_g",
                    "transform": "translate(" + (shape.offsetX - shape.width / 2) * scale + "," + (shape.offsetY - shape.height / 2) * scale + "),rotate(" + shape.rotateAngle + "," + (shape.width / 2) * scale + "," + (shape.height / 2) * scale + ")"
                });
                if (diagram.activeTool.name != "move" && diagram.activeTool.name != "rotate" || shape.type != "node")
                    this._updateResizeHandle(shape, svg, scale, constraints);
                //if (diagram.activeTool.name != "move" && diagram.activeTool.name != "resize") {
                if (diagram.model.userHandles && diagram.model.userHandles.length > 0) {
                    var isMultipleSelection = false;
                    if (diagram.selectionList[0].name == "multipleSelection")
                        isMultipleSelection = true;
                    if (constraints & ej.Diagram.SelectorConstraints.UserHandles)
                        this.updateUserHandles(diagram.model.userHandles, shape, svg, isMultipleSelection, false, scale);
                    //}
                }
            } else {
                this._updateEndPointHandle(shape, svg, scale);
                if (constraints & ej.Diagram.SelectorConstraints.UserHandles)
                    this.updateUserHandles(diagram.model.userHandles, shape, svg, isMultipleSelection, true, scale);
            }
        },

        _updateResizeHandle: function (node, svg, scale, constraints) {
            var corner = node._corners;
            var rotate = new ej.Diagram.Point(node.width / 2, -20 / scale);
            if (constraints & ej.Diagram.SelectorConstraints.Rotator) {
                this._updateRotateHandle("rotate", rotate.x, rotate.y, scale, svg, node);
                this._updatePivotLine(node, svg, scale);
            }
            //this._updateResizeCorner("pivot", node.offsetX * scale, node.offsetY * scale, svg);
            if (constraints & ej.Diagram.SelectorConstraints.Resizer) {
                this._updateResizeCorner("nw-resize", 0, 0, svg);
                this._updateResizeCorner("n-resize", (node.width / 2) * scale, 0, svg);
                this._updateResizeCorner("ne-resize", node.width * scale, 0, svg);
                this._updateResizeCorner("w-resize", 0, (node.height / 2) * scale, svg);
                this._updateResizeCorner("e-resize", node.width * scale, (node.height / 2) * scale, svg);
                this._updateResizeCorner("sw-resize", 0, node.height * scale, svg);
                this._updateResizeCorner("s-resize", (node.width / 2) * scale, node.height * scale, svg);
                this._updateResizeCorner("se-resize", node.width * scale, node.height * scale, svg);
                this._updateResizeBorder("resizeBorder", node, svg, scale);
            }
        },

        _updateRotateHandle: function (thumb, cx, cy, scale, svg, node) {
            var width = 20;
            var height = 20;
            var dist = 20;
            var attr = {
                "id": "rotatehandle",
                "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
            };
            svg.path(attr);
            svg.circle({
                "id": "rotate",
                "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
            });
        },

        _updatePivotLine: function (node, svg, scale) {
            var rotateThumb = new ej.Diagram.Point(node.width / 2, -20 / scale);
            var pivot = new ej.Diagram.Point(node.width / 2, 0);
            var line = svg.line({
                "id": "pivotLine", "x1": rotateThumb.x * scale, "y1": rotateThumb.y * scale, "x2": pivot.x * scale, "y2": pivot.y * scale
            });
        },

        _updateResizeBorder: function (id, node, svg, scale) {
            svg.rect({
                "id": id, "width": node.width * scale, "height": node.height * scale,
            });
        },

        _updateResizeCorner: function (corner, cx, cy, svg) {
            var attr = { "id": corner, "cx": cx, "cy": cy };
            svg.circle(attr);
            var attr = { "id": corner + "_transparent", "cx": cx, "cy": cy };
            svg.circle(attr);
        },

        _updateEndPointHandle: function (connector, svg, scale) {

            var points = connector.line.points;

            this._updateEndPointCorner("targetEndPoint", points[points.length - 1].x * scale, points[points.length - 1].y * scale, connector.isTargetConnected(), svg,
                connector.canDragTargetEnd());
            this._updateEndPointCorner("sourceEndPoint", points[0].x * scale, points[0].y * scale, connector.isSourceConnected(), svg,
                connector.canDragSourceEnd);
            if (connector.line.type == "bezier") {
                var line = connector.line;
                var enabled = connector.canDragSegmentThumbs();
                this._updateEndPointCorner("bezierpoint1", line._point1.x * scale, line._point1.y * scale, false, svg, enabled);
                this._updateEndPointCorner("bezierpoint2", line._point2.x * scale, line._point2.y * scale, false, svg, enabled);
                this._updateBezierLine("bezierline1", connector.line.startPoint, connector.line._point1, scale, svg);
                this._updateBezierLine("bezierline2", connector.line.endPoint, connector.line._point2, scale, svg);
            }
            //if (points.length > 2) {
            //    for (var i = 1, len = points.length; i < len - 1; ++i) {
            //        this._updateControlPointCorner("controlPoint", points[i], i, svg);
            //    }
            //}
        },

        _updateEndPointCorner: function (corner, x, y, isConnected, svg, isenabled) {
            var fill = isConnected ? "#8CC63F" : "white";
            if (!isenabled) fill = "darkgray";
            var attr = { "id": corner, "fill": fill, "cx": x, "cy": y };
            svg.circle(attr);
        },
        _updateBezierLine: function (id, start, end, scale, svg) {
            var line = svg.line({
                "id": id, "x1": start.x * scale, "y1": start.y * scale, "x2": end.x * scale, "y2": end.y * scale
            });
        },
        _updateControlPointCorner: function (corner, point, index, svg) {
            svg.circle({ "id": corner + "_" + index, "cx": point.x, "cy": point.y });
        },

        updateUserHandles: function (handles, node, svg, isMultipleSelection, isDragging, scale) {
            if (handles && handles.length > 0) {
                for (var handleIndex = 0; handleIndex < handles.length; handleIndex++) {
                    if ((isMultipleSelection && handles[handleIndex].enableMultiSelection) || !isMultipleSelection)
                        this._updateHandle(handles[handleIndex], node, svg, isDragging, scale);
                }
            }
        },

        _updateHandle: function (handle, node, svg, isDragging, scale) {
            var position = this._getHandlePosition(handle, node, scale);
            if (!isDragging && handle.visibility) {
                var d = this._updatePath((position.x * scale) - (handle.size) / 4, (position.y * scale) - (handle.size) / 4, (handle.size / 2), (handle.size / 2), handle.pathData, svg);
                this._updateHandleShape(handle, position.x * scale, position.y * scale, svg);
                this._updateHanleIcon(handle, d, svg);
                if (svg.getElementById(handle.name + "_shape") && handle.visibility) {
                    svg.getElementById(handle.name + "_shape").style.display = "block";
                    svg.getElementById(handle.name + "_icon").style.display = "block";
                }
            }
            else {
                if (svg.getElementById(handle.name + "_shape")) {
                    svg.getElementById(handle.name + "_shape").style.display = "none";
                    svg.getElementById(handle.name + "_icon").style.display = "none";
                }
            }
        },

        clearSelector: function (svg, parent) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            if (handle)
                parent.removeChild(handle);
            handle = svg.getElementById(svg.document.id + "userHandle_g");
            if (handle)
                parent.removeChild(handle);
        },

        _initializePageBreaks: function (svg, attr, g) {
            var g1 = svg.g(attr);
            g.appendChild(g1);
            return g1;
        },

        _renderVPageBreakLine: function (start, end, svg, g) {
            var line = svg.line({
                "x1": start.x, "y1": start.y,
                "x2": end.x, "y2": end.y,
                "style": "stroke:#aaaaaa;stroke-width:1;stroke-dasharray:10,10 "
            });
            g.appendChild(line);
        },

        _renderPageSettingsRect: function (start, end, svg, g, _stroke, _fill, _strokewidth) {
            var rect = svg.rect({
                "id": "pageback",
                "x": start.x, "y": start.y, "width": end.x, "height": end.y,
                "style": "fill:" + _fill + ";stroke:" + _stroke + ";stroke-width:" + _strokewidth + ""
            });
            g.appendChild(rect);
        },

        _updatePageBakground: function (start, end) {
            var rect = document.getElementById("pageback");
            var attr = {
                "x": start.x, "y": start.y, "width": end.x, "height": end.y
            };
            if (rect)
                ej.Diagram.Util.attr(rect, attr);
        },

        _renderHPageBreakLine: function (start, end, svg, g) {
            var line = svg.line({
                "x1": start.x, "y1": start.y,
                "x2": end.x, "y2": end.y,
                "style": "stroke:#aaaaaa;stroke-width:1;stroke-dasharray:10,10 "
            });
            g.appendChild(line);
        },

        _removePageBreaks: function (svg, g, layer) {
            if (g != null) {
                if (layer != null) {
                    layer.removeChild(g);
                }
            }
        },

        _removePageSetting: function (svg, g) {
            if (g != null) {
                if (g.firstChild) {
                    return true;
                }
            }
        },

        //#endregion
    };
    //#endregion

})(jQuery, Syncfusion);;
/// <reference path="ej.diagramcommon.js" />
/// <reference path="ej.diagramsvg.js" />

(function ($, ej, undefined) {
    "use strict";

    //#region SymbolPalette widget
    ej.widget("ejSymbolPalette", "ej.SymbolPalette", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,

        //#region Initialization
        defaults: {
            width: 250,
            height: 400,
            paletteItemWidth: 50,
            paletteItemHeight: 50,
            previewWidth: 100,
            previewHeight: 100,
            previewX: 102,
            previewY: 102,
            diagramId: null,
            headerHeight: 30,
            selectedPaletteIndex: 0,
            cssClass: "e-symbolpalette",
            showPaletteItemText: true,
            allowDrag: true,
            palettes: null
        },

        dataTypes: {
            width: "string",
            height: "string",
            paletteItemWidth: "number",
            paletteItemHeight: "number",
            diagramId: "string",
            headerHeight: "number",
            selectedPaletteIndex: "number",
            cssClass: "string",
            showPaletteItemText: "boolean",
            allowDrag: "boolean"
        },

        _canvas: null,

        _init: function () {
            this.selectedItem = null;
            this.activePalette = null;
            this._selectedElement = null;
            this._renderSymbolPalette();
            this._wireEvents();
            this.model.palettes.addEventHandler("collectionchanged", $.proxy(this._collectionChanged, this));
            this._refresh();
        },

        _setModel: function (options) {
            for (var option in options) {
                switch (option) {
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "paletteItemWidth":
                        this.model.paletteItemWidth = options[option];
                        this._redrawContent();
                        break;
                    case "paletteItemHeight":
                        this.model.paletteItemHeight = options[option];
                        this._redrawContent();
                        break;
                    case "headerHeight":
                        this._setHeaderHeight(options[option]);
                        break;
                    case "cssClass":
                        this._setCssClass(options[option]);
                        break;
                    case "showPaletteItemText":
                        this._showItemText(options[option]);
                        break;
                }
            }
        },

        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },
        //#endregion

        //#region Rendering
        _renderSymbolPalette: function () {
            this._canvas = document.createElement("div");
            this._canvas.setAttribute("class", "e-scanvas");
            this._canvas.setAttribute("id", this.element[0].id + "_canvas");
            this.element[0].setAttribute("style", "height:" + this.model.height + "; width:" + this.model.width + ";");
            this.element.append(this._canvas);

            this.element.ejScroller({ width: this._setViewPortWidth(), height: this._setViewPortHeight() });
            var palettes = this.model.palettes;
            if (palettes) {
                if (!(palettes.type === "collection") && palettes instanceof Array) {
                    var collection = new ej.Diagram.Collection();
                    for (var i = 0; i < palettes.length; i++) {
                        var palette = new ej.Diagram.Palette(palettes[i]);
                        collection.add(palette);
                    }
                    this.model.palettes = palettes = collection;
                }
                for (var i = 0, len = palettes.length; i < len; i++) {
                    palette = palettes[i];
                    this._renderPalette(palette);
                }
            }
        },
        _itemCollectionChanged: function (args) {
            var content = document.getElementById(this.activePalette.name + "_content");
            if (args.changeType == "remove") {
                content.removeChild(content.getElementsByClassName("e-paletteItem e-draggable e-js").item(args.element.name + "_paletteItem"));
                this.selectedItem = null;
            }
            else {
                this._renderItem(args.element, content);
            }
        },
        _setViewPortWidth: function () {
            var w = $(this.element[0].parentNode).width();
            var left = this.element[0].offsetLeft;

            return Math.abs(w - left);
        },

        _setViewPortHeight: function () {
            var h = $(this.element[0].parentNode).height();
            var top = this.element[0].offsetTop;

            return Math.abs(h - top);
        },
        _renderPalette: function (palette) {
            this._renderHeader(palette);
            this._renderContent(palette);
            this._collapse(palette);
        },

        _renderHeader: function (palette) {
            var header = document.createElement("div");
            var attr = { "id": palette.name + "_header", "class": "e-header", "style": "height:" + this.model.headerHeight + "px;" };
            ej.Diagram.Util.attr(header, attr);
            var index = this.model.palettes.indexOf(palette);
            this._renderHeaderArrow(header, palette.expanded, index);
            this._renderHeaderText(header, palette.name);
            this._addExpandedClass(header, palette.expanded);
            if (this.model.selectedPaletteIndex === index) {
                $(header).addClass("selected");
                this.activePalette = palette;
            }
            this._canvas.appendChild(header);
        },

        _renderHeaderText: function (header, text) {
            var span = document.createElement("span");
            span.setAttribute("class", "e-header-text");
            span.innerHTML = text;
            header.appendChild(span);
        },

        _renderHeaderArrow: function (header, expanded, index) {
            var arrow = document.createElement("span");
            arrow.setAttribute("class", "e-header-arrow");
            arrow.setAttribute("style", "top:" + (this.model.headerHeight - 16) / 2 + "px;");
            this._addExpandedClass(arrow, expanded);
            if (this.model.selectedPaletteIndex === index)
                $(arrow).addClass("selected");
            header.appendChild(arrow);
        },

        _renderContent: function (palette) {
            var content = document.createElement("div");
            content.setAttribute("id", palette.name + "_content");
            content.setAttribute("class", "e-scontent");
            if ((this.model.palettes.length - 1) === this.model.palettes.indexOf(palette))
                content.setAttribute("style", "border-bottom-width: 0px;");
            this._canvas.appendChild(content);
            var paletteItems = palette.items;
            if (paletteItems && paletteItems.length > 0) {
                for (var i = 0, len = paletteItems.length; i < len; i++) {
                    this._renderItem(paletteItems[i], content);
                }
            }
        },

        _renderItem: function (paletteItem, content) {
            var svg = this._renderItemContainer(paletteItem.name, content);
            if (paletteItem.type === "connector")
                this._renderConnector(paletteItem, svg);
            else
                this._renderNode(paletteItem, svg);
        },

        _renderItemContainer: function (id, content, isDrag) {
            var container = document.createElement("div");
            var attr = { "id": id + "_paletteItem", "draggable": "true", "height": this.model.paletteItemHeight + "px", "width": this.model.paletteItemWidth + "px", "class": "e-paletteItem", "style": "-ms-touch-action: none;touch-action: none;" };
            ej.Diagram.Util.attr(container, attr);
            content.appendChild(container);
            var paletteId = this.element[0].id;
            var context = this;
            if (!isDrag) {
                $(container).ejDraggable({
                    clone: true,
                    cursorAt: { top: context.model.previewY, left: context.model.previewX },
                    helper: function (event) {
                        var paletteItemWidth, paletteItemHeight;
                        paletteItemWidth = context.model.paletteItemWidth;
                        paletteItemHeight = context.model.paletteItemHeight;
                        context.model.paletteItemWidth = context.model.previewWidth;
                        context.model.paletteItemHeight = context.model.previewHeight;
                        var svg = context._renderItemContainer(context.selectedItem.name + "_clone", document.body, true);
                        svg.document.setAttribute("drag", "true");
                        var item = ej.Diagram.Util.clone(context.selectedItem);
                        if (context.selectedItem.type === "connector") {
                            var bounds = item.getBoundingBox();
                            var delWidth = (context.model.previewWidth - 10) / bounds.width;
                            var delHeight = (context.model.previewHeight - 10) / bounds.height;
                            context.selectedItem._scale(new ej.Diagram.Matrix(), delWidth, delHeight, bounds.location());
                            context._renderConnector(context.selectedItem, svg);
                        }
                        else {
                            item.width = context.model.previewWidth - 10;
                            item.height = context.model.previewHeight - 10;
                            item.offsetX = item.width / 2 + 5;
                            item.offsetY = item.height / 2 + 5;
                            context._renderNode(item, svg);
                        }
                        var helper = document.getElementById(context.selectedItem.name + "_clone_paletteItem");
                        $(helper).addClass("dragClone");
                        context.model.paletteItemWidth = paletteItemWidth;
                        context.model.paletteItemHeight = paletteItemHeight;
                        return $(helper);
                    },
                    drag: function () {
                        var proxy = this;
                        var element = $(".dragClone")[0];
                        if (element) {
                            element.setAttribute("paletteId", paletteId);
                        }
                    },
                    dragStop: function (event, ui) {
                        $(".dragClone").remove();
                    }
                });
            }
            return this._renderAnchor(container, id);
        },

        _renderAnchor: function (container, id) {
            var anchor = document.createElement("div");
            var attr = { "id": id + "_anchor", "height": this.model.paletteItemHeight + "px", "width": this.model.paletteItemWidth + "px", "class": "e-anchor" };
            ej.Diagram.Util.attr(anchor, attr);
            container.appendChild(anchor);
            var svg = this._renderSvg(anchor, id);
            this._renderText(anchor, id);
            return svg;
        },

        _renderSvg: function (anchor, id) {
            var wrapper = document.createElement("div");
            var attr = {
                "id": id + "_svgWrapper", "width": this.model.paletteItemWidth + "px;",
                "height": +this.model.paletteItemHeight + "px;", "class": "e-svg-container"
            };
            ej.Diagram.Util.attr(wrapper, attr);
            attr = {
                "id": id + "_svg", "width": this.model.paletteItemWidth, "height": this.model.paletteItemHeight,
                "version": "1.1", "xlink": "http://www.w3.org/1999/xlink"
            };
            var svg = new ej.Diagram.Svg(attr);
            wrapper.appendChild(svg.document);
            anchor.appendChild(wrapper);
            return svg;
        },

        _renderText: function (anchor, text) {
            var div = document.createElement("div");
            div.setAttribute("class", "e-text-container");
            div.setAttribute("style", "width:" + this.model.paletteItemWidth + "px;");
            var span = document.createElement("span");
            span.setAttribute("class", "e-paletteItemText");
            span.innerHTML = text;
            div.appendChild(span);
            if (!this.model.showPaletteItemText)
                div.style.display = "none";
            anchor.appendChild(div);
        },

        _renderNode: function (node, svg) {
            var group = ej.Diagram.SvgContext.renderNode(node, svg);
            var isDrag = false;
            if (svg.document.getAttribute("drag") != null) {
                isDrag = true;
            }
            this._transformItem(group, node.width, node.height, node, isDrag);
        },

        _renderConnector: function (connector, svg) {
            var group = ej.Diagram.SvgContext.renderConnector(connector, svg);
            var bounds = ej.Diagram.Geometry.rect(connector.line.points);
            var isDrag = false;
            if (svg.document.getAttribute("drag") != null) {
                isDrag = true;
            }
            this._transformItem(group, bounds.width, bounds.height, null, isDrag);
        },

        //#endregion

        //#region Events
        _wireEvents: function () {
            this._on(this.element, ej.eventType.click, ".e-header", this._headerClick);
            this._on(this.element, ej.eventType.mouseDown, ".e-header", this._headerMouseDown);
            this._on(this.element, "mouseover", ".e-header", this._headerMouseOver);
            this._on(this.element, "mouseout", ".e-header", this._headerMouseOut);
            this._on(this.element, ej.eventType.mouseDown, ".e-paletteItem", this._itemMouseDown);
            this._on(this.element, "mouseover", ".e-paletteItem", this._itemMouseOver);
            this._on(this.element, "mouseout", ".e-paletteItem", this._itemMouseOut);
            this._on(this.element, ej.eventType.click, ".e-anchor", this._itemClick);

            if (window) {

                this._on($(window), "resize", this.updateScrollerViewport);
            }
        },

        _headerClick: function (evt) {
            var proxy = this;
            $(evt.currentTarget).next(".e-scontent").slideToggle(200, function () {
                proxy._refresh();
            });
            this._updateHeaderState(evt.currentTarget);
            var palette = this._getActivePalette(evt.currentTarget.id);
            if (palette && palette !== this.activePalette)
                this.activePalette = palette;


        },

        _headerMouseDown: function (evt) {
            evt.preventDefault();
            evt.stopPropagation();
        },

        _headerMouseOver: function (evt) {
            $(evt.currentTarget).addClass("hover");
            $(evt.currentTarget).find(".e-header-arrow").addClass("hover");
        },

        _headerMouseOut: function (evt) {
            $(evt.currentTarget).removeClass("hover");
            $(evt.currentTarget).find(".e-header-arrow").removeClass("hover");
        },

        _itemMouseDown: function (evt) {
            if (this.model.diagramId) {
                var diagram = $("#" + this.model.diagramId).ejDiagram("instance");
                if (diagram.activeTool instanceof ej.Diagram.LineTool) {
                    diagram.activeTool._showAllPorts(true);
                    diagram.deActivateTool();
                }
            }
            if (!this.model.allowDrag) {
                evt.preventDefault();
                evt.stopPropagation();
            }
            if (this._selectedElement) {
                if (this._selectedElement !== evt.currentTarget) {
                    $(this._selectedElement).removeClass("selected");
                    this._selectedElement = evt.currentTarget;
                }
            }
            else
                this._selectedElement = evt.currentTarget;
            if (!($(evt.currentTarget).hasClass("selected")))
                $(evt.currentTarget).addClass("selected");

            var palette = this._getActivePalette(evt.currentTarget.parentElement.id);
            if (palette && palette !== this.activePalette) {
                this._clearHeaderSelection();
                this._updateHeaderSelection($(evt.currentTarget.parentElement).prev());
                this.activePalette = palette;
            }

            if (this.activePalette && this._selectedElement) {
                var name = this._selectedElement.id.slice(0, this._selectedElement.id.lastIndexOf('_'));
                var items = this.activePalette.items;
                for (var i = 0, len = items.length; i < len; i++) {
                    var item = items[i];
                    if (item.name === name)
                        this.selectedItem = item;
                }
            }
        },

        _itemMouseOver: function (evt) {
            $(evt.currentTarget).addClass("hover");
        },

        _itemMouseOut: function (evt) {
            $(evt.currentTarget).removeClass("hover");
        },

        _itemDraStart: function (evt) {
            var dataTransfer = evt.originalEvent.dataTransfer;
            dataTransfer.setData("Text", this.element[0].id);
            dataTransfer.effectAllowed = "copy";
        },

        _itemClick: function (evt) {
            if (this.model.diagramId) {
                var diagram = $("#" + this.model.diagramId).ejDiagram("instance");
                var item = ej.Diagram.Util.clone(this.selectedItem);
                item.name += ej.Diagram.Util.randomId();
                diagram.add(item);
            }
        },

        _collectionChanged: function (args) {
            if (args.changeType == "insert") {
                this._renderPalette(args.element);
            }
            else if (args.changeType == "remove") {
                var palette = args.element;
                if (palette != null) {
                    var paletteHeader = document.getElementById(palette.name + "_header");
                    if (paletteHeader)
                        paletteHeader.parentNode.removeChild(paletteHeader);
                    var paletteContent = document.getElementById(palette.name + "_content");
                    if (paletteContent)
                        paletteContent.parentNode.removeChild(paletteContent);
                }
            }
        },
        //#endregion

        //#region Public methods
        setWidth: function (value) {
            this.model.width = value;
            this._setWidth(value);
        },

        setHeight: function (value) {
            this.model.height = value;
            this._setHeight(value);
        },

        setPaletteItemWidth: function (value) {
            this.model.paletteItemWidth = value;
            this._redrawContent();
        },

        setPaletteItemHeight: function (value) {
            this.model.paletteItemHeight = value;
            this._redrawContent();
        },

        setHeaderHeight: function (value) {
            this.model.headerHeight = value;
            this._setHeaderHeight(value);
        },

        setCssClass: function (value) {
            this._setCssClass(value);
            this.model.cssClass = value;
        },

        showPaletteItemText: function (value) {
            this._showItemText(value);
            this.model.showPaletteItemText = value;
        },

        searchPalette: function (palette, key) {
            var paletteCollection = new ej.Diagram.Collection();
            var paletteItems = null;
            if (key != "") {
                palette = this._getSearchPaletteItems(key);
                paletteCollection.add(palette);
            }
            return paletteCollection;
        },
        //#endregion

        //#region Helper methods
        _getSearchPaletteItems: function (key) {
            var palette = new ej.Diagram.Palette();
            var palettes = this.model.palettes, i, j;
            for (i = 0; i < palettes.length; i++) {
                if ((((palettes[i].name).toLocaleLowerCase()).search(key.toLocaleLowerCase())) >= 0) {
                    for (j = 0; j < palettes[i].items.length; j++) {
                        this.element.find("#" + palettes[i].items[j].fullName() + "_paletteItem")[0].style.display = "block";
                    }
                }
                else {
                    for (j = 0; j < palettes[i].items.length; j++) {
                        if ((((palettes[i].items[j].name).toLocaleLowerCase()).search(key.toLocaleLowerCase())) >= 0) {
                            this.element.find("#" + palettes[i].items[j].fullName() + "_paletteItem")[0].style.display = "block";
                        }
                        else
                            this.element.find("#" + palettes[i].items[j].fullName() + "_paletteItem")[0].style.display = "none";
                    }
                }
            }
            return palette;
        },

        _addExpandedClass: function (element, expanded) {
            if (expanded)
                $(element).addClass("expanded");
            else
                $(element).addClass("collapsed");
        },

        _collapse: function (palette) {
            if (!palette.expanded)
                $(document.getElementById(palette.name + "_header")).next(".e-scontent").slideToggle(10);

        },

        _transformItem: function (group, width, height, node, isDrag) {
            if (!isDrag) {
                var scale = this._getScaleFactor(width, height);
                var translate = this._getTranslateFactor(width, height);
                var transform = "";
                //translate
                transform += "translate(" + translate + "," + translate + ")";
                //scale
                transform += "scale(" + scale + "," + scale + ")";
                group.setAttribute("transform", transform);
            }
            if (node && ej.browserInfo().name === "msie") {
                if (node.shape.type === "html") {
                    var htmlDiv = document.getElementById(node.name + "_html");
                    htmlDiv.setAttribute("style", "zoom:" + scale + ";");
                }
            }
        },

        _getScaleFactor: function (width, height) {
            var fx = (30 * this.model.paletteItemWidth) / 100;
            var fy = (30 * this.model.paletteItemHeight) / 100;
            var sx = (this.model.paletteItemWidth - fx) / width;
            var sy = (this.model.paletteItemHeight - fy) / height;
            sx = sx > 1 ? 1 : sx;
            sy = sy > 1 ? 1 : sy;
            return Math.min(sx, sy);
        },

        _getTranslateFactor: function (width, height) {
            var x = (15 / 100) * this.model.paletteItemWidth;
            var y = (15 / 100) * this.model.paletteItemHeight;
            return Math.min(x, y);
        },

        _setWidth: function (val) {
            this.element.css("width", val);
        },

        _setHeight: function (val) {
            this.element.css("height", val);
        },

        _setHeaderHeight: function (val) {
            this.element.find(".e-header").css("height", val);
            this.element.find(".e-header-arrow").css("top", (val - 16) / 2);
        },

        _setCssClass: function (val) {
            this.element.removeClass(this.model.cssClass).addClass(val);
        },

        _showItemText: function (val) {
            if (val !== this.model.showPaletteItemText)
                if (val) {
                    this.element.find(".e-text-container").show();
                }
                else
                    this.element.find(".e-text-container").hide();
        },

        _redrawContent: function () {
            var contents = this.element.find(".e-scontent");
            for (var i = 0, len = contents.length; i < len; ++i) {
                var content = contents[i];
                $(content).empty();
                var palette = this.model.palettes[i];
                var paletteItems = palette.items;
                if (paletteItems && paletteItems.length > 0) {
                    for (var count = 0, l = paletteItems.length; count < l; count++) {
                        this._renderItem(paletteItems[count], content);
                    }
                }
            }
        },

        _getActivePalette: function (elementId) {
            var name = elementId.slice(0, elementId.lastIndexOf('_'));
            var palettes = this.model.palettes;
            for (var i = 0, len = palettes.length; i < len; i++) {
                var palette = palettes[i];
                if (palette.name === name && palette !== this.activePalette)
                    return palette;
            }
            return null;
        },

        _updateHeaderState: function (target) {
            this._clearHeaderSelection();
            this._updateExpandState(target);
            this._updateHeaderSelection(target);
        },

        _updateExpandState: function (target) {
            if (($(target).hasClass("expanded"))) {
                $(target).removeClass("expanded").addClass("collapsed");
                $(target).find(".e-header-arrow").removeClass("expanded").addClass("collapsed");
            }
            else {
                $(target).removeClass("collapsed").addClass("expanded");
                $(target).find(".e-header-arrow").removeClass("collapsed").addClass("expanded");
            }
        },

        _updateHeaderSelection: function (target) {
            if (!($(target).hasClass("selected"))) {
                $(target).addClass("selected");
                $(target).find(".e-header-arrow").addClass("selected");
            }
        },

        _clearHeaderSelection: function () {
            this.element.find(".e-header").removeClass("selected");
            this.element.find(".e-header-arrow").removeClass("selected");
        },

        _addItem: function (palette, item) {
            var content = document.getElementById(palette.name + "_content");
            this._renderItem(item, content);
        },

        _removeItem: function (palette, item) {
            var content = document.getElementById(palette.name + "_content");
            content.removeChild(document.getElementById(item.name + "_paletteItem"));
            if (item === this._selectedElement)
                this._selectedElement = null;
        },

        updateScrollerViewport: function () {
            var scroller = this.element.ejScroller("instance");
            scroller.model.touchScroll = false;
            scroller.model.width = this._setViewPortWidth();
            scroller.model.height = this._setViewPortHeight();
            this._refresh();
        },

        _refresh: function () {
            this.element.ejScroller("refresh");
        },

        //#endregion
    });
    //#endregion

})(jQuery, Syncfusion);;


(function ($, ej) {
    "use strict";

    //#region Tool
    var Tool = (function () {
        function Tool(name, diagram) {
            this.name = name;
            this.diagram = diagram;
            this.helper = null;
            this.inAction = false;
            this.selectedObject = null;
            this.startPoint = new ej.Diagram.Point(0, 0);
            this.currentPoint = new ej.Diagram.Point(0, 0);
            this.previousPoint = new ej.Diagram.Point(0, 0);
            this.cursor = "default";
            this._currentPossibleConnection = null;
            this._previousPossibleConnection = null;
            this._possibleConnectionPort = null;
            this.singleAction = false;
            this.svgHelper = null;
            this.single = null;
            this._centralPoint = new ej.Diagram.Point(0, 0);
            this._enableAutoNode = null;
            this._prevTool = null;
        }
        Tool.prototype.abort = function (evt) {
            this._endAction();
        };
        Tool.prototype.keydown = function (evt) {
            var keycode = evt.keyCode ? evt.keyCode : evt.which;
            if (keycode === 27 && this.inAction) {
                this.abort();
            }
            else if (!this.inAction && keycode == 27) {
                this.diagram._clearSelection();
            }
        };
        Tool.prototype._findNodeUnderMouse = function (evt, skip) {
            var obj;
            var parents = $(evt.target).parents("g");
            if (parents.length > 3) {
                var parent;
                var type;
                //check selection exists already
                var selections = this.diagram.selectionList;
                if (selections.length > 0) {
                    var count = parents.length - 3;
                    if (count > 1 && parents[count - 1].getAttribute("class") == "ej-d-group") {
                        var id = (parents[count - 1]).getAttribute("id");
                        if (id !== (this.diagram._svg.document.id + "handle_g")) {
                            var i;
                            var child;
                            var group = this._findObj(parents[count - 1]);
                            if (skip) {
                                obj = group;
                                if (selections[0].type === "pseudoGroup" || selections.contains(group)) {
                                    obj = selections[0];
                                }
                                else {
                                    for (i = count - 2; i >= 0; i--) {
                                        if (group.type === "group") {
                                            child = this._findObj(parents[i], group);
                                            if (selections.contains(child)) {
                                                obj = selections[0];
                                                break;
                                            }
                                            else if (group.children.contains(selections[0])) {
                                                obj = child;
                                            }
                                        }
                                        group = child;
                                    }
                                }
                            }
                            else {
                                if (selections.contains(group) || group.children.contains(selections[0])) {
                                    for (i = count - 2; i >= 0; i--) {
                                        //if (group.getAttribute("class") == "group") 
                                        {
                                            child = this._findObj(parents[i], group);
                                            if (selections.contains(child)) {
                                                if (child.type === "node" || child.type === "connector") {
                                                    obj = group;
                                                    break;
                                                }

                                            }
                                            else {
                                                obj = child;
                                                break;
                                            }
                                        }
                                    }
                                }
                                else {

                                    obj = group;
                                }
                            }
                        }
                    }
                    else if (selections[0].type === "pseudoGroup") {
                        //if (skip) {
                        //    obj = selections[0];
                        //}
                        //else
                        {
                            obj = this._findObj(parents[count - 1], selections[0]);
                            if (!obj) {
                                obj = this._findObj(parents[parents.length - 4]);
                            }
                        }
                    }
                    else {
                        obj = this._findObj(parents[parents.length - 4]);
                    }
                }
                else {
                    obj = this._findObj(parents[parents.length - 4]);
                }
            }
            return obj;
        };
        Tool.prototype._findObj = function (element, group) {
            var obj;
            var id = element.getAttribute("id");
            if (group) {
                obj = this.diagram._findChildren(group, id);
            }
            else {
                var type = element.getAttribute("class");
                if (type) {
                    if (type === "ej-d-node" || type === "ej-d-group") {
                        obj = this.diagram._findNode(id);
                    }
                    else if (type === "ej-d-connector") {
                        obj = this.diagram._findConnector(id);
                    }
                }
            }
            return obj;
        };
        Tool.prototype.mousedown = function (evt) {
            this.startPoint = this.mousePosition(evt);
            this.previousPoint = this.startPoint;
        };

        Tool.prototype.mousemove = function (evt) {
            this.currentPoint = this.mousePosition(evt);
        };
        Tool.prototype.mouseup = function (evt) {
            this._endAction();
            if (this.singleAction) {
                this.diagram.activateTool("select");
            }
        };
        Tool.prototype.mousePosition = function (evt) {
            return this.diagram._mousePosition(evt);
        };
        Tool.prototype.updateCursor = function (cursor) {
            this.diagram._currentCursor = cursor;
            this.diagram._updateCursor();
        };
        Tool.prototype.nearestGridPoint = function (point) {
            var snapPt = new ej.Diagram.Point();
            if (this.diagram.model.snap & ej.Diagram.Snap.Grid) {
                var magnification = this.diagram.model.magnification / 100;
                var hSpacing = this.diagram.model.grid.horizontalSpacing * magnification;
                var vSpacing = this.diagram.model.grid.verticalSpacing * magnification;
                var width = point.x % hSpacing;
                var height = point.y % vSpacing;
                if (width >= (hSpacing / 2)) {
                    snapPt.x = point.x + hSpacing - width;
                }
                else {
                    snapPt.x = point.x - width;
                }
                if (height >= (vSpacing / 2)) {
                    snapPt.y = point.y + vSpacing - height;
                }
                else {
                    snapPt.y = point.y - height;
                }
            }
            return snapPt;
        };
        Tool.prototype.snap = function (point) {
            if (this.diagram.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToHorizontalLines)
                point.x = this.diagram._snapping._round(point.x, this.diagram.model.snapSettings.horizontalGridlines._snapinterval);
            if (this.diagram.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToVerticalLines)
                point.y = this.diagram._snapping._round(point.y, this.diagram.model.snapSettings.verticalGridlines._snapinterval);
            return point;

        };
        Tool.prototype._endAction = function () {
            this.inAction = false;

            if (this.svgHelper) {
                var helper = this.helper;
                if (helper.type === "node" || helper.type === "connector") {
                    this.diagram._spatialsearch._removeFromaQuad(helper);
                }
                else {
                    var length = this.helper.children.length - 1;
                    for (var i = length; i >= 0; i--) {
                        this.diagram._spatialsearch._removeFromaQuad(helper.children[i]);
                    }
                }
            }
            if (this.helper) {
                if (this.svgHelper)
                    this.diagram._svg.removeChild(this.svgHelper, this.diagram._diagramLayer);
                if (this.helper.gradient) {
                    var defs = this.diagram._svg.getElementsByTagName("defs")[0];
                    defs.removeChild(this.diagram._svg.getElementById(this.helper.fullName() + "_gradient"));
                }
            }
            if (this._previousPossibleConnection) {
                this._previousPossibleConnection = null;
                ej.Diagram.SvgContext._removePortHighlighter(this.diagram._svg, this.diagram._adornerLayer);
            }
            this._currentPossibleConnection = null;
            this._possibleConnectionPort = null;
            this.helper = null;
            this.svgHelper = null;
            this.selectedObject = null;
            this.startPoint = new ej.Diagram.Point(0, 0);
            this.currentPoint = new ej.Diagram.Point(0, 0);
            this.previousPoint = new ej.Diagram.Point(0, 0);
        };
        Tool.prototype._isResizeCorner = function (type) {
            var isResizeCorner = false;
            switch (type) {
                case "n-resize":
                case "e-resize":
                case "w-resize":
                case "s-resize":
                case "ne-resize":
                case "nw-resize":
                case "se-resize":
                case "sw-resize":
                case "targetEndPoint":
                case "sourceEndPoint":
                    isResizeCorner = true;
                    break;
            }
            return isResizeCorner;
        };
        Tool.prototype._initHelper = function () {
            if (this.selectedObject) {
                this.helper = ej.Diagram.Util.clone(this.selectedObject);
                this._updateHelperName(this.helper);
            }
        };
        Tool.prototype._updateHelperName = function (helper, child) {
            if (child) {
                helper.name += "helper";
            }
            else
                helper.name = "helper";
            helper.opacity = 0.5;
            var labels = helper.labels;
            for (var i = 0, len = labels.length; i < len; ++i) {
                labels[i].name += "helper";
            }
            if (helper.ports) {
                var ports = helper.ports;
                for (i = 0, len = ports.length; i < len; ++i) {
                    ports[i].name += "helper";
                }
            }
            if (helper.type === "group") {
                var children = helper.children;
                for (i = 0, len = children.length; i < len; i++) {
                    this._updateHelperName(children[i], true);
                }
            }
        };
        Tool.prototype._raiseEvent = function (type, args) {
            this.diagram._trigger(type, args);
            return args;
        };
        Tool.prototype._checkConnectionPossible = function (evt) {
            var port = null;
            var pt = new ej.Diagram.Point(this.currentPoint.x, this.currentPoint.y);
            var node = this._findNodeUnderMouse(evt);
            if (node && !(node.type === "connector")) {
                if (!(node.type === "pseudoGroup")) {
                    this._showPorts(node);
                    this._currentPossibleConnection = node;
                    if (this._previousPossibleConnection == null || this._previousPossibleConnection.fullName() !== this._currentPossibleConnection.fullName()) {
                        this._previousPossibleConnection = this._currentPossibleConnection;
                    }
                }
            }
            else {
                this._showPorts();
                this._currentPossibleConnection = null;
                this._possibleConnectionPort = null;
            }
            if (this._currentPossibleConnection) {
                port = this.diagram._findPortAtPoint(pt, this._currentPossibleConnection);
                if ((port && port.canConnect()) || this._currentPossibleConnection.canConnect()) {
                    ej.Diagram.SvgContext._drawPortHighlighter(port, this._currentPossibleConnection, this.diagram._svg, this.diagram._adornerLayer,
                        this.diagram._zoomPan.currentZoom);
                }
                else {
                    ej.Diagram.SvgContext._removePortHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                }
                this._possibleConnectionPort = port;
                if (port && port.canConnect()) {
                    this.currentPoint = this._possibleConnectionPort._getPosition(this._currentPossibleConnection.getBoundingBox());
                }
            }
            else {
                if (this._previousPossibleConnection) {
                    this._previousPossibleConnection = null;
                    ej.Diagram.SvgContext._removePortHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                    this._possibleConnectionPort = null;
                }
            }

        };
        Tool.prototype._showPorts = function (node) {
            if (node) {
                if (this._currentPossibleConnection && this._currentPossibleConnection.fullName() !== node.fullName()) {
                    this._showPort(this._currentPossibleConnection, true);
                }
                this._showPort(node);
            }
            else {
                if (this._currentPossibleConnection != null) {
                    this._showPort(this._currentPossibleConnection, true);
                }
            }
        };
        Tool.prototype._showPort = function (node, hide) {
            var ports;
            var port;
            var portShape;
            var i;
            var len;
            if (hide) {
                ports = node.ports;
                for (i = 0, len = ports.length; i < len; ++i) {
                    port = ports[i];
                    if (port.visibility & ej.Diagram.PortVisibility.Hover || port.visibility & ej.Diagram.PortVisibility.Connect) {
                        portShape = this.diagram._svg.getElementById(node.fullName() + "_" + port.name);
                        portShape.setAttribute("visibility", "hidden");
                    }
                }
            }
            else {
                ports = node.ports;
                for (i = 0, len = ports.length; i < len; ++i) {
                    port = ports[i];
                    if (port.visibility & ej.Diagram.PortVisibility.Hover || port.visibility & ej.Diagram.PortVisibility.Connect) {
                        portShape = this.diagram._svg.getElementById(node.fullName() + "_" + port.name);
                        portShape.setAttribute("visibility", "visible");
                    }
                }
            }
        };
        Tool.prototype._showAllPorts = function (hide) {
            var i;
            var len;
            var nodes = this.diagram.model.nodes;
            if (hide) {
                for (i = 0, len = nodes.length; i < len; ++i) {
                    this._showPort(nodes[i], true);
                }
            }
            else {
                for (i = 0, len = nodes.length; i < len; ++i) {
                    this._showPort(nodes[i]);
                }
            }
        };
        Tool.prototype._disconnect = function (connector) {
            var args;
            if (connector.targetNode != null) {
                args = this._raiseEvent("connectionChanging", { element: connector, connection: connector.targetNode, port: connector.targetPort, cancel: false });
                if (!args.cancel) {
                    connector.targetNode.inEdges.remove(connector);
                    connector.targetNode = null;
                    connector.targetPort = null;
                    this._raiseEvent("connectionChanged", { element: connector, connection: connector.targetNode, port: connector.targetPort });
                }

            }
            if (connector.sourceNode != null) {
                args = this._raiseEvent("connectionChanging", { element: connector, connection: connector.sourceNode, port: connector.sourcePort, cancel: false });
                if (!args.cancel) {
                    connector.sourceNode.outEdges.remove(connector);
                    connector.sourceNode = null;
                    connector.sourcePort = null;
                    this._raiseEvent("connectionChanged", { element: connector, connection: connector.sourceNode, port: connector.sourcePort });
                }
            }
        };
        return Tool;
    })();

    ej.Diagram.Tool = Tool;
    //#endregion

    //#region SelectTool
    var SelectTool = (function (base) {
        ej.Diagram.extend(SelectTool, base);
        function SelectTool(diagram) {
            base.call(this, "select", diagram);
            this.cursor = "default";
            this._svgHelper = null;
        }
        SelectTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            var node = this._findNodeUnderMouse(evt);
            if (node) {
                this.selectedObject = node;
            }
        };
        SelectTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (!this.startPoint.isEmpty()) {
                if (!this.inAction && (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y)) {
                    this.inAction = true;
                    this._initHelper();
                }
                this._updateHelper();
            }
        };
        SelectTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.currentPoint = this.mousePosition(evt);
                var width = Math.abs(this.startPoint.x - this.currentPoint.x);
                var height = Math.abs(this.startPoint.y - this.currentPoint.y);
                var x = (this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x;
                var y = (this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y;
                var rect = new ej.Diagram.Rectangle(x, y, width, height);
                if (width !== 0 || height !== 0) {
                    var i;
                    var len;
                    var collection = new ej.Diagram.Collection();
                    var nodes = this.diagram.model.nodes;
                    for (i = 0, len = nodes.length; i < len; i++) {
                        var node = nodes[i];
                        if (node.canSelect()) {
                            if (rect.contains(node._bounds))
                                collection.add(node);
                            else if (node.type === "group") {
                                this._checkGroupChildren(collection, node, rect);
                            }
                        }
                    }
                    var connectors = this.diagram.model.connectors;
                    for (i = 0, len = connectors.length; i < len; i++) {
                        var connector = connectors[i];
                        if (connector.canSelect() && rect.contains(connector._bounds))
                            collection.add(connector);
                    }
                    if (this.diagram._hasSelection()) {
                        this.diagram._clearSelection();
                    }
                    if (collection.length > 0) {
                        var children = collection;
                        var selection;
                        if (children.length > 1) {
                            var pseudoGroup = new ej.Diagram.PseudoGroup({ "name": "multipleSelection" });
                            for (i = 0, len = children.length; i < len; i++) {
                                pseudoGroup.children.add(children[i]);
                            }
                            this.diagram.model.nodes.add(pseudoGroup);
                            selection = pseudoGroup;
                        }
                        else {
                            selection = collection[0];
                        }
                        this.diagram._addSelection(selection);
                    }
                }
                else {
                    if (this.diagram._hasSelection() && !this.diagram._isDropped) {
                        this.diagram._clearSelection();
                    }
                }
            }
            else {

                if (this.selectedObject || this.diagram._hasSelection()) {
                    if (!evt.ctrlKey && !evt.shiftKey) {
                        if (this.diagram._hasSelection()) {
                            this.diagram._clearSelection();
                        }
                    }
                    this.diagram._addSelection(this.selectedObject);
                }
                else if (this.diagram._hasSelection() && !this.diagram._isDropped) {
                    this.diagram._clearSelection();
                }
            }
            this.diagram._isDropped = false;
            base.prototype.mouseup.call(this, evt);
        };
        SelectTool.prototype._endAction = function () {
            if (this._svgHelper) {
                this.diagram._adornerLayer.removeChild(this._svgHelper);
                this.selectedObject = null;
                this._svgHelper = null;
            }
            base.prototype._endAction.apply(this);
        };
        SelectTool.prototype._initHelper = function () {
            if (!this._svgHelper) {
                var g = this.diagram._svg.g();
                this.diagram._adornerLayer.appendChild(g);
                var scale = this.diagram._zoomPan.currentZoom;
                var selectionRect = this.diagram._svg.rect({
                    "id": "helper", "x": this.startPoint.x * scale, "y": this.startPoint.y * scale,
                    "fill": "transparent", "stroke": "gray", "stroke-dasharray": "2 2", "shape-rendering": "crispEdges"
                });
                g.appendChild(selectionRect);
                g.setAttribute("pointer-events", "none");
                this._svgHelper = g;
            }
        };
        SelectTool.prototype._updateHelper = function () {
            var helper = this.diagram._svg.getElementById("helper");
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var scale = this.diagram._zoomPan.currentZoom;
            var x = ((this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x) * scale;
            var y = ((this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y) * scale;

            if (helper) {
                this.diagram._svg.rect({
                    "id": helper.id, "x": x,
                    "y": y, "width": width * scale, "height": height * scale
                });
            }
        };
        SelectTool.prototype._checkGroupChildren = function (collection, group, bounds) {
            for (var i = 0; i < group.children.length; i++) {
                var child = group.children[i];
                if (bounds.contains(child._bounds)) {
                    collection.add(child);
                }
                else if (child.type === "group") {
                    this._checkGroupChildren(collection, child, bounds);
                }
            }
        }
        return SelectTool;
    })(Tool);

    ej.Diagram.SelectTool = SelectTool;
    //#endregion

    //#region MoveTool
    var MoveTool = (function (base) {
        ej.Diagram.extend(MoveTool, base);
        function MoveTool(diagram) {
            base.call(this, "move", diagram);
            this.cursor = "move";
            this._isMouseDown = false;
            this.diffx = 0;
            this.diffy = 0;
            this.undoObject = null;
        }
        MoveTool.prototype.mousedown = function (evt) {
            this.diagram._snapping._removeGuidelines();
            this.diffx = 0;
            this.diffy = 0;
            base.prototype.mousedown.call(this, evt);
            this._isMouseDown = true;
            this.updateCursor(this.cursor);
        };
        MoveTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            this.diagram._snapping._removeGuidelines();
            if (this._isMouseDown && !this.inAction && (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y)) {
                if (!this.selectedObject) {

                    if (this.diagram.selectionList.length > 0) {
                        var selectedObject = this._findNodeUnderMouse(evt, true);
                    }
                    else {
                        var selectedObject = this._findNodeUnderMouse(evt);
                    }
                    if (selectedObject) {
                        if (this.diagram.selectionList[0] && this.diagram.selectionList[0].type === "pseudoGroup") {
                            if (!(this.diagram.selectionList[0].children.contains(selectedObject))) {
                                this.selectedObject = selectedObject;
                                this.diagram._clearSelection();
                            }
                            else this.selectedObject = this.diagram.selectionList[0];
                        }
                        else {
                            if (this.diagram.selectionList[0] != selectedObject) {
                                this.diagram._clearSelection();
                                this.selectedObject = selectedObject;
                            }
                            else this.selectedObject = this.diagram.selectionList[0];
                        }
                        this.undoObject = ej.Diagram.Util.clone(this.selectedObject);
                    }
                }
            }
            if (this.selectedObject && this.selectedObject.canSelect()) {
                if (!this.inAction) {
                    //ej.Diagram.SvgContext._removePivotPoint(this.selectedObject, this.diagram._svg, this.diagram._zoomPan.currentZoom);
                    this.inAction = true;
                    this.updateCursor("move");
                    this._updateSelection();
                }
                this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                if (this.selectedObject.type == "connector") {
                    var connector = this.selectedObject;
                    if (connector.isTargetConnected() || connector.isSourceConnected()) {
                        this._disconnect(connector);
                    }
                    ej.Diagram.SvgContext.updateConnector(connector, this.diagram._svg);
                }
                else if (this.selectedObject.type === "node") {
                    ej.Diagram.SvgContext.updateNode(this.selectedObject, this.diagram._svg);
                }
                else if (this.selectedObject.type === "pseudoGroup") {
                    var child;
                    for (var i = 0, len = this.selectedObject.children.length; i < len; i++) {
                        child = this.selectedObject.children[i];
                        //child.parent = null;
                        if (child.type === "connector") {
                            ej.Diagram.SvgContext.updateConnector(child, this.diagram._svg);
                        }
                        else if (child.type === "node") {
                            ej.Diagram.SvgContext.updateNode(child, this.diagram._svg);
                        }
                        else if (child.type === "group") {
                            ej.Diagram.SvgContext.updateGroup(child, this.diagram._svg);
                        }
                        //child.parent = this.selectedObject;
                    }
                }
                else if (this.selectedObject.type === "group") {
                    ej.Diagram.SvgContext.updateGroup(this.selectedObject, this.diagram._svg);
                }

                this.diagram._renderTooltip(this.selectedObject);
                //this._updateSelection();
                this.diagram._updateSelectionHandle(true);
            }
            this.previousPoint = this.currentPoint;
        };
        MoveTool.prototype.mouseup = function (evt) {
            this._isMouseDown = false;
            this.diagram._snapping._removeGuidelines();
            var clickedObject;
            if (this.inAction) {
                this.inAction = false;
                this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                this._updateObject();
                var entry = new ej.Diagram.HistoryEntry(new ej.Diagram.MoveHandleCmd(this.undoObject, ej.Diagram.Util.clone(this.selectedObject), this.selectedObject.type === "pseudoGroup" ? true : false));
                this.diagram.historyManager.addHistoryEntry(entry);
                this.diagram._updateSelectionHandle();
                clickedObject = this.selectedObject;
            }
            else {
                if (!this.selectedObject) {
                    if (evt.which == 3) {
                        if (this.diagram.selectionList && !(this.diagram.selectionList[0].type === "pseudoGroup")) {
                            if (this.diagram.selectionList.length > 0) {
                                this.selectedObject = this._findNodeUnderMouse(evt, true);
                                clickedObject = this.selectedObject;
                            } else {
                                this.selectedObject = this._findNodeUnderMouse(evt);
                                clickedObject = this.selectedObject;
                            }
                            if (this.selectedObject && !this.diagram.selectionList.contains(this.selectedObject))
                                this.diagram._clearSelection();
                        }
                    }
                    else {
                        if (evt.ctrlKey || evt.shiftKey) {
                            clickedObject = this._processCtrlKey(evt);
                        }
                        else {
                            this.selectedObject = this._findNodeUnderMouse(evt);
                            clickedObject = this.selectedObject;
                        }
                        if (this.selectedObject && !this.diagram.selectionList.contains(this.selectedObject))
                            this.diagram._clearSelection();
                    }
                }
                else {
                    clickedObject = this._findNodeUnderMouse(evt);
                }
            }
            if (this.selectedObject) {
                this._updateSelection();
            }
            this.diagram._removeTooltip();
            this.diagram._endEdit();
            //raise click event
            if (clickedObject) {
                this._raiseEvent("click", { element: clickedObject });
            }
            base.prototype.mouseup.call(this, evt);
        };
        MoveTool.prototype._updateObject = function () {
            if (this.selectedObject.type === "connector") {
                var connector = this.selectedObject;
                if (!connector.isTargetConnected() && !connector.isSourceConnected()) {
                    ej.Diagram.SvgContext.updateConnector(connector, this.diagram._svg);
                }
            }
            else if (this.selectedObject.type === "node") {
                ej.Diagram.SvgContext.updateNode(this.selectedObject, this.diagram._svg);
            }
            else if (this.selectedObject.type === "pseudoGroup") {
                var child;
                for (var i = 0, len = this.selectedObject.children.length; i < len; i++) {
                    child = this.selectedObject.children[i];
                    //child.parent = null;
                    if (child.type === "connector") {
                        ej.Diagram.SvgContext.updateConnector(child, this.diagram._svg);
                    }
                    else if (child.type === "node") {
                        ej.Diagram.SvgContext.updateNode(child, this.diagram._svg);
                    }
                    else if (child.type === "group") {
                        ej.Diagram.SvgContext.updateGroup(child, this.diagram._svg);
                    }
                    //child.parent = this.selectedObject;
                }
            }
            else if (this.selectedObject.type === "group") {
                ej.Diagram.SvgContext.updateGroup(this.selectedObject, this.diagram._svg);
            }
        };
        MoveTool.prototype._endAction = function () {
            base.prototype._endAction.apply(this);
            this._isMouseDown = false;
            this._mouseOffset = new ej.Diagram.Size(0, 0);
        };
        MoveTool.prototype._processCtrlKey = function (evt) {
            var obj;
            if (this.diagram.selectionList.length > 0) {
                var selectionList = this.diagram.selectionList;
                var isMultipleSelection;
                if (selectionList[0].type === "pseudoGroup") {
                    isMultipleSelection = true;
                    obj = this._findNodeUnderMouse(evt);
                }
                else {
                    obj = this._findNodeUnderMouse(evt, true);
                }
                if (!selectionList.contains(obj)) {
                    if (isMultipleSelection) {
                        var mulSelection = selectionList[0];
                        if (!mulSelection.children.contains(obj)) {
                            mulSelection.children.add(obj);
                            this.selectedObject = selectionList[0];
                            if (this.diagram._hasSelection()) {
                                this.diagram._clearSelection();
                            }
                            this.diagram._addSelection(this.selectedObject);
                        }
                        else {
                            mulSelection.children.remove(obj);
                            if (mulSelection.children.length == 1) {
                                var node = mulSelection.children[0];
                                if (this.diagram._hasSelection()) {
                                    this.diagram._clearSelection();
                                }
                                this.selectedObject = node;
                            }
                            else {
                                this.selectedObject = selectionList[0];
                                if (this.diagram._hasSelection()) {
                                    this.diagram._clearSelection();
                                }
                            }
                            this.diagram._addSelection(this.selectedObject);
                        }
                    }
                    else {
                        var pseudoGroup = new ej.Diagram.PseudoGroup({ "name": "multipleSelection" });
                        pseudoGroup.children.add(this.diagram.selectionList[0]);
                        pseudoGroup.children.add(obj);
                        this.diagram.model.nodes.add(pseudoGroup);
                        if (this.diagram._hasSelection()) {
                            this.diagram._clearSelection();
                        }
                        this.selectedObject = pseudoGroup;
                        this.diagram._addSelection(this.selectedObject);
                    }
                }
                else {
                    this.selectedObject = obj;
                }
            }
            else {
                obj = this._findNodeUnderMouse(evt);
                this.selectedObject = obj;
            }
            return obj;
        };
        MoveTool.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                this.helper.updatebounds();

                if (this.helper.type === "connector")
                    this.svgHelper = ej.Diagram.SvgContext.renderConnector(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                else if (this.helper.type === "node")
                    this.svgHelper = ej.Diagram.SvgContext.renderNode(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                else if (this.helper.type === "group")
                    this.svgHelper = ej.Diagram.SvgContext.renderGroup(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper.setAttribute("pointer-events", "none");
            }
        };
        MoveTool.prototype._updateHelper = function () {
            this._updateXY(this.helper, this.previousPoint, this.currentPoint);
            if (this.helper.type === "connector")
                ej.Diagram.SvgContext.updateConnector(this.helper, this.diagram._svg);
            else if (this.helper.type === "node")
                ej.Diagram.SvgContext.updateNode(this.helper, this.diagram._svg);
            else if (this.helper.type === "group")
                ej.Diagram.SvgContext.updateGroup(this.helper, this.diagram._svg);
        };
        MoveTool.prototype._updateXY = function (shape, startPoint, endPoint) {
            var offsetwithoutsnap = new ej.Diagram.Point(endPoint.x - startPoint.x, endPoint.y - startPoint.y);
            //if (this.diagram.model.snap & ej.Diagram.Snap.Grid) {
            //    if (offset.x != 0 && this._mouseOffset.width != 0) {
            //        offset.x -= this._mouseOffset.width;
            //        this._mouseOffset.width = 0;
            //    }
            //    if (offset.y != 0 && this._mouseOffset.height != 0) {
            //        offset.y -= this._mouseOffset.height;
            //        this._mouseOffset.height = 0;
            //    }
            //}s
            var towardsLeft = endPoint.x < startPoint.x;
            var towardsTop = endPoint.y < startPoint.y;
            var difx = this.diffx + (endPoint.x - startPoint.x);
            var dify = this.diffy + (endPoint.y - startPoint.y);
            var offset;

            if ((this.diagram.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToHorizontalLines) ||
                (this.diagram.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToVerticalLines) || this.diagram.model.snapSettings.snapToObject)
                offset = this.diagram._snapping._snapPosition(this.selectedObject, towardsLeft, towardsTop, new ej.Diagram.Point(difx, dify), endPoint, startPoint);

            if (!offset)
                offset = new ej.Diagram.Point(difx, dify);

            this.diffx = difx - offset.x;
            this.diffy = dify - offset.y;
            var type = "Dragging";
            if (!this.inAction) {
                type = "Dragged";
            }
            if (this.selectedObject.type === "connector") {
                type = "connector" + type;
            }
            else {
                type = "node" + type;
            }
            if (!offset.isEmpty()) {
                var args = this._raiseEvent(type, { element: this.selectedObject, offset: offset, cancel: false });
                if (!args.cancel) {
                    shape._translate(offset.x, offset.y, offsetwithoutsnap);
                }
            }
            else if (!this.inAction) {
                var args = this._raiseEvent(type, { element: this.selectedObject, offset: offset, cancel: false });
            }
        };
        MoveTool.prototype._updateSelection = function () {
            if (!this.diagram.selectionList.contains(this.selectedObject)) {
                this.selectedObject.updatebounds();
                this.diagram._addSelection(this.selectedObject);
            }
        };
        return MoveTool;
    })(Tool);

    ej.Diagram.MoveTool = MoveTool;
    //#endregion

    //#region ResizeTool
    var ResizeTool = (function (base) {
        ej.Diagram.extend(ResizeTool, base);
        function ResizeTool(diagram) {
            base.call(this, "resize", diagram);
            this.cursor = "default";//update resize cursor during action i.e in action
            this._resizeDirection = null;
            this._mouseOffset = new ej.Diagram.Size(0, 0);
            this.diffx = 0;
            this.diffy = 0;
            this.undoObject = null;
        }
        ResizeTool.prototype.mousedown = function (evt) {
            this.diagram._snapping._removeGuidelines();
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.selectionList[0];
            this.undoObject = ej.Diagram.Util.clone(this.selectedObject);
            this._resizeDirection = evt.target.getAttribute("class");
            this.diffx = 0;
            this.diffy = 0;
        };
        ResizeTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            this.diagram._snapping._removeGuidelines();
            if (this.selectedObject) {
                if (!this.inAction) {
                    ej.Diagram.SvgContext._removePivotPoint(this.selectedObject, this.diagram._svg, this.diagram._zoomPan.currentZoom);
                    this.inAction = true;
                }
                this._updateSize(this.selectedObject, this.previousPoint, this.currentPoint);
                if (this.selectedObject.type === "node")
                    ej.Diagram.SvgContext.updateNode(this.selectedObject, this.diagram._svg, this.diagram._diagramLayer);
                else
                    ej.Diagram.SvgContext.updateGroup(this.selectedObject, this.diagram._svg, this.diagram._diagramLayer);
                this.diagram._renderTooltip(this.selectedObject);
                this.diagram._updateSelectionHandle(true);
            }

            this.previousPoint = this.currentPoint;
        };
        ResizeTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.diagram._snapping._removeGuidelines();
                this.inAction = false;
                this._updateSize(this.selectedObject, this.previousPoint, this.currentPoint);
                if (this.selectedObject.type === "node")
                    ej.Diagram.SvgContext.updateNode(this.selectedObject, this.diagram._svg, this.diagram._diagramLayer);
                else
                    ej.Diagram.SvgContext.updateGroup(this.selectedObject, this.diagram._svg, this.diagram._diagramLayer);
                this.diagram._updateSelectionHandle();
                var entry = new ej.Diagram.HistoryEntry(new ej.Diagram.ResizeHandleCmd(this.undoObject, ej.Diagram.Util.clone(this.selectedObject), this.selectedObject.type === "pseudoGroup" ? true : false));
                this.diagram.historyManager.addHistoryEntry(entry);
                this.diagram._removeTooltip();
            }
            base.prototype.mouseup.call(this, evt);
        };
        ResizeTool.prototype._updateSize = function (shape, startPoint, endPoint, x1, y1) {
            var mat = new ej.Diagram.Matrix();
            var _matrix = mat._createIdentity();
            _matrix._rotate(-shape.rotateAngle);
            var horizontalsnap = { snapped: false, offset: 0, left: false, right: false };
            var verticalsnap = { snapped: false, offset: 0, top: false, bottom: false };
            var difx = (x1 ? x1 : (endPoint.x - startPoint.x)) + this.diffx;
            var dify = (y1 ? y1 : (endPoint.y - startPoint.y)) + this.diffy;
            var dely = dify;
            var delx = difx;
            this.diffx = 0;
            this.diffy = 0;
            var pivot = new ej.Diagram.Point(0, 0);
            var trans = new ej.Diagram.Matrix()._createIdentity();
            var pt = shape.pivot;
            var x = shape.offsetX;
            var y = shape.offsetY;
            trans._rotateAt(shape.rotateAngle, x, y);
            var w = shape.width;
            var h = shape.height;
            x = x - w * pt.x;
            y = y - h * pt.y;
            switch (this._resizeDirection) {
                case "n-resize":
                    pivot = trans._transform(new ej.Diagram.Point(x + w / 2, y + h));
                    var deltaWidth = 1;
                    dify = this._snapTop(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    var diff = _matrix._transform(new ej.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    var deltaHeight = (shape.height - dify) / shape.height;
                    break;
                case "e-resize":
                    pivot = trans._transform(new ej.Diagram.Point(x, y + h / 2));
                    difx = this._snapRight(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    var diff = _matrix._transform(new ej.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    var deltaWidth = (shape.width + difx) / shape.width;
                    var deltaHeight = 1;
                    break;
                case "w-resize":
                    pivot = trans._transform(new ej.Diagram.Point(x + w, y + h / 2));
                    difx = this._snapLeft(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    var diff = _matrix._transform(new ej.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    var deltaWidth = (shape.width - difx) / shape.width;
                    var deltaHeight = 1;
                    break;
                case "s-resize":
                    pivot = trans._transform(new ej.Diagram.Point(x + w / 2, y));
                    var deltaWidth = 1;
                    dify = this._snapBottom(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    var diff = _matrix._transform(new ej.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    var deltaHeight = (shape.height + dify) / shape.height;
                    break;
                case "ne-resize":
                    dify = this._snapTop(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    pivot = trans._transform(new ej.Diagram.Point(x, y + h));
                    difx = this._snapRight(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    var diff = _matrix._transform(new ej.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    var deltaWidth = (shape.width + difx) / shape.width;
                    var deltaHeight = (shape.height - dify) / shape.height;
                    break;
                case "nw-resize":
                    dify = this._snapTop(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    pivot = trans._transform(new ej.Diagram.Point(x + w, y + h));
                    difx = this._snapLeft(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    var diff = _matrix._transform(new ej.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    var deltaWidth = (shape.width - difx) / shape.width;
                    var deltaHeight = (shape.height - dify) / shape.height;
                    break;
                case "se-resize":
                    pivot = trans._transform(new ej.Diagram.Point(x, y));
                    dify = this._snapBottom(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    difx = this._snapRight(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    var diff = _matrix._transform(new ej.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    var deltaWidth = (shape.width + difx) / shape.width;
                    var deltaHeight = (shape.height + dify) / shape.height;
                    break;
                case "sw-resize":
                    pivot = trans._transform(new ej.Diagram.Point(x + w, y));
                    dify = this._snapBottom(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    difx = this._snapLeft(horizontalsnap, verticalsnap, delx, dely, shape, endPoint == startPoint);
                    var diff = _matrix._transform(new ej.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    var deltaWidth = (shape.width - difx) / shape.width;
                    var deltaHeight = (shape.height + dify) / shape.height;
                    break;
            }
            var size = new ej.Diagram.Size();
            size.width = deltaWidth;
            size.height = deltaHeight;
            mat = new ej.Diagram.Matrix();
            shape._scale(mat, deltaWidth, deltaHeight, pivot);
            if (!size.isEmpty()) {
                var cancel = false;
                var args;
                if (!this.inAction) {
                    var args = this._raiseEvent("nodeSizeChanged", { element: this.selectedObject, offset: size, cancel: false });
                    cancel = args.cancel;
                }
                else {
                    var args = this._raiseEvent("nodeSizeChanging", { element: this.selectedObject, offset: size, cancel: false });
                    cancel = args.cancel;
                }
            }
        };
        ResizeTool.prototype._snapTop = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var dify = dely;
            verticalsnap.top = true;
            if (this.diagram.model.snapSettings.snapToObject)
                this.diagram._snapping._snapToObject_size(horizontalsnap, verticalsnap, delx, dely, this.selectedObject, ended);
            if (!verticalsnap.snapped) {
                if (this.diagram.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToVerticalLines) {
                    var actualTop = shape._corners._top.y + dely;
                    var roundedTop = this.diagram._snapping._round(actualTop, this.diagram.model.snapSettings.verticalGridlines._snapinterval);
                    dify = roundedTop - shape._corners._top.y;
                    this.diffy = actualTop - roundedTop;
                }
            }
            else {
                this.diffy = dify - verticalsnap.offset;
                dify = verticalsnap.offset;
            }
            return dify;
        };
        ResizeTool.prototype._snapBottom = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var dify = dely;
            verticalsnap.bottom = true;
            if (this.diagram.model.snapSettings.snapToObject)
                this.diagram._snapping._snapToObject_size(horizontalsnap, verticalsnap, delx, dely, this.selectedObject, ended);;
            if (!verticalsnap.snapped) {
                if (this.diagram.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToVerticalLines) {
                    var actualBottom = shape._corners._bottom.y + dely;
                    var roundedBottom = this.diagram._snapping._round(actualBottom, this.diagram.model.snapSettings.verticalGridlines._snapinterval);
                    dify = roundedBottom - shape._corners._bottom.y;
                    this.diffy = actualBottom - roundedBottom;
                }
            }
            else {
                this.diffy = dify - verticalsnap.offset;
                dify = verticalsnap.offset;
            }
            return dify;
        };
        ResizeTool.prototype._snapLeft = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var difx = delx;
            horizontalsnap.left = true;
            if (this.diagram.model.snapSettings.snapToObject)
                this.diagram._snapping._snapToObject_size(horizontalsnap, verticalsnap, delx, dely, this.selectedObject, ended);;
            if (!horizontalsnap.snapped) {
                if (this.diagram.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToHorizontalLines) {

                    var actualLeft = shape._corners._left.x + delx;
                    var roundedLeft = this.diagram._snapping._round(actualLeft, this.diagram.model.snapSettings.horizontalGridlines._snapinterval);
                    difx = roundedLeft - shape._corners._left.x;
                    this.diffx = actualLeft - roundedLeft;
                }
            }
            else {
                this.diffx = difx - horizontalsnap.offset;
                difx = horizontalsnap.offset;
            }
            return difx;
        };
        ResizeTool.prototype._snapRight = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var difx = delx;
            horizontalsnap.right = true;
            if (this.diagram.model.snapSettings.snapToObject)
                this.diagram._snapping._snapToObject_size(horizontalsnap, verticalsnap, delx, dely, this.selectedObject, ended);;
            if (!horizontalsnap.snapped) {
                if (this.diagram.model.snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToHorizontalLines) {
                    var actualRight = shape._corners._right.x + delx;
                    var roundedRight = this.diagram._snapping._round(actualRight, this.diagram.model.snapSettings.horizontalGridlines._snapinterval);
                    difx = roundedRight - shape._corners._right.x;
                    this.diffx = actualRight - roundedRight;
                }
            }
            else {
                this.diffx = difx - horizontalsnap.offset;
                difx = horizontalsnap.offset;
            }
            return difx;
        };
        return ResizeTool;
    })(Tool);

    ej.Diagram.ResizeTool = ResizeTool;
    //#endregion

    //#region EndPointTool
    var EndPointTool = (function (base) {
        ej.Diagram.extend(EndPointTool, base);
        function EndPointTool(diagram) {
            base.call(this, "endPoint", diagram);
            this.cursor = "move";
            this._endPoint = null;
            this._undoObject = null;
        }
        EndPointTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.selectionList[0];
            this._undoObject = this._getClonedObject(this.selectedObject);
            this._endPoint = evt.target.getAttribute("class");
        };
        EndPointTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this.selectedObject) {
                if (!this.inAction && (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y)) {
                    this.inAction = true;
                    $(this.diagram._svg.getElementById(this.selectedObject.name)).attr("pointer-events", "none");
                    $(this.diagram._svg.getElementById(this.selectedObject.name + "_line_hitTest")).attr("pointer-events", "none");
                    $("#" + this.diagram._svg.document.id + "handle_g").attr("pointer-events", "none");
                    //this._initHelper();
                    //this._renderHelper();
                }
                else {
                    //this._updateHelper();
                    if (this._endPoint == "targetEndPoint" || this._endPoint == "sourceEndPoint")
                        this._checkConnectionPossible(evt);
                    this._updatePoints(this.selectedObject);
                    this._disconnect(this.selectedObject);
                    this._updateConnection(this.selectedObject);
                    ej.Diagram.SvgContext.updateConnector(this.selectedObject, this.diagram._svg);
                    this.diagram._updateSelectionHandle(true);
                }
            }
            this.previousPoint = this.currentPoint;
        };
        EndPointTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.inAction = false;
                this._updatePoints(this.selectedObject);
                this._disconnect(this.selectedObject);
                var args;
                if (this._currentPossibleConnection && !this.inAction) {
                    this._updateConnection(this.selectedObject);
                }
                else {
                    if (this.selectedObject.sourceNode || this.selectedObject.targetNode) {
                        this._updateConnection(this.selectedObject);
                    }
                }
                ej.Diagram.SvgContext.updateConnector(this.selectedObject, this.diagram._svg);
                var entry = new ej.Diagram.HistoryEntry(new ej.Diagram.EndPointHandleCmd(this._undoObject, this._getClonedObject(this.selectedObject)));
                this.diagram.historyManager.addHistoryEntry(entry);
                this.diagram._updateSelectionHandle();
            }
            $(this.diagram._svg.getElementById(this.selectedObject.name)).removeAttr("pointer-events");
            $(this.diagram._svg.getElementById(this.selectedObject.name + "_line_hitTest")).attr("pointer-events", "stroke");
            $("#" + this.diagram._svg.document.id + "handle_g").removeAttr("pointer-events");
            base.prototype.mouseup.call(this, evt);
        };
        EndPointTool.prototype._getClonedObject = function (connector) {
            var clonedObject = ej.Diagram.Util.clone(connector);
            clonedObject.targetNode = connector.targetNode;
            clonedObject.sourceNode = connector.sourceNode;
            clonedObject.targetPort = connector.targetPort;
            clonedObject.sourcePort = connector.sourcePort;
            return clonedObject;
        };
        EndPointTool.prototype._updateConnection = function (connector) {
            var args;
            if (this._currentPossibleConnection) {
                if (this._endPoint == "targetEndPoint") {
                    if (this._possibleConnectionPort && this._possibleConnectionPort.canConnect()) {
                        args = this._raiseEvent("connectionChanging", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort, cancel: false });
                        if (!args.cancel) {
                            connector.targetNode = this._currentPossibleConnection;
                            connector.targetPort = this._possibleConnectionPort;
                            this._currentPossibleConnection.inEdges.add(connector);
                            this._raiseEvent("connectionChanged", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort });
                        }
                    }
                    else if (this._currentPossibleConnection.canConnect()) {
                        args = this._raiseEvent("connectionChanging", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort, cancel: false });
                        if (!args.cancel) {
                            connector.targetNode = this._currentPossibleConnection;
                            this._currentPossibleConnection.inEdges.add(connector);
                            this._raiseEvent("connectionChanged", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort });
                        }
                    }
                }
                else {
                    if (this._possibleConnectionPort && this._possibleConnectionPort.canConnect()) {
                        args = this._raiseEvent("connectionChanging", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort, cancel: false });
                        if (!args.cancel) {
                            connector.sourceNode = this._currentPossibleConnection;
                            connector.sourcePort = this._possibleConnectionPort;
                            this._currentPossibleConnection.outEdges.add(connector);
                            this._raiseEvent("connectionChanged", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort });
                        }
                    }
                    else if (this._currentPossibleConnection.canConnect()) {
                        args = this._raiseEvent("connectionChanging", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort, cancel: false });
                        if (!args.cancel) {
                            connector.sourceNode = this._currentPossibleConnection;
                            this._currentPossibleConnection.outEdges.add(connector);
                            this._raiseEvent("connectionChanged", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort });
                        }
                    }

                }
            }
            connector.dock();
        };
        EndPointTool.prototype._disconnect = function (connector) {
            var args;
            if (this._endPoint == "targetEndPoint" && connector.targetNode != null) {
                args = this._raiseEvent("connectionChanging", { element: connector, endPoint: this._endPoint, connection: connector.targetNode, port: connector.targetPort, cancel: false });
                if (!args.cancel) {
                    connector.targetNode.inEdges.remove(connector);
                    connector.targetNode = null;
                    connector.targetPort = null;
                    this._raiseEvent("connectionChanged", { element: connector, endPoint: this._endPoint, connection: connector.targetNode, port: connector.targetPort });
                }

            }
            if ((this._endPoint == "sourceEndPoint") && connector.sourceNode != null) {
                args = this._raiseEvent("connectionChanging", { element: connector, endPoint: this._endPoint, connection: connector.sourceNode, port: connector.sourcePort, cancel: false });
                if (!args.cancel) {
                    connector.sourceNode.outEdges.remove(connector);
                    connector.sourceNode = null;
                    connector.sourcePort = null;
                    this._raiseEvent("connectionChanged", { element: connector, endPoint: this._endPoint, connection: connector.sourceNode, port: connector.sourcePort });
                }
            }
        };
        EndPointTool.prototype._initHelper = function () {
            base.prototype._initHelper.call(this);
            this.helper.targetNode = this.selectedObject.targetNode;
            this.helper.targetPort = this.selectedObject.targetPort;
            this.helper.sourceNode = this.selectedObject.sourceNode;
            this.helper.sourcePort = this.selectedObject.sourcePort;
        };
        EndPointTool.prototype._endAction = function () {
            this._showPorts();
            var possibleConnection;
            if (this._currentPossibleConnection && !this._possibleConnectionPort) {
                possibleConnection = this._currentPossibleConnection;
            }
            base.prototype._endAction.apply(this);
            if (this._endPoint) {
                this._endPoint = null;
                if (possibleConnection) {
                    this.diagram._checkToolToActivate(possibleConnection, this.currentPoint);
                    this.diagram._updateCursor();
                }
            }
        };
        EndPointTool.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                this.svgHelper = ej.Diagram.SvgContext.renderConnector(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper.setAttribute("pointer-events", "none");
            }
        };
        EndPointTool.prototype._updateHelper = function () {
            //this._updatePoints(this.helper);
            //this._disconnect(this.helper);
            //this._updateConnection(this.helper);
            //ej.Diagram.SvgContext.updateConnector(this.helper, this.diagram._svg);
        };
        EndPointTool.prototype._updatePoints = function (connector) {
            var isTarget = (this._endPoint == "targetEndPoint") ? true : false;
            var isSource = (this._endPoint == "sourceEndPoint") ? true : false;
            var endPoint;
            if (this._endPoint) {
                switch (this._endPoint) {
                    case "targetEndPoint":
                        endPoint = connector.line.endPoint;
                        break;
                    case "sourceEndPoint":
                        endPoint = connector.line.startPoint;
                        break;
                    case "bezierpoint1":
                        endPoint = connector.line._point1;
                        break;
                    case "bezierpoint2":
                        endPoint = connector.line._point2;
                        break;
                }

                var ptCur = this.currentPoint;
                if (this._possibleConnectionPort == null) {
                    ptCur = this.snap(ptCur);
                }
                var offset = new ej.Diagram.Point(endPoint.x - ptCur.x, endPoint.y - ptCur.y);
                if (!offset.isEmpty()) {
                    var cancel = false;
                    var args;
                    var type;
                    if (!this.inAction) {
                        if (isTarget) {
                            type = "target";
                        }
                        else if (isSource) {
                            type = "source";
                        }
                        args = this._raiseEvent("connectorEndDragged", { end: type, element: this.selectedObject, cancel: false });
                        cancel = args.cancel;
                    }
                    // var args = this._raiseEvent(type, { element: this.selectedObject, cancel: false });
                    if (!cancel) {
                        if (isTarget || isSource)
                            connector._setEndPoint(new ej.Diagram.Point(ptCur.x, ptCur.y), isTarget, true);
                        else
                            connector.line._setBezierPoint(new ej.Diagram.Point(ptCur.x, ptCur.y), this._endPoint);
                    }
                }
            }
        };
        return EndPointTool;
    })(Tool);

    ej.Diagram.EndPointTool = EndPointTool;
    //#endregion

    //#region LineTool
    var LineTool = (function (base) {
        ej.Diagram.extend(LineTool, base);
        function LineTool(name, diagram) {
            base.call(this, name, diagram);
            this.cursor = "crosshair";
            this._isMouseDown = false;
            this._targetPossibleConnection = null;
            this._sourcePossibleConnection = null;
            this._targetPort = null;
            this._sourcePort = null;
        }
        LineTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this._isMouseDown = true;
            if (this.diagram._hasSelection()) {
                this.diagram._clearSelection();
            }
            if (this._currentPossibleConnection) {
                this._sourcePossibleConnection = this._currentPossibleConnection;
                this._sourcePort = this._possibleConnectionPort;
            }
        };
        LineTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            this._checkConnectionPossible(evt);
            if (this._isMouseDown) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                    this._renderHelper();
                }
                else {
                    this._targetPossibleConnection = this._currentPossibleConnection;
                    this._targetPort = this._possibleConnectionPort;
                    this._updateHelper();
                }
            }
            this.previousPoint = this.currentPoint;
        };
        LineTool.prototype.mouseup = function (evt) {
            this._isMouseDown = false;
            if (this.inAction) {
                this.inAction = false;
                var ptStart = this._getStartPoint();
                var ptEnd = this._getCurrentPoint();
                var distance = ej.Diagram.Geometry.distance(ptStart, ptEnd);
                if (distance > 0) {
                    var connector = this.createConnector(ptStart, ptEnd);
                    connector.name = "connector" + ej.Diagram.Util.randomId();
                    var args;
                    if (this._sourcePossibleConnection) {
                        if (this._sourcePort && this._sourcePort.canConnect()) {
                            args = this._raiseEvent("connectionChanging", { element: connector, connection: this._sourcePossibleConnection, port: this._sourcePort, cancel: false });
                            if (!args.cancel) {
                                connector.sourceNode = this._sourcePossibleConnection;
                                connector.sourcePort = this._sourcePort;
                                this._sourcePossibleConnection.outEdges.remove(this.helper);
                                this._sourcePossibleConnection.outEdges.add(connector);
                                this._raiseEvent("connectionChanged", { element: connector, connection: this._sourcePossibleConnection, port: this._sourcePort });
                            }
                        }
                        else if (this._sourcePossibleConnection.canConnect()) {
                            args = this._raiseEvent("connectionChanging", { element: connector, connection: this._sourcePossibleConnection, port: this._sourcePort, cancel: false });
                            if (!args.cancel) {
                                connector.sourceNode = this._sourcePossibleConnection;
                                this._sourcePossibleConnection.outEdges.remove(this.helper);
                                this._sourcePossibleConnection.outEdges.add(connector);
                                this._raiseEvent("connectionChanged", { element: connector, connection: this._sourcePossibleConnection, port: this._sourcePort });
                            }
                        }
                    }
                    if (this._targetPossibleConnection) {
                        if (this._targetPort && this._targetPort.canConnect()) {
                            args = this._raiseEvent("connectionChanging", { element: connector, connection: this._targetPossibleConnection, port: this._targetPort, cancel: false });
                            if (!args.cancel) {
                                connector.targetNode = this._targetPossibleConnection;
                                connector.targetPort = this._targetPort;
                                this._targetPossibleConnection.inEdges.remove(this.helper);
                                this._targetPossibleConnection.inEdges.add(connector);
                                this._raiseEvent("connectionChanged", { element: connector, connection: this._targetPossibleConnection, port: this._targetPort });
                            }
                        }
                        if (this._targetPossibleConnection.canConnect()) {
                            args = this._raiseEvent("connectionChanging", { element: connector, connection: this._targetPossibleConnection, port: this._targetPort, cancel: false });
                            if (!args.cancel) {
                                connector.targetNode = this._targetPossibleConnection;
                                this._targetPossibleConnection.inEdges.remove(this.helper);
                                this._targetPossibleConnection.inEdges.add(connector);
                                this._raiseEvent("connectionChanged", { element: connector, connection: this._targetPossibleConnection, port: this._targetPort });
                            }
                        }

                    }
                    this.diagram.add(connector);
                }
            }
            if (this.singleAction)
                this.diagram.deActivateTool();
            if (this._prevTool)
                this._prevTool.inAction = false;
            this._showPorts();
            base.prototype.mouseup.call(this, evt);
        };

        LineTool.prototype._getStartPoint = function () {
            var ptStart;
            if (this._sourcePossibleConnection && this._sourcePort) {
                ptStart = this._sourcePort._getPosition(this._sourcePossibleConnection.getBoundingBox());
            }
            else {
                ptStart = this.snap(this.startPoint);
            }
            return ptStart;
        };
        LineTool.prototype._getCurrentPoint = function () {
            var ptCur;
            if (this._targetPossibleConnection && this._targetPort) {
                ptCur = this._targetPort._getPosition(this._targetPossibleConnection.getBoundingBox());
            }
            else {
                ptCur = this.snap(this.currentPoint);
            }
            return ptCur;
        };
        LineTool.prototype._initHelper = function () {
            this.helper = this.createConnector(new ej.Diagram.Point(0, 0), new ej.Diagram.Point(0, 0));
            this.helper.name = "helper";
        };
        LineTool.prototype.createConnector = function (startPoint, endPoint) {
            return null;
        };
        LineTool.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                this.svgHelper = ej.Diagram.SvgContext.renderConnector(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper.setAttribute("pointer-events", "none");
                $(this.diagram._svg.getElementById(this.svgHelper.id)).attr("pointer-events", "none");
                $(this.diagram._svg.getElementById(this.svgHelper.id + "_line_hitTest")).attr("pointer-events", "none");
            }
        };
        LineTool.prototype._updateHelper = function () {
            var points = new ej.Diagram.Collection();
            var ptStart = this._getStartPoint();
            var ptEnd = this._getCurrentPoint();
            points.add(ptStart);
            points.add(ptEnd);
            this.helper._setEndPoint(ptStart, false);
            this.helper._setEndPoint(ptEnd, true);
            this._disconnect(this.helper);
            if (this._sourcePossibleConnection) {
                if (this._sourcePort && this._sourcePort.canConnect()) {
                    this.helper.sourceNode = this._sourcePossibleConnection;
                    this.helper.sourcePort = this._sourcePort;
                    this._sourcePossibleConnection.outEdges.add(this.helper);
                }
                else if (this._sourcePossibleConnection.canConnect()) {
                    this.helper.sourceNode = this._sourcePossibleConnection;
                    this._sourcePossibleConnection.outEdges.add(this.helper);
                }
            }
            if (this._targetPossibleConnection) {
                if (this._targetPort && this._targetPort.canConnect()) {
                    this.helper.targetNode = this._targetPossibleConnection;
                    this.helper.targetPort = this._targetPort;
                    this._targetPossibleConnection.inEdges.add(this.helper);
                }
                else if (this._targetPossibleConnection.canConnect()) {
                    this.helper.targetNode = this._targetPossibleConnection;
                    this._targetPossibleConnection.inEdges.add(this.helper);
                }
            }
            this.helper.dock();
            ej.Diagram.SvgContext.updateConnector(this.helper, this.diagram._svg);
        };
        LineTool.prototype._endAction = function () {
            base.prototype._endAction.apply(this);
            this._targetPossibleConnection = null;
            this._sourcePossibleConnection = null;
            this._targetPort = null;
            this._sourcePort = null;
            this._isMouseDown = false;
        };
        LineTool.prototype._showPorts = function (node) {
        };
        LineTool.prototype._showAllPorts = function (hide) {
            base.prototype._showAllPorts.call(this, hide);
        };
        return LineTool;
    })(Tool);

    ej.Diagram.LineTool = LineTool;
    //#endregion

    //#region StraightLineTool
    var StraightLineTool = (function (base) {
        ej.Diagram.extend(StraightLineTool, base);
        function StraightLineTool(diagram) {
            base.call(this, "straightLine", diagram);
        }
        StraightLineTool.prototype.createConnector = function (startPoint, endPoint) {
            var connector = new ej.Diagram.Connector();
            connector.line = new ej.Diagram.Lines.Straight({ "startPoint": startPoint, "endPoint": endPoint });
            return connector;
        };
        return StraightLineTool;
    })(LineTool);

    ej.Diagram.StraightLineTool = StraightLineTool;
    //#endregion

    //#region OrthogonalLineTool
    var OrthogonalLineTool = (function (base) {
        ej.Diagram.extend(OrthogonalLineTool, base);
        function OrthogonalLineTool(diagram) {
            base.call(this, "orthogonalLine", diagram);
        }
        OrthogonalLineTool.prototype.createConnector = function (startPoint, endPoint) {
            var connector = new ej.Diagram.Connector();
            connector.line = new ej.Diagram.Lines.Orthogonal({ "startPoint": startPoint, "endPoint": endPoint });
            return connector;
        };
        return OrthogonalLineTool;
    })(LineTool);

    ej.Diagram.OrthogonalLineTool = OrthogonalLineTool;
    //#endregion

    //#region RotateTool
    var RotateTool = (function (base) {
        ej.Diagram.extend(RotateTool, base);
        function RotateTool(diagram) {
            base.call(this, "rotatetool", diagram);
            this._mouseOffset = new ej.Diagram.Size(0, 0);
            this.undoObject = null;
        }
        RotateTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.selectionList[0];
            this.undoObject = ej.Diagram.Util.clone(this.selectedObject);
            var scale = this.diagram._zoomPan.currentZoom;
            //ej.Diagram.SvgContext.renderPivotPoint(this.selectedObject, this.diagram._zoomPan.currentZoom, this.diagram._svg);

        };
        RotateTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);

            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                }
                else {
                    this._rotate(this.selectedObject);
                    if (this.selectedObject.type === "node")
                        ej.Diagram.SvgContext.updateNode(this.selectedObject, this.diagram._svg);
                    else
                        ej.Diagram.SvgContext.updateGroup(this.selectedObject, this.diagram._svg);
                    this.diagram._updateSelectionHandle(true);
                    this.diagram._renderTooltip(this.selectedObject);
                }
            }
            this.previousPoint = this.currentPoint;
        };
        RotateTool.prototype._rotate = function (node) {
            var cancel = false;
            var args;
            if (!this.inAction) {
                args = this._raiseEvent("nodeRotationChanged", { element: node, cancel: false });
                cancel = args.cancel;
            }
            if (!cancel) {
                var pt = new ej.Diagram.Point(node.offsetX, node.offsetY);
                var crntpoint = this.currentPoint;
                //var scale = this.diagram._zoomPan.currentZoom;
                //var hoffset = this.diagram._scroller._horizontalOffset;
                //var voffset = this.diagram._scroller._verticalOffset;
                var angle = ej.Diagram.Geometry.findAngle(new ej.Diagram.Point(pt.x, pt.y), crntpoint) + 90;
                angle = this.diagram._snapping._snapAngle(angle);
                node._rotate(angle - node.rotateAngle);
            }
        };
        RotateTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.inAction = false;
                this._rotate(this.selectedObject);
                if (this.selectedObject.type === "node")
                    ej.Diagram.SvgContext.updateNode(this.selectedObject, this.diagram._svg);
                else
                    ej.Diagram.SvgContext.updateGroup(this.selectedObject, this.diagram._svg);
                var entry = new ej.Diagram.HistoryEntry(new ej.Diagram.RotationHandleCmd(this.undoObject, ej.Diagram.Util.clone(this.selectedObject), this.selectedObject.type === "pseudoGroup" ? true : false));
                this.diagram.historyManager.addHistoryEntry(entry);
                this.diagram._updateSelectionHandle();
                this.diagram._removeTooltip();
            }
            //this.diagram._removeTooltip();
            base.prototype.mouseup.call(this, evt);
        };
        return RotateTool;
    })(Tool);

    ej.Diagram.RotateTool = RotateTool;
    //#endregion 

    //#region PivotTool
    var PivotTool = (function (base) {
        ej.Diagram.extend(PivotTool, base);
        function PivotTool(diagram) {
            base.call(this, "PivotTool", diagram);
            this._mouseOffset = new ej.Diagram.Size(0, 0);
        }
        PivotTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.selectionList[0];
        };
        PivotTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);

            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                }
                else {
                    var delx = this.currentPoint.x - this.previousPoint.x;
                    var dely = this.currentPoint.y - this.previousPoint.y;
                    //delx /= this.diagram._zoomPan.currentZoom;
                    //dely /= this.diagram._zoomPan.currentZoom;
                    var mat = new ej.Diagram.Matrix();
                    var matrix = mat._createIdentity();
                    matrix._rotate(-this.selectedObject.rotateAngle);
                    var delta = _matrix._transform(new ej.Diagram.Point(delx, dely));
                    var difx = (this.selectedObject.width * this.selectedObject.pivot.x + delta.x) / this.selectedObject.width;
                    var dify = (this.selectedObject.height * this.selectedObject.pivot.y + delta.y) / this.selectedObject.height;
                    this.selectedObject.pivot.x = difx;
                    this.selectedObject.pivot.y = dify;
                    this.selectedObject.offsetX += delx;
                    this.selectedObject.offsetY += dely;
                    this.diagram._updateSelectionHandle();
                }
            }
            this.previousPoint = this.currentPoint;
        };
        PivotTool.prototype._rotate = function (node) {
            var pt = new ej.Diagram.Point(node.offsetX, node.offsetY);
            var crntpoint = this.currentPoint;
            var angle = this.FindAngle(pt, crntpoint);
            node._rotate(angle - node.rotateAngle, pt);
        };
        PivotTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.inAction = false;
                this.diagram._updateSelectionHandle();
            }
            base.prototype.mouseup.call(this, evt);
        };
        return PivotTool;
    })(Tool);
    ej.Diagram.PivotTool = PivotTool;
    //#endregion

    //#region PanTool
    var PanTool = (function (base) {
        ej.Diagram.extend(PanTool, base);
        function PanTool(diagram) {
            base.call(this, "panTool", diagram);
            this.diagram = diagram;
            this._isMouseDown = false;
            this.cursor = "pointer";
        }
        PanTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            if (this.diagram.activeTool instanceof ej.Diagram.PanTool) {
                var svgDocument = this.diagram._svg.document;
                var pt = this.diagram._svg.document.createSVGPoint();
                pt.x = evt.clientX; pt.y = evt.clientY;
                var mousept = pt.matrixTransform(this.diagram._svg.document.getScreenCTM().inverse());
                svgDocument.lastMouseX = mousept.x;
                svgDocument.lastMouseY = mousept.y;
                this._isMouseDown = true;
                return false;
            }
        };
        PanTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this.diagram.activeTool instanceof ej.Diagram.PanTool && this._isMouseDown) {
                this.inAction = true;
                var svgDocument = this.diagram._svg.document;
                var pt = this.diagram._svg.document.createSVGPoint();
                pt.x = evt.clientX; pt.y = evt.clientY;
                var mousept = pt.matrixTransform(this.diagram._svg.document.getScreenCTM().inverse());
                var ex = mousept.x;
                var ey = mousept.y;
                var nx, ny;

                nx = ((ex - svgDocument.lastMouseX));
                ny = ((ey - svgDocument.lastMouseY));
                if (!(document.getElementById(this.diagram.element[0].id).getElementsByClassName('e-hhandle').length > 0))
                    nx = 0;
                if (!(document.getElementById(this.diagram.element[0].id).getElementsByClassName('e-vhandle').length > 0))
                    ny = 0;
                this.diagram._zoomPan.zoomPanDelta(nx, ny, 1, new ej.Diagram.Point(ex, ey));
                svgDocument.lastMouseX = ex;
                svgDocument.lastMouseY = ey;
                return false;
            }
        };
        PanTool.prototype.mouseup = function (evt) {
            if (this.diagram.activeTool instanceof ej.Diagram.PanTool) {
                this.inAction = false;
                document.onmousemove = null;
                document.onmouseup = null;
                this._isMouseDown = false;
            }
            base.prototype.mouseup.call(this, evt);
        };
        return PanTool;
    })(Tool);
    ej.Diagram.PanTool = PanTool;
    //#endregion

    //#region RectangleToolBase
    var RectangleToolBase = (function (base) {
        ej.Diagram.extend(RectangleToolBase, base);
        function RectangleToolBase(name, diagram) {
            base.call(this, name, diagram);
            this.cursor = "crosshair";
        }
        RectangleToolBase.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.createNode();
        };
        RectangleToolBase.prototype.createNode = function () {
        };
        RectangleToolBase.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                    this._renderHelper();
                    this._calcOffset();
                }
                else {
                    this._updateHelper();
                }
            }
        };
        RectangleToolBase.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.inAction = false;
                if (this.diagram.model.snapSettings.snapConstraints.snapToLines) {
                    this._calcOffset();
                }
                this._updateSize(this.selectedObject, this.startPoint, this.currentPoint);
                this.selectedObject.updatebounds();
                this.diagram.model.nodes.add(this.selectedObject);
                ej.Diagram.SvgContext.renderNode(this.selectedObject, this.diagram._svg, this.diagram._diagramLayer);
                this.diagram._clearSelection();
                this.diagram._addSelection(this.selectedObject);
            }
            this.diagram._toolToActivate = "selectTool";
            this.diagram._drawingTool = false;
            base.prototype.mouseup.call(this, evt);
        };
        RectangleToolBase.prototype._calcOffset = function () {
            var scale = this.diagram.model.magnification / 100;
            var bounds = this.helper.getBoundingBox();
            var location = bounds.location();
            location = new ej.Diagram.Point(bounds.x + bounds.width, bounds.y + bounds.height);
            var x = location.x * scale;
            var y = location.y * scale;
            var xy = new ej.Diagram.Point(x, y);
            var snapxy = this.snap(new ej.Diagram.Point(x, y));
            this._mouseOffset = new ej.Diagram.Size(xy.x - snapxy.x, xy.y - snapxy.y);
        };
        RectangleToolBase.prototype.getBoundingRect = function (startPoint, currentPoint) {
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var diff = new ej.Diagram.Size(width, height);
            if (this.diagram.model.snapSettings.snapConstraints.snapToHorizontalLines) {
                if (this._mouseOffset.width != 0) {
                    diff.width -= this._mouseOffset.width;
                    this._mouseOffset.width = 0;
                }
            }
            if (this.diagram.model.snapSettings.snapConstraints.snapToVerticalLines) {
                if (this._mouseOffset.height != 0) {
                    diff.height -= this._mouseOffset.height;
                    this._mouseOffset.height = 0;
                }
            }
            var x = (this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x;
            var y = (this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y;
            return { size: diff, offset: new ej.Diagram.Point(x, y) };
        };
        RectangleToolBase.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                this.svgHelper = ej.Diagram.SvgContext.renderNode(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper.setAttribute("pointer-events", "none");
            }
        };
        RectangleToolBase.prototype._updateHelper = function () {
            this._updateSize(this.helper, this.startPoint, this.currentPoint);
            ej.Diagram.SvgContext.updateNode(this.helper, this.diagram._svg);
        };
        RectangleToolBase.prototype._updateSize = function (shape, startPoint, currentPoint) {
            var rect = this.getBoundingRect(startPoint, currentPoint);
            var size = rect.size;
            var offset = rect.offset;
            if (!size.isEmpty()) {
                shape.offsetX = offset.x + size.width / 2;
                shape.offsetY = offset.y + size.height / 2;
                shape.width = size.width;
                shape.height = size.height;
            }
        };
        return RectangleToolBase;
    })(ej.Diagram.Tool);
    ej.Diagram.RectangleToolBase = RectangleToolBase;
    //#endregion

    //#region PolygonToolBase
    var PolygonToolBase = (function (base) {
        ej.Diagram.extend(PolygonToolBase, base);
        function PolygonToolBase(name, diagram) {
            base.call(this, name, diagram);
            this.cursor = "crosshair";
        }
        PolygonToolBase.prototype.createNode = function () {

        };
        PolygonToolBase.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
        };
        PolygonToolBase.prototype.mousemove = function (evt) {

            base.prototype.mousemove.call(this, evt);
            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                    this._renderHelper();
                    this._calcOffset();
                }
                else {
                    this._updateHelper(this.snap(this.currentPoint));
                }
            }
        };
        PolygonToolBase.prototype.mouseup = function (evt) {
            if (!this.selectedObject) {
                if (this.startPoint) {
                    this.selectedObject = this.createNode();
                    var sp = this.snap(this.startPoint);
                    var ep = this.snap(this.currentPoint);
                    this.selectedObject.shape.points.add(new ej.Diagram.Point(sp.x, sp.y));
                    this.selectedObject.shape.points.add(new ej.Diagram.Point(ep.x, ep.y));
                }
            }
            else if (this.inAction) {
                this.helper.shape.points.add(new ej.Diagram.Point(this.currentPoint.x, this.currentPoint.y));
            }
        };
        PolygonToolBase.prototype._calcOffset = function () {
            var scale = this.diagram.model.magnification / 100;
            var bounds = this.helper.getBoundingBox();
            var location = bounds.location();
            location = new ej.Diagram.Point(bounds.x + bounds.width, bounds.y + bounds.height);
            var x = location.x * scale;
            var y = location.y * scale;
            var xy = new ej.Diagram.Point(x, y);
            var snapxy = this.snap(new ej.Diagram.Point(x, y));
            this._mouseOffset = new ej.Diagram.Size(xy.x - snapxy.x, xy.y - snapxy.y);
        };
        PolygonToolBase.prototype._renderHelper = function () {
            if (!this.svgHelper) {

                var rect = ej.Diagram.Geometry.rect(this.helper.shape.points);
                this.helper.offsetX = rect.x + rect.width / 2;
                this.helper.offsetY = rect.y + rect.height / 2;
                this.helper.width = rect.width;
                this.helper.height = rect.height;
                this.svgHelper = ej.Diagram.SvgContext.renderNode(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper.setAttribute("pointer-events", "none");
            }
        };
        PolygonToolBase.prototype.doubleclick = function (currentPoint) {
            if (this.selectedObject) {
                this.selectedObject.updatebounds();
                this.helper.shape.points.remove(this.helper.shape.points[this.helper.shape.points.length - 2]);
                this.helper.shape.points.remove(this.helper.shape.points[this.helper.shape.points.length - 1]);
                this.selectedObject.shape.points = this.helper.shape.points;
                var rect = ej.Diagram.Geometry.rect(this.helper.shape.points);
                this.selectedObject.offsetX = rect.x + rect.width / 2;
                this.selectedObject.offsetY = rect.y + rect.height / 2;
                this.selectedObject.width = rect.width;
                this.selectedObject.height = rect.height;
                this.diagram.model.nodes.add(this.selectedObject);
                this.selectedObject.updatebounds();
                ej.Diagram.SvgContext.renderNode(this.selectedObject, this.diagram._svg, this.diagram._diagramLayer);

                this.diagram._clearSelection();
                this.diagram._addSelection(this.selectedObject);
            }
            if (this._svgHelper) {
                this.diagram._svg.removeChild(this._svgHelper);
                this._svgHelper = null;
            }
            this._endAction();
            this.diagram._drawingTool = false;
            this.diagram._toolToActivate = "selectTool";
            this._mouseOffset = new ej.Diagram.Size(0, 0);
        };
        PolygonToolBase.prototype._updateHelper = function (currentPoint) {
            this.helper.shape.points[this.helper.shape.points.length - 1] = new ej.Diagram.Point(currentPoint.x, currentPoint.y);
            var rect = ej.Diagram.Geometry.rect(this.helper.shape.points);
            this.helper.offsetX = rect.x + rect.width / 2;
            this.helper.offsetY = rect.y + rect.height / 2;
            this.helper.width = rect.width;
            this.helper.height = rect.height;
            ej.Diagram.SvgContext.updateNode(this.helper, this.diagram._svg);
        };
        return PolygonToolBase;
    })(ej.Diagram.Tool);
    ej.Diagram.PolygonToolBase = PolygonToolBase;
    //#endregion

    //#region TextToolBase
    var TextToolBase = (function (base) {
        ej.Diagram.extend(TextToolBase, base);
        function TextToolBase(name, diagram) {
            base.call(this, name, diagram);
            this.boundingRect = new ej.Diagram.Rectangle();
            this.activeLabel = null;
            this.cursor = "crosshair";
        }
        TextToolBase.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            if (this.diagram._isEditing) {
                this.diagram._endEdit();
            }
        };
        TextToolBase.prototype.createNode = function (rect) {

        };
        TextToolBase.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (!this.selectedObject && !this.startPoint.isEmpty() && !this.diagram._isEditing) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                }
                else {
                    this._updateHelper();
                }
            }
        };
        TextToolBase.prototype.mouseup = function (evt) {
            this.activeLabel = null;
            var minHeight = 12;
            if (this.inAction) {
                var rect = this.getBoundingRect(this.startPoint, this.currentPoint);
                this._endAction();
                if (ej.browserInfo().name === "chrome" && (rect.width === 0 || rect.height === 0)) {
                    rect.width = 90;
                    rect.height = minHeight;
                }
                this.selectedObject = this.createNode(rect.x + (rect.width / 2), (rect.y + (rect.height / 2)), rect.width, rect.height);
                this.diagram._startEdit(this.selectedObject);
            }
            else {
                var node = this._findNodeUnderMouse(evt);
                if (node != null) {
                    this.selectedObject = node;
                    this.diagram._startEdit(this.selectedObject);
                }
                else {
                    var rect = new ej.Diagram.Rectangle(this.currentPoint.x - (90 / 2), this.currentPoint.y - (minHeight / 2), 90, minHeight);
                    this._endAction();
                    this.selectedObject = this.createNode(rect.x + (rect.width / 2), rect.y + (rect.height / 2), rect.width, minHeight);
                    this.diagram._startEdit(this.selectedObject);
                }
            }
        };
        TextToolBase.prototype.getBoundingRect = function (startPoint, currentPoint) {
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var x = (this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x;
            var y = (this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y;
            return { x: x, y: y, width: width, height: height };
        };
        TextToolBase.prototype._initHelper = function () {
            if (!this._svgHelper) {
                var g = this.diagram._svg.g();
                this.diagram._adornerLayer.appendChild(g);
                var scale = this.diagram._zoomPan.currentZoom;
                var selectionRect = this.diagram._svg.rect({ "id": "helper", "x": this.startPoint.x * scale, "y": this.startPoint.y * scale, "fill": "transparent", "stroke": "gray", "stroke-dasharray": "2 2", "shape-rendering": "crispEdges" });
                g.appendChild(selectionRect);
                g.setAttribute("pointer-events", "none");
                this._svgHelper = g;
            }
        };
        TextToolBase.prototype._updateHelper = function () {
            var helper = this.diagram._svg.getElementById("helper");
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var scale = this.diagram._zoomPan.currentZoom;
            var x = ((this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x) * scale;
            var y = ((this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y) * scale;
            if (helper) {
                this.diagram._svg.rect({ "id": helper.id, "x": x, "y": y, "width": width * scale, "height": height * scale });
            }
        };
        TextToolBase.prototype._endAction = function () {
            if (this._svgHelper) {
                this.diagram._adornerLayer.removeChild(this._svgHelper);
                this._svgHelper = null;
            }
            base.prototype._endAction.apply(this);
        };
        TextToolBase.prototype._findNodeUnderMouse = function (evt, skip) {
            var obj;
            var textElement = $(evt.target).parents("text")[0];
            if (textElement != null) {
                if (textElement.getAttribute("class") == "ej-d-label") {
                    var parentNode = $(textElement).parents("g")[0];
                    obj = this._findObj(parentNode);
                    if (obj.shape.type != "text") {
                        var label = textElement.id.split('_');
                        this.activeLabel = obj.labels.find(label[1]);
                    }
                }
            }
            else {
                var parents = $(evt.target).parents("g");
                if (parents.length > 3) {
                    var element = parents[parents.length - 4];
                    var id = element.getAttribute("id");
                    var type = element.getAttribute("class");
                    if (type) {
                        if (type === "ej-d-node" || type === "ej-d-group") {
                            obj = this.diagram._findNode(id);
                            var curlabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                        else if (type === "ej-d-connector") {
                            obj = this.diagram._findConnector(id);
                            var curlabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                    }
                }
            }
            return obj;
        };
        TextToolBase.prototype.getLabelUnderMouse = function () {
            return this.activeLabel;
        };
        return TextToolBase;
    })(ej.Diagram.Tool);
    ej.Diagram.TextToolBase = TextToolBase;
    //#endregion

    //#region RectangleTool
    var RectangleTool = (function (base) {
        ej.Diagram.extend(RectangleTool, base);
        function RectangleTool(diagram) {
            base.call(this, "rectangleTool", diagram);
        }
        RectangleTool.prototype.createNode = function () {
            var node = new ej.Diagram.Node();
            node.name = "rectangle" + ej.Diagram.Util.randomId();
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;
            node.updatebounds();
            return node;
        };
        return RectangleTool;
    })(ej.Diagram.RectangleToolBase);
    ej.Diagram.RectangleTool = function (diagram) {
        return new RectangleTool(diagram);
    };
    //#endregion

    //#region EllipseTool
    var EllipseTool = (function (base) {
        ej.Diagram.extend(EllipseTool, base);
        function EllipseTool(diagram) {
            base.call(this, "ellipseTool", diagram);
        }
        EllipseTool.prototype.createNode = function () {
            var node = new ej.Diagram.Node();
            node.name = "ellipse" + ej.Diagram.Util.randomId();
            node.shape = new ej.Diagram.Shapes.Ellipse();
            node.fillColor = "white";
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;
            node.updatebounds();
            return node;
        };
        return EllipseTool;
    })(ej.Diagram.RectangleToolBase);
    ej.Diagram.EllipseTool = function (diagram) {
        return new EllipseTool(diagram);
    };
    //#endregion

    //#region RoundRectTool
    var RoundRectTool = (function (base) {
        ej.Diagram.extend(RoundRectTool, base);
        function RoundRectTool(diagram) {
            base.call(this, "roundRectTool", diagram);
        }
        RoundRectTool.prototype.createNode = function () {
            var node = new ej.Diagram.Node();
            node.name = "roundrect" + ej.Diagram.Util.randomId();
            node.shape = new ej.Diagram.Shapes.Rectangle(15);;
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;
            node.updatebounds();
            return node;
        };
        return RoundRectTool;
    })(ej.Diagram.RectangleToolBase);
    ej.Diagram.RoundRectTool = function (diagram) {
        return new RoundRectTool(diagram);
    };
    //#endregion

    //#region TextTool
    var TextTool = (function (base) {
        ej.Diagram.extend(TextTool, base);
        function TextTool(diagram) {
            base.call(this, "textTool", diagram);
        }
        TextTool.prototype.createNode = function (x, y, width, height) {
            var node = new ej.Diagram.Node();
            node.name = "text" + ej.Diagram.Util.randomId();
            node.shape = new ej.Diagram.Shapes.Text(new ej.Diagram.TextBlock());
            node.offsetX = x;
            node.offsetY = y;
            node.width = width;
            node.height = height;
            node.borderColor = "transparent";
            node.updatebounds();
            return node;
        };
        return TextTool;
    })(ej.Diagram.TextToolBase);
    ej.Diagram.TextTool = function (diagram) {
        return new TextTool(diagram);
    };
    //#endregion

    //#region PolygonTool
    var PolygonTool = (function (base) {
        ej.Diagram.extend(PolygonTool, base);
        function PolygonTool(diagram) {
            base.call(this, "polygonTool", diagram);
        }
        PolygonTool.prototype.createNode = function () {
            var node = new ej.Diagram.Node();
            node.name = "polygon" + ej.Diagram.Util.randomId();
            node.shape = new ej.Diagram.Shapes.Polygon(new ej.Diagram.Collection());
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;
            node.updatebounds();
            return node;
        };
        return PolygonTool;
    })(ej.Diagram.PolygonToolBase);
    ej.Diagram.PolygonTool = function (diagram) {
        return new PolygonTool(diagram);
    };;
    //#endregion

    //#region ImageTool
    var ImageTool = (function (base) {
        ej.Diagram.extend(ImageTool, base);
        function ImageTool(saveUrl, removeUrl, uploadfileUrl, autoOpenUpload, diagram) {
            base.call(this, "imageTool", diagram);
            this.saveUrl = saveUrl;
            this.removeUrl = removeUrl;
            this.uploadfileUrl = uploadfileUrl;
            this.autoOpenUpload = autoOpenUpload;
            this.bounds = new ej.Diagram.Rectangle(0, 0, 0, 0);
            this.cursor = "crosshair";
        }
        ImageTool.prototype.createNode = function (imageUrl) {
            var node = new ej.Diagram.Node();
            node.name = "image" + ej.Diagram.Util.randomId();
            node.shape = new ej.Diagram.Shapes.Image(imageUrl);
            node.borderDashArray = "5,5";
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;
            node.updatebounds();
            return node;
        };
        ImageTool.prototype.mouseup = function (evt) {
            if (this.inAction)
                this.bounds = this.getBoundingRect(this.snap(this.startPoint), this.snap(this.currentPoint));
            this._renderDialogControl();
            this.diagram._toolToActivate = "selectTool";
            this.diagram._drawingTool = false;
            this._endAction();
        };
        ImageTool.prototype._showPopup = function () {
            this._renderDialogControl();
            this.diagram._toolToActivate = "selectTool";
            this.diagram._drawingTool = false;
            this._endAction();
        };
        ImageTool.prototype._renderDialogControl = function () {
            var dialogDiv = document.createElement("div");
            var attr = {
                "id": "basicDialog",
                "title": "Insert Image",
                "style": "display:none"
            };
            ej.Diagram.Util.attr(dialogDiv, attr);
            document.body.appendChild(dialogDiv);
            var uploadDiv = document.createElement("div");
            attr = {
                "id": "UploadDefault",
                "style": "padding-Left:15px; float:left"
            };
            ej.Diagram.Util.attr(uploadDiv, attr);
            var table = document.createElement("table");
            var tr = document.createElement("tr");
            var td = document.createElement("td");
            attr = {
                "align": "left",
                "style": "padding-Top:10px"
            }
            ej.Diagram.Util.attr(td, attr);
            var childDiv = document.createElement("div");
            childDiv.style.cssFloat = "left";
            var rdoComputer = document.createElement("input");
            attr = {
                "type": "radio",
                "name": "uploadOption",
                "id": "rdoComputer",
                "checked": "true"
            };
            ej.Diagram.Util.attr(rdoComputer, attr);
            rdoComputer.onchange = $.proxy(this._rdoChanged, this);
            childDiv.appendChild(rdoComputer);
            var label = document.createElement("label");
            label.innerHTML = "My Computer";
            label.className = "dialogLabel";
            attr = {
                "style": "font-weight:normal; padding-left:5px; vertical-align: top;"
            }
            ej.Diagram.Util.attr(label, attr);
            label.onclick = $.proxy(this._labelClick, this);
            childDiv.appendChild(label);
            td.style.paddingBottom = "5px";
            td.appendChild(childDiv);
            td.appendChild(uploadDiv);
            var label = document.createElement("label");
            attr = {
                "id": "lblComputerUrl",
                "style": "font-weight:normal;padding-left:10px; vertical-align: top;color:red;"
            };
            ej.Diagram.Util.attr(label, attr);
            td.appendChild(label);
            tr.appendChild(td);
            table.appendChild(tr);
            tr = document.createElement("tr");
            td = document.createElement("td");
            var rdoWebUrl = document.createElement("input");
            attr = {
                "id": "rdoWebUrl",
                "type": "radio",
                "name": "uploadOption"
            };
            ej.Diagram.Util.attr(rdoWebUrl, attr);
            rdoWebUrl.onchange = $.proxy(this._rdoChanged, this);
            td.appendChild(rdoWebUrl);
            var label = document.createElement("label");
            label.innerHTML = "Web URL";
            label.className = "dialogLabel";
            attr = {
                "style": "font-weight:normal; padding-left:5px; vertical-align: top;"
            };
            ej.Diagram.Util.attr(label, attr);
            label.onclick = $.proxy(this._labelClick, this);
            td.appendChild(label);
            var label = document.createElement("label");
            attr = {
                "id": "lblWebUrl",
                "style": "font-weight:normal;padding-left:25px; vertical-align: top;color:red;"
            };
            ej.Diagram.Util.attr(label, attr);
            td.appendChild(label);
            tr.appendChild(td);
            table.appendChild(tr);
            tr = document.createElement("tr");
            td = document.createElement("td");
            attr = {
                "style": "padding-Top:5px;padding-Bottom:15px"
            };
            ej.Diagram.Util.attr(td, attr);
            var txtImageUrl = document.createElement("input");
            attr = {
                "id": "txtImageUrl",
                "type": "text",
                "placeholder": "Image URL",
                "style": "width:460px;height:20px"
            };
            ej.Diagram.Util.attr(txtImageUrl, attr);
            txtImageUrl.onfocus = $.proxy(this._textFocus, this);
            txtImageUrl.onfocusout = $.proxy(this._textFocusOut, this);
            td.appendChild(txtImageUrl);
            tr.appendChild(td);
            table.appendChild(tr);
            tr = document.createElement("tr");
            td = document.createElement("td");
            var btnOk = document.createElement("input");
            attr = {
                "value": "Ok",
                "type": "button",
                "id": "btnOk",
                "style": "width : 100px; height: 30px; margin-Right: 5px; border: 1px solid #F7952E; color : white; font-Family:Segoe UI;font-weight:bold"
            };
            ej.Diagram.Util.attr(btnOk, attr);
            btnOk.onclick = $.proxy(this._uploadClick, this);
            td.appendChild(btnOk);
            var btnCancel = document.createElement("input");
            attr = {
                "value": "Cancel",
                "type": "button",
                "id": "btnCancel",
                "style": "width : 100px; height: 30px; border: 1px solid #DFDFDE; font-Family:Segoe UI;font-weight:bold"
            };
            ej.Diagram.Util.attr(btnCancel, attr);
            btnCancel.onclick = $.proxy(this._closeDialog, this);
            td.appendChild(btnCancel);
            attr = {
                "align": "right"
            };
            ej.Diagram.Util.attr(td, attr);
            tr.appendChild(td);
            table.appendChild(tr);
            dialogDiv.appendChild(table);
            $("#UploadDefault").ejUploadbox({
                saveUrl: this.saveUrl,
                removeUrl: this.removeUrl,
                complete: $.proxy(this.fileuploadComplete, this)
            });
            $("#basicDialog").ejDialog({ height: 230, width: 500, modal: true });
            $("#basicDialog").ejDialog("open");
        };
        ImageTool.prototype.fileuploadComplete = function (e, ui) {
            var fileName = this.uploadfileUrl + "/" + e.files.name;
            this._renderImageNode(fileName);
            this._closeDialog();
        };
        ImageTool.prototype._uploadClick = function () {
            var imageUrl = document.getElementById("txtImageUrl").value;
            if (/^(http:\/\/www\.|https:\/\/www\.)[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/.test(imageUrl)) {
                this._renderImageNode(imageUrl);
                this._closeDialog();
            }
            else {
                if (document.getElementById("rdoComputer").checked === true) {
                    var lblComputerUrl = document.getElementById("lblComputerUrl");
                    var lblWebUrl = document.getElementById("lblWebUrl");
                    lblComputerUrl.innerHTML = "Browse any image to insert";
                    lblWebUrl.innerHTML = "";
                }
                else {
                    var lblComputerUrl = document.getElementById("lblComputerUrl");
                    var lblWebUrl = document.getElementById("lblWebUrl");
                    lblComputerUrl.innerHTML = "";
                    lblWebUrl.innerHTML = "Invalid Url";
                    document.getElementById("txtImageUrl").focus();
                }
            }
        };
        ImageTool.prototype._rdoChanged = function () {
            var lblComputerUrl = document.getElementById("lblComputerUrl");
            var lblWebUrl = document.getElementById("lblWebUrl");
            lblComputerUrl.innerHTML = "";
            lblWebUrl.innerHTML = "";
        };
        ImageTool.prototype._closeDialog = function () {
            $("#basicDialog").ejDialog("close");
            var dialog = document.getElementById("basicDialog");
            dialog.parentNode.removeChild(dialog);
        };
        ImageTool.prototype._labelClick = function (evt) {
            if (evt.target.innerHTML == "My Computer") {
                var radio = document.getElementById("rdoComputer");
                radio.checked = true;
            }
            else {
                var radio = document.getElementById("rdoWebUrl");
                radio.checked = true;
            }
            this._rdoChanged();
        };
        ImageTool.prototype._textFocus = function (evt) {
            var radio = document.getElementById("rdoWebUrl");
            radio.checked = true;
        };
        ImageTool.prototype._renderImageNode = function (fileName) {
            var node = this.createNode(fileName);
            if (this.bounds.size.width != undefined && !this.bounds.size.isEmpty()) {
                node.offsetX = this.bounds.offset.x + this.bounds.size.width / 2;
                node.offsetY = this.bounds.offset.y + this.bounds.size.height / 2;
                node.width = this.bounds.size.width;
                node.height = this.bounds.size.height;
            }
            else {
                node.offsetX = 150;
                node.offsetY = 150;
                node.width = 100;
                node.height = 100;
            }
            node.borderDashArray = "";
            node.updatebounds();
            this.diagram.model.nodes.add(node);
            ej.Diagram.SvgContext.renderNode(node, this.diagram._svg, this.diagram._diagramLayer);

            this.diagram._clearSelection();
            this.diagram._addSelection(node);
        };
        return ImageTool;
    })(ej.Diagram.RectangleToolBase);
    ej.Diagram.ImageTool = function (saveUrl, removeUrl, uploadfileUrl, autoOpenUpload, diagram) {
        return new ImageTool(saveUrl, removeUrl, uploadfileUrl, autoOpenUpload, diagram);
    };;
    //#endregion

})(jQuery, Syncfusion);;

(function ($, ej) {
    "use strict";

    //#region Layout
    var Layout = (function () {
        function Layout(options) {
            var obj = options || {};
            this.type = null;
            this.orientation = obj.orientation || "topToBottom";
            this.horizontalSpacing = obj.horizontalSpacing || 30;
            this.verticalSpacing = obj.verticalSpacing || 30;
            this.marginX = obj.marginX || 0;
            this.marginY = obj.marginY || 0;
            this.fixedNode = obj.fixedNode || null;
            return this;
        }
        return Layout;
    })();
    ej.Diagram.Layout = Layout;


    //#region Hierarchical layout
    var HierarchicalLayout = (function (base) {
        ej.Diagram.extend(HierarchicalLayout, base);
        //Constructor
        function HierarchicalLayout(options) {
            base.call(this, options);
            this._processingLayout = false;
            this.type = "hierarchicalTree";
            this._model = null;
            this._passedNodes = [];
            this._passedNodesList = [];
            this._selectedNodeToLayout = null;
            this._improvePerformance = null;
            this._leftMargin = 0;
            this._topMargin = 0;
            this._lstRankOrdered = [];
            this._fictitiousNodes = [];
            this._biggestDimensions = [];
            this._yBorder = 0;
            this._xBorder = 0;
            this._leftChild = null;
            this._leftParent = null;
            this._rightChild = null;
            this._rightParent = null;
            this._fX = 0;
        }

        HierarchicalLayout.prototype._setModel = function (model, diagram) {
            this._model = model;
            this._diagram = diagram;
        };

        HierarchicalLayout.prototype.update = function () {
            this._diagram._updateEdgeCollection();
            this.doLayout();
            this._diagram._updateNodes();
            this._diagram._updateConnectors();
            if (this._diagram.selectionList[0]) {
                ej.Diagram.SvgContext.updateSelector(this._diagram.selectionList[0], this._diagram._svg, this._diagram._zoomPan.currentZoom, this._diagram, this._model.selectorConstraints);
            }
        };

        HierarchicalLayout.prototype._initLayout = function () {
            this._passedNodes = [];
            this._passedNodesList = [];
            this._selectedNodeToLayout = null;
            this._lstRankOrdered = [];
            this._fictitiousNodes = [];
            this._biggestDimensions = []; this._fX = 0;
            this._leftChild = null;
            this._leftParent = null;
            this._rightChild = null;
            this._rightParent = null;
        };

        //Instance Member
        HierarchicalLayout.prototype.doLayout = function () {
            this._processingLayout = true;
            this._initLayout();
            this._leftMargin = 0;
            this._topMargin = 0;
            var nodesCount = this._model.nodes.length;
            this._visitedNodes = 0;
            if (nodesCount > 0) {
                while (this._model.nodes.length > this._visitedNodes) {
                    this._getNodesToPosition(this._model.nodes);
                    if (this._selectedNodeToLayout == null)
                        continue;
                    var fixedPosition;
                    if (this.fixedNode)
                        fixedPosition = new ej.Diagram.Point(this.fixedNode.offsetX, this.fixedNode.offsetY);
                    this._doLayout();
                    this._updateOrientation();
                    this._arrangeNodesToLeftMost(fixedPosition);
                    if (!this._diagram.model.layout || !this._diagram.model.layout._processingLayout) {
                        if (this._diagram._pageBackgroundLayer) {
                            this._diagram.model.pageSettings._updatePageSize(this._diagram, true);
                        }
                    }
                }
                return this._model.nodes;
            }
            this._processingLayout = false;
        };

        HierarchicalLayout.prototype._doLayout = function () {
            var selectedNode = this._selectedNodeToLayout;
            var node = selectedNode;
            //var graph = selectedNode.GraphNodes;
            //var bounds = selectedNode.Bounds;
            if (node != null) {
                // layout if selected node is graph
                if (node.ConnectorType == undefined)
                    this._doGraphLayout();
            }

        };

        HierarchicalLayout.prototype._doGraphLayout = function () {
            var graph = this._selectedNodeToLayout;
            if (graph != null)
                this._makeLayout(graph);
        };

        HierarchicalLayout.prototype._makeLayout = function (graphSorting) {
            if (graphSorting != null) {
                this._doInitXLayout(graphSorting);
                this._placeParentsRelativeToChildren(graphSorting.TypeOrdered);
                //this._makeTreeLayout(graphSorting.TypeOrdered);
            }
        };

        //Arranging elements by rank

        HierarchicalLayout.prototype._doInitXLayout = function (graphToSort) {
            var lstToSort = graphToSort.TypeOrdered;
            var counter = 0;
            for (var lstSortIndex = 0; lstSortIndex < lstToSort.length; lstSortIndex++) {
                var lstSort = lstToSort[lstSortIndex];
                this._makeLayoutByRank(lstSort, counter, graphToSort);
                counter++;
            }
        };

        HierarchicalLayout.prototype._makeLayoutByRank = function (lstRank, nRank, graph) {
            if (this.orientation == "topToBottom" || this.orientation == "bottomToTop") {
                this._fX = 0;
                for (var boundsIndex = 0; boundsIndex < lstRank.length; boundsIndex++) {
                    var bounds = lstRank[boundsIndex];
                    if (bounds != null)
                        this._xCoordinateRoutine(bounds, this._fX);
                }
                // ============== Y Coordinate Routine ================
                var fYRank = this._calcRankPositionVerticalLayout(nRank, graph);
                this._yCoordinateRoutine(lstRank, fYRank);
            }
            else {
                this._fY = 0;
                for (var boundsIndex = 0; boundsIndex < lstRank.length; boundsIndex++) {
                    var bounds = lstRank[boundsIndex];
                    if (bounds != null) {
                        bounds.offsetY = this._fY + bounds.height / 2;
                        this._fY = bounds.offsetY + bounds.height / 2 + this.horizontalSpacing;
                    }
                }
                // ============== X Coordinate Routine ================
                var fXRank = this._calcRankPositionHorizontalLayout(nRank, graph);
                this._xCoordinateRoutineforhLayouts(lstRank, fXRank);
            }

        };

        HierarchicalLayout.prototype._xCoordinateRoutineforhLayouts = function (lstRank, fXRank) {
            for (var entryIndex = 0; entryIndex < lstRank.length; entryIndex++) {
                var entry = lstRank[entryIndex];
                entry.offsetX = fXRank + entry.width / 2;
            }
        };

        HierarchicalLayout.prototype._calcRankPositionHorizontalLayout = function (nRank, graph) {
            var fRankPositionToReturn = 0;
            var szRankDimensions = this._biggestDimensions[nRank];

            if (nRank == 0)
                fRankPositionToReturn = 0;
            else {
                var fGraphHeightTillCurrentRank = this._getSubGraphDimension(0, nRank, "Width", graph);
                fRankPositionToReturn = fGraphHeightTillCurrentRank;
            }

            return fRankPositionToReturn + this._xBorder;
        };

        HierarchicalLayout.prototype._calcRankPositionVerticalLayout = function (nRank, graph) {
            var fRankPositionToReturn = 0;
            var szRankDimensions = this._biggestDimensions[nRank];

            if (nRank == 0)
                fRankPositionToReturn = 0;
            else {
                var fGraphHeightTillCurrentRank = this._getSubGraphDimension(0, nRank, "Height", graph);
                fRankPositionToReturn = fGraphHeightTillCurrentRank;
            }

            return fRankPositionToReturn + this._yBorder;
        };

        HierarchicalLayout.prototype._xCoordinateRoutine = function (gnBounds, fX) {
            gnBounds.offsetX = this._fX + gnBounds.width / 2;
            this._fX += gnBounds.width / 2 + this.horizontalSpacing;
            //gnBounds.offsetX += this.horizontalSpacing;
        };

        HierarchicalLayout.prototype._yCoordinateRoutine = function (lstRank, fYRank) {
            for (var entryIndex = 0; entryIndex < lstRank.length; entryIndex++) {
                var entry = lstRank[entryIndex];
                entry.offsetY = fYRank + entry.height / 2;
            }
        };

        HierarchicalLayout.prototype._getSubGraphDimension = function (nFromRank, nToRank, dimension, graph) {
            var fGraphDimensionToReturn = 0;
            var nCounter = nFromRank;
            var nIncrease = 1;

            if (nFromRank > nToRank)
                nIncrease = -nIncrease;
            if (this.orientation == "topToBottom" || this.orientation == "bottomToTop") {

                while (nCounter != nToRank) {
                    var szRankBiggestDimensions = this._biggestDimensions[nCounter];

                    switch (dimension) {
                        case "Height":
                            fGraphDimensionToReturn += szRankBiggestDimensions.height;
                            fGraphDimensionToReturn += this.verticalSpacing;
                            break;
                        case "Width":
                            fGraphDimensionToReturn += szRankBiggestDimensions.width;
                            fGraphDimensionToReturn += this.horizontalSpacing;
                            break;
                    }
                    nCounter += nIncrease;
                }
            } else {

                while (nCounter != nToRank) {
                    var szRankBiggestDimensions = this._biggestDimensions[nCounter];

                    switch (dimension) {
                        case "Height":
                            fGraphDimensionToReturn += szRankBiggestDimensions.height;
                            fGraphDimensionToReturn += this.horizontalSpacing;
                            break;
                        case "Width":
                            fGraphDimensionToReturn += szRankBiggestDimensions.width;
                            fGraphDimensionToReturn += this.verticalSpacing;
                            break;
                    }
                    nCounter += nIncrease;
                }
            }

            return fGraphDimensionToReturn;
        };

        //Arranging elements with respect to parents and children

        HierarchicalLayout.prototype._placeParentsRelativeToChildren = function (lstToSort) {
            if (lstToSort.length > 1) {
                for (var i = lstToSort.length - 2; i >= 0; i--) {
                    var list = lstToSort[i + 1];
                    this._placeRelativeToChildren(list);
                    this._normalizeNodes(lstToSort[i]);
                    var j = i + 1;
                    while (j <= lstToSort.length - 1) {
                        var k = j;
                        while (k >= i) {
                            this._normalizeNodes(lstToSort[k]);
                            this._placeRelativeToChildren(lstToSort[k]);
                            k--;
                        }
                        j++;
                    }
                }
            }
        };

        HierarchicalLayout.prototype._placeRelativeToChildren = function (lstRelatives) {

            var fLeftChildPosition = 0;
            var fRightChildPosition = 0;

            var lstParents = new Array();
            for (var nodeIndex = 0; nodeIndex < lstRelatives.length; nodeIndex++) {
                var node = lstRelatives[nodeIndex];
                for (var parentIndex = 0; parentIndex < node.Parents.length; parentIndex++) {
                    var parent = node.Parents[parentIndex];
                    if (lstParents.indexOf(parent) == -1)
                        lstParents.push(parent);
                }
            }

            if (this.orientation == "topToBottom" || this.orientation == "bottomToTop") {
                for (var parentIndex = 0; parentIndex < lstParents.length; parentIndex++) {
                    var parent = lstParents[parentIndex];
                    var lstChildren = parent.LayoutChilds;

                    if (lstChildren.length > 0) {
                        var nodeLeft = lstChildren[0];
                        var nodeRight = lstChildren[lstChildren.length - 1];
                        fLeftChildPosition = nodeLeft.offsetX - nodeLeft.width / 2;
                        fRightChildPosition = nodeRight.offsetX + nodeRight.width / 2;
                        parent.offsetX = fLeftChildPosition + ((fRightChildPosition - fLeftChildPosition) / 2);

                    }
                }
            }
            else {
                for (var parentIndex = 0; parentIndex < lstParents.length; parentIndex++) {
                    var parent = lstParents[parentIndex];
                    var lstChildren = parent.LayoutChilds;

                    if (lstChildren.length > 0) {
                        var nodeTop = lstChildren[0];
                        var nodeBottom = lstChildren[lstChildren.length - 1];
                        fLeftChildPosition = nodeTop.offsetY - nodeTop.height / 2;
                        fRightChildPosition = nodeBottom.offsetY + nodeBottom.height / 2;
                        parent.offsetY = fLeftChildPosition + ((fRightChildPosition - fLeftChildPosition) / 2);

                    }
                }
            }
            //this._normalizeNodes(lstRelatives);
        };

        HierarchicalLayout.prototype._normalizeNodes = function (nodes) {
            var bSuccess = false;
            var nodeBound = 0;

            if (this.orientation == "topToBottom" || this.orientation == "bottomToTop") {
                for (var nCounter = 0; nCounter < nodes.length - 1; nCounter++) {
                    var node = nodes[nCounter];
                    var nodeNext = nodes[nCounter + 1];

                    nodeBound = node.offsetX + node.width / 2 + this.horizontalSpacing;

                    if (nodeBound > (nodeNext.offsetX - nodeNext.width / 2)) {
                        var fOffset = nodeBound + nodeNext.width / 2 - nodeNext.offsetX;
                        nodeNext.offsetX = nodeBound + nodeNext.width / 2;
                        bSuccess = true;

                        this._moveChildren(nodeNext, fOffset);
                    }
                }
            }
            else {
                for (var nCounter = 0; nCounter < nodes.length - 1; nCounter++) {
                    var node = nodes[nCounter];
                    var nodeNext = nodes[nCounter + 1];

                    nodeBound = node.offsetY + node.height / 2 + this.horizontalSpacing;
                    if (nodeBound > (nodeNext.offsetY - nodeNext.height / 2)) {
                        var fOffset = nodeBound + nodeNext.height / 2 - nodeNext.offsetY;
                        nodeNext.offsetY = nodeBound + nodeNext.height / 2;
                        bSuccess = true;

                        this._moveChildrenVertically(nodeNext, fOffset);
                    }
                }
            }
            return bSuccess;
        };

        HierarchicalLayout.prototype._moveChildren = function (nodeNext, fOffset) {
            // Move children
            if (nodeNext.ConnectorType == undefined) {
                var dtgnNodeNext = nodeNext;
                for (var gnChildIndex = 0; gnChildIndex < dtgnNodeNext.LayoutChilds.length; gnChildIndex++) {
                    var gnChild = dtgnNodeNext.LayoutChilds[gnChildIndex];
                    gnChild.offsetX += fOffset;
                    this._moveChildren(gnChild, fOffset);
                }
            }
        };

        HierarchicalLayout.prototype._moveChildrenVertically = function (nodeNext, fOffset) {
            // Move children
            if (nodeNext.ConnectorType == undefined) {
                var dtgnNodeNext = nodeNext;
                for (var gnChildIndex = 0; gnChildIndex < dtgnNodeNext.LayoutChilds.length; gnChildIndex++) {
                    var gnChild = dtgnNodeNext.LayoutChilds[gnChildIndex];
                    gnChild.offsetY += fOffset;
                    this._moveChildrenVertically(gnChild, fOffset);
                }
            }
        };

        HierarchicalLayout.prototype._updateOrientation = function () {
            if (this.orientation != "topToBottom" || this.orientation != "leftToRight") {
                var nodes = this._model.nodes, node;
                //var tempX, tempY;
                //var minXNode = this._getMinXNode(nodes);
                var maxXNode = this._getMaxXNode(nodes);
                //var minYNode = this._getMinYNode(nodes);
                var maxYNode = this._getMaxYNode(nodes);
                //var minX = minXNode.offsetX;
                var maxX = maxXNode.offsetX;
                //var minY = minYNode.offsetY;
                var maxY = maxYNode.offsetY;

                if (this.orientation == "bottomToTop") {
                    for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                        node = nodes[nodeIndex];
                        if (!node.exclude) {
                            node.offsetX = maxX - node.offsetX;
                            node.offsetY = maxY - node.offsetY;
                        }
                    }
                }
                else if (this.orientation == "rightToLeft") {
                    for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                        node = nodes[nodeIndex];
                        if (!node.exclude) {
                            node.offsetX = maxX - node.offsetX;
                            node.offsetY = maxY - node.offsetY;
                        }
                    }
                }
                //}
            }
        };

        HierarchicalLayout.prototype._getMaxXNode = function (nodes) {
            var maxX = nodes[0].offsetX, node, index = 0;
            for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                node = nodes[nodeIndex];
                if (!node.exclude) {
                    if (node.offsetX > maxX) {
                        maxX = node.offsetX;
                        index = nodeIndex;
                    }
                }
            }
            return nodes[index];
        };

        HierarchicalLayout.prototype._getMaxYNode = function (nodes) {
            var maxY = nodes[0].offsetY, node, index = 0;
            for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                node = nodes[nodeIndex];
                if (!node.exclude) {
                    if (node.offsetY > maxY) {
                        maxY = node.offsetY;
                        index = nodeIndex;
                    }
                }
            }
            return nodes[index];
        };

        HierarchicalLayout.prototype._arrangeNodesToLeftMost = function (fixedPosition) {
            var nodes = this._model.nodes, node;
            if (nodes.length > 0) {
                if (fixedPosition) {
                    var difx = fixedPosition.x - this.fixedNode.offsetX;
                    var dify = fixedPosition.y - this.fixedNode.offsetY;
                    for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                        node = nodes[nodeIndex];
                        if (!node.exclude) {
                            node.offsetX = node.offsetX + difx;
                            node.offsetY = node.offsetY + dify;
                        }
                        node.updatebounds();
                    }
                }
                else {
                    for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                        node = nodes[nodeIndex];
                        node.updatebounds();
                        this._adjustMargin(node);
                    }
                }
            }
        };

        HierarchicalLayout.prototype._adjustMargin = function (node) {
            node.offsetX = this.marginX + node.offsetX;
            node.offsetY = this.marginY + node.offsetY;
        };

        //separating elements by rank

        HierarchicalLayout.prototype._getNodesToPosition = function (nodes) {
            for (var index = 0; index < nodes.length; index++) {
                var node = nodes[index];
                if (this._passedNodes[node.name] == null && !node.exclude) {
                    this._selectNodes(node);
                    break;
                }
                else {
                    if (node.exclude) this._visitedNodes++;
                }
            }
        };

        HierarchicalLayout.prototype._selectNodes = function (node) {
            var nodeGraph = node;
            if (node.ConnectorType != undefined) {
                this._exploreGraphEdge(node);
            }
            else if (nodeGraph != null) {
                if (this._addNodeInCollection(node, "Passed")) {
                    this._addNodeInCollection(node, "Selected");

                    if (this._isConnectedToAnotherNode(nodeGraph)) {
                        // create the graph and set as Selected
                        this._selectedNodeToLayout = this._graph(node, this._improvePerformance, this._leftMargin, this._topMargin);
                        // fill graph
                        this._selectConnectedNodes(nodeGraph);
                        this.GraphNodes = this._selectedNodeToLayout.GraphNodes;
                        this._selectedNodeToLayout.TypeOrdered = [];
                        var typeOrdered = this._setTypeOrderd();
                        this._selectedNodeToLayout.TypeOrdered = typeOrdered;
                    }
                }
            }
        };

        HierarchicalLayout.prototype._selectConnectedNodes = function (nodeGraph) {
            var graph = this._selectedNodeToLayout;
            if (graph != null) {
                var node = nodeGraph;
                if (node != null && this._addNodeInCollection(node, "Passed")) {
                    var nodeName = node.name;
                    if (graph.GraphNodes[nodeName] == null) {
                        var gnNode = this._addGraphNode(node);
                        this._getConnectedRelatives(gnNode);
                        this._exploreRelative(nodeGraph);
                    }
                    else {
                        var graphNode = graph.GraphNodes[nodeName];
                        if (graphNode.Added) {
                            graphNode.Added = false;
                            this._getConnectedRelatives(graphNode);
                            this._exploreRelative(nodeGraph);
                        }
                    }
                }
            }
        };

        HierarchicalLayout.prototype._getConnectedRelatives = function (graphNode) {
            this._getConnectedParents(graphNode);
            this._getConnectedChildren(graphNode);
        };

        HierarchicalLayout.prototype._getConnectedChildren = function (graphNode) {
            var graph = this._selectedNodeToLayout;

            if (graph != null) {
                var nodeGraph = graphNode;
                for (var edgeIndex = 0; edgeIndex < nodeGraph.outEdges.length; edgeIndex++) {
                    var edge = nodeGraph.outEdges[edgeIndex];
                    var toNode = this._findNodeById(edge.targetNode.name)[0];
                    var node = toNode;
                    if (node && !node.exclude) {
                        if (this._model.nodes.indexOf(node) != -1 && node != null && node.visible) {
                            var gnNode;

                            if (graph.GraphNodes[node.name] == null) {
                                gnNode = this._addGraphNode(node);
                                gnNode.Added = true;
                            }
                            else {
                                gnNode = graph.GraphNodes[node.name];
                            }

                            // set Reference to parent node
                            this._setNode(gnNode.Parents, graphNode);

                            // add new node as child to parent node
                            if (this._findNode(graphNode.LayoutChilds, gnNode.name) < 0)
                                graphNode.LayoutChilds.push(gnNode);
                        }
                    }
                }
            }
        };

        HierarchicalLayout.prototype._getConnectedParents = function (graphNode) {
            var graph = this._selectedNodeToLayout;

            if (graph != null) {

                var nodeGraph = graphNode;
                for (var edgeIndex = 0; edgeIndex < nodeGraph.inEdges.length; edgeIndex++) {
                    var edge = nodeGraph.inEdges[edgeIndex];
                    var node = this._findNodeById(edge.sourceNode.name)[0];
                    if (node && !node.exclude) {
                        if (this._model.nodes.indexOf(node) != -1 && node != null && node.visible) {
                            var gnNode;
                            if (graph.GraphNodes[node.name] == null) {
                                gnNode = this._addGraphNode(node);
                                gnNode.Added = true;
                            }
                            else {
                                gnNode = graph.GraphNodes[node.name];
                            }

                            // set Reference to child node
                            this._setNode(gnNode.LayoutChilds, graphNode);

                            // add new node as parent to child node
                            if (this._findNode(graphNode.Parents, gnNode.name) < 0)
                                graphNode.Parents.push(gnNode);
                        }
                    }
                }
            }
        };

        HierarchicalLayout.prototype._setNode = function (list, node) {
            var nIndex = this._findNode(list, node.name);

            if (nIndex >= 0 && nIndex < list.length) {
                list[nIndex] = node;
            }
            else {
                list.push(node);
            }
        };

        HierarchicalLayout.prototype._findNode = function (list, name) {
            var nIndex = -1;

            if (list != null && name != "") {
                for (var i = 0, nLength = list.length; i < nLength; i++) {
                    var gnNode = list[i];

                    if (gnNode != null && gnNode.name == name) {
                        nIndex = i;
                        break;
                    }
                }
            }

            return nIndex;
        };

        HierarchicalLayout.prototype._exploreRelative = function (nodeGraph) {
            this._exploreRelatives(nodeGraph, "Parents");
            this._exploreRelatives(nodeGraph, "Children");
        };

        HierarchicalLayout.prototype._exploreRelatives = function (nodeGraph, relativesToExplore) {
            var edges = null;
            var graph = this._selectedNodeToLayout;

            if (relativesToExplore == "Parents") {
                edges = nodeGraph.inEdges;
            }
            else if (relativesToExplore == "Children") {
                edges = nodeGraph.outEdges;
            }
            for (var edgeIndex = 0; edgeIndex < edges.length; edgeIndex++) {
                var edge = edges[edgeIndex];
                if (this._addNodeInCollection(edge, "Passed")) {
                    var fromNode = this._findNodeById(edge.sourceNode.name)[0];
                    var toNode = this._findNodeById(edge.targetNode.name)[0];

                    if (relativesToExplore == "Parents" && fromNode != null && !fromNode.exclude && this._model.nodes.indexOf(fromNode) != -1) {
                        this._selectConnectedNodes(fromNode);
                    }
                    else if (relativesToExplore == "Children" && toNode != null && !toNode.exclude && this._model.nodes.indexOf(toNode) != -1) {
                        this._selectConnectedNodes(toNode);
                    }
                }
            }
        };

        HierarchicalLayout.prototype._addGraphNode = function (node) {
            var graph = this._selectedNodeToLayout;

            var gnNode = this._graphNode(node);

            if (graph != null) {
                graph.GraphNodes[gnNode.name] = gnNode;
            }

            return gnNode;
        };

        HierarchicalLayout.prototype._setTypeOrderd = function () {
            var m_bGraphTypeDetermined = true;

            var typeGraphToReturn = "OneParentDirectedTree";
            var graphNodes = this.GraphNodes;
            var gnNode = null;
            var bNoneDirected = true;
            var bMultipleParent = false;

            // search for node without parents.
            for (var nodeIndex = 0; nodeIndex < graphNodes.length; nodeIndex++) {
                var gnNode = graphNodes[nodeIndex];

                if (gnNode != null && gnNode.Parents) {
                    if (gnNode.Parents.length == 0)
                        bNoneDirected = false;

                    if (gnNode.Parents.length > 1) {
                        bMultipleParent = true;

                        if (!bNoneDirected)
                            break;
                    }
                }
            }

            if (bNoneDirected)
                typeGraphToReturn = "NonDirectedGraph";
            else if (bMultipleParent)
                typeGraphToReturn = "MultiParentDirectedTree";

            // 1 - sort nodes by y positions
            this._rankGraphNodeInitialization();

            if (this._lstRankOrdered != null && this._lstRankOrdered.length > 0) {
                // 2 -  add fictitious nodes
                this._addFictitiousNodes();

                // 3 - optimize graph. Remove intercetion.
                if (!this.m_improvePerformance) {
                    this._optimizeGraph();
                }

                // 4 Sort relatives to rank order
                // SortRelatives();
                // 6 - check for dimetions
                this._checkRankDimensions();
            }

            return this._lstRankOrdered;
        };

        HierarchicalLayout.prototype._addFictitiousNodes = function () {
            var lstParentsRank;
            var lstChildrenRank;
            for (var nLevel = 0, nLevelCount = this._lstRankOrdered.length - 1; nLevel < nLevelCount; nLevel++) {
                lstParentsRank = this._lstRankOrdered[nLevel];
                lstChildrenRank = this._lstRankOrdered[nLevel + 1];
                for (var gnNodeIndex = 0; gnNodeIndex < lstParentsRank.length; gnNodeIndex++) {
                    var gnNode = lstParentsRank[gnNodeIndex];
                    var lstChildren = gnNode.LayoutChilds;

                    for (var i = 0, nLength = lstChildren.length; i < nLength; i++) {
                        var gnChild = lstChildren[i];

                        // if child rank don't contains node child then add fictition node
                        if (lstChildrenRank.indexOf(gnChild) == -1) {
                            var gnNewNode = new graphNode(null);
                            gnNewNode.Size = GetMinimumDimension(lstChildrenRank);

                            var nChildIndex = gnNode.LayoutChilds.indexOf(gnChild);
                            var nParentIndex = gnChild.Parents.indexOf(gnNode);

                            //// parent change Reference to child
                            gnNode.LayoutChilds[nChildIndex] = gnNewNode;
                            //// child change reference to parent
                            gnChild.Parents[nParentIndex] = gnNewNode;
                            //// Ficted node set new  parent and child
                            gnNewNode.Parents.push(gnNode);
                            gnNewNode.LayoutChilds.push(gnChild);
                            lstChildrenRank.push(gnNewNode);
                            //// add node to storage list
                            this._fictitiousNodes.push(gnNewNode);
                        }
                    }
                }
            }
        };

        HierarchicalLayout.prototype._optimizeTopRank = function () {
            if (this._lstRankOrdered.length > 0) {
                var lstRank = this._lstRankOrdered[0];
                var lstRankCheck = lstRank;
                for (var gnNodeIndex = 0; gnNodeIndex < lstRankCheck.length; gnNodeIndex++) {
                    var gnNode = lstRankCheck[gnNodeIndex];
                    if (gnNode.Parents.length == 0 && gnNode.LayoutChilds.length == 1) {
                        var gnChild = gnNode.LayoutChilds[0];

                        // place node in center of children parents
                        var dCenterIndex = this._getCentralNode(gnChild.Parents, lstRank);

                        this.moveNode(lstRank, gnNode, dCenterIndex);
                    }
                }
            }
        };

        HierarchicalLayout.prototype._checkRankDimensions = function () {
            for (var nRankCounter = 0; nRankCounter < this._lstRankOrdered.length; nRankCounter++) {
                var lstRank = this._lstRankOrdered[nRankCounter];

                for (var nCounter = 0; nCounter < lstRank.length; nCounter++) {
                    var nodeToCompare = lstRank[nCounter];
                    this._checkDimensions(nodeToCompare, nRankCounter);
                }
            }
        };

        HierarchicalLayout.prototype._checkDimensions = function (nodeToCompare, nRank) {
            var nLength = this._biggestDimensions.length;
            var szRankBiggestDimension;

            if (nLength > nRank && nRank >= 0) {
                szRankBiggestDimension = this._biggestDimensions[nRank];

                if (this._isCurrentNodeDimensionBigger(szRankBiggestDimension, nodeToCompare, "Width"))
                    this._update(szRankBiggestDimension, nodeToCompare, "Width");

                if (this._isCurrentNodeDimensionBigger(szRankBiggestDimension, nodeToCompare, "Height"))
                    this._update(szRankBiggestDimension, nodeToCompare, "Height");

            }
            else {
                while (nRank >= nLength) {

                    this._biggestDimensions.push(this._sizeF());
                    nLength++;
                }

                while (nRank < 0) {
                    this._biggestDimensions.splice(0, 0, this._sizeF());
                    nRank++;
                }

                szRankBiggestDimension = this._sizeF();
                szRankBiggestDimension.width = nodeToCompare.width;
                szRankBiggestDimension.height = nodeToCompare.height;
            }
            this._biggestDimensions[nRank] = szRankBiggestDimension;
        };

        HierarchicalLayout.prototype._update = function (szDimensionToUpdate, node, dimension) {
            switch (dimension) {
                case "Width":
                    szDimensionToUpdate.width = node.width;
                    break;
                case "Height":
                    szDimensionToUpdate.height = node.height;
                    break;
            }
        };

        HierarchicalLayout.prototype._sizeF = function () {
            return { "Width": 0, "Height": 0 };
        };

        HierarchicalLayout.prototype._isCurrentNodeDimensionBigger = function (szRankBiggestDimension, boundsNode, dimension) {
            var bIsCurrentNodeDimensionBigger = false;

            switch (dimension) {
                case "Width":
                    if (boundsNode.width > szRankBiggestDimension.width)
                        bIsCurrentNodeDimensionBigger = true;

                    break;
                case "Height":
                    if (boundsNode.height > szRankBiggestDimension.height)
                        bIsCurrentNodeDimensionBigger = true;

                    break;
            }

            return bIsCurrentNodeDimensionBigger;
        };

        HierarchicalLayout.prototype._getCentralNode = function (lstRealtive, lstRankList) {
            var dToReturn = 0;
            var dIndex = 0;
            for (var gnNodeindex = 0; gnNodeindex < lstRealtive.length; gnNodeindex++) {
                var gnNode = lstRealtive[gnNodeindex];
                var index = lstRankList.indexOf(gnNode);

                if (index >= 0) {
                    dIndex += index + 0.5;
                }
            }

            if (lstRealtive.length > 0) {
                dToReturn = dIndex / lstRealtive.length;
            }

            return dToReturn;
        };

        HierarchicalLayout.prototype.moveNode = function (list, gnNode, nIndex) {
            var nCurIndex = list.indexOf(gnNode);

            if (nCurIndex != nIndex) {
                if (nIndex > nCurIndex)
                    nIndex--;

                if (nIndex >= 0) {
                    list.pop(gnNode);
                    list.splice(0, nIndex, gnNode)
                }
                else if (nIndex == list.Count) {
                    list.push(gnNode);
                }
            }
        };

        HierarchicalLayout.prototype._optimizeGraph = function () {
            var nBeforeIterceptionCount = this._getInterceptionCount();
            var nAlwaysIterceptionCount = this._getAlwaysInterceptionCount();
            var nAfterIterceptionCount = nAlwaysIterceptionCount;
            var nFinallyInterceptionCount = nAfterIterceptionCount;

            var lstOrderedClone = this._getListOrderedClone();

            // optimize by sort nodes in list
            if (nAlwaysIterceptionCount < nBeforeIterceptionCount) {
                this._sortRankNodes();
            }

            this._optimizeTopRank();

            var arrInterceptions = this._getInterception();
            nAfterIterceptionCount = arrInterceptions.length;

            if (nAfterIterceptionCount < nBeforeIterceptionCount) {
                lstOrderedClone = this._getListOrderedClone();
                nBeforeIterceptionCount = nAfterIterceptionCount;
            }

            if (nAlwaysIterceptionCount < nBeforeIterceptionCount) {
                this._removeInterceptions(arrInterceptions);
            }

            // get current interceptions
            nAfterIterceptionCount = this._getInterceptionCount();

            // if interception increase then roll back last ordered
            if (nAfterIterceptionCount > nBeforeIterceptionCount)
                this._lstRankOrdered = lstOrderedClone;

            nFinallyInterceptionCount = this._getInterceptionCount();
        };

        HierarchicalLayout.prototype._sortRankNodes = function () {
            var lstParentsRankList;
            var lstChildrenRankList;

            for (var i = 0, nLength = this._lstRankOrdered.length - 1; i < nLength; i++) {
                lstParentsRankList = this._lstRankOrdered[i];
                lstChildrenRankList = this._lstRankOrdered[i + 1];

                if (i == 0) {
                    this._lstRankOrdered[i + 1] = this._sortChildrenRankNodes(lstParentsRankList, lstChildrenRankList);

                    lstChildrenRankList = this._lstRankOrdered[i + 1];
                    this._lstRankOrdered[i] = this.sortParentRankNodes(lstParentsRankList, lstChildrenRankList);
                    lstParentsRankList = this._lstRankOrdered[i];

                    optimizeTopRank();
                }

                this._lstRankOrdered[i + 1] = this._sortChildrenRankNodes(lstParentsRankList, lstChildrenRankList);
            }
        };

        HierarchicalLayout.prototype._sortChildrenRankNodes = function (lstParentsRankList, lstChildrenRankList) {
            var arrChildrenRankList = lstChildrenRankList;
            var nLength = arrChildrenRankList.length;

            var arrChildCount = []
            var arrPosition = []

            for (var i = 0; i < nLength; i++) {
                var gnChild = lstChildrenRankList[i];
                arrChildCount[i] = gnChild.Parents.length;
            }

            Array.sort(arrChildCount, arrChildrenRankList);
            var lstNewRankList = new Array(lstChildrenRankList);

            for (var i = 0; i < nLength; i++) {
                var gnChild = arrChildrenRankList[i];

                var nParentCount = lstParentsRankList.length;
                var nChildrenCount = lstChildrenRankList.length;

                if (nChildrenCount > 0 && nParentCount > 0) {
                    var dIndex = lstChildrenRankList.indexOf(gnChild) + Number(0.5);
                    var dParentIndex = this._getCentralNode(gnChild.Parents, lstParentsRankList);
                    var dChildIndex = this._getChildIndexInParentList(dIndex, nChildrenCount, nParentCount);

                    dIndex = this._getParentIndexInChildList(dParentIndex, nChildrenCount, nParentCount);

                    var dMoveNode = this._compareChilds(gnChild, lstParentsRankList);
                    var dPalceNode = this._compareChilds(arrChildrenRankList[dIndex], lstParentsRankList);

                    // move left or right by remove interceptions
                    if (dMoveNode > 0 && dPalceNode < 0)
                        dIndex++;

                    arrPosition[i] = dIndex;
                }
            }

            Array.sort(arrPosition, arrChildrenRankList);

            // return lstNewRankList;
            return new Array(arrChildrenRankList);
        };

        HierarchicalLayout.prototype._getListOrderedClone = function () {
            var lstToReturn = new Array();
            for (var listIndex = 0; listIndex < this._lstRankOrdered.length; listIndex++) {
                var list = this._lstRankOrdered[listIndex];
                lstToReturn.push(ej.Diagram.Util.clone($.extend(list)[0]));
            }

            return lstToReturn;
        };

        HierarchicalLayout.prototype._getInterceptionCount = function () {
            return this._getInterception().length;
        };

        HierarchicalLayout.prototype._getInterception = function () {
            var lstInterceptions = new Array();
            var lstParents;
            var lstChildren;
            var nCurIndex = -1;

            for (var i = 0, nLength = this._lstRankOrdered.length - 1; i < nLength; i++) {
                lstParents = this._lstRankOrdered[i];
                lstChildren = this._lstRankOrdered[i + 1];
                for (var leftChildIndex = 0; leftChildIndex < lstChildren.length; leftChildIndex++) {
                    var leftChild = lstChildren[leftChildIndex];
                    var nIndex = -1;
                    var nParentIndex = -1;
                    nCurIndex = lstChildren.indexOf(leftChild);
                    for (var rightParentIndex = 0; rightParentIndex < leftChild.Parents.length; rightParentIndex++) {
                        var rightParent = leftChild.Parents[rightParentIndex];
                        nIndex = lstParents.indexOf(rightParent);
                        for (var rightChildIndex = 0; rightChildIndex < lstChildren.length; rightChildIndex++) {
                            var rightChild = lstChildren[rightChildIndex];
                            if (leftChild == rightChild || nCurIndex >= lstChildren.indexOf(rightChild))
                                continue;
                            for (var leftParentIndex = 0; leftParentIndex < rightChild.Parents.length; leftParentIndex++) {
                                var leftParent = rightChild.Parents[leftParentIndex];
                                nParentIndex = lstParents.indexOf(leftParent);

                                if (nParentIndex >= 0 && nIndex >= 0 && nParentIndex < nIndex) {
                                    var bSuccess = true;
                                    for (var interceptionIndex = 0; interceptionIndex < lstInterceptions.length; interceptionIndex++) {
                                        var interception = lstInterceptions[interceptionIndex];
                                        if ((interception.LeftParent == leftParent && interception.RightParent == rightParent
                                                && interception.LeftChild == leftChild && interception.RightChild == rightChild)
                                                || (interception.LeftParent == rightParent && interception.RightParent == leftParent
                                                && interception.LeftChild == rightChild && interception.RightChild == leftChild)) {
                                            bSuccess = false;
                                            break;
                                        }
                                    }

                                    if (bSuccess)
                                        lstInterceptions.push(this._interception(leftParent, rightChild, rightParent, leftChild));
                                }
                            }
                        }
                    }
                }
            }

            return lstInterceptions;
        };

        HierarchicalLayout.prototype._interception = function (leftParent, rightChild, rightParent, leftChild) {
            this._leftChild = leftChild;
            this._leftParent = leftParent;
            this._rightChild = rightChild;
            this._rightParent = rightParent;

            var bSuccess = this._xCrossCondition();

            if (!bSuccess)
                bSuccess = !this._parentParentCondition();

            bCanFixed = !bSuccess;
        };

        HierarchicalLayout.prototype._xCrossCondition = function () {
            var lstLeftParents = this._leftChild.Parents;
            var lstRightParents = this._rightChild.Parents;

            return (lstLeftParents.indexOf(this._leftParent) != -1 && lstRightParents.indexOf(this._rightParent) != -1);
        };

        HierarchicalLayout.prototype._parentParentCondition = function () {
            var bSuccess = false;

            if (this._leftParent.Parents.length == 1 && this._rightParent.Parents.length == 1) {
                bSuccess = (this._leftParent.Parents[0] == this._rightParent.Parents[0]);
            }

            return bSuccess;
        };

        HierarchicalLayout.prototype._getAlwaysInterceptionCount = function () {
            var nCrossCount = 0;
            var nXCrossCount = 0;

            var lstCrossedNodes = new Array();
            var lstParentsPassed = new Array();
            var lstChildrenPassed = new Array();

            // get current and next list in rank list Collection
            for (var n = 0, nLstLegth = this._lstRankOrdered.length - 1; n < nLstLegth; n++) {
                // parent list
                var lstParentRank = this._lstRankOrdered[n];

                // children list
                var lstChildrenRank = this._lstRankOrdered[n + 1];

                // look throwparent
                for (var i = 0, nLength = lstParentRank.length; i < nLength; i++) {
                    var gnParent = lstParentRank[i];
                    lstParentsPassed.push(gnParent);

                    // X CROSS CONDITION
                    // if parent has child, who has a parent, who has a child, who has a parent 
                    // what equals start parent then we have a interception.
                    // check left children
                    for (var gnLeftChildIndex = 0; gnLeftChildIndex < gnParent.LayoutChilds.length; gnLeftChildIndex++) {
                        var gnLeftChild = gnParent.LayoutChilds[gnLeftChildIndex];
                        if (lstChildrenRank.indexOf(gnLeftChild) == -1)
                            continue;

                        // chek right parents
                        for (var gnRightParentIndex = 0; gnRightParentIndex < gnLeftChild.Parents.length; gnRightParentIndex++) {
                            var gnRightParent = gnLeftChild.Parents[gnRightParentIndex];
                            if (gnRightParent != gnParent && lstParentsPassed.indexOf(gnRightParent) == -1) {
                                // check right children
                                for (var gnRightChildIndex = 0; gnRightChildIndex < gnRightParent.LayoutChilds.length; gnRightChildIndex++) {
                                    var gnRightChild = gnRightParent.LayoutChilds[gnRightChildIndex];
                                    // if that child already used skip it
                                    if (lstChildrenRank.indexOf(gnRightChild) == -1)
                                        continue;

                                    if (gnRightChild != gnLeftChild) {
                                        // chek left parents
                                        for (var gnLeftParentIndex = 0; gnLeftParentIndex < gnRightChild.Parents.length; gnLeftParentIndex++) {
                                            var gnLeftParent = gnRightChild.Parents[gnLeftParentIndex];
                                            // if start parent equals end parent then we have a link interception
                                            if (gnLeftParent == gnParent) {
                                                nXCrossCount++;
                                                lstCrossedNodes.push(gnLeftParent);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                lstChildrenPassed = [];
                for (var j = 0, count = lstChildrenRank.length; j < count; j++) {
                    var gnChild = lstChildrenRank[j];

                    // CHILD PARENTS CONDITION
                    // if we has a node who has three and more parent with 2 
                    // and more children then we have a interception.
                    for (var gnParentNodeIndex = 0; gnParentNodeIndex < gnChild.Parents.length; gnParentNodeIndex++) {
                        var gnParentNode = gnChild.Parents[gnParentNodeIndex];
                        if (lstCrossedNodes.indexOf(gnParentNode) != -1) {
                            lstChildrenPassed.Clear();
                            break;
                        }

                        var nCount = gnParentNode.LayoutChilds.length;

                        if (nCount > 1)
                            lstChildrenPassed.push(nCount);
                    }

                    if (lstChildrenPassed.length > 2) {
                        var counts = lstChildrenPassed;

                        for (var k = 0, length = counts.length - 2; k < length; k++)
                            nCrossCount += counts[k] - 1;
                    }

                    lstChildrenPassed = [];
                }
            }

            nCrossCount += nXCrossCount / 2;

            return nCrossCount;
        };

        HierarchicalLayout.prototype._rankGraphNodeInitialization = function () {
            var gnTopNode = this._getGraphFirstTopNode();
            this._rankGraphNodes(gnTopNode, 0, false);
            // remove emptyLevels
            var lstEmpty = new Array();
            for (var listIndex = 0; listIndex < this._lstRankOrdered.length; listIndex++) {
                var list = this._lstRankOrdered[listIndex];
                if (list.length == 0)
                    lstEmpty.push(list);
            }
            for (var listIndex = 0; listIndex < lstEmpty.length; listIndex++) {
                var list = lstEmpty[listIndex];
                this._lstRankOrdered.Remove(list);
            }
        };

        HierarchicalLayout.prototype._addNodeInTypeOrederd = function (gnNodeToAdd, nRank) {
            if (this._lstRankOrdered == null)
                this._lstRankOrdered = new Array();

            var nLength = this._lstRankOrdered.length;

            while (nRank < 0) {
                this._lstRankOrdered.splice(0, 0, new Array());
                nRank++;
            }

            while (nRank >= nLength) {
                this._lstRankOrdered.push(new Array());
                nLength++;
            }

            var lstRankToAddNodeTo = this._lstRankOrdered[nRank];
            lstRankToAddNodeTo.push(gnNodeToAdd);

            return nRank;
        };

        HierarchicalLayout.prototype._getGraphFirstTopNode = function () {
            var gnTopToReturn = null;
            for (var gnNodeIndex in this.GraphNodes) {
                var gnNode = this.GraphNodes[gnNodeIndex];
                if (gnNode != null && gnNode.Parents) {
                    if (gnNode.Parents.length == 0) {
                        gnTopToReturn = gnNode;
                        break;
                    }
                }
            }

            return gnTopToReturn;
        };

        HierarchicalLayout.prototype._rankGraphNodes = function (gnNode, nRank, toParent) {
            if (gnNode != null) {
                var nIndex = 0;
                var nNodeRank = this._getGraphNodeRank(gnNode, nIndex);

                // if node is exist in rank list
                if (nNodeRank >= 0) {
                    // if him level right then return current rank
                    if (this._checkRankPosition(gnNode, nNodeRank, toParent, nRank)
                            || nRank == nNodeRank) {
                        return nRank;
                    }

                    // else remove this node from his rankLevel list
                    var lstRank = this._lstRankOrdered[nNodeRank];
                    lstRank.pop(gnNode);
                }

                // add and get current node rank
                nRank = this._addNodeInTypeOrederd(gnNode, nRank);
                for (var gnChildIndex = 0; gnChildIndex < gnNode.LayoutChilds.length; gnChildIndex++) {
                    var gnChild = gnNode.LayoutChilds[gnChildIndex];
                    this._rankGraphNodes(gnChild, nRank + 1, true);
                }
                if (!this.m_improvePerformance) {
                    for (var gnParentIndex = 0; gnParentIndex < gnNode.Parents.length; gnParentIndex++) {
                        var gnParent = gnNode.Parents[gnParentIndex];
                        this._rankGraphNodes(gnParent, nRank - 1, false);
                    }
                }
            }

            return nRank;
        };

        HierarchicalLayout.prototype._getGraphNodeRank = function (gnNode, nIndex) {
            var nRankToReturn = -1;
            var bSuccess = false;
            nIndex = -1;

            if (this._lstRankOrdered != null) {
                for (var lstRankIndex = 0; lstRankIndex < this._lstRankOrdered.length; lstRankIndex++) {
                    var lstRank = this._lstRankOrdered[lstRankIndex];
                    if (bSuccess)
                        break;
                    for (var gnRankIndex = 0; gnRankIndex < lstRank.length; gnRankIndex++) {
                        var gnRank = lstRank[gnRankIndex];
                        if (gnRank.name == gnNode.name) {
                            nRankToReturn = this._lstRankOrdered.indexOf(lstRank);
                            nIndex = lstRank.indexOf(gnRank);

                            bSuccess = true;
                            break;
                        }
                    }
                }
            }

            return nRankToReturn;
        };

        HierarchicalLayout.prototype._checkRankPosition = function (gnNode, nNodeRank, bParentChild, nRank) {
            nRank = nNodeRank;
            var bCorrectPosition = true;

            var nIndex;
            var nMaxParentIndex = Number.MIN_VALUE;
            var nParentRank = -1;
            var nMinChildrenIndex = Number.MAX_VALUE;
            var nChildRank = -1;
            for (var gnParentIndex = 0; gnParentIndex < gnNode.Parents.length; gnParentIndex++) {
                var gnParent = gnNode.Parents[gnParentIndex];
                nParentRank = this._getGraphNodeRank(gnParent, nIndex);

                if (nParentRank < 0)
                    nParentRank = 0;

                nMaxParentIndex = Math.max(nMaxParentIndex, nParentRank);
            }
            for (var gnChildIndex = 0; gnChildIndex < gnNode.LayoutChilds.length; gnChildIndex++) {
                var gnChild = gnNode.LayoutChilds[gnChildIndex];
                nChildRank = this._getGraphNodeRank(gnChild, nIndex);

                if (nIndex != -1)
                    nMinChildrenIndex = Math.min(nMinChildrenIndex, nChildRank);
            }

            if (nMaxParentIndex != Number.MIN_VALUE) {
                nRank = nMaxParentIndex + 1;
                bCorrectPosition = (nNodeRank >= nRank);
            }
            else if (nMinChildrenIndex != Number.MAX_VALUE && nMinChildrenIndex != 0) {
                nRank = nMinChildrenIndex - 1;
                bCorrectPosition = (nNodeRank == nRank);
            }

            return bCorrectPosition;
        };

        HierarchicalLayout.prototype._graph = function (node, improvePerformance, leftMargin, topMargin) {
            this._improvePerformance = improvePerformance;
            this._leftMargin = leftMargin;
            this._topMargin = topMargin;
            var GraphNodes = [];
            var Added = false;
            var Bounds = { X: node.offsetX, Y: node.offsetY, Width: node.width, Height: node.height };
            var Location = { X: node.offsetX, Y: node.offsetY };
            return {
                "GraphNodes": GraphNodes,
                "Added": Added,
                "Bounds": Bounds,
                "Location": Location
            };
        };

        HierarchicalLayout.prototype._graphNode = function (node) {
            node.Tag = null;
            node.Visited = false;
            node.LayoutChilds = [];
            node.Parents = [];
            node.Distance = [];
            return node;
        };

        HierarchicalLayout.prototype._exploreGraphEdge = function (node) {
            var nodeLink = node;

            if (nodeLink != null && this._addNodeInCollection(node, "Passed")) {
                this._addNodeInCollection(node, "Selected");
                var fromNode = this._findNodeById(nodeLink.targetNode.name)[0];
                var toNode = this._findNodeById(nodeLink.sourceNode.name)[0];

                if (fromNode != null) {
                    this._selectNodes(fromNode);
                }
                else if (toNode != null) {
                    this._selectNodes(toNode);
                }
                else
                    this._selectedNodeToLayout = node;
            }
        };

        HierarchicalLayout.prototype._isConnectedToAnotherNode = function (gnNode) {
            var bFoundConnectedNode = false;

            if ((gnNode.inEdges != null) && (gnNode.inEdges.length > 0))
                bFoundConnectedNode = this._searchEdgeCollection(gnNode.inEdges, "FromNode");

            if ((!bFoundConnectedNode) && (gnNode.outEdges != null) && (gnNode.outEdges.length > 0))
                bFoundConnectedNode = this._searchEdgeCollection(gnNode.outEdges, "ToNode");


            return bFoundConnectedNode;
        };

        HierarchicalLayout.prototype._searchEdgeCollection = function (edgesToSearchThrough, connectionDirection) {
            var foundConnectedNode = false;
            for (var edgeIndex = 0; edgeIndex < edgesToSearchThrough.length; edgeIndex++) {
                var edge = edgesToSearchThrough[edgeIndex];
                if (!this._addNodeInCollection(edge, "Passed"))
                    continue;
                var fromNode = this._findNodeById(edge.sourceNode.name)[0];
                var toNode = this._findNodeById(edge.targetNode.name)[0];
                if ((connectionDirection == "FromNode" && fromNode != null && !fromNode.exclude)
                        || (connectionDirection == "ToNode" && toNode != null && !toNode.exclude)) {
                    foundConnectedNode = true;
                    break;
                }
            }
            return foundConnectedNode;
        };

        HierarchicalLayout.prototype._findNodeById = function (nodeId) {
            // get the node
            var findNode = [];
            var nodes = this._model.nodes;
            if (nodeId != null && nodeId != "") {
                for (var i = 0; i < nodes.length; i++) {
                    if (nodes[i].name == nodeId) {
                        findNode.push(nodes[i]);
                        break;
                    }
                    if (findNode.length == 0) {
                        if (nodes[i].LayoutChilds && nodes[i].LayoutChilds.length > 0) {
                            for (var j = 0; j < nodes[i].LayoutChilds.length; j++) {
                                if (nodes[i].LayoutChilds[j].name == nodeId) {
                                    findNode.push(nodes[i].LayoutChilds[j]);
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            return findNode;
        };

        HierarchicalLayout.prototype._addNodeInCollection = function (node, collectionToAdd) {
            var result = true;
            if (node != null) {

                if (collectionToAdd == "Passed" || !node.visible) {
                    if (this._passedNodes[node.name] == null) {
                        this._passedNodes[node.name] = node;
                        this._passedNodesList.push(node);
                        this._visitedNodes++;
                    }
                }

                // Check for node visible
                if (!node.visible)
                    return false;
            }
            return result;
        };


        //to comment

        HierarchicalLayout.prototype._getMinXNode = function (nodes) {
            var minX = nodes[0].offsetX, node, index = 0;
            for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                node = nodes[nodeIndex];
                if (node.offsetX < minX) {
                    minX = node.offsetX;
                    index = nodeIndex;
                }
            }
            return nodes[index];
        };

        HierarchicalLayout.prototype._getMaxXNode = function (nodes) {
            var maxX = nodes[0].offsetX, node, index = 0;
            for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                node = nodes[nodeIndex];
                if (node.offsetX > maxX) {
                    maxX = node.offsetX;
                    index = nodeIndex;
                }
            }
            return nodes[index];
        };

        HierarchicalLayout.prototype._getMaxYNode = function (nodes) {
            var maxY = nodes[0].offsetY, node, index = 0;
            for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                node = nodes[nodeIndex];
                if (!node.exclude) {
                    if (node.offsetY > maxY) {
                        maxY = node.offsetY;
                        index = nodeIndex;
                    }
                }
            }
            return nodes[index];
        };

        HierarchicalLayout.prototype._getMinYNode = function (nodes) {
            var minY = nodes[0].offsetY, node, index = 0;
            for (var nodeIndex = 0; nodeIndex < nodes.length; nodeIndex++) {
                node = nodes[nodeIndex];
                if (!node.exclude) {
                    if (node.offsetY < minY) {
                        minY = node.offsetY;
                        index = nodeIndex;
                    }
                }
            }
            return nodes[index];
        };

        HierarchicalLayout.prototype._resetLayoutInfo = function () {
            this._passedNodesList = [];
            this._passedNodes = [];
            this._lstRankOrdered = [];
            this._fictitiousNodes = [];
            this._biggestDimensions = [];
            this._selectedNodeToLayout = null;
            this._improvePerformance = null;
            this._lstRankOrdered = [];
            this._fictitiousNodes = [];
            this._biggestDimensions = [];
            this._yBorder = 0;
            this._xBorder = 0;
            this._leftChild = null;
            this._leftParent = null;
            this._rightChild = null;
            this._rightParent = null;
            this._fX = 0;
        };

        HierarchicalLayout.prototype._makeTreeLayout = function (lstToSort) {
            for (var nCounter = lstToSort.length - 1; nCounter >= 0; nCounter--) {
                var lstSort = lstToSort[nCounter];
                this._updateTreeToTop(lstToSort, nCounter);
                this._makeXLayoutRelative(lstToSort, lstSort, false, nCounter);
            }
            if (this.orientation == "topToBottom" || this.orientation == "bottomToTop") {
                for (var nCounter = 0; nCounter < lstToSort.length; nCounter++) {
                    var hashToSort = lstToSort[nCounter];
                    for (var nodeIndex = 0; nodeIndex < hashToSort.length; nodeIndex++) {
                        var node = hashToSort[nodeIndex];
                        if (node.Parents.length > 1) {
                            var startNode = node.Parents[0];
                            var fMinX = startNode.offsetX;
                            var fMaxX = startNode.offsetX;
                            for (var i = 0; i < node.Parents.Count; i++) {
                                var currentNode = node.Parents[i];
                                if (currentNode.offsetX < fMinX)
                                    fMinX = currentNode.offsetX;
                                if (currentNode.offsetX > fMaxX)
                                    fMaxX = currentNode.offsetX;
                            }
                            var fCenter = (fMaxX - fMinX) / 2;
                            node.offsetX = fMinX;
                            node.offsetX = node.offsetX + fCenter;
                            for (var childIndex = 0; childIndex < node.LayoutChilds.length; childIndex++) {
                                var child = node.LayoutChilds[childIndex];
                                child.offsetX = child.offsetX + fCenter + node.Distance;
                                child.Distance = node.Distance;
                                for (var innerChildIndex = 0; innerChildIndex < child.LayoutChilds.length; innerChildIndex++) {
                                    var innerChild = child.LayoutChilds[innerChildIndex];
                                    innerChild.Distance = fCenter;
                                }
                            }
                        }
                    }
                }
            }
            else {
                for (var nCounter = 0; nCounter < lstToSort.length; nCounter++) {
                    var hashToSort = lstToSort[nCounter];
                    for (var nodeIndex = 0; nodeIndex < hashToSort.length; nodeIndex++) {
                        var node = hashToSort[nodeIndex];
                        if (node.Parents.length > 1) {
                            var startNode = node.Parents[0];
                            var fMinX = startNode.offsetY;
                            var fMaxX = startNode.offsetY;
                            for (var i = 0; i < node.Parents.Count; i++) {
                                var currentNode = node.Parents[i];
                                if (currentNode.offsetY < fMinX)
                                    fMinX = currentNode.offsetY;
                                if (currentNode.offsetY > fMaxX)
                                    fMaxX = currentNode.offsetY;
                            }
                            var fCenter = (fMaxX - fMinX) / 2;
                            node.offsetY = fMinX;
                            node.offsetY = node.offsetY + fCenter;
                            for (var childIndex = 0; childIndex < node.LayoutChilds.length; childIndex++) {
                                var child = node.LayoutChilds[childIndex];
                                child.offsetY = child.offsetY + fCenter + node.Distance;
                                child.Distance = node.Distance;
                                for (var innerChildIndex = 0; innerChildIndex < child.LayoutChilds.length; innerChildIndex++) {
                                    var innerChild = child.LayoutChilds[innerChildIndex];
                                    innerChild.Distance = fCenter;
                                }
                            }
                        }
                    }
                }
            }
        };

        HierarchicalLayout.prototype._updateTreeToTop = function (hashSorting, nRank) {
            var nCounter = nRank;

            while (nCounter >= 0) {
                var hashToSort = hashSorting[nCounter];
                this._makeXLayoutRelative(hashSorting, hashToSort, true, nCounter);
                nCounter--;
            }
        };

        HierarchicalLayout.prototype._makeXLayoutRelative = function (lstRankedNodes, lstToSort, bUpdateTree, nRank) {
            if (lstToSort == null)
                return;

            var lstToNormalize = lstToSort;
            for (var nodeIndex = 0; nodeIndex < lstToSort.length; nodeIndex++) {
                var node = lstToSort[nodeIndex];
                if (bUpdateTree) {
                    this._placeRelativeToChildren(node.LayoutChilds);
                }
                else if (nRank > 0) {
                    this._placeRelativeToParent(node.Parents);
                }
                this._normalizeNodes(lstToNormalize);
            }

        };

        HierarchicalLayout.prototype._placeRelativeToParent = function (lstParents) {
            var fParentMedian = 0;
            var nPosition = 0;
            var nChildrenWidth = 0;

            var lstChildren;

            if (this.orientation == "topToBottom" || this.orientation == "bottomToTop") {

                for (var nodeParentIndex = 0; nodeParentIndex < lstParents.length; nodeParentIndex++) {
                    var nodeParent = lstParents[nodeParentIndex];
                    fParentMedian = nodeParent.offsetX;// + (nodeParent.width / 2);
                    nChildrenWidth = -this.horizontalSpacing;

                    lstChildren = nodeParent.LayoutChilds;
                    for (var nodeIndex = 0; nodeIndex < lstChildren.length; nodeIndex++) {
                        var node = lstChildren[nodeIndex];
                        nChildrenWidth += node.width + this.horizontalSpacing;
                    }

                    nPosition = -(nChildrenWidth) / 2;
                    for (var nodeIndex = 0; nodeIndex < lstChildren.length; nodeIndex++) {
                        var node = lstChildren[nodeIndex];
                        node.offsetX = fParentMedian + nPosition + node.width / 2;
                        node.Visited = true;
                        nPosition += node.width + this.horizontalSpacing;
                    }
                    for (var nodeIndex = 0; nodeIndex < lstChildren.length; nodeIndex++) {
                        var node = lstChildren[nodeIndex];
                        var parents = node.Parents;
                        if (parents.length > 1) {
                            var fX = this._getCenterXPosition(parents);
                            node.offsetX = fX;// - node.width / 2;
                        }
                    }
                }
            }
            else {
                for (var nodeParentIndex = 0; nodeParentIndex < lstParents.length; nodeParentIndex++) {
                    var nodeParent = lstParents[nodeParentIndex];
                    fParentMedian = nodeParent.offsetY;// + (nodeParent.height / 2);
                    nChildrenWidth = -this.verticalSpacing;

                    lstChildren = nodeParent.LayoutChilds;
                    for (var nodeIndex = 0; nodeIndex < lstChildren.length; nodeIndex++) {
                        var node = lstChildren[nodeIndex];
                        nChildrenWidth += node.height + this.verticalSpacing;
                    }

                    nPosition = -(nChildrenWidth) / 2;
                    for (var nodeIndex = 0; nodeIndex < lstChildren.length; nodeIndex++) {
                        var node = lstChildren[nodeIndex];
                        node.offsetY = fParentMedian + nPosition + node.height / 2;
                        node.Visited = true;
                        nPosition += node.height + this.verticalSpacing;
                    }
                    for (var nodeIndex = 0; nodeIndex < lstChildren.length; nodeIndex++) {
                        var node = lstChildren[nodeIndex];
                        var parents = node.Parents;
                        if (parents.length > 1) {
                            var fY = this._getCenterYPosition(parents);
                            node.offsetY = fY;// - node.height / 2;
                        }
                    }
                }
            }
        };

        HierarchicalLayout.prototype._getNodesWidth = function (list) {
            var fWidth = 0;
            var nCount = list.length;

            if (nCount > 0) {
                var gnFirstNode = list[0];
                var gnLastNodet = list[nCount - 1];

                fWidth = (gnLastNodet.offsetX + gnLastNodet.width) - gnFirstNode.offsetX;
            }

            return fWidth;
        };

        HierarchicalLayout.prototype._getNodesHeight = function (list) {
            var fWidth = 0;
            var nCount = list.length;

            if (nCount > 0) {
                var gnFirstNode = list[0];
                var gnLastNodet = list[nCount - 1];

                fWidth = (gnLastNodet.offsetY + gnLastNodet.height) - gnFirstNode.offsetY;
            }

            return fWidth;
        };

        HierarchicalLayout.prototype._getCenterXPosition = function (list) {
            var fCenter = 0;
            var startNode = list[0];
            var fMinX = startNode.offsetX;
            var fMaxX = startNode.offsetX;
            for (var i = 0; i < list.length; i++) {
                var currentNode = list[i];
                if (currentNode.offsetX < fMinX)
                    fMinX = currentNode.offsetX;
                if (currentNode.offsetX > fMaxX)
                    fMaxX = currentNode.offsetX;
            }
            fCenter = (fMaxX - fMinX) / 2;
            return fCenter;
        };

        HierarchicalLayout.prototype._getCenterYPosition = function (list) {
            var fCenter = 0;
            var startNode = list[0];
            var fMinX = startNode.offsetY;
            var fMaxX = startNode.offsetY;
            for (var i = 0; i < list.length; i++) {
                var currentNode = list[i];
                if (currentNode.offsetY < fMinX)
                    fMinX = currentNode.offsetY;
                if (currentNode.offsetY > fMaxX)
                    fMaxX = currentNode.offsetY;
            }
            fCenter = (fMaxX - fMinX) / 2;
            return fCenter;
        };

        HierarchicalLayout.prototype._getChildByParentCount = function (lstChildren, nParentCount) {
            var lstToReturn = new Array();
            for (var gnChildIndex = 0; gnChildIndex < lstChildren.length; gnChildIndex++) {
                var gnChild = lstChildren[gnChildIndex];
                if (gnChild.Parents.length >= nParentCount)
                    lstToReturn.push(gnChild);
            }

            return lstToReturn;
        };

        HierarchicalLayout.prototype._moveLeftRelatives = function (node, fOffset) {
            if (node.ConntectorType == undefined) {
                var dtgnNode = node;
                for (var dtgnParentIndex = 0; dtgnParentIndex < dtgnNode.Parents.length; dtgnParentIndex++) {

                    var dtgnParent = dtgnNode.Parents[dtgnParentIndex];
                    for (var nodeChildIndex = 0; nodeChildIndex < dtgnParent.LayoutChilds.length; nodeChildIndex++) {
                        var nodeChild = dtgnParent.LayoutChilds[nodeChildIndex];
                        if (node != nodeChild)
                            break;

                        nodeChild.offsetX += fOffset;
                        this._moveChildren(nodeChild, fOffset);
                    }
                }
            }
        };

        return HierarchicalLayout;
    })(Layout);

    ej.Diagram.HierarchicalLayout = function (options) {
        return new HierarchicalLayout(options);
    };
    //#endregion

    //#endregion

})(jQuery, Syncfusion);;

(function ($, ej) {
    "use strict";

    //#region ZoomAndPan
    var ZoomAndPan = (function () {
        function ZoomAndPan(diagram) {
            this.diagram = diagram;
            this.currentZoom = 1;
        }

        ZoomAndPan.prototype.zoomPanDelta = function (deltaX, deltaY, deltaScale, focusPoint, isZoom) {
            var mat = new ej.Diagram.Matrix();
            mat = mat._createIdentity();
            mat._scale(this.currentZoom, this.currentZoom);
            mat._translate(-this.diagram._scroller._horizontalOffset, -this.diagram._scroller._verticalOffset);
            var newScale = this.currentZoom * deltaScale;
            if (newScale > 30) {
                deltaScale = 30 / this.currentZoom;
            }
            else if (newScale < 0.25) {
                deltaScale = 0.25 / this.currentZoom;
            }
            var delta = new ej.Diagram.Matrix();
            delta = delta._createIdentity();
            var pivot;
            if (focusPoint) {
                pivot = mat._transform(focusPoint);
            }
            else {
                pivot = new ej.Diagram.Point($(this.diagram.element).width() / 2, $(this.diagram.element).height() / 2);
            }
            delta._scaleAt(deltaScale, deltaScale, pivot.x, pivot.y);
            newScale = this.currentZoom * deltaScale;
            delta._translate(deltaX, deltaY);
            mat._multiplyWith(delta);
            var point = mat._transform(new ej.Diagram.Point(0, 0));
            if (!isZoom) {
                if (point.x > 0)
                    point.x = 0;
                if (point.y > 0)
                    point.y = 0;
            }
            this._applyTransform(newScale, point, isZoom);
        };
        ZoomAndPan.prototype._applyTransform = function (scale, point, isZoom) {
            this.currentZoom = scale;
            var bounds = this.diagram._getDigramBounds();
            var zoomx = 0;
            var zoomy = 0;
            var width = this.diagram._svgWidth * scale;
            var height = this.diagram._svgHeight * scale;
            if (point.x * -1 > 0) {
                if (!((bounds.width * scale) < $(this.diagram.element).width())) {
                    zoomx = point.x * -1;
                }
            }
            if (point.y * -1 > 0) {
                if (!((bounds.height * scale) < $(this.diagram.element).height())) {
                    zoomy = point.y * -1;
                }
            }
            if (!isZoom) {
                var scrollModel = this.diagram._scroller._currentScroller.model;
                if (point.x * -1 > 0) {
                    if (point.x * -1 + (scrollModel.width - scrollModel.buttonSize) > bounds.width * this.currentZoom) {
                        zoomx = (bounds.width * this.currentZoom) - (scrollModel.width - scrollModel.buttonSize);
                    }
                }
                if (point.y * -1 > 0) {
                    if (point.y * -1 + (scrollModel.height - scrollModel.buttonSize) > bounds.height * this.currentZoom) {
                        zoomy = (bounds.height * this.currentZoom) - (scrollModel.height - scrollModel.buttonSize);
                    }
                }
            }
            this.diagram._scroller._updateOffset(zoomx, zoomy);

        };
        return ZoomAndPan;
    })();

    ej.Diagram.ZoomAndPan = ZoomAndPan;
    //#endregion 

    //#region PageSettings
    var PageSettings = function (props) {
        this.pageWidth = undefined;
        this.pageHeight = undefined;
        this.multiplePage = false;
        this.pageBorderWidth = 0;
        this.pageBackgroundColor = "#ffffff";
        this.pageBorderColor = "#565656";
        this.pageMargin = 24;
        this.showPageBreaks = false;
        this.pageOrientation = ej.Diagram.Orientation.Portrait;
        ej.Diagram.merge(this, props);
    };

    ej.Diagram.PageSettings = function (props) {
        return new PageSettings(props);
    };
    ej.Diagram.Orientation =
        {
            Landscape: "landscape",
            Portarit: "portarit"
        };

    PageSettings.prototype =
        {
            _setPageSize: function (width, height, diagram) {
                diagram.model.canvas.width = width;
                diagram.model.canvas.height = height;
                var canvas = diagram._canvas;
                canvas.style.width = diagram.model.canvas.width + "px";
                canvas.style.height = diagram.model.canvas.height + "px";
                var svg = diagram._svg.document;
                svg.style.width = diagram.model.canvas.width + "px";
                svg.style.height = diagram.model.canvas.height + "px";
            },

            _createPageBreaks: function (diagram) {
                this._removePageBreaks(diagram);
                var attr = { id: diagram._canvas.id + "pagebreaks", "pointer-events": "none" };
                var g = ej.Diagram.SvgContext._initializePageBreaks(diagram._svg, attr, diagram._pageBackgroundLayer);
                this._updatePageBreaks(g, diagram);
            },

            _updatePageSettings: function () {
                var swap = false;
                if (this.pageOrientation == ej.Diagram.Orientation.Landscape) {
                    if (this.pageHeight > this.pageWidth) {
                        swap = true;
                    }
                }
                else {
                    if (this.pageWidth > this.pageHeight) {
                        swap = true;
                    }
                }
                //if (swap)
                //{
                //    double temp = PageWidth;
                //    PageWidth = PageHeight;
                //    PageHeight = temp;
                //}
                //if (this.pageWidth > this.pageHeight) {
                //    this.pageOrientation = ej.Diagram.Orientation.Landscape;
                //}
                //var swap = false;
                //if (this.pageHeight > this.pageWidth) {
                //    this.pageOrientation = ej.Diagram.Orientation.Portarit;
                //}

                //if (this.pageOrientation == ej.Diagram.Orientation.Landscape) {
                //    if (this.pageHeight > this.pageWidth) {
                //        swap = true;
                //    }
                //}
                //else {
                //    if (this.pageWidth > this.pageHeight) {
                //        swap = true;
                //    }
                //}

                if (swap) {
                    var temp = this.pageWidth;
                    this.pageWidth = this.pageHeight;
                    this.pageHeight = temp;
                }
            },

            _updatePageBreaks: function (g, diagram) {

                var bounds = diagram._getDigramBounds();
                var _left = bounds.x;
                var _right = bounds.width;
                var _top = bounds.y;
                var _bottom = bounds.height;
                var zoom = 1;
                if (diagram._zoomPan) {
                    zoom = diagram._zoomPan.currentZoom;
                }

                var columncount = this._setColumnCount(_right - _left, this.pageWidth);
                var rowcount = this._setRowCount(_bottom - _top, this.pageHeight);

                var viewLeft = diagram._canvas.clientLeft;
                var viewTop = diagram._canvas.clientTop;
                var left = -viewLeft + (this.pageMargin + _left) * zoom;
                var top = -viewTop + (this.pageMargin + _top) * zoom;

                var width = (this.pageWidth * columncount - this.pageMargin * 2) * zoom;
                var height = (this.pageHeight * rowcount - this.pageMargin * 2) * zoom;

                ej.Diagram.SvgContext._renderVPageBreakLine(new ej.Diagram.Point(left, top), new ej.Diagram.Point(left + width, top), diagram._svg, g);
                ej.Diagram.SvgContext._renderHPageBreakLine(new ej.Diagram.Point(left, top), new ej.Diagram.Point(left, top + height), diagram._svg, g);

                for (var column = 1; column < columncount; column++) {
                    ej.Diagram.SvgContext._renderVPageBreakLine(new ej.Diagram.Point(left + (this.pageWidth * column - this.pageMargin) * zoom, top),
                        new ej.Diagram.Point(left + (this.pageWidth * column - this.pageMargin) * zoom, top + height), diagram._svg, g);
                }
                for (var row = 1; row < rowcount; row++) {
                    ej.Diagram.SvgContext._renderHPageBreakLine(new ej.Diagram.Point(left, top + (this.pageHeight * row - this.pageMargin) * zoom),
                        new ej.Diagram.Point(left + width, top + (this.pageHeight * row - this.pageMargin) * zoom), diagram._svg, g);
                }
                ej.Diagram.SvgContext._renderVPageBreakLine(new ej.Diagram.Point(left + width, top), new ej.Diagram.Point(left + width, top + height), diagram._svg, g);
                ej.Diagram.SvgContext._renderHPageBreakLine(new ej.Diagram.Point(left, top + height), new ej.Diagram.Point(left + width, top + height), diagram._svg, g);
            },

            _setColumnCount: function (width, compare) {
                return parseInt(Math.ceil(width / compare));
            },

            _setRowCount: function (height, compare) {
                return parseInt(Math.ceil(height / compare));
            },

            _updatePageSize: function (diagram, check) {
                var bounds = diagram._getDigramBounds();
                var zoom = 0;
                if (diagram._zoomPan) {
                    zoom = diagram._zoomPan.currentZoom;
                }
                else {
                    zoom = 1;
                }
                var left = bounds.x * zoom;
                var top = bounds.y * zoom;
                var width = (bounds.width - bounds.x) * zoom;
                var height = (bounds.height - bounds.y) * zoom;
                if (!(ej.Diagram.SvgContext._removePageSetting(diagram._svg, diagram._pageBackgroundLayer))) {
                    ej.Diagram.SvgContext._renderPageSettingsRect(new ej.Diagram.Point(left, top), new ej.Diagram.Point(width, height), diagram._svg, diagram._pageBackgroundLayer, this.pageBorderColor, this.pageBackgroundColor, this.pageBorderWidth);
                }
                else {
                    ej.Diagram.SvgContext._updatePageBakground(new ej.Diagram.Point(left, top), new ej.Diagram.Point(width, height));
                }
                if (diagram._scroller._render) {
                    this._updatePageSettings();
                    var width = bounds.width * zoom;
                    var height = bounds.height * zoom;
                    diagram._svg.document.style.left = 0;
                    diagram._svg.document.style.top = 0;
                    var ViewWidth = diagram._scroller._getViewPort(diagram, check).x;
                    var ViewHeight = diagram._scroller._getViewPort(diagram, check).y;
                    var w = ViewWidth;
                    var h = ViewHeight;
                    var _width;
                    var _height;
                    if (width >= ViewWidth) {
                        _width = width;
                    }
                    else if (width < ViewWidth) {
                        _width = ViewWidth;
                    }

                    if (height >= ViewHeight) {
                        _height = height;
                    }
                    else if (height < ViewHeight) {
                        _height = ViewHeight;
                    }
                    if (this.pageWidth && this.pageHeight) {
                        if (this.showPageBreaks) {
                            this._createPageBreaks(diagram);
                        }
                    }
                    diagram._scroller._updateExtentedSize(_width, _height);
                }
            },
            _removePageBreaks: function (diagram) {
                var g = diagram._svg.getElementById(diagram._canvas.id + "pagebreaks");
                if (g != null) { ej.Diagram.SvgContext._removePageBreaks(diagram._svg, g, diagram._pageBackgroundLayer); }
            },
        };
    //#endregion

    //#region Scroller
    ej.Diagram.Scroller = function (diagram) {
        this.diagram = diagram;
        this._extendedWidth = 0;
        this._extendedHeight = 0;
        this._viewPortWidth = 0;
        this._viewportHeight = 0;
        this._horizontalOffset = 0;
        this._verticalOffset = 0;
        this._buttonSize = 0;
        this._render = false;
        this._currentScroller = null;
        this._scrollPixel = 30;
        this.scrollX = 0;
        this.scrollY = 0;
    };

    ej.Diagram.Scroller.prototype =
        {
            _renderScroller: function (value) {
                this._render = true;
                var _current = this.diagram;
                var $content = _current.element;
                var div = document.getElementById(_current._canvas.id + "_scroller");
                var div1 = document.getElementById(_current._canvas.id + "_contentHolder_content");
                this._setViewPortWidth($content);
                this._setViewPortHeight($content);
                $(div).ejScroller({ height: _current._scroller._viewportHeight, width: _current._scroller._viewPortWidth });
                this._currentScroller = $(div).ejScroller("instance");
                _current._scroller._buttonSize = this._currentScroller.model.buttonSize;
                $(div).ejScroller({
                    scroll: function (e) {
                        if (e.source != null) {
                            var isScroll = false;
                            if (e.source == "thumb" || e.source == "button" || e.source == "key") {
                                if (e.scrollData != null) {
                                    if (e.scrollData.clientXy == "pageX") {
                                        if ((e.model.scrollLeft + e.model.width) <= _current._scroller._extendedWidth)
                                            _current._scroller._updateScroll(e.model.scrollLeft, e.model.scrollTop, isScroll, e.scrollData.handle, e.scrollData.handleSpace);
                                    }
                                    else if (e.scrollData.clientXy == "pageY") {

                                        if ((e.model.scrollTop + e.model.height) <= _current._scroller._extendedHeight)
                                            _current._scroller._updateScroll(e.model.scrollLeft, e.model.scrollTop, isScroll, e.scrollData.handle, e.scrollData.handleSpace);
                                    }
                                }
                            }
                        }
                    }
                });
                div1.style.width = $(_current._svg.document).width() + "px";
                div1.style.height = $(_current._svg.document).height() + "px";
                this._invokeRefresh();
            },

            _updateTransformation: function (hOffset, vOffset, isScroll) {

                var div = document.getElementById(this.diagram._canvas.id + "_scroller");
                if (!isScroll) {
                    $(div).ejScroller("scrollX", hOffset, true);
                    $(div).ejScroller("scrollY", vOffset, true);
                }
                ej.Diagram.SvgContext.updateTransformation(this.diagram, -hOffset, -vOffset);
                ej.Diagram.SvgContext.updateScaling(this.diagram, this.diagram._zoomPan.currentZoom);
                this.diagram.model.pageSettings._updatePageSize(this.diagram, true);
                this.diagram._updateGrid(0, 0, this.diagram._zoomPan.currentZoom);
            },

            _updateScrollTransformation: function (hOffset, vOffset) {
                var x = hOffset;
                var y = vOffset;
                ej.Diagram.SvgContext.updateTransformation(this.diagram, -x, -y);
            },

            _updateNegativeLayer: function (x, y) {
                var bounds = this.diagram._getDigramBounds();
                var negativeX = bounds.x < 0 ? bounds.x : 0;
                var negativeY = bounds.y < 0 ? bounds.y : 0;
                if (negativeX < 0) {
                    negativeX = -(negativeX + (x * 2));
                    this.scrollOffsetX = negativeX;
                }
                if (negativeY < 0) {
                    negativeY = -(negativeY + (y * 2));
                }
                ej.Diagram.SvgContext.updateNegativeLayer(this.diagram, negativeX, negativeY);
            },

            _updateScrollNegativeLayer: function (x, y) {
                var bounds = this.diagram._getDigramBounds();
                var negativeX = bounds.x < 0 ? bounds.x : 0;
                var negativeY = bounds.y < 0 ? bounds.y : 0;
                if (negativeX < 0) {
                    negativeX = (negativeX * -1 + (this.scrollX * 2)) + x;
                    this.scrollOffsetX = negativeX;
                }
                if (negativeY < 0) {
                    negativeY = (negativeY * -1 + (this.scrollY * 2)) + y;
                }
                ej.Diagram.SvgContext.updateNegativeLayer(this.diagram, negativeX, negativeY);
            },

            _updateOffset: function (hOffset, vOffset, isScroll) {
                this._horizontalOffset = hOffset;
                this._verticalOffset = vOffset;
                this._updateTransformation(this._horizontalOffset, this._verticalOffset, isScroll);
                //this._updateNegativeLayer(-hOffset, -vOffset);
            },

            _updateScroll: function (hOffset, vOffset, isScroll, handle, space) {
                this._horizontalOffset = hOffset;
                this._verticalOffset = vOffset;
                this._updateScrollTransformation(hOffset, vOffset);

                //if(this._horizontalOffset==0)
                //{
                var check = false;
                if (!this.diagram.model.pageSettings.pageWidth) {
                    if (!this.diagram.model.pageSettings.pageHeight) {
                        check = true;
                    }
                }
                this.diagram.model.pageSettings._updatePageSize(this.diagram, check);
                //}
                // var x, y;
                // x = ((this.scrollX + hOffset) * 2);
                // y = ((this.scrollY + vOffset) * 2);
                //this._updateScrollNegativeLayer(x, y);
            },

            _updateExtentedSize: function (width, height) {
                this._extendedWidth = width;
                this._extendedHeight = height;
                var div1 = this._getDivfromId(this.diagram._canvas.id + "_contentHolder_content");
                if (div1) {
                    div1.style.height = height + "px";
                    div1.style.width = width + "px";
                }
                ej.Diagram.SvgContext.updateSize(this.diagram._svg, width, height);
                this._invokeRefresh();
            },

            _updateViewPort: function () {
                var hoffset = 0;

                var $content = this.diagram.element;
                var div = document.getElementById(this.diagram._canvas.id + "_scroller");
                var div1 = document.getElementById("content");
                this._setViewPortWidth($content);
                this._setViewPortHeight($content);
                this.diagram._svgWidth = this._viewPortWidth;
                this.diagram._svgHeight = this._viewportHeight;
                if (this._currentScroller) {
                    this._currentScroller.model.width = this._viewPortWidth;
                    this._currentScroller.model.height = this._viewportHeight;
                    this._invokeRefresh();
                }
            },

            _invokeRefresh: function () {
                var div = this._getDivfromId(this.diagram._canvas.id + "_scroller");
                $(div).ejScroller("refresh");
            },

            _setViewPortWidth: function (diagram) {
                var $content = diagram;
                var check = $content[0].getBoundingClientRect();
                var no = (window.screenX < 0) ? window.screenX * -1 : window.screenX;
                if ($($content).width() == 0) {
                    this._viewPortWidth = Math.floor(((window.innerWidth - no) - Math.floor(check.left)));
                }
                else {
                    this._viewPortWidth = $($content).width();
                }
            },

            _setViewPortHeight: function (diagram) {
                var $content = diagram;
                var check = $content[0].getBoundingClientRect();
                var no = (window.screenY < 0) ? window.screenY * -1 : window.screenY;
                if ($($content).height() == 0) {
                    this._viewportHeight = Math.floor(((window.innerHeight - no) - Math.floor(check.top)));
                }
                else {
                    this._viewportHeight = $($content).height();
                }
            },

            _getViewPort: function (digram, check) {
                if (check) {
                    return new ej.Diagram.Point(this._viewPortWidth + this._horizontalOffset, this._viewportHeight + this._verticalOffset);
                }
                else {
                    return new ej.Diagram.Point(this._viewPortWidth - this._horizontalOffset, this._viewportHeight - this._verticalOffset);
                }
            },
            _getDivfromId: function (value) {
                var div1 = document.getElementById(value);
                return div1;
            },
        };
    //#endregion

    //#region Snapping
    ej.Diagram.Snapping = function (diagram) {
        this.diagram = diagram;
        this._adjustableDistance = 5;
    };

    ej.Diagram.Snapping.prototype = {

        _snapAngle: function (angle) {
            return this.round(angle, this.diagram.model.snapSettings.snapAngle);
        },

        //#region snapping while dragging

        _snapPosition: function (selectedObject, towardsLeft, towardsTop, del, endPoint, startPoint) {
            //this._adjustableDistance = 5;
            var offset = new ej.Diagram.Point();
            var bounds = selectedObject._bounds;
            var hsnap = 0, vsnap = 0;
            var horizontallysnapped = { snapped: false, offset: 0 };
            var verticallysnapped = { snapped: false, offset: 0 };
            var snapSettings = this.diagram.model.snapSettings;
            if (snapSettings.snapToObject)
                this._snapToObject_position(selectedObject, horizontallysnapped,
                    verticallysnapped, del, startPoint == endPoint);
            //original position
            var left = bounds.x + del.x;
            var top = bounds.y + del.y;
            var right = bounds.x + bounds.width + del.x;
            var bottom = bounds.y + bounds.height + del.y;

            //snapped positions
            var roundedright = this._round(right, snapSettings.horizontalGridlines._snapinterval);
            var roundedleft = this._round(left, snapSettings.horizontalGridlines._snapinterval);
            var roundedtop = this._round(top, snapSettings.verticalGridlines._snapinterval);
            var roundedbottom = this._round(bottom, snapSettings.verticalGridlines._snapinterval);

            //currentposition

            var currentright = bounds.x + bounds.width;

            var currentbottom = bounds.y + bounds.height;
            if (!horizontallysnapped.snapped) {
                if (snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToHorizontalLines) {
                    if (Math.abs(del.x) >= 1) {
                        if (towardsLeft) {
                            if (Math.abs(roundedright - currentright) > Math.abs(roundedleft - bounds.x)) {
                                offset.x += roundedleft - bounds.x;
                            }
                            else
                                offset.x += roundedright - currentright;
                        }
                        else {
                            if (Math.abs(roundedright - currentright) < Math.abs(roundedleft - bounds.x)) {
                                offset.x += roundedright - currentright;
                            }
                            else
                                offset.x += roundedleft - bounds.x;
                        }
                    }
                } else {
                    offset.x = endPoint.x - startPoint.x;
                }
            }
            else
                if (snapSettings.snapToObject)
                    offset.x = horizontallysnapped.offset;
                else
                    offset.x = endPoint.x - startPoint.x;


            if (!verticallysnapped.snapped) {
                if (snapSettings.snapConstraints & ej.Diagram.SnapConstraints.SnapToVerticalLines) {
                    if (Math.abs(del.y) >= 1) {
                        if (towardsTop) {
                            if (Math.abs(roundedbottom - currentbottom) > Math.abs(roundedtop - bounds.y)) {
                                offset.y += roundedtop - bounds.y;
                            }
                            else
                                offset.y += roundedbottom - currentbottom;
                        }
                        else {
                            if (Math.abs(roundedbottom - currentbottom) < Math.abs(roundedtop - bounds.y)) {
                                offset.y += roundedbottom - currentbottom;
                            }
                            else
                                offset.y += roundedtop - bounds.y;
                        }
                    }
                }
                else
                    offset.y = endPoint.y - startPoint.y;
            }
            else
                offset.y = verticallysnapped.offset;

            return offset;
        },

        round: function (value, cutoff) {
            var width = value % cutoff;
            if (width >= (cutoff / 2)) {
                return value + cutoff - width;
            }
            else {
                return value - width;
            }
            return value;
        },

        _round: function (value, snapintervals, cutoff) {
            var scale = this.diagram._zoomPan.currentZoom;
            if (!cutoff) {
                cutoff = 0;
                for (var i = 0; i < snapintervals.length; i++) {
                    cutoff += snapintervals[i];
                }
            }
            cutoff /= scale;
            var quotient = Math.floor(Math.abs(value) / cutoff);
            var bal = value % cutoff;
            var prev = quotient * cutoff;

            if (value >= 0) {
                for (var i = 0; i < snapintervals.length ; i++) {
                    if (bal <= snapintervals[i] / scale) {
                        return prev + (bal < (snapintervals[i] / (2 * scale)) ? 0 : snapintervals[i] / scale);
                    }
                    else {
                        prev += snapintervals[i] / scale;
                        bal -= snapintervals[i] / scale;
                    }
                }
            }
            else {
                prev = prev * -1;
                for (var i = snapintervals.length - 1; i >= 0; i--) {
                    if (Math.abs(bal) <= snapintervals[i] / scale) {
                        return prev - (Math.abs(bal) < (snapintervals[i] / (2 * scale)) ? 0 : snapintervals[i] / scale);
                    }
                    else {
                        prev -= snapintervals[i] / scale;
                        bal += snapintervals[i] / scale;
                    }
                }
            }
            return value;
        },

        _snapToObject_position: function (selectedObject, hsnap, vsnap, del, ended) {
            var length_x = null;
            var length_y = null;
            var horizontaltarget;
            var verticaltarget;
            var objectsAtLeft = [];
            var objectsAtRight = [];
            var objectsAtTop = [];
            var objectsAtBottom = [];
            var bounds = selectedObject._bounds;
            var scale = this.diagram._zoomPan.currentZoom;
            var hoffset = this.diagram._scroller._horizontalOffset;
            var voffset = this.diagram._scroller._verticalOffset;

            var hintersectedrect = new ej.Diagram.Rectangle(hoffset / scale, (selectedObject._bounds.y - 5), this.diagram._scroller._viewPortWidth / scale,
                (selectedObject._bounds.height + 10));
            var vintersectedrect = new ej.Diagram.Rectangle((selectedObject._bounds.x - 5), voffset / scale,
                (selectedObject._bounds.width + 10), this.diagram._scroller._viewportHeight / scale);
            var viewport = new ej.Diagram.Rectangle(hoffset / scale, voffset / scale, this.diagram._scroller._viewPortWidth / scale,
                this.diagram._scroller._viewportHeight / scale);
            var nodes = this._findNodes(selectedObject, vintersectedrect, viewport);
            for (var i = 0; i < nodes.length; i++) {
                var target = nodes[i];
                if (target.type === "node" && this._canConsider(selectedObject, target)) {
                    var targetbounds = target._bounds;
                    if (targetbounds.y + targetbounds.height < bounds.y + del.y) {
                        objectsAtTop[objectsAtTop.length] = { object: target, distance: Math.abs(bounds.y + del.y - targetbounds.y - targetbounds.height) };
                    }
                    else if (targetbounds.y > bounds.y + del.y + bounds.height) {
                        objectsAtBottom[objectsAtBottom.length] = { object: target, distance: Math.abs(bounds.y + del.y + bounds.height - targetbounds.y) };
                    }

                    if (length_x == null || length_x > Math.abs(targetbounds.y - bounds.y - del.y)) {
                        if (Math.abs(targetbounds.x + targetbounds.width / 2 - (bounds.x + bounds.width / 2 + del.x)) <= this._adjustableDistance) {
                            horizontaltarget = this._createSnapObject(targetbounds, bounds, "CenterX");
                            length_x = Math.abs(targetbounds.y - bounds.y);
                        }
                        else if (Math.abs(targetbounds.x - (bounds.x + del.x)) <= this._adjustableDistance) {
                            horizontaltarget = this._createSnapObject(targetbounds, bounds, "Left");
                            length_x = Math.abs(targetbounds.y - bounds.y);
                        }
                        else if (Math.abs(targetbounds.x + targetbounds.width - (bounds.x + bounds.width + del.x)) <= this._adjustableDistance) {
                            horizontaltarget = this._createSnapObject(targetbounds, bounds, "Right");
                            length_x = Math.abs(targetbounds.y - bounds.y);
                        }
                        else if (Math.abs(targetbounds.x + targetbounds.width - (bounds.x + del.x)) <= this._adjustableDistance) {
                            horizontaltarget = this._createSnapObject(targetbounds, bounds, "LeftRight");
                            length_x = Math.abs(targetbounds.y - bounds.y);
                        }
                        else if (Math.abs(targetbounds.x - (bounds.x + bounds.width + del.x)) <= this._adjustableDistance) {
                            horizontaltarget = this._createSnapObject(targetbounds, bounds, "RightLeft");
                            length_x = Math.abs(targetbounds.y - bounds.y);
                        }
                    }
                }
            }
            nodes = this._findNodes(selectedObject, hintersectedrect, viewport);
            for (var i = 0; i < nodes.length; i++) {
                var target = nodes[i];
                if (target.type == "node" && this._canConsider(selectedObject, target)) {

                    var targetbounds = target._bounds;
                    if (targetbounds.x + targetbounds.width < bounds.x + del.x) {
                        objectsAtLeft[objectsAtLeft.length] = { object: target, distance: Math.abs((bounds.x + del.x) - targetbounds.x - targetbounds.width) };
                    }
                    if (targetbounds.x > bounds.x + del.x + bounds.width) {
                        objectsAtRight[objectsAtRight.length] = { object: target, distance: Math.abs(bounds.x + del.x + bounds.width - targetbounds.x) };
                    }
                    if (length_y == null || length_y > Math.abs(targetbounds.x - bounds.x - del.x)) {
                        if (Math.abs(targetbounds.y + targetbounds.height / 2 - (bounds.y + bounds.height / 2 + del.y)) <= this._adjustableDistance) {
                            verticaltarget = this._createSnapObject(targetbounds, bounds, "CenterY");
                            length_y = Math.abs(targetbounds.x - bounds.x);
                        }
                        else if (Math.abs(targetbounds.y - bounds.y - del.y) <= this._adjustableDistance) {
                            verticaltarget = this._createSnapObject(targetbounds, bounds, "Top");
                            length_y = Math.abs(targetbounds.x - bounds.x);
                        }
                        else if (Math.abs(targetbounds.y + targetbounds.height - (bounds.y + bounds.height + del.y)) <= this._adjustableDistance) {
                            verticaltarget = this._createSnapObject(targetbounds, bounds, "Bottom");
                            length_y = Math.abs(targetbounds.x - bounds.x);
                        }
                        else if (Math.abs(targetbounds.y + targetbounds.height - bounds.y - del.y) <= this._adjustableDistance) {
                            verticaltarget = this._createSnapObject(targetbounds, bounds, "TopBottom");
                            length_y = Math.abs(targetbounds.x - bounds.x);

                        }
                        else if (Math.abs(targetbounds.y - (bounds.y + bounds.height + del.y)) <= this._adjustableDistance) {
                            verticaltarget = this._createSnapObject(targetbounds, bounds, "BottomTop");
                            length_y = Math.abs(targetbounds.x - bounds.x);
                        }
                    }
                }
            }


            var g = this._createGuidelines(horizontaltarget, verticaltarget, hsnap, vsnap, ended);
            if (!hsnap.snapped) {
                this._createHorizontalSpacingLines(selectedObject, objectsAtLeft, objectsAtRight, hsnap, vsnap, ended, g, del);
            }
            if (!vsnap.snapped) {
                this._createVerticalSpacingLines(selectedObject, objectsAtTop, objectsAtBottom, hsnap, vsnap, ended, g, del);
            }
        },

        _canConsider: function (selectedObject, target) {
            if (selectedObject.type === "node") {
                return !(selectedObject == target);
            }
            else if (selectedObject.type === "group" || selectedObject.type === "pseudoGroup") {
                return !this._contains(selectedObject, target);
            }
        },

        _contains: function (group, child) {
            for (var i = 0; i < group.children.length; i++) {
                var element = group.children[i];
                if (element.type === "node") {
                    if (element == child) {
                        return true;
                    }
                }
                else if (element.type === "group" || element.type === "pseudoGroup") {
                    if (this._contains(element, child)) {
                        return true;
                    }
                }
            }
            return false;
        },
        _createHorizontalSpacingLines: function (shape, objectsAtLeft, objectsAtRight, hsnap, vsnap, ended, g, del) {
            var top = null;
            objectsAtLeft.sort(function (a, b) { return a.distance - b.distance });
            objectsAtRight.sort(function (a, b) { return a.distance - b.distance });
            var equallySpaced = [];
            var bounds = shape._bounds;
            var nearestleft, nearestright;

            if (objectsAtLeft.length > 0) {
                equallySpaced[equallySpaced.length] = objectsAtLeft[0];
                nearestleft = objectsAtLeft[0].object._bounds;
                top = nearestleft.y;
                if (objectsAtLeft.length > 1) {

                    var targetbounds = objectsAtLeft[1].object._bounds;
                    var equaldistance = nearestleft.x - targetbounds.x - targetbounds.width;
                    if (Math.abs(equaldistance - objectsAtLeft[0].distance) <= this._adjustableDistance) {
                        top = this._findequallyspacedobjectsatleft(objectsAtLeft, equaldistance, top, equallySpaced);

                    } else equaldistance = objectsAtLeft[0].distance;
                } else equaldistance = objectsAtLeft[0].distance;
            }

            equallySpaced.sort(function (a, b) { return b.distance - a.distance });
            equallySpaced[equallySpaced.length] = { object: shape, distance: 0 };
            top = bounds.y < top || !top ? bounds.y : top;

            if (objectsAtRight.length > 0) {
                nearestright = objectsAtRight[0].object._bounds;
                top = nearestright.y < top ? nearestright.y : top;
                if (objectsAtRight.length > 1) {
                    var targetbounds = objectsAtRight[1].object._bounds;
                    var dist = targetbounds.x - nearestright.x - nearestright.width;
                }

                if (objectsAtLeft.length > 0) {
                    if (Math.abs(objectsAtRight[0].distance - objectsAtLeft[0].distance) <= this._adjustableDistance) {
                        var adjustablevalue = Math.abs(objectsAtRight[0].distance - objectsAtLeft[0].distance) / 2;
                        (objectsAtRight[0].distance < objectsAtLeft[0].distance) ?
                        equaldistance -= adjustablevalue : equaldistance += adjustablevalue;
                        equallySpaced[equallySpaced.length] = objectsAtRight[0];
                    }
                    else if (objectsAtLeft.length == 1) {
                        nearestleft = undefined;
                        equallySpaced.splice(0, 1);
                        equallySpaced[equallySpaced.length] = objectsAtRight[0];
                        equaldistance = dist;
                    }
                }
                else {
                    equaldistance = dist;
                    equallySpaced[equallySpaced.length] = objectsAtRight[0];
                }
                if (objectsAtRight.length > 1 && nearestright.x + nearestright.width < targetbounds.x) {
                    top = this._findequallyspacednodesatright(objectsAtRight, dist, top, equallySpaced);
                }

            }
            if (equallySpaced.length > 2) {
                this._addHorisontalSpacingLine(equallySpaced, ended, g, top);
                var delta = 0;
                if (ended)
                    delta = del.x;
                if (nearestleft)
                    hsnap.offset = equaldistance - Math.abs(bounds.x + delta - nearestleft.x - nearestleft.width) + delta;
                else if (nearestright)
                    hsnap.offset = Math.abs(bounds.x + bounds.width + delta - nearestright.x) - equaldistance + delta;
                hsnap.snapped = true;
            }
        },

        _addHorisontalSpacingLine: function (equallySpaced, ended, g, top) {
            var scale = this.diagram._zoomPan.currentZoom;
            var hoffset = this.diagram._scroller._horizontalOffset;
            var voffset = this.diagram._scroller._verticalOffset;
            if (equallySpaced.length > 2) {
                if (!ended) {
                    for (var i = 0; i < equallySpaced.length - 1; i++) {
                        var crnt = equallySpaced[i].object._bounds;
                        var next = equallySpaced[i + 1].object._bounds;
                        ej.Diagram.SvgContext._renderSpacingLines(new ej.Diagram.Point(crnt.x + crnt.width, top - 15),
                       new ej.Diagram.Point(next.x, top - 15), g, this.diagram._svg, scale, hoffset, voffset);
                    }
                }

            }
        },

        _findequallyspacedobjectsatleft: function (objectsAtLeft, equaldistance, top, equallySpaced) {
            for (var i = 1; i < objectsAtLeft.length; i++) {
                var prevbounds = objectsAtLeft[i - 1].object._bounds;
                var targetbounds = objectsAtLeft[i].object._bounds;
                var dist = prevbounds.x - targetbounds.x - targetbounds.width;
                if (Math.abs(dist - equaldistance) <= 1) {
                    equallySpaced[equallySpaced.length] = objectsAtLeft[i];
                    if (targetbounds.y < top)
                        top = targetbounds.y;
                }
                else
                    break;
            }
            return top;
        },

        _findequallyspacednodesatright: function (objectsAtRight, equaldistance, top, equallySpaced) {
            var actualdistance = objectsAtRight[0].distance;
            if (Math.abs(equaldistance - actualdistance) <= this._adjustableDistance) {
                for (var i = 0; i < objectsAtRight.length - 1; i++) {
                    var target = objectsAtRight[i].object;
                    var targetbounds = objectsAtRight[i + 1].object._bounds;
                    var prevbounds = target._bounds;
                    var dist = targetbounds.x - prevbounds.x - prevbounds.width;
                    if (Math.abs(dist - equaldistance) <= 1) {
                        equallySpaced[equallySpaced.length] = objectsAtRight[i + 1];
                        if (prevbounds.y < top) {
                            top = prevbounds.y
                        }
                    }
                    else
                        break;
                }
            }
            return top;
        },

        _createVerticalSpacingLines: function (shape, objectsAtTop, objectsAtBottom, hsnap, vsnap, ended, g, del) {
            var right = null;
            objectsAtTop.sort(function (a, b) { return a.distance - b.distance });
            objectsAtBottom.sort(function (a, b) { return a.distance - b.distance });
            var equallySpaced = [];
            var bounds = shape._bounds;
            var nearesttop, nearestbottom;

            if (objectsAtTop.length > 0) {
                equallySpaced[equallySpaced.length] = objectsAtTop[0];
                nearesttop = objectsAtTop[0].object._bounds;
                right = nearesttop.x + nearesttop.width;
                if (objectsAtTop.length > 1) {
                    var targetbounds = objectsAtTop[1].object._bounds;
                    var equaldistance = nearesttop.y - targetbounds.y - targetbounds.height;
                    if (Math.abs(equaldistance - objectsAtTop[0].distance) <= this._adjustableDistance) {
                        right = this._findequallyspacedobjectsattop(objectsAtTop, equaldistance, right, equallySpaced);
                    } else equaldistance = objectsAtTop[0].distance;
                } else equaldistance = objectsAtTop[0].distance;
            }

            equallySpaced.sort(function (a, b) { return b.distance - a.distance });
            equallySpaced[equallySpaced.length] = { object: shape, distance: 0 };
            right = bounds.x + bounds.width > right || !right ? bounds.x + bounds.width : right;

            if (objectsAtBottom.length > 0) {
                nearestbottom = objectsAtBottom[0].object._bounds;
                right = nearestbottom.x + nearestbottom.width > right ? nearestbottom.x + nearestbottom.width : right;
                if (objectsAtBottom.length > 1) {
                    var targetbounds = objectsAtBottom[1].object._bounds;
                    var dist = targetbounds.y - nearestbottom.y - nearestbottom.height;
                }

                if (objectsAtTop.length > 0) {
                    if (Math.abs(objectsAtBottom[0].distance - objectsAtTop[0].distance) <= this._adjustableDistance) {
                        var adjustablevalue = Math.abs(objectsAtBottom[0].distance - objectsAtTop[0].distance) / 2;
                        (objectsAtBottom[0].distance < objectsAtTop[0].distance) ?
                        equaldistance -= adjustablevalue : equaldistance += adjustablevalue;
                        equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                    }
                    else if (objectsAtTop.length == 1) {
                        nearesttop = undefined;
                        equallySpaced.splice(0, 1);
                        equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                        equaldistance = dist;
                    }
                }
                else {
                    equaldistance = dist;
                    equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                }
                if (objectsAtBottom.length > 1 && targetbounds.y > nearestbottom.y + nearestbottom.height) {
                    right = this._findequallyspacednodesatbottom(objectsAtBottom, dist, right, equallySpaced);
                }

            }
            if (equallySpaced.length > 2) {
                this._addVerticalSpacingLine(equallySpaced, ended, g, right);
                var delta = 0;
                if (ended)
                    delta = del.y;
                if (nearesttop)
                    vsnap.offset = equaldistance - Math.abs(bounds.y + delta - nearesttop.y - nearesttop.height) + delta;
                else if (nearestbottom)
                    vsnap.offset = Math.abs(bounds.y + bounds.height + delta - nearestbottom.y) - equaldistance + delta;
                vsnap.snapped = true;
            }
        },

        _addVerticalSpacingLine: function (equallySpaced, ended, g, right) {
            var scale = this.diagram._zoomPan.currentZoom;
            var hoffset = this.diagram._scroller._horizontalOffset;
            var voffset = this.diagram._scroller._verticalOffset;
            if (equallySpaced.length > 2) {
                if (!ended) {
                    for (var i = 0; i < equallySpaced.length - 1; i++) {
                        var crnt = equallySpaced[i].object._bounds;
                        var next = equallySpaced[i + 1].object._bounds;
                        ej.Diagram.SvgContext._renderSpacingLines(new ej.Diagram.Point(right + 15, crnt.y + crnt.height),
                      new ej.Diagram.Point(right + 15, next.y), g, this.diagram._svg, scale, hoffset, voffset);
                    }
                }

            }
        },

        _findequallyspacedobjectsattop: function (objectsAtTop, equaldistance, right, equallySpaced) {
            for (var i = 1; i < objectsAtTop.length; i++) {
                var prevbounds = objectsAtTop[i - 1].object._bounds;
                var targetbounds = objectsAtTop[i].object._bounds;
                var dist = prevbounds.y - targetbounds.y - targetbounds.height;
                if (Math.abs(dist - equaldistance) <= 1) {
                    equallySpaced[equallySpaced.length] = objectsAtTop[i];
                    if (targetbounds.x + targetbounds.width > right)
                        right = targetbounds.x + targetbounds.width;
                }
                else
                    break;
            }
            return right;
        },

        _findequallyspacednodesatbottom: function (objectsAtBottom, equaldistance, right, equallySpaced) {
            var actualdistance = objectsAtBottom[0].distance;
            if (Math.abs(equaldistance - actualdistance) <= this._adjustableDistance) {
                for (var i = 0; i < objectsAtBottom.length - 1; i++) {
                    var target = objectsAtBottom[i].object;
                    var targetbounds = objectsAtBottom[i + 1].object._bounds;
                    var prevbounds = target._bounds;
                    var dist = targetbounds.y - prevbounds.y - prevbounds.height;
                    if (Math.abs(dist - equaldistance) <= 1) {
                        equallySpaced[equallySpaced.length] = objectsAtBottom[i + 1];
                        if (prevbounds.x + prevbounds.width > right)
                            right = prevbounds.x + prevbounds.width;
                    }

                    else
                        break;
                }
            }
            return right;
        },

        //#endregion

        //#region Snap to object while resizing

        _snapToObject_size: function (hsnap, vsnap, delx, dely, selectedobject, ended) {
            var length_x = null;
            var length_y = null;
            var horizontaltarget;
            var verticaltarget;
            var bounds = selectedobject._bounds;

            var samewidth = [];
            var sameheight = [];
            var scale = this.diagram._zoomPan.currentZoom;
            var hoffset = this.diagram._scroller._horizontalOffset;
            var voffset = this.diagram._scroller._verticalOffset;
            var hintersectedrect = new ej.Diagram.Rectangle(hoffset / scale, (selectedobject._bounds.y - 5) / scale,
                this.diagram._scroller._viewPortWidth / scale, (selectedobject._bounds.height + 10) / scale);
            var vintersectedrect = new ej.Diagram.Rectangle((selectedobject._bounds.x - 5) / scale, voffset / scale,
                (selectedobject._bounds.width + 10) / scale, this.diagram._scroller._viewportHeight / scale);
            var viewport = new ej.Diagram.Rectangle(hoffset / scale, voffset / scale, this.diagram._scroller._viewPortWidth / scale,
                this.diagram._scroller._viewportHeight / scale);
            var elementsinview = [];
            var nodes = this._findNodesinView(selectedobject, vintersectedrect, elementsinview, viewport);
            for (var i = 0; i < nodes.length; i++) {
                var target = nodes[i];
                if (this._canConsider(selectedobject, target) && target.type === "node") {
                    var targetbounds = target._bounds;
                    if (length_x == null || length_x > Math.abs(targetbounds.y - bounds.y)) {
                        if (hsnap.left) {
                            if (Math.abs(bounds.x + delx - targetbounds.x) <= this._adjustableDistance) {
                                horizontaltarget = this._createSnapObject(targetbounds, bounds, "Left");
                                length_x = Math.abs(targetbounds.y - bounds.y);
                            }
                            else if (Math.abs(bounds.x + delx - targetbounds.x - targetbounds.width) <= this._adjustableDistance) {
                                horizontaltarget = this._createSnapObject(targetbounds, bounds, "LeftRight");
                                length_x = Math.abs(targetbounds.y - bounds.y);
                            }
                        }
                        else if (hsnap.right) {
                            if (Math.abs(bounds.x + delx + bounds.width - targetbounds.x - targetbounds.width) <= this._adjustableDistance) {
                                horizontaltarget = this._createSnapObject(targetbounds, bounds, "Right");
                                length_x = Math.abs(targetbounds.y - bounds.y);
                            }
                            else if (Math.abs(bounds.x + delx + bounds.width - targetbounds.x) <= this._adjustableDistance) {
                                horizontaltarget = this._createSnapObject(targetbounds, bounds, "RightLeft");
                                length_x = Math.abs(targetbounds.y - bounds.y);
                            }
                        }
                    }
                }
            }
            nodes = this._findNodes(selectedobject, hintersectedrect, viewport);
            for (var i = 0; i < nodes.length; i++) {
                var target = nodes[i];
                if (this._canConsider(selectedobject, target) && target.type === "node") {
                    var targetbounds = target._bounds;
                    if (length_y == null || length_y > Math.abs(targetbounds.x - bounds.x)) {
                        if (vsnap.top) {
                            if (Math.abs(bounds.y + dely - targetbounds.y) <= this._adjustableDistance) {
                                verticaltarget = this._createSnapObject(targetbounds, bounds, "Top");
                                length_y = Math.abs(targetbounds.x - bounds.x);
                            }
                            else if (Math.abs(bounds.y + dely - targetbounds.y - targetbounds.height) <= this._adjustableDistance) {
                                verticaltarget = this._createSnapObject(targetbounds, bounds, "TopBottom");
                                length_y = Math.abs(targetbounds.x - bounds.x);
                            }
                        }
                        else if (vsnap.bottom) {
                            if (Math.abs(bounds.y + bounds.height + dely - targetbounds.y - targetbounds.height) <= this._adjustableDistance) {
                                verticaltarget = this._createSnapObject(targetbounds, bounds, "Bottom");
                                length_y = Math.abs(targetbounds.x - bounds.x);
                            }
                            else if (Math.abs(bounds.y + bounds.height + dely - targetbounds.y) <= this._adjustableDistance) {
                                verticaltarget = this._createSnapObject(targetbounds, bounds, "BottomTop");
                                length_y = Math.abs(targetbounds.x - bounds.x);
                            }
                        }
                    }
                }

            }
            for (var i = 0; i < elementsinview.length; i++) {
                var target = elementsinview[i];
                if (this._canConsider(selectedobject, target)) {
                    var targetbounds = target._bounds;
                    var del = hsnap.left ? -delx : delx;
                    var diff = Math.abs(bounds.width + del - targetbounds.width);
                    if (diff <= this._adjustableDistance) {
                        var actualdiff = hsnap.left ? -targetbounds.width + bounds.width : targetbounds.width - bounds.width;
                        samewidth[samewidth.length] = { source: target, diff: diff, offset: actualdiff };
                    }
                    del = vsnap.top ? -dely : dely;
                    var dify = Math.abs(bounds.height + del - targetbounds.height);
                    if (dify <= this._adjustableDistance) {
                        var actualdiff = vsnap.top ? -targetbounds.height + bounds.height : targetbounds.height - bounds.height;
                        sameheight[sameheight.length] = { source: target, diff: dify, offset: actualdiff };
                    }
                }

            }
            var g = this._createGuidelines(horizontaltarget, verticaltarget, hsnap, vsnap, ended, delx, dely);
            if (!hsnap.snapped && samewidth.length > 0 && (hsnap.left || hsnap.right)) {
                this._CreateSameWidthLines(samewidth, hsnap, ended, g, selectedobject, delx, dely, elementsinview);
            }
            if (!vsnap.snapped && sameheight.length > 0 && (vsnap.top || vsnap.bottom)) {
                this._CreateSameHeightLines(sameheight, vsnap, ended, g, selectedobject, delx, dely, elementsinview);
            }
        },


        _CreateSameWidthLines: function (samewidth, hsnap, ended, g, shape, delx, dely) {
            var scale = this.diagram._zoomPan.currentZoom;
            var hoffset = this.diagram._scroller._horizontalOffset;
            var voffset = this.diagram._scroller._verticalOffset;
            samewidth.sort(function (a, b) { return a.offset - b.offset; });
            var bounds = shape._bounds;
            var target = samewidth[0];
            var targetbounds = target.source._bounds;
            var samesized = [];
            samesized[0] = samewidth[0];
            for (var i = 1; i < samewidth.length; i++) {
                var crntbounds = samewidth[i].source._bounds;
                if (crntbounds.width == targetbounds.width)
                    samesized[samesized.length] = samewidth[i];
            }
            if (samesized.length > 0) {

                if (!ended) {
                    ej.Diagram.SvgContext._renderSpacingLines(new ej.Diagram.Point(bounds.x + target.offset, bounds.y - 15),
                        new ej.Diagram.Point(bounds.x + bounds.width + target.offset, bounds.y - 15), g, this.diagram._svg, scale, hoffset, voffset);

                    for (var i = 0; i < samesized.length; i++) {
                        var _bounds = samesized[i].source._bounds;
                        ej.Diagram.SvgContext._renderSpacingLines(new ej.Diagram.Point(_bounds.x, _bounds.y - 15),
                        new ej.Diagram.Point(_bounds.x + _bounds.width, _bounds.y - 15), g, this.diagram._svg, scale, hoffset, voffset);
                    }
                }
            }
            hsnap.offset = target.offset;
            hsnap.snapped = true;

        },

        _CreateSameHeightLines: function (sameheight, vsnap, ended, g, shape, delx, dely) {
            var scale = this.diagram._zoomPan.currentZoom;
            var hoffset = this.diagram._scroller._horizontalOffset;
            var voffset = this.diagram._scroller._verticalOffset;
            sameheight.sort(function (a, b) { return a.offset - b.offset; });
            var bounds = shape._bounds;
            var target = sameheight[0];
            var targetbounds = target.source._bounds;
            var samesized = [];
            samesized[0] = sameheight[0];
            for (var i = 0; i < sameheight.length; i++) {
                var crntbounds = sameheight[i].source._bounds;
                if (crntbounds.height == targetbounds.height)
                    samesized[samesized.length] = sameheight[i];
            }
            if (samesized.length > 0) {

                if (!ended) {
                    ej.Diagram.SvgContext._renderSpacingLines(new ej.Diagram.Point(bounds.x + bounds.width + 15, bounds.y + target.offset),
                        new ej.Diagram.Point(bounds.x + bounds.width + 15, bounds.y + bounds.height + target.offset), g, this.diagram._svg, scale, hoffset, voffset);

                    for (var i = 0; i < samesized.length; i++) {
                        var _bounds = samesized[i].source._bounds;
                        ej.Diagram.SvgContext._renderSpacingLines(new ej.Diagram.Point(_bounds.x + _bounds.width + 15, _bounds.y),
                        new ej.Diagram.Point(_bounds.x + _bounds.width + 15, _bounds.y + _bounds.height), g, this.diagram._svg, scale, hoffset, voffset);

                    }
                }
                vsnap.offset = target.offset;
                vsnap.snapped = true;
            }
        },

        //#endregion

        _createSnapObject: function (targetbounds, bounds, snap) {
            switch (snap) {
                case "Left":
                    return {
                        st: new ej.Diagram.Point(targetbounds.x, Math.min(targetbounds.y, bounds.y)),
                        end: new ej.Diagram.Point(targetbounds.x, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetx: targetbounds.x - bounds.x,
                        type: "SideAlignment"
                    };
                    break;
                case "Right":
                    return {
                        st: new ej.Diagram.Point(targetbounds.x + targetbounds.width, Math.min(targetbounds.y, bounds.y)),
                        end: new ej.Diagram.Point(targetbounds.x + targetbounds.width, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetx: targetbounds.x + targetbounds.width - bounds.x - bounds.width,
                        type: "SideAlignment"
                    };
                    break;
                case "Top":
                    return {
                        st: new ej.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y),
                        end: new ej.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y),
                        offsety: targetbounds.y - bounds.y,
                        type: "SideAlignment"
                    };
                    break;
                case "Bottom":
                    return {
                        st: new ej.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height),
                        end: new ej.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height),
                        offsety: targetbounds.y + targetbounds.height - bounds.y - bounds.height,
                        type: "SideAlignment"
                    };
                    break;
                case "TopBottom":
                    return {
                        st: new ej.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height),
                        end: new ej.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height),
                        offsety: targetbounds.y + targetbounds.height - bounds.y,
                        type: "SideAlignment"
                    };
                    break;
                case "BottomTop":
                    return {
                        st: new ej.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y),
                        end: new ej.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y),
                        offsety: targetbounds.y - bounds.y - bounds.height,
                        type: "SideAlignment"
                    };
                    break;
                case "LeftRight":
                    return {
                        st: new ej.Diagram.Point(targetbounds.x + targetbounds.width, Math.min(targetbounds.y, bounds.y)),
                        end: new ej.Diagram.Point(targetbounds.x + targetbounds.width, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetx: targetbounds.x + targetbounds.width - bounds.x,
                        type: "SideAlignment"
                    };
                    break;
                case "RightLeft":
                    return {
                        st: new ej.Diagram.Point(targetbounds.x, Math.min(targetbounds.y, bounds.y)),
                        end: new ej.Diagram.Point(targetbounds.x, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetx: targetbounds.x - bounds.x - bounds.width,
                        type: "SideAlignment"
                    };
                    break;
                case "CenterX":
                    return {
                        st: new ej.Diagram.Point(targetbounds.x + targetbounds.width / 2, Math.min(targetbounds.y, bounds.y)),
                        end: new ej.Diagram.Point(targetbounds.x + targetbounds.width / 2, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetx: targetbounds.x + targetbounds.width / 2 - (bounds.x + bounds.width / 2),
                        type: "CenterAlignment"
                    };
                    break;
                case "CenterY":
                    return {
                        st: new ej.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height / 2),
                        end: new ej.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height / 2),
                        offsety: targetbounds.y + targetbounds.height / 2 - (bounds.y + bounds.height / 2),
                        type: "CenterAlignment"
                    };
                    break;
            }
        },

        _createGuidelines: function (htarget, vtarget, hsnap, vsnap, ended) {
            var attr = { id: this.diagram._canvas.id + "Guideline", "pointer-events": "none" };
            var g = ej.Diagram.SvgContext._initializeGuidelines(this.diagram._svg, attr, this.diagram._adornerLayer);
            var scale = this.diagram._zoomPan.currentZoom;
            var hoffset = this.diagram._scroller._horizontalOffset;
            var voffset = this.diagram._scroller._verticalOffset;
            if (htarget) {
                hsnap.offset = htarget.offsetx;
                hsnap.snapped = true;
                if (!ended) {
                    if (htarget.type == "SideAlignment") {
                        ej.Diagram.SvgContext._renderSideAlignmentLines(htarget.st, htarget.end, g, this.diagram._svg, scale, hoffset, voffset);
                    }
                    else if (htarget.type == "CenterAlignment") {
                        ej.Diagram.SvgContext._renderCenterAlignmentLines(htarget.st, htarget.end, g, this.diagram._svg, scale, hoffset, voffset);
                    }
                }
            }
            if (vtarget) {
                vsnap.offset = vtarget.offsety;
                vsnap.snapped = true;
                if (!ended) {
                    if (vtarget.type == "SideAlignment") {
                        ej.Diagram.SvgContext._renderSideAlignmentLines(vtarget.st, vtarget.end, g, this.diagram._svg, scale, hoffset, voffset);
                    }
                    else if (vtarget.type == "CenterAlignment") {
                        ej.Diagram.SvgContext._renderCenterAlignmentLines(vtarget.st, vtarget.end, g, this.diagram._svg, scale, hoffset, voffset);
                    }
                }
            }
            return g;
        },

        _removeGuidelines: function () {
            var g = this.diagram._svg.getElementById(this.diagram._canvas.id + "Guideline");
            if (g != null) { ej.Diagram.SvgContext._removeGuidelines(this.diagram._adornerLayer, g); }
        },

        _findNodes: function (node, child, viewport) {
            var nodelist = [];
            var quads = this.diagram._spatialsearch._findQuads(viewport);
            for (var i = 0; i < quads.length; i++) {
                var quad = quads[i];
                if (quad._objects.length > 0) {
                    for (var j = 0; j < quad._objects.length; j++) {
                        var nd = quad._objects[j];
                        if (nd.type === "node") {
                            var nod = nd._bounds;
                            if (nodelist.indexOf(nd) == -1 && child.intersectsWith(nod))
                                nodelist.push(nd);
                        }
                    }
                }
            }
            return nodelist;
        },

        _findNodesinView: function (node, child, nodesinView, viewport) {
            var nodelist = [];
            var quads = this.diagram._spatialsearch._findQuads(viewport);
            for (var i = 0; i < quads.length; i++) {
                var quad = quads[i];
                if (quad._objects.length > 0) {
                    if (quad._objects.length > 0) {
                        for (var j = 0; j < quad._objects.length; j++) {
                            var nd = quad._objects[j];
                            if (nd.type == "node") {
                                var nod = nd._bounds;
                                if (nodelist.indexOf(nd) == -1 && child.intersectsWith(nod))
                                    nodelist.push(nd);
                                if (nodesinView.indexOf(nd) && viewport.intersectsWith(nod)) {
                                    nodesinView.push(nd);
                                }
                            }
                        }
                    }
                }
            }
            return nodelist;
        }
    };
    //#endregion

    //#region SpatialSearch
    ej.Diagram.Quad = function (left, top, width, height, spatialSearch, parent) {
        this._left = left;
        this._top = top;
        this._height = height;
        this._width = width;
        this._parent = parent;
        this._first = null;
        this._second = null;
        this._third = null;
        this._fourth = null;
        this._objects = [];
        this.spatialSearch = spatialSearch;
    };
    ej.Diagram.Quad.prototype = {
        _addIntoAQuad: function (node) {
            var isAdded = false;
            this.spatialSearch._setCurrentNode(node);
            while (!isAdded) {
                isAdded = this.spatialSearch._parentQuad._add();
            }
        },

        _add: function () {
            if (this._isContained()) {
                this._selectQuad();
                return true;
            }
            else {
                var newParent;
                var isempty = !this._objects.length && this._first == null && this._second == null && this._third == null && this._fourth == null;
                if (this.spatialSearch._childLeft < this._left) {
                    if (this.spatialSearch._childTop < this._top) {
                        newParent = new ej.Diagram.Quad(this._left - this._width, this._top - this._height,
                                             this._width * 2, this._height * 2,
                                             this.spatialSearch);
                        if (!isempty)
                            newParent._fourth = this;
                    }
                    else {
                        newParent = new ej.Diagram.Quad(this._left - this._width, this._top,
                                             this._width * 2, this._height * 2,
                                             this.spatialSearch);
                        if (!isempty)
                            newParent._second = this;
                    }
                }
                else if (this.spatialSearch._childTop < this._top) {
                    newParent = new ej.Diagram.Quad(this._left, this._top - this._height,
                                         this._width * 2, this._height * 2,
                                         this.spatialSearch);
                    if (!isempty)
                        newParent._third = this;
                }
                else {
                    newParent = new ej.Diagram.Quad(this._left, this._top,
                                         this._width * 2, this._height * 2,
                                         this.spatialSearch);
                    if (!isempty)
                        newParent._first = this;
                }
                this._parent = newParent;
                this.spatialSearch._parentQuad = newParent;
                return false;
                //newParent._addIntoAQuad(node);
            }
        },

        _isContained: function () {
            if (this.spatialSearch._childLeft >= this._left && this.spatialSearch._childRight <= this._left + this._width &&
                this.spatialSearch._childTop >= this._top && this.spatialSearch._childBottom <= this._top + this._height) {
                return true;
            }
            return false;
        },

        _selectQuad: function () {
            var current = this;
            while (current != null) {
                current = current._getQuad();
            }
        },

        _getQuad: function () {
            var halfWidth = this._width / 2;
            var halfHeight = this._height / 2;
            if (halfWidth >= 100 && halfHeight >= 100) {
                var xCenter = this._left + halfWidth;
                var yCenter = this._top + halfHeight;

                if (this.spatialSearch._childRight <= xCenter) {
                    if (this.spatialSearch._childBottom <= yCenter) {
                        return this._first ? this._first :
                               this._first = new ej.Diagram.Quad(this._left, this._top, halfWidth, halfHeight, this.spatialSearch, this);
                    }
                    if (this.spatialSearch._childTop >= yCenter) {
                        return this._third ? this._third :
                                this._third = new ej.Diagram.Quad(this._left, yCenter, halfWidth, halfHeight, this.spatialSearch, this);
                    }

                }
                else if (this.spatialSearch._childLeft >= xCenter) {
                    if (this.spatialSearch._childBottom <= yCenter) {
                        return this._second ? this._second :
                               this._second = new ej.Diagram.Quad(xCenter, this._top, halfWidth, halfHeight, this.spatialSearch, this);
                    }
                    if (this.spatialSearch._childTop >= yCenter) {
                        return this._fourth ? this._fourth :
                                this._fourth = new ej.Diagram.Quad(xCenter, yCenter, halfWidth, halfHeight, this.spatialSearch, this);
                    }
                }
            }
            this._objects.push(this.spatialSearch._childnode);
            this.spatialSearch._childnode.Quad = this;
            return null;
        },

        _isIntersect: function (t) {
            if (this._left + this._width < t.left() || this._top + this._height < t.top() || this._left > t.right() || this._top > t.bottom()) {
                return false;
            }
            return true;
        },

        _findQuads: function (_mCurrentViewPort) {
            {
                if (this._first != null && this._first._isIntersect(_mCurrentViewPort)) {
                    this._first._findQuads(_mCurrentViewPort);
                }

                if (this._second != null && this._second._isIntersect(_mCurrentViewPort)) {
                    this._second._findQuads(_mCurrentViewPort);
                }

                if (this._third != null && this._third._isIntersect(_mCurrentViewPort)) {
                    this._third._findQuads(_mCurrentViewPort);
                }

                if (this._fourth != null && this._fourth._isIntersect(_mCurrentViewPort)) {
                    this._fourth._findQuads(_mCurrentViewPort);

                }
                if (this._objects.length > 0)
                    this.spatialSearch._quads.push(this);
            }
        }
    };

    ej.Diagram.SpatialSearch = function (diagram) {
        this._pageLeft = null;
        this._pageRight = null;
        this._pageTop = null;
        this._pageBottom = null;
        this._topElement = null;
        this._bottomElement = null;
        this._rightElement = null;
        this._leftElement = null;
        this._parentQuad = new ej.Diagram.Quad(0, 0, 200, 200, this);
        this._childnode = null;
        this._childLeft = null;
        this._childTop = null;
        this._childRight = null;
        this._childBottom = null;
        this._quads = null;
        this.diagram = diagram;
    };
    ej.Diagram.SpatialSearch.prototype =
         {
             _updateQuad: function (node) {
                 this._setCurrentNode(node);
                 var nodBounds = node._bounds;
                 //if (!(nodBounds.X.IsValid() && nodBounds.Y.IsValid() && nodBounds.Width.IsValid() && nodBounds.Height.IsValid()))
                 if (!nodBounds)
                     return;
                 //nodBounds = new Rect(nodBounds.X.Valid(), nodBounds.Y.Valid(), nodBounds.Width.Valid(), nodBounds.Height.Valid());
                 if (node.Quad != null) {
                     if (node.type === "connector") {
                         this._removeFromaQuad(node);
                         this._parentQuad._addIntoAQuad(node);
                     }
                     else {
                         if (!node.Quad._isContained()) {
                             this._removeFromaQuad(node);
                             this._parentQuad._addIntoAQuad(node);
                         }
                     }
                 }
                 else {
                     if (!this._parentQuad)
                         this._parentQuad = new ej.Diagram.Quad(0, 0, 200, 200, this);
                     this._parentQuad._addIntoAQuad(node);
                 }
                 var nod = node.source ? node.source : node;
                 if (this._isWithinPageBounds(nodBounds) &&
                     this._leftElement != nod &&
                     this._topElement != nod &&
                     this._rightElement != nod &&
                     this._bottomElement != nod) {
                 }
                 else {

                     var modified = false;
                     if (this._pageLeft >= this._childLeft || this._pageLeft == null) {
                         this._pageLeft = this._childLeft;
                         this._leftElement = nod;
                         modified = true;
                     }
                     else if (nod == this._leftElement) {
                         this._pageLeft = null;
                         this._findLeft(this._parentQuad);
                         modified = true;
                     }

                     if (this._pageTop >= this._childTop || this._pageTop == null) {
                         this._pageTop = this._childTop;
                         this._topElement = nod;
                         modified = true;
                     }
                     else if (nod == this._topElement) {
                         this._pageTop = null;
                         this._findTop(this._parentQuad);
                         modified = true;
                     }

                     if (this._pageBottom <= this._childBottom || this._pageBottom == null) {
                         modified = true;
                         this._pageBottom = this._childBottom;
                         this._bottomElement = nod;
                     }
                     else if (nod == this._bottomElement) {
                         this._pageBottom = null;
                         this._findBottom(this._parentQuad);
                         modified = true;
                     }

                     if (this._pageRight <= this._childRight || this._pageRight == null) {
                         this._pageRight = this._childRight;
                         this._rightElement = nod;
                         modified = true;
                     }
                     else if (nod == this._rightElement) {
                         this._pageRight = null;
                         this._findRight(this._parentQuad);
                         modified = true;
                     }

                 }
                 if (modified && (!this.diagram.model.layout || !this.diagram.model.layout._processingLayout)) {
                     if (this.diagram._pageBackgroundLayer) {
                         this.diagram.model.pageSettings._updatePageSize(this.diagram, true);
                     }
                 }

             },

             _removeFromaQuad: function (node) {
                 if (node.Quad != null) {
                     var index = node.Quad._objects.indexOf(node);
                     node.Quad._objects.splice(index, 1);
                     this._update(node.Quad);
                     node.Quad = null;
                 }
             },

             _update: function (quad) {
                 if (quad._parent != null && quad._objects.length == 0 && quad._first == null &&
                     quad._second == null && quad._third == null && quad._fourth == null) {
                     var parent = quad._parent;
                     if (parent._first == quad) {
                         parent._first = null;
                     }
                     else if (parent._second == quad) {
                         parent._second = null;
                     }
                     else if (parent._third == quad) {
                         parent._third = null;
                     }
                     else if (parent._fourth == quad) {
                         parent._fourth = null;
                     }
                     this._update(quad._parent);
                 }
                 else
                     return;
             },

             _isWithinPageBounds: function (node) {
                 if (node.left() >= this._pageLeft && node.right() <= this._pageRight &&
                     node.top() >= this._pageTop && node.bottom() <= this._pageBottom)
                     return true;
                 else
                     return false;
             },

             _findQuads: function (_mCurrentViewport) {
                 this._quads = [];
                 this._parentQuad._findQuads(_mCurrentViewport);
                 return this._quads;
             },

             _updateBounds: function (node) {
                 var modified = false;
                 if (node == this._topElement) {
                     modified = true;
                     this._pageTop = null;
                     this._findTop(this._parentQuad);
                 }
                 if (node == this._leftElement) {
                     modified = true;
                     this._pageLeft = null;
                     this._findLeft(this._parentQuad);
                 }
                 if (node == this._rightElement) {
                     modified = true;
                     this._pageRight = null;
                     this._findRight(this._parentQuad);
                 }
                 if (node == this._bottomElement) {
                     modified = true;
                     this._pageBottom = null;
                     this._findBottom(this._parentQuad);
                 }
                 if (modified) {
                     if (this.diagram._pageBackgroundLayer) {
                         this.diagram.model.pageSettings._updatePageSize(this.diagram, true);
                     }
                 }
             },



             _findBottom: function (quad) {
                 if (quad._third != null || quad._fourth != null) {
                     if (quad._third != null) {
                         this._findBottom(quad._third);
                     }
                     if (quad._fourth != null) {
                         this._findBottom(quad._fourth);
                     }
                 }
                 else {
                     if (quad._second != null) {
                         this._findBottom(quad._second);

                     }
                     if (quad._first != null) {
                         this._findBottom(quad._first);
                     }
                 }

                 for (var i = 0; i < quad._objects.length; i++) {
                     var bounds = quad._objects[i]._bounds;
                     if (this._pageBottom <= bounds.bottom() || this._pageBottom == null) {
                         this._pageBottom = bounds.bottom();
                         this._bottomElement = quad._objects[i];
                     }
                 }
             },

             _findRight: function (quad) {
                 {
                     if (quad._second != null || quad._fourth != null) {
                         if (quad._second != null) {
                             this._findRight(quad._second);

                         }
                         if (quad._fourth != null) {
                             this._findRight(quad._fourth);
                         }
                     }
                     else {
                         if (quad._first != null) {
                             this._findRight(quad._first);

                         }
                         if (quad._third != null) {
                             this._findRight(quad._third);
                         }
                     }
                 }

                 for (var i = 0; i < quad._objects.length; i++) {

                     var bounds = quad._objects[i]._bounds;
                     if (this._pageRight <= bounds.right() || this._pageRight == null) {
                         this._pageRight = bounds.right();
                         this._rightElement = quad._objects[i];
                     }
                 }
             },

             _findLeft: function (quad) {//if (quad.Quads.Count == 4)
                 {
                     if (quad._first != null || quad._third != null) {
                         if (quad._first != null) {
                             this._findLeft(quad._first);
                         }
                         if (quad._third != null) {
                             this._findLeft(quad._third);
                         }
                     }
                     else {
                         if (quad._second != null) {
                             this._findLeft(quad._second);
                         }
                         if (quad._fourth != null) {
                             this._findLeft(quad._fourth);
                         }
                     }
                 }
                 for (var i = 0; i < quad._objects.length; i++) {
                     var bounds = quad._objects[i]._bounds;
                     if (this._pageLeft >= bounds.left() || this._pageLeft == null) {
                         this._pageLeft = bounds.left();
                         this._leftElement = quad._objects[i];
                     }
                 }
             },

             _findTop: function (quad) {
                 //if (quad.Quads.Count == 4)
                 {
                     if (quad._first != null || quad._second != null) {
                         if (quad._first != null) {
                             this._findTop(quad._first);
                         }
                         if (quad._second != null) {
                             this._findTop(quad._second);
                         }
                     }
                     else {
                         if (quad._third != null) {
                             this._findTop(quad._third);
                         }
                         if (quad._fourth != null) {
                             this._findTop(quad._fourth);
                         }
                     }
                 }
                 for (var i = 0; i < quad._objects.length; i++) {
                     var bounds = quad._objects[i]._bounds;
                     if (this._pageTop >= bounds.top() || this._pageTop == null) {
                         this._pageTop = bounds.top();
                         this._topElement = quad._objects[i];
                     }
                 }
             },

             _setCurrentNode: function (node) {
                 this._childnode = node;
                 var r = node._bounds;
                 this._childLeft = r.left();
                 this._childTop = r.top();
                 this._childRight = r.right();
                 this._childBottom = r.bottom();
             },

             _initializeNodes: function () {
                 for (var i = 0; i < this.diagram.model.nodes.length; i++) {
                     var nod = this.diagram.model.nodes[i];
                     nod._diagram = this.diagram;
                     if (nod.type === "group") {
                         this._initializeGroup(nod);
                     }
                     else {
                         nod._diagram = this.diagram;
                         nod.updatebounds();
                     }
                 }
                 this.diagram.model.nodes.addEventHandler("collectionchanged", $.proxy(this._collectionChanged, this));
             },

             _initializeGroup: function (group) {
                 for (var i = 0; i < group.children.length; i++) {
                     var child = group.children[i];
                     child._diagram = this.diagram;
                     if (child.type === "group") {
                         this._initializeGroup(child);
                     } else {
                         child.updatebounds();
                         this._updateQuad(child);
                     }
                 }
             },
             _deleteGroup: function (group) {
                 for (var i = 0; i < group.children.length; i++) {
                     var child = group.children[i];
                     if (child.type === "group") {
                         this._deleteGroup(child);
                     } else {
                         if (child.type === "node") {
                             if (!this.diagram.model.nodes.contains(child)) {
                                 this._removeFromaQuad(child);
                                 this._updateBounds(child);
                             }
                         }
                         else
                             if (child.type === "connector") {
                                 if (!this.diagram.model.connectors.contains(child)) {
                                     this._removeFromaQuad(child);
                                     this._updateBounds(child);
                                 }
                             }
                     }
                 }
             },
             _initializeConnectors: function () {
                 for (var i = 0; i < this.diagram.model.connectors.length; i++) {
                     var conn = this.diagram.model.connectors[i];
                     conn._diagram = this.diagram;
                     conn.updatebounds();
                     this._updateQuad(conn);
                 }

                 this.diagram.model.connectors.addEventHandler("collectionchanged", $.proxy(this._collectionChanged, this));
             },

             _collectionChanged: function (evt) {
                 if (!(evt.element.type === "pseudoGroup")) {
                     if (evt.changeType === "insert") {

                         var nod = evt.element;
                         if (nod.type === "group") {
                             nod._diagram = this.diagram;
                             this._initializeGroup(nod);
                         }
                         else {
                             nod._diagram = this.diagram;
                             nod.updatebounds();
                             this._updateQuad(nod);
                         }
                     }
                     else {
                         var nod = evt.element;
                         if (nod.type === "group") {
                             this._deleteGroup(nod);
                         }
                         else {
                             this._removeFromaQuad(nod);
                             this._updateBounds(nod);
                         }

                     }
                 }

             },
         };
    //#endregion

})(jQuery, Syncfusion);;
/**
* @fileOverview OLAP Grid control to visualize the data
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    /**
    * @namespace ej
	* @class ejOlapGrid
	* @requires jQuery
	* @requires ej.common.all
	* @requires ej.tools.js
	* @classdesc Custom Design for OLAP Grid control.
	* @example 
	* &lt;div id="OlapGrid"&gt; &lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create OLAP Grid
    * $("#OlapGrid").ejOlapGrid(...); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejOlapGrid", "ej.OlapGrid", {

        // Root Css Class
        _rootCSS: "e-olapgrid",

        // Widget element will be automatically set in this
        element: null,

        // User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        defaults: {
            //properties
            /**		
			* Connects the service using the specified URL for any server updates.	See {@link url}
			* @default ""
			* @type {String}
			* @example 
			* //To set service url during initialization  
			* $("#OlapGrid").ejOlapGrid({url: "/wcf/OlapGridService.svc"});
			* @example 
			* //Get or set the size API, after initialization:<br>
			* //Gets the url value  
			* $("#OlapGrid").ejOlapGrid("option","url");<br>   
			* //Sets the url value 
			* $("#OlapGrid").ejOlapGrid("option","url", "/wcf/OlapGridService.svc" ); 
			* @memberof ejOlapGrid
			* @instance
			*/
            url: "",

            /**		
			* Sets the CSS name for custom operation.
			* @default ""
			* @type {String}
			* @example 
			* //To set the CSS name during initialization  
			* $("#OlapGrid").ejOlapGrid({cssClass: "gradient-lime"});	
			* @example 
			* //Get or set css class for initialization:<br>
			*	// Gets the css name.		
			*	$("#OlapGrid").ejOlapGrid("option", "cssClass");			
			*	// Sets the rounded corner to button
			*	$("#OlapGrid").ejOlapGrid("option", "cssClass",  "gradient-lime" ); 		
			* @memberof ejOlapGrid
			* @instance
			*/
            cssClass: "",

            /**		
			* Allows the user access the different layouts for OLAP Grid.
			* @default ej.OlapGrid.gridLayout.Normal
			* @type {enum}
			* @example 
			* //To set the layout during initialization  
			* $("#OlapGrid").ejOlapGrid({gridLayout: ej.OlapGrid.gridLayout.NoSummaries});
            * @example 
			* //Get or set the grid layout, after initialization:<br>
			*	//Gets the layout value
			*	$("#OlapGrid").ejOlapGrid("option", "gridLayout");<br>			
			*	//Sets the grid layout
			*	$("#OlapGrid").ejOlapGrid("option", "gridLayout",ej.OlapGrid.gridLayout.Normal);			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            gridLayout: "Normal",

            /**		
			* Allows the user to load large amount of data into pages.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set the virtual scrolling during initialization  
			* $("#OlapGrid").ejOlapGrid({enableVirtualScrolling: true});
            * @example 
			* //Get or set the virtual scrolling, after initialization:<br>
			*	//Gets the virtual calling state
			*	$("#OlapGrid").ejOlapGrid("option", "enableVirtualScrolling");<br>		
			*	//Sets the virtual scrolling 
			*	$("#OlapGrid").ejOlapGrid("option", "enableVirtualScrolling","true");			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            enableVirtualScrolling: false,

            /**		
			* Allows the user to access/operate each cell through a custom design on right click.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set the cell context option during initialization  
			* $("#OlapGrid").ejOlapGrid({enableCellContext: true});
            * @example 
			* //Get or set the cell context, after initialization:<br>
			*	//Gets the cell context state
			*	$("#OlapGrid").ejOlapGrid("option", "enableCellContext");<br>			
			*	//Sets the cell context value
			*	$("#OlapGrid").ejOlapGrid("option", "enableCellContext","true");			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            enableCellContext: false,

            /**		
			* Allows the user to enable/diable hyperlink for value cells.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set the Hyperlink option for value cell during initialization  
			* $("#OlapGrid").ejOlapGrid({enableValueCellHyperlink: true});
            * @example 
			* //Get or set the hyper link for cell, after initialization:<br>
			*	//Gets the hyper link state
			*	$("#OlapGrid").ejOlapGrid("option", "enableValueCellHyperlink");<br>			
			*	//Sets the cell hyperlink value
			*	$("#OlapGrid").ejOlapGrid("option", "enableValueCellHyperlink","true");			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            enableValueCellHyperlink: false,

            /**		
			* Allows the user to enable/diable hyperlink for row header.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set the Hyperlink option for row header during initialization  
			* $("#OlapGrid").ejOlapGrid({enableRowHeaderHyperlink: true});
            * @example 
			* //Get or set the hyper link for row header, after initialization:<br>
			*	//Gets the row header hyperlink state
			*	$("#OlapGrid").ejOlapGrid("option", "enableRowHeaderHyperlink");<br>		
			*	//Sets the row header hyperlink
			*	$("#OlapGrid").ejOlapGrid("option", "enableRowHeaderHyperlink","true");			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            enableRowHeaderHyperlink: false,

            /**		
			* Allows the user to enable/diable hyperlink for column header.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set the Hyperlink option for column header during initialization  
			* $("#OlapGrid").ejOlapGrid({enableColumnHeaderHyperlink: true});
            * @example 
			* //Get or set the hyper link for column header, after initialization:<br>
			*	//Gets the hyper link state
			*	$("#OlapGrid").ejOlapGrid("option", "enableColumnHeaderHyperlink");<br>   				
			*	//Sets the column header hyper link state
			*	$("#OlapGrid").ejOlapGrid("option", "enableColumnHeaderHyperlink","true");			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            enableColumnHeaderHyperlink: false,

            /**		
			* Allows the user to enable/diable hyperlink for summary cells.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set the Hyperlink option for summary cell during initialization  
			* $("#OlapGrid").ejOlapGrid({enableSummaryCellHyperlink: true});
            * @example 
			* //Get or set the hyper link for summary cell, after initialization:<br>
			*	//Gets the summary cell hyperlink state
			*   $("#OlapGrid").ejOlapGrid("option", "enableSummaryCellHyperlink");<br>   		
			*	//Sets the summary cell hyperlink 
			*   $("#OlapGrid").ejOlapGrid("option", "enableSummaryCellHyperlink","true");			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            enableSummaryCellHyperlink: false,

            /**		
			* Sets the progress indicator type which arises on any UI operation.
			* @default this.model.progressMode.Infinite
			* @type {enum}
			* @example 
			* //To set the progress mode during initialization  
			* $("#OlapGrid").ejOlapGrid({progressMode: this.model.progressMode.Infinite});
            * @example 
			* //Get or set the progress mode, after initialization:<br>
			*	//Gets the hyper link state
			* $("#OlapGrid").ejOlapGrid("option", "progressMode");<br>   			
			*	//Sets the progress mode value
			* $("#OlapGrid").ejOlapGrid("option", "progressMode", this.model.progressMode.Progress ); 
			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            progressMode: "infinite",

            /**  
			* Allows the user to set the custom name for the methods at service-end invoked on AJAX post.
			* @default {}
			* @type {string}
			* @example 
			* //To set serviceMethods API value during initialization  
            * $("#OlapGrid").ejOlapGrid({ servieMethods.initialize: "InitializeOlapGrid" });
			* @example 
			* //Get or set the serviceMethods API, after initialization:<br>
			* //Gets the serviceMethods value   
			* $("#OlapGrid").ejOlapGrid("option", "serviceMethods");<br>   						
			* //Sets the serviceMethods value   			
			* $("#OlapGrid").ejOlapGrid("option", "serviceMethods",  {initialize: "InitializeOlapGrid"} ); 
			* @memberof ejOlapGrid
			* @instance
			*/
            serviceMethods: {
                initialize: "InitializeOlapGrid", drillDown: "DrillOlapGrid", exportOptions: "ExportOptions", paging: "Paging"
            },

            /**		
			* Custom object to pass additional information between client-end and service-end.
			* @default null
			* @type {Object}
			* @example 
			* // To pass additional information between client-end and service-end   
			*  $("#OlapGrid").ejOlapGrid({customObject: { Language: "en-US" }}); 
			* @example 
			* //Get or set the custom object API, after initialization:<br>
			*	//Gets the custom object value  
			*	$("#OlapGrid").ejOlapGrid("option","customObject");<br>			
			*	//Sets the custom object value 
			*	$("#OlapGrid").ejOlapGrid("option","customObject", {Language: "en-US"} );  		
			* @memberof ejOlapGrid
			* @instance
			*/
            customObject: {},

            /**		
			* Sets the localized language for the control.
			* @default "en-US"
			* @type {String}
			* @example 
			* //Sets the localized language    
			*  $("#OlapGrid").ejOlapGrid({localization: "en-US"}}); 
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the localization string value  
			*	$("#OlapGrid").ejOlapGrid("option","localization");<br>			
			*	//Sets the localization string value 
			*	$("#OlapGrid").ejOlapGrid("option","localization", "en-US" );  		
			* @memberof ejOlapGrid
			* @instance
			*/
            localization: "en-US",

            //events

            /**     
            * Fires when the value cell is clicked once if OLAP Grid enabled with hyperlink.
            * @event
            * @name ejOlapGrid#valueCellHyperlinkClick 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.   
            * @example 
            * //valueCellHyperlinkClick event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    valueCellHyperlinkClick: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            valueCellHyperlinkClick: "",

            /**     
            * Fires when the row header cell is clicked once if OLAP Grid enabled with hyperlink.
            * @event
            * @name ejOlapGrid#rowHeaderHyperlinkClick 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.   
            * @example 
            * //rowHeaderHyperlinkClick event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    rowHeaderHyperlinkClick: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            rowHeaderHyperlinkClick: "",

            /**     
            * Fires when the column header cell is clicked once if OLAP Grid enabled with hyperlink.
            * @event
            * @name ejOlapGrid#columnHeaderHyperlinkClick 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.     
            * @example 
            * //columnHeaderHyperlinkClick event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    columnHeaderHyperlinkClick: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            columnHeaderHyperlinkClick: "",

            /**     
            * Fires when the summary cell is clicked.
            * @event
            * @name ejOlapGrid#summaryCellHyperlinkClick 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.
            * @example 
            * //summaryCellHyperlinkClick event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    summaryCellHyperlinkClick: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            summaryCellHyperlinkClick: "",

            /**     
            * Fires before service invoked.
            * @event
            * @name ejOlapGrid#beforeServiceInvoke 	
            * @param {Object} argument Event parameters from OLAP Grid  
            * @param {string}  argument.Action return the current action of OLAP Grid control.		 
            * @param {object}  argument.CustomObject return the custom object bounds with OLAP Grid control.
            * @param {string}  argument.Element return the outer HTML of OLAP Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //beforeServiceInvoke event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    beforeServiceInvoke: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            beforeServiceInvoke: null,

            /**     
            * Fires after the service is invoked.
            * @event
            * @name ejOlapGrid#afterServiceInvoke 	
            * @param {Object} argument Event parameters from OLAP Grid 
            * @param {string}  argument.Action return the current action of OLAP Grid control.		 
            * @param {object}  argument.CustomObject return the custom object bounds with OLAP Grid control.
            * @param {string}  argument.Element return the outer HTML of OLAP Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model
            * @param {string}  argument.type returns the name of the event			 
            * @example 
            * //afterServiceInvoke event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    afterServiceInvoke: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            afterServiceInvoke: null,

            /**     
            * Fires after drill down of OLAP Grid.
            * @event
            * @name ejOlapGrid#drillSuccess 	
            * @param {Object} argument Event parameters from OLAP Grid    
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model
            * @param {string}  argument.type returns the name of the event				 
            * @example 
            * //drillSuccess event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    drillSuccess: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            drillSuccess: null,

            /**     
            * Fires when the cell is right clicked.
            * @event
            * @name ejOlapGrid#cellContextEvent 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.
            * @example 
            * //cellContextEvent event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    cellContextEvent: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            cellContextEvent: ""
        },

        /**
         * Provides information about the properties of type Array and Object.
         *
         * @private
         */
        dataTypes: {
            serviceMethods: "data",
            customObject: "data"
        },

        observables: ["gridLayout", "enableCellContext", "enableValueCellHyperlink", "enableRowHeaderHyperlink", "enableColumnHeaderHyperlink", "enableSummaryCellHyperlink"],
        gridLayout: ej.util.valueFunction("gridLayout"),
        enableCellContext: ej.util.valueFunction("enableCellContext"),
        enableValueCellHyperlink: ej.util.valueFunction("enableValueCellHyperlink"),
        enableRowHeaderHyperlink: ej.util.valueFunction("enableRowHeaderHyperlink"),
        enableColumnHeaderHyperlink: ej.util.valueFunction("enableColumnHeaderHyperlink"),
        enableSummaryCellHyperlink: ej.util.valueFunction("enableSummaryCellHyperlink"),
        localization: ej.util.valueFunction("localization"),

        /**
         * Gets the serialized current OLAP Report of the control.
         *
         * @private 
         */
        getOlapReport: function () {
            return this._olapReport;
        },

        /**
         * Sets the serialized current OLAP Report of the control.
         *
         * @private 
         */
        setOlapReport: function (value) {
            this._olapReport = value;
        },

        /**
        * Gets the JSON formatted records for the control.
        *
        * @private 
        */
        getJSONRecords: function () {
            return this._JSONRecords;
        },

        /**
        * Sets the JSON formatted records for the control.
        *
        * @private 
        */
        setJSONRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        /**
        * Gets the virtual scroll state.
        *
        * @private 
        */
        get_enableVirtualScrolling: function () {
            return this.enableVirtualScrolling;
        },

        /**
        * Sets the virtual scroll state.
        *
        * @private 
        */
        set_enableVirtualScrolling: function (value) {
            this.enableVirtualScrolling = value;
        },

        /**
         * Initializes the variables and loads the widget. 
         *
         * @private
         */
        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        /**
         * Destroys the widget while it's no longer required. 
         *
         * @private
         */
        _destroy: function () {
            this.element.empty().removeClass("e-olapgrid" + this.model.cssClass);
        },

        /**
         * Initializes the local variables. 
         *
         * @private
         */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            this._olapReport = "";
            this._JSONRecords = null;
            this._drillCaption = "";
            this._drillAction = "";
            this._startDrilldown = false;
            this._pagerObj = null;
            this._seriesPageCount = null;
            this._categPageCount = null;
            this._seriesCurrentPage = 1;
            this._categCurrentPage = 1;
            ogridWaitingPopup = null;
            _proxy = this;
            ogridProgressBar = null;
            ogridprogressText = null;
            oGridTimer = null;
            this._maxViewLoading = null;
        },

        /**
         * This function is the point were loading/rendering of the OLAP Grid widget is initiated.
         *
         * @private  
         */
        _load: function () {
            this.element.addClass(this.model.cssClass);
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null)
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ autoDisplay: true });
                    this._maxViewLoading = ("#" + oclientProxy._id + "_maxView").data("ejWaitingPopup");
                }
                else
                    oclientWaitingPopup.show();
            else {
                if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                    $("#" + this._id).ejWaitingPopup({ autoDisplay: true });
                    ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                }
                else {
                    $("#" + _proxy._id + "progressContainer").remove();
                    this._createProgressBar();
                    this._progressStatus(0, 70, 140);
                }
            }

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { Action: ej.OlapGrid.gridAction.Initialize, Element: this.element, CustomObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
            if (this.element[0] != null && this.element.parents().find(".controlPanel").length > 0 && this.gridLayout() != "" && this.gridLayout() != ej.OlapGrid.gridLayout.Normal)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({ "action": "initializeGrid", "olapReport": this.model.olapReport, "gridLayout": this.gridLayout(), "customObject": serializedCustomObject }), this._renderControlSuccess);
            else if (this.element[0] != null && this.element.parents().find(".controlPanel").length > 0)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({ "action": "initializeGrid", "olapReport": this.model.olapReport, "customObject": serializedCustomObject }), this._renderControlSuccess);
            else if (this.gridLayout() != "" && this.gridLayout() != ej.OlapGrid.gridLayout.Normal)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({ "action": "initializeGrid", "gridLayout": this.gridLayout(), "customObject": serializedCustomObject }), this._renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({ "action": "initializeGrid", "customObject": serializedCustomObject }), this._renderControlSuccess);
        },
        _setFirst: true,

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                    case "JsonData": this.setOlapReport(options[key]); break;
                    case "RefreshOlapGrid": this.element.renderControlFromJSON(options[key]); break;
                    case "gridLayout": this._load(); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "enableCellContext": this._load(); break;
                    case "enableValueCellHyperlink": this._load(); break;
                    case "enableRowHeaderHyperlink": this._load(); break;
                    case "enableColumnHeaderHyperlink": this._load(); break;
                    case "enableSummaryCellHyperlink": this._load(); break;
                    case "localization": this._load(); break;
                }
            }
        },

        /**
         * This function is invoked to wire the events for UI interaction.
         *
         * @private  
         */
        _wireEvents: function () {
            if (this.enableCellContext() == true) {
                if (document.addEventListener) {
                    document.removeEventListener('contextmenu', _proxy._cellContext, false);
                    document.addEventListener('contextmenu', _proxy._cellContext, false);
                }
                else {
                    document.detachEvent('oncontextmenu', _proxy._cellContext);
                    document.attachEvent('oncontextmenu', _proxy._cellContext);
                }
            }
            this.drillDownHandler = $.proxy(this._drillDown, this);
            this.addHyperlinkHandler = $.proxy(this._addHyperlink, this);
            this.removeHyperlinkHandler = $.proxy(this._removeHyperlink, this);
            this._on(this.element, "mouseover", ".value, .rowheader, .colheader", this.addHyperlinkHandler);
            this._on(this.element, "mouseleave", ".value, .rowheader, .colheader", this.removeHyperlinkHandler);
            this._on(this.element, "click", ".value, .colheader, .rowheader", function (e) {
                var cellclick = null;
                if (this.enableSummaryCellHyperlink() && e.target.className.indexOf("summary value") >= 0 && this.model.summaryCellHyperlinkClick != "")
                    cellclick = eval(this.model.summaryCellHyperlinkClick);
                else if (this.enableValueCellHyperlink() && e.target.className.indexOf("value") >= 0 && this.model.valueCellHyperlinkClick != "")
                    cellclick = eval(this.model.valueCellHyperlinkClick);
                else if (this.enableRowHeaderHyperlink() && e.target.className.indexOf("rowheader") >= 0 && this.model.rowHeaderHyperlinkClick != "")
                    cellclick = eval(this.model.rowHeaderHyperlinkClick);
                else if (this.enableColumnHeaderHyperlink() && e.target.className.indexOf("colheader") >= 0 && this.model.columnHeaderHyperlinkClick != "")
                    cellclick = eval(this.model.columnHeaderHyperlinkClick);
                if (cellclick != null) {
                    var cellPos = $(e.target).attr("p");
                    var rawdata = this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
                    var cellInfo = {
                        cellValue: e.target.innerHTML,
                        cellPosition: cellPos,
                        cellType: e.target.className.split(' ')[0],
                        uniqueName: rawdata.split('::')[0],
                        args: e,
                        rawdata: rawdata
                    };
                    if (cellInfo.cellValue != "") cellclick(cellInfo);
                }
            });
            this._on(this.element, "click", ".expand, .collapse", ej.proxy(this._drillDown, this));
            this._on(this.element, "mouseover", ".colheader, .rowheader, .expand, .collapse", function (evt) {
                if (evt.target.className.indexOf("expand") > -1 && !$(evt.target).hasClass("header-hover-expand"))
                    $(evt.target).addClass("header-hover-expand");
                else if (evt.target.className.indexOf("collapse") > -1 && !$(evt.target).hasClass("header-hover-collapse"))
                    $(evt.target).addClass("header-hover-collapse");
                else if ((evt.target.className.indexOf("expand") < 0 || evt.target.className.indexOf("collapse") < 0) && $(evt.target).children("span").length > 0) {
                    var child = $(evt.target).children("span")[0];
                    if (!$(child).hasClass("header-hover-expand") && child.className.indexOf("expand") > -1)
                        $(child).addClass("header-hover-expand");
                    else if (!$(child).hasClass("header-hover-collapse") && child.className.indexOf("collapse") > -1)
                        $(child).addClass("header-hover-collapse");
                }
            });
            this._on(this.element, "mouseleave", ".colheader, .rowheader, .expand, .collapse", function (evt) {
                this.element.find(".expand, .collapse").removeClass("header-hover-expand header-hover-collapse");
            });
        },

        /**
         * This function is invoked to unwire the events registered for UI interaction.
         *
         * @private  
         */
        _unWireEvents: function () {
            this._off(this.element, "click", ".expand, .collapse");
            this._off(this.element, "click", ".value, .colheader, .rowheader");
            this._off(this.element, "mouseover", ".colheader, .rowheader, .expand, .collapse");
            this._off(this.element, "mouseleave", ".colheader, .rowheader, .expand, .collapse");
            this._off(this.element, "mouseover", ".value, .rowheader, .colheader", this.addHyperlinkHandler);
            this._off(this.element, "mouseleave", ".value, .rowheader, .colheader", this.removeHyperlinkHandler);
        },

        /**
         * This function is used to create and handle virtual scrollbar.
         *
         * @private  
         */
        _applyVScrolling: function () {
            var _proxy = this;
            var oGridDiv = $(this.element).find('.oGridOuterDiv');
            if (oGridDiv.width() > this.element.width() - 40) $(oGridDiv).css({ width: this.element.width() - 40, overflow: "auto" });
            if (this._seriesPageCount > 1) {
                var getMinVerticalPos = function () { return (_proxy.element.find(".vScrollPanel").position().top + 1); };
                var vScrollBar = $(this.element).find(".vScrollPanel");
                vScrollBar.height($(oGridDiv).height());
                vScrollBar.html(ej.buildTag("div.vScrollThumb")[0].outerHTML);
                var vScrollHandle = vScrollBar.find(".vScrollThumb");
                vScrollHandle.height(Math.max(Math.ceil($(oGridDiv).height() / _proxy._seriesPageCount), 15));
                var vDragRange = vScrollBar.height() - vScrollHandle.height() - 2;
                vScrollHandle.offset({ top: getMinVerticalPos() + (_proxy._seriesCurrentPage - 1) * (vDragRange / (_proxy._seriesPageCount - 1)) });

                $(vScrollHandle).bind('mousedown', function (e) {
                    $(this).addClass("dragging");
                    var initCursorPos = e.pageY - getMinVerticalPos();
                    var initThumbPos = vScrollHandle.position().top - getMinVerticalPos();
                    _proxy.element.find(".seriesPageIndicator").removeClass("inActive").css({ top: vScrollHandle.position().top, left: vScrollHandle.position().left + 20 });
                    var currentPage = _proxy._seriesCurrentPage;
                    $(document).bind('mousemove', function (e) {
                        if (window.navigator.userAgent.indexOf('MSIE ') > -1)
                            $(document).on("selectstart", function (e) { e.preventDefault(); });
                        else
                            window.getSelection().removeAllRanges();
                        var movedDistance = (e.pageY - getMinVerticalPos()) - initCursorPos;
                        if (movedDistance > 0 && initThumbPos + movedDistance <= vDragRange)
                            vScrollHandle.offset({ top: initThumbPos + movedDistance + getMinVerticalPos() });
                        else if (movedDistance < 0 && initThumbPos + movedDistance >= 0)
                            vScrollHandle.offset({ top: initThumbPos + movedDistance + getMinVerticalPos() });
                        else if (initThumbPos + movedDistance > vDragRange)
                            vScrollHandle.offset({ top: getMinVerticalPos() + vDragRange });
                        else if (initThumbPos + movedDistance < 0)
                            vScrollHandle.offset({ top: getMinVerticalPos() });
                        currentPage = Math.ceil((vScrollHandle.position().top - getMinVerticalPos()) / (vDragRange / _proxy._seriesPageCount)) == 0 ? 1 : Math.ceil((vScrollHandle.position().top - getMinVerticalPos()) / (vDragRange / _proxy._seriesPageCount));
                        _proxy.element.find(".seriesPageIndicator").css({ top: vScrollHandle.position().top });
                        _proxy.element.find(".series_CurrentPage").html(currentPage);
                    });
                    $(document).bind('mouseup', function () {
                        $(document).off("selectstart");
                        $(this).unbind('mousemove').unbind('mouseup');
                        $(vScrollHandle).removeClass("dragging");
                        if (currentPage != _proxy._seriesCurrentPage) {
                            _proxy._seriesCurrentPage = currentPage;
                            _proxy.refreshPagedOlapGrid("series", _proxy._seriesCurrentPage);
                        }
                        else
                            _proxy.element.find(".seriesPageIndicator").addClass("inActive");
                    });
                });
            }

            if (this._categPageCount > 1) {
                var getMinHorizontalPos = function () { return (_proxy.element.find(".hScrollPanel").position().left + 1); };
                var hScrollBar = $(this.element).find(".hScrollPanel");
                hScrollBar.width($(oGridDiv).width());
                hScrollBar.html(ej.buildTag("div.hScrollThumb")[0].outerHTML);
                var hScrollHandle = hScrollBar.find(".hScrollThumb");
                hScrollHandle.width(Math.max(Math.ceil($(oGridDiv).width() / _proxy._categPageCount), 15));
                var hDragRange = hScrollBar.width() - hScrollHandle.width() - 2;
                hScrollHandle.offset({ left: getMinHorizontalPos() + (_proxy._categCurrentPage - 1) * (hDragRange / (_proxy._categPageCount - 1)) });

                $(hScrollHandle).bind('mousedown', function (e) {
                    $(this).addClass("dragging");
                    var initCursorPos = e.pageX - getMinHorizontalPos();
                    var initThumbPos = hScrollHandle.position().left - getMinHorizontalPos();
                    _proxy.element.find(".categPageIndicator").removeClass("inActive").css({ left: hScrollHandle.position().left, top: hScrollHandle.position().top + 20 });
                    var currentPage = _proxy._categCurrentPage;
                    $(document).bind('mousemove', function (e) {
                        if (window.navigator.userAgent.indexOf('MSIE ') > -1)
                            $(document).on("selectstart", function (e) { e.preventDefault(); });
                        else
                            window.getSelection().removeAllRanges();
                        var movedDistance = (e.pageX - getMinHorizontalPos()) - initCursorPos;
                        if (movedDistance > 0 && initThumbPos + movedDistance <= hDragRange)
                            hScrollHandle.offset({ left: initThumbPos + movedDistance + getMinHorizontalPos() });
                        else if (movedDistance < 0 && initThumbPos + movedDistance >= 0)
                            hScrollHandle.offset({ left: initThumbPos + movedDistance + getMinHorizontalPos() });
                        else if (initThumbPos + movedDistance > hDragRange)
                            hScrollHandle.offset({ left: getMinHorizontalPos() + hDragRange });
                        else if (initThumbPos + movedDistance < 0)
                            hScrollHandle.offset({ left: getMinHorizontalPos() });
                        currentPage = Math.ceil((hScrollHandle.position().left - getMinHorizontalPos()) / (hDragRange / _proxy._categPageCount)) == 0 ? 1 : Math.ceil((hScrollHandle.position().left - getMinHorizontalPos()) / (hDragRange / _proxy._categPageCount));
                        _proxy.element.find(".categPageIndicator").css({ left: hScrollHandle.position().left });
                        _proxy.element.find(".categ_CurrentPage").html(currentPage);
                    });
                    $(document).bind('mouseup', function () {
                        $(document).off("selectstart");
                        $(this).unbind('mousemove').unbind('mouseup');
                        $(hScrollHandle).removeClass("dragging");
                        if (currentPage != _proxy._categCurrentPage) {
                            _proxy._categCurrentPage = currentPage;
                            _proxy.refreshPagedOlapGrid("categ", _proxy._categCurrentPage);
                        }
                        else
                            _proxy.element.find(".categPageIndicator").addClass("inActive");
                    });
                });
            }
        },

        /**
        * To get the localized labels.
        * @private
        */
        //get Localization text
        _getLocalizedLabels: function (property) {

            return ej.OlapGrid.localization[this.localization()][property] === undefined ? ej.OlapGrid.localization["en-US"][property] : ej.OlapGrid.localization[this.localization()][property];
        },

        /**
         * This function would be raised while clicking expand/collapse icons to perform drill up/down operations in OLAP Grid.
         *
         * @private  
         */
        _drillDown: function (e) {
            this._off(this.element, "click", ".expand, .collapse");
            var targetClass = e.target.className;
            this._startDrilldown = true;
            targetClass = targetClass.split(" ");
            if (targetClass[0] == "expand")
                this._drillAction = "drilldown";
            else if (targetClass[0] == "collapse")
                this._drillAction = "drillup";

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { Action: ej.OlapGrid.gridAction.DrillDown, Element: this.element, CustomObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            var cellPos = $(e.target).parent().attr("p");
            this._drillCaption = $.trim($(e.currentTarget).parent().text());

            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ autoDisplay: true });
                    this._maxViewLoading = $("#" + oclientProxy._id + "_maxView").data("ejWaitingPopup");
                }
                else
                    oclientWaitingPopup.show();
            }
            else {
                if (_proxy.model.progressMode == "infinite")
                    ogridWaitingPopup.show();
                else {
                    this._progressStatus(0, 70, 140);
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
                }
            }
            if (this.gridLayout() != "" || this.gridLayout() != "Normal") {
                if (typeof (oclientProxy) != "undefined")
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": this.getOlapReport(), "clientReports": reports, "headerInfo": this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info, "gridLayout": this.gridLayout(), "customObject": serializedCustomObject }), this._drillDownSuccess);
                else
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": this.getOlapReport(), "headerInfo": this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info, "gridLayout": this.gridLayout(), "customObject": serializedCustomObject }), this._drillDownSuccess);
            }
            else {
                if (typeof (oclientProxy) != "undefined")
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": this.getOlapReport(), "clientReports": reports, "headerInfo": this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info, "customObject": serializedCustomObject }), this._drillDownSuccess);
                else
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": this.getOlapReport(), "headerInfo": this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info, "customObject": serializedCustomObject }), this._drillDownSuccess);
            }
        },

        /**
         * This function would apply hyperlink to the appropriate cells.
         *
         * @private  
         */
        _addHyperlink: function (e) {
            if ($(".hyperlinkValueCell")[0])
                $($(".hyperlinkValueCell")[0]).removeClass("hyperlinkValueCell");
            if ($(".hyperlinkHeaderCell")[0])
                $($(".hyperlinkHeaderCell")[0]).removeClass("hyperlinkHeaderCell");
            if (e.target.className == "value" && this.enableValueCellHyperlink())
                $(e.target).addClass("hyperlinkValueCell");
            else if (e.target.className == "summary value" && this.enableSummaryCellHyperlink())
                $(e.target).addClass("hyperlinkValueCell");
            else if (e.target.className == "rowheader" && this.enableRowHeaderHyperlink())
                $(e.target).addClass("hyperlinkHeaderCell");
            else if (e.target.className == "colheader" && this.enableColumnHeaderHyperlink())
                $(e.target).addClass("hyperlinkHeaderCell");
        },

        /**
         * This function would remove/disable hyperlink in OLAP Grid.
         *
         * @private  
         */
        _removeHyperlink: function (e) {
            $(e.target).removeClass("hyperlinkValueCell").removeClass("hyperlinkHeaderCell");
        },

        /**
         * This function would be raised while right clicking over any cell in OLAP Grid.
         *
         * @private  
         */
        _cellContext: function (e) {
            var currentCell = e.target || e.srcElement;
            if (_proxy.enableCellContext() == true && $(currentCell).parents("#" + _proxy._id)[0]) {
                var cellPos = $(currentCell).attr("p");
                if (cellPos) {
                    var cellPos = $(currentCell).attr("p");
                    var rawdata = _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
                    var cellInfo = {
                        cellValue: currentCell.innerHTML,
                        cellPosition: cellPos,
                        cellType: currentCell.className.split(' ')[0],
                        uniqueName: rawdata.split('::')[0],
                        args: e,
                        rawdata: rawdata
                    };
                    var clickevent = eval(_proxy.model.cellContextEvent);
                    if (cellInfo.cellType != "" && cellInfo.cellValue != "") clickevent(cellInfo);
                    e.target ? e.preventDefault() : (window.event.returnValue = false);
                }
            }
        },

        /**
         * This function would be raised while exporting OLAP Grid to Excel.
         *
         * @private  
         */
        exportToExcel: function () {
            var args = {
                currentReport: _proxy.getOlapReport(),
                layout: _proxy.gridLayout(),
                summaryCellColor: $(".summary").css("background-color")
            };
            this.doFullPost(_proxy.model.url + "/" + _proxy.model.serviceMethods.exportOptions, args);
        },

        /**
         * This function is used to create progress bar containing status text and percentage.
         *
         * @private  
         */
        _createProgressBar: function () {
            var progressContainer = ej.buildTag("div#" + this._id + "progressContainer.progressContainer", "", { top: typeof (oclientProxy) != "undefined" ? ((oclientProxy.model.displayOptions.controlPlacement == "tab" || oclientProxy.model.displayOptions.mode != "chartAndGrid") ? oclientProxy.progressPos.top + 300 : (oclientProxy.model.displayOptions.defaultView == "chart" ? oclientProxy.progressPos.top + 500 : oclientProxy.progressPos.top + 200)) : ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 30), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left + 550 : ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 100) })[0].outerHTML;
            var progressBar = ej.buildTag("div#" + this._id + "progressBar.progressBar", "")[0].outerHTML;
            var blockControl = ej.buildTag("div#" + this._id + "blockDiv.blockDiv", "", { width: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).width()) : $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).height()) : $("#" + this._id).height(), position: 'fixed', top: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.top : ($("#" + this._id).position().top), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left : ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + _proxy._id).after(progressContainer);
            if (_proxy.model.progressMode == "normal") {
                $("#" + this._id + "progressContainer").css({ width: 0, height: 0 });
                $("#" + _proxy._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + this._id + "progressText.progressText", "OlapGrid initial loading...")[0].outerHTML;
                $("#" + _proxy._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + _proxy._id + "progressBar").ejProgressBar({ value: 0, text: 0 + " %", complete: "progressbarCompleted" });
            ogridProgressBar = $("#" + _proxy._id + "progressBar").data("ejProgressBar");
            ogridProgressText = $("#" + _proxy._id + "progressText");
        },

        /**
         * This function is used update the progress status.
         *
         * @private  
         */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            ogridProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + _proxy._id + "progressContainer").show();
            $("#" + _proxy._id + "blockDiv").position = $("#" + _proxy._id).position();
            oGridTimer = window.setInterval(function () {
                ogridProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(oGridTimer);
                    if (progressText != undefined)
                        ogridProgressBar.setModel({ value: 100, text: progressText[1] });

                };
            }, progress_Speed);
        },

        /**
         * 	This function receives the controls update from service-end which would be utilized for rendering the widget.
         *
         * @private  
         */
        _renderControlSuccess: function (msg) {
            _proxy = this;
            if (_proxy.model.progressMode != "infinite") {
                if (_proxy.model.progressMode == "progress")
                    $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
                window.clearInterval(oGridTimer);
            }
            try {
                if (_proxy.model.progressMode != "infinite") {
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingStarted") + "...";
                    _proxy._progressStatus(70, 85, 50);
                    window.setTimeout(function () { progressStatus() }, 300);
                }
                else
                    window.setTimeout(function () { progressStatus() }, 1);
                function progressStatus() {
                    if (msg[0] != undefined && msg.length > 0) {
                        _proxy.setJSONRecords(msg[0].Value); _proxy.setOlapReport(msg[1].Value);
                        if (msg[2] != null && msg[2] != undefined && msg[2].Key == "PageSettings") {
                            if (_proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg[3].Value).Column / JSON.parse(msg[2].Value).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg[3].Value).Row / JSON.parse(msg[2].Value).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg[2].Value).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg[2].Value).SeriesPageSize;
                            }
                            else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg[3].Value), JSON.parse(msg[2].Value));
                            if (msg[4] != null && msg[4] != undefined)
                                _proxy.model.customObject = msg[4].Value;
                        }
                        if (msg[2] != null && msg[2] != undefined && msg[2].Key != "PageSettings")
                            _proxy.model.customObject = msg[2].Value;
                    }
                    else if (msg.d != undefined && msg.d.length > 0) {
                        _proxy.setJSONRecords(msg.d[0].Value); _proxy.setOlapReport(msg.d[1].Value);
                        if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "PageSettings") {
                            if (_proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Column / JSON.parse(msg.d[2].Value).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Row / JSON.parse(msg.d[2].Value).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg.d[2].Value).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg.d[2].Value).SeriesCurrentPage;
                            }
                            else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg.d[3].Value), JSON.parse(msg.d[2].Value));
                            if (msg.d[4] != null && msg.d[4] != undefined)
                                _proxy.model.customObject = msg.d[4].Value;
                        }
                        if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Value == "PageSettings")
                            _proxy.model.customObject = msg.d[2].Value;
                    }
                    else if (msg != undefined && msg.length > 0) {
                        _proxy.setJSONRecords(msg.PivotRecords); _proxy.setOlapReport(msg.OlapReport);
                        if (msg.length > 2) {
                            if (_proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Column / JSON.parse(msg.PageSettings).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Row / JSON.parse(msg.PageSettings).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg.PageSettings).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg.PageSettings).SeriesCurrentPage;
                            }
                            else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg.HeaderCounts), JSON.parse(msg.PageSettings));
                        }
                        if (msg.customObject != null && msg.customObject != undefined)
                            _proxy.model.customObject = msg.customObject;
                    }
                    if (_proxy.model.afterServiceInvoke != null)
                        _proxy._trigger("afterServiceInvoke", { Action: ej.OlapGrid.gridAction.DrillDown, Element: _proxy.element, CustomObject: _proxy.model.customObject });
                    if (_proxy.getJSONRecords() != null && _proxy.getJSONRecords().length > 0)
                        _proxy.renderControlFromJSON(_proxy.getJSONRecords());
                    if (_proxy.model.enableVirtualScrolling)
                        _proxy._applyVScrolling();
                    if (_proxy.model.progressMode != "infinite") {
                        if (_proxy._JSONRecords == null)
                            $("#" + _proxy._id + "progressContainer").hide();
                        else {
                            if (_proxy.model.progressMode == "progress")
                                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingSucceeded") + "...";
                        }
                    }
                    else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0] && this._maxViewLoading) {
                            this._maxViewLoading.hide();
                        }
                        else
                            oclientWaitingPopup.hide();
                    }
                    else
                        ogridWaitingPopup.hide();
                }
            }
            catch (err) {
                if (_proxy.model.progressMode != "infinite") {
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingFailed") + "!!!";
                    window.clearInterval(oChartTimer);
                    ogridProgressBar.setModel({ value: 70, text: " Failed" });
                }
            }
        },

        /**
         * 	This function receives the controls update from service-end after drilldown which would be utilized for rendering the widget.
         *
         * @private  
         */
        _drillDownSuccess: function (msg) {
            _proxy = this;
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
            if (msg[0] != undefined) {
                this.setJSONRecords(msg[0].Value);
                this.setOlapReport(msg[1].Value);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg[1].Value;
                    oclientProxy.reports = msg[2].Value;
                }
                if (msg[2] != null && msg[2] != undefined && msg[2].Key == "PageSettings") {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg[3].Value).Column / JSON.parse(msg[2].Value).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg[3].Value).Row / JSON.parse(msg[2].Value).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg[2].Value).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg[2].Value).SeriesPageSize;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg[3].Value), JSON.parse(msg[2].Value));
                    if (msg[4] != null && msg[4] != undefined)
                        _proxy.model.customObject = msg[4].Value;
                }
                if (msg[2] != null && msg[2] != undefined && msg[2].Key != "PageSettings")
                    _proxy.model.customObject = msg[2].Value;
            }
            else if (msg.d != undefined) {
                this.setJSONRecords(msg.d[0].Value);
                this.setOlapReport(msg.d[1].Value);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg.d[1].Value;
                    oclientProxy.reports = msg.d[2].Value;
                }
                if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "PageSettings") {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Column / JSON.parse(msg.d[2].Value).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Row / JSON.parse(msg.d[2].Value).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg.d[2].Value).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg.d[2].Value).SeriesCurrentPage;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg.d[3].Value), JSON.parse(msg.d[2].Value));
                    if (msg.d[4] != null && msg.d[4] != undefined)
                        _proxy.model.customObject = msg.d[4].Value;
                }
                if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Value == "PageSettings")
                    _proxy.model.customObject = msg.d[2].Value;
            }
            else {
                this.setJSONRecords(msg.PivotRecords);
                this.setOlapReport(msg.OlapReport);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg.OlapReport;
                    oclientProxy.reports = msg.reports;
                }
                if (msg.length > 2) {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Column / JSON.parse(msg.PageSettings).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Row / JSON.parse(msg.PageSettings).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg.PageSettings).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg.PageSettings).SeriesCurrentPage;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg.HeaderCounts), JSON.parse(msg.PageSettings));
                }
                if (msg.customObject != null && msg.customObject != undefined)
                    _proxy.model.customObject = msg.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: ej.OlapGrid.gridAction.DrillDown, Element: this.element, CustomObject: this.model.customObject });
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingStarted") + "...";
            this.renderControlFromJSON(this.getJSONRecords());
            this._trigger("drillSuccess", this.element);
            if (_proxy.model.enableVirtualScrolling)
                _proxy._applyVScrolling();
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                if ($("#" + oclientProxy._id + "_maxView")[0] && this._maxViewLoading) {
                    this._maxViewLoading.hide();
                    oclientWaitingPopup.hide();
                }
                else if (oclientProxy.model.displayOptions.mode == "gridOnly")
                    oclientWaitingPopup.hide();
            }
            else if (typeof oclientProxy != 'undefined') {
                if (oclientProxy && !oclientProxy.ogridObj._startDrilldown && !oclientProxy.ochartObj._startDrilldown || oclientProxy.model.displayOptions.mode == "gridOnly")
                    oclientWaitingPopup.hide();
            }
            else if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                ogridWaitingPopup.hide();
            }
            if (typeof oclientProxy != 'undefined')
                oclientProxy.ogridObj._startDrilldown = false;
            else
                ogridWaitingPopup.hide();
        },

        /**
         * Perform an asynchronous HTTP (Ajax) request.
         * @return jQuery
         * @example 
         * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Create OLAP Grid
         * $('#OlapGrid').ejOlapGrid({
         *       url: "OlapGridService.svc",
         *   });
         * var gridObj = $("#OlapGrid").data("ejOlapGrid");
         * gridObj.doAjaxPost("POST", "/OlapGridService.svc/Initialize", {"key", "Hello World"}, "_renderControlSuccess", null);
         * // Initiate an Ajax request.
         * &lt;/script&gt;
         * @memberof ejOlapGrid
         * @instance
         */
        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: $.proxy(onComplete, this),
                error: function (msg, textStatus, errorThrown) {
                    if (_proxy.model.progressMode != "infinite") {
                        window.clearInterval(oGridTimer);
                        if (_proxy.model.progressMode == "progress")
                            $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        $("#" + ogridProgressBar._id).show();
                        ogridProgressBar.setModel({ value: 0, text: errorThrown });
                    }
                    else
                        ogridWaitingPopup.hide();
                }
            });
        },

        /**
         * Perform an asynchronous HTTP (FullPost) submit.
         * @return jQuery
         * @example 
         * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Submitting required informatin to the service.
         * $('#OlapGrid').ejOlapGrid({
         *       url: "OlapGridService.svc",
         *   });
         * var gridObj = $("#OlapGrid").data("ejOlapGrid");
         * gridObj.doFullPost("/OlapGridService.svc/Initialize", {"key", "Hello World"});
         * // Initiate an submit.
         * &lt;/script&gt;
         * @memberof ejOlapGrid
         * @instance
         */
        doFullPost: function (url, params) {
            var form = $('<form>').attr({ 'action': url, 'method': 'POST', 'name': 'export' });
            var addParam = function (paramName, paramValue) {
                var input = $('<input type="hidden" title="params">').attr({
                    'id': paramName,
                    'name': paramName,
                    'value': paramValue
                }).appendTo(form);
            };
            for (var item in params)
                addParam(item, params[item]);
            form.appendTo(document.body).submit().remove();
        },

        /**
         * 	This function re-renders the OLAP Grid on clicking the navigating buttons on OLAP Pager.
         * @return jQuery
         * @example 
         * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Refreshing OLAP Grid while perform paging.
         * $('#OlapGrid').ejOlapGrid({
         *       url: "OlapGridService.svc",
         *   });
         * var gridObj = $("#OlapGrid").data("ejOlapGrid");
         * gridObj.refreshPagedOlapGrid("series", 2);
         * // Initiate an Ajax request.
         * &lt;/script&gt;
         * @memberof ejOlapGrid
         * @instance
         */
        refreshPagedOlapGrid: function (axis, pageNo) {
            if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                ogridWaitingPopup.show();
            }
            else {
                this._progressStatus(0, 70, 140);
                if (_proxy.model.progressMode == "progress")
                    $("#" + _proxy._id + "progressText")[0].innerHTML = "Preparing and executing MDX query...";
            }
            axis = axis.indexOf('categ') != -1 ? "categorical" : "series";
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.paging, JSON.stringify({ "action": "Paging", "pagingInfo": axis + ":" + pageNo, "currentReport": this.getOlapReport(), "gridLayout": this.gridLayout(), "customObject": null }), this._renderControlSuccess);
        },

        /**
         * 	This function receives the JSON formatted datasource to render the OLAP Grid control.
         * @return jQuery
         * @example 
         * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Rendering OLAP Grid from given JSON formatted data.
         * $('#OlapGrid').ejOlapGrid({
         *       url: "OlapGridService.svc",
         *   });
         * var gridObj = $("#OlapGrid").data("ejOlapGrid");
         * gridObj.renderControlFromJSON({this.getJSONRecords()});
         * // Rendering OLAP Grid.
         * &lt;/script&gt;
         * @memberof ejOlapGrid
         * @instance
         */
        renderControlFromJSON: function (jsonObj) {
            _proxy = this;
            if (_proxy.model.progressMode != "infinite")
                window.clearInterval(oGridTimer);
            var tableOlapGrid = $("<table cellspacing=\"0\" cellpadding=\"0\" role='grid'  aria-readonly='true'></table>");
            this.rowCount = 0;

            if (jsonObj != null) {
                jQuery.each(jsonObj, function (index, value) {
                    if (this.Index[0] == 0)
                        _proxy.rowCount++;
                });

                var theadOlapGrid = $("<thead></thead>");
                var tbodyOlapGrid = $("<tbody></tbody>");
                var istheadOlapGrid = true; var columnHeaderHeight = 0;
                if (jsonObj[0].CSS == "none")
                    columnHeaderHeight = jsonObj[0].RowSpan - 1;
                else {
                    jQuery.each(jsonObj, function (index, value) {
                        columnHeaderHeight++;
                        return (jsonObj[index].CSS == "colheader");
                    });
                    columnHeaderHeight = columnHeaderHeight - 2;
                }

                jQuery.each(jsonObj, function (index, value) {
                    if (index < _proxy.rowCount) {
                        if (istheadOlapGrid) {
                            istheadOlapGrid = (index >= columnHeaderHeight) ? false : true;
                            var tRow = "<tr role='row'>";
                            for (var column = index; column < jsonObj.length; column = column + _proxy.rowCount) {
                                if (jsonObj[column].Span != "Block") {
                                    tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=" + jsonObj[column].CSS) + " p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='columnheader'>" + (jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\" title=\"Collapsed\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</th>";
                                    if (jsonObj[column].RowSpan > 1 && jsonObj[column].ColSpan <= 1) {
                                        for (var i = column + 1; i < column + jsonObj[column].RowSpan; i++)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan <= 1) {
                                        for (var i = column + _proxy.rowCount; i < column + (jsonObj[column].ColSpan * _proxy.rowCount) ; i = i + _proxy.rowCount)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan > 1) {
                                        for (var i = column; i < column + jsonObj[column].RowSpan; i++) {
                                            for (var j = i + _proxy.rowCount; j < i + (jsonObj[column].ColSpan * _proxy.rowCount) ; j = j + _proxy.rowCount)
                                                jsonObj[j].Span = "Block";
                                            jsonObj[i].Span = "Block";
                                        }
                                    }
                                }
                            }
                            tRow += "</tr>";
                            $(theadOlapGrid).append(tRow);
                        }
                        else {
                            var tRow = "<tr role='row'>";
                            for (var column = index; column < jsonObj.length; column = column + _proxy.rowCount) {
                                if (jsonObj[column].Span != "Block") {
                                    if (jsonObj[column].CSS == "rowheader" || jsonObj[column].CSS == "summary" || jsonObj[column].CSS == "summary")
                                        tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='rowheader'>" + (jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</th>";
                                    else
                                        tRow = tRow + "<td" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='gridcell'>" + (jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</td>";

                                    if (jsonObj[column].RowSpan > 1 && jsonObj[column].ColSpan <= 1) {
                                        for (var i = column + 1; i < column + jsonObj[column].RowSpan; i++)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan <= 1) {
                                        for (var i = column + _proxy.rowCount; i < column + (jsonObj[column].ColSpan * _proxy.rowCount) ; i = i + _proxy.rowCount)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan > 1) {
                                        for (var i = column; i < column + jsonObj[column].RowSpan; i++) {
                                            for (var j = i + _proxy.rowCount; j < i + (jsonObj[column].ColSpan * _proxy.rowCount) ; j = j + _proxy.rowCount)
                                                jsonObj[j].Span = "Block";
                                            jsonObj[i].Span = "Block";
                                        }
                                    }
                                }
                            }
                            tRow += "</tr>";
                            $(tbodyOlapGrid).append(tRow);
                        }

                    }
                });
                if (_proxy.model.progressMode != "infinite") {
                    window.clearInterval(oGridTimer);
                    this._progressStatus(85, 90, 80);
                    window.setTimeout(function () { progressStatus() }, 400);
                    function progressStatus() {
                        $(tableOlapGrid).append(theadOlapGrid);
                        $(tableOlapGrid).append(tbodyOlapGrid);
                    }
                }
                else {
                    $(tableOlapGrid).append(theadOlapGrid);
                    $(tableOlapGrid).append(tbodyOlapGrid);
                }
            }
            if (_proxy.model.progressMode != "infinite") {
                window.clearInterval(oGridTimer);
                this._progressStatus(90, 101 + ":100%", 80);
                window.setTimeout(function () { timeOut() }, 900);
                function timeOut() {
                    _proxy.element.html("");
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._createVirtualOlapGrid(tableOlapGrid);
                    }
                    else
                        this.element.append(tableOlapGrid);
                    _proxy._unWireEvents();
                    _proxy._wireEvents();
                    $("#" + _proxy._id + "progressContainer").hide();
                }
            }
            else {
                this.element.html("");
                if (_proxy.model.enableVirtualScrolling) {
                    _proxy._createVirtualOlapGrid(tableOlapGrid);
                }
                else
                    this.element.append(tableOlapGrid);
                this._unWireEvents();
                this._wireEvents();
            }
        },

        /**
         * 	This function appends virtual scroll bar to the OLAP Grid.
         *
         * @private  
         */
        _createVirtualOlapGrid: function (tableOlapGrid) {
            var oGridDiv = ej.buildTag("div.oGridOuterDiv", tableOlapGrid[0].outerHTML);
            $(oGridDiv).css("display", "inline-block");
            var verticalScroll = _proxy._seriesPageCount > 1 ? ej.buildTag("div.vScrollPanel")[0].outerHTML : "";
            var horizontalScroll = _proxy._categPageCount > 1 ? ej.buildTag("div.hScrollPanel")[0].outerHTML : "";
            var virtualOlapGrid = ej.buildTag("div", oGridDiv[0].outerHTML + verticalScroll + horizontalScroll);
            this.element.html(virtualOlapGrid[0].innerHTML);
            var seriesPageIndicator = ej.buildTag("div.seriesPageIndicator inActive", ej.buildTag("span.axislabel", this._getLocalizedLabels("SeriesPage"))[0].outerHTML + ej.buildTag("span.series_CurrentPage", _proxy._seriesCurrentPage)[0].outerHTML + " / " + ej.buildTag("span.series_pageCount", _proxy._seriesPageCount)[0].outerHTML)[0].outerHTML;
            if (_proxy._seriesPageCount > 1) _proxy.element.append(seriesPageIndicator);
            var categPageIndicator = ej.buildTag("div.categPageIndicator inActive", ej.buildTag("span.axislabel", this._getLocalizedLabels("CategoricalPage"))[0].outerHTML + ej.buildTag("span.categ_CurrentPage", _proxy._categCurrentPage)[0].outerHTML + " / " + ej.buildTag("span.categ_pageCount", _proxy._categPageCount)[0].outerHTML)[0].outerHTML;
            if (_proxy._categPageCount > 1) _proxy.element.append(categPageIndicator);
        }
    });

    /**
     * Enum for Layout Types. Allows the user access the different layouts for OLAP Grid. 
     *
     * @enum {String}
	 * @global
     */
    ej.OlapGrid.gridLayout = {
        Normal: "Normal",
        NormalTopSummary: "NormalTopSummary",
        NoSummaries: "NoSummaries",
        ExcelLikeLayout: "ExcelLikeLayout"
    };

    //OLAP Grid Localization
    ej.OlapGrid.localization = {};
    //Default english localized values
    ej.OlapGrid.localization["en-US"] = {
        SeriesPage: "Series Page",
        CategoricalPage: "Categorical Page",
        MDXqueryExecutionFailed: "MDX query execution failed",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingStarted: "Rendering Started",
        RenderingSucceeded: "Rendering Succeeded",
        RenderingFailed: "Rendering failed"
    };

    /**
     * Enum for Grid Action. Contains the information about current action of the OLAP Grid. 
     *
     * @enum {String}
	 * @global
     */
    ej.OlapGrid.gridAction = {
        Initialize: "initialize",
        DrillDown: "drillDown"
    };

    /**
     * Enum for Progress Mode. Allows the user to select a appropriate progress indicator. 
     *
     * @enum {String}
	 * @global
     */
    ej.OlapGrid.progressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to experience Html OLAP Client component elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejOlapClient
	* @requires jQuery
	* @requires ej.common.all
    * @requires ej.tools
    * @requires ej.olapgrid
    * @requires ej.chart
    * @requires ej.olapchart
	* @classdesc Custom Design for Html OLAP Client control.
	* @example 
    * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create OLAP Client
    * $("#OlapClient").ejOlapClient(...); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejOlapClient", "ej.OlapClient", {

        //Root Css Class
        _rootCSS: "e-olapclient",

        //Widget element will be automatically set in this
        element: null,

        //User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        //Default model values
        defaults: {
            /**		
			* Connects the service using the specified URL for any server updates.
			* @default null
			* @type {string}
			* @example 
			* //To set size API value during initialization
            * $("#OlapClient").ejOlapClient({ url: "/wcf/OlapClientService.svc" });
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#OlapClient").ejOlapClient("option","url");<br>			
			*	//Sets the size value 
			*	$("#OlapClient").ejOlapClient("option","url", "/wcf/OlapClientService.svc" ); 
			 * @memberof ejOlapClient
			* @instance
			*/
            url: "",

            /**		
			* Sets the CSS name for custom operation.
			* @default null
			* @type {string}
			* @example 
			* //To set size API value during initialization
            * $("#OlapClient").ejOlapClient({ cssClass: "Olive" });
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#OlapClient").ejOlapClient("option","cssClass");<br>			
			*	//Sets the size value 
			*	$("#OlapClient").ejOlapClient("option","cssClass", "Olive" ); 
			* @memberof ejOlapClient
			* @instance
			*/
            cssClass: "",

            /**		
			* Sets the title for OLAP Client.
			* @default null
			* @type {string}
			* @example 
			* //To set size API value during initialization
            * $("#OlapClient").ejOlapClient({ title: "Olap Browser" });
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#OlapClient").ejOlapClient("option","title");<br>			
			*	//Sets the size value 
			*	$("#OlapClient").ejOlapClient("option","title", "Olap Browser" ); 
			* @memberof ejOlapClient
			* @instance
			*/
            title: "",

            /**		
			* Sets the layout for OLAP Grid component inside OLAP Client.
			* @default ej.OlapGrid.gridLayout.Normal
			* @type {enum}
			* @example 
			* //To set size API value during initialization
            * $("#OlapClient").ejOlapClient({ gridLayout: ej.OlapGrid.gridLayout.NoSummaries });
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#OlapClient").ejOlapClient("option","gridLayout");<br>			
			*	//Sets the size value 
			*	$("#OlapClient").ejOlapClient("option","gridLayout", ej.OlapGrid.gridLayout.NormalTopSummary); 
			* @memberof ejOlapClient
			* @instance
			*/
            gridLayout: "Normal",

            displayOptions: {
                /**		
			    * Sets the display mode (Only Chart/Only Grid/Both Chart And Grid) of OLAP Client.
			    * @default ej.OlapClient.displayMode.ChartAndGrid
			    * @type {enum}
			    * @example 
			    * //To set size API value during initialization
                * $("#OlapClient").ejOlapClient({ displayOptions: { mode: ej.OlapClient.displayMode.ChartOnly }});
			    * @example 
			    * //Get or set the size API, after initialization:<br>
			    *	//Gets the size value  
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.mode");<br>			
			    *	//Sets the size value 
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.mode", ej.OlapClient.displayMode.ChartOnly ); 
			    * @memberof ejOlapClient
			    * @instance
			    */
                mode: "chartAndGrid",

                /**		
			    * Lets the user to set Chart or Grid tab as default.
			    * @default ej.OlapClient.defaultView.Grid
			    * @type {enum}
			    * @example 
			    * //To set size API value during initialization
                * $("#OlapClient").ejOlapClient({ displayOptions: {defaultView: ej.OlapClient.defaultView.Grid} });
			    * @example 
			    * //Get or set the size API, after initialization:<br>
			    *	//Gets the size value  
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.defaultView");<br>			
			    *	//Sets the size value 
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.defaultView", ej.OlapClient.defaultView.Chart ); 
			    * @memberof ejOlapClient
			    * @instance
			    */
                defaultView: "grid",

                /**		
			    * Lets the user to customize the display of OLAP Chart and OLAP Grid controls inside the OLAP Client component.
			    * @default ej.OlapClient.controlPlacement.Tab
			    * @type {enum}
			    * @example 
			    * //To set size API value during initialization
                * $("#OlapClient").ejOlapClient({ displayOptions: {controlPlacement: ej.OlapClient.controlPlacement.Tab} });
			    * @example 
			    * //Get or set the size API, after initialization:<br>
			    *	//Gets the size value  
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.controlPlacement");<br>			
			    *	//Sets the size value 
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.controlPlacement", ej.OlapClient.controlPlacement.Tile ); 
			    * @memberof ejOlapClient
			    * @instance
			    */
                controlPlacement: "tab",

                /**		
			    * Sets the Toggle Panel visibility mode.
			    * @default false
			    * @type {boolean}
			    * @example 
			    * //To set size API value during initialization
                * $("#OlapClient").ejOlapClient({ displayOptions.togglePanel: true });
			    * @example 
			    * //Get or set the size API, after initialization:<br>
			    *	//Gets the size value  
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.togglePanel");<br>			
			    *	//Sets the size value 
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.togglePanel", true ); 
			    * @memberof ejOlapClient
			    * @instance
			    */
                togglePanel: false,

                /**		
			    * Enables/ disables the full screen view of the OLAP components(OLAP Chart, OLAP Grid) in OLAP Client.
			    * @default false
			    * @type {boolean}
			    * @example 
			    * //To set size API value during initialization
                * $("#OlapClient").ejOlapClient({ displayOptions.fullScreenView: true });
			    * @example 
			    * //Get or set the size API, after initialization:<br>
			    *	//Gets the size value  
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.fullScreenView");<br>			
			    *	//Sets the size value 
			    *	$("#OlapClient").ejOlapClient("option","displayOptions.fullScreenView", true ); 
			    * @memberof ejOlapClient
			    * @instance
			    */
                fullScreenView: false
            },

            /**		
			* Allows the user to set custom name for the methods at service-end invoked on AJAX post.
            * @default null
            * @type {object}
            * @example 
            * //To set serviceMethods API value during initialization  
            *  $("#OlapClient").ejOlapClient({  serviceMethods: {initialize: "MyMethod"});     * 
            * @example 
            * //Get or set the serviceMethods API, after initialization:<br>
            * //Gets the serviceMethods value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethods");<br>   
            * //Sets the serviceMethods value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethods",  {initialize: "MyMethod"} ); 
            * @memberof ejOlapClient
            * @instance
			*/
            serviceMethods: {
                initialize: "InitializeOlapClient", removeSplitButton: "RemoveSplitButton", filterElement: "FilterElement", toolbarSelection: "ToolbarSelection",
                nodeDropped: "NodeDropped", fetchMemberTreeNodes: "FetchMemberTreeNodes", cubeChanged: "CubeChanged", toolbarServices: "ToolbarOperations",
                memberExpand: "MemberExpanded", saveReport: "SaveReportToDB", fetchReportList: "FetchReportListFromDB", loadReport: "LoadReportFromDB"
            },

            /**		
			* Custom object to pass additional information between client-end and service-end.
			* @default {}
			* @type {Object}
			* @example 
			* //To set size API value during initialization
            * $("#OlapClient").ejOlapClient({ customObject: {"MyObject": "Hi Syncfusion!!"} });
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#OlapClient").ejOlapClient("option","customObject");<br>			
			*	//Sets the size value 
			*	$("#OlapClient").ejOlapClient("option","customObject", {"MyObject": "Hello World!!"} ); 
			* @memberof ejOlapClient
			* @instance
			*/
            customObject: {},

            /**		
			* Sets the progress indicator type which arises on any UI operation.
			* @default ej.OlapChart.progressMode.Infinite
			* @type {enum}
			* @example 
			* //To set size API value during initialization
            * $("#OlapClient").ejOlapClient({ progressMode: ej.OlapChart.progressMode.Normal });
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#OlapClient").ejOlapClient("option","progressMode");<br>			
			*	//Sets the size value 
			*	$("#OlapClient").ejOlapClient("option","progressMode", ej.OlapChart.progressMode.Progress ); 
			* @memberof ejOlapClient
			* @instance
			*/
            progressMode: "infinite",

            /**		
			* Sets the localized language for the control.
			* @default "en-US"
			* @type {string}
			* @example 
			* //To set size API value during initialization
            * $("#OlapClient").ejOlapClient({ localization: "en-US" });
			* @example 
			* //Get or set the size API, after initialization:<br>
			*	//Gets the size value  
			*	$("#OlapClient").ejOlapClient("option","localization");<br>			
			*	//Sets the size value 
			*	$("#OlapClient").ejOlapClient("option","localization", "fr-FR" ); 
			* @memberof ejOlapClient
			* @instance
			*/
            localization: "en-US",

            //events

            /**     
			 * Fires when the control is rendered success
			 * @event
			 * @name ejOlapClient#clientSuccess 	
			 * @param {Object} argument Event parameters from OLAP Client component.
			 * @param {string}  argument.element returns the outer HTML of OLAP Client component.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Client model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //success event for rendering the control
             * $("#OlapClient").ejOlapClient({
             *    clientSuccess: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            clientSuccess: null,

            /**     
			 * Fires when error occured in rendering the control
			 * @event
			 * @name ejOlapClient#clientError 	
			 * @param {Object} argument Event parameters from OLAP Client component
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Client model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //event triggered when error occurs in rendering the control
             * $("#OlapClient").ejOlapClient({
             *    clientError: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            clientError: null,

            /**     
			 * Fires on completion of rendering control
			 * @event
			 * @name ejOlapClient#clientComplete 	
			 * @param {Object} argument Event parameters from OLAP Client component
			 * @param {string}  argument.element returns the outer HTML of OLAP Client component.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Client model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //event triggered after completing rendering the control
             * $("#OlapClient").ejOlapClient({
             *    clientComplete: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            clientComplete: null,

            /**     
			 * Fires on loading the control
			 * @event
			 * @name ejOlapClient#load 	
			 * @param {Object} argument Event parameters from OLAP Client component.
			 * @param {string}  argument.element returns the outer HTML of OLAP Client component.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Client model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //event triggered on loading the control
             * $("#OlapClient").ejOlapClient({
             *    load: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            load: null,

            /**     
			 * Fires before rendering the chart
			 * @event
			 * @name ejOlapClient#chartPreRender 	
			 * @param {Object} argument Event parameters from OLAP Chart component.
			 * @param {string}  argument.Action return the current action of OLAP Chart control.		 
			 * @param {object}  argument.CustomObject return the custom object bounds with OLAP Chart control.
			 * @param {string}  argument.Element return the outer HTML of OLAP Chart control.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Chart model.
			 * @param {string}  argument.type returns the name of the event.	
			 * @example 
			 * //event triggered when OLAP Chart begins rendering.
             * $("#OlapClient").ejOlapClient({
             *    chartPreRender: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            chartPreRender: "",

            /**    
            * Fires when the summary cell is clicked.
            * @event
            * @name ejOlapClient#beforeServiceInvoke 
			* @param {Object} argument Event parameters from OLAP Client component.
			* @param {string}  argument.Action return the current action of OLAP Client control.		 
			* @param {object}  argument.CustomObject return the custom object bounds with OLAP Client control.
			* @param {string}  argument.Element return the outer HTML of OLAP Client control.
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the OLAP Client model.
			* @param {string}  argument.type returns the name of the event.
            * @example 
            * //beforeServiceInvoke event
            * $("#OlapClient").ejOlapClient({
			* 		beforeServiceInvoke: function (args) {}
			* });
            * @memberof ejOlapClient
            * @instance
            */
            beforeServiceInvoke: null,

            /**    
            * Fires after the service is invoked.
            * @event
            * @name ejOlapClient#afterServiceInvoke 
			* @param {Object} argument Event parameters from OLAP Client component.
			* @param {string}  argument.Action return the current action of OLAP Client control.		 
			* @param {object}  argument.CustomObject return the custom object bounds with OLAP Client control.
			* @param {string}  argument.Element return the outer HTML of OLAP Client control.
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the OLAP Client model.
			* @param {string}  argument.type returns the name of the event.
            * @example 
            * //afterServiceInvoke event
            * $("#OlapClient").ejOlapClient({
			*		afterServiceInvoke: function (args) {}
			* });
            * @memberof ejOlapClient
            * @instance
            */
            afterServiceInvoke: null
        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            displayOptions: "data",
            serviceMethods: "data",
            customObject: "data"
        },

        observables: ["title", "gridLayout", "displayOptions.mode", "displayOptions.defaultView", "displayOptions.controlPlacement", "displayOptions.togglePanel", "localization"],
        title: ej.util.valueFunction("title"),
        gridLayout: ej.util.valueFunction("gridLayout"),
        displayMode: ej.util.valueFunction("displayOptions.mode"),
        defaultView: ej.util.valueFunction("displayOptions.defaultView"),
        controlPlacement: ej.util.valueFunction("displayOptions.controlPlacement"),
        togglePanel: ej.util.valueFunction("displayOptions.togglePanel"),
        localization: ej.util.valueFunction("localization"),

        /**
         * Initializes the variables and loads the widget. 
         *
         * @private
         */
        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        /**
         * Initializes the local variables. 
         *
         * @private
         */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            oclientProxy = this;
            currentReport = "";
            currentCubeName = "";
            reports = "";
            reportsCount = 0;
            ogridObj = null;
            ochartObj = null;
            chartObj = null;
            this._keypress = false;
            memberTreeObj = null;
            this._chartHeight = 0;
            this._chartWidth = 0;
            this._gridHeight = 0;
            this._gridWidth = 0;
            this._initStyles = new Array();
            this._toggleStyles = new Array();
            this._initToggle = true;
            this._dimensionName = "";
            this._dllSortMeasure = null;
            this._axis = "";
            this._isSorted = false;
            this._isFiltered = false;
            this._sortOrFilterTab = '';
            this.pNode = "";
            this.progressPos = null;
            draggedSplitBtn = null;
            isDragging = false;
            oclientWaitingPopup = null;
            currentTab = this.defaultView();
        },

        /**
         * Destroys the widget while it's no longer required. 
         *
         * @private
         */
        _destroy: function () {
            this.element.empty().removeClass("e-olapclient" + this.model.cssClass);
        },

        /**
         * This function is the point where loading/rendering of the OLAP Client widget is initiated.
         *
         * @private  
         */
        _load: function () {
            this.element.addClass(this.model.cssClass);
            if (this.model.progressMode != "infinite") {
                $("#" + oclientProxy._id + "progressContainer").remove();
                this._createProgressBar();
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutStarted");
                this._progressStatus(5, 70, 140);
            }
            else {
                $("#" + this._id).ejWaitingPopup({ autoDisplay: true });
                oclientWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
            }

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { Action: "initializeClient", Element: this.element, CustomObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({
                "action": "initializeClient", "customObject": serializedCustomObject
            }), this._renderControlSuccess);
        },

        _setFirst: true,

        /**
         * This function is invoked on any change in the widgets property in-order to assign the altered value.
         *
         * @private 
         */
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "title": this.element.find(".titleText").text(this.title()); break;
                    case "currentCubeName": this.currentCubeName = options[key]; break;
                    case "gridLayout": this._renderOlapGrid(); break;
                    case "olapReport": this.currentReport = options[key]; break;
                    case "clientReports": this.reports = options[key]; break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "localization": { this.localization(); if (this.ogridObj || this.ochartObj) this._load(); break; }
                    case "displayOptions": { if (this.ogridObj || this.ochartObj) this._load(); break; }
                }
            }
        },

        /**
         * This function is used to get the list of measure elements added in the current report.
         *
         * @private  
         */
        _getMeasuresList: function () {
            var measureName = "";
            this.element.find(".memberEditorDiv").find("div").each(function () { measureName += $(this)[0].id + ","; });
            return measureName;
        },

        /**
         * This function is used to get the unchecked nodes from the tree view inside member editor.
         *
         * @private  
         */
        _getUnSelectedNodes: function () {
            var treeElement = this.element.find(".editorTreeView")[0];
            var unselectedNodes = "";
            var data = $(treeElement).find(":input.nodecheckbox:not(:checked)");
            for (var i = 0; i < data.length; i++) {
                var parentNode = $(data[i]).parents('li:eq(0)');
                unselectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
            }
            return unselectedNodes;
        },

        /**
         * This function is used to get the checked nodes from the tree view inside member editor.
         *
         * @private  
         */
        _getSelectedNodes: function () {
            var treeElement = this.element.find(".editorTreeView")[0];
            var selectedNodes = "";
            var data = $(treeElement).find("input[type=checkbox]:checked");
            for (var i = 0; i < data.length; i++) {
                var parentNode = $(data[i]).parents('li:eq(0)');
                selectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
            }
            return selectedNodes;
        },

        /**
         * This function is used to remove a Split Button.
         *
         * @private  
         */
        _removeSplitBtn: function () {
            var dragBtn = $(document).find(".dragClone"); var btnTag;
            var clientBtns = this.element.find(".splitBtn");
            jQuery.each(clientBtns, function (index, value) {
                if ($($(value).children()[0]).text() == $(dragBtn).text()) {
                    btnTag = $(value).attr("tag");
                    $(value).remove();
                }
            });
            $(".dragClone").remove();
            if (this.model.progressMode != "infinite") {
                oclientProxy._progressStatus(0, 80, 50);
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
            }
            else
                oclientWaitingPopup.show();
            if (oclientProxy.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { Action: "removeSplitButton", Element: this.element, CustomObject: oclientProxy.model.customObject });
            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.removeSplitButton, JSON.stringify({ "action": "removeSplitButton", "clientParams": btnTag, "olapReport": currentReport, "clientReports": reports, "customObject": serializedCustomObject }), oclientProxy._removeSplitButtonSuccess)
        },

        /**
         * This function is invoked to wire the events for UI interaction.
         *
         * @private  
         */
        _wireEvents: function () {
            oclientProxy._wireDialogEvent();
            oclientProxy._wireEditorRemoveEvent();
            oclientProxy._wireMeasureRemoveEvent();
            this._on($(document), 'keydown', this._KeyPress);
            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejDraggable({
                handle: 'button',
                clone: true,
                cursorAt: { top: -10, left: -10 },
                dragStart: function (event, ui) {
                    event.event.preventDefault();
                    $(this.element.find(".e-txt")).unbind('touchstart');
                    $(this.element.find(".e-txt")).unbind(ej.eventType.click);
                    isDragging = true;
                    this.element.find(".e-dialog").hide();
                    draggedSplitBtn = event.event.target;
                },
                dragStop: function (event, ui) {
                    this.element.find(".targetAxis").removeClass("targetAxis");
                    isDragging = false;
                    var cssName = null; var droppedPosition;
                    if (event.event.type == "touchend")
                        droppedPosition = oclientProxy.setSplitBtnTargetPos(event);
                    else
                        droppedPosition = oclientProxy.setSplitBtnTargetPos(event.event);
                    if (oclientProxy._dropAxisClassName != undefined && oclientProxy._dropAxisClassName != "") {
                        if (oclientProxy._dropAxisClassName == "outOfAxis")
                            oclientProxy._removeSplitBtn();
                        else
                            cssName = oclientProxy._dropAxisClassName;
                        oclientProxy._dropAxisClassName = "";
                    }
                    else if (event.target != undefined) {
                        if ($(event.target).hasClass("e-btn") || $(event.target).hasClass("removeSplitBtn"))
                            cssName = $(event.target).parents("div:eq(1)").attr("class");
                        else if (jQuery.type(event.target.className) != "string")
                            oclientProxy._removeSplitBtn();
                        else {
                            if (event.target.className.indexOf("splitBtn") > -1)
                                cssName = $(event.target).parents("div:eq(0)").attr("class");
                            else
                                cssName = event.target.className;
                        }
                    }
                    else {
                        if ($(event.event.originalEvent.srcElement).hasClass("e-btn") || $(event.event.originalEvent.srcElement).hasClass("removeSplitBtn"))
                            cssName = $(event.event.originalEvent.srcElement).parents("div:eq(1)").attr("class");
                        else if (event.event.originalEvent.srcElement.className.indexOf("splitBtn") > -1)
                            cssName = $(event.event.originalEvent.srcElement).parents("div:eq(0)").attr("class");
                        else
                            cssName = event.event.originalEvent.srcElement.className;
                    }
                    if (cssName != undefined && cssName != null) {
                        var axisName = (cssName.indexOf("categoricalAxis") > cssName.indexOf("rowAxis")) ? ((cssName.indexOf("categoricalAxis") > cssName.indexOf("slicerAxis")) ? "Categorical" : "Slicer") : ((cssName.indexOf("rowAxis") > cssName.indexOf("slicerAxis")) ? "Series" : (cssName.indexOf("slicerAxis") >= 0) ? "Slicer" : null);
                        if (axisName != null)
                            oclientProxy._splitButtonDropped(axisName, droppedPosition);
                        else
                            oclientProxy._removeSplitBtn();
                    }
                },
                helper: function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(oclientProxy.element);
                    else
                        return false;
                }
            });

            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").mouseover(function () {
                if (isDragging)
                    $(this).addClass("targetAxis");
            });
            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").mouseleave(function () {
                $(this).removeClass("targetAxis");
            });

            this.element.find(".e-btn").mouseover(function (evt) {
                $(evt.target.parentNode).find("span").css("display", "inline");
            });

            this.element.find(".splitBtn").mouseover(function (evt) {
                if (isDragging)
                    $(this).addClass("dropIndicator");
            });
            this.element.find(".splitBtn").mouseleave(function (evt) {
                $(this).find("span").css("display", "none");
                $(this).removeClass("dropIndicator");
            });
            this.element.find(".sortDisable, .sortEnable, .filterDisable, .filterEnable ").bind(ej.eventType.click, function (evt) {
                if (evt.target.className == "sortDisable") {
                    $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').attr("disabled", "disabled");
                    oclientProxy._dllSortMeasure.disable();
                }
                else if (evt.target.className == "sortEnable") {
                    $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').removeAttr("disabled");
                    oclientProxy._dllSortMeasure.enable();
                }
                else if (evt.target.className == "filterDisable") {
                    $('.filterFrom, .filterTo').attr("disabled", "disabled");
                    oclientProxy._dllFilterCondition.disable();
                    oclientProxy._dllfMeasuresList.disable();
                }
                else if (evt.target.className == "filterEnable") {
                    $('.filterFrom').removeAttr("disabled");
                    oclientProxy._dllFilterCondition.enable();
                    oclientProxy._dllfMeasuresList.enable();
                }
            });
            this.element.find(".filterFrom , .filterTo").keypress(function (event) {
                if (event.which == 8 || event.which == 0) {
                    return true;
                }

                if (event.which < 46 || event.which > 58) {
                    return false;
                }
                if ((event.which == 46 && $(this).val().indexOf('.') != -1) || event.which == 47) {
                    return false;
                }
            });
            this.element.find(".toggleExpandButton").click(function (evt) {
                if (oclientProxy._initToggle) {
                    oclientProxy._initToggle = false; var items = {};
                    var tempWidth = oclientProxy.element.find(".cubeTable").width() + oclientProxy.element.find(".controlPanel").width() - 17
                    items["controlPanelWidth"] = tempWidth;
                    items["chartOuterWidth"] = items["chartModelWidth"] = items["gridOuterWidth"] = tempWidth - 17;
                    oclientProxy._toggleStyles.push(items);
                }
                oclientProxy._performToggleAction(oclientProxy._toggleStyles);
                oclientProxy.element.find(".csHeader, .cubeTable").hide();
                oclientProxy.element.find(".toggleCollapseButton").show();
                oclientProxy.element.find(".toggleText").show();

            });
            this.element.find(".toggleCollapseButton").click(function (evt) {
                oclientProxy._performToggleAction(oclientProxy._initStyles);
                oclientProxy.element.find(".csHeader, .cubeTable").show();
                oclientProxy.element.find(".toggleCollapseButton").hide();
                oclientProxy.element.find(".toggleText").hide();
            });
            this.element.find(".maximizedView").click(function (evt) {
                oclientProxy._maxViewBtnClick();
            });
            $(document).on('click', '.winCloseBtn', function () {
                oclientProxy._maxViewClsBtnClick();
            });
        },

        /**
         * This function is used to perform UI interactions on pressing the keyboard keys.
         *
         * @private
         */
        _KeyPress: function (e) {
            if (e.keyCode == 9) {
                if ($(".e-dialog").length > 0) {
                    $(".clientDialog").focus();
                    $(".reportName").addClass("e-focus");
                }
            }
            if ($(".e-dialog").length > 0 && e.keyCode == 13) {
                this.element.find(".dialogOKBtn").trigger("click");
                oclientProxy._off(this.element, "click", ".dialogOKBtn");
            }
            else if ($(".e-dialog").length > 0 && e.keyCode == 27) {
                this.element.find(".dialogCancelBtn").trigger("click");
                oclientProxy._off(this.element, "click", ".dialogCancelBtn");
            }
            else if ($(".e-dialog").length == 0 && e.keyCode == 27 && $(".fullScreenView").length != 0) {
                this._maxViewClsBtnClick();
            }
            else if (e.keyCode == 9 && $(".borderFilterSortDlg").length > 0) {

            }
            else if (e.keyCode == 13) {
                if (document.activeElement.className.indexOf("newReportImg") > -1) {
                    this.element.find(".newReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("addReportImg") > -1) {
                    this.element.find(".addReportImg ").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("removeReportImg") > -1) {
                    this.element.find(".removeReportImg ").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("renameReportImg") > -1) {
                    this.element.find(".renameReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("saveReportImg") > -1) {
                    this.element.find(".saveReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("loadReportImg") > -1) {
                    this.element.find(".loadReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("excelExportImg") > -1) {
                    this.element.find(".excelExportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("maximizedView") > -1) {
                    this.element.find(".maximizedView").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("colSortFilterImg") > -1) {
                    this.element.find(".colSortFilterImg ").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("rowSortFilterImg") > -1) {
                    this.element.find(".rowSortFilterImg ").trigger("click").off("click");
                }
            }
        },

        /**
         * This function is invoked to wire the events for removing a measure.
         *
         * @private  
         */
        _wireMeasureRemoveEvent: function () {
            this.element.find(".removeMeasure").click(function (evt) {
                $($(evt.target).parent()).remove();
            });
        },

        /**
         * This function is invoked to wire the events for removing a Split Button.
         *
         * @private  
         */
        _wireEditorRemoveEvent: function () {
            this.element.find(".removeSplitBtn").click(function (evt) {
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressStatus(0, 80, 50);
                else
                    oclientWaitingPopup.show();
                $($(evt.target).parent()).remove();
                oclientProxy._off(this.element, "click", ".removeSplitBtn");
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                if (oclientProxy.model.beforeServiceInvoke != null)
                    oclientProxy._trigger("beforeServiceInvoke", { Action: "removeSplitButton", Element: this.element, CustomObject: oclientProxy.model.customObject });
                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.removeSplitButton, JSON.stringify({ "action": "removeSplitButton", "clientParams": $(evt.target).parent().attr("tag"), "olapReport": currentReport, "clientReports": reports, "customObject": serializedCustomObject }), oclientProxy._removeSplitButtonSuccess);
            });
        },

        /**
         * This function is invoked to wire the toolbar dialog events.
         *
         * @private
         */
        _wireDialogEvent: function () {
            oclientProxy.element.find(".newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .saveReportImg, .loadReportImg, .e-txt,.colSortFilterImg, .rowSortFilterImg").bind(ej.eventType.click, function (evt) {
                if (!isDragging) {
                    evt.preventDefault();
                    if (evt.target.innerHTML != oclientProxy._getLocalizedLabels("Cancel") && evt.target.innerHTML != oclientProxy._getLocalizedLabels("OK")) {
                        oclientProxy._off(oclientProxy.element, "click", ".newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .saveReportImg, .loadReportImg, .e-txt,.colSortFilterImg, .rowSortFilterImg");
                        if (evt.target.className.indexOf("loadReportImg") >= 0) {
                            if (oclientProxy.model.progressMode != "infinite") {
                                if (oclientProxy.model.progressMode == "progress")
                                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialog") + "...";
                                oclientProxy._progressStatus(0, 80, 50);
                            }
                            else
                                oclientWaitingPopup.show();
                            if (oclientProxy.model.beforeServiceInvoke != null)
                                oclientProxy._trigger("beforeServiceInvoke", { Action: "FetchingReportList", Element: oclientProxy.element, CustomObject: oclientProxy.model.customObject });
                            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.fetchReportList, JSON.stringify({ "customObject": serializedCustomObject }), oclientProxy._fetchReportListSuccess);
                        }
                        else if (evt.target.className.indexOf("colSortFilterImg") >= 0 || evt.target.className.indexOf("rowSortFilterImg") >= 0) {
                            if (evt.target.className.indexOf("SortImg") >= 0)
                                oclientProxy._sortOrFilterTab = "sort";
                            else
                                oclientProxy._sortOrFilterTab = "filter";
                            if (oclientProxy.model.progressMode != "infinite") {
                                if (oclientProxy.model.progressMode == "progress")
                                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialog") + "...";
                                oclientProxy._progressStatus(0, 80, 50);
                            }
                            else
                                oclientWaitingPopup.show();
                            oclientProxy._isSorted = false;
                            oclientProxy._isFiltered = false;
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.toolbarServices, JSON.stringify({ "action": "FetchSortState", "toolbarOperation": null, "clientInfo": (oclientProxy._axis = (evt.target.className.indexOf("colSortFilterImg") >= 0) ? "Column" : "Row"), "olapReport": currentReport, "clientReports": "" }), oclientProxy._fetchSortState);
                        }
                        else
                            oclientProxy._createDialogRequest(evt);
                        oclientProxy._dimensionName = $(evt.currentTarget).parent().attr("tag");
                    }
                }
            });

            this.element.find(".excelExportImg").click(function (evt) {
                if (oclientProxy.ogridObj)
                    oclientProxy.ogridObj.exportToExcel();
            });

            this.element.find(".dialogCancelBtn").bind(ej.eventType.click, function (e) {
                e.preventDefault();
                oclientProxy.element.find(".e-dialog").hide();
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
            });

            this.element.find(".dialogOKBtn").bind(ej.eventType.click, function (e) {
                e.preventDefault();
                if (oclientProxy.element.find(".e-titlebar").first().text() == oclientProxy._getLocalizedLabels("LoadReport")) {
                    var selectedReport = oclientProxy.element.find(".reportNameList")[0].value;
                    if (selectedReport != "") {
                        if (oclientProxy.model.progressMode != "infinite") {
                            if (oclientProxy.model.progressMode == "progress")
                                $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                            oclientProxy._progressStatus(0, 80, 50);
                        }
                        else
                            oclientWaitingPopup.show();
                        if (oclientProxy.model.beforeServiceInvoke != null)
                            oclientProxy._trigger("beforeServiceInvoke", { Action: "loadReport", Element: this.element, CustomObject: oclientProxy.model.customObject });
                        var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.loadReport, JSON.stringify({
                            "reportName": selectedReport, "olapReport": currentReport, "clientReports": reports, "customObject": serializedCustomObject
                        }), oclientProxy._toolbarOperationSuccess);
                    }
                    else {
                        alert("Please select a valid report");
                        return false;
                    }
                }

                else if (oclientProxy.element.find(".sortingDlg").length > 0) {
                    if (oclientProxy.element.find(".sortEnable")[0].checked == true && oclientProxy.element.find(".measuresList")[0].value == "") {
                        alert("Please select a valid measure.");
                        return false;
                    }
                    else if (oclientProxy.element.find(".filterEnable")[0].checked == true) {
                        if (oclientProxy._axis == "Row" && $(".rowAxis ")[0].childElementCount == 0) {
                            alert("Element not found in row axis. Please add an element in row axis for filtering."); return false;
                        }
                        if (oclientProxy._axis == "Column" && $(".categoricalAxis")[0].childElementCount == 0) {
                            alert("Element not found in column axis. Please add an element in column axis for filtering."); return false;
                        }
                        if (oclientProxy.element.find(".fMeasuresList")[0].value == "") {
                            alert("Please select a valid measure."); return false;
                        }
                        else if (oclientProxy.element.find(".filterCondition")[0].value == "") {
                            alert("Please set a valid condition."); return false;
                        }
                        else if (oclientProxy.element.find(".filterFrom")[0].value == "") {
                            alert("Please set a start value."); return false;
                        }
                        else if ((oclientProxy.element.find(".filterCondition")[0].value == "Between" || oclientProxy.element.find(".filterCondition")[0].value == "NotBetween") && oclientProxy.element.find(".filterTo")[0].value == "") {
                            alert("Please set a end value."); return false;
                        }

                    }
                    var sortingDetails = null, filteringDetails = null;
                    if ((oclientProxy._isSorted == false || oclientProxy._isSorted == true) && oclientProxy.element.find(".sortEnable")[0].checked == true)
                        sortingDetails = $("li[tag*='" + oclientProxy.element.find(".measuresList")[0].value + "']").attr("tag") + "::" + ((oclientProxy.element.find(".radioBtnAsc")[0].checked == true) ? "ASC" : "DESC") + "::" + oclientProxy._axis + "::" + ((oclientProxy.element.find(".preserveHrchy")[0].checked == true) ? "PHT" : "PHF");
                    else if (oclientProxy._isSorted == true && oclientProxy.element.find(".sortDisable")[0].checked == true)
                        sortingDetails = "Disable Sorting" + "::" + " " + "::" + oclientProxy._axis + "::" + " ";
                    else
                        sortingDetails = " " + "::" + " " + "::" + oclientProxy._axis + "::" + " ";
                    if ((oclientProxy._isFiltered == false || oclientProxy._isFiltered == true) && oclientProxy.element.find(".filterEnable")[0].checked == true)
                        filteringDetails = $("li[tag*='" + oclientProxy.element.find(".fMeasuresList")[0].value + "']").attr("tag") + "::" + oclientProxy.element.find(".filterCondition")[0].value + "::" + oclientProxy.element.find(".filterFrom")[0].value + "::" + oclientProxy.element.find(".filterTo")[0].value;
                    else if (oclientProxy._isFiltered == true && oclientProxy.element.find(".filterDisable")[0].checked == true)
                        filteringDetails = "Disable Filtering";
                    else
                        filteringDetails = "";
                    oclientProxy._SortFilterDetails = sortingDetails + "||" + filteringDetails;

                    if (oclientProxy.model.progressMode != "infinite") {
                        if (oclientProxy.model.progressMode == "progress")
                            $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                        oclientProxy._progressStatus(0, 80, 50);
                    }
                    else
                        oclientWaitingPopup.show();
                    if (oclientProxy.element.find(".sortEnable")[0].checked == true || oclientProxy.element.find(".filterEnable")[0].checked == true) {
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "SortOrFilter", "clientInfo": oclientProxy._SortFilterDetails, "olapReport": currentReport, "clientReports": reports }), oclientProxy._toolbarOperationSuccess);
                    }
                    else if (oclientProxy.element.find(".sortDisable")[0].checked == true || oclientProxy.element.find(".filterDisable")[0].checked == true) {
                        if (oclientProxy._isSorted == true || oclientProxy._isFiltered == true) {
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "SortOrFilter", "clientInfo": oclientProxy._SortFilterDetails, "olapReport": currentReport, "clientReports": reports }), oclientProxy._toolbarOperationSuccess);
                            oclientProxy._isSorted = false;
                        }
                        else {
                            alert("Invalid operation !");
                            if (oclientProxy.model.progressMode == "progress")
                                oclientProxy._progressHide();
                            else
                                oclientWaitingPopup.hide();
                            return false;
                        }
                    }

                }
                else {
                    if (oclientProxy._getSelectedNodes() == 0 && args_innerHTML != oclientProxy._getLocalizedLabels("Measures") && args_innerHTML != "ToolbarButtons")
                        return false;
                    var unselectedNodes = null;
                    if (args_innerHTML != "ToolbarButtons") {
                        if (oclientProxy.model.progressMode != "infinite")
                            oclientProxy._progressStatus(0, 80, 50);
                        else
                            oclientWaitingPopup.show();
                        if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures"))
                            unselectedNodes = "Measures:" + oclientProxy._getMeasuresList();
                        else if (args_innerHTML != "ToolbarButtons" && args_innerHTML != undefined)
                            unselectedNodes = oclientProxy._getUnSelectedNodes() + "CHECKED" + oclientProxy._getSelectedNodes();
                        if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures") && unselectedNodes != null && unselectedNodes.split(":")[1] == "") {
                            $(oclientProxy.element).find(".splitBtn").each(function (index, value) {
                                if (value.firstChild.innerHTML == oclientProxy._getLocalizedLabels("Measures"))
                                    $(value).remove();
                            });
                        }
                        $(oclientProxy).find(".e-dialog").hide();
                        if (oclientProxy.model.progressMode == "progress")
                            $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                        if (oclientProxy.model.beforeServiceInvoke != null)
                            oclientProxy._trigger("beforeServiceInvoke", { "Action": "filtering", Element: this.element, CustomObject: oclientProxy.model.customObject });
                        var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.filterElement, JSON.stringify({ "action": "filtering", "clientParams": unselectedNodes, "olapReport": currentReport, "clientReports": reports, "customObject": serializedCustomObject }), oclientProxy._filterElementSuccess);
                    }
                    else {
                        if (oclientProxy.element.find(".e-titlebar").first().text() == oclientProxy._getLocalizedLabels("RemoveReport") && reportsCount < 2)
                            oclientProxy.element.find(".e-dialog").hide();
                        else if (oclientProxy.element.find(".reportName").val() == "") {
                            return false;
                        }
                        else {
                            var reportName = oclientProxy.element.find(".reportName").val() || reportDropTarget.selectedTextValue;
                            var operation = oclientProxy.element.find(".e-titlebar").first().attr("tag");
                            var olapReport = (operation == "New Report" ? "" : currentReport);
                            var clientReports = (operation == "New Report" ? "" : reports);
                            oclientProxy.element.find(".e-dialog").hide();
                            if (oclientProxy.model.progressMode == "progress")
                                $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialog") + "...";
                            if (operation == "Save Report") {
                                if (oclientProxy.model.beforeServiceInvoke != null)
                                    oclientProxy._trigger("beforeServiceInvoke", { Action: "saveReport", Element: this.element, CustomObject: oclientProxy.model.customObject });
                                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.saveReport, JSON.stringify({
                                    "reportName": reportName, "olapReport": olapReport, "clientReports": clientReports, "customObject": serializedCustomObject
                                }), oclientProxy._toolbarOperationSuccess);

                            } else
                                if (oclientProxy.model.beforeServiceInvoke != null)
                                    oclientProxy._trigger("beforeServiceInvoke", { Action: "toolbarOperation", Element: this.element, CustomObject: oclientProxy.model.customObject });
                            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.toolbarServices, JSON.stringify({
                                "action": "toolbarOperation", "toolbarOperation": operation, "clientInfo": reportName, "olapReport": olapReport, "clientReports": clientReports, "customObject": serializedCustomObject
                            }), oclientProxy._toolbarOperationSuccess);
                        }
                    }
                }
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
            });
            this.element.find(".measureEditor").mouseover(function (evt) {
                $(evt.target).find("span").css("display", "inline");
            });
            this.element.find(".measureEditor").mouseleave(function (evt) {
                $(this).find("span").css("display", "none");
            });
            this.element.find(".unCheckAll,.checkAll").click(function (evt) {
                if (evt.target.className.indexOf("checkAll") > -1)
                    oclientProxy.element.find(".editorTreeView").ejTreeView("checkAll");
                else
                    oclientProxy.element.find(".editorTreeView").ejTreeView("unCheckAll");
            });
        },

        /**
         * This function is used to extract the appropriate localized text.
         *
         * @private  
         */
        _getLocalizedLabels: function (property) {
            return ej.OlapClient.localization[this.localization()][property] === undefined ? ej.OlapClient.localization["en-US"][property] : ej.OlapClient.localization[this.localization()][property];
        },

        /**
         * This function is invoked to unwire the events registered for UI interaction.
         *
         * @private  
         */
        _unWireEvents: function () {
            $(this.element.find(".dialogCancelBtn, .dialogOKBtn, .newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .e-txt, .removeSplitBtn, .unCheckAll, .checkAll, .removeMeasure, .toggleCollapseButton, .toggleExpandButton, .saveReportImg, .loadReportImg,.maximizedView,.colSortFilterImg, .rowSortFilterImg")).unbind(ej.eventType.click);
            $(this.element.find(".dialogCancelBtn, .dialogOKBtn, .newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .e-txt, .removeSplitBtn, .unCheckAll, .checkAll, .removeMeasure, .toggleCollapseButton, .toggleExpandButton, .saveReportImg, .loadReportImg,.maximizedView,.colSortFilterImg, .rowSortFilterImg")).unbind('click');
            $(document).find(".winCloseBtn").unbind(ej.eventType.click);
            $(document).find(".winCloseBtn").unbind('click');
            this._off($(document), 'keydown', this._KeyPress);
        },

        /**
         * This function is used to create progress bar containing status text and percentage.
         *
         * @private  
         */
        _createProgressBar: function () {
            var progressContainer = ej.buildTag("div#" + oclientProxy._id + "progressContainer.clientProgressContainer", "", { top: ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 30), left: ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 50) })[0].outerHTML;
            var blockControl = ej.buildTag("div#" + oclientProxy._id + "blockDiv.blockDiv", "", { width: $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: $("#" + this._id).height(), position: 'fixed', top: ($("#" + this._id).position().top), left: ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + oclientProxy._id).after(progressContainer);
            var progressBar = ej.buildTag("div#" + oclientProxy._id + "progressBar.clientProgressBar", "", {})[0].outerHTML;
            if (oclientProxy.model.progressMode == "normal") {
                $("#" + oclientProxy._id + "progressContainer").css({ width: 0, height: 0 });
                $("#" + oclientProxy._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + oclientProxy._id + "progressText.clientProgressText", "Initializing...", {})[0].outerHTML;
                $("#" + oclientProxy._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + oclientProxy._id + "progressBar").ejProgressBar({ value: 10, text: 10 + " %", complete: "progressbarCompleted" });
            oclientProgressBar = $("#" + oclientProxy._id + "progressBar").data("ejProgressBar");
        },

        /**
         * This function is used to update the progress status.
         *
         * @private  
         */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            oclientProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + oclientProxy._id + "progressContainer").show();
            oClientTimer = window.setInterval(function () {
                oclientProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(oClientTimer);
                    if (progressText != undefined) {
                        oclientProgressBar.setModel({ value: 100, text: progressText[1] });
                    }

                };
            }, progress_Speed);

        },

        /**
         * This function is used to hide the progress bar.
         *
         * @private  
         */
        _progressHide: function () {
            if (oclientProxy.model.progressMode != "infinite") {
                window.clearInterval(oClientTimer);
                $("#" + oclientProxy._id + "progressContainer").hide();
            }
            else
                oclientWaitingPopup.hide();
        },

        /**
         * This function is used to perform appropriate action (visibility) while toggling panel.
         *
         * @private  
         */
        _performToggleAction: function (styles) {
            this.element.find(".controlPanel").width(styles[0].controlPanelWidth);
            if ((this.controlPlacement() != "horizontal") || (this.controlPlacement() == "horizontal" && this.displayMode() == "chartOnly") ||
                this.controlPlacement() == "horizontal" && this.displayMode() == "gridOnly") {
                this.element.find(".e-olapchart").width(styles[0].chartOuterWidth);
                this.element.find(".e-olapgrid").width(styles[0].gridOuterWidth);
                if (this.ochartObj != null || this.ochartObj != undefined) {
                    this.element.find("#" + this.ochartObj._id).ejOlapChart("option", "width", styles[0].chartOuterWidth);
                    this.chartObj = this.element.find("#" + this.ochartObj._id + "Container").data("ejChart");
                    if (this.chartObj) {
                        this.element.find("#" + this.chartObj._id).ejChart("option", { "model": { size: { width: styles[0].chartOuterWidth } } });
                        this.element.find("#" + this.chartObj._id + "_svg").width(styles[0].chartOuterWidth);
                        this.chartObj.redraw();
                    }
                }
            }
        },

        /**
         * This function is used to update data in OLAP controls while dragging and dropping the Split Button.
         *
         * @private  
         */
        _splitButtonDropped: function (axis, droppedPosition) {
            if (draggedSplitBtn != null) {
                var params = this.element.find(".cubeName").html() + "--" + $(draggedSplitBtn).parent("div:eq(0)").attr("Tag") + "--" + axis + "--" + droppedPosition;
                draggedSplitBtn = null;
                if (oclientProxy.model.progressMode != "infinite") {
                    if (oclientProxy.model.progressMode == "progress")
                        $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                    this._progressStatus(0, 80, 100);
                }
                else
                    oclientWaitingPopup.show();
                if (oclientProxy.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { Action: "nodeDropped", Element: this.element, CustomObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.nodeDropped, JSON.stringify({ "action": "nodeDropped", "dropType": "SplitButton", "nodeInfo": params, "olapReport": currentReport, "clientReports": reports, "customObject": serializedCustomObject }), oclientProxy._nodeDroppedSuccess);
            }
        },

        /**
         * This function receives the controls' update from service-end which would be utilized after the filtering action.
         *
         * @private  
         */
        _filterElementSuccess: function (report) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
                window.clearInterval(oClientTimer);
            }
            if (report[0] != undefined) {
                currentReport = report[0].Value; reports = report[1].Value;
                if (report[2] != null && report[2] != undefined)
                    oclientProxy.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                currentReport = report.d[0].Value; reports = report.d[1].Value;
                if (report.d[2] != null && report.d[2] != undefined)
                    oclientProxy.model.customObject = report.d[2].Value;
            }
            else {
                currentReport = report.UpdatedReport; reports = report.ClientReports;
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "filtering", Element: this.element, CustomObject: this.model.customObject });
            if (this.model.progressMode != "infinite") {
                this._progressStatus(90, 101 + ":100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                if (oclientProxy.model.progressMode != "infinite") {
                    window.clearInterval(oClientTimer);
                    if (oclientProxy.model.progressMode == "progress")
                        $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded");
                    oclientProxy._progressHide();
                }
                oclientProxy._renderControls();
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
            }
            oclientProxy._successAction = "Filter";
            oclientProxy._trigger("clientSuccess", oclientProxy);
        },

        /**
         * This function is used to switch the control(s) to maximized view.
         *
         * @private  
         */
        _maxViewBtnClick: function () {
            var winWidth = $(window).width() - 150;
            var winHeight = $(window).height() - 100;
            var docDivTag = ej.buildTag("div.fullScreenView", "", { width: $(document).width(), height: $(document).height() });
            var winDivTag = ej.buildTag("div#" + oclientProxy._id + "_maxView.maximumView", "", { width: winWidth, height: winHeight });
            var maxViewCls = ej.buildTag("div#Winclose.winCloseBtn e-icon", "");
            if (oclientProxy.displayMode() == "chartOnly" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                oclientProxy._fullScreenView(100, 160);
                winDivTag.append(oclientProxy.element.find("#OlapChart"));
            }
            if (oclientProxy.displayMode() == "gridOnly")
                winDivTag.append(oclientProxy.element.find("#OlapGrid").css({ "max-height": winHeight, "width": winWidth }));
            if (oclientProxy.controlPlacement() == ej.OlapClient.controlPlacement.Tab && oclientProxy.displayMode() == "chartAndGrid") {
                if (currentTab == "grid")
                    winDivTag.append(oclientProxy.element.find("#OlapGrid").css({ "max-height": winHeight, "width": winWidth }));
                else if (currentTab == "chart" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                    oclientProxy._fullScreenView(100, 160);
                    winDivTag.append(oclientProxy.element.find("#OlapChart"));
                }
            }
            else if (oclientProxy.controlPlacement() == ej.OlapClient.controlPlacement.Tile && oclientProxy.displayMode() == "chartAndGrid") {
                var hght = ($(window).height() / 2) - 50;
                oclientProxy.element.find("#OlapGrid").css({ "max-height": hght, "margin-left": "0px" });
                oclientProxy._fullScreenView(50, 160);
                if (($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]))
                    if (oclientProxy.defaultView() == "chart")
                        winDivTag.append(oclientProxy.element.find("#OlapChart")).append(oclientProxy.element.find("#OlapGrid").width(($(window).width() - 160)));
                    else
                        winDivTag.append(oclientProxy.element.find("#OlapGrid").width($(window).width() - 160)).append(oclientProxy.element.find("#OlapChart"));
                winDivTag.append(oclientProxy.element.find("#OlapGrid").width($(window).width() - 160));
            }
            winDivTag.append(maxViewCls);
            docDivTag.append(winDivTag);
            $("body").append(docDivTag);
        },

        /**
         * This function is used to restore the control(s) from maximized view.
         *
         * @private  
         */
        _maxViewClsBtnClick: function () {
            var fullScreen = $(".fullScreenView");
            fullScreen.find("#OlapGrid").css("margin-left", "7px");
            fullScreen.find("#OlapChart").appendTo(oclientProxy.element.find("#chartPanel"));
            if (oclientProxy.displayMode() == "chartOnly" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]))
                oclientProxy._fullScreenViewCls(560, 790);
            if (oclientProxy.displayMode() == "gridOnly")
                fullScreen.find("#OlapGrid").css({ "width": "inherit", "max-height": '' });
            if (oclientProxy.controlPlacement() == ej.OlapClient.controlPlacement.Tab && oclientProxy.displayMode() == "chartAndGrid") {
                if (currentTab == "chart" && $("#" + oclientProxy.ochartObj._id + "Container_svg")[0])
                    oclientProxy._fullScreenViewCls(547, 557);
                else
                    fullScreen.find("#OlapGrid").css({ "max-height": "530px", "width": "555px" });
            }
            if (oclientProxy.controlPlacement() == ej.OlapClient.controlPlacement.Tile && oclientProxy.displayMode() == "chartAndGrid")
                if ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]) {
                    oclientProxy._fullScreenViewCls(275, 555);
                    fullScreen.find("#OlapGrid").css({ "max-height": "300px", "width": "555px" });
                } else
                    fullScreen.find("#OlapGrid").css({ "max-height": "275px", "width": "555px" });
            if (oclientProxy.togglePanel() && (oclientProxy.element.find(".cubeTable").is(':visible')) == false)
                fullScreen.find("#OlapGrid").css("width", "950px");
            fullScreen.find("#OlapGrid").appendTo(oclientProxy.element.find("#gridPanel"));
            $(".maximumView").remove();
            $(".fullScreenView").remove();
        },

        /**
         * This function changes the default view to full screen view. It displays only the active control in full screen mode.
         *
         * @private  
         */
        _fullScreenView: function (height, width) {
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
                var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = $(window).width() - width;
                if (oclientProxy.controlPlacement() == ej.OlapClient.controlPlacement.Tile && oclientProxy.displayMode() == "chartAndGrid")
                    var chartHeight = oclientProxy.ochartObj.model.size.height = oclientProxy.chartObj.model.size.height = ($(window).height() / 2) - height;
                else
                    var chartHeight = oclientProxy.ochartObj.model.size.height = oclientProxy.chartObj.model.size.height = $(window).height() - height;
                oclientProxy.element.find("#OlapChart").css({ "min-height": chartHeight, "width": chartWidth });
                oclientProxy.chartObj.redraw();
            }
        },

        /**
         * This function resets the control to default view from maximized view.
         *
         * @private  
         */
        _fullScreenViewCls: function (height, width) {
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined) {
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
                var chartHeight = oclientProxy.chartObj.model.size.height = oclientProxy.ochartObj.model.size.height = height;
                if (oclientProxy.togglePanel() && (oclientProxy.element.find(".cubeTable").is(':visible')) == false)
                    var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = 950;
                else
                    var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = width;
                oclientProxy.element.find("#OlapChart").css({ "min-height": chartHeight, "width": chartWidth });
                oclientProxy.chartObj.redraw();
            }
        },

        /**
         * This function receives the control's update from service-end which would be utilized after dropping a treenode to Axis Element Builder.
         *
         * @private  
         */
        _nodeDroppedSuccess: function (data) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
                window.clearInterval(oClientTimer);
                oclientProgressBar.setModel({ value: 85, text: 85 + " %" });
            }

            var columnElements, rowElements, slicerElements;
            if (data[0] != undefined) {
                columnElements = data[0].Value; rowElements = data[1].Value; slicerElements = data[2].Value;
                currentReport = data[3].Value; reports = data[4].Value;
                if (data[5] != null && data[5] != undefined)
                    oclientProxy.model.customObject = data[5].Value;
            }
            else if (data.d != undefined) {
                columnElements = data.d[0].Value; rowElements = data.d[1].Value; slicerElements = data.d[2].Value;
                currentReport = data.d[3].Value; reports = data.d[4].Value;
                if (data.d[5] != null && data.d[5] != undefined)
                    oclientProxy.model.customObject = data.d[5].Value;
            }
            else {
                columnElements = data.Columns; rowElements = data.Rows; slicerElements = data.Slicers;
                currentReport = data.UpdatedReport; reports = data.ClientReports;
                if (data.customObject != null && data.customObject != undefined)
                    oclientProxy.model.customObject = data.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "nodeDropped", Element: this.element, CustomObject: this.model.customObject });
            if (oclientProxy.model.progressMode != "infinite")
                oclientProgressBar.setModel({ value: 90, text: 90 + " %" });
            this.element.find(".splitBtn, .e-btn").remove();
            $(oclientProxy._createSplitButtons(columnElements, "Columns")).appendTo(".categoricalAxis");
            $(oclientProxy._createSplitButtons(rowElements, "Rows")).appendTo(".rowAxis");
            $(oclientProxy._createSplitButtons(slicerElements, "Slicers")).appendTo(".slicerAxis");
            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
            if (oclientProxy.model.progressMode != "infinite") {
                this._progressStatus(95, 101 + "::100%", 50);
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded");
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                oclientProxy._renderControls();
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite") {
                    window.clearInterval(oClientTimer);
                    oclientProxy._progressHide();
                }
            }
            oclientProxy._successAction = "NodeDrop";
            oclientProxy._trigger("clientSuccess", oclientProxy);
        },

        /**
         * This function receives the control's update from service-end after getting the members to populate the tree inside member editor.
         *
         * @private  
         */
        _editorTreeInfoSuccess: function (editorTree) {
            if (editorTree[0] != undefined) {
                if (editorTree[2] != null && editorTree[2] != undefined)
                    oclientProxy.model.customObject = editorTree[2].Value;
            }
            else if (editorTree.d != undefined) {
                if (editorTree.d[2] != null && editorTree.d[2] != undefined)
                    oclientProxy.model.customObject = editorTree.d[2].Value;
            }
            else {
                if (editorTree.customObject != null && editorTree.customObject != undefined)
                    oclientProxy.model.customObject = editorTree.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "fetchMemberTreeNodes", Element: this.element, CustomObject: this.model.customObject });
            if (oclientProxy.model.progressMode != "infinite") {
                window.clearInterval(oClientTimer);
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialogSucceeded") + "...";
                this._progressStatus(95, 101 + "::100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                oclientProxy._createDialog(args_className, args_innerHTML, editorTree);
                if (oclientProxy.model.progressMode != "infinite") {
                    window.clearInterval(oClientTimer);
                    oclientProxy._progressHide();
                }
                else
                    oclientWaitingPopup.hide();
            }
            oclientProxy._successAction = "EditorTreeInfo";
            oclientProxy._trigger("clientSuccess", oclientProxy);
        },

        /**
         * 	This function receives the control's update from service-end which would be utilized for rendering the widget.
         *
         * @private  
         */
        _renderControlSuccess: function (msg) {
            oclientProxy._trigger("load", oclientProxy);
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutCompleted") + "...";
                window.clearInterval(oClientTimer);
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingStarted") + "...";
                oclientProgressBar.setModel({ value: 80, text: "80%" });
            }
            var cubes, columnElements, rowElements, slicerElements, cubeTreeInfo;
            if (msg[0] != undefined) {
                cubes = msg[0].Value; columnElements = msg[1].Value; rowElements = msg[2].Value;
                slicerElements = msg[3].Value; cubeTreeInfo = msg[4].Value; currentReport = msg[5].Value;
                reports = msg[6].Value; reportsCount = msg[7].Value; reportList = $.parseJSON(msg[8].Value);
                if (msg[10] != null && msg[10] != undefined)
                    oclientProxy.model.customObject = msg[10].Value;
            }
            else if (msg.d != undefined) {
                cubes = msg.d[0].Value; columnElements = msg.d[1].Value; rowElements = msg.d[2].Value;
                slicerElements = msg.d[3].Value; cubeTreeInfo = msg.d[4].Value; currentReport = msg.d[5].Value;
                reports = msg.d[6].Value; reportsCount = msg.d[7].Value; reportList = $.parseJSON(msg.d[8].Value);
                if (msg.d[9] != null && msg.d[9] != undefined)
                    oclientProxy.model.customObject = msg.d[9].Value;
            }
            else {
                cubes = msg.Cubes; columnElements = msg.Columns; rowElements = msg.Rows;
                slicerElements = msg.Slicers; cubeTreeInfo = msg.CubeTreeInfo; currentReport = msg.CurrentReport;
                reports = msg.ClientReports; reportList = $.parseJSON(msg.ReportList); reportsCount = msg.ClientReportsCount;
                if (msg.customObject != null && msg.customObject != undefined)
                    oclientProxy.model.customObject = msg.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "initialize", Element: this.element, CustomObject: this.model.customObject });

            if (oclientProxy.model.progressMode != "infinite") {
                this._progressStatus(89, 101 + ":100%", 10);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutStarted") + "...";
                var treeViewData = $.parseJSON(cubeTreeInfo);
                oclientProxy.currentCubeName = $.parseJSON(cubes)[0].name;
                var reportBar = ej.buildTag("div#reportToolbar.reportToolbar", oclientProxy._reportToolbar(), { width: "410px", height: "29px" })[0].outerHTML;
                var browserPanel = oclientProxy._createCubeSelector() + "<table class=cubeTable><tr><td valign=\"bottom\">" + oclientProxy._createCubeBrowser(cubeTreeInfo) + "</td><td valign=\"bottom\" style='padding-left:5px;'>" + oclientProxy._createAxisElementBuilder(columnElements, rowElements, slicerElements) + "</td></tr></table>";
                var htmlTag = ej.buildTag("span.outerPanel", "", { width: "1000px", height: "600px" }).append("<div class=\"titleText\"><span style='padding-left:10px'>" + oclientProxy.title() + "</span></div>" + reportBar + "<table class=\"outerTable\"><tr><td>" + browserPanel + "</td><td class=\"controlPanelTD\">" + oclientProxy._controlPanel() + "</td></tr></table>");
                oclientProxy.element.html(htmlTag).width($(".outerPanel").width());
                oclientProxy.element.find(".reportToolbar").ejToolbar();
                var reportDropDown = "<div class ='reportList' ><input type='text' id='reportList' class='reportlist' /></div>";
                $(reportDropDown).appendTo(".reportToolbar");
                oclientProxy.element.find(".reportlist").ejDropDownList({
                    dataSource: reportList,
                    fields: { text: "name", value: "name" },
                    height: "20px"
                });
                oclientProxy.element.find(".reportlist").attr("tabindex", 0);
                var tempddlwidth = oclientProxy.element.find(".csHeader").width() - oclientProxy.element.find(".cubeText").width() - oclientProxy.element.find(".toggleExpandButton").width() - 20;
                var cubeddlWidth = oclientProxy.togglePanel() ? tempddlwidth - 25 : tempddlwidth;
                oclientProxy.element.find(".cubeSelector").ejDropDownList({
                    dataSource: $.parseJSON(cubes),
                    fields: { text: "name", value: "name" },
                    width: "" + cubeddlWidth + "px"
                });
                ddlTarget = oclientProxy.element.find('.cubeSelector').data("ejDropDownList"); reportDropTarget = oclientProxy.element.find('#reportList').data("ejDropDownList");
                ddlTarget.selectItemByIndex(0); reportDropTarget.selectItemByIndex(0);
                oclientProxy.element.find(".cubeSelector").ejDropDownList("option", "change", ej.proxy(oclientProxy.CubeChanged, oclientProxy));
                oclientProxy.element.find(".cubeSelector").attr("tabindex", 0);
                oclientProxy.element.find(".reportlist").ejDropDownList("option", "change", ej.proxy(oclientProxy.ReportChanged, oclientProxy));
                oclientProxy.element.find(".cubeTreeView").ejTreeView({
                    fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                    dragAndDrop: true,
                    dropChild: false,
                    dropSibling: false,
                    dragAndDropAcrossControl: true,
                    dragStart: function () {
                        isDragging = true;
                    },
                    dropped: ej.proxy(oclientProxy.NodeDropped, oclientProxy),
                    height: $(".cubeBrowser").height()
                });
                oclientProxy.element.find(".cubeTreeView").attr("tabindex", 0);
                var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
                for (var i = 0; i < treeViewElements.length; i++) {
                    treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
                }
                var cubeName = ej.buildTag("div.cubeName", $.parseJSON(cubes)[0].name)[0].outerHTML;
                $(cubeName).prependTo(".cubeTreeView");
                oclientProxy.element.find(".categoricalAxis, .rowAxis, .slicerAxis").ejDroppable({ drop: ej.proxy(oclientProxy.onDropped, oclientProxy) });
                oclientProxy.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
                if (oclientProxy.togglePanel()) {
                    $(ej.buildTag("div.toggleExpandButton e-icon")[0].outerHTML).appendTo(oclientProxy.element.find(".csHeader"));
                    $(oclientProxy.element.find(".outerTable").find("td")[0]).append(ej.buildTag("div.toggleCollapseButton e-icon")[0].outerHTML);
                    $((ej.buildTag("div.toggleText")[0].outerHTML)).insertAfter(oclientProxy.element.find(".toggleCollapseButton"));
                    oclientProxy.element.find(".toggleCollapseButton").hide();
                    oclientProxy.element.find(".toggleText").hide();
                }
                oclientProxy.element.find("#clientTab").ejTab({ active: ej.proxy(oclientProxy.OnTabClick, oclientProxy) });
                if (oclientProxy.defaultView() == ej.OlapClient.defaultView.Chart) {
                    oclientProxy.element.find("#OlapChart").ejOlapChart({ url: oclientProxy.model.url, customObject: currentReport, localization: oclientProxy.localization(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: ej.OlapChart.chartTypes.Column, tooltip: { visible: true } }, beforeServiceInvoke: oclientProxy.model.chartPreRender, drillSuccess: ej.proxy(oclientProxy.ChartDrillSuccess, oclientProxy) });
                    if (oclientProxy.gridLayout() != ej.OlapGrid.gridLayout.Normal)
                        oclientProxy.element.find("#OlapGrid").ejOlapGrid({ url: oclientProxy.model.url, olapReport: currentReport, gridLayout: oclientProxy.gridLayout(), drillSuccess: ej.proxy(oclientProxy.GridDrillSuccess, oclientProxy) });
                    else
                        oclientProxy.element.find("#OlapGrid").ejOlapGrid({ url: oclientProxy.model.url, olapReport: currentReport, drillSuccess: ej.proxy(oclientProxy.GridDrillSuccess, oclientProxy) });
                }
                else {
                    if (oclientProxy.gridLayout() != ej.OlapGrid.gridLayout.Normal)
                        oclientProxy.element.find("#OlapGrid").ejOlapGrid({ url: oclientProxy.model.url, olapReport: currentReport, gridLayout: oclientProxy.gridLayout(), drillSuccess: ej.proxy(oclientProxy.GridDrillSuccess, oclientProxy) });
                    else
                        oclientProxy.element.find("#OlapGrid").ejOlapGrid({ url: oclientProxy.model.url, olapReport: currentReport, drillSuccess: ej.proxy(oclientProxy.GridDrillSuccess, oclientProxy) });
                    oclientProxy.element.find("#OlapChart").ejOlapChart({ url: oclientProxy.model.url, customObject: currentReport, localization: oclientProxy.localization(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: ej.OlapChart.chartTypes.Column, tooltip: { visible: true } }, drillSuccess: ej.proxy(oclientProxy.ChartDrillSuccess, oclientProxy), beforeServiceInvoke: oclientProxy.model.chartPreRender });
                }
                if (oclientProxy.displayMode() != "chartOnly")
                    oclientProxy.ogridObj = oclientProxy.element.find('#OlapGrid').data("ejOlapGrid");
                if (oclientProxy.displayMode() != "gridOnly")
                    oclientProxy.ochartObj = oclientProxy.element.find('#OlapChart').data("ejOlapChart");
                var items = {};
                if (oclientProxy.ochartObj != null) {
                    oclientProxy.element.find('#OlapChart').width(oclientProxy.ochartObj.model.size.width);
                    items["chartModelWidth"] = oclientProxy.ochartObj.model.size.width;
                }
                items["controlPanelWidth"] = oclientProxy.element.find(".controlPanel").width();
                items["chartOuterWidth"] = oclientProxy._chartWidth;
                items["gridOuterWidth"] = oclientProxy._gridWidth;
                oclientProxy._initStyles.push(items);
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite") {
                    oclientProxy._progressStatus(100, 101, 10);
                    oclientProxy._progressHide();
                    $("#" + oclientProxy._id + "progressContainer").css({ top: ($("#" + oclientProxy._id).height() / 2) - 30, left: (($("#" + oclientProxy._id).width() / 2)), position: 'absolute' });
                    $("#" + oclientProxy._id + "blockDiv").css({ height: $("#" + oclientProxy._id).height(), width: $("#" + oclientProxy._id).width() });
                }
            }
            if (oclientProxy.model.progressMode == "progress")
                $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded") + "...";
            oclientProxy._successAction = "ClientRender";
            oclientProxy._trigger("clientSuccess", oclientProxy);
            oclientProxy.progressPos = $("#" + oclientProxy._id).position();
        },

        /**
         * 	This function receives the control's update from service-end after changing the cube.
         *
         * @private  
         */
        _cubeChangedSuccess: function (report) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutCompleted") + "...";
                window.clearInterval(oClientTimer);
                oclientProgressBar.setModel({ value: 83, text: 83 + " %" });
            }
            var cubeTreeInfo;
            if (oclientProxy.model.progressMode == "progress")
                $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingStarted") + "...";
            if (report[0] != undefined) {
                currentReport = report[0].Value; cubeTreeInfo = report[1].Value; reports = report[2].Value;
                reportsCount = report[3].Value; reportList = $.parseJSON(report[4].Value);
                if (report[5] != null && report[5] != undefined)
                    oclientProxy.model.customObject = report[5].Value;
            }
            else if (report.d != undefined) {
                currentReport = report.d[0].Value; cubeTreeInfo = report.d[1].Value; reports = report.d[2].Value;
                reportsCount = report.d[3].Value; reportList = $.parseJSON(report.d[4].Value);
                if (report.d[5] != null && report.d[5] != undefined)
                    oclientProxy.model.customObject = report.d[5].Value;
            }
            else {
                currentReport = report.NewReport; cubeTreeInfo = report.CubeTreeInfo; reports = report.ClientReports;
                reportsCount = report.ReportsCount; reportList = $.parseJSON(report.ReportList);
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "cubeChanged", Element: this.element, CustomObject: this.model.customObject });
            this.element.find(".splitBtn, .e-olapgrid, .e-olapchart, .e-treeview, .cubeTreeView").remove();
            var cubeTree = ej.buildTag("div#cubeTreeView.cubeTreeView")[0].outerHTML;
            var cubeName = ej.buildTag("div.cubeName", this.currentCubeName)[0].outerHTML;
            var treeViewData = $.parseJSON(cubeTreeInfo);
            $(cubeTree).appendTo(".cubeBrowser");
            this.element.find(".cubeTreeView").ejTreeView({
                fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                dragAndDrop: true,
                dropChild: false,
                dropSibling: false,
                dragAndDropAcrossControl: true,
                dropped: ej.proxy(this.NodeDropped, this),
                height: $(".cubeBrowser").height()
            });
            var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            this.element.find(".reportlist").ejDropDownList("option", "change", "");
            this.element.find(".reportlist").ejDropDownList("option", "dataSource", reportList);
            reportDropTarget.selectItemByIndex(reportList.length - 1);
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded") + "...";
                this._progressStatus(90, 101 + ":100%", 20);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                $(cubeName).prependTo(".cubeTreeView");
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressHide();
                else
                    oclientWaitingPopup.hide();
            }
            oclientProxy._successAction = "CubeChange";
            oclientProxy._trigger("clientSuccess", oclientProxy);
        },

        /**
         * 	This function receives the control's update from service-end after removing Split Button.
         *
         * @private  
         */
        _removeSplitButtonSuccess: function (report) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
                window.clearInterval(oClientTimer);
                oclientProgressBar.setModel({ value: 89, text: 89 + " %" });
            }
            if (report[0] != undefined) {
                currentReport = report[0].Value; reports = report[1].Value;
                if (reports[2] != null && reports[2] != undefined)
                    oclientProxy.model.customObject = reports[2].Value;
            }
            else if (report.d != undefined) {
                currentReport = report.d[0].Value; reports = report.d[1].Value;
                if (report.d[2] != null && report.d[2] != undefined)
                    oclientProxy.model.customObject = report.d[2].Value;
            }
            else {
                currentReport = report.UpdatedReport; reports = report.ClientReports;
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "removeSplitButton", Element: this.element, CustomObject: this.model.customObject });
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded");
                this._progressStatus(95, 101 + ":100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                oclientProxy._renderControls();
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressHide();
            }
            oclientProxy._successAction = "ButtonRemove";
            oclientProxy._trigger("clientSuccess", oclientProxy);
        },

        /**
         * 	This function receives the control's update from service-end which would be utilized after performing any toolbar operation.
         *
         * @private  
         */
        _toolbarOperationSuccess: function (report) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialogSucceeded") + "...";
                window.clearInterval(oClientTimer);
            }
            if (oclientProxy.model.progressMode == "progress")
                $("#" + oclientProxy._id + "progressText")[0].innerHTML = "Rendering started";
            if (report.d || report[0] || report.CurrentReport) {
                var columnElements, rowElements, slicerElements, action, renamedReport;
                if (report.length > 1 && report[0] != undefined) {
                    currentReport = report[0].Value; reports = report[1].Value; reportsCount = report[2].Value; columnElements = report[3].Value;
                    rowElements = report[4].Value; slicerElements = report[5].Value; action = report[6].Value; reportList = $.parseJSON(report[7].Value);
                    renamedReport = report[8].Value;
                    if (report[9] != null && report[9] != undefined)
                        oclientProxy.model.customObject = report[9].Value;
                }
                else if (report.d != undefined) {
                    currentReport = report.d[0].Value; reports = report.d[1].Value; reportsCount = report.d[2].Value; columnElements = report.d[3].Value;
                    rowElements = report.d[4].Value; slicerElements = report.d[5].Value; action = report.d[6].Value; reportList = $.parseJSON(report.d[7].Value);
                    renamedReport = report.d[8].Value;
                    if (report.d[9] != null && report.d[9] != undefined)
                        oclientProxy.model.customObject = report.d[9].Value;
                }
                else {
                    currentReport = report.CurrentReport; reports = report.Reports; reportsCount = report.ReportsCount; columnElements = report.Columns;
                    rowElements = report.Rows; slicerElements = report.Slicers; action = report.CurrentAction; reportList = $.parseJSON(report.ReportList);
                    renamedReport = report.RenamedReport;
                    if (report.customObject != null && report.customObject != undefined)
                        oclientProxy.model.customObject = report.customObject;
                }
                if (this.model.afterServiceInvoke != null)
                    this._trigger("afterServiceInvoke", { Action: "toolbarOperation", Element: this.element, CustomObject: this.model.customObject });
                if (action != "Report Change") {
                    this.element.find(".reportlist").ejDropDownList("option", "change", "");
                    this.element.find(".reportlist").ejDropDownList("option", "dataSource", reportList);
                    if (action == "Load Report")
                        reportDropTarget.selectItemByIndex(0);
                    else if (action != "Rename Report")
                        reportDropTarget.selectItemByIndex(reportList.length - 1);
                    else
                        reportDropTarget.selectItemByText(renamedReport);
                    this.element.find(".reportlist").ejDropDownList("option", "change", ej.proxy(this.ReportChanged, this));
                }
                if (action != "Rename Report")
                    this.element.find(".splitBtn, .e-olapgrid, .e-olapchart").remove();
                if (action == "Remove Report" || action == "Report Change" || action == "Load Report" || action == "SortOrFilter" || action == "Save Report") {
                    $(oclientProxy._createSplitButtons(columnElements, "Columns")).appendTo(".categoricalAxis");
                    $(oclientProxy._createSplitButtons(rowElements, "Rows")).appendTo(".rowAxis");
                    $(oclientProxy._createSplitButtons(slicerElements, "Slicers")).appendTo(".slicerAxis");
                    this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
                    if (oclientProxy.model.progressMode == "progress")
                        $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded");
                    oclientProxy._renderControls();
                    if (oclientProxy.model.progressMode != "infinite")
                        oclientProxy._progressHide();
                }
            }
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            if (oclientProxy.model.progressMode != "infinite")
                oclientProxy._progressHide();
            oclientProxy._successAction = "ToolbarOperation";
            oclientProxy._trigger("clientSuccess", oclientProxy);
        },

        /**
         * 	This function receives the control's update from service-end after fetching the child nodes.
         *
         * @private  
         */
        _fetchChildNodeSuccess: function (data) {
            var newDataSource; var parentNode;
            if (data.length > 1 && data[0] != undefined)
                newDataSource = JSON.parse(data[0].Value);
            else if (data.d != undefined)
                newDataSource = JSON.parse(data.d[0].Value);
            else
                newDataSource = JSON.parse(data.ChildNodes);
            if (data[0] != undefined) {
                if (data[2] != null && data[2] != undefined)
                    oclientProxy.model.customObject = data[2].Value;
            }
            else if (data.d != undefined) {
                if (data.d[2] != null && data.d[2] != undefined)
                    oclientProxy.model.customObject = data.d[2].Value;
            }
            else {
                if (data.customObject != null && data.customObject != undefined)
                    oclientProxy.model.customObject = data.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "memberExpanded", Element: this.element, CustomObject: this.model.customObject });
            var mapper = { id: "id", parentId: "pid", hasChild: "hasChildren", text: "name", isChecked: "checkedStatus" };
            if ($(this.pNode).parents("li").length > 1)
                parentNode = $(this.pNode).parents("li").first();
            else
                parentNode = $(this.pNode).parents("li");
            this.memberTreeObj._createSubNodesWhenLoadOnDemand(newDataSource, this.pNode, mapper);
            $.each($(parentNode).children().find("li"), function (index, value) {
                if (!newDataSource[index].checkedStatus && $(value).find(".e-checkbox")[0].checked) $(value).find(".e-chkbox-wrap").trigger("click");
                value.setAttribute("tag", newDataSource[index].tag);
            });
            this.memberTreeObj._expandCollapseAction(this.pNode);
            oclientProxy._successAction = "FetchChildNode";
            oclientProxy._trigger("clientSuccess", oclientProxy);
        },

        /**
         * 	This function is used to build the toolbar icons.
         *
         * @private
         */
        _reportToolbar: function () {
            return ej.buildTag("ul", ej.buildTag("li.newReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("NewReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.addReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("AddReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.removeReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("RemoveReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.renameReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("RenameReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.saveReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SaveReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.loadReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("LoadReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.excelExportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("ExportToExcel"), tabindex: 0 })[0].outerHTML +
            (oclientProxy.model.displayOptions.fullScreenView ? (ej.buildTag("li.maximizedView e-icon", "", {}).attr({ "title": this._getLocalizedLabels("FullScreen"), tabindex: 0 })[0].outerHTML) : "") +
			ej.buildTag("li.colSortFilterImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SortOrFilterColumn"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.rowSortFilterImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SortOrFilterRow"), tabindex: 0 })[0].outerHTML);
        },

        /**
         * 	This function is used to create Cube Selector.
         *
         * @private  
         */
        _createCubeSelector: function () {
            return ej.buildTag("div.csHeader", ej.buildTag("span.cubeText", this._getLocalizedLabels("CubeSelector"), { "padding-left": "6px", "float": "left" })[0].outerHTML + ej.buildTag("div", "<input type='text' id='cubeSelector' class ='cubeSelector' />", { display: "inline", "float": "left", "padding-left": "5px", "position": "relative", "top": "-3px" })[0].outerHTML, { width: "415px", height: "20px" })[0].outerHTML;
        },

        /**
         * 	This function is used to create Cube Dimension Browser.
         *
         * @private  
         */
        _createCubeBrowser: function (cubeTreeInfo) {
            return ej.buildTag("div.cdbHeader", ej.buildTag("span", this._getLocalizedLabels("CubeDimensionBrowser"), { "padding-left": "5px" })[0].outerHTML, { width: "200px", height: "30px" })[0].outerHTML + ej.buildTag("div.cubeBrowser", ej.buildTag("div.cubeTreeView.visibleHide")[0].outerHTML, { width: "200px", height: "523px" })[0].outerHTML;
        },

        /**
         * 	This function is used to create Axis Element Builder.
         *
         * @private  
         */
        _createAxisElementBuilder: function (columnDimensions, rowDimensions, slicerDimensions) {
            var AEBPanel = ej.buildTag("div.categoricalAxis", this._createSplitButtons(columnDimensions, "Columns"), { width: "200px", height: "150px" })[0].outerHTML + ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Column"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML +
            ej.buildTag("div.rowAxis", this._createSplitButtons(rowDimensions, "Rows"), { width: "200px", height: "150px" })[0].outerHTML + ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Row"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML +
            ej.buildTag("div.slicerAxis", this._createSplitButtons(slicerDimensions, "Slicers"), { width: "200px", height: "150px" })[0].outerHTML + ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Slicer"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML;
            return AEBPanel;
        },

        /**
         * 	This function is used to create the Split Button(s).
         *
         * @private  
         */
        _createSplitButtons: function (dimensionElements, axis) {
            var splitButtons = '';
            var captionName = '';
            if (dimensionElements.indexOf("~") > -1)
                for (var i = 0; i < dimensionElements.split('~')[0].split('#').length - 1; i++) {
                    captionName = dimensionElements.split('~')[0].split('#')[i + 1].split('.')[0];
                    if (captionName == "Measures") {
                        captionName = oclientProxy._getLocalizedLabels("Measures");
                    }
                    else if (!(dimensionElements.startsWith("#Measure")))
                        captionName = dimensionElements.split('~')[i + 2];
                    else if ((dimensionElements.indexOf("#Measure")) > -1)
                        captionName = dimensionElements.split('~')[i + 1];
                    if (captionName == undefined)
                        captionName = dimensionElements.split('~')[i + 1];
                    splitButtons += ej.buildTag("div.splitBtn", ej.buildTag("button", captionName).attr({ "title": dimensionElements.split('~')[0].split('#')[i + 1].replace('.', ' - ') })[0].outerHTML + ej.buildTag("span.removeSplitBtn e-icon")[0].outerHTML, "", { "tag": axis + ':' + dimensionElements.split('~')[0].split('#')[i + 1] })[0].outerHTML;
                }
            else
                for (var i = 0; i < dimensionElements.split('#').length - 1; i++) {
                    var caption = dimensionElements.split('#')[i + 1].split('.')[0] == "Measures" ? oclientProxy._getLocalizedLabels("Measures") : dimensionElements.split('#')[i + 1].split('.')[0];
                    splitButtons += ej.buildTag("div.splitBtn", ej.buildTag("button", caption).attr({ "title": dimensionElements.split('#')[i + 1].replace('.', ' - ') })[0].outerHTML + ej.buildTag("span.removeSplitBtn e-icon")[0].outerHTML, "", { "tag": axis + ':' + dimensionElements.split('#')[i + 1] })[0].outerHTML;
                }
            return splitButtons;
        },

        /**
         * This function renders and returns the outer panel of OLAP Client control.
         *
         * @private
         */
        _controlPanel: function () {
            var controlTable; var gridDiv
            if (this.controlPlacement() == ej.OlapClient.controlPlacement.Tab && (this.displayMode() == ej.OlapClient.displayMode.ChartAndGrid || this.displayMode() == "")) {
                this._chartHeight = 547;
                this._chartWidth = 557;
                this._gridHeight = "530px";
                this._gridWidth = 555;
                this._panelOverflow = "auto";
            }
            else if (this.controlPlacement() == ej.OlapClient.controlPlacement.Tile && (this.displayMode() == ej.OlapClient.displayMode.ChartAndGrid || this.displayMode() == "")) {
                this._chartHeight = 275;
                this._chartWidth = 555;
                this._gridHeight = "530px";
                this._gridWidth = 555;
                this._panelOverflow = "hidden";
            }
            else {
                this._chartHeight = 560;
                this._chartWidth = 790;
                this._gridHeight = "550px";
                this._gridWidth = 555;
                this._panelOverflow = "auto";
            }
            if (this.displayMode() != ej.OlapClient.displayMode.ChartOnly) {
                if (this.controlPlacement() == ej.OlapClient.controlPlacement.Tile && this.displayMode() == ej.OlapClient.displayMode.ChartAndGrid)
                    gridDiv = ej.buildTag("div#gridPanel.gridPanel", ej.buildTag("div#OlapGrid", "", { "margin": "5px 7px 7px", "max-height": "300px", "width": "563px", "overflow": "auto" })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
                else
                    gridDiv = ej.buildTag("div#gridPanel.gridPanel", ej.buildTag("div#OlapGrid", "", { "margin": "5px 7px 7px", "min-height": "300px", "overflow": "auto", "max-height": this._gridHeight })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
            }
            if (this.displayMode() != ej.OlapClient.displayMode.GridOnly)
                var chartDiv = ej.buildTag("div#chartPanel.chartPanel", ej.buildTag("div#OlapChart", "", { "min-height": "275px" })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
            if (this.controlPlacement() == ej.OlapClient.controlPlacement.Tab && this.displayMode() == ej.OlapClient.displayMode.ChartAndGrid) {
                var tabUl; var tabDiv;
                var gridAnchor = "<a style='font: bold 12px Segoe UI' href='#gridPanel' tabindex='0'>" + this._getLocalizedLabels("Grid") + "</a>";
                var chartAnchor = "<a style='font: bold 12px Segoe UI' href='#chartPanel' tabindex='0'>" + this._getLocalizedLabels("Chart") + "</a>";
                if (this.defaultView() == ej.OlapClient.defaultView.Chart) {
                    tabUl = ej.buildTag("ul.clientTab", ej.buildTag("li", chartAnchor)[0].outerHTML + ej.buildTag("li", gridAnchor)[0].outerHTML)[0].outerHTML;
                    tabDiv = ej.buildTag("div#clientTab", tabUl + chartDiv + gridDiv)[0].outerHTML;
                }
                else {
                    tabUl = ej.buildTag("ul.clientTab", ej.buildTag("li", gridAnchor)[0].outerHTML + ej.buildTag("li", chartAnchor)[0].outerHTML)[0].outerHTML;
                    tabDiv = ej.buildTag("div#clientTab", tabUl + gridDiv + chartDiv)[0].outerHTML;
                }
                controlTable = tabDiv;
            }
            else {
                controlTable = this._createControlContainer(this.controlPlacement(), chartDiv, gridDiv);
            }

            return ej.buildTag("div.controlPanel", controlTable, { width: "572px", height: "600px", "overflow": this._panelOverflow })[0].outerHTML
        },

        /**
         * This function creates the HTML tag containing the control and returns it.
         *
         * @private  
         */
        _createControlContainer: function (controlPlacement, chartDiv, gridDiv) {
            var controlTable;
            if (this.displayMode() == ej.OlapClient.displayMode.ChartOnly)
                controlTable = "<table><tr><td>" + chartDiv + "</td></tr></table>";
            else if (this.displayMode() == ej.OlapClient.displayMode.GridOnly)
                controlTable = "<table><tr><td>" + gridDiv + "</td></tr></table>";
            else {
                if (this.controlPlacement() == ej.OlapClient.controlPlacement.Tile) {
                    if (this.defaultView() == ej.OlapClient.defaultView.Chart)
                        controlTable = "<table><tr><td>" + chartDiv + "</td></tr><tr><td>" + gridDiv + "</td></tr></table>";
                    else
                        controlTable = "<table><tr><td>" + gridDiv + "</td></tr><tr><td>" + chartDiv + "</td></tr></table>";
                }
                else {
                    if (this.defaultView() == ej.OlapClient.defaultView.Chart)
                        controlTable = "<table><tr><td valign='top'>" + chartDiv + "</td><td valign='top'>" + gridDiv + "</td></tr></table>";
                    else
                        controlTable = "<table><tr><td valign='top'>" + gridDiv + "</td><td valign='top'>" + chartDiv + "</td></tr></table>";
                }
            }
            return controlTable;
        },

        /**
         * 	This function is used to render the OLAP Grid component inside of OLAP Client component.
         *
         * @private  
         */
        _renderOlapGrid: function (tagInfo) {
            this.element.find(".e-dialog").hide();
            if (this.displayMode() != ej.OlapClient.displayMode.ChartOnly) {
                var gridDiv;
                if (this.ochartObj != undefined && this.ochartObj._startDrilldown) {
                    this.ogridObj.doAjaxPost("POST", this.model.url + "/" + this.ogridObj.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": "", "currentReport": this.ogridObj.getOlapReport(), "clientReports": reports, "headerInfo": tagInfo }), this.ogridObj._drillDownSuccess);
                }
                else {
                    this.element.find(".e-olapgrid, #OlapGrid").remove();
                    if (this.controlPlacement() == ej.OlapClient.controlPlacement.Tile && this.displayMode() == ej.OlapClient.displayMode.ChartAndGrid)
                        gridDiv = ej.buildTag("div#OlapGrid", "", { "margin": "5px 7px 7px", "max-height": "300px", "width": "563px", "overflow": "auto" })[0].outerHTML;
                    else
                        gridDiv = ej.buildTag("div#OlapGrid", "", { "margin": "5px 7px 7px", "min-height": "300px", "overflow": "auto", "max-height": this._gridHeight })[0].outerHTML;
                    $(gridDiv).appendTo(this.element.find(".gridPanel"));
                    if (this.gridLayout() != ej.OlapGrid.gridLayout.Normal)
                        this.element.find("#OlapGrid").ejOlapGrid({ url: this.model.url, olapReport: currentReport, gridLayout: this.gridLayout(), drillSuccess: ej.proxy(this.GridDrillSuccess, this) });
                    else
                        this.element.find("#OlapGrid").ejOlapGrid({ url: this.model.url, olapReport: currentReport, drillSuccess: ej.proxy(this.GridDrillSuccess, this) });
                    this.ogridObj = this.element.find('#OlapGrid').data("ejOlapGrid");
                }
            }
        },

        /**
         * 	This function is used to render the OLAP Chart component inside of OLAP Client component.
         *
         * @private  
         */
        _renderOlapChart: function (drillTag, drillAction) {
            this.element.find(".e-dialog").hide();
            if (this.displayMode() != ej.OlapClient.displayMode.GridOnly) {
                if (this.ogridObj != undefined && this.ogridObj._startDrilldown) {
                    this.ochartObj.doAjaxPost("POST", this.model.url + "/" + this.ochartObj.model.serviceMethods.drillDown, JSON.stringify({ "action": drillAction, "drilledSeries": drillTag, "olapReport": this.ochartObj.getOlapReport(), "clientReports": reports }), this.ochartObj.renderControlSuccess);
                }
                else {
                    this.element.find(".e-olapchart").remove();
                    var chartDiv = ej.buildTag("div#OlapChart", "", { "min-height": "275px" })[0].outerHTML;
                    $(chartDiv).appendTo(this.element.find(".chartPanel"));
                    this.element.find("#OlapChart").ejOlapChart({ url: this.model.url, customObject: currentReport, localization: oclientProxy.localization(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: ej.OlapChart.chartTypes.Column, tooltip: { visible: true } }, beforeServiceInvoke: oclientProxy.model.chartPreRender, drillSuccess: ej.proxy(oclientProxy.ChartDrillSuccess, oclientProxy) });
                    this.ochartObj = this.element.find('#OlapChart').data("ejOlapChart");
                    this.element.find('#OlapChart').width(this.ochartObj.model.size.width);
                }
            }
        },

        /**
         * 	This function is used to render the OLAP Chart and OLAP Grid widgets.
         *
         * @private  
         */
        _renderControls: function () {
            if (this.defaultView() == ej.OlapClient.defaultView.Chart) {
                oclientProxy._renderOlapChart();
                oclientProxy._renderOlapGrid();
            }
            else {
                oclientProxy._renderOlapGrid();
                oclientProxy._renderOlapChart();
            }
        },

        /**
         * 	This function is used to initiate the dialog creation.
         *
         * @private  
         */
        _createDialogRequest: function (evt) {
            if (oclientProxy.model.progressMode != "infinite")
                this._progressStatus(0, 80, 50);
            args_className = evt.target.className;
            args_innerHTML = (evt.target.innerHTML == "" ? "ToolbarButtons" : evt.target.innerHTML);
            hierarchyCaption = (evt.currentTarget.attributes == "" ? "" : evt.currentTarget.attributes.getNamedItem("title").value);
            this.element.find(".e-dialog, .clientDialog").remove();
            if (args_innerHTML != "ToolbarButtons" && args_innerHTML != undefined) {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialog") + "...";
                else if (oclientProxy.model.progressMode == "infinite")
                    oclientWaitingPopup.show();
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { Action: "fetchMemberTreeNodes", Element: this.element, CustomObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                this.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.fetchMemberTreeNodes, JSON.stringify({ "action": "fetchMemberTreeNodes", "dimensionName": $(evt.target).parent().attr("Tag"), "olapReport": currentReport, "customObject": serializedCustomObject }), oclientProxy._editorTreeInfoSuccess);
            }
            else {
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressHide();
                else
                    oclientProxy._createDialog(args_className, args_innerHTML, "");
            }
        },

        /**
         * 	This function receives the list of report names from the database.
         *
         * @private  
         */
        _fetchReportListSuccess: function (reportNameList) {
            if (reportNameList[0] != undefined) {
                if (reportNameList[2] != null && reportNameList[2] != undefined)
                    oclientProxy.model.customObject = reportNameList[2].Value;
            }
            else if (reportNameList.d != undefined) {
                if (reportNameList.d[2] != null && reportNameList.d[2] != undefined)
                    oclientProxy.model.customObject = reportNameList.d[2].Value;
            }
            else {
                if (reportNameList.customObject != null && reportNameList.customObject != undefined)
                    oclientProxy.model.customObject = reportNameList.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "fetchReportList", Element: this.element, CustomObject: this.model.customObject });
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialogSucceeded") + "...";
                window.clearInterval(oClientTimer);
                this._progressStatus(90, 101 + ":100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else {
                oclientWaitingPopup.hide();
                window.setTimeout(function () { progressStatus() }, 1);
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Action: "fetchReportList", Element: this.element, CustomObject: this.model.customObject });
            function progressStatus() {
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
                reportNameList = (reportNameList.d[0].Value || reportNameList[0].Value);
                oclientProxy._createDialog("LoadReport", reportNameList, "");
            }
        },

        /**
         * 	This function is used to maintain the sorting state inside the sort dialog.
         *
         * @private 
         */
        _fetchSortState: function (value) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialogSucceeded");
                window.clearInterval(oClientTimer);
                this._progressStatus(90, 101 + ":100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else {
                oclientWaitingPopup.hide();
                window.setTimeout(function () { progressStatus() }, 1);
            }
            function progressStatus() {
                if (value[0] != undefined) {
                    oclientProxy._createDialog("SortFilterDlg", value[0].Value, "");
                }
                else if (value.d != undefined) {
                    oclientProxy._createDialog("SortFilterDlg", value.d[0].Value, "");
                }
                else {
                    oclientProxy._createDialog("SortFilterDlg", value.Value, "");
                }

            }
        },

        /**
         * 	This function is used to create the toolbar, member editor and measure editor dialogs
         *
         * @private
         */
        _createDialog: function (args_className, args_innerHTML, editorTree) {
            var sortDlgField = "";
            var sortState = "", filterState = "";
            if (this.model.progressMode != "infinite")
                window.clearInterval(oClientTimer);
            var editorTreeInfo;
            if (editorTree[0] != undefined)
                editorTreeInfo = editorTree[0].Value;
            else if (editorTree.d != undefined)
                editorTreeInfo = editorTree.d[0].Value;
            else
                editorTreeInfo = editorTree.EditorTreeInfo;
            var ejDialog = ej.buildTag("div#clientDialog.clientDialog", { "opacity": "1" })[0].outerHTML;
            var dialogContent; var dialogTitle; var currentTag;

            if (args_className.split(" ")[0] == "newReportImg") {
                dialogTitle = this._getLocalizedLabels("NewReport");
                currentTag = "New Report";
                var newReportLabel = "<p class='dialogPara'>" + this._getLocalizedLabels("ReportName") + "</p>";
                var newReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = newReportLabel + newReport;
            }
            else if (args_className.split(" ")[0] == "addReportImg") {
                dialogTitle = this._getLocalizedLabels("AddReport");
                currentTag = "Add Report";
                var addReportLabel = "<p class='dialogPara'>" + this._getLocalizedLabels("ReportName") + "</p>";
                var addReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = addReportLabel + addReport;
            }
            else if (args_className.split(" ")[0] == "removeReportImg") {
                dialogTitle = this._getLocalizedLabels("RemoveReport");
                currentTag = "Remove Report";
                if (reportsCount > 1)
                    dialogContent = "<p>" + this._getLocalizedLabels("AreYouSureToDeleteTheReport") + "?</p>";
                else
                    dialogContent = "<p>" + this._getLocalizedLabels("CannotRemoveSingleReport") + "!</p>";
            }
            else if (args_className.split(" ")[0] == "SortFilterDlg") {

                sortDlgField = args_innerHTML.split("||");
                if (sortDlgField[1] != "")
                    sortState = sortDlgField[1].split("<<");
                if (sortDlgField[2] != "")
                    filterState = sortDlgField[2].split("<<");
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
                dialogTitle = this._getLocalizedLabels("SortingAndFiltering");

                var sortingLbl = ej.buildTag("label#sortingLbl.sortingLbl", this._getLocalizedLabels("Sorting"))[0].outerHTML;
                var measureListLbl = ej.buildTag("label#measureListLbl.measureListLbl", this._getLocalizedLabels("Measure"))[0].outerHTML;
                var orderLbl = ej.buildTag("label#orderLbl.orderLbl", this._getLocalizedLabels("Order"))[0].outerHTML;
                var measureListDropDown = ej.buildTag("input#measuresList.measuresList", "", {}, { type: 'text', accesskey: 'M' })[0].outerHTML;

                var sortDisableRadio = ej.buildTag("input#sortDisable.sortDisable", "", {}, { name: "sort", type: "radio", checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'i' })[0].outerHTML + ej.buildTag("label.sortDisableLbl", this._getLocalizedLabels("Disable"))[0].outerHTML;
                var sortEnableRadio = ej.buildTag("input#sortEnable.sortEnable", "", {}, { name: 'sort', type: 'radio', role: 'radio', tabindex: 1, accesskey: 'n' })[0].outerHTML + ej.buildTag("label.sortEnableLbl", this._getLocalizedLabels("Enable"))[0].outerHTML + "<br/>";
                var newascRadio = ej.buildTag("input#radioBtnAsc.radioBtnAsc", "", {}, { name: 'order', type: 'radio', checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'A' })[0].outerHTML + ej.buildTag("label.radioBtnAscLbl", this._getLocalizedLabels("Ascending"))[0].outerHTML + "<br/>";
                var newdescRadio = ej.buildTag("input#radioBtnDesc.radioBtnDesc", "", {}, { name: 'order', type: 'radio', role: 'radio', tabindex: 2, accesskey: 'e' })[0].outerHTML + ej.buildTag("label.radioBtnDescLbl", this._getLocalizedLabels("Descending"))[0].outerHTML;
                var preservHierarchy = ej.buildTag("input#preserveHrchy.preserveHrchy", "", {}, { type: 'checkbox', checked: 'checked', tabindex: 0, role: 'radio', accesskey: 'r' })[0].outerHTML + ej.buildTag("label.preserveHrchyLbl", this._getLocalizedLabels("PreserveHierarchy"))[0].outerHTML;

                var filter = ej.buildTag("label#filterLbl.filterLbl", this._getLocalizedLabels("Filtering"))[0].outerHTML;
                var filterDisableRadio = ej.buildTag("input#filterDisable.filterDisable", "", {}, { name: 'filter', type: 'radio', checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'i' })[0].outerHTML + ej.buildTag("label.filterDisableLbl", this._getLocalizedLabels("Disable"))[0].outerHTML;
                var filterEnableRadio = ej.buildTag("input#filterEnable.filterEnable", "", {}, { name: 'filter', type: 'radio', role: 'radio', tabindex: 0, accesskey: 'n' })[0].outerHTML + ej.buildTag("label.filterDisableLbl", this._getLocalizedLabels("Enable"))[0].outerHTML + "<br/>";
                var conditionLbl = ej.buildTag("label#conditionLbl.conditionLbl", this._getLocalizedLabels("Condition"))[0].outerHTML;
                var conditionDropDown = ej.buildTag("input#filterCondition.filterCondition", "", {}, { type: 'text', tabindex: 0, accesskey: 'o' })[0].outerHTML;

                var filterFrom = ej.buildTag("div#filterfrom.filterFrmDiv", ej.buildTag("input#filterFrom.filterFrom", "", {}, { name: 'inputVal', type: 'text', tabindex: 0 })[0].outerHTML)[0].outerHTML;
                var filterbtw = ej.buildTag("div#filterbtw.filterBtw", "<p>" + this._getLocalizedLabels("and") + "</p>")[0].outerHTML;
                var filterTo = ej.buildTag("div#filterto.filterToDiv", ej.buildTag("input#filterTo.filterTo", "", {}, { name: 'inputVal', type: 'text', tabindex: 0, accesskey: 'd' })[0].outerHTML)[0].outerHTML;
                var fMeasureListLbl = ej.buildTag("label#filterMeasureListLbl.filterMeasureListLbl", this._getLocalizedLabels("Measure"))[0].outerHTML;
                var fMeasureList = ej.buildTag("input#fMeasuresList.fMeasuresList", "", {}, { type: 'text', tabindex: 0, accesskey: 'M' })[0].outerHTML;
                var fValueLbl = ej.buildTag("label#filterValueLbl.filterValueLbl", this._getLocalizedLabels("Value"), {}, { tabindex: 0, accesskey: 'a' })[0].outerHTML;

                var sortDiv = ej.buildTag("div#sortingDlg.sortingDlg", "<table class='sortReportTbl'><tr><td>" + sortingLbl + "</td><td>" + sortEnableRadio + sortDisableRadio + "</td></tr><tr><td>" + measureListLbl + "</td><td>" + measureListDropDown + "</td></tr> <tr><td>" + orderLbl + "</td><td>" + newascRadio + newdescRadio + "</td></tr><tr><td></td><td>" + preservHierarchy + "</td></tr></table>")[0].outerHTML;
                var filterDiv = ej.buildTag("div#filteringDlg.filteringDlg", "<table class='sortReportTbl'><tr><td>" + filter + "</td><td>" + filterEnableRadio + filterDisableRadio + "</td></tr><tr><td>" + fMeasureListLbl + "</td><td>" + fMeasureList + "</td></tr><tr><td>" + conditionLbl + "</td><td>" + conditionDropDown + "</td></tr><tr><td>" + fValueLbl + "</td><td>" + filterFrom + filterbtw + filterTo + "</td></tr></table>")[0].outerHTML;

                var sortTab = "<li><a style='font: bold 12px Segoe UI' href='#sortingDlg'>" + this._getLocalizedLabels("Sorting") + "</a></li>";
                var filterTab = "<li><a style='font: bold 12px Segoe UI' href='#filteringDlg'>" + this._getLocalizedLabels("Filtering") + "</a></li>";
                var sortfilterTab = ej.buildTag("div#sortfilterTab.sortfilterTab", "<ul class ='sortfiltTab'>" + (sortTab + filterTab + "</ul>" + "<div class='borderFilterSortDlg'>" + sortDiv + filterDiv + "</div>"))[0].outerHTML;

                dialogContent = sortfilterTab;
            }
            else if (args_className.split(" ")[0] == "renameReportImg") {
                dialogTitle = this._getLocalizedLabels("RenameReport");
                currentTag = "Rename Report";
                var renameReportLabel = "<p class='dialogPara'>" + oclientProxy._getLocalizedLabels("ReportName") + "</p>";
                renameReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = renameReportLabel + renameReport;
            }
            else if (args_className.split(" ")[0] == "saveReportImg") {
                dialogTitle = this._getLocalizedLabels("SaveReport");
                currentTag = "Save Report";
                var reportNameLabel = "<p class='dialogPara'>" + oclientProxy._getLocalizedLabels("ReportName") + "</p>";
                reportName = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = reportNameLabel + reportName;
            }
            else if (args_className.split(" ")[0] == "LoadReport") {
                dialogTitle = this._getLocalizedLabels("LoadReport");
                currentTag = "Load Report"
                var reportNameLabel = "<table class='loadReportTbl'><tr><td class='loadReportTd'>" + oclientProxy._getLocalizedLabels("ReportName") + "</td>"
                var reportNamesDropdown = "<td><input type='text' id='reportNameList' class='reportNameList'/></td></tr></table>";
                dialogContent = reportNameLabel + reportNamesDropdown;
            }
            else if (args_className.indexOf("e-txt") > -1) {

                if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures")) {
                    dialogTitle = this._getLocalizedLabels("MeasureEditor");
                    dialogContent = "<p class='editorPara'>" + this._getLocalizedLabels("Measures") + "</p>"
                }
                else {
                    dialogTitle = this._getLocalizedLabels("MemberEditor");
                    dialogContent = "<p class='editorPara'>" + hierarchyCaption + "</p>"
                }
                var memberEditor; var checkOption = "";
                $(innerDiv).appendTo(EditorDiv);
                if (args_innerHTML != oclientProxy._getLocalizedLabels("Measures")) {
                    var checkAll = ej.buildTag("div.checkAll e-icon")[0].outerHTML;
                    var unCheckAll = ej.buildTag("div.unCheckAll e-icon")[0].outerHTML;
                    checkOption = ej.buildTag("div", checkAll + unCheckAll, { "height": "19px" })[0].outerHTML;
                }
                if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures")) {
                    memberEditor = oclientProxy._createMeasureEditor(editorTree);
                }
                else
                    memberEditor = ej.buildTag("div#editorTreeView.editorTreeView")[0].outerHTML;
                var memberEditorDiv = ej.buildTag("div.memberEditorDiv", memberEditor)[0].outerHTML;
                var innerDiv = ej.buildTag("div", checkOption + memberEditorDiv)[0].outerHTML;
                var EditorDiv = ej.buildTag("div#EditorDiv.editorDiv", innerDiv)[0].outerHTML;

                dialogContent += EditorDiv + "</br>";
            }
            var dialogFooter;
            var OKBtn = "<button id=OKBtn class='dialogOKBtn'>" + this._getLocalizedLabels("OK") + "</button>";
            var CancelBtn = "<button id=CancelBtn class='dialogCancelBtn'>" + this._getLocalizedLabels("Cancel") + "</button>";
            dialogFooter = ej.buildTag("div", OKBtn + CancelBtn, { "float": "right", "margin": "-5px 0 6px" })[0].outerHTML;
            $(ejDialog).appendTo("#" + oclientProxy._id);
            $(dialogContent + dialogFooter).appendTo(this.element.find(".clientDialog"));
            this.element.find(".clientDialog").ejDialog({ width: args_className.split(" ")[0] == "SortFilterDlg" ? 320 : 265, contentContainer: "#" + oclientProxy._id, resizable: false });
            this.element.find(".e-titlebar").prepend(ej.buildTag("div", dialogTitle, { "display": "inline" })[0].outerHTML)[0].setAttribute("tag", currentTag)
            this.element.find(".reportName").ejMaskEdit({ width: "155px" });
            this.element.find(".e-mask").addClass("dialogInput");
            this.element.find(".dialogOKBtn, .dialogCancelBtn").ejButton();
            $(".sortfilterTab").ejTab();
            $(".reportNameList").ejDropDownList({
                dataSource: args_innerHTML.split("__"),
                width: "150px",
                height: "20px",
            });
            $(".filterFrom,.filterTo").ejMaskEdit({
                name: "mask",
                inputMode: ej.inputMode.Text,
                width: "80px",
                height: "20px",
            });
            $(".measuresList").ejDropDownList({
                dataSource: args_innerHTML.split("||")[0].split("__"),
                width: "200px",
                height: "20px",
            });
            $(".fMeasuresList").ejDropDownList({
                dataSource: args_innerHTML.split("||")[0].split("__"),
                width: "200px",
                height: "20px",
            });
            $(".filterCondition").ejDropDownList({
                change: "_onActiveConditionChange",
                dataSource: ['EqualTo', 'NotEquals', 'GreaterThan', 'GreaterThanOrEqualTo', 'LessThan', 'LessThanOrEqualTo', 'Between', 'NotBetween'],
                width: "200px",
                height: "20px",
            });
            oclientProxy.element.find(".filterCondition").ejDropDownList("option", "change", ej.proxy(oclientProxy._onActiveConditionChange, oclientProxy));
            oclientProxy._dllSortMeasure = $(".measuresList").data("ejDropDownList");
            oclientProxy._dllFilterCondition = $(".filterCondition").data("ejDropDownList");
            oclientProxy._dllfMeasuresList = $(".fMeasuresList").data("ejDropDownList");
            if (sortState != "") {
                this.element.find(".radioBtnAsc")[0].checked = sortState[0] == "ASC" ? true : false;
                this.element.find(".radioBtnDesc")[0].checked = sortState[0] == "DESC" ? true : false;
                this.element.find(".sortDisable")[0].checked = !(this.element.find(".sortEnable")[0].checked = true);
                this.element.find(".preserveHrchy")[0].checked = sortState[1] == "PH" ? true : false;
                oclientProxy._isSorted = true;
                oclientProxy._dllSortMeasure.setModel({ value: (sortState[2]) });
            }
            if (filterState != "") {
                this.element.find(".filterEnable")[0].checked = true;
                oclientProxy._dllfMeasuresList.setModel({ value: (filterState[0]) });
                oclientProxy._dllFilterCondition.setModel({ value: (filterState[1]) });
                oclientProxy.element.find(".filterFrom").val(filterState[2]);
                if (filterState[3] != undefined && filterState[3] != "" && (filterState[1] == "Between" || filterState[1] == "NotBetween"))
                    oclientProxy.element.find(".filterTo").val(filterState[3]);
                else
                    $(".filterTo").attr("disabled", "disabled");
                oclientProxy._isFiltered = true;
            }
            if (this.element.find(".sortDisable")[0] != undefined && this.element.find(".sortDisable")[0].checked == true) {
                $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').attr("disabled", "disabled");
                oclientProxy._dllSortMeasure.disable();
            }
            if (this.element.find(".filterDisable")[0] != undefined && this.element.find(".filterDisable")[0].checked == true) {
                $('.filterFrom, .filterTo').attr("disabled", "disabled");
                oclientProxy._dllFilterCondition.disable();
                oclientProxy._dllfMeasuresList.disable();
            }
            this.element.find(".e-widget-content").height("auto");
            if (editorTree != "" && editorTree != undefined && args_innerHTML != oclientProxy._getLocalizedLabels("Measures")) {
                var treeViewData = $.parseJSON(editorTreeInfo);
                this.element.find(".editorTreeView").ejTreeView({
                    showCheckbox: true,
                    loadOnDemand: true,
                    height: $(".memberEditorDiv").height(),
                    fields: { id: "id", hasChild: "hasChildren", text: "name", isChecked: "checkedStatus", dataSource: treeViewData },
                    click: ej.proxy(this._onNodeExpand, this),
                });
                var treeViewElements = this.element.find(".editorTreeView").find("li");
                for (var i = 0; i < treeViewElements.length; i++) {
                    treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
                }
            }
            else {
                if (this.model.progressMode != "infinite")
                    this._progressHide();
            }
            this.memberTreeObj = this.element.find('.editorTreeView').data("ejTreeView");
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
        },

        /**
         * 	This function is used to create the measure editor dialog.
         *
         * @private  
         * @param {String} editorTree - Contains the information of the measures included in current report.
         * @return {String} String containing the html content of the measure editor.
         */
        _createMeasureEditor: function (editorTree) {
            var editorTreeInfo;
            if (editorTree[0] != undefined)
                editorTreeInfo = $.parseJSON(editorTree[0].Value);
            else if (editorTree.d != undefined)
                editorTreeInfo = $.parseJSON(editorTree.d[0].Value);
            else
                editorTreeInfo = $.parseJSON(editorTree.EditorTreeInfo);
            var measureEditor = "";
            for (var i = 0; i < editorTreeInfo.length; i++) {
                measureEditor += "<div id=\"" + editorTreeInfo[i].uniqueName + "\"class=measureEditor>" + editorTreeInfo[i].name + "<span class='removeMeasure e-icon'></span></div>";
            }
            return measureEditor;
        },

        /**
         * This function is raised to fetch the child nodes on node expansion.
         *
         * @private  
         */
        _onNodeExpand: function (args) {
            var selectedItem = args.Event.originalEvent == undefined ? args.Event.target : args.Event.originalEvent.target; var tagInfo;
            var childCount = $(selectedItem).parents("li").first().children().find("li").length;
            if (selectedItem.className.indexOf("e-plus") > -1 && childCount == 0) {
                var checkedState = $(selectedItem).parent().find("input[type=checkbox]").prop("checked");
                var tagInfo = $(selectedItem).parents("li").first().attr("tag");
                this.pNode = selectedItem;
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { Action: "memberExpanded", Element: this.element, CustomObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.memberExpand, JSON.stringify({
                    "action": "memberExpanded", "checkedStatus": checkedState, "parentNode": $(selectedItem).parents("li")[0].id, "tag": tagInfo, "dimensionName": this._dimensionName, "cubeName": this.currentCubeName, "olapReport": currentReport, "clientReports": reports, "customObject": serializedCustomObject
                }), this._fetchChildNodeSuccess);
            }
        },

        /**
         * 	The function is used to perform action while changing the condition in filter dialog.
         *
         * @private  
         */
        _onActiveConditionChange: function (args) {
            if (args.selectedText == "Between" || args.selectedText == "NotBetween")
                $(".filterTo").removeAttr("disabled");
            else {
                $(".filterTo").attr("disabled", "disabled");
                oclientProxy.element.find(".filterTo").val("");
            }
        },

        /**
        * This function is used to drill down the OLAP Grid widget once OLAP Chart drill down.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.GridDrillSuccess(ej.proxy(function(){}, this));
        * // raised after OLAP Chart drill down
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        ChartDrillSuccess: function (sender) {
            if (oclientProxy.currentReport != undefined)
                currentReport = oclientProxy.currentReport;
            if (oclientProxy.reports != undefined)
                reports = oclientProxy.reports;
            if (this.ochartObj._startDrilldown && this.ogridObj != null && this.ogridObj != undefined)
                this._renderOlapGrid(this.ochartObj._selectedTagInfo);
            else if (oclientProxy.ogridObj != undefined && oclientProxy.ogridObj.element.find("table") == "block")
                oclientWaitingPopup.hide();
        },

        /**
        * This function is used to drill down the OLAP Chart widget once OLAP Grid component drill down.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.GridDrillSuccess(ej.proxy(function(){}, this));
        * // raised after OLAP Grid drill down
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        GridDrillSuccess: function (sender) {
            if (oclientProxy.currentReport != undefined)
                currentReport = oclientProxy.currentReport;
            if (oclientProxy.reports != undefined)
                reports = oclientProxy.reports;
            if (this.ogridObj._startDrilldown && this.ochartObj != null && this.ochartObj != undefined) {
                var masktag; var levelCaption; var levelTag;
                levelCaption = this.ogridObj._drillCaption;
                jQuery.each(this.ochartObj._labelCurrentTags, function (index, value) {
                    if (value.name == levelCaption) {
                        levelTag = value.tag;
                        masktag = value;
                    }
                });
                oclientProxy.ochartObj._drillAction = this.ogridObj._drillAction;
                oclientProxy.ochartObj._selectedItem = levelCaption;
                if (this.ogridObj._drillAction == "drilldown")
                    oclientProxy.ochartObj._tagCollection.push(masktag)
                else if (!ej.isNullOrUndefined(masktag)) {
                    jQuery.each(this.element.find(".rowAxis").find("button"), function (index, value) {
                        if (value.innerHTML == masktag.tag.split("[")[1].split("]")[0])
                            oclientProxy.ochartObj._dimensionIndex = index;
                    });
                    jQuery.each(oclientProxy.ochartObj._tagCollection, function (index, value) {
                        if (value.name == levelCaption) {
                            oclientProxy.ochartObj._selectedIndex = index + oclientProxy.ochartObj._dimensionIndex;
                            oclientProxy.ochartObj._tagCollection.splice(index, oclientProxy.ochartObj._tagCollection.length);
                            return false;
                        }
                    });
                }
                if (levelTag != null && levelTag != undefined)
                    oclientProxy._renderOlapChart(levelTag, this.ogridObj._drillAction);
                else
                    oclientWaitingPopup.hide();
            }
            else if ($("#" + oclientWaitingPopup._id + "_WaitingPopup")[0].style.display == "block")
                oclientWaitingPopup.hide();
        },

        /**
        * This function is used to perform required action after dropping a tree node in Axis Element builder.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.NodeDropped(ej.proxy(function(){}, this));
        * // raised while dropping tree nodes
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        NodeDropped: function (sender) {
            if (sender.target.prevObject != undefined)
                sender.target.prevObject.removeClass("targetAxis");
            else
                $(sender.target).removeClass("targetAxis");
            isDragging = false;
            var droppedPosition = this.setSplitBtnTargetPos(sender.event);
            sender.target.prevObject != undefined ? sender.target.prevObject.parents().find(".e-dialog").hide() : $(sender.target).parents().find(".e-dialog").hide();
            var cssName = null;
            if ($(sender.target).hasClass("splitBtn"))
                cssName = $(sender.target).parents("div:eq(0)").attr("class");
            else if (sender.event.type == "touchend")
                cssName = sender.target.context.className;
            else if (sender.event.target != undefined) {
                if (sender.event.target.className == undefined || sender.event.target.className == null)
                    return false;
                if (sender.event.target.className.indexOf("splitBtn") > -1)
                    cssName = sender.target.className;
                else
                    cssName = sender.event.target.className;
            }
            else {
                if (sender.event.originalEvent.srcElement.className.indexOf("splitBtn") > -1)
                    cssName = sender.target.className;
                else
                    cssName = sender.event.originalEvent.srcElement.className;
            }

            var axisName = (cssName.indexOf("categoricalAxis") > cssName.indexOf("rowAxis")) ? ((cssName.indexOf("categoricalAxis") > cssName.indexOf("slicerAxis")) ? "Categorical" : "Slicer") : ((cssName.indexOf("rowAxis") > cssName.indexOf("slicerAxis")) ? "Series" : (cssName.indexOf("slicerAxis") >= 0) ? "Slicer" : null);
            var tagVal = $(sender.dropedElement.parents("li:eq(0)")[0]).attr("tag");
            if (axisName != null && tagVal != "Measures" && tagVal.lastIndexOf(".DF") != tagVal.length - ".DF".length) {
                if (oclientProxy.model.progressMode != "infinite") {
                    if (oclientProxy.model.progressMode == "progress")
                        $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                    this._progressStatus(10, 80, 120);
                }
                else
                    oclientWaitingPopup.show();
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { Action: "nodeDropped", Element: this.element, CustomObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                var params = oclientProxy.currentCubeName + "--" + $(sender.dropedElement.parents("li:eq(0)")[0]).attr("tag") + "--" + axisName + "--" + droppedPosition;
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethods.nodeDropped, JSON.stringify({ "action": "nodeDropped", "dropType": "TreeNode", "nodeInfo": params, "olapReport": currentReport, "clientReports": reports, "customObject": serializedCustomObject }), oclientProxy._nodeDroppedSuccess);
            }
        },

        /**
        * This function is used to set the position to currently dropped split button in respective axes.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.setSplitBtnTargetPos(eve);
        * // used to set the position to the dropped split button
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        setSplitBtnTargetPos: function (event) {
            var targetPosition = ""; var AEBdiv; var targetSplitBtn; var className;
            if (event.event != undefined && event.event.type == "touchend") {
                targetSplitBtn = event.event.originalEvent.target != null ? $(event.event.originalEvent.target).parents(".splitBtn") : $(event.event.originalEvent.srcElement).parents(".splitBtn");
                className = event.event.originalEvent.target != null ? event.event.originalEvent.target.className : event.event.originalEvent.srcElement.className;
            }
            else {
                targetSplitBtn = event.originalEvent.target != null ? $(event.originalEvent.target).parents(".splitBtn") : $(event.originalEvent.srcElement).parents(".splitBtn");
                className = event.originalEvent.target != null ? event.originalEvent.target.className : event.originalEvent.srcElement.className;
            }
            if (targetSplitBtn[0] || (className != undefined && className != null && jQuery.type(className) == "string" && className.indexOf("splitBtn") > -1)) {
                targetSplitBtn = targetSplitBtn[0] ? targetSplitBtn[0] : event.originalEvent.target != null ? event.originalEvent.target : event.originalEvent.srcElement;
                if (event.event != undefined && event.event.type == "touchend")
                    AEBdiv = event.target;
                else if (this._dropAxisClassName != null && this._dropAxisClassName != undefined && this._dropAxisClassName != "")
                    AEBdiv = this.element.find("." + this._dropAxisClassName)[0];
                else
                    AEBdiv = targetSplitBtn.parentNode;
                for (var i = 0; i < AEBdiv.children.length; i++) {
                    if (AEBdiv.childNodes[i] == targetSplitBtn)
                        targetPosition = i;
                }
            }
            return targetPosition;
        },

        /**
        * This function is raised while changing the cube.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.CubeChanged(ej.proxy(function(){}, this));
        * // raised while changing cubes
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        CubeChanged: function (sender) {
            this.currentCubeName = sender.selectedText;
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutStarted");
                this._progressStatus(0, 80, 120);
            }
            else
                oclientWaitingPopup.show();
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { Action: "cubeChanged", Element: this.element, CustomObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.cubeChanged, JSON.stringify({ "action": "cubeChanged", "cubeName": this.currentCubeName, "customObject": serializedCustomObject }), this._cubeChangedSuccess);
        },

        /**
        * This function is raised while changing the report.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.ReportChanged(ej.proxy(function(){}, this));
        * // raised while changing reports
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        ReportChanged: function (sender) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                this._progressStatus(5, 80, 100);
            }
            else
                oclientWaitingPopup.show();
            if (oclientProxy.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { Action: "toolbarOperation", Element: this.element, CustomObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "Report Change", "clientInfo": sender.selectedText, "olapReport": null, "clientReports": reports, "customObject": serializedCustomObject }), this._toolbarOperationSuccess);
        },

        /**
        * This function is raised after dropping the element in Axis Element Builder.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.onDropped(ej.proxy(function(){}, this));
        * // raised while dropping tree nodes
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        onDropped: function (sender) {
            if (sender.target.className.indexOf("e-button") > -1) {
                var columnPosInfo = this.GetAxisPosition(this.element.find(".categoricalAxis"));
                var rowPosInfo = this.GetAxisPosition(this.element.find(".rowAxis"));
                var slicerPosInfo = this.GetAxisPosition(this.element.find(".slicerAxis"));
                this._dropAxisClassName;
                if (sender.pageX != undefined && sender.pageY != undefined) {
                    if (sender.pageX > columnPosInfo.left && sender.pageX < columnPosInfo.right && sender.pageY > columnPosInfo.top && sender.pageY < columnPosInfo.bottom)
                        this._dropAxisClassName = "categoricalAxis";
                    else if (sender.pageX > rowPosInfo.left && sender.pageX < rowPosInfo.right && sender.pageY > rowPosInfo.top && sender.pageY < rowPosInfo.bottom)
                        this._dropAxisClassName = "rowAxis";
                    else if (sender.pageX > slicerPosInfo.left && sender.pageX < slicerPosInfo.right && sender.pageY > slicerPosInfo.top && sender.pageY < slicerPosInfo.bottom)
                        this._dropAxisClassName = "slicerAxis";
                    else
                        this._dropAxisClassName = "outOfAxis";
                }
            }
        },

        /**
         * 	This function is used to get the position of the axis element builders.
         * @return jQuery
         * @example 
         * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Create OLAP Client
         * $('#OlapClient').ejOlapClient({
         *       url: "OlapClientService.svc"
         *   });
         * var clientObj = $("#OlapClient").data("ejOlapClient");
         * clientObj.GetAxisPosition(eve);
         * // used to get the position of the axis element builders
         * &lt;/script&gt;
         * @memberof ejOlapClient
         * @instance
         */
        GetAxisPosition: function (targetElement) {
            var tempPos = targetElement.position();
            var item = {};
            item["top"] = tempPos.top;
            item["right"] = tempPos.left + $(targetElement).width();
            item["bottom"] = tempPos.top + $(targetElement).height();
            item["left"] = tempPos.left;
            return item;
        },

        /**
         * This function is raised while tab changes.
         * @return jQuery
         * @example 
         * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Create OLAP Client
         * $('#OlapClient').ejOlapClient({
         *       url: "OlapClientService.svc"
         *   });
         * var clientObj = $("#OlapClient").data("ejOlapClient");
         * clientObj.OnTabClick(ej.proxy(function(){}, this));
         * // raised while changing tabs
         * &lt;/script&gt;
         * @memberof ejOlapClient
         * @instance
         */
        OnTabClick: function (args) {
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined)
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
            currentTab = "chart";
            if (args.newContentPanel.className.indexOf("gridPanel") > -1) {
                currentTab = "grid";
                oclientProxy.element.find(".gridPanel").css("display", "inline");
            }
            else if (oclientProxy.chartObj != null && oclientProxy.chartObj != undefined) {
                currentTab = "chart";
                if (oclientProxy.model.progressMode == "infinite")
                    oclientWaitingPopup.show();
                oclientProxy.chartObj.redraw();
                if (oclientProxy.model.progressMode == "infinite")
                    oclientWaitingPopup.hide();
            }
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.doAjaxPost("POST", "/OlapClientService.svc/Initialize", {"key", "Hello World"}, "renderControlSuccess", null);
        * // initiate an Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: function () {
                    oclientProxy._trigger("clientComplete", oclientProxy.element);
                },
                error: function (msg, textStatus, errorThrown) {
                    if (oclientProxy.model.progressMode != "infinite") {
                        window.clearInterval(oClientTimer);
                        if (oclientProxy.model.progressMode == "progress")
                            $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        $("#" + oclientProxy._id).show();
                        oclientProgressBar.setModel({ value: 0, text: "Failed!!!" });
                    }
                    else {
                        oclientWaitingPopup = $("#" + oclientProxy._id).data("ejWaitingPopup");
                        oclientWaitingPopup.hide();
                    }
                    oclientProxy._trigger("clientError", oclientProxy.element);
                }
            });
        }
    });

    //OLAP Client Localization
    ej.OlapClient.localization = {};
    //Default english localized values
    ej.OlapClient.localization["en-US"] = {
        Column: "Column",
        Row: "Row",
        Slicer: "Slicer",
        CubeSelector: "Cube Selector",
        ReportName: "Report Name",
        NewReport: "New Report",
        CubeDimensionBrowser: "Cube Dimension Browser",
        AddReport: "Add Report",
        RemoveReport: "Remove Report",
        CannotRemoveSingleReport: "Can not remove Single Report",
        AreYouSureToDeleteTheReport: "Are you sure to delete the Report",
        RenameReport: "Rename Report",
        SaveReport: "Save Report",
        LoadReport: "Load Report",
        ExportToExcel: "Export To Excel",
        FullScreen: "Full Screen",
        Grid: "Grid",
        Chart: "Chart",
        OK: "OK",
        Cancel: "Cancel",
        MeasureEditor: "Measure Editor",
        MemberEditor: "Member Editor",
        Measures: "Measures",
        OpeningDialog: "Opening dialog",
        OpeningDialogSucceeded: "Opening dialog succeeded",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutionFailed: "MDX query execution failed",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingLayoutStarted: "Rendering Layout Started",
        RenderingLayoutCompleted: "Rendering Layout Completed",
        RenderingStarted: "Rendering Started",
        RenderingSucceeded: "Rendering Succeeded",
        SortOrFilterColumn: "Sort/Filter (Column)",
        SortOrFilterRow: "Sort/Filter (Row)",
        SortingAndFiltering: "Sorting And Filtering",
        Sorting: "Sorting",
        Measure: "<u>M</u>easure",
        Order: "Order",
        Filtering: "Filtering",
        Condition: "C<u>o</u>ndition",
        Value: "Val<u>u</u>e",
        PreserveHierarchy: "P<u>r</u>eserve  Hierarchy",
        Ascending: "<u>A</u>scending",
        Descending: "D<u>e</u>scending",
        Enable: "E<u>n</u>able",
        Disable: "D<u>i</u>sable",
        and: "<u>a</u>nd",
    }

    /**
     * Enum for control placement. Allows the user to select a appropriate view for positioning the controls. 
     * @enum {String}
	 * @global
     */
    ej.OlapClient.controlPlacement = {
        Tab: "tab",
        Tile: "tile"
    };

    /**
     * Enum for display mode. Allows the user to select a appropriate display mode. 
     * @enum {String}
	 * @global
     */
    ej.OlapClient.displayMode = {
        ChartOnly: "chartOnly",
        GridOnly: "gridOnly",
        ChartAndGrid: "chartAndGrid"
    };

    /**
     * Enum for default view. Allows the user to set the default view of the control. 
     * @enum {String}
	 * @global
     */
    ej.OlapClient.defaultView = {
        Chart: "chart",
        Grid: "grid"
    };

    /**
     * Enum for progress mode. Allows the user to select an appropriate progress indicator. 
     * @enum {String}
	 * @global
     */
    ej.OlapClient.progressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html OLAP Chart elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejOlapChart
	* @requires jQuery
	* @requires ej.common.all
	* @requires ej.tools
	* @requires ej.chart
	* @classdesc Custom Design for Html OLAP Chart control.
	* @example 
	* &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create OLAP Chart
    * $("#OlapChart").ejOlapChart(...); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejOlapChart", "ej.OlapChart", {

        //Root Css Class
        _rootCSS: "e-olapchart",

        //Widget element will be automatically set in this
        element: null,

        //User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        //Default model values
        defaults: $.extend(ej.Chart.prototype.defaults, {
            //Properties
            /**		
			* Connects the service using the specified URL for any server updates.
			* @default ""
			* @type {string}
			* @example 
			* //To set url API value during initialization  
			* 	$("#OlapChart").ejOlapChart({  url: "/OlapChartService.svc"});					* 
			* @example 
			* //Get or set the url API, after initialization:<br>
			*	//Gets the url value  
			*	$("#OlapChart").ejOlapChart("option", "url");<br>			
			*	//Sets the url value 
			*	$("#OlapChart").ejOlapChart("option", "url",  "/OlapChartService.svc" ); 
			 * @memberof ejOlapChart
			* @instance
			*/
            url: "",
            /**		
			* Specify the CSS class to OLAP Chart to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the CSS class during initialization. 			
			* 	$("#OlapChart").ejOlapChart({  cssClass : "gradient-lime" });			* 
			* @example 
			* //Get or set the CSS class after initialization:<br>
			*	// Gets the CSS class value.		
			*	$("#OlapChart").ejOlapChart("option", "cssClass");			
			*	// Sets the CSS class to OLAP Chart
			*	$("#OlapChart").ejOlapChart("option", "cssClass",  "gradient-lime" );
			 * @memberof ejOlapChart
			* @instance
			*/
            cssClass: "",
            /**		
			* Custom object to pass additional information between client-end and service-end.
			* @default null
			* @type {object}
			* @example 
			* //To set customObject API value during initialization  
			* 	$("#OlapChart").ejOlapChart({  customObject: {"key":"Hello World"}});					* 
			* @example 
			* //Get or set the customObject API, after initialization:<br>
			*	//Gets the customObject value  
			*	$("#OlapChart").ejOlapChart("option", "customObject");<br>			
			*	//Sets the customObject value 
			*	$("#OlapChart").ejOlapChart("option", "customObject",  {"key":"Hi Syncfusion!"} ); 
			 * @memberof ejOlapChart
			* @instance
			*/
            customObject: {},
            /**		
			* Allows the user to set custom name for the methods at service-end invoked on AJAX post.
			* @default null
			* @type {object}
			* @example 
			* //To set serviceMethods API value during initialization  
			* 	$("#OlapChart").ejOlapChart({  serviceMethods: {initialize: "MyMethod", drillDown: "DrillChart"});					* 
			* @example 
			* //Get or set the serviceMethods API, after initialization:<br>
			*	//Gets the serviceMethods value  
			*	$("#OlapChart").ejOlapChart("option", "serviceMethods");<br>			
			*	//Sets the serviceMethods value 
			*	$("#OlapChart").ejOlapChart("option", "serviceMethods",  {initialize: "MyMethod", drillDown: "DrillChart"} ); 
			 * @memberof ejOlapChart
			* @instance
			*/
            serviceMethods: { initialize: "InitializeOlapChart", drillDown: "DrillOlapChart" },
            /**		
			* Sets the progress indicator type which arises on any UI operation.	See {@link progressMode}
			* @default ej.OlapChart.progressMode.Infinite
			* @type {enum}
			* @example 
			* //To set progressMode API value during initialization  
			* 	$("#OlapChart").ejOlapChart({  progressMode: ej.OlapChart.progressMode.Normal});					* 
			* @example 
			* //Get or set the progressMode API, after initialization:<br>
			*	//Gets the progressMode value  
			*	$("#OlapChart").ejOlapChart("option", "progressMode");<br>			
			*	//Sets the progressMode value 
			*	$("#OlapChart").ejOlapChart("option", "progressMode",  ej.OlapChart.progressMode.Progress ); 
			 * @memberof ejOlapChart
			* @instance
			*/
            progressMode: "infinite",
            /**		
			* Sets the localized language for the control.
			* @default "en-US"
			* @type {string}
			* @example 
			* //To set localization API value during initialization  
			* 	$("#OlapChart").ejOlapChart({  localization: "fr-FR"});					* 
			* @example 
			* //Get or set the localization API, after initialization:<br>
			*	//Gets the localization value  
			*	$("#OlapChart").ejOlapChart("option", "localization");<br>			
			*	//Sets the localization value 
			*	$("#OlapChart").ejOlapChart("option", "localization",  "fr-FR" ); 
			 * @memberof ejOlapChart
			* @instance
			*/
            localization: "en-US",
            //Events
            /**     
			 * Fires when drilldown happens in OLAP Chart control.
			 * @event
			 * @name ejOlapChart#drillSuccess 	
			 * @param {Object} argument Event parameters from OLAP Chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Chart model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //drillSuccess event for OLAP Chart
             * $("#OlapChart").ejOlapChart({
             *    drillSuccess: function (args) {}
             * });      
			 * @memberof ejOlapChart
			 * @instance
			 */
            drillSuccess: null,
            /**     
			 * Fires before any Ajax request passed from OLAP Chart to service methods.
			 * @event
			 * @name ejOlapChart#beforeServiceInvoke 	
			 * @param {Object} argument Event parameters from OLAP Chart    
			 * @param {string}  argument.Action return the current action of OLAP Chart control.		 
			 * @param {object}  argument.CustomObject return the custom object bounds with OLAP Chart control.
			 * @param {string}  argument.Element return the outer HTML of OLAP Chart control.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Chart model.
			 * @param {string}  argument.type returns the name of the event.			 
			 * @example 
			 * //beforeServiceInvoke event for OLAP Chart
             * $("#OlapChart").ejOlapChart({
             *    beforeServiceInvoke: function (args) {}
             * });      
			 * @memberof ejOlapChart
			 * @instance
			 */
            beforeServiceInvoke: null,
            /**     
			 * Fires when it reaches client script after any Ajax request.
			 * @event
			 * @name ejOlapChart#afterServiceInvoke 	
			 * @param {Object} argument Event parameters from OLAP Chart  
			 * @param {string}  argument.Action return the current action of OLAP Chart control.		 
			 * @param {object}  argument.CustomObject return the custom object bounds with OLAP Chart control.
			 * @param {string}  argument.Element return the outer HTML of OLAP Chart control.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Chart model.
			 * @param {string}  argument.type returns the name of the event.			 
			 * @example 
			 * //afterServiceInvoke event for OLAP Chart
             * $("#OlapChart").ejOlapChart({
             *    afterServiceInvoke: function (args) {}
             * });      
			 * @memberof ejOlapChart
			 * @instance
			 */
            afterServiceInvoke: null,
        }),

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            marker: "data",
            crossHair: "data",
            size: "data",
            serviceMethods: "data",
            zooming: "data",
            customObject: "data"
        },

        observables: ["title.text", "commonSeriesOptions.type", "localization"],
        titleText: ej.util.valueFunction("title.text"),
        seriesType: ej.util.valueFunction("commonSeriesOptions.type"),
        localization: ej.util.valueFunction("localization"),

        /**
        * Gets the serialized current OLAP Report of the control.
        *
        * @private 
        */
        //get the current OLAP Report
        getOlapReport: function () {
            return this._olapReport;
        },

        /**
        * Sets the serialized current OLAP Report of the control.
        *
        * @private 
        */
        //set the current OLAP Report
        setOlapReport: function (value) {
            this._olapReport = value;
        },

        /**
        * Gets the JSON formatted records for the control.
        *
        * @private 
        */
        //get the JSON formatted records
        getJsonRecords: function () {
            return this._JSONRecords;
        },

        /**
        * Sets the JSON formatted records for the control.
        *
        * @private 
        */
        //set the JSON formatted records
        setJsonRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        // constructor function
        /**
        * Initializes the variables and loads the widget. 
        *
        * @private
        */
        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        /**
        * Destroy the OLAP Chart widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Create OLAP Chart
		* var chartObj = $("#OlapChart").data("ejOlapChart");
		* chartObj.destroy(); // destroy the OLAP Chart
		* &lt;/script&gt;
		* @example 
		* &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // enable the OLAP Chart
		* $("#OlapChart").ejOlapChart("destroy");	
		* &lt;/script&gt;
		* @memberof ejOlapChart
		* @instance
        */
        _destroy: function () {
            this.element.empty().removeClass("e-olapchart" + this.model.cssClass);
        },

        /**
        * To configure the private properties of OLAP Chart		
        * @private
        */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            ochartProxy = this;
            this._olapReport = "";
            this._JSONRecords = null;
            this._currentAction = "initialize";
            this._selectedItem = "";
            this._selectedIndex = -1;
            this._selectedTagInfo = null;
            this._tagCollection = new Array();
            this._labelCurrentTags = new Array();
            this._startDrilldown = false;
            this._drillAction = "";
            this._initZooming = false;
            this._dimensionIndex = -1;
            this._isDrilled = false;
            this._selectedMenuItem = "";
            ochartWaitingPopup = null;
            ochartProgressBar = null;
            oChartTimer = null;
        },

        /**
        * This function is the point were loading/rendering of the OLAP Chart widget is initiated.
        *
        * @private  
        */
        _load: function () {
            this.element.addClass(this.model.cssClass);
            if (this.model.progressMode != "infinite") {
                $("#" + ochartProxy._id + "progressContainer").remove();
                this._createProgressBar();
                this._progressStatus(5, 69, 140);
            }
            else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null)
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ autoDisplay: true });
                }
                else
                    oclientWaitingPopup.show();
            else {
                this.element.ejWaitingPopup({ autoDisplay: true });
                ochartWaitingPopup = this.element.data("ejWaitingPopup");
            }
            if (this.model.zooming != "")
                this._initZooming = true;
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { Action: this._currentAction, Element: this.element, CustomObject: this.model.customObject });
            if (ochartProxy.model.progressMode == "progress")
                $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({ "action": "initialize", "customObject": serializedCustomObject }), this.renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({ "action": "initialize" }), this.renderControlSuccess);
        },
        _setFirst: true,

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var chartObj = this.element.find("#" + this._id + "Container").data("ejChart");
            for (var key in options) {
                switch (key) {
                    case "olapReport": this.setOlapReport(options[key]); break;
                    case "jsonData": this.setJsonRecords(options[key]); break;
                    case "refreshOlapChart": this.element.renderChartFromJSON(options[key]); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "height": this.model.size.height = options[key]; break;
                    case "width": this.model.size.width = options[key]; break;
                    case "commonSeriesOptions":
                        {
                            if (chartObj) {
                                chartObj.model.commonSeriesOptions = options[key];
                                chartObj.model.type = chartObj.model.commonSeriesOptions.type = this.seriesType();
                                for (var i = 0; i < chartObj.model.series.length; i++)
                                    chartObj.model.series[i].type = chartObj.model.type;
                            } break;
                        }
                    case "title":
                        {
                            if (chartObj)
                                chartObj.model.title.text = this.titleText();
                            break;
                        }
                    case "animation": this.model.animation = options[key]; break;
                    case "crossHair": this.model.crossHair = options[key]; break;
                    case "marker": this.model.marker = options[key]; break;
                    case "zooming": this.model.zooming = options[key]; break;
                    case "legend": this.model.legend = options[key]; break;
                    case "primaryXAxis": this.model.primaryXAxis = options[key]; break;
                    case "primaryYAxis": this.model.primaryYAxis = options[key]; break;
                    case "localization": this.localization(); break;
                    default:
                        { $.extend(true, this.model, {}, options[key]); }
                }
            }
            if (chartObj)
                this.renderControlSuccess({ "JsonRecords": JSON.stringify(this.getJsonRecords()), "OlapReport": this.getOlapReport() });
        },

        /**
        * This function is invoked to wire the events for UI interaction.
        *
        * @private  
        */
        _wireEvents: function () {
            window.clearInterval(oChartTimer);
            this._on(this.element, "click", ".menuList", function (evt) {
                if (evt.target.innerHTML == ochartProxy._getLocalizedLabels("Exit")) {
                    this.element.find("#" + ochartProxy._id + "ExpandMenu, .expandMenu, .e-dialog").remove();
                }
                else if (evt.target.innerHTML.indexOf(ochartProxy._getLocalizedLabels("Expand")) > -1) {
                    if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0])
                            $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ autoDisplay: true });
                        else
                            oclientWaitingPopup.show();
                    }
                    else {
                        if (this.model.progressMode != "infinite")
                            ochartProxy._progressStatus(0, 70, 120);
                        else
                            ochartWaitingPopup.show();
                    }
                    if (!this._isDrilled) {
                        this._isDrilled = true;
                        jQuery.each(evt.target.parentElement.children, function (index, value) {
                            if (value.innerHTML == evt.target.innerHTML)
                                ochartProxy._dimensionIndex = index;
                        });
                    }
                    this._drillAction = "drilldown";
                    this._selectedItem = $.trim(evt.target.innerHTML.split("-")[1]);
                    jQuery.each(this._labelCurrentTags, function (index, value) {
                        if (value.name == ochartProxy._selectedItem) {
                            ochartProxy._tagCollection.push(value);
                            ochartProxy._selectedTagInfo = value.tag;
                            ochartProxy._selectedMenuItem = value.name;
                            return false;
                        }
                    });
                    this._startDrilldown = true;
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { Action: this._drillAction, Element: this.element, CustomObject: this.model.customObject });
                    if (ochartProxy.model.progressMode == "progress")
                        $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined) {
                        if (typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "clientReports": reports, "customObject": serializedCustomObject }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "customObject": serializedCustomObject }), this.renderControlSuccess);
                    }
                    else {
                        if (!typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport() }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "clientReports": reports }), this.renderControlSuccess);
                    }

                }
                else {
                    if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0])
                            $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ autoDisplay: true });
                        else
                            oclientWaitingPopup.show();
                    }
                    else {
                        if (this.model.progressMode != "infinite")
                            ochartProxy._progressStatus(0, 70, 120);
                        else
                            ochartWaitingPopup.show();
                    }
                    this._drillAction = "drillup";
                    this._selectedItem = $.trim(evt.target.innerHTML.split("-")[1]);
                    jQuery.each(this._tagCollection, function (index, value) {
                        if (value.name == ochartProxy._selectedItem) {
                            ochartProxy._selectedIndex = index + ochartProxy._dimensionIndex;
                            ochartProxy._selectedTagInfo = value.tag;
                            ochartProxy._tagCollection.splice(index, ochartProxy._tagCollection.length);
                            return false;
                        }
                    });
                    if (this._tagCollection.length == 0)
                        this._isDrilled = false;
                    this._startDrilldown = true;
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { Action: this._drillAction, Element: this.element, CustomObject: this.model.customObject });
                    if (ochartProxy.model.progressMode == "progress")
                        $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined) {
                        if (typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "clientReports": reports, "customObject": serializedCustomObject }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "customObject": serializedCustomObject }), this.renderControlSuccess);
                    }
                    else {
                        if (!typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport() }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "clientReports": reports }), this.renderControlSuccess);
                    }
                }
            });
        },

        /**
         * This function is invoked to unwire the events registered for UI interaction.
         *
         * @private  
         */
        _unWireEvents: function () {
            $(document.body).unbind("click");
            this._off(this.element, "click", ".menuList");
        },

        /**
        * This function receives the controls update from service-end which would be utilized for rendering the widget.
        * @return jQuery
        * @example 
        * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Chart
        * $('#OlapChart').ejOlapChart({
        *       url: "OlapChartService.svc",
		*		animation: true, type: ej.OlapChart.chartTypes.Column, 
        *                        commonSeriesOptions: { type: ej.OlapChart.chartTypes.Column, tooltip: { visible: true} },
        *                        size: { height: 460, width: 950 }, primaryXAxis: { title: { text: "Fiscal Year" }, labelRotation: 0 },
        *                        primaryYAxis: { title: { text: "Customer Count"} }, legend: { visible: true, rowCount: 2 },
        *                        load: "loadTheme"
        *   });
        * var chartObj = $("#OlapChart").data("ejOlapChart");
        * chartObj.renderControlSuccess({"OlapReport": this.getOlapReport(), "JsonRecords": this.getJSONRecords()});
        * // creating OLAP Chart after Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapChart
        * @instance
        */
        renderControlSuccess: function (msg) {
            if (ochartProxy.model.progressMode == "progress")
                $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
            else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null)
                if ($("#" + oclientProxy._id + "_maxView")[0])
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ autoDisplay: true });
                else
                    oclientWaitingPopup.show();
            try {
                if (ochartProxy.model.progressMode != "infinite") {
                    window.clearInterval(oChartTimer);
                    if (ochartProxy.model.progressMode == "progress")
                        $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("RenderingStarted") + "...";
                    ochartProxy._progressStatus(75, 98, 50);
                    window.setTimeout(function () { progressStatus() }, 400);
                }
                else window.setTimeout(function () { progressStatus() }, 1);

                function progressStatus() {
                    if (msg[0] != undefined) {
                        ochartProxy.setJsonRecords(msg[0].Value); ochartProxy.setOlapReport(msg[1].Value);
                        if (typeof (oclientProxy) != "undefined") {
                            oclientProxy.currentReport = msg[1].Value;
                            if (msg[2] != null && msg[2] != undefined && msg[2].Key == "ClientReports")
                                oclientProxy.reports = msg[2].Value;
                        }
                        if (msg[2] != null && msg[2] != undefined && !msg[2].Key == "ClientReports")
                            ochartProxy.model.customObject = msg[2].Value;
                    }
                    else if (msg.d != undefined) {
                        ochartProxy.setJsonRecords(msg.d[0].Value); ochartProxy.setOlapReport(msg.d[1].Value);
                        if (typeof (oclientProxy) != "undefined") {
                            oclientProxy.currentReport = msg.d[1].Value;
                            if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "ClientReports")
                                oclientProxy.reports = msg.d[2].Value;
                        }
                        if (msg.d[2] != null && msg.d[2] != undefined && !msg.d[2].Key == "ClientReports")
                            ochartProxy.model.customObject = msg.d[2].Value;
                    }
                    else {
                        ochartProxy.setJsonRecords(msg.JsonRecords); ochartProxy.setOlapReport(msg.OlapReport);
                        if (msg.customObject != null && msg.customObject != null)
                            ochartProxy.model.customObject = msg.customObject;
                        if (typeof (oclientProxy) != "undefined") {
                            if (typeof (oclientProxy.currentReport) != "undefined")
                                oclientProxy.currentReport = msg.OlapReport;
                            if (typeof (oclientProxy.reports) != "undefined")
                                oclientProxy.reports = msg.reports;
                        }
                    }
                    if (ochartProxy.model.afterServiceInvoke != null) {
                        var eventArgs;
                        if (ochartProxy._drillAction != "")
                            eventArgs = { Action: ochartProxy._drillAction, Element: ochartProxy.element, CustomObject: ochartProxy.model.customObject };
                        else
                            eventArgs = { Action: ochartProxy._currentAction, Element: ochartProxy.element, CustomObject: ochartProxy.model.customObject };
                        ochartProxy._trigger("afterServiceInvoke", eventArgs);
                    }
                    var htmlTag = ej.buildTag("div#" + ochartProxy._id + "Container")[0].outerHTML;
                    ochartProxy.element.html(htmlTag);
                    ochartProxy.renderChartFromJSON(ochartProxy.getJsonRecords());
                    if (ochartProxy.model.progressMode != "infinite") {
                        ochartProxy._progressStatus(100, 101, 50);
                        if (ochartProxy.model.progressMode == "progress")
                            $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("RenderingSucceeded") + "...";
                    }
                    ochartProxy._unWireEvents();
                    ochartProxy._wireEvents();
                    if (ochartProxy._drillAction != "")
                        ochartProxy._trigger("drillSuccess", ochartProxy.element);
                    if (ochartProxy.model.progressMode != "infinite" && (ochartProxy._JSONRecords == null || ochartProxy._JSONRecords.chartLables.length == 0 || ochartProxy._JSONRecords.points_Y.length == 0)) {
                        $("#" + ochartProxy._id + "progressContainer").hide();
                    }
                    else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0]) {
                            $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ autoDisplay: false });
                            oclientWaitingPopup.hide();
                        }
                        else if (typeof oclientProxy.ogridObj != "undefined") {
                            if (oclientProxy && (oclientProxy.ogridObj._drillAction && !oclientProxy.ogridObj._startDrilldown) || oclientProxy.ochartObj._drillAction && !oclientProxy.ochartObj._startDrilldown)
                                oclientWaitingPopup.hide();
                            else if (oclientProxy && oclientProxy.ogridObj._drillAction == "" && oclientProxy.ochartObj._drillAction == "" && !oclientProxy.ogridObj._startDrilldown && !oclientProxy.ochartObj._startDrilldown)
                                oclientWaitingPopup.hide();
                            else if (oclientProxy.ochartObj._startDrilldown && !oclientProxy.ogridObj._startDrilldown && !$("#" + oclientProxy._id + "_maxView")[0])
                                oclientWaitingPopup.show();
                            else if (!oclientProxy.ochartObj._startDrilldown && !oclientProxy.ogridObj._startDrilldown && oclientProxy.ochartObj._drillAction == "" && oclientProxy.ogridObj._drillAction == "")
                                oclientWaitingPopup.hide();
                        }
                        if (oclientProxy.model.displayOptions.mode == "chartOnly")
                            oclientWaitingPopup.hide();
                    }
                    else
                        ochartWaitingPopup.hide();
                    if (typeof oclientProxy != 'undefined')
                        oclientProxy.ochartObj._startDrilldown = false;
                }
            }
            catch (err) {
                if (ochartProxy.model.progressMode != "infinite") {
                    if (ochartProxy.model.progressMode == "progress")
                        $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("RenderingFailed") + "!!!";
                    window.clearInterval(oChartTimer);
                    ochartProgressBar.setModel({ value: 70, text: " Failed" });
                }
            }
        },

        /**
        * This function receives the JSON formatted datasource to render the OLAP Chart control.
        * @return jQuery
        * @example 
        * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Chart
        * $('#OlapChart').ejOlapChart({
        *       url: "OlapChartService.svc",
		*		animation: true, type: ej.OlapChart.chartTypes.Column, 
        *                        commonSeriesOptions: { type: ej.OlapChart.chartTypes.Column, tooltip: { visible: true} },
        *                        size: { height: 460, width: 950 }, primaryXAxis: { title: { text: "Fiscal Year" }, labelRotation: 0 },
        *                        primaryYAxis: { title: { text: "Customer Count"} }, legend: { visible: true, rowCount: 2 },
        *                        load: "loadTheme"
        *   });
        * var chartObj = $("#OlapChart").data("ejOlapChart");
        * chartObj.renderControlFromJSON(this.getJSONRecords());
        * // render the OLAP Chart from JSON formatted data.
        * &lt;/script&gt;
        * @memberof ejOlapChart
        * @instance
        */
        renderChartFromJSON: function (jsonData) {
            window.clearInterval(oChartTimer);
            if (jsonData != null && jsonData.chartLables.length > 0 && jsonData.points_Y.length > 0) {
                var chartSeries = new Array(); var seriesPoints = new Array(); var XValues = jsonData.chartLables; var tempArray = new Array();
                var YValues = new Array(); var pointsCount = 0; var count = 0; var seriesName = new Array(); var points;

                jQuery.each(jsonData.chartLables, function (index, value) {
                    var YPoints = new Array();
                    for (var i = 0; i < jsonData.points_Y[index].length; i++) {
                        points = { "xValues": jsonData.chartLables[index], "yValues": (jsonData.points_Y[index])[i] };
                        YPoints.push(points);
                    }
                    seriesPoints.push(YPoints);
                });

                jQuery.each(jsonData.seriesNames, function (index, value) {
                    tempArray.push(new Array());
                });
                jQuery.each(jsonData.chartLables, function (index, value) {
                    for (var i = 0; i < seriesPoints[index].length; i++) {
                        tempArray[i].push((seriesPoints[index])[i]);
                    }
                });
                jQuery.each(jsonData.seriesNames, function (index, value) {
                    chartSeries[pointsCount] = { dataSource: { data: tempArray[index], xName: "xValues", yName: ["yValues"] }, name: jsonData.seriesNames[pointsCount] };
                    pointsCount++;
                });
                if (jsonData.chartLables.length > 10 && this.model.zooming == "")
                    this.model.zooming = { enable: true, type: 'x,y', enableMouseWheel: true };
                else if (jsonData.chartLables.length < 10 && this._initZooming == "")
                    this.model.zooming = this._initZooming;
                if (jQuery.inArray(this.seriesType(), ["line", "spline", "area", "splinearea", "stepline", "steparea", "stackingarea"]) > -1 && !this.model.commonSeriesOptions.marker.visible)
                    this.model.commonSeriesOptions.marker = {
                        shape: ej.OlapChart.symbolShapes.Circle,
                        size: { height: 12, width: 12 },
                        visible: true,
                        connectorLine: { height: 30, type: "line" },
                        border: { width: 3, color: 'white' }
                    };
                else if ((this.seriesType() == "pie" || this.seriesType() == "pyramid")) {
                    this.model.commonSeriesOptions.marker = {
                        dataLabel: {
                            visible: true,
                            font: { color: 'white', size: '15px', fontWeight: 'lighter' }
                        }
                    };
                }
                else if (jQuery.inArray(this.seriesType(), ["column", "stackingcolumn", "bar", "stackingcolumn"]) > -1 && this.model.commonSeriesOptions.marker) {
                    if (this.model.commonSeriesOptions.marker.dataLabel)
                        this.model.commonSeriesOptions.marker.dataLabel.visible = false;
                    this.model.commonSeriesOptions.marker.visible = false;
                }
                if (this.model.crosshair.visible) {
                    this.model.commonSeriesOptions.tooltip.format = "#point.x# : #point.y#";
                }
                else
                    this.model.commonSeriesOptions.tooltip.format = (this._getLocalizedLabels("Measure")) + " : " + jsonData.measureNames + " <br/>" + (this._getLocalizedLabels("Row")) + " : #series.name# <br/>" + (this._getLocalizedLabels("Column")) + " : #point.x# <br/>" + (this._getLocalizedLabels("Value")) + " : #point.y#";
                if (this.model.progressMode != "infinite") {
                    ochartProxy._progressStatus(99, 100 + ":100%", 70);
                    window.setTimeout(function () { progressStatus() }, 300);
                }
                else
                    window.setTimeout(function () { progressStatus() }, 1);
                function progressStatus() {
                    $("#" + ochartProxy._id + "Container").ejChart(
                            {
                                border: ochartProxy.model.border,
                                chartArea: ochartProxy.model.chartArea,
                                primaryXAxis: ochartProxy.model.primaryXAxis,
                                primaryYAxis: ochartProxy.model.primaryYAxis,
                                secondaryX: ochartProxy.model.secondaryX,
                                secondaryY: ochartProxy.model.secondaryY,
                                striplineDefault: ochartProxy.model.striplineDefault,
                                title: {
                                    text: ochartProxy.titleText(),
                                    textAlignment: ochartProxy.model.title.textAlignment,
                                    font: {
                                        color: ochartProxy.model.title.color,
                                        fontFamily: ochartProxy.model.title.fontFamily,
                                        fontWeight: ochartProxy.model.title.fontWeight,
                                        opacity: ochartProxy.model.title.opacity,
                                        size: ochartProxy.model.title.size,
                                        fontStyle: ochartProxy.model.title.fontStyle
                                    }
                                },
                                lineCap: ochartProxy.model.lineCap,
                                lineJoin: ochartProxy.model.lineJoin,
                                legendAlignment: ochartProxy.model.legendAlignment,
                                legendPosition: ochartProxy.model.legendPosition,
                                legend: ochartProxy.model.legend,
                                animation: ochartProxy.model.animation,
                                crosshair: ochartProxy.model.crosshair,
                                commonSeriesOptions: {
                                    doughnutCoefficient: ochartProxy.model.commonSeriesOptions.doughnutCoefficient,
                                    explodeOffset: ochartProxy.model.commonSeriesOptions.explodeOffset,
                                    pyramidMode: ochartProxy.model.commonSeriesOptions.pyramidMode,
                                    gapRatio: ochartProxy.model.commonSeriesOptions.gapRatio,
                                    pieCoefficient: ochartProxy.model.commonSeriesOptions.pieCoefficient,
                                    doughnutSize: ochartProxy.model.commonSeriesOptions.doughnutSize,
                                    startAngle: ochartProxy.model.commonSeriesOptions.startAngle,
                                    xAxisName: ochartProxy.model.commonSeriesOptions.xAxisName,
                                    yAxisName: ochartProxy.model.commonSeriesOptions.yAxisName,
                                    explodeAll: ochartProxy.model.commonSeriesOptions.explodeAll,
                                    explodeIndex: ochartProxy.model.commonSeriesOptions.explodeIndex,
                                    tooltipOptions: ochartProxy.model.commonSeriesOptions.tooltipOptions,
                                    marker: ochartProxy.model.commonSeriesOptions.marker || ochartProxy.model.marker,
                                    font: ochartProxy.model.commonSeriesOptions.font,
                                    type: ochartProxy.seriesType(), animation: ochartProxy.model.animation,
                                    style: ochartProxy.model.commonSeriesOptions.style,
                                    explode: ochartProxy.model.commonSeriesOptions.explode,
                                    labelPosition: ochartProxy.model.commonSeriesOptions.labelPosition,
                                    tooltip: ochartProxy.model.commonSeriesOptions.tooltip
                                },
                                seriesStyle: ochartProxy.model.seriesStyle,
                                pointStyle: ochartProxy.model.pointStyle,
                                textStyle: ochartProxy.model.textStyle,
                                initSeriesRender: ochartProxy.model.initSeriesRender,
                                theme: ochartProxy.model.theme,
                                canResize: ochartProxy.model.canResize,
                                zooming: ochartProxy.model.zooming,
                                margin: ochartProxy.model.margin,
                                elementSpacing: ochartProxy.model.elementSpacing,
                                seriesColors: ochartProxy.model.seriesColors,
                                seriesBorderColors: ochartProxy.model.seriesBorderColors,
                                pointColors: ochartProxy.model.pointColors,
                                pointBorderColors: ochartProxy.model.pointBorderColors,
                                series: chartSeries,
                                size: ochartProxy.model.size,
                                //Events
                                load: ochartProxy.model.load,
                                axesRangeCalculate: ochartProxy.model.axesRangeCalculate,
                                axesTitleRendering: ochartProxy.model.axesTitleRendering,
                                chartAreaBoundsCalculate: ochartProxy.model.chartAreaBoundsCalculate,
                                legendItemRendering: ochartProxy.model.legendItemRendering,
                                lengendBoundsCalculate: ochartProxy.model.lengendBoundsCalculate,
                                preRender: ochartProxy.model.preRender,
                                seriesRendering: ochartProxy.model.seriesRendering,
                                symbolRendering: ochartProxy.model.symbolRendering,
                                titleRendering: ochartProxy.model.titleRendering,
                                axesLabelsInitialize: ochartProxy.model.axesLabelsInitialize,
                                pointRegionMouseMove: ochartProxy.model.pointRegionMouseMove,
                                legendItemClick: ochartProxy.model.legendItemClick,
                                legendItemMouseMove: ochartProxy.model.legendItemMouseMove,
                                displayTextRendering: ochartProxy.model.displayTextRendering,
                                toolTipInitialize: ochartProxy.model.toolTipInitialize,
                                trackAxisToolTip: ochartProxy.model.trackAxisToolTip,
                                trackToolTip: ochartProxy.model.trackToolTip,
                                animationComplete: ochartProxy.model.animationComplete,
                                destroy: ochartProxy.model.destroy,
                                create: ochartProxy.model.create,
                                axesLabelRendering: ej.proxy(ochartProxy._onLabelRenders, ochartProxy),
                                pointRegionClick: ej.proxy(ochartProxy._onSeriesClick, ochartProxy)
                            });
                    $("#" + ochartProxy._id + "progressContainer").hide();
                }
                ochartProxy._labelCurrentTags.splice(0, ochartProxy._labelCurrentTags.length);
                jQuery.each(jsonData.labelTags, function (index, value) {
                    var currentTag = new Object();
                    var splitData = value.split("::");
                    currentTag = { "name": splitData[2], "state": splitData[splitData.length - 1], "tag": value };
                    ochartProxy._labelCurrentTags.push(currentTag);
                });
            }
            if (ochartWaitingPopup != null && ochartProxy.model.progressMode == "infinite")
                ochartWaitingPopup.hide();
            else
                $("#" + ochartProxy._id + "progressContainer").hide();
        },

        /**
         * This function would fetch the label text which could be seen beneath each series.
         *
         * @private  
         */
        _onLabelRenders: function (args) {
            ochartProxy._trigger("axesLabelRendering", args);
            if ((args.Axis.orientation.toLowerCase() == "horizontal" && args.Label.Text != undefined) || ((ochartProxy.seriesType() == "bar" || ochartProxy.seriesType() == "stackingbar") && args.Axis.orientation.toLowerCase() == "vertical" && args.Label.Text != undefined)) {
                var splitName = args.Label.Text.toString().split('~');
                if (ochartProxy._tagCollection.length == 0)
                    args.Label.Text = splitName[splitName.length - 1];
                else if (ochartProxy._drillAction == "drilldown" && splitName.indexOf(ochartProxy._selectedItem) + 1 < splitName.length)
                    args.Label.Text = splitName[splitName.indexOf(ochartProxy._selectedItem) + 1];
                else if (ochartProxy._drillAction == "drilldown" && splitName.indexOf(ochartProxy._selectedItem) + 1 >= splitName.length)
                    args.Label.Text = splitName[splitName.indexOf(ochartProxy._selectedItem)];
                else if (splitName.length == 1)
                    args.Label.Text = splitName[0];
                else if (ochartProxy._drillAction == "drillup")
                    args.Label.Text = splitName[ochartProxy._selectedIndex];
                else
                    args.Label.Text = splitName[splitName.length - 1];
            }
        },

        /**
        * To get the localized labels.
        * @private
        */
        //get Localization text
        _getLocalizedLabels: function (property) {

            return ej.OlapChart.localization[this.localization()][property] === undefined ? ej.OlapChart.localization["en-US"][property] : ej.OlapChart.localization[this.localization()][property];
        },

        /**
         * This function is used to create progress bar containing status text and percentage.
         *
         * @private  
         */
        _createProgressBar: function () {
            var progressContainer = ej.buildTag("div#" + this._id + "progressContainer.progressContainer", "", { top: typeof (oclientProxy) != "undefined" ? ((oclientProxy.model.displayOptions.controlPlacement == "tab" || oclientProxy.model.displayOptions.mode != "chartAndGrid") ? oclientProxy.progressPos.top + 300 : (oclientProxy.model.displayOptions.defaultView == "chart" ? oclientProxy.progressPos.top + 200 : oclientProxy.progressPos.top + 500)) : ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 35), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left + 550 : ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 50), zIndex: 999 })[0].outerHTML;
            var progressBar = ej.buildTag("div#" + this._id + "progressBar.progressBar", "")[0].outerHTML;
            var blockControl = ej.buildTag("div#" + this._id + "blockDiv.blockDiv", "", { width: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).width()) : $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).height()) : $("#" + this._id).height(), position: 'fixed', top: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.top : ($("#" + this._id).position().top), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left : ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + ochartProxy._id).after(progressContainer);
            if (ochartProxy.model.progressMode == "normal") {
                $("#" + this._id + "progressContainer").css({ height: 0, width: 0 });
                $("#" + this._id + "progressContainer").removeClass(".progressContainer");
                $("#" + ochartProxy._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + this._id + "progressText.progressText", "OlapChart initial loading...")[0].outerHTML;
                $("#" + ochartProxy._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + ochartProxy._id + "progressBar").ejProgressBar({ value: 0, text: 0 + " %", complete: "progressbarCompleted" });
            ochartProgressBar = $("#" + ochartProxy._id + "progressBar").data("ejProgressBar");
        },

        /**
         * This function is used to update the progress status.
         *
         * @private  
         */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            window.clearInterval(oChartTimer);
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            ochartProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + ochartProxy._id + "progressContainer").show();
            oChartTimer = window.setInterval(function () {
                ochartProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(oChartTimer);
                    if (progressText != undefined)
                        ochartProgressBar.setModel({ value: 100, text: progressText[1] });
                };
            }, progress_Speed);
        },

        /**
         * This function would be raised while clicking the series.
         *
         * @private  
         */
        _onSeriesClick: function (sender) {
            ochartProxy = this; var tempLabel; var selectedLabel; var pointIndex;
            var selectedTags = new Array(); var menuList = new Array();
            var chartOffset = this.element.position();
            if (sender.Data.Region != null && sender.Data.Region != undefined) {
                selectedLabel = sender.model.primaryXAxis.labels[sender.Data.Region.Region.PointIndex];
                pointIndex = sender.Data.Region.Region.PointIndex;
            }
            else if (sender.Data.point.x != null && sender.Data.point.x != undefined && sender.Data.point.x != "") {
                selectedLabel = sender.Data.series.xAxis.labels[sender.Data.point.X];
                pointIndex = sender.Data.point.X;
            }

            if ($($("#" + ochartProxy._id + "Container_vml_XAxisLabels_0")[0]).children()[pointIndex] != undefined)
                tempLabel = $($("#" + ochartProxy._id + "Container_vml_XAxisLabels_0")[0]).children()[pointIndex].innerHTML;
            else if ($("#" + ochartProxy._id + "Container_svg_XAxisLabels_0").children()[pointIndex])
                tempLabel = $("#" + ochartProxy._id + "Container_svg_XAxisLabels_0").children()[pointIndex].textContent;
            else if (this.seriesType() == "bar" || this.seriesType() == "stackingbar") {
                if ($($("#" + ochartProxy._id + "Container_vml_YAxisLabels_0")[0]).children()[pointIndex] != undefined)
                    tempLabel = $($("#" + ochartProxy._id + "Container_vml_YAxisLabels_0")[0]).children()[pointIndex].innerHTML;
                else if ($("#" + ochartProxy._id + "Container_svg_YAxisLabels_0").children()[pointIndex])
                    tempLabel = $("#" + ochartProxy._id + "Container_svg_YAxisLabels_0").children()[pointIndex].textContent;
                else {
                    var temp = sender.model.primaryXAxis.labels[pointIndex].split('~');
                    tempLabel = temp[temp.indexOf(ochartProxy._selectedMenuItem) + 1];
                }
            }
            else {
                var temp = sender.model.primaryXAxis.labels[pointIndex].split('~');
                tempLabel = temp[temp.indexOf(ochartProxy._selectedMenuItem) + 1];
            }

            if (tempLabel) {
                $("#" + ochartProxy._id).find("#" + ochartProxy._id + "ExpandMenu, .expandMenu, .e-dialog").remove();
                var splitLabel = selectedLabel.split('~');

                jQuery.each(splitLabel, function (index, value) {
                    jQuery.each(ochartProxy._labelCurrentTags, function (key, val) {
                        if ($.trim(value) == val.name) {
                            selectedTags.push(val);
                            return false;
                        }
                    });
                });

                jQuery.each(selectedTags, function (index, value) {
                    if (value.state > 1 && ochartProxy._tagCollection.length == 0)
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Expand")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    else if (value.state > 1 && ochartProxy._tagCollection.length > 0 && value.name == tempLabel) {
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Expand")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    }
                });
                if (ochartProxy._tagCollection.length > 0) {
                    jQuery.each(ochartProxy._tagCollection, function (index, value) {
                        for (var i = 0; i < menuList.length; i++) {
                            if ($.trim(menuList[i].split('-')[1].replace("</li>", "")).replace("</LI>", "") == value.name)
                                menuList.splice(i, 1);
                        }
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Collapse")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    });
                }
                else if (!ochartProxy._JSONRecords.addInfo.namedsetEnabled) {
                    jQuery.each(selectedTags, function (index, value) {
                        if (value.state == 1) {
                            menuList.push($(ej.buildTag("li.menuList", ochartProxy._getLocalizedLabels("Collapse") + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                            ochartProxy._tagCollection.push(value);
                        }
                    });
                }
                menuList.push($(ej.buildTag("li.menuList", ochartProxy._getLocalizedLabels("Exit"))[0]).attr("role", "presentation")[0].outerHTML);
                var expandMenu = $(ej.buildTag("div#" + ochartProxy._id + "ExpandMenu.expandMenu", menuList)[0]).attr("role", "presentation")[0].outerHTML;
                $(expandMenu).ejDialog({ width: "auto", contentContainer: "#" + ochartProxy._id, resizable: false });
                $("#" + ochartProxy._id + "ExpandMenu_wrapper").appendTo(this.element).css({ "left": sender.Data.location.X + 8 + chartOffset.left, "top": sender.Data.location.Y + 8 + chartOffset.top });
                this.element.find(".e-titlebar, .e-header").remove();
            }
            ochartProxy._trigger("pointRegionClick", sender);
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Chart
        * $('#OlapChart').ejOlapChart({
        *       url: "OlapChartService.svc",
		*		animation: true, type: ej.OlapChart.chartTypes.Column, 
        *                        commonSeriesOptions: { type: ej.OlapChart.chartTypes.Column, tooltip: { visible: true} },
        *                        size: { height: 460, width: 950 }, primaryXAxis: { title: { text: "Fiscal Year" }, labelRotation: 0 },
        *                        primaryYAxis: { title: { text: "Customer Count"} }, legend: { visible: true, rowCount: 2 },
        *                        load: "loadTheme"
        *   });
        * var chartObj = $("#OlapChart").data("ejOlapChart");
        * chartObj.doAjaxPost("POST", "/OlapChartService.svc/Initialize", {"key", "Hello World"}, "renderControlSuccess", null);
        * // initiate an Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapChart
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess) {
            $.ajax({
                type: type,
                url: url,
                data: data,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                success: $.proxy(onSuccess, this),
                error: function (msg, textStatus, errorThrown) {
                    if (ochartProxy.model.progressMode != "infinite") {
                        if (ochartProxy.model.progressMode == "progress")
                            $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        window.clearInterval(oChartTimer);
                        ochartProgressBar.setModel({ value: 10, text: " Failed" });
                    }
                    else
                        ochartWaitingPopup.hide();
                }
            });
        }
    });

    //OLAP Chart Localization
    ej.OlapChart.localization = {};
    //Default english localized values
    ej.OlapChart.localization["en-US"] = {
        Measure: "Measure",
        Row: "Row",
        Column: "Column",
        Value: "Value",
        Expand: "Expand",
        Collapse: "Collapse",
        Exit: "Exit",
        MDXqueryExecutionFailed: "MDX query execution failed",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingSucceeded: "Rendering Succeeded",
        RenderingStarted: "Rendering Started",
        RenderingFailed: "Rendering failed"
    };

    /**
     * Enum for Chart Types. Allows the user to select a appropriate chart type. 
     * @enum {String}
	 * @global
     */
    ej.OlapChart.chartTypes = {
        Line: 'line', Spline: 'spline', Column: 'column', Area: 'area', SplineArea: 'splinearea', StepLine: 'stepline', StepArea: 'steparea', Pie: 'pie', Bar: 'bar', StackingArea: 'stackingarea', StackingColumn: 'stackingcolumn', StackingBar: 'stackingbar', Pyramid: 'pyramid'
    };

    /**
     * Enum for Progress Mode. Allows the user to select a appropriate progress indicator. 
     * @enum {String}
	 * @global
     */
    ej.OlapChart.progressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

    /**
     * Enum for Shapes. Allows the user to select a appropriate chart symbol shape. 
     * @enum {String}
	 * @global
     */
    ej.OlapChart.symbolShapes = {
        None: "none",
        LeftArrow: "leftArrow",
        RightArrow: "rightArrow",
        Circle: "circle",
        Cross: "cross",
        HorizLine: "horizLine",
        VertLine: "vertLine",
        Diamond: "diamond",
        Rectangle: "rectangle",
        Triangle: "triangle",
        InvertedTriangle: "invertedTriangle",
        Hexagon: "hexagon",
        Pentagon: "pentagon",
        Star: "star",
        Ellipse: "ellipse",
        Wedge: "wedge",
        Trapezoid: "trapezoid",
        UpArrow: "upArrow",
        DownArrow: "downArrow",
        Image: "image"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html OLAP Gauge elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejOlapGauge
	* @requires jQuery
	* @requires ej.common.all
	* @requires ej.tools
	* @requires ej.circulargauge
	* @classdesc Custom Design for Html OLAP Gauge control.
	* @example 
	* &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create OLAP Gauge
    * $("#OlapGauge").ejOlapGauge(...); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejOlapGauge", "ej.OlapGauge", {

        //Root Css Class
        _rootCSS: "e-olapgauge",

        // Widget element will be automatically set in this
        element: null,

        // User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        defaults: {
            /**		
			* Connects the service using the specified URL for any server updates.
			* @default ""
			* @type {string}
			* @example 
			* //To set url API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  url: "/OlapGaugeService.svc"});					* 
			* @example 
			* //Get or set the url API, after initialization:<br>
			*	//Gets the url value  
			*	$("#OlapGauge").ejOlapGauge("option", "url");<br>			
			*	//Sets the url value 
			*	$("#OlapGauge").ejOlapGauge("option", "url",  "/OlapGaugeService" ); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            url: "",
            /**		
			* Specify the CSS class to OLAP Gauge to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* // Set the CSS class during initialization. 			
			* 	$("#OlapGauge").ejOlapGauge({  cssClass : "gradient-lime" });			* 
			* @example 
			* //Get or set the CSS class after initialization:<br>
			*	// Gets the CSS class value.		
			*	$("#OlapGauge").ejOlapGauge("option", "cssClass");			
			*	// Sets the CSS class to OLAP Gauge
			*	$("#OlapGauge").ejOlapGauge("option", "cssClass",  "gradient-lime" );
			 * @memberof ejOlapGauge
			* @instance
			*/
            cssClass: "",
            /**		
			* Sets the number of rows in OLAP Gauges for its arrangement.
			* @default 0
			* @type {number}
			* @example 
			* //To set rowsCount API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  rowsCount: 1});					* 
			* @example 
			* //Get or set the rowsCOunt API, after initialization:<br>
			*	//Gets the rowsCount value  
			*	$("#OlapGauge").ejOlapGauge("option", "rowsCount");<br>			
			*	//Sets the rowsCount value 
			*	$("#OlapGauge").ejOlapGauge("option", "rowsCount", 2); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            rowsCount: 0,
            /**		
			* Sets the number of colunsCount in OLAP Gauges for its arrangement.
			* @default 0
			* @type {number}
			* @example 
			* //To set columnsCount API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  columnsCount: 1});					* 
			* @example 
			* //Get or set the columnsCount API, after initialization:<br>
			*	//Gets the columnsCount value  
			*	$("#OlapGauge").ejOlapGauge("option", "columnsCount");<br>			
			*	//Sets the columnsCount value 
			*	$("#OlapGauge").ejOlapGauge("option", "columnsCount", 2 ); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            columnsCount: 0,
            /**		
			* Enables/disables tooltip in OLAP Gauge.
			* @default false
			* @type {boolean}
			* @example 
			* //To set showTooltip API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  showTooltip: true});					* 
			* @example 
			* //Get or set the showTooltip API, after initialization:<br>
			*	//Gets the showTooltip value  
			*	$("#OlapGauge").ejOlapGauge("option", "showTooltip");<br>			
			*	//Sets the showTooltip value 
			*	$("#OlapGauge").ejOlapGauge("option", "showTooltip", false ); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            showTooltip: false,
            /**		
			* Enables/disables the header labels in OLAP Gauge.
			* @default true
			* @type {boolean}
			* @example 
			* //To set showHeaderLabels API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  showHeaderLabels: false});					* 
			* @example 
			* //Get or set the showHeaderLabels API, after initialization:<br>
			*	//Gets the showHeaderLabels value  
			*	$("#OlapGauge").ejOlapGauge("option", "showHeaderLabels");<br>			
			*	//Sets the showHeaderLabels value 
			*	$("#OlapGauge").ejOlapGauge("option", "showHeaderLabels", true); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            showHeaderLabels: true,
            /**		
			* Sets the scale values such as pointers, indicators, etc... for OLAP Gauge.
			* @default null
			* @type {object}
			* @example 
			* //To set scales API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  scales: {showRanges: true, showIndicators: true}});					* 
			* @example 
			* //Get or set the scales API, after initialization:<br>
			*	//Gets the scales value  
			*	$("#OlapGauge").ejOlapGauge("option", "scales");<br>			
			*	//Sets the scales value 
			*	$("#OlapGauge").ejOlapGauge("option", "scales", {showRanges: true, showIndicators: true}); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            scales: ej.CircularGauge.prototype._defaultScaleValues(),
            /**		
			* Custom object to pass additional information between client-end and service-end.
			* @default null
			* @type {object}
			* @example 
			* //To set customObject API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  customObject: {"key":"Hello World"}});					* 
			* @example 
			* //Get or set the customObject API, after initialization:<br>
			*	//Gets the customObject value  
			*	$("#OlapGauge").ejOlapGauge("option", "customObject");<br>			
			*	//Sets the customObject value 
			*	$("#OlapGauge").ejOlapGauge("option", "customObject",  {"key":"Hi Syncfusion!"} ); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            customObject: {},
            /**		
			* Sets the progress indicator type which arises on any UI operation.	See {@link progressMode}
			* @default ej.OlapGauge.progressMode.Infinite
			* @type {enum}
			* @example 
			* //To set progressMode API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  progressMode: ej.OlapGauge.progressMode.Normal});					* 
			* @example 
			* //Get or set the progressMode API, after initialization:<br>
			*	//Gets the progressMode value  
			*	$("#OlapGauge").ejOlapGauge("option", "progressMode");<br>			
			*	//Sets the progressMode value 
			*	$("#OlapGauge").ejOlapGauge("option", "progressMode",  ej.OlapGauge.progressMode.Progress ); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            progressMode: "infinite",
            /**		
			* Sets the localized language for the control.
			* @default "en-US"
			* @type {string}
			* @example 
			* //To set localization API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  localization: "fr-FR"});					* 
			* @example 
			* //Get or set the localization API, after initialization:<br>
			*	//Gets the localization value  
			*	$("#OlapGauge").ejOlapGauge("option", "localization");<br>			
			*	//Sets the localization value 
			*	$("#OlapGauge").ejOlapGauge("option", "localization",  "fr-FR" ); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            localization: "en-US",
            /**		
			* Allows the user to set custom name for the methods at service-end invoked on AJAX post.
			* @default null
			* @type {object}
			* @example 
			* //To set serviceMethods API value during initialization  
			* 	$("#OlapGauge").ejOlapGauge({  serviceMethods: {initialize: "MyMethod"});					* 
			* @example 
			* //Get or set the serviceMethods API, after initialization:<br>
			*	//Gets the serviceMethods value  
			*	$("#OlapGauge").ejOlapGauge("option", "serviceMethods");<br>			
			*	//Sets the serviceMethods value 
			*	$("#OlapGauge").ejOlapGauge("option", "serviceMethods",  {initialize: "MyMethod"} ); 
			 * @memberof ejOlapGauge
			* @instance
			*/
            serviceMethods: { initialize: "InitializeOlapGauge" },
            /**     
			 * Fires when OLAP Gauge control successfully reaches client script after any Ajax request.
			 * @event
			 * @name ejOlapGauge#clientSuccess 	
			 * @param {Object} argument Event parameters from OLAP Gauge.   
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Gauge model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //clientSuccess event for OLAP Gauge.
             * $("#OlapGauge").ejOlapGauge({
             *    clientSuccess: function (args) {}
             * });      
			 * @memberof ejOlapGauge
			 * @instance
			 */
            clientSuccess: null,
            /**     
			 * Fires when OLAP Gauge control completes all operations at client script after any Ajax request.
			 * @event
			 * @name ejOlapGauge#clientComplete 	
			 * @param {Object} argument Event parameters from OLAP Gauge.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Gauge model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //clientComplete event for OLAP Gauge
             * $("#OlapGauge").ejOlapGauge({
             *    clientComplete: function (args) {}
             * });      
			 * @memberof ejOlapGauge
			 * @instance
			 */
            clientComplete: null,
            /**     
			 * Fires when error occurs during Ajax request.
			 * @event
			 * @name ejOlapGauge#clientError 	
			 * @param {Object} argument Event parameters from OLAP Gauge.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Gauge model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.responseJSON returns the JSON formatted response while error occurs.
			 * @example 
			 * //clientError event for OLAP Gauge
             * $("#OlapGauge").ejOlapGauge({
             *    clientError: function (args) {}
             * });      
			 * @memberof ejOlapGauge
			 * @instance
			 */
            clientError: null,
            /**     
			 * Fires before any Ajax request passed from OLAP Gauge to service methods.
			 * @event
			 * @name ejOlapGauge#beforeServiceInvoke 	
			 * @param {Object} argument Event parameters from OLAP Gauge.
			 * @param {string}  argument.Action return the current action of OLAP Gauge control.			 
			 * @param {object}  argument.CustomObject return the custom object bounds with OLAP Gauge control.
			 * @param {string}  argument.Element return the outer HTML of OLAP Gauge control.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Gauge model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //beforeServiceInvoke event for OLAP Gauge
             * $("#OlapGauge").ejOlapGauge({
             *    beforeServiceInvoke: function (args) {}
             * });      
			 * @memberof ejOlapGauge
			 * @instance
			 */
            beforeServiceInvoke: null,
            /**     
			 * Fires when it reaches client script after any Ajax request.
			 * @event
			 * @name ejOlapGauge#afterServiceInvoke 	
			 * @param {Object} argument Event parameters from OLAP Gauge.
			 * @param {string}  argument.Action return the current action of OLAP Gauge control.			 
			 * @param {object}  argument.CustomObject return the custom object bounds with OLAP Gauge control.
			 * @param {string}  argument.Element return the outer HTML of OLAP Gauge control.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Gauge model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //afterServiceInvoke event for OLAP Gauge
             * $("#OlapGauge").ejOlapGauge({
             *    afterServiceInvoke: function (args) {}
             * });      
			 * @memberof ejOlapGauge
			 * @instance
			 */
            afterServiceInvoke: null,
            /**     
			 * Fires when gauge started loading at client-side.
			 * @event
			 * @name ejOlapGauge#load 	
			 * @param {Object} argument Event parameters from OLAP Gauge.  
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Gauge model.
			 * @param {string}  argument.type returns the name of the event.			 
			 * @example 
			 * //load event for OLAP Gauge
             * $("#OlapGauge").ejOlapGauge({
             *    load: function (args) {}
             * });      
			 * @memberof ejOlapGauge
			 * @instance
			 */
            load: null
        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            serviceMethods: "data",
            customObject: "data",
            scales: "data",
            progressMode: "enum"
        },

        observables: ["rowsCount", "columnsCount", "showHeaderLabels", "localization", "radius", "frameType"],
        rowsCount: ej.util.valueFunction("rowsCount"),
        columnsCount: ej.util.valueFunction("columnsCount"),
        showHeaderLabels: ej.util.valueFunction("showHeaderLabels"),
        localization: ej.util.valueFunction("localization"),
        radius: ej.util.valueFunction("radius"),
        frameType: ej.util.valueFunction("frameType"),

        /**
        * Gets the serialized current OLAP Report of the control.
        *
        * @private 
        */
        //get the current OLAP Report
        getOlapReport: function () {
            return this._olapReport;
        },

        /**
        * Sets the serialized current OLAP Report of the control.
        *
        * @private 
        */
        //set the current OLAP Report
        setOlapReport: function (value) {
            this._olapReport = value;
        },

        /**
        * Gets the JSON formatted records for the control.
        *
        * @private 
        */
        //get the JSON formatted records
        getJSONRecords: function () {
            return this._JSONRecords;
        },

        /**
        * Sets the JSON formatted records for the control.
        *
        * @private 
        */
        //set the JSON formatted records
        setJSONRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        // constructor function
        /**
        * Initializes the variables and loads the widget. 
        *
        * @private
        */
        _init: function () {
            this.model = $.extend({}, ej.CircularGauge.prototype.defaults, this.model);
            this._initPrivateProperties();
            this._load();
        },

        /**
        * Destroy the OLAP Gauge widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="OlapGauge1"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Create OLAP Gauge
		* var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
		* gaugeObj.destroy(); // destroy the OLAP Gauge
		* &lt;/script&gt;
		* @example 
		* &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // enable the OLAP Gauge
		* $("#OlapGauge").ejOlapGauge("destroy");	
		* &lt;/script&gt;
		* @memberof ejOlapGauge
		* @instance
        */
        _destroy: function () {
            this._unWireEvents();
            this.element.empty().removeClass("e-olapgauge" + this.model.cssClass);
        },

        /**
        * To configure the private properties of OLAP Gauge		
        * @private
        */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            this._olapReport = "";
            this._JSONRecords = null;
            this._maximum = new Array();
            this._maxValue = 0;
            this._pointerValue_0 = 0;
            this._pointerValue_1 = 0;
            this._currentLayout = "";
            this._defaultArea = 0;
            this._defaultWidth = 0;
            this._gaugeObj = {};
            this._ogaugeWaitingPopup = null;
            this._ogaugeProgressBar = null;
            this._ogaugeProgressText = null;
            this._ogaugeTimer = null;
        },

        /**
        * This function is the point were loading/rendering of the OLAP Gauge widget is initiated.
        *
        * @private  
        */
        _load: function () {
            this.element.addClass(this.model.cssClass);
            if (this.model.progressMode == "infinite") {
                $("#" + this._id).ejWaitingPopup({ autoDisplay: true });
                this._ogaugeWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
            }
            else {
                $("#" + this._id + "progressContainer").remove();
                this._createProgressBar();
                this._progressStatus(0, 70, 140);
            }

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { Element: this.element, CustomObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (this.model.progressMode == "progress")
                $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("PreparingAndExecutingMDXquery");
            if (this.model.customObject != null && this.model.customObject != undefined && this.model.customObject != {})
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({ "action": "initializeGauge", "customObject": serializedCustomObject }), this._renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethods.initialize, JSON.stringify({ "action": "initializeGauge" }), this._renderControlSuccess);
        },

        _setFirst: true,

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                    case "JsonData": this.setOlapReport(options[key]); break;
                    case "RefreshOlapGauge": this.element.renderControlFromJSON(options[key]); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "localization": this.localization(); break;
                    case "rowsCount": this.rowsCount(); break;
                    case "columnsCount": this.columnsCount(); break;
                    case "showHeaderLabels": this.showHeaderLabels(); break;
                    case "localization": this.localization(); break;
                    case "radius": this.radius(); break;
                    case "frameType": this.frameType(); break;
                    case "scales": break;
                }
            }
            if (this._gaugeObj["obj0"]) {
                this._renderControlSuccess({ "PivotRecords": this.getJSONRecords(), "OlapReport": this.getOlapReport() });
            }
        },

        /**
        * This function is invoked to wire the events for UI interaction.
        *
        * @private  
        */
        _wireEvents: function () {
            if (this.model.showTooltip) {
                this.element.find("canvas").mouseenter(ej.proxy(function (evt) {
                    this._showTooltip(evt);
                }, this)).mouseout(ej.proxy(function () {
                    $(this._tooltip).fadeOut("slow");
                }, this)).mousemove(ej.proxy(function (evt) {
                    this._showTooltip(evt);
                    this._hideTooltip();
                }, this));
            }
        },

        /**
        * To get the localized labels.
        * @private
        */
        //get Localization text
        _getLocalizedLabels: function (property) {
            return ej.OlapGauge.localization[this.localization()][property] === undefined ? ej.OlapGauge.localization["en-US"][property] : ej.OlapGauge.localization[this.localization()][property];
        },

        /**
        * This function is invoked to unwire the events registered for UI interaction.
        *
        * @private  
        */
        _unWireEvents: function () {

        },

        /**
        * This function is used to initialize and manage the tooltip in OLAP Gauges.
        *
        * @private  
        */
        _showTooltip: function (event) {
            $(this._tooltip).remove();
            var gaugeIndexValues = event.target.parentNode.id.split("_") == "" ? event.target.parentNode.parentNode.id.split("_") : event.target.parentNode.id.split("_");
            var gaugeIndex = gaugeIndexValues[gaugeIndexValues.length - 1];
            var goal = this._JSONRecords[gaugeIndex].GoalValue || 0;
            var value = this._JSONRecords[gaugeIndex].MeasureValue || this._JSONRecords[gaugeIndex].Value;
            var member = this._JSONRecords[gaugeIndex].MemberName || this._JSONRecords[gaugeIndex].Measure;
            var tooltipText = member + "<br/>" + this._getLocalizedLabels("RevenueGoal") + ": " + goal + "<br/>" + this._getLocalizedLabels("RevenueValue") + ": " + value;
            this._tooltip = ej.buildTag("div.e-olapgauge-tooltip", tooltipText);
            $(this._tooltip).appendTo('body');
            $(this._tooltip).addClass(".e-olapgauge-active");
            this._changeTooltipPos(event);
        },

        /**
        * This function is used to update the tooltip while mouse moves between OLAP Gauges.
        *
        * @private  
        */
        _changeTooltipPos: function (event) {
            var tooltipX = event.pageX - 8;
            var tooltipY = event.pageY + 8;
            $(this._tooltip).css({ top: tooltipY, left: tooltipX });
        },

        /**
        * 	This function is used to hide the tooltip while mouse leaves the OLAP Gauge.
        *
        * @private  
        */
        _hideTooltip: function () {
            window.setTimeout(ej.proxy(function () { $(this._tooltip).fadeOut(1500, "linear"); }, this), 3000);
        },

        /**
        * This function is used to create progress bar containing status text and percentage.
        *
        * @private  
        */
        _createProgressBar: function () {
            var progressContainer = ej.buildTag("div#" + this._id + "progressContainer.progressContainer", "", { top: ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 30), left: ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 50) })[0].outerHTML;
            var progressBar = ej.buildTag("div#" + this._id + "progressBar.progressBar", "")[0].outerHTML;
            var blockControl = ej.buildTag("div#" + this._id + "blockDiv.blockDiv", "", { width: $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: $("#" + this._id).height(), position: 'fixed', top: ($("#" + this._id).position().top), left: ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + this._id).after(progressContainer);
            if (this.model.progressMode == "normal") {
                $("#" + this._id + "progressContainer").css({ width: 0, height: 0 });
                $("#" + this._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + this._id + "progressText.progressText", "OlapGauge initial loading...")[0].outerHTML;
                $("#" + this._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + this._id + "progressBar").ejProgressBar({ value: 0, text: 0 + " %", complete: "progressbarCompleted" });
            this._ogaugeProgressBar = $("#" + this._id + "progressBar").data("ejProgressBar");
            this._ogaugeProgressText = $("#" + this._id + "progressText");
        },

        /**
        * This function is used to update the progress status.
        *
        * @private  
        */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            this._ogaugeProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + this._id + "progressContainer").show();
            $("#" + this._id + "blockDiv").position = $("#" + this._id).position();
            this._ogaugeTimer = window.setInterval(ej.proxy(function () {
                this._ogaugeProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(this._ogaugeTimer);
                    if (progressText != undefined)
                        this._ogaugeProgressBar.setModel({ value: 100, text: progressText[1] });

                };
            }, this), progress_Speed);
        },

        /**
        * This function receives the control's update from service-end which would be utilized for rendering the widget.
        *
        * @private  
        */
        _renderControlSuccess: function (msg) {
            if (this.model.progressMode != "infinite") {
                if (this.model.progressMode == "progress")
                    $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
                window.clearInterval(this._ogaugeTimer);
            }
            try {
                if (this.model.progressMode != "infinite") {
                    if (this.model.progressMode == "progress")
                        $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("RenderingStarted") + "...";
                    this._progressStatus(70, 100, 50);
                    window.setTimeout(ej.proxy(this.progressStatus(msg), this), 300);
                }
                else
                    window.setTimeout(ej.proxy(this.progressStatus(msg), this), 1);
            }
            catch (err) {
                if (this.model.progressMode != "infinite") {
                    if (this.model.progressMode == "progress")
                        $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("RenderingFailed") + "!!!";
                    window.clearInterval(this._ogaugeTimer);
                    this._ogaugeProgressBar.setModel({ value: 70, text: " Failed" });
                }
            }
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
		*		showTooltip: true,
		*		scales: [{
		*		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            pointerLength: 120,
        *                            pointerWidth: 7,
        *                            capBorderColor: "#f5b43f",
        *                            capBackgroundColor: "#f5b43f"
        *                        },
        *                {
        *                    pointerType: "Marker",
        *                    markerStyle: "Diamond",
        *                    distanceFromScale: 5,
        *                    pointerPosition: "Center",
        *                    backgroundColor: "#29A4D9",
        *                    pointerLength: 25,
        *                    pointerWidth: 15
        *                }]
		*				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.doAjaxPost("POST", "/OlapGaugeService.svc/Initialize", {"key", "Hello World"}, "renderControlSuccess", null);
        * // initiate an Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function () {
                    this._trigger("clientComplete", this);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    if (this.model.progressMode != "infinite") {
                        window.clearInterval(this._ogaugeTimer);
                        if (this.model.progressMode == "progress")
                            $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        $("#" + this._ogaugeProgressBar._id).show();
                        this._ogaugeProgressBar.setModel({ value: 0, text: errorThrown });
                    }
                    else
                        this._ogaugeWaitingPopup.hide();
                    this._trigger("clientError", msg);
                }, this)
            });
        },

        /**
        * This function receives JSON data and prepares for rendering the widget after service request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
		*		showTooltip: true,
		*		scales: [{
		*		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            pointerLength: 120,
        *                            pointerWidth: 7,
        *                            capBorderColor: "#f5b43f",
        *                            capBackgroundColor: "#f5b43f"
        *                        },
        *                {
        *                    pointerType: "Marker",
        *                    markerStyle: "Diamond",
        *                    distanceFromScale: 5,
        *                    pointerPosition: "Center",
        *                    backgroundColor: "#29A4D9",
        *                    pointerLength: 25,
        *                    pointerWidth: 15
        *                }]
		*				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.progressStatus({"OlapReport": this.getOlapReport(), "JsonRecords": this.getJSONRecords()});
        * // creating OLAP Gauge after Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        progressStatus: function (msg) {
            var htmlTag = "", stausImage = "", trendImage = "", liTag = "", gaugeId = 0;
            if (msg[0] != undefined && msg.length > 0) {
                this.setJSONRecords(msg[0].Value);
                this.setOlapReport(msg[1].Value);
                if (msg[2] != null && msg[2] != undefined)
                    this.model.customObject = msg[2].Value;
            }
            else if (msg.d != undefined && msg.d.length > 0) {
                this.setJSONRecords(msg.d[0].Value);
                this.setOlapReport(msg.d[1].Value);
                if (msg.d[2] != null && msg.d[2] != undefined)
                    this.model.customObject = msg.d[2].Value;
            }
            else if (msg.length > 0) {
                this.setJSONRecords(msg.PivotRecords);
                this.setOlapReport(msg.OlapReport);
                if (msg.customObject != null && msg.customObject != undefined)
                    this.model.customObject = msg.customObject;
            }

            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { Element: this.element, CustomObject: this.model.customObject });
            var ogaugeInfo = this.getJSONRecords();

            if (ogaugeInfo != null && ogaugeInfo.length > 0) {
                if (this.rowsCount() < 1 && this.columnsCount() < 1) {
                    this._currentLayout = "WrapPanel";
                    for (var k = 0; k < ogaugeInfo.length; k++) {
                        stausImage = ej.buildTag("img#stausIndicator_" + k + ".kpiiconvalue")[0].outerHTML;
                        trendImage = ej.buildTag("img#trendIndicator_" + k + ".kpiiconvalue")[0].outerHTML;
                        liTag += ej.buildTag("li", ej.buildTag("div#" + this._id + "_" + k, stausImage + trendImage)[0].outerHTML)[0].outerHTML;
                    }
                    htmlTag = ej.buildTag("ul.wrapLayout", liTag)[0].outerHTML;
                }
                else {
                    this._currentLayout = "Table";
                    if (this.columnsCount() == undefined || this.columnsCount() < 1)
                        this.columnsCount((ogaugeInfo.length / this.rowsCount()).toFixed());
                    if (this.rowsCount() == undefined || this.rowsCount() < 1)
                        this.rowsCount((ogaugeInfo.length / this.columnsCount()).toFixed());
                    htmlTag = "<table><tbody>";
                    for (var i = 0; i < this.rowsCount() ; i++) {
                        htmlTag += "<tr>";
                        for (var j = 0; j < this.columnsCount() ; j++) {
                            stausImage = ej.buildTag("img#stausIndicator_" + gaugeId + ".kpiiconvalue")[0].outerHTML;
                            trendImage = ej.buildTag("img#trendIndicator_" + gaugeId + ".kpiiconvalue")[0].outerHTML;
                            htmlTag += "<td>" + ej.buildTag("div#" + this._id + "_" + gaugeId, stausImage + trendImage)[0].outerHTML + "</td>";
                            gaugeId++;
                        }
                        htmlTag += "</tr>";
                    }
                    htmlTag += "</tbody></table>";
                }
                this.element.html(htmlTag);
                this.renderControlFromJSON(ogaugeInfo);
                this._defaultWidth = this.element.find("canvas").width();
                this._defaultArea = this.element.find("table").height() * this.element.find("table").width();
            }
            if (this.model.progressMode != "infinite") {
                if (this.model.progressMode == "normal")
                    $("#" + this._id + "progressContainer").delay(1500).fadeOut("slow");
                else if (this.model.progressMode == "progress") {
                    $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("RenderingSucceeded") + "...";
                    $("#" + this._id + "progressContainer").delay(2000).fadeOut("slow");
                }
            }
            else
                this._ogaugeWaitingPopup.hide();
            this._trigger("clientSuccess", this);
        },

        /**
        * This function receives the processed KPI value and factor to calculate the upper limit.
        *
        * @private  
        */
        _getCalculatedValue: function (value) {
            var stringValue = value.toString();
            var factor = 1; var len; var dividend;
            if (stringValue.indexOf('.') > -1)
                len = stringValue.split('.')[0].length;
            else
                len = stringValue.length;
            dividend = parseInt(stringValue);
            for (var i = 0; i < len; i++) {
                if (dividend > this._maxValue) {
                    dividend = dividend / 10;
                    factor = factor * 10;
                }
            }
            return factor;
        },

        /**
        * This function receives the processed KPI value and factor to calculate the upper limit.
        *
        * @private  
        */
        _findNearestUpperLimit: function (number, factor) {
            var convertedNumber = number.toString();
            var j = parseInt(convertedNumber);
            var k = j % factor;
            return j + (factor - k);
        },

        /**
        * This function receives the KPI goal and KPI values to calculate the factor.
        *
        * @private  
        */
        _setMaxScaleValue: function (goal, value, i) {
            var factor;
            if (value > goal) {
                factor = this._getCalculatedValue(value);
                this._maximum[i] = this._findNearestUpperLimit((value / factor), 10);
            }
            else {
                factor = this._getCalculatedValue(goal);
                this._maximum[i] = this._findNearestUpperLimit((((goal / factor) * 4) / 3), 10);
            }
            return factor;
        },

        /**
        * This function removes the KPI image tags in OLAP Gauge control.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
		*		showTooltip: true,
		*		scales: [{
		*		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            pointerLength: 120,
        *                            pointerWidth: 7,
        *                            capBorderColor: "#f5b43f",
        *                            capBackgroundColor: "#f5b43f"
        *                        },
        *                {
        *                    pointerType: "Marker",
        *                    markerStyle: "Diamond",
        *                    distanceFromScale: 5,
        *                    pointerPosition: "Center",
        *                    backgroundColor: "#29A4D9",
        *                    pointerLength: 25,
        *                    pointerWidth: 15
        *                }]
		*				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.removeImg();
        * // removes HTML image tags inside of OLAP Gauge.
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        removeImg: function () {
            this.element.find("img").remove();
        },

        /**
        * This function receives the JSON formatted datasource to render the OLAP Gauge control.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
		*		showTooltip: true,
		*		scales: [{
		*		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            pointerLength: 120,
        *                            pointerWidth: 7,
        *                            capBorderColor: "#f5b43f",
        *                            capBackgroundColor: "#f5b43f"
        *                        },
        *                {
        *                    pointerType: "Marker",
        *                    markerStyle: "Diamond",
        *                    distanceFromScale: 5,
        *                    pointerPosition: "Center",
        *                    backgroundColor: "#29A4D9",
        *                    pointerLength: 25,
        *                    pointerWidth: 15
        *                }]
		*				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.renderControlFromJSON(this.getJSONRecords());
        * // render the OLAP Gauge from JSON formatted data.
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        renderControlFromJSON: function (jsonObj) {
            //Calculating pointer value, danger and safer regions
            this._maxValue = 99, maxGaugeCount = 0;
            var pointerCapRadius = (this.radius() / 22);
            if (this._currentLayout == "Table" && (this.rowsCount() * this.columnsCount()) <= jsonObj.length)
                maxGaugeCount = this.rowsCount() * this.columnsCount();
            else if (this._currentLayout == "WrapPanel" || (this.rowsCount() * this.columnsCount()) > jsonObj.length)
                maxGaugeCount = jsonObj.length;
            for (var i = 0; i < maxGaugeCount; i++) {
                var goal = 0.0, value = 0.0, startValue = 0.0, endValue = 0.0, dValue = 0.0, factor = 0, mark = 0.0,
                percentageValue = 0.0, customLabel = "", imgDiv, extendRanges = new Array(), extendPointers = new Array(),
                extendLabels = new Array(), extendCustomLabels = new Array(), extendTicks = new Array(), extendIndicators = new Array(),
                extendScales = new Array(), statusClass = "", trendClass = "";

                if (jsonObj[i].MeasureValue != null && jsonObj[i].MeasureValue != undefined) {
                    value = parseFloat(jsonObj[i].MeasureValue) || 0;
                    goal = parseFloat(jsonObj[i].GoalValue) || 0;
                    customLabel = jsonObj[i].MemberName;
                }
                else {
                    value = parseFloat(jsonObj[i].Value);
                    goal = 0.0;
                    customLabel = jsonObj[i].Measure;
                    this.model.scales[0].showIndicators = false;
                }
                factor = this._setMaxScaleValue(goal, value, i);
                dValue = value;
                mark = goal / factor;
                percentageValue = (dValue / factor);
                this._pointerValue_0 = endValue + percentageValue;
                this._pointerValue_1 = mark + endValue;
                startValue = endValue;
                endValue = endValue + percentageValue;

                //Edting Model values
                if (jsonObj[i].StatusGraphic == "Traffic Signals") {
                    if (jsonObj[i].StatusValue == -1)
                        statusClass = "kpiredroad";
                    else if (jsonObj[i].StatusValue == 0)
                        statusClass = "kpiallcolor";
                    else if (jsonObj[i].StatusValue == 1)
                        statusClass = "kpigreenroad";
                }
                else {
                    if (jsonObj[i].StatusValue == -1)
                        statusClass = "kpidiamond";
                    else if (jsonObj[i].StatusValue == 0)
                        statusClass = "kpitriangle";
                    else if (jsonObj[i].StatusValue == 1)
                        statusClass = "kpicircle";
                }
                imgDiv = this.element.find("#stausIndicator_" + [i]).addClass(statusClass);
                if (imgDiv != null && imgDiv != undefined) {
                    var statusImage = $(imgDiv).css('background-image');
                    var statusIndicator = {
                        indicatorHeight: 15,
                        indicatorWidth: 15,
                        indicatorStyle: "Image",
                        imageUrl: statusImage.split("(")[1].split(")")[0].replace('"', "").replace('"', ""),
                        indicatorLocation: { x: this.radius() - 20, y: 260 }
                    };
                }

                if (jsonObj[i].TrendGraphic != null && jsonObj[i].TrendGraphic != undefined && jsonObj[i].TrendGraphic != "" && jsonObj[i].TrendGraphic.toLowerCase() == "standard arrow") {
                    if (jsonObj[i].TrendValue == -1)
                        trendClass = "kpidownarrow";
                    else if (jsonObj[i].TrendValue == 0)
                        trendClass = "kpirightarrow";
                    else if (jsonObj[i].TrendValue == 1)
                        trendClass = "kpiuparrow";
                    imgDiv = this.element.find("#trendIndicator_" + [i]).toggleClass(trendClass);
                }
                if (!ej.isNullOrUndefined(imgDiv)) {
                    var trendImage = $(imgDiv).css('background-image');
                    var trendIndicator = {
                        indicatorHeight: 15,
                        indicatorWidth: 15,
                        indicatorStyle: "Image",
                        imageUrl: trendImage.split("(")[1].split(")")[0].replace('"', "").replace('"', ""),
                        indicatorLocation: { x: this.radius(), y: 260 }
                    };
                }
                if (!ej.isNullOrUndefined(this.model.scales[0])) {
                    if (this.frameType() == "fullCircle") {
                        this.model.scales[0].startAngle = 122;
                        this.model.scales[0].sweepAngle = 296;
                    }
                    else if (this.frameType() == "halfCircle") {
                        this.model.scales[0].startAngle = 180;
                        this.model.scales[0].sweepAngle = 180;
                    }
                    this.model.scales[0].indicators = this.model.scales[0].showIndicators ? [statusIndicator, trendIndicator] : this.model.scales[0].indicators;
                    if (!ej.isNullOrUndefined(this.model.scales[0].pointers[0])) {
                        this.model.scales[0].pointers[0].value = this._pointerValue_0;
                        this.model.scales[0].pointers[0].pointerCapRadius = pointerCapRadius;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].pointers[1])) {
                        this.model.scales[0].pointers[1].value = this._pointerValue_1;
                        this.model.scales[0].pointers[1].pointerType = this.model.scales[0].pointers[1].pointerType || "Marker";
                        this.model.scales[0].pointers[1].markerStyle = this.model.scales[0].pointers[1].markerStyle || "Diamond";
                        this.model.scales[0].pointers[1].pointerPosition = this.model.scales[0].pointers[1].pointerPosition || "Center";
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].ranges[0])) {
                        this.model.scales[0].ranges[0].startValue = startValue;
                        this.model.scales[0].ranges[0].endValue = mark;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].ranges[1])) {
                        this.model.scales[0].ranges[1].startValue = mark;
                        this.model.scales[0].ranges[1].endValue = this._maximum[i];
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabel[0]))
                        this.model.scales[0].customLabel[0].labelValue = this.showHeaderLabels() ? "X " + factor : "";
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabel[1]))
                        this.model.scales[0].customLabel[1].labelValue = this.showHeaderLabels() ? customLabel : "";
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabel[2]))
                        this.model.scales[0].customLabel[2].labelValue = this.showHeaderLabels() ? this._getLocalizedLabels("RevenueFor") + " " + customLabel : "";

                    for (var j = 0; j < this.model.scales[0].ranges.length; j++)
                        extendRanges.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues().ranges[0], this.model.scales[0].ranges[j]));
                    for (var j = 0; j < this.model.scales[0].pointers.length; j++)
                        extendPointers.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues().pointers[0], this.model.scales[0].pointers[j]));
                    for (var j = 0; j < this.model.scales[0].labels.length; j++)
                        extendLabels.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues().labels[0], this.model.scales[0].labels[j]));
                    for (var j = 0; j < this.model.scales[0].labels.length; j++)
                        extendLabels.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues().labels[0], this.model.scales[0].labels[j]));
                    for (var j = 0; j < this.model.scales[0].customLabel.length; j++)
                        extendCustomLabels.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues().customLabel[0], this.model.scales[0].customLabel[j]));
                    for (var j = 0; j < this.model.scales[0].ticks.length; j++)
                        extendTicks.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues().ticks[0], this.model.scales[0].ticks[j]));
                    if (this.model.scales[0].indicators) {
                        for (var j = 0; j < this.model.scales[0].indicators.length; j++)
                            extendIndicators.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues().indicators[0], this.model.scales[0].indicators[j]));
                    }

                    newScale = {
                        capBorderColor: this.model.scales[0].capBorderColor,
                        capBackgroundColor: this.model.scales[0].capBackgroundColor,
                        majorIntervalValue: this.model.scales[0].majorIntervalValue,
                        minorIntervalValue: this.model.scales[0].minorIntervalValue,
                        minimum: this.model.scales[0].minimum,
                        borderColor: this.model.scales[0].borderColor,
                        backgroundColor: this.model.scales[0].backgroundColor,
                        labelAutoAngle: this.model.scales[0].labelAutoAngle,
                        scaleDirection: this.model.scales[0].scaleDirection,
                        pointerCapRadius: pointerCapRadius || this.model.scales[0].pointerCapRadius,
                        pointerCapBorderWidth: this.model.scales[0].pointerCapBorderWidth,
                        capInteriorGradient: this.model.scales[0].capInteriorGradient,
                        showPointers: this.model.scales[0].showPointers,
                        showTicks: this.model.scales[0].showTicks,
                        showLabels: this.model.scales[0].showLabels,
                        showIndicators: this.model.scales[0].showIndicators,
                        showRanges: this.model.scales[0].showRanges,
                        showScaleBar: this.model.scales[0].showScaleBar,
                        startAngle: this.model.scales[0].startAngle,
                        sweepAngle: this.model.scales[0].sweepAngle,
                        scaleRadius: this.model.scales[0].scaleRadius,
                        scaleBarSize: this.model.scales[0].scaleBarSize,
                        scaleBorderWidth: this.model.scales[0].scaleBorderWidth,
                        maximum: this.model.scales[0].maximum || this._maximum[i],
                        pointers: extendPointers,
                        ticks: extendTicks,
                        labels: extendLabels,
                        ranges: extendRanges,
                        customLabel: extendCustomLabels,
                        indicators: extendIndicators
                    };
                    for (var j = 0; j < this.model.scales.length; j++) {
                        if (j == 0)
                            extendScales.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues(), newScale));
                        else
                            extendScales.push($.extend({}, ej.CircularGauge.prototype._defaultScaleValues(), this.model.scales[j]));
                    }
                }
                //Creating Gauges
                $("#" + this._id + "_" + i).ejCircularGauge({
                    backgroundColor: this.model.backgroundColor,
                    frameType: this.frameType(),
                    radius: this.radius(),
                    width: this.model.width, // Width Should be 2*radious 
                    height: this.model.height,
                    interiorGradient: this.model.interiorGradient,
                    readOnly: this.model.readOnly,
                    halfCircleFrameStartAngle: this.model.halfCircleFrameStartAngle,
                    halfCircleFrameEndAngle: this.model.halfCircleFrameEndAngle,
                    animate: this.model.animate,
                    animationSpeed: this.model.animationSpeed,
                    theme: this.model.theme,
                    isRadialGradient: this.model.isRadialGradient,
                    //Events
                    load: this.model.load,
                    drawTicks: this.model.drawTicks,
                    drawLabels: this.model.drawLabels,
                    drawPointers: this.model.drawPointers,
                    drawRange: this.model.drawRange,
                    drawCustomLabel: this.model.drawCustomLabel,
                    drawIndicators: this.model.drawIndicators,
                    drawPointerCap: this.model.drawPointerCap,
                    renderComplete: this.model.renderComplete,
                    mouseClick: this.model.mouseClick,
                    mouseClickMove: this.model.mouseClickMove,
                    mouseClickUp: this.model.mouseClickUp,
                    scales: extendScales
                });
                this._gaugeObj["obj" + i] = $("#" + this._id + "_" + i).data("ejCircularGauge");
            }
            this.removeImg();
            this._wireEvents();
        },

        /**
        * This function is used to refresh the OLAP Gauge at client side.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
		*		showTooltip: true,
		*		scales: [{
		*		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            pointerLength: 120,
        *                            pointerWidth: 7,
        *                            capBorderColor: "#f5b43f",
        *                            capBackgroundColor: "#f5b43f"
        *                        },
        *                {
        *                    pointerType: "Marker",
        *                    markerStyle: "Diamond",
        *                    distanceFromScale: 5,
        *                    pointerPosition: "Center",
        *                    backgroundColor: "#29A4D9",
        *                    pointerLength: 25,
        *                    pointerWidth: 15
        *                }]
		*				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.refresh();
        * // refresh the OLAP Gauge in client-side.
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        refresh: function () {
            if (this._gaugeObj == null || this._gaugeObj == undefined) {
                for (var i = 0; i < this._JSONRecords.length; i++) {
                    this._gaugeObj["obj" + i] = $("#" + this._id + "_" + i).data("ejCircularGauge");
                    this._gaugeObj["obj" + i].refresh();
                }
            }
            else {
                for (key in this._gaugeObj) {
                    this._gaugeObj[key].refresh();
                }
            }
        }
    });

    //OLAP Gauge Localization
    ej.OlapGauge.localization = {};
    //Default english localized values
    ej.OlapGauge.localization["en-US"] = {
        RevenueGoal: "Revenue Goals",
        RevenueValue: "Revenue Values",
        RevenueFor: "Revenue For",
        MDXqueryExecutionFailed: "MDX query execution failed",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingStarted: "Rendering Started",
        RenderingSucceeded: "Rendering Succeeded",
        RenderingFailed: "Rendering failed"

    };

    /**
    * Enum for Progres Mode. Allows the user to select a appropriate progress indicator. 
    * @enum {String}
	* @global
    */
    ej.OlapGauge.progressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

    /**
    * Enum for Frame Types. Allows the user to select an appropriate frame type. 
    * @enum {String}
	* @global
    */
    ej.OlapGauge.frameType = {
        FullCircle: "fullCircle",
        HalfCircle: "halfCircle"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html OLAP Pager elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejOlapPager
	* @requires jQuery
	* @requires ej.common.all
	* @requires ej.olapgrid
	* @classdesc Custom Design for Html OLAP Pager control.
	* @example 
	* &lt;div id="OlapPager"&gt;&lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create OLAP Pager
    * $("#OlapPager").ejOlapPager(...); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejOlapPager", "ej.OlapPager", {

        //Root Css Class
        _rootCSS: "e-olappager",

        // Widget element will be automatically set in this
        element: null,

        // User-defined model will be automatically set in this
        model: null,

        /**
         * Returns the model of OLAP Pager widget.
         *
         * @private
         */
        _getModel: function () {
            return this.model;
        },

        /**
         * Sets the model of OLAP Pager widget.
         *
         * @private
         */
        _setModel: function (value) {
            this.model = value;
        },

        defaults: {
            /**		
             * Contains the id of the target element for which paging is enabled.
             * @default ""
             * @type {string}
             * @example 
             * //To set targetControlID API value during initialization  
             * 	$("#OlapPager").ejOlapPager({  targetControlID: "OlapGrid"});					* 
             * @example 
             * //Get or set the targetControlID API, after initialization:<br>
             *	//Gets the targetControlID value  
             *	$("#OlapPager").ejOlapPager("option", "targetControlID");<br>			
             *	//Sets the targetControlID value 
             *	$("#OlapPager").ejOlapPager("option", "targetControlID",  "OlapGrid1" ); 
              * @memberof ejOlapPager
             * @instance
             */
            targetControlID: "",
            /**		
			* Contains the current page number in categorical axis.
			* @default 1
			* @type {number}
			* @example 
			* //To set categoricalCurrentPage API value during initialization  
			* 	$("#OlapPager").ejOlapPager({  categoricalCurrentPage: 1});					* 
			* @example 
			* //Get or set the categoricalCurrentPage API, after initialization:<br>
			*	//Gets the categoricalCurrentPage value  
			*	$("#OlapPager").ejOlapPager("option", "categoricalCurrentPage");<br>			
			*	//Sets the categoricalCurrentPage value 
			*	$("#OlapPager").ejOlapPager("option", "categoricalCurrentPage", 2 ); 
			 * @memberof ejOlapPager
			* @instance
			*/
            categoricalCurrentPage: 1,
            /**		
			* Contains the current page number in series axis.
			* @default 1
			* @type {number}
			* @example 
			* //To set seriesCurrentPage API value during initialization  
			* 	$("#OlapPager").ejOlapPager({  seriesCurrentPage: 1});					* 
			* @example 
			* //Get or set the seriesCurrentPage API, after initialization:<br>
			*	//Gets the seriesCurrentPage value  
			*	$("#OlapPager").ejOlapPager("option", "seriesCurrentPage");<br>			
			*	//Sets the seriesCurrentPage value 
			*	$("#OlapPager").ejOlapPager("option", "seriesCurrentPage", 2 ); 
			 * @memberof ejOlapPager
			* @instance
			*/
            seriesCurrentPage: 1,
            /**		
			* Contains the total page count in series axis.
			* @default 1
			* @type {number}
			* @example 
			* //To set seriesPageCount API value during initialization  
			* 	$("#OlapPager").ejOlapPager({  seriesPageCount: 1});					* 
			* @example 
			* //Get or set the seriesPageCount API, after initialization:<br>
			*	//Gets the seriesPageCount value  
			*	$("#OlapPager").ejOlapPager("option", "seriesPageCount");<br>			
			*	//Sets the seriesPageCount value 
			*	$("#OlapPager").ejOlapPager("option", "seriesPageCount", 2 ); 
			 * @memberof ejOlapPager
			* @instance
			*/
            seriesPageCount: 0,
            /**		
			* Contains the total page count in categorical axis.
			* @default 1
			* @type {number}
			* @example 
			* //To set categPageCount API value during initialization  
			* 	$("#OlapPager").ejOlapPager({  categPageCount: 1});					* 
			* @example 
			* //Get or set the categPageCount API, after initialization:<br>
			*	//Gets the categPageCount value  
			*	$("#OlapPager").ejOlapPager("option", "categPageCount");<br>			
			*	//Sets the categPageCount value 
			*	$("#OlapPager").ejOlapPager("option", "categPageCount", 2 ); 
			 * @memberof ejOlapPager
			* @instance
			*/
            categPageCount: 0,
            /**		
			* Sets the localized language for the control.
			* @default 1
			* @type {string}
			* @example 
			* //To set localization API value during initialization  
			* 	$("#OlapPager").ejOlapPager({  localization: "en-US"});					* 
			* @example 
			* //Get or set the localization API, after initialization:<br>
			*	//Gets the localization value  
			*	$("#OlapPager").ejOlapPager("option", "localization");<br>			
			*	//Sets the localization value 
			*	$("#OlapPager").ejOlapPager("option", "localization", "fr-FR" ); 
			 * @memberof ejOlapPager
			* @instance
			*/
            localization: "en-US",
            /**		
			* Sets the pager mode for the Olap pager.
			* @default ej.OlapPager.pagerModes.Both
			* @type {enum}
			* @example 
			* //To set pagerMode API value during initialization  
			* 	$("#OlapPager").ejOlapPager({  pagerMode: ej.OlapPager.pagerModes.Series});					* 
			* @example 
			* //Get or set the pagerMode API, after initialization:<br>
			*	//Gets the pagerMode value  
			*	$("#OlapPager").ejOlapPager("option", "pagerMode");<br>			
			*	//Sets the pagerMode value 
			*	$("#OlapPager").ejOlapPager("option", "pagerMode", ej.OlapPager.pagerModes.Categorical ); 
			 * @memberof ejOlapPager
			* @instance
			*/
            pagerMode: "Both",
        },

        /**
         * Initializes the variables and loads the widget. 
         *
         * @private
         */
        _init: function () {
            if ($("#" + this.model.targetControlID).hasClass('e-olapgrid')) {
                this.targetControlName = "OlapGrid";
                this.targetControl = $("#" + this.model.targetControlID).data("ejOlapGrid");
                this.targetControl._pagerObj = this;
            }
            this._createPager();
            this.unwireEvents();
            this.wireEvents();
        },

        /**
         * Destroys the widget while it's no longer required. 
         *
         * @private
         */
        _destroy: function () {
            this.element.empty().removeClass("e-olappager");
        },

        /**
        * 	This function is invoked to wire the events for UI interaction.
        * @return jQuery
        * @example 
        * &lt;div id="OlapPager"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Wiring events to OLAP Pager control.
        * $('#OlapPager').ejOlapPager({
        *       url: "Pager.svc",
        *   });
        * var pagerObj = $("#OlapPager").data("ejOlapPager");
        * pagerObj.wireEvents();
        * // Wiring events with OLAP Pager.
        * &lt;/script&gt;
        * @memberof ejOlapPager
        * @instance
        */
        wireEvents: function () {
            this._on(this.element, "keydown", ".pagerTextBox", this._pagerTextBoxClick);
            this._on(this.element, "click", ".moveNext", this._moveNextPage);
            this._on(this.element, "click", ".moveLast", this._moveLastPage);
            this._on(this.element, "click", ".moveFirst", this._moveFirstPage);
            this._on(this.element, "blur", ".pagerTextBox", this._restorePageNo);
            this._on(this.element, "click", ".movePrevious", this._movePreviousPage);
        },

        /**
        * 	This function is invoked to unwire the events registered for UI interaction.
        * @return jQuery
        * @example 
        * &lt;div id="OlapPager"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Unwiring events from OLAP Pager control.
        * $('#OlapPager').ejOlapPager({
        *       url: "Pager.svc",
        *   });
        * var pagerObj = $("#OlapPager").data("ejOlapPager");
        * pagerObj.unwireEvents();
        * // Unwiring events from OLAP Pager.
        * &lt;/script&gt;
        * @memberof ejOlapPager
        * @instance
        */
        unwireEvents: function () {
            this._off(this.element, "click", ".moveNext", this._moveNextPage);
            this._off(this.element, "blur", ".pagerTextBox", this._restorePageNo);
            this._off(this.element, "click", ".moveLast", this._moveLastPage);
            this._off(this.element, "click", ".moveFirst", this._moveFirstPage);
            this._off(this.element, "click", ".movePrevious", this._movePreviousPage);
            this._off(this.element, "keydown", ".pagerTextBox", this._pagerTextBoxClick);
        },

        /**
         * This function is used to extract the appropriate localized text.
         *
         * @private  
         */
        _getLocalizedLabels: function (property) {
            return ej.OlapPager.localization[this.model.localization][property] === undefined ? ej.OlapPager.localization["en-US"][property] : ej.OlapPager.localization[this.model.localization][property];
        },

        /**
         * This function creates the OlapPager element.
         *
         * @private
         */
        _createPager: function () {
            $(this.element).addClass("olapPager").attr("targetControlID", this.model.targetControlID);
            var pagerContent = "";
            if (this.model.pagerMode != "Series") {
                var categoricalPager = ej.buildTag("td.categPagerTd", "");
                if (this.model.pagerMode == "Both") $(categoricalPager[0]).css("border-right", "1px solid #808080");
                $(categoricalPager).html(ej.buildTag("div.pagerDiv", ej.buildTag("span.moveFirst e-icon")[0].outerHTML + ej.buildTag("span.movePrevious e-icon")[0].outerHTML + ej.buildTag("span.pagerLabel", this._getLocalizedLabels("CategoricalPage"))[0].outerHTML + ej.buildTag("input#" + this._id + "_CategCurrentPage.pagerTextBox")[0].outerHTML + ej.buildTag("span.categPageCount")[0].outerHTML + ej.buildTag("span.moveNext e-icon")[0].outerHTML + ej.buildTag("span.moveLast e-icon")[0].outerHTML));
                pagerContent = categoricalPager[0].outerHTML;
            }
            if (this.model.pagerMode != "Categorical") {
                var seriesPager = ej.buildTag("td.seriesPagerTd", "");
                $(seriesPager).html(ej.buildTag("div.pagerDiv", ej.buildTag("span.moveFirst e-icon")[0].outerHTML + ej.buildTag("span.movePrevious e-icon")[0].outerHTML + ej.buildTag("span.pagerLabel", this._getLocalizedLabels("SeriesPage"))[0].outerHTML + ej.buildTag("input#" + this._id + "_SeriesCurrentPage.pagerTextBox")[0].outerHTML + ej.buildTag("span.seriesPageCount")[0].outerHTML + ej.buildTag("span.moveNext e-icon")[0].outerHTML + ej.buildTag("span.moveLast e-icon")[0].outerHTML));
                pagerContent += seriesPager[0].outerHTML;
            }
            var pagerTable = ej.buildTag("table", ej.buildTag("tbody", ej.buildTag("tr", pagerContent)))[0];
            $(this.element).html(pagerTable);
        },

        /**
        * 	This function initializes the page counts and page numbers of the OlapPager.
        * @return jQuery
        * @example 
        * &lt;div id="OlapPager"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Initializing page count and numbers.
        * $('#OlapPager').ejOlapPager({
        *       url: "Pager.svc",
        *   });
        * var pagerObj = $("#OlapPager").data("ejOlapPager");
        * pagerObj.initPagerProperties(10, {CategorialPageSize: 1, SeriesPageSize: 1, CategorialCurrentPage: 3, SeriesCurrentPage: 4});
        * // Initializing page count and numbers.
        * &lt;/script&gt;
        * @memberof ejOlapPager
        * @instance
        */
        initPagerProperties: function (headerCounts, pageSettings) {
            this.model.categPageCount = Math.ceil(headerCounts.Column / pageSettings.CategorialPageSize);
            this.model.seriesPageCount = Math.ceil(headerCounts.Row / pageSettings.SeriesPageSize);
            this.model.categoricalCurrentPage = pageSettings.CategorialCurrentPage;
            this.model.seriesCurrentPage = pageSettings.SeriesCurrentPage;
            if (this.model.pagerMode != "Series") {
                $(this.element.find(".categPageCount")[0]).html("/ " + this.model.categPageCount);
                $("#" + this._id + "_CategCurrentPage")[0].value = pageSettings.CategorialCurrentPage;
                this._setNavigators(this.model.categoricalCurrentPage, this.model.categPageCount, this.element.find(".categPagerTd")[0]);
            }
            if (this.model.pagerMode != "Categorical") {
                $(this.element.find(".seriesPageCount")[0]).html("/ " + this.model.seriesPageCount);
                $("#" + this._id + "_SeriesCurrentPage")[0].value = pageSettings.SeriesCurrentPage;
                this._setNavigators(this.model.seriesCurrentPage, this.model.seriesPageCount, this.element.find(".seriesPagerTd")[0]);
            }
        },

        /**
         * This function is used to move the target control to next page.
         *
         * @private  
         */
        _moveNextPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).removeClass("disabled");
                pagerInput.value = parseInt(pagerInput.value) + 1;
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value;
                    if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("categorical", pagerInput.value);
                    if (pagerInput.value == this.model.categPageCount) {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveLast")[0]).addClass("disabled");
                    }
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value;
                    if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("series", pagerInput.value);
                    if (pagerInput.value == this.model.seriesPageCount) {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveLast")[0]).addClass("disabled");
                    }
                }
            }
        },

        /**
         * This function is used to move the target control to previous page.
         *
         * @private  
         */
        _movePreviousPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveLast")[0]).removeClass("disabled");
                pagerInput.value = parseInt(pagerInput.value) - 1;
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value;
                    if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("categorical", pagerInput.value);
                    if (pagerInput.value == "1") {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).addClass("disabled");
                    }
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value;
                    if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("series", pagerInput.value);
                    if (pagerInput.value == "1") {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).addClass("disabled");
                    }
                }
            }
        },

        /**
         * This function is used to move the target control to last page.
         *
         * @private  
         */
        _moveLastPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).removeClass("disabled");
                $(e.target).addClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).addClass("disabled");
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value = this.model.categPageCount;
                    if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("categorical", pagerInput.value);
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value = this.model.seriesPageCount;
                    if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("series", pagerInput.value);
                }
            }
        },

        /**
         * This function is used to move the target control to first page.
         *
         * @private  
         */
        _moveFirstPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveLast")[0]).removeClass("disabled");
                $(e.target).addClass("disabled");
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).addClass("disabled");
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value = 1;
                    if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("categorical", pagerInput.value);
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value = 1;
                    if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("series", pagerInput.value);
                }
            }
        },

        /**
         * This function is used to move the target control to entered page.
         *
         * @private  
         */
        _pagerTextBoxClick: function (e) {
            if (e.which == 13) {
                e.preventDefault();
                var pagerTd = $(e.target).parents('td')[0];
                var pageCount = pagerTd.className.indexOf('categ') != -1 ? this.model.categPageCount : this.model.seriesPageCount;
                if (parseInt(e.target.value) > pageCount || parseInt(e.target.value) < 1 || e.target.value == "") alert("Enter valid page number");
                else {
                    if (pagerTd.className.indexOf('categ') != -1 && this.model.categoricalCurrentPage != parseInt(e.target.value)) {
                        this.model.categoricalCurrentPage = e.target.value;
                        if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid(pagerTd.className, e.target.value);
                    }
                    else if (pagerTd.className.indexOf('series') != -1 && this.model.seriesCurrentPage != parseInt(e.target.value)) {
                        this.model.seriesCurrentPage = e.target.value;
                        if (this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid(pagerTd.className, e.target.value);
                    }
                    this._setNavigators(e.target.value, pageCount, pagerTd);
                    e.target.blur();
                }
            }
            else {
                if ($.inArray(e.keyCode, [46, 8, 9, 27, 13, 190]) !== -1 || (e.keyCode == 65 && e.ctrlKey === true) || (e.keyCode >= 35 && e.keyCode <= 39)) return;
                if ((e.shiftKey || (e.keyCode < 48 || e.keyCode > 57)) && (e.keyCode < 96 || e.keyCode > 105)) e.preventDefault();
            }
        },

        /**
         * This function is used to restore the current page numbers in Olap Pager.
         *
         * @private  
         */
        _restorePageNo: function (e) { e.target.value = e.target.id.indexOf("Categ") != -1 ? this.model.categoricalCurrentPage : this.model.seriesCurrentPage; },

        /**
         * This function is used to enable/disable the navigators according to the current page on initial loading.
         *
         * @private  
         */
        _setNavigators: function (value, pageCount, pagerTd) {
            if (value == pageCount) {
                $($(pagerTd).find(".moveNext")[0]).addClass("disabled");
                $($(pagerTd).find(".moveLast")[0]).addClass("disabled");
            }
            if (value == "1") {
                $($(pagerTd).find(".movePrevious")[0]).addClass("disabled");
                $($(pagerTd).find(".moveFirst")[0]).addClass("disabled");
            }
            if (value > 1) {
                $($(pagerTd).find(".movePrevious")[0]).removeClass("disabled");
                $($(pagerTd).find(".moveFirst")[0]).removeClass("disabled");
            }
            if (value < pageCount) {
                $($(pagerTd).find(".moveNext")[0]).removeClass("disabled");
                $($(pagerTd).find(".moveLast")[0]).removeClass("disabled");
            }
        }
    })

    //OLAP Pager Localization
    ej.OlapPager.localization = {};
    //Default english localized values
    ej.OlapPager.localization["en-US"] = {
        SeriesPage: "Series Page",
        CategoricalPage: "Categorical Page",
    };

    /**
     * Enum for PagerModes. Allows the user to select a pager mode to enable the pager for both or any one of the axes. 
     *
     * @enum {String}
	 * @global
     */
    ej.OlapPager.pagerModes = {
        Both: "Both",
        Categorical: "Categorical",
        Series: "Series"
    };

})(jQuery, Syncfusion);;

;